#ifndef __EDITOR_H__
#define __EDITOR_H__

#include "xbefore.h"
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/cursorfont.h>
#include "xafter.h"

#include <iostream.h>

#include "dialog.h"
#include "wrapper.h"
#include "linked.h"
#include "position.h"
#include "nodecl.h"
#include "conncl.h"
#include "objbar.h"
#include "event.h"
#include "logwin.h"
#include "text.h"

#undef NULL
#define NULL ((void*)0)

struct ApplicationData
{
	int initial_zoom;			// 100 (/100)
	int initial_mvgrid;		// 5 (pix)
	int initial_scgrid; //  10 (/100)
	int initial_rotgrid; // 15 (deg)
	String initial_netclass; // "SPNL"
	int click_delay; 			// 150
	int update_delay; 		// 5
	bool display_grid;		// false
	bool expert_mode;			// false
	String right_click;		// "Scrolls"
};

typedef ApplicationData* ApplicationDataP;
	
struct EditorSettings
{
	double zoom,mvgrid,scgrid,rotgrid;
	bool grid_visib,expert_mode;
	Ident rclick;
	
	EditorSettings(double z,double m,double s,double r,bool gv,bool em):
		zoom(z),
		mvgrid(m),
		scgrid(s),
		rotgrid(r),
		grid_visib(gv),
		expert_mode(em),
		rclick("Scrolls") {}
};		

class Net;

class Editor;
class Editor: public Wrapper<Editor,void*>
{
	static LinkedList<void*> editors;

	friend int main(int argc,char* argv[]);
	static void MainLoop();
	
	Display* display;
	ApplicationData app_data;
	Widget app_shell;
	Dialog<Editor,bool> about;
	void OpenAboutBox();
	
	LogWindow help;
	Text curr_path;
	
	Dimension width,height;
	Pixmap pixmap;
	GC clear_gc,grid_gc1,grid_gc2,dragbox_gc;

	Net* net;

	void CreateMenuBar(Widget parent);
	
	void Destroy(Widget,void*,XtPointer);
	void Menu(Widget,void*,XtPointer);
	void Redraw(Widget,void*,XtPointer);
	void Input(Widget,void*,XtPointer);
	
	EventInfo saveevent;
	WrapperID click_id;
	bool ClickTimer(void* para);
	
	int last_x,last_y;
	XEvent last_event;

	WrapperID update_id;
	bool DoUpdate(void* udata);
	
	void Dispatch(Widget w,void* udata);
	
	void GetScroll(int& x,int& y,int& w,int& h);
	void SetScroll(int x,int y);

	friend class Page;
	friend class Net;
	void Lock(bool set);
	void Clear();
	void Copy();
	void NewNetClass(NetClass* nc);
	Widget NewModule(char*);
	void RemoveModule(char*);
	
	ObjectBar<NodeClass> nodebar;
	ObjectBar<ConnectorClass> connbar;
	
	Widget shell,main,menu,menush,cmds,pos,
		box,view,hsb,vsb,clip,darea,mods,opts;
	Widget msg;
	
	EditorSettings settings;

	// privater Konstruktor: NewEditor() nehmen
	Editor(Net* n,Display* dpy);
	static Editor* NewEditor(Net* n,char* display_string = 0,int load_cmd = 0);

public:	
	~Editor();

	static int NumEditors() { return editors; }
	static Iterator<void*> Editors() { return Iterator<void*>(editors); }
	static bool Exists(Editor* editor);
		
	EditorSettings Settings() { return settings; }
	char* CurrentPath() { return curr_path; }
	double Zoom() { return settings.zoom; }
	Widget Cmds() { return cmds; }
	Widget Mods() { return mods; }
	Widget Area() { return darea; }
	Widget Box() { return box; }
	Widget Shell() { return shell; }
	Display* Dpy() { return display; }
	char* DpyName() { return DisplayString(display); }
	Drawable Pixm() { return pixmap; }
	GC DragboxGC() { return dragbox_gc; }
	ObjectBar<NodeClass>& NodeBar() { return nodebar; }
	ObjectBar<ConnectorClass>& ConnBar() { return connbar; }

	void SetSettings(EditorSettings& set);
	void SetCurrentPath(char* path) { curr_path = path; }
	void SetZoom(double z,bool check = true);
	void SetDimension(unsigned w,unsigned h);

	void SetNet(Net* n);
	Net* GetNet() { return net; }

	Widget CreatePulldown(char* name,Widget parent = 0);

	// Aenderungen fuer das Tokengame:
	Widget CreateEntry(char* name,WidgetClass cl,Widget pulldown,void*,
			   WrapperID& id);
	void  DestroyEntry(Widget w, WrapperID id);
	// end Tokengame


	void CreateEntries(Widget pulldown,WidgetClass cl,void* udata,char* names);
	Widget CreateSeparator(Widget pulldown);

	void Update();
	
	void Mouse(bool define,unsigned int shape = XC_watch);
	Position MousePosition();
	void ScrollVisible(Position p,bool center = false);
	
	char* curr_clname;
	void Message(char *s);
	void ButtonState(char* names,bool state);
	void Coordinate(Position pos = Position(0,0));
	void Title(char *s);
	
	NodeClass* CurrNodeClass();
	ConnectorClass* CurrConnectorClass();	
	
	void Scroll();

	static Editor* GetEditor(Widget w);
};

typedef Editor* EditorP;

#endif
