/* enum.h:

Der Aufzaehlungstyp (Enumeration) wird von der Klasse Enum
bereitgestellt. Die Typparameter enthalten dabei eine Liste von
Identifizierern,von denen der Typ immer jeweils genau einen zum Wert
hat.  Als Eingabewidget bietet sich in diesem Fall ein
Motif-Popupbutton an,der immer die aktuelle Wahl anzeigt und bei
dessen Anwahl eine Liste der moeglichen Alternativen in Form eines
Popups erscheint. */

#ifndef __ENUM_H__
#define __ENUM_H__

#include <strstream.h>

#include "value.h"
#include "ident.h"
#include "linked.h"

class Enum;
class Enum: public Value
{
protected:
	Widget popup,option;
	Ident* val;
	LinkedList<Ident*> defs;

	void UpdateWidget();

public:
	Enum(NetClass *nc = 0, bool hidden = false);
	Enum(const Enum& e);
	~Enum();
	
	Value* Copy() const { return new Enum(*this); }
	Value* Cast() const { return new Text((char*)*val); }
	
	Widget CreateWidget(Widget parent);
	bool EvalWidget();
	
	istream& Parse(istream& is);
	istream& Input(istream& is);
	ostream& Output(ostream& os) const;
	
	operator char*() const { return val?(char*)*val:0; }

	Value& operator =(const char* def);
	Value& operator =(const Ident& id) { return *this=(char*)id; }
	Value& operator =(const Enum& e) { return *this=(char*)e; }
	Value& operator =(const Value& v) { return *this=(char*)v; }

	bool operator ==(const Enum& e) const { return this->val==e.val; }
};

typedef Enum* EnumP;

#endif
