#ifndef __FLOAT_H__
#define __FLOAT_H__

#include <strstream.h>
extern "C" {
#include <values.h>
#ifndef MAXDOUBLE
#define MAXDOUBLE 1e300
#endif
}

#include "value.h"

class Float;
class Float: public Value
{
protected:
	double val,max,min;
	Widget text;

	void UpdateWidget();

public:
	Float(NetClass* nc,bool hidden = false);
	Float(double d = 0.0);
	Float(const Float& f);
	~Float() { 	UncatchDestroy(&text); }
	
	Value* Copy() const { return new Float(*this); }
	Value* Cast() const { return Copy(); }

	Widget CreateWidget(Widget parent);
	bool EvalWidget();
	
	istream& Parse(istream& is);
	istream& Input(istream& is);
	ostream& Output(ostream& os) const { return os<<val; }
	
	operator int() const { return int(val); }
	operator double() const { return val; }
	operator char *() const; 

	Value& operator =(const double d);
	Value& operator =(const int i) { return *this=double(i); };
	Value& operator =(const Float& f) { return *this=double(f); };
	Value& operator =(const Value& v) { return *this=double(v); };
	
	bool operator ==(double v) { return val==v; }
	bool operator !=(double v) { return val!=v; }
	bool operator >(double v)  { return val>v; }
	bool operator <(double v)  { return val<v; }
	Float operator -()      { return Float(-val); }
	Float operator !() 		 { return Float(!int(val)); }
	Float operator -(double v)  { return Float(val-v); }
	Float operator +(double v)  { return Float(val+v); }
	Float operator *(double v)  { return Float(val*v); }
	Float operator /(double v)  { return Float(val/v); }
	
private:
	friend class Operation;
	Value* Eq(Value& v) { return new Float(val==double(v)?1:0); }
	Value* Gt(Value& v)  { return new Float(val>double(v)?1:0); }
	Value* Lt(Value& v)  { return new Float(val<double(v)?1:0); }
	Value* Neg()      { return new Float(-val); }
	Value* Not() 		 { return new Float(!int(val)); }
	Value* Sub(Value& v)  { return new Float(val-double(v)); }
	Value* Add(Value& v)  { return new Float(val+double(v)); }
	Value* Mul(Value& v)  { return new Float(val*double(v)); }
	Value* Div(Value& v)  { return new Float(val/double(v)); }
	
	Value* And(Value& v) { return new Float(val && double(v)); }
	Value* Or(Value& v) { return new Float(val || double(v)); }
};

typedef Float* FloatP;

#endif
