#ifndef __INT_H__
#define __INT_H__

extern "C" {
#include <values.h>
#ifndef MAXINT
#define MAXINT int(~(1U<<(8*sizeof(int))-1))
#endif
#undef MININT
#define MININT (-MAXINT)
}

#include <strstream.h>

#include "value.h"
#include "repeater.h"
#include "float.h"
#include "wrapper.h"

class Int;
class Int: public Value, public Wrapper<Int,bool>
{
protected:
	int val,max,min;
	Widget form,text;
	Repeater up_r,dn_r;
	
	void Edit(Widget,bool,XtPointer) {}
	void Arrow(Widget w,bool,XtPointer);
	
	void UpdateWidget(int val);

public:
	Int(NetClass* nc,bool hidden = false);
	Int(int v = 0);
	Int(const Int& i);
	~Int() { 	UncatchDestroy(&form,&text); }
	
	Value* Copy() const { return new Int(*this); }
	Value* Cast() const { return new Float(val); }

	Widget CreateWidget(Widget parent);
	bool EvalWidget();
	
	istream& Parse(istream& is);
	istream& Input(istream& is);
	ostream& Output(ostream& os) const { return os<<val; }
	
	operator int() const { return val; }
	operator bool() const { return bool(val); }
	operator double() const { return double(val); }
	operator char *() const; 
	
	Value& operator =(const int i);
	Value& operator =(const Int& i) { return *this=(int)i; }
	Value& operator =(const Value& v) { return *this=(int)v; }
	
	bool operator ==(int v) { return val==v; }
	bool operator !=(int v) { return val!=v; }
	bool operator >(int v)  { return val>v; }
	bool operator <(int v)  { return val<v; }
	Int operator -()      { return Int(-val); }
	Int operator !() 		 { return Int(!val); }
	Int operator -(int v)  { return Int(val-v); }
	Int operator +(int v)  { return Int(val+v); }
	Int operator *(int v)  { return Int(val*v); }
	Int operator /(int v)  { return Int(val/v); }
	
	Int& operator ++()  { ++val; return *this; }
};

typedef Int* IntP;

#endif
