#ifndef __LINKED_H__
#define __LINKED_H__

template<class T> class Container;
template<class T> class Iterator;
template<class T> class OpaqueIterator;
template<class T> class LinkedList;


template<class T> class Container
{
public:
	T val;
	Container<T>* next,* prev;
	Container(const T& v,Container<T>* n,Container<T>* p):
		val(v),next(n),prev(p) {}
};


template<class T> class LinkedList
{
protected:
	friend class Iterator<T>;
	friend class OpaqueIterator<T>;
	int length;
	Container<T>* first,* last;

	void _Remove(Container<T>* r);
	Container<T>* _Idx(int pos) const;

public:
	int Index(const T& val) const;

	bool Append(const T& val);
	bool Insert(const T& val,int pos); // pos = 0 fuer erste Position
	bool Remove(const T& val);
	bool RemoveIndex(int pos);
	bool Replace(const T& oldval,const T& newval);
	
	int Length() const { return length; }
	T* First() const { return first?&first->val:0; }
	T* Last() const { return last?&last->val:0; }
	
	void Clear();
	
	bool IsIn(const T& t) const;
	
	operator int() const { return length; }
	operator bool() const { return length!=0; }
	operator !() const { return length==0; }
	T* operator [] (int pos) const;
	
	LinkedList<T>& operator =(const LinkedList<T>& l);
	bool operator ==(const LinkedList<T>& l);

	LinkedList(): length(0),first(0),last(0) {}
	LinkedList(const LinkedList<T>& i): 
		length(0),first(0),last(0) { *this = i; }
	~LinkedList();
};


template<class T> class Iterator
{
	const LinkedList<T>* list;
	Container<T>* curr;
	int cindex;

public:
	Iterator(const LinkedList<T>& l): list(&l),curr(list->first),cindex(0) {}
	
	T First() { return list->first?list->first->val:0; }
	T Last() { return list->last?list->last->val:0; }
	
	operator int() { return list->length; }
	operator bool() { return list->length!=0; }
	operator !() { return list->length==0; }
	
	// Rueckwaerts-Iteration
	T operator -()
		{ cindex=list->length-1; return (curr=list->last)?curr->val:0; }
	T operator --() { cindex--; return (curr=curr->prev)?curr->val:0; }
	
	// Vorwaerts-Iteration
	operator T() { cindex=0; return (curr=list->first)?curr->val:0; }
	T operator ++() { cindex++; return (curr=curr->next)?curr->val:0; }
	
	int Index() { return cindex; }
	bool IsIn(const T& t) const { return list->IsIn(t); }
	T operator [](int pos) const { return *(*list)[pos]; }
	
	Iterator<T>& operator =(LinkedList<T>& l) 
		{ curr = (list = &l)->first; return *this; }
};

// Beispieliteration mit <Iterator>:
// LinkedList<Object*> objects;
// Iterator<Object*> i(objects);
// for (Object* o=i; o; o=++i) {}


template<class T> class OpaqueIterator
{
	const LinkedList<T>* list;
	Container<T>* curr;
	int cindex;

public:
	OpaqueIterator(const LinkedList<T>& l): 
		list(&l),curr(list->first),cindex(0) {}
	
	T* First() { return list->first?&list->first->val:0; }
	T* Last() { return list->last?&list->last->val:0; }
	
	operator int() { return list->length; }
	operator bool() { return list->length!=0; }
	operator !() { return list->length==0; }
	
	// Rueckwaerts-Iteration
	T* operator -()
		{ cindex=list->length-1; return (curr=list->last)?&curr->val:0; }
	T* operator --() { cindex--; return (curr=curr->prev)?&curr->val:0; }
	
	// Vorwaerts-Iteration
	operator T*() { cindex=0; return (curr=list->first)?&curr->val:0; }
	T* operator ++() { cindex++; return (curr=curr->next)?&curr->val:0; }
	
	int Index() { return cindex; }
	bool IsIn(const T& t) const { return list->IsIn(t); }
	T* operator [](int pos) const { return (*list)[pos]; }
	
	OpaqueIterator<T>& operator =(LinkedList<T>& l) 
		{ curr = (list = &l)->first; return *this; }
};

// Beispieliteration mit <OpaqueIterator>:
// LinkedList<Object> objects;
// OpaqueIterator<Object> i(objects);
// for (Object* o=i; o; o=++i) {}


/*
typedef template<class T> bool (*FilterFkt<T>)(T* obj);

template<class T> class Filter;
template<class T> class Filter
{
	const LinkedList<T>* list;
	Container<T>* curr;
	FilterFkt filter;

	Container<T>* Next(bool first)
	{
		curr = first?list->first:curr->next;
		while (curr && !filter(&curr->val)) curr = curr->next;
		return curr;
	}

public:
	Filter(const LinkedList<T>& l,FilterFkt filt):
		 list(&l),curr(Next(true)),filter(filt) {}
	
	operator int() { return list->length; } // Stimmt noch nicht!
	
	operator T() { return (curr=Next(true))?curr->val:0; }
	operator T*() { return (curr=Next(true))?&curr->val:0; }
	T operator ++() { return (curr=Next(false))?curr->val:0; }
	T* operator ++() { return (curr=Next(false))?&curr->val:0; }
	
	Iterator<T>& operator =(LinkedList<T>& l) 
		{ list = &l; curr = Next(true); return *this; }
};
*/

#endif
