#ifndef __LIST_H__
#define __LIST_H__

#include "value.h"
#include "linked.h"
#include "wrapper.h"

#include <strstream.h>

class List: public Value, public Wrapper<List,int>
{
protected:
	Widget form,list,toolbar;
	
	Value *def;
	LinkedList<Value*> vals;
	long maxlen;
	
	/* Xt Callbacks */
	void Selection(Widget w,int,XtPointer);
	void Button(Widget w,int,XtPointer);
	
	void ReNumber();
	Widget CreateButton(Widget parent,char *name);

	XmStringTable cutbuf;
	int cutbuf_len;
	XmString BuildString(Value* v,int cnt);
	XmStringTable CopyTable(XmStringTable src,int len);
	void KillTable(XmStringTable& tbl,int& len);
	void MultiXmSelect(int* poslist,int cnt,int firstpos = -1);

	void UpdateWidget();
	void UpdateToolbar(int selcnt);
	
public:
	List(NetClass* nc = 0,bool hidden = false);
	List(const List& l);
	~List();
	Value* Copy() const { return new List(*this); }
	Value* Cast() const { return (Value*)this; }

	bool IsList() const { return true; }
	
	bool Composite() const { return true; }
	bool Resizable() const { return true; }

	Widget CreateWidget(Widget parent);
	bool EvalWidget();
	
	istream& Parse(istream& is);
	istream& Input(istream& is);
	ostream& Output(ostream& os) const;
	
	Value* EntryType() const { return def; }
	int Length() const { return vals.Length(); }
	bool Append(Value* v);
	bool Insert(Value* v,int pos);
	bool Remove(Value* v);
	void Clear(bool update = true);
	
	Iterator<Value*> Entries() { return Iterator<Value*>(vals); }
	Value* GetEntry(int pos) { Value** vp = vals[pos]; return vp?*vp:0; }
	
	operator int() const { return vals.Length(); }
	operator double() const { return int(); }

	Value& operator =(const List& l);
	Value& operator =(const Value& v);

private:
	friend class Operation;
	Value* Idx(int pos) { return GetEntry(pos); }
};

typedef List* ListP;

#endif
