#ifndef __PAGE_H__
#define __PAGE_H__

#include "xbefore.h"
#include <X11/Xlib.h>
#include "xafter.h"

#include <iostream.h>

#include "linked.h"
#include "array.h"
#include "ident.h"
#include "position.h"
#include "text.h"
#include "event.h"
#include "struct.h"

class Net;
class Node;
class Connector;
class Editor;
class EditorSettings;
class NetClass;
class Object;
class ObjectClass;

class Page;
class Page
{
	Net* net;
	NetClass* netcl;
	Node* parent;
//	Struct attr;

	LinkedList<Object*> objects;
	Iterator<Object*> oiter;
	LinkedList<Node*> nodes;
	Iterator<Node*> niter;
	LinkedList<Connector*> connectors;
	Iterator<Connector*> citer;

	Object* curr_ob;
	
	Position dragbox_start,dragbox_dim;
	
	friend class Net;
	friend class Object;
	friend class Node;
	friend class Connector;
	void AddObject(Object* ob);
	void RemoveObject(Object* ob);

	bool Read(Scope& sn,istream& is);
	void Write(ostream& os);
	void Renumber();
	Text FullName();

	void InitActives(bool linked_conns = true);
	LinkedList<Object*> actives;

	Object* UnderMouse(Display* dpy,Position& pos);
	Node* NodeForMelt(Display* dpy,Position& pos);

	void Menu(Editor&,Ident& name);
	void Draw(Editor& e,Display* dpy,Drawable d,bool after);
	bool Event(Editor& e,EventInfo& ev);

  bool (Page::*action) (Editor&,EventInfo&,bool = true);
  Ident curr_action; // fuer SPNLControl
  bool desel_allowed; // fuer SPNLControl
  bool melt_allowed; // fuer SPNLControl

	Position last_pos;
	bool Moving(Editor& editor,EventInfo& ev,bool undo_copy=true);
	double last_angle;
  bool Rotating(Editor& editor,EventInfo& ev,bool dummy=true);
  bool Dragging(Editor& editor,EventInfo& ev,bool dummy=true);
  bool Scaling(Editor& editor,EventInfo& ev,bool dummy=true);
  bool ConnCreate(Editor& editor,EventInfo& ev,bool dummy=true);
	void FinishAction();

  friend class TemplateIncluder;
  friend class TimeNETLink;
	void PageCopy(Page& src,bool selection = false,Array<Editor*,Page*>* pages = 0);
	Object* NewObject(ObjectClass* obcl);
	Object* GetObject(unsigned id);
	Node* GetNode(unsigned id);
	Connector* GetConnector(unsigned id);

	Node* NearestNode(Position ps,Position& nearest);

	void Dragbox(Display *dpy,Drawable d,GC gc,double zoom);

	void SetNet(Net* n) { net = n; }
  void SetClass(NetClass* cl) { netcl = cl; }

public:
  void ForbidDeselection() { desel_allowed = false; }
  void ResetDeselection() { desel_allowed = true; }
  void ForbidMelting() { melt_allowed = false; }
  void ResetMelting() { melt_allowed = true; }
  Ident CurrAction() { return curr_action; }
  bool SubMoving(Editor& editor,EventInfo& ev,bool dummy=true); // Funktionen waren
  Object* GetCurrOb() { return curr_ob; }       // ehemals privat,
  void SetCurrOb(Display* dpy,Position& pos)    // werden aber von Modul
       { curr_ob = UnderMouse(dpy,pos); }       // SPNLControl ben"otigt

  bool DragboxInside(Position pos);
               
	Net* GetNet() { return net; }
	NetClass* Class() { return netcl; }

	Node* Parent() { return parent; }
	Page* SuperPage();
	Page* Prime();

	int NumObjects() { return int(objects); }
	int NumNodes() { return int(nodes); }
	int NumConnectors() { return int(connectors); }
	Iterator<Object*> Objects() { return Iterator<Object*>(objects); }
	Iterator<Node*> Nodes() { return Iterator<Node*>(nodes); }
	Iterator<Connector*> Connectors()
		{ return Iterator<Connector*>(connectors); }
	Node* GetNode(const char* name);
	Connector* GetConnector(const char* name);

	bool NameExists(Object* nob,bool check = false);

	void UpdateHierarchy();
	void Update();
	
	void Clear();
	void Deselect();
	void Remove();
	void Select(char* expr = 0);
	void Copy();
	void Paste();
        void Paste(Editor& ed,Display* dpy,Position pos);
	Object* NextSelected(Object* curr = 0);
	void SelectNextObject(bool fwd);
	Page* CurrSubPage();

	Page(Net* n,NetClass* nc,Node* parent = 0);
	Page(Page& p,Net* parent,bool selection = false,Array<Editor*,Page*>* pages = 0);
	Page* Copy(Net* parent,bool selection = false,Array<Editor*,Page*>* pages = 0);
	~Page();

	istream& Input(istream&);
	ostream& Output(ostream&);
};

typedef Page* PageP;

inline istream& operator >>(istream& is,Page& n) {	return n.Input(is); }
inline ostream& operator <<(ostream& os,Page& n) {	return n.Output(os); }

#endif
