#ifndef __TEXT_H__
#define __TEXT_H__

#include <strstream.h>
extern "C" {
#include <ctype.h>
#include <stdlib.h>
}
#include "value.h"

class Text;
class Text: public Value
{
protected:
	Widget scroll,text;
	short rows;
	char* p;
	
	char* Sure(char* s) const { return s?s:""; }

public:
	Text(NetClass* nc = 0,bool hidden = false);
	Text(const char* s);
	Text(const Text& t);
	~Text() { UncatchDestroy(&scroll,&text); if (p) delete[] p; }
	Value* Copy() const { return new Text(*this); }
	Value* Cast() const { return Copy(); }
	
	bool Composite() const { return true; }
	bool Resizable() const { return rows>1; }
	bool Defined() const { return p!=0; }
	
	int Length() { return p?strlen(p):0; }
	Text Left(int i);
	
	Widget CreateWidget(Widget parent);
	bool EvalWidget();
	
	istream& Parse(istream& is);
	istream& Input(istream& is);
	ostream& Output(ostream& os) const;
	
	operator char*() const { return p; }
	operator bool() const { return p!=0; }
	operator double() const { return p && isdigit(*p)?atof(p):0.0; }
	
	Value& operator =(const char* s);
	Value& operator =(const double d);
	Value& operator =(const int i);
	Value& operator =(const Text& s) { return *this=s.p; }
	Value& operator =(const Value& v) { return *this=(char*)v; }
	
	bool operator ==(const char* v) const;
	bool operator !=(const char* v) const;
	bool operator >(const char* v) const;
	bool operator <(const char* v) const; // alphabetischer String-Vergleich
	Text operator -() const; // String invertieren
	Text operator -(const char* v) const; // String-Subtraktion
	Text operator +(const char* v) const; // String-Addition
	Text operator *(const char* v) const;
	Text operator /(const char* v) const { return *this-v; } // Abbildung auf -

	Text& operator +=(const char* s);
	Text& operator +=(const char c);
	Text& operator +=(Text& t) { return *this+=(const char*)t; }
  Text& operator +=(const double d);

private:
	friend class Operation;
	Value* Eq(Value& v);
	Value* Gt(Value& v);
	Value* Lt(Value& v);
	Value* Neg() { return new Text(-*this); }
	Value* Not(); // String-Laenge
	Value* Sub(Value& v) { return new Text(*this-v); }
	Value* Add(Value& v) { return new Text(*this+v); }
	Value* Mul(Value& v) { return new Text(*this*v); }
	Value* Div(Value& v) { return new Text(*this/v); }
	
	Value* And(Value& v) { return new Text(*this+v); }
	Value* Or(Value& v) { return new Text(*this+v); }
};

typedef Text* TextP;

#endif
