// -----------------------------------------------------------------
// Importe
// -----------------------------------------------------------------

#include "tokengam.h"
// Tokengameklasse

// -----------------------------------------------------------------
// EDSPNTokengame-Klassendeclaration
// -----------------------------------------------------------------

class EDSPNTokengame: public Tokengame
{
protected:

  bool              ParserError;
  enum parser_type  ParserType;
  ObjectP           ParserObject, ParserObject2;
  TextP             ParserText;
  char              buffer[64000];
  bool              RecursionError;
  LinkedList<NodeP> Recursion;
  // Fuer die Fehlerausgaben des Parser

  LinkedList<NodeP> Immediates;
  LinkedList<NodeP> Timed;

  bool BackwardEnable, ForwardEnable;
  // Statusvariablen

  signed int  Steps, StepsCnt;
  // fuer Backward/Forward

  int  HighestPriority;
  // hoechste Prioritaet der Immediates

  bool ConfigurationChange;
  // Hat sich die Konfiguration veraendert?

  void ParseAllMarkings();
  NodeP FindMarkingParameter(charP Name);

  void MakeReadyToFire(NodeP node);

  void CollectEnabledTrans();
  bool Enabled(NodeP node);
  void ShowEnabledTrans();

  void SaveParams(StructP params);
  Text Int2Steps(int value);
  int  Steps2Int(StructP params);
  void OnDialogButton(Ident& button);
  // Dialog-Hilfsfunktionen

// -----------------------------------------------------------------
public:

  EDSPNTokengame(Net& n): Tokengame(n), ParserError(false), ParserType(none), 
                         ParserObject(NULL), ParserObject2(NULL), 
                         ParserText(NULL), RecursionError(false),
                         BackwardEnable(false), ForwardEnable(false), 
                         Steps(1), StepsCnt(1),
                         HighestPriority(0), 
                         ConfigurationChange(true)
    {
      name= new char[15];sprintf(name,"%s","EDSPNTokengame");
    };

  ~EDSPNTokengame()
    { 
      delete[] (this->name);
    };

  bool Init() {return (net.Prime()->Class()->Name() == "EDSPN");}; 

  void          ParseError();
  unsigned long ParseMarking(charP string, unsigned int length);
  double        ParseParameter(charP string, unsigned int length);
  double        ParseRealParameter(charP string, unsigned int length);
  unsigned long ParseIntegerParameter(charP string, unsigned int length);
  // Parser-Hilfsfunktionen

  bool  IsPlace(NodeP);
  bool  IsTransition(NodeP);
  bool  IsImmediate(NodeP node);
  bool  IsInhibitor(ConnectorP conn);
  bool  IsParameter(NodeP node);
  bool  IsMarkPar(NodeP node);
  bool  IsDelayPar(NodeP node);
  TextP GetMarking(NodeP);
  int   GetPriority(NodeP node);
  TextP Get_md_enable(NodeP node);
  TextP GetMultiplicity(ConnectorP c);
  int   GetMarkPar(NodeP);
  TextP GetDelayPar(NodeP);
  TextP GetColor(NodeP);
  // Tokengame-Hilfsfunktionen

  void  StartTokengame();
  void  EndTokengame();

  bool  EventEnable();
  void  OpenOptionsMenu();

  bool  SelectEnabledTrans(PageP pg);
  bool  DeselectEnabledTrans(PageP pg);

  void  OpenFireMenu(Node& node);
  void  FireTrans(NodeP node, int, bool undo);
  // Tokengame-Hauptfunktionen
};

MODULE_NOTIFY(EDSPNTokengame)

// -----------------------------------------------------------------
// protected Tokengame-Hilfsfunktionen
// -----------------------------------------------------------------

void EDSPNTokengame::ParseAllMarkings() 
{
  PageP pg = net.Prime();
  
  Iterator<NodeP> ListNodes = pg->Nodes();
  for (NodeP node=ListNodes; node; node=++ListNodes)
  // fuer jeden Knoten tue
    
    if (IsPlace(node)) 
    // Knoten ist eine Stelle?
      
      if (isalpha(((charP)*GetMarking(node))[0]))
	{// Markierungsparameter in der Stelle?
	  
	  NodeP markpar= FindMarkingParameter(*GetMarking(node));
	  if (markpar)
	    *GetMarking(node)= GetMarkPar(markpar);
	  else
	    {
	      ParserType= none;
	      ParserObject= node;
	      ParserObject2= NULL;
	      ParserText= GetMarking(node);
	      *outs << "mark_par does not exist:\n";
	      Out(*GetMarking(node), strlen(*GetMarking(node)));
	    }
	}
}

// -----------------------------------------------------------------

NodeP EDSPNTokengame::FindMarkingParameter(charP Name)
{
  PageP pg = net.Prime();

  NodeP node;
  Iterator<NodeP> ListNodes = pg->Nodes();
  for (node=ListNodes; node; node=++ListNodes)
    if (!strcmp((charP)node->Name(), Name))
      if (IsParameter(node) && IsMarkPar(node))
	return (node);

  return NULL;
}

// -----------------------------------------------------------------

void EDSPNTokengame::MakeReadyToFire(NodeP node)
{
  FireItemP item= new FireItem(node);
  
  int *index= new int;
  *index= 0;
  item->Bindings.Append(index);
  // nur ein Uebergang ist je Transition moeglich (Nummer 0), gemaess Netzklasse
  
  ReadyToFire.Append(item);
}

// -----------------------------------------------------------------

void EDSPNTokengame::CollectEnabledTrans() 
{
  PageP pg = net.Prime();
  
  Iterator<NodeP> ListNodes = pg->Nodes();
  for (NodeP n=ListNodes; n; n=++ListNodes)
    {
      if (IsTransition(n) && Enabled(n))
	{// Transition, elementar und schaltfaehig?
	  
	  if (IsImmediate(n))
	    {// Immediate Transition?
	      
	      if (GetPriority(n) > HighestPriority)
		HighestPriority= GetPriority(n);
	      // hoechste Prioritaet merken
	      
	      Immediates.Append(n);
	      // zeitlose Transition merken
	    }
	  
	  else
	    Timed.Append(n);
	  // und zeitbehaftete Transition merken
	}    
    }
}

// -----------------------------------------------------------------

bool  EDSPNTokengame::Enabled(NodeP node)
{
  if (IsImmediate(node))
    {
      ParserType= md_enable;
      ParserObject= node;
      ParserObject2= NULL;
      ParserText= Get_md_enable(node);
      Recursion.Clear();

      if (!Parse_md_enable(*Get_md_enable(node)))
	// global guard disabled?

	return false;
    }
  
  Iterator<ConnectorP> InputConns = node->Inputs();
  for (ConnectorP c= InputConns; c; c=++InputConns)
    {
      ParserType= md_arc_mult;
      ParserObject= c;
      ParserObject2= NULL;
      ParserText= GetMultiplicity(c);
      Recursion.Clear();

      if (IsInhibitor(c))
	{// hemmende Kante
	  
	  if (double(*GetMarking(c->GetInput())) >=
	      Parse_md_arc_mult(*GetMultiplicity(c)))
	    return false;
	}
      else
	{// Eingangskante
	  
	  if (double(*GetMarking(c->GetInput())) < 
	      Parse_md_arc_mult(*GetMultiplicity(c)))
	    return false;
	}
    }

  return true;
  // Transition ist schaltfaehig
}

// -----------------------------------------------------------------

void EDSPNTokengame::ShowEnabledTrans()
{
  for (int i= 0; i<ReadyToFire.Length(); ++i)
    (*ReadyToFire[i])->node->Select(true);

  if (BackwardEnable)
    {
      if (!--StepsCnt || !FiredCount)
	{
	  BackwardEnable= false;
	  StepsCnt= Steps;
	}
      
      if (FiredCount)
	{
	  FiredItemP item= *Fired[FiredCount-1];
	  
	  SelectTrans(item->node);
	  
	  if (FiredCount==FiredCount_save)
	    {
	      Text DialogContense;
	      DialogContense+= (charP)Text("Warning: this command ");
	      DialogContense+= (charP)Text("may lead to an inconsistent ");
	      DialogContense+= (charP)Text("net-status, ");
	      DialogContense+= (charP)Text("if you changed the net!");
	      
	      OpenMsgDialog(*this->ed, "EDSPNTokengame_Warning",
			    (charP)DialogContense,false,"Cancel");
	    }
	  
	  item->Do(BACKWARD, this);  

	  this->ed->Update();
	  // Editor-Screen update
	}
    }
  else if (ForwardEnable)
    {
      if (!--StepsCnt || FiredCount==Fired.Length())
	{
	  ForwardEnable= false;
	  StepsCnt= Steps;
	}
      
      if (FiredCount<Fired.Length())
	{
	  FiredItemP item= *Fired[FiredCount];
	  
	  SelectTrans(item->node);
	  
	  item->Do(FORWARD, this);

	  this->ed->Update();
	  // Editor-Screen update
	}
    }
}

// -----------------------------------------------------------------

void EDSPNTokengame::SaveParams(StructP params)
{
  Steps= Steps2Int(params);
  StepsCnt= Steps;
}

// -----------------------------------------------------------------

Text EDSPNTokengame::Int2Steps(int value)
{
  if (value == 1)
    return Text("onestep");
  else if (value == 10)
    return Text("tensteps");
  else
    return Text("fiftysteps");
}

// -----------------------------------------------------------------

int EDSPNTokengame::Steps2Int(StructP params)
{
  charP member= (charP)*params->GetMember("steps");

  if (!strcmp(member,"onestep"))
    return (1);
  else if (!strcmp(member,"tensteps"))
    return (10);
  else if (!strcmp(member,"fiftysteps"))
    return (50);
  else return (1);
}

// -----------------------------------------------------------------

void EDSPNTokengame::OnDialogButton(Ident& button)
{

  StructP params= GetParams();
  
  if ((button=="Forward") && EventEnable())
    {// Forward angeklickt?

      SaveParams(params);

      ForwardEnable= true;
    }
  else if ((button=="Backward") && EventEnable())
    {// Backward angeklickt?

      SaveParams(params);

      BackwardEnable= true;
    }
  else if ((button=="Stop"))
    {// Stop angeklickt?

      SaveParams(params);

      StepsCnt= Steps;
      BackwardEnable= false;
      ForwardEnable= false;
    }
  else if ((button=="Close"))
    {// Close angeklickt?

      SaveParams(params);

      CloseDialog();
      DestroyDialog();
    }
  else if (button=="Cancel")
    {// Cancel angeklickt?

      if (params->GetMember("end"))
	{// Cancel bei Ende des Tokengame?

	  TokenGameEnable= true;
	  XmToggleButtonSetState(this->ToggleWidget,true,true);
	  // ToggleButton selektieren
	}

      CloseDialog();
      DestroyDialog();
    }
  else if ((button=="Quit"))
    {// Quit angeklickt?

      DeInitTokengame(false);

      CloseDialog();
      DestroyDialog();
    }
  else if ((button=="Restore"))
    {// Restore angeklickt?

      DeInitTokengame(true);

      CloseDialog();
      DestroyDialog();
    }
}

// -----------------------------------------------------------------
// Parser-Hilfsfunktionen
// -----------------------------------------------------------------

void EDSPNTokengame::ParseError()
{
  ParserError= true;

  Text DialogContense;
  
  if (ParserType==md_enable)
    DialogContense+= (charP)
      Text("an error occured in function Parse_md_enable.\n");
  else if (ParserType==md_arc_mult)
    DialogContense+= (charP)
      Text("an error occured in function Parse_md_arc_mult.\n");
  
  if (ParserObject && ParserObject2)
    {
      DialogContense+= (charP)Text("while parsing:\n");
      DialogContense+= (charP)ParserObject->Class()->GroupName();
      DialogContense+= (charP)Text("(");
      DialogContense+= (charP)ParserObject->Class()->Name();
      DialogContense+= (charP)Text(") ");
      DialogContense+= (charP)ParserObject->Name();
      DialogContense+= (charP)Text("\n");

      DialogContense+= (charP)Text("last object was:\n");
      DialogContense+= (charP)ParserObject2->Class()->GroupName();
      DialogContense+= (charP)Text("(");
      DialogContense+= (charP)ParserObject2->Class()->Name();
      DialogContense+= (charP)Text(") ");
      DialogContense+= (charP)ParserObject2->Name();
      DialogContense+= (charP)Text("\n");
    }
  else if (ParserObject)
    {
      DialogContense+= (charP)Text("last object was:\n");
      DialogContense+= (charP)ParserObject->Class()->GroupName();
      DialogContense+= (charP)Text("(");
      DialogContense+= (charP)ParserObject->Class()->Name();
      DialogContense+= (charP)Text(") ");
      DialogContense+= (charP)ParserObject->Name();
      DialogContense+= (charP)Text("\n");
    }
  
  if (ParserText)
    {
      DialogContense+= (charP)Text("last string was:\n");
      DialogContense+= (charP)*ParserText;
      DialogContense+= (charP)Text("\n");
    }

  DialogContense+= buffer;
  // und nun noch die Meldungen des Parsers
  
  OpenMsgDialog(*this->ed, "EDSPNTokengame_Error",
		DialogContense,true,"Cancel");
}

// -----------------------------------------------------------------

unsigned long EDSPNTokengame::ParseMarking(charP string, unsigned int length)
{// wird von ParseIntegerValue aufgerufen

  PageP pg = net.Prime();
  
  Iterator<NodeP> ListNodes = pg->Nodes();

  for (NodeP node=ListNodes; node; node=++ListNodes)
    // fuer alle Knoten tue

    if (IsPlace(node) && (length-1==strlen(node->Name())) && 
	!strncmp(node->Name(), string+1, length-1))
      // Stelle mit gewuenschtem Namen gefunden (ohne # am Anfang, z.B. "#P1")?

      if (((charP)*GetMarking(node))[0])
	// kein Leerstring?

	return (unsigned long)
	  (double(*GetMarking(node)));
        // Markeninhalt ist ganzzahliger Wert

      else
	return 0;
        // kein Markeninhalt

  ParserObject2= NULL;
  ParserText= TextP(NULL);

  *outs << "Place does not exist:\n";
  Out(string+1, length-1);

  return 0;
}

// -----------------------------------------------------------------

double EDSPNTokengame::ParseParameter(charP string, unsigned int length)
{// wird von ParseRealValue aufgerufen

  return (ParseRealParameter(string, length));
  // Parameter sind in dieser Netzklasse immer Integer
  // aber es wird mit double-Zahlen gerechnet
}

// -----------------------------------------------------------------
double EDSPNTokengame::ParseRealParameter(charP string, unsigned int length)
{// wird von ParseParameter aufgerufen

  return (double)(ParseIntegerParameter(string, length));
  // RealParameter sind casted IntegerParameter
}

// -----------------------------------------------------------------
unsigned long EDSPNTokengame::ParseIntegerParameter(charP string, 
						    unsigned int length)
{// wird von ParseRealParameter aufgerufen

  if (RecursionError)
    return 0;

  PageP pg = net.Prime();
  
  Iterator<NodeP> ListNodes = pg->Nodes();
  
  for (NodeP node=ListNodes; node; node=++ListNodes)
    // fuer alle Knoten tue
    
    if (IsParameter(node))
      // suche einen Parameter

      if ((length==strlen(node->Name()) &&
	   !strncmp(node->Name(), string, length)))
	// Parameter mit gewuenschtem Namen gefunden?
	
	if (IsMarkPar(node))
	  // Stelle ist MarkingParameter?
	  
	  return (unsigned long)
	    (GetMarkPar(node));
  
	else if (IsDelayPar(node))
	  {// Stelle ist DelayParameter?
	    
	    if (Recursion.IsIn(node))
	      {
		RecursionError= true;
		*outs << "illegal recursion:\n";
		Out(string, length);
		return 0;
	      }
	    
	    Recursion.Append(node);

	    ParserObject2= node;
	    ParserText= GetDelayPar(node);

	    unsigned long value= (unsigned long)
	      (Parse_param_def(*GetDelayPar(node)));

	    Recursion.Remove(node);

	    return value;
	  }
	else 
	  {
	    *outs << "unknown Parameter type:\n";
	    Out(string, length);
	    return 0;
	  }
  *outs << "Parameter does not exist:\n";
  Out(string, length);
  return 0;
}

// -----------------------------------------------------------------
// Tokengame-Hilfsfunktionen
// -----------------------------------------------------------------

bool EDSPNTokengame::IsPlace(NodeP node)
{
  return (!strcmp(node->Class()->GroupName(),"Places") ? true : false);
}


// -----------------------------------------------------------------

bool EDSPNTokengame::IsTransition(NodeP node)
{
  return (!strcmp(node->Class()->GroupName(),"Transition") ? true : false);
}

// -----------------------------------------------------------------

bool EDSPNTokengame::IsImmediate(NodeP node)
{// vorher IsTransition() aufrufen
  return (!strcmp(node->Class()->Name(),"ImmTrans") ? true : false);
}

// -----------------------------------------------------------------

bool EDSPNTokengame::IsInhibitor(ConnectorP conn)
{
  return (!strcmp(conn->Class()->Name(),"Inhibitor") ? true : false);
}

// -----------------------------------------------------------------

bool EDSPNTokengame::IsParameter(NodeP node)
{
  return (!strcmp(node->Class()->GroupName(),"Parameter") ? true : false);
}

// -----------------------------------------------------------------

bool EDSPNTokengame::IsMarkPar(NodeP node)
{// vorher IsParameter() aufrufen
  return (!strcmp(node->Class()->Name(),"MarkPar") ? true : false);
}

// -----------------------------------------------------------------

bool EDSPNTokengame::IsDelayPar(NodeP node)
{// vorher IsParameter() aufrufen
  return (!strcmp(node->Class()->Name(),"DelayPar") ? true : false);
}

// -----------------------------------------------------------------

TextP EDSPNTokengame::GetMarking(NodeP node)
{
  return (TextP)node->Attr("marking");
}

// -----------------------------------------------------------------

int EDSPNTokengame::GetPriority(NodeP node)
{
  return ((int)(*node->Attr("priority")));
}

// -----------------------------------------------------------------

TextP EDSPNTokengame::Get_md_enable(NodeP node)
{
  return (TextP)node->Attr("md_enable");
}

// -----------------------------------------------------------------

TextP EDSPNTokengame::GetMultiplicity(ConnectorP c)
{
  return (TextP)c->Attr("multi");
}

// -----------------------------------------------------------------

int EDSPNTokengame::GetMarkPar(NodeP node)
{
  return (int)*node->Attr("value");
}

// -----------------------------------------------------------------

TextP EDSPNTokengame::GetDelayPar(NodeP node)
{
  return (TextP)node->Attr("delay");
}

// -----------------------------------------------------------------

TextP EDSPNTokengame::GetColor(NodeP node)
{
  return (TextP)node->Attr("selcol");
}

// -----------------------------------------------------------------
// Tokengame-Hauptfunktionen
// -----------------------------------------------------------------

void  EDSPNTokengame::StartTokengame() 
{
  Text DialogContense;
  DialogContense+= (charP)Text("Instructions:\nto fire a transition,\n");
  DialogContense+= (charP)Text("double-click on the right mouse-button!");
  
  OpenMsgDialog(*this->ed, "EDSPNTokengame_Info",
		(charP)DialogContense,false,"Cancel");

  outs= new ostrstream(buffer, sizeof(buffer), ios::trunc);

  HighestPriority= 0;

  ConfigurationChange= true;

  ParserError= false;
  RecursionError= false;

  ParseAllMarkings();

  this->ed->Update();
  // Editor-Screen update
}

// -----------------------------------------------------------------

void  EDSPNTokengame::EndTokengame() 
{
  ConfigurationChange= true;

  HighestPriority= 0;

  Timed.Clear();
  Immediates.Clear();

  StepsCnt= Steps;
  BackwardEnable= false;
  ForwardEnable= false;

  ParserType= none;
  ParserObject= NULL;
  ParserObject2= NULL;
  ParserText= NULL;
  Recursion.Clear();
  delete (outs);
}

// -----------------------------------------------------------------

bool  EDSPNTokengame::EventEnable() 
{
 
  return (!BackwardEnable && !ForwardEnable);
}

// -----------------------------------------------------------------

void EDSPNTokengame::OpenOptionsMenu()
{
  Text DialogContense;
  
  DialogContense+= (charP)Text("[enum ( \"1 step\" onestep, ");
  DialogContense+= (charP)Text("\"10 steps\" tensteps, ");
  DialogContense+= (charP)Text("\"50 steps\" fiftysteps ) ");
  DialogContense+= (charP)Text("\"Steps for Forward/Backward\"steps= ");
  DialogContense+= (charP)Int2Steps(Steps);
  DialogContense+= (charP)Text("]");
  CreateDialog((charP)DialogContense);

  OpenDialog(*this->ed, "EDSPNTokengame_Control", 
	     "Backward Forward Stop Close");
}

// -----------------------------------------------------------------

bool  EDSPNTokengame::SelectEnabledTrans(PageP pg= 0) 
{
  if (!pg) pg= net.Prime();

  if (ParserError)
    {// error nach FireTrans()?

      DeInitTokengame(true);
  
      XmToggleButtonSetState(this->ToggleWidget,false,true);
      // ToggleButton deselektieren
      
      return false;
    }

  if (ConfigurationChange)
    {
      ConfigurationChange= false;

      HighestPriority= 0;

      Timed.Clear();
      Immediates.Clear();

      CollectEnabledTrans();
      // Alle schaltfaehigen Transitionen in Listen merken

      if (ParserError)
	{// error nach CollectEnabledTrans()?

	  DeInitTokengame(true);
  
	  XmToggleButtonSetState(this->ToggleWidget,false,true);
	  // ToggleButton deselektieren

	  return false;
	}
      

      if (Immediates.Length())
	{// zeitlose Transitionen haben Vorrang 
	  
	  Iterator<NodeP> ListImmTrans = Immediates;
	  for (NodeP n= ListImmTrans; n; n= ++ListImmTrans)
	    if (GetPriority(n) == HighestPriority)
	      MakeReadyToFire(n);
	  // nur zeitlose Transitionen hoechster Prioritaet
	}
      else
	{
	  Iterator<NodeP> ListTimedTrans = Timed;
	  for (NodeP n= ListTimedTrans; n; n= ++ListTimedTrans)
	    MakeReadyToFire(n);
	  // alle zeitbehafteten Transitionen sind gleichberechtigt
	  
	 }
    }
  ShowEnabledTrans();

  return true;
}

// -----------------------------------------------------------------

bool  EDSPNTokengame::DeselectEnabledTrans(PageP) 
{
  DeselectAllTrans();
  
  return true;
}

// -----------------------------------------------------------------

void  EDSPNTokengame::OpenFireMenu(Node& node) 
{
  FireItemP item;
  Iterator<FireItemP> ListItems= ReadyToFire;
  for (item= ListItems; (item && (*item!=&node)); item= ++ListItems);
  if (item)
    {
      int index= 0;
      item->Fire(&index, this);
    }
  // die Auswahl eines Uebergangs der Transition entfaellt,
  // da nur Uebergang Nummer 0 existiert, gemaess Netzklasse
}

// -----------------------------------------------------------------

void  EDSPNTokengame::FireTrans(NodeP node, int, bool undo) 
{
  ConnectorP c;

  if (undo)
    {// umgekehrtes Schalten entspricht undo  

      Iterator<ConnectorP> OutputConns = node->Outputs();
      for (c= OutputConns; c; c=++OutputConns)
	{
	  ParserType= md_arc_mult;
	  ParserObject= c;
	  ParserObject2= NULL;
	  ParserText= GetMultiplicity(c);
	  Recursion.Clear();

	  *GetMarking(c->GetOutput())= double(*GetMarking(c->GetOutput())) -
	                               Parse_md_arc_mult(*GetMultiplicity(c));
	}
      Iterator<ConnectorP> InputConns = node->Inputs();
      for (c= InputConns; c && !IsInhibitor(c); c=++InputConns)
	{
	  ParserType= md_arc_mult;
	  ParserObject= c;
	  ParserObject2= NULL;
	  ParserText= GetMultiplicity(c);
	  Recursion.Clear();

	  *GetMarking(c->GetInput())=  double(*GetMarking(c->GetInput()))  +
	                               Parse_md_arc_mult(*GetMultiplicity(c));
	}
    }
  else
    {
      Iterator<ConnectorP> InputConns = node->Inputs();
      for (c= InputConns; c && !IsInhibitor(c); c=++InputConns)
	{
	  ParserType= md_arc_mult;
	  ParserObject= c;
	  ParserObject2= NULL;
	  ParserText= GetMultiplicity(c);
	  Recursion.Clear();

	  *GetMarking(c->GetInput())=  double(*GetMarking(c->GetInput()))  -
	                               Parse_md_arc_mult(*GetMultiplicity(c));
	}
      Iterator<ConnectorP> OutputConns = node->Outputs();
      for (c= OutputConns; c; c=++OutputConns)
	{
	  ParserType= md_arc_mult;
	  ParserObject= c;
	  ParserObject2= NULL;
	  ParserText= GetMultiplicity(c);
	  Recursion.Clear();

	  *GetMarking(c->GetOutput())= double(*GetMarking(c->GetOutput())) +
	                               Parse_md_arc_mult(*GetMultiplicity(c));
	}
    }
  
  this->ed->Update();
  // Editor-Screen update

  ReadyToFire.Clear();

  ConfigurationChange= true;

  SetModified();
  // Das Netz wurde modifiziert
}

// -----------------------------------------------------------------
