// -----------------------------------------------------------------
// Debug Definitionen
// -----------------------------------------------------------------

//#define __OUTLIST__ 
// Ausgabe der Stellen und Transitionslisten

// -----------------------------------------------------------------
// Importe
// -----------------------------------------------------------------

#include <signal.h>
// Fuer Signale

#include <iostream.h>
// Fuer Ein-/Ausgabe 

#include "../../MSTranslate/src/interface.h"
// Fuer die Erstellung der .table-Datei

#include "tokengam.h"
// Tokengameklasse

// -----------------------------------------------------------------
// Definitionen
// -----------------------------------------------------------------

#define NETFILENAME   ".net"
#define TABLEFILENAME ".table"
#define OUTFILENAME   ".out"
#define LISTFILENAME  ".list"
#define FIREFILENAME  ".fire"
// Dateikennungen

#define MAX_LENGTH 256
// maximale Laenge eines Transitionsnamens
// mit Pfadangabe und Indexnummern feuernder bindings
// Bei Aenderungen auch den Slave.cc aendern!

// -----------------------------------------------------------------
// HCPNTokengame-Klassendeclaration
// -----------------------------------------------------------------

class HCPNTokengame: public Tokengame
{ 
protected:

  ofstream FireFile;
  // Datei mit feuernden Transitionen

  int         pid;
  // Process-ID des gestarteten Slave

  bool        AniEnable, BackwardEnable, 
              ForwardEnable, 
              RealTime, Received;
  // Statusvariablen

  signed int  Steps, StepsCnt, Speed, SpeedCnt, LowestSpeed;
  // Schrittzaehler und Geschwindigkeiten fuer Automatismen 
  // (Backward/Forward/Animation)

  NodeP node;
  // die als naechstes zu schaltende Transition

  Dialog<HCPNTokengame,bool> dialog;
  // Fire-Dialogbox
  void DialogCallback(Dialog<HCPNTokengame,bool>& dia,bool,Ident& btn);
  // dazugehoeriger CallBack
  StructP setup;
  // Fuer die Parameterliste

  NodeP Fastest;
  int*  FastestB;
  double FastestTimePercentage;
  // vom Slave: schnellste Transition und binding und delay

  void ConnectionWithSlave(Text& netname);
  // Kontakt mit dem Slave herstellen
    
  Text Bool2Text(bool value);
  Text Int2Text(int value);
  Text Int2Steps(int value);
  int  Steps2Int(StructP params);

  void SetSpeed();
  // Setzt die Geschwindigkeit je nach "RealTime" nach
  // der derzeit schnellsten Transition oder auf 0

  bool ReadInput(ifstream& infile);
  // Empfaengt und liest die Listen des Slave ein

  NodeP FindNode(PageP pg, charP PathName, bool SearchForTrans,
		 bool SelectSubTrans);
  // Liefert den Knoten mit Namen SearchName 
  // (z.B. "\SubTrans_1\...\SubTrans_n\SearchName")

  void OnDialogButton(Ident& button);
  // Wenn ein Dialog-Button angeklickt wird

public:

  HCPNTokengame(Net& n): Tokengame(n),
                         pid(0),  
                         AniEnable(false), 
                         BackwardEnable(false), ForwardEnable(false), 
                         RealTime(true), Received(false), 
                         Steps(1), StepsCnt(1), Speed(0), SpeedCnt(0), 
                         LowestSpeed(20),
                         dialog(this), 
                         FastestB(0)
    {
      name= new char[14];sprintf(name,"%s","HCPNTokengame");
    }

  ~HCPNTokengame()      
    { 
      delete[] (this->name);
    }

  bool Init() {return (net.Prime()->Class()->Name() == "HCPN");}; 

  bool  IsPlace(NodeP node);
  bool  IsTransition(NodeP node);
  TextP GetMarking(NodeP node);
  TextP GetColor(NodeP node);

  void StartTokengame();
  void EndTokengame();

  bool EventEnable(){return (!AniEnable && !BackwardEnable && !ForwardEnable);};

  void OpenOptionsMenu();

  bool SelectEnabledTrans(PageP);
  bool DeselectEnabledTrans(PageP);

  void OpenFireMenu(Node& n);
  void FireTrans(NodeP, int, bool);
  
};

MODULE_NOTIFY(HCPNTokengame)

// -----------------------------------------------------------------
// HCPNTokengame-Klassendefinition Hilfsfunktionen
// -----------------------------------------------------------------

Text HCPNTokengame::Bool2Text(bool value)
{
  return (value ? Text("true") : Text("false"));
}

Text HCPNTokengame::Int2Text(int value)
{
  char buf[17];
  sprintf(buf,"%d",value);
  return Text(buf);
}

// -----------------------------------------------------------------

Text HCPNTokengame::Int2Steps(int value)
{
  if (value == 1)
    return Text("onestep");
  else if (value == 10)
    return Text("tensteps");
  else
    return Text("fiftysteps");
}

// -----------------------------------------------------------------

int HCPNTokengame::Steps2Int(StructP params)
{
  charP member= (charP)*params->GetMember("steps");

  if (!strcmp(member,"onestep"))
    return (1);
  else if (!strcmp(member,"tensteps"))
    return (10);
  else if (!strcmp(member,"fiftysteps"))
    return (50);
  else return (1);
}

// -----------------------------------------------------------------

void HCPNTokengame::SetSpeed()
{
  if (RealTime)
    Speed= (signed int)(LowestSpeed*FastestTimePercentage);
  else
    Speed= 0;
  SpeedCnt= Speed;
}

// -----------------------------------------------------------------

void HCPNTokengame::ConnectionWithSlave(Text& netname)
{
  Error error(*this->ed);
  Text sim_dir= agnes_dir + "/../MSSimulation/bin/";
  Text infile=  netname + TABLEFILENAME;
  Text outfile= netname + OUTFILENAME;
  if (((charP) infile)[0]!='/') infile=  model_dir + "/" + infile;  
  // Pfad korrigieren!
  if (((charP)outfile)[0]!='/') outfile= model_dir + "/" + outfile; 
  // Pfad korrigieren!
  
  ofstream file((charP)infile);
  
  AnalyserInterface::DescribeSymbols(net.Prime(), file); 
  // Alle Definitionen des Netzes speichern
  AnalyserInterface::DescribePage(net.Prime(), file, error);
  // Die Beschreibumg aller Bestandteile aller Seiten des Netzes speichern
  
  file.close();
  if (!file) cout << "HCPNTokengame: i cannot create the table-file!" << endl;
  
  Text params = sim_dir + "slave" + " -g " + " -n " + (charP)netname;

  pid= Exec(params, outfile, infile);
  // Simulator aufrufen mit Parametern Params
}

// -----------------------------------------------------------------
// HCPNTokengame-Klassendefinition Tokengamefunktionen
// -----------------------------------------------------------------

bool HCPNTokengame::IsPlace(NodeP node)
{
  return (!strcmp(node->Class()->GroupName(),"Place") ? true : false);
}

// -----------------------------------------------------------------

bool HCPNTokengame::IsTransition(NodeP node)
{
  return (!strcmp(node->Class()->GroupName(),"Transition") ? true : false);
}

// -----------------------------------------------------------------

TextP HCPNTokengame::GetMarking(NodeP node)
{
  return TextP(node->Attr("marking"));
}

// -----------------------------------------------------------------

TextP HCPNTokengame::GetColor(NodeP node)
{
  return (TextP(node->Attr("selcol")));
}

// -----------------------------------------------------------------

void HCPNTokengame::StartTokengame()
{
  Text DialogContense;
  DialogContense+= (charP)Text("Instructions:\nto fire a transition,\n");
  DialogContense+= (charP)Text("double-click on the right mouse-button!");
  
  OpenMsgDialog(*this->ed, "HCPNTokengame_Info",
		(charP)DialogContense,false,"Cancel");

  Text netname= Text(FileName()) - NETFILENAME;  
  ConnectionWithSlave(netname);
  // Slave starten
  
  Text firefile= netname + FIREFILENAME;
  if (((charP)firefile)[0]!='/') firefile= model_dir + "/" + firefile; 
  // richtiger Pfad!
  
  FireFile.open((charP)firefile, ios::trunc); FireFile.close();
  // firefile loeschen
}

// -----------------------------------------------------------------

void HCPNTokengame::EndTokengame()
{
  AniEnable=  false;
  StepsCnt= Steps;
  BackwardEnable= false;
  ForwardEnable= false;

  kill(pid, SIGINT);
  // Stoppen der Simulation
  
  Text netname=  Text(net.FileName()) - NETFILENAME;  
  Text infile=   netname + TABLEFILENAME;
  Text outfile=  netname + OUTFILENAME;
  Text firefile= netname + FIREFILENAME;
  Text listfile= netname + LISTFILENAME;
  if (((charP) infile)[0]!='/') infile=  model_dir + "/" + infile;  
  if (((charP)outfile)[0]!='/') outfile= model_dir + "/" + outfile; 
  if (((charP)firefile)[0]!='/') firefile= model_dir + "/" + firefile; 
  if (((charP)listfile)[0]!='/') firefile= model_dir + "/" + listfile; 
  // Pfade korrigieren!
  remove((charP)infile);
  remove((charP)outfile);
  remove((charP)firefile);
  remove((charP)listfile);
}

// -----------------------------------------------------------------

void HCPNTokengame::OpenOptionsMenu()
{
  Text DialogContense;
  
  DialogContense+= (charP)Text("[enum ( \"1 step\" onestep, ");
  DialogContense+= (charP)Text("\"10 steps\" tensteps, ");
  DialogContense+= (charP)Text("\"50 steps\" fiftysteps ) ");
  DialogContense+= (charP)Text("\"Steps for Forward/Backward\"steps= ");
  DialogContense+= (charP)Int2Steps(Steps);
  DialogContense+= (charP)Text(";bool \"RealTime-Animation\" realtime= ");
  DialogContense+= (charP)Bool2Text(RealTime);
  DialogContense+= (charP)Text(";int[1,100] \"Lowest Speed of RealTime-Animation\" lowest= ");
  DialogContense+= (charP)Int2Text (LowestSpeed);
  DialogContense+= (charP)Text("]");
  CreateDialog((charP)DialogContense);

  OpenDialog(*this->ed, "HCPNTokengame_Control", 
	     "Animate Backward Forward Stop Close");
}

// -----------------------------------------------------------------

void HCPNTokengame::OpenFireMenu(Node& node)
{
  Text DialogContense;
  ValueP val;
  int max,tmp;

  this->node= &node;
  // Dieser Knoten soll schalten

  FireItemP item;
  Iterator<FireItemP> FireList= ReadyToFire;
  for (item= FireList; (item && *item!=&node); item=++FireList);
  // Such das FireItem, mit der Trans. die gefeuert werden soll!

  if (item)
    {
      if (item->Bindings.Length() > 1)
	{
	  max= tmp= 0;
	  DialogContense+=("(enum(");
	  
	  Iterator<int*> bindings=item->Bindings;
	  for (int* b=bindings; b; b=++bindings) 
	    {// Alle feuerbaren Bindings traversieren

	      val= ((ListP)(node.Attr("bindings")))->GetEntry(*b);
	      // Binding holen
	      if ((tmp= strlen((charP)(*TextP(val)))) > max)
		max= tmp;
	      // maximale Laenge aller Bindings merken

	      DialogContense+= (charP)Text("\"");
	      DialogContense+= (charP)(*TextP(val));
	      DialogContense+= (charP)Text("\" binding");
	      DialogContense+= double(*b);
	      DialogContense+= (charP)Text(" , ");
	    }
	  DialogContense+= (charP)Text(")");
	  DialogContense+= (charP)Text("\" \" binding= binding");
	  DialogContense+= double(**item->Bindings[0]);
	  DialogContense+= (charP)Text(";)");
      
	  setup= new Struct((charP)(DialogContense));
	  dialog.Create("HCPNTokengameFireDialog",
			"select binding to fire",
			this->ed->Shell(),false,DialogCallback,"Ok Cancel");

	  Arg arg={"width",max*6+50};
	  XtSetValues(Widget(dialog),&arg,1);
         // set width

	  setup->EnterDialog(dialog,false);
	  dialog.Manage();
	}
      else if (Received)
	{
	  Received= false;

	  item->Fire(*item->Bindings[0], this);
	  // feuern des ersten schaltfaehigen bindings
	}
    }
}

// -----------------------------------------------------------------

void HCPNTokengame::FireTrans(NodeP node, int binding, bool Backward)
{
  char tmp[256],*tmp2,*tmp3;
  sprintf(tmp,"%s",(charP)node->FullName());
  // kopiert "/Path/Models/filename.net.Subtrans_1.Subtrans_n.TransName" nach tmp

  tmp2= tmp;
  tmp3= strtok(tmp,"/");
  while (tmp2) {tmp3= tmp2;tmp2= strtok(NULL,"/");};
  // abschneiden des kompletten Directorypfades
  strtok(tmp3, ".");
  // liefert den Dateinamen zurueck
  strtok(NULL, ".");
  // liefert die Filekennung zurueck
  // der TransName samt Pfad bleibt uebrig in strtok()

  Text netname= Text(FileName()) - NETFILENAME;  
  Text firefile= netname + FIREFILENAME;
  if (((charP)firefile)[0]!='/') firefile= model_dir + "/" + firefile; 
  // richtiger Pfad!
  
  FireFile.open((charP)firefile);
  // Oeffnen der Fire-Datei (open)

  int FirstTime= 0;
  charP name;
  while ((name= strtok(NULL, ".")))
    { 
      if (!FirstTime++)
	{
	  if (Backward)
	    FireFile << "Undo: ";
	  
	  FireFile << "\\__ROOT__\\";
	  // Am Anfang kommt ROOT
	}
      else
	FireFile << "\\";
      // und zwischen den Namen ein Trennzeichen

      FireFile << name;
    }

  FireFile  << " " << binding << endl;

  FireFile << flush;
  // Signal an den Slave

  FireFile.close();
  // Schliessen der Fire-Datei
}

// -----------------------------------------------------------------

NodeP HCPNTokengame::FindNode(PageP pg, charP PathName, bool SearchForTrans, 
			      bool SelectSubTrans)
{
  Iterator<NodeP> ListNodes = pg->Nodes();

  for (NodeP node=ListNodes; node; node=++ListNodes)
    {// gehe alle Knoten dieser Page durch

      if (strcmp((charP)(node->Name()), PathName))
	continue;
      // gesuchten Knoten noch nicht gefunden?

      if (!IsTransition(node))
	if (!SearchForTrans && IsPlace(node)) 
	  return (node);
        else 
	  continue;
      // Knoten ist keine Transition aber die gesuchte  Stelle, dann fertig
      // weder noch, dann wietermachen

      charP NextPath= strtok(NULL,"\\");

      if ((!NextPath &&  node->SubPage()) || (NextPath &&  !node->SubPage()))
	return (NULL);

      if (!node->SubPage())
	{
	  return (node);
	  // Elementartransition: dann fertig
	}
      else
	{
	  if (SelectSubTrans && SearchForTrans)
	    if (!node->Selected())
	      {
		node->Select(true);
		// Als eingetragen vermerken

		FireItemP item= new FireItem(node);
		ReadyToFire.Append(item);
		// SubTransition zum blinken eintragen
	      }
	  return (FindNode(node->SubPage(), NextPath, SearchForTrans, 
			   SelectSubTrans));
	  // Elementartransition in dem SubNetz heraussuchen
	}
    }
  return (NULL);
}

// -----------------------------------------------------------------

bool HCPNTokengame::ReadInput(ifstream& infile)
{
  char buffer[MAX_LENGTH];

  infile >> buffer;

  if (!strcmp(buffer,"Place-List:"))
    {

#ifdef __OUTLIST__
      cout  << endl << buffer << endl << endl;
#endif

      infile >> buffer;
      while (!infile.eof() && strcmp(buffer,"Transition-List:"))
	{
#ifdef __OUTLIST__
	  cout << buffer << " ";
#endif

	  NodeP node= FindNode(net.Prime(), strtok(buffer,"\\"), false, 
			       !AniEnable);

	  if (!node)
	    return false;
	  
	  infile >> buffer;	
#ifdef __OUTLIST__
	  cout << buffer << endl;
#endif
	  
	  TextP marking= GetMarking(node);
	  if (!strcmp(buffer,"0"))
	    *marking= "";
	  else
	    *marking= buffer;

	  infile >> buffer;		  
	}
#ifdef __OUTLIST__
      cout << endl;
#endif
    }
  if (!strcmp(buffer,"Transition-List:"))
    {
#ifdef __OUTLIST__
      cout << buffer << endl << endl;
#endif
      infile >> buffer;
      // Namen einlesen

      while (!infile.eof() && strcmp(buffer,"Fastest:"))
	{// solange die Datei noch nicht zu Ende ist
	 // und die naechsten Meldungen noch nicht da sind
	  
#ifdef __OUTLIST__
	  cout << buffer << " ";
#endif
	      
	  NodeP node= FindNode(net.Prime(), strtok(buffer,"\\"), true, 
	      	         true);
	  // Finde den Zeiger auf die Transition
	      
	  if (!node)
	    return false;
	      
	  FireItemP item= new FireItem(node);
	  // neues FireItem

	  while (!infile.eof())
	    {// Solange noch bindings kommen tue
		  
	      int* index= new int;
	      infile >> *index;
	      // neuen Index fuer Binding-Index Liste erzeugen
		  
	      if (*index < 0) 
		{
		  delete(index);
		  break;
		}
#ifdef __OUTLIST__
	      cout << *index << ",";
#endif
		  
	      item->Bindings.Append(index);
	    }
#ifdef __OUTLIST__
	  cout << endl;      
#endif
	  ReadyToFire.Append(item);
	 
	  infile >> buffer;
	  // Namen einlesen
	}

      if (!infile.eof())
	{
	  int FastestT;
	  
	  infile >> FastestT;
#ifdef __OUTLIST__
	  cout << "fastest: " << FastestT << " ";      
#endif
	  FireItemP item;
	  for (int i= 0; i<ReadyToFire.Length(); i++)
	    {// Alle Transitionen und SubTransitionen,
	      // die feuern koennen
	      
	      item= *(ReadyToFire[i]);
	      NodeP node= item->node;
	      
	      if (!node->SubPage())
		if(!FastestT--)
		  break;
	    }
	  this->Fastest= item->node;
	  
	  if (this->FastestB)
	    delete(this->FastestB);
	  this->FastestB= new int;

	  infile >> *this->FastestB;
#ifdef __OUTLIST__
	  cout << *this->FastestB << " ";      
#endif
	  *this->FastestB= **item->Bindings[*this->FastestB];

	  infile >> this->FastestTimePercentage;

	  SetSpeed();

#ifdef __OUTLIST__
	  cout << Speed << endl;      
#endif
	}
    }
  else
    {
      Text DialogContense= "";
      DialogContense+= buffer;
      
      char c; 
      if (!infile.eof())
        c= (char)infile.get();

      while (!infile.eof())
	{// solange noch Meldungen da sind      
	  DialogContense+= c;
	  c= (char)infile.get();
	}
      CreateDialog();
      OpenMsgDialog(*this->ed,"HCPNTokengame-Error", (charP)DialogContense, true, 
		    "Cancel");

      infile.close();
      
      return (false);
      // Das Einlesen schlug fehl
    }
  return true;
  // Das Einlesen hat geklappt
}

// -----------------------------------------------------------------

bool HCPNTokengame::SelectEnabledTrans(PageP)
{// Wird zyklisch aufgerufen

  char buffer[MAX_LENGTH];

  Text netname= Text(FileName()) - NETFILENAME;  
  Text infile= netname + LISTFILENAME;
  Text outfile= netname + OUTFILENAME;
  if (((charP) infile)[0]!='/') infile=  model_dir + "/" + infile;  
  if (((charP) outfile)[0]!='/') outfile=  model_dir + "/" + outfile;  
  // richtiger Pfad!
  
  ifstream OutFile((charP)(outfile));
  // Eingabestrom mit der Ausgabedatei
  
  if (OutFile)
    {// Datei erfolgreich geoeffnet?
      
      OutFile.seekg(0, ios::end);
      int FileEnd= OutFile.tellg();
      // FilePointer am Ende der Datei ermitteln
      
      if (FileEnd)
	{// kamen Meldungen hinzu?
	  OutFile.seekg(0, ios::beg);
	  OutFile >> buffer;
	  if (strcmp(buffer, "End_Tokengame"))
	    {
	      Text DialogContense= "";
	      DialogContense+= buffer;
	      
	      char c; 
	      if (!OutFile.eof())
		c= (char)OutFile.get();
	      
	      while (!OutFile.eof())
		{// solange noch Meldungen da sind      
		  DialogContense+= c;
		  c= (char)OutFile.get();
		}
	      CreateDialog();
	      OpenMsgDialog(*this->ed,"HCPNTokengame-Error", 
			    (charP)DialogContense, true, "Cancel");
	      
	      OutFile.close();
	      
	      DeInitTokengame(true);
	      
	      XmToggleButtonSetState(this->ToggleWidget,false,true);
	      // ToggleButton deselektieren
	      
	      return false;
	    }
	}
      OutFile.close();
    }
  
  ifstream ListFile((charP)(infile));
  // Eingabestrom mit der Liste der
  // schaltfaehigen bindings
  
  if (ListFile)
    {// Datei erfolgreich geoeffnet?
      
      ListFile.seekg(0, ios::end);
      int FileEnd= ListFile.tellg();
      // FilePointer am Ende der Datei ermitteln
      
      ListFile.close();
      
      if (FileEnd)
	{// kamen neue Transitionen hinzu?
	  
	  DeselectAllTrans();
	  ReadyToFire.Clear();
	  
	  ifstream ListFile((charP)(infile));
	  // Eingabestrom mit der Liste der
	  // schaltfaehigen bindings
	  
	  if (!ReadInput(ListFile))
	    {
	      DeInitTokengame(true);
	      
	      XmToggleButtonSetState(this->ToggleWidget,false,true);
	      // ToggleButton deselektieren
	      
	      return false;
	    }
	  // ListFile nach Listen durchsuchen
	  
	  Received= true;
	  
	  SetModified();
	  // Das Netz wird modifiziert
	  
	  ListFile.close();
	  
	  ListFile.open((charP)infile, ios::trunc); ListFile.close();
	  // infile loeschen
	  
	  this->ed->Update();
	  // Editor-Screen update
	}
    }
  
  if (ReadyToFire.Length())
    {
      for (int i= 0; i<ReadyToFire.Length(); ++i)
	(*ReadyToFire[i])->node->Select(true);
      // Alle Transitionen und SubTransitionen,
      // die feuern koennen, selektieren
	
      
      if (Received && BackwardEnable)
	{
	  if (!--StepsCnt || !FiredCount)
	    {
	      BackwardEnable= false;
	      StepsCnt= Steps;
	    }
	  
	  if (FiredCount)
	    {
	      Received= false;
	      
	      FiredItemP item= *Fired[FiredCount-1];
	      
	      SelectTrans(item->node);
	      
	      if (FiredCount==FiredCount_save)
		{
		  Text DialogContense;
		  DialogContense+= (charP)Text("Warning: this command ");
		  DialogContense+= (charP)Text("may lead to an inconsistent  net-status, ");
		  DialogContense+= (charP)Text("if you changed the net!");
		  
		  OpenMsgDialog(*this->ed, "HCPNTokengame_Warning",
				(charP)DialogContense,false,"Cancel");
		}
	      
	      item->Do(BACKWARD, this);  
	    }
	}
      else if (Received && ForwardEnable)
	{
	  if (!--StepsCnt || FiredCount==Fired.Length())
	    {
	      ForwardEnable= false;
	      StepsCnt= Steps;
	    }
	  
	  if (FiredCount<Fired.Length())
	    {
	      Received= false;
	      
	      FiredItemP item= *Fired[FiredCount];
	      
	      SelectTrans(item->node);
	      
	      item->Do(FORWARD, this);
	    }
	}
      else if (Received && AniEnable)
	{
	  SelectTrans(this->Fastest);
	  
	  if (--SpeedCnt == -1)
	    {
	      Received= false;
	      
	      SpeedCnt= Speed;
	      
	      FireItemP item;
	      Iterator<FireItemP> FireList= ReadyToFire;
	      for (item= FireList; (item && *item!=this->Fastest); 
		   item=++FireList);
	      // Such das FireItem, mit der schnellsten Transition
	      
	      if (item)
		item->Fire(FastestB, this);
	      // feuern
	    }
	}
    }
  return true;
}

// -----------------------------------------------------------------

bool HCPNTokengame::DeselectEnabledTrans(PageP)
{// Wird zyklisch aufgerufen

  if (AniEnable || BackwardEnable || ForwardEnable)
    {
      FireItemP item;
      Iterator<FireItemP> FireList= ReadyToFire;
      for (item= FireList; (item && *item!=this->Fastest); 
	   item=++FireList);
      // Such das FireItem, mit der schnellsten Transition
      
      if (item)
	DeselectTrans(item->node);
      // Deselektieren
    }
  else 
    {
      Iterator<FireItemP> FireList= ReadyToFire;
      for (FireItemP item= FireList; item; item=++FireList)
	if (!item->node->SubPage())
	  item->node->Select(false);
      // alle schaltfaehigen Elementartransitionen deselektieren 
    }
  return true;
}

// -----------------------------------------------------------------

void HCPNTokengame::OnDialogButton(Ident& button)
{// Wird bei Mausklick auf die Dialog-Buttons aufgerufen

  StructP params= GetParams();

  if ((button=="Restore"))
    {// END angeklickt?

      DeInitTokengame(true);
      
      CloseDialog();
      DestroyDialog();
    }
  if ((button=="Quit"))
    {// END angeklickt?

      if (!EventEnable())
	{
	  Text DialogContense;
	  DialogContense+= (charP)Text("you must first stop the animation");
	  
	  CloseDialog();
	  DestroyDialog();

	  OpenMsgDialog(*this->ed, "HCPNTokengame_Warning",
			(charP)DialogContense,true,"Cancel");

	  TokenGameEnable= true;
	  XmToggleButtonSetState(this->ToggleWidget,true,true);
	  // ToggleButton selektieren
	}
      else
	{
	  DeInitTokengame(false);
	  
	  CloseDialog();
	  DestroyDialog();
	}
    }
  else if ((button=="Forward") && !AniEnable && !BackwardEnable && !ForwardEnable)
    {// Forward angeklickt?

      Steps= Steps2Int(params);
      StepsCnt= Steps;
      RealTime= ((bool)*params->GetMember("realtime"));
      LowestSpeed= ((int)*params->GetMember("lowest"));
      SetSpeed();

      ForwardEnable= true;
    }
  else if ((button=="Backward") && !AniEnable && !BackwardEnable && !ForwardEnable)
    {// Backward angeklickt?

      Steps= Steps2Int(params);
      StepsCnt= Steps;
      RealTime= ((bool)*params->GetMember("realtime"));
      LowestSpeed= ((int)*params->GetMember("lowest"));
      SetSpeed();

      BackwardEnable= true;
    }
  else if ((button=="Animate") && !AniEnable && !BackwardEnable && !ForwardEnable)
    {// Animate  angeklickt?

      Steps= Steps2Int(params);
      StepsCnt= Steps;
      RealTime= ((bool)*params->GetMember("realtime"));
      LowestSpeed= ((int)*params->GetMember("lowest"));
      SetSpeed();

      AniEnable= true;
    }
  else if ((button=="Stop"))
    {// Stop  angeklickt?

      Steps= Steps2Int(params);
      StepsCnt= Steps;
      RealTime= ((bool)*params->GetMember("realtime"));
      LowestSpeed= ((int)*params->GetMember("lowest"));
      SetSpeed();

      AniEnable=  false;
      BackwardEnable= false;
      ForwardEnable= false;
    }
  else if (button=="Close")
    {// Close angeklickt?

      Steps= Steps2Int(params);
      StepsCnt= Steps;
      RealTime= ((bool)*params->GetMember("realtime"));
      LowestSpeed= ((int)*params->GetMember("lowest"));
      SetSpeed();
	      
      CloseDialog();
      DestroyDialog();
    }
  else if (button=="Cancel")
    {// Cancel  angeklickt?

      if (params->GetMember("end"))
	{// Cancel bei Ende des Tokengame?

	  TokenGameEnable= true;
	  XmToggleButtonSetState(this->ToggleWidget,true,true);
	  // ToggleButton selektieren
	}
      
      CloseDialog();
      DestroyDialog();
    }
}

void HCPNTokengame::DialogCallback(Dialog<HCPNTokengame,bool>& dia,bool,Ident& btn)
{
  if(Received && TokenGameEnable && btn=="Ok")
    {// OK angeklickt waehrend das HCPNTokengame laeuft?
      Received= false;

      setup->EvalWidget();

      FireItemP item;
      Iterator<FireItemP> FireList= ReadyToFire;
      for (item= FireList; (item && *item!=node); item=++FireList);
      // Such das FireItem, mit der Trans. die gefeuert werden soll!
      
      int* b;
      Iterator<int*> bindings=item->Bindings;
      for (b=bindings; b; b=++bindings) 
	{// Alle feuerbaren Binding-Indices traversieren
	  
	  Text member;
	  member+= "binding";
	  member+= double(*b);
	  if (member==((charP)*setup->GetMember("binding"))) break;
	  // welches binding wurde ausgewaehlt?
	}
      
      if (b)
	item->Fire(b, this);

      dia.Close();
    }	
  else if (btn=="Cancel")
    dia.Close();

  delete(setup);
}

// -----------------------------------------------------------------
