#ifndef __COMMMOD_H__
#define __COMMMOD_H__

#include "module.h"
#include "dialog.h"
#include "logwin.h"

class Net;
class Page;
class Editor;
class Object;
class Struct;
class CommDialog;
class CommLogwin;
class CommFilesel;

//-----------------------------------------------------------------------------
// Das CommModul bildet eine Basisklasse fr alle Module, die Dialogfenster,
// Fileselektoren oder Ausgabefenster benutzen wollen. Es stellt auch ein
// CommandLogwindow zu Verfgung, bei dem neben der Ausgabe auch spezielle
// Kommandos eingelesen und ausgefhrt werden (Siehe dazu LogwinCommand).
//
// Das CommModul bernimmt dabei die Verwaltung der dazu ntigen
// Datenstrukturen und enthlt einfache Methoden zum Erzeugen, ffnen,
// Schlieen und Zerstren von Fenstern (Reihenfolge beachten!).
// Rckmeldung bei Ereignissen erhlt man durch berschreiben der
// entsprechenden virtuellen Methoden (OnDialogButton, OnLogwinButton).
// Mehrere Fenster werden in einer Liste verwaltet, auf deren Index man ber
// (Set)CurrentDialog bzw. -Logwin zugreifen kann.
//

class CommModule: public Module
{
  LinkedList<CommDialog*> dialogs;
  LinkedList<CommLogwin*> logwins;
  CommFilesel* filesel;

  CommDialog* current_dialog;
  CommLogwin* current_logwin;
  Editor* last_editor;

  friend class CommDialog;
  friend class CommLogwin;
 
  void OwnDialogCB(Dialog<CommModule, Editor*> &, Editor* ed, Ident& button);
  void OwnLogwinCB(Dialog<CommModule, bool>& lw, bool, Ident& button);
  bool OwnOnLogwinChar(char c);
	void OwnFileselCB(Editor*, char* file, bool ok);

public:
  // Command Methods
  Object* CommandGetObject(istrstream& cmd);
  
  void CommandSetAttribute(istrstream& cmd);
  void CommandSetNetAttribute(istrstream& cmd);
  void CommandShowPage(istrstream& cmd);
  void CommandShowObject(istrstream& cmd);
  void CommandErrorMessage(istrstream& cmd);
  void CommandDeleteLastLine();

public:
  CommModule(Net& net, char* name);
  virtual ~CommModule();

  // Dialog Methods
  void SetCurrentDialog(int i);
  int CurrentDialog();
  
  bool DialogVisible();
  Text Param(char* name);
  Struct* GetParams();

  void CreateDialog(char* params="");
  void OpenDialog(Editor& ed, char* title, char* buttons = "Ok");
  void OpenMsgDialog(Editor& ed, char* title, char* msg, bool errormsg = true, 
                     char* buttons = "Ok");
  void CloseDialog();
  void DestroyDialog();

  virtual void OnDialogButton(Ident& button);

  // Logwindow Methods
  void SetCurrentLogwin(int i);
  int CurrentLogwin();
  
  void CreateLogwin(char* filename);
  void CreateLogwin2(int filedes = 0);
  void SetCommandLog(bool on);

  void OpenLogwin(Editor& ed, char* title, char* buttons = "Ok");
  void CloseLogwin();
  void DestroyLogwin();

  virtual void OnLogwinButton(Ident& button);
  virtual bool OnLogwinChar(char) { return false; }
  virtual void LogwinCommand(Text& command);

  // Fileselector Methods
  void CreateFilesel(char* dir="");
  void OpenFilesel(Editor& ed, char* title, char* mask = 0);
  void CloseFilesel();
  void DestroyFilesel();

  virtual void OnFilesel(char* file, bool ok);

  // Misc
  int Exec(char* command, char* outfile=0, char* infile=0);
  Editor* LastEditor() { return last_editor; }
  void SetLastEditor(Editor* ed) { last_editor = ed; }
  
  virtual bool Open(Editor& ed, Page&) { last_editor = &ed; return true; }
  virtual bool Change(Editor& ed, Page&, Page&) { last_editor=&ed;return true;}
  virtual void Close(Editor& ed, Page&) { if (&ed==last_editor) last_editor=0;}
  virtual bool ModuleLock();
};


#endif
