#ifndef __EVALMOD_H__
#define __EVALMOD_H__

#include "commmod.h"
#include "struct.h"

//-----------------------------------------------------------------------------
// Das EvalModule bildet eine Basisklasse zur Anbindung von externen
// Evaluierungskomponenten an Agnes. Auerdem ist eine Routine zur Durchfhrung
// von einfachen Experimenten enthalten.
//
// Um dieses Modul zu benutzen, mssen folgende Methoden implementiert werden:
// 
//   EvalSettings     - Strukturdefinition aller Auswertungs-Parameter
//   AllowExperiments - Experimente mglich?
//   SetValue         - setzt die Experimentvariable auf den angegebenen Wert
//   EvalCall         - Erzeugt Files und Parameter zum Aufruf der externen
//                      Evaluierungskomponente. Dabei sind die Text-Parameter 
//                      Rckgabeparameter, und bedeuten folgendes:
//     command        - Auszufhrende Befehlszeile
//     stdout_file    - File, aus dem die StdEingabe fr ob. Befehl kommen soll
//     stdin_file     - File, in den die StdAusgabe umgelenkt werden soll
//     net_file       - File, in dem die Netzinformationen gespeichert wurden
//     result_file    - File, in dem die ermittelten Ergebnisse gesp. werden
//     logwin_title   - Titel des Ausgabefensters
//     logwin_buttons - Buttons des Ausgabefensters
//
// Die Ergebnisse der Auswertung werden ber das CommandLog eingelesen. Falls
// das nicht fr alle Auswertungskomponenten mglich ist, mte man dieses
// Modul ggf. erweitern.
//
class EvalModule: public CommModule
{
  bool save_options;
  int pid;
  
public:
  EvalModule(Net& net, char* name): CommModule(net, name), pid(0) {}
  virtual ~EvalModule() {}
  int LastPid() { return pid; }

  // Own functions
  virtual void Call(Editor& editor, Page&);  
  virtual void OnDialogButton(Ident& button);
  virtual void OnLogwinButton(Ident& button);
  virtual void OnFilesel(char* file, bool ok);

  virtual Text ExperimentSettings();
  virtual bool StartExperiment();
  virtual void StartEvaluation();

  // User defined functions
  virtual Text EvalSettings();
  virtual bool AllowExperiments() { return false; }
  virtual bool SetValue(Text& /*name*/, double /*val*/) { return false; }
  virtual bool EvalCall(bool experiment, Text& command, Text& stdout_file,
    Text& stdin_file, Text& net_file, Text& result_file, Text& logwin_title,
    Text& logwin_buttons) = 0;
};


#endif

