// -----------------------------------------------------------------
// Importe
// -----------------------------------------------------------------

#include "parser.h"

#include <ctype.h>
// fuer isdigit()

#include <math.h>
// fuer pow()

//------------------------------------------------------------------------
//Hilfsfunktionen
//------------------------------------------------------------------------

void Parser::DestroyBlancs(charP string)
{
  int length= (int)strlen(string);
  for (int i=0; i<length; i++)
    if (string[i]==' ')
      {
	memmove(string+i, string+i+1, length-i);
	length--;
      }
}

//------------------------------------------------------------------------

signed int Parser::SearchOpeningBracket(charP string, int endpos)
{
  signed int i;
  signed int bracecnt= 0;
  for (i= endpos; i>=0; i--)
    {
      if      (string[i] == ')') bracecnt --;
      else if (string[i] == '(') bracecnt ++;
      if      (!bracecnt) break;
    }
  return i;
  // search for opening bracket:
  // ...(..(..(..(...)..)..)..)
  //    ^
}

//------------------------------------------------------------------------

signed int Parser::SearchLOperator(charP string, int endpos)
{
//  *outs << "SearchLOperator:";
//  Out (string,endpos+1); 
  bool not_found= false;
  bool andor_found= false;
  signed int op_index= -1;
  for (signed int i= endpos; i>=0; i--)
    {
      if ((!strncmp(string+i, "AND", 3)) ||
	  (!strncmp(string+i, "OR",  2)))
	{// (AND/OR) occured?

	  andor_found= true;
	  op_index= i;
	  // save position of last operand
	  break;
	  // interrupt loop
	}

      else if (!strncmp(string+i, "NOT", 3))
	{
	  not_found= true;
	  op_index= i;
	  // save position of last operand
	}
      if (not_found && op_index==i+3)
	break;
    }
  return op_index;
  // search for operator before logic condition
  // ...AND/OR.NOT..NOT...
  //    ^
}

//------------------------------------------------------------------------

signed int Parser::SearchOperator(charP string, int endpos)
{
//  *outs << "SearchOperator:";
//  Out (string,endpos+1);
  signed int op_index= -1;
  bool op_found= false;
  for (signed int i= endpos; i>=0; i--)
    {
      if ((string[i] == '*') ||
	  (string[i] == '/') ||
	  (string[i] == '^'))
	{
	  op_found= true;
	  op_index= i;
	  // save position of last operand
	}
      else if (((string[i] == '+') ||
		(string[i] == '-')) && !IsSignOfExponent(string,i))
	{// is no sign of exponent?
	  op_found= true;
	  op_index= i;
	  // save position of last operand
	}
      else if (op_found)
	break;
    }
  return op_index;
  // search for operator before expression
  // ...-...-3e-12
  //    ^
}

//------------------------------------------------------------------------

signed int Parser::SearchLowestLOperator(charP string, int endpos, bool& error)
{
  bool and_found= false;
  bool or_found= false;
  bool not_found= false;
  signed int index, and_index, or_index, not_index;
  error= false;
  
  do
    {
      if (string[endpos] == ')')
	{// "...)" <-- logic condition is in brackets?
	  
	  index= SearchOpeningBracket(string,endpos);
	  // "...(...)" <-- expression in brackets
	  //     ^
	  if (index==-1)
	    {
	      error= true;
	      *outs << " '(' is missing in logic condition: ";
	      Out(string, endpos+1);
	      return -1;
	      // error, no opening bracket found
	    }
	}
      else
	index= endpos;
      
      index= SearchLOperator(string, index);
      
      if (index!=-1)
	if ((string[index]=='N') && !not_found)
	  {
	    not_found= true;
	    not_index= index;
	  }
	else if ((string[index]=='O') && !or_found)
	  {
	    or_found= true;
	    or_index= index;
	  }
	else if ((string[index]=='A') && !and_found)
	  {
	    and_found= true;
	    and_index= index;
	  }

    } while ((endpos= index-1)>0);
  // search the rest of the whole string

  if (or_found)
    return or_index;
  // lowest priority:
  // OR found

  else if (and_found)
    return and_index;
  // AND found

  else if (not_found && !not_index)
    return not_index;
  // highest priority:
  // NOT found

  else return -1;
  // nothing found

  return -1;
  // to keep the compiler happy!!!
}

//------------------------------------------------------------------------

signed int Parser::SearchLowestOperator(charP string, int endpos, bool& error)
{
  bool pow_found= false;
  bool muldiv_found= false;
  bool plusmin_found= false;
  signed int index, pow_index, muldiv_index, plusmin_index;
  error= false;
  
  do
    {
      if (string[endpos] == ')')
	{// "...)" <-- expression is in brackets?
	  
	  index= SearchOpeningBracket(string,endpos);
	  // "...(...)" <-- expression in brackets
	  //     ^
	  if (index==-1)
	    {
	      error= true;
	      *outs << " '(' is missing in expression: ";
	      Out(string, endpos+1);
	      return -1;
	      // error, no opening bracket found
	    }
	}
      else
	index= endpos;
      
      index= SearchOperator(string, index);
      
      if (index!=-1)
	if (((string[index]=='+') || (string[index]=='-')) && !plusmin_found)
	  {
	    plusmin_found= true;
	    plusmin_index= index;
	  }
	else if (((string[index]=='*') || (string[index]=='/')) && !muldiv_found)
	  {
	    muldiv_found= true;
	    muldiv_index= index;
	  }
	else if (string[index]=='^' && !pow_found)
	  {
	    pow_found= true;
	    pow_index= index;
	  }

    } while ((endpos= index-1)>0);
  // search the rest of the whole string

  if (plusmin_found && plusmin_index)
    return plusmin_index;
  // lowest priority:
  // +- found, as an operator

  else if (muldiv_found)
    return muldiv_index;
  // */ found

  else if (pow_found)
    return pow_index;
  // ^ found

  else if (plusmin_found && !plusmin_index)
    return plusmin_index;
  // highest priority:
  // +- found, as a sign

  else return -1;
  // nothing found

  return -1;
  // to keep the compiler happy!!!
}

//------------------------------------------------------------------------

bool Parser::IsSignOfExponent(charP string, int endpos)
{
  if (!endpos)
    return false;

  if ((string[endpos-1]!='E') && (string[endpos-1]!='e'))
    return false;
  // no exponent sign found?

  signed int i;
  for (i= endpos-1; i>=0; i--)
    if ((string[i] == '+') ||
	(string[i] == '-') ||
	(string[i] == '*') ||
	(string[i] == '/') ||
	(string[i] == '^') ||
	(string[i] == '('))
      break;
  // search for beginning of expression

  if (isalpha(string[i+1]))
    return false;
  // exponent sign is part of variable-name

  return true;
}

//------------------------------------------------------------------------
// Parser fuer IfExpressions
//------------------------------------------------------------------------

double Parser::ParseIfExpression(charP string)
{
  int endpos;
  int length;

  if (!string || !*string)
    return 0;
  else
    {
      length= (signed int)strlen(string);
      endpos= length-1;
    }
 
  if (string[endpos]==';')
    {// cuts ';' after if_expr

      length--;
      endpos--;
    }
  
  charP h_string= string;
  do
    {
      if (!strncmp(h_string,"IF", 2))
	{// IF found?
	  
	  charP pointer;
	  if ((pointer= strchr(h_string,':')))
	    {
	      signed int k= pointer-h_string;
	      // IF...:...
	      //      ^

	      bool is_true= ParseLogicCondition(h_string+2, k-2);
		
	      if ( (pointer= strstr(h_string+k,"IF")) ||
		  (pointer= strstr(h_string+k,"ELSE")) )
		{
		  if (is_true)
		    return (ParseExpression(h_string+k+1, pointer-(h_string+k+1)));
		  h_string= pointer;
		}
	      else
		{
		  *outs << "keyword must be 'ELSE' after if_expr: ";
		  Out(h_string, strlen(h_string)-1);
		  return 0;
		}
	    }
	  else
	    {
	      *outs << "':' is missing in if_expr: ";
	      Out(h_string, strlen(h_string)-1);
	      return 0;
	    }
	}
      else if (!strncmp(h_string, "ELSE", 4))
	{// ELSE found?
	  return ParseExpression(h_string+4,strlen(h_string+4)-1);
	}
      else if (h_string==string)
	{
	  return ParseExpression(h_string, length);
	  // only one expression
	}
      else
	{
	  *outs << "keyword must be 'ELSE' after if_expr: ";
	  Out(h_string, strlen(h_string)-1);
	  return 0;
	}
    } while (h_string<string+length);

  return 0.0;  // to keep the compiler happy!!!
}

//------------------------------------------------------------------------
// Parser fuer Expressions
//------------------------------------------------------------------------

double Parser::ParseExpression(charP string, unsigned int length)
{
//  *outs << "ParseExpression:";
//  Out(string,length);

  int   endpos;

  if (!string || !*string || (length==0) || length>strlen(string))
    return 0.0;
  else
    endpos= length-1;
  // NULL or "" <-- no expression or wrong value

  bool error;
  signed int index= SearchLowestOperator(string, endpos, error);
  if (error)
    return 0.0;

  if (index==-1)
    {// no operator found
      return (((string[0]=='(') && (string[endpos]==')'))? 
	      ParseExpression(string+1, length-2): 
	      ParseRealValue(string, length));
    }
  // "(...)" <-- only one expression in brackets
  // "..." <-- or only one expression
  
  char op= string[index];

  if (!index)
    {// "sign_1...sign_n(...)" <-- only signs befor the expression
      
      switch (op)
	{
	case '-':
	  {
	    return -1*ParseExpression(string+1, length-1);
	    // "-.sign_1...sign_n(...)" <-- minus expression
	    break;
	  }
	default:
	  {
	    *outs << "illegal sign in expression: ";
	    Out(string, length);
	    return 0.0;
	    // error, illegal sign
	  }
	}
    }
  
  // else:
  // "...op..." <-- expression operator expression
  
  if (string+index == string+endpos)
    {
      *outs << "missing operand2 in expression: ";
      Out(string, length);
      return 0.0;
      // error, operand2 missing
    }

  switch (op)
    {
    case '+':
      return (ParseExpression(string,index) + 
	      ParseExpression(string+index+1,endpos-index));
      break;
    case '-':
      return (ParseExpression(string,index) - 
	      ParseExpression(string+index+1,endpos-index));
      break;
    case '*':
      return (ParseExpression(string,index) * 
	      ParseExpression(string+index+1,endpos-index));
      break;
    case '/':
      return (ParseExpression(string,index) / 
	      ParseExpression(string+index+1,endpos-index));
      break;
    case '^':
      return (pow(ParseExpression(string,index), 
		  ParseExpression(string+index+1,endpos-index)));
      break;
    default:
      *outs << "illegal operator in expression: ";
      Out(string, length);
      return 0.0;
      // error, illegal operator
    }            

  return 0.0;  // to keep the compiler happy!!!
}

// -----------------------------------------------------------------
// Parser fuer RealValues
// -----------------------------------------------------------------

double Parser::ParseRealValue(charP string, unsigned int length)
{
//  *outs << "ParseRealValue:";
//  Out(string,length);
  int endpos;
  
  if (!string || !*string || (length==0) || length>strlen(string))
    return 0.0;
  else
    endpos= length-1;
  // NULL or "" <-- no expression or wrong value
  
  if (isalpha(string[0]))
    return ParseParameter(string, length);
  // RealParameter or IntegerParameter

  bool real_const= false;
  for (signed int i= endpos; i>=0; i--)
    {
      if (string[i]=='.')
	real_const= true;
      else if ((string[i]=='E') ||
		(string[i]=='e'))
	real_const= true;
    }
  if (real_const)
    return ParseRealConstant(string, length);
  // RealConstant
  else
    return (double)ParseIntegerConstant(string, length);
  // IntegerConstant
}

//------------------------------------------------------------------------
// Parser fuer IntegerValues
// -----------------------------------------------------------------

unsigned long Parser::ParseIntegerValue(charP string, unsigned int length)
{
//  *outs << "ParseIntegerValue:";
//  Out(string,length);
  int endpos;
  
  if (!string || !*string || (length==0) || length>strlen(string))
    return 0;
  else
    endpos= length-1;
  // NULL or "" <-- no expression or wrong value
  
  if (isalpha(string[0]))
    return ParseIntegerParameter(string, length);
  // IntegerParameter

  else if (string[0]=='#')
    return ParseMarking(string,length);
  // Marking

  else
    return ParseIntegerConstant(string, length);
  // IntegerConstant
}

//------------------------------------------------------------------
// Parser fuer RealConstants
// -----------------------------------------------------------------

double Parser::ParseRealConstant(charP string, unsigned int length)
{
//  *outs << "ParseRealConstant:";
//  Out(string,length);
  double value= 0.0;

  if (!string || !*string || (length==0) || length>strlen(string))
    return 0.0;
  // NULL or "" <-- no expression or wrong value

  bool error= false;
  bool point_found= false;
  bool e_found= false;
  for (signed int i=0; i<(signed int)length; i++)
    {
     if (string[i]=='.')
       if (point_found || e_found)
	 {
	   error= true;
	   break;
	 }// only one point allowed, in front of exponent sign
	else 
	  {
	    point_found= true;
	    continue;
	  }
     else if ((string[i]=='E') || (string[i]=='e'))
       if (e_found || (i==0) || (i+1==(signed int)length))
	 {
	   error= true;
	   break;
	 }// only one exponent sign allowed, not at the beginning or the end
       else 
	 {
	   e_found= true;
	   continue;
	 }
     else if ((string[i]=='+')  || (string[i]=='-'))
       if ((i==0) && (string[i]=='+'))
	 {
	   error= true;
	   break;
	 }// negative sign only in front of mantisse allowed
     
       else if ( (i-1 >=0) &&
		(string[i-1]!='E') && (string[i-1]!='e') )
	 {
	   error= true;
	   break;
	 }// or sign only after exponent sign allowed
       else
	 continue;
     
     else if (!isdigit(string[i]))
       {
	 error= true;
	 break;
       }// the rest must be digits
   }
  if (error)
    {
      *outs << "illegal real constant: ";
      Out(string, length);
      return 0.0;
    }
  
  sscanf(string, "%lf", &value);	      

  return value;
}

// -----------------------------------------------------------------
// Parser fuer IntegerConstants
// -----------------------------------------------------------------

unsigned long Parser::ParseIntegerConstant(charP string, unsigned int length)
{
//  *outs << "ParseIntegerConstant:";
//  Out(string,length);
  int endpos= length-1;
  unsigned long value= 0;

  if (!string || !*string || (length==0) || length>strlen(string))
    return 0;
  // NULL or "" <-- no integer constant or wrong value

  for (signed int i= endpos; i>=0; i--)
    if (!isdigit(string[i]))
      {
	*outs << "illegal integer constant: ";
	Out(string, length);
	return 0;
      }
  sscanf(string, "%ld", &value);	      

  return value;
}

//------------------------------------------------------------------------
// Parser fuer LogicConditions
//------------------------------------------------------------------------

bool Parser::ParseLogicCondition(charP string, unsigned int length)
{
//  *outs << "ParseLogicCondition:";
//  Out(string,length);
//cout << "ParseLogicCondition:";
//for (int i=0;i<length;i++)
//  cout << string[i];
//cout << endl;

  int endpos;

  if (!string || !*string || (length==0) || length>strlen(string))
    return true;
  else
    endpos= length-1;
  // NULL or "" <-- no logic condition or wrong value

  bool error;
  signed int index= SearchLowestLOperator(string, endpos, error);
  if (error)
    return false;

  if (index==-1)
    {// no operator found
      return (((string[0]=='(') && (string[endpos]==')'))? 
	      ParseLogicCondition(string+1, length-2): 
	      ParseComparison(string, length));
    }
  // "(...)" <-- only one logic condition in brackets
  // "..." <-- or only comparison
  

  char op= string[index];

  if (!index)
    {// "sign_1...sign_n..." <-- only signs befor the expression
      
      switch (op)
	{
	case 'N':
	  {
	    return !ParseLogicCondition(string+3, length-3);
	    // "NOT.sign_1...sign_n(...)" <-- NOT logic condition
	    break;
	  }
	default:
	  {
	    *outs << "illegal sign in logic condition: ";
	    Out(string, length);
	    return false;
	    // error, illegal sign
	  }
	}
    }
  
  // else:
  // "...op..." <-- logic condition operator logic condition
  
  signed int i=3;
  if (op=='O')
    i= 2;
  // i is length of operator(NOT, AND, OR)

  if (string+index+i > string+endpos)
    {
      *outs << "missing condition2 in logic condition: ";
      Out(string, length);
      return false;
      // error, condition2 missing
    }
  
  switch (op)
    {
    case 'A':
      return (ParseLogicCondition(string,index) & 
	      ParseLogicCondition(string+index+i,endpos-(index+i)+1));
      break;
    case 'O':
      return (ParseLogicCondition(string,index) | 
	      ParseLogicCondition(string+index+i,endpos-(index+i)+1));
      break;
    default:
      *outs << "illegal operator in logic condition: ";
      Out(string, length);
      return false;
      // error, illegal operator
    }            

  return false;  // to keep the compiler happy!!!
}

//------------------------------------------------------------------------
// Parser fuer Comparisons
//------------------------------------------------------------------------

bool Parser::ParseComparison(charP string, unsigned int length)
{
  if (!string || !*string || (length==0) || (length>strlen(string)))
    return true;

  int endpos= length-1;
  signed int i;

  for (i=0; i<(signed int)length; i++)
    {
      if (string[i]=='=')
	return (ParseMarkFunc(string,i) == ParseMarkFunc(string+i+1,endpos-i));
      // MarkFunc1 = MarkFunc2
      
      else if ((string[i]=='/') && ((i+1 <= endpos) && string[i+1]=='='))
	return (ParseMarkFunc(string,i) != ParseMarkFunc(string+i+2,endpos-i-1));
      // MarkFunc1 != MarkFunc2
      
      else if (string[i]=='>')
	if ((i+1 <= endpos) && string[i+1]=='=')
	  return (ParseMarkFunc(string,i) >= ParseMarkFunc(string+i+2,endpos-i-1));
	else
	  return (ParseMarkFunc(string,i) > ParseMarkFunc(string+i+1,endpos-i));
      // MarkFunc1 >= MarkFunc2 or MarkFunc1 > MarkFunc2
      
      else if (string[i]=='<')
	if ((i+1 <= endpos) && string[i+1]=='=')
	  return (ParseMarkFunc(string,i) <= ParseMarkFunc(string+i+2,endpos-i-1));
        else
	  return (ParseMarkFunc(string,i) < ParseMarkFunc(string+i+1,endpos-i));
      // MarkFunc1 <= MarkFunc2 or MarkFunc1 < MarkFunc2
    }
  
  *outs << "illegal comp_oper in comparison: ";
  Out(string, length);
  return false;
}

//------------------------------------------------------------------------
// Parser fuer marc_funcs
//------------------------------------------------------------------------

unsigned long Parser::ParseMarkFunc(charP string, unsigned int length)
{
//  *outs << "ParseMarkFunc:";
//  Out(string,length);

  int   endpos;

  if (!string || !*string || (length==0) || length>strlen(string))
    return 0;
  else
    endpos= length-1;
  // NULL or "" <-- no MarkFunc or wrong value

  bool error;
  signed int index= SearchLowestOperator(string, endpos, error);
  if (error)
    return 0;

  if (index==-1)
    {// no operator found
      return (((string[0]=='(') && (string[endpos]==')'))? 
	      ParseMarkFunc(string+1, length-2): 
	      ParseIntegerValue(string, length));
    }
  // "(...)" <-- only one MarkFunc in brackets
  // "..." <-- or only one MarkFunc
  
  char op= string[index];

  if (!index)
    {// "sign_1...sign_n(...)" <-- only signs befor the MarkFunc
      
      *outs << "illegal sign in MarkFunc: ";
      Out(string, length);
      return 0;
      // error, illegal sign
    }
  
  // else:
  // "...op..." <-- MarkFunc operator MarkFunc
  
  if (string+index == string+endpos)
    {
      *outs << "missing operand2 in MarkFunc: ";
      Out(string, length);
      return 0;
      // error, operand2 missing
    }
  switch (op)
    {
    case '+':
      return (ParseMarkFunc(string,index) + 
	      ParseMarkFunc(string+index+1,endpos-index));
      break;
    case '-':
      return (ParseMarkFunc(string,index) - 
	      ParseMarkFunc(string+index+1,endpos-index));
      break;
    case '*':
      return (ParseMarkFunc(string,index) * 
	      ParseMarkFunc(string+index+1,endpos-index));
      break;
    case '/':
      return (ParseMarkFunc(string,index) / 
	      ParseMarkFunc(string+index+1,endpos-index));
      break;
    case '^':
      return ((unsigned long)pow(ParseMarkFunc(string,index), 
				 ParseMarkFunc(string+index+1,endpos-index)));
      break;
    default:
      *outs << "illegal operator in MarkFunc: ";
      Out(string, length);
      return 0;
      // error, illegal operator
    }            

  return 0;  // to keep the compiler happy!!!
}

