// -----------------------------------------------------------------
// Importe
// -----------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <strstream.h>
#include <string.h>

// -----------------------------------------------------------------
// Definitionen
// -----------------------------------------------------------------

#define charP char*
// Pointerabkuerzung

enum parser_type {none, md_enable, md_arc_mult};

// -----------------------------------------------------------------
// marking-dependent expression-parser Klassendeklaration
// -----------------------------------------------------------------

class Parser
{
 private:

  void DestroyBlancs(charP string);
  int  SearchOpeningBracket(charP string, int endpos);
  int  SearchLOperator(charP string, int endpos);
  int  SearchOperator(charP string, int endpos);
  int  SearchLowestLOperator(charP string, int endpos, bool& error);
  int  SearchLowestOperator(charP string, int endpos, bool& error);
  bool IsSignOfExponent(charP string, int endpos);
  
  public:
  
  ostrstream* outs;
  // Umleitung von Parser errors
  
  void Out(charP string, unsigned int length)
    {
      for (int i=0; i< (int)length; i++)
	*outs << string[i];
      *outs << endl << ends << flush;

      ParseError();
      // Mitteilung eines Fehlers
    }
  // zu Ausgabezwecken
    
  //---------------------------------------------------------------------
  // Parser fuer <MD> (markierungsabhaengige Ausdruecke)
  // folgende Funktionen stehen zum Parsen zur Verfuegung
  // und erwarten den String als Parameter
  // saemtliche Funktionen parsen nach festgelegter EBNF fuer das 
  // TimeNET Dateiformat
  //---------------------------------------------------------------------

  double Parse_md_weight(charP string) 
    {
      if (!string || !*string)
	return 0.0;
      
      DestroyBlancs(string);

      return (ParseIfExpression (string));
    };
  
  bool Parse_md_enable(charP string)
    {
      if (!string || !*string)
	return true;
      
      DestroyBlancs(string);

      if (string[strlen(string)-1]!=';')
	return (ParseLogicCondition (string, strlen(string)));
      else
	return (ParseLogicCondition (string, strlen(string)-1));
    };
  
  double Parse_md_arc_mult(charP string) 
    {
      if (!string || !*string)
	return 0.0;
      
      DestroyBlancs(string);

      return (ParseIfExpression (string));
    };
  
  double Parse_param_def(charP string)
    {
      if (!string || !*string)
	return 0.0;
      
      DestroyBlancs(string);

      if (string[strlen(string)-1]!=';')
	return (ParseExpression (string, strlen(string)));
      else
	return (ParseExpression (string, strlen(string)-1));
    };
  
  //---------------------------------------------------------------------
  // folgende Funktionen stehen zum Parsen zur Verfuegung
  // und erwarten den String und seine Laenge als Parameter
  // saemtliche Funktionen parsen nach festgelegter EBNF fuer das 
  // TimeNET Dateiformat
  //---------------------------------------------------------------------

  double ParseIfExpression(charP string);
  // fuer IfExpressions

  double ParseExpression(charP string, unsigned int length);
  // fuer Expressions

  double ParseRealValue(charP string, unsigned int length);
  // fuer RealValues or IntegerValues
  
  double ParseRealConstant(charP string, unsigned int length);
  // fuer RealConstants

  bool ParseLogicCondition(charP string, unsigned int length);
  // fuer LogicConditions

  bool ParseComparison(charP string, unsigned int length);
  // fuer Comparisons

  unsigned long ParseMarkFunc(charP string, unsigned int length);
  // fuer MarkFuncs

  unsigned long ParseIntegerValue(charP string, unsigned int length);
  // fuer IntegerValues

  unsigned long ParseIntegerConstant(charP string, unsigned int length);
  // fuer IntegerConstants

  //---------------------------------------------------------------------
  // folgende Funktionen muessen selber definiert werden:
  //---------------------------------------------------------------------

  virtual void ParseError() {};
  // wird im Fehlerfall aufgerufen

  virtual unsigned long ParseMarking(charP , unsigned int ) {return 1;};
  // fuer Markings
  // Input:  Name und Laenge einer Stelle ("#P1", 3)
  // Output: Markierung der Stelle: P1

  virtual double ParseParameter(charP , unsigned int ) {return 1.0;};
  // fuer Parameters (sowohl real als auch int)
  // Input:  Name und Laenge eines Parameter ("Anzahl", 6)
  // Output: Aufruf von einer der beiden folgenden Funktionen,
  //         je nach Typ des Parameter

  virtual double ParseRealParameter(charP , unsigned int ) {return 1.0;};
  // fuer RealParameters
  // Input:  Name und Laenge eines RealParameter ("rvalue", 6)
  // Output: ausgewerteter Inhalt des RealParameter (ParseExpression(inhalt...))

  virtual unsigned long ParseIntegerParameter(charP , unsigned int ) {return 1;};
  // fuer IntegerParameters
  // Input:  Name und Laenge eines IntegerParameter ("ivalue", 6)
  // Output: ausgewerteter Inhalt des IntegerParameter (ParseExpression(inhalt...))
};

