#include <Xm/RowColumn.h>
#include <Xm/ToggleB.h>

#include "scope.h"
#include "bool.h"
#include "tool.h"

VALUE_NOTIFY("bool",Bool)


Bool::Bool(NetClass* nc, bool hidden):
	Value(nc,hidden),
	rowcol(0),
	on(0),
	off(0),
	val(false)
{
}


Bool::Bool(const Bool& b):
	Value(b),
	rowcol(0),
	on(0),
	off(0),
	onstr(b.onstr),
	offstr(b.offstr),
	val(b.val)
{
}


istream& Bool::Parse(istream& is)
{
	Scope sb(is,0,2);	
	while (is && sb.Get())
		switch (sb.field)
		{
			case 1: if (!(is>>onstr)) cerr<<"label-string expected!"<<endl; break;
			case 2: if (!(is>>offstr)) cerr<<"second label expected!"<<endl; break;
		}
	if (!bool(onstr))
	{
		istrstream("\"\"")>>onstr;
//	istrstream("\"Off\"")>>offstr;
	}
	return is;
}

istream& Bool::Input(istream& is)
{
	Ident id(false);
	if (is>>id)
	{
		if (id=="true") *this = true;
		else if (id=="false") *this = false;
		else is.clear(ios::badbit);
	}
	return is;
}

ostream& Bool::Output(ostream& os) const
{
	return os<<(val==true?"true":"false");
}

Widget Bool::CreateWidget(Widget parent)
{
	if (bool(offstr))
	{
		rowcol = XmCreateRadioBox(parent,"Bool",NULL,0);

		on = XtVaCreateManagedWidget("On",xmToggleButtonWidgetClass,rowcol,NULL);
		off = XtVaCreateManagedWidget("Off",xmToggleButtonWidgetClass,rowcol,NULL);
		SetStringValue(on,XmNlabelString,onstr);
		SetStringValue(off,XmNlabelString,offstr);
		*this = val;
		XtManageChild(rowcol);
		CatchDestroy(&rowcol,&on,&off);
		return rowcol;
	}
	else
	{
		on = XtVaCreateManagedWidget("Bool",xmToggleButtonWidgetClass,parent,NULL);
		SetStringValue(on,XmNlabelString,onstr);
		
		*this = val;
		CatchDestroy(&on);
		return on;
	}
}

bool Bool::EvalWidget()
{
	Boolean b = val?True:False; // Xt Boolean!
	if (on)
		XtVaGetValues(on,XmNset,&b,NULL);
	else
		cerr<<"Evaluating nonexisting Widget!"<<endl;
	val = b?true:false;
	return true;
}

Value& Bool::operator =(const bool b)
{
	val = b;
	if (on) XtVaSetValues(on,XmNset,b?True:False,NULL);
	if (off) XtVaSetValues(off,XmNset,b?False:True,NULL);
	return *this;
}

