#include "nodecl.h"
#include "node.h"
#include "conncl.h"
#include "connect.h"
#include "netcl.h"
#include "net.h"

ConnectorClass::ConnectorClass(NetClass& nc): 
	ObjectClass(nc,"C"), 
	iter(rules)
{
}


ConnectorClass::~ConnectorClass()
{
}


bool ConnectorClass::AllowedConnection(Node* i, Node* o)
{
	if (!rules) return true;

	NodeClass* ic = i?i->Class():0,* oc = o ? o->Class():0;
	
	for (Rule* r=iter; r; r=++iter)
		if (r->AllowedConnection(ic,oc)) return true;
	
	return false;
}


istream& ConnectorClass::Input(istream& is)
{
	Ident id(false);
	Scope sp(is);
	while (is && sp.GetId(id))
	{
		if (id=="rules")
		{
			Scope sc(is);
			Ident nid;
			while (is && sc.Get())
			{
				LinkedList<NodeClass*> in,out;
				Scope sr(is,2,2);
				
				while (is && sr.Get())
					switch (sr.field)
					{
					case 1:	if (is>>nid) in = netcl.NodeClassesOfName(nid); break;
					case 2:	if (is>>nid) out = netcl.NodeClassesOfName(nid); break;
					}
				
				if (in && out)
					rules.Append(Rule(in,out));
				else
					cerr<<"Unknown Nodeclassnames in rule-definition"<<endl;
			}
		}
		else
			ObjectClass::Input(is, id);
	}
	return is;
}


void ConnectorClass::DrawButton(Display* dpy, Drawable d, int x, int y, int w, int h)
{
	Ident empty("");
	Position mid(x,y),s(w/200.0,h/200.0),
		p1 = Position(100.0,-20.0)*s+mid,p2 = Position(-100.0,20.0)*s+mid;
	LinkedList<Position*> poslist;
	Iterator<Position*> piter(poslist);
	Iterator<Position*> riter(relatives);
	
	poslist.Append(&p1);
	poslist.Append(&p2);
	
	bool mono = DefaultDepth(dpy,XDefaultScreen(dpy))==1;
	ShapeData sd(1,mid,defscale,0,piter,riter,false);
	ExprData ed(attr,empty,false,false,false,mono);
	
	ObjectClass::Draw(dpy,d,sd,ed);
}
