#include "ellarc.h"
#include "scope.h"

extern "C" {
#include <math.h>
}

SHAPE_NOTIFY("ellarc",Ellarc)


double Ellarc::Distance(ShapeData& sd, Position& ps, Position& nearest)
{
	Position psc = Pos(sd),rsc = (r*sd.scale).Abs(),psz = ps*sd.zoom;
	double angle = Position(psz.x-psc.x,psz.y-psc.y).Angle();
	return (nearest = psc+Position(cos(angle)*rsc.x,sin(angle)*rsc.y)).Distance(psz);
}


int Ellarc::Inside(Display*, Position pos, ShapeData& sd, ExprData&)
{
	Position nearest,psc = Pos(sd);
	Distance(sd,pos,nearest);
	return psc.Distance(pos*sd.zoom)<=psc.Distance(nearest)?RelativePos():0;
}


void Ellarc::DoDraw(Display* dpy, Drawable d, GC gc, ShapeData& sd, ExprData&)
{
	XPoint ps = Pos(sd),rs = Radius(sd);
	int shadow = int(sd.zoom*context.shadow);
	
	if (shadow)
		(context.filled?XFillArc:XDrawArc)(dpy,d,gc,
			ps.x-rs.x+shadow,ps.y-rs.y+shadow,rs.x*2,rs.y*2,angle1,angle2);
	
	(context.filled?XFillArc:XDrawArc)(dpy,d,gc,
		ps.x-rs.x,ps.y-rs.y,rs.x*2,rs.y*2,angle1,angle2);
}


istream& Ellarc::Input(istream& is, int field)
{
	double angle;
	switch (field)
	{
		case 1:	is>>p; break;
		case 2: if (!(is>>r.x)) cerr<<"radius expected."; r.y = r.x; break;
		case 3: if (!(is>>r.y)) cerr<<"y-radius expected."; break;
		case 4: if (!(is>>angle)) cerr<<"start-angle expected."; 
						angle1 = short(angle*64); break;
		case 5:	if (!(is>>angle)) cerr<<"end-angle expected.";
						angle2 = short(angle*64); break;
	}
	return is;
}

Metrics Ellarc::GetMetrics(Display*,ShapeData& sd,ExprData&)
{
  Position r = Radius(sd);
  Position p = Pos(sd);
  return Metrics(p.y-r.y,p.y+r.y,p.x-r.x,p.x+r.x);
}

/*
ostream& Ellarc::Output(ostream& os)
{
	return os<<<<p<<','<<r.x<<','<<r.y<<','<<angle1<<','<<angle2;
}
*/
