#include "xbefore.h"
#include <Xm/RowColumn.h>
#include <Xm/PushBG.h>
#include <Xm/CascadeB.h>
#include <Xm/PushB.h>
#include "xafter.h"

#include "enum.h"
#include "tool.h"
#include "scope.h"

VALUE_NOTIFY("enum",Enum)


Enum::Enum(NetClass* nc, bool hidden): 
	Value(nc,hidden),
	popup(0),
	option(0),
	val(0)
{
}


Enum::Enum(const Enum& e): 
	Value(e),
	popup(0),
	option(0),
	val(0)
{
	Iterator<Ident*> iter(e.defs);
	for (Ident* id=iter; id; id=++iter)
	{
		Ident* n = new Ident(*id);
		if (id==e.val) val = n;
		defs.Append(n);
	}
	
	if (!val && defs)
		val = *defs.First();
}


Enum::~Enum()
{
	Iterator<Ident*> iter(defs);
	for (Ident* id=iter; id; id=++iter)
		delete id;
	UncatchDestroy(&option,&popup);
}


istream& Enum::Parse(istream& is)
{
	Ident* id;
	Scope se(is,1);
	
	while (is && se.Get())
	{
		if (is>>*(id = new Ident))
		{
			// id vielleicht schon in def?
			defs.Append(id);
		}
		else
		{
			delete id;
			cerr<<"identifier expected!"<<endl;
		}
	}
	if (!val) val = *defs.First();
	return is;
}


istream& Enum::Input(istream& is)
{	
	Ident id;

	if (is>>id)
	{
		*this = id;
		return is;
	}
	else
		cerr<<"initial identifier expected!"<<endl;
	is.clear(ios::badbit);
	
	return is;
}


ostream& Enum::Output(ostream& os) const
{
	return os<<*val;
}


void Enum::UpdateWidget()
{
	if (option && popup)
	{
		WidgetList c; Cardinal nc;
		XtVaGetValues(popup,XmNchildren,&c,XmNnumChildren,&nc,NULL);
		while (nc--)
			if (XmIsPushButtonGadget(c[nc]))
			{
				XmString string; Value* v;
				XtVaGetValues(c[nc],XmNlabelString,&string,XmNuserData,&v,NULL);
				if (v==val)
				{
					XtVaSetValues(option,
						XmNmenuHistory,c[nc],XmNlabelString,string,NULL);
					break;
				}
		 }
	}
}


Widget Enum::CreateWidget(Widget parent)
{
	Widget curr = 0;
	Ident* i;
	Iterator<Ident*> iter(defs);
	
	popup = XmCreatePulldownMenu(parent,"EnumPopup",NULL,0);
	for (i=iter; i; i=++iter)
	{
		Widget entry = XmCreatePushButtonGadget(popup,"Entry",NULL,0);
		XtVaSetValues(entry,XmNuserData,i,NULL);
		SetStringValue(entry,XmNlabelString,i->Label());
		XtManageChild(entry);

		if (i==val) curr = entry;
	}
	option = XmCreateOptionMenu(parent,"Enum",NULL,0);
	XtDestroyWidget(XmOptionLabelGadget(option));
	XtVaSetValues(option,XmNsubMenuId,popup,NULL);
	if (curr)
		XtVaSetValues(option,XmNmenuHistory,curr,NULL);
	XtManageChild(option);
	CatchDestroy(&option,&popup);
	
	return option;
}


bool Enum::EvalWidget()
{
	Widget curr = 0;
	
	XtVaGetValues(option,XmNmenuHistory,&curr,NULL);
	if (curr)
		XtVaGetValues(curr,XmNuserData,&val,NULL);

	return curr?true:false;
}


Value& Enum::operator =(const char* def)
{
	Iterator<Ident*> iter(defs);
	
	for (Ident *i=iter; i; i=++iter)
		if (*i==def)
		{
			val = i;
			UpdateWidget();
			return *this;
		}
	
	cerr<<"not an enum-member!"<<endl;
	return *this;
}
