#include "xbefore.h"
#include <Xm/Protocols.h>
#include <Xm/DialogS.h>
#include <Xm/FileSB.h>
#include <Xm/SelectioB.h>
#include "xafter.h"

#include "tool.h"
#include "filesel.h"

template<class T, class U> void FileSelector<T, U>::Callback(
	Widget,bool ok,XtPointer calldata)
{
	char* filename = ok?
		NormalString(((XmSelectionBoxCallbackStruct*)calldata)->value):"";
	
	XtUnmanageChild(fsel);
	if (cb) (instance->*cb)(udata,filename,ok);
}


template<class T, class U> FileSelector<T, U>::FileSelector(T* i):
	Wrapper<FileSelector<T,U>,bool>(this),
	instance(i),
	parent(0),shell(0),fsel(0),
	cb(0)
{
}


template<class T, class U> FileSelector<T, U>::~FileSelector()
{
	UncatchDestroy(&shell,&fsel);
	if (shell)
		XtDestroyWidget(shell);
}


template<class T, class U> char* FileSelector<T, U>::GetDir()
{
	XmString dir;

	XtVaGetValues(fsel,XmNdirectory,&dir,NULL);
	return NormalString(dir);
}


template<class T, class U> void FileSelector<T, U>::CreateWidget()
{
	shell = XtVaCreateWidget("Fileselector",xmDialogShellWidgetClass,parent,NULL);
	fsel = XtVaCreateWidget("SelectionBox",
		xmFileSelectionBoxWidgetClass,shell,NULL);

	CatchDestroy(&shell,&fsel);	
	
	AddCB(fsel,XmNcancelCallback,Callback,false);
	AddCB(fsel,XmNokCallback,Callback,true);
	WinMgrCB(shell,"WM_DELETE_WINDOW",Callback,false);
	XtUnmanageChild(XmSelectionBoxGetChild(fsel,XmDIALOG_HELP_BUTTON));
}

	
template<class T, class U> void FileSelector<T, U>::Open(Widget par, char* title, 
	void (T::*callb)(U, char*, bool), U u, char *dir, char *mask)
{ 
  if (fsel && XtIsManaged(fsel))
		return;

	if (parent && parent!=par)
	{
		UncatchDestroy(&shell,&fsel);	
		XtDestroyWidget(shell);
		shell = fsel = 0;
	}

	if ((parent = par))
		CreateWidget();

	XtVaSetValues(shell,XmNtitle,title,NULL);
	cb = callb;
	udata = u;

	if (dir) SetStringValue(fsel, XmNdirectory, dir);
	if (mask) SetStringValue(fsel, XmNpattern, mask);
	else    SetStringValue(fsel, XmNpattern, "*");

	XtManageChild(fsel);
}

#define FSELINSTANCIATE(the_class,the_udata) \
class the_class;\
template class FileSelector< the_class , the_udata >;\
template class Wrapper<FileSelector< the_class , the_udata >,bool>;

// for g++
#ifndef NoExplicitInstanciation
#define NoExplicitInstanciation

#include "wrapper.cc"

FSELINSTANCIATE(Net,bool)

#endif
