#include "xbefore.h"
#include <Xm/TextF.h>
#include <Xm/ArrowB.h>
#include <Xm/PushB.h>
#include <Xm/Form.h>
#include "xafter.h"

#include <strstream.h>

#include "int.h"
#include "scope.h"

VALUE_NOTIFY("int",Int)


Int::Int(NetClass* nc, bool hidden): 
	Value(nc,hidden),
	Wrapper<Int,bool>(this),
	val(0),
	max(MAXINT),
	min(MININT),
	form(0),
	text(0),
	up_r(300,50),
	dn_r(300,50)
{
}


Int::Int(int v): 
	Wrapper<Int,bool>(this),
	val(v),
	max(MAXINT),
	min(MININT),
	form(0),
	text(0),
	up_r(300,50),
	dn_r(300,50)
{
}


Int::Int(const Int& i): 
	Value(i),
	Wrapper<Int,bool>(this),
	val(i.val),
	max(i.max),
	min(i.min),
	form(0),
	text(0),
	up_r(300,50),
	dn_r(300,50)
{
}


istream& Int::Parse(istream& is)
{
	Scope si(is,0,2);	
	min = 0;
	while (is && si.Get())
		switch (si.field)
		{
			case 1:
				if (!(is>>min))
					cerr<<"integer (min. value) expected!"<<endl;
				break;
			case 2:
				if (is>>max)
				{
					if (max<min)
					{
						int tmp = max;
						max = min;
						min = tmp;
					}
				}
				else
					cerr<<"integer (max. value) expected!"<<endl;
				break;
		}
	return is;
}


istream& Int::Input(istream& is)
{
	if (is>>val)
	{
		*this = val;
		return is;	
	}
	else
		cerr<<"integer expected!"<<endl;
	is.clear(ios::badbit);
	return is;
}


void Int::Arrow(Widget, bool up, XtPointer calldata)
{
	// XmPushButtonCallbackStruct* abcs = (XmPushButtonCallbackStruct*)calldata;
	// bool dclick = abcs->click_count>1;
	istrstream istr(XmTextFieldGetString(text));
	int tmp;
	
	istr>>tmp;
	if (up)
		tmp = tmp>=max ? max:tmp+1;
	else
		tmp = tmp<=min ? min:tmp-1;
	
	UpdateWidget(tmp);
}


void Int::UpdateWidget(int val)
{
	char buf[32]; ostrstream ostr(buf,32);
	ostr<<val<<ends;
	XmTextFieldSetString(text,buf);
	XmTextFieldSetInsertionPosition(text,XmTextFieldGetLastPosition(text));
}


Widget Int::CreateWidget(Widget parent)
{
	Widget up,dn;
	form = XtVaCreateManagedWidget("Int",xmFormWidgetClass,parent,NULL);
	dn_r = dn = XtVaCreateManagedWidget("Dn",xmPushButtonWidgetClass,form,NULL);
	text = XtVaCreateManagedWidget("Edit",xmTextFieldWidgetClass,form,NULL);
	up_r = up = XtVaCreateManagedWidget("Up",xmPushButtonWidgetClass,form,NULL);
	CatchDestroy(&form,&text);
	AddCB(up,XmNarmCallback,Arrow,true);
	AddCB(dn,XmNarmCallback,Arrow,false);
	AddCB(text,XmNvalueChangedCallback,Edit,true);
	UpdateWidget(val);
	return form;
}

bool Int::EvalWidget()
{
	if (text)
	{
		istrstream istr(XmTextFieldGetString(text));
		if (istr>>*this) UpdateWidget(val);
		return !istr.fail();
	}
	cerr<<"Evaluating nonexisting Widget!"<<endl;
	return true;
}

Value& Int::operator =(const int i)
{
	if ((val = i)>max) val = max; else if (val<min) val = min;
	if (text)
		UpdateWidget(val);
	
	return *this;
}


Int::operator char*() const
{
	static char buf[64]; ostrstream ostr(buf,64);
	ostr<<val<<ends;
	return buf;
}
