#include "net.h"
#include "page.h"
#include "object.h"
#include "node.h"
#include "editor.h"
#include "ident.h"
#include "netcl.h"
#include "module.h"
#include <iostream.h>
#include <fstream.h>
#include <strstream.h>

#include "xbefore.h"
#include <Xm/RowColumn.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include "xafter.h"

#include "main.h"
#include "regex.h"

#define ENDING ".net"

LinkedList<void*> Net::nets; // void* == Workaround fuer g++ 2.6.3 Bug
char* NoFilename = "Unnamed";


Net::Net(NetClass* nc, char* fname):
	prime(new Page(this,nc)),
	undo(0),
	initial(0),
	curr_editor(0),
	netcl(nc),
	attr(nc->Attributes()),
	modified(false),
	locked(false),
	filename(fname),
	request(this),
	message(this),
	fsel(this)
{
	nets.Append(this);

	if (!bool(filename))
		filename = NoFilename;

	InitModules();
}


Net::Net(Net& n): 
	prime(n.prime->Copy(this)),
	undo(n.undo->Copy(this)),
	curr_editor(0),
	netcl(n.netcl),
	attr(n.attr),
	modified(n.modified),
	locked(false),
	filename(n.filename),
	request(this),
	message(this),
	fsel(this)
{
	nets.Append(this);
	InitModules();
}

Net* Net::Copy()
{
	return new Net(*this);
}


Net::~Net()
{
	ExitModules();
	nets.Remove(this);

	if (undo) delete undo;
	if (prime) delete prime;
}


Net* Net::GetNet(Ident& /*name*/)
{
/*	Iterator<void*> iter(nets);

	for (void* n=iter; n; n=++iter)
		if (NetP(n)->Name()==name)
			return NetP(n);

	cerr<<"Unknown net '"<<name<<"' requested!"<<endl;
*/	return 0;
}


bool Net::Locked()
{
  return locked;
}


bool Net::Save(Page* root)
{
	ofstream os((char*)filename);
	if (!root) root = prime;

	if (!os)
	{
		cerr<<"Error creating file "<<filename<<"!"<<endl;
		return false;
	}
	
	os<<root->Class()->Name()<<" ("<<endl;
	root->Write(os);
	os<<endl<<")"<<endl;

	if (os && root==prime) 
	{
		modified = false;
		Update();
	}	
	
	return os;
}


void Net::Kill(Widget shell, bool ask)
{
	if (ask && Modified())
		request.OpenMsg("CloseRequester",shell,0,RequestCB,"Yes No Cancel");
	else
	{
		ArrayIterator<Editor*,Page*> iter(pages);
		Tuple<Editor*,Page*>* t;
		while ((t = iter))
			CloseEditor(*t->Index());
		delete this;
	}
}


void Net::Exit(Widget shell, bool ask)
{
	Iterator<void*> iter(nets);
	void *n;

	for (n=iter; ask && n; n=++iter)
		if (NetP(n)->Modified())
		{
			request.OpenMsg("ExitRequester",shell,0,RequestCB,"Ok Cancel");
			return;
		}

	while ((n = iter))
		NetP(n)->Kill(shell,false);
}


void Net::SaveAsCB(bool, char* filename, bool ok)
{
	if (ok)
	{
    Ident file = filename;
    char* end = filename+strlen(filename)-strlen(ENDING);
    if (strcmp(end, ENDING)) file += ENDING;

		SetFileName(file);
		Save(); 
                for (int i= strlen(filename)-1; i>=0; i--)
	          if (filename[i]=='/') {filename[i]='\0';break;}
		curr_editor->SetCurrentPath(filename);
	}
}


void Net::LoadCB(bool, char* file, bool ok)
{
	if (ok)
	{
		Net* newnet = Load(file);
		if (newnet)
			newnet->OpenEditor(0,curr_editor->DpyName());
                for (int i= strlen(file)-1; i>=0; i--)
	          if (file[i]=='/') {file[i]='\0';break;}
		curr_editor->SetCurrentPath(file);
	}
}


void Net::AddCB(bool, char* file, bool ok)
{
	if (ok)
	{
		Page& curr_page = **pages[curr_editor];
		Net* addnet = Load(file);
		if (!addnet) return;

		if (curr_page.Class()==addnet->Prime()->Class())
		{
			UndoCopy();
			curr_page.Deselect();
			curr_page.PageCopy(*addnet->Prime());
		}
		
		delete addnet;
                for (int i= strlen(file)-1; i>=0; i--)
	          if (file[i]=='/') {file[i]='\0';break;}
		curr_editor->SetCurrentPath(file);
                RefreshModules();

		modified= true;
	}
}


Value* Net::Attr(char* name)
{
	return attr.GetMember(name);
}


void Net::UndoCopy()
{
	modified = true;
	if (undo) delete undo;
	undopages = pages;
	undo = prime->Copy(this,false,&undopages);
}


void Net::Undo()
{
	if (!undo) return;
	
	Page* save = prime;
	prime = undo;
	undo = save;
	
	Array<Editor*,Page*> arr = pages;
	pages = undopages;
	undopages = arr;

	ArrayIterator<Editor*,Page*> iter(pages);
	for (Tuple<Editor*,Page*>* t=iter; t; t=++iter)
		t->Index()->Update();
}


Net* Net::Load(char* fpath)
{
	ifstream fis(fpath);

	if (fis)
	{
		Ident ntid;
		NetClass* nc;
		if (fis>>ntid && (nc = NetClass::GetNetClass(ntid)))
		{
			Net* n = new Net(nc,fpath);
			Scope sn(fis);
			sn.Get();
			if (!n->prime->Read(sn,fis))
			{
				delete n; n = 0;
			}
			
			if (n) {
				n->prime->Renumber();
        n->UndoCopy();
      }

		n->modified= false;

		return n;
		}
	}
	else
		cerr<<"Can't open Netfile"<<endl;

	return 0;
}


void Net::InitModules()
{
	Iterator<ModuleInstanciator> iter = Module::Modules();
	for (ModuleInstanciator mi=iter; mi; mi=++iter)
	{
		Module* m = mi(*this);
		if (m->Init())
			modules.Append(m);
		else
			delete m;
	}
}


void Net::RefreshModules()
{
        Page& curr_page = **pages[curr_editor];
        Module* m;
        Iterator<Module*> miter(modules);
        for (m=miter; m; m=++miter)
          m->Refresh(curr_editor,curr_page);
        //cout << "RefreshModules\n";
}


void Net::ExitModules()
{
	Module* m;
	Iterator<Module*> miter(modules);
	while ((m = miter))
	{
		modules.Remove(m);
		delete m;
	}
}


Page* Net::GetPage(Editor* ed)
{
	return *pages[ed];
}


Editor* Net::OpenEditor(Page* p, char* dpy_str)
{
	initial = p; // zum weiterleiten nach EditorOpened
  Editor* ed = Editor::NewEditor(this, dpy_str);
  if (!ed) initial = 0;
  return ed;
}


	
void Net::EditorOpened(Editor& ed)
{
	Page* p = initial;
	if (!p) p = prime;
	initial = 0;
	pages.Set(&ed,p);
	Iterator<Module*> miter(modules);
	for (Module* m=miter; m; m=++miter)
		if ((m->Open(ed,*p)))
			ed.NewModule(m->Name());
	ed.NewNetClass(p->Class());
	UpdateButtonStates(ed,*p);
	ed.Update();
}


void Net::ChangeEditor(Editor& ed, Page* p)
{
	Page* old = *pages[&ed];
	pages.Set(&ed,p);
	if (old->Class()!=p->Class())
		ed.NewNetClass(p->Class());
	ed.Update();

 	Iterator<Module*> miter(modules);
	for (Module* m=miter; m; m=++miter)
	{
		if (m->Change(ed,*old,*p))
			ed.NewModule(m->Name());
		else
			ed.RemoveModule(m->Name());
	}
}


void Net::CloseEditor(Editor& ed)
{
	delete &ed;
}


void Net::EditorClosed(Editor& ed)
{
	if (ed.Dpy())
		Context::DisplayWillClose(ed.Dpy());
	
	Page& pg = **pages[&ed];
	
	Iterator<Module*> miter(modules);
	for (Module* m=miter; m; m=++miter)
		m->Close(ed,pg);
	
	pages.Unset(&ed);
	undopages.Unset(&ed);
}


void Net::PageKilled(Page* p)
{
	ArrayIterator<Editor*,Page*> iter(pages);
	for (Tuple<Editor*,Page*>* t=iter; t; )
		if (t->Value()==p)
		{
			CloseEditor(*t->Index());
			t = iter;
		}
		else t = ++iter;
}


void Net::Update(Page* p)
{
	ArrayIterator<Editor*,Page*> iter(pages);
	for (Tuple<Editor*,Page*>* t=iter; t; t=++iter)
		if (!p || t->Value()==p)
			t->Index()->Update();
}


bool Net::Lock(bool on)
{
	if (on)
	{
		if (locked) return false;
		locked = true;
	}
	else
	{
		if (!locked) return false;
		locked = false;
	}
		
	ArrayIterator<Editor*,Page*> iter(pages);
	for (Tuple<Editor*,Page*>* t=iter; t; t=++iter)
	{
		Editor* ed = t->Index();
		if (ed)
		{
			if(ed!=curr_editor)
				ed->Lock(locked);
			UpdateButtonStates(*ed,*t->Value());
		}
	}

	if (!locked) curr_editor = 0;
	return true;
}


void Net::Draw(Editor& editor, Display* dpy, Drawable d)
{
	Page& curr_page = **pages[&editor];
	Module* m;
	Iterator<Module*> miter(modules);

	bool dont_draw = false;
	for (m=miter; m; m=++miter)
		dont_draw = dont_draw || m->Draw(editor,curr_page,dpy,d,true);

	if (!dont_draw)
		curr_page.Draw(editor,dpy,d,true);
	
	editor.Copy();
	
	for (m=miter; m; m=++miter)
		m->Draw(editor,curr_page,dpy,d,false);

	if (!dont_draw)
		curr_page.Draw(editor,dpy,d,false);
}


char* Net::Title(Editor& editor)
{
	Page& curr_page = **pages[&editor];
	return curr_page.FullName();
}


void Net::Event(Editor& editor, EventInfo& ev)
{
	if (curr_editor && curr_editor!=&editor)
		return;
	
	Page& curr_page = **pages[&editor];
	EditorSettings set = editor.Settings();
	Module* m;
	Iterator<Module*> miter(modules);
	
	// gibt zurueck, ob die Aktion weiter aktiv ist (locking!)
	for (m=miter; m; m=++miter)
		if (m->ModuleLock()) {
      Lock(true); return; }
  Lock(false);

	// gibt zurueck, ob der Event weiterverarbeitet werden soll
	for (m=miter; m; m=++miter)
		if (m->Event(editor,curr_page,ev,true))
			return;

	// gibt zurueck, ob die Aktion weiter aktiv ist (locking!)
	curr_editor = &editor;
  bool lock = curr_page.Event(editor,ev);
  locked = !lock; Lock(lock);  // Lock() fragts ab!

  if (!Editor::Exists(&editor)) return;
	
	editor.Coordinate(ev.pos.Grid(set.mvgrid));
	
	for (m=miter; m; m=++miter)
	// Rueckgabewert wird nicht beachtet
		m->Event(editor,curr_page,ev,false);
}


void Net::UpdateButtonStates(Editor& editor, Page& page)
{
	bool sel = page.NextSelected()!=0, objs = page.NumObjects()!=0;

	editor.ButtonState("Save",filename!=NoFilename);
	editor.ButtonState("Remove Cut Copy",sel);
	editor.ButtonState("Undo",undo!=0);
	editor.ButtonState("Enter",page.CurrSubPage()!=0);
	editor.ButtonState("Leave",page.SuperPage()!=0);
	editor.ButtonState("Next Prev Select SelectAll",objs);
}



void Net::Menu(Editor& editor, Ident& name)
{
	Iterator<Module*> miter(modules);
	Module* m;
	for (m=miter; m; m=++miter)
		if (m->ModuleLock()) {
      Lock(true); return; }

	if (locked)
		return;
	
	Page& curr_page = **pages[curr_editor = &editor];
	EditorSettings set = editor.Settings();
	Widget shell = editor.Shell();
//	Module* m;
	
//	Iterator<Module*> miter(modules);
	for (m=miter; m; m=++miter)
		if (m->Menu(editor,curr_page,name,true))
			return;
	
	Lock(true);

	if (name=="NewEditor")
		OpenEditor(&curr_page,editor.DpyName());
	else if (name=="NewDisplay")
	{
		AskForDisplay(shell);
		return;
	}
	if (name=="Load" || name=="Add") 
	{
		char* path = editor.CurrentPath();

		if (name=="Add")
			fsel.Open(shell,"Select Net to add...",AddCB,false,path,"*" ENDING);
		else
			fsel.Open(shell,"Select Net to load...",LoadCB,false,path,"*" ENDING);
		return;
	}
	else if (name=="Save" && filename!=NoFilename)
		Save();
	else if (name=="SaveAs" || name=="Save")
	{
		Text title = Text("Save Net ")+FullName()+" as...";
		char* path = editor.CurrentPath();
		fsel.Open(shell,title,SaveAsCB,false,path,"*" ENDING);

//geaendert fuer SPNL von Ronald Markworth (indy@cs.tu-berlin.de)
//		for (m=miter; m; m=++miter)
//		  m->Menu(editor,curr_page,name,false);
//geaendert fuer SPNL von Ronald Markworth (indy@cs.tu-berlin.de)

		return;
	}
  else if (name=="Enter")
  {
    Page* pg = curr_page.CurrSubPage();      
		if (pg)
			ChangeEditor(editor,pg);
  }
  else if (name=="Leave")
  {
		Page* pg = curr_page.SuperPage();
		if (pg) {
      pg->Deselect();
      curr_page.Parent()->Select(true);
			ChangeEditor(editor,pg);
    }
  }
	else if (name=="SelectAll")
		curr_page.Select(0);
	else if (name=="Select")
	{
		AskForExpression(shell);
		return;
	}
	else if (name=="Prev" || name=="Next")
	{
		curr_page.SelectNextObject(name=="Next");	
	}
	else if (name=="Cut" || name=="Remove")
	{
		if (name=="Cut")
			curr_page.Copy();

		UndoCopy();
		curr_page.Remove();
	}
	else if (name=="ObjInfo")
	{
		Object* ob = curr_page.NextSelected();
		if (ob)
		{
			ObjectDialog(shell,ob,set.expert_mode, false);
			return;
		}
	}

	if ((name=="ObjInfo" || name=="NetInfo") && attr.NumMembers())
	{
		NetDialog(shell,set.expert_mode);
		return;
	}
	else if (name=="Undo")
		Undo();
	else if (name=="Quit")
	{
		Exit(shell);
		return;
	}
	else if (name=="Kill")
	{
		Kill(shell);
		return;
	}
	else if (name=="Close")
	{
		if (EditorCount()>1)
		{
			CloseEditor(editor);
			Lock(false);
		}
		else
			Kill(shell);
		return;
	}
	else if (name=="Copy")
		curr_page.Copy();
	else if (name=="Paste")
	{
		UndoCopy();
		curr_page.Paste();
	}
	else // uU New Page -> Netzklasse oder Modul gewaehlt
	{
		Iterator<void*> iter = NetClass::NetClasses();
		void* nc;

		for (nc=iter; nc; nc=++iter)
			if (NetClassP(nc)->Name()==name)
			{
				Net* n = new Net(NetClassP(nc));
				n->OpenEditor(PageP(0),editor.DpyName());
				break;
			}
		
		if (!nc) // war keine Netzklasse, also vielleicht Modul?
			for (m=miter; m; m=++miter)
				if (name==m->Name())
					m->Call(editor,curr_page);
	}

	Lock(false);

	for (m=miter; m; m=++miter)
		if (m->Menu(editor,curr_page,name,false))
			return;
}


void Net::ObjectDialog(Widget shell, Object* ob, bool hidden, bool readonly)
{
	Struct* attr = ob->Attribute();
	Ident& name = ob->Name();
	if (attr)
	{
		Text title = ob->FullName();

		UndoCopy();

		if (readonly)
		  request.Create("ObjectDialog",title,shell,ob,RequestCB,"Cancel");
		else 
		  request.Create("ObjectDialog",title,shell,ob,RequestCB,"Ok Apply Cancel");
		
		Widget form = XtVaCreateManagedWidget("InfoForm",xmFormWidgetClass,
			request,0);
		Widget nform = XtVaCreateManagedWidget("NameForm",xmFormWidgetClass,
			form,0);
		XtVaCreateManagedWidget("Label",xmLabelWidgetClass,nform,0);

		name.CreateWidget(nform);
		attr->EnterDialog(form,hidden);

		request.Manage();
	}
}


void Net::NetDialog(Widget shell, bool hidden)
{
	UndoCopy();
	request.Create("NetDialog",filename,shell,0,RequestCB,"Ok Apply Cancel");

	attr.EnterDialog(request,hidden);

	request.Manage();
}


void Net::AskForDisplay(Widget shell)
{
	request.Create("DisplayRequester","Open Editor...",
		shell,0,RequestCB,"Ok Apply Cancel");

	Widget form = XtVaCreateManagedWidget("DisplayForm",
		xmFormWidgetClass,request,0);
	XtVaCreateManagedWidget("Label",xmLabelWidgetClass,form,0);

	input = "";
	input.CreateWidget(form);

	request.Manage();
}


void Net::AskForExpression(Widget shell)
{
	request.Create("SelectRequester","Select by Expression...",
		shell,0,RequestCB,"Ok Apply Cancel");

	Widget form = XtVaCreateManagedWidget("SelectForm",
		xmFormWidgetClass,request,0);
	XtVaCreateManagedWidget("Label",xmLabelWidgetClass,form,0);

	input = "";
	input.CreateWidget(form);

	request.Manage();
}


void Net::RequestCB(Dialog<Net, Object*>& d, Object* ob, Ident& btn)
{
	Ident name(XtName(d));
	Page& curr_page = **pages[curr_editor];

	if (name=="NetDialog")
	{
		if (btn!="Cancel" && !attr.EvalWidget())
		{
			message.OpenMsg("AttributeError",d,0);
			return;
		}

		Update();
		if (btn=="Apply")
			return;
	}
	else if (name=="ObjectDialog")
	{
		if (btn!="Cancel")
		{
			Ident old = ob->Name();
			if (!ob->Name().EvalWidget())
			{	
				message.OpenMsg("IncompleteName",d,0);
				return;
			}
			else if (curr_page.NameExists(ob))
			{
				ob->Name() = old;
				message.OpenMsg("NameExists",d,0);
				return;
			}

			if (!ob->Attribute()->EvalWidget())
			{
				message.OpenMsg("AttributeError",d,0);
				return;
			}
                        RefreshModules();
			Update();
			if (btn=="Apply")
				return;

			if ((ob = curr_page.NextSelected(ob)))
			{
				d.Close();
				ObjectDialog(curr_editor->Shell(),ob,true, false);
				return;
			}
		}
	}
	else if (name=="CloseRequester")
	{
		if (btn=="Yes")
			Save();

		if (btn=="No" || btn=="Yes")
		{
			Lock(false);
			Kill(0,false);
			return;
		}
	}
	else if (name=="ExitRequester")
	{
		if (btn=="Ok")
		{
			Lock(false);
			Exit(0,false);
			return;
		}
	}
	else if (name=="DisplayRequester")
	{
		if (btn=="Ok" || btn=="Apply")
		{
			input.EvalWidget();
			OpenEditor(&curr_page,input);
			
			if (btn=="Apply") return;
		}
	}
	else if (name=="SelectRequester")
	{
		if (btn=="Ok" || btn=="Apply")
		{
			input.EvalWidget();

			curr_page.Select(input);
			if (btn=="Apply") return;
		}
	}

	Lock(false);
	d.Close();
}
