#include "xbefore.h"
#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>
#include <Xm/DrawnB.h>
#include "xafter.h"

#include "objcl.h"
#include "object.h"
#include "scope.h"
#include "attrib.h"
#include "shape.h"
#include "netcl.h"


ObjectClass::ObjectClass(NetClass &nc, char* nprefix): 
	netcl(nc),
	attr(&nc,&nc.Attributes()),
	prefix(nprefix),
	defscale(1,1)
{
}


ObjectClass::~ObjectClass()
{
	Iterator<Shape*> siter(shapes);
	Shape* s;
	
	for (s=siter; s; s=++siter) delete s; 
	
	Iterator<Position*> piter(relatives);
	for (Position* p=piter; p; p=++piter) delete p; 
}


istream& ObjectClass::Input(istream& is, Ident& id)
{
	if (id=="name")
	{
		if (!(is>>name))
			cerr<<"Name of objectclass expected!"<<endl;
	}
	else if (id=="nameprefix")
	{
		if (!(is>>prefix))
			cerr<<"Nameprefix of objectclass expected!"<<endl;
	}
	else if (id=="defscale")
		is>>defscale;
	else if (id=="relatives")
	{
		Scope sp(is);
		while (is && sp.Get())
		{
			Position* p = new Position;
			if (p && is>>*p)
				relatives.Append(p);
		}
	}
	else if (id=="attributes")
	{
		attr.Parse(is);
	}
	else if (id=="shape")
	{
		Scope sps(is); Shape *s;
		while (is && sps.Get() && (s = Shape::GetShape(is, 0)))
			shapes.Append(s);
	}
	else if (id=="group")
	{
		if (!(is>>gname))
			cerr<<"Name of objectclass-group expected!"<<endl;
	}
	else
	{
		cerr<<"Unknown Identifier '"<<id<<"'!"<<endl;
		is.clear(ios::badbit);	
	}

	return is;
}


void ObjectClass::Draw(Display* dpy, Drawable d, ShapeData& sd, ExprData& ed)
{
	Iterator<Shape*> iter(shapes);
	for (Shape *s=iter; s; s=++iter)
		s->Draw(dpy,d,sd,ed,ed.draft);
}


double ObjectClass::Nearest(ShapeData& sd, Position& ps, Position& nearest)
{
	return (*shapes.First())->Distance(sd,ps,nearest);
}


int ObjectClass::IsUnder(Display* dpy,Position pos,ShapeData& sd,ExprData& ed)
{
	Iterator<Shape*> iter(shapes);
	int rel = 0;
	
	for (Shape *s=iter; s; s=++iter)
	{
		int xrel = s->Inside(dpy,pos,sd,ed);
		if (xrel>0)
			return xrel;
		if (xrel) rel = xrel;
	}
	
	return rel;
}

Metrics ObjectClass::GetMetrics(Display* dpy,ShapeData& sd,ExprData& ed)
{
  Iterator<Shape*> iter(shapes);
  Metrics akt_metrics;
  
  for (Shape* s=iter; s; s=++iter)
    akt_metrics.Compare(s->GetMetrics(dpy,sd,ed));

  return akt_metrics;
}

    
