#include "object.h"
#include "objcl.h"
#include "page.h"
#include "netcl.h"
#include "int.h"


unsigned Object::global_id = 1;

Object::Object(Page* n, ObjectClass* cl, char* nam, Struct* st): 
	selected(false),
	page(n),
	objcl(cl),
	attr(st),
	scale(cl->Scale()),
	curr(0),
	id(NewID()),
	name(nam)
{
	Iterator<Position*> iter = cl->Relatives();
	for (Position* r=iter; r; r=++iter)
		relatives.Append(new Position(*r));
		
	if(!name) AutoName();
	page->AddObject(this);
}


Object::Object(Object& o, Page* parent, Struct* st): 
	selected(o.selected),
	page(parent),
	objcl(o.objcl),
	attr(st),
	scale(o.scale),
  curr(0),
	id(o.id),
	name(o.name)
{
	Iterator<Position*> iter(o.relatives);
	for (Position* p=iter; p; p=++iter) 
		relatives.Append(new Position(*p));
	
	if (parent->Class()->CheckNames()) 
    AutoName();
  
	page->AddObject(this);
}


Object::~Object()
{
	Iterator<Position*> iter(relatives);
	
	for (Position* p=iter; p; p=++iter) delete p;
	page->RemoveObject(this);
}


void Object::AutoName()
{
	Int cnt = 0;
	while (!name || page->NameExists(this,true))
		name = objcl->NamePrefix()+ ++cnt;
}


void Object::SetClass(ObjectClass* cl)
{
	objcl = cl;

  if (relatives.Length() != cl->relatives.Length()) {
    Iterator<Position*> iter = relatives;
    Position* r;
    
    for (r=iter; r; r=++iter) delete r; 
    relatives.Clear();
	
    for (iter=cl->Relatives(),r=iter; r; r=++iter)
      relatives.Append(new Position(*r));
  }
	scale = cl->Scale();
}


void Object::SetRelative(int pos, Position p)
{
	if (pos>=0 && pos<int(relatives))
		**relatives[pos] = p;
}

Position Object::GetRelative(int pos)
{
	if (pos>=0 && pos<int(relatives))
		return **relatives[pos];
	else
		return Position();
}


Text Object::FullName()
{
	return page->FullName()+"."+name;
}


void Object::Select(bool dosel)
{
	if (selected==dosel) return;
	selected = dosel;
	page->Update();
}


bool Object::Event(Display* dpy, EventInfo& ev)
{
	int rel;
	switch (ev.event)
	{
	case EventInit:
		if ((rel = IsUnder(dpy,ev.pos))>0)
		{
			curr = *relatives[rel-1];
			return true;
		}
		return false;
	case EventMotion:
		if (curr)
			*curr = (ev.pos-Mid())/scale;
		return curr!=0;
	case EventQuickExit:
	case EventExit:
	default:
		curr = 0;
		return true;
	}
	return false;
}


Value* Object::Attr(char* name)
{
	return attr?attr->GetMember(name):0;
}


istream& Object::Input(istream& is, Ident& iid)
{
	if (iid=="name")
		is>>name;
	else if (iid=="scale")
		is>>scale;
	else if (iid=="id")
	{
		if (!(is>>id))
			cerr<<"ID of objecttype expected!"<<endl;
	}
	else if (iid=="attributes")
	{
		if (attr) is>>*attr;
	}
	else if (iid=="relatives")
	{
		Scope sp(is,0,int(relatives));
		Iterator<Position*> iter(relatives);
		Position* p = iter;
		
		while (is && sp.Get() && p)
		{
			is>>*p;
			p = ++iter;
		}
	}
	else
	{
		cerr<<"Unknown Identifier '"<<iid<<"'!"<<endl;
		is.clear(ios::badbit);	
	}
	return is;
}


ostream& Object::Output(ostream& os) // called only by Node/Connector::Output
{
	Iterator<Position*> iter(relatives);
	os<<objcl->Name()<<" ("<<"name = "<<name<<"; id = "<<id<<"; scale = "<<scale;

	if (attr && !Virtual()) os<<"; attributes = "<<*attr;
	os<<"; relatives = (";
	for (Position* p=iter; p; )
	{
		os<<*p;
		if ((p = ++iter)) os<<',';
	}
	return os<<')';
}



void Object::Draw(Display* dpy, Drawable d, double zoom, bool draft)
{
	ShapeData sd = GetShapeData(zoom,draft);
	ExprData ed = 
		GetExprData(DefaultDepth(dpy,XDefaultScreen(dpy))==1,draft);
	
	objcl->Draw(dpy,d,sd,ed);
}



int Object::IsUnder(Display* dpy, Position ps)
{
	ShapeData sd = GetShapeData(1,true);
	ExprData ed = 
		GetExprData(DefaultDepth(dpy,XDefaultScreen(dpy))==1,true);
	return objcl->IsUnder(dpy,ps,sd,ed);
}


Metrics Object::GetMetrics(Display* dpy)
{
        ShapeData sd = GetShapeData(1,true);
        ExprData ed = 
                GetExprData(DefaultDepth(dpy,XDefaultScreen(dpy))==1,true);
        return objcl->GetMetrics(dpy,sd,ed);
}




