#include "scope.h"
#include "text.h"
#include "ident.h"
#include "shape.h"
#include "value.h"
#include "array.h"


static Array<Ident,ShapeInstanciator> shapes;

ShapeNotification::ShapeNotification(char* name,ShapeInstanciator si) 
{
	shapes.Set(Ident(name,false),si);
}


Shape *Shape::GetShape(istream& is, Context* super)
{
	Shape *s = 0;
	Ident id(false);
	if (is>>id)
	{
		ShapeInstanciator* si = shapes[id];
		if(si)
			s = (*si)(is,super);
	}
	else
		cerr<<"name of shape expected!"<<endl;
	
	return s;
}


Position Shape::Absolute(ShapeData& p, Position r)
{
	Position pos = p.pos;
	double angle = p.angle;
	
	if (!p.node)
	{
		double length = 0;

		Position* p1 = p.iter,* p2 = ++p.iter,* last = p.iter.Last();

		for (; p2; p1=p2,p2=++p.iter) length += p1->Distance(*p2); 

		double dist,clen = (length*context.cpos)/100.0;
	
		for (p1=p.iter,p2=++p.iter; p2; p1=p2,p2=++p.iter)
			if ((clen -= (dist = p1->Distance(*p2)))<=0.0 || p2==last)
				break;
	
    Position tang = *p2-*p1;
    angle = tang.Angle();
    
		if (dist==0) pos = *p1;
    else pos = *p1+tang*((clen+dist)/dist);
	}

	if (context.relative>=0 && context.relative<int(p.rels))
		pos += *p.rels[context.relative]*p.scale;

	return ((r*p.scale).Rotate(angle)+pos)*p.zoom;
}


Position Shape::PointToMid(ShapeData& sd, Position& ps)
{
  Position nearest;
  Distance(sd, ps, nearest);
  return nearest;
}


void Shape::Draw(Display* dpy, Drawable d, ShapeData& sd, ExprData& ed, bool draw_xor)
{
	context.Evaluate(ed);
	if (context.condition)
	{
		GC gc = context.GetGC(dpy,draw_xor);
		context.SetZoom(dpy,gc,sd.zoom);
		DoDraw(dpy,d,gc,sd,ed);
	}
}


istream& Shape::Input(istream& is)
{
	Scope sp(is);
	while (is && sp.Get())
		switch (sp.field)
		{
			case 1: is>>context; break;
			default: Input(is,sp.field-1);
		}
	return is;
}
