#include "xbefore.h"
#include <Xm/Form.h>
#include <Xm/ScrolledW.h>
#include <Xm/PanedW.h>
#include <Xm/PushB.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/RowColumn.h>
#include "xafter.h"

#include "struct.h"
#include "tool.h"
#include "scope.h"
#include "netcl.h"
#include "object.h"

VALUE_NOTIFY("struct",Struct)


Struct::Struct(NetClass* nc, bool hidden): 
	Value(nc,hidden),
	Wrapper<Struct,int>(this),
	scroll(0),clip(0),paned(0),
	super(nc ? &nc->Attributes() : 0)
{
}


Struct::Struct(const Struct& s, Struct* sup): 
	Value(s),
	Wrapper<Struct,int>(this),
	scroll(0),clip(0),paned(0),
	super(sup?sup:s.super)
{
	Iterator<Attrib*> iter(s.members);
	for (Attrib* a=iter; a; a=++iter)
		members.Append(new Attrib(*a));
}


Struct::Struct(char* def, bool hidden): 
	Value(0,hidden),
	Wrapper<Struct,int>(this),
	scroll(0),clip(0),paned(0),
	super(NULL)
{
	istrstream is(def);
	Parse(is);
}


Struct::~Struct()
{
	if (clip)
	{
		RemoveCB(resize_id);
		if (expose_id) RemoveCB(expose_id);
	}
	UncatchDestroy(&scroll,&clip,&paned);
	Iterator<Attrib*> iter(members);
	for (Attrib* a=iter; a; a=++iter)
		delete a;
}


bool Struct::EvalWidget()
{
	if (paned)
	{
		Iterator<Attrib*> iter(members);
		for (Attrib* a=iter; a; a=++iter)
			if ((call_hidden || !a->Hidden()) && !a->EvalWidget())
				return false;
	}
	
	return true;
}


istream& Struct::Parse(istream& is)
{
	Iterator<Attrib*> iter(members);
	Attrib* a;
	for (a=iter; a; a=++iter)
		delete a;
	members.Clear();
	
	Scope dp(is);
	while (is && dp.Get())
	{
		a = new Attrib(netcl);
		if (is>>*a)
			members.Append(a);
		else
			delete a;
	}
	
	return is;
}


istream& Struct::Input(istream& is)
{
	Ident id;
	Scope dp(is);
	
	while (dp.GetId(id))
	{
		Iterator<Attrib*> iter(members);
		Attrib* attr;
		
		for (attr=iter; attr; attr=++iter)
			if (attr->Name()==id) break;
		
		if (!attr)
			cerr<<"Unknown member '"<<id<<"'!"<<endl;
		else if (is>>*attr->Type())
			continue;
		
		is.clear(ios::badbit);
	}
	
	return is;
}


ostream& Struct::Output(ostream& os) const
{
	bool first = true;
	
	os<<"{";
	Iterator<Attrib*> iter(members);
	for (Attrib* attr=iter; attr; attr=++iter)
	{
		if (first) first = false; else os<<' ';
		os<<*attr;
	}
	return os<<"}";
}


void Struct::ResizeCB(Widget, int resize, XtPointer)
{
	Dimension height = 0,width,st;

	if (resize)
	{
		XtVaGetValues(paned,XmNshadowThickness,&st,NULL);
		XtVaGetValues(clip,XmNwidth,&width,NULL);
		if (width>2*st)
			XtVaSetValues(paned,XmNwidth,width-2*st,NULL);
	}
	else
	{
		Iterator<Attrib*> iter(members);
		Attrib* a;
		for (a=iter; a; a=++iter)
			if (a->Val().IsStruct())
				((Struct&)a->Val()).ResizeCB(0,0,0);
	
		XtVaGetValues(scroll,XmNshadowThickness,&st,NULL);
		XtVaGetValues(paned,XmNheight,&height,NULL);
		if (height>600) height = 600;
		XtVaSetValues(scroll,XmNheight,height+2*st,NULL);
		
		RemoveCB(expose_id);
		expose_id = 0;
		expose_id = AddCB(clip,XmNexposeCallback,ResizeCB,1); // a first workaround...
		
		for (a=iter; a; a=++iter)
			if (call_hidden || !a->Hidden())
				a->LimitPaneSize();
	}
}


Widget Struct::CreateWidget(Widget parent)
{
	return EnterDialog(parent);
}


Widget Struct::EnterDialog(Widget parent, bool hidden)
{
	if (scroll) return scroll;
	scroll = XtVaCreateWidget("Struct",
		xmScrolledWindowWidgetClass,parent,NULL);
	_FixHighlight(scroll);

	XtVaGetValues(scroll,XmNclipWindow,&clip,NULL);
	paned = XtVaCreateManagedWidget("Paned",xmPanedWindowWidgetClass,scroll,NULL);
	if (clip)
	{
		resize_id = AddCB(clip,XmNresizeCallback,ResizeCB,1);
		expose_id = AddCB(clip,XmNexposeCallback,ResizeCB,0);
	}
	CatchDestroy(&scroll,&paned,&clip);
	
	call_hidden = hidden;
	Iterator<Attrib*> iter(members);
	for (Attrib* a=iter; a; a=++iter)
		if (hidden || !a->Hidden())
			a->CreateWidget(paned);
	
	if (members)
		XtManageChild(scroll);
	
	return scroll;
}


Value* Struct::GetMember(char* name) const
{
	Iterator<Attrib*> iter(members);
	for (Attrib* m=iter; m; m=++iter)
		if (m->Name()==name) return m->Type();
	
	return super?super->GetMember(name):0;
}


Value& Struct::operator =(const Struct& s)
{
	Iterator<Attrib*> iter(members);
	Value* v;
	
	for (Attrib* a=iter; a; a=++iter)
		if ((v = s.GetMember(a->Name())))
			a->Val() = *v;
	
	return *this;	
}


Value& Struct::operator =(const Value& v)
{
	if (v.IsStruct())
		*this = (Struct&)v;
	
	return *this;
}
