#include "xbefore.h"
#include <Xm/Text.h>
#include <Xm/ScrolledW.h>
#include <Xm/TextF.h>
#include "xafter.h"
#include <iostream.h>

#include "text.h"
#include "scope.h"
#include "float.h"
#include "int.h"

VALUE_NOTIFY("text",Text)


Text::Text(NetClass* nc, bool hidden):
	 Value(nc,hidden),
	 scroll(0),text(0),
	 rows(1),
	 p(0)
{
}

		 
Text::Text(const Text& t): 
	Value(t),
	scroll(0),text(0),
	rows(t.rows),
	p(0)
{
	*this = t.p;
}


Text::Text(const char *s): 
	scroll(0),text(0),
	rows(1),
	p(0)
{
	*this = s;
}


istream& Text::Parse(istream& is)
{
	Scope sb(is,0,1);	
	while (is && sb.Get())
		if (!(is>>rows)) cerr<<"integer (number of rows) expected!"<<endl;

	return is;
}


istream& Text::Input(istream& is)
{
	char c,*b,reading = 0; long len = 1; // Nullbyte reservieren

	if (is>>c && c=='"')
	{
		streampos start = is.tellg();
_again:	while (is.get(c))
			switch (c)
			{
			case '"':
				if (reading)
				{
					*b = 0;
					if (text) XmTextSetString(text,p);
					return is;
				}
				else
				{
					if (p) delete [] p;
					p = b = new char[len];
					reading++;
					is.seekg(start);
					goto _again;
				}
			case '\\':
				is.get(c);
				if (c=='n') c = '\n';
				// fall thru
			default:
				if (reading) *b++ = c; else len++;
			}
	}
	else
		cerr<<"\" expected!"<<endl;
	is.clear(ios::badbit);
	return is;
}


ostream& Text::Output(ostream& os) const
{
	os<<'"';
	for (char *c=p; c && *c; c++)
		switch (*c)
		{
		case '\r':
			break;
		case '\n':
			os<<"\\n";
			break;
		case '"':
		case '\\':
			os<<'\\';
			// fall thru
		default:
			os<<*c;
		}
	return os<<'"';
}


Text Text::Left(int i)
{
  if (i>=Length()) return *this;
  if (i<0) return (const char *) 0;
 
  char* c = new char[i+1];
  strncpy(c, p, i);
  c[i]=0;

  Text t;
  t.p = c;
  return t;
}


Widget Text::CreateWidget(Widget parent)
{
	if (rows>1)
	{
		scroll = XtVaCreateManagedWidget("Text",
			xmScrolledWindowWidgetClass,parent,NULL);
	
		text = XtVaCreateManagedWidget("Scrolled",
			xmTextWidgetClass,scroll,
			XmNeditMode,XmMULTI_LINE_EDIT,
			XmNrows,rows,NULL);
		CatchDestroy(&scroll,&text);
		XmTextSetString(text,Sure(p));
		return scroll;
	}
	else
	{
		text = XtVaCreateManagedWidget((rows==1 ? "Text" : "Ident"),
			xmTextFieldWidgetClass,parent,NULL);
		CatchDestroy(&text);
		XmTextSetString(text,Sure(p));
		return text;
	}
}
  

bool Text::EvalWidget()
{
	if (text)
		*this = XmTextGetString(text);
	else
		cerr<<"Evaluating nonexisting Widget!"<<endl;
	return true;
}


Value& Text::operator =(const char* s)
{
	if (p) delete[] p;
	if (s) strcpy(p = new char [strlen(s)+1],s); else p = 0;
	if (text) XmTextSetString(text,Sure(p));
	return *this;
}

Value& Text::operator =(const double d)
{
	return *this = (char*)Float(d);
}

Value& Text::operator =(const int i)
{
	return *this = (char*)Int(i);
}


bool Text::operator ==(const char* v) const
{
	return p && v && strcmp(p,v)==0;
}

bool Text::operator !=(const char* v) const
{
	return !(*this==v);
}


bool Text::operator >(const char* v) const
{
	return p && v && strcmp(p,v)>0;
}


bool Text::operator <(const char* v) const
{
	return p && v && strcmp(p,v)<0;
}


Text Text::operator -() const
{
	char* s1 = Sure(p),* local = new char[strlen(s1)+1]+strlen(s1);
	*local = 0; while (*s1) *--local = *s1++;
	Text t;
	t.p = local;
	return t;
}

Text Text::operator -(const char* v) const
{
	char* p1;
	if (p && v && (p1 = strstr(p,v)))
	{
		char* local = new char[strlen(p)-strlen(v)+1];
		strncpy(local,p,p1-p); strcpy(local+(p1-p),p1+strlen(v));
		Text t;
		t.p = local;
		return t;
	}
	else return Text(*this);
}


Text Text::operator +(const char* v) const
{
	if (v)
	{
		char* s1 = Sure(p),* local = new char[strlen(s1)+strlen(v)+1];
		strcpy(local,s1); strcat(local,v);
		Text t;
		t.p = local;
		return t;
	}
	else return Text(*this);
}


Text& Text::operator +=(const char* s)
{
	if (s)
	{
		char* np = new char[(p?strlen(p):0)+strlen(s)+1];
		*np = 0;
		if (p)
		{
			strcat(np,p);
			delete[] p;
		}
		
		strcat(p = np,s);
		if (text)
			XmTextSetString(text,p);
	}
	
	return *this;
}


Text& Text::operator +=(const char c)
{
	char s[] = {c,0};
	return *this += s;
}


Text& Text::operator +=(const double d)
{
        static char buf[256];
        ostrstream os(buf,256);
        os<<d<<'\0';
        if (buf)
        {
                char* np = new char[(p?strlen(p):0)+strlen(buf)+1];
                *np = 0;
                if (p)
                {
                        strcat(np,p);
                        delete[] p;
                }

                strcat(p = np,buf);
                if (text)
                        XmTextSetString(text,p);
        }

        return *this;
}


Text Text::operator *(const char* v) const
{
	if (v)
	{
		char* s1 = Sure(p); int cnt = Maxi(0,atoi(v));
		char* local = new char[cnt*strlen(s1)+1];
		*local = 0;
		while (cnt--)
			strcat(local,s1);
		Text t;
		t.p = local;
		return t;
	}
	else return Text(*this);
}

Value* Text::Eq(Value& v)
{
	return new Float(*this==v?1:0);
}

Value* Text::Gt(Value& v)
{
	return new Float(*this>v?1:0);
}

Value* Text::Lt(Value& v)
{
	return new Float(*this<v?1:0); 
} 

Value* Text::Not()
{
  return new Float(strlen(Sure(p)));
}
