#include "xbefore.h"
#include <Xm/Xm.h>
#include <Xm/AtomMgr.h>
#include <Xm/Protocols.h>
#include "xafter.h"

#include "wrapper.h"
#include "main.h"

// g++ Workaround for templated member function ptrs...
#define SCB(name) void (T::*name)(Widget w, U udata, XtPointer calld)
#define TCB(name) bool (T::*name)(U udata)
#define ICB(name) bool (T::*name)(int fd,U udata)


template<class T, class U> void Wrapper<T, U>::_Callback(Widget w, XtPointer cld, XtPointer calld)
{
	CBInfo& cbi = *(CBInfo*)cld;
	(cbi.wr.instance->*cbi.cb)(w,cbi.udata,calld);
}


template<class T, class U> void Wrapper<T, U>::_TimeOut(XtPointer cld, XtIntervalId*)
{
	CBInfo& cbi = *(CBInfo*)cld;
	if ((cbi.wr.instance->*cbi.tcb)(cbi.udata))
		cbi.tid = XtAppAddTimeOut(app,cbi.interval,_TimeOut,XtPointer(&cbi));
	else
		delete &cbi;
}

template<class T, class U> void Wrapper<T, U>::_Input(XtPointer cld,int*,XtInputId*)
{
	CBInfo& cbi = *(CBInfo*)cld;
	if (!(cbi.wr.instance->*cbi.icb)(cbi.fd,cbi.udata))
	{
		XtRemoveInput(cbi.iid);
		delete &cbi;
	}
}


template<class T, class U> void Wrapper<T, U>::_Destroy(Widget, XtPointer cld, XtPointer)
{
	CBInfo* cbi = (CBInfo*)cld;
	cbi->wr.RemoveCB(WrapperID(cbi));
}


template<class T, class U> WrapperID Wrapper<T, U>::AddCB(Widget w, 
	String cbname,SCB(callback),U udata)
{
	CBInfo* cbi = new CBInfo(*this,w,cbname,callback,udata,true);
	
	XtAddCallback(w,cbname,_Callback,XtPointer(cbi));
	XtAddCallback(w,XmNdestroyCallback,_Destroy,XtPointer(cbi));
	
	return cbi;
}


template<class T, class U> WrapperID Wrapper<T, U>::TimerCB(
	unsigned long interval, TCB(callback), U udata)
{
	CBInfo* cbi = new CBInfo(*this,interval,callback,udata);
	cbi->tid = XtAppAddTimeOut(app,interval,_TimeOut,XtPointer(cbi));
	
	return cbi;
}


template<class T, class U> WrapperID Wrapper<T, U>::InputCB(
	int fd, XtInputMask cond, ICB(callback), U udata)
{
	CBInfo* cbi = new CBInfo(*this,fd,callback,udata);
	cbi->iid = XtAppAddInput(app,fd,XtPointer(cond),_Input,XtPointer(cbi));
	
	return cbi;
}


template<class T, class U> WrapperID Wrapper<T, U>::WinMgrCB(Widget shell, 
	String atomname, SCB(callback), U udata)
{
	CBInfo* cbi = new CBInfo(*this,shell,atomname,callback,udata,false);
	Atom atom = XmInternAtom(XtDisplay(shell),atomname,False);
	
	XmAddWMProtocolCallback(shell,atom,_Callback,XtPointer(cbi));	
	XtAddCallback(shell,XmNdestroyCallback,_Destroy,XtPointer(cbi));
	
	return cbi;
}


template<class T, class U> void Wrapper<T, U>::RemoveCB(WrapperID id)
{
	CBInfo* cbi = (CBInfo*)id;
	
	switch (cbi->type)
	{
	case CBInfo::Standard:
		XtRemoveCallback(cbi->w,XmNdestroyCallback,_Destroy,XtPointer(cbi));
		XtRemoveCallback(cbi->w,cbi->cbn,_Callback,XtPointer(cbi));
		break;
	
	case CBInfo::WinMgr:
	{
		XtRemoveCallback(cbi->w,XmNdestroyCallback,_Destroy,XtPointer(cbi));
		Atom atom = XmInternAtom(XtDisplay(cbi->w),cbi->cbn,False);
		XmRemoveWMProtocolCallback(cbi->w,atom,_Callback,XtPointer(cbi));
		break;
	}
	
	case CBInfo::Timer:
		XtRemoveTimeOut(cbi->tid);
		break;
	
	case CBInfo::Input:
		XtRemoveInput(cbi->iid);
		break;
	}
		
	delete cbi;
}


// for g++
#ifndef NoExplicitInstanciation

class Int; template class Wrapper<Int,bool>;
class Net; template class Wrapper<Net,bool>;
class List; template class Wrapper<List,int>;
class Code; template class Wrapper<Code,int>;
class Ident; template class Wrapper<Ident,int>;
class Editor; template class Wrapper<Editor,void*>;
class Struct; template class Wrapper<Struct,int>;
class NetClass; template class Wrapper<NetClass,int>;
class LogWindow; template class Wrapper<LogWindow,bool>;
class SLogWindow; template class Wrapper<SLogWindow,bool>;
class Tokengame; template class Wrapper<Tokengame,bool>;
class NetImporter; template class Wrapper<NetImporter,bool>;

#endif
