//---------------------------------------------------------------------
// Importe
//---------------------------------------------------------------------

extern "C" {
#include <math.h>
#include <string.h>
#include <time.h>
#include "netio/netio.h"
}

#include <iostream.h>

#include "module.h"
#include "main.h"
#include "dialog.h"
#include "filesel.h"
#include "editor.h"
#include "net.h"
#include "netcl.h"
#include "node.h"
#include "connect.h"
#include "list.h"
#include "text.h"
#include "tool.h"
#include "int.h"
#include "ident.h"

#ifndef NoExplicitInstanciation
#define NoExplicitInstanciation
#include "dialog.cc"
#include "filesel.cc"
#include "wrapper.cc"
FSELINSTANCIATE(TimeNETLink,Editor*)
DIAINSTANCIATE(TimeNETLink,bool)
#endif

//---------------------------------------------------------------------
// Definitionen
//---------------------------------------------------------------------

extern char* NoFilename;
#define ENDING  ".net"
#define ENDING2 ".TN"
// Fuer Load & Save

//---------------------------------------------------------------------
// Klassendeclarationen
//---------------------------------------------------------------------

class TimeNETLink: public Module
{
  enum { INPUTARC = 0,OUTPUTARC,INHIBITOR };
  // alle moeglichen Arc-Typen

  bool EDSPNen;

  bool CheckNet();
  // Testet das Netz, ob es TimeNET tauglich ist
  
  Dialog<TimeNETLink,bool> error;
  // Dialogbox fuer Fehlermeldungen
  
  FileSelector<TimeNETLink,Editor*> fsel;
  // Fuer das Load & Save Menu
  void AddCB(Editor*, char* filename, bool ok);
  // CallBack zum addieren von .net oder .TN Dateien
  
  void SaveAsCB(Editor*, char* filename, bool ok);
  // CallBack zum speichern von .net oder .TN Dateien
  void SaveAsCB2(Editor*, char* filename, bool ok);
  // CallBack zum exportieren von .net oder .TN Dateien
  void FselCallback(Editor*,char* file,bool ok);
  // CallBack zum Laden von .net oder .TN Dateien
  
  NodeClass* ncplace,*ncexpt,*ncdett,*ncgent,*ncimmt,*ncmark,*ncdelay;
  ConnectorClass* ccarc,*ccinh;
  // benoetigte Knoten & Kantenklassen fuer alle Netzelemente
  
  Int arccnt;
  // Kantenzaehler
  
  Net* LoadTN(char* file);
  // Laedt eine .TN-Datei
  void SaveTN(Editor* ed, char* file);
  // Speichert eine .TN-Datei
  
  ostream& WriteTN(ostream& os);
  // schreibt das Netz im .TN-Format
  ostream& WriteArc(ostream& os,Connector* conn,int type);
  ostream& WriteInch(ostream& os,Position pos);
  ostream& WriteDelay(ostream& os,Position pos);
  ostream& WriteMD(ostream& os,char* keyword,char* type,char* attr);
  // Ausgabestroeme
  
  Position InchToPix(Coord& coord) 
    {
      return Position(coord.x*135,coord.y*135);
    }
  Position PixToPix(Coord& coord) 
    {
      return Position(coord.x*1.35,coord.y*1.35);
    }
  void CreateArcs(Node* trans,Page* prime,int type,Arc* arc,int num_arcs);
  // Hilfsfunktionen
  
//---------------------------------------------------------------------
	
public:

  TimeNETLink(Net& n): Module(n,"TimeNET-Link"),error(this),fsel(this) {}

  bool Init() {return true;};
  bool Open(Editor& ed, Page&) 
    {
      if (net.Prime()->Class()->Name()=="EDSPN")
	{
	  ed.SetDimension(3000,1500); 
	  EDSPNen= true;
	}
      else
	EDSPNen= false;
      
      return net.Prime()->Class()->Name()=="EDSPN";
    };
  bool Change(Editor& ed ,Page&, Page& page) {return Open(ed, page);};

  bool Menu(Editor&, Page&, Ident&, bool before);
  void Call(Editor& ed, Page&);
};

MODULE_NOTIFY(TimeNETLink)

//---------------------------------------------------------------------
// Klassendefinitionen
//---------------------------------------------------------------------
	
void TimeNETLink::Call(Editor& ed, Page&)
  {
    if (!EDSPNen || !CheckNet())
      {
	Text ErrorMsg;
	ErrorMsg+= "There are Errors in the Net!\n";
	ErrorMsg+= "Please check the Error Window.";
	error.OpenMsg("TNLinkMessage","Error",ed.Shell(),XmDIALOG_ERROR,
		      ErrorMsg, false);
	return;
      }
    // Testet die Syntax des Netzes

    char* models=getenv("MODELDIR"), *tnethome = getenv("TNETHOME");
    // Liest die Environmentvariablen aus

    if (!models || !tnethome)
      {
	Text ErrorMsg;
	ErrorMsg+= "Please set Environment-variables\n$TNETHOME and $MODELDIR!";
	error.OpenMsg("TNLinkMessage","Error",ed.Shell(),XmDIALOG_ERROR,
		      ErrorMsg, false);
	return;
      }
    // Diese Variablen muessen gesetzt sein

    char tmp[256],fname[256],call[256],*base;
    if ((base = strrchr(tmpnam(tmp),'/'))==NULL)
      return;
    // erzeugt einen temporaeren Dateinamen, base: /FileName

    strcat(strcat(strcpy(fname,models),base),ENDING2);
    SaveTN(&ed, fname);
    // speichert das .TN-File unter einer temp-Datei ab
    strcat(strcat(strcat(strcpy(call,tnethome),
			 "/Interface/bin/TimeNET "),base+1),"&");
    system(call);
    // TimeNET starten mit der temporaeren Datei
  }

//---------------------------------------------------------------------
	
void TimeNETLink::SaveTN(Editor* ed, char* fname)
  {
    ofstream os(fname);

    ed->Mouse(true);
    // Mauspfeil sperren

    if (os)
      {	
	WriteTN(os);
	os.close();

	net.modified= false;
	// fuer die Sicherheitsabfrage

	net.Update();
      }
    // .TN-File schreiben

    ed->Mouse(false);
    // Mauspfeil freigeben
  }

//---------------------------------------------------------------------	

void TimeNETLink::FselCallback(Editor* ed, char* file, bool ok)
{
  if (ok)
    {
      ed->Mouse(true);
      // Mauspfeil sperren

      Net* net = LoadTN(file);
      // versuche als .TN Datei zu laden

      if (!net) net = Net::Load(file);
      // geht nicht, dann als .net-Datei

      if (net) 
	{
	  net->OpenEditor(0,ed->DpyName());
	
	  net->SetFileName(file);
	  // Filenamen sichern
	  ed->Update();
      	}
      else 
	{
	  error.OpenMsg("TNLinkMessage","Error",ed->Shell(),XmDIALOG_ERROR,
			"unable to load the desired net.",
			false);
	}

      ed->Mouse(false);
      // Mauspfeil freigeben
    }	
}
		
//---------------------------------------------------------------------	

void TimeNETLink::AddCB(Editor* ed, char* file, bool ok)
{
  if (ok)
    {
      Page& curr_page = **(net.pages[ed]);

      Net* addnet = LoadTN(file);
      // versuche als .TN Datei zu laden

      if (!addnet) addnet = Net::Load(file);
      // geht nicht, dann als .net-Datei

      if (!addnet) return;

      if (curr_page.Class()==addnet->Prime()->Class())
	{
	  net.UndoCopy();
	  curr_page.Deselect();
	  curr_page.PageCopy(*addnet->Prime());
	}
		
      delete addnet;

      ed->SetCurrentPath(file);
      ed->Update();

       net.RefreshModules();
    }
}

//---------------------------------------------------------------------	

void TimeNETLink::SaveAsCB(Editor* ed, char* filename, bool ok)
{
  if (ok)
    {
      char tmp[256],*tmp2,*tmp3;
      sprintf(tmp,"%s",filename);

      tmp2= tmp;
      tmp3= strtok(tmp,"/");
      while (tmp2) {tmp3= tmp2;tmp2= strtok(NULL,"/");};

      strtok(tmp3, ".");
      char* end= strtok(NULL, ".");
      if (end)
	end= filename+strlen(filename)-strlen(end)-1;
      // Die Endung in filename lesen

      Ident file(filename);
      if (!end) 
	{
	  end= (char*)file+strlen(file);
	  file += ENDING;
	}
      // keine Endung, dann Standartmaessig .net anfuegen
      
      net.SetFileName(file);
      // Filenamen sichern

      if (!strcmp(end, ENDING2))
	{
	  if (!EDSPNen)
	    {
	      Text ErrorMsg;
	      ErrorMsg+= " NetClass must be EDSPN to be saved as .TN";
	      error.OpenMsg("TNLinkMessage","Error",ed->Shell(),XmDIALOG_ERROR,
			    ErrorMsg, false);
	      return;
	    }
	  else
	    SaveTN(ed, file);
	}
      else if (!strcmp(end, ENDING))
	net.Save();
      else
	{
	  Text ErrorMsg;
	  ErrorMsg+= "Ending of the filename must be either "ENDING2" or "ENDING;
	  error.OpenMsg("TNLinkMessage","Error",ed->Shell(),XmDIALOG_ERROR,
			ErrorMsg, false);
	  return;
	}
      // Netz speichern als .TN oder .net
      
      for (int i= strlen(filename)-1; i>=0; i--)
	if (filename[i]=='/') {filename[i]='\0';break;}
      ed->SetCurrentPath(filename);
      // Pfad dauerhaft aendern
      ed->Update();
    }
}

//---------------------------------------------------------------------	

void TimeNETLink::SaveAsCB2(Editor* ed, char* filename, bool ok)
{
  if (ok)
    {
      char tmp[256],*tmp2,*tmp3;
      sprintf(tmp,"%s",filename);

      tmp2= tmp;
      tmp3= strtok(tmp,"/");
      while (tmp2) {tmp3= tmp2;tmp2= strtok(NULL,"/");};

      strtok(tmp3, ".");
      char* end= strtok(NULL, ".");
      if (end)
	end= filename+strlen(filename)-strlen(end)-1;
      // Die Endung in filename lesen

      Ident file(filename);
      if (!end) 
	{
	  end= (char*)file+strlen(file);
	  file += ENDING2;
	}
      // keine Endung, dann Standartmaessig .net anfuegen
      
      net.SetFileName(file);
      // Filenamen sichern

      for (int i= strlen(filename)-1; i>=0; i--)
	if (filename[i]=='/') {filename[i]='\0';break;}
      ed->SetCurrentPath(filename);
      // Pfad dauerhaft aendern
      ed->Update();

      if (!strcmp(end, ENDING2))
	{
	  if (!EDSPNen)
	    {
	      Text ErrorMsg;
	      ErrorMsg+= "NetClass must be EDSPN to be saved as .TN";
	      error.OpenMsg("TNLinkMessage","Error",ed->Shell(),XmDIALOG_ERROR,
			    ErrorMsg, false);
	      return;
	    }
	  else
	    SaveTN(ed, file);
	}
      else if (!strcmp(end, ENDING))
	net.Save();
      else
	{
	  Text ErrorMsg;
	  ErrorMsg+= "Ending of the filename must be either "ENDING2" or "ENDING;
	  error.OpenMsg("TNLinkMessage","Error",ed->Shell(),XmDIALOG_ERROR,
			ErrorMsg, false);
	  return;
	}
      // Netz speichern als .TN oder .net
      
    }
}

//---------------------------------------------------------------------	

bool TimeNETLink::Menu(Editor& ed, Page&, Ident& name, bool before)
{
  if (EDSPNen && name=="Load" && before)
    {
      char* path = ed.CurrentPath();
      // aktuellen Pfad lesen
   
      fsel.Open(ed.Shell(),"select Net-file to load.",FselCallback,&ed,
		path ? path : agnes_dir,"*" ENDING);
      // Lademenue oeffnen mit Vorgabe: .net

      return true;
      // keine weitere Behandlung fuer Load
    }
  if (EDSPNen && name=="Add" && before)
    {
      char* path = ed.CurrentPath();
      // aktuellen Pfad lesen
   
      fsel.Open(ed.Shell(),"select Net-file to add (.net or .TN).",AddCB,&ed,
		path ? path : agnes_dir,"*" ENDING);
      // Lademenue oeffnen mit Vorgabe: .net

      return true;
      // keine weitere Behandlung fuer Load
    }
  else if (name=="Import" && before)
    {
      char* path = ed.CurrentPath();
      // aktuellen Pfad lesen
      
      fsel.Open(ed.Shell(),"select TN-file to import",FselCallback,&ed,
		path ? path : agnes_dir,"*" ENDING2);
      // Importmenue oeffnen mit Vorgabe: .TN
      return true;
      // keine weitere Behandlung fuer Load
    }
  else if (EDSPNen && name=="Save" && (net.FileName() != NoFilename) && before)
    {
      char tmp[256],*tmp2,*tmp3;
      sprintf(tmp,"%s",net.FileName());

      tmp2= tmp;
      tmp3= strtok(tmp,"/");
      while (tmp2) {tmp3= tmp2;tmp2= strtok(NULL,"/");};

      strtok(tmp3, ".");
      char* end= strtok(NULL, ".");
      if (end)
	end= net.FileName()+strlen(net.FileName())-strlen(end)-1;
      // Die Endung in filename lesen
      
      Ident file(net.FileName());
      if (!end) 
	{
	  end= (char*)file+strlen(file);
	  file += ENDING;
	}
      // keine Endung, dann Standartmaessig .net anfuegen
      
      if (!strcmp(end, ENDING))
	net.Save();
      else if (!strcmp(end, ENDING2))
	SaveTN(&ed, file);

      return true;
      // keine weitere Behandlung fuer Save
    }
  else if (EDSPNen && (name=="SaveAs" || name=="Save") && before)
    {
      Text title = Text("Save Net ")+net.FullName()+" as...";
      char* path = ed.CurrentPath();
     
      fsel.Open(ed.Shell(),title,SaveAsCB,&ed,
		path ? path : agnes_dir,"*" ENDING);
      
      return true;
      // keine weitere Behandlung fuer SaveAs
    }
  else if (EDSPNen && name=="Export" && before)
    {
      Text title = Text("Export Net ")+net.FullName()+" as...";
      char* path = ed.CurrentPath();
      fsel.Open(ed.Shell(),title,SaveAsCB2,&ed,
		path ? path : agnes_dir,"*" ENDING2);
      
      return true;
      // keine weitere Behandlung fuer Export
    }
  else
    return false;
  // Der Menuepunkt geht dieses Modul nichts an!
}

//---------------------------------------------------------------------
	
static int NetioError(char*,int) { return 0; }

Net* TimeNETLink::LoadTN(char* file)
{
//	double start = (double)clock()/CLOCKS_PER_SEC;
	NetDescr* netdesc = load_net(file,NetioError);
	
	if (netdesc)
	{
		NetClass* nc = NetClass::GetNetClass("EDSPN");
		Text name = Text(file)-".TN";
		Net* net = new Net(nc,name);
		Page* prime = net->Prime();
		int i;
		
		arccnt = 1;
		ncplace = nc->GetNodeClass("Place");
		ncexpt = nc->GetNodeClass("ExpTrans");
		ncdett = nc->GetNodeClass("DetTrans");
		ncgent = nc->GetNodeClass("GenTrans");
		ncimmt = nc->GetNodeClass("ImmTrans");
		ncmark = nc->GetNodeClass("MarkPar");
		ncdelay = nc->GetNodeClass("DelayPar");
		ccarc = nc->GetConnectorClass("Arc");
		ccinh = nc->GetConnectorClass("Inhibitor");

		for (i=0; i<netdesc->num_markpars; i++)
		{
			Markpar* m = &netdesc->markpar[i];												 
			Node* mpar = new Node(ncmark,prime,m->name,InchToPix(m->pos));
	 		*mpar->Attr("value") = m->value;
		}
	
		for (i=0; i<netdesc->num_delaypars; i++)
		{												 
			Delaypar* d = &netdesc->delaypar[i];
			Node* dpar = new Node(ncdelay,prime,d->name,InchToPix(d->pos));
			if (!d->is_value) 
				*dpar->Attr("delay") = d->delay.md_delay;
			else
				*dpar->Attr("delay") = d->delay.value;
		}
				
		for (i=0; i<netdesc->num_places; i++) 
		{												 
			Place* p = &netdesc->place[i];
			Node* place = new Node(ncplace,prime,p->name,InchToPix(p->pos));
			
			*(Node**)&p->pos = place; // Iiiih...
			
			place->SetRelative(0,
				(InchToPix(p->name_pos)-place->Mid())/place->Scale()*1.4);
			
			if (p->is_value)
				*place->Attr("marking") = p->marking.value;
			else
				*place->Attr("marking") = p->marking.markpar->name;
		}
		
		for (i=0;i<netdesc->num_trans;i++)
		{
			Transition* t = &netdesc->trans[i];
			Node* trans;
			double angle = (t->orient==ORIENT_45?45.0:
				t->orient==ORIENT_90?90.0:
				t->orient==ORIENT_M45?-45.0:0.0)/180.0*M_PI;
			
			switch (t->type)
			{
			case IMM:
				trans = new Node(ncimmt,prime,t->name,InchToPix(t->pos),angle);
				
				switch (t->delay_type)
				{
				case DELAY:
					*trans->Attr("weight") = t->delay.value;
					break;
				case DELAYPAR:
					*trans->Attr("weight") = t->delay.delay_par->name;
					break;
				case MARKDEP:
					*trans->Attr("weight") = t->delay.md_delay;
					break;
				default:
					break;
				}

				*trans->Attr("priority") = t->priority;
				if (t->md_enable)
					*trans->Attr("md_enable") = t->md_enable;
				break;
			
			case DET:
				trans = new Node(ncdett,prime,t->name,InchToPix(t->pos),angle);
				goto _exp_delay;
				break;
			
			case EXP:
				trans = new Node(ncexpt,prime,t->name,InchToPix(t->pos),angle);
			
			*trans->Attr("inf_serv") = t->inf_server?true:false;
				_exp_delay:
				switch (t->delay_type)
				{
				case DELAY:
					*trans->Attr("delay") = t->delay.value;
					break;
				case DELAYPAR:
					*trans->Attr("delay") = t->delay.delay_par->name;
					break;
				case MARKDEP:
					*trans->Attr("delay") = t->delay.md_delay;
					break;
				default:
					break;
				}
				break;
			
			  case GEN:
				trans = new Node(ncgent,prime,t->name,InchToPix(t->pos),angle);
				*trans->Attr("pmfunc") = t->delay.md_delay;
				break;
			}			
			
			trans->SetRelative(0,
				(InchToPix(t->name_pos)-trans->Mid())/trans->Scale()*1.4);
			trans->SetRelative(1,PixToPix(t->delay_pos)/trans->Scale()*1.4);
			CreateArcs(trans,prime,INPUTARC,t->in_arc,t->num_inarcs);
			CreateArcs(trans,prime,OUTPUTARC,t->out_arc,t->num_outarcs);
			CreateArcs(trans,prime,INHIBITOR,t->inh_arc,t->num_inharcs);
		}				
			
		List& rewards = *(List*)net->Attr("rewards");
		for (i=0; i<netdesc->num_rewards; i++)	 
		{
			Text* text = new Text(Text(netdesc->reward[i].name)+" =");
			while (text->Length()<15) *text += ' ';
				
			*text	+= netdesc->reward[i].def;
			rewards.Append(text);
		}		
	
		free_net(netdesc);
		return net;
	}
	
	return 0;
}


void TimeNETLink::CreateArcs(Node* trans, Page* prime, int type, 
	Arc* arc, int num_arcs)
{
	ConnectorClass* cc = type==INHIBITOR?ccinh:ccarc;
				
	for (int j=0; j<num_arcs; j++)
	{
		Arc* a = &arc[j];
		Node* place = *(Node**)&a->place->pos; // Iiiihhh...
		Ident name = cc->NamePrefix()+ ++arccnt;
		Connector* arc = new Connector(cc,prime,name);

		if (type==OUTPUTARC)
			arc->SetConnections(trans,place);
		else
			arc->SetConnections(place,trans);
			
		if (a->is_value)
		        *arc->Attr("multi") = a->multipl.value;
		else 
		        *arc->Attr("multi") = a->multipl.md_mult;
		
		for (int k=0; k<a->num_pts; k++)
			arc->InsertPos(type==OUTPUTARC?k+1:1,InchToPix(a->point[k]));
			
		arc->Rearrange();
	}
}


#define NET_TYPE   "NET_TYPE:"
#define DESCRIPTION "DESCRIPTION:"
#define PLACES     "PLACES:"
#define TRANSITIONS "TRANSITIONS:"
#define DELAYS     "DELAY_PARAMETERS:"
#define MARKINGS   "MARKING_PARAMETERS:"
#define RESULTS   "REWARD_MEASURES:"

#define MARKPAR    "MARKPAR"
#define PLACE      "PLACE"
#define DELAYPARAM   "DELAYPAR"
#define TRANSITION "TRANSITION"

#define DELAYPAR_DEF "DELAYPAR_DEF"
#define EXP_DELAY    "EXP_DELAY"
#define DET_DELAY    "DET_DELAY"
#define GEN_DELAY    "GEN_DELAY"
#define WEIGHT       "WEIGHT"
#define FUNCTION     "FUNCTION"
#define ARC        "CARDINALITY"
#define MEASURE    "MEASURE"

#define IsMD(def) (def && strchr((def),';')!=0)



ostream& TimeNETLink::WriteInch(ostream& os,Position pos)
{
	double x = int(pos.x/1.35+.5)/100.0,y = int(pos.y/1.35+.5)/100.0;
	return os<<x<<' '<<y;
}

ostream& TimeNETLink::WriteDelay(ostream& os,Position pos)
{
	double x = int(pos.x*74.0+.5)/100.0,y = int(pos.y*74.0+.5)/100.0;
	return os<<x<<' '<<y;
}

ostream& TimeNETLink::WriteMD(ostream& os,char* keyword,char* type,char* attr)
{
	bool flag = false;
	Iterator<Node*> niter = net.Prime()->Nodes();
	for (Node* node=niter; node; node=++niter)
		if (node->Class()->Name()==type)
		{
			char* delay = *node->Attr(attr);
			if (IsMD(delay))
			{
				os<<keyword<<' '<<node->Name()<<' '<<delay<<endl;
				flag = true;
			}
		}
	
	if (flag) os<<endl;
	return os;
}


ostream& TimeNETLink::WriteArc(ostream& os,Connector* conn,int type)
{
	char* multi = type==INHIBITOR?"1":(char*)*conn->Attr("multi");
	if (IsMD(multi)) multi = "<MD>";
	Node* place = conn->GetInput();
	
	if (place->Class()->Name()!="Place") place = conn->GetOutput();
	Iterator<Position*> piter = conn->Positions();
	int cnt = int(piter)-2;
	Position* pos;

	os<<multi<<' '<<place->Name()<<' '<<cnt<<endl;

	if (type==OUTPUTARC)
		for (pos=piter; cnt-- && (pos=++piter);)
			WriteInch(os,*pos)<<' '<<endl;
	else
		for (pos=-piter; cnt-- && (pos=--piter);)
			WriteInch(os,*pos)<<' '<<endl;
	
	return os;
}

bool TimeNETLink::CheckNet()
{
	Iterator<Node*> niter = net.Prime()->Nodes();
	Node* node;
	int errcnt = 0;
	List& rewards = *ListP(net.Attr("rewards"));

	cerr<<"Checking Net for simple Errors first:"<<endl;
	
	// Check Marking-Parameters
	for (node=niter; node; node=++niter)
		if (node->Class()->Name()=="MarkPar")
		{
			char* mark = *node->Attr("value");
			if (!mark || !*mark)
			{
				errcnt++;
				cerr<<"Missing marking definition in marking parameter '"<<node->Name()<<"'!"<<endl;
			}
		}
	
	// Check Places
	for (node=niter; node; node=++niter)
		if (node->Class()->Name()=="Place")
		{
			char* mark = *node->Attr("marking");
			if (!mark || !*mark)
			{
				errcnt++;
				cerr<<"Missing marking in Place '"<<node->Name()<<"'!"<<endl;
			}
			else
			{
				istrstream ms(mark);
				int ma;
				if (ms>>ma) continue;
				istrstream mm(mark);
				Ident mp;
				if (mm>>mp)
				{
					Iterator<Node*> miter = net.Prime()->Nodes();
					Node* mpar;
					for (mpar=miter; mpar; mpar=++miter)
						if (mpar->Class()->Name()=="MarkPar" && mpar->Name()==mp)
							break;
					if (!mpar)
					{
						errcnt++;
						cerr<<"Unknown marking parameter '"<<mp<<"' used in Place '"<<node->Name()<<"'!"<<endl;
					}
				}
				else
				{
					errcnt++;
					cerr<<"Invalid marking '"<<mark<<"' in Place '"<<node->Name()<<"'!"<<endl;
				}
			}
	}

	// Check Transitions
	for (node=niter; node; node=++niter)
	{
		Ident type = node->Class()->Name();
		
		if (strstr(type,"Trans"))
		{
			if (type=="GenTrans")
			{
				char* pmfunc = *node->Attr("pmfunc");
				if (!pmfunc || !*pmfunc)
				{
					errcnt++;
					cerr<<"Missing pmf-def '"<<pmfunc<<"' in Transition '"<<node->Name()<<"'!"<<endl;
				}
			}
			else if (type=="ImmTrans")
			{
				char* weight = *node->Attr("weight");
				if (!weight || !*weight)
				{
					errcnt++;
					cerr<<"Missing weight definition in Transition '"<<node->Name()<<"'!"<<endl;
				}
				else
				{
					istrstream iw(weight); int i;
					istrstream idw(weight); Ident id;
					if (!IsMD(weight) && !(iw>>i) && !(idw>>id))
					{
						errcnt++;
						cerr<<"Invalid weight '"<<weight<<"' in Transition '"<<node->Name()<<"'!"<<endl;
					}
				}
			}
			else
			{
				char* delay = (char*)*node->Attr("delay");
				if (!delay || !*delay)
				{
					errcnt++;
					cerr<<"Missing delay definition in Transition '"<<node->Name()<<"'!"<<endl;
				}
				else 
				{
					istrstream id(delay); istrstream ii(delay);
					int d; Ident i;
					if (!IsMD(delay) && !(id>>d) && !(ii>i))
					{
						errcnt++;
						cerr<<"Invalid delay '"<<delay<<"' in Transition '"<<node->Name()<<"'!"<<endl;
					}
				}
			}
		}
	}
	
	// Check Delay-Parameters
	for (node=niter; node; node=++niter)
		if (node->Class()->Name()=="DelayPar")
		{
			char* delay = *node->Attr("delay");
			if (!delay || !*delay)
			{
				errcnt++;
				cerr<<"Missing delay definition in delay parameter '"<<node->Name()<<"'!"<<endl;
			}
		}

	// Check Rewards
	Iterator<Value*> riter = rewards.Entries();
	for (Value* r=riter; r; r=++riter)
	{
		char* rew = (char*)*TextP(r);
		if (!rew || !*rew)
		{
			errcnt++;
			cerr<<"Empty reward definition number "<<riter.Index()+1<<"!"<<endl;
		}
		else
		{
			istrstream is(rew);
			Ident rew_name;
			char c;

			if (is>>rew_name && is>>c && c=='=' && is>>c)
				continue;
			else
			{
				errcnt++;
				cerr<<"Invalid reward definition '"<<(char*)*TextP(r)<<"'!"<<endl;
			}
		}
	}

	cerr<<endl<<errcnt<<" error(s) found. "<<(errcnt?"Aborting Transfer.":"Transfering Net.")<<endl<<endl;
	return !errcnt;
}
	
ostream& TimeNETLink::WriteTN(ostream& os)
{		
	char* arcstr[] = {"INPARCS","OUTPARCS","INHARCS"};
//	char* firestr[] = {"RS","RA","RE"};
	char* ntstr[] = {"GSPN","DSPN","EDSPN","FSPN"};
	enum {NT_GSPN = 0,NT_DSPN,NT_EDSPN,NT_FSPN};
	int nettype = NT_GSPN;
	int nplaces = 0,ntrans = 0,nmarkings = 0,ndelays = 0;
	Iterator<Node*> niter = net.Prime()->Nodes();
	Node* node;
	bool flag;
	List& rewards = *ListP(net.Attr("rewards"));

	for (node=niter; node; node=++niter)
	{
		Ident type = node->Class()->Name();
		if (type=="Place") nplaces++;
//		else if (type=="FluidPlace") nettype = Maxi(nettype,NT_FSPN);
		else if (type=="MarkPar") nmarkings++;
		else if (type=="DelayPar") ndelays++;
		else if (strstr(type,"Trans"))
		{
			ntrans++;
			if (type=="DetTrans") nettype = Maxi(nettype,NT_DSPN);
			if (type=="GenTrans") nettype = Maxi(nettype,NT_EDSPN);
		}
	}

	char* fname = strrchr(net.FileName(),'/');
	if (fname) fname++; else fname = net.FileName();
	os<<"-- FILE "<<fname
		<<" CONTAINING STRUCTURAL DESCRIPTION OF A NET"<<endl<<endl;
	os<<NET_TYPE<<"\t\t\t"<<ntstr[nettype]<<endl;
	os<<DESCRIPTION<<"\t\t\t?"<<endl;
	os<<PLACES<<"\t\t\t"<<nplaces<<endl;
	os<<TRANSITIONS<<"\t\t"<<ntrans<<endl;
	os<<DELAYS<<"\t"<<ndelays<<endl;
	os<<MARKINGS<<"\t"<<nmarkings<<endl;
	os<<RESULTS<<"\t"<<rewards.Length()<<endl;
 
	os<<endl<<"-- LIST OF MARKING PARAMETERS (NAME, VALUE, (X,Y)-POSITION):"
		<<endl<<endl;
	for (node=niter; node; node=++niter)
		if (node->Class()->Name()=="MarkPar")
		{
			os<<MARKPAR<<' '<<node->Name()<<' '<<int(*node->Attr("value"))<<' ';
			WriteInch(os,node->Mid())<<endl;
		}

	if (nmarkings) os<<endl;

	os<<"-- LIST OF PLACES (NAME, MARKING, (X,Y)-POSITION (PLACE & TAG)):"
		<<endl<<endl;

	for (node=niter; node; node=++niter)
		if (node->Class()->Name()=="Place")
		{
			os<<PLACE<<' '<<node->Name()<<' '<<(char*)(*node->Attr("marking"))<<' ';
			WriteInch(os,node->Mid())<<' ';
			WriteInch(os,node->Mid()+(node->GetRelative(0)/1.4*node->Scale()));
			os<<endl;
		}
	if (nplaces) os<<endl;

	os<<"-- LIST OF DELAY PARAMETERS (NAME, VALUE, (X,Y)-POSITION):"<<endl<<endl;

	for (node=niter; node; node=++niter)
		if (node->Class()->Name()=="DelayPar")
		{
			char* delay = *node->Attr("delay");
			if (IsMD(delay)) delay = "<MD>";
			os<<DELAYPARAM<<' '<<node->Name()<<' '<<delay<<' ';
			WriteInch(os,node->Mid())<<endl;
		}
		if (ndelays) os<<endl;

	os<<"-- LIST OF TRANSITIONS"<<endl;
	os<<"-- \t(NAME, DELAY, ENABLING DEPENDENCE, KIND, FIRING POLICY, PRIORITY,"
		<<endl;
	os<<"-- \tORIENTATION, (X,Y)-POSITION (TRANSITION, TAG & DELAY), ARCS)"
		<<endl;
		
	for (node=niter; node; node=++niter)
	{
		Ident type = node->Class()->Name();
		
		if (strstr(type,"Trans"))
		{
			Connector* conn;
			char* ttype,* server = "SS";
			int prior = 0;
			double ang = node->Angle()*180.0/M_PI;
			if (ang<0.0) ang += 180.0;
			int orient = 
				ang<22.5?ORIENT_0:ang<67.5?ORIENT_45:ang<112.5?ORIENT_90:ORIENT_M45;

			os<<endl<<TRANSITION<<' '<<node->Name()<<' ';

			if (type=="GenTrans")
			{
				os<<"<MD>";
				ttype = "GEN";
			}
			else if (type=="ImmTrans")
			{
				char* weight = *node->Attr("weight");
				if (IsMD(weight)) weight = "<MD>";
				os<<weight;
				prior = int(*node->Attr("priority"));
				ttype = "IM";
			}
			else
			{
				char* delay = (char*)*node->Attr("delay");
				if (IsMD(delay)) delay = "<MD>";
				os<<delay;
				if (type=="ExpTrans")
				{
					if (bool(*node->Attr("inf_serv"))) server = "IS";
					ttype = "EXP";
				}
				else ttype = "DET";
			}
			
			os<<' '<<server<<' '<<ttype<<' '<<(char*)*node->Attr("fire_pol")
				<<' '<<prior<<' '<<orient<<' ';
			
			WriteInch(os,node->Mid())<<' ';
			WriteInch(os,node->Mid()+(node->GetRelative(0)/1.4*node->Scale()))<<' ';
			WriteDelay(os,node->GetRelative(1)/1.4*node->Scale());
			os<<endl;
			
			Iterator<Connector*> iiter = node->Inputs();
			Iterator<Connector*> oiter = node->Outputs();
			int n[3] = {0,0,0};

			for (conn=iiter; conn; conn=++iiter)
					n[conn->Class()->Name()=="Inhibitor"?INHIBITOR:INPUTARC]++;

			for (conn=oiter; conn; conn=++oiter)
					n[OUTPUTARC]++;

			os<<arcstr[INPUTARC]<<' '<<n[INPUTARC]<<endl;
			for (conn=iiter; conn; conn=++iiter)
				if (conn->Class()->Name()!="Inhibitor")
					WriteArc(os,conn,INPUTARC);

			os<<arcstr[OUTPUTARC]<<' '<<n[OUTPUTARC]<<endl;
			for (conn=oiter; conn; conn=++oiter)
				WriteArc(os,conn,OUTPUTARC);
			
			os<<arcstr[INHIBITOR]<<' '<<n[INHIBITOR]<<endl;
			for (conn=iiter; conn; conn=++iiter)
				if (conn->Class()->Name()=="Inhibitor")
					WriteArc(os,conn,INHIBITOR);
		}
	}
	if (ntrans) os<<endl;

	os<<"-- DEFINITION OF PARAMETERS:"<<endl<<endl;
	WriteMD(os,DELAYPAR_DEF,"DelayPar","delay");

	os<<"-- MARKING DEPENDENT FIRING DELAYS FOR EXP. TRANSITIONS:"<<endl<<endl;
	WriteMD(os,EXP_DELAY,"ExpTrans","delay");

	os<<"-- MARKING DEPENDENT FIRING DELAYS FOR DET. TRANSITIONS:"<<endl<<endl;
	WriteMD(os,DET_DELAY,"DetTrans","delay");

	os<<"-- PROBABILITY MASS FUNCTION DEFINITIONS FOR GEN. TRANSITIONS:"
		<<endl<<endl;
	for (flag=false,node=niter; node; node=++niter)
		if (node->Class()->Name()=="GenTrans")
		{
			os<<GEN_DELAY<<' '<<node->Name()<<' '<<(char*)*node->Attr("pmfunc")
				<<endl;
			flag = true;
		}
	if (flag) os<<endl;

	os<<"-- MARKING DEPENDENT WEIGHTS FOR IMMEDIATE TRANSITIONS:"<<endl<<endl;
	WriteMD(os,WEIGHT,"ImmTrans","weight");

	os<<"-- ENABLING FUNCTIONS FOR IMMEDIATE TRANSITIONS:"<<endl<<endl;
	for (flag=false,node=niter; node; node=++niter)
		if (node->Class()->Name()=="ImmTrans")
		{
			char* func = (char*)*node->Attr("md_enable");
			if (func && *func)
				os<<FUNCTION<<' '<<node->Name()<<' '<<func<<endl;
		}
	if (flag) os<<endl;

	os<<"-- MARKING DEPENDENT ARC CARDINALITIES:"<<endl<<endl;
	Iterator<Connector*> citer = net.Prime()->Connectors();
	Connector* conn;
	for (flag=false,conn=citer; conn; conn=++citer)
		if (conn->Class()->Name()=="Arc")
		{
			char* multi = *conn->Attr("multi");
			if (IsMD(multi))
			{
				Node* place = conn->GetInput(),* trans = conn->GetOutput();
				bool inp = place->Class()->Name()=="Place";
				
				if (!inp)	{
					Node* tmp = place; place = trans; trans = tmp;
				}
				os<<ARC<<' '<<(inp?"INPUTARC":"OUTPUTARC")
					<<' '<<trans->Name()<<' '<<place->Name()<<endl<<multi<<endl;
				flag = true;
			}
		}
	if (flag) os<<endl;

	os<<"-- REWARD MEASURES:"<<endl<<endl;
	Iterator<Value*> riter = rewards.Entries();
	for (Value* r=riter; r; r=++riter)
	{
		istrstream is((char*)*TextP(r));
		Ident rew_name;
		char c;

		if (is>>rew_name && is>>c && c=='=' && is>>c)
		{
			os<<MEASURE<<" "<<rew_name<<endl<<c;
			while (is.get(c)) os<<c;
			if (c!=';') os<<';';
			os<<endl<<endl;
		}
	}
	if (rewards.Length()) os<<endl;

	return os<<endl<<"-- END OF SPECIFICATION FILE"<<endl;
}
