/*
 *	Function:     bsearch2()
 *
 * Parameter:    char *key          - Suchschluessel
 *               char *base         - Adresse des ersten Elements (Wurzel)
 *               unsigned nel       - Anzahl der Listenelemente
 *               unsigned elemsize  - Groesse der Listenelemente
 *               int (*compar)()    - Elementvergleichsfunktion
 *               char **pposition   - Adresse auf die Stelle an der das ge-
 *                                    suchte Element steht oder stehen sollte
 *                                    (called by referenz!!)
 *
 * Return Value: char *found
 *               Zeiger auf das gefundene Element oder falls nicht vorhanden
 *               (char*)0. Im zweiten Fall kann pposition verwendet werden um
 *               das Element an der Stelle einzufuegen.
 *
 * Side Effects: Falls pposition != (char*)0 wird darin die gefundene Stelle
 *               in der Liste abgelegt
 *
 *	Description:  Binaere Suche auf Arrays von Elementen beliebiger Groesse. Man
 *               kann auch die Libraryfunktion bsearch() verwenden. Moechte man
 *               aber ein nicht vorhandenes Element einfuegen, so liefert das
 *               Orginal bsearch() keinen Zeiger auf die passende Einfuege-
 *               stelle. (hier pposition)
 * Programmierer: Thomas Kuhlmann, TU-Berlin (public domain)
 */
char *bsearch2(key,base,nel,elemsize,compar,pposition)
char *key, *base, **pposition;
unsigned nel, elemsize;
int (*compar)();
{
	register int low, high, mid, cond;
	char *rc;

	if (!nel) {
		if (pposition) *pposition = base;
		return (char *)0;
	}
#ifdef POINTERCHECK
	if (!key || !base || !compar || !elemsize) {
		if (pposition) *pposition = (char *)0;
		return (char *)0;
	}
#endif
	/* pre: nel >= 1, elemsize >= 1 */
	low  = 0;
	high = nel-1;
	/* high >= low, d.h. Schleife wird mindestens einmal durchlaufen! */
	while (low <= high) {
		mid = low + (high-low)/2;
		if ((cond=(*compar)(key,rc=base+(mid*elemsize)))<0)
			high = mid -1;
		else if (cond>0)
			low  = mid +1;
		else {
			if (pposition) *pposition = rc;
			return rc;
		}
	}
	/*pre: element not found, low > high, rc is valid, cond=(*compar)(key,rc) */
	if (cond>0)
		rc += elemsize; /* key gehoert ueber mid (an die Stelle low) */
	if (pposition) *pposition = rc;
	return (char *)0;
}
