/******************************************************************************
* Programm  : gensol                                                          *
* Komponente: compare.c                                                       *
* Aufgabe   : Vergleichsfunktionen auf diversen Objekten (zur totalen Ordnung)*
* Projektleiter: Reinhard German, TU-Berlin                                   *
* Programmierer: Thomas Kuhlmann, TU-Berlin                                   *
* Datum:         Mon Jun 21 18:18:40 MET DST 1993                             *
******************************************************************************/
#include "expand.h"

int TimedTraCompare(pa,pb)
timed_tra_t *pa, *pb;
{
	return (int)(pa->timed_no)-(int)(pb->timed_no);
}

/*---------------------------------------------------------------------------
 *	Die Liste von Verweisen auf neue Markierungen ist erstens nach der Nummer
 * der zugeordneten alten Markierung (old_no), und zweitens nach der Phasen-
 * liste geordnet. Durch mehrfache Ordnungskriteria wird erreicht, dass die
 *	Liste weiterhin in totaler Ordnung ist und binaere Suche anwendbar bleibt.
 */
int PNewMarkCompare(pa,pb)
new_mark_t **pa, **pb;
{
	int diff;

	if (diff=((int)((*pa)->old_no)) - ((int)((*pb)->old_no))) return diff;
	/*
	 *	beide neue Markierungen sind der selben alten zugeordnet, sie koennen
	 * aber trotzdem einen anderen Zustand beschreiben, wenn die Schaltphasen
	 * in denen sich die deterministischen Transitionen befinden sich unter-
	 * scheiden.
	 */
	return PHASE_COMPARE((*pa)->phase,(*pb)->phase);
}

/*---------------------------------------------------------------------------
 * Zum Loesen des linearen Gleichungssystems mit dnspiv, muessen die Marken
 * nach der Anzahl ihrer Vorgaenger sortiert sein:
 */
int PNewMarkPredsComp(pa, pb)
new_mark_t **pa, **pb;
{
	return ((int) ((*pa)->no_preds - (*pb)->no_preds));
}

/*---------------------------------------------------------------------------
 * Zum Loesen des linearen Gleichungssystems mit sor, muessen die Marken
 * nach der neuen Markierungsnummer sortiert sein:
 */
int PNewMarkNewNoComp(ppa,ppb)
new_mark_t **ppa, **ppb;
{
	return ((int)((*ppa)->new_no)) - ((int)((*ppb)->new_no));
}

/*---------------------------------------------------------------------------
 *	Die Vorgaengerlisten sind nach der from_mark sortiert, bei gleicher
 *	from_mark werden die zwei Kanten zu einer zusammengefasst (Summe der Raten).
 */
int PredCompare(pa,pb)
pred_t *pa, *pb;
{
	return ((int)(pa->from_mark))-((int)(pb->from_mark));
}

/*---------------------------------------------------------------------------
 *	Vergleichsfunktion fuer Suchschluessel new_to_mark und exponentiellen Pfad
 *    (wird fuer bsearch() benoetigt)
 */
int ExpPathCompare( pa, pb)
exp_path_t    *pa, *pb;
{
	return ((int)(pa->to_mark)) - ((int)(pb->to_mark));
}

/*---------------------------------------------------------------------------
 * Um festzustellen ob eine deterministische Transition schon in der Liste der
 * det. Schaltpfade steht, muss diese Liste binaer durchsucht werden. Als
 * Elementvergleichfunktion dient DetTraCompare() (nach der Nummer sortiert)
 */
int DetTraCompare(pa,pb)
det_tran_t *pa, *pb;
{
	return ((int)(pa->tra_no)) - ((int)(pb->tra_no));
}

/*---------------------------------------------------------------------------
 * Totale Ordnungsfunktion auf der Liste von deterministischen Schaltpfaden je
 * alter Markierung. Es darf keine zwei Kanten mit selbem Ziel und selber
 * Quelle und selber det. Transition im graph geben:
 */
int DetPathCompare(pa,pb)
det_path_t *pa, *pb;
{
	return ((int)(pa->to_mark)) - ((int)(pb->to_mark));
}
