/******************************************************************************
* Programm  : gensol                                                          *
* Komponente: expand.h                                                        *
* Parameter :                                                                 *
* Rueckgabe :                                                                 *
* Aufgabe   : Typendefinitionen und #defines fuer gensol                      *
* Projektleiter: Reinhard German, TU-Berlin                                   *
* Programmierer: Thomas Kuhlmann, TU-Berlin                                   *
* Datum:         Mon Jun 21 18:18:40 MET DST 1993                             *
*                Wed Jun 30 19:58:00 MET DST 1993 MAXDNSPIVMEM                *
******************************************************************************/
#ifndef __expand_h__

#define __expand_h__

#include <stdio.h>
#include <math.h>
#include <memory.h>

#define MAXDNSPIVMEM 120
#define EPSILON  ((double)(1e-15))
#define COMMENT  '#'
#define NEWLINE  '\n'
#define MAXIMUM(a,b) ((a>b)?a:b)
#define NULLPTR  ((char*)0)

#ifndef FALSE 
#define FALSE     0
#endif

#ifndef TRUE
#define TRUE     !FALSE
#endif

#define FOUND     TRUE
#define NEW      !FOUND

#define NACHITERATION 0
#define ITERATION !NACHITERATION

#define PHASE_COMPARE(phase_a,phase_b) memcmp((char*)phase_a,(char*)phase_b,(int)(sizeof(phase_t)*no_det_trans))
#define PHASELIST_COPY(phase_a,phase_b) memcpy((char*)phase_a,(char*)phase_b,(int)(sizeof(phase_t)*no_det_trans))

/******************************************************************************
* Verwendet man fuer die Phasenzaehler keine bytes (unsigned chars) sondern   *
* Objekte andere Groesse, so muss man selber eine Initialiserungsfunktion     *
* 'phase_init()' schreiben. Dann ohne -DFASTPHASES uebersetzen!!!!            *
******************************************************************************/
typedef unsigned char phase_t;
#ifdef FASTPHASES
#define INIT_PHASES(phase_list) (void)memset((char*)phase_list,(int)no_phases,(int)no_det_trans)
#else
#define INIT_PHASES(phase_list) phase_init(phase_list)
#endif

typedef int bool_t;
typedef unsigned old_mark_no_t;
/******************************************************************************
* Da fuer Listen auf unterschiedlichen Objekten, die selbe 'insert()'-Funktion*
* verwendet wird, diese aber nur mit Listengroessen vom Typ 'unsigned'        *
* arbeitet sind die folgenden Typen alle gleich unsigned. Hat man mehrere     *
* insert-Funktionen, so kann man fuer diese Objekte ggf. Speicher sparen.     *
******************************************************************************/
typedef unsigned list_index_t;
#ifdef MIXED_INSERT_FUNCTION
/*
 *	unterschiedliche Listengroessen:
 */
typedef unsigned new_mark_no_t;       /* hier reicht vielleicht short */
typedef unsigned short det_path_no_t; /* hier reicht vielleicht sogar char */
typedef unsigned short det_tra_no_t;  /* hier reicht vielleicht sogar char */
typedef unsigned short det_cnt_t;     /* hier reicht vielleicht sogar char */
typedef unsigned short timed_tra_no_t;
typedef unsigned short exp_path_no_t;
typedef unsigned short pred_cnt_t;    /* predecessors per Node */
#else
/*
 *	gleiche Listengroessen:
 */
typedef list_index_t new_mark_no_t;
typedef list_index_t det_path_no_t;
typedef list_index_t det_tra_no_t;
typedef list_index_t det_cnt_t;
typedef list_index_t timed_tra_no_t;
typedef list_index_t exp_path_no_t;
typedef list_index_t pred_cnt_t;      /* predecessors per Node */
#endif

typedef enum { ERLANG, COX } phase_typ;
typedef enum { RESAMPLING=0, ENABLING=1, AGE=2 } policy_t;
typedef enum
{ 
  AUTOMATIC=0, 
  ITERATIV=1,
  STABIL=2,
  DIRECT=3, 
} solution_t;
typedef enum { EXPONENTIAL, DETERMINISTIC } timed_tra_type_t;
typedef enum { SINGLE, INFINTE } server_type_t;

typedef struct {
	timed_tra_no_t   timed_no;
	timed_tra_type_t type;
	server_type_t    server_type;
} timed_tra_t;

/*
 *	deterministic fireing path to old marking 'to_mark', with propablity 'prob'
 */
typedef struct {
	double        rate;    /* hier ist prob mit eingerechnet */
	old_mark_no_t to_mark;
} det_path_t;

/*
 *	fireable deterministic transition 'tra_no', with rate and 'no_paths'
 * possible fireing paths
 */
typedef struct {
	/*
	 *	rate ist im Erlangenfall no_phasen/delay,
	 *          im Coxfall      (1+(no_phasen-1)*a)/delay.
	 */
	double        rate;  /* Schaltrate einer aus der Kette von Exponentiellen */
	double        prob_sum; /* Summe der Wk`s aller Schaltpfade (1.0)         */
	det_path_t   *det_path; /* Liste von moeglichen deterministischen Pfaden  */
	det_path_no_t no_paths; /* entstanden durch feuern zeitloser Transitionen */
	det_tra_no_t  tra_no;   /* eindeutige Transitionsnummer */
} det_tran_t;

/*
 *	exponential fireing path to old marking 'to_mark', with 'rate'.
 */
typedef struct {
	double        rate;
	old_mark_no_t to_mark;
} exp_path_t;

/*
 *	marking of old reduced reachability graph with 'no_exp_paths' exponential
 * fireing paths and also 'no_det_trans' fireable deterministic transitions
 */
typedef struct {
	det_tran_t   *det_tran;
	exp_path_t   *exp_path;
	det_cnt_t     no_det_trans;
	exp_path_no_t no_exp_paths;
	double        time_spent;   /* steady state solution for this marking */
} old_mark_t;

/*
 *	predecessor of new node (edge in ERG), weighted with 'rate'.
 */
typedef struct {
	double        rate;
	new_mark_no_t from_mark;
} pred_t;

/*
 *	marking of the new expanded reachability graph
 */
typedef struct new_mark_t {
	struct new_mark_t *next;		/* next new mark to compute predecessors  */
	phase_t           *phase;     /* list of phasecounters per det.trans    */
	pred_t            *pred;		/* list of predecessors in expanded graph */
	double             ratesum;   /* sum of rates to all successors         */
	new_mark_no_t      new_no;    /*              new marking number (id)   */
	old_mark_no_t      old_no;    /* subordinated old marking number (id)   */
	pred_cnt_t         no_preds;  /* size of predecessors list              */
} new_mark_t;

/*
 *	the following functions may be used by all modules of this program:
 */
extern char *bsearch2();
extern bool_t insert();
extern void initNewMarks();
#ifndef __utils_c__

/*
 *	these functions are used by all modules except the defining one itself:
 */
extern char *emalloc (/*      nelems, elemsize */);
extern char *ecalloc (/*      nelems, elemsize */);
extern char *erealloc(/* ptr, nelems, elemsize */);
extern void  eperror (/* char */);
extern int  *optequal(/* char *s1, *s2 */);
extern int   skip_comment_lines(/* FILE *stream */);
#endif
#ifndef __main_c__

/*
 *	global variables defined in main.c but used by all others:
 */
extern char         *progname;
extern phase_t       no_phases;
extern phase_typ     phase_type;
extern double        variation;
extern double        a;
extern double        eins_minus_a;
extern policy_t      policy;
extern det_tra_no_t  no_det_trans;
extern timed_tra_no_t no_timed_trans;
extern timed_tra_t  *timed_tran;
extern old_mark_no_t no_old_marks;
extern old_mark_t   *old_mark;
extern new_mark_no_t no_new_marks;
extern new_mark_t  **new_mark;
extern new_mark_t   *curr_new;
extern int           no_old_edges;
extern unsigned long no_new_edges;
extern bool_t verbose;
#endif
#endif
