/*************************************************************************
*                                                                        *
* KOMPONENTE  :   gauss_skal_pivot.c		   	                 *
* PROGRAMM    :	  solve_LGS.c	                                         *
*                                                                        *
* ZWECK	      :	  Gauss-Algorithmus mit skalierter Spalten-Pivotsuche    *
*		  zur L"osung sehr gro"ser d"unn besetzter Matrizen      *
*                                                                        *
* BEMERKUNG   :   meist stabil                                           *
* ====================================================================== *
* erstellt am    :   Juni 1994                                           *
* Programmierer  :   Anja Seibt                                          *
* Betreuer       :   Reinhard German                                     *
*************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "expand.h"

#include "struct_types.h"
#include "global.h"
#include "extern_fkt.h"


static  Piv_T  pivot;
static  EXACT;


/* -----------------------------------------------------------------------
   liefert Pivotelement, -Zeilenindex
----------------------------------------------------------------------- */
unsigned*  p_pivot_holen( spalte)
unsigned     spalte;     /* spalten */
{
  double    hfd, hfv, sum;
  unsigned  i, k;
  unsigned   *sp_el;
  int  found;


  if(NULL == (sp_el = (unsigned*) malloc ((Dm + 1) * sizeof(unsigned))))
    {
      perror("ERROR: malloc sp_el");
      exit(1);
    }
  pivot.wert   = 0.0;
  pivot.zz     = 0;
  hfd          = 0.0;	
  sp_el[0]     = 0;
  found = 0;


  for (i = spalte; i < Dm; i++)
    {
      if (spalte == Matrix[i].sp[0])
	{
	  sum = 0.0;
	  for (k = 0; k < Matrix[i].anz; k++)
	    {
	      sum += fabs(Matrix[i].aa[k]);
	    }
	  hfv = fabs( Matrix[i].aa[0] / sum);
	  if (hfd < hfv)
	    {
	      found = 1;
	      pivot.wert = Matrix[i].aa[0];
	      pivot.zz   = i;
	      hfd = hfv;
	    }
	  sp_el[0]++;
	  sp_el[sp_el[0]] = i;
	}
    }
  if (! found)
    {
	printf("\nERROR: Diagonalelement = 0!  in spalte %u\n", spalte);
	exit(1);
    }

  if( NULL == (sp_el = (unsigned*) realloc (sp_el, (sp_el[0] + 1) 
					            * sizeof(unsigned))))
    {
      perror("ERROR: p_pivot_el_zeile  --  realloc sp_el\n");
      exit(1);
    }

   return sp_el;

}  /* p_pivot_holen */



/* -----------------------------------------------------------------------
   berechnet eine Zeile neu, in Abh. von der Pivotzeile
----------------------------------------------------------------------- */
void  p_m_zeile_berechnen_skal( zeile)
unsigned      zeile; 
{
  double     faktor; 
  unsigned   zz_anz,
             piv_anz,
             anzahl, 
             zz_cnt, piv_cnt;
  int        zz_ende = 0;
  int        piv_ende = 0;
  Matrix_T    new;


  faktor = Matrix[zeile].aa[0] / pivot.wert;

  anzahl = Matrix[zeile].anz + Matrix[pivot.zz].anz - 2;
  if (anzahl > Dm)
    anzahl = Dm;
  if(NULL == (new.aa = (double*) malloc (anzahl * sizeof(double))))
    {
      perror("ERROR: p_m_zeile_berechnen_skal  --  malloc new.aa");
      exit(1);
    }
  if(NULL == (new.sp = (unsigned*) malloc (anzahl * sizeof(unsigned))))
    {
      perror("ERROR: p_m_zeile_berechnen_skal  -- malloc new.sp");
      exit(1);
    }
  new.anz = 0;

  zz_anz = Matrix[zeile].anz;
  piv_anz = Matrix[pivot.zz].anz;

  if (zz_anz < 2) 
    zz_ende = 1;
  else
    zz_cnt = 1;
  if (piv_anz < 2) 
    piv_ende = 1;
  else
    piv_cnt = 1;


  while ((! zz_ende) || (! piv_ende))
    {
      if ((Matrix[zeile].sp[zz_cnt] == Matrix[pivot.zz].sp[piv_cnt])
	  &&
	  (! zz_ende)
	  &&
	  (! piv_ende))
	{
	  new.aa[new.anz] = Matrix[zeile].aa[zz_cnt] - 
	                    (Matrix[pivot.zz].aa[piv_cnt] * faktor);

	  if (EXACT 
	      || 
	      ((! EXACT) && (fabs(new.aa[new.anz]) > Droptol)))
	    {
	      new.sp[new.anz] = Matrix[zeile].sp[zz_cnt];
	      new.anz++;
	    }
	  zz_cnt++;	
	  if (zz_cnt >= zz_anz)  zz_ende = 1;
	  piv_cnt++;
	  if (piv_cnt >= piv_anz)  piv_ende = 1;

	  continue;
	}

      if (piv_ende
	  ||
	  ((Matrix[zeile].sp[zz_cnt] < Matrix[pivot.zz].sp[piv_cnt])
	   &&
	   (! zz_ende)))
	{
	  new.aa[new.anz] = Matrix[zeile].aa[zz_cnt];
	  new.sp[new.anz] = Matrix[zeile].sp[zz_cnt];
	  new.anz++;
	  zz_cnt++;	
	  if (zz_cnt >= zz_anz)  zz_ende = 1;
 
	  continue;
	}

      if ((Matrix[zeile].sp[zz_cnt] > Matrix[pivot.zz].sp[piv_cnt])
	  ||
	  (zz_ende))
	{
	  new.aa[new.anz] = 0.0 - (Matrix[pivot.zz].aa[piv_cnt] * faktor);

	  if (EXACT 
	      || 
	      ((! EXACT) && (fabs(new.aa[new.anz]) > Droptol)))
	    {
	      new.sp[new.anz] = Matrix[pivot.zz].sp[piv_cnt];
	      new.anz++;
	    }
	  piv_cnt++;
	  if (piv_cnt >= piv_anz)  piv_ende = 1;

	  continue;
	}
      printf("ERROR: p_m_zeile_berechnen_skal  -->  seltsamer Fehler in der Matrix!\n");
      exit(1);
   }
  new.b = Matrix[zeile].b - (Matrix[pivot.zz].b * faktor);

  free((char*)Matrix[zeile].aa);
  free((char*)Matrix[zeile].sp);

  if(NULL == (new.aa = (double*) realloc (new.aa, new.anz 
					          * sizeof(double))))
    {
      perror("ERROR: p_m_zeile_berechnen_skal  --  realloc new.sp");
      exit(1);
    }
  if(NULL == (new.sp = (unsigned*) realloc (new.sp, new.anz 
					            * sizeof(unsigned))))
    {
      perror("ERROR: p_m_zeile_berechnen_skal  --  realloc new.sp");
      exit(1);
    }

  Matrix[zeile] = new;

}  /* p_m_zeile_berechnen_skal */



/* ------------------------------------------------------------ main -----
    Gauss-Algorithmus mit teilweiser (Spalten-) Pivotisierung
----------------------------------------------------------------------- */
double*  p_gauss_skal_Pivot( Maxdiagonal, exact)
double   Maxdiagonal;
bool_t   exact;
{
  double      rest_zeile;
  double      fehler;
  double     *XX;
  int         ka;
  int         dropped = 0;
  Matrix_T    hf;
  unsigned    j, i;
  unsigned   *piv_zz;       /* anzahl und angabe der Zeilen mit 
			             nicht-0-werten in der pivot-spalte */


  /*  printf("\nGauss-Algorithmus mit skalierter Spaltenpivotsuche\n\n");
  fflush(stdout); */

  /* ----------------------- P-Matrix aus Erreichbarkeitsgraphen holen und
     -------------------------- allozierten Speicherplatz ablegen ----- */
  p_m_matrix_besorgen( Maxdiagonal);
  p_m_sortieren();
  p_m_mit_1_fuellen();

  EXACT = exact;

  if (! EXACT)
    dropped = p_m_drop();

  printf("\nDirect solution ...\n");
  fflush(stdout);

  /* ------------- Pivotisierung, Dreiecksform des Gls herstellen ----- */
  ka = 0;
  if (Dm >= 100)
    {
      printf("\nNumber of computed states (stepsize = 100):\n");
      fflush(stdout);
    }

  for(j = 0; j < Dm; j++)
    {
      ka++;
       if (ka == 100)
	{
	  printf("_%u", j+1);
	  fflush(stdout);
	  ka = 0;
	}
	
      /* --- Zeile mit Pivotelement in Spalte j ab Zeile j suchen ----- */
      piv_zz = p_pivot_holen( j); 

      /* --------- alle noch zu bearbeitende Zeilen neu berechnen ----- */
      for(i = 1; i <= piv_zz[0]; i++)
	{
	  if (pivot.zz != piv_zz[i])
	    p_m_zeile_berechnen_skal( piv_zz[i]);
	}
      if (pivot.zz != j)
	{
	  hf = Matrix[j];
	  Matrix[j] = Matrix[pivot.zz];
	  Matrix[pivot.zz] = hf;
	}
    }

  /* ------------- rekursives Berechnen der Loesung aus neuem Gls ----- */
  /* ------------------------------- Speicher fuer Loesungsvektor ----- */
  if(NULL == (XX = (double*) malloc ((int)Dm * sizeof(double))))
    {
      perror("ERROR: gauss_spalten_pivot --  malloc: vector XX  \n");
      exit(1);
    }
  /* -------- X-Werte aus der sortierten Dreiecksmatrix berechnen ----- */
  for (i = Dm; i--;)
    {
      rest_zeile = 0.0;
      for (j = 1; j < Matrix[i].anz; j++)
	{
	  rest_zeile += Matrix[i].aa[j] * XX[Matrix[i].sp[j]];
	}
      XX[i] = (Matrix[i].b - rest_zeile) / Matrix[i].aa[0];
    }

  p_m_matrix_besorgen( Maxdiagonal);
  fehler = p_fehler( Matrix, XX);

  if (Dm >= 100)
    printf("\n");
  printf("\nSolution process terminated\n");
  printf("Droptolerance: %lg\n",Droptol);
  printf("Number of dropped elements: %d\n",dropped);
  printf("Achieved accuracy: %lg\n",fehler);
  fflush(stdout);
  
  /* printf("\n\nmaximaler Fehler:  %.6g\n\n", fehler); */
 
  if (fehler <= Prec)
    Nachiter = FALSE;

  OK = TRUE;

  return XX;
  
}  /* p_gauss_skal_Pivot */
