/*************************************************************************
*                                                                        *
* KOMPONENTE  :   gauss_voll_pivot.c				         *
* PROGRAMM    :   solve_LGS.c	                                         *
*                                                                        *
* ZWECK	      :   Gauss-Algorithmus mit vollstaendiger Pivotsuche        *
*		  zur L"osung sehr gro"ser d"unn besetzter Matrizen      *
*			                                                 *
* BEMERKUNG   :   meist stabil                                           *
* ====================================================================== *
* erstellt am    :   Juni 1994                                           *
* Programmierer  :   Anja Seibt                                          *
* Betreuer       :   Reinhard German                                     *
*************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "expand.h"

#include "struct_types.h"
#include "global.h"
#include "extern_fkt.h"


static  Piv_voll_T  pivot;



/* -----------------------------------------------------------------------
   liefert entsprechendes Element
----------------------------------------------------------------------- */
double  p_element( zz, spalte )
unsigned  zz;
unsigned  spalte;
{
  unsigned  i=0;


  if (spalte > Matrix[zz].sp[Matrix[zz].anz-1])
    return 0.0;

  if (spalte < Matrix[zz].sp[0])
    return 0.0;

  while ((spalte > (Matrix[zz].sp[i]))
         &&
         (i < Matrix[zz].anz))
    i++;

  if (spalte == Matrix[zz].sp[i])
    return Matrix[zz].aa[i];
  else
    return  0.0;

}  /* p_element */



/* -----------------------------------------------------------------------
   liefert Pivotelement, -Zeilenindex, Spaltenindex
----------------------------------------------------------------------- */
void  p_pivot_holen_voll( zz)
unsigned     zz;
{
  double    hfd, hfv;
  unsigned  i, j;


  pivot.wert   = 0.0;
  pivot.zz     = 0;
  hfd          = 0.0;	

  for (i = zz; i < Dm; i++)
    {
      for (j = 0; j < Matrix[i].anz; j++)
	{
	  hfv = fabs( Matrix[i].aa[j]);
	  if (hfd < hfv)
	    {
	      pivot.wert = Matrix[i].aa[j];
	      pivot.zz   = i;
	      pivot.sp   = Matrix[i].sp[j];
	      hfd = hfv;
	    }
	}
    }
}  /* p_pivot_holen_voll*/



/* -----------------------------------------------------------------------
   berechnet eine Zeile neu, in Abh. von der Pivotzeile
----------------------------------------------------------------------- */
void  p_m_zeile_berechnen_voll( zeile)
unsigned  zeile; 
{
  double     i_el, faktor; 
  unsigned   zz_anz,
             piv_anz,
             anzahl, 
             zz_cnt, piv_cnt;
  int        zz_ende = 0;
  int        piv_ende = 0;
  Matrix_T   new;


  i_el = p_element( zeile, pivot.sp);
  if (0.0 != i_el)
    {
      faktor = i_el / pivot.wert;

      anzahl = Matrix[zeile].anz + Matrix[pivot.zz].anz;
      if (anzahl > Dm)
	anzahl = Dm;
      if(NULL == (new.aa = (double*) malloc (anzahl * sizeof(double))))
	{
	  perror("ERROR: malloc new.aa");
	  exit(1);
	}
      if(NULL == (new.sp = (unsigned*) malloc (anzahl 
					       * sizeof(unsigned))))
	{
	  perror("ERROR: malloc new.sp");
	  exit(1);
	}
      new.anz = 0;

      zz_anz = Matrix[zeile].anz;
      piv_anz = Matrix[pivot.zz].anz;

      if (0 == zz_anz) 
	zz_ende = 1;
      else
	zz_cnt = 0;
      if (0 == piv_anz) 
	piv_ende = 1;
      else
	piv_cnt = 0;


      while ((! zz_ende) || (! piv_ende))
	{
	  if (pivot.sp == Matrix[zeile].sp[zz_cnt])
	    {
	      zz_cnt++;
	      if (zz_cnt >= zz_anz)  zz_ende = 1;
	      continue;
	    }
	  if (pivot.sp == Matrix[pivot.zz].sp[piv_cnt])
	    {
	      piv_cnt++;
	      if (piv_cnt >= piv_anz)  piv_ende = 1;
	      continue;
	    }

	  if ((Matrix[zeile].sp[zz_cnt] == Matrix[pivot.zz].sp[piv_cnt])
	      &&
	      (! zz_ende)
	      &&
	      (! piv_ende))
	    {
	      new.aa[new.anz] = Matrix[zeile].aa[zz_cnt] - 
	                        (Matrix[pivot.zz].aa[piv_cnt] * faktor);

	      new.sp[new.anz] = Matrix[pivot.zz].sp[piv_cnt];
	      new.anz++;

	      zz_cnt++;	
	      if (zz_cnt >= zz_anz)  zz_ende = 1;
	      piv_cnt++;
	      if (piv_cnt >= piv_anz)  piv_ende = 1;
	      
	      continue;
	    }

	  if (piv_ende
	      ||
	      ((Matrix[zeile].sp[zz_cnt] < Matrix[pivot.zz].sp[piv_cnt])
	       &&
	       (! zz_ende)))
	    {
	      new.aa[new.anz] = Matrix[zeile].aa[zz_cnt];
	      new.sp[new.anz] = Matrix[zeile].sp[zz_cnt];
	      new.anz++;

	      zz_cnt++;	
	      if (zz_cnt >= zz_anz)  zz_ende = 1;
	      
	      continue;
	    }

	  if ((Matrix[zeile].sp[zz_cnt] > Matrix[pivot.zz].sp[piv_cnt])
	      ||
	      (zz_ende))
	    {
	      new.aa[new.anz] = 0.0 - (Matrix[pivot.zz].aa[piv_cnt] 
				       * faktor);
	      
	      new.sp[new.anz] = Matrix[pivot.zz].sp[piv_cnt];
	      new.anz++;

	      piv_cnt++;
	      if (piv_cnt >= piv_anz)  piv_ende = 1;
	      
	      continue;
	    }

	  printf("ERROR: seltsamer Fehler in der Matrix! -- berechnen\n");
	  exit(1);
	}
      new.b = Matrix[zeile].b - (Matrix[pivot.zz].b * faktor);

      free((char*)Matrix[zeile].aa);
      free((char*)Matrix[zeile].sp);

      if (NULL == (new.aa = (double*) realloc (new.aa, new.anz 
					               * sizeof(double))))
	{
	  perror("ERROR: realloc new.aa");
	  exit(1);
	}
      if (NULL == (new.sp = (unsigned*) realloc (new.sp, new.anz 
						     * sizeof(unsigned))))
	{
	  perror("ERROR: realloc new.sp");
	  exit(1);
	}

      Matrix[zeile] = new;
    }
}  /* p_m_zeile_berechnen_voll */



/* ------------------------------------------------------------ main -----
    Gauss-Algorithmus mit teilweiser (Spalten-) Pivotisierung
----------------------------------------------------------------------- */
double*  p_gauss_voll_Pivot( Maxdiagonal)
double       Maxdiagonal;
{
  double     fehler;
  double    *XX;
  int        ka;
  Matrix_T   hf;
  unsigned   j, i;
  unsigned  *piv_zz;        /* anzahl und angabe der Zeilen mit 
                                     nicht-0-werten in der pivot-spalte */


  printf("\nGauss-Algorithmus mit vollstaendiger Pivotsuche\n\n");
  fflush(stdout);

  /* ----------------------- P-Matrix aus Erreichbarkeitsgraphen holen und
     -------------------------- allozierten Speicherplatz ablegen ----- */
  p_m_matrix_besorgen( Maxdiagonal);
  p_m_sortieren();
  p_m_mit_1_fuellen();

  /* ------------- Pivotisierung, Dreiecksform des Gls herstellen ----- */
  ka = 0;
  for(j = 0; j < Dm; j++)
    {
       if (ka == 100)
	{
	  printf("_%u", j);
	  fflush(stdout);
	  ka = 0;
	}
      ka++;
	
      /* --- Zeile mit Pivotelement in Spalte j ab Zeile j suchen ----- */
      /* ......................  0 bis (j-1) = schon pivotzeile gewesen */
      p_pivot_holen_voll( j);

      /* --------- alle noch zu bearbeitende Zeilen neu berechnen ----- */
      for(i = 0; i < Dm; i++)
	{
	  if (i != pivot.zz)
	    p_m_zeile_berechnen_voll( i);
	}
      if (pivot.zz != j)
	{
	  hf = Matrix[j];
	  Matrix[j] = Matrix[pivot.zz];
	  Matrix[pivot.zz] = hf;
	}
    }

  /* ------------- rekursives Berechnen der Loesung aus neuem Gls ----- */
  /* ------------------------------- Speicher fuer Loesungsvektor ----- */
  if(NULL == (XX = (double*) malloc ((int)Dm * sizeof(double))))
    {
      perror("gauss_voll_pivot --  malloc: vector XX  \n");
      exit(1);
    }
  /* -------- X-Werte aus der sortierten Dreiecksmatrix berechnen ----- */
  for (i = 0; i < Dm; i++)
    {
      XX[Matrix[i].sp[0]] = Matrix[i].b / Matrix[i].aa[0];
    }

  p_m_matrix_besorgen( Maxdiagonal);
  fehler = p_fehler( Matrix, XX);

  printf("\n\nmaximaler Fehler: %.10g\n\n", fehler); 
  fflush(stdout);

  if (fehler <= Prec)
    Nachiter = FALSE;

  OK = TRUE;

  return XX;
  
}  /* p_gauss_voll_Pivot */

