/*************************************************************************
*                                                                        *
* KOMPONENtE  :   hilf_fkt.c                                             *
* PROGRAMM    :   gensol                                                 *
*                                                                        *
* ZWECK       :   Ansammlung von Hilfsfunktionen                         *
* ====================================================================== *
* erstellt am   :   15. Juni 1994                                        *
* Programmierer :   Anja Seibt                                           *
* Betreuer      :   Reinhard German, TU-Berlin                           *
* ---------------------------------------------------------------------- *
* Funktionen:                                                            *
*             void  p_m_mit_1_fuellen();                                 *
*             void  p_m_matrix_besorgen();                               *
*             void  p_m_sortieren();                                     *
*             void  p_m_minus();                                         *
*             void  p_m_drop();                                          *
*             double p_fehler();                                         *
*************************************************************************/


#include <stdio.h> 
#include <math.h>  /* extern double fabs() */

#include "expand.h"
#include "struct_types.h"
#include "global.h"



/* -----------------------------------------------------------------------
   Einser Zeile setzen in Zeile 0
----------------------------------------------------------------------- */
void  p_m_mit_1_fuellen()
{
  unsigned  i;


  if(NULL == (Matrix[0].aa = (double*) realloc (Matrix[0].aa,
						  Dm * sizeof(double))))
    {
      perror("ERROR: realloc Matrix[0].aa");
      exit(1);
    }
   if(NULL == (Matrix[0].sp = (unsigned*) realloc (Matrix[0].sp, 
						  Dm * sizeof(unsigned))))
    {
      perror("ERROR: realloc Matrix[0].sp");
      exit(1);
    }

  for (i =  0; i < Dm; i++)
    {
      Matrix[0].aa[i] = 1.0;
      Matrix[0].sp[i] = i;
    }
  Matrix[0].anz = Dm;
  Matrix[0].b = 1.0;

}  /* p_m_mit_1_fuellen */



/* -----------------------------------------------------------------------
   Speicher besorgen und P-Matrix reinlegen
----------------------------------------------------------------------- */
void  p_m_matrix_besorgen( max)
double  max;
{
  double    hf;
  register unsigned  i, j, no;
  int        fd;
  new_mark_t *pnmark;
  pred_t     *ppred;

  if (NULL == (Matrix =(Matrix_T*) malloc (Dm * sizeof(Matrix_T))))
    {
      perror("ERROR: malloc Matrix");
      exit(1);
    }
  for (i = 0; i < Dm; i++)
    {
      pnmark = new_mark[i];
      ppred  = pnmark->pred;
      no = pnmark->no_preds +1;

      if (NULL == (Matrix[i].aa = (double*) malloc (no * sizeof(double))))
	{
	  perror("ERROR: malloc Matrix.aa");
	  exit(1);
	}
      if (NULL == (Matrix[i].sp = (unsigned*) malloc (no * 
						      sizeof(unsigned))))
	{
	  perror("ERROR: malloc Matrix.sp");
	  exit(1);
	}
      Matrix[i].anz = no;
      Matrix[i].b = 0.0;
    }

  for(i = 0; i < Dm; i++)
    {
      pnmark = new_mark[i];
      ppred  = pnmark->pred;
      /* ......... Spalte aus Zeile mit Inhalt fuellen (== transponieren),
       * ............................ dabei mit  maxdiagonal  Normieren */
      for( no = pnmark->no_preds; no--;)
        {
          hf = ppred[no].rate/max;
          Matrix[i].aa[no] = hf;
          Matrix[i].sp[no] = ppred[no].from_mark;
	}
      Matrix[i].sp[Matrix[i].anz-1] = i;
      Matrix[i].aa[Matrix[i].anz-1] = (double) -(pnmark->ratesum)/max;
    }
}  /* p_m_matrix_besorgen */



/* -----------------------------------------------------------------------
   Matrix sortieren fuer Gauss
----------------------------------------------------------------------- */
void  p_m_sortieren()
{
  unsigned  i, j;
  int       found;
  double    dd;
  unsigned  uu;


  for (i = 0; i < Dm; i++)
    {
      /* .............................................. bubble sort ... */
      found = 1;
      while (found)
	{
	  found = 0;
	  for (j = 0; j < Matrix[i].anz-1; j++)
	    {
	      if (Matrix[i].sp[j] > Matrix[i].sp[j+1])
		{
		  found = 1;
		  uu = Matrix[i].sp[j];
		  dd = Matrix[i].aa[j];
		  Matrix[i].sp[j] = Matrix[i].sp[j+1];
		  Matrix[i].aa[j] = Matrix[i].aa[j+1];
		  Matrix[i].sp[j+1] = uu;
		  Matrix[i].aa[j+1] = dd;
		}
	    }
	}
    }
}  /* p_m_sortieren */



/* -----------------------------------------------------------------------
   ersetzt rauszuwerfendes Element einer Zeile mit dem letzten des Vektors
   und elimeniert letzteres
----------------------------------------------------------------------- */
void  p_m_minus( zz, sp_ind )
unsigned     zz;
unsigned     sp_ind;
{
  unsigned   i;
  unsigned*  newu;
  double*    newd;


  if (sp_ind < Matrix[zz].anz)     /* Element gefunden */
    {
      Matrix[zz].anz--;

      if(NULL == (newu = (unsigned*) malloc (Matrix[zz].anz * 
					     sizeof(unsigned))))
	{
	  perror("ERROR: p_m_minus u");
	  exit(1);
	}
      if(NULL == (newd = (double*) malloc (Matrix[zz].anz * 
					   sizeof(double))))
	{  
	  perror("ERROR: p_m_minus d");
	  exit(1);
	}

      i = 0;
      while ((i != sp_ind) && (i <= Matrix[zz].anz))
	{
	  newd[i] = Matrix[zz].aa[i];
	  newu[i] = Matrix[zz].sp[i];
	  i++;
	}
      while (i < Matrix[zz].anz)
	{
	  newd[i] = Matrix[zz].aa[i+1];
	  newu[i] = Matrix[zz].sp[i+1];
	  i++;
	}
      
      free((char*)Matrix[zz].aa);
      free((char*)Matrix[zz].sp);
      
      Matrix[zz].aa = newd;
      Matrix[zz].sp = newu;
    }

  else
    printf("!!\n ERROR   gauss_?_pivot --  p_m_minus():\n%s",
           "nicht existentes Vektorelement zu loeschen versucht!\n!!\n");

}  /* p_m_minus */



/* -----------------------------------------------------------------------
   streicht Elemente aus der Matrix, die kleiner als Droptol sind
   und gibt die Anzahl der gestrichenen Elemente zur"uck
----------------------------------------------------------------------- */
int p_m_drop()
{
  unsigned  i, j;
  int dropped = 0;


  for(i = 0; i < Dm; i++)
      for(j = 0; j < Matrix[i].anz; j++)
          if (Droptol > fabs(Matrix[i].aa[j]))
	    {
              p_m_minus(i, j);
              ++dropped;
            }
  return(dropped);

}  /* p_m_drop */



/* -----------------------------------------------------------------------
   bestimmt den maximalen Fehler der Loesung
----------------------------------------------------------------------- */
double p_fehler( m, x)
Matrix_T* m;
double*   x;
{
  double    ff, hf;
  unsigned  i, j;


  ff = 0.0;
  for (i = 0; i < Dm; i++)
    {
      hf = 0.0;
      for (j = 0; j < m[i].anz; j++)
	{
	  hf += m[i].aa[j] * x[m[i].sp[j]];
	}
      hf = fabs(hf);
      if (ff < hf)
	{
	  ff = hf;
	}
    }
  return ff;

}  /* p_fehler */
