/******************************************************************************
* Programm  : gensol                                                          *
* Komponente: init.c (initNewMarks())                                         *
* Parameter : globale Variablen                                               *
* Rueckgabe :                                                                 *
* Aufgabe   : uebertragen der alten Markierungen in die Tabelle von Neuen,    *
*             wobei die neuen Markierungen mit dem Startwert initialisierte   *
*             Phasenzaehler erhalten.                                         *
* Projektleiter: Reinhard German, TU-Berlin                                   *
* Programmierer: Thomas Kuhlmann, TU-Berlin                                   *
* Datum:         Mon Jun 21 18:18:40 MET DST 1993                             *
******************************************************************************/
#include <memory.h>
#include "expand.h"
#undef EPSILON
#define EPSILON 1.0e-11

/*
 *	Fuer jede alte Markierung eine Neue erzeugen, gleichzeitig fuer jede
 * schaltfaehige deterministische Transition ueberpruefen, ob die Summe der
 * Wahrscheinlichkeiten 1.0 ergibt. (Aufsummierung bereits in read..())
 *
 * Es exisitiert eine geordnete Liste von Verweisen auf alle neuen Markierungen
 * (new_mark,no_new_marks). Ausserdem sind die neuen Markierungen zusaetzlich
 * als Liste verkettet. Die Wurzel dieser Kette ist curr_new. Die Kette ent-
 * haelt die noch nicht bearbeiteten neuen Markierungen. Alle Neuen werden der
 * Kette hinzugefuegt, alle Bearbeiteten werden der Kette entnommen.
 *
 * vordefiniert muessen sein:
 *		no_old_marks, no_det_trans,
 *		old_mark, old_mark[..].no_det_trans, old_mark[..].det_tran,
 *		old_mark[..].det_tran[..].prob_sum
 * definiert werden:
 *		new_mark, no_new_marks, curr_new
 */
void initNewMarks()
{
	det_cnt_t      det_tra;
	old_mark_no_t  mark;
	new_mark_t   *pmark;

	new_mark = (new_mark_t **)emalloc(no_old_marks, sizeof(new_mark_t *));
	curr_new = (new_mark_t *)0;
	for( mark=no_old_marks; mark--; ) {
		/*
		 *	zuerst Summe der Wahrscheinlichkeiten einer jeden det Trans pruefen:
		 */
		for( det_tra=old_mark[mark].no_det_trans; det_tra--; ) {
			if ( fabs( old_mark[mark].det_tran[det_tra].prob_sum - (double)1.0 )
					> EPSILON) {
				fprintf(stderr,"ERROR: sum of posibilities for a deterministic transition in marking %d is %.15g\n", mark,
					old_mark[mark].det_tran[det_tra].prob_sum );
				exit(1);
			}
		}
		old_mark[mark].time_spent = (double)0.0;

		new_mark[mark]  = pmark = (new_mark_t *)emalloc(1,sizeof(new_mark_t));
		pmark->new_no   = pmark->old_no   = mark;
		pmark->no_preds = (pred_cnt_t)0;
		pmark->pred     = (pred_t *)0;
		pmark->phase    = (phase_t*)emalloc(no_det_trans,sizeof(phase_t));
		/*
		 *	initialize Phases to start (phase=no_phases)
		 */
		INIT_PHASES(pmark->phase);
		pmark->next     = curr_new;
		curr_new        = pmark;
	}
	no_new_marks = no_old_marks;
}
