#include "expand.h"
#include "portab.h"

/*
 *	Fuegt ein Element elem (Zeiger auf Struktur) in eine Liste von Elementen ein
 * Alle Elemente haben einen Speicherbedarf von je elemsize. Das Element wird
 * falls noch nicht vorhanden, neu eingetragen und die Eintraege aus elem in
 * die Liste uebernommen (nicht jedoch rekursive Datenstrukturen). ppos zeigt
 * auf die Stelle an der das Element steht. Bei Vergroesserung der Liste ver-
 * aendern sich ggf. *pbase und *pnel (called by referenz).
 *	RETURNVALUE: NEW if something was inserted, FOUND if it is just in list
 * Programmierer: Thomas Kuhlmann, TU-Berlin (public domain)
 */
bool_t insert( elem, pbase, pnel, elemsize, compar, ppos)
char      *elem;
char     **pbase;
unsigned  *pnel;
unsigned   elemsize;
int      (*compar)();
char     **ppos;
{
	unsigned index;
#ifdef POINTERCHECK
	if (!elem || !pbase || !pnel || !elemsize || !compar || !ppos) {
		fprintf(stderr,"INTERNAL ERROR insert() called with wrong pointer\n");
		exit(1);
	}
#endif
	if (*pbase) {
		/*
		 *	Liste enthaelt schon Elemente, suche binaer
		 */
		if (bsearch2(elem,*pbase,*pnel,elemsize,compar,ppos)) return FOUND;
		/*
		 *	Element nicht in der Liste => an der Stelle *ppos einfuegen
		 * Vorsicht bei realloc() kann sich die Wurzel der Liste verschieben!
		 * Daher vorher Abstand merken, nachher wieder dazuaddieren:
		 */
		index  = *ppos - *pbase;  /* Abstand merken        */
		*pbase = erealloc( *pbase, ++(*pnel), elemsize );
		*ppos  = *pbase + index;  /* Abstand wieder gleich */
		/*
		 * Rest nach hinten verschieben:
		 */
		(void) memmove(*ppos+elemsize, *ppos,
				(int) ((unsigned)(*pnel-1) * elemsize - index) );
	} else {
		/*
		 *	Liste noch leer, erzeuge erstes Element:
		 */
		*ppos = *pbase = emalloc( ++(*pnel), elemsize );
	}
	/*
	 *	Werte der structur elem uebernehmen:
	 */
	memcpy(*ppos,elem,(int)elemsize);
	return NEW;
}
