/*************************************************************************
*                                                                        *
*  KOMPONENTE  :   iteration.c				                 *
*  PROGRAMM    :   gensol	                                         *
*                                                                        *
*  ZWECK       :   Gesamtschrittverfahren mit Unterrelaxation            *
*                  zur                                                   *
*                  L"osung sehr gro"ser d"unn besetzter Matrizen         *
* ====================================================================== *
* erstellt am    :   Juli 1994                                           *
* Programmierer  :   Anja Seibt                                          *
* Betreuer       :   Reinhard German                                     *
*************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include "expand.h"

#include "struct_types.h"
#include "global.h"
#include "extern_fkt.h"

#include "test.h"



/* -----------------------------------------------------------------------
   Speicher besorgen und P-Matrix reinlegen
----------------------------------------------------------------------- */
Matrix_T*  p_m_besorgen_iter( max)
double  max;
{
  double    hf;
  register unsigned  i, j, no;
  int        fd;
  new_mark_t *pnmark;
  pred_t     *ppred;
  Matrix_T*  Mm;


  if (NULL == (Mm =(Matrix_T*) malloc (Dm * sizeof(Matrix_T))))
    {
      perror("ERROR: malloc Mm");
      exit(1);
    }
  for (i = 0; i < Dm; i++)
    {
      pnmark = new_mark[i];
      ppred  = pnmark->pred;
      no = pnmark->no_preds +1;

      if (NULL == (Mm[i].aa = (double*) malloc (no * sizeof(double))))
	{
	  perror("ERROR: malloc Mm.aa");
	  exit(1);
	}
      if (NULL == (Mm[i].sp = (unsigned*) malloc (no * sizeof(unsigned))))
	{
	  perror("ERROR: malloc Mm.sp");
	  exit(1);
	}
      Mm[i].anz = no;
      Mm[i].b = 0.0;
    }

  for(i = 0; i < Dm; i++)
    {
      pnmark = new_mark[i];
      ppred  = pnmark->pred;
      /* ......... Spalte aus Zeile mit Inhalt fuellen (== transponieren),
       * ............................ dabei mit  maxdiagonal  Normieren */
      for( no = pnmark->no_preds; no--;)
        {
          hf = ppred[no].rate/max;
          Mm[i].aa[no] = hf;
          Mm[i].sp[no] = ppred[no].from_mark;
	}
      Mm[i].sp[Mm[i].anz-1] = i;
      Mm[i].aa[Mm[i].anz-1] = (double) -(pnmark->ratesum)/max;
    }
  return Mm;

}  /* p_m_besorgen_iter */



/* -----------------------------------------------------------------------
   Matrix sortieren fuer SOR und Diagonalelemente eliminieren
----------------------------------------------------------------------- */
void  p_m_sort_iter()
{
  /* ... es wird davon ausgegangen, dass die Diagonalelemente an letzter
     ........................ Stelle in den  jeweiligen Vektoren stehen */
  unsigned  i, j;
  int       found;
  double    dd, diagonal;
  unsigned  uu;


  for (i = 0; i < Dm; i++)
    {
      diagonal = Matrix[i].aa[Matrix[i].anz-1];
      Matrix[i].anz--;
      /* .............................................. bubble sort ... */
      found = 1;
      while (found)
	{
	  found = 0;
	  for (j = 0; j < Matrix[i].anz-1; j++)
	    {
	      if (Matrix[i].sp[j] > Matrix[i].sp[j+1])
		{
		  found = 1;
		  uu = Matrix[i].sp[j];
		  dd = Matrix[i].aa[j];
		  Matrix[i].sp[j] = Matrix[i].sp[j+1];
		  Matrix[i].aa[j] = Matrix[i].aa[j+1];
		  Matrix[i].sp[j+1] = uu;
		  Matrix[i].aa[j+1] = dd;
		}
	    }
	} /* sort fertig */

      if (1.0 == diagonal)
	continue;

      for (j = 0; j < Matrix[i].anz; j++)
	{
	  Matrix[i].aa[j] = Matrix[i].aa[j] / diagonal;
	}
    }
}  /* p_m_sort_iter */



/* -----------------------------------------------------------------------
   Einser Zeile setzen fuer Iterationen, in erste Zeile
----------------------------------------------------------------------- */
void  p_m_1_fuellen_iter()
{
  unsigned  i;


  if(NULL == (Matrix[0].aa = (double*) realloc (Matrix[0].aa,
					      (Dm - 1) * sizeof(double))))
    {
      perror("ERROR: realloc Matrix.aa");
      exit(1);
    }
   if(NULL == (Matrix[0].sp = (unsigned*) realloc (Matrix[0].sp, 
					    (Dm - 1) * sizeof(unsigned))))
    {
      perror("ERROR: realloc Matrix.sp");
      exit(1);
    }

  /* ........................................ Diagonalelement entfaellt */
  Matrix[0].anz = Dm - 1;

  for (i = 0; i < Matrix[0].anz; i++)
    {
      Matrix[0].aa[i] = 1.0;
      Matrix[0].sp[i] = i + 1 ;
    }
  Matrix[0].b = 1.0;

}  /* p_m_1_fuellen_iter */



/* ------------------------------------------------------------ main -----
   Gesamtschrittverfahren mit Unterrelaxation
----------------------------------------------------------------------- */
double*  p_iteration( Maxdiagonal, MaxIter, Xold, EXACT, status)
double   Maxdiagonal;
int      MaxIter;
double*  Xold;
int      EXACT;
int      status;
{
  double      *Xnew,
              *Xdirect;
  double      rest_zeile;
  double      omega;
  double      ff, hf, ffold, fdirect;
  int         dropped = 0;
  unsigned    j, i, k, ka, kk;
  Matrix_T    *ffm;

  if (status == NACHITERATION)
    printf("\nIterative refinement of solution with SOR ...\n");
  else
    printf("\nSolution with SOR ...\n");
  fflush(stdout);
	  
  /* printf("\nRelaxation bei Gesamtschrittverfahren\n\n");
  fflush(stdout); */

  /* ----------------------- P-Matrix aus Erreichbarkeitsgraphen holen und
                               alloziierten Speicherplatz ablegen ----- */
  p_m_matrix_besorgen( Maxdiagonal);
  p_m_sort_iter();
  p_m_1_fuellen_iter();

  if (NULL == (Xnew = (double*) malloc ((int)Dm * sizeof(double))))
    {
      perror("p_sor --  malloc: vector Xnew\n");
      exit(1);
    }

  if (status == NACHITERATION)
    {
      if (NULL == (Xdirect = (double*) malloc ((int)Dm * sizeof(double))))
        {
          perror("p_sor --  malloc: vector Xdirect\n");
          exit(1);
        }
        Xdirect = (double*)memcpy((char*)Xdirect, (char*)Xold,
                       (int)Dm*(int)sizeof(double)/(int)sizeof(char));
    }
 
  /* ....................... Berechnung: Gesamtschrittverfahren mit 
     ................................... adaptiver omega-Bestimmung
     ............................... Startvektor == (1/Dm,...,1/Dm) ... */
  k = 0;
  ka = 0;
  kk = 0;
  omega = 1.0;
  ffold = 1.0;
  ffm = p_m_besorgen_iter( Maxdiagonal);
  if (!EXACT)
    dropped = p_m_drop(); 

  if ( MaxIter >= 100 )
    {
      printf("\nnumber of performed iterations (stepsize = 100):\n");
      fflush(stdout);
    }

  while (k < MaxIter)
    {
      ++k;
      ka++;
      if (ka == 100)
	{
	  printf("_%u", k);
	  fflush(stdout);
	  ka = 0;
	}

      for (i = 0; i < Dm; i++)
	{
	  rest_zeile = 0.0;
	  for (j = 0; j < Matrix[i].anz; j++)
	    {
	      rest_zeile += Matrix[i].aa[j] * Xold[Matrix[i].sp[j]];
	    }
	  Xnew[i] = Matrix[i].b - rest_zeile;
	}

      ff = p_fehler( ffm, Xnew);
      if (ff < Prec)
	break;
      
      if (ff < ffold)
	omega = 0.8;
      else
	omega = fabs((ffold / ff) - (omega * 0.24));

      for (i = 0; i < Dm; i++)
	{
	  Xnew[i] = Xold[i] + (omega * (Xnew[i] - Xold[i]));
  
	  Xold[i] = Xnew[i];
	}

      ff = p_fehler( ffm, Xnew);
      if (ff < Prec)
	break;
      
      ffold = ff;
    }
 
  /*  printf("\nmaximaler Fehler: %.4g \nnach %u Iterationen\n\n", fabs(ff), k);
  fflush(stdout); */

  if (k >= 100)
    printf("\n");

  if (!EXACT)
    {
      printf("\nDroptolerance: %lg\n",Droptol);
      printf("Number of dropped elements: %d",dropped);
    }

  if (ff < Prec)
    {
      OK = TRUE;
      Nachiter = FALSE;
      printf("\nConvergence in %d iterations with accuracy %lg\n",k,ff);
    }
  else
    {
      printf("\nNo convergence within %d iterations.\n",MaxIter);
      printf("Achieved accuracy: %lg\n",ff);
      fflush(stdout);
      
      if (status == NACHITERATION)
	{
          fdirect = p_fehler( ffm, Xdirect);
          if (fdirect < ff)
	    {
            printf("\nIterativ refinement was not successfull ... keeping direct solution.\n\n");
            Xnew = Xdirect;
	    }
	}
    }
 
  return Xnew;
  
}  /* p_iteration */
