/******************************************************************************
* Programm  : gensol                                                          *
* Komponente: main.c                                                          *
* Parameter : r c s m p i o e x v h ? (siehe gensol -h)                       *
* Rueckgabe : 0 - falls .sol gueltig erzeugt, !0 - sonst                      *
* Aufgabe   : Oeffnen der Files, Optionen und Parameter ueberpruefen          *
* Projektleiter: Reinhard German, TU-Berlin                                   *
* Programmierer: Thomas Kuhlmann, TU-Berlin                                   *
* Datum:         Mon Jun 21 18:18:40 MET DST 1993                             *
*                Wed Jun 30 20:03:11 MET DST 1993 option -e maxdnspivmem      *
*                Mon Jul 19 14:11:04 MET DST 1993 option -k koeffizient       *
*-----------------------------------------------------------------------------*
* veraendert: Anja Seibt
*             August 1 1994   option -n Nachiteration
*             August 1 1994   option -t minimale Matrixwerte => 'exact'
******************************************************************************/
#define __main_c__
#include <time.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>
#include "expand.h"

char         *progname;    /* fuer Fehlerausgaben */
phase_t       no_phases    = (phase_t) 10;
phase_typ     phase_type   = ERLANG;
policy_t      policy       = ENABLING;
double        variation;   /* Variationskoeffizient bei Coxsubnetz */
double        a            = (double)0.0; /* muss nicht initialisiert werden */
double        eins_minus_a = (double)0.0; /* muss nicht initialisiert werden */
det_tra_no_t  no_det_trans = (det_tra_no_t )0;
timed_tra_no_t no_timed_trans = (timed_tra_no_t)0;
timed_tra_t  *timed_tran   = (timed_tra_t *)0;
old_mark_no_t no_old_marks = (old_mark_no_t)0;
old_mark_t   *old_mark     = (old_mark_t  *)0;
new_mark_no_t no_new_marks = (new_mark_no_t)0;
new_mark_t  **new_mark     = (new_mark_t **)0;
new_mark_t   *curr_new     = (new_mark_t  *)0;
int           no_old_edges;
unsigned long no_new_edges = (unsigned long)0l;
solution_t    sol_typ      = AUTOMATIC;
bool_t verbose   = FALSE;

static char *policy_str[] = { "resampling", "enabling", "age"    };
static char *method_str[] = { "automatic",  "iterative", "stabil", "direct" };
static int   maxdnspivmem = MAXDNSPIVMEM;

int main(ac,av)
int   ac;
char *av[];
{
	extern char  *optarg;
	extern int    optind;
	extern double atof();
	extern void   p_solve_LGS();
	extern void   showXRG();
	extern int    read_opt();
	static double precision    = 1.e-8;
	static double tol_drop     = 0.0;
	static int    iterations   = 1000;
	FILE         *fp;
	bool_t terminate = FALSE;
	bool_t show_xrg  = FALSE;
	bool_t nachiter  = FALSE;
	char *model_name = (char*)0;
	int c, i;
	int  exact = FALSE;

	for(i=strlen(av[0]); i--; )
	  if (av[0][i]=='/') break;
	progname = &(av[0][i+1]);
	while ((c=getopt(ac,av,"r:c:k:s:m:p:t:i:o:e:n:xvh?")) != -1) 
	  {
	    switch(c) 
	      {
	      case 'r':
		       if (((i = atoi(optarg))<1) || (i>255))
			 fprintf(stderr,"-%c %d ignored (1..255)\n",c,i);
		       else
			 no_phases = (phase_t) i;
		       break;
	      case 'c':
		       if ((i = atoi(optarg))>0) 
			 {
			   phase_type = COX;
			   variation  = ((double)1.0)/((double)i);
			 }
		       else
			 fprintf(stderr,
				 "-%c %d ignored, value less or equal zero\n",
				 c,i);
		       break;
	      case 'k':
		       phase_type = COX;
		       variation  = atof(optarg);
		       break;
	      case 's':
		      for(i=(int)AGE; i>=0; i--)
			if (optequal(optarg, policy_str[i])) break;
		       if (i<0)
			 fprintf(stderr,
				 "-%c %s ignored, no valid policy\n",c,optarg);
		       else
			 policy = (policy_t) i;
		       break;
	      case 'm':
		       for(i=(int)DIRECT; i>=0; i--)
			 if (optequal(optarg, method_str[i])) break;
		       if (i<0)
			 fprintf(stderr,
				 "-%c %s ignored, no valid method\n",c,optarg);
		       else
			 sol_typ = (solution_t)i;
		       break;
	      case 'p':
		       if ((a = atof(optarg)) <= 0.0)
			 fprintf(stderr,
				 "-%c %f ignored, value less or equal zero\n",
				 c,a);
		       else
			 precision = a;
		       a = 0.0;
		       break;
	      case 't':
		       if ((a = atof(optarg)) < 0.0)
			 fprintf(stderr,
				 "-%c %f ignored, value less\n",
				 c,a);
		       else
			 tol_drop = a;
		       a = 0.0;
		       break;
	      case 'i':
		       if ((i=atoi(optarg))>0)
			 iterations = i;
		       else
			 fprintf(stderr,
				 "-%c %d ignored, value less or equal zero\n",
				 c,i);
		       break;
	      case 'o':
		       model_name = optarg;
		       break;
	      case 'e':
		       if ((i=atoi(optarg))>0)
			 maxdnspivmem = i;
		       else
			 fprintf(stderr,
				 "-%c %d ignored, value less or equal zero\n",
				 c,i);
		       break;
	      case 'n':
		       nachiter = TRUE;
		       optind--;
		       break;
	      case 'x':
		       show_xrg = TRUE;
		       break;
	      case 'v':
		       verbose = TRUE;
		       break;
	      default:
		       if (terminate) break;
		       fprintf(stderr,"\
usage: %s [options] [<model>.rrg], possible options are:\n\
 -r BYTE   number of generated phases per transition (%d)\n\
 -c INT    Coxparameter, without '-c' Erlang-propability is used\n\
 -k DOUBLE Variationskoeffizient (Cox only)\n\
 -s STRING transition fire semantic:'%s'(%c)", progname, no_phases,
			       policy_str[0], (policy==(policy_t)0)?'x':'-');
		       for(i=1; i<=(int)AGE; i++)
			 fprintf(stderr,", '%s'(%c)",
				 policy_str[i], (policy==(policy_t)i)?'x':'-');
		       fprintf(stderr,"\n -m STRING solutionMethod '%s'(%c)",
			       method_str[0], (sol_typ==(solution_t)0)?'x':'-');
		       for(i=1; i<=(int)DIRECT; i++)
			 fprintf(stderr,", '%s'(%c)",
				 method_str[i], (sol_typ==(solution_t)i)?'x':'-');
		       fprintf(stderr,"\n\
 -p DOUBLE precision      (used for iterative method)(%.2e)\n\
 -i INT    max iterations (used for iterative method)(%d)\n\
 -t DOUBLE droptolerance \n\
 -n        postiteration \n\
 -o STRING set outputfilename <model>.sol (for input from stdin)\n\
 -e INT    set expanded memory(%d) for gaus-elimination method\n\
 -x        write file '<model>.xrg' (eXpanded reachabilty graph)\n\
 -v        switches on verbose-mode\n\
 -h or -?  this help\n\
when no <model>.rrg -filename is given, then the reduced reachability graph\n\
is read in from stdin and '-o <model>.sol' has to be set.\n\n",
			       precision, iterations, maxdnspivmem );
		       terminate = TRUE;
		       break;
		     }
	  }
	if (terminate) 
	  exit(0);

	if (!model_name) 
	  {
	    /*
	     *	no modelname set by -o option, input from file 'parameter'
	     */
	    if (! (model_name = av[optind])) 
	      {
		fprintf(stderr,"%s: ERROR <model>.rrg missing (try -h)\n",progname);
		exit(1);
	      }
	    if (!(fp=fopen(model_name,"r"))) eperror(model_name);
	  }
	else 
	  {
	    /*
	     *	modelname was set by -o option, read from stdin
	     if (!(fp=fdopen(STDIN_FILENO,"r"))) eperror(progname);
	     */
	    fp = stdin;
	  }
	/*
	   if (read_opt(model_name,&no_phases,&phase_type,&variation)) {
	   fprintf(stderr,"%s:ERROR %s in WRONG FORMAT\n",progname,model_name );
	   exit(1);
	   }*/
	if (no_phases<=1) phase_type = ERLANG;
	if (phase_type == COX) 
	  {
	    a = ((double)1.0) -
	      ( ((double)no_phases)*variation + ((double)no_phases)*(double)0.5
	       - ((double)1.0) -
	       sqrt( -((double)no_phases)*variation
		    + ((double)(no_phases*no_phases))*(double)0.25
		    + (double)1.0
		    )
	       )/(
		  ((double)(no_phases-1))*(variation + (double)1.0)
		  );
	    eins_minus_a = ((double)(1.0)) - a;
	  }
        clock(); /* Prozessuhr starten.. ( fuer show_rusage() )*/
	if (read_rrg(fp)) 
	  {
	    fprintf(stderr,"%s:ERROR RRG in WRONG FORMAT\n",progname );
	    exit(1);
	  }
	if (verbose) 
	  {
	    printf("old_markings:%6u,\told_edges:%8d\n", no_old_marks, no_old_edges );
	    fflush(stdout);
	  }
	initNewMarks();
	if (expand()) exit(1);
	if (verbose) 
	  {
	    printf("new_markings:%6u,\tnew_edges:%8u\n", no_new_marks, no_new_edges );
	    /*  printf("\nRESOURCEUSAGE FOR EXPANDING:\n"); */
	    show_rusage();
	    fflush(stdout);
	  }
	if (show_xrg) 
	  {
	    i = strlen(model_name);
	    for (optarg = "grx."; i && *optarg; model_name[--i] = *(optarg++));
	    (void) showXRG(model_name);
	  }


	/* ---------------------------------- Aufruf der Loesungsroutine ----- */
        clock(); /* Prozessuhr starten.. ( fuer show_rusage() )*/
	p_solve_LGS (sol_typ,
		     tol_drop,
		     precision,
		     iterations, 
		     nachiter);

        show_rusage();


	i = strlen(model_name);
	for (	optarg = "los."; i && *optarg; model_name[--i] = *(optarg++) );
	write_result(model_name);

	return (0);
}



