/******************************************************************************
* Programm  : gensol                                                          *
* Komponente: options.c                                                       *
* Aufgabe   : lesen der L"osungsoptionen aus der Datei <model>.opt            *
* Projektleiter: Reinhard German, TU-Berlin                                   *
* Programmierer: Thomas Kuhlmann, TU-Berlin                                   *
* Datum:         Mon Mar  7 22:09:03 MET 1994                                 *
******************************************************************************/
#define __options_c__
#include "expand.h"
#define TNAMELEN 255
#define OK    FALSE
#define ERROR TRUE

/*
 *	Lesen der Datei <model>.opt
 * Diese gibt f"ur jede deterministische Transition an, mit welcher Methode
 * sie approximiert werden soll.
 * In dieser ersten Version ist nur Methode gloabl w"ahlbar, so da"s eine
 * Fehlermeldung generiert wird, falls unterschiedliche Methoden gew"ahlt
 * werden.
 */
int read_opt(rrg_name,p_no_phases,p_phase_type,p_variation)
char      *rrg_name;
phase_t   *p_no_phases;
phase_typ *p_phase_type;
double    *p_variation;
{
	FILE *fp;
	char *ptr, tname[TNAMELEN+1], c;
	int i, phases, coxparameter, values_set;

	i = strlen(rrg_name);
	for (	ptr = "tpo."; i && *ptr; rrg_name[--i] = *(ptr++) );
	if (!(fp=fopen(rrg_name,"r"))) {
		perror(rrg_name);
		fprintf(stderr,"%s: WARNING using default options!\n",progname);
		return OK;
	}
	values_set = 0;
	while (!skip_comment_lines(fp)) {
		if (fscanf(fp,"TRANSITION %s %c", tname, &c )!=2) return ERROR;
		switch(c){
		case 'E':
			if ((i=fgetc(fp))==EOF) return ERROR;
			switch(c=(char)i){
			case 'X':
				if (fscanf(fp,"ACT;\n")) return ERROR;
				fprintf(stderr,"%s: '%s' EXACT solution not yet implemented.\n",
					progname, rrg_name);
				exit(1);
			case '(':
				if (fscanf(fp,"%d);\n", &phases)!=1) return ERROR;
				if (values_set++) {
					if ((*p_no_phases  != (phase_t) phases)||
						 (*p_phase_type != ERLANG)) {
						fprintf(stderr,"%s: '%s' transition dependent solve-options not yet implemented.\n", progname, rrg_name );
						exit(1);
					}
				} else {
					*p_no_phases  = (phase_t) phases;
					*p_phase_type = ERLANG;
				}
				break;
			default: return ERROR;
			}
			break;
		case 'G':
			if(fscanf(fp,"C(%d,%d);\n", &phases, &coxparameter)!=2) return ERROR;
			if (values_set++) {
				if ((*p_no_phases  != (phase_t) phases)||
					 (*p_phase_type != COX)             ||
					 (*p_variation  != (double)1.0/(double)coxparameter)) {
					fprintf(stderr,"%s: '%s' transition dependent solve-options not yet implemented.\n", progname, rrg_name );
					exit(1);
				}
			} else { /* first setting of values: */
				*p_no_phases  = (phase_t) phases;
				*p_phase_type = COX;
				*p_variation  = (double)1.0/(double)coxparameter;
			}
			break;
		default: return ERROR;
		}
	}
	fclose(fp);
	return OK;
}
