/*****************************************************************************

	Programm	:	struct

	Komponente	:	struct_main.c
				Hauptprogramm und globale Variablen

	Parameter	:	Optionen [-apseix]
				Netzname

	Rueckgabewert	:	0 -> Programm erfolgreich beendet
				1 -> es trat ein Fehler auf

	Aufgabe		:	struct ermittelt Netzstruktur-Informationen
				fuer das graph-Programm, teilt das DSPN in
				Subnetze zeitloser Transitionen und testet,
				ob Confusions auftreten

	Programmierer	:	Armin Zimmermann

	Datum		:	Juni 1992
----------------------- GEAENDERT --------------------------------------
	Parameter	:	Optionen [-ap]
				Netzname
	Aufgabe		:	struct wird nur benutzt, um die 
				place-invarianten zu berechnen und um die
				token-bounds in "Netzname".bnd zu speichern.

	Programmierer	:	Robert Zijal

	Datum		:	Maerz 1995

*****************************************************************************/

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include "struct_types.h"			/* definitions and types */

#ifndef __USE_FIXED_PROTOTYPES__
#define __USE_FIXED_PROTOTYPES__
#endif

extern int  	minus();
extern sp_place **sparse_function();
extern void 	read_net();			/* from struct_pin */
extern void 	p_invariants();
extern void	p_output();
extern void	write_bounds();


		/******* global variables *******/

int no_places;			/* number of places in the net */
int no_trans;			/* number of transitions in the net */
int no_md_arcs;			/* number of marking dependent arcs */
int no_timed_trans;		/* number of timed transitions */
int no_pinvar;			/* number of place-invariants */


sp_place 	**i_matrix, **o_matrix, **c_matrix;	/* in, out and c-matrix */
sp_place 	**h_matrix;			/* inhibitor matrix */
md_arc		*md_arcs;			/* marking dep. arcs */
pinvar 		*pinvars;			/* resulting P-invariants */
name_nr		*m_par;				/* marking parameters */
name_delay	*r_par;				/* real parameters */
place		*places;			/* place description */
transition  	*transitions;			/* transition description */
FILE 		*struct_file, *inf_file;	/* input and output files */
int		pin_bit;	/* options for output of results */


/*
 * Usage:
 */
static void usage()
{
	printf("Usage: struct [-ap] netname\n");
	printf("  p = print place invariants\n");

	exit( -1 );
}


/*	deter_in reads information about determining places from a file */
void
deter_in(f, p)
	FILE	*f;
	int	**p;
{
	int	i, no_deter = 0;
	char	dum[50];

	fscanf(f, "%s", dum);
	fscanf(f, "%s", dum); no_deter = atoi(dum);
	*p = (int *)nMalloc(no_places * sizeof(int));
	for (i = 0; i < no_deter; i++)
	{	fscanf(f, "%s", dum);
		(*p)[atoi(dum)] = 1;
	}
}



/*	read_info reads additional informations about marking dependent net parts,
	<net>.DEFINFO is provided by proc_MDF */
void
read_info(inf)
	FILE	*inf;
{
	int 	i, id, tra, no_ena_fcts, no_weights;
	md_arc	*p;
	char	dum[50];
	sp_place *p_w;

	/* read information from <net>.DEFINFO */
	fscanf(inf, "%s", dum);
	fscanf(inf, "%s", dum);
	no_md_arcs = atoi(dum);
	md_arcs = (md_arc *)nMalloc(no_md_arcs * sizeof(md_arc));
	for (i = 0; i < no_md_arcs; i++)
	{	p = &(md_arcs[i]);
		fscanf(inf, "%s", dum);
		fscanf(inf, "%s", dum); p->id = atoi(dum);
		fscanf(inf, "%s", dum); p->type = atoi(dum);
		fscanf(inf, "%s", dum); p->trans = atoi(dum);
		fscanf(inf, "%s", dum); p->place = atoi(dum);
		deter_in(inf, &(p->deter));
		switch (p->type)
		{ case INPUT:	p_w = i_matrix[p->trans]; break;
		  case OUTPUT:	p_w = o_matrix[p->trans]; break;
		  case INHIBIT:	p_w = h_matrix[p->trans]; break;
		  default:	exit(1);
		}
		while (p_w != NULL && p_w->num != p->place)
			p_w = p_w->next;
		if (p_w == NULL)
			exit(1);
		p_w->plu_id = p->id;
	}
	fscanf(inf, "%s", dum);
	fscanf(inf, "%s", dum);
	no_ena_fcts = atoi(dum);
	for (i = 0; i < no_ena_fcts; i++)
	{	fscanf(inf, "%s", dum);
		fscanf(inf, "%s", dum); id = atoi(dum);
		fscanf(inf, "%s", dum); tra = atoi(dum);
		transitions[tra].ena_id = id;		
		deter_in(inf, &(transitions[tra].e_deter));
	}
	fscanf(inf, "%s", dum);
	fscanf(inf, "%s", dum);
	no_weights = atoi(dum);
	for (i = 0; i < no_weights; i++)
	{	fscanf(inf, "%s", dum);
		fscanf(inf, "%s", dum); id = atoi(dum);
		fscanf(inf, "%s", dum); tra = atoi(dum);
		transitions[tra].rate_id = id;
		deter_in(inf, &(transitions[tra].w_deter));
	}
}

/*	the main procedure handles options and file access and
	calls subordinated procedures */
main(argc, argv)
	int argc;
	char ** argv;
{
	char net_name[FILE_NAME], inf_name[FILE_NAME];
	char bnd_name[FILE_NAME];	/* new */
	char options[10];
	int i, c;
	extern int optind;

	/* check arguments */
	if (argc == 1)
		usage();
	pin_bit = 0;

	/* parse options */
	while( (c = getopt(argc,argv,"ap")) != -1 )
		switch( c )
		{
		case 'a':
			pin_bit = 1;
			break;
		case 'p':
			pin_bit = 1;
			break;
		default:
			usage();
		}
	if( optind == argc )
		usage();

	/* build filenames */
	sprintf(net_name,"%s", argv[optind]);
	sprintf(inf_name,"%s.DEFINFO", net_name);
	sprintf(bnd_name,"%s.BND", net_name);	/* new */

	/* open DEFINFO file */
	if ((inf_file = fopen(inf_name, "r")) == NULL)
	{	printf("ERROR: could not open %s for reading\n", inf_name);
		exit(1);
	}

	/* begin net analysis */
	printf("\nSTRUCTURAL ANALYSIS ...\n");
	read_net(net_name);


	read_info(inf_file);		/* read additional information */


	if (no_places < 1)
	{	printf("Error: no places.\n");
		exit(1);
	}
	if (no_timed_trans < 1)
	{	printf("Error: no timed transitions.\n");
		exit(1);
	}

#ifdef DEBUG
	printf("I-Matrix:\n");
	print_matrix(i_matrix);
	printf("O-Matrix:\n");
	print_matrix(o_matrix);
	printf("M0:\n");

	for(i=0; i<no_places; i++)
	printf("%d ", places[i].m0);
#endif
					/* calculate the global c-matrix */
	c_matrix = sparse_function(o_matrix, i_matrix, no_trans, minus); 
#ifdef DEBUG
	printf("C-Matrix:\n");
 	print_matrix(c_matrix); 
#endif

	p_invariants();			/* calculate P-Invariants */
	p_output();			/* print p-invariants and calculate upper bounds */
	write_bounds(bnd_name);		/* new */
	return 0;			/* o.k */
}

