/*****************************************************************************

Programm	:	struct

Komponente	:	struct_pin.c
			Einlesen des <netname>.NET - Files,
			Ermitteln der Place-Invarianten des DSPN
			( Martinez / Silva )

Programmer	:	Armin Zimmermann
Date		:	Juni 1992

Change		:	Thu Mar  3 17:12:50 MET 1994

		IMM immediate (zeitlose)           transition.type = IMM (0)
		DET deterministische               transition.type = DET (1)
		EXP exponentielle                  transition.type = EXP (2)
		GEN general (beliebig verteilt)    transition.type = GEN (3)

		SS single   server race enabling   transition.infinite = SS  (0)
		IS infinite server race enabling   transition.infinite = IS  (1)

		rs single   server race resampling transition.infinite = RS  (2)
		re single   server race enabling   transition.infinite = RE  (3)
		ra single   server race age        transition.infinite = RA  (4)

Change		:	May/June 1994 (Ulrich Hertlein)

		adaption to .TN file format, uses load_net library function;
		NetDescr is converted to local data format, after that, everything
		unchanged

Change		:	March 1995 (Robert Zijal)
		function "write_bounds()" added.   

*****************************************************************************/


#include <stdio.h>
#include <string.h>

#include "struct_types.h"			/* definitions and types */
#include "struct_extern.h"			/* global definitions */

#include "netio.h"


row 	*zero, *positive, *negative;		/* rows sorted by active column */
int 	row_width;                  		/* width of one algorithm-row,  */
						/*       = no_places + no_trans */


/*-----------------------------------------------------------*/


/*      this function is used as the parameter-function for sparse_function
        to calculate the c-matrix = o_matrix - i_matrix                 */
void
minus(a, b, r)
        sp_place *a, *b, *r;
{
        if (a == NULL)
        {       r->min_id = b->plu_id;
                r->plu_id = 0;
                r->value =  - b->value;
                return;
        }
        if (b == NULL)
        {       r->min_id = 0;
                r->plu_id = a->plu_id;
                r->value =  a->value;
                return;
        }       
        if (a->min_id || b->min_id)
        {       printf("Error in minus.\n");
                exit(1);
        }
        r->min_id = b->plu_id;
        r->plu_id = a->plu_id;
        r->value = a->value - b->value;
}

/*      sparse_function_row obtains a new sparse row as a result of the 
        parameter-function of two sparse rows                           */
sp_place *
sparse_function_row(row1, row2, function)
        sp_place *row1, *row2;
        void    (* function)();
{
        sp_place *now1, *now2, *last3, *new3;
        sp_place *row3;                         /* row3 = function(row1, row2) */

        now1 = row1;
        now2 = row2;
        row3 = new3 = last3 = NULL;
        while(now1 != NULL || now2 != NULL)
        {       new3 = (sp_place *)nMalloc(sizeof(sp_place));
                if (now1 == NULL)                       /* no more 1-entries */
                {       (*function)(NULL, now2, new3);
                        new3->num = now2->num;
                        now2 = now2->next;
                } else if (now2 == NULL)                /* no more 2-entries */
                {       (*function)(now1, NULL, new3);
                        new3->num = now1->num;
                        now1 = now1->next;
                } else if (now1->num == now2->num)      /* same entries */
                {       (*function)(now1, now2, new3);
                        new3->num = now1->num;
                        now1 = now1->next;  now2 = now2->next;
                } else if (now1->num < now2->num)       /* 1-entry */
                {       (*function)(now1, NULL, new3);
                        new3->num = now1->num;
                        now1 = now1->next;
                } else                                  /* 2-entry */
                {       (*function)(NULL, now2, new3);
                        new3->num = now2->num;
                        now2 = now2->next;
                }
                if (new3->value || IS_MD(new3))
                {       if (last3 == NULL)
                                row3 = new3;            /* first entry */
                        else
                                last3->next = new3;     /* next entry */
                        last3 = new3;
                } else
                {       free(new3);                     /* no new entry */
                        new3 = last3;
                }
        }
        if (new3 != NULL)
                new3->next = NULL;                      /* last entry */
        return row3;
}


/*      sparse_function obtains a new sparse matrix as a result of the 
        parameter-function of two sparse matrices                       */
sp_place **
sparse_function(matrix1, matrix2, no_elem, function)
        sp_place **matrix1, **matrix2;
        int     no_elem;
        int     (* function)();                 /* arguments: (int, int) */
{
        int elem_nr;
        sp_place **matrix3;                     /* matrix3 = function(matrix1, matrix2) */

        matrix3 = (sp_place **)nMalloc(no_elem * sizeof(sp_place *));
        for(elem_nr = 0; elem_nr < no_elem; elem_nr++)
                matrix3[elem_nr] = sparse_function_row(matrix1[elem_nr],
                                                matrix2[elem_nr], function);
        return matrix3;
}






/*-----------------------------------------------------------*/


/*	The following procedure returns the [row, col]-value
	of a sparse-matrix. Rows and cols count from 0      */	
int
sp_access(matrix, sp_row, sp_col)
	sp_place **matrix;
	int sp_row, sp_col;
{  sp_place *now;

	now = matrix[sp_row];
	while(now)
	{	if (now->num == sp_col)
			return now->value;
		if (now->num > sp_col)
			return 0;
		now = now->next;
	}
	return 0;
}


#ifdef DEBUG
/*	print a sparse matrix (test only) */	
void
print_matrix(mat)
	sp_place **mat;
{	int i, j;
	sp_place *now;

	printf("\nMatrix:\n");
	for(i=0; i<no_trans; i++)           /* via sp_access */
	{ 	for(j=0; j<no_places; j++)
  		printf(" %2d", sp_access(mat, i, j));
  		printf("\n");
	}
	return;
}

/*	print a sparse row (test only) */	
void
print_rows(rowp)
	row *rowp;
{  int i;

	if (!rowp)
	{	printf("\nNo Rows found.\n");
		return;
	}
	printf("\nRows:\n");
	while(rowp)
	{	for(i=0; i<no_places+no_trans; i++)
			printf(" %2d", rowp->data[i]);
		printf("\n");
		rowp = rowp->next;
	}
}
#endif


/*	print the obtained p-invariants (option -p) and calculate the upper bounds
	for number of tokens in each place 						*/
void
p_output()
{	sp_place *act_place;
	pinvar *pin;
	int pin_num;

	for(pin_num = 0; pin_num < no_places; pin_num++)
   	{	places[pin_num].bound = MAX_TOKEN;		/* maximum upper bound */
		places[pin_num].covered = 0;
	}
	if (pin_bit)
		printf("\nThe net contains %d P-invariants.", no_pinvar);
	pin = pinvars;  pin_num = 0;
	while(pin)
	{	if (pin_bit)
			printf("\n==> Number %d (token-count = %d):\n", 
	                	++pin_num, pin->token_count);
		act_place = pin->first;
		while(act_place)
		{	if (pin_bit)
			{	if (act_place->value != 1)
					printf("%dx", act_place->value);
				printf("%s ", places[act_place->num].name);
			}
			if (places[act_place->num].bound > pin->token_count / act_place->value)
				places[act_place->num].bound = 
							pin->token_count / act_place->value;
			places[act_place->num].covered = 1;	/* place is covered */
			act_place = act_place->next;
		}
		pin = pin->next;
	}
	if (pin_bit)
		printf("\n\n");
	for(pin_num = 0; pin_num < no_places; pin_num++)
	if (places[pin_num].covered)
	{	if (pin_bit)
			printf("Place %s is %d-bounded.\n", 
				places[pin_num].name, places[pin_num].bound);
	}
	else	if (pin_bit)
   	   		printf("Warning-place %s is not contained in any P-invariant.\n",
				places[pin_num].name);
}


/*	release the allocated memory of a sparse row */
void
free_rows(rowp)
	row **rowp;
{  row *now, *next;

	now = *rowp;
	*rowp = NULL;
	if (!now)
   	return;
	while(now)
	{	next = now->next;
		free(now->data);
		free(now);
		now = next;
	}
}

/*	sort all invariants (of the current level) by positive/negative entries */
void
sort_rows(level)
	int level;
{	row *z_now, *z_last, *p_now, *n_now;

	z_now = zero;  z_last = NULL;
	p_now = NULL; n_now = NULL;
	while(z_now)
	{	if (z_now->data[level] != 0)
		{	if (z_now->data[level] > 0)     /* positive */
			{	if (p_now)
					p_now->next = z_now;
				else
					positive = z_now;
				p_now = z_now;
			} else {                       /* negative */
				if (n_now)
					n_now->next = z_now;
				else
					negative = z_now;
				n_now = z_now;
			}
			z_now = z_now->next;
			if (z_last)
				z_last->next = z_now;
			else
				zero = z_now;
		} else {
			z_last = z_now;		/* drop row with zero-entry */
			z_now = z_now->next;
		}
	}
	if (n_now) n_now->next = NULL;
	if (p_now) p_now->next = NULL;
}

int
kgv(a, b)
	int a,b;
{	int sum_a, sum_b;

	if (!a || !b)
	{	printf("kgv of 0 not defined.\n");
		exit(1);
	}
	sum_a = a;  sum_b = b;
	while(sum_a != sum_b)
	{	if (sum_a > sum_b)
			sum_b += b;
		else
			sum_a += a;
	}
   return sum_a;
}

/*	obtain all new rows for a level by combining all positive with all negative rows */
void
new_rows(level)
	int level;
{	row *n_now, *p_now, *z_now, *new_row, *test_now;
	int p_data, n_data, min_mul, minimal;
	int *new_data, data_nr, no_entries;

	z_now = zero;
	if (zero)
		while(z_now->next)       /* move z_now to the last zero-entry */
			z_now = z_now->next;
	p_now = positive;
	while(p_now)
	{	n_now = negative;
		while(n_now)
		{	p_data = p_now->data[level];
			n_data = - n_now->data[level];
			min_mul = kgv(p_data, n_data);
			p_data = min_mul / p_data;
			n_data = min_mul / n_data;
			new_data = (int *)nMalloc(row_width * sizeof(int));
			for(data_nr = 0; data_nr < row_width; data_nr++)
				new_data[data_nr] = p_data * p_now->data[data_nr] +
						 n_data * n_now->data[data_nr];
			/* test for minimal support */
         		no_entries = 0;
			for(data_nr = no_trans; data_nr < row_width; data_nr++)
				if (new_data[data_nr])
					no_entries++;
			test_now = zero;  minimal = 1;
			while(test_now && minimal)
			{	if (test_now->no_entries <= no_entries)
				{	data_nr = no_trans-1;
					while(++data_nr < row_width) 
						if (test_now->data[data_nr] >
								new_data[data_nr])
							break;
            				minimal = (data_nr < row_width);
            			}
				test_now = test_now->next;
			}
			if (!minimal)		/* invariant was not minimal */
				free(new_data);
			else {	new_row = (row *)nMalloc(sizeof(sp_place));
				new_row->data = new_data;
				new_row->no_entries = no_entries;
				new_row->next = NULL;
				if (z_now)
					z_now->next = new_row;
				else
					zero = new_row;
				z_now = new_row;
         		}
         		n_now = n_now->next;
		}
		p_now = p_now->next;
	}
	if (z_now)
		z_now->next = NULL;
}

/*	main procedure to calculate the minimal-support p-invariants */
void
p_invariants()
{	int pla_nr, tra_nr, *data, level, i;
	row *r_now, *r_last;
	pinvar *pin_now, *pin_last;
	sp_place *place_now, *place_last;
	char	inter[30];

	pinvars = NULL;   zero = NULL;
	positive = NULL;  negative = NULL;
	/* generate the first set of rows */
	row_width = no_places + no_trans;
	r_last = NULL;
	for(pla_nr = 0; pla_nr < no_places; pla_nr++)
	{	for (i = 0; i < no_md_arcs; i++)
			if (md_arcs[i].place == pla_nr && 
			   (md_arcs[i].type == INPUT || md_arcs[i].type == OUTPUT))
				break;
		if (i < no_md_arcs)
			continue;		/* connected to marking dep. arc */
		data = (int *)nMalloc(row_width * sizeof(int)); /* int data array */
		for(tra_nr = 0; tra_nr < no_trans; tra_nr++)
			data[tra_nr] = sp_access(c_matrix, tra_nr, pla_nr); /* transpose */
		for(tra_nr = no_trans; tra_nr < row_width; tra_nr++)
			data[tra_nr] = 0;
		data[no_trans+pla_nr] = 1;	/* unity matrix */
		r_now = (row *)nMalloc(sizeof(row));
		r_now->data = data;
		r_now->no_entries = 1;
		if (r_last)
			r_last->next = r_now;
		else
			zero = r_now;
		r_last = r_now;
	}
	if (r_last != NULL) r_now->next = NULL; /* was buggy when all places conn. to md-arcs */
#ifdef DEBUG
	printf("Rows start:\n");
	print_rows(zero);
#endif
	/* start algorithm */
	for(level = 0; level < no_trans; level++)
	{	
		sort_rows(level);
#ifdef DEBUG
		printf("Level %d\n", level);
		printf("zero");
		print_rows(zero);
		printf("pos");
		print_rows(positive);
		printf("negative");
		print_rows(negative); 
#endif
		new_rows(level);
#ifdef DEBUG
		printf("New rows now:\n");
		print_rows(zero);
#endif
		free_rows(&positive); free_rows(&negative);
	}
	/* get p-invars from last set of rows */
	no_pinvar = 0; pin_last = pinvars;
	r_now = zero;
	pin_now = NULL;
	while(r_now)
	{	no_pinvar++;
		pin_now = (pinvar *)nMalloc(sizeof(pinvar));
		pin_now->token_count = 0;
		pin_now->count_str[0] = 0;
		if (pin_last)
			pin_last->next = pin_now;
		else
			pinvars = pin_now;
		pin_last = pin_now;
 		pin_now->first = NULL;
		data = &(r_now->data[no_trans]);
		place_last = NULL;
		for(pla_nr = 0; pla_nr < no_places; pla_nr++)
		{	if (!data[pla_nr])
				continue;
			place_now = (sp_place *)nMalloc(sizeof(sp_place));
			if (place_last)
				place_last->next = place_now;
			else
				pin_now->first = place_now;
			place_last = place_now;
			place_now->num = pla_nr;
			place_now->value = data[pla_nr];
			place_now->min_id = place_now->plu_id = 0;
			if (data[pla_nr] * places[pla_nr].m0)
			{	sprintf(inter, "%d*", data[pla_nr]);
				if (pin_now->token_count)
					strcat(pin_now->count_str, " + ");
				if (data[pla_nr] > 1)
					strcat(pin_now->count_str, inter);
				strcat(pin_now->count_str, places[pla_nr].m0_str);
			}
			pin_now->token_count += data[pla_nr] * places[pla_nr].m0;
		}
		place_now->next = NULL;
		r_now = r_now->next;
	}
	if (pin_now)
		pin_now->next = NULL;
	free_rows(&zero);
}


/*
 * config_arcs: convert arc format
 */
int config_arcs(net, i, type)
NetDescr *net;
int i,type;
{
   int no_arcs;
   Arc *net_arc;
   sp_place **ptr;

   sp_place *last_p, *new_p;
   int j, no_point, point;
   char pl_name[50], help[20];

   /* determine arc type */
   switch( type )
   {
   case 0:
      no_arcs = net->trans[i].num_inarcs;
      net_arc = net->trans[i].in_arc;
      ptr = &(i_matrix[i]);
      break;
   case 1:
      no_arcs = net->trans[i].num_outarcs;
      net_arc = net->trans[i].out_arc;
      ptr = &(o_matrix[i]);
      break;
   case 2:
      no_arcs = net->trans[i].num_inharcs;
      net_arc = net->trans[i].inh_arc;
      ptr = &(h_matrix[i]);
      break;
   }

   for( j = 0; j < no_arcs; j++ )
   {
      new_p = (sp_place *)nMalloc(sizeof(sp_place));

      /* find index of place */
      strcpy(pl_name,net_arc[j].place->name);
      for (point = 0; point < no_places; point++)
         if (!strcmp(pl_name, places[point].name))
            break;
      if (point >= no_places)
      {
         printf("Unknown place %s in arc definition\n", pl_name);
         exit(1);
      }
      new_p->num = point;
      new_p->min_id = new_p->plu_id = 0;

      if( net_arc[j].is_value )
         new_p->value = net_arc[j].multipl.value;

      /* sort-by-insert */
      last_p = *ptr;
      if (last_p == NULL || last_p->num > new_p->num)
      {
         new_p->next = *ptr;
         *ptr = new_p;             /* first entry */
      }
      else
      {
         while( last_p->next != NULL && last_p->next->num < new_p->num )
            last_p = last_p->next;

         new_p->next = last_p->next;
         last_p->next = new_p;		/* normal entry */
      }
   }

   return( 0 );
}


/*
 * config_net: convert from netio-dataformat to local format
 */
int config_net(net)
NetDescr *net;
{
   int no_mpar, no_rpar;

   int i;

   /* create places list */
   no_places = net->num_places;
   places = (place *)nMalloc(no_places * sizeof(place));

   /* create transitions list */
   no_trans = net->num_trans;
   transitions = (transition *)nMalloc(no_trans * sizeof(transition));

   i_matrix = (sp_place **)nMalloc(no_trans * sizeof(sp_place *));
   o_matrix = (sp_place **)nMalloc(no_trans * sizeof(sp_place *));
   h_matrix = (sp_place **)nMalloc(no_trans * sizeof(sp_place *));
   for( i = 0; i < no_trans; i++ )
   {
      i_matrix[i] = NULL;
      o_matrix[i] = NULL;
      h_matrix[i] = NULL;
   }

   /* create delay parameters list */
   no_rpar = net->num_delaypars;
   r_par = (name_delay *)nMalloc(no_rpar * sizeof(name_delay));

   /* create marking parameters list */
   no_mpar = net->num_markpars;
   m_par = (name_nr *)nMalloc(no_mpar * sizeof(name_nr));


   /* config marking parameters */
   for( i = 0; i < no_mpar; i++ )
   {
      strcpy(m_par[i].name,net->markpar[i].name);
      m_par[i].val = net->markpar[i].value;
   }

   /* config places */
   for( i = 0; i < no_places; i++ )
   {
      strcpy(places[i].name,net->place[i].name);
      if( net->place[i].is_value )
      {
         places[i].m0 = net->place[i].marking.value;
         places[i].m0_str = (char *)nMalloc(16);
         sprintf(places[i].m0_str,"%d", places[i].m0);
      }
      else
      {
         places[i].m0 = net->place[i].marking.markpar->value;
         places[i].m0_str = net->place[i].marking.markpar->name;
      }
   }

   /* config delay parameters */
   for( i = 0; i < no_rpar; i++ )
   {
      r_par[i].name = net->delaypar[i].name;
      if( net->delaypar[i].is_value )
      {
         r_par[i].type = Value;
         r_par[i].values.val = net->delaypar[i].delay.value;
      }
      else
         r_par[i].type = Definition;
   }

   /* config transitions */
   for( i = 0; i < no_trans; i++ )
   {
      transitions[i].number = i;
      transitions[i].mark_dep_nr = 0;
      strcpy(transitions[i].name,net->trans[i].name);
      transitions[i].priority = net->trans[i].priority;

      switch( net->trans[i].delay_type )
      {
      case DELAY:
         transitions[i].value = net->trans[i].delay.value;
         break;
      case DELAYPAR:
         if( net->trans[i].delay.delay_par->is_value )
            transitions[i].value = net->trans[i].delay.delay_par->delay.value;
         else
            transitions[i].mark_dep_nr = i+1;
         break;
      case MARKDEP:
         transitions[i].mark_dep_nr = i+1;
         break;
      case PMFDEF:
         break;
      }

      /* Definitively buggy! inf_server is either True or False, by definition.
       * This is, however, how it was done in the original file ...
       */
      transitions[i].infinite = -1;
      if( net->trans[i].inf_server )
         transitions[i].infinite = s_IS;
      else
         transitions[i].infinite = s_SS;

      if( transitions[i].infinite == -1 )
         switch( net->trans[i].f_policy )
         {
         case RS:
            transitions[i].infinite = s_RS;
            break;
         case RA:
            transitions[i].infinite = s_RA;
            break;
         case RE:
            transitions[i].infinite = s_RE;
            break;
         }

      switch( net->trans[i].type )
      {
      case IMM:
         transitions[i].type = s_IMM;
         break;
      case EXP:
         transitions[i].type = s_EXP;
         break;
      case DET:
         transitions[i].type = s_DET;
         break;
      case GEN:
         transitions[i].type = s_GEN;
         break;
      }
      no_timed_trans += (transitions[i].type != s_IMM);

      transitions[i].rate_id = 0;
      transitions[i].w_deter = NULL;
      transitions[i].ena_id = 0;
      transitions[i].e_deter = NULL;

      /* config arcs */
      config_arcs(net,i,0);        /* input arcs */
      config_arcs(net,i,1);        /* output arcs */
      config_arcs(net,i,2);        /* inhibitor arcs */
   }

   return 0;
}



void read_net(filename)
char *filename;
{
   NetDescr *net;

   if (net = load_net(filename,NULL))
   {
      config_net(net);
   }
   else
   {
      printf("ERROR: could not read %s properly\n", filename);
      exit(1);
   }

   return;
}


void write_bounds(bnd_name)
char *bnd_name;
{
  FILE *bnd_file;
  int place_num;

  if (pin_bit) printf("Writing upper token-bounds to %s .\n\n", bnd_name);

  if ((bnd_file = fopen(bnd_name, "w")) == NULL) {
    printf("ERROR: could not open %s for writing\n", bnd_name);
    exit(1);
  }

  for(place_num = 0; place_num < no_places; place_num++) {
    if (!places[place_num].covered)
   	printf("Warning-place %s is not contained in any P-invariant.\n",
				places[place_num].name);
    fprintf(bnd_file, "%d %d\n", (Bool)places[place_num].covered, places[place_num].bound);
  }
  fclose(bnd_file);
}
