/*******************************************************************************
* Program:	genrrg
*
* File:		genrrg_main.c
*
* Purpose:	main file for the generation of the RRG 
*
* Programmer:	Robert Zijal
*
* History:	March 1995
*
*  1.6.96 adapted for show_rusage
* 15.8.96 changed all errormessages in /src to stdout due to TN-monitor ignorance
*******************************************************************************/

/* suppress not valid error-messages from new gcc-compiler */
#ifndef __USE_FIXED_PROTOTYPES__
#define __USE_FIXED_PROTOTYPES__
#endif

#include <stdio.h>
#include <time.h>
#include <portab.h>
#include "genrrg.h"
 
/* global access for more efficiency (please READ ONLY !) */
NetDescr *dtnet;
double DEB_accuracy = 1e-15;
Bool Embedding = FALSE;
Bool SaveSpace = FALSE;
unsigned long int but_step = 1; /*basic underlying timestep^-1 */

extern void show_rusage();

int main(argc, argv)
int argc;
char ** argv;
{ 
double tmp;
char command[500];
int my_pid;

static time_t start_time;
 clock(); /* Prozessuhr starten.. ( fuer show_rusage() )*/
 
 printf("\n\nGENERATING THE REDUCED REACHABILITY GRAPH ...\n");
   fflush(stdout); 
   start_time = time (NULL);

  if ((argc > 2) && !strcmp(argv[2], "-et")) {
    Embedding = TRUE;
    printf("Using embedding technique with time-saving option.\n");
  }
  if ((argc > 2) && !strcmp(argv[2], "-em")) {
    Embedding = TRUE;
    SaveSpace = TRUE;
    printf("Using embedding technique with memory-saving option.\n");
  }

  if (argc > 3) sscanf(argv[3], "%lg", &DEB_accuracy);
  printf("Accuracy set to %g.\n", DEB_accuracy);

  /* deal with basic underlying time-step */
  if (argc > 4) sscanf(argv[4], "%lg", &tmp);

  but_step = (unsigned long int)tmp;
    /* check legal value */
  if ( tmp != ((double) but_step) ) {
        fprintf(stdout,"ERROR: Basic underlying timestep = 1/%lg\n", tmp);
	fprintf(stdout,"Must be a positive rational.\n");
	exit( TRUE );
  } 

  printf("Basic underlying timestep set to 1/%lu . \n", but_step);
  if (but_step>1)
    printf( 
    "Attention, a small timestep may lead to a state space explosion!\n\n");


#ifdef SHOWMEMUSAGE
 no_mtree_reallocs = 0;
 no_phtree_reallocs = 0;

 m_bnd_size = 0;
 m_unbnd_size = 0;
 m_unbnd_mxentries = 0;
 m_unbnd_entries = 0;
 m_state_size = 0;

 ph_bnd_size = 0;
 ph_unbnd_size = 0;
 ph_unbnd_mxentries = 0;
 ph_unbnd_entries = 0;
 ph_state_size = 0;
#endif



  dtnet = load_dtn(argv[1]);
 
  /*show_dtnet(dtnet);*/
  /*show_net(dtnet);*/

  /* test_tree(); */ 
  /* test_graph(); */
  genrrg(); 

#ifdef SHOWMEMUSAGE
  printf(
  "\n ---------- Memory consumption for storage of tangible states ---------- \n");

  printf("  #reallocs for marking-tree\t: %lu \n", no_mtree_reallocs);
  printf("  #reallocs for phase-tree  \t: %lu \n\n", no_phtree_reallocs);

  printf("  bound marking-indices \t: %7lg \t  KB \n",
	 					((double)m_bnd_size)/1000);
  printf("  UNbound marking-indices \t: %7lg \t  KB", 
	 					((double)m_unbnd_size)/1000);
  if (m_unbnd_mxentries!=0) 
    printf("  \t (%lg %% used)\n", 
	   (((double)m_unbnd_entries) / ((double)m_unbnd_mxentries)) * 100.0);
  else printf("\n");

  printf("  state-marking          \t: %7lg \t  KB \n", 
	 					((double)m_state_size)/1000);
  printf("   \t total marking tree \t: %7lg \t  KB \n\n", 
	 ((double)(m_bnd_size+m_unbnd_size+m_state_size))/1000 );

  printf("  bound phase  -indices \t: %7lg \t  KB \n", 
	 					((double)ph_bnd_size)/1000);
  printf("  UNbound phase  -indices \t: %7lg \t  KB", 
	 					((double)ph_unbnd_size)/1000);
  if (ph_unbnd_mxentries!=0) 
    printf("  \t (%lg %% used)\n", 
	   (((double)ph_unbnd_entries) / ((double)ph_unbnd_mxentries)) * 100.0);
  else printf("\n");

  printf("  state-phase            \t: %7lg \t  KB \n", 
	 					((double)ph_state_size)/1000);
  printf("   \t total phase tree   \t: %7lg \t  KB \n\n", 
	 	((double)(ph_bnd_size+ph_unbnd_size+ph_state_size))/1000 );

  printf("   Total \t\t\t: %7lg \t  KB \n", 
   ((double)(m_bnd_size+m_unbnd_size+m_state_size+
	     ph_bnd_size+ph_unbnd_size+ph_state_size))/1000
	 );
  printf(
  "---------------------------------------------------------\n\n");
#endif


  fflush(stdout); 


  /* For Renate and Armin: The following can be made system-dependent */
  /* and included in other solution methods too! */

#if defined(sun) && defined(SOL2)
  /*----------------------------------------------*/
  /* new usage statistics for solaris 2 uses simply 'ps' (R.Zijal) */
  /* but it fails if ps has no '-o' option */

  my_pid = getpid();
  sprintf(command,"/usr/bin/ps -o \"pid etime time vsz\" | awk '/^[ ]*%d /{printf \"Elapsed CPU time:\\t %%s\\nUsed CPU time:\\t %%s\\nUsed Memory:\\t %%d KB (%%d MB)\\n\",$2,$3,$4,$4/1000}'",my_pid);
  system(command);
#else
  /*----------------------------------------------*/
  /* old usage statistics show no memory consumption for solaris 2 */
  show_rusage();
#endif

  fflush(stdout); 

  printf("\n\n\n");

  return 0;                       /* o.k */
}





