/*******************************************************************************
* Program:	genrrg
*
* File:		genrrg_tree.h
*
* Purpose:	Type-definitions and constants, headers.
*
* Programmer:	Robert Zijal
*
* History:	April 1995
*
* 18.9.96
* - extended phase storage to additional unbound mode (makes sense when embedding)
*******************************************************************************/

#ifndef _GRRG_TREE_H
#define _GRRG_TREE_H

#include <stdio.h>
#include <malloc.h>
#include <alloca.h>
#include <memory.h>




/*-------*/
/* TYPES */
/*-------*/

/* for export */
typedef struct  state{	/* State = (m, ph) */
     word *m;		
     word *ph;  
} State;		

/*new*/
typedef unsigned char	byte;		/* 1 byte */



/* tangible reachability graph types */

    /* marking-tree nodes */
typedef void *Mt_bnd_node;      /* marking tree item, bound node*/
                                /* token-indexed array of * to next nodes */

typedef struct mt_entry{	/* entry-indexed array of next nodes */
     word  no_tokens;		/* number of tokens in place */
     void *next_node;  
} Mt_entry;		

typedef struct mt_unbnd_node {	/* marking tree item, unbound node*/
  word      size;		/* size of array */
  word      num_e;		/* number of entries in array */
  Mt_entry *entry;		/* array of entries*/
} Mt_unbnd_node; 
 
typedef struct mt_marking {	/* marking tree item, leaf node */
  union{			/* save memory in marking if possible */
    byte *b;
    word *w;
  } m;
  void *Ph_tree;		/* first node of phase tree */
} Mt_marking;


    /* phase-tree node */
typedef void *Ph_bnd_node;      /* phase tree item, bound node*/
                                /* phase-indexed array of * to next nodes */

typedef struct pht_entry{	/* entry-indexed array of next nodes */
     word  no_phases;		/* number of phases of transition */
     void *next_node;  
} PHt_entry;		

typedef struct pht_unbnd_node {	/* phase tree item, unbound node*/
  word      size;		/* size of array */
  word      num_e;		/* number of entries in array */
  PHt_entry *entry;		/* array of entries*/
} PHt_unbnd_node; 


typedef struct ph_rfts {	/* phase tree item, leaf node */
  union{			/* save memory in phases if possible */
    byte *b;
    word *w;
  } ph;
  Mt_marking *mt_leaf;		/* backward chaining to mt_leaf above */
  struct ph_rfts *chain;	/* Chain of all S_tree-leafs */
  unsigned long int index;	/* unique state index */
} Ph_rfts;


typedef Ph_rfts State_ref;	/* for export, unique state reference */


/*--------------------------------------------*/
/* contain the state transition probabilities */
/*--------------------------------------------*/
typedef struct  van_arcs{       /* Multiple van. arcs to one tangible state */
     byte  *count;              /* transition firing count vector */
     double prob;               /* single path probability */
     struct van_arcs *next_arc;
} Van_arcs;             

typedef struct  van_paths{      /* Vanishing paths to tangible states */
     State_ref  *to_s;          /* tangible state (in S_tree) */
     Van_arcs   *mult_arcs;     /* Multiple vanishing paths to "to_s" */
     struct van_paths *next;
} Van_paths;            
/*--------------------------------------------*/


/*------------------*/
/* FUNCTION-HEADERS */
/*------------------*/

/* aux. functions */
extern void Copy_s(/* State *from, *to */);
extern void Copy_m(/* State *from; word *to */);
extern State *New_s();	
extern word  *New_m();	
extern word  *New_ph();	
extern void  Free_s(/* State *s */);	

extern void print_s(/*  FILE *stream,  State *s */);
extern void print_s_simple(/*  FILE *stream,  State *s */);
extern void print_m(/*  FILE *stream,  word  *m */);
extern void print_ph(/* FILE *stream,  word  *ph */);

/* testing functions */
extern void dump_nxtS();
extern void dump_S();
extern void dump_Stree();

/* tree functions */
extern void init_S();
extern State_ref  *ch_add_S(/* State *s */);
extern Bool get_nxtS(/* State *s, State_ref **s_ref */);

/* storing states and measures */
extern void init_file_handles();

extern void extr_init_prob(/* vp *initial_vp */);
extern void extr_res_store(/* vp *vp_head, State *s, State_ref *s_ref */);
extern void clean_up();
extern void finish_storage();




unsigned tmp_size; 

/* Creates new state on callers stack frame */
#define New_St_s(s) \
{ \
 tmp_size = sizeof(State); \
 if( (s = (State *)alloca(tmp_size)) == NULL ){ \
  fprintf(stdout,"New_St_s_a: Unable to alloca %u bytes on Stack\n", tmp_size);\
  exit( -1 ); \
 }; \
 memset((char *)s,0,tmp_size); \
 \
 tmp_size = (dtnet->num_places + dtnet->num_timed_trans) * sizeof(word); \
 if( (s->m = (word *)alloca(tmp_size)) == NULL ){ \
  fprintf(stdout,"New_St_s_b: Unable to alloca %u bytes on Stack\n", tmp_size);\
  exit( -1 ); \
 }; \
 memset((char *)s->m,0,tmp_size); \
 \
 s->ph = s->m + dtnet->num_places; \
}


#endif
