/*******************************************************************************
* Program:	genrrg
*
* File:		load_dtn.c
*
* Purpose:	Load the DTPN (interface to "Adapted"), 
*		Create own PN-data structures
*
* Programmer:	Robert Zijal
*
* History:	March 1995
*
* 1.8.96 
* - improved geom_usage message
* - init phases directly from delay (no string_trick need anymore)
* - init weights directly (no *.WP file needed anymore)
* 15.8.96
* - improved show_net-bug
* - added (post selection) priorities & m_dep input arc for flushing tokens
* - removed geom-delay (real not int) bug
* - added own errormessage-handler
* 
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "genrrg.h"
#include "expoly.h"

/*---------------------*/
/* COMPILE-SWITCH-KEYS */
/*---------------------*/

#define _LOADDEBUG  
#define PMFDEBUG

extern void adjust_expoly(/* time_l *expoly,int underlying Timestep */);
extern double *create_dtp(/* time_l *expoly,int *omega */); /*from create_dtp.c*/
extern time_l  *parse_pmf(); /*from parsdfunc.h */

#ifdef PMFDEBUG
extern void print_dtp(/* double *dtp_array,int num_phases*/);
extern void print_expoly();
#endif

/*
 Shows all data associated with the given network description.
 Copied from netio and extended to show additional dt-information.
 Graphical positions of items (not relevant for us) are suppressed.
*/
void show_dtnet(net)
NetDescr *net;
{
  int i;
  
  printf( "netname:\t\t\"%s\"\n",
    net->netname);
  printf( "nettype:\t\t\"%sSPN\"\n",
    net->type==GSPN?"G":
    net->type==DSPN?"D":
    net->type==EDSPN?"ED":
    net->type==CDSPN?"CD":
    "???");
  printf( "comment:\t\t\"%s\"\n",
    net->comment);

  printf( "error_func:\t\t\"%s\"\n",
    net->error_func != DefaultError ? "OWN" : "DEFAULT");
net->error_func( "I'm error_func.\n", FALSE );  


  printf( "\n%d markpars:\n", net->num_markpars );
  for(i=0;i<net->num_markpars;i++) {
    printf( "\t%2d.name\t\t\t\"%s\"\n",
      i, net->markpar[i].name);

    /* graphics
    printf( "\t  .pos\t\t\t(%lg;%lg)\n",
      net->markpar[i].pos.x, net->markpar[i].pos.y); 
    */
    printf( "\t  .value\t\t\t<%d>\n",
      net->markpar[i].value);
  } /* for */



  printf( "\n%d places:\n", net->num_places );
  printf("\tReordering overview:\n");
  for(i=0; i<net->num_places; i++) 
    printf("\t %d. place:%s   bound:%d   index:%d\n", 
      i, net->p_ranks[i]->name, net->p_ranks[i]->bound, net->p_ranks[i]->index);
  for(i=0;i<net->num_places;i++) {
    printf( "\t%2d.name\t\t\t\"%s\"\n",
      i, net->place[i].name);
    printf( "\t  .index\t\t<%d>\n",
        net->place[i].index);
    printf( "\t  .r_index\t\t<%d>\n",
        net->place[i].r_index);
    /* graphics
    printf( "\t  .pos\t\t\t(%lg;%lg)\n",
      net->place[i].pos.x, net->place[i].pos.y); 
    printf( "\t  .name_pos\t\t(%lg;%lg)\n",
      net->place[i].name_pos.x, net->place[i].name_pos.y); 
    */
    if( net->place[i].is_value )
      printf( "\t  .marking.value\t<%d>\n",
        net->place[i].marking.value);
    else
      printf( "\t  .marking.mark_par\t\"%s\"\n",
        net->place[i].marking.markpar->name);

    if( net->place[i].is_bound )
      printf( "\t   place is bound(%d) with:\n\t  .bound\t<%d>\n",
        net->place[i].is_bound, net->place[i].bound);
    else
      printf( "\t   place is NOT bound(%d), MAX_value:\n\t  .bound\t<%d>\n",
        net->place[i].is_bound, net->place[i].bound);
    printf( "\t  .m_0\t\t<%d>\n",
        net->place[i].m_0);
  } /* for */

  


  printf( "\n%d delaypars:\n", net->num_delaypars );
  for(i=0;i<net->num_delaypars;i++) {
    printf( "\t%2d.name\t\t\t\"%s\"\n",
      i, net->delaypar[i].name);
    /* graphics
    printf( "\t  .pos\t\t\t(%lg;%lg)\n",
      net->delaypar[i].pos.x, net->delaypar[i].pos.y); 
    */
    if( net->delaypar[i].is_value )
      printf( "\t  .delay.value\t\t<%g>\n",
        net->delaypar[i].delay.value);
    else
      printf( "\t  .delay.md_delay\t\"%s\"\n",
        net->delaypar[i].delay.md_delay);
  } /* for */



  printf( "\n%d transitions:\n", net->num_trans );
  printf( "\n%d timed transitions:\n", net->num_timed_trans );
  printf("\n\tReordering overview:\n");
  for(i=0; i<net->num_trans; i++) 
    printf("\t %d. trans:%s   phases:%d   index:%d\n", 
     i, net->t_ranks[i]->name, net->t_ranks[i]->phases, net->t_ranks[i]->index);
  for(i=0;i<net->num_trans;i++) {
    int a;
    Transition *t=&net->trans[i];
    
    printf( "\n\t%2d.name\t\t\t\"%s\"\n",    i, t->name);
    printf( "\t  .index\t\t<%d>\n",    t->index);
    printf( "\t  .r_index\t\t<%d>\n",
        net->trans[i].r_index);
    /* graphics*/

    printf( "\t  .pos\t\t\t(%lg;%lg)\n",
      t->pos.x, t->pos.y); 
    printf( "\t  .name_pos\t\t(%lg;%lg)\n",
      t->name_pos.x, t->name_pos.y); 
    printf( "\t  .delay_pos\t\t(%lg;%lg)\n",
      t->delay_pos.x, t->delay_pos.y); 

    printf( "\t  .priority\t\t(%d)\n",
      t->priority);
    printf( "\t  .type\t\t\t\"%s\"\n",
      t->type==IMM?"IMM":
      t->type==EXP?"EXP":
      t->type==DET?"DET":
      t->type==GEN?"GEN":
      "???");
    printf( "\t  .delay_type\t\t\"%s\"\n",
      t->delay_type==DELAY?"DELAY":
      t->delay_type==DELAYPAR?"DELAYPAR":
      t->delay_type==MARKDEP?"MARKDEP":
      t->delay_type==PMFDEF?"PMFDEF":
      "???");
    /* graphics*/
    printf( "\t  .orient\t\t\"ORIENT_%s\"\n",
      t->orient==ORIENT_0?"0":
      t->orient==ORIENT_45?"45":
      t->orient==ORIENT_90?"90":
      t->orient==ORIENT_M45?"M45":
      "???");
    printf( "\t  .inf_server\t\t\"%s\"\n",
      t->inf_server?"YES":"NO");
    printf( "\t  .f_policy\t\t\"R%s\"\n",
      t->f_policy==RS?"S":
      t->f_policy==RA?"A":
      t->f_policy==RE?"E":
      "???");
    /* Bug in show_net */
    if( (t->type == IMM) && (t->md_enable != NULL)) {
	          printf( "\t  .md_enable\t\t\"%s\"\n", t->md_enable); 
    } 
	 
    if( t->delay_type == DELAY )
      printf( "\t  .delay.value\t\t\"%g\"\n",
        t->delay.value);
    else if( t->delay_type == DELAYPAR )
      printf( "\t  .delay.delay_par\t\"%s\"\n",
        t->delay.delay_par->name);
    else
      printf( "\t  .delay.md_delay\t\t\"%s\"\n",
        t->delay.md_delay);

    printf( "\t  .phases\t<%d>\n",    t->phases);
    printf( "\t  .prob\t\t<%g>\n",   t->prob);
    printf( "\t  .group\t\t<%d>\n",   t->group);
    printf( "\t  .group_weight\t<%g>\n",   t->group_weight);
    if (t->type==GEN) {
      printf("DTP defined :\n");
      print_dtp(t);
    }

    printf( "\t   %d inarcs:\n", t->num_inarcs );
    for(a=0;a<t->num_inarcs;a++) {

      printf( "\t\t%2d.place\t\t\"%s\"\n",
        a, t->in_arc[a].place->name);
      printf( "\t\t  .r_index\t<%d>\t\"%s\"\n",
          t->in_arc[a].r_index, net->p_ranks[t->in_arc[a].r_index]->name);


      /* graphics
      printf( "\t\t  .point (%d)\t\t",
        t->in_arc[a].num_pts );
      for(c=0;c<t->in_arc[a].num_pts;c++) {
        printf( "(%lg;%lg) ",
          t->in_arc[a].point[c].x, t->in_arc[a].point[c].y);
      } /* for */

      if(t->in_arc[a].is_value)
        printf( "\t\t  .multipl.value\t<%d>\n",
          t->in_arc[a].multipl.value);
      else
        printf( "\t\t  .multipl.md_mult\t\"%s\"\n",
          t->in_arc[a].multipl.md_mult);
        printf( "\t\t  .md_id = %d\n",
          t->in_arc[a].md_id);
    } /* for */
    
    printf( "\t   %d outarcs:\n", t->num_outarcs );
    for(a=0;a<t->num_outarcs;a++) {

      printf( "\t\t%2d.place\t\t\"%s\"\n",
        a, t->out_arc[a].place->name);
      printf( "\t\t  .r_index\t<%d>\t\"%s\"\n",
          t->out_arc[a].r_index, net->p_ranks[t->out_arc[a].r_index]->name);

      /* graphics
      printf( "\t\t  .point (%d)\t\t",
        t->out_arc[a].num_pts );
      for(c=0;c<t->out_arc[a].num_pts;c++) {
        printf( "(%lg;%lg) ",
          t->out_arc[a].point[c].x, t->out_arc[a].point[c].y);
      } /* for */
      if(t->out_arc[a].is_value)
        printf( "\t\t  .multipl.value\t<%d>\n",
          t->out_arc[a].multipl.value);
      else
        printf( "\t\t  .multipl.md_mult\t\"%s\"\n",
          t->out_arc[a].multipl.md_mult);
        printf( "\t\t  .md_id = %d\n",
          t->out_arc[a].md_id);
    } /* for */
    
    printf( "\t   %d inharcs:\n", t->num_inharcs );
    for(a=0;a<t->num_inharcs;a++) {

      printf( "\t\t%2d.place\t\t\"%s\"\n",
        a, t->inh_arc[a].place->name);
      printf( "\t\t  .r_index\t<%d>\t\"%s\"\n",
          t->inh_arc[a].r_index, net->p_ranks[t->inh_arc[a].r_index]->name);

     /* graphics
     printf( "\t\t  .point (%d)\t\t",
        t->inh_arc[a].num_pts );
      for(c=0;c<t->inh_arc[a].num_pts;c++) {
        printf( "(%lg;%lg) ",
          t->inh_arc[a].point[c].x, t->inh_arc[a].point[c].y);
      } /* for */
      if(t->inh_arc[a].is_value)
        printf( "\t\t  .multipl.value\t<%d>\n",
          t->inh_arc[a].multipl.value);
      else
        printf( "\t\t  .multipl.md_mult\t\"%s\"\n",
          t->inh_arc[a].multipl.md_mult);
        printf( "\t\t  .md_id = %d\n",
          t->inh_arc[a].md_id);
    } /* for */
  } /* for */
    

  printf( "%d rewards:\n", net->num_rewards );
  for(i=0;i<net->num_rewards;i++) {
    printf( "\t%2d.name\t\t\t\"%s\"\n",
      i, net->reward[i].name);
    printf( "\t  .def\t\t\t\"%s\"\n",
      net->reward[i].def);
  } /* for */
}


/*
 Read the tokens bounds from <net_name.BND> and store them into the <net>
*/
void read_bounds(net)
NetDescr *net;
{
  char bnd_name[255];
  FILE *bnd_file;
  int i;
  long dummy1, dummy2; /*large dummies */

  sprintf(bnd_name,"%s.BND", net->netname);   

#ifdef LOADDEBUG
  printf("Reading upper token-bounds from %s .\n\n", bnd_name);
#endif

  if ((bnd_file = fopen(bnd_name, "r")) == NULL) {
      fprintf(stdout,"ERROR: in file %s \n", bnd_name);
      net->error_func("Could not open file for reading.\n",TRUE);
  }

  for(i=0;i<net->num_places;i++) {
    fscanf(bnd_file, "%ld %ld\n", &dummy1, &dummy2);
    net->place[i].is_bound = (Bool) dummy1;

    if (dummy2 > MAX_TOKEN) {
      fclose(bnd_file);
      fprintf(stdout,"ERROR: in place %s: %ld > %d\n", 
             net->place[i].name, dummy2, MAX_TOKEN);  
      net->error_func("Max. number of tokens in bound exceeded.\n",TRUE);
    }
    else net->place[i].bound = (word) dummy2;

#ifdef LOADDEBUG
    printf("%d %d\n", net->place[i].is_bound, net->place[i].bound);
#endif
  }

  fclose(bnd_file);
}

/* 
 Add indices to places and transitions. 
*/
void index_pt(net)
NetDescr *net;
{
 int i;
 for(i=0; i<net->num_places; i++) 
      net->place[i].index = i;

 for(i=0;i<net->num_trans;i++) 
      net->trans[i].index = i;
}

/*
 Assgin initial marking to places.
*/
void init_marking(net)
NetDescr *net;
{
 long i, tmp;

 for(i=0; i<net->num_places; i++) { 
    if( net->place[i].is_value )
        tmp = net->place[i].marking.value;
    else
        tmp = net->place[i].marking.markpar->value;
        
    if (tmp > MAX_TOKEN) {
      fprintf(stdout,"ERROR: in place %s: %ld > %d\n", 
	     net->place[i].name, tmp, MAX_TOKEN);  
      net->error_func("Max. number of tokens in m_0 exceeded.\n",TRUE);
    }
    else 
      net->place[i].m_0 = (word) tmp;
 }
}


/*
 Explain Specification of geom. phase distributions.
 Sub-function of init_phasedistr.
*/
void geom_usage(net, t, real_delay)
NetDescr *net;
Transition *t;
double real_delay;
{
  printf(
  "\n\nError: Wrong geometric distribution for transition \"%s\":\n", t->name);

  if (t->prob == 0.0)
    printf("  computed illegal marking dependent delay: %lg <= %d * 1/%lu \n",
	   	  real_delay, t->phases, but_step );
  else
    printf("  delay: %lg <= %d * 1/%lu \n", real_delay, t->phases, but_step );

  printf(
  "Delay must be larger than (number of phases * basic time-step).\n");

  /* old message
  printf("Phase number=%d (default = 1) must be smaller than delay=%g.\n\n", 
	 t->phases, ((double)(t->phases))/(t->prob) );
  
  printf("Geom(%d,%g) = Geom(phases, probability) ",t->phases, t->prob); 

  printf("\t where probability = phases/delay\n\n");
  
  printf("Correct example: phase=4 and delay=8  =>  Geom(4, 0.5)\n");
 
  printf(
  "\t transition fires after 4 phases with probability 0.5 where 0<0.5<1\n\n");  */
 
  net->error_func("Wrong Geometric transition.\n",TRUE);
  
}



/*
 Assign phase distributions to transitions, as discrete firing times.
*/
void init_phasedistr(net)
NetDescr *net;
{
 int i;

 unsigned long int int_delay;
 double real_delay;

 Transition *t;

 time_l *expoly;
 char buf[256];

 Bool MD_delay;

  net->num_timed_trans = net->num_trans;

  for(i=0;i<net->num_trans;i++) {
    t=&net->trans[i];


    /* ONLY delay values allowed, yet */
/*     if ( t->delay_type == PMFDEF ) { */
/*       fprintf(stdout,"Error: transition %d: \"%s\",\t", i, t->name); */
/*       fprintf(stdout,"pmf delay: \"%s\"\n", t->delay.md_delay); */
/*       net->error_func( */
/* 	 "General transition delays not yet implemented.\n",TRUE); */
/*     } */
/*     else */ 

      if ( (t->delay_type != DELAY) && (t->delay_type != PMFDEF) ) {
	if ( t->delay_type != DELAYPAR ) {
	  /* we have a MARKDEP delay */
	  if ( t->type == IMM ) {
	    fprintf(stdout,"\nImmediate transition %d: \"%s\",\t", i, t->name);
	    fprintf(stdout,"weight: \"%s\"\n", t->delay.md_delay);
	    net->error_func("Marking dependent weight ignored in DDSPN.\n", FALSE);
	  }
	  else if ( t->type != EXP){
	    fprintf(stdout,"\nError: transition %d: \"%s\",\t", i, t->name);
	    fprintf(stdout,"delay: \"%s\"\n", t->delay.md_delay);
	    net->error_func(
	    "Marking dependent delay only for Geometric transition implemented.\n", TRUE);
	  }
	}
	else if ( !(t->delay.delay_par->is_value) ) {
 	  /* we have a marking dependent DELAYPAR */
	  if ( t->type == IMM ) {
	    fprintf(stdout,"\nImmediate transition %d: \"%s\",\t", i, t->name);
	    fprintf(stdout,"weight: \"%s\"\n", t->delay.delay_par->delay.md_delay);
	    net->error_func("Marking dependent weight ignored in DDSPN.\n", FALSE);
	  }
	  else if ( t->type != EXP){
	    fprintf(stdout,"\nError: transition %d: \"%s\",\t", i, t->name);
	    fprintf(stdout,"delay: \"%s\"\n", t->delay.delay_par->delay.md_delay);
	    net->error_func(
	    "Marking dependent delay only for Geometric transition implemented.\n", TRUE);
	  }
	}
      }

    /* check priorities */
    if ( t->priority < 0 ) {
        fprintf(stdout,"ERROR: in transition \"%s\": %d <= 0\n", 
		t->name, t->priority); 
        net->error_func("Priority must be a positive integer.\n",TRUE);
    } 

    switch( t->type ) {
    case IMM:
      t->prob = 1;
      t->phases = 0;
      net->num_timed_trans--;
      break;      
    case DET:    
      t->prob = 1;
      if (t->delay_type ==  DELAY)
        real_delay = t->delay.value;
      else
	real_delay = t->delay.delay_par->delay.value;

      real_delay *= but_step;

      int_delay = (unsigned long int) real_delay;

      if ( (real_delay - ((double)int_delay) ) > DEB_accuracy  ) {
	fprintf(stdout,
        "ERROR: in transition \"%s\", delay = %g,  basic time-step = 1/%lu \n", 
		t->name, real_delay/but_step, but_step); 
	net->error_func(
        "Deterministic delay must be a multiple of the basic time-step.\n",TRUE);
      }

      if (int_delay > MAX_PHASE) {
	if (but_step == 1) {
	  fprintf(stdout,"ERROR: in transition \"%s\", delay %ld > %d\n", 
		  t->name, int_delay, MAX_PHASE);
	  net->error_func("Max. deterministic delay exceeded.\n",TRUE);
	}
	else {
	  fprintf(stdout,
	  "ERROR: in transition \"%s\", delay = %g, basic time-step = 1/%lu \n", 
                t->name, real_delay/but_step, but_step); 
           net->error_func(
           "Rescaling of deterministic firing time exceeds maximum # of phases.\n",
             TRUE);
	}
      }
      else 	/* everything seems OK */
       t->phases = (word) int_delay;


       /* total number of DET-RFT states */
       t->bound = t->phases;
       t->is_bound = TRUE;	/* phases are in principle always bound */

       /* sometimes an unbound representation helps saving space */
#ifdef DID_NOT_WORK_WELL 
       if ( (Embedding == TRUE) ) {
	 if ( (SaveSpace == TRUE) && (t->bound > 5) ) 
	   /* 1 - 5 take less space when bounded */
	   t->is_bound = FALSE;
	 if ( (SaveSpace != TRUE) && (t->bound > 20) )
	   /* 1 - 20 bounded saves more time, see min_ph_entry in genrrg_tree.c */
	   t->is_bound = FALSE;
       }
#endif
      /* begin replacing DID_NOT_WORK_WELL */
       if ( (Embedding == TRUE) ) {
	 if (SaveSpace == TRUE) 
	   /* min unbounded size is always 1, see min_ph_entry in genrrg_tree.c */
	   t->is_bound = FALSE;
	 else if (t->bound > 5)
	   /* 1 - 5 take less space when bounded, but saves time */
	   t->is_bound = FALSE;
       }
      /* end replacing DID_NOT_WORK_WELL */

      break;
    case EXP:
      if (t->phases > MAX_PHASE) {
	fprintf(stdout,"ERROR: in transition \"%s\": %d > %d\n", 
		t->name, t->phases, MAX_PHASE); 
	net->error_func("Max. number of geom. phases exceeded.\n",TRUE);
      }

      MD_delay = FALSE;
      if ( t->delay_type == MARKDEP ) MD_delay = TRUE;
      if ( t->delay_type == DELAYPAR ) {
	if ( !(t->delay.delay_par->is_value) )  MD_delay = TRUE;
      }

      if ( MD_delay == TRUE )
	t->prob = 0.0; /* used to trigger computation of firing prob at run-time! */
      else {
        if ( t->delay_type == DELAY) 
	  real_delay = t->delay.value;
	if ( t->delay_type == DELAYPAR )
	  real_delay = t->delay.delay_par->delay.value;

	t->prob = ((double) t->phases)/ (real_delay * ((double)but_step)  );
	if (t->prob >= 1) geom_usage(net, t, real_delay);
      }


      /* total number of GEOM-RFT states */
      t->bound = t->phases;
      t->is_bound = TRUE;	/* phases are in principle always bound */


      /* sometimes an unbound representation helps saving space */
#ifdef DID_NOT_WORK_WELL 
      if ( (Embedding == TRUE) ) {
	if ( (SaveSpace == TRUE) && (t->bound > 5) ) 
	  /* 1 - 5 take less space when bounded */
	  t->is_bound = FALSE;
	if ( (SaveSpace != TRUE) && (t->bound > 20) )
	  /* 1 - 20 bounded saves more time, see min_ph_entry in genrrg_tree.c */
	  t->is_bound = FALSE;
      }
#endif
      /* begin replacing DID_NOT_WORK_WELL */
       if ( (Embedding == TRUE) ) {
	 if (SaveSpace == TRUE) 
	   /* min unbounded size is always 1, see min_ph_entry in genrrg_tree.c */
	   t->is_bound = FALSE;
	 else if (t->bound > 5)
	   /* 1 - 5 take less space when bounded, but saves time */
	   t->is_bound = FALSE;
       }
      /* end replacing DID_NOT_WORK_WELL */

      break;      
      case GEN: 
	  sprintf(buf,"%s.pmf",net->netname);
	  if( (expoly = parse_pmf(buf,t->name)) == NULL ) {
	   fprintf(stdout,"ERROR: in transition '%s'\n", t->name);
	   net->error_func("No PMF defined\n",TRUE);
	  }

#ifdef PMFDEBUG
	  else {
	    printf("\nPmf defined for transition '%s'\n", t->name);
	    print_expoly(expoly);}
#endif

 	  /*Anpassen der Verteilung an den underlying Timestep*/ 
/* 	  if (but_step != 1) adjust_expoly(expoly,but_step); */

/* #ifdef PMFDEBUG	     */
/* 	  if (but_step != 1) */
/* 	  {	     */
/* 	    printf("\n New pmf for transition '%s'\n", t->name); */
/* 	    print_expoly(expoly); */
/* 	  } */
/* #endif */

	  /*erzeugen der dtp und bestimmen der Phasenanzahl*/
	  if(!create_dtp(expoly,t,but_step)) {
	    fprintf(stdout,"ERROR: in transition '%s'\n", t->name);
	    net->error_func("No dtp defined.\n",TRUE);
	  }
#ifdef _PMFDEBUG
	  else print_dtp(t);
#endif
	  
	  /* total number of GEN-RFT states */
	  t->bound = t->phases;
	  t->is_bound = TRUE;	/* phases are in principle always bound */
	  
	  /* sometimes an unbound representation helps saving space */
#ifdef DID_NOT_WORK_WELL 
	  if ( (Embedding == TRUE) ) {
	    if ( (SaveSpace == TRUE) && (t->bound > 5) ) 
	      /* 1 - 5 take less space when bounded */
	      t->is_bound = FALSE;
	    if ( (SaveSpace != TRUE) && (t->bound > 20) )
	      /* 1 - 20 bounded saves more time, see min_ph_entry in genrrg_tree.c */
	      t->is_bound = FALSE;
	  }
#endif
	  /* begin replacing DID_NOT_WORK_WELL */
	  if ( (Embedding == TRUE) ) {
	    if (SaveSpace == TRUE) 
	      /* min unbounded size is always 1, see min_ph_entry in genrrg_tree.c */
	      t->is_bound = FALSE;
	    else if (t->bound > 5)
	      /* 1 - 5 take less space when bounded, but saves time */
	      t->is_bound = FALSE;
	  }
	  /* end replacing DID_NOT_WORK_WELL */
	  break;

    /*   example for more distributions 
    case UNIFORM:
      ... to be written ... for UNIF(t1,t2,omega)
      Note: phases will be here (mis-)used for omega
      / total number of UNIFORM-RFT states /
      t->bound = #unif_states; (eigentliche phasen)
      bnd/unbnd decision depends on t1 and omega
      break;      
    */
     default:
      fprintf(stdout,"Error: transition %d: \"%s\"\n", i, t->name);
      net->error_func("Unknown discrete phases transition type.\n",TRUE);
      break;
    }
  }
}



/* 
 Initialize locally defined weights in <net> 
*/
 void init_weights(net) 
 NetDescr *net; 
{
  int i;
  Transition *t;

  /* immediates have delays as weights and may have delay-parameters too */
  for(i=0;i<net->num_trans;i++){
    t = &net->trans[i];
    if (t->type == IMM)
	if (t->delay_type == DELAY)
	     t->group_weight = t->delay.value;
	else t->group_weight = t->delay.delay_par->delay.value;
  }
  /* initialize transitions with no group specification (=DEFAULT_GRP) */
  for(i=0;i<net->num_trans;i++){
    t = &net->trans[i];

    if (t->group == DEFAULT_GRP) {
      t->group_weight = DEFAULT_GRP_WEIGHT;
    }

    /* btw: check all for illegal weight */
    if (t->group_weight <= 0) {
         fprintf(stdout,"ERROR: in transition \"%s\": weight %g <= 0\n",
                t->name, t->group_weight);
         net->error_func("Group weights must be greater zero.\n",TRUE);
    }
    if (t->group < 0) {
         fprintf(stdout,"ERROR: in transition \"%s\": group %d < 0\n",
                t->name, t->group);
         net->error_func("Group-index must be greater zero.\n",TRUE);
    }

  }
}



/*
 Reorder places and transitions according to bounds resp. phases.
*/
void sort(net)
NetDescr *net;
{
 int p, t, r, timed, imm;

 Place *nxt_p, *tmp_p;
 Transition *nxt_t, *tmp_t;
   
#ifdef LOADDEBUG
 printf("Re-ordering places and transitions\n");
#endif

 net->p_ranks = (Place **)nMalloc(net->num_places*sizeof(Place *));

 for(p=0; p<net->num_places; p++) {
      nxt_p = &net->place[p];
      for(r=0; r<p; r++)
           if (net->p_ranks[r]->bound > nxt_p->bound ) {
	     /* swap */
             tmp_p = net->p_ranks[r];
             net->p_ranks[r] = nxt_p;
             nxt_p = tmp_p;
	   }
      net->p_ranks[p] = nxt_p;
    }

 /* write rank - indices to net (sorts of backward chaining) */
 for(p=0; p<net->num_places; p++) net->p_ranks[p]->r_index = p;

 /* sort timed transitions & append immediate transitions */
 net->t_ranks = (Transition **)nMalloc(net->num_trans*sizeof(Transition *));

 for(t = timed = 0, imm = net->num_trans-1; t<net->num_trans; t++) {
      nxt_t = &net->trans[t];
      if (nxt_t->type != IMM) {
        for(r=0; r<timed; r++)
           if (net->t_ranks[r]->bound > nxt_t->bound) {
	     /* swap */
             tmp_t = net->t_ranks[r];
             net->t_ranks[r] = nxt_t;
             nxt_t = tmp_t;
        }
        net->t_ranks[timed++] = nxt_t;
      }
      else
        net->t_ranks[imm--] = nxt_t;
 }

 /* write rank - indices to net (sorts of backward chaining) */
 for(t=0; t<net->num_trans; t++)  net->t_ranks[t]->r_index = t;

}


/*
 Sub-function of init_arcs.
*/
void index_arc(net, t, md_id, s, first, count)
NetDescr *net;
int t;
int *md_id;
char *s;
Arc *first;
int count;
{
 int a;
 char inp_place[255];

 if (count > net->num_places) {
   fprintf(stdout,"ERROR: %s-arcs of transition %s: %d > %d\n", 
	      s, net->trans[t].name, count, net->num_places);
   net->error_func("No. of arcs greater than no. of places.\n",TRUE);
 }

 for(a=0;a<count;a++) {
	/* make directly reachable and more efficient */
   first[a].r_index = first[a].place->r_index; 


   if (!first[a].is_value){
      first[a].md_id = *md_id;
      (*md_id)++; /* running index for marking dependent arc definitions */
/* OLD
   only special case of md_input_arcs for flushing tokens allowed 
     sprintf(inp_place,"");
     sprintf(inp_place,"#%s;", first[a].place->name);

     if ((strcmp(s, "Input")) || (strcmp(first[a].multipl.md_mult, inp_place))){
       fprintf(stdout,
       "ERROR: %s-arc between transition \"%s\" and \"place\" %s, value: %s\n", 
	 s, net->trans[t].name, first[a].place->name, first[a].multipl.md_mult);
       net->error_func("Marking dependent arcs not implemented.\n",TRUE);
     }
*/
   }
   else {
      first[a].md_id = 0; /* dummy default */
      if (first[a].multipl.value > MAX_TOKEN) {
          fprintf(stdout,"ERROR: %s-arc of transition %s: %d > %d\n", 
	      s, net->trans[t].name, first[a].multipl.value , MAX_TOKEN);
          net->error_func("Max. arc-multiplicity exceeded.\n",TRUE);
      }

   }
  }
}


/*  
 Rank-index arcs
*/
void init_arcs(net)
NetDescr *net;
{
 int t;
 int md_id; /* running index for marking dependent arcs */

 md_id=1;

  for(t=0; t<net->num_trans; t++) {
   index_arc(net,t, &md_id, "Input", net->trans[t].in_arc,  net->trans[t].num_inarcs);
   index_arc(net,t, &md_id, "Output", net->trans[t].out_arc, net->trans[t].num_outarcs);
   index_arc(net,t, &md_id, "Inhibitor", net->trans[t].inh_arc, 
	      net->trans[t].num_inharcs);
  }
}


/*  
 Create sparse incidence matrix 
*/
void init_inc_matrix(net)
NetDescr *net;
{
 int a, t, p;
 long *marking;


 	/* no of rows = no of transitions */
  net->Inc_Matrix = (Sparse_Row *)nMalloc(net->num_trans*sizeof(Sparse_Row));
       
  marking = (long *)nMalloc(net->num_places*sizeof(long));

#ifdef LOADDEBUG
	printf("\nIncidence Matrix:\n");
#endif

  for(t=0; t<net->num_trans; t++) {
    Sparse_Row *row = &net->Inc_Matrix[t];

	/* fire transition on empty marking */
    for(p=0; p<net->num_places; p++) marking[p]=0;

    for(a=0; a<net->trans[t].num_inarcs; a++) {
      p = net->trans[t].in_arc[a].r_index;
      marking[p] = marking[p] - net->trans[t].in_arc[a].multipl.value;
    };
    for(a=0; a<net->trans[t].num_outarcs; a++) {
      p = net->trans[t].out_arc[a].r_index;
      marking[p] = marking[p] + net->trans[t].out_arc[a].multipl.value;
    };

    	/* count number of entries in row from marking */
    for(row->no_columns = 0, p=0; p<net->num_places; p++) 
      if (marking[p] != 0) row->no_columns++;
	/* allocate memory for row-entries */ 
    row->column = (Column_Entry *)nMalloc(row->no_columns*sizeof(Column_Entry));
    	/* store entries in row from marking */
    for(a=0, p=0; p<net->num_places; p++) 
      if (marking[p] != 0) {
	row->column[a].entry = marking[p];
	row->column[a++].r_index = p;
      }

#ifdef LOADDEBUG
    for(p=0; p<net->num_places; p++) printf("%3ld   ",marking[p]);
    printf("\n");
#endif

  }

#ifdef LOADDEBUG
  printf("\n");

  for(t=0; t<net->num_trans; t++) {
   Sparse_Row *row = &net->Inc_Matrix[t];

   printf("T%d, C  : ",t);
   for(a=0; a<row->no_columns; a++) 
	printf("%d:%2ld   ",row->column[a].r_index ,row->column[a].entry );

   printf("\n    in : ");
   for(a=0; a<net->trans[t].num_inarcs; a++) 
     printf("%d:%2d   ",
	 net->trans[t].in_arc[a].r_index,
	 net->trans[t].in_arc[a].multipl.value );

   printf("\n    out: ");
   for(a=0; a<net->trans[t].num_outarcs; a++) 
     printf("%d:%2d   ",
	 net->trans[t].out_arc[a].r_index,
	 net->trans[t].out_arc[a].multipl.value );
   printf("\n");
  }
  printf("\n");
#endif

  Free(marking);
}


/*
 * My Error message:
 */
int My_Error(msg, fatal)
char *msg;
int fatal;
{
  /* print to stdout, because stderr does not appear in TimeNET monitor */
   if( fatal )
   {
      fprintf(stdout,"FATAL: %s\n", msg);
      exit( fatal );
   }
   else
      fprintf(stdout,"WARNING: %s\n", msg);

   return( 0 );
}


NetDescr *load_dtn(net_name)
char *net_name;
{
  NetDescr *net;

  net = load_net(net_name, My_Error);

  /* check max no of places & transitions */
  if (net->num_places > MAX_PLACE) {
     fprintf(stdout,"ERROR: %d > %d\n", net->num_places, MAX_PLACE);  
     net->error_func("Max. number of places exceeded.\n",TRUE);
  };
  if (net->num_trans > MAX_TRANS) {
     fprintf(stdout,"ERROR: %d > %d\n", net->num_trans, MAX_TRANS);  
     net->error_func("Max. number of transitions exceeded.\n",TRUE);
  };

  /* read the token bounds */
  read_bounds(net);

  /* add indices to places and transitions */
  index_pt(net);

 /*  Assgin initial marking to places. */
  init_marking(net);

 /*  Assign discrete firing times */
  init_phasedistr(net);

 /*  Initialize locally defined weights */
  init_weights(net);

 /*  Determine ranks (re-ordering) */
  sort(net);

 /*  Rank-index arcs */
  init_arcs(net);

 /* Create sparse incidence matrix */
 /* Attention: Inc_Matrix not used. Moreover, causes problems when arcs mark. dep. 
 init_inc_matrix(net); 
 */

#ifdef LOADDEBUG
	printf("Here's the Tnet %s\n", net_name);
  	show_dtnet(net);
#endif

 return(net);

} 


