/***************************************************************************/
/* Program:        solve                                                   */
/*                                                                         */
/* File:           derive_disp.c                                           */
/*                                                                         */
/* Purpose:        Prepare transient solution for output.                  */
/*                                                                         */
/* adapted from TransientSolution/src/shared.c                             */
/***************************************************************************/

/* #define DEBUG_R */
#define BUF_SIZE    1000

#include <sys/param.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>

#include "netio.h"  

#include "transient_sol.h"


/* Import von _ERC.c file */
extern double help[];

/* export nach _ERC.c file */
extern int digits;

/* Import aus /Shared/src/netio/load_net */
extern NetDescr *load_net();
/* Import aus transient_sol */
extern int toptan;
extern int one_step;
extern NetDescr* net;
extern double *mprob;
extern int now_marking;
extern t_stat_opts stat_opts;

/* lokale variable */
static Result_Basis resbas;
static double *prob_list;
static int help_max;

/******************************************************************************
* open_solve: oeffnet .tempcurfes file und eventuell x fenster
 ******************************************************************************/
FILE *open_solve(netname,num,minX,maxX)
char *netname;
int num;
double minX,maxX;
{
  FILE *fcurves;
  char curves_file[MAXPATHLEN];
  int i;
  char command[2*MAXPATHLEN];
  FILE *ferc;
  char erc_file[MAXPATHLEN];
  char pathless_netname[BUF_SIZE];
  char ENV[MAXPATHLEN];
  char tnethome[MAXPATHLEN];
  String tnet;
  String modelpath;
  char path[MAXPATHLEN];
  char buf[BUF_SIZE];
  int  equal;
  char opts_stat[40];

  sprintf(ENV,"MODELDIR");
  if ((modelpath = (String)getenv("MODELDIR")) == NULL)
  {
    printf("Environment variable %s not set; can't specified model\n",ENV);
    exit(-1);
  }
  sprintf(ENV,"TNETHOME");
  if ((tnet = (String)getenv("TNETHOME")) == NULL)
  {
    printf("Environment variable %s not set; can't specified model\n",ENV);
    exit(-1);
  }
  sprintf(path,modelpath);
  sprintf(tnethome,tnet);
   
  /* File zum schreiben oeffnen */
  sprintf(curves_file,"%s.tempcurves",netname);
  
  equal = strlen(path);
  i = 0;
  while (netname[i+equal+1] != NULL)
  { 
    pathless_netname[i] = netname[i + equal+1];
    i++;
  }
  pathless_netname[i] =NULL;
  if (!(fcurves=fopen(curves_file,"w"))) 
  {
    fprintf(stderr,"Unable to open file '%s' for writing.\n", curves_file);
    exit(-1);
  }

  resbas.maxY = (double *)malloc(sizeof(double)*(net -> num_rewards));
  resbas.minY = (double *)malloc(sizeof(double)*(net -> num_rewards));

  /* header schreiben */
  fprintf(fcurves,"# file %s containing solution of TransSolve.\n#\n",curves_file);

  fprintf(fcurves,"# number of x values: a\n");
  fprintf(fcurves,"%d\n", num);

  fprintf(fcurves,"# number of defined reward measures: b\n");
  fprintf(fcurves,"%d\n", net -> num_rewards);

  fprintf(fcurves,"# list of reward measure names:\n");
  for(i=0;i<net -> num_rewards;i++)
    fprintf(fcurves,"%s ", net->reward[i].name);
  fprintf(fcurves,"\n");

  fprintf(fcurves,"# first x value:\n");
  fprintf(fcurves,"%.*g\n", digits, minX/stat_opts.scale);

  fprintf(fcurves,"# last x value:\n");
  fprintf(fcurves,"%.*g\n", digits, maxX/stat_opts.scale);

  fprintf(fcurves,"# maximum y value:\n");
  for(i=0;i<net -> num_rewards;i++)
  {
    resbas.maxY[i] = 0.0;
  }
  fprintf(fcurves,"\n");

  fprintf(fcurves,"# minimum y value:\n");
  for(i=0;i<net -> num_rewards;i++)
  {
    resbas.minY[i] = 1.0e300;
  }
  fprintf(fcurves,"\n");

  fprintf(fcurves,"# plotfactor for underlying time_step = 1 ? : c\n");
  fprintf(fcurves,"%d\n",one_step );   
 
  fprintf(fcurves,"# results: c times a x b matrices\n");
  fflush(fcurves);
  sprintf(opts_stat,"%s %s. %d %d %lg",stat_opts.emb,stat_opts.sol,stat_opts.maxiter,stat_opts.scale,stat_opts.prec);
  /* start Backend */
  if(fork()==0)
  {  
    sprintf(command,"%s/DTSPN/TransSol/bin/Backend %s.tempcurves %d %s -geometry 820x410+200+100", tnethome, pathless_netname,one_step,opts_stat);
    /*sprintf(command,"%s/DTSPN/TransSol/bin/Backend %s.tempcurves %d %s -geometry 720x360+200+100", tnethome, pathless_netname,one_step,opts_stat);*/
    system(command);    
    exit(0);
  }
  /* allocate list of probabilities*/
  if ( (prob_list = (double *)malloc((toptan+1) * sizeof(double))) == NULL )
  {
    fprintf(stderr,"Unable to allocate list of probabilities.\n");
    exit(1);
  }
  /* _ERC.c benutzt help array, und ich muss dessen groesse wissen, um es in jedem Durchgang loeschen zu koennen */
  sprintf(erc_file,"%s/%s_ERC.c",modelpath,pathless_netname);
  if (!(ferc=fopen(erc_file,"r"))) 
  {
    fprintf(stderr,"Unable to open file '%s'.\n", erc_file);
    exit(-1);
  }
  do
  {
    fgets(buf,BUF_SIZE,ferc);
  } while( sscanf(buf,"double help[%d];",&help_max)!=1 );

  return(fcurves);
}

/******************************************************************************
 * update_solve: erzeugt prob_list; ruft fkt. der _ERC.c Komponente auf, laesst
 *               also .RESULTS file erzeugen und kopiert diese ins .tempcurves file
 ******************************************************************************/
void update_solve(netname,ftmp,res_vec,t,minX,maxX,prec,status)
char *netname;
FILE *ftmp;
double *res_vec;
double t;
double minX;
double maxX;
double prec;
int status;
{
   char res_file[MAXPATHLEN];
   FILE *res_fp;
   int i;
   char dummy[256]; /* reward measure name */
   double d;        /* Ergebnis des measures */
   char tnethome[MAXPATHLEN];
   char ENV[MAXPATHLEN];
   String tnet;
   String modelpath;
   char path[MAXPATHLEN];
   int equal;


   for (i=0;i<toptan;i++)
   {
     prob_list[i+1] = mprob[i];
   }
   for(i=0;i<help_max;i++)
     help[i] = 0.0;

  sprintf(res_file,"%s.RESULTS",netname);

  if ((res_fp = fopen(res_file, "w")) == NULL)
  {
    printf("Can't open file %s for writing\n", res_file);
    exit(1);
  }
  if (toptan > 0)
  {
    /*computeprob(res_vec);    z.Zt. nicht noetig*/
    for (now_marking = 1; now_marking <= toptan; now_marking++)
      sum_prob(prob_list[now_marking]);
    calc_results(res_fp); 
  } 
  fclose(res_fp);


  /* copy results to .tempcurves */
  /* open .RESULT File for reading */
   if( (res_fp = fopen(res_file,"r")) == NULL )
   {
     fprintf(stderr,"Unable 1 to open file '%s' for reading.\n", res_file);
     exit( 1 );
   }
   /* print status meldung */
   fprintf(ftmp,"#status: %d %% \n",(int)(100.0*(t-minX)/ (maxX-minX)));

   fprintf(ftmp,"%.*g ", digits, t/stat_opts.scale);  
   for(i=0; i<net -> num_rewards; i++)
   {
     fscanf(res_fp,"%s = %lf",dummy,&d);
     if(resbas.maxY[i]<d)
       resbas.maxY[i] = d;
     if(resbas.minY[i]>d)
       resbas.minY[i] = d;
     fprintf(ftmp,"%.*g ", digits, d);
   }
   fprintf(ftmp,"\n");
   fflush(ftmp);
   fclose(res_fp);

   fflush(ftmp);
}

 
/******************************************************************************
 * close_solve: schreibt .curves file mit neuem minY und maxY 
 ******************************************************************************/
void close_solve(netname,ftempc)
char *netname;
FILE *ftempc;

{
  FILE *fcurves;
  char file_name[MAXPATHLEN];
  char buf[BUF_SIZE];
  int i;

net = load_net(netname);
  fclose(ftempc);

  sprintf(file_name,"%s.tempcurves",netname);
  if (!(ftempc=fopen(file_name,"r"))) 
  {
    fprintf(stderr,"Unable 2 to open file '%s' for reading.\n", file_name);
    exit(-1);
  }

  sprintf(file_name,"%s.CURVES",netname);
  if (!(fcurves=fopen(file_name,"w"))) 
  {
    fprintf(stderr,"Unable to open file '%s' for writing.\n", file_name);
    exit(-1);
  }

  fgets(buf,BUF_SIZE,ftempc);
  fprintf(fcurves,"# file %s containing solution of TransSolve.\n",file_name);

  /* dirty (besser: zeile suchen mit strcmp) */
  for(i=0;i<12;i++)  /**/
  {
    fgets(buf,BUF_SIZE,ftempc);
    fputs(buf,fcurves);
  }

  fgets(buf,BUF_SIZE,ftempc);
  for(i=0;i<net -> num_rewards;i++)
    fprintf(fcurves,"%.*g ", digits, resbas.maxY[i]);
  fprintf(fcurves,"\n");

  fgets(buf,BUF_SIZE,ftempc);
  fputs(buf,fcurves);

  fgets(buf,BUF_SIZE,ftempc);
  for(i=0;i<net -> num_rewards;i++)
    fprintf(fcurves,"%.*g ", digits, resbas.minY[i]);
  fprintf(fcurves,"\n");

  fgets(buf,BUF_SIZE,ftempc);
  fputs(buf,fcurves);
  fgets(buf,BUF_SIZE,ftempc);
  fputs(buf,fcurves);
  fgets(buf,BUF_SIZE,ftempc);
  fputs(buf,fcurves);

  while(!feof(ftempc))
  {
    fgets(buf,BUF_SIZE,ftempc);

    if( (!feof(ftempc)) && (buf[0]!='#') )
      fputs(buf,fcurves);
  }
  
  fclose(ftempc);
  fclose(fcurves);
}

