/*******************************************************************************
Modul:              showTransCurves.c

Purpose:            Routinen fuer plot des trans.  Verhaltens definierter measures

Project Director:   Reinhard German

Programmer:         Joerg W. Mitzlaff

History:            10.94 (Dipl.Arbeit)

                    12.96 erweitert speziell fuer zeitdisdrete Anwendungen von

		      Andrea Zisowsky (Einblenden der stationaeren Loesung)

		    03.97 Zoom-Funktion und Scalierung "uniform linear"

*******************************************************************************/

/* #define DEBUG */

#include <X11/Intrinsic.h>  /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/

#include <X11/Shell.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/Toggle.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/AsciiText.h>

#include <sys/param.h>
#include <math.h>
#include <stdio.h>
#include <string.h> 
#include <memory.h>
#include "environment.h"
#include "frontend.h"
#include "showTransCurves.h"
#include "bitmaps/toggle.bm"

#define _nameDEBUG

/* file name buffer size */
#define FNAME_SIZE  256

#define TIMEOUT          500

#define BUTTONWIDTH      120     /* alle Werte in Pixel, Erfahrungswerte */
#define BUTTONHEIGHT      25
#define INIT_GC_BREITE   820 /*720*/    
#define INIT_GC_HOEHE    410 /*360*/
#define SCHOENER_RAND    20
#define text_width       90
#define text_height      25
#define MIN_ZAHL_ABSTAND_HORI 10
#define MIN_ZAHL_ANSTAND_VERT 10
#define MAX_ZAHL_BREITE  40
#define MAX_ZAHL_HOEHE   15
#define BIG_TICS         5
#define LITTLE_TICS      3
#define ANZ_LITTLE_TICS  4
#define FONTHALBEHOEHE   6
#define FONTHALBEBREITE   2
#define TOGGLE_LEFT_MARGIN 16
#define line_width 2
#define slash '\\'
#define maxreal 10E60

/* lokale Variablen */
static XtAppContext app_context;  
static Widget topLevel;
static Pixmap toggle_pixmap;      /* Haekchen */
static Drawing_Basis drabas;      /* Daten aus .curves file */
static FILE *fcurves;
static FILE *res_fp;
static char statusline[1000];
static char *emb_opt;
static char sol_opt[2];
static long max_iter;
static int timescale;
static int stat_failed = 0;
double prec;
static int no_data_blocks;
static char *net_name;
static int one_step;
static int repeateddisplay = False;
static char *buf_sol_opt;
static int imax,imin,numX;
static int display_only = 1;
static double output;

static Display *disp;
static Colormap cmap;
static int depth;
static GC gc;

static double maxdiffX;


static Widget CpannedWin;
static Widget CmainForm;
static Widget CbuttonForm;
static Widget CmswitchButton, CmswitchMenu, *CmswitchEntry;
static Widget ClswitchButton, ClswitchMenu, ClswitchEntry[4];
static Widget CbreakButton;     /* nur waehrend des loesungsvorgangs zu benutzen */
static Widget ChelpButton;
static Widget CDisplayedTimeButton;
static Widget Cstat_analysisButton;  /* */
static Widget CstatusLabel;
static Widget CplotCore;

static Widget timePopup,
                  TimeBox, 
                      TimeForm,
                        TimeForm2,
                          TimeLabel1,
                          TimeLabel2,
                          timeSpaceLabel,
                          TimeMinValueLabel, 
                          TimeMinValueText,
                          TimeMaxValueLabel,
                          TimeMaxValueText,
                          TimeStepsizeText,
                          TimeStepsizeLabel,
                        timeCancelButton,
                        timeOKButton,
                        timeDefaultButton;
static Widget helpPopup, helpBox, helpdoneButton, helpText;


static int gc_breite=INIT_GC_BREITE; 
static int gc_hoehe=INIT_GC_HOEHE;

/* folgende Vars. gelten nur fuer akt. Kachel. */
static int ka_breite;
static int ka_hoehe;
static int offsetx;
static int offsety;
double minX;
static double absminX;
double showminX;
double maxX;
static double absmaxX;
double showmaxX;
static double minY;
static double maxY;
static double uniform_minY;
static double uniform_maxY;
static double result_static;
static int scale;
static double umaxX;     /* pixelbildgroesse / reale argumentbereichsgroesse */
static double udiffY;    /*   -||-  */
static double rmaxY;     /* aufgerundetes maxY ist aber kleiner maxY+rticstepy */
static double rminY;     /* abgerundetes maxY ist aber groesser  minY-rticstepy */
static double lrminY;    /* log10(rminY); */
static int    startx;    /* zum schnellen unrealx berechnen */
static int    starty;    /* zum schnellen unrealy berechnen */
static int    Sonderfall;


/* Prototyping */
static void initDrawingBasis();
static void help_setup();
static void initmswitch();
static void get_input();
static void spool();
static char *next_word();
static void mswitchCB();
static void lswitchCB();
static void abbrechenCB();
static void ChelpCB();
static void CDisplayedTimeCB();
static void CstatCB();         /* */
static void set_uniformMinMaxY();
static void helpDone();
static void exposeCurves();
static void redrawAll();
static void redrawKachel();
static void drawAxes();
static int unrealX();
static int unrealY();
static void drawNextStep();
static String fallback_resources[]=
{
#include "fallbacks.h"
  NULL
};


/* actions & translations for the CplotCore widget */
static XtActionsRec graphActions[1] = {{ "Crefresh", exposeCurves }};
static String graphTrans = "<Expose>: Crefresh()";



/********************************************************************************/
int main(argc,argv)     /* einzige Argument ist der volle Name der .curves File */
/********************************************************************************/
int argc;
char **argv;
{
  topLevel = XtVaAppInitialize(
                 &app_context,"transsol",
                 NULL,0,
                 &argc,argv,
                 /*NULL,*/
		 fallback_resources,
		 NULL);

  toggle_pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
			RootWindowOfScreen(XtScreen(topLevel)),
			toggle_bits,toggle_width,toggle_height);


  CpannedWin = XtVaCreateManagedWidget(
                        "CpannedWin",
		        panedWidgetClass,
		        topLevel,
		        NULL);

  CmainForm = XtVaCreateManagedWidget(
                        "CmainForm",
		        formWidgetClass,
			CpannedWin,
		 XtNshowGrip,False,
		        NULL);

  CmswitchButton = XtVaCreateManagedWidget(
                        "CmswitchButton",
		        menuButtonWidgetClass,
			CmainForm,
		        XtNlabel,"switch measure",
			XtNwidth,BUTTONWIDTH,
		       XtNheight,BUTTONHEIGHT,
                   XtNright,XtChainLeft,
		        NULL);
  CmswitchMenu = XtVaCreatePopupShell(
        		"menu",
		        simpleMenuWidgetClass,
        		CmswitchButton,
		        NULL);

  ClswitchButton = XtVaCreateManagedWidget(
        		"ClswitchButton",
		        menuButtonWidgetClass,
        		CmainForm,
        		XtNfromHoriz,CmswitchButton,
		        XtNlabel,"scale type",
			XtNwidth,BUTTONWIDTH,
		       XtNheight,BUTTONHEIGHT,
                     XtNleft,XtChainLeft,
                     XtNright,XtChainLeft,
		        NULL);
  ClswitchMenu = XtVaCreatePopupShell(
        		"menu",
		        simpleMenuWidgetClass,
        		ClswitchButton,
		        NULL);

  ClswitchEntry[0] = XtVaCreateManagedWidget(
        		"adaptive linear",
		        smeBSBObjectClass,
        		ClswitchMenu,
			XtNleftMargin,TOGGLE_LEFT_MARGIN,
		        NULL);
  XtAddCallback(ClswitchEntry[0],XtNcallback,(XtCallbackProc)lswitchCB,(XtPointer) Adaptive);
  ClswitchEntry[1] = XtVaCreateManagedWidget(
        		"absolute linear",
		        smeBSBObjectClass,
        		ClswitchMenu,
			XtNleftMargin,TOGGLE_LEFT_MARGIN,
		        NULL);
  XtAddCallback(ClswitchEntry[1],XtNcallback,(XtCallbackProc)lswitchCB,(XtPointer) Absolute);

  ClswitchEntry[2] = XtVaCreateManagedWidget(
        		"logarithmic",
		        smeBSBObjectClass,
        		ClswitchMenu,
			XtNleftMargin,TOGGLE_LEFT_MARGIN,
		        NULL);
  XtAddCallback(ClswitchEntry[2],XtNcallback,(XtCallbackProc)lswitchCB,(XtPointer) Logarithmic);

  ClswitchEntry[3] = XtVaCreateManagedWidget(
        		"uniform linear",
		        smeBSBObjectClass,
        		ClswitchMenu,
			XtNleftMargin,TOGGLE_LEFT_MARGIN,
		        NULL);
  XtAddCallback(ClswitchEntry[3],XtNcallback,(XtCallbackProc)lswitchCB,(XtPointer) Equal);

  XtVaSetValues(ClswitchEntry[0],XtNleftBitmap,toggle_pixmap,NULL);

  
 CbreakButton = XtVaCreateManagedWidget(
                        "CbreakButton",
		        commandWidgetClass,
                        CmainForm,
                        XtNfromHoriz,ClswitchButton,
 		        XtNlabel,"Done",
			XtNsensitive,False,
			XtNwidth,BUTTONWIDTH,
		       XtNheight,BUTTONHEIGHT,
                   XtNleft,XtChainRight,
                   XtNright,XtChainRight,
		        NULL);
  XtAddCallback(CbreakButton,XtNcallback,(XtCallbackProc)abbrechenCB,NULL);

  ChelpButton = XtVaCreateManagedWidget(
                        "ChelpButton",
		        commandWidgetClass,
                        CmainForm,
			XtNlabel,"Help",
                        XtNfromHoriz,CbreakButton, 
			XtNwidth,BUTTONWIDTH,
		       XtNheight,BUTTONHEIGHT,
		       XtNleft,XtChainRight,
		        NULL);
  XtAddCallback(ChelpButton,XtNcallback,(XtCallbackProc)ChelpCB,NULL);
  /* */
  CDisplayedTimeButton = XtVaCreateManagedWidget(
                         "CDisplayedTimeButton",
                         commandWidgetClass,
                         CmainForm,
                         XtNfromHoriz,ClswitchButton,
                         XtNlabel,"zoom",
                         XtNsensitive,True,
                         XtNwidth,BUTTONWIDTH,
                         XtNheight,BUTTONHEIGHT,
                         XtNleft,XtChainLeft,
                         XtNright,XtChainLeft,
                         NULL);
  XtAddCallback(CDisplayedTimeButton,XtNcallback,(XtCallbackProc)CDisplayedTimeCB,NULL);

  Cstat_analysisButton = XtVaCreateManagedWidget(
                         "Cstat_analysisButton",
                         commandWidgetClass,
                         CmainForm,
                         XtNfromHoriz,CDisplayedTimeButton,
                         XtNlabel,"stationary",
                         XtNsensitive,False,
                         XtNwidth,BUTTONWIDTH,
                         XtNheight,BUTTONHEIGHT,
                         XtNleft,XtChainLeft,
                         XtNright,XtChainLeft,
                         NULL);
  XtAddCallback(Cstat_analysisButton,XtNcallback,(XtCallbackProc)CstatCB,NULL);
                
  memset(statusline,'\0',1000);
  CstatusLabel = XtVaCreateManagedWidget(
                       "CstatusLabel",
                       asciiTextWidgetClass,
		       CpannedWin,
		       XtNheight,BUTTONHEIGHT,
                       XtNdisplayCaret,False,
                       XtNeditType,XawtextRead,
                       XtNfromVert,CmswitchButton,
 		       XtNwidth,gc_breite,	
		       XtNstring,statusline,
		       XtNjustify,XtJustifyLeft,
		 XtNshowGrip,False,
		       NULL);
  /* parse action table */
  XtAppAddActions(app_context,graphActions,XtNumber(graphActions));
  CplotCore = XtVaCreateManagedWidget(
                       "CplotCore",
		       coreWidgetClass,
		       CpannedWin,
                       XtNfromVert,CstatusLabel,
	               XtNwidth,gc_breite,
                       XtNheight,gc_hoehe,
                       XtNtranslations,XtParseTranslationTable(graphTrans),
		 XtNshowGrip,False,
		       NULL);

  
  disp = XtDisplay(topLevel);
  gc = DefaultGC(disp,0);
  depth = DefaultDepth(disp,0);
  cmap = DefaultColormap(disp,0);

  if ( (argc != 8)&&(argc != 2) )
  {
    printf("\nError: Wrong number of arguments to Backend\n");
    exit(1);
  }
  initDrawingBasis(argv[1]);
  net_name = argv[1];

  if (argc == 8)
  {
    sscanf(argv[2],"%u",&one_step);
    emb_opt = argv[3];
    buf_sol_opt = argv[4];
    strncpy(sol_opt,buf_sol_opt,2);
    sscanf(argv[5],"%d",&max_iter);
    sscanf(argv[6],"%d",&timescale);
    sscanf(argv[7],"%lg",&prec);
    display_only = 0;
  }
  /*  if (display_only)*/
  XtSetSensitive(Cstat_analysisButton,False);

  no_data_blocks = 2;

  initmswitch();

  XtRealizeWidget(topLevel);
  XtAppAddTimeOut(app_context,TIMEOUT,(XtTimerCallbackProc)get_input,NULL); 
  XtAppMainLoop(app_context);


  return 0;
}
/******************************************************************************/
static void initDrawingBasis(arg1)
/******************************************************************************/
char *arg1;
{
  int i,j;
  int dummy;
  char buffer[BUF_SIZE];
  char *bs, *be;
  String modelpath;
  char fullfile[FNAME_SIZE];
#ifdef nameDEBUG 
  printf("initDrawingBasis\n");
#endif
  /* get environment variables */
  if ((modelpath = (String)getenv(MODELDIR_ENV)) == NULL)
  {
    printf("Environment variable %s not set; can't specified model",MODELDIR_ENV);
    exit(-1);
  }

  /* open file */
  sprintf(fullfile,"%s/%s",modelpath,arg1);
  if (!(fcurves=fopen(fullfile,"r"))) 
  {
      fprintf(stderr,"Unable to open file '%s' for reading.\n",fullfile );
      exit(-1);
  }
  /* read header info */
  /* anzahl x werte */
  spool(fcurves,buffer);
  sscanf(buffer,"%d", &(drabas.numDatas));
  sscanf(buffer,"%d", &numX);

  drabas.aktData = 0;
  drabas.show_stat = False;
  drabas.derived_stat = False;

  /* anzahl measures */
  spool(fcurves,buffer);
  sscanf(buffer,"%d", &(drabas.numMeasures));

  /* allen moeglichen speicher allozieren */
  drabas.mdata = (Measure_Datas *)malloc(sizeof(Measure_Datas)*(drabas.numMeasures));
  drabas.time = (double *)malloc(sizeof(double)*(drabas.numDatas));
  for(i=0;i<drabas.numMeasures;i++)
  {
    drabas.mdata[i].Datas = (double *)malloc(sizeof(double)*(drabas.numDatas));
  }
  
  /* alle namen der measures */
  spool(fcurves,buffer);
  bs = buffer;
  for(i=0;i<drabas.numMeasures;i++)
  {
    be = next_word(bs);
    drabas.mdata[i].name = (char *)strdup(bs);
    bs = be;
  }

  /* minX wert */
  spool(fcurves,buffer);
  sscanf(buffer,"%lf", &(minX));
  absminX = minX;
  showminX = minX;

  /* maxX wert */
  spool(fcurves,buffer);
  sscanf(buffer,"%lf", &(drabas.mdata[0].maxX));
  for(i=0;i<drabas.numMeasures;i++)
  {
    drabas.mdata[i].maxX = drabas.mdata[0].maxX;
  }
  absmaxX = drabas.mdata[0].maxX;
  showmaxX = absmaxX;
  imin = 0;
  imax = numX;

  one_step = (rint((absmaxX - absminX)/numX) == 1.0);

  /* alle maxY je measure */
  spool(fcurves,buffer);
  if(buffer[0]!='\n')
  {
    bs = buffer;
    for(i=0;i<drabas.numMeasures;i++)
    {
      sscanf(bs,"%lf", &(drabas.mdata[i].maxY));

      /* this spools buf to the next char after a the first space */
      if(i<drabas.numMeasures)
	while( *bs++ != ' ' )
	  ;
    }
  }
  else
  {
    for(i=0;i<drabas.numMeasures;i++)
    {
      drabas.mdata[i].maxY = 0.0;
    }
  }

  /* alle minY je measure */
  spool(fcurves,buffer);
  if(buffer[0]!='\n')
  {
    bs = buffer;
    for(i=0;i<drabas.numMeasures;i++)
    {
      sscanf(bs,"%lf", &(drabas.mdata[i].minY));

      /* this spools buf to the next char after a the first space; ausser beim letzten Eintrag */
      if(i<drabas.numMeasures)
	while( *bs++ != ' ' )
	  ;
    }  
  }
  else
  {
    for(i=0;i<drabas.numMeasures;i++)
    {
      drabas.mdata[i].minY = 1.0e300;
    }
  }


  /* Kurvenverlauf lines oder steps */
  spool(fcurves,buffer);
  sscanf(buffer,"%d", &dummy);

  /* letzten Kommentar weglesen */
  fgets(buffer,BUF_SIZE,fcurves);

  /* set sonstiges fuer alle measures */
  for(i=0;i<drabas.numMeasures;i++)
  {
    drabas.mdata[i].scale = Adaptive;
    drabas.mdata[i].visible = True;
    drabas.mdata[i].rminY = 1.0e300;
    drabas.mdata[i].rmaxY = 0;
  }
}

/******************************************************************************/
static void help_setup()
/******************************************************************************/
{
  Dimension height;
  XFontStruct *fontStruct;

#ifdef nameDEBUG
  printf("help_setup\n");
#endif
  helpPopup = XtVaCreatePopupShell(
			"Transient Solution Output Help Popup",
			transientShellWidgetClass,
			topLevel,
			NULL);
  helpBox = XtVaCreateManagedWidget(
		"helpBox",
		boxWidgetClass,
		helpPopup,
		XtNborderWidth,0,
		XtNhSpace,4,
		XtNvSpace,4,
		NULL);
  helpdoneButton = XtVaCreateManagedWidget(
		   "doneButton",
                    commandWidgetClass,
                    helpBox,
		    XtNwidth, BUTTONWIDTH,
                    XtNlabel,"Done",
                    XtNjustify,XtJustifyCenter,
                    NULL);


  XtAddCallback(helpdoneButton,XtNcallback,helpDone,NULL);

  helpText = XtVaCreateManagedWidget(
		"helpText",
		asciiTextWidgetClass,
		helpBox,
		XtNheight, 300,
                XtNwidth, 600,
		XtNstring,"There is no help-text available.\nSorry.",
		XtNlength, 300,
	        XtNdisplayCaret,False,
		XtNscrollVertical,XawtextScrollWhenNeeded,
		XtNwrap, XawtextWrapWord,
		XtNeditType, XawtextRead,
		NULL);

  XtVaGetValues (helpText,
		 XtNfont, &fontStruct,
		 XtNheight, &height,
		 NULL);  
}

/******************************************************************************/
static void initmswitch()
/******************************************************************************/
{
  int i;
#ifdef nameDEBUG
  printf("initmswitch\n");
#endif
  CmswitchEntry = (Widget *)emalloc( sizeof(Widget) * (drabas.numMeasures + 1) ); 

  for(i=0;i<drabas.numMeasures;i++)
  {
    CmswitchEntry[i] = XtVaCreateManagedWidget(
        		drabas.mdata[i].name,
		        smeBSBObjectClass,
        		CmswitchMenu,
			XtNleftMargin,TOGGLE_LEFT_MARGIN,
		        NULL);
    XtAddCallback(CmswitchEntry[i],XtNcallback,(XtCallbackProc)mswitchCB,(XtPointer) i);

    XtVaSetValues(CmswitchEntry[i],XtNleftBitmap,None,NULL);
    
    drabas.mdata[i].visible = True;
  }
  CmswitchEntry[i] = XtVaCreateManagedWidget(
        	     "show all",
		     smeBSBObjectClass,
        	     CmswitchMenu,
		     XtNleftMargin,TOGGLE_LEFT_MARGIN,
		     NULL);
  XtAddCallback(CmswitchEntry[i],XtNcallback,(XtCallbackProc)mswitchCB,(XtPointer) i);

  XtVaSetValues(CmswitchEntry[i],XtNleftBitmap,toggle_pixmap,NULL);
}


/******************************************************************************/
static void get_input()
/******************************************************************************/
{
  char buf[BUF_SIZE];
  char zeichen;
  char *bs;
  char *newl;
  int i,j;
  int zwerg;
  int redraw;
  static int offset = 0;
#ifdef nameDEBUG
  printf("get_input\n");
#endif

#ifdef DEBUG
  printf("enter -->\n");
#endif

  clearerr(fcurves);
  while(!feof(fcurves))
  {
    zeichen = getc(fcurves);

#ifdef DEBUG
  printf("%c",zeichen);
#endif

    if(!feof(fcurves))
    {
      buf[offset++] = zeichen;

      if(zeichen == '\n')
      {
	if(buf[0] == '#')
	{
	  memset(statusline,'\0',1000);
	  memccpy(statusline,&(buf[1]),'\n',1000);
	  newl=memchr(statusline,'\n',1000);
	  *newl = '\0';
	  XtVaSetValues(CstatusLabel,XtNstring,statusline,NULL);
	  XawTextEnableRedisplay(CstatusLabel);
	}
	else
	{
	  bs = buf;
	  /* zeit lesen */
	  sscanf(bs,"%lf", &(drabas.time[drabas.aktData]));

	  /* this spools buf to the next char after a the first space; ausser beim letzten Eintrag */
	  while( *bs++ != ' ' )
	    ;

	  /* probs der measures lesen */
	  for(i=0;i<drabas.numMeasures;i++)
	  {
	    sscanf(bs,"%lf", &(drabas.mdata[i].Datas[drabas.aktData]));
	    
	    /* this spools buf to the next char after a the first space; ausser beim letzten Eintrag */
	    if(i<drabas.numMeasures)
	      while( *bs++ != ' ' )
		;
	  }
	  
	  for(i=0;i<drabas.numMeasures;i++)
	  {
	    if(!finite(drabas.mdata[i].Datas[drabas.aktData]))
	    {
	      printf("warning: measure %s has an infinite value t time %g.\n",drabas.mdata[i].name,drabas.time[drabas.aktData]);
		drabas.mdata[i].Datas[drabas.aktData] = 0.0;
	    }
	  }
	  
	  /* max und min veraendert? */
	  for(i=0;i<drabas.numMeasures;i++)
	  {
	    redraw = False;
	    if(drabas.mdata[i].Datas[drabas.aktData] < drabas.mdata[i].minY)
	    {
	      drabas.mdata[i].minY = drabas.mdata[i].Datas[drabas.aktData];
	      if(drabas.mdata[i].minY<drabas.mdata[i].rminY)
		redraw = True;
	    }
	    if(drabas.mdata[i].Datas[drabas.aktData] > drabas.mdata[i].maxY)
	    {
	      drabas.mdata[i].maxY = drabas.mdata[i].Datas[drabas.aktData];
	      if(drabas.mdata[i].maxY>drabas.mdata[i].rmaxY)
		redraw = True;
	    }
	    if(redraw)
	    {
	      set_uniformMinMaxY();
	      redrawKachel(i);
	    }
	      
	    if ( (drabas.aktData<=imax)&&(drabas.aktData>imin)) /* */
	      if(drabas.mdata[i].visible == True)
		drawNextStep(drabas.aktData,i);
	  }

	  if(drabas.time[drabas.aktData] >= drabas.mdata[0].maxX)
	  {
	    XtSetSensitive(  CbreakButton,True);
            /*if (display_only == 0) */
	    XtSetSensitive(  Cstat_analysisButton,True);
	    repeateddisplay = True;
	    fclose(fcurves);
	    return;             /* UND TSCHUESS */
	  }

	  drabas.aktData++;
	  /* eingefuegt am 20.8.95 fuer Verfahren mit adaptiver Schrittweite : */
	  if (drabas.aktData >= drabas.numDatas) {
	    drabas.time = (double *)realloc( drabas.time, (no_data_blocks*drabas.numDatas)*sizeof(double));
	    for(j=0;j<drabas.numMeasures;j++) {
	      drabas.mdata[j].Datas = (double *)realloc( drabas.mdata[j].Datas, (no_data_blocks*drabas.numDatas)*sizeof(double));
	    };
	    no_data_blocks++;
	  };

	}
	offset = 0;
      }
    }
  }

#ifdef DEBUG
  printf("--> leave\n");
#endif
  XtAppAddTimeOut(app_context,TIMEOUT,(XtTimerCallbackProc)get_input,NULL); 
}

/*******************************************************************************/
static void spool(fp, buf)
/*******************************************************************************/
FILE *fp;
char *buf;
{
   do
   {
      fgets(buf,BUF_SIZE,fp);
   } while( buf[0] == COMMENT );

   if( strlen(buf) > 1 )
      buf[strlen(buf) - 1] = '\0';
}


/*******************************************************************************/
static char *next_word(start)
/*******************************************************************************/
char *start;
{
   char *end;

   /* make sure we don't segmentation fault */
   if( start == NULL )
      return( NULL );

   /* search for end of word */
   for( end = start; (*end != ' ') && (*end != '\t') && (*end != '\0'); end++ )
      ;

   /* check if we are at the end of the string */
   if( *end == '\0' )
      return( NULL );
   else
   {
      *end++ = '\0';
      return( end );
   }
}

/******************************************************************************/
static void mswitchCB(w, clientData, callData)
/******************************************************************************/
Widget w;
XtPointer clientData, callData;
{
  int i,j;
  int num;
  int findex;
  Drawable drawon;
#ifdef nameDEBUG
  printf("mswitchCB\n");
#endif
  num =  (int)((long)clientData);


  if(num == drabas.numMeasures)         /* want all measures */
  {
    for(i=0;i<drabas.numMeasures;i++)
    {
      XtVaSetValues(CmswitchEntry[i],XtNleftBitmap,None,NULL);
      drabas.mdata[i].visible = True;
    }
    XtVaSetValues(CmswitchEntry[ num ],XtNleftBitmap,toggle_pixmap,NULL);
  }
  else                                  /* want special measure */
  {  
    for(i=0;i<drabas.numMeasures;i++)
      if(drabas.mdata[i].visible == False)
	break;
    if(i==drabas.numMeasures)           /* was  previously everything enabled ? */
    {
      for(i=0;i<drabas.numMeasures;i++)
      {
	XtVaSetValues(CmswitchEntry[i],XtNleftBitmap,None,NULL);
	drabas.mdata[i].visible = False;
      }

      XtVaSetValues(CmswitchEntry[drabas.numMeasures],XtNleftBitmap,None,NULL);
      
      XtVaSetValues(CmswitchEntry[ num ],XtNleftBitmap,toggle_pixmap,NULL);
      drabas.mdata[num].visible = True;
    }
    else                                /* there where still some measures to enable */
    {
      if(drabas.mdata[num].visible)        /* want to switch "off" special measure */
      {
	XtVaSetValues(CmswitchEntry[num],XtNleftBitmap,None,NULL);
	drabas.mdata[num].visible = False;
      }
      else                                 /* want to switch "on" special measure */
      {
	XtVaSetValues(CmswitchEntry[num],XtNleftBitmap,toggle_pixmap,NULL);
	drabas.mdata[num].visible = True;
      }

      for(i=0;i<drabas.numMeasures;i++)    /* last measures switched off ? */
	if(drabas.mdata[i].visible == True)
	  break;
      if(i==drabas.numMeasures)            /* yes --> wanna see all measures */
      {
	for(i=0;i<drabas.numMeasures;i++)
        {
	  drabas.mdata[i].visible = True;

	}
	XtVaSetValues(CmswitchEntry[ drabas.numMeasures ],XtNleftBitmap,toggle_pixmap,NULL);
      }

      for(i=0;i<drabas.numMeasures;i++)   /* last measures switched on ? */
	if(drabas.mdata[i].visible == False)
	  break;
      if(i==drabas.numMeasures)           /* yes --> wanna see all measures */
      {
	for(i=0;i<drabas.numMeasures;i++)
        {
	  XtVaSetValues(CmswitchEntry[i],XtNleftBitmap,None,NULL);
	}

	XtVaSetValues(CmswitchEntry[ drabas.numMeasures ],XtNleftBitmap,toggle_pixmap,NULL);
      }
    }
  }

  /* set logscale haekchen */
  findex = 0;
  while( !(drabas.mdata[findex].visible) )
    findex++;

  for(i=0;i<4;i++)
    XtVaSetValues(ClswitchEntry[i],XtNleftBitmap,None,NULL);

  XtVaSetValues(ClswitchEntry[ drabas.mdata[findex].scale ],XtNleftBitmap,toggle_pixmap,NULL);


  for(i=0;i<drabas.numMeasures;i++)
    if( (drabas.mdata[i].scale != drabas.mdata[findex].scale) && drabas.mdata[i].visible )
      for(j=0;j<4;j++)
	  XtVaSetValues(ClswitchEntry[j],XtNleftBitmap,None,NULL);
  

  /* clear all */
  drawon = XtWindow(CplotCore);
  XSetForeground(disp,gc,WhitePixel(disp,0));
  XFillRectangle(disp,drawon,gc,0,0,gc_breite,gc_hoehe);

  /* neue uniform y-scale-Werte setzen */
  set_uniformMinMaxY();
  redrawAll();
}

/******************************************************************************/
static void lswitchCB(w, clientData, callData)
/******************************************************************************/
Widget w;
XtPointer clientData, callData;
{
  int i,j;
  int log;

#ifdef nameDEBUG
  printf("lswitchCB\n");
#endif

  log = (int)((long)clientData);

  /* set logscale fuer alle sichtbaren measures */
  for(i=0;i<drabas.numMeasures;i++)
  {
    if(drabas.mdata[i].visible == True)
    {
      drabas.mdata[i].scale = log;
      if(log==Adaptive)
	drabas.mdata[i].rminY = drabas.mdata[i].minY;
    }
  }
  if(log==Equal)
    set_uniformMinMaxY();

  /* Haekchen verwalten */
  for(i=0;i<4;i++)
    XtVaSetValues(ClswitchEntry[i],XtNleftBitmap,None,NULL);

  XtVaSetValues(ClswitchEntry[log],XtNleftBitmap,toggle_pixmap,NULL);

  redrawAll();
}


/******************************************************************************/
static void abbrechenCB(w,clientData,callData)
/******************************************************************************/
Widget w;
XtPointer clientData,callData;
{
  XtPopdown(topLevel);
  exit(1);
}
/******************************************************************************/
static void timeDone(w, clientData, callData)
/******************************************************************************/
    Widget w;
    XtPointer clientData, callData;
{
    XtSetSensitive(  CDisplayedTimeButton, True);
    XtPopdown(timePopup);
}
/******************************************************************************/
static void timeOK(w, clientData, callData)
/******************************************************************************/
    Widget w;
    XtPointer clientData, callData;
{   String buffer;
    double neumin,neumax,dx;

    timeDone(w, clientData, callData);

    XtVaGetValues(TimeMinValueText,XtNstring,&buffer,NULL);
    sscanf(buffer,"%lf",&neumin);
    XtVaGetValues(TimeMaxValueText,XtNstring,&buffer,NULL);
    sscanf(buffer,"%lf",&neumax);
    
    /*dx = (int)(rint((absmaxX - absminX)/numX));*/
    dx = output;
    neumin = rint(neumin/dx)*dx;
    /* absmaxX muss kein Vielfaches von dx sein, da der Rest so berechnet wird.*/
    if (neumax != absmaxX)       
      neumax = rint(neumax/dx)*dx;

    /* neumin = (int)neumin;
    neumax = (int)neumax;*/
    if ( (neumin < absminX)
	||((neumax == maxX)&&(neumin != minX)&&(neumin >= maxX)) )
      showminX = minX = absminX;
    else
      if (neumin < absmaxX)
	minX = showminX = neumin;

    if ((neumax <= showminX)||(neumax > absmaxX))
      showmaxX = maxX = absmaxX;
    else
      showmaxX = maxX = neumax;

    imin = floor((showminX - absminX)/dx);
    imax = ceil((showmaxX - absminX)/dx);

    set_uniformMinMaxY();

    redrawAll();}
/******************************************************************************/
static void timeDefault(w, clientData, callData)
/******************************************************************************/
    Widget w;
    XtPointer clientData, callData;
{ char old[50];
  XawTextBlock tb;

  showmaxX = maxX = absmaxX;
  showminX = minX = absminX;
  imin = 0;
  imax = numX;
  
  sprintf(old,"%d",(int)(minX/*/timescale*/));
  tb.firstPos = 0;
  tb.length = strlen(old);
  tb.ptr = old;
  tb.format =  FMT8BIT;
  XawTextReplace(TimeMinValueText,0,10,&tb);

  sprintf(old,"%d",(int)(maxX/*/timescale*/));
  tb.firstPos = 0;
  tb.length = strlen(old);
  tb.ptr = old;
  tb.format =  FMT8BIT;
  XawTextReplace(TimeMaxValueText,0,10,&tb);
  /*  timeDone(w, clientData, callData);
  redrawAll();*/
}
/******************************************************************************/
static void time_setup()
/******************************************************************************/
{
  XawTextBlock tb;
  char old[50];
  int tHeight=150, tWidth=340, myButtonHeight = 23, myButtonWidth = 100;
  int digits,places;
  double p,output_buffer;

#define SMALL_BUF_SIZE 500

  timePopup = XtVaCreatePopupShell(
			" Change Displayed Time ",
			transientShellWidgetClass,
			topLevel,
			NULL);
  XtVaSetValues(timePopup,XtNwidth,tWidth,XtNheight,tHeight+50,NULL);
  TimeBox = XtVaCreateManagedWidget(
		       "TimeBox",
		       boxWidgetClass,
		       timePopup,
		       XtNwidth,tWidth,
		       XtNheight,tHeight+50,
		       XtNvSpace,10,
		       XtNhSpace,10,
		       XtNborderWidth,0,
		       NULL);
 TimeForm = XtVaCreateManagedWidget(
		       "TimeForm",
		       formWidgetClass,
		       TimeBox,
		       XtNvSpace,10,
		       XtNwidth,tWidth,
		       XtNheight,tHeight+50,
		       XtNhSpace,10,
		       XtNborderWidth,0,
		       NULL);

 TimeForm2 = XtVaCreateManagedWidget(
		       "TimeForm2",
		       formWidgetClass,
		       TimeForm,
		       XtNvSpace,0,
		       XtNwidth,tWidth,
		       XtNheight,tHeight,
		       XtNhSpace,0,
		       XtNborderWidth,0,
		       NULL);

 TimeLabel1 =  XtVaCreateManagedWidget(
		      "TimeLabel1",
		      labelWidgetClass,
		      TimeForm2,
		      XtNheight, text_height,
		      XtNwidth, 150,
		      XtNjustify,XtJustifyLeft,
		      XtNlabel,"New smallest Value ",
		      NULL);

 TimeMinValueLabel =  XtVaCreateManagedWidget(
		      "TimeMinValueLabel",
		      labelWidgetClass,
		      TimeForm2,
		      XtNheight, text_height,
		      XtNwidth, 50,
		      XtNfromVert,TimeLabel1,
		      XtNjustify,XtJustifyLeft,
		      XtNlabel,"T min",
		      NULL);


  TimeMinValueText = XtVaCreateManagedWidget(
		      "TimeMinValueText",
		      asciiTextWidgetClass,
		      TimeForm2,
		      XtNheight, text_height,
		      XtNwidth, text_width,
		      XtNfromHoriz,TimeMinValueLabel,
		      XtNfromVert,TimeLabel1,
		      XtNeditType,XawtextEdit,
		      XtNdisplayCaret,True,			
		      XtNlength,SMALL_BUF_SIZE,
		      NULL);
  XtSetSensitive(TimeMinValueText,True);

 TimeLabel2 =  XtVaCreateManagedWidget(
		      "TimeLabel2",
		      labelWidgetClass,
		      TimeForm2,
		      XtNheight, text_height,
		      XtNwidth, 150,
		      XtNjustify,XtJustifyLeft,
		      XtNfromHoriz,TimeLabel1,
		      XtNlabel,"New largest Value ",
		      NULL);

 TimeMaxValueLabel =  XtVaCreateManagedWidget(
		      "TimeMaxValueLabel",
		      labelWidgetClass,
		      TimeForm2,
		      XtNheight, text_height,
		      XtNwidth, 50,
                      XtNfromVert,TimeLabel2,
		      XtNfromHoriz,TimeLabel1,
		      /*XtNjustify,XtJustifyLeft,*/
		      XtNlabel,"T max",
		      NULL);

  TimeMaxValueText = XtVaCreateManagedWidget(
		      "TimeMaxValueText",
		      asciiTextWidgetClass,
		      TimeForm2,
		      XtNheight, text_height,
		      XtNwidth, text_width,
		      XtNfromHoriz,TimeMaxValueLabel,
		      XtNfromVert,TimeLabel2,
		      XtNeditType,XawtextEdit,
		      XtNdisplayCaret,True,			
		      XtNlength,SMALL_BUF_SIZE,
		      NULL);
  XtSetSensitive(TimeMaxValueText,True);

  timeSpaceLabel = XtVaCreateManagedWidget(
                       "timeSpaceLabel",
                       asciiTextWidgetClass,
		       TimeForm2,
                       XtNeditType,XawtextRead,
                       XtNfromVert,TimeMinValueText,
 		       XtNwidth,345,
		       XtNheight,15,
		       XtNstring,"   ",
		       XtNjustify,XtJustifyLeft,
		       XtNshowGrip,False,
		       NULL);
 
  TimeStepsizeLabel =  XtVaCreateManagedWidget(
		      "TimeStepsizeLabel",
		      labelWidgetClass,
		      TimeForm2,
		      XtNheight, 40,
		      XtNwidth, 130,
                      XtNfromVert,timeSpaceLabel,
		      XtNjustify,XtJustifyLeft,
		      XtNlabel,"stepsize was set to",
		      NULL);
  if (display_only)
    output = rint((absmaxX - absminX)/numX);
  else
    output = rint((absmaxX - absminX)/numX*timescale)/timescale;
  for( places = 0,p = (int)output; p >= 1.0; places++, p /= 10.0);
  for( digits = 0, p = output; p < 1.0,digits < 3; digits++, p *= 10.0 );

  sprintf(old,"%.*g",places+digits,output);
  TimeStepsizeText = XtVaCreateManagedWidget(
                      "TimeStepsizeText",
		      labelWidgetClass,
		      TimeForm2,
		      XtNheight, 40,
		      XtNwidth, 100,
		      XtNfromHoriz,TimeStepsizeLabel,
		      XtNfromVert,timeSpaceLabel,
		      XtNlabel,old,
		      NULL);

  timeCancelButton = XtVaCreateManagedWidget(
		   "timeCancelButton",
                    commandWidgetClass,
		    TimeForm,
		    XtNwidth, myButtonWidth,
		    XtNheight,myButtonHeight,
                    XtNlabel,"Cancel",
		    XtNfromVert,/*timeSpaceLabel*/TimeForm2,
                    XtNjustify,XtJustifyCenter,
                    NULL);

  XtAddCallback(timeCancelButton,XtNcallback,timeDone,NULL);


  timeOKButton = XtVaCreateManagedWidget(
		   "timeOKButton",
                    commandWidgetClass,
		    TimeForm,
		    XtNwidth, myButtonWidth,
		    XtNheight,myButtonHeight,
                    XtNlabel,"OK",
		    XtNfromVert,/*timeSpaceLabel*/TimeForm2,
                    XtNfromHoriz,timeCancelButton,
                    NULL);

  XtAddCallback(timeOKButton,XtNcallback,timeOK,NULL);

  timeDefaultButton = XtVaCreateManagedWidget(
		   "timeDefaultButton",
                    commandWidgetClass,
		    TimeForm,
		    XtNwidth, myButtonWidth,
		    XtNheight,myButtonHeight,
                    XtNlabel,"Default",
		    XtNfromVert,/*timeSpaceLabel*/TimeForm2,
                    XtNfromHoriz,timeOKButton,
                    NULL);

  XtAddCallback(timeDefaultButton,XtNcallback,timeDefault,NULL);

  for( places = 0, p = (int)minX; p >= 1.0; places++, p /=10.0);
  output_buffer = minX - (int)minX;
  for( digits = 0, p = output_buffer; p < 1.0,digits < 3; digits++, p *= 10.0 );
  sprintf(old,"%.*g",digits+places,minX);
  tb.firstPos = 0;
  tb.length = strlen(old);
  tb.ptr = old;
  tb.format =  FMT8BIT;
  XawTextReplace(TimeMinValueText,0,10,&tb);

  for( places = 0, p = (int)maxX; p >= 1.0; places++, p /=10.0);
  output_buffer = maxX - (int)maxX;
  for( digits = 0, p = output_buffer; p < 1.0,digits < 3; digits++, p *= 10.0 );
  if( (places+digits) > 8)
    sprintf(old,"%.8lf",maxX);
  else
    sprintf(old,"%.*g",digits+places,maxX);
  tb.firstPos = 0;
  tb.length = strlen(old);
  tb.ptr = old;
  tb.format =  FMT8BIT;
  XawTextReplace(TimeMaxValueText,0,10,&tb);
 
 

}

/******************************************************************************/
static void CDisplayedTimeCB(w,clientData,callData)    /* */
/******************************************************************************/
Widget w;
XtPointer clientData,callData;
{  Position x,y;

 
   time_setup();
   XtTranslateCoords(topLevel,0,200,&x,&y);
   XtVaSetValues(timePopup,
		 XtNx,x-5,
		 XtNy,y+110,
		 NULL);
   XtSetSensitive(  CDisplayedTimeButton,False); 

   XtPopup(timePopup, XtGrabNone);
}
/******************************************************************************/
static void CstatCB(w,clientData,callData)    /* */
/******************************************************************************/
Widget w;
XtPointer clientData,callData;
{    char res_file[FNAME_SIZE];
     char command[FNAME_SIZE];
     char namecomp[20];
     static char model_name[2*FNAME_SIZE];
     String model_path,tnethome;
     int i,exe_status;
     char dummy[256]; /* reward measure name */
     double d;        /* Ergebnis des measures */

 if ( drabas.show_stat == True)
  drabas.show_stat = False;
 else
  drabas.show_stat = True;
 if ((drabas.show_stat == True)&&(drabas.derived_stat == False))
 {
  /* get environment variables */
  if ((model_path = (String)getenv(MODELDIR_ENV)) == NULL)
  {
    printf("Environment variable %s not set; can't specified model",MODELDIR_ENV);
    exit(-1);
  }
  if ((tnethome = (String)getenv(DSPNPACKAGE_ENV)) == NULL)
  {
    printf("Environment variable %s not set; can't specified model",DSPNPACKAGE_ENV);
    exit(-1);
  }
  if (display_only)
    sprintf(namecomp,".CURVES");
  else
    sprintf(namecomp,".tempcurves");
  i = strlen(net_name) - strlen(namecomp);
  strncpy(model_name,net_name,i);
  if (display_only)
    sprintf(command,"%s/DTSPN/scripts/dtspn %s -s",tnethome,model_name);
  else
    sprintf(command,"%s/DTSPN/scripts/dtspn %s -s %s %s %d %d %lg",tnethome,model_name,emb_opt,sol_opt,max_iter,timescale,prec);
  exe_status = system(command);
  printf("Attention: Evaluation/reward measures are now defined by stationary analysis,\n         Command show place is still defined by transient analysis!");
  fflush(stdout); 
  if (exe_status != 0)
  {
    drabas.show_stat = FALSE;
    stat_failed = 1;
    drabas.derived_stat = FALSE;
  }
  else
  {
    stat_failed = 0;
    sprintf(res_file,"%s/%s.RESULTS",model_path,model_name);
    if (!(res_fp=fopen(res_file,"r"))) 
    {
      fprintf(stderr,"Unable to open file '%s' for reading.\n",res_file );
      exit(-1);
    }
    for(i=0; i<drabas.numMeasures; i++)
    {
      fscanf(res_fp,"%s = %lf",dummy,&d);
      drabas.mdata[i].stat_res = d;
      drabas.mdata[i].maxY_before_stat = drabas.mdata[i].maxY;
      drabas.mdata[i].minY_before_stat = drabas.mdata[i].minY;
      if(drabas.mdata[i].maxY<d)
	drabas.mdata[i].maxY_stat = d;
      else
	drabas.mdata[i].maxY_stat =drabas.mdata[i].maxY;
      if(drabas.mdata[i].minY>d)
	drabas.mdata[i].minY_stat = d;
      else
	drabas.mdata[i].minY_stat =drabas.mdata[i].minY;
    }
    
    fclose(res_fp);
    drabas.derived_stat = True;
  }
 }
 if ((drabas.show_stat == False)&&(stat_failed == 0))
   for (i =0;i<drabas.numMeasures;i++)
   {
     drabas.mdata[i].maxY = drabas.mdata[i].maxY_before_stat;
     drabas.mdata[i].minY = drabas.mdata[i].minY_before_stat;
   }
 else
   for (i =0;i<drabas.numMeasures;i++)
   {
     drabas.mdata[i].maxY = drabas.mdata[i].maxY_stat;
     drabas.mdata[i].minY = drabas.mdata[i].minY_stat;
   }
 set_uniformMinMaxY();
 redrawAll();
}
/******************************************************************************/
static void set_uniformMinMaxY()
{
  int i,j;

  uniform_minY = maxreal;
  uniform_maxY = -maxreal;
  for(i=0;i<drabas.numMeasures;i++)
    if ( drabas.mdata[i].visible == True )
      for (j=0;j<=drabas.aktData;j++)
	if ( (j >= imin) && (j <= imax) )
	{
	  if ( drabas.mdata[i].Datas[j] < uniform_minY)
	  {
	    uniform_minY = drabas.mdata[i].Datas[j];
	  }
	  if ( drabas.mdata[i].Datas[j] > uniform_maxY)
	  {
	    uniform_maxY = drabas.mdata[i].Datas[j];
	  }
	}
  if (drabas.show_stat == True)
  {
    for (i=0;i<drabas.numMeasures;i++)
      if ( drabas.mdata[i].visible == True )
      {
	if (drabas.mdata[i].stat_res > uniform_maxY)
	  uniform_maxY = drabas.mdata[i].stat_res;
	if (drabas.mdata[i].stat_res < uniform_minY)
	  uniform_minY = drabas.mdata[i].stat_res;
      }
  }   
}
/******************************************************************************/
static void ChelpCB(w,clientData,callData)
/******************************************************************************/
Widget w;
XtPointer clientData,callData;
{
   String tnethome;
   char path[MAXPATHLEN];
   Position x,y;


   /* get environment variables */
   if ((tnethome = (String)getenv(DSPNPACKAGE_ENV)) == NULL)
   {
     printf("Environment variable %s not set; can't find multi-solve program",DSPNPACKAGE_ENV);
     exit(-1);
   } 
   sprintf(path,"%s/DTSPN/TransSol/src/TransCurves.help", tnethome);

   /* check if widget exists */
   if( !helpPopup )
      help_setup();

   /* initialize box */
   XtVaSetValues(helpText, 
		 XtNstring, path, 
		 XtNtype, XawAsciiFile,
		 NULL);

   XtTranslateCoords(topLevel,0,200,&x,&y);
   XtVaSetValues(helpPopup,
      XtNx,x-5,
      XtNy,y+110,
      
      NULL);

   /* set parent insensitive, popup shell */
   XtSetSensitive(  ChelpButton,False); 
   XtPopup(helpPopup, XtGrabNone);
}

/******************************************************************************/
static void helpDone(w, clientData, callData)
/******************************************************************************/
    Widget w;
    XtPointer clientData, callData;
{

    XtSetSensitive(  ChelpButton, True);
    XtPopdown(helpPopup);
}


/******************************************************************************/
static void exposeCurves(w, event, params, num_params)
/******************************************************************************/
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
   Dimension b,h;
   int i,zwerg;
#ifdef nameDEBUG
   printf("exposeCurves\n");
#endif
   /* only process last event */
   if( event==NULL || event->xexpose.count == 0 )  /* achtung bus error provoziert */
   {
     
     /* get present size of window */
     XtVaGetValues(w,
		   XtNwidth,&b,
		   XtNheight,&h,
		   NULL);
     gc_breite=(int)b;
     gc_hoehe=(int)h;

     redrawAll();
   }
}



/******************************************************************************/
static void redrawAll()  
/******************************************************************************/
{
  int i,j;
  int n;
  int columns,rows,kachel_index,akt_kachel,rest;
  int kachel_breite;
  int kachel_hoehe;
  int x,y;
  int str_mitte;
  char *buf;

#ifdef nameDEBUG
  printf("redrawAll\n");
#endif

  /* berechnung der kacheln */
  n = 0;
  for(i=0;i<drabas.numMeasures;i++)
    if(drabas.mdata[i].visible == True)
      n++;

  columns = ceil(sqrt((double)n));  /* maximal benoetigte anzahl an kacheln */
  rows = columns - (int)( (columns * columns - n ) / columns ); /* kann ich zeilen einsparen ? */
  kachel_breite = gc_breite / columns;
  kachel_hoehe = gc_hoehe / rows;
 
  akt_kachel = 0;
  kachel_index = 0;
  while( !(drabas.mdata[kachel_index].visible) )
    kachel_index++;
  for(i=0;i<rows;i++)  
  {
    if(  (n - akt_kachel) >= columns )
    {
      for(j=0;j<columns;j++) 
      {
	drabas.mdata[kachel_index].kachel_breite = kachel_breite;
	drabas.mdata[kachel_index].kachel_hoehe = kachel_hoehe;
	drabas.mdata[kachel_index].kachel_x = j*kachel_breite;
	drabas.mdata[kachel_index].kachel_y = i*kachel_hoehe;
	akt_kachel++;
	kachel_index++;
	while( !(drabas.mdata[kachel_index].visible) )
	  kachel_index++;
      }
    }
    else /* weniger kacheln uebrig als spaltenplaetze */
    {
      rest = n - akt_kachel;

      /* clear freien rest */
      XSetForeground(disp,gc,WhitePixel(disp,0));
      XFillRectangle(disp,XtWindow(CplotCore),gc,
		     0, 
		     i*kachel_hoehe,
		     kachel_breite*(columns-rest)/2,
		     gc_hoehe);
      XFillRectangle(disp,XtWindow(CplotCore),gc,
		     gc_breite - kachel_breite*(columns-rest)/2,
		     i*kachel_hoehe,
		     gc_breite,
		     gc_hoehe);

      for(j=0;j<rest;j++) 
      {
	drabas.mdata[kachel_index].kachel_breite = kachel_breite;
	drabas.mdata[kachel_index].kachel_hoehe = kachel_hoehe;
	drabas.mdata[kachel_index].kachel_x = j*kachel_breite + kachel_breite*(columns-rest)/2;
	drabas.mdata[kachel_index].kachel_y = i*kachel_hoehe;
	akt_kachel++;
	kachel_index++;
	while( !(drabas.mdata[kachel_index].visible) )
	  kachel_index++;
      }
    }
  }

  /* gehe alle kacheln durch und lasse sie zeichnen */
  for(i=0;i<drabas.numMeasures;i++)
     if( drabas.mdata[i].visible == True )
           redrawKachel(i); 


}

/******************************************************************************/
static void redrawKachel(num)
/******************************************************************************/
int num;
{
  int i;
  Drawable drawon;
  int x,y;
  int str_mitte;
  char *buf;

  ka_breite = drabas.mdata[num].kachel_breite;
  ka_hoehe = drabas.mdata[num].kachel_hoehe;
  offsetx = drabas.mdata[num].kachel_x;
  offsety = drabas.mdata[num].kachel_y;
  if (drabas.mdata[num].maxX < showmaxX)
    maxX = drabas.mdata[num].maxX;
  else
    maxX = showmaxX;
  
  minY = maxreal;
  maxY = -maxreal;
  for(i=0;i<=drabas.aktData;i++)
  {
    if ((i >= imin)&&(i <= imax))
    { 
      if(drabas.mdata[num].Datas[i] < minY)
      minY = drabas.mdata[num].Datas[i];
    if(drabas.mdata[num].Datas[i] > maxY)
      maxY = drabas.mdata[num].Datas[i];
    }
  }
  if (drabas.mdata[num].scale == Equal)
  {
    minY = uniform_minY;
    maxY = uniform_maxY;
  }
  if (drabas.show_stat == TRUE)
  {
    if (drabas.mdata[num].stat_res < minY)
      minY = drabas.mdata[num].stat_res;
    if (drabas.mdata[num].stat_res > maxY)
      maxY = drabas.mdata[num].stat_res;
  }

  scale = drabas.mdata[num].scale;
  Sonderfall = False;

/*   printf("Kachel:%d   maxY %lf minY %lf scale %d sonderfall\n",num,maxY,minY,scale,Sonderfall); */
 
  drawon = XtWindow(CplotCore);


   if(scale==Logarithmic && minY<=0)
   {
     minY = 1.0e300;
     for(i=0;i<drabas.aktData;i++)
     {

       if( (drabas.mdata[num].Datas[i] > 0) && (drabas.mdata[num].Datas[i] < minY) )
       minY = drabas.mdata[num].Datas[i];
     }
     if(minY==1.0e300)
     {
       printf("logarithmic output makes no sense : switch to linear output.\n");
       scale = Adaptive;
       drabas.mdata[num].scale = Adaptive;
     }
   }
  result_static = drabas.mdata[num].stat_res;
  drawAxes(drawon); 
  if(scale != drabas.mdata[num].scale)
  {
    for(i=0;i<4;i++)
      XtVaSetValues(ClswitchEntry[i],XtNleftBitmap,None,NULL);
  }
  drabas.mdata[num].scale = scale;
  drabas.mdata[num].umaxX = umaxX;
  drabas.mdata[num].udiffY = udiffY;
  if(!Sonderfall)
  {
    drabas.mdata[num].rminY = rminY;
    drabas.mdata[num].rmaxY = rmaxY;
  }
  drabas.mdata[num].lrminY = lrminY;
  drabas.mdata[num].startx = startx;
  drabas.mdata[num].starty = starty;

/*   printf("Kachel:%d   rmaxY %lf rminY %lf scale %d sonderfall\n", 
 	 num, 
 	 drabas.mdata[num].rmaxY, 
 	 drabas.mdata[num].rminY, 
 	 drabas.mdata[num].scale, 
 	 Sonderfall); */


 /* Titel zeichnen */
  x = offsetx + ka_breite/2;
  y = offsety +  MAX_ZAHL_HOEHE;
  buf = drabas.mdata[num].name;
  str_mitte = XTextWidth(XQueryFont(disp,XGContextFromGC(gc)),buf,strlen(buf))/2;
  XDrawImageString(disp,drawon,gc,x-str_mitte,y,buf,strlen(buf));
  for(i=0;i<drabas.aktData;i++) 
  {
    if ((i > imin)&&(i <= imax))         /* hizugefuegt andrea z. 28.11.96 */
      drawNextStep(i,num);
  }
  if ((repeateddisplay == True)        /**/ /* hizugefuegt andrea z. 12.8.96 */
      &&( drabas.aktData <= imax))
    drawNextStep(drabas.aktData,num);  /**/
}

/******************************************************************************/
static void drawAxes(drawon)  
/******************************************************************************/
Drawable drawon;
{
    double rmaxX;
    double lmaxY;     /* log10(maxY) oder 0 */
    double lminY;
    double ticstep;   /* genaue beschriftungsabstand */
    double fticstep;  /* Groessenordnung von ticstep z.B. bei 965 -> 2 */
    double rticstepx;  /* auf eine stelle aufgerundet fuer x Achse */
    double rticstepy;  /* auf eine stelle aufgerundet fuer x Achse */
    double max_zahlen;
    int anz_ytics,anz_xtics;   
    int tic,x,y;            /* zum zeichnen der tics */
    double i;
    double teile;
    char buf[BUF_SIZE];
    int str_mitte;
    double  zae,nen;

    /* x-achse beschriften vorbereiten */
    sprintf(buf,"%.3g\0",maxX/*/timescale*/);
    str_mitte = XTextWidth(XQueryFont(disp,XGContextFromGC(gc)),buf,strlen(buf));

    zae = (double) (ka_breite - (2*SCHOENER_RAND - MAX_ZAHL_BREITE - str_mitte));
    nen = (double) (MIN_ZAHL_ABSTAND_HORI + MAX_ZAHL_BREITE);
    max_zahlen = ceil(zae / nen);
    if(max_zahlen < 1)
      max_zahlen = 1;

    ticstep = (maxX-minX)/max_zahlen;       /* max_zahlen ist mind. 1 und maxX ist >0  --> ticstep>0 */

    fticstep = pow(10.0, floor(log10(ticstep)) + 1);   /* grobes aufrunden */
    rticstepx = ceil(ticstep/fticstep)*fticstep;
    anz_xtics = ceil((maxX-minX) / rticstepx);
    rmaxX = anz_xtics*rticstepx;
    maxdiffX = maxX - minX;

    if( (rmaxX/(maxX-minX) > 1.2) || ((max_zahlen/anz_xtics > 2.0)&&((maxX-minX)>anz_xtics)) ) /* zuviel Platz verschenkt, oder viel weniger Zahlen ? */
    {
      fticstep = pow(10.0,floor(log10(ticstep)));  /* faktor, also groessenordnung */
      rticstepx = ceil(ticstep/fticstep)*fticstep; /* ja --> schrittweite auf erste stelle aufrunden */
      anz_xtics = ceil((maxX-minX) / rticstepx);          /* aufrunden, wenn noetig */
      rmaxX = anz_xtics*rticstepx;
    }

    umaxX = (ka_breite-2*SCHOENER_RAND-MAX_ZAHL_BREITE)/rmaxX;
    startx = SCHOENER_RAND+MAX_ZAHL_BREITE;

    /* y-achse beschriften vorbereiten */
    zae = (double)(ka_hoehe-SCHOENER_RAND-2*MAX_ZAHL_HOEHE);
    nen = (double)(MIN_ZAHL_ANSTAND_VERT+MAX_ZAHL_HOEHE);
    max_zahlen = ceil(zae / nen );
    if(max_zahlen < 1)
      max_zahlen = 1;

    if(scale==Absolute)
      minY = 0.0;

    if(minY >= maxY)                                                           /****** Sonderfall *******/
    {
        Sonderfall = True;
	scale = Absolute;
        if(maxY==0.0)    /* sind alle Werte gleich und 0 */
	{
	  rminY = 0.0;
	  rmaxY = 1.0;
	  anz_ytics = 1;
	  rticstepy = 1.0;
	}
	else           /* sind alle Werte nur gleich */
	{	
	  rminY = 0.0;
	  rmaxY = 2.0*maxY;
	  anz_ytics = 2;
	  rticstepy = maxY;
	}

	udiffY = (ka_hoehe-2*SCHOENER_RAND-MAX_ZAHL_HOEHE) / (rmaxY-rminY);
    }
    else if(scale==Logarithmic )                          /****** lorarithmic *****/
    {
        lminY = floor(log10(minY));                    
        lmaxY = ceil(log10(maxY));                     
        rticstepy = ceil((lmaxY-lminY) / max_zahlen);  /* exponentenschrittweit/(drabas.maxX-minX)*rmaxXe aufgerundet*/
	
	rmaxY = pow(10.0,lmaxY);  
        anz_ytics = ceil((lmaxY-lminY) / rticstepy);    
	rminY = pow(10.0,lmaxY-anz_ytics*rticstepy);
	lrminY=log10(rminY);                           /* 10 hoch irgendwas ist nie null */
        udiffY = (ka_hoehe-2*SCHOENER_RAND-MAX_ZAHL_HOEHE) / (log10(rmaxY)-lrminY); /* 10 hoch irgendwas ist nie null */
    }
    else                                                                 /****** NOT lorarithmic *****/
    {
	ticstep = (maxY - minY)/max_zahlen;     /* ticstep leider rational --> ticstep runden */    

	fticstep = pow(10.0, floor(log10(ticstep)) + 1);  /* max_zahlen ist mind. 1 und diffY ungleich 0 */
	rticstepy = ceil(ticstep/fticstep)*fticstep;  /* aufrunden */
	rminY = floor(minY/fticstep)*fticstep;   /* maxY ist immer groesser ticstep */
	
	ticstep = (maxY-rminY)/max_zahlen;     /* ticstep leider rational --> ticstep runden */    
	fticstep = pow(10.0, floor(log10(ticstep)) + 1);  /* max_zahlen ist mind. 1 und diffY ungleich 0 */
	rticstepy = ceil(ticstep/fticstep)*fticstep;  /* aufrunden */

	anz_ytics = ceil( (maxY-rminY)/rticstepy );
	rmaxY = rminY+anz_ytics*rticstepy;

	if( (rmaxY/maxY > 1.2) || (max_zahlen/anz_ytics > 2.0) ) /* zuviel Platz verschenkt, oder zu wenig Zahlen ? */
        {
	  fticstep = pow(10.0,floor(log10(ticstep)));  /* max_zahlen ist mind. 1 und diffY ungleich 0 */
	  rticstepy = ceil(ticstep/fticstep)*fticstep;  /* aufrunden */
	  rminY = floor(minY/fticstep)*fticstep;   /* maxY ist immer groesser ticstep */
	
	  ticstep = (maxY-rminY)/max_zahlen;     /* ticstep leider rational --> ticstep runden */    
	  fticstep = pow(10.0,floor(log10(ticstep)));  /* max_zahlen ist mind. 1 und diffY ungleich 0 */
	  rticstepy = ceil(ticstep/fticstep)*fticstep;  /* aufrunden */

	  anz_ytics = ceil( (maxY-rminY)/rticstepy );
	  rmaxY = rminY+anz_ytics*rticstepy;
	}
        udiffY = (ka_hoehe-2*SCHOENER_RAND-MAX_ZAHL_HOEHE) / (rmaxY-rminY);
    }
    starty=SCHOENER_RAND+MAX_ZAHL_HOEHE;

    /* clear Kachel */
    XSetForeground(disp,gc,WhitePixel(disp,0));
    XFillRectangle(disp,drawon,gc,offsetx,offsety,ka_breite,ka_hoehe);

    /* achsen zeichnen */
    XSetForeground(disp,gc,BlackPixel(disp,0));
    XSetLineAttributes(disp,gc,2,LineSolid,0,0);
    XDrawLine(disp,drawon,gc,unrealX(0.0),
                          unrealY(rmaxY),
                          unrealX(0.0),
                          unrealY(rminY));  /* y achse */
    XDrawLine(disp,drawon,gc,unrealX(0.0),
                          unrealY(rminY),
                          unrealX(rmaxX/*-minX*/),                      
                          unrealY(rminY));   /* x achse */
    /*printf("xAchse: %d .. %d\nyAchse: %d .. %d\n",unrealX(0.0),unrealX(rmaxX),unrealY(rmaxY),unrealY(rminY));
printf("offsetx: %d startx: %d umaxX:%lf x: %lf\n",offsetx,startx,umaxX,rmaxX);*/    
    /* y tics und text zeichnen */
    x=unrealX(0.0);
    if(scale==Logarithmic)                                               /****** lorarithmic *****/
    {
      for(tic=0;tic<anz_ytics;tic++)
      {
	XSetLineAttributes(disp,gc,2,LineSolid,0,0);
	y = unrealY(rminY*pow(10.0,tic*rticstepy));
	sprintf(buf,"10e%d\0",(int)(lrminY+tic*rticstepy));
        XDrawLine(disp,drawon,gc,x,y,x+BIG_TICS,y);
	XDrawImageString(disp,drawon,gc,offsetx+SCHOENER_RAND,y+FONTHALBEHOEHE,buf,strlen(buf));
	XSetLineAttributes(disp,gc,1,LineSolid,0,0);
	teile = (double)(ANZ_LITTLE_TICS*2);
	teile++;
	for(i=2; i<teile; i++)
        {
	  y = unrealY(i/teile*rminY*pow(10.0,(tic+1)*rticstepy));
	  XDrawLine(disp,drawon,gc,x,y,x+LITTLE_TICS,y);
	}
      }
      XSetLineAttributes(disp,gc,2,LineSolid,0,0);  /* letzte Zahl aber keine Tics mehr */
      y = unrealY(rminY*pow(10.0,tic*rticstepy));    /* oder rmaxY */
      XDrawLine(disp,drawon,gc,x,y,x+BIG_TICS,y);
      sprintf(buf,"10e%d\0",(int)(lrminY+tic*rticstepy));  /* oder rmaxY */
      XDrawImageString(disp,drawon,gc,offsetx+SCHOENER_RAND,y+FONTHALBEHOEHE,buf,strlen(buf));

    }
    else                                                      /****** NOT lorarithmic *****/
    {
      for(tic=0;tic<anz_ytics;tic++)
      {
	XSetLineAttributes(disp,gc,2,LineSolid,0,0);
	y = unrealY(rminY+tic*rticstepy);
	sprintf(buf,"%5.4g\0",rminY+tic*rticstepy);
	XDrawLine(disp,drawon,gc,x,y,x+BIG_TICS,y);
	XDrawImageString(disp,drawon,gc,offsetx+SCHOENER_RAND,y+FONTHALBEHOEHE,buf,strlen(buf));      
	XSetLineAttributes(disp,gc,1,LineSolid,0,0);
	teile = (double)ANZ_LITTLE_TICS;
	teile++;
	for(i=1; i<teile; i++)
	{
	  y = unrealY(rminY+tic*rticstepy + i/teile*rticstepy);
	  XDrawLine(disp,drawon,gc,x,y,x+LITTLE_TICS,y);
	}
      }
      XSetLineAttributes(disp,gc,2,LineSolid,0,0);  /* letzte Zahl aber keine Tics mehr */
      y = unrealY(rminY+tic*rticstepy);    /* oder rmaxY */
      XDrawLine(disp,drawon,gc,x,y,x+BIG_TICS,y);
      sprintf(buf,"%5.4g\0",rminY+tic*rticstepy);
      XDrawImageString(disp,drawon,gc,offsetx+SCHOENER_RAND,y+FONTHALBEHOEHE,buf,strlen(buf));
    }
    /* x tics und text zeichnen */
    y = unrealY(rminY);
    x = unrealX(0.0);
    sprintf(buf,"%.4g\0",minX/*/timescale*/);
    str_mitte = XTextWidth(XQueryFont(disp,XGContextFromGC(gc)),buf,strlen(buf))/2;
    XDrawImageString(disp,drawon,gc,x-str_mitte,y+MAX_ZAHL_HOEHE,buf,strlen(buf));
    for(tic=1;tic<=anz_xtics;tic++)
    {
        XSetLineAttributes(disp,gc,2,LineSolid,0,0);
        x = unrealX(tic*rticstepx);
        XDrawLine(disp,drawon,gc,x,y,x,y-BIG_TICS);
        sprintf(buf,"%.4g\0",(tic*rticstepx+minX)/*/timescale*/);
	str_mitte = XTextWidth(XQueryFont(disp,XGContextFromGC(gc)),buf,strlen(buf))/2;
        XDrawImageString(disp,drawon,gc,x-str_mitte,y+MAX_ZAHL_HOEHE,buf,strlen(buf));
        XSetLineAttributes(disp,gc,1,LineSolid,0,0);
	teile= (double)ANZ_LITTLE_TICS;
	teile++;
        for(i=1;i<teile;i++)
        {
            x = unrealX(tic*rticstepx-i/teile*rticstepx);
            XDrawLine(disp,drawon,gc,x,y,x,y-LITTLE_TICS);
        }
    }

 if (drabas.show_stat  == True)
 {
     XSetLineAttributes(disp,gc,2,LineDoubleDash,0,0);
     XDrawImageString(disp,drawon,gc,unrealX((maxX-minX)+0.01*(maxX-minX)),unrealY(result_static)-3,"s",1); 
     XDrawLine(disp,drawon,gc, 
	        unrealX(0.0),
	        unrealY(result_static)-1,
	        unrealX((maxX-minX)+0.05*(maxX-minX)),
	        unrealY(result_static)-1);
 }
}

/******************************************************************************/
static double convertX(x)
/******************************************************************************/
double x;
{
  double akt_maxX;
  
  if (maxX < drabas.mdata[0].maxX)
    akt_maxX = maxX;
  else
    akt_maxX = drabas.mdata[0].maxX;
  return((x - minX)/(akt_maxX - minX)*maxdiffX);
}

/******************************************************************************/
static int unrealX(x)
/******************************************************************************/
double x;
{
  return(offsetx+startx + x*umaxX);
}

/******************************************************************************/
static int unrealY(y)
/******************************************************************************/
double y;
{

  if(scale==Logarithmic)
  {
    if(y>0)
    {
      /* udiffY=(ka_hoehe-unwichtig)/lrdiffY;       */ 
      /* lrdiffY=lrmaxY-lrminY; lrminY=log10(rminY) */
      return(offsety + ka_hoehe - starty - (log10(y)-lrminY)*udiffY);
    }
    else
    {
      return(offsety + ka_hoehe - starty);
    }
  }
  else if(scale==Adaptive && !Sonderfall)
  {
    /* udiffY=(ka_hoehe-unwichtig)/rdiffY; rdiffY=rmaxY-rminY */
    return(offsety + ka_hoehe - starty - (y-rminY)*udiffY); 
  }
  else if(scale == Equal)
    return(offsety + ka_hoehe - starty - (y-/*uniform_minY*/rminY)*udiffY);
  else
  {
    /* udiffY=(ka_hoehe-unwichtig)/rdiffY; rdiffY=rmaxY-rminY */
    return(offsety + ka_hoehe - starty - y*udiffY); 
  }
}

/******************************************************************************/
static void drawNextStep(index,num)
/******************************************************************************/
int index;
int num;
{
  Drawable drawon;

  drawon = XtWindow(CplotCore);

  if(index==0)
    return;
  
  scale = drabas.mdata[num].scale;
  ka_hoehe = drabas.mdata[num].kachel_hoehe;
  offsetx = drabas.mdata[num].kachel_x;
  offsety = drabas.mdata[num].kachel_y;
  umaxX = drabas.mdata[num].umaxX;
  udiffY = drabas.mdata[num].udiffY;
  rminY = drabas.mdata[num].rminY;
  rmaxY = drabas.mdata[num].rmaxY;
  lrminY = drabas.mdata[num].lrminY;
  startx = drabas.mdata[num].startx;
  starty = drabas.mdata[num].starty;

  XSetLineAttributes(disp,gc,line_width,LineSolid,0,0);

  if( !(scale==Logarithmic && drabas.mdata[num].Datas[index-1]==0 && drabas.mdata[num].Datas[index]==0) )
    {
      if (one_step != 1)
      {
	/*	printf("von (%lf,%lf)  nach (%lf,%lf)\n",drabas.time[index - 1],drabas.mdata[num].Datas[index-1],drabas.time[index],drabas.mdata[num].Datas[index]);fflush(stdout);
printf("von (%d,%d)  nach (%d,%d)\n",unrealX(drabas.time[index - 1]),unrealY(drabas.mdata[num].Datas[index-1])-1,unrealX(drabas.time[index]),unrealY(drabas.mdata[num].Datas[index])-1);fflush(stdout);
printf("offsetx: %d startx: %d umaxX:%lf x: %lf..%lf\n\n",offsetx,startx,umaxX,(drabas.time[index - 1]-minX)/(drabas.mdata[0].maxX-minX)*maxdiffX,(drabas.time[index]-minX)/(drabas.mdata[0].maxX-minX)*maxdiffX); */
      XDrawLine(disp,drawon,gc, 
	        unrealX(convertX(drabas.time[index - 1])),
	        unrealY(drabas.mdata[num].Datas[index-1]) - 1,
	        unrealX(convertX(drabas.time[index])),
	        unrealY(drabas.mdata[num].Datas[index]) - 1);
      }
      else
      {
      XDrawLine(disp,drawon,gc, 
	        unrealX(convertX(drabas.time[index - 1])),
	        unrealY(drabas.mdata[num].Datas[index-1]) - 1,
	        unrealX(convertX(drabas.time[index])),
	        unrealY(drabas.mdata[num].Datas[index-1]) - 1);
      XDrawLine(disp,drawon,gc, 
	        unrealX(convertX(drabas.time[index])),
	        unrealY(drabas.mdata[num].Datas[index-1]) - 1,
	        unrealX(convertX(drabas.time[index])),
	        unrealY(drabas.mdata[num].Datas[index]) - 1);
      }

    }
}



    

