/*******************************************************************************
Modul   	:	NetWidget.h

Purpose		: 	"offentliches Headerfile der Klasse NetWidget

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/


#ifndef _ORANetWidget_h
#define _ORANetWidget_h

/*
 * NetWidget Widget public include file
 
   Class Hierarchie:
   Core -> Composite -> NetWidget
 */

/*
 * This include not needed unless the application includes Intrinsic.h
 * after this file.   Anyway, it doesn't hurt.
 */
#include <X11/Composite.h>
#include "Polygon.h"

/* Resources:

Currently no more Resources

 * Name		     Class		RepType		Default Value
 * ----		     -----		-------		-------------
  XtNmodelName       XtCmodelName       XtRstring       NULL
  XtNcommand  	     XtCCommand	        XtRcommands     NOCOMMAND
  XtNobject  	     XtCObject	        XtRobjects	NOOBJEC
  XtNobjectSize	     XtCobjectSize	XtRobjectSize	SixTeen
  XtNtextCallback    XtCtextCallback	XtRCallback	Null
  XtNinfoCallback    XtCinfoCallback	XtRCallback	Null
  XtNtokenDistributionCallback    XtCtokenDistributionCallback	XtRCallback	Null
  XtNtransResultCallback    XtCtransResultCallback	XtRCallback	Null
  XtNarcShowCallback XtCarcShowCallback XtRCallback     Null
  XtNpictureModified XtCpictureModified	XtRBoolean	False
  XtNnetModified     XtCNetModified	XtRBoolean	False
  XtNratesVisible    XtCratesVisible	XtRBoolean	False
  XtNtagsVisible     XtCtagsVisible	XtRBoolean	False
  XtNspline    	     XtCspline		XtRBoolean	False
  XtNsolutionExpected XtCsolutionExpected XtRBoolean	False
  XtNgridsize        XtCgridsize        XtRgridsize	Off
  XtNrescaleFactor    XtCrescaleFactor  XtRrescaleFactor OneZero
  XtNprecision       XtCprecision       XtRCardinal     6
  XtNtokenGame       XtCTokenGame       XtRBoolean      False
  XtNreadOnly       XtCreadOnly       XtRBoolean      False
  XtNselectionBorder XtCSelectionBorder XtRPolygon      NULL
  XtNNWforeground       XtCNWforeground       XtRPixel      Black
 */
#define XtNmodelName "modelName"
#define XtCmodelName "ModelName"

#define XtNCommands "commands"
#define XtCCommands "Commands"
#define XtRCommands "Commands"

#define XtNObjects "objects"
#define XtCObjects "Objects"
#define XtRObjects "Objects"

#define XtNobjectSize "objectSize"
#define XtCobjectSize "ObjectSize"
#define XtRobjectSize "ObjectSize"

#define XtNtextCallback "textCallback"
#define XtCtextCallback "TextCallback"

#define XtNinfoCallback "infoCallback"
#define XtCinfoCallback "InfoCallback"

#define XtNtokenDistributionCallback "tokenDistributionCallback"
#define XtCtokenDistributionCallback "TokenDistributionCallback"

#define XtNtransResultCallback "transResultCallback"
#define XtCtransResultCallback "TransResultCallback"

#define XtNarcShowCallback "arcShowCallback"
#define XtCarcShowCallback "ArcShowCallback"

#define XtNpictureModified "pictureModified"
#define XtCpictureModified "PictureModified"

#define XtNnetModified "netModified"
#define XtCNetModified "NetModified"

#define XtNnetType "netType"
#define XtCNetType "NetType"

#define XtNratesVisible "ratesVisible"
#define XtCratesVisible "RatesVisible"

#define XtNtagsVisible "tagsVisible"
#define XtCtagsVisible "TagsVisible"

#define XtNspline "spline"
#define XtCspline "Spline"

#define XtNsolutionExpected "solutionExpected"
#define XtCsolutionExpected "SolutionExpected"

#define XtNgridsize "gridsize"
#define XtCgridsize "Gridsize"
#define XtRgridsize "Gridsize"

#define XtNrescaleFactor "rescaleFactor"
#define XtCrescaleFactor "RescaleFactor"
#define XtRrescaleFactor "RescaleFactor"

#define XtNprecision "precision"
#define XtCprecision "Precision"

#define XtNtokenGame "tokenGame"
#define XtCTokenGame "TokenGame"

#define XtNreadOnly "readOnly"
#define XtCreadOnly "ReadOnly"

#define XtNselectionBorder "selectionBorder"
#define XtCSelectionBorder "SelectionBorder"
#define XtRPolygon         "Polygon"

#define XtNNWforeground "foreground"
#define XtCNWforeground "Foreground"

#define netClassName "NetWidget"

typedef enum {TN_DSPN,TN_GSPN,TN_ESPN} TimeNetType;

/* Aufzaehlungstypen fuer Objekte, Kommandos, Gittergroesse */

#define NUMCOMMANDS 	8
typedef enum {NOOPERATION,ADD,DELETE,MOVE,CHANGE,SHOW,SELECT,FIRE } Commands;

#define NUMOBJECTS 	14
typedef enum {NOOBJECT,PLACE,IMTRANS,EXPTRANS,DETTRANS,GENTRANS,ARC,MARKPAR,RATEPAR,
		  RESPAR,TAG,UNCOMPLETE_SELECTION,COMPLETE_SELECTION,
	      /*new*/ GROUP} Objects;

typedef enum {Off,Five,Ten,Fifteen,Twenty,Twentyfive,Thirty,Thirtyfive,Forty} Gridsize;

typedef enum {Reset,
	      ZeroFive,ZeroSix,ZeroSeven,ZeroEight,ZeroNine,
	      OneZero,OneOne,OneTwo,OneThree,OneFour,OneFive,
	      OneSix,OneSeven,OneEight,OneNine,
	      TwoZero} RescaleFactor;

typedef enum {Eight,SixTeen,TwentyFour,ThirtyTwo} ObjectSize;

#define isPlace(X) 	((X)==PLACE)
#define isGenTrans(X) 	((X)==GENTRANS)
#define isDetTrans(X) 	((X)==DETTRANS)
#define isExpTrans(X) 	((X)==EXPTRANS)
#define isImTrans(X) 	((X)==IMTRANS)
#define isTransition(X) (((X)==IMTRANS)||((X)==EXPTRANS)||((X)==GENTRANS)||((X)==DETTRANS))
#define isNode(X)	(((X)==PLACE)||isTransition(X))
#define isArc(X)	((X)==ARC)
#define isMarkPar(X)	((X)==MARKPAR)
#define isRatePar(X)	((X)==RATEPAR)
#define isResPar(X)	((X)==RESPAR)
#define isParam(X)	(((X) ==MARKPAR)||((X)==RATEPAR)||((X)==RESPAR))
#define isTag(X) 	((X)==TAG)
#define isUncompleteSelection(X) ((X)==UNCOMPLETE_SELECTION)
#define isCompleteSelection(X)   ((X)==COMPLETE_SELECTION)
/*new*/
#define isGroup(X) 	((X)==GROUP)

#define EDIT_FIELD_WIDTH 2000
#define EDIT_FIELD_HEIGHT 1000
#define DEFAULT_SIZE 16
#define DEFAULT_SPLINE_RADIUS 32
/* max distance between start and end for closing selection: */ 
#define TOLERANCE 8 

#define INFO 0
#define INPUT 1

#define PRIMARY_ACTION 0
#define SECONDARY_ACTION 1

/* Class record constants  and Macros*/

#define isNetWidget(W) XtIsSubclass(W,netWidgetClass)

extern WidgetClass netWidgetClass;

typedef struct _NWCallbackInfo {
		int dialogKind;
		Boolean MD_Option;
		String label;
		String text;
		Dimension root_x,root_y;
	       } NWCallbackInfo;

typedef struct _NWCallbackInfo_2 {
		Widget object;
		Commands edit_command;
		Objects edit_object;
		Dimension root_x,root_y;
	       } NWCallbackInfo_2;

/* status info fuer NWshowTokenDistributionInfo */
#define RESULTS_AVAILABLE 0
#define NO_RESULTS_AVAILABLE 1

typedef struct _NWshowTokenDistributionInfo {
		int status,e_x,e_y;
		String place_name;
		double mean,error;
		String token_distribution;
		} NWshowTokenDistributionInfo;

typedef struct _NWshowTransitionResultsInfo {
		int status,e_x,e_y;
		String transition_name;
		double throughput,error;
		String firing_rate;
		} NWshowTransitionResultsInfo;

typedef struct {
               int e_x, e_y;
	       String arc_name, titel, text;
	       } NWshowArcInfo;

typedef struct _NetWidgetClassRec *NetWidgetClass;
typedef struct _NetWidgetRec      *NetWidget;

/* public function */
extern void getResults();
extern Boolean parameterAvailable();
extern void ExpectResults();

#ifdef __STDC__
extern void get_MD_RParameters(Widget w, 
			       String **rparams, Cardinal *numrparams);
#else
extern void get_MD_RParameters();
#endif

extern void DeleteSelection();
extern void RescaleNet();
extern int kill_MDC();
extern void Save();

#endif /* _ORANetWidget_h */
/* DON'T ADD STUFF AFTER THIS #endif */
