/*******************************************************************************
Modul   	:	NetWidgetP.h

Purpose		: 	privates Headerfile der Klasse NetWidget

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/


/* 
 * NetWidgetP.h - Private definitions for NetWidget widget

   Class Hierarchie:
   Core -> Composite -> NetWidget
 */

#ifndef _ORANetWidgetP_h
#define _ORANetWidgetP_h

/*
 * This include not needed unless the .c file includes IntrinsicP.h
 * after this file.   Anyway, it doesn't hurt.
 */
#include <ctype.h>

#include <X11/CompositeP.h>

/*
 * This one is always needed!
 */
#include "NetWidget.h"
#include "environment.h"

/*These definitions are private to NetWidget*/
typedef void (*voidFKT)();


/* New fields for the NetWidget widget class record */

typedef struct {
	int dummy;	/* dummy */
} NetWidgetClassPart;

/* Full class record declaration */
typedef struct _NetWidgetClassRec {
    CoreClassPart	core_class_part;
    CompositeClassPart	composite_class_part;
    NetWidgetClassPart	netWidget_class_part;
} NetWidgetClassRec;

extern NetWidgetClassRec netWidgetClassRec;

/* New fields for the NetWidget widget record */

/* history structures for 'undo' */
typedef struct _ObjectRecord {
		Widget object;
		ArgList args;
		Cardinal num_args;
		} ObjectRecord;

typedef struct _History {
		ObjectRecord ** object_list;
		Cardinal num_objects;
		} History;

typedef enum {NoneM, UndrawM, DrawM} DrawMode;

typedef struct {

    /* resources */
    String model_name;
    Commands command;
    Objects object;
    ObjectSize	object_size;
    XtCallbackList textCallback;
    XtCallbackList infoCallback;
    XtCallbackList tokenDistributionCallback;
    XtCallbackList transResultCallback;
    XtCallbackList arcShowCallback;
    Boolean picture_modified;
    Boolean net_modified;       /* modifikations of structure/parameters */
    int net_type;				/* Typ des Netzes: one of ESPN, DSPN, GSPN */
	Boolean tagsVisible;
    Boolean ratesVisible;
    Boolean spline;
    Boolean read_only;
    Boolean solution_expected;
    Gridsize gridsize;
    RescaleFactor rescale_factor;
    Cardinal precision;
    Boolean tokenGame;
    Polygon *selectionBorder;
    Pixel foreground;

    /* private state */
    Cardinal numbase,numplace,numtrans,numarc,nummarkpar,
	     numratepar,numresults;
    DrawMode multiplMode;
    Widget selectedObject;
    Boolean actionTerminated;
    Boolean allowMotionAction;
    Boolean allowRotationAction;
    Boolean ignoreButtonEvents;
    Boolean results_collected;
    Boolean throughput_computed;
    int selected_intermediate_point;
    int prev_x, prev_y;
    int selectionCenter_x, selectionCenter_y;
    int currTransOrient;
    float rescale_value;
    History *history;
    Boolean vanish;
    
    /* Graphics Components */
    Pixmap picture, flash;
    Pixmap spott, stipple;     /* Pixmap for highLighting */
    GC draw_GC, screen_GC, copy_GC, motion_GC, clear_GC;

    XtIntervalId timer;
    Boolean flashed;
    int MDCpid;                /* pid of process <net>_MDC */
    /* pipe to process <net>_MDC (mark-dep.card.) */
    FILE *readMDC, *writeMDC;  

} NetWidgetPart;

/*
 * Full instance record declaration
 */
typedef struct _NetWidgetRec {
    CorePart		core;
    CompositePart	composite;
    NetWidgetPart	netWidget;
} NetWidgetRec;

/* Constraints ensuring right order of children array */
/* Reihenfolge (arcs -> places -> trans etc.) */
/* Arcs MUESSEN zuerst stehen! */

#define FirstArc(W)	(0)

#define FirstPlace(W)	((W)->netWidget.numarc)

#define FirstTrans(W)	((W)->netWidget.numarc+(W)->netWidget.numplace)

#define FirstRatePar(W)	((W)->netWidget.numplace+\
				(W)->netWidget.numtrans+\
				(W)->netWidget.numarc)

#define FirstResult(W)	((W)->netWidget.numplace+\
				(W)->netWidget.numtrans+\
				(W)->netWidget.numarc+\
				(W)->netWidget.numratepar)

#define FirstMarkPar(W)	((W)->netWidget.numplace+\
				(W)->netWidget.numtrans+\
				(W)->netWidget.numarc+\
				(W)->netWidget.numratepar+\
				(W)->netWidget.numresults)

#define LastArc(W)	FirstPlace(W)
#define LastPlace(W)	FirstTrans(W)
#define LastTrans(W)	FirstRatePar(W)
#define LastRatePar(W)	FirstResult(W)
#define LastResult(W)	FirstMarkPar(W)
#define LastMarkPar(W)	(FirstMarkPar(W)+(W)->netWidget.nummarkpar)

/*CONSTANTS*/

/*return value from function look_up, if no object found*/
#define NothingFound -2

/* Mode Flags for getObjectInArea */
#define SELECTION	0
#define PUT	1

/* modes for drawing entire net*/
#define NORMAL 0
#define CLEAR 1
#define TAGS_ONLY 2
#define RATES_ONLY 3

/* points to get in the procedures 'AddMarkPar', 'AddRatePar', 'ChangeMarkPar', 
   ChangeRatePar' */
#define NAME 0
#define VALUE 1
#define NAME_OR_VALUE 2
#define ENABLINGDEPENDENCE 3

/* seperators for textinput */
#define TEXT_SEPERATORS " \t\n:="

/*MACROS*/

#define isWord(S) (isalpha((int)*(S)) || *(S) == '_')

#define CallMethod(CMD,P1,P2,P3,P4)  (void)(*(CmdKeyTransTable[CMD]))(P1,P2,P3,P4)
#define CallMethod_2(CMD,P1,P2,P3,P4)  (void)(*(ExtCmdKeyTransTable[CMD]))(P1,P2,P3,P4)
#define Draw(OBJ,INST,DPY,PX,GC,X,Y,OS,M)    (void)(*(DrawFktTable[OBJ]))(INST,DPY,PX,GC,X,Y,OS,M)

#define is_PxT_arc(OBJ1,OBJ2) (isPlaceObject(OBJ1) && isTransObject(OBJ2))
#define is_TxP_arc(OBJ1,OBJ2) (isTransObject(OBJ1) && isPlaceObject(OBJ2))
#define isValidArc(OBJ1,OBJ2) (is_PxT_arc(OBJ1,OBJ2) || is_TxP_arc(OBJ1,OBJ2))

extern Widget editCommandBox,editObjectBox;

#define ignoreSettings(NW) {\
		(NW)->netWidget.actionTerminated = False;\
  		XtSetSensitive(editCommandBox,False);\
    		XtSetSensitive(editObjectBox,False);\
	}

#define allowSettings(NW) {\
		(NW)->netWidget.actionTerminated = True;\
		(NW)->netWidget.selectedObject = NULL;\
		(NW)->netWidget.ignoreButtonEvents = False;\
		(NW)->netWidget.allowMotionAction = False;\
		(NW)->netWidget.prev_x = -1;\
		(NW)->netWidget.prev_y = -1;\
  		XtSetSensitive(editCommandBox,True);\
    		XtSetSensitive(editObjectBox,True);\
	}

#define ignoreSettingsAndEvents(NW) {\
		ignoreSettings(NW);\
		(NW)->netWidget.ignoreButtonEvents = True;\
	}

#define SetModifiedFlag(NW,VAL) (NW)->netWidget.picture_modified = (VAL)
#define SetNetModifiedFlag(NW,VAL) (NW)->netWidget.net_modified = (VAL)
#define SetCollectResultFlag(NW,VAL) (NW)->netWidget.results_collected = (VAL)
#define SetComputeThroughputFlag(NW,VAL) (NW)->netWidget.throughput_computed = (VAL)

/* Macros to transfer pixelsizes to inch */
#define PixPerInch 90.0
#define PixToInch(X)	((float)(X) / (PixPerInch))
#define InchToPix(X)	(int)((X)*(PixPerInch))


/* declaring all external references for NW-source files */

/* from NetWidget.c*/
#ifndef _NWMAIN_
extern voidFKT DrawFktTable[];
extern void Redisplay();
extern String infoStrings[];
#endif

/*from NWUtilities,c*/
extern Widget arcExists();
extern Boolean deleteConnectedArcs();
extern void redrawArcs();
extern int lookUp();
extern Widget findArc();
extern String getObjName();
extern void DrawNet();
extern Widget getArc();

/*from NWAddActions.c*/
extern void Add();
extern void Add_2();

/*from NWDeleteActions.c*/
extern void Delete();

/*from NWMoveActions.c*/
extern void Move();
extern void Move_2();

/*from NWChangeActions.c*/
extern void Change();
extern void Change_2();

/*from NWDraw.c*/
extern void DrawPlace();
extern void DrawTrans();
extern void DrawTag();
extern void DrawRate();
extern void DrawMarkPar();
extern void DrawRatePar();
extern void DrawArc();
extern void DrawArcPiece();
extern void ClearNetPixmap();
extern void showEnabledTranss();
extern void undrawSelectionBorder();
extern void undrawSelectedObjects();
extern int getSection();
extern void drawRectangle();
extern void ShowNet();
extern void CopyToFlash();

/*from NWFileActions.c*/
extern void Save();
extern void Load();

/* from NWAppActions.c*/
extern void ClearNet();
extern void Redraw();

/* from NWResultActions */
extern void AddResult();
extern void DeleteResult();
extern void ChangeResult();

/* from NWShowActions.c*/
extern void ShowResults();

/* from NWSelectActions.c*/
extern void RSelect();
extern void RSelect_2();
extern void PSelect();
extern void SelectionOff();
extern void drawCenter();
extern void undrawSelectedTags();

/* from NWGrid.c */
extern void Init_Grid();
extern void Drawgrid();
extern void Free_Grid();
extern void AdjustEventToGrid();

/* from NWTokengame.c*/
extern int start_MDC();


#endif /* _ORANetWidgetP_h */
