/******************************************************************************
Modul   	:	TransObj.h

Purpose		: 	"offentliches Headerfile der Klasse TransObject

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
				release 2: Maerz 1994  (Kolja Koischwitz)
******************************************************************************/


#ifndef _ORATransObj_h
#define _ORATransObj_h

/*
 * TransObj Widget public include file
 
   Class Hierarchie:
   Object -> BaseObj -> TagedObj -> TransObj
 */

/*
 * This include not needed unless the application includes Intrinsic.h
 * after this file.   Anyway, it doesn't hurt.
 */
#include "TagedObj.h"
#include "PlaceObj.h"
#include "ArcObj.h"          /* Multiplicity */

/* Resources:
 * Name		     Class		RepType		Default Value
 * ----		     -----		-------		-------------
  XtNtransType	     XtCtransType       XtRtransType	NoTransType
  XtNtransOrient     XtCtransOrient     XtRtransOrient	ZERO
  XtNpriority	     XtCpriority	XtRCardinal	LOWEST_PRI
  XtNenablingDependence XtCenablingDependence XtRenablingDependence SINGLE_SERVER
  XtNrates	     XtCrates		XtRrates	NULL
  XtNratePos_x	     XtCratePos_x	XtRint		DEFAULT_RATE_POSITION
  XtNratePos_y	     XtCratePos_y	XtRint		DEFAULT_RATE_POSITION
  XtNenabled         XtCEnabled         XtRBoolean      True
  XtNinpConnection   XtCInpConnection   XtRConnections  NULL
  XtNnumInputs       XtCNumInputs       XtRCardinal     0
  XtNoutpConnection  XtCOutpConnection  XtRConnections  NULL
  XtNnumOutputs      XtCNumOutputs      XtRCardinal     0
  XtNinhibitConnection  XtCInhibitConnection  XtRConnections  NULL
  XtNnumInhibits     XtCNumInhibits     XtRCardinal     0
  XtNthroughput      XtCThroughput      XtRDouble       0
  XtNaddConnection   XtCAddConnection   XtRFunction     NULL
  XtNdelConnection   XtCDelConnection   XtRFunction     NULL
  XtNselected        XtCSelected        XtRBoolean      False
  XtNcopy            XtCCopy            XtRWidget       NULL
  --new--
  XtNphase           XtCPhase	        XtRCardinal     LOWEST_PHASE
  XtNgroup	     XtCGroup	        XtRint		DEFAULT_GROUP
  XtNgroup_weight    XtCGroup_weight    XtRDouble       DEFAULT_GROUP_WEIGHT
  XtNracePolicy      XtCracePolicy      XtRracePolicy   RE
  --new--
  */

typedef enum {EXPONENTIEL,DETERMINISTIC,IMMEDIATE,GENERAL} TransType;

#define XtNtransType "transType"
#define XtCtransType "TransType"
#define XtRtransType "TransType"

#define XtEexponential 	  "exponential"
#define XtEimmediate 	  "immediate"
#define XtEdeterministic  "deterministic"
#define XtEgeneral     	  "general"

typedef enum {ZERO,FORTYFIVE,NINETY,MINUSFORTYFIVE} TransOrientation;
#define nextOrientation(A) ((A+1) % (MINUSFORTYFIVE+1))

#define XtNtransOrient "transOrient"
#define XtCtransOrient "TransOrient"
#define XtRtransOrient "TransOrientation"

#define XtNpriority	"priority"
#define XtCpriority	"Priority"

typedef enum {INFINITE_SERVER,SINGLE_SERVER} EnablingDependence;
#define XtNenablingDependence "enablingDependence"
#define XtCenablingDependence "EnablingDependence"
#define XtRenablingDependence "EnablingDependence"

#define XtEinfinite_server "infinite_server"
#define XtEsingle_server "single_server"

#define XtEmarking_dependent "marking_dependent"
#define XtEpmf_function "pmf_function"
#define XtEbrowse_file "browse_file"
#define XtEstandard_rate "standard rate"


typedef struct _Rate {
		int type;		/* RATEVALUE or RATEPARAM or MD_DEF or PMF_FUNC */
		union {			/* DATAFILE */
			double ratevalue; /*either doublevalue */
			XtPointer rateparam; /* or ratepar */
			String md_definition; /* or marking dependent */
                        String pmf_function; /* or prob. mass function for genT */
                        String DataFile;     /* or Data-file  */
		      } values;
		char* enable_fkt;
		} Rate,RatePtr;
#define XtNrates "rates"
#define XtCrates "Rates"
#define XtRrates "Rate"

#define XtNthroughput "throughput"    
#define XtCThroughput "Throughput"

#define XtNratePos_x "ratePos_x"
#define XtCratePos_x "RatePos_x"

#define XtNratePos_y "ratePos_y"
#define XtCratePos_y "RatePos_y"

#define XtNenabled   "enabled"
#define XtCEnabled   "Enabled"

typedef struct _Connection {
  PlaceObj place;
  Multiplicity *multiplicity;
} Connection, *Connections;

#define XtNinpConnection "inpConnection"
#define XtCInpConnection "InpConnection"
#define XtRConnections "Connections"

#define XtNoutpConnection "outpConnection"
#define XtCOutpConnection "OutpConnection"

#define XtNinhibitConnection "inhibitConnection"
#define XtCInhibitConnection "InhibitConnection"

#define XtNnumInputs "numInputs"
#define XtCNumInputs "NumInputs"

#define XtNnumOutputs "numOutputs"
#define XtCNumOutputs "NumOutputs"

#define XtNnumInhibits "numInhibits"
#define XtCNumInhibits "NumInhibits"

#define XtNaddConnection "addConnection"
#define XtCAddConnection "AddConnection"

#define XtNdelConnection "delConnection"
#define XtCDelConnection "DelConnection"

#define XtNselected "selected"
#define XtCSelected "Selected"

#define XtNcopy "copy"
#define XtCCopy "Copy"

#define XtNstartPhi "startPhi"
#define XtCStartPhi "StartPhi"

#define XtNstartOrient "startOrient"
#define XtCStartOrient "StartOrient"

/*new*/
#define XtNphase	"phase"
#define XtCPhase	"Phase"

#define XtNgroup	"group"    
#define XtCGroup        "Group" 

#define XtNgroup_weight "group_weight"   
#define XtCGroup_weight "Group_weight"
/*new*/
typedef enum {RACE_S,RACE_A,RACE_E} RacePolicy;
#define XtNracePolicy "racePolicy"
#define XtCracePolicy "RacePolicy"
#define XtRracePolicy "RacePolicy"

#define XtErace_e "race_e"
#define XtErace_a "race_a"
#define XtErace_s "race_s"

#define transClassName "TransObj"


#define NUMROTKEYS	4
#define Rotate(TRK)	(((TRK)+1)%NUMROTKEYS)

#define PRIORITY_UNDEFINED	0
#define LOWEST_PRI	1

#define DEFAULT_RATE_POSITION 10

#define DEFAULT_PMF_FUNCTION "UNIFORM(0.0,1.0);"
#define DEFAULT_DATA_FILE "~/"
#define RATE_ONLY       010
#define RATE_NO         020

/* Types of Transition-Delays */
#define RATEVALUE 0
#define RATEPARAM 1
#define MD_DEF 2
#define PMF_FUNC 3
#define DATAFILE 4


/*new*/
#define DEFAULT_GROUP   0
#define DEFAULT_GROUP_WEIGHT  1.0
#define LOWEST_PHASE    1

/* Class record constants */

#define isTransObject(W) ((W)?XtIsSubclass(W,transObjClass):0)

extern WidgetClass transObjClass;

/* public functions 
extern void DrawTrans();
extern void DrawRate();*/

typedef struct _TransObjClassRec *TransObjClass;
typedef struct _TransObjRec      *TransObj;

#endif /* _ORATransObj_h */
/* DON'T ADD STUFF AFTER THIS #endif */






