/*******************************************************************************
Modul   	:	browseModels.c

Purpose		: 	Realisierung der Cascaded Popups hinter dem
				Load-Eintrag des File-Menues

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
				release 2: Maerz 1994  Kolja Koischwitz
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include <X11/Intrinsic.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/

#include <X11/Shell.h>     
#include <X11/Xaw/MenuButton.h> 
#include <X11/Xaw/SimpleMenu.h> 
#include <X11/Xaw/SmeBSB.h> 

#include "NetWidget.h"
#include "layout.h"
#include "environment.h"
#include "util.h"

extern void setModelName();

extern Pixmap rightPtr;
extern Widget topLevel;
extern XtAppContext app_context;
extern void LoadNet();

/* forward definition of private actions */
static void PopupNextDirectoryEntry();
static void PopupMainShell();
static void SetNewModelName();
static void Motion();
static void EnterWindow();
static void PopdownBrowseSubmenues();

/* forward definition of private functions */
static void PlaceSubshell();
static void CollectFiles();
static int compare_names();

static char model_name[500];
static XtTranslations popup_translations;

static Cardinal num_subshell = 0;
static char subshell_name_head[] = "sub_shell_";

#define NAME_HEAD_LENGTH 10

static XtActionsRec browse_rec[] = {
		{"popupNextDirectoryEntry",PopupNextDirectoryEntry},
		{"popupMainShell",PopupMainShell},
		{"popdownBrowseSubmenues",PopdownBrowseSubmenues},
		{"setNewModelName",SetNewModelName},
		{"motion",Motion},
		{"enterWindow",EnterWindow},
			};

static char simpleMenuTransTable[] = "\
	<EnterWindow>: 	enterWindow()			\n\
	<LeaveWindow>:	popupNextDirectoryEntry()	\n\
	<Motion>:	motion()			\n\
	<BtnUp>:	setNewModelName()  ";

/*******************************************************************************
Name: 
	initBrowsePopups
--------------------------------------------------------------------------------
Funktionstyp: 
	public Function
--------------------------------------------------------------------------------
Aufgerufen von:
	Funktion initFilePopup() in der Initialisierungsphase
--------------------------------------------------------------------------------
Zweck:
	-Registrieren der hier definierten Actions
	-Initialisierung der modulglobalen Groessen
*******************************************************************************/
void initBrowsePopup()
{
#ifdef debug
printf("browseModels: initBrowsePopup \n");
#endif
  XtAppAddActions(app_context,browse_rec,XtNumber(browse_rec));
  popup_translations = XtParseTranslationTable(simpleMenuTransTable);
}

/*******************************************************************************
Name:
	PopupNextDirectoryEntry
--------------------------------------------------------------------------------
Funktionstyp:
	Actionprocedure
--------------------------------------------------------------------------------
Triggert bei event:
	<LeaveWindow>:	popupNextDirectoryEntry()
	in der simpleMenutranstable, die fuer jede Popupshell registriert wird
--------------------------------------------------------------------------------
Zweck:
	Realisierung der Semantik der Modelpopups:
	Das Modelpopup ist alphabetisch (directories zuerst) geordnet.
	Anklicken des Modelsbutten zeigt den Inhalt des Verzeichnisses
	$MODELDIR. Verzeichnisse sind am rechten Rand besonders
	markiert. 
	Steuerung folgender Teilsemantik:
	Wird das Popup mit dem Pointer rechts ueber einen Ver-
	zeichnisnamen verlassen, erscheint als Popup dessen Inhalt.
	Wird das Popup mit dem Pointer rechts ueber einen Modelnamen
	verlassen, wird nur die Markierung des Namens geloescht.
	Wird das Popup sonstwo verlassen erfolgt ein popdown.
*******************************************************************************/
static void PopupNextDirectoryEntry(w,event,params,numparams)
Widget w;
XEvent *event;
String* params;
Cardinal *numparams;
{
  XLeaveWindowEvent *leave_event = (XLeaveWindowEvent *)event;
  String shell_name = XtName(w);
  String shell_nbr_string = shell_name + NAME_HEAD_LENGTH;
  Dimension width,height;
  Widget active_entry,sub_shell;
  String label,*entries,s_ptr;
  char shellname[20];
  char slash = '/';
  Cardinal num_entries,num_dirs,shell_nbr;
  int i;

#ifdef debug
printf("browseModels:  PopupNextDirectoryEntry \n");
#endif
  sscanf(shell_nbr_string,"%u",&shell_nbr);
  if (shell_nbr != num_subshell -1) 
    return;
  if ((active_entry = XawSimpleMenuGetActiveEntry(w)) == NULL) 
    return;
  XtVaGetValues(w,
		XtNheight,&height,
		XtNwidth,&width,
		NULL);

/*   falls leave_event und rechts verlassen und ueber directory eintrag verlassen:
     subshell creieren und popup */

  if (   leave_event->x >= (int)width-1
      && leave_event->y >0 
      && leave_event->y < (int)height ) 
    {
      CollectFiles(w,&entries,&num_entries,&num_dirs);
      if (num_entries > 0) 
	{
	  XtVaGetValues(active_entry,
			XtNlabel,&label,
			NULL);
	  if (strcmp(model_name,"") != 0)
	    strcat(model_name,"/");
	  strcat(model_name,label);
	  sprintf(shellname,"%s%u",subshell_name_head,num_subshell++);
	  sub_shell = XtVaCreatePopupShell(
					   shellname,
					   simpleMenuWidgetClass,
					   topLevel,
					   XtNtranslations,popup_translations,
					   NULL);
	  qsort((char*)entries,num_entries,sizeof(String),compare_names);
	  for (i=0;i<num_entries;i++) 
	    {
	      if (i < num_dirs) 
		/* das erste Zeichen ist '/' oder '{' soll nicht als
		   label erscheinen */
		XtVaCreateManagedWidget(entries[i]+1,
					smeBSBObjectClass,
					sub_shell,
					XtNborderWidth,0,
					XtNrightMargin,16,
					XtNrightBitmap,rightPtr,
					NULL);	
	      else 
		/* das erste Zeichen ist '/' oder '{' soll nicht als
		   label erscheinen */
		XtVaCreateManagedWidget(entries[i]+1,
					smeBSBObjectClass,
					sub_shell,
					XtNborderWidth,0,
					NULL); 
	    }
	  PlaceSubshell(w,sub_shell,active_entry,leave_event->x,leave_event->y);
	  XtPopup(sub_shell,XtGrabNonexclusive);
	  for (i=0;i<num_entries;i++)
	    XtFree(entries[i]);
	  XtFree((char*)entries);
	}
      else 
	{
	  XtCallActionProc(w,"unhighlight",event,params,*numparams);
	}
    }
  else 
    if  ( leave_event->x < 0 )
      {
	if ((s_ptr = strrchr(model_name,(int)slash)) != NULL)
	  *s_ptr = '\0';
	else
	  strcpy(model_name,"");
	XtPopdown(w);
  	XtDestroyWidget(w);
	num_subshell--;
      }	
}

/*******************************************************************************
Name:
	PopupMainShell
--------------------------------------------------------------------------------
Funktionstyp:
	Actionprocedure
--------------------------------------------------------------------------------
Triggert bei event:
	<LeaveWindow>:	 popupMainShell()
	in der Transtable, die fuer den Menuentry 'Load' registriert wird
--------------------------------------------------------------------------------
Parameter: params[0]: Name des fileButton Widgets
--------------------------------------------------------------------------------
Zweck:
	Aufbau des ersten Popups nach betaetigen des LoadButten
*******************************************************************************/
static void PopupMainShell(w,event,params,numparams)
Widget w;
XEvent *event;
String* params;
Cardinal *numparams;
{
  Widget sub_shell,fileButton,active_entry;
  String* entries;
  Cardinal num_entries,num_dirs;
  int i;
  char shell_name[20];
  XLeaveWindowEvent *leave_event = (XLeaveWindowEvent *)event;
  Dimension height,width;
  
#ifdef debug
  printf("browseModels: PopupMainShell \n");
#endif
  if(*numparams != 1) return;
  fileButton=XtNameToWidget(w,params[0]);
  active_entry=XawSimpleMenuGetActiveEntry(w);
  if(active_entry == NULL) return;
/*  	 printf("No active Widget\n");
  else printf("active: %s (filePopup: %lx, active: %lx)\n",XtName(active_entry),fileButton,active_entry); */
  if(active_entry != fileButton) return;
  XtVaGetValues(w,XtNheight,&height,XtNwidth,&width,NULL);
/*   falls leave_event und rechts verlassen: subshell creieren und popup */
/*  printf("l->x=%d, l->y=%d, width=%d, height=%d\n",leave_event->x,leave_event->y,(int)width, (int)height); */

  if (leave_event->x >= (int)width
      && leave_event->y >0 
      && leave_event->y < (int)height)
  { 
  strcpy(model_name, "");
  CollectFiles(NULL,&entries,&num_entries,&num_dirs);
  sprintf(shell_name,"%s%u",subshell_name_head,num_subshell++);
  sub_shell = XtVaCreatePopupShell(
				   shell_name,
				   simpleMenuWidgetClass,
				   topLevel,
				   XtNtranslations,popup_translations,
				   NULL);
  if ( num_entries > 0 )
    {
      qsort((char*)entries,num_entries,sizeof(String),compare_names);
      for (i=0;i<num_entries;i++) 
	{
	  if (i < num_dirs)
	    /* das erste Zeichen ist '/' oder '{' soll nicht als
	       label erscheinen */
	    XtVaCreateManagedWidget(entries[i]+1,
				    smeBSBObjectClass,
				    sub_shell,
				    XtNborderWidth,0,
				    XtNrightMargin,16,
				    XtNrightBitmap,rightPtr,
				    NULL);	
	  else
	    /* das erste Zeichen ist '/' oder '{' soll nicht als
	       label erscheinen */
	    XtVaCreateManagedWidget(entries[i]+1,
				    smeBSBObjectClass,
				    sub_shell,
				    XtNborderWidth,0,
				    NULL); 

	}
	  PlaceSubshell(w,sub_shell,active_entry,leave_event->x,leave_event->y);
      for (i=0;i<num_entries;i++)
		XtFree(entries[i]);
      XtFree((char*)entries);
    }
  else
    {
      Widget entry;

      entry = XtVaCreateManagedWidget("no models",
				      smeBSBObjectClass,
				      sub_shell,
				      XtNborderWidth,0,
				      NULL);
      XtSetSensitive(entry,False);
    }
	XtPopup(sub_shell,XtGrabNonexclusive);
  }
}

/*******************************************************************************
Name:
	Motion
--------------------------------------------------------------------------------
Funktionstyp:
	Actionprocedure
--------------------------------------------------------------------------------
Triggert bei event:
	<Motion>:	motion()		
	in der simpleMenutranstable, die fuer jede Popupshell registriert wird
--------------------------------------------------------------------------------
Zweck:
	Ruft Action "highlight" des simpleMenuWidgets auf, falls der Pointer
	sich im Popup befindet
*******************************************************************************/
static void Motion(w,event,params,numparams)
Widget w;
XEvent *event;
String* params;
Cardinal *numparams;
{
  XLeaveWindowEvent *leave_event = (XLeaveWindowEvent *)event;
  String shell_name = XtName(w);
  String shell_nbr_string = shell_name + NAME_HEAD_LENGTH;
  Dimension width,height;
  Cardinal shell_nbr;

  sscanf(shell_nbr_string,"%u",&shell_nbr);
  if (shell_nbr == num_subshell -1) 
    {
      XtVaGetValues(w,
		    XtNwidth,&width,
		    XtNheight,&height,
		    NULL);

      if (   leave_event->x < (int)width 
	  && leave_event->x > 0
	  && leave_event->y > 0 
	  && leave_event->y < (int)height ) 
  	{
	  XtCallActionProc(w,"highlight",event,params,*numparams);
  	}
    }
}

/*******************************************************************************
Name:
	EnterWindow
--------------------------------------------------------------------------------
Funktionstyp:
	Actionprocedure
--------------------------------------------------------------------------------
Triggert bei event:
	<EnterWindow>: 	enterWindow()				
	in der simpleMenutranstable, die fuer jede Popupshell registriert wird
--------------------------------------------------------------------------------
Zweck:
	Ruft Action "highlight" des simpleMenuWidgets auf, falls der Pointer
	sich im letzten kreierten Popup befindet
*******************************************************************************/
static void EnterWindow(w,event,params,numparams)
Widget w;
XEvent *event;
String* params;
Cardinal *numparams;
{
  String shell_name = XtName(w);
  String shell_nbr_string = shell_name + NAME_HEAD_LENGTH;
  Cardinal shell_nbr;

#ifdef debug
printf("browseModels: EnterWindow \n");
#endif
  sscanf(shell_nbr_string,"%u",&shell_nbr);
  if (shell_nbr == num_subshell -1) 
    {
      XtCallActionProc(w,"highlight",event,params,*numparams);
    }
}

/*******************************************************************************
Name:
	PopdownBrowseSubmenues
--------------------------------------------------------------------------------
Funktionstyp:
	Actionprocedure
--------------------------------------------------------------------------------
Triggert bei event:
	<BtnUp>: popdownBrowseSubmenues() ...
	in der simpleMenutranstable, die fuer fileMenu registriert ist
--------------------------------------------------------------------------------
Zweck:
	Moegliche vorhandene BrowsePopups werden allesamt entfernt
*******************************************************************************/
static void PopdownBrowseSubmenues(w,event,params,numparams)
Widget w;
XMotionEvent *event;
String* params;
Cardinal *numparams;
{  
  char s_name[20];
  Widget subshell;
  
    while (num_subshell) 
    {
      sprintf(s_name,"%s%u",subshell_name_head,--num_subshell);
      if ((subshell = XtNameToWidget(topLevel,s_name)) != NULL) 
        {
		  XtPopdown(subshell);
          XtDestroyWidget(subshell);
        }
      else
        {
          XtWarning("SetModelName: subshell named '%s' not found");
        }
    }
}

/*******************************************************************************
Name:
	SetNewModelName
--------------------------------------------------------------------------------
Funktionstyp:
	Actionprocedure
--------------------------------------------------------------------------------
Triggert bei event:
	<BtnUp>:	setNewModelName()  		
	in der simpleMenutranstable, die fuer jede Popupshell registriert wird
--------------------------------------------------------------------------------
Zweck:
	falls ein Entrag in dem Popup gekennzeichnet ist, wird die Public-
	function "setModelName" aus netNameText.c aufgerufen.
	In jedem Fall werden alle erschienen Popups zerstoert.
*******************************************************************************/
static void SetNewModelName(w,event,params,numparams)
Widget w;
XMotionEvent *event;
String* params;
Cardinal *numparams;
{
  String shell_name = XtName(w);
  String shell_nbr_string = shell_name + NAME_HEAD_LENGTH;
  Cardinal shell_nbr;
  Widget active_entry,subshell;
  String label;
  char s_name[20],path[500];
  struct stat s_buf;

#ifdef debug
printf("browseModels: SetNewModelName \n");
#endif
  sscanf(shell_nbr_string,"%u",&shell_nbr);
  if (shell_nbr == num_subshell -1 &&
      (active_entry = XawSimpleMenuGetActiveEntry(w)) != NULL) 
    {
      XtVaGetValues(
                    active_entry,
                    XtNlabel,&label,
                    NULL);
      if (strcmp(model_name,"") == 0)
        strcpy(model_name,label);
      else
        sprintf(model_name,"%s/%s",model_name,label);
    }
  while (num_subshell) 
    {
      sprintf(s_name,"%s%u",subshell_name_head,--num_subshell);
      if ((subshell = XtNameToWidget(topLevel,s_name)) != NULL) 
        {
		  XtPopdown(subshell);
          XtDestroyWidget(subshell);
        }
      else
        {
          XtWarning("SetModelName: subshell named '%s' not found");
        }
    }
  sprintf(path,
          "%s/%s%s",
          getenv(MODELDIR_ENV),model_name,NET_EXTENSION);
  if (stat(path,&s_buf)==0 && S_ISREG(s_buf.st_mode))
    {
      setModelName(model_name);
      LoadNet(model_name);
    }
}

/*******************************************************************************
Name:
        CollectFiles
--------------------------------------------------------------------------------
Funktionstyp:
        Private function
--------------------------------------------------------------------------------
Aufgerufen von:
        PopupNextDirectoryEntry
        CreateMainShell
--------------------------------------------------------------------------------
Parameter:
        w:                zugehoeriges Widget
        Ausgabe:
        entries:        die eintrage des akutellen Verzeichnisses
        num_ntries:        Anzahl der eintraege
        num_dirs:        Anzahl der Verzeichnisse
--------------------------------------------------------------------------------
Zweck:
        Oeffnen des aktuellen Verzeichnisses und sammeln des Inhalts
*******************************************************************************/
static void CollectFiles(w,entries,num_entries,num_dirs)
Widget w;
String **entries;
Cardinal *num_entries,*num_dirs;
{
  char path[400],model_path[400];
  DIR *dir_ptr;
  struct dirent *dir_entry;
  String n_name,s_ptr,label,model_dir;
  String *entry_names = NULL;
  char point = '.';
  struct stat s_buf;
  Cardinal n_entries = 0;
  Cardinal n_dirs = 0;
  Widget active_entry;

#ifdef debug
printf("browseModels: CollectFiles \n");
#endif

  entry_names = NULL;
  if (w != NULL) 
    {
      active_entry = XawSimpleMenuGetActiveEntry(w);
      XtVaGetValues(active_entry,
                    XtNlabel,&label,
                    NULL);
    }
  else
    label = "";

  if ((model_dir = getenv(MODELDIR_ENV)) == NULL)
    {
      XtWarning("browse models: Environmentvariable MODELDIR not set, can't find any models");
      *entries = NULL;
      *num_entries = 0;
      *num_dirs = 0;
      return;
    }      
  sprintf(path,"%s/%s/%s",model_dir,model_name,label);
  if ((dir_ptr = opendir(path)) == NULL) 
    {
      *entries = NULL;
      *num_entries = 0;
      *num_dirs = 0;
      return;
    }
  while (dir_entry = readdir(dir_ptr)) 
    {
      n_name = dir_entry->d_name;
      if (*n_name != '.') 
        {
          /* falls name == ~.NET : .NET abschneiden und rein*/
          if ((s_ptr = strrchr(n_name,(int)point)) != NULL &&
              strcmp(s_ptr,NET_EXTENSION) == 0 ) 
            {
              *s_ptr = '\0';
              entry_names = (String*)XtRealloc((char*)entry_names,
                                               (n_entries+1)*sizeof(String));
              /* kleiner trick, falls  directory mit gleichen namen existiert
                 die compare fkt von qsort erkennt diesen dann als garantiert groesser */
              entry_names[n_entries] = XtMalloc(strlen(dir_entry->d_name)+2);
              strcpy(entry_names[n_entries],"{");                   
              strcat(entry_names[n_entries],dir_entry->d_name);
              n_entries++;
            }
          else 
            {
              sprintf(model_path,"%s/%s",path,n_name);
              if (stat(model_path,&s_buf) == 0 && S_ISDIR(s_buf.st_mode)) 
                {                
                  entry_names = (String*)XtRealloc((char*)entry_names,
                                                   (n_entries+1)*sizeof(String));
                  entry_names[n_entries] = XtMalloc(strlen(dir_entry->d_name)+2);
                  /* kleiner trick, falls  directory mit gleichen namen existiert
                     die compare fkt von qsort erkennt diesen dann als 
                     garantiert kleiner */
                  strcpy(entry_names[n_entries],"/");                   
                  strcat(entry_names[n_entries],dir_entry->d_name);
                  n_entries++;
                  n_dirs++;
                    }
            }
        }
    }
  closedir(dir_ptr);
  *entries = entry_names;
  *num_entries = n_entries;
  *num_dirs = n_dirs;
}

/*******************************************************************************
Name:
        PlaceSubshell
--------------------------------------------------------------------------------
Funktionstyp:
        private function
--------------------------------------------------------------------------------
Aufgerufen von:
        PopupNextDirectoryEntry
--------------------------------------------------------------------------------
Parameter:
        shell:                das oberste zur Zeit sichtbare Popup
        sub_shell:        das zu platzierende Popup
        active:                das gerade aktive Widget im zZt sichtbaren Popup
        e_x,e_y:        realitive Koordinaten des Ereignisses
--------------------------------------------------------------------------------
Zweck:
        Positionieren des neuen Popups auf die Hoehe des oberen Randes des 
        hervorgehobenen        Eintrags, und 12 Pixel links vom rechten Rand des
        obersten Popups.
*******************************************************************************/
static void PlaceSubshell(shell,subshell,active,e_x,e_y)
Widget shell,subshell,active;
int e_x,e_y;
{
  Dimension width,height,screen_width,screen_height;
  Position x,y,r_x,r_y,shell_x;
  
  XtVaGetValues(shell,
                XtNwidth,&width,
                XtNx,&shell_x,
                NULL);
  XtVaGetValues(active,        XtNy,&y, NULL);
  x=Max(0,(Position)width - 12);
  XtTranslateCoords(shell,x,y,&r_x,&r_y);

  if (!XtIsRealized(subshell))
    XtRealizeWidget(subshell);

  XtVaGetValues(subshell,
                XtNwidth,&width,
                XtNheight,&height,
                NULL);

  screen_width = (Dimension)WidthOfScreen(XtScreen(subshell));
  screen_height = (Dimension)HeightOfScreen(XtScreen(subshell));

  if (r_y < 0)
    r_y = 0;
  else
    if (r_y + (Position)height > (Position)screen_height)
      r_y = (Position)screen_height - (Position)height - 4;

  if (r_x < 0)
    r_x = 0;
  else
    if (r_x + (Position)width > (Position)screen_width )
      r_x = (Position)screen_width - (Position)width - 4;

  XtMoveWidget(subshell,r_x,r_y);
}

/*******************************************************************************
Name:
        compare_names
--------------------------------------------------------------------------------
Funktionstyp:
        private function
--------------------------------------------------------------------------------
Aufgerufen von:
        qsort (build-in function)
--------------------------------------------------------------------------------
Parameter:
        fn1,fn2: Pointer auf zu vergleichende Strings
--------------------------------------------------------------------------------
Rueckgabewert:
        Rueckgabe der Funktion 'strcmp'
--------------------------------------------------------------------------------
Zweck:
        wird als Aufrufparameter der Funktion qsort gebraucht
--------------------------------------------------------------------------------
Ablauf:
*******************************************************************************/
static int compare_names(fn1,fn2)
String *fn1,*fn2;
{
return strcmp(*fn1,*fn2);
}
