/*******************************************************************************
Modul   	:	cascadedPopup.c

Purpose		: 	Umdefinition der Semantik der Xaw-Widgetklasse
                        'SimpleMenu'

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/


/*******************************************************************************
Modulname:
	cascadedPopup.c
--------------------------------------------------------------------------------
Zweck:
	Umdefinition der Semantik der Xaw-Widgetklasse 'SimpleMenu'
*******************************************************************************/
#include <stdio.h>

#include <X11/IntrinsicP.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SimpleMenu.h>

#include "bitmaps/rightptr.bm"

extern Widget topLevel;

static void PopdownSubmenues();
static void HighlightWhenSubmenuesUnvisible();
static void LeaveMainPopup();
static void LeaveSubmenu();
void SetSubmenuPopedupFlag();

Pixmap rightPtr;

static Boolean submenuPopedup = False;

/* new action record for leave window event */
static XtActionsRec actions[] = {
		{"leaveMainPopup",LeaveMainPopup},
		{"leaveSubmenu",LeaveSubmenu},
		{"popdownSubmenues",PopdownSubmenues},
		{"highlightWhenSubmenuesUnvisible",HighlightWhenSubmenuesUnvisible},
			};

/* Muster Transtable : wo noetig buttons und popups eintragen 
static char mainmenu_transtable[] = "\
	<EnterWindow>:  highlightWhenSubmenuesUnvisible() \n\
	<LeaveWindow>:	leaveEditPopup(MainMenuName MainMenuEntryName_1 SubMenu_1 ... ) \n\
	<BtnMotion>:	highlightWhenSubmenuesUnvisible() \n\
	<BtnUp>:	popdownSubmenues(SubMenu_1 ... ) MenuPopdown() notify() unhighlight()";

static char submenu_transtable[] ="\
	<LeaveWindow>:	leaveSubmenu() ";

*/

/*******************************************************************************
Name:
	initCascadedPopupActions
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
benutzt in:
	main.c
--------------------------------------------------------------------------------
Parameter:
	app_context: Application-Context
	topLevel:    Toplevel Shell der Applikation
--------------------------------------------------------------------------------
Zweck:
	Registrieren der eigen Aktionen
	Kreiren der Pixmap fuer den rechten Pfeil
*******************************************************************************/

void initCascadedPopupActions(app_context,topLevel)
XtAppContext app_context;
Widget topLevel;
{
  XtAppAddActions(app_context,actions,XtNumber(actions));
  rightPtr = XCreateBitmapFromData(XtDisplay(topLevel),
			RootWindowOfScreen(XtScreen(topLevel)),
			rightptr_bits,rightptr_width,rightptr_height);
}

/*******************************************************************************
Name:
	LeaveMainPopup
--------------------------------------------------------------------------------
Funktionstyp:
	Aktion
--------------------------------------------------------------------------------
triggert bei event:
	<LeaveWindow>:	leaveEditPopup() 
--------------------------------------------------------------------------------
Parameter:
	params:
		params[0]: Namen des Referenzwidgets
		params[i]: Namen des Widgets ueber dessen Eintrag bei dessen 
			   Verlassen ein Popup erscheinen soll (i ungerade)
		params[i+1]: Name der Popushell, die gezeigt werden soll
--------------------------------------------------------------------------------
Zweck:
	Steuerung des popups . Falls das popup rechts
	ueber die entsprechenden Eintraege verlassen wird, erscheint das ent-
	sprechende Popup.
*******************************************************************************/
static void LeaveMainPopup(w,event,params,num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
XLeaveWindowEvent *leave_event = (XLeaveWindowEvent*) event;
Position x,y,r_x,r_y;
Dimension width,height,p_width,p_height,screen_width,screen_height;
int i;
Widget reference,button,popup;

  reference = XtNameToWidget(topLevel,params[0]);
  for (i=1;i<*num_params;i+=2)
  {
	button = XtNameToWidget(reference,params[i]);
	popup = XtNameToWidget(topLevel,params[i+1]);
  	XtVaGetValues(button,
		XtNx,&x,
		XtNy,&y,
		XtNwidth,&width,
		XtNheight,&height,
		NULL);
  	if(leave_event->x >= (int)width 
      	     && leave_event->y > (int)y
      	     && leave_event->y < (int)(y + height))
  	{	   
	    XtTranslateCoords(button,
				x+(Position)width-12,0,&r_x,&r_y);

	    if (!XtIsRealized(popup))
	      XtRealizeWidget(popup);

	    XtVaGetValues(popup,
			  XtNwidth,&p_width,
			  XtNheight,&p_height,
			  NULL);
	    screen_width = (Dimension)WidthOfScreen(XtScreen(popup));
	    screen_height = (Dimension)HeightOfScreen(XtScreen(popup));
	    if (r_x < 0)
	      r_x = 0;
	    else
	      if (r_x + (Position)p_width > (Position)screen_width)
		r_x = (Position)screen_width - (Position)p_width;

	    if (r_y < 0)
	      r_y = 0;
	    else
	      if (r_y + (Position)p_height > (Position)screen_height)
		r_y = (Position)screen_height - (Position)p_height;


	    XtMoveWidget(popup,r_x,r_y);
	    XtPopup(popup,XtGrabNonexclusive);
	    return;
  	}
  }
  if (! submenuPopedup)
		XtCallActionProc(w,"unhighlight",event,params,*num_params);
}

/*******************************************************************************
Name:
	LeaveSubmenu
--------------------------------------------------------------------------------
Funktionstyp:
	Aktion
--------------------------------------------------------------------------------
triggert bei event:
	<LeaveWindow>:	leaveEditPopup() 
--------------------------------------------------------------------------------
Zweck:
	Steuerung des popups . Falls das popup links
	verlassen wird, erfolgt popdown, sonst deaktivierung
*******************************************************************************/
static void LeaveSubmenu(w,event,params,num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
XLeaveWindowEvent *leave_event = (XLeaveWindowEvent*) event;
Position x;
Dimension width,height;

  XtVaGetValues(w,
		XtNx,&x,
		NULL);
  if (leave_event->x_root < x) 
  {
	XtPopdown(w);
  }
  else
  {
	XtCallActionProc(w,"unhighlight",event,params,*num_params);
  }
}

/*******************************************************************************
Name:
	HighlightWhenSubmenuesUnvisible
--------------------------------------------------------------------------------
Funktionstyp:
	Aktion
--------------------------------------------------------------------------------
triggert bei event:
	<BtnMotion>:	highlightWhenSubmenuesUnvisible() 
	registriert fuer edit-popup
--------------------------------------------------------------------------------
Zweck:
	Verhindert, dass die Aktion 'highlight' ausgefuehrt wird, falls ein
	Untermenue (grid,rescale) sichtbar ist.
*******************************************************************************/
static void HighlightWhenSubmenuesUnvisible(w,event,params,num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
  if (! submenuPopedup)
  	XtCallActionProc(w,"highlight",event,params,*num_params);
}

/*******************************************************************************
Name:
	PopdownSubmenues	
--------------------------------------------------------------------------------
Funktionstyp:
	Aktion
--------------------------------------------------------------------------------
triggert bei event:
	<BtnUp>: popdownSubmenues() MenuPopdown() notify() unhighlight()
	registriert fuer edit-popup.
--------------------------------------------------------------------------------
Parameter:
	params: namen der Popups, die verschwinden sollen
--------------------------------------------------------------------------------
Zweck:
	popdown von grid-popup oder rescale-popup
*******************************************************************************/

static void PopdownSubmenues(w, event, params,num_params)
Widget w;
XEvent *event;
String * params;
Cardinal *num_params;
{
int i;
   
  for (i=0;i<*num_params;i++)
  {
	w = XtNameToWidget(topLevel,params[i]);
  	XtPopdown(w);
  }
}

/*******************************************************************************
Name:
	SetSubmenuPopedupFlag
--------------------------------------------------------------------------------
Funktionstyp:	
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	XtNpopupCallback von edit-popup
	XtNpopdownCallback von edit-popup
--------------------------------------------------------------------------------
Parameter:
	client_data: True, falls von XtNpopupCallback gerufen;
		     Fallse, falls von XtNpopdownCallback gerufen;
--------------------------------------------------------------------------------
Zweck:
	Setzen des Flags 'submenuPopedup' zur Steuerung der Ereignisse in edit-
	popup.
*******************************************************************************/

void SetSubmenuPopedupFlag(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
 submenuPopedup = client_data ? True : False;
}

