/*******************************************************************************
Modul   	:	commandToggles.c

Purpose		: 	Bereitstellen von Funktionen zur Manipulierung der
                        Kommandotoggles

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/

#include <stdio.h>
#include <limits.h>

#include <X11/Intrinsic.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/

#include <X11/Xaw/Label.h>
#include <X11/Xaw/Toggle.h> 

#include "NetWidget.h"
#include "Polygon.h"
#include "layout.h"

extern Widget getCurrentNet();
extern void undrawSelectionBorder();
extern void AddSelection();
extern Polygon *createPolygon();
extern void destroyPolygon();
extern void ResetObjectToggles();
extern void replaceInfoString();

extern Widget editObjectBox;

/* Callbacks */
static void Set_Command();
static void Copy_Callback();
static void Delete_Callback();

/* Functions */
void setCommandSelect();
void setCommandNormal();

static Widget
comRadioGroupLabel,
addToggle,
moveToggle,
deleteToggle,
changeToggle;

Widget
    showToggle;     /* need that in validateButton (setting sensitivity) */

/*******************************************************************************
Name:
	initCommandRadioGroup
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
Aufgerufen in:
	main.c in der Initialisierungsphase
--------------------------------------------------------------------------------
Parameter:
	parent: Elterwidget an das die Radiogroup gebunden wird
--------------------------------------------------------------------------------
Zweck:
	Initialisierung der Kommandotoggles.
	Binden der Callbackfunktionen.
	Setzen der Radiogroup
*******************************************************************************/

void initCommandRadioGroup(parent)
Widget parent;

{

comRadioGroupLabel = XtVaCreateManagedWidget(
                "comRadioGroupLabel", 
                labelWidgetClass,     
                parent,          
		XtNlabel,"Commands",
		XtNforeground,0,
		XtNbackground,1,
		XtNwidth,COMMAND_TOGGLE_WIDTH,
                NULL);           


addToggle = XtVaCreateManagedWidget(
                "addToggle",     
                toggleWidgetClass,
                parent,           
		XtNlabel,"add",
		XtNwidth,COMMAND_TOGGLE_WIDTH,
		XtNheight,COMMAND_TOGGLE_HEIGHT,
		XtNjustify,COMMAND_TOGGLE_JUSTIFY,
                NULL);            

moveToggle = XtVaCreateManagedWidget(
                "moveToggle",     
                toggleWidgetClass,
                parent,           
		XtNlabel,"move",
		XtNwidth,COMMAND_TOGGLE_WIDTH,
		XtNheight,COMMAND_TOGGLE_HEIGHT,
		XtNjustify,COMMAND_TOGGLE_JUSTIFY,
                NULL);            

deleteToggle = XtVaCreateManagedWidget(
                "deleteToggle",   
                toggleWidgetClass,
                parent,           
		XtNlabel,"delete",
		XtNwidth,COMMAND_TOGGLE_WIDTH,
		XtNheight,COMMAND_TOGGLE_HEIGHT,
		XtNjustify,COMMAND_TOGGLE_JUSTIFY,
                NULL);            

changeToggle = XtVaCreateManagedWidget(
                "changeToggle",   
                toggleWidgetClass,
                parent,           
		XtNlabel,"change",
		XtNwidth,COMMAND_TOGGLE_WIDTH,
		XtNheight,COMMAND_TOGGLE_HEIGHT,
		XtNjustify,COMMAND_TOGGLE_JUSTIFY,
                NULL);            

showToggle = XtVaCreateManagedWidget(
                "showToggle",     
                toggleWidgetClass,
                parent,           
		XtNlabel,"show",
		XtNwidth,COMMAND_TOGGLE_WIDTH,
		XtNheight,COMMAND_TOGGLE_HEIGHT,
		XtNjustify,COMMAND_TOGGLE_JUSTIFY,
                NULL);   

XtAddCallback(addToggle, XtNcallback, Set_Command, (XtPointer)ADD );
XtAddCallback(addToggle, XtNcallback, Copy_Callback, NULL);
XtAddCallback(moveToggle, XtNcallback, Set_Command, (XtPointer)MOVE );
XtAddCallback(deleteToggle, XtNcallback, Set_Command, (XtPointer)DELETE );
XtAddCallback(deleteToggle, XtNcallback, Delete_Callback, NULL);
XtAddCallback(changeToggle, XtNcallback, Set_Command, (XtPointer)CHANGE );
XtAddCallback(showToggle, XtNcallback, Set_Command, (XtPointer)SHOW );

XtVaSetValues(moveToggle,
        XtNradioGroup,addToggle,
        NULL);

XtVaSetValues(deleteToggle,
        XtNradioGroup,addToggle,
        NULL);

XtVaSetValues(changeToggle,
        XtNradioGroup,addToggle,
        NULL);

XtVaSetValues(showToggle,
        XtNradioGroup,addToggle,
        NULL);

}

/*******************************************************************************
Name:
 	ResetCommandToggles
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
Aufgerufen in:
	fileButton.c
--------------------------------------------------------------------------------
Zweck:
	Rucksetzen der Radiogroup
*******************************************************************************/

void ResetCommandToggles()
{
XawToggleUnsetCurrent(addToggle);
}

/*******************************************************************************
Name:
	Set_Command
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit Widget:
	addToggle,
	moveToggle,
	deleteToggle,
	changeToggle,
	showToggle;	
--------------------------------------------------------------------------------
Parameter:
       client_data: commando an Hand Aufz"ahlungstypen Commands
--------------------------------------------------------------------------------
Zweck:
	Aenderungen an der Radiogroup werden dem aktuellen NetWidget durch
	setzen der Resource 'XtNCommands' mitgeteilt.
	Der Infostring wird aktualisiert.

*******************************************************************************/

static void Set_Command(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{

/*falls kein Toggle gesetzt 
	auf nocommand setzen
  sonst auf clientdata
dies sollte gutgehen, da callback fuer ruckgesetztes toggle zuerst aufgerufen wird 
*/

#ifdef debug
    printf("Set_Command \n");
#endif
    
    if ( XawToggleGetCurrent(addToggle) == NULL) 
      XtVaSetValues(getCurrentNet(),
		    XtNCommands,(int)NOOPERATION,
		    NULL);
    else  
      XtVaSetValues(getCurrentNet(),
		    XtNCommands,(int)((long)client_data),
		    NULL);

    replaceInfoString();
}


#ifdef __STDC__
static void Copy_Callback (Widget w, XtPointer client_data, XtPointer call_data)
#else
static void Copy_Callback (w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
#endif
{
    Objects curr_object;
    Boolean state;

    XtVaGetValues(getCurrentNet(),
		  XtNObjects, &curr_object,
		  NULL);
    XtVaGetValues(addToggle,
		  XtNstate, &state,
		  NULL);

    /* IF SELECTION THE 'ADD'-TOGGLE IS A BUTTON */
    if (curr_object == COMPLETE_SELECTION && 
	state ) {

	AddSelection((NetWidget)getCurrentNet());

	ResetCommandToggles();
	replaceInfoString();
    }

} /* Copy_Callback */

#ifdef __STDC__
static void Delete_Callback (Widget w, XtPointer client_data, XtPointer call_data)
#else
static void Delete_Callback (w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
#endif
{
    Objects curr_object;
    Boolean state;

    XtVaGetValues(getCurrentNet(),
		  XtNObjects, &curr_object,
		  NULL);
    XtVaGetValues(deleteToggle,
		  XtNstate, &state,
		  NULL);

    /* IF SELECTION THE 'DELETE'-TOGGLE IS A BUTTON */
    if (curr_object == COMPLETE_SELECTION && 
	state ) 
      DeleteSelection((NetWidget)getCurrentNet());

} /* Delete_Callback */

#ifdef __STDC__
void setCommandsSelect(void)
#else
void setCommandsSelect()
#endif
{
    ResetCommandToggles();
    XtVaSetValues(addToggle,
		  XtNlabel, "copy",
		  XtNwidth,COMMAND_TOGGLE_WIDTH,
		  NULL);
    XtVaSetValues(changeToggle,
		  XtNlabel, "rotate",
		  XtNwidth,COMMAND_TOGGLE_WIDTH,
		  NULL);
    XtSetSensitive(showToggle, False);
} /* setCommandsSelect */

#ifdef __STDC__
void setCommandNormal(void)
#else
void setCommandNormal()
#endif
{
    ResetCommandToggles();
    XtVaSetValues(addToggle,
		  XtNlabel, "add",
		  XtNwidth,COMMAND_TOGGLE_WIDTH,
		  NULL);
    XtVaSetValues(changeToggle,
		  XtNlabel, "change",
		  XtNwidth,COMMAND_TOGGLE_WIDTH,
		  NULL);
    XtSetSensitive(showToggle, True);
} /* setCommandNormal */


