/*******************************************************************************
Modul   	:	commitPupUp.c

Purpose		: 	Bereitstellen der Commitbox

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: November 1993
*******************************************************************************/


#include <stdio.h>
#include <ctype.h>

#include <X11/IntrinsicP.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/

/*
 * Public include file for widgets we actually use in this file.
 */
#include <X11/Shell.h>     
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h> 
#include <X11/Xaw/Label.h> 
#include <X11/Xaw/AsciiText.h> 

#include "util.h"



/* forward definition of private functions */
/*Actions*/
static void Popdown();


/* private var */
static Widget	
dialogFrame,
dialogBox,
  dialogLabel,
  dialogText,
  buttonBox,
  ok_button,
  cancel_button;

static XtCallbackProc old_handler;

static XtPopdownIDRec popdownrec;

#define INPUT_BUF_SIZE 2000

static char inputBuf[INPUT_BUF_SIZE];

/*	Ctrl<Key>Y:		unkill()			\n\*/

static char transtable[] = "\
	Ctrl<Key>F: 		forward-character()		\n\
	Ctrl<Key>B: 		backward-character()    	\n\
	Ctrl<Key>D: 		delete-next-character()		\n\
	Ctrl<Key>A: 		beginning-of-line()		\n\
	Ctrl<Key>E: 		end-of-line()			\n\
	Ctrl<Key>H: 		delete-previous-character()	\n\
	Ctrl<Key>K:		kill-to-end-of-line()		\n\
	Ctrl<Key>W:		kill-selection()		\n\
	Ctrl<Key>:		no-op()				\n\
	    <Key>Right: 	forward-character()		\n\
	    <Key>Left:		backward-character()    	\n\
	    <Key>Delete:	delete-previous-character()	\n\
	    <Key>BackSpace:	delete-previous-character()	\n\
	    <Key>Linefeed:	no-op()				\n\
	    <Key>Return: 	no-op()				\n\
	    <Key>:		insert-char()			\n\
	    <Btn1Down>:		select-start()			\n\
	    <Btn1Motion>:	extend-adjust()			\n\
	    <Btn1Up>:		extend-end(PRIMARY,CUT_BUFFER0)	\n\
	    <Btn2Down>: 	insert-selection(PRIMARY,CUT_BUFFER0)\n\
	    <Btn3Down>: 	extend-start()			\n\
	    <Btn3Motion>:	extend-adjust()			\n\
	    <Btn3Up>:		extend-end(PRIMARY,CUT_BUFFER0) ";

/*******************************************************************************
Name:
        init_dialogBox
--------------------------------------------------------------------------------
Funktionstyp:
        "offentliche Funktion
--------------------------------------------------------------------------------
Parameter:
        topLevel: toplevelshell der Applikation
--------------------------------------------------------------------------------
Zweck:
        Initialisierung der Dialogboxen
*******************************************************************************/

void init_dialogBox(topLevel)
Widget topLevel;
{
  XtTranslations translations;

  translations = XtParseTranslationTable(transtable);

  dialogFrame = XtVaCreatePopupShell(
			"dialogPopup",
			transientShellWidgetClass,
			topLevel,
			XtNallowShellResize,True,
			XtNinput,True,
			NULL);

  dialogBox = XtVaCreateManagedWidget(
		"dialogBox",
		boxWidgetClass,
		dialogFrame,
		XtNborderWidth,0,
		NULL);

  dialogLabel = XtVaCreateManagedWidget(
			"dialogLabel",
			labelWidgetClass,
			dialogBox,
         		XtNborderWidth,0,
			NULL);

  dialogText = XtVaCreateWidget(
			"dialogText",
			asciiTextWidgetClass,
			dialogBox,
			XtNlength,INPUT_BUF_SIZE,
		        XtNstring,inputBuf,
		        XtNeditType,XawtextEdit,
		        XtNborderWidth,0,
		        XtNtranslations,translations,
			NULL);

  buttonBox = XtVaCreateManagedWidget(
			"buttonBox",
			formWidgetClass,
			dialogBox,
		        XtNborderWidth,0,
			NULL);

  ok_button = XtVaCreateManagedWidget(
			"ok_button",
			commandWidgetClass,
			buttonBox,
			XtNlabel,"OK",
			XtNleft,XtChainLeft,
			NULL);

  cancel_button = XtVaCreateManagedWidget(
			"cancel_button",
			commandWidgetClass,
			buttonBox,
			XtNlabel,"cancel",
			XtNfromHoriz,ok_button,
		        XtNright,XtChainRight,
			NULL);

  popdownrec.shell_widget = dialogFrame;
  popdownrec.enable_widget = NULL;

  XtAddCallback(ok_button,XtNcallback,XtCallbackPopdown,(XtPointer)&popdownrec);
  XtAddCallback(cancel_button,XtNcallback,XtCallbackPopdown,(XtPointer)&popdownrec);

}

/*******************************************************************************
Name:
        Dialog
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        doneButton
--------------------------------------------------------------------------------
Zweck:
       Gesammelte Information aus edit_state an die Klasse NetWidget weiter-
       leiten
*******************************************************************************/
void Dialog(question,default_input,handler,move_x,move_y)
String question,default_input;
XtCallbackProc handler;
Position move_x,move_y;
{
  Dimension lwidth,lheight,dwidth,dheight,c_width,c_height,theight,twidth,
            screen_width,screen_height,distance;

  XFontStruct *font;
  XawTextBlock tblock;

  if (old_handler)
  {
      XtRemoveCallback(ok_button,XtNcallback,old_handler,(XtPointer)True );
      XtRemoveCallback(cancel_button,XtNcallback,old_handler,(XtPointer)False );
  }
  old_handler = handler;
  XtAddCallback(ok_button,XtNcallback,handler,(XtPointer)True );
  XtAddCallback(cancel_button,XtNcallback,handler,(XtPointer)False );

  /* Folgender Aufruf muss 2mal gemacht werden, damit die Bestimmung der
     Textbreite in jedem Fall funktioniert */
 
  XtVaSetValues(dialogLabel,
		XtNlabel,(XtArgVal)question,
		NULL);

  XtVaSetValues(dialogLabel,                 
		XtNlabel,(XtArgVal)question,
		NULL);

  XtVaGetValues(dialogLabel,
		XtNwidth,&lwidth,
		XtNheight,&lheight,
		NULL);

  twidth = 0;

  if (default_input)
    {
      strcpy(inputBuf,default_input);

      XtVaGetValues(dialogText,
		    XtNheight,&theight,
		    XtNfont,&font,
		    NULL);

      twidth = (Dimension)(strlen(inputBuf)+10)*
	  (Dimension)(font->max_bounds.width);
      tblock.firstPos = 0;
      tblock.length = strlen(default_input);
      tblock.ptr = default_input;
      tblock.format = FMT8BIT;

      XawTextReplace(dialogText,0,INPUT_BUF_SIZE,&tblock);
      XtVaSetValues(dialogText,
		    XtNwidth,twidth,
/*		    XtNlength,INPUT_BUF_SIZE,
		    XtNstring,inputBuf,*/
		    NULL);
            
      XawTextSetInsertionPoint(dialogText,(XawTextPosition)strlen(inputBuf));

      XtManageChild(dialogText);
      XtSetKeyboardFocus(dialogFrame,dialogText);
      XtVaGetValues(dialogText,
		    XtNheight,&theight,
		    NULL);
    }
  else
     XtUnmanageChild(dialogText);

  if (!XtIsRealized(dialogFrame))
    XtRealizeWidget(dialogFrame);

  XtVaGetValues(buttonBox,
		XtNwidth,&c_width,
		XtNheight,&c_height,
		NULL);

  dwidth = Max(twidth,Max(lwidth,c_width));
  dheight = lheight + c_height;
  distance = 12;

  if (default_input)
    {
      dheight += theight;
      distance += 4;
    }
  XtResizeWidget(dialogFrame,dwidth+8,dheight+distance,1);

  XtVaGetValues(dialogFrame,
		XtNwidth,&dwidth,
		XtNheight,&dheight,
		NULL);
  screen_width = (Dimension)WidthOfScreen(XtScreen(dialogFrame));
  screen_height = (Dimension)HeightOfScreen(XtScreen(dialogFrame));

  if (move_x < (Position)0)
    move_x = 0;
  else
    if (move_x + (Position)dwidth > (Position)screen_width)
      move_x =  (Position)screen_width - (Position)dwidth;

  if (move_y < (Position)0)
    move_y = 0;
  else
    if (move_y + (Position)dheight > (Position)screen_height)
      move_y =  (Position)screen_height - (Position)dheight;

  XtMoveWidget(dialogFrame,move_x,move_y);
  XtPopup(dialogFrame,XtGrabExclusive);

}


/*******************************************************************************
Name:
        getDialogInput
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        doneButton
--------------------------------------------------------------------------------
Zweck:
       Gesammelte Information aus edit_state an die Klasse NetWidget weiter-
       leiten
*******************************************************************************/

String getDialogInput()
{
  String buf;

  XtVaGetValues(dialogText,
		XtNstring,&buf,
		NULL);
  return buf;
}
