/*******************************************************************************
Modul   	:	editButton.c

Purpose		: 	Initialisierung des edit-buttons, des main-popups, 
                        grid-popups und	rescale-popups. Bereitstellen von 
                        Aktion und Callbacks zur Steuerung der Ereignisse.

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/

#include <stdio.h>
#include <sys/param.h>

#include <X11/Intrinsic.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SimpleMenu.h>

#include "NetWidget.h"

#include "layout.h"

extern Pixmap rightPtr;
extern Widget getCurrentNet();
extern void SetSubmenuPopedupFlag();
extern void ResetOptionButtons();
static void Kill();
static void KillCurrentBuffer();

#ifdef UNDO
extern void UndoNet();
#endif UNDO

static void Clear();
static void Redraw();
static void SetRescaleFactor();
static void SetGrid();
static void Undo();

/* external Widgets */
extern Widget printOperationPopup;

/* main popup */
static Widget 
bufferMenuButton,
bufferPopup,
	bufferButton,
	undoButton,
	rescaleButton,
	redrawButton,
	gridButton,
	clearButton,
	killButton;
Widget printButton; /* exported to printButton.c */

/* grid popup */
static Widget 
gridPopup,
	grid_off,
	grid_10,
	grid_20,
	grid_30;

/* rescale popup */
static Widget
rescalePopup,
	rescale_reset,
	rescale_05,
	rescale_06,
	rescale_07,
	rescale_08,
	rescale_09,
	rescale_10,
	rescale_12,
	rescale_14,
	rescale_16,
	rescale_18,
	rescale_20;


static char transtable[] = "\
	<EnterWindow>:  highlightWhenSubmenuesUnvisible() \n\
	<LeaveWindow>:	leaveMainPopup(bufferPopup bufferButton netBufferPopup rescaleButton rescalePopup gridButton gridPopup) \n\
	<BtnMotion>:	highlightWhenSubmenuesUnvisible() \n\
	<BtnUp>:	popdownSubmenues(netBufferPopup rescalePopup gridPopup) MenuPopdown() notify() unhighlight()";

static char submenu_transtable[] ="\
	<LeaveWindow>:	leaveSubmenu() ";

/*******************************************************************************
Name:
	initEditPopup
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
benutzt in:
	main.c
--------------------------------------------------------------------------------
Parameter:
	app_context: Application-Context
	topLevel:    Toplevel Shell der Applikation
	parent:	     Elter-Widget fuer edit-button
--------------------------------------------------------------------------------
Zweck:
	Initialisierung des edit-buttons,des main-popups, grid-popups,
	rescale-popups
*******************************************************************************/

void initEditPopup(app_context,topLevel,parent)
XtAppContext app_context;
Widget topLevel,parent;

{
XtTranslations translations;

/******* Buffer Menu ***************************************************/  


  bufferMenuButton = XtVaCreateManagedWidget(
                "bufferMenuButton",          
                menuButtonWidgetClass, 
                parent, 
                XtNlabel,"Buffer",
                XtNwidth,COMMAND_BUTTON_WIDTH,        
                XtNheight,COMMAND_BUTTON_HEIGHT,
                XtNjustify,COMMAND_BUTTON_JUSTIFY,
                XtNmenuName,"bufferPopup",
                NULL);             

  bufferPopup = XtVaCreatePopupShell(
                "bufferPopup",
                simpleMenuWidgetClass,
                topLevel,
                NULL);
  translations = XtParseTranslationTable(transtable);
  XtOverrideTranslations(bufferPopup,translations);

#ifdef UNDO
  undoButton = XtVaCreateManagedWidget(
		"undoButton",
		smeBSBObjectClass,
		bufferPopup,
		XtNlabel,"Undo",
		XtNjustify,MENU_ENTRY_JUSTIFY,
		XtNrightMargin,16,
		NULL);
#endif

  bufferButton = XtVaCreateManagedWidget(
                "bufferButton",
                 smeBSBObjectClass,
                 bufferPopup,
                 XtNlabel,"Switch",
                 XtNjustify,MENU_ENTRY_JUSTIFY,
                 XtNrightMargin,16,
                 XtNrightBitmap,rightPtr,
                 NULL);

  printButton = XtVaCreateManagedWidget(
               "printButton",
               smeBSBObjectClass,
               bufferPopup,
               XtNlabel,"Print..",
                        XtNjustify,MENU_ENTRY_JUSTIFY,
                        XtNrightMargin,16,
               NULL);  
  XtAddCallback(printButton,XtNcallback,XtCallbackNone,printOperationPopup);

  killButton = XtVaCreateManagedWidget(
                "killButton",
                smeBSBObjectClass,
                bufferPopup,
                XtNlabel,"Kill",
                XtNjustify,MENU_ENTRY_JUSTIFY,
                XtNrightMargin,16,
                NULL);
  XtAddCallback(killButton,XtNcallback,Kill,0);

  clearButton = XtVaCreateManagedWidget(
		"clearButton",
		smeBSBObjectClass,
		bufferPopup,
		XtNlabel,"Clear",
		XtNjustify,MENU_ENTRY_JUSTIFY,
		XtNrightMargin,16,
		NULL);

  redrawButton = XtVaCreateManagedWidget(
		"redrawButton",
		smeBSBObjectClass,
		bufferPopup,
		XtNlabel,"Redraw",
		XtNjustify,MENU_ENTRY_JUSTIFY,
		XtNrightMargin,16,
		NULL);

  rescaleButton = XtVaCreateManagedWidget(
		"rescaleButton",
		smeBSBObjectClass,
		bufferPopup,
		XtNlabel,"Rescale",
		XtNjustify,MENU_ENTRY_JUSTIFY,
		XtNrightMargin,16,
		XtNrightBitmap,rightPtr,
		NULL);

  gridButton = XtVaCreateManagedWidget(
		"gridButton",
		smeBSBObjectClass,
		bufferPopup,
		XtNlabel,"Grid",
		XtNjustify,MENU_ENTRY_JUSTIFY,
		XtNrightMargin,16,
		XtNrightBitmap,rightPtr,
		NULL);
#ifdef UNDO
  XtAddCallback(undoButton,XtNcallback,Undo,0);
#endif
  XtAddCallback(clearButton,XtNcallback,Clear,0);
  XtAddCallback(redrawButton,XtNcallback,Redraw,0);

  translations = XtParseTranslationTable(transtable);
  XtOverrideTranslations(bufferPopup,translations);

  /* initialize grid popup */
  gridPopup = XtVaCreatePopupShell(
		"gridPopup",
		simpleMenuWidgetClass,
		topLevel,
		XtNwidth,COMMAND_BUTTON_WIDTH,
		NULL);
  XtAddCallback(gridPopup,XtNpopupCallback,SetSubmenuPopedupFlag,
		(XtPointer)True);
  XtAddCallback(gridPopup,XtNpopdownCallback,SetSubmenuPopedupFlag,
		(XtPointer)False);
  translations = XtParseTranslationTable(submenu_transtable);
  XtOverrideTranslations(gridPopup,translations);

  grid_off = XtVaCreateManagedWidget(
		"grid_off",
		 smeBSBObjectClass,
		 gridPopup,
		 XtNlabel,"Off",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 NULL);

  grid_10 = XtVaCreateManagedWidget(
		"grid_10",
		 smeBSBObjectClass,
		 gridPopup,
		 XtNlabel,"10 Pixels",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 NULL);

  grid_20 = XtVaCreateManagedWidget(
		"grid_20",
		 smeBSBObjectClass,
		 gridPopup,
		 XtNlabel,"20 Pixels",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 NULL);

  grid_30 = XtVaCreateManagedWidget(
		"grid_30",
		 smeBSBObjectClass,
		 gridPopup,
		 XtNlabel,"30 Pixels",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 NULL);

  XtAddCallback(grid_off,XtNcallback,SetGrid,(XtPointer)Off);
  XtAddCallback(grid_10,XtNcallback,SetGrid,(XtPointer)Ten);
  XtAddCallback(grid_20,XtNcallback,SetGrid,(XtPointer)Twenty);
  XtAddCallback(grid_30,XtNcallback,SetGrid,(XtPointer)Thirty);

  /* initialize rescale popup */
  rescalePopup = XtVaCreatePopupShell(
		"rescalePopup",
		simpleMenuWidgetClass,
		topLevel,
		XtNwidth,COMMAND_BUTTON_WIDTH,
		NULL);
  XtAddCallback(rescalePopup,XtNpopupCallback,SetSubmenuPopedupFlag,
		(XtPointer)True);
  XtAddCallback(rescalePopup,XtNpopdownCallback,SetSubmenuPopedupFlag,
		(XtPointer)False);
  translations = XtParseTranslationTable(submenu_transtable);
  XtOverrideTranslations(rescalePopup,translations);

  rescale_reset = XtVaCreateManagedWidget(
		"rescale_reset",
		 smeBSBObjectClass,
		 rescalePopup,
		 XtNlabel,"Reset",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 NULL);

  rescale_05 = XtVaCreateManagedWidget(
		"rescale_05",
		 smeBSBObjectClass,
		 rescalePopup,
		 XtNlabel,"0.5",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 NULL);

  rescale_06 = XtVaCreateManagedWidget(
		"rescale_06",
		 smeBSBObjectClass,
		 rescalePopup,
		 XtNlabel,"0.6",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 NULL);

  rescale_07 = XtVaCreateManagedWidget(
		"rescale_07",
		 smeBSBObjectClass,
		 rescalePopup,
		 XtNlabel,"0.7",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 NULL);

  rescale_08 = XtVaCreateManagedWidget(
		"rescale_08",
		 smeBSBObjectClass,
		 rescalePopup,
		 XtNlabel,"0.8",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 NULL);

  rescale_09 = XtVaCreateManagedWidget(
		"rescale_09",
		 smeBSBObjectClass,
		 rescalePopup,
		 XtNlabel,"0.9",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 NULL);

  rescale_10 = XtVaCreateManagedWidget(
		"rescale_10",
		 smeBSBObjectClass,
		 rescalePopup,
		 XtNlabel,"1.0",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 NULL);

  rescale_12 = XtVaCreateManagedWidget(
		"rescale_12",
		 smeBSBObjectClass,
		 rescalePopup,
		 XtNlabel,"1.2",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 NULL);

  rescale_14 = XtVaCreateManagedWidget(
		"rescale_14",
		 smeBSBObjectClass,
		 rescalePopup,
		 XtNlabel,"1.4",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 NULL);

  rescale_16 = XtVaCreateManagedWidget(
		"rescale_16",
		 smeBSBObjectClass,
		 rescalePopup,
		 XtNlabel,"1.6",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 NULL);

  rescale_18 = XtVaCreateManagedWidget(
		"rescale_18",
		 smeBSBObjectClass,
		 rescalePopup,
		 XtNlabel,"1.8",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 NULL);

  rescale_20 = XtVaCreateManagedWidget(
		"rescale_20",
		 smeBSBObjectClass,
		 rescalePopup,
		 XtNlabel,"2.0",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 NULL);

  XtAddCallback(rescale_reset,XtNcallback,SetRescaleFactor,(XtPointer)Reset);
  XtAddCallback(rescale_05,XtNcallback,SetRescaleFactor,(XtPointer)ZeroFive);
  XtAddCallback(rescale_06,XtNcallback,SetRescaleFactor,(XtPointer)ZeroSix);
  XtAddCallback(rescale_07,XtNcallback,SetRescaleFactor,(XtPointer)ZeroSeven);
  XtAddCallback(rescale_08,XtNcallback,SetRescaleFactor,(XtPointer)ZeroEight);
  XtAddCallback(rescale_09,XtNcallback,SetRescaleFactor,(XtPointer)ZeroNine);
  XtAddCallback(rescale_10,XtNcallback,SetRescaleFactor,(XtPointer)OneZero);
  XtAddCallback(rescale_12,XtNcallback,SetRescaleFactor,(XtPointer)OneTwo);
  XtAddCallback(rescale_14,XtNcallback,SetRescaleFactor,(XtPointer)OneFour);
  XtAddCallback(rescale_16,XtNcallback,SetRescaleFactor,(XtPointer)OneSix);
  XtAddCallback(rescale_18,XtNcallback,SetRescaleFactor,(XtPointer)OneEight);
  XtAddCallback(rescale_20,XtNcallback,SetRescaleFactor,(XtPointer)TwoZero);

}

 
/*******************************************************************************
Name: KillCurrentBuffer
        
--------------------------------------------------------------------------------
Funktionstyp: Callback vom 'Really kill..' Requester
        
*******************************************************************************/
static void KillCurrentBuffer(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
  Boolean committed = client_data == NULL ? False : True;
  String curr_name;
  if(committed)
  {
    XtVaGetValues(getCurrentNet(),
                    XtNmodelName,&curr_name,
                    NULL);
    killBuffer(curr_name);
  }
}

/*******************************************************************************
Name: Kill
        
--------------------------------------------------------------------------------
Funktionstyp: Callback von filePopup/kill Buffer
        
*******************************************************************************/
static void Kill(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
  String curr_name;
  Boolean p_mod,n_mod;

  XtVaGetValues(getCurrentNet(),
                    XtNpictureModified,&p_mod,
                    XtNnetModified,&n_mod,
                    XtNmodelName,&curr_name,
                    NULL);
      if (p_mod || n_mod)
    {
      char question[MAXPATHLEN+30];
      Position x,y;
      Dimension height;

      XtVaGetValues(bufferMenuButton,
                    XtNheight,&height,
                    NULL);
      XtTranslateCoords(bufferMenuButton,0,0,&x,&y);
      sprintf(question,"Really kill modified buffer %s ?",curr_name);
      Dialog(question,NULL,KillCurrentBuffer,x,y+(Position)height);
  }
  else KillCurrentBuffer(NULL,1L,NULL);
}

/*******************************************************************************
Name:
	Clear
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	XtNCalback des clear-buttons
--------------------------------------------------------------------------------
Zweck:
	Propagierung des Ereignisses, indem die Action 'clear' des NetWidgets
	fuer das aktuelle Modell aufgerufen wird.
*******************************************************************************/

static void Clear(w, client_data, call_data)
    Widget w;
    XtPointer client_data, call_data;
{
    ResetOptionButtons();
    XtCallActionProc(getCurrentNet(),"clear",NULL,NULL,0);
}

/*******************************************************************************
Name:
	SetGrid
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	XtNCallback aller grid buttons des grid-popup
--------------------------------------------------------------------------------
Parameter:
	client_data: entsprechende gewuenschte Gittergroesse
--------------------------------------------------------------------------------
Zweck:
	Setzen der Resource XtNgridsize des aktuellen Modelles auf den neuen
	Wert.
*******************************************************************************/

static void SetGrid(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
Gridsize gridsize = (Gridsize)client_data;

  XtVaSetValues(getCurrentNet(),
		XtNgridsize,gridsize,
		NULL);
}


/*******************************************************************************
Name:
        Redraw
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        redrawButton
--------------------------------------------------------------------------------
Zweck:
        Varanlasst Behandlung des Ereignisses
*******************************************************************************/

static void Redraw(w, client_data, call_data)
    Widget w;
    XtPointer client_data, call_data;
{
    XtCallActionProc(getCurrentNet(),"redraw",NULL,NULL,0);
}

/*******************************************************************************
Name:
        SetRescaleFactor
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        allen Men"ueintr"agen des rescalePopup
--------------------------------------------------------------------------------
Parameter:
        client_data: entsprechender Faktor
--------------------------------------------------------------------------------
Zweck:
        Veranlasst Behandlung des Ereignisses
*******************************************************************************/
	
static void SetRescaleFactor(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
RescaleFactor factor = (RescaleFactor)client_data;

  RescaleNet(getCurrentNet(),factor);
}

/* not implemented yet */
static void Undo(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
#ifdef UNDO
    UndoNet(getCurrentNet());
    XtCallActionProc(getCurrentNet(),"redraw",NULL,NULL,0);
#else
    printf("undo not installed\n");
#endif
}


