/*******************************************************************************
Modul           :       evalMenu.c

Purpose         :       Realisierung der Evaluation-Menues
			Initialisierung aller zugehoerigen Popups (aus optionPopups.c)

Programmer      :       Kolja Koischwitz
			transientes von Joerg Mitzlaff in 10.94

History         :       release 1: January 1994
                        release 2: Sommer 1996 (Heiko Bergmann)
*******************************************************************************/

#include <X11/X.h>
#include <X11/Intrinsic.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/

#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Toggle.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/AsciiText.h>

/*#include <Xm/Xm.h>*/
#include <X11/Shell.h>

#include "NetWidget.h"
#include "layout.h"
#include "environment.h"
#include "Options.h"
#include "inputCheck.h"
#include "bitmaps/toggle.bm"

#define TOGGLE_LEFT_MARGIN 16
#define MENU_RIGHT_MARGIN 12

extern ExperimentOptions experimentOptions; /* aus solveOps.c */

enum {AnalyticSolution=0,Approximation,Simulation,TransientSolution,TransientSimulation};
static char *experiment_title[]={
"Analytic Experiment",
"Approximation Experiment",
"Simulation Experiment",
"Transient Experiment",
"Transient Simulation Experiment",
"Discret Time Analytic Experiment"};
static int evaluationMode;

/* new Timemodes */
enum {ContTime=0,DiscTime};
static int timeMode;
static Boolean Started = False ;  
Pixmap toggle_pixmap; /* wird nach optionButton.c und pmfEdit.c exportiert.. */

/* Exportierte Funktionen */
void initEvaluationMenu(); /* kommt nach main.c */

int GetTimeMode();         /* kommt nach netWidget.c */

/*void onlySimulation(int state);*/

/* Callback Functions */
static void ToggleEvaluationMode();
static void ToggleTimeMode();  /*new*/
static void StartEvaluation();
static void ShowOptionsPopup();
static void ExperimentCallback();
static void ExperimentPopupLayout();
static void CheckSimulationAvailability();

/* Lokal */
static void initExperimentPopup();
static void ExperimentPopup();

/* Importierte Funktionen */

/*new objectToggles */
extern void SetGroupToggle();

/*netNameText.c*/
extern void setDisplayedNetType();

/* from resultEdit.c */
extern Widget resultEditor;

/* From optionPopups.c */
extern void initOptionPopups();

extern void TransientSolutionPopup();
extern void TransientSimulationPopup();
extern void AnalyticSolutionPopup();
extern void ApproximationPopup();
extern void SimulationPopup();
extern void DTAnalyticPopupLayout();
extern void DTTransientPopupLayout();

/* From solveOps.c */
extern void StartTransientSolution();
extern void StartTransientSimulation();    /* --- noch nicht implentiert  ---*/
extern void StartAnalyticSolution();
extern void StartApproximation();
extern void StartSimulation();
extern void StartDTSPNAnalyticSolution();
extern void StartDTSPNTransientSolution();


extern void StartTransientExperiment();
extern void StartTransientSimulationExperiment();   /* --- noch nicht implentiert --- */
extern void StartAnalyticExperiment();
extern void StartApproximationExperiment();
extern void StartSimulationExperiment();
/*new*/
extern void StartDTAnalyticExperiment();
 
extern Widget getCurrentNet();
extern Widget topLevel; /* from main.c */
extern void replaceInfoString();

Widget rewardMeasureEntry; /* Exported to resultEdit.c */

static Widget /* Widgets des Menues */
	evalMenuButton,
	evalPopup,
        /* TimeMode */
	conttimeToggle,
	disctimeToggle,
        /* transient Solution */
	transientToggle,
	transientsimToggle,
	/* static Solution */
	analyticToggle,
	approximationToggle,
	simulationToggle,
	/*------------------*/
	startEvalEntry,
	experimentEntry,
	optionsEntry;

static Widget /* Widgets des Experiment Popups */
	experimentPopup,
	EmainForm,
	EvaryingParameterLabel,
	EvaryingParameterText,
	EparameterTypeLabel,
	EdelaySwitchToggle,
	EmarkingSwitchToggle,
	EfromValueLabel,
	EfromValueText,
	EtoValueLabel,
	EtoValueText,
	EstepValueLabel,
	EstepValueText,
	EstartExperimentButton,
	ElastValuesButton,
	EcancelButton;
char EvaryingParameterBuffer[VAL_BUF],EfromValueBuffer[VAL_BUF],
     EtoValueBuffer[VAL_BUF],EstepValueBuffer[VAL_BUF];
enum {startExperimentButton,lastValuesButton,cancelButton};

/* forward definition of actions */
void ChangeKeyboardFocus();

KeyboardFocus experiment_focus=
{0,4,&experimentPopup,{&EvaryingParameterText,&EfromValueText,&EtoValueText,&EstepValueText}};

extern KeyboardFocus transient_focus, transientsim_focus, analytic_focus, 
approximation_focus, simulation_focus, dtanalytic_focus, dttransient_focus;

#define NUM_KEYBOARD_FOCUS 8
KeyboardFocus *keyboard_focus[]=
	{&transient_focus, &transientsim_focus, &experiment_focus, 
	 &analytic_focus, &approximation_focus, &simulation_focus, &dtanalytic_focus, &dttransient_focus};

static char trans_toggle[] = "<Btn1Down>,<Btn1Up>: set() notify()";
XtTranslations toggle_transtable; /* wird auch nach optionPopups.c exportiert */

/* Erweiterte Translations fuer Text-Widgets (auch in optionPopups.c) */
static char text_translations[] = "\
	Ctrl<Key>F:             forward-character()             \n\
	Ctrl<Key>B:             backward-character()            \n\
	Ctrl<Key>D:             delete-next-character()         \n\
	Ctrl<Key>A:             beginning-of-line()             \n\
	Ctrl<Key>E:             end-of-line()                   \n\
	Ctrl<Key>H:             delete-previous-character()     \n\
	Ctrl<Key>K:                 kill-to-end-of-line()               \n\
	Ctrl<Key>W:             kill-selection()                \n\
	Ctrl<Key>:                  no-op()                             \n\
	Shift<Key>Right:        end-of-line()           \n\
	Shift<Key>Left:         beginning-of-line()   \n\
	    <Key>Right:         forward-character()             \n\
	    <Key>Left:          backward-character()   \n\
	<Key>Up:        change-exp-keyboard-focus(UP)   \n\
	<Key>Down:      change-exp-keyboard-focus(DOWN) \n\
	    <Key>Delete:        delete-previous-character()\n\
	    <Key>BackSpace:     delete-previous-character()\n\
	    <Key>Linefeed:      no-op()\n\
	    <Key>Return:        no-op()\n\
   Shift<Key>Tab:       change-exp-keyboard-focus(UP)\n\
	<Key>Tab:       change-exp-keyboard-focus(DOWN)\n\
	    <Key>:                  insert-char()\n\
	    <Btn1Down>:         change-exp-keyboard-focus(SET) select-start()\n\
	    <Btn1Motion>:       extend-adjust()                 \n\
	    <Btn1Up>:           extend-end(PRIMARY,CUT_BUFFER0) \n\
	    <Btn2Down>:         change-exp-keyboard-focus(SET) insert-selection(PRIMARY,CUT_BUFFER0)\n\
	    <Btn3Down>:         change-exp-keyboard-focus(SET) extend-start()                   \n\
	    <Btn3Motion>:       extend-adjust()                 \n\
	    <Btn3Up>:           extend-end(PRIMARY,CUT_BUFFER0) ";

static char text_accelerator_table[] = "\
	    <KeyPress>Right:            forward-character()     \n\
	    <KeyPress>Left:             backward-character() \n\
	    <KeyPress>Delete:           delete-previous-character()     \n\
	    <KeyPress>BackSpace:        delete-previous-character()     \n\
	    <KeyPress>Linefeed:         no-op() \n\
	    <KeyPress>Return:           no-op() \n\
	    <KeyPress>:                 insert-char()";

static char ok_button_acc_table[]="  <KeyPress>Return: 		set() notify() unset()";
	    
static XtActionsRec solve_rec[] = {
			{"change-exp-keyboard-focus",ChangeKeyboardFocus},
			};

XtTranslations text_transtable; /* Exportiert nach optionPopups.c */
XtAccelerators text_accelerators; /* Exportiert nach optionPopups.c */
XtAccelerators ok_button_accelerators; /* Exportiert nach optionPopups.c */


/*******************************************************************************
Name:
	query_mouse
--------------------------------------------------------------------------------
Zweck:
	Findet die aktuelle Mouse-Position heraus
*******************************************************************************/
static void query_mouse(w,x,y)
Widget w; int *x,*y;
{
	Window client,root; int wx,wy,mask;
	
	XQueryPointer(XtDisplay(w),XtWindow(w),&root,&client,
	  x,y,&wx,&wy,&mask);
}

/*******************************************************************************
Name:
	CenterWidget
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
Aufgerufen aus:
	Initialisierungsroutinen diverser OptionPopups
--------------------------------------------------------------------------------
Zweck:
	Positionieren eines Popups (Mausorientiert/Screenmitte)
*******************************************************************************/
void CenterWidget(w)
Widget w;
{
    Position x,y; int mousex,mousey;
	Dimension height,width,screen_width,screen_height;
    int auto_realized=0;
	
	if(!XtIsRealized(w))
	{
	  XtVaSetValues(w,XtNmappedWhenManaged,False,NULL);
      XtRealizeWidget(w);
	  auto_realized=1;
    }
    XtVaGetValues(w,XtNwidth,&width,XtNheight,&height,NULL);
    screen_width=(Dimension)WidthOfScreen(XtScreen(w));
    screen_height=(Dimension)HeightOfScreen(XtScreen(w));
    
	query_mouse(w,&mousex,&mousey);
	x=mousex-width/2;
	y=mousey-height/2;

    if (y < 0) y = 0;
      else
	if (y + (Position)height > (Position)screen_height)
	  y = (Position)screen_height - (Position)height;
      
    if (x < 0) x = 0;
      else
	if (x + (Position)width > (Position)screen_width )
	  x = (Position)screen_width - (Position)width;

    XtMoveWidget(w,x,y);
    
	if(auto_realized)
	{  
	  XtUnrealizeWidget(w);
      XtVaSetValues(w,XtNmappedWhenManaged,True,NULL);
    }
}

/*******************************************************************************
Name:
	CalculateButtonWidth
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
Aufgerufen aus:
	Initialisierungsroutinen diverser OptionPopups
--------------------------------------------------------------------------------
Zweck:
	*einfache* Berechnung einer einheitlichen Buttonbreite =>
	es sollte nicht zuviel Text in den Buttons sein!
*******************************************************************************/
Dimension CalculateButtonWidth(w,bnum)
Widget w; int bnum;
{
    Dimension width; int distance;

    XtVaGetValues(w,XtNwidth,&width,XtNdefaultDistance,&distance,NULL);
	return((Dimension)(width-(bnum+1)*distance)/bnum-2);
}


/*******************************************************************************
Name:
	WidgetWidth
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
Aufgerufen aus:
	Initialisierungsroutinen diverser OptionPopups
--------------------------------------------------------------------------------
Zweck:
	gibt Breite eines Widgets zurueck
*******************************************************************************/
Dimension WidgetWidth(w)
Widget w;
{       Dimension width;
	XtVaGetValues(w,XtNwidth,&width,NULL);
	return width;
}

/*******************************************************************************
Name:
	getKeyboardFocusStructure
--------------------------------------------------------------------------------
Funktionstyp:
	private Function
--------------------------------------------------------------------------------
Benutzt in:
	SetCaretsCallback, ChangeKeyboardFocus
--------------------------------------------------------------------------------
Zweck:
	Bestimmt die KeyboardFocus-Struktur, die zu dem Popup des ueber-
	gebenen Widgets gehoert
*******************************************************************************/
static KeyboardFocus *getKeyboardFocusStructure(w)
Widget w;
{       int i=0;
    while(w && XtClass(w)!=transientShellWidgetClass) w=XtParent(w);
	for(i=0;i<NUM_KEYBOARD_FOCUS;i++)
	   if(w==*keyboard_focus[i]->shell) return keyboard_focus[i];
	puts("getKeyboardFocusStructure called with unknown Widget..");
	return NULL;
}

/*******************************************************************************
Name:
	SetCaretsCallback
--------------------------------------------------------------------------------
Funktionstyp:
	Callback (public)
--------------------------------------------------------------------------------
Registriert mit:
	XtNPopup der Options- und des ExperimentPopups  
--------------------------------------------------------------------------------
Zweck:
	setzt die Carets (Cursor) aller Text-Children ans Ende
	setzt den KeyboardFocus des Popups auf das aktuelle TextWidget
*******************************************************************************/
void SetCaretsCallback(w,client_data,call_data)
Widget w;
XtPointer client_data,call_data;
{       
	Widget Form=(Widget)client_data,*child; int i,num_children;
	String s;
	KeyboardFocus *f=getKeyboardFocusStructure(w);

	XtVaGetValues(Form,XtNchildren,&child,XtNnumChildren,&num_children,NULL);
	for(i=0;i<num_children;i++)
		if(XtClass(child[i])==asciiTextWidgetClass)
		{
			XtVaGetValues(child[i],XtNstring,&s,NULL);
			XtVaSetValues(child[i],XtNinsertPosition,strlen(s),NULL);

		}
	XtSetKeyboardFocus(*f->shell,*f->field[f->curr]);
}

/*******************************************************************************
Name:
       ChangeKeyboardFocus
--------------------------------------------------------------------------------
Funktionstyp:
       Action (public, auch in optionPopups.c)
--------------------------------------------------------------------------------
triggert bei event:
	   <BtnDown>: changeExpKeyBoardFocus(SET)
       <Key>Tab: changeExpKeyBoardFocus(DOWN)
       Shift<Key>Tab: changeExpKeyBoardFocus(DOWN)
       <Key>Up: changeExpKeyBoardFocus(UP)
       <Key>Down: changeExpKeyBoardFocus(DOWN)
--------------------------------------------------------------------------------
Zweck:
	Aendern des Keyboardfocus der Textfelder eines Popups
*******************************************************************************/
void ChangeKeyboardFocus(w,event,params,numparams)
Widget w;
XEvent *event; 
String*params;
Cardinal* numparams;
{

FocusMovingDirection direction = DOWN;
KeyboardFocus *f=getKeyboardFocusStructure(w);

#ifdef debug
printf( "ChangeKeyboardfocus  \n");
#endif

  if (numparams && *numparams > 0)
  {
	  if (strcmp(params[0],"UP") == 0)
	    direction = UP;
      else
      if (strcmp(params[0],"DOWN") == 0)
	    direction = DOWN;
	  else if(strcmp(params[0],"SET") == 0)
	    direction = SET;
    }

  XtVaSetValues(*f->field[f->curr],XtNdisplayCaret,False,NULL);
 
  if (direction==DOWN)
    do
      f->curr=(f->curr+1)%f->num;
    while(!XtIsSensitive(*f->field[f->curr]));
  else if(direction==UP)
    do
     f->curr=(f->curr+f->num-1)%f->num;
   while(!XtIsSensitive(*f->field[f->curr]));

  else /* direction==SET */
  { 
    int i;
    for(i=0;i<f->num;i++) if(*f->field[i]==w)
	{       f->curr=i;
		break;
	}
  }

  XtSetKeyboardFocus(*f->shell,*f->field[f->curr]);
  XtVaSetValues(*f->field[f->curr],XtNdisplayCaret,True,NULL);
}

/*******************************************************************************
Name:
	initEvaluationMenu
--------------------------------------------------------------------------------
Funktionstyp:
	oeffentliche Funtion
--------------------------------------------------------------------------------
aufgerufen in:
	main.c
--------------------------------------------------------------------------------
Parameter:
	app_context: Applikation Context
	topLevel: toplevel-shell der Applikation
	parent:   Elterwidget des Buttons "Evaluation"
	initialEvalMode: TransientSolution,AnalyticSolution,Approximation oder Simulation
	initialTimeMode: ContTime oder DiscTime --new
--------------------------------------------------------------------------------
Zweck:
    Initialisierung des Buttons und des Menues
	Registrieren der Callback-Funktionen
	Toggle-Pixmap berechnen
	Initialisieren des Experiment-Popups
	Initialisieren der drei Option-Popups
	Parsieren der Uebersetzungstafeln.
	Registrieren von hier benoetigten Applikationsaktionen.
*******************************************************************************/

void initEvaluationMenu(app_context,topLevel,parent,initialEvalMode,initialTimeMode)
XtAppContext app_context;
Widget topLevel,parent;
int initialEvalMode;
int initialTimeMode;
{ 
  XtAppAddActions(app_context,solve_rec,XtNumber(solve_rec));
  text_transtable = XtParseTranslationTable(text_translations);
  text_accelerators = XtParseAcceleratorTable(text_accelerator_table);
  ok_button_accelerators = XtParseAcceleratorTable(ok_button_acc_table);
  toggle_transtable = XtParseTranslationTable(trans_toggle);
  toggle_pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
			RootWindowOfScreen(XtScreen(topLevel)),
			toggle_bits,toggle_width,toggle_height);

  evalMenuButton = XtVaCreateManagedWidget(
		"evalMenuButton",          
		menuButtonWidgetClass, 
		parent, 
		XtNlabel,"Evaluation",
		XtNmin,COMMAND_BUTTON_WIDTH,
		XtNheight,COMMAND_BUTTON_HEIGHT,
		XtNjustify,COMMAND_BUTTON_JUSTIFY,
		XtNmenuName,"evalPopup",      /* name of Popup-Shell */
		NULL);             

  evalPopup = XtVaCreatePopupShell(
		"evalPopup",
		simpleMenuWidgetClass,
		topLevel,
		NULL);
  XtAddCallback(evalPopup,XtNpopupCallback,CheckSimulationAvailability,NULL);

  rewardMeasureEntry = XtVaCreateManagedWidget(
		"rewardMeasureEntry",         
		smeBSBObjectClass, 
		evalPopup, 
		XtNlabel,"Reward Measures..",
		XtNjustify,MENU_ENTRY_JUSTIFY,
		XtNrightMargin,MENU_RIGHT_MARGIN,
		NULL);  
  XtAddCallback(rewardMeasureEntry,XtNcallback,XtCallbackNone,resultEditor);

  XtVaCreateManagedWidget("seperator11", smeLineObjectClass, evalPopup, XtNlineWidth,2, NULL);

   conttimeToggle = XtVaCreateManagedWidget(           /*new*/
		"conttimeToggle",
		 smeBSBObjectClass,
		 evalPopup,
		 XtNlabel,"Continous Time",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		XtNrightMargin,MENU_RIGHT_MARGIN,
		 XtNleftMargin,TOGGLE_LEFT_MARGIN,
		 NULL);

   disctimeToggle = XtVaCreateManagedWidget(           /*new*/
		"disctimeToggle",
		 smeBSBObjectClass,
		 evalPopup,
		 XtNlabel,"Discrete Time",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		XtNrightMargin,MENU_RIGHT_MARGIN,
		 XtNleftMargin,TOGGLE_LEFT_MARGIN,
		 NULL);           

  XtVaCreateManagedWidget("seperator12", smeLineObjectClass, evalPopup, XtNlineWidth,2, NULL);/*new*/

   analyticToggle = XtVaCreateManagedWidget(
		"analyticToggle",
		 smeBSBObjectClass,
		 evalPopup,
		 XtNlabel,"Stationary Analysis",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		XtNrightMargin,MENU_RIGHT_MARGIN,
		 XtNleftMargin,TOGGLE_LEFT_MARGIN,
		 NULL);

  approximationToggle = XtVaCreateManagedWidget(
		"approximationToggle",
		 smeBSBObjectClass,
		 evalPopup,
		 XtNlabel,"Stationary Approximation",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		XtNrightMargin,MENU_RIGHT_MARGIN,
		 XtNleftMargin,TOGGLE_LEFT_MARGIN,
		 NULL);

  simulationToggle = XtVaCreateManagedWidget(
		"simulationToggle",
		 smeBSBObjectClass,
		 evalPopup,
		 XtNlabel,"Stationary Simulation",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 XtNrightMargin,MENU_RIGHT_MARGIN,
		 XtNleftMargin,TOGGLE_LEFT_MARGIN,
		 NULL);

  XtVaCreateManagedWidget("seperator15",smeLineObjectClass,evalPopup,NULL);

   transientToggle = XtVaCreateManagedWidget(
		"transientToggle",
		 smeBSBObjectClass,
		 evalPopup,
		 XtNlabel,"Transient Analysis",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 XtNrightMargin,MENU_RIGHT_MARGIN,
		 XtNleftMargin,TOGGLE_LEFT_MARGIN,
		 NULL);

   transientsimToggle =  XtVaCreateManagedWidget(
		"transientsimToggle",
		 smeBSBObjectClass,
		 evalPopup,
		 XtNlabel,"Transient Simulation",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 XtNrightMargin,MENU_RIGHT_MARGIN,
		 XtNleftMargin,TOGGLE_LEFT_MARGIN,
		 NULL);

  XtVaCreateManagedWidget("seperator2",smeLineObjectClass,evalPopup,XtNlineWidth,2,NULL);
  
  startEvalEntry = XtVaCreateManagedWidget(
		"startEvalButton",
		smeBSBObjectClass,
		evalPopup,
		XtNlabel,"Start Evaluation",
		XtNjustify,MENU_ENTRY_JUSTIFY,
		XtNrightMargin,MENU_RIGHT_MARGIN,
		NULL);

  experimentEntry = XtVaCreateManagedWidget(
		"experimentButton",
		smeBSBObjectClass,
		evalPopup,
		XtNlabel,"Experiment..",
		XtNjustify,MENU_ENTRY_JUSTIFY,
		XtNrightMargin,MENU_RIGHT_MARGIN,
		NULL);

  optionsEntry = XtVaCreateManagedWidget(
		"optionsButton",
		smeBSBObjectClass,
		evalPopup,
		XtNlabel,"Options..",
		XtNjustify,MENU_ENTRY_JUSTIFY,
		XtNrightMargin,MENU_RIGHT_MARGIN,
		NULL);

  XtAddCallback(transientToggle,XtNcallback,ToggleEvaluationMode,(XtPointer)TransientSolution);
  XtAddCallback(transientsimToggle,XtNcallback,ToggleEvaluationMode,(XtPointer)TransientSimulation);
  XtAddCallback(analyticToggle,XtNcallback,ToggleEvaluationMode,(XtPointer)AnalyticSolution);
  XtAddCallback(approximationToggle,XtNcallback,ToggleEvaluationMode,(XtPointer)Approximation);
  XtAddCallback(simulationToggle,XtNcallback,ToggleEvaluationMode,(XtPointer)Simulation);

           /*new*/
  XtAddCallback(conttimeToggle,XtNcallback,ToggleTimeMode,(XtPointer)ContTime);
  XtAddCallback(disctimeToggle,XtNcallback,ToggleTimeMode,(XtPointer)DiscTime);

  XtAddCallback(startEvalEntry,XtNcallback,StartEvaluation,NULL);
  XtAddCallback(experimentEntry,XtNcallback,ExperimentPopup,NULL);
  XtAddCallback(optionsEntry,XtNcallback,ShowOptionsPopup,NULL);

  /* Initialisierung des Experiment- und der Options Popups */
  initOptionPopups(topLevel);
  initExperimentPopup(topLevel);
  ExperimentPopupLayout();   

    /* Initiale Toggle-Markierung setzen */
  ToggleEvaluationMode(NULL,(XtPointer)((long)initialEvalMode),NULL);
/*new*/
  ToggleTimeMode(NULL,(XtPointer)((long)initialTimeMode),NULL);
  Started=True;
}

/*******************************************************************************
Name:
	ToggleEvaluationMode
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	analyticToggle,approximationToggle,simulationToggle
--------------------------------------------------------------------------------
Zweck:
	Umschalten des Evaluierungs-Verfahrens, setzen der Togglemarkierung
*******************************************************************************/

static void ToggleEvaluationMode(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
  XtVaSetValues(transientToggle,XtNleftBitmap,None,NULL);
  XtVaSetValues(transientsimToggle,XtNleftBitmap,None,NULL);
  XtVaSetValues(analyticToggle,XtNleftBitmap,None,NULL);
  XtVaSetValues(approximationToggle,XtNleftBitmap,None,NULL);
  XtVaSetValues(simulationToggle,XtNleftBitmap,None,NULL);
  
  switch(evaluationMode=(int)((long)client_data))
  {
  case TransientSolution:
	  XtVaSetValues(transientToggle,XtNleftBitmap,toggle_pixmap,NULL);
	  ToggleTimeMode(NULL,(XtPointer)((long)timeMode),NULL);
	  break;
  case TransientSimulation:
	  XtVaSetValues(transientsimToggle,XtNleftBitmap,toggle_pixmap,NULL);
	  break;
  case AnalyticSolution:
	  XtVaSetValues(analyticToggle,XtNleftBitmap,toggle_pixmap,NULL);
	  ToggleTimeMode(NULL,(XtPointer)((long)timeMode),NULL);
	  break;
  case Approximation:
	  XtVaSetValues(approximationToggle,XtNleftBitmap,toggle_pixmap,NULL);
	  break;
  case Simulation:
	  XtVaSetValues(simulationToggle,XtNleftBitmap,toggle_pixmap,NULL);
	  break;
  default: /* Falscher Wert uebergeben! */
	  evaluationMode=AnalyticSolution;
	  XtVaSetValues(analyticToggle,XtNleftBitmap,toggle_pixmap,NULL);
  }
  /* Titel sofort anpassen, falls das Popup schon gePopupt ist.. */

  if ((timeMode==DiscTime) && (evaluationMode==AnalyticSolution))
    XtVaSetValues(experimentPopup,XtNtitle,
		  (XtArgVal)experiment_title[evaluationMode+5],NULL);
  else 
    XtVaSetValues(experimentPopup,XtNtitle,
		  (XtArgVal)experiment_title[evaluationMode],NULL);
}

/*new*/
/*******************************************************************************
Name:
	ToggleTimeMode
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	conttimeToggle,disctimeToggle
--------------------------------------------------------------------------------
Zweck:
	Umschalten des Time Mode, setzen der Togglemarkierung
*******************************************************************************/

static void ToggleTimeMode(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{ TimeNetType net_type;
  Widget nw;
    if (!Started) net_type=TN_GSPN;
    else  {  nw=getCurrentNet();
	     XtVaGetValues(nw,XtNnetType,&net_type,NULL);
           }
  XtVaSetValues(conttimeToggle,XtNleftBitmap,None,NULL);
  XtVaSetValues(disctimeToggle,XtNleftBitmap,None,NULL);

  switch(timeMode=(int)((long)client_data))
  {
  case ContTime:
	  XtVaSetValues(conttimeToggle,XtNleftBitmap,toggle_pixmap,NULL);
	  setDisplayedNetType(net_type==TN_ESPN?"EDSPN":net_type==TN_DSPN?"DSPN":"GSPN");
	  XtSetSensitive(approximationToggle,True);
	  XtSetSensitive(simulationToggle,True);
	  XtSetSensitive(transientsimToggle,True);
	  XtSetSensitive(experimentEntry,True);
	  break;
  case DiscTime:
          if ((evaluationMode!=AnalyticSolution) && (evaluationMode!=TransientSolution))
	    ToggleEvaluationMode(NULL,NULL,(XtPointer)((long)AnalyticSolution));
	  XtVaSetValues(disctimeToggle,XtNleftBitmap,toggle_pixmap,NULL);
	  setDisplayedNetType("DDSPN");
	  XtSetSensitive(approximationToggle,False);
	  XtSetSensitive(simulationToggle,False);
	  XtSetSensitive(transientsimToggle,False);
	  if (evaluationMode==AnalyticSolution) XtSetSensitive(experimentEntry,True);
	  else XtSetSensitive(experimentEntry,False);
	  break;
  default: /* Falscher Wert uebergeben! */
	  timeMode=ContTime;
	  XtVaSetValues(conttimeToggle,XtNleftBitmap,toggle_pixmap,NULL);
	  setDisplayedNetType(net_type==TN_ESPN?"EDSPN":net_type==TN_DSPN?"DSPN":"GSPN");
	  XtSetSensitive(simulationToggle,True);
	  XtSetSensitive(transientsimToggle,True);
	  XtSetSensitive(experimentEntry,True);
	  XtSetSensitive(approximationToggle,True); 
  }
  if (Started) SetGroupToggle(timeMode);
  if (Started) replaceInfoString();
}


/*******************************************************************************
Name:
	StartEvaluation
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	startEvalEntry
--------------------------------------------------------------------------------
Zweck:
	Evaluierung entsprechend dem aktuellen <evaluationMode>/<timeMode> starten.
*******************************************************************************/

static void StartEvaluation(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{  
  switch(evaluationMode)
  {
  case TransientSolution:
    {
	  if(timeMode==DiscTime) StartDTSPNTransientSolution();    /*new*/
	  else StartTransientSolution();
	  break;
    }
  case TransientSimulation:
	  StartTransientSimulation();
	  break;
  case AnalyticSolution:
    {
	  if(timeMode==DiscTime) StartDTSPNAnalyticSolution();    /*new*/
	  else StartAnalyticSolution();
	  break;
    }
  case Approximation:
	  StartApproximation();
	  break;
  case Simulation:
	  StartSimulation();
	  break;
  }
}

/*******************************************************************************
Name:
	ShowOptionsPopup
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	optionsEntry
--------------------------------------------------------------------------------
Zweck:
	Anzeigen des zum aktuellen <evaluationMode> passenden Options-Popup.
*******************************************************************************/

static void ShowOptionsPopup(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{  
  switch(evaluationMode)
  {
  case TransientSolution:
          if (timeMode==DiscTime) DTTransientOptionsPopup(); /*new*/
	  else TransientOptionsPopup();
	  break;
  case TransientSimulation:
	  TransientSimulationOptionsPopup();
	  break;
  case AnalyticSolution:
          if (timeMode==DiscTime) DTAnalyticOptionsPopup(); /*new*/
	  else AnalyticOptionsPopup();
	  break;
  case Approximation:
	  ApproximationOptionsPopup();
	  break;
  case Simulation:
	  SimulationOptionsPopup();
	  break;
  }
}

/*******************************************************************************
Name:
	initExperimentPopup
--------------------------------------------------------------------------------
Funktionstyp:
	private function
--------------------------------------------------------------------------------
aufgerufen von:
	initEvaluationMenu()
--------------------------------------------------------------------------------
Parameter:
	topLevel: toplevel-shell der Applikation
--------------------------------------------------------------------------------
Zweck:
	Aufbau des Experiment Popups
	Registrieren der erforderlichen Callbacks.
*******************************************************************************/
static void initExperimentPopup(topLevel)
Widget topLevel;
{ 
  experimentPopup = XtVaCreatePopupShell(
		"Options for Experiment",
		transientShellWidgetClass,
		topLevel,
		NULL);
  XtAddCallback(experimentPopup,XtNpopupCallback,CenterWidget,NULL);
  
  EmainForm = XtVaCreateManagedWidget(
		"EmainForm",
		formWidgetClass,
		experimentPopup,
		NULL);

  EvaryingParameterLabel = XtVaCreateManagedWidget(
		"EvaryingParameterLabel",
		labelWidgetClass,
		EmainForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"varying parameter: ",
		NULL);
  
  EvaryingParameterText = XtVaCreateManagedWidget(
		"EvaryingParameterText",
		asciiTextWidgetClass,
		EmainForm,
		XtNfromHoriz,EvaryingParameterLabel,
		XtNeditType,XawtextEdit,
		XtNstring,EvaryingParameterBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
/*              XtNaccelerators,text_accelerators,*/
		XtNdisplayCaret,True,
		NULL);
/*  XtInstallAccelerators(experimentPopup,EvaryingParameterText);*/

  EparameterTypeLabel = XtVaCreateManagedWidget(
		"EparameterTypeLabel",
		labelWidgetClass,
		EmainForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"parameter type: ",
		XtNfromVert,EvaryingParameterText,
		NULL);
  
  EdelaySwitchToggle = XtVaCreateManagedWidget(
		"EdelaySwitchToggle",
		toggleWidgetClass,
		EmainForm,
		XtNlabel,"delay",
		XtNradioData,Delay,
		XtNtranslations,toggle_transtable,
		XtNfromHoriz,EparameterTypeLabel,
		XtNfromVert,EvaryingParameterText,
		NULL);
  
  EmarkingSwitchToggle = XtVaCreateManagedWidget(
		"EmarkingSwitchToggle",
		toggleWidgetClass,
		EmainForm,
		XtNlabel,"marking",
		XtNradioData,Marking,
		XtNtranslations,toggle_transtable,
		XtNfromVert,EvaryingParameterText,
		XtNfromHoriz,EdelaySwitchToggle,
		NULL);
  
  XawToggleChangeRadioGroup(EmarkingSwitchToggle,EdelaySwitchToggle);
  
  EfromValueLabel = XtVaCreateManagedWidget(
		"EfromValueLabel",
		labelWidgetClass,
		EmainForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"from value: ",
		XtNfromVert,EmarkingSwitchToggle,
		NULL);
  
  EfromValueText = XtVaCreateManagedWidget(
		"EfromValueText",
		asciiTextWidgetClass,
		EmainForm,
		XtNfromVert,EmarkingSwitchToggle,
		XtNfromHoriz,EfromValueLabel,
		XtNeditType,XawtextEdit,
		XtNstring,EfromValueBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);
  
  EtoValueLabel = XtVaCreateManagedWidget(
		"EtoValueLabel",
		labelWidgetClass,
		EmainForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"to value: ",
		XtNfromVert,EfromValueText,
		NULL);
  
  EtoValueText = XtVaCreateManagedWidget(
		"EtoValueText",
		asciiTextWidgetClass,
		EmainForm,
		XtNfromVert,EfromValueText,
		XtNfromHoriz,EtoValueLabel,
		XtNeditType,XawtextEdit,
		XtNstring,EtoValueBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);

  EstepValueLabel = XtVaCreateManagedWidget(
		"EstepValueLabel",
		labelWidgetClass,
		EmainForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"step size: ",
		XtNfromVert,EtoValueText,
		NULL);
  
  EstepValueText = XtVaCreateManagedWidget(
		"EstepValueText",
		asciiTextWidgetClass,
		EmainForm,
		XtNfromVert,EtoValueText,
		XtNfromHoriz,EstepValueLabel,
		XtNeditType,XawtextEdit,
		XtNstring,EstepValueBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);
  
  EstartExperimentButton=XtVaCreateManagedWidget(
		"EstartExperimentButton",
		commandWidgetClass,
		EmainForm,
		XtNborderWidth,2,
		XtNfromVert,EstepValueText,
		XtNlabel,"Start Experiment",
		XtNaccelerators,ok_button_accelerators,
		NULL);
  XtInstallAccelerators(experimentPopup,EstartExperimentButton);
  XtAddCallback(EstartExperimentButton,XtNcallback,ExperimentCallback,
   (XtPointer)startExperimentButton);
  
  ElastValuesButton=XtVaCreateManagedWidget(
		"ElastValuesButton",
		commandWidgetClass,
		EmainForm,
		XtNfromVert,EstepValueText,
		XtNfromHoriz,EstartExperimentButton,
		XtNlabel,"Prev. values",
		NULL);
  XtAddCallback(ElastValuesButton,XtNcallback,ExperimentCallback,
   (XtPointer)lastValuesButton);
  
  EcancelButton=XtVaCreateManagedWidget(
		"EcancelButton",
		commandWidgetClass,
		EmainForm,
		XtNfromVert,EstepValueText,
		XtNfromHoriz,ElastValuesButton,
		XtNlabel,"Cancel",
		NULL);
  XtAddCallback(EcancelButton,XtNcallback,ExperimentCallback,
   (XtPointer)cancelButton);

}


/*******************************************************************************
Name:
	ExperimentPopupLayout
--------------------------------------------------------------------------------
Funktionstyp:
	private Function
--------------------------------------------------------------------------------
aufgerufen aus:
	initEvaluationMenu
--------------------------------------------------------------------------------
Zweck:
	Noch n bissl am Layout feilen..
*******************************************************************************/
static void ExperimentPopupLayout()
{  
  Dimension width,height,mfwidth; int dd;
  
  XtVaSetValues(experimentPopup,XtNmappedWhenManaged,False,NULL);
  XtRealizeWidget(experimentPopup);

  XtVaGetValues(EmainForm,XtNdefaultDistance,&dd,XtNwidth,&mfwidth,NULL);
  
  XtVaSetValues(EvaryingParameterText,XtNhorizDistance,
   mfwidth-WidgetWidth(EvaryingParameterText)-WidgetWidth(EvaryingParameterLabel)-2*dd,NULL);
  XtVaSetValues(EfromValueText,XtNhorizDistance,
   mfwidth-WidgetWidth(EfromValueText)-WidgetWidth(EfromValueLabel)-2*dd,NULL);
  XtVaSetValues(EtoValueText,XtNhorizDistance,
   mfwidth-WidgetWidth(EtoValueText)-WidgetWidth(EtoValueLabel)-2*dd,NULL);
  XtVaSetValues(EstepValueText,XtNhorizDistance,
   mfwidth-WidgetWidth(EstepValueText)-WidgetWidth(EstepValueLabel)-2*dd,NULL);

  XtVaSetValues(EdelaySwitchToggle,XtNhorizDistance,
    mfwidth-WidgetWidth(EdelaySwitchToggle)-
	  WidgetWidth(EmarkingSwitchToggle)-WidgetWidth(EparameterTypeLabel)-4*dd,NULL);
  
  XawFormDoLayout(EmainForm,True);
  XtUnrealizeWidget(experimentPopup);
  XtVaSetValues(experimentPopup,XtNmappedWhenManaged,True,NULL);
}

/*******************************************************************************
Name:
	setTextForWidget
--------------------------------------------------------------------------------
Funktionstyp:
	public Funktion
--------------------------------------------------------------------------------
aufgerufen von:
	diversen Initialisierungs-Routinen
--------------------------------------------------------------------------------
Zweck:
	Den Text eines TextWidgets ersetzen
*******************************************************************************/
void setTextForWidget(w, new)
Widget w; String  new;
{
  XawTextBlock tb; String curr; 
  
  tb.firstPos = 0;
  tb.length = strlen(new);
  tb.ptr = new;
  tb.format = FMT8BIT;
  XawTextReplace(w,0,(XawTextPosition)VAL_BUF,&tb);
}


/*******************************************************************************
Name:
	getTextOfWidget
--------------------------------------------------------------------------------
Funktionstyp:
	public Funktion
--------------------------------------------------------------------------------
aufgerufen von:
	diversen Popup-Auswerungs-Routinen
--------------------------------------------------------------------------------
Zweck:
	Den aktuellen Text eines Widgets in einen uebergebenen Bereich uebertragen
*******************************************************************************/
String getTextOfWidget(w)
Widget w;
{
  String curr;
  
  XtVaGetValues(w,XtNstring,&curr,NULL);
  return curr;
}

/*******************************************************************************
Name:
	ExperimentPopup
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	experimentEntry
--------------------------------------------------------------------------------
Zweck:  Reaktion auf Anwahl des Menueeintrags "Experiment":
	Textfelder loeschen und Experiment Popup anzeigen.
*******************************************************************************/
static void ExperimentPopup()
{
  /* Clear Edit Field */
  XawToggleSetCurrent(EmarkingSwitchToggle,
    (XtPointer)experimentOptions.parameter_type);
  setTextForWidget(EvaryingParameterText,"");
  setTextForWidget(EfromValueText,"");
  setTextForWidget(EtoValueText,"");
  setTextForWidget(EstepValueText,"");
  /*new*/
  if ((timeMode==DiscTime) && (evaluationMode==AnalyticSolution))
    XtVaSetValues(experimentPopup,XtNtitle,
		  (XtArgVal)experiment_title[evaluationMode+5],NULL);
  else 
    XtVaSetValues(experimentPopup,XtNtitle,
		  (XtArgVal)experiment_title[evaluationMode],NULL);
  XtPopup(experimentPopup,XtGrabNone);
  XtSetKeyboardFocus(experimentPopup,*experiment_focus.field[experiment_focus.curr]);
}

/*******************************************************************************
Name:
	ExperimentCallback
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	EstartExperimentButton,ElastValuesButton,EcancelButton
--------------------------------------------------------------------------------
Zweck:  Entsprechend auf den gedrueckten Batten reagieren, zB
	Experiment zum aktuellen <evaluationMode> starten.
		Bei illegalen Experiment Parametern mit Warnung zurueckkehren!
*******************************************************************************/

static void ExperimentCallback(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{  
  switch((long)client_data)
  {
	case startExperimentButton:
	{
	  String name,from,to,step;
	
	  experimentOptions.parameter_type=
	    (ParameterType)XawToggleGetCurrent(EmarkingSwitchToggle);
	  name=getTextOfWidget(EvaryingParameterText);
	  from=getTextOfWidget(EfromValueText);
	  to=getTextOfWidget(EtoValueText);
	  step=getTextOfWidget(EstepValueText);
      
	  if(!isIdentifier(name)
	  || (!parameterAvailable(getCurrentNet(),name,RATEPAR)
      && !parameterAvailable(getCurrentNet(),name,MARKPAR)))
      {   
	XtWarning("Not an existing Parameter in field \"parameter name\"!");
		return;
      }
      if(experimentOptions.parameter_type==Delay)
      {
	if (!isFloat(from) || !isFloat(to) || !isFloat(step))
	    {
	       XtWarning("Delay parameter must be a real positive value!");
	       return;
	    }
      }
      else /* parameter_type == Marking */
      {
	if (!isMarking(from) || !isMarking(to) || !isMarking(step))
	{
	       XtWarning("Marking parameter must be a positive integer <= 10000!");
	       return;
	}
      }
	  strcpy(experimentOptions.parameter_name,name);
	  strcpy(experimentOptions.from_value,from);
	  strcpy(experimentOptions.to_value,to);
	  strcpy(experimentOptions.step_value,step);
  
      XtPopdown(experimentPopup);
	  switch(evaluationMode)
      {
	   case AnalyticSolution:
	     if (timeMode==DiscTime) StartDTAnalyticExperiment();
	     else StartAnalyticExperiment();
	     break;
	   case Approximation:
	     StartApproximationExperiment();
	     break;
	   case Simulation:
	     StartSimulationExperiment();
	     break;
	  }
	  break;
    }
	case cancelButton:
      XtPopdown(experimentPopup);
	break;
	case lastValuesButton:
	  XawToggleSetCurrent(EmarkingSwitchToggle,
	    (XtPointer)experimentOptions.parameter_type);
	  setTextForWidget(EvaryingParameterText,
	    experimentOptions.parameter_name);
	  setTextForWidget(EfromValueText,experimentOptions.from_value);
	  setTextForWidget(EtoValueText,experimentOptions.to_value);
	  setTextForWidget(EstepValueText,experimentOptions.step_value);
	break;
    }
}


/*******************************************************************************
Name:
	CheckSimulationAvailability
--------------------------------------------------------------------------------
Funktionstyp:
	Callback 
--------------------------------------------------------------------------------
registriert mit:
	Popup des Evaluation-Menus
--------------------------------------------------------------------------------
Zweck:
	bei ESPN und Toggle auf Simulation: "Start Evaluation" disablen
*******************************************************************************/
static void CheckSimulationAvailability()
{
	TimeNetType net_type;
	Widget nw = getCurrentNet();
	
	XtVaGetValues(nw,XtNnetType,&net_type,NULL);
	XtVaSetValues(startEvalEntry,
		XtNsensitive,(Boolean)(net_type!=TN_ESPN || evaluationMode!=Approximation),
		NULL);
	
}

/*******************************************************************************
Name:
	GetPREC
--------------------------------------------------------------------------------
Funktionstyp:
	public 
--------------------------------------------------------------------------------
aufgerufen von:
	diversen Routinen zur Text-Formatierung..
--------------------------------------------------------------------------------
Rueckgabewert:
	Precision des aktuellen Netzes
--------------------------------------------------------------------------------
Zweck:
	Format-Parameter fuer die Anzeige der Results
*******************************************************************************/
Cardinal GetPREC()
{  
  Cardinal prec;
  
/*  XtVaGetValues(getCurrentNet(),
		XtNprecision,&prec,
		NULL);*/
		
  return 11;
}

/*******************************************************************************
Name:
	GetTimeMode
--------------------------------------------------------------------------------
Funktionstyp:
	public 
--------------------------------------------------------------------------------
aufgerufen von:
	Redisplay in NetWidget.c
--------------------------------------------------------------------------------
Rueckgabewert:
             TimeMode
--------------------------------------------------------------------------------
Zweck:
	Parameter fuer Anzeige des Netztypes
*******************************************************************************/
int GetTimeMode()
{
  return timeMode;
}


/*******************************************************************************
void onlySimulation(int state)
*******************************************************************************/
/*void onlySimulation(state)
int state;
{
 XtSetSensitive(rewardMeasureEntry,!state);
 XtSetSensitive(analyticToggle,!state);
 XtSetSensitive(approximationToggle,!state);
 XtSetSensitive(transientToggle,!state);
 XtSetSensitive(transientsimToggle,!state);
 ToggleEvaluationMode(simulationToggle,(XtPointer)Simulation,NULL);
}
 */






