/*******************************************************************************
Modul   	:	fileButton.c

Purpose		: 	Realisierung der Menues zum Sichern, Laden von Modellen,
                zum Loeschen von bestimmten Files sowie
				zum Beenden des Programms

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
				release 2: Maerz 1994   Kolja Koischwitz
*******************************************************************************/


#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h> 
#include <sys/stat.h>
#include <sys/param.h>

#include <X11/Intrinsic.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/

#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/Dialog.h>

#include "NetWidget.h"

#include "layout.h"
#include "environment.h"


#define SAVE 0
#define LOAD 1

typedef enum _RemoveMode { Results,Model,Directory,Hierarchy} RemoveMode;
extern int sys_nerr;
extern char *sys_errlist[];

extern Widget getCurrentNet();
extern Pixmap rightPtr;
extern void SetSubmenuPopedupFlag();
extern Widget createNewNet();
extern Widget setBufferToModel();
extern void ShowBufferMenu();
extern Boolean bufferExisting();
extern void killBuffer();
extern void Dialog();
extern void setModifikationTime();
extern time_t getModifikationTime();
extern String getDialogInput();
extern void setModelName();
extern void ResetOptionButtons();
extern void ResetCommandToggles();
extern void ResetObjectToggles();
extern void replaceInfoString();
extern String getWord();
extern void initBrowsePopup();

/* Callbacks */
void LoadNet(); /* not static ,need in browseModels*/
void SaveNet();
static void Remove();
static void RemoveDir();
static void RemoveHierarchy();
static void LoadNetCommitted();
static void SaveNetAsCommitted();
void GetModelName();
 void LoadInputCallback();
 void SaveInputCallback();
 void SaveNetCommitted();

extern void removeAllNets();
extern void DestroyDistributionBuf();
extern Boolean bufferModified();
extern void Dialog();

static void Quit1();
static void Quit2();


static Widget 
fileMenuButton,
filePopup,
	saveButton,
	loadButton,
	saveAsButton,
	removeButton,
	quitButton;
	
static Widget
removePopup,
        rm_results,
        rm_net,
        rm_dir,
        rm_dir_tree;
        
static char transtable[] = "\
        <EnterWindow>:  highlightWhenSubmenuesUnvisible() \n\
        <LeaveWindow>:        popupMainShell(loadButton) leaveMainPopup(filePopup removeButton removePopup) \n\
        <BtnMotion>:        highlightWhenSubmenuesUnvisible() \n\
        <BtnUp>:        popdownBrowseSubmenues() popdownSubmenues(removePopup) MenuPopdown() notify() unhighlight()";


static char submenu_transtable[] ="\
        <LeaveWindow>:        leaveSubmenu() ";

static char next_model[MAXPATHLEN];

/*******************************************************************************
Name:
        initFilePopup
--------------------------------------------------------------------------------
Funktionstyp:
        oeffentliche Funtion
--------------------------------------------------------------------------------
gebraucht in:
        main.c
--------------------------------------------------------------------------------
Parameter:
        topLevel: toplevel-shell der Applikation
        parent:   Elterwidget des Buttons "File"
--------------------------------------------------------------------------------
Zweck:
        Initialisierung des Buttons und der Menues
        Aendern der Semantik der Menues
        Registrieren der Callbacks
*******************************************************************************/

void initFilePopup(topLevel,parent)
Widget topLevel,parent;

{
XtTranslations translations;

  fileMenuButton = XtVaCreateManagedWidget(
                "fileMenuButton",          
                menuButtonWidgetClass, 
                parent, 
                XtNlabel,"File",
                XtNwidth,COMMAND_BUTTON_WIDTH,        
                XtNheight,COMMAND_BUTTON_HEIGHT,
                XtNjustify,COMMAND_BUTTON_JUSTIFY,
                XtNmenuName,"filePopup",
                NULL);             

  filePopup = XtVaCreatePopupShell(
                "filePopup",
                simpleMenuWidgetClass,
                topLevel,
                NULL);
  translations = XtParseTranslationTable(transtable);
  XtOverrideTranslations(filePopup,translations);

  loadButton = XtVaCreateManagedWidget(
                "loadButton",
                smeBSBObjectClass,
                filePopup,
                XtNlabel,"Load Net",
                XtNjustify,MENU_ENTRY_JUSTIFY,
                XtNrightMargin,16,
                XtNrightBitmap,rightPtr,
                NULL);

  saveButton = XtVaCreateManagedWidget(
                "saveButton",
                 smeBSBObjectClass,
                 filePopup,
                 XtNlabel,"Save Net",
                 XtNjustify,MENU_ENTRY_JUSTIFY,
                 XtNrightMargin,16,
                 NULL);

  saveAsButton = XtVaCreateManagedWidget(
                "saveAsButton",
                 smeBSBObjectClass,
                 filePopup,
                 XtNlabel,"Save Net as..",
                 XtNjustify,MENU_ENTRY_JUSTIFY,
                 XtNrightMargin,16,
                 NULL);

  removeButton = XtVaCreateManagedWidget(
                "removeButton",
                smeBSBObjectClass,
                filePopup,
                XtNlabel,"Remove..",
                XtNjustify,MENU_ENTRY_JUSTIFY,
                XtNrightMargin,16,
                XtNrightBitmap,rightPtr,
                NULL);
  
  XtVaCreateManagedWidget("seperator2",smeLineObjectClass,filePopup,NULL);
  
  quitButton = XtVaCreateManagedWidget(
                "quitButton",
                smeBSBObjectClass,
                filePopup,
                XtNlabel,"Quit",
                XtNjustify,MENU_ENTRY_JUSTIFY,
                XtNrightMargin,16,
                NULL);

  XtAddCallback(saveButton,XtNcallback,SaveNet,0);
  XtAddCallback(saveAsButton,XtNcallback,GetModelName,(XtPointer)SAVE);
  XtAddCallback(quitButton, XtNcallback, Quit1,(XtPointer)0);
  initBrowsePopup();

  removePopup = XtVaCreatePopupShell(
                "removePopup",
                simpleMenuWidgetClass,
                topLevel,
                NULL);
  translations = XtParseTranslationTable(submenu_transtable);
  XtOverrideTranslations(removePopup,translations);
  XtAddCallback(removePopup,XtNpopupCallback,SetSubmenuPopedupFlag,
                (XtPointer)True);
  XtAddCallback(removePopup,XtNpopdownCallback,SetSubmenuPopedupFlag,
                (XtPointer)False);

  rm_results = XtVaCreateManagedWidget(
                "rm_results",
                 smeBSBObjectClass,
                 removePopup,
                 XtNlabel,"Results",
                 XtNrightMargin,16,
                 XtNjustify,MENU_ENTRY_JUSTIFY,
                 NULL);
  rm_net = XtVaCreateManagedWidget(
                "rm_net",
                 smeBSBObjectClass,
                 removePopup,
                 XtNlabel,"Model",
                 XtNrightMargin,16,
                 XtNjustify,MENU_ENTRY_JUSTIFY,
                 NULL);
  rm_dir = XtVaCreateManagedWidget(
                "rm_dir",
                 smeBSBObjectClass,
                 removePopup,
                 XtNlabel,"Directory..",
                 XtNrightMargin,16,
                 XtNjustify,MENU_ENTRY_JUSTIFY,
                 NULL);
  rm_dir_tree = XtVaCreateManagedWidget(
                "rm_dir_tree",
                 smeBSBObjectClass,
                 removePopup,
                 XtNlabel,"Hierarchy..",
                 XtNrightMargin,16,
                 XtNjustify,MENU_ENTRY_JUSTIFY,
                 NULL);
  XtAddCallback(rm_results,XtNcallback,Remove,(XtPointer)Results);
  XtAddCallback(rm_net,XtNcallback,Remove,(XtPointer)Model);
  XtAddCallback(rm_dir,XtNcallback,Remove,(XtPointer)Directory);
  XtAddCallback(rm_dir_tree,XtNcallback,Remove,(XtPointer)Hierarchy);
  
}

/*******************************************************************************
Name:
        SaveNet
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        saveButton
--------------------------------------------------------------------------------
Zweck:
        veranlasst sichern eines Modelles
*******************************************************************************/

void SaveNet(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
  String name,model_dir;
  Boolean picture_modified,net_modified;
  char path[MAXPATHLEN];
  struct stat s_buf;
  time_t l_mod;
  Position x,y;
  Dimension height;

  XtVaGetValues(getCurrentNet(),
                XtNpictureModified,&picture_modified,
                XtNnetModified, &net_modified,
                XtNmodelName,&name,
                NULL);

  if (name == NULL || strcmp(name,"") == 0)
    {
      XtWarning("There is no model name specified");
      return;
    }

  if (! picture_modified && ! net_modified) 
    return;

  if ((model_dir = getenv(MODELDIR_ENV)) == NULL)
    {
      char msg[200];
      
      sprintf(msg,
              "Environmentvariable %s not set, can't find any models",
              MODELDIR_ENV);
      XtWarning(msg);
      return;
    }
  sprintf(path,"%s/%s%s",model_dir,name,NET_EXTENSION);
  if (stat(path,&s_buf) == 0)
    {
      l_mod = getModifikationTime();
      if (l_mod < s_buf.st_mtime)
        {
          XtVaGetValues(fileMenuButton,
                        XtNheight,&height,
                        NULL);
          XtTranslateCoords(fileMenuButton,0,0,&x,&y);
          Dialog("Model modified on disk; overwrite?",NULL,
                 SaveNetCommitted,x,y+(Position)height);
          return;
        }
    }
  SaveNetCommitted(NULL,(XtPointer)True,NULL);
}

/*******************************************************************************
Name:
        SaveNetCommitted
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        saveButton
--------------------------------------------------------------------------------
Zweck:
        veranlasst sichern eines Modelles
*******************************************************************************/

void SaveNetCommitted(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
  String name,model_dir,dir,sptr,dname;
  char path[MAXPATHLEN];
  String *params;
  Cardinal numparams;
  char command[MAXPATHLEN+80];
  Boolean committed = client_data == NULL ? False : True;
  struct stat s_buf;

  if (!committed)
    return;

  if ((model_dir = getenv(MODELDIR_ENV)) == NULL)
    {
      char msg[200];
      
      sprintf(msg,
              "Environmentvariable %s not set, can't find any models",
              MODELDIR_ENV);
      XtWarning(msg);
      return;
    }

  XtVaGetValues(getCurrentNet(),
                XtNmodelName,&name,
                NULL);

  dir = XtNewString(name);
  sptr = strrchr(dir,'/');
  if (sptr)
    {
      *sptr = '\0';
      dname = strtok(dir,"/");
      strcpy(path,model_dir);
      while (dname != NULL) 
        {
          strcat(path,"/");
          strcat(path,dname);
          if (stat(path,&s_buf) == -1) 
            {
              if(mkdir(path,0777) != 0)
                {
                  char msg[1000];
                  
                  if (errno < sys_nerr)
                    sprintf(msg,"Save Net: %s",sys_errlist[errno]);
                  else
                    sprintf(msg,"Save Net: can't create directories");
                  XtWarning(msg);
                  return;
                }
            }
          dname = strtok(NULL,"/");
        }
    }
  XtFree(dir);

  params = (String*)XtMalloc(sizeof(String));
  params[0] = name;
  numparams = 1;
  XtCallActionProc(getCurrentNet(),"save",NULL,params,numparams);
  /* Remove old printouts */
  sprintf(command,
          "%s/%s/REMOVE -p %s >/dev/null 2>/dev/null",
          getenv(DSPNPACKAGE_ENV),SOLVEPATH,name);
  system(command);
  XtFree((String)params);
  if ((model_dir = getenv(MODELDIR_ENV)) == NULL)
    {
      char msg[200];
      
      sprintf(msg,
              "Environmentvariable %s not set, can't find any models",
              MODELDIR_ENV);
      XtWarning(msg);
      return;
    }
  sprintf(path,"%s/%s%s",model_dir,name,NET_EXTENSION);
  if (stat(path,&s_buf) == 0)
    setModifikationTime(s_buf.st_mtime);
}

/*******************************************************************************
Name:
        GetModelName
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        loadButton
--------------------------------------------------------------------------------
Zweck:
        veranlasst laden eines Modelles
*******************************************************************************/

void GetModelName(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
  char question[100];
  String ptr,model,name;
  Position x,y;
  Dimension height;
  XtCallbackProc handler;
  int action = (int)((long)client_data);

  strcpy(question,action == SAVE?"Save model as:" :"Load model:");
  XtVaGetValues(getCurrentNet(),
                XtNmodelName,&name,
                NULL);

  model = XtNewString(name);
  if ((ptr = strrchr(model,'/')) != NULL)
    *++ptr = '\0';
  else
    *model = '\0';

  XtVaGetValues(fileMenuButton,
                XtNheight,&height,
                NULL);
  XtTranslateCoords(fileMenuButton,0,0,&x,&y);
  handler = action == SAVE ? SaveInputCallback : LoadInputCallback;
  Dialog(question,model,handler,x,y+(Position)height);
  XtFree(model);
}
/*******************************************************************************
Name:
        SaveInputCallback
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        loadButton
--------------------------------------------------------------------------------
Zweck:
        veranlasst laden eines Modelles
*******************************************************************************/

void SaveInputCallback(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
  Boolean input_done = client_data == NULL ? False : True;
  String input,name,model_dir;
  char path[MAXPATHLEN];
  struct stat s_buf;
  Dimension height;
  Position x,y;
  int nrs,nlen;

  if (! input_done)
    return;

  name = NULL;
  input = getDialogInput();

  if (input == NULL)
    {
      XtWarning("There is no model name specified");
      return;
    }
  name = getWord(input);
  if (name == NULL)
    {
      XtWarning("There is no model name specified");
      return;
    }
  if (*name == '\0')
    {
      XtWarning("There is no model name specified");
      return;
    }
  nlen = strlen(name);
  if (name[nlen - 1] == '/')
    {
      XtWarning("Model name can't end with '/'; ignored");
      return;
    }

  if ((model_dir = getenv(MODELDIR_ENV)) == NULL)
    {
      char msg[200];
      
      sprintf(msg,
              "Environmentvariable %s not set, can't find any models",
              MODELDIR_ENV);
      XtWarning(msg);
      return;
    }
  sprintf(path,"%s/%s%s",model_dir,name,NET_EXTENSION);
  nrs = stat(path,&s_buf);
  if (nrs == 0)
    {
      XtVaGetValues(fileMenuButton,
                    XtNheight,&height,
                    NULL);
      XtTranslateCoords(fileMenuButton,0,0,&x,&y);
      strcpy(next_model,name); 
      Dialog("Model already exists; overwrite?",NULL,
             SaveNetAsCommitted,x,y+(Position)height);
      return;
    }
  strcpy(next_model,name); 
  SaveNetAsCommitted(NULL,(XtPointer)True,NULL);
}
/*******************************************************************************
Name:
        SaveNetAsCommitted
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        saveButton
--------------------------------------------------------------------------------
Zweck:
        veranlasst sichern eines Modelles
*******************************************************************************/

static void SaveNetAsCommitted(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
  String model_dir,dname,dir,sptr;
  char path[MAXPATHLEN];
  String *params;
  Cardinal numparams;
  char command[MAXPATHLEN+80];
  Boolean committed = client_data == NULL ? False : True;
  struct stat s_buf;
  String old_model_name; Boolean old_model_readonly;
   
  if (!committed)
    return;

  if ((model_dir = getenv(MODELDIR_ENV)) == NULL)
    {
      char msg[200];
      
      sprintf(msg,
              "Environmentvariable %s not set, can't find any models",
              MODELDIR_ENV);
      XtWarning(msg);
      return;
    }

  dir = XtNewString(next_model);
  sptr = strrchr(dir,'/');
  if (sptr)
    {
      *sptr = '\0';
      dname = strtok(dir,"/");
      strcpy(path,model_dir);
      while (dname != NULL) 
        {
          strcat(path,"/");
          strcat(path,dname);
          if (stat(path,&s_buf) == -1) 
            {
              if(mkdir(path,0777) != 0)
                {
                  char msg[1000];
                  
                  if (errno < sys_nerr)
                    sprintf(msg,"Save Net: %s",sys_errlist[errno]);
                  else
                    sprintf(msg,"Save Net: can't create directories");
                  XtWarning(msg);
                  return;
                }
            }
          dname = strtok(NULL,"/");
        }
    }
  XtFree(dir);

  XtVaGetValues(getCurrentNet(),
                XtNmodelName,&old_model_name,
                XtNreadOnly,&old_model_readonly,
                NULL);
  XtVaSetValues(getCurrentNet(),
                XtNmodelName,next_model,
                XtNreadOnly,False,
                NULL);
  params = (String*)XtMalloc(sizeof(String));
  params[0] = next_model;
  numparams = 1;

  XtCallActionProc(getCurrentNet(),"save",NULL,params,numparams);
  XtFree((String)params);
  XtVaSetValues(getCurrentNet(),
                XtNmodelName,old_model_name,
                XtNreadOnly,old_model_readonly,
                NULL);
  /* Remove old printouts
  sprintf(command,
          "%s/%s/REMOVE -p %s >/dev/null 2>/dev/null",
          getenv(DSPNPACKAGE_ENV),SOLVEPATH,next_model);
  system(command);*/

  sprintf(path,"%s/%s%s",model_dir,next_model,NET_EXTENSION);
  if (stat(path,&s_buf) == 0)
    setModifikationTime(s_buf.st_mtime);
}

/*******************************************************************************
Name:
        LoadInputCallback
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        loadButton
--------------------------------------------------------------------------------
Zweck:
        veranlasst laden eines Modelles
*******************************************************************************/

void LoadInputCallback(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
  Boolean input_done = client_data == NULL ? False : True;
  String input,model;
  int nlen;

  if (! input_done)
    return;

  model = NULL;
  input = getDialogInput();
  if (input == NULL)
    {
      XtWarning("Load net: no name specified");      
      return;
    }
  model = getWord(input);
  if (model == NULL)
    {
      XtWarning("Load net: no name specified");      
      return;
    }
  if (*model == '\0')
    {
      XtWarning("Load net: no name specified");      
      return;
    }

  nlen = strlen(model);
  if (model[nlen - 1] == '/')
    {
      XtWarning("Model name can't end with '/'");
      return;
    }

  LoadNet(model);
  XtFree(model);
}
/*******************************************************************************
Name:
        LoadNet
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        loadButton
--------------------------------------------------------------------------------
Zweck:
        veranlasst laden eines Modelles
*******************************************************************************/

void LoadNet(model_name)
String model_name;
{
  String model_dir;

  if (model_name == NULL || strcmp(model_name,"") == 0)
  {
        XtWarning("There is no model name specified");
        return;
  }
  if ((model_dir = getenv(MODELDIR_ENV)) == NULL)
    {
      char msg[200];

      sprintf(msg,
              "Environmentvariable %s not set, can't find any models",
              MODELDIR_ENV);
      XtWarning(msg);
      return;
    }
  strcpy(next_model,model_name);
  if (bufferExisting(model_name))
    {
      char question[MAXPATHLEN+30];
      Position x,y;
      Dimension height;

      XtVaGetValues(fileMenuButton,
                    XtNheight,&height,
                    NULL);
      XtTranslateCoords(fileMenuButton,0,0,&x,&y);
      sprintf(question,"Delete existing buffer %s ?",model_name);
      Dialog(question,NULL,LoadNetCommitted,x,y+(Position)height);
      return;
    }
  LoadNetCommitted(NULL,NULL,(XtPointer)True);
}

/*******************************************************************************
Name:
        LoadNetCommitted
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        loadButton
--------------------------------------------------------------------------------
Zweck:
        veranlasst laden eines Modelles
*******************************************************************************/

static void LoadNetCommitted(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
  Widget model;
  String model_dir;
  Boolean committed = client_data == NULL ? False : True;
  Boolean read_only;

  if (bufferExisting(next_model))
    {
      if (committed)
        killBuffer(next_model);
      else
        return;
    }

  if ((model = createNewNet(next_model,0)) != NULL) 
    { /* load model from disk */
      String *params;
      Cardinal numparams;
      struct stat ns_buf;
      char npath[MAXPATHLEN];
      int user,grp,nrs;
      Boolean new_net;

      setModelName("");
      if ((model_dir = getenv(MODELDIR_ENV)) == NULL)
        {
          char msg[200];
          
          sprintf(msg,
                  "Environmentvariable %s not set, can't find any models",
                  MODELDIR_ENV);
          XtWarning(msg);
          return;
        }
      
      sprintf(npath,"%s/%s%s",model_dir,next_model,NET_EXTENSION);
      nrs = stat(npath,&ns_buf);

      if (nrs == 0) new_net = False;
      else
        {
          char msg[500];

          sprintf(msg,"New model: %s",next_model);
          XtWarning(msg);          
          new_net = True;
        }

      if (new_net)
        {
          setModifikationTime(0);
          read_only = False;
        }
      else
        {
          user = getuid();
          grp = getgid();
          if (user == (int)ns_buf.st_uid)
            {
              if (ns_buf.st_mode & S_IWUSR)
                read_only = False;
              else
                read_only = True;
            }
          else
          if (grp == (int)ns_buf.st_gid)
            {
              if (ns_buf.st_mode & S_IWGRP)
                read_only = False;
              else
                read_only = True;
            }
          else
            {
              if (ns_buf.st_mode & S_IWOTH)
                read_only = False;
              else
                read_only = True;
            }

          params = (String*)XtMalloc(sizeof(String));
          params[0] = next_model;
          numparams = 1;
          XtCallActionProc(model,"load",NULL,params,numparams);
          XtFree((String)params);
          setModifikationTime(ns_buf.st_mtime);
        }
    }
  else
    {
      XtWarning("Couldn`t create new model");
      return;
    }
  
  XtVaSetValues(model,
                XtNCommands,NOOPERATION,
                XtNObjects,NOOBJECT,
                XtNreadOnly,read_only,
                NULL);
  setModelName(next_model);
  ResetOptionButtons();
  ResetCommandToggles();
  ResetObjectToggles();
  replaceInfoString();
}


/*******************************************************************************
Name:
        Remove
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        rm_results, rm_net, rm_dir, rm_dir_tree
--------------------------------------------------------------------------------
Parameter:
        client_data: Information was gel"oscht werden soll, an Hand des Auf-
                     z"ahlungstypen RemoveMode
--------------------------------------------------------------------------------
Zweck:
        veranlasst l"oschen von resultaten, Modellen Verzeichnissen und 
        Hierarchien
*******************************************************************************/

static void Remove(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
  RemoveMode mode = (RemoveMode)client_data;
  char option[5],command[500];
  String pkg_dir;
  String model_name,sptr;
  Position x,y;
  Dimension height;

  XtVaGetValues(getCurrentNet(),
                XtNmodelName,&model_name,
                NULL);
  if (model_name == NULL || strcmp(model_name,"") == 0)
  {
    XtWarning("There is no model specified");
    return;
  }
  if ((pkg_dir = getenv(DSPNPACKAGE_ENV)) == NULL)
    {
      char msg[200];

      sprintf(msg,
              "Environmentvariable %s not set, can't find remove command",
              MODELDIR_ENV);
      XtWarning(msg);
      return;
    }

  strcpy(next_model,model_name);  
  switch (mode) 
  {
    case Model:
        strcpy(option,"-n");
        break;
    case Results:
        strcpy(option,"-r");
        break;
    case Directory:
        sptr = strrchr(next_model,'/');
        if (sptr)
          *sptr = '\0';
        else
          *next_model = '\0';
        XtVaGetValues(fileMenuButton,
                      XtNheight,&height,
                      NULL);
        XtTranslateCoords(fileMenuButton,0,0,&x,&y);        
        Dialog("Remove directory:",next_model,RemoveDir,x,y+(Position)height);
        return;
    case Hierarchy:
        sptr = strrchr(next_model,'/');
        if (sptr)
          *sptr = '\0';
        else
          *next_model = '\0';
        XtVaGetValues(fileMenuButton,
                      XtNheight,&height,
                      NULL);
        XtTranslateCoords(fileMenuButton,0,0,&x,&y);        
        Dialog("Remove directory hierarchy:",next_model,RemoveHierarchy,x,y+(Position)height);
        return;
      default:
        strcpy(option,"");        
        break;
  }

  sprintf(command,
          "csh -e -f %s/StatAnalysis/scripts/REMOVE %s %s >/dev/null 2>/dev/null",
          pkg_dir,option,model_name);
  system(command);
}

static void RemoveDir(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
  Boolean committed = client_data == NULL ? False : True;
  char command[MAXPATHLEN + 200];
  String pkg_dir,model_dir,rem_dir,input;
  char path[MAXPATHLEN];
  struct stat s_buf;

  if (!committed)
    return;

  if ((pkg_dir = getenv(DSPNPACKAGE_ENV)) == NULL)
    {
      char msg[200];

      sprintf(msg,
              "Environmentvariable %s not set, can't find remove command",
              MODELDIR_ENV);
      XtWarning(msg);
      return;
    }
  
  if ((model_dir = getenv(MODELDIR_ENV)) == NULL)
    {
      char msg[200];

      sprintf(msg,
              "Environmentvariable %s not set, can't find remove command",
              MODELDIR_ENV);
      XtWarning(msg);
      return;
    }
  
  input = getDialogInput();
  rem_dir = getWord(input);
  if (rem_dir == NULL)
    {
      XtWarning("There is no model name specified");
      return;
    }
  if (*rem_dir == '\0')
    {
      XtWarning("There is no model name specified");
      return;
    }

  sprintf(path,"%s/%s",model_dir,rem_dir);
  if (stat(path,&s_buf) != 0)
    {
      XtWarning("Specified directory doesn't exist");
      return;
    }
  if (!S_ISDIR(s_buf.st_mode))
    {
      XtWarning("Specified name is not a directory");
      return;
    }


  sprintf(command,
          "csh -e -f %s/StatAnalysis/scripts/REMOVE %s %s >/dev/null 2>/dev/null",
          pkg_dir,"-d",rem_dir);
  system(command);
}

static void RemoveHierarchy(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
  Boolean committed = client_data == NULL ? False : True;
  char command[MAXPATHLEN + 200];
  String pkg_dir,model_dir,rem_dir,input;
  char path[MAXPATHLEN];
  struct stat s_buf;

  if (!committed)
    return;

  if ((pkg_dir = getenv(DSPNPACKAGE_ENV)) == NULL)
    {
      char msg[200];

      sprintf(msg,
              "Environmentvariable %s not set, can't find remove command",
              MODELDIR_ENV);
      XtWarning(msg);
      return;
    }
  
  if ((model_dir = getenv(MODELDIR_ENV)) == NULL)
    {
      char msg[200];

      sprintf(msg,
              "Environmentvariable %s not set, can't find remove command",
              MODELDIR_ENV);
      XtWarning(msg);
      return;
    }
  
  input = getDialogInput();
  rem_dir = getWord(input);
  if (rem_dir == NULL)
    {
      XtWarning("There is no model name specified");
      return;
    }
  if (*rem_dir == '\0')
    {
      XtWarning("There is no model name specified");
      return;
    }

  sprintf(path,"%s/%s",model_dir,rem_dir);
  if (stat(path,&s_buf) != 0)
    {
      XtWarning("Specified directory doesn't exist");
      return;
    }
  if (!S_ISDIR(s_buf.st_mode))
    {
      XtWarning("Specified name is not a directory");
      return;
    }

  sprintf(command,
          "csh -e -f %s/StatAnalysis/scripts/REMOVE %s %s >/dev/null 2>/dev/null ",
          pkg_dir,"-h",rem_dir);
  system(command);
}

/*******************************************************************************
Name:
        Quit1
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        quitButton
--------------------------------------------------------------------------------
Zweck:
        Beenden des Programms
*******************************************************************************/

static void Quit1(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
  Position rx,ry;
  Dimension height;

  XtVaGetValues(fileMenuButton,
                XtNheight,&height,
                NULL);
  XtTranslateCoords(fileMenuButton,0,0,&rx,&ry);

  if (bufferModified())
    {
      Dialog("Modified buffers exist; really quit?",
             NULL,(XtCallbackProc)Quit2,rx,ry+(Position)height);
    }
  else Quit2(NULL,(XtPointer)True,NULL);
}

/*******************************************************************************
Name:
        Quit2
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        quitButton
--------------------------------------------------------------------------------
Zweck:
        Beenden des Programms
*******************************************************************************/

static void Quit2(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
  Boolean quit = client_data == NULL ? False : True;

  if (quit)
    {
      removeAllNets();
      exit(0);
    }
}
