/*******************************************************************************
Modul           :       helpPopup.

Purpose         :       Aufbau des help-Popups;
                        Bereitstellen von Callbackfunktionen zur Anzeige der 
                        der Help-Files 

Project Director:       Christoph Lindemann TU Berlin

Programmer      :       Christian L"uhe

History         :       release 1: October 1992
*******************************************************************************/

#include <stdio.h>
#include <dirent.h>
#include <curses.h>
#include <stdlib.h>

#include <X11/Intrinsic.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/

/*
 * Public include file for widgets we actually use in this file.
 */
#include <X11/Shell.h>     
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Form.h>    
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Paned.h>    
#include <X11/Xaw/AsciiText.h>    

#include "NetWidget.h"
#include "layout.h"
#include "environment.h"
#include "util.h"

/* timeout intervall f"ur info popup */
#define TIME_OUT 5000

extern Widget getCurrentNet();

extern XtAppContext app_context;
extern Widget app_root;
extern Widget resultEditor;
extern Widget  printOperationPopup;
extern Widget pmfShell;
extern Widget transientPopup;
extern Dimension x_event, y_event;

/* forward definition of public functions */
#ifdef __STDC__
void initHelpPopup(Widget topLevel);
void PopUpHelp(Widget w, XtPointer client_data, XtPointer call_data);
void helpDone(Widget w, XtPointer client_data, XtPointer call_data);
void initInfoPopup(Widget topLevel);
void PopUpInfo(Widget w, XtPointer client_data, XtPointer call_data);
void infoDone(Widget w, XtPointer client_data, XtPointer call_data);
static void PopdownInfo(XtPointer client_data, XtIntervalId *id);
static void SetPoppedUpFlag(Widget w, XtPointer client_data, XtPointer call_data);
#else
void initHelpPopup();
void PopUpHelp();
void helpDone();
void initInfoPopup();
void InfoPopup();
void infoDone();
static void PopdownInfo();
static void SetPoppedUpFlag();
#endif

void Warning();
/* forward definition of private functions */
#ifdef __STDC__
static void RebuildPopup (Dimension width,
			  Dimension height,
			  Dimension text_width,
			  Dimension text_height,
			  Boolean flag);
static void resetToDefault (Boolean);
#else
static void RebuildPopup();
static void resetToDefault ();
#endif

#define TOP_DISTANCE 8
#define Button_width 100
#define Button_height 20
#define Info_width 400
#define Info_height 20

static int char_height, char_width;

static Widget
  helpPopup,
  helpBox,
        doneButton,
        helpText;

static Widget
  infoPopup,
  infoBox,
         infoText;

static XtIntervalId timeout_id = 0;
static String warning_msg=NULL;
static Boolean info_popped_up = False;

/*******************************************************************************
Name:
	initHelpPopup
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
aufgerufen von:
	main.c
--------------------------------------------------------------------------------
Parameter:
	topLevel: toplevel-shell der Applikation
--------------------------------------------------------------------------------
Zweck:
	Aufbau und Initialisierung des help-popups.
	Registrieren der erforderlichen Callbacks.
*******************************************************************************/

#ifdef __STDC__
void initHelpPopup(Widget topLevel)
#else
void initHelpPopup(topLevel)
    Widget topLevel;
#endif
{
  Dimension height;
  XFontStruct *fontStruct;

  helpPopup = XtVaCreatePopupShell(
			"TimeNET Help Popup",
			transientShellWidgetClass,
			topLevel,
			NULL);
  helpBox = XtVaCreateManagedWidget(
		"helpBox",
		boxWidgetClass,
		helpPopup,
		XtNborderWidth,0,
		XtNhSpace,4,
		XtNvSpace,4,
		NULL);
  doneButton = XtVaCreateManagedWidget(
                "doneButton",
                commandWidgetClass,
                helpBox,
		XtNheight, Button_height,
                XtNlabel,"done",
                XtNjustify,XtJustifyCenter,
                NULL);
  XtAddCallback(doneButton,XtNcallback,helpDone,0);
  helpText = XtVaCreateManagedWidget(
		"helpText",
		asciiTextWidgetClass,
		helpBox,
		/* XtNheight,HELP_HEIGHT, */
                XtNwidth,HELP_WIDTH+20,
		XtNstring,"There is no help-text available.\nSorry.",
		XtNlength, 50,  
	        XtNdisplayCaret,False,
		XtNscrollVertical,XawtextScrollWhenNeeded,
		XtNwrap, XawtextWrapWord,
		XtNeditType, XawtextRead,
		NULL);

  XtVaGetValues (helpText,
		 XtNfont, &fontStruct,
		 XtNheight, &height,
		 NULL);  
  char_width = XTextWidth (fontStruct, "X", 1);
  char_height = height;

}

/*******************************************************************************
Name:
        initInfoPopup
--------------------------------------------------------------------------------
Funktionstyp:
        public function
--------------------------------------------------------------------------------
aufgerufen von:
        main.c
--------------------------------------------------------------------------------
Parameter:
        topLevel: toplevel-shell der Applikation
--------------------------------------------------------------------------------
Zweck:
        Aufbau und Initialisierung des help-popups.
        Registrieren der erforderlichen Callbacks.
*******************************************************************************/

#ifdef __STDC__
void initInfoPopup(Widget topLevel)
#else
void initInfoPopup(topLevel)
    Widget topLevel;
#endif
{
  infoPopup = XtVaCreatePopupShell(
                        "TimeNET Info Popup",
                        transientShellWidgetClass,
                        topLevel,
                        NULL);
  XtAddCallback(infoPopup,XtNpopdownCallback,SetPoppedUpFlag,(XtPointer)False);
  XtAddCallback(infoPopup,XtNpopupCallback,SetPoppedUpFlag,(XtPointer)True);

  infoBox = XtVaCreateManagedWidget(
                "infoBox",
                boxWidgetClass,
                infoPopup,
                XtNborderWidth,0,
                XtNhSpace,4,
                XtNvSpace,4,
                NULL);
  infoText = XtVaCreateManagedWidget(
                "infoText",
                asciiTextWidgetClass,
                infoBox,
                XtNheight,Info_height,
                XtNwidth,Info_width,
                XtNstring,"There is no info-text available.\nSorry.",
                XtNlength, 50,
                XtNdisplayCaret,False,
                XtNwrap, XawtextWrapWord,
                XtNeditType, XawtextRead,
                NULL);

/**********/

XtAppSetWarningHandler(app_context,Warning);
}

/*******************************************************************************
Name:
        RebuildPopup
--------------------------------------------------------------------------------
Funktionstyp:
	private function
--------------------------------------------------------------------------------
Registriert mit:
	PopUpHelp,
	PopUpInfo,
--------------------------------------------------------------------------------
Parameter:
        Breiten und H"ohen
--------------------------------------------------------------------------------
Zweck:
        Hilfe-Fenster mit neuer gew"unschter Gr"osze aufbauen
*******************************************************************************/
#ifdef __STDC__
static void RebuildPopup (Dimension width,
			  Dimension height,
			  Dimension text_width,
			  Dimension text_height,
			  Boolean flag)
#else
static void RebuildPopup(width, height,text_width, text_height, flag)
Dimension width, height, text_width, text_height;
Boolean flag;
#endif
{
  if (flag){
          XtResizeWidget(doneButton,
                         Button_width-10,
                         Button_height,
                         1);
          XtResizeWidget(helpText,
                         text_width,
                         text_height,
                         1);
          XtResizeWidget(helpPopup,
                         width,
                         height,
                         0);
  }
  else {
          XtResizeWidget(infoText,
                         text_width,
                         text_height,
                         1);
          XtResizeWidget(infoPopup,
                         width,
                         height,
                         0);
  }
}

/*******************************************************************************
Name:
	helpDone	
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	done - Button
--------------------------------------------------------------------------------
Parameter:
	unbenutzt
--------------------------------------------------------------------------------
Zweck:
	Popdown des help-Popups.
*******************************************************************************/

#ifdef __STDC__ 
void helpDone(Widget w, XtPointer client_data, XtPointer call_data)
#else 
void helpDone(w, client_data, call_data)
    Widget w;
    XtPointer client_data, call_data;
#endif
{
Boolean flag;

#ifdef debug
printf(" helpDone \n");
#endif

  flag = True;
  XtPopdown(helpPopup);
  resetToDefault(flag);
}

/*******************************************************************************
Name:
        infoDone
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        verschiedenen Popups
--------------------------------------------------------------------------------
Parameter:
        unbenutzt
--------------------------------------------------------------------------------
Zweck:
        Popdown des info-Popups.
*******************************************************************************/

#ifdef __STDC__
void infoDone(Widget w, XtPointer client_data, XtPointer call_data)
#else
void infoDone(w, client_data, call_data)
    Widget w;
    XtPointer client_data, call_data;
#endif
{
Boolean flag;

#ifdef debug
printf(" infoDone \n");
#endif

  flag = False;
  XtPopdown(infoPopup);
  resetToDefault(flag);
}

/*******************************************************************************
Name:
	resetToDefault
--------------------------------------------------------------------------------
Funktionstyp:
	private Funktion
--------------------------------------------------------------------------------
aufgerufen von:
	helpDone
--------------------------------------------------------------------------------
Zweck:
	Rucksetzen von helpText auf initiale Werte
*******************************************************************************/

#ifdef __STDC__ 
static void resetToDefault(Boolean flag)
#else
static void resetToDefault(flag)
     Boolean flag;
#endif
{
  if (flag)
    XtVaSetValues(helpText,
		  XtNtype, XawAsciiString,
		  XtNstring,"There is no help-text available.\nSorry.",
		  XtNlength, 50,
		  NULL);
  else
    XtVaSetValues(infoText,
                  XtNtype, XawAsciiString,
                  XtNstring,"There is no info-text available.\nSorry.",
                  XtNlength, 50,
                  NULL);

}

/*******************************************************************************
Name:
	PopUpHelp
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
        HelpButton verschiedener Popups
--------------------------------------------------------------------------------
Zweck:
	Lesen und anzeigen des Help-files, mappen des Popups
*******************************************************************************/

#ifdef __STDC__
void PopUpHelp(Widget w, XtPointer client_data, XtPointer call_data)
#else
void PopUpHelp(w, client_data, call_data)
    Widget w;
    XtPointer client_data, call_data;
#endif
{
    String file, filename;
    char path[MAXNAMLEN];
    Dimension width, height, text_width, text_height,popup_height,popup_width;
    Position r_x, r_y;
    Boolean flag;

    flag = True;
    file = (String)client_data;

    sprintf (path, "%s/%s/%s",
	     getenv (DSPNPACKAGE_ENV),INTERFACEHELP, file);
    filename = XtFindFile(path, NULL, 0, NULL);

    if (filename) {
	XtVaSetValues(helpText, 
		      XtNstring, path, 
		      XtNtype, XawAsciiFile,
		      NULL);
    }

    if (!strcmp(file, "TransientOptions.help")) {
      XtTranslateCoords(transientPopup, 0, 0, &r_x, &r_y);
      XtVaGetValues(transientPopup,
		    XtNheight,&popup_height,
		    XtNwidth,&popup_width,
		    NULL);
      width =popup_width+60 ; height = popup_height;
      text_width = popup_width+50; text_height = popup_height -10;
      r_y = r_y + popup_height + 10;
      XtVaSetValues(helpPopup, XtNtitle, "Oh, nicht schlecht - ein HELP-Menu!!!", NULL);
    }
    else
        if (!strcmp(file, "expoly.help")) {
      XtTranslateCoords(pmfShell, 0, 0, &r_x, &r_y);
      XtVaGetValues(pmfShell,
		    XtNheight,&popup_height,
		    XtNwidth,&popup_width,
		    NULL);
      width =popup_width ; height = popup_height;
      text_width = popup_width -8; text_height = popup_height;
      r_y += popup_height;
      XtVaSetValues(helpPopup, XtNtitle, "BNF for defining probability mass function", NULL);
    }
    else
	if (!strcmp(file, "Reward.help")) {
      XtTranslateCoords(resultEditor, 0, 0, &r_x, &r_y);
      XtVaGetValues(resultEditor,
		    XtNheight,&popup_height,
		    NULL);
      width = 604; height = 220;
      text_width = 596; text_height = 180;
/*      r_x -= 300;*/
      r_y += 345;
      XtVaSetValues(helpPopup, 
		    XtNtitle, "BNF for specifying reward measures", 
		    NULL);
    }
    else
    if (!strcmp(file, "Print.help")) {
      XtTranslateCoords(printOperationPopup, 0, 0, &r_x, &r_y);
      XtVaGetValues(printOperationPopup,
		    XtNheight,&popup_height,
		    XtNwidth,&popup_width,
		    NULL);
      width =popup_width ; height = popup_height+50;
      text_width = popup_width -8; text_height = popup_height;
/*      r_x += 78;*/
      r_y += popup_height;
      XtVaSetValues(helpPopup, XtNtitle, "How to print a DSPN", NULL);
    }
    else
    if (!strcmp(file, "Delay.help")) {
      width = 454; height = 150;
      text_width = 446; text_height = 110;
      r_x = x_event-10;
      r_y = y_event+200;
      XtVaSetValues(helpPopup, 
		    XtNtitle, "BNF for defining marking-dependent delays", 
		    NULL);
    }
    else if (!strcmp(file, "ArcMD.help")) {
	width = 454; height = 150;
	text_width = 446; text_height = 110;
	r_x = x_event-10;
	r_y = y_event+200;
	XtVaSetValues(helpPopup, 
		      XtNtitle, 
		      "BNF for defining marking-dependent arc cardinalities", 
		      NULL);
    }
    else {
      XtTranslateCoords(app_root, 0, 0, &r_x, &r_y);
      width = 204; height = 150;
      text_width = 196; text_height = 110;
      r_x += 78;
      r_y += 125;
      XtVaSetValues(helpPopup, 
		    XtNtitle, "UNUSED", 
		    NULL);
    }

    RebuildPopup(width, height,text_width, text_height,flag);

    {
      Dimension p_width,p_height,screen_width,screen_height;

      if (!XtIsRealized(helpPopup))
	XtRealizeWidget(helpPopup);
      
      XtVaGetValues(helpPopup,
		    XtNwidth,&p_width,
		    XtNheight,&p_height,
		    NULL);

      screen_width = (Dimension)WidthOfScreen(XtScreen(helpPopup));
      screen_height = (Dimension)HeightOfScreen(XtScreen(helpPopup));
      
      if (r_y < 0)
	r_y = 0;
      else
	if (r_y + (Position)p_height > (Position)screen_height)
	  r_y = (Position)screen_height - (Position)p_height;
      
      if (r_x < 0)
	r_x = 0;
      else
	if (r_x + (Position)p_width > (Position)screen_width )
	  r_x = (Position)screen_width - (Position)p_width;

      XtMoveWidget(helpPopup, r_x, r_y);
    }

    if (!strcmp(file, "Delay.help") || 
    !strcmp(file, "expoly.help") || 
	!strcmp(file, "ArcMD.help"))
      XtPopup(helpPopup, XtGrabNonexclusive);
    else
      XtPopup(helpPopup, XtGrabNone);
}

/*******************************************************************************
Name:
        PopUpInfo
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        HelpButton verschiedener Popups
--------------------------------------------------------------------------------
Zweck:
        Lesen und anzeigen des Info-files, mappen des Popups
*******************************************************************************/

#ifdef __STDC__
void PopUpInfo(Widget w, XtPointer client_data, XtPointer call_data)
#else
void PopUpInfo(w, client_data, call_data)
    Widget w;
    XtPointer client_data, call_data;
#endif
{
    String file, filename;
    char path[MAXNAMLEN];
    Dimension width, height, text_width, text_height;
    Position r_x, r_y;
    Boolean flag;


    if (info_popped_up)
      XtPopdown(infoPopup);
 
    flag = False;
    file = (String)client_data;
    sprintf (path, "%s/Interface/help/%s",
             getenv (DSPNPACKAGE_ENV), file);
    filename = XtFindFile(path, NULL, 0, NULL);

    if (filename) {
        XtVaSetValues(infoText,
                      XtNstring, path,
                      XtNtype, XawAsciiFile,
                      NULL);

      XtTranslateCoords(app_root, 0, 0, &r_x, &r_y);
      width = Info_width + 10; height = Info_height + 10;
      text_width = Info_width; text_height = Info_height;
      r_x += ((NETWIDGET_WIDTH/2)-(width/2));
      r_y += 100;
      XtVaSetValues(infoPopup, XtNtitle, " INVALID INPUT ", NULL);
    }
    else{
      XtTranslateCoords(app_root, 0, 0, &r_x, &r_y);
      width = 204; height = 150;
      text_width = 196; text_height = 110;
      r_x += 78;
      r_y += 125;
      XtVaSetValues(infoPopup, XtNtitle, "UNUSED", NULL);
    }

    RebuildPopup(width, height,text_width, text_height,flag);

    {
      Dimension p_width,p_height,screen_width,screen_height;

      if (!XtIsRealized(infoPopup))
	XtRealizeWidget(infoPopup);
      
      XtVaGetValues(infoPopup,
		    XtNwidth,&p_width,
		    XtNheight,&p_height,
		    NULL);

      screen_width = (Dimension)WidthOfScreen(XtScreen(infoPopup));
      screen_height = (Dimension)HeightOfScreen(XtScreen(infoPopup));
      
      if (r_y < 0)
	r_y = 0;
      else
	if (r_y + (Position)p_height > (Position)screen_height)
	  r_y = (Position)screen_height - (Position)p_height;
      
      if (r_x < 0)
	r_x = 0;
      else
	if (r_x + (Position)p_width > (Position)screen_width )
	  r_x = (Position)screen_width - (Position)p_width;

      XtMoveWidget(infoPopup, r_x, r_y);
    }

    {
      char bell = 7;
      printf("%c",bell);
      fflush(stdout);
    }
    if (timeout_id)
      XtRemoveTimeOut(timeout_id);
    timeout_id = XtAppAddTimeOut(app_context,TIME_OUT,PopdownInfo,(XtPointer)infoPopup);
    XtPopup(infoPopup, XtGrabNone);
}
/*******************************************************************************
Name:
        PopdownInfo
--------------------------------------------------------------------------------
Funktionstyp:
        TimeoutCallback
--------------------------------------------------------------------------------
Registriert mit:
        Timeout
--------------------------------------------------------------------------------
Zweck:
        popdown des info-popups nach ablauf des timeouts
*******************************************************************************/

static void PopdownInfo(client_data,id)
XtPointer client_data;
XtIntervalId *id;
{

infoDone((Widget)client_data,NULL,NULL);
timeout_id = 0;

}

/*******************************************************************************
Name:
        Warning
--------------------------------------------------------------------------------
Funktionstyp:
        WarningHandler
--------------------------------------------------------------------------------
Registriert mit:
        
--------------------------------------------------------------------------------
Zweck:
        Ausgabe der Warnings in popup
*******************************************************************************/

#ifdef __STDC__
void Warning(String msg)
#else
void Warning(msg)
    String msg;
#endif
{
  Dimension width, height, text_width, text_height;
  Position r_x, r_y;
  int i,j,lines,chars;

  if (info_popped_up)
    XtPopdown(infoPopup);
 
  if (timeout_id)
    XtRemoveTimeOut(timeout_id);
  XtFree(warning_msg);
  warning_msg = XtNewString(msg);
  XtVaSetValues(infoText,
		XtNstring,warning_msg,
		XtNtype,XawAsciiString,
		NULL);

  /* count the number of lines in msg */
  for (i = chars = 0, j = lines = 1; i < strlen(warning_msg); j++,i++) 
      if (warning_msg[i] == '\n') {
	  lines++;
	  chars = j>chars ? j : chars;
	  j = 0;
      }
  chars = j>chars ? j : chars;  /* for the last line again */
  chars++;

  text_width = char_width * ( chars > 50 ? chars : 50) ;
  text_height = char_height * lines;
  width = text_width + 10; height = text_height + 10;

/*  XtTranslateCoords(app_root, 0, 0, &r_x, &r_y);*/
  XtTranslateCoords(getCurrentNet(), 0, 0, &r_x, &r_y);
  /* width = Info_width + 10; height = 2*Info_height + 10; */
  /* text_width = Info_width; text_height = 2*Info_height; */
/*
  r_x += ((NETWIDGET_WIDTH/2)-(width/2));
  r_y += 100;
*/
  XtVaSetValues(infoPopup, XtNtitle, " WARNING ", NULL);

  RebuildPopup(width, height,text_width, text_height,False); 

    {
      Dimension p_width,p_height,screen_width,screen_height;

      if (!XtIsRealized(infoPopup))
	XtRealizeWidget(infoPopup);
      
      XtVaGetValues(infoPopup,
		    XtNwidth,&p_width,
		    XtNheight,&p_height,
		    NULL);

      screen_width = (Dimension)WidthOfScreen(XtScreen(infoPopup));
      screen_height = (Dimension)HeightOfScreen(XtScreen(infoPopup));
      
      if (r_y < 0)
	r_y = 0;
      else
	if (r_y + (Position)p_height > (Position)screen_height)
	  r_y = (Position)screen_height - (Position)p_height;
      
      if (r_x < 0)
	r_x = 0;
      else
	if (r_x + (Position)p_width > (Position)screen_width )
	  r_x = (Position)screen_width - (Position)p_width;

      XtMoveWidget(infoPopup, r_x, r_y);
    }

  {
    char bell = 7;
    printf("%c",bell);
    fflush(stdout);
  }
  timeout_id = XtAppAddTimeOut(app_context,TIME_OUT,PopdownInfo,(XtPointer)infoPopup);
  XtPopup(infoPopup, XtGrabNone);
}


/*******************************************************************************
Name:
        SetPoppedUpFlag
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        Popup- und Popdown-callback des Helppopup
--------------------------------------------------------------------------------
Zweck:
        Setzen des popup-flags
*******************************************************************************/

#ifdef __STDC__
static void SetPoppedUpFlag(Widget w, XtPointer client_data, XtPointer call_data)
#else
static void SetPoppedUpFlag(w, client_data, call_data)
    Widget w;
    XtPointer client_data, call_data;
#endif
{
  info_popped_up = client_data ? True : False;
}
