/*******************************************************************************
Modul   	:	main.c

Purpose		: 	Hauptprogramm der Applikation

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/

#include <stdio.h>
#include <sys/param.h>

#include <X11/Intrinsic.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/

/*
 * Public include file for widgets we actually use in this file.
 */
#include <X11/Xaw/Label.h>   
#include <X11/Xaw/Form.h>   
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Box.h>   
#include <X11/Xaw/Toggle.h>
#include <X11/Xaw/SimpleMenu.h>

#include "layout.h"
#include "NetWidget.h"

/* application class name 
   gleichzeitig name des application defaults files */
#define APP_CLASS_NAME "TimeNET"

#define TOP_LEFT_CHAINED \
		XtNright,XawChainLeft,\
		XtNleft,XawChainLeft,\
		XtNtop,XawChainTop,\
		XtNbottom,XawChainTop


extern void initCascadedPopupActions();
extern void init_resultEditor();
extern void init_textPopUp();
extern void initInfoField();
extern void initFilePopup();
extern void initEditPopup();
extern void initValidatePopup();
extern void initEvaluationMenu();
extern void initInvariantsPopup();
extern void initEcsPopup();
extern void initOptionsPopup();
extern void init_netName();
extern void initCommandRadioGroup();
extern void initObjectRadioGroup();
extern Boolean init_netManager();
extern Widget getCurrentNet();
extern void initPrintPopup();
extern void sizePrintPopup();
extern void initHelpPopup();
extern void initInfoPopup();
extern void init_dialogBox();
extern void setModelName();
extern void pmfSetup();
extern Widget infoText;

Widget 	
topLevel, 		/*Transient fuer windowmanager*/
app_root,		/*appl. topwindow (form)*/
	header,
	netDisplayed,
	commands,
	editCommandBox,
	editObjectBox
	;

/* PopUpShells */
XtAppContext app_context;

/* Fallback Resources */
static String fallback_resources[]={
#include "TimeNET_fallbacks.h"
  NULL
};

typedef struct
{       Boolean mac_mode;
} AppData, *AppDataPtr;

AppData app_data;

#define XtNmacMode "macMode"
#define XtCmacMode "MacMode"

static XtResource resources[]={
{       XtNmacMode,
        XtCmacMode,
        XtRBoolean,
        sizeof(Boolean),
        XtOffset(AppDataPtr,mac_mode),
        XtRImmediate,
        (XtPointer)False,
}};

static XrmOptionDescRec options[]={
        {"-mac",".macMode",XrmoptionNoArg, (XtPointer)"True"}};


/*******************************************************************************
Name:
	main
--------------------------------------------------------------------------------
Parameter:
	Argumente der Kommandozeile
--------------------------------------------------------------------------------
Zweck:
	Hauptprozedur
*******************************************************************************/
void main(argc, argv)
int argc;
char **argv;
{
char model_name[MAXPATHLEN];
Boolean load_model;
String name;

  topLevel = XtVaAppInitialize(
        &app_context,       /* Application context */
        APP_CLASS_NAME,         /* Application class */
        options, XtNumber(options), /* command line option list */
        &argc, argv,        /* command line args */
        fallback_resources, /* for missing app-defaults file */
        NULL);              /* terminate varargs list */

  XtVaGetApplicationResources(topLevel,&app_data,resources,XtNumber(resources),NULL);
  if(app_data.mac_mode)
    XtVaSetValues(topLevel,XtNwidth,620,XtNheight,420,NULL);
    
  if (argc>1 && argv[1] != NULL)
    strcpy(model_name,argv[1]);
  else
    strcpy(model_name,"");

  
  init_resultEditor(app_context,topLevel);
  initCascadedPopupActions(app_context,topLevel);
  init_textPopUp(app_context,topLevel);

  app_root = XtVaCreateManagedWidget(
                "app_root",             
                formWidgetClass,       
                topLevel,             
		XtNdefaultDistance,0,
		XtNx, 0,
		XtNy, 0,
	        NULL);            

    header = XtVaCreateManagedWidget(
		"headerForm",        
	       	formWidgetClass,
	       	app_root,       
		XtNwidth,APPLICATION_WIDTH+2,
		XtNleft,XawChainLeft,
		XtNright,XawChainRight,
		XtNtop,XawChainTop,
		XtNbottom,XawChainTop,
		XtNborderWidth,0,
		XtNbackground,1,
		XtNforeground,0,
		NULL);
{
  Widget header_left,header_right;  Dimension left_width,right_width;

  header_left = XtVaCreateManagedWidget(
                "leftHeader",
                labelWidgetClass,
                header,
		XtNlabel,MAIN_HEADER_LEFT,
		XtNjustify,MAIN_LABEL_JUSTIFY,
		XtNright,XawChainLeft,
		XtNleft,XawChainLeft,
		XtNbackground,1,
		XtNforeground,0,
		NULL);
  header_right = XtVaCreateManagedWidget(
                "rightHeader",
                labelWidgetClass,
                header,
		XtNlabel,MAIN_HEADER_RIGHT,
		XtNjustify,MAIN_LABEL_JUSTIFY,
                XtNfromHoriz,header_left,
		XtNright,XawChainRight,
		XtNleft,XawChainRight,
		XtNbackground,1,
		XtNforeground,0,
		NULL);
  XtVaGetValues(header_left,XtNwidth,&left_width,NULL);
  XtVaGetValues(header_right,XtNwidth,&right_width,NULL);

  XtVaSetValues(header_right,
	        XtNhorizDistance,APPLICATION_WIDTH-left_width-right_width-4,
		NULL);
}
  netDisplayed = XtVaCreateManagedWidget(
		"netDisplayed",
		formWidgetClass,
		app_root,      
		XtNborderWidth,0,
		XtNorientation,XtorientHorizontal,
		XtNhSpace,0,
		XtNfromVert,NETDISPLAYED_FROM_VERT,
		XtNright,XawChainRight,
		XtNleft,XawChainLeft,
		XtNtop,XawChainTop,
		XtNbottom,XawChainTop,
		NULL);        
  init_netName(netDisplayed);
  initInfoField(!app_data.mac_mode, app_root);
  commands = XtVaCreateManagedWidget(
		"commands",  
		panedWidgetClass, 
		app_root,      
		XtNborderWidth,1,
		XtNorientation,XtorientHorizontal,
/*		XtNhSpace,15,*/
		XtNhorizDistance,COMMANDS_HORIZ_DISTANCE,
		TOP_LEFT_CHAINED,
		XtNfromVert,app_data.mac_mode ? netDisplayed : infoText,
	 	NULL);  
  initPrintPopup(topLevel,commands);
  initFilePopup(topLevel,commands);
  initEditPopup(app_context,topLevel,commands);
  initInvariantsPopup(topLevel);
  initEcsPopup(topLevel);
  initValidatePopup(topLevel,commands);
  initEvaluationMenu(app_context,topLevel,commands,0,0);
  initOptionsPopup(topLevel,commands);
  initHelpPopup(topLevel);
  initInfoPopup(topLevel);
  pmfSetup(topLevel);

  editCommandBox = XtVaCreateManagedWidget(
		"editCommmandBox",   
		panedWidgetClass,     
		app_root,          
/*		XtNborderWidth,0,
		XtNhSpace,0,
		XtNvSpace,0,*/
		XtNfromVert,EDITCOMMANDBOX_FROM_VERT,
		XtNvertDistance,5,
		TOP_LEFT_CHAINED,
		NULL);            
  initCommandRadioGroup(editCommandBox);
  editObjectBox = XtVaCreateManagedWidget(
		"editObjectBox", 
		panedWidgetClass, 
		app_root,      
/*		XtNborderWidth,0,
		XtNhSpace,0,
		XtNvSpace,0,*/
		XtNfromVert,EDITOBJECTBOX_FROM_VERT,
		XtNvertDistance,app_data.mac_mode?
		   EDITOBJECTBOX_VERT_DISTANCE_MAC : EDITOBJECTBOX_VERT_DISTANCE,
		TOP_LEFT_CHAINED,
		NULL);        
  initObjectRadioGroup(editObjectBox);
  init_dialogBox(topLevel);
  load_model = init_netManager(topLevel,app_root,model_name);

/*
 *  Create windows for widgets and map them.
 */

  XawSimpleMenuAddGlobalActions(app_context);
  
  XtRealizeWidget(topLevel);
  if (load_model)
    {
      String * params;
      Cardinal numparams;

      XtVaGetValues(getCurrentNet(),
		    XtNmodelName,&name,
		    NULL);
      params = (String*)XtMalloc(sizeof(String));
      params[0] = XtNewString(name);
      numparams = 1;
      XtCallActionProc(getCurrentNet(),"load",NULL,params,numparams);
      XtFree(params[0]);
      XtFree((String)params);
      setModelName(name);
    }

  /* make the sizes in PrintPopup dynamiclaly nice */
  sizePrintPopup(); 


/*ACCELERATORS*/
/*	XtInstallAccelerators(tag_toggle,edit_win);*/

/*
 *  Loop for events.
 */
/*
{
	extern Widget infoText;
    XtResizeWidget(infoText,640,1,1);
}
*/
  XtAppMainLoop(app_context);
}

