/*******************************************************************************
Modul           :        manageNets.c

Purpose         :        Verwaltung der Editierten Modelle

Project Director:        Christoph Lindemann TU Berlin

Programmer      :        Martin Muller

History         :        release 1: October 1992
                         release 2: June 1994        Kolja Koischwitz
*******************************************************************************/

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <stdlib.h>

#include <X11/IntrinsicP.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/

#include <X11/Shell.h>

#include <X11/Xaw/Viewport.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>

#include "NetWidget.h"
#include "layout.h"
#include "environment.h"
#include "layout.h"

/* memory management for buffer */
#define CHUNK_SIZE 5

/* if desired, define default model name */
#define VOID_MODEL_NAME "NEW"

extern void PreparePopup();
extern void showInfo();
extern void replaceInfoString();
extern void ShowTokenDistribution();
extern void ShowTransitionResults();
extern void ShowArcMulitplicity();
extern void setModelName();
extern void SaveNet();
extern void SetSubmenuPopedupFlag();

extern Cardinal GetPREC();

/*exported functions */
void killBuffer();
Boolean bufferExisting();
Boolean bufferModified();
time_t getModifikationTime();
void setModifikationTime();
Widget getCurrentNet();
Widget createNewNet();

/* private functions */
static void BufferPopupNotify();
static void displayBuffer();

extern Pixmap toggle_pixmap;

typedef struct _Net {
        Widget popupEntry;
		Widget net;
        time_t modified;
        } Net;

static Widget netManager;
static Widget netBufferPopup; /* angemeldet in filePopup.c als Submenu */

static char submenu_transtable[] ="\
	<LeaveWindow>:	leaveSubmenu() ";

static Net *models;
/* index of current model */
static Cardinal curr_net = -1;
/* number of models and first free entry in model array if num_slots != 0 */
static Cardinal num_nets = 0;
/* number of slots in model array */
static Cardinal num_slots = CHUNK_SIZE;

/*******************************************************************************
Name:
        init_netManager
--------------------------------------------------------------------------------
Funktionstyp:
        "offentliche Funktion
--------------------------------------------------------------------------------
benutzt in:
        main.c
--------------------------------------------------------------------------------
Parameter:
        parent: Elterwidget f"ur ViewPort-Instanz
        name:   Initial zu editierendes Modell
--------------------------------------------------------------------------------
Zweck:
        Erzeugen der Instanz eines Viewport-Widget,Erzeugen der Instanz eines
        Modelles und dieses unter Verwaltung von Viewport stellen
*******************************************************************************/
extern void Load();
extern Widget commands;

Boolean init_netManager(topLevel,parent,name)
Widget topLevel,parent;
String name;

{
  char nmodel_path[MAXPATHLEN],dmodel_path[MAXPATHLEN],model_name[MAXPATHLEN];
  String model_dir;
  struct stat ns_buf;
  Boolean load_model = False;
  Widget model;
  XtTranslations translations;
  Boolean read_only,new_net;
  int user,grp,nrs;
  Dimension t_height;

  netManager = XtVaCreateManagedWidget(
                "netManager",             /* arbitrary widget name */
                viewportWidgetClass,       /* widget class from NetWidget.h */
                parent,               /* parent widget */
                XtNwidth,NETMANAGER_WIDTH,
                XtNheight,NETMANAGER_HEIGHT,
                XtNallowHoriz,True,
                XtNallowVert,True,
                XtNforceBars,True,
                XtNfromVert,NETMANAGER_FROM_VERT,
                XtNhorizDistance,NETMANAGER_HORIZ_DISTANCE,
                XtNleft,XawChainLeft,
                XtNtop,XawChainTop,
                NULL);              /* terminate varargs list */

  if (netManager == NULL)
    XtError("can't create netManager");

  models = (Net *)XtCalloc(CHUNK_SIZE,sizeof(Net));
  if (models == NULL)
    {
      XtError("No memory for model buffer");
    }

  /* analyse given name argument */
  if (name != NULL && *name != '\0') 
    {
      if ((model_dir = getenv(MODELDIR_ENV))==NULL)
        {
          char msg[500];
      
          sprintf(msg,
                  "Environmentvariable %s not set, can't find any models",
                  MODELDIR_ENV);
          fprintf(stderr,msg);
        }
      else
        {
          sprintf(nmodel_path,"%s/%s",model_dir,name);
          if (stat(nmodel_path,&ns_buf) == 0 && S_ISDIR(ns_buf.st_mode))
            { /* on commandline we found a directory */
              sprintf(model_name,"%s/%s",name,VOID_MODEL_NAME);
              read_only=False;
            }
          else
            {
              strcat(nmodel_path,NET_EXTENSION);
              nrs = stat(nmodel_path,&ns_buf);

              if (nrs == 0)
                new_net = False;
              else
                  {
                    char msg[500];
                    
                    sprintf(msg,"New model: %s",name);
                    fprintf(stderr,msg);          
                    new_net = True;
                  }
              if (!new_net)
                { /* on commandline we found a modelname */
                  user = getuid();
                  grp = getgid();
                  if (user == (int)ns_buf.st_uid)
                    {
                      if (ns_buf.st_mode & S_IWUSR)
                        read_only = False;
                      else
                        read_only = True;
                    }
                  else
                  if (grp == (int)ns_buf.st_gid)
                    {
                      if (ns_buf.st_mode & S_IWGRP)
                        read_only = False;
                      else
                        read_only = True;
                    }
                  else
                    {
                      if (ns_buf.st_mode & S_IWOTH)
                        read_only = False;
                      else
                        read_only = True;
                    }
                  load_model = True;
                  sprintf(model_name,"%s",name);
                }
              else
                {
                  sprintf(model_name,"%s",name);
                }
            }
        }
    }
  else
    {
      sprintf(model_name,"%s",VOID_MODEL_NAME);
      read_only = False;
    }

  /* initialize BufferMenuPopup */
  netBufferPopup = XtVaCreatePopupShell(
				   "netBufferPopup",
				   simpleMenuWidgetClass,
				   topLevel,
				   NULL);
  translations = XtParseTranslationTable(submenu_transtable);
  XtOverrideTranslations(netBufferPopup,translations);
  XtAddCallback(netBufferPopup,XtNpopupCallback,SetSubmenuPopedupFlag,
		(XtPointer)True);
  XtAddCallback(netBufferPopup,XtNpopdownCallback,SetSubmenuPopedupFlag,
		(XtPointer)False);

  /* Insert dummy Net NEW */
  createNewNet(model_name);

  return load_model;
}

/*******************************************************************************
Name:
        createNewNet
--------------------------------------------------------------------------------
Funktionstyp:
        oeffentliche Funktion
--------------------------------------------------------------------------------
benutzt in:
        fileButton.c
--------------------------------------------------------------------------------
Parameter:
        name: Modellname
--------------------------------------------------------------------------------
Rueckgabewert:
        erzeigte Widgetinstanz, falls Erzeugung erfolgreich, sonst NULL
--------------------------------------------------------------------------------
Zweck:
        Erzeugen einer neuen Instanz eines Modelles, uebernahme als
		aktueller Puffer
*******************************************************************************/

Widget createNewNet(name)
String name;
{
  Widget newnet;
  char widgetname[32];
  static int netcnt=0;

  sprintf(widgetname,"Net%d",netcnt++);
  newnet = XtVaCreateManagedWidget(
                                   widgetname,
                                   netWidgetClass,
                                   netManager,
                                   XtNwidth,NETWIDGET_WIDTH,
                                   XtNheight,NETWIDGET_HEIGHT,
                                   XtNprecision, GetPREC(),
                                   XtNmodelName,name,
                                   NULL);
  if (newnet == NULL)
      return NULL;

  if (num_slots == 0)
    {
      models = (Net *)XtRealloc((char*)models,(num_nets+CHUNK_SIZE)*sizeof(Net));
      num_slots = CHUNK_SIZE;
    }

  models[num_nets].net = newnet;
  models[num_nets].modified = 0;
  models[num_nets].popupEntry = XtVaCreateManagedWidget(
                 name,
                 smeBSBObjectClass,
                 netBufferPopup,
                 XtNlabel,name,
                 XtNrightMargin,16,
                 XtNleftMargin,16,
                 XtNjustify,MENU_ENTRY_JUSTIFY,
                 NULL);
  
  XtAddCallback(models[num_nets].popupEntry,XtNcallback,BufferPopupNotify,
  	(XtPointer)newnet);

/* CallBack fuer Text registrieren*/
  XtAddCallback(newnet,XtNinfoCallback,showInfo,0);
  XtAddCallback(newnet,XtNtextCallback,PreparePopup,0);
  XtAddCallback(newnet,XtNtokenDistributionCallback,ShowTokenDistribution,0);
  XtAddCallback(newnet,XtNtransResultCallback,ShowTransitionResults,0);
  XtAddCallback(newnet,XtNarcShowCallback, ShowArcMulitplicity, 0);

  num_nets++;
  num_slots--;

  displayBuffer(num_nets-1);
  return models[num_nets-1].net;
}

/*******************************************************************************
Name:
        bufferExisting
--------------------------------------------------------------------------------
Funktionstyp:
        offentliche Funktion
--------------------------------------------------------------------------------
benutzt in:
        fileButton.c
*******************************************************************************/
Boolean bufferExisting(name)
String name;

{
  Cardinal i;
  String model_name;

  /* check, whether model in buffer exists */
  for (i=0;i<num_nets;i++) 
    {
      XtVaGetValues(models[i].net,
                    XtNmodelName,&model_name,
                    NULL);
      if (strcmp(name,model_name) == 0 ) 
        return True;
    }
  return False;
}


/*******************************************************************************
Name:
        bufferModified
--------------------------------------------------------------------------------
Funktionstyp:
        offentliche Funktion
--------------------------------------------------------------------------------
benutzt in:
        fileButton.c
*******************************************************************************/
Boolean bufferModified()
{
  Cardinal i;
  Boolean p_mod,n_mod;

  for (i=0;i<num_nets;i++) 
    {
      XtVaGetValues(models[i].net,
                    XtNpictureModified,&p_mod,
                    XtNnetModified,&n_mod,
                    NULL);
      if (p_mod || n_mod) 
        return True;
    }
  return False;
}

/*******************************************************************************
Name:
        getCurrentNet
--------------------------------------------------------------------------------
Funktionstyp:
        oeffentliche funktion
--------------------------------------------------------------------------------
benutzt in:
        ueberall wo auf aktuelle Instanz eines Modelles zurueckgegriffen wird
--------------------------------------------------------------------------------
Rueckgabewert:
        Instanz des aktuellen Modelles
*******************************************************************************/

Widget getCurrentNet()
{
     return models[curr_net].net;
 }
/*******************************************************************************
Name:
        setModificationTime
--------------------------------------------------------------------------------
Funktionstyp:
        oeffentliche funktion
*******************************************************************************/

void setModifikationTime(time)
time_t time;
{
  models[curr_net].modified = time;
}

/*******************************************************************************
Name:
        setModificationTime
--------------------------------------------------------------------------------
Funktionstyp:
        oeffentliche funktion
*******************************************************************************/

time_t getModifikationTime()
{
  return models[curr_net].modified;
}

/*******************************************************************************
Name:
        removeAllNets
--------------------------------------------------------------------------------
Funktionstyp:
        oeffentliche Funtkion
--------------------------------------------------------------------------------
benutzt in:
        fileButton.c bei Quit
--------------------------------------------------------------------------------
Zweck:
        zerstoert alle editierten Modelle
*******************************************************************************/
void removeAllNets()
{
int i;

#ifdef debug
printf("removeAllNets \n");
#endif
  for (i=0;i<num_nets;i++) 
      if (models[i].net) {
          kill_MDC((NetWidget)models[i].net);
          XtDestroyWidget(models[i].net);
      }
}
/*******************************************************************************
Name:   killBuffer
        
--------------------------------------------------------------------------------
Funktionstyp:
        oeffentliche Funtkion
--------------------------------------------------------------------------------
benutzt in: fileButton
        
--------------------------------------------------------------------------------
Zweck:
        zerstoert angegebenen Puffer
*******************************************************************************/

void killBuffer(model_name)
String model_name;
{
  int i,index;
  String name;

  if (num_nets == 1)
    {
      char msg[MAXPATHLEN+30];

      sprintf(msg,"can't remove the last buffer!");
      XtWarning(msg);
      return;
    }
  index = num_nets;
  for (i=0;i<num_nets;i++) 
    {
      XtVaGetValues(models[i].net,
                    XtNmodelName,&name,
                    NULL);
      if (strcmp(name,model_name) == 0)
        {
          index = i;
          break;
        }
    }
  if (index>=num_nets)
    {
      char msg[MAXPATHLEN+30];

      sprintf(msg,"buffer %s not found",model_name);
      XtWarning(msg);
      return;
    }
  kill_MDC((NetWidget)models[index].net);
  if (models[index].net)
  {
    XtDestroyWidget(models[index].net);
    XtDestroyWidget(models[index].popupEntry);
  }

  num_nets--;
  for (i=index;i<num_nets;i++) 
   	models[i]=models[i+1];

  if(curr_net > index)
    curr_net--;
  else if(index == curr_net)
  {
    curr_net = -1;
	displayBuffer(index>=num_nets ? 0 : index);
  }
  num_slots++;
}

/*******************************************************************************
Name: BufferPopupNotify
        
--------------------------------------------------------------------------------
Funktionstyp: Callback der Eintraege des NetbufferPopup's
        
*******************************************************************************/
static void BufferPopupNotify(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
  Widget net = (Widget)client_data;
  int i,display_model = num_nets;

  for (i=0;i<num_nets;i++) /* Model mit dem entsprechenden Namen suchen */
      if (net==models[i].net)
        {
          display_model = i;
          break;
        }
  if (display_model>=num_nets)
    {
      char msg[MAXPATHLEN+30];

      sprintf(msg,"buffer not found");
      XtWarning(msg);
      return;
    }
  if (display_model >= 0 && curr_net != display_model)
    displayBuffer(display_model);
}
 
/*******************************************************************************
Name: displayBuffer
        
--------------------------------------------------------------------------------
Funktionstyp: private Funktion
        
*******************************************************************************/
static void displayBuffer(display_model)
int display_model;
{
  String model_name;

  if(curr_net != -1)
  {
    XtUnmanageChild(models[curr_net].net);
    XtVaSetValues(models[curr_net].popupEntry,
      XtNleftBitmap,None,
      NULL);
  }
  curr_net = display_model;
  XtVaGetValues(models[curr_net].net,
    XtNmodelName,&model_name,
    NULL);
  setModelName(model_name);
  ResetCommandToggles();
  ResetObjectToggles();
  XtVaSetValues(models[curr_net].net,
                XtNCommands,NOOPERATION,
                XtNObjects,NOOBJECT,
                NULL);
  XtManageChild(models[curr_net].net);

  XtVaSetValues(models[curr_net].popupEntry,
    XtNleftBitmap,toggle_pixmap,
    NULL);

}
