/*******************************************************************************
Modul   	:	netNameText.c

Purpose		: 	Bereitstellen von Funktionen zur Manipulierung des 
                angezeigten Modellnamens/Modell-Typs

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
                release 2: Maerz 1994  Kolja Koischwitz
*******************************************************************************/

#include <stdio.h>

#include <X11/Intrinsic.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Form.h>

#include "layout.h"

Widget netName;
static Widget netNameLabel;
static Widget netType;
static Widget netTypeLabel;

/*******************************************************************************
Name:
	init_netName
--------------------------------------------------------------------------------
Funktionstyp:
	public funktion
--------------------------------------------------------------------------------
Aufgerufen von:
	Funktion 'main' in der Initialisierungsphase
--------------------------------------------------------------------------------
Parameter:
	parent: Elterwidget, an den das Textwidget gebunden wird
--------------------------------------------------------------------------------
Zweck:
	Bereitstellen der eingeschraenkten Aktionstafel fuer das Textwidget.
	Initialisierung des Labels und des Textwidgets.
	
*******************************************************************************/
void init_netName(parent)
Widget parent;

{
Dimension label_width;

#ifdef debug
printf("init_netName \n");
#endif

  netNameLabel = XtVaCreateManagedWidget(
		"netNameLabel",
		labelWidgetClass,
		parent,
		XtNright,XawChainLeft,
		XtNleft,XawChainLeft,
		XtNlabel,NET_NAME_LABEL,
		XtNborderWidth,0,
		NULL);

  netName = XtVaCreateManagedWidget(
		"netName",
		labelWidgetClass,
		parent,
		XtNleft,XawChainLeft,
		XtNright,XawChainRight,
		XtNborderWidth,0,
		XtNfromHoriz,netNameLabel,
        XtNjustify,XtJustifyLeft,
		XtNresize,False,
		NULL);

  netTypeLabel = XtVaCreateManagedWidget(
		"netTypeLabel",
		labelWidgetClass,
		parent,
		XtNborderWidth,0,
		XtNleft,XawChainRight,
		XtNright,XawChainRight,
		XtNlabel,"Net Type: ",
		XtNfromHoriz,netName,
        XtNjustify,XtJustifyLeft,
		NULL);

  netType = XtVaCreateManagedWidget(
		"netType",
		labelWidgetClass,
		parent,
		XtNleft,XawChainRight,
		XtNright,XawChainRight,
		XtNfromHoriz,netTypeLabel,
		XtNborderWidth,0,
		XtNresize,False,
		XtNlabel,"eDSPN",
        XtNjustify,XtJustifyLeft,
		NULL);
  
  /* Some layouting */
  {
    Dimension nwidth,lwidth,twidth;

    XtVaGetValues(netNameLabel,XtNwidth,&nwidth,NULL);
    XtVaGetValues(netTypeLabel,XtNwidth,&lwidth,NULL);
    XtVaGetValues(netType,XtNwidth,&twidth,NULL);
	XtVaSetValues(netName,
		XtNwidth,APPLICATION_WIDTH-nwidth-twidth-lwidth-20,
		NULL);
  }
  XtVaSetValues(netType,XtNlabel,"GSPN",NULL);
}

/*******************************************************************************
Name:
	getDisplayedNetName
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
Aufgerufen in:
	browseModels.c
	fileButton.c
	resultEdit.c
	solveMenu.c
	keine
--------------------------------------------------------------------------------
Rueckgabewert:
	Name, der im Textwidget angezeigt wird.
--------------------------------------------------------------------------------
Zweck:
	Ruckgabe des Namens, der im Textwidget angezeigt wird.
*******************************************************************************/
String getDisplayedNetName()
{
String buf;

  XtVaGetValues(netName,
		XtNlabel,&buf,
		NULL);
  return XtNewString(buf);
}

/*******************************************************************************
Name:
	setModelName
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
Aufgerufen in:
	browseModels.c
--------------------------------------------------------------------------------
Parameter:
	name: neuer Modellname
--------------------------------------------------------------------------------
Zweck:
	Ersetzen des alten Modellnamens durch den neuen
*******************************************************************************/
void setModelName(name)
String name;
{

  if (name == NULL)
    {
      XtWarning("setModelName: no model name specified");
      return;
    }

  XtVaSetValues(netName,
		XtNlabel,name,
		NULL);
}

/*******************************************************************************
Name:
	setDisplayedNetType
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
Aufgerufen in:
	NetWidget.c (etwas unsauber, aber was solls)
--------------------------------------------------------------------------------
Parameter:
	text: Neu zu setzender Text 
--------------------------------------------------------------------------------
Zweck:
	Setzen des Textes des NetType-Widgets
*******************************************************************************/
void setDisplayedNetType(text)
String text;
{

  if (text == NULL)
    {
      XtWarning("setDisplayedNetType: no text specified");
      return;
    }

  XtVaSetValues(netType,
		XtNlabel,text,
		NULL);
}
