/*******************************************************************************
Modul   	:	objectToggles.c

Purpose		: 	Bereitstellen von Funktionen zur Manipulierung der
                        Objekttoggles

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
                        geaendert Juli 96 einfuegen des Group-Toggles
			Heiko Bergmann
*******************************************************************************/



#include <stdio.h>

#include <X11/Intrinsic.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/

#include <X11/Xaw/Label.h>  
#include <X11/Xaw/Toggle.h> 
#include <X11/Xaw/Paned.h> 

#include "NetWidget.h"

#include "layout.h"
#include "bitmaps/arc.bm"
#include "bitmaps/place.bm"
#include "bitmaps/imT.bm"
#include "bitmaps/expT.bm"
#include "bitmaps/genT.bm"
#include "bitmaps/detT.bm"

static Pixmap arc_pixmap,place_pixmap,imT_pixmap,expT_pixmap,detT_pixmap,genT_pixmap;

extern Widget getCurrentNet();
extern void replaceInfoString();
/*new*/
extern int GetTimeMode();
enum {ContTime=0,DiscTime};

void ResetObjectToggles();
void SetGroupToggle();

static void Set_Object();

static Widget
objRadioGroupLabel,
placeToggle,
imTransToggle,
expTransToggle,
detTransToggle,
genTransToggle,
arcToggle,
markingParToggle,
rateParToggle,
tagToggle,
  /*new*/
groupToggle;

/*******************************************************************************
Name:
        initObjectRadioGroup
--------------------------------------------------------------------------------
Funktionstyp:
        public function
--------------------------------------------------------------------------------
Aufgerufen in:
        main.c in der Initialisierungsphase
--------------------------------------------------------------------------------
Parameter:
        parent: Elterwidget an das die Radiogroup gebunden wird
--------------------------------------------------------------------------------
Zweck:
        Initialisierung der Objekttoggles.
        Binden der Callbackfunktionen.
        Setzen der Radiogroup
*******************************************************************************/

void initObjectRadioGroup(parent)
Widget parent;
{
  arc_pixmap = XCreateBitmapFromData(XtDisplay(parent),
			RootWindowOfScreen(XtScreen(parent)),
			arc_bits,arc_width,arc_height);
  place_pixmap = XCreateBitmapFromData(XtDisplay(parent),
			RootWindowOfScreen(XtScreen(parent)),
			place_bits,place_width,place_height);
  detT_pixmap = XCreateBitmapFromData(XtDisplay(parent),
			RootWindowOfScreen(XtScreen(parent)),
			detT_bits,detT_width,detT_height);
  imT_pixmap = XCreateBitmapFromData(XtDisplay(parent),
			RootWindowOfScreen(XtScreen(parent)),
			imT_bits,imT_width,imT_height);
  expT_pixmap = XCreateBitmapFromData(XtDisplay(parent),
			RootWindowOfScreen(XtScreen(parent)),
			expT_bits,expT_width,expT_height);
  genT_pixmap = XCreateBitmapFromData(XtDisplay(parent),
			RootWindowOfScreen(XtScreen(parent)),
			genT_bits,genT_width,genT_height);
  
objRadioGroupLabel = XtVaCreateManagedWidget(
                "objRadioGroupLabel",
                labelWidgetClass,
                parent,          
		XtNlabel,"Objects",
		XtNforeground,0,
		XtNbackground,1,
		XtNwidth,OBJECT_TOGGLE_WIDTH,
		XtNjustify,OBJECT_TOGGLE_JUSTIFY,
                NULL);           

{
  Widget paned = XtVaCreateManagedWidget(
  	"paned",panedWidgetClass,parent,
		XtNorientation,XtorientHorizontal,
		XtNwidth,OBJECT_TOGGLE_WIDTH,
		XtNheight,OBJECT_TOGGLE_WIDTH/2,
		XtNpreferredPaneSize,OBJECT_TOGGLE_WIDTH/2,
		XtNancestorSensitive,True,
		NULL);

  placeToggle = XtVaCreateManagedWidget(
                "placeToggle",   
                toggleWidgetClass,
                paned,           
		XtNleftBitmap,place_pixmap,
		XtNlabel,"",
		XtNpreferredPaneSize,OBJECT_TOGGLE_WIDTH/2,
		XtNjustify,OBJECT_TOGGLE_JUSTIFY,
                NULL);            

  imTransToggle = XtVaCreateManagedWidget(
                "imTransToggle",  
                toggleWidgetClass,
                paned,           
		XtNleftBitmap,imT_pixmap,
		XtNlabel,"",
		XtNpreferredPaneSize,OBJECT_TOGGLE_WIDTH/2,
		XtNjustify,OBJECT_TOGGLE_JUSTIFY,
                NULL);            
}
{
  Widget paned = XtVaCreateManagedWidget(
  	"paned",panedWidgetClass,parent,
		XtNorientation,XtorientHorizontal,
		XtNwidth,OBJECT_TOGGLE_WIDTH,
		XtNheight,OBJECT_TOGGLE_WIDTH/2,
		XtNpreferredPaneSize,OBJECT_TOGGLE_WIDTH/2,
		XtNancestorSensitive,True,
		NULL);

  expTransToggle = XtVaCreateManagedWidget(
                "expTransToggle", 
                toggleWidgetClass,
                paned,           
		XtNleftBitmap,expT_pixmap,
		XtNlabel,"",
		XtNpreferredPaneSize,OBJECT_TOGGLE_WIDTH/2,
		XtNjustify,OBJECT_TOGGLE_JUSTIFY,
                NULL);            

  detTransToggle = XtVaCreateManagedWidget(
                "detTransToggle", 
                toggleWidgetClass,
                paned,           
		XtNleftBitmap,detT_pixmap,
		XtNlabel,"",
		XtNpreferredPaneSize,OBJECT_TOGGLE_WIDTH/2,
		XtNjustify,OBJECT_TOGGLE_JUSTIFY,
                NULL);            
}

{
  Widget paned = XtVaCreateManagedWidget(
  	"paned",panedWidgetClass,parent,
		XtNorientation,XtorientHorizontal,
		XtNwidth,OBJECT_TOGGLE_WIDTH,
		XtNheight,OBJECT_TOGGLE_WIDTH/2,
		XtNpreferredPaneSize,OBJECT_TOGGLE_WIDTH/2,
		XtNancestorSensitive,True,
		NULL);

  genTransToggle = XtVaCreateManagedWidget(
                "genTransToggle", 
                toggleWidgetClass,
                paned,           
		XtNleftBitmap,genT_pixmap,
		XtNlabel,"",
		XtNpreferredPaneSize,OBJECT_TOGGLE_WIDTH/2,
		XtNjustify,OBJECT_TOGGLE_JUSTIFY,
                NULL);            

  arcToggle = XtVaCreateManagedWidget(
                "arcToggle",      
                toggleWidgetClass,
                paned,           
		XtNleftBitmap,arc_pixmap,
		XtNlabel,"",
		XtNpreferredPaneSize,OBJECT_TOGGLE_WIDTH/2,
		XtNjustify,OBJECT_TOGGLE_JUSTIFY,
                NULL);            

}
  markingParToggle = XtVaCreateManagedWidget(
                "markingParToggle",
                toggleWidgetClass, 
                parent,           
		XtNlabel,"marking",
		XtNwidth,OBJECT_TOGGLE_WIDTH,
		XtNheight,OBJECT_TOGGLE_HEIGHT,
		XtNjustify,OBJECT_TOGGLE_JUSTIFY,
                NULL);            

rateParToggle = XtVaCreateManagedWidget(
                "rateParToggle",  
                toggleWidgetClass,
                parent,           
		XtNlabel,"delay",
		XtNwidth,OBJECT_TOGGLE_WIDTH,
		XtNheight,OBJECT_TOGGLE_HEIGHT,
		XtNjustify,OBJECT_TOGGLE_JUSTIFY,
                NULL);            

tagToggle = XtVaCreateManagedWidget(
                "tagToggle",      
                toggleWidgetClass,
                parent,           
		XtNlabel,"tag",
		XtNwidth,OBJECT_TOGGLE_WIDTH,
		XtNheight,OBJECT_TOGGLE_HEIGHT,
		XtNjustify,OBJECT_TOGGLE_JUSTIFY,
                NULL); 

groupToggle  = XtVaCreateManagedWidget(
               "groupToggle",      
	       toggleWidgetClass,
	       parent,           
	       XtNlabel,"group",
	       XtNwidth,OBJECT_TOGGLE_WIDTH,
	       XtNheight,OBJECT_TOGGLE_HEIGHT,
	       XtNjustify,OBJECT_TOGGLE_JUSTIFY,
	       NULL);          

XtAddCallback(placeToggle, XtNcallback, Set_Object, (XtPointer)PLACE );
XtAddCallback(imTransToggle, XtNcallback, Set_Object, (XtPointer)IMTRANS );
XtAddCallback(expTransToggle, XtNcallback, Set_Object, (XtPointer)EXPTRANS );
XtAddCallback(detTransToggle, XtNcallback, Set_Object, (XtPointer)DETTRANS );
XtAddCallback(genTransToggle, XtNcallback, Set_Object, (XtPointer)GENTRANS );
XtAddCallback(arcToggle, XtNcallback, Set_Object, (XtPointer)ARC );
XtAddCallback(markingParToggle, XtNcallback, Set_Object, (XtPointer)MARKPAR );
XtAddCallback(rateParToggle, XtNcallback, Set_Object, (XtPointer)RATEPAR );
XtAddCallback(tagToggle, XtNcallback, Set_Object, (XtPointer)TAG );
/*new*/
XtAddCallback(groupToggle, XtNcallback, Set_Object, (XtPointer)GROUP );

XtVaSetValues(imTransToggle,
        XtNradioGroup,placeToggle,
        NULL);

XtVaSetValues(expTransToggle,
        XtNradioGroup,placeToggle,
        NULL);

XtVaSetValues(detTransToggle,
        XtNradioGroup,placeToggle,
        NULL);

XtVaSetValues(genTransToggle,
        XtNradioGroup,placeToggle,
        NULL);

XtVaSetValues(arcToggle,
        XtNradioGroup,placeToggle,
        NULL);

XtVaSetValues(markingParToggle,
        XtNradioGroup,placeToggle,
        NULL);

XtVaSetValues(rateParToggle,
        XtNradioGroup,placeToggle,
        NULL);

XtVaSetValues(tagToggle,
        XtNradioGroup,placeToggle,
        NULL);

/*new*/
XtVaSetValues(groupToggle,
        XtNradioGroup,placeToggle,
        NULL);

SetGroupToggle(GetTimeMode());
}

/*******************************************************************************
Name:
        ResetObjectToggles
--------------------------------------------------------------------------------
Funktionstyp:
        public function
--------------------------------------------------------------------------------
Aufgerufen in:
        fileButton.c
--------------------------------------------------------------------------------
Zweck:
        Rucksetzen der Radiogroup
*******************************************************************************/

void ResetObjectToggles()
{
XawToggleUnsetCurrent(placeToggle);
}

/*******************************************************************************
Name:
        Set_Object
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit Widget:
	placeToggle,
	imTransToggle,
	expTransToggle,
	detTransToggle,
	genTransToggle,
	arcToggle,
	markingParToggle,
	rateParToggle,
	resultParToggle,
	tagToggle
	groupToggle;
--------------------------------------------------------------------------------
Parameter:
        client_data : jewiliges Object an Hand Aufz"alungstypen Objects
--------------------------------------------------------------------------------
Zweck:
        Aenderungen an der Radiogroup werden dem aktuellen NetWidget durch
        setzen der Resource 'XtNObjects' mitgeteilt.
        Der Infostring wird aktualisiert.


*******************************************************************************/
static void Set_Object(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
Widget curr_net = getCurrentNet();

/*falls kein Toggle gesetzt 
	auf noobject setzen
  sonst auf clientdata
dies sollte gutgehen, da callback fuer ruckgesetztes toggle zuerst aufgeru
fen wird 
*/

#ifdef debug
printf(" Set_Object\n");
#endif
  if ( XawToggleGetCurrent(placeToggle) == NULL) {
  	XtVaSetValues(curr_net,
		XtNObjects,(int)NOOBJECT,
		NULL);
  }
  else {
  	XtVaSetValues(curr_net,
		XtNObjects,(int)((long)client_data),
		NULL);
  }
  replaceInfoString();
}
/*******************************************************************************
Name:
        SetGroupToggle
--------------------------------------------------------------------------------
Funktionstyp:
        public function
--------------------------------------------------------------------------------
Aufgerufen in:
        evalMenu.c
--------------------------------------------------------------------------------
Zweck:
        en/disable grouptoggle
*******************************************************************************/

void SetGroupToggle(TimeMode)
int TimeMode;
{
 if (TimeMode == ContTime) 
 {
   ResetObjectToggles();
   XtSetSensitive(groupToggle,False);
 }
else
  XtSetSensitive(groupToggle,True);
}




