/*******************************************************************************
Modul   	:	optionButton.c

Purpose		: 	Realisirung der Funktionalit"at des Optionbuttons und
                        der erscheinenden Man"us

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/


#include <stdio.h>

#include <X11/Intrinsic.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/

#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SimpleMenu.h>

#define TOGGLE_LEFT_MARGIN 16

#include "NetWidget.h"

#include "layout.h"

extern Widget getCurrentNet();

static void ChangeRateVisibility();
static void ChangeSplineMode();
static void ChangeTagVisibility();

extern Pixmap toggle_pixmap;

Widget 
optionsMenuButton;

static Widget
optionsPopup,
	    tagVisibleButton,
	    rateVisibleButton,
	    splineButton;

/*******************************************************************************
Name:
        initOptionsPopup
--------------------------------------------------------------------------------
Funktionstyp:
        "offentliche Funktion
--------------------------------------------------------------------------------
ben"utzt in:
       main.c
--------------------------------------------------------------------------------
Parameter:
        topLevel: toplevel-shell der Applikation
	parent: Elterwidget f"r Kommandobutton
--------------------------------------------------------------------------------
Zweck:
        Initialiserung des Buttons, des Men"us und Registrierung der Callbacks
*******************************************************************************/
	
void initOptionsPopup(topLevel,parent)
Widget topLevel,parent;

{
optionsMenuButton = XtVaCreateManagedWidget(
		"optionsMenuButton",        
		menuButtonWidgetClass,  
		parent,  
		XtNlabel,"Options",
		XtNwidth,COMMAND_BUTTON_WIDTH,
		XtNheight,COMMAND_BUTTON_HEIGHT,
		XtNjustify,COMMAND_BUTTON_JUSTIFY,  
		XtNmenuName,"optionsPopup",   
		NULL);           

optionsPopup = XtVaCreatePopupShell(
		"optionsPopup",
		simpleMenuWidgetClass,
		topLevel,
		NULL);

tagVisibleButton = XtVaCreateManagedWidget(
		"tagVisibleButton",
		 smeBSBObjectClass,
		 optionsPopup,
                 XtNlabel,"Display Tags",
                 XtNjustify,MENU_ENTRY_JUSTIFY,
		 XtNrightMargin,16,
		 XtNleftMargin,TOGGLE_LEFT_MARGIN,
		 NULL);

XtAddCallback(tagVisibleButton,XtNcallback,ChangeTagVisibility,0);

rateVisibleButton = XtVaCreateManagedWidget(
		"rateVisibleButton",
		 smeBSBObjectClass,
		 optionsPopup,                
		 XtNlabel,"Display Delays",
                 XtNjustify,MENU_ENTRY_JUSTIFY,
		 XtNrightMargin,16,
		 XtNleftMargin,TOGGLE_LEFT_MARGIN,
		 NULL);

XtAddCallback(rateVisibleButton,XtNcallback,ChangeRateVisibility,0);

splineButton = XtVaCreateManagedWidget(
		"splineButton",
		 smeBSBObjectClass,
		 optionsPopup,
                 XtNlabel,"Use Splines",
                 XtNjustify,MENU_ENTRY_JUSTIFY,
		 XtNrightMargin,16,
		 XtNleftMargin,TOGGLE_LEFT_MARGIN,
		 NULL);

XtAddCallback(splineButton,XtNcallback,ChangeSplineMode,0);

}

/*******************************************************************************
Name:
        ResetOptionButtons
--------------------------------------------------------------------------------
Funktionstyp:
        "offentliche funktion
--------------------------------------------------------------------------------
benutzt in:
         editButton.c
         fileButton.c
--------------------------------------------------------------------------------
Zweck:
        R"ucksetzen der Labels in den Eintr"agen zu Defaultwerten
*******************************************************************************/

void ResetOptionButtons()
{
  XtVaSetValues(tagVisibleButton,XtNleftBitmap,None,NULL);
  XtVaSetValues(rateVisibleButton,XtNleftBitmap,None,NULL);
  XtVaSetValues(splineButton,XtNleftBitmap,None,NULL);
}

/*******************************************************************************
Name:
        ChangeTagVisibility
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        tagVisableButton
--------------------------------------------------------------------------------
Zweck:
        Veranlasst Anzeigen oder Verschwinden von Etiketten
*******************************************************************************/

static void ChangeTagVisibility(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
Boolean tagsvisible;
Widget curr_net = getCurrentNet();

  XtVaGetValues(curr_net,
		XtNtagsVisible,&tagsvisible,
		NULL);
  if (tagsvisible) {
    XtVaSetValues(tagVisibleButton,XtNleftBitmap,None,NULL);
	XtVaSetValues(curr_net,XtNtagsVisible,False,NULL);
  }
  else {
    XtVaSetValues(tagVisibleButton,XtNleftBitmap,toggle_pixmap,NULL);
	XtVaSetValues(curr_net,XtNtagsVisible,True,NULL);
  }
}

/*******************************************************************************
Name:
        ChangeRateVisibility
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        rateVisableButton
--------------------------------------------------------------------------------
Zweck:
        Veranlasst Anzeigen oder Verschwinden von Raten
*******************************************************************************/

static void ChangeRateVisibility(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
Boolean ratesvisible;
Widget curr_net = getCurrentNet();

  XtVaGetValues(curr_net,
		XtNratesVisible,&ratesvisible,
		NULL);
  if (ratesvisible) {
    XtVaSetValues(w,XtNleftBitmap,None,NULL);
	XtVaSetValues(curr_net,XtNratesVisible,False,NULL);
  }
  else {
    XtVaSetValues(w,XtNleftBitmap,toggle_pixmap,NULL);
	XtVaSetValues(curr_net,XtNratesVisible,True,NULL);
  }
}
/*******************************************************************************
Name:
         ChangeSplineMode
--------------------------------------------------------------------------------
Funktionstyp:
         Callback
--------------------------------------------------------------------------------
Registriert mit:
        splineButton
--------------------------------------------------------------------------------
Parameter:
--------------------------------------------------------------------------------
Rueckgabewert:
--------------------------------------------------------------------------------
Zweck:
        Veranlasst anzeigen oder Verschwinden der Splines
*******************************************************************************/


static void ChangeSplineMode(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
    Widget nw = getCurrentNet();
    Boolean spline;

    XtVaGetValues(nw,
		  XtNspline, &spline,
		  NULL);
    if (spline) {
    XtVaSetValues(w,XtNleftBitmap,None,NULL);
	XtVaSetValues(nw,XtNspline, False,NULL);
  }
  else {
    XtVaSetValues(w,XtNleftBitmap,toggle_pixmap,NULL);
	XtVaSetValues(nw,XtNspline, True,NULL);
  }
}
