/*******************************************************************************
Modul           :       optionPopups.c (war: solveMenu.c)

Purpose         :       Bereitstellen der unterschiedlichen Option-Popups
                        fuer TransientSolution,AnalyticSolution,Approximation,Simulation

Programmer      :       Kolja Koischwitz
		transientes von Joerg Mitzlaff in 10.94

History         :       release 1: January 1994
*******************************************************************************/

#include <X11/Intrinsic.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/

#include <X11/Xaw/Command.h>
#include <X11/Xaw/Toggle.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/AsciiText.h>

/*#include <Xm/Xm.h>*/
#include <X11/Shell.h>

#include "Options.h" /* Parameter-Strukturen fuer die Evaluierung */
#include "NetWidget.h" /* Zum Ueberpruefen des Netztyps */

#define SMALL_LABEL_WIDTH 150
#define SMALL_TEXT_WIDTH 140
#define BUTTON_WIDTH 67
#define BUTTON_WIDTH2 40

extern Widget getCurrentNet();

/* Diese Strukturen enthalten die von SolveOps.c benutzten Einstellungen
   Ausserdem die Default-Einstellungen fuer die drei Option-Popups, die beim
   Druck auf den "Default"-Button in das jeweile Popup uebernommen werden */
/* Import aus SolveOps.c */
extern TransientOptions transientOptions,transientDefaults;
extern TransientsimOptions transientsimOptions,transientsimDefaults;
extern AnalyticOptions analyticOptions,analyticDefaults;
extern ApproximationOptions approximationOptions,approximationDefaults;
extern SimulationOptions simulationOptions,simulationDefaults;
extern DTSPNAnalyticOptions dtspnanalyticOptions,dtspnanalyticDefaults;
extern DTSPNTransientOptions dtspntransientOptions,dtspntransientDefaults;

/* Importe aus helpPopup.c */
extern void PopUpHelp();
extern void helpDone();

 /* Importe aus evalMenu.c */
extern void CenterWidget();
extern Dimension WidgetWidth();
extern Dimension CalculateButtonWidth();
extern void setTextForWidget();
extern String getTextOfWidget();
extern void SetCaretsCallback();

extern XtTranslations text_transtable; /* aus evalMenu.c */
extern XtTranslations toggle_transtable; /* aus evalMenu.c */
/* extern XtAccelerators text_accelerators; aus evalMenu.c */
extern XtAccelerators ok_button_accelerators; /* aus evalMenu.c */

/* Exportiertes (nach evalMenu.c) */
void initOptionPopups(); /* Baut die drei Option-Popups zusammen */

/* Stellt im jeweilen Popup die Daten der uebergeben Struktur ein */
static void initTransientSolutionPopup();
static void initTransientSimulationPopup();
static void initAnalyticSolutionPopup();
static void initApproximationPopup();
static void initSimulationPopup();
static void initDTAnalyticSolutionPopup(); /*new*/
static void initDTTransientSolutionPopup(); /*new*/


/* Lokales, Callbacks */
static Boolean evalTransientSolutionPopup();
static Boolean evalTransientSimulationPopup();
static Boolean evalAnalyticSolutionPopup();
static Boolean evalApproximationPopup();
static Boolean evalSimulationPopup();
static Boolean evalDTAnalyticSolutionPopup(); /*new*/
static Boolean evalDTTransientSolutionPopup(); /*new*/

static void TransientPopupLayout();
static void TransientSimulationPopupLayout();
static void AnalyticPopupLayout();
static void ApproximationPopupLayout();
static void SimulationPopupLayout();
static void DTAnalyticPopupLayout(); /*new*/
static void DTTransientPopupLayout(); /*new*/

static void saveOptions();
/* Besser natuerlich als Ascii */

/* Sammel-Callbacks fuer die jeweils 4 unteren Buttons der Option-Popups */
enum {okButton,defaultButton,cancelButton,helpButton}; /* fuer client_data */
static void TransientCallback();
static void TransientSimulationCallback();
static void AnalyticCallback();
static void ApproximationCallback();
static void SimulationCallback();

/* forward definition of callbacks */
static void SolutionTypeCallback();
static void SampleNumberCallback();
static void MinFiringsCallback();
static void StandardCallback();
static void FastCallback();
static void ModelTimeCallback();
static void CPUTimeCallback();
static void TS_CPUTimeCallback();
static void TSensitivityCallback(),TSSensitivityCallback(),ASSensitivityCallback(),ASensitivityCallback();

/* Widgets des Transient Options Popup */
Widget            transientPopup;
static Widget     TmainForm;      /* Fensterrahmen */
static Widget     TtransientTimeLabel;
static Widget     TtransientTimeText;

static Widget     TgenprecLabel;  /* precision */
static Widget     TgenprecText;

static Widget     TleerLabel;

static Widget     ToutputLabel;
static Widget     TdurationToggle;  /* curve Schalter */
static Widget     TpointToggle;

static Widget     Tstepsize2Label;  /* stepsize_result */
static Widget     Tstepsize2Text;

static Widget     Tleer15Label;

static Widget     TvariantLabel;    /* Variant a oder b */
static Widget     ToldToggle;  
static Widget     TnewToggle;

static Widget     TprecisionLabel;  /* stepsize_method */
static Widget     TprecisionText;

static Widget     TclusterLabel;  /* cluster size */
static Widget     TclusterText;

static Widget     Tleer2Label;

static Widget     TokButton;
static Widget     TcancelButton;
static Widget     TdefaultButton;
static Widget     ThelpButton;

static char       TprecisionBuffer[VAL_BUF];
static char       Tstepsize2Buffer[VAL_BUF];
static char       TgenprecBuffer[VAL_BUF];
static char       TclusterBuffer[VAL_BUF];
static char       TtransientTimeBuffer[VAL_BUF];

/********************************************************************************/
/* Widgets des Transient Simulation Options Popup */
Widget  transientSimulationPopup;
static Widget TSmainForm,
	      TStransimForm,
              TStransinstimeLabel,
              TStransinstimeText,
              TSnumsampointsLabel,
              TSnumsampointsText,
	      TSsimulationTypeLabel,
	      TSsequentialToggle,
	      TSdistributedToggle,
	      TSruleTypeLabel,
	      TSruleONToggle,
	      TSruleOFFToggle,
	      TSconfidenceLevelLabel,
	      TSconfidenceLevelText,
	      TSmaxRelativeErrorLabel,
	      TSmaxRelativeErrorText,
	      TSseedValueLabel,
	      TSseedValueText,
	      TSmaxCPUTimeLabel,
	      TSnoCPUTimeLimitToggle,
	      TSmaxCPUTimeLimitText,
	      TSokButton,
	      TScancelButton,
	      TSdefaultButton,
	      TShelpButton;
static char TStransintimeBuffer[VAL_BUF],
            TSnumsampointsBuffer[VAL_BUF],
            TSconfidenceLevelBuffer[VAL_BUF],
	    TSmaxRelativeErrorBuffer[VAL_BUF],
	    TSmaxSampleNumberBuffer[VAL_BUF],
	    TSseedValueBuffer[VAL_BUF],
	    TSmaxCPUTimeLimitBuffer[VAL_BUF];
/**************************************************************************/

Widget /* Widgets des Analytic Options Popup */
	analyticPopup;
static Widget ASmainForm,
	ASprecisionLabel,
	ASprecisionText,
	ASsolveMethodLabel,
	ASiterativeToggle,
        ASdirectToggle,
	ASmaxIterationsLabel,
	ASmaxIterationsText,
	AScalculationKindLabel,
	ASparallelToggle,
	ASsequentialToggle,
	ASokButton,
	AScancelButton,
	ASdefaultButton,
	AShelpButton;
static char ASprecisionBuffer[VAL_BUF],
    ASmaxIterationsBuffer[VAL_BUF];

Widget /* Widgets des Approximation Options Popup */
	approximationPopup;
static Widget AmainForm,
	AprecisionLabel,
	AprecisionText,
	AdroptoleranceLabel,
	AdroptoleranceText,
	AsolveMethodLabel,
	AiterativeToggle,
    AdirectToggle,
	AmaxIterationsLabel,
	AmaxIterationsText,
	AphaseNumberLabel,
	AphaseNumberText,
	AvirtPhaseNumberLabel,
	AvirtPhaseNumberText,
	AokButton,
	AcancelButton,
	AdefaultButton,
	AhelpButton;
static char AprecisionBuffer[VAL_BUF],
     AphaseNumberBuffer[VAL_BUF],AvirtPhaseNumberBuffer[VAL_BUF],
	 AmaxIterationsBuffer[VAL_BUF],AdroptoleranceBuffer[VAL_BUF];

/* Widgets des Simulation Options Popup ************************/
Widget	simulationPopup;
static Widget SmainForm,
	SsimtechniqueLabel,
	SstandardToggle,
	SfastToggle,
	SsteadyStateForm,
	SsimulationTypeLabel,
	SsequentialToggle,
	SdistributedToggle,
	SdetectTypeLabel,
	SdetectOnToggle,
	SdetectOffToggle,
	SconfidenceLevelLabel,
	SconfidenceLevelText,
        SerrPercentLevelLabel,
        SerrPercentLevelText,
	SmaxRelativeErrorLabel,
	SmaxRelativeErrorText,
	SseedValueLabel,
	SseedValueText,
	SmaxSampleNumberLabel,
	SnoSampleLimitToggle,
	SmaxSampleNumberText,
        SminFiringsLabel,
        SminFiringsText,
	SmaxModelTimeLabel,
	SnoModelTimeLimitToggle,
	SmaxModelTimeLimitText,
	SmaxCPUTimeLabel,
	SnoCPUTimeLimitToggle,
	SmaxCPUTimeLimitText,
        SvisualLabel,
        SvisualOnToggle,
        SvisualOffToggle,
	SstandardForm,
	SfastForm,
	SstatisticalMethodLabel,
	SspectralVarianceToggle,
	ScontrolVarialesToggle,
	SoptThreshLabel,
	SoptThreshText,
	SCPUTimeRunLabel,
	SCPUTimeRunText,
	SokButton,
	ScancelButton,
	SdefaultButton,
	ShelpButton;
static char SconfidenceLevelBuffer[VAL_BUF],
            SerrPercentLevelBuffer[VAL_BUF],
	    SmaxRelativeErrorBuffer[VAL_BUF],
	    SmaxSampleNumberBuffer[VAL_BUF],
            SminFiringsBuffer[VAL_BUF],
	    SseedValueBuffer[VAL_BUF],
	    SmaxModelTimeLimitBuffer[VAL_BUF],
	    SmaxCPUTimeLimitBuffer[VAL_BUF],
	    SoptThreshBuffer[VAL_BUF],
	    SCPUTimeRunBuffer[VAL_BUF];
/*new*/
/**************************************************************************/

Widget /* Widgets des Discrete Time Analytic Options Popup */
	dtanalyticPopup;
static Widget DTASmainForm,
	DTASprecisionLabel,
	DTASprecisionText,
	DTASsolveMethodLabel,
	DTASiterativeToggle,
        DTASdirectToggle,
	DTASmaxIterationsLabel,
	DTASmaxIterationsText,
	DTASembeddedKindLabel,
	DTASembnoToggle,
	DTASembtimeToggle,
        DTASembmemToggle,
        DTASscalingLabel,
	DTASscalingText,
	DTASokButton,
	DTAScancelButton,
	DTASdefaultButton,
	DTAShelpButton;
static char DTASprecisionBuffer[VAL_BUF],
            DTASscalingBuffer[VAL_BUF],
            DTASmaxIterationsBuffer[VAL_BUF];

/*new*/
/**************************************************************************/

Widget /* Widgets des Discrete Time Transient Options Popup */
	dttransientPopup;
static Widget DTTSmainForm,
	DTTSprecisionLabel,
	DTTSprecisionText,
	DTTStimeStepsLabel, /*end time*/
	DTTStimeStepsText,
        DTTSoutputLabel,
        DTTSdurationToggle,  /* curve Schalter */
        DTTSpointToggle,
        DTTSstepsizeLabel,  /* stepsize_result */
        DTTSstepsizeText,
        DTTSfirststepLabel,  /* first_step */
        DTTSfirststepText,
        DTTSembeddedKindLabel,
        DTTSembtimeToggle,
        DTTSembmemToggle,
	DTTSembnoToggle,
        DTTSscalingLabel,
	DTTSscalingText,
	DTTSokButton,
	DTTScancelButton,
	DTTSdefaultButton,
	DTTShelpButton;
static char DTTSprecisionBuffer[VAL_BUF],DTTStimeStepsBuffer[VAL_BUF],DTTSstepSizeBuffer[VAL_BUF],DTTSfirstStepBuffer[VAL_BUF],DTTSscalingBuffer[VAL_BUF];

KeyboardFocus
transient_focus={0,5,&transientPopup,
		   {&TtransientTimeText,&TprecisionText,&Tstepsize2Text,&TgenprecText,&TclusterText}};
KeyboardFocus
transientsim_focus={0,6,&transientSimulationPopup,
		   {&TStransinstimeText,&TSnumsampointsText,&TSconfidenceLevelText,&TSmaxRelativeErrorText,&TSseedValueText,&TSmaxCPUTimeLimitText}};
KeyboardFocus
analytic_focus={0,2,&analyticPopup,
		  {&ASprecisionText,&ASmaxIterationsText /*,&AStransientTimeText*/}},
approximation_focus={0,5,&approximationPopup,
	{&AprecisionText,&AmaxIterationsText,&AdroptoleranceText,&AphaseNumberText,&AvirtPhaseNumberText}},
  /*new*/
dtanalytic_focus={0,3,&dtanalyticPopup,
		  {&DTASprecisionText,&DTASmaxIterationsText,&DTASscalingText}},
dttransient_focus={0,5,&dttransientPopup,
		  {&DTTSprecisionText,&DTTStimeStepsText,&DTTSstepsizeText,
		   &DTTSfirststepText,&DTTSscalingText}},
simulation_focus={0,10,&simulationPopup,
	{&SerrPercentLevelText,&SconfidenceLevelText,&SmaxRelativeErrorText,&SseedValueText,
         &SmaxSampleNumberText,&SminFiringsText,&SmaxModelTimeLimitText,
         &SmaxCPUTimeLimitText,&SoptThreshText,&SCPUTimeRunText}};
	   

			
/*******************************************************************************
Name:
	initOptionPopups
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
aufgerufen aus:
	evalMenu.c
--------------------------------------------------------------------------------
Parameter:
	topLevel: toplevel-shell der Applikation
--------------------------------------------------------------------------------
Zweck:
	Aufbau der drei OptionPopups
	Registrieren der erforderlichen Callbacks.
*******************************************************************************/
void initOptionPopups(topLevel)
Widget topLevel;
{
/* t r a n s i e n t Options Popup ********************************************/
  transientPopup = XtVaCreatePopupShell( 
	    "Options For Transient Solution",
			transientShellWidgetClass,
			topLevel,
			NULL);
  
  TmainForm = XtVaCreateManagedWidget(
		"TmainForm",
		formWidgetClass,
		transientPopup,
		NULL);
  XtAddCallback(transientPopup,XtNpopupCallback,SetCaretsCallback, (XtPointer)TmainForm);
  XtAddCallback(transientPopup,XtNpopupCallback,CenterWidget,NULL);

  TtransientTimeLabel = XtVaCreateManagedWidget(
		"TtransientTimeLabel",
		labelWidgetClass,
		TmainForm,
		XtNlabel,"transient instant of time:",
		XtNjustify,XtJustifyLeft,
		NULL);

  TtransientTimeText = XtVaCreateManagedWidget(
		"TtransientTimeText",
		asciiTextWidgetClass,
		TmainForm,
		XtNfromHoriz,TtransientTimeLabel,
		XtNjustify,XtJustifyRight,
		XtNeditType,XawtextEdit,
		XtNstring,TtransientTimeBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,True,
		NULL);


  TgenprecLabel = XtVaCreateManagedWidget(
		"TgenprecLabel",
		labelWidgetClass,
		TmainForm,
		XtNfromVert,TtransientTimeLabel,
		XtNlabel,"precision: ",
		XtNjustify,XtJustifyLeft,
		NULL);

  TgenprecText = XtVaCreateManagedWidget(
		"TgenprecText",
		asciiTextWidgetClass,
		TmainForm,
		XtNfromHoriz,TgenprecLabel,
		XtNfromVert,TtransientTimeText,
		XtNjustify,XtJustifyRight,
		XtNeditType,XawtextEdit,
		XtNstring,TgenprecBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);

  TleerLabel = XtVaCreateManagedWidget(
		"TleerLabel",
		labelWidgetClass,
		TmainForm,
		XtNheight,10,
		XtNfromVert,TgenprecLabel,
		XtNlabel," ",
		NULL);
  
  ToutputLabel = XtVaCreateManagedWidget(
		"ToutputLabel",
		labelWidgetClass,
		TmainForm,
		XtNfromVert,TleerLabel,
		XtNlabel,"output form:",
		XtNjustify,XtJustifyLeft,
		NULL);
  TdurationToggle = XtVaCreateManagedWidget(
		"TdurationToggle",
		toggleWidgetClass,
		TmainForm,
		XtNfromVert,TleerLabel,
		XtNfromHoriz,ToutputLabel,
		XtNlabel,"curve",
		XtNradioData,Duration,
		XtNtranslations,toggle_transtable,
		NULL);
  
  TpointToggle = XtVaCreateManagedWidget(
		"TpointToggle",
		toggleWidgetClass,
		TmainForm,
		XtNlabel,"point",
		XtNradioData,Point,
		XtNtranslations,toggle_transtable,
		XtNfromVert,TleerLabel,
		XtNfromHoriz,TdurationToggle,
		NULL);

  XawToggleChangeRadioGroup(TdurationToggle,TpointToggle);
  XtAddCallback(TdurationToggle,XtNcallback,TSensitivityCallback,0);
  XtAddCallback(TpointToggle,XtNcallback,TSensitivityCallback,0);
    
  Tstepsize2Label = XtVaCreateManagedWidget(
		"Tstepsize2Label",
		labelWidgetClass,
		TmainForm,
		XtNheight,24,
		XtNfromVert,ToutputLabel,
		XtNlabel,"stepsize for output: ",
		XtNjustify,XtJustifyLeft,
		NULL);

  Tstepsize2Text = XtVaCreateManagedWidget(
		"Tstepsize2Text",
		asciiTextWidgetClass,
		TmainForm,
		XtNheight,24,
		XtNfromHoriz,Tstepsize2Label,
		XtNfromVert,TdurationToggle,
		XtNjustify,XtJustifyRight,
		XtNeditType,XawtextEdit,
		XtNstring,Tstepsize2Buffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);

  Tleer15Label = XtVaCreateManagedWidget(
		"Tleer15Label",
		labelWidgetClass,
		TmainForm,
		XtNheight,10,
		XtNfromVert,Tstepsize2Label,
		XtNlabel," ",
		NULL);

  TvariantLabel = XtVaCreateManagedWidget(
		"TvariantLabel",
		labelWidgetClass,
		TmainForm,
		XtNfromVert,Tleer15Label,
		XtNlabel,"algorithm variant:",
		XtNjustify,XtJustifyLeft,
		NULL);

  ToldToggle = XtVaCreateManagedWidget(
		"ToldToggle",
		toggleWidgetClass,
		TmainForm,
		XtNfromVert,Tleer15Label,
		XtNfromHoriz,TvariantLabel,
		XtNlabel,"   a   ",
		XtNradioData,1,
		XtNtranslations,toggle_transtable,
		NULL);
  
  TnewToggle = XtVaCreateManagedWidget(
		"TnewToggle",
		toggleWidgetClass,
		TmainForm,
		XtNlabel,"   b   ",
		XtNradioData,2,
		XtNtranslations,toggle_transtable,
		XtNfromVert,Tleer15Label,
		XtNfromHoriz,ToldToggle,
		NULL);

  XawToggleChangeRadioGroup(ToldToggle,TnewToggle);
  XtAddCallback(ToldToggle,XtNcallback,TSSensitivityCallback,0);
  XtAddCallback(TnewToggle,XtNcallback,TSSensitivityCallback,0);

  TprecisionLabel = XtVaCreateManagedWidget(
		"TprecisionLabel",
		labelWidgetClass,
		TmainForm,
		XtNjustify,XtJustifyLeft,
		XtNfromVert,TvariantLabel,
		XtNlabel,"internal stepsize: ",
		NULL);

  TprecisionText = XtVaCreateManagedWidget(
	"TprecisionText",
		asciiTextWidgetClass,
		TmainForm,
		XtNfromHoriz,TprecisionLabel,
		XtNfromVert,ToldToggle,
		XtNeditType,XawtextEdit,
		XtNstring,TprecisionBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);

  TclusterLabel = XtVaCreateManagedWidget(
		"TclusterLabel",
		labelWidgetClass,
		TmainForm,
		XtNfromVert,TprecisionLabel,
		XtNlabel,"cluster size: ",
		XtNjustify,XtJustifyLeft,
		NULL);

  TclusterText = XtVaCreateManagedWidget(
		"TclusterText",
		asciiTextWidgetClass,
		TmainForm,
		XtNfromHoriz,TclusterLabel,
		XtNfromVert,TprecisionText,
		XtNjustify,XtJustifyRight,
		XtNeditType,XawtextEdit,
		XtNstring,TclusterBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);

  Tleer2Label = XtVaCreateManagedWidget(
		"Tleer2Label",
		labelWidgetClass,
		TmainForm,
		XtNheight,8,
		XtNfromVert,TclusterLabel,
		XtNlabel," ",
		NULL);

  TokButton = XtVaCreateManagedWidget(
	"TokButton",
		commandWidgetClass,
		TmainForm,
		XtNfromVert,Tleer2Label,
		XtNlabel,"OK",
		XtNaccelerators,ok_button_accelerators,
		NULL);
		XtInstallAccelerators(transientPopup,TokButton);
		XtAddCallback(TokButton,XtNcallback,TransientCallback,(XtPointer)okButton);
  
  TcancelButton = XtVaCreateManagedWidget(
	"TcancelButton",
		commandWidgetClass,
	TmainForm,
	XtNfromHoriz,TokButton,
	XtNfromVert,Tleer2Label,
		XtNlabel,"Cancel",
		NULL);
  XtAddCallback(TcancelButton,XtNcallback,TransientCallback,
    (XtPointer)cancelButton);

  TdefaultButton = XtVaCreateManagedWidget(
	"TdefaultButton",
		commandWidgetClass,
	TmainForm,
	XtNfromHoriz,TcancelButton,
	XtNfromVert,Tleer2Label,
		XtNlabel,"Default",
		NULL);
  XtAddCallback(TdefaultButton,XtNcallback,TransientCallback,
    (XtPointer)defaultButton);
  
  ThelpButton = XtVaCreateManagedWidget(
	"ThelpButton",
		commandWidgetClass,
		TmainForm,
		XtNlabel,"Help",
		XtNsensitive,True,
		XtNfromHoriz,TdefaultButton,
		XtNfromVert,Tleer2Label,
		NULL);
  XtAddCallback(ThelpButton,XtNcallback,TransientCallback,
    (XtPointer)helpButton);

/**********************************************************************************/
/* t r a n s i e n t S i m u l a t i o n Options Popup ****************************/
/**********************************************************************************/  
  transientSimulationPopup = XtVaCreatePopupShell(
	    "Options For Transient Simulation",
			transientShellWidgetClass,
			topLevel,
			NULL);
  
  TSmainForm = XtVaCreateManagedWidget(
		"TSmainForm",
		formWidgetClass,
		transientSimulationPopup,
		NULL);
  XtAddCallback(transientSimulationPopup,XtNpopupCallback,SetCaretsCallback, (XtPointer)TSmainForm);
  XtAddCallback(transientSimulationPopup,XtNpopupCallback,CenterWidget,NULL);

  TStransimForm = XtVaCreateManagedWidget(
		"TStransimForm",
		formWidgetClass,
		TSmainForm,
		XtNfromVert,SstandardToggle,
		NULL);

  TStransinstimeLabel = XtVaCreateManagedWidget(
		"TStransinstimeLabel",
		labelWidgetClass,
		TStransimForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Transient instant of time:",
		NULL);

  TStransinstimeText = XtVaCreateManagedWidget(
		"TStransinstimeText",
		asciiTextWidgetClass,
		TStransimForm,
		XtNfromHoriz,TStransinstimeLabel,
		XtNeditType,XawtextEdit,
		XtNstring,TStransintimeBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,True,
		NULL);
  TSnumsampointsLabel = XtVaCreateManagedWidget(
		"TSnumsampoints Label",
		labelWidgetClass,
		TStransimForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Number of sampling points: ",
		XtNfromVert,TStransinstimeLabel,
		NULL);

  TSnumsampointsText = XtVaCreateManagedWidget(
		"TSnumsampointsText",
		asciiTextWidgetClass,
		TStransimForm,
		XtNfromVert,TStransinstimeLabel,
		XtNfromHoriz,TSnumsampointsLabel,
		XtNeditType,XawtextEdit,
		XtNstring,TSnumsampointsBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,True,
		NULL);
  
  TSsimulationTypeLabel = XtVaCreateManagedWidget(
		"TSsimulationTypeLabel",
		labelWidgetClass,
		TStransimForm,
		XtNfromVert,TSnumsampointsLabel,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Simulation: ",
		NULL);

  TSsequentialToggle = XtVaCreateManagedWidget(
		"TSsequentialToggle",
		toggleWidgetClass,
		TStransimForm,
		XtNlabel,"sequential",
		XtNradioData,Sequential,
		XtNtranslations,toggle_transtable,
		XtNfromVert,TSnumsampointsLabel,
		XtNfromHoriz,TSsimulationTypeLabel,
		NULL);
		
  TSdistributedToggle = XtVaCreateManagedWidget(
		"TSdistributedToggle",
		toggleWidgetClass,
		TStransimForm,
		XtNlabel,"distributed",
		XtNradioData,Parallel,
		XtNtranslations,toggle_transtable,
		XtNfromVert,TSnumsampointsLabel,
		XtNfromHoriz,TSsequentialToggle,
		NULL);

  XawToggleChangeRadioGroup(TSsequentialToggle,TSdistributedToggle);

  
  TSruleTypeLabel = XtVaCreateManagedWidget(
		"TSruleTypeLabel",
		labelWidgetClass,
		TStransimForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Percentage rule",
		XtNfromVert,TSsimulationTypeLabel,
		NULL);

  TSruleONToggle = XtVaCreateManagedWidget(
		"TSruleONToggle",
		toggleWidgetClass,
		TStransimForm,
		XtNlabel,"ON",
		XtNradioData,Rule_ON,
		XtNtranslations,toggle_transtable,
		XtNfromVert,TSsimulationTypeLabel,
		XtNfromHoriz,TSruleTypeLabel,
		NULL);
		
  TSruleOFFToggle = XtVaCreateManagedWidget(
		"TSruleOFFToggle",
		toggleWidgetClass,
		TStransimForm,
		XtNlabel,"OFF",
		XtNradioData,Rule_OFF,
		XtNtranslations,toggle_transtable,
		XtNfromVert,TSsimulationTypeLabel,
		XtNfromHoriz,TSruleONToggle,
		NULL);
  
  XawToggleChangeRadioGroup(TSruleONToggle,TSruleOFFToggle);
  
  TSconfidenceLevelLabel = XtVaCreateManagedWidget(
		"TSconfidenceLevelLabel",
		labelWidgetClass,
		TStransimForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Confidence level [%]: ",
		XtNfromVert,TSruleTypeLabel,
		NULL);

  TSconfidenceLevelText = XtVaCreateManagedWidget(
		"TSconfidenceLevelText",
		asciiTextWidgetClass,
		TStransimForm,
		XtNfromVert,TSruleTypeLabel,
		XtNfromHoriz,TSconfidenceLevelLabel,
		XtNeditType,XawtextEdit,
		XtNstring,TSconfidenceLevelBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,True,
		NULL);
 
  TSmaxRelativeErrorLabel = XtVaCreateManagedWidget(
		"TSmaxRelativeErrorLabel",
		labelWidgetClass,
		TStransimForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Maximum relative error [%]: ",
		XtNfromVert,TSconfidenceLevelText,
		NULL);
  
  TSmaxRelativeErrorText = XtVaCreateManagedWidget(
		"TSmaxRelativeErrorText",
		asciiTextWidgetClass,
		TStransimForm,
		XtNfromVert,TSconfidenceLevelText,
		XtNfromHoriz,TSmaxRelativeErrorLabel,
		XtNeditType,XawtextEdit,
		XtNstring,TSmaxRelativeErrorBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);
  
  TSseedValueLabel = XtVaCreateManagedWidget(
		"TSseedValueLabel",
		labelWidgetClass,
		TStransimForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Seed value:",
		XtNfromVert,TSmaxRelativeErrorText,
		NULL);
  
  TSseedValueText = XtVaCreateManagedWidget(
		"TSseedValueText",
		asciiTextWidgetClass,
		TStransimForm,
		XtNfromVert,TSmaxRelativeErrorText,
		XtNfromHoriz,TSseedValueLabel,
		XtNeditType,XawtextEdit,
		XtNstring,TSseedValueBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);

  TSmaxCPUTimeLabel = XtVaCreateManagedWidget(
		"TSmaxCPUTimeLabel",
		labelWidgetClass,
		TStransimForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Max real time [sec.]:",
		XtNfromVert,TSseedValueLabel,
		NULL);
  
  TSnoCPUTimeLimitToggle = XtVaCreateManagedWidget(
		"TSnoCPUTimeLimitToggle",
		toggleWidgetClass,
		TStransimForm,
		XtNlabel,"no limit",
		XtNfromHoriz,TSmaxCPUTimeLabel,
		XtNfromVert,TSseedValueLabel,
		NULL);
  XtAddCallback(TSnoCPUTimeLimitToggle,XtNcallback,TS_CPUTimeCallback,NULL);

  TSmaxCPUTimeLimitText = XtVaCreateManagedWidget(
		"TSmaxCPUTimeLimitText",
		asciiTextWidgetClass,
		TStransimForm,
		XtNfromHoriz,TSnoCPUTimeLimitToggle,
		XtNfromVert,TSseedValueLabel,
		XtNeditType,XawtextEdit,
		XtNstring,TSmaxCPUTimeLimitBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);
    
   TSokButton = XtVaCreateManagedWidget(
		"TSokButton",
		commandWidgetClass,
		TSmainForm,
		XtNlabel,"OK",
		XtNfromVert,TStransimForm,
		XtNaccelerators,ok_button_accelerators,
		NULL);
  XtInstallAccelerators(transientSimulationPopup,TSokButton);
  XtAddCallback(TSokButton,XtNcallback,TransientSimulationCallback,(XtPointer)okButton);
  
  TScancelButton = XtVaCreateManagedWidget(
		"TScancelButton",
		commandWidgetClass,
		TSmainForm,
		XtNlabel,"Cancel",
		XtNfromVert,TStransimForm,
		XtNfromHoriz,TSokButton,
		NULL);
  XtAddCallback(TScancelButton,XtNcallback,TransientSimulationCallback,(XtPointer)cancelButton);

  TSdefaultButton = XtVaCreateManagedWidget(
		"TSdefaultButton",
		commandWidgetClass,
		TSmainForm,
		XtNlabel,"Default",
		XtNfromHoriz,TScancelButton,
		XtNfromVert,TStransimForm,
		NULL);
  XtAddCallback(TSdefaultButton,XtNcallback,TransientSimulationCallback,(XtPointer)defaultButton);
  
  TShelpButton = XtVaCreateManagedWidget(
		"TShelpButton",
		commandWidgetClass,
		TSmainForm,
		XtNlabel,"Help",
		XtNsensitive,False,
		XtNfromHoriz,TSdefaultButton,
		XtNfromVert,TStransimForm,
		NULL);
  XtAddCallback(TShelpButton,XtNcallback,TransientSimulationCallback,
    (XtPointer)helpButton);

					     
/******************************************************************************/
/* analyticOptions Popup ******************************************************/
/******************************************************************************/

  analyticPopup = XtVaCreatePopupShell(
			"Options For Analytic Solution",
			transientShellWidgetClass,
			topLevel,
			NULL);
/*  XtAddCallback(analyticPopup,XtNpopdownCallback,PopdownCallback,0); */
  
  ASmainForm = XtVaCreateManagedWidget(
		"ASmainForm",
		formWidgetClass,
		analyticPopup,
		NULL);

  XtAddCallback(analyticPopup,XtNpopupCallback,SetCaretsCallback,
    (XtPointer)ASmainForm);
  XtAddCallback(analyticPopup,XtNpopupCallback,CenterWidget,NULL);

  ASprecisionLabel = XtVaCreateManagedWidget(
		"ASprecisionLabel",
		labelWidgetClass,
		ASmainForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Precision: ",
		NULL);

  ASprecisionText = XtVaCreateManagedWidget(
		"ASprecisionText",
		asciiTextWidgetClass,
		ASmainForm,
		XtNfromHoriz,ASprecisionLabel,
		XtNeditType,XawtextEdit,
		XtNstring,ASprecisionBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,True,
		NULL);
/*  XtInstallAccelerators(analyticPopup,ASprecisionText);*/

  ASsolveMethodLabel = XtVaCreateManagedWidget(
		"ASsolveMethodLabel",
		labelWidgetClass,
		ASmainForm,
		XtNfromVert,ASprecisionText,
		XtNlabel,"Linear system solution:",
		NULL);

  ASiterativeToggle = XtVaCreateManagedWidget(
		"ASiterativeToggle",
		toggleWidgetClass,
		ASmainForm,
		XtNfromHoriz,ASsolveMethodLabel,
		XtNfromVert,ASprecisionText,
		XtNlabel,"iterative",
		XtNradioData,Iterative,
		XtNtranslations,toggle_transtable,
		NULL);
  
  ASdirectToggle = XtVaCreateManagedWidget(
		"ASdirectToggle",
		toggleWidgetClass,
		ASmainForm,
		XtNlabel,"direct",
		XtNradioData,Direct,
		XtNtranslations,toggle_transtable,
		XtNfromHoriz,ASiterativeToggle,
		XtNfromVert,ASprecisionText,
		NULL);

  XawToggleChangeRadioGroup(ASiterativeToggle,ASdirectToggle);
  XtAddCallback(ASiterativeToggle,XtNcallback,ASSensitivityCallback,0);
  XtAddCallback(ASdirectToggle,XtNcallback,ASSensitivityCallback,0);
    
  ASmaxIterationsLabel = XtVaCreateManagedWidget(
		"ASmaxIterationsLabel",
		labelWidgetClass,
		ASmainForm,
		XtNlabel,"maximum iterations:",
		XtNfromVert,ASdirectToggle,
		NULL);
  
  ASmaxIterationsText = XtVaCreateManagedWidget(
		"ASmaxIterationsText",
		asciiTextWidgetClass,
		ASmainForm,
		XtNfromVert,ASdirectToggle,
		XtNfromHoriz,ASmaxIterationsLabel,
		XtNeditType,XawtextEdit,
		XtNstring,ASmaxIterationsBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);
  
  AScalculationKindLabel = XtVaCreateManagedWidget(
		"AScalculationKindLabel",
		labelWidgetClass,
		ASmainForm,
		XtNlabel,"Solution of the SMCs:",
		XtNfromVert,ASmaxIterationsText,
		NULL);

  ASsequentialToggle = XtVaCreateManagedWidget(
		"ASsequentialToggle",
		toggleWidgetClass,
		ASmainForm,
		XtNlabel,"sequential",
		XtNradioData,Sequential,
		XtNtranslations,toggle_transtable,
		XtNfromHoriz,AScalculationKindLabel,
		XtNfromVert,ASmaxIterationsText,
		NULL);
  ASparallelToggle = XtVaCreateManagedWidget(
		"ASparallelToggle",
		toggleWidgetClass,
		ASmainForm,
		XtNlabel,"distributed",
		XtNradioData,Parallel,
		XtNtranslations,toggle_transtable,
		XtNfromHoriz,ASsequentialToggle,
		XtNfromVert,ASmaxIterationsText,
		NULL);
  
  XawToggleChangeRadioGroup(ASsequentialToggle,ASparallelToggle);

  /*  AStransientForm = XtVaCreateManagedWidget(
		"AStransientForm",
		formWidgetClass,
		ASmainForm,
		XtNfromVert,ASsteadyStateForm,
		NULL);
  
  AStransientTimeLabel = XtVaCreateManagedWidget(
		"AStransientTimeLabel",
		labelWidgetClass,
		AStransientForm,
		XtNlabel,"Transient instant of time: ",
		NULL);

  AStransientTimeText = XtVaCreateManagedWidget(
		"AStransientTimeText",
		asciiTextWidgetClass,
		AStransientForm,
		XtNfromHoriz,AStransientTimeLabel,
		XtNeditType,XawtextEdit,
		XtNstring,AStransientTimeBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL); */
  
  ASokButton = XtVaCreateManagedWidget(
		"ASokButton",
		commandWidgetClass,
		ASmainForm,
		XtNfromVert,ASparallelToggle,/*AStransientForm,*/
		XtNlabel,"OK",
		XtNaccelerators,ok_button_accelerators,
		NULL);
  XtInstallAccelerators(analyticPopup,ASokButton);
  XtAddCallback(ASokButton,XtNcallback,AnalyticCallback,(XtPointer)okButton);
  
  AScancelButton = XtVaCreateManagedWidget(
		"AScancelButton",
		commandWidgetClass,
		ASmainForm,
		XtNfromHoriz,ASokButton,
		XtNfromVert,ASparallelToggle,/*AStransientForm,*/
		XtNlabel,"Cancel",
		NULL);
  XtAddCallback(AScancelButton,XtNcallback,AnalyticCallback,
    (XtPointer)cancelButton);

  ASdefaultButton = XtVaCreateManagedWidget(
		"ASdefaultButton",
		commandWidgetClass,
		ASmainForm,
		XtNfromHoriz,AScancelButton,
		XtNfromVert,ASparallelToggle,/*AStransientForm,*/
		XtNlabel,"Default",
		NULL);
  XtAddCallback(ASdefaultButton,XtNcallback,AnalyticCallback,
    (XtPointer)defaultButton);
  
  AShelpButton = XtVaCreateManagedWidget(
		"AShelpButton",
		commandWidgetClass,
		ASmainForm,
		XtNlabel,"Help",
		XtNsensitive,False,
		XtNfromHoriz,ASdefaultButton,
		XtNfromVert,ASparallelToggle,/*AStransientForm,*/
		NULL);
  XtAddCallback(AShelpButton,XtNcallback,AnalyticCallback,
    (XtPointer)helpButton);

  /*new*/					     
/******************************************************************************/
/* dtanalyticOptions Popup *****Discrete Time Analytic solutions **************/
/******************************************************************************/

  dtanalyticPopup = XtVaCreatePopupShell(
			"Options For Discrete Time Analytic Solution",
			transientShellWidgetClass,
			topLevel,
			NULL);
/*  XtAddCallback(analyticPopup,XtNpopdownCallback,PopdownCallback,0); */
  
  DTASmainForm = XtVaCreateManagedWidget(
		"DTASmainForm",
		formWidgetClass,
		dtanalyticPopup,
		NULL);
  XtAddCallback(dtanalyticPopup,XtNpopupCallback,SetCaretsCallback,
    (XtPointer)DTASmainForm);
  XtAddCallback(dtanalyticPopup,XtNpopupCallback,CenterWidget,NULL);

  DTASprecisionLabel = XtVaCreateManagedWidget(
		"DTASprecisionLabel",
		labelWidgetClass,
		DTASmainForm,
		XtNjustify,XtJustifyLeft,
		XtNwidth,SMALL_LABEL_WIDTH,
		XtNlabel,"Precision: ",
		NULL);

  DTASprecisionText = XtVaCreateManagedWidget(
		"DTASprecisionText",
		asciiTextWidgetClass,
		DTASmainForm,
		XtNjustify,XtJustifyRight,
		XtNfromHoriz,DTASprecisionLabel,
		XtNeditType,XawtextEdit,
		XtNwidth,SMALL_TEXT_WIDTH,
		XtNstring,DTASprecisionBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,True,
		NULL);
  
  DTASsolveMethodLabel = XtVaCreateManagedWidget(
		"DTASsolveMethodLabel",
		labelWidgetClass,
		DTASmainForm,
		XtNfromVert,DTASprecisionText,
		XtNwidth,SMALL_LABEL_WIDTH,
		XtNlabel,"Linear system solution:",
		NULL);
  DTASiterativeToggle = XtVaCreateManagedWidget(
		"DTASiterativeToggle",
		toggleWidgetClass,
		DTASmainForm,
		XtNfromVert,DTASprecisionText,
		XtNfromHoriz,DTASsolveMethodLabel,
		XtNwidth,BUTTON_WIDTH,
		XtNlabel,"iterative",
		XtNradioData,Iterative,
		XtNtranslations,toggle_transtable,
		NULL);
  
  DTASdirectToggle = XtVaCreateManagedWidget(
		"DTASdirectToggle",
		toggleWidgetClass,
		DTASmainForm,
		XtNwidth,BUTTON_WIDTH,
		XtNlabel,"direct",
		XtNradioData,Direct,
		XtNtranslations,toggle_transtable,
		XtNfromVert,DTASprecisionText,
		XtNfromHoriz,DTASiterativeToggle,
		NULL);

  XawToggleChangeRadioGroup(DTASiterativeToggle,DTASdirectToggle);
  XtAddCallback(DTASiterativeToggle,XtNcallback,ASSensitivityCallback,0);
  XtAddCallback(DTASdirectToggle,XtNcallback,ASSensitivityCallback,0);
    
  DTASmaxIterationsLabel = XtVaCreateManagedWidget(
		"DTASmaxIterationsLabel",
		labelWidgetClass,
		DTASmainForm,
		XtNwidth,SMALL_LABEL_WIDTH,
		XtNlabel,"maximum iterations:",
		XtNjustify,XtJustifyLeft,
		XtNfromVert,DTASdirectToggle,
		NULL);
  
  DTASmaxIterationsText = XtVaCreateManagedWidget(
		"DTASmaxIterationsText",
		asciiTextWidgetClass,
		DTASmainForm,
		XtNfromVert,DTASdirectToggle,
		XtNfromHoriz,DTASmaxIterationsLabel,
		XtNjustify,XtJustifyRight,
		XtNeditType,XawtextEdit,
		XtNwidth,SMALL_TEXT_WIDTH,
		XtNstring,DTASmaxIterationsBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);
  
  DTASembeddedKindLabel = XtVaCreateManagedWidget(
		"DTASembeddedKindLabel",
		labelWidgetClass,
		DTASmainForm,
		XtNwidth,SMALL_LABEL_WIDTH,
		XtNlabel,"Embedding:",
		XtNfromVert,DTASmaxIterationsText,
		XtNjustify,XtJustifyLeft,
		NULL);

  DTASembnoToggle = XtVaCreateManagedWidget(
		"DTASembnoToggle",
		toggleWidgetClass,
		DTASmainForm,
		XtNwidth,BUTTON_WIDTH2,
		XtNlabel,"no",
		XtNradioData,embno,
		XtNtranslations,toggle_transtable,
		XtNfromHoriz,DTASembeddedKindLabel,
		XtNfromVert,DTASmaxIterationsText,
		NULL);
  DTASembtimeToggle = XtVaCreateManagedWidget(
  		"DTASembtimeToggle",
		toggleWidgetClass,
		DTASmainForm,
		XtNwidth,BUTTON_WIDTH2,
		XtNlabel,"time",
		XtNradioData,embtime,
		XtNtranslations,toggle_transtable,
		XtNfromHoriz,DTASembnoToggle,
		XtNfromVert,DTASmaxIterationsText,
		NULL);

  DTASembmemToggle = XtVaCreateManagedWidget(
  		"DTASembmemToggle",
		toggleWidgetClass,
		DTASmainForm,
		XtNwidth,BUTTON_WIDTH2,
		XtNlabel,"memory",
		XtNradioData,embmem,
		XtNtranslations,toggle_transtable,
		XtNfromHoriz,DTASembtimeToggle,
		XtNfromVert,DTASmaxIterationsText,
		NULL);
  XawToggleChangeRadioGroup(DTASembtimeToggle,DTASembnoToggle);
  XawToggleChangeRadioGroup(DTASembmemToggle,DTASembnoToggle);
 
  DTASscalingLabel = XtVaCreateManagedWidget(
		"DTASscalingLabel",
		labelWidgetClass,
		DTASmainForm,
		XtNheight,24,
		XtNfromVert,DTASembeddedKindLabel,
		XtNwidth,SMALL_LABEL_WIDTH,
		XtNlabel,"underlying time-step: 1/",
		XtNjustify,XtJustifyLeft,
		NULL);

  DTASscalingText = XtVaCreateManagedWidget(
		"DTASscalingText",
		asciiTextWidgetClass,
		DTASmainForm,
		XtNheight,24,
		XtNfromHoriz,DTASscalingLabel,
		XtNfromVert,DTASembeddedKindLabel,
		XtNjustify,XtJustifyRight,
		XtNeditType,XawtextEdit,
		XtNwidth,SMALL_TEXT_WIDTH,
		XtNstring,DTASscalingBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);

  DTASokButton = XtVaCreateManagedWidget(
		"DTASokButton",
		commandWidgetClass,
		DTASmainForm,
		XtNfromVert,DTASscalingLabel,
		XtNlabel,"OK",
		XtNaccelerators,ok_button_accelerators,
		NULL);
  XtInstallAccelerators(dtanalyticPopup,DTASokButton);
  XtAddCallback(DTASokButton,XtNcallback,AnalyticCallback,(XtPointer)okButton);
  
  DTAScancelButton = XtVaCreateManagedWidget(
		"DTAScancelButton",
		commandWidgetClass,
		DTASmainForm,
		XtNfromVert,DTASscalingLabel,
		XtNfromHoriz,DTASokButton,
		XtNlabel,"Cancel",
		NULL);
  XtAddCallback(DTAScancelButton,XtNcallback,AnalyticCallback,
    (XtPointer)cancelButton);

  DTASdefaultButton = XtVaCreateManagedWidget(
		"DTASdefaultButton",
		commandWidgetClass,
		DTASmainForm,
		XtNfromVert,DTASscalingLabel,
		XtNfromHoriz,DTAScancelButton,
		XtNlabel,"Default",
		NULL);
  XtAddCallback(DTASdefaultButton,XtNcallback,AnalyticCallback,
    (XtPointer)defaultButton);
  
  DTAShelpButton = XtVaCreateManagedWidget(
		"DTAShelpButton",
		commandWidgetClass,
		DTASmainForm,
		XtNlabel,"Help",
		XtNsensitive,False,
		XtNfromVert,DTASscalingLabel,
		XtNfromHoriz,DTASdefaultButton,
		NULL);
  XtAddCallback(DTAShelpButton,XtNcallback,AnalyticCallback,
    (XtPointer)helpButton);

  /*new*/					     
/******************************************************************************/
/* dttransientOptions Popup *****Discrete Time Transient solutions **************/
/******************************************************************************/

  dttransientPopup = XtVaCreatePopupShell(
			"Options For Discrete Time Transient Solution",
			transientShellWidgetClass,
			topLevel,
			NULL);
  
  DTTSmainForm = XtVaCreateManagedWidget(
		"DTTSmainForm",
		formWidgetClass,
		dttransientPopup,
		NULL);
  XtAddCallback(dttransientPopup,XtNpopupCallback,SetCaretsCallback,
    (XtPointer)DTTSmainForm);
  XtAddCallback(dttransientPopup,XtNpopupCallback,CenterWidget,NULL);

  DTTSfirststepLabel = XtVaCreateManagedWidget(
		"DTTSfirststepLabel",
		labelWidgetClass,
		DTTSmainForm,
		XtNheight,24,
		XtNwidth,SMALL_LABEL_WIDTH,
		XtNlabel,"begin time for output:",
		XtNjustify,XtJustifyLeft,
		NULL);

  DTTSfirststepText = XtVaCreateManagedWidget(
		"DTTSfirststepText",
		asciiTextWidgetClass,
		DTTSmainForm,
		XtNheight,24,
		XtNfromHoriz,DTTSfirststepLabel,
		XtNjustify,XtJustifyRight,
		XtNeditType,XawtextEdit,
		XtNwidth,SMALL_TEXT_WIDTH,
		XtNstring,DTTSfirstStepBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,True,
		NULL);

  DTTStimeStepsLabel = XtVaCreateManagedWidget(
		"DTTStimeStepsLabel",
		labelWidgetClass,
		DTTSmainForm,
		XtNjustify,XtJustifyLeft,
		XtNfromVert,DTTSfirststepLabel,
		XtNwidth,SMALL_LABEL_WIDTH,
		XtNlabel,"end time for output:",
		NULL);
  
  DTTStimeStepsText = XtVaCreateManagedWidget(
		"DTTStimeStepsText",
		asciiTextWidgetClass,
		DTTSmainForm,
		XtNfromVert,DTTSfirststepLabel,
		XtNfromHoriz,DTTStimeStepsLabel,
		XtNeditType,XawtextEdit,
		XtNwidth,SMALL_TEXT_WIDTH,
		XtNstring,DTTStimeStepsBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);

  DTTSscalingLabel = XtVaCreateManagedWidget(
		"DTTSscalingLabel",
		labelWidgetClass,
		DTTSmainForm,
		XtNheight,24,
		XtNfromVert,DTTStimeStepsLabel,
		XtNwidth,SMALL_LABEL_WIDTH,
		XtNlabel,"underlying time-step (uts):  1/",
		XtNjustify,XtJustifyLeft,
		NULL);

  DTTSscalingText = XtVaCreateManagedWidget(
		"DTTSscalingText",
		asciiTextWidgetClass,
		DTTSmainForm,
		XtNheight,24,
		XtNfromHoriz,DTTSscalingLabel,
		XtNfromVert,DTTStimeStepsLabel,
		XtNjustify,XtJustifyRight,
		XtNeditType,XawtextEdit,
		XtNwidth,SMALL_TEXT_WIDTH,
		XtNstring,DTTSscalingBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);

  DTTSstepsizeLabel = XtVaCreateManagedWidget(
		"DTTSstepsizeLabel",
		labelWidgetClass,
		DTTSmainForm,
		XtNheight,24,
		XtNfromVert,DTTSscalingLabel,
		XtNwidth,SMALL_LABEL_WIDTH,
		XtNlabel,"plotfactor for uts:",
		XtNjustify,XtJustifyLeft,
		NULL);

  DTTSstepsizeText = XtVaCreateManagedWidget(
		"DTTSstepsizeText",
		asciiTextWidgetClass,
		DTTSmainForm,
		XtNheight,24,
		XtNfromHoriz,DTTSstepsizeLabel,
		XtNfromVert,DTTSscalingLabel,
		XtNjustify,XtJustifyRight,
		XtNeditType,XawtextEdit,
		XtNwidth,SMALL_TEXT_WIDTH,
		XtNstring,DTTSstepSizeBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);

  DTTSprecisionLabel = XtVaCreateManagedWidget(
		"DTTSprecisionLabel",
		labelWidgetClass,
		DTTSmainForm,
		XtNfromVert,DTTSstepsizeLabel,
		XtNjustify,XtJustifyLeft,
		XtNwidth,SMALL_LABEL_WIDTH,
		XtNlabel,"Precision: ",
		NULL);

  DTTSprecisionText = XtVaCreateManagedWidget(
		"DTTSprecisionText",
		asciiTextWidgetClass,
		DTTSmainForm,
		XtNfromVert,DTTSstepsizeLabel,
		XtNfromHoriz,DTTSprecisionLabel,
		XtNeditType,XawtextEdit,
		XtNwidth,SMALL_TEXT_WIDTH,
		XtNstring,DTTSprecisionBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,	
		XtNjustify,XtJustifyRight,
		NULL);
  
  DTTSoutputLabel = XtVaCreateManagedWidget(
		"DTTSoutputLabel",
		labelWidgetClass,
		DTTSmainForm,
		XtNfromVert,DTTSprecisionLabel,
		XtNwidth,SMALL_LABEL_WIDTH,
		XtNlabel,"output form:",
		XtNjustify,XtJustifyLeft,
		NULL);
  DTTSdurationToggle = XtVaCreateManagedWidget(
		"DTTSdurationToggle",
		toggleWidgetClass,
		DTTSmainForm,
		XtNfromVert,DTTSprecisionLabel,
		XtNfromHoriz,DTTSoutputLabel,
		XtNwidth,BUTTON_WIDTH,
		XtNlabel,"curve",
		XtNradioData,Duration,
		XtNtranslations,toggle_transtable,
		NULL);
  
  DTTSpointToggle = XtVaCreateManagedWidget(
		"DTTSpointToggle",
		toggleWidgetClass,
		DTTSmainForm,
		XtNwidth,BUTTON_WIDTH,
		XtNlabel,"point",
		XtNradioData,Point,
		XtNtranslations,toggle_transtable,
		XtNfromVert,DTTSprecisionLabel,
		XtNfromHoriz,DTTSdurationToggle,
		NULL);

  XawToggleChangeRadioGroup(DTTSdurationToggle,DTTSpointToggle);
  XtAddCallback(DTTSdurationToggle,XtNcallback,TSensitivityCallback,0);
  XtAddCallback(DTTSpointToggle,XtNcallback,TSensitivityCallback,0);

  DTTSembeddedKindLabel = XtVaCreateManagedWidget(
		"DTTSembeddedKindLabel",
		labelWidgetClass,
		DTTSmainForm,
		XtNwidth,SMALL_LABEL_WIDTH,
		XtNlabel,"Embedding:",
		XtNfromVert,DTTSoutputLabel,
		XtNjustify,XtJustifyLeft,
		NULL);

  DTTSembnoToggle = XtVaCreateManagedWidget(
		"DTTSembnoToggle",
		toggleWidgetClass,
		DTTSmainForm,
		XtNwidth,BUTTON_WIDTH2,
		XtNlabel,"no",
		XtNradioData,embno,
		XtNtranslations,toggle_transtable,
		XtNfromHoriz,DTTSembeddedKindLabel,
		XtNfromVert,DTTSoutputLabel,
		NULL);
  DTTSembtimeToggle = XtVaCreateManagedWidget(
  		"DTTSembtimeToggle",
		toggleWidgetClass,
		DTTSmainForm,
		XtNwidth,BUTTON_WIDTH2,
		XtNlabel,"time",
		XtNradioData,embtime,
		XtNtranslations,toggle_transtable,
		XtNfromHoriz,DTTSembnoToggle,
		XtNfromVert,DTTSoutputLabel,
		NULL);
  DTTSembmemToggle = XtVaCreateManagedWidget(
  		"DTTSembmemToggle",
		toggleWidgetClass,
		DTTSmainForm,
		XtNwidth,BUTTON_WIDTH2,
		XtNlabel,"memory",
		XtNradioData,embmem,
		XtNtranslations,toggle_transtable,
		XtNfromHoriz,DTTSembtimeToggle,
		XtNfromVert,DTTSoutputLabel,
		NULL);
  XawToggleChangeRadioGroup(DTTSembtimeToggle,DTTSembnoToggle);
  XawToggleChangeRadioGroup(DTTSembmemToggle,DTTSembnoToggle);

  DTTSokButton = XtVaCreateManagedWidget(
		"DTTSokButton",
		commandWidgetClass,
		DTTSmainForm,
		XtNfromVert,DTTSembeddedKindLabel,
		XtNlabel,"OK",
		XtNaccelerators,ok_button_accelerators,
		NULL);
  XtInstallAccelerators(dttransientPopup,DTTSokButton);
  XtAddCallback(DTTSokButton,XtNcallback,AnalyticCallback,(XtPointer)okButton);
  
  DTTScancelButton = XtVaCreateManagedWidget(
		"DTTScancelButton",
		commandWidgetClass,
		DTTSmainForm,
		XtNfromVert,DTTSembeddedKindLabel,
		XtNfromHoriz,DTTSokButton,
		XtNlabel,"Cancel",
		NULL);
  XtAddCallback(DTTScancelButton,XtNcallback,AnalyticCallback,
    (XtPointer)cancelButton);

  DTTSdefaultButton = XtVaCreateManagedWidget(
		"DTTSdefaultButton",
		commandWidgetClass,
		DTTSmainForm,
		XtNfromVert,DTTSembeddedKindLabel,
		XtNfromHoriz,DTTScancelButton,
		XtNlabel,"Default",
		NULL);
  XtAddCallback(DTTSdefaultButton,XtNcallback,AnalyticCallback,
    (XtPointer)defaultButton);
  
  DTTShelpButton = XtVaCreateManagedWidget(
		"DTTShelpButton",
		commandWidgetClass,
		DTTSmainForm,
		XtNlabel,"Help",
		XtNsensitive,False,
		XtNfromVert,DTTSembeddedKindLabel,
		XtNfromHoriz,DTTSdefaultButton,
		NULL);
  XtAddCallback(DTTShelpButton,XtNcallback,AnalyticCallback,
    (XtPointer)helpButton);

/* approximationOptions Popup *************************************************/
  approximationPopup = XtVaCreatePopupShell(
			"Options For Approximation",
			transientShellWidgetClass,
			topLevel,
			NULL);
/*  XtAddCallback(approximationPopup,XtNpopdownCallback,PopdownCallback,0); */
  
  AmainForm = XtVaCreateManagedWidget(
		"AmainForm",
		formWidgetClass,
		approximationPopup,
		NULL);
  
  XtAddCallback(approximationPopup,XtNpopupCallback,SetCaretsCallback,
    (XtPointer)AmainForm);
  XtAddCallback(approximationPopup,XtNpopupCallback,CenterWidget,NULL);

  AprecisionLabel = XtVaCreateManagedWidget(
		"AprecisionLabel",
		labelWidgetClass,
		AmainForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Precision:",
		NULL);

  AprecisionText = XtVaCreateManagedWidget(
		"AprecisionText",
		asciiTextWidgetClass,
		AmainForm,
		XtNfromHoriz,AprecisionLabel,
		XtNeditType,XawtextEdit,
		XtNstring,AprecisionBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,True,
		NULL);
/*  XtInstallAccelerators(approximationPopup,AprecisionText);*/
  
  AsolveMethodLabel = XtVaCreateManagedWidget(
		"AsolveMethodLabel",
		labelWidgetClass,
		AmainForm,
		XtNfromVert,AprecisionText,
		XtNlabel,"Linear system solution:",
		NULL);
  
  AiterativeToggle = XtVaCreateManagedWidget(
		"AiterativeToggle",
		toggleWidgetClass,
		AmainForm,
		XtNfromHoriz,AsolveMethodLabel,
		XtNfromVert,AprecisionText,
	        XtNlabel,"iterative",
		XtNradioData,Iterative,
		XtNtranslations,toggle_transtable,
		NULL);

  AdirectToggle = XtVaCreateManagedWidget(
		"AdirectToggle",
		toggleWidgetClass,
		AmainForm,
		XtNlabel,"direct",
		XtNradioData,Direct,
		XtNtranslations,toggle_transtable,
		XtNfromHoriz,AiterativeToggle,
		XtNfromVert,AprecisionText,
		NULL);
  XawToggleChangeRadioGroup(AiterativeToggle,AdirectToggle);
  XtAddCallback(AiterativeToggle,XtNcallback,ASensitivityCallback,0);
  XtAddCallback(AdirectToggle,XtNcallback,ASensitivityCallback,0);
  
  AmaxIterationsLabel = XtVaCreateManagedWidget(
		"AmaxIterationsLabel",
		labelWidgetClass,
		AmainForm,
		XtNlabel,"maximum iterations:",
		XtNfromVert,AdirectToggle,
		NULL);
  
  AmaxIterationsText = XtVaCreateManagedWidget(
		"AmaxIterationsText",
		asciiTextWidgetClass,
		AmainForm,
		XtNfromVert,AdirectToggle,
		XtNfromHoriz,AmaxIterationsLabel,
		XtNeditType,XawtextEdit,
		XtNstring,AmaxIterationsBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);
  
  AdroptoleranceLabel = XtVaCreateManagedWidget(
		"AdroptoleranceLabel",
		labelWidgetClass,
		AmainForm,
		XtNlabel,"droptolerance:",
		XtNfromVert,AmaxIterationsText,
		NULL);

  AdroptoleranceText = XtVaCreateManagedWidget(
		"AdroptoleranceText",
		asciiTextWidgetClass,
		AmainForm,
		XtNfromVert,AmaxIterationsText,
		XtNfromHoriz,AdroptoleranceLabel,
		XtNeditType,XawtextEdit,
		XtNstring,AdroptoleranceBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);

  AphaseNumberLabel = XtVaCreateManagedWidget(
		"AphaseNumberLabel",
		labelWidgetClass,
		AmainForm,
		XtNlabel,"Number of Phases:",
		XtNfromVert,AdroptoleranceText,
		NULL);
  
  AphaseNumberText = XtVaCreateManagedWidget(
		"AphaseNumberText",
		asciiTextWidgetClass,
		AmainForm,
		XtNfromVert,AdroptoleranceText,
		XtNfromHoriz,AphaseNumberLabel,
		XtNeditType,XawtextEdit,
		XtNstring,AphaseNumberBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);

  AvirtPhaseNumberLabel = XtVaCreateManagedWidget(
		"AvirtPhaseNumberLabel",
		labelWidgetClass,
		AmainForm,
		XtNlabel,"virtual Number of Phases: ",
		XtNfromVert,AphaseNumberText,
		NULL);
  
  AvirtPhaseNumberText = XtVaCreateManagedWidget(
		"AvirtPhaseNumberText",
		asciiTextWidgetClass,
		AmainForm,
		XtNfromVert,AphaseNumberText,
		XtNfromHoriz,AvirtPhaseNumberLabel,
		XtNeditType,XawtextEdit,
		XtNstring,AvirtPhaseNumberBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);
  
  AokButton = XtVaCreateManagedWidget(
		"AokButton",
		commandWidgetClass,
		AmainForm,
		XtNfromVert,AvirtPhaseNumberText,
		XtNlabel,"OK",
		XtNaccelerators,ok_button_accelerators,
		NULL);
  XtInstallAccelerators(approximationPopup,AokButton);
  XtAddCallback(AokButton,XtNcallback,ApproximationCallback,
    (XtPointer)okButton);
  
  AcancelButton = XtVaCreateManagedWidget(
		"AcancelButton",
		commandWidgetClass,
		AmainForm,
		XtNfromVert,AvirtPhaseNumberText,
		XtNfromHoriz,AokButton,
		XtNlabel,"Cancel",
		NULL);
  XtAddCallback(AcancelButton,XtNcallback,ApproximationCallback,
    (XtPointer)cancelButton);

  AdefaultButton = XtVaCreateManagedWidget(
		"AdefaultButton",
		commandWidgetClass,
		AmainForm,
		XtNfromVert,AvirtPhaseNumberText,
		XtNfromHoriz,AcancelButton,
		XtNlabel,"Default",
		NULL);
  XtAddCallback(AdefaultButton,XtNcallback,ApproximationCallback,
    (XtPointer)defaultButton);
  
  AhelpButton = XtVaCreateManagedWidget(
		"AhelpButton",
		commandWidgetClass,
		AmainForm,
		XtNfromVert,AvirtPhaseNumberText,
		XtNfromHoriz,AdefaultButton,
		XtNlabel,"Help",
		XtNsensitive,False,
		NULL);
  XtAddCallback(AhelpButton,XtNcallback,ApproximationCallback,
    (XtPointer)helpButton);
  
/******************************************************************************/
/* Simulation Options Popup ***************************************************/
/******************************************************************************/  
  simulationPopup = XtVaCreatePopupShell(
		"Options For Simulation",            
		transientShellWidgetClass,
		topLevel,
		NULL);
  
  SmainForm = XtVaCreateManagedWidget(
		"SmainForm",
		formWidgetClass,
		simulationPopup,
		NULL);
  XtAddCallback(simulationPopup,XtNpopupCallback,SetCaretsCallback,(XtPointer)SmainForm);
  XtAddCallback(simulationPopup,XtNpopupCallback,CenterWidget,NULL);

  SsimtechniqueLabel = XtVaCreateManagedWidget(
		"SsimtechniqueLabel",
		labelWidgetClass,
		SmainForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Simulation technique: ",
		NULL);
  
  SstandardToggle = XtVaCreateManagedWidget(
		"SstandardToggle",
		toggleWidgetClass,
		SmainForm,
		XtNlabel,"standard",
		XtNtranslations,toggle_transtable,
		XtNradioData,Ssimstandard,
		XtNfromVert,SsimtechniqueLabel,
		NULL);
  XtAddCallback(SstandardToggle,XtNcallback,StandardCallback,NULL);

  SfastToggle = XtVaCreateManagedWidget(
		"SfastToggle",
		toggleWidgetClass,
		SmainForm,
		XtNlabel,"fast",
		XtNradioData,Ssimfast,
		XtNtranslations,toggle_transtable,
		XtNfromHoriz,SstandardToggle,
		XtNfromVert,SsimtechniqueLabel,
		NULL);

  XtAddCallback(SfastToggle,XtNcallback,FastCallback,NULL);
 
  XawToggleChangeRadioGroup(SstandardToggle,SfastToggle);
  
  SsteadyStateForm = XtVaCreateManagedWidget(
		"SsteadyStateForm",
		formWidgetClass,
		SmainForm,
		XtNfromVert,SstandardToggle,
		NULL);
  
  SsimulationTypeLabel = XtVaCreateManagedWidget(
		"SsimulationTypeLabel",
		labelWidgetClass,
		SsteadyStateForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Simulation: ",
		/* XtNfromVert,SspectralVarianceToggle, */
		NULL);

  SsequentialToggle = XtVaCreateManagedWidget(
		"SsequentialToggle",
		toggleWidgetClass,
		SsteadyStateForm,
		XtNlabel,"sequential",
		XtNradioData,Sequential,
		XtNtranslations,toggle_transtable,
		XtNfromHoriz,SsimulationTypeLabel,
		NULL);
		
  SdistributedToggle = XtVaCreateManagedWidget(
		"SdistributedToggle",
		toggleWidgetClass,
		SsteadyStateForm,
		XtNlabel,"distributed",
		XtNradioData,Parallel,
		XtNtranslations,toggle_transtable,
		XtNfromHoriz,SsequentialToggle,
		NULL);

  XawToggleChangeRadioGroup(SsequentialToggle,SdistributedToggle);
  
  SdetectTypeLabel = XtVaCreateManagedWidget(
		"SdetectTypeLabel",
		labelWidgetClass,
		SsteadyStateForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Detect initial transient: ",
		XtNfromVert,SsimulationTypeLabel,
		NULL);

  SdetectOnToggle = XtVaCreateManagedWidget(
		"SdetectOnToggle",
		toggleWidgetClass,
		SsteadyStateForm,
		XtNlabel,"ON",
		XtNradioData,Detect_ON,
		XtNtranslations,toggle_transtable,
		XtNfromVert,SsimulationTypeLabel,
		XtNfromHoriz,SdetectTypeLabel,
		NULL);
		
  SdetectOffToggle = XtVaCreateManagedWidget(
		"SdetectOffToggle",
		toggleWidgetClass,
		SsteadyStateForm,
		XtNlabel,"OFF",
		XtNradioData,Detect_OFF,
		XtNtranslations,toggle_transtable,
		XtNfromVert,SsimulationTypeLabel,
		XtNfromHoriz,SdetectOnToggle,
		NULL);
  
  XawToggleChangeRadioGroup(SdetectOnToggle,SdetectOffToggle);
  
  SconfidenceLevelLabel = XtVaCreateManagedWidget(
		"SconfidenceLevelLabel",
		labelWidgetClass,
		SsteadyStateForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Confidence level [%]: ",
		XtNfromVert,SdetectTypeLabel,
		NULL);

  SconfidenceLevelText = XtVaCreateManagedWidget(
		"SconfidenceLevelText",
		asciiTextWidgetClass,
		SsteadyStateForm,
		XtNfromVert,SdetectTypeLabel,
		XtNfromHoriz,SconfidenceLevelLabel,
		XtNeditType,XawtextEdit,
		XtNstring,SconfidenceLevelBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,True,
		NULL);
/*  XtInstallAccelerators(simulationPopup,SconfidenceLevelText);*/
 
  SerrPercentLevelLabel = XtVaCreateManagedWidget(
		"SerrPercentLevelLabel",
		labelWidgetClass,
		SsteadyStateForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"permitted difference at the Confidence level for probability-measures [%]: ",
		XtNfromVert,SconfidenceLevelLabel,
		NULL);

  SerrPercentLevelText = XtVaCreateManagedWidget(
		"SerrPercentLevelText",
		asciiTextWidgetClass,
		SsteadyStateForm,
		XtNfromVert,SconfidenceLevelLabel,
		XtNfromHoriz,SerrPercentLevelLabel,
		XtNeditType,XawtextEdit,
		XtNstring,SerrPercentLevelBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,True,
		NULL);

  SmaxRelativeErrorLabel = XtVaCreateManagedWidget(
		"SmaxRelativeErrorLabel",
		labelWidgetClass,
		SsteadyStateForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Maximum relative error [%]: ",
		XtNfromVert,SerrPercentLevelText,
		NULL);
  
  SmaxRelativeErrorText = XtVaCreateManagedWidget(
		"SmaxRelativeErrorText",
		asciiTextWidgetClass,
		SsteadyStateForm,
		XtNfromVert,SerrPercentLevelText,
		XtNfromHoriz,SmaxRelativeErrorLabel,
		XtNeditType,XawtextEdit,
		XtNstring,SmaxRelativeErrorBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);
  
  SseedValueLabel = XtVaCreateManagedWidget(
		"SseedValueLabel",
		labelWidgetClass,
		SsteadyStateForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Seed value:",
		XtNfromVert,SmaxRelativeErrorText,
		NULL);
  
  SseedValueText = XtVaCreateManagedWidget(
		"SseedValueText",
		asciiTextWidgetClass,
		SsteadyStateForm,
		XtNfromVert,SmaxRelativeErrorText,
		XtNfromHoriz,SseedValueLabel,
		XtNeditType,XawtextEdit,
		XtNstring,SseedValueBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);
 
  SmaxSampleNumberLabel = XtVaCreateManagedWidget(
		"SmaxSampleNumberLabel",
		labelWidgetClass,
		SsteadyStateForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Max. # of samples:",
		XtNfromVert,SseedValueText,
		NULL);
  
  SnoSampleLimitToggle = XtVaCreateManagedWidget(
		"SnoSampleLimitToggle",
		toggleWidgetClass,
		SsteadyStateForm,
		XtNlabel,"no limit",
		XtNfromVert,SseedValueText,
		XtNfromHoriz, SmaxSampleNumberLabel,
		NULL);
  XtAddCallback(SnoSampleLimitToggle,XtNcallback,SampleNumberCallback,NULL);

  SmaxSampleNumberText = XtVaCreateManagedWidget(
		"SmaxSampleNumberText",
		asciiTextWidgetClass,
		SsteadyStateForm,
		XtNfromVert,SseedValueText,
		XtNfromHoriz,SnoSampleLimitToggle,
		XtNeditType,XawtextEdit,
		XtNstring,SmaxSampleNumberBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);

  SminFiringsLabel = XtVaCreateManagedWidget(
		"SminFiringsLabel",
		labelWidgetClass,
		SsteadyStateForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Min. # of firings for each transition:",
		XtNfromVert,SmaxSampleNumberText,
		NULL);
  
  SminFiringsText = XtVaCreateManagedWidget(
		"SminFiringsText",
		asciiTextWidgetClass,
		SsteadyStateForm,
		XtNfromVert,SmaxSampleNumberText,
		XtNfromHoriz,SminFiringsLabel,
		XtNeditType,XawtextEdit,
		XtNstring,SminFiringsBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);

  SmaxModelTimeLabel = XtVaCreateManagedWidget(
		"SmaxModelTimeLabel",
		labelWidgetClass,
		SsteadyStateForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Max model time:",
		XtNfromVert,SminFiringsLabel,
		NULL);
  
  SnoModelTimeLimitToggle = XtVaCreateManagedWidget(
		"SnoModelTimeLimitToggle",
		toggleWidgetClass,
		SsteadyStateForm,
		XtNlabel,"no limit",
		XtNfromHoriz,SmaxModelTimeLabel,
		XtNfromVert,SminFiringsLabel,
		NULL);
  XtAddCallback(SnoModelTimeLimitToggle,XtNcallback,ModelTimeCallback,NULL);

  SmaxModelTimeLimitText = XtVaCreateManagedWidget(
		"SmaxModelTimeLimitText",
		asciiTextWidgetClass,
		SsteadyStateForm,
		XtNfromHoriz,SnoModelTimeLimitToggle,
		XtNfromVert,SminFiringsLabel,
		XtNeditType,XawtextEdit,
		XtNstring,SmaxModelTimeLimitBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);
    
  SmaxCPUTimeLabel = XtVaCreateManagedWidget(
		"SmaxCPUTimeLabel",
		labelWidgetClass,
		SsteadyStateForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Max real time [sec.]:",
		XtNfromVert,SmaxModelTimeLabel,
		NULL);
  
  SnoCPUTimeLimitToggle = XtVaCreateManagedWidget(
		"SnoCPUTimeLimitToggle",
		toggleWidgetClass,
		SsteadyStateForm,
		XtNlabel,"no limit",
		XtNfromHoriz,SmaxCPUTimeLabel,
		XtNfromVert,SmaxModelTimeLabel,
		NULL);
  XtAddCallback(SnoCPUTimeLimitToggle,XtNcallback,CPUTimeCallback,NULL);

  SmaxCPUTimeLimitText = XtVaCreateManagedWidget(
		"SmaxCPUTimeLimitText",
		asciiTextWidgetClass,
		SsteadyStateForm,
		XtNfromHoriz,SnoCPUTimeLimitToggle,
		XtNfromVert,SmaxModelTimeLabel,
		XtNeditType,XawtextEdit,
		XtNstring,SmaxCPUTimeLimitBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);
  SvisualLabel = XtVaCreateManagedWidget(
		"SvisualLabel",
		labelWidgetClass,
		SsteadyStateForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Visualisation: ",
		XtNfromVert,SmaxCPUTimeLabel,
		NULL);

  SvisualOnToggle = XtVaCreateManagedWidget(
		"SvisualOnToggle",
		toggleWidgetClass,
		SsteadyStateForm,
		XtNlabel,"ON",
		XtNradioData,Visual_ON,
		XtNtranslations,toggle_transtable,
		XtNfromVert,SmaxCPUTimeLabel,
		XtNfromHoriz,SvisualLabel,
		NULL);
		
  SvisualOffToggle = XtVaCreateManagedWidget(
		"SvisualOffToggle",
		toggleWidgetClass,
		SsteadyStateForm,
		XtNlabel,"OFF",
		XtNradioData,Visual_OFF,
		XtNtranslations,toggle_transtable,
		XtNfromVert,SmaxCPUTimeLabel,
		XtNfromHoriz,SvisualOnToggle,
		NULL);   

  XawToggleChangeRadioGroup(SvisualOnToggle,SvisualOffToggle);
 
  SstandardForm = XtVaCreateManagedWidget(
		"SstandardForm",
		formWidgetClass,
		SmainForm,
		XtNfromVert,SsteadyStateForm,
		NULL);
  
  SfastForm = XtVaCreateManagedWidget(
		"SfastForm",
		formWidgetClass,
		SmainForm,
		XtNfromHoriz,SstandardForm,
		XtNfromVert, SsteadyStateForm,
		NULL);
    
  SstatisticalMethodLabel = XtVaCreateManagedWidget(
		"SstatisticalMethodLabel",
		labelWidgetClass,
		SstandardForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Method for statistical analysis: ",
		NULL);
  
  SspectralVarianceToggle = XtVaCreateManagedWidget(
		"SspectralVarianceToggle",
		toggleWidgetClass,
		SstandardForm,
		XtNlabel,"spectral variance",
		XtNtranslations,toggle_transtable,
		XtNradioData,SpectralVariance,
		XtNfromVert,SstatisticalMethodLabel,
		NULL);
  
  ScontrolVarialesToggle = XtVaCreateManagedWidget(
		"ScontrolVarialesToggle",
		toggleWidgetClass,
		SstandardForm,
		XtNlabel,"control variables",
		XtNradioData,ControlVariales,
		XtNtranslations,toggle_transtable,
		XtNfromHoriz,SspectralVarianceToggle,
		XtNfromVert,SstatisticalMethodLabel,
		NULL);
  
  XawToggleChangeRadioGroup(ScontrolVarialesToggle,SspectralVarianceToggle);

  SoptThreshLabel = XtVaCreateManagedWidget(
		"SoptThreshLabel",
		labelWidgetClass,
		SfastForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"Initial # thresholds",
		NULL);

  SoptThreshText = XtVaCreateManagedWidget(
		"SoptThreshText",
		asciiTextWidgetClass,
		SfastForm,
		XtNfromHoriz,SoptThreshLabel,
		XtNeditType,XawtextEdit,
		XtNstring,SoptThreshBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);

  SCPUTimeRunLabel = XtVaCreateManagedWidget(
		"SCPUTimeRunLabel",
		labelWidgetClass,
		SfastForm,
		XtNjustify,XtJustifyLeft,
		XtNlabel,"CPU - time pilot run [sec.]",
		XtNfromVert,SoptThreshLabel,
		NULL);

  SCPUTimeRunText = XtVaCreateManagedWidget(
		"SCPUTimeRunText",
		asciiTextWidgetClass,
		SfastForm,
		XtNfromHoriz,SCPUTimeRunLabel,
		XtNfromVert,SoptThreshLabel,
		XtNeditType,XawtextEdit,
		XtNstring,SCPUTimeRunBuffer,
		XtNlength,VAL_BUF,
		XtNtranslations,text_transtable,
		XtNdisplayCaret,False,
		NULL);

  SokButton = XtVaCreateManagedWidget(
		"SokButton",
		commandWidgetClass,
		SmainForm,
		XtNlabel,"OK",
		XtNfromVert,SfastForm,
		XtNaccelerators,ok_button_accelerators,
		NULL);
  XtInstallAccelerators(simulationPopup,SokButton);
  XtAddCallback(SokButton,XtNcallback,SimulationCallback,(XtPointer)okButton);
  
  ScancelButton = XtVaCreateManagedWidget(
		"ScancelButton",
		commandWidgetClass,
		SmainForm,
		XtNlabel,"Cancel",
		XtNfromVert,SfastForm,
		XtNfromHoriz,SokButton,
		NULL);
  XtAddCallback(ScancelButton,XtNcallback,SimulationCallback,
    (XtPointer)cancelButton);

  SdefaultButton = XtVaCreateManagedWidget(
		"SdefaultButton",
		commandWidgetClass,
		SmainForm,
		XtNlabel,"Default",
		XtNfromHoriz,ScancelButton,
		XtNfromVert,SfastForm,
		NULL);
  XtAddCallback(SdefaultButton,XtNcallback,SimulationCallback,
    (XtPointer)defaultButton);
  
  ShelpButton = XtVaCreateManagedWidget(
		"ShelpButton",
		commandWidgetClass,
		SmainForm,
		XtNlabel,"Help",
		XtNsensitive,False,
		XtNfromHoriz,SdefaultButton,
		XtNfromVert,SfastForm,
		NULL);
  XtAddCallback(ShelpButton,XtNcallback,SimulationCallback,
    (XtPointer)helpButton);



  TransientPopupLayout();
  TransientSimulationPopupLayout();
  AnalyticPopupLayout();
  ApproximationPopupLayout();
  SimulationPopupLayout();
  DTAnalyticPopupLayout(); /*new*/
  DTTransientPopupLayout(); /*new*/
  
  initAnalyticSolutionPopup(&analyticDefaults);
  initApproximationPopup(&approximationDefaults);
  initSimulationPopup(&simulationDefaults);
  initDTAnalyticSolutionPopup(&dtspnanalyticDefaults);  /*new*/
  initDTTransientSolutionPopup(&dtspntransientDefaults);  /*new*/
}

/*******************************************************************************
Name:
	TransientPopupLayout
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
aufgerufen aus:
	initOptionPopups
--------------------------------------------------------------------------------
Zweck:
	Noch n bissl am Layout feilen..
*******************************************************************************/
static void TransientPopupLayout()
{  
  Dimension width,height,mfwidth; int dd; /* default-distance, main-form-width.. */

  XtVaSetValues(transientPopup,XtNmappedWhenManaged,False,NULL);
  XtRealizeWidget(transientPopup);

  XtVaGetValues(TmainForm,XtNdefaultDistance,&dd,XtNwidth,&mfwidth,NULL);
   
  XtVaSetValues(TtransientTimeText,XtNhorizDistance,
   mfwidth-WidgetWidth(TtransientTimeLabel)-WidgetWidth(TtransientTimeText)-3*dd,NULL);

  XtVaSetValues(TprecisionText,XtNhorizDistance,
   mfwidth-WidgetWidth(TprecisionLabel)-WidgetWidth(TprecisionText)-3*dd,NULL);

  XtVaSetValues(TdurationToggle,XtNhorizDistance,
    mfwidth-WidgetWidth(TdurationToggle)-
	  WidgetWidth(TpointToggle)-WidgetWidth(ToutputLabel)-6*dd,NULL);

  XtVaSetValues(Tstepsize2Text,XtNhorizDistance,
   mfwidth-WidgetWidth(Tstepsize2Label)-WidgetWidth(Tstepsize2Text)-3*dd,NULL);

  XtVaSetValues(ToldToggle,XtNhorizDistance,
    mfwidth-WidgetWidth(ToldToggle)-
	  WidgetWidth(TnewToggle)-WidgetWidth(TvariantLabel)-6*dd,NULL);

  XtVaSetValues(TgenprecText,XtNhorizDistance,
   mfwidth-WidgetWidth(TgenprecLabel)-WidgetWidth(TgenprecText)-3*dd,NULL);

  XtVaSetValues(TclusterText,XtNhorizDistance,
   mfwidth-WidgetWidth(TclusterLabel)-WidgetWidth(TclusterText)-3*dd,NULL);

  width=CalculateButtonWidth(TmainForm,4);
  XtVaGetValues(TokButton,XtNheight,&height,NULL);
  XtResizeWidget(TokButton,width,height,(Dimension)2);
  XtResizeWidget(TcancelButton,width,height,(Dimension)1);
  XtResizeWidget(TdefaultButton,width,height,(Dimension)1);
  XtResizeWidget(ThelpButton,width,height,(Dimension)1);
  XawFormDoLayout(TmainForm,True);

  XtUnrealizeWidget(transientPopup);
  XtVaSetValues(transientPopup,XtNmappedWhenManaged,True,NULL);
}


/*******************************************************************************
Name:
	TransientSimulationPopupLayout
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
aufgerufen aus:
	initOptionPopups
--------------------------------------------------------------------------------
Zweck:
	Noch n bissl am Layout feilen..
*******************************************************************************/
static void TransientSimulationPopupLayout()
{    
  Dimension width,height,mfwidth,sfwidth,ffwidth;
  int dd;
  
  XtVaSetValues(transientSimulationPopup,XtNmappedWhenManaged,False,NULL);
  XtRealizeWidget(transientSimulationPopup);
	
  XtVaGetValues(TSmainForm,XtNdefaultDistance,&dd,XtNwidth,&mfwidth,NULL);
  XtVaSetValues(TStransimForm,XtNwidth,mfwidth,NULL);
  
  XtVaSetValues(TSmaxRelativeErrorText,XtNhorizDistance,mfwidth
		-WidgetWidth(TSmaxRelativeErrorText)-WidgetWidth(TSmaxRelativeErrorLabel)-5*dd,NULL);
  XtVaSetValues(TStransinstimeText,XtNhorizDistance,mfwidth
		-WidgetWidth(TStransinstimeText)-WidgetWidth(TStransinstimeLabel)-5*dd,NULL);
  XtVaSetValues(TSnumsampointsText,XtNhorizDistance,mfwidth
		-WidgetWidth(TSnumsampointsText)-WidgetWidth(TSnumsampointsLabel)-5*dd,NULL);
  XtVaSetValues(TSconfidenceLevelText,XtNhorizDistance,
    mfwidth-WidgetWidth(TSconfidenceLevelText)-WidgetWidth(TSconfidenceLevelLabel)-5*dd,NULL);
  XtVaSetValues(TSseedValueText,XtNhorizDistance,mfwidth-WidgetWidth(TSseedValueText)-
          WidgetWidth(TSseedValueLabel)-5*dd,NULL);
  XtVaSetValues(TSsequentialToggle,XtNhorizDistance,mfwidth-WidgetWidth(TSsequentialToggle)-
	  WidgetWidth(TSdistributedToggle)-WidgetWidth(TSsimulationTypeLabel)-7*dd,NULL);
  XtVaSetValues(TSruleONToggle,XtNhorizDistance,mfwidth-WidgetWidth(TSruleONToggle)-
	  WidgetWidth(TSruleOFFToggle)-WidgetWidth(TSruleTypeLabel)-7*dd,NULL);
   XtVaSetValues(TSnoCPUTimeLimitToggle,XtNhorizDistance,mfwidth-WidgetWidth(TSmaxCPUTimeLabel)-
	  WidgetWidth(TSnoCPUTimeLimitToggle)-WidgetWidth(TSmaxCPUTimeLimitText)-7*dd,NULL);

  width=CalculateButtonWidth(TSmainForm,4);
  XtResizeWidget(TSokButton,width,height,(Dimension)1);
  XtResizeWidget(TScancelButton,width,height,(Dimension)1);
  XtResizeWidget(TSdefaultButton,width,height,(Dimension)1);
  XtResizeWidget(TShelpButton,width,height,(Dimension)1);

  XawFormDoLayout(TSmainForm,True);
  XtUnrealizeWidget(transientSimulationPopup);
  XtVaSetValues(transientSimulationPopup,XtNmappedWhenManaged,True,NULL);
}


/*******************************************************************************
Name:
	AnalyticPopupLayout
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
aufgerufen aus:
	initOptionPopups
--------------------------------------------------------------------------------
Zweck:
	Noch n bissl am Layout feilen..
*******************************************************************************/
static void AnalyticPopupLayout()
{  
  Dimension width,height,mfwidth; int dd; /* default-distance, main-form-width.. */
  
  XtVaSetValues(analyticPopup,XtNmappedWhenManaged,False,NULL);
  XtRealizeWidget(analyticPopup);

  
  XtVaGetValues(ASmainForm,XtNdefaultDistance,&dd,XtNwidth,&mfwidth,NULL);

  XtVaSetValues(ASprecisionText,XtNhorizDistance,
    mfwidth-WidgetWidth(ASprecisionText)-WidgetWidth(ASprecisionLabel)-2*dd,NULL);
  XtVaSetValues(ASiterativeToggle,XtNhorizDistance,
    mfwidth-WidgetWidth(ASsolveMethodLabel)-WidgetWidth(ASiterativeToggle)
	-WidgetWidth(AdirectToggle)-4*dd,NULL);
  XtVaSetValues(ASmaxIterationsText,XtNhorizDistance,
   mfwidth-WidgetWidth(ASmaxIterationsLabel)-WidgetWidth(ASmaxIterationsText)-2*dd,NULL);

  width=CalculateButtonWidth(ASmainForm,4);
  XtVaGetValues(ASokButton,XtNheight,&height,NULL);
  XtResizeWidget(ASokButton,width,height,(Dimension)2);
  XtResizeWidget(AScancelButton,width,height,(Dimension)1);
  XtResizeWidget(ASdefaultButton,width,height,(Dimension)1);
  XtResizeWidget(AShelpButton,width,height,(Dimension)1);
  XtResizeWidget(ASmaxIterationsText,100,21,(Dimension)1);
  XawFormDoLayout(ASmainForm,True);

  XtUnrealizeWidget(analyticPopup);
  XtVaSetValues(analyticPopup,XtNmappedWhenManaged,True,NULL);
}

/*new*/

/*******************************************************************************
Name:
	DTAnalyticPopupLayout
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
aufgerufen aus:
	initOptionPopups
--------------------------------------------------------------------------------
Zweck:
	Noch n bissl am Layout feilen..
*******************************************************************************/
static void DTAnalyticPopupLayout()
{  
  Dimension width,height,mfwidth; int dd; /* default-distance, main-form-width.. */
  
  XtVaSetValues(dtanalyticPopup,XtNmappedWhenManaged,False,NULL);
  XtRealizeWidget(dtanalyticPopup);

  
  XtVaGetValues(DTASmainForm,XtNdefaultDistance,&dd,XtNwidth,&mfwidth,NULL);

  XtVaSetValues(DTASprecisionText,XtNhorizDistance,
   mfwidth-WidgetWidth(DTASprecisionLabel)-WidgetWidth(DTASprecisionText)-2*dd,NULL);


  XtVaSetValues(DTASiterativeToggle,XtNhorizDistance,
    mfwidth-WidgetWidth(DTASsolveMethodLabel)-WidgetWidth(DTASiterativeToggle)
	-WidgetWidth(DTASdirectToggle)-4*dd,NULL);

  XtVaSetValues(DTASmaxIterationsText,XtNhorizDistance,
  mfwidth-WidgetWidth(DTASmaxIterationsLabel)-WidgetWidth(DTASmaxIterationsText)-3*dd,NULL);

  XtVaSetValues(DTASscalingText,XtNhorizDistance,
  mfwidth-WidgetWidth(DTASscalingLabel)-WidgetWidth(DTASscalingText)-3*dd,NULL);

 /*  XtVaSetValues(DTASembyesToggle,XtNhorizDistance,mfwidth-WidgetWidth(DTASembeddedKindLabel) */
/* 		-WidgetWidth(DTASembyesToggle)-WidgetWidth(DTASembnoToggle)-9*dd,NULL); */
  
  XtVaGetValues(DTASmaxIterationsText,XtNheight,&height,XtNwidth,&width,NULL);
  XtResizeWidget(DTASmaxIterationsText,(Dimension)(width+2),
		 (Dimension)(height+2),(Dimension)1);

  XtVaGetValues(DTASscalingText,XtNheight,&height,XtNwidth,&width,NULL);
  XtResizeWidget(DTASscalingText,(Dimension)(width+2),
		 (Dimension)(height+2),(Dimension)1);

  XtVaGetValues(dtanalyticPopup,XtNheight,&height,XtNwidth,&width,NULL);
  XtResizeWidget(dtanalyticPopup,(Dimension)(width+20),
		 (Dimension)(height+20),(Dimension)1);

  width=CalculateButtonWidth(DTASmainForm,4);
  XtVaGetValues(DTASokButton,XtNheight,&height,NULL);
  XtResizeWidget(DTASokButton,width,height,(Dimension)2);
  XtResizeWidget(DTAScancelButton,width,height,(Dimension)1);
  XtResizeWidget(DTASdefaultButton,width,height,(Dimension)1);
  XtResizeWidget(DTAShelpButton,width,height,(Dimension)1);
  XawFormDoLayout(DTASmainForm,True);

  XtUnrealizeWidget(dtanalyticPopup);
  XtVaSetValues(dtanalyticPopup,XtNmappedWhenManaged,True,NULL);

}
/*new*/

/*******************************************************************************
Name:
	DTTransientPopupLayout
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
aufgerufen aus:
	initOptionPopups
--------------------------------------------------------------------------------
Zweck:
	Noch n bissl am Layout feilen..
*******************************************************************************/
static void DTTransientPopupLayout()
{  
  Dimension width,height,mfwidth,mfheight; int dd; /* default-distance, main-form-width.. */
  
  XtVaSetValues(dttransientPopup,XtNmappedWhenManaged,False,NULL);
  XtRealizeWidget(dttransientPopup);

  XtVaGetValues(DTTSmainForm,XtNdefaultDistance,&dd,XtNwidth,&mfwidth,XtNheight,&mfheight,NULL);

  XtVaSetValues(DTTSprecisionText,XtNhorizDistance,
   mfwidth-WidgetWidth(DTTSprecisionLabel)-WidgetWidth(DTTSprecisionText)-2*dd,NULL);

  XtVaSetValues(DTTStimeStepsText,XtNhorizDistance,
   mfwidth-WidgetWidth(DTTStimeStepsLabel)-WidgetWidth(DTTStimeStepsText)-2*dd,NULL);

  XtVaSetValues(DTTSstepsizeText,XtNhorizDistance,
   mfwidth-WidgetWidth(DTTSstepsizeLabel)-WidgetWidth(DTTSstepsizeText)-2*dd,NULL);

  XtVaSetValues(DTTSfirststepText,XtNhorizDistance,
   mfwidth-WidgetWidth(DTTSfirststepLabel)-WidgetWidth(DTTSfirststepText)-2*dd,NULL);

  XtVaSetValues(DTTSdurationToggle,XtNhorizDistance,
    mfwidth-WidgetWidth(DTTSoutputLabel)-WidgetWidth(DTTSpointToggle)-WidgetWidth(DTTSdurationToggle)-4*dd,NULL);

  XtVaSetValues(DTTSscalingText,XtNhorizDistance,
   mfwidth-WidgetWidth(DTTSscalingLabel)-WidgetWidth(DTTSscalingText)-2*dd,NULL);

  XtVaGetValues(DTTStimeStepsText,XtNheight,&height,XtNwidth,&width,NULL);
  XtResizeWidget(DTTStimeStepsText,(Dimension)(width+20),
		 (Dimension)(height+2),(Dimension)1);

  XtVaGetValues(DTTSprecisionText,XtNheight,&height,XtNwidth,&width,NULL);
  XtResizeWidget(DTTSprecisionText,(Dimension)(width+20),
		 (Dimension)(height+2),(Dimension)1);

  XtVaGetValues(DTTSstepsizeText,XtNheight,&height,XtNwidth,&width,NULL);
  XtResizeWidget(DTTSstepsizeText,(Dimension)(width+20),
		 (Dimension)(height+2),(Dimension)1);

  XtVaGetValues(DTTSfirststepText,XtNheight,&height,XtNwidth,&width,NULL);
  XtResizeWidget(DTTSfirststepText,(Dimension)(width+20),
		 (Dimension)(height+2),(Dimension)1);

  XtVaGetValues(DTTSscalingText,XtNheight,&height,XtNwidth,&width,NULL);
  XtResizeWidget(DTTSscalingText,(Dimension)(width+20),
		 (Dimension)(height+2),(Dimension)1);

  XtVaGetValues(dttransientPopup,XtNheight,&height,XtNwidth,&width,NULL);
  XtResizeWidget(dttransientPopup,(Dimension)(width+100),
		 (Dimension)(height+25),(Dimension)1);

  width=(Dimension)CalculateButtonWidth(DTTSmainForm,4);
  XtVaGetValues(DTTSokButton,XtNheight,&height,NULL);
  XtResizeWidget(DTTSokButton,width,height,(Dimension)2);
  XtResizeWidget(DTTScancelButton,width,height,(Dimension)1);
  XtResizeWidget(DTTSdefaultButton,width,height,(Dimension)1);
  XtResizeWidget(DTTShelpButton,width,height,(Dimension)1);

  XtVaSetValues(DTTSokButton,XtNvertDistance,(Dimension)10,NULL);
  XtVaSetValues(DTTScancelButton,XtNvertDistance,(Dimension)10,NULL);
  XtVaSetValues(DTTSdefaultButton,XtNvertDistance,(Dimension)10,NULL);
  XtVaSetValues(DTTShelpButton,XtNvertDistance,(Dimension)10,NULL);

  XawFormDoLayout(DTTSmainForm,True);

  XtUnrealizeWidget(dttransientPopup);
  XtVaSetValues(dttransientPopup,XtNmappedWhenManaged,True,NULL);
}

/*******************************************************************************
Name:
	ApproximationPopupLayout
--------------------------------------------------------------------------------
Funktionstyp:
	private Funktion
--------------------------------------------------------------------------------
aufgerufen aus:
	initOptionPopups
--------------------------------------------------------------------------------
Zweck:
	Noch n bissl am Layout feilen..
*******************************************************************************/
static void ApproximationPopupLayout()
{  
  Dimension width,height,mfwidth; int dd;
  
  XtVaSetValues(approximationPopup,XtNmappedWhenManaged,False,NULL);
  XtRealizeWidget(approximationPopup);

  XtVaGetValues(AmainForm,XtNdefaultDistance,&dd,XtNwidth,&mfwidth,NULL);

  XtVaSetValues(AvirtPhaseNumberText,XtNhorizDistance,
    mfwidth-WidgetWidth(AvirtPhaseNumberText)-WidgetWidth(AvirtPhaseNumberLabel)-2*dd,NULL);
  XtVaSetValues(AphaseNumberText,XtNhorizDistance,
    mfwidth-WidgetWidth(AphaseNumberText)-WidgetWidth(AphaseNumberLabel)-2*dd,NULL);
  XtVaSetValues(AprecisionText,XtNhorizDistance,
    mfwidth-WidgetWidth(AprecisionText)-WidgetWidth(AprecisionLabel)-2*dd,NULL);
  XtVaSetValues(AiterativeToggle,XtNhorizDistance,
    mfwidth-WidgetWidth(AsolveMethodLabel)-WidgetWidth(AiterativeToggle)
	-WidgetWidth(AdirectToggle)-4*dd,NULL);
  XtVaSetValues(AmaxIterationsText,XtNhorizDistance,
   mfwidth-WidgetWidth(AmaxIterationsLabel)-WidgetWidth(AmaxIterationsText)-2*dd,NULL);
  XtVaSetValues(AdroptoleranceText,XtNhorizDistance,
   mfwidth-WidgetWidth(AdroptoleranceLabel)-WidgetWidth(AdroptoleranceText)-2*dd,NULL);

  width=CalculateButtonWidth(AmainForm,4);
  XtVaGetValues(AokButton,XtNheight,&height,NULL);
  XtResizeWidget(AokButton,width,height,(Dimension)2);
  XtResizeWidget(AcancelButton,width,height,(Dimension)1);
  XtResizeWidget(AdefaultButton,width,height,(Dimension)1);
  XtResizeWidget(AhelpButton,width,height,(Dimension)1);
  XtResizeWidget(AprecisionText,100,21,(Dimension)1); 
  XtResizeWidget(AmaxIterationsText,100,21,(Dimension)1); 
  XtResizeWidget(AdroptoleranceText,100,21,(Dimension)1); 
  XtResizeWidget(AphaseNumberText,100,21,(Dimension)1); 
  XtResizeWidget(AvirtPhaseNumberText,100,22,(Dimension)1); 
  XawFormDoLayout(ASmainForm,True);

  XtUnrealizeWidget(approximationPopup);
  XtVaSetValues(approximationPopup,XtNmappedWhenManaged,True,NULL);
}

/*******************************************************************************
Name:
	SimulationPopupLayout
--------------------------------------------------------------------------------
Funktionstyp:
	private Function
--------------------------------------------------------------------------------
aufgerufen aus:
	initOptionPopups
--------------------------------------------------------------------------------
Zweck:
	Noch n bissl am Layout feilen..
*******************************************************************************/
static void SimulationPopupLayout()
{  
  Dimension width,height,mfwidth,sfwidth,ffwidth;
  int dd;
  
  XtVaSetValues(simulationPopup,XtNmappedWhenManaged,False,NULL);
  XtRealizeWidget(simulationPopup);
	
  XtVaGetValues(SokButton,XtNheight,&height,NULL);
  width=CalculateButtonWidth(SmainForm,2);
  XtResizeWidget(SstandardToggle,width,height,(Dimension)1);
  XtResizeWidget(SfastToggle,width,height,(Dimension)1);
 
  XtVaGetValues(SmainForm,XtNdefaultDistance,&dd,XtNwidth,&mfwidth,NULL);
  XtVaSetValues(SsteadyStateForm,XtNwidth,mfwidth,NULL);
  
  XtVaSetValues(SmaxRelativeErrorText,XtNhorizDistance,mfwidth
		-WidgetWidth(SmaxRelativeErrorText)-WidgetWidth(SmaxRelativeErrorLabel)-5*dd,NULL);
  XtVaSetValues(SconfidenceLevelText,XtNhorizDistance,
    mfwidth-WidgetWidth(SconfidenceLevelText)-WidgetWidth(SconfidenceLevelLabel)-5*dd,NULL);
  XtVaSetValues(SerrPercentLevelText,XtNhorizDistance,
    mfwidth-WidgetWidth(SerrPercentLevelText)-WidgetWidth(SerrPercentLevelLabel)-5*dd,NULL);
  XtVaSetValues(SseedValueText,XtNhorizDistance,mfwidth-WidgetWidth(SseedValueText)-
          WidgetWidth(SseedValueLabel)-5*dd,NULL);
  XtVaSetValues(SsequentialToggle,XtNhorizDistance,mfwidth-WidgetWidth(SsequentialToggle)-
	  WidgetWidth(SdistributedToggle)-WidgetWidth(SsimulationTypeLabel)-7*dd,NULL);
  XtVaSetValues(SdetectOnToggle,XtNhorizDistance,mfwidth-WidgetWidth(SdetectOnToggle)-
	  WidgetWidth(SdetectOffToggle)-WidgetWidth(SdetectTypeLabel)-7*dd,NULL);
  XtVaSetValues(SnoSampleLimitToggle,XtNhorizDistance,mfwidth-WidgetWidth(SmaxSampleNumberLabel)-
	  WidgetWidth(SnoSampleLimitToggle)-WidgetWidth(SmaxSampleNumberText)-7*dd,NULL);
  XtVaSetValues(SminFiringsText,XtNhorizDistance,mfwidth-WidgetWidth(SminFiringsText)-
          WidgetWidth(SminFiringsLabel)-5*dd,NULL);
  XtVaSetValues(SnoModelTimeLimitToggle,XtNhorizDistance,mfwidth-WidgetWidth(SmaxModelTimeLabel)-
	  WidgetWidth(SnoModelTimeLimitToggle)-WidgetWidth(SmaxModelTimeLimitText)-7*dd,NULL);
  XtVaSetValues(SnoCPUTimeLimitToggle,XtNhorizDistance,mfwidth-WidgetWidth(SmaxCPUTimeLabel)-
	  WidgetWidth(SnoCPUTimeLimitToggle)-WidgetWidth(SmaxCPUTimeLimitText)-7*dd,NULL);
  XtVaSetValues(SvisualOnToggle,XtNhorizDistance,mfwidth-WidgetWidth(SvisualOnToggle)-
	  WidgetWidth(SvisualOffToggle)-WidgetWidth(SvisualLabel)-7*dd,NULL);

  XtVaSetValues(SfastForm,XtNhorizDistance,mfwidth-WidgetWidth(SfastForm)-
          WidgetWidth(SstandardForm)-3*dd,NULL);
  XtVaGetValues(SfastForm,XtNwidth,&ffwidth,NULL);
  XtVaSetValues(SoptThreshText,XtNhorizDistance,ffwidth-WidgetWidth(SoptThreshLabel)-
          WidgetWidth(SoptThreshText)-3*dd,NULL);
  XtVaSetValues(SCPUTimeRunText,XtNhorizDistance,ffwidth-WidgetWidth(SCPUTimeRunLabel)-
          WidgetWidth(SCPUTimeRunText)-3*dd,NULL);

  width=CalculateButtonWidth(SmainForm,4);
  XtResizeWidget(SokButton,width,height,(Dimension)1);
  XtResizeWidget(ScancelButton,width,height,(Dimension)1);
  XtResizeWidget(SdefaultButton,width,height,(Dimension)1);
  XtResizeWidget(ShelpButton,width,height,(Dimension)1);

  XawFormDoLayout(SmainForm,True);
  XtUnrealizeWidget(simulationPopup);
  XtVaSetValues(simulationPopup,XtNmappedWhenManaged,True,NULL);
}


/*******************************************************************************
Name:
	TSensitivityCallback
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
registriert mit:
	TdurationToggle,TpointToggle
--------------------------------------------------------------------------------
Zweck:
	Entsprechend dem Status des Battens das Widget Tstepsize2Label ... en/disablen
*******************************************************************************/
static void TSensitivityCallback(w,cl_data,call_data)
Widget w;
caddr_t cl_data,call_data;
{
  char buffer[VAL_BUF];

  if (w==TdurationToggle)
  {
    long current=(long)XawToggleGetCurrent(TpointToggle);
    XtSetSensitive(Tstepsize2Label,current==Duration);
    XtSetSensitive(Tstepsize2Text,current==Duration);
  }
  else
  {
    long current=(long)XawToggleGetCurrent(DTTSpointToggle);
     if (current!=Duration)
     {
       sprintf(buffer,"%ld",dtspntransientDefaults.first_step);
       setTextForWidget(DTTSfirststepText,buffer);
     }
    XtSetSensitive(DTTSstepsizeLabel,current==Duration);
    XtSetSensitive(DTTSstepsizeText,current==Duration);
    XtSetSensitive(DTTSfirststepLabel,current==Duration);
    XtSetSensitive(DTTSfirststepText,current==Duration);
  }
}
/*******************************************************************************
Name:
	TSSensitivityCallback
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
registriert mit:
	ToldToggle,TnewToggle
--------------------------------------------------------------------------------
Zweck:
	Entsprechend dem Status des Battens das Widget TclusterLabel ... en/disablen
*******************************************************************************/
static void TSSensitivityCallback(w,cl_data,call_data)
Widget w;
caddr_t cl_data,call_data;
{
  long current=(long)XawToggleGetCurrent(TnewToggle);
  XtSetSensitive(TclusterLabel,current==2);
  XtSetSensitive(TclusterText,current==2);
}

/*******************************************************************************
Name:
	StandardCallback
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
registriert mit:
	SstandardToggle
--------------------------------------------------------------------------------
Zweck:
	Status dieses Toggles toggeln und gleichzeitig SfastForm
	 en- bzw. disabeln
*******************************************************************************/
static void StandardCallback(w,cl_data,call_data)
Widget w;
caddr_t cl_data,call_data;
{
  Boolean curr_state;

  XtVaGetValues(SstandardToggle,XtNstate,&curr_state,NULL);
  XtSetSensitive(SfastForm,!curr_state);
  XtSetSensitive(SerrPercentLevelLabel, curr_state);
  XtSetSensitive(SerrPercentLevelText, curr_state);
  XtSetSensitive(SvisualLabel, curr_state);
  XtSetSensitive(SvisualOnToggle, curr_state);
  XtSetSensitive(SvisualOffToggle, curr_state);

}

/*******************************************************************************
Name:
	FastCallback
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
registriert mit:
	SfastToggle
--------------------------------------------------------------------------------
Zweck:
	Status dieses Toggles toggeln und gleichzeitig SstandardForm
	en- bzw. disabeln
*******************************************************************************/
static void FastCallback(w,cl_data,call_data)
Widget w;
caddr_t cl_data,call_data;
{
  Boolean curr_state;

  XtVaGetValues(SfastToggle,XtNstate,&curr_state,NULL);
  /*XtSetSensitive(S*/
  XtSetSensitive(SstandardForm,!curr_state);
}


/*******************************************************************************
Name:
	SampleNumberCallback
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
registriert mit:
	SnoSampleLimitToggle
--------------------------------------------------------------------------------
Zweck:
	Status dieses Toggles toggeln und gleichzeitig SmaxSampleNumber-
	Text en- bzw. disabeln
*******************************************************************************/
static void SampleNumberCallback(w,cl_data,call_data)
Widget w;
caddr_t cl_data,call_data;
{
  Boolean curr_state;

  XtVaGetValues(SnoSampleLimitToggle,XtNstate,&curr_state,NULL);
  XtSetSensitive(SmaxSampleNumberText,!curr_state);
}



/*******************************************************************************
Name:
	ModelTimeCallback
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
registriert mit:
	SnoModelTimeLimitToggle
--------------------------------------------------------------------------------
Zweck:
	Status dieses Toggles toggeln und gleichzeitig SmaxSampleNumber-
	Text en- bzw. disabeln
*******************************************************************************/
static void ModelTimeCallback(w,cl_data,call_data)
Widget w;
caddr_t cl_data,call_data;
{
  Boolean curr_state;

  XtVaGetValues(SnoModelTimeLimitToggle,XtNstate,&curr_state,NULL);
  XtSetSensitive(SmaxModelTimeLimitText,!curr_state);
}

/*******************************************************************************
Name:
	CPUTimeCallback
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
registriert mit:
	SnoCPUTimeLimitToggle
--------------------------------------------------------------------------------
Zweck:
	Status dieses Toggles toggeln und gleichzeitig SmaxSampleNumber-
	Text en- bzw. disabeln
*******************************************************************************/
static void CPUTimeCallback(w,cl_data,call_data)
Widget w;
caddr_t cl_data,call_data;
{
  Boolean curr_state;

  XtVaGetValues(SnoCPUTimeLimitToggle,XtNstate,&curr_state,NULL);
  XtSetSensitive(SmaxCPUTimeLimitText,!curr_state);
}
/*******************************************************************************
Name:
	TS_CPUTimeCallback
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
registriert mit:
	TSnoCPUTimeLimitToggle
--------------------------------------------------------------------------------
Zweck:
	Status dieses Toggles toggeln und gleichzeitig SmaxSampleNumber-
	Text en- bzw. disabeln
*******************************************************************************/
static void TS_CPUTimeCallback(w,cl_data,call_data)
Widget w;
caddr_t cl_data,call_data;
{
  Boolean curr_state;

  XtVaGetValues(TSnoCPUTimeLimitToggle,XtNstate,&curr_state,NULL);
  XtSetSensitive(TSmaxCPUTimeLimitText,!curr_state);
}


/*******************************************************************************
Name:
	ASSensitivityCallback
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
registriert mit:
	ASiterativeToggle,ASdirectToggle,DTASiterativeToggle,DTASdirectToggle
--------------------------------------------------------------------------------
Zweck:
	Entsprechend dem Status des Battens das Widget ASmaxIterations.. en/disablen
*******************************************************************************/
static void ASSensitivityCallback(w,cl_data,call_data)
Widget w;
caddr_t cl_data,call_data;
{if (w==ASiterativeToggle)
   {
     long current=(long)XawToggleGetCurrent(ASiterativeToggle);
     XtSetSensitive(ASmaxIterationsLabel,current==Iterative);
     XtSetSensitive(ASmaxIterationsText,current==Iterative);
   }
 else
   {
     long current=(long)XawToggleGetCurrent(DTASiterativeToggle);
     XtSetSensitive(DTASmaxIterationsLabel,current==Iterative);
     XtSetSensitive(DTASmaxIterationsText,current==Iterative);
   }

}
/*******************************************************************************
Name:
	ASensitivityCallback
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
registriert mit:
	AiterativeToggle,AdirectToggle
--------------------------------------------------------------------------------
Zweck:
	Entsprechend dem Status des Battens das Widget ASmaxIterations.. en/disablen
*******************************************************************************/
static void ASensitivityCallback(w,cl_data,call_data)
Widget w;
caddr_t cl_data,call_data;
{
  long current=(long)XawToggleGetCurrent(AiterativeToggle);
  XtSetSensitive(AmaxIterationsLabel,current==Iterative);
  XtSetSensitive(AmaxIterationsText,current==Iterative);
}


/*******************************************************************************
Name:
	initTransientSolutionPopup
--------------------------------------------------------------------------------
Funktionstyp:
	public Funktion
--------------------------------------------------------------------------------
aufgerufen aus:
	evalMenu.c
	ButtonCallback
--------------------------------------------------------------------------------
Zweck:
	Uebernehmen der Daten aus einer TransientOptions-Struktur in das
	analytic Options Popup (Gegenstueck ist evalTransientSolutionPopup)
*******************************************************************************/
static void initTransientSolutionPopup(op)
TransientOptions *op;
{
  char buffer[VAL_BUF];
  int zwerg;
  
  if(op->old_case_iii==0)
    zwerg = 2;
  else
    zwerg = 1;
  
  XawToggleSetCurrent(TdurationToggle,(XtPointer)op->output_data);
  XawToggleSetCurrent(ToldToggle,(XtPointer)zwerg);
  TSensitivityCallback(TpointToggle,NULL,NULL);
  TSensitivityCallback(ToldToggle,NULL,NULL);
  
  sprintf(buffer,"%lg",op->transient_time);
  setTextForWidget(TtransientTimeText,buffer);

  sprintf(buffer,"%lg",op->stepsize_method);
  setTextForWidget(TprecisionText,buffer);

  sprintf(buffer,"%lg",op->stepsize_result);
  setTextForWidget(Tstepsize2Text,buffer);

  sprintf(buffer,"%lg",op->precision);
  setTextForWidget(TgenprecText,buffer);

  sprintf(buffer,"%d",op->cluster_size);
  setTextForWidget(TclusterText,buffer);
  
  XtSetSensitive(Tstepsize2Label,op->output_data==Duration);
  XtSetSensitive(Tstepsize2Text,op->output_data==Duration);
}


/*******************************************************************************
Name:
	initTransientSimulationPopup
--------------------------------------------------------------------------------
Funktionstyp:
	public Funktion
--------------------------------------------------------------------------------
aufgerufen aus:
	evalMenu.c
	ButtonCallback
--------------------------------------------------------------------------------
Zweck:
	Uebernehmen der Daten aus einer TransientsimOptions-Struktur in das
	Transient Simulation Options Popup (Gegenstueck ist evalTransientSimulationPopup)
*******************************************************************************/
static void initTransientSimulationPopup(op)
TransientsimOptions *op;
{
  char buffer[VAL_BUF];

  XawToggleSetCurrent(TSdistributedToggle,
    (XtPointer)op->distribution);
  XawToggleSetCurrent(TSruleONToggle,
    (XtPointer)op->PercentageRule);

  sprintf(buffer,"%lg",op->TransientInstantOfTime);
  setTextForWidget(TStransinstimeText,buffer);
  sprintf(buffer,"%ld",op->NumberOfSamplingPoints);
  setTextForWidget(TSnumsampointsText,buffer);
  sprintf(buffer,"%lg",op->confidence_level);
  setTextForWidget(TSconfidenceLevelText,buffer);
  sprintf(buffer,"%lg",op->max_relative_error);
  setTextForWidget(TSmaxRelativeErrorText,buffer);
  sprintf(buffer,"%ld",op->seed_value); 
  setTextForWidget(TSseedValueText,buffer);
  sprintf(buffer,"%ld",op->max_CPU_time); 
  setTextForWidget(TSmaxCPUTimeLimitText,buffer);
  
  XtVaSetValues(TSnoCPUTimeLimitToggle,XtNstate,op->no_CPU_time_limit,NULL);
  XtSetSensitive(TSmaxCPUTimeLimitText,!op->no_CPU_time_limit);
}



/*******************************************************************************
Name:
       initAnalyticSolutionPopup
--------------------------------------------------------------------------------
Funktionstyp:
	public Funktion
--------------------------------------------------------------------------------
aufgerufen aus:
	evalMenu.c
	ButtonCallback
--------------------------------------------------------------------------------
Zweck:
	Uebernehmen der Daten aus einer AnalyticOptions-Struktur in das
	analytic Options Popup (Gegenstueck ist evalAnalyticSolutionPopup)
*******************************************************************************/
static void initAnalyticSolutionPopup(op)
AnalyticOptions *op;
{
  char buffer[VAL_BUF];

  XawToggleSetCurrent(ASiterativeToggle,(XtPointer)op->solution_method);
  ASSensitivityCallback(ASiterativeToggle,NULL,NULL);
  XawToggleSetCurrent(ASparallelToggle,(XtPointer)op->smc_solution);
  
  sprintf(buffer,"%ld",op->max_iterations);
  setTextForWidget(ASmaxIterationsText,buffer);
  sprintf(buffer,"%lg",op->precision);
  setTextForWidget(ASprecisionText,buffer);
}

/*******************************************************************************
Name:
	initApproximationPopup
--------------------------------------------------------------------------------
Funktionstyp:
	public Funktion
--------------------------------------------------------------------------------
aufgerufen aus:
	evalMenu.c
	ButtonCallback
--------------------------------------------------------------------------------
Zweck:
	Uebernehmen der Daten aus einer ApproximationOptions-Struktur in das
	approximation Options Popup (Gegenstueck: evalApproximationPopup)
*******************************************************************************/
static void initApproximationPopup(op)
ApproximationOptions *op;
{
  char buffer[VAL_BUF];
  
  XawToggleSetCurrent(AiterativeToggle,(XtPointer)op->solution_method);
  ASensitivityCallback(AiterativeToggle,NULL,NULL); 
  
  sprintf(buffer,"%ld",op->max_iterations);
  setTextForWidget(AmaxIterationsText,buffer);
  sprintf(buffer,"%lg",op->precision);
  setTextForWidget(AprecisionText,buffer);
  sprintf(buffer,"%ld",op->phases);
  setTextForWidget(AphaseNumberText,buffer);
  sprintf(buffer,"%ld",op->virtual_phases);
  setTextForWidget(AvirtPhaseNumberText,buffer);
  sprintf(buffer,"%lg",op->droptolerance);
  setTextForWidget(AdroptoleranceText,buffer);
}

/*******************************************************************************
Name:
	initSimulationPopup
--------------------------------------------------------------------------------
Funktionstyp:
	public Funktion
--------------------------------------------------------------------------------
aufgerufen aus:
	evalMenu.c
	ButtonCallback
--------------------------------------------------------------------------------
Zweck:
	Uebernehmen der Daten aus einer SimulationOptions-Struktur in das
	Simulation Options Popup (Gegenstueck: evalSimulationPopup)
*******************************************************************************/
static void initSimulationPopup(op)
SimulationOptions *op;
{
  char buffer[VAL_BUF];

  XawToggleSetCurrent(SfastToggle,
    (XtPointer)op->simulation_tech);  
  XawToggleSetCurrent(SspectralVarianceToggle,
    (XtPointer)op->statistical_method);
  XawToggleSetCurrent(SdistributedToggle,
    (XtPointer)op->distribution);
  XawToggleSetCurrent(SdetectOnToggle,
    (XtPointer)op->detect_init);
  XawToggleSetCurrent(SvisualOnToggle,
    (XtPointer)op->visualisation);
  
  sprintf(buffer,"%lg",op->confidence_level);
  setTextForWidget(SconfidenceLevelText,buffer);
  sprintf(buffer,"%lg",op->errPercent);
  setTextForWidget(SerrPercentLevelText,buffer);
  sprintf(buffer,"%lg",op->max_relative_error);
  setTextForWidget(SmaxRelativeErrorText,buffer);
  sprintf(buffer,"%ld",op->max_sample_number);
  setTextForWidget(SmaxSampleNumberText,buffer);
  sprintf(buffer,"%ld",op->minFirings);
  setTextForWidget(SminFiringsText,buffer);
  sprintf(buffer,"%ld",op->max_model_time); 
  setTextForWidget(SmaxModelTimeLimitText,buffer);
  sprintf(buffer,"%ld",op->max_CPU_time); 
  setTextForWidget(SmaxCPUTimeLimitText,buffer);
  sprintf(buffer,"%ld",op->seed_value); 
  setTextForWidget(SseedValueText,buffer);
  sprintf(buffer,"%ld",op->opt_thresh); 
  setTextForWidget(SoptThreshText,buffer);
  sprintf(buffer,"%ld",op->CPU_Time_Run); 
  setTextForWidget(SCPUTimeRunText,buffer);
  
  
  XtVaSetValues(SnoSampleLimitToggle,XtNstate,op->no_sample_limit,NULL);
  XtSetSensitive(SmaxSampleNumberText,!op->no_sample_limit);
  XtVaSetValues(SnoModelTimeLimitToggle,XtNstate,op->no_model_time_limit,NULL);
  XtSetSensitive(SmaxModelTimeLimitText,!op->no_model_time_limit);
  XtVaSetValues(SnoCPUTimeLimitToggle,XtNstate,op->no_CPU_time_limit,NULL);
  XtSetSensitive(SmaxCPUTimeLimitText,!op->no_CPU_time_limit);
}


/*new*/
/*******************************************************************************
Name:
	initDTAnalyticSolutionPopup
--------------------------------------------------------------------------------
Funktionstyp:
	public Funktion
--------------------------------------------------------------------------------
aufgerufen aus:
	evalMenu.c
	ButtonCallback
--------------------------------------------------------------------------------
Zweck:
	Uebernehmen der Daten aus einer AnalyticOptions-Struktur in das
	analytic Options Popup (Gegenstueck ist evalAnalyticSolutionPopup)
*******************************************************************************/
static void initDTAnalyticSolutionPopup(op)
DTSPNAnalyticOptions *op;
{
  char buffer[VAL_BUF];
  XawToggleSetCurrent(DTASiterativeToggle,(XtPointer)op->solution_method);
  ASSensitivityCallback(DTASiterativeToggle,NULL,NULL);
  XawToggleSetCurrent(DTASembnoToggle,(XtPointer)op->embedding);
  
  sprintf(buffer,"%lg",op->precision);
  setTextForWidget(DTASprecisionText,buffer);
  sprintf(buffer,"%ld",op->max_iterations);
  setTextForWidget(DTASmaxIterationsText,buffer);
  sprintf(buffer,"%ld",op->scaling);
  setTextForWidget(DTASscalingText,buffer);
  /*  XtSetSensitive(AStransientForm,op->solution_type==Transient);*/

}

/*new*/
/*******************************************************************************
Name:
	initDTTransientSolutionPopup
--------------------------------------------------------------------------------
Funktionstyp:
	public Funktion
--------------------------------------------------------------------------------
aufgerufen aus:
	evalMenu.c
	ButtonCallback--------------------------------------------------------------------------------
Zweck:
	Uebernehmen der Daten aus einer DTTransientOptions-Struktur in das
	DTTransient Options Popup 
*******************************************************************************/
static void initDTTransientSolutionPopup(op)
DTSPNTransientOptions *op;
{
  char buffer[VAL_BUF];
  OutputData output;

  sprintf(buffer,"%ld",op->time_steps);
  setTextForWidget(DTTStimeStepsText,buffer);
  sprintf(buffer,"%lg",op->precision);
  setTextForWidget(DTTSprecisionText,buffer);
  sprintf(buffer,"%ld",op->first_step);
  setTextForWidget(DTTSfirststepText,buffer);
  sprintf(buffer,"%ld",op->scaling);
  setTextForWidget(DTTSscalingText,buffer);

  if (op->step_size >= 1) 
    {
      output = Duration;
      sprintf(buffer,"%ld",op->step_size);
      setTextForWidget(DTTSstepsizeText,buffer);
    }
  else output = Point;

  XawToggleSetCurrent(DTTSdurationToggle,(XtPointer)output);  
  XawToggleSetCurrent(DTTSembnoToggle,(XtPointer)op->embedding);
  TSensitivityCallback(DTTSdurationToggle,NULL,NULL);
  XtSetSensitive(DTTSstepsizeLabel,output==Duration);
  XtSetSensitive(DTTSstepsizeText,output==Duration);
}

/*******************************************************************************
Name:
	evalTransientSolutionPopup
--------------------------------------------------------------------------------
Funktionstyp:
	public Funktion
--------------------------------------------------------------------------------
aufgerufen aus:
	evalMenu.c
	ButtonCallback
--------------------------------------------------------------------------------
Zweck:
	Uebernehmen der Daten aus dem Popup in die transientOptions-Struktur
	Gibt False zurueck, falls Werte fehlerhaft sind..
*******************************************************************************/

static Boolean evalTransientSolutionPopup(cop)
TransientOptions *cop;
{
  TransientOptions op; op=*cop;

  op.output_data = (int)((long)XawToggleGetCurrent(TdurationToggle));
/* printf("out:%d\n",op.output_data); */
  op.old_case_iii = (int)((long)XawToggleGetCurrent(ToldToggle));
  if(op.old_case_iii == 2)
    op.old_case_iii = 0;
  
  if(*getTextOfWidget(TtransientTimeText)==0
	|| !sscanf(getTextOfWidget(TtransientTimeText),"%lg",&op.transient_time))
  {  
     XtWarning("Missing value in field \"transient time\"!");
     return False;
  }
  if(op.transient_time<=0.0)
  {
     XtWarning("Value \"transient time\" must be positive!");
     return False;
  }
 
  if(*getTextOfWidget(TprecisionText)==0
  || !sscanf(getTextOfWidget(TprecisionText),"%lg",&op.stepsize_method))
  {
    XtWarning("Missing value in field \"stepsize of method\"!");
	return False;
  }
  if(op.stepsize_method<=0.0)
  {
    XtWarning("Range for value \"stepsize fo method\" must be positive!");
	return False;
  }

  if(*getTextOfWidget(Tstepsize2Text)==0
	|| !sscanf(getTextOfWidget(Tstepsize2Text),"%lg",&op.stepsize_result))
  {  
     XtWarning("Missing value in field \"stepsize of result\"!");
     return False;
  }
  if(op.stepsize_result<=0.0)
  {
     XtWarning("Value \"stepsize of result\" must be positive!");
     return False;
  }

  if(*getTextOfWidget(TgenprecText)==0
	|| !sscanf(getTextOfWidget(TgenprecText),"%lg",&op.precision))
  {  
     XtWarning("Missing value in field \"stepsize of result\"!");
     return False;
  }
  if(op.precision<=0.0)
  {
     XtWarning("Value \"precision\" must be positive!");
     return False;
  }

  if(*getTextOfWidget(TclusterText)==0
	|| !sscanf(getTextOfWidget(TclusterText),"%d",&op.cluster_size))
  {  
     XtWarning("Missing value in field \"stepsize of result\"!");
     return False;
  }
  if(op.cluster_size<=0.0)
  {
     XtWarning("Value \"cluster size\" must be positive!");
     return False;
  }
 
  *cop=op;
  return True;
}


/*******************************************************************************
Name:
	evalTransientSimulationPopup
--------------------------------------------------------------------------------
Funktionstyp:
	public Funktion
--------------------------------------------------------------------------------
aufgerufen aus:
	evalMenu.c
	ButtonCallback
--------------------------------------------------------------------------------
Zweck:
	Uebernehmen der Daten aus dem Popup in die transientOptions-Struktur
	Gibt False zurueck, falls Werte fehlerhaft sind..
*******************************************************************************/

static Boolean evalTransientSimulationPopup(cop)
TransientsimOptions *cop;
{
  TransientsimOptions op; op=*cop;

  op.distribution=(int)((long)XawToggleGetCurrent(TSdistributedToggle));
  op.PercentageRule=(int)((long)XawToggleGetCurrent(TSruleONToggle));
  XtVaGetValues(TSnoCPUTimeLimitToggle,XtNstate,&op.no_CPU_time_limit,NULL);

  if(*getTextOfWidget(TStransinstimeText)==0
  || !sscanf(getTextOfWidget(TStransinstimeText),"%lg",&op.TransientInstantOfTime))
  {
    XtWarning("Missing value in field \"Transient instant of time\"!");
	return False;
  }
  if(op.TransientInstantOfTime <= 0.0)
  {
    XtWarning("Transient instant must be > 0.0 !");
	return False;
  }

  if(*getTextOfWidget(TSnumsampointsText)==0
  || !sscanf(getTextOfWidget(TSnumsampointsText),"%ld",&op.NumberOfSamplingPoints))
  {
    XtWarning("Missing value in field \"Number of sampling points\"!");
	return False;
  }
  if(op.NumberOfSamplingPoints<3 || op.NumberOfSamplingPoints>100)
  {
    XtWarning("Range for value \"Number of sampling points\": 3 .. 100!");
	return False;
  }
  
  if(*getTextOfWidget(TSconfidenceLevelText)==0
  || !sscanf(getTextOfWidget(TSconfidenceLevelText),"%lg",&op.confidence_level))
  {
    XtWarning("Missing value in field \"confidence level\"!");
	return False;
  }
  if(op.confidence_level<80.0 || op.confidence_level>99.99)
  {
    XtWarning("Range for value \"confidence level\": 80..99,99%!");
	return False;
  }
  if(*getTextOfWidget(TSmaxRelativeErrorText)==0
  || !sscanf(getTextOfWidget(TSmaxRelativeErrorText),"%lg",&op.max_relative_error))
  {
    XtWarning("Missing value in field \"maximum relative error\"!");
	return False;
  }
  if(op.max_relative_error<0.5 || op.max_relative_error>20.0)
  {
    XtWarning("Range for value \"maximum relative error\": 0,5..20%!");
	return False;
  }
  if(*getTextOfWidget(TSseedValueText)==0 
  || !sscanf(getTextOfWidget(TSseedValueText),"%ld",&op.seed_value))
  {
    XtWarning("Missing value in field \"seed value\"!");
	return False;
  }
  if(op.seed_value<1)
  {
    XtWarning("\"seed value\" must be >0");
	return False;
  }
  if(!op.no_CPU_time_limit)
  {
    if(*getTextOfWidget(TSmaxCPUTimeLimitText)==0 
    || !sscanf(getTextOfWidget(TSmaxCPUTimeLimitText),"%ld",&op.max_CPU_time))
    {
      XtWarning("Missing value in field \"maximum CPU time\"!");
          return False;
    }
    if(op.max_CPU_time < 60)
    {
      XtWarning("\"maximum CPU time\" must be >= 60");
          return False;
    }
  }
  *cop=op;
  return True;
}


/*******************************************************************************
Name:
	evalAnalyticSolutionPopup
--------------------------------------------------------------------------------
Funktionstyp:
	public Funktion
--------------------------------------------------------------------------------
aufgerufen aus:
	evalMenu.c
	ButtonCallback
--------------------------------------------------------------------------------
Zweck:
	Uebernehmen der Daten aus dem Popup in die analyticOptions-Struktur
	Gibt False zurueck, falls Werte fehlerhaft sind..
*******************************************************************************/
static Boolean evalAnalyticSolutionPopup(cop)
AnalyticOptions *cop;
{
  AnalyticOptions op; op=*cop;

  op.solution_method=(int)((long)XawToggleGetCurrent(ASiterativeToggle));
  op.smc_solution=(int)((long)XawToggleGetCurrent(ASparallelToggle));
  
  if(*getTextOfWidget(ASprecisionText)==0
  || !sscanf(getTextOfWidget(ASprecisionText),"%lg",&op.precision))
  {
    XtWarning("Missing value in field \"precision\"!");
	return False;
  }
  if(op.precision<0.0 || op.precision>1.0)
  {
    XtWarning("Range for value \"precision\" is: 0..1!");
	return False;
  }
  if(op.solution_method==Iterative)
  { 
    if(*getTextOfWidget(ASmaxIterationsText)==0
    || !sscanf(getTextOfWidget(ASmaxIterationsText),"%ld",&op.max_iterations))
    {
      XtWarning("Missing value in field \"max. iterations\"!");
	  return False;
    }
    if(op.max_iterations<10 || op.max_iterations>100000L)
    {
      XtWarning("Range for value \"max. iterations\" is: 10..100000!");
	  return False;
    }
 }
  
  *cop=op;
  return True;
}

/*new*/
/*******************************************************************************
Name:
	evalDTAnalyticSolutionPopup
--------------------------------------------------------------------------------
Funktionstyp:
	public Funktion
--------------------------------------------------------------------------------
aufgerufen aus:
	evalMenu.c
	ButtonCallback
--------------------------------------------------------------------------------
Zweck:
	Uebernehmen der Daten aus dem Popup in die analyticOptions-Struktur
	Gibt False zurueck, falls Werte fehlerhaft sind..
*******************************************************************************/

static Boolean evalDTAnalyticSolutionPopup(cop)
DTSPNAnalyticOptions *cop;
{
  DTSPNAnalyticOptions op; op=*cop;

  op.solution_method=(int)((long)XawToggleGetCurrent(DTASiterativeToggle));
  op.embedding=(int)((long)XawToggleGetCurrent(DTASembnoToggle));
  
  if(*getTextOfWidget(DTASprecisionText)==0
  || !sscanf(getTextOfWidget(DTASprecisionText),"%lg",&op.precision))
  {
    XtWarning("Missing value in field \"precision\"!");
	return False;
  }
  if(op.precision<0.0 || op.precision>1.0)
  {
    XtWarning("Range for value \"precision\" is: 0..1!");
	return False;
  }
  if(op.solution_method==Iterative)
  { 
    if(*getTextOfWidget(DTASmaxIterationsText)==0
    || !sscanf(getTextOfWidget(DTASmaxIterationsText),"%ld",&op.max_iterations))
    {
      XtWarning("Missing value in field \"max. iterations\"!");
	  return False;
    }
    if(op.max_iterations<10 || op.max_iterations>100000L)
    {
      XtWarning("Range for value \"max. iterations\" is: 10..100000!");
	  return False;
    }
  }
  if(*getTextOfWidget(DTASscalingText)==0
    || !sscanf(getTextOfWidget(DTASscalingText),"%ld",&op.scaling))
    {
      XtWarning("Missing value in field \"scaling\"!");
	  return False;
    }
    if(op.scaling<1 || op.scaling>100000L)
    {
      XtWarning("Range for value \"scaling\" is: 1..100000!");
	  return False;
    }
  
  *cop=op;
  return True;
}

/*new*/
/*******************************************************************************
Name:
	evalDTTransientSolutionPopup
--------------------------------------------------------------------------------
Funktionstyp:
	public Funktion
--------------------------------------------------------------------------------
aufgerufen aus:
	evalMenu.c
	ButtonCallback
--------------------------------------------------------------------------------
Zweck:
	Uebernehmen der Daten aus dem Popup in die analyticOptions-Struktur
	Gibt False zurueck, falls Werte fehlerhaft sind..
*******************************************************************************/

static Boolean evalDTTransientSolutionPopup(cop)
DTSPNTransientOptions *cop;
{  
  long current=(long)XawToggleGetCurrent(DTTSpointToggle);
  DTSPNTransientOptions op; op=*cop;

  op.embedding=(int)((long)XawToggleGetCurrent(DTTSembnoToggle));
  if(*getTextOfWidget(DTTSprecisionText)==0
  || !sscanf(getTextOfWidget(DTTSprecisionText),"%lg",&op.precision))
  {
    XtWarning("Missing value in field \"precision\"!");
	return False;
  }
  if(op.precision<0.0 || op.precision>1.0)
  {
    XtWarning("Range for value \"precision\" is: 0..1!");
	return False;
  }
  if(*getTextOfWidget(DTTStimeStepsText)==0
     || !sscanf(getTextOfWidget(DTTStimeStepsText),"%ld",&op.time_steps))
  {
    XtWarning("Missing value in field \"time steps\"!");
    return False;
  }
  if(op.time_steps<1 || op.time_steps>10000000L)
  {
    XtWarning("Range for value \"time steps\" is: 1..10000000!");
    return False;
  }
  if(*getTextOfWidget(DTTSfirststepText)==0
     || !sscanf(getTextOfWidget(DTTSfirststepText),"%ld",&op.first_step))
  {
    XtWarning("Missing value in field \"first step\"!");
    return False;
  }
  if (current == Duration)
  {
    if (*getTextOfWidget(DTTSstepsizeText)==0
	|| !sscanf(getTextOfWidget(DTTSstepsizeText),"%ld",&op.step_size))
    {
      XtWarning("Missing value in field \"step size\"!");
      return False;
    }
    if(op.step_size<1 || op.step_size > op.time_steps)
    {
      XtWarning("Range for value \"step size\" : 1..\"time steps\"!");
      return False;
    }
  }
  else op.step_size = 0;

  if(*getTextOfWidget(DTTSscalingText)==0
    || !sscanf(getTextOfWidget(DTTSscalingText),"%ld",&op.scaling))
    {
      XtWarning("Missing value in field \"scaling\"!");
	  return False;
    }
    if(op.scaling<1 || op.scaling>100000L)
    {
      XtWarning("Range for value \"scaling\" is: 1..100000!");
	  return False;
    }
  *cop=op;
  return True;
}


/*******************************************************************************
Name:
	evalApproximationPopup
--------------------------------------------------------------------------------
Funktionstyp:
	public Funktion
--------------------------------------------------------------------------------
aufgerufen aus:
	evalMenu.c
	ButtonCallback
--------------------------------------------------------------------------------
Zweck:
	Uebernehmen der Daten aus dem Popup in die Approximation Options-
	Struktur Gibt False zurueck, falls Werte fehlerhaft sind..
*******************************************************************************/
static Boolean evalApproximationPopup(cop)
ApproximationOptions *cop;
{
  ApproximationOptions op; op=*cop;
  
  op.solution_method=(int)((long)XawToggleGetCurrent(AiterativeToggle));
  
  if(op.solution_method==Iterative)
  { 
    if(*getTextOfWidget(AmaxIterationsText)==0
    || !sscanf(getTextOfWidget(AmaxIterationsText),"%ld",&op.max_iterations))
    {
      XtWarning("Missing value in field \"max. iterations\"!");
	  return False;
    }
    if(op.max_iterations<10 || op.max_iterations>100000L)
    {
      XtWarning("Range for value \"max. iterations\" is: 10..100000!");
	  return False;
    }
 }
  if(*getTextOfWidget(AprecisionText)==0
  || !sscanf(getTextOfWidget(AprecisionText),"%lg",&op.precision))
  {
    XtWarning("Missing value in field \"precision\"!");
	return False;
  }
  if(op.precision<0.0 || op.precision>1.0)
  {
    XtWarning("Range for value \"precision\" is: 0..1!");
	return False;
  }
  if(*getTextOfWidget(AdroptoleranceText)==0
  || !sscanf(getTextOfWidget(AdroptoleranceText),"%lg",&op.droptolerance))
  {
    XtWarning("Missing value in field \"droptolerance\"!");
	return False;
  }
  if(op.droptolerance<0.0 || op.droptolerance>1.0)
  {
    XtWarning("Range for value \"droptolerance\" is: 0..1!");
	return False;
  }
  if(*getTextOfWidget(AphaseNumberText)==0
  || !sscanf(getTextOfWidget(AphaseNumberText),"%ld",&op.phases))
  {
    XtWarning("Missing value in field \"number of phases\"!");
	return False;
  }
  if(op.phases<1)
  {
    XtWarning("\"number of phases\" must be >0!");
	return False;
  }
  if(*getTextOfWidget(AvirtPhaseNumberText)==0
  || !sscanf(getTextOfWidget(AvirtPhaseNumberText),"%ld",&op.virtual_phases))
  {
    XtWarning("Missing value in field \"virtual number of phases\"!");
	return False;
  }
  if(op.virtual_phases<1)
  {
    XtWarning("\"virtual number of phases\" must be >0");
	return False;
  }
  
  *cop=op;
  return True;
}

/*******************************************************************************
Name:
	evalSimulationPopup
--------------------------------------------------------------------------------
Funktionstyp:
	public Funktion
--------------------------------------------------------------------------------
aufgerufen aus:
	evalMenu.c
	ButtonCallback
--------------------------------------------------------------------------------
Zweck:
	Uebernehmen der Daten aus dem Popup in die Simulation Options-
	Struktur Gibt False zurueck, falls Werte fehlerhaft sind..
*******************************************************************************/
static Boolean evalSimulationPopup(cop)
SimulationOptions *cop;
{
  SimulationOptions op; op=*cop;

  op.simulation_tech=(int)((long)XawToggleGetCurrent(SfastToggle));
  op.statistical_method=(int)((long)XawToggleGetCurrent(SspectralVarianceToggle));
  op.distribution=(int)((long)XawToggleGetCurrent(SdistributedToggle));
  op.detect_init=(int)((long)XawToggleGetCurrent(SdetectOnToggle));
  op.visualisation=(int)((long)XawToggleGetCurrent(SvisualOnToggle));
  XtVaGetValues(SnoSampleLimitToggle,XtNstate,&op.no_sample_limit,NULL);
  XtVaGetValues(SnoModelTimeLimitToggle,XtNstate,&op.no_model_time_limit,NULL);
  XtVaGetValues(SnoCPUTimeLimitToggle,XtNstate,&op.no_CPU_time_limit,NULL);
  
  if(*getTextOfWidget(SconfidenceLevelText)==0
  || !sscanf(getTextOfWidget(SconfidenceLevelText),"%lg",&op.confidence_level))
  {
    XtWarning("Missing value in field \"confidence level\"!");
	return False;
  }
  if(op.confidence_level<80.0 || op.confidence_level>99.99)
  {
    XtWarning("Range for value \"confidence level\": 80..99,99%!");
	return False;
  }
  if(*getTextOfWidget(SerrPercentLevelText)==0
  || !sscanf(getTextOfWidget(SerrPercentLevelText),"%lg",&op.errPercent))
  {
    XtWarning("Missing value in field \"difference for ..\"!");
	return False;
  }
  if(op.errPercent<0.0 || op.errPercent>50.00)
  {
    XtWarning("Range for value \"difference for ..\": 0..50.00%!");
	return False;
  }
  if(*getTextOfWidget(SmaxRelativeErrorText)==0
  || !sscanf(getTextOfWidget(SmaxRelativeErrorText),"%lg",&op.max_relative_error))
  {
    XtWarning("Missing value in field \"maximum relative error\"!");
	return False;
  }
  if(op.max_relative_error<0.5 || op.max_relative_error>20.0)
  {
    XtWarning("Range for value \"maximum relative error\": 0,5..20%!");
	return False;
  }
  if(*getTextOfWidget(SseedValueText)==0 
  || !sscanf(getTextOfWidget(SseedValueText),"%ld",&op.seed_value))
  {
    XtWarning("Missing value in field \"seed value\"!");
	return False;
  }
  if(op.seed_value<1)
  {
    XtWarning("\"seed value\" must be >0");
	return False;
  }
  if(!op.no_sample_limit)
  {
    if(*getTextOfWidget(SmaxSampleNumberText)==0 
    || !sscanf(getTextOfWidget(SmaxSampleNumberText),"%ld",&op.max_sample_number))
    {
      XtWarning("Missing value in field \"maximum number of samples\"!");
          return False;
    }
    if(op.max_sample_number<50)
    {
      XtWarning("\"maximum sample number\" must be >=50");
          return False;
    }
  }

  if(*getTextOfWidget(SminFiringsText)==0 
    || !sscanf(getTextOfWidget(SminFiringsText),"%ld",&op.minFirings))
  {
    XtWarning("Missing value in field \"minimal number of firings\"!");
          return False;
  }
  if(op.minFirings<0)
  {
    XtWarning("\"minimum firings number\" must be >=0");
         return False;
  }

  if(!op.no_model_time_limit)
  {
    if(*getTextOfWidget(SmaxModelTimeLimitText)==0 
    || !sscanf(getTextOfWidget(SmaxModelTimeLimitText),"%ld",&op.max_model_time))
    {
      XtWarning("Missing value in field \"maximum model time\"!");
          return False;
    }
    if(op.max_model_time < 0)
    {
      XtWarning("\"maximum model time\" must be positive");
          return False;
    }
  }
  if(!op.no_CPU_time_limit)
  {
    if(*getTextOfWidget(SmaxCPUTimeLimitText)==0 
    || !sscanf(getTextOfWidget(SmaxCPUTimeLimitText),"%ld",&op.max_CPU_time))
    {
      XtWarning("Missing value in field \"maximum CPU time\"!");
          return False;
    }
    if(op.max_CPU_time < 60)
    {
      XtWarning("\"maximum CPU time\" must be >= 60");
          return False;
    }
  }
  if(op.simulation_tech==Ssimfast)
  {
    if(*getTextOfWidget(SoptThreshText)==0 
    || !sscanf(getTextOfWidget(SoptThreshText),"%ld",&op.opt_thresh))
    {
      XtWarning("Missing value in field \"Initial number of thresholds\"!");
	  return False;
    }
    if(op.opt_thresh < 3 || op.opt_thresh > 20)
    {
      XtWarning("Range for value \"Optimal number of thresholds\": 3 .. 20 !");
	  return False;
    }
    if(*getTextOfWidget(SCPUTimeRunText)==0 
    || !sscanf(getTextOfWidget(SCPUTimeRunText),"%ld",&op.CPU_Time_Run))
    {
      XtWarning("Missing value in field \"CPU-time pilot run [sec]\" !");
	  return False;
    }
    if(op.CPU_Time_Run < 0 )
    {
      XtWarning("\"CPU-time pilot run [sec]\" must be positive ");
	  return False;
    }
  }
  *cop=op;
  return True;
}

/*******************************************************************************
Name:   TransientCallback
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
registriert mit:
	TokButton,TcancelButton,ThelpButton,TdefaultButton,
--------------------------------------------------------------------------------
Zweck:
	Entsprechend auf den gedrueckten Batten reagieren, ggf. Popdown
*******************************************************************************/
static void TransientCallback(w,client_data,call_data)
Widget w;
caddr_t client_data,call_data;
{
  switch((long)client_data)
  {
    case okButton:
      if(!evalTransientSolutionPopup(&transientOptions))
		break;
      /* else NO break */
    case cancelButton:
      XtPopdown(transientPopup);
      break;
    case defaultButton:
      initTransientSolutionPopup(&transientDefaults);
      break;
    case helpButton:
      PopUpHelp(w,"TransientOptions.help");
      /* rufe mein help auf */
      break;
  }
}   

/*******************************************************************************
Name:
	TransientSimulationCallback
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
registriert mit:
	TSokButton,TScancelButton,TShelpButton,TSdefaultButton,
--------------------------------------------------------------------------------
Zweck:
	Entsprechend auf den gedrueckten Batten reagieren, ggf. Popdown
*******************************************************************************/
static void TransientSimulationCallback(w,client_data,call_data)
Widget w;
caddr_t client_data,call_data;
{
  switch((long)client_data)
  {
    case okButton:
      if(!evalTransientSimulationPopup(&transientsimOptions))
		break;
      /* else NO break */
    case cancelButton:
      XtPopdown(transientSimulationPopup);
      break;
    case defaultButton:
      initTransientSimulationPopup(&transientsimDefaults);
      break;
    case helpButton:
      /*PopUpHelp(w,"TransientSimulationOptions.help");*/
      /* rufe mein help auf */
      break;
  }
}   

/*******************************************************************************
Name:
	AnalyticCallback
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
registriert mit:
	ASokButton,AScancelButton,AShelpButton,ASdefaultButton,
	DTASokButton,DTAScancelButton,DTAShelpButton,DTASdefaultButton,
--------------------------------------------------------------------------------
Zweck:
	Entsprechend auf den gedrueckten Batten reagieren, ggf. Popdown
*******************************************************************************/
static void AnalyticCallback(w,client_data,call_data)
Widget w;
caddr_t client_data,call_data;
{
  if (w==ASokButton||w==AScancelButton||w==AShelpButton||w==ASdefaultButton)
    {
      switch((long)client_data)
	{
	case okButton:
	  if(!evalAnalyticSolutionPopup(&analyticOptions))
		break;
	  /* else NO break */
        case cancelButton:
	  XtPopdown(analyticPopup);
	  break;
	case defaultButton:
	  initAnalyticSolutionPopup(&analyticDefaults);
	  break;
	case helpButton:
	  break;
	}
    }
  else 
    if (w==DTASokButton||w==DTAScancelButton||w==DTAShelpButton||w==DTASdefaultButton)
    {
      switch((long)client_data)
	{
	case okButton:
	  if(!evalDTAnalyticSolutionPopup(&dtspnanalyticOptions))
	    break;
	  /* else NO break */
	case cancelButton:
	  XtPopdown(dtanalyticPopup);
	  break;
	case defaultButton:
	  initDTAnalyticSolutionPopup(&dtspnanalyticDefaults);
	  break;
	case helpButton:
	  break;
	}
    }
    else
     {
      switch((long)client_data)
	{
	case okButton:
	  if(!evalDTTransientSolutionPopup(&dtspntransientOptions))
	    break;
	  /* else NO break */
	case cancelButton:
	  XtPopdown(dttransientPopup);
	  break;
	case defaultButton:
	  initDTTransientSolutionPopup(&dtspntransientDefaults);
	  break;
	case helpButton:
	  break;
	}
    } 
}   

/*******************************************************************************
Name:
	ApproximationCallback
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
registriert mit:
	AokButton,AcancelButton,AhelpButton,AdefaultButton,
--------------------------------------------------------------------------------
Zweck:
	Entsprechend auf den gedrueckten Batten reagieren, ggf. Popdown
*******************************************************************************/
static void ApproximationCallback(w,client_data,call_data)
Widget w;
caddr_t client_data,call_data;
{
  switch((long)client_data)
  {
    case okButton:
      if(!evalApproximationPopup(&approximationOptions))
		break;
      /* else NO break */
    case cancelButton:
      XtPopdown(approximationPopup);
      break;
    case defaultButton:
      initApproximationPopup(&approximationDefaults);
      break;
    case helpButton:
      break;
  }
}    

/*******************************************************************************
Name:
	SimulationCallback
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
registriert mit:
	SokButton,ScancelButton,ShelpButton,SdefaultButton
--------------------------------------------------------------------------------
Zweck:
	Entsprechend auf den gedrueckten Batten reagieren, ggf. Popdown
*******************************************************************************/
static void SimulationCallback(w,client_data,call_data)
Widget w;
caddr_t client_data,call_data;
{
  switch((long)client_data)
  {
    case okButton:
      if(!evalSimulationPopup(&simulationOptions))
		break;
      /* else NO break */
    case cancelButton:
      XtPopdown(simulationPopup);
      break;
    case defaultButton:
      initSimulationPopup(&simulationDefaults);
      break;
    case helpButton:
      break;
  }
}


 /******************************************************************************* 
 Name:                              						      		      
 	CheckNetType
 -------------------------------------------------------------------------------- 
 Funktionstyp:
 	private function 
 -------------------------------------------------------------------------------- 
 aufgerufen aus: 
 	AnalyticOptionsPopup 
 -------------------------------------------------------------------------------- 
 Zweck:
     bei DSPNs und ESPNs das transiente Loesen verbieten
 *******************************************************************************/ 
/* static void CheckNetType() */
/* { */
/* 	TimeNetType net_type; */
/* 	Widget nw = getCurrentNet(); */
	
/* 	XtVaGetValues(nw,XtNnetType,&net_type,NULL); */
/* 	switch(net_type) */
/* 	{ */
/* 	  case TN_GSPN: */
/* 	    XtSetSensitive(AStransientToggle,True); */
/* 		break; */
/* 	  case TN_DSPN: */
/* 	  case TN_ESPN: */
/* 	    XtSetSensitive(AStransientToggle,False); */
/* 		XawToggleSetCurrent(AStransientToggle,(XtPointer)SteadyState); */
/* 	XtSetSensitive(AStransientForm,False); */
/* 	XtSetSensitive(ASsteadyStateForm,True); */
/* 	} */
/* } */

/*******************************************************************************
Namen:
	TransientOptionsPopup, TransientSimulationOptionPopup, 
	AnalyticOptionsPopup, ApproximationOptionsPopup, SimulationOptionsPopup
--------------------------------------------------------------------------------
Funktionstypen:
	public functions
--------------------------------------------------------------------------------
aufgerufen aus:
	evalMenu.c
--------------------------------------------------------------------------------
Zweck:
	Popup des entsprechenden Widgets, vorherige Initialisierung mit den
	aktuellen Daten aus den entsprechenden Options-Strukturen
*******************************************************************************/
void TransientOptionsPopup()
{       
	initTransientSolutionPopup(&transientOptions);
	XtPopup(transientPopup,XtGrabNone);
	XtSetKeyboardFocus(transientPopup,*transient_focus.field[transient_focus.curr]);
}

void TransientSimulationOptionsPopup()
{       
   initTransientSimulationPopup(&transientsimOptions);
   XtPopup(transientSimulationPopup,XtGrabNone);
   XtSetKeyboardFocus(transientSimulationPopup,*transientsim_focus.field[transientsim_focus.curr]);
}

void AnalyticOptionsPopup()
{       initAnalyticSolutionPopup(&analyticOptions);
/*	CheckNetType(); */
	XtPopup(analyticPopup,XtGrabNone);
	XtSetKeyboardFocus(analyticPopup,*analytic_focus.field[analytic_focus.curr]);
}

void DTAnalyticOptionsPopup()
{       initDTAnalyticSolutionPopup(&dtspnanalyticOptions);
/*	CheckNetType();*/
	XtPopup(dtanalyticPopup,XtGrabNone);
	XtSetKeyboardFocus(dtanalyticPopup,*dtanalytic_focus.field[dtanalytic_focus.curr]);
}

void DTTransientOptionsPopup()
{       initDTTransientSolutionPopup(&dtspntransientOptions);
	XtPopup(dttransientPopup,XtGrabNone);
	XtSetKeyboardFocus(dttransientPopup,*dttransient_focus.field[dttransient_focus.curr]);
}
void ApproximationOptionsPopup()
{       initApproximationPopup(&approximationOptions);
	XtPopup(approximationPopup,XtGrabNone);
	XtSetKeyboardFocus(approximationPopup,*approximation_focus.field[approximation_focus.curr]);
}

void SimulationOptionsPopup()
{       initSimulationPopup(&simulationOptions);
	XtPopup(simulationPopup,XtGrabNone);
	XtSetKeyboardFocus(simulationPopup,*simulation_focus.field[simulation_focus.curr]);
}

/*static void saveOptions() {};*/








