/******************************************************************************
 File:              buffer.c

 Purpose:           Manage temporary buffer;
                    call parser if changes are applied.

 Project Director:  Reinhard German

 Programmer:        Ulrich Hertlein & Joerg W. Mitzlaff

 History:           Release 1:
                    Release 2: 3/1994  Kolja Koischwitz
******************************************************************************/

#include <stdio.h>
#include <string.h>

#include "environment.h"
#include "pmfEdit.h"
#include "pmfParsdfunc.h"

extern struct time_l *erg;
extern int DET_OR_UNI;
extern void XtWarning();

/*
 * Global variables
 */
static char pmf_temp[PMF_BUFLEN];
static char browse_temp[DF_BUFLEN]={"Unknown"};
double erwartungswert; /* Wird woanders auch verwendet! */

/*




 * Function Prototyping
 */
extern void append_pointmass();
extern void range_warning();

/*
 * buffer_apply:
 */
int buffer_apply(type,function)
int *type;
char *function;
{
   /* get buffer from text widget */
   get_text(function);
   if (*type==PMF_FUNC)
   {
    if(buffer_evaluate(function))
    {
   	    strcpy(pmf_temp,function);
	    
	    return(1);
    }
   }
   else  
   {
      char *pBF;  
      pBF=function;
      pBF+=strlen(function)-4;
      if(*pBF!='.') 
      {
       strcat(function,".DAT");
      }

   
    if (check_file(function))
    {
     strcpy(browse_temp,function);  
     return(1);
    }
   }
   return(0);
}

/*
 * buffer_discard:
 */
int buffer_discard(type,function)
int *type;
char *function;
{
   /* discard changes */
 if(*type==PMF_FUNC) strcpy(function,pmf_temp);
 else  if (strcmp(browse_temp,"Unknown")) strcpy(function,browse_temp);
 set_text(function);   
}

/*
 * buffer_clear:
 */
int buffer_clear(function)
char *function;
{
   *function=0;
   set_text(function);
}

/*
 * buffer_evaluate:
 */
int buffer_evaluate(function)
char *function;
{
    double offneu;


    if(do_parse(function)==0)
    {
      /* pointmass warnings auswerten */
      
      if ((erg->type != NORMAL) && (erg->type != WEIBULL))
	erwartungswert =  expected_value(erg); 
      else
	erwartungswert = expected_value2(erg);

      if(!DET_OR_UNI)
	offneu = integrate(erg,INTEGRATE_NORMAL);
      else
	offneu=1;

      /* ist es auch eine korrekte fkt.? */
      if( (offneu-1>NULL_UMGEBUNG) || (offneu-1< -NULL_UMGEBUNG) )
        if(1-offneu>NULL_UMGEBUNG)
	  {
	    char warn[128];
		
		sprintf(warn,"warning: PDF-function reaches not 1 on y-achses. Pointmass %.4g appended.",1-offneu);
		XtWarning(warn);
		
	    append_pointmass(offneu);  /* fkt. mit punktmasse zu 1 ergaenzen ? */
	  }
        else	
	  {
		char warn[128];
		sprintf(warn,"warning: PDF-function is about  %.4g bigger than 1.",offneu - 1);
        XtWarning(warn);
	    range_warning(offneu - 1);
	  }
#ifdef DEBUG      
      printf("expolynomial distribution:\n");
      print_expoly(erg);
#endif
      xlib_draw_graph();

	  return(1);
    }
    else
		return(0);
}

int check_file(filename)
char *filename;

{
 FILE *f;
 char *text;
/* char modelpath[511];*/
 text=filename;
/* strcpy(modelpath,(char*)(getenv(MODELDIR_ENV)));
 strcat(modelpath,"/");
 printf("modelpath=%s\n",modelpath);*/
 do text++; while ((*text!='.') && (text!=NULL));
 text++; 
 if (!strcmp(text,"DAT"))
 {
/*  strcat(modelpath,filename); 
  printf(" : %s\n",modelpath); */
  f=fopen(filename,"r");
  if (f != NULL)
   {
    fclose(f);
    return(1);
   }
  else 
  {
   XtWarning("File not exist");
   return(0);
  }
 }
 else
 {
  XtWarning("Unable to open the file !!!");
  return(0);
 }
 return(0);
}

