/******************************************************************************
 File:              pmfEdit.c

 Purpose:           Create the user-interface, install callbacks, and handle
                    user interaction. The only interface to the existing
                    software is the function <pmfEdit> which takes a Transition-
					Name and a Function to modify. Everything else is hidden from
                    the application.

                    The variable app_context must be imported in order to
                    install actions and the translation table.

 Project Director:  Reinhard German

 Programmer:        Ulrich Hertlein & Joerg W. Mitzlaff

 History:           Release 1:
 					Release 2: 3/1994   Kolja
******************************************************************************/

#include <stdio.h>
#include <string.h>

#include "pmfEdit.h"
#include "pmfXlib.h"
#include "pmfParsdfunc.h"
#include "environment.h"

#include <X11/Shell.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Viewport.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Dialog.h>

#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/Toggle.h>

#define PMF_CHANGE_TITLE "Edit probability mass function of transition %s"
#define PMF_SHOW_TITLE   "probability mass function of transition %s"
#define BUTTONWIDTH 70
#define WIDTH_2     (WIDTH >> 1)
#define HEIGHT_2    (HEIGHT >> 1)

#define TOGGLE_LEFT_MARGIN 16

#define MAXNAMLEN	256


/*
 * Global variables
 */
extern Pixmap toggle_pixmap;

extern XtAppContext app_context;
extern void PopUpHelp();
extern void helpDone();
extern void pmfDialogDone(); /* Import aus textPopup.c */
extern void CenterWidget();

extern void GetModelName();
int *type;		 	/* hezi: 11/1/97 */
int func_mode = MODE_PMF; /* Function Mode at Startup */
int file_mode = 0;

extern time_l *erg;
extern double erwartungswert; /* aus pmfBuffer.c */

/*
 * Local variables
 */
static  char bufglobal[PMF_BUFLEN]; /* von append_pointmass() nach ja_func() */
static double impulse_offset;   /* von append_pointmass() nach ja_func() */
static int mode = MODE_EDIT;
static char *function; /* Current Function */
static char *Browse_File; /* hezi 12/96 */
static char title[MAXNAMLEN];

Widget pmfShell;

static Widget button2Form;
static Widget browseToggle, funkToggle;
static Widget editButton,cancelButton,clearButton,
              modeButton,mode2Button,helpButton,doneButton;

static Widget textField;
static Widget graphView;
static Widget graphCore;

static Widget switchMenu, switchEntry[3];
static Widget switch2Menu, switch2Entry[3];

static Widget dialogPopup, dialogWnd, jaButton, neinButton;
  
/* actions & translations for the core widget */
static XtActionsRec graphActions[1] = {{ "refresh", xlib_refresh }};
static String graphTrans = "<Expose>: refresh()";


/* Prototyping */
void FunktionEdit();
void BrowseFile();

/*
 * set_edit: set textField widget editable or not
 */
static int set_edit(edit)
int edit;
{
   /* set read/write */
   XtVaSetValues(textField,
      XtNeditType,edit == True ? XawtextEdit : XawtextRead,
	  XtNdisplayCaret,edit == True ? True : False,
      NULL);
}

/*******************************************************************
 * switch_mode:
 ******************************************************************/
static int switch_mode()
{
   int sense;
   int prev;

   /* determine new mode */
   prev = mode;
   if( mode == MODE_EDIT )
   {
      sense = True;
      mode = MODE_APPLY;
   }
   else
   {
      sense = False;
      mode = MODE_EDIT;
   }

   /* change Edit-button label */
   XtVaSetValues(editButton,
      XtNlabel,mode == MODE_EDIT ? "Edit": "Apply",
      NULL);

   /* change Cancel- & Done-button sensitivity */
   XtSetSensitive(cancelButton,sense);
   XtSetSensitive(clearButton,sense);
   XtSetSensitive(doneButton,!sense);
   if (*type==PMF_FUNC) 
    XtSetSensitive(modeButton,!sense);
   else 
    XtSetSensitive(modeButton,0);
   XtSetSensitive(button2Form,!sense);
   /*   XtSetSensitive(browseToggle,!sense);   
   XtSetSensitive(funkToggle,!sense);   */

   /* change read/write status of textField */
   set_edit(sense);

   return( prev );
}

/*
 * set_text: set textField string
 */
void set_text(text)
char *text;
{
   /* set string */
   XtVaSetValues(textField,
		 XtNstring,text,
		 NULL);
}

/*
 * get_text: get textField string
 */
void get_text(dest)
char *dest;
{
   char   *ptr;
   String inp_text;
   
   /* get string */
   XtVaGetValues(textField,
      XtNstring,&inp_text,
      NULL);
   /*if(*ptr = strrchr(inp_text,'/') != NULL) *++ptr='\0';*/
   strcpy(dest,inp_text);
}

/*******************************************************************
 editCB:
****************************************************************** */
int editCB(w, clientData, callData)
Widget w;
XtPointer clientData, callData;
{
   /* switch mode */
   if( mode == MODE_APPLY )
   {
     if (*type==PMF_FUNC)
     {
      if ( buffer_apply(type,function) )
         switch_mode();
     }
     else 
     {
      if(buffer_apply(type,Browse_File))
       switch_mode();
     }
   }
   else
      switch_mode();
}

/*
 * cancelCB:
 */
int cancelCB(w, clientData, callData)
Widget w;
XtPointer clientData, callData;
{
   /* set new mode */
    if( switch_mode() )
     if (*type==PMF_FUNC)
       buffer_discard(type,function);
     else
     {
       buffer_discard(type,Browse_File);
       if (!strcmp(Browse_File,"")) 
       {
	 *type=PMF_FUNC;
	 XawToggleSetCurrent(browseToggle,(XtPointer)*type);
	 FunktionEdit();
       }
     }
}

/*
 * clearCB:
 */
int clearCB(w, clientData, callData)
Widget w;
XtPointer clientData, callData;
{
 if (*type==PMF_FUNC)
    buffer_clear(function);
 else 
    buffer_clear(Browse_File);
}
/*
 * modeCB:
 */
int modeCB(w, clientData, callData)
Widget w;
XtPointer clientData, callData;
{
  int i;
  func_mode = (int)((long)clientData);
  for(i=0;i<3;i++)
    XtVaSetValues(switchEntry[i],XtNleftBitmap,None,NULL);
  XtVaSetValues(switchEntry[func_mode],XtNleftBitmap,toggle_pixmap,NULL);
  
  buffer_apply(type, function);
}

/*
 * fileModeCB:
 */
int fileModeCB(w, clientData, callData)
Widget w;
XtPointer clientData, callData;
{
  int i;
  file_mode = (int)((long)clientData);
  for(i=0;i<3;i++)
    XtVaSetValues(switch2Entry[i],XtNleftBitmap,None,NULL);
  XtVaSetValues(switch2Entry[file_mode],XtNleftBitmap,toggle_pixmap,NULL);
  
}


/*
 * doneCB:
 */
int doneCB(w, clientData, callData)
Widget w;
XtPointer clientData, callData;
{
 if (*type==DATAFILE)
  if(!buffer_apply(type,Browse_File))
  {
   *type=PMF_FUNC;
   XawToggleSetCurrent(browseToggle,(XtPointer)*type);
   XtWarning("File not exist ! Switch to function mode.");
  }
 switch(file_mode) {
   case SEQUENCE: strcat(Browse_File,":SEQUENCE");
   break;
   case SEQU_REV: strcat(Browse_File,":SEQU_REV");
   break;
   case RAND_UNI: strcat(Browse_File,":RAND_UNI");
   break;
 }
 /* pop-down shell */
 helpDone(w,NULL,NULL);
 XtPopdown(pmfShell);
}
/***********************************************************************/
void BrowseFile()
/***********************************************************************/
{ 
  *type=DATAFILE;
  XtSetSensitive(modeButton,False ); 
  XtSetSensitive(mode2Button,True ); 
  XtUnmanageChild(graphView);
  set_text(Browse_File);
}

/***********************************************************************/
void FunktionEdit()
/***********************************************************************/
{
  *type=PMF_FUNC;
  XtSetSensitive(modeButton,True); 
  XtSetSensitive(mode2Button,False ); 
  XtManageChild(graphView);
  set_text(function);
  buffer_apply(type, function);
  
}


/*
 * pmfSetup:
 */
void pmfSetup(parent)
Widget parent;
{
   Widget Box;
   Widget panedWin;
   Widget infoText;
   Widget buttonForm;
   /*Widget graphView;**10/96 hezi**/
   static char trans_toggle[] = "<Btn1Down>,<Btn1Up>: set() notify()"; 
                            /* aus evalMenu.c uebern. hezi: wofuer: ?*/

   /* create pop-up shell */
   pmfShell = XtVaCreatePopupShell(
                 "pmfShell",transientShellWidgetClass,parent,
                 XtNminWidth,MIN_WIDTH,
                 XtNminHeight,MIN_HEIGHT,
                 NULL);
  XtAddCallback(pmfShell,XtNpopupCallback,CenterWidget,NULL);
  
   /* create paned window */
   panedWin = XtVaCreateManagedWidget(
                 "editPane",panedWidgetClass,pmfShell,
                  NULL);
   /* 
    * create a box to buttons
    */
   button2Form =  XtVaCreateManagedWidget(
                   "button2Form",formWidgetClass,panedWin,
                   XtNshowGrip,False,
                   NULL);
   buttonForm = XtVaCreateManagedWidget(
                   "buttonForm",formWidgetClass,panedWin,
                   XtNshowGrip,False,
		   XtNfromVert,button2Form,
                   NULL);
 		   
  /* create edit, quit, and help buttons .. */
  browseToggle = XtVaCreateManagedWidget(
		"browseToggle",
		toggleWidgetClass,
		button2Form,
		XtNlabel,"browse File",
		XtNtranslations,XtParseTranslationTable(trans_toggle),
		XtNradioData,DATAFILE,
		NULL);
  XtAddCallback(browseToggle,XtNcallback,BrowseFile,NULL);

  funkToggle = XtVaCreateManagedWidget(
		"funkToggle",
		toggleWidgetClass,
		button2Form,
		XtNlabel,"funk",
		XtNtranslations,XtParseTranslationTable(trans_toggle),
		XtNradioData,PMF_FUNC,
		XtNfromHoriz,browseToggle,
		NULL);
  XtAddCallback(funkToggle,XtNcallback,FunktionEdit,NULL);

  XawToggleChangeRadioGroup(browseToggle,funkToggle);

   editButton = XtVaCreateManagedWidget(
                   "Edit",commandWidgetClass,buttonForm,
                   XtNwidth,BUTTONWIDTH,
                   XtNright,XtChainLeft,
                   NULL);
   cancelButton = XtVaCreateManagedWidget(
                     "Undo",commandWidgetClass,buttonForm,  /* Undo ist Cancel ! */
                     XtNwidth,BUTTONWIDTH,
                     XtNleft,XtChainLeft,
                     XtNright,XtChainLeft,
                     XtNfromHoriz,editButton,
                     NULL);
   clearButton = XtVaCreateManagedWidget(
                     "clear",commandWidgetClass,buttonForm,  
                     XtNwidth,BUTTONWIDTH,
                     XtNleft,XtChainLeft,
                     XtNright,XtChainLeft,
                     XtNfromHoriz,cancelButton,
                     NULL);
   modeButton = XtVaCreateManagedWidget(
                     "switch f.",menuButtonWidgetClass,buttonForm,  
                     XtNwidth,BUTTONWIDTH,
                     XtNleft,XtChainLeft,
                     XtNright,XtChainLeft,
                     XtNfromHoriz,clearButton,
                     NULL);
   switchMenu = XtVaCreatePopupShell(
                     "menu",simpleMenuWidgetClass,modeButton,  
		      NULL);
   switchEntry[0] = XtVaCreateManagedWidget(
                     "pmf function", smeBSBObjectClass, switchMenu,
					 XtNleftMargin,TOGGLE_LEFT_MARGIN,
		      NULL);		 
   XtAddCallback(switchEntry[0],XtNcallback, (XtCallbackProc)modeCB,(XtPointer) 0);

   switchEntry[1] = XtVaCreateManagedWidget(
                     "PDF function", smeBSBObjectClass, switchMenu,
		 XtNleftMargin,TOGGLE_LEFT_MARGIN,
		      NULL);		 
   XtAddCallback(switchEntry[1],XtNcallback, (XtCallbackProc)modeCB,(XtPointer) 1);

   switchEntry[2] = XtVaCreateManagedWidget(
                     "1-PDF function", smeBSBObjectClass, switchMenu,
		 XtNleftMargin,TOGGLE_LEFT_MARGIN,
		      NULL);		 
   XtAddCallback(switchEntry[2],XtNcallback, (XtCallbackProc)modeCB,(XtPointer) 2);

   XtVaSetValues(switchEntry[func_mode],XtNleftBitmap,toggle_pixmap,NULL);


   mode2Button = XtVaCreateManagedWidget(
                     "switch ....",menuButtonWidgetClass,buttonForm,  
                     XtNwidth,BUTTONWIDTH,
                     XtNleft,XtChainLeft,
                     XtNright,XtChainLeft,
                     XtNfromHoriz,modeButton,
                     NULL);

   switch2Menu = XtVaCreatePopupShell(
                     "menu",simpleMenuWidgetClass,mode2Button,  
		      NULL);
   switch2Entry[0] = XtVaCreateManagedWidget(
                     "sequence", smeBSBObjectClass, switch2Menu,
					 XtNleftMargin,TOGGLE_LEFT_MARGIN,
		      NULL);		 
   XtAddCallback(switch2Entry[0],XtNcallback, (XtCallbackProc)fileModeCB,(XtPointer) 0);

   switch2Entry[1] = XtVaCreateManagedWidget(
                     "sequence reverse", smeBSBObjectClass, switch2Menu,
		 XtNleftMargin,TOGGLE_LEFT_MARGIN,
		      NULL);		 
   XtAddCallback(switch2Entry[1],XtNcallback, (XtCallbackProc)fileModeCB,(XtPointer) 1);

   switch2Entry[2] = XtVaCreateManagedWidget(
                     "uniform random", smeBSBObjectClass, switch2Menu,
		 XtNleftMargin,TOGGLE_LEFT_MARGIN,
		      NULL);		 
   XtAddCallback(switch2Entry[2],XtNcallback, (XtCallbackProc)fileModeCB,(XtPointer) 2);

   XtVaSetValues(switch2Entry[file_mode],XtNleftBitmap,toggle_pixmap,NULL);
   
   helpButton = XtVaCreateManagedWidget(
                   "Help",commandWidgetClass,buttonForm,
                   XtNwidth,BUTTONWIDTH,
                   XtNleft,XtChainRight,
                   XtNright,XtChainRight,
                   XtNfromHoriz,modeButton,
                   NULL);
   doneButton = XtVaCreateManagedWidget(
                   "Done",commandWidgetClass,buttonForm,
                   XtNwidth,BUTTONWIDTH,
                   XtNleft,XtChainRight,
                   XtNfromHoriz,helpButton,
                   NULL);

   /*
    * create text field
   */
   textField = XtVaCreateManagedWidget(
                  "textField",
                  asciiTextWidgetClass,
                  panedWin,
                  XtNheight,60,                  
                  XtNscrollVertical,XawtextScrollWhenNeeded,
                  XtNborderWidth,0,
                  XtNshowGrip,True,
                  NULL);

   /*
    * create graphics display
    */
   graphView = XtVaCreateManagedWidget(
                  "graphView",viewportWidgetClass,panedWin,
                   XtNshowGrip,True,
                  NULL);

   /* parse action table */
   XtAppAddActions(app_context,graphActions,XtNumber(graphActions));

   /* create core widget to draw on */
   graphCore = XtVaCreateManagedWidget(
                  "graphCore",coreWidgetClass,graphView,
                  XtNwidth,WIDTH,
                  XtNheight,HEIGHT,
                  XtNtranslations,XtParseTranslationTable(graphTrans),
                  NULL);

   /* create GC, Pixmap, etc */
   xlib_setup(parent);


  dialogPopup = XtVaCreatePopupShell(
		    "TimeNET Dialog",
		    transientShellWidgetClass,
		    pmfShell,
		    NULL);

  dialogWnd = XtVaCreateManagedWidget(
		    "dialogWnd",
		    dialogWidgetClass,
		    dialogPopup,
		    XtNwidth,100,
	  	    XtNheight,100,
                    XtNlabel,"Add that pointmass?",
		    XtNvalue,"",
                    NULL);

  jaButton = XtVaCreateManagedWidget(
		   "jaButton",
                    commandWidgetClass,
                    dialogWnd,
                    XtNlabel,"Yes",
                    NULL);
  neinButton = XtVaCreateManagedWidget(
		   "neinButton",
                    commandWidgetClass,
                    dialogWnd,
                    XtNlabel,"No",
                    NULL);

   /* bind callbacks */
   XtAddCallback(editButton,XtNcallback,(XtCallbackProc)editCB,NULL);
   XtAddCallback(cancelButton,XtNcallback,(XtCallbackProc)cancelCB,NULL);
   XtAddCallback(clearButton,XtNcallback,(XtCallbackProc)clearCB,NULL);

   XtAddCallback(helpButton,XtNcallback,PopUpHelp,(XtPointer)"expoly.help");
   XtAddCallback(doneButton,XtNcallback,(XtCallbackProc)doneCB,NULL);
   XtAddCallback(doneButton,XtNcallback,(XtCallbackProc)pmfDialogDone,NULL);
   XtSetSensitive(cancelButton,False);
   XtSetSensitive(clearButton,False);
   XtAddCallback(jaButton,XtNcallback,(XtCallbackProc)ja_func,NULL);
   XtAddCallback(neinButton,XtNcallback,(XtCallbackProc)nein_func,NULL);
}

/*****************************************************************************
 * pmfEdit:
 * In: transname, funcptr, show_only;
 ****************************************************************************/
void pmfEdit(transname, type_ptr, func_ptr, file_ptr, show_only)
char *transname;
int *type_ptr;
char *func_ptr;
char *file_ptr;
int show_only;
{
  int i;
  char *h;
   /* reference to transition */
   sprintf(title, show_only?PMF_SHOW_TITLE:PMF_CHANGE_TITLE, transname);
   /* global */ 
   type=type_ptr;

   function=func_ptr;
#ifdef debug
   printf("pmfEdit.c: function ist = %s\n", function);    
#endif
   Browse_File=file_ptr;
   h=Browse_File;
   while(strncmp(h,"DAT",3)) h++;
   h+=4;
   if (!strncmp(h,"SEQUENCE",8)) file_mode=SEQUENCE;
   else if (!strncmp(h,"SEQU_REV",8)) file_mode=SEQU_REV;
   else if (!strncmp(h,"RAND_UNI",8)) file_mode=RAND_UNI;
   else {printf("file_mode=%s ungueltig\n",h); exit(0);}
   for(i=0;i<3;i++)
    XtVaSetValues(switch2Entry[i],XtNleftBitmap,None,NULL);
   XtVaSetValues(switch2Entry[file_mode],XtNleftBitmap,toggle_pixmap,NULL);
   h--;   
   *h='\0';
#ifdef debug  
   printf("file_mode=%d\n",file_mode);   
   printf("pmfEdit.c:Browse_File ist = %s\n",Browse_File);
#endif
   /* check if widget exists */
   if( !pmfShell )
   {	fprintf(stderr,"pmfShell not initialized. Call pmf_setup first!\n");
   		exit();
   }

   XtVaSetValues(pmfShell,XtNtitle,title,NULL);
   
   XtVaSetValues(editButton,XtNsensitive,show_only?False:True,NULL);
   XawToggleSetCurrent(browseToggle,(XtPointer)*type);

   if(*type==PMF_FUNC)
   {
    set_text(function);
    buffer_apply(type, function);
   }
   else 
   {
    set_text(Browse_File);
   }
   set_edit(False);

   XtPopup(pmfShell,XtGrabNonexclusive);
}

/**************************************************************************/
void append_pointmass(offset)
     double offset;
/**************************************************************************/
{
  Dimension width,height;
  Position  x,y;
  int semi;

  /* vorschlag fuer text buffer */
  impulse_offset = offset;
  semi = 0;
  while(function[semi]!=';')
    semi ++;
  sprintf(bufglobal,"%.16g",1-offset);
  if(!strchr(bufglobal,'.'))  /* NULL if . not found */
    strcat(bufglobal,".0");

  XtVaSetValues(dialogWnd,
		XtNvalue,bufglobal,
		NULL);

  XtVaGetValues(pmfShell, 
		XtNwidth, &width, 
		XtNheight, &height,
		NULL);
  
  XtTranslateCoords(pmfShell,(Position)width/2,(Position)height/2,&x,&y);
  XtVaSetValues(dialogPopup,
		XtNx,x-50,
		XtNy,y-30,
		NULL);
  XtPopup(dialogPopup,XtGrabNonexclusive);

  XtSetSensitive(editButton,False);
  XtSetSensitive(helpButton,False);
  XtSetSensitive(doneButton,False);
  XtSetSensitive(modeButton,False); 
}

void ja_func(w, clientData, callData)
    Widget w;
    XtPointer clientData, callData;
{
  time_l *i;
  char bufa[PMF_BUFLEN];
  char bufb[PMF_BUFLEN];

  /* struktur erweitern */
  i = erg;
  while(i->next != NULL)
    i = i->next;
  i->p += 1 - impulse_offset;
  i->power->expo->b = 0;

  /* textbuffer erweitern */  
  bufa[0]='\0';
  strcpy(bufa," + ");
  strcat(bufa,bufglobal); /* faktor */

  sprintf(bufb,"*I[%.16g",i->x1);
  if(!strchr(bufb,'.'))
    strcat(bufb,".0");
  strcat(bufb,"];\n");

  /* gibt es diesen Impus schon im Text? (impliziert durch ; das er auch am ende ist) */
  if(strrchr(function,'*')!=NULL) /* darf ich die naechste Frage ueberhaupt stellen */
    {
/*      printf("%s\n%s\n%s\n",strrchr(function,'*'),bufb,bufglobal); */
      if(strcmp( strrchr(function,'*'), bufb)==0)
	{
	  sprintf(bufglobal,"%.16g",i->p);
	  printf("%s %g\n",bufglobal,i->p);
	  if(!strchr(bufglobal,'.'))  /* NULL if . not found */
	    strcat(bufglobal,".0");

	  bufa[0]='\0';
	  strcpy(bufa," + ");
	  strcat(bufa,bufglobal); /* addierter faktor */

	  strcpy( strrchr(function,'+'),  strcat(bufa,bufb));
	}
      else
	{
	  strcpy( strchr(function,';'), strcat(bufa,bufb));
	}
    }
  else
    strcpy( strchr(function,';'), strcat(bufa,bufb));
  
  set_text(function);

  XtPopdown(dialogPopup);
  XtSetSensitive(helpButton,True);
  XtSetSensitive(editButton,True);
  XtSetSensitive(doneButton,True);
  XtSetSensitive(modeButton,True);

  erwartungswert =  expected_value(erg);

  printf("expolynomial distribution:\n");
  print_expoly(erg);
  xlib_draw_graph();	
}

void nein_func(w, clientData, callData)
    Widget w;
    XtPointer clientData, callData;
{
  XtPopdown(dialogPopup);
  XtSetSensitive(helpButton,True);
  XtSetSensitive(doneButton,True);
  XtSetSensitive(editButton,True);
  XtSetSensitive(modeButton,True); 

  xlib_draw_graph();	
}

void range_warning(offset)
     double offset;
{

}


