/******************************************************************************
 File:              graph.c

 Purpose:           Convert the internal time_l structure into a set of
                    virtual coordinates, which can then be drawn. They should
                    resemble the graph of the given formula.

 Project Director:  Reinhard German

 Programmer:        Ulrich Hertlein & Joerg W. Mitzlaff

 History:           Release 1:
******************************************************************************/

#include <stdio.h>
#include <math.h>

#include "pmfEdit.h"
#include "pmfGraph.h"

#define DUMMY 815.0
extern char *Malloc();

/*
 * alloc_new:
 */
struct g_data *alloc_new(x,y,t)
double x,y;
char t;
{
   struct g_data *n;

   /* allocate new entry */
/*  printf("%f %f %i\n",x,y,t); */
   n = (struct g_data *)Malloc(sizeof(struct g_data));
   n->x = x;
   n->y = y;
   n->type = t;
   n->next = NULL;

   return( n );
}

/*
 * graph_build:
 */
struct g_data *graph_build(rx,ry)
double *rx,*ry;
{
   double x,step, val,val2;
   double faktor;
   double impus;

   struct g_data *gd, *prev, *new;
   struct time_l *ttime;      /* Tea time !!! */
   struct power_m *tpow;
   struct expo_n *tex;

   gd = NULL;
   /* reset boundaries */
   *rx = 0.0;
   *ry = 0.0;

   /* determine maximum x */
   for( ttime = erg; ttime->next; ttime = ttime->next )
   {
     *rx = ttime->x2;
   }

   /* calculate step size */
   x = 0.0;
   impus = 0.0;
   *rx *= 1.1;
   if((erg->type==NORMAL)&&(func_mode != MODE_PMF))
     step = *rx / (double )NUM_STEP_NORMAL;
   else
     step = *rx / (double )NUM_STEP;

   /* walk down chained list of intervals */
   for( ttime = erg; ttime; ttime = ttime->next )
   {
      new = alloc_new((double)DUMMY, ttime->p, INTERVAL);
      if( !gd )
	gd = new;
      else
	prev->next = new;
      prev = new;

      /* to get max. y-value ... */
      if(func_mode == MODE_PMF)
	{
	  /* ...store impulse height */
	  if( ttime->p > *ry )
	    *ry = ttime->p;
	}
      if(func_mode == MODE_PDF)
	{
	  /* ...add impulse */
	  impus += ttime->p;
	}
      
      
      /* all X's in the current interval */
      while(ttime->next && x <= (ttime->x2))
      {
	 /* look at power list */
         val = 0.0;
	 if(ttime->type == WEIBULL)
	 {
	   switch(func_mode)
	   {
	     case MODE_PMF: 
	       if (ttime->sigma != 0.0)
		 val = (ttime->mu/ttime->sigma)*hb_pow(x/ttime->sigma,ttime->mu-1)*exp(-hb_pow(x/ttime->sigma,ttime->mu));
	       else val = 0.0;
	       break;
	     case MODE_PDF: 
	       if (ttime->sigma != 0.0)
		 val = 1-exp(-hb_pow(x/ttime->sigma,ttime->mu));
	       else val = 0.0;
	       break;
		case MODE_1MPDF: 
	       if (ttime->sigma != 0.0)
		 val = exp(-hb_pow(x/ttime->sigma,ttime->mu));
	       else val = 0.0;
	       break;
		}  
	 }
	 else
	 if(ttime->type == NORMAL)
	 {
	   switch(func_mode)
	   {
	     case MODE_PMF: 
	       if (ttime->sigma != 0.0)
		 val = 1.0/(ttime->sigma*sqrt(2.0*PI))*exp(-hb_pow(x-ttime->mu,2.0)/(2.0*hb_pow(ttime->sigma,2.0)));
	       else val = 0.0;
	       break;
	     case MODE_PDF: 
	       if (ttime->sigma != 0.0)
		 val = qromb(NEG_INFINITY, x, ttime->sigma, ttime->mu, CDF_NORMAL);
	       else val = 0.0;
	       break;
		case MODE_1MPDF: 
	       if (ttime->sigma != 0.0)
		 val = 1-qromb(NEG_INFINITY, x, ttime->sigma, ttime->mu, CDF_NORMAL);
	       else val = 0.0;
	       break;
		}  
	 }
	 else
         for( tpow = ttime->power; tpow; tpow = tpow->next )
	 {
            /* look at e list */
            val2 = 0.0;
            for( tex = tpow->expo; tex; tex = tex->next )
	    {
		switch(func_mode)
		{
		case MODE_PMF: 
		    faktor = tex->c;
		    break;
		case MODE_PDF:  /* zurueckrechnen */
		      if((tpow->m == 0) && (tex->lambda == 0))
			{
			  faktor = (tex->b - 1) * (-1);
			}
		      else
			{
			  faktor = tex->b * (-1);
			}
		      break;
		case MODE_1MPDF:
		      faktor = tex->b;
		      break;
		}
		val2 += faktor * exp((-1) * tex->lambda * x);
	    }

	    val += val2 * pot(x,tpow->m);
/*	    printf("%i. potenz  val=%f\n",tpow->m,val); */
	  }

	  /* to get max y */
	  if( val > *ry )
	    *ry = val;


         /* build linked-list */
         new = alloc_new(x,val,CONNECT);
         if( !gd )
            gd = new;
         else
            prev->next = new;
         prev = new;

	 /* tricky part for reaching always end of intervall */
	 if(((x+step) > ttime->x2) && (x < ttime->x2))
	     x = ttime->x2;
	 else
	     x += step;

     }
   }

   if(func_mode == MODE_PDF)
     *ry +=impus;

   return( gd );
}

/*
 * graph_free: free linked list of graph data
 */
struct g_data *graph_free(gd)
struct g_data *gd;
{
   struct g_data *this,*old;

   /* free linked list */
   this = gd;

   while(this!=NULL)
   {
      old = this;
      this = this->next;
      free(old);
   }
   return(NULL);
}
