/* $Header: /home/pdv/timenet/CVS/TimeNET/Interface/src/application/printMenu.c,v 1.2 1994/12/06 18:27:46 joa Exp $ */
/*******************************************************************************
Modul           :       printMenu.c

Purpose         :       Aufbau und Verwaltung des print-popups

Project Director:       Christoph Lindemann TU Berlin

Programmer      :       Christian L"uhe

History         :       release 1: October 1992
*******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <sys/param.h>

#include <X11/IntrinsicP.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/

/*
 * Public include file for widgets we actually use in this file.
 */
#include <X11/Shell.h>     
#include <X11/Xaw/Label.h>    
#include <X11/Xaw/Command.h>    
#include <X11/Xaw/Form.h>   
#include <X11/Xaw/Box.h>    
#include <X11/Xaw/Toggle.h>     
#include <X11/Xaw/MenuButton.h>     
#include <X11/Xaw/SimpleMenu.h>     
#include <X11/Xaw/Dialog.h>    

#include "NetWidget.h"
#include "layout.h"
#include "environment.h"
#include "util.h"

extern void PopUpHelp();
extern void helpDone();

extern char* which();
extern Widget getCurrentNet();

extern XtAppContext app_context;
extern Widget topLevel;
extern void CenterWidget(); /* from evalMenu.c */
extern Widget printButton;  /* from evalMenu.c */

/* forward definition of private functions */
static void DialogDone();
static void DialogCancel();
static void SetChildrenSensitive();
static void resetToDefault();
static void graphicOutput();
static void textOutput();
static void toPrinter();
static void managePrinterName();
static void cancelGetPrinter();
void GivePrinterOk();

static void PopdownCallback();
static void PositionPrintPopup();
static void SetAllBoxesSameWidth ();
static void SetOutputBoxToggleWidth();
static void SetPrintBoxTogglesWidth();
static void SetComBoxTogglesWidth();

#define TOP_DISTANCE 8

typedef enum {nonePaint,printerPaint,ps_filePaint,eps_filePaint} PaintType;
typedef enum {nonePrint,printerPrint,ps_filePrint,tex_filePrint} PrintType;

typedef struct _PPState {
		PaintType paint_type;
		PrintType print_type;
		Boolean save_net_first;
		} PPState;

static PPState ppState;

 Widget
  printOperationPopup;       /* transient shell */
static Widget  mainBox,
        printBox,
            printLabel,
	    graphicToggle,
            textToggle,
        outputBox,
            outputLabel,
            printerToggle,
            fileToggle,
	typeBox,
	    typeLabel,
	    graphicFileLabel,
	    textFileLabel,
		graphicPSToggle, 
		graphicEPSToggle,
		textPSToggle,
		textTeXToggle, 
	comBox,
	    helpButton,
	    okButton,
	    cancelButton;

/* for Printer-Dialog */
static Widget
  getPrinterShell,
  getPrinterDialog,
  getPrinterOk,
  getPrinterCancel;

#define PRINTERLEN 20
static char printerName[PRINTERLEN];

static char trans_toggle[] = "<Btn1Down>,<Btn1Up>: set() notify()";
static XtTranslations toggle_transtable;

static char getPrinterTranslations[] = "  <Key>Return:  givePrinterOk()";

XtActionsRec getPrinterActions[] = {
    {"givePrinterOk", GivePrinterOk}};

/*******************************************************************************
Name:
	initPrintPopup
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
aufgerufen von:
	main.c
--------------------------------------------------------------------------------
Parameter:
	topLevel: toplevel-shell der Applikation
	parent: Elter-widget des print-Buttons
--------------------------------------------------------------------------------
Rueckgabewert:
	keiner
--------------------------------------------------------------------------------
Zweck:
	Aufbau und Initialisierung des print-menues.
	Registrieren der erforderlichen Callbacks.
	Parsieren der Uebersetzungstafeln.
	Registrieren von hier benoetigten Applikationsaktionen.
	Layout des Menues.
*******************************************************************************/

void initPrintPopup(topLevel, parent)
Widget topLevel, parent;
{

  ppState.save_net_first = False;
  ppState.paint_type = ps_filePaint;
  ppState.print_type = ps_filePrint;

  toggle_transtable = XtParseTranslationTable(trans_toggle);

  printOperationPopup = XtVaCreatePopupShell(
			"Print Options",
			transientShellWidgetClass,
			topLevel,
			NULL);
  XtAddCallback(printOperationPopup,XtNpopupCallback,PositionPrintPopup,0);
  XtAddCallback(printOperationPopup,XtNpopupCallback,CenterWidget,NULL);
  XtAddCallback(printOperationPopup,XtNpopdownCallback,PopdownCallback,0);
  mainBox = XtVaCreateManagedWidget(
		"mainBox",
		boxWidgetClass,
		printOperationPopup,
		XtNborderWidth,0,
		XtNhSpace,0,
		XtNvSpace,0,
		NULL);
  printBox = XtVaCreateManagedWidget(
		"printBox",
		formWidgetClass,
		mainBox,
		XtNborderWidth,1,
		XtNhorizDistance,4,
		NULL); 
  printLabel = XtVaCreateManagedWidget(
		"printLabel",
		labelWidgetClass,
		printBox,
                XtNforeground, 0,
                XtNbackground, 1,
                XtNlabel, "Choose the description file to be printed",
		NULL);
  graphicToggle = XtVaCreateManagedWidget(
		"graphicToggle",
		toggleWidgetClass,
		printBox,
		XtNlabel,"graphical description",
		XtNstate,True,
		XtNvertDistance,TOP_DISTANCE,
                XtNfromVert, printLabel,
		NULL);
  XtAddCallback (graphicToggle, XtNcallback, graphicOutput, 0);
  textToggle = XtVaCreateManagedWidget(
		"textToggle",
		toggleWidgetClass,
		printBox,
		XtNlabel,"textual description",
		XtNstate,True,
		XtNvertDistance,TOP_DISTANCE,
		XtNfromHoriz, graphicToggle,
		XtNfromVert, printLabel,
		NULL);
  XtAddCallback (textToggle, XtNcallback, textOutput, 0);
  outputBox = XtVaCreateManagedWidget(
		"ouputBox",
		formWidgetClass,
		mainBox,
		XtNborderWidth,0,
/*		XtNdefaultDistance,0,*/
                XtNfromVert, printBox,
		NULL);
  outputLabel = XtVaCreateManagedWidget(
		"outputLabel",
		labelWidgetClass,
		outputBox,
		XtNborderWidth,0,
		XtNlabel,"Destination",
		XtNvertDistance,TOP_DISTANCE,
		NULL);
  printerToggle = XtVaCreateManagedWidget(
		"printerToggle",
		toggleWidgetClass,
		outputBox,
		XtNlabel,"Printer",
		XtNstate,False,
		XtNfromHoriz,outputLabel,
		NULL);
  XtOverrideTranslations(printerToggle,toggle_transtable);
  XtAddCallback(printerToggle,XtNcallback,toPrinter,NULL);
  fileToggle = XtVaCreateManagedWidget(
		"fileToggle",
		toggleWidgetClass,
		outputBox,
		XtNlabel,"File",
		XtNstate,True,
		XtNfromHoriz,printerToggle,
		NULL);
  XtOverrideTranslations(fileToggle,toggle_transtable);
  XawToggleChangeRadioGroup(fileToggle,printerToggle);
  typeBox = XtVaCreateManagedWidget(
		"typeBox",
		formWidgetClass,
		mainBox,
		NULL);
  typeLabel = XtVaCreateManagedWidget(
		"typeLabel",
		labelWidgetClass,
		typeBox,
		XtNborderWidth,0,
		XtNlabel,"File format",
		XtNvertDistance,TOP_DISTANCE,
		NULL);
  graphicFileLabel = XtVaCreateManagedWidget(
		"graphicFileLabel",
		labelWidgetClass,
		typeBox,
		XtNborderWidth,0,
		XtNlabel,"graphical description: ",
		XtNfromVert,typeLabel,
		NULL);
  textFileLabel = XtVaCreateManagedWidget(
		"textFileLabel",
		labelWidgetClass,
		typeBox,
		XtNborderWidth,0,
		XtNlabel,"textual description: ",
		XtNfromVert,graphicFileLabel,
		NULL);
  graphicPSToggle = XtVaCreateManagedWidget(
		"graphicPSToggle",
		toggleWidgetClass,
		typeBox,
		XtNlabel,"PostScript",
		XtNstate,True,
		XtNfromVert,typeLabel,
		XtNfromHoriz,graphicFileLabel,
		NULL);
  XtOverrideTranslations(graphicPSToggle,toggle_transtable);
  graphicEPSToggle = XtVaCreateManagedWidget(
		"graphicEPSToggle",
		toggleWidgetClass,
		typeBox,
		XtNlabel,"Encaps. PostScript",
		XtNstate,False,
		XtNfromVert,typeLabel,
		XtNfromHoriz,graphicPSToggle,
		NULL);
  XawToggleChangeRadioGroup(graphicPSToggle,graphicEPSToggle);
  XtOverrideTranslations(graphicEPSToggle,toggle_transtable);
  textPSToggle = XtVaCreateManagedWidget(
		"textPSToggle",
		toggleWidgetClass,
		typeBox,
		XtNlabel,"PostScript",
		XtNstate,True,
		XtNfromVert,graphicFileLabel,
		XtNfromHoriz,graphicFileLabel,
		NULL);
  XtOverrideTranslations(textPSToggle,toggle_transtable);
  textTeXToggle = XtVaCreateManagedWidget(
		"textTeXToggle",
		toggleWidgetClass,
		typeBox,
		XtNlabel,"LaTeX source",
		XtNstate,False,
		XtNfromVert,graphicFileLabel,
		XtNfromHoriz,textPSToggle,
		NULL);
  XtOverrideTranslations(textTeXToggle,toggle_transtable);
  XawToggleChangeRadioGroup(textPSToggle,textTeXToggle);
  comBox = XtVaCreateManagedWidget(
		"comBox",
		formWidgetClass,
		mainBox,
		XtNborderWidth,0,
		NULL);
  okButton = XtVaCreateManagedWidget(
                "okButton",
                commandWidgetClass,
                comBox,
                XtNlabel,"start printing",
                NULL);
  XtAddCallback (okButton, XtNcallback, DialogDone, 0);
  helpButton = XtVaCreateManagedWidget(
                "helpButton",
                commandWidgetClass,
                comBox,
                XtNlabel,"help",
		XtNfromHoriz,okButton,
                NULL);
  XtAddCallback(helpButton, XtNcallback, PopUpHelp,"Print.help");
  cancelButton = XtVaCreateManagedWidget(
                "cancelButton",
                commandWidgetClass,
                comBox,
                XtNlabel,"cancel",
		XtNfromHoriz,helpButton,
                NULL);
  XtAddCallback(cancelButton,XtNcallback,DialogCancel,0);  

  getPrinterShell = XtCreatePopupShell ("Setup",
					transientShellWidgetClass,
					topLevel,
					NULL, 0);
  
  printerName[0]='\0';
  printerName[PRINTERLEN-1]='\0';
  if (getenv("PRINTER"))
      strncpy(printerName, getenv("PRINTER"), PRINTERLEN-1);
  
  getPrinterDialog = XtVaCreateManagedWidget ("getPrinterDialog",
					      dialogWidgetClass,
					      getPrinterShell,
					      XtNlabel, "Printer:",
					      XtNvalue, printerName,
					      NULL);
  getPrinterOk = XtVaCreateManagedWidget ("getPrinterOk",
					  commandWidgetClass,
					  getPrinterDialog,
					  XtNlabel, "Ok",
					  NULL);
  XtAddCallback(getPrinterOk, XtNcallback, managePrinterName, NULL);
  getPrinterCancel = XtVaCreateManagedWidget ("getPrinterCancel",
					      commandWidgetClass,
					      getPrinterDialog,
					      XtNlabel, "Cancel",
					      NULL);
  XtAddCallback(getPrinterCancel, XtNcallback, cancelGetPrinter, NULL);

/*  XtAddActions(getPrinterActions, XtNumber(getPrinterActions)); */
/*  XtOverrideTranslations(getPrinterDialog, getPrinterXtTranslations); */
  
  XtInstallAccelerators(getPrinterDialog, getPrinterOk);
}

/*******************************************************************************
Name:
	PopdownCallback	
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	XtNpopdownCallback von printOperationPopup
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
Zweck:
	Popdown des print-menues.
*******************************************************************************/
static void PopdownCallback(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{ 
    helpDone();
}

/*******************************************************************************
Name:
	PositionPrintPopup	
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	XtNpopupCallback von printOperationPopup
--------------------------------------------------------------------------------
Zweck:
	Positionieren des print-menues.
*******************************************************************************/
static void PositionPrintPopup(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
    Dimension height,p_width,p_height,screen_width,screen_height;
    Position r_x,r_y;

    if (!XtIsRealized(printOperationPopup))
      XtRealizeWidget(printOperationPopup);

    {
    Dimension width=0, wbuf, boxwidth, height, border;
    Cardinal n, numChildren, numToggles=0, lastToggle=0;
    WidgetList children;
    int distance;
    
    XtVaGetValues (printBox,
		   XtNchildren, &children,
		   XtNnumChildren, &numChildren,
		   XtNwidth, &boxwidth,
		   NULL);
/* printf("printBox-width: %d\n", boxwidth); */
    for (n=0; n < numChildren; n++) {
	if (XtIsSubclass(children[n], labelWidgetClass)) {
	    numToggles++;
	    XtVaGetValues (children[n],
			   XtNwidth, &wbuf,
			   XtNhorizDistance, &distance, 
			   NULL);
/* printf("toggle-width: %d\n", wbuf); */
	    width = Max (width, wbuf);
	}
    }
    }
}

/*******************************************************************************
Name:
	DialogDone	
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	OK - Button
--------------------------------------------------------------------------------
Parameter:
	unbenutzt
--------------------------------------------------------------------------------
Rueckgabewert:
	keiner
--------------------------------------------------------------------------------
Zweck:
	entsprechend den gesetzten Toggles die scripte Paint/Print starten
	Popdown des print-menues.
*******************************************************************************/

static void DialogDone(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
    char command[1000],
    programpath[MAXPATHLEN],options[100];
    
    String netname;
    String dspnhome;

    String *params;
    Cardinal numparams;
    Boolean picture_modified, state;
    
    XtVaGetValues(getCurrentNet(),
		  XtNmodelName,&netname,
		  NULL);
    if (!strcmp(netname,""))
    {
	XtWarning("There is no model name specified");
	return;
    }
    XtVaGetValues((Widget)getCurrentNet(),
		  XtNpictureModified,&picture_modified,
		  NULL);
    if (picture_modified) 
    {
	params = (String*)XtMalloc(sizeof(String));
	params[0] = XtNewString(netname);
	numparams = 1;
	XtCallActionProc((Widget)getCurrentNet(),
			 "save",NULL,params,numparams);
	XtFree(params[0]);
	XtFree((char *)params);
    }  
    
    /* get current toggles and set appropriate ppStates */
    XtVaGetValues (graphicToggle, XtNstate, &state, NULL);
    if (!state) 
	ppState.paint_type = nonePaint;
    else {
	XtVaGetValues (printerToggle, XtNstate, &state, NULL);
	if (state)
	    ppState.paint_type = printerPaint;
	else {
	    XtVaGetValues (graphicPSToggle, XtNstate, &state, NULL);
	    if (state)
		ppState.paint_type = ps_filePaint;
	    else
		ppState.paint_type = eps_filePaint;
	}
    }
    XtVaGetValues (textToggle, XtNstate, &state, NULL);
    if (!state) 
	ppState.print_type = nonePrint;
    else {
	XtVaGetValues (printerToggle, XtNstate, &state, NULL);
	if (state)
	    ppState.print_type = printerPrint;
	else {
	    XtVaGetValues (textPSToggle, XtNstate, &state, NULL);
	    if (state)
		ppState.print_type = ps_filePrint;
	    else
		ppState.print_type = tex_filePrint;
	}
    }
    
    if ((dspnhome = getenv(DSPNPACKAGE_ENV)) == NULL)
      {
	char msg[200];

	sprintf(msg,
	       "Environment variable %s not set; can't find print program",
	       DSPNPACKAGE_ENV);
	XtWarning(msg);
	return;
      }
    sprintf(programpath,"csh -e -f %s/%s",
	    dspnhome,PRINTPATH);

    /* -----------------PAINT------------------*/
    strcpy(options,"");
    if (ppState.paint_type != nonePaint) {
	Boolean spline;

	switch (ppState.paint_type) {
	  case printerPaint :
	    strcat (options, " -P");
	    strcat (options, XawDialogGetValueString(getPrinterDialog));
	    break;
	  case ps_filePaint :
            /*	    strcat (options, "-ps");  DONT need it - its default */
	    break;
	  case eps_filePaint :
	    strcat (options, " -e");
	    break;
	  default : ;
	}

	XtVaGetValues((Widget)getCurrentNet(),
		      XtNspline, &spline,
		      NULL);

	if (spline) 
	  strcat (options, " -c");

	sprintf(command,"%s/Paint %s %s", programpath, options, netname);
	system(command);
    }
    
    /* -----------------PRINT------------------*/
    strcpy(options,"");
    if (ppState.print_type != nonePrint) {
	switch (ppState.print_type) {
	case printerPrint :
	    strcat (options, "-P");
	    strcat (options, XawDialogGetValueString(getPrinterDialog));
/*	    strcat (options, printerName);*/
	    break;
	case ps_filePrint :
	    /*     strcat (options, "-ps");  DONT need it - its default */
	    if (!which("latex")) {
		XtWarning("Command \"latex\" not found, but needed.\n\
Can't generate Textual Description in PostScript.\n");
		strcat (options, "-t");
		break;
	    }
	    if (!which("dvips")) {
		XtWarning("Command \"dvips\" not found, but needed.\n\
Can't generate Textual Description in PostScript.\n");
		strcat (options, "-t");
		break;
	    }
	    break;
	case tex_filePrint :
	    strcat (options, "-t");
	    break;
	default: ;
	}

	sprintf(command, "%s/Print %s %s", programpath, options, netname);
        system(command);
    }

    
  XtPopdown(printOperationPopup);
  XtSetSensitive(printButton,True);
  resetToDefault();
}

/*******************************************************************************
Name:
	DialogCancel	
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	cancel - Button
--------------------------------------------------------------------------------
Zweck:
	Popdown des print-menues.
*******************************************************************************/
static void DialogCancel(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
    {
    Dimension width=0, wbuf, boxwidth, height, border;
    Cardinal n, numChildren, numToggles=0, lastToggle=0;
    WidgetList children;
    int distance;
    
    XtVaGetValues (printBox,
		   XtNchildren, &children,
		   XtNnumChildren, &numChildren,
		   XtNwidth, &boxwidth,
		   NULL);
/* printf("printBox-width: %d\n", boxwidth); */
    for (n=0; n < numChildren; n++) {
	if (XtIsSubclass(children[n], labelWidgetClass)) {
	    numToggles++;
	    XtVaGetValues (children[n],
			   XtNwidth, &wbuf,
			   XtNhorizDistance, &distance, 
			   NULL);
/* printf("toggle-width: %d\n", wbuf); */
	    width = Max (width, wbuf);
	}
    }
    }

  XtPopdown(printOperationPopup);
  XtSetSensitive(printButton,True);
  resetToDefault();
}

/*******************************************************************************
Name:
	resetToDefault
--------------------------------------------------------------------------------
Funktionstyp:
	private Funktion
--------------------------------------------------------------------------------
aufgerufen von:
	DialogDone, DialogCancel
--------------------------------------------------------------------------------
Zweck:
	Rucksetzen von printState und Toggle-States auf initiale Werte
*******************************************************************************/

static void resetToDefault()
{
    caddr_t data;
    
    XtVaGetValues(fileToggle, XtNradioData, &data, NULL);
    XawToggleSetCurrent(fileToggle, data);
    XtVaGetValues(graphicPSToggle, XtNradioData, &data, NULL);
    XawToggleSetCurrent(graphicPSToggle, data);
    XtVaGetValues(textPSToggle, XtNradioData, &data, NULL);
    XawToggleSetCurrent(textPSToggle, data);

    SetChildrenSensitive (mainBox, True);

    XtVaSetValues(graphicToggle,
		  XtNstate,True,
		  NULL);
    XtVaSetValues(textToggle,
		  XtNstate,True,
		  NULL);

    ppState.save_net_first = False;
    ppState.paint_type = ps_filePaint;
    ppState.print_type = ps_filePrint;
}

/*******************************************************************************
Name:
	SetChildrenSensitive
--------------------------------------------------------------------------------
Funktionstyp:
	private Funktion
--------------------------------------------------------------------------------
aufgerufen von:
	resetToDefault
--------------------------------------------------------------------------------
Zweck:
	setzen der Sensitivitaet fuer alle Kinder
*******************************************************************************/

static void SetChildrenSensitive (w,b)
Widget w;
Boolean b;
{
    Cardinal n, numChildren;
    WidgetList children;
    
    XtSetSensitive (w,b);
    if (XtIsComposite(w)){
	XtVaGetValues (w, 
		       XtNchildren, &children,
		       XtNnumChildren, &numChildren,
		       NULL);
	if (children && numChildren)
	    for (n=0; n < numChildren; n++)
		SetChildrenSensitive (children[n], b);
    }
}

/*******************************************************************************
Name:
	SetAllBoxesSameWidth
--------------------------------------------------------------------------------
Funktionstyp:
	private Funktion
--------------------------------------------------------------------------------
aufgerufen von:
	sizePrintPopup
--------------------------------------------------------------------------------
Zweck:
	macht alle Box's in mainBox gleichweit
*******************************************************************************/
static void SetAllBoxesSameWidth ()
{
    Dimension width=0, wbuf;
    Cardinal n, numChildren;
    WidgetList children;


    /* get number & list of children -> visit them without hard-coding */
    XtVaGetValues (mainBox, 
		   XtNnumChildren, &numChildren, 
		   XtNchildren, &children,
		   NULL);
    if (numChildren==0) 
	XtWarning ("Print-mainBox has NO children");
    else {
	/* get width of each child and overall maximum */
	for (n=0; n < numChildren; n++) {
	    XtVaGetValues (children[n], XtNwidth, &wbuf, NULL);
	    width = Max(width,wbuf);
	}
/* printf("maximum box-width: %d\n", width); */
	/* set each children-width to the maximum */
	for (n=0; n < numChildren; n++) 
	    XtVaSetValues (children[n], XtNwidth, width, NULL);
    }
}

/*******************************************************************************
Name:
	SetPrintBoxToggleWidth
--------------------------------------------------------------------------------
Funktionstyp:
	private Funktion
--------------------------------------------------------------------------------
aufgerufen von:
	sizePrintPopup
--------------------------------------------------------------------------------
Zweck:
	macht alle Toggles in PrintBox gleichweit
*******************************************************************************/
static void SetPrintBoxTogglesWidth()
{
    Dimension width=0, wbuf, boxwidth, height, border;
    Cardinal n, numChildren, numToggles=0, lastToggle=0;
    WidgetList children;
    int distance;
    
    XtVaGetValues (printBox,
		   XtNchildren, &children,
		   XtNnumChildren, &numChildren,
		   XtNwidth, &boxwidth,
		   NULL);
/* printf("printBox-width: %d\n", boxwidth); */
    for (n=0; n < numChildren; n++) {
	if (XtIsSubclass(children[n], toggleWidgetClass)) {
	    numToggles++;
	    XtVaGetValues (children[n],
			   XtNwidth, &wbuf,
			   XtNhorizDistance, &distance, 
			   NULL);
/* printf("toggle-width: %d\n", wbuf); */
	    width = Max (width, wbuf);
	}
    }

    /* following should not be */
    if (numToggles == 0) XtWarning ("There are no Toggles in PrintBox");

    width = (Dimension)((boxwidth-(Dimension)((1+numToggles)*distance))
			/numToggles); 
/* printf("set to: %d\n", width);
printf("distance is: %d\n", distance); */
    /* apply the width to all toggles */
    for (n=0; n < numChildren; n++) {
	if (XtIsSubclass(children[n], toggleWidgetClass)) {
/*	    XtVaSetValues (children[n], XtNmappedWhenManaged, False, NULL);
	    XtVaGetValues (children[n], 
			   XtNheight, &height, 
			   XtNborderWidth, &border,
			   NULL);
	    XtResizeWidget (children[n], width, height, border); 
	    XtVaSetValues(children[n], XtNmappedWhenManaged, True, NULL);
	    if (lastToggle>0)
		XtVaSetValues(children[n],
			      XtNfromHoriz, children[lastToggle],
			      NULL);
	    else 
		XtVaSetValues(children[n],
			      XtNfromHoriz, NULL,
			    XtNleft, XtChainLeft, 
			      NULL);
*/	    lastToggle=n;
	}
	else
	if (XtIsSubclass(children[n], labelWidgetClass)) {
	    XtVaSetValues (children[n], XtNmappedWhenManaged, False, NULL);
	    XtVaGetValues (children[n], 
			   XtNheight, &height, 
			   XtNborderWidth, &border,
			   NULL);
	    XtResizeWidget (children[n], 
			    (width*numToggles+(numToggles-1)*distance), 
			    height, border); 
	    XtVaSetValues (children[n], XtNmappedWhenManaged, True, NULL);
	}
    }
}


/*******************************************************************************
Name:
	SetOutputBoxToggleWidth
--------------------------------------------------------------------------------
Funktionstyp:
	private Funktion
--------------------------------------------------------------------------------
aufgerufen von:
	sizePrintPopup
--------------------------------------------------------------------------------
Zweck:
	macht alle Toggles in outputBox gleichweit
*******************************************************************************/
static void SetOutputBoxToggleWidth()
{
    Dimension width=0, wbuf;
    Cardinal n, numChildren, numToggles=0;
    WidgetList children;
    
    XtVaGetValues (outputBox, 
		   XtNchildren, &children,
		   XtNnumChildren, &numChildren,
		   NULL);
    for (n=0; n < numChildren; n++) {
	if (XtIsSubclass(children[n], toggleWidgetClass)) {
	    numToggles++;
	    XtVaGetValues (children[n], XtNwidth, &wbuf, NULL);
	    width = Max (width, wbuf);
	}
    }

    /* following should not be */
    if (numToggles == 0) XtWarning ("There are no Toggles in outputBox");

    /* apply the width to all buttons */
    for (n=0; n < numChildren; n++) {
	if (XtIsSubclass(children[n], toggleWidgetClass)) 
	    XtVaSetValues (children[n], XtNwidth, width, NULL);
    }
}

/*******************************************************************************
Name:
	SetComBoxToggleWidth
--------------------------------------------------------------------------------
Funktionstyp:
	private Funktion
--------------------------------------------------------------------------------
aufgerufen von:
	sizePrintPopup
--------------------------------------------------------------------------------
Zweck:
	macht alle Toggles in comBox gleichweit
*******************************************************************************/
static void SetComBoxTogglesWidth()
{
    Dimension width=0, wbuf;   
    Cardinal n, numChildren, numCommands=0;
    WidgetList children;
    
    XtVaGetValues (comBox, 
		   XtNchildren, &children,
		   XtNnumChildren, &numChildren,
		   NULL);
    for (n=0; n < numChildren; n++) {
	if (XtIsSubclass(children[n], commandWidgetClass)) {
	    numCommands++;
	    XtVaGetValues (children[n], XtNwidth, &wbuf, NULL);
	    width = Max (width, wbuf);
	}
    }

    /* following should not be */
    if (numCommands == 0) XtWarning ("There are no Commands in ComBox");

    /* apply the width to all buttons */
    for (n=0; n < numChildren; n++) {
	if (XtIsSubclass(children[n], commandWidgetClass)) 
	    XtVaSetValues (children[n], XtNwidth, width, NULL);
    }
}

/*******************************************************************************
Name:
	sizePrintPopup
--------------------------------------------------------------------------------
Funktionstyp:
	public Funktion
--------------------------------------------------------------------------------
aufgerufen von:
        main.c
--------------------------------------------------------------------------------
Zweck:
	macht das PrintPopup-Layout huebsch
*******************************************************************************/
void sizePrintPopup ()
{
  /* dont show the Popup now - but realize it - cause we need the geometry */
  XtVaSetValues (printOperationPopup, XtNmappedWhenManaged, False, NULL);
  XtRealizeWidget (printOperationPopup); 
  
  SetComBoxTogglesWidth();
  SetOutputBoxToggleWidth();
  SetAllBoxesSameWidth();
  SetPrintBoxTogglesWidth();

  XtVaSetValues (printOperationPopup, XtNmappedWhenManaged, True, NULL);
}

void GivePrinterOk(w, event, params, numP)
     Widget w;
     XEvent *event;
     String *params;
     Cardinal numP;
{

    managePrinterName(w, NULL, NULL);

} /* givePrinterOk */

static void managePrinterName(w, client_data, call_data)
     Widget w;
     XtPointer client_data, call_data;
{

    XtPopdown(getPrinterShell);
    XtSetSensitive(printerToggle, True);

    if (strlen(XawDialogGetValueString(getPrinterDialog))==0) {
	/* there's no string in printerName */
	XtVaSetValues(fileToggle,
		      XtNstate, True,
		      NULL); 
    }
    
} /* managePrinterName */

static void cancelGetPrinter(w, client_data, call_data)
     Widget w;
     XtPointer client_data, call_data;
{

    XtPopdown (getPrinterShell);
    XtSetSensitive(printerToggle, True);

    XtVaSetValues(fileToggle,
		  XtNstate, True,
		  NULL);

} /* cancelGetPrinter */

/***************************************************************************
Name:
        toPrinter 
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        printerToggle
--------------------------------------------------------------------------------
Zweck:
        Reaktion auf PrinterToggle
*******************************************************************************/
static void toPrinter()
{
    Boolean state;
    
    XtVaGetValues(printerToggle,
		  XtNstate, &state,
		  NULL);
    if (state) {
	Position x,y;

	XtSetSensitive (printerToggle, False);
	XtSetSensitive (typeBox, False);
	
	/* get coordinates of printerToggle */
	XtTranslateCoords(printerToggle,
			  0, 0,
			  &x, &y);
	/* put getPrinter-Popup there */
	XtVaSetValues(getPrinterShell,
		      XtNx, x,
		      XtNy, y,
		      NULL);
	XtPopup (getPrinterShell, XtGrabExclusive);
    }
    else
	XtSetSensitive (typeBox, True);
}

/***************************************************************************
Name:
        graphicOutput
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        graphicToggle
--------------------------------------------------------------------------------
Zweck:
        Reaktion auf graphicToggle
*******************************************************************************/
static void graphicOutput()
{
    Boolean state, textState;

    XtVaGetValues(graphicToggle, XtNstate, &state, NULL);
    XtVaGetValues(textToggle, XtNstate, &textState, NULL);
    if (state) {
	XtSetSensitive (graphicFileLabel, True);
	XtSetSensitive (graphicPSToggle, True);
	XtSetSensitive (graphicEPSToggle, True);
	XtSetSensitive (outputBox, True);
	XtSetSensitive (okButton, True);
    }
    else {
	XtSetSensitive (graphicFileLabel, False);
	XtSetSensitive (graphicPSToggle, False);
	XtSetSensitive (graphicEPSToggle, False);
	if (!textState) {
	    XtSetSensitive (outputBox, False);
	    XtSetSensitive (okButton, False);
	}
    }
}

/***************************************************************************
Name:
        textOutput
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        textToggle
--------------------------------------------------------------------------------
Zweck:
        Reaktion auf textToggle
*******************************************************************************/
static void textOutput()
{
    Boolean state, graphicState;

    XtVaGetValues(textToggle, XtNstate, &state, NULL);
    XtVaGetValues(graphicToggle, XtNstate, &graphicState, NULL);

    if (state) {
	XtSetSensitive (textFileLabel, True);
	XtSetSensitive (textPSToggle, True);
	XtSetSensitive (textTeXToggle, True);
	XtSetSensitive (outputBox, True);
	XtSetSensitive (okButton, True);
    }
    else {
	XtSetSensitive (textFileLabel, False);
	XtSetSensitive (textPSToggle, False);
	XtSetSensitive (textTeXToggle, False);
	if (!graphicState) {
	    XtSetSensitive (outputBox, False);
	    XtSetSensitive (okButton, False);
	}
    }
}



