/* $Header: /home/pdv/timenet/CVS/TimeNET/Interface/src/application/resultEdit.c,v 1.4 1995/04/18 14:16:20 kkk Exp $ */
/*******************************************************************************
Modul   	:	resultEdit.c

Purpose		: 	
                        Aufbau und Initialisierung des Result-Editors. 
			Bereitsellen von Callbacks zur Reaktion auf 
			Benutzerwuensche

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
                Entbugt July 27 1993  Kolja Koischwitz

*******************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>

#include <X11/Intrinsic.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/

/*
 * Public include file for widgets we actually use in this file.
 */
#include <X11/Shell.h>     
#include <X11/Xaw/Label.h>     
#include <X11/Xaw/Command.h>    
#include <X11/Xaw/Box.h>     
#include <X11/Xaw/Form.h>    
#include <X11/Xaw/Paned.h>   
#include <X11/Xaw/AsciiText.h>    
#include "NetWidget.h"

#include "layout.h"
#include "inputCheck.h"
#include "environment.h"

#define REWARD_EDITOR_WIDTH 600
#define BUTTON_WIDTH 100
#define HORIZ_DIST (REWARD_EDITOR_WIDTH - BUTTON_WIDTH -2)
#define EDIT_TEXT_HEIGHT 50
#define DEFINITION_TEXT_HEIGHT 200
#define RESULT_TEXT_HEIGHT 200
#define LABEL_MIN_SIZE 20
#define LABEL_MAX_SIZE 20

#define EDIT_BUF_SIZE 1000
#define DEFINITION_BUF_SIZE 10000
#define RESULT_BUF_SIZE 10000

#define TEXT_CHANGE_ERROR 0
#define TEXT_CHANGED 1

/* END OF DEFINITION */
#define EOD '\04'
#define DEFINITION_SEPERATOR "\04"

#define RESULT_DEFAULT_STRING "no results available"

extern void PopUpHelp();
extern void helpDone();
extern Widget getCurrentNet();
extern int checkSyntax();
extern int getRidOf();
extern void CenterWidget(); /* from evalMenu.c */
extern XtAppContext app_context;
extern Widget rewardMeasureEntry; /* from evalMenu.c */

typedef enum {VoidCommand,Add,Delete,Change} EditCommand;

typedef struct _EditState {
		Boolean results_available;
		char edit_buf[EDIT_BUF_SIZE];
		char definition_buf [DEFINITION_BUF_SIZE];
		char result_buf	[RESULT_BUF_SIZE];
		} EditState;


static EditState edit_state;
static Dimension popup_width;

Widget
  resultEditor;   
static Widget
  resultMainBox,
    resultCommands,
	resultDeleteButton,
	resultChangeButton,
	resultAddButton,
	resultHelpButton,
	resultQuitButton,
    resultTextBox,
	resultEditLabel,
	resultEdit,
	resultDefinitionLabel,
	resultDefinition,
	resultResultLabel,
	resultResult;

Widget
measureButton;

/* forward definition of callback procedures */
static void ChangeCommandState();
static void InputDone();
/* static void Help(); */
static void Quit();
static void PrepareResultEditor();
static void ClearResultEditor();

/* private functions*/
static void getNameAndDefinition();
static int changeDefinitionText();

/*******************************************************************************
Name:
	init_resultEditor
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
benutzt in:
	main.c
--------------------------------------------------------------------------------
Parameter:
	topLevel: toplevel-Shell der Applikation
--------------------------------------------------------------------------------
Zweck:
	Aufabau und Initialisierung des Result-Editors. Registrierung der
	Callbacks

*******************************************************************************/

void init_resultEditor(app_context,topLevel)
XtAppContext app_context;
Widget topLevel;

{
XtTranslations translations;

  edit_state.results_available = False;

  resultEditor = XtVaCreatePopupShell(
		"resultEditor",
		transientShellWidgetClass,
		topLevel,
		XtNtitle,"Editor for defining reward measures",
		NULL);

  XtAddCallback(resultEditor,XtNpopupCallback,CenterWidget,0);
  XtAddCallback(resultEditor,XtNpopupCallback,PrepareResultEditor,0);
  XtAddCallback(resultEditor,XtNpopdownCallback,ClearResultEditor,0);

  resultMainBox= XtVaCreateManagedWidget(
			"resultMainBox",
			formWidgetClass,
			resultEditor,
			XtNborderWidth,0,
			XtNorientation,XtorientVertical,
			XtNdefaultDistance,0,
			NULL);

/*
  resultMainLabel = XtVaCreateManagedWidget(
			"resultMainLabel",
			labelWidgetClass,
			resultMainBox,
			XtNlabel,"Result Measure Editor",
			XtNjustify,XtJustifyCenter,
			XtNwidth,REWARD_EDITOR_WIDTH,
			XtNforeground,0,
			XtNbackground,1,
			NULL);
*/

  resultCommands = XtVaCreateManagedWidget(
			"resultCommands",
			formWidgetClass,
			resultMainBox,
			XtNborderWidth,0,
/*			XtNfromVert,resultMainLabel, */
			NULL);

  resultAddButton = XtVaCreateManagedWidget(
			"resultAddButton",
			commandWidgetClass,
			resultCommands,
			XtNwidth,BUTTON_WIDTH,
			XtNjustify,XtJustifyCenter,
			XtNlabel,"Add",
			NULL);
  XtAddCallback(resultAddButton,XtNcallback,InputDone,(XtPointer) Add);

  resultChangeButton = XtVaCreateManagedWidget(
			"resultChangeButton",
			commandWidgetClass,
			resultCommands,
			XtNlabel,"Change",
			XtNwidth,BUTTON_WIDTH,
			XtNjustify,XtJustifyCenter,
			XtNfromHoriz,resultAddButton,
			NULL);
  XtAddCallback(resultChangeButton,XtNcallback,InputDone,(XtPointer) Change);

  resultDeleteButton = XtVaCreateManagedWidget(
			"resultDeleteButton",
			commandWidgetClass,
			resultCommands,
			XtNlabel,"Delete",
			XtNwidth,BUTTON_WIDTH,
			XtNjustify,XtJustifyCenter,
			XtNfromHoriz,resultChangeButton,
			NULL);
  XtAddCallback(resultDeleteButton,XtNcallback,InputDone,(XtPointer) Delete);
			
  resultHelpButton = XtVaCreateManagedWidget(
			"resultHelpButton",
			commandWidgetClass,
			resultCommands,
			XtNlabel,"Help",
			XtNwidth,BUTTON_WIDTH,
			XtNjustify,XtJustifyCenter,
			XtNhorizDistance,REWARD_EDITOR_WIDTH-(5*BUTTON_WIDTH+6*4),
			XtNfromHoriz,resultDeleteButton,
			NULL);
  XtAddCallback(resultHelpButton,XtNcallback,PopUpHelp,"Reward.help");

  resultQuitButton = XtVaCreateManagedWidget(
			"resultQuitButton",
			commandWidgetClass,
			resultCommands,
			XtNlabel,"Quit",
			XtNwidth,BUTTON_WIDTH,
			XtNjustify,XtJustifyCenter,
			XtNfromHoriz,resultHelpButton,
			NULL);
  XtAddCallback(resultQuitButton,XtNcallback,Quit,0);
  XtAddCallback(resultQuitButton,XtNcallback,helpDone,0);

  resultTextBox = XtVaCreateManagedWidget(
			"TextBox",
			panedWidgetClass,
			resultMainBox,
			XtNfromVert,resultCommands,
			NULL);

  resultEditLabel = XtVaCreateManagedWidget(
			"resultEditLabel",
			labelWidgetClass,
			resultTextBox,
			XtNlabel,"To Add or change a Reward: <name> : <definition>, to Delete: <name>",
			XtNjustify,XtJustifyLeft,
			XtNwidth,REWARD_EDITOR_WIDTH,
			XtNshowGrip,False,
			XtNmin,LABEL_MIN_SIZE,
			XtNmax,LABEL_MAX_SIZE,
			NULL);

  resultEdit= XtVaCreateManagedWidget(
			"resultEdit",
			asciiTextWidgetClass,
			resultTextBox,
			XtNheight,EDIT_TEXT_HEIGHT,
			XtNwidth,REWARD_EDITOR_WIDTH,
			XtNstring,edit_state.edit_buf,
			XtNlength,EDIT_BUF_SIZE,
			XtNeditType,XawtextEdit,
			XtNscrollVertical,XawtextScrollWhenNeeded,
			XtNautoFill,True,
			NULL);

  resultDefinitionLabel = XtVaCreateManagedWidget(
			"resultDefinitionLabel",
			labelWidgetClass,
			resultTextBox,
			XtNlabel,"Defined reward measures :",
			XtNwidth,REWARD_EDITOR_WIDTH,
			XtNshowGrip,False,
			XtNmin,LABEL_MIN_SIZE,
			XtNmax,LABEL_MAX_SIZE,
			NULL);

  resultDefinition = XtVaCreateManagedWidget(
			"resultDefinition",
			asciiTextWidgetClass,
			resultTextBox,
			XtNheight,DEFINITION_TEXT_HEIGHT,
			XtNwidth,REWARD_EDITOR_WIDTH,
			XtNstring,edit_state.definition_buf,
			XtNlength,DEFINITION_BUF_SIZE,
			XtNdisplayCaret,False,
			XtNdisplayNonprinting,False,
			XtNscrollVertical,XawtextScrollWhenNeeded,
			XtNwrap,XawtextWrapWord,
			NULL);

  resultResultLabel = XtVaCreateManagedWidget(
			"resultResultLabel",
			labelWidgetClass,
			resultTextBox,
			XtNlabel,"Available results:",
		/*	XtNlabel,"no results available",*/
			XtNwidth,REWARD_EDITOR_WIDTH,
			XtNshowGrip,False,
			XtNmin,LABEL_MIN_SIZE,
			XtNmax,LABEL_MAX_SIZE,
			NULL);

  strcpy(edit_state.result_buf,RESULT_DEFAULT_STRING);

  resultResult = XtVaCreateManagedWidget(
			"resultResult",
			asciiTextWidgetClass,
			resultTextBox,
			XtNheight,RESULT_TEXT_HEIGHT,
			XtNwidth,REWARD_EDITOR_WIDTH,
			XtNstring,edit_state.result_buf,
			XtNlength,RESULT_BUF_SIZE,
			XtNdisplayCaret,False,
			XtNscrollVertical,XawtextScrollWhenNeeded,
			XtNwrap,XawtextWrapWord,
			NULL);

  XtVaGetValues(resultResultLabel,
		XtNwidth,&popup_width,
		NULL);

}

/*******************************************************************************
Name:
	Quit
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	XtNcallback des Kommandobuttons 'resultQuitButton'
--------------------------------------------------------------------------------
Zweck:
	Popdown des Result-Editors
*******************************************************************************/

static void Quit(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
    char measure_path[MAXPATHLEN];
    char *modelpath,*name;


#ifdef debug
printf("resultEditor: Quit \n");
#endif

    XtPopdown(resultEditor);
	XtSetSensitive(rewardMeasureEntry,True);

    if ((modelpath = getenv(MODELDIR_ENV)) == NULL)
      {
	char msg[200];

	sprintf(msg,
		"Environment variable %s not set",MODELDIR_ENV);
	XtWarning(msg);
 	return;
      }
    XtVaGetValues(getCurrentNet(),
		  XtNmodelName,&name,
		  NULL);
    sprintf(measure_path,"%s/%s%s",modelpath,name,MSR_EXTENSION);
    unlink(measure_path);
}

/*******************************************************************************
Name:
	InputDone
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
angemeldet fuer:   Add,Change und Delete Button
--------------------------------------------------------------------------------
Zweck:
	Aendern der angezeigten Resultmeasures entsprechend der gewuenschten
	Aktion (Add,Delete,Change).
	Aufruf der entsprechenden Aktion der aktuellen Instanz der Klasse
	NetWidget
*******************************************************************************/

static void InputDone(w, client_data, call_data)
    Widget w;
	XtPointer client_data, call_data;
{
    String newparams[2],buf,name,definition,old_definition;
    Cardinal numnewparams;
    XawTextBlock tblock;
    int i,textlen, command;

    String *old_rewards;
    Cardinal num_old_rewards;

#ifdef debug
    printf("resultEditor: InputDone \n");
#endif
    command = (int)(long) client_data;
    XtVaGetValues(resultEdit,
		  XtNstring,&buf,
		  NULL);
    textlen = strlen(buf);
    if ( textlen == 0 )
	return;
    getNameAndDefinition(buf,&name,&definition);
    if (name == NULL)
    {
	XtWarning("Can't find resultname");
	XtFree(name);
	XtFree(definition);
	return;
    }
    if (command != Delete && (definition == NULL || *definition == '\0'))
    {
	XtWarning("Can't find resultdefinition");
	XtFree(name);
	XtFree(definition);
	return;
    }

  getResults(getCurrentNet(),&old_rewards,&num_old_rewards);
  for(old_definition=NULL,i=0;i< num_old_rewards;i+=2) 
  {
	if(strcmp(name,old_rewards[i])==0)
	  old_definition = old_rewards[i+1];
	else
	  XtFree(old_rewards[i+1]);
	XtFree(old_rewards[i]);
  }
  XtFree((String)old_rewards);
    
  if (command == Add && old_definition != NULL)
  {
	XtWarning("This result measure already exists: ignored");
	XtFree(name);
	XtFree(definition);
	return;
  }
    
  if (command != Add && old_definition == NULL)
  {
	XtWarning("This result measure does not exist!");
	XtFree(name);
	XtFree(definition);
	return;
  }
  
  numnewparams = 2;
  newparams[0] = XtNewString(name);
  newparams[1] = XtNewString(definition);
  switch(command)
  {
  case Add:
	XtCallActionProc(getCurrentNet(),"addResult",NULL,newparams,numnewparams);
	break;
  case Change:
	XtCallActionProc(getCurrentNet(),"changeResult",NULL,newparams,numnewparams);
	break;
  case Delete:
	XtCallActionProc(getCurrentNet(),"deleteResult",NULL,newparams,numnewparams);
	break;
  }

  XtFree(newparams[0]);
  XtFree(newparams[1]);
  newparams[0] = XtNewString(name);
  newparams[1] = old_definition;
  /* check syntax of reward definition */
  if (command != Delete && checkSyntax() != 0)
  {
    switch(command) /* Sache wieder rueckgaengig machen */
    {
    case Change:
  	  XtCallActionProc(getCurrentNet(),"changeResult",NULL,newparams,numnewparams);
	  break;
    case Add:
  	  XtCallActionProc(getCurrentNet(),"deleteResult",NULL,newparams,numnewparams);
	  break;
    }
	return;
  }
  changeDefinitionText(name,definition,command);
  XtFree(name);
  XtFree(definition);
  XtFree(newparams[0]);
  if(old_definition != NULL) XtFree(old_definition);
  
  
  tblock.firstPos = 0;
  tblock.length = 0;
  tblock.ptr = "";
  tblock.format = FMT8BIT;
  XawTextReplace(resultEdit,0,(XawTextPosition)textlen,&tblock);
  XtVaSetValues(resultResult,
		XtNtype,XawAsciiString,
		XtNstring,edit_state.result_buf,
		NULL);
  XawTextDisplay(resultEdit);
  XawTextDisplay(resultResult);
  XawTextDisplay(resultDefinition);
}

/*******************************************************************************
Name:
	PrepareResultEditor
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	XtpopupCallback von resultEditor
--------------------------------------------------------------------------------
Zweck:
	Ermitteln der fuer die aktuellen Instanz der Klasse NetWidget definierten
	Resultmeasures und deren Ausgabe im entsprechenden Textfeld.
	Ausgabe von erhaltenen Resultaten im entsprechenden Textfeld.

*******************************************************************************/

static void PrepareResultEditor(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
String * results;
Cardinal numresults;
char res_path[MAXPATHLEN],def_path[MAXPATHLEN],
     measure_path[MAXPATHLEN], command[MAXPATHLEN+256],
     *modelpath,*name;
struct stat s_res_buf, s_def_buf, s_measure_buf,s_buf;
int i;
Boolean modified,env_found;

#ifdef debug
printf("resultEditor: PrepareResultEditor  \n");
#endif

  getResults(getCurrentNet(),&results,&numresults);

  /* rewrite definition field */
  for(i=0;i< numresults;i+=2) 
  {
	changeDefinitionText(results[i],results[i+1],Add);
	XtFree(results[i]);
	XtFree(results[i+1]);
  }
  XtFree((String)results);

  env_found = True;
  if ((modelpath = getenv(MODELDIR_ENV)) == NULL)
    {
      char msg[200];
      
      sprintf(msg,
	      "Environment variable %s not set",MODELDIR_ENV);
      XtWarning(msg);
      env_found = False;
    }

  XtVaGetValues(getCurrentNet(),
		XtNmodelName,&name,
		NULL);

  sprintf(res_path,"%s/%s%s",modelpath,name,RESULT_EXTENSION);
  sprintf(measure_path,"%s/%s%s",modelpath,name,MSR_EXTENSION);

  if (env_found && stat(res_path,&s_buf) != -1)
    {
      sprintf(command,
	      "awk '$0 == %cUSER DEFINED REWARD MEASURES%c {next} $0 == %cTHROUGHPUT RESULTS OF TIMED TRANSITIONS%c { exit } {print}' %s > %s", 
	      '"','"','"','"', res_path, measure_path);

      system(command);
    }
  sprintf(def_path,"%s/%s%s",modelpath,name,NET_EXTENSION);

  XtVaGetValues(getCurrentNet(),
		XtNnetModified,&modified,
		NULL);
  XawTextDisableRedisplay(resultResult);
  if (env_found &&
      !modified &&
      stat(res_path,&s_res_buf)==0 &&
      stat(def_path,&s_def_buf)==0 &&
      stat(measure_path,&s_measure_buf)==0 &&
      s_def_buf.st_mtime < s_res_buf.st_mtime &&
      s_res_buf.st_mtime < s_measure_buf.st_mtime)

  {
      XtVaSetValues(resultResult,
		    XtNstring,measure_path,
		    XtNtype,XawAsciiFile,
		    NULL);
  }
  else {
	strcpy(edit_state.result_buf,RESULT_DEFAULT_STRING);
	XtVaSetValues(resultResult,
			XtNstring,edit_state.result_buf,
			XtNtype,XawAsciiString,
			NULL);
  }
  XawTextEnableRedisplay(resultResult);

}

/*******************************************************************************
Name:
	ClearResultEditor
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	XtNpopdownCallback von 'resultEditor'
--------------------------------------------------------------------------------
Zweck:
	Loeschen der Texteintraege in allen Textabschnitten
*******************************************************************************/

static void ClearResultEditor(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
String buf;
XawTextBlock tblock;
XButtonEvent event;

#ifdef debug
printf("resultEditor: ClearResultEditor  \n");
#endif
  
  XtSetSensitive(resultEditor,True);
  /* why doesn't Xt call unset ??? */
  XtCallActionProc(resultQuitButton,"unset", (XEvent*)&event,NULL,0);
  /* clear definition field */
  tblock.firstPos = 0;
  tblock.length = 0;
  tblock.ptr = "";
  tblock.format = FMT8BIT;
  XtVaGetValues(resultDefinition,
		XtNstring,&buf,
		NULL);
  XtVaSetValues(resultDefinition,
		XtNeditType,XawtextEdit,
		NULL);
  XawTextReplace(resultDefinition,0,(XawTextPosition)strlen(buf),&tblock);
  XtVaSetValues(resultDefinition,
		XtNeditType,XawtextRead,
		NULL);
}


/*******************************************************************************
Name:
	changeDefinitionText
--------------------------------------------------------------------------------
Funktionstyp:
	private function
--------------------------------------------------------------------------------
benutzt in:
	PrepareResultEditor(), InputDone()
--------------------------------------------------------------------------------
Parameter:
	rew_name:	name des resultmesures
	rew_definition: definition des resultmesures
	cammand:	gewuenschte Aktion
--------------------------------------------------------------------------------
Zweck:
	Anzeigen einer neuen Definition, Aenderung oder Loeschung resultmesures
*******************************************************************************/

static int changeDefinitionText(rew_name,rew_definition,command)
String rew_name,rew_definition;
EditCommand command;
{
XawTextBlock tblock,repblock;
XawTextPosition tpos,tpos_2,textlen;
String tbuf,definition,nkey;
int ret =  TEXT_CHANGED; 

#ifdef debug
printf("resultEditor:  changeDefinitionText \n");
#endif

  nkey = definition = NULL;

  if (rew_name == NULL || (rew_definition == NULL && command != Delete))
	return TEXT_CHANGE_ERROR;
  XtVaGetValues(resultDefinition,
		XtNstring,&tbuf,
		NULL);
  textlen = (XawTextPosition)strlen(tbuf);
  /*look for string "<rew_name>:" */
  XawTextSetInsertionPoint(resultDefinition,0);
  nkey = XtMalloc((strlen(rew_name)+2)*sizeof(char));
  sprintf(nkey,"%s:",rew_name);
  tblock.firstPos = 0;
  tblock.length = strlen(nkey);
  tblock.ptr = nkey;
  tblock.format = FMT8BIT;
  tpos = XawTextSearch(resultDefinition,XawsdRight,&tblock);
  XtVaSetValues(resultDefinition,
		XtNeditType,XawtextEdit,
		NULL);
  switch(command) { 
  case Add:
/* search insertion point , alphabetically order 
  	name=strtok(tbuf,":");
	while(name != NULL && strcmp(name,rew_name) < 0) {
	     skip definition until EOD 
	    name = strtok(NULL,DEFINITION_SEPERATOR);
	     get next name part 
	    if (name != NULL)
	    	name = strtok(NULL,":");
 	     skip last newline 
	    if (name != NULL)
	    	name++;
	}
*/
 	definition =
		XtMalloc((strlen(rew_name)+strlen(rew_definition)+6)*sizeof(char));
	sprintf(definition,"%s: %s\n%c\n",rew_name,rew_definition,EOD);
	repblock.firstPos = 0;
	repblock.length = strlen(definition);
 	repblock.ptr = definition;
	repblock.format = FMT8BIT;   
/* alphabetische Ordnung implementiert, falls .RESULT alphabetisch geordnet wird */
/*
	if (name != NULL) {
	    tblock.firstPos = 0;
	    tblock.length = strlen(name);
 	    tblock.ptr = name;
	    tblock.format = FMT8BIT;
	    inspos = XawTextSearch(resultDefinition,XawsdRight,&tblock);
	    XawTextReplace(resultDefinition,inspos,inspos,&repblock);
	}
	else 
	}
*/
	    XawTextReplace(resultDefinition,textlen,textlen,&repblock);
	    XawTextSetInsertionPoint(resultDefinition,textlen);
	/* } */
 	break;
  case Change:
	if (tpos == XawTextSearchError) {
	    XtWarning("This result doesn't exist");
	    ret = TEXT_CHANGE_ERROR;
 	    break;
	}
	tblock.firstPos = 0;
	tblock.length = 1;
 	tblock.ptr = ":" ;
	tblock.format = FMT8BIT;
	XawTextSetInsertionPoint(resultDefinition,tpos);
	tpos = XawTextSearch(resultDefinition,XawsdRight,&tblock);
	tblock.firstPos = 0;
	tblock.length = 1;
 	tblock.ptr = DEFINITION_SEPERATOR ;
	tblock.format = FMT8BIT;
	XawTextSetInsertionPoint(resultDefinition,tpos);
	tpos_2 = XawTextSearch(resultDefinition,XawsdRight,&tblock);
 	definition =
		XtMalloc((strlen(rew_definition)+5)*sizeof(char));
	sprintf(definition," %s\n%c\n",rew_definition,EOD);
	tblock.firstPos = 0;
	tblock.length = strlen(definition);
 	tblock.ptr = definition;
	tblock.format = FMT8BIT;
	
	XawTextReplace(resultDefinition,tpos+1,tpos_2+2,&tblock) ;
	XawTextSetInsertionPoint(resultDefinition,tpos+1);
	break;
  case Delete:
	if (tpos == XawTextSearchError) {
	    XtWarning("This result doesn't exist");
	    ret = TEXT_CHANGE_ERROR;
 	    break;
	}
	tblock.firstPos = 0;
	tblock.length = 1;
 	tblock.ptr = DEFINITION_SEPERATOR ;
	tblock.format = FMT8BIT;
	XawTextSetInsertionPoint(resultDefinition,tpos);
	tpos_2 = XawTextSearch(resultDefinition,XawsdRight,&tblock);
	tblock.firstPos = 0;
	tblock.length = 0;
 	tblock.ptr = "";
	tblock.format = FMT8BIT;
	XawTextReplace(resultDefinition,tpos,tpos_2+2,&tblock);
	XawTextSetInsertionPoint(resultDefinition,tpos);
	break;
  default: ret = TEXT_CHANGE_ERROR; break;
  };
  XtVaSetValues(resultDefinition,
		XtNeditType,XawtextRead,
		NULL);
  XtFree(nkey);
  XtFree(definition);
  return ret;
}

/**************************************************************************

NOCH NICHT KOMPLETT

************************************************************************/

/* aus str das erste Wort isolieren */
static void getNameAndDefinition(str,name,definition)
String str,*name,*definition;
{
String n_ptr,d_ptr,d_ptr_end;
String n;
static char seperator[] = ":";

  n_ptr = strtok(str,seperator);
  if (n_ptr == NULL)
    {
      *name = NULL;
      *definition = NULL;
      return;
    }
  n = XtCalloc(strlen(n_ptr)+1,sizeof(char));
  sscanf(n_ptr,"%s",n);
  if (isalpha((int)n[0])||n[0]=='_')
	*name = XtNewString(n);
  else
  {
   	*name = NULL;
  }
  XtFree(n);
  
  d_ptr = strtok(NULL,"\0");
  if (d_ptr != NULL) 
    {
      /* "uberlesen f"uhrender leerzeichen */
      for ( ; 
	   *d_ptr != '\0' &&
	   (*d_ptr == ' ' || *d_ptr == '\n' || *d_ptr == '\t' );
	   d_ptr++)
	;
      /* "uberlesen folgender leerzeichen */
      for (d_ptr_end = d_ptr + strlen(d_ptr) - 1;
	   d_ptr_end != d_ptr &&
	   (*d_ptr_end == ' ' || *d_ptr_end == '\n' || *d_ptr_end == '\t' );
	   d_ptr_end--)
	;
      *(d_ptr_end+1)='\0';
      *definition = XtNewString(d_ptr);
    }
  else
    *definition = NULL;

}

