/*******************************************************************************
Modul   	:	showResults.

Purpose		: 	Bereitstellen von Callbackfunktionen zur Anzeige der 
                        Tokenverteilungen der einzelnen Stellen, der Raten 
			und des Berechneten Durchsatzes der Transitionen.

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/

#include <stdio.h>

#include <X11/Intrinsic.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/

#include <X11/Shell.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/AsciiText.h>

#include "NetWidget.h"

extern Widget topLevel;
extern Widget getCurrentNet();

static String* distribution_buf;
static Cardinal distribution_buf_size = 0;
static Cardinal distribution_buf_num_slots = 0;

static void RemoveFromDistributionBuf();
static Cardinal CopyToDistributionBuf();
static void Done();

#define TEXT_HEIGHT 100
#define TEXT_HEIGHT2 170
#define TEXT_WIDTH  400
#define TEXT_WIDTH2  430

#define NUM_SLOTS 5

/*******************************************************************************
Name:
	ShowTokenDistribution
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	XtNtokenDistributionCallback von NetWidget
--------------------------------------------------------------------------------
Parameter:
	w: Instanz des NetWidgets
	client_data: nicht benutzt
	call_data: Zeiger auf Struktur NWshowTokenDistributionInfo
--------------------------------------------------------------------------------
Zweck:
	Aufbau des Popups mit Hilfe der Information aus call_data
*******************************************************************************/

void ShowTokenDistribution(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
NWshowTokenDistributionInfo *info = (NWshowTokenDistributionInfo*)call_data;
Position x,y,r_x,r_y;
Dimension width,label_width,doneButton_width;
Widget shell,form,doneButton,label,text;
char shell_name[30],label_string[200],format[80];
Cardinal index;
Cardinal precision;

  XtVaGetValues (w,
		 XtNprecision, &precision,
		 NULL);

  if (info -> status == NO_RESULTS_AVAILABLE) 
	sprintf(shell_name,"NO RESULTS");
  else
    	sprintf(shell_name,"%s",info->place_name);

  if ((shell = XtNameToWidget(topLevel,shell_name)) != NULL)
	return;

  shell = XtVaCreatePopupShell(
			shell_name,
			transientShellWidgetClass,
			topLevel,
			NULL);

  index = CopyToDistributionBuf(info->token_distribution);
  XtAddCallback(shell,XtNpopdownCallback,RemoveFromDistributionBuf,
  	(XtPointer)((long)index));


  form = XtVaCreateManagedWidget(
			"token_distribution_box",
			formWidgetClass,
			shell,
			XtNdefaultDistance,0,
			NULL);
  if (info->status != NO_RESULTS_AVAILABLE)
    { 
      sprintf(format, "Place %%s :  mean %%.%df", precision);
      sprintf(label_string,format,info->place_name,info->mean);
    }
  else
    sprintf(label_string,"Place %s                  ",info->place_name);

  label = XtVaCreateManagedWidget(
			"token_distribution_label",
			labelWidgetClass,
			form,
			XtNlabel,label_string,
			XtNjustify,XtJustifyLeft,
			XtNtop,XtChainTop,
			NULL);

  doneButton = XtVaCreateManagedWidget(
			"token_distribution_done",
			commandWidgetClass,
			form,
			XtNlabel,"done",
			XtNfromHoriz,label,
			XtNtop,XtChainTop,
			NULL);

  if (info -> status == NO_RESULTS_AVAILABLE) 
	XtAddCallback(doneButton,
		      XtNcallback,Done,(XtPointer)XtNewString("NO RESULTS"));
  else
  	XtAddCallback(doneButton,
		      XtNcallback,Done,(XtPointer)XtNewString(info->place_name));

  XtVaGetValues(label,
		XtNwidth,&label_width,
		NULL);

  XtVaGetValues(doneButton,
		XtNwidth,&doneButton_width,
		NULL);

  width = label_width + doneButton_width;

  text = XtVaCreateManagedWidget(
			"token_distribution_text",
			asciiTextWidgetClass,
			form,
			XtNstring,distribution_buf[index],
			XtNlength,strlen(distribution_buf[index]),
			XtNdisplayCaret,False,
			XtNheight,TEXT_HEIGHT,
			XtNwidth,width +2 ,
			XtNscrollVertical,XawtextScrollWhenNeeded,
			XtNfromVert,label,
			NULL);

  x = (Position)info->e_x - (Position)width/2;
  y = (Position)info->e_y - TEXT_HEIGHT/2;
  XtTranslateCoords(getCurrentNet(),x,y,&r_x,&r_y);

  {
    Dimension p_width,p_height,screen_width,screen_height;

    if (!XtIsRealized(shell))
      XtRealizeWidget(shell);
    
    XtVaGetValues(shell,
		  XtNwidth,&p_width,
		  XtNheight,&p_height,
		  NULL);
    
    screen_width = (Dimension)WidthOfScreen(XtScreen(shell));
    screen_height = (Dimension)HeightOfScreen(XtScreen(shell));
    
    if (r_y + (Position)p_height > (Position)screen_height)
      r_y = (Position)screen_height - (Position)p_height;
    if (r_y < 0)
      r_y = 0;
    
    if (r_x < 0)
      r_x = 0;
    else
      if (r_x + (Position)p_width > (Position)screen_width )
	r_x = (Position)screen_width - (Position)p_width;
    
    XtMoveWidget(shell, r_x, r_y);
  }

  XtPopup(shell,XtGrabNone);

}

void DestroyDistributionBuf()
{
Cardinal i;

  for (i=0;i<distribution_buf_size;i++)
	XtFree(distribution_buf[i]);
  XtFree((String)distribution_buf);
}

/*******************************************************************************
Name:
	ShowTransitionResults
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	XtNtransResultCallback von NetWidget
--------------------------------------------------------------------------------
Parameter:
	w: Instanz des NetWidgets
	client_data: nicht benutzt
	call_data: Zeiger auf Struktur NWshowTransResultInfo
--------------------------------------------------------------------------------
Zweck:
	Aufbau des Popups mit Hilfe der Information aus call_data
*******************************************************************************/

void ShowTransitionResults(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
NWshowTransitionResultsInfo *info = (NWshowTransitionResultsInfo*)call_data;
Position x,y,r_x,r_y;
Dimension width,doneButton_width;
Widget shell,form,doneButton,label,text,result_label;
char shell_name[30],label_string[200], format[80];
Cardinal precision;

  XtVaGetValues(w,
		XtNprecision, &precision,
		NULL);

  sprintf(shell_name,"%s",info->transition_name);

  if ((shell = XtNameToWidget(topLevel,shell_name)) != NULL)
	return;

  shell = XtVaCreatePopupShell(
			shell_name,
			transientShellWidgetClass,
			topLevel,
			NULL);

  form = XtVaCreateManagedWidget(
			"transition_result_box",
			formWidgetClass,
			shell,
			XtNdefaultDistance,0,
			NULL);

  sprintf(label_string,"Transition %s ",info->transition_name);
  label = XtVaCreateManagedWidget(
			"transition_result_label",
			labelWidgetClass,
			form,
			XtNlabel,label_string,
			XtNjustify,XtJustifyLeft,
			NULL);

  doneButton = XtVaCreateManagedWidget(
			"transition_result_done",
			commandWidgetClass,
			form,
			XtNlabel,"done",
			XtNfromHoriz,label,
			NULL);

  XtAddCallback(doneButton,XtNcallback,Done,(XtPointer)XtNewString(info->transition_name));

  XtVaGetValues(doneButton,
		XtNwidth,&doneButton_width,
		NULL);

  width = TEXT_WIDTH - doneButton_width - 2;

  XtVaSetValues(label,
		XtNwidth,width,
		NULL);

  text = XtVaCreateManagedWidget(
			"transition_result_text",
			asciiTextWidgetClass,
			form,
			XtNstring,info->firing_rate,
			XtNlength,strlen(info->firing_rate),
			XtNdisplayCaret,False,
			XtNheight,TEXT_HEIGHT2,
			XtNwidth,TEXT_WIDTH2 ,
			XtNscrollVertical,XawtextScrollWhenNeeded,
			XtNfromVert,label,
			NULL);

  if (info->status == NO_RESULTS_AVAILABLE) 
  	sprintf(label_string,"no throughput results available");
  else {
      sprintf(format, "Throughput: %%.%df", precision);
      sprintf(label_string,format,info->throughput);
  }
  result_label = XtVaCreateManagedWidget(
			"transition_throughput_label",
			labelWidgetClass,
			form,
			XtNlabel,label_string,
			XtNfromVert,text,
			XtNwidth,TEXT_WIDTH ,
			XtNjustify,XtJustifyLeft,
			NULL);

  x = (Position)info->e_x - (Position)width/2;
  y = (Position)info->e_y - TEXT_HEIGHT/2;
  XtTranslateCoords(getCurrentNet(),x,y,&r_x,&r_y);
  
  {
    Dimension p_width,p_height,screen_width,screen_height;

    if (!XtIsRealized(shell))
      XtRealizeWidget(shell);
    
    XtVaGetValues(shell,
		  XtNwidth,&p_width,
		  XtNheight,&p_height,
		  NULL);
    
    screen_width = (Dimension)WidthOfScreen(XtScreen(shell));
    screen_height = (Dimension)HeightOfScreen(XtScreen(shell));
    
    if (r_y + (Position)p_height > (Position)screen_height)
      r_y = (Position)screen_height - (Position)p_height;
    if (r_y < 0)
      r_y = 0;
    
    if (r_x < 0)
      r_x = 0;
    else
      if (r_x + (Position)p_width > (Position)screen_width )
	r_x = (Position)screen_width - (Position)p_width;
    
    XtMoveWidget(shell, r_x, r_y);
  }

  XtPopup(shell,XtGrabNone);
}

/*******************************************************************************
Name:
	ShowArcMulitplicity
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	XtNarcShowCallback von NetWidget
--------------------------------------------------------------------------------
Parameter:
	w: Instanz des NetWidgets
	client_data: nicht benutzt
	call_data: Zeiger auf Struktur NWshowArcInfo
--------------------------------------------------------------------------------
Zweck:
	Aufbau des Popups mit Hilfe der Information aus call_data
*******************************************************************************/

void ShowArcMulitplicity(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
NWshowArcInfo *info = (NWshowArcInfo*)call_data;
Position x,y,r_x,r_y;
Dimension width,doneButton_width;
Widget shell,form,doneButton,label,text;


  if ((shell = XtNameToWidget(topLevel,info->arc_name)) != NULL)
	return;

  shell = XtVaCreatePopupShell(info->arc_name,
			       transientShellWidgetClass,
			       topLevel,
			       XtNtitle, "arc multiplicity",
			       NULL);

  form = XtVaCreateManagedWidget(
			"arc_show_box",
			formWidgetClass,
			shell,
			XtNdefaultDistance,0,
			NULL);

  label = XtVaCreateManagedWidget(
			"arc_show_label",
			labelWidgetClass,
			form,
			XtNlabel,info->titel,
			XtNjustify,XtJustifyLeft,
			NULL);

  doneButton = XtVaCreateManagedWidget(
			"arc_show_done",
			commandWidgetClass,
			form,
			XtNlabel,"done",
			XtNfromHoriz,label,
			NULL);

  XtAddCallback(doneButton,XtNcallback,Done,XtNewString(info->arc_name));

  XtVaGetValues(doneButton,
		XtNwidth,&doneButton_width,
		NULL);

  width = TEXT_WIDTH - doneButton_width - 2;

  XtVaSetValues(label,
		XtNwidth,width,
		NULL);

  text = XtVaCreateManagedWidget(
			"arc_show_text",
			asciiTextWidgetClass,
			form,
			XtNstring,info->text,
			XtNlength,strlen(info->text),
			XtNdisplayCaret,False,
			XtNheight,TEXT_HEIGHT,
			XtNwidth,TEXT_WIDTH ,
			XtNscrollVertical,XawtextScrollWhenNeeded,
			XtNfromVert,label,
			NULL);

  x = (Position)info->e_x - (Position)width/2;
  y = (Position)info->e_y - TEXT_HEIGHT/2;
  XtTranslateCoords(getCurrentNet(),x,y,&r_x,&r_y);
  {
    Dimension p_width,p_height,screen_width,screen_height;

    if (!XtIsRealized(shell))
      XtRealizeWidget(shell);
    
    XtVaGetValues(shell,
		  XtNwidth,&p_width,
		  XtNheight,&p_height,
		  NULL);
    
    screen_width = (Dimension)WidthOfScreen(XtScreen(shell));
    screen_height = (Dimension)HeightOfScreen(XtScreen(shell));
    
    if (r_y + (Position)p_height > (Position)screen_height)
      r_y = (Position)screen_height - (Position)p_height;
    if (r_y < 0)
      r_y = 0;
    
    if (r_x < 0)
      r_x = 0;
    else
      if (r_x + (Position)p_width > (Position)screen_width )
	r_x = (Position)screen_width - (Position)p_width;
    
    XtMoveWidget(shell, r_x, r_y);
  }
  XtPopup(shell,XtGrabNone);
}

/*******************************************************************************
Name:
	Done
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	XtNcallback des command-buttons 'done' der jeweiligen Popups
--------------------------------------------------------------------------------
Parameter:
	w: Instanz des command-buttons (nicht benutzt)
	client_data: Zeiger auf den Namen des kreierten Popups
	call_data: nicht benutzt
--------------------------------------------------------------------------------
Zweck:
	Abbau des Popups dessen Name ueber call_data mitgeteilt wird
*******************************************************************************/

static void Done(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
Widget shell;

  shell = XtNameToWidget(topLevel,(String)client_data);
  XtPopdown(shell);
  XtFree(client_data);
  XtDestroyWidget(shell);

}

/******************************************************************

NEUE FEATURES, NOCH UNDOKUMENTIERT

*****************************************************************/
static Cardinal CopyToDistributionBuf(distribution_string)
String distribution_string;

{
  Cardinal i;

  if (distribution_buf_num_slots == 0)
  {
	distribution_buf = 
	  (String*)XtRealloc((String)distribution_buf,
			     (distribution_buf_size + NUM_SLOTS)*sizeof(String));
	for (i = distribution_buf_size; 
	     i< distribution_buf_size + NUM_SLOTS;
	     i++)
	  {
	    distribution_buf[i] = NULL;
	  }
	distribution_buf_num_slots += NUM_SLOTS;
	distribution_buf_size += NUM_SLOTS;
  }
  distribution_buf[distribution_buf_size - distribution_buf_num_slots] 
		= XtNewString(distribution_string);
  distribution_buf_num_slots--;
  return distribution_buf_size - distribution_buf_num_slots - 1;
}


static void RemoveFromDistributionBuf(w,client_data,call_data)
Widget w;
XtPointer client_data,call_data;

{
Cardinal index = (Cardinal)((long)client_data);
Cardinal i;

#ifdef debug
	printf("RemoveFromDistributionBuf\n");
#endif

   XtFree(distribution_buf[index]);
   distribution_buf_num_slots++;
   for (i=index;i<distribution_buf_size-distribution_buf_num_slots;i++)
	distribution_buf[i] = distribution_buf[i+1];
}
