/*******************************************************************************
Modul           :       solveOps.c (vormals solveMenu.c)

Purpose         :       Starten von Evaluierungsvorgaengen
			und Experimenten anhand ubergebener Parameter

Programmer      :       Martin Mueller
			Kolja Koischwitz
			transientes von Joerg Mitzlaff in 10.94

History         :       release 1: 10/1992
			release 2:  3/1994
*******************************************************************************/

#include <X11/Intrinsic.h>

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <math.h>
#include <sys/types.h> 
#include <sys/wait.h> 
#include <sys/stat.h>
#include <sys/param.h>

#include "Options.h"
#include "NetWidget.h"
#include "environment.h"
#include "util.h"

extern Widget topLevel;
extern Widget getCurrentNet();

/* Importe aus helpPopup.c */
extern void PopUpHelp();
extern void helpDone();
extern void infoDone();
extern void PopUpInfo();

/* Strukturen und Default-Werte ***********************************************/

TransientOptions
  transientDefaults={ 10.0,0.1,Duration,0.1,10,1e-7,False,True,True,False },
  transientOptions={ 10.0,0.1,Duration,0.1,10,1e-7,False,True,True,False };

TransientsimOptions
  transientsimDefaults={ 10.0,10,Sequential,Rule_ON,95.0,10.0,12345,3600,True},
  transientsimOptions={10.0,10,Sequential,Rule_ON,95.0,10.0,12345,3600,True};

AnalyticOptions
  analyticDefaults={ Direct,10000,Sequential,1e-7 },
  analyticOptions={ Direct,10000,Sequential,1e-7 };

ApproximationOptions
  approximationDefaults={ Direct,10000,2,10,1e-7,0.0 },
  approximationOptions={ Direct,10000,2,10,1e-7,0.0 };

SimulationOptions
  simulationDefaults={ Ssimstandard,SpectralVariance,Sequential,Detect_ON,95.0,50.0,10.0,12345,100000,50,10000,3600,Visual_ON,True,True,True,6,600 },
  simulationOptions={ Ssimstandard,SpectralVariance,Sequential,Detect_ON,95.0,50.0,10.0,12345,100000,50,10000,3600,Visual_ON,True,True,True,6,600};

ExperimentOptions experimentOptions={"","","","",Delay};

DTSPNAnalyticOptions
  dtspnanalyticDefaults={ Direct,1000,embno,1e-7,1 },
  dtspnanalyticOptions={ Direct,1000,embno,1e-7,1 };

DTSPNTransientOptions
  dtspntransientDefaults={ 10,1e-7,1,0,embno,1 },
  dtspntransientOptions={ 10,1e-7,1,0,embno,1 };

/* Pfade **********************************************************************/

static String tnethome,modelpath,netname;

/*******************************************************************************
Name:
	CheckEnvironment
--------------------------------------------------------------------------------
Funktionstyp:
	private Function
--------------------------------------------------------------------------------
Zweck:
	Die Environmentvariablen TNETHOME und MODELDIR werden ermittelt
	Der aktuelle Modelname wird ermittelt
	Wurde das Model modifizert, wird es gespeichert
*******************************************************************************/
static Boolean CheckEnvironment()
{
  Boolean picture_modified,net_modified,succ;
  String *params;
  Cardinal numparams;

  XtVaGetValues(getCurrentNet(),XtNmodelName,&netname,NULL);
  if (netname == NULL || strcmp(netname,"") == 0)
  {
	XtWarning("There is no model name specified");
	return False;
  }
  
  if ((tnethome = getenv(DSPNPACKAGE_ENV)) == NULL)
  {
      char msg[200];

      sprintf(msg,
	      "Environment variable %s not set; can't find multi-solve program",
	      DSPNPACKAGE_ENV);
      XtWarning(msg);
      return False;
  }
  
  if ((modelpath = getenv(MODELDIR_ENV)) == NULL)
  {
      char msg[200];

      sprintf(msg,
	      "Environment variable %s not set; can't specified model",
	      "Environment variable %s not set; can't find specified model",
	      MODELDIR_ENV);
      XtWarning(msg);
      return False;
  }
    
  /* Save the Net */
  XtVaGetValues(getCurrentNet(),
		XtNpictureModified,&picture_modified,
		XtNnetModified, &net_modified,
		NULL);
  if (picture_modified || net_modified) 
  {
	params = (String*)XtMalloc(sizeof(String));
	params[0] = XtNewString(netname);
	numparams = 1;
	XtCallActionProc(getCurrentNet(),"save",NULL,params,numparams);
	XtFree(params[0]);
	XtFree((char*)params);
  }  
  return True;
}

/*******************************************************************************
Name:
    StartTransientSolution
--------------------------------------------------------------------------------
Funktionstyp:
	public function (Aufruf aus evalMenu.c)
--------------------------------------------------------------------------------
Zweck:
    forkt zum parallelen Prozess der trans. Analyse
*******************************************************************************/
void StartTransientSolution()
{
  char label[500];
  char solve_command[2*MAXPATHLEN];
  char log_file[MAXPATHLEN];
  char command[MAXPATHLEN];
  struct stat buf1,buf2;
  int statres;

#ifdef debug
  printf(" StartTransientSolution \n");
#endif
  
  if(!CheckEnvironment()) return;
  
  sprintf(label,"Transient solution of NET %s till time %lg",netname,transientOptions.transient_time);

  /* script starten oder gleich compilierte Version des Modells aufrufen ? */
  sprintf(command,"%s/%s.TN",modelpath,netname);
  if(stat(command, &buf1))
  {
    printf("stat error reading .TN file\n");
    return;
  }
  sprintf(command,"%s/%s.TransSolve",modelpath,netname);
  statres = stat(command, &buf2);
/*   printf("out:%d\n",transientOptions.output_data); */
/*   printf("old:%d\n",transientOptions.old_case_iii); */
  if(statres || buf1.st_mtime >= buf2.st_mtime)  /* .TN file neuer als .TransSolve file ? */
  {
    sprintf(solve_command,"%s/TransientSolution/scripts/TransSolve -t %lg -s %lg -o %s -r %lg -c %d -p %lg %s -x -k %s",
	    tnethome,
	    transientOptions.transient_time,
	    transientOptions.stepsize_method,
	    (transientOptions.output_data==Duration ? "curve" : "point"),
	    transientOptions.stepsize_result,
	    transientOptions.cluster_size,
	    transientOptions.precision,
	    (transientOptions.old_case_iii ? "-a" : ""),
	    netname
	    );
  }
  else
  {
    sprintf(solve_command,"%s/%s.TransSolve -t %lg -s %lg -o %s -r %lg -c %d -p %lg %s -x -k %s",
	    modelpath,
	    netname,
	    transientOptions.transient_time,
	    transientOptions.stepsize_method,
	    (transientOptions.output_data==Duration ? "curve" : "point"),
	    transientOptions.stepsize_result,
	    transientOptions.cluster_size,
	    transientOptions.precision,
	    (transientOptions.old_case_iii ? "-a" : ""),
	    netname
	    );
  }

  sprintf(log_file,"%s/%s%s",modelpath,netname,LOG_EXTENSION);
  unlink(log_file);

  sprintf(command,
	  "%s/%s/monitor -display %s -geometry +0+0 -m %s -c \"%s\" -l \"%s\" %s&",
	  tnethome,INTERFACEPATH,DisplayString(XtDisplay(topLevel)),netname,
	  solve_command,label,log_file);
	
  XtVaSetValues(getCurrentNet(),
		XtNprecision,(XtArgVal) -log10(transientOptions.precision),
		XtNsolutionExpected,True,
		NULL);
	
  system(command);
  ExpectResults(getCurrentNet());
}

/*******************************************************************************
Name:
	StartTransientExperiment
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
Aufgerufen aus:
	evalMenu.c
--------------------------------------------------------------------------------
Zweck:
    forkt zum parallelen Prozess der trans. Analyse und noch mehr !?
*******************************************************************************/
Boolean StartTransientExperiment()
{
  return;
}


/*******************************************************************************
Name:
    StartTransientSimulation
--------------------------------------------------------------------------------
Funktionstyp:
	public function (Aufruf aus evalMenu.c)
--------------------------------------------------------------------------------
Zweck:
    forkt zum parallelen Prozess der trans. Simulation
*******************************************************************************/
void StartTransientSimulation()
{
  char command[2*MAXPATHLEN],solve_command[MAXPATHLEN],
     label[500],log_file[MAXPATHLEN];
  
#ifdef debug
  printf(" StartAnalyticSolution \n");
#endif
  
  if(!CheckEnvironment()) return;
  
   sprintf(label,"Transient - Simulation for %s",netname);
  
   sprintf(solve_command,"%s/%s/SIM_TRAN %s -%s -%s  %.1f %ld %lg %lg %ld %.1f",
	   tnethome,
	   SIMULATEPATH,
	   netname,
           transientsimOptions.distribution==Parallel?"D":"S",
	   transientsimOptions.PercentageRule==Rule_ON ? "Pon" : "Poff",     /* Pon | Poff !!! */
           transientsimOptions.TransientInstantOfTime,
           transientsimOptions.NumberOfSamplingPoints,                       /* no bases */            
	   transientsimOptions.confidence_level,
	   transientsimOptions.max_relative_error,
	   transientsimOptions.seed_value,
  	   transientsimOptions.no_CPU_time_limit ? 0.0 : transientsimOptions.max_CPU_time
	   );
 
  sprintf(log_file,"%s/%s%s",modelpath,netname,LOG_EXTENSION);
  
  sprintf(command,
	  "%s/%s/monitor -display %s -geometry +0+0 -m %s -c \"%s\" -l \"%s\" %s&",
	  tnethome,INTERFACEPATH,
	  DisplayString(XtDisplay(topLevel)),netname,
	  solve_command,label,log_file);
  
  system(command);
  ExpectResults(getCurrentNet());
}



/*******************************************************************************
Name:
	StartTransientSimulationExperiment
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
Aufgerufen aus:
	evalMenu.c
--------------------------------------------------------------------------------
Zweck:
    forkt zum parallelen Prozess der trans. Simulation und noch mehr !?
*******************************************************************************/
Boolean StartTransientSimulationExperiment()
{
     XtWarning("Noch nicht implementiert !");
  return;
}

/*******************************************************************************
Name:
	StartAnalyticSolution
--------------------------------------------------------------------------------
Funktionstyp:
	public function (Aufruf aus evalMenu.c)
--------------------------------------------------------------------------------
Zweck:
	Mit den Informationen aus der Struktur analyticOptions das ent-
	sprechende Loesungsverfahren im Monitor parallel starten.
*******************************************************************************/
void StartAnalyticSolution()
{
char command[2*MAXPATHLEN],solve_command[MAXPATHLEN],
     label[500],log_file[MAXPATHLEN];

#ifdef debug
printf(" StartAnalyticSolution \n");
#endif
  
  if(!CheckEnvironment()) return;
  
    /* Steady State Solution */
    {
	  char maxiter[100];
	  sprintf(maxiter,"%ld ",analyticOptions.max_iterations);
	
    {
	TimeNetType net_type;
	Widget nw = getCurrentNet();
	
	XtVaGetValues(nw,XtNnetType,&net_type,NULL);
	switch(net_type)
	{
	  case TN_GSPN:
                sprintf(label,"Steady state solution of GSPN %s ",netname);
		break;
	  case TN_DSPN:
                sprintf(label,"Steady state solution of DSPN %s ",netname);
		break;
	  case TN_ESPN:
                sprintf(label,"Steady state solution of EDSPN %s ",netname);
		break;
	}
    }
      sprintf(solve_command,
	  "%s/%s/NEWSOLVE %s -%s%s %s%lg",
	  tnethome,
	  SOLVEPATH,
	  netname,
	  analyticOptions.solution_method == Iterative ? "i" : "d",
	  analyticOptions.smc_solution == Parallel ? "p" : "",
	  analyticOptions.solution_method == Iterative ? maxiter : "",
	  analyticOptions.precision);
    }

    /*    printf("\n%s\n",solve_command);
    fflush(stdout); */
	
	sprintf(log_file,"%s/%s%s",modelpath,netname,LOG_EXTENSION);
	  
	sprintf(command,
	  "%s/%s/monitor -display %s -geometry +0+0 -m %s -c \"%s\" -l \"%s\" %s&",
	    tnethome,INTERFACEPATH,DisplayString(XtDisplay(topLevel)),netname,
	    solve_command,label,log_file);
	
	XtVaSetValues(getCurrentNet(),
		XtNprecision,(XtArgVal) -log10(analyticOptions.precision),
		XtNsolutionExpected,True,
		NULL);
	
	system(command);
    ExpectResults(getCurrentNet());
}


/*******************************************************************************
Name:
	StartAnalyticExperiment
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
Aufgerufen aus:
	evalMenu.c
--------------------------------------------------------------------------------
Zweck:
	Aufruf des shell-scriptes MSOLVE an Hand der information aus
	der experimentOptions-Struktur
*******************************************************************************/
Boolean StartAnalyticExperiment()
{
  char command[2*MAXPATHLEN],solve_command[MAXPATHLEN],
     label[500],log_file[MAXPATHLEN];

#ifdef debug
printf(" StartAnalyticExperiment \n");
#endif
  if(!CheckEnvironment()) return False;

  sprintf(label,"Analytic Experiment for %s",netname);
  
  sprintf(solve_command,
	  "%s/%s/MSOLVE %s %s %s %s %s %s",
	  tnethome,
	  SOLVEPATH,
	  netname,
	  experimentOptions.parameter_name,
	  experimentOptions.parameter_type==Delay? "d" : "m",
	  experimentOptions.from_value,
	  experimentOptions.to_value,
	  experimentOptions.step_value);
  
  sprintf(log_file,"%s/%s%s",modelpath,netname,LOG_EXTENSION);
  
  sprintf(command,
	  "%s/%s/monitor -display %s -geometry +0+0 -m %s -c \"%s\" -l \"%s\" %s&",
	  tnethome,INTERFACEPATH,
	  DisplayString(XtDisplay(topLevel)),netname,
	  solve_command,label,log_file);
	  
   system(command);
}

/*******************************************************************************
Name:
	StartApproximationExperiment
--------------------------------------------------------------------------------
Funktionstyp:
	public function (Aufruf aus evalMenu.c)
--------------------------------------------------------------------------------
Zweck:
	Mit den Informationen aus der Struktur experimentOptions ein
	Approximations-Experiment im Monitor parallel starten.
*******************************************************************************/
Boolean StartApproximationExperiment()
{
  if(!CheckEnvironment()) return False;
#ifdef debug
  printf(" StartApproximationExperiment \n");
#endif

  return False;
}

/*******************************************************************************
Name:
	StartApproximation
--------------------------------------------------------------------------------
Funktionstyp:
	public function (Aufruf aus evalMenu.c)
--------------------------------------------------------------------------------
Zweck:
	Mit den Informationen aus der Struktur approximationOptions eine
	Approximation im Monitor parallel starten.
*******************************************************************************/
void StartApproximation()
{
  char command[2*MAXPATHLEN],solve_command[MAXPATHLEN],
     label[500],log_file[MAXPATHLEN];

#ifdef debug
  printf(" StartApproximation \n");
#endif

  if(!CheckEnvironment()) return;

    {
	TimeNetType net_type;
	Widget nw = getCurrentNet();
	
	XtVaGetValues(nw,XtNnetType,&net_type,NULL);
	switch(net_type)
	{
	  case TN_GSPN:
                sprintf(label,"Approximation of GSPN %s ",netname);
		break;
	  case TN_DSPN:
                sprintf(label,"Approximation of DSPN %s ",netname);
		break;
	  case TN_ESPN:
                sprintf(label,"Approximation of EDSPN %s ",netname);
		break;
	}
    }
  
if (approximationOptions.droptolerance != 0.0)
  sprintf(solve_command,
	  "%s/%s/APPROX -m %s -p %lg -i %ld -r %ld -c %ld -t %lg %s %s",
	  tnethome,
	  APPROXPATH,
	  approximationOptions.solution_method==Iterative ? "iterate" : "direct",
	  approximationOptions.precision,
	  approximationOptions.solution_method==Iterative ? approximationOptions.max_iterations : 10,
	  approximationOptions.phases,
	  approximationOptions.virtual_phases,
	  approximationOptions.droptolerance,
	  approximationOptions.solution_method==Iterative ? "" : "-n",
	  netname);
else 
  sprintf(solve_command,
	  "%s/%s/APPROX -m %s -p %lg -i %ld -r %ld -c %ld %s %s",
	  tnethome,
	  APPROXPATH,
	  approximationOptions.solution_method==Iterative ? "iterate" : "direct",
	  approximationOptions.precision,
	  approximationOptions.solution_method==Iterative ? approximationOptions.max_iterations : 10,
	  approximationOptions.phases,
	  approximationOptions.virtual_phases,
	  approximationOptions.solution_method==Iterative ? "" : "-n",
	  netname);

/*  printf("\n%s\n",solve_command);
  fflush(stdout); */
  
  sprintf(log_file,"%s/%s%s",modelpath,netname,LOG_EXTENSION);
  
  sprintf(command,
	  "%s/%s/monitor -display %s -geometry +0+0 -m %s -c \"%s\" -l \"%s\" %s&",
	  tnethome,INTERFACEPATH,
	  DisplayString(XtDisplay(topLevel)),netname,
	  solve_command,label,log_file);

  XtVaSetValues(getCurrentNet(),
		XtNprecision,(XtArgVal) -log10(approximationOptions.precision),
		XtNsolutionExpected,True,
		NULL);

  system(command);
  ExpectResults(getCurrentNet());
}

/*******************************************************************************
Name:
	StartSimulationExperiment
--------------------------------------------------------------------------------
Funktionstyp:
	public function (Aufruf aus evalMenu.c)
--------------------------------------------------------------------------------
Zweck:
	Mit den Informationen aus der Struktur experimentOptions ein
	Simulation-Experiment im Monitor parallel starten.
*******************************************************************************/
Boolean StartSimulationExperiment()
{
  char command[2*MAXPATHLEN],solve_command[MAXPATHLEN],
     label[500],log_file[MAXPATHLEN];
  
#ifdef debug
  printf(" StartSimulationExperiment \n");
#endif
  
  if(!CheckEnvironment()) return False;

  sprintf(label,"Simulation Experiment for %s",netname);
  
  sprintf(solve_command,"%s/%s/MSIM %s %s %s %s %s %s",
	  tnethome,
	  SIMULATEPATH,
	  netname,
      experimentOptions.parameter_name,
      experimentOptions.parameter_type==Marking ? "m" : "r",
      experimentOptions.from_value,
      experimentOptions.to_value,
      experimentOptions.step_value
	  );
	  
  sprintf(log_file,"%s/%s%s",modelpath,netname,LOG_EXTENSION);
  
  sprintf(command,
	  "%s/%s/monitor -display %s -geometry +0+0 -m %s -c \"%s\" -l \"%s\" %s&",
	  tnethome,INTERFACEPATH,
	  DisplayString(XtDisplay(topLevel)),netname,
	  solve_command,label,log_file);
  
  system(command);
}

/*******************************************************************************
Name:
	StartSimulation
--------------------------------------------------------------------------------
Funktionstyp:
	public function (Aufruf aus evalMenu.c)
--------------------------------------------------------------------------------
Zweck:
	Mit den Informationen aus der Struktur simulationOptions eine
	Simulation im Monitor parallel starten.
*******************************************************************************/
void StartSimulation()
{
  char command[2*MAXPATHLEN],solve_command[MAXPATHLEN],
     label[500],log_file[MAXPATHLEN];
  
#ifdef debug
  printf(" StartAnalyticSolution \n");
#endif
  
  if(!CheckEnvironment()) return;
  
  if(simulationOptions.simulation_tech==1) {

   sprintf(label,"Standard - Simulation for %s",netname);
  
   sprintf(solve_command,"%s/%s/SIMULATE %s -%s -%s -%s %lg %lg %lg %ld %ld %ld %.1f %.1f -%s",
	   tnethome,
	   SIMULATEPATH,
	   netname,
                   simulationOptions.distribution==Parallel?"D":"S",
	   simulationOptions.statistical_method==SpectralVariance?"S":"C",
	   simulationOptions.detect_init==1 ? "Ton" : "Toff",
	   simulationOptions.confidence_level,
           simulationOptions.errPercent,
	   simulationOptions.max_relative_error,
	   simulationOptions.seed_value,
	   simulationOptions.no_sample_limit ? 0 : simulationOptions.max_sample_number,
           simulationOptions.minFirings,
	   simulationOptions.no_model_time_limit ? 0.0 : simulationOptions.max_model_time,
	   simulationOptions.no_CPU_time_limit ? 0.0 : simulationOptions.max_CPU_time,
           simulationOptions.visualisation==1 ? "Von" : "Voff"
	   );  
  }
  else {
   sprintf(label,"Fast - Simulation for %s",netname);	  
   
   sprintf(solve_command,"%s/%s/RESTART %s -%s %ld %ld -%s %lg %lg %ld %ld %.1f %.1f",  
           tnethome,
           SIMULATEPATH,
           netname,
           simulationOptions.distribution==Parallel?"D":"S",
           simulationOptions.CPU_Time_Run,
           simulationOptions.opt_thresh,
           simulationOptions.detect_init==1 ? "Ton" : "Toff",
           simulationOptions.confidence_level,
           simulationOptions.max_relative_error,
           simulationOptions.seed_value,
           simulationOptions.no_sample_limit ? 0 : simulationOptions.max_sample_number,
           simulationOptions.minFirings,
           simulationOptions.no_model_time_limit ? 0.0 : simulationOptions.max_model_time,
           simulationOptions.no_CPU_time_limit ? 0.0 : simulationOptions.max_CPU_time);
  }

  sprintf(log_file,"%s/%s%s",modelpath,netname,LOG_EXTENSION);
  
  sprintf(command,
	  "%s/%s/monitor -display %s -geometry +0+0 -m %s -c \"%s\" -l \"%s\" %s&",
	  tnethome,INTERFACEPATH,
	  DisplayString(XtDisplay(topLevel)),netname,
	  solve_command,label,log_file);
  
  system(command);
  ExpectResults(getCurrentNet());
}

/*new*/
/*******************************************************************************
Name:
	StartDTSPNAnalyticSolution
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
Aufgerufen aus:
	evalMenu.c
--------------------------------------------------------------------------------
Zweck:
	Aufruf des shell-scriptes DTSPN im DTSOLVEPATH-directory 
*******************************************************************************/
Boolean StartDTSPNAnalyticSolution()
{
  char command[2*MAXPATHLEN],solve_command[MAXPATHLEN],
     label[500],log_file[MAXPATHLEN],maxiter[100],emb[2];

#ifdef debug
printf(" StartDTSPNAnalyticSolution \n");
#endif
  if(!CheckEnvironment()) return False;
 
  sprintf(maxiter,"%ld ",dtspnanalyticOptions.max_iterations);
  if (dtspnanalyticOptions.embedding == embtime) sprintf(emb,"-et");
  else 
    if (dtspnanalyticOptions.embedding == embmem) sprintf(emb,"-em"); 
    else sprintf(emb,"-en");
  sprintf(label,"Time-Discrete Analytic Solution for %s",netname);
  
  sprintf(solve_command,
	  "%s/%s/dtspn %s %s %s %s %s %ld %lg",
	  tnethome,
	  DTSOLVEPATH,
	  netname,
	  "-s",
	  emb,
	  dtspnanalyticOptions.solution_method == Iterative ? "-i" : "-a",
	  dtspnanalyticOptions.solution_method == Iterative ? maxiter : maxiter,
	  dtspnanalyticOptions.scaling,
	  dtspnanalyticOptions.precision);
  
  sprintf(log_file,"%s/%s%s",modelpath,netname,LOG_EXTENSION);
  
  sprintf(command,
	  "%s/%s/monitor -display %s -geometry +0+0 -m %s -c \"%s\" -l \"%s\" %s&",
	  tnethome,INTERFACEPATH,
	  DisplayString(XtDisplay(topLevel)),netname,
	  solve_command,label,log_file);
	  
   system(command);
   ExpectResults(getCurrentNet());
}
/*new*/
/*******************************************************************************
Name:
	StartDTSPNTransientSolution
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
Aufgerufen aus:
	evalMenu.c
--------------------------------------------------------------------------------
Zweck:
	Aufruf des shell-scriptes DTSPN im DTSOLVEPATH-directory 
*******************************************************************************/
Boolean StartDTSPNTransientSolution()
{
  char command[2*MAXPATHLEN],solve_command[MAXPATHLEN],
     label[500],log_file[MAXPATHLEN],maxiter[100],emb[2];

#ifdef debug
printf(" StartDTSPNTransientSolution \n");
#endif
  if(!CheckEnvironment()) return False;
 
  sprintf(label,"Time-Discrete Transient Solution for %s",netname);

  sprintf(maxiter,"%ld ",dtspnanalyticOptions.max_iterations);
  if (dtspntransientOptions.embedding == embtime) sprintf(emb,"-et");
  else 
    if (dtspntransientOptions.embedding == embmem) sprintf(emb,"-em"); 
    else sprintf(emb,"-en");
  
  if (dtspntransientOptions.step_size != 0)
    sprintf(solve_command,
	    "%s/%s/dtspn %s %s %s %s %s %ld %lg %s %ld %ld %ld",
	    tnethome,
	    DTSOLVEPATH,
	    netname,
	    "-t",
	    emb,
	    dtspnanalyticOptions.solution_method == Iterative ? "-i" : "-a",
	    dtspnanalyticOptions.solution_method == Iterative ? maxiter : maxiter,
	    dtspntransientOptions.scaling,
	    dtspntransientOptions.precision,
	    "-c",
	    dtspntransientOptions.first_step,
	    dtspntransientOptions.time_steps,
	    dtspntransientOptions.step_size);
 else
   sprintf(solve_command,
	   "%s/%s/dtspn %s %s %s %s %s %ld %lg %s %ld",
	   tnethome,
	   DTSOLVEPATH,
	   netname,
	   "-t",
	   emb,
	   dtspnanalyticOptions.solution_method == Iterative ? "-i" : "-a",
	   dtspnanalyticOptions.solution_method == Iterative ? maxiter : maxiter,
	   dtspntransientOptions.scaling,
	   dtspntransientOptions.precision,
	   "-p",
	   dtspntransientOptions.time_steps);

  sprintf(log_file,"%s/%s%s",modelpath,netname,LOG_EXTENSION);
  
  sprintf(command,
	  "%s/%s/monitor -display %s -geometry +0+0 -m %s -c \"%s\" -l \"%s\" %s&",
	  tnethome,INTERFACEPATH,
	  DisplayString(XtDisplay(topLevel)),netname,
	  solve_command,label,log_file);
	  
   system(command);
   ExpectResults(getCurrentNet());
}
/*new*/
/*******************************************************************************
Name:
	StartDTAnalyticExperiment
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
Aufgerufen aus:
	evalMenu.c
--------------------------------------------------------------------------------
Zweck:
	Aufruf des shell-scriptes MSOLVE aus DTSOLVE an Hand der information aus
	der experimentOptions-Struktur
*******************************************************************************/
Boolean StartDTAnalyticExperiment()
{
  char command[2*MAXPATHLEN],solve_command[MAXPATHLEN],
     label[500],log_file[MAXPATHLEN],maxiter[100],emb[2];

#ifdef debug
printf(" StartDTAnalyticExperiment \n");
#endif
  if(!CheckEnvironment()) return False;

  sprintf(maxiter,"%ld ",dtspnanalyticOptions.max_iterations);
  if (dtspnanalyticOptions.embedding == embtime) sprintf(emb,"-et");
  else 
    if (dtspnanalyticOptions.embedding == embmem) sprintf(emb,"-em"); 
    else sprintf(emb,"-en");
  sprintf(label,"Discrete Time Analytic Experiment for %s",netname);
  
  sprintf(solve_command,
	  "%s/%s/MSOLVE %s %s %s %s %s %s %s %s %s %ld %lg",
	  tnethome,
	  DTSOLVEPATH,
	  netname,
	  experimentOptions.parameter_name,
	  experimentOptions.parameter_type==Delay? "d" : "m",
	  experimentOptions.from_value,
	  experimentOptions.to_value,
	  experimentOptions.step_value,
	  emb,
	  dtspnanalyticOptions.solution_method == Iterative ? "-i" : "-a",
	  dtspnanalyticOptions.solution_method == Iterative ? maxiter : maxiter,
	  dtspnanalyticOptions.scaling,
	  dtspnanalyticOptions.precision);
  
  sprintf(log_file,"%s/%s%s",modelpath,netname,LOG_EXTENSION);
  
  sprintf(command,
	  "%s/%s/monitor -display %s -geometry +0+0 -m %s -c \"%s\" -l \"%s\" %s&",
	  tnethome,INTERFACEPATH,
	  DisplayString(XtDisplay(topLevel)),netname,
	  solve_command,label,log_file);
	  
   system(command);
}














