#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>

#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Text.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/cursorfont.h>

#include "NetWidget.h"
#include "util.h"
#include "environment.h"

#ifdef __STDC__

extern Widget getCurrentNet();
extern void Save (Widget w, XEvent *event, String *params, Cardinal *numparams);
void invariantsPopup
  (Widget w, XtPointer client_data, XtPointer call_data); 
void ecsPopup
  (Widget w, XtPointer client_data, XtPointer call_data); 

static void invariantsPopdown
  (Widget w, XtPointer client_data, XtPointer call_data);
static void ecsPopdown
  (Widget w, XtPointer client_data, XtPointer call_data);
static void sizeInvariantsPopup(void);
static void sizeEcsPopup(void);
static Boolean uptodate(Widget nw, String extension);

#else

extern Widget getCurrentNet();
extern void Save();

void invariantsPopup();
void ecsPopup();

static void invariantsPopdown();
static void ecsPopdown();
static void sizeInvariantsPopup();
static void sizeEcsPopup();
static Boolean uptodate();

#endif __STDC__

extern char *sys_errlist[];

extern Widget app_root;

Widget 
    invariantsShell;
static Widget
    invariantsMainBox,
    invariantsCommandForm,
    invariantsLabel,
    invariantsDone,
    invariantsText;

Widget 
    ecsShell;
static Widget
    ecsMainBox,
    ecsCommandForm,
    ecsLabel,
    ecsDone,
    ecsText;

    
#define SORRYI "The <model>.INV file is not existent/uptodate.\nSolve the model first."
#define SORRYE "The <model>.ECS file is not existent/uptodate.\nSolve the model first."

#ifdef __STDC__
void initInvariantsPopup(Widget topLevel)
#else
void initInvariantsPopup(topLevel)
    Widget topLevel;
#endif
{
    int width;
    Dimension height;
    XFontStruct *fontStruct;

    invariantsShell = XtVaCreatePopupShell("Invariants",
					   transientShellWidgetClass,
					   topLevel,
					   XtNallowShellResize, False,
					   NULL);
    
    invariantsMainBox = XtVaCreateManagedWidget("invariantsMainBox",
						boxWidgetClass,
						invariantsShell,
					      XtNorientation, XtorientVertical,
						XtNvSpace, 0,
						XtNhSpace, 0,
						NULL);

    invariantsCommandForm = XtVaCreateManagedWidget("invariantsCommandForm",
						    formWidgetClass,
						    invariantsMainBox,
						    XtNborderWidth, 0,
						    NULL);
					      
    invariantsLabel = XtVaCreateManagedWidget("invariantsLabel",
					      labelWidgetClass,
					      invariantsCommandForm,
					      XtNlabel, 
					      "Displaying Place-Invariants of ",
					      XtNborderWidth, 0,
					      XtNresizable, True,
					      NULL);
 
    invariantsDone = XtVaCreateManagedWidget("invariantsDone",
					     commandWidgetClass,
					     invariantsCommandForm,
					     XtNlabel, "Done",
					     NULL);

    XtAddCallback(invariantsDone, XtNcallback, invariantsPopdown, NULL);

    invariantsText = XtVaCreateManagedWidget("invariantsText",
					 asciiTextWidgetClass,
					 invariantsMainBox,
					 XtNdisplayCaret, False,
					 XtNscrollVertical, 
					    XawtextScrollWhenNeeded,
					     /* XtNscrollHorizontal, 
						XawtextScrollWhenNeeded,
		       Horizontal scrolling not allowed with wrapping active. */

					 XtNwrap, XawtextWrapWord,
					 XtNtype, XawAsciiString,
					 XtNstring, "",
					 NULL);

    /* make it fitting for 80x20 chars */
    XtVaGetValues (invariantsText,
		   XtNfont, &fontStruct,
		   XtNheight, &height,
		   NULL);
    width = XTextWidth (fontStruct, "X", 1);
    width *= 80;
    height *= 20;
    XtVaSetValues (invariantsText,
		   XtNheight, height,
		   XtNwidth, width,
		   NULL);

    sizeInvariantsPopup(); 
}

#ifdef __STDC__
void initEcsPopup(Widget topLevel)
#else
void initEcsPopup(topLevel)
    Widget topLevel;
#endif
{
    int width;
    Dimension height;
    XFontStruct *fontStruct;

    ecsShell = XtVaCreatePopupShell("ECS",
				    transientShellWidgetClass,
				    topLevel,
				    NULL);
    
    ecsMainBox = XtVaCreateManagedWidget("ecsMainBox",
					 boxWidgetClass,
					 ecsShell,
					 XtNorientation, XtorientVertical,
					 XtNvSpace, 0,
					 XtNhSpace, 0,
					 NULL);
    
    ecsCommandForm = XtVaCreateManagedWidget("ecsCommandForm",
					     formWidgetClass,
					     ecsMainBox,
					     XtNborderWidth, 0,
					     NULL);
					
    ecsLabel = XtVaCreateManagedWidget("ecsLabel",
				       labelWidgetClass,
				       ecsCommandForm,
				       XtNlabel, "Extended Conflict Set of ",
				       XtNborderWidth, 0,
				       XtNresizable, True,
				       NULL);

    ecsDone = XtVaCreateManagedWidget("ecsDone",
				      commandWidgetClass,
				      ecsCommandForm,
				      XtNlabel, "Done",
				      NULL);
    
    XtAddCallback(ecsDone, XtNcallback, ecsPopdown, NULL);

    ecsText = XtVaCreateManagedWidget("ecsText",
				      asciiTextWidgetClass,
				      ecsMainBox,
				      XtNdisplayCaret, False,
				      XtNscrollVertical, 
				      XawtextScrollWhenNeeded,
				      XtNwrap, XawtextWrapWord,
				      XtNtype, XawAsciiString,
				      XtNstring, "",
				      NULL);
    /* make it fitting for 80x20 chars */
    XtVaGetValues (ecsText,
		   XtNfont, &fontStruct,
		   XtNheight, &height,
		   NULL);
    width = XTextWidth (fontStruct, "X", 1);
    width *= 80;
    height *= 20;
    XtVaSetValues (ecsText,
		   XtNheight, height,
		   XtNwidth, width,		
		   NULL);

    sizeEcsPopup(); 
}

#ifdef __STDC__
void invariantsPopup
  (Widget w, XtPointer client_data, XtPointer call_data)
#else
void invariantsPopup(w, client_data, call_data)
     Widget w;
     XtPointer client_data, call_data;
#endif __STDC__
{
    Dimension width;
    Position x,y;
    char *name,*modelpath,*dspnhome;
    char net[MAXPATHLEN], msg[MAXPATHLEN+80], command[MAXPATHLEN];
    int error = 0;
    
    XtVaGetValues(getCurrentNet(),
		  XtNmodelName,&name,
		  NULL);
    if (name == 0 || strcmp(name,"") == 0) {
	XtWarning("There is no model name specified");
	return;
    }

    if ((modelpath = getenv(MODELDIR_ENV)) == NULL)
      {
	char msg[200];
	
	sprintf(msg,
		"Environment variable %s not set; can't find specified model",
		MODELDIR_ENV);
	XtWarning(msg);
	return;
      }

    sprintf(msg, "Displaying Place-Invariants of %s", name);
    XtVaSetValues (invariantsLabel, 
		   XtNlabel, msg,
		   NULL);

    XtVaGetValues (invariantsText,
		   XtNwidth, &width,
		   NULL);

    XtTranslateCoords (w, 0,0, &x, &y);
    x -= (Position)width/3;

  {
    Dimension p_width,p_height,screen_width,screen_height;

    if (!XtIsRealized(invariantsShell))
      XtRealizeWidget(invariantsShell);
    
    XtVaGetValues(invariantsShell,
		  XtNwidth,&p_width,
		  XtNheight,&p_height,
		  NULL);
    
    screen_width = (Dimension)WidthOfScreen(XtScreen(invariantsShell));
    screen_height = (Dimension)HeightOfScreen(XtScreen(invariantsShell));
    
    if (y + (Position)p_height > (Position)screen_height)
      y = (Position)screen_height - (Position)p_height;
    if (y < 0)
      y = 0;
    
    if (x < 0)
      x = 0;
    else
      if (x + (Position)p_width > (Position)screen_width )
	x = (Position)screen_width - (Position)p_width;
    
    XtMoveWidget(invariantsShell, x, y);
  }

    if (!uptodate(getCurrentNet(),INV_EXTENSION)) {
	Cursor cursor;

	if ((dspnhome = getenv(DSPNPACKAGE_ENV)) == NULL)
	  {
	    char msg[200];
	    
	    sprintf(msg,
		    "Environment variable %s not set; can't find structure program",
		    DSPNPACKAGE_ENV);
	    XtWarning(msg);
	    return;
	  }
	/* display WATCH while computing structur */
	cursor = XCreateFontCursor (XtDisplay(app_root), XC_watch);
	XDefineCursor(XtDisplay(app_root), XtWindow(app_root), cursor);
	XFlush(XtDisplay(app_root));

	sprintf(command, 
		"csh -e -f %s/StatAnalysis/scripts/structure %s ", 
		dspnhome, name);
	error = system (command);
	    
	XUndefineCursor(XtDisplay(app_root),XtWindow(app_root));
    }

    /* path to <model>.INV */
    if (error)
	sprintf(net, "%s/%s.err", modelpath, name); 
    else 
	sprintf(net, "%s/%s%s", modelpath, name, INV_EXTENSION); 

    XtVaSetValues(invariantsText,
		  XtNtype, XawAsciiFile,
		  XtNstring, net,
		  NULL);

    XtPopup (invariantsShell, XtGrabNone);
}

#ifdef __STDC__
void ecsPopup
  (Widget w, XtPointer client_data, XtPointer call_data)
#else
void ecsPopup(w, client_data, call_data)
     Widget w;
     XtPointer client_data, call_data;
#endif __STDC__
{
    Dimension width;
    Position x,y;
    Boolean picture_modified,net_modified;
    char *name,*modelpath,*dspnhome;
    char net[MAXPATHLEN], msg[MAXPATHLEN+80], command[MAXPATHLEN];
    int error = 0;

    XtVaGetValues(getCurrentNet(),
          XtNpictureModified,&picture_modified,
          XtNnetModified, &net_modified,
 		  XtNmodelName,&name,
		  NULL);
    if (name == NULL || strcmp(name,"") == 0) {
	XtWarning("There is no model name specified");
	return;
    }

    if ((modelpath = getenv(MODELDIR_ENV)) == NULL)
      {
	char msg[200];
	
	sprintf(msg,
		"Environment variable %s not set; can't find specified model",
		MODELDIR_ENV);
	XtWarning(msg);
	return;
      }

    sprintf(msg, "Displaying Extended Conflict Set of %s", name);
    XtVaSetValues (ecsLabel,
		   XtNlabel, msg,
		   NULL);

    XtVaGetValues (ecsText,
		   XtNwidth, &width,
		   NULL);

    XtTranslateCoords (w, 0,0, &x, &y);
    x -= (Position)width/3;

    {
      Dimension p_width,p_height,screen_width,screen_height;
      
      if (!XtIsRealized(ecsShell))
	XtRealizeWidget(ecsShell);
      
      XtVaGetValues(ecsShell,
		    XtNwidth,&p_width,
		    XtNheight,&p_height,
		    NULL);
      
      screen_width = (Dimension)WidthOfScreen(XtScreen(ecsShell));
      screen_height = (Dimension)HeightOfScreen(XtScreen(ecsShell));
      
      if (y + (Position)p_height > (Position)screen_height)
	y = (Position)screen_height - (Position)p_height;
      if (y < 0)
	y = 0;
      
      if (x < 0)
	x = 0;
      else
	if (x + (Position)p_width > (Position)screen_width )
	  x = (Position)screen_width - (Position)p_width;
      
      XtMoveWidget(ecsShell, x, y);
  }

    if (!uptodate(getCurrentNet(),ECS_EXTENSION)){
	Cursor cursor;

	if ((dspnhome = getenv(DSPNPACKAGE_ENV)) == NULL)
	  {
	    char msg[200];
	    
	    sprintf(msg,
		    "Environment variable %s not set; can't find structure program",
		    DSPNPACKAGE_ENV);
	    XtWarning(msg);
	    return;
	  }
	/* display WATCH while computing structur */
	cursor = XCreateFontCursor (XtDisplay(app_root), XC_watch);
	XDefineCursor(XtDisplay(app_root), XtWindow(app_root), cursor);
	XFlush(XtDisplay(app_root));

    if ( picture_modified || net_modified ) 
    { 
	  String params[1];
	  params[0] = name;
      XtCallActionProc(getCurrentNet(),"save",NULL,params,1);
    }

	sprintf(command, "csh -e -f %s/StatAnalysis/scripts/structure %s",
		getenv(DSPNPACKAGE_ENV), name);

	error = system (command);
	    
	XUndefineCursor(XtDisplay(app_root),XtWindow(app_root));
    }

    /* path to <model>.ECS */
    if (error)
	sprintf(net, "%s/%s.err", modelpath, name); 
    else 
	sprintf(net, "%s/%s%s", modelpath, name,ECS_EXTENSION);

    XtVaSetValues (ecsText,
		   XtNtype, XawAsciiFile,
		   XtNstring, net,
		   NULL);    

    XtPopup (ecsShell, XtGrabNone);
}

#ifdef __STDC__
static void sizeInvariantsPopup (void)
#else
static void sizeInvariantsPopup ()
#endif
{
    Dimension width=0, height, wbuf;
    Cardinal n, numChildren;
    WidgetList children;    
    
    /* dont show the Popup now - but realize it - cause we need the geometry */
    XtVaSetValues (invariantsShell, XtNmappedWhenManaged, False, NULL);
    XtRealizeWidget(invariantsShell);
    
    /* get the greatest width in MainBox */
    XtVaGetValues (invariantsMainBox,
		   XtNnumChildren, &numChildren,
                   XtNchildren, &children,
                   NULL);
    if (numChildren==0)
        XtWarning ("Invariants-mainBox has NO children");
    else {
        /* get width of each child and overall maximum */
        for (n=0; n < numChildren; n++) {
            XtVaGetValues (children[n], XtNwidth, &wbuf, NULL);
            width = Max(width,wbuf);
        }
    }

    /* set Done-Button to the right side */
    XtVaGetValues (invariantsDone, XtNwidth, &wbuf, NULL);

    XtVaSetValues (invariantsDone, XtNhorizDistance, (int)width-wbuf-4, NULL);

    XtVaGetValues (invariantsShell, 
		   XtNheight, &height,
		   XtNwidth, &width,
		   NULL);
    XtVaSetValues (invariantsShell, 
		   XtNmappedWhenManaged, True,
		   XtNmaxHeight, height,
		   XtNminHeight, height,
		   XtNmaxWidth, width,
		   XtNminWidth, width,
		   NULL);
	    
} /* sizeInvariantsPopup */

#ifdef __STDC__
static void sizeEcsPopup ()
#else
static void sizeEcsPopup ()
#endif
{
    Dimension width=0, height, wbuf;
    Cardinal n, numChildren;
    WidgetList children;    
    
    /* dont show the Popup now - but realize it - cause we need the geometry */
    XtVaSetValues (ecsShell, XtNmappedWhenManaged, False, NULL);
    XtRealizeWidget(ecsShell);
    
    /* get the greatest width in MainBox */
    XtVaGetValues (ecsMainBox,
		   XtNnumChildren, &numChildren, 
                   XtNchildren, &children,
                   NULL);
    if (numChildren==0) 
        XtWarning ("Ecs-mainBox has NO children");
    else {
        /* get width of each child and overall maximum */
        for (n=0; n < numChildren; n++) {
            XtVaGetValues (children[n], XtNwidth, &wbuf, NULL);
            width = Max(width,wbuf);
        }
    }

    /* set Done-Button to the right side */
    XtVaGetValues (ecsDone, XtNwidth, &wbuf, NULL);
    XtVaSetValues (ecsDone, XtNhorizDistance, (int)width-wbuf-4, NULL);

    XtVaGetValues (invariantsShell, 
		   XtNheight, &height,
		   XtNwidth, &width,
		   NULL);
    XtVaSetValues (ecsShell, 
		   XtNmappedWhenManaged, True,
		   XtNmaxHeight, height,
		   XtNminHeight, height,
		   XtNmaxWidth, width,
		   XtNminWidth, width,
		   NULL);
	    
} /* sizeEcsPopup */

#ifdef __STDC__
static void invariantsPopdown
  (Widget w, XtPointer client_data, XtPointer call_data)
#else
static void invariantsPopdown(w, client_data, call_data)
     Widget w;
     XtPointer client_data, call_data;
#endif __STDC__
{

    XtPopdown (invariantsShell);
    
}

#ifdef __STDC__
static void ecsPopdown
  (Widget w, XtPointer client_data, XtPointer call_data)
#else
static void ecsPopdown(w, client_data, call_data)
     Widget w;
     XtPointer client_data, call_data;
#endif __STDC__
{

    XtPopdown (ecsShell);
    
}

#ifdef __STDC__
static Boolean uptodate(Widget nw, String extension)
#else
static Boolean uptodate(nw,extension)
    Widget nw;
    String extension;
#endif
{
    char *name,*modelpath;
    char path[MAXPATHLEN], net[MAXPATHLEN];
    Boolean modified;
    struct stat status;
    time_t NETtime, Ftime;

    XtVaGetValues(getCurrentNet(),
		  XtNmodelName,&name,
		  NULL);
    if (name == NULL || strcmp(name,"") == 0) {
	XtWarning("There is no model name specified");
	return False;
    }

    if ((modelpath = getenv(MODELDIR_ENV)) == NULL)
      {
	char msg[200];
	
	sprintf(msg,
		"Environment variable %s not set; can't find specified model",
		MODELDIR_ENV);
	XtWarning(msg);
	return False;
      }

    /* path to <model>.* */
    sprintf(path, "%s/%s",modelpath, name);

    /* uptodate ? */
    XtVaGetValues (nw,
		   XtNnetModified, &modified,
		   NULL);
    if (modified)
      {
	return False;
      }
    else {
	sprintf(net, "%s%s", path,NET_EXTENSION);
	if (!stat(net, &status))
	    NETtime = status.st_mtime;
	else {
/*	    sprintf(msg, "%s: %s", net, sys_errlist[errno]);
	    XtWarning(msg); */
	    return False;
	}
	sprintf(net, "%s%s", path, extension);
	if (!stat(net, &status))
	    Ftime = status.st_mtime;
	else {
/*	    sprintf(msg, "%s: %s", net, sys_errlist[errno]);
	    XtWarning(msg); */
	    return False;
	}
	return (Ftime >= NETtime);
    }
    return False;
}


