/*******************************************************************************
Modul   	:	textPopup.c

Purpose		: 	Bereitstellen der Dialogboxen zur Texteingabe

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/


#include <stdio.h>
#include <ctype.h>
#include <sys/param.h>

#include <X11/IntrinsicP.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/

/*
 * Public include file for widgets we actually use in this file.
 */
#include <X11/Shell.h>     
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Command.h> 
#include <X11/Xaw/Box.h>    
#include <X11/Xaw/Toggle.h>
#include <X11/Xaw/AsciiText.h>  
#include <X11/Xaw/Form.h>  

/*
 * Our include files.
 */
#include "NetWidget.h"
#include "TransObj.h"
#include "PlaceObj.h"
#include "ArcObj.h"
#include "MarkParObj.h"
#include "RateParObj.h"

#include "util.h"
#include "inputCheck.h"
#include "environment.h"

#include "pmfEdit.h"

#define SMALL_BUF_SIZE 500
#define BIG_BUF_SIZE 20000

#define MAX_TOKEN 10000

#define SMALL_LABEL_WIDTH 150
#define BIG_LABEL_WIDTH 312
#define BIG_LABEL_WIDTH2 420
#define BIG_LABEL_WIDTH_1 400
#define SMALL_TEXT_WIDTH 156
#define SMALL_TEXT_WIDTH2 245
#define BIG_TEXT_HEIGHT 300
#define BIG_TEXT_WIDTH 445
#define BUTTON_WIDTH 100
#define BUTTON_WIDTH2 120

#define MAX_WIDGETS 10

#define ARC_MD 0
#define IMMEDIATE_MD 1
#define EXPONENTIEL_MD 2
#define DETERMINISTIC_MD 3
#define PARAMETER_ADD 4
#define PARAMETER_CHANGE 5

extern void PopUpHelp();
extern void helpDone();
extern void PopUpInfo();
extern void infoDone();
extern Widget getCurrentNet();
extern Widget netName;
extern char *vlstrcpy();
extern char *vlstrcat();
extern int checkSyntax();
extern int TestRParamName();
#ifdef __STDC__
   extern void onlySimulation(int state);
#else
   extern void onlySimulation();
#endif

typedef enum {New=1,Merge,GroupReset} GroupState;   /*new*/
/*new*/
extern int GetTimeMode();   
enum {ContTime=0,DiscTime};

typedef struct _EditState {
		Commands edit_command;
		Objects edit_object;
		Widget object,managed_form,key_board_input_text;
		ArcType arctype;
		TransType transtype;
		Boolean md_def,inf_serv;
                int type;
                char pmffunc[256];
                char DataFile[256];
                RacePolicy racepolicy;
		} EditState;

/*NWCallbackInfo_2 *nwInfo;*/
Position x_event, y_event;

/* forward definition of private functions */
/*Actions*/
static void ChangeKeyboardFocus();

/*Callbacks*/
static void DialogDone();
static void DialogCancel();
static void ShowMarkDepEditor();
static void ResetPopup();
static void ChangeServState();
static void ChangeTranstypeState();
static void ChangeTranspolicyState();
static void ChangeArctypeState();
/*new*/
static void ChangeGroupCallback();

static char add_buf_1[SMALL_BUF_SIZE];
static char add_buf_2[SMALL_BUF_SIZE];
static char change_buf[SMALL_BUF_SIZE];
static char change_delay_buf[SMALL_BUF_SIZE];
static char change_arc_buf[SMALL_BUF_SIZE];
static char change_trans_buf[SMALL_BUF_SIZE];
static char md_def_buf[BIG_BUF_SIZE];
/*new*/
static char change_phase_buf[SMALL_BUF_SIZE];
static char change_group_weight_buf[SMALL_BUF_SIZE];
static char change_group_new_buf[SMALL_BUF_SIZE];
static char change_group_merge_buf[SMALL_BUF_SIZE];

/*int noDataFileTransition;*/
/* Public var */
static Widget textPopup;

/* private var */
static Widget	
mainBox,
    add_form,
	add_label_1,
	add_text_1,
	add_label_2,
	add_text_2,
    add_Ddef_button,
    change_form,
	change_label,
	change_text,
    change_Ddef_button,
    change_arc_form,
	change_arc_label_1,
	change_arc_input_toggle,
	change_arc_output_toggle,
	change_arc_inhibitor_toggle,
	change_arc_label_2,
	change_arc_text,
	change_arc_md_button,
    change_trans_form,
	change_trans_label_1,
	change_trans_immediate_toggle,
	change_trans_exponential_toggle,
	change_trans_deterministic_toggle,
	change_trans_general_toggle,
	change_trans_label_2,
	change_trans_text,
  /*new*/
        change_trans_label_3,
        change_trans_RE_toggle,
	change_trans_RA_toggle,  
    change_delay_form,
	change_delay_label,
	change_delay_text,
	change_delay_md_button,
	change_delay_inf_button,
	change_delay_single_button,
  /*new*/
        change_delay_phase_label,
	change_delay_phase_text,
        change_delay_group_weight_label,
        change_delay_group_weight_text,

     md_def_form,
        md_done_button,
        md_help_button,
        md_cancel_button,
  	md_def_label,
	md_def_text,
  /*new*/
     change_group_form,
        change_group_label,
        change_group_new_toggle,
        change_group_merge_toggle,
        change_group_reset_toggle,
        change_group_new_label,
        change_group_new_text,
        change_group_merge_label,
        change_group_merge_text,

     main_command_form,
	done_button,
	help_button,
	cancel_button;

static XtActionsRec actions[] = {
                                    {"changeKeyboardFocus",ChangeKeyboardFocus},
                                   };
	
static char trans_toggle[] = "<Btn1Down>,<Btn1Up>: set() notify()";

/*        Ctrl<Key>Y:             unkill()                        \n\*/

static char trans_text[] = "\
        Ctrl<Key>F:             forward-character()             \n\
        Ctrl<Key>B:             backward-character()            \n\
        Ctrl<Key>D:             delete-next-character()         \n\
        Ctrl<Key>A:             beginning-of-line()             \n\
        Ctrl<Key>E:             end-of-line()                   \n\
        Ctrl<Key>H:             delete-previous-character()     \n\
        Ctrl<Key>K:             kill-to-end-of-line()           \n\
        Ctrl<Key>W:             kill-selection()                \n\
        Ctrl<Key>:              no-op()                         \n\
            <Key>Right:         forward-character()             \n\
            <Key>Left:          backward-character()            \n\
            <Key>Up:            changeKeyboardFocus()           \n\
            <Key>Down:          changeKeyboardFocus()           \n\
            <Key>Delete:        delete-previous-character()     \n\
            <Key>BackSpace:     delete-previous-character()     \n\
            <Key>Linefeed:      no-op()                         \n\
            <Key>Return:        changeKeyboardFocus()           \n\
            <Key>Tab:           changeKeyboardFocus()           \n\
            <Key>:              insert-char()                   \n\
            <Btn1Down>:         select-start()                  \n\
            <Btn1Motion>:       extend-adjust()                 \n\
            <Btn1Up>:           extend-end(PRIMARY,CUT_BUFFER0) \n\
            <Btn2Down>:         insert-selection(PRIMARY,CUT_BUFFER0)\n\
            <Btn3Down>:         extend-start()                  \n\
            <Btn3Motion>:       extend-adjust()                 \n\
            <Btn3Up>:           extend-end(PRIMARY,CUT_BUFFER0) ";


static EditState edit_state;
 

/*******************************************************************************
Name:
        init_textPopUp
--------------------------------------------------------------------------------
Funktionstyp:
        "offentliche Funktion
--------------------------------------------------------------------------------
Parameter:
        topLevel: toplevelshell der Applikation
--------------------------------------------------------------------------------
Zweck:
        Initialisierung der Dialogboxen
*******************************************************************************/

void init_textPopUp(appContext,topLevel)
XtAppContext appContext;
Widget topLevel;
{
XtTranslations toggle_translations,text_translations;


  XtAppAddActions(appContext,actions,XtNumber(actions));
  toggle_translations = XtParseTranslationTable(trans_toggle);
  text_translations = XtParseTranslationTable(trans_text);

  textPopup = XtVaCreatePopupShell(
			"textPopup",
			transientShellWidgetClass,
			topLevel,
			NULL);
  XtAddCallback(textPopup,XtNpopdownCallback,ResetPopup,0);

  mainBox = XtVaCreateManagedWidget(
		"mainBox",
		boxWidgetClass,
		textPopup,
		XtNvSpace,0,
		XtNhSpace,0,
		XtNborderWidth,0,
		NULL);

  add_form = XtVaCreateWidget(
		"add_form",
		formWidgetClass,
		mainBox,
		XtNborderWidth,0,
		NULL);
  add_label_1 = XtVaCreateManagedWidget(
			"add_label_1",
			labelWidgetClass,
			add_form,
			XtNforeground,0,
			XtNbackground,1,
			XtNwidth,SMALL_LABEL_WIDTH,
			XtNlabel,"Name:",
			XtNresize,False,
			XtNjustify,XtJustifyRight,
			NULL);
  add_text_1  = XtVaCreateManagedWidget(
			"add_text_1",
			asciiTextWidgetClass,
			add_form,
			XtNwidth,SMALL_TEXT_WIDTH,
			XtNstring,add_buf_1,
			XtNlength,SMALL_BUF_SIZE,
			XtNeditType,XawtextEdit,
			XtNtranslations,text_translations,	
			XtNfromHoriz,add_label_1,
			NULL);
  add_label_2 = XtVaCreateManagedWidget(
			"add_label_2",
			labelWidgetClass,
			add_form,
			XtNforeground,0,
			XtNbackground,1,
			XtNwidth,SMALL_LABEL_WIDTH,
			XtNlabel,"Value:",
			XtNresize,False,
			XtNjustify,XtJustifyRight,
			XtNfromVert,add_label_1,
			NULL);
  add_text_2 = XtVaCreateManagedWidget(
			"add_text_2",
			asciiTextWidgetClass,
			add_form,
/*			XtNborderWidth,0,*/
			XtNdisplayCaret,False,
			XtNwidth,SMALL_TEXT_WIDTH,
			XtNstring,add_buf_2,
			XtNlength,SMALL_BUF_SIZE,
			XtNeditType,XawtextEdit,
			XtNtranslations,text_translations,
			XtNfromVert,add_label_1,	
			XtNfromHoriz,add_label_2,
			NULL);

  add_Ddef_button = XtVaCreateWidget("add_Ddef_button",
				     commandWidgetClass,
				     add_form,
				     XtNlabel,"definition...",
				     XtNwidth,BUTTON_WIDTH,
				     XtNfromVert,add_label_2,	
				     NULL);

  XtAddCallback(add_Ddef_button, XtNcallback, ShowMarkDepEditor,
		(XtPointer)PARAMETER_ADD);

  change_form = XtVaCreateWidget(
		"change_form",
		formWidgetClass,
		mainBox,
		XtNborderWidth,0,
		NULL);
  change_label = XtVaCreateManagedWidget(
			"change_label",
			labelWidgetClass,
			change_form,
			XtNforeground,0,
			XtNbackground,1,
			XtNwidth,SMALL_LABEL_WIDTH,
			XtNresize,False,
			XtNjustify,XtJustifyRight,
			NULL);
  change_text  = XtVaCreateManagedWidget(
			"change_text",
			asciiTextWidgetClass,
			change_form,
			XtNwidth,SMALL_TEXT_WIDTH,
			XtNstring,change_buf,
			XtNlength,SMALL_BUF_SIZE,
			XtNtranslations,text_translations,
			XtNeditType,XawtextEdit,	
			XtNfromHoriz,add_label_1,
			NULL);

  change_Ddef_button = XtVaCreateWidget("change_Ddef_button",
				     commandWidgetClass,
				     change_form,
				     XtNlabel,"definition...",
				     XtNwidth,BUTTON_WIDTH,
				     XtNfromVert,change_label,	
				     NULL);

  XtAddCallback(change_Ddef_button, XtNcallback, 
		ShowMarkDepEditor, (XtPointer)PARAMETER_CHANGE);

  change_arc_form = XtVaCreateWidget(
		"change_arc_form",
		formWidgetClass,
		mainBox,
		XtNborderWidth,0,
		NULL);
  change_arc_label_1 = XtVaCreateManagedWidget(
			"change_arc_label_1",
			labelWidgetClass,
			change_arc_form,
			XtNforeground,0,
			XtNbackground,1,
			XtNwidth,BIG_LABEL_WIDTH,
			XtNlabel,"Type of arc:",
			XtNresize,False,
	/*		XtNjustify,XtJustifyRight,*/
			NULL);
  change_arc_input_toggle = XtVaCreateManagedWidget(
			"change_arc_input_toggle",
			toggleWidgetClass,
			change_arc_form,
			XtNwidth,BUTTON_WIDTH,
			XtNlabel,"input",
			XtNstate,True,
			XtNfromVert,change_arc_label_1,
			NULL);
  change_arc_output_toggle = XtVaCreateManagedWidget(
			"change_arc_output_toggle",
			toggleWidgetClass,
			change_arc_form,
			XtNwidth,BUTTON_WIDTH,
			XtNlabel,"output",
			XtNfromVert,change_arc_label_1,
			XtNfromHoriz,change_arc_input_toggle,
			NULL);
  change_arc_inhibitor_toggle = XtVaCreateManagedWidget(
			"change_arc_inhibitor_toggle",
			toggleWidgetClass,
			change_arc_form,
			XtNwidth,BUTTON_WIDTH,
			XtNlabel,"inhibitor",
			XtNfromVert,change_arc_label_1,
			XtNfromHoriz,change_arc_output_toggle,
			NULL);
  XawToggleChangeRadioGroup(change_arc_output_toggle,
				change_arc_input_toggle);
  XawToggleChangeRadioGroup(change_arc_inhibitor_toggle,
				change_arc_input_toggle);
  XtOverrideTranslations(change_arc_output_toggle,toggle_translations);
  XtOverrideTranslations(change_arc_input_toggle,toggle_translations);
  XtOverrideTranslations(change_arc_inhibitor_toggle,toggle_translations);
  XtAddCallback(change_arc_input_toggle,XtNcallback,ChangeArctypeState,
		(XtPointer)INPUTARC);
  XtAddCallback(change_arc_output_toggle,XtNcallback,ChangeArctypeState,
		(XtPointer)OUTPUTARC);
  XtAddCallback(change_arc_inhibitor_toggle,XtNcallback,ChangeArctypeState,
		(XtPointer)INHIBITORARC);
  change_arc_label_2 = XtVaCreateManagedWidget(
			"change_arc_label_2",
			labelWidgetClass,
			change_arc_form,
			XtNforeground,0,
			XtNbackground,1,
			XtNwidth,SMALL_LABEL_WIDTH,
			XtNlabel,"Multiplicity:",
			XtNresize,False,
			XtNjustify,XtJustifyRight,
			XtNfromVert,change_arc_inhibitor_toggle,
			NULL);
  change_arc_text  = XtVaCreateManagedWidget(
			"change_arc_text",
			asciiTextWidgetClass,
			change_arc_form,
			XtNwidth,SMALL_TEXT_WIDTH,
			XtNstring,change_arc_buf,
			XtNlength,SMALL_BUF_SIZE,
			XtNeditType,XawtextEdit,
			XtNtranslations,text_translations,	
			XtNfromHoriz,change_arc_label_2,
			XtNfromVert,change_arc_inhibitor_toggle,
			NULL);
  change_arc_md_button = XtVaCreateManagedWidget(
			"change_arc_md_button",
			commandWidgetClass,
			change_arc_form,
			XtNwidth,BUTTON_WIDTH,
			XtNlabel,"mark.-dep",
			/* XtNstate,True, */
			XtNfromVert,change_arc_text,
			NULL);
/*  XtOverrideTranslations(change_arc_md_button,toggle_translations); */
  XtAddCallback(change_arc_md_button,XtNcallback,ShowMarkDepEditor, 
		(XtPointer)ARC_MD);

  change_trans_form = XtVaCreateWidget(
		"change_trans_form",
		formWidgetClass,
		mainBox,
		XtNborderWidth,0,
		NULL);
  change_trans_label_1 = XtVaCreateManagedWidget(
			"change_trans_label_1",
			labelWidgetClass,
			change_trans_form,
			XtNforeground,0,
			XtNbackground,1,
			XtNwidth,BIG_LABEL_WIDTH_1,
			XtNlabel,"Type of transition:",
			XtNresize,False,
		/*	XtNjustify,XtJustifyRight,*/
			NULL);
  change_trans_immediate_toggle = XtVaCreateManagedWidget(
			"change_trans_immediate_toggle",
			toggleWidgetClass,
			change_trans_form,
			XtNwidth,BUTTON_WIDTH,
			XtNlabel,"immediate",
			XtNfromVert,change_trans_label_1,
			NULL);
  change_trans_exponential_toggle = XtVaCreateManagedWidget(
			"change_trans_exponential_toggle",
			toggleWidgetClass,
			change_trans_form,
			XtNwidth,BUTTON_WIDTH,
			XtNlabel,"exponential",
			XtNfromVert,change_trans_label_1,
			XtNfromHoriz,change_trans_immediate_toggle,
			NULL);
  change_trans_deterministic_toggle = XtVaCreateManagedWidget(
			"change_arc_trans_deterministic_toggle",
			toggleWidgetClass,
			change_trans_form,
			XtNwidth,BUTTON_WIDTH,
			XtNlabel,"deterministic",
			XtNfromVert,change_trans_label_1,
			XtNfromHoriz,change_trans_exponential_toggle,
			NULL);
  change_trans_general_toggle = XtVaCreateManagedWidget(
			"change_arc_trans_general_toggle",
			toggleWidgetClass,
			change_trans_form,
			XtNwidth,BUTTON_WIDTH,
			XtNlabel,"generalT",
			XtNfromVert,change_trans_label_1,
			XtNfromHoriz,change_trans_deterministic_toggle,
			NULL);
  XawToggleChangeRadioGroup(change_trans_deterministic_toggle,
				change_trans_immediate_toggle);
  XawToggleChangeRadioGroup(change_trans_exponential_toggle,
				change_trans_immediate_toggle);
  XawToggleChangeRadioGroup(change_trans_general_toggle,
				change_trans_immediate_toggle);
  XtOverrideTranslations(change_trans_immediate_toggle,toggle_translations);
  XtOverrideTranslations(change_trans_exponential_toggle,toggle_translations);
  XtOverrideTranslations(change_trans_deterministic_toggle,toggle_translations);
  XtOverrideTranslations(change_trans_general_toggle,toggle_translations);
  XtAddCallback(change_trans_immediate_toggle,XtNcallback,
		ChangeTranstypeState,(XtPointer)IMMEDIATE);
  XtAddCallback(change_trans_exponential_toggle,XtNcallback,
		ChangeTranstypeState,(XtPointer)EXPONENTIEL);
  XtAddCallback(change_trans_deterministic_toggle,XtNcallback,
		ChangeTranstypeState,(XtPointer)DETERMINISTIC);
  XtAddCallback(change_trans_general_toggle,XtNcallback,
		ChangeTranstypeState,(XtPointer)GENERAL);
  change_trans_label_2 = XtVaCreateManagedWidget(
			"change_trans_label_2",
			labelWidgetClass,
			change_trans_form,
			XtNforeground,0,
			XtNbackground,1,
			XtNwidth,SMALL_LABEL_WIDTH,
			XtNlabel,"Priority:",
			XtNresize,False,
			XtNjustify,XtJustifyRight,
			XtNfromVert,change_trans_deterministic_toggle,
			NULL);
  change_trans_text  = XtVaCreateManagedWidget(
			"change_trans_text",
			asciiTextWidgetClass,
			change_trans_form,
			XtNwidth,SMALL_TEXT_WIDTH2,
			XtNstring,change_trans_buf,
			XtNlength,SMALL_BUF_SIZE,
			XtNeditType,XawtextEdit,
			XtNtranslations,text_translations,
			XtNfromVert,change_trans_deterministic_toggle,	
			XtNfromHoriz,change_trans_label_2,
			NULL);
  /*new*/
  change_trans_label_3 = XtVaCreateManagedWidget(
			"change_trans_label_3",
			labelWidgetClass,
			change_trans_form,
			XtNforeground,0,
			XtNbackground,1,
			XtNwidth,SMALL_LABEL_WIDTH,
			XtNlabel,"Type of race policy:",
			XtNresize,False,
			XtNjustify,XtJustifyRight,
			XtNfromVert,change_trans_label_2,
			NULL);
  change_trans_RE_toggle = XtVaCreateManagedWidget(
			"change_trans_RE_toggle",
			toggleWidgetClass,
			change_trans_form,
			XtNwidth,BUTTON_WIDTH2,
			XtNlabel,"RE",
			XtNfromVert,change_trans_label_2,
			XtNfromHoriz,change_trans_label_3,
			NULL);
  change_trans_RA_toggle = XtVaCreateManagedWidget(
			"change_trans_RA_toggle",
			toggleWidgetClass,
			change_trans_form,
			XtNwidth,BUTTON_WIDTH2,
			XtNlabel,"RA",
			XtNfromVert,change_trans_label_2,
			XtNfromHoriz,change_trans_RE_toggle,
			NULL);
  XawToggleChangeRadioGroup(change_trans_RE_toggle,
			    change_trans_RA_toggle);
  XtOverrideTranslations(change_trans_RE_toggle,toggle_translations);
  XtOverrideTranslations(change_trans_RA_toggle,toggle_translations);
  XtAddCallback(change_trans_RE_toggle,XtNcallback,
		ChangeTranspolicyState,(XtPointer)RACE_E);
  XtAddCallback(change_trans_RA_toggle,XtNcallback,
		ChangeTranspolicyState,(XtPointer)RACE_A);

  change_delay_form = XtVaCreateWidget(
			"change_delay_form",
			formWidgetClass,
			mainBox,
			XtNborderWidth,0,
			NULL);
  change_delay_label = XtVaCreateManagedWidget(
			"change_delay_label",
			labelWidgetClass,
			change_delay_form,
			XtNwidth,SMALL_TEXT_WIDTH,
			XtNresize,False,
			XtNforeground,0,
			XtNbackground,1,
			NULL);
  change_delay_text  = XtVaCreateManagedWidget(
			"change_delay_text",
			asciiTextWidgetClass,
			change_delay_form,
			XtNwidth,SMALL_TEXT_WIDTH-6,
			XtNstring,change_delay_buf,
			XtNlength,SMALL_BUF_SIZE,
			XtNeditType,XawtextEdit,
			XtNtranslations,text_translations,
			XtNfromHoriz,change_delay_label,
			XtNdisplayCaret,True,
			NULL);

  /*new*/
  change_delay_phase_label = XtVaCreateManagedWidget(
			"change_delay_phase_label",
			labelWidgetClass,
			change_delay_form,
			XtNwidth,SMALL_TEXT_WIDTH,
			XtNlabel,"Phase:",
			XtNresize,False,
			XtNforeground,0,
			XtNbackground,1,
			XtNfromVert,change_delay_label,
			NULL);
  change_delay_phase_text  = XtVaCreateManagedWidget(
			"change_delay_phase_text",
			asciiTextWidgetClass,
			change_delay_form,
			XtNwidth,SMALL_TEXT_WIDTH-6,
			XtNstring,change_phase_buf,
			XtNlength,SMALL_BUF_SIZE,
			XtNeditType,XawtextEdit,
			XtNtranslations,text_translations,
			XtNfromHoriz,change_delay_phase_label,
			XtNfromVert,change_delay_label,
			XtNdisplayCaret,False,
			NULL); 

  change_delay_group_weight_label = XtVaCreateManagedWidget(
			"change_delay_group_weight_label",
			labelWidgetClass,
			change_delay_form,
			XtNwidth,SMALL_TEXT_WIDTH,
			XtNlabel,"Weight:",
			XtNresize,False,
			XtNforeground,0,
			XtNbackground,1,
			XtNfromVert,change_delay_phase_label,
			NULL);
  change_delay_group_weight_text  = XtVaCreateManagedWidget(
			"change_delay_group_weight_text",
			asciiTextWidgetClass,
			change_delay_form,
			XtNwidth,SMALL_TEXT_WIDTH-6,
			XtNstring,change_group_weight_buf,
			XtNlength,SMALL_BUF_SIZE,
			XtNeditType,XawtextEdit,
			XtNtranslations,text_translations,
			XtNfromHoriz,change_delay_group_weight_label,
			XtNfromVert,change_delay_phase_label,
			XtNdisplayCaret,False,
			NULL);
  /*stop new*/
  change_delay_md_button = XtVaCreateManagedWidget(
			"change_delay_md_button",
			commandWidgetClass,
			change_delay_form,
			XtNwidth,BUTTON_WIDTH,
			XtNlabel,"mark.-dep.",
			XtNfromVert,change_delay_group_weight_label,
			NULL);

  change_delay_single_button = XtVaCreateManagedWidget(
			"change_delay_single_button",
			toggleWidgetClass,
			change_delay_form,
			XtNwidth,BUTTON_WIDTH,
			XtNlabel,"single-serv.",
			XtNstate,True,
			XtNfromVert,change_delay_group_weight_label,
			XtNfromHoriz,change_delay_md_button,
			NULL);

  change_delay_inf_button = XtVaCreateManagedWidget(
			"change_delay_inf_button",
			toggleWidgetClass,
			change_delay_form,
			XtNwidth,BUTTON_WIDTH,
			XtNfromVert,change_delay_group_weight_label,
			XtNfromHoriz,change_delay_single_button,
			XtNlabel,"inf.-serv.",
			NULL);

  XawToggleChangeRadioGroup(change_delay_inf_button,change_delay_single_button);
  XtOverrideTranslations(change_delay_inf_button,toggle_translations);
  XtOverrideTranslations(change_delay_single_button,toggle_translations);
  XtAddCallback(change_delay_inf_button,XtNcallback,ChangeServState,
		(XtPointer)True);
  XtAddCallback(change_delay_single_button,XtNcallback,ChangeServState,
		(XtPointer)False);

  md_def_form = XtVaCreateWidget(
			"md_def_form",
			formWidgetClass,
			mainBox,
			XtNborderWidth,0,
			NULL);
  md_done_button = XtVaCreateManagedWidget(
                        "md_done_button",
                        commandWidgetClass,
                        md_def_form,
                        XtNlabel,"done",
                        XtNwidth,BUTTON_WIDTH,
                        NULL);
  XtAddCallback(md_done_button,XtNcallback,DialogDone,0);

  md_help_button = XtVaCreateWidget(
                        "md_help_button",
                        commandWidgetClass,
                        md_def_form,
                        XtNlabel,"help",
                        XtNwidth,BUTTON_WIDTH,
                        XtNfromHoriz,md_done_button,
                        NULL);
/* Cant do AddCallback here, because we yet dont know for what object 
   (Arc-Multipl./Trans-Delay)
  
   XtAddCallback(md_help_button,XtNcallback,PopUpHelp,"Delay.help"); 
*/

  md_cancel_button = XtVaCreateManagedWidget(
                        "md_cancel_button",
                        commandWidgetClass,
                        md_def_form,
                        XtNlabel,"cancel",
                        XtNwidth,BUTTON_WIDTH,
                        XtNfromHoriz,md_help_button,
                        NULL);
  XtAddCallback(md_cancel_button,XtNcallback,DialogCancel,0);

  md_def_label = XtVaCreateManagedWidget(
			"md_def_label",
			labelWidgetClass,
			md_def_form,
			XtNfromVert,md_done_button,	 
			XtNforeground,0,
			XtNbackground,1,
			XtNwidth,BIG_TEXT_WIDTH,
			XtNlabel,"Insert definition:",
			XtNresize,False,
			NULL);
  md_def_text  = XtVaCreateManagedWidget(
			"md_def_text",
			asciiTextWidgetClass,
			md_def_form,
			XtNwidth,BIG_TEXT_WIDTH,
			XtNheight,BIG_TEXT_HEIGHT,
			XtNstring,md_def_buf,
			XtNlength,BIG_BUF_SIZE,
			XtNeditType,XawtextEdit,	
			XtNfromVert,md_def_label,
			NULL);
  /*new*/
  change_group_form = XtVaCreateWidget(
			"change_group_form",
			formWidgetClass,
			mainBox,
			XtNborderWidth,0,
			NULL);
  change_group_label = XtVaCreateManagedWidget(
			"change_group_label",
			labelWidgetClass,
			change_group_form,
			XtNforeground,0,
			XtNbackground,1,
			XtNwidth,BIG_LABEL_WIDTH,
		/*	XtNlabel,"T1 : Group 1 ", */
			XtNresize,False,
		/*	XtNjustify,XtJustifyRight,*/
			NULL);
  change_group_new_toggle = XtVaCreateManagedWidget(
			"change_group_new_toggle",
			toggleWidgetClass,
			change_group_form,
			XtNwidth,BUTTON_WIDTH,
			XtNlabel,"New Group",
			XtNradioData,New,
			XtNfromVert,change_group_label,
			NULL);
  change_group_merge_toggle = XtVaCreateManagedWidget(
			"change_group_merge_toggle",
			toggleWidgetClass,
			change_group_form,
			XtNwidth,BUTTON_WIDTH,
			XtNlabel,"Merge Group",
			XtNradioData,Merge,
			XtNfromVert,change_group_label,
			XtNfromHoriz,change_group_new_toggle,
			NULL);
  change_group_reset_toggle = XtVaCreateManagedWidget(
			"change_group_reset_toggle",
			toggleWidgetClass,
			change_group_form,
			XtNwidth,BUTTON_WIDTH,
			XtNlabel,"Reset Group",
			XtNradioData,GroupReset,
			XtNfromVert,change_group_label,
			XtNfromHoriz,change_group_merge_toggle,
			NULL);
  XawToggleChangeRadioGroup(change_group_new_toggle,change_group_merge_toggle);
  XawToggleChangeRadioGroup(change_group_reset_toggle,change_group_merge_toggle);
  XtOverrideTranslations(change_group_new_toggle,toggle_translations);
  XtOverrideTranslations(change_group_merge_toggle,toggle_translations);
  XtOverrideTranslations(change_group_reset_toggle,toggle_translations);
  XtAddCallback(change_group_new_toggle,XtNcallback,ChangeGroupCallback,0);
  XtAddCallback(change_group_merge_toggle,XtNcallback,ChangeGroupCallback,0);
  XtAddCallback(change_group_reset_toggle,XtNcallback,ChangeGroupCallback,0);

  change_group_new_label = XtVaCreateManagedWidget(
			"change_group_new_label",
			labelWidgetClass,
			change_group_form,
			XtNwidth,SMALL_LABEL_WIDTH,
			XtNlabel,"New Group:",
			XtNresize,False,
			XtNforeground,0,
			XtNbackground,1,
			XtNfromVert,change_group_new_toggle,
			NULL);
  change_group_new_text = XtVaCreateManagedWidget(
			"change_group_new_text",
			asciiTextWidgetClass,
			change_group_form,
			XtNwidth,SMALL_TEXT_WIDTH,
			XtNstring,change_group_new_buf,
			XtNlength,SMALL_BUF_SIZE,
			XtNeditType,XawtextEdit,
			XtNtranslations,text_translations,
			XtNfromHoriz,change_group_new_label,
			XtNfromVert,change_group_new_toggle,
			XtNdisplayCaret,True,
			NULL);
			
  change_group_merge_label = XtVaCreateManagedWidget(
                        "change_group_merge_label",
			labelWidgetClass,
			change_group_form,
			XtNwidth,SMALL_LABEL_WIDTH,
			XtNlabel,"Merge with Group:",
			XtNresize,False,
			XtNforeground,0,
			XtNbackground,1,
			XtNfromVert,change_group_new_label,
			NULL);
  change_group_merge_text = XtVaCreateManagedWidget(
                        "change_group_merge_text",
			asciiTextWidgetClass,
			change_group_form,
			XtNwidth,SMALL_TEXT_WIDTH,
			XtNstring,change_group_merge_buf,
			XtNlength,SMALL_BUF_SIZE,
			XtNeditType,XawtextEdit,
			XtNtranslations,text_translations,
			XtNfromHoriz,change_group_merge_label,
			XtNfromVert,change_group_new_label,
			XtNdisplayCaret,False,
			NULL);

  main_command_form = XtVaCreateManagedWidget(
			"main_command_form",
			formWidgetClass,
			mainBox,
			XtNborderWidth,0,
			NULL);

  done_button = XtVaCreateManagedWidget(
			"done_button",
			commandWidgetClass,
			main_command_form,
			XtNlabel,"done",
			XtNwidth,BUTTON_WIDTH,
			NULL);
  XtAddCallback(done_button,XtNcallback,DialogDone,0);

  help_button = XtVaCreateWidget(
			"help_button",
			commandWidgetClass,
			main_command_form,
			XtNlabel,"help",
			XtNwidth,BUTTON_WIDTH,
			XtNfromHoriz,done_button,
			NULL);
  XtAddCallback(help_button,XtNcallback,PopUpHelp,"Delay.help");
  cancel_button = XtVaCreateManagedWidget(
			"cancel_button",
			commandWidgetClass,
			main_command_form,
			XtNlabel,"cancel",
			XtNwidth,BUTTON_WIDTH,
			XtNfromHoriz,help_button,
			NULL);
  XtAddCallback(cancel_button,XtNcallback,DialogCancel,0);

}
/*******************************************************************************
Name:
        ChangeKeyboardFocus
--------------------------------------------------------------------------------
Funktionstyp:
        Action
--------------------------------------------------------------------------------
Registriert mit:
        <Key>Tab: changeKeyboardFocus()
--------------------------------------------------------------------------------
Zweck:
       "Andern des Keybordfocus zu einem gew"unschten Textfeld
*******************************************************************************/
static void ChangeKeyboardFocus(w,event,params,numparams)
Widget w;
XEvent *event;
String *params;
Cardinal *numparams;

{

  if (edit_state.managed_form == add_form)
    {
      if (edit_state.key_board_input_text == add_text_1)
	{
	  XtSetKeyboardFocus(textPopup,add_text_2);
	  edit_state.key_board_input_text = add_text_2;
	  XtVaSetValues(add_text_1,
			XtNdisplayCaret,False,
			NULL);
	  XtVaSetValues(add_text_2,
			XtNdisplayCaret,True,
			NULL);
	}
      else
	{
	  XtSetKeyboardFocus(textPopup,add_text_1);
	  edit_state.key_board_input_text = add_text_1;
	  XtVaSetValues(add_text_2,
			XtNdisplayCaret,False,
			NULL);
	  XtVaSetValues(add_text_1,
			XtNdisplayCaret,True,
			NULL);
	}
    }
  if (edit_state.managed_form == change_delay_form)
    {
      if (edit_state.key_board_input_text == change_delay_text)      
	if (XtIsManaged(change_delay_phase_text))
	    {
	      XtSetKeyboardFocus(textPopup,change_delay_phase_text);
	      edit_state.key_board_input_text = change_delay_phase_text;
	      XtVaSetValues(change_delay_text,
			    XtNdisplayCaret,False,
			    NULL);
	      XtVaSetValues(change_delay_phase_text,
			    XtNdisplayCaret,True, 
			    NULL);
	    }
	 else
	   {
	      XtSetKeyboardFocus(textPopup,change_delay_group_weight_text);
	      edit_state.key_board_input_text = change_delay_group_weight_text;
	      XtVaSetValues(change_delay_text,
			    XtNdisplayCaret,False,
			    NULL);
	      XtVaSetValues(change_delay_group_weight_text,
			    XtNdisplayCaret,True, 
			    NULL);
	    }   
      else
	if (edit_state.key_board_input_text == change_delay_phase_text)
	{
	  if (XtIsSensitive(change_delay_group_weight_text))
	  {	  
	    XtSetKeyboardFocus(textPopup,change_delay_group_weight_text);
	    edit_state.key_board_input_text = change_delay_group_weight_text;
	    XtVaSetValues(change_delay_phase_text,
			  XtNdisplayCaret,False,
			  NULL);
	    XtVaSetValues(change_delay_group_weight_text,
			  XtNdisplayCaret,True,
			  NULL);
	  }
	}
	else
	  if (edit_state.key_board_input_text == change_delay_group_weight_text)
	  {
	    XtSetKeyboardFocus(textPopup,change_delay_text);
	    edit_state.key_board_input_text = change_delay_text;
	    XtVaSetValues(change_delay_group_weight_text,
			  XtNdisplayCaret,False,
			  NULL);
	    XtVaSetValues(change_delay_text,
			  XtNdisplayCaret,True,
			  NULL);
	  }
    }
}


/*******************************************************************************
Name:
        PreparePopup
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        XtNpopupCallback der Klasse NetWidget
--------------------------------------------------------------------------------
Parameter:
        call_date: Verweis auf die Struktur NWCallbackInfo
--------------------------------------------------------------------------------
Zweck:
       Anzeigen der gew"unschten Dialogbox an hand von call_data
	   Achtung! Bei CHANGE DELAY einer genT wird nicht das Standard-Popup
	   modifiziert und dann angezeigt, sondern pmfEdit angesprungen!
*******************************************************************************/

void PreparePopup(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{

NWCallbackInfo_2 * nwInfo = (NWCallbackInfo_2 *)call_data;
TransType trans_type;
RacePolicy race_policy;
ArcType arc_type;
Dimension c_width,width,c_height,height,screen_width,screen_height;
int TimeMode = GetTimeMode();

#ifdef debug
    printf(" PreparePopup\n");
#endif

    if (nwInfo == NULL) {
	XtCallActionProc(getCurrentNet(),"callEditAction",NULL,NULL,0);
	return;
    }
    edit_state.edit_command = nwInfo->edit_command;
    edit_state.edit_object = nwInfo->edit_object;
    edit_state.object = nwInfo->object;
	
	/* Test for CHANGE DELAY ON genTs: ***/
	/* In this case a different PopUp has to be called with pmfEdit(); */
	
	if(nwInfo->edit_command == CHANGE
	&& isTransObject(nwInfo->object)   /* Transitions-Delay soll geaendert werden */
	&& nwInfo->edit_object == RATEPAR) /* "Delay"-Toggle angewaehlt */
	{	String tname;
	        String net_Name;
                char defaultDataFileName[511];
		Rate *rate;
		
		XtVaGetValues(nwInfo->object,
			      XtNtransType,&trans_type,
			      XtNtagName,&tname,
			      XtNrates,&rate,
			      NULL);
		if(trans_type == GENERAL)
		{	
#ifdef debug
                 printf("textPopup.c:rate->type=%d\trate->values.pmf_function=%s\t ..DataFile=%s\n",
			rate->type,
			rate->values.pmf_function,
			rate->values.DataFile);
#endif
                 edit_state.type = rate->type;
                 if (rate->type==PMF_FUNC)
                 {
		  strcpy(defaultDataFileName,(char*)(getenv(MODELDIR_ENV)));
                  strcat(defaultDataFileName,"/");
                  XtVaGetValues(netName,XtNlabel,&net_Name,NULL);
		  strcat(defaultDataFileName,net_Name);
		  strcat(defaultDataFileName,"_");
		  strcat(defaultDataFileName,tname);
		  strcat(defaultDataFileName,".DAT");
		  strcat(defaultDataFileName,":RAND_UNI");
#ifdef debug
		  printf("textPopUp.c: defaultTransName = %s\n",defaultDataFileName);
#endif
                  strcpy(edit_state.pmffunc,rate->values.pmf_function);
                  strcpy(edit_state.DataFile,defaultDataFileName);
#ifdef debug
                  printf("tname=%s\tedit_state.DataFile=%s\n",tname,edit_state.DataFile);
#endif
                  pmfEdit(tname,&(edit_state.type),edit_state.pmffunc,edit_state.DataFile,0); /* 0 Means editable */
                 }
                 else
                 {
                  char temp[255];
         
                  strcpy(edit_state.pmffunc,DEFAULT_PMF_FUNCTION);
                  strcpy(edit_state.DataFile,rate->values.DataFile);
                  if(!strncmp(edit_state.DataFile,"$MODELDIR",strlen("$MODELDIR")))
                  {
                   strcpy(temp,edit_state.DataFile+strlen("$MODELDIR"));
                   strcpy(edit_state.DataFile,(char*)(getenv(MODELDIR_ENV)));
                   strcat(edit_state.DataFile,temp);
                  }
#ifdef debug
                  printf("PreparePopup():edit_state.DataFile=%s\n",edit_state.DataFile);
#endif
                  pmfEdit(tname,&(edit_state.type),edit_state.pmffunc,edit_state.DataFile,0); /* 0 Means editable */
                 }
                 return;
		}
	}
	
	/*** Otherwise Start Konfiguring && Displaying the Standard Text PopUp ***/
    /*** for changing the Rate of a Transition.. ***/
	XtVaGetValues(main_command_form,
		  XtNwidth,&c_width,
		  XtNheight,&c_height,
		  NULL);
    switch(nwInfo->edit_command) {
      case ADD:
	if (isMarkParObject(nwInfo->object) || 
	    isRateParObject(nwInfo->object)) {
	    edit_state.managed_form = add_form;
	    edit_state.key_board_input_text = add_text_1;
	    if (isMarkParObject(nwInfo->object)) {
	      XtVaSetValues(textPopup,
			      XtNtitle,"Add a marking parameter",
			    NULL);
	      XtUnmanageChild(add_Ddef_button);
	    }
	    else {
		XtVaSetValues(textPopup,
			    XtNtitle,"Add a delay parameter",
			      NULL);
		XtManageChild(add_Ddef_button);
	    } 
	    /* popup groesse anpassen */
	    XtManageChild(add_form);
	    if (XtIsRealized(textPopup)) {
		XtVaGetValues( add_form,
			      XtNwidth,&width,
			      XtNheight,&height,
			      NULL);
		XtResizeWidget(textPopup,
			       Max(width,c_width),
			       height+c_height,
			       1);
	    }
	}
        break;
    case CHANGE:
	if (nwInfo->edit_object == TAG)
	{
	    edit_state.managed_form = change_form;	
	    edit_state.key_board_input_text = change_text;
	    XtManageChild(change_form);
	    XtVaSetValues(change_label,
			XtNlabel,"New tag:",
			NULL);
	    XtVaSetValues(textPopup,
			XtNtitle,"Change the tag",
			NULL);
	    /* popup groesse anpassen */
	    if (XtIsRealized(textPopup))
	    {
  		XtVaGetValues(change_form,
				XtNwidth,&width,
				XtNheight,&height,
				NULL);
		XtResizeWidget(textPopup,
				Max(width,c_width),
				height+c_height,
				1);
	    }
	}
	else
	if (isMarkParObject(nwInfo->object) || isRateParObject(nwInfo->object))
	{
	    edit_state.managed_form = change_form;
	    edit_state.key_board_input_text = change_text;
	    XtVaSetValues(change_label,
			XtNlabel,"New value:",
			NULL);
	    if (isMarkParObject(nwInfo->object)) {
		XtVaSetValues(textPopup,
			      XtNtitle,"Change the value of the marking parameter",
			      NULL);
		XtUnmanageChild(change_Ddef_button);
	    }
	    else {
		XtVaSetValues(textPopup,
			      XtNtitle,"Change the value of the delay parameter",
			      NULL);
		XtManageChild(change_Ddef_button);
	    }
	    /* popup groesse anpassen */
	    XtManageChild(change_form);
	    if (XtIsRealized(textPopup))
	    {
  		XtVaGetValues(change_form,
				XtNwidth,&width,
				XtNheight,&height,
				NULL);
		XtResizeWidget(textPopup,
				Max(width,c_width),
				height+c_height,
				1);
	    }
	}
	else
	if (isPlaceObject(nwInfo->object))
	{
	    edit_state.managed_form = change_form;
	    edit_state.key_board_input_text = change_text;
	    XtManageChild(change_form);
	    XtVaSetValues(change_label,
			XtNlabel,"Value or parameter:",
			NULL);
	    XtVaSetValues(textPopup,
			XtNtitle,"Change the number of tokens in the place",
			NULL);
	    /* popup groesse anpassen */
	    if (XtIsRealized(textPopup))
	    {
  		XtVaGetValues(change_form,
				XtNwidth,&width,
				XtNheight,&height,
				NULL);
		XtResizeWidget(textPopup,
				Max(width,c_width),
				height+c_height,
				1);
	    }
	}
	else
	if (isArcObject(nwInfo->object))
	{
	    edit_state.managed_form = change_arc_form;
	    edit_state.key_board_input_text = change_arc_text;
	    XtManageChild(change_arc_form);
	    XtVaGetValues(nwInfo->object,
			XtNarcType,&arc_type,
			NULL);
	    XawToggleUnsetCurrent(change_arc_input_toggle);
	    if (arc_type == INPUTARC)
     	    {
		edit_state.arctype = INPUTARC;
		XtVaSetValues(change_arc_input_toggle,
				XtNstate,True,
				NULL);
	    }
	    else
	    if (arc_type == OUTPUTARC)
     	    {
		edit_state.arctype = OUTPUTARC;
		XtVaSetValues(change_arc_output_toggle,
				XtNstate,True,
				NULL);
	    }
	    else
	    if (arc_type == INHIBITORARC)
     	    {
		edit_state.arctype = INHIBITORARC;
		XtVaSetValues(change_arc_inhibitor_toggle,
				XtNstate,True,
				NULL);
	    }
	    XtVaSetValues(textPopup,
			XtNtitle,"Change the type of the arc",
			NULL);
	    /* popup groesse anpassen */
	    if (XtIsRealized(textPopup))
	    {
  		XtVaGetValues(change_arc_form,
				XtNwidth,&width,
				XtNheight,&height,
				NULL);
		XtResizeWidget(textPopup,
				Max(width,c_width),
				height+c_height,
				1);
	    }
	}
	else
	if (isTransObject(nwInfo->object) && ((nwInfo->edit_object != RATEPAR) 
					      && (nwInfo->edit_object != GROUP)))
	{
	    edit_state.managed_form = change_trans_form;
	    edit_state.key_board_input_text = change_trans_text;
	    XtManageChild(change_trans_form);
	    XtVaGetValues(nwInfo->object,
			  XtNtransType,&trans_type,
			  XtNracePolicy,&race_policy,
			  NULL);
	    if (TimeMode == DiscTime) 
	    {
	      XtVaSetValues(change_trans_exponential_toggle,
			    XtNlabel,"geometric",
			    NULL);
	      XtSetSensitive(change_trans_label_3,True);
	      XtSetSensitive(change_trans_RE_toggle,True);
	      XtSetSensitive(change_trans_RA_toggle,True);
	    }
	    else
	    {
	      XtVaSetValues(change_trans_exponential_toggle,
			    XtNlabel,"exponential",
			    NULL);
	      XtSetSensitive(change_trans_label_3,False);
	      XtSetSensitive(change_trans_RE_toggle,False);
	      XtSetSensitive(change_trans_RA_toggle,False);
	    }
	    XawToggleUnsetCurrent(change_trans_immediate_toggle);
	    XawToggleUnsetCurrent(change_trans_RA_toggle);
	    if (race_policy == RACE_A) 
	    {
	      edit_state.racepolicy = RACE_A;
	      XtVaSetValues(change_trans_RA_toggle,
			    XtNstate,True,
			    NULL);
	    }
	    else
	    {
	      edit_state.racepolicy = RACE_E;
	      XtVaSetValues(change_trans_RE_toggle,
				XtNstate,True,
				NULL);
	    }
	    if (trans_type == IMMEDIATE)
	    {
	        edit_state.transtype = IMMEDIATE;
	        XtVaSetValues(change_trans_immediate_toggle,
				XtNstate,True,
				NULL);
		XtVaSetValues(change_trans_label_2,
			      XtNlabel,"Priority:",
			      NULL);
		XtSetSensitive(change_trans_label_2,True);
		XtSetSensitive(change_trans_text,True);
		XtSetSensitive(change_trans_label_3,False);
		XtSetSensitive(change_trans_RE_toggle,False);
		XtSetSensitive(change_trans_RA_toggle,False);
	    }
	    else
	    if (trans_type == EXPONENTIEL)
	    {
		edit_state.transtype = EXPONENTIEL;
		XtVaSetValues(change_trans_exponential_toggle,
				XtNstate,True,
				NULL);
		if (TimeMode == DiscTime) {	/*new*/
		  XtVaSetValues(change_trans_label_2,
				XtNlabel,"Post-Priority:",
				NULL);
		  XtSetSensitive(change_trans_label_2,True);
		  XtSetSensitive(change_trans_text,True);
		}
		  else {
		    XtVaSetValues(change_trans_label_2,
				XtNlabel,"Priority:",
				NULL);
		    XtSetSensitive(change_trans_label_2,False);
		    XtSetSensitive(change_trans_text,False);
		  }
	    }
	    else
	    if (trans_type == DETERMINISTIC)
	    {
		edit_state.transtype = DETERMINISTIC;
		XtVaSetValues(change_trans_deterministic_toggle,
				XtNstate,True,
				NULL);
		if (TimeMode == DiscTime) {	/*new*/
		  XtVaSetValues(change_trans_label_2,
				XtNlabel,"Post-Priority:",
				NULL);
		  XtSetSensitive(change_trans_label_2,True);
		  XtSetSensitive(change_trans_text,True);
		}
		else {
		  XtVaSetValues(change_trans_label_2,
				XtNlabel,"Priority:",
				NULL);
		  XtSetSensitive(change_trans_label_2,False);
		  XtSetSensitive(change_trans_text,False);
		}
	    }
	    else
	    if (trans_type == GENERAL)
	    {
		edit_state.transtype = GENERAL;
		XtVaSetValues(change_trans_general_toggle,
				XtNstate,True,
				NULL);		
		if (TimeMode == DiscTime) {	/*new*/
		  XtVaSetValues(change_trans_label_2,
				XtNlabel,"Post-Priority:",
				NULL);
		  XtSetSensitive(change_trans_label_2,True);
		  XtSetSensitive(change_trans_text,True);
		}
		else {
		  XtVaSetValues(change_trans_label_2,
				XtNlabel,"Priority:",
				NULL);
		  XtSetSensitive(change_trans_label_2,False);
		  XtSetSensitive(change_trans_text,False);
		}

	    }
	    XtVaSetValues(textPopup,
			XtNtitle,"Change the type of the transition",
			NULL);
	    /* popup groesse anpassen */
	    if (XtIsRealized(textPopup))
	    {
  		XtVaGetValues(change_trans_form,
				XtNwidth,&width,
				XtNheight,&height,
				NULL);
		XtResizeWidget(textPopup,
				Max(width,c_width),
				height+c_height,
				1);
	    }
	}
	else
	if (isTransObject(nwInfo->object) && (nwInfo->edit_object == RATEPAR))
	{
		XtVaGetValues(nwInfo->object,
			XtNtransType,&trans_type,
			NULL);
		if (trans_type == IMMEDIATE)
		{
		    XtVaSetValues(textPopup,
				  XtNtitle,
			       "Change weight of an immediate transition",
				  NULL);
		    XtVaSetValues(change_delay_label,
				  XtNlabel,"Weight or parameter:",
				  NULL);
/* 		    if (TimeMode == DiscTime) */
/* 		      XtSetSensitive(change_delay_md_button, False); */
/* 		    else { */
		      XtSetSensitive(change_delay_md_button, True);
		      XtAddCallback(change_delay_md_button, XtNcallback,
				    ShowMarkDepEditor, 
				    (XtPointer)IMMEDIATE_MD);
/* 		    } */
		    XtUnmanageChild(change_delay_inf_button);
		    XtUnmanageChild(change_delay_single_button);
		    /*new*/
		    XtUnmanageChild(change_delay_phase_label);
		    XtUnmanageChild(change_delay_phase_text);
		    XtUnmanageChild(change_delay_group_weight_label);
		    XtUnmanageChild(change_delay_group_weight_text);
		} 
		else
		if (trans_type == EXPONENTIEL)
		  if (TimeMode == ContTime)
		  {
		    XtVaSetValues(textPopup,
				  XtNtitle,"Change delay of an exp. timed transition",
				  NULL);
		    XtVaSetValues(change_delay_label,
				  XtNlabel,"Delay or parameter:",
				  NULL);
		    XtSetSensitive(change_delay_md_button,True);
		    XtSetSensitive(change_delay_inf_button,True);
		    XtSetSensitive(change_delay_single_button,True);
		    XtAddCallback(change_delay_md_button, XtNcallback,
				  ShowMarkDepEditor, 
				  (XtPointer)EXPONENTIEL_MD);
		    /*new*/
		    XtSetSensitive(change_delay_phase_label,False);
		    XtSetSensitive(change_delay_phase_text,False);
		    XtSetSensitive(change_delay_group_weight_label,False);
		    XtSetSensitive(change_delay_group_weight_text,False);
		  }
		  else    /*new...we're in discrete time...*/
		  {
		    XtSetSensitive(change_delay_phase_label,True);
		    XtSetSensitive(change_delay_phase_text,True);
		    XtSetSensitive(change_delay_group_weight_label,True);
		    XtSetSensitive(change_delay_group_weight_text,True);
		    XtVaSetValues(textPopup,
				  XtNtitle,"Change delay or phase ",
				  NULL);
		    XtVaSetValues(change_delay_label,
				  XtNlabel,"Delay or parameter:",
				  NULL);
/* 		    XtSetSensitive(change_delay_md_button,False); */
		    XtSetSensitive(change_delay_md_button,True);
		    XtAddCallback(change_delay_md_button, XtNcallback,
				  ShowMarkDepEditor, 
				  (XtPointer)EXPONENTIEL_MD);

		    XtSetSensitive(change_delay_inf_button,False);
		    XtSetSensitive(change_delay_single_button,False);
		  } 
		if (trans_type == DETERMINISTIC)
		{
		  XtVaSetValues(textPopup,
				  XtNtitle,"Change delay of a det. timed transition",
				  NULL);
		  XtVaSetValues(change_delay_label,
				  XtNlabel,"Delay or parameter:",
				  NULL);
		  /*new*/
		  XtUnmanageChild(change_delay_phase_label);
		  XtUnmanageChild(change_delay_phase_text);
		  if (TimeMode == DiscTime) {
		   /*  XtSetSensitive(change_delay_md_button,False); */
		    XtSetSensitive(change_delay_md_button,True);
		    XtSetSensitive(change_delay_inf_button,False);
		    XtSetSensitive(change_delay_group_weight_label,True);
		    XtSetSensitive(change_delay_group_weight_text,True);
		    XtAddCallback(change_delay_md_button, XtNcallback,
				  ShowMarkDepEditor, 
				  (XtPointer)DETERMINISTIC_MD);
		  }
		  else {
		    XtSetSensitive(change_delay_group_weight_label,False);
		    XtSetSensitive(change_delay_group_weight_text,False);
		    XtSetSensitive(change_delay_md_button,True);
		    XtSetSensitive(change_delay_inf_button,True);
		    XtAddCallback(change_delay_md_button, XtNcallback,
				  ShowMarkDepEditor, 
				  (XtPointer)DETERMINISTIC_MD);
		  }
		}
		edit_state.key_board_input_text = change_delay_text; 
		XtVaSetValues(change_delay_group_weight_text,
			      XtNdisplayCaret,False,
			      NULL);
		XtVaSetValues(change_delay_phase_text,
			      XtNdisplayCaret,False,
			      NULL);
		XtVaSetValues(change_delay_text,
			      XtNdisplayCaret,True,
			      NULL);
		edit_state.managed_form = change_delay_form;
		XtManageChild(change_delay_form);
		if (XtIsRealized(textPopup))
		{
		  XtVaGetValues(change_delay_form,
				XtNwidth,&width,
				XtNheight,&height,
				NULL);
		  XtResizeWidget(textPopup,
				 Max(width,c_width),
				 height+c_height,
				 1);
		}
	}
	else
	if (isTransObject(nwInfo->object) && (nwInfo->edit_object == GROUP))
	{String tag ;
	int group = 1 ;
	char label[128];
	        edit_state.key_board_input_text = change_group_new_text; 
		edit_state.managed_form = change_group_form;
		XtVaSetValues(textPopup,
				  XtNtitle,"Change group of a transition",
				  NULL);
		XtVaGetValues(nwInfo->object,
			      XtNtagName,&tag,
			      XtNgroup,&group,
			      NULL);
		if (group < 1) 
		  sprintf(label,"Transition %s : No Group specified ",tag);
		else  sprintf(label,"Transition %s in Group %d",tag,group);
		XtVaSetValues(change_group_label,
				  XtNlabel,label,
				  NULL);
		XtSetSensitive(change_group_merge_label,False);
		XtSetSensitive(change_group_merge_text,False);
		XawToggleSetCurrent(change_group_new_toggle,(XtPointer)New);
		XtManageChild(change_group_form);	
		if (XtIsRealized(textPopup))
		{
		       XtVaGetValues(change_group_form,
				     XtNwidth,&width,
				     XtNheight,&height,
				     NULL);
		       XtResizeWidget(textPopup,
				      Max(width,c_width),
				      height+c_height,
				      1);
	 }
	}
	break;
      default:
	break;
    }

  XtManageChild(main_command_form);

  if (!XtIsRealized(textPopup))
    XtRealizeWidget(textPopup);

  XtVaGetValues(textPopup,
		XtNwidth,&width,
		XtNheight,&height,
		NULL);

  x_event = (Position)nwInfo->root_x - (Position)width/2;
  y_event = (Position)nwInfo->root_y - (Position)height/2;

  screen_width = (Dimension)WidthOfScreen(XtScreen(textPopup));
  screen_height = (Dimension)HeightOfScreen(XtScreen(textPopup));

  if (y_event < 0)
    y_event = 0;
  else
    if (y_event + (Position)height > (Position)screen_height)
      y_event = (Position)screen_height - (Position)height - (Position)height/4;

  if (x_event < 0)
    x_event = 0;
  else
    if (x_event + (Position)width > (Position)screen_width )
      x_event = (Position)screen_width - (Position)width;

  XtMoveWidget(textPopup,x_event,y_event);
  XtPopup(textPopup,XtGrabNonexclusive);
  XtSetKeyboardFocus(textPopup,edit_state.key_board_input_text);
}

/*******************************************************************************
Name:
        pmfDialogDone
--------------------------------------------------------------------------------
Funktionstyp:
        Callback (exportiert nach pmfEdit.c)
--------------------------------------------------------------------------------
Registriert mit:
        doneButton des pmfEditPopups
--------------------------------------------------------------------------------
Zweck:
        Uebernimmt die Funktionalitaet von DialogDone (Weiterleiten
		des verstellten Rate-Parameters) fuer den Spezialfall Change Delay
		auf eine genT (pmfEdit).., also:
      	ggf geaenderte pmf-Function an die Klasse NetWidget weiterleiten
*******************************************************************************/

void pmfDialogDone(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{	
	String params[5];
	int num_params = 5;
	Rate *rate;
		
	XtVaGetValues(edit_state.object,
	    XtNrates,&rate,
			NULL);
#ifdef debug
        printf("edit.type=%d\tedit.pmffunc=%s\tedit.DataFile=%s\n",
           	edit_state.type,
                edit_state.pmffunc,
                edit_state.DataFile);
#endif
        rate->type=edit_state.type;        
        if (edit_state.type==PMF_FUNC)
        {
	  /*if(noDataFileTransition>0) noDataFileTransition-=1;
         if(noDataFileTransition==0) onlySimulation(0);*/
         /* Wurde die pmf-Function in pmfEdit veraendert?? */
         params[0] =strcmp(edit_state.pmffunc,rate->values.pmf_function)==0?
		 XtNewString("") : XtNewString(edit_state.pmffunc);
 	 params[1] = XtEpmf_function;     /* ="pmf_function":definiert in TransObj.h */
        }
        else
        {
         /* Wurde das DataFile in pmfEdit veraendert?? */
         char temp[255];
         int templen;
         
         strcpy(temp,(char*)(getenv(MODELDIR_ENV)));
         templen = strlen(temp);
         /*noDataFileTransition=1;
         if (noDataFileTransition==1) onlySimulation(1);*/
         if(!strncmp(edit_state.DataFile,temp,templen))
         {
          strcpy(temp,edit_state.DataFile+templen);
          strcpy(edit_state.DataFile,"$MODELDIR");
          strcat(edit_state.DataFile,temp);
         }
#ifdef debug
         printf("pmfDialogDone:edit_state.DataFile=%s\n",edit_state.DataFile);
#endif
         params[0] =strcmp(edit_state.DataFile,rate->values.DataFile)==0?
		 XtNewString("") : XtNewString(edit_state.DataFile);
 	 params[1] = XtEbrowse_file;
        }
        params[2] = XtEsingle_server;
	params[3] == NULL;
	params[4] == NULL;
#ifdef debug
        printf("rate->values.pmf_function=%s\n",rate->values.pmf_function);
        printf("params[1]=%s\n",params[1]);
#endif
        XtCallActionProc(getCurrentNet(),"callEditAction",NULL,params,num_params);
	XtFree(params[0]);
	return;		
}

/*******************************************************************************
Name:
        DialogDone
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        doneButton
--------------------------------------------------------------------------------
Zweck:
       Gesammelte Information aus edit_state an die Klasse NetWidget weiter-
       leiten
*******************************************************************************/

static void DialogDone(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
    NetWidget nw = (NetWidget)getCurrentNet();
    String  buf,*params;
    Cardinal num_params = 0;
    Boolean succ = True;
    int error;

#ifdef debug
    printf("textPopup.c: DialogDone \n");
#endif

    if (edit_state.md_def) /* TEST FOR SEMIKOLON */
    {
	int i;
	XtVaGetValues(md_def_text,  
		      XtNstring,&buf,          
		      NULL);

	/* last non white has to be a semikolon */
	for (i=strlen(buf)-1; i>=0; i--)
	{
	    if (isspace(buf[i]))
		continue;
	    if (buf[i]!=';')
	    {
		XtWarning
		    ("Don't forget the \";\" at the end of an definition!");
		return;
	    }
	    else 
		break;
	}
    }

    switch(edit_state.edit_command)
    {

    case ADD:  /* ADD ADD ADD ADD ADD ADD ADD ADD ADD ADD ADD ADD ADD */
#ifdef debug
        printf("ADD\n");
#endif
	if (isMarkParObject(edit_state.object) ||        /* MARKING */
	    isRateParObject(edit_state.object))          /* RATEPAR */
	{
	    params = (String*)XtCalloc(3,sizeof(String));
	    num_params = 2;
	    XtVaGetValues(add_text_1,
			  XtNstring,&buf,
			  NULL);
	    if (isIdentifier(buf))
		params[0] = getWord(buf);
	    else
	    {
		PopUpInfo(w,(XtPointer)"Identifier.help",NULL);
		succ = False;
		break;
	    }
	    XtVaGetValues(add_text_2,
			  XtNstring,&buf,
			  NULL);

	    /* Eingabe abfragen */

            if (isMarkParObject(edit_state.object))         /* MARKING */
	    {
		if(isMarking(buf))
		    params[1] = getWord(buf);
		else
		{
		    PopUpInfo(w,(XtPointer)"Marking.help",NULL);
		    succ = False;
		}
	    }
	    else
	    if (isRateParObject(edit_state.object))        /* RATEPAR */
	    {
		if (edit_state.md_def)            /* RATEPAR DEFINITION */
		{
		    RateValue newRate;
		    String netName;
		    
		    XtVaGetValues(getCurrentNet(),
				  XtNmodelName,&netName,
				  NULL);
		    XtVaGetValues(md_def_text,  
				  XtNstring,&buf,          
				  NULL);
		    
		    /* make the new Rate */
		    params[1] = MD;
		    params[2] = buf;
		    num_params = 3;
		    
		    newRate.valueType = Definition;
		    newRate.values.definition = params[2];

		    /* 1. SET THE VALUES */
		    XtVaSetValues(edit_state.object,
				  XtNrateParName, params[0],
				  XtNrateParValue, &newRate,
				  NULL);

		    /* 2. CHECK THE SYNTAX */
		    if (error = checkSyntax())
			{
			   /* RatePar has to be removed! (But how?) */
  			   return;
		    }
		}
		else                               /* RATEPAR VALUE */
		{ 
		    if(isFloat(buf))
			params[1] = getWord(buf);
		    else
		    {
			PopUpInfo(w,(XtPointer)"Float.help",NULL);
			succ = False;
		    }
		}
	    }
	}
	break;
	
    case CHANGE:  /* CHANGE CHANGE CHANGE CHANGE CHANGE CHANGE CHANGE */
	
	if (edit_state.edit_object == TAG)                   /* TAG */
	{
	    XtVaGetValues(change_text,
			XtNstring,&buf,
			NULL);
	    if(isNoInput(buf))
	      {
		params = (String*)NULL;
		num_params = 0;
	      }
	    else
	    if (isIdentifier(buf))
	      {
		params = (String*)XtCalloc(1,sizeof(String));
		num_params = 1;
		params[0] = getWord(buf);
	      }
	    else
	      {
		PopUpInfo(w,(XtPointer)"Identifier.help",NULL);
		succ = False;
		break;
	      }
	}
	else

	if (isMarkParObject(edit_state.object))             /* MARKING */
	{
	    XtVaGetValues(change_text,
			  XtNstring,&buf,
			  NULL);
	    /* Eingabe abfragen */
	    if(isNoInput(buf))
	    {
		params = (String*)NULL;
		num_params = 0;
	    }
	    else
		if(isMarking(buf))
		{
		    params = (String*)XtCalloc(1,sizeof(String));
		    num_params = 1;
		    params[0] = getWord(buf);
		}
		else
		{
		    PopUpInfo(w,(XtPointer)"Marking.help",NULL);
		    succ = False;
		}
	}
	else
	    
	if (isRateParObject(edit_state.object))             /* RATEPAR */
	{
#ifdef debug
            printf("Change => RatePar\n");
#endif
            if (edit_state.md_def)            /* RATEPAR DEFINITION */
	    {
		RateValue *currRate, newRate;
		String netName;
		String name;
		int i;
		
		XtVaGetValues(getCurrentNet(),
			      XtNmodelName,&netName,
			      NULL);
		XtVaGetValues(md_def_text,  
			      XtNstring,&buf,          
			      NULL);

		/* get the current Rate */
                XtVaGetValues(edit_state.object,
			      XtNrateParName, &name,
			      XtNrateParValue, &currRate,
			      NULL);


		newRate.valueType = Definition;
		newRate.values.definition = buf;
		XtVaSetValues(edit_state.object,
			      XtNrateParValue, &newRate,
			      NULL);
		
		if (error = checkSyntax()) {
		    XtVaSetValues (edit_state.object,
				   XtNrateParValue, currRate,
				   NULL);
                    return;
                }

		/* reset for redraw */
		XtVaSetValues (edit_state.object,
			       XtNrateParValue, currRate,
			       NULL);
		
		params = (String*)XtCalloc(2,sizeof(String));
		num_params = 2;		
		params[0] = MD;
		params[1] = buf;
		

	    }
	    else                              /* RATEPAR VALUE */
	    {
		XtVaGetValues(change_text,
			      XtNstring,&buf,
			      NULL);
		if(isNoInput(buf))
		{
		    params = (String*)NULL;
		    num_params = 0;
		}
		else
		    if(isFloat(buf))
		    {
			params = (String*)XtCalloc(1,sizeof(String));
			num_params = 1;
			params[0] = getWord(buf);
		    }
		    else
		    {
			PopUpInfo(w,(XtPointer)"Float.help",NULL);
			succ = False;
		    }
	    }
	}
	else

	if (isPlaceObject(edit_state.object))               /* PLACE */
	{
	    XtVaGetValues(change_text,
			  XtNstring,&buf,
			  NULL);
	    if(isNoInput(buf))
	    {
		params = (String*)NULL;
		num_params = 0;
	    }
	    else
	    {
		params = (String*)XtCalloc(1,sizeof(String));
		num_params = 1;
		if(isMarking(buf)) 
		    params[0] = getWord(buf);
		else
		    if (isIdentifier(buf))
		    {
			params[0] = getWord(buf);
			if(!parameterAvailable(nw,
					       params[0],MARKPAR))
			{
			    succ = False;
			    PopUpInfo(w,(XtPointer)"NoParam.help",NULL);
			}
		    }
		    else
		    {
			succ = False;
			PopUpInfo(w,(XtPointer)"Marking_2.help",NULL);
		    }
	    }
	}
	else

	if (isArcObject(edit_state.object))                     /* ARC */
	{
	    params = (String*)XtCalloc(3,sizeof(String));
	    params[0]=params[1]=params[2]=NULL;
	    num_params = 3;
	    XtVaGetValues(change_arc_text, /* f"ur Multiplizit"at */
			  XtNstring,&buf,
			  NULL);
	    /* Eingabe abfragen: Leere Multipl. bei MD-Def. erlaubt!*/
	    if(isNoInput(buf)          /* True bei einfacher MD-Eingabe */
	       && !edit_state.md_def) /* beides leere Eingaben ? */
		params[0] = NULL;    /* dann .. */
	    else

	    if (edit_state.md_def) 
	    {
		String fromName, toName, name=NULL;
		Widget from, to;
		Multiplicity *currMultiplicity, newMultiplicity;
		String netName;
		int i;

		XtVaGetValues(getCurrentNet(),
			      XtNmodelName,&netName,
			      NULL);

	     	XtVaGetValues(md_def_text, /* f"ur MD-Definition */
 			XtNstring,&buf,    
			NULL);

		/* SYNTAX CHECK */

		/* get TYPE of arc and NAMES of connected objects */
		XtVaGetValues(edit_state.object,
			      XtNfromObj, &from,
			      XtNtoObj, &to,
			      XtNmultiplicity, &currMultiplicity,
			      NULL);
		XtVaGetValues(from, 
			      XtNtagName, &fromName,
			      NULL);
		XtVaGetValues(to, 
			      XtNtagName, &toName,
			      NULL);
		vlstrcpy(&name, (edit_state.arctype == INPUTARC ?
				 "INPARC " : 
				 (edit_state.arctype == OUTPUTARC ?
				  "OUTPARC " : "INHARC ")));
		if (isTransObject(from)) {
		    vlstrcat(&name, fromName); 
		    vlstrcat(&name, " ");
		    vlstrcat(&name, toName);
		}
		else {
		    vlstrcat(&name, toName); 
		    vlstrcat(&name, " ");
		    vlstrcat(&name, fromName);
		}
		
	    newMultiplicity.type = MD_DEF;
	    newMultiplicity.values.md_definition = buf;
	    XtVaSetValues (edit_state.object,
			   XtNmultiplicity, &newMultiplicity,
			   NULL);

		if (error = checkSyntax()) {
			String *para = (String*)XtMalloc(sizeof(String));

			XtVaSetValues (edit_state.object,
				       XtNmultiplicity, currMultiplicity,
				       NULL);
			return;
		}
		
		/* need to reset the multipl. in edit_state.object 
		   for drawing the changed comment onto the arc */
		XtVaSetValues (edit_state.object,
			       XtNmultiplicity, currMultiplicity,
			       NULL);
		
		params[0] = XtNewString(buf);
		params[2] = XtEtrue;
	    }
	    else
	    {
		if(isMultiplicity(buf))
		    /* [2] nicht false setzen, sonst geht Change schief */
		    params[0] = getWord(buf); 
		else
		{
		    PopUpInfo(w,(XtPointer)"Arc.help",NULL);
		    succ = False;
		    break;
		}
	    }
	    if (edit_state.arctype == INPUTARC)
     	    {
		params[1] = XtEinputArc;
	    }
	    else
	    if (edit_state.arctype == OUTPUTARC)
     	    {
		params[1] = XtEoutputArc;
	    }
	    else
	    if (edit_state.arctype == INHIBITORARC)
     	    {
		params[1] = XtEinhibitorArc;
	    }
	}
	else

	if (isTransObject(edit_state.object) &&         /* TRANS && ! RATEPAR  && !GROUP*/
	    ((edit_state.edit_object != RATEPAR) && (edit_state.edit_object !=GROUP)))
	{
#ifdef debug
            printf("textpopup.c:DiologDone():TRANS && !RATEPAR\n");
#endif
	    params = (String*)XtCalloc(3,sizeof(String));
	    num_params = 3;
/* 	    if (edit_state.transtype != GENERAL) Auch zeitbehaftete Transitionen koennen eine PRIORITAET haben */
/* 	    {                                      und zwar alle */
		XtVaGetValues(change_trans_text,
			      XtNstring,&buf,
			      NULL);
		/* Eingabe abfragen */
		if (isNoInput(buf))
		    params[0]=NULL;
		else
		    if (isPriority(buf))
			params[0] = getWord(buf);
		    else
		    {
			PopUpInfo(w,(XtPointer)"Priority.help",NULL);
			succ = False;
			break;
		    }
/* 	    } */
	    if (edit_state.transtype == EXPONENTIEL)
	      params[1] = XtEexponential;
	    else
	      if (edit_state.transtype == DETERMINISTIC)
		params[1] = XtEdeterministic;	    
	      else
		if (edit_state.transtype == IMMEDIATE)
		  params[1] = XtEimmediate;
		else
		  if (edit_state.transtype == GENERAL)
		  {
/* 		    params[0]=NULL; */
		    params[1] = XtEgeneral;
		  }
	    if (edit_state.racepolicy == RACE_E)
	      params[2] = XtErace_e;
	    else
	      if (edit_state.racepolicy == RACE_A)
		params[2] = XtErace_a;
	}
	else
	  if (isTransObject(edit_state.object) &&          /* TRANS && RATEPAR */
	      edit_state.edit_object == RATEPAR)
	  {
	    TransType trans_type;
	    String name;
	    
	    params = (String*)XtCalloc(5,sizeof(String));
	    num_params = 5;
	    if (edit_state.md_def)
	    {
	      int i;
		Rate *rateCurr, rateNew;
		String netName;

		XtVaGetValues(getCurrentNet(),
			      XtNmodelName,&netName,
			      NULL);

	     	XtVaGetValues(md_def_text,
			XtNstring,&buf,
			NULL);
		
		/* -- S Y N T A X   C H E C K  -- */
		XtVaGetValues(edit_state.object,
			      XtNtransType, &trans_type,
			      XtNtagName, &name,
			      XtNrates, &rateCurr,
			      NULL);

	    rateNew.type = MD_DEF;
	    rateNew.values.md_definition = buf;
	    XtVaSetValues (edit_state.object,
			   XtNrates, &rateNew,
			   NULL);

		if (error = checkSyntax()) {

		    /* RESTORE the old Values */
		    if (rateCurr->type != MD_DEF) {
			String *para = (String*)XtMalloc(sizeof(String));

			XtVaSetValues (edit_state.object,
				       XtNrates, rateCurr,
				       NULL);
		    }
			return ;
		}

		/* need to reset the delay in edit_state.object 
		   for drawing the changed delay onto the transition */
		XtVaSetValues (edit_state.object,
			       XtNrates, rateCurr,
			       NULL);

		params[0] = XtNewString(buf);
		params[1] = XtEmarking_dependent;
	    }
	    else
	    {
		XtVaGetValues(change_delay_text,
			      XtNstring,&buf,
			      NULL);
		if (isNoInput(buf))
		    params[0]=NULL;
		else
		    if (isFloat(buf))
			params[0] = getWord(buf);
		    else
			if (isIdentifier(buf))
			{
			    params[0] = getWord(buf);
			    if (! parameterAvailable(nw,
						     params[0],RATEPAR))
			    {
				succ = False;
				PopUpInfo(w,(XtPointer)"NoParam.help",NULL);
			    }
			}      
			else
			{
			    succ = False;
			    PopUpInfo(w,(XtPointer)"Float_2.help",NULL);
			    break;
			}
		params[1] = XtEstandard_rate;
	    }
	    if (edit_state.inf_serv)
		params[2] = XtEinfinite_server;
	    else
		params[2] = XtEsingle_server;
	    /*new*/
	    XtVaGetValues(change_delay_phase_text,
			  XtNstring,&buf,
			  NULL);
	    if (isNoInput(buf))
	      params[3]=NULL;
	    else
	      if (isCardinal(buf))
		params[3] = getWord(buf);
	      else 
	      {
		succ = False;
		PopUpInfo(w,(XtPointer)"Cardinal.help",NULL);
		break;
	      }
	    XtVaGetValues(change_delay_group_weight_text,
			  XtNstring,&buf,
			  NULL);
	    if (isNoInput(buf))
	      params[4]=NULL;
	    else
	      if (isFloat(buf))
		params[4] = getWord(buf);
	      else 
	      {
		succ = False;
		PopUpInfo(w,(XtPointer)"Float_3.help",NULL);
		break;
	      }
	 }
	  else
	  if (isTransObject(edit_state.object) &&          /* TRANS && GROUP */
	      edit_state.edit_object == GROUP)

	  { long current=(long)XawToggleGetCurrent(change_group_new_toggle); 
	    
	    params = (String*)XtCalloc(4,sizeof(String));
	    num_params = 2;

	    if (current == GroupReset)
	    {
	      params[0]= XtNewString("reset");
	      params[1]= XtNewString("reset");
	    }
	    else
	    if (edit_state.key_board_input_text == change_group_new_text) 
	    {
	      XtVaGetValues(change_group_new_text,
			    XtNstring,&buf,
			    NULL);
	      if (isNoInput(buf)) 
	      {
		params[0]= NULL;
		params[1]= NULL;
	      }
	      else
		if (isCardinal(buf))
		{
		  params[0] = XtNewString("new");
		  params[1] = getWord(buf);
		}
		else 
		{
		  succ = False;
		  PopUpInfo(w,(XtPointer)"Cardinal_2.help",NULL);
		  break;
		}
	    }
	    else
	    {
	      XtVaGetValues(change_group_merge_text,
			    XtNstring,&buf,
			    NULL);
	      if (isNoInput(buf)) 
	      {
		params[0]= NULL;
		params[1]= NULL;
	      }
	      else
		if (isCardinal(buf))
		{
		  params[0] = XtNewString("merge");
		  params[1] = getWord(buf);
		}
		else 
		{
		  succ = False;
		  PopUpInfo(w,(XtPointer)"Cardinal_3.help",NULL);
		  break;
		}
	    }
	  }
	    break;
      default:
	return;
      }
  if (succ) {
    infoDone(w,NULL,NULL);
    helpDone(w,NULL,NULL);
    XtPopdown(textPopup);
    XtCallActionProc(getCurrentNet(),"callEditAction",NULL,params,num_params);
  }
  if(num_params)
  {
    if(params[0]!=NULL) XtFree((String)params[0]);
    XtFree((String)params);
  }
}

/*******************************************************************************
Name:
        DialogCancel
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        cancelButton
--------------------------------------------------------------------------------
Zweck:
       Abbruch des Dialoges
*******************************************************************************/

static void DialogCancel(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{

#ifdef debug
printf(" DialogCancel \n");
#endif

  infoDone(w,NULL,NULL);
  helpDone(w,NULL,NULL);
  XtPopdown(textPopup);
  XtCallActionProc(getCurrentNet(),"callEditAction",NULL,NULL,0);
}

/*******************************************************************************
Name:
        ShowMarkDepEditor
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        change_delay_md_button
        change_arc_md_button (fuer diesen Button geht's nun auch)
--------------------------------------------------------------------------------
Zweck:
        Anzeigen des Editors zur Definition markierungsabh. Raten
*******************************************************************************/

static void ShowMarkDepEditor(w, client_data, call_data)
    Widget w;
    XtPointer client_data, call_data;
{
    String name;
    Dimension c_width,c_height,height,width;
	XawTextBlock tblock;

    tblock.firstPos = 0;
    tblock.ptr = "";
    tblock.format = FMT8BIT;
	    
    /* TEST THE NAME */
    if (client_data == (XtPointer)PARAMETER_ADD) {
	XtVaGetValues(add_text_1,
		      XtNstring,&name,
		      NULL);
	if (TestRParamName(getCurrentNet(),name))
	    return;
    }

    edit_state.managed_form = md_def_form;
    edit_state.key_board_input_text = md_def_text;
    edit_state.md_def = True;

    if (client_data == (XtPointer)PARAMETER_ADD || 
	client_data == (XtPointer)PARAMETER_CHANGE) {
	XtUnmanageChild((client_data == (XtPointer)PARAMETER_ADD) ? 
			add_form : change_form); 
	XtVaSetValues(textPopup,
		      XtNtitle,
		      "Editor for defining a parametric delay or weight",
		      NULL);
	XtAddCallback(md_help_button,XtNcallback,PopUpHelp,
		      "Delay.help");
    }
    else if (client_data == (XtPointer)ARC_MD) {    /* md-def for an ARC */
	XtUnmanageChild(change_arc_form);    
	{
	   Multiplicity *currMultiplicity;
	   XtVaGetValues(edit_state.object,XtNmultiplicity, &currMultiplicity,NULL);
	   if(currMultiplicity->type == MD_DEF)
	   	  tblock.ptr=currMultiplicity->values.md_definition;
    }
	XtVaSetValues(textPopup,
		      XtNtitle,
		      "Editor for defining marking-dependent arc values",
		      NULL);
	XtAddCallback(md_help_button,XtNcallback,PopUpHelp,"ArcMD.help");
    }
    else {                           /* md-def for a TRANSITION */
	XawToggleUnsetCurrent(change_delay_inf_button); 
	XtVaSetValues(change_delay_single_button,
		      XtNstate,True,
		      NULL); 
	XtUnmanageChild(change_delay_form);
	{
	   Rate *currRate;
	   XtVaGetValues(edit_state.object,XtNrates, &currRate,NULL);
	   if(currRate->type == MD_DEF)
	   	  tblock.ptr=currRate->values.md_definition;
	}
	XtAddCallback(md_help_button,XtNcallback,PopUpHelp,"Delay.help");

	if (client_data == (XtPointer)IMMEDIATE_MD)   /* IMMEDIATE */
	  XtVaSetValues(textPopup,
			XtNtitle,
			"Editor for defining marking-dependent weights",
			NULL);
	else                                       /* EXP./DET. */
	  XtVaSetValues(textPopup,
			XtNtitle,
			"Editor for defining marking-dependent firing delays",
			NULL);
    }

    tblock.length = strlen( tblock.ptr );
    XawTextReplace(md_def_text,0,BIG_BUF_SIZE,&tblock);

    XtUnmanageChild(main_command_form);
    XtManageChild(md_def_form);
    XtManageChild(md_help_button);



  /* popup groesse anpassen */
  
  XtVaSetValues(textPopup,XtNmappedWhenManaged,False,NULL);
  
  if (XtIsRealized(textPopup))
  {
  XtVaGetValues(main_command_form,
		      XtNwidth,&c_width,
		      XtNheight,&c_height,
		      NULL);
  XtVaGetValues(md_def_form,
		      XtNwidth,&width,
		      XtNheight,&height,
		      NULL);
  XtResizeWidget(textPopup,
		       Max(width,c_width),
		       height,
		       1);
  }
  else
      XtRealizeWidget(textPopup);

  {  
      Dimension width,height,screen_width,screen_height;
      Position r_x,r_y;

/*      if (!XtIsRealized(textPopup))
          XtRealizeWidget(textPopup);*/

      XtTranslateCoords(textPopup,0,0,&r_x,&r_y);

      XtVaGetValues(textPopup,
		    XtNwidth,&width,
		    XtNheight,&height,
		    NULL);

      screen_width = (Dimension)WidthOfScreen(XtScreen(textPopup));
      screen_height = (Dimension)HeightOfScreen(XtScreen(textPopup));

      if (r_y < 0)
	r_y = 0;
      else
	if (r_y + (Position)height > (Position)screen_height)
	  r_y = (Position)screen_height - (Position)height;
      
      if (r_x < 0)
	r_x = 0;
      else
	if (r_x + (Position)width > (Position)screen_width )
	  r_x = (Position)screen_width - (Position)width;

      XtMoveWidget(textPopup,r_x,r_y);
  }   
    
  XtVaSetValues(textPopup,XtNmappedWhenManaged,True,NULL);
  XtSetKeyboardFocus(textPopup,edit_state.key_board_input_text);
}
/*******************************************************************************
Name:
        ChangeTranstypeState
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        change_trans_immediate_toggle,change_trans_exponential_toggle,
	change_trans_deterministic_toggle,change_trans_general_toggle
--------------------------------------------------------------------------------
Parameter:
        client_data: Transitionstyp, der dem Toggle entspricht
--------------------------------------------------------------------------------
Zweck:
        Reaktion auf umgesetzten Transitionstypen
*******************************************************************************/

static void ChangeTranstypeState(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
TransType transtype = (TransType)client_data;
XawTextBlock tblock;
String buf;
int TimeMode = GetTimeMode();

  tblock.firstPos = 0;
  tblock.length = 0;
  tblock.ptr = "";
  tblock.format = FMT8BIT;

  edit_state.transtype = transtype;
#ifdef debug
  printf("texPopup.c:ChangeTranstypeState\n");
#endif
  if(TimeMode == ContTime)
    switch (transtype)
    {
      case IMMEDIATE:
	XtSetSensitive(change_trans_label_2,True);
	XtSetSensitive(change_trans_text,True);
	break;
      default:
	tblock.firstPos = 0;
	tblock.length = 0;
	tblock.ptr = "";
	tblock.format = FMT8BIT;
	XtVaGetValues(change_trans_text,
		      XtNstring,&buf,
		      NULL);
	XawTextReplace(change_trans_text,0,strlen(buf),&tblock);
	XtSetSensitive(change_trans_label_2,False);
	XtSetSensitive(change_trans_text,False);
	break;
    }
/*   else */
/*     switch (transtype) */
/*     { */
/*       case GENERAL:	 */
/* 	tblock.firstPos = 0; */
/* 	tblock.length = 0; */
/* 	tblock.ptr = ""; */
/* 	tblock.format = FMT8BIT; */
/* 	XtVaGetValues(change_trans_text, */
/* 		      XtNstring,&buf, */
/* 		      NULL); */
/* 	XawTextReplace(change_trans_text,0,strlen(buf),&tblock); */
/* 	XtSetSensitive(change_trans_label_2,False); */
/* 	XtSetSensitive(change_trans_text,False); */
/* 	break; */
/*       default: */
/* 	XtSetSensitive(change_trans_label_2,True); */
/* 	XtSetSensitive(change_trans_text,True); */
/* 	break; */
/*     } */
}
/*new*/
/*******************************************************************************
Name:
        ChangeTranspolicyState
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
      change_trans_RE_toggle,change_trans_RA_toggle
--------------------------------------------------------------------------------
Parameter:
        client_data: Race policy, des Toggles 
--------------------------------------------------------------------------------
Zweck:
        Reaktion auf umgesetzte Policy
*******************************************************************************/

static void ChangeTranspolicyState(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
RacePolicy racepolicy = (RacePolicy)client_data;

  edit_state.racepolicy = racepolicy;
}
/*******************************************************************************
Name:
        ChangeArctypeState
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        change_arc_input_toggle,
	change_arc_output_toggle,
	change_arc_inhibitor_toggle,
--------------------------------------------------------------------------------
Parameter:
        client_data: neuer Kantentyp
--------------------------------------------------------------------------------
Zweck:
        Reaktion auf umsetzen des Kantentyps
*******************************************************************************/

static void ChangeArctypeState(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
#ifdef debug
printf("ChangeArctypeState\n");
#endif

  edit_state.arctype = (ArcType)client_data;
}

/*******************************************************************************
Name:
        ChangeServState
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        change_delay_inf_button,
	change_delay_single_button
--------------------------------------------------------------------------------
Parameter:
        client_data: True, falls von change_delay_inf_button gerufen, False 
	sonst
--------------------------------------------------------------------------------
Zweck:
        Reaktion auf umsetzen der Schaltungsart
*******************************************************************************/

static void ChangeServState(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
#ifdef debug
printf("ChangeServState\n");
#endif

edit_state.inf_serv = (Boolean)(long)client_data;
}
/*******************************************************************************
Name:
        ResetPopup
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        XtNpopdownCallback von textPopup
--------------------------------------------------------------------------------
Zweck:
        Herstellung der Defaulteinstellungen der Dialogboxen
*******************************************************************************/

static void ResetPopup(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
XawTextBlock tblock;
String buf;

#ifdef debug
printf(" ResetPopup \n");
#endif
  tblock.firstPos = 0;
  tblock.length = 0;
  tblock.ptr = "";
  tblock.format = FMT8BIT;
  /*
  if (edit_state.managed_form == add_form)
  {
	XawTextReplace(add_text_1,0,SMALL_BUF_SIZE,&tblock);
	XawTextReplace(add_text_2,0,SMALL_BUF_SIZE,&tblock);
  }
  else
  if (edit_state.managed_form == change_form)
	XawTextReplace(change_text,0,SMALL_BUF_SIZE,&tblock);
  else
  if (edit_state.managed_form == change_arc_form)
	XawTextReplace(change_arc_text,0,SMALL_BUF_SIZE,&tblock);
  else
  if (edit_state.managed_form == change_trans_form)
	XawTextReplace(change_trans_text,0,SMALL_BUF_SIZE,&tblock);
  else
  if (edit_state.managed_form == change_delay_form)
	XawTextReplace(change_delay_text,0,SMALL_BUF_SIZE,&tblock);
  else
  {
	XawTextReplace(change_delay_text,0,SMALL_BUF_SIZE,&tblock);
	XawTextReplace(md_def_text,0,BIG_BUF_SIZE,&tblock);
  }
  */
  if (edit_state.managed_form == add_form)
  {
	XtVaGetValues(add_text_1,
			XtNstring,&buf,
			NULL);
	XawTextReplace(add_text_1,0,strlen(buf),&tblock);
	XtVaGetValues(add_text_2,
			XtNstring,&buf,
			NULL);
	XawTextReplace(add_text_2,0,strlen(buf),&tblock);
  }
  else
  if (edit_state.managed_form == change_form)
  {
	XtVaGetValues(change_text,
			XtNstring,&buf,
			NULL);
	XawTextReplace(change_text,0,strlen(buf),&tblock);
  }
  else
  if (edit_state.managed_form == change_arc_form)
  {
	XtVaGetValues(change_arc_text,
			XtNstring,&buf,
			NULL);
	XawTextReplace(change_arc_text,0,strlen(buf),&tblock);
  }
  else
  if (edit_state.managed_form == change_trans_form)
  {
	XtVaGetValues(change_trans_text,
			XtNstring,&buf,
			NULL);
	XawTextReplace(change_trans_text,0,strlen(buf),&tblock);
  }
  else
  if (edit_state.managed_form == change_delay_form)
  {
  	XtVaGetValues(change_delay_text,
			XtNstring,&buf,
			NULL);
	XawTextReplace(change_delay_text,0,strlen(buf),&tblock);

	XtVaGetValues(change_delay_phase_text,
			XtNstring,&buf,
			NULL);
	XawTextReplace(change_delay_phase_text,0,strlen(buf),&tblock);

	XtVaGetValues(change_delay_group_weight_text,
			XtNstring,&buf,
			NULL);
	XawTextReplace(change_delay_group_weight_text,0,strlen(buf),&tblock);
  }
  else
  if (edit_state.managed_form == change_group_form)
  {  
        XtVaGetValues(edit_state.key_board_input_text,
		      XtNstring,&buf,
		      NULL);
	XawTextReplace(edit_state.key_board_input_text ,0,strlen(buf),&tblock);
  }
  else
  {
	XtVaGetValues(change_delay_text,
			XtNstring,&buf,
			NULL);
	XawTextReplace(change_delay_text,0,strlen(buf),&tblock);
	XtVaGetValues(md_def_text,
			XtNstring,&buf,
			NULL);
	XawTextReplace(md_def_text,0,strlen(buf),&tblock);
  }
  XtUnmanageChild(edit_state.managed_form);
  XtManageChild(change_delay_inf_button);
  XtManageChild(change_delay_single_button);
  XtManageChild(change_delay_phase_label);
  XtManageChild(change_delay_phase_text);
  XtManageChild(change_delay_group_weight_label);
  XtManageChild(change_delay_group_weight_text); 
  XtSetSensitive(change_delay_inf_button,True);
  XtSetSensitive(change_delay_md_button,True);
  XtSetSensitive(change_trans_label_2,True);
  XtSetSensitive(change_trans_text,True);
  XawToggleUnsetCurrent(change_delay_single_button);
  XtVaSetValues(change_delay_single_button,
		XtNstate,True,
		NULL);
  edit_state.md_def = False;
  edit_state.inf_serv = False;
  XtVaSetValues(add_text_2,
		XtNdisplayCaret,False,
		NULL);
  XtVaSetValues(add_text_1,
		XtNdisplayCaret,True,
		NULL);
}
/*new*/
/*******************************************************************************
Name:
	ChangeGroupCallback
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
registriert mit:
             change_group_new_toggle,change_group_merge_toggle
--------------------------------------------------------------------------------
Zweck:
	Entsprechend dem Status des Buttons die Widget change_group_new_... 
	oder  change_group_merge_... en/disablen
*******************************************************************************/
static void ChangeGroupCallback(w,cl_data,call_data)
Widget w;
caddr_t cl_data,call_data;
{ XawTextBlock tblock;
  String buf;
  long current=(long)XawToggleGetCurrent(change_group_new_toggle);

  tblock.firstPos = 0;
  tblock.length = 0;
  tblock.ptr = "";
  tblock.format = FMT8BIT;

  if (current==New) 
  {	
    XtVaGetValues(change_group_merge_text,
		  XtNstring,&buf,
		  NULL);
    XawTextReplace(change_group_merge_text,0,strlen(buf),&tblock);

    XtSetSensitive(change_group_new_label,True);
    XtSetSensitive(change_group_new_text,True);
    XtSetSensitive(change_group_merge_label,False);
    XtSetSensitive(change_group_merge_text,False); 
    
    XtVaSetValues(change_group_merge_text,
		  XtNdisplayCaret,False,
		  NULL);
    XtVaSetValues(change_group_new_text,
		  XtNdisplayCaret,True,
		  NULL);
    edit_state.key_board_input_text = change_group_new_text;
  }
  else
    if (current==Merge)  
    {    
      XtVaGetValues(change_group_new_text,
		    XtNstring,&buf,
		    NULL);
      XawTextReplace(change_group_new_text,0,strlen(buf),&tblock);
  
      XtSetSensitive(change_group_new_label,False);
      XtSetSensitive(change_group_new_text,False);
      XtSetSensitive(change_group_merge_label,True);
      XtSetSensitive(change_group_merge_text,True); 
      
      XtVaSetValues(change_group_new_text,
		    XtNdisplayCaret,False,
		NULL);
      XtVaSetValues(change_group_merge_text,
		    XtNdisplayCaret,True,
		    NULL); 
      edit_state.key_board_input_text = change_group_merge_text;
    }
    else 
      if (edit_state.key_board_input_text == change_group_new_text)
      {
	XtVaGetValues(change_group_new_text,
		      XtNstring,&buf,
		      NULL);
	XawTextReplace(change_group_new_text,0,strlen(buf),&tblock);
      
	XtSetSensitive(change_group_new_label,False);
	XtSetSensitive(change_group_new_text,False);
	XtVaSetValues(change_group_new_text,
		    XtNdisplayCaret,False,
		NULL);
      }
      else
      {	
	XtVaGetValues(change_group_merge_text,
		      XtNstring,&buf,
		      NULL);
	XawTextReplace(change_group_merge_text,0,strlen(buf),&tblock);
	
	XtSetSensitive(change_group_merge_label,False);
	XtSetSensitive(change_group_merge_text,False);
	XtVaSetValues(change_group_merge_text,
		    XtNdisplayCaret,False,
		    NULL); 
      }
  XtSetKeyboardFocus(textPopup,edit_state.key_board_input_text);
}
