/*******************************************************************************
Modul   	:	validateButton.c

Purpose		: 	Realisierung des Buttons "Validate",Bereitstellen des
                        Men"us zum Aufruf des Tokengame und Anzeige der 
			Invarianten

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/
#include <stdio.h>

#include <X11/Intrinsic.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/

#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SimpleMenu.h>

#include "NetWidget.h"

#include "layout.h"

extern Widget 
    editCommandBox,editObjectBox,showToggle,
    commands,measureButton,optionsMenuButton,quitButton;
extern Widget
  structShell;

#ifdef __STDC__
extern void TokenGame(NetWidget nw, Boolean state);
extern void invariantsPopup
    (Widget w, XtPointer client_data, XtPointer call_data);
extern void ecsPopup(Widget w, XtPointer client_data, XtPointer call_data);
extern Widget getCurrentNet(void);

static void setLabels(Widget w, XtPointer client_data, XtPointer call_data);
static void SetChildrenSensitive(Widget w, Boolean state);

#else

extern void TokenGame();
extern Widget getCurrentNet();

extern void invariantsPopup ();
extern void ecsPopup();

static void setLabels();
static void SetChildrenSensitive();
#endif

static void Tokengame();

extern Pixmap toggle_pixmap;

static Widget
validateMenuButton,
validatePopup,
	tokengameButton,
	invariantsButton,
        ecsButton;

/*******************************************************************************
Name:
        initValidatePopup
--------------------------------------------------------------------------------
Funktionstyp:
        "offentliche Funktion
--------------------------------------------------------------------------------
Parameter:
        topLevel: Toplevelshell der Applikation
	parent: Elterwidget f"ur Button Validate
--------------------------------------------------------------------------------
Zweck:
        initialisierung des Buttons und des Men"us
*******************************************************************************/

void initValidatePopup(topLevel,parent)
Widget topLevel,parent;

{
validateMenuButton = XtVaCreateManagedWidget(
		"validateMenuButton",         
		menuButtonWidgetClass, 
		parent, 
		XtNlabel,"Validate",
		XtNwidth,COMMAND_BUTTON_WIDTH,
		XtNheight,COMMAND_BUTTON_HEIGHT,
		XtNjustify,COMMAND_BUTTON_JUSTIFY, 
		XtNmenuName,"validatePopup",  
		NULL);  

validatePopup = XtVaCreatePopupShell(
		"validatePopup",
		simpleMenuWidgetClass,
		topLevel,
		NULL);

XtAddCallback(validatePopup, XtNpopupCallback, setLabels, 0);

tokengameButton = XtVaCreateManagedWidget(
		"tokengameButton",
		 smeBSBObjectClass,
		 validatePopup,
		 XtNlabel,"Tokengame",
		 XtNrightMargin,16,
		 XtNleftMargin,16,
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 NULL);

XtAddCallback(tokengameButton,XtNcallback,Tokengame,0);

invariantsButton = XtVaCreateManagedWidget(
		"invariantsButton",
		 smeBSBObjectClass,
		 validatePopup,
		 XtNlabel,"Invariants",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 XtNrightMargin,16,
		 XtNsensitive, True,
		 NULL);

XtAddCallback(invariantsButton,XtNcallback, invariantsPopup, 0);

ecsButton = XtVaCreateManagedWidget(
		"ecsButton",
		 smeBSBObjectClass,
		 validatePopup,
		 XtNlabel,"ECS",
		 XtNjustify,MENU_ENTRY_JUSTIFY,
		 XtNrightMargin,16,
		 XtNsensitive, True,
		 NULL);

XtAddCallback(ecsButton,XtNcallback, ecsPopup, 0);

}

/*****************************************************************************/
#ifdef __STDC__
static void setLabels(Widget w, XtPointer client_data, XtPointer call_data)
#else
static void setLabels(w, client_data, call_data)
    Widget w;
    XtPointer client_data, call_data;
#endif
{
    Boolean tokengame;

    XtVaGetValues(getCurrentNet(),
		  XtNtokenGame, &tokengame,
		  NULL);

#ifdef debug
    fprintf(stderr, "SetLabels %s\n", (tokengame?"TRUE":"FALSE"));
#endif

	XtVaSetValues(tokengameButton,
		      XtNleftBitmap,tokengame?toggle_pixmap:None,
		      NULL);
}

/*******************************************************************************
Name:
        Tokengame
--------------------------------------------------------------------------------
Funktionstyp:
        Callback
--------------------------------------------------------------------------------
Registriert mit:
        tokengameButton
--------------------------------------------------------------------------------
Zweck:
        Rumpffunktion zur sp"ateren Implementierung 
*******************************************************************************/
static void Tokengame(w, client_data, call_data)
Widget w;
XtPointer client_data, call_data;
{
    Boolean playing;

    XtVaGetValues(getCurrentNet(),
		  XtNtokenGame, &playing,
		  NULL);

#ifdef debug
    fprintf(stderr, "Tokengame %s\n", (playing?"TRUE":"FALSE"));
#endif

    if (playing) {
	SetChildrenSensitive(editCommandBox,True);
	XtSetSensitive(editObjectBox,True);
	SetChildrenSensitive(commands,True);
    }
    else {
/*	SetChildrenSensitive(editCommandBox,False);  */
/* 	XtSetSensitive(showToggle,True);  */
/*	XtSetSensitive(editObjectBox,False);
	SetChildrenSensitive(commands,False);
	XtSetSensitive(measureButton, True);
	XtSetSensitive(validateMenuButton, True);
	XtSetSensitive(optionsMenuButton, True);
	XtSetSensitive(quitButton, True); */ 
    }
    TokenGame((NetWidget)getCurrentNet(),!playing);
}

#ifdef __STDC__
static void SetChildrenSensitive(Widget w, Boolean state)
#else
static void SetChildrenSensitive(w,state)
    Widget w;
    Boolean state;
#endif
{
    int i;
    WidgetList children;
    Cardinal numChildren;
    
    if (XtIsComposite(w)) {
	XtVaGetValues(w,
		      XtNnumChildren, &numChildren,
		      XtNchildren, &children,
		      NULL);
	for (i=0; i<numChildren; i++)
	    if (XtIsSubclass(children[i],commandWidgetClass))
		XtSetSensitive(children[i], state);
    }
}
