/*
  original : /usr/local/src/which6/which6.c
    -> the which command

  Anpassung als Funktion:  char* which(char *);
  Christian Luehe
  Portiert:
  Kolja Koischwitz
  */

#include	<sys/param.h>
#include	<sys/stat.h>
#include	<stdio.h>
#include    <string.h>
#include    <unistd.h>
#include    <limits.h>

#ifndef NGROUPS_MAX
#define NGROUPS_MAX NGROUPS
#endif

int groups_member();

#ifdef ultrix
/* Spezialbehandlung fuer die DEC-Kisten: */
static char *strdup (string)
char *string;
{
  register char *new_string = (char *)malloc (strlen (string) + 1);
  if (new_string)
    strcpy (new_string, string);
  return new_string;
}
#endif

#define		BUF_SIZE	512
#define		M_USR		0700
#define		M_GRP		0070
#define		M_OTH		0007
#define		X_ALL		0111
#define		R_ALL		0444

char	Version[] =
	"@(#)which 6.0 91/02/02 Maarten Litmaath @ VU Informatika Amsterdam";

char* which(command) 
char* command;
{
	char	*path, *s, **pathv, **p;
	char	*strcpy(), *getenv(), *fgets(), buf[BUF_SIZE], **convert();
	int	uid, gid, mask,
		xmask, rmask;
	struct	stat	st;

	if (!(path = strdup(getenv("PATH")))) {
	    return NULL;
	}

	if (!*path)
		path = ".";		/* another dubious convention */

	pathv = convert(path);		/* convert path string to vector */

	uid = geteuid();
	gid = getegid();
	if (uid == 0) {
		xmask = X_ALL;
		rmask = R_ALL;
	}

	for (p = pathv; path = *p++; ) {	/* try every component */
		s = buf;
		while (*s++ = *path++)
			;
		(void) strcpy(s, command);
		*--s = '/';

		if (stat(buf, &st) != 0 || (st.st_mode & S_IFMT) != S_IFREG)
			continue;

		/* file exists and is regular */

		if (uid != 0) {
			mask = st.st_uid == uid ? M_USR :
				st.st_gid == gid ? M_GRP :
				groups_member(st.st_gid) ? M_GRP :
				M_OTH;
			xmask = X_ALL & mask;
			rmask = R_ALL & mask;
		}

		if (!(st.st_mode & xmask))
			continue;

		/* file is executable */

		*s = 0;
		if (stat(buf, &st) != 0) {
			continue;
		}

		/* warn user if the directory is unreadable */

		if (!(st.st_mode & rmask))
		    continue;

		*s = '/';
		return strdup(buf);
	    }
	return NULL;
}


char	**convert(path)
char	*path;
{
	register char	*s, c;
	register int	pathc;		/* the length of the path vector */
	char	**v, **pathv, *malloc();

	for (s = path, pathc = 2; c = *s++; )
		if (c == ':')
			++pathc;

	if (!(pathv = (char **) malloc(pathc * sizeof(char *)))) {
		perror("malloc");
		exit(1);
	}

	for (s = path, v = pathv; (c = *s) != '\0'; ) {
		if (c == ':') {
			/*
			 * This colon is spurious.  According to some
			 * dubious convention it is made equivalent to a dot.
			 */
			*v++ = ".";
			if (*++s == '\0')
				*v++ = ".";
				/*
				 * The PATH ended in a spurious colon.
				 * To be consistent we add another dot
				 * to the path vector.  One day you'll
				 * be glad we did.
				 */
		} else {
			*v++ = s;
			while ((c = *++s) != '\0')
				if (c == ':') {
					*s++ = '\0';
					if (*s == '\0')
						*v++ = ".";
						/*
						 * The PATH ended in a
						 * (spurious) colon, so
						 * add dot to the vector.
						 */
					break;
				}
		}
	}

	*v = 0;		/* Signal the end of the path vector. */

	return pathv;
}


int	groups_member(gid)
int	gid;
{
	register
	int	*p, ngroups;
	int	groups[NGROUPS_MAX];

	if ((ngroups = getgroups(NGROUPS_MAX, groups)) < 0)
		return 0;
	p = groups;
        while (--ngroups >= 0)
		if (*p++ == gid)
			return 1;
	return 0;
}


