/*******************************************************************************
Programm   	:	monitor.c

Purpose		: 	Anzeigen der Ausgaben der L"osungsprogramme

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller
			Kolja Koischwitz

History		:	release 1: march 1993
			release 2: January 1994
________________________________________________________________________________
"usage: monitor [-c <command>] [-m model] [-l <label>] <logfile>"
        
Komplexe Kommandos und Labels m"ussen in "" oder '' eingeschlossen werden
Bei Angabe eines Kommandos wird dieses gestarten, die Standarteingabe und Standart
fehlerausgabe nach <logfile> umgeleitet und dieses in einem Textfeld angezeigt.
Mit <label> kann  ein informierende "Uberschrift angegeben werden.
Fehlt <command> wird nur das <logfile> angezeigt.
Die Anzeige des <logfiles> wird periodisch aktualisert.
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/param.h>
#include <sys/wait.h>

#include <X11/X.h>
#include <X11/Intrinsic.h>     /* Intrinsics Definitions*/
#include <X11/StringDefs.h>    /* Standard Name-String definitions*/

/*
 * Public include file for widgets we actually use in this file.
 */
#include <X11/Shell.h>
#include <X11/Xaw/Label.h>   
#include <X11/Xaw/Form.h>   
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Text.h>
#include <X11/Xaw/AsciiText.h>

#include "environment.h"

/* application class name 
   gleichzeitig name des application defaults files */
#define APP_CLASS_NAME "TimeNET"

#define USAGE "usage: monitor [-c <command>] [-m model] [-l <label>] <logfile>"

#define DEFAULT_LABEL "Command output"

#define TIMEOUT 100

static void Quit();
static void popdownCallback();
static void args_panic();
static void get_input();
static void get_arguments();
static void registerFileInput();

enum {Continue,Stop,Cancel}; /* fuer popdownCallback */

static Widget 	
topLevel, 		/*Transient fuer windowmanager*/
app_root,		/*appl. topwindow (form)*/
	header,
	quitButton,
	textField,
dialogFrame,		/* Shell fuer das Request Popup */
dialogForm,
	dialogLabel,
buttonBox,
	continueButton,stopButton,cancelButton
	;

static XtAppContext app_context;

static struct _inputClientData 
              {
		Widget text;
		FILE *stream;
	      } inputClientData;

static char mon_command[2*MAXPATHLEN],tmp_file[MAXPATHLEN],*netname=NULL;
static String command,log_file,label;
static Boolean show_only,process_running=0;
static char header_string[1000];
static char text_buf[10000];
static spid=-1; /* Process id des gestarteten Solve Prozesses */

static FILE *fd;

/* disable all keyfunctions,allow selections */
static char transtable[] = "\
	    <Btn1Down>:		select-start()			\n\
	    <Btn1Motion>:	extend-adjust()			\n\
	    <Btn1Up>:		extend-end(PRIMARY,CUT_BUFFER0)	\n\
	    <Btn3Down>: 	extend-start()			\n\
	    <Btn3Motion>:	extend-adjust()			\n\
	    <Btn3Up>:		extend-end(PRIMARY,CUT_BUFFER0) ";

static String fallback_resources[]={
#include "TimeNET_fallbacks.h"
  NULL
};

static void sigchild_handler()
{
  int status;

  waitpid (-1, &status, WNOHANG);
  if(WIFEXITED(status))
  {
	process_running=0;
	rename(tmp_file,log_file);
	/*	perror(tmp_file);*/

  }
}

static void sigquit_handler()
{
  int status;
  
  process_running=0;
  rename(tmp_file,log_file);
	/*	perror(tmp_file);*/
  exit(2);
}

/*******************************************************************************
Name:
	query_mouse
--------------------------------------------------------------------------------
Zweck:
	Findet die aktuelle Mouse-Position heraus
*******************************************************************************/
static void query_mouse(x,y)
int *x,*y;
{
	Window client,root; int wx,wy,mask;
	
	XQueryPointer(XtDisplay(topLevel),XtWindow(topLevel),&root,&client,
	  x,y,&wx,&wy,&mask);
}

/*******************************************************************************
Name:
	CenterWidget
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
Zweck:
	Positionieren eines Popups (Mausorientiert/Screenmitte)
*******************************************************************************/
static void CenterWidget(w)
Widget w;
{
    Position x,y; int mousex,mousey;
    Dimension height,width,screen_width,screen_height;
    int auto_realized=0;
	
	query_mouse(&mousex,&mousey);
	if(!XtIsRealized(w))
	{
	  XtVaSetValues(w,XtNmappedWhenManaged,False,NULL);
      XtRealizeWidget(w);
	  auto_realized=1;
    }
    XtVaGetValues(w,XtNwidth,&width,XtNheight,&height,NULL);
    screen_width=(Dimension)WidthOfScreen(XtScreen(w));
    screen_height=(Dimension)HeightOfScreen(XtScreen(w));
    
	x=mousex-width/2;
	y=mousey-height/2;

    if (y < 0) y = 0;
      else
	if (y + (Position)height > (Position)screen_height)
	  y = (Position)screen_height - (Position)height;
      
    if (x < 0) x = 0;
      else
	if (x + (Position)width > (Position)screen_width )
	  x = (Position)screen_width - (Position)width;

    XtMoveWidget(w,x,y);
    
	if(auto_realized)
	{  
	  XtUnrealizeWidget(w);
      XtVaSetValues(w,XtNmappedWhenManaged,True,NULL);
    }
}

/*******************************************************************************
Name:
	main
--------------------------------------------------------------------------------
Parameter:
	Argumente der Kommandozeile
--------------------------------------------------------------------------------
Zweck:
	Hauptprozedur
*******************************************************************************/
main(argc, argv)
int argc;
char **argv;
{
XtTranslations translations;
Dimension width,height;
char solution[30];
String model_dir,dspnhome;

  if ((int)signal (SIGCHLD, sigchild_handler) < 0)
  {
    perror ("signal");
    exit (1);
  }
  
  if ((int)signal (SIGQUIT, sigquit_handler) < 0)
  {
    perror ("signal");
    exit (1);
  }
  
  topLevel = XtVaAppInitialize(
        &app_context,       /* Application context */
        APP_CLASS_NAME,     /* Application class */
        NULL, 0,            /* command line option list */
        &argc, argv,        /* command line args */
        fallback_resources, /* for missing app-defaults file */
        NULL);              /* terminate varargs list */

  get_arguments(argc,argv);

  app_root = XtVaCreateManagedWidget(
            "app_root",             
            formWidgetClass,
            topLevel,
			XtNdefaultDistance,0,    
	        NULL);            

  header = XtVaCreateManagedWidget(
		"monitorHeader",        
	    labelWidgetClass,
	    app_root,
		XtNborderWidth,0,
		XtNjustify,XtJustifyLeft,
		XtNlabel,label,
		NULL);         

  quitButton = XtVaCreateManagedWidget(
		"quit",
		commandWidgetClass,
		app_root, 
		XtNfromHoriz,header,
		XtNwidth,100,
		XtNlabel,"Quit",
		NULL);

  XtVaGetValues(quitButton,XtNwidth,&width,XtNheight,&height,NULL);
  XtResizeWidget(header,596-width,height,1);
  
  XtAddCallback(quitButton,XtNcallback,Quit,NULL);
  
  translations = XtParseTranslationTable(transtable);

  textField = XtVaCreateManagedWidget(
		"textField",  
		asciiTextWidgetClass, 
		app_root,      
		XtNfromVert,header,
		XtNborderWidth,0,
		XtNwidth,600,				      
		XtNheight,400,
		XtNeditType,XawtextAppend,
 		XtNstring,text_buf,	
		XtNscrollVertical,XawtextScrollAlways,
		XtNdisplayCaret,False,	
		XtNtranslations,translations,	
		XtNwrap,XawtextWrapWord,		      
	 	NULL);  

  /* Initialisierung des Request Popups */
  dialogFrame = XtVaCreatePopupShell(
			 "TimeNET Request",
			transientShellWidgetClass,
			topLevel,
			XtNallowShellResize,True,
			XtNinput,True,
			NULL);

  dialogForm = XtVaCreateManagedWidget(
		"dialogForm",
		boxWidgetClass,
		dialogFrame,
		XtNborderWidth,0,
		NULL);

  dialogLabel = XtVaCreateManagedWidget(
			"dialogLabel1",
			labelWidgetClass,
			dialogForm,
			XtNlabel,"Continue or Stop with Solution?",
			NULL);

  continueButton = XtVaCreateManagedWidget(
			"continueButton",
			commandWidgetClass,
			dialogForm,
			XtNlabel,"Continue",
			XtNfromVert,dialogLabel,
			NULL);

  stopButton = XtVaCreateManagedWidget(
			"stopButton",
			commandWidgetClass,
			dialogForm,
			XtNlabel,"Stop",
			XtNfromHoriz,continueButton,
			XtNfromVert,dialogLabel,
			NULL);

  cancelButton = XtVaCreateManagedWidget(
			"cancelButton",
			commandWidgetClass,
			dialogForm,
			XtNlabel,"Cancel",
			XtNfromHoriz,stopButton,
			XtNfromVert,dialogLabel,
			NULL);

  XtAddCallback(continueButton,XtNcallback,popdownCallback,(XtPointer)Continue);
  XtAddCallback(stopButton,XtNcallback,popdownCallback,(XtPointer)Stop);
  XtAddCallback(cancelButton,XtNcallback,popdownCallback,(XtPointer)Cancel);

  if (!show_only)
    {
	  char log_back[MAXPATHLEN], *pfx;
	  
	  strcpy(log_back,log_file);
	  if((pfx=strrchr(log_back,'/'))!=NULL) *pfx++=0;
	  strcpy(tmp_file,tempnam(log_back,pfx));
	  if((fd = fopen(tmp_file,"w"))==NULL)
	{
	  char msg[1000];
	  sprintf(msg,"monitor: can't open file %s",tmp_file);
	  XtError(msg);
	}
      fprintf(fd,"");
      fclose(fd); /* Damit das Logfile schon existiert.. */
      if((spid=fork())==0)
      {  
	     setpgid(getpid(),getpid());
		 sprintf(mon_command,"%s 2>%s >%s",command,tmp_file,tmp_file);
         system(mon_command);
	     exit(0);
      }
	  else process_running=1;
    }
  if((fd = fopen(tmp_file,"r"))==NULL)
    {
      char msg[1000];
      sprintf(msg,"monitor: can't open file %s",tmp_file);
      XtError(msg);
    }
  
  inputClientData.text = textField;
  inputClientData.stream = fd;

  XtAppAddInput(app_context,fileno(fd),(XtPointer)XtInputReadMask,
  	get_input,(XtPointer)&inputClientData);

  XtRealizeWidget(topLevel);

  XtAppMainLoop(app_context);

}

/*******************************************************************************
Name:
	Quit
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	Quit-Button
--------------------------------------------------------------------------------
Zweck:
	Beenden des Programmes
	Ggf. vorher Requester-Popup aufrufen
*******************************************************************************/
static void Quit(w,client_data,call_data)
Widget w;
XtPointer client_data,call_data;
{ 
  if(!show_only && process_running)
  {  
     CenterWidget(dialogFrame);
     XtPopup(dialogFrame,XtGrabExclusive);
  }
  else
  {
    fclose(fd);
    exit(0);
  }
}

/*******************************************************************************
Name:
	popdownCallback
--------------------------------------------------------------------------------
Funktionstyp:
	Callback
--------------------------------------------------------------------------------
Registriert mit:
	continueButton, stopButton und CancelButton des Quit-Requesters
--------------------------------------------------------------------------------
Zweck:
	Beenden des Programmes mit/ohne Beenden des Solve-Prozesses
	(oder Cancel natuerlich)
*******************************************************************************/
static void popdownCallback(w,client_data,call_data)
Widget w;
XtPointer client_data,call_data;
{ int button=(int)client_data;
  
  if(button==Cancel)
  {	XtPopdown(dialogFrame);
  	return;
  }
  if(button==Stop && spid!=-1)
  {
    char clean_cmd[1024];
    
#ifdef __svr4__
	kill(-spid,SIGTERM);
        spid = -1;
#else
	killpg(spid,SIGTERM);
#endif

	if(netname!=NULL)
	{
/*	  XtWarning("Solution stopped. Cleaning up..");*/
	  sprintf(clean_cmd,"%s/Shared/scripts/CLEANUP %s >/dev/null",
	  	getenv("TNETHOME"),netname);
	  system(clean_cmd);
    }
  }
  fclose(fd);
  exit(0);
}

/*******************************************************************************
Name:
	get_input	
--------------------------------------------------------------------------------
Funktionstyp:
	Input-Callback
--------------------------------------------------------------------------------
Registriert mit:
	Eingabestrom "uber Log-file
--------------------------------------------------------------------------------
Zweck:
	Anzeigen der Eingabe
*******************************************************************************/
static void get_input(client_data,fd,id)
XtPointer client_data;
int *fd;
XtInputId *id;

{
#define BUFSIZE 1024

struct _inputClientData *data = (struct _inputClientData *)client_data;
FILE *stream = data->stream;
Widget text = data->text;
char in_buf[BUFSIZE],*out_buf;
int i,nbytes,out_len;
XawTextBlock textblock;

if ((nbytes = read(*fd,in_buf,BUFSIZE)) == -1)
  XtError("get_input: can't read from file");
if(nbytes)
  {
    textblock.firstPos = 0;
    textblock.length = nbytes;
    textblock.ptr = in_buf;
    textblock.format = FMT8BIT;
    XtVaGetValues(text,
		  XtNstring,&out_buf,
		  NULL);
    out_len = strlen(out_buf);
    XawTextReplace(text,out_len,out_len,&textblock);
    XtVaGetValues(text,
		  XtNstring,&out_buf,
		  NULL);
    XawTextSetInsertionPoint(text,strlen(out_buf));
  }
else
  {
    /* wait TIMEOUT msec until next read */
    XtRemoveInput(*id);
    XtAppAddTimeOut(app_context,TIMEOUT,registerFileInput,NULL);
  }
}
/*******************************************************************************
Name:
	registerFileInput	
--------------------------------------------------------------------------------
Funktionstyp:
	Timer-Callback
--------------------------------------------------------------------------------
Registriert mit:
	
--------------------------------------------------------------------------------
Zweck:
	Nach Ablauf des Timers wird der Eingabestrom wieder erm"oglicht
*******************************************************************************/
static void registerFileInput(client_data,id)
XtPointer client_data;
XtIntervalId *id;
{

XtAppAddInput(app_context,
	      fileno(fd),
	      (XtPointer)XtInputReadMask,
	      get_input,
	      (XtPointer)&inputClientData);

}
/*******************************************************************************
Name:
	get_arguments	
--------------------------------------------------------------------------------
Funktionstyp:
	private Funktion
--------------------------------------------------------------------------------
Zweck:
	Analysieren der Argumente
*******************************************************************************/
static void  get_arguments(argc,argv)
int argc;
String * argv;
{
  int i;
  Boolean command_found,label_found,logfile_found;

  command_found = label_found = logfile_found = False;
  for (i=1;i<argc;i++)
    { 
      if (!strcmp(argv[i],"-c"))
	  {
	    if (++i < argc && !command_found)
	      {
		command = argv[i];
		command_found = True;
	      }
	    else
	      args_panic();
	  }
      else
	  if (!strcmp(argv[i],"-m"))
	  {
	    if (++i < argc)
	      {
		netname = argv[i];
	      }
	    else
	      args_panic();
	  }
       else 
       if (!strcmp(argv[i],"-l"))
	 {
	   if (++i < argc && !label_found)
	     {
	       label = argv[i];
	       label_found = True;
	     }
	   else
	     args_panic();
	 }
       else
	 {
	   if (!logfile_found)
	     {
	       log_file = argv[i];
	       logfile_found = True;
	     }
	   else
	     args_panic(); 
	 }
    }
  if (!logfile_found)
    args_panic();
  if (!command_found)
    show_only = True;
  if (!label_found)
    label = DEFAULT_LABEL;   
}


/*******************************************************************************
Name:
	args_panic
--------------------------------------------------------------------------------
Funktionstyp:
	private Funktion
--------------------------------------------------------------------------------
Zweck:
	Terminiert, wegen falscher Argumente; Benutzungsinfo;
*******************************************************************************/
static void args_panic()
{
printf("%s\n",USAGE);
exit(1);
}
