/*******************************************************************************
Modul   	:	NWAddActions.c

Purpose		: 	Aktionen zum Hinzuf"ugen von Objekten

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/


/*
 * NetWidget.c - Manages all objects of a PN.

   Class Hierarchie:
   Core -> Composite -> NetWidget
 */

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include <stdio.h>
#include <string.h>

#include "BaseObj.h"
#include "MarkParObj.h"
#include "RateParObj.h"
#include "PlaceObj.h"
#include "TransObj.h"
#include "ArcObj.h"
#include "Polygon.h"
#include "environment.h"

#include "NetWidgetP.h"

/* intern defined actions */

static void AddArc();
static void AddPlace();
static void AddImTrans();
static void AddExpTrans();
static void AddDetTrans();
static void AddGenTrans();
static void AddMarkPar();
static void AddRatePar();

void AddSelection();

static void AddArc_2();
static void AddMarkPar_2();
static void AddRatePar_2();

static void copySelection();

/* Definition of Actions ******************************************************/

/*******************************************************************************
Name:
        Add
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Verzeigen zu einzelaktion
*******************************************************************************/

void Add(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
int curr_obj;

#ifdef debug
printf(" Add\n");
#endif

  if (nw->netWidget.read_only)
    {
      XtWarning("Model is read only");
      return;
    }
  curr_obj = nw->netWidget.object;

  if (isPlace(curr_obj)) 
	AddPlace(w,event,params,numparams);
  else 
  if (isImTrans(curr_obj)) 
	AddImTrans(w,event,params,numparams);
  else 
  if (isExpTrans(curr_obj)) 
	AddExpTrans(w,event,params,numparams);
  else 
  if (isDetTrans(curr_obj)) 
	AddDetTrans(w,event,params,numparams);
  else 
  if (isGenTrans(curr_obj)) 
	AddGenTrans(w,event,params,numparams);
  else 
  if (isArc(curr_obj)) 
	AddArc(w,event,params,numparams);
  else
  if (isMarkPar(curr_obj)) 
	AddMarkPar(w,event,params,numparams);
  else
  if (isRatePar(curr_obj)) 
	AddRatePar(w,event,params,numparams);
  else 
	return;
}

/*******************************************************************************
Name:
        Add_2
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Verzeigen zu einzelaktion
*******************************************************************************/

void Add_2(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
int curr_obj;

#ifdef debug
printf(" Add\n");
#endif

curr_obj = nw->netWidget.object;

  if (isArc(curr_obj)) 
	AddArc_2(w,event,params,numparams);
  else
  if (isMarkPar(curr_obj)) 
	AddMarkPar_2(w,event,params,numparams);
  else
  if (isRatePar(curr_obj)) 
	AddRatePar_2(w,event,params,numparams);
  else 
	return;
}

/*******************************************************************************
Name:
        AddPlace
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Hinzuf"ugen von Stellen
*******************************************************************************/

static void AddPlace(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
int grid_x,grid_y;

#ifdef debug
printf(" AddPlace\n");
#endif

switch (event->button) {
case Button1:
  AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
  if (lookUp(w,grid_x,grid_y,PUT) != NothingFound)
	return;
  if ((nw->netWidget.selectedObject = 
	     XtVaCreateWidget(getObjName(),
			      placeObjClass,
			      w,
			      XtNxpos,grid_x,
			      XtNypos,grid_y,
			      NULL))  == NULL){
	    /*sorry, not possible to create object */
	    return;
  }
  Draw(PLACE,nw->netWidget.selectedObject,XtDisplay(nw),nw->netWidget.picture,
	nw->netWidget.draw_GC,grid_x,grid_y,nw->netWidget.object_size,0);
  if (nw->netWidget.tagsVisible)
	DrawTag(nw->netWidget.selectedObject,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);
  SetModifiedFlag(nw,True);
  SetNetModifiedFlag(nw,True);
  Redisplay(w,NULL);
  break;
default:
	return;
}
}
/*******************************************************************************
Name:
        AddImTrans
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Hinzuf"ugen von zeitlosen transitionen
*******************************************************************************/

static void AddImTrans(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
int grid_x,grid_y;
#ifdef debug
printf(" AddImTrans");
#endif

switch (event->button) {
case Button1:
  AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
  if (lookUp(w,grid_x,grid_y,PUT) != NothingFound)
	return;
  if ((nw->netWidget.selectedObject = 
	     XtVaCreateWidget(getObjName(),
			      transObjClass,
			      w,
			      XtNxpos,(XtArgVal)grid_x,
			      XtNypos,(XtArgVal)grid_y,
			      XtNtransType,(XtArgVal)IMMEDIATE, 
			      XtNtransOrient,
			        (XtArgVal)nw->netWidget.currTransOrient, 
			      XtNratePos_x, nw->netWidget.object_size,
			      XtNratePos_y, nw->netWidget.object_size,
			      NULL))  == NULL){
	    /*sorry, not possible to create object */
	    return;
  }
  Draw(IMTRANS,nw->netWidget.selectedObject,
	XtDisplay(nw),nw->netWidget.picture,
	nw->netWidget.draw_GC,
	grid_x,grid_y,nw->netWidget.object_size,0);
  if (nw->netWidget.tagsVisible)
	DrawTag(nw->netWidget.selectedObject,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);
  if (nw->netWidget.ratesVisible)
	DrawRate(nw->netWidget.selectedObject,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);

  SetModifiedFlag(nw,True);
  SetNetModifiedFlag(nw,True);
  Redisplay(w,NULL);
  break;
case Button2:
#ifdef debug
  printf("Change Orient\n");
#endif
  nw->netWidget.currTransOrient = Rotate(nw->netWidget.currTransOrient);
  SetModifiedFlag(nw,True);
  break;
default: ;
}
}
/*******************************************************************************
Name:
        AddexpTrans
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Himzuf"ugen von exp. Transitionen
*******************************************************************************/

static void AddExpTrans(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
int grid_x,grid_y;
#ifdef debug
printf(" AddExpTrans");
#endif

  switch (event->button) 
  {
  case Button1:
     AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
     if (lookUp(w,grid_x,grid_y,PUT) != NothingFound)
	return;
     if ((nw->netWidget.selectedObject = 
	     XtVaCreateWidget(getObjName(),
			      transObjClass,
			      w,
			      XtNxpos,(XtArgVal)grid_x,
			      XtNypos,(XtArgVal)grid_y,
			      XtNtransType,(XtArgVal)EXPONENTIEL,
			      XtNtransOrient,
			         (XtArgVal)nw->netWidget.currTransOrient, 
			      XtNratePos_x, nw->netWidget.object_size,
			      XtNratePos_y, nw->netWidget.object_size,
			      NULL))  == NULL){
	    /*sorry, not possible to create object */
	    return;
     }
     Draw(EXPTRANS,nw->netWidget.selectedObject,
	XtDisplay(nw),nw->netWidget.picture,
	nw->netWidget.draw_GC,
	grid_x,grid_y,nw->netWidget.object_size,0);
     if (nw->netWidget.tagsVisible)
	DrawTag(nw->netWidget.selectedObject,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);
     if (nw->netWidget.ratesVisible)
	DrawRate(nw->netWidget.selectedObject,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);

     SetModifiedFlag(nw,True);
     SetNetModifiedFlag(nw,True);
     Redisplay(w,NULL);
     break;
   case Button2:
	/* change Orientation */
#ifdef debug
	printf("Change Orient\n");
#endif
	nw->netWidget.currTransOrient = Rotate(nw->netWidget.currTransOrient);
        SetModifiedFlag(nw,True);
	break;
   default: ;
   }
}
/*******************************************************************************
Name:
        AddDetTrans
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Hinzuf"ugen von det. Transitionen
*******************************************************************************/

static void AddDetTrans(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
int grid_x,grid_y;

#ifdef debug
printf(" AddDetTrans");
#endif

switch (event->button) {
case Button1:
  AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
  if (lookUp(w,grid_x,grid_y,PUT) != NothingFound)
	return;
  if ((nw->netWidget.selectedObject = 
	     XtVaCreateWidget(getObjName(),
			      transObjClass,
			      w,
			      XtNxpos,(XtArgVal)grid_x,
			      XtNypos,(XtArgVal)grid_y,
			      XtNtransType,(XtArgVal)DETERMINISTIC,
			      XtNtransOrient,
			         (XtArgVal)(nw->netWidget.currTransOrient), 
			      XtNratePos_x, nw->netWidget.object_size,
			      XtNratePos_y, nw->netWidget.object_size,
			      NULL))  == NULL){
	    /*sorry, not possible to create object */
	    return;
  }
  Draw(DETTRANS,nw->netWidget.selectedObject,
	XtDisplay(nw),nw->netWidget.picture,
	nw->netWidget.draw_GC,
	grid_x,grid_y,nw->netWidget.object_size,0);
  if (nw->netWidget.tagsVisible)
	DrawTag(nw->netWidget.selectedObject,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);
  if (nw->netWidget.ratesVisible)
	DrawRate(nw->netWidget.selectedObject,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);

  SetModifiedFlag(nw,True);
  SetNetModifiedFlag(nw,True);
  Redisplay(w,NULL);
  break;
case Button2:
	/* change Orientation */
#ifdef debug
  printf("Change Orient\n");
#endif
  nw->netWidget.currTransOrient = Rotate(nw->netWidget.currTransOrient);
  SetModifiedFlag(nw,True);
  break;
default: ;
}
}

/*******************************************************************************
Name:
        AddGenTrans
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Hinzuf"ugen von gen. Transitionen
*******************************************************************************/

static void AddGenTrans(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
int grid_x,grid_y;

#ifdef debug
printf(" AddGenTrans");
#endif

switch (event->button) {
case Button1:
  AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
  if (lookUp(w,grid_x,grid_y,PUT) != NothingFound)
	return;
  if ((nw->netWidget.selectedObject = 
	     XtVaCreateWidget(getObjName(),
			      transObjClass,
			      w,
			      XtNxpos,(XtArgVal)grid_x,
			      XtNypos,(XtArgVal)grid_y,
			      XtNtransType,(XtArgVal)GENERAL,
			      XtNtransOrient,
			         (XtArgVal)(nw->netWidget.currTransOrient), 
			      XtNratePos_x, nw->netWidget.object_size,
			      XtNratePos_y, nw->netWidget.object_size,
			      NULL))  == NULL){
	    /*sorry, not possible to create object */
	    return;
  }
  Draw(GENTRANS,nw->netWidget.selectedObject,
	XtDisplay(nw),nw->netWidget.picture,
	nw->netWidget.draw_GC,
	grid_x,grid_y,nw->netWidget.object_size,0);
  if (nw->netWidget.tagsVisible)
	DrawTag(nw->netWidget.selectedObject,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);
  if (nw->netWidget.ratesVisible)
	DrawRate(nw->netWidget.selectedObject,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);

  SetModifiedFlag(nw,True);
  SetNetModifiedFlag(nw,True);
  Redisplay(w,NULL);
  break;
case Button2:
	/* change Orientation */
#ifdef debug
  printf("Change Orient\n");
#endif
  nw->netWidget.currTransOrient = Rotate(nw->netWidget.currTransOrient);
  SetModifiedFlag(nw,True);
  break;
default: ;
}
}

/*******************************************************************************
Name:
        AddMarkPar
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Hinzuf"ugen von markenparameter, erstaktion
*******************************************************************************/

static void AddMarkPar(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
NWCallbackInfo_2 info;
int grid_x,grid_y;

#ifdef debug
printf(" AddMarkPar");
#endif

switch (event->button) {
case Button1:
  AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
  if (lookUp(w,grid_x,grid_y,PUT) != NothingFound)
	return;
  ignoreSettingsAndEvents(nw);
  nw->netWidget.prev_x=grid_x;
  nw->netWidget.prev_y=grid_y;
  nw->netWidget.selectedObject = 
	     XtVaCreateWidget(getObjName(),
			      markParObjClass,
			      w,
			      XtNxpos,grid_x,
			      XtNypos,grid_y,
			      NULL);
  /* jetzt eine callback zur applikation */
  info.object = nw->netWidget.selectedObject;
  info.edit_command = nw->netWidget.command;
  info.edit_object = nw->netWidget.object;
  if (event->x_root > 0)
	info.root_x = (Dimension)event->x_root;
  else
	info.root_x = 0;
  if (event->y_root > 0)
  	info.root_y = (Dimension)event->y_root;
  else
	info.root_y = 0;
  XtCallCallbacks(w,XtNtextCallback,(XtPointer)&info);
  break;
default:
	return;
}
}
/*******************************************************************************
Name:
        AddMarkPar_2
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Parameter:
        params[0]: name
        params[1]: wert
--------------------------------------------------------------------------------
Zweck:
        Hinzuf"ugen von markenparametern, zweitaktion
*******************************************************************************/

static void AddMarkPar_2(w,event,params,numparams)
Widget w;
XButtonEvent *event;
String * params;
Cardinal * numparams;

{
NetWidget nw = (NetWidget)w;
int m0,i;
String mpar_name;

#ifdef debug
printf("AddMarkPar_2 \n");
#endif
  if (params == NULL)
  {	/* cancel action */
	XtDestroyWidget(nw->netWidget.selectedObject);
  	allowSettings(nw);
	return;
  }
  if (*numparams < 2) {
	XtWarning("Creating new marking param: not enough parameter found\n");
	XtDestroyWidget(nw->netWidget.selectedObject);
  	allowSettings(nw);
	return;
  }
  if  (params[0] == NULL)	
  {
	XtWarning("Creating new marking param: no name found\n");
	XtDestroyWidget(nw->netWidget.selectedObject);
  	allowSettings(nw);
	return;
  }
  if (!isWord(params[0])) 
  {
	XtWarning("Creating new marking param: no name found\n");
	XtDestroyWidget(nw->netWidget.selectedObject);
  	allowSettings(nw);
	return;
  }
  if  (params[1] == NULL)	
  {
	XtWarning("Creating new marking param: no value found\n");
	XtDestroyWidget(nw->netWidget.selectedObject);
  	allowSettings(nw);
	return;
  }
  if (!sscanf(params[1],"%d",&m0)) 
  {
	XtWarning("Creating new marking param: no value found\n");
	XtDestroyWidget(nw->netWidget.selectedObject);
  	allowSettings(nw);
	return;
  }
  if (m0 < 0 ) 
  {
	XtWarning("Creating new marking param: no positiv value found\n");
	XtDestroyWidget(nw->netWidget.selectedObject);
	allowSettings(nw);
	return;
  }
  for (i = FirstMarkPar(nw);i<LastMarkPar(nw);i++)
    {
      XtVaGetValues(nw->composite.children[i],
		    XtNmparName,&mpar_name,
		    NULL);
      if (strcmp(mpar_name,params[0]) == 0)
	{
	  XtWarning("This Markingparameter already exists");
	  XtDestroyWidget(nw->netWidget.selectedObject);
	  allowSettings(nw);
	  return;
	}
    }
  XtVaSetValues(nw->netWidget.selectedObject,
		      XtNmparName,params[0],
		      XtNmparValue,(Cardinal)m0,
		      NULL);
 
  Draw(MARKPAR,nw->netWidget.selectedObject,
       		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
		nw->netWidget.prev_x,nw->netWidget.prev_y,nw->netWidget.object_size,0);
  SetModifiedFlag(nw,True);
  SetNetModifiedFlag(nw,True);
  Redisplay(w,NULL);
  allowSettings(nw);
}
/*******************************************************************************
Name:
        AddRatePar
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Hinzuf"ugen von ratenparametern, erstaktion
*******************************************************************************/

static void AddRatePar(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
NWCallbackInfo_2 info;
int grid_x,grid_y;

#ifdef debug
printf(" AddRatePar");
#endif

  switch (event->button) {
  case Button1:
  	AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
  	if (lookUp(w,grid_x,grid_y,PUT) != NothingFound)
	    return;
  	ignoreSettingsAndEvents(nw);
  	nw->netWidget.prev_x=grid_x;
  	nw->netWidget.prev_y=grid_y;
  	nw->netWidget.selectedObject = 
	     XtVaCreateWidget(getObjName(),
			      rateParObjClass,
			      w,
			      XtNxpos,(XtArgVal)grid_x,
			      XtNypos,(XtArgVal)grid_y,
			      NULL);
  	info.object = nw->netWidget.selectedObject;
  	info.edit_command = nw->netWidget.command;
  	info.edit_object = nw->netWidget.object;
  	if (event->x_root > 0)
	    info.root_x = (Dimension)event->x_root;
	else
	    info.root_x = 0;
	if (event->y_root > 0)
  	    info.root_y = (Dimension)event->y_root;
	else
	    info.root_y = 0;
  	XtCallCallbacks(w,XtNtextCallback,(XtPointer)&info);
  	break;
  default:
	return;
  }
}

/*******************************************************************************
Name:
        AddRatePar_2
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Parameter:
        params[0]: name
	params[1]: wert
--------------------------------------------------------------------------------
Zweck:
        Hinzuf"ugen von ratenprametern, zweitaktion
*******************************************************************************/

static void AddRatePar_2(w,event,params,numparams)
Widget w;
XButtonEvent *event;
String * params;
Cardinal * numparams;

{
NetWidget nw = (NetWidget)w;
RateValue rval;
double dw;
int i;
String rpar_name;

#ifdef debug
printf("AddRatePar_2 \n");
#endif

  if (params == NULL)
  {	/* cancel action */
	XtDestroyWidget(nw->netWidget.selectedObject);
  	allowSettings(nw);
	return;
  }
  if (*numparams < 2) 
  {
	XtWarning("Creating new delay param:  not enough parameter found\n");
	XtDestroyWidget(nw->netWidget.selectedObject);
  	allowSettings(nw);
	return;
  }
  if  (params[0] == NULL)	 
  {
	XtWarning("Creating new delay param: no name found\n");
	XtDestroyWidget(nw->netWidget.selectedObject);
  	allowSettings(nw);
	return;
  }
  if (!isWord(params[0])) 
  {
	XtWarning("Creating new delay param: no name found\n");
	XtDestroyWidget(nw->netWidget.selectedObject);
  	allowSettings(nw);
	return;
  }
  if  (params[1] == NULL)	 
  {
	XtWarning("Creating new delay param: no value found\n");
	XtDestroyWidget(nw->netWidget.selectedObject);
  	allowSettings(nw);
	return;
  }
  if (!strcmp(params[1], MD))
  {
      rval.valueType = Definition;
      if (params[2] == NULL)
      {
	  XtWarning("Creating new delay param: no definition found\n");
	  XtDestroyWidget(nw->netWidget.selectedObject);
	  allowSettings(nw);
	  return;
      }
      rval.values.definition = params[2];
  }
  else 
  {
      rval.valueType = Ratevalue;
      if (!sscanf(params[1],"%lf",&dw)) 
      {
	  XtWarning("Creating new delay param: no value found\n");
	  XtDestroyWidget(nw->netWidget.selectedObject);
	  allowSettings(nw);
	  return;
      }
      if (dw  < 0.0 ) 
      {
	  XtWarning("Creating new delay param: no positiv value found\n");
	  XtDestroyWidget(nw->netWidget.selectedObject);
	  allowSettings(nw);
	  return;
      }
      rval.values.value = dw;
  }
  for (i = FirstRatePar(nw);i<LastRatePar(nw);i++)
  {
      if (nw->composite.children[i] != nw->netWidget.selectedObject)
      {
	  XtVaGetValues(nw->composite.children[i],
			XtNrateParName,&rpar_name,
			NULL);
	  if (strcmp(rpar_name,params[0]) == 0)
	  {
	      XtWarning("This Delay/Weightparameter already exists");
	      XtDestroyWidget(nw->netWidget.selectedObject);
	      allowSettings(nw);
	      return;
	  }
      }
  }

  XtVaSetValues(nw->netWidget.selectedObject,
		XtNrateParName,params[0],
		XtNrateParValue,&rval,
		NULL);

  Draw(RATEPAR,nw->netWidget.selectedObject,
       XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
       nw->netWidget.prev_x,nw->netWidget.prev_y,
       nw->netWidget.object_size,0);
  SetModifiedFlag(nw,True);
  SetNetModifiedFlag(nw,True);
  Redisplay(w,NULL);
  allowSettings(nw);
}

/*******************************************************************************
Name:
        AddArc
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Hinzuf"ugen von Kanten, Erstaktion
*******************************************************************************/

static void AddArc(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
    Widget fromObject;
    NetWidget nw = (NetWidget)w;
    ArcType arctype;
    int childInd,x,y;
    
#ifdef debug
    printf(" Add_Arc \n");
#endif
    
    childInd = lookUp(w,event->x,event->y,PUT);
    if (childInd == NothingFound)
	return;
    fromObject = nw->composite.children[childInd];
    switch(event->button){
    case Button1:
	if (isPlaceObject(fromObject))
	    arctype = INPUTARC;
	else 
	    if (isTransObject(fromObject))
		arctype = OUTPUTARC;
	    else 
		return;
	break;
    case Button2:
	if (isPlaceObject(fromObject) || isTransObject(fromObject))
	    arctype = INHIBITORARC;
	else
	    return;
	break;
    default: return;
    }
    nw->netWidget.selectedObject=XtVaCreateWidget(getObjName(),
						  arcObjClass,
						  w,
						  XtNfromObj,fromObject,
						  XtNarcType,arctype,
						  NULL);
    XtVaGetValues(fromObject,
		  XtNxpos,&x,
		  XtNypos,&y,
		  NULL);
    nw->netWidget.prev_x = x;
    nw->netWidget.prev_y = y;
    
    createImPoint(nw->netWidget.selectedObject, x, y, 0);

    nw->netWidget.selected_intermediate_point = 0;

    ignoreSettings(nw);
    nw->netWidget.allowMotionAction = True; 
    XtCallCallbacks(w,XtNinfoCallback,(XtPointer)SECONDARY_ACTION);

    /* notwendig, weil motion auf prev_x,prev_y ein undraw macht*/
    DrawArc(nw->netWidget.selectedObject,
	    XtDisplay(nw),
	    nw->netWidget.picture,
	    nw->netWidget.motion_GC,
	    nw->netWidget.object_size);
}


/*******************************************************************************
Name:
        AddArc_2
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Hinzuf"ugen von Kanten, zweitaktion
*******************************************************************************/

static void AddArc_2(w,event,params,numparams)
     Widget w;
     XButtonEvent* event;
     String* params;
     Cardinal* numparams;
{
    static Widget fromObject,toObject;
    NetWidget nw = (NetWidget)w;
    ArcObject oldArc;
    ArcType arctype;
    int childInd,grid_x,grid_y,index;

#ifdef debug
    printf(" AddArc_2\n");
#endif

    switch (event->button) {
      case Button1:
	childInd=lookUp(w,event->x,event->y,PUT);
	if(childInd != NothingFound) {
 	    toObject = nw->composite.children[childInd];
	    /*get the beginning obj of the arc*/
	    XtVaGetValues(nw->netWidget.selectedObject,
			  XtNfromObj,&fromObject,
			  XtNarcType,&arctype,
			  NULL);
	    /* check PN-Syntax: PxT or TxP */
	    if(!isValidArc(fromObject,toObject))
	      return;
	    
	    /* syntax ok */
	    /* undraw old arc */
   	    DrawArc(
		    nw->netWidget.selectedObject,
		    XtDisplay(nw),
		    nw->netWidget.picture,
		    nw->netWidget.draw_GC,
		    nw->netWidget.object_size);
	    
	    /* check whether Arc already exists -> increase multiplicity */
	    if ((oldArc = (ArcObject)getArc(w,fromObject,toObject,arctype)) != NULL) {
	        /* undraw alreadey drawn pieces of arc */
   	        DrawArc(
			oldArc,
			XtDisplay(nw),
			nw->netWidget.picture,
			nw->netWidget.draw_GC,
			nw->netWidget.object_size);
	        setMultiplicity(oldArc,1,INCR);
	        XtDestroyWidget(nw->netWidget.selectedObject);
   	        DrawArc(
			oldArc,
			XtDisplay(nw),
			nw->netWidget.picture,
			nw->netWidget.draw_GC,
			nw->netWidget.object_size);
	    }
	    else {                /* set ENDPOINT of arc */
	        XtVaSetValues(nw->netWidget.selectedObject,
			      XtNtoObj,toObject,
			      NULL);
		
		destroyImPoint(nw->netWidget.selectedObject, 
			       nw->netWidget.selected_intermediate_point);
		
		DrawArc(
			nw->netWidget.selectedObject,
			XtDisplay(nw),
			nw->netWidget.picture,
			nw->netWidget.draw_GC,
			nw->netWidget.object_size);
	    }
    	    XtCallCallbacks(w,XtNinfoCallback,PRIMARY_ACTION);
	    allowSettings(nw);
	}
	else {       /* nothing found -> NEW INTERMEDIATE POINT */

  	    AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
	    index = nw->netWidget.selected_intermediate_point;

	    createImPoint(nw->netWidget.selectedObject,grid_x,grid_y,index+1);

	    nw->netWidget.prev_x = grid_x;
	    nw->netWidget.prev_y = grid_y;
	    nw->netWidget.selected_intermediate_point++;
	}

  	SetModifiedFlag(nw,True);
	SetNetModifiedFlag(nw,True);
       	Redisplay(w,NULL);
       	break;
 case Button3:
     /* undraw alreadey drawn pieces of arc */
     DrawArc(
	     nw->netWidget.selectedObject,
	     XtDisplay(nw),
	     nw->netWidget.picture,
	     nw->netWidget.draw_GC,
	     nw->netWidget.object_size);
     Redisplay(w,NULL);
     XtDestroyWidget(nw->netWidget.selectedObject);
     XtCallCallbacks(w,XtNinfoCallback,PRIMARY_ACTION);
     allowSettings(nw);
     break;
 default:
     return;
 }
}

/*** End of the Actions.. *****************************************************/

void AddSelection(nw)
NetWidget nw;
{
    double dx, dy;

    dx = dy = DEFAULT_SIZE;
    
    copySelection(nw, (int) dx, (int) dy);
    
    shiftPolygon(nw->netWidget.selectionBorder, dx, dy);
    
    Redraw ((Widget)nw, NULL, NULL, NULL);
    
    nw->netWidget.command = NOOPERATION;

} /* AddSelection */

#ifdef __STDC__
static void copySelection (NetWidget nw, int x, int y)
#else
static void copySelection (nw, x, y)
     NetWidget nw;
     int x, y;
#endif
{
    Cardinal i,j;
    Boolean selected, toSelected, fromSelected;
    Widget fromObj, toObj, copy;
    Widget *transList, *placeList, *arcList;
    Cardinal numTranss, numPlaces, numArcs;

    TransType transType;
    TransOrientation transOrient;
    Cardinal priority;
    EnablingDependence enablingDependence;
    int ratePos_x, ratePos_y;
    Rate *rate;

    /*new*/
    Cardinal phase;
    int group;
    double group_weight;
    RacePolicy racepolicy;

    Cardinal actualMarking;
    InitialMarking *initialMarking;

    ArcType arcType;
    Cardinal multiplicity;
    DrawingDirection drawingDirection;
    Cardinal numIPs;
    Widget *IPs;

    int tagXPos, tagYPos;
    int posX, posY;


    /* copy of the old lists */
    /* trans */
    numTranss = LastTrans(nw) - FirstTrans(nw);
    transList = (Widget*)XtCalloc(numTranss,
			  sizeof(Widget));
    for (i = FirstTrans(nw), j = 0; i < LastTrans(nw); i++, j++) 
      transList[j] = nw->composite.children[i];

    /* place */
    numPlaces = LastPlace(nw) - FirstPlace(nw);
    placeList = (Widget*)XtCalloc(numPlaces,
			  sizeof(Widget));
    for (i = FirstPlace(nw), j = 0; i < LastPlace(nw); i++, j++) 
      placeList[j] = nw->composite.children[i];

    /* arc */
    numArcs = LastArc(nw) - FirstArc(nw);
    arcList = (Widget*)XtCalloc(numArcs,
			  sizeof(Widget));
    for (i = FirstArc(nw), j = 0; i < LastArc(nw); i++, j++) 
      arcList[j] = nw->composite.children[i];
    
    /* COPY SELECTED TRANSS */
    for (i = 0; i < numTranss; i++) {
	XtVaGetValues(transList[i],
		      XtNselected, &selected,
		      NULL);
	if (selected) {
	    XtVaGetValues(transList[i],
			  XtNtransType, &transType,
			  XtNtransOrient, &transOrient,
			  XtNpriority, &priority,
			  XtNenablingDependence, &enablingDependence,
			  XtNrates, &rate,
			  XtNratePos_x, &ratePos_x,
			  XtNratePos_y, &ratePos_y,
			  XtNtagXPos, &tagXPos, 
			  XtNtagYPos, &tagYPos,
			  XtNxpos, &posX,
			  XtNypos, &posY,
			  /*new*/
			  XtNphase, &phase,       
			  XtNgroup, &group,       
			  XtNgroup_weight, &group_weight,
			  XtNracePolicy, &racepolicy,
			  NULL);
	    if ((nw->netWidget.selectedObject = 
		 XtVaCreateWidget(getObjName(),
				  transObjClass,
				  (Widget)nw,
				  XtNxpos, posX + x,
				  XtNypos, posY + y,
				  XtNtransType, transType,
				  XtNtransOrient, transOrient,
				  XtNpriority, priority,
				  XtNenablingDependence, enablingDependence,
				  XtNrates, rate,
				  XtNratePos_x, ratePos_x,
				  XtNratePos_y, ratePos_y,
				  XtNtagXPos, tagXPos, 
				  XtNtagYPos, tagYPos,
				  /*new*/
				 /*  XtNphase,(Cardinal)phase,            */
				 /*  XtNgroup, group,                     */
				 /*  XtNgroup_weight,(float)group_weight, */
				  XtNracePolicy, racepolicy,
				  NULL))  == NULL){
		XtWarning
		  ("copySelection: sorry, not possible to create object");
		return;
	    }
	    /*new*/
	    XtVaSetValues(nw->netWidget.selectedObject,
			  XtNgroup_weight,&group_weight,
			  XtNphase,(Cardinal)phase,
			  XtNgroup,(int)group,
			  NULL);
	    XtVaSetValues(transList[i],
			  XtNcopy, nw->netWidget.selectedObject,
			  NULL);
	} /*if selected*/
    } /*for trans*/

    /* COPY SELECTED PLACES */
    for (i = 0; i < numPlaces; i++) {
	XtVaGetValues(placeList[i],
		      XtNselected, &selected,
		      NULL);
	if (selected) {
	    XtVaGetValues(placeList[i],
			  XtNinitialMarking, &initialMarking,
			  XtNactualMarking, &actualMarking,
			  XtNxpos, &posX,
			  XtNypos, &posY,
			  XtNtagXPos, &tagXPos,
			  XtNtagYPos, &tagYPos,
			  NULL);
	    if ((nw->netWidget.selectedObject = 
		 XtVaCreateWidget(getObjName(),
				  placeObjClass,
				  (Widget)nw,

				  XtNxpos, posX + x,
				  XtNypos, posY + y,
				  XtNinitialMarking, initialMarking,
				  XtNactualMarking, actualMarking,
				  XtNtagXPos, tagXPos, 
				  XtNtagYPos, tagYPos,
				  NULL))  == NULL) {
		XtWarning
		  ("copySelection: sorry, not possible to create object");
		return;
	    }
	    XtVaSetValues(placeList[i],
			  XtNcopy, nw->netWidget.selectedObject,
			  NULL);
	} /*if selected*/
    } /*for places*/

    /* COPY SELECTED ARCS */
    for (i = 0; i < numArcs; i++) {
	XtVaGetValues(arcList[i],
		      XtNfromObj, &fromObj,
		      XtNtoObj, &toObj,
		      NULL);
	XtVaGetValues(fromObj,
		      XtNselected, &fromSelected,
		      NULL);
	XtVaGetValues(toObj,
		      XtNselected, &toSelected,
		      NULL);

	if (fromSelected && toSelected) {
	    XtVaGetValues(arcList[i],
			  XtNarcType, &arcType,
			  XtNmultiplicity, &multiplicity,
			  XtNdrawingDirection, &drawingDirection,
			  XtNnumChildren, &numIPs,
			  XtNchildren, &IPs,
			  NULL);
	    XtVaGetValues(fromObj,
			  XtNcopy, &fromObj,
			  NULL);
	    XtVaGetValues(toObj,
			  XtNcopy, &toObj,
			  NULL);
	    if ((nw->netWidget.selectedObject = 
		 XtVaCreateWidget(getObjName(),
				  arcObjClass,
				  (Widget)nw,
				  XtNfromObj, fromObj,
				  XtNtoObj, toObj,
				  XtNarcType, arcType,

				  XtNmultiplicity, multiplicity,
				  XtNdrawingDirection, drawingDirection,
				  NULL)) == NULL) {
		XtWarning
		  ("copySelection: sorry, not possible to create object");
		return;
	    }
	    /* iPoints */
	    for (j=0; j < numIPs; j++) {
		XtVaGetValues(IPs[j],
			      XtNxpos, &posX,
			      XtNypos, &posY,
			      NULL);
		createImPoint(nw->netWidget.selectedObject,
			      posX + x, posY + y, 
			      j);
	    }
	} /*if selected*/
    } /*for arcs*/

    /* now reset selected flag in original objects and set in copied */
    for (i = 0; i < numTranss; i++) {
	XtVaGetValues(transList[i],
		      XtNselected, &selected,
		      XtNcopy, &copy,
		      NULL);
	if (selected) {
	    XtVaSetValues(transList[i],
			  XtNselected, False,
			  XtNcopy, NULL,
			  NULL);
	    XtVaSetValues(copy,
			  XtNselected, True,
			  NULL);
	}
    } /* trans */
    for (i = 0; i < numPlaces; i++) {
	XtVaGetValues(placeList[i],
		      XtNselected, &selected,
		      XtNcopy, &copy,
		      NULL);
	if (selected) {
	    XtVaSetValues(placeList[i],
			  XtNselected, False,
			  XtNcopy, NULL,
			  NULL);
	    XtVaSetValues(copy,
			  XtNselected, True,
			  NULL);
	}
    } /* place */

    XtFree((char*)transList);
    XtFree((char*)placeList);
    XtFree((char*)arcList);
    SetNetModifiedFlag(nw,True);
    SetModifiedFlag(nw,True);
}


