/*******************************************************************************
Modul   	:	NWChangeActions.c

Purpose		: 	Aktionen zum "Amder von Objektcharakterisika

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
				release 2: Maerz 1994  (Kolja Koischwitz)
*******************************************************************************/


/*
 * NetWidget.c - Manages all objects of a PN.

   Class Hierarchie:
   Core -> Composite -> NetWidget
 */

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include <stdio.h>
#include <string.h>
#include <math.h>

#include "BaseObj.h"
#include "MarkParObj.h"
#include "RateParObj.h"
#include "PlaceObj.h"
#include "TransObj.h"
#include "ArcObj.h"

#include "NetWidgetP.h"

extern Widget simpleTextedit;
extern Widget getCurrentNet();

/* intern defined actions */

static void ChangeArc();
static void ChangeTag();
static void ChangeTransition();
static void ChangeMarking();
static void ChangeRatePar();

static void ChangeTransition_2();
static void ChangeTag_2();
static void ChangeArc_2();
static void ChangeMarking_2();
static void ChangeRatePar_2();
static void ChangeRateFromTrans();
static void ChangeValueFromRatePar();
static void RotateSelection();
static void RotateSelection_2();
/*new*/
static void ChangeGroup();
static void ChangeGroup_2();

/* Definition of Actions */
/*******************************************************************************
Name:
        Change
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber callEditAction
--------------------------------------------------------------------------------
Zweck:
        Verzweigen zu Einzelaktionen
*******************************************************************************/

void Change(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
int curr_obj;

#ifdef debug
printf(" Change\n");
#endif

  if (nw->netWidget.read_only)
    {
      XtWarning("Model is read only");
      return;
    }

  curr_obj = nw->netWidget.object;

  if (isImTrans(curr_obj)
  ||isExpTrans(curr_obj)
  ||isDetTrans(curr_obj)
  ||isGenTrans(curr_obj)) 
	ChangeTransition(w,event,params,numparams);
  else 
  if (isArc(curr_obj)) 
	ChangeArc(w,event,params,numparams);
  else
  if (isMarkPar(curr_obj)) 
	ChangeMarking(w,event,params,numparams);
  else
  if (isRatePar(curr_obj)) 
	ChangeRatePar(w,event,params,numparams);
  else 
  if (isTag(curr_obj)) 
	ChangeTag(w,event,params,numparams);
  else
  if (isCompleteSelection(curr_obj))
        RotateSelection(w, event, params, numparams);
  /*new*/
  else
  if (isGroup(curr_obj))
    	ChangeGroup(w,event,params,numparams);
  else 
	return;
}

/*******************************************************************************
Name:
        Change_2
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber callEditAciton
--------------------------------------------------------------------------------
Zweck:
        Verzweigung zu  Einzelaktionen 
*******************************************************************************/

void Change_2(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
int curr_obj;

#ifdef debug
printf(" Change\n");
#endif

curr_obj = nw->netWidget.object;

  if (isImTrans(curr_obj)
  ||isExpTrans(curr_obj)
  ||isDetTrans(curr_obj) 
  ||isGenTrans(curr_obj))  
	ChangeTransition_2(w,event,params,numparams);
  else 
  if (isArc(curr_obj)) 
	ChangeArc_2(w,event,params,numparams);
  else
  if (isMarkPar(curr_obj)) 
	ChangeMarking_2(w,event,params,numparams);
  else
  if (isRatePar(curr_obj)) 
	ChangeRatePar_2(w,event,params,numparams);
  else 
  if (isTag(curr_obj)) 
	ChangeTag_2(w,event,params,numparams);
  else
  if (isCompleteSelection(curr_obj))
        RotateSelection_2(w, event, params, numparams);
  /*new*/
  else
  if (isGroup(curr_obj))
    	ChangeGroup_2(w,event,params,numparams);
  else 
	return;
}

/*******************************************************************************
Name:
        ChangeTransition
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber callEditAction
--------------------------------------------------------------------------------
Zweck:
        "Andern Transitionseigenschaften, Erstaktion
*******************************************************************************/

static void ChangeTransition(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
int childInd,xpos,ypos;
TransOrientation orientation;
NWCallbackInfo_2 info;

#ifdef debug
 printf("NWChangeActions.c:ChangeTrans\n");
#endif

  switch (event->button) 
  {
    case Button1:
  	childInd = lookUp(w,event->x,event->y,SELECTION);
  	if (childInd ==  NothingFound)
		return;
  	if (!isTransObject(nw->composite.children[childInd]))
		return;
  	nw->netWidget.selectedObject = nw->composite.children[childInd];
  	ignoreSettingsAndEvents(nw);
  	info.object = nw->netWidget.selectedObject;
  	info.edit_command = nw->netWidget.command;
  	info.edit_object = nw->netWidget.object;
	info.root_x = event->x_root;
	info.root_y = event->y_root;
        XtCallCallbacks(w,XtNtextCallback,&info);
  	break;
    case Button2:
  	childInd = lookUp(w,event->x,event->y,SELECTION);
  	if (childInd ==  NothingFound)
		return;
  	if (!isTransObject(nw->composite.children[childInd]))
		return;
	XtVaGetValues(nw->composite.children[childInd],
			XtNxpos,&xpos,
			XtNypos,&ypos,
			XtNtransOrient,&orientation,
			NULL);
	redrawArcs(w,nw->composite.children[childInd],
			nw->netWidget.draw_GC,
			xpos,ypos);
	Draw(IMTRANS,nw->composite.children[childInd],
	     XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
	     xpos,ypos,nw->netWidget.object_size,0);
	XtVaSetValues(nw->composite.children[childInd],
			XtNtransOrient,Rotate(orientation),
			NULL);
	redrawArcs(w,nw->composite.children[childInd],
			nw->netWidget.draw_GC,
			xpos,ypos);
	Draw(IMTRANS,nw->composite.children[childInd],
	     XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
	     xpos,ypos,nw->netWidget.object_size,0);
  	SetModifiedFlag(nw,True);
	Redisplay(w,NULL);
    default:
	return;
  }
}

/*******************************************************************************
Name:
        ChangeTransition_2
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber callEditAciton
--------------------------------------------------------------------------------
Parameter:
        params[0]: Priorit"at
        params[1]: Transitionstyp
--------------------------------------------------------------------------------
Zweck:
        "Andern von Transitionseigenschaften, Zweitaktion
*******************************************************************************/

static void ChangeTransition_2(w,event,params,numparams)
Widget w;
XButtonEvent *event;
String * params;
Cardinal * numparams;

{
NetWidget nw = (NetWidget)w;
int xpos,ypos,rpos_x,rpos_y,priority;
TransType transtype;
Rate rate;
Objects curr_obj = nw->netWidget.object;
Boolean pri_changed = True;
int i;
RacePolicy racepolicy;

#ifdef debug
 printf("NWChangeActions.c:ChangeTrans_2 \n");
 for(i=1;i<*numparams;i++)
  printf("NWChangeActions.c:params[%d]=%s\n",i,params[i]);
#endif

  if (params == NULL)
  {
  	allowSettings(nw);
	return;
  }
  if (*numparams < 3) 
  {
	XtWarning("Change transition: not enough parameter found\n");
  	allowSettings(nw);
	return;
  }
  if  (params[0] == NULL || *params[0] == '\0' )	 
	pri_changed = False;
  else
    if (sscanf(params[0],"%d",&priority) == 0 || priority < 0) 
      {
	XtWarning("Change marking: no appropriate priority value found\n");
	allowSettings(nw);
	return;
      }
  if  (params[1] == NULL)	 
    {
      XtWarning("Change marking: no transition type found\n");
      allowSettings(nw);
      return;
    }
  if (!strcmp(params[1],XtEexponential)) 
	transtype = EXPONENTIEL;
  else
  if (!strcmp(params[1],XtEimmediate)) 
	transtype = IMMEDIATE;
  else
  if (!strcmp(params[1],XtEdeterministic)) 
	transtype = DETERMINISTIC;
  else
  if (!strcmp(params[1],XtEgeneral))
	transtype = GENERAL;
  else
  {
	XtWarning("Change transition: unknown transition kind");
	allowSettings(nw);
	return;
  }
  if  (params[2] == NULL)	 
    {
      XtWarning("Change marking: no race policy found\n");
      allowSettings(nw);
      return;
    }
  if (!strcmp(params[2],XtErace_e)) 
	racepolicy = RACE_E;
  else
  if (!strcmp(params[2],XtErace_a)) 
        racepolicy = RACE_A;
  else
  if (!strcmp(params[2],XtErace_s)) 
        racepolicy = RACE_S;

  /* first undraw old */
  XtVaGetValues(nw->netWidget.selectedObject,
		XtNxpos,&xpos,
		XtNypos,&ypos,
		XtNratePos_x,&rpos_x,
		XtNratePos_y,&rpos_y,
		NULL);
  Draw(curr_obj,nw->netWidget.selectedObject,
       XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
       xpos,ypos,nw->netWidget.object_size,0);
  if (nw->netWidget.ratesVisible)
	DrawRate(nw->netWidget.selectedObject,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);

  if (pri_changed)
    XtVaSetValues(nw->netWidget.selectedObject,
		  XtNtransType,transtype,
		  XtNpriority,(Cardinal)priority,
		  XtNracePolicy,racepolicy,
		  NULL);
  else 
    XtVaSetValues(nw->netWidget.selectedObject,
		  XtNtransType,transtype,
		  XtNracePolicy,racepolicy,
		  NULL);

  Draw(curr_obj,nw->netWidget.selectedObject,
       	 XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
	 xpos,ypos,nw->netWidget.object_size,0);
  if (nw->netWidget.ratesVisible)
	DrawRate(nw->netWidget.selectedObject,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);

  SetModifiedFlag(nw,True);
  SetNetModifiedFlag(nw,True);
  Redisplay(w,NULL);
  allowSettings(nw);
#ifdef debug
printf("NWChangeActions.c:ChangeTrans_2 \n");
#endif

}

/*******************************************************************************
Name:
        ChangeArc
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber callEditAciton
--------------------------------------------------------------------------------
Zweck:
        "Andern von Kanteneigenschaften, Erstaktion
*******************************************************************************/

static void ChangeArc(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
NWCallbackInfo_2 info;
int i,ip_number;

#ifdef debug
printf(" Change_Arc \n");
#endif

  switch (event->button) 
  {
    case Button1:
  	for (i = FirstArc(nw),ip_number = NOT_FOUND ;
	       i<LastArc(nw) && ip_number == NOT_FOUND;
	       i++)
 	{
	   ip_number = recievePointOfArc(nw->composite.children[i],
				       event->x,event->y,
				       nw->netWidget.object_size,SECTION);
	}
	if (ip_number == NOT_FOUND)
	    return;
	nw->netWidget.selectedObject = nw->composite.children[i-1];
  	ignoreSettingsAndEvents(nw);
  	/* jetzt ein callback zur applikation */
  	info.object = nw->netWidget.selectedObject;
  	info.edit_command = nw->netWidget.command;
  	info.edit_object = nw->netWidget.object;
	info.root_x = event->x_root;
	info.root_y = event->y_root;
  	XtCallCallbacks(w,XtNtextCallback,&info);
  	break;
    default:
	return;
  }
}

/*******************************************************************************
Name:
        ChangeArc_2
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber callEditAciton
--------------------------------------------------------------------------------
Parameter:
        params[0]: Multiplizit"at od. md-definition
        params[1]: Kantentyp
	params[2]: Cardinal- oder mark-dep. - Wert (False, True)

-------------------------------------------------------------------------------	
Zweck:
        "Andern von Kanteneigenschaften, Zweitaktion
*******************************************************************************/

static void ChangeArc_2(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;
{
    NetWidget nw = (NetWidget)w;
    ArcType arctype;
    Multiplicity *mult;
    Boolean mult_changed = True;

#ifdef debug
  printf("ChangeArc_2 \n");
#endif

  if (params == NULL)
  {
      allowSettings(nw);
      return;
  }
  if (*numparams < 3) 
  {
      XtWarning("Change arc: no parameter found\n");
      allowSettings(nw);
      return;
  }
  if  (params[0] == NULL || *params[0] == '\0' )
    mult_changed = False;
  else 
  { if((mult = XtNew(Multiplicity)) == 0) /* Speicher fuer Struktur */
    {
	XtWarning("Change arc2: cannot allocate Memory for temp. structure!\n");
  	allowSettings(nw);
	return;
    }  

    if  (params[2]!= NULL) 
    { 
	mult->type = MD_DEF; 
	if ((mult->values.md_definition = XtNewString(params[0])) == NULL) 
        {
	    XtWarning
	      ("Change arc2: cannot allocate Memory for temp. String!\n");
	    allowSettings(nw);
	    return;
	}    
    }
    else
    {
      if (sscanf(params[0],"%u",&mult->values.arcvalue) == 0 ) 
      {
	XtWarning("Change arc: no appropriate multiplicity found\n");
  	allowSettings(nw);
	return;
      }
      else  mult->type = ARCVALUE; 
    }
  }
  if (params[1] == NULL) 
  {
	XtWarning("Change arc: no appropriate type of arc found\n");
  	allowSettings(nw);
	return;
  }
  if (!strcmp(params[1],XtEoutputArc))
	arctype = OUTPUTARC;
  else 
  if (!strcmp(params[1],XtEinputArc))
	arctype = INPUTARC;
  else
  if (!strcmp(params[1],XtEinhibitorArc))
	arctype = INHIBITORARC;
  else
  {
	XtWarning("Change arc: no appropriate type of arc found");
	allowSettings(nw);
	return;
  }

  /* first undraw old */ 
  nw->netWidget.multiplMode = UndrawM;
  DrawArc(nw->netWidget.selectedObject,
	  XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
	  nw->netWidget.object_size);

  if (mult_changed){
      mult->actualValue = Impossible;
      XtVaSetValues(nw->netWidget.selectedObject,
		    XtNarcType,arctype,
		    XtNmultiplicity,mult,
		    NULL);

      if (mult->type == MD_DEF)
	  XtFree(mult->values.md_definition);
      XtFree((char*)mult);
  }
  else
      XtVaSetValues(nw->netWidget.selectedObject,
		    XtNarcType, arctype,
		    NULL);
    
  nw->netWidget.multiplMode = DrawM;
  if (!nw->netWidget.tokenGame)
      DrawArc(nw->netWidget.selectedObject,
	      XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
	      nw->netWidget.object_size);
  else {
      DrawArc(nw->netWidget.selectedObject,
	      XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
	      nw->netWidget.object_size);
  }

  SetModifiedFlag(nw,True);
  SetNetModifiedFlag(nw,True);
  Redisplay(w,NULL);
  allowSettings(nw);
}

/*******************************************************************************
Name:
        ChangeMarking
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber callEditAciton
--------------------------------------------------------------------------------
Zweck:
        "Andern von Markenparametern, der initialen Markierung, Erstaktion
*******************************************************************************/

static void ChangeMarking(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
int childInd;
NWCallbackInfo_2 info;

#ifdef debug
printf(" ChangeMarkPar");
#endif

  switch (event->button) 
  {
    case Button1:
  	childInd = lookUp(w,event->x,event->y,SELECTION);
  	if (childInd == NothingFound)
	    return;
  	if (!(   isPlaceObject(nw->composite.children[childInd]) 
		|| isMarkParObject(nw->composite.children[childInd])))
		    return;
  	nw->netWidget.selectedObject = nw->composite.children[childInd];
  	ignoreSettings(nw);
  	/* jetzt ein callback zur applikation */
  	info.object = nw->netWidget.selectedObject;
  	info.edit_command = nw->netWidget.command;
  	info.edit_object = nw->netWidget.object;
  	info.root_x = event->x_root;
  	info.root_y = event->y_root;
  	XtCallCallbacks(w,XtNtextCallback,&info);
  	break;
    default:
	return;
  }
}
/*******************************************************************************
Name:
        ChangeMarking_2
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber callEditAciton
--------------------------------------------------------------------------------
Parameter:
        params[0]: neuer Wert des Markenparameters
--------------------------------------------------------------------------------
Zweck:
        "Andern von Markenparametern, der initialen Markierung, Zweitaktion
*******************************************************************************/

static void ChangeMarking_2(w,event,params,numparams)
Widget w;
XButtonEvent *event;
String * params;
Cardinal * numparams;

{
NetWidget nw = (NetWidget)w;
String mparname;
Boolean found = FALSE;
InitialMarking M0;
int i,inum,xpos,ypos;

#ifdef debug
printf("ChangeMarkPar_2 \n");
#endif

  XtVaGetValues(nw->netWidget.selectedObject,
			XtNxpos,&xpos,
			XtNypos,&ypos,
			NULL);
  if (params == NULL)
  {
  	    allowSettings(nw);
	    return;
  }
  if (*numparams < 1) {
	XtWarning("Change marking: not enough parameter found\n");
  	allowSettings(nw);
	return;
  }
  if  (params[0] == NULL)	 
  {
	XtWarning("Change marking: no value or marking param found\n");
  	allowSettings(nw);
	return;
  }
  if (isPlaceObject(nw->netWidget.selectedObject)) 
  {
  	if (isWord(params[0])) 
	{
	    /* find mpar */
	    for(i=FirstMarkPar(nw);i<LastMarkPar(nw);i++) 
	    {
		XtVaGetValues(nw->composite.children[i],
				  XtNmparName,&mparname,
				  NULL);
		if (!strcmp(params[0],mparname)) 
		{
		    M0.type = MARKINGPARAM;
		    M0.values.markingParam=(XtPointer)nw->composite.children[i];
		    found = True;
		    break;
		}
	    }
	    if (found == False)  
	    {
	    	XtWarning("Change marking: no corresponding marking parameter found\n");
  	    	allowSettings(nw);
	    	return;
	    }
	}	
	else
	{
	    if (sscanf(params[0],"%d",&inum) && inum >= 0) 
	    {
	    	M0.type = MARKINGVALUE;
	    	M0.values.markingValue = inum;
	    }
	    else 
	    {
	    	XtWarning("Change marking: no appropriate value found\n");
  	    	allowSettings(nw);
	    	return;
	    }
	}
	    /* first undraw old */
  	Draw(PLACE,nw->netWidget.selectedObject,
       		 XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
		 xpos,ypos,nw->netWidget.object_size,0);
	XtVaSetValues(nw->netWidget.selectedObject,
				XtNinitialMarking,&M0,
				NULL);
  	Draw(PLACE,nw->netWidget.selectedObject,
       		 XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
		 xpos,ypos,nw->netWidget.object_size,0);
  	SetModifiedFlag(nw,True);
	SetNetModifiedFlag(nw,True);
  	Redisplay(w,NULL);
  }
  else
  if (isMarkParObject(nw->netWidget.selectedObject)) 
  {
	if (!(sscanf(params[0],"%d",&inum) && inum >= 0)) 
	{
	    XtWarning("Change marking: no appropriate value found\n");
	    allowSettings(nw);
	    return;
	}
  	Draw(MARKPAR,nw->netWidget.selectedObject,
       		 XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
		 xpos,ypos,nw->netWidget.object_size,0);
	XtVaSetValues(nw->netWidget.selectedObject,
				XtNmparValue,(Cardinal)inum,
				NULL);
  	Draw(MARKPAR,nw->netWidget.selectedObject,
       		 XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
		 xpos,ypos,nw->netWidget.object_size,0);
  	SetModifiedFlag(nw,True);
	SetNetModifiedFlag(nw,True);
  	Redisplay(w,NULL);
  }
  allowSettings(nw);
}

/*******************************************************************************
Name:
        ChangeRatePar
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber callEditAciton
--------------------------------------------------------------------------------
Zweck:
        "Andern von Ratenparametern, der Ratendefinition von Transitionen,
	Erstaktion
*******************************************************************************/

static void ChangeRatePar(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
int childInd;
NWCallbackInfo_2 info;

#ifdef debug
printf(" ChangeRatePar");
#endif

  switch (event->button) 
  {
    case Button1:
  	childInd = lookUp(w,event->x,event->y,SELECTION);
  	if (childInd == NothingFound)
		return;
  	if (!(   isTransObject(nw->composite.children[childInd])
	      || isRateParObject(nw->composite.children[childInd])))
			return;
  	nw->netWidget.selectedObject = nw->composite.children[childInd];
	nw->netWidget.prev_x = event->x;
	nw->netWidget.prev_y = event->y;
  	ignoreSettings(nw);
  	/* jetzt eine callback zur applikation */
  	info.object = nw->netWidget.selectedObject;
  	info.edit_command = nw->netWidget.command;
  	info.edit_object = nw->netWidget.object;
	info.root_x = event->x_root;
	info.root_y = event->y_root;
  	XtCallCallbacks(w,XtNtextCallback,&info);
  	break;
    default:
	return;
  }
}

/*******************************************************************************
Name:
        ChangeRatePar_2
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber callEditAciton
--------------------------------------------------------------------------------
Zweck:
        Verzweigung zu Einzelaktionen
*******************************************************************************/

static void ChangeRatePar_2(w,event,params,numparams)
Widget w;
XButtonEvent *event;
String * params;
Cardinal * numparams;

{
NetWidget nw = (NetWidget)w;

  if (isTransObject(nw->netWidget.selectedObject))
	ChangeRateFromTrans(w,event,params,numparams);
  else
  if (isRateParObject(nw->netWidget.selectedObject))
	ChangeValueFromRatePar(w,event,params,numparams);
  else
	XtWarning("Change delay : no appropriate Object selected");
}

/*******************************************************************************
Name:
        ChangeRateFromTrans
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber callEditAciton
--------------------------------------------------------------------------------
Parameter:
        params[0]: Ratendefinition/MD/pmf-Definition
        params[1]: XtEmarking_dependent, XtEpmf_function, Xtbrowse_file,
                   XtEstandard_rate
	params[2]: XtEsingleServer, XtEinifiniteServer
	--new--
	params[3]: Phasenangabe
	params[4]: Gruppengewichtsangabe		
--------------------------------------------------------------------------------
Zweck:
        "Andern der Ratendefinition einer Transition
*******************************************************************************/
		
static void ChangeRateFromTrans(w,event,params,numparams)
Widget w;
XButtonEvent *event;
String * params;
Cardinal * numparams;

{
NetWidget nw = (NetWidget)w;	
Rate rate;
String rparname;
Boolean found;
EnablingDependence en_dep ;
int i,xpos,ypos;
double	 fval,group_weight;
Boolean value_unchanged = False;
Cardinal phase;

#ifdef debug
printf("NWChangeActions.c:ChangeRateFromTrans \n");
#endif

  if (params == NULL)
  {
  	allowSettings(nw);
	return;
  }
  if ( *numparams < 5 || params[1] == NULL || params[2] == NULL) 
  {
	XtWarning("Change rate: not enough parameter found\n");
  	allowSettings(nw);
	return;
  }
 if (params[0] == NULL || *params[0] == '\0')
	value_unchanged = True;
 else
  {
  if (!strcmp(params[1],XtEmarking_dependent))
  {
	rate.type = MD_DEF;
	rate.values.md_definition=params[0];
  }
  else if (!strcmp(params[1],XtEpmf_function))
  {
  	rate.type = PMF_FUNC;
	rate.values.pmf_function=params[0];
#ifdef debug
        printf("NWchangeActions: pmf_function=%s\n"
               ,rate.values.pmf_function);
#endif
  }  
  else if (!strcmp(params[1],XtEbrowse_file))
  {
  	rate.type = DATAFILE;
  	rate.values.DataFile=params[0];
#ifdef debug
        printf("NWChangeActions: DataFile=%s\n"
               ,rate.values.DataFile);
#endif
  }
  else /* params[1] == XtEstandard_rate */
  {
  	if (isWord(params[0])) 
 	{
	    /* find rpar */
	    for(i=FirstRatePar(nw);i<LastRatePar(nw);i++) 
 	    {
		XtVaGetValues(nw->composite.children[i],
				  XtNrateParName,&rparname,
				  NULL);
		if (!strcmp(params[0],rparname)) 
		{
		    rate.type = RATEPARAM;
		    rate.values.rateparam=(XtPointer)nw->composite.children[i];
		    found = True;
		    break;
		}
	    }
	    if (found == False) 
	    {
	    	XtWarning("Change rate: no corresponding delay parameter found\n");
  	    	allowSettings(nw);
	    	return;
	    }
	}	
	else
	if (sscanf(params[0],"%lg",&fval) && fval > 0.0) 
	{
	    rate.type = RATEVALUE;
	    rate.values.ratevalue = fval;
	}
	else 
	{
	    	XtWarning("Change rate: no appropriate delay/weight value found\n");
  	    	allowSettings(nw);
	    	return;
	}
  }
 }
  if (!strcmp(params[2],XtEinfinite_server))
  	en_dep = INFINITE_SERVER;
  else
  if (!strcmp(params[2],XtEsingle_server))
  	en_dep = SINGLE_SERVER;

  XtVaGetValues(nw->netWidget.selectedObject,
		XtNratePos_x,&xpos,
		XtNratePos_y,&ypos,
		NULL);
  if (nw->netWidget.ratesVisible)
  	DrawRate(nw->netWidget.selectedObject,
       		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);

  if (value_unchanged)
  	XtVaSetValues(nw->netWidget.selectedObject,
		XtNenablingDependence,en_dep,
		NULL);

  else XtVaSetValues(nw->netWidget.selectedObject,
		XtNrates,&rate,
		XtNenablingDependence,en_dep,
		NULL);
                
/*  if (nw->netWidget.ratesVisible)
  	DrawRate(nw->netWidget.selectedObject,
		 XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);
*/
  /*new*/
  if (!(params[3] == NULL || *params[3] == '\0'))
    if (sscanf(params[3],"%i",&phase) && phase > 0) 
		XtVaSetValues(nw->netWidget.selectedObject,
			      XtNphase,(Cardinal)phase,
			      NULL);
    else 
	{
	    	XtWarning("Change phase: no appropriate phase value found\n");
  	    	allowSettings(nw);
	    	return;
	}
  if (!(params[4] == NULL || *params[4] == '\0'))
    if (sscanf(params[4],"%lg",&group_weight) && group_weight > 0.0) 
		XtVaSetValues(nw->netWidget.selectedObject,
			      XtNgroup_weight,&group_weight,
			      NULL);
    else 
	{
	    	XtWarning("Change phase: no appropriate group_weight value found\n");
  	    	allowSettings(nw);
	    	return;
	}
  SetModifiedFlag(nw,True);
  SetNetModifiedFlag(nw,True);
  Redisplay(w,NULL);
  allowSettings(nw);
#ifdef debug
printf("ChangeRateFromTrans: return \n");
#endif
}

/*******************************************************************************
Name:
        ChangeValueFromRatePar
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber callEditAciton
--------------------------------------------------------------------------------
Parameter:
        params[0]: Wert des Ratenparameters
--------------------------------------------------------------------------------
Zweck:
        "Andern des Wertes einer Ratenparameters
*******************************************************************************/

static void ChangeValueFromRatePar(w,event,params,numparams)
Widget w;
XButtonEvent *event;
String * params;
Cardinal * numparams;

{
NetWidget nw = (NetWidget)w;
RateValue rateval;
int xpos,ypos;
double	 fval;

#ifdef debug
printf("ChangeValueFromRatePar \n");
#endif
  
  if (params == NULL)
  {
  	allowSettings(nw);
	return;
  }
  if (*numparams < 1 || params[0] == NULL) {
	XtWarning("Change rate: no value or delay param found\n");
  	allowSettings(nw);
	return;
  }
  if (!strcmp(MD, params[0])) {
      if (*numparams < 2 || params[1] == NULL) {
	  XtWarning("Change rate: no parameter definition found\n");
	  allowSettings(nw);
	  return;
      }
      rateval.valueType = Definition;
      rateval.values.definition = params[1];
  }
  else 
  {
      if (sscanf(params[0],"%lf",&fval) == 0 || fval < 0.0) 
      {
	  XtWarning("Change rate: no appropriate value found\n");
	  allowSettings(nw);
	  return;
      }
      rateval.valueType = Ratevalue;
      rateval.values.value = fval;
  }
  XtVaGetValues(nw->netWidget.selectedObject,
		XtNxpos,&xpos,
		XtNypos,&ypos,
		NULL);
  Draw(RATEPAR,nw->netWidget.selectedObject,
       XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
       xpos,ypos,nw->netWidget.object_size,0);
  XtVaSetValues(nw->netWidget.selectedObject,
			XtNrateParValue,&rateval,
			NULL);
  Draw(RATEPAR,nw->netWidget.selectedObject,
       XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
       xpos,ypos,nw->netWidget.object_size,0);
  SetModifiedFlag(nw,True);
  SetNetModifiedFlag(nw,True);
  Redisplay(w,NULL);
  allowSettings(nw);
}


/*******************************************************************************
Name:
        ChangeTag
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber callEditAciton
--------------------------------------------------------------------------------
Zweck:
        "Andern von Etiketten , Erstaktion
*******************************************************************************/

static void ChangeTag(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
NWCallbackInfo_2 info;
int childInd;

#ifdef debug
printf(" ChangeTag\n");
#endif

  switch (event->button) 
  {
    case Button1:
  	childInd = lookUp(w,event->x,event->y,SELECTION);
  	if (childInd == NothingFound)
		return;
  	if (!(isTransObject(nw->composite.children[childInd]) || 
	      isPlaceObject(nw->composite.children[childInd])))
		return;
  	nw->netWidget.selectedObject = nw->composite.children[childInd];
  	ignoreSettingsAndEvents(nw);
  	/* jetzt ein callback zur applikation */
	info.object = nw->netWidget.selectedObject;
	info.edit_command = CHANGE;
	info.edit_object = TAG;
	info.root_x = event->x_root;
	info.root_y = event->y_root;
  	XtCallCallbacks(w,XtNtextCallback,&info);
  	break;
    default:
	return;
  }
}

/*******************************************************************************
Name:
        ChangeTag_2
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber callEditAciton
--------------------------------------------------------------------------------
Parameter:
        params[0]: neuer Etiketenname
--------------------------------------------------------------------------------
Zweck:
        "Andern der Etiketten, Zweitaktion
*******************************************************************************/

static void ChangeTag_2(w,event,params,numparams)
Widget w;
XButtonEvent *event;
String * params;
Cardinal * numparams;

{
NetWidget nw = (NetWidget)w;
String objname;
int xpos,ypos,i;

#ifdef debug
printf("ChangeTag_2 \n");
#endif

  if (params == NULL)
  {
  	allowSettings(nw);
	return;
  }
  if (*numparams < 1) {
	XtWarning("Change tag: no parameter found\n");
  	allowSettings(nw);
	return;
  }
  if  (params[0] == NULL || !isWord(params[0]))	 
  {
	XtWarning("Change tag: no new tag found");
  	allowSettings(nw);
	return;
  }
  /* check whether tag exists */
  for (i=FirstPlace(nw);i<LastPlace(nw);i++)
  {
	XtVaGetValues(nw->composite.children[i],
		XtNtagName,&objname,
		NULL);
	if (!strcmp(objname,params[0]))
	{
	    XtWarning("Change tag: This tag already exists");
  	    allowSettings(nw);
	    return;
	}
  }
  for (i=FirstTrans(nw);i<LastTrans(nw);i++)
  {
	XtVaGetValues(nw->composite.children[i],
		XtNtagName,&objname,
		NULL);
	if (!strcmp(objname,params[0]))
	{
	    XtWarning("Change tag: This tag already exists");
  	    allowSettings(nw);
	    return;
	}
  }
  /* first undraw old */
  XtVaGetValues(nw->netWidget.selectedObject,
		XtNtagXPos,&xpos,
		XtNtagYPos,&ypos,
		NULL);
  DrawTag(nw->netWidget.selectedObject,
       		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);
  XtVaSetValues(nw->netWidget.selectedObject,
			XtNtagName,params[0],
			NULL);
  DrawTag(nw->netWidget.selectedObject,
       		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);
  SetModifiedFlag(nw,True);
  SetNetModifiedFlag(nw,True);
  Redisplay(w,NULL);
  allowSettings(nw);
}

#ifdef __STDC__
/******************************************************************************/
static void RotateSelection(Widget w, XButtonEvent *event, 
/******************************************************************************/
			    String *params, Cardinal *numparams)
#else
static void RotateSelection(w,event,params,numparams)
Widget w;
XButtonEvent *event;
String * params;
Cardinal * numparams;
#endif
{
    NetWidget nw = (NetWidget)w;
    Widget fromObj, toObj, *ipList;
    Cardinal i, j, numIp;
    double x, y, dx, dy, phi;
    Boolean selected, fromSelected, toSelected;
    int posX, posY;
    TransOrientation orient;

    if (isInPolygon(nw->netWidget.selectionBorder, 
		    (double) event->x, (double) event->y)) {
	
	computePolygonCenter(nw->netWidget.selectionBorder, &x, &y);
	drawCenter(nw, (int)x, (int)y);
	
	Redisplay (w, NULL);
	
	nw->netWidget.prev_x = event->x;
	nw->netWidget.prev_y = event->y;
	nw->netWidget.selectionCenter_x = (int) x;
	nw->netWidget.selectionCenter_y = (int) y;
	ignoreSettings(nw);
	
	/* set the dblX/-Y and the other fields for rotation */
	/* dbl-values are needed so the positions are of off round-errors */
	for (i = FirstTrans(nw); i < LastTrans(nw); i++) {
	    XtVaGetValues(nw->composite.children[i],
			  XtNselected, &selected,
			  XtNxpos, &posX,
			  XtNypos, &posY,
			  XtNtransOrient, &orient,
			  NULL);
	    if (selected) {
		
		dx = (double)posX; dy = (double)posY;
		
		phi = atan2(dy - y, dx - x);
		
		XtVaSetValues(nw->composite.children[i],
			      XtNdblX, &dx,
			      XtNdblY, &dy,
			      XtNstartOrient, orient,
			      XtNstartPhi, &phi,
			      NULL);
	    }
	}
	for (i = FirstPlace(nw); i < LastPlace(nw); i++) {
	    XtVaGetValues(nw->composite.children[i],
			  XtNselected, &selected,
			  XtNxpos, &posX,
			  XtNypos, &posY,
			  NULL);
	    if (selected) {
		
		dx = (double)posX; dy = (double)posY;
		
		XtVaSetValues(nw->composite.children[i],
			      XtNdblX, &dx,
			      XtNdblY, &dy,
			      NULL);
	    }
	}
	/* set dblX/-Y in intermediatePoints */
	for (i = FirstArc(nw); i < LastArc(nw); i++) {
	    XtVaGetValues(nw->composite.children[i],
			  XtNfromObj, &fromObj,
			  XtNtoObj, &toObj,
			  XtNnumChildren, &numIp,
			  XtNchildren, &ipList,
			  NULL);
	    XtVaGetValues(fromObj,
			  XtNselected, &fromSelected,
			  NULL);
	    XtVaGetValues(toObj,
			  XtNselected, &toSelected,
			  NULL);
	    if (fromSelected && toSelected)
	      for (j=0; j < numIp; j++) {
		  XtVaGetValues(ipList[j],
				XtNxpos,&posX,
				XtNypos,&posY,
				NULL);
		  dx = (double) posX; dy = (double) posY;
		  XtVaSetValues(ipList[j],
				NULL);
		  dx = (double) posX; dy = (double) posY;
		  XtVaSetValues(ipList[j],
				XtNdblX, &dx,
				XtNdblY, &dy,
				NULL);
	      }
	}
	
	/* undraw tags & rates */
	if (nw->netWidget.tagsVisible || nw->netWidget.ratesVisible)
	  undrawSelectedTags(nw);
	
	nw->netWidget.allowRotationAction = True;
	
	XtCallCallbacks(w,XtNinfoCallback,(XtPointer)SECONDARY_ACTION);
    }
    else 
      nw->netWidget.command = NOOPERATION;

} /* RotateSelection */

#ifdef __STDC__
/******************************************************************************/
static void RotateSelection_2(Widget w, XButtonEvent *event, 
/******************************************************************************/
			    String *params, Cardinal *numparams)
#else
static void RotateSelection_2(w,event,params,numparams)
Widget w;
XButtonEvent *event;
String * params;
Cardinal * numparams;
#endif
{
    NetWidget nw = (NetWidget)w;

    /* allowSettings(nw); */
    nw->netWidget.actionTerminated = True;
    nw->netWidget.selectedObject = NULL;
    nw->netWidget.allowMotionAction = False;
    nw->netWidget.allowRotationAction = False;
    nw->netWidget.prev_x = -1;
    nw->netWidget.prev_y = -1;
    XtSetSensitive(editCommandBox,True);
    
    drawCenter(nw, nw->netWidget.selectionCenter_x, 
	       nw->netWidget.selectionCenter_y);
	
    SetModifiedFlag(nw,True);
    Redisplay(nw, NULL);

    /* redraw tags & rates */
    if (nw->netWidget.tagsVisible || nw->netWidget.ratesVisible)
      undrawSelectedTags(nw);
    
    XtCallCallbacks(w,XtNinfoCallback,(XtPointer)PRIMARY_ACTION);
} /* RotateSelection_2 */

/*new*/
/*******************************************************************************
Name:
        ChangeGroup
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber callEditAciton
--------------------------------------------------------------------------------
Zweck:
        "Andern der Gruppe einer Transition, Erstaktion
*******************************************************************************/

static void ChangeGroup(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
int childInd,xpos,ypos;
TransOrientation orientation;
NWCallbackInfo_2 info;
String tag;

#ifdef debug
printf(" ChangeGroup");
#endif

  switch (event->button) 
  {
    case Button1:
  	childInd = lookUp(w,event->x,event->y,SELECTION);
  	if (childInd ==  NothingFound)
		return;
  	if (!isTransObject(nw->composite.children[childInd]))
		return;
  	nw->netWidget.selectedObject = nw->composite.children[childInd];
  	ignoreSettingsAndEvents(nw);
	XtVaGetValues(nw->netWidget.selectedObject,
		      XtNtagName,&tag,
		      NULL);
  	info.object = nw->netWidget.selectedObject;
  	info.edit_command = CHANGE;
  	info.edit_object = GROUP;
	info.root_x = event->x_root;
	info.root_y = event->y_root;
  	XtCallCallbacks(w,XtNtextCallback,&info);
  	break;
    default:
	return;
  }
}

/*new*/
/*******************************************************************************
Name:
        ChangeGroup_2
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber callEditAciton
--------------------------------------------------------------------------------
Zweck:
        "Andern der Gruppe einer Transition, Zweitaktion
*******************************************************************************/

static void ChangeGroup_2(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
int group, to_group, from_group, i;
NetWidget nw = (NetWidget)w;
#ifdef debug
printf("ChangeTag_2 \n");
#endif
  if (params == NULL)
  {
  	allowSettings(nw);
	return;
  }
  if (*numparams < 2) {
	XtWarning("Change group: not enough parameter found\n");
  	allowSettings(nw);
	return;
  }  
  if (params[0] == NULL) 
  {
    XtWarning("Change group: no method found\n");
    allowSettings(nw);
    return;
  }
  else
  {
    XtVaGetValues(nw->netWidget.selectedObject,
		  XtNgroup,&from_group,
		  NULL);
    if (!strcmp(params[0],"reset"))
      if (from_group ==  DEFAULT_GROUP)
      {
	XtWarning("Change group: Transition doesn't belong to a group\n");
	allowSettings(nw);
	return;
      }
      else
	for(i = FirstTrans(nw); i < LastTrans(nw); i++) 
	{
	  XtVaGetValues(nw->composite.children[i],
			XtNgroup,&group,
			NULL);
	  if (group == from_group)
	    XtVaSetValues(nw->composite.children[i],
			  XtNgroup,(int)DEFAULT_GROUP,
			  NULL);
	}
    else
    {
      if  (params[1] == NULL || *params[1] == '\0' )
      {
      	allowSettings(nw);
	return;
      }
      else
	if (sscanf(params[1],"%d",&to_group) == 0 || to_group < 0) 
	{
	  XtWarning("Change group: no appropriate group identifier found\n");
	  allowSettings(nw);
	  return;
	}
      if (to_group == from_group)
      {
	XtWarning("Change group: no new group identifier found\n");
	allowSettings(nw);
	return;
      }
      if (!strcmp(params[0],"new"))     
	XtVaSetValues(nw->netWidget.selectedObject,
		      XtNgroup,(int)to_group,
		      NULL);
      else
	if (!strcmp(params[0],"merge"))
	  if (from_group ==  DEFAULT_GROUP)
	  {
	    XtWarning("Change group: Transition doesn't belong to a group\n");
	    allowSettings(nw);
	    return;
	  }
	  else
	    for(i = FirstTrans(nw); i < LastTrans(nw); i++) 
	    {
	      XtVaGetValues(nw->composite.children[i],
			    XtNgroup,&group,
			    NULL);
	      if (group == from_group && (group !=  DEFAULT_GROUP))
		XtVaSetValues(nw->composite.children[i],
			      XtNgroup,(int)to_group,
			      NULL);
	    }
    }      
  }
  SetModifiedFlag(nw,True);
  SetNetModifiedFlag(nw,True);
  Redisplay(w,NULL);
  allowSettings(nw);
}

