/*******************************************************************************
Modul   	:	NWDeleteActions.c

Purpose		: 	Aktionen zum L"oschen von Objekten

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/

/*
 * NetWidget.c - Manages all objects of a PN.

   Class Hierarchie:
   Core -> Composite -> NetWidget
 */

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include <stdio.h>
#include <string.h>

#include "BaseObj.h"
#include "MarkParObj.h"
#include "RateParObj.h"
#include "PlaceObj.h"
#include "TransObj.h"
#include "ArcObj.h"
#include "util.h"

#include "NetWidgetP.h"

extern void setCommandNormal();
extern void replaceInfoString();
extern void UpdateNetType(); /* from NWAddActions.c */

void DeleteSelection();

/* the following are private functions */
static void DeleteArc();
static void DeleteNode();
static void DeleteNodeObj();
static void DeleteParam();
void DeleteAction();

/*******************************************************************************
Name:
        Delete
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Verzweigen zu einzelaktionen
*******************************************************************************/

void Delete(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;
{
    NetWidget nw = (NetWidget)w;
    int curr_obj = nw -> netWidget.object;
    
#ifdef debug
    printf(" Delete \n");
#endif

  if (nw->netWidget.read_only)
    {
      XtWarning("Model is read only");
      return;
    }

  if (isPlace(curr_obj)) 
	DeleteNode(w,event,params,numparams);
  else 
  if (isImTrans(curr_obj)) 
	DeleteNode(w,event,params,numparams);
  else 
  if (isExpTrans(curr_obj)) 
	DeleteNode(w,event,params,numparams);
  else 
  if (isDetTrans(curr_obj)) 
	DeleteNode(w,event,params,numparams);
  else 
  if (isGenTrans(curr_obj)) 
	DeleteNode(w,event,params,numparams);
  else 
  if (isArc(curr_obj)) 
	DeleteArc(w,event,params,numparams);
  else
  if (isMarkPar(curr_obj)) 
	DeleteParam(w,event,params,numparams);
  else
  if (isRatePar(curr_obj)) 
	DeleteParam(w,event,params,numparams);
  else 
	return;
}

/*****************************************************************************/
void DeleteAction(w,event,params,numparams)
/*****************************************************************************/
     Widget w;
     XKeyEvent* event;
     String* params;
     Cardinal* numparams;
{
    NetWidget nw = (NetWidget)w;

    if (isCompleteSelection(nw->netWidget.object))
      DeleteSelection(nw);
}

/*******************************************************************************
Name:
        DeleteNode
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        Delete
--------------------------------------------------------------------------------
Zweck:
        L"oschen von Transitionen, Stellen
*******************************************************************************/

static void DeleteNode(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;
{
NetWidget nw = (NetWidget)w;
int curr_obj,childInd;

  curr_obj = nw -> netWidget.object;
  switch (event->button) {
  case Button1:
	childInd = lookUp(w,event->x,event->y,SELECTION);
  	if (childInd == NothingFound)
	    return;
	if (curr_obj == PLACE 
	    && !isPlaceObject(nw->composite.children[childInd])) 
		return;
	if ((curr_obj == IMTRANS  
	     || curr_obj == EXPTRANS 
	     || curr_obj == GENTRANS 
	     || curr_obj == DETTRANS )
	    && !isTransObject(nw->composite.children[childInd]))
			return;

	DeleteNodeObj(nw, nw->composite.children[childInd]);

	SetNetModifiedFlag(nw,True);
	SetModifiedFlag(nw,True);
	Redisplay(w,NULL);
  	break;
  default:
	return;
  }
}
/*******************************************************************************
Name:
        DeleteParam
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        L"oschen von Parametern
*******************************************************************************/

static void DeleteParam(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;
{
NetWidget nw = (NetWidget)w;
Widget obj;
int center_x,center_y;
int curr_obj,childInd,i,x,y;
InitialMarking *m0_buf,M0;
Rate *rate_buf,rate;

  curr_obj = nw -> netWidget.object;
  switch (event->button) {
  case Button1:
	childInd = lookUp(w,event->x,event->y,SELECTION);
	if (childInd == NothingFound)
		return;
	if ((isMarkParObject(nw->composite.children[childInd])) &&
           (isMarkPar(curr_obj)))
	    for (i=FirstPlace(nw);i<LastPlace(nw);i++)
	    {
		XtVaGetValues(nw->composite.children[i],
			XtNinitialMarking,&m0_buf,
			XtNxpos,&x,
			XtNypos,&y,
			NULL);
		if (m0_buf->type == VALUE)
			continue;
		if ((Widget)m0_buf->values.markingParam == nw->composite.children[childInd])
		{
		    M0.type = VALUE;
		    M0.values.markingValue = 0;
		    Draw(PLACE,nw->composite.children[i],
			XtDisplay(nw),
			nw->netWidget.picture,
			nw->netWidget.draw_GC,
			x,y,nw->netWidget.object_size,
			0);
		    XtVaSetValues(nw->composite.children[i],
			XtNinitialMarking,&M0,
			NULL);
		    Draw(PLACE,nw->composite.children[i],
			XtDisplay(nw),
			nw->netWidget.picture,
			nw->netWidget.draw_GC,
			x,y,nw->netWidget.object_size,
			0);
		}
	    }
	else	    
	if ((isRateParObject(nw->composite.children[childInd])) &&
           (isRatePar(curr_obj)))
	    for (i=FirstTrans(nw);i<LastTrans(nw);i++)
	    {
		XtVaGetValues(nw->composite.children[i],
			XtNrates,&rate_buf,
			XtNratePos_x,&x,
			XtNratePos_y,&y,
			NULL);
		if (rate_buf->type == RATEVALUE || rate_buf->type == MD_DEF)
			continue;
		if ((Widget)rate_buf->values.rateparam == nw->composite.children[childInd])
		{
		    rate.type = RATEVALUE;
		    rate.values.ratevalue = 1.0;
		    if (nw->netWidget.ratesVisible)
		      DrawRate(nw->composite.children[i],
			       XtDisplay(nw),
			       nw->netWidget.picture,
			       nw->netWidget.draw_GC);

		    XtVaSetValues(nw->composite.children[i],
			XtNrates,&rate,
			NULL);
		    if (nw->netWidget.ratesVisible)
		      DrawRate(nw->composite.children[i],
			       XtDisplay(nw),
			       nw->netWidget.picture,
			       nw->netWidget.draw_GC);
		}
	    }
	else
		return;
  	obj=nw->composite.children[childInd];
	XtVaGetValues(obj,
			XtNxpos,&center_x,
			XtNypos,&center_y,
			NULL);
	Draw(curr_obj,obj,
		XtDisplay(nw),
		nw->netWidget.picture,
		nw->netWidget.draw_GC,
		center_x,center_y,nw->netWidget.object_size,
		0);
  	SetModifiedFlag(nw,True);
	Redisplay(w,NULL);
	XtDestroyWidget(obj);
	/*
	XtVaSetValues(obj,
		XtNtoDestroy,True,
		NULL);
	*/
	SetNetModifiedFlag(nw,True);
	SetModifiedFlag(nw,True);
  	break;
  default:
	return;
  }
}
/*******************************************************************************
Name:
        DeleteArc
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        L"oschen von Kanten
*******************************************************************************/

static void DeleteArc(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;
{
NetWidget nw = (NetWidget)w;
int i,ip_number;


#ifdef debug
printf(" DeleteArc \n");
#endif

  switch (event->button) {
  case Button1:
	    /*del next IP */
     	for (i = FirstArc(nw),ip_number = NOT_FOUND ; 
	     i<LastArc(nw) && ip_number == NOT_FOUND; 
	     i++) {
	    ip_number = recievePointOfArc(nw->composite.children[i],
				      event->x,event->y,
				      nw->netWidget.object_size,
				      POINT);
  	}
	if (ip_number == NOT_FOUND)
	    return;

	nw->netWidget.multiplMode = UndrawM;
	DrawArc(nw->composite.children[i-1],
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
		nw->netWidget.object_size);

	destroyImPoint(nw->composite.children[i-1],ip_number);

	nw->netWidget.multiplMode = DrawM;
	DrawArc(nw->composite.children[i-1],
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
		nw->netWidget.object_size);

  	SetModifiedFlag(nw,True);
	Redisplay(w,NULL);
	break;
  case Button2:
  	for (i = FirstArc(nw),ip_number = NOT_FOUND  ; 
	    i<LastArc(nw) && ip_number == NOT_FOUND; 
	    i++)
  	{
	    ip_number = recievePointOfArc(nw->composite.children[i],
				      event->x,event->y,
				      nw->netWidget.object_size,
				      SECTION);
  	}
	if (ip_number == NOT_FOUND)
	    return;
    	DrawArc(nw->composite.children[i-1],
		 XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
		 nw->netWidget.object_size);
  	SetModifiedFlag(nw,True);
	SetNetModifiedFlag(nw,True);
    	Redisplay(w,NULL);
	XtDestroyWidget(nw->composite.children[i-1]);
	/*
	XtVaSetValues(nw->composite.children[i-1],
		XtNdestroyed,True,
		NULL);
	*/
    	break;
  default: return;
  }
}

/*****************************************************************************/
void DeleteSelection(nw)
/*****************************************************************************/
     NetWidget nw;
{
    Cardinal i;
    Boolean selected;

    if (!nw->netWidget.selectionBorder)
      return;

    XSetFunction(XtDisplay(nw), nw->netWidget.draw_GC, GXclear);

    for (i = FirstTrans(nw); i < LastTrans(nw); i++) {
	XtVaGetValues(nw->composite.children[i],
		      XtNselected, &selected,
		      NULL);
	if (selected) 
	  DeleteNodeObj(nw, nw->composite.children[i]);
    }
    for (i = FirstPlace(nw); i < LastPlace(nw); i++) {
	XtVaGetValues(nw->composite.children[i],
		      XtNselected, &selected,
		      NULL);
	if (selected) 
	  DeleteNodeObj(nw, nw->composite.children[i]);
    }

    XSetFunction(XtDisplay(nw), nw->netWidget.draw_GC, GXxor);

    nw->netWidget.object = NOOBJECT;
    nw->netWidget.command = NOOPERATION;

    undrawSelectionBorder(nw);

    destroyPolygon(nw->netWidget.selectionBorder);
    nw->netWidget.selectionBorder = NULL; 

    SetNetModifiedFlag(nw,True);
    SetModifiedFlag(nw,True);
    Redisplay((Widget)nw,NULL); 
    setCommandNormal();
    XtSetSensitive(editObjectBox, True);

    replaceInfoString();

} /* DeleteSelection */

/*****************************************************************************/
static void DeleteNodeObj(nw, obj)
/*****************************************************************************/
NetWidget nw;
Widget obj;
{
    int center_x, center_y, 
        tag_x, tag_y;

    XtVaGetValues(obj,
		  XtNxpos, &center_x,
		  XtNypos, &center_y,
		  XtNtagXPos, &tag_x,
		  XtNtagYPos, &tag_y,
		  NULL);
    if (isTransObject(obj))
      DrawTrans(obj,
		XtDisplay(nw), nw->netWidget.picture, nw->netWidget.draw_GC,
		center_x, center_y,
		nw->netWidget.object_size, 0);
    else if (isPlaceObject(obj))
      DrawPlace(obj,
		XtDisplay(nw), nw->netWidget.picture, nw->netWidget.draw_GC,
		center_x, center_y,
		nw->netWidget.object_size, 0);
    if (nw->netWidget.tagsVisible)
      DrawTag(obj,
	      XtDisplay(nw), nw->netWidget.picture, nw->netWidget.draw_GC);
    if (nw->netWidget.ratesVisible && isTransObject(obj)) 
	DrawRate(obj,
		 XtDisplay(nw), nw->netWidget.picture, nw->netWidget.draw_GC);

    deleteConnectedArcs((Widget)nw,obj);
    SetModifiedFlag(nw,True);
    SetNetModifiedFlag(nw,True);
    XtDestroyWidget(obj);
}
