/*******************************************************************************
Modul   	:	NWDraw.c 

Purpose		: 	Zeichnen von Objekten

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/


/* NWAppActions.c: Action routines called from Application 
 *
 * NetWidget.c - Manages all objects of a PN.

   Class Hierarchie:
   Core -> Composite -> NetWidget
 */

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/cursorfont.h>

#include <stdio.h>
#include <string.h>
#include <math.h>

#include "util.h"

#include "BaseObj.h"
#include "MarkParObj.h"
#include "RateParObj.h"
#include "PlaceObj.h"
#include "TransObj.h"
#include "ArcObj.h"

#include "NetWidgetP.h"

extern XtAppContext app_context;
extern Widget app_root;

#ifdef __STDC__

extern void Redraw(Widget w, XButtonEvent* event,
		   String* params, Cardinal* numparams);
extern Widget getCurrentNet(void);

#else

extern void Redraw();
extern Widget getCurrentNet();

#endif

#ifdef __STDC__

void drawArc (NetWidget nw,
	      int x, int y,
	      unsigned int width, unsigned int height,
	      int angle1, int angle2);
void drawLine (NetWidget nw, int x1, int y1, int x2, int y2);
void drawLines (NetWidget nw, XPoint *points, int npoints);
void drawTransRectangle (NetWidget nw, XPoint *points, int npoints);
void drawPoints (NetWidget nw, XPoint *points, int npoints, int mode);
void drawString (NetWidget nw, int x, int y, char *s, int l);
void drawImageString (NetWidget nw, int x, int y, char *s, int l);
void fillPolygon (NetWidget nw, 
		  XPoint *points, int npoints, int shape, int mode);
void fillStyle(NetWidget nw,int fill_style);
void setSolidLine(NetWidget nw);
void setDottedLine(NetWidget nw);
void setDashedLine(NetWidget nw);
void setBrightLine(NetWidget nw);
void splineLines(NetWidget nw, XPoint *points, int npoints);
void arcTo(NetWidget nw, double r,
	   double ax0, double ay0, 
	   double ax1, double ay1, 
	   double ax2, double ay2,
	   double* lastX, double* lastY);
static void drawArcIn (NetWidget nw, double r, 
		      double x0, double y0, 
		      double x3, double y3, 
		      double xc, double yc);

#else

void drawArc ();
void drawLine ();
void drawLines ();
void drawTransRectangle();
void drawPoints ();
void drawString ();
void drawImageString ();
void fillPolygon ();
void fillStyle();
void setSolidLine();
void setDashedLine();
void setDottedLine();
void setBrightLine();
void splineLines();
void arcTo();
static void drawArcIn ();

#endif

static void highLight();

/***************************DRAWING A PLACE*********************************/

/*******************************************************************************
Name:
       DrawPlace 
--------------------------------------------------------------------------------
Funktionstyp:

--------------------------------------------------------------------------------
Parameter:
        x,y: Koordinaten, an denen Objekt zu zeichnen ist
	obj_size: Objektgr"o"se
--------------------------------------------------------------------------------
Zweck:
       Zeichnet Stelle
*******************************************************************************/

typedef enum {Actual, Initial} ValueMode;

/* this is the graphical representation of one token (the point you can see) */
XPoint mark[]={{0,0},   /* center */
               {0,1},
               {0,1},   /* unten-mitte */
               {0,-3},
               {0,-1},  /* oben-mitte */
               {-1,0},  /* oben-links */
               {0,1},
               {0,1},
               {0,1},
               {0,1},   /* unten-links */
               {-1,-1}, /* links-unten */
               {0,-1},
               {0,-1},   /* links-oben */
               {3,-1},  /* oben-rechts */
               {0,1},
               {0,1},
               {0,1},
               {0,1},   /* unten-rechts */
               {1,-1},  /* rechts-unten */
               {0,-1},
               {0,-1}}; /* rechts-oben */
int npoints=21;

void draw_place(w,display,drawable,gc,x,y,obj_size,vmode)
     Widget w;
     Display *display;
     Drawable drawable;
     GC gc;
     int x,y,obj_size;
     ValueMode vmode;
{
  PlaceObj place = (PlaceObj)w;
  int px=x-obj_size/2;
  int py=y-obj_size/2;
  int DELTA=3; 
  InitialMarking *m0;
  char markingString[100],*markingName;
  Cardinal value;
  Boolean selected;
  NetWidget nw = (NetWidget)getCurrentNet();

#ifdef debug
  printf("DrawPlace \n");
#endif

#define CIRCLE_START 0
#define CIRCLE_END 360*64

  XtVaGetValues(w, 
		XtNselected, &selected,
		NULL);
  if (selected)
    setBrightLine(nw);  /*setBrightLine*/

  /*Draw Placeimage*/
  drawArc(nw,
	  px,py, obj_size,obj_size, CIRCLE_START,CIRCLE_END);
  
  if (selected)
    setSolidLine(nw);

  /* Draw marking */
  XtVaGetValues(w,
		XtNinitialMarking,&m0,
		NULL);
  if (vmode==Initial && m0->type==MARKINGPARAM) 
  {
    /*retrieve name of param*/
    XtVaGetValues((Widget)m0->values.markingParam,
		  XtNmparName,&markingName,
		  NULL);
    sprintf(markingString,"%s",markingName);
    drawString(nw,
	       px+obj_size/3,py+obj_size*3/4,
	       markingString,strlen(markingString));
  }
  else {
    if (vmode==Initial)
      value=m0->values.markingValue;
    else
      XtVaGetValues((Widget)place,
		    XtNactualMarking, &value,
		    NULL);
    switch (value) {
    case 0: break;
    case 1: 
      mark[0].x=x; mark[0].y=y;
      drawPoints(nw,
		 mark, npoints, CoordModePrevious);
      break;
    case 2: 
      mark[0].x=x-DELTA; mark[0].y=y;
      drawPoints(nw,
		 mark, npoints, CoordModePrevious);
      mark[0].x=x+DELTA; mark[0].y=y;
      drawPoints(nw,
		 mark, npoints, CoordModePrevious);
      break;
    case 3:{
      float dx, dy_u, dy_d;
      
      dx = DELTA;
      dy_u = (float)(float)DELTA/cos(M_PI/6.0);
      dy_d = (float)(float)DELTA*tan(M_PI/6.0);
      /* the center of an equal sided triangle */
      mark[0].x=x-dx; mark[0].y=y-dy_d;
      drawPoints(nw,
		 mark, npoints, CoordModePrevious);
      mark[0].x=x+dx; mark[0].y=y-dy_d;
      drawPoints(nw,
		 mark, npoints, CoordModePrevious);
      mark[0].x=x; mark[0].y=y+dy_u;
      drawPoints(nw,
		 mark, npoints, CoordModePrevious);
      break;
    }
    case 4:
      mark[0].x=x-DELTA; mark[0].y=y-DELTA;
      drawPoints(nw,
		 mark, npoints, CoordModePrevious);
      mark[0].x=x+DELTA; mark[0].y=y-DELTA;
      drawPoints(nw,
		 mark, npoints, CoordModePrevious);
      mark[0].x=x-DELTA; mark[0].y=y+DELTA;
      drawPoints(nw,
		 mark, npoints, CoordModePrevious);
      mark[0].x=x+DELTA; mark[0].y=y+DELTA;
      drawPoints(nw,
		 mark, npoints, CoordModePrevious);
      break;
    default: 
      sprintf(markingString,"%d",value);
      drawString(nw,
		 px+obj_size/3,py+obj_size*3/4,
		 markingString,strlen(markingString));
    }
  }
}

void DrawPlace(w,display,drawable,gc,x,y,obj_size,mode)
Widget w;
Display *display;
Drawable drawable;
GC gc;
int x,y,obj_size,mode; /* was soll mode ??? */
{
    if (((NetWidget)getCurrentNet())->netWidget.tokenGame)
	draw_place(w,display,drawable,gc,x,y,obj_size,Actual);
    else
	draw_place(w,display,drawable,gc,x,y,obj_size,Initial);
}

/****************************DRAWING A TRANSITION****************************/

static void CollectPoints();

/*******************************************************************************
Name:
        DrawTrans
--------------------------------------------------------------------------------
Funktionstyp:

--------------------------------------------------------------------------------
Parameter:
        x,y: Objektkoordianten
	obj_size: Objektgr"o"se
--------------------------------------------------------------------------------
Zweck:
        Zeichnet eine Transition
*******************************************************************************/

void DrawTrans(w,display,drawable,gc,x,y,obj_size,mode)
     Widget w;
     Display *display;
     Drawable drawable;
     GC gc;
     int x,y,obj_size,mode;

{
    TransType transType;
    Cardinal priority;
    int xpos=x-obj_size/2;
    int ypos=y-obj_size/2;
    XPoint * points;
    int numpoints;
    Boolean selected;

#ifdef debug
    printf("DrawTrans \n");
#endif

    /* mode flag currently not used */
    XtVaGetValues(w,
		  XtNtransType,&transType,
		  XtNpriority,&priority,
		  XtNselected, &selected,
		  NULL);
    CollectPoints(w,xpos,ypos,obj_size,&points,&numpoints);
    
    if (selected) setBrightLine((NetWidget)getCurrentNet());
    if (transType == EXPONENTIEL)
      drawTransRectangle((NetWidget)getCurrentNet(), points,numpoints);
    else 
    { 
	  if(transType == GENERAL)
	   fillStyle((NetWidget)getCurrentNet(),FillTiled);
	  fillPolygon((NetWidget)getCurrentNet(),
		  points,numpoints,Convex,CoordModeOrigin);
	  if(transType == GENERAL)
	   fillStyle((NetWidget)getCurrentNet(),FillSolid);
/*	  drawTransRectangle((NetWidget)getCurrentNet(), points,numpoints);*/
    }
	if (selected) setSolidLine((NetWidget)getCurrentNet());

    XtFree((char *)points);	
}


/*******************************************************************************
*******************************************************************************/
void showEnabledTranss(nw)
    NetWidget nw;
{
    int i;
    Boolean enabled;
    TransType ttype;
    
    for (i=FirstTrans(nw); i<LastTrans(nw); i++) {
	XtVaGetValues(nw->composite.children[i],
		      XtNenabled, &enabled,
		      XtNtransType, &ttype,
		      NULL);
	if (enabled && (ttype==IMMEDIATE || !nw->netWidget.vanish)) 
	    highLight(nw, nw->composite.children[i]);
    }
}

static void highLight(nw, w)
    NetWidget nw;
    Widget w;
{
    int x, y, dest_x, dest_y;
    unsigned int size = (unsigned)nw->netWidget.object_size*1.5;
    
    if (isTransObject(w) || isPlaceObject(w)) {
	XtVaGetValues(w,
		      XtNxpos, &x,
		      XtNypos, &y,
		      NULL);
	dest_x = x - size/2;
	dest_y = y - size/2;

	/* make spott stippled */
	XSetFunction(XtDisplay(nw),nw->netWidget.draw_GC,GXcopy);
	XCopyArea(XtDisplay(nw),
		  nw->netWidget.stipple, nw->netWidget.spott,
		  nw->netWidget.draw_GC,
		  0, 0,
		  size, size,
		  0,0); 
	XSetFunction(XtDisplay(nw),nw->netWidget.draw_GC,GXxor);

	/* now clear all pixels set in the object plus the neighbours */
	XSetFunction(XtDisplay(nw),nw->netWidget.draw_GC,GXandInverted);

#define clearSpott(x,y)    XCopyArea(XtDisplay(nw),\
			   nw->netWidget.flash,nw->netWidget.spott,\
			   nw->netWidget.draw_GC,\
			   dest_x-1,dest_y-1,size+2,size+2,x,y)
	clearSpott(-1,-1);
	clearSpott(-1,-2);
	clearSpott(-2,-2);
	clearSpott(-2,-1);
	clearSpott(-2,0);
	clearSpott(-1,0);
	clearSpott(0,0);
	clearSpott(0,-1);
	clearSpott(0,-2);
	XSetFunction(XtDisplay(nw),nw->netWidget.draw_GC,GXxor);

	/* and last copy the spott into the picture */
	XCopyArea(XtDisplay(nw),
		  nw->netWidget.spott, nw->netWidget.flash,
		  nw->netWidget.draw_GC,
		  0, 0,
		  size, size,
		  dest_x, dest_y); 
    }
    else
	XtWarning("can't highLight any thing but Places/Transitions");
}

/*******************************************************************************
Name:
        CollectPoints
--------------------------------------------------------------------------------
Funktionstyp:

--------------------------------------------------------------------------------
Parameter:
        xpos,ypos: Basispunkt einer Transition
	obj_size: Objektgr"o"se
    R"uckgabe:
        points: Punktfeld f"ur das Transitionsimage
	numpoints: Anzahl der Punkte
--------------------------------------------------------------------------------
Zweck:
        Berechnet die St"utzpunkte zum Zeichnen von Transitionen
*******************************************************************************/

static void CollectPoints(w,xpos,ypos,obj_size,points,numpoints)
Widget w;
int xpos,ypos,obj_size,*numpoints;
XPoint **points;
{
TransOrientation orientation;
TransType transtype ;
int t_width;
XPoint * pts;

  XtVaGetValues(w,
		XtNtransOrient,&orientation,
		XtNtransType,&transtype,
		NULL);

  pts=(XPoint*)XtMalloc(5*sizeof(XPoint));
  t_width = transtype == IMMEDIATE ? obj_size/4 : obj_size/2;
  switch (orientation)
  {
     case ZERO:
  	pts[0].x = xpos + obj_size/2 - t_width/2;
  	pts[0].y = ypos;
  	pts[1].x = xpos + obj_size/2 + t_width/2;
  	pts[1].y = ypos;
  	pts[2].x = xpos + obj_size/2 + t_width/2;
  	pts[2].y = ypos + obj_size;
  	pts[3].x = xpos + obj_size/2 - t_width/2;
  	pts[3].y = ypos + obj_size;
	pts[4].x = pts[0].x;
	pts[4].y = pts[0].y;
	break;
    case NINETY:
  	pts[0].x = xpos;
  	pts[0].y = ypos + obj_size/2 - t_width/2;
  	pts[1].x = xpos + obj_size;
  	pts[1].y = ypos + obj_size/2 - t_width/2;
  	pts[2].x = xpos + obj_size;
  	pts[2].y = ypos + obj_size/2 - t_width/2 + t_width;
  	pts[3].x = xpos;
  	pts[3].y = ypos + obj_size/2 - t_width/2 + t_width;
	pts[4].x = pts[0].x;
	pts[4].y = pts[0].y;
	break;
    case FORTYFIVE:
  	pts[0].x = xpos + obj_size - t_width/2-1;
  	pts[0].y = ypos;
  	pts[1].x = xpos + obj_size;
  	pts[1].y = ypos + t_width/2+1;
  	pts[2].x = xpos + t_width/2+1;
  	pts[2].y = ypos + obj_size;
  	pts[3].x = xpos;
  	pts[3].y = ypos + obj_size - t_width/2-1;
	pts[4].x = pts[0].x;
	pts[4].y = pts[0].y;
	break;
    case MINUSFORTYFIVE:
  	pts[0].x = xpos + t_width/2+1;
  	pts[0].y = ypos;
  	pts[1].x = xpos + obj_size;
  	pts[1].y = ypos + obj_size - t_width/2-1;
  	pts[2].x = xpos + obj_size - t_width/2-1;
  	pts[2].y = ypos + obj_size;
  	pts[3].x = xpos;
  	pts[3].y = ypos + t_width/2+1;
	pts[4].x = pts[0].x;
	pts[4].y = pts[0].y;
	break;
  }
  *points = pts;
  *numpoints = 5;
}



/***************************DRAWING A RATE FROM A TRANSITION******************/


/*******************************************************************************
Name:
        DrawRate
--------------------------------------------------------------------------------
Funktionstyp:

--------------------------------------------------------------------------------
Parameter:

--------------------------------------------------------------------------------
Zweck:
        Zeichnet die Ratendefinition einer Transition

*******************************************************************************/

void DrawRate(w,display,drawable,gc)
     Widget w;
     Display *display;
     Drawable drawable;
     GC gc;
{
    int x,xpos,y,ypos,ascent,descent,dummy,hight;
    XCharStruct dummy2;
    String rateParName;
    char rateString[100];
    Rate *rate;
    EnablingDependence enablingDependence;

#ifdef debug
    printf("DrawRate\n");
#endif
    XtVaGetValues(w,
		  XtNrates,&rate,
		  XtNenablingDependence,&enablingDependence,
		  NULL);
    switch (rate->type) 
    {
      case RATEPARAM: 
	XtVaGetValues(rate->values.rateparam,
		      XtNrateParName,&rateParName,
		      NULL);
	sprintf(rateString,"%s",rateParName);
  	break;
      case RATEVALUE:
	sprintf(rateString,"%1.15g",/*enDepString,*/
		rate->values.ratevalue);
 	break;
      case MD_DEF:
	sprintf(rateString,"mark.-dep.");
	break;
      case PMF_FUNC:
  	sprintf(rateString,"pmf-func.");
        break;
      case DATAFILE:
        sprintf(rateString,"Data-File");
	break;
      default:
	sprintf(rateString,"???");
    }

    /* Position of rate is relativ to transition-position */
    XtVaGetValues(w,
		  XtNxpos,&xpos,
		  XtNypos,&ypos,
		  XtNratePos_x, &x,
		  XtNratePos_y, &y,
		  NULL);
    xpos += x;
    ypos += y;

    drawString((NetWidget)getCurrentNet(),
	       xpos,ypos,
	       rateString,strlen(rateString));
    
    if (enablingDependence == INFINITE_SERVER) { 
	XTextExtents(XQueryFont(display,XGContextFromGC(gc)),
		     rateString,strlen(rateString),
		     &dummy,&ascent,&descent,&dummy2);
	hight = ascent+descent;
	drawString((NetWidget)getCurrentNet(),
		   xpos,ypos+hight,
		   "inf_serv",strlen("inf_serv"));
    }
}




/***************************DRAWING A TAG*************************************/


/*******************************************************************************
Name:
        DrawTag
--------------------------------------------------------------------------------
Funktionstyp:

--------------------------------------------------------------------------------
Parameter:

--------------------------------------------------------------------------------
Zweck:
        Zeichnet Etiketten
*******************************************************************************/

void DrawTag(w,display,drawable,gc)
     Widget w;
     Display *display;
     Drawable drawable;
     GC gc;
{
    String tagname;
    int xpos,ypos, tag_x,tag_y;

    XtVaGetValues(w,
		  XtNtagName,&tagname,
		  XtNxpos,&xpos,
		  XtNypos,&ypos,
		  XtNtagXPos, &tag_x,
		  XtNtagYPos, &tag_y,
		  NULL);

    drawString((NetWidget)getCurrentNet(),
	       xpos+tag_x,ypos+tag_y,
	       tagname,strlen(tagname));
}


/**********DRAWING AN ARC ****************************************************/

#define INHIBITOR_IMAGE 0
#define ARROWHEAD 1
#define NO_IMAGE 2

#define ARROW_LENGTH 12.0
#define INH_IMAGE_LENGTH 4.0

static void adjustPoint();
static void DrawArrowHead();
static void DrawInhImage();
static void DrawMultiplicity();
static int getOrientationShift();
static Objects convertTranstypeToObject();

/*******************************************************************************
Name:
        DrawArc
--------------------------------------------------------------------------------
Funktionstyp:
       Global im Berich der Klasse NetWidget
--------------------------------------------------------------------------------
Parameter:
        w: Instanz der Klasse ArcObj
	obj_size: Objektgr"osse
--------------------------------------------------------------------------------
Zweck:
        Zeichnen der ganzen bis jetzt definierten Kante
*******************************************************************************/

void DrawArc(w,display,drawable,gc,obj_size)
     Widget w;
     Display *display;
     Drawable drawable;
     GC gc;
     int obj_size;
     
{
    ArcObject arc=(ArcObject)w;
    Widget toObj,fromObj,*ip_list;
    DrawingDirection drawing_direction;
    ArcType arc_type;
    TransOrientation orientation;
    TransType trans_type;
    Objects object;
    int x,y,to_x,to_y,from_x,from_y,new_x,new_y;
    XPoint *points;
    int section,i,num_points,off_set,image;
    Cardinal num_ip;
    Boolean fromSelected = False, toSelected = False;
    NetWidget nw = (NetWidget)getCurrentNet(); /* XtParent(w); */ 
    
#ifdef debug
    printf("DrawArc \n");
#endif
    
    XtVaGetValues(w,
		  XtNarcType,&arc_type,
		  XtNfromObj,&fromObj,
		  XtNtoObj,&toObj,
		  XtNdrawingDirection,&drawing_direction,
		  XtNnumChildren,&num_ip,
		  XtNchildren,&ip_list,
		  NULL);
    if (toObj == NULL && num_ip == 0)
      return;
    XtVaGetValues(fromObj,
		  XtNxpos,&from_x,
		  XtNypos,&from_y,
		  XtNselected, &fromSelected,
		  NULL);
    if (toObj == NULL) {
	off_set = 1;
	num_points = num_ip+1;
    }
    else {
	XtVaGetValues(toObj,
		      XtNxpos,&to_x,
		      XtNypos,&to_y,
		      XtNselected, &toSelected,
		      NULL);
	off_set = 2;
	num_points = num_ip+2;
    }
    
    if (fromSelected && toSelected) setBrightLine((NetWidget)getCurrentNet());
    
    points = (XPoint*)XtMalloc(num_points*sizeof(XPoint));
    /* Zwischenpunkte eintragen */
    {
	int j, destroyed=0;

	for (j=i=0;i<num_ip;i++) {

	    XtVaGetValues(ip_list[i],
			  XtNxpos,&x,
			  XtNypos,&y,
			  NULL);

	    if (x==DESTROYED && y==DESTROYED) {
		destroyed++;
		if (drawing_direction != UP) {
		    int p;
		    for (p = num_points - (j+off_set);
			 p < num_points - off_set; 
			 p++)
		      points[p].x = points[p+1].x,
		      points[p].y = points[p+1].y;
		    j++;
		}
		continue;
	    }

	    if (drawing_direction == UP) {
		points[j+1].x = x;
		points[j+1].y = y;
	    }
	    else {
		points[num_points - (j+off_set)].x = x;
		points[num_points - (j+off_set)].y = y;
	    }
	    j++;
	}

	num_ip -= destroyed;
	num_points -= destroyed;
    }

    /* Punkte in Objektnaehe berechnen und eintragen */
    /* erster Punkt*/
    if (isTransObject(fromObj))
      {
	  XtVaGetValues(fromObj,
			XtNtransOrient,&orientation,
			XtNtransType,&trans_type,
			NULL);
	  object = convertTranstypeToObject(trans_type);
	  if (arc_type == INPUTARC)
	    image = ARROWHEAD;
	  else
	    if (arc_type == INHIBITORARC)
	      image = INHIBITOR_IMAGE;
	    else
	      image = NO_IMAGE;
      }
    else
      {
	  if(isPlaceObject(fromObj))
	    object = PLACE;
	  else
	    object = NOOBJECT;
	  if (arc_type == OUTPUTARC)
	    image = ARROWHEAD;
	  else
	    image = NO_IMAGE;
      }   
    if (drawing_direction == UP)
      {
	  if (num_ip > 0)
	    section = getSection(from_x,from_y,points[1].x,points[1].y);
	  else
	    section = getSection(from_x,from_y,to_x,to_y);
	  adjustPoint(from_x,from_y,
		      section,obj_size,image,object,orientation,
		      &new_x,&new_y);
	  points[0].x = new_x;
	  points[0].y = new_y;
      }
    else
      {
	  if (num_ip > 0)
	    section = getSection(from_x,from_y,
				 points[num_points - 2].x,
				 points[num_points - 2].y);
	  else
	    section = getSection(from_x,from_y,to_x,to_y);
	  adjustPoint(from_x,from_y,
		      section,obj_size,image,object,orientation,
		      &new_x,&new_y);
	  points[num_points - 1].x = new_x;
	  points[num_points - 1].y = new_y;
      }
    if (image == ARROWHEAD)
      DrawArrowHead(display,drawable,gc,section,obj_size/2,new_x,new_y);
    else
      if (image == INHIBITOR_IMAGE)
	DrawInhImage(display,drawable,gc,section,obj_size,new_x,new_y);
    /* zweiter Punkt */
    if (toObj != NULL)
      {
	  if (isTransObject(toObj))
	    {
		XtVaGetValues(toObj,
			      XtNtransOrient,&orientation,
			      XtNtransType,&trans_type,
			      NULL);
		object = convertTranstypeToObject(trans_type);
		if (arc_type == INPUTARC)
		  image = ARROWHEAD;
		else
		  if (arc_type == INHIBITORARC)
		    image = INHIBITOR_IMAGE;
		  else
		    image = NO_IMAGE;
	    }
	  else
	    {
		if(isPlaceObject(toObj))
		  object = PLACE;
		else
		  object = NOOBJECT;
		if (arc_type == OUTPUTARC)
		  image = ARROWHEAD;
		else
		  image = NO_IMAGE;
	    }
	  if (drawing_direction == UP)
	    {
		section = getSection(to_x,to_y,
				     points[num_points - 2].x,
				     points[num_points - 2].y);
		adjustPoint(to_x,to_y,
			    section,obj_size,image,object,orientation,
			    &new_x,&new_y);
		points[num_points - 1].x = new_x;
		points[num_points - 1].y = new_y;
	    }
	  else
	    {
		section = getSection(to_x,to_y,points[1].x,points[1].y);
		adjustPoint(to_x,to_y,
			    section,obj_size,image,object,orientation,
			    &new_x,&new_y);
		points[0].x = new_x;
		points[0].y = new_y;
	    }
	  if (image == ARROWHEAD)	
	    DrawArrowHead(display,drawable,gc,section,obj_size/2,new_x,new_y);	
	  else
	    if (image == INHIBITOR_IMAGE)
	      DrawInhImage(display,drawable,gc,section,obj_size,new_x,new_y);
      }
    /* ausgabe des Kantenverlaufs */
    drawLines(nw,
	      points,num_points);
    
    if (drawing_direction == UP)
      DrawMultiplicity(arc,display,drawable,gc,
		       points[0].x,points[0].y,points[1].x,points[1].y);
    else
      DrawMultiplicity(arc,display,drawable,gc,
		       points[num_points-1].x,points[num_points-1].y,
		       points[num_points-2].x,points[num_points-2].y);

    if (fromSelected && toSelected) setSolidLine(nw);
}

/*******************************************************************************
Name:
        DrawArcPiece
--------------------------------------------------------------------------------
Funktionstyp:
       Global im Berich der Klasse NetWidget
--------------------------------------------------------------------------------
Parameter:
        w: Instanz der Klasse NetWidget
	e_x,e_y:Ereigniskoordinaten
	obj+_size: Objektgr"ose
	index: Index des selektierten Zwischenpunktes
	var_obj: NULL, falls variables Objekt ein Zwischenpunkt, sonst
	         Instanz des Objektes
--------------------------------------------------------------------------------
Zweck:
        Zeichnet einen Kantenabschnitt
*******************************************************************************/

void DrawArcPiece(w,display,drawable,gc,e_x,e_y,obj_size,index,var_obj)
     Widget w,var_obj;
     Display *display;
     Drawable drawable;
     GC gc;
     int obj_size,e_x,e_y,index;

{
    ArcObject arc=(ArcObject)w;
    Widget toObj,fromObj,obj,*ip_list;
    ArcType arc_type;
    TransOrientation orientation;
    TransType trans_type;
    Objects object;
    int x,y,new_x1,new_y1,new_x2,new_y2;
    int section,num_ip,image;
    
#ifdef debug
    printf("DrawArc \n");
#endif
    
    XtVaGetValues((Widget)arc,
		  XtNarcType,&arc_type,
		  XtNfromObj,&fromObj,
		  XtNtoObj,&toObj,
		  XtNnumChildren,&num_ip,
		  XtNchildren,&ip_list,
		  NULL);

    if (index < 0)
      obj= fromObj;
    else
      if (index >= num_ip)
	obj= toObj;
      else
	obj= ip_list[index];

    /* image bestimmen */
    if (   isPlaceObject(obj) && arc_type == OUTPUTARC
	|| isTransObject(obj) && arc_type == INPUTARC)
      image = ARROWHEAD;
    else
      if (isTransObject(obj) && arc_type == INHIBITORARC)
	image = INHIBITOR_IMAGE;
      else
	image = NO_IMAGE;

    /* x,y Koordinaten des Fixpunktes, Lage und Typ von Transitionen */
    if (isTransObject(obj)) {	
	XtVaGetValues(obj,
		      XtNxpos,&x,
		      XtNypos,&y,
		      XtNtransOrient,&orientation,
		      XtNtransType,&trans_type,
		      NULL);
	object = convertTranstypeToObject(trans_type);
    }
    else {
	XtVaGetValues(obj,
		      XtNxpos,&x,
		      XtNypos,&y,
		      NULL);
	if (isPlaceObject(obj))
	  object = PLACE;
	else
	  object = NOOBJECT;
    }
    
    /* x,y ggf. justieren */
    if (isTransObject(obj) || isPlaceObject(obj)) {
	section = getSection(x,y,e_x,e_y);
	adjustPoint(x,y,
		    section,obj_size,image,object,orientation,
		    &new_x1,&new_y1);
	if (image == ARROWHEAD)
	  DrawArrowHead(display,drawable,gc,section,obj_size/2,new_x1,new_y1);
	else
	  if (image == INHIBITOR_IMAGE)
	    DrawInhImage(display,drawable,gc,section,obj_size,new_x1,new_y1);
    }
    else {
	new_x1 = x;
	new_y1 = y;
    }
    
    /* falls event koordinaten zu einem Objekt gehoeren: justieren */
    if (var_obj != NULL) {

	/* image bestimmen */
	if (   isPlaceObject(var_obj) && arc_type == OUTPUTARC
	    || isTransObject(var_obj) && arc_type == INPUTARC)
	  image = ARROWHEAD;
	else
	  if (isTransObject(var_obj) && arc_type == INHIBITORARC)
	    image = INHIBITOR_IMAGE;
	  else
	    image = NO_IMAGE;
	
      /* x,y Koordinaten des variablen punktes, Lage und Typ von Transitionen */
	if (isTransObject(var_obj)) {	
	    XtVaGetValues(var_obj,
			  XtNtransOrient,&orientation,
			  XtNtransType,&trans_type,
			  NULL);
	    object = convertTranstypeToObject(trans_type);
	}
	else
	  if (isPlaceObject(var_obj))
	    object = PLACE;
	  else
	    object = NOOBJECT;
	section = getSection(e_x,e_y,x,y);
	adjustPoint(e_x,e_y,
		    section,obj_size,image,object,orientation,
		    &new_x2,&new_y2);
	if (image == ARROWHEAD)
	  DrawArrowHead(display,drawable,gc,section,obj_size/2,new_x2,new_y2);
	else
	  if (image == INHIBITOR_IMAGE)
	    DrawInhImage(display,drawable,gc,section,obj_size,new_x2,new_y2);
    }
    else {
	new_x2 = e_x;
	new_y2 = e_y;
    }
    drawLine((NetWidget)getCurrentNet(),
	     new_x1,new_y1,new_x2,new_y2);

    if (index == -1)
      DrawMultiplicity(arc,display,drawable,gc,new_x1,new_y1,new_x2,new_y2);
    else
      if (index == 0 && var_obj == fromObj)
	DrawMultiplicity(arc,display,drawable,gc,new_x2,new_y2,new_x1,new_y1);
}




/**************************************************************/

#define pi 3.1415927
#define NumSect 24
#define SECTOR_ANGLE (pi/(double)(NumSect/2))


/*******************************************************************************
Name:
        getSection
--------------------------------------------------------------------------------
Funktionstyp:
       Global im Berich der Klasse NetWidget
--------------------------------------------------------------------------------
Parameter:
        p1x,p1y: erster Punkt
	p2x,p2y: zweiter Punkt
--------------------------------------------------------------------------------
Rueckgabewert:
        Berechnete Sektorennummer
--------------------------------------------------------------------------------
Zweck:
        Berechnet Sektorennummer f"ur den Streckenabschnitt
*******************************************************************************/

int getSection(p1x,p1y,p2x,p2y)
int p1x,p1y,p2x,p2y;

{
double alpha,beta;
int i;

  if(p1x - p2x != 0){
	alpha = atan((double)(p1y - p2y)/(double)(p1x - p2x));
	/*
	printf("alpha: %f \n",alpha);
	*/
	for (beta= -pi/2.0+SECTOR_ANGLE/2.0,i=0;i<NumSect/2;i++,beta+=SECTOR_ANGLE)
	    if(alpha<beta) 
	    {
		if(p2x < p1x)
		    return i;
		else 
		    return i+NumSect/2;
	    }
  }
  if(p2y < p1y)
	return NumSect/2;
  else 
	return 0;
}


double determOffSet(section,orientation)
int section;
TransOrientation orientation;
{
int nbr = section - getOrientationShift(orientation);
switch (nbr) {
case 3: 
case 4:
case 5:
case 6:
case 7:
case 8:
case 9:
case 10: return 2.0;
default: return 1.0;
}
}

#define fmin(A,B) (A) < (B) ? (A) : (B)


/*******************************************************************************
Name:
        adjustPoint
--------------------------------------------------------------------------------
Funktionstyp:
       Global im Berich der Klasse NetWidget
--------------------------------------------------------------------------------
Parameter:
        w: Instanz der Klasse NetWidget
	section: Sektionsnummer
	obj_size: Objektgr"o"se
	image: zu ber"ucksichtigendes Image
	Objekt: Objektinstanz
	orientation: bei Transitionen, deren Orientierung
	R"uckgabe:
	    new_x,new_y: Justierter Punkt
--------------------------------------------------------------------------------
Zweck:
        Justiert die Koordinatenangabe x,y unter Ber"ucksichtigung des Objektes
	, des Images und der Orientierung des Objektes.
*******************************************************************************/

static void adjustPoint(x,y,section,obj_size,image,object,orientation,new_x,new_y)
int x,y,*new_x,*new_y;
int obj_size,image,section;
Objects object;
TransOrientation orientation;

{
double delta_x,delta_y,alpha,length,t_width,nx,ny;

  if (object == PLACE)
  	length = (double)obj_size/2.0+1.0;
  else
  {
	if (object == EXPTRANS || object == DETTRANS)
	    t_width = (double)obj_size/2.0;
 	else
	    t_width = (double)obj_size/4.0;
	alpha = (double)(abs(/*6 + */getOrientationShift(orientation) - section)%12)*SECTOR_ANGLE;
	if (alpha == 0.0)
	    length = (double)obj_size/2.0+1.0;
	else
	    length = Min(0.5*sqrt(obj_size*obj_size+t_width*t_width),t_width/(2.0*sin(alpha)));
/*
  	length += determOffSet(section,orientation);
*/
  }

  if (image == ARROWHEAD) 
	length += ARROW_LENGTH +  1.0;
  if (image == INHIBITOR_IMAGE)
	length += INH_IMAGE_LENGTH + 1.0;

  delta_x = (length+1.0) * cos((double)section * SECTOR_ANGLE);
  delta_y = (length+1.0) * sin((double)section * SECTOR_ANGLE);
/*
printf("sector:%d	delta_x: %f delta_y: %f\n",sector,delta_x,delta_y);
*/
  nx = (double)x-(delta_y);
  ny = (double)y+(delta_x);
  *new_x = (int)(nx+0.5);
  *new_y = (int)(ny+0.5);
}



#define fsign(X) ((X) >= 0.0 ? 1 : -1)


/*******************************************************************************
Name:
        DrawArrowHead
--------------------------------------------------------------------------------
Funktionstyp:
       private Funktion
--------------------------------------------------------------------------------
Parameter:
        sector: Sektorennummer
	x,y: Basispunkt der Pfeilspitze
	
--------------------------------------------------------------------------------
Zweck:
        Zeichnet Pfeilspitze
*******************************************************************************/

static void DrawArrowHead(display,drawable,gc,sector,os,x,y)
     Display* display;
     Drawable drawable;
     GC gc;
     int sector,os;
     int x,y;

{
    XPoint arrowhead[3];

    double delta_x,delta_y,s_length;

#define ARROW_ANGLE (pi/10.0)
    
    delta_x = ARROW_LENGTH*sin((double)sector * SECTOR_ANGLE);
    delta_y = ARROW_LENGTH*cos((double)sector * SECTOR_ANGLE);
    arrowhead[1].x = x + (int)(delta_x + fsign(delta_y)*0.5);
    arrowhead[1].y = y - (int)(delta_y + fsign(delta_y)*0.5);
    
    s_length = ARROW_LENGTH/cos(ARROW_ANGLE);
    
    delta_x = s_length*sin((double)sector * SECTOR_ANGLE + ARROW_ANGLE);
    delta_y = s_length*cos((double)sector * SECTOR_ANGLE + ARROW_ANGLE);
    arrowhead[0].x = arrowhead[1].x - (int)(delta_x + fsign(delta_y)*0.5);
    arrowhead[0].y = arrowhead[1].y + (int)(delta_y + fsign(delta_y)*0.5);
    
    delta_x = s_length*sin((double)sector * SECTOR_ANGLE - ARROW_ANGLE);
    delta_y = s_length*cos((double)sector * SECTOR_ANGLE - ARROW_ANGLE);
    arrowhead[2].x = arrowhead[1].x - (int)(delta_x + fsign(delta_y)*0.5);
    arrowhead[2].y = arrowhead[1].y + (int)(delta_y + fsign(delta_y)*0.5);
    
    /*
      if (arrowhead[1].x > arrowhead[2].x)
      {
      arrowhead[1].x += 1;
      if (arrowhead[2].y < arrowhead[1].y)
      arrowhead[1].y += 1;
      else
      arrowhead[1].y -= 1;
      }
      else
      if (arrowhead[1].x < arrowhead[2].x)
      {
      arrowhead[2].x += 1;
      if (arrowhead[2].y < arrowhead[1].y)
      arrowhead[2].y += 1;
      else
      arrowhead[2].y -= 1;
      }
      */
    /*
      drawLines(nw,arrowhead,3);
      */
    fillPolygon((NetWidget)getCurrentNet(),
		arrowhead,3,Convex,CoordModeOrigin);
    
}


/*******************************************************************************
Name:
        DrawInhImage
--------------------------------------------------------------------------------
Funktionstyp:
       Global im Berich der Klasse NetWidget
--------------------------------------------------------------------------------
Parameter:
        sector: Sektorennummer
	x,y: Basispunkt des Images
	
--------------------------------------------------------------------------------
Zweck:
       Zeichnet Image f"ur hemmende kanten 
*******************************************************************************/

static void DrawInhImage(display,drawable,gc,sector,os,x,y)
     Display* display;
     Drawable drawable;
     GC gc;
     int sector,os;
     int x,y;

{
    double p_x,p_y;
    int inh_width = (int)INH_IMAGE_LENGTH;
    double delta_x,delta_y;
    double size =  INH_IMAGE_LENGTH/2.0 + 1.5;


#ifdef debug
    printf("DrawInhImage \n");
#endif

    delta_x = size *  sin((double)sector * SECTOR_ANGLE);
    delta_y = size *  cos((double)sector * SECTOR_ANGLE);
    
    p_x = (double)x + (delta_x) - (INH_IMAGE_LENGTH/2.0);	 
    p_y = (double)y - (delta_y) - (INH_IMAGE_LENGTH/2.0);	 

    drawArc((NetWidget)getCurrentNet(),
	     (int)(p_x+0.5),(int)(p_y+0.5),inh_width,inh_width,0,360*64);
}

static int getOrientationShift(orientation)
     TransOrientation orientation;
{
    switch(orientation) {
      case ZERO:return 0;
      case FORTYFIVE:return 3;
      case NINETY:return 6;
      case MINUSFORTYFIVE:return 9;
    }
}

static Objects convertTranstypeToObject(transtype)
     TransType transtype;
{
    switch (transtype) {
      case EXPONENTIEL: return EXPTRANS;
      case DETERMINISTIC: return DETTRANS;
      case IMMEDIATE: return IMTRANS;
    }
}


/*******************************************************************************
Name: DrawMultiplicity
--------------------------------------------------------------------------------
Funktionstyp:
--------------------------------------------------------------------------------
Registriert mit/ triggert bei event:
--------------------------------------------------------------------------------
Parameter:
--------------------------------------------------------------------------------
Rueckgabewert:
--------------------------------------------------------------------------------
Zweck:
--------------------------------------------------------------------------------
Bemerkung:
	Damits besser aussieht, fonthoehe abfragen.
	falls p1_y > p2_y: base_y um fonthoehe erhoehen
	falls p1_y < p2_y: base_y um fonthoehe erniedrigen
*******************************************************************************/

static void DrawMultiplicity(w,display,drawable,gc,p1_x,p1_y,p2_x,p2_y)
     Widget w;
     Display *display;
     Drawable drawable;
     GC gc;
     int p1_x,p1_y,p2_x,p2_y;

{
    int mbase_x,mbase_y,ascent,descent,dummy;
    XCharStruct overall;
    Multiplicity *mult;
    char multString[100];
    DrawMode multiplMode = ((NetWidget)getCurrentNet())->netWidget.multiplMode;

    if (multiplMode != NoneM)
    {
	XtVaGetValues(w,
		      XtNmultiplicity,&mult,
		      NULL);
	if ( (mult->type != ARCVALUE) || (mult->values.arcvalue > 1)) { 
	    /*
	    font = XLoadFont(display,"7x13");
	    new_values.font = font;
	    XSetFont(display,gc,font);
	    XGetGCValues(display,gc,mask,&old_values);
	    XChangeGC(display,gc,mask,&new_values);
	    */

	    if (((NetWidget)getCurrentNet())->netWidget.tokenGame) 
		if (mult->actualValue == Impossible)
		    multString[0]='\0';
		else 
		    sprintf(multString, "%u", mult->actualValue);
	    else
		if (mult->type != ARCVALUE)
		    sprintf(multString,"mark.-dep");
		else
		    sprintf(multString,"%d",mult->values.arcvalue);
	    
	    XTextExtents(XQueryFont(display,XGContextFromGC(gc)),
			 multString,strlen(multString),
			 &dummy,&ascent,&descent,&overall);
	    mbase_x = p1_x + (p2_x - p1_x)/2 - overall.width/2;
	    mbase_y = p1_y + (p2_y - p1_y)/2 + ascent/2;
	    
	    if (multiplMode == UndrawM)
		drawImageString((NetWidget)getCurrentNet(),
				mbase_x,mbase_y,"         ",strlen(multString));
	    else /* Draw */
		drawImageString((NetWidget)getCurrentNet(),
				mbase_x,mbase_y,multString,strlen(multString)); 

        /*
	XChangeGC(display,gc,mask,&old_values);
	XSetFont(display,gc,old_values.font);
	XUnloadFont(display,font);
	*/
	}
    }
}

/*******************DRAWING A MARKINGPARAMETER*********************************/

/*******************************************************************************
Name:
        DrawMarkPar
--------------------------------------------------------------------------------
Funktionstyp:

--------------------------------------------------------------------------------
Parameter:
        x,y: Koordinaten, an denen Objekt zu Zeichnen ist
--------------------------------------------------------------------------------
Zweck:
        Zeichnet Markenparameter
*******************************************************************************/

void DrawMarkPar(w,display,drawable,gc,x,y,obj_size,mode)
     Widget w;
     Display *display;
     Drawable drawable;
     GC gc;
     int x,y,obj_size,mode;
     
{
    String name;
    char str[100];
    Cardinal value;
    
#ifdef debug
    printf("MarkPar: Draw \n");
#endif

    XtVaGetValues(w,
		  XtNmparName,&name,
		  XtNmparValue,&value,
		  NULL);
    sprintf(str,"%s=%d",name,value);
    drawString((NetWidget)getCurrentNet(),
	       x,y,str,strlen(str));
}

/*****************************DRAWING A RATEPARAMETER************************/


/*******************************************************************************
Name:
      DrawRatePar  
--------------------------------------------------------------------------------
Funktionstyp:

--------------------------------------------------------------------------------
Parameter:
        x,y: Koordianten, an denen Objekt zu zeichnen ist
	
--------------------------------------------------------------------------------
Zweck:
        Zeichnet Ratenparameter
*******************************************************************************/
/* IS IT USED ??? Yes in Draw() => DrawFktTable */

void DrawRatePar(w,display,drawable,gc,x,y,obj_size,mode)
     Widget w;
     Display *display;
     Drawable drawable;
     GC gc;
     int x,y,obj_size,mode;

{
    String name;
    char str[80], *s;
    RateValue *value;

#ifdef debug
    printf("RatePar: Draw \n");
#endif

    XtVaGetValues(w,
		  XtNrateParName,&name,
		  XtNrateParValue,&value,
		  NULL);
    if (value->valueType == Ratevalue) 
    {
	sprintf(str,"%s = %1.15g",name,value->values.value);
	drawString((NetWidget)getCurrentNet(),
		   x,y,str,strlen(str));
    }
    else
    {
	s = XtMalloc((strlen(name)+strlen(value->values.definition)+
		     strlen(" = ")+1)*sizeof(char));
	sprintf(s,"%s = %s ", name, value->values.definition);
	drawString((NetWidget)getCurrentNet(),
		   x,y,s,strlen(s));
	XtFree(s);
    }
}

/*****************************************************************************/
void ClearNetPixmap(nw)
/*****************************************************************************/
     NetWidget nw;
{
    XCopyArea(XtDisplay(nw), nw->netWidget.picture,nw->netWidget.picture,
              nw->netWidget.clear_GC,
              0,0, nw->core.width,nw->core.height, 0,0);
}

/*****************************************************************************/
void ShowNet(nw)
/*****************************************************************************/
NetWidget nw;
{
    ClearNetPixmap(nw);
    Drawgrid((Widget)nw,Off,nw->netWidget.gridsize);
    DrawNet((Widget)nw, NORMAL);
}

#ifdef __STDC__
/*****************************************************************************/
void CopyToDisplay
/*****************************************************************************/
    (NetWidget nw, Pixmap map, int x, int y, int width, int height)
#else
void CopyToDisplay(nw, map, x, y, width, height)
     NetWidget nw;
     Pixmap map;
     int x, y, width, height;
#endif
{

    if (DefaultDepthOfScreen(XtScreen(nw)) == 1)
      XCopyArea(XtDisplay(nw), 
		map,
		XtWindow(nw),
		nw->netWidget.copy_GC,x,y,width,height,x,y);
    else 
      XCopyPlane(XtDisplay(nw), 
		 map,
		 XtWindow(nw),
		 nw->netWidget.copy_GC,x,y,width,height,x,y,1);
}

#ifdef __STDC__
/*****************************************************************************/
void CopyToFlash(NetWidget nw)
/*****************************************************************************/
#else
void CopyToFlash(nw)
     NetWidget nw;
#endif
{
    XSetFunction(XtDisplay(nw),nw->netWidget.draw_GC,GXcopy);
    XCopyArea(XtDisplay(nw), 
	      nw->netWidget.picture, nw->netWidget.flash,
	      nw->netWidget.draw_GC,0,0,nw->core.width,nw->core.height,0,0);
    XSetFunction(XtDisplay(nw),nw->netWidget.draw_GC,GXxor);
}

#ifdef __STDC__
/*****************************************************************************/
void FlashNet(NetWidget nw)
/*****************************************************************************/
#else
void FlashNet(nw)
     NetWidget nw;
#endif
{
    
    if (nw->netWidget.flashed) {
	CopyToDisplay(nw, nw->netWidget.picture,
		      0,0,nw->core.width,nw->core.height);
    }
    else {
	CopyToFlash(nw);
	showEnabledTranss(nw);
	CopyToDisplay(nw, nw->netWidget.flash,
		      0,0,nw->core.width,nw->core.height);
    }
}

/* shows selected objects */
#ifdef __STDC__
/*****************************************************************************/
void showSelectedObjects(NetWidget nw)
/*****************************************************************************/
#else
void showSelectedObjects(nw)
    NetWidget nw;
#endif
{
    Redraw ((Widget)nw, NULL, NULL, NULL);
}


#ifdef __STDC__
/*****************************************************************************/
void undrawSelectedObjects(NetWidget nw)
/*****************************************************************************/
#else
void undrawSelectedObjects(nw)
    NetWidget nw;
#endif
{
    int i, x, y;
    Widget fromObj, toObj;
    int fromX, fromY, toX, toY;
    Cardinal num_children;
    Boolean selected, fromSelected, toSelected;

    for (i=FirstArc(nw); i < LastArc(nw); i++) {
	XtVaGetValues(nw->composite.children[i],
		      XtNfromObj, &fromObj,
		      XtNtoObj, &toObj,
		      XtNnumChildren, &num_children,
		      NULL);
	XtVaGetValues(fromObj,
		      XtNselected, &fromSelected,
		      XtNxpos, &fromX,
		      XtNypos, &fromY,
		      NULL);
	XtVaGetValues(toObj,
		      XtNselected, &toSelected,
		      XtNxpos, &toX,
		      XtNypos, &toY,
		      NULL);
	if (fromSelected || toSelected) 
	  DrawArc(nw->composite.children[i],
		  XtDisplay(nw), nw->netWidget.picture,
		  nw->netWidget.draw_GC,
		  nw->netWidget.object_size);
    }
    for (i=FirstTrans(nw); i < LastTrans(nw); i++) {
	XtVaGetValues(nw->composite.children[i],
		      XtNselected, &selected,
		      XtNxpos, &x,
		      XtNypos, &y,
		      NULL);
	if (selected)
	  DrawTrans(nw->composite.children[i],
		    XtDisplay(nw), nw->netWidget.picture,
		    nw->netWidget.draw_GC,
		    x,y,
		    nw->netWidget.object_size,0);
    }
    for (i=FirstPlace(nw); i < LastPlace(nw); i++) {
	XtVaGetValues(nw->composite.children[i],
		      XtNselected, &selected,
		      XtNxpos, &x,
		      XtNypos, &y,
		      NULL);
	if (selected)
	  DrawPlace(nw->composite.children[i],
		      XtDisplay(nw), nw->netWidget.picture,
		      nw->netWidget.draw_GC,
		      x,y, nw->netWidget.object_size, 0);
    }
}


#ifdef __STDC__
/*****************************************************************************/
void closeSelection(NetWidget nw, int x, int y)
/*****************************************************************************/
#else
void closeSelection(nw, x, y)
     NetWidget nw;
     int x, y;
#endif
{
  if (nw->netWidget.selectionBorder)
    if (nw->netWidget.selectionBorder->numVertexes) {
      setDashedLine(nw);
      drawLine(nw, 
	       (int) nw->netWidget.selectionBorder->vertexes
	       [nw->netWidget.selectionBorder->numVertexes - 1].x,
	       (int) nw->netWidget.selectionBorder->vertexes
	       [nw->netWidget.selectionBorder->numVertexes - 1].y,
	       x, y);
      drawLine(nw,
	       (int) nw->netWidget.selectionBorder->vertexes
	       [nw->netWidget.selectionBorder->numVertexes - 1].x,
	       (int) nw->netWidget.selectionBorder->vertexes
	       [nw->netWidget.selectionBorder->numVertexes - 1].y,
	       (int) nw->netWidget.selectionBorder->vertexes[0].x,
	       (int) nw->netWidget.selectionBorder->vertexes[0].y);
      setSolidLine(nw);
    }
}

/* undraws the selectionBorder */
#ifdef __STDC__
/*****************************************************************************/
void undrawSelectionBorder(NetWidget nw)
/*****************************************************************************/
#else
void undrawSelectionBorder(nw)
    NetWidget nw;
#endif
{
  unsigned i;

  if (nw->netWidget.selectionBorder) {
      setDashedLine(nw);
      for (i=0; i < nw->netWidget.selectionBorder->numVertexes-1; i++) {
	  drawLine(nw,
		   (int) nw->netWidget.selectionBorder->vertexes[i].x,
		   (int) nw->netWidget.selectionBorder->vertexes[i].y,
		   (int) nw->netWidget.selectionBorder->vertexes[i+1].x,
		   (int) nw->netWidget.selectionBorder->vertexes[i+1].y);
      }
      drawLine(nw,
	       (int) nw->netWidget.selectionBorder->vertexes[i].x,
	       (int) nw->netWidget.selectionBorder->vertexes[i].y,
	       (int) nw->netWidget.selectionBorder->vertexes[0].x,
	       (int) nw->netWidget.selectionBorder->vertexes[0].y);
      setSolidLine(nw);
  }
}

#ifdef __STDC__
/*****************************************************************************/
void undrawCanceldSelectionBorder(NetWidget nw, int x, int y)
/*****************************************************************************/
#else
void undrawCanceldSelectionBorder(nw, x, y)
     NetWidget nw;
     int x, y;
#endif
{
  unsigned i;

  if (nw->netWidget.selectionBorder &&
      nw->netWidget.selectionBorder->numVertexes) {
      setDottedLine(nw);
      drawArc(nw,
	      (int) nw->netWidget.selectionBorder->vertexes[0].x - TOLERANCE,
	      (int) nw->netWidget.selectionBorder->vertexes[0].y - TOLERANCE,
	      TOLERANCE*2, TOLERANCE*2, 0, 360*64);
      setDashedLine(nw);
      drawLine(nw, 
	       (int) nw->netWidget.selectionBorder->vertexes
	       [nw->netWidget.selectionBorder->numVertexes - 1].x,
	       (int) nw->netWidget.selectionBorder->vertexes
	       [nw->netWidget.selectionBorder->numVertexes - 1].y,
	       x,y);
      for (i=0; i < nw->netWidget.selectionBorder->numVertexes - 1; i++)
	  drawLine(nw,
		   (int) nw->netWidget.selectionBorder->vertexes[i].x,
		   (int) nw->netWidget.selectionBorder->vertexes[i].y,
		   (int) nw->netWidget.selectionBorder->vertexes[i+1].x,
		   (int) nw->netWidget.selectionBorder->vertexes[i+1].y);
      setSolidLine(nw);
  }
}


#ifdef __STDC__
/*****************************************************************************/
/* drawLine in pixmap AND on screen with draw_GC / screen_GC */
void drawLine(NetWidget nw, int x1, int y1, int x2, int y2) 
/*****************************************************************************/
#else 
void drawLine(nw, x1, y1, x2, y2)
    NetWidget nw;
    int x1, y1, x2, y2;
#endif 
{
    XDrawLine(XtDisplay(nw), nw->netWidget.picture, nw->netWidget.draw_GC,
	      x1, y1,
	      x2, y2);
#ifdef DRAW_ON_SCREEN
    XDrawLine(XtDisplay(nw), XtWindow(nw), nw->netWidget.screen_GC,
	      x1, y1,
	      x2, y2);
#endif
}

#ifdef __STDC__
void drawRectangle(NetWidget nw, int x, int y, unsigned w, unsigned h)
#else 
void drawRectangle(nw, x, y, w, h)
     NetWidget nw;
     int x,y;
     unsigned w,h;
#endif 
{
    XDrawRectangle(XtDisplay(nw), nw->netWidget.picture, nw->netWidget.draw_GC,
		   x,y, w,h);
    XDrawRectangle(XtDisplay(nw), XtWindow(nw), nw->netWidget.screen_GC,
		   x,y, w,h);

}


#ifdef __STDC__
void drawTransRectangle(NetWidget nw, XPoint *points, int npoints)
#else 
void drawTransRectangle(nw, points, npoints)
     NetWidget nw;
     XPoint *points;
     int npoints;
#endif 
{
	XDrawLines(XtDisplay(nw), nw->netWidget.picture, nw->netWidget.draw_GC,
	       points, npoints, CoordModeOrigin);
}


#ifdef __STDC__
/*****************************************************************************/
/* drawLines in pixmap AND on screen with draw_GC / screen_GC */
void drawLines(NetWidget nw, XPoint *points, int npoints)
/*****************************************************************************/
#else 
void drawLines(nw, points, npoints)
     NetWidget nw;
     XPoint *points;
     int npoints;
#endif 
{
    if (nw->netWidget.spline && npoints > 2) 
      splineLines(nw, points, npoints);
    else
      XDrawLines(XtDisplay(nw), nw->netWidget.picture, nw->netWidget.draw_GC,
		 points, npoints, CoordModeOrigin);

#ifdef DRAW_ON_SCREEN
    XDrawLines(XtDisplay(nw), XtWindow(nw), nw->netWidget.screen_GC,
	      points, npoints, CoordModeOrigin);
#endif
}

#ifdef __STDC__
/*****************************************************************************/
void splineLines(NetWidget nw, XPoint *points, int npoints)
/*****************************************************************************/
#else 
void splineLines(nw, points, npoints)
    NetWidget nw;
    XPoint *points;
    int npoints;
#endif 
{
    int pIndex = 0;
    double l1, l2, r;
    double lastX, lastY;

    if (npoints > 0) lastX = (double)points[0].x, lastY = (double)points[0].y;

    while ( pIndex+2 < npoints ) {
	l1 = sqrt (pow(lastX-(double)points[pIndex+1].x,2.0) +
		   pow(lastY-(double)points[pIndex+1].y,2.0));
	l2 = sqrt (pow((double)(points[pIndex+1].x-
				(double)points[pIndex+2].x),2.0) +
		   pow((double)(points[pIndex+1].y-
				(double)points[pIndex+2].y),2.0));
	if (pIndex+3 != npoints) l2 /= 2.0;

	r = (l1 > l2) ? l2 : l1;
	r = (r > DEFAULT_SPLINE_RADIUS) ? /* resize !!! */
	                (double)DEFAULT_SPLINE_RADIUS : r;

	arcTo(nw, r,
	      lastX, lastY,
	      (double)points[pIndex+1].x,(double)points[pIndex+1].y,
	      (double)points[pIndex+2].x,(double)points[pIndex+2].y,
	      &lastX, &lastY);
	pIndex++;
    }
    drawLine (nw, 
	      (int)lastX, (int)lastY, 
	      points[npoints-1].x, points[npoints-1].y);
}

#ifdef __STDC__
void arcTo(NetWidget nw, double r,
	   double ax0, double ay0, 
	   double ax1, double ay1, 
	   double ax2, double ay2,
	   double* lastX, double* lastY)
#else
void arcTo(nw, r, ax0, ay0, ax1, ay1, ax2, ay2, lastX, lastY)
    NetWidget nw;
    double r, ax0, ay0, ax1, ay1, ax2, ay2;
    double *lastX, *lastY;
#endif
{	
    double xt0, yt0, xt2, yt2;
    
    if ( r < 0 ) {
	XtWarning("Arcto with rad < 0. Take 0.");
	r = 0.0;
    }

    {	/* Now we have to compute the tangent points. */
	/* Basically, the idea is to compute the tangent */
	/* of the bisector by using tan(x+y) and tan(z/2) */
	/* formulas, without ever using any trig. */
	double dx0 = ax0 - ax1, dy0 = ay0 - ay1;
	double dx2 = ax2 - ax1, dy2 = ay2 - ay1;
	
	/* Compute the squared lengths from p1 to p0 and p2. */
	double sql0 = dx0 * dx0 + dy0 * dy0;
	double sql2 = dx2 * dx2 + dy2 * dy2;
	
	/* Compute the distance from p1 to the tangent points. */
	/* This is the only hairy part. */
	double num = dy0 * dx2 - dy2 * dx0;
	double denom = sqrt(sql0 * sql2) - (dx0 * dx2 + dy0 * dy2);
	
	/* Check for collinear points. */
	if ( fabs(num) < 1.0e-6 || fabs(denom) < 1.0e-6 ) {	
	    drawLine (nw, (int)ax0, (int)ay0, (int)ax1, (int)ay1);
	    *lastX = ax1;
	    *lastY = ay1;
	}
	else {	   /* not collinear */
	    double dist = fabs(r * num / denom);
	    double l0 = dist / sqrt(sql0), l2 = dist / sqrt(sql2);
	    double sqlt, sqlt0, sqlt2, sqlc, sqlm, xm, ym, xc, yc;

	    xt0 = ax1 + dx0 * l0;
	    yt0 = ay1 + dy0 * l0;
	    xt2 = ax1 + dx2 * l2;
	    yt2 = ay1 + dy2 * l2;

	    /* compute the center of the arc .... */
	    /* square length from final points of the arc to the point 
	       at which the extended tangents meet */
	    sqlt = ( (xt0-ax1)*(xt0-ax1) + (yt0-ay1)*(yt0-ay1) );  
	    /* length from tangents cross point to arc center */
	    sqlc = ( sqlt + r*r );       /* Pythagoras */
	    /* middle point between final points of the arc */
	    xm = (xt0+xt2) / 2.0, ym = (yt0+yt2) / 2.0;
	    /* length from tangents cross point to middle point */
	    sqlm = ( (xm-ax1)*(xm-ax1) + (ym-ay1)*(ym-ay1) );
	    /* x,y of the arc center */
	    xc = ( ax1 - (ax1-xm) * sqrt(sqlc) / sqrt(sqlm) ),
	    yc = ( ay1 - (ay1-ym) * sqrt(sqlc) / sqrt(sqlm) );

#define sqlength(x1,y1,x2,y2) ((x2-x1)*(x2-x1)+(y2-y1)*(y2-y1))
	  
	    sqlt0=sqlength(ax1,ay1,xt0,yt0);
	    sqlt2=sqlength(ax1,ay1,xt2,yt2);
	    if ( (sql0 < sqlt0) ||
		 (sql2 < sqlt2)) {
		/* the radius of the arc has to be smaller ... */		
		/* got root from squared length, dont be confused from names */
		sql0 = sqrt(sql0); sql2 = sqrt(sql2);
		sqlt0 = sqrt(sqlt0); sqlt2 = sqrt(sqlt2);

		l0 = sqlt0 - sql0; l2 = sqlt2 - sql2;
		dist = (l0 > l2) ? l0 : l2; /* max */
		
		if (dist==l0) {
		    xt0 = ax0; yt0 = ay0;     /* lastXY */
		    dist = sql0/sqlt2;
		    xt2 = ax1 + (xt2-ax1) * dist;
		    yt2 = ay1 + (yt2-ay1) * dist;
		    xc  = ax1 + (xc -ax1) * dist;
		    yc  = ay1 + (yc -ay1) * dist;
		    r   = r * sql0/sqlt0;
		}
		else {
		    xt2 = ax2; yt2 = ay2;
		    dist = sql2/sqlt0;
		    xt0 = ax1 + (xt0-ax1) * dist; 
		    yt0 = ay1 + (yt0-ay1) * dist; 
		    xc  = ax1 + (xc -ax1) * dist;
		    yc  = ay1 + (yc -ay1) * dist;
		    r   = r * sql2/sqlt2;
		}
	    }

	    drawLine (nw, (int)*lastX, (int)*lastY, (int)xt0, (int)yt0);
	    drawArcIn (nw, r, xt0, yt0, xt2, yt2, xc, yc);

	    *lastX = xt2;
	    *lastY = yt2;
	}
    }
}

#ifdef __STDC__
static void drawArcIn (NetWidget nw, double r, 
	  double x0, double y0, 
	  double x3, double y3, 
	  double xc, double yc)
#else
static void drawArcIn ( nw, r, x0, y0, x3, y3, xc, yc)
    NetWidget nw;
    double r, x0,  y0,  x3, y3, xc, yc;
#endif
{
    /* angles from center to final points of the arc */
    int angle0 = (int)( atan2( yc-y0, x0-xc ) / M_PI * 180.0 * 64.0 );
    int angle1 = (int)( atan2( yc-y3, x3-xc ) / M_PI * 180.0 * 64.0 );
    int help;

    angle0 = (angle0 < 0) ? 360*64 + angle0 : angle0;
    angle1 = (angle1 < 0) ? 360*64 + angle1 : angle1;

    if (angle1 < angle0) help = angle1, angle1 = angle0, angle0 = help;

    help = angle1 - angle0;
    if (help > 180*64) 
      help = help - 360*64;

    drawArc(nw, (int)(xc-r), (int)(yc-r), 
	    (unsigned)r*2, (unsigned)r*2, 
	    angle0, help);
}


	
#ifdef __STDC__
/*****************************************************************************/
/* drawArc in pixmap AND on screen with draw_GC / screen_GC */
void drawArc(
/*****************************************************************************/
	     NetWidget nw,
	     int x, int y, 
	     unsigned int width, unsigned int height, 
	     int angle1, int angle2) 
#else 
void drawArc(nw, x, y, width, height, angle1, angle2) 
    NetWidget nw;
    int x, y;
    unsigned int width, height;
    int angle1, angle2;
#endif 
{
    XDrawArc(XtDisplay(nw), nw->netWidget.picture, nw->netWidget.draw_GC,
	     x, y,
	     width, height,
	     angle1, angle2);
#ifdef DRAW_ON_SCREEN
    XDrawArc(XtDisplay(nw), XtWindow(nw), nw->netWidget.screen_GC,
	     x, y,
	     width, height,
	     angle1, angle2);
#endif
}

#ifdef __STDC__
void drawPoints (NetWidget nw, XPoint *points, int npoints, int mode)
#else
void drawPoints (nw, points, npoints, mode)
     NetWidget nw;
     XPoint *points;
     int npoints;
     int mode;
#endif
{
    XDrawPoints(XtDisplay(nw), nw->netWidget.picture, nw->netWidget.draw_GC,
		points, npoints, mode);
#ifdef DRAW_ON_SCREEN
    XDrawPoints(XtDisplay(nw), XtWindow(nw), nw->netWidget.screen_GC,
		points, npoints, mode);
#endif
}

#ifdef __STDC__
void drawString (NetWidget nw, int x, int y, char *s, int l)
#else
void drawString (nw, x, y, s, l)
     NetWidget nw;
     int x,y;
     char *s;
     int l;
#endif
{
    XDrawString(XtDisplay(nw), nw->netWidget.picture, nw->netWidget.draw_GC,
		x,y, s,l);
#ifdef DRAW_ON_SCREEN
    XDrawString(XtDisplay(nw), XtWindow(nw), nw->netWidget.screen_GC,
		x,y, s,l);
#endif
}

#ifdef __STDC__
void drawImageString (NetWidget nw, int x, int y, char *s, int l)
#else
void drawImageString (nw, x, y, s, l)
     NetWidget nw;
     int x,y;
     char *s;
     int l;
#endif
{
    Display *display = XtDisplay(nw);
    unsigned long mask = GCFunction;
    XGCValues old_values, new_values;

    XGetGCValues(display,nw->netWidget.draw_GC,mask,&old_values);

    new_values.function = GXcopy;
    XChangeGC(display,nw->netWidget.draw_GC,mask,&new_values);

    XDrawImageString(XtDisplay(nw), nw->netWidget.picture, 
		     nw->netWidget.draw_GC,
		     x,y, s,l);
    
    XChangeGC(display,nw->netWidget.draw_GC,mask,&old_values);
}

#ifdef __STDC__
void fillPolygon (NetWidget nw, 
		  XPoint *points, int npoints, int shape, int mode)
#else
void fillPolygon (nw, points, npoints, shape, mode)
     NetWidget nw; 
     XPoint *points;
     int npoints, shape, mode;
#endif
{
    XFillPolygon(XtDisplay(nw), nw->netWidget.picture, nw->netWidget.draw_GC,
		 points, npoints, shape, mode);
#ifdef DRAW_ON_SCREEN
    XFillPolygon(XtDisplay(nw), XtWindow(nw), nw->netWidget.screen_GC,
		 points, npoints, shape, mode);
#endif
}


#ifdef __STDC__
void fillStyle(NetWidget nw,int fill_style)
#else
void fillStyle (nw,fill_style)
	NetWidget nw;
	int fill_style;
#endif
{
	XSetFillStyle(XtDisplay(nw), nw->netWidget.draw_GC, fill_style);
#ifdef DRAW_ON_SCREEN
	XSetFillStyle(XtDisplay(nw), nw->netWidget.screen_GC, fill_style);
#endif
}

#ifdef __STDC__
/*****************************************************************************/
void setDottedLine(NetWidget nw)
/*****************************************************************************/
#else
void setDottedLine(nw)
     NetWidget nw;
#endif
{
  XSetLineAttributes(XtDisplay(nw), nw->netWidget.screen_GC,
		     1,
		     LineOnOffDash,
		     CapButt,
		     JoinMiter);
  XSetDashes(XtDisplay(nw), nw->netWidget.screen_GC,
	     0,
	     "\002\002",
	     2);  
  XSetLineAttributes(XtDisplay(nw), nw->netWidget.draw_GC,
		     1,
		     LineOnOffDash,
		     CapButt,
		     JoinMiter);
  XSetDashes(XtDisplay(nw), nw->netWidget.draw_GC,
	     0,
	     "\002\002",
	     2);  
}

#ifdef __STDC__
/*****************************************************************************/
void setDashedLine(NetWidget nw)
/*****************************************************************************/
#else
void setDashedLine(nw)
     NetWidget nw;
#endif
{
  XSetLineAttributes(XtDisplay(nw), nw->netWidget.screen_GC,
		     1,
		     LineOnOffDash,
		     CapButt,
		     JoinMiter);
  XSetDashes(XtDisplay(nw), nw->netWidget.screen_GC,
	     0,
	     "\004\004",
	     2);
  XSetLineAttributes(XtDisplay(nw), nw->netWidget.draw_GC,
		     1,
		     LineOnOffDash,
		     CapButt,
		     JoinMiter);
  XSetDashes(XtDisplay(nw), nw->netWidget.draw_GC,
	     0,
	     "\004\004",
	     2);
}

#ifdef __STDC__
/*****************************************************************************/
void setSolidLine(NetWidget nw)
/*****************************************************************************/
#else
void setSolidLine(nw)
     NetWidget nw;
#endif
{
  XSetLineAttributes(XtDisplay(nw), nw->netWidget.screen_GC,
		     1,
		     LineSolid,
		     CapButt,
		     JoinMiter);
  XSetLineAttributes(XtDisplay(nw), nw->netWidget.draw_GC,
		     1,
		     LineSolid,
		     CapButt,
		     JoinMiter);
}

#ifdef __STDC__
/*****************************************************************************/
void setBrightLine(NetWidget nw)
/*****************************************************************************/
#else
void setBrightLine(nw)
     NetWidget nw;
#endif
{
  XSetLineAttributes(XtDisplay(nw), nw->netWidget.screen_GC,
		     2,
		     LineSolid,
		     CapButt,
		     JoinMiter);
  XSetLineAttributes(XtDisplay(nw), nw->netWidget.draw_GC,
		     2,
		     LineSolid,
		     CapButt,
		     JoinMiter);
}

#ifdef __STDC__
void setWaitCursor(void)
#else
void setWaitCursor()
#endif
{
    Cursor cursor;

    cursor = XCreateFontCursor (XtDisplay(app_root), XC_watch);
    XDefineCursor(XtDisplay(app_root), XtWindow(app_root), cursor);
    XFlush(XtDisplay(app_root));
}

#ifdef __STDC__
void setNormalCursor(void)
#else
void setNormalCursor()
#endif
{
    XUndefineCursor(XtDisplay(app_root),XtWindow(app_root));
}
