/*******************************************************************************
Modul           :        NWFileActions.c

Purpose         :        Aktionen zum Sichern und Laden von Modellen

Project Director:        Christoph Lindemann TU Berlin

Programmer      :        Martin M"uller
                         Kolja Koischwitz

History:                 release 1: October 1992
			 release 2: May 1994 ( netio-Modul, *.TN-Fileformat )
*******************************************************************************/

/* Action to save netdescription */
/* TAG und RATE pos NICHT auf 0 setzen !!!*/

#include <stdio.h> 
#include <stdlib.h> 
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include "PlaceObj.h"
#include "TransObj.h"
#include "ArcObj.h"
#include "MarkParObj.h"
#include "RateParObj.h"
#include "ResultObject.h"
#include "environment.h"

/* this is a part of NetWidget, so this include is allowed */
#include "NetWidgetP.h"
#include "netio.h"

#define ERROR         { XtWarning("loadNet: wrong netdeskription");\
                        fclose(netfd);\
                        fclose(deffd);\
                        ClearNet(w);\
                        return; }
#define ERROR_SUB     { XtWarning("loadNet: wrong netdeskription");\
                        return -1; }

/* declaration of private functions */
/* ..for Save() */
static int     printRateDEF();
static int     printArcs();
static void    printMD();
static void    printEnabling ();
static int     getArcsNbr();
static void    printArcMD();
/* ..for Load() */
static int     CreateArcs();

/*int noDataFileTransition;                
extern void onlySimulation(int state);*/

extern void DrawNet();

#define OK 0
#define OPEN_ERROR 1

#define NET_TYPE   "NET_TYPE:"
#define DESCRIPTION "DESCRIPTION:"
#define PLACES     "PLACES:"
#define TRANSITIONS "TRANSITIONS:"
#define DELAYS     "DELAY_PARAMETERS:"
#define MARKINGS   "MARKING_PARAMETERS:"
#define RESULTS   "REWARD_MEASURES:"

#define MARKPAR    "MARKPAR"
#define PLACE      "PLACE"
#define DELAYPARAM   "DELAYPAR"
#define TRANSITION "TRANSITION"

#define DELAYPAR_DEF "DELAYPAR_DEF"
#define EXP_DELAY    "EXP_DELAY"
#define DET_DELAY    "DET_DELAY"
#define GEN_DELAY    "GEN_DELAY"
#define WEIGHT       "WEIGHT"
#define FUNCTION     "FUNCTION"
#define ARC        "CARDINALITY"
#define MEASURE    "MEASURE"


static char *ArcTypeStr[]   = {"INPARCS","OUTPARCS","INHARCS"};
/* benutzt auch in NWTokenGame.c */
char *ArcTypeStrDEF[]= {"INPARC","OUTPARC","INHARC"};
static char *TransTypeStr[] = {"EXP","DET","IM","GEN"};
static char *EnableDepStr[] = {"IS","SS"};
static char *FiringPolicyStr[] = {"RS","RA","RE"};


/*******************************************************************************
Name:
        WriteNet
--------------------------------------------------------------------------------
Funktionstyp:
        Hilfsfunktion
--------------------------------------------------------------------------------
Aufruf aus:
        Save(), checkSyntax.c, NWTokengame.c [vorgesehen (O:]
--------------------------------------------------------------------------------
Zweck:
        Schreiben eines Modelles auf einen Filepointer
*******************************************************************************/
void WriteNet(nw, fp)
NetWidget nw;
FILE *fp;
{    
  int i,j,x,y,tag_x,tag_y;
  int n;
  int mdarcs=0;
  int *mdarctab;

  mdarctab = (int*)calloc((size_t)LastArc(nw)-FirstArc(nw),sizeof(int));
  fprintf(fp,
  	"-- FILE %s.TN CONTAINING STRUCTURAL DESCRIPTION OF A NET\n\n",
  	nw->netWidget.model_name);

  {
     TimeNetType net_type=nw->netWidget.net_type;

     fprintf(fp,"%s\t\t\t%s\n", NET_TYPE,
	   net_type == TN_ESPN ? "EDSPN":
	   net_type == TN_DSPN ? "DSPN":
	   net_type == TN_GSPN ? "GSPN":"CDSPN");
  }
  fprintf(fp,"%s\t\t\t%s\n", DESCRIPTION, "?");

  n = LastPlace(nw)-FirstPlace(nw);
  for (i = FirstPlace(nw); i<LastPlace(nw); i++)
      if ((nw->composite.children[i])->core.being_destroyed)
          n--;
  fprintf(fp,"%s\t\t\t%d\n", PLACES, n);

  n = LastTrans(nw)-FirstTrans(nw);
  for (i = FirstTrans(nw); i<LastTrans(nw); i++)
      if ((nw->composite.children[i])->core.being_destroyed)
          n--;
  fprintf(fp,"%s\t\t%d\n", TRANSITIONS, n);

  n = LastRatePar(nw)-FirstRatePar(nw);
  for (i = FirstRatePar(nw); i<LastRatePar(nw); i++)
      if ((nw->composite.children[i])->core.being_destroyed)
          n--;
  fprintf(fp,"%s\t%d\n", DELAYS, n);

  n = LastMarkPar(nw)-FirstMarkPar(nw);
  for (i = FirstMarkPar(nw); i<LastMarkPar(nw); i++)
      if ((nw->composite.children[i])->core.being_destroyed)
          n--;
  fprintf(fp,"%s\t%d\n", MARKINGS, n);

  n = LastResult(nw)-FirstResult(nw);
  for (i = FirstResult(nw); i<LastResult(nw); i++)
      if ((nw->composite.children[i])->core.being_destroyed)
          n--;
  fprintf(fp,"%s\t%d\n\n", RESULTS, n);
 
  {                          /*PRINT LIST OF MPAR */
    Cardinal mparvalue;
    fprintf(fp,"-- %s %s:\n\n",
            "LIST OF MARKING PARAMETERS",
            "(NAME, VALUE, (X,Y)-POSITION)");
    for (i=FirstMarkPar(nw);i<LastMarkPar(nw);i++)
	{
      String name;
	  if ((nw->composite.children[i])->core.being_destroyed)
          continue;

      XtVaGetValues(nw->composite.children[i],
                    XtNxpos,&x,
                    XtNypos,&y,
                    XtNmparName,&name,
                    XtNmparValue,&mparvalue,
                    NULL);
      fprintf(fp,"%s %s %d %.2f %.2f\n",
              MARKPAR,name,mparvalue,PixToInch(x),PixToInch(y));
    }
    if (LastMarkPar(nw)>FirstMarkPar(nw))
      fprintf(fp,"\n");
  }
  {                         /* PRINT LIST OF PLACES */
    InitialMarking *M0;
    
    fprintf(fp,"-- %s %s:\n\n",
            "LIST OF PLACES",
            "(NAME, MARKING, (X,Y)-POSITION (PLACE & TAG))");

    for (i=FirstPlace(nw); i<LastPlace(nw);i++)
	{
      String name;
	  
	  if ((nw->composite.children[i])->core.being_destroyed)
          continue;

      XtVaGetValues(nw->composite.children[i],
                    XtNxpos,&x,
                    XtNypos,&y,
                    XtNtagName,&name,
                    XtNtagXPos,&tag_x,
                    XtNtagYPos,&tag_y,
                    XtNinitialMarking,&M0,
                    NULL);
      if (M0->type == MARKINGVALUE) 
        fprintf(fp,"%s %s %d %.2f %.2f %.2f %.2f\n",
                /* key-word */          PLACE,
                /* tag */                 name,
                /* M0 */                M0->values.markingValue,
                /* x coord in inch */        PixToInch(x),
                /* y coord in inch */        PixToInch(y),
                /* absolute coordinates for tag-position because of downward */
                /* compatibility                                             */
                /* tag coords */        PixToInch(tag_x+x), 
                /* y tag coord */        PixToInch(tag_y+y));
      else 
        for (j=FirstMarkPar(nw); j<LastMarkPar(nw);j++) {
          String mparName=(String)NULL;
          
          if((Widget)M0->values.markingParam == 
             (Widget)nw->composite.children[j]) 
            {
              XtVaGetValues((Widget)nw->composite.children[j],
                            XtNmparName, &mparName,
                            NULL);
              fprintf(fp,"%s %s %s %.2f %.2f %.2f %.2f\n",
                      /* key-word */          PLACE,
                      /* tag */               name,
                      /* markingPar. name */  mparName,
                      /* x coord in inch */   PixToInch(x),
                      /* y coord in inch */   PixToInch(y),
                      /* absolute coordinates for tag-position because of */
                      /* downward compatibility                           */
                      /* tag coords */              PixToInch(tag_x+x),
                      /* y tag coord */              PixToInch(tag_y+y));
              break;
            }
        }
    }
    if (LastPlace(nw)>FirstPlace(nw))
      fprintf(fp,"\n");
  }
  {                       /* PRINT LIST OF RATEPAR */
    RateValue *rateparval;
    
    fprintf(fp,"-- %s %s:\n\n", 
            "LIST OF DELAY PARAMETERS",
            "(NAME, VALUE, (X,Y)-POSITION)");

    for (i=FirstRatePar(nw); i<LastRatePar(nw);i++)
	{
      String name;
	  if ((nw->composite.children[i])->core.being_destroyed)
          continue;

      XtVaGetValues(nw->composite.children[i],
                    XtNxpos,&x,
                    XtNypos,&y,
                    XtNrateParName,&name,
                    XtNrateParValue,&rateparval,
                    NULL);
      if (rateparval->valueType == Ratevalue) 
        fprintf(fp,"%s %s %1.15e %.2f %.2f\n",
                /* key-word */        DELAYPARAM,
                /* tag */             name,
                /* delay */           rateparval->values.value,
                /* x coord in inch */ PixToInch(x),
                /* y coord in inch */ PixToInch(y));
      else
        fprintf(fp,"%s %s %s %.2f %.2f\n",
                /* key-word */        DELAYPARAM,
                /* tag */             name,
                /* MD */              MD,
                /* x coord in inch */ PixToInch(x),
                /* y coord in inch */ PixToInch(y));
    }
    if (LastRatePar(nw)>FirstRatePar(nw))
      fprintf(fp,"\n");
  }
  {                  /* PRINT LIST OF TRANSITIONS AND ARCS */
    int ratePos_x,ratePos_y;
    TransType transtype;
    TransOrientation orientation;
    EnablingDependence enablingDependence;
    Cardinal priority;
    Rate *rate; String name;

    /*new*/
    Cardinal phase;
    int group;
    double group_weight;
    RacePolicy racepolicy;

    fprintf(fp,"-- %s\n-- \t%s\n-- \t%s\n-- \t%s\n",
            "LIST OF TRANSITIONS",
            "(NAME, DELAY, ENABLING DEPENDENCE, KIND, FIRING POLICY, PRIORITY,",
            "ORIENTATION, PHASE, GROUP, GROUP_WEIGHT,",
	    "(X,Y)-POSITION (TRANSITION, TAG & DELAY), ARCS)");
    
    for (i=FirstTrans(nw);i<LastTrans(nw);i++) {
      if ((nw->composite.children[i])->core.being_destroyed)
          continue;

      XtVaGetValues(nw->composite.children[i],
                    XtNxpos,&x,
                    XtNypos,&y,
                    XtNtagName,&name,
                    XtNtagXPos,&tag_x,
                    XtNtagYPos,&tag_y,
                    XtNtransType,&transtype,
                    XtNtransOrient,&orientation,
                    XtNpriority,&priority,
                    XtNenablingDependence,&enablingDependence,
                    XtNrates,&rate,
                    XtNratePos_x,&ratePos_x,
                    XtNratePos_y,&ratePos_y,
		    /*new*/
		    XtNphase, &phase,       
		    XtNgroup, &group,       
		    XtNgroup_weight, &group_weight,
		    XtNracePolicy, &racepolicy,
		    NULL);

      if ((transtype != IMMEDIATE)&&(priority == 1))
	priority = 0;

      switch (rate->type) {
      case RATEVALUE:
        fprintf(fp,
                "\n%s %s %1.15e %s %s %s %d %d %d %d %f %.2f %.2f %.2f %.2f %.2f %.2f\n",
                /* key-word */          TRANSITION,
                /*tag*/                        name,
                /*rate value*/                rate->values.ratevalue,
                /*enabling dependance*/        EnableDepStr[enablingDependence],
                /*kind*/                TransTypeStr[transtype],
		/*firing policy*/       FiringPolicyStr[racepolicy],
                /*pri */                priority,
                /*orientation*/         orientation,
		/*PHASE*/               phase,
		/*GROUP*/               group,
		/*GROUP_WEIGHT*/        group_weight,
                /* x coord */                PixToInch(x),
                /* y coord */                PixToInch(y),
                /* x tagpos */                PixToInch(tag_x+x),
                /* y tagpos */                PixToInch(tag_y+y),
                /* x ratepos */                PixToInch(ratePos_x),
                /* y ratepos */                PixToInch(ratePos_y));
        break;
      case RATEPARAM:
        for (j=FirstRatePar(nw); j<LastRatePar(nw);j++)  {
          String rparName=(String)NULL;
          
          if ((Widget)rate->values.rateparam == 
              (Widget)nw->composite.children[j]) {
            XtVaGetValues((Widget)nw->composite.children[j],
                          XtNrateParName,&rparName,
                          NULL);
            fprintf(fp,
                    "\n%s %s %s %s %s %s %d %d %d %d %f %.2f %.2f %.2f %.2f %.2f %.2f\n",
                    /* key-word */          TRANSITION,
                    /*tag*/                    name,
                    /*rate par. name*/            rparName,
                    /*enabling dependance*/ EnableDepStr[enablingDependence],
                    /*kind*/                    TransTypeStr[transtype],
		    /*firing policy*/       FiringPolicyStr[racepolicy],
                    /* pri */                    priority,
                    /*orientation*/            orientation,
		    /*PHASE*/               phase,
		    /*GROUP*/               group,
		    /*GROUP_WEIGHT*/        group_weight,
                    /* x coord */            PixToInch(x),
                    /* y coord */            PixToInch(y),
                    /* x tagpos */            PixToInch(tag_x+x),
                    /* y tagpos */            PixToInch(tag_y+y),
                    /* x ratepos */            PixToInch(ratePos_x),
                    /* y ratepos */            PixToInch(ratePos_y));
            break;
          }
        }
        break;
      case MD_DEF:
      case PMF_FUNC:
      case DATAFILE:
        fprintf(fp,
                "\n%s %s %s %s %s %s %d %d %d %d %f %.2f %.2f %.2f %.2f %.2f %.2f\n",
                /* key-word */              TRANSITION,
                /*tag*/                            name,
                /*marking dependent */      MD,
                /*dependance*/              EnableDepStr[enablingDependence],
                /*kind*/                    TransTypeStr[transtype],
	       	/*firing policy*/           FiringPolicyStr[racepolicy],
                /* pri */                    priority,
                /*orientation*/                orientation,
		/*PHASE*/               phase,
		/*GROUP*/               group,
		/*GROUP_WEIGHT*/        group_weight,	
                /* x coord */                  PixToInch(x),
                /* y coord */                  PixToInch(y),
                /* x tagpos */                 PixToInch(tag_x+x),
                /* y tagpos */                 PixToInch(tag_y+y),
                /* x ratepos */                PixToInch(ratePos_x),
                /* y ratepos */                    PixToInch(ratePos_y));
		break;
      default: ;
      }
      {
        ArcType at;

        for (at=INPUTARC;at<=INHIBITORARC;at++) {
          fprintf(fp,"%s %d\n",
                  ArcTypeStr[at],getArcsNbr((Widget)nw,nw->composite.children[i],at));
          mdarcs=printArcs((Widget)nw,nw->composite.children[i],fp,at,mdarctab,mdarcs);

        }
      }
    }
  }

  /* print list of delay- or weight-parameter definitions */
  fprintf(fp,"-- DEFINITION OF PARAMETERS:\n\n");
  printRateDEF(fp,nw);

  /* print list of marking dependent firing delays for exp. transitions */
  fprintf(fp,"-- MARKING DEPENDENT FIRING DELAYS FOR EXP. TRANSITIONS:\n\n");
  printMD(fp,nw,EXPONENTIEL,EXP_DELAY);

  /* print list of marking dependent firing delays for det. transitions */
  fprintf(fp,"-- MARKING DEPENDENT FIRING DELAYS FOR DET. TRANSITIONS:\n\n");
  printMD(fp,nw,DETERMINISTIC,DET_DELAY);

  /* print list of marking dependent firing delays for det. transitions */
  fprintf(fp,"-- PROBABILITY MASS FUNCTION DEFINITIONS FOR GEN. TRANSITIONS:\n\n");
  printMD(fp,nw,GENERAL,GEN_DELAY);

  /* print list of marking dependent weights for immediate transitions */
  fprintf(fp,"-- MARKING DEPENDENT WEIGHTS FOR IMMEDIATE TRANSITIONS:\n\n");
  printMD(fp,nw,IMMEDIATE,WEIGHT);

  /* print list of enabling functions for immediate transitions */
  fprintf(fp,"-- ENABLING FUNCTIONS FOR IMMEDIATE TRANSITIONS:\n\n");
  printEnabling(fp,nw,"FUNCTION");
  
  /* print marking dependent arc cardinalities */
  fprintf(fp,"-- MARKING DEPENDENT ARC CARDINALITIES:\n\n");
  printArcMD(fp,nw,mdarctab,mdarcs);

  {/* print results */
    String rew_name,rew_def;
      
    fprintf(fp,"-- REWARD MEASURES:\n\n");
    for (i=FirstResult(nw);i<LastResult(nw);i++) {
      XtVaGetValues(nw->composite.children[i],
                    XtNresultName,&rew_name,
                    XtNresultDefinition,&rew_def,
                    NULL);
      fprintf(fp,"%s %s\n%s\n\n",MEASURE,rew_name,rew_def);
    }
    if (LastResult(nw)>FirstResult(nw))
      fprintf(fp,"\n");
  }

  fprintf(fp,"\n-- END OF SPECIFICATION FILE\n");

  free(mdarctab);
}


/*******************************************************************************
Name:
        Save
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
triggert bei event:
        globale Aktion, ohne Registrierung
--------------------------------------------------------------------------------
Zweck:
        Sichern eines Modells
*******************************************************************************/
void Save(w,event,params,numparams)
    Widget w;
    XEvent *event;
    String *params;
    Cardinal *numparams;
{
  NetWidget nw = (NetWidget)w;
  FILE *fp;
  char msg[200], npath[512]; char *modelpath;
  
#ifdef debug
  printf("Action Save \n");
#endif
  
  if (*numparams < 1) {
    XtWarning("save net: no netname found");
    return;
  }

  if ((modelpath = getenv(MODELDIR_ENV)) == NULL) {
    sprintf(msg,
      "Environment variable %s not set; can't open model description",
	  MODELDIR_ENV);
    XtWarning(msg);
    return;
  }
  sprintf(npath,"%s/%s%s",modelpath,params[0],NET_EXTENSION);
  if ((fp = fopen(npath,"w")) == NULL) {
    sprintf(msg,"save net: can't open %s for writing ",npath);
    XtWarning(msg);
    return;
  }

  WriteNet(nw,fp);
  
  fclose(fp);
/* fclose(pmffp);*/

  SetModifiedFlag(nw,False);
  SetNetModifiedFlag(nw,False);
  SetCollectResultFlag(nw,False);
}


/*******************************************************************************
Name:
        getArcsNbr
--------------------------------------------------------------------------------
Funktionstyp:
        private Funktion
-------------------------------------------------------------------------------
Paramter:
        w: Instanz der Klasse NetWidget
        trans: Instanz der Klasse TransObj
        type: gesuchter Kantentyp
--------------------------------------------------------------------------------
Zweck:
        Ermittelt die Anzahl der Kanten eines Types
*******************************************************************************/

static int getArcsNbr(w,trans,type)
     Widget w,trans;
     ArcType type;
{
  ArcObject arc;
  NetWidget nw = (NetWidget)w;
  Widget fromObj,toObj;
  ArcType arctype;
  int i,nbrarc;
  
  nbrarc = 0;
  for (i=FirstArc(nw);i<LastArc(nw);i++) {
    arc=(ArcObject)nw->composite.children[i];
    XtVaGetValues((Widget)arc,
                  XtNfromObj,&fromObj,
                  XtNtoObj,&toObj,
                  XtNarcType,&arctype,
                  NULL);
    switch (type) {
    case OUTPUTARC:
      if (arctype == OUTPUTARC && (fromObj == trans || toObj == trans))
        nbrarc++;
      break;
    case INPUTARC:
      if (arctype == INPUTARC  && (fromObj == trans || toObj == trans))
        nbrarc++;
      break;
    case INHIBITORARC:
      if (arctype == INHIBITORARC && (fromObj == trans || toObj == trans))
        nbrarc++;
      break;
    default: ;
    }
  }
  return nbrarc;
}

/*******************************************************************************
Name:
        printArcs
--------------------------------------------------------------------------------
Funktionstyp:
        private Funktion
--------------------------------------------------------------------------------
Parameter
        w: Instanz der Klasse NetWidget
        trans: Instanz der Klasse TransObj
        FD:Dateideskriptor
        type: Kantentyp der abgespeichert wird
        tab: Tabelle mit Nummern von Kanten mit markierungsanhaengigen Werten
        num: momentane Anzahl der Kanten mit markierungsanhaengigen Werten
--------------------------------------------------------------------------------
Rueckgabewert:
        neue Anzahl der Kanten mit markierungsanhaengigen Werten
--------------------------------------------------------------------------------
Zweck:
        Schreiben der zur Transition "trans" gehoerigen Kanten vom
        Typ "type".
        Falls eine Kante einen markierungsabhaengigen Wert hat, 
        wird Ihre Nummer in "tab" gespeichert und die Anzahl "num"
        inkrementiert, so dass diese Kanten spaeter in der selben
        Reihenfolge wie hier ausgegeben werden koennen.   
*******************************************************************************/

static int printArcs(w,trans,FD,type,tab,num)
     Widget w,trans;
     FILE* FD;
     ArcType type;
     int tab[];
     int num;
{
  ArcObject arc;
  NetWidget nw = (NetWidget)w;
  int i,j,x,y;
  ArcType arctype;
  Multiplicity *mult;
  DrawingDirection drawing_direction;
  Cardinal num_children;
  Widget fromObj,toObj;
  WidgetList ipoints;
  String placeName;

  for(i=FirstArc(nw);i<LastArc(nw);i++) {
    arc = (ArcObject)nw ->composite.children[i];
    XtVaGetValues((Widget)arc,
                  XtNfromObj,&fromObj,
                  XtNtoObj,&toObj,
                  XtNarcType,&arctype,
                  XtNmultiplicity,&mult,
                  NULL);
    if (arctype!=type)
      continue;

    if (toObj==trans)
      XtVaGetValues(fromObj,
                    XtNtagName,&placeName,
                    NULL);
    else 
      if (fromObj==trans)
        XtVaGetValues(toObj,
                      XtNtagName,&placeName,
                      NULL);
      else
        continue;

    XtVaGetValues((Widget)arc,
                  XtNnumChildren,&num_children,
                  XtNchildren,&ipoints,
                  XtNdrawingDirection,&drawing_direction,
                  NULL);

    switch(mult->type) {
      case ARCVALUE:
        fprintf(FD,"%d %s %d\n",
                /* multiplicity */              mult->values.arcvalue, 
                /* connected to place */ placeName,
                /* nbr of IP */             num_children);
        break;
      case MD_DEF:
        fprintf(FD,"%s %s %d\n",
                /* multiplicity */              MD, 
                /* connected to place */ placeName,
                /* nbr of IP */             num_children);
	tab[num]=i;
        num++;        
        break;
    }

    for (j=0;j<num_children;j++) {
      if (drawing_direction == UP)
        XtVaGetValues(ipoints[j],
                      XtNxpos,&x,
                      XtNypos,&y,
                      NULL);
      else
        XtVaGetValues(ipoints[num_children-j-1],
                      XtNxpos,&x,
                      XtNypos,&y,
                      NULL);
      fprintf(FD,"%.2f %.2f \n",
              /* x coord of IP */ PixToInch(x),
              /* y coord of IP */ PixToInch(y) );
    }
  }
  return num;
}


/*******************************************************************************
Name:
        printMD
--------------------------------------------------------------------------------
Funktionstyp:
        private Funktion
--------------------------------------------------------------------------------
Parameter:
        fd: Filediskriptor
        nw: NetWidget
        tt: TransType
        key: Keyword
--------------------------------------------------------------------------------
Zweck:
        Schreiben der MD-Transitionen
*******************************************************************************/

static void printMD ( fd,nw,tt,key )
     FILE *fd;
     NetWidget nw;
     TransType tt;
	 char *key;
{
  int i;
  TransType transtype;
  Rate *rate;
  String name;
  char flag=0;

  for (i=FirstTrans(nw);i<LastTrans(nw);i++) {
    if ((nw->composite.children[i])->core.being_destroyed)
        continue;

    XtVaGetValues(nw->composite.children[i],
                  XtNtransType,&transtype,
                  XtNtagName,&name,
                  XtNrates,&rate,
                  NULL);
    if (transtype==tt && rate->type==MD_DEF) {
      fprintf(fd,"%s %s %s\n",
              key, 
              name, 
              rate->values.md_definition);
      flag=1;
    }
    if (transtype==tt && rate->type==PMF_FUNC) {
#ifdef debug
      printf("NWFileActions: %s %s %s\n",
              key, 
              name, 
              rate->values.pmf_function);
#endif
      fprintf(fd,"%s %s %s\n",
              key, 
              name, 
              rate->values.pmf_function);
      flag=1;
    }
    if (transtype==tt && rate->type==DATAFILE) {
#ifdef debug
      printf("NWFileActions: %s %s %s\n",
              key, 
              name, 
              rate->values.DataFile);
#endif
      fprintf(fd,"%s %s FILE_REF(%s);\n",
              key, 
              name, 
              rate->values.DataFile);
      flag=1;
    }
  }
  if (flag)
    fprintf(fd,"\n");
}

static void printEnabling ( fd,nw,key )
     FILE *fd;
     NetWidget nw;
	 char *key;
{
  int i;
  TransType transtype;
  Rate *rate;
  String name;
  char flag=0;

  for (i=FirstTrans(nw);i<LastTrans(nw);i++) {
    if ((nw->composite.children[i])->core.being_destroyed)
        continue;

    XtVaGetValues(nw->composite.children[i],
                  XtNtransType,&transtype,
                  XtNtagName,&name,
                  XtNrates,&rate,
                  NULL);
    if (rate->enable_fkt)
    {
	  fprintf(fd,"%s %s %s\n",key, name, rate->enable_fkt);
	  flag=1;
	}
  }
  if (flag)
    fprintf(fd,"\n");
}

/*******************************************************************************
Name:
        printArcMD
--------------------------------------------------------------------------------
Funktionstyp:
        private Funktion
-------------------------------------------------------------------------------
Paramter:
        fd: Filediskriptor
        w: Instanz der Klasse NetWidget
        tab: Tabelle mit Nummern von Kanten mit markierungsanhaengigen Werten
        num: Anzahl der Kanten mit markierungsanhaengigen Werten
--------------------------------------------------------------------------------
Zweck:
       Schreiben aller Kanten mit markierungsanhaengigen Werten
       in der Reihenfolge der zugehoerigen Transitionen

*******************************************************************************/

static void printArcMD(fd,w,tab,num)
     FILE *fd;
     Widget w;
     int tab[];
     int num;
{
  ArcObject arc;
  NetWidget nw = (NetWidget)w;
  Widget fromObj,toObj;
  ArcType arctype;
  int i;
  String TransName,PlaceName; 
  Multiplicity* mult;
  char flag=0;
  
 
  for (i=0;i<num;i++) {
    arc=(ArcObject)nw->composite.children[tab[i]];
    XtVaGetValues((Widget)arc,
                  XtNfromObj,&fromObj,
                  XtNtoObj,&toObj,
                  XtNarcType,&arctype,
                  XtNmultiplicity,&mult,
                  NULL);

    if (mult->type==MD_DEF) {
      if (isTransObject(fromObj)) {
         XtVaGetValues(toObj,
                       XtNtagName,&PlaceName,
                       NULL);
         XtVaGetValues(fromObj,
                       XtNtagName,&TransName,
                       NULL);
      }
      else {
         XtVaGetValues(fromObj,
                       XtNtagName,&PlaceName,
                       NULL);
         XtVaGetValues(toObj,
                       XtNtagName,&TransName,
                       NULL);
      }

      fprintf(fd,"%s %s %s %s %s\n",
              ARC,ArcTypeStrDEF[arctype],TransName,PlaceName,
              mult->values.md_definition);
      flag=1;
    }
  }
  if (flag)
    fprintf(fd,"\n");
}

/******************************************************************************
Name:
        printRateDEF
-------------------------------------------------------------------------------
Funktionstyp:
        private Funktion
-------------------------------------------------------------------------------
Paramter:
        fd: Filediskriptor
        nw: Instanz der Klasse NetWidget
-------------------------------------------------------------------------------
Zweck:
        Schreibt die delay/weight-Definitionen in <net>.TN
******************************************************************************/
#ifdef __STDC__
static int printRateDEF(FILE *fd, NetWidget nw)
#else
static int printRateDEF(fd, nw)
     FILE *fd;
     NetWidget nw;
#endif
{                       /* PRINT LIST OF RATEPAR_DEFs */
    int i, x, y;
    String name;
    RateValue *rateparval;
    
    for (i=FirstRatePar(nw); i<LastRatePar(nw);i++) {
        if ((nw->composite.children[i])->core.being_destroyed)
            continue;

        XtVaGetValues(nw->composite.children[i],
                      XtNxpos,&x,
                      XtNypos,&y,
                      XtNrateParName,&name,
                      XtNrateParValue,&rateparval,
                      NULL);
        if (rateparval->valueType == Definition) {
            fprintf(fd,"%s %s %s\n", 
                    /* key-word */ DELAYPAR_DEF,
                    /* tag */      name,
                    /* MD */       rateparval->values.definition);
        }
    }
    if (LastRatePar(nw)>FirstRatePar(nw))
      fprintf(fd,"\n");
    return 0;
}

/*******************************************************************************
Name:
        error_handler
--------------------------------------------------------------------------------
Funktionstyp:
        handler
--------------------------------------------------------------------------------
aufgerufen von:
        globale load_net()-Routine, im Fehlerfall
*******************************************************************************/
void error_handler(str,fatal)
char *str;
int fatal;
{
  char buf[256];
  
  sprintf(buf,"%s during load of net: %s!",fatal?"Error":"Warning",str);
  XtWarning(buf);
}

/*******************************************************************************
Name:
        Load
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
triggert bei event:
        globale Aktion, ohne Registrierung
--------------------------------------------------------------------------------
Zweck:
        Laden eines Modelles
*******************************************************************************/

void Load(w,event,params,numparams)
     Widget w;
     XEvent *event;
     String *params;
     Cardinal *numparams;
{
  NetWidget nw = (NetWidget)w;
  int i,j;
  NetDescr *net;
    
  if (*numparams < 1) {
    XtWarning("Load: no netname found");
    return;
  }
  
  net=load_net(params[0],error_handler);
  /*noDataFileTransition=0;  */

  /* Create MARKPARs *****************************************************/
  for(i=0;i<net->num_markpars;i++)
  {
     Markpar *m=&net->markpar[i];                         
     XtVaCreateWidget(m->name,markParObjClass,w,
                      XtNxpos,InchToPix(m->pos.x),
                      XtNypos,InchToPix(m->pos.y),
                      XtNmparName,XtNewString(m->name),
                      XtNmparValue,m->value,
                      NULL);
  }
  
  /*  Create PLACEs ******************************************************/
  for(i=0;i<net->num_places;i++) 
  {                         
      Place *p=&net->place[i];
	  InitialMarking M0;
      M0.type=p->is_value ? MARKINGVALUE : MARKINGPARAM;
	  if(p->is_value)
	    M0.values.markingValue = p->marking.value;
	  else
       for (j=FirstMarkPar(nw);j<LastMarkPar(nw);j++)
	   {
          String mparname;
		  XtVaGetValues(nw->composite.children[j],
                        XtNmparName,&mparname,
                        NULL);
          if (strcmp(mparname,p->marking.markpar->name) == 0)
		  {
            M0.values.markingParam = (XtPointer)nw->composite.children[j];
            break;
          }
	   }
 	    
	  XtVaCreateWidget(p->name,placeObjClass,w,
                       XtNxpos,InchToPix(p->pos.x),
                       XtNypos,InchToPix(p->pos.y),
                       XtNtagName,XtNewString(p->name),
                       XtNtagXPos,InchToPix(p->name_pos.x)-InchToPix(p->pos.x),
                       XtNtagYPos,InchToPix(p->name_pos.y)-InchToPix(p->pos.y),
                       XtNinitialMarking,&M0,
                       NULL);
  }
  
  /* Create DELAYPARs ****************************************************/  
  for(i=0;i<net->num_delaypars;i++)
  {                         
    Delaypar *d=&net->delaypar[i];
    RateValue rparValue;

	rparValue.valueType = d->is_value ? Ratevalue : Definition;
	
    if (!d->is_value) 
      rparValue.values.definition = XtNewString(d->delay.md_delay);
    else
      rparValue.values.value = d->delay.value;

    XtVaCreateWidget(d->name,rateParObjClass,w,
                     XtNxpos,InchToPix(d->pos.x),
                     XtNypos,InchToPix(d->pos.y),
                     XtNrateParName,d->name,
                     XtNrateParValue,&rparValue,
                     NULL);
  }
  
  /* Create TRANSITIONs & ARCs *******************************************/
  for(i=0;i<net->num_trans;i++)
  {
    Transition *t=&net->trans[i];
    Rate rate;
    TransitionType transKind;
    Widget trans;
    RacePolicy racePolicy;

	rate.enable_fkt=t->type == IMM ? XtNewString(t->md_enable) : NULL;
	
    transKind =
	  t->type == IMM ? IMMEDIATE :
	  t->type == EXP ? EXPONENTIEL :
	  t->type == DET ? DETERMINISTIC : GENERAL;

    racePolicy = 
	  t->f_policy == RS ? RACE_S :
	  t->f_policy == RE ? RACE_E : RACE_A;

    switch(t->delay_type)
	{
	   case DELAY:
	     rate.type = RATEVALUE;
		 rate.values.ratevalue = t->delay.value;
		 break;
	   case DELAYPAR:
         for (j=FirstRatePar(nw);j<LastRatePar(nw);j++)
		 {
           String rateParName;
           XtVaGetValues(nw->composite.children[j],
                          XtNrateParName,&rateParName,
                          NULL);
           if (!strcmp(rateParName,t->delay.delay_par->name))
		   {
             rate.type = RATEPARAM;
             rate.values.rateparam = (XtPointer)nw->composite.children[j];
             break;
           }
         }
		 break;
	   case MARKDEP:
         rate.type = MD_DEF;
         rate.values.md_definition = XtNewString(t->delay.md_delay);
		 break;
	   case PMFDEF:
           { 
             char* FileOrFunc;
             FileOrFunc = XtNewString(t->delay.md_delay);
#ifdef debug
             printf("NWFileActions.c:  FileOrFunc=%s\n",  FileOrFunc);
#endif
             if(!strstr(FileOrFunc,"DAT"))
             {
	      rate.type = PMF_FUNC;
              rate.values.pmf_function = XtNewString(FileOrFunc);
#ifdef debug
              printf("NWFileActions: type=%d\tpmf=%s\n", 
                	rate.type,
                        rate.values.pmf_function);
#endif
             }
             else
             {
	      rate.type = DATAFILE;
              FileOrFunc+=9;                   /* "FILE_REF(" weg */
              rate.values.DataFile = XtNewString(FileOrFunc);
              rate.values.DataFile[(unsigned)strlen(rate.values.DataFile)-2]='\0';
#ifdef debug
              printf("NWFileActions: type=%d\tDataFile=%s\n", 
                	rate.type,
                        rate.values.DataFile);
#endif
             }              
           }             
           break;
	}
      
    trans = XtVaCreateWidget(t->name,transObjClass,w,
                             XtNxpos,InchToPix(t->pos.x),
                             XtNypos,InchToPix(t->pos.y),
                             XtNtagName,XtNewString(t->name),
                             XtNtagXPos,InchToPix(t->name_pos.x)-InchToPix(t->pos.x),
                             XtNtagYPos,InchToPix(t->name_pos.y)-InchToPix(t->pos.y),
                             XtNtransType,transKind,
                             XtNtransOrient,t->orient,
                             XtNenablingDependence,t->inf_server ? INFINITE_SERVER : SINGLE_SERVER,
                             XtNpriority,(Cardinal)t->priority,
                             XtNratePos_x,InchToPix(t->delay_pos.x),
                             XtNratePos_y,InchToPix(t->delay_pos.y),
                             XtNrates,&rate,
			     /*new*/
			     /*XtNphase,(Cardinal)t->phases,*/
			     /*XtNgroup,t->group,*/         
			     /*	     XtNgroup_weight,(float)t->group_weight, */ 
			     XtNracePolicy, racePolicy,
                             NULL);
    XtVaSetValues(trans,XtNgroup_weight,&t->group_weight,
		        XtNphase,(Cardinal)t->phases,
		        XtNgroup,(int)t->group,
		        NULL);
    
    CreateArcs(w,trans,nw,INPUTARC,t->in_arc,t->num_inarcs);
    CreateArcs(w,trans,nw,OUTPUTARC,t->out_arc,t->num_outarcs);
    CreateArcs(w,trans,nw,INHIBITORARC,t->inh_arc,t->num_inharcs);
    

  }
  /*if (noDataFileTransition >0) onlySimulation(1);
  else onlySimulation(0);*/
  
  /* Create Reward-MEASUREs ********************************************/
  for(i=0;i<net->num_rewards;i++)   
  {
    XtVaCreateWidget(net->reward[i].name,resultObjectClass,w,
                 XtNresultName,XtNewString(net->reward[i].name),
                 XtNresultDefinition,XtNewString(net->reward[i].def),
                 NULL);        
  }
  
  free_net(net);
  DrawNet(w,NORMAL);
  Redisplay(w,NULL);
  SetModifiedFlag(nw,False);
  SetNetModifiedFlag(nw,False);
  
}

/*******************************************************************************
Name:
        CreateArcs
--------------------------------------------------------------------------------
Funktionstyp:
        private Funktion
--------------------------------------------------------------------------------
Parameter:
        w,obj: NetWidget
           nw: NetWidget
           at: ArcType
--------------------------------------------------------------------------------
Zweck:
        Wird zum Erzeugen der 3 Sorten Arcs aus Load aufgerufen
*******************************************************************************/
static int CreateArcs (w,trans,nw,at,arc,num_arcs)
     Widget w,trans;
     NetWidget nw;
     ArcType at;
	 Arc *arc;
     int num_arcs;
{
  int p,j,k;
  static int glob_numarcs=0;

  for (j=0;j<num_arcs;j++)
  {
    Multiplicity multiplicity;
    Widget w_arc; String name;
    char arcname[128];
	Arc *a = &arc[j];
	
	
	multiplicity.type = a->is_value ? ARCVALUE : MD_DEF;
    if(a->is_value)
	  multiplicity.values.arcvalue = a->multipl.value;
    else 
      multiplicity.values.md_definition = XtNewString(a->multipl.md_mult);
    
    sprintf(arcname,"A%d",glob_numarcs++);
    for (p=FirstPlace(nw);p<LastPlace(nw);p++)
	{
      XtVaGetValues(nw->composite.children[p],
                    XtNtagName,&name,
                    NULL);
      if (strcmp(a->place->name,name) == 0) 
	  {
        w_arc = XtVaCreateWidget(XtNewString(arcname),arcObjClass,w,
                               XtNmultiplicity,&multiplicity,
                               XtNarcType,at,
                               XtNtoObj,nw->composite.children[p],
                               XtNfromObj,trans,
                               NULL);
        for (k=0;k<a->num_pts;k++)
          createImPoint(w_arc,InchToPix(a->point[k].x),InchToPix(a->point[k].y),k);
		break; /* Place found */
      }
    }
  }
  return 0;
}
