/*******************************************************************************
Modul   	:	NWMotionActions.c

Purpose		: 	Bewegungsaktionen (Reaktion auf Ereignis <Motion>)

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/


/*
 * NetWidget.c - Manages all objects of a PN.

   Class Hierarchie:
   Core -> Composite -> NetWidget
 */

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#ifdef  __hpux
double copysign(x,y) /* Returns x with y's sign */
  double x,y;
{
  return x<0?(y<0?x:-x):(y<0?-x:x);
}
#endif

#include "BaseObj.h"
#include "MarkParObj.h"
#include "RateParObj.h"
#include "PlaceObj.h"
#include "TransObj.h"
#include "ArcObj.h"

#include "NetWidgetP.h"

#ifdef __STDC__
extern void drawLine(NetWidget nw, int x1, int y1, int x2, int y2);
extern void setSolidLine(NetWidget nw);
extern void setDashedLine(NetWidget nw);
extern void setDottedLine(NetWidget nw);
extern Widget getCurrentNet(void);
#else
extern void drawLine();
extern void setSolidLine();
extern void setDashedLine();
extern void setDottedLine();
extern Widget getCurrentNet();
#endif

#ifdef __STDC__
static void rotatePoint(double *x, double *y,
		 double cx, double cy, double dphi);
#else
static void rotatePoint();
#endif

static void ArcMotion();
static void NodeMotion();
static void ParamMotion();
static void RateMotion();
static void TagMotion();
static void LineMotion();
static void SelectionMotion();
static void SelectionRotation();

void MotionRSelect(w,event,params,numparams)
Widget w;
XMotionEvent* event;
String* params;
Cardinal* numparams;
{
    NetWidget nw = (NetWidget)getCurrentNet();
    register int x1,x2,y1,y2;

    if (!nw->netWidget.selectionBorder ||
	nw->netWidget.selectionBorder->numVertexes < 1)
      return;

    if ((int)nw->netWidget.selectionBorder->vertexes[0].x <
	nw->netWidget.prev_x){
	x1 = (int)nw->netWidget.selectionBorder->vertexes[0].x;
	x2 = nw->netWidget.prev_x;
    }
    else {
	x1 = nw->netWidget.prev_x;
	x2 = (int)nw->netWidget.selectionBorder->vertexes[0].x;
    }
    if ((int)nw->netWidget.selectionBorder->vertexes[0].y <
	nw->netWidget.prev_y){
	y1 = (int)nw->netWidget.selectionBorder->vertexes[0].y;
	y2 = nw->netWidget.prev_y;
    }
    else {
	y1 = nw->netWidget.prev_y;
	y2 = (int)nw->netWidget.selectionBorder->vertexes[0].y;
    }

    /* undraw old */
    setDashedLine(nw);
    drawRectangle (nw, x1, y1, x2-x1, y2-y1);
    
    nw->netWidget.prev_x = event->x;
    nw->netWidget.prev_y = event->y;

    if ((int)nw->netWidget.selectionBorder->vertexes[0].x <
	nw->netWidget.prev_x){
	x1 = (int)nw->netWidget.selectionBorder->vertexes[0].x;
	x2 = nw->netWidget.prev_x;
    }
    else {
	x1 = nw->netWidget.prev_x;
	x2 = (int)nw->netWidget.selectionBorder->vertexes[0].x;
    }
    if ((int)nw->netWidget.selectionBorder->vertexes[0].y <
	nw->netWidget.prev_y){
	y1 = (int)nw->netWidget.selectionBorder->vertexes[0].y;
	y2 = nw->netWidget.prev_y;
    }
    else {
	y1 = nw->netWidget.prev_y;
	y2 = (int)nw->netWidget.selectionBorder->vertexes[0].y;
    }

    drawRectangle (nw, x1, y1, x2-x1, y2-y1);
    setSolidLine(nw);

}

/*******************************************************************************
Name:
        Motion
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
triggert bei event:
        <Motion>: motion
--------------------------------------------------------------------------------
Zweck:
        Verzweigung zu Einzelaktionen
*******************************************************************************/

void Motion(w,event,params,numparams)
Widget w;
XMotionEvent* event;
String* params;
Cardinal* numparams;

{
    NetWidget nw = (NetWidget)w;
    int curr_obj;

  curr_obj = nw->netWidget.object;

  if (nw->netWidget.allowMotionAction) {
	if (isPlace(curr_obj))
	    NodeMotion(w,event,params,numparams);
	else
	if (isImTrans(curr_obj))
	    NodeMotion(w,event,params,numparams);
	else
	if (isExpTrans(curr_obj))
	    NodeMotion(w,event,params,numparams);
	else
	if (isDetTrans(curr_obj))
	    NodeMotion(w,event,params,numparams);
	else
	if (isGenTrans(curr_obj))
	    NodeMotion(w,event,params,numparams);
	else
	if (isArc(curr_obj))
	    ArcMotion(w,event,params,numparams);
	else
	if (isMarkPar(curr_obj))
	    ParamMotion(w,event,params,numparams);
	else
	if (isRatePar(curr_obj))
	    RateMotion(w,event,params,numparams);
	else
	if (isTag(curr_obj))
	    TagMotion(w,event,params,numparams);
	else
	if (isUncompleteSelection(curr_obj)) {
	    String xy[2]; char x_buf[64],y_buf[64];
	    Cardinal n=2;

	    /* set the start x/y in xy[] */
	    if (nw->netWidget.selectionBorder &&
		nw->netWidget.selectionBorder->vertexes &&
		nw->netWidget.selectionBorder->numVertexes) {

		sprintf(x_buf,"%i",
			nw->netWidget.selectionBorder->vertexes
		    [nw->netWidget.selectionBorder->numVertexes-1].x);
		xy[0] = x_buf;

		sprintf(y_buf,"%i",
			nw->netWidget.selectionBorder->vertexes
		    [nw->netWidget.selectionBorder->numVertexes-1].y);
		xy[1] = y_buf;
		LineMotion(w, event, xy, &n);
	    }
	    else
		XtError("Motion: can't LineMotion without start-point!!");
	}
	if (isCompleteSelection(curr_obj))
	  SelectionMotion(w, event, params, numparams);
    }
    else if (nw->netWidget.allowRotationAction) {
	if (isCompleteSelection(curr_obj))
	  SelectionRotation(w, event, params, numparams);
    }
}

/*******************************************************************************
Name:
        NodeMotion
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        Motion
--------------------------------------------------------------------------------
Zweck:
v        Bewegen von Stellen, Transitionen
*******************************************************************************/

static void NodeMotion(w,event,params,numparams)
     Widget w;
     XMotionEvent* event;
     String* params;
     Cardinal* numparams;

{
    NetWidget nw = (NetWidget)w;
    int grid_x,grid_y;

#ifdef debug
    printf("NodeMotion \n");
#endif

    AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
    redrawArcs(w,nw->netWidget.selectedObject,nw->netWidget.motion_GC,
	       nw->netWidget.prev_x, nw->netWidget.prev_y);
    Draw(nw -> netWidget.object,nw -> netWidget.selectedObject,
	 XtDisplay(nw),
	 nw->netWidget.picture,
	 nw->netWidget.motion_GC,
	 nw->netWidget.prev_x, nw->netWidget.prev_y,
	 nw->netWidget.object_size,TAG_NO|RATE_NO);
    nw->netWidget.prev_x=grid_x;
    nw->netWidget.prev_y=grid_y;
    
    XtVaSetValues(nw->netWidget.selectedObject,
		  XtNxpos, grid_x,
		  XtNypos, grid_y,
		  NULL);
    
    redrawArcs(w,nw->netWidget.selectedObject,nw->netWidget.motion_GC,
	       grid_x,grid_y);
    Draw(nw -> netWidget.object,nw -> netWidget.selectedObject,
	 XtDisplay(nw),
	 nw->netWidget.picture,
	 nw->netWidget.motion_GC,
	 grid_x,grid_y,nw->netWidget.object_size,TAG_NO|RATE_NO);
    Redisplay(w,NULL);
}

/*******************************************************************************
Name:
        TagMotion
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        Motion
--------------------------------------------------------------------------------
Zweck:
        Bewegen von Etiketten
*******************************************************************************/

static void TagMotion(w,event,params,numparams)
     Widget w;
     XMotionEvent* event;
     String* params;
     Cardinal* numparams;

{
    NetWidget nw = (NetWidget)w;
    int grid_x,grid_y, x, y;

#ifdef debug
    printf("TagMotion \n");
#endif

    AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
    DrawTag(nw->netWidget.selectedObject,
	    XtDisplay(nw),
	    nw->netWidget.picture,
	    nw->netWidget.motion_GC);

    XtVaGetValues(nw->netWidget.selectedObject,
		  XtNxpos, &x,
		  XtNypos, &y,
		  NULL);
    XtVaSetValues(nw->netWidget.selectedObject,
		  XtNtagXPos, grid_x - x,           /* relative coordinates */
		  XtNtagYPos, grid_y - y,
		  NULL);

    DrawTag(nw -> netWidget.selectedObject,
	    XtDisplay(nw),
	    nw->netWidget.picture);
    Redisplay(w,NULL);
}

/*******************************************************************************
Name:
        ParamMotion
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        Motion
--------------------------------------------------------------------------------
Zweck:
        Bewegen von Marken- und Ratenprametern
*******************************************************************************/

static void ParamMotion(w,event,params,numparams)
     Widget w;
     XMotionEvent* event;
     String* params;
     Cardinal* numparams;

{
    NetWidget nw = (NetWidget)w;
    int grid_x,grid_y;

#ifdef debug
    printf("ParamMotion \n");
#endif

    AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
    Draw(nw -> netWidget.object,nw -> netWidget.selectedObject,
	 XtDisplay(nw),
	 nw->netWidget.picture,
	 nw->netWidget.motion_GC,
	 nw->netWidget.prev_x, nw->netWidget.prev_y,
	 nw->netWidget.object_size,0);
    nw->netWidget.prev_x=grid_x;
    nw->netWidget.prev_y=grid_y;
    Draw(nw -> netWidget.object,nw -> netWidget.selectedObject,
	 XtDisplay(nw),
	 nw->netWidget.picture,
	 nw->netWidget.motion_GC,
	 grid_x,grid_y,nw->netWidget.object_size,0);
    Redisplay(w,NULL);
}

/*******************************************************************************
Name:
        RateMotion
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        Motion
--------------------------------------------------------------------------------
Zweck:
        Bewegen von Raten bei Transitionen
*******************************************************************************/

static void RateMotion(w,event,params,numparams)
     Widget w;
     XMotionEvent* event;
     String* params;
     Cardinal* numparams;

{
    NetWidget nw = (NetWidget)w;
    int grid_x,grid_y;

#ifdef debug
    printf("RateMotion \n");
#endif

    AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);

    if(isRateParObject(nw->netWidget.selectedObject)) {
	Draw(nw -> netWidget.object,nw -> netWidget.selectedObject,
	     XtDisplay(nw),
	     nw->netWidget.picture,
	     nw->netWidget.motion_GC,
	     nw->netWidget.prev_x, nw->netWidget.prev_y,
	     nw->netWidget.object_size,0);
	nw->netWidget.prev_x=grid_x;
	nw->netWidget.prev_y=grid_y;
	Draw(nw -> netWidget.object,nw -> netWidget.selectedObject,
	     XtDisplay(nw),
	     nw->netWidget.picture,
	     nw->netWidget.motion_GC,
	     grid_x,grid_y,nw->netWidget.object_size,0);
    }
    else {
	int xpos, ypos;

  	DrawRate(nw->netWidget.selectedObject,
		 XtDisplay(nw), nw->netWidget.picture, nw->netWidget.motion_GC);
	XtVaGetValues(nw->netWidget.selectedObject,
		      XtNxpos, &xpos,
		      XtNypos, &ypos,
		      NULL);
	XtVaSetValues(nw->netWidget.selectedObject,
		      XtNratePos_x, grid_x - xpos,
		      XtNratePos_y, grid_y - ypos,
		      NULL);

  	nw->netWidget.prev_x=grid_x;
  	nw->netWidget.prev_y=grid_y;
  	DrawRate(nw -> netWidget.selectedObject,
		 XtDisplay(nw), nw->netWidget.picture, nw->netWidget.motion_GC);
    }
    Redisplay(w,NULL);
}

/*******************************************************************************
Name:
        ArcMotion
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        Motion
--------------------------------------------------------------------------------
Zweck:
        Bewegen von Kanten
*******************************************************************************/

static void ArcMotion(w,event,params,numparams)
     Widget w;
     XMotionEvent* event;
     String* params;
     Cardinal* numparams;

{
    NetWidget nw = (NetWidget)w;
    Widget selObj = nw->netWidget.selectedObject;
    int grid_x,grid_y;

    switch (nw->netWidget.command) {
      case ADD: 
      case MOVE:
  	AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);

	DrawArc(selObj,
		XtDisplay(nw), nw->netWidget.picture, nw->netWidget.motion_GC,
		nw->netWidget.object_size);

        nw->netWidget.prev_x=grid_x;
	nw->netWidget.prev_y=grid_y;

	resetImPoint(selObj,grid_x,grid_y,
                     nw->netWidget.selected_intermediate_point);
	DrawArc(selObj,
		XtDisplay(nw), nw->netWidget.picture, nw->netWidget.motion_GC,
		nw->netWidget.object_size);

	break;
      default: return;
    }
    Redisplay(w,NULL);
}

#ifdef __STDC__
/*****************************************************************************/
static void LineMotion(
/*****************************************************************************/
		       Widget w,
		       XMotionEvent* event,
		       String* params,
		       Cardinal* numparams)
#else
static void LineMotion(w,event,params,numparams)
    Widget w;
    XMotionEvent* event;
    String* params;
    Cardinal* numparams;
#endif
{
	int param_x,param_y;
    NetWidget nw=(NetWidget)w;

    if (numparams==NULL || *numparams==0) {
	XtWarning("LineMotion: need 2 parameters (x,y)start-point!");
	return;
    }

    setDashedLine(nw);
	param_x=atoi(params[0]); param_y=atoi(params[1]);
    /* undraw previous line */
    /* dont do it if start-point and prev.-point are equal */
    if (param_x!=nw->netWidget.prev_x || param_y!=nw->netWidget.prev_y) {

      drawLine(nw,
	       param_x,param_y,
	       nw->netWidget.prev_x,nw->netWidget.prev_y);
    }

    /* draw new line */
    drawLine(nw,
	     param_x,param_y,
	     event->x,event->y);

    nw->netWidget.prev_x=event->x; nw->netWidget.prev_y=event->y;
    setSolidLine(nw);
    
    Redisplay(w,NULL);
} /* LineMotion */

/*****************************************************************************/
static void SelectionMotion(w,event,params,numparams)
/*****************************************************************************/
    Widget w;
    XMotionEvent* event;
    String* params;
    Cardinal* numparams;
{
    NetWidget nw=(NetWidget)w;
    double dx, dy;
      /*  dx = (double) (event->x - nw->netWidget.prev_x),
	  dy = (double) (event->y - nw->netWidget.prev_y);  */
    int i, j;
    Cardinal numIp;
    Widget toObj, fromObj, *ipList;
    Boolean toSelected, fromSelected, selected;
    int grid_x,grid_y, x,y;

    AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);

    dx = (double) (grid_x - nw->netWidget.prev_x);
    dy = (double) (grid_y - nw->netWidget.prev_y);

    if (nw->netWidget.selectionBorder &&
	nw->netWidget.selectionBorder->numVertexes) {

	/* undraw previous */
	undrawSelectedObjects(nw);
	undrawSelectionBorder(nw);

	/* compute new positions */

	/*newBorderPosition(nw, dx, dy);*/
	/* test for leaving the draw area */
	for (i=0; i < nw->netWidget.selectionBorder->numVertexes; i++) {
	    if ((nw->netWidget.selectionBorder->vertexes[i].x + dx) < 0.0)
	      dx = - nw->netWidget.selectionBorder->vertexes[i].x;
	    if ((nw->netWidget.selectionBorder->vertexes[i].y + dy) < 0.0)
	      dy = - nw->netWidget.selectionBorder->vertexes[i].y;
	}
	/* compute the new position */
	for (i=0; i < nw->netWidget.selectionBorder->numVertexes; i++) {
	    nw->netWidget.selectionBorder->vertexes[i].x += dx;
	    nw->netWidget.selectionBorder->vertexes[i].y += dy;
	}

	/*newObjectPositions(nw, dx, dy);*/
	/* for all arcs */
	for (i = FirstArc(nw); i<LastArc(nw); i++) {
	    XtVaGetValues(nw->composite.children[i],
			  XtNfromObj, &fromObj,
			  XtNtoObj, &toObj,
			  XtNnumChildren, &numIp,
			  XtNchildren, &ipList,
			  NULL);
	    XtVaGetValues(fromObj,
			  XtNselected, &fromSelected,
			  NULL);
	    XtVaGetValues(toObj,
			  XtNselected, &toSelected,
			  NULL);
	    if (fromSelected && toSelected)
	      for (j=0; j < numIp; j++) {
		  XtVaGetValues(ipList[j],
				XtNxpos,&x,
				XtNypos,&y,
				NULL);
		  XtVaSetValues(ipList[j],
				XtNxpos,x + (int)dx,
				XtNypos,y + (int)dy,
				NULL);
	      }
	}
	/* for all trans */
	for (i = FirstTrans(nw); i<LastTrans(nw); i++) {
	    XtVaGetValues(nw->composite.children[i],
			  XtNselected, &selected,
			  XtNxpos, &x,
			  XtNypos, &y,
			  NULL);
	    if (selected)
	      XtVaSetValues(nw->composite.children[i],
			    XtNxpos,x + (int)dx,
			    XtNypos,y + (int)dy,
			    NULL);
	}
	/* for all places */
	for (i = FirstPlace(nw); i < LastPlace(nw); i++) {
	    XtVaGetValues(nw->composite.children[i],
			  XtNselected, &selected,
			  XtNxpos, &x,
			  XtNypos, &y,
			  NULL);
	    if (selected)
	      XtVaSetValues(nw->composite.children[i],
			    XtNxpos,x + (int)dx,
			    XtNypos,y + (int)dy,
			    NULL);
	}

	/* draw new */
	/* don't be confused undraws DRAWS with xor */
	undrawSelectionBorder(nw);
	undrawSelectedObjects(nw);

	nw->netWidget.prev_x=grid_x; nw->netWidget.prev_y=grid_y;
    }

    Redisplay(w,NULL);

} /* SelectionMotion */

/*****************************************************************************/
static void SelectionRotation (w,event,params,numparams)
/*****************************************************************************/
    Widget w;
    XMotionEvent* event;
    String* params;
    Cardinal* numparams;
{
    NetWidget nw=(NetWidget)w;
    double cx, cy;
    double dx, dy;
    double phi, phi1, phi2, dphi, distance;
    Cardinal i;
    Boolean selected, fromSelected, toSelected;
    Widget fromObj, toObj, *ipList;
    Cardinal numIp, j;
    TransOrientation orientation;

    cx = (double)nw->netWidget.selectionCenter_x;
    cy = (double)nw->netWidget.selectionCenter_y;

    dx = (double)nw->netWidget.prev_x - cx;
    dy = (double)nw->netWidget.prev_y - cy;

    phi1 = atan2(dy, dx);

    dx = (double)event->x - cx;
    dy = (double)event->y - cy;

    phi2 = atan2(dy, dx);
    dphi = phi2 - phi1;

    /* test for leaving drawing area ----------------------------------------*/
    /* compute a phi so no vertex is out of bounds */
    for (i = 0; i < nw->netWidget.selectionBorder->numVertexes; i++) {
        dx = nw->netWidget.selectionBorder->vertexes[i].x - cx;
        dy = nw->netWidget.selectionBorder->vertexes[i].y - cy;

	distance = sqrt((dx*dx) + (dy*dy));

	phi = atan2(dy, dx);

	/* x */
	if (cx + (distance * cos(phi+dphi)) < 0.0) {
	    if (dy < 0.0)
	      dy = -1.0 * sqrt(distance*distance - (cx*cx));
	    else
	      dy = sqrt(distance*distance - (cx*cx));
	    phi2 = atan2(dy,-1.0*cx);
	    dphi = phi2 - phi;
	}
	/* y */
	if (cy + (distance * sin(phi+dphi)) < 0.0) {
	    if (dx < 0.0)
	      dx = -1.0 * sqrt(distance*distance - (cy*cy));
	    else
	      dx = sqrt(distance*distance - (cy*cy));
	    dphi = atan2(-1.0*cy,dx) - phi;
	}
    } /* for numVertexes */

    /* now check the new phi */
    for (i = 0; i < nw->netWidget.selectionBorder->numVertexes; i++) {
        dx = nw->netWidget.selectionBorder->vertexes[i].x - cx;
        dy = nw->netWidget.selectionBorder->vertexes[i].y - cy;

	distance = sqrt((dx*dx) + (dy*dy));

	phi = atan2(dy, dx);

	if ((cx + (distance * cos(phi+dphi)) < 0.0) ||
	    (cy + (distance * sin(phi+dphi)) < 0.0))
	  dphi = 0.0;
    }

    if (fabs(dphi) > (M_PI/500.0)) {

	undrawSelectionBorder(nw);
	undrawSelectedObjects(nw);

	/* now set it -------------------------------------------------------*/
	/* set in selectionBorder */
	for (i = 0; i < nw->netWidget.selectionBorder->numVertexes; i++) {

	    dx = nw->netWidget.selectionBorder->vertexes[i].x - cx;
	    dy = nw->netWidget.selectionBorder->vertexes[i].y - cy;

	    distance = sqrt((dx*dx) + (dy*dy));

	    phi = atan2(dy, dx);
	    phi += dphi;

	    nw->netWidget.selectionBorder->vertexes[i].x =
	      cx + (distance * cos(phi));
	    nw->netWidget.selectionBorder->vertexes[i].y =
	      cy + (distance * sin(phi));

	} /* for numVertexes */

	undrawSelectionBorder(nw);

	/* set in TRANSS */
	for (i = FirstTrans(nw); i<LastTrans(nw); i++) {

	    XtVaGetValues(nw->composite.children[i],
			  XtNselected, &selected,
			  XtNdblX, &dx,
			  XtNdblY, &dy,
			  XtNstartPhi, &phi1,
			  XtNstartOrient, &orientation,
			  NULL);

	    if (selected) {

		rotatePoint (&dx, &dy, cx, cy, dphi);

		phi2 = atan2((dy - cy), (dx -cx));
		phi = phi2 - phi1;

		if ( phi < 0.0)
		  phi += 2.0 * M_PI;
		
		if (abs(phi) > (M_PI/8.0)) { /* change the ORIENTATION */
		    int o;
		    phi += copysign((M_PI/8.0),phi); 
		    phi /= M_PI_4;
		    phi = floor(phi);

		    for (o=0; o<phi; o++) 
		      orientation = nextOrientation(orientation);

		} /* change the ORIENTATION */

		XtVaSetValues(nw->composite.children[i],
			      XtNxpos,(int)dx,
			      XtNypos,(int)dy,
			      XtNdblX, &dx,
			      XtNdblY, &dy,
			      XtNtransOrient, orientation,
			      NULL);
	    } /* if selected */
	}/* set in TRANSS */

	/* set in PLACES */
	for (i = FirstPlace(nw); i < LastPlace(nw); i++) {

	    XtVaGetValues(nw->composite.children[i],
			  XtNselected, &selected,
			  XtNdblX, &dx,
			  XtNdblY, &dy,
			  NULL);

	    if (selected) {

		rotatePoint (&dx, &dy, cx, cy, dphi);

		XtVaSetValues(nw->composite.children[i],
			      XtNxpos,(int)dx,
			      XtNypos,(int)dy,
			      XtNdblX, &dx,
			      XtNdblY, &dy,
			      NULL);
	    }
	}/* set in PLACES */

	/* set in ARCS */
	for (i = FirstArc(nw); i<LastArc(nw); i++) {

	    XtVaGetValues(nw->composite.children[i],
			  XtNfromObj, &fromObj,
			  XtNtoObj, &toObj,
			  XtNnumChildren, &numIp,
			  XtNchildren, &ipList,
			  NULL);
	    XtVaGetValues(fromObj,
			  XtNselected, &fromSelected,
			  NULL);
	    XtVaGetValues(toObj,
			  XtNselected, &toSelected,
			  NULL);
	    
	    if (fromSelected && toSelected)
	      for (j=0; j < numIp; j++) {
		  XtVaGetValues(ipList[j],
				XtNdblX, &dx,
				XtNdblY, &dy,
				NULL);

		  rotatePoint (&dx, &dy, cx, cy, dphi);

		  XtVaSetValues(ipList[j],
				XtNxpos,(int)dx,
				XtNypos,(int)dy,
				XtNdblX, &dx,
				XtNdblY, &dy,
				NULL);
	      }

	}/* set in ARCS */

	undrawSelectedObjects(nw);

    } /* if phi big enough */

    nw->netWidget.prev_x = event->x; nw->netWidget.prev_y = event->y;

    Redisplay(w,NULL);

} /* SelectionRotation */

#ifdef __STDC__
static void rotatePoint(double *x, double *y,
		 double cx, double cy, double dphi)
#else
static void rotatePoint( x, y, cx, cy, dphi)
     double *x, *y, cx, cy, dphi;
#endif
{
    double dx, dy, distance, phi;

    dx = *x - cx;
    dy = *y - cy;

    distance = sqrt((dx*dx) + (dy*dy));

    phi = atan2(dy, dx);
    phi += dphi;

    *x = cx + (distance * cos(phi));
    *y = cy + (distance * sin(phi));
}
