/*******************************************************************************
Modul   	:	NWMoveActions.c

Purpose		: 	Aktionen zum Bewegen der Objekte

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/


/*
 * NetWidget.c - Manages all objects of a PN.

   Class Hierarchie:
   Core -> Composite -> NetWidget
 */

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include <stdio.h>
#include <string.h>

#include "BaseObj.h"
#include "MarkParObj.h"
#include "RateParObj.h"
#include "PlaceObj.h"
#include "TransObj.h"
#include "ArcObj.h"

#include "NetWidgetP.h"

static void MoveNode();
static void MoveTag();
static void MoveParam();
static void MoveRates();
static void MoveArc();
static void MoveNode_2();
static void MoveTag_2();
static void MoveParam_2();
static void MoveRates_2();
static void MoveArc_2();
static void MoveSelection();
static void MoveSelection_2();

/* the following are private functions unique to NetWidget */

extern void redrawArcs();
/*******************************************************************************
Name:
        Move
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Verzweigung zu einzelaktionen
*******************************************************************************/

void Move(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
int curr_obj;

#ifdef debug
printf(" Move\n");
#endif

  if (nw->netWidget.read_only)
    {
      XtWarning("Model is read only");
      return;
    }
  curr_obj = nw->netWidget.object;

  if (isNode(curr_obj)) 
	MoveNode(w,event,params,numparams);
  else 
  if (isArc(curr_obj)) 
	MoveArc(w,event,params,numparams);
  else 
  if (isMarkPar(curr_obj)) 
	MoveParam(w,event,params,numparams);
  else 
  if (isRatePar(curr_obj)) 
	MoveRates(w,event,params,numparams);
  else
  if (isTag(curr_obj)) 
	MoveTag(w,event,params,numparams);
  else 
  if (isCompleteSelection(curr_obj))
        MoveSelection(w,event,params,numparams);
  else
	return;
}

/*******************************************************************************
Name:
        Move_2
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Verzweigung zu Einzelaktion
*******************************************************************************/

void Move_2(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
int curr_obj = nw->netWidget.object;

#ifdef debug
printf(" Move_2\n");
#endif

  if (isNode(curr_obj)) 
	MoveNode_2(w,event,params,numparams);
  else 
  if (isArc(curr_obj)) 
	MoveArc_2(w,event,params,numparams);
  else 
  if (isMarkPar(curr_obj)) 
	MoveParam_2(w,event,params,numparams);
  else 
  if (isRatePar(curr_obj)) 
	MoveRates_2(w,event,params,numparams);
  else
  if (isTag(curr_obj)) 
	MoveTag_2(w,event,params,numparams);
  else 
  if (isCompleteSelection(curr_obj))
        MoveSelection_2(w,event,params,numparams);
  else
	return;
}

/*******************************************************************************
Name:
        MoveNode
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Bewegen von Stellen,Transitionen, Erstaktion
*******************************************************************************/

static void MoveNode(w,event,params,numparams)
     Widget w;
     XButtonEvent* event;
     String* params;
     Cardinal* numparams;
{
    NetWidget nw = (NetWidget)w;
    Objects object = nw->netWidget.object;
    int center_x,center_y;
    int childInd,grid_x,grid_y;

#ifdef debug
    printf(" MoveNode \n");
#endif

    /* look whether there is an appropriate object under the cursor */
    switch (event->button) {
      case Button1:
	AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
	/* hier lookup ueber event */
	childInd = lookUp(w,event->x,event->y,SELECTION);
	if (childInd ==  NothingFound)
	  return;
	if (object == PLACE
	    && !isPlaceObject(nw->composite.children[childInd]))
	  return;
	if ((object == IMTRANS || object == EXPTRANS || object == DETTRANS || object == GENTRANS)
	    && !isTransObject(nw->composite.children[childInd]))
	  return;
	nw->netWidget.selectedObject = nw->composite.children[childInd];
	/* make sure that noone can disturb this action */
	/* get the stored x,y coordinates of the object */
	XtVaGetValues(nw -> netWidget.selectedObject,
		      XtNxpos,&center_x,
		      XtNypos,&center_y,
		      NULL);

	if (nw->netWidget.tagsVisible)
	  DrawTag(nw->netWidget.selectedObject,
		  XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);
  	if (nw->netWidget.ratesVisible  
	    && isTransObject(nw->netWidget.selectedObject))
	    DrawRate(nw->netWidget.selectedObject,
		     XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);

	nw->netWidget.multiplMode = UndrawM;
	redrawArcs(w,nw->netWidget.selectedObject,nw->netWidget.draw_GC,
		   center_x, center_y);
	nw->netWidget.multiplMode = NoneM;
	redrawArcs(w,nw->netWidget.selectedObject,nw->netWidget.draw_GC,
		   center_x, center_y);

	ignoreSettings(nw);
	nw -> netWidget.allowMotionAction = True;
	nw->netWidget.prev_x=center_x;
	nw->netWidget.prev_y=center_y;

	XtCallCallbacks(w,XtNinfoCallback,(XtPointer)SECONDARY_ACTION);
  	SetModifiedFlag(nw,True);
	/*Redisplay(w,NULL);*/
	break;
      default:
	return;
    }
}/*MoveNode*/

/*******************************************************************************
Name:
        MoveNode_2
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Bewegen von Transitionen, Stellen, zweitaktion
*******************************************************************************/

static void MoveNode_2(w,event,params,numparams)
     Widget w;
     XButtonEvent* event;
     String* params;
     Cardinal* numparams;
{
    Widget selObj;
    NetWidget nw = (NetWidget)w;
    int curr_obj,childInd,grid_x,grid_y;
    
#ifdef debug
    printf(" MoveNode_2 \n");
#endif

    curr_obj = nw -> netWidget.object;
    switch (event->button) {
      case Button1:
	AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
	childInd = lookUp(w,grid_x,grid_y,PUT);
	if (childInd != NothingFound 
	    && nw->composite.children[childInd] != nw->netWidget.selectedObject)
	  return;
	selObj = nw -> netWidget.selectedObject;
	XtVaSetValues(selObj,
		      XtNxpos,grid_x,
		      XtNypos,grid_y,
		      NULL);
	Draw(curr_obj,selObj,
	     XtDisplay(nw),nw->netWidget.picture,nw->netWidget.motion_GC,
	     nw->netWidget.prev_x,nw->netWidget.prev_y,
	     nw->netWidget.object_size,0);
	Draw(curr_obj,selObj,
	     XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
	     grid_x,grid_y,nw->netWidget.object_size,0);
	SetModifiedFlag(nw,True);
	break;
      default:
	return;
    }

    if (nw->netWidget.tagsVisible)
	DrawTag(selObj,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);
    
    if (nw->netWidget.ratesVisible
	&& isTransObject(selObj))
  	DrawRate(nw->netWidget.selectedObject,
		 XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);

    redrawArcs(w,nw->netWidget.selectedObject,nw->netWidget.draw_GC,
	       grid_x, grid_y);
    nw->netWidget.multiplMode = DrawM;
    redrawArcs(w,nw->netWidget.selectedObject,nw->netWidget.draw_GC,
	       grid_x, grid_y);

    allowSettings(nw);
    XtCallCallbacks(w,XtNinfoCallback,PRIMARY_ACTION);
    Redisplay(w,NULL); 
}/*MoveNode_2*/


/*******************************************************************************
  Name:
        MoveTag
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Bewegen von Etiketten, erstaktion
*******************************************************************************/

static void MoveTag(w,event,params,numparams)
     Widget w;
     XButtonEvent* event;
     String* params;
     Cardinal* numparams;
{
    NetWidget nw = (NetWidget)w;
    int childInd,grid_x,grid_y;
    
#ifdef debug
    printf(" MoveTag \n");
#endif
    
    AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
    /* hier lookup ueber event */
    childInd = lookUp(w,event->x,event->y,SELECTION);
    if (childInd == NothingFound)
      return;
    if (!(isTransObject(nw->composite.children[childInd])
	  || isPlaceObject(nw->composite.children[childInd])))
      return;
    nw->netWidget.selectedObject = nw->composite.children[childInd];

    switch (event->button) {
      case Button1:
  	ignoreSettings(nw);
  	nw -> netWidget.allowMotionAction = True;
  	XtCallCallbacks(w,XtNinfoCallback,(XtPointer)SECONDARY_ACTION);
  	break;
      case Button2:
	DrawTag(nw->netWidget.selectedObject,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);
	XtVaSetValues(nw->netWidget.selectedObject,
		      XtNtagXPos,nw->netWidget.object_size,
		      XtNtagYPos,-nw->netWidget.object_size/2,
		      NULL);
	DrawTag(nw->netWidget.selectedObject,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);
	SetModifiedFlag(nw,True);
	/*Redisplay(w,NULL);*/
	break;
      default:
	return;
    }
}/*Move*/


/*******************************************************************************
Name:
        MoveTag_2
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Bewegen von Etiketten, zweitaktion
*******************************************************************************/

static void MoveTag_2(w,event,params,numparams)
     Widget w;
     XButtonEvent* event;
     String* params;
     Cardinal* numparams;
{
    Widget selObj;
    NetWidget nw = (NetWidget)w;
    int x,y;
    int curr_obj,childInd,grid_x,grid_y;
    
#ifdef debug
    printf(" MoveTag_2 \n");
#endif

    curr_obj = nw -> netWidget.object;
    switch (event->button) {
      case Button1:
	AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
	childInd = lookUp(w,grid_x,grid_y,PUT);
	if (childInd != NothingFound 
	    && nw->composite.children[childInd] != nw->netWidget.selectedObject)
	  return;
	selObj = nw -> netWidget.selectedObject;
	DrawTag(selObj,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.motion_GC);

	XtVaGetValues(selObj,
		      XtNxpos, &x,
		      XtNypos, &y,
		      NULL);

	XtVaSetValues(selObj,
		      XtNtagXPos,grid_x - x,   /* relative coordinates */
		      XtNtagYPos,grid_y - y,
		      NULL);

	DrawTag(selObj,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);
	allowSettings(nw);
	SetModifiedFlag(nw,True);
	/*Redisplay(w,NULL);*/
	XtCallCallbacks(w,XtNinfoCallback,PRIMARY_ACTION);
	break;
      default:
	return;
    }
}/*MoveTag_2*/

/*******************************************************************************
Name:
        MoveRates
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Bewegen von Ratenparametern oder Raten von transistionen,erstaktion
*******************************************************************************/

static void MoveRates(w,event,params,numparams)
     Widget w;
     XButtonEvent* event;
     String* params;
     Cardinal* numparams;
{
    NetWidget nw = (NetWidget)w;
    int x,y,childInd,grid_x,grid_y;
    
#ifdef debug
    printf(" MoveRates \n");
#endif

    switch (event->button) {
      case Button1:
  	AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
  	/* hier lookup ueber event */
  	childInd = lookUp(w,event->x,event->y,SELECTION);
  	if (childInd == NothingFound)
	  return;
  	if (!(   isTransObject(nw->composite.children[childInd])
	      || isRateParObject(nw->composite.children[childInd])))
	  return;
  	nw->netWidget.selectedObject = nw->composite.children[childInd];
	if (isTransObject(nw->netWidget.selectedObject))
	  XtVaGetValues(nw -> netWidget.selectedObject,
			XtNratePos_x,&x,
			XtNratePos_y,&y,
			NULL);
	else
	  XtVaGetValues(nw -> netWidget.selectedObject,
			XtNxpos,&x,
			XtNypos,&y,
			NULL);
	ignoreSettings(nw);
	nw -> netWidget.allowMotionAction = True;
	nw->netWidget.prev_x=x;
	nw->netWidget.prev_y=y;
  	XtCallCallbacks(w,XtNinfoCallback,(XtPointer)SECONDARY_ACTION);
    	break;
      case Button2:
  	AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
  	/* hier lookup ueber event */
  	childInd = lookUp(w,event->x,event->y,SELECTION);
  	if (childInd == NothingFound)
	  return;
  	if (!isTransObject(nw->composite.children[childInd]))
	  return;
  	nw->netWidget.selectedObject = nw->composite.children[childInd];
	DrawRate(nw->netWidget.selectedObject,
		 XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);
	XtVaSetValues(nw->netWidget.selectedObject,
		      XtNratePos_x,nw->netWidget.object_size,
		      XtNratePos_y,nw->netWidget.object_size,
		      NULL);
	DrawRate(nw->netWidget.selectedObject,
		 XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);
  	SetModifiedFlag(nw,True);
	/*Redisplay(w,NULL);*/
	break;
      default:
	return;
    }
}/*MoveRate*/


/*******************************************************************************
Name:
        MoveRates_2
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Bewegen von Ratenparametern, Raten von Transitionen, Zweitaktion
*******************************************************************************/

static void MoveRates_2(w,event,params,numparams)
     Widget w;
     XButtonEvent* event;
     String* params;
     Cardinal* numparams;
{
    Widget selObj;
    NetWidget nw = (NetWidget)w;
    int x,y,curr_obj,childInd,grid_x,grid_y;
    
#ifdef debug
    printf(" MoveRates_2 \n");
#endif
    
    curr_obj = nw -> netWidget.object;
    switch (event->button) {
      case Button1:
  	AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
  	childInd = lookUp(w,grid_x,grid_y,PUT);
  	if (   childInd != NothingFound 
            && nw->composite.children[childInd] != nw->netWidget.selectedObject)
	  return;
  	selObj = nw -> netWidget.selectedObject;
	/* set the new x,y coordinates of the object */
  	if (isTransObject(selObj)) {
	    DrawRate(selObj,
		   XtDisplay(nw),nw->netWidget.picture,nw->netWidget.motion_GC);
	    XtVaGetValues(selObj, 
			  XtNxpos, &x,
			  XtNypos, &y,
			  NULL);
	    XtVaSetValues(selObj,
			  XtNratePos_x,grid_x - x,
			  XtNratePos_y,grid_y - y,
			  NULL);
	    DrawRate(selObj,
		     XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC);
  	}
	else {
	    XtVaSetValues(selObj,
			  XtNxpos,grid_x,
			  XtNypos,grid_y,
			  NULL);
            Draw(curr_obj,selObj,
		 XtDisplay(nw),nw->netWidget.picture,nw->netWidget.motion_GC,
		 nw->netWidget.prev_x,nw->netWidget.prev_y,
		 nw->netWidget.object_size,0);
	    Draw(curr_obj,selObj,
		 XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
		 grid_x,grid_y,nw->netWidget.object_size,0);
  	}
  	allowSettings(nw);
  	SetModifiedFlag(nw,True);
  	/*Redisplay(w,NULL);*/
  	XtCallCallbacks(w,XtNinfoCallback,PRIMARY_ACTION);
  	break;
      default:
	return;
    }
}/*MoveRate_2*/

/*******************************************************************************
Name:
        MoveParam
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Bewegen von Parametern,erstaktion
*******************************************************************************/

static void MoveParam(w,event,params,numparams)
     Widget w;
     XButtonEvent* event;
     String* params;
     Cardinal* numparams;
{
    NetWidget nw = (NetWidget)w;
    int center_x,center_y,childInd,grid_x,grid_y;
    
#ifdef debug
    printf(" MoveParam \n");
#endif
    
    switch (event->button) {
      case Button1:
  	AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
  	/* hier lookup ueber event */
	childInd= lookUp(w,event->x,event->y,SELECTION);
	if (childInd == NothingFound)
	  return;
	if (!(   isRateParObject(nw->composite.children[childInd])
	      || isMarkParObject(nw->composite.children[childInd])))
	  return;
  	nw->netWidget.selectedObject = nw->composite.children[childInd];
	XtVaGetValues(nw -> netWidget.selectedObject,
		      XtNxpos,&center_x,
		      XtNypos,&center_y,
		      NULL);
	ignoreSettings(nw);
	nw -> netWidget.allowMotionAction = True;
	nw->netWidget.prev_x=center_x;
	nw->netWidget.prev_y=center_y;
	XtCallCallbacks(w,XtNinfoCallback,(XtPointer)SECONDARY_ACTION);
    	break;
      default:
	return;
    }
}/*MoveParam*/

/*******************************************************************************
  Name:
  MoveParam_2
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Bewegen von Parametern, Zweitaktion
*******************************************************************************/

static void MoveParam_2(w,event,params,numparams)
     Widget w;
     XButtonEvent* event;
     String* params;
     Cardinal* numparams;
{
    Widget selObj;
    NetWidget nw = (NetWidget)w;
    static int curr_obj,childInd,grid_x,grid_y;

#ifdef debug
    printf(" MoveParam_2 \n");
#endif
    curr_obj = nw -> netWidget.object;
    switch (event->button) {
      case Button1:
  	AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
   	childInd = lookUp(w,grid_x,grid_y,PUT);
   	if (   childInd != NothingFound
            && nw->composite.children[childInd] != nw->netWidget.selectedObject)
	  return;
	selObj = nw -> netWidget.selectedObject;
	XtVaSetValues(selObj,
		      XtNxpos,grid_x,
		      XtNypos,grid_y,
		      NULL);
        Draw(curr_obj,selObj,
	     XtDisplay(nw),nw->netWidget.picture,nw->netWidget.motion_GC,
	     nw->netWidget.prev_x,nw->netWidget.prev_y,
	     nw->netWidget.object_size,0);
	Draw(curr_obj,selObj,
	     XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
	     grid_x,grid_y,nw->netWidget.object_size,0);
  	SetModifiedFlag(nw,True);
	/*Redisplay(w,NULL);*/
	allowSettings(nw);
  	XtCallCallbacks(w,XtNinfoCallback,PRIMARY_ACTION);
    	break;
      default:
	return;
    }
}/*MoveParam_2*/


/*******************************************************************************
Name:
        MoveArc
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Bewegen von Kanten, erstaktion
*******************************************************************************/

static void MoveArc(w,event,params,numparams)
     Widget w;
     XButtonEvent* event;
     String* params;
     Cardinal* numparams;
{
    NetWidget nw = (NetWidget)w;
    int x,y,grid_x,grid_y,ip_num,i;

#ifdef debug
    printf(" MoveArc \n");
#endif

    switch (event->button) {
      case Button1:
  	for (i = FirstArc(nw),ip_num = NOT_FOUND ; 
	     i<LastArc(nw) && ip_num == NOT_FOUND; i++)
	  {
	      ip_num = recievePointOfArc(nw->composite.children[i],
					 event->x,event->y,
					 nw->netWidget.object_size,
					 POINT);
	  }
	if (ip_num == NOT_FOUND)
	  return;
  	nw->netWidget.selectedObject = nw->composite.children[i-1];
	getImPCoords(nw->netWidget.selectedObject,ip_num,&x,&y);
	nw -> netWidget.prev_x = x;
	nw -> netWidget.prev_y = y;
  	nw->netWidget.selected_intermediate_point = ip_num;
  	SetModifiedFlag(nw,True);
	break;
      case Button2:
  	for (i = FirstArc(nw),ip_num = NOT_FOUND ; 
	     i<LastArc(nw) && ip_num == NOT_FOUND; i++)
	  {
	      ip_num = recievePointOfArc(nw->composite.children[i],
					 event->x,event->y,
					 nw->netWidget.object_size,
					 SECTION);
	  }
	if (ip_num == NOT_FOUND)
	  return;
  	AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
  	nw->netWidget.selectedObject = nw->composite.children[i-1];

	/* wegmalen */
	nw->netWidget.multiplMode = UndrawM;
	DrawArc(nw->netWidget.selectedObject,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
		nw->netWidget.object_size);
	
	createImPoint(nw->netWidget.selectedObject,grid_x,grid_y,ip_num);

	/* hinmalen */
	nw->netWidget.multiplMode = DrawM;
	DrawArc(nw->netWidget.selectedObject,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
		nw->netWidget.object_size);

	nw -> netWidget.prev_x = grid_x;
	nw -> netWidget.prev_y = grid_y;
  	nw->netWidget.selected_intermediate_point = ip_num;
  	SetModifiedFlag(nw,True);
	/*Redisplay(w,NULL);*/
	break;
      default: return;
    }
    ignoreSettings(nw);

    /* wegmalen (mit Multiplizitaet) */
    nw->netWidget.multiplMode = UndrawM;
    DrawArc(nw->netWidget.selectedObject,
	    XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
	    nw->netWidget.object_size);

    nw -> netWidget.allowMotionAction = True; 

    /* hinmalen (ohne Multiplizitaet) */
    nw->netWidget.multiplMode = NoneM;
    DrawArc(nw->netWidget.selectedObject,
	    XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
	    nw->netWidget.object_size);

    XtCallCallbacks(w,XtNinfoCallback,(XtPointer)SECONDARY_ACTION);
    Redisplay(w,NULL,NULL);
}/*MoveArc*/

/*******************************************************************************
Name:
        MoveArc_2
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Bewegen von Kanten, Zweitaktion
*******************************************************************************/

static void MoveArc_2(w,event,params,numparams)
     Widget w;
     XButtonEvent* event;
     String* params;
     Cardinal* numparams;
{
    Widget selObj;
    NetWidget nw = (NetWidget)w;
    static int childInd,grid_x,grid_y;

#ifdef debug
    printf(" MoveArc_2 \n");
#endif
    selObj = nw->netWidget.selectedObject;
    /* find out wether there is any object in the area of the cursor */
    switch (event->button) {
      case Button1:
  	AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
  	childInd = lookUp(w,grid_x,grid_y,SELECTION);
  	if (childInd != NothingFound)
	  return;

	/* wegmalen */
	DrawArc(nw->netWidget.selectedObject,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
		nw->netWidget.object_size);

	/* set the new x,y coordinates of the object */
	resetImPoint(selObj,grid_x,grid_y,
		     nw->netWidget.selected_intermediate_point);

	/* hinmalen mit Multipl. */
	nw->netWidget.multiplMode = DrawM;
	DrawArc(nw->netWidget.selectedObject,
		XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
		nw->netWidget.object_size);

	allowSettings(nw);
  	XtCallCallbacks(w,XtNinfoCallback,PRIMARY_ACTION);
  	SetModifiedFlag(nw,True);
	/*Redisplay(w,NULL);*/
    	break;
      default:
	return;
    }
    Redisplay(w,NULL,NULL);
}/*MoveArc_2*/

#ifdef __STDC__
/******************************************************************************/
static void MoveSelection(Widget w, 
/******************************************************************************/
			  XButtonEvent* event, 
			  String* params, 
			  Cardinal* numparams)
#else
static void MoveSelection(w,event,params,numparams)
     Widget w;
     XButtonEvent* event;
     String* params;
     Cardinal* numparams;
#endif
{
    NetWidget nw = (NetWidget)w;
    int grid_x, grid_y;

    if (isInPolygon(nw->netWidget.selectionBorder, 
		    (double) event->x, (double) event->y)) {
	
	AdjustEventToGrid(w,event->x,event->y,&grid_x,&grid_y);
	
	nw->netWidget.prev_x = grid_x;
	nw->netWidget.prev_y = grid_y;
	ignoreSettings(nw);
	
	nw->netWidget.allowMotionAction = True;     
	nw->netWidget.multiplMode = NoneM;

	Redraw((Widget)nw, NULL, NULL, NULL);

	/* undraw tags & rates */
	if (nw->netWidget.tagsVisible || nw->netWidget.ratesVisible)
	  undrawSelectedTags(nw);

	XtCallCallbacks(w,XtNinfoCallback,(XtPointer)SECONDARY_ACTION);
    }
    else 
      nw->netWidget.command = NOOPERATION;

} /* MoveSelection */

#ifdef __STDC__
/******************************************************************************/
static void MoveSelection_2(Widget w, 
/******************************************************************************/
		      XButtonEvent* event, 
		      String* params, 
		      Cardinal* numparams)
#else
static void MoveSelection_2(w,event,params,numparams)
     Widget w;
     XButtonEvent* event;
     String* params;
     Cardinal* numparams;
#endif
{
    NetWidget nw = (NetWidget)w;

    /* allowSettings(nw); */
    nw->netWidget.actionTerminated = True;
    nw->netWidget.selectedObject = NULL;
    nw->netWidget.allowMotionAction = False;
    nw->netWidget.prev_x = -1;
    nw->netWidget.prev_y = -1;
    nw->netWidget.multiplMode = DrawM;

    XtSetSensitive(editCommandBox,True);
    SetModifiedFlag(nw,True);
    
    /* redraw tags & rates */
    if (nw->netWidget.tagsVisible || nw->netWidget.ratesVisible)
      undrawSelectedTags(nw);    

    XtCallCallbacks(w,XtNinfoCallback,(XtPointer)PRIMARY_ACTION);
} /* MoveSelection_2 */
