/*******************************************************************************
Modul   	:	NWRescale.c

Purpose		: 	Realisation des Rescaling

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/


/* NWAppActions.c: Action routines called from Application 
 *
 * NetWidget.c - Manages all objects of a PN.

   Class Hierarchie:
   Core -> Composite -> NetWidget
 */

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include <stdio.h>
#include <string.h>

#include "ArcObj.h"
#include "PlaceObj.h"
#include "TransObj.h"
#include "MarkParObj.h"
#include "RateParObj.h"

/* die Abhaengigkeit zur Aussenwelt (global.h) sollte abgeschaft werden
   Communikation ueber CallBack (XtSetSensitive)?? */

#include "NetWidgetP.h"

static float ComputeRescaleValue();

/*******************************************************************************
Name:
	RescaleNet
--------------------------------------------------------------------------------
Funktionstyp:
	Im Programmkomplex 'netWidget' oeffentliche Funktion
--------------------------------------------------------------------------------
Parameter:
        w: Instanz der Klasse NetWidget
	factor: Gew"unschter rescalefactor
--------------------------------------------------------------------------------
Zweck:
	Vergr"ossern eines Modelles
*******************************************************************************/

void RescaleNet(w,factor)
Widget w;
RescaleFactor factor;
{
NetWidget nw = (NetWidget)w;
float rescale_value = ComputeRescaleValue(w,factor);
Cardinal i;
int x,y,tag_x,tag_y,rate_x,rate_y;

  nw->netWidget.rescale_value *= rescale_value;
  for (i=0;i<nw->composite.num_children;i++)
  {
	if (FirstArc(nw)<=i && i< LastArc(nw))
	{
	    rescaleImPoint(nw->composite.children[i],rescale_value);
	}
	else
	if (FirstPlace(nw)<=i && i< LastPlace(nw))
	{
	    XtVaGetValues(nw->composite.children[i],
			XtNxpos,&x,
			XtNypos,&y,
			XtNtagXPos,&tag_x,
			XtNtagYPos,&tag_y,
			NULL);
  	    XtVaSetValues(nw->composite.children[i],
			XtNxpos,(int)((float)x*rescale_value),
			XtNypos,(int)((float)y*rescale_value),
			XtNtagXPos,(int)((float)tag_x*rescale_value),
			XtNtagYPos,(int)((float)tag_y*rescale_value),
			NULL);
	}
	else
	if (FirstTrans(nw)<=i && i< LastTrans(nw))
	{
	    XtVaGetValues(nw->composite.children[i],
			XtNxpos,&x,
			XtNypos,&y,
			XtNtagXPos,&tag_x,
			XtNtagYPos,&tag_y,
			XtNratePos_x,&rate_x,
			XtNratePos_y,&rate_y,
			NULL);
  	    XtVaSetValues(nw->composite.children[i],
			XtNxpos,(int)((float)x*rescale_value),
			XtNypos,(int)((float)y*rescale_value),
			XtNtagXPos,(int)((float)tag_x*rescale_value),
			XtNtagYPos,(int)((float)tag_y*rescale_value),
			XtNratePos_x,(int)((float)rate_x*rescale_value),
			XtNratePos_y,(int)((float)rate_y*rescale_value),
			NULL);
	}
	else
	{
	    XtVaGetValues(nw->composite.children[i],
			XtNxpos,&x,
			XtNypos,&y,
			NULL);
  	    XtVaSetValues(nw->composite.children[i],
			XtNxpos,(int)((float)x*rescale_value),
			XtNypos,(int)((float)y*rescale_value),
			NULL);
	}
   }
   XFillRectangle(XtDisplay(nw),nw->netWidget.picture,nw->netWidget.clear_GC,
		0,0,nw->core.width,nw->core.height);
   DrawNet(w,NORMAL);
   Redisplay(w,NULL);
   nw->netWidget.picture_modified = True;
}

/*******************************************************************************
Name:
	ComputeRescaleValue
--------------------------------------------------------------------------------
Funktionstyp:
	private funktion
--------------------------------------------------------------------------------
Parameter:
        rescaleFactor: Faktor als Typ RescaleFacto
--------------------------------------------------------------------------------
Rueckgabewert:
        Faktor als float
--------------------------------------------------------------------------------
Zweck:
	Konvertiert Typ RescaleFactor zu float
*******************************************************************************/
static float ComputeRescaleValue(w,rescaleFactor)
Widget w;
RescaleFactor rescaleFactor;
{
NetWidget nw = (NetWidget)w;

  switch (rescaleFactor)
  {
    case Reset:		return 1.0/nw->netWidget.rescale_value;
    case ZeroFive:	return 0.5;
    case ZeroSix:	return 0.6;
    case ZeroSeven:	return 0.7;
    case ZeroEight:	return 0.8;
    case ZeroNine:	return 0.9;
    case OneZero:	return 1.0;
    case OneOne :	return 1.1;
    case OneTwo:	return 1.2;
    case OneThree :	return 1.3;
    case OneFour:	return 1.4;
    case OneFive :	return 1.5;
    case OneSix:	return 1.6;
    case OneSeven :	return 1.7;
    case OneEight:	return 1.8;
    case OneNine :	return 1.9;
    case TwoZero:	return 2.0;
    default:		return 1.0;
  }
}
