/*******************************************************************************
Modul   	:	NWResultActions.c

Purpose		: 	Bereitstellen globaler Aktionen zur Behandlung der 
                        Rewardma"se

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/


/* NWAppActions.c: Action routines called from Application 
 *
 * NetWidget.c - Manages all objects of a PN.

   Class Hierarchie:
   Core -> Composite -> NetWidget
 */

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include <stdio.h>
#include <string.h>

#include "ResultObject.h"

#include "NetWidgetP.h"

/*******************************************************************************
Name:
	AddResult
--------------------------------------------------------------------------------
Funktionstyp:
	Aktion
--------------------------------------------------------------------------------
triggert bei event:
	aufgerufen ueber XtCallActionProc in Funktion 'InputDone' in 
	resultEdit.c
--------------------------------------------------------------------------------
Parameter:	
	params[0]: name des resultmeasures
	params[1]: definition des resultmeasures
	numparams: 2
--------------------------------------------------------------------------------
Zweck:
	Zeugen einer neuen Instanz der Klasse 'ResultObject'
*******************************************************************************/

void AddResult(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
String name;
int i;

#ifdef debug
printf("AddResult \n");
#endif

  if (*numparams < 2) {
	XtWarning("Add Result: not enough parameter found, ignored");
	return;
  }
  for (i=FirstResult(nw);i<LastResult(nw);i++) 
  {
	XtVaGetValues(nw->composite.children[i],
			XtNresultName,&name,
			NULL);
	if (!strcmp(name,params[0])) 
  	{
	    XtWarning("Add Result: This result measure already exists: ignored");
	    return;
	}
  }
  XtVaCreateWidget(params[0],
		   resultObjectClass,
		   w,
		   XtNresultName,params[0],
		   XtNresultDefinition,params[1],
		   NULL);
  SetNetModifiedFlag(nw,True);
}

/*******************************************************************************
Name:
	DeleteResult
--------------------------------------------------------------------------------
Funktionstyp:
	Aktion
--------------------------------------------------------------------------------
triggert bei event:
	aufgerufen ueber XtCallActionProc in Funktion 'InputDone' in 
	resultEdit.c
--------------------------------------------------------------------------------
Parameter:	
	params[0]: name des resultmeasures
	params[1]: definition des resultmeasures
	numparams: 2
--------------------------------------------------------------------------------
Zweck:
	Zerstoeren einer Instanz der Klasse 'ResultObject'
*******************************************************************************/

void DeleteResult(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
String name;
char msg[500];
int i;

#ifdef debug
printf("DeleteResult \n");
#endif

  if (*numparams < 1) {
	XtWarning("Delete Result: not enough parameter found, ignored");
	return;
  }
  for (i=FirstResult(nw);i<LastResult(nw);i++) {
	XtVaGetValues(nw->composite.children[i],
			XtNresultName,&name,
			NULL);
	if (!strcmp(name,params[0])) {
	    XtDestroyWidget(nw->composite.children[i]);
  	    SetNetModifiedFlag(nw,True);
	    return;
	}
  }
  sprintf(msg,"Delete Result: resultmesure named %s not found",params[0]);
  XtWarning(msg);
}

/*******************************************************************************
Name:
	ChangeResult
--------------------------------------------------------------------------------
Funktionstyp:
	Aktion
--------------------------------------------------------------------------------
triggert bei event:
	aufgerufen ueber XtCallActionProc in Funktion 'InputDone' in 
	resultEdit.c
--------------------------------------------------------------------------------
Parameter:	
	params[0]: name des resultmeasures
	params[1]: definition des resultmeasures
	numparams: 2
--------------------------------------------------------------------------------
Zweck:
	Aendern der Definition einer Instanz der Klasse 'ResultObject'
*******************************************************************************/

void ChangeResult(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
String name;
char msg[500];
int i;

#ifdef debug
printf("ChangeResult \n");
#endif

  if (*numparams < 2) {
	XtWarning("Change Result: not enough parameter found, ignored");
	return;
  }
  for (i=FirstResult(nw);i<LastResult(nw);i++) {
	XtVaGetValues(nw->composite.children[i],
			XtNresultName,&name,
			NULL);
	if (!strcmp(name,params[0])) {
	    XtVaSetValues(nw->composite.children[i],
				XtNresultDefinition,params[1],
				NULL);
  	    SetNetModifiedFlag(nw,True);
	    return;
	}
  }
  sprintf(msg,"Change Result: resultmesure named %s not found",params[0]);
  XtWarning(msg);

}

/*******************************************************************************
Name:
	getResults
--------------------------------------------------------------------------------
Funktionstyp:
	public function
--------------------------------------------------------------------------------
benutzt in:
	resultEdit.c
--------------------------------------------------------------------------------
Parameter:
	w: 	Aktuelle Instanz des Modells
    Ausgabe:
	results: Array in denen die vorhanden Resultmeasures gesammelt werden
	numresults: Groesse des Arrays
--------------------------------------------------------------------------------
Zweck:
	Sammeln der Namen und Definition der result-measures
--------------------------------------------------------------------------------
Bemerkung:
	results[i] := name
	results[i+1] := definition
	numresults := 2* Anzahl resultObjekte

  	Aufrufende Funktion muss Speicher mit XtFree freigeben.
*******************************************************************************/

void getResults(w,results,numresults)
Widget w;
String **results;
Cardinal * numresults;

{
NetWidget nw = (NetWidget)w;
String * rew_rec;
Cardinal num_rew_rec;
String rew_name,rew_def;
int i,j;

#ifdef debug
printf("getResults \n");
#endif

  num_rew_rec = 2*nw->netWidget.numresults;
  rew_rec = (String*)XtMalloc(num_rew_rec*sizeof(String));
  for(i=FirstResult(nw),j=0;i<LastResult(nw),j<num_rew_rec;i++,j+=2) {
	XtVaGetValues(nw->composite.children[i],
			XtNresultName,&rew_name,
			XtNresultDefinition,&rew_def,
			NULL);
	rew_rec[j] = XtNewString(rew_name);
	rew_rec[j+1] = XtNewString(rew_def);
  }
  *results = rew_rec;
  *numresults = num_rew_rec;
}
