/*******************************************************************************
Modul   	:	NWSelectActions.c

Purpose		: 	Aktionen zum Selektieren von Objekten

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Christian Luehe

History		:	release 1: Februar 1993
*******************************************************************************/

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include <stdio.h>
#include <string.h>
#include <math.h>

#include "NetWidgetP.h"
#include "TransObj.h"
#include "PlaceObj.h"
#include "Polygon.h"
#include "ArcObj.h"

/* intern defined actions */

void Select();
void Select_2();
void computeSelectedObjects();

#ifdef __STDC__
void undrawSelectedTags (NetWidget nw);
void drawCenter (NetWidget nw, int x, int y);
void setSelectedFalse (NetWidget nw);
#else
void undrawSelectedTags ();
void drawCenter ();
void setSelectedFalse ();
#endif

/* extern functions */
#ifdef __STDC__

extern void showSelectedObjects(NetWidget nw);
extern void closeSelection(NetWidget nw, int x, int y);
extern void undrawSelectionBorder(NetWidget nw);
extern void undrawCanceldSelectionBorder(NetWidget nw, int x, int y);
extern void drawArc(NetWidget nw, int x, int y, 
		    unsigned int width, unsigned int height,
		    int a1, int a2);
extern void drawLine(NetWidget nw, int x1, int y1, int x2, int y2);
extern void setDottedLine(NetWidget nw);
extern void setSolidLine(NetWidget nw);
extern Widget getCurrentNet(void);
extern void ResetObjectToggles(void);
extern void setCommandsSelect(void);
extern void setCommandNormal(void);
extern void replaceInfoString(void);

#else

extern void showSelectedObjects();
extern void closeSelection();
extern void undrawSelectionBorder();
extern void undrawCanceldSelectionBorder();
extern void drawArc();
extern void drawLine();
extern void setDottedLine();
extern void setSolidLine();
extern Widget getCurrentNet();
extern void ResetObjectToggles();
extern void setCommandsSelect();
extern void setCommandNormal();
extern void replaceInfoString();

#endif

/* Definition of Actions -----------------------------------------------------*/

/*******************************************************************************
Name:
        RSelect
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        Translation-Management (Ctrl-Button1Down)
--------------------------------------------------------------------------------
Zweck:
        Selektieren von Objekten durch umschliessen mit einem Rechteck
*******************************************************************************/

void RSelect(w,event,params,numparams)
    Widget w;
    XButtonEvent* event;
    String* params;
    Cardinal* numparams;

{
    NetWidget nw = (NetWidget)getCurrentNet();

    if (nw->netWidget.object == UNCOMPLETE_SELECTION &&
	nw->netWidget.command == SELECT)
	return;     /* PSelect */

    /* free old selection if there is one */
    if (nw->netWidget.selectionBorder)
      destroyPolygon(nw->netWidget.selectionBorder);
    nw->netWidget.selectionBorder = createPolygon();
    createVertex(nw->netWidget.selectionBorder, 
		 (double) event->x, (double) event->y);

    nw->netWidget.prev_x = event->x;
    nw->netWidget.prev_y = event->y;

    nw->netWidget.object = UNCOMPLETE_SELECTION;
    nw->netWidget.allowMotionAction = True; 

} /* RSelect */

/*******************************************************************************
Name:
        RSelect_2
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        Translation-Management (Ctrl-Button1Up)
--------------------------------------------------------------------------------
Zweck:
        Selektieren von Objekten durch umschliessen mit einem Rechteck
*******************************************************************************/

void RSelect_2(w,event,params,numparams)
    Widget w;
    XButtonEvent* event;
    String* params;
    Cardinal* numparams;

{
    NetWidget nw =  (NetWidget)getCurrentNet();

    if (nw->netWidget.object == UNCOMPLETE_SELECTION &&
	nw->netWidget.command == SELECT)
	return;     /* PSelect */

    if (!nw->netWidget.selectionBorder ||
	nw->netWidget.selectionBorder->numVertexes < 1)
      return;

    createVertex(nw->netWidget.selectionBorder, 
		 nw->netWidget.selectionBorder->vertexes[0].x,
		 (double) event->y);
    createVertex(nw->netWidget.selectionBorder, 
		 (double) event->x, (double) event->y);
    createVertex(nw->netWidget.selectionBorder, 
		 (double) event->x, 
		 nw->netWidget.selectionBorder->vertexes[0].y);
    
    computeSelectedObjects(nw);

    nw->netWidget.selectedObject = NULL;
    nw->netWidget.allowMotionAction = False;
    nw->netWidget.prev_x = -1;
    nw->netWidget.prev_y = -1;
    
    ResetObjectToggles();

    nw->netWidget.command = NOOPERATION;
    nw->netWidget.object  = COMPLETE_SELECTION;

    XtSetSensitive(editObjectBox,False); 
    setCommandsSelect();

    Redraw(nw, NULL, NULL, NULL);

} /* RSelect_2 */

/*******************************************************************************
Name:
        PSelect
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callEditAction
--------------------------------------------------------------------------------
Zweck:
        Selektieren von Objekten durch umschliessen mit einem Polygon
*******************************************************************************/

void PSelect(w,event,params,numparams)
    Widget w;
    XButtonEvent* event;
    String* params;
    Cardinal* numparams;

{
    NetWidget nw = (NetWidget)getCurrentNet();

    /* START SELECTION BY POLYGON */
    if (nw->netWidget.object != UNCOMPLETE_SELECTION) {

	/* free old selection if there is one */
	if (nw->netWidget.selectionBorder)
	    destroyPolygon(nw->netWidget.selectionBorder);
	nw->netWidget.selectionBorder = createPolygon();
	createVertex(nw->netWidget.selectionBorder, 
		     (double) event->x, (double) event->y);

	nw->netWidget.object = UNCOMPLETE_SELECTION;
	nw->netWidget.command = SELECT;

	ignoreSettings(nw);
	XtCallCallbacks(w,XtNinfoCallback,(XtPointer)SECONDARY_ACTION);

	nw->netWidget.prev_x = event->x;
	nw->netWidget.prev_y = event->y;

	nw->netWidget.allowMotionAction = True;

	setDottedLine(nw);
	drawArc(nw, 
		event->x-TOLERANCE, event->y-TOLERANCE,
		TOLERANCE*2, TOLERANCE*2, 0, 360*64);
	
	Redisplay(w,NULL);
	
	setSolidLine(nw);
	
    } /* START */

    /* CONTINUE/CLOSE SELECTION BY POLYGON */
    else {
	double x,y;

	if (!nw->netWidget.selectionBorder ||
	    nw->netWidget.selectionBorder->numVertexes < 1)
	    return;

	/*  new vertex or closing of the polygon */
        if (event->button == Button2) {
            x = (double) event->x;
            y = (double) event->y;
        }
        /* close */
        else if (event->button == Button3)
          if (nw->netWidget.selectionBorder->numVertexes > 2) {
              x = nw->netWidget.selectionBorder->vertexes[0].x;
              y = nw->netWidget.selectionBorder->vertexes[0].y;
          }

        /* if closed  */
	if (polygonClosed(nw->netWidget.selectionBorder, 
                          x,y, (double)TOLERANCE)) {

            closeSelection(nw, event->x, event->y);
            
	    computeSelectedObjects(nw);

	    nw->netWidget.selectedObject = NULL;
            nw->netWidget.actionTerminated = True;
	    nw->netWidget.allowMotionAction = False;
	    nw->netWidget.prev_x = -1;
	    nw->netWidget.prev_y = -1;

	    ResetObjectToggles();

	    nw->netWidget.command = NOOPERATION;
	    nw->netWidget.object  = COMPLETE_SELECTION;
	    
	    XtSetSensitive(editObjectBox,False); 
	    XtSetSensitive(editCommandBox,True);
	    setCommandsSelect();	    
	    
            XtCallCallbacks((Widget)nw,XtNinfoCallback,PRIMARY_ACTION);
	    Redraw(nw, NULL, NULL, NULL);	
	}
	/* new vertex */
	else {
	    createVertex(nw->netWidget.selectionBorder, x,y);
	}
    } /* CONTINUE/CLOSE */

} /* PSelect */

/*******************************************************************************
Name:
        SelectionOff
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
aufgerufen "uber:
        callMethod
--------------------------------------------------------------------------------
Zweck:
        Free des Selection-Polygons, Reset der Selektion
*******************************************************************************/

void SelectionOff(w,event,params,numparams)
    Widget w;
    XButtonEvent* event;
    String* params;
    Cardinal* numparams;

{
    NetWidget nw = (NetWidget)getCurrentNet();
    
    if (!nw->netWidget.selectionBorder)
      return;

    destroyPolygon(nw->netWidget.selectionBorder);
    nw->netWidget.selectionBorder = NULL;
    
    nw->netWidget.object = NOOBJECT;
    setSelectedFalse(nw);

    setCommandNormal();
    XtSetSensitive(editCommandBox, True);
    XtSetSensitive(editObjectBox, True);

    replaceInfoString();
    Redraw(nw, NULL, NULL, NULL);
}

/*******************************************************************************
Name:
        computeSelectedObjects
--------------------------------------------------------------------------------
Funktionstyp:
        oeffentlich
--------------------------------------------------------------------------------
aufgerufen "uber:
        
--------------------------------------------------------------------------------
Zweck:
        berechnen der durch die selectionBorder selektierten Objekte
*******************************************************************************/
#ifdef __STDC__
void computeSelectedObjects (NetWidget nw)
#else
void computeSelectedObjects (nw)
     NetWidget nw;
#endif
{
    int i;
    int x,y;
    Boolean yes;

    /* for all transitions */
    for (i = FirstTrans(nw); i<LastTrans(nw); i++) {
	XtVaGetValues(nw->composite.children[i],
		      XtNxpos,&x,
		      XtNypos,&y,
		      NULL);
	yes = isInPolygon(nw->netWidget.selectionBorder, 
			  (double) x, (double) y);
	XtVaSetValues(nw->composite.children[i],
		      XtNselected, yes,
		      NULL);
    }

    /* for all places */
    for (i = FirstPlace(nw); i<LastPlace(nw); i++) {
	XtVaGetValues(nw->composite.children[i],
		      XtNxpos,&x,
		      XtNypos,&y,
		      NULL);
	yes = isInPolygon(nw->netWidget.selectionBorder, 
			  (double) x, (double) y);
	XtVaSetValues(nw->composite.children[i],
		      XtNselected, yes,
		      NULL);
    }
}

#ifdef __STDC__
/*****************************************************************************/
void setSelectedFalse (NetWidget nw)
/*****************************************************************************/
#else
void setSelectedFalse (nw)
     NetWidget nw;
#endif
{
    int i;

    /* for all transitions */
    for (i = FirstTrans(nw); i<LastTrans(nw); i++) 
      XtVaSetValues(nw->composite.children[i],
		    XtNselected, (XtPointer)False,
		    NULL);

    /* for all places */
    for (i = FirstPlace(nw); i<LastPlace(nw); i++) 
      XtVaSetValues(nw->composite.children[i],
		    XtNselected, (XtPointer)False,
		    NULL);
}


#ifdef __STDC__
/*****************************************************************************/
void undrawSelectedTags(NetWidget nw)
/*****************************************************************************/
#else
void undrawSelectedTags(nw)
     NetWidget nw;
#endif
{
    Cardinal i;
    Boolean selected;

    for (i=FirstTrans(nw); i < LastTrans(nw); i++) {
	XtVaGetValues (nw->composite.children[i],
		       XtNselected, &selected,
		       NULL); 
	if (selected) {
	    if (nw->netWidget.tagsVisible)
	      DrawTag(nw->composite.children[i],
		      XtDisplay(nw),nw->netWidget.picture,
		      nw->netWidget.draw_GC);
	    if (nw->netWidget.ratesVisible)
	      DrawRate(nw->composite.children[i],
		       XtDisplay(nw),nw->netWidget.picture,
		       nw->netWidget.draw_GC);
	}
    }
    if (nw->netWidget.tagsVisible)
      for (i=FirstPlace(nw); i < LastPlace(nw); i++) {
	  XtVaGetValues(nw->composite.children[i],
			XtNselected, &selected,
			NULL);
	  if (selected)
	    DrawTag(nw->composite.children[i],
		    XtDisplay(nw),nw->netWidget.picture,
		    nw->netWidget.draw_GC);
      }
}/* undrawSelectedTags */

#ifdef __STDC__
/*****************************************************************************/
void drawCenter(NetWidget nw, int x, int y) 
/*****************************************************************************/
#else
void drawCenter(nw, x, y)
     NetWidget nw;
     int x,y;
#endif
{
#define CENTERRADIUS 5
    drawLine (nw, x-CENTERRADIUS, y, x+CENTERRADIUS, y);
    drawLine (nw, x, y-CENTERRADIUS, x, y+CENTERRADIUS);
} /* drawCenter */

