/*******************************************************************************
Modul   	:	NWShowActions.c

Purpose		: 	Bereitstellen von Aktionen zur Anzeige erhaltener Ergebnisse:
                        Verteilung der Token einzelner Stellen, Anzeige der 
			Ratendefinitionen einzelner Transitionen und deren 
			Berechneter Durchsatz
                        

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
				release 2: Maerz 1994  (Kolja Koischwitz)
				release 3: Sommer 1996 (Heiko Bergmann)
*******************************************************************************/

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <errno.h>

#include "MarkParObj.h"
#include "RateParObj.h"
#include "PlaceObj.h"
#include "TransObj.h"
#include "ArcObj.h"

#include "NetWidgetP.h"

#ifdef __STDC__
extern char *vlstrcat(char **dst, char *src);
#else
extern char *vlstrcat();
#endif

extern int GetTimeMode();
enum {ContTime=0,DiscTime};

#define RESULTS_COLLECTED 0
#define NO_RESULTS 1
#define FILE_ERROR 2

static void ShowTokenDistribution();
static void ShowTransitionResults();
static void ShowArcDefinition();
static int CollectPlaceResults();
static int CollectTransitionResults();

/*******************************************************************************
Name:
	ExpectResults
--------------------------------------------------------------------------------
Funktionstyp:
	"offentliche Funktion
--------------------------------------------------------------------------------
Zweck:
        R"ucksetzen des Flags 'results_collected'
*******************************************************************************/

void ExpectResults(w)
Widget w;
{
  NetWidget nw = (NetWidget)w;

  nw->netWidget.results_collected = False;
}

/*******************************************************************************
Name:
	ShowResults
--------------------------------------------------------------------------------
Funktionstyp:
	Aktion
--------------------------------------------------------------------------------
triggert bei event:
	eingetragen in der Aktionstafel des Netwidgets
--------------------------------------------------------------------------------
Zweck:
	Erkennung des unter dem Pointer liegenden Objekts und Verzweigung
	zu der jeweils benoetigten Aktion.
	
*******************************************************************************/

void ShowResults(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
    NetWidget nw = (NetWidget)w;
    int childInd,i,ip_number;

#ifdef debug
    printf(" ShowResults \n");
#endif

    if (event->button != Button1)
	return;

  childInd = lookUp(w,event->x,event->y,SELECTION);

  if (childInd ==  NothingFound) {
#ifdef debug
      printf(" \tNothingFound \n");
#endif
      for (i = FirstArc(nw),ip_number = NOT_FOUND ;
	   i<LastArc(nw) && ip_number == NOT_FOUND;
	   i++) {
	  ip_number = recievePointOfArc(nw->composite.children[i],
					event->x,event->y,
					nw->netWidget.object_size,SECTION);
      }
      if (ip_number == NOT_FOUND) /* really nothing found */
	  return;                
      if (isArcObject(nw->composite.children[i-1]))
      {
	  nw->netWidget.selectedObject = nw->composite.children[i-1];
	  ShowArcDefinition(w,event,params,numparams);
      }
      return;
  }
  if (isPlaceObject(nw->composite.children[childInd])) 
  {
	nw->netWidget.selectedObject = nw->composite.children[childInd];
	ShowTokenDistribution(w,event,params,numparams);
  }
  else
  if (isTransObject(nw->composite.children[childInd]))
  {
	nw->netWidget.selectedObject = nw->composite.children[childInd];
	ShowTransitionResults(w,event,params,numparams);
  }
  else
	return;
}

/*******************************************************************************
Name:
	ShowTokenDistribution
--------------------------------------------------------------------------------
Funktionstyp:
	Aktion
--------------------------------------------------------------------------------
triggert bei event:
	aufgerufen von ShowResults()
--------------------------------------------------------------------------------
Zweck:
	Zusammenstellung des NWshowTokenDistributionInfo und Aufruf der Callbacks
	der Resource XtNtokenDistributionCallback
	
*******************************************************************************/

static void ShowTokenDistribution(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
NetWidget nw = (NetWidget)w;
NWshowTokenDistributionInfo info;
Distribution *distribution;
String tag;

  switch (CollectPlaceResults(w))
  {
    case NO_RESULTS: 
    case FILE_ERROR: 
	XtVaGetValues(nw->netWidget.selectedObject,
			XtNtagName,&tag,
			NULL);
	info.status = NO_RESULTS_AVAILABLE;
	info.e_x = event->x;
	info.e_y = event->y;
	info.place_name = tag;
	info.error = info.mean = 0.0;
	info.token_distribution = "no results available";
	break;
    default: 
	XtVaGetValues(nw->netWidget.selectedObject,
			XtNtagName,&tag,
			XtNdistribution,&distribution,
			NULL);
	info.status = RESULTS_AVAILABLE;
	info.e_x = event->x;
	info.e_y = event->y;
	info.place_name = tag;
	info.error = distribution -> error;
	info.mean = distribution -> mean;
	info.token_distribution = XtNewString(distribution -> distribution_string);
  }

  XtCallCallbacks(w,XtNtokenDistributionCallback,(XtPointer)&info);
  XtFree(info.token_distribution);
}

/*******************************************************************************
Name:
	ShowTransitionResults
--------------------------------------------------------------------------------
Funktionstyp:
	Aktion
--------------------------------------------------------------------------------
triggert bei event:
	aufgerufen von ShowResults()
--------------------------------------------------------------------------------
Zweck:
	Zusammenstellung des NWshowTransitionResultsInfo und Aufruf der Callbacks
	der Resource XtNtransResultCallback
	
*******************************************************************************/

static void ShowTransitionResults(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;

{
    NetWidget nw = (NetWidget)w;
    NWshowTransitionResultsInfo info;
    String tag,ratepar_name, tag1,ratepar_name1;
    char firing_rate[10000],pri_string[40],format[100],p_string[30],g_string[3000],gw_string[300];
    Rate *rate,*rate1;
    RateValue *rate_val, *rate_val1;
    EnablingDependence en_dep;
    TransType transtype,transtype1;
    Cardinal priority;
    /*new*/
    Cardinal phase;
    int group, group1, i;
    double group_weight ;
    int TimeMode = GetTimeMode();
    RacePolicy racepolicy;
    static char *FiringPolicyStr[] = {" race resampling (RS)",
				      " race age (RA)",
				      " race enabling (RE)"};

    Cardinal precision=nw->netWidget.precision;
    
    switch (CollectTransitionResults(w))
    {
    case NO_RESULTS: 
    case FILE_ERROR: 
	XtVaGetValues(nw->netWidget.selectedObject,
		      XtNtagName,&tag,
		      XtNrates,&rate,
		      XtNpriority,&priority,
		      XtNenablingDependence,&en_dep,
		      XtNtransType,&transtype,
		      /*new*/
		      XtNphase,&phase,
		      XtNgroup,&group,
		      XtNracePolicy,&racepolicy,
		      NULL);
	info.status = NO_RESULTS_AVAILABLE;
	info.throughput = 0.0;
	break;
    default: 
	XtVaGetValues(nw->netWidget.selectedObject,
		      XtNtagName,&tag,
		      XtNrates,&rate,
		      XtNpriority,&priority,
		      XtNenablingDependence,&en_dep,
		      XtNtransType,&transtype,
		      XtNthroughput,&info.throughput, /* <----- */
		      /*new*/
		      XtNphase,&phase,
		      XtNgroup,&group,
		      XtNracePolicy,&racepolicy,
		      NULL);
	info.status = RESULTS_AVAILABLE;
    }
    if (transtype == IMMEDIATE) 
      if (!(info.throughput > 0.0)) info.status = NO_RESULTS_AVAILABLE;
    info.e_x = event->x;
    info.e_y = event->y;
    info.transition_name = tag;
    info.error = 0.0;              /* <---- */
    if (rate->type == RATEVALUE ) 
    {
	if (transtype == EXPONENTIEL)
	{
	    sprintf(format,"Firing delay: %%.%df", precision);
	    sprintf(firing_rate, format,rate->values.ratevalue);
	}
	else
	    if (transtype == DETERMINISTIC)
	    {
		sprintf(format,"Firing delay: %%.%df", precision);
		sprintf(firing_rate,format,rate->values.ratevalue);
	    }
	    else
		if (transtype == IMMEDIATE)
		{
		    sprintf(format,"Weight: %%.%df", precision);
		    sprintf(firing_rate,format,rate->values.ratevalue);
		}
		else if (transtype == GENERAL)
		{
		    sprintf(format,"p.m.-Function: %%s");
		    sprintf(firing_rate,format,rate->values.pmf_function);
		}
	if (en_dep == INFINITE_SERVER)
	    strcat(firing_rate,"    (infinite server)");
/* 	if (priority >= LOWEST_PRI)  */
	if ((transtype == IMMEDIATE) &&  (priority >= LOWEST_PRI)) 
	  { 
	    if (priority == LOWEST_PRI) 
	      sprintf(pri_string,"    (No priority defined.)");
	    else
	      sprintf(pri_string,"    (priority level: %u)",priority);
	    strcat(firing_rate,pri_string);
	  }
	  if ((TimeMode == DiscTime)&&(transtype != IMMEDIATE))
	  {
	    if (priority <= LOWEST_PRI) sprintf(pri_string,"    (No postpriority defined.)");
	    else sprintf(pri_string,"    (postpriority level: %u)",priority);
	    strcat(firing_rate,pri_string);
	  }  
    }
    else
	if (rate->type == RATEPARAM)
	{
	    XtVaGetValues((Widget)rate->values.rateparam,
			  XtNrateParName,&ratepar_name,
			  XtNrateParValue,&rate_val,
			  NULL);
	    
	    if (transtype == IMMEDIATE)
	    {
		if (rate_val->valueType == Ratevalue) 
		{
		    sprintf(format,"Weight: \nParameter %%s = %%.%df"
			    , precision);
		    sprintf(firing_rate,format,
			    ratepar_name,rate_val->values.value);
		}
		else /* Definition */
		{
		    sprintf(firing_rate,
			    "Weight: \nParameter %s = \"%s\"",
			    ratepar_name,
			    rate_val->values.definition ? 
			    rate_val->values.definition :
			    "NULL");
		}
	    }
	    else /* transtype == EXPONENTIEL || transtype == DETERMINISTIC */
	    {
		if (rate_val->valueType == Ratevalue) 
		{
		    sprintf(format,"Firing delay: \nParameter %%s = %%.%df"
			    , precision);
		    sprintf(firing_rate,format,
			    ratepar_name,rate_val->values.value);
		}
		else /* Definition */
		{
		    sprintf(firing_rate, 
			    "Firing delay: \nParameter %s = \"%s\"",
			    ratepar_name,
			    rate_val->values.definition ?
			    rate_val->values.definition :
			    "NULL");
		}
	    }
	    if (en_dep == INFINITE_SERVER)
		strcat(firing_rate,"    (infinite server)");
/* 	    if (priority >= LOWEST_PRI)  */
	    if ((transtype == IMMEDIATE) && (priority >= LOWEST_PRI)) 
	    {
	      sprintf(pri_string,"    (priority level: %u)",priority);
	      strcat(firing_rate,pri_string);
	    }
	    if ((TimeMode == DiscTime) && (transtype != IMMEDIATE))
	    {
	      if (priority <= LOWEST_PRI) sprintf(pri_string,"    (No postpriority defined ..)");
	      else sprintf(pri_string,"    (postpriority level: %u)",priority);
	      strcat(firing_rate,pri_string);
	      } 
	}
	else if( rate->type == MD_DEF ) /* Marking-Dependent firing Delay */
	{
	    if (transtype == EXPONENTIEL)
		sprintf(firing_rate,"Marking dependent firing delay: \n%s",
			rate->values.md_definition);
	    else
		if (transtype == DETERMINISTIC)
		    sprintf(firing_rate,
			    "Marking dependent firing delay: \n%s",
			    rate->values.md_definition);
	    if (transtype == IMMEDIATE)
		sprintf(firing_rate,"Marking dependent weight: \n%s",
			rate->values.md_definition);

	    if ((transtype == IMMEDIATE) && (priority >= LOWEST_PRI)) 
	    {
	      sprintf(pri_string,"    (priority level: %u)",priority);
	      strcat(firing_rate,pri_string);
	    }
	    if ((TimeMode == DiscTime) && (transtype != IMMEDIATE))
	    {
	      if (priority <= LOWEST_PRI) sprintf(pri_string,"    (No postpriority defined ..)");
	      else sprintf(pri_string,"    (postpriority level: %u)",priority);
	      strcat(firing_rate,pri_string);
	      }
	}
        else if(transtype == GENERAL)
        {
         if( rate->type == PMF_FUNC) /*Transtype must be genT!!! */
		    sprintf(firing_rate,"pmf-Function: %s",rate->values.pmf_function);
         else if(rate->type == DATAFILE)  /*Transtype must be genT!!! */
         {
            char temp[255];
            if(!strncmp(rate->values.DataFile,"$MODELDIR",strlen("$MODELDIR")))
            {
             strcpy(temp,(char*)(getenv(MODELDIR_ENV)));
             strcat(temp,rate->values.DataFile+strlen("$MODELDIR"));
	     sprintf(firing_rate,"DataFile: %s",temp);
            }
            else sprintf(firing_rate,"DataFile: %s",rate->values.DataFile);
	 }
	 if ((transtype == IMMEDIATE) && (priority >= LOWEST_PRI)) 
	    {
	      sprintf(pri_string,"    (priority level: %u)",priority);
	      strcat(firing_rate,pri_string);
	    }
	    if ((TimeMode == DiscTime) && (transtype != IMMEDIATE))
	    {
	      if (priority <= LOWEST_PRI) sprintf(pri_string,"    (No postpriority defined ..)");
	      else sprintf(pri_string,"    (postpriority level: %u)",priority);
	      strcat(firing_rate,pri_string);
	      }
        }
        else sprintf(firing_rate,"???");
        
/*    else /* Rate can only be a pmf-Function, Transtype must be genT!!! */
/*		if (transtype == GENERAL)
		    sprintf(firing_rate,"pmf-Function: %s",rate->values.pmf_function);
		else sprintf(firing_rate,"???");
*/
	
    
    /*new*/
	if (transtype == EXPONENTIEL && TimeMode == DiscTime)
	{
	  sprintf(p_string,"\nPhase: %u",phase); 
	  strcat(firing_rate,p_string);
	}
	if (TimeMode == DiscTime)
	{
	  if (transtype != IMMEDIATE)
	  {
	    sprintf(p_string,"\nFiring policy: %s",FiringPolicyStr[racepolicy]);
	    strcat(firing_rate,p_string);
	  }
	  if (group == DEFAULT_GROUP) 
	    sprintf(g_string,"\n\nGroup: No Group specified.");
	  else 
	  { 	  
	    sprintf(g_string,"\n\nTransition %s belongs to group %d\nMembers:\tWeight:", tag, group);
	    
	    for(i = FirstTrans(nw); i < LastTrans(nw); i++) 
	    {
	      XtVaGetValues(nw->composite.children[i],
			    XtNtagName,&tag1,
			    XtNgroup,&group1,
			    XtNgroup_weight,&group_weight,
			    XtNtransType,&transtype1,
			    XtNrates,&rate1,
			    NULL);
	      if (group1 == group ) 
		if (transtype1 != IMMEDIATE)
		{
		  sprintf(gw_string,"\n%s\t\t%G",tag1,group_weight);
		  strcat(g_string,gw_string);
		}
		else
		  if (rate1->type == RATEVALUE) 
		  {
		    sprintf(format,"\n%%s\t\t%%.%dG", precision);
		    sprintf(gw_string,format,
			    tag1,rate1->values.ratevalue);		  
		    strcat(g_string,gw_string);
		  }
		  else
		    if (rate1->type == RATEPARAM)
		    {
		      XtVaGetValues((Widget)rate1->values.rateparam,
				    XtNrateParName,&ratepar_name1,
				    XtNrateParValue,&rate_val1,
				    NULL);
		      if (rate_val1->valueType == Ratevalue) 
		      {
			sprintf(format,"\n%%s\t\tParameter %%s = %%.%dG", precision);
			sprintf(gw_string,format,
				tag1,ratepar_name1,rate_val1->values.value);
			strcat(g_string,gw_string);
		      }
		      else /* Definition */
		      {
			sprintf(gw_string,
				"\n%s\t\tParameter %s = \"%s\"",
				tag1,
				ratepar_name1,
				rate_val1->values.definition ? 
				rate_val1->values.definition :
				"NULL");		  
			strcat(g_string,gw_string);
		      }
		    }
	    }
	  }
	  strcat(firing_rate,g_string);
	}
	info.firing_rate = XtNewString(firing_rate);
    
    
    XtCallCallbacks(w,XtNtransResultCallback,(XtPointer)&info);
}

/*******************************************************************************
Name:
	CollectPlaceResults
--------------------------------------------------------------------------------
Funktionstyp:
	private Funktion
--------------------------------------------------------------------------------
aufgerufen von:
	ShowTokenDistribution
--------------------------------------------------------------------------------
Parameter:
	w: aktuelle Instanz des Netzes
--------------------------------------------------------------------------------
Rueckgabewert:
	RESULTS_COLLECTED: die berechneten Resultate sind gesammelt worden
	NO_RESULTS: kein Resultate vorhanden
	FILE_ERROR: oeffnen,lesen der entsprechende Dateien unmoeglich
--------------------------------------------------------------------------------
Zweck:
	Analysieren der in der Datei <modelname>.PROBTOK vorhandenen Daten.
	Zusammenstellen und setzen der Resource XtNdistribution fuer jede
	einzelne Stelle.
*******************************************************************************/

static int CollectPlaceResults(w)
Widget w;
{
    NetWidget nw = (NetWidget)w;
    FILE *fp;
    struct stat sbuf_1,sbuf_2;
    char path[500],d_string[500],format[25];
    String token_distr_string = NULL;
    String modelpath;
    int i,j,min_token,max_token;
    double d_buf,mean;
    Distribution distribution;
    Cardinal precision=nw->netWidget.precision;
    String name = nw->netWidget.model_name;

    if (nw->netWidget.net_modified) {
	SetCollectResultFlag(nw,False);
	return NO_RESULTS;
    }

  if ((modelpath = getenv(MODELDIR_ENV)) == NULL)
    {
      char msg[200];
      
      sprintf(msg,
	      "Environment variable %s not set; can't open model description",
	      MODELDIR_ENV);
      return FILE_ERROR;
    }

  sprintf(path,"%s/%s%s",modelpath,name,NET_EXTENSION);
  if (stat(path,&sbuf_2) < 0)
	return NO_RESULTS;
  sprintf(path,"%s/%s%s",modelpath,name,PRTOK_EXTENSION);
  if (stat(path,&sbuf_1) < 0)
	return NO_RESULTS;
  if (sbuf_1.st_mtime < sbuf_2.st_mtime)
	    return NO_RESULTS;

  if (nw->netWidget.results_collected)
    	return RESULTS_COLLECTED;

  /* error wert und to_time im .mpd file */ 
  if ((fp = fopen(path,"r")) == NULL)
	return FILE_ERROR;

  /* make a format-string for big and little values */
  /*  sprintf(format,"%%5d : %%g\n"); */
  sprintf(format,"%%5d : %%.%d%s\n",precision,"g");

  for (i=FirstPlace(nw);i<LastPlace(nw);i++)
  {
	if (fread((char*)&d_buf,sizeof(double),1,fp) != 1)
	  {
	    fclose(fp);
	    return FILE_ERROR;
	  }
    	min_token = (int)d_buf;
	if (fread((char*)&d_buf,sizeof(double),1,fp) != 1)
	  {
	    fclose(fp);
	    return FILE_ERROR;
	  }
    	max_token = (int)d_buf;
	mean = 0.0;
 	for (j=min_token;j<=max_token;j++) 
	{
	    if (fread((char*)&d_buf,sizeof(double),1,fp) != 1)
	      {
		fclose(fp);
		XtFree(token_distr_string);
		return FILE_ERROR;
	      }
	    mean += (double)j*d_buf;
	    sprintf(d_string,format,j,d_buf);
	    if (token_distr_string != NULL)
	      token_distr_string = 
		XtRealloc(token_distr_string,
			  (strlen(token_distr_string)+strlen(d_string)+1)*sizeof(char));
	    else
	      token_distr_string = XtCalloc(strlen(d_string)+1,sizeof(char));
	    strcat(token_distr_string,d_string);
	}
	distribution.mean = mean;
	distribution.error = 0.0;
	distribution.distribution_string = token_distr_string;
	XtVaSetValues(nw->composite.children[i],
			XtNdistribution,&distribution,
			NULL);
	XtFree(token_distr_string);
	token_distr_string = NULL;
  }
  SetCollectResultFlag(nw,True);
  fclose(fp);
  return RESULTS_COLLECTED;
}

/*******************************************************************************
Name:
	CollectTransitionResults
--------------------------------------------------------------------------------
Funktionstyp:
	private Funktion
--------------------------------------------------------------------------------
aufgerufen von:
	ShowTransitionResults
--------------------------------------------------------------------------------
Parameter:
	w: aktuelle Instanz des Netzes
--------------------------------------------------------------------------------
Rueckgabewert:
	RESULTS_COLLECTED: die berechneten Resultate sind gesammelt worden
	NO_RESULTS: kein Resultate vorhanden
	FILE_ERROR: oeffnen,lesen der entsprechende Dateien unmoeglich
--------------------------------------------------------------------------------
Zweck:
	Analysieren der in der Datei <modelname>. ?? vorhandenen Daten.
	Zusammenstellen und setzen der Resource XtN ?? fuer jede
	einzelne Transition.
--------------------------------------------------------------------------------
Ablauf:


*******************************************************************************/

static int CollectTransitionResults(w)
     Widget w;
{
    NetWidget     nw = (NetWidget)w;
    FILE          *results_fp;
    struct        stat sbuf, sbuf_net;
    char          results_path[MAXPATHLEN], net_path[MAXPATHLEN];
    char          s[256];
    double        throughput;
    int           i;
    String name = nw->netWidget.model_name;
    String modelpath;

    if (nw->netWidget.net_modified) {
	SetComputeThroughputFlag(nw,False);
	return NO_RESULTS;
    }

    if ((modelpath = getenv(MODELDIR_ENV)) == NULL)
    {
      char msg[200];
      
      sprintf(msg,
	      "Environment variable %s not set; can't open model description",
	      MODELDIR_ENV);
      return FILE_ERROR;
    }

    sprintf(results_path,"%s/%s%s",modelpath,name,RESULT_EXTENSION);
	    
    if (stat(results_path,&sbuf) < 0) {
	return NO_RESULTS;
    }
    
    sprintf(net_path,"%s/%s%s",modelpath,name,NET_EXTENSION);
	  
    if (stat(net_path,&sbuf_net) < 0) {
	perror(net_path);
	return NO_RESULTS;
    }

    if (sbuf.st_mtime < sbuf_net.st_mtime)
      return NO_RESULTS;

    if ((nw->netWidget.throughput_computed) && (nw->netWidget.results_collected))
      return RESULTS_COLLECTED;

    if ((results_fp = fopen(results_path,"r")) == NULL) {
	perror(results_path);
	return FILE_ERROR;
    }
    
    do {
	if (fscanf(results_fp, "%[^\n]", s) == EOF) {
	    perror(results_path);
	    return FILE_ERROR;
	}
	if (fscanf(results_fp, "%*[\n]") == EOF) {
	    perror(results_path);
	    return FILE_ERROR;
	}
	
    } while (strcmp(s,"THROUGHPUT RESULTS OF TIMED TRANSITIONS"));
    if (fscanf(results_fp, "%*[^\n]%*[\n]") == EOF) {
	perror(results_path);
	return FILE_ERROR;
    }

    for (i = FirstTrans(nw); i < LastTrans(nw); i++) {
	if (fscanf(results_fp, "%lf", &throughput) != 1) {
	    perror(results_path);
	    return FILE_ERROR;
	}
	XtVaSetValues(nw->composite.children[i],
		  XtNthroughput,&throughput,
		  NULL);
    }
  
  fclose(results_fp);
  
  SetComputeThroughputFlag(nw,True);

  return RESULTS_COLLECTED;
}


/*******************************************************************************
Name:
	ShowArcDefinition
--------------------------------------------------------------------------------
Funktionstyp:
	Aktion
--------------------------------------------------------------------------------
triggert bei event:
	aufgerufen von ShowResults()
--------------------------------------------------------------------------------
Zweck:
	Anzeige der Multiplizitaetsdefinition
	
*******************************************************************************/

static void ShowArcDefinition(w,event,params,numparams)
Widget w;
XButtonEvent* event;
String* params;
Cardinal* numparams;
{
    NetWidget nw = (NetWidget)w;
    String fromName, toName;
    char titel[128];
    Multiplicity *multiplicity;
    ArcType arctype;
    Widget from, to;
    NWshowArcInfo info;
    
#ifdef debug
    fprintf(stderr, "ShowArcDefinition\n");
#endif
    XtVaGetValues(nw->netWidget.selectedObject,
		  XtNfromObj, &from,
		  XtNtoObj, &to,
		  XtNarcType, &arctype,
		  XtNmultiplicity, &multiplicity,
		  NULL);

    XtVaGetValues(from,
		  XtNtagName, &fromName,
		  NULL);
    XtVaGetValues(to,
		  XtNtagName, &toName,
		  NULL);
    
    switch  (arctype) {
    case INPUTARC:
	if (isPlaceObject(from))
	    sprintf(titel, "Input arc from place %s to transition %s",
		    fromName, toName);
	else 
	    sprintf(titel, "Input arc from place %s to transition %s",
		    toName, fromName);
	break;
    case OUTPUTARC:
	if (isPlaceObject(from))
	    sprintf(titel, "Output arc from transition %s to place %s",
		    toName, fromName);
	else 
	    sprintf(titel, "Output arc from transition %s to place %s",
		    fromName, toName);
	break;
    case INHIBITORARC:
	if (isPlaceObject(from))
	    sprintf(titel, "Inhibitor arc from place %s to transition %s",
		    fromName, toName);
	else 
	    sprintf(titel, "Inhibitor arc from place %s to transition %s",
		    toName, fromName);
	break;
    }

    info.titel = XtNewString(titel);
    
    /* make an unique name */
    sprintf(titel, "%s%s%s", 
	    (arctype == INPUTARC ? "Inputarc" :
	     (arctype == OUTPUTARC ? "Outputarc" : "Inhibitorarc")),
	    fromName,
	    toName);
    info.arc_name = XtNewString(titel);

    if (multiplicity->type == ARCVALUE) {
	info.text = XtNewString
	    ("fixed multiplicity:                                          ");
	sprintf(info.text, 
		"fixed arc multiplicity: %d", multiplicity->values.arcvalue);
    }
    else /* MD_DEF */ {
	info.text = XtNewString("marking dependent arc multiplicity:\n\n");
	vlstrcat(&info.text, multiplicity->values.md_definition);
    }
    
    info.e_x = event->x;
    info.e_y = event->y;
    
    XtCallCallbacks(w,XtNarcShowCallback,(XtPointer)&info);
}













