void spline


void arcTo(nw NetWidget,
	   double ax0, double ay0, 
	   double ax1, double ay1, 
	   double ax2, double ay2, 
	   double arad)
{	
    float xt0, yt0, xt2, yt2;
    
    if ( arad < 0 ) {
	XtWarning("Arcto with rad < 0. Take 0.");
	arad = 0.0;
    }

    {	/* Now we have to compute the tangent points. */
	/* Basically, the idea is to compute the tangent */
	/* of the bisector by using tan(x+y) and tan(z/2) */
	/* formulas, without ever using any trig. */
	float dx0 = ax0 - ax1, dy0 = ay0 - ay1;
	float dx2 = ax2 - ax1, dy2 = ay2 - ay1;
	
	/* Compute the squared lengths from p1 to p0 and p2. */
	double sql0 = dx0 * dx0 + dy0 * dy0;
	double sql2 = dx2 * dx2 + dy2 * dy2;
	
	/* Compute the distance from p1 to the tangent points. */
	/* This is the only hairy part. */
	double num = dy0 * dx2 - dy2 * dx0;
	double denom = sqrt(sql0 * sql2) - (dx0 * dx2 + dy0 * dy2);
	
	/* Check for collinear points. */
	if ( fabs(num) < 1.0e-6 || fabs(denom) < 1.0e-6 ) {	
	    drawLine (nw, (int)ax0, (int)ay0, (int)ax1, (int)ay1);
	    drawLine (nw, (int)ax1, (int)ay1, (int)ax2, (int)ay2);
	}
	else {	   /* not collinear */
	    double dist = fabs(arad * num / denom);
	    double l0 = dist / sqrt(sql0), l2 = dist / sqrt(sql2);
	    xt0 = ax1 + dx0 * l0;
	    yt0 = ay1 + dy0 * l0;
	    xt2 = ax1 + dx2 * l2;
	    yt2 = ay1 + dy2 * l2;
	    drawLine (nw, (int)ax0, (int)ay0, (int)xt0, (int)yt0);
	    setArcIn(arad, xt0, yt0, xt2, yt2, ax1, ay1);
	    drawLine (nw, (int)xt2, (int)yt2, (int)ax2, (int)ay2);
	}
    }
}

static void
setArcIn (nw NetWidget,
	  double r, 
	  double x0, double y0, 
	  double x3, double y3, 
	  double xt, double yt)
{	
    /* square length from final points of the arc to the point at which the 
       extended tangents meet */
    double sqlt = ( (x0-xt)*(x0-xt) + (y0-yt)*(y0-yt) );  
    /* length from tangents cross point to arc center */
    double sqlc = ( sqlt + r*r );
    /* middle point between final points of the arc */
    double xm = (x0+x3) / 2.0, ym = (y0+y3) / 2.0;
    /* length from tangents cross point to middle point */
    double sqlm = ( (xm-xt)*(xm-xt) + (ym-yt)*(ym-yt) );
    double xc = ( xt + (xm-xt) * sqlc / sqlm ),
           yc = ( yt + (ym-yt) * sqlc / sqlm );
    /* angles from center to final points of the arc */
    int angle0 = (int)( atan2( x0-xc, y0-yc ) / 180 / 64 );
    int angle1 = (int)( atan2( x1-xc, y1-yc ) / 180 / 64 );
    
    drawArc(nw, (int)xc, (int)yc,  r, r, angle1, angle2);
}

