/*******************************************************************************
Modul           :       NWTokenGame

Purpose         :       interaktive Simulation des Petri-Netzes

Project Director:       Christoph Lindemann TU Berlin

Programmer      :       Christian L"uhe

History         :       release 1: November 1992
*******************************************************************************/

#define  __NEW_BY_JOA__  /* alles ausser: MDF <-> TN */

#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <errno.h>
#include <sys/param.h>
#include <signal.h>
#include <unistd.h>
#include <stdlib.h>

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include "BaseObj.h"
#include "MarkParObj.h"
#include "RateParObj.h"
#include "PlaceObj.h"
#include "TransObj.h"
#include "ArcObj.h"

#include "NetWidgetP.h"

#include "fgetline.h"

/*============================================================ declaration */

extern XtAppContext app_context;

#ifdef __STDC__

extern void showEnabledTranss(NetWidget nw);
extern void replaceInfoString(void);
extern void FlashNet(NetWidget nw);
extern void setWaitCursor(void);
extern void setNormalCursor(void);
extern int getRidOf (int fd, char* s);
extern char *vlstrcat(char **dst, char *src);

void TokenGame(NetWidget nw, Boolean state);
void CallTokenGameMethod(NetWidget nw, XButtonEvent *event, 
			 String *params, Cardinal numparams);

void initializeMarking(NetWidget nw);
void computeActualMultiplicity(NetWidget nw);
void computeEnabledTranss(NetWidget nw);
void EndGame(NetWidget nw);
int getMDC(NetWidget nw, Widget arc);
static void fireTrans(NetWidget nw, TransObj tw);
int MakeMDC(NetWidget nw);
void updateTokenGame (XtPointer w, XtIntervalId *id);
#ifdef UNDO
void histTokenGameState (NetWidget nw);
#endif UNDO

#else

extern void showEnabledTranss();
extern void replaceInfoString();
extern void FlashNet();
extern void setWaitCursor();
extern void setNormalCursor();
extern int getRidOf ();
extern char *vlstrcat();

void TokenGame();
void CallTokenGameMethod();

void initializeMarking();
void computeActualMultiplicity();
void computeEnabledTranss();
void EndGame();
int getMDC();
static void fireTrans();
int MakeMDC();
void updateTokenGame();
#ifdef UNDO
void histTokenGameState ();
#endif UNDO

#endif

/*=========================================================== implementation */

/******************************************************************************
 TokenGame
 initialisiert das NetWidget fuer das Tokengame
 *****************************************************************************/
#ifdef __STDC__
void TokenGame(NetWidget nw, Boolean state)
#else
void TokenGame(nw, state)
NetWidget nw;
Boolean state;
#endif
{

  nw->netWidget.tokenGame = state;

  if (state) {

      /* it can take a while ... set cursor to clock */
      setWaitCursor();

      /* */
      initializeMarking(nw);

      /* start the process for getting the actual mdep cardinality of an arc */
      if (start_MDC(nw)) {
	  setNormalCursor();
	  EndGame(nw);
	  return;
      }

      computeEnabledTranss(nw);

      /* clear the actual picture-map and draw the Net new into it */
      /* replace marking-parameters by there values */
      /* replace mdep-multiplicities by there values */
      ShowNet(nw);

      /* make a copy of the picture-map for highlighting enabled trans. */
      CopyToFlash(nw);

      /* highlight enabled trans. in flash-pixmap */
      showEnabledTranss(nw);

      nw->netWidget.flashed=False;

      /* timer for blinking */
      nw->netWidget.timer=XtAppAddTimeOut(app_context,500,updateTokenGame,
					  (XtPointer)nw);
      
      /* set cursor normal */
      setNormalCursor();
  }
  else {

      /* nothing shall blink */
      XtRemoveTimeOut(nw->netWidget.timer);

      setWaitCursor();

      /* clear the actual picture-map and draw the Net new into it */
      ShowNet(nw);

      /* kill the MDC process */
      kill_MDC(nw);

      /* set cursor normal */
      setNormalCursor();
  }

  /* set InfoString so the user know what to do */
  replaceInfoString();

  /* copy the picture-map to display */
  Redisplay((Widget)nw,NULL,NULL);
}

/*****************************************************************************/
#ifdef __STDC__
void EndGame(NetWidget nw)
#else
void EndGame(nw)
    NetWidget nw;
#endif
{
    nw->netWidget.tokenGame = False;
    
    setWaitCursor();

    kill_MDC(nw);
    
    replaceInfoString();
    ShowNet(nw);
    Redisplay((Widget)nw,NULL,NULL);

    setNormalCursor();
}

/******************************************************************************
  start_MDC
  erzeugt und started den <net>_MDC prozess
******************************************************************************/
#ifdef __STDC__
int start_MDC(NetWidget nw)
#else
int start_MDC(nw)
    NetWidget nw;
#endif
{
    String netName = nw->netWidget.model_name;
    int numParams = 1;

    /* save if needed */
    if (nw->netWidget.net_modified) 
	Save(nw, NULL, &netName, &numParams);

    if (MakeMDC(nw))
	return -1;
    
    /* -------------------------------start the <net>_MDC processs */
{
    int to[2], from[2];
    int pid;
    char path[MAXPATHLEN];
    char *argv[2];
    
    /* close the pipe if open */
    if (nw->netWidget.readMDC)
	fclose(nw->netWidget.readMDC);
    if (nw->netWidget.writeMDC)
	fclose(nw->netWidget.writeMDC);
    nw->netWidget.readMDC = nw->netWidget.writeMDC = NULL;
      
    if (pipe(to) || pipe(from)) {
	perror("ERROR in pipe() ");
	return -1;
    }
    
    sprintf(path, "%s/%s_MDC", 
	    getenv ("MODELDIR"), nw->netWidget.model_name);
    argv[0] = path;
    argv[1] = NULL;
    
    switch (pid=fork()) {
    case -1: /* error */
	
	perror("ERROR in fork() ");
	return -1;
	break;
	
    case 0:  /* child */

	close (fileno(stdin));               /* input from DSPN */
	dup (to[0]);
	close (fileno(stdout));              /* output to  DSPN */
	dup (from[1]);

	execve(path,argv,NULL);              /* und TSCHUESS */
	fprintf(stderr, "file %s : ", argv[0]);
	perror("ERROR in execve() ");
	exit(-1);

	break;
	  
    default: /* parent */
	
	if (!(nw->netWidget.readMDC = fdopen (from[0], "r")) || 
	    !(nw->netWidget.writeMDC = fdopen (to[1], "w"))) {
	    perror("ERROR in fdopen() ");
	    return -1;
	}

	close (from[1]); /* reading the other side will never ...   */
	close (to[0]); /* return EOF otherwise, because open fd's */
	break;
    } /* switch */
    
    nw->netWidget.MDCpid = pid;
}
return 0;
}

/******************************************************************************
  kill_MDC
  beendet den <net>_MDC prozess
******************************************************************************/
#ifdef __STDC__
int kill_MDC(NetWidget nw)
#else
int kill_MDC(nw)
    NetWidget nw;
#endif
{
    int status;

    if (!nw || nw->netWidget.MDCpid == 0)
	return 0;
    
    if (nw->netWidget.writeMDC)
	fclose(nw->netWidget.writeMDC);
    if (nw->netWidget.readMDC)
	fclose(nw->netWidget.readMDC);
    nw->netWidget.readMDC = nw->netWidget.writeMDC = NULL;
    
    kill (nw->netWidget.MDCpid,SIGQUIT);
    waitpid(nw->netWidget.MDCpid,&status,0);

    nw->netWidget.MDCpid = 0;
    return 0;
}

/******************************************************************************
 CallTokenGameMethod
 ruft die zu Object und Button gehoerige Funktion auf
******************************************************************************/
#ifdef __STDC__
void CallTokenGameMethod (NetWidget nw, XButtonEvent *event, 
			 String *params, Cardinal numparams)
#else
void CallTokenGameMethod (nw, event, params, numparams)
NetWidget nw;
XButtonEvent* event;
String* params;
Cardinal numparams;
#endif
{
    int childInd;
    Commands command;

    childInd =  lookUp((Widget)nw,event->x,event->y,SELECTION);

    if (childInd == NothingFound)
	return;

    if (event->button != Button3)
	return;

    if (isTransObject(nw->composite.children[childInd])) 
    {
	XtRemoveTimeOut(nw->netWidget.timer);
	fireTrans(nw, (TransObj)nw->composite.children[childInd]); 
	computeEnabledTranss(nw);
	ShowNet(nw);
	CopyToFlash(nw);
	showEnabledTranss(nw);
	nw->netWidget.flashed=False;
	nw->netWidget.timer=XtAppAddTimeOut(app_context,500,updateTokenGame,
					    (XtPointer)nw);
	Redisplay((Widget)nw,NULL,NULL);
	
	/* following is only for info-string */
	XtVaGetValues((Widget)nw,
		      XtNCommands, &command,
		      NULL);
	if (command==NOOPERATION)
	    XtVaSetValues((Widget)nw,
			  XtNCommands, (int)FIRE,
			  NULL);
    }
}

/******************************************************************************
  initializeMarking
  initialisiert die actualMarking Ressource 
******************************************************************************/
#ifdef __STDC__
void initializeMarking(NetWidget nw)
#else
void initializeMarking(nw)
NetWidget nw;
#endif
{
    Widget w;
    int p;
    InitialMarking *M0;
    Cardinal value;

    for (p = FirstPlace(nw); p < LastPlace(nw); p++) {
	w = nw->composite.children[p];
	XtVaGetValues(w,
		      XtNinitialMarking, &M0,
		      NULL);
	if (M0->type == MARKINGVALUE)
	    XtVaSetValues(w, 
			  XtNactualMarking, M0->values.markingValue,
			  NULL);
	else {
	    XtVaGetValues((Widget)M0->values.markingParam,
			  XtNmparValue, &value,
			  NULL);
	    XtVaSetValues(w, 
			  XtNactualMarking, value,
			  NULL);
	}
    }
}


/******************************************************************************
 computeEnabledTranss
 berechnet die durch die aktuellen Markenwerte schaltfaehigen Transitionen
******************************************************************************/
#ifdef __STDC__
void computeEnabledTranss(NetWidget nw)
#else
void computeEnabledTranss(nw)
NetWidget nw;
#endif
{
    Widget transition;
    TransType ttype;
    Cardinal marking;
    Cardinal priority, max_priority=LOWEST_PRI;
    Boolean enabled;
    int t,c;                         /* Transition- and Connection-Offset */
    Cardinal numInputs, numInhibits;
    Connections input, inhibit;

    nw->netWidget.vanish=False;            /* set it initialy tangible */

    /* get the actual mdep. mutliplicity of the arcs */
    computeActualMultiplicity(nw);

    /* for all transitions ... */
    for (t=FirstTrans(nw); t<LastTrans(nw); t++) {
	transition = nw->composite.children[t];
	XtVaSetValues(transition,
		      XtNenabled, True,    /* set it initialy enabled */
		      NULL);
	XtVaGetValues(transition,
		      XtNnumInputs, &numInputs,
		      XtNnumInhibits, &numInhibits,
		      XtNinpConnection, &input,
		      XtNinhibitConnection, &inhibit,
		      NULL);

	/* CHECK THE INPUTCONNECTIONS */
	for (c=0; c < numInputs; c++) {
	    XtVaGetValues((Widget)input[c].place,
			  XtNactualMarking, &marking,
			  NULL);
	    if (marking < input[c].multiplicity->actualValue 
		/* || input[c].multiplicity->actualValue == 0 */)
		XtVaSetValues((Widget)transition,
			      XtNenabled, False,
			      NULL);
	}
	/* CHECK THE INHIBITCONNECTIONS */
	for (c=0; c < numInhibits; c++) {
	    XtVaGetValues((Widget)inhibit[c].place,
			  XtNactualMarking, &marking,
			  NULL);
	    if (inhibit[c].multiplicity->actualValue != 0 &&
		marking >= inhibit[c].multiplicity->actualValue)
		XtVaSetValues((Widget)transition,
			      XtNenabled, False,
			      NULL);
	}
	XtVaGetValues ((Widget)transition,
		       XtNenabled, &enabled,
		       XtNtransType, &ttype,
		       XtNpriority, &priority,
		       NULL);
	if (enabled && ttype==IMMEDIATE)
	    nw->netWidget.vanish=True;       /* now the marking is vanish */
    } /* end for all transitions */

    /* if marking is vanish disable all NOT IMMEDIATE transitions
       if marking is vanish take the prioryties of the immediate transitions
       into account */
    if (nw->netWidget.vanish) {
	max_priority=LOWEST_PRI;
	/* get the highest immediate && enabled Priority */
        for (t=FirstTrans(nw); t < LastTrans(nw); t++) {
            XtVaGetValues(nw->composite.children[t],
                          XtNtransType, &ttype,
                          XtNenabled, &enabled,
                          XtNpriority, &priority,
                          NULL);
            if (ttype==IMMEDIATE && enabled)
                max_priority = max_priority<priority ? priority : max_priority;
        }
	/* set all transs lower then the highest priority disabled */
        for (t=FirstTrans(nw); t < LastTrans(nw); t++) {
            XtVaGetValues(nw->composite.children[t],
                          XtNtransType, &ttype,
                          XtNenabled, &enabled,
                          XtNpriority, &priority,
                          NULL);
            if (ttype != IMMEDIATE || priority<max_priority)
                XtVaSetValues(nw->composite.children[t],
                              XtNenabled, False,
                              NULL);
            }
    } /* if vanish */
} /* computeEnabledTranss */

/******************************************************************************
  computeActualMultiplicity
  berechnet die, durch die aktuelle markierung gegebenen, 
  Kanten-Multiplizitaeten
******************************************************************************/
#ifdef __STDC__
void computeActualMultiplicity(NetWidget nw)
#else
void computeActualMultiplicity(nw)
    NetWidget nw;
#endif
{
    Widget arc;
    int a, actM;
    Multiplicity *m;
    
    /* for all arcs ... */
    for (a=FirstArc(nw); a<LastArc(nw); a++) {
	arc = nw->composite.children[a];
	XtVaGetValues(arc,
		      XtNmultiplicity, &m,
		      NULL);

	if (m->type == ARCVALUE)
	    actM = m->values.arcvalue;
	else 
	    if ((actM = getMDC(nw, arc)) < 0) {
		fprintf(stderr, "ERROR in getMDC().\n");
		actM = 0;
	    }

	if (m->actualValue != actM) {
	    DrawArc(arc, 
		    XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
		    nw->netWidget.object_size);
	    m->actualValue = actM;
	    DrawArc(arc,  
		    XtDisplay(nw),nw->netWidget.picture,nw->netWidget.draw_GC,
		    nw->netWidget.object_size);   
	}
    }
}

#ifdef __STDC__
int getMDC(NetWidget nw, Widget arc)
#else
int getMDC(nw, arc)
    NetWidget nw;
    Widget arc;
#endif
{
    Widget fromObj, toObj;
    ArcType arcType;
    String transName, placeName, line;
    char s[1024];
    int p, m;
    int fd = fileno(nw->netWidget.writeMDC);
    size_t l;

    extern char *ArcTypeStrDEF[3];

    XtVaGetValues(arc,
		  XtNfromObj, &fromObj,
		  XtNtoObj, &toObj,
		  XtNarcType, &arcType,
		  NULL);
    
    if (isTransObject(fromObj)) {
	XtVaGetValues(fromObj,
		      XtNtagName, &transName,
		      NULL);
	XtVaGetValues(toObj,
		      XtNtagName, &placeName,
		      NULL);
    }
    else {
	XtVaGetValues(fromObj,
		      XtNtagName, &placeName,
		      NULL);
	XtVaGetValues(toObj,
		      XtNtagName, &transName,
		      NULL);
    }
    
    sprintf(s, "%s %s %s\n", 
	    ArcTypeStrDEF[arcType],
	    transName,
	    placeName);

    getRidOf(fd, s);
    
    for (p = FirstPlace(nw); p < LastPlace(nw); p++) {
	XtVaGetValues(nw->composite.children[p],
		      XtNactualMarking, &m,
		      NULL);
	sprintf(s, "%d\n", m);

	getRidOf(fd, s);
    }

    line = fgetline (nw->netWidget.readMDC,&l); 

    if (sscanf(line, "%d", &m) != 1) {
	fprintf(stderr, "ERROR reading on stream 'readMDC'.\n");
	perror("");
	return -1;
    }

    return m;
	
}

/******************************************************************************
 fireTrans
 berechnet den Folge-Zustand nach dem feuern einer Transition
******************************************************************************/
#ifdef __STDC__
static void fireTrans(NetWidget nw, TransObj tw)
#else
static void fireTrans(nw, tw)
    NetWidget nw;
    TransObj tw;
#endif
{
    Boolean enabled;
    TransType ttype;
    Cardinal arcvalue;
    Cardinal marking;
    Cardinal numInputs, numOutputs, c;
    Connections input, output;

    XtVaGetValues((Widget)tw,
		  XtNenabled, &enabled,
		  XtNtransType, &ttype,
		  XtNnumInputs, &numInputs,
		  XtNnumOutputs, &numOutputs,
		  XtNinpConnection, &input,
		  XtNoutpConnection, &output,
		  NULL);

    if (!enabled || (enabled && ttype!=IMMEDIATE && nw->netWidget.vanish))
	return;  

#ifdef UNDO
    histTokenGameState(nw);
#endif UNDO

    /* sub tokens from input */
    for (c=0; c < numInputs; c++) {
	XtVaGetValues((Widget)input[c].place,
		      XtNactualMarking, &marking,
		      NULL);

	arcvalue = input[c].multiplicity->actualValue;

	XtVaSetValues((Widget)input[c].place,
		      XtNactualMarking, marking-arcvalue,
		      NULL);
    }

    /* add tokens in output */
    for (c=0; c < numOutputs; c++) {
	XtVaGetValues((Widget)output[c].place,
		      XtNactualMarking, &marking,
		      NULL);

	arcvalue = output[c].multiplicity->actualValue;

	XtVaSetValues((Widget)output[c].place,
		      XtNactualMarking, marking+arcvalue,
		      NULL);
    }
} /* fireTrans */

/******************************************************************************
  updateTokenGame
  remake <net>_MDC prozess if needed
  recompute mdep. arc multiplicities
  recompute enabled transitions
  flash net
******************************************************************************/
#ifdef __STDC__
void updateTokenGame (XtPointer w, XtIntervalId *id)
#else
void updateTokenGame (w, id)
    XtPointer w;
    XtIntervalId *id;
#endif
{
    NetWidget nw = (NetWidget)w;
    String netName = nw->netWidget.model_name;
    int numParams = 1;
    int error = 0;
    
    if (nw->netWidget.net_modified) {  /* SAVE, remake <net>_MDC */
	setWaitCursor();
	Save (nw, NULL, &netName, &numParams);
	kill_MDC (nw);
	error = start_MDC (nw);
	setNormalCursor();
	if (error) {
	    EndGame(nw);
	    return;
	}
    }    

    if (nw->netWidget.flashed) {
	nw->netWidget.timer=XtAppAddTimeOut
	    (app_context,400, updateTokenGame, w);
    }
    else {
	computeEnabledTranss(nw);
	
	nw->netWidget.timer=XtAppAddTimeOut
	    (app_context,900, updateTokenGame, w);
    }

    FlashNet(nw);

    nw->netWidget.flashed=!nw->netWidget.flashed;
}

/******************************************************************************
******************************************************************************/ 
#ifdef __STDC__
int MakeMDC(NetWidget nw)
#else
int MakeMDC(nw)
    NetWidget nw;
#endif
{
    char command[MAXPATHLEN];
    char path[MAXPATHLEN];
    String netName = nw->netWidget.model_name;
    int error;
    struct stat buf;

#ifdef  __NEW_BY_JOA__

    sprintf(path, "%s/%s%s", getenv("MODELDIR"),netName, NET_EXTENSION);
    if (lstat(path, &buf)) {
	perror("lstat(<net>.TN)");
	return -1;
    }

#else   __NEW_BY_JOA__

    time_t NETtime, DEFtime;
    sprintf(path, "%s/%s.NET", getenv("MODELDIR"),netName);
    if (lstat(path, &buf)) {
	perror("lstat(<net>.NET)");
	return -1;
    }
    NETtime = buf.st_mtime;
    
    sprintf(path, "%s/%s.DEF", getenv("MODELDIR"),netName);
    if (lstat(path, &buf)) {
	perror("lstat(<net>.DEF)");
	return -1;
    }
    DEFtime = buf.st_mtime;
    
    /* ------------------------ test for <net>_MDC */
    sprintf(path, "%s/%s_MDC", getenv("MODELDIR"),netName);
    if (lstat(path, &buf)) 
	if (errno != ENOENT) { /* No such file or directory */
	    perror("lstat(<net>_MDC)"); 
	    return -1;
	}

    if (errno != ENOENT && 
	NETtime < buf.st_mtime && DEFtime < buf.st_mtime) 
	/* --------------- its uptodate */
	return 0;

    /* ------------------------ test for <net>_MDF.c */
    sprintf(path, "%s/%s_MDF.c", getenv("MODELDIR"),netName);
    if (lstat(path, &buf)) 
	if (errno != ENOENT) { /* No such file or directory */
	    perror("lstat(<net>_MDF.c)"); 
	    return -1;
	}

    if (errno == ENOENT || 
	NETtime > buf.st_mtime || DEFtime > buf.st_mtime) {
    }
#endif  __NEW_BY_JOA__

	/* ------------------ make <net>_MDC --------------- */
	sprintf(command, 
		"%s/Interface/Tokengame %s",getenv ("TNETHOME"),netName);
    
   	 if (error = system(command)) {
		fprintf(stderr, "ERROR: %s\nEXIT-STATUS: %d.\n", command, error);
		return -1;
	    }
    return 0;

}

#ifdef UNDO
/******************************************************************************
 setzt die history (fuer undo) 
******************************************************************************/
#ifdef __STDC__
void histTokenGameState (NetWidget nw)
#else
void histTokenGameState (nw)
     NetWidget nw;
#endif
{
    Cardinal i;
    Widget w;
    ObjectRecord rec;
    Arg arg;
    Cardinal num_args;
    Cardinal m;

    /* remove old history */
    clearHistory(nw);
    
    for (i=FirstPlace(nw); i<LastPlace(nw); i++)
    {
	XtVaGetValues(nw->composite.children[i],
		      XtNactualMarking, &m,
		      NULL);

	/* m is Cardinal -> no New(m) needed */
	arg.name = XtNactualMarking;
	arg.value = m;

	rec.object = nw->composite.children[i];
	rec.num_args = 1;
	rec.args = &arg;

	insToHistory(nw, &rec);
    }
} /* histTokenGameState */
#endif UNDO
