/*******************************************************************************
Modul   	:	NWUtilities.c

Purpose		: 	Bereitstellen von im Berich der Klasse Netwidget 
                        globalen Hilfsfunktionen

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/


/*
 * NetWidget.c - Manages all objects of a PN.

   Class Hierarchie:
   Core -> Composite -> NetWidget
 */

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include <stdio.h>
#include <string.h>

#include "BaseObj.h"
#include "MarkParObj.h"
#include "RateParObj.h"
#include "PlaceObj.h"
#include "TransObj.h"
#include "ArcObj.h"

#include "NetWidgetP.h"

extern Widget getCurrentNet();

/* the following are private functions unique to NetWidget */

/* Private functions */
/*******************************************************************************
Name:
        getArc
--------------------------------------------------------------------------------
Funktionstyp:
        private Funktion im Modulbereich der Klasse NetWidget
--------------------------------------------------------------------------------
Parameter:
        w: Instanz der Klasse NetWidget
	from,to: Objektinstanzen
	arcType: Kantentyp
--------------------------------------------------------------------------------
Rueckgabewert:
        Instanz der gefundenen Kante
--------------------------------------------------------------------------------
Zweck:
        Liefert eine Kante gleichen Typs von Objekt from zu Objekt to
*******************************************************************************/

Widget getArc(w,from,to,arcType)
Widget w,from,to;
ArcType arcType;

{
NetWidget nw = (NetWidget)w;
Widget fromObj,toObj;
int i;
ArcType at;

  for (i=FirstArc(nw);i<LastArc(nw);i++) {
	XtVaGetValues(nw->composite.children[i],
		      XtNfromObj,&fromObj,
		      XtNtoObj,&toObj,
		      XtNarcType,&at,
		      NULL);
	if(from==fromObj && to==toObj && at == arcType
	   || from==toObj && to==fromObj && at == arcType) 
	      return (Widget)nw->composite.children[i];
  }
  return NULL;
}
/*******************************************************************************
Name:
        deleteConnectedArcs
--------------------------------------------------------------------------------
Funktionstyp:
        private Funktion im Modulbereich der Klasse NetWidget
--------------------------------------------------------------------------------
Parameter:
        w: Instanz der Klasse netWidget
	obj: Objektinstanz
--------------------------------------------------------------------------------
Rueckgabewert:
        True: falls Kanten gel"oscht wurden, False sonst
--------------------------------------------------------------------------------
Zweck:
        L"oschen von Kanten, dien mit dem Objekt verbunden sind
*******************************************************************************/

Boolean deleteConnectedArcs(w,obj)
Widget w,obj;

{
NetWidget nw = (NetWidget)w;
ArcObject arc;
Widget from,to;
Boolean found = False;
int i;

  for (i=FirstArc(nw);i<LastArc(nw);i++) {
	arc = (ArcObject)nw->composite.children[i];
	XtVaGetValues((Widget)arc,
		      XtNfromObj,&from,
		      XtNtoObj,&to,
		      NULL);
	if(from==obj || to == obj) 
	{
	    DrawArc(arc,XtDisplay(nw),nw->netWidget.picture,
		   nw->netWidget.draw_GC,nw->netWidget.object_size);
	    XtDestroyWidget((Widget)arc);
	    /*
	    XtVaSetValues(arc,
			XtNdestroyed,True,
			NULL);
	    */
	    found = True;
	}
  }
return found;
}
/*******************************************************************************
Name:
        redrawArcs
--------------------------------------------------------------------------------
Funktionstyp:
        private Funktion im Modulbereich der Klasse NetWidget
--------------------------------------------------------------------------------
Parameter:
        w: instanz der Klasse NetWidget
	obj: Objektinstanz
	gc: graphischer Kontext
	e_x,e_y: Ereigniskoordinatan,auf denen Objekt sich befindet
--------------------------------------------------------------------------------
Zweck:
       Zeichnet mit Objekt verbundene Kanten neu 
*******************************************************************************/

void redrawArcs(w,obj,gc,e_x,e_y)
Widget w,obj;
GC gc;
int e_x,e_y;
{
NetWidget nw = (NetWidget)w;
ArcObject arc;
Widget from,to;
int i;
Cardinal num_children;

  for (i=FirstArc(nw);i<LastArc(nw);i++) {
	arc = (ArcObject)nw->composite.children[i];
	XtVaGetValues((Widget)arc,
		      XtNfromObj,&from,
		      XtNtoObj,&to,
		      XtNnumChildren,&num_children,
		      NULL);
	if (from==obj && to != obj) 
	{
	    DrawArc((Widget)arc, XtDisplay(nw), nw->netWidget.picture, gc,
		    nw->netWidget.object_size);
	}
	if (to==obj) 
	{
	    DrawArc((Widget)arc, XtDisplay(nw), nw->netWidget.picture, gc,
		    nw->netWidget.object_size);
	}
  }
}
/*******************************************************************************
Name:
        lookUp
--------------------------------------------------------------------------------
Funktionstyp:
        private Funktion im Modulbereich der Klasse NetWidget
--------------------------------------------------------------------------------
Parameter:
        w: Instans der Klasse netWidget
	x,y: Ereigniskoordinaten
	mode == PUT: ein Objekt soll platziert werden
	mode == SELECTION ein Objekt soll selektiert werden
--------------------------------------------------------------------------------
Rueckgabewert:
        index eines gefunden Objektes im Childrenarray
--------------------------------------------------------------------------------
Zweck:
        Selektieren eines Objektes, oder nachschauen ob Platz f"ur Objekt ist
*******************************************************************************/

int lookUp(w,x,y,mode)
Widget w;
int x,y,mode;

{
NetWidget nw=(NetWidget)w; 
int i;
int xpos,ypos,area;
int dummy1,ascent,descent;
XCharStruct  overall;
XFontStruct *font;
String text;
Boolean pointerOverText = False;

#ifdef debug
printf(" lookUp\n");
#endif

  area = nw->netWidget.object_size;
  font = XQueryFont(XtDisplay(nw),XGContextFromGC(nw->netWidget.draw_GC));
  for (i=0;i<nw->composite.num_children;i++) {
	if (FirstArc(nw)<=i && i<LastArc(nw)) 
		continue;
        if (FirstMarkPar(nw)<=i && i<LastMarkPar(nw)) {
	    text = getDisplayedMarkingString(nw->composite.children[i]);
	    XTextExtents(font,
				text,strlen(text),
				&dummy1,&ascent,&descent,&overall);
	    pointerOverText = True;
	    XtFree(text);
        }
        if(FirstRatePar(nw)<=i && i<LastRatePar(nw)) {
    	    text = getDisplayedRateString(nw->composite.children[i]);
	    XTextExtents(font,
				text,strlen(text),
				&dummy1,&ascent,&descent,&overall);
	    pointerOverText = True;
	    XtFree(text);
        }
        XtVaGetValues(nw->composite.children[i],
		  XtNxpos,&xpos,
		  XtNypos,&ypos,
		  NULL);
        switch(mode) {
        case PUT:
	    if (pointerOverText) {
	        if (   xpos-area <= x && x<= xpos+overall.width
		    && ypos-(ascent+area) <= y && y<= ypos+descent)
		{
	    		    XFreeFontInfo(NULL,font,0);
			    return i;
		}
	    }
	    else {
	        if(abs(x-(int)xpos)<= area && abs(y-(int)ypos) <= area)
		{
	    		XFreeFontInfo(NULL,font,0);
			return i;
		}
	    }
	    break;
        case SELECTION:
	    if (pointerOverText) 
	    {
	        if (    xpos<=x && x<=xpos+overall.width 
		     && ypos-ascent<=y && y<=ypos+descent)
		{
	    		XFreeFontInfo(NULL,font,0);
			return i;
		}
	    }
	    else 
	    {
	    	if (xpos-area/2<=x && x<=xpos+area/2 
		    && ypos-area/2<=y && y<=ypos+area/2)
	    	{
	    		XFreeFontInfo(NULL,font,0);
			return i;
	    	}
	    }
	    break;
        default: break;
        }
  }/* for*/
  XFreeFontInfo(NULL,font,0);
  return NothingFound;
} /*lookUp*/

/*******************************************************************************
Name:
       getObjName 
--------------------------------------------------------------------------------
Funktionstyp:
       private Funktion im Modulbereich der Klasse NetWidget
--------------------------------------------------------------------------------
Rueckgabewert:
       Defaultobejektname
--------------------------------------------------------------------------------
Zweck:
       Ermittelt   Defaultobejektname
*******************************************************************************/

String getObjName()

{
    static numObj = 0;
    char name[10];
    
    sprintf(name,"O%d",numObj++);
    return XtNewString(name);
}

/*******************************************************************************
Name:
       DrawNet 
--------------------------------------------------------------------------------
Funktionstyp:
        private Funktion im Modulbereich der Klasse NetWidget
--------------------------------------------------------------------------------
Parameter:
        w: Instanz der Klasse NetWidget
	mode==NORMAL ganzes Modell zeichnen
	mode==TAGS_ONLY: nur Etiketten zeichnen
	mode== RATES_ONLY: nur Raten zeichen
--------------------------------------------------------------------------------
Zweck:
        Zeichnen eines ganzen Modelles
*******************************************************************************/

void DrawNet(w,mode)
Widget w;
int mode;

{
NetWidget nw = (NetWidget)w;
GC gc;
int i,x,y;

  if (mode == CLEAR)
	gc = nw->netWidget.clear_GC;
  else
	gc = nw->netWidget.draw_GC;

  for (i=FirstPlace(nw);i<LastPlace(nw);i++) {
	XtVaGetValues(nw->composite.children[i],
			XtNxpos,&x,
			XtNypos,&y,
			NULL);
	if (mode != TAGS_ONLY && mode != RATES_ONLY)
		Draw(PLACE,nw->composite.children[i],
			XtDisplay(nw),nw->netWidget.picture,gc,
			x,
			y,
			nw->netWidget.object_size,0);
	if (mode != RATES_ONLY &&(nw->netWidget.tagsVisible 
				  || mode == TAGS_ONLY))
	    DrawTag(nw->composite.children[i],
		    XtDisplay(nw),nw->netWidget.picture,gc);
    }

  for (i=FirstTrans(nw);i<LastTrans(nw);i++) {
	XtVaGetValues(nw->composite.children[i],
			XtNxpos,&x,
			XtNypos,&y,
			NULL);

	if (mode != RATES_ONLY && mode != TAGS_ONLY)
	    Draw(IMTRANS,nw->composite.children[i],
		     XtDisplay(nw),nw->netWidget.picture,gc,
		     x,
		     y,
		     nw->netWidget.object_size,0);

	if (mode != RATES_ONLY &&(nw->netWidget.tagsVisible 
				  || mode == TAGS_ONLY))
	    DrawTag(nw->composite.children[i],
		    XtDisplay(nw),nw->netWidget.picture,gc);

  	if (mode != TAGS_ONLY && (nw->netWidget.ratesVisible 
				  || mode == RATES_ONLY))
  	    DrawRate(nw->composite.children[i],
		     XtDisplay(nw),nw->netWidget.picture,gc);
  }
  if (mode != RATES_ONLY && mode != TAGS_ONLY)
  {
  	for (i=FirstArc(nw);i<LastArc(nw);i++) 
	{
	    DrawArc(nw->composite.children[i],
		XtDisplay(nw),nw->netWidget.picture,gc,
		nw->netWidget.object_size);
  	}
  	for (i=FirstMarkPar(nw);i<LastMarkPar(nw);i++) 
	{
	    XtVaGetValues(nw->composite.children[i],
			XtNxpos,&x,
			XtNypos,&y,
			NULL);
	    Draw(MARKPAR,nw->composite.children[i],
		XtDisplay(nw),nw->netWidget.picture,gc,
		x,
		y,
		nw->netWidget.object_size,0);
  	}
  	for (i=FirstRatePar(nw);i<LastRatePar(nw);i++) 
	{
	    XtVaGetValues(nw->composite.children[i],
			XtNxpos,&x,
			XtNypos,&y,
			NULL);
	    Draw(RATEPAR,nw->composite.children[i],
		XtDisplay(nw),nw->netWidget.picture,gc,
		x,
		y,
		nw->netWidget.object_size,0);
  	}
  }
}


/*******************************************************************************
Name:
       parameterAvailabel 
--------------------------------------------------------------------------------
Funktionstyp:
        "offentliche Funktion 
--------------------------------------------------------------------------------
Parameter:
        w: Instanz der Klasse NetWidget
	name: Name des Parameters
	type: Typ des Parameters
--------------------------------------------------------------------------------
Zweck:
        Findet heraus, ob es einen Parameter des Namens gibt.
*******************************************************************************/

Boolean parameterAvailable(w,name,type)
Widget w;
String name;
Objects type;

{
NetWidget nw = (NetWidget)w;
Cardinal i;
String p_name;

switch (type)
  {
  case RATEPAR:
    for(i=FirstRatePar(nw);i<LastRatePar(nw);i++) 
      {
	XtVaGetValues(nw->composite.children[i],
		      XtNrateParName,&p_name,
		      NULL);
	if (!strcmp(name,p_name)) 
	    return True;
      }
    return False;
  case MARKPAR:
    for(i=FirstMarkPar(nw);i<LastMarkPar(nw);i++) 
      {
	XtVaGetValues(nw->composite.children[i],
		      XtNmparName,&p_name,
		      NULL);
	if (!strcmp(name,p_name)) 
	    return True;
      }
    return False;
  default:
    return False;
  }
}

String nwUniquePlaceName()
{
    NetWidget nw = (NetWidget)getCurrentNet();
    Cardinal nr = nw->netWidget.numplace;
    char name[128];
    String tag;
    Boolean unique;
    int p;

    do {
	nr++;
	sprintf(name, "P%u", nr);
	unique = True;
	for (p = FirstPlace(nw); p < LastPlace(nw); p++) {
	    XtVaGetValues(nw->composite.children[p],
			  XtNtagName, &tag,
			  NULL);
	    if (!strcmp(tag, name)) {
		unique = False;
		break;
	    }
	}
    } while (!unique);
    return XtNewString(name);
}

#ifdef __STDC__
String nwUniqueTransitionName(TransType type)
#else
String nwUniqueTransitionName(type)
     TransType type;
#endif
{
    NetWidget nw = (NetWidget)getCurrentNet();
    Cardinal nr = nw->netWidget.numtrans;
    char name[128];
    String tag;
    Boolean unique;
    int t;

    do {
	nr++;
	sprintf(name, "%c%u", type == IMMEDIATE ? 't' : 'T', nr);
	unique = True;
	for (t = FirstTrans(nw); t < LastTrans(nw); t++) {
	    XtVaGetValues(nw->composite.children[t],
			  XtNtagName, &tag,
			  NULL);
	    if (!strcmp(tag, name)) {
		unique = False;
		break;
	    }
	}
    } while (!unique);
    return XtNewString(name);
}
