/*******************************************************************************
Modul   	:	NetWidget.c

Purpose		: 	Realisation der Klasse NetWidget, Klassenstruktur,
                        Methoden, Resourcendefinition, Aktionsrekord, 
			"Ubersetzungstafel

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/
/*  
 * NetWidget.c - Manages all objects of a PN.

   Class Hierarchie:
   Core -> Composite -> NetWidget
 */

#define _NWMAIN_

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/Core.h>
#include <X11/CoreP.h>

#include <stdio.h>
#include <string.h>

#include "BaseObj.h"
#include "MarkParObj.h"
#include "RateParObj.h"
#include "ResultObject.h"
#include "PlaceObj.h"
#include "TransObj.h"
#include "ArcObj.h"
#include "Polygon.h"

#include "NetWidgetP.h"
#include "bitmaps/schrafiert.bm"

#define offset(field) XtOffsetOf(NetWidgetRec, field)

#ifdef __STDC__
extern void CopyToDisplay(NetWidget nw, Pixmap map, 
			  int x, int y, int widht, int height);
extern void FlashNet(NetWidget nw);
extern void undrawSelectionBorder(NetWidget nw);

#else

extern void CopyToDisplay();
extern void FlashNet();
#endif

extern void setDisplayedNetType(); /* etwas deplaziert, ich weiss... */
extern int GetTimeMode();         /* aus evalMenue.c*/
static XtResource resources[] = {
	{
	XtNmodelName,
	XtCmodelName,
	XtRString,
	sizeof(String),
	offset(netWidget.model_name),
	XtRPointer,
	NULL,
	},
	{
	XtNCommands,
	XtCCommands,
	XtRInt,
	sizeof(int),
	offset(netWidget.command),
	XtRImmediate,
	(XtPointer)NOOPERATION,
	},
	{
	XtNObjects,
	XtCObjects,
	XtRInt,
	sizeof(int),
	offset(netWidget.object),
	XtRImmediate,
	(XtPointer)NOOBJECT,
	},
	{
	XtNobjectSize,
	XtCobjectSize,
	XtRInt,
	sizeof(int),
	offset(netWidget.object_size),
	XtRImmediate,
	(XtPointer)DEFAULT_SIZE,
	},
	{
	XtNtextCallback,
	XtCtextCallback,
	XtRCallback,
	sizeof(XtCallbackList),
	offset(netWidget.textCallback),
	XtRPointer,
	NULL,
	},
	{
	XtNinfoCallback,
	XtCinfoCallback,
	XtRCallback,
	sizeof(XtCallbackList),
	offset(netWidget.infoCallback),
	XtRPointer,
	NULL,
	},
	{
	XtNtokenDistributionCallback,
	XtCtokenDistributionCallback,
	XtRCallback,
	sizeof(XtCallbackList),
	offset(netWidget.tokenDistributionCallback),
	XtRPointer,
	NULL,
	},
	{
	XtNtransResultCallback,
	XtCtransResultCallback,
	XtRCallback,
	sizeof(XtCallbackList),
	offset(netWidget.transResultCallback),
	XtRPointer,
	NULL,
	},
	{
	XtNarcShowCallback,
	XtCarcShowCallback,
	XtRCallback,
	sizeof(XtCallbackList),
	offset(netWidget.arcShowCallback),
	XtRPointer,
	NULL,
	},
	{
	XtNratesVisible,
	XtCratesVisible,
	XtRBoolean,
	sizeof(Boolean),
	offset(netWidget.ratesVisible),
	XtRImmediate,
	(XtPointer)False,
	},
	{
	XtNpictureModified,
	XtCpictureModified,
	XtRBoolean,
	sizeof(Boolean),
	offset(netWidget.picture_modified),
	XtRImmediate,
	(XtPointer)False,
	},
	{
	XtNnetModified,
	XtCNetModified,
	XtRBoolean,
	sizeof(Boolean),
	offset(netWidget.net_modified),
	XtRImmediate,
	(XtPointer)False,
	},
	{
	XtNnetType,
	XtCNetType,
	XtRInt,
	sizeof(int),
	offset(netWidget.net_type),
	XtRImmediate,
	(XtPointer)TN_GSPN,
	},
	{
	XtNtagsVisible,
	XtCtagsVisible,
	XtRBoolean,
	sizeof(Boolean),
	offset(netWidget.tagsVisible),
	XtRImmediate,
	(XtPointer)False,
	},
	{
	XtNspline,
	XtCspline,
	XtRBoolean,
	sizeof(Boolean),
	offset(netWidget.spline),
	XtRImmediate,
	(XtPointer)False,
	},
	{
	XtNsolutionExpected,
	XtCsolutionExpected,
	XtRBoolean,
	sizeof(Boolean),
	offset(netWidget.solution_expected),
	XtRImmediate,
	(XtPointer)False,
	},
	{
	XtNgridsize,
	XtCgridsize,
	XtRgridsize,
	sizeof(Gridsize),
	offset(netWidget.gridsize),
	XtRImmediate,
	(XtPointer)Off,
	},
	{
	XtNrescaleFactor,
	XtCrescaleFactor,
	XtRrescaleFactor,
	sizeof(RescaleFactor),
	offset(netWidget.rescale_factor),
	XtRImmediate,
	(XtPointer)OneZero,
	},
	{
        XtNprecision,
        XtCprecision,
        XtRCardinal,
        sizeof(Cardinal),
        offset(netWidget.precision),
	XtRImmediate,
	(XtPointer)6,
	},
	{
        XtNtokenGame,
        XtCTokenGame,
        XtRBoolean,
        sizeof(Boolean),
        offset(netWidget.tokenGame),
	XtRImmediate,
	(XtPointer)False,
        },
	{
        XtNreadOnly,
        XtCreadOnly,
        XtRBoolean,
        sizeof(Boolean),
        offset(netWidget.read_only),
	XtRImmediate,
	(XtPointer)False,
        },
        {
        XtNselectionBorder,
	XtCSelectionBorder,
	XtRPolygon,
	sizeof(Polygon *),
	offset(netWidget.selectionBorder),
	XtRImmediate,
	(XtPointer)NULL,
        },
	{
        XtNNWforeground,
        XtCNWforeground,
        XtRPixel,
        sizeof(Pixel),
        offset(netWidget.foreground),
	XtRString,
	XtDefaultForeground,
        },
};

/* the following are actions */

void CallEditAction();
extern void DeleteAction();
void Dummy_Action();
extern void Motion();
extern void MotionRSelect();
void No_Action();

extern void CallTokenGameMethod();

extern void AddMarkPar_2();
/* extern defined actions 

from NWFileActions.c 
extern void Load();	
extern void Save();	

from NWAddActions.c 
extern void Add();
extern void Add_2();
extern void AddMarkPar_2();

from NwDeleteActions.c 
extern void Delete();

from NWMoveActions.c 
extern void Move();
extern void Move_2();

from NWMotionActions.c
extern void Motion();

from NWShowActions.c
extern void ShowResults();
*/

extern void GivePrinterOk();

/* Actiontable */

static XtActionsRec editActions[] = {
			{"callEditAction",CallEditAction},
			{"no_Action",No_Action},
			{"motion",Motion},
			{"save",Save},
			{"load",Load},
			{"clear",ClearNet},
			{"redraw",Redraw},
			{"addResult",AddResult},
			{"deleteResult",DeleteResult},
			{"changeResult",ChangeResult},
			{"deleteAction",DeleteAction},
			{"givePrinterOk", GivePrinterOk},
			{"rSelect", RSelect},
			{"rSelect_2", RSelect_2},
			{"motionRSelect", MotionRSelect},
			{"pSelect", PSelect},
			};

/* Definition of default Translations */

static char defaultTranslations [] =
                "!Ctrl<Btn1Motion>:           motionRSelect()  \n\
                 <Motion>:                    motion()         \n\
                 !Ctrl<Btn1Down>:             rSelect()        \n\
                 !Ctrl<Btn1Up>:               rSelect_2()      \n\
                 !Ctrl<Btn2Down>:             pSelect()        \n\
                 !Ctrl<Btn3Down>:             pSelect()        \n\
		 <BtnDown>: 	              callEditAction() \n\
                 <Key>BackSpace:              deleteAction()   \n\
                 <Key>Delete:                 deleteAction()   \n";

/* Declaration of methods */

/* CORE METHODS */
/*used*/
static void 	Initialize();
static void     Realize();
static void 	Destroy();
static Boolean 	SetValues();
static void 	ClassInitialize();
void 	Redisplay();

/*inherited

  realize();
  set_values_almost();
  display_accelerator();
*/
/*unused

  class_part_initialize();
  get_values_hook();
  query_geometry();
  resize();
*/

/*COMPOSITE METHODS*/
static void 	InsertChild();
static Cardinal insertPosition();
static void 	DeleteChild();

/*inherited
  geometry_manager();
  change_managed();
*/

/* Definition of Extensionrecord */

CompositeClassExtensionRec netWidgetExtension = {
	/* next extension	*/ NULL,
	/* record type		*/ NULLQUARK,
	/* version		*/ XtCompositeExtensionVersion,
	/* record size		*/ sizeof(CompositeClassExtensionRec),
	/* accepts_objects	*/ TRUE,
	};

CompositeClassExtension netWidgetExtensionPtr = &netWidgetExtension;

static voidFKT CmdKeyTransTable[] = {
				SelectionOff,
				Add,
				Delete,
				Move,
				Change,
				ShowResults,
				No_Action,
				Dummy_Action,
			      };

static voidFKT ExtCmdKeyTransTable[] = {
				No_Action,
				Add_2,
				No_Action,
				Move_2,
				Change_2,
				No_Action,
				No_Action,
				No_Action,
			      };

/* drawfunctions are declared in appropriate hedaerfile */
voidFKT DrawFktTable[] = {
			      No_Action,
			      DrawPlace,
			      DrawTrans,
			      DrawTrans,
			      DrawTrans,
			      DrawTrans,
			      NULL,
			      DrawMarkPar,
			      DrawRatePar,
			      No_Action,
			      DrawPlace,
			    };

/* Initialization of Class Record */ 
NetWidgetClassRec netWidgetClassRec = {
    {
    /* core fields */
    /* superclass	  	 */ (WidgetClass) &compositeClassRec,
    /* class_name	  	 */ netClassName,
    /* widget_size	  	 */ sizeof(NetWidgetRec),
    /* class_initialize   	 */ ClassInitialize,
    /* class_part_initialize	 */ NULL,
    /* class_inited       	 */ FALSE,
    /* initialize	  	 */ Initialize,
    /* initialize_hook		 */ NULL,
    /* realize		  	 */ /* XtInheritRealize, */
				    Realize,
    /* actions		  	 */ editActions,
    /* num_actions	  	 */ XtNumber(editActions),
    /* resources	  	 */ resources,
    /* num_resources	  	 */ XtNumber(resources),
    /* xrm_class	  	 */ NULLQUARK,
    /* compress_motion	  	 */ True,
    /* compress_exposure  	 */ 0,
    /* compress_enterleave	 */ 0,
    /* visible_interest	  	 */ 0,
    /* destroy		  	 */ NULL,
    /* resize		  	 */ NULL,
    /* expose		  	 */ Redisplay,
    /* set_values	  	 */ SetValues,
    /* set_values_hook		 */ NULL,
    /* set_values_almost	 */ XtInheritSetValuesAlmost,
    /* get_values_hook		 */ NULL,
    /* accept_focus	 	 */ NULL,
    /* version			 */ XtVersion,
    /* callback_private   	 */ NULL,
    /* tm_table		   	 */ defaultTranslations,
    /* query_geometry		 */ NULL,
    /* display_accelerator       */ XtInheritDisplayAccelerator,
    /* extension                 */ NULL,
    },
    {
    /* composite fields */
    /* geometry_manager         */ NULL/*XtInheritGeometryManager*/,
    /* change_managed           */ NULL/*XtInheritChangeManaged*/,
    /* insert_child             */ InsertChild,
    /* delete_child             */ DeleteChild,
    /* extension structure      */ NULL,
    },
    {
    /* dummy      */  		    0,
    },
};

WidgetClass netWidgetClass = (WidgetClass) & netWidgetClassRec;


/* the following are private functions unique to NetWidget */
static void Init_GC();

/* Definition of Core Methods */

/*******************************************************************************
Name:
        ClassInitialize
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
--------------------------------------------------------------------------------
Zweck:
        Erm"oglicht Verwalten von Objekten
*******************************************************************************/

/* ARGSUSED */
static void
ClassInitialize( )
{

    netWidgetClassRec.composite_class_part.extension = 
	(XtPointer)netWidgetExtensionPtr;

    netWidgetClassRec.core_class_part.compress_motion = True;
}

/*******************************************************************************
Name:
        Initialize
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
--------------------------------------------------------------------------------
Zweck:
        Instanzeninitialisierung
*******************************************************************************/

/* ARGSUSED */
static void
Initialize(treq, tnew, args, num_args)
     Widget treq, tnew;
     ArgList args;
     Cardinal *num_args;
{
  NetWidget new = (NetWidget) tnew;
  
#ifdef debug
  printf("Initialize \n");
#endif

  if (new->core.width == 0)
	new->core.width = EDIT_FIELD_WIDTH;
  if (new->core.height == 0)
	new->core.height = EDIT_FIELD_HEIGHT;

  /* Add own insert_position method */
  new->composite.insert_position = insertPosition;

  /*Public fields */
  if (new->netWidget.model_name)
    new->netWidget.model_name = XtNewString(new->netWidget.model_name);

  if (new->netWidget.object == RATEPAR 
      && (new->netWidget.command == MOVE 
	  || new->netWidget.command == CHANGE))
  {
		new->netWidget.ratesVisible = True;
  }
  if (new->netWidget.object == TAG 
      && (new->netWidget.command == MOVE 
	  || new->netWidget.command == CHANGE))
  {
		new->netWidget.tagsVisible = True;
  }
  if (new->netWidget.picture_modified == True) {
	XtWarning("NW initialize: setting modified state not allowed: ignored");
	new->netWidget.picture_modified = False;
  }
  if (new->netWidget.net_modified == True) {
	XtWarning("NW initialize: setting modified state not allowed: ignored");
	new->netWidget.net_modified = False;
  }
  if (new->netWidget.gridsize != Off)
  {
	XtWarning("NW initialize: no setting of grid in initialize: ignored");
	new->netWidget.gridsize = Off;
  }
  if (new->netWidget.rescale_factor != OneZero)
  {
	XtWarning("NW initialize: no setting of rescalefactor in initialize: ignored");
	new->netWidget.rescale_factor = OneZero;
  }
  if (new->netWidget.precision > 15) {
      XtWarning
	  ("NW initialize: setting of precision greater then 15 not allowed");
      new->netWidget.precision = 15;
  }
  new->netWidget.tokenGame = False;

  new->netWidget.selectionBorder = NULL;  

  /* private fields */
  new->netWidget.numbase = 0;
  new->netWidget.numplace = 0;
  new->netWidget.numtrans = 0;
  new->netWidget.numarc = 0;
  new->netWidget.nummarkpar = 0;
  new->netWidget.numratepar = 0;
  new->netWidget.numresults = 0;

  new->netWidget.multiplMode = DrawM;
  new->netWidget.selectedObject = NULL;
  new->netWidget.actionTerminated = True;
  new->netWidget.allowMotionAction = False;
  new->netWidget.allowRotationAction = False;
  new->netWidget.ignoreButtonEvents= False;
  new->netWidget.results_collected= False;
  new->netWidget.throughput_computed= False;
  new->netWidget.currTransOrient = ZERO;

  new->netWidget.prev_x = -1;
  new->netWidget.prev_y = -1;

  new->netWidget.rescale_value = 1.0;

  new->netWidget.vanish = False;
  new->netWidget.timer = (XtIntervalId)0;
  new->netWidget.flashed = False;
  new->netWidget.readMDC = NULL;
  new->netWidget.writeMDC = NULL;

  /* initialize history for 'undo' */
  new->netWidget.history = XtNew(History);
  new->netWidget.history->object_list = NULL;
  new->netWidget.history->num_objects = 0;

  Init_GC(new);
  Init_Grid(new);

}/*Initialize*/

/*******************************************************************************
Name:
        SetValues
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
--------------------------------------------------------------------------------
Zweck:
        Resourcenverwaltung
*******************************************************************************/

/* ARGSUSED */
static Boolean
SetValues(current, request, new, args, num_args)
Widget current, request, new;
ArgList args;
Cardinal *num_args;
{
NetWidget curnw = (NetWidget) current;
NetWidget newnw = (NetWidget) new;
String name = newnw->netWidget.model_name;
Commands command = newnw->netWidget.command;
Objects object = newnw->netWidget.object;
Boolean tagsvisible = newnw->netWidget.tagsVisible;
Boolean ratesvisible = newnw->netWidget.ratesVisible;
Boolean spline = newnw->netWidget.spline;
Boolean figure_modified = newnw->netWidget.picture_modified;
Boolean net_modified = newnw->netWidget.net_modified;
Boolean solution_expected = newnw->netWidget.solution_expected;
Gridsize gridsize = newnw->netWidget.gridsize;
Boolean do_redisplay = False;

#ifdef debug
printf(" SetValues\n");
#endif

  if(name && name != curnw->netWidget.model_name)
    newnw->netWidget.model_name = XtNewString(name);

  if (object != curnw->netWidget.object) {
	if (object == RATEPAR && (command == MOVE || command == CHANGE)
		&&  newnw->netWidget.ratesVisible == False)
	{
		newnw->netWidget.ratesVisible = True;
		DrawNet(newnw,RATES_ONLY);
		do_redisplay = True;
	}
	if (object == TAG && (command == MOVE || command == CHANGE)
		&& newnw->netWidget.tagsVisible == False)
	{
		newnw->netWidget.tagsVisible = True;
		DrawNet(newnw,TAGS_ONLY);
		do_redisplay = True;
	}
  }
  if (command != curnw->netWidget.command) {
	if (object == RATEPAR && (command == MOVE || command == CHANGE)
		&&  newnw->netWidget.ratesVisible == False)
	{
		newnw->netWidget.ratesVisible = True;
		DrawNet(newnw,RATES_ONLY);
		do_redisplay = True;
	}
	if (object == TAG && (command == MOVE || command == CHANGE)
		 && newnw->netWidget.tagsVisible == False)
	{
		newnw->netWidget.tagsVisible = True;
		DrawNet(newnw,TAGS_ONLY);
		do_redisplay = True;
	}
  }
  if (tagsvisible != curnw->netWidget.tagsVisible) {
	DrawNet(newnw,TAGS_ONLY);
	do_redisplay = True;
  }
  if (ratesvisible != curnw->netWidget.ratesVisible) {
	DrawNet(newnw,RATES_ONLY);
	do_redisplay = True;
  }
  if (spline != curnw->netWidget.spline) {
        ClearNetPixmap(newnw);
	undrawSelectionBorder(newnw);
	Drawgrid(newnw,Off,newnw->netWidget.gridsize);
        DrawNet(newnw,NORMAL);
	do_redisplay = True;
  }
  if (figure_modified != curnw->netWidget.picture_modified) {
	XtWarning("NW Set Values: changing modified state not allowed: ignored");
	newnw->netWidget.picture_modified = curnw->netWidget.picture_modified;
  }
  if (net_modified != curnw->netWidget.net_modified) {
	XtWarning("NW Set Values: changing modified state not allowed: ignored");
	newnw->netWidget.net_modified = curnw->netWidget.net_modified;
  }
  if (newnw->netWidget.object_size != curnw->netWidget.object_size) {
	XtWarning("NW Set Values: changing objectsize not allowed");
	newnw->netWidget.object_size = DEFAULT_SIZE;
	/* if it is allowed someday : change the netWidget.spott pixmap 
	   to size 1.5times object_size, same for neWidget.stipple */
  }
  if (gridsize != curnw->netWidget.gridsize)
  {
	Drawgrid(newnw,curnw->netWidget.gridsize,gridsize);
	do_redisplay = True;
  }
  /*
  if (rescale_factor != curnw->netWidget.rescale_factor) 
  {
	RescaleNet(newnw,rescale_factor);
	do_redisplay = True;
  }
  */
  if (solution_expected != curnw->netWidget.solution_expected)
	if (solution_expected == True) {
	    newnw->netWidget.results_collected = False;
	    newnw->netWidget.throughput_computed = False;
	}
  if (newnw->netWidget.precision > 15) {
      XtWarning ("NW Set Values: precision greater 15 not allowed");
      newnw->netWidget.precision=curnw->netWidget.precision;
  }
  return do_redisplay;
}/*SetValues*/

/*******************************************************************************
Name:
        Destroy
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
--------------------------------------------------------------------------------
Zweck:
        Freigeben der graphischen Kontexte
*******************************************************************************/

static void Destroy(w)
Widget w;
{

  NetWidget nw = (NetWidget) w;

#ifdef debug
  printf("NetWidget: Destroy \n");
#endif
  /*
  ResetHistory(w,NOOPERATION);
  XtFree(nw->netWidget.history);
  */
  XtFree(nw->netWidget.model_name);
  nw->netWidget.selectedObject = NULL;
  XFreeGC(XtDisplay(w),nw->netWidget.draw_GC);
  XFreeGC(XtDisplay(w),nw->netWidget.screen_GC);
  XFreeGC(XtDisplay(w),nw->netWidget.motion_GC); 
  XFreeGC(XtDisplay(w),nw->netWidget.copy_GC);
  XFreeGC(XtDisplay(w),nw->netWidget.clear_GC);
  /* XFreeGC(XtDisplay(w),nw->netWidget.mult_GC); */
  XFreePixmap(XtDisplay(w),nw->netWidget.picture);
  XFreePixmap(XtDisplay(w),nw->netWidget.flash);
  XFreePixmap(XtDisplay(w),nw->netWidget.spott);
  XFreePixmap(XtDisplay(w),nw->netWidget.stipple);
  Free_Grid(w);

}/*Destroy*/

/*******************************************************************************
Name:
        Redisplay
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
--------------------------------------------------------------------------------
Zweck:
        Ausgabe der Pixmap zum server
*******************************************************************************/

void Redisplay(w,event,region)
Widget w;
XExposeEvent *event;
Region region;
{
  int x,y,width,height;
  NetWidget nw = (NetWidget)(w);
  
#ifdef debug
  printf("Redisplay \n");
  fflush(stdout);
#endif
  
  if (!XtIsRealized((Widget)nw))
    return;
  
  if (event) { 
    x=event->x;
    y=event->y;
    width=event->width;
    height=event->height;
  }
  else {
    x = y = 0;
    width = nw->core.width;
    height = nw->core.height;
  }
  CopyToDisplay(nw, nw->netWidget.picture, x,y, width,height);
  
  /* Ueberpreufung des Netzes, veraenderten Netztyp sofort im netWidget setzen! */
   /* etwas deplaziert, aber tut brav seinen Dienst.. */
  {
	Widget *child=nw->composite.children;
	Cardinal num_children=nw->composite.num_children;
	int i, expT=0, imT=0, detT=0, genT=0, net_type;
	static int last_displayed_net_type=TN_GSPN;
	
	for(i=0;i<num_children;i++)
		if(isTransObject(child[i]) && !child[i]->core.being_destroyed)
		{	
		  TransType trans_type;
		  
		  XtVaGetValues(child[i],XtNtransType,&trans_type,NULL);
		  if(trans_type==EXPONENTIEL) expT++;
		  else if(trans_type==IMMEDIATE) imT++;
		  else if(trans_type==DETERMINISTIC) detT++;
		  else if(trans_type==GENERAL) genT++;
	    }
   	
	if(genT) net_type=TN_ESPN; 
	  else if(detT) net_type=TN_DSPN; 
	    else net_type=TN_GSPN;
		
       	if(net_type!=last_displayed_net_type)
	{ last_displayed_net_type=net_type;
	  if (GetTimeMode() == 1) 
	      setDisplayedNetType("DDSPN");

	  else setDisplayedNetType(net_type==TN_ESPN?"EDSPN":net_type==TN_DSPN?"DSPN":"GSPN");
	   }
	nw->netWidget.net_type=net_type;
  }

}/* Redisplay*/


/* COMPOSITE METHODS */

/*******************************************************************************
Name:
        InsertChild
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
--------------------------------------------------------------------------------
Zweck:
        Verwaltung des Feldes der verwalteten Objekte
*******************************************************************************/

static void 
InsertChild(w,args,numargs)
Widget w;
ArgList args;
Cardinal *numargs;

{
CompositeWidgetClass superclass;	
NetWidget parent = (NetWidget)XtParent(w);

#ifdef debug
printf(" InsertChild\n");
#endif

superclass = (CompositeWidgetClass)compositeWidgetClass;
/*(*superclass->composite_class.insert_child)(w,args,numargs);*/
(*superclass->composite_class.insert_child)(w);

if (isPlaceObject(w))
     (unsigned)(parent->netWidget.numplace)++;
else
if (isTransObject(w))
     (unsigned)(parent->netWidget.numtrans)++;
else
if (isArcObject(w))
     (unsigned)(parent->netWidget.numarc)++;
else
if (isMarkParObject(w))
     (unsigned)(parent->netWidget.nummarkpar)++;
else
if (isRateParObject(w))
     (unsigned)(parent->netWidget.numratepar)++;
else
if (isResultObject(w))
     (unsigned)(parent->netWidget.numresults)++;

}/*InsertChild*/


/*******************************************************************************
Name:
        DeleteChild
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
--------------------------------------------------------------------------------
Zweck:
        Verwaltung des Feldes der verwalteten Objekte
*******************************************************************************/

static void 
DeleteChild(w)
Widget w;

{
CompositeWidgetClass superclass;	
NetWidget parent = (NetWidget)XtParent(w);

#ifdef debug
printf("NetWidget: DeleteChild\n");
#endif

if (isPlaceObject(w))
     (unsigned)(parent->netWidget.numplace)--;
else
if (isTransObject(w))
     (unsigned)(parent->netWidget.numtrans)--;
else
if (isArcObject(w))
     (unsigned)(parent->netWidget.numarc)--;
else
if (isMarkParObject(w))
     (unsigned)(parent->netWidget.nummarkpar)--;
else
if (isRateParObject(w))
     (unsigned)(parent->netWidget.numratepar)--;
else
if (isResultObject(w))
     (unsigned)(parent->netWidget.numresults)--;

superclass = (CompositeWidgetClass)compositeWidgetClass;
(*superclass->composite_class.delete_child)(w);

}


/*******************************************************************************
Name:
        insertPosition
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
--------------------------------------------------------------------------------
Zweck:
        Liefert Einf"ugeposition im Feld der verwalteten Objekte
*******************************************************************************/

static Cardinal insertPosition(w)
Widget w;

{
NetWidget parent = (NetWidget)XtParent(w);

#ifdef debug
printf(" insertPostion\n");
#endif

if (isPlaceObject(w))
	 return LastPlace(parent);
else
if (isTransObject(w))
	 return LastTrans(parent);
else
if (isArcObject(w))
	 return LastArc(parent);
else
if (isMarkParObject(w))
	 return LastMarkPar(parent);
else
if (isRateParObject(w))
	 return LastRatePar(parent);
else
if (isResultObject(w))
	 return LastResult(parent);
XtWarning("insertPosition() for not expected widget typ!\n");
return 0;
}/*insertPosition*/ 


/* Definition der Aktionen */


/*******************************************************************************
Name:
        CallEditAction
--------------------------------------------------------------------------------
Funktionstyp:
        Aktion
--------------------------------------------------------------------------------
triggert bei event:
        <BtnDown>: 	callEditAction()
--------------------------------------------------------------------------------
Zweck:
        Verzweigt zu den Einzelaktionen
*******************************************************************************/

void CallEditAction(w,event,params,numparams)
     Widget w;
     XButtonEvent* event;
     String* params;
     Cardinal* numparams;
{
  NetWidget nw = (NetWidget)(w);

#ifdef debug
  printf("CallEditAction \n");
#endif

  if (nw->netWidget.ignoreButtonEvents && event)
    return;

  if (nw->netWidget.actionTerminated) {
    if (nw->netWidget.tokenGame)
      CallTokenGameMethod(w,event,params,numparams);
    CallMethod(nw->netWidget.command,w,event,params,numparams);
  }
  else
    CallMethod_2(nw->netWidget.command,w,event,params,numparams);
}

void No_Action()
{
;
}

void Dummy_Action(w,event,params,numparams)
     Widget w;
     XButtonEvent* event;
     String* params;
     Cardinal* numparams;
{
  NetWidget nw = (NetWidget)w;
  nw->netWidget.command = NOOPERATION;
}

/*******************************************************************************
Name:
        Init_GC
--------------------------------------------------------------------------------
Funktionstyp:
        private Funktion
--------------------------------------------------------------------------------
Parameter:
        w: Instanz der Klasse NetWidget
--------------------------------------------------------------------------------
Zweck:
        Initialisieren der graphischen Kontexte und der Pixmap
*******************************************************************************/

static void Init_GC(w)
Widget w;

{
    NetWidget nw = (NetWidget)w;
    XGCValues values;
    XtGCMask mask = GCForeground | GCBackground | GCLineWidth | GCFunction;
    unsigned depth=1; 

#ifdef debug
    printf("Init_GC \n");
#endif

    nw->netWidget.picture = XCreatePixmap(XtDisplay(nw),
					  RootWindow(XtDisplay(nw),
						  DefaultScreen(XtDisplay(nw))),
					  nw->core.width,
					  nw->core.height,
					  depth);
    nw->netWidget.flash = XCreatePixmap(XtDisplay(nw),
					  RootWindow(XtDisplay(nw),
						  DefaultScreen(XtDisplay(nw))),
					  nw->core.width,
					  nw->core.height,
					  depth);

    nw->netWidget.spott = XCreatePixmap(XtDisplay(nw),
			       RootWindow(XtDisplay(nw),
					  DefaultScreen(XtDisplay(nw))),
			       (unsigned)(nw->netWidget.object_size * 1.5),
			       (unsigned)(nw->netWidget.object_size * 1.5),
			       depth);  
    nw->netWidget.stipple = XCreatePixmap(XtDisplay(nw),
			       RootWindow(XtDisplay(nw),
					  DefaultScreen(XtDisplay(nw))),
			       (unsigned)(nw->netWidget.object_size * 1.5),
			       (unsigned)(nw->netWidget.object_size * 1.5),
			       depth); 

    /* with clear_GC we make a white pixmap */

    if (DefaultDepthOfScreen(XtScreen(nw)) == 1 &&
	nw->core.background_pixel == BlackPixelOfScreen(XtScreen(nw)))
      values.function = GXset;
    else
      values.function = GXclear;  /* unset all pixels */
    nw->netWidget.clear_GC = XCreateGC(XtDisplay(nw),
				       nw->netWidget.picture,
				       GCFunction,
				       &values);

    /* so we make a clear Pixmap */

    XCopyArea(XtDisplay(nw), nw->netWidget.picture,nw->netWidget.picture,
	      nw->netWidget.clear_GC,
	      0,0, nw->core.width,nw->core.height, 0,0);

    XCopyArea(XtDisplay(nw), nw->netWidget.spott,nw->netWidget.spott,
	      nw->netWidget.clear_GC,
	      0,0, 
	      (int)(nw->netWidget.object_size * 1.5),
	      (int)(nw->netWidget.object_size * 1.5),
	      0,0);

    XCopyArea(XtDisplay(nw), nw->netWidget.stipple,nw->netWidget.stipple,
	      nw->netWidget.clear_GC,
	      0,0, 
	      (int)(nw->netWidget.object_size * 1.5),
	      (int)(nw->netWidget.object_size * 1.5),
	      0,0);
    /* with copy_GC we copy the pixmap to the window */
    values.function = GXcopy;  /* overwrite all */
    values.foreground = nw->netWidget.foreground;
    values.background = nw->core.background_pixel;

    nw->netWidget.copy_GC = XtGetGC((Widget)nw,
				    GCFunction | GCForeground | GCBackground,
				    &values); 

    /* with xor we unset overlayings (are invisible) but we can delete some 
       objects - and then the overlayings will be again visible */
    /* later we should use GXor and redraw the net after deletion */
    /* but then we need an other GC for highLighting with spott-pixmap 
       and for motion */
    values.function = GXxor;
    values.foreground = 1;
    values.background = 0;
    values.line_width = 1;
    nw->netWidget.draw_GC = XCreateGC(XtDisplay(nw),
				      nw->netWidget.picture,
				      mask,
				      &values);

    /* for direct painting in screen */
/*    nw->netWidget.screen_GC = XCreateGC(XtDisplay(nw),
					DefaultScreen(XtDisplay(nw)),
					mask, &values);
*/
    values.function = GXxor;
    nw->netWidget.motion_GC = XCreateGC(XtDisplay(nw),nw->netWidget.picture,
					mask,&values); 
    /* nw->netWidget.mult_GC = XCreateGC(XtDisplay(nw),nw->netWidget.picture,
				      mask,&values); */
    
    /* now we make the stipple-pixmap stippled */
  {
#define stipple_width 16
#define stipple_height 16
    static unsigned char stipple_bits[] = {
       0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa,
       0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa,
       0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa};
    Pixmap stipple,tiled;

    stipple = XCreateBitmapFromData(XtDisplay(nw), nw->netWidget.stipple,
				stipple_bits, stipple_width, stipple_height);
    tiled = XCreateBitmapFromData(XtDisplay(nw),RootWindowOfScreen(XtScreen(nw)),
				schrafiert_bits, schrafiert_width, schrafiert_height);
    XSetFunction(XtDisplay(nw),nw->netWidget.draw_GC,GXcopy);
    XSetStipple(XtDisplay(nw),nw->netWidget.draw_GC,stipple);
    XSetTile(XtDisplay(nw),nw->netWidget.draw_GC,tiled);
    if (nw->core.background_pixel == 1)
      XSetFillStyle(XtDisplay(nw),nw->netWidget.draw_GC,FillStippled);
    else
      XSetFillStyle(XtDisplay(nw),nw->netWidget.draw_GC,FillStippled);

    XFillRectangle(XtDisplay(nw), nw->netWidget.stipple,nw->netWidget.draw_GC,
		   0,0,
		   (int)(nw->netWidget.object_size * 1.5),
		   (int)(nw->netWidget.object_size * 1.5));
    XSetFillStyle(XtDisplay(nw),nw->netWidget.draw_GC,FillSolid);
    XSetFunction(XtDisplay(nw),nw->netWidget.draw_GC,GXxor);
  }
    
    
}/* Init_GC*/


static void Realize(w, vm, wa)
     Widget w;
     XtValueMask *vm;
     XSetWindowAttributes *wa;
{
  CompositeWidgetClass superclass;
  NetWidget nw = (NetWidget)w;
  XtGCMask mask = GCForeground | GCBackground | GCLineWidth | GCFunction;
  XGCValues values;  

  values.foreground = nw->netWidget.foreground;
  values.background = nw->core.background_pixel;
  values.line_width = 0;
  values.function = GXxor;

  superclass = (CompositeWidgetClass)compositeWidgetClass;
  (*superclass->core_class.realize)(w, vm, wa);

  nw->netWidget.screen_GC = XCreateGC(XtDisplay(w),
				      XtWindow(w),
				      mask, &values);

}
