/*******************************************************************************
Modul   	:	BaseObj.c

Purpose		: 	Realisation der Klasse BaseObject

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/


/*
 * BaseObj.c - base object.

   Class Hierarchie:
   Object -> BaseObj
 */
#include <stdio.h>

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include "BaseObjP.h"
#include "util.h"

#define offset(field) XtOffsetOf(BaseObjRec, field)

static XtResource resources[] = {
	{
	XtNxpos,
	XtCxpos,
	XtRInt,
	sizeof(int),
	offset(baseObj.x),
	XtRImmediate,
	(XtPointer) UNSPECIFIED_COORDINATE
	},
	{
	XtNypos,
	XtCypos,
	XtRInt,
	sizeof(int),
	offset(baseObj.y),
	XtRImmediate,
	(XtPointer) UNSPECIFIED_COORDINATE
	},
        {
        XtNdblX,
        XtCDblX,
        XtRFloat,
        sizeof(double),
        offset(baseObj.dblX),
        XtRString,
        "0.0"
        },
        {
        XtNdblY,
        XtCDblY,
        XtRFloat,
        sizeof(double),
        offset(baseObj.dblY),
        XtRString,
        "0.0"
        },
};

/* Declaration of methods */
static void Initialize();
static Boolean SetValues();

/* these Core methods not needed by BaseObj:
 *
 * static void ClassInitialize();
 * static void Realize();
 */

/* the following are private functions unique to BaseObj */

/* the following are actions of BaseObj */

/* The following are public functions of BaseObj, declared extern
 * in the public include file: */


BaseObjClassRec baseObjClassRec = {
    {
    /* object_class fields */
    /* superclass	  	 */ (WidgetClass) &objectClassRec,
    /* class_name	  	 */ baseClassName,
    /* widget_size	  	 */ sizeof(BaseObjRec),
    /* class_initialize   	 */ NULL,
    /* class_part_initialize	 */ NULL,
    /* class_inited       	 */ FALSE,
    /* initialize	  	 */ Initialize,
    /* initialize_hook		 */ NULL,
    /* realize		  	 */ NULL,
    /* actions		  	 */ NULL,
    /* num_actions	  	 */ 0,
    /* resources	  	 */ resources,
    /* num_resources	  	 */ XtNumber(resources),
    /* xrm_class	  	 */ NULLQUARK,
    /* compress_motion	  	 */ 0,
    /* compress_exposure  	 */ 0,
    /* compress_enterleave	 */ 0,
    /* visible_interest	  	 */ 0,
    /* destroy		  	 */ NULL,
    /* resize		  	 */ NULL,
    /* expose		  	 */ NULL,
    /* set_values	  	 */ SetValues,
    /* set_values_hook		 */ NULL,
    /* set_values_almost	 */ NULL,
    /* get_values_hook		 */ NULL,
    /* accept_focus	 	 */ NULL,
    /* version			 */ XtVersion,
    /* callback_private   	 */ NULL,
    /* tm_table		   	 */ NULL,
    /* query_geometry		 */ NULL,
    /* display_accelerator       */ NULL,
    /* extension                 */ NULL
    },
    {
    /* dummy               */ 0,
    },
};

WidgetClass baseObjClass = (WidgetClass) & baseObjClassRec;

/*******************************************************************************
Name:
        Initialize
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/

/* ARGSUSED */
static void
Initialize(treq, tnew, args, num_args)
Widget treq, tnew;
ArgList args;
Cardinal *num_args;
{
BaseObj  new = (BaseObj ) tnew;

#ifdef debug
printf("BaseObj: Initialize \n");
#endif

  if (new->baseObj.x <0 || new->baseObj.y < 0)
  {
	new->baseObj.x = UNSPECIFIED_COORDINATE;
	new->baseObj.y = UNSPECIFIED_COORDINATE;
  }
}

/*******************************************************************************
Name:
        SetValues
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/

/* ARGSUSED */
static Boolean
SetValues(current, request, new, args, num_args)
Widget current, request, new;
ArgList args;
Cardinal *num_args;
{
BaseObj newcw = (BaseObj) new;
Boolean do_redisplay = False;

  if ((newcw->baseObj.x < 0 || newcw->baseObj.y < 0) &&
      (newcw->baseObj.x != DESTROYED && newcw->baseObj.y != DESTROYED))
  {
	newcw->baseObj.x = UNSPECIFIED_COORDINATE;
	newcw->baseObj.y = UNSPECIFIED_COORDINATE;
  }
  return do_redisplay;
}

Boolean isDestroyed(w)
Widget w;

{
BaseObj base = (BaseObj)w;

return base->object.being_destroyed;
}

