/*******************************************************************************
Modul   	:	MarkParObj.c

Purpose		: 	Realisation der Klasse MarkParObjext

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/


/*
 * MarkParObj.c - marking parameter object.

   Class Hierarchie:
   Object -> BaseObj -> MarkParObj
 */

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include <stdio.h>

#include "MarkParObjP.h"

#define offset(field) XtOffsetOf(MarkParObjRec, field)

static XtResource resources[] = {
	{
	XtNmparName,
	XtCmparName,
	XtRString,
	sizeof(char*),
	offset(markParObj.mpar_name),
	XtRPointer,
	(XtPointer) NULL
	},
	{
	XtNmparValue,
	XtCmparValue,
	XtRCardinal,
	sizeof(Cardinal),
	offset(markParObj.mpar_value),
	XtRImmediate,
	(XtPointer) 0
	},
};

/* Declaration of methods */

static void Initialize();
static void Destroy();
static Boolean SetValues();

/* these Core methods not needed by MarkParObj:
 *
 * static void ClassInitialize();
 * static void Realize();
 */

/* the following are private functions unique to MarkParObj */
static String getDefaultName();

/* The following are public functions of MarkParObj, declared extern
 * in the public include file: */


MarkParObjClassRec markParObjClassRec = {
    {
    /* object_class fields */
    /* superclass	  	 */ (WidgetClass) &baseObjClassRec,
    /* class_name	  	 */ "markParClass",
    /* widget_size	  	 */ sizeof(MarkParObjRec),
    /* class_initialize   	 */ NULL,
    /* class_part_initialize	 */ NULL,
    /* class_inited       	 */ FALSE,
    /* initialize	  	 */ Initialize,
    /* initialize_hook		 */ NULL,
    /* realize		  	 */ NULL,
    /* actions		  	 */ NULL,
    /* num_actions	  	 */ 0,
    /* resources	  	 */ resources,
    /* num_resources	  	 */ XtNumber(resources),
    /* xrm_class	  	 */ NULLQUARK,
    /* compress_motion	  	 */ 0,
    /* compress_exposure  	 */ 0,
    /* compress_enterleave	 */ 0,
    /* visible_interest	  	 */ 0,
    /* destroy		  	 */ Destroy,
    /* resize		  	 */ NULL,
    /* expose		  	 */ NULL,
    /* set_values	  	 */ SetValues,
    /* set_values_hook		 */ NULL,
    /* set_values_almost	 */ NULL,
    /* get_values_hook		 */ NULL,
    /* accept_focus	 	 */ NULL,
    /* version			 */ XtVersion,
    /* callback_private   	 */ NULL,
    /* tm_table		   	 */ NULL,
    /* query_geometry		 */ NULL,
    /* display_accelerator       */ NULL,
    /* extension                 */ NULL
    },
    {
    /* dummy               */ 0,
    },
    {
    /* num_created               */ 0,
    },
};

WidgetClass markParObjClass = (WidgetClass) & markParObjClassRec;

/*******************************************************************************
Name:
        Initialize
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/

/* ARGSUSED */
static void
Initialize(treq, tnew, args, num_args)
Widget treq, tnew;
ArgList args;
Cardinal *num_args;
{
MarkParObj new = (MarkParObj) tnew;

#ifdef debug
printf("MPAR: Initialize \n");
#endif
  if (new->markParObj.mpar_name == NULL || 
      *(new->markParObj.mpar_name) == '\0')
	new->markParObj.mpar_name = getDefaultName(new);
  else
	new->markParObj.mpar_name = XtNewString(new->markParObj.mpar_name);
  markParObjClassRec.markParObj_class.num_created++;
}

/*******************************************************************************
Name:
        SetValues
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/

/* ARGSUSED */
static Boolean
SetValues(current, request, new, args, num_args)
Widget current, request, new;
ArgList args;
Cardinal *num_args;
{
MarkParObj curpar = (MarkParObj) current;
MarkParObj newpar = (MarkParObj) new;
Boolean do_redisplay = False;

#ifdef debug
printf("MPAR: SetValues \n");
#endif
  
  if (newpar->markParObj.mpar_name != curpar->markParObj.mpar_name) 
	newpar->markParObj.mpar_name = 
		XtNewString(newpar->markParObj.mpar_name);
    return do_redisplay;
}

/*******************************************************************************
Name:
        Destroy
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/

static void
Destroy(w)
Widget w;
{
MarkParObj mpar = (MarkParObj) w;

#ifdef debug
printf("MPAR: Destroy \n");
#endif

  XtFree(mpar->markParObj.mpar_name);
}

static String getDefaultName(w)
     Widget w;
{
    char name[100];
    
    sprintf(name,"mpar%d", markParObjClassRec.markParObj_class.num_created+1);
    return XtNewString(name);
}

String getDisplayedMarkingString(w)
     Widget w;
{
    MarkParObj mpar = (MarkParObj)w;
    char string[200];

    sprintf(string,"%s=%d", 
	    mpar->markParObj.mpar_name,mpar->markParObj.mpar_value);
    return XtNewString(string);
}


