/*******************************************************************************
Modul   	:	PlaceObj.c

Purpose		: 	Realisation der Klasse PlaceObject

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/


/*
 * PlaceObj.c - base object.

   Class Hierarchie:
   Object -> PlaceObj
 */
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include "MarkParObj.h"

#include <stdio.h>
#include <string.h>

#include "PlaceObjP.h"

#define offset(field) XtOffsetOf(PlaceObjRec, field)

static XtResource resources[] = {
	{
	XtNinitialMarking,
	XtCinitialMarking,
	XtRinitialMarking,
	sizeof(InitialMarkingPtr),
	offset(placeObj.M0),
	XtRPointer,
	NULL,
	},
	{
	XtNdistribution,
	XtCdistribution,
	XtRdistribution,
	sizeof(Distribution *),
	offset(placeObj.distribution),
	XtRPointer,
	NULL,
	},
	{
	XtNactualMarking,
	XtCActualMarking,
	XtRCardinal,
	sizeof(Cardinal),
	offset(placeObj.actualMarking),
	XtRImmediate,
	(XtPointer)0,
        },
	{
	XtNselected,
	XtCSelected,
	XtRBoolean,
	sizeof(Boolean),
	offset(placeObj.selected),
	XtRImmediate,
	(XtPointer)False
        },
        {
        XtNcopy,
        XtCCopy,
        XtRWidget,
        sizeof(Widget),
        offset(placeObj.copy),
        XtRImmediate,
        (XtPointer)NULL
        },
/* now in baseObj
        {
        XtNdblX,
        XtCDblX,
        XtRFloat,
        sizeof(double),
        offset(placeObj.dblX),
        XtRImmediate,
        (XtPointer)0
        },
        {
        XtNdblY,
        XtCDblY,
        XtRFloat,
        sizeof(double),
        offset(placeObj.dblY),
        XtRImmediate,
        (XtPointer)0
        },
*/
};

/* Declaration of methods */

static void Initialize();
static void Destroy();
static Boolean SetValues();

/* these Core methods not needed by PlaceObj:
 *
 * static void ClassInitialize();
 * static void Realize();
 */

/* the following are private functions unique to PlaceObj */
static String getDefaultTagName();

/* The following are public functions of PlaceObj, declared extern
 * in the public include file: */


PlaceObjClassRec placeObjClassRec = {
    {
    /* object_class fields */
    /* superclass	  	 */ (WidgetClass) &tagedObjClassRec,
    /* class_name	  	 */ placeClassName,
    /* widget_size	  	 */ sizeof(PlaceObjRec),
    /* class_initialize   	 */ NULL,
    /* class_part_initialize	 */ NULL,
    /* class_inited       	 */ FALSE,
    /* initialize	  	 */ Initialize,
    /* initialize_hook		 */ NULL,
    /* realize		  	 */ NULL,
    /* actions		  	 */ NULL,
    /* num_actions	  	 */ 0,
    /* resources	  	 */ resources,
    /* num_resources	  	 */ XtNumber(resources),
    /* xrm_class	  	 */ NULLQUARK,
    /* compress_motion	  	 */ 0,
    /* compress_exposure  	 */ 0,
    /* compress_enterleave	 */ 0,
    /* visible_interest	  	 */ 0,
    /* destroy		  	 */ Destroy,
    /* resize		  	 */ NULL,
    /* expose		  	 */ NULL,
    /* set_values	  	 */ SetValues,
    /* set_values_hook		 */ NULL,
    /* set_values_almost	 */ NULL,
    /* get_values_hook		 */ NULL,
    /* accept_focus	 	 */ NULL,
    /* version			 */ XtVersion,
    /* callback_private   	 */ NULL,
    /* tm_table		   	 */ NULL,
    /* query_geometry		 */ NULL,
    /* display_accelerator       */ NULL,
    /* extension                 */ NULL
    },
    {
    /* dummy base              	 */ 0,
    },
    {
    /* dummy   Taged           	 */ 0,
    },
    {
    /* dummy place             	 */ 0,
    },
};


WidgetClass placeObjClass = (WidgetClass) & placeObjClassRec;

/*******************************************************************************
Name:
        Initialize
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/

/* ARGSUSED */
static void
Initialize(treq, tnew, args, num_args)
Widget treq, tnew;
ArgList args;
Cardinal *num_args;
{
    PlaceObj new = (PlaceObj) tnew;
    InitialMarking *m0;
    Distribution * distribution;
    Cardinal value;

#ifdef debug
    printf("PlaceObj: Initialize \n");
#endif
    /* if necessary initialize tag */

    if (new->tagedObj.tagName == NULL) {
  	new->tagedObj.tagName = getDefaultTagName(new);
    }
    if ((m0=(InitialMarking *)XtMalloc(sizeof(InitialMarking)))==NULL) {
	XtError("place initialize: no mem for m0");
    }
    if (new->placeObj.M0 == NULL) {
	m0->type = MARKINGVALUE;
	m0->values.markingValue = 0;
    }
    else {
	m0->type = new->placeObj.M0->type;
	switch(m0->type) {
	case  MARKINGVALUE:
	    m0->values.markingValue = new->placeObj.M0->values.markingValue;
	    break;
	case MARKINGPARAM:
	    m0->values.markingParam = new->placeObj.M0->values.markingParam;
	    break;
	default:
	    XtWarning
	    ("place initialize: unknown parameter type, reset to default (0)");
	    m0->type = MARKINGVALUE;
	    m0->values.markingValue = 0;
	}
    }
    new->placeObj.M0 = m0;
    /* check distribution */
    if (new->placeObj.distribution != NULL)
    {
	if ((distribution = XtNew(Distribution)) == NULL)
	{
	    XtError(" no place vor destribution record ");
	}
	else
	{
	    distribution->mean = new->placeObj.distribution->mean;
	    distribution->error = new->placeObj.distribution->error;
	    distribution->distribution_string = 
		XtNewString(new->placeObj.distribution->distribution_string);
	    new->placeObj.distribution = distribution;
	}
    }
    switch (new->placeObj.M0->type) {
    case MARKINGVALUE:
	new->placeObj.actualMarking = new->placeObj.M0->values.markingValue;
	break;
    case MARKINGPARAM:
	XtVaGetValues((Widget)new->placeObj.M0->values.markingParam,
		      XtNmparValue, &value,
		      NULL);
	new->placeObj.actualMarking = value;
	break;
    }
    new->placeObj.selected = False;

    /* private */

    placeObjClassRec.placeObj_class.num_created++;
}


/*******************************************************************************
Name:
        SetValues
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/
/* ARGSUSED */
static Boolean
SetValues(current, request, new, args, num_args)
Widget current, request, new;
ArgList args;
Cardinal *num_args;
{
    PlaceObj curplace = (PlaceObj) current;
    PlaceObj newplace = (PlaceObj) new;
    InitialMarking *m0;
    Distribution *distribution;
    Cardinal value;
    Boolean do_redisplay = False;
    
    /* if new M0: copy the new fields to the old 
       and then set new pointer to the old */
    if (curplace->placeObj.M0 != newplace->placeObj.M0 ) 
    {
	if ((m0 = XtNew(InitialMarking)) == NULL) 
	{
	    XtError("place setvalues: no mem for mO");
	}
	m0->type = newplace->placeObj.M0->type;
	if (m0->type == MARKINGVALUE) {
	    m0->values.markingValue=newplace->placeObj.M0->values.markingValue;
	    newplace->placeObj.actualMarking = m0->values.markingValue;
	}
	else {
	    m0->values.markingParam=newplace->placeObj.M0->values.markingParam;
	    XtVaGetValues((Widget)m0->values.markingParam,
			  XtNmparValue, &value,
			  NULL);
	    newplace->placeObj.actualMarking = value;
	}
	newplace->placeObj.M0 = m0;
    }
    /* check distribution */
    if (newplace->placeObj.distribution != curplace->placeObj.distribution)
    {
	if ((distribution = XtNew(Distribution)) == NULL)
	{
	    XtError("place setvalues: no mem for distribution");
	}
	distribution->mean = newplace->placeObj.distribution->mean;
	distribution->error = newplace->placeObj.distribution->error;
	distribution->distribution_string = 
	    XtNewString(newplace->placeObj.distribution->distribution_string);
	newplace->placeObj.distribution = distribution;
  }
    return do_redisplay;
}

/*******************************************************************************
Name:
        Destroy
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/

static void
Destroy(w)
Widget w;
{
PlaceObj place = (PlaceObj) w;

#ifdef debug
printf("PlaceObj: Destroy \n");
#endif
    XtFree((char *)place->placeObj.M0);
    if (place->placeObj.distribution)
      {
	XtFree((char *)place->placeObj.distribution->distribution_string);
	XtFree((char *)place->placeObj.distribution);
      }
}

extern String nwUniquePlaceName();

static char* getDefaultTagName(w)
     Widget w;
{
    return nwUniquePlaceName();
}

