#define POLYGON_C
 
#include <math.h>
#include <limits.h>
#include <X11/Intrinsic.h>
#include "Polygon.h"

#ifdef __STDC__
static Boolean interSection(double x, double y, Polygon *p, unsigned int i);
static double nextYneqY(Polygon *p, unsigned int i, double y);
static double xIS (double x1, double y1, double x2, double y2, double y);
#else
static Boolean interSection();
static double nextYneqY();
static double xIS ();
#endif

#ifdef __STDC__
Polygon *createPolygon(void)
#else
Polygon *createPolygon()
#endif
{
    Polygon *p;

    p = XtNew(Polygon);

    p->numVertexes = 0;
    
    p->vertexes = (Coordinate *)XtCalloc(20,sizeof(Coordinate));
    p->maxVertexes = 20;
    return p;
}

#ifdef __STDC__
void destroyPolygon(Polygon *p)
#else
void destroyPolygon(p)
     Polygon *p;
#endif
{
    if (p) {
	XtFree((char *)p->vertexes);
	XtFree((char *)p);
    }
}

#ifdef __STDC__
void createVertex (Polygon *p, double x, double y)
#else
void createVertex (p, x, y)
    Polygon *p;
    double x, y;
#endif
{
    if (!p) {
	XtWarning ("createVertex: can't create vertex in NULL polygon!");
	return;
    }
    
    /* don't make a new vertex if new is on last edge which is a 
       horizontal or vertical line */
    if ((p->numVertexes > 2) &&
	(p->vertexes[p->numVertexes].x == x) &&
	(p->vertexes[p->numVertexes - 1].x == x)) 
      p->vertexes[p->numVertexes].y = y;
    else if ((p->numVertexes > 2) &&
	(p->vertexes[p->numVertexes].y == y) &&
	(p->vertexes[p->numVertexes - 1].y == y)) 
      p->vertexes[p->numVertexes].x = x;

    else {

	/* if there are all vertex-places full, reallocate more place */
	if (p->numVertexes == p->maxVertexes) 
	  p->vertexes = (Coordinate*)XtRealloc
	    ((char *)p->vertexes, sizeof(Coordinate)*(p->maxVertexes+=20));
	
	p->vertexes[p->numVertexes].x = x;
	p->vertexes[p->numVertexes].y = y;
	p->numVertexes++;
    }

}

#ifdef __STDC__
Boolean polygonClosed(Polygon *p, double x, double y, double d)
#else
Boolean polygonClosed(p, x,y, d)
    Polygon *p;
    double x, y, d;
#endif
{
    if (!p || !p->vertexes || !p->numVertexes) {
	XtWarning("polygonClosed: NULL-polygon or -vertexes!");
	return False;
    }
    if (sqrt(pow(p->vertexes[0].x - x ,2.0) + pow(p->vertexes[0].y - y ,2.0)) 
	<= d)
	return True; 
    else 
	return False;
}

#ifdef __STDC__
Boolean isInPolygon(Polygon *p, double x, double y)
#else
Boolean isInPolygon(p, x, y)
     Polygon *p;
     double x, y;
#endif
{
    unsigned int i, iS = 0;
    
    if (!p) {
	XtWarning ("isInPolygon: nothing can be in a NULL polygon!");
	return False;
    }
    if (!p->numVertexes) {
	XtWarning ("isInPolygon: nothing can be in an empty polygon!");
	return False;
    }

    for (i=0; i < p->numVertexes; i++)
      iS += interSection(x,y, p, i) ? 1 : 0;

    if (iS % 2)
      return True;
    else 
      return False;
}

#define nextVertex(x,y) ((x+1)%y)

#ifdef __STDC__
static Boolean interSection(double x, double y, Polygon *p, unsigned int i)
#else 
static Boolean interSection(x,y, p, i)
     double x,y;
     Polygon *p;
     unsigned int i;
#endif
{
    double x1 = p->vertexes[i].x,
        y1 = p->vertexes[i].y, 
        x2 = p->vertexes[nextVertex(i,p->numVertexes)].x,
        y2 = p->vertexes[nextVertex(i,p->numVertexes)].y;
    
    if (x1<=x && x2<=x) return False;
    if ((y1<=y && y2<y) || 
	(y1>=y && y2>y)) return False;
    if (y2==y)
      if ((nextYneqY(p,i,y)<y &&
	  y1>y &&
	  x2>x) ||
	  (nextYneqY(p,i,y)>y &&
	  y1<y &&
	  x2>x)) return True;
      else return False;
    if (x < xIS(x1,y1, x2,y2, y)) return True;
    else return False;
}

#ifdef __STDC__
static double nextYneqY(Polygon *p, unsigned int i, double y)
#else
static double nextYneqY(p, i, y)
     Polygon *p;
     unsigned int i;
     int y;
#endif
{
    unsigned int k;
    
    for (k=i+1; k != i; k=nextVertex(k,p->numVertexes))
      if (p->vertexes[k].y != y)
	return p->vertexes[k].y;
    return y;
}

#ifdef __STDC__
static double xIS (double x1, double y1, double x2, double y2, double y)
#else
static double xIS (x1,y1, x2,y2, y)
     double x1,y1, x2,y2, y;
#endif
{
    return (float)x1 + (float)(x2-x1)/(float)(y2-y1)*(float)(y-y1);
}

#ifdef __STDC__
void computePolygonCenter (Polygon *p, double *x, double *y)
#else
void computePolygonCenter (p, x, y)
     Polygon *p; 
     double *x, *y;
#endif
{
    double maxX = (double)INT_MIN,
        minX = (double)INT_MAX,
        maxY = (double)INT_MIN, 
        minY = (double)INT_MAX;
    unsigned int i;
    
    for (i = 0; i < p->numVertexes; i++) {
	if (p->vertexes[i].x > maxX) maxX = p->vertexes[i].x;
	if (p->vertexes[i].x < minX) minX = p->vertexes[i].x;
	if (p->vertexes[i].y > maxY) maxY = p->vertexes[i].y;
	if (p->vertexes[i].y < minY) minY = p->vertexes[i].y;
    }
    *x = (maxX + minX)/2;
    *y = (maxY + minY)/2;
}

#ifdef __STDC__
void shiftPolygon (Polygon *p, double x, double y)
#else
void shiftPolygon (p, x, y)
     Polygon *p;
     double x, y;
#endif
{
    unsigned int i;
    
    for (i = 0; i < p->numVertexes; i++) {
	p->vertexes[i].x += x;
	p->vertexes[i].y += y;
    }
}
