/*******************************************************************************
Modul   	:	RateParObj.c

Purpose		: 	Realisation der Klasse RateParObject

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/


/*
 * RateParObj.c - rate parameter object.

   Class Hierarchie:
   Object -> BaseObj -> RateParObj
 */

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include <stdio.h>

#include "RateParObjP.h"

#define offset(field) XtOffsetOf(RateParObjRec, field)

static XtResource resources[] = {
	{
	XtNrateParName,
	XtCrateParName,
	XtRString,
	sizeof(char*),
	offset(rateParObj.ratepar_name),
	XtRPointer,
	(XtPointer) NULL
	},
	{
	XtNrateParValue,
	XtCrateParValue,
	XtRCardinal,
	sizeof(RateValue *),
	offset(rateParObj.ratepar_value),
	XtRPointer,
	NULL
	},
};

/* Declaration of methods */

static void Initialize();
static void Destroy();
static Boolean SetValues();

/* these Core methods not needed by RateParObj:
 *
 * static void ClassInitialize();
 * static void Realize();
 */

/* the following are private functions unique to RateParObj */
static String getDefaultName();


/* The following are public functions of RateParObj, declared extern
 * in the public include file: */
 String getDisplayedRateString();


RateParObjClassRec rateParObjClassRec = {
    {
    /* object_class fields */
    /* superclass	  	 */ (WidgetClass) &baseObjClassRec,
    /* class_name	  	 */ "rateParClass",
    /* widget_size	  	 */ sizeof(RateParObjRec),
    /* class_initialize   	 */ NULL,
    /* class_part_initialize	 */ NULL,
    /* class_inited       	 */ FALSE,
    /* initialize	  	 */ Initialize,
    /* initialize_hook		 */ NULL,
    /* realize		  	 */ NULL,
    /* actions		  	 */ NULL,
    /* num_actions	  	 */ 0,
    /* resources	  	 */ resources,
    /* num_resources	  	 */ XtNumber(resources),
    /* xrm_class	  	 */ NULLQUARK,
    /* compress_motion	  	 */ 0,
    /* compress_exposure  	 */ 0,
    /* compress_enterleave	 */ 0,
    /* visible_interest	  	 */ 0,
    /* destroy		  	 */ Destroy,
    /* resize		  	 */ NULL,
    /* expose		  	 */ NULL,
    /* set_values	  	 */ SetValues,
    /* set_values_hook		 */ NULL,
    /* set_values_almost	 */ NULL,
    /* get_values_hook		 */ NULL,
    /* accept_focus	 	 */ NULL,
    /* version			 */ XtVersion,
    /* callback_private   	 */ NULL,
    /* tm_table		   	 */ NULL,
    /* query_geometry		 */ NULL,
    /* display_accelerator       */ NULL,
    /* extension                 */ NULL
    },
    {
    /* dummy               */ 0,
    },
    {
    /* num_created               */ 0,
    },
};

WidgetClass rateParObjClass = (WidgetClass) & rateParObjClassRec;
/*******************************************************************************
Name:
        Initialize
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/

/* ARGSUSED */
static void
Initialize(treq, tnew, args, num_args)
Widget treq, tnew;
ArgList args;
Cardinal *num_args;
{
RateParObj new = (RateParObj) tnew;
RateValue *newval,*rval;

#ifdef debug
printf("RatePAR: Initialize \n");
#endif
  if (new->rateParObj.ratepar_name == NULL || 
      *(new->rateParObj.ratepar_name) == '\0')
	new->rateParObj.ratepar_name = getDefaultName(new);
  else
	new->rateParObj.ratepar_name = XtNewString(new->rateParObj.ratepar_name);

  newval = XtNew(RateValue);
  rval = new->rateParObj.ratepar_value;
  if (rval == NULL) {
	newval ->valueType = Ratevalue;
	newval ->values.value = 1.0;
  }
  else {
	newval->valueType = rval->valueType;
	if (newval->valueType == Ratevalue)
	    newval->values.value = rval->values.value;
	else
	    newval->values.definition = rval->values.definition;
  }
  new->rateParObj.ratepar_value = newval;
  rateParObjClassRec.rateParObj_class.num_created++;
}
/*******************************************************************************
Name:
        SetValues
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/


/* ARGSUSED */
static Boolean
SetValues(current, request, new, args, num_args)
Widget current, request, new;
ArgList args;
Cardinal *num_args;
{
RateParObj curpar = (RateParObj) current;
RateParObj newpar = (RateParObj) new;
Boolean do_redisplay = False;
RateValue *rval,*newval;

#ifdef debug
printf("RatePAR: SetValues \n");
#endif

  if (curpar->rateParObj.ratepar_name != newpar->rateParObj.ratepar_name)
	newpar->rateParObj.ratepar_name = 
		XtNewString(newpar->rateParObj.ratepar_name);

  if (curpar->rateParObj.ratepar_value != newpar->rateParObj.ratepar_value) {
	rval = newpar->rateParObj.ratepar_value;
	newval = XtNew(RateValue);
	newval->valueType = rval->valueType;
	if (newval->valueType == Ratevalue)
	   newval->values.value = rval->values.value;
	else
	   newval->values.definition=XtNewString(rval->values.definition);
	newpar->rateParObj.ratepar_value = newval;
  }
  return do_redisplay;
}

/*******************************************************************************
Name:
        Destroy
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/

static void
Destroy(w)
Widget w;
{
RateParObj rpar = (RateParObj) w;

#ifdef debug
printf("RatePAR: Destroy \n");
#endif

  XtFree(rpar->rateParObj.ratepar_name);
  XtFree((char*)rpar->rateParObj.ratepar_value);
}

static String getDefaultName(w)
     Widget w;
{
    char name[100];

    sprintf(name,"rpar%d", rateParObjClassRec.rateParObj_class.num_created+1);
    return XtNewString(name);
}


String getDisplayedRateString(w)
Widget w;
{
    RateParObj rpar = (RateParObj)w;
    char string[200];

    if (rpar->rateParObj.ratepar_value->valueType == Ratevalue) {
	sprintf(string,"%s=%f", 
		rpar->rateParObj.ratepar_name,
		rpar->rateParObj.ratepar_value->values.value);
	return XtNewString(string);
    }
    else {
	sprintf(string,"%s= MD", rpar->rateParObj.ratepar_name);
	return XtNewString(string);
    }
}









