/*******************************************************************************
Modul   	:	RewardObject.c

Purpose		: 	Realisation der Klasse RewardObjekt

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/


/*
 * ResultObject.c - base object.

   Class Hierarchie:
   Object -> ResultObject
 */

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include <stdio.h>

#include "ResultObjectP.h"

#define offset(field) XtOffsetOf(ResultObjectRec, field)

static XtResource resources[] = {
	{
	XtNresultName,
	XtCresultName,
	XtRString,
	sizeof(String),
	offset(resultObject.result_name),
	XtRPointer,
	NULL
	},
	{
	XtNresultDefinition,
	XtCresultDefinition,
	XtRString,
	sizeof(String),
	offset(resultObject.result_definition),
	XtRPointer,
	NULL
	},
};

/* Declaration of methods */

static void Initialize();
static void Destroy();
static Boolean SetValues();

/* these Core methods not needed by ResultObject:
 *
 * static void ClassInitialize();
 * static void Realize();
 */

/* the following are private functions unique to ResultObject */

/* the following are actions of ResultObject */

/* The following are public functions of ResultObject, declared extern
 * in the public include file: */


ResultObjectClassRec resultObjectClassRec = {
    {
    /* object_class fields */
    /* superclass	  	 */ (WidgetClass) &objectClassRec,
    /* class_name	  	 */ resultObjectClassName,
    /* widget_size	  	 */ sizeof(ResultObjectRec),
    /* class_initialize   	 */ NULL,
    /* class_part_initialize	 */ NULL,
    /* class_inited       	 */ FALSE,
    /* initialize	  	 */ Initialize,
    /* initialize_hook		 */ NULL,
    /* realize		  	 */ NULL,
    /* actions		  	 */ NULL,
    /* num_actions	  	 */ 0,
    /* resources	  	 */ resources,
    /* num_resources	  	 */ XtNumber(resources),
    /* xrm_class	  	 */ NULLQUARK,
    /* compress_motion	  	 */ 0,
    /* compress_exposure  	 */ 0,
    /* compress_enterleave	 */ 0,
    /* visible_interest	  	 */ 0,
    /* destroy		  	 */ Destroy,
    /* resize		  	 */ NULL,
    /* expose		  	 */ NULL,
    /* set_values	  	 */ SetValues,
    /* set_values_hook		 */ NULL,
    /* set_values_almost	 */ NULL,
    /* get_values_hook		 */ NULL,
    /* accept_focus	 	 */ NULL,
    /* version			 */ XtVersion,
    /* callback_private   	 */ NULL,
    /* tm_table		   	 */ NULL,
    /* query_geometry		 */ NULL,
    /* display_accelerator       */ NULL,
    /* extension                 */ NULL
    },
    {
    /* dummy               */ 0,
    },
};

WidgetClass resultObjectClass = (WidgetClass) & resultObjectClassRec;

/*******************************************************************************
Name:
        Initialize
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/

/* ARGSUSED */
static void
Initialize(treq, tnew, args, num_args)
Widget treq, tnew;
ArgList args;
Cardinal *num_args;
{
ResultObject  new = (ResultObject ) tnew;

#ifdef debug
printf("ResultObject: Initialize \n");
#endif

 if (new->resultObject.result_name != NULL)
	new->resultObject.result_name = XtNewString(new->resultObject.result_name);
 if (new->resultObject.result_definition != NULL)
	new->resultObject.result_definition = XtNewString(new->resultObject.result_definition);
}

/*******************************************************************************
Name:
        SetValues
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/

/* ARGSUSED */
static Boolean
SetValues(current, request, new, args, num_args)
Widget current, request, new;
ArgList args;
Cardinal *num_args;
{
ResultObject currew = (ResultObject) current;
ResultObject newrew = (ResultObject) new;
Boolean do_redisplay = False;

 if  (newrew->resultObject.result_name != currew->resultObject.result_name)
	newrew->resultObject.result_name=
			XtNewString(newrew->resultObject.result_name);
 if  (newrew->resultObject.result_definition != 
				currew->resultObject.result_definition)
	newrew->resultObject.result_definition=
			XtNewString(newrew->resultObject.result_definition);
  return do_redisplay;
}

/*******************************************************************************
Name:
        Destroy
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/

static void
Destroy(w)
Widget w;
{
	ResultObject rew = (ResultObject) w;

#ifdef debug
printf("Base: Destroy \n");
#endif
    XtFree(rew->resultObject.result_name);
    XtFree(rew->resultObject.result_definition);
}

