/*******************************************************************************
Modul   	:	TagedObj.c

Purpose		: 	Realisation der Klasse TagedObject

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/


/*
 * TagedObj.c - taged object.

   Class Hierarchie:
   Object -> BaseObj -> TagedObj
 */
#include <stdio.h>
#include <string.h>

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include "TagedObjP.h"

#define offset(field) XtOffsetOf(TagedObjRec, field)

static XtResource resources[] = {
	{
	XtNtagName,
	XtCtagName,
	XtRString,
	sizeof(String),
	offset(tagedObj.tagName),
	XtRPointer,
	NULL
	},
	{
	XtNtagXPos,
	XtCtagXPos,
	XtRInt,
	sizeof(int),
	offset(tagedObj.tagXPos),
	XtRImmediate,
	(XtPointer)DEFAULT_TAG_xPOSITION
	},
	{
	XtNtagYPos,
	XtCtagYPos,
	XtRInt,
	sizeof(int),
	offset(tagedObj.tagYPos),
	XtRImmediate,
	(XtPointer)DEFAULT_TAG_yPOSITION
	},
};

/* Declaration of methods */

static void Initialize();
static void Destroy();
static Boolean SetValues();

/* these Core methods not needed by TagedObj:
 *
 * static void ClassInitialize();
 * static void Realize();
 */

/* The following are public functions of TagedObj, declared extern
 * in the public include file: */


TagedObjClassRec tagedObjClassRec = {
    {
    /* object_class fields */
    /* superclass	  	 */ (WidgetClass) &baseObjClassRec,
    /* class_name	  	 */ tagedClassName,
    /* widget_size	  	 */ sizeof(TagedObjRec),
    /* class_initialize   	 */ NULL,
    /* class_part_initialize	 */ NULL,
    /* class_inited       	 */ FALSE,
    /* initialize	  	 */ Initialize,
    /* initialize_hook		 */ NULL,
    /* realize		  	 */ NULL,
    /* actions		  	 */ NULL,
    /* num_actions	  	 */ 0,
    /* resources	  	 */ resources,
    /* num_resources	  	 */ XtNumber(resources),
    /* xrm_class	  	 */ NULLQUARK,
    /* compress_motion	  	 */ 0,
    /* compress_exposure  	 */ 0,
    /* compress_enterleave	 */ 0,
    /* visible_interest	  	 */ 0,
    /* destroy		  	 */ Destroy,
    /* resize		  	 */ NULL,
    /* expose		  	 */ NULL,
    /* set_values	  	 */ SetValues,
    /* set_values_hook		 */ NULL,
    /* set_values_almost	 */ NULL,
    /* get_values_hook		 */ NULL,
    /* accept_focus	 	 */ NULL,
    /* version			 */ XtVersion,
    /* callback_private   	 */ NULL,
    /* tm_table		   	 */ NULL,
    /* query_geometry		 */ NULL,
    /* display_accelerator       */ NULL,
    /* extension                 */ NULL
    },
    {
    /* dummy               */ 0,
    },
    {
    /* dummy               */ 0,
    },
};

WidgetClass tagedObjClass = (WidgetClass) & tagedObjClassRec;

/*******************************************************************************
Name:
        Initialize
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/

/* ARGSUSED */
static void
Initialize(treq, tnew, args, num_args)
Widget treq, tnew;
ArgList args;
Cardinal *num_args;
{
TagedObj new = (TagedObj) tnew;

#ifdef debug
printf("TagedObj: Initialize \n");
#endif

  if (new->tagedObj.tagName != NULL) 
	new->tagedObj.tagName = XtNewString(new->tagedObj.tagName);

/* now we've got relative coordinates to the objekt, so neg. is ok */
/*
  if (new->tagedObj.tagXPos < 0 || new->tagedObj.tagYPos < 0) 
  {
	new->tagedObj.tagXPos = DEFAULT_TAG_xPOSITION;
	new->tagedObj.tagYPos = DEFAULT_TAG_yPOSITION;
  }
*/
}

/*******************************************************************************
Name:
        SetValues
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/
/* ARGSUSED */
static Boolean
SetValues(current, request, new, args, num_args)
Widget current, request, new;
ArgList args;
Cardinal *num_args;
{
TagedObj  curtag = (TagedObj) current;
TagedObj newtag = (TagedObj) new;
Boolean do_redisplay = False;

#ifdef debug
printf("TagObj: SetValues \n");
#endif
  if (curtag->tagedObj.tagName != newtag->tagedObj.tagName) 
	newtag->tagedObj.tagName = XtNewString(newtag->tagedObj.tagName);

/* now we've got relative coordinates to the objekt, so neg. is ok */
/*
  if (newtag->tagedObj.tagXPos < 0 || newtag->tagedObj.tagYPos < 0) 
  {
	newtag->tagedObj.tagXPos = DEFAULT_TAG_xPOSITION;
	newtag->tagedObj.tagYPos = DEFAULT_TAG_yPOSITION;
  }
*/
  return do_redisplay;
}


/*******************************************************************************
Name:
        Destroy
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/
static void
Destroy(w)
Widget w;
{
TagedObj tagobj = (TagedObj) w;

#ifdef debug
printf("Taged: Destroy \n");
#endif
    XtFree(tagobj->tagedObj.tagName);
}


