/*******************************************************************************
Modul   	:	TransObj.c

Purpose		: 	Realisation der Klasse TransObjekt

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Martin M"uller

History		:	release 1: October 1992
*******************************************************************************/

/*
 * TransObj.c - base object.

   Class Hierarchie:
   Object -> BaseObj -> taged -> TransObj
 */

#include <stdio.h>

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include "RateParObj.h"
#include "ArcObj.h"
#include "TransObjP.h"

#include "util.h"

static void addMethod();
static void delMethod();

#define offset(field) XtOffsetOf(TransObjRec, field)

#define NumTransPoints 5

static XtResource resources[] = {
	{
	XtNtransType,
	XtCtransType,
	XtRtransType,
	sizeof(TransType),
	offset(transObj.transType),
	XtRImmediate,
	(XtPointer)IMMEDIATE,
	},
	{
	XtNtransOrient,
	XtCtransOrient,
	XtRtransOrient,
	sizeof(TransOrientation),
	offset(transObj.transOrient),
	XtRImmediate,
	(XtPointer)ZERO,
	},
	{
	XtNpriority,
	XtCpriority,
	XtRCardinal,
	sizeof(Cardinal),
	offset(transObj.priority),
	XtRImmediate,
	(XtPointer)PRIORITY_UNDEFINED,
	},
	{
	XtNenablingDependence,
	XtCenablingDependence,
	XtRenablingDependence,
	sizeof(EnablingDependence),
	offset(transObj.enablingDependence),
	XtRImmediate,
	(XtPointer)SINGLE_SERVER,
	},
	{
	XtNrates,
	XtCrates,
	XtRrates,
	sizeof(caddr_t),
	offset(transObj.rate),
	XtRImmediate,
	(XtPointer)NULL,
	},
	{
	XtNratePos_x,
	XtCratePos_x,
	XtRInt,
	sizeof(int),
	offset(transObj.ratePos_x),
	XtRImmediate,
	(XtPointer)DEFAULT_RATE_POSITION,
	},
	{
	XtNratePos_y,
	XtCratePos_y,
	XtRInt,
	sizeof(int),
	offset(transObj.ratePos_y),
	XtRImmediate,
	(XtPointer)DEFAULT_RATE_POSITION,
	},
	{
	XtNenabled,
	XtCEnabled,
	XtRBoolean,
	sizeof(Boolean),
	offset(transObj.enabled),
	XtRImmediate,
	(XtPointer)True,
        },
        {
        XtNinpConnection,
	XtCInpConnection,
	XtRConnections,
	sizeof(Connections),
	offset(transObj.inpConnection),
	XtRImmediate,
	(XtPointer)NULL
        },
	{
	XtNoutpConnection,
	XtCOutpConnection,
	XtRConnections,
	sizeof(Connections),
	offset(transObj.outpConnection),
	XtRImmediate,
	(XtPointer)NULL
        },
	{
	XtNinhibitConnection,
	XtCInhibitConnection,
	XtRConnections,
	sizeof(Connections),
	offset(transObj.inhibitConnection),
	XtRImmediate,
	(XtPointer)NULL
        },
	{
	XtNnumInputs,
	XtCNumInputs,
	XtRCardinal,
	sizeof(Cardinal),
	offset(transObj.numInputs),
	XtRImmediate,
	(XtPointer)0
	},
	{
	XtNnumOutputs,
	XtCNumOutputs,
	XtRCardinal,
	sizeof(Cardinal),
	offset(transObj.numOutputs),
	XtRImmediate,
	(XtPointer)0
	},
	{
	XtNnumInhibits,
	XtCNumInhibits,
	XtRCardinal,
	sizeof(Cardinal),
	offset(transObj.numInhibits),
	XtRImmediate,
	(XtPointer)0
	},
	{
	XtNaddConnection,
	XtCAddConnection,
	XtRFunction,
	sizeof(caddr_t),
	offset(transObj.addConnectionMethod),
	XtRImmediate,
	(XtPointer)addMethod
	},
	{
	XtNdelConnection,
	XtCDelConnection,
	XtRFunction,
	sizeof(caddr_t),
	offset(transObj.delConnectionMethod),
	XtRImmediate,
	(XtPointer)delMethod
	},
	{
	XtNthroughput,
	XtCThroughput,
        XtRFloat,
	sizeof(double),
	offset(transObj.throughput),
	XtRString,
        "0.0"
	},
	{
	XtNselected,
	XtCSelected,
	XtRBoolean,
	sizeof(Boolean),
	offset(transObj.selected),
	XtRImmediate,
	(XtPointer)False
        },
	{
	XtNcopy,
	XtCCopy,
	XtRWidget,
	sizeof(Widget),
	offset(transObj.copy),
	XtRImmediate,
	(XtPointer)NULL
        },
	{
	XtNstartPhi,
	XtCStartPhi,
	XtRFloat,
	sizeof(double),
	offset(transObj.startPhi),
	XtRString,
	"0.0"
	},
	{
	XtNstartOrient,
	XtCStartOrient,
	XtRtransOrient,
	sizeof(TransOrientation),
	offset(transObj.startOrient),
	XtRImmediate,
	(XtPointer)ZERO,
	},
	/*new*/
	{
	XtNphase,
	XtCPhase,
	XtRCardinal,
	sizeof(Cardinal),
	offset(transObj.phase),
	XtRImmediate,
	(XtPointer)LOWEST_PHASE
	},
	{
	XtNgroup,
	XtCGroup,
	XtRInt,
	sizeof(int),
	offset(transObj.group),
	XtRImmediate,
	(XtPointer)DEFAULT_GROUP
	},
	{
	XtNgroup_weight,
	XtCGroup_weight,
	XtRFloat,
	sizeof(double),
	offset(transObj.group_weight),
	XtRString,
	"1.0"
	},
	{
	XtNracePolicy,
	XtCracePolicy,
	XtRracePolicy,
	sizeof(RacePolicy),
	offset(transObj.racePolicy),
	XtRImmediate,
	(XtPointer)RACE_E,
	},
};

/*importierte Funktionen*/

extern int GetTimeMode();

/* Declaration of methods */
/* core */
static void Initialize();
static void Destroy();
static Boolean SetValues();

/* TransObj */

/* these Core methods not needed by TransObj:
 *
 * static void ClassInitialize();
 * static void ClassPartInitialize();
 */

/* the following are private functions unique to TransObj */

static String getDefaultTagName();

/* The following are public functions of TransObj, declared extern
 * in the public include file: */


TransObjClassRec transObjClassRec = {
    {
    /* object_class fields */
    /* superclass	  	 */ (WidgetClass) &tagedObjClassRec,
    /* class_name	  	 */ transClassName,
    /* widget_size	  	 */ sizeof(TransObjRec),
    /* class_initialize   	 */ NULL,
    /* class_part_initialize	 */ NULL,
    /* class_inited       	 */ FALSE,
    /* initialize	  	 */ Initialize,
    /* initialize_hook		 */ NULL,
    /* realize		  	 */ NULL,
    /* actions		  	 */ NULL,
    /* num_actions	  	 */ 0,
    /* resources	  	 */ resources,
    /* num_resources	  	 */ XtNumber(resources),
    /* xrm_class	  	 */ NULLQUARK,
    /* compress_motion	  	 */ 0,
    /* compress_exposure  	 */ 0,
    /* compress_enterleave	 */ 0,
    /* visible_interest	  	 */ 0,
    /* destroy		  	 */ Destroy,
    /* resize		  	 */ NULL,
    /* expose		  	 */ NULL,
    /* set_values	  	 */ SetValues,
    /* set_values_hook		 */ NULL,
    /* set_values_almost	 */ NULL,
    /* get_values_hook		 */ NULL,
    /* accept_focus	 	 */ NULL,
    /* version			 */ XtVersion,
    /* callback_private   	 */ NULL,
    /* tm_table		   	 */ NULL,
    /* query_geometry		 */ NULL,
    /* display_accelerator       */ NULL,
    /* extension                 */ NULL
    },
    {
    /* dummyB	                 */ 0,
    },
    {
    /* dummyT          		 */ 0,
    },
    {
    /* dummy trans         	 */ 0,
    },
};

WidgetClass transObjClass = (WidgetClass) & transObjClassRec;


/*******************************************************************************
Name:
        Initialize
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/

/* ARGSUSED */
static void
Initialize(treq, tnew, args, num_args)
Widget treq, tnew;
ArgList args;
Cardinal *num_args;
{
TransObj new = (TransObj) tnew;
Rate* rate;

#ifdef debug
printf("TransObj: Initialize \n");
#endif
  /* solving conflict with taged object */
  if (new->tagedObj.tagName == NULL) {
	new->tagedObj.tagName = getDefaultTagName(new);
  }
  /* checking transition kind */
  switch (new->transObj.transType ) {
  case GENERAL:
  case EXPONENTIEL:
  case DETERMINISTIC:
  case IMMEDIATE:	break;
  default:
	XtWarning("transition initialize: missing transition kind set to 'immediate'");
	new->transObj.transType = IMMEDIATE;
  }
  /* checking orientation */
  switch (new->transObj.transOrient) {
  case ZERO:
  case FORTYFIVE:
  case NINETY:
  case MINUSFORTYFIVE: break;
  default:
	XtWarning("Transition initialize: no valid orientation, reset to 'zero'");
	new->transObj.transOrient = ZERO;
  }
  /* checking priority */  
  /*new..Post Selection Priority for timed Transitions*/

/*   if (new->transObj.transType == GENERAL  */
/*       && new->transObj.priority !=  PRIORITY_UNDEFINED) { */
/* 	XtWarning("Transition initialize: no priority defined for gen. transitions");  */
/* 	new->transObj.priority = PRIORITY_UNDEFINED; */
/*   } */
/*   else */
    if (new->transObj.transType == IMMEDIATE 
	&& new->transObj.priority < LOWEST_PRI) {
      new->transObj.priority = LOWEST_PRI;
    }
    else
      if (new->transObj.transType != IMMEDIATE
	  && new->transObj.priority < LOWEST_PRI) {
	new->transObj.priority = PRIORITY_UNDEFINED;
      }
  /* checking enabling dependence */
  switch (new->transObj.enablingDependence) { 
  case INFINITE_SERVER:
	if (new->transObj.transType == IMMEDIATE) {
	    XtWarning("Transition initialize: infinite server not defined for immediate transitions");
	    new->transObj.enablingDependence = SINGLE_SERVER;
	}
	break;
  case SINGLE_SERVER:break;
  default: 
	XtWarning("Transition initialize: no valid enabling dependence, reset to default"); 
	new->transObj.enablingDependence =  SINGLE_SERVER;
  }
  /*new*/
   /* checking race policy */
  switch (new->transObj.racePolicy) { 
  case RACE_A:
	if (new->transObj.transType == IMMEDIATE) {
	    XtWarning("Transition initialize: RA race policy not defined for immediate transitions");
	    new->transObj.racePolicy = RACE_E;
	}
	break;
  case RACE_E:break;
  default: 
	XtWarning("Transition initialize: no valid race policy, reset to default"); 
	new->transObj.racePolicy =  RACE_E;
  }
  /* checking rate setting */
  if ((rate=XtNew(Rate))==NULL) {
	XtError("Transition initialize: Can't allocate mem for new rate");
  }
  if (new->transObj.rate == NULL) /* Real NEW Object */
  {	
  	if(new->transObj.transType == GENERAL )
	{       
		rate->type = PMF_FUNC;
		rate->values.pmf_function = XtNewString(DEFAULT_PMF_FUNCTION);
	}
	else /* every other case */
	{
		rate->type = RATEVALUE;
		rate->values.ratevalue = 1.0;
	}
	rate->enable_fkt=NULL;
	
  }
  else {
	rate->type = new->transObj.rate->type;
	switch (rate -> type) { 
	case RATEVALUE:
	    rate->values.ratevalue = new->transObj.rate->values.ratevalue;
	    break;
	case RATEPARAM: 
	    rate->values.rateparam = new->transObj.rate->values.rateparam;
	    break;
	case MD_DEF: 
    	rate->values.md_definition = 
			XtNewString(new->transObj.rate->values.md_definition);
	    break;
	case PMF_FUNC:
    	rate->values.pmf_function = 
			XtNewString(new->transObj.rate->values.pmf_function);
        case DATAFILE:
        rate->values.DataFile = 
			XtNewString(new->transObj.rate->values.DataFile);
                      
	    break;
	default:
	    XtWarning("Transition initialize: no valid rate type, reset to default (1.0)");
	    rate->type = RATEVALUE;
	    rate->values.ratevalue = 1.0;
	}
    rate->enable_fkt=new->transObj.rate->enable_fkt;
  }
  new->transObj.rate = rate;

  /* ratePos is now relative, so checking is not useful ...
  if (new->transObj.ratePos_x < 0 || new->transObj.ratePos_y < 0 ) {
	new->transObj.ratePos_x = 
	new->transObj.ratePos_y = 
  }*/

  new->transObj.inpConnection = NULL;
  new->transObj.outpConnection = NULL;
  new->transObj.inhibitConnection = NULL;
  new->transObj.numInputs = 0;
  new->transObj.numOutputs = 0;
  new->transObj.numInhibits = 0;
  new->transObj.throughput = 0.0;
  new->transObj.selected = False;

  /*new*/
  new->transObj.phase = LOWEST_PHASE;
  new->transObj.group = DEFAULT_GROUP;
  new->transObj.group_weight = DEFAULT_GROUP_WEIGHT;

  /* private */

  transObjClassRec.transObj_class.num_created++;
} /*Initialize*/

/*******************************************************************************
Name:
        SetValues
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/

/* ARGSUSED */
static Boolean
SetValues(current, request, new, args, num_args)
Widget current, request, new;
ArgList args;
Cardinal *num_args;
{
TransObj curtrans = (TransObj) current;
TransObj newtrans = (TransObj) new;
Rate * rate;
Boolean do_redisplay = False;
Connections connections;

  /* checking transition kind */
  if (newtrans->transObj.transType != curtrans->transObj.transType) { 
	switch (newtrans->transObj.transType ) {
	case GENERAL:
		if (newtrans->transObj.priority < LOWEST_PRI)
			newtrans->transObj.priority = PRIORITY_UNDEFINED;
		newtrans->transObj.enablingDependence = SINGLE_SERVER;
                newtrans->transObj.rate->type = PMF_FUNC;
                newtrans->transObj.rate->values.pmf_function =
                                	XtNewString(DEFAULT_PMF_FUNCTION);

        break;
	case EXPONENTIEL:
		if(newtrans->transObj.rate->type == PMF_FUNC ||
                   newtrans->transObj.rate->type == DATAFILE) {
		    newtrans->transObj.rate->type = RATEVALUE;
		    newtrans->transObj.rate->values.ratevalue = 1.0;
		}
		if (newtrans->transObj.priority < LOWEST_PRI)
			newtrans->transObj.priority = PRIORITY_UNDEFINED;
		break;
  	case DETERMINISTIC:
		if (newtrans->transObj.priority < LOWEST_PRI)
			newtrans->transObj.priority = PRIORITY_UNDEFINED;
		newtrans->transObj.enablingDependence = SINGLE_SERVER;
		if (newtrans->transObj.rate->type == PMF_FUNC 
                ||  newtrans->transObj.rate->type == DATAFILE
		||  newtrans->transObj.rate->type == MD_DEF) {
		    newtrans->transObj.rate->type = RATEVALUE;
		    newtrans->transObj.rate->values.ratevalue = 1.0;
		}
		break;
  	case IMMEDIATE:	
		newtrans->transObj.enablingDependence = SINGLE_SERVER;
		if (newtrans->transObj.priority < LOWEST_PRI)
			newtrans->transObj.priority = LOWEST_PRI;
		if (newtrans->transObj.rate->type == PMF_FUNC 
                 || newtrans->transObj.rate->type == DATAFILE
		 || newtrans->transObj.rate->type == MD_DEF) {
		    newtrans->transObj.rate->type = RATEVALUE;
		    newtrans->transObj.rate->values.ratevalue = 1.0;
		}
		break;
  	default:
	    XtWarning("transition setvalues: unknown transition kind, reset to old kind");
	    newtrans->transObj.transType = curtrans->transObj.transType;
  	}
  }
  /* checking orientation */
  if (newtrans->transObj.transOrient != curtrans->transObj.transOrient) { 
  	switch (newtrans->transObj.transOrient) {
  	case ZERO:
  	case FORTYFIVE:
  	case NINETY:
  	case MINUSFORTYFIVE: break;
  	default:
	    XtWarning("Transition setvalues: no valid orientation, reset to old orientation ");
	    newtrans->transObj.transOrient = curtrans->transObj.transOrient;
      }
  }
  /* checking priority */
  if (newtrans->transObj.priority != curtrans->transObj.priority) {  
    /*new ... TimeMode...*/
    if (!GetTimeMode()) {        /*ContTimeMode ...*/
      if (newtrans->transObj.transType != IMMEDIATE && 
	  newtrans->transObj.priority != PRIORITY_UNDEFINED) {
	XtWarning("Transition setvalues: priority only defined for immediate Transitions");
	newtrans->transObj.priority = PRIORITY_UNDEFINED;
      }
      else
  	if (newtrans->transObj.transType == IMMEDIATE &&
	    newtrans->transObj.priority < LOWEST_PRI) {
	  XtWarning("Transition setvalues: no valid priority, reset to old priority"); 
	  newtrans->transObj.priority =  curtrans->transObj.priority;
	}
    }
    else { /*DiscTimeMode ... new PostSelectionPriority for timed Transitions*/
      if  (newtrans->transObj.transType == IMMEDIATE && 
	   newtrans->transObj.priority < LOWEST_PRI) {
 	XtWarning("Transition setvalues: no priority defined for immediate Transition"); 
	newtrans->transObj.priority = LOWEST_PRI;
      }
 
/*       if  (newtrans->transObj.transType == GENERAL &&  */
/* 	   newtrans->transObj.priority != PRIORITY_UNDEFINED) { */
/* 	XtWarning("Transition setvalues: no priority defined"); */
/* 	newtrans->transObj.priority = PRIORITY_UNDEFINED; */
/*       } */
/*       else */
/*   	if (newtrans->transObj.transType != GENERAL && */
/* 	    newtrans->transObj.priority < LOWEST_PRI) { */
/* 	  XtWarning("Transition setvalues: no valid priority, reset to old priority");  */
/* 	  newtrans->transObj.priority =  curtrans->transObj.priority; */
/* 	} */
    }
  }
  /* checking enabling dependence */
  if (newtrans->transObj.enablingDependence != curtrans->transObj.enablingDependence) { 
  	switch (newtrans->transObj.enablingDependence) { 
  	case INFINITE_SERVER:
	    if (newtrans->transObj.transType == IMMEDIATE) {
		XtWarning("Transition setvalues: infinite server not defined for immediate transitions");
		newtrans->transObj.enablingDependence = SINGLE_SERVER;
	    }
/*	    if (newtrans->transObj.transType == DETERMINISTIC) {
		XtWarning("Transition setvalues: infinite server not yet defined for immediate transitions");
		newtrans->transObj.enablingDependence = SINGLE_SERVER;
	    }*/
	    break;
  	case SINGLE_SERVER:break;
  	default: 
	    XtWarning("Transition setvalues: no valid enabling dependence, reset to old dependency"); 
	    newtrans->transObj.enablingDependence =  curtrans->transObj.enablingDependence;
	}
  }
  /*new*/
  /* checking race policy */
  if (newtrans->transObj.racePolicy != curtrans->transObj.racePolicy) { 
  	switch (newtrans->transObj.racePolicy) { 
  	case RACE_A:
	    if (newtrans->transObj.transType == IMMEDIATE) {
		XtWarning("Transition setvalues: RA race policy not defined for immediate transitions");
		newtrans->transObj.enablingDependence = RACE_E;
	    }
	    break;
  	case RACE_E:break;
  	default: 
	    XtWarning("Transition setvalues: no valid race policy, reset to old race policy"); 
	    newtrans->transObj.racePolicy =  curtrans->transObj.racePolicy;
	}
  }
  /* checking rate setting */
  if (newtrans->transObj.rate != curtrans->transObj.rate) { 
  	if ((rate=XtNew(Rate))==NULL) {
	    XtError("Transition initialize: Can't allocate mem for new rate");
  	}
	rate->enable_fkt=NULL;
	if (newtrans->transObj.rate == NULL) {
	    XtWarning("Transition setvalues: no rate found, using old setting");
	    rate->type = curtrans->transObj.rate->type;
	    switch (rate->type) {
	    case RATEVALUE:
	        rate->values.ratevalue = curtrans->transObj.rate->values.ratevalue;
		break;
	    case RATEPARAM:
	        rate->values.rateparam = curtrans->transObj.rate->values.rateparam;
		break;
	    case MD_DEF:
	        rate->values.md_definition = 
		    	XtNewString(curtrans->transObj.rate->values.md_definition);
		break;
	    case PMF_FUNC:
	        rate->values.pmf_function = 
		    	XtNewString(curtrans->transObj.rate->values.pmf_function);
            case DATAFILE:
	        rate->values.DataFile = 
		    	XtNewString(curtrans->transObj.rate->values.DataFile);
		break;
	    default: ;
	    }
  	}
  	else {
	  	*rate=*curtrans->transObj.rate;
	    rate->type = newtrans->transObj.rate->type;
	    switch (rate -> type) { 
	    case RATEVALUE:
	        rate->values.ratevalue = newtrans->transObj.rate->values.ratevalue;
	        break;
	    case RATEPARAM: 
	        rate->values.rateparam = newtrans->transObj.rate->values.rateparam;
	        break;
	    case MD_DEF: 
		rate->values.md_definition = 
		  XtNewString(newtrans->transObj.rate->values.md_definition);
	        break;
	    case PMF_FUNC: 
		rate->values.pmf_function = 
		  XtNewString(newtrans->transObj.rate->values.pmf_function);
            case DATAFILE:
		rate->values.DataFile = 
		  XtNewString(newtrans->transObj.rate->values.DataFile);
                  
	        break;
	    default:
	        XtWarning
	   ("Transition setvalues: no valid rate type, reset to default (1.0)");
	        rate->type = curtrans->transObj.rate->type;
	    	switch (rate -> type) { 
	    	case RATEVALUE:
	            rate->values.ratevalue = curtrans->transObj.rate->values.ratevalue;
		    break;
	    	case RATEPARAM:
	            rate->values.rateparam = curtrans->transObj.rate->values.rateparam;
		    break;
	    	case MD_DEF:
	            rate->values.md_definition =
				XtNewString(curtrans->transObj.rate->values.md_definition);
			break;
		    case PMF_FUNC:
		        rate->values.pmf_function = 
			    	XtNewString(curtrans->transObj.rate->values.pmf_function);
                    case DATAFILE:
		        rate->values.DataFile = 
			    	XtNewString(curtrans->transObj.rate->values.DataFile);
                    
			break;
	    	default: ;
		}
	    }
  	}
  	newtrans->transObj.rate = rate;
  	if (newtrans->transObj.ratePos_x < 0 || newtrans->transObj.ratePos_y < 0 ) {
	    newtrans->transObj.ratePos_x = DEFAULT_RATE_POSITION;
	    newtrans->transObj.ratePos_y = DEFAULT_RATE_POSITION;
  	}
  }

  /* copy changed Connections */
  if (newtrans->transObj.inpConnection != curtrans->transObj.inpConnection) { 
    connections = 
      (Connections)XtCalloc(newtrans->transObj.numInputs, sizeof(Connection));
    memcpy(connections, newtrans->transObj.inpConnection, 
	   newtrans->transObj.numInputs*sizeof(Connection));
    newtrans->transObj.inpConnection = connections;
  }
  if (newtrans->transObj.outpConnection != curtrans->transObj.outpConnection) { 
    connections = 
      (Connections)XtCalloc(newtrans->transObj.numOutputs, sizeof(Connection));
    memcpy(connections, newtrans->transObj.outpConnection, 
	   newtrans->transObj.numOutputs*sizeof(Connection));
    newtrans->transObj.outpConnection = connections;
  }
  if (newtrans->transObj.inhibitConnection 
      != curtrans->transObj.inhibitConnection) { 
    connections = 
      (Connections)XtCalloc(newtrans->transObj.numInhibits, sizeof(Connection));
    memcpy(connections, newtrans->transObj.inhibitConnection, 
	   newtrans->transObj.numInhibits*sizeof(Connection));
    newtrans->transObj.inhibitConnection = connections;
  }

  return  do_redisplay;
}

/*******************************************************************************
Name:
        Destroy
--------------------------------------------------------------------------------
Funktionstyp:
        Methode
*******************************************************************************/

static void
Destroy(w)
Widget w;
{
    TransObj trans = (TransObj) w;

#ifdef debug
    printf("TransObj: Destroy \n");
#endif
    if(trans->transObj.rate->type == PMF_FUNC)
		XtFree((char*)trans->transObj.rate->values.pmf_function);
    if(trans->transObj.rate->type == DATAFILE)
		XtFree((char*)trans->transObj.rate->values.DataFile);                
    if(trans->transObj.rate->type == MD_DEF)
	    XtFree((char*)trans->transObj.rate->values.md_definition);
    
	XtFree((char*)trans->transObj.rate);
    XtFree((char*)trans->transObj.inpConnection);
    XtFree((char*)trans->transObj.outpConnection);
    XtFree((char*)trans->transObj.inhibitConnection);
}

#ifdef __STDC__
extern String nwUniqueTransitionName(TransType type);
#else
extern String nwUniqueTransitionName();
#endif

/*****************************************************************************/
static char* getDefaultTagName(w)
/*****************************************************************************/
Widget w;

{
    TransObj trans = (TransObj)w;
    return nwUniqueTransitionName(trans->transObj.transType);
}

#ifdef __STDC__ 
/*****************************************************************************/
static void addMethod 
/*****************************************************************************/
    (TransObj trans,
     ArcType type, 
     PlaceObj place, 
     Multiplicity *mult)
#else
static void addMethod (trans, type, place, mult)
    TransObj trans;
    ArcType type;
    PlaceObj place;
    Multiplicity *mult;
#endif
{
    Connections cs;
    Cardinal num;
    
    if (place==NULL) 
      return;

    switch (type) {
    case INPUTARC:
	cs = trans->transObj.inpConnection;
	num = trans->transObj.numInputs;
	break;
    case OUTPUTARC:
	cs = trans->transObj.outpConnection;
	num = trans->transObj.numOutputs;
	break;
    case INHIBITORARC:
	cs = trans->transObj.inhibitConnection;
	num = trans->transObj.numInhibits;
	break;
    }/*switch*/

    cs = (Connections)XtRealloc((char*)cs, sizeof(Connection)*(num+1));

    cs[num].place = place;
    cs[num].multiplicity = mult;

    switch (type) {
    case INPUTARC:
	trans->transObj.inpConnection = cs;
	trans->transObj.numInputs++;
	break;
    case OUTPUTARC:
	trans->transObj.outpConnection = cs;
	trans->transObj.numOutputs++;
	break;
    case INHIBITORARC:
	trans->transObj.inhibitConnection = cs;
	trans->transObj.numInhibits++;
	break;
    }/*switch*/
    
}

#ifdef __STDC__ 
/*****************************************************************************/
static void delMethod 
/*****************************************************************************/
    (TransObj trans,
     ArcType type, 
     PlaceObj place)
#else
static void delMethod (trans, type, place)
    TransObj trans;
    ArcType type;
    PlaceObj place;
#endif
{
    Connections cs;
    Cardinal num, i;
    
    switch (type) {
    case INPUTARC:
	cs = trans->transObj.inpConnection;
	num = trans->transObj.numInputs;
	break;
    case OUTPUTARC:
	cs = trans->transObj.outpConnection;
	num = trans->transObj.numOutputs;
	break;
    case INHIBITORARC:
	cs = trans->transObj.inhibitConnection;
	num = trans->transObj.numInhibits;
	break;
    }/*switch*/

    if (!num) {
	return;
    }

    for (i=0; i<num; i++) {
	if (cs[i].place==place) 
	    cs[i]=cs[--num];   /* overwrite the connection to be deleted 
				      with the last connection in list */
    }

    /* Realloc the table-size so that the last element fall from it */
    cs = (Connections)XtRealloc((char*)cs, sizeof(Connection)*(num));

    switch (type) {
    case INPUTARC:
	trans->transObj.inpConnection = cs;
	trans->transObj.numInputs = num;
	break;
    case OUTPUTARC:
	trans->transObj.outpConnection = cs;
	trans->transObj.numOutputs = num;
	break;
    case INHIBITORARC:
	trans->transObj.inhibitConnection = cs;
	trans->transObj.numInhibits = num;
	break;
    }
}

