%{
extern "C" {
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
}
#include "inc_classes.h" 
#include "inc_lex_yacc_globals.h" 

/*------------------------------------------------------*/
/*			fehler				*/
/*------------------------------------------------------*/
void 	yyerror		(char*);
void	SetError	(int);

/*------------------------------------------------------*/
/*		string/path-handling			*/
/*------------------------------------------------------*/
char* 	StripOfPath	(char*);
char* 	GetNumberString	(int);
int 	GetNumberSize	(int);

/*------------------------------------------------------*/
/*			create				*/
/*------------------------------------------------------*/
int 	MakeActPageName	(char*);
void	MakeEvalCell	(Cell*);

/*------------------------------------------------------*/
/*			save				*/
/*------------------------------------------------------*/
int	SaveToken	(char*, char*, char*); 
int 	SaveDefinition	(Definition*);


%}
%union
{	
	//-----------------------------------------------
	//	"elementare" typen
	//-----------------------------------------------
	double  		d_val;
	int     		i_val;
	char*   		s_val;
	Operator		_Operator;
	//-----------------------------------------------
	//	"elmentare" Klassen
	//-----------------------------------------------
	Identifier*		_IdentifierPtr;
	Cell*			_CellPtr;
	//-----------------------------------------------
	// 	Definitions Klassen
	//-----------------------------------------------
	Definition*		_DefinitionPtr;
	DefinitionList*		_DefinitionListPtr;
}
/*------------------------------------------------------*/
/*			sections			*/
/*------------------------------------------------------*/
%token  DEFINITION 
%token	PAGE 
%token	EVALUATE

/*------------------------------------------------------*/
/*			signs				*/
/*------------------------------------------------------*/
%token  COMMA POINT DOUBLEPOINT QUESTIONMARK 
%token	OPEN_I OPEN_II OPEN_III CLOSE_I CLOSE_II CLOSE_III

/*------------------------------------------------------*/
/*			operator			*/
/*------------------------------------------------------*/
%token  PLUS MINUS DIVIDE MULTIPLY TIMES
%token	NOT AND OR WAHR FALSCH
%token  EQUAL LESS GREATER NOT_EQUAL 
%token	LESS_OR_EQUAL GREATER_OR_EQUAL 

/*------------------------------------------------------*/
/*			mset				*/
/*------------------------------------------------------*/
%token  E
%token  E_MSET 
%token 	P_MSET 

/*------------------------------------------------------*/
/*			miscs				*/
/*------------------------------------------------------*/
%token	INT_CONST 
%token	REAL_CONST 
%token	NUMERIC 
%token	LOGIC
%token	STRING
%token  ERROR 

/*------------------------------------------------------*/
/*			priority			*/
/*------------------------------------------------------*/
%left   NOT AND OR EQUAL
%left   PLUS MINUS
%left   MULTIPLY DIVIDE

/*------------------------------------------------------*/
/*			reals				*/
/*------------------------------------------------------*/
%type   <_CellPtr>      	NUMERIC		
%type	<i_val> 		INT_CONST	
%type   <d_val>			REAL_CONST
%type	<_CellPtr>		int_const 
%type   <_CellPtr>              real_const 
%type   <_CellPtr>              real_expr 

/*------------------------------------------------------*/
/*			logic				*/
/*------------------------------------------------------*/
%type   <_CellPtr>		LOGIC
%type	<_Operator>		comp_op
%type   <_CellPtr>              comparison 
%type   <_CellPtr>              logic_condition 

/*------------------------------------------------------*/
/*		      multisets				*/
/*------------------------------------------------------*/
%type	<_CellPtr>		multiset_expr
%type   <_CellPtr>      	E_MSET 		
%type   <_CellPtr>		P_MSET
%type	<_CellPtr>		e_mset 
%type	<_CellPtr>		p_mset 
%type	<_CellPtr>		e_mset_1
%type	<_CellPtr>		p_mset_1
%type	<_CellPtr>		e_mset_0 
%type	<_CellPtr>		p_mset_0 
%type	<_CellPtr>		e_mset_const 
%type   <_CellPtr>              p_mset_const
%type   <i_val> 		unit
%type	<s_val>			part_version 

/*------------------------------------------------------*/
/*		      definitions			*/
/*------------------------------------------------------*/
%type	<_DefinitionPtr> 	definition	
%type	<i_val> 		definitions	
%type	<i_val> 		definition_def	
%type	<i_val> 		all_definitions	

/*------------------------------------------------------*/
/*		      miscs				*/
/*------------------------------------------------------*/
%type	<s_val> 		STRING 
%type	<i_val> 		page	
%type	<i_val> 		evaluate	

%%
/********************************************************************************************************************/
/************************************************ Gesamter Eintrag **************************************************/
/********************************************************************************************************************/
eval_entry		: all_definitions page evaluate				{ ; 				    } 

/********************************************************************************************************************/
/************************************************ Definitionen ******************************************************/
/********************************************************************************************************************/
all_definitions         : definition_def                                        { $$ = 0;			    } 
                        | definition_def definitions                            { $$ = 0; 	    		    }
/*------------------------------------------------------------------------------------------------------------------*/
definitions             : definition						
			{ 
				if (SaveDefinition($1))
				{
					SetError(DEF_EXISTS_ERR);
					return  (DEF_EXISTS_ERR);
				}
				$$ = 0;
			}
			| definitions OR definition
			{ 
				if (SaveDefinition($3))
				{
					SetError(DEF_EXISTS_ERR);
					return  (DEF_EXISTS_ERR);
				}
				$$ = 0;
			}
/*------------------------------------------------------------------------------------------------------------------*/
definition              : STRING EQUAL real_expr                                { $$ = new Definition($1, $3);      }
			| STRING EQUAL multiset_expr                            { $$ = new Definition($1, $3);      }
			| STRING EQUAL logic_condition                          { $$ = new Definition($1, $3);      }
			| STRING EQUAL STRING 
			{
				if (!strcmp($1,$3)) 
				{
					SetError(DEF_LOOP);
					return  (DEF_LOOP);
				}
				$$ = new Definition($1,new ConstVal($3)); 
			}
/*------------------------------------------------------------------------------------------------------------------*/
definition_def          : DEFINITION DOUBLEPOINT                                { $$ = 0;                           }

/********************************************************************************************************************/
/************************************************ Page  *************************************************************/
/********************************************************************************************************************/
page			: PAGE DOUBLEPOINT STRING			    	
			{ 
				if (MakeActPageName($3))  
				{
					SetError(PAGE_NAME_ERR);
					return  (PAGE_NAME_ERR);	
				}
				$$ = 0;
			}

/********************************************************************************************************************/
/************************************************ Evaluate **********************************************************/
/********************************************************************************************************************/
evaluate		: EVALUATE DOUBLEPOINT real_expr			{ MakeEvalCell($3);	$$ = 0;	    } 
			| EVALUATE DOUBLEPOINT multiset_expr	 	    	{ MakeEvalCell($3);     $$ = 0;     }
			| EVALUATE DOUBLEPOINT logic_condition	 	    	{ MakeEvalCell($3);     $$ = 0;     }

/********************************************************************************************************************/
/************************************************ Logic-Condition ***************************************************/
/********************************************************************************************************************/
logic_condition		: comparison					    	{ $$ = $1; 			    }
			| LOGIC							{ $$ = $1;			    }
			| WAHR						    	{ $$ = new ConstVal(true);  	    }
			| FALSCH					    	{ $$ = new ConstVal(false); 	    }
			| NOT logic_condition				    	{ $$ = new UnaerOp(logic_not,$2);   }
			| logic_condition OR  logic_condition		    	{ $$ = new BinaerOp(logic_or,$1,$3);}
			| logic_condition AND logic_condition		    	{ $$ = new BinaerOp(logic_and,$1,$3);}
/*------------------------------------------------------------------------------------------------------------------*/
comparison		: real_expr     comp_op real_expr	    	    	{ $$ = new BinaerOp($2,$1,$3);	    }
			| multiset_expr comp_op multiset_expr	    	    	{ $$ = new BinaerOp($2,$1,$3);	    }
/*------------------------------------------------------------------------------------------------------------------*/
comp_op			: EQUAL						    	{ $$ = equal; 		            }
			| LESS						    	{ $$ = less;  	 	            }
			| GREATER					    	{ $$ = greater; 	            }
			| NOT_EQUAL					    	{ $$ = not_equal;	            }
			| LESS_OR_EQUAL					    	{ $$ = less_or_equal; 	            }
			| GREATER_OR_EQUAL				    	{ $$ = greater_or_equal;    	    }

/********************************************************************************************************************/
/************************************************ Multisets *********************************************************/
/********************************************************************************************************************/
multiset_expr           : p_mset                                                { $$ = $1;                          }
			| e_mset                                                { $$ = $1;                          } 
			| OPEN_I multiset_expr CLOSE_I                          { $$ = $2;                          }
			| MINUS multiset_expr                                   { $$ = new UnaerOp (minus,   $2);   }
			| multiset_expr PLUS     multiset_expr                  { $$ = new BinaerOp(plus,    $1,$3);}
			| multiset_expr MINUS    multiset_expr                  { $$ = new BinaerOp(minus,   $1,$3);}
			| multiset_expr DIVIDE   real_expr                      { $$ = new BinaerOp(divide,  $1,$3);}
			| real_expr     MULTIPLY multiset_expr                  { $$ = new BinaerOp(multiply,$1,$3);}
/*------------------------------------------------------------------------------------------------------------------*/
p_mset                  : p_mset_const                                          { $$ = $1;                          }
			| P_MSET                                                { $$ = $1;                          }
			| p_mset_1 CLOSE_III                                    { $$ = $1;                          }

e_mset                  : e_mset_const                                          { $$ = $1;                          }
			| E_MSET                                                { $$ = $1;                          }
			| e_mset_1 CLOSE_III                                    { $$ = $1;                          }
/*------------------------------------------------------------------------------------------------------------------*/
p_mset_1                : OPEN_III p_mset_0                                     { $$ = $2;                          }
			| OPEN_III P_MSET                                       { $$ = $2;                          }
			| OPEN_III p_mset_const                                 { $$ = $2;                          }
			| p_mset_1 COMMA p_mset_0                               { $$ = new BinaerOp(plus, $1, $3);  }
			| p_mset_1 COMMA p_mset_const                           { $$ = new BinaerOp(plus, $1, $3);  }

e_mset_1                : OPEN_III e_mset_0                                     { $$ = $2;                          }
			| OPEN_III E_MSET                                       { $$ = $2;                          }
			| OPEN_III e_mset_const                                 { $$ = $2;                          }
			| e_mset_1 COMMA e_mset_0                               { $$ = new BinaerOp(plus, $1, $3);  }
			| e_mset_1 COMMA e_mset_const                           { $$ = new BinaerOp(plus, $1, $3);  }
/*------------------------------------------------------------------------------------------------------------------*/
p_mset_0                : OPEN_I real_expr CLOSE_I MULTIPLY p_mset_const        { $$ = new BinaerOp(multiply,$2,$5);}
e_mset_0                : OPEN_I real_expr CLOSE_I MULTIPLY e_mset_const        { $$ = new BinaerOp(multiply,$2,$5);}
/*------------------------------------------------------------------------------------------------------------------*/
p_mset_const            : unit                                     { $$ = new ConstVal(new TokenMSVal($1, 1));      }
			| INT_CONST TIMES unit                     { $$ = new ConstVal(new TokenMSVal($3,$1));      }

e_mset_const            : E                                        { $$ = new ConstVal(new TokenMSVal(NO_COLOR, 1));}
			| INT_CONST TIMES E                        { $$ = new ConstVal(new TokenMSVal(NO_COLOR,$1));}

/********************************************************************************************************************/
/************************************************ Token *************************************************************/
/********************************************************************************************************************/
unit                    : STRING part_version STRING	    { $$=SaveToken(StripOfPath($1),$2,StripOfPath($3));     }
			| STRING part_version INT_CONST	    { $$=SaveToken(StripOfPath($1),$2,GetNumberString($3)); }
			| STRING QUESTIONMARK		    { $$=SaveToken(StripOfPath($1),WILDCARD, WILDCARD);     }
/*------------------------------------------------------------------------------------------------------------------*/
part_version            : OPEN_II STRING       CLOSE_II				{ $$ = StripOfPath($2);    	    }
			| OPEN_II QUESTIONMARK CLOSE_II				{ $$ = WILDCARD;           	    }
			| OPEN_II INT_CONST    CLOSE_II				{ $$ = GetNumberString($2);	    }
			| POINT							{ $$ = STDVERSION;         	    }

/********************************************************************************************************************/
/************************************************ reals /real_expr **************************************************/
/********************************************************************************************************************/
real_expr               : NUMERIC                                               { $$ = $1;                          }
			| int_const                                             { $$ = $1;                          }
			| real_const                                            { $$ = $1;                          }
			| OPEN_I real_expr CLOSE_I                              { $$ = $2;                          }
			| MINUS real_expr                                       { $$ = new UnaerOp(minus,$2);       }
			| real_expr PLUS     real_expr                          { $$ = new BinaerOp(plus,$1,$3);    }
			| real_expr MINUS    real_expr                          { $$ = new BinaerOp(minus,$1,$3);   }
			| real_expr DIVIDE   real_expr                          { $$ = new BinaerOp(divide,$1,$3);  }
			| real_expr MULTIPLY real_expr                          { $$ = new BinaerOp(multiply,$1,$3);}
/*------------------------------------------------------------------------------------------------------------------*/
real_const              : REAL_CONST                                            { $$ = new ConstVal($1);            }
int_const               : INT_CONST                                             { $$ = new ConstVal($1);            }

/*------------------------------------------------------------------------------------------------------------------*/
%%
#include "evaluate_scanner.cc"

/*------------------------------------------------------------------------------------------------------------------*/
/*														    */
/*		 pfad vom gesuchten String abtrennen und string zurueckgeben					    */
/*														    */
/*------------------------------------------------------------------------------------------------------------------*/
char* StripOfPath(char* s)
{
	int   count = 0;
	int c_count = 0;
	char* h;

	for (h = s; *h != '\0'; h++)
		if ( *h == '\\' ) count++;

	for (h = s; count; h++,c_count++)
		if ( *h == '\\' ) count--;

	char* h1 = new char[strlen(s)-c_count+1];
	char* h2 = h1;

	for (; *h != '\0'; h++, h1++)
		*h1 = *h;
	*h1 = '\0';
	return h2;
}
/*------------------------------------------------------------------------------------------------------------------*/
/*														    */
/*			int in char* umwandeln									    */
/*														    */
/*------------------------------------------------------------------------------------------------------------------*/
char* GetNumberString(int number)
{
	int   number_size = GetNumberSize(number);
	int   rest 	  = 0;
	char* c1   	  = new char[number_size+2](NULL);

	*c1++ = '\0';
	while (number/10 >= 1.0)
	{
		rest   = number % 10;
		number = number / 10 - rest / 10;
		*c1++  = char(rest+48);
	}
	*c1++ = char(number+48);
	*c1   ='\0';

	char* c2 = new char[sizeof(c1)+1](NULL);
	char* c3 = c2;

	for (c1--; *c1 != '\0';) *c2++ = *c1--;
	*c2 = '\0';
	return c3;
}
/*------------------------------------------------------------------------------------------------------------------*/
/*														    */
/*			groesse eines int ermitteln 								    */
/*														    */
/*------------------------------------------------------------------------------------------------------------------*/
int GetNumberSize(int number) 
{
        int number_size=0;
	for (number_size = 0; number/10 >= 1.0; number = number/10 - (number%10)/10, number_size++);
	return number_size;
}
/*------------------------------------------------------------------------------------------------------------------*/
/*														    */
/*			zu evaluierende Celle kreieren								    */
/*														    */
/*------------------------------------------------------------------------------------------------------------------*/
void	MakeEvalCell(Cell* eval_cell)
{
	if (eval_cell)  EvalCell = eval_cell->Copy();
	else		EvalCell = new ConstVal();
}
/*------------------------------------------------------------------------------------------------------------------*/
/*														    */
/*			Seitennamen erstellen	 								    */
/*														    */
/*------------------------------------------------------------------------------------------------------------------*/
int	MakeActPageName(char* str)
{
	if (!str) return (PAGE_NAME_ERR);

	sprintf(actual_page = new char[(strlen(str) + 1)], str); 
	return 0; 
}
/*------------------------------------------------------------------------------------------------------------------*/
/*														    */
/*			Definition in DefinitionRefList speichern 						    */
/*														    */
/*------------------------------------------------------------------------------------------------------------------*/
int	SaveDefinition(Definition* def)
{
        if ( DefinitionRefList.UniqueNameAppend(*def) < 0 )
		return (DEF_EXISTS_ERR);
	return 0;
}
/*------------------------------------------------------------------------------------------------------------------*/
/*														    */
/*			- Token kreieren									    */
/*			- Token in TokenRefList speichern							    */
/*			- Referenz zureuckgeben									    */
/*														    */
/*------------------------------------------------------------------------------------------------------------------*/
int	SaveToken(char* name, char* version, char* step) 
{ 
	Token    tok(name, version, step);
	int tr = TokenRefList.UniqueAppend(tok);
	if (tr < 0) tr = -tr;
	if (tr > MAX_TREF) yyerror("too much tokens defined (max = 65536)");
	return TokenRef(tr);
}
/*------------------------------------------------------------------------------------------------------------------*/
/*														    */
/*			fehlerausgabe										    */
/*														    */
/*------------------------------------------------------------------------------------------------------------------*/
void	SetError(int err_no)
{
	if (err_set) return;
	ErrNo   = err_no;
	err_set = true;
}
/*------------------------------------------------------------------------------------------------------------------*/
void 	yyerror(char*)
{
	SetError(SYNTAX_ERR);
	return;
}
/*------------------------------------------------------------------------------------------------------------------*/
