%{

#ifdef YY_INPUT
#undef YY_INPUT
#endif
#define YY_INPUT(buf,result,max_size) \
  if ( yy_current_buffer->yy_is_interactive ) \
  { \
    for (i=0; i<max_size; i++) { \
      buf[i] = *evaluation_string; \
      if (!buf[i]) break; \
      evaluation_string++; \
    } \
    result = i; \
  }

/*------------------------------------------------------*/
/*		path-get-and-add			*/
/*------------------------------------------------------*/
char*	GetPath			(char*);
char*	AddRoot			(char*);
char*	AddActDefPath		(char*);
char*	AddActPagePath		(char*);
/*------------------------------------------------------*/
/*		return-functions			*/
/*------------------------------------------------------*/
int	ReturnString		(char*);
int	ReturnDefString		(char*);
int     ReturnCellContents	(Cell*);
/*------------------------------------------------------*/
/*		definition-tests			*/
/*------------------------------------------------------*/
int	DefinitionOrString	(char*);
Cell*   TestIfDefinition	(char*, TextualList*);
bool    DefinitionLoop		(char*, Textual);
/*------------------------------------------------------*/
/*		copy-functions				*/
/*------------------------------------------------------*/
void    CopyCellPtr		(Cell*);
void    CopyString		(char*);

%}
D	[0-9]
E	[Ee][+-]?[0-9]+
%%
[ \t]+ 			;	
"\n"			; 
{D}+			{ 	
			/*-------------------------------------------------------------------------------*/
			/*                              INT_CONST                                        */
			/*-------------------------------------------------------------------------------*/
				yylval.i_val = strtol(yytext, NULL, 10); 
				if (errno == ERANGE)
				{
					SetError(STR_TO_L_ERR);
					return (ERROR);
				}
				return (INT_CONST);
			}
{D}+"."{D}*({E})?	|
{D}+{E}			{ 
			/*-------------------------------------------------------------------------------*/
			/*                              REAL_CONST                                       */
			/*-------------------------------------------------------------------------------*/
				if ( ((yylval.d_val=strtod(yytext, NULL)) == 0) || errno == ERANGE) 
				{
					SetError(STR_TO_D_ERR);
					return (ERROR);
				}
				return (REAL_CONST);
			}
"e"			return (E);
"TRUE"			return (WAHR);
"FALSE"			return (FALSCH);

"Page"			{ section = page_sec; return (PAGE);		} 
"Evaluate"		{ section = eval_sec; return (EVALUATE);	}
"Definition"		{ section = def_sec;  return (DEFINITION);	}
"__ROOT__"              { SetError(ROOT_ERR); return (ERROR);		}

"\\"                    { 
                        /*-------------------------------------------------------------------------------*/
			/*                              STRING (root-page)                               */
			/*-------------------------------------------------------------------------------*/
				if (section = page_sec) 
				{
					strcpy(yylval.s_val = new char[10],"\\__ROOT__");
					return(STRING);
				}
				else    
				{
					SetError(ROOT_SECTION_ERR);
					return (ERROR);
				}
			}

"\\"[a-zA-Z_][a-zA-Z_0-9]*['\\'a-zA-Z_0-9]* {
			/*-------------------------------------------------------------------------------*/
			/*                       STRING (absolute "Pfadangabe")				 */
			/*-------------------------------------------------------------------------------*/
				switch (section)
				{
					case page_sec	: return (ReturnString      (AddRoot(yytext))); 
					case def_sec	: return (ReturnDefString   (AddRoot(yytext)));
					case eval_sec	: return (DefinitionOrString(AddRoot(yytext)));
					default		: SetError(SECTION_ERR); return (ERROR);
				}
			}

[a-zA-Z_][a-zA-Z0-9_]*  {
                        /*-------------------------------------------------------------------------------*/
			/*                      STRING (keine "Pfadangabe")                              */
			/*-------------------------------------------------------------------------------*/
				switch (section)
				{
					case def_sec	: return (ReturnString      (AddActDefPath (yytext)));
					case eval_sec	: return (DefinitionOrString(AddActPagePath(yytext))); 
					case page_sec	: SetError(NO_PAGE_PATH_ERR); return (ERROR);
					default		: SetError(SECTION_ERR);      return (ERROR);
				}
			}

[a-zA-Z_][a-zA-Z_0-9]*['\\'a-zA-Z_0-9]* {
			/*-------------------------------------------------------------------------------*/
			/*                      STRING (relative "Pfadangabe")                		 */
			/*-------------------------------------------------------------------------------*/
				switch (section)
				{
					case def_sec	: return (ReturnString      (AddActDefPath (yytext)));
					case eval_sec	: return (DefinitionOrString(AddActPagePath(yytext))); 
					case page_sec	: SetError(REL_PAGE_PATH_ERR); return (ERROR);
					default		: SetError(SECTION_ERR);       return (ERROR);
				}
			}
"'"			return (TIMES); 
"+"			return (PLUS); 
"-"			return (MINUS); 
"/"			return (DIVIDE); 
"*"			return (MULTIPLY); 

"!"			return (NOT);
"&"			return (AND);
"|"			{
				if ((section == def_sec) && actual_def_path)
				{
					delete[] actual_def_path; 
					actual_def_path = NULL; 
				}
				return (OR);
			}

"#"			;
"@"			;

"="			return (EQUAL);
"<"			return (LESS);
">"			return (GREATER);
"!="			return (NOT_EQUAL);
"<="			return (LESS_OR_EQUAL);
">="			return (GREATER_OR_EQUAL);

","			return (COMMA);
"."			return (POINT);
":"			return (DOUBLEPOINT);
"?"			return (QUESTIONMARK);
")"			return (CLOSE_I);
"("			return (OPEN_I); 
"]"			return (CLOSE_II); 
"["			return (OPEN_II); 
"}"			return (CLOSE_III); 
"{"			return (OPEN_III); 
%%
//################################################################################################################
//
//	fuegt "str" "\__ROOT__" hinzu	
//
//################################################################################################################
char*	AddRoot(char* str)
{
	char*   ret_str = NULL;
	sprintf(ret_str = new char[(strlen(str)+11)], "\\__ROOT__%s", str);
	return (ret_str);
}
//################################################################################################################
//
//	fuegt "str" "actual_page" hinzu	
//
//################################################################################################################
char*	AddActPagePath(char* str)
{
	char*   ret_str = NULL;
	sprintf(ret_str = new char[(strlen(actual_page)+strlen(str)+3)], "%s\\%s", actual_page, str);
	return (ret_str);
}
//################################################################################################################
//
//	fuegt "str" "actual_def_path" hinzu	
//
//################################################################################################################
char*	AddActDefPath(char* str)
{
	char*   ret_str = NULL;
	sprintf(ret_str = new char[(strlen(actual_def_path)+strlen(str)+3)], "%s\\%s", actual_def_path, str);
	return (ret_str);
}
//################################################################################################################
//
//	kopiert "str" in die Parservariable "yylval.s_val" 
//
//################################################################################################################
int	ReturnString(char* str)
{
	sprintf(yylval.s_val  = new char[(strlen(str) + 1)], "%s", str);
        return (STRING);
}
//################################################################################################################
//
//	- sofern "actual_def_path" nicht gesetzt ist wird sie durch den Pfad von "str" gesetzt
//	- kopiert "str" in die Parservariable "yylval.s_val" 
//
//################################################################################################################
int	ReturnDefString(char* str)
{
	if (!actual_def_path)	
	{
		char* path = GetPath(str);
		if (!path) return (ERROR);
		sprintf(actual_def_path = new char[(strlen(path) + 1)], "%s", path);
	}
	return (ReturnString(str));
}
//################################################################################################################
//
//	ueberprueft ob der gefundene String einer Variablen der Definitionen entspricht
//
//################################################################################################################
int	DefinitionOrString(char* str)
{
	Cell* val = NULL;
	if ((val=TestIfDefinition(str, new TextualList())))
		return (ReturnCellContents(val));
	return (ReturnString(str));
}
//################################################################################################################
//
//	gibt den pfad eines strings zurueck 
//
//################################################################################################################
char* GetPath(char* s)
{
        if (*s != '\\') 
        { 
        	SetError(GET_PATH_ERR); 
        	return NULL; 
       	}

        int     slash_count = 0;
	int     cp_c_count  = 0;
	int     c_count     = 0;
	char*   h;

	for (h = s; *h != '\0'; h++,c_count++)
	{
		if ( *h == '\\' ) 
		{
			slash_count++;
			cp_c_count = c_count;
		}
	}

	char* path   = new char[cp_c_count+1];
	char* h_path = path;

	for (h = s; slash_count; h++,h_path++)
	{
		*h_path = *h;
		if ( *h == '\\' ) slash_count--;
	}
	*h_path = '\0';
	return path;
}

//################################################################################################################
//
//      Ueberpruefung, ob es Definitions-Schleifen gibt 
//
//################################################################################################################
bool    DefinitionLoop(char* old_val, Textual old_var)
{
	for (DefinitionListElement* h=DefinitionRefList.first; h; h=h->next)
	{
		if (h->lelement.GetVar() == old_val)
		{
			if (h->lelement.GetVal()->GetResultType() != textual) return false;

			if (old_var == h->lelement.GetVal()->Calculate()->GetText()) 
				return true;
			return (DefinitionLoop(h->lelement.GetVal()->Calculate()->GetText(),old_var));
		}
	}
	return false;
}
//################################################################################################################
//
// 	Testet, ob es zum gefundenen String einen passenden Eintrag in der Referenzliste "ExprRefList"
// 	gibt. 
// 		Ja   => Returnwert = gefundenen Eintrag
// 		Nein => Returnwert = NULL
//
//################################################################################################################
Cell*   TestIfDefinition(char* test_str, TextualList* tl)
{
	if (!DefinitionRefList.size) return NULL;

	Textual txt(test_str);
	for (DefinitionListElement* h=DefinitionRefList.first; h; h=h->next)
	{
		if (h->lelement.GetVar() == txt)
		{
			if ((tl->UniqueAppend(txt)) < 0 ) { SetError(DEF_LOOP); return NULL; }

			if (h->lelement.GetVal()->GetResultType() != textual) return (h->lelement.GetVal());

			if (DefinitionLoop(h->lelement.GetVal()->Calculate()->GetText(),txt)) 
			{
				SetError(DEF_LOOP);
				return  NULL;
			}

			Cell* r_val = NULL;
			if ((r_val=TestIfDefinition(h->lelement.GetVal()->Calculate()->GetText(),tl)))
				return (r_val);
			return (h->lelement.GetVal());
		}
	}
	return NULL;
}
//################################################################################################################
//
// 	Copier-Funktionen zur Uebergabe spezifischer Definitionswerte an den Parser
//
//################################################################################################################
void    CopyCellPtr(Cell* def_val)      { yylval._CellPtr     = def_val->Copy(); }
void    CopyString(char*  def_str)   	{ strcpy(yylval.s_val = new char[strlen(def_str)+1], def_str); }

//################################################################################################################
//
// 	Untersucht welche Art von Definition vorliegt.
//	Es wird die "rechte Seite" der Definition zurueckgegeben.
// 	Im Fall von numeric, p_multiset, e_multiset wird ein CellPtr zurueckgegeben
// 	Im Fall von textual wird ein char* zurueckgegeben.
//
//################################################################################################################
int     ReturnCellContents(Cell* def_val)
{
        switch (def_val->GetResultType())
	{
		case logic_expr		: CopyCellPtr(def_val);				return (LOGIC);
		case p_mset_expr	: CopyCellPtr(def_val);				return (P_MSET); 
		case e_mset_expr	: CopyCellPtr(def_val);				return (E_MSET);
		case numeric   		: CopyCellPtr(def_val);				return (NUMERIC); 
		case textual  		: CopyString(def_val->Calculate()->GetText());	return (STRING); 
		default      		: SetError(RESULT_TYPE_ERR);			return (ERROR);
	}
	return 0;
}
