#ifndef __AgnesToTimeNet_cc__
#define __AgnesToTimeNet_cc__

extern "C" {
#include <math.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdio.h>
#include <signal.h>
#include <errno.h>
#include <netdb.h>
#include <malloc.h>
#include <fcntl.h>
#include <setjmp.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include "portab.h"
#include "c_types.h"
}

#ifdef SOL2
extern "C" {
#include <sys/filio.h>
}
#endif

extern "C" {
#include <rpc/rpc.h>
#include "rusers.h"             /* <rpcsvc/rusers.h> not identical on all systems */
#include "rstat.h"              /* <rpcsvc/rstat.h> as well */
}


#include "inc_classes.h"
#include "inc_info.h"
#include "inc_agnes_to_timenet.h"

//###########################################################################################################
//
//      defines
//
//###########################################################################################################
#define MAX_KB_PER_SEC_SUM      150

//---------------------------------------------------------------------------------------------------------------
//
//      extern var's
//
//---------------------------------------------------------------------------------------------------------------
extern  RRGError*               RRGErr;
extern  char*                   TmpDir;
extern  bool                    DEBUG;
extern  bool                    SHOW_INFOS;
extern  unsigned char           bits[];


//---------------------------------------------------------------------------------------------------------------
//
//      extern functions 
//
//---------------------------------------------------------------------------------------------------------------
extern          void    PrepareEnd();
extern 		void 	ShowRandInfo(randinfo*&, int);
extern		void	ExecuteRandProcesses (char*&, ExecMode, double&, randinfo*&, int&, double&);
extern		void	ExecuteErandProcesses(char*&, double&, int&, int&, ListOfSMCLists&);

extern  "C" {   void    SolveLGS	(char*, char*, int, double); 		}
extern  "C" {   char*	strdup		(const char*); 			   	}
extern  "C" {   int	strncasecmp	(const char*, const char*, size_t); 	}
extern  "C" {  	pid_t 	fork		(); 		    	    	    	}
extern  "C" {  	pid_t 	vfork		(); 		    	    	    	}

#if defined(__linux__) || defined(LINUX) || defined (Linux)
extern  "C" {   int     fileno          (FILE*);                                }
extern  "C" {   FILE*   fdopen          (int, const char*);                     }
#endif


//###############################################################################################################
//
//	Fehlerbehandlung
//
//###############################################################################################################
void	AgnesToTimeNetError(char* s1, char* s2, int line)
{
	if (s2) RRGErr = new RRGError(__FILE__, s1, line, NULL, s2);
	else	RRGErr = new RRGError(__FILE__, s1, line);
	PrepareEnd();
}
/*
//###############################################################################################################
//
//	Vergleichsfunktion zum Sortieren der randinfo-Strukturen 
//	nach dem geschaetzen Aufwand zur Loesung der Markoff-Ketten
//
//###############################################################################################################
static int CompareRandInfos(randinfo* item1, randinfo* item2)
{
	return (int)(item2->effort - item1->effort);
}
*/
//###############################################################################################################
//
//      Erloesen von Zombi-Prozessen
//
//###############################################################################################################
static void SigChldHandler(int)
{
	int status;
	waitpid (-1, &status, WNOHANG);
}


//###############################################################################################################
//
//	Kreiert die Files <model_name>.CONV00 und <model_name>.ENV00, die die exponentiellen
//	Markierungen und ihre Uebergaenge beschreiben (P- und C-Matrix)
//
//###############################################################################################################
void	WriteFilesForMExp(char*	model_name, MarkingPtrList& exp_marking_list)
{
	FILE* emc_fp;
	FILE* conv_fp;
	char* conv_filename = new char[(strlen(TmpDir) + strlen(model_name) + 12)];
	char* emc_filename  = new char[(strlen(TmpDir) + strlen(model_name) + 12)];

	sprintf(conv_filename, "%s/%s.CONV00", TmpDir, model_name);
	sprintf(emc_filename,  "%s/%s.EMC00" , TmpDir, model_name);

	if ( !(emc_fp  = fopen(emc_filename, "w")) )	
		AgnesToTimeNetError("can't open file for writing", emc_filename, __LINE__ );

	if ( !(conv_fp = fopen(conv_filename,"w")) )	
		AgnesToTimeNetError("can't open file for writing", conv_filename, __LINE__);

	if (exp_marking_list.size)
	{
		for (MarkingPtrListElement* em=exp_marking_list.first; em; em=em->next)
		{
			double	 sum_of_rates = 0.0;
			if (em->plelement) sum_of_rates = em->plelement->GetSumOfRates();

			if (!sum_of_rates) 
				AgnesToTimeNetError("Division by zero", "Sum of rates is zero", __LINE__);

			double   div_rate_sum = 1.0 / sum_of_rates;

			P_row_header.row_index      = unsigned(em->plelement->GetNumber());
			P_row_header.entries_in_row = em->plelement->GetNoOfStateTrans();

			C0_entry.sojourn_time       = div_rate_sum;
			C0_entry.state_number       = unsigned(em->plelement->GetNumber());

			if (fwrite(&P_row_header, SIZEOF_P_row_header, 1, emc_fp)  != 1) 
				AgnesToTimeNetError("Can't write to file", emc_filename, __LINE__);

			if (fwrite(&C0_entry,     SIZEOF_C0_entry,     1, conv_fp) != 1) 
				AgnesToTimeNetError("Can't write to file", conv_filename, __LINE__);


			if (em->plelement->first)
			{
				for (state_transition* st=em->plelement->first; st; st=st->nextS)
				{
					if (st->nextM)
					{
						P_row_entry.column_index = unsigned(st->nextM->GetNumber());
						P_row_entry.probability  = st->delay.GetDelayVal() * div_rate_sum;

						if (fwrite(&P_row_entry, SIZEOF_P_row_entry, 1, emc_fp)  != 1) 
							AgnesToTimeNetError("can't write to file", emc_filename, __LINE__);
					}
				}
			}
		}
	}
	fclose(emc_fp);  delete[] emc_filename;
	fclose(conv_fp); delete[] conv_filename;
}

//###############################################################################################################
//
//	Kreiert das File <model_name>.AUX, das die Anzahl der tangiblen Markierungen enthaelt
//	und das von "SolveLGS" benoetigt wird.
//
//###############################################################################################################
void	WriteAux(char* model_name, int& no_tang_markings)
{
	register FILE*	aux_file;
	char*		filename = new char[(strlen(TmpDir) + strlen(model_name) + 12)];

	sprintf(filename, "%s/%s.AUX", TmpDir, model_name);

	if (!(aux_file = fopen(filename, "w")))	AgnesToTimeNetError("can't open file", filename, __LINE__);

	fprintf(aux_file, "TANGIBLE MARKINGS %u\n", unsigned(no_tang_markings));
	fclose(aux_file);
}

//###############################################################################################################
//
//	Informationen ueber eine einzelne SMC sammeln
//
//###############################################################################################################
void	GetInfosAboutSMC(Marking*& member_of_smc, int& no_exp_path, double& sum_of_rates) 
{
	double tmp_sum_of_rates = 0.0;

	if (member_of_smc->first)
	{
		for (state_transition* st=member_of_smc->first; st; st=st->nextS)
		{
			if ((int(st->delay.delay_typ) == _EXPONENTIAL) || (int(st->delay.delay_typ) == _PREEMP_EXP))
			{
				tmp_sum_of_rates += st->delay.GetDelayVal();
				no_exp_path++;
			}
		}

		if (tmp_sum_of_rates > sum_of_rates) sum_of_rates = tmp_sum_of_rates;

		for (state_transition* st1=member_of_smc->first; st1; st1=st1->nextS)
		{
			if ((int(st1->delay.delay_typ) == _EXPONENTIAL))
			{
				if ( (st1->nextM) 				  && 
				     (int(st1->nextM->status)    == _VISITED) 	  &&
				     (int(member_of_smc->status) != _LAST_OF_SCC) )
						GetInfosAboutSMC(st1->nextM, no_exp_path, sum_of_rates);
			}
		}
	}
}
//###############################################################################################################
//
//	Informationen fuer ExecuteRandProcess sammeln
//
//###############################################################################################################
void	ObtainRandInfos(ListOfSMCLists& list_of_det_marking_lists, 
			randinfo*& 	rand_info_array, 
			double& 	sum_of_efforts)
{
	randinfo* tmp_rand_info_array = rand_info_array;

	for (ListElementOfSMCLists* smcs=list_of_det_marking_lists.first; smcs; smcs=smcs->next)
	{
		tmp_rand_info_array->tau	   = 1.0;
		tmp_rand_info_array->effort	   = 0.0;
		tmp_rand_info_array->kb_per_sec    = 0;
		tmp_rand_info_array->connection_fd = 0;
		tmp_rand_info_array->connection_fp = NULL;
		tmp_rand_info_array->host          = NULL;
		tmp_rand_info_array->status        = no_process;
		tmp_rand_info_array->smcs          = &(smcs->lelement);      

		if (smcs->lelement.det_or_gen_markings.size)
		{
	 		double max_qii	    = 0.0;
	 		int    no_exp_path  = 0;

			for (MarkingPtrListElement* smc=smcs->lelement.det_or_gen_markings.first; smc; smc=smc->next)
			{
 				double sum_of_rates = 0.0;

			 	if (smc->plelement && (int(smc->plelement->status) == _INITIAL))
			 		GetInfosAboutSMC(smc->plelement, no_exp_path, sum_of_rates); 

				if (max_qii < sum_of_rates) max_qii = sum_of_rates;
			}
			tmp_rand_info_array->max_qii = max_qii;
			tmp_rand_info_array->eta     = no_exp_path;

		}
		else 	
		{
			tmp_rand_info_array->max_qii = 0.0;
			tmp_rand_info_array->eta     = 0;
		}
		if (smcs->next) tmp_rand_info_array++;
	}

	randinfo* info=rand_info_array;
	int	  n   = 1;
	for (; n < list_of_det_marking_lists.size+1; n++, info++)
	{
		if (!(info->max_qii))
		{
			info->effort = 0.0;
			info->kb_per_sec = 0;
		}
		else
		{
			info->effort     = info->max_qii * double(info->eta) * 0.00261;
			info->kb_per_sec = unsigned(sqrt (double(info->eta) / info->max_qii));
	
			if (info->effort < 1.0)	info->kb_per_sec = unsigned(sqrt (double(info->kb_per_sec) * info->effort));
	
			if (info->kb_per_sec > MAX_KB_PER_SEC_SUM) info->kb_per_sec = MAX_KB_PER_SEC_SUM; 
		}
		sum_of_efforts += info->effort;
	}
//	qsort(&rand_info_array[0], list_of_det_marking_lists.size+1, sizeof(randinfo), CompareRandInfos);
}

//###############################################################################################################
//
//	Schnittstellenfunktion zwischen neuer Agnes-RRG-Erstellung und alter TimeNet-LGS-Loesung
//
//###############################################################################################################
void	AgnesToTimeNet(char* 		model_name, 
		       char*		auto_iter,
		       int& 		no_tang_markings,
		       int&		maxiter,
		       double&		epsilon,
		       ExecMode		exec_mode,
		       MarkingPtrList& 	exp_marking_list, 
		       ListOfSMCLists&	list_of_det_marking_lists,
		       ListOfSMCLists&	list_of_gen_marking_lists)
{
	int 	  no_det_trans 	  = list_of_det_marking_lists.size;
	int 	  no_gen_trans 	  = list_of_gen_marking_lists.size;

	double    sum_of_efforts  = 0.0;
	randinfo* rand_info_array = new randinfo[(no_det_trans + 1)]; 

	setvbuf(stdout, NULL, _IOLBF,BUFSIZ);
	setvbuf(stderr, NULL, _IOLBF,BUFSIZ);

	if ((void *)signal (SIGCHLD, SigChldHandler) == (void *)-1L) 
		AgnesToTimeNetError("signal", NULL, __LINE__);


	//-----------------------------------------------------------------------------------------------------
	//	- Fuer exponentialle Markierungen koenne wir die Q- und P-Matrix direkt schreiben	
	//	- Die Angabe der tangiblen Markierungen im .AUX - File braucht SolveLGS
	//-----------------------------------------------------------------------------------------------------
	if (DEBUG) cout <<"\t writing CONV00-, EMC00- and AUX-files ..."<<endl;

	WriteFilesForMExp(model_name, exp_marking_list);
	WriteAux(model_name, no_tang_markings);

	if (DEBUG) cout <<"\t writing CONV00-, EMC00- and AUX-files finished !"<<endl;

	if (DEBUG)
	{
		     if (no_gen_trans) 	cout <<endl<<"\t EDSPN TO EVALUATE"<<endl<<endl;
		else if (no_det_trans) 	cout <<endl<<"\t DSPN TO EVALUATE"<<endl<<endl;
		else			cout <<endl<<"\t GSPN TO EVALUATE"<<endl<<endl;
	}
	//-----------------------------------------------------------------------------------------------------
	//	Mist, doch ein DSPN oder EDSPN !!!!
	//	Na gut, Infos fuer die (e)rand-Prozesse holen und diese verteilt oder lokal starten
	//-----------------------------------------------------------------------------------------------------
	if (no_det_trans || no_gen_trans)
	{
		int i = 0;
		for (randinfo* tmp=rand_info_array; i < no_det_trans; i++, tmp++) tmp = NULL;

		ObtainRandInfos(list_of_det_marking_lists, rand_info_array, sum_of_efforts);

		if (SHOW_INFOS)
		{
			ShowRandInfo(rand_info_array, (no_det_trans + 1));
			cout <<"sum_of_efforts : "<<sum_of_efforts<<"\n";
		}
	}

	if (no_det_trans)
	{
		if (DEBUG) cout <<"\t starting rand process execution ..."<<endl;

		ExecuteRandProcesses (model_name, exec_mode, epsilon, rand_info_array, no_det_trans, sum_of_efforts);

		if (DEBUG) cout <<"\t starting rand process execution finished !"<<endl;
	}
	if (no_gen_trans)
	{
		if (DEBUG) cout <<"\t starting erand process execution ..."<<endl;

		ExecuteErandProcesses(model_name, epsilon, no_gen_trans, no_det_trans, list_of_gen_marking_lists);

		if (DEBUG) cout <<"\t starting erand process execution finished !"<<endl;
	}

	//-------------------------------------------------------------------------------------------------------
	//		los geht die Rechnerei
	//-------------------------------------------------------------------------------------------------------
	cout <<"\t solving LGS ..."<<endl;

	SolveLGS(auto_iter, model_name, maxiter, epsilon);

	cout <<"\t solving LGS finished !"<<endl;
}
//###############################################################################################################
#endif
