#ifndef __BindingElement_cc__
#define __BindingElement_cc__

#include <iostream.h>

#include "Identifier.h"
#include "Textual.h"
#include "TextualList.h"

#include "token_multiset.h"
#include "PlaceHeader.h"

#include "Token.h"

#include "CellVal.h"
#include "Cell.h"

#include "Place.h"

#include "Delay.h"

#include "ArcBindElement.h"
#include "ArcBindList.h"

#include "BindingElement.h"

//#######################################################################################################################
BindingElement::BindingElement()    			 
	      : weight		(DEF_ES_WEIGHT), 
	        lc_result	(DEF_COND_RES),
	        del_result	(DEF_DEL_RES),
	      	inarcbind	(), 
	      	outarcbind	()
	      { 
		delay     = new Immediate();
		localcond = new ConstVal();
	      }
BindingElement::BindingElement(int) 			 
	      : weight		(DEF_ES_WEIGHT), 
	        lc_result	(DEF_COND_RES),
	        del_result	(DEF_DEL_RES),
	      	inarcbind	(), 
	      	outarcbind	()
	      { 
		delay     = new Immediate();
		localcond = new ConstVal();
	      }
BindingElement::BindingElement(const BindingElement& be) 
	      : weight		(be.weight), 
	        lc_result	(be.lc_result),
	        del_result	(be.del_result),
	      	inarcbind	(be.inarcbind), 
	      	outarcbind	(be.outarcbind)
	      { 
		if (!be.delay)     delay     = new Immediate(); else delay     = be.delay->Copy();
		if (!be.localcond) localcond = new ConstVal();  else localcond = be.localcond->Copy();
	      }

BindingElement::BindingElement(Weight w, const Cell* lc, const Delay* d, const ArcBindList& i, const ArcBindList& o) 
	      : weight		(w) , 
	        lc_result	(DEF_COND_RES),
	        del_result	(DEF_DEL_RES),
	      	inarcbind	(i), 
	      	outarcbind	(o)
	      { 
		if (!d)  delay     = new Immediate(); else delay     = d->Copy();
		if (!lc) localcond = new ConstVal();  else localcond = lc->Copy();
	      }
BindingElement::BindingElement(Weight w, const Cell* lc, const Delay* d, const ArcBindList* i, const ArcBindList* o) 
	      : weight		(w),
	        lc_result	(DEF_COND_RES),
	        del_result	(DEF_DEL_RES)
	      { 
		if (!d)  delay     = new Immediate(); 	     else  delay      = d->Copy();
		if (!lc) localcond = new ConstVal();  	     else  localcond  = lc->Copy();
		if (i) inarcbind   = *(new ArcBindList(*i)); else  inarcbind  = *(new ArcBindList()); 
		if (o) outarcbind  = *(new ArcBindList(*o)); else  outarcbind = *(new ArcBindList()); 
	      }

//#######################################################################################################################
char*			BindingElement::GetName()		  		{ return DEF_STRING; 			}
Delay*   		BindingElement::GetDelay()	          		{ return delay; 			}
Weight   		BindingElement::GetWeight()	          		{ return weight; 			}
ArcBindList&		BindingElement::GetInArcBind()	          		{ return inarcbind; 			}
ArcBindList&		BindingElement::GetOutArcBind()		  		{ return outarcbind; 			}
void	 		BindingElement::ChangeInArcBind(const ArcBindList& i)   { inarcbind.Clear();  inarcbind  = i; 	}
void	 		BindingElement::ChangeOutArcBind(const ArcBindList& o)  { outarcbind.Clear(); outarcbind = o; 	}
void	 		BindingElement::ChangeWeight(Weight w)			{ weight = w; 				}
bool 	 		BindingElement::LessThan(BindingElement& be)		{ return ( weight < be.weight ); 	}
bool	 		BindingElement::operator!=(BindingElement& be)		{ return !(*this==be); 			} 
//----------------------------------------------------------------------------------------------------------------------
Cell*	 		BindingElement::GetLocalCondition()	  		
{ 
	if (localcond) return localcond; 
	return new ConstVal(lc_result);
}
//----------------------------------------------------------------------------------------------------------------------
double            	BindingElement::GetDelayVal()         
{ 
	if (del_result != DEF_DEL_RES) 
		return del_result;

	double ret = delay->CalculateFloat();

	if (delay->IsPlaceDependent()) 
		return ret;

	del_result = ret;
	return del_result;
}
//----------------------------------------------------------------------------------------------------------------------
bool            	BindingElement::LocalCondTrue()         
{ 
	if (lc_result != DEF_COND_RES) 
		return (lc_result == 0?false:true);

	CellVal* cv  = localcond->Calculate();
	double   val = cv->GetFloat();

	cv->Clear();
	delete cv;

	if (localcond->IsPlaceDependent())
	{
		if (!val) return false;
		return true;
	}

	if (!val) lc_result = 0;
	else      lc_result = 1;

	if (localcond)
	{
		localcond->Clear();
		delete localcond;
		localcond = NULL;
	}
	return (lc_result == 0?false:true);
}
//----------------------------------------------------------------------------------------------------------------------
void	BindingElement::Clear()
{
	if (localcond)  { localcond->Clear(); delete localcond;  localcond  = NULL; }
	if (delay)      { delay->Clear();     delete delay;      delay      = NULL; }
	lc_result  = DEF_COND_RES;
	del_result = DEF_DEL_RES;
	inarcbind.Clear(); 
	outarcbind.Clear();
}
//----------------------------------------------------------------------------------------------------------------------
void	 BindingElement::ChangeLocalCondition(const Cell* c)      
{ 
	if (c) 
	{ 
		if (localcond)
		{
			localcond->Clear();
			delete localcond; 
			localcond = NULL;
		}
		localcond = c->Copy(); 
		lc_result = DEF_COND_RES;
	}
}
//----------------------------------------------------------------------------------------------------------------------
void	 BindingElement::ChangeDelay(const Delay* d)              
{ 
	if (d) 
	{ 
		if (delay)
		{
			delay->Clear();
			delete delay; 
			delay = NULL; 
		}
		delay      = d->Copy(); 
		del_result = DEF_DEL_RES;
	} 
}
//----------------------------------------------------------------------------------------------------------------------
bool	 BindingElement::operator==(BindingElement& be) 
{ 
	return (weight    == be.weight    && *delay     == *(be.delay) && *localcond==*(be.localcond) && 
		inarcbind == be.inarcbind && outarcbind == be.outarcbind);
}
//----------------------------------------------------------------------------------------------------------------------
BindingElement& BindingElement::operator =(const BindingElement& be)
{
	Clear();
	weight     = be.weight;
	inarcbind  = be.inarcbind;
	outarcbind = be.outarcbind;
	lc_result  = be.lc_result;
	del_result = be.del_result;
	if (!be.delay)     delay     = new Immediate(); else  delay      = be.delay->Copy();
	if (!be.localcond) localcond = new ConstVal();  else  localcond  = be.localcond->Copy();
	return *this;
}
//#######################################################################################################################
#endif
