#ifndef __Cell_cc__
#define __Cell_cc__

extern "C" {
#include "string.h"
}
#include <iostream.h>

#include "Identifier.h"
#include "Textual.h"
#include "TextualList.h"

#include "token_multiset.h"
#include "PlaceHeader.h"

#include "Token.h"
#include "TokenList.h"

#include "CellVal.h"
#include "Cell.h"

//-------------------------------------------------------------------------------------------------------------

extern TokenList     	TokenRefList;	// fuer die Klasse Funktion
extern "C" { char*      rindex(const char*, int); }

//###############################################################################################################
//				CONSTVAL
//###############################################################################################################
ConstVal::ConstVal()                  						{ constval = new FloatVal();  	} 
ConstVal::ConstVal(int i)							{ constval = new FloatVal(i); 	} 
ConstVal::ConstVal(bool b)             						{ constval = new FloatVal(b); 	} 
ConstVal::ConstVal(char* c)							{ constval = new TextVal(c);  	} 
ConstVal::ConstVal(double d)           						{ constval = new FloatVal(d); 	} 
ConstVal::ConstVal(token_multiset* t)         					{ constval = new TokenMSVal(t); } 

ConstVal::ConstVal(const Cell& c)     	
{ 
	if (c.GetCellVal()) *this = *(c.Copy());
	else		    constval = new FloatVal();
}
ConstVal::ConstVal(const CellVal* cv) 	
{ 
	if (cv) constval = cv->Copy(); 
	else    constval = new FloatVal(); 
} 
//###############################################################################################################
void		ConstVal::Clear() 	        	
{ 
	if (constval) 
	{ 
		constval->Clear();
		delete constval; 
		constval = NULL; 
	} 
}
//-------------------------------------------------------------------------------------------------------------
Cell*   	ConstVal::Copy() 		const 			{ return new ConstVal(constval);      }
CellVal* 	ConstVal::GetCellVal()		const 		   	{ return constval;    		      }
CellType   	ConstVal::GetConstCellType()	const 		  	{ return const_val;   		      }
CellType   	ConstVal::GetCellType()					{ return const_val;   		      }
ResultType 	ConstVal::GetResultType()	 			{ return (constval->GetResultType()); }
double		ConstVal::CalculateFloat()				{ return (constval->GetFloat());      }
CellVal* 	ConstVal::Calculate()		 			{ return (constval->Copy());          } // changed
bool	 	ConstVal::IsPlaceDependent() 				{ return false;       		      }
bool	 	ConstVal::IsOnTheSamePage(char*) 			{ return true;       		      }
bool	 	ConstVal::operator!=(const Cell& c) 			{ return !(*this==c); 		      }
//-------------------------------------------------------------------------------------------------------------
Cell&    	ConstVal::operator =(const Cell& c)
{
	if (c.GetConstCellType() == const_val) 
	{ 
	 	Clear(); 
	 	*this = *(c.Copy());
	}
	return *this;
}
//-------------------------------------------------------------------------------------------------------------
bool	 	ConstVal::operator==(const Cell& c) 
{ 
	if (c.GetConstCellType() != const_val) return false;

	CellVal* cv1 = (*constval == *(c.GetCellVal()));	// changed
	bool     ret = bool(cv1->GetFloat());

	cv1->Clear(); 
	delete cv1;						// changed
	return ret;
}

//###############################################################################################################
//				UNAEROP
//###############################################################################################################
UnaerOp::UnaerOp()				: op(empty) 	    			{ opd = new ConstVal(); }
UnaerOp::UnaerOp(Operator o, int i)		: op(o) 	     			{ opd = new ConstVal(i);}
UnaerOp::UnaerOp(Operator o, bool b)		: op(o) 	     			{ opd = new ConstVal(b);}
UnaerOp::UnaerOp(Operator o, double d)		: op(o) 	     			{ opd = new ConstVal(d);}

UnaerOp::UnaerOp(const Cell& c)	             
{ 
	if (c.GetConstCellType() == unaer_op)	
	{ 
		op  = c.GetOp(); 		// changed
		opd = (c.GetOpd()->Copy()); 	// changed
	} 
	else					
	{  
		op  = empty;     
		opd = new ConstVal();	      
	}
}

UnaerOp::UnaerOp(Operator o, const Cell* c) : op(o) 
{ 
	if (!c) opd = new ConstVal(); 
	else	opd = c->Copy();

	if (opd->GetResultType() == textual) CellError("UnaerOp : can't calculate textual-cells");
}
//##############################################################################################################
Cell*		UnaerOp::Copy() 			const 		{ return new UnaerOp(op,opd); 	      }
CellType   	UnaerOp::GetConstCellType() 		const		{ return unaer_op;    		      }
Operator 	UnaerOp::GetOp()			const	 	{ return op;  	      		      }
Cell*		UnaerOp::GetOpd()			const	 	{ return opd; 	      		      }
CellType   	UnaerOp::GetCellType()	 				{ return unaer_op;    		      }
bool	 	UnaerOp::IsPlaceDependent() 				{ return (opd->IsPlaceDependent());   }
bool	 	UnaerOp::IsOnTheSamePage(char* p) 			{ return (opd->IsOnTheSamePage(p));   }
bool		UnaerOp::operator!=(const Cell& c) 			{ return !(*this==c); 		      }
//-------------------------------------------------------------------------------------------------------------
void     	UnaerOp::Clear() 					
{ 
	if (opd) 
	{ 
		opd->Clear();
		delete opd; 
		opd = NULL; 
	}
}
//-------------------------------------------------------------------------------------------------------------
ResultType 	UnaerOp::GetResultType()	 				
{ 
	switch (op)
	{
		case '!' : return logic_expr; 				break;
		case '-' : return opd->GetResultType();			break;
		default	 : CellError("UnaerOp : unknown operator");	break;
	}
	return opd->GetResultType();
}
//-------------------------------------------------------------------------------------------------------------
double 		UnaerOp::CalculateFloat() 
{ 
	CellVal* cv  = opd->Calculate();
	double   val = cv->GetFloat();

	if (cv)			// changed
	{			// changed
		cv->Clear();	// changed
		delete cv;	// changed
	}			// changed

	switch (op)
	{
		case '-'  : val = -(val); break;
		case '!'  : val = !(val); break;
		default   : CellError("UnaerOp : unknown operator");   	break;
	}
	return val;
}
//-------------------------------------------------------------------------------------------------------------
CellVal* 	UnaerOp::Calculate() 
{ 
	CellVal* cv  = opd->Calculate();
	CellVal* ret = NULL;

	switch (op)
	{
		case '-'  : ret = -(*cv); break;
		case '!'  : ret = !(*cv); break;
		default   : CellError("UnaerOp : unknown operator");   	break;
	}

	if (cv)			// changed
	{			// changed
		cv->Clear();	// changed
		delete cv;	// changed
	}			// changed
	return ret;
}
//------------------------------------------------------------------------------------------------------------
Cell&    	UnaerOp::operator =(const Cell& c)
{
	 if (c.GetConstCellType() == unaer_op)
	 {
	 	Clear();
	 	op  = (c.GetOp()); 		// changed
	 	opd = (c.GetOpd()->Copy());	// changed
	 }
	 return *this;
}
//------------------------------------------------------------------------------------------------------------
bool	 UnaerOp::operator==(const Cell& c) 
{
	if (c.GetConstCellType() != unaer_op) return false;
	if (op != c.GetOp())		      return false;

	Cell* tmp_opd = c.GetOpd();
	bool  ret     = false;

	if (*opd == *tmp_opd) ret = true;

	if (tmp_opd)			// changed
	{				// changed
		tmp_opd->Clear();	// changed
		delete tmp_opd;		// changed
	}				// changed
	return ret;
}
//#######################################################################################################################
//				BINAEROP
//#######################################################################################################################
BinaerOp::BinaerOp() 				    : op(empty) 	{ lopd=new ConstVal();   ropd=new ConstVal();   }
BinaerOp::BinaerOp(Operator o, int i,    int i1)    : op(o) 		{ lopd=new ConstVal(i);  ropd=new ConstVal(i1); }
BinaerOp::BinaerOp(Operator o, int i,    bool b)    : op(o) 		{ lopd=new ConstVal(i);  ropd=new ConstVal(b);  }
BinaerOp::BinaerOp(Operator o, int i,    double d)  : op(o) 		{ lopd=new ConstVal(i);  ropd=new ConstVal(d);  }
BinaerOp::BinaerOp(Operator o, bool b,   int i)     : op(o) 		{ lopd=new ConstVal(b);  ropd=new ConstVal(i);  }
BinaerOp::BinaerOp(Operator o, bool b,   bool b1)   : op(o) 		{ lopd=new ConstVal(b);  ropd=new ConstVal(b1); }
BinaerOp::BinaerOp(Operator o, bool b,   double d)  : op(o) 		{ lopd=new ConstVal(b);  ropd=new ConstVal(d);  }
BinaerOp::BinaerOp(Operator o, double d, int i)     : op(o) 		{ lopd=new ConstVal(d);  ropd=new ConstVal(i);  }
BinaerOp::BinaerOp(Operator o, double d, bool b)    : op(o) 		{ lopd=new ConstVal(d);  ropd=new ConstVal(b);  }
BinaerOp::BinaerOp(Operator o, double d, double d1) : op(o) 		{ lopd=new ConstVal(d);  ropd=new ConstVal(d1); }
BinaerOp::BinaerOp(Operator o,  token_multiset* t1, 
				token_multiset* t2) : op(o) 		{ lopd=new ConstVal(t1); ropd=new ConstVal(t2); }

BinaerOp::BinaerOp(const Cell& c) 				
{ 
	if (c.GetConstCellType() == binaer_op) 
	{ 
		op   = (c.GetOp()); 
		lopd = (c.GetLeftOpd()->Copy()); 	// changed
		ropd = (c.GetRightOpd()->Copy()); 	// changed
	}
	else		    
	{ 
		op   = empty;	    
		lopd = new ConstVal();	   
		ropd = new ConstVal();	   	   
	}
}

BinaerOp::BinaerOp(Operator o, const Cell* l, const Cell* r)  
{ 
	op     = o;

	if (!l) lopd = new ConstVal(); 
	else	lopd = l->Copy();
	if (!r) ropd = new ConstVal();
	else    ropd = r->Copy();

	if (lopd->GetResultType() == textual) CellError("BinaerOp (left) : can't calculate textual-cells");
	if (ropd->GetResultType() == textual) CellError("BinaerOp (right): can't calculate textual-cells");
}
BinaerOp::BinaerOp(Operator o, const CellVal* l, const CellVal* r)  
{ 
	op     = o;

	if (!l) lopd = new ConstVal( ); 
	else	lopd = new ConstVal(l);
	if (!r) ropd = new ConstVal( );
	else    ropd = new ConstVal(r);

	if (lopd->GetResultType() == textual) CellError("BinaerOp (left) : can't calculate textual-cells");
	if (ropd->GetResultType() == textual) CellError("BinaerOp (right): can't calculate textual-cells");
}
//#############################################################################################################
Cell*		BinaerOp::Copy() 		const 			{ return new BinaerOp(op,lopd,ropd);  }
Operator 	BinaerOp::GetOp()		const	 		{ return op; 	     		      }
Cell*	 	BinaerOp::GetLeftOpd()		const	 		{ return lopd; 	     		      }
Cell*	 	BinaerOp::GetRightOpd()		const	 		{ return ropd; 	     		      }
CellType   	BinaerOp::GetConstCellType() 	const			{ return binaer_op;  		      }
CellType   	BinaerOp::GetCellType()	 				{ return binaer_op;  		      }
bool		BinaerOp::operator!=(const Cell& c) 			{ return !(*this==c);		      }
//------------------------------------------------------------------------------------------------------------
bool	 	BinaerOp::IsOnTheSamePage(char* p) 	
{ 
	return ((lopd->IsOnTheSamePage(p)) && (ropd->IsOnTheSamePage(p)));   
}
//------------------------------------------------------------------------------------------------------------
bool	 	BinaerOp::IsPlaceDependent() 	
{ 
	return ((lopd->IsPlaceDependent()) || (ropd->IsPlaceDependent()));   
}
//------------------------------------------------------------------------------------------------------------
void 		BinaerOp::Clear()
{
	if (lopd)   { lopd->Clear(); delete lopd; lopd = NULL; }
	if (ropd)   { ropd->Clear(); delete ropd; ropd = NULL; }

}
//------------------------------------------------------------------------------------------------------------
ResultType BinaerOp::GetResultType()	 
{ 

	switch (op)
	{
		case '=' : return logic_expr; break; 
		case '>' : return logic_expr; break;
		case '<' : return logic_expr; break;
		case 'n' : return logic_expr; break;
		case 'g' : return logic_expr; break;
		case 'l' : return logic_expr; break;
		case '&' : return logic_expr; break;
		case '|' : return logic_expr; break;
		default  : break;
	}

	ResultType lrtype = (lopd->GetResultType()); 
	ResultType rrtype = (ropd->GetResultType()); 

	if (lrtype == logic_expr || rrtype == logic_expr)
		return logic_expr;

	switch (lrtype)
	{
		case e_mset_expr  : 	if (rrtype == p_mset_expr) 
						CellError("BinaerOp : e_mset_expr op p_mset_expr");
				    	return e_mset_expr;
				    	break;
		case p_mset_expr  : 	if (rrtype == e_mset_expr) 
						CellError("BinaerOp : p_mset_expr op e_mset_expr");
				    	return p_mset_expr;
				    	break;
		case c_function   :	switch (rrtype)
					{
						case e_mset_expr : return e_mset_expr;  break;
						case p_mset_expr : return p_mset_expr;  break;
						default  	 : return c_function ;	break;
					}
					break;
		case a_function	  :	switch (rrtype)
					{
						case e_mset_expr : return e_mset_expr;  break;
						case p_mset_expr : return p_mset_expr;  break;
						case c_function  : return c_function ;  break;
						default  	 : return a_function ;	break;
					}
					break;
		case numeric	  : 	switch (rrtype)
					{
						case e_mset_expr : return e_mset_expr; 	break;
						case p_mset_expr : return p_mset_expr; 	break;
						case c_function  : return c_function ; 	break;
						default		 : return numeric;	break;
					}
					break;
		default		  : 	CellError("BinaerOp (left) : unexpected result-type");
	}
	return lrtype;
}
//-----------------------------------------------------------------------------------------------------------
double BinaerOp::CalculateFloat() 
{

	CellVal* cv1  = lopd->Calculate();
	CellVal* cv2  = ropd->Calculate();
	double   vall = cv1->GetFloat();
	double   valr = cv2->GetFloat();
	double   ret  = 0;

	if (cv1) { cv1->Clear(); delete cv1; }
	if (cv2) { cv2->Clear(); delete cv2; }

	if (valr == 0 && op == '/')
		CellError("BinaerOp : division by zero");

	switch (op)
	{
		case '+' : ret = (vall +  valr); break;
		case '-' : ret = (vall -  valr); break;
		case '*' : ret = (vall *  valr); break;
		case '/' : ret = (vall /  valr); break;
		case '=' : ret = (vall == valr); break;
		case '>' : ret = (vall >  valr); break;
		case '<' : ret = (vall <  valr); break;
		case 'n' : ret = (vall != valr); break;
		case 'g' : ret = (vall >= valr); break;
		case 'l' : ret = (vall <= valr); break;
		case '&' : ret = (vall && valr); break;
		case '|' : ret = (vall || valr); break;
		default  : CellError("BinaerOp : unknown operator"); break;
	}
	return ret;
}
//-----------------------------------------------------------------------------------------------------------
CellVal* BinaerOp::Calculate() 
{

	CellVal* cvl = lopd->Calculate();
	CellVal* cvr = ropd->Calculate();
	CellVal* ret = NULL;

	switch (op)
	{
		case '+' : ret = (*cvl +  *cvr); break;
		case '-' : ret = (*cvl -  *cvr); break;
		case '*' : ret = (*cvl *  *cvr); break;
		case '/' : ret = (*cvl /  *cvr); break;
		case '=' : ret = (*cvl == *cvr); break;
		case '>' : ret = (*cvl >  *cvr); break;
		case '<' : ret = (*cvl <  *cvr); break;
		case 'n' : ret = (*cvl != *cvr); break;
		case 'g' : ret = (*cvl >= *cvr); break;
		case 'l' : ret = (*cvl <= *cvr); break;
		case '&' : ret = (*cvl && *cvr); break;
		case '|' : ret = (*cvl || *cvr); break;
		default  : CellError("BinaerOp : unknown operator"); break;
	}
	if (cvl) { cvl->Clear(); delete cvl; }	// changed
	if (cvr) { cvr->Clear(); delete cvr; } 	// changed

	return ret;
}
//-----------------------------------------------------------------------------------------------------------
Cell&    BinaerOp::operator =(const Cell& c)
{
	 if (c.GetConstCellType() == binaer_op)
	 {
	 	Clear();
		op   = (c.GetOp()); 
		lopd = (c.GetLeftOpd()->Copy()); 	// changed
		ropd = (c.GetRightOpd()->Copy()); 	// changed
	 }
	 return *this;
}
//-----------------------------------------------------------------------------------------------------------
bool	BinaerOp::operator==(const Cell& c)
{
	   if (c.GetConstCellType() != binaer_op) return false;

	   if (op != c.GetOp()) return false;

	   Cell* tmp_lop = c.GetLeftOpd();
	   Cell* tmp_rop = c.GetRightOpd();
	   bool  ret     = false;

	   if (*lopd==*tmp_lop && *ropd==*tmp_rop)
	   	ret = true;

	   if (tmp_lop) { delete tmp_lop; tmp_lop->Clear(); }	// changed
	   if (tmp_lop) { delete tmp_lop; tmp_rop->Clear(); }	// changed

	   return ret;
}

//#############################################################################################################
//				Function
//#############################################################################################################
Function::Function(const Cell& c)  
{ 
	if (c.GetConstCellType() != function) 
		CellError("unknown funtion in constructor Function(const Cell& c)");

	tms = NULL;

	switch (c.GetOp())
	{
		case '#' : op = c.GetOp(); place = c.GetPlaceHeader(); break;
		case '@' : op = c.GetOp(); place = c.GetPlaceHeader(); break;
		default	 : CellError("unknown operator in constructor Function(const Cell& c)");
	}
}

Function::Function(Operator o, char* p)	
{
	if (!p)	CellError("can't create funktion without placename");

	tms = NULL;

	switch (o)
	{
		case '#' : op = o; place = new PlaceHeader(p); break;
		case '@' : op = o; place = new PlaceHeader(p); break;
		default	 : CellError("unknown operator in constructor Function(const Cell& c)");
	}
}

Function::Function(Operator o, char* p, token_multiset* t)
{
	if (!p)	CellError("can't create funktion without placename");

	tms = copy_token_multiset(t);

	switch (o)
	{
		case '#' : op = o; place = new PlaceHeader(p); break;
		case '@' : op = o; place = new PlaceHeader(p); break;
		default	 : CellError("unknown operator in constructor Function(const Cell& c)");
	}
}

Function::Function(Operator o, PlaceHeader* p, token_multiset* t)
{
	if (!p)	CellError("can't create funktion without placeheader");

	tms = copy_token_multiset(t);

	switch (o)
	{
		case '#' : op = o; place = p; break;
		case '@' : op = o; place = p; break;
		default	 : CellError("unknown operator in constructor Function(const Cell& c)");
	}
}

Function::Function(Operator o, char* p, const Cell* c)
{
	if (!p)	CellError("can't create funktion without placename");

	tms = NULL;

	if (c)  
	{
		Cell*   tmp_cell = c->Copy();

		if ((tmp_cell->GetResultType() == e_mset_expr) || (tmp_cell->GetResultType() == p_mset_expr) )
		{
			CellVal* tmp_cv = tmp_cell->Calculate();
			tms = copy_token_multiset(tmp_cv->Gettoken_multiset());
			tmp_cv->Clear(); delete tmp_cv;
		}
		tmp_cell->Clear(); delete tmp_cell;
	}

	switch (o)
	{
		case '#' : op = o; place = new PlaceHeader(p); break;
		case '@' : op = o; place = new PlaceHeader(p); break;
		default	 : CellError("unknown operator in constructor Function(const Cell& c)");
	}
}

//#############################################################################################################
Cell*		Function::Copy() const 			  	     { return new Function(op, place, tms); }
CellType	Function::GetCellType()	 					{ return function;  	    }
token_multiset*	Function::Gettoken_multiset()			const		{ return tms;		    }
Operator	Function::GetOp()	  			const		{ return op;        	    }
CellType	Function::GetConstCellType()	 		const		{ return function;  	    }
PlaceHeader* 	Function::GetPlaceHeader() 			const		{ return place;     	    }
void	       	Function::ChangePlaceHeader(PlaceHeader* p)			{ if (p) place = p; 	    }
bool     	Function::operator!=(const Cell& c) 				{ return !(*this == c);	    }
bool	 	Function::IsPlaceDependent() 					{ return true;      	    }
void		Function::Clear()		 	   			{ clear_tms(tms); 	    }
//-----------------------------------------------------------------------------------------------------------
bool	 	Function::IsOnTheSamePage(char* p)
{ 
	char* tmp        = NULL;
	char* place_name = NULL;

	if (!p || !place)     			return false;
	if (!(place->GetPlaceName().GetChar()))	return false;

	strcpy(place_name = new char[(strlen(place->GetPlaceName().GetChar()) + 1)],
					    (place->GetPlaceName().GetChar()));

	if (!(tmp = rindex(place_name,'\\')))			{ delete[] place_name; return false; }
	if (strlen(p) != (strlen(place_name) - strlen(tmp)))	{ delete[] place_name; return false; }
	if (!strncmp(p, place_name, strlen(p))) 		{ delete[] place_name; return true;  }

	delete[] place_name;
	return false;
}
//-----------------------------------------------------------------------------------------------------------
ResultType 	Function::GetResultType()	 
{
	switch (op)
	{
		case '@' : return c_function; 	break;
		case '#' : return a_function;  	break;
		default	 : CellError("unknown operator in function GetResultType()");
	}
	return c_function;
}
//-----------------------------------------------------------------------------------------------------------
void       Function::CompNames(token_multiset& p_tms, Textual& n, int& quant, token_multiset*& tms) 
{
	switch (op)
	{
		case '#' : for (token_referenz* tr0=p_tms.first; tr0; tr0=tr0->next)
				if (TokenRefList.GetElement(int(tr0->tref)).GetTokenName() == n) 
					quant += int(tr0->quant);
			   break;
		case '@' : for (token_referenz* tr1=p_tms.first; tr1; tr1=tr1->next)
				if (TokenRefList.GetElement(int(tr1->tref)).GetTokenName() == n) 
					add_token(tms, tr1);
			   break;
		default	 : CellError("unknown operator in function CompNames()");
	}
}
//-------------------------------------------------------------------------------------------------------------
void       Function::CompNamesAndSteps(token_multiset& p_tms,Textual& n,Identifier& s,int& quant,token_multiset*& tms) 
{
	switch (op)
	{
		case '#' : for (token_referenz* tr0=p_tms.first; tr0; tr0=tr0->next)
			   {
				Token tok = TokenRefList.GetElement(int(tr0->tref));
				if (tok.GetTokenName() == n && tok.GetStep() == s)
					quant += int(tr0->quant);
			   }
			   break;
		case '@' : for (token_referenz* tr1=p_tms.first; tr1; tr1=tr1->next)
			   {
				Token tok = TokenRefList.GetElement(int(tr1->tref));
				if (tok.GetTokenName() == n && tok.GetStep() == s)
					add_token(tms, tr1);
			   }
			   break;
		default	 : CellError("unknown operator in function CompNamesAndSteps()");
	}
}
//-------------------------------------------------------------------------------------------------------------
void       Function::CompToken(token_multiset& p_tms, int tref, int& quant, token_multiset*& tms) 
{
	switch (op)
	{
		case '#' : for (token_referenz* tr0=p_tms.first; tr0; tr0=tr0->next)
				if (tr0->tref == tref) quant += int(tr0->quant);
			   break;
		case '@' : for (token_referenz* tr1=p_tms.first; tr1; tr1=tr1->next)
				if (tr1->tref == tref) add_token(tms, tr1);
			   break;
		default	 : CellError("unknown operator in function CompToken()");
	}
}
//-------------------------------------------------------------------------------------------------------------
double       Function::CalculateFloat() 
{
	if (op != '#') return 0;

	token_multiset* p_tms = NULL;

	if (place) p_tms = place->Gettoken_multiset();	
	else	   return 0;

	if (!p_tms || !(p_tms->first)) return 0;
	if (!tms   || !(tms->first))   return (double(get_quantity(p_tms)));

	token_multiset* ret_tms  = NULL;
	int             ret_quant = 0;

	//---------------------------------------------------------------------------------------------------
	//	multiset aus ParameterCell holen
	//---------------------------------------------------------------------------------------------------
   	for (token_referenz* tr=tms->first; tr; tr=tr->next)
   	{
   		Token tok = TokenRefList.GetElement(int(tr->tref));
  		//-------------------------------------------------------------------------------------------
  		//	token = x?  
  		//-------------------------------------------------------------------------------------------
  		if (strcmp(tok.GetStep().GetChar(),WILDCARD) == 0) 
  		{
  			CompNames(*p_tms, tok.GetTokenName(), ret_quant, ret_tms);
  			continue;
		}
  		//-------------------------------------------------------------------------------------------
  		//	token = x?z  
  		//-------------------------------------------------------------------------------------------
		if (strcmp(tok.GetVersion().GetChar(),WILDCARD) == 0)
		{
			CompNamesAndSteps(*p_tms, tok.GetTokenName(), tok.GetStep(), ret_quant, ret_tms);
			continue;
		}
  		//-------------------------------------------------------------------------------------------
  		//	token = xyz  
  		//-------------------------------------------------------------------------------------------
		CompToken(*p_tms, int(tr->tref), ret_quant, ret_tms);
	}
   	return double(ret_quant);
}
//-------------------------------------------------------------------------------------------------------------
CellVal*       Function::Calculate() 
{
	token_multiset* p_tms = NULL;

	if (place) p_tms = place->Gettoken_multiset();	
	else	   p_tms = NULL;

	if (!(p_tms) || !(p_tms->first))	
	{
		switch (op)
		{
			case '@' : return new TokenMSVal(); break;
			case '#' : return new FloatVal();   break;
			default	 : CellError("unknown operator in function Calculate()");
		}
	}

	if (!tms || !(tms->first)) 
	{
		switch (op)
		{
			case '@' : return new TokenMSVal(p_tms); break;
			case '#' : return new FloatVal(get_quantity(p_tms)); break;
			default	 : CellError("unknown operator in function CalculateAll()");
		}
	}

	token_multiset* ret_tms   = NULL;
	int		ret_quant = 0;
	//---------------------------------------------------------------------------------------------------
	//	multiset aus ParameterCell holen
	//---------------------------------------------------------------------------------------------------
   	for (token_referenz* tr=tms->first; tr; tr=tr->next)
   	{
   		Token tok = TokenRefList.GetElement(int(tr->tref));
  		//-------------------------------------------------------------------------------------------
  		//	token = x?  
  		//-------------------------------------------------------------------------------------------
  		if (strcmp(tok.GetStep().GetChar(),WILDCARD) == 0) 
  		{
  			CompNames(*p_tms, tok.GetTokenName(), ret_quant, ret_tms);
  			continue;
		}
  		//-------------------------------------------------------------------------------------------
  		//	token = x?z  
  		//-------------------------------------------------------------------------------------------
		if (strcmp(tok.GetVersion().GetChar(),WILDCARD) == 0)
		{
			CompNamesAndSteps(*p_tms, tok.GetTokenName(), tok.GetStep(), ret_quant, ret_tms);
			continue;
		}
  		//-------------------------------------------------------------------------------------------
  		//	token = xyz  
  		//-------------------------------------------------------------------------------------------
		CompToken(*p_tms, int(tr->tref), ret_quant, ret_tms);
	}
	CellVal* ret_val;

	switch (op)
	{
		case '@' : ret_val = new TokenMSVal(ret_tms); break;
		case '#' : ret_val = new FloatVal(ret_quant); break;
		default	 : CellError("unknown operator in function Calculate()");
	}

	if (ret_tms) { clear_tms(ret_tms); delete ret_tms; }

	return ret_val;
}
//-----------------------------------------------------------------------------------------------------------
Cell&    Function::operator =(const Cell& c)
{
	if (c.GetConstCellType() != function) 
		CellError("unknown funtion in operator =(const Cell& c)");

	Clear();
	tms = copy_token_multiset(c.Gettoken_multiset()); 

	switch (c.GetOp())
	{
		case '#' : op = c.GetOp(); place = c.GetPlaceHeader(); break;
		case '@' : op = c.GetOp(); place = c.GetPlaceHeader(); break;
		default	 : CellError("unknown operator in operator =(const Cell& c)");
	}
	return *this;
}
//-----------------------------------------------------------------------------------------------------------
bool     Function::operator==(const Cell& c)
{
	if (c.GetConstCellType() != function) 
		return false; 

	return (op == c.GetOp() && *(place) == *(c.GetPlaceHeader()) && tms == c.Gettoken_multiset());
}

//#############################################################################################################
//				RatingCell
//#############################################################################################################
RatingCell::RatingCell(RatingType r, Cell* c1, Cell* c2) : rt(r)     	{  	
								  		NewC1(c1); 	           
								  		NewC2(c2);  
								 	}
RatingCell::RatingCell(RatingType r, int   i1, Cell* c2) : rt(r)     	{ 	
										cell1=new ConstVal(i1);
										NewC2(c2);  
									}
RatingCell::RatingCell(RatingType r, Cell* c1, int   i2) : rt(r)     	{ 
										NewC1(c1);             
										cell2=new ConstVal(i2);
									}
RatingCell::RatingCell(RatingType r, int   i1, int   i2) : rt(r)     	{
										cell1=new ConstVal(i1);
										cell2=new ConstVal(i2);
									}
RatingCell::RatingCell(RatingType r, Cell* c) 		 : rt(r)     	{
										NewC1(c);  	           
										cell2=NULL; 
									}
RatingCell::RatingCell(RatingType r, char* c) 		 : rt(r)     	{ 
										cell1=new ConstVal(c); 
										cell2=NULL; 
									}
RatingCell::RatingCell(RatingType r, int i) 		 : rt(r)     	{
										cell1=new ConstVal(i); 
										cell2=NULL; 
									}
RatingCell::RatingCell(const Cell& c)  				     	{ 
										NewRatCell(c);
									}
RatingCell::RatingCell()				 : rt(no_rt) 	{
										cell1=NULL;		   
										cell2=NULL; 
									}
//#############################################################################################################
void		RatingCell::NewRatCell(const Cell& c)
{
	if   (c.GetConstCellType() != rating_cell) 
	{ 
		rt = no_rt; 
		cell1 = NULL; 
		cell2 = NULL; 
		return; 
	}
	rt  = c.GetRatingType();

	Cell* c1 = c.GetCell1();
	Cell* c2 = c.GetCell2();

	NewC1(c1);
	NewC2(c2);

	if (c1) c1->Clear();
	if (c2) c2->Clear();
}
//-----------------------------------------------------------------------------------------------------------
void		RatingCell::NewC1(Cell* c)
{
	if (!c) { cell1 = NULL;  return; }
	switch (rt)
	{
		case expect	:	if (c->GetResultType() == a_function)	{ cell1 = c->Copy(); return; }
					if (c->GetResultType() == numeric)	{ cell1 = c->Copy(); return; }
					else					{ cell1 = NULL;      return; }
		case prob	:	if (c->GetResultType() == logic_expr)	{ cell1 = c->Copy(); return; } 
					if (c->GetResultType() == numeric)	{ cell1 = c->Copy(); return; }
					else					{ cell1 = NULL;      return; }
		case bayes_p	:	if (c->GetResultType() == logic_expr)	{ cell1 = c->Copy(); return; } 
					if (c->GetResultType() == numeric)	{ cell1 = c->Copy(); return; }
					else					{ cell1 = NULL;      return; }
		case bayes_e	:	if (c->GetResultType() == a_function)	{ cell1 = c->Copy(); return; } 
					if (c->GetResultType() == numeric)	{ cell1 = c->Copy(); return; }
					else					{ cell1 = NULL;      return; }
		case through_p	:	if (c->GetResultType() == textual)	{ cell1 = c->Copy(); return; } 
					else					{ cell1 = NULL;      return; }
		default		:	cell1 = NULL;
					return;
	}
}
//-----------------------------------------------------------------------------------------------------------
void		RatingCell::NewC2(Cell* c)
{
	if (!c) { cell2 = NULL;  return; }
	switch (rt)
	{
		case	bayes_p	  :	if (c->GetResultType() == logic_expr)	{ cell2 = c->Copy(); return; }
					if (c->GetResultType() == numeric)      { cell2 = c->Copy(); return; }
					else					{ cell2 = NULL;      return; }
		case	bayes_e	  :	if (c->GetResultType() == logic_expr)	{ cell2 = c->Copy(); return; }
					if (c->GetResultType() == numeric)      { cell2 = c->Copy(); return; }
					else					{ cell2 = NULL;      return; }
		case	through_p :	if (c->GetResultType() == numeric)	{ cell2 = c->Copy(); return; }
					else					{ cell2 = NULL;      return; }
		default		  :	cell2 = NULL;
					return;
	}
}
//-----------------------------------------------------------------------------------------------------------
void		RatingCell::Clear()	     	
{ 
	if (cell1) { cell1->Clear(); delete cell1; cell1 = NULL; } 
	if (cell2) { cell2->Clear(); delete cell2; cell2 = NULL; } 
}
//-----------------------------------------------------------------------------------------------------------
void		RatingCell::SetCell2(int no)			
{ 
	if (cell2)
	{
		cell2->Clear();
		delete cell2;
		cell2 = NULL;
	}
	cell2 = new ConstVal(no);
}
//-----------------------------------------------------------------------------------------------------------
Cell*		RatingCell::Copy() 			const	{ return new RatingCell(rt, cell1, cell2);  }
Cell*		RatingCell::GetCell1()			const	{ return cell1;				    }
Cell*		RatingCell::GetCell2()			const	{ return cell2;				    }
RatingType	RatingCell::GetRatingType()		const 	{ return rt;        	    	    	    }
ResultType 	RatingCell::GetResultType()		      	{ return a_function;	    	    	    }
CellType	RatingCell::GetCellType()		      	{ return rating_cell;        	    	    }
CellType	RatingCell::GetConstCellType()		const 	{ return rating_cell;        	    	    }
bool     	RatingCell::operator!=(const Cell& c) 		{ return !( *this == c );   	    	    }
Cell&    	RatingCell::operator =(const Cell& c) 		{ Clear(); NewRatCell(c); return *this;     }
//------------------------------------------------------------------------------------------------------------
bool	 	RatingCell::IsOnTheSamePage(char* p) 	
{ 
	if (cell1 && cell2) 	return ((cell1->IsOnTheSamePage(p)) && (cell2->IsOnTheSamePage(p)));   
	if (cell1) 		return  (cell1->IsOnTheSamePage(p));   
	if (cell2) 		return  (cell2->IsOnTheSamePage(p));   
	return false;
}
//------------------------------------------------------------------------------------------------------------
bool	 	RatingCell::IsPlaceDependent() 	
{ 
	if (cell1 && cell2) 	return ((cell1->IsPlaceDependent()) || (cell2->IsPlaceDependent()));   
	if (cell1) 		return  (cell1->IsPlaceDependent());   
	if (cell2) 		return  (cell2->IsPlaceDependent());   
	return false;
}
//-----------------------------------------------------------------------------------------------------------
double       	RatingCell::CalculateCell1() 			
{
	if (rt != bayes_e) return 0;
	if (cell1)
	{
		CellVal* cv  = cell1->Calculate();
		double   ret = cv->GetFloat();

		cv->Clear();
		delete cv;

		return ret;
	}
	return 0;
}
//-----------------------------------------------------------------------------------------------------------
double       	RatingCell::CalculateCell2() 			
{
	if (rt != bayes_e) return 0;
	if (cell2)
	{
		CellVal* cv  = cell2->Calculate();
		double   ret = cv->GetFloat();

		cv->Clear();
		delete cv;

		return ret;
	}
	return 0;
}
//-----------------------------------------------------------------------------------------------------------
double       	RatingCell::CalculateFloat() 			
{
	CellVal* c1  = NULL;
	CellVal* c2  = NULL;
	double   d1  = 0;
	double   d2  = 0;
	double   ret = 0;

	if (cell1) { c1 = cell1->Calculate(); d1 = c1->GetFloat(); c1->Clear(); delete c1; }
	if (cell2) { c2 = cell2->Calculate(); d2 = c2->GetFloat(); c2->Clear(); delete c2; }

	switch (rt)
	{
		case	expect	  : ret = d1; break;
		case	prob	  : ret = d1; break;
		case	through_p : ret = d2; break;
		case	bayes_p	  : if (d2)
				    {
					  if (d1) ret = 2;
					  else	  ret = 1;
				    }
				    break; 
		default		  : break;
	}
	return ret;
}
//-----------------------------------------------------------------------------------------------------------
CellVal*       	RatingCell::Calculate() 			
{
	CellVal* c1 = NULL;
	CellVal* c2 = NULL;
	double   d1 = 0;
	double   d2 = 0;
	double   ret = 0;

	if (cell1) { c1 = cell1->Calculate(); d1 = c1->GetFloat(); c1->Clear(); delete c1; }
	if (cell2) { c2 = cell2->Calculate(); d2 = c2->GetFloat(); c2->Clear(); delete c2; }

	switch (rt)
	{
		case	expect	  : ret = d1; break;
		case	prob	  : ret = d1; break;
		case	through_p : ret = d2; break;
		case	bayes_p	  : if (d2)
				    {
				  	  if (d1) ret = 2;
					  else	  ret = 1;
				    }
				    break;
		default		  : break;
	}
	return new FloatVal(ret);
}
//-----------------------------------------------------------------------------------------------------------
bool     	RatingCell::operator==(const Cell& c)
{
	if (c.GetConstCellType() != rating_cell)	return false;
	if (c.GetRatingType() 	 != rt) 		return false;

	Cell* c1 = c.GetCell1();
	Cell* c2 = c.GetCell2();
	bool  ret = false;

	if (rt != bayes_p || rt != bayes_e)
	{
		if (!cell1 && !c1) return true; 
		if ( cell1 &&  c1) 
		{ 
			if (*cell1 == *c1) ret = true;
			else		   ret = false;
		}
		if (c1) c1->Clear();
		return ret;
	}
	if (!cell1 && !c1) 
	{
		if (!cell2 && !c2) return true; 
		if ( cell2 &&  c2)
		{ 
			if (*cell2 == *c2) ret = true;
			else		   ret = false;
		}
		if (c2) c2->Clear();
		return ret;
	}
	if (cell1 && c1) 
	{
		if (!cell2 && !c2) 
		{
			if (*cell1 == *c1) ret = true;
                        else               ret = false;
			c1->Clear();
			return ret; 
		}
		if ( cell2 &&  c2)
		{ 
			if (*cell2 == *c2 && *cell1 == *c1) ret = true;
			else		   		    ret = false;
			c1->Clear();
			c2->Clear();
			return ret; 
		}
		if (c1) c1->Clear();
		if (c2) c2->Clear();
		return ret;
	}
	return false;
}
//#############################################################################################################
#endif
