#ifndef __CellVal_cc__
#define __CellVal_cc__

#include "Textual.h"
#include "token_multiset.h"

#include "CellVal.h"

//########################################################################################################
//				TextVal
//########################################################################################################
TextVal::TextVal()						: tval(DEF_TXT_NAME)	 { };
TextVal::TextVal(char* c)					: tval(c)		 { };
TextVal::TextVal(Textual t)					: tval(t)	 	 { };
TextVal::TextVal(const TextVal& tv) 				: tval(tv.tval)		 { };
//########################################################################################################
void		TextVal::Clear()				{ tval.Clear();            }
CellVal*       	TextVal::Copy()     		const		{ return new TextVal(tval);}
char*		TextVal::GetText() 		const		{ return tval.GetChar();   }
ResultType	TextVal::GetResultType()	const		{ return textual;	   }
//-------------------------------------------------------------------------------------------------------
bool     	TextVal::IsTMS()    		const   { CellValError("op","????? op char*");   return false; } 
CellVal*	TextVal::operator +(const CellVal&) 	{ CellValError("+", "char* +  ???????"); return NULL;  }
CellVal*	TextVal::operator -(const CellVal&) 	{ CellValError("-", "char* -  ???????"); return NULL;  }
CellVal*	TextVal::operator *(const CellVal&) 	{ CellValError("*", "char* *  ???????"); return NULL;  }
CellVal*	TextVal::operator /(const CellVal&) 	{ CellValError("/", "char* /  ???????"); return NULL;  }
CellVal*	TextVal::operator&&(const CellVal&)	{ CellValError("&&","char* && ???????"); return NULL;  }
CellVal*	TextVal::operator||(const CellVal&)	{ CellValError("||","char* || ???????"); return NULL;  }
CellVal*	TextVal::operator>=(CellVal&) 		{ CellValError(">=","char* >= ???????"); return NULL;  }
CellVal*	TextVal::operator<=(CellVal&) 		{ CellValError("<=","char* <= ???????"); return NULL;  }
CellVal*	TextVal::operator >(CellVal&) 		{ CellValError(">"," char* >  ???????"); return NULL;  }
CellVal*	TextVal::operator <(CellVal&)	 	{ CellValError("<"," char* <  ???????"); return NULL;  }
CellVal*	TextVal::operator!(void) 		{ CellValError("!(unary)","!(char*)");   return NULL;  }
CellVal*	TextVal::operator-(void) 		{ CellValError("-(unary)","-(char*)");   return NULL;  }
CellVal*	TextVal::Abs(void) 			{ CellValError("#","#(char*)"); 	 return NULL;  }
CellVal*	TextVal::Contents(void) 		{ CellValError("@","@(char*)"); 	 return NULL;  }
//-------------------------------------------------------------------------------------------------------
CellVal&	TextVal::operator =(const CellVal& cv)
{
	if (!cv.GetText()) CellValError("=","char* = ????????");
	tval = cv.GetText();
	return *this;
}
//-------------------------------------------------------------------------------------------------------
CellVal*	TextVal::operator ==(CellVal& cv)
{
	if (!cv.GetText()) return new FloatVal(false);
	return new FloatVal(tval == cv.GetText());
}
//-------------------------------------------------------------------------------------------------------
CellVal*	TextVal::operator !=(CellVal& cv)
{
	if (!cv.GetText()) return new FloatVal(true);
	return new FloatVal(tval != cv.GetText());
}
//########################################################################################################
//				FLOATVAL
//########################################################################################################
FloatVal::FloatVal()						: val(0.0)               { };
FloatVal::FloatVal(int i)					: val(double(i))         { };
FloatVal::FloatVal(bool b)					: val(double(b))         { };
FloatVal::FloatVal(double d)					: val(d)                 { };
FloatVal::FloatVal(const FloatVal& fv) 				: val(fv.val)            { };
//########################################################################################################
void		FloatVal::Clear()				{ ; }
CellVal*       	FloatVal::Copy()     		const		{ return new FloatVal(val);}
bool     	FloatVal::IsTMS()    		const           { return false; 	   }
double		FloatVal::GetFloat() 		const		{ return val;		   }
ResultType	FloatVal::GetResultType()	const		{ return numeric;	   }
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator!(void) 	     { return new FloatVal(!(bool(val)));  }
CellVal*	FloatVal::operator-(void) 	     { return new FloatVal(-val); 	   }
CellVal*	FloatVal::Abs(void)       	     { return new FloatVal(val<0?-val:val);}
CellVal*	FloatVal::Contents(void)  	     { return new FloatVal(val); 	   }
//--------------------------------------------------------------------------------------------------------
CellVal&	FloatVal::operator =(const CellVal& cv)
{
	if (cv.IsTMS())	CellValError("=","double = token_multiset");
	val = cv.GetFloat();
	return *this;
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator +(const CellVal& cv)
{
	if (cv.IsTMS()) CellValError("+","double + token_multiset"); 
	return new FloatVal(val + cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator -(const CellVal& cv)
{
	if (cv.IsTMS()) CellValError("-","double - token_multiset"); 
	return new FloatVal(val - cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator *(const CellVal& cv)
{
	if (cv.IsTMS()) return ( *(cv.Copy()) * *this );   
	return new FloatVal(val * cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator /(const CellVal& cv)
{
	if (cv.IsTMS()) CellValError("/","double / token_multiset"); 

	double d = cv.GetFloat();
	if (d == 0) return new FloatVal(val);
	else        return new FloatVal(val / d);
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator ==(CellVal& cv)
{
	if (cv.IsTMS()) return new FloatVal(false);
	return new FloatVal(val == cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator !=(CellVal& cv)
{
	if (cv.IsTMS()) return new FloatVal(false); 
	return new FloatVal(val != cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator >=(CellVal& cv)
{
	if (cv.IsTMS())  return new FloatVal(false);
	return new FloatVal(val >= cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator <=(CellVal& cv)
{
	if (cv.IsTMS())  return new FloatVal(false);
	return new FloatVal(val <= cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator >(CellVal& cv)
{
	if (cv.IsTMS())  return new FloatVal(false);
	return new FloatVal(val > cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator <(CellVal& cv)
{
	if (cv.IsTMS())  return new FloatVal(false);
	return new FloatVal(val < cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator &&(const CellVal& cv)
{
	if (cv.IsTMS())  
	{
		CellVal* tmp_cv = cv.Copy();			
		double   ret    = (val && get_quantity(tmp_cv->Gettoken_multiset()));
		tmp_cv->Clear(); delete tmp_cv;			
		return new FloatVal(ret);	
	}
	return new FloatVal(val && cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator ||(const CellVal& cv)
{
	if (cv.IsTMS())  
	{
		CellVal* tmp_cv = cv.Copy();			
		double   ret    = (val || get_quantity(tmp_cv->Gettoken_multiset()));
		tmp_cv->Clear(); delete tmp_cv;			
		return new FloatVal(ret);
	}
	return new FloatVal(val || cv.GetFloat());
}

//########################################################################################################
//				TOKENMSVAL
//########################################################################################################
TokenMSVal::TokenMSVal()                     						   { tms = NULL; } 
TokenMSVal::TokenMSVal(int)                  						   { tms = NULL; }
TokenMSVal::TokenMSVal(const TokenMSVal& t)  				{ tms=copy_token_multiset(t.tms);}
TokenMSVal::TokenMSVal(token_multiset* tset) 				{ tms=copy_token_multiset(tset); }
TokenMSVal::TokenMSVal(TokenRef tr, Quant q) 
{
	tms               = new token_multiset;
	tms->first        = new token_referenz;
	tms->quant	  = q;
	tms->key	  = int(tr + q);
	tms->first->next  = NULL; 
	tms->first->tref  = tr;
	tms->first->quant = q;
}
//########################################################################################################
void		 TokenMSVal::Clear()				{ clear_tms(tms); 			 }
CellVal*       	 TokenMSVal::Copy()            	    const	{ return new TokenMSVal(tms); 	 	 }
bool           	 TokenMSVal::IsTMS()  	       	    const	{ return true; 			 	 }
token_multiset*  TokenMSVal::Gettoken_multiset() 		{ return tms; 				 }
CellVal* 	 TokenMSVal::Abs(void)		    		{ return new FloatVal(get_quantity(tms));}
CellVal* 	 TokenMSVal::Contents(void)	    		{ return new TokenMSVal(tms);	 	 }
//--------------------------------------------------------------------------------------------------------
CellVal*	TokenMSVal::operator -(void)	    
{ 
	CellValError("-(unary)","-(token_multiset)"); 
	return NULL; 
}
//--------------------------------------------------------------------------------------------------------
ResultType	TokenMSVal::GetResultType()	const	
{ 
	if (!tms || !(tms->first))	  return e_mset_expr;
	if (tms->first->tref == NO_COLOR) return e_mset_expr;
	return p_mset_expr;
}
//--------------------------------------------------------------------------------------------------------
CellVal& 	TokenMSVal::operator =(const CellVal& cv)
{
	if (!cv.IsTMS()) CellValError("=","TokenMSVal = ?????");

	CellVal* tmp = cv.Copy();
	tms = copy_token_multiset(tmp->Gettoken_multiset());
	tmp->Clear(); delete tmp;
	return *this;
}
//--------------------------------------------------------------------------------------------------------
CellVal* 	TokenMSVal::operator +(const CellVal& cv) 
{ 
	if (!cv.IsTMS()) CellValError("+","token_multiset + ?????");

	CellVal* 	tmp_cv  = cv.Copy();
	token_multiset*	tmp_tms = copy_token_multiset(tmp_cv->Gettoken_multiset());
	token_multiset*	ret_tms = copy_token_multiset(tms);

	if (tmp_tms && tmp_tms->first)
		for (token_referenz* tr=tmp_tms->first; tr; tr=tr->next)
			add_token(ret_tms, tr);

	CellVal* ret = new TokenMSVal(ret_tms);

	clear_tms(ret_tms); 
	clear_tms(tmp_tms);

	tmp_cv->Clear();
	delete tmp_cv;

        return ret; 
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator -(const CellVal& cv) 
{ 
	if (!cv.IsTMS()) CellValError("-","token_multiset - ?????");

	CellVal* 	tmp_cv  = cv.Copy();
	token_multiset*	tmp_tms = copy_token_multiset(tmp_cv->Gettoken_multiset());
	token_multiset*	ret_tms = copy_token_multiset(tms); 

	if (tmp_tms && tmp_tms->first)
		for (token_referenz* tr=tmp_tms->first; tr; tr=tr->next)
			sub_token(ret_tms, tr);

	CellVal* ret = new TokenMSVal(ret_tms);

	clear_tms(ret_tms);
	clear_tms(tmp_tms); 
	tmp_cv->Clear();
	delete tmp_cv;

        return ret; 
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator *(const CellVal& cv)
{
	if (cv.IsTMS()) CellValError("*","token_multiset * token_multiset");

	int tmp_quant	        = 0;
	int multiplicator       = int(cv.GetFloat());

	if (multiplicator <  0) CellValError("*","token_multiset * -(double)"); 
	if (multiplicator == 0) return new TokenMSVal(); 

	token_multiset*	ret_tms = copy_token_multiset(tms);

	if (ret_tms && ret_tms->first)
	{
		for (token_referenz* tmp=ret_tms->first; tmp; tmp=tmp->next)
		{
			ret_tms->quant -= tmp->quant;
			ret_tms->key   -= tmp->quant;

			if ((tmp_quant = int(tmp->quant * multiplicator)) > MAX_QUANT) 
				tmp->quant = MAX_QUANT;

			ret_tms->quant += tmp->quant;
			ret_tms->key   += tmp->quant;
		}
	}
	CellVal* ret = new TokenMSVal(ret_tms);
	clear_tms(ret_tms); 
        return ret; 
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator /(const CellVal& cv)
{
	if (cv.IsTMS()) CellValError("/","token_multiset / token_multiset");

	int	tmp_quant       = 0;
	double	divisor         = cv.GetFloat();
	token_multiset*	ret_tms = copy_token_multiset(tms);

	if (divisor <  0) CellValError("/","token_multiset / -(double)"); 
	if (divisor == 0) CellValError("/","token_multiset / NULL"); 

	if (ret_tms && ret_tms->first)
	{
		for (token_referenz* tmp=ret_tms->first; tmp; tmp=tmp->next)
		{
			ret_tms->quant -= tmp->quant;
			ret_tms->key   -= tmp->quant;

			tmp_quant     = int(tmp->quant / divisor); 
			tmp->quant    = tmp_quant;

			ret_tms->quant += tmp->quant;
			ret_tms->key   += tmp->quant;
		}
	}
	CellVal* ret = new TokenMSVal(ret_tms);
	clear_tms(ret_tms); 
        return ret; 
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator==(CellVal& cv)
{
	if (!cv.IsTMS()) return new FloatVal(false); 
	return new FloatVal( tms_are_equal(tms, (cv.Gettoken_multiset())) );
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator!=(CellVal& cv)
{
	if (!cv.IsTMS()) return new FloatVal(false); 
	return new FloatVal( !(tms_are_equal(tms, (cv.Gettoken_multiset()))) );
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator <(CellVal& cv)
{
	if (!cv.IsTMS()) return new FloatVal(false); 
	return new FloatVal( tms_is_less(tms, (cv.Gettoken_multiset())) );
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator >(CellVal& cv)
{
	if (!cv.IsTMS()) return new FloatVal(false); 
	return new FloatVal( tms_is_more(tms, (cv.Gettoken_multiset())) );
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator<=(CellVal& cv)
{
	if (!cv.IsTMS()) return new FloatVal(false); 
	return new FloatVal( tms_is_less_or_equal(tms, (cv.Gettoken_multiset())) );
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator>=(CellVal& cv)
{
	if (!cv.IsTMS()) return new FloatVal(false); 
	return new FloatVal( tms_is_more_or_equal(tms, (cv.Gettoken_multiset())) );
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator&&(const CellVal& cv) 
{ 	
	if (!cv.IsTMS()) return new FloatVal(false);

	CellVal* tmp = cv.Copy();
	double   ret = (tms && tmp->Gettoken_multiset());
	tmp->Clear(); delete tmp;
	return new FloatVal(ret);
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator||(const CellVal& cv) 
{ 	
	if (!cv.IsTMS()) return new FloatVal(false);

	CellVal* tmp = cv.Copy();
	double   ret = (tms || tmp->Gettoken_multiset());
	tmp->Clear(); delete tmp;
	return new FloatVal(ret);
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator !(void)        
{ 
	if (tms) return new FloatVal(true);
        return new FloatVal(false);
}
//########################################################################################################
#endif
