#ifndef __CHECKS_CC__
#define __CHECKS_CC__

#include "inc_classes.h"

//----------------------------------------------------------------------------------------------------------
//	externe Variablen/Funktionen
//----------------------------------------------------------------------------------------------------------
extern RRGError* 	RRGErr;
extern void		PrepareEnd();
extern PlaceList	PlaceRefList;
extern PlacePtrList	PlaceCapDepRefList;
extern void		init_global_actual_objects(token_multiset**);
extern ArcBindList*	inarcs;		// globale Liste der aktuell zu checkenden In/Out-Arcs (inc_rrg_globals.h)
extern ArcBindList*	outarcs;	// globale Liste der aktuell zu checkenden In/Out-Arcs (inc_rrg_globals.h)

Place* p_in  = NULL;
Place* p_out = NULL;


//----------------------------------------------------------------------------------------------------------
//
//	Fehler bei der RG-Errechnung	
//
//----------------------------------------------------------------------------------------------------------
void	CheckError(char* s1, char* s2, int line)
{
	if (s2) RRGErr = new RRGError(__FILE__, s1, line, NULL, s2);
	else    RRGErr = new RRGError(__FILE__, s1, line);
	PrepareEnd();
}
//----------------------------------------------------------------------------------------------------------
//
//	Eingangsstelleninhalt ueberpruefen	
//
//----------------------------------------------------------------------------------------------------------
bool	CheckIn()
{
	for (ArcBindListElement* iarc=inarcs->first; iarc; iarc=iarc->next) 
	{
		p_in = NULL;
		p_in = iarc->lelement.GetPlace();

		if (!p_in) CheckError("place is NULL", NULL, __LINE__);
		if (!(p_in->TMSIsInPlace(iarc->lelement.Gettoken_multiset())))
			return false;
	}
	return true;
}
//----------------------------------------------------------------------------------------------------------
//
//	Ausgangsstellenkapazitaet und -Place-Token-Konsistenz ueberpruefen	
//
//----------------------------------------------------------------------------------------------------------
bool	CheckOut()
{
	for (ArcBindListElement* oarc=outarcs->first; oarc; oarc=oarc->next)
	{
		p_out = NULL;
		p_out = oarc->lelement.GetPlace();

		if (!p_out) CheckError("place is NULL", NULL, __LINE__);

		if ( !(p_out->ColorConsistent(oarc->lelement.GetActColor())) || 
		     !(p_out->EnoughCapacity (oarc->lelement.GetActCapacity()))  )
			return false; 
	}
	return true;
}
//----------------------------------------------------------------------------------------------------------
//
//	Eingangsstelleninhalt, Ausgangsstellenkapazitaet und -Place-Token-Konsistenz ueberpruefen	
//
//----------------------------------------------------------------------------------------------------------
bool	CheckInAndOut()
{
	for (ArcBindListElement* iarc=inarcs->first; iarc; iarc=iarc->next) 
	{
		p_in = NULL;
		p_in = iarc->lelement.GetPlace();

		if (!p_in) CheckError("place is NULL", NULL, __LINE__);

		if (!(p_in->TMSIsInPlace(iarc->lelement.Gettoken_multiset())))
			return false;
	}

	for (ArcBindListElement* oarc=outarcs->first; oarc; oarc=oarc->next)
	{
		p_out = NULL;
		p_out = oarc->lelement.GetPlace();

		if (!p_out) CheckError("place is NULL", NULL, __LINE__);

		bool checked = false;

		for (ArcBindListElement* iarc=inarcs->first; iarc; iarc=iarc->next)
		{
			p_in = NULL;
			p_in = iarc->lelement.GetPlace();

			if (!p_in) CheckError("place is NULL", NULL, __LINE__);

			if (p_in->GetPlaceNo() == p_out->GetPlaceNo())
			{
				int diff = (oarc->lelement.GetActCapacity() - iarc->lelement.GetActCapacity());

				if (!(p_out->EnoughCapacity(diff)))
					return false;	
				checked = true;
				break;	
			}
		}

		if (checked) continue;

		if ( !(p_out->ColorConsistent(oarc->lelement.GetActColor())) || 
		     !(p_out->EnoughCapacity (oarc->lelement.GetActCapacity()))  )
			return false; 
	}
	return true;
}
//----------------------------------------------------------------------------------------------------------
//
//	Eingangsstelleninhalt, Ausgangsstellenkapazitaet und -Place-Token-Konsistenz ueberpruefen
//
//----------------------------------------------------------------------------------------------------------
bool	ArcsOk()
{
	if (!outarcs->size && !inarcs->size) return false;
	if ( outarcs->size &&  inarcs->size) return (CheckInAndOut());
	if (!outarcs->size &&  inarcs->size) return (CheckIn      ());
	if ( outarcs->size && !inarcs->size) return (CheckOut     ());
	return true;
}
//----------------------------------------------------------------------------------------------------------
//
//	Ueberpruefen ob eine neue Markierung (direkt nach dem Schalten) gegen die 
//	Stellenabhaengigen-Stellenkapazitaeten verstoesst
//
//----------------------------------------------------------------------------------------------------------
bool	PlaceCapOkAfterFire(token_multiset** new_tms, token_multiset** old_tms)
{
	if ( !(PlaceCapDepRefList.size) ) return true;

	init_global_actual_objects(new_tms);

	for (PlacePtrListElement* place=PlaceCapDepRefList.first; place; place=place->next)
	{
		if ( !(place->plelement) ) CheckError("place is NULL-pointer", NULL, __LINE__);
		if ( !(place->plelement->EnoughCapacity()) ) return false;
	}

	init_global_actual_objects(old_tms);
	return true;
}
//----------------------------------------------------------------------------------------------------------
//
//	Ueberpruefen ob die initiale Markierung gegen die Stellenkapazitaeten verstoesst
//
//----------------------------------------------------------------------------------------------------------
void	CheckInitialMarkingAndPlaceCapacities()
{
	if (PlaceRefList.size)
	{
		for (PlaceListElement* place=PlaceRefList.first; place; place=place->next)
			if (!(place->lelement.EnoughCapacity()))
				CheckError("capacity < initial_marking", place->lelement.GetName(), __LINE__);
	}
}

//----------------------------------------------------------------------------------------------------------
#endif
