#ifndef __DebugFunctions_cc__
#define __DebugFunctions_cc__

#include "inc_classes.h"
#include "inc_examine_rrg.h"
#include "inc_info.h"
#include "inc_debug_funcs.h"

//-----------------------------------------------------------------------------------------------------------
//	extern var's	
//-----------------------------------------------------------------------------------------------------------
extern	char*	ModelDir;

//-----------------------------------------------------------------------------------------------------------
//	lokale Ausgabefunktionen	
//-----------------------------------------------------------------------------------------------------------
void  ShowParsedPlaces		();
void  ShowParsedDefinitions	();
void  ShowPIVector		(int);
void  ShowResults		(char*);
void  ShowNetStructur		();
void  ShowParsedPages		();
void  ShowDelay			(DelayVal);
void  ShowReachabilityGraph	(Marking*);
void  ShowSMCs			(ListOfSMCLists&);
void  ShowMarking		(Marking*, DelayVal*, int);
void  SRG			(Marking*, int&, MarkingList&);
void  ShowSMC			(Marking*&, DelayVal*, int_list*&);

void  EvaluateCell		(Cell*);
void  EvaluatePlaceList		(PlaceList&);
void  EvaluateBindings		(BindingList*);
void  EvaluateTransitions	(TransitionList&);
void  EvaluateArcBindings	(ArcBindList&);
void  EvaluateMarkingType	(unsigned char);
void  EvaluateMarkingStatus	(unsigned char);
void  Evaluate_place_multiset	(token_multiset**);
void  Evaluate_token_multiset	(token_multiset*);
void  Evaluate_token_referenz	(token_referenz*);
void  EvaluateActiveTransArray	(short*);

//---------------------------------------------------------------------------------------------------------------
//		.RESULT - File ausgeben	
//---------------------------------------------------------------------------------------------------------------
void	ShowResults(char* netname)
{
	FILE*	fptr     = NULL;
	char*	filename = NULL;
	char    c        = ' ';

	cout <<"\n\n----------------------------------------------------------\n\n";
	cout <<"RESULTS : \n\n";

	sprintf(filename = new char[(strlen(ModelDir) + strlen(netname) + 2)], "%s/%s.RESULTS", ModelDir, netname);

	if (!(fptr = fopen(filename, "r")))
	{
		cout <<"can't open resultfile : "<<filename<<"\n\n";
		return;
	}
	while (1)
	{
		c = fgetc(fptr);
		if (c == EOF) break;
		cout <<c;
	}
	cout <<"\n\n----------------------------------------------------------\n\n";
}
//---------------------------------------------------------------------------------------------------------------
//		PI-Vektor ausgeben
//---------------------------------------------------------------------------------------------------------------
void  	ShowPIVector(int no_tang_markings)
{
	double pii_sum = 0.0;
	cout <<"\nPI-Vector = ( "; 
	for (int i=0; i < no_tang_markings-1; i++)
	{
		pii_sum += pi_vector[i];
		cout <<pi_vector[i]<<", ";
	}
	pii_sum += pi_vector[no_tang_markings-1];
	cout <<pi_vector[no_tang_markings-1]<<" )\n";
	cout <<"\nPIi-Sum = "<<pii_sum<<"\n";
}
//---------------------------------------------------------------------------------------------------------------
//		hostinfo ausgeben
//---------------------------------------------------------------------------------------------------------------
void	ShowHostInfo(hostinfo*& hi)
{
	cout <<"-----------------------------\n";
	if (hi)
	{
		for (hostinfo* tmp = hi; tmp; tmp=tmp->next)
		{
			cout <<"HOST       : "<<tmp->hostname		<<"\n";
			cout <<"run. procs : "<<tmp->running_processes	<<"\n";
			cout <<"load       : "<<tmp->load		<<"\n";
			cout <<"priority   : "<<tmp->priority		<<"\n";
			cout <<"no_rusers  : "<<tmp->no_rusers		<<"\n";
			cout <<"free       : "<<tmp->free		<<"\n";
			cout <<"-----------------------------\n";
		}
	}
	else	
	{
		cout <<"NO HOSTINFO\n";
		cout <<"-----------------------------\n";
	}
}
//---------------------------------------------------------------------------------------------------------------
//		randinfo ausgeben
//---------------------------------------------------------------------------------------------------------------
void	ShowRandInfo(randinfo*& ri, int sz)
{
	cout <<"-----------------------------\n";
	if (ri)
	{
		int i = 1;
		for (randinfo* tmp = ri; i < sz; i++, tmp++)
		{
			cout <<"tau	: "<<tmp->tau		<<"\n";
			cout <<"max_qii	: "<<tmp->max_qii	<<"\n";
			cout <<"effort	: "<<tmp->effort	<<"\n";
			cout <<"kb_p_sec: "<<tmp->kb_per_sec	<<"\n";
			cout <<"eta	: "<<tmp->eta		<<"\n";
			cout <<"conn_fd	: "<<tmp->connection_fd	<<"\n";
			cout <<"status	: "<<tmp->status	<<"\n";

			if (tmp->host)    	cout <<"host    : not NULL\n";
			else       	   	cout <<"host    : NULL    \n"; 
			
			if (tmp->smcs)    	cout <<"SMCs    : not NULL\n";
			else       	   	cout <<"SMCs    : NULL    \n"; 

			if (tmp->connection_fp) cout <<"conn_fp : not NULL\n";
			else       	   	cout <<"conn_fp : NULL    \n"; 
			cout <<"-----------------------------\n";
		}
	}
	else
	{
		cout <<"NO RANDINFO\n";
		cout <<"-----------------------------\n";
	}
}
//---------------------------------------------------------------------------------------------------------------
//		SMC zeigen
//---------------------------------------------------------------------------------------------------------------
void	ShowSMC(Marking*& marking, DelayVal* delay, int_list*& il)
{
	if ((marking->first) && !(marking->first->nextS))
	{
		if (!MarkingIsInList(marking->GetNumber(), il))
		{
	   		AddMarkingToList(marking->GetNumber(), il);
	   		ShowMarking(marking, delay, 2);
		}
	}
	else
	{
		if ((marking->first) && (marking->first->nextS))
	   	{
			if (!MarkingIsInList(marking->GetNumber(), il))
			{
				AddMarkingToList(marking->GetNumber(), il);
	   			ShowMarking(marking, delay, 2);

				for (state_transition* st=marking->first; st; st=st->nextS)
				{
					if (!MarkingIsInList(st->nextM->GetNumber(), il))
					{
						if ((int(st->delay.delay_typ) == _EXPONENTIAL))
							ShowSMC(st->nextM, &(st->delay), il);

						if ((int(st->delay.delay_typ) == _PREEMP_EXP))
						{
							AddMarkingToList(st->nextM->GetNumber(), il);
							cout <<"PREEMPTING\n\n";
							ShowMarking(st->nextM, &(st->delay), 3);
						}
		  			}
				}
	     		}
	  	}
		else	cout <<"Det. marking without state-transitions\n";
	}
}

//---------------------------------------------------------------------------------------------------------------
//		SMC's zeigen
//---------------------------------------------------------------------------------------------------------------
void	ShowSMCs(ListOfSMCLists& det_marking_list)
{
	if (det_marking_list.size)
	{
		for (ListElementOfSMCLists* h=det_marking_list.first; h; h=h->next)
		{
			cout <<"==================== SMC FOR DET. TRANSITION NO.: "<<h->lelement.GetTransitionRef()<<"\n\n";
			if (h->lelement.det_or_gen_markings.size)
			{
				for (MarkingPtrListElement* mple=h->lelement.det_or_gen_markings.first; mple; mple=mple->next)
				{
					if ((int(mple->plelement->status) == _INITIAL))
					{
						int_list* il = new int_list;
						if ((mple->plelement->first) && !(mple->plelement->first->nextS))
						{
							cout <<"\n\nONE-MARKING-SMC\n\n";
							ShowSMC(mple->plelement,NULL, il);
						}
						else
						{
							cout <<"\n\nMULTIPLE-MARKING-SMC\n\n";
							ShowSMC(mple->plelement,NULL, il);
						}
					}
				}
			}
			else	cout <<"No det. markings in det_marking_list\n";
		}
	}
	else	cout <<"No det. markings in det_marking_list_list\n";
}
//---------------------------------------------------------------------------------------------------------------
//	Markierung ausgeben
//---------------------------------------------------------------------------------------------------------------
void	ShowMarking(Marking* marking, DelayVal* delay, int level)
{
	if (marking)
	{
		switch (level)
		{
			case 0 :
			cout <<"**********************************************************************************\n";
			cout <<"Marking No. : "<<marking->GetNumber()<<"\n";
			cout <<"MarkingType : "; EvaluateMarkingType(marking->mtype); 
			cout <<"**********************************************************************************\n";
			cout <<"ANZAHL VON UEBERGAENGEN : "<<marking->GetNoOfStateTrans()<<"\n";
			break;

			case 1:
			cout <<"----------------------------------------------------------------------------------\n";
			if (delay) ShowDelay(*delay); 
			cout <<" ----> Submarking"; cout <<" No.: "<<marking->GetNumber()<<"\n";
			cout <<"----------------------------------------------------------------------------------\n";
			break;

			case 2:
			cout <<"++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
			cout <<"Marking No. : "<<marking->GetNumber()<<"\n";
			cout <<"DelayType   : "; if (delay) ShowDelay(*delay); cout <<"\n"; 
			cout <<"MarkingType : "; EvaluateMarkingType	(marking->mtype); 
			cout <<"ExamineStat.: "; EvaluateMarkingStatus	(marking->status);
			cout <<"++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
			cout <<"ANZAHL VON UEBERGAENGEN : "<<marking->GetNoOfStateTrans()<<"\n";
			break;

			case 3:
			cout <<"----------------------------------------------------------------------------------\n";
			cout <<"Marking No. : "<<marking->GetNumber()<<"\n";
			cout <<"DelayType   : "; if (delay) ShowDelay(*delay); cout <<"\n";
			cout <<"MarkingType : "; EvaluateMarkingType	(marking->mtype); 
			cout <<"ExamineStat.: "; EvaluateMarkingStatus	(marking->status);
			cout <<"----------------------------------------------------------------------------------\n";

			default : break;
		}
		Evaluate_place_multiset(marking->pms);
		if (marking->GetActiveTransArray())
		{
			cout <<"Active transitions: \n";
			EvaluateActiveTransArray(marking->GetActiveTransArray());
		}
		else	cout <<"No active transition \n";
	}
	else	cout <<"Marking is NULL-pointer\n";
}
//---------------------------------------------------------------------------------------------------------------
//		RG ausgeben	
//---------------------------------------------------------------------------------------------------------------
void	ShowReachabilityGraph(Marking* initial_marking)
{
	cout <<"********************* statistiks *********************\n";
	cout <<" no_of_gen_transitions : "<<no_of_gen_transitions  <<"\n";
	cout <<" no_of_exp_transitions : "<<no_of_exp_transitions  <<"\n";
	cout <<" no_of_det_transitions : "<<no_of_det_transitions  <<"\n";
	cout <<" no_of_imm_transitions : "<<no_of_imm_transitions  <<"\n";
	cout <<" no_of_mix_transitions : "<<no_of_mix_transitions  <<"\n";
	cout <<"********************* statistiks *********************\n";

	if (initial_marking)
	{
		int 		marking_count = 0;
		MarkingList*	mlist 	      = new MarkingList();

		SRG(initial_marking, marking_count, *mlist);

		cout <<"ANZ. MARKING : "<<marking_count<<"\n";
	}
	else
	{
		cout <<"marking is NULL-Pointer\n";
	}
}
//---------------------------------------------------------------------------------------------------------------
void	SRG(Marking* initial_marking, int& marking_count, MarkingList& mlist)
{
	if ((mlist.UniqueAppend(*(new Marking(initial_marking->pms,initial_marking->GetActiveTransArray()))) > 0))
	{
		marking_count++;
		ShowMarking(initial_marking, NULL, 0);

		if (initial_marking->first)
		{
			for (state_transition* s=initial_marking->first; s; s=s->nextS)
				ShowMarking(s->nextM, &(s->delay), 1);
			for (state_transition* s1=initial_marking->first; s1; s1=s1->nextS)
				if (s1->nextM) SRG(s1->nextM, marking_count, mlist);
		}
	}
}
//---------------------------------------------------------------------------------------------------------------
//	EvaluateActiveTransArray
//---------------------------------------------------------------------------------------------------------------
void	EvaluateActiveTransArray(short* ata)
{
	for (int i=0; i < no_of_exc_transitions-1; i++)
	{
		if (ata[i] != 0) cout <<"T["<<i+1<<"] = B["<<ata[i]<<"] \n";
		else		 cout <<"T["<<i+1<<"] = 0 \n";
	}
	cout <<"\n";
}
//---------------------------------------------------------------------------------------------------------------
//	EvaluateMarkingStatus
//---------------------------------------------------------------------------------------------------------------
void	EvaluateMarkingStatus(unsigned char es)
{
	switch (int(es))
	{
		case	_COUNTED	: 	cout <<" (counted)    \n"; break;
		case	_VISITED	:	cout <<" (visited)    \n"; break;
		case	_INITIAL	:	cout <<" (initial)    \n"; break;
		case	_LAST_OF_SCC	:	cout <<" (last_of)    \n"; break;
		default			:	cout <<" ()           \n"; break;
	}
}
//---------------------------------------------------------------------------------------------------------------
//	EvaluateMarkingType
//---------------------------------------------------------------------------------------------------------------
void	EvaluateMarkingType(unsigned char mtype)
{
	switch (int(mtype))
	{
		case	_IMMEDIATE	:	cout <<" (immediate)          \n"; break;
		case	_EXPONENTIAL	:	cout <<" (exponential)        \n"; break;
		case	_DETERMINISTIC	:	cout <<" (deterministic)      \n"; break;
		case	_GENERAL	:	cout <<" (general)            \n"; break;
		case	_PROBABILITY	:	cout <<" (probability)        \n"; break;
		default			:	cout <<" ()                   \n"; break;
	}
}
//---------------------------------------------------------------------------------------------------------------
//	 Showdelay	
//---------------------------------------------------------------------------------------------------------------
void ShowDelay(DelayVal d)
{
	switch (int(d.delay_typ))
	{
		case _IMMEDIATE     	: cout <<"Imm("<<d.GetDelayVal()<<") TRef = "<<d.GetTransitionRef(); break;
		case _EXPONENTIAL   	: cout <<"Exp("<<d.GetDelayVal()<<") TRef = "<<d.GetTransitionRef(); break;
		case _PREEMP_EXP	: cout <<"PRE("<<d.GetDelayVal()<<") TRef = "<<d.GetTransitionRef(); break;
		case _DETERMINISTIC 	: cout <<"Det("<<d.GetDelayVal()<<") TRef = "<<d.GetTransitionRef(); break;
		case _PROBABILITY   	: cout <<"Pro("<<d.GetDelayVal()<<") TRef = "<<d.GetTransitionRef(); break;
		case _GENERAL       	: cout <<"Gen("<<d.GetDelayVal()<<") TRef = "<<d.GetTransitionRef(); break;
		default            	: cout <<"XXX()                      TRef = "<<d.GetTransitionRef(); break;
	}
}

//---------------------------------------------------------------------------------------------------------------
//		PlaceList ausgeben	
//---------------------------------------------------------------------------------------------------------------
void	EvaluatePlaceList(PlaceList& pl)
{
	cout <<"*********************** PlaceList begin *************************\n";
	if (pl.size)
	{
		for (PlaceListElement* ple=pl.first; ple; ple=ple->next)
		{
			cout <<"pname : "<<ple->lelement.GetName()	<<"\n";
			cout <<"ptype : "<<ple->lelement.GetPlaceType()	<<"\n";
			cout <<"koc   : "<<ple->lelement.GetKindOfCap()	<<"\n";
			cout <<"TMS   : ";
			if (ple->lelement.Gettoken_multiset()) 
			{ 
				Evaluate_token_multiset(ple->lelement.Gettoken_multiset()); 
				cout <<"\n"; 
			}
			else     cout <<"tokenmultiset is empty\n";
		}
	}
	else	cout <<"placelist is empty\n";
	cout <<"*********************** PlaceList end ***************************\n";
}

//---------------------------------------------------------------------------------------------------------------
//		Marking-PlaceMultiSet ausgeben	
//---------------------------------------------------------------------------------------------------------------
void	Evaluate_place_multiset(token_multiset** pms)
{
	if (pms)
	{
		cout <<"\n";
		for (int i=0; i<no_of_places-1; i++)
		{
			Evaluate_token_multiset(pms[i]);
			cout <<PlaceHeaderPtrRefList.GetElement(i + 1)->GetName()<<"\t";
			cout <<PlaceHeaderPtrRefList.GetElement(i + 1)->GetPlaceNo()<<"\n";
		}
	}
	else	cout <<"empty placemultiset !\n";
}
//---------------------------------------------------------------------------------------------------------------
//		token_multiset ausgeben	
//---------------------------------------------------------------------------------------------------------------
void	Evaluate_token_multiset(token_multiset* tms)
{
	if (tms && tms->first)
	{
		for (token_referenz* tmp_tr=tms->first; tmp_tr; tmp_tr=tmp_tr->next)
		{
			Evaluate_token_referenz(tmp_tr);
			if (tmp_tr->next) cout <<"\n";
		}
		cout <<"\tKEY = "<<tms->key<<"\t";
	}
	else	cout <<"empty token_multiset\t\t\t\t\t\t\t";
}
//---------------------------------------------------------------------------------------------------------------
//		token_referenz ausgeben	
//---------------------------------------------------------------------------------------------------------------
void	Evaluate_token_referenz(token_referenz* t_ref)
{
	if (t_ref)
	{
		cout <<"TokenRef  : "<< t_ref->tref <<"\t";
		switch (t_ref->tref)
		{
			case  0 : cout <<"TokenName : e\t\t\t"; break;
			default : Token	 tok  = TokenRefList.GetElement(int(t_ref->tref));
				  char*	 name = tok.GetTokenName().GetChar();
				  char*	 vers = tok.GetVersion().GetChar();
				  char*	 step = tok.GetStep().GetChar();
				  cout <<"TokenName : "<< name <<"["<< vers <<"]"<< step <<"\t\t";
				  break;
		}
		cout <<"Quantity  : "<< t_ref->quant;
	}
	else	cout <<"empty \t\t\t\t\t\t\t";
}
//---------------------------------------------------------------------------------------------------------------
//		Geparste Definitions ausgeben
//---------------------------------------------------------------------------------------------------------------
void  ShowParsedDefinitions()
{
	cout <<"\n\n";
	cout <<"--------------------------------------------------\n";
	cout <<"----------------- Definitions --------------------\n";
	cout <<"--------------------------------------------------\n";
	for (DefinitionListElement* h=DefinitionRefList.first; h; h=h->next)
	{
		cout <<"DefinitionVar  : " <<h->lelement.GetVar().GetChar()<<"\n";
		cout <<"DefinitionVal  : ";  EvaluateCell(h->lelement.GetVal());
	}
	cout <<"\n\n";
}
		
//---------------------------------------------------------------------------------------------------------------
//		Geparste Stellen ausgeben
//---------------------------------------------------------------------------------------------------------------
void  ShowParsedPlaces()
{
	cout <<"\n\n";
	cout <<"--------------------------------------------------\n";
	cout <<"----------------- Places -------------------------\n";
	cout <<"--------------------------------------------------\n";
	for (PlaceListElement* h=PlaceRefList.first; h; h=h->next)
	{
		cout <<"PlaceName      : " <<h->lelement.GetName()		 <<"\n";
		cout <<"PlaceType      : " <<h->lelement.GetPlaceType()          <<"\n";
		cout <<"Capacity       : ";  EvaluateCell(h->lelement.GetCapacity());
		cout <<"InitialMarking : \n"; 
		cout <<"---------------- \n"; 
		Evaluate_token_multiset(h->lelement.Gettoken_multiset());
		cout <<"\n";
	}
	cout <<"\n\n";
}

//---------------------------------------------------------------------------------------------------------------
//		Geparste Seiten ausgeben
//---------------------------------------------------------------------------------------------------------------
void  ShowParsedPages()
{
	if (PageRefList.size)
	{
		for (PageListElement* page=PageRefList.first; page; page=page->next)
		{
			cout <<"###################################################\n";
			cout <<"PageName     : "<<page->lelement.GetName()<<"\n";
			cout <<"---------------------------------------------------\n";
			if (page->lelement.GetSubPages())
			{
				for (TextualListElement* subp=page->lelement.GetSubPages()->first; subp; subp=subp->next)
					cout <<"subpage      : "<<subp->lelement.GetChar()<<"\n";
			}
			else	cout <<"no subpages\n";
		}
		cout <<"\n";
	}
}

//---------------------------------------------------------------------------------------------------------------
//		Geparste Transition ausgeben
//---------------------------------------------------------------------------------------------------------------
void  EvaluateTransitions(TransitionList& tl)
{
	int  szi=0; 
	if ((szi=tl.size) > 0 )
	{
		for (TransitionListElement* h=tl.first; h; h=h->next)
		{
			cout <<"\n\n";
			cout <<"--------------------------------------------------\n";
			cout <<"----------------- RealTransition -----------------\n";
			cout <<"--------------------------------------------------\n";
			cout <<"TransitionName : "<<h->lelement.GetTransitionName().GetChar()	<<"\n";
			cout <<"TransitionRef  : "<<h->lelement.GetGlobalTransitionRef()	<<"\n";
			cout <<"ServerType     : "<<h->lelement.GetServerType()            	<<"\n";
			cout <<"GlobalCond     : "; 
			EvaluateCell    (h->lelement.GetGlobalCondition());
			EvaluateBindings(h->lelement.GetBindings());

		}
		cout <<"\n\n";
	}
}

//---------------------------------------------------------------------------------------------------------------
//				Bindings ausgeben
//---------------------------------------------------------------------------------------------------------------
void EvaluateBindings(BindingList* bil)
{
	int count = 1; 
	for (BindingOfList* h=bil->first; h; h=h->next,count++)
	{
			cout <<"\n****************-> Binding "<<count<<"\n";
			cout <<"Weight         : " <<h->lelement.GetWeight()<<"\n";
			cout <<"LocalCond      : ";  EvaluateCell(h->lelement.GetLocalCondition());
			Cell* dc; 
			if ((dc=h->lelement.GetDelay()->GetImmediate()))     cout <<"Imm Delay      : ";
			if ((dc=h->lelement.GetDelay()->GetDeterministic())) cout <<"Det Delay      : ";
			if ((dc=h->lelement.GetDelay()->GetExponential()))   cout <<"Exp Delay      : "; 
			EvaluateCell(dc);
			EvaluateArcBindings(h->lelement.GetInArcBind());
			EvaluateArcBindings(h->lelement.GetOutArcBind());
	}
}

//---------------------------------------------------------------------------------------------------------------
//				ArcBind ausgeben
//---------------------------------------------------------------------------------------------------------------
void EvaluateArcBindings(ArcBindList& al)
{
	int count = 1; 
	if (al.size)
	{
		for (ArcBindListElement* h=al.first; h; h=h->next, count++)
		{
			cout <<"****************-> ArcBind "<<count<<"\n";
			cout <<"PlaceRef       : ";
			cout <<h->lelement.GetPlace()->GetName()<<"\n";
			Evaluate_token_multiset(h->lelement.Gettoken_multiset());
		}
	}
	else	cout <<"\n\nBinding is empty \n\n";
}

//---------------------------------------------------------------------------------------------------------------
//				Cell ausgeben
//---------------------------------------------------------------------------------------------------------------
void EvaluateCell(Cell* c)
{
	if (c->GetResultType() == textual) 
	{
		cout <<"Textual        : "<<c->Calculate()->GetText()<<"\n";
		return;
	}

	CellVal* cell = c->Calculate();

	if (cell->IsTMS()) 	Evaluate_token_multiset(cell->Gettoken_multiset());
	else 			cout <<"FloatVal       : "<<cell->GetFloat()<<"\n";
}

#endif
