#ifndef __Delay_cc__
#define __Delay_cc__

extern "C" {
#include <math.h>
#include <stdio.h>
#include <string.h>
}
#include <iostream.h>

#include "Identifier.h"
#include "Textual.h"
#include "TextualList.h"

#include "token_multiset.h"
#include "PlaceHeader.h"
#include "Token.h"

#include "CellVal.h"
#include "Cell.h"
#include "Delay.h"

//#######################################################################################################
//				IMMEDIATE
//#######################################################################################################
Immediate::Immediate()				 
{ 
	imm = new ConstVal(DEF_IMM_WEIGHT); 
}
Immediate::Immediate(const Cell*  c)		 
{ 
	if (!c) imm = new ConstVal(DEF_IMM_WEIGHT);
	else    imm = c->Copy();
}
Immediate::Immediate(const Delay& d)					
{ 
	if ((d.GetDelayType()) == _IMMEDIATE) *this = *(d.Copy());
	else				      imm  = new ConstVal(DEF_IMM_WEIGHT);

}
//#######################################################################################################
Delay*		Immediate::Copy() 			const 	 { return new Immediate(imm);		}
Cell*		Immediate::GetImmediate() 		const	 { return imm; 		 		}
Cell*		Immediate::GetCell() 			const	 { return imm; 		 		}
int		Immediate::GetDelayType()		const	 { return _IMMEDIATE; 	 		}
bool		Immediate::operator!=(const Delay& d)		 { return !( *this == d );		}
bool		Immediate::IsPlaceDependent()		 	 { return (imm->IsPlaceDependent());	}
//-------------------------------------------------------------------------------------------------------
void		Immediate::Clear()				 
{ 
	if (imm) 
	{ 
		imm->Clear();
		delete imm; 
		imm = NULL; 
	} 
}
//-------------------------------------------------------------------------------------------------------
double  	Immediate::CalculateFloat()			  
{ 
	CellVal* cv  = imm->Calculate();
	double lamda = cv->GetFloat();

	cv->Clear();
	delete cv;

	if (!lamda || lamda < 0 ) 
		DelayError("Value given to immediate-delay is zero or less than zero");

	return lamda;
}
//-------------------------------------------------------------------------------------------------------
CellVal*  	Immediate::Calculate()			  
{ 
	CellVal* cv  = imm->Calculate();
	double lamda = cv->GetFloat();

	cv->Clear();
	delete cv;

	if (!lamda || lamda < 0 ) 
		DelayError("Value given to immediate-delay is zero or less than zero");

	return new FloatVal(lamda);
}
//-------------------------------------------------------------------------------------------------------
Delay&    	Immediate::operator =(const Delay& d)
{
	if ((d.GetDelayType()) == _IMMEDIATE)
	{
		Clear();
		*this = *(d.Copy());
	}
	return *this;
}
//-------------------------------------------------------------------------------------------------------
Delay*     	Immediate::operator =(const Delay* d)
{
	if (d)
	{
		if ((d->GetDelayType()) == _IMMEDIATE)
		{
			Clear();
			*this = *(d->Copy());
		}
	}
	return this;
}
//-------------------------------------------------------------------------------------------------------
bool       	Immediate::operator==(const Delay& d) 
{ 
	if ((d.GetDelayType()) == _IMMEDIATE) 
	{
		Cell* c = d.GetImmediate();
		return (*imm == *c);
	}
	return false; 
}

//#######################################################################################################
//				EXPONENTIAL
//#######################################################################################################
Exponential::Exponential()						
{ 
	expo = new ConstVal();
}
Exponential::Exponential(const Cell* c)					
{  
	if (!c) expo = new ConstVal();
	else    expo = c->Copy();
}
Exponential::Exponential(const Delay& d)				
{ 
	if ((d.GetDelayType()) == _EXPONENTIAL) *this = *(d.Copy());
	else					expo = new ConstVal();
}
//#######################################################################################################
Delay*		Exponential::Copy() 			const 	  { return new Exponential(expo);       }
Cell*		Exponential::GetExponential()		const	  { return expo; 	                }
Cell*		Exponential::GetCell()			const	  { return expo; 	                }
int		Exponential::GetDelayType()		const	  { return _EXPONENTIAL; 	        }
bool		Exponential::operator!=(const Delay& d) 	  { return !( *this == d );             }
bool		Exponential::IsPlaceDependent()		 	  { return (expo->IsPlaceDependent());	}
//-------------------------------------------------------------------------------------------------------
void		Exponential::Clear() 				  
{ 
	if (expo) 
	{ 
		expo->Clear();
		delete expo;
		expo=NULL; 
	}
}
//-------------------------------------------------------------------------------------------------------
double  	Exponential::CalculateFloat()			  
{ 
	//------------------------------------------------------------------
	//	berechnet aus k_lambda = 1/lambda den Kehrwert, also lambda
	//------------------------------------------------------------------
	CellVal* cv     = expo->Calculate();
	double k_lambda = cv->GetFloat();

	cv->Clear();
	delete cv;

	if (!k_lambda || k_lambda < 0) 
		DelayError("Value given to exponential-delay is zero or less than zero");
	return (1 / k_lambda); 
}
//-------------------------------------------------------------------------------------------------------
CellVal*  	Exponential::Calculate()			  
{ 
	//------------------------------------------------------------------
	//	berechnet aus k_lambda = 1/lambda den Kehrwert, also lambda
	//------------------------------------------------------------------
	CellVal* cv     = expo->Calculate();
	double k_lambda = cv->GetFloat();

	cv->Clear();
	delete cv;

	if (!k_lambda || k_lambda < 0) 
		DelayError("Value given to exponential-delay is zero or less than zero");
	return new FloatVal( 1 / k_lambda); 
}
//-------------------------------------------------------------------------------------------------------
Delay&	  	Exponential::operator =(const Delay& d)
{
	if ((d.GetDelayType()) == _EXPONENTIAL) 
	{
		Clear();
		*this = *(d.Copy());
	}
	return *this;
}
//-------------------------------------------------------------------------------------------------------
Delay*		Exponential::operator =(const Delay* d)
{
	if (d)
	{
		if ((d->GetDelayType()) == _EXPONENTIAL) 
		{
			Clear();
			*this = *(d->Copy());
		}
	}
	return this;
}
//-------------------------------------------------------------------------------------------------------
bool		Exponential::operator==(const Delay& d)
{
	if ((d.GetDelayType()) == _EXPONENTIAL) 
	{
		Cell* c = d.GetExponential();
		return (*expo == *c);
	}
	return false;
}
//#######################################################################################################
//				DETERMINISTIC
//#######################################################################################################
Deterministic::Deterministic()						
{ 
	det = new ConstVal();
}
Deterministic::Deterministic(const Cell* c)				
{  
	if (!c) det = new ConstVal();
	else    det = c->Copy();
}
Deterministic::Deterministic(const Delay& d)				
{ 
	if ((d.GetDelayType()) == _DETERMINISTIC) *this = *(d.Copy());
	else					  det  = new ConstVal();
}
//#######################################################################################################
Delay*	Deterministic::Copy() 				const	{ return new Deterministic(det);	}
Cell*	Deterministic::GetDeterministic() 		const	{ return det; 				}
Cell*	Deterministic::GetCell() 			const	{ return det; 				}
int	Deterministic::GetDelayType()			const	{ return _DETERMINISTIC; 		}
bool	Deterministic::operator!=(const Delay& d)		{ return !( *this == d ); 		}
bool	Deterministic::IsPlaceDependent()		 	{ return (det->IsPlaceDependent());	}
//-------------------------------------------------------------------------------------------------------
void	Deterministic::Clear() 				  	
{ 
	if (det) 
	{ 
		det->Clear();
		delete det; 
		det = NULL; 
	}	
}
//-------------------------------------------------------------------------------------------------------
double 		Deterministic::CalculateFloat()			  
{
	CellVal* cv  = det->Calculate();
	double lamda = cv->GetFloat();

	cv->Clear();
	delete cv;

	if (!lamda || lamda < 0 ) 
		DelayError("Value given to deterministic-delay is zero or less than zero");

	return lamda;
}
//-------------------------------------------------------------------------------------------------------
CellVal* 	Deterministic::Calculate()			  
{
	CellVal* cv  = det->Calculate();
	double lamda = cv->GetFloat();

	cv->Clear();
	delete cv;

	if (!lamda || lamda < 0 ) 
		DelayError("Value given to deterministic-delay is zero or less than zero");
	return new FloatVal(lamda);
}
//-------------------------------------------------------------------------------------------------------
Delay&		Deterministic::operator =(const Delay& d)
{
	if ((d.GetDelayType()) == _DETERMINISTIC)
	{
		Clear();
		*this = *(d.Copy());
	}
	return *this;
}
//-------------------------------------------------------------------------------------------------------
Delay*		Deterministic::operator =(const Delay* d)
{
	if (d)
	{
		if ((d->GetDelayType()) == _DETERMINISTIC)
		{
			Clear();
			*this = *(d->Copy());
		}
	}
	return this;
}
//-------------------------------------------------------------------------------------------------------
bool		Deterministic::operator==(const Delay& d)
{
	if ((d.GetDelayType()) == _DETERMINISTIC)
	{
		Cell* c = d.GetDeterministic();
		return (*det == *c);
	}
	return false;
}
//#######################################################################################################
//				GENERAL
//#######################################################################################################
General::General()							 { pmf = NULL; 			}
General::General(char* pl)					 
{ 
	if (pl) strcpy(pmf = new char[(strlen(pl) + 1)], pl);
	else    pmf = NULL;
}
General::General(const Delay& d)					
{ 
	if ((d.GetDelayType()) == _GENERAL) 	
	{
		char* pl = d.GetGeneral();

		if (pl) strcpy(pmf = new char[(strlen(pl) + 1)], pl);
		else    pmf = NULL;
	}
	else	pmf = NULL;	
}
//#######################################################################################################
Delay*		General::Copy() 			const 	{ return new General(pmf); 		}
char*		General::GetGeneral() 			const	{ return pmf; 				}
Cell*		General::GetCell()			const	{ return NULL;				}
int		General::GetDelayType()			const	{ return _GENERAL; 			}
CellVal*	General::Calculate()				{ return new FloatVal(DEF_IMM_WEIGHT); 	}
double 		General::CalculateFloat()			{ return double(DEF_IMM_WEIGHT); 	}
bool		General::operator!=(const Delay& d) 		{ return !( *this == d ); 		}
bool		General::IsPlaceDependent()		 	{ return false;				}
void       	General::PrintPMF(FILE* fp) 			{ if (pmf) fprintf(fp,"%s",pmf); 	}
//-------------------------------------------------------------------------------------------------------
void		General::Clear() 				
{ 
	if (pmf) 
	{ 
		delete[] pmf; 
		pmf = NULL; 
	}
}
//-------------------------------------------------------------------------------------------------------
Delay&   	General::operator =(const Delay& d)
{
	if ((d.GetDelayType()) == _GENERAL)
	{
		char* pl = d.GetGeneral();

                if (pl) 
                {
			Clear();
			strcpy(pmf = new char[(strlen(pl) + 1)], pl);
		}
	}
	return *this;
}
//-------------------------------------------------------------------------------------------------------
Delay*		General::operator =(const Delay* d)
{
	if (d)
	{
		if ((d->GetDelayType()) == _GENERAL)
		{
			char* pl = d->GetGeneral();

			if (pl) 
                	{
				Clear();
				strcpy(pmf = new char[(strlen(pl) + 1)], pl);
			}
		}
	}
	return this;
}
//-------------------------------------------------------------------------------------------------------
bool     	General::operator==(const Delay& d)
{
	if ((d.GetDelayType()) == _GENERAL)
	{
		char* pl = d.GetGeneral();

		if (!pl && !pmf) return true;
                if ( pl &&  pmf)
                {
                	if (!strcmp(pmf,pl)) return true;
                	else                 return false;
		}
	}
	return false;
} 
//#######################################################################################################
//				UNIFORM	
//#######################################################################################################
UniForm::UniForm()				: beg(0), end(0) {};		 
UniForm::UniForm(double b, double e)		: beg(b), end(e) {};
UniForm::UniForm(const Delay& d)					
{ 
	if ((d.GetDelayType()) == _GENERAL && !(d.GetGeneral()))
	{
		beg = d.GetUniformBegin();
		end = d.GetUniformEnd();
	}
	else
	{
		beg = 0;
		end = 0;
	} 

}
//#######################################################################################################
Delay*		UniForm::Copy() 			const 	 { return new UniForm(beg, end);	}
double		UniForm::GetUniformBegin() 		const	 { return beg; 		 		}
double		UniForm::GetUniformEnd() 		const	 { return end; 		 		}
Cell*		UniForm::GetCell() 			const	 { return NULL;		 		}
int		UniForm::GetDelayType()			const	 { return _GENERAL; 	 		}
bool		UniForm::operator!=(const Delay& d)		 { return !( *this == d );		}
bool		UniForm::IsPlaceDependent()		 	 { return false;			}
void		UniForm::Clear()				 { beg = 0; end = 0;			}
double  	UniForm::CalculateFloat()			 { return beg;				} 
CellVal*  	UniForm::Calculate()			  	 { return new FloatVal(end);		}
void       	UniForm::PrintPMF(FILE* fp) 			 { fprintf(fp,"UNIFORM(%f,%f)",beg,end);}
//-------------------------------------------------------------------------------------------------------
Delay&    	UniForm::operator =(const Delay& d)
{
	if ((d.GetDelayType()) == _GENERAL && !(d.GetGeneral()))
	{
		beg = d.GetUniformBegin();
		end = d.GetUniformEnd(); 
	}
	return *this;
}
//-------------------------------------------------------------------------------------------------------
Delay*     	UniForm::operator =(const Delay* d)
{
	if (d)
	{
		if ((d->GetDelayType()) == _GENERAL && !(d->GetGeneral()))
		{
			beg = d->GetUniformBegin();
			end = d->GetUniformEnd(); 
		}
	}
	return this;
}
//-------------------------------------------------------------------------------------------------------
bool       	UniForm::operator==(const Delay& d) 
{ 
	if ((d.GetDelayType()) == _GENERAL && !(d.GetGeneral())) 
		return (beg == d.GetUniformBegin() && end == d.GetUniformEnd());
	return false; 
}

//########################################################################################################
#endif
