#ifndef __Delay_h__
#define __Delay_h__

extern "C" {
#include <ctype.h>
#include <string.h>
#include <stdio.h>
}
#include "Types.h"
//###############################################################################################
class Delay
{
public :
	Delay() {};
	virtual ~Delay() {};

	//---------------------------------------------------------------------------------------
	//	Getten/Calculate
	//---------------------------------------------------------------------------------------
	virtual char*		GetGeneral()		const		{ return NULL;		}
	virtual double		GetUniformBegin() 	const		{ return 0; 		}
	virtual double		GetUniformEnd() 	const		{ return 0; 		}
	virtual Cell*		GetImmediate() 		const		{ return NULL; 		}
	virtual Cell*		GetExponential()	const		{ return NULL; 		}
	virtual Cell*		GetDeterministic()	const		{ return NULL; 		}
	virtual void		PrintPMF(FILE*)				{ ;			}

	virtual	CellVal*   	Calculate() 		    		= 0;	
	virtual	double   	CalculateFloat() 	    		= 0;	
	virtual Cell*		GetCell()		const   	= 0;	
	virtual int		GetDelayType()		const   	= 0;	
	virtual bool		IsPlaceDependent()			= 0;	

	//---------------------------------------------------------------------------------------
	//	Create
	//---------------------------------------------------------------------------------------
	virtual Delay*	   	Copy()          	const  		= 0; 
	virtual void		Clear()					= 0;

	//---------------------------------------------------------------------------------------
	//	Operatoren	
	//---------------------------------------------------------------------------------------
	virtual Delay*     operator =(const Delay*)			= 0;
	virtual Delay&     operator =(const Delay&)			= 0;
	virtual bool       operator==(const Delay&)			= 0;
	virtual bool       operator!=(const Delay&)			= 0;

	//---------------------------------------------------------------------------------------
	//	Error	
	//---------------------------------------------------------------------------------------
	void	DelayError(char* s)
	{
		cout <<"Error in class Delay : "<<s<<"\n";
		exit(DELAY_ERROR);
	}
};
//###############################################################################################
class Immediate : public Delay
{
	Cell*		imm;
public:
	Immediate();
	Immediate(const Cell*);
	Immediate(const Delay&);
	~Immediate() { Clear(); }

	//--------------------------------------------------------------------------------------
	//	Create/Clear
	//--------------------------------------------------------------------------------------
	Delay*     Copy() const;
	void       Clear();

	//--------------------------------------------------------------------------------------
	//	Operatoren	
	//--------------------------------------------------------------------------------------
	Delay*		operator =(const Delay*);
	Delay&		operator =(const Delay&);
	bool		operator==(const Delay&);
	bool		operator!=(const Delay&);

	//--------------------------------------------------------------------------------------
	//	Getten/Calculate
	//--------------------------------------------------------------------------------------
	Cell*		GetCell()          const;
	Cell*		GetImmediate()     const;
	int		GetDelayType()     const;
	bool		IsPlaceDependent();
	double		CalculateFloat();
	CellVal*	Calculate();
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class Exponential : public Delay
{
	Cell*		expo;
public:
	Exponential();
	Exponential(const Cell*);
	Exponential(const Delay&);
	~Exponential() { Clear(); };

	//--------------------------------------------------------------------------------------
	//	Create/Clear
	//--------------------------------------------------------------------------------------
	Delay*     Copy() const;
	void       Clear();

	//--------------------------------------------------------------------------------------
	//	Operatoren	
	//--------------------------------------------------------------------------------------
	Delay*     	operator =(const Delay*);
	Delay&		operator =(const Delay&);
	bool		operator==(const Delay&);
	bool		operator!=(const Delay&);

	//--------------------------------------------------------------------------------------
	//	Getten/Calculate
	//--------------------------------------------------------------------------------------
	CellVal*	Calculate();
	double		CalculateFloat();
	bool		IsPlaceDependent();
	Cell*     	GetCell()          const;
	Cell*     	GetExponential()   const;
	int      	GetDelayType()	   const;
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class Deterministic : public Delay
{
	Cell*		det;
public:
	Deterministic();
	Deterministic(const Cell*);
	Deterministic(const Delay&);
	~Deterministic() { Clear(); }

	//--------------------------------------------------------------------------------------
	//	Create/Clear
	//--------------------------------------------------------------------------------------
	Delay*     Copy() const;
	void       Clear();

	//--------------------------------------------------------------------------------------
	//	Operatoren	
	//--------------------------------------------------------------------------------------
	Delay*     	operator =(const Delay*);
	Delay&		operator =(const Delay&);
	bool		operator==(const Delay&);
	bool		operator!=(const Delay&);

	//--------------------------------------------------------------------------------------
	//	Getten/Calculate
	//--------------------------------------------------------------------------------------
	CellVal*	Calculate();
	double		CalculateFloat();
	bool		IsPlaceDependent();
	Cell*     	GetCell()          const;
	Cell*     	GetDeterministic() const;
	int      	GetDelayType()	   const;
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class UniForm : public Delay
{
	double		beg;
	double		end;
public:
	UniForm();
	UniForm(double, double);
	UniForm(const Delay&);
	~UniForm() { Clear(); }

	//--------------------------------------------------------------------------------------
	//	Create/Clear
	//--------------------------------------------------------------------------------------
	Delay*     Copy() const;
	void       Clear();

	//--------------------------------------------------------------------------------------
	//	Operatoren	
	//--------------------------------------------------------------------------------------
	Delay*		operator =(const Delay*);
	Delay&		operator =(const Delay&);
	bool		operator==(const Delay&);
	bool		operator!=(const Delay&);

	//--------------------------------------------------------------------------------------
	//	Getten/Calculate
	//--------------------------------------------------------------------------------------
	double		GetUniformBegin()  const;
	double		GetUniformEnd()    const;
	Cell*		GetCell()          const;
	int		GetDelayType()     const;
	void		PrintPMF(FILE*);
	bool		IsPlaceDependent();
	double		CalculateFloat();
	CellVal*	Calculate();
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class General : public Delay
{
	char*	pmf;
public:
	General();
	General(char*);
	General(const Delay&);
	~General() { Clear(); };

	//--------------------------------------------------------------------------------------
	//	Create/Clear
	//--------------------------------------------------------------------------------------
	Delay*     Copy() const;
	void       Clear();

	//--------------------------------------------------------------------------------------
	//	Operatoren	
	//--------------------------------------------------------------------------------------
	Delay*   operator =(const Delay*);
	Delay&   operator =(const Delay&);
	bool     operator==(const Delay&);
	bool     operator!=(const Delay&);

	//--------------------------------------------------------------------------------------
	//	Getten/Calculate
	//--------------------------------------------------------------------------------------
	CellVal*      	Calculate();
	double		CalculateFloat();
	bool		IsPlaceDependent();
	Cell*     	GetCell()       const;
	char*		GetGeneral()	const;
	int      	GetDelayType()  const;
	void		PrintPMF(FILE*);
};
//###############################################################################################
#endif
