#ifndef __ExamineRRG_cc__
#define __ExamineRRG_cc__

#include "inc_classes.h"
#include "inc_examine_rrg.h"

//----------------------------------------------------------------------------------------------------------------
//	externe Variablen
//----------------------------------------------------------------------------------------------------------------
extern RRGError*	RRGErr;
extern bool		SHOW_RRG;
extern bool		SHOW_SMC;

//----------------------------------------------------------------------------------------------------------------
//	externe Funktionen
//----------------------------------------------------------------------------------------------------------------
extern void		PrepareEnd();
extern void		ShowReachabilityGraph(Marking*);
extern void		ShowSMCs(ListOfSMCLists&);

//#################################################################################################################
//
//	Fehlerbehandlung
//
//#################################################################################################################
void	ExErr(char* s, int line)
{
	RRGErr = new RRGError(__FILE__, s, line);
	PrepareEnd();
}					 

//#################################################################################################################
//
//	haengt ein integer-list-element an den Anfang der uebergebenen Liste (struct int_list)
//
//#################################################################################################################
void	AddMarkingToList(int nr, int_list*& markings_of_this_SMC)
{
	int_list* new_member = new int_list;
	new_member->ref      = nr;
	new_member->next     = markings_of_this_SMC;
	markings_of_this_SMC = new_member;
}

//#################################################################################################################
//
//	ueberprueft ob ein integer-Wert bereits in der uebergebenen Liste (struct int_list) vorhanden ist
//
//#################################################################################################################
bool	MarkingIsInList(int nr, int_list*& markings_of_this_SMC)
{
	for (int_list* tmp = markings_of_this_SMC; tmp; tmp=tmp->next)
		if (tmp->ref == nr) return true;
	return false;
}

//#################################################################################################################
//
//	eine SMC durchnummerieren
//
//#################################################################################################################
void	RenumberSMC(Marking*& marking, int& m_counter, int det_or_gen)
{
	if (marking && marking->first)
	{
		for (state_transition* st=marking->first; st; st=st->nextS)
		{
			if (( (st->nextM)) 				  	&& 
			    ( (st->nextM->GetDetOrGenTransRef())) 	  	&&
			    ( (int(st->nextM->mtype)        == det_or_gen    )) &&
			    ( (int(st->nextM->status)       != _INITIAL      )) &&
			    ( (int(st->nextM->status)       != _R_VISITED    )) &&
			    ( (int(st->nextM->status)       != _R_LAST_OF_SCC)) &&
			    ( (st->nextM->GetDetOrGenTransRef()) == (marking->GetDetOrGenTransRef())))
			{
				st->nextM->SetNumber(m_counter++);
				switch (int(st->nextM->status))
				{
					case _LAST_OF_SCC : st->nextM->status = (0 | bits[_R_LAST_OF_SCC]); break;
					case _VISITED     : st->nextM->status = (0 | bits[_R_VISITED]);     break;
					default		  : break;
				}
				RenumberSMC(st->nextM, m_counter, det_or_gen);
			}
		}
	}
}
//#################################################################################################################
//
//	det. Markierungen neu nach der Reihenfolge in der MC nummerieren
//
//#################################################################################################################
void	RenumberDetOrGenMarkings(ListOfSMCLists& det_or_gen_ml, int& no_det_or_gen_m, int det_or_gen)
{
	if (!det_or_gen_ml.size) return;

	int renumbered_m = 0;

	for (ListElementOfSMCLists* smc = det_or_gen_ml.first; smc; smc=smc->next)
	{
		if (smc->lelement.det_or_gen_markings.size)
		{
			int m_counter = smc->lelement.det_or_gen_markings.first->plelement->GetNumber();
			int old_m_num = m_counter;
			for (MarkingPtrListElement* mp=smc->lelement.det_or_gen_markings.first; mp; mp=mp->next)
			{
				if ((int(mp->plelement->status) == _INITIAL))
				{
				        mp->plelement->SetNumber(m_counter++);
					RenumberSMC(mp->plelement, m_counter, det_or_gen);
				}
			}

			if ( (smc->lelement.det_or_gen_markings.size != (m_counter - old_m_num)) )
			{
				cout <<"size      = "<<smc->lelement.det_or_gen_markings.size	<<"\n";
				cout <<"old_m_num = "<<old_m_num				<<"\n";
				cout <<"m_counter = "<<m_counter				<<"\n";
				ExErr("couldn't number all det. markings of a SMC", __LINE__);
			}
			renumbered_m = renumbered_m + (m_counter - old_m_num);
		}
	}

	if (no_det_or_gen_m != renumbered_m)
		ExErr("couldn't number all det. markings of all SMCs", __LINE__);

	for (ListElementOfSMCLists* smc1 = det_or_gen_ml.first; smc1; smc1=smc1->next)
	{
		if (smc1->lelement.det_or_gen_markings.size)
		{
			for (MarkingPtrListElement* mp=smc1->lelement.det_or_gen_markings.first; mp; mp=mp->next)
			{
				switch (int(mp->plelement->status))
				{
					case _R_LAST_OF_SCC : mp->plelement->status = (0 | bits[_LAST_OF_SCC]); break;
					case _R_VISITED     : mp->plelement->status = (0 | bits[_VISITED]);     break;
					default		    : break;
				}
			}
		}
	}
}
//#################################################################################################################
//
//
//
//#################################################################################################################
bool	FoundNewSMCMember(Marking*& 			det_or_gen_m, 
			  state_transition*& 		to_nextM, 
			  int&				no_preempting_arcs,
			  int_list*& 			markings_of_this_SMC,
			  int				det_or_gen)
{
	if ((int(to_nextM->nextM->mtype) == _EXPONENTIAL))
	{
		//-------------------------------------------------------------------------------------------------
		//	In der naechste Markierung sind nur exp. Transitionen schaltfaehig.
		//	Folglich handelt es sich um einen aus der SMC herausfuehernden Uebergang.
		//-------------------------------------------------------------------------------------------------
		to_nextM->delay.delay_typ = (0 | bits[_PREEMP_EXP]); 
		no_preempting_arcs++;
		return false; 
	}

	if (int(to_nextM->nextM->mtype) != det_or_gen)
		ExErr("unexpected det- or gen-marking-type while creating SMC", __LINE__);

	//---------------------------------------------------------------------------------------------------------
	//	In der naechste Markierung ist eine det. Transition schaltfaehig. Folglich kann es sich um einen 
	//	Uebergang zu einem weitern Mitglied der betrachteten SMC, oder um einen Uebergang aus dieser SMC 
	//	handeln.
	//---------------------------------------------------------------------------------------------------------
	if ((to_nextM->nextM->GetDetOrGenTransRef()) && 
	    (to_nextM->nextM->GetDetOrGenTransRef() != det_or_gen_m->GetDetOrGenTransRef()) )
		return false;

	//---------------------------------------------------------------------------------------------------------
	//	Es handelt sich um eine Markierung, die zu dieser SMC gehoeren muss.
	//---------------------------------------------------------------------------------------------------------
	//-------------------------------------------------------------------------------------------------
	// 	Die gefundene Markierung wurde bereits zu dieser SMC zugeordnet. Es handelt sich also um 
	//	eine zyklische SMC.
	//-------------------------------------------------------------------------------------------------
	if (MarkingIsInList(to_nextM->nextM->GetNumber(), markings_of_this_SMC))
	{
		switch (int(to_nextM->nextM->status))
		{
			case	_COUNTED	:	ExErr("cyclic-SMC-error (counted)",__LINE__); break;
			case	_UNKNOWN	:	ExErr("cyclic-SMC-error (unknown)",__LINE__); break;
			default			:	break;
		}
		det_or_gen_m->status = (0 | bits[_LAST_OF_SCC]);
		return false;
	}
	//-------------------------------------------------------------------------------------------------
	// 	Die gefundene Markierung wurde noch nicht zu dieser SMC zugeordnet.
	//-------------------------------------------------------------------------------------------------
	else
	{
		AddMarkingToList(to_nextM->nextM->GetNumber(), markings_of_this_SMC);

		switch (int(to_nextM->nextM->status))
		{
			case	_LAST_OF_SCC	: return true;
			case	_UNKNOWN	: ExErr("try to make SMC with uncounted marking", __LINE__);
			default			: break;
		}
		to_nextM->nextM->status = (0 | bits[_VISITED]);
		return true;
	}
	return false;
}
//#################################################################################################################
//
//
//
//#################################################################################################################
void	CreateSMC(Marking*& det_or_gen_m, int& no_preempting_arcs, int_list*& smc_markings, int det_or_gen)
{
	if (!(det_or_gen_m->first)) 	   ExErr("marking with firable det/gen transition and no statetr.", __LINE__);
	if (!(det_or_gen_m->first->nextS)) return;

	for (state_transition* st=det_or_gen_m->first; st; st=st->nextS)
	{
		switch (int(st->delay.delay_typ))
		{
			case _EXPONENTIAL   : if (FoundNewSMCMember(det_or_gen_m, st, no_preempting_arcs, 
								    smc_markings, det_or_gen))
						 CreateSMC(st->nextM, no_preempting_arcs, smc_markings, det_or_gen);
					      break;
			case _DETERMINISTIC : break;
			case _GENERAL       : break;
			case _PREEMP_EXP    : break; 
			case _IMMEDIATE     : ExErr("unexpected imm-delay-type while creating SMC", __LINE__); break;
			case _PROBABILITY   : ExErr("unexpected pro-delay-type while creating SMC", __LINE__); break;
			case _UNKNOWN       : ExErr("unexpected unk-delay-type while creating SMC", __LINE__); break;
			case _MIXED         : ExErr("unexpected mix-delay-type while creating SMC", __LINE__); break;
			case _ABSORBING     : ExErr("unexpected abs-delay-type while creating SMC", __LINE__); break;
			default		    : ExErr("unexpected delay-type while creating SMC",     __LINE__); break;
		}
	}
}
//#################################################################################################################
//
//
//
//#################################################################################################################
void	MakeSMC(Marking*& det_or_gen_m, int& no_preempting_arcs, int det_or_gen)
{
	int_list* markings_of_this_SMC = new int_list;

	switch (int(det_or_gen_m->status))
	{
		case	_COUNTED	:	det_or_gen_m->status = (0 | bits[_INITIAL]);
						markings_of_this_SMC->ref = det_or_gen_m->GetNumber();
						CreateSMC(det_or_gen_m ,no_preempting_arcs, markings_of_this_SMC,
						          det_or_gen);
						break;

		case	_INITIAL	:	ExErr("found multiple initial marking",          __LINE__);
		case	_UNKNOWN	:	ExErr("try to make SMC with uncounted marking",  __LINE__);
		default			:	break;
	}
}

//#################################################################################################################
//
//	Bildet subordinated markow chains (SMCs) fuer alle det. bzw. gen. Markierungen
//	Dabei werden 
//
//	- allen Markierungen, die einen Eintritspunkt der betrachteten SMC darstellen, der 
//	  ***> status initial <*** zugeordnet,
//
//	- allen nicht exponentiellen, aus der betrachteten SMC herausfuehrenden Uebergaengen, der 
//	  ***> delay_type preempting_exp <*** zugeordnet,
//
//	- allen Markierungen, die auf eine Markierungen der gerade betrachteten SMC zurueckfuehren, der 
//	  ***> status last_of_scc <*** zugeordnet,  
//
//	- und allen Markierungen die Bestandteil einer SMC und dabei initial oder last_of_scc sind der 
//	  ***> status visited <*** zugeordnet.
//
//#################################################################################################################
void	MakeSMCs(ListOfSMCLists& list_of_det_or_gen_mls, int det_or_gen)
{
	if (!list_of_det_or_gen_mls.size) return;

	for (ListElementOfSMCLists* h=list_of_det_or_gen_mls.first; h; h=h->next)
	{
		if (h->lelement.det_or_gen_markings.size)
		{
			int no_preempting_arcs = 0;
			for (MarkingPtrListElement* h1=h->lelement.det_or_gen_markings.first; h1; h1=h1->next)
				 MakeSMC(h1->plelement, no_preempting_arcs, det_or_gen);

			h->lelement.SetNoOfPreemptingArcs(no_preempting_arcs);
			h->lelement.SetNoOfStates	 (h->lelement.det_or_gen_markings.size);
		}
	}
}
//#################################################################################################################
//
//	alle Markierungen nummerieren
//		1) 0 ..x1 exponentielle
//		2) x1..x2 deterministische
//		3) x2..x3 general
//		4) der Rest 
//
//#################################################################################################################
void	NumberMarkings(int& no_markings, marking_lists*& m_lists)
{
	int counter = 1;
	if (m_lists->exp_marking_list.size)
	{
		for (MarkingPtrListElement* em=m_lists->exp_marking_list.first; em; em=em->next)
			em->plelement->SetNumber(counter++);
	}
	if (m_lists->det_marking_list.size)
	{
		for (ListElementOfSMCLists* dl=m_lists->det_marking_list.first; dl; dl=dl->next)
		{
			if (dl->lelement.det_or_gen_markings.size)
			{
				dl->lelement.SetFirstStateNumber(counter);
				for (MarkingPtrListElement* dm=dl->lelement.det_or_gen_markings.first; dm; dm=dm->next)
					dm->plelement->SetNumber(counter++);
			}
		}
	}
	if (m_lists->gen_marking_list.size)
	{
		for (ListElementOfSMCLists* gl=m_lists->gen_marking_list.first; gl; gl=gl->next)
		{
			if (gl->lelement.det_or_gen_markings.size)
			{
				gl->lelement.SetFirstStateNumber(counter);
				for (MarkingPtrListElement* gm=gl->lelement.det_or_gen_markings.first; gm; gm=gm->next)
					gm->plelement->SetNumber(counter++);
			}
		}
	}
	if (m_lists->absorb_marking_list.size)
	{
		for (MarkingPtrListElement* am=m_lists->absorb_marking_list.first; am; am=am->next)
			am->plelement->SetNumber(counter++);
	}
	if (m_lists->imm_marking_list.size)
	{
		for (MarkingPtrListElement* im=m_lists->imm_marking_list.first; im; im=im->next)
			im->plelement->SetNumber(counter++);
	}
	if (m_lists->pro_marking_list.size)
	{
		for (MarkingPtrListElement* pm=m_lists->pro_marking_list.first; pm; pm=pm->next)
			pm->plelement->SetNumber(counter++);
	}
	if ((counter-1) != no_markings)	ExErr("number of numbered-markings != number of markings", __LINE__);
}
//#################################################################################################################
//
//
//
//#################################################################################################################
void	ExamineIntermediateMarking(Marking*& marking)
{
	int det_or_gen_tref = -1; 

	for (state_transition* st=marking->first; st; st=st->nextS)
	{
		bool found_det_or_gen_delay = false;

		for (state_transition* s=st->nextM->first; s && !found_det_or_gen_delay; s=s->nextS)
			if ((int(s->delay.delay_typ) == _DETERMINISTIC) || (int(s->delay.delay_typ) == _GENERAL))
				found_det_or_gen_delay = true;

		if (found_det_or_gen_delay) 
		{
		     	if (det_or_gen_tref < 0) det_or_gen_tref = st->nextM->GetDetOrGenTransRef();
			else 
				if (det_or_gen_tref != st->nextM->GetDetOrGenTransRef()) 
					ExErr("two det/gen transitions are fireable in one marking", __LINE__);
		}
	}
}
//#################################################################################################################
//
//	Ueberpruefen, ob nicht erlaubte Kombinationen von Uebergaengen vorhanden sind
//	(Beispielsweise mehrere deterministische Uebergaenge)
//
//#################################################################################################################
void	CheckAndSetDelayType(int& first_dt, int dt, int& first_tr, int tr)
{
	switch (first_dt)
	{
		case _IMMEDIATE     	: if (dt != _IMMEDIATE  ) ExErr("imm-xxx-statetrans-mix",__LINE__); break;
		case _PROBABILITY   	: if (dt != _PROBABILITY) ExErr("pro-xxx-statetrans-mix",__LINE__); break;
		case _GENERAL       	: if (dt != _EXPONENTIAL) ExErr("gen-xxx-statetrans-mix",__LINE__); break;
		case _DETERMINISTIC 	: if (dt != _EXPONENTIAL) ExErr("det-xxx-statetrans-mix",__LINE__); break;
		case _EXPONENTIAL   	: if (dt == _IMMEDIATE  ) ExErr("exp-imm-statetrans-mix",__LINE__); 
			     	     	  if (dt == _PROBABILITY) ExErr("exp-pro-statetrans-mix",__LINE__);
			     	    	  first_dt = dt;	     
				    	  first_tr = tr;
				    	  break;
		case _UNKNOWN	   	: ExErr("state-transition without delay",          __LINE__); 
		case _ABSORBING	   	: ExErr("unallowed absorbing-state-transition",    __LINE__); 
		case _PREEMP_EXP	: ExErr("unallowed preempting-state-transition",   __LINE__); 
		case _MIXED         	: ExErr("unallowed mixed-state-transition",        __LINE__); 
	}
}
//#################################################################################################################
//
//	- Ueberprufen von Uebergangskombinationen.
//	- Markierungen den Uebergaengen entsprechend in Listen einordnen.
//	- Markierungen den Uebergaengen entsprechend zaehlen.
//
//#################################################################################################################
void	CountMarkingsMakeLists(Marking*& marking, marking_count*& m_count, marking_lists*& m_lists)
{
  	if (!marking || (int(marking->status) == _COUNTED)) return;

	m_count->no_markings++;
	marking->status = (0 | bits[_COUNTED]);

  	if (!(marking->first))
	{
		m_count->no_absorb_markings++;
		m_count->no_tang_markings++;
		marking->mtype = (0 | bits[_ABSORBING]);
		m_lists->absorb_marking_list.Append(marking);
		return;
	}

	int  first_delay_type = int(marking->first->delay.delay_typ);
	int  first_trans_ref  = marking->first->delay.GetTransitionRef(); 

	if (marking->first->nextS)
	{
		for (state_transition* st=marking->first->nextS; st; st=st->nextS)
		{
			int delay_type = int(st->delay.delay_typ);
			int trans_ref  = st->delay.GetTransitionRef(); 

			if (delay_type == _UNKNOWN) ExErr("state-transition without delay" , __LINE__);
			CheckAndSetDelayType(first_delay_type, delay_type, first_trans_ref, trans_ref);
		}
	}

	bool saved = false;
	switch (first_delay_type)
	{
		case _DETERMINISTIC: m_count->no_det_markings++; 
				     m_count->no_tang_markings++; 
				     marking->mtype = (0 | bits[_DETERMINISTIC]);
				     for (ListElementOfSMCLists* h=m_lists->det_marking_list.first; h; h=h->next)
				     {
				     		if (h->lelement.GetTransitionRef() == first_trans_ref)
				     		{
				     			h->lelement.det_or_gen_markings.Append(marking);
				     			saved = true;
				     			break;
						}
				     }
				     if (!saved) ExErr("can't save deterministic marking", __LINE__);
				     break;
		case _GENERAL      : m_count->no_gen_markings++; 
				     m_count->no_tang_markings++; 
				     marking->mtype = (0 | bits[_GENERAL]);
				     for (ListElementOfSMCLists* h1=m_lists->gen_marking_list.first; h1; h1=h1->next)
				     {
				     		if (h1->lelement.GetTransitionRef() == first_trans_ref)
				     		{
				     			h1->lelement.det_or_gen_markings.Append(marking);
				     			saved = true;
				     			break;
						}
				     }
				     if (!saved) ExErr("can't save general marking", __LINE__);
				     break;
		case _EXPONENTIAL  : m_count->no_exp_markings++; 
				     m_count->no_tang_markings++; 
				     marking->mtype = (0 | bits[_EXPONENTIAL]);
				     m_lists->exp_marking_list.Append(marking);
				     break;
		case _IMMEDIATE    : m_count->no_imm_markings++; 		    
				     marking->mtype = (0 | bits[_IMMEDIATE]);
				     m_lists->imm_marking_list.Append(marking);
				     break;
		case _PROBABILITY  : m_count->no_pro_markings++; 		    
				     marking->mtype = (0 | bits[_PROBABILITY]);
				     ExamineIntermediateMarking(marking);   			            
				     m_lists->pro_marking_list.Append(marking);
				     break;
		case _UNKNOWN      : ExErr("state-transition without rate/delay",   __LINE__);
		case _ABSORBING	   : ExErr("unallowed absorbing-state-transition",  __LINE__); 
		case _PREEMP_EXP   : ExErr("unallowed preempting-state-transition", __LINE__); 
		case _MIXED	   : ExErr("unallowed mixed-state-transition",      __LINE__); 
	}

	for (state_transition* s=marking->first; s; s=s->nextS)
		CountMarkingsMakeLists(s->nextM, m_count, m_lists);
}
//#################################################################################################################
//
//	Erstellten RRG untersuchen und dabei :
//
//#################################################################################################################
void	InitCounter(marking_count*& m_count)
{
	m_count->no_det_markings	= 0;
	m_count->no_exp_markings	= 0;
	m_count->no_gen_markings	= 0;
	m_count->no_absorb_markings	= 0;
}
//#################################################################################################################
//
//	Erstellten RRG untersuchen und dabei :
//
//		- Markierungen typebezogen und gesamt zaehlen
//		- Markierungen typebezogen in Listen einordnen
//		- Markierungen durchnummerieren
//		- zahlenmaessigen konsistenz ueberpruefen 
//
//#################################################################################################################
void	ExamineRRG(Marking*& marking, marking_count*& m_count, marking_lists*& m_lists)
{
	//---------------------------------------------------------------------------------------------------------
	//	Markierungen zaehlen und in Listen einsortieren	
	//---------------------------------------------------------------------------------------------------------
	InitCounter(m_count);
	CountMarkingsMakeLists(marking, m_count, m_lists);

	//---------------------------------------------------------------------------------------------------------
	//	Markierunegn nummerieren	
	//---------------------------------------------------------------------------------------------------------
	NumberMarkings(m_count->no_markings, m_lists);

	//---------------------------------------------------------------------------------------------------------
	//	Debug-Ausgaben	
	//---------------------------------------------------------------------------------------------------------
	if (SHOW_RRG)
	{
	        cout <<"\n\n********************* rrg-statistiks *********************\n";
		cout <<"   no_det_markings     : "<<m_count->no_det_markings        <<"\n";
		cout <<" + no_exp_markings     : "<<m_count->no_exp_markings        <<"\n";
		cout <<" + no_gen_markings     : "<<m_count->no_gen_markings        <<"\n";
		cout <<" + no_imm_markings     : "<<m_count->no_imm_markings        <<"\n";
		cout <<" + no_absorb_markings  : "<<m_count->no_absorb_markings     <<"\n";
		cout <<  "----------------------------------------------------\n";
		cout <<" = no_tang_markings    : "<<m_count->no_tang_markings     <<"\n\n";
		cout <<" + no_pro_markings     : "<<m_count->no_pro_markings        <<"\n";
		cout <<" + no_imm_markings     : "<<m_count->no_imm_markings        <<"\n";
		cout <<  "----------------------------------------------------\n";
		cout <<" = no_markings         : "<<m_count->no_markings            <<"\n";
		cout <<"********************* rrg-statistiks *********************\n\n";

		cout <<"\n\n========================= REDUCED-REACHABILITY-GRAPH =========================\n\n";
		ShowReachabilityGraph(marking);
		cout <<"\n\n=========================         END OF RRG         =========================\n\n";
	}

	//---------------------------------------------------------------------------------------------------------
	//	zahelenmaessige Konsistenz ueberpruefen	
	//---------------------------------------------------------------------------------------------------------
	if (m_count->no_absorb_markings)
		ExErr("Found absorbing marking(s) ! No steady-state solution possible !", __LINE__);

	if ((m_count->no_det_markings + 
	     m_count->no_exp_markings + 
	     m_count->no_gen_markings + 
	     m_count->no_absorb_markings) != m_count->no_tang_markings)
		ExErr("Sum of det.- exp.- and gen.-markings != No. of tangible markings", __LINE__);

	if ((m_count->no_tang_markings + 
	     m_count->no_pro_markings  + 
	     m_count->no_imm_markings) != m_count->no_markings)
		ExErr("Sum of all kinds of markings != total sum of markings", __LINE__);

	//---------------------------------------------------------------------------------------------------------
	//	Markow-Ketten bilden und umnummerieren det det. Markierungen	
	//---------------------------------------------------------------------------------------------------------
	if (m_count->no_det_markings) 
	{ 
		MakeSMCs(m_lists->det_marking_list, int(_DETERMINISTIC));
		RenumberDetOrGenMarkings(m_lists->det_marking_list, m_count->no_det_markings, int(_DETERMINISTIC));
	}
	if (m_count->no_gen_markings) 
	{ 
		MakeSMCs(m_lists->gen_marking_list, int(_GENERAL));
		RenumberDetOrGenMarkings(m_lists->gen_marking_list, m_count->no_gen_markings, int(_GENERAL));
	}

	//---------------------------------------------------------------------------------------------------------
	//	Debug-Ausgaben	
	//---------------------------------------------------------------------------------------------------------
	if (SHOW_SMC)
	{
		cout <<"\n\n=======================         SMCs (det.)           ========================\n\n";
		ShowSMCs(m_lists->det_marking_list);
		cout <<"\n\n=======================         END OF SMCs           ========================\n\n";

		cout <<"\n\n=======================         SMCs (gen.)           ========================\n\n";
		ShowSMCs(m_lists->gen_marking_list);
		cout <<"\n\n=======================         END OF SMCs           ========================\n\n";

		cout <<"\n\n=======================  REDUCED-REACHABILITY-GRAPH   ========================\n";
		cout <<    "======================= (renumbered det/gen Markings) ========================\n\n";
		ShowReachabilityGraph(marking);
		cout <<"\n\n=======================          END OF RRG           ========================\n\n";
	}
}
//#################################################################################################################
#endif
