#ifndef __ExecErand_cc__
#define __ExecErand_cc__

//#############################################################################################################
//
//	Program:		rg
//
//	component:		ExecErand.cc
//
//	Parameters:		The complete path name of the EDSPN to be solved and epsilon
//
//	Purpose:		start a couple of rand processes on the local machine. 
//				Send all necessary information to them. 
//				Wait for all started rand processes to
//				finish their operations.
//
//	Project Director:	Reinhard German
//
//	Programmer:		Patrick Drescher
//
//	History:		Release 1: June 1995
//
//------------------------------------------------------------------------------------------------------------
//
//	Modified:		July 1997 by Frank Bayer
//	Purpose:		Adaptation to the C++-structure of Agnes 
//
//#############################################################################################################

extern "C" {
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <malloc.h>
#include <signal.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include "portab.h"
}

#ifdef SOL2
extern "C" {
#include <sys/filio.h>
}
#endif

// #include "graph_types.h"
// #include "csmc_types.h"

#include "inc_classes.h"
#include "inc_exec_erand.h"
#include "matrices_Q_delta.h"

//-------------------------------------------------------------------------------------------------------------
//
//	extern	var's and func's
//
//-------------------------------------------------------------------------------------------------------------
extern	RRGError*	RRGErr;			/* Fehler-Klasse			   		*/
extern	void		PrepareEnd();		/* Fehler-Funktion			   		*/
extern	char*		BinDir;			/* absolut path of the bin-directory	   		*/
extern	char*		TmpDir;			/* absolut path of the tmp-directory	   		*/
extern	bool		DEBUG;			/* flag=1 if debug infos have to be shown	   	*/
extern	bool		SHOW_ERAND;		/* flag=1 if ernad infos have to be shown   		*/
extern  unsigned char 	bits[];

#if defined(SUNOS4) 
extern  "C" { int	listen		(int,  int);						}
extern  "C" { int	socket		(int,  int, int);					}
extern  "C" { int	bzero 		(char*,int);						}
extern  "C" { int	bind		(int,  struct sockaddr*, int);				}
extern  "C" { int	accept		(int,  struct sockaddr*, int*);				}
extern  "C" { int	getsockname	(int,  struct sockaddr*, int*);				}
extern  "C" { int	gethostname	(char*,size_t);						}
extern  "C" { int	select 		(int,  fd_set*, fd_set*, fd_set*, struct timeval*);	}
#endif

#if defined(SOL2)
extern  "C" { int	gethostname	(char*,int);						}
#endif

#if defined(__linux__) || defined(LINUX) || defined (Linux)
extern  "C" { int  	fileno		(FILE*); 						}
extern  "C" { FILE*	fdopen		(int, const char*);					}
extern  "C" { char*	strdup		(const char*);						}
extern  "C" { int	gethostname     (char*, size_t);					}
#endif


//-------------------------------------------------------------------------------------------------------------
//
//	package-global-var's
//
//-------------------------------------------------------------------------------------------------------------
static int       no_run   = 0;   		/* number of started rand processes (global) */
static connect_t *con;				/* list of connections              (global) */
static int       serv_fd;			/* server socket                    (global) */
static int       port_no;			/* server port number               (global) */
       double	 ru_sum   = (double )0.0;	/* sum of rusage_time 		    (global) */

//###############################################################################################################
//
//      Fehlerbehandlung
//
//###############################################################################################################
void    ExecErandErr(char* s1, char* s2, int line)
{
	if (s2) RRGErr = new RRGError(__FILE__, s1, line, NULL, s2);
	else    RRGErr = new RRGError(__FILE__, s1, line);
	PrepareEnd();
}
//#############################################################################################################
//
//	Function:	FreeConnect
//			frees all connection dependent data
//
//	Function:	ChldHandler
//			release zombie children from proc-table
//
//#############################################################################################################
void	FreeConnect()    { close(serv_fd); free(con); } 
void 	ChldHandler(int) { int status; waitpid(-1,&status,WNOHANG); }

//#############################################################################################################
//
//	Function:	Fwrite
//			write a buffer to a file pointer
//				if the write fails partially, try again
//				if it fails completely, abort program execution
//
//	Arguments:	a file pointer
//			pointer to data buffer
//			length of data buffer
//
//	Result:		length of buffer written
//
//#############################################################################################################
static int Fwrite(FILE* fp, char* data, unsigned len)
{
	int rt;

again:
	errno = 0;

	if(unsigned((rt = write((int)fileno(fp),data,len))) != len )
	{
		if( rt >= 0 )
		{
			/* partial write */
			data += rt;
			len -= rt;
			goto again;
		}
		ExecErandErr("pipe broken; can't write to socket",NULL,__LINE__);
	}

	return rt;
}

//#############################################################################################################
//
//	Function:	InitConnection
//
//			allocate memory for connections with rand processes and for server
//
//	Parameter:	number of connections to allocate memory for (usually the number of transitions)
//
//	Result:		none
//
//#############################################################################################################
void	InitConnection(int amount)
{
	con = new connect_t[amount+1];	

	for( int i = 0; i < amount; i++ )
	{
		con[i].index = -1;
		con[i].fd = -1;
		con[i].fp = NULL;
	}

	if( (void *)signal(SIGCHLD,ChldHandler) == (void *)-1L )
		ExecErandErr("unable to install catcher for SIGCHLD",NULL,__LINE__);
}

//#############################################################################################################
//
//	Function:	CreateServer
//			create server socket and bind it to an address, get port
//			number of the connection, listen for calls
//
//	Arguments:	none
//
//	Result:		none
//
//#############################################################################################################
void	CreateServer()
{
	struct sockaddr_in server;
	int    length = sizeof(server);

	if( (serv_fd = socket(AF_INET,SOCK_STREAM,0)) < 0 )			// create server socket
		ExecErandErr("unable to create server socket",NULL,__LINE__);

	memset(&server,0,sizeof(server));
	server.sin_family = AF_INET;
	server.sin_port = 0;

	if( bind(serv_fd,(struct sockaddr *)&server,sizeof(server)) )		// bind server socket
		ExecErandErr("unable to bind socket to address",NULL,__LINE__);

	getsockname(serv_fd,(struct sockaddr *)&server,&length);		// retrieve port number

	if( length != sizeof(server) )
		ExecErandErr("unable to get server port number", NULL,__LINE__);

	port_no = ntohs(server.sin_port);

	if (DEBUG) cout <<"\t\t\t socket created: port_no = "<< port_no <<endl;

	if( listen(serv_fd, BACKLOG) )						// listen for incoming calls
		ExecErandErr("unable to listen for potential clients",NULL,__LINE__);
}

//#############################################################################################################
//
//	Function:	StartRand
//			fork process and execute a rand process on the local host
//
//	Arguments:	filename of the EDSPN, without extension,
//			index in names list of the started transition,
//			filename of the rand process to execute
//
//	Result:		none
//
//#############################################################################################################
void	StartRand(char* filename, char* local_host, int index, char* rand_path, double epsilon)
{
	char* port_str = NULL;
	char* prec_str = NULL;

   	switch( fork() )					
   	{
   		case -1: ExecErandErr("unable to fork erand process",NULL,__LINE__);
   		case  0:
      		{
			sprintf(port_str = new char[8], "%u", port_no);
			sprintf(prec_str = new char[16],"%g", epsilon);

			execl(rand_path, "erand", local_host, port_str, prec_str, filename, NULL);
 
			ExecErandErr("execl() failed",NULL,__LINE__); 		/* exec() must not return */
		}
		default: if (DEBUG) cout <<"\t\t\t successful fork() !"<<endl; break;
	}
	con[no_run].index = index;
	con[no_run].fd = -1;
	con[no_run].fp = NULL;
	no_run++;
}

//#############################################################################################################
//
//	Function:	WaitConnect
//			waits on the server socket for replies from a rand process,
//			it then accepts the connection and creates an individual link with that process
//
//	Arguments:	non
//
//	Result:		file pointer for a rand process, or NULL if the connection failed
//
//#############################################################################################################
FILE* WaitConnect(int no_gen_trans)
{
	struct	timeval 	t_out;
	struct 	sockaddr_in 	remote_addr;
		int 		remote_addrsize = sizeof(remote_addr);
		int		sel_ret;
		fd_set  	fd_rset;
		int 		fd;

	while( 1 )
	{
		FD_ZERO(&fd_rset);				// listen on server socket for a max. of 20 sec
		FD_SET(serv_fd, &fd_rset);
		t_out.tv_sec = 20L;
		t_out.tv_usec = 0L;

								// wait for rand process to connect 
		do
		{
			sel_ret = select(FD_SETSIZE,&fd_rset,NULL,NULL,&t_out);
		} while( sel_ret < 0 && errno == EINTR );

		if( sel_ret < 0 ) ExecErandErr("Select() failed",NULL,__LINE__);
		if(!sel_ret ) 	  return NULL;

		memset(&remote_addr,0,sizeof(remote_addr));	// connect with rand process
		do
		{
			fd = accept(serv_fd,(struct sockaddr *)&remote_addr,&remote_addrsize);
		} while( fd < 0 && errno == EINTR );

		if( fd < 0 )	
			ExecErandErr("Accept() failed",NULL,__LINE__);

		if (DEBUG) cout <<"\t\t\t connection accepted (fd = "<< fd <<")"<< endl;

		for( int i = 0; i < no_gen_trans; i++ )				// find index in host list
		{
			if( con[i].fd < 0 )
			{

				if (DEBUG) cout <<"\t\t\t running rand (i = "<< i <<")"<< endl;
	
				con[i].fd = fd;					// buffered input/output

				if( !(con[i].fp = fdopen(fd,"w")) )
					ExecErandErr("unable to re-open file pointer",NULL,__LINE__);

				return( con[i].fp );
			}
		}
	}
}
//#############################################################################################################
//
//	Function:	send_mdet
//			recursive help-function for SendMDET
//
//	Arguments:	a file pointer,
//			a marking of the SMC which have to be sended
//			first state number of the SMC
//			last state number of the SMC
//
//	Result:		none
//
//#############################################################################################################
void	send_mdet(FILE* fp, Marking*& marking, int first_state_no, int last_state_no)
{
	unsigned char	det_info = (unsigned char)0;
	unsigned 	state_nr = (unsigned)marking->GetNumber();

	if ((int(marking->mtype)  != _GENERAL))
	                ExecErandErr("non gen. marking in while sending data", NULL, __LINE__);

	if ((int(marking->status) != _INITIAL))
	{
	                marking->status = (0 | bits[_UNKNOWN]);
	}
	else
	{
		det_info = det_info | (unsigned char)2;
	}

	Fwrite(fp,(char*)&state_nr,sizeof(state_nr));
	Fwrite(fp,(char*)&det_info,sizeof(det_info));
	//-----------------------------------------------------------------------------------------------
	//      Weitere SMC-Mitglieder suchen und ggf. deren Daten senden
	//-----------------------------------------------------------------------------------------------
	for (state_transition* st=marking->first; st; st=st->nextS)
	{
		if (!(st->nextM))                             continue; 
		if ((int(st->nextM->mtype)  != _GENERAL	  ))  continue;
		if ((int(st->nextM->status) == _INITIAL   ))  continue;
		if ((int(st->nextM->status) == _UNKNOWN   ))  continue;
		if (st->nextM->GetNumber() > last_state_no)   continue; 
		if (st->nextM->GetNumber() < first_state_no)  continue;

		send_mdet(fp, st->nextM, first_state_no, last_state_no);
	}
}

//#############################################################################################################
//
//	Function:	SendMDET
//			sends basic information (Markov-Chain number) to the erand-process,
//			followed by the number of markings; 
//			the marking numbers must be in the same order as the
//			rows of the Q and delta matrices
//
//	Arguments:	a file pointer,
//			index of the SMC to be send
//			number of det. transitions (offset for number of the EMC & CONV files)
//
//	Result:		none
//
//#############################################################################################################
void	SendMDET(FILE* fp, SMCLists& smc, int index, int no_det_trans)
{

	if (DEBUG) cout <<"\t\t\t sending infos ... "<<endl;

	int	 chain_no   	 = no_det_trans + index + 1;			// no. of the EMC & CONV file
	int	 trans_ref  	 = smc.GetTransitionRef();
	int	 no_states  	 = smc.GetNoOfStates();
	int	 first_state_no  = smc.GetFirstStateNumber();
	int	 last_state_no   = first_state_no + no_states; 
	char* 	 trans_name 	 = TransitionNameRefList.GetElement(trans_ref).GetChar(); 
	unsigned len 	    	 = strlen(trans_name);

	Fwrite(fp,(char*)&chain_no, sizeof(chain_no));				// send number of Markov-Chain
	Fwrite(fp,(char*)&len,      sizeof(len));				// send length of name
	Fwrite(fp,trans_name,len);						// send name

	for(MarkingPtrListElement* mark=smc.det_or_gen_markings.first; mark; mark=mark->next) 
		if (mark->plelement && (int(mark->plelement->status) == _INITIAL)) 
			send_mdet(fp, mark->plelement, first_state_no, last_state_no); 

	unsigned state_nr = (unsigned )0;					// write end marker
	Fwrite(fp,(char*)&state_nr,sizeof(state_nr));

	if (DEBUG) cout <<"\t\t\t sending infos finished ! "<<endl;
}

//#############################################################################################################
//
//	Function:	send_q
//			recursive help-function for SendQ
//
//	Arguments:	a file pointer,
//			a marking of the SMC which have to be sended
//			first state number of the SMC
//			last state number of the SMC
//
//	Result:		none
//
//#############################################################################################################
void	send_q(FILE* fp, Marking*& marking, int first_state_no, int last_state_no)
{
	if ((int(marking->mtype)  != _GENERAL))
	                ExecErandErr("non gen. marking in while sending Q-mat.", NULL, __LINE__);
	if ((int(marking->status) != _INITIAL))
	                marking->status = (0 | bits[_Q_SENDED]);

	Q_row_header.neg_sum 		= -(marking->GetSumOfRates());		// write row header
	Q_row_header.row_index 		=   marking->GetNumber();
	Q_row_header.entries_in_row 	=   marking->GetNoOfExpPaths();

	Fwrite(fp,(char*)&Q_row_header,SIZEOF_Q_row_header);
     
	for (state_transition* s=marking->first; s; s=s->nextS)
	{
		if (int(s->delay.delay_typ) == _EXPONENTIAL)
		{
			Q_row_entry.rate 	 = s->delay.GetDelayVal();
			Q_row_entry.column_index = s->nextM->GetNumber();
			Fwrite(fp,(char*)&Q_row_entry,SIZEOF_Q_row_entry);
		}
	}
	//-----------------------------------------------------------------------------------------------
	//      Weitere SMC-Mitglieder suchen und ggf. deren Daten senden
	//-----------------------------------------------------------------------------------------------
	for (state_transition* st=marking->first; st; st=st->nextS)
	{
		if (!(st->nextM))                              continue; 
		if ((int(st->nextM->mtype)  != _GENERAL	   ))  continue;
		if ((int(st->nextM->status) == _INITIAL    ))  continue;
		if ((int(st->nextM->status) == _Q_SENDED   ))  continue;
		if (st->nextM->GetNumber() > last_state_no)    continue; 
		if (st->nextM->GetNumber() < first_state_no)   continue;

		send_q(fp, st->nextM, first_state_no, last_state_no);
	}
}

//#############################################################################################################
//
// 	Function:	SendQ
// 			sends the Q matrix of a general transition to a rand process;
// 			the row header includes the row index, negative row sum, and the number of columns
//
// 	Arguments:	a file pointer,
// 			index of the ESMC
//
// 	Result:		none
//
//#############################################################################################################
void	SendQ(FILE* fp, SMCLists& smc)
{
	int	 no_states  	 = smc.GetNoOfStates();
	int	 first_state_no  = smc.GetFirstStateNumber();
	int	 last_state_no   = first_state_no + no_states; 

	if (DEBUG) cout <<"\t\t\t sending q-matrix ..."<<endl;

	for(MarkingPtrListElement* mark=smc.det_or_gen_markings.first; mark; mark=mark->next) 
		if (mark->plelement && (int(mark->plelement->status) == _INITIAL)) 
			send_q(fp, mark->plelement, first_state_no, last_state_no); 

	if (DEBUG) cout <<"\t\t\t sending q-matrix finished !"<<endl;
}

//#############################################################################################################
//
//	Function:	send_delta
//			recursive help-function for SendDELTA
//
//	Arguments:	a file pointer,
//			a marking of the SMC which have to be sended
//			first state number of the SMC
//			last state number of the SMC
//
//	Result:		none
//
//#############################################################################################################
void	send_delta(FILE* fp, Marking*& marking, int first_state_no, int last_state_no)
{
	state_transition* gen_st = NULL;
	Marking*          next_m = NULL;

	if ((int(marking->mtype)  != _GENERAL))
	                ExecErandErr("non gen. marking in while sending Q-mat.", NULL, __LINE__);
	if ((int(marking->status) != _INITIAL))
	                marking->status = (0 | bits[_D_SENDED]);

	//-------------------------------------------------------------------------------------------------------
	//      Es sind mehrere Transition schaltfaehig. Folglich muss der gen. Uebergang gefunden werden
	//-------------------------------------------------------------------------------------------------------
	if (marking->first->nextS)
	{
		for (state_transition* g_st=marking->first; g_st; g_st=g_st->nextS)
		{
			if ((int(g_st->delay.delay_typ) == _GENERAL)) 
			{
				gen_st = g_st;
				break;
			}
		}

		if (!gen_st)
			ExecErandErr("SMC member without gen. statetransition", NULL, __LINE__);
	}
	//-------------------------------------------------------------------------------------------------------
	//      Es ist nur eine Transition schaltfaehig und diese muss eine generelle sein.
	//-------------------------------------------------------------------------------------------------------
	else
	{
		gen_st = marking->first;
                if ((int(gen_st->delay.delay_typ) != _GENERAL))
			ExecErandErr("SMC member without gen. statetransition", NULL, __LINE__);
	}

	if (!(next_m = gen_st->nextM))
		ExecErandErr("gen. statetransition without successor marking", NULL, __LINE__);


	//-------------------------------------------------------------------------------------------------------
	//      Sofern nach dem Schalten der gen. Transition eine zeitlose Transition schaltfaehig geworden ist
	//      oder eine Exclusive-Server-Transition aktiviert worden ist, muessen deren Uebergangswahrschein-
	//      lichkeiten in die delta-Matrix eingetragen werden.
	//      Andernfalls ist nach dem Schalten der gen. Transition weder eine zeitlose Transition schaltfaehig
	//      noch eine Exclusive-Server-Transition aktiviert worden. D.h., dass die Markierung
	//      mit der Nummer "next_m->GetNumber()" mit der Wahrscheinlichkeit "1.0" erreicht wird.
	//-------------------------------------------------------------------------------------------------------
	if ( (int(next_m->mtype) != _PROBABILITY) && (int(next_m->mtype) != _IMMEDIATE) )
	{
		delta_row_header.row_index 	= marking->GetNumber();
		delta_row_header.entries_in_row = 1; 
		Fwrite(fp,(char*)&delta_row_header,SIZEOF_delta_row_header);

		delta_row_entry.probability 	= 1.0; 
		delta_row_entry.column_index	= next_m->GetNumber();
		Fwrite(fp,(char*)&delta_row_entry,SIZEOF_delta_row_entry);
	}
	else
	{
		if (!(next_m->first))
			ExecErandErr("statetransition without successor markings", NULL, __LINE__);

		delta_row_header.row_index 	= marking->GetNumber();
		delta_row_header.entries_in_row = next_m->GetNoOfStateTrans(); 
		Fwrite(fp,(char*)&delta_row_header,SIZEOF_delta_row_header);

		int i = 0;
		for (state_transition* st=next_m->first; st; i++, st=st->nextS)
		{
			if (!(st->nextM))
				ExecErandErr("statetransition without successor marking", NULL, __LINE__);

			delta_row_entry.probability 	= st->delay.GetDelayVal();
			delta_row_entry.column_index	= st->nextM->GetNumber();
			Fwrite(fp,(char*)&delta_row_entry,SIZEOF_delta_row_entry);

		}
	}
	//-----------------------------------------------------------------------------------------------
	//      Weitere SMC-Mitglieder suchen und ggf. deren Daten senden
	//-----------------------------------------------------------------------------------------------
	for (state_transition* st=marking->first; st; st=st->nextS)
	{
		if (!(st->nextM))                              continue; 
		if ((int(st->nextM->mtype)  != _GENERAL	   ))  continue;
		if ((int(st->nextM->status) == _INITIAL    ))  continue;
		if ((int(st->nextM->status) == _D_SENDED   ))  continue;
		if (st->nextM->GetNumber() > last_state_no)    continue; 
		if (st->nextM->GetNumber() < first_state_no)   continue;

		send_delta(fp, st->nextM, first_state_no, last_state_no);
	}
}

//#############################################################################################################
//
//	Function:	SendDELTA
//			sends the delta matrix of a general transition to its rand process; 
//			each protocol element consists of a row header, containing the row index and 
//			the number of column entries in that row; it is followed by a number of column entries, 
//			each with column index and the value of that entry
//
//	Arguments:	a file pointer to a connection (socket or file),
//			index of the ESMC
//
//	Result:		none
//
//#############################################################################################################
void	SendDELTA(FILE* fp, SMCLists& smc)
{
	int	 no_states  	 = smc.GetNoOfStates();
	int	 first_state_no  = smc.GetFirstStateNumber();
	int	 last_state_no   = first_state_no + no_states; 

	if (DEBUG) cout <<"\t\t\t sending delta-matrix ..."<< endl;

	for(MarkingPtrListElement* mark=smc.det_or_gen_markings.first; mark; mark=mark->next) 
		if (mark->plelement && (int(mark->plelement->status) == _INITIAL)) 
			send_delta(fp, mark->plelement, first_state_no, last_state_no); 

	if (DEBUG) cout <<"\t\t\t sending delta-matrix finished !"<< endl;
}

//#############################################################################################################
//
//	Function:	MsgReceive
//			checks the open connections for incoming messages, the
//			messages of each connection are collected in a buffer and
//			printed when the process finishes ("ready."); the buffer is
//			scanned for keywords such as the run-time of that process,
//			the overall run-time is calculated
//
//	Arguments:	receive mode: check for a message, receive just one, or
//			receive all messages
//
//	Result:		none
//
//#############################################################################################################
void	MsgReceive(int mode, int no_gen_trans)
{
	fd_set	fdset;
	char*	msg;
	char 	msg_buf[256];
	short 	again = 0;
	short	ready = 1;
	short 	rand_abort;
	int 	read_ret;
	int 	select_ret;
	double 	ru_time;

	static	char**	output;


  	if( !output ) 
  	{ 
  		output = new char*[no_gen_trans]; 
  		for (int n=0; n<no_gen_trans; n++)
  			output[n] = NULL;
	}

	do										// receive messages
	{
		FD_ZERO(&fdset);							// create fd-readset
		for( int i = 0; i < no_gen_trans; i++ )
		{
			if( con[i].fd >= 0 )
			{
				FD_SET(con[i].fd, &fdset);
				ready = 0;
			}
		}
		if( ready ) return;							// all rands finished?

		do									// check pending messages
		{
			select_ret = select(FD_SETSIZE,&fdset,NULL,NULL,NULL);
		} while( select_ret < 0 && errno == EINTR );

		if( select_ret < 0 ) 
			ExecErandErr("message select failed",NULL,__LINE__);

		if( select_ret > 0 )							// check received messages
		{
			for( int j = 0; j < no_gen_trans; j++ )
			{
				if( con[j].fd >= 0 && FD_ISSET(con[j].fd,&fdset) )
				{
					FD_CLR(con[j].fd, &fdset);			// clear entry in read set
					msg = msg_buf - 1;				// read message by chars
					rand_abort = 0;

					do
					{
						if( (read_ret = read(con[j].fd,++msg,1)) < 0 )
							ExecErandErr("read on socket failed",NULL,__LINE__);
						if( !read_ret )
						{
							rand_abort = 1;
							break;
						}
					} while( *msg && *msg != '\n' && (msg - msg_buf) < 199 );

					*msg = '\n';
					*++msg = '\0';

					/* if (SHOW_ERAND) cout <<"\t\t\t msg_receive: msg_buf = "<< msg_buf << endl; */

					if( rand_abort || !strncmp(msg_buf,"ready.",6) )
					{
					  if (SHOW_ERAND) 
					  {
						fprintf(stdout,"\n");
						fprintf(stdout,"\t\t\t ------------------------------------------------------------------\n");
						fprintf(stdout,"\t\t\t -------------------- Erand process terminated ! ------------------\n");
						fprintf(stdout,"\t\t\t ------------------------------------------------------------------\n");
						fprintf(stdout,"\t\t\t Received messages from erand process:\n");
						fprintf(stdout,"\n");
						fputs(output[j],stdout);
						fprintf(stdout,"\t\t\t ------------------------------------------------------------------\n");
						fprintf(stdout,"\t\t\t ----------------- end of erand process information ---------------\n");
						fprintf(stdout,"\t\t\t ------------------------------------------------------------------\n");
						fprintf(stdout,"\n");
						fflush(stdout);
					  }

						fclose(con[j].fp);

						con[j].fp = NULL;
						con[j].fd = -1;
						
						free(output[j]);
						output[j] = NULL;

						if( mode == WAIT_ONE ) return;			// wait for more?
					}
					else
					{
						if( !strncmp(msg_buf,"Elapsed CPU time",16) )
						{
							sscanf(msg_buf + 18,"%lf", &ru_time);
							ru_sum += ru_time;
						}

						if( !output[j] )				// buffer messages
						{
							if( !(output[j] = strdup(msg_buf)) )
								ExecErandErr("unable to duplicate string",NULL,__LINE__);
						}
						else
						{
							char*  tmp = new char[(strlen(output[j]) + 1)];
							strcpy(tmp, output[j]);

							output[j] = NULL;
							output[j] = new char[(strlen(tmp) + strlen(msg_buf) + 1)];

							strcpy(output[j], tmp);
							strcat(output[j],msg_buf);

							delete[] tmp;
						}
					}
					again = 1;
				}
			}
		}
	} while( again );
}

//#############################################################################################################
//
//	Function:	ExecuteErandProcesses
//
//				- set-up server for rand process execution; 
//				- start one rand process per general transition; 
//				- wait for processes to respond and collect their replies
// 
//	Arguments:	model filename,
//			epsilon
// 
//	Result:          none
// 
//#############################################################################################################
void	 ExecuteErandProcesses (char*& 		netname, 
				double& 	epsilon, 
				int& 		no_gen_trans, 
				int& 		no_det_trans, 
				ListOfSMCLists& smcs)
{
	int   i		 = 0;
	FILE* fp         = NULL;
	char* filename   = NULL;
	char* rand_path  = NULL;
	char  local_host[30];

	if (gethostname (local_host, 29))
		ExecErandErr("can't get hostname",NULL,__LINE__);
/*
	if ( (!(local_host = getenv("HOST"))))					
		ExecErandErr("can't get environment variable HOST",NULL,__LINE__);
*/

	sprintf(filename =new char[(strlen(TmpDir) + strlen(netname) + 3)], "%s/%s",    TmpDir,netname);
	sprintf(rand_path=new char[(strlen(BinDir) + 8)],                   "%s/erand", BinDir);	

	//-----------------------------------------------------------------------------------------------------
	//	connection initialisierungen
	//-----------------------------------------------------------------------------------------------------
	if (DEBUG) cout <<endl<<"\t\t initializing connections for " << no_gen_trans <<" transitions ..."<< endl;

	InitConnection(no_gen_trans);					

	if (DEBUG) cout <<"\t\t initializing connections for " << no_gen_trans <<" transitions finished !"<< endl;
	//-----------------------------------------------------------------------------------------------------
	//	sockets establishen	
	//-----------------------------------------------------------------------------------------------------
	if (DEBUG) cout <<endl<<"\t\t establishing sockets ... "<<endl;

	CreateServer();						

	if (DEBUG) cout <<"\t\t establishing sockets finished ! "<<endl;
	//-----------------------------------------------------------------------------------------------------
	//	senden	
	//-----------------------------------------------------------------------------------------------------
	for(ListElementOfSMCLists* smc=smcs.first; i < no_gen_trans && smc; i++, smc=smc->next)	
	{
		//---------------------------------------------------------------------------------------------
		//	Erand-Prozesse starten	
		//---------------------------------------------------------------------------------------------
		if (DEBUG) cout <<endl<<"\t\t starting erand procs ..."<<endl;

		StartRand(filename, &(local_host[0]), i, rand_path, epsilon);	

		if ( (!(fp = WaitConnect(no_gen_trans))) )	
			ExecErandErr("unable to establish connection",NULL,__LINE__);

		if (DEBUG) cout <<"\t\t starting erand procs finished !"<<endl;
		//---------------------------------------------------------------------------------------------
		//	Infos und Matrizen senden	
		//---------------------------------------------------------------------------------------------
		if (DEBUG) cout <<endl<<"\t\t sending infos to erand proc ..."<<endl;

		SendMDET (fp, smc->lelement, i, no_det_trans);		
		SendQ    (fp, smc->lelement);			
		SendDELTA(fp, smc->lelement);		

		if (DEBUG) cout <<"\t\t sending infos to erand proc finished !"<<endl;
	}
	//-----------------------------------------------------------------------------------------------------
	//	Nachrichten empfangen	
	//-----------------------------------------------------------------------------------------------------
	if (DEBUG) cout <<endl<<"\t\t waiting for messages ..."<<endl;

	MsgReceive(WAIT_ONE, no_gen_trans);	

	if (DEBUG) cout <<"\t\t waiting for messages finished !"<<endl;
	//-----------------------------------------------------------------------------------------------------
	//	Nachrichten empfangen	
	//-----------------------------------------------------------------------------------------------------
	if (DEBUG) cout <<endl<<"\t\t freeing connections ..."<<endl;

	FreeConnect();			

	if (DEBUG) cout <<"\t\t freeing connections finished !"<<endl;

   fflush(stdout);
}
//#############################################################################################################
#endif
