#ifndef __ExecRand_cc__
#define __ExecRand_cc__

extern "C" {
#include <math.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdio.h>
#include <signal.h>
#include <errno.h>
#include <netdb.h>
#include <malloc.h>
#include <fcntl.h>
#include <setjmp.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include "portab.h"
#include "c_types.h"
}

#ifdef SOL2
extern "C" {
#include <sys/filio.h>
}
#endif

extern "C" {
#include <rpc/rpc.h>
#include "rusers.h"             /* <rpcsvc/rusers.h> not identical on all systems */
#include "rstat.h"              /* <rpcsvc/rstat.h> as well */
}

#include "inc_classes.h"
#include "inc_info.h"
#include "inc_exec_rand.h"

//---------------------------------------------------------------------------------------------------------------
//
//      extern var's
//
//---------------------------------------------------------------------------------------------------------------
extern  RRGError*               RRGErr;
extern  char**                  environ;
extern  bool                    DEBUG;
extern  bool                    SHOW_COMM;
extern  bool                    SHOW_INFOS;
extern  unsigned char           bits[];

extern  char*                   TimeNETRC;
extern  char*                   BinDir;
extern  char*                   TmpDir;


//---------------------------------------------------------------------------------------------------------------
//
//      extern functions 
//
//---------------------------------------------------------------------------------------------------------------
extern          void    PrepareEnd();
extern 		void 	ShowHostInfo(hostinfo*&);
extern 		void 	ShowRandInfo(randinfo*&, int);

extern  "C" {   char*	strdup		(const char*); 			   			}
extern  "C" {   int	strncasecmp	(const char*, const char*, size_t); 			}
extern  "C" {  	pid_t 	fork		(); 		    	    	    			}
extern  "C" {  	pid_t 	vfork		(); 		    	    	    			}
extern  "C" {   void	my_qsort	(void*, int, int);					}

#if defined(SUNOS4) 
extern  "C" { int       listen          (int,  int);                                            }
extern  "C" { int       socket          (int,  int, int);                                       }
extern  "C" { int       bzero           (char*,int);                                            }
extern  "C" { int       bind            (int,  struct sockaddr*, int);                          }
extern  "C" { int       accept          (int,  struct sockaddr*, int*);                         }
extern  "C" { int       getsockname     (int,  struct sockaddr*, int*);                         }
extern  "C" { int       gethostname     (char*,size_t);                                         }
extern  "C" { int       select          (int,  fd_set*, fd_set*, fd_set*, struct timeval*);	}
#endif

#if defined(SOL2)
extern  "C" { int       gethostname     (char*,int);                                         	}
#endif

#if defined(__linux__) || defined(LINUX) || defined (Linux)
extern  "C" {   int     gethostname     (char*, size_t);                        		}
extern  "C" {   int     fileno          (FILE*);                                		}
extern  "C" {   FILE*   fdopen          (int, const char*);                     		}
#endif


//###############################################################################################################
//
//	Fehlerbehandlung
//
//###############################################################################################################
void	ExecRandErr(char* s1, char* s2, int line)
{
	if (s2) RRGErr = new RRGError(__FILE__, s1, line, NULL, s2);
	else	RRGErr = new RRGError(__FILE__, s1, line);
	PrepareEnd();
}

/*
//###############################################################################################################
//
//	Vergleichsfunktion zum Sortieren der freien Rechner (Array sorted_hosts) nach ihrer Auslastung und
//      Prioritaet mittels qsort()
//
//###############################################################################################################
static int CompareHostinfos (register hostinfo** item1, register hostinfo** item2)
{
	register int 	value1 = ((*item1)->load + 256) / ((*item1)->priority + 1),
			value2 = ((*item2)->load + 256) / ((*item2)->priority + 1);

	return (value1 - value2);
}
*/

//###############################################################################################################
//
//	Senden von Datenpaketen (ueber eine Socket)	
//
//###############################################################################################################
void my_fwrite (char* data, unsigned length, FILE* filepointer)
{
	int rt;

nochmal:
	errno = 0;

	if (unsigned((rt = write ((int)fileno (filepointer), data, length))) != length)
	{
		if (rt >= 0)
		{
			data   += rt;
			length -= rt;
			goto nochmal;
		}
		ExecRandErr("can't write to socket!", NULL, __LINE__);
	}
}

//###############################################################################################################
//
//	Erstellen einer Socket, an der sich spaeter die rand-Prozesse melden. 
//	Die Portnummer wird dabei vom Kernel bestimmt.	
//
//###############################################################################################################
void	EstablishServerSocket(execinfo*& e_info)
{
	struct sockaddr_in server;
	int                length = sizeof (server);

	//------------------------------------------------------------------------------------------------------
	// 		Server-Socket oeffnen
	//------------------------------------------------------------------------------------------------------
	if ( (e_info->server_fd = socket (AF_INET, SOCK_STREAM, 0)) < 0 )	
		ExecRandErr("Can't create socket", NULL, __LINE__);

	memset (&server,0, sizeof (server));
	server.sin_family      = AF_INET;
	server.sin_port        = 0;
	server.sin_addr.s_addr = INADDR_ANY;

	//------------------------------------------------------------------------------------------------------
	// 		Server-Socket benennen 
	//------------------------------------------------------------------------------------------------------
	if ( bind(e_info->server_fd, (struct sockaddr*)&server, sizeof(server)) ) 
		ExecRandErr("Can't bind socket", NULL, __LINE__);

	//------------------------------------------------------------------------------------------------------
	// 		Portnummer ermitteln	
	//------------------------------------------------------------------------------------------------------
	getsockname(e_info->server_fd, (struct sockaddr *) &server, &length);

	if (length == sizeof (server))		e_info->portnumber = ntohs (server.sin_port);
	else					ExecRandErr("getsockname()", NULL, __LINE__);
	if (listen (e_info->server_fd, 5))	ExecRandErr("listen()",      NULL, __LINE__);
}

//###############################################################################################################
//
//	Liest aus dem ENV-File <filename> fuer den weiteren Verlauf wichtige Informationen
//
//###############################################################################################################
void ReadEnvironment(execinfo*& e_info, double&	start_distribution_at)
{
	FILE*		fp;
	char 		line[300];
    	char 		hostname[200];
	hostinfo*	info          	= NULL;
	hostinfo*	prev_hostinfo 	= NULL;
	unsigned 	priority 	= 1;

	enum 	{ 
			NONE = 0, 
			SUN, 
			DEC 
		} reading_hostlist = NONE;

	if (!(fp = fopen (TimeNETRC, "r"))) 
		ExecRandErr("can't open file for reading", TimeNETRC, __LINE__);

	do
  	{
  		//-----------------------------------------------------------------------------------------------
  		// 		Zeile aus env-fiel einlesen
  		//-----------------------------------------------------------------------------------------------
    		fgets (line, 298, fp);				
  		//-----------------------------------------------------------------------------------------------
  		//		Kommentarzeilen ignorieren	
  		//-----------------------------------------------------------------------------------------------
    		if (line[0]=='#'|| line[0]=='!' || line[0]=='\n' || line[0]=='\0') continue;
  		//-----------------------------------------------------------------------------------------------
  		//		Newline am Zeilenende entfernen
  		//-----------------------------------------------------------------------------------------------
    		if (line[strlen(line) - 1]=='\n') line[strlen (line) - 1] = '\0';  

		if (!strncasecmp (line, "HOSTLISTSTART", 13))
		{
#ifdef ultrix
			reading_hostlist = DEC;
#else
			reading_hostlist = SUN;
#endif
			continue;
		}
		if (!strncasecmp (line, "HOSTLISTEND", 11))
		{
			reading_hostlist = NONE;
			continue;
		}
	
#ifdef ultrix
		if (reading_hostlist == DEC)
#else
		if (reading_hostlist == SUN)
#endif
		{
  			//--------------------------------------------------------------------------------------
  			//		evntuell Prioritaet einlesen
  			//--------------------------------------------------------------------------------------
			if (!strncasecmp (line, "PRIORITY", 8))
			{
				sscanf (line + 9, "%d", &priority);
				continue;
			}
	
			info = new hostinfo;
  			//--------------------------------------------------------------------------------------
  			//		Host-Namen einlesen
  			//--------------------------------------------------------------------------------------
			strcpy(info->hostname   = new char[strlen(line) + 1], strdup(line)); 
			info->no_rusers 	= false;
      			info->running_processes = 0;
      			info->load              = 0;
      			info->free 		= false;
      			info->priority 		= priority;

      			if (prev_hostinfo) prev_hostinfo->next = info;
      			else 		   e_info->hostlist    = info;
	
      			prev_hostinfo = info;
		}
  		//----------------------------------------------------------------------------------------------
  		//		eventuell max_processes_per_host einlesen
  		//----------------------------------------------------------------------------------------------
		else if (!strncasecmp (line, "MAX PROCESSES PER HOST", 22))
		{
      			sscanf (line + 23, "%u", &(e_info->max_processes_per_host));			
      			if (e_info->max_processes_per_host < 1) e_info->max_processes_per_host = 1;
    		}
  		//----------------------------------------------------------------------------------------------
  		//		eventuell start_distribution_at einlesen
  		//----------------------------------------------------------------------------------------------
		else if (!strncasecmp (line, "START DISTRIBUTION AT", 21))
		{
			sscanf (line + 22, "%lf", &start_distribution_at);
			if (start_distribution_at < 0.0) start_distribution_at = 200.0;
		}
	}
  	while (!feof (fp));

  	if (info) info->next = NULL; 
  	else	  ExecRandErr("no hostlist found in file", TimeNETRC, __LINE__);


	//-----------------------------------------------------------------------------------------------------
  	// 	Testen, ob der Rechner, auf dem dieser Prozess gerade laeuft, in der Hostliste vermerkt ist
  	//-----------------------------------------------------------------------------------------------------
	if (gethostname (hostname, 199)) ExecRandErr("gethostname()", NULL, __LINE__);

	for (info = e_info->hostlist; info; info = info->next)
		if (!strcmp (hostname, info->hostname)) break;

	if (info) e_info->local_host = info;
	else
	{
		info = new hostinfo;
		strcpy(info->hostname     = new char[strlen(hostname) + 1], hostname);
		info->running_processes   = 0;
		info->free                = false;
		info->priority            = 0;
     		info->load                = 0;
		info->next                = e_info->hostlist;
		if (!e_info->hostlist) 	e_info->hostlist   = e_info->local_host = info;
		else	       		e_info->local_host = info;
	}
	e_info->local_hostname = e_info->local_host->hostname;

	fclose (fp);
}
//###############################################################################################################
//
//	Starten eines rand-Prozesses auf dem Rechner, der durch randinfo->host->hostname spezifiziert ist
//
//###############################################################################################################
void	StartRandProcess(randinfo*& info, execinfo*& e_info)
{
	char	     command_path[200];
	int	     stderr_backup_fd;

	static   char       port[10];
	static   char       epsilon_string[64];
	register hostinfo*  host = info->host;

	if (!port[0])	 sprintf(port, "%u", e_info->portnumber);
	sprintf(epsilon_string, "%g", e_info->epsilon);

	//-----------------------------------------------------------------------------------------------------
	//	auszufuehrendes Programm bestimmen 
	//-----------------------------------------------------------------------------------------------------
	sprintf(command_path, "%s/rand", BinDir);

	//-----------------------------------------------------------------------------------------------------
	//	Prozess starten
	//-----------------------------------------------------------------------------------------------------
#if !defined(ultrix) && !defined(__hpux) && !defined(SOL2) && !defined(__osf__)
	switch (vfork())
#else
	switch (fork())
#endif
	{
		case -1 : ExecRandErr("(v)fork", NULL, __LINE__);
		case  0 : 
		{
#ifndef ultrix
			  stderr_backup_fd = dup(2);
#endif
			  if (host != e_info->local_host)
			  {
				//----------------------------------------------------------------------------------
				//	"rand" remote starten
				//----------------------------------------------------------------------------------
				if (SHOW_COMM)
				{
			  		execl(RSH_CMD_PATH, RSH_CMD, host->hostname, "-n", "nice", "+4", command_path, 
			  		      e_info->local_hostname, port, e_info->model_name, 
			  		      "debug", TmpDir, epsilon_string, NULL);
				}
				else
				{
			  		execl(RSH_CMD_PATH, RSH_CMD, host->hostname, "-n", "nice", "+4", command_path, 
			  		      e_info->local_hostname, port, e_info->model_name, 
			  		      "nodebug", TmpDir, epsilon_string, NULL);
				}
				ExecRandErr("can't execute rsh process", NULL, __LINE__);
			   }
			  else
			  {
				//----------------------------------------------------------------------------------
				//	"rand" local starten
				//----------------------------------------------------------------------------------
				if (SHOW_COMM)
				{
			  		execl(command_path, "rand", e_info->local_hostname, port, e_info->model_name, 
			  		      "debug", TmpDir, epsilon_string, NULL);
				}
				else
				{
			  		execl(command_path, "rand", e_info->local_hostname, port, e_info->model_name, 
			  		      "nodebug", TmpDir, epsilon_string, NULL);
				}
				ExecRandErr("can't execute rsh process", NULL, __LINE__);
			  } 
		}
	}
	++host->running_processes;
	info->status = waiting_for_connection;

	if (DEBUG) 
	{
		cout <<"\t\t\t starting transient analysis on station ";
		cout <<host->hostname<<" with port-nr. "<<port<<endl;
	}
}
//###############################################################################################################
//
//	Auf die Verbindung zu einem rand-Prozess warten. Ist eine Verbindungsaufnahme erfolgt, werden die Felder
//	status, connection_fd und connection_fp, der zur Verbindung gehoerende randinfo-Struktur modifiziert. 
//
//###############################################################################################################
static randinfo* 	WaitForConnection(execinfo*& e_info)
{
	struct sockaddr_in 	connection_addr;
	struct hostent*		hostentry;
	struct timeval 		timeout;
	fd_set 	 		fd_readset;
	int 	 		fd; 
	int 	 		select_return;
	int 	 		connection_addrlen;

	while (1)
	{
		FD_ZERO (&fd_readset);
		FD_SET  (e_info->server_fd, &fd_readset);

		timeout.tv_sec  = 20L;
		timeout.tv_usec = 0L;

		//---------------------------------------------------------------------------------------------
		// 	Maximal 20 Sekunden auf die Verbindung zu einem rand-Prozess warten:
		//---------------------------------------------------------------------------------------------
		do
			select_return = select (FD_SETSIZE, &fd_readset, NULL, NULL, &timeout);
		while (select_return < 0 && errno == EINTR);

		if ( select_return < 0)	ExecRandErr("select", NULL, __LINE__);
		if (!select_return)	return NULL;

		//---------------------------------------------------------------------------------------------
		// 	Verbindung herstellen: 
		//---------------------------------------------------------------------------------------------
		memset(&connection_addr,0, sizeof (connection_addr));
		do
		{
			connection_addrlen = sizeof (connection_addr);
			fd = accept (e_info->server_fd, (struct sockaddr *) &connection_addr, &connection_addrlen);
		}
		while (fd < 0 && errno == EINTR);

		if (fd < 0) ExecRandErr("accept", NULL, __LINE__);

		//---------------------------------------------------------------------------------------------
		// 	Adresse des Rechners, auf welchem der rand-Prozesse laeuft, ermitteln: 
		//---------------------------------------------------------------------------------------------
		if (!(hostentry = gethostbyaddr ( (char *)&connection_addr.sin_addr.s_addr, 4, AF_INET)))
			ExecRandErr("gethostbyaddr", NULL, __LINE__);
		{
			register char *dot = strchr (hostentry->h_name, '.');
			if (dot) *dot = '\0';
		}

		for (int i=0; i < e_info->no_det_trans; i++)
		{
			register randinfo* info = &(e_info->rand_info_array[i]);

			if (info->status == waiting_for_connection && !strcmp (hostentry->h_name, info->host->hostname))
			{
				info->status = connected;
				info->connection_fd = fd;

				//-----------------------------------------------------------------------------
				// 	Buffermechanismen der C-Library an die Socketverbindung haengen:
				//-----------------------------------------------------------------------------
				if (!(info->connection_fp = fdopen (fd, "w")))
					ExecRandErr("fdopen", NULL, __LINE__);
				return info;
			}
		}

		//---------------------------------------------------------------------------------------------
		// 	Fehler: Verbindung zu einem unbekannten Prozess wurde hergestellt 
		//---------------------------------------------------------------------------------------------
		cout <<"unknown process on station "<<hostentry->h_name<<" has tried to connect.\n";
		close (fd);
	}
}
//###############################################################################################################
//
//	SMC-Daten fuer die Q-Matrix an einen Rand-Prozess senden
//
//###############################################################################################################
void	SendQMatrix(Marking*& marking, FILE* fp, int first_state_no, int last_state_no)
{
	int lastofscc = 0;
	my_fwrite((char *)&lastofscc, sizeof(int), fp);
	if (SHOW_COMM) cout <<"\t\t\t\t SendQMatrix : last_of_scc = "<<lastofscc<<endl;

	if ((int(marking->mtype)  != _DETERMINISTIC))
		ExecRandErr("non det. marking in SendQMatrix", NULL, __LINE__);

	if ((int(marking->status) != _INITIAL))
		marking->status = (0 | bits[_Q_SENDED]);

	if (!(marking->first->nextS))
	{
		//-----------------------------------------------------------------------------------------------
		//	Es gibt nur einen Uebergang und das muss ein deterministischer sein
		//-----------------------------------------------------------------------------------------------
		int    e_num     = 1;
		int    p_num     = 0;
		int    to_node   = marking->GetNumber(); 
		double rate      = 0.0;

		if (SHOW_COMM) 
		{
			cout <<"\t\t\t\t SendQMatrix : no_n_states = "<<e_num	<<endl;
			cout <<"\t\t\t\t SendQMatrix : state_no	   = "<<to_node	<<endl;
			cout <<"\t\t\t\t SendQMatrix : rate    	   = "<<rate   	<<endl;
			cout <<"\t\t\t\t SendQMatrix : no_p_states = "<<p_num	<<endl;
		}
		my_fwrite((char *)&e_num,    sizeof(int),    fp);
		my_fwrite((char *)&to_node,  sizeof(int),    fp);
		my_fwrite((char *)&rate,     sizeof(double), fp);
		my_fwrite((char *)&p_num,    sizeof(int),    fp);
	}
	else
	{
		//-----------------------------------------------------------------------------------------------
		//	Es gibt mehrere Uebergaenge, also eventuell auch Exponentielle
		//-----------------------------------------------------------------------------------------------
		double sum_of_rates = marking->GetSumOfRates(); 
		int    no_exp_paths = marking->GetNoOfExpPaths();
		edge*  sort_edges   = new edge[(no_exp_paths + 2)];

		//-----------------------------------------------------------------------------------------------
		//	Eintrag der Matrixdiagonalen
		//-----------------------------------------------------------------------------------------------
		sort_edges[0].to_node = marking->GetNumber();
		sort_edges[0].rate    = -sum_of_rates;

		//-----------------------------------------------------------------------------------------------
		//	sort_edges[] von vorne mit exponentiellen Uebergaengen und von hinten mit
		//	preempting (also herausfuehrenden) Uebergaengen fuellen.
		//-----------------------------------------------------------------------------------------------
		int j;
		int e_num = 1;
		int p_num = no_exp_paths;
		for (state_transition* st=marking->first; st; st=st->nextS)
		{
			switch (int(st->delay.delay_typ))
			{
				case _PREEMP_EXP 	: 	j = p_num--; 
						      		sort_edges[j].rate    = st->delay.GetDelayVal();
						      		break;
				case _EXPONENTIAL	: 	j = e_num++;
						      		sort_edges[j].to_node = st->nextM->GetNumber();
						      		sort_edges[j].rate    = st->delay.GetDelayVal();
						      		break;
				default		    	: 	break;
			}
		}
		//-----------------------------------------------------------------------------------------------
		//	sort_edges nach to_node (Markierungsnummer) sortieren
		//-----------------------------------------------------------------------------------------------
		my_qsort(sort_edges, e_num, sizeof(edge));

		//-----------------------------------------------------------------------------------------------
		//	Erst alle Mitglieder der SMC, also Eintraege der Q-Matrix senden (Position und Wert)
		//-----------------------------------------------------------------------------------------------
		my_fwrite((char *)&e_num, sizeof(int), fp);
		if (SHOW_COMM) 	cout <<"\t\t\t\t SendQMatrix : no_n_states = "<<e_num<<"\n";

		for (int i=0; i < e_num; i++)
		{
			if (SHOW_COMM) 
			{
				cout <<"\t\t\t\t SendQMatrix : state_no    = "<<sort_edges[i].to_node<<endl;
				cout <<"\t\t\t\t SendQMatrix : rate        = "<<sort_edges[i].rate   <<endl;
			}
			my_fwrite((char *)&(sort_edges[i].to_node), sizeof(int),    fp);
			my_fwrite((char *)&(sort_edges[i].rate),    sizeof(double), fp);
		}
		//-----------------------------------------------------------------------------------------------
		//	Dann alle aus der SMC herausfuehernden exponentiellen Raten (Werte) senden.
		//-----------------------------------------------------------------------------------------------
		int no_p_states = no_exp_paths - p_num;
		my_fwrite((char *)&no_p_states, sizeof(int), fp);
		if (SHOW_COMM) cout <<"\t\t\t\t SendQMatrix : no_p_states = "<<no_p_states<<endl;


		for (int n=(p_num + 1); n < (no_exp_paths + 1); n++)
		{
			if (SHOW_COMM) cout <<"\t\t\t\t SendQMatrix : rate        = "<<sort_edges[n].rate<<endl;
			my_fwrite((char *)&(sort_edges[n].rate), sizeof(double), fp);
		}

		//-----------------------------------------------------------------------------------------------
		//	ein bisschen aufraeumen
		//-----------------------------------------------------------------------------------------------
		delete[] sort_edges; 
		sort_edges = NULL;

		//-----------------------------------------------------------------------------------------------
		//	Weitere SMC-Mitglieder suchen und ggf. deren Daten senden
		//-----------------------------------------------------------------------------------------------
		for (state_transition* st1=marking->first; st1; st1=st1->nextS)
		{
			if (!(st1->nextM)) 					continue; 
			if ((int(st1->nextM->mtype)  != _DETERMINISTIC)) 	continue;
		    	if ((int(st1->nextM->status) == _INITIAL      )) 	continue;
		    	if ((int(st1->nextM->status) == _Q_SENDED     ))	continue;
			if (st1->nextM->GetNumber() > last_state_no)		continue; 
			if (st1->nextM->GetNumber() < first_state_no) 		continue;

			SendQMatrix(st1->nextM, fp, first_state_no, last_state_no);
		}
	}
}
//###############################################################################################################
//
//	SMC-Daten fuer die Delta-Matrix an einen Rand-Prozess senden	
//
//###############################################################################################################
void	SendDeltaMatrix(Marking*& marking, FILE* fp, int first_state_no, int last_state_no)
{
	state_transition* det_st    = NULL;
	Marking*	  next_m    = NULL;
	int		  initial_m = 1;

	my_fwrite((char *)&initial_m, sizeof(int), fp);

	if ((int(marking->mtype)  != _DETERMINISTIC)) 
		ExecRandErr("non det. marking in SendDeltaMatrix", NULL, __LINE__);

	if ((int(marking->status) != _INITIAL))
		marking->status = (0 | bits[_D_SENDED]);

	//-------------------------------------------------------------------------------------------------------
	//	Es sind mehrere Transition schaltfaehig. Folglich muss der det. Uebergang gefunden werden
	//-------------------------------------------------------------------------------------------------------
	if (marking->first->nextS)
	{
		for (state_transition* d_st=marking->first; d_st; d_st=d_st->nextS)
		{
			if ((int(d_st->delay.delay_typ) == _DETERMINISTIC)) 
			{
				det_st = d_st;
				break;
			}
		}

		if (!det_st)	
		    ExecRandErr("SMC member without det. statetransition", NULL, __LINE__);
	}
	//-------------------------------------------------------------------------------------------------------
	//	Es ist nur eine Transition schaltfaehig und diese muss eine deterministische sein.
	//-------------------------------------------------------------------------------------------------------
	else
	{
		det_st = marking->first;

		if ((int(det_st->delay.delay_typ) != _DETERMINISTIC))
		     ExecRandErr("SMC member without det. statetransition", NULL, __LINE__);
	}

	if (!(next_m = det_st->nextM))
	    ExecRandErr("Det. statetransition without successor marking", NULL, __LINE__);

	//-------------------------------------------------------------------------------------------------------
	//	Sofern nach dem Schalten der det. Transition eine zeitlose Transition schaltfaehig geworden ist
	//	oder eine Exclusive-Server-Transition aktiviert worden ist, muessen deren Uebergangswahrschein-
	//	lichkeiten in die delta-Matrix eingetragen werden.
	//-------------------------------------------------------------------------------------------------------
	if ( (int(next_m->mtype) == _PROBABILITY) || (int(next_m->mtype) == _IMMEDIATE) )
	{

		if (!(next_m->first))
		    ExecRandErr("statetransition without successor markings", NULL, __LINE__);

		int   no_of_stt  = next_m->GetNoOfStateTrans();
		edge* sort_edges = new edge[(no_of_stt + 1)];

		int i = 0;
		for (state_transition* st=next_m->first; st; i++, st=st->nextS)
		{
			if (!(st->nextM))
			    ExecRandErr("Statetransition without successor marking", NULL, __LINE__);

			//--------------------------------------------------------------------------------------
			//	delta-Matrixreiheneintraege sammeln
			//--------------------------------------------------------------------------------------
			sort_edges[i].to_node = st->nextM->GetNumber();
			sort_edges[i].rate    = st->delay.GetDelayVal();
		}
		//-----------------------------------------------------------------------------------------------
		//	delta-Matrixreiheneintraege sortieren
		//-----------------------------------------------------------------------------------------------
		my_qsort(sort_edges, no_of_stt, sizeof(edge));

		my_fwrite((char *)&no_of_stt, sizeof(int), fp);
		if (SHOW_COMM) cout <<"\t\t\t\t SendDeltaMatrix : no_n_states = "<<no_of_stt<<endl;

		//-----------------------------------------------------------------------------------------------
		//	delta-Matrixreihe senden
		//-----------------------------------------------------------------------------------------------
		for (i = 0; i < no_of_stt; i++)
		{
			if (SHOW_COMM) 
			{
				cout <<"\t\t\t\t SendDeltaMatrix : state_no    = "<<sort_edges[i].to_node<<endl;
				cout <<"\t\t\t\t SendDeltaMatrix : rate        = "<<sort_edges[i].rate   <<endl;
			}
			my_fwrite((char *)&(sort_edges[i].to_node), sizeof(int),    fp);
			my_fwrite((char *)&(sort_edges[i].rate),    sizeof(double), fp);
		}
	}
	else
	{
		//-----------------------------------------------------------------------------------------------
		//	Nach dem Schalten der det. Transition ist weder eine zeitlose Transition schaltfaehig
		//	noch eine Exclusive-Server-Transition aktiviert worden. D.h., dass die Markierung
		//	mit der Nummer "to_node" mit der Wahrscheinlichkeit "1.0" erreicht wird.
		//-----------------------------------------------------------------------------------------------
		int 	sz 	= 1;
		int     to_node = next_m->GetNumber(); 
		double	prob    = 1.0;

		if (SHOW_COMM) 
		{
			cout <<"\t\t\t\t SendDeltaMatrix : no_n_states = "<<sz		<<endl;
			cout <<"\t\t\t\t SendDeltaMatrix : state_no    = "<<to_node	<<endl;
			cout <<"\t\t\t\t SendDeltaMatrix : rate        = "<<prob   	<<endl;
		}
		my_fwrite((char *)&sz, 	    sizeof(int),    fp);
		my_fwrite((char *)&to_node, sizeof(int),    fp);
		my_fwrite((char *)&prob,    sizeof(double), fp);
	}
	//-----------------------------------------------------------------------------------------------
	//	Weitere SMC-Mitglieder suchen und ggf. deren delta-Matrix-Daten senden
	//-----------------------------------------------------------------------------------------------
	if (marking->first)
	{
		for (state_transition* st1=marking->first; st1; st1=st1->nextS)
		{
			if (!(st1->nextM)) 					continue; 
			if ((int(st1->nextM->mtype)  != _DETERMINISTIC)) 	continue;
		    	if ((int(st1->nextM->status) == _INITIAL      )) 	continue;
		    	if ((int(st1->nextM->status) == _D_SENDED     ))	continue;
			if (st1->nextM->GetNumber() > last_state_no)		continue;
			if (st1->nextM->GetNumber() < first_state_no) 		continue;

			SendDeltaMatrix(st1->nextM, fp, first_state_no, last_state_no);
		}
	}
}

//###############################################################################################################
//
//	SMC-Daten an einen Rand-Prozess senden
//
//###############################################################################################################
void	SendDataToRand(FILE* fp, SMCLists*& smcs, int& class_id)
{
	if (!smcs) 				 ExecRandErr("SMC is NULL-pointer",     NULL, __LINE__);
	if (!(smcs->det_or_gen_markings.size))	 ExecRandErr("SMC is empty", 	        NULL, __LINE__);
	if (!(smcs->GetDetOrGenDelay()) )	 ExecRandErr("no tau defined in SMC",   NULL, __LINE__);
	if (((smcs->GetFirstStateNumber())==-1)) ExecRandErr("first state number is -1",NULL, __LINE__);

	int  	no_preempting_arcs	= smcs->GetNoOfPreemptingArcs();
	int  	first_state_no		= smcs->GetFirstStateNumber();
	int  	no_states		= smcs->GetNoOfStates();
	int	last_state_no		= first_state_no + no_states;
	double  tau			= smcs->GetDetOrGenDelay();
	int  	no_isom_csmcs  		= 1;
	int  	no_tau_csmcs  		= 1;
	int  	no_total_csmcs  	= 1;
	int 	md_tau    		= 0;
	int  	branch    		= 0;
	int 	acyclic   		= 0;

	if (SHOW_COMM)
	{
		cout <<"\t\t\t SendDataToRand : no_preempting_arcs = "<<no_preempting_arcs	<<endl;
		cout <<"\t\t\t SendDataToRand : first_state_no     = "<<first_state_no		<<endl;
		cout <<"\t\t\t SendDataToRand : no_states          = "<<no_states		<<endl;
		cout <<"\t\t\t SendDataToRand : class_id           = "<<class_id		<<endl;
		cout <<"\t\t\t SendDataToRand : tau                = "<<tau			<<endl;
		cout <<"\t\t\t SendDataToRand : md_tau             = "<<md_tau 			<<endl;
		cout <<"\t\t\t SendDataToRand : branch             = "<<branch 			<<endl;
		cout <<"\t\t\t SendDataToRand : acyclic            = "<<acyclic 		<<endl;
		cout <<"\t\t\t SendDataToRand : no_tau_csmcs       = "<<no_tau_csmcs 		<<endl;
		cout <<"\t\t\t SendDataToRand : no_total_csmcs     = "<<no_total_csmcs 		<<endl;
		cout <<endl;
	}
	//-------------------------------------------------------------------------------------------------------
	//	ID fuer die Nummerierung der CONV- und ENV-Files
	//-------------------------------------------------------------------------------------------------------
	my_fwrite((char *)&class_id, sizeof(int), fp);

	//-------------------------------------------------------------------------------------------------------
	//	Anzahl der Zustaende	
	//-------------------------------------------------------------------------------------------------------
	my_fwrite((char *)&no_states, sizeof(int), fp);

	//-------------------------------------------------------------------------------------------------------
	//	Nummer des ersten Zustandes
	//-------------------------------------------------------------------------------------------------------
	my_fwrite((char *)&first_state_no, sizeof(int), fp);

	//-------------------------------------------------------------------------------------------------------
	//	Anzahl der aus der MC herausfuehrenden Uebergaenge
	//-------------------------------------------------------------------------------------------------------
	my_fwrite((char *)&no_preempting_arcs, sizeof(int), fp);

	//-------------------------------------------------------------------------------------------------------
	//	Anzahl aller CSMC's dieser Klasse ( = 1, da nur die zu einer det. Transition zugeordnete SMC
	//						 betrachtet wird)
	//	frueher : no_total_csmcs
	//-------------------------------------------------------------------------------------------------------
	my_fwrite((char *)&no_total_csmcs, sizeof(int), fp);

	//-------------------------------------------------------------------------------------------------------
	//	Klasse wird auf zyklisch gesetzt
	//-------------------------------------------------------------------------------------------------------
	my_fwrite((char *)&acyclic, sizeof(int), fp);

	//-------------------------------------------------------------------------------------------------------
	//	Klasse wird auf nicht branch gesetzt
	//-------------------------------------------------------------------------------------------------------
	my_fwrite((char *)&branch, sizeof(int), fp);

	//-------------------------------------------------------------------------------------------------------
	//	Q-Matrix schreiben	
	//-------------------------------------------------------------------------------------------------------
	if (DEBUG) cout <<"\t\t\t sending q-matrix ..."<<endl;

	for (MarkingPtrListElement* smc=smcs->det_or_gen_markings.first; smc; smc=smc->next)
	{
		if (smc->plelement && (int(smc->plelement->status) == _INITIAL))
		{
			SendQMatrix(smc->plelement, fp, first_state_no, last_state_no);
		}
	}

	if (DEBUG) cout <<"\t\t\t sending q-matrix finished !"<<endl<<endl;
	//-------------------------------------------------------------------------------------------------------
	//	no_tau_csmcs (=1) wird nur gebraucht wenn auf Isomorphieklassen geachtet wird	
	//-------------------------------------------------------------------------------------------------------
	my_fwrite((char *)&no_tau_csmcs,  sizeof(int), fp);

	//-------------------------------------------------------------------------------------------------------
	//	no_isom_csmcs (=1) 
	//-------------------------------------------------------------------------------------------------------
	my_fwrite((char *)&no_isom_csmcs, sizeof(int), fp);

	//-------------------------------------------------------------------------------------------------------
	//	det. Schaltzeit tau uebergeben
	//-------------------------------------------------------------------------------------------------------
	my_fwrite((char *)&tau, sizeof(double), fp);

	//-------------------------------------------------------------------------------------------------------
	//	marking dependent tau wird auf false gesetzt	
	//-------------------------------------------------------------------------------------------------------
	my_fwrite((char *)&md_tau, sizeof(int), fp);

	//-------------------------------------------------------------------------------------------------------
	//	det. Schaltzeit tau nochmals aus historischen Gruenden uebergeben
	//-------------------------------------------------------------------------------------------------------
	my_fwrite((char *)&tau, sizeof(double), fp);

	//-------------------------------------------------------------------------------------------------------
	//	delta-Matrix senden
	//-------------------------------------------------------------------------------------------------------
	if (DEBUG) cout <<"\t\t\t sending delta-matrix ..."<<endl;

	for (MarkingPtrListElement* smc1=smcs->det_or_gen_markings.first; smc1; smc1=smc1->next)
	{
		if (smc1->plelement && (int(smc1->plelement->status) == _INITIAL))
		{
			SendDeltaMatrix(smc1->plelement, fp, first_state_no, last_state_no);
		}
	}

	if (DEBUG) cout <<"\t\t\t sending delta-matrix finished !"<<endl<<endl;
	//-------------------------------------------------------------------------------------------------------
	//	preempting_exp-info's senden
	//-------------------------------------------------------------------------------------------------------
	if (DEBUG) cout <<"\t\t\t sending preempting_exp-infos ..."<<endl;

	for (MarkingPtrListElement* smc2=smcs->det_or_gen_markings.first; smc2; smc2=smc2->next)
	{
		//-----------------------------------------------------------------------------------------------
		//	Folgemarkierungsnummern von preempting exp. Uebergaengen senden.
		//-----------------------------------------------------------------------------------------------
		if (smc2->plelement && smc2->plelement->first)
		{
			for (state_transition* st=smc2->plelement->first; st; st=st->nextS)
			{
				if ((int(st->delay.delay_typ) != _PREEMP_EXP)) 
					continue;
				if (!(st->nextM)) 
					ExecRandErr("Exp. stt without successor marking", NULL, __LINE__);

				int  mark_no = st->nextM->GetNumber(); 
				my_fwrite((char *)&mark_no, sizeof(int), fp);
			}
		}
	}
	if (DEBUG) cout <<"\t\t\t sending preempting_exp-infos finished !"<<endl;
}

//###############################################################################################################
//
//	Lesen einer Nachricht (Bildschirmausgabe) eines rand-Prozesses.
//	Die Nachrichten eines rand-Prozesses werden gesammelt ausgegeben, sobald der Prozess terminiert ist. 
//	Je nach mode wartet die Funktion evtl. bis ein oder alle rand-Prozesse terminiert sind.
//
//###############################################################################################################
void	ReceiveMessages(ReceiveMode mode, execinfo*& e_info, unsigned& terminated_processes)
{
	static char **outputs;     	 // Array von Strings, zum speichern der Ausgaben jedes rand-Prozesses 
	fd_set 	    fd_readset;		 // Menge von Read-Filedescriptoren fuer select 		      
	char 	    message_buffer[200]; 
	unsigned    running_processes = 0;
	unsigned    kb_per_sec_sum    = 0;
	char*	    message;
	bool 	    ready; 
	bool	    once_again; 
	bool	    rand_abgebrochen;
	int 	    select_return; 
	int	    read_return;
	int	    i;

	if (!outputs)
		if (!(outputs = (char **)calloc (e_info->no_det_trans+1, sizeof (char *)))) OUT_OF_MEMORY;

	do
	{
		//---------------------------------------------------------------------------------------------
		// 	Filedescriptor readset erstellen: 
		//---------------------------------------------------------------------------------------------
		FD_ZERO (&fd_readset);
		for (ready = true, i = 0; i < e_info->no_det_trans; i++)
		{
			if (e_info->rand_info_array[i].status == running)
			{
				FD_SET (e_info->rand_info_array[i].connection_fd, &fd_readset);
				ready = false;
			}
		}
		//---------------------------------------------------------------------------------------------
		// 	Alle rand-Prozesse fertig
		//---------------------------------------------------------------------------------------------
		if (ready) return; 

		once_again = false;

		//---------------------------------------------------------------------------------------------
		// 	Auf Nachrichten der rand-Prozesse warten: 
		//---------------------------------------------------------------------------------------------
		do
		{
			select_return = select (FD_SETSIZE, &fd_readset, NULL, NULL, NULL);
		}
		while (select_return < 0 && errno == EINTR);

		if (select_return < 0)	ExecRandErr("select", NULL, __LINE__);
		if (select_return > 0)
		{
			for (i = 0; i < e_info->no_det_trans; i++)
			{
				register randinfo* info = &e_info->rand_info_array[i];

				if (info->status == running && FD_ISSET (info->connection_fd, &fd_readset))
				{
					//--------------------------------------------------------------------
					// 	Von rand-Prozess i ist eine Nachricht da 
					//--------------------------------------------------------------------
					message 	 = message_buffer - 1;
					rand_abgebrochen = false;

					do
					{
						if ((read_return = read (info->connection_fd, ++message, 1)) < 0)
							ExecRandErr("read on socket after select",NULL,__LINE__);
						else 	if (!read_return) { rand_abgebrochen = true; break; }
					}
					while (*message && *message != '\n' && message - message_buffer < 199);

					*message   = '\n';
					*++message = '\0';

					if (rand_abgebrochen || !strncmp (message_buffer, "ready.", 6))
					{
						//--------------------------------------------------------------------
						// rand-Prozess i ist fertig, alle gesammelten Nachrichten ausgeben: 
						//--------------------------------------------------------------------
						if (DEBUG)
						{
							if (rand_abgebrochen && e_info->distributed_execution)
							{
								cout <<"\t\t\t transient analysis of isomorphism class failed!";
								cout <<endl;
								cout <<"\t\t\t trying again ..."<<endl;
							}
							else
							{
								cout <<"\t\t\t transient analysis of isomorphism class terminated!";
								cout <<endl;
								if (outputs[i]) cout <<outputs[i]<<endl;
							}
						}

						//--------------------------------------------------------------------
						// 	Socket-Verbindung schliessen: 
						//--------------------------------------------------------------------
						fclose (info->connection_fp);

						info->connection_fp = (FILE *)0;
						info->connection_fd = 0;
						--info->host->running_processes;
						info->host          = (hostinfo *)0;

						if (rand_abgebrochen && e_info->distributed_execution)
							info->status = no_process;
						else 	info->status = terminated;

						++terminated_processes;
						--running_processes;
						kb_per_sec_sum -= info->kb_per_sec;

						free (outputs[i]);
						outputs[i] = (char *)0;

						if (mode == wait_for_one) return;
					}	
					else
					{
						if (!strncmp (message_buffer, "Elapsed CPU time", 16))
						{
							//----------------------------------------------------------
							// 	Summe der CPU-Zeiten korrigieren: 
							//----------------------------------------------------------
							double rt1, rt2, rt3;
							sscanf (message_buffer + 17, "%lf:%lf:%lf", &rt1, &rt2, &rt3);
							e_info->sum_of_rusage_times += rt3 + 60 * (rt2 + 60 * rt1);

							//----------------------------------------------------------
							// 	Die Nachricht des rand-Prozesses bis zu seiner 
							//	Beendigung speichern: 
							//----------------------------------------------------------
							if (!outputs[i])
							{
								if (!(outputs[i] = strdup (message_buffer))) 
									OUT_OF_MEMORY;
							}
							else
							{
								if (!(outputs[i] = (char *)realloc (outputs[i], (strlen(outputs[i]) + strlen(message_buffer) + 1))) ) 
									OUT_OF_MEMORY;
								strcat (outputs[i], message_buffer);
							}
						}
						once_again = true;

					}	// end : else (if (rand_abgebrochen || ...)
				}		// end : if (info->status == running && ...
			}			// end : for (i = 1; i < e_info->no_det_trans; ++i)
		}				// end : if (select_return > 0) 	
	}
	while (once_again);
}
//###############################################################################################################
//
//	Bestimmung des naechsten freien Rechners aus der Hostliste.
//	Frei bedeutet, dass niemand an der Console eingeloggt ist.
//
//###############################################################################################################
static hostinfo* 	NextFreeHost(hostinfo*& hostlist, unsigned& free_hosts)
{
	unsigned j;
	register hostinfo*	host;
	static   hostinfo*	next_host = (hostinfo *)-1;

	if ((void *)next_host == (void *)-1)
		next_host = hostlist;

	if (!next_host) return NULL;

	for (host = next_host; host; host = host->next)
	{
		utmpidlearr 		  rusersinfo;
		register struct utmpidle* array;
		bool 			  found = false;

		if (!host->no_rusers)
		{
			found = false;
			// found = rusers2 (host->hostname, &rusersinfo)==0 ? true : false;
			if (found)
			{
				//-------------------------------------------------------------------------------
				// 	Ein Rechner gilt als frei, wenn niemand an der Konsole eingeloggt ist 
				//-------------------------------------------------------------------------------
				array = rusersinfo.utmpidlearr_val;
				for (j = 0; j < rusersinfo.utmpidlearr_len; j++)
				{
					if (!strncmp (array[j].ui_utmp.ut_line, "console", 8))
					{
						found = false;
						break;
					}
				}
			}
		}
		else found = true;

		if (found)
		{
			host->free = true;
			++free_hosts;
			next_host = host->next;
			return host;
		}
	}
	next_host = NULL;
	return NULL;
}

//###############################################################################################################
//
//	Sortieren des Arrays sorted_hosts nach Auslastung und Prioritaet der Rechner
//
//###############################################################################################################
static void SortHostsByLoad(hostinfo*& hostlist, hostinfo**& sorted_hosts, unsigned& free_hosts)
{
  	register hostinfo*  host;
  	register hostinfo** host_in_array;

	if (!(sorted_hosts = (hostinfo **)malloc (free_hosts * sizeof (hostinfo *))))
		OUT_OF_MEMORY;

	host_in_array = sorted_hosts;
	for (host = hostlist; host; host = host->next)
		if (host->free) *host_in_array++ = host;

	if (host_in_array - sorted_hosts != (int)free_hosts)
		ExecRandErr("free_hosts incorrect", NULL, __LINE__);

	//-------------------------------------------------------------------------------------------------------
	// 	Rechner nach ihrer Prioritaet und ihrer Auslastung sortieren
	//-------------------------------------------------------------------------------------------------------
//	qsort (sorted_hosts, free_hosts, sizeof (hostinfo *), CompareHostinfos);
}

//###############################################################################################################
//
//	Testet, ob alle rand-Prozesse aktiviert wurden, d.h. entweder laufen oder bereits fertig sind	
//
//###############################################################################################################
static bool AllRandsActive (execinfo*& e_info)
{
	for (int i = 0; i <= e_info->no_det_trans; ++i)
		if (e_info->rand_info_array[i].status != running && 
		    e_info->rand_info_array[i].status != terminated)
			return false;
	return true;
}

//###############################################################################################################
//
//	Testet, ob bereits alle rand-Prozesse fertig sind
//
//###############################################################################################################
static bool AllRandsTerminated(execinfo*& e_info)
{
	for (int i = 0; i <= e_info->no_det_trans; ++i)
		if (e_info->rand_info_array[i].status != terminated)
			return false;
	return true;
}

//###############################################################################################################
//
//	Diese Funktion steuert den Ablauf im Falle einer lokalen Ausfuehrung der rand-Prozesse. 
//	Entsprechend dem Eintrag MAX_PROCESSES_PER_HOST aus dem Environment-File koennen evtl. 
//	mehrere Prozesse parallel laufen.
//
//###############################################################################################################
void	LocalExecution (execinfo*& e_info)
{
	unsigned processes_to_start   = e_info->max_processes_per_host + 1;
	unsigned terminated_processes = 0;
	FILE*	 rand_fp;

	int rand_count;
	int class_id;
	for (rand_count = 0, class_id = 1; rand_count < e_info->no_det_trans; class_id++, rand_count++)
	{
		randinfo* info = &(e_info->rand_info_array[rand_count]);

		if (info->status == terminated) continue;

		while (1)
		{
			info->host = e_info->local_host;

			//--------------------------------------------------------------------------------------
			//	rand Prozesse starten
			//--------------------------------------------------------------------------------------
		        if (DEBUG) cout <<"\t\t starting rand process ..."<<endl;
			
			StartRandProcess(info, e_info);

		        if (DEBUG) cout <<"\t\t starting rand process finished !"<<endl<<endl;

			if (WaitForConnection(e_info) != info) 
				ExecRandErr("can't connect to rand process", e_info->local_hostname, __LINE__);
			else	break;
		}
		rand_fp = info->connection_fp;

		//---------------------------------------------------------------------------------------------
		//	Daten an den rand Prozess senden	
		//---------------------------------------------------------------------------------------------
		if (DEBUG) cout <<"\t\t sending data to rand process ..."<<endl;

		SendDataToRand(rand_fp, info->smcs, class_id);

		if (DEBUG) cout <<"\t\t sending data to rand process finished !"<<endl<<endl;

		if (info->connection_fp) fflush(rand_fp);

		info->status = running;

		if (!--processes_to_start)
		{
			if (terminated_processes)
			{
				processes_to_start   = terminated_processes;
				terminated_processes = 0;
			}
		}
		else
		{
			if (DEBUG) cout <<"\t\t waiting for one messages ..."<<endl;
			do
			{
				ReceiveMessages(wait_for_one, e_info, terminated_processes);
			}
			while (!terminated_processes);
			if (DEBUG) cout <<"\t\t waiting for one messages finished !"<<endl<<endl;

			processes_to_start   = terminated_processes;
			terminated_processes = 0;
		}
	}
	if (DEBUG) cout <<"\t\t waiting for all messages ..."<<endl;

	ReceiveMessages(wait_for_all, e_info, terminated_processes);

	if (DEBUG) cout <<"\t\t waiting for all messages finished !"<<endl;
}

//###############################################################################################################
//
//	Diese Funktion uebernimmt das Management der Prozessverteilung im Falle einer verteilten Ausfuehrung 
//	der rand-Prozesse, also:
//					- benutzbare Rechner bestimmen
//					- rand-Prozesse verteilen (Neustart durch Fehler abgebrochener Prozesse)
//					- Daten an rand-Prozesse senden
//					- warten, bis alle rand-Prozesse terminiert sind
//
//###############################################################################################################
static void DistributedExecution (execinfo* e_info)
{
	int 		i;
	hostinfo*	host; 
	hostinfo**	next_host;
	hostinfo**	sorted_hosts;
	unsigned	free_hosts = 0;
	bool 		process_started; 
	bool		one_process_per_host;
	unsigned    	kb_per_sec_sum    = 0;
	unsigned 	running_processes = 0;
	unsigned 	terminated_processes = 0;

	printf ("Free hosts found:\n       ");

	do
	{
		if ((host = NextFreeHost(e_info->hostlist, free_hosts)))
		{
			printf ("%s ", host->hostname);
			fflush (stdout);
		}
	}
	while (host);

	putchar ('\n');

	if (free_hosts < 2)
	{
		puts ("\nLess than 2 free hosts found, switching to local execution...\n");
		LocalExecution(e_info);
		return;
	}

	printf ("loads: ");
	for (host = e_info->hostlist; host; host = host->next)
	{
		if (host->free)
		{
			register unsigned len = strlen (host->hostname);
			char 	 format[20];

			sprintf (format, "%%-%d.2f ", len);
			printf  (format, (float)host->load / (float)256.0);
			fflush (stdout);
			if (host->load > 3 * 256)
			{
				//-------------------------------------------------------------------------------
				// 	Rechner mit einer load > 3.0 werden nicht verwendet 
				//-------------------------------------------------------------------------------
				host->free = false;
				--free_hosts;
			}
		}
		puts ("\n");

		if (free_hosts < 2)
		{
			puts ("Less than 2 usable hosts found, switching to local execution...\n");
			LocalExecution(e_info);
			return;
		}

		if (!(sorted_hosts = (hostinfo **)malloc (free_hosts * sizeof (hostinfo *))))
			OUT_OF_MEMORY;

		SortHostsByLoad(e_info->hostlist, sorted_hosts, free_hosts);

		//	old MEXP :	e_info->rand_info_array->status = terminated;	

		one_process_per_host = false;
		next_host 	     = sorted_hosts;

		while (!AllRandsTerminated (e_info))
		{
			//---------------------------------------------------------------------------------------
			// 	Soviel rand-Prozesse wie moeglich verteilt parallel laufen lassen
			//---------------------------------------------------------------------------------------
			process_started = false;
			for (; next_host < &sorted_hosts[free_hosts] && !AllRandsActive(e_info); ++next_host)
			{
				host = *next_host;
				if ( host->free && 
				     host->running_processes < e_info->max_processes_per_host &&
				    (!one_process_per_host || !host->running_processes))
				{
					//-----------------------------------------------------------------------
					// Auf dem Rechner host->hostname kann ein rand-Prozess gestartet werden
					//-----------------------------------------------------------------------
					for (i = 0; i <= e_info->no_det_trans; ++i)
					{
						randinfo* info = &(e_info->rand_info_array[i]);

						if (info->status == no_process && running_processes < MAX_PARALLEL &&
						   (kb_per_sec_sum + info->kb_per_sec <= MAX_KB_PER_SEC_S ||
						    running_processes < MIN_PARALLEL))
						{
							info->host = host;

							StartRandProcess(info, e_info); 

							process_started = true;
							++running_processes;
							kb_per_sec_sum += info->kb_per_sec;
							break;
                                                                                                                         
							if (i > e_info->no_det_trans)
							{
								//-----------------------------------------------
								// Es konnte kein rand-Prozess gestartet werden 
								//-----------------------------------------------
								next_host            = sorted_hosts;
								one_process_per_host = true;
								break;
							}
						}
					}
				}
				if (next_host >= &sorted_hosts[free_hosts])
				{
					next_host            = sorted_hosts;
					one_process_per_host = false;
				}
				if (process_started && !one_process_per_host && !AllRandsActive(e_info))
						continue;

				terminated_processes = 0;

				//-----------------------------------------------------------------------------
				// 	Auf eine Verbindung mit den gerade gestarteten rand-Prozessen warten
				//-----------------------------------------------------------------------------
				for (i = 0; i <= e_info->no_det_trans; i++)
				{
					if (e_info->rand_info_array[i].status == waiting_for_connection)
					{
						i = 0;
						if (!WaitForConnection(e_info))
						{
							//----------------------------------------------------
							// 	Fehler : Ein rand-Prozess meldet sich nicht 
							//----------------------------------------------------
							for (int k = 0; k <= e_info->no_det_trans; ++k)
							{
								randinfo* r_info = &(e_info->rand_info_array[k]);

								if (r_info->status == waiting_for_connection)
								{
									fprintf (stderr, "can't connect to rand ");
									fprintf (stderr, "process on station '%s' !\n",
										 r_info->host->hostname);

									r_info->status     = no_process;
									r_info->host->free = false;
									--r_info->host->running_processes;
									r_info->host       = (hostinfo *)0;
								}
								putchar ('\n');
								break;
							}
						}
					}
				}

				//-----------------------------------------------------------------------------
				// 		Daten an die gestarteten Prozesse senden
				//-----------------------------------------------------------------------------
				int class_id;
				for (i=0, class_id=1; i <= e_info->no_det_trans; i++, class_id++)
				{
					register randinfo* info = &(e_info->rand_info_array[i]);
					if (info->status == connected)
					{
						if (DEBUG) cout <<"\t\t sending data to rand process ..."<<endl;

						SendDataToRand(info->connection_fp, info->smcs, class_id);
						
						if (DEBUG) cout <<"\t\t sending data to rand process finished !"<<endl<<endl;

						if (info->connection_fp) fflush(info->connection_fp);
						info->status = running;
					}
				}
				//-----------------------------------------------------------------------------
				// 	Eventuell warten, bis ein rand-Prozess terminiert ist
				//-----------------------------------------------------------------------------
				if (!terminated_processes)
				{
					//---------------------------------------------------------------------
					// 	Auf die Beendigung eines rand-Prozesses warten, aber den fuer
					// 	die Menge Mexp zustaendigen Prozess nicht beachten
					//---------------------------------------------------------------------
					do
					{
						ReceiveMessages(wait_for_one, e_info, terminated_processes);
					}
					while (!terminated_processes);
				}
			} 	// end : for (; next_host < &sorted_hosts[free_hosts] ...
		}		// end : while (!all_rands_terminated ()) ...
	}			// end : for (host = e_info->hostlist; ...
}
//###############################################################################################################
//
//	Funktion zur remote-verteilten oder lokal-verteilten rand-Prozessierung 
//
//###############################################################################################################
void	ExecuteRandProcesses(char*&	model_name,
			     ExecMode 	exec_mode, 
			     double& 	epsilon, 
			     randinfo*& rand_info_array, 
			     int& 	no_det_trans,
			     double&	sum_of_efforts)	
{
	execinfo*	e_info = new execinfo;
	double		start_distribution_at = 200.0;

	//-------------------------------------------------------------------------------------------------------
	//	Uebergabeparameter sammeln
	//-------------------------------------------------------------------------------------------------------
	e_info->epsilon			= epsilon;
	e_info->model_name		= model_name;
	e_info->no_det_trans		= no_det_trans;
	e_info->rand_info_array		= rand_info_array;
	e_info->sum_of_rusage_times     = 0.0;
	e_info->max_processes_per_host  = 2;
	
	//-------------------------------------------------------------------------------------------------------
	//	hostlist fuellen aus $HOME/.TimeNET
	//-------------------------------------------------------------------------------------------------------
	if (DEBUG) cout <<"\t\t reading environment ..."<<endl;

	ReadEnvironment(e_info, start_distribution_at);

	if (DEBUG) cout <<"\t\t reading environment finished !"<<endl<<endl;

	if (SHOW_INFOS)
	{
		ShowHostInfo(e_info->local_host);
		cout <<"local_hostname         : "<<e_info->local_hostname        <<endl;
		cout <<"max_processes_per_host : "<<e_info->max_processes_per_host<<endl;
		cout <<"start_distribution_at  : "<<start_distribution_at 	  <<endl;
	}

	//-------------------------------------------------------------------------------------------------------
	//	Socket kreieren	
	//-------------------------------------------------------------------------------------------------------
	if (DEBUG) cout <<"\t\t establishing socket ..."<<endl;

	EstablishServerSocket(e_info);

	if (DEBUG) cout <<"\t\t establishing socket finished !"<<endl;
	//-------------------------------------------------------------------------------------------------------
	//	Festlegung ob verteilt oder lokal gearbeitet werden soll
	//-------------------------------------------------------------------------------------------------------
	switch	(exec_mode)
	{
		case sequential : e_info->distributed_execution = false; break;
		case parallel   : e_info->distributed_execution = true;  break;
		default		: e_info->distributed_execution = false; break;
	}

	//-------------------------------------------------------------------------------------------------------
	//	Prozess(e) starten
	//-------------------------------------------------------------------------------------------------------
	if (e_info->no_det_trans < 2 || !(e_info->distributed_execution) || sum_of_efforts < start_distribution_at)
	{
		if (DEBUG) 
		{
			cout <<endl;
			cout <<"\t\t starting all rand processes locally ";
			if (e_info->max_processes_per_host > 1)
				cout <<"with "<<e_info->max_processes_per_host<<" concurrent processes !";
			else	cout <<"with sequiential execution !";
			cout <<endl;
			cout <<endl;
		}
		LocalExecution(e_info);
		rand_info_array = e_info->rand_info_array;
	}
	else
	{
		if (DEBUG)
		{
			cout <<endl;
			cout <<"\t\t starting all rand processes parallel on different hosts ";
			if (e_info->max_processes_per_host > 1)
				cout <<"with "<<e_info->max_processes_per_host<<" concurrent processes ! ";
			else	cout <<"! ";
			cout <<endl;
			cout <<endl;
		}

		DistributedExecution(e_info);
		rand_info_array = e_info->rand_info_array;
	}
}

//###############################################################################################################
#endif
