#ifndef __HandleExcTransitions_cc__
#define __HandleExcTransitions_cc__

#include "inc_classes.h"
#include "inc_handle_exc_t.h"

//******************************************************************************************************************
//
//	Reduzieren der Immediate-Uebergaenge
//
//******************************************************************************************************************
void	Reduce(Marking& first_marking, Marking& ret_marking, double& delay) 
{
	if (first_marking.first)
	{
		for (state_transition* h=first_marking.first; h; h=h->nextS)
		{
			double local_delay = 0;

			if (h->nextM)
			{
				local_delay = delay * h->delay.GetDelayVal();

				if (h->nextM->first) 
				{
					delay = local_delay;
					Reduce(*(h->nextM), ret_marking, delay);
				}
				else 
				{
					DelayVal dv(local_delay, _PROBABILITY);
					Marking* m = new Marking(h->nextM->pms, h->nextM->GetActiveTransArray());
					ret_marking.AppendNewMarking(*m,dv);
				}
			}
		}
	}
}

//******************************************************************************************************************
//
//	- Ueberpruefen der Schaltfaehigkeit der Immediate-Transitionen bezueglich :
//		a) globaler Schaltbedingungen
//		b) lokaler  Schaltbedingungen
//		c) Tokenanzahl in der Eingangsstelle
//		d) Tokenanzahl in der Ausgangsstelle
//		e) Kapazitaet der Ausgangsstelle
//	- Berechnen des Divisors, der der Summe der Gewichte aller aktivierebaren Bindings entspricht.
//	- Berechnen der Wahrscheinlichkeit der entsprecheneden Bindingaktivierung (gewicht v. bind. / divisor).
//	- Erstellen eines Arrays, das angibt ob ein Binding zu aktivieren ist oder nicht. 
//	- Aktivieren der aktivierbaren Bindings, indem neue Markierungen erstellt werden, bei denen das
//	  "active_trans_array" an der Stelle der aktuellen Transition-Nummer veraendert wird (mit der Nummer des 
//	  aktivierten Bindings).
//
//******************************************************************************************************************
void	CheckAndActExcTransition(Marking& actual_mark, Transition& transition, int& active_trans)
{
	//----------------------------------------------------------------------------------------------------------
	//	Globale Schaltbedingung der Transition ueberpruefen
	//----------------------------------------------------------------------------------------------------------
	if ( (transition.GlobalCondTrue()) && (transition.GetBindingSize()) )
	{
		short   binding_count   = 0;
		Weight	weight_divisor  = 0.0;
		char*   active_bindings = new char[(transition.GetBindingSize() + 1)]; 
		Weight*	weights         = new Weight[(transition.GetBindingSize() + 1)];

		for (BindingListElement* ble=transition.GetBindings()->first; ble; ble=ble->next, binding_count++)
		{
			//------------------------------------------------------------------------------------------
			//  Lokale Schaltbedingung, Vorgaenger- und Nachfolgerstellen der Transition ueberpruefen
			//------------------------------------------------------------------------------------------
			inarcs  = NULL; inarcs  = &(ble->lelement.GetInArcBind());
			outarcs = NULL; outarcs = &(ble->lelement.GetOutArcBind());

			if ((ble->lelement.LocalCondTrue()) && ArcsOk() )
			{
		   		weight_divisor                += ble->lelement.GetWeight();
				weights[binding_count]	       = ble->lelement.GetWeight();
				active_bindings[binding_count] = ACTIVE;
			}
			else 	
			{
				weights[binding_count]	       = 0.0; 
				active_bindings[binding_count] = PASSIV;
			}
			inarcs  = NULL;
			outarcs = NULL;
		}
		for (short i=0; i < binding_count; i++)
		{
			if (active_bindings[i] == ACTIVE)
			{
				//----------------------------------------------------------------------------------
				//	Wahrscheinlichkeit der entsprecheneden Aktivierung berechenen
				//	und neuen (Zwischen-) Zustand an den aktuellen Zustand anhaengen
				//----------------------------------------------------------------------------------
				double   val = weights[i] / weight_divisor;
				DelayVal dv(val, _PROBABILITY, transition.GetGlobalTransitionRef());
				Marking* m   = new Marking(actual_mark.pms, actual_mark.GetActiveTransArray());

				actual_mark.AppendNewMarking(*m,dv);
				actual_mark.first->nextM->ModifyActiveTransArray(active_trans, short(i+1));
			}
		}
		delete[] active_bindings;
		delete[] weights;
	}
}

//******************************************************************************************************************
//
//******************************************************************************************************************
void	CheckAndActExcTransitions(Marking& actual_mark, TransitionListElement& transition_le, int active_tr)
{
	//----------------------------------------------------------------------------------------------------------
	//	aktiviert alle schaltfaehigen Bindings einer Transition und gibt die so enstandenen
	//	(Zwischen-) Markierungen als Uebergaenge von actual_mark zurueck 
	//----------------------------------------------------------------------------------------------------------
	short* ata = actual_mark.GetActiveTransArray();
	if (ata[active_tr] == 0) CheckAndActExcTransition(actual_mark, transition_le.lelement, active_tr);

	if (transition_le.next)
	{
		active_tr++;

		if (actual_mark.first)
		{
			for (state_transition* h=actual_mark.first; h; h=h->nextS)
				if (h->nextM && !(h->nextM->first))
				  	CheckAndActExcTransitions(*(h->nextM), *(transition_le.next), active_tr);
		}
		else 	 CheckAndActExcTransitions(actual_mark, *(transition_le.next), active_tr);
	}
}

//******************************************************************************************************************
//
//	Durch ES-Transitionen entstehende Zwischenzustaende behandeln
//
//******************************************************************************************************************
void	HandleExcTransitions(Marking& ret_marking)
{
	if (ExclusiveTransitionRefList.size)
	{
		//--------------------------------------------------------------------------------------------------
		//	1. Markierung des aus Probability-Uebergaengen bestehenden Mini-Graphen erstellen	
		//--------------------------------------------------------------------------------------------------
		Marking first_marking(ret_marking.pms, ret_marking.GetActiveTransArray());

		//--------------------------------------------------------------------------------------------------
		//	act_trans_array eventuell kreieren und initialisieren
		//--------------------------------------------------------------------------------------------------
		if (!(first_marking.GetActiveTransArray())) first_marking.InitActiveTransArray();
		if (!(first_marking.pms))    		    first_marking.InitPlaceMultiSet();

		//--------------------------------------------------------------------------------------------------
		//	Mini-Graph aus Probability-Uebergaengen erstellen	
		//--------------------------------------------------------------------------------------------------
		init_global_actual_objects (first_marking.pms);
		CheckAndActExcTransitions  (first_marking, *(ExclusiveTransitionRefList.first), 0); 
	
		//--------------------------------------------------------------------------------------------------
		//	Mini-Graph aus Probability-Uebergaengen reduzieren	
		//--------------------------------------------------------------------------------------------------
		double delay = 1.0;
		Reduce(first_marking, ret_marking, delay);

		//--------------------------------------------------------------------------------------------------
		//	aufraeumen	
		//--------------------------------------------------------------------------------------------------
		first_marking.ClearTotal();
  	}
}
//******************************************************************************************************************
#endif
