#ifndef __HandleImmTransitions_cc__
#define __HandleImmTransitions_cc__

#include "inc_classes.h"
#include "inc_handle_imm_t.h"

//******************************************************************************************************************
//
//	Fehlerbehandlung
//
//******************************************************************************************************************
void	HandleImmErr(char* s1, char* s2, int line)
{
	if (s2) RRGErr = new RRGError(__FILE__, s1, line, NULL, s2);
	else	RRGErr = new RRGError(__FILE__, s1, line);
	PrepareEnd();
}
//******************************************************************************************************************
//
//	- Schalten aller schaltfaehigen Bindings (enabled_bindings) 
//	-> Kreieren einer neuen Markierung 
//	-> Neue Markierung in eine Liste aus Markierungen einbringen (np_marking_ptr_list)
//
//******************************************************************************************************************
void	FireEnabledImmBindings (NonPersistentMarking& act_marking)
{
	for (BindingListElement* ble=enabled_bindings.first; ble; ble=ble->next)
	{
		token_multiset** tms = new token_multiset*[no_of_places];

		int i = 0;
		for (PlaceHeaderPtrListElement* ph=PlaceHeaderPtrRefList.first; ph ; ph=ph->next, i++)
		{
			if (!(ph->plelement)) HandleImmErr("PlaceHeader is NULL-pointer", NULL, __LINE__);

			tms[i] = copy_token_multiset(ph->plelement->Gettoken_multiset());

			arcs = NULL; arcs = &(ble->lelement.GetInArcBind());
			FireArcBindings(minus, ph->plelement->GetPlaceNo(), tms[i]);

			arcs = NULL; arcs = &(ble->lelement.GetOutArcBind());
			FireArcBindings(plus, ph->plelement->GetPlaceNo(), tms[i]);

			arcs = NULL;
		}

		if ( !(PlaceCapOkAfterFire(tms, act_marking.pms)) ) 
		{
			clear_pms(tms, (no_of_places-1));
			return;
		}

		DelayVal 		new_delay(ble->lelement.GetDelayVal(), _IMMEDIATE);
		NonPersistentMarking* 	new_nonp_mark = new NonPersistentMarking(tms, new_delay);

		np_marking_ptr_list.Append(new_nonp_mark);
		clear_pms(tms, (no_of_places-1));
	}
}

//******************************************************************************************************************
//
//	- Ueberpruefen der Schaltfaehigkeit der Immediate-Transitionen bezueglich :
//		a) globaler Schaltbedingungen
//		b) lokaler  Schaltbedingungen
//		c) Tokenanzahl in der Eingangsstelle
//		d) Tokenanzahl in der Ausgangsstelle
//		e) Kapazitaet der Ausgangsstelle
//	- Erstellen einer Liste aller Bindings die a) bis e) erfuellen (enabled_bindings)
//	- Berechnen des Divisors, der sich aus der Summe der Delays aller in dieser Markierung
//	  schaltfaehigen Bindings ergibt (imm_divisor).
//
//******************************************************************************************************************
void	CheckImmTransitions(double& imm_divisor)
{
	for (TransitionListElement* tle=ImmediateTransitionRefList.first; tle; tle=tle->next)
	{
		if ( (tle->lelement.GlobalCondTrue()) && (tle->lelement.GetBindingSize()) )
		{
			for (BindingListElement*  ble=tle->lelement.GetBindings()->first ; ble; ble=ble->next)
			{
				inarcs  = NULL; inarcs  = &(ble->lelement.GetInArcBind());
				outarcs = NULL; outarcs = &(ble->lelement.GetOutArcBind());

				if ( (ble->lelement.LocalCondTrue()) && ArcsOk() )
				{
			   		enabled_bindings.Append(ble->lelement);
			   		double imm_delay = ble->lelement.GetDelayVal();

			   		if (imm_delay <= 0)  
			   			HandleImmErr("imm-delay <= 0", tle->lelement.GetName(), __LINE__);

			   		imm_divisor = imm_divisor + imm_delay;
				}
				inarcs  = NULL;
				outarcs = NULL;
			}
		}
	}
}

//******************************************************************************************************************
//
//	- Aendern aller Delays ab tmp_marking durch zusammentreffen paralleler Zweige von Imm-Uebergaengen 
//
//******************************************************************************************************************
void	ChangeAllDelays(NonPersistentMarking*& tmp_marking, DelayVal new_delay)
{
	if (!tmp_marking) return;

	double old_delay_val = tmp_marking->GetDelay().GetDelayVal();
	double new_delay_val = new_delay.GetDelayVal();

	tmp_marking->ChangeDelay(new_delay);

	if (!(tmp_marking->first)) return;

	double next_dv     = 0;

	for (np_state_transition* h=tmp_marking->first; h; h=h->nextS)
	{
		if (!(h->nextM)) continue;

		next_dv = 0;
		next_dv = ((h->nextM->GetDelay().GetDelayVal()) - (old_delay_val * (h->delay.GetDelayVal()))) + 
			  (new_delay_val * (h->delay.GetDelayVal()));

		DelayVal  new_next_dv(next_dv, _IMMEDIATE);
		ChangeAllDelays(h->nextM, new_next_dv);
	}
}

//******************************************************************************************************************
//
//	- Ueberpruefung von Endlosschleifen
//
//******************************************************************************************************************
bool	EndlessLoop(NonPersistentMarking& act_marking, NonPersistentMarking* tmp_marking)
{
	for (np_state_transition* h=tmp_marking->first; h; h=h->nextS)
	{
		if (h->nextM)
			if ( *(h->nextM) == act_marking ) return true; 
			else return EndlessLoop(act_marking, h->nextM);
	}
	return false;
}

//******************************************************************************************************************
//
//	- Falls eine Markierung bereits existiert     	=> new_nonp_mark    = bereits existierende Markierung
//							=> existing_marking = neue M., die der bereits existierenden 
//							 		      entspricht
//	- Falls eine Markierung wirklich neu ist	=> new_nonp_mark    = neue M.
//							=> existing_marking = NULL
//
//******************************************************************************************************************
void	MakeDelay(double 		 imm_divisor, 
		  NonPersistentMarking&  act_marking, 
		  NonPersistentMarking*& new_nonp_mark, 
		  NonPersistentMarking*  existing_marking) 
{
	double last_delay_val       = act_marking.GetDelay().GetDelayVal();
	double transition_delay_val = 0;

	if (!existing_marking)
	{
		           transition_delay_val = new_nonp_mark->GetDelay().GetDelayVal() / imm_divisor;
		double     new_delay_val        = transition_delay_val * last_delay_val;
		DelayVal   new_delay(new_delay_val, _IMMEDIATE);

		new_nonp_mark->ChangeDelay(new_delay);
	}
	else
	{
		if (EndlessLoop(act_marking, new_nonp_mark))
		{
			HandleImmErr("endless loop : new_marking exists in own line", NULL, __LINE__);
		}
		else
		{
				   transition_delay_val = existing_marking->GetDelay().GetDelayVal() / imm_divisor;
			double     tmp_delay_val	= transition_delay_val * last_delay_val;
			double     new_delay_val        = new_nonp_mark->GetDelay().GetDelayVal() + tmp_delay_val;
			DelayVal   new_delay(new_delay_val, _IMMEDIATE);

			ChangeAllDelays(new_nonp_mark, new_delay);
		}
	}
	DelayVal transition_delay(transition_delay_val, _IMMEDIATE);
	act_marking.AppendNewMarking(*new_nonp_mark, transition_delay); 
}

//******************************************************************************************************************
//
//	- Schaltfaehige Markierungen unique im Hash speichern 
//	- Uebergangsdelay errechnen
//
//******************************************************************************************************************
void	SaveMarkingMakeDelay(NonPersistentMarking& act_marking, double imm_divisor)
{
	for (NonPersistentMarkingPtrListElement* mple=np_marking_ptr_list.first; mple; mple=mple->next)
	{
		if (!(mple->plelement)) continue;

		bool exists = false;
		NonPersistentMarking* new_nonp_mark = SaveMarkingInImmHash(mple->plelement, exists);

		if (!new_nonp_mark) HandleImmErr("can't create new marking in immediate-marking-hash", NULL, __LINE__);

		if (!exists) 	MakeDelay(imm_divisor, act_marking, new_nonp_mark, NULL);
		else 	 	MakeDelay(imm_divisor, act_marking, new_nonp_mark, mple->plelement);
	}
	np_marking_ptr_list.Clear();
}

//******************************************************************************************************************
//
//	- Ueberpruefung aller Bindings auf Schaltfaehigkeit --> schaltfaehige Bindings (enabled_bindings)
//	- Schalten schaltfaehiger Bindings                  --> neue Markierungen      (np_marking_ptr_list)
//
//******************************************************************************************************************
void	CheckAndFireImmTrans(NonPersistentMarking& act_marking)
{
	double imm_divisor = 0;

	CheckImmTransitions(imm_divisor);

	if (enabled_bindings.size)
	{
		FireEnabledImmBindings(act_marking);

		if (np_marking_ptr_list.size) SaveMarkingMakeDelay(act_marking, imm_divisor);
		else HandleImmErr("transition was fired, but no new marking was created", NULL, __LINE__);
	}
	enabled_bindings.Clear();
}

//******************************************************************************************************************
//
//	Reduzieren der Immediate-Uebergaenge
//
//******************************************************************************************************************
void	Reduce(NonPersistentMarking& first_marking, Marking& ret_marking)
{
	if (!first_marking.first) return;

	for (np_state_transition* h=first_marking.first; h; h=h->nextS)
	{
		if (h->nextM)
		{
			if (h->nextM->first) Reduce(*(h->nextM), ret_marking);
			else 
			{
				if (h->nextM->GetKey() != REDUCED)
				{
					h->nextM->SetKey(REDUCED);
					Marking* m = new Marking(h->nextM->pms);
					ret_marking.AppendNewMarking(*m, h->nextM->GetDelay());
				}
			}
		}
	}
}

//******************************************************************************************************************
//
//******************************************************************************************************************
void	CheckAndFireImmTransitions(NonPersistentMarking& act_marking)
{
	CheckAndFireImmTrans(act_marking);

	if (act_marking.first)
	{
		for (np_state_transition* h=act_marking.first; h; h=h->nextS)
		{
			if (h->nextM && !(h->nextM->first))
			{
				init_global_actual_objects(h->nextM->pms);
				CheckAndFireImmTransitions(*(h->nextM));
			}
		}
	}
}
//******************************************************************************************************************
//
//	Die entstandenen Markierung untersuchen, ob exclusive-server-Bindings durch das Schalten der 
//	immediate-Transitionen deaktiviert wurden.
//
//******************************************************************************************************************
void	CheckActTransArrays(Marking& ret_marking)
{
	if (!(ExclusiveTransitionRefList.size)) return;
	if (!(ret_marking.first))		return;

	short* active_trans_array = ret_marking.GetActiveTransArray();

	for (state_transition* h=ret_marking.first; h; h=h->nextS)
	{
		if (!h->nextM) continue;

		int    t_counter          = 0;
		init_global_actual_objects (h->nextM->pms);

		for (TransitionListElement* tle=ExclusiveTransitionRefList.first; tle; tle=tle->next, t_counter++)
		{
			h->nextM->ModifyActiveTransArray(t_counter, 0);
			//------------------------------------------------------------------------------------------
			//      passive Transition braucht nicht ueberprueft zu werden
			//------------------------------------------------------------------------------------------
			if (active_trans_array[t_counter] == 0)
				continue;

			//------------------------------------------------------------------------------------------
			//      aktive Transition muss auf Schaltfaehigkeit ueberprueft werden
			//------------------------------------------------------------------------------------------
			if ( (tle->lelement.GlobalCondTrue()) && (tle->lelement.GetBindingSize()) )
			{
				int  b_counter = 1;
				for (BindingListElement* ble=tle->lelement.GetBindings()->first; ble; ble=ble->next, 
													b_counter++)
				{
					if (b_counter != active_trans_array[t_counter])
						continue;

					inarcs  = NULL; inarcs  = &(ble->lelement.GetInArcBind());
					outarcs = NULL; outarcs = &(ble->lelement.GetOutArcBind());

					if ( (ble->lelement.LocalCondTrue()) && ArcsOk() )
					    	h->nextM->ModifyActiveTransArray(t_counter, b_counter);

					inarcs  = NULL;
					outarcs = NULL;
					break;
				}
			}
		}
	}
}

//******************************************************************************************************************
//
//	Durch Immediate-Transitionen entstehende Uebergaenge behandeln
//
//******************************************************************************************************************
void	HandleImmTransitions(Marking& ret_marking)
{
	if (ImmediateTransitionRefList.size)
	{
		CreateAndInitImmHash();
		//-------------------------------------------------------------------------------------------
		//	erste Markierung des aus Immediate-Uebergaengen bestehenden Mini-Graphen  erstellen	
		//-------------------------------------------------------------------------------------------
		NonPersistentMarking  first_marking(ret_marking.pms);

		if (!(first_marking.pms)) first_marking.InitPlaceMultiSet();

		//-------------------------------------------------------------------------------------------
		//	Mini-Graph aus Immediate-Uebergaengen erstellen	
		//-------------------------------------------------------------------------------------------

		init_global_actual_objects(ret_marking.pms);
		CheckAndFireImmTransitions(first_marking);
	
		//-------------------------------------------------------------------------------------------
		//	Mini-Graph aus Immediate-Uebergaengen reduzieren	
		//-------------------------------------------------------------------------------------------

		Reduce(first_marking, ret_marking);
		CheckActTransArrays(ret_marking);

		//-------------------------------------------------------------------------------------------
		//	"aufraeumen"
		//-------------------------------------------------------------------------------------------
		CleanImmHash();
		np_marking_ptr_list.Clear();
  	}
}
//******************************************************************************************************************
#endif
