#ifndef __HandlePersistentMarking_cc__
#define __HandlePersistentMarking_cc__

#include "inc_classes.h"
#include "inc_handle_persistent_m.h"

//**********************************************************************************************************************
//
//	Fehlerbehandlung
//
//**********************************************************************************************************************
void	HandlePerErr(char* s1, int line)
{
	RRGErr = new RRGError(__FILE__, s1, line);
	PrepareEnd();
}

//**********************************************************************************************************************
//
//     	Alle nicht mehr schaltfaehigen Bindings werden deaktiviert 
//
//**********************************************************************************************************************
void	DeaktivateBindings(Marking*& tmp_marking)
{
	int    t_counter 	  = 0;
	short* active_trans_array = tmp_marking->GetActiveTransArray();

	for (TransitionListElement* tle=ExclusiveTransitionRefList.first; tle; tle=tle->next, t_counter++)
	{
		//------------------------------------------------------------------------------------------------------
		//	passive Transition braucht nicht aktiviert zu werden
		//------------------------------------------------------------------------------------------------------
		if (active_trans_array[t_counter] == 0)
			continue;
		
		//------------------------------------------------------------------------------------------------------
		//	aktive Transition muss auf Schaltfaehigkeit ueberprueft werden	
		//------------------------------------------------------------------------------------------------------
		if ( (tle->lelement.GlobalCondTrue()) && (tle->lelement.GetBindingSize()) )
		{
			int  b_counter = 1;
			for (BindingListElement* ble=tle->lelement.GetBindings()->first; ble; ble=ble->next, b_counter++)
			{
				if (b_counter == active_trans_array[t_counter])
				{
					inarcs  = NULL; inarcs  = &(ble->lelement.GetInArcBind());
					outarcs = NULL; outarcs = &(ble->lelement.GetOutArcBind());

					if ( !(ble->lelement.LocalCondTrue()) || !(ArcsOk()) )
						active_trans_array[t_counter] = 0;

					inarcs  = NULL;
					outarcs = NULL;
					break;
				}
			}
		}
		else	active_trans_array[t_counter] = 0;
	}
}
//**********************************************************************************************************************
//
//      - Schalten eines schaltfaehigen Bindings (enabled_binding) 
//      	-> Kreieren einer neuen Markierung (new_marking)
//      	-> Neue Markierung in eine Liste aus Markierungen einbringen (marking_list)
//
//**********************************************************************************************************************
void	FireEnabledBinding(int global_trans_ref, int multiplicator)
{
	token_multiset** tms = new token_multiset*[no_of_places];
	int 		 i   = 0;

	for (PlaceHeaderPtrListElement* ph=PlaceHeaderPtrRefList.first; ph ; i++, ph=ph->next)
	{
		if (!(ph->plelement)) HandlePerErr("PlaceHeader is NULL-pointer", __LINE__);

		tms[i] = copy_token_multiset(ph->plelement->Gettoken_multiset());

		arcs = NULL; arcs = &(enabled_binding->GetInArcBind());
		FireArcBindings(minus, ph->plelement->GetPlaceNo(), tms[i]);

		arcs = NULL; arcs = &(enabled_binding->GetOutArcBind());
		FireArcBindings(plus, ph->plelement->GetPlaceNo(), tms[i]);

		arcs = NULL;
	}
	if (!(PlaceCapOkAfterFire(tms, new_marking->pms))) 
	{
		clear_pms(tms, (no_of_places-1));
		return;
	}

	Marking* tmp_marking = new Marking(tms, new_marking->GetActiveTransArray());
	DelayVal new_delay;

	if (enabled_binding->GetDelay()->GetDelayType() != _GENERAL)
	{
		new_delay = DelayVal((enabled_binding->GetDelayVal() * (multiplicator>0?multiplicator:1)), 
		   	              enabled_binding->GetDelay()->GetDelayType(), global_trans_ref);
	}
	else 	new_delay = DelayVal(0.0, _GENERAL, global_trans_ref);

	init_global_actual_objects(tms);
	DeaktivateBindings	  (tmp_marking);
	init_global_actual_objects(new_marking->pms);

	if ((int(new_delay.delay_typ) == _EXPONENTIAL))
	{
		bool found = false;
		if (new_marks_and_delays->first)
		{
			int pos = 1;
			for (m_element* md=new_marks_and_delays->first; md; pos++, md=md->next)
			{
				if (!(md->marking)) continue;

				if (*(md->marking) == *tmp_marking)
				{
					if ((int(md->delay.delay_typ) == _EXPONENTIAL))
					{
						md->delay.ChangeDelayVal(md->delay.GetDelayVal() + 
									 new_delay.GetDelayVal());
						found = true;
						break;
					}
				}
			}
		}
		if (!found) append(new_marks_and_delays, tmp_marking, new_delay);
	}
	else 	append(new_marks_and_delays, tmp_marking, new_delay);

	clear_pms(tms, (no_of_places-1));
}
//**********************************************************************************************************************
//
//      - virtuelles Schalten eines schaltfaehigen Bindings (enabled_binding) 
//	  Dabei wird die aktuelle Markierung temporaer geaendert um so feststellen zu koennen, ob in
//	  der Folgemarkierung das Binding einer IS-Transition immer noch schaltfaehig ist.
//      	-> Kreieren einer neuen Markierung die temporaer zur Aktuellen gemacht wird 
//
//**********************************************************************************************************************
void	VirtualFireEnabledBinding(int count)
{
	token_multiset** tms = new token_multiset*[no_of_places];
	int 		 i   = 0;

	for (PlaceHeaderPtrListElement* ph=PlaceHeaderPtrRefList.first; ph ; i++, ph=ph->next)
	{
		if (!(ph->plelement)) HandlePerErr("PlaceHeader is NULL-pointer", __LINE__);

		tms[i] = copy_token_multiset(ph->plelement->Gettoken_multiset());

		arcs = NULL; arcs = &(enabled_binding->GetInArcBind());
		FireArcBindings(minus, ph->plelement->GetPlaceNo(), tms[i]);

		arcs = NULL; arcs = &(enabled_binding->GetOutArcBind());
		FireArcBindings(plus , ph->plelement->GetPlaceNo(), tms[i]);

		arcs = NULL;

		if (count) clear_tms((ph->plelement->tms));
		ph->plelement->tms = copy_token_multiset(tms[i]);
	}
	clear_pms(tms, (no_of_places-1));
}
//**********************************************************************************************************************
//
//	Schaltfaehigkeit von Infinte-Server-Transitionen ueberprufen und ggf. schalten
//
//**********************************************************************************************************************
void    CheckAndFireInfTransitions()
{
	for (TransitionListElement* tle = InfiniteTransitionRefList.first; tle; tle = tle->next)
	{
		if ( (tle->lelement.GlobalCondTrue()) && (tle->lelement.GetBindingSize()) )
		{
			for (BindingListElement*  ble=tle->lelement.GetBindings()->first ; ble; ble = ble->next)
			{
				//-------------------------------------------------------------------------------------
				//	Haeufigkeit eines hintereinander schaltbaren Bindings zaehlen (fireable_new)
				//-------------------------------------------------------------------------------------
				int fireable_new=0; 
				int fireable_old=0; 
				do
				{
					fireable_old = fireable_new;

					inarcs  = NULL; inarcs  = &(ble->lelement.GetInArcBind());
					outarcs = NULL; outarcs = &(ble->lelement.GetOutArcBind());

					if ( (ble->lelement.LocalCondTrue()) && ArcsOk() )
					{
						enabled_binding = NULL;
						enabled_binding = &(ble->lelement);

						VirtualFireEnabledBinding(fireable_new);

						enabled_binding = NULL;
						fireable_new++;
					}
					inarcs  = NULL;
					outarcs = NULL;
				} while (fireable_new > fireable_old);

				if (fireable_new) clean_global_actual_objects();
				init_global_actual_objects (new_marking->pms);

				//-------------------------------------------------------------------------------------
				//	Binding schalten, falls dieses ein oder mehrmals hintereinander schaltbar war	
				//-------------------------------------------------------------------------------------
				if (fireable_new > 0)
				{
					enabled_binding = NULL;
					enabled_binding = &(ble->lelement);

					FireEnabledBinding(tle->lelement.GetGlobalTransitionRef(), fireable_new);

					enabled_binding = NULL;
				}
			}
		}
	}
}

//**********************************************************************************************************************
//
//	Schaltfaehigkeit von Exclusive-Server-Transitionen ueberprufen und ggf. schalten
//
//**********************************************************************************************************************
void    CheckAndFireExcTransitions()
{
	int     trans_counter      = 0;
	short   act_bind_no        = 0;
	short*	active_trans_array = new_marking->GetActiveTransArray();

	for (TransitionListElement* tle = ExclusiveTransitionRefList.first; tle; trans_counter++, tle=tle->next)
	{
		if ( ((act_bind_no = active_trans_array[trans_counter]) != 0) )
		{
			enabled_binding = NULL;
			enabled_binding = tle->lelement.GetBindings()->GetElementPtr(act_bind_no);
			int	g_tref  = tle->lelement.GetGlobalTransitionRef();

			if (enabled_binding)  FireEnabledBinding(g_tref, 0);
			else    	      HandlePerErr("not existing binding referenced", __LINE__);
		}
	}
	enabled_binding = NULL;
}

//**********************************************************************************************************************
//
//	Schalten von ES/IS-Transitionen in einer andauernden Markierung 
//
//**********************************************************************************************************************
void	HandlePersistentMarking()
{
	if (!new_marking) return;

	if (ExclusiveTransitionRefList.size)
	{
		init_global_actual_objects(new_marking->pms);
		CheckAndFireExcTransitions();
	}
	if (InfiniteTransitionRefList.size)
	{
		init_global_actual_objects(new_marking->pms);
		CheckAndFireInfTransitions();
	}
}
//**********************************************************************************************************************
#endif
