#ifndef __Marking_cc__
#define __Marking_cc__

#include <iostream.h>

#include "DelayVal.h"
#include "token_multiset.h"
#include "state_transition.h"

#include "Marking.h"

extern int no_of_exc_transitions;
extern int no_of_places;

//######################################################################################################################
//						MARKING
//######################################################################################################################
inline void	Marking::NewPMS(token_multiset** p)
{
	pms = NULL;
	if (!p) return;

	pms = new token_multiset*[no_of_places];
	for (int i=0; i < no_of_places-1; i++)
		pms[i] = copy_token_multiset(p[i]);
}
//--------------------------------------------------------------------------------------------------------------
inline void	Marking::NewATA(short* ata)
{
	active_trans_array = NULL;
    	if (ata)
    	{
    		active_trans_array = new short[(no_of_exc_transitions + 1)];
    		for (int i=0; i < no_of_exc_transitions; i++)
    			active_trans_array[i] = ata[i]; 
	}
}
//--------------------------------------------------------------------------------------------------------------
inline void	Marking::CreateKey()
{
	key = 0;
	if (pms)
	{
		for (int i=0; i < no_of_places-1; i++)
			if (pms[i])   key +=((i + 1) * pms[i]->key); 
	}

	if (active_trans_array)
	{
    		for (int i=0; i < no_of_exc_transitions; i++)
    		  	if (active_trans_array[i]) key *=((i + 1) * active_trans_array[i]); 
	}
	if (key < 0) key = -key;
}
//######################################################################################################################
Marking::Marking()			 		 
:	key			(DEF_KEY),
	active_trans_array	(NULL),
	pms                     (NULL),
	status			(0),
  	mtype			(0) 
  	{ first = NULL; }

Marking::Marking(int)		 		 
:	key			(DEF_KEY),
	active_trans_array	(NULL),
	pms                     (NULL),
	status			(0),
  	mtype			(0) 
  	{ first = NULL; }

Marking::Marking(token_multiset** p) 		 
: 	active_trans_array	(NULL),
	status			(0),
  	mtype			(0) 
  { 
  	NewPMS(p);
  	CreateKey();
  	first = NULL; 
  }

Marking::Marking(const Marking& m)	 		 
: 	key			(m.key),
	status			(m.status),
  	mtype			(m.mtype) 
  { 
  	NewPMS(m.pms);
  	NewATA(m.active_trans_array);
  	first = NULL;
  }

Marking::Marking(short* ata) 		 
:	key			(DEF_KEY),		
	pms                     (NULL),
	status			(0),
  	mtype			(0) 
  { 
  	NewATA(ata);
  	first = NULL;
  }

Marking::Marking(token_multiset** p, short* ata) 		 
:	status			(0),
  	mtype			(0) 
  { 
  	NewPMS(p);
  	NewATA(ata);
  	CreateKey(); 
  	first = NULL;
  }

//######################################################################################################################
inline void    Marking::ClearPMS()
{
	if (!pms) return;

	clear_pms(pms, (no_of_places-1));

	pms = NULL;
}
//--------------------------------------------------------------------------------------------------------------
void	Marking::ClearTotal() 			 	 
{
	if (first)
	{
		for (state_transition* h=first; h; h=h->nextS)
		{
			if (h->nextM)
			{
				h->nextM->ClearTotal();
				h->nextM->Clear();
				delete h->nextM;
				h->nextM = NULL;
			}
		}
	}
	Clear();
}
//--------------------------------------------------------------------------------------------------------------
void	Marking::Clear() 			 	 
{
	if (first)
	{
		if (first->nextS)
		{
			for (state_transition* h=first->nextS; h; h=h->nextS)
			{
				delete first;
				first = NULL;
				first = h;
			}
			delete first;
			first = NULL;
		}
		else
		{
			delete first;
			first = NULL;
		}
	}

	if (active_trans_array) 
	{
		delete[] active_trans_array;
		active_trans_array = NULL; 
	}

	ClearPMS(); 

	key    = DEF_KEY;
	mtype  = 0;
	status = 0; 
}
//--------------------------------------------------------------------------------------------------------------
int	Marking::GetNoOfStateTrans()
{
	if (!first) return 0;

	int i = 0;

	for (state_transition* st=first; st; st=st->nextS, i++);

	return i;
}
//--------------------------------------------------------------------------------------------------------------
int	Marking::GetNoOfExpPaths()
{
	if (!first) return 0;

	int no_exp_paths = 0;

	for (state_transition* st=first; st; st=st->nextS)
		if ((int(st->delay.delay_typ) == _EXPONENTIAL) || (int(st->delay.delay_typ) == _PREEMP_EXP ) )
			no_exp_paths++;
	return no_exp_paths;
}
//--------------------------------------------------------------------------------------------------------------
double	Marking::GetSumOfRates()
{
	if (!first) return 0.0;

	double sum_of_rates = 0.0;

	for (state_transition* st=first; st; st=st->nextS)
		if ((int(st->delay.delay_typ) == _EXPONENTIAL) || (int(st->delay.delay_typ) == _PREEMP_EXP ) )
			sum_of_rates = sum_of_rates + st->delay.GetDelayVal();
	return sum_of_rates;
}
//--------------------------------------------------------------------------------------------------------------
int	Marking::GetDetOrGenTransRef()
{
	if (!first) return 0;

	for (state_transition* st=first; st; st=st->nextS)
		if ((int(st->delay.delay_typ) == _DETERMINISTIC) || (int(st->delay.delay_typ) == _GENERAL))
			return st->delay.GetTransitionRef();
	return 0;
}
//--------------------------------------------------------------------------------------------------------------
double	Marking::GetDetOrGenDelay()
{
	if (!first) return 0.0;

	for (state_transition* st=first; st; st=st->nextS)
		if ((int(st->delay.delay_typ) == _DETERMINISTIC) || (int(st->delay.delay_typ) == _GENERAL))
			return st->delay.GetDelayVal();
	return 0.0;
}
//--------------------------------------------------------------------------------------------------------------
int	Marking::SetNumber(int n)			{ key = n; return 1;        }
int	Marking::GetNumber()				{ return key;		    }
int	Marking::GetKey()				{ return key;		    }
char*	Marking::GetName()				{ return DEF_STRING;        }
bool	Marking::LessThan(Marking& m)			{ return (key < m.key);     }
bool	Marking::operator!=(Marking& m)			{ return !( *this == m );   }
//--------------------------------------------------------------------------------------------------------------
void			Marking::ModifyActiveTransArray(int pos, short val)
{
	if (!active_trans_array) InitActiveTransArray();
	if (pos < 0 || pos > no_of_exc_transitions) 
	{
		CreateKey();
		return;
	}
	active_trans_array[pos] = val;
	CreateKey();
}
//--------------------------------------------------------------------------------------------------------------
short*			Marking::GetActiveTransArray()	             		   
{ 
	if (active_trans_array) return active_trans_array;

	short* ret_array = new short[(no_of_exc_transitions + 1)];
	for (int i=0; i < no_of_exc_transitions; i++)
		ret_array[i] = 0; 
	return ret_array;
}
//--------------------------------------------------------------------------------------------------------------
void			Marking::InitPlaceMultiSet()          
{ 
	ClearPMS();

	pms = new token_multiset*[no_of_places];
	for (int i=0; i < no_of_places-1; i++)
		pms[i] = NULL;
}
//--------------------------------------------------------------------------------------------------------------
void			Marking::InitActiveTransArray()          
{ 
	if (active_trans_array)
	{
		delete[] active_trans_array;
		active_trans_array = NULL;
	}
	active_trans_array = new short[no_of_exc_transitions]; 
	for (int i=0; i < no_of_exc_transitions; i++)
		active_trans_array[i] = 0;
}
//--------------------------------------------------------------------------------------------------------------
bool			Marking::operator==(Marking& m)		      
{ 
	if (key   != m.key)      return false;
	if (mtype != m.mtype)    return false;

	if (pms && m.pms)
	{
		for (int i=0; i < no_of_places-1; i++)
			if ( !(tms_are_equal(pms[i], m.pms[i])) )
				return false;
	}
	else	if (!(!pms && !m.pms)) return false;

	if (!active_trans_array && !m.active_trans_array) return true;
	if ( active_trans_array &&  m.active_trans_array) 
	{
		for (int j=0; j < no_of_exc_transitions; j++)
			if (active_trans_array[j] != m.active_trans_array[j]) 
				return false;
	}
	else    return false;

	return true;
}
//--------------------------------------------------------------------------------------------------------------
Marking&		Marking::operator =(const Marking& m)	
{ 
	Clear(); 

	key   	= m.key;
	mtype 	= m.mtype;
	status 	= m.status;

	NewPMS(m.pms);
	NewATA(m.active_trans_array);

	return *this; 
}
//--------------------------------------------------------------------------------------------------------------
void	Marking::AppendNewMarking(Marking& m, DelayVal& d)				
{ 
	if (!first)
	{
		first	     = new state_transition;
		first->delay = d;
		first->nextM = &m;
		first->nextS = NULL;
		return;
	}

        for (state_transition* st=first; st; st=st->nextS)
        {
		if (*(st->nextM) == m)
		{
			if (st->delay == d) return;
			else
			{
				state_transition* new_s = new state_transition;

				new_s->delay = d;
				new_s->nextM = st->nextM;
				new_s->nextS = first;
				first        = new_s;
				return;
			}
		}
	}

	state_transition* new_st = new state_transition;

	new_st->delay = d;
	new_st->nextM = &m;
	new_st->nextS = first;
	first         = new_st;
}
//##############################################################################################################
#endif
