#ifndef __NetEvaluation_cc__
#define __NetEvaluation_cc__

extern "C" {
#include <string.h>
}
extern "C" { char* index(const char*, int);	}

#include "inc_classes.h"
#include "inc_examine_rrg.h"
#include "inc_net_evaluation.h"

//##################################################################################################################
//
//	Fehlerbehandlung
//
//##################################################################################################################
void	NetEvalError(char* s1, char* s2, int line)
{
	RRGErr = new RRGError(__FILE__, s1, line, NULL, s2);
	PrepareEnd();
}
//##################################################################################################################
//
//	errechnet mit pi-vektor und Markierungsliste Wahrscheinlichkeiten
//
//##################################################################################################################
void	EvaluateProbability(MarkingPtrList& markings, Cell* c, double& result)
{
	for (MarkingPtrListElement* mp=markings.first; mp; mp=mp->next)
	{
		if (!(mp->plelement)) continue;

		init_global_actual_objects(mp->plelement->pms);

		if ((c->CalculateFloat())) 
			result += pi_vector[(mp->plelement->GetNumber() - 1)];
	}
}
//##################################################################################################################
//
//	errechnet mit pi-vektor und Markierungsliste Erwartungswerte 
//
//##################################################################################################################
void	EvaluateExpectVal(MarkingPtrList& markings, Cell* c, double& result)
{
	for (MarkingPtrListElement* mp=markings.first; mp; mp=mp->next)
	{
		if (!(mp->plelement)) continue;

		init_global_actual_objects(mp->plelement->pms);
		int n   = int(c->CalculateFloat());
		result += n * pi_vector[(mp->plelement->GetNumber() - 1)];
	}
}
//##################################################################################################################
//
//	errechnet mit pi-vektor und Markierungsliste bedingte Wahrscheinlichkeiten
//
//##################################################################################################################
void	EvaluateBayesP(MarkingPtrList& markings, Cell* c, double& result, double& divisor)
{
	for (MarkingPtrListElement* mp=markings.first; mp; mp=mp->next)
	{
		if (!(mp->plelement)) continue;

		init_global_actual_objects(mp->plelement->pms);

		switch (int(c->CalculateFloat()))
		{
			case 1 : divisor += pi_vector[(mp->plelement->GetNumber() - 1)]; break;
			case 2 : divisor += pi_vector[(mp->plelement->GetNumber() - 1)];
				 result  += pi_vector[(mp->plelement->GetNumber() - 1)]; break;
			default: break;
		}
	}
}
//##################################################################################################################
//
//	errechnet mit pi-vektor und Markierungsliste bedingte Erwartungswerte
//
//##################################################################################################################
void	EvaluateBayesE(MarkingPtrList& markings, Cell* c, double& result, double& divisor)
{
	for (MarkingPtrListElement* mp=markings.first; mp; mp=mp->next)
	{
		if (!(mp->plelement)) continue;

		init_global_actual_objects(mp->plelement->pms);

		if (c->CalculateCell2())
		{
			divisor += pi_vector[(mp->plelement->GetNumber() - 1)];
			int n1   = int(c->CalculateCell1());
			result  += n1 * pi_vector[(mp->plelement->GetNumber() - 1)];
		}
	}
}
//##################################################################################################################
//
//	errechnet mit pi-vektor und Markierungsliste die mittlere Schaltfrequanz (through_p) einer Transition
//
//##################################################################################################################
void	EvaluateThroughPut(MarkingPtrList& markings, Cell* c, double& result)
{
	int	t_no	= 0;

	t_no = int(c->GetCell2()->CalculateFloat());
	for (MarkingPtrListElement* mp=markings.first; mp; mp=mp->next)
	{
		if (!(mp->plelement))       continue;
	  	if (!(mp->plelement->first)) continue;

		for (state_transition* st=mp->plelement->first; st; st=st->nextS)
	    	{
	    		if ( !((st->delay.GetTransitionRef()) == t_no) ) continue;
	    		result += st->delay.GetDelayVal() * pi_vector[(mp->plelement->GetNumber() - 1)];
		}

	}
}
//##################################################################################################################
//
//	errechnet mit pi-vektor und Markierungsliste je nach Cellen-Inhalt
//		- Wahrscheinlichkeit 		(prob)
//		- Erwartungswert 		(expect) 
//		- bedingte Wahrscheinlichkeit 	(bayes_p)
//		- bedingter Erwartungswert 	(bayes_e)
//		- mittlere Schaltfrequanz 	(through_p) 
//
//##################################################################################################################
void	EvaluateList(MarkingPtrList& markings, Cell* c, double& divisor, double& result)
{
	switch (c->GetRatingType())
	{
		case bayes_p	: 	EvaluateBayesP		(markings, c, result, divisor);	break;
		case bayes_e	: 	EvaluateBayesE		(markings, c, result, divisor);	break;
		case prob	: 	EvaluateProbability	(markings, c, result);		break;
		case expect	:	EvaluateExpectVal	(markings, c, result);		break;
		case through_p	:	EvaluateThroughPut	(markings, c, result);		break;
		default    	:	break;
	}
}
//##################################################################################################################
//
//	berechnet RatingCellen
//
//##################################################################################################################
double	EvaluateRatingCell(Cell* c)
{
	double divisor = 0.0;
	double result  = 0.0;

	if (m_lists->exp_marking_list.size) 
		EvaluateList(m_lists->exp_marking_list, c, divisor, result);

	if (c->GetRatingType() == through_p) 
		return (result);

	if (m_lists->det_marking_list.size)
	{
		for (ListElementOfSMCLists* dsmcs=m_lists->det_marking_list.first; dsmcs; dsmcs=dsmcs->next)
			if (dsmcs->lelement.det_or_gen_markings.size)
				EvaluateList(dsmcs->lelement.det_or_gen_markings, c, divisor, result);
	}
	if (m_lists->gen_marking_list.size)
	{
		for (ListElementOfSMCLists* gsmcs=m_lists->gen_marking_list.first; gsmcs; gsmcs=gsmcs->next)
			if (gsmcs->lelement.det_or_gen_markings.size)
				EvaluateList(gsmcs->lelement.det_or_gen_markings, c, divisor, result);
	}

	if ( (c->GetRatingType() == bayes_p) || (c->GetRatingType() == bayes_e) ) 
	{
		if (divisor) 	result /= divisor;
		else		result = 0.0;
	}
	return (result);
}
//##################################################################################################################
//
//	sucht RatingCells um diese zustandsbedingt auszurechenen 
//
//##################################################################################################################
double	SubstitudeRatingCells(Cell* c)
{
	if (!c) return NULL;

	BinaerOp	bop;
	UnaerOp		uop;
	Cell*		op1 = NULL;
	Cell*		op2 = NULL;
	double		tmp = -1;

	switch (c->GetCellType())
	{
		case	const_val	: tmp = c->CalculateFloat(); 	  break;
		case	function	: tmp = c->CalculateFloat(); 	  break; 
		case	rating_cell	: return (EvaluateRatingCell(c)); break;

		case	binaer_op	: bop = BinaerOp(
					  		 c->GetOp(),
					  		 SubstitudeRatingCells((op1=c->GetLeftOpd())),
					  		 SubstitudeRatingCells((op2=c->GetRightOpd()))
							);
					  tmp = bop.CalculateFloat(); 
					  bop.Clear();
					  break;

		case	unaer_op	: uop = UnaerOp(
					  		c->GetOp(), 
					  		SubstitudeRatingCells((op1=c->GetOpd()))
					  	       ); 
					  tmp = uop.CalculateFloat(); 
					  uop.Clear();
					  break;

		default			: break;
	}
	if (op1) op1->Clear();
	if (op2) op2->Clear();
	return (tmp);
}
//##################################################################################################################
//
//	trennt "\__ROOT__" vom String "str"
//
//##################################################################################################################
char*	StripOfROOT(char* str)
{
	char* ret_str = NULL;
	char* tmp_str = str;
	if (!str) return "unknown";
	tmp_str++;
	if (! tmp_str) return "unknown";
	if (!(ret_str = index(tmp_str, '\\'))) return "unknown";
	return (ret_str);
}
//##################################################################################################################
//
//	- oeffnet das .RESULT-File 
//	- substituiert RatingCells mit deren Ergebniswert
//	- evaluiert die Ergebnisse der EvalRefList
//	- schreibt die Ergebnisse ins .RESULT-File
//
//##################################################################################################################
void	NetEvaluation(char* netname)
{
	//---------------------------------------------------------------------------------------------------------
	//	.RESULT-File bestimmen und oeffnen
	//---------------------------------------------------------------------------------------------------------
	double 	result		= -1;
	FILE*   result_fp       = NULL;
	char*   result_filename = NULL; 
	sprintf(result_filename = new char[(strlen(ModelDir) + (strlen(netname) + 12))], "%s/%s.RESULTS",ModelDir,netname);

	if ( !(result_fp = fopen(result_filename, "w")) )
		NetEvalError("can't open file", result_filename, __LINE__);

	//---------------------------------------------------------------------------------------------------------
	//	nichts zu berechnen	
	//---------------------------------------------------------------------------------------------------------
	if (!(EvalRefList.size))
	{
		fprintf(result_fp, "nothing to evaluate\n");
		fclose (result_fp);
		return;
	}
	//---------------------------------------------------------------------------------------------------------
	//	Start der Berechnungen	
	//---------------------------------------------------------------------------------------------------------
	for (DefinitionListElement* dle=EvalRefList.first; dle; dle=dle->next)
	{
		result = SubstitudeRatingCells(dle->lelement.GetVal());
		fprintf(result_fp, "\n%s\t=\t%g\n", StripOfROOT(dle->lelement.GetName()), result);
		cout << "!setattribute " << StripOfROOT(dle->lelement.GetName()) <<" val \""<< result <<"\"\n"; 
		result = -1;
	} 
	fclose(result_fp);
}
//##################################################################################################################
#endif
