#ifndef __NonPersistentMarking_cc__
#define __NonPersistentMarking_cc__

#include <iostream.h>

#include "DelayVal.h"
#include "state_transition.h"
#include "token_multiset.h"

#include "NonPersistentMarking.h"

extern int no_of_exc_transitions;
extern int no_of_places;

//######################################################################################################################
//						MARKING
//######################################################################################################################
inline void    NonPersistentMarking::NewPMS(token_multiset** p)
{
	pms = NULL;
	if (!p) return;

	pms = new token_multiset*[no_of_places];
	for (int i=0; i < no_of_places-1; i++)
		pms[i] = copy_token_multiset(p[i]);
}
//--------------------------------------------------------------------------------------------------------------
inline void    NonPersistentMarking::NewATA(short* ata)
{
	active_trans_array = NULL;
	if (ata)
	{
		active_trans_array = new short[(no_of_exc_transitions + 1)];
		for (int i=0; i < no_of_exc_transitions; i++)
			active_trans_array[i] = ata[i]; 
	}
}
//--------------------------------------------------------------------------------------------------------------
inline void 	NonPersistentMarking::CreateKey()
{
        key = 0;
	if (pms)
	{
		for (int i=0; i < no_of_places-1; i++)
			if (pms[i]) key +=((i + 1) * pms[i]->key);
	}

	if (active_trans_array)
	{
		for (int i=0; i < no_of_exc_transitions; i++)
			if (active_trans_array[i])  key *=((i + 1) * active_trans_array[i]); 
	}

	if (key < 0) key  = -key;
}
//######################################################################################################################
NonPersistentMarking::NonPersistentMarking()
:	key			(DEF_KEY),
	np_delay		(),
	active_trans_array	(NULL),
	pms                     (NULL)
	{ first = NULL; }

NonPersistentMarking::NonPersistentMarking(int)
:	key			(DEF_KEY),
	np_delay		(),
	active_trans_array	(NULL),
	pms                     (NULL)
	{ first = NULL; }

NonPersistentMarking::NonPersistentMarking(const NonPersistentMarking& m)
: 	key			(m.key),
	np_delay		(m.np_delay)
  { 
  	NewPMS(m.pms);
  	NewATA(m.active_trans_array);  
  	first = NULL; 
  }

NonPersistentMarking::NonPersistentMarking(token_multiset** p)
: 	np_delay		(),
	active_trans_array	(NULL)
  { 
  	NewPMS(p);
  	CreateKey(); 
  	first = NULL; 
  } 

NonPersistentMarking::NonPersistentMarking(token_multiset** p, const DelayVal& d)
:	key			(DEF_KEY),
	np_delay		(d),
	active_trans_array	(NULL),
	pms                     (NULL)
  { 
  	NewPMS(p);
  	CreateKey(); 
  	first = NULL; 
  } 

NonPersistentMarking::NonPersistentMarking(const DelayVal& d)
:	key			(DEF_KEY),
	np_delay		(d),
	active_trans_array	(NULL),
	pms                     (NULL)
	{ first = NULL; }

NonPersistentMarking::NonPersistentMarking(token_multiset** p, short* ata)
: 	np_delay()
  { 
  	NewPMS(p);
  	NewATA(ata); 
  	CreateKey(); 
  	first = NULL; 
  } 

NonPersistentMarking::NonPersistentMarking(token_multiset** p, const DelayVal& d, short* ata)
: 	np_delay(d)
  { 
  	NewPMS(p);
  	NewATA(ata); 
  	CreateKey();  
  	first = NULL; 
  } 

//######################################################################################################################
inline void	NonPersistentMarking::ClearPMS() 			 	 
{
	if (!pms) return;

	clear_pms(pms, (no_of_places-1));

	pms = NULL;
}
//--------------------------------------------------------------------------------------------------------------
void    NonPersistentMarking::ClearTotal()
{
	if (first)
	{
		for (np_state_transition* h=first; h; h=h->nextS)
		{
			if (h->nextM)
			{
				h->nextM->Clear();
				delete h->nextM;
				h->nextM = NULL;
			}
		}
	}
	Clear();
}
//--------------------------------------------------------------------------------------------------------------
void	NonPersistentMarking::Clear() 			 	 
{
	if (first)
	{
		if (first->nextS)
		{
			for (np_state_transition* h=first->nextS; h; h=h->nextS)
			{
				delete first;
				first = NULL;
				first = h;
			}
			delete first;
			first = NULL;
		}
		else 	
		{
			delete first;
			first = NULL;
		}
	}

	if (active_trans_array) 
	{
		delete[] active_trans_array;
		active_trans_array = NULL;
	}

	ClearPMS(); 

	key = DEF_KEY;
}
//--------------------------------------------------------------------------------------------------------------
DelayVal&		NonPersistentMarking::GetDelay()		    	  { return np_delay;           }
Key			NonPersistentMarking::GetKey()				  { return key; 	       }
void			NonPersistentMarking::SetKey(Key k)			  { key = k;		       }
char*			NonPersistentMarking::GetName()				  { return DEF_STRING;         }
void			NonPersistentMarking::ChangeDelay(const DelayVal& d)   	  { np_delay = d;	       }
bool			NonPersistentMarking::LessThan(NonPersistentMarking& m)	  { return (key < m.key);      }
bool			NonPersistentMarking::operator!=(NonPersistentMarking& m) { return !(*this == m);      }
//--------------------------------------------------------------------------------------------------------------
short*			NonPersistentMarking::GetActiveTransArray()               
{ 
	if (active_trans_array) return active_trans_array; 

	short* ret_array = new short[(no_of_exc_transitions + 1)];
	for (int i=0; i < no_of_exc_transitions; i++)
		ret_array[i] = 0; 
	return ret_array;
}
//--------------------------------------------------------------------------------------------------------------
void	NonPersistentMarking::InitPlaceMultiSet()  
{
        if (pms)
	{
		delete[] pms;
		pms = NULL;
	}
	pms = new token_multiset*[no_of_places];
	for (int i=0; i < no_of_places-1 ; i++)
		pms[i] = NULL;
}
//--------------------------------------------------------------------------------------------------------------
void	NonPersistentMarking::InitActiveTransArray()  
{
        if (active_trans_array)
	{
		delete[] active_trans_array;
		active_trans_array = NULL;
	}
	active_trans_array = new short[no_of_exc_transitions];
	for (int i=0; i < no_of_exc_transitions ; i++)
		active_trans_array[i] = 0;
}
//--------------------------------------------------------------------------------------------------------------
bool	NonPersistentMarking::operator==(NonPersistentMarking& m)	
{ 
	if (key != m.key) return false;

	if (pms && m.pms)
	{
		for (int i=0; i < no_of_places-1; i++)
			if ( !(tms_are_equal(pms[i], m.pms[i])) )
				return false;
	}
	else 	if (!(!pms && !m.pms)) return false;

	if (!active_trans_array && !m.active_trans_array) return true;
	if ( active_trans_array &&  m.active_trans_array) 
	{
		for (int j=0; j < no_of_exc_transitions; j++)
			if (active_trans_array[j] != m.active_trans_array[j])
				return false;
	}
	else    return false;

	return true;
}
//--------------------------------------------------------------------------------------------------------------
NonPersistentMarking&		NonPersistentMarking::operator =(const NonPersistentMarking& m)	
{ 
	Clear(); 

	key      = m.key;
	np_delay = m.np_delay;

	NewPMS(m.pms);
	NewATA(m.active_trans_array);

	return *this; 
}
//--------------------------------------------------------------------------------------------------------------
void	NonPersistentMarking::AppendNewMarking(NonPersistentMarking& m, DelayVal& d)				
{ 
	if (!first)
	{
		first        = new np_state_transition; 
		first->delay = d;
		first->nextM = &m;
		first->nextS = NULL;
		return;
	}
	np_state_transition* new_st = new np_state_transition;

	new_st->delay = d;
	new_st->nextM = &m;
	new_st->nextS = first;
	first         = new_st;
}
//##############################################################################################################
#endif
