#ifndef __PLACE_CC__
#define __PLACE_CC__

#include <iostream.h>

#include "Identifier.h"
#include "Textual.h"

#include "token_multiset.h"
#include "PlaceHeader.h"

#include "Token.h"

#include "CellVal.h"
#include "Cell.h"

#include "Place.h"

//######################################################################################################################
Place::Place(const Place& p)  		        
	: ptype		(p.ptype),	
	  koc		(p.koc), 
	  cap_result	(p.cap_result)
	  { 
	  	if (!(p.p_head)) p_head = NULL;
	  	else 
	  	{
	  		p_head = new PlaceHeader(p.p_head->GetName(), 
	  					 p.p_head->GetPlaceNo(), 
	  					 copy_token_multiset(p.p_head->Gettoken_multiset())); 	    
		}

	  	if (p.cap)    	cap    = p.cap->Copy(); 	    
	  	else 		cap    = new ConstVal();
	  } 
//----------------------------------------------------------------------------------------------------------------------
Place::Place()   			  	
	: ptype		(noplacetype),	
	  koc		(nokind),
	  cap_result	(DEF_PLACE_CAP)
	  { 
	 	p_head = NULL;
	  	cap    = new ConstVal();
	  } 
//----------------------------------------------------------------------------------------------------------------------
Place::Place(int)               		
	: ptype		(noplacetype),	
	  koc		(nokind), 
	  cap_result	(DEF_PLACE_CAP)
	  { 
	 	p_head = NULL;
	  	cap    = new ConstVal(); 
	  } 
//----------------------------------------------------------------------------------------------------------------------
Place::Place(char* n, int no, PlaceType pt, KindOfCap kc, const Cell* c)
	: ptype 	(pt),
	  koc		(kc),
	  cap_result	(DEF_PLACE_CAP)
	{
	  	p_head = new PlaceHeader(n, no);
	  	if (c) cap = c->Copy(); 
	  	else   cap = new ConstVal();
	}
//----------------------------------------------------------------------------------------------------------------------
Place::Place(char* n, int no, KindOfCap kc, const Cell* c)
	: ptype 	(noplacetype),
	  koc		(kc),
	  cap_result	(DEF_PLACE_CAP)
	{
	  	p_head = new PlaceHeader(n, no);
	  	if (c) cap = c->Copy(); 
	  	else   cap = new ConstVal();
	}
//----------------------------------------------------------------------------------------------------------------------
Place::Place(char* n, int no, PlaceType pt, KindOfCap kc, const Cell* c, token_multiset* t)
	: ptype 	(pt),
	  koc		(kc),
	  cap_result	(DEF_PLACE_CAP)
	{
	  	p_head = new PlaceHeader(n, no, (copy_token_multiset(t)));
	  	if (c) cap = c->Copy(); 
	  	else   cap = new ConstVal();
	}
//----------------------------------------------------------------------------------------------------------------------
Place::Place(char* n, int no, KindOfCap kc, const Cell* c, token_multiset* t) 
	: ptype 	(noplacetype),
	  koc		(kc),
	  cap_result	(DEF_PLACE_CAP)
	{
	  	p_head = new PlaceHeader(n, no, (copy_token_multiset(t))); 
	  	if (c) cap = c->Copy(); 
	  	else   cap = new ConstVal();
	}
//#######################################################################################################################
PlaceHeader*	Place::GetPlaceHeader()				{ return p_head;					}
PlaceType	Place::GetPlaceType()				{ return ptype;						}
KindOfCap	Place::GetKindOfCap()				{ return koc;  						}
Cell*&	 	Place::GetCapacity()	 			{ return cap;  						}
bool		Place::LessThan(Place&) 			{ return false;    					}
bool		Place::operator!=(Place& p) 			{ return !( *this == p );				}
void		Place::ChangePlaceType(PlaceType pt)		{ ptype = pt; 						}
void		Place::ChangeKindOfCap(KindOfCap k)		{ koc   = k; 						}
void    	Place::Changetoken_multiset(token_multiset* t)  { if (p_head) p_head->Changetoken_multiset(t); 		}
//-----------------------------------------------------------------------------------------------------------------------
void    	Place::ChangeCapacity(const Cell* c) 	  	
{ 
	if (c && cap) 
	{ 
		cap->Clear();
		delete cap;  
		cap = c->Copy(); 
	}
}
//-----------------------------------------------------------------------------------------------------------------------
void		Place::Clear() 					
{ 
	if (cap)  	
	{ 
		cap->Clear();
		delete cap;  
		cap        = NULL; 
		cap_result = DEF_PLACE_CAP; 	
	}	
	p_head = NULL; 
}
//----------------------------------------------------------------------------------------------------------------------
char* 		Place::GetName() 	 			
{ 
	if (p_head) return (p_head->GetName()); 
	return NULL; 
}
//----------------------------------------------------------------------------------------------------------------------
token_multiset* Place::Gettoken_multiset()  	
{ 
	if (p_head) return (p_head->Gettoken_multiset()); 
	return NULL; 
}
//----------------------------------------------------------------------------------------------------------------------
int    		Place::GetPlaceNo() 	 	
{ 
	if (p_head) return (p_head->GetPlaceNo());  
	return 0; 
}		
//----------------------------------------------------------------------------------------------------------------------
Textual		Place::GetPlaceName() 	 	
{ 
	if (p_head) return p_head->GetPlaceName();  
	return (*(new Textual(DEF_TXT_NAME))); 
}		
//----------------------------------------------------------------------------------------------------------------------
bool		Place::operator==(Place& p)      	
{ 
	if ( p_head && !p.p_head)	return false;
	if (!p_head &&  p.p_head)	return false;
	if (!p_head && !p.p_head)	return true;

	return ( ptype==p.ptype && *p_head==*(p.p_head) );
}
//----------------------------------------------------------------------------------------------------------------------
Place&	Place::operator =(const Place& p)
{
	Clear();

	ptype      = p.ptype;
	koc        = p.koc;
	cap_result = p.cap_result;

	if (p.cap)       cap    = p.cap->Copy();  
	else 		 cap    = new ConstVal();

	if (!(p.p_head)) p_head = NULL;
	else 
	{
	 	p_head = new PlaceHeader(p.p_head->GetName(), 
	  				 p.p_head->GetPlaceNo(), 
	  				 copy_token_multiset(p.p_head->Gettoken_multiset())); 	    
	}
	return *this;
}
//----------------------------------------------------------------------------------------------------------------------
bool    Place::ColorConsistent(int color)
{
	if (color == EMPTY_COLOR) return true;

	switch (ptype)
	{
		case elementary : if (color != NO_COLOR) return false; break;
		case object     : if (color == NO_COLOR) return false; break; 
		default         : PlaceError("ColorConsistent() : placetype is undefined");
	}
	return true;  
}
//----------------------------------------------------------------------------------------------------------------------
bool    Place::TMSIsInPlace(token_multiset* tms)
{
	if (!tms || !(tms->first)) return true;
	if (!p_head)		   PlaceError("TMSIsInPlace() : empty place-tms");

	token_multiset* place_tms = p_head->Gettoken_multiset();

	if (!(place_tms) || !(place_tms->first)) 	 return false;
	if (get_quantity(place_tms) < get_quantity(tms)) return false;

	for (token_referenz* tr=tms->first; tr; tr=tr->next)
		if (get_tref_quantity(place_tms, tr->tref) < tr->quant) 
			return false;
	return true;
}
//----------------------------------------------------------------------------------------------------------------------
bool    Place::EnoughCapacity()
{
	int tmp_res = DEF_PLACE_CAP;

	if (cap_result != DEF_PLACE_CAP) 
		tmp_res = cap_result;
	else
	{
		CellVal* cv  = cap->Calculate();
		double   val = cv->GetFloat();
		cv->Clear(); delete cv;

		tmp_res = int(val);
		if (!(cap->IsPlaceDependent())) 
			cap_result = tmp_res;
	}

	switch (koc)
	{
		case logic       : return (tmp_res == 0?false:true); 
		case quantitativ : if (tmp_res == UNFINITE)      return true; 
				   if (tmp_res <= DEF_PLACE_CAP) return false;
				   if (p_head) 
			   		return (tmp_res >= (get_quantity(p_head->Gettoken_multiset())));
				   return true; 
		default          : return false; 
	}
	return false;
}
//----------------------------------------------------------------------------------------------------------------------
bool    Place::EnoughCapacity(int tms_size)
{
	int tmp_res = DEF_PLACE_CAP;

	if (cap_result != DEF_PLACE_CAP) 
		tmp_res = cap_result;
	else
	{
		CellVal* cv  = cap->Calculate();
		double   val = cv->GetFloat();
		cv->Clear(); delete cv;

		tmp_res = int(val);
		if (!(cap->IsPlaceDependent())) 
			cap_result = tmp_res;
	}

	switch (koc)
	{
		case logic       : return (tmp_res == 0?false:true); 
		case quantitativ : if (tmp_res == UNFINITE) return true; 
				   if (p_head) 
			   		return (tmp_res >= (get_quantity(p_head->Gettoken_multiset()) + tms_size));
				   return (tmp_res >= tms_size);
		default          : return false; 
	}
	return false;
}
//----------------------------------------------------------------------------------------------------------------------
void	Place::PlaceError(char* s)
{
	cout <<"\n";
	cout <<"PlaceError \n";
	cout <<s<<"\n";
	cout <<"\n";
	exit(-1);
}
//#######################################################################################################################
#endif 

