#ifndef __PTRLIST_CC__
#define __PTRLIST_CC__

#include <iostream.h>
#include "PtrList.h"

//##############################################################################################
//			PTRLISTELEMENT
//##############################################################################################
template<class T> inline  PtrListElement<T>::PtrListElement() 
{
	plelement = NULL;
	next 	  = NULL;
}

template<class T> inline  PtrListElement<T>::PtrListElement(T* le) 
{ 
	if (le) plelement = le; 
	else    plelement = NULL;
	next = NULL;
}

template<class T> inline  PtrListElement<T>::PtrListElement(T* le, PtrListElement<T>* n) 
{ 
	if (n && n->plelement) next = new PtrListElement<T>((n->plelement));
	else		       next = NULL;
	plelement = le; 
}

template<class T> inline  PtrListElement<T>::PtrListElement(const PtrListElement<T>& le)
{ 
	if (le.next && le.next->plelement) next = new PtrListElement<T>((le.next->plelement));
	else				   next = NULL;
	plelement = le.plelement; 
}
//################################################################################################
template<class T> inline  PtrListElement<T>& PtrListElement<T>::operator =(const PtrListElement<T>& le)
{
	plelement = le.plelement;
	next      = NULL;
	return *this;
}

//################################################################################################
//				LIST
//################################################################################################
template<class T> inline  PtrList<T>::PtrList()   
{
	first = NULL; 
	last  = NULL;
	size  = 0;
}

template<class T> inline  PtrList<T>::PtrList(T* le) 
{ 
	first = new PtrListElement<T>(le); 
	last  = first;
	size  = 1;
}

template<class T> inline  PtrList<T>::PtrList(const PtrList<T>& l) 
{
	first = NULL;
	last  = NULL;
	size  = 0;
	if (l.first) 
	{
		first = new PtrListElement<T>((l.first->plelement));
		last  = first;
		if (l.first->next)
		{
			for (PtrListElement<T>* h=l.first->next; h; last=last->next, h=h->next)
	    			last->next = new PtrListElement<T>((h->plelement));
		}
		size = l.size;
	}
}

template<class T> inline  PtrList<T>::PtrList(const PtrList<T>& l1, const PtrList<T>& l2) 
{
	first = NULL;
	last  = NULL;
	size  = 0;
	if (l1.first)
	{
		first = new PtrListElement<T>((l1.first->plelement));
		last  = first;

		if (l1.first->next)
		{
			for (PtrListElement<T>* h1 = l1.first->next; h1; last=last->next,h1=h1->next)
	    			last->next = new PtrListElement<T>((h1->plelement));
		}
	}
	if (l2.first)
	{
		if (!first)
		{
			first = new PtrListElement<T>((l2.first->plelement));
			last  = first;

			if (l2.first->next)
			{
				for (PtrListElement<T>* h2 = l2.first->next; h2; last=last->next, h2=h2->next)
	    				last->next = new PtrListElement<T>((h2->plelement));
			}
		}
		else
		{
			for (PtrListElement<T>* h3 = l2.first; h3; last=last->next, h3=h3->next)
	    			last->next = new PtrListElement<T>((h3->plelement));
		}
	}
	size = l1.size + l2.size;
}

//----------------------------------------------------------------------------
template<class T> inline  void PtrList<T>::Clear()
{
	PtrListElement<T>* next;
	for (;size;size--)
	{
		if (!first) break;
		if (!first->next) 
		{
			if (first->plelement)
			{
				first->plelement->Clear();
				delete first->plelement;
				first->plelement = NULL;
			}
			delete first;
			first = NULL;
			break;
		}
		next = first->next; 
		if (first->plelement)
		{
			first->plelement->Clear();
			delete first->plelement;
			first->plelement = NULL;
		}
		delete first;
		first = NULL;
     		first = next;
     	}
	first = NULL;
	last  = NULL;
	size  = 0;
}
//----------------------------------------------------------------------------
template<class T> inline  int PtrList<T>::Append(T* e)
{
	if (first)
	{
		if (!size) 
		{
			delete first;
			first = new PtrListElement<T>(e);
                        last  = first;
                        size  = 1;
		}
		else
		{
			last->next = new PtrListElement<T>(e);
			last       = last->next;
			size++;
		}
	}
	else
	{
		first = new PtrListElement<T>(e);
		last  = first;
		size  = 1;
	}
	return size;
}
//----------------------------------------------------------------------------
template<class T> inline  int PtrList<T>::UniqueAppend(T* e)
{
	if (!first || !size) return Append(e);

	int pos = 1;
	for (PtrListElement<T>* h=first; h; h=h->next, pos++)
		if (*(h->plelement) == *e) return -pos;
	return Append(e);
}
//----------------------------------------------------------------------------
template<class T> inline  T* PtrList<T>::UniqueRefAppend(T* e)
{
	if (!first || !size) 
	{
		Append(e);
		return first->plelement;
	}	
	int pos = 1;
	for (PtrListElement<T>* h=first; h; h=h->next, pos++)
		if (*(h->plelement) == *e) return NULL;
	Append(e);
	return last->plelement;
}
//----------------------------------------------------------------------------
template<class T> inline  T* PtrList<T>::UniqueRefAppendInvers(T* e)
{
	if (!first || !size) 
	{
		Append(e);
		return NULL;
	}	
	int pos = 1;
	for (PtrListElement<T>* h=first; h; h=h->next, pos++)
		if (*(h->plelement) == *e) return h->plelement;
	Append(e);
	return NULL; 
}
//----------------------------------------------------------------------------
template<class T> inline  int PtrList<T>::DeleteElement(int pos)
{
	if (pos < 1 || pos > size || !first ) return LIST_FAULT_VAL;

	if (pos == 1)
	{
		if (first->next) 
		{
			PtrListElement<T>* h = first;
			first                = first->next;
			delete h;
			size--;
		}
		else             
		{
			delete first;
			first = new PtrListElement<T>();
			last  = first;
			size  = 0;
		}
		return  size;
	}

	PtrListElement<T>* h0=first;
	PtrListElement<T>* h1=first;
	for (int i=1; i<pos; i++) 
	{
		h0 = h1;
		h1 = h1->next;
	}
	if (pos == size) last = h0;
	h0->next = h1->next;
	delete h1;
	size--;
	return size;
}
//----------------------------------------------------------------------------
template<class T> inline  int PtrList<T>::DeleteElement(T* e)
{
	if (!first) return LIST_FAULT_VAL;

	if (*(first->plelement) == *e)
	{
		if (first->next) 
		{
		        PtrListElement<T>* h=first;
			first = first->next;
			delete h;
			size--;
		}
		else             
		{
			delete first;
			first = new PtrListElement<T>();
			last  = first;
			size  = 0;
		}
		return size;
	}

	int pos = 1;
	for (PtrListElement<T>* h0=first, *h1=first; h1; h0=h1, h1=h1->next, pos++)
	{
		if (*(h1->plelement) == *e)
		{	
			if (h1 == last) last = h0;
			h0->next = h1->next;
			delete h1;
			size--;
			return size;
		}
	}
	return LIST_FAULT_VAL;
}
//----------------------------------------------------------------------------
template<class T> inline  int PtrList<T>::ChangeElement(int pos, T* new_e)
{
	if (pos > size || pos < 1 || !first ) return LIST_FAULT_VAL;

   	int		   i=1;
   	PtrListElement<T>* h=first; 

   	for (; i<pos; h=h->next, i++) ;
	h->plelement=new_e;
	return i; 
}
//----------------------------------------------------------------------------
template<class T> inline  int PtrList<T>::ChangeElement(T* old_e, T* new_e)
{
	if (!first) return LIST_FAULT_VAL;

	int pos=1;
   	for (PtrListElement<T>* h=first; h ; h=h->next, pos++) 
	{
		if (*(h->plelement) == *old_e)
		{
			h->plelement=new_e;
			return pos;
		}
	}
	return LIST_FAULT_VAL;
}
//----------------------------------------------------------------------------
template<class T> inline  int PtrList<T>::GetPos(T* e) 
{
	if (!first) return LIST_FAULT_VAL;

	int	pos=1;
	for (PtrListElement<T>* h=first; h; h=h->next, pos++)
		if (*(h->plelement) == *e) return pos;
	return LIST_FAULT_VAL;
}
//----------------------------------------------------------------------------
template<class T> inline  T* PtrList<T>::GetElement(T* le) 
{
	if (!first) return NULL;

	for (PtrListElement<T>* h=first; h; h=h->next)
       	    if (*(h->plelement) == *le)  return h->plelement;
	return NULL;
}
//----------------------------------------------------------------------------
template<class T> inline  T* PtrList<T>::GetElement(char* le_name) 
{
	if (!first) return NULL;

	for (PtrListElement<T>* h=first; h; h=h->next)
       	    if (strcmp(h->plelement->GetName(), le_name) == 0)  
       	    		return h->plelement;
	return NULL;
}
//----------------------------------------------------------------------------
template<class T> inline  T* PtrList<T>::GetElement(int pos) 
{
	if (pos < 1 || pos > size || !first )  return NULL;

       	int i=1;
	for (PtrListElement<T>* h=first; h; i++, h=h->next)
		if (i==pos) return h->plelement;
       	return NULL; 
}
//----------------------------------------------------------------------------
template<class T> inline  bool PtrList<T>::IsIn(T* le) 
{
	if (!size || !first) return false;

	for (PtrListElement<T>* h=first; h; h=h->next)
       	    if (*(h->plelement) == *le)  return true;
	return false;
}
//----------------------------------------------------------------------------
template<class T> inline  bool PtrList<T>::IsIn(char* le_name) 
{
	if (!size || !first) return false;

	for (PtrListElement<T>* h=first; h; h=h->next)
       	    if (strcmp(h->plelement->GetName(), le_name) == 0)  return true;
	return false;
}
//----------------------------------------------------------------------------
template<class T> inline  PtrList<T>& PtrList<T>::operator =(const PtrList<T>& list)
{ 	
	Clear(); 
	first = new PtrListElement<T>();
	last  = first;
	if (list.first) 
	{
	        first->plelement = list.first->plelement;
		last             = first;

		if (list.first->next)
		{
			for (PtrListElement<T>* h=list.first->next; h; last=last->next, h=h->next)
	    			last->next = new PtrListElement<T>((h->plelement));
		}
		size = list.size;
	}
	return *this;
}
//----------------------------------------------------------------------------
template<class T> inline  bool PtrList<T>::operator==(PtrList<T>& list) 
{
	PtrList tmp(list);
	int hit=0;

	if ( list.size !=  size) return false;
	if (!list.size && !size) return true;

	for (PtrListElement<T>* h=first; h ; h=h->next) 
	{
		for (PtrListElement<T>* h1=tmp.first; h1; h1=h1->next)
	           if (*(h->plelement) == *(h1->plelement)) 
	           { 
	           	tmp.DeleteElement(h->plelement); 
	            	hit++; 
	            	break; 
	           }
        }
	if (hit == size) return true;
	return false;
}
//----------------------------------------------------------------------------
template<class T> inline  bool PtrList<T>::operator!=(PtrList<T>& list)
{
	return !(*this == list);
}
//############################################################################
#include "Textual.h"
#include "DelayVal.h"
#include "token_multiset.h"
#include "PlaceHeader.h"
#include "state_transition.h"
#include "Marking.h"
#include "NonPersistentMarking.h"
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
template class PtrListElement<PlaceHeader>;
template class PtrListElement<Marking>;
template class PtrListElement<NonPersistentMarking>;
//----------------------------------------------------------------------------
template class PtrList<PlaceHeader>;
template class PtrList<Marking>;
template class PtrList<NonPersistentMarking>;
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#include "Identifier.h"
#include "TextualList.h"
#include "Token.h"
#include "CellVal.h"
#include "Cell.h"
#include "Place.h"
#include "Delay.h"
#include "BindingElement.h"
#include "BindingList.h"
#include "Transition.h"
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
template class PtrListElement<Place>;
template class PtrListElement<Transition>;
//----------------------------------------------------------------------------
template class PtrList<Place>;
template class PtrList<Transition>;
//############################################################################
#endif
