#ifndef __RRGAlgo_cc__
#define __RRGAlgo_cc__

#include "inc_classes.h"
#include "inc_rrg_algo.h"

void	RRGAlgoFunction	(Marking*&, Marking*&, DelayVal&);

//#################################################################################################################
//
//      Fehlerbehandlung
//
//#################################################################################################################
void    RRGAlgoErr(char* s, int line)
{
	RRGErr = new RRGError(__FILE__, s, line);
	PrepareEnd();
}
//####################################################################################################################
//
//	Sofern der letzte Uebergang (last) ein Exponetieller war und der jetztige ein Sofortiger (imm/prob) 
//	ist, wird das entsprechenede exponentielle delay mit der Uebergangswahrscheinlichkeit des reduzierten, 
//	gerade behandelten Immediate/Probability-Uebergangs multipliziert.
//
//####################################################################################################################
void	CheckAndCorrectDelayVal(DelayVal& last, DelayVal& now)
{
	if ((int(last.delay_typ) == _EXPONENTIAL) && 
	   ((int(now.delay_typ)  == _PROBABILITY) || (int(now.delay_typ) == _IMMEDIATE)) ) 
	{
		double	  dv1 = last.GetDelayVal() * now.GetDelayVal();
		now.delay_typ = last.delay_typ;
		now.ChangeDelayVal     (dv1);
		now.ChangeTransitionRef(last.GetTransitionRef());
		return;
	}
	if (((int(last.delay_typ) == _PROBABILITY) || (int(last.delay_typ) == _IMMEDIATE)) &&
	    ((int(now.delay_typ)  == _PROBABILITY) || (int(now.delay_typ)  == _IMMEDIATE)) )
	{
		double	dv2   = last.GetDelayVal() * now.GetDelayVal();
		now.delay_typ = last.delay_typ;
		now.ChangeDelayVal     (dv2);
		now.ChangeTransitionRef(last.GetTransitionRef());
		return;
	}
}
//####################################################################################################################
//
//	Sofern der letzte Uebergang ein Deterministischer oder ein Uebergang mit einer "generellen" Schaltverteilung 
//	war, darf "marking" nicht "vergessen" werden. Vielmehr muss "marking" als "saved_marking" gespeichert werden 
//	und die neuen Uebergaenge muessen an ihn angehaengt werden.  
//
//####################################################################################################################
void	SaveDetOrGenMarkings(Marking& start_mark)
{
	bool	 exists=false;
	Marking* saved_marking;

	SaveAndAppendMarking(saved_marking, exists);

	for (state_transition* s = start_mark.first; s; s=s->nextS)
		RRGAlgoFunction(s->nextM, saved_marking, s->delay);
}
//####################################################################################################################
//
//  	Die entstandenen andauernden Zustaende werden im Hash als tangible_marking gespeichert und an last_marking 
//	angehaengt. Die is- und die es-transitionen werden auf Schaltfaehigkeit in tangible_marking  ueberprueft und 
//	ggf. geschaltet. Es entstehen ggf. neue Zustaende.  
//
//####################################################################################################################
void	HandleActivatedMarking()
{
	bool	 exists=false;
	Marking* saved_marking;

	SaveAndAppendMarking(saved_marking, exists);

	marking_and_delay_list  new_mds; 

	init_mdl(new_mds);
	new_marks_and_delays = NULL; 
	new_marks_and_delays = &(new_mds);

	if (!exists) HandlePersistentMarking(); 

	if (new_mds.first)
	{
		//----------------------------------------------------------------------------------------------------
		//  	Die durch das Schalten der is/es-transitionen in der  Funktion  HandlePersistentMarking() 
		//	entstandenen Folgemarkierungen werden nach gewohnten Muster bearbeitet.
		//----------------------------------------------------------------------------------------------------
		for (m_element* new_md = new_mds.first; new_md; new_md=new_md->next)
			RRGAlgoFunction(new_md->marking, saved_marking, new_md->delay); 

		clear_mdl(new_mds);
	}
}
//####################################################################################################################
//
//	RRG-Algorithmus-Funktion
//
//####################################################################################################################
void	RRGAlgoFunction(Marking*& new_m, Marking*& last_m, DelayVal& last_d)
{
	//------------------------------------------------------------------------------------------------------------
	// 	Es werden alle immediate-Transitionen auf Schaltfaehigkeit in new_marking ueberprueft. Ist keine 
	//	imm-transition schaltfaehig, so handelt es sich bei new_marking um einen andauernden Zustand oder 
	//	einen Zwischenzustand. Sind imm-transitionen Schaltfaehig so werden alle Uebergaenge geschaltet 
	//	bis keine imm-transition mehr schaltfaehig ist. Der entstandene Subgraph wird reduziert und alle 
	//	andauernden Zustaende bzw. Zwischenzustaende direkt an imm_start_mark angehaengt.
	//------------------------------------------------------------------------------------------------------------

	if (!new_m)  RRGAlgoErr("new marking is NULL-pointer",  __LINE__);
	if (!last_m) RRGAlgoErr("last marking is NULL-pointer", __LINE__);

	Marking imm_start_mark(new_m->pms, new_m->GetActiveTransArray());
	
	HandleImmTransitions(imm_start_mark);

	if (imm_start_mark.first)
	{
		if ((int(last_d.delay_typ) == _DETERMINISTIC) || (int(last_d.delay_typ) == _GENERAL))
		{
			last_marking = NULL; last_marking = last_m;
			new_marking  = NULL; new_marking  = new_m;
			new_delay    = NULL; new_delay    = &(last_d);

			SaveDetOrGenMarkings(imm_start_mark);
		}
		else
		{
			//--------------------------------------------------------------------------------------------
			//	imm_start_mark war kein andauernder Zustand. Die aus imm_start_mark entstandenen 
			//	(Zwischen-) Zustaende sind durch Zustandsuebergaenge an imm_start_mark angehaengt.
			//--------------------------------------------------------------------------------------------
			for (state_transition* s=imm_start_mark.first; s; s=s->nextS)
			{

				if (!(s->nextM)) continue;

				CheckAndCorrectDelayVal(last_d, s->delay);

				//------------------------------------------------------------------------------------
				//	Es wird ueberprueft ob die durch das Schalten der imm-Transitionen enstandenen
				//	Zustaende Zwischenzustaende sind. In diesem Fall werden diese durch die
				//	Funktion HandleExcTransitions() aktiviert, also andauernde Zustaende erzeugt.
				//	Dabei wird aehnlich wie bei der Funktion HandleImmTransitions() ein Subgraph
				//	aufgebaut, reduziert und die daraus entstandenen andauernden Zaustaende an
				//	exc_start_mark angehaengt. Der SubGraph ist jedoch im Gegensatz zu dem in
				// 	HandleImmTransitions() enstandenen ein linearer SubGraph, da keine Multisets
				//	veraendert werden. Vielmehr wird nur das active_trans_array veraendert.
				//------------------------------------------------------------------------------------

				Marking exc_start_mark(s->nextM->pms, s->nextM->GetActiveTransArray());

				HandleExcTransitions(exc_start_mark);

				if (exc_start_mark.first)
				{
					//----------------------------------------------------------------------------
					//	exc_start_mark war ein Zwischenzustand. Die aus exc_start_mark ent-
					//	standenen Zustaende sind andauernde Zustaende und durch Zustandsueber-
					//	gaenge an exc_start_mark angehaengt.
					//-----------------------------------------------------------------------------
					for (state_transition* st=exc_start_mark.first; st; st=st->nextS)
					{
						if (!(st->nextM)) continue;

						CheckAndCorrectDelayVal(s->delay, st->delay);

						last_marking = NULL; last_marking = last_m;
						new_marking  = NULL; new_marking  = st->nextM;
						new_delay    = NULL; new_delay    = &(st->delay);

						HandleActivatedMarking();
					}
				}
				else 	
				{
					last_marking = NULL; last_marking = last_m;
					new_marking  = NULL; new_marking  = s->nextM;
					new_delay    = NULL; new_delay    = &(s->delay);

					HandleActivatedMarking();	
				}
				exc_start_mark.ClearTotal();
			}
		}
	}
	else 
	{
		//----------------------------------------------------------------------------------------------------
		//	In imm_start_mark war keine imm-Transition schaltbar. Folglich ist imm_start_mark ein 
		//	(Zwischen-) Zustand. Es wird ueberprueft ob imm_start_mark ein Zwischenzustand ist. In diesem 
		//	Fall wird imm_start_mark durch die Funktion HandleExcTransitions() aktiviert, also andauernde 
		//	Zustaende erzeugt. Dabei wird aehnlich wie bei der Funktion HandleImmTransitions() ein 
		//	Subgraph aufgebaut, reduziert und die daraus entstandenen andauernden Zaustaende an
		//	exc_start_mark angehaengt. Der SubGraph ist jedoch im Gegensatz zu dem in HandleImmTransitions() 
		//	enstandenen ein linearer SubGraph, da keine Multisets veraendert werden. Vielmehr wird nur das
		//	active_trans_array veraendert.
		//----------------------------------------------------------------------------------------------------

		Marking exc_start_mark(imm_start_mark.pms, imm_start_mark.GetActiveTransArray());

		HandleExcTransitions(exc_start_mark);

		if (exc_start_mark.first)
		{
			if ((int(last_d.delay_typ) == _DETERMINISTIC) || (int(last_d.delay_typ) == _GENERAL))
			{
				last_marking = NULL; last_marking = last_m;
				new_marking  = NULL; new_marking  = new_m;
				new_delay    = NULL; new_delay    = &(last_d);

				SaveDetOrGenMarkings(exc_start_mark);
			}
			else
			{
				//-----------------------------------------------------------------------------------
				//	exc_start_mark war ein Zwischenzustand. Die aus exc_start_mark entstandenen 
				//	Zustaende sind andauernde Zustaende und durch Zustandsuebergaenge an 
				//	exc_start_mark angehaengt.
				//------------------------------------------------------------------------------------
				for (state_transition* st=exc_start_mark.first; st; st=st->nextS)
				{
					if (!(st->nextM)) continue;

					CheckAndCorrectDelayVal(last_d, st->delay);

					last_marking = NULL; last_marking = last_m;
					new_marking  = NULL; new_marking  = st->nextM;
					new_delay    = NULL; new_delay    = &(st->delay);

					HandleActivatedMarking();
				}
			}
		}
		else 	
		{
			last_marking = NULL; last_marking = last_m;
			new_marking  = NULL; new_marking  = &(exc_start_mark);
			new_delay    = NULL; new_delay    = &(last_d);

			HandleActivatedMarking();	
		}
		exc_start_mark.ClearTotal();
	}
	imm_start_mark.ClearTotal();
}

//####################################################################################################################
//
//	eigentlicher RRG-Algorithmus
//
//####################################################################################################################
void	RRGAlgo(Marking*& initial_marking)
{
	Marking*	tmp_marking;
	DelayVal	tmp_delay;

	//------------------------------------------------------------------------------------------------------------
	//	Initiale Markierung und Stellenkapazitaeten ueberpruefen
	//------------------------------------------------------------------------------------------------------------
	CheckInitialMarkingAndPlaceCapacities();

	//------------------------------------------------------------------------------------------------------------
	//	initiale Markierung bilden 
	//------------------------------------------------------------------------------------------------------------
	initial_marking = new Marking(make_place_multiset());
	tmp_marking 	= new Marking(make_place_multiset());
	RRGAlgoFunction(initial_marking, tmp_marking, tmp_delay);

	if (SHOW_HASH) ShowHashDistribution();

	initial_marking->Clear();
	delete initial_marking;

	if (!(tmp_marking->first)) 		 { initial_marking = tmp_marking; 		 return; }
	if (!(tmp_marking->first->nextS))	 { initial_marking = tmp_marking->first->nextM;  return; }

	initial_marking = tmp_marking;

}
//###################################################################################################################
#endif

