#ifndef __SMCLISTS_CC__
#define __SMCLISTS_CC__

#include "DelayVal.h"
#include "token_multiset.h"
#include "state_transition.h"
#include "Marking.h"
#include "MarkingPtrList.h"

#include "SMCLists.h"
//##############################################################################################################
SMCLists::SMCLists() 			: transition_ref(DEF_TRANS_REF), 	
					  no_states(0),
					  first_state_no(-1),
					  no_preempting_arcs(0),
					  det_or_gen_markings()		     	{};

SMCLists::SMCLists(int tref) 		: transition_ref(tref),			
					  no_states(0),
					  first_state_no(-1),
					  no_preempting_arcs(0),
					  det_or_gen_markings() 	     	{};

SMCLists::SMCLists(const SMCLists& n)	: transition_ref(n.transition_ref),	
					  no_states(n.no_states),
					  first_state_no(n.first_state_no),
					  no_preempting_arcs(n.no_preempting_arcs),
					  det_or_gen_markings(n.det_or_gen_markings) 		{};
//##############################################################################################################
void	SMCLists::Clear()			
{ 
	det_or_gen_markings.Clear(); 
	transition_ref     = DEF_TRANS_REF;  
	no_preempting_arcs = 0;
	first_state_no 	   = -1;
	no_states          = 0;
}
//-------------------------------------------------------------------------------------------------------------
double	SMCLists::GetDetOrGenDelay()		
{ 
	if (det_or_gen_markings.size)
		return (det_or_gen_markings.first->plelement->GetDetOrGenDelay());
	return 0.0;
}
//-------------------------------------------------------------------------------------------------------------
int	SMCLists::GetFirstStateNumber()		{ return first_state_no;     }
int	SMCLists::GetTransitionRef()		{ return transition_ref;     }
int  	SMCLists::GetNoOfStates()       	{ return no_states; 	     }
int  	SMCLists::GetNoOfPreemptingArcs()       { return no_preempting_arcs; }
void	SMCLists::SetNoOfStates(int n)  	{ no_states          = n;    }
void	SMCLists::SetFirstStateNumber(int n)  	{ first_state_no     = n;    }
void	SMCLists::SetNoOfPreemptingArcs(int n)  { no_preempting_arcs = n;    }
void	SMCLists::ChangeTransitionRef(int tref)	{ transition_ref     = tref; }
bool	SMCLists::LessThan(SMCLists&)		{ return false;		     }
char*	SMCLists::GetName()			{ return DEF_STRING; 	     }
bool	SMCLists::operator!=(SMCLists& n)	{ return !( *this == n );    }
//-------------------------------------------------------------------------------------------------------------
bool	SMCLists::operator==(SMCLists& n)
{
	return (transition_ref     == n.transition_ref     && 
		no_states 	   == n.no_states          && 
		first_state_no     == n.first_state_no); 
/*
		no_preempting_arcs  == n.no_preempting_arcs && 
		det_or_gen_markings == n.det_or_gen_markings);
*/
}
//-------------------------------------------------------------------------------------------------------------
SMCLists&	SMCLists::operator =(const SMCLists& n)
{
	Clear();
	transition_ref      = n.transition_ref;
	no_preempting_arcs  = n.no_preempting_arcs;
	first_state_no      = n.first_state_no;
	no_states           = n.no_states;
	det_or_gen_markings = n.det_or_gen_markings;
	return *this;
}
//##############################################################################################################
#endif

