#ifndef __SaveMarking_cc__
#define __SaveMarking_cc__

#include "inc_classes.h"

#define INTERVAL	      500
#define MARKING_HASH_SIZE     4999              // Default-Map-Groesse des Marking-Hash
#define MARKING_IMMHASH_SIZE  50               // Default-Map-Groesse des Marking-ImmHash

extern  Marking*                	last_marking; 	// zuletzt betrachteten Markierung  (inc_rrg_algo.h)
extern  Marking*			new_marking;	// abzuspeichernde, neue Markierung (inc_rrg_algo.h)
extern  DelayVal*			new_delay;	// abzuspeicherndes, neues Delay    (inc_rrg_algo.h)
extern 	MarkingList**			Hash;
extern 	NonPersistentMarkingList**	ImmHash;
extern 	int				marking_counter;

int	interval			= INTERVAL;

//###################################################################################################
//
//	Hashs kreieren und initialisieren
//
//###################################################################################################
void	CreateAndInitHash()
{
	Hash = new MarkingList*[MARKING_HASH_SIZE];

	for (int i=0; i < MARKING_HASH_SIZE; i++)
		Hash[i] = NULL;
}
//---------------------------------------------------------------------------------------------------
void 	CreateAndInitImmHash()
{
	ImmHash = new NonPersistentMarkingList*[MARKING_IMMHASH_SIZE];

	for (int i=0; i < MARKING_IMMHASH_SIZE; i++)
		ImmHash[i] = NULL;
}
//---------------------------------------------------------------------------------------------------
void	CreateAndInitHashs()
{
	CreateAndInitHash   ();
	CreateAndInitImmHash();
}
//---------------------------------------------------------------------------------------------------
void	DeleteImmHash()	
{ 
	if (ImmHash) delete[] ImmHash;
}
//---------------------------------------------------------------------------------------------------
void	DeleteHash()	
{ 
	if (Hash) delete[] Hash;
}
//---------------------------------------------------------------------------------------------------
void	InitImmHash()	
{ 
	if (ImmHash)
	{
		for (int i=0; i < MARKING_IMMHASH_SIZE; i++)
			ImmHash[i] = NULL;
	}
}
//---------------------------------------------------------------------------------------------------
void	CleanImmHash()	
{ 
	if (ImmHash)
	{
		for (int i=0; i < MARKING_IMMHASH_SIZE; i++)
		{
			if (ImmHash[i]) 
			{
				ImmHash[i]->Clear();
				delete ImmHash[i]; 
				ImmHash[i] = NULL;
			}
		}
	}
	delete ImmHash;
	ImmHash = NULL;
}
//---------------------------------------------------------------------------------------------------
void	CleanHash()	
{ 
	if (Hash)
	{
		for (int i=0; i < MARKING_HASH_SIZE; i++)
		{
			if (Hash[i]) 
			{
				Hash[i]->Clear();
				delete Hash[i]; 
				Hash[i] = NULL;
			}
		}
	}
}
//---------------------------------------------------------------------------------------------------
void	CleanHashs()
{
	CleanHash();
	CleanImmHash();
}

//###################################################################################################
//
//	Abspeichern einer Markierung im Hash
//
//###################################################################################################
Marking*	SaveMarkingInHash(Marking& m, bool& exists)
{
	int	i	= 1;
	int	sub	= 1;
	Key	key     = m.GetKey();
	Key     old_key = key               * 123456789;
	int	mod	= MARKING_HASH_SIZE * 123456789;

	if (old_key < 0) old_key = -old_key;

	while (key >= MARKING_HASH_SIZE)  
	{
		sub = int(MARKING_HASH_SIZE / i++);
		mod = int(mod / 10) - sub; 
		key = (key % mod)   - sub; 
	}

	if (key < 0) key = -key;
	key = (old_key % MARKING_HASH_SIZE);
	if (key < 0) key = -key;

	if (!Hash[key]) 
	{
		Hash[key] = new MarkingList(m);
		exists    = false;
		return &(Hash[key]->first->lelement);
	}
	return (Hash[key]->Insert(m,exists));

}
//---------------------------------------------------------------------------------------------------
NonPersistentMarking*	SaveMarkingInImmHash(NonPersistentMarking* m, bool& exists)
{
	Key	key     = m->GetKey();

	while (key >= MARKING_IMMHASH_SIZE)  
		key = key % MARKING_IMMHASH_SIZE;

	if (!ImmHash[key]) 
	{
		ImmHash[key] = new NonPersistentMarkingList(*m);
		exists       = false;
		return &(ImmHash[key]->first->lelement);
	}
	return (ImmHash[key]->Insert(*m,exists));
}

//###################################################################################################
//
//	Abspeichern einer Markierung im Hash und Anhaengen dieser Markierung an eine andere
//
//###################################################################################################
void	SaveAndAppendMarking(Marking*& saved_marking, bool& exists)	
{
	saved_marking = SaveMarkingInHash(*new_marking, exists);
	last_marking->AppendNewMarking(*saved_marking, *(new_delay));
	if (!exists)
	{
		marking_counter++;
		if (marking_counter == interval)
		{
			cout <<"\tno. of marking       = "<<interval<<endl;
			interval = interval + INTERVAL;
		}
	}
}
//###################################################################################################
//
//	Verteilung der Markierungen im Hash ausgeben
//
//###################################################################################################
void	ShowHashDistribution()
{
	int max_fill  = 0;
	int no_mark   = 0;
	int l_size    = 0;
	int hash_size = 0;
	int free_place= 0;

	cout <<"\n";
	if (Hash)
	{
		for (int i=0; i < MARKING_HASH_SIZE; i++)
		{
			if (!Hash[i])
			{
				free_place++;
				// cout <<"key : "<<i<<" entries : 0\n";
				continue;
			}
			hash_size++;
			l_size   = Hash[i]->size;
			no_mark += l_size;

			if (l_size > max_fill)
				max_fill = l_size;

			cout <<"key : "<<i<<" entries : "<<l_size<<" ";
			for (int n=0; n < l_size; n++) 
			cout <<"*";
			cout <<"\n";
		}
				cout <<"\n";
				cout <<"no. of markings = "<<no_mark			<<"\n";
				cout <<"empty places    = "<<free_place			<<"\n";
				cout <<"max. filling    = "<<max_fill			<<"\n";
		if (hash_size) 	cout <<"average filling = "<<double(no_mark/hash_size)	<<"\n\n";
	}
	else	cout <<"\n\nHASH IS EMPTY\n\n";
}
//---------------------------------------------------------------------------------------------------
void	ShowImmHashDistribution()
{
	int max_fill  = 0;
	int no_mark   = 0;
	int l_size    = 0;
	int hash_size = 0;
	int free_place= 0;

	cout <<"\n";
	if (ImmHash)
	{
		for (int i=0; i < MARKING_IMMHASH_SIZE; i++)
		{
			if (!ImmHash[i]) 
			{
				free_place++;
				continue;
			}
			hash_size++;
			l_size   = ImmHash[i]->size;
			no_mark += l_size;

			if (l_size > max_fill)
				max_fill = l_size;
		}
				cout <<"\n";
				cout <<"no. of markings = "<<no_mark			<<"\n";
				cout <<"empty places    = "<<free_place			<<"\n";
				cout <<"max. filling    = "<<max_fill			<<"\n";
		if (hash_size) 	cout <<"average filling = "<<double(no_mark/hash_size)	<<"\n\n";
	}
	else	cout <<"\n\nIMMHASH IS EMPTY\n\n";
}
//###################################################################################################
#endif
