#ifndef __StructuralAnalysis_cc__
#define __StructuralAnalysis_cc__

#include <iostream.h>
#include "inc_classes.h"

extern	PageList	PageRefList;
extern	bool		DEBUG;

char*   _page_name_	= NULL;
Page*	_page_		= NULL;
Cell*	_cell_		= NULL;

//#################################################################################################################
//
//	Ueberpruefen, ob eine Celle von anderen Seiten abhaengig ist
//
//#################################################################################################################
bool	CellIsStructuralIndependent()
{
	if (!((_cell_)->IsPlaceDependent()) )		return true;
	if ( ((_cell_)->IsOnTheSamePage(_page_name_)) )	return true;
	return false;
}
//#################################################################################################################
//
//	Ueberpruefen, ob eine Celle von anderen Seiten abhaengig ist
//
//#################################################################################################################
bool	StructuralIndependent(char* s)
{
	bool  erg=false; 
	if (!(erg=CellIsStructuralIndependent()) && DEBUG)
		cout <<"\t"<<s<<" is not structural independent ..."<<endl;
	return erg;
}
//#################################################################################################################
//
//	- Ueberpruefen, ob alle Transitionen (globale Bed., In/OutArcs, lokale Bed., Delays) und
//	  alle Stellen strukturell von einer anderen Seite abhaengen.	=> keine Strukturanalyse
//	- Ueberprufen ob es Subseiten gibt.				=> keine Strukturanalyse
//
//#################################################################################################################
bool	PageIsStructuralIndependent()
{
	if ( ((_page_)->GetSubPages()) && ((_page_)->GetSubPages()->size) )
	{
		if (DEBUG) cout <<"\tthere are subpages on this page ..."<<endl;
		return false;
	}
	for (PlacePtrListElement* pl=(_page_)->GetPlaces()->first; pl; pl=pl->next)
	{
		_cell_=pl->plelement->GetCapacity();
		if (!StructuralIndependent("place capacity")) return false;
	}

	for (TransitionListElement* t=(_page_)->GetTransitions()->first; t; t=t->next)
	{
		_cell_=t->lelement.GetGlobalCondition();
		if (!StructuralIndependent("global condition")) return false;

		for (BindingListElement* b=t->lelement.GetBindings()->first; b; b=b->next)
		{
			_cell_=b->lelement.GetLocalCondition();
			if (!StructuralIndependent("local condition")) return false;

			_cell_=b->lelement.GetDelay()->GetCell();
			if (!StructuralIndependent("delay")) return false;

			for (ArcBindListElement* ai=b->lelement.GetInArcBind().first; ai; ai=ai->next)
			{
				_cell_=ai->lelement.Gettoken_multisetCell();
				if (!StructuralIndependent("in arc")) return false;
			}
			for (ArcBindListElement* ao=b->lelement.GetOutArcBind().first; ao; ao=ao->next)
			{
				_cell_=ao->lelement.Gettoken_multisetCell();
				if (!StructuralIndependent("out arc")) return false;
			}
		}
	}
	return true;
}
//#################################################################################################################
//
//	Ueberpruefen, ob es Seiten gibt, die strukturell unabhaengig von anderen Seiten sind.
//
//#################################################################################################################
void	StructuralAnalysis()
{
	int pgs = PageRefList.size;
	int pos = 1;
	for (PageListElement* pa=PageRefList.first; pa; pa=pa->next, pos++)
	{
		_page_      = &(pa->lelement);
		strcpy(_page_name_ = new char[(strlen(pa->lelement.GetPageName().GetChar()) + 1)],
						     (pa->lelement.GetPageName().GetChar()));

		if (PageIsStructuralIndependent()) 	
		{
			if (DEBUG) 
			{
				cout <<"\tpage "<<pa->lelement.GetPageName().GetChar();
				cout <<" is structural independent"<<endl<<endl;
			}
		}
		else	
		{
			if (DEBUG) 
			{
				cout <<"\tpage "<<pa->lelement.GetPageName().GetChar();
				cout <<" is not structural independent"<<endl<<endl;
			}
			PageRefList.DeleteElement(pos);
			pos--;
		}
		delete[] _page_name_;
		_page_name_ = NULL;
	}
	if (DEBUG) cout <<"\tno. of pages                        : "<<pgs		<<endl;
	if (DEBUG) cout <<"\tno. of structural independent pages : "<<PageRefList.size	<<endl;
}
//#################################################################################################################
#endif
