#ifndef __Transition_cc__
#define __Transition_cc__

extern "C" {
#include <strings.h>
}
#include <iostream.h>

#include "Identifier.h"
#include "Textual.h"
#include "TextualList.h"

#include "token_multiset.h"
#include "PlaceHeader.h"
#include "Token.h"

#include "CellVal.h"
#include "Cell.h"

#include "Place.h"

#include "Delay.h"
#include "BindingElement.h"
#include "BindingList.h"

#include "Transition.h"

//###################################################################################################################
Transition::Transition()		    
	  : 	name		(DEF_TXT_NAME),
	    	gc_result	(DEF_COND_RES),
	    	global_ref	(DEF_TRANS_REF)
	  { 
		servertype = Exclusive; 
		bindings   = new BindingList();
		globalcond = new ConstVal();
	  }
Transition::Transition(int)		    
	  : 	name		(DEF_TXT_NAME),
	    	gc_result	(DEF_COND_RES),
	    	global_ref	(DEF_TRANS_REF)
	  { 
		servertype = Exclusive; 
		bindings   = new BindingList();
		globalcond = new ConstVal();
	  }

Transition::Transition(const Transition& t) 
	  : 	name		(t.name),
	    	gc_result	(t.gc_result),
	    	global_ref	(t.global_ref)
	  {
		if (!t.bindings) 
		{
			servertype = Exclusive; 
			bindings   = new BindingList();
			globalcond = new ConstVal();
			return;
		}

		servertype = t.servertype;        
		if (t.bindings->size)
		{
			if (servertype == Infinite && 
			    t.bindings->first->lelement.GetDelay()->GetDelayType() == _DETERMINISTIC)
				TransitionError(name);
		}
		bindings = new BindingList(*(t.bindings));
		if (!t.globalcond)  globalcond = new ConstVal();    
		else                globalcond = t.globalcond->Copy();
	}

Transition::Transition(char* t, ServerType st, const BindingList* bl, const Cell* c) 
	  : 	name		(t),
	    	gc_result	(DEF_COND_RES),
	    	global_ref	(DEF_TRANS_REF)
	  {
		if (!bl) 
		{
			servertype = Exclusive; 
			bindings   = new BindingList();
			globalcond = new ConstVal();
			return;
		}

		servertype = st;        
		if (bl->size)
		{
			if (servertype == Infinite && 
			    bl->first->lelement.GetDelay()->GetDelayType() == _DETERMINISTIC)
				TransitionError(name);
		}
		bindings = new BindingList(*bl);
		if (!c)  globalcond = new ConstVal();    
		else     globalcond = c->Copy();
	}
//###################################################################################################################
Textual		Transition::GetTransitionName()					{ return name;	 	 	    }
ServerType	Transition::GetServerType()					{ return servertype; 	 	    }
BindingList*	Transition::GetBindings()					{ return bindings; 	 	    }
int		Transition::GetBindingSize()					{ return (bindings->size);  	    }
int		Transition::GetGlobalTransitionRef()				{ return global_ref; 	 	    }
void		Transition::SetGlobalTransitionRef(int gr)			{ global_ref = gr; 	 	    }
char*		Transition::GetName()	 					{ return name.GetChar(); 	    }
bool		Transition::operator!=(Transition& t) 				{ return !(*this==t); 	 	    }
bool            Transition::LessThan(Transition& t) 				{ return (name<t.name);  	    }
//-------------------------------------------------------------------------------------------------------------------
Cell*		Transition::GetGlobalCondition()				
{ 
	if (globalcond) return globalcond; 	 	    
	return new ConstVal(gc_result);
}
//-------------------------------------------------------------------------------------------------------------------
bool		Transition::GlobalCondTrue()	     			
{ 
	if (gc_result != DEF_COND_RES)  
		return (gc_result == 0?false:true);

	CellVal* cv  = globalcond->Calculate();
	double   val = cv->GetFloat();

	cv->Clear();
	delete cv;

	if (globalcond->IsPlaceDependent())
	{
		if (!val) 
			return false;
		return true;
	}

	if (!val) gc_result = 0;
	else	  gc_result = 1;

	if (globalcond) 
	{
		globalcond->Clear();
		delete globalcond;
		globalcond = NULL;
	}

	return (gc_result == 0?false:true);
}
//------------------------------------------------------------------------------------------------------------------
void	Transition::Clear()		     
{ 
	name.Clear();
	if (globalcond) { globalcond->Clear(); delete globalcond; globalcond = NULL; }
	if (bindings)   { bindings->Clear();   delete bindings;   bindings   = NULL; }
	global_ref = DEF_TRANS_REF;
	gc_result  = DEF_COND_RES;
}
//------------------------------------------------------------------------------------------------------------------
Transition&	Transition::operator =(const Transition& t) 
{ 
	Clear(); 
	name        = t.name;
	servertype  = t.servertype;
	global_ref  = t.global_ref;
	gc_result   = t.gc_result;

	if (!t.bindings)    bindings   = new BindingList(); else bindings   = new BindingList(*(t.bindings));
	if (!t.globalcond)  globalcond = new ConstVal();    else globalcond = t.globalcond->Copy();
	return *this;
}
//------------------------------------------------------------------------------------------------------------------
bool		Transition::operator==(Transition& t) 
{ 
return ( name==t.name && servertype==t.servertype && *bindings==*(t.bindings) && *globalcond==*(t.globalcond) ); 
}
//------------------------------------------------------------------------------------------------------------------
void		Transition::TransitionError(Textual& tname)
{
	cerr <<"Error in new Transition "<<tname.GetChar();
	cerr <<" : servertype = infinite, delaytype = deterministic\n";
	exit (SERVERTYPE_DELAY_ERROR);
}

//##################################################################################################################
#endif

