/****************************************************************************
PROGRAMM	:	SolveLGS.c	

KOMPONENTE      :       dnspiv.c					     

ZWECK		:	Gauss-Eliminationsverfahren zur L"osung gro"ser d"unn
                        besetzter Matrizen.
			C-Version und leichte Modifikation der Fortran-routine
			'dnspiv' von A. H. Sherman 
			(siehe A. H. Sherman: NSPIV, A Fortran Subroutine for
			 Sparse Gaussian Elimination With Partial Pivoting
			 in: ACM Trans. on Math. SW, Vol. 4 No. 4, Dec. 1978)

Datum		:	Okt. 93

Programmierer  	:	Witold Smilowski: C-Version
                        Martin Mueller: Speicherverwaltung, Optimierungen
	
Projektleiter  	: 	Christoph Lindemann				     
******************************************************************************

Modifiziert	:	Frank Bayer, 1997, Einbindung in agnes

******************************************************************************/

/*
#define DEBUG_SOLVE
*/
/*
#define MSG
*/
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <math.h>

/* Nice hacks */

#define forever for(;;)
#define elsif else if

/* FORTRAN functions, which I have translated to C... */

#define DABS(x) fabs(x)

/*  void dnspiv(n, ia, ja, a, b, max, r, c, ic, x, itemp, rtemp, ierr) */

/*  NSPIV CALLS NSPIV1 WHICH USES SPARSE GAUSSIAN ELIMINATION WITH
    COLUMN INTERCHANGES TO SOLVE THE LINEAR SYSTEM A X = B.  THE
    ELIMINATION PHASE PERFORMS ROW OPERATIONS ON A AND B TO OBTAIN
    A UNIT UPPER TRIANGULAR MATRIX U AND A VECTOR Y.  THE SOLUTION
    PHASE SOLVES U X = Y.
  
  
    INPUT ARGUMENTS---
  
    N      INTEGER NUMBER OF EQUATIONS AND UNKNOWNS
  
    IA     INTEGER ARRAY OF N+1 ENTRIES CONTAINING ROW POINTERS TO A
           (SEE MATRIX STORAGE DESCRIPTION BELOW)
  
    JA     INTEGER ARRAY WITH ONE ENTRY PER NONZERO IN A, CONTAINING
           COLUMN NUMBERS OF THE NONZEROES OF A.  (SEE MATRIX STORAGE
           DESCRIPTION BELOW)
  
    A      DOUBLE PRECISION ARRAY WITH ONE ENTRY PER NONZERO IN A, CONTAINING THE
           ACTUAL NONZEROES.  (SEE MATRIX STORAGE DESCRIPTION BELOW)
  
    B      DOUBLE PRECISION ARRAY OF N ENTRIES CONTAINING RIGHT HAND SIDE DATA
  
    MAX    INTEGER NUMBER SPECIFYING MAXIMUM NUMBER OF OFF-DIAGONAL
           NONZERO ENTRIES OF U WHICH MAY BE STORED
  
    R      INTEGER ARRAY OF N ENTRIES SPECIFYING THE ORDER OF THE
           ROWS OF A (I.E., THE ELIMINATION ORDER FOR THE EQUATIONS)
  
    C      INTEGER ARRAY OF N ENTRIES SPECIFYING THE ORDER OF THE
           COLUMNS OF A.  C IS ALSO AN OUTPUT ARGUMENT
  
    IC     INTEGER ARRAY OF N ENTRIES WHICH IS THE INVERSE OF C
           (I.E., IC(C(I)) = I).  IC IS ALSO AN OUTPUT ARGUMENT
  
    ITEMP  INTEGER ARRAY OF 2*N + MAX + 2 ENTRIES, FOR INTERNAL USE
  
    RTEMP  DOUBLE PRECISION ARRAY OF N + MAX ENTRIES FOR INTERNAL USE
  
  
    OUTPUT ARGUMENTS---
  
    C      INTEGER ARRAY OF N ENTRIES SPECIFYING THE ORDER OF THE
           COLUMNS OF U.  C IS ALSO AN INPUT ARGUMENT
  
    IC     INTEGER ARRAY OF N ENTRIES WHICH IS THE INVERSE OF C
           (I.E., IC(C(I)) = I).  IC IS ALSO AN INPUT ARGUMENT
  
    X      DOUBLE PRECISION ARRAY OF N ENTRIES CONTAINING THE SOLUTION VECTOR
  
    IERR   INTEGER NUMBER WHICH INDICATES ERROR CONDITIONS OR
           THE ACTUAL NUMBER OF OFF-DIAGONAL ENTRIES IN U (FOR
           SUCCESSFUL COMPLETION)
  
           IERR VALUES ARE---
  
           0 LT IERR             SUCCESSFUL COMPLETION.  U HAS IERR
                                 OFF-DIAGONAL NONZERO ENTRIES
  
           IERR = 0              ERROR.  N = 0
  
           -N LE IERR LT 0       ERROR.  ROW NUMBER IDABS(IERR) OF A IS
                                 IS NULL
  
           -2*N LE IERR LT -N    ERROR.  ROW NUMBER IDABS(IERR+N) HAS A
                                 DUPLICATE ENTRY
  
           -3*N LE IERR LT -2*N  ERROR.  ROW NUMBER IDABS(IERR+2*N)
                                 HAS A ZERO PIVOT
  
           -4*N LE IERR LT -3*N  ERROR.  ROW NUMBER IDABS(IERR+3*N)
                                 EXCEEDS STORAGE
  
  
    STORAGE OF SPARSE MATRICES---
  
    THE SPARSE MATRIX A IS STORED USING THREE ARRAYS IA, JA, AND A.
    THE ARRAY A CONTAINS THE NONZEROES OF THE MATRIX ROW-BY-ROW, NOT
    NECESSARILY IN ORDER OF INCREASING COLUMN NUMBER.  THE ARRAY JA
    CONTAINS THE COLUMN NUMBERS CORRESPONDING TO THE NONZEROES STORED
    IN THE ARRAY A (I.E., IF THE NONZERO STORED IN A(K) IS IN
    COLUMN J, THEN JA(K) = J).  THE ARRAY IA CONTAINS POINTERS TO THE
    ROWS OF NONZEROES/COLUMN INDICES IN THE ARRAY A/JA (I.E.,
    A(IA(I))/JA(IA(I)) IS THE FIRST ENTRY FOR ROW I IN THE ARRAY A/JA).
    IA(N+1) IS SET SO THAT IA(N+1) - IA(1) = THE NUMBER OF NONZEROES IN A */

/*
#ifndef SOL2
void dnspiv(n, ia, ja, a, b, max, r, c, ic, x, itemp, rtemp, ierr)
int n, *ia, *ja;
double *a, *b;
int max, *r, *c, *ic;
double *x;
int *itemp;
double *rtemp;
int *ierr;
#else
void dnspiv(int,int*,int*,double*,double*,int,int*,int*,int*,double*,int*,double*,int*)
#endif
{
    void nspiv1();
    nspiv1(n, ia, ja, a, b, max, r, c, ic, x, rtemp, itemp, itemp + n + 1, itemp + 2 * n + 2, rtemp + n, ierr);

}
*/

/*  void dnspiv1_mod(n, ia, ja, a, nrnbr, max, r, c, ic, x, y, p, iu, juorg, uorg, tol, num_drop, ierr) */

/*  NSPIV1 USES SPARSE GAUSSIAN ELIMINATION WITH
    COLUMN INTERCHANGES TO SOLVE THE LINEAR SYSTEM A X = B.  THE
    ELIMINATION PHASE PERFORMS ROW OPERATIONS ON A AND B TO OBTAIN
    A UNIT UPPER TRIANGULAR MATRIX U AND A VECTOR Y.  THE SOLUTION
    PHASE SOLVES U X = Y.
  
  
    SEE NSPIV FOR DESCRIPTIONS OF ALL INPUT AND OUTPUT ARGUMENTS
    OTHER THAN THOSE DESCRIBED BELOW
  
    INPUT ARGUMENTS (USED INTERNALLY ONLY)---
  
    Y   DOUBLE PRECISION ARRAY OF N ENTRIES USED TO COMPUTE THE UPDATED
        RIGHT HAND SIDE
  
    P   INTEGER ARRAY OF N+1 ENTRIES USED FOR A LINKED LIST.
        P(N+1) IS THE LIST HEADER, AND THE ENTRY FOLLOWING
        P(K) IS IN P(P(K)).  THUS, P(N+1) IS THE FIRST DATA
        ITEM, P(P(N+1)) IS THE SECOND, ETC.  A POINTER OF
        N+1 MARKS THE END OF THE LIST
  
    IU  INTEGER ARRAY OF N+1 ENTRIES USED FOR ROW POINTERS TO U
        (SEE MATRIX STORAGE DESCRIPTION BELOW)
  
    JU  INTEGER ARRAY OF MAX ENTRIES USED FOR COLUMN NUMBERS OF
        THE NONZEROES IN THE STRICT UPPER TRIANGLE OF U.  (SEE
        MATRIX STORAGE DESCRIPTION BELOW)
  
    U   DOUBLE PRECISION ARRAY OF MAX ENTRIES USED FOR THE ACTUAL NONZEROES IN
        THE STRICT UPPER TRIANGLE OF U.  (SEE MATRIX STORAGE
        DESCRIPTION BELOW)
  
  
    STORAGE OF SPARSE MATRICES---
  
    THE SPARSE MATRIX A IS STORED USING THREE ARRAYS IA, JA, AND A.
    THE ARRAY A CONTAINS THE NONZEROES OF THE MATRIX ROW-BY-ROW, NOT
    NECESSARILY IN ORDER OF INCREASING COLUMN NUMBER.  THE ARRAY JA
    CONTAINS THE COLUMN NUMBERS CORRESPONDING TO THE NONZEROES STORED
    IN THE ARRAY A (I.E., IF THE NONZERO STORED IN A(K) IS IN
    COLUMN J, THEN JA(K) = J).  THE ARRAY IA CONTAINS POINTERS TO THE
    ROWS OF NONZEROES/COLUMN INDICES IN THE ARRAY A/JA (I.E.,
    A(IA(I))/JA(IA(I)) IS THE FIRST ENTRY FOR ROW I IN THE ARRAY A/JA).
    IA(N+1) IS SET SO THAT IA(N+1) - IA(1) = THE NUMBER OF NONZEROES IN
    A.  IU, JU, AND U ARE USED IN A SIMILAR WAY TO STORE THE STRICT UPPER
    TRIANGLE OF U, EXCEPT THAT JU ACTUALLY CONTAINS C(J) INSTEAD OF J */

#ifndef SOL2
void dnspiv1_mod(n, ia, ja, a, nrnbr, max, r, c, ic, x, y, p, iu, juorg, uorg, tol, num_drop, ierr) 
int n, *ia, *ja;
double *a;
int nrnbr,max, *r, *c, *ic;
double *x, *y;
int *p, *iu, **juorg;
double **uorg,tol;
int *num_drop,*ierr;
#else
void dnspiv1_mod(int n, int* ia, int* ja, double* a, int nrnbr, int max, int* r, int* c, int* ic, 
		 double* x, double* y, int* p, int* iu, int** juorg, double** uorg, double tol, 
		 int* num_drop, int* ierr) 
#endif
{
    double dk, lki, one, xpv, xpvmax, yk, zero, utmp, *u;
    int ck, pk, ppk, pv, v, vi, vj, vk, j, juptr, k, jmin, jmax, maxc, nzcnt,
        jaj, juj, i, maxcl, max1, *ju, nd;

    u = *uorg;
    ju = *juorg;

/* Parameter adjustments */

    --ia;
    --ja;
    --a;
/*    --b;*/
    --r;
    --c;
    --ic;
    --x;
    --y;
    --p;
    --iu;
    --ju;
    --u;

    if(!n) {

/*  N = 0 */

	*ierr = 0;
	return;
    }
    max1 = max;
    one = 1.0;
    zero = 0.0;
    nd = 0;

/*  INITIALIZE WORK STORAGE AND POINTERS TO JU */

    for(j = 1; j <= n; j++)
	x[j] = zero;
    iu[1] = 1;
    juptr = 0;

/*  PERFORM SYMBOLIC AND NUMERIC FACTORIZATION ROW BY ROW
    VK (VI,VJ) IS THE GRAPH VERTEX FOR ROW K (I,J) OF U */

    for(k = 1; k <= n; k++) {

#ifdef MSG
      if(n>10000 && !(k%1000))
	{
	  printf("Berechnung von Zeile: %d\n",k);
	  fflush(stdout);
	}
#endif

/*  INITIALIZE LINKED LIST AND FREE STORAGE FOR THIS ROW
    THE R(K)-TH ROW OF A BECOMES THE K-TH ROW OF U. */

	p[n + 1] = n + 1;
	vk = r[k];

/*  SET UP ADJACENCY LIST FOR VK, ORDERED IN
    CURRENT COLUMN ORDER OF U.  THE LOOP INDEX
    GOES DOWNWARD TO EXPLOIT ANY COLUMNS
    FROM A IN CORRECT RELATIVE ORDER */

	jmin = ia[vk];
	jmax = ia[vk + 1] - 1;
	if(jmin > jmax) {

/*  ROW K OF A IS NULL */

	    *ierr = -k;
	    return;
	}
	j = jmax;
	do {
	    jaj = ja[j];
	    vj = ic[jaj];

/*  STORE A(K,J) IN WORK VECTOR */
 	      x[vj] = a[j];

/*  THIS CODE INSERTS VJ INTO ADJACENCY LIST OF VK */

	    ppk = n + 1;
	    do {
		pk = ppk;
		ppk = p[pk];
	    } while(ppk < vj);
	    if(ppk == vj) {

/*  ROW K OF A HAS A DUPLICATE ENTRY */

		*ierr = -(n + k);
		return;
	    }
	    p[vj] = ppk;
	    p[pk] = vj;
	} while(--j >= jmin);

/*  THE FOLLOWING CODE COMPUTES THE K-TH ROW OF U */

	vi = n + 1;
/*	yk = b[vk];*/
	yk = (vk == nrnbr ? 1.0 : 0.0);
	forever {
	    vi = p[vi];
	    if(vi >= k)
		break;

/*  VI LT VK -- PROCESS THE L(K,I) ELEMENT AND MERGE THE
    ADJACENCY OF VI WITH THE ORDERED ADJACENCY OF VK */

	    lki = -x[vi];
	    x[vi] = zero;

/*  ADJUST RIGHT HAND SIDE TO REFLECT ELIMINATION */

	    yk += lki * y[vi];
	    ppk = vi;
	    jmin = iu[vi];
	    jmax = iu[vi + 1] - 1;
	    if(jmin > jmax)
		continue;
	    for(j = jmin; j <= jmax; j++) {
		juj = ju[j];
		vj = ic[juj];

/*  IF VJ IS ALREADY IN THE ADJACENCY OF VK,
    SKIP THE INSERTION */

		if(x[vj] != zero) {
		    x[vj] += lki * u[j];
		    continue;
		}

/*  INSERT VJ IN ADJACENCY LIST OF VK.
    RESET PPK TO VI IF WE HAVE PASSED THE CORRECT
    INSERTION SPOT.  (THIS HAPPENS WHEN THE ADJACENCY OF
    VI IS NOT IN CURRENT COLUMN ORDER DUE TO PIVOTING.) */

		if(vj < ppk)
		    ppk = vi;
		elsif(vj == ppk) {
		    x[vj] += lki * u[j];
		    continue;
		}
		do {
		    pk = ppk;
		    ppk = p[pk];
		} while(ppk < vj);
		if(ppk == vj) {
		    x[vj] += lki * u[j];
		    continue;
		}
		p[vj] = ppk;
		p[pk] = ppk = vj;

/*  COMPUTE L(K,J) = L(K,J) - L(K,I)*U(I,J) FOR L(K,I) NONZERO
    COMPUTE U*(K,J) = U*(K,J) - L(K,I)*U(I,J) FOR U(K,J) NONZERO
    (U*(K,J) = U(K,J)*D(K,K)) */

		x[vj] += lki * u[j];
	    }
	}

/*  PIVOT--INTERCHANGE LARGEST ENTRY OF K-TH ROW OF U WITH
    THE DIAGONAL ENTRY.
  
    FIND LARGEST ENTRY, COUNTING OFF-DIAGONAL NONZEROES */

	if(vi > n) {

/*  ZERO PIVOT IN ROW K */

	    *ierr = -(2 * n + k);
	    return;
	}
	xpvmax = DABS(x[vi]);
	maxc = vi;
	nzcnt = 0;
	pv = vi;
	forever {
	    v = pv;
	    pv = p[pv];
	    if(pv > n)
		break;
	    nzcnt++;
	    if((xpv = DABS(x[pv])) <= xpvmax)
		continue;
	    xpvmax = xpv;
	    maxc = pv;
	    maxcl = v;
	}
	if(xpvmax == zero) {
/*  ZERO PIVOT IN ROW K */
/* wegen drop-tolerance, kann die matrix singulaer geworden sein
   in dem Fall wird dk auf 1.0 gesetzt (Nullzeile)
*/
/*
	    *ierr = -(2 * n + k);
	    return;
*/
	  fprintf(stdout,"Singularity of Matrix detected in row k = %d (r[k]=%d)\n",k,r[k]);
	  fflush(stdout);
	  dk = 1.0;
	}
      else
	{
/*  IF VI = K, THEN THERE IS AN ENTRY FOR DIAGONAL
    WHICH MUST BE DELETED.  OTHERWISE, DELETE THE
    ENTRY WHICH WILL BECOME THE DIAGONAL ENTRY */

	  if(vi == k || vi == maxc)
	    vi = p[vi];
	  else
	    p[maxcl] = p[maxc];

/*  COMPUTE D(K) = 1/L(K,K) AND PERFORM INTERCHANGE. */

	  if (!x[maxc])
	  {
	  	printf("SolveLGS : division by zero in function dnspiv1_mod()");
	  	exit(1);
	  }
	  dk = one / x[maxc];
	  x[maxc] = x[k];
	  i = c[k];
	  c[k] = c[maxc];
	  c[maxc] = i;
	  ck = c[k];
	  ic[ck] = k;
	  ic[i] = maxc;
	  x[k] = zero;
	}

/*  UPDATE RIGHT HAND SIDE. */

	y[k] = yk * dk;

/*  COMPUTE VALUE FOR IU(K+1) AND CHECK FOR STORAGE OVERFLOW */

	if((iu[k + 1] = iu[k] + nzcnt) > max1 + 1) {

/*  STORAGE FOR U EXCEEDED ON ROW K */
	  /* n - k Zeilen noch zu bearbeiten; Annahme, dass nzcnt repraesentativ ist */
	  max1 += nzcnt*(n-k+1);
/*
	  printf("\nStorage exceeded, new size = %d\n\n",max1);
*/
	  ju++;
	  u++;

#ifdef DEBUG_SOLVE
	  *ierr = -(3 * n + k);
	  return;
#endif

	  if ((ju=(int*)realloc(ju,max1*sizeof(int)))==NULL)
	    {
	      *ierr = -(3 * n + k);
	      *juorg = ju;
	      *uorg = u;
	      return;
	    }
	  if ((u=(double*)realloc(u,max1*sizeof(double)))==NULL)
	    {
	      *ierr = -(3 * n + k);
	      *juorg = ju;
	      *uorg = u;
	      return;
	    }
	  ju--;
	  u--;
	}

/*  MOVE COLUMN INDICES FROM LINKED LIST TO JU.
    COLUMNS ARE STORED IN CURRENT ORDER WITH ORIGINAL
    COLUMN NUMBER (C(J)) STORED FOR CURRENT COLUMN J */

	if(vi > n)
	    continue;
	j = vi;
	do {
	    utmp = x[j] * dk;
	    /* drop-toleranz */
	    if (DABS(utmp) >= tol)
	      {
		ju[++juptr] = c[j];
		u[juptr] = utmp;
	      }
	    else
	      {
		iu[k+1] -= 1;
		nd++;
	      }
	    x[j] = zero;
	    j = p[j];
	} while(j <= n);
    }

/*  BACKSOLVE U X = Y, AND REORDER X TO CORRESPOND WITH A */

    k = n;
    for(i = 1; i <= n; i++) {
	yk = y[k];
	jmin = iu[k];
	jmax = iu[k + 1] - 1;
	if(jmin <= jmax)
	    for(j = jmin; j <= jmax; j++) {
		juj = ju[j];
		juj = ic[juj];
		yk -= u[j] * y[juj];
	    }
	y[k] = yk;
	ck = c[k];
	x[ck] = yk;
	k--;
    }

/*  RETURN WITH IERR = NUMBER OF OFF-DIAGONAL NONZEROES IN U */

    ++u;++ju;
    *juorg = ju;
    *uorg = u;
    *num_drop = nd;
    *ierr = iu[n + 1] - iu[1];
}
