#ifndef __inc_rrg_globals_h__
#define __inc_rrg_globals_h__

extern "C" {
#include <sys/time.h>
#include <sys/resource.h>
#include <dirent.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#if defined(__linux__) || defined(LINUX) || defined (Linux)
#include <linux/resource.h>
#endif
#include "portab.h" 
}

//###########################################################################################################
//	externe globale ReferenzListen und Variablen 
//###########################################################################################################
extern TransitionList		GeneralTransitionRefList;    // alle General-Transitionen des Netzes
extern TransitionList		InfiniteTransitionRefList;   // alle Infinite-Server Transitionen des Netzes
extern TransitionList		ImmediateTransitionRefList;  // alle Immediate-Transitionen des Netzes
extern TransitionList		ExclusiveTransitionRefList;  // alle Exclusive-Server Transitionen des Netzes
extern TextualList		TransitionNameRefList;	     // alle Transitionen des Netzes
extern PlaceHeaderPtrList	PlaceHeaderPtrRefList;	     // alle Stellen-header des Netzes
extern PlacePtrList		PlaceCapDepRefList;	     // alle Stellen, mit stellenabhaengig Kapzitaeten
extern ListOfSMCLists		DetMarkingListList;	     // alle det.-Markow-Ketten des Netzes
extern ListOfSMCLists		GenMarkingListList;	     // alle gen.-Markow-Ketten des Netzes
extern SocketPortList		SocketPortRefList;	     // alle Socket-Port-Assignments 
extern TokenList		TokenRefList;		     // alle Token des Netzes
extern PlaceList		PlaceRefList;		     // alle Stellen des Netzes
extern PageList			PageRefList;		     // alle Seiten des Netzes
extern DefinitionList   	DefinitionRefList;	     // alle Ausdruecke des Netzes
extern DefinitionList		EvalRefList;		     // alle zu berecheneneden Bewertungsgroessen des 
							     // Netzes 

//-----------------------------------------------------------------------------------------------------------
//	Variablen fuer die Funktion getopt()
//-----------------------------------------------------------------------------------------------------------
extern char*     	optarg;
extern int       	optind; 
extern int		opterr; 
extern int		optopt;

//-----------------------------------------------------------------------------------------------------------
//	Variable fuer die Funktion getenv()
//-----------------------------------------------------------------------------------------------------------
extern	char**		environ;

//###########################################################################################################
//	globale Variablen
//###########################################################################################################
char*		table_string;			 // eingelesene oder uebergebene Uebergangstabelle
double*		pi_vector      = (double*)NULL;	 // Ergebnisvektor
Marking*	InitialMarking = new Marking();	 // Markierung, an der der RRG aufgehangen ist
bool     	default_mode   = false;		 // Flag, ob mit Defaultwerten gearbeitet werden soll
char*    	auto_iter      = "-a";		 // solutionmode = iterativ | automated
char*    	p_maxiter      = NULL;		 // tmp_variable fuer die String-Version von maxiter
int      	maxiter        = 10000;		 // Defaultwert der maxiamlen Iterationen in SolveLGS()
char*    	netname        = "default";	 // Defaultwert des Netz/Modellnamens
char*    	p_epsilon      = NULL;		 // tmp_variable fuer die String-Version von epsilon
double   	epsilon        = 1.0e-7;	 // Defaultwert der maxiamlen Genauigkeit in SolveLGS()
ExecMode 	execmode       = sequential;	 // execmode = sequential | parallel
RRGError*	RRGErr 	       = new RRGError(); // Fehlervariable aller Funktionen
ArcBindList*	arcs	       = NULL;		 // globale Variable der aktuell zu schaltenden In/Out-Arcs
ArcBindList*	inarcs	       = NULL;		 // globale Variable der aktuell zu checkenden  In/Out-Arcs
ArcBindList*	outarcs	       = NULL;		 // globale Variable der aktuell zu checkenden  In/Out-Arcs

int		marking_counter= 0;		 // Anzahl aller rrg-Markierungen
long		all_start_time = 0;		 // Start-Gesamtzeit
long		all_stop_time  = 0;		 // Stop-Gesamtzeit
long		all_diff_time  = 0;		 // Differenz-Gesamtzeit
long		rrg_start_time = 0;		 // Start-Zeit 		fuer die RRG-Erstellung
long		rrg_stop_time  = 0;		 // Stop-Zeit 		fuer die RRG-Erstellung
long		rrg_diff_time  = 0;		 // Differenz-Zeit 	fuer die RRG-Erstellung

unsigned char	bits[9]	       = {0, 1, 2, 3, 4, 5, 6, 7, 8}; // bits fuer die Markierungszustaende/typen

//-----------------------------------------------------------------------------------------------------------
//	Variablen fuer Statistikzwecke und fuer die Erstellung der SMCs	
//-----------------------------------------------------------------------------------------------------------
int	no_of_det_transitions = 0;
int	no_of_exp_transitions = 0;
int	no_of_gen_transitions = 0;
int	no_of_imm_transitions = 0;
int	no_of_exc_transitions = 0;
int	no_of_mix_transitions = 0;
int	no_of_places	      = 0;
int	no_of_trans 	      = 0;
int	place_no	      = 1;

//-----------------------------------------------------------------------------------------------------------
//	Flags fuer Option-Verwaltung (Debugmodi)
//-----------------------------------------------------------------------------------------------------------
bool	DEBUG	   = false;
bool	SHOW_PI    = false;
bool	SHOW_RRG   = false;
bool	SHOW_RES   = false;
bool	SHOW_SMC   = false;
bool	SHOW_TIME  = false;
bool	SHOW_COMM  = false;
bool	SHOW_HASH  = false;
bool	SHOW_INFOS = false;
bool	SHOW_ERAND = false;
int 	SHOW_LGS   = 0;

//-----------------------------------------------------------------------------------------------------------
//	Uebergabeparameter fuer die Funktion ExamineRRG()
//-----------------------------------------------------------------------------------------------------------
marking_count* m_count 	    = new marking_count;
marking_lists* m_lists 	    = new marking_lists;

//###########################################################################################################
//      externe Funktionen
//###########################################################################################################

extern void	StartTime	  (long&);		// Startet Timer fuer die Zeitmessung
extern void	StopTime	  (long&);		// Stopt Timer fuer die Zeitmessung
extern void	DiffTime	  (long&,long&,long&);	// Errechnet Zeitdifferenz
extern void	ShowTime	  (long&,char*);	// Gibt Zeitdifferenz aus
extern void	CleanHash	  ();
extern void	CleanImmHash	  ();
extern void	DeleteHash	  ();
extern void	DeleteImmHash	  ();

extern void	ShowPIVector	  (int);		// aus DebugFunctions.cc
extern void	ShowResults	  (char*);		// aus DebugFunctions.cc

extern int      yyparse		  ();			// Der Scanner und Parser der Uebergangstabelle

extern void	WritePMFFile	  (char*);		// Schreibt das .pmf-file, falls General-Transitionen 
							// existieren.

extern void     TxtToRef          ();			// Setzt offizielle Referenzen in Funktionen der 
							// Klasse Cell, die beim Parsen noch nicht gesetzt
							// werden konnten

extern void     StructuralAnalysis();			// Ueberprueft, ob eine Seite unabhaengig von anderen 
							// Seiten ist 

extern void     CleanHashs	  ();			// Funktionen zur Hash-Table-Erstellung/Initialisierung 
extern void     CreateAndInitHash ();			// und zur Hash-Table-Saeuberung

extern void     RRGAlgo		  (Marking*&); 		// Hauptfunktion zur RRG-Erstellung

extern void     ExamineRRG	  (Marking*&, 		// initiale Markierung (Startknoten des RRG)
				   marking_count*&, 	// struct zum zaehlen/nummerieren der Markierung
				   marking_lists*&);	// nach Markierungstypen geordnete Markierungslisten

extern  void    AgnesToTimeNet   (char*,            	// Modell-Name
				  char*,		// "-i" oder "-a" (fuer SolveLGS)
				  int&,             	// Anzahl tangibler Markierungen
				  int&,          	// maxiter (Anzahl der Iterationen bei -i Option
				  			//	    von SolveLGS)
				  double&,          	// epsilon (Genaugikeit der Berechnung)
				  ExecMode,         	// parallel oder sequential
				  MarkingPtrList&,  	// exp. Markierungen
				  ListOfSMCLists&, 	// det. Markierungen
				  ListOfSMCLists&); 	// gen. Markierungen

extern	void	NetEvaluation	 (char*);               // Modell-Name
//###########################################################################################################
#endif
