%{
#include "inc_classes.h" 
#include "inc_lex_yacc_globals.h" 

char* 	StripOfPath	 	(char*);
//-----------------------------------------------------------------
//	Error-handling
//-----------------------------------------------------------------
void 	yyerror			(char*);
void 	YaccError		(char*, char*);
//-----------------------------------------------------------------
//	Checks und "getten"	
//-----------------------------------------------------------------
void	CheckAndCountTransition (char*, ServerType, BindingList*&);
Place*	CheckIfPortGetRef	(char*);
char*	CheckIfPortGetName	(char*);
char* 	GetNumberString		(int);
int 	GetNumberSize		(int);
Place*	GetPlaceRef		(char*);
//-----------------------------------------------------------------
//	"kreieren"	
//-----------------------------------------------------------------
char*	MakePortName	 	(char*);
void	MakeActualPageName	(char*);
void	MakeActualSubPageName	(char*);
Place*	SavePlace		(Place*&, PlaceType);
void	SaveDefinition		(Definition*&);
void	SaveTransition		(Transition*&);
void	SaveSubTransition	(char*);
Page*	CheckAndSavePage	(char*);


%}
%union
{	
	//-----------------------------------------------
	//	"elmentare" typen
	//-----------------------------------------------
	double  		d_val;
	int     		i_val;
	char*   		s_val;
	TokenRef		_TokenRef;	
	ServerType		_ServerType;
	Operator		_Operator;
	//-----------------------------------------------
	//	"elmentare" Klassen
	//-----------------------------------------------
	Identifier*		_IdentifierPtr;
	Textual*		_TextualPtr;
	Delay*			_DelayPtr;
	Cell*			_CellPtr;
	//-----------------------------------------------
	// 	Transitionsbestandteile
	//-----------------------------------------------
	ArcBindElement*		_ArcBindElementPtr;
	ArcBindList*		_ArcBindListPtr;
	arc_binding*		_arc_binding_ptr;
	BindingElement*		_BindingElementPtr;
	BindingList*		_BindingListPtr;
	//-----------------------------------------------
	// 	Transitions Klassen
	//-----------------------------------------------
	Transition*		_TransitionPtr;
	TransitionList*		_TransitionListPtr;
	//-----------------------------------------------
	// 	Stellen Klassen
	//-----------------------------------------------
	Place*			_PlacePtr;
	PlaceList*		_PlaceListPtr;
	PlacePtrList*		_PlacePtrListPtr;
	PlaceHeaderList*	_PlaceHeaderListPtr;
	//-----------------------------------------------
	// 	Definitions Klassen
	//-----------------------------------------------
	Definition*		_DefinitionPtr;
	DefinitionList*		_DefinitionListPtr;
	//-----------------------------------------------
	// 	"die" Seiten	
	//-----------------------------------------------
	Page*			_PagePtr;
}
%token	INT_CONST REAL_CONST STRING 						/* elementare Typen		*/
%token  COMMA POINT DOUBLEPOINT SEMICOLON QUESTIONMARK 				/* Zeichensetzung		*/
%token	OPEN_I OPEN_II OPEN_III CLOSE_I CLOSE_II CLOSE_III			/* Klammerung			*/
%token  PLUS MINUS DIVIDE MULTIPLY RAUTE TIMES AT 				/* algeb. Operatoren 		*/
%token	NOT AND OR WAHR FALSCH							/* log.   Operatoren		*/
%token  EQUAL LESS GREATER NOT_EQUAL LESS_OR_EQUAL GREATER_OR_EQUAL 		/* comp.  Operatoren		*/
%token  EXPONENTIAL DETERMINISTIC GENERAL IMMEDIATE UNIFORM			/* Delay-Typen			*/
%token  TRANSITION DEFINITION ELEMENTARYPLACE PRODUCTPLACE PAGE PARAMETER	/* Sections-Bezeichner		*/
%token  EVALUATE 
%token  E ES IS IF INARC OUTARC ARROW ASSIGN SUBPAGE				/* Seiten-Struktur-Elemente	*/
%token  LOGIC NUMERIC E_MSET P_MSET C_FUNCTION A_FUNCTION			/* Cell-Rueckgabewerte		*/
%token	EXPECT_OPEN PROB_OPEN TP_OPEN						/* Beginn eines Rating-Ausdrucks*/

%left   AND OR 
%left   NOT EQUAL 
%left   PLUS MINUS
%left   MULTIPLY DIVIDE
%left   RAUTE AT 

/****************************************************************************************/
/*				Elementare-Typen					*/
/****************************************************************************************/
%type	<s_val> 		GENERAL 
%type	<s_val> 		STRING 
%type	<i_val> 		INT_CONST
%type	<d_val> 		REAL_CONST
%type	<_Operator>		comp_op
%type   <_CellPtr>      	LOGIC
%type   <_CellPtr>      	NUMERIC
%type	<_CellPtr>		int_const
%type	<_CellPtr>		real_const 
%type	<_CellPtr>		real_expr 
%type   <_CellPtr>              comparison
%type   <_CellPtr>              logic_condition

/****************************************************************************************/
/*			Token/Token-Multisets						*/
/****************************************************************************************/
%type   <_TokenRef> 		unit
%type	<s_val>		    	part_version 
%type	<_CellPtr>		e_mset_const 	  	p_mset_const
%type	<_CellPtr>		e_mset_const_expr 	p_mset_const_expr
%type	<_CellPtr>		mset_const_expr

%type	<s_val>			c_function_def	  	a_function_def
%type   <_CellPtr>              C_FUNCTION 		A_FUNCTION
%type	<_CellPtr>		contents_function 	absolut_function

%type	<_CellPtr>		e_mset_0		p_mset_0
%type	<_CellPtr>		e_mset_1		p_mset_1
%type	<_CellPtr>		e_mset 			p_mset
%type   <_CellPtr>      	E_MSET 			P_MSET 
%type	<_CellPtr>		multiset_expr

/****************************************************************************************/
/*			Transitionen/Transitions-Bestandteile				*/
/****************************************************************************************/
%type	<d_val>			weight
%type	<_ServerType>		server_type
%type	<_DelayPtr>		delay
%type	<_CellPtr>		guard

%type	<_ArcBindElementPtr>	in_arc_bind 	 out_arc_bind
%type	<_ArcBindListPtr>	in_arc_bind_list out_arc_bind_list
%type	<_arc_binding_ptr>	arc_bindings	
%type	<_BindingElementPtr>	binding
%type	<_BindingListPtr>	binding_list

%type	<_TransitionPtr>	transition
%type	<_TransitionListPtr>	transitions

/****************************************************************************************/
/*			Subtransitionen/Subtransitions-Bestandteile			*/
/****************************************************************************************/
%type	<i_val>			port_assignment
%type	<i_val>			port_assignment_list
%type	<s_val>			sub_trans_definition

%type	<s_val>			sub_transition
%type	<i_val>			sub_transitions

%type	<_TransitionListPtr>	all_transitions

/****************************************************************************************/
/*					Stellen						*/
/****************************************************************************************/
%type	<_PlacePtr>		place
%type	<_PlacePtrListPtr>	product_places 
%type	<_PlacePtrListPtr>	elementary_places
%type	<_PlacePtrListPtr>	all_places

/****************************************************************************************/
/*		   Definitionen/Parameter/Evaluate-Definitionen				*/
/****************************************************************************************/
%type	<i_val> 		definition_def
%type	<i_val> 		parameter_def
%type	<_DefinitionPtr> 	definition	
%type	<_DefinitionListPtr> 	definitions	
%type	<_DefinitionListPtr> 	all_parameters	
%type	<i_val> 		all_definitions	
%type	<i_val> 		all_evaluations	

/****************************************************************************************/
/*					Evaluate					*/
/****************************************************************************************/
%type	<_CellPtr>		expectation_value
%type	<_CellPtr>		probability
%type	<_CellPtr>		bayes_rule_prob
%type	<_CellPtr>		bayes_rule_expect
%type	<_CellPtr>		through_put
%type   <_CellPtr>		eval_expr
%type   <i_val>                 eval_def

/****************************************************************************************/
/*					Seiten						*/
/****************************************************************************************/
%type	<_PagePtr>		page_def
%type	<_PagePtr>		page_param
%type	<i_val>			page
%type	<i_val>			pages

%%
/********************************************************************************************************************/
/************************************************ Gesamter-String  **************************************************/
/********************************************************************************************************************/
page_table		: all_definitions pages 								{ ; }
/********************************************************************************************************************/
/************************************************ Seiten-Eintraege **************************************************/
/********************************************************************************************************************/
pages			: page 							 { $$ = 0;  		    	    }
			| pages page				   		 { $$ = 0;  		    	    }
/*------------------------------------------------------------------------------------------------------------------*/
page			: page_param all_places all_transitions all_evaluations  
			{ 
				if ($2) { $1->AddPlaces($2); }
				if ($3) { $1->AddTransitions(*$3); $3->Clear(); delete $3; }
				$$ = 0;
			}
/*------------------------------------------------------------------------------------------------------------------*/
page_param		: all_parameters page_def 			 
			{ 
				if ($1) $2->AddParameters(*$1); 
				$$ = $2;  
			}
/*------------------------------------------------------------------------------------------------------------------*/
page_def		: PAGE DOUBLEPOINT STRING 				 { $$ = CheckAndSavePage($3); 	    }

/********************************************************************************************************************/
/*********************************** Definitionen / Parameter / Eval-Definitionen ***********************************/
/********************************************************************************************************************/
all_definitions		: definition_def 					{ $$ = 0; 	    	    	    } 
			| definition_def definitions		         	{ $$ = 0; 	    	    	    }
/*------------------------------------------------------------------------------------------------------------------*/
all_parameters		: parameter_def				 		{ $$ = NULL; 	    	    	    }
			| parameter_def definitions			 	{ $$ = $2;			    }
/*------------------------------------------------------------------------------------------------------------------*/
all_evaluations		: eval_def				 		{ $$ = 0; 	    	    	    }
			| eval_def definitions			 		{ $$ = 0;			    }
/*------------------------------------------------------------------------------------------------------------------*/
definitions		: definition		    		      		      
			{ 
				switch (section)
				{
					case	def_sec	  :	SaveDefinition($1);         $$ = NULL; break;
					case	def_end	  :	SaveDefinition($1);         $$ = NULL; break;
					case	eval_sec  :	EvalRefList.Append(*$1);    $$ = NULL; break;
					case	eval_end  :	EvalRefList.Append(*$1);    $$ = NULL; break;
					case	param_sec :	$$ = new DefinitionList(*$1);          break;
					case	param_end :	$$ = new DefinitionList(*$1);          break;
					default	 	  :	yyerror("definition in wrong section");	
				}
				$1->Clear(); delete $1;
			}
			| definitions OR definition				      
			{ 
				switch (section)
				{
					case	def_sec	  :	SaveDefinition($3);      $$ = NULL; break;
					case	def_end	  :	SaveDefinition($3);      $$ = NULL; break;
					case	eval_sec  :	EvalRefList.Append(*$3); $$ = NULL; break;
					case	eval_end  :	EvalRefList.Append(*$3); $$ = NULL; break;
					case	param_sec :	$1->Append(*$3);         $$ = $1;   break;	
					case	param_end :	$1->Append(*$3);         $$ = $1;   break;	
					default	 	  :	yyerror("definition in wrong section");	
				}
				$3->Clear(); delete $3;
			}
/*------------------------------------------------------------------------------------------------------------------*/
definition		: STRING EQUAL real_expr    	 			
			{ 
				$$ = new Definition($1, $3); 	    
				$3->Clear(); delete $3;
			}
			| STRING EQUAL logic_condition   			
			{ 
				$$ = new Definition($1, $3); 	    
				$3->Clear(); delete $3;
			}
			| STRING EQUAL multiset_expr
			{ 
				if (section==eval_sec || section==eval_end) 
					YaccError("mset definition in eval section", $1);
				$$ = new Definition($1, $3); 	    
				$3->Clear(); delete $3;
			}
			| STRING EQUAL eval_expr   				
			{ 
				if (section!=eval_sec && section!=eval_end) 
					YaccError("eval definition in wrong section", $1);
				$$ = new Definition($1, $3); 	    
				$3->Clear(); delete $3;
			}
			| STRING EQUAL STRING 	   
			{
				if (!strcmp($1,$3)) 
					YaccError("definition loop", $1);
				$$ = new Definition($1,new ConstVal($3)); 
			}
/*------------------------------------------------------------------------------------------------------------------*/
parameter_def		: PARAMETER  DOUBLEPOINT				{ $$ = 0;			    }
definition_def		: DEFINITION DOUBLEPOINT				{ $$ = 0;                           }
eval_def     		: EVALUATE   DOUBLEPOINT				{ $$ = 0;                           }

/********************************************************************************************************************/
/************************************************ Stellen ***********************************************************/
/********************************************************************************************************************/
all_places		: elementary_places product_places			
			{ 
				if (!$1 && !$2) $$ = NULL;
				if ( $1 &&  $2) $$ = new PlacePtrList(*$1,*$2);
				else
				{
					if ($1) $$ = $1;
					else    $$ = $2;
				}
			}
			| product_places elementary_places			
			{ 
				if (!$1 && !$2) $$ = NULL;
				if ( $1 &&  $2) $$ = new PlacePtrList(*$1,*$2);
				else
				{
					if ($1) $$ = $1;
					else    $$ = $2;
				}
			}
/*------------------------------------------------------------------------------------------------------------------*/
elementary_places	: ELEMENTARYPLACE DOUBLEPOINT				{ $$ = NULL; 			    }
			| ELEMENTARYPLACE DOUBLEPOINT place 			
			{ 
				Place* p=SavePlace($3, elementary); 
				if (p) $$ = new PlacePtrList(p); 
				else   $$ = new PlacePtrList();
			}
			| elementary_places OR place				
			{ 
				Place* p=SavePlace($3, elementary); 
				if (p) $1->Append(p);
				$$ = $1;
			}
/*------------------------------------------------------------------------------------------------------------------*/
product_places		: PRODUCTPLACE DOUBLEPOINT				{ $$ = NULL; 		    	    }
			| PRODUCTPLACE DOUBLEPOINT place			
			{ 
				Place* p=SavePlace($3, object);
				if (p) $$ = new PlacePtrList(p); 
				else   $$ = new PlacePtrList();
			}
			| product_places OR place				
			{ 
				Place* p=SavePlace($3, object); 
				if (p) $1->Append(p);
				$$ = $1;
			}
/*------------------------------------------------------------------------------------------------------------------*/
place                   : STRING SEMICOLON 			SEMICOLON  	     
			{ 
				$$ = new Place($1, place_no++, quantitativ, new ConstVal(UNFINITE)); 
			}
			| STRING SEMICOLON real_expr 		SEMICOLON             	
			{ 
				$$ = new Place($1, place_no++, quantitativ, $3); 
				$3->Clear(); delete $3;
			}
			| STRING SEMICOLON logic_condition	SEMICOLON       	
			{ 
				$$ = new Place($1, place_no++, logic, $3); 
				$3->Clear(); delete $3;
			}
			| STRING SEMICOLON real_expr 		SEMICOLON multiset_expr 
			{
				CellVal* cv = $5->Calculate();
				$$ = new Place($1, place_no++, quantitativ, $3, 
					       copy_token_multiset(cv->Gettoken_multiset()));
				cv->Clear(); delete cv;
				$3->Clear(); delete $3;
				$5->Clear(); delete $5;
			}
			| STRING SEMICOLON logic_condition 	SEMICOLON multiset_expr 
			{
				CellVal* cv = $5->Calculate();
				$$ = new Place($1, place_no++, logic, $3, 
					       copy_token_multiset(cv->Gettoken_multiset()));
				cv->Clear(); delete cv;
				$3->Clear(); delete $3;
				$5->Clear(); delete $5;
			}
			| STRING SEMICOLON 			SEMICOLON multiset_expr               
			{ 
				CellVal* cv = $4->Calculate();
				$$ = new Place($1, place_no++, quantitativ, new ConstVal(UNFINITE), 
					       copy_token_multiset(cv->Gettoken_multiset()));
				cv->Clear(); delete cv;
				$4->Clear(); delete $4;
			}
/********************************************************************************************************************/
/************************************************ Transitionen ******************************************************/
/********************************************************************************************************************/
all_transitions		: TRANSITION DOUBLEPOINT				      { $$ = NULL;		    }
			| TRANSITION DOUBLEPOINT sub_transitions		      { $$ = NULL;  		    }
			| TRANSITION DOUBLEPOINT transitions 		     	      { $$ = $3;  		    }
			| TRANSITION DOUBLEPOINT transitions     sub_transitions      { $$ = $3;  		    }
			| TRANSITION DOUBLEPOINT sub_transitions transitions 	      { $$ = $4;  		    }
/*------------------------------------------------------------------------------------------------------------------*/
transitions             : transition                                                  
			{ 
				SaveTransition($1);
				$$ = new TransitionList(*$1);
				$1->Clear(); delete $1;
			}
			| transitions transition				      
			{ 
				SaveTransition($2); 
				$1->Append(*$2);
				$$ = $1; 
				$2->Clear(); delete $2;
			}
/*------------------------------------------------------------------------------------------------------------------*/
transition		: STRING                	OPEN_II binding_list CLOSE_II
			{
				CheckAndCountTransition($1,Exclusive,$3);
				$$ =     new Transition($1,Exclusive,$3,new ConstVal(true));
				$3->Clear(); delete $3;
			} 
			| STRING 		guard 	OPEN_II binding_list CLOSE_II  
			{
				CheckAndCountTransition($1,Exclusive,$4);
				$$ =     new Transition($1,Exclusive,$4,$2);
				$2->Clear(); delete $2;
				$4->Clear(); delete $4;
			}
			| STRING server_type 		OPEN_II binding_list CLOSE_II  
			{
				CheckAndCountTransition($1,$2,$4);
				$$ =     new Transition($1,$2,$4,new ConstVal(true));
				$4->Clear(); delete $4;
			}
			| STRING server_type 	guard 	OPEN_II binding_list CLOSE_II  
			{
				CheckAndCountTransition($1,$2,$5);
				$$ =     new Transition($1,$2,$5,$3);
				$3->Clear(); delete $3; 
				$5->Clear(); delete $5;
			}
/********************************************************************************************************************/
/************************************************ Subtransitionen ***************************************************/
/********************************************************************************************************************/
sub_transitions		: sub_transition					  { SaveSubTransition($1); $$ = 0;  }
			| sub_transitions sub_transition			  { SaveSubTransition($2); $$ = 0;  }
/*------------------------------------------------------------------------------------------------------------------*/
sub_transition		: sub_trans_definition ASSIGN port_assignment_list CLOSE_II  			 { $$ = $1; }
/*------------------------------------------------------------------------------------------------------------------*/
port_assignment_list	: port_assignment			      		     			 { $$ = $1; } 
			| port_assignment_list COMMA port_assignment  		     			 { $$ = $3; }
/*------------------------------------------------------------------------------------------------------------------*/
port_assignment		: STRING ARROW STRING	        
			{ 
				SocketPortPair* spp = new SocketPortPair(GetPlaceRef($1), MakePortName($3));
			  	if (SocketPortRefList.UniqueAppend(*spp) < 0)
			  		YaccError("socket-port-pair exists", $1);
			  	spp->Clear(); delete spp;
			  	$$ = 0; 
			}
/*------------------------------------------------------------------------------------------------------------------*/
sub_trans_definition	: STRING OPEN_II SUBPAGE STRING SEMICOLON
			{
				if (PageRefList.GetPos($4) > 0) 
					YaccError("page-loop in subtransition", $4); 

				Page* p;
				if (!(p = PageRefList.GetElementPtr(actual_page_name)))
					YaccError("master-page of subtransition is not defined", $4);
				p->AddSubPage($4);
				MakeActualSubPageName($4);
				$$ = $1;
			}
/********************************************************************************************************************/
/********************************************* zu evaluierende Ausdruecke *******************************************/
/********************************************************************************************************************/
eval_expr		: expectation_value					{ $$=$1;			    }
			| probability						{ $$=$1;			    }
			| bayes_rule_prob					{ $$=$1;			    }
			| bayes_rule_expect					{ $$=$1;			    }
			| through_put						{ $$=$1;			    }
			| OPEN_I eval_expr CLOSE_I				{ $$=$2;			    }
			| MINUS  eval_expr 					
			{ 
				$$=new UnaerOp (minus ,$2);
				$2->Clear(); delete $2;
			}
			| eval_expr 	MULTIPLY eval_expr			
			{ 
				$$=new BinaerOp(multiply,$1, $3); 
				$1->Clear(); delete $1;
				$3->Clear(); delete $3;
			}
			| eval_expr 	DIVIDE   eval_expr			
			{ 
				$$=new BinaerOp(divide,$1, $3); 
				$1->Clear(); delete $1; 
				$3->Clear(); delete $3;
			}
			| eval_expr 	MINUS    eval_expr			
			{ 
				$$=new BinaerOp(minus,$1, $3); 
				$1->Clear(); delete $1; 
				$3->Clear(); delete $3;
			}
			| eval_expr	PLUS     eval_expr			
			{ 
				$$=new BinaerOp(plus,$1, $3); 
				$1->Clear(); delete $1; 
				$3->Clear(); delete $3;
			}
			| real_expr 	MULTIPLY eval_expr			
			{ 
				$$=new BinaerOp(multiply,$1, $3); 
				$1->Clear(); delete $1; 
				$3->Clear(); delete $3;
			}
			| real_expr 	DIVIDE   eval_expr			
			{ 
				$$=new BinaerOp(divide,$1, $3); 
				$1->Clear(); delete $1; 
				$3->Clear(); delete $3;
			}
			| real_expr 	MINUS    eval_expr			
			{ 
				$$=new BinaerOp(minus,$1, $3); 
				$1->Clear(); delete $1; 
				$3->Clear(); delete $3;
			}
			| real_expr 	PLUS     eval_expr			
			{ 
				$$=new BinaerOp(plus,$1, $3); 
				$1->Clear(); delete $1; 
				$3->Clear(); delete $3;
			}
			| eval_expr 	MULTIPLY real_expr			
			{ 
				$$=new BinaerOp(multiply,$1, $3); 
				$1->Clear(); delete $1; 
				$3->Clear(); delete $3;
			}
			| eval_expr 	DIVIDE   real_expr			
			{ 
				$$=new BinaerOp(divide,$1, $3); 
				$1->Clear(); delete $1; 
				$3->Clear(); delete $3;
			}
			| eval_expr 	MINUS    real_expr			
			{ 
				$$=new BinaerOp(minus,$1, $3); 
				$1->Clear(); delete $1; 
				$3->Clear(); delete $3;
			}
			| eval_expr 	PLUS     real_expr			
			{ 
				$$=new BinaerOp(plus,$1, $3); 
				$1->Clear(); delete $1; 
				$3->Clear(); delete $3;
			}
/*------------------------------------------------------------------------------------------------------------------*/
bayes_rule_expect 	: EXPECT_OPEN real_expr IF logic_condition CLOSE_II 
			{ 
				$$=new RatingCell(bayes_e,$2,$4); 
				$2->Clear(); delete $2; 
				$4->Clear(); delete $4;
			}
			| EXPECT_OPEN real_expr IF NUMERIC CLOSE_II 
			{ 
				$$=new RatingCell(bayes_e,$2,$4); 
				$2->Clear(); delete $2; 
				$4->Clear(); delete $4;
			}
			| EXPECT_OPEN real_expr IF INT_CONST CLOSE_II 
			{ 
				$$=new RatingCell(bayes_e,$2,$4); 
				$2->Clear(); delete $2;  
			}

bayes_rule_prob 	: PROB_OPEN logic_condition IF logic_condition CLOSE_II 
			{ 
				$$=new RatingCell(bayes_p,$2,$4); 
				$2->Clear(); delete $2; 
				$4->Clear(); delete $4;
			}
			| PROB_OPEN logic_condition IF NUMERIC CLOSE_II 
			{ 
				$$=new RatingCell(bayes_p,$2,$4); 
				$2->Clear(); delete $2; 
				$4->Clear(); delete $4;
			}
			| PROB_OPEN logic_condition IF INT_CONST CLOSE_II 
			{ 
				$$=new RatingCell(bayes_p,$2,$4); 
				$2->Clear(); delete $2;
			}
			| PROB_OPEN NUMERIC IF NUMERIC CLOSE_II 
			{ 
				$$=new RatingCell(bayes_p,$2,$4); 
				$2->Clear(); delete $2; 
				$4->Clear(); delete $4;
			}
			| PROB_OPEN NUMERIC IF logic_condition CLOSE_II 
			{ 
				$$=new RatingCell(bayes_p,$2,$4); 
				$2->Clear(); delete $2; 
				$4->Clear(); delete $4;
			}
			| PROB_OPEN NUMERIC IF INT_CONST CLOSE_II 
			{ 
				$$=new RatingCell(bayes_p,$2,$4);
				$2->Clear(); delete $2;
			}
			| PROB_OPEN INT_CONST IF INT_CONST CLOSE_II 
			{ 
				$$=new RatingCell(bayes_p,$2,$4); 
			}
			| PROB_OPEN INT_CONST IF NUMERIC CLOSE_II 
			{ 
				$$=new RatingCell(bayes_p,$2,$4); 
				$4->Clear(); delete $4;
			}
			| PROB_OPEN INT_CONST IF logic_condition CLOSE_II 
			{ 
				$$=new RatingCell(bayes_p,$2,$4); 
				$4->Clear(); delete $4;
			}

expectation_value	: EXPECT_OPEN real_expr CLOSE_II       { $$=new RatingCell(expect,$2);$2->Clear();delete $2;}

probability		: PROB_OPEN logic_condition    CLOSE_II	 { $$=new RatingCell(prob,$2);$2->Clear();delete $2;}
			| PROB_OPEN NUMERIC	       CLOSE_II	 { $$=new RatingCell(prob,$2);$2->Clear();delete $2;}
			| PROB_OPEN INT_CONST	       CLOSE_II	 { $$=new RatingCell(prob,$2);      		    }

through_put		: TP_OPEN   STRING	       CLOSE_II	 { $$=new RatingCell(through_p,$2); 		    }

/********************************************************************************************************************/
/****************************************** Transitions-Bestandteile ************************************************/
/********************************************************************************************************************/
binding_list		: binding			 { $$ = new BindingList(*$1);        $1->Clear(); delete $1; }
			| binding_list OR binding	 { $1->Append(*$3); $$ = $1;         $3->Clear(); delete $3; }
/*------------------------------------------------------------------------------------------------------------------*/
binding			: guard arc_bindings SEMICOLON delay 
			{
				$$ = new BindingElement(DEF_ES_WEIGHT,$1,$4,$2->in,$2->out);
				$1->Clear(); delete $1;
				$4->Clear(); delete $4;
				delete $2; 
			}
			| guard arc_bindings weight SEMICOLON delay 
			{
				if ($3 <= 0) yyerror("unallowed weight");
				$$ = new BindingElement($3,$1,$5,$2->in,$2->out);
				$1->Clear(); delete $1;
				$5->Clear(); delete $5;
				delete $2; 
			}
			| arc_bindings weight SEMICOLON delay 
			{
				if ($2 <= 0) yyerror("unallowed weight");
				$$ = new BindingElement($2,new ConstVal(true),$4,$1->in,$1->out);
				$4->Clear(); delete $4;
				delete $1;
			}
			| arc_bindings SEMICOLON delay 
			{
				$$ = new BindingElement(DEF_ES_WEIGHT,new ConstVal(true),$3,$1->in,$1->out);
				$3->Clear(); delete $3; 
				delete $1;
			}
/*------------------------------------------------------------------------------------------------------------------*/
guard			: OPEN_I logic_condition CLOSE_I				{ $$ = $2;		    }
			| OPEN_I real_expr       CLOSE_I				{ $$ = $2;		    }
			| OPEN_I 		 CLOSE_I				{ $$ = new ConstVal(true);  }
/*------------------------------------------------------------------------------------------------------------------*/
weight			: SEMICOLON INT_CONST					      	{ $$ = double($2);	    }
			| SEMICOLON REAL_CONST					      	{ $$ = $2;		    }
/*------------------------------------------------------------------------------------------------------------------*/
server_type		: ES					    		      	{ $$ = Exclusive;	    }
			| IS					    		      	{ $$ = Infinite;	    }
/*------------------------------------------------------------------------------------------------------------------*/
delay			: IMMEDIATE				 { $$=new Immediate();	    			    }
			| IMMEDIATE     OPEN_I real_expr CLOSE_I { $$=new Immediate($3);     $3->Clear(); delete $3;}
			| EXPONENTIAL   OPEN_I real_expr CLOSE_I { $$=new Exponential($3);   $3->Clear(); delete $3;} 
			| DETERMINISTIC OPEN_I real_expr CLOSE_I { $$=new Deterministic($3); $3->Clear(); delete $3;}
			| GENERAL       			 { $$=new General($1);                              }
			| UNIFORM	OPEN_I real_expr COMMA real_expr CLOSE_I
			{ 
				if ($3->IsPlaceDependent() || $5->IsPlaceDependent())
					YaccError("placedependent uniform delay is not allowed", NULL);

				double beg = $3->CalculateFloat();
				double end = $5->CalculateFloat();
				$3->Clear(); delete $3;
				$5->Clear(); delete $5;

				if (beg > end) 
					YaccError("timeinterval for uniform delay is not correct", NULL);

				$$=new UniForm(beg,end);    
			}
/*------------------------------------------------------------------------------------------------------------------*/
arc_bindings		: out_arc_bind_list			   
			{
				arc_binding* arc_b = new arc_binding;
				arc_b->in	   = NULL;
				arc_b->out	   = $1;
				$$ = arc_b;
			}
			| in_arc_bind_list
			{
				arc_binding* arc_b = new arc_binding;
				arc_b->in	   = $1;
				arc_b->out	   = NULL;
				$$ = arc_b;
			}
			| in_arc_bind_list COMMA out_arc_bind_list
			{
				arc_binding* arc_b = new arc_binding;
				arc_b->in	   = $1;
				arc_b->out	   = $3;
				$$ = arc_b;
			}
/*------------------------------------------------------------------------------------------------------------------*/
out_arc_bind_list	: out_arc_bind				    		
			{ 
				$$ = new ArcBindList(*$1);   	    
				$1->Clear(); delete $1;
			}
			| out_arc_bind_list COMMA out_arc_bind 	    		
			{ 
				$1->Append(*$3); 
				$$ = $1; 	    
				$3->Clear(); delete $3;
			}
/*------------------------------------------------------------------------------------------------------------------*/
in_arc_bind_list	: in_arc_bind				                
			{ 
				$$ = new ArcBindList(*$1);   	    
				$1->Clear(); delete $1;
			}
			| in_arc_bind_list COMMA in_arc_bind 	    		
			{ 
				$1->Append(*$3); 
				$$ = $1; 	    
				$3->Clear(); delete $3;
			}
/*------------------------------------------------------------------------------------------------------------------*/
in_arc_bind		: INARC  STRING EQUAL multiset_expr	
			{ 
				if (strncmp(actual_page_name, $2, strlen(actual_page_name)) != 0)
					YaccError("arc-bind-place is not from this page", $2);
				$$ = new ArcBindElement(CheckIfPortGetRef($2),$4);
				$4->Clear(); delete $4;
			}
out_arc_bind		: OUTARC STRING EQUAL multiset_expr	
			{ 
				if (strncmp(actual_page_name, $2, strlen(actual_page_name)) != 0)
					YaccError("arc-bind-place is not from this page", $2);
				$$ = new ArcBindElement(CheckIfPortGetRef($2),$4);
				$4->Clear(); delete $4;
			}

/********************************************************************************************************************/
/************************************************ Logic-Condition ***************************************************/
/********************************************************************************************************************/
logic_condition		: comparison					    	{ $$ = $1; 			    }
			| LOGIC							{ $$ = $1;			    }
			| WAHR						    	{ $$ = new ConstVal(true);  	    }
			| FALSCH					    	{ $$ = new ConstVal(false); 	    }
			| MINUS 					    	{ $$ = new ConstVal(true);  	    }
			| OPEN_I logic_condition CLOSE_I		    	{ $$ = $2;        		    }
			| NOT logic_condition				    	
			{ 
				$$ = new UnaerOp(logic_not,$2);   
				$2->Clear(); delete $2; 
			}
			| logic_condition AND logic_condition		    	
			{ 
				$$ = new BinaerOp(logic_and,$1,$3);
				$1->Clear(); $3->Clear();
				delete $1; delete $3;
			}
			| logic_condition OR  logic_condition		    	
			{ 
				$$ = new BinaerOp(logic_or,$1,$3);
				$1->Clear(); $3->Clear();
				delete $1; delete $3;
			}
/*------------------------------------------------------------------------------------------------------------------*/
comparison		: real_expr comp_op real_expr	    	    	
			{ 
				$$ = new BinaerOp($2,$1,$3);      
				$1->Clear(); $3->Clear();
				delete $1; delete $3;
			}
			| multiset_expr comp_op multiset_expr	    	    	
			{ 	
				$$ = new BinaerOp($2,$1,$3);      
				$1->Clear(); $3->Clear();
				delete $1; delete $3;
			}
/*------------------------------------------------------------------------------------------------------------------*/
comp_op			: EQUAL						    	{ $$ = equal; 		            }
			| LESS						    	{ $$ = less;  	 	            }
			| GREATER					    	{ $$ = greater; 		    }
			| NOT_EQUAL					    	{ $$ = not_equal;		    }
			| LESS_OR_EQUAL					    	{ $$ = less_or_equal; 	            }
			| GREATER_OR_EQUAL				    	{ $$ = greater_or_equal; 	    }

/********************************************************************************************************************/
/************************************** Token-Multisets/ Token-Multiset-Expr  ***************************************/
/********************************************************************************************************************/
multiset_expr		: p_mset						{ $$ = $1; 			    }
			| e_mset						{ $$ = $1; 			    } 
			| contents_function					{ $$ = $1; 			    }
			| OPEN_I multiset_expr CLOSE_I			     	{ $$ = $2; 			    }
			| MINUS multiset_expr				     	
			{ 
				$$ = new UnaerOp(minus,$2);       
				$2->Clear(); delete $2;
			}
			| multiset_expr PLUS     multiset_expr		     	
			{ 
				$$ = new BinaerOp(plus,$1,$3);    
				$1->Clear(); delete $1;
				$3->Clear(); delete $3;
			}
			| multiset_expr MINUS    multiset_expr		     	
			{ 
				$$ = new BinaerOp(minus,$1,$3);   
				$1->Clear(); delete $1;
				$3->Clear(); delete $3;
			}
			| real_expr     MULTIPLY multiset_expr		     	
			{ 
				$$ = new BinaerOp(multiply,$1,$3);
				$1->Clear(); delete $1;
				$3->Clear(); delete $3;
			}
			| multiset_expr DIVIDE   real_expr	             	
			{ 
				$$ = new BinaerOp(divide,$1,$3);  
				$1->Clear(); delete $1;
				$3->Clear(); delete $3;
			}
/*------------------------------------------------------------------------------------------------------------------*/
p_mset               	: p_mset_const                                  	{ $$ = $1; 			    }
			| P_MSET                                        	{ $$ = $1; 			    }
			| p_mset_1 CLOSE_III					{ $$ = $1; 			    }

e_mset         		: e_mset_const                                  	{ $$ = $1; 			    }
			| E_MSET                                        	{ $$ = $1; 			    }
			| e_mset_1 CLOSE_III					{ $$ = $1; 			    }

p_mset_1		: OPEN_III p_mset_0					{ $$ = $2; 			    }
			| OPEN_III P_MSET      					{ $$ = $2; 			    }
			| OPEN_III p_mset_const					{ $$ = $2; 			    }
			| p_mset_1 COMMA p_mset_0				
			{ 
				$$ = new BinaerOp(plus, $1, $3);  
				$1->Clear(); delete $1;
				$3->Clear(); delete $3;
			}
			| p_mset_1 COMMA p_mset_const				
			{ 
				$$ = new BinaerOp(plus, $1, $3);  
				$1->Clear(); delete $1;
				$3->Clear(); delete $3;
			}

e_mset_1		: OPEN_III e_mset_0					{ $$ = $2; 			    }
			| OPEN_III E_MSET      					{ $$ = $2; 			    }
			| OPEN_III e_mset_const					{ $$ = $2; 			    }
			| e_mset_1 COMMA e_mset_0
			{ 
				$$ = new BinaerOp(plus, $1, $3);  
				$1->Clear(); delete $1;
				$3->Clear(); delete $3;
			}
			| e_mset_1 COMMA e_mset_const
			{ 
				$$ = new BinaerOp(plus, $1, $3);  
				$1->Clear(); delete $1;
				$3->Clear(); delete $3;
			}

p_mset_0		: OPEN_I real_expr CLOSE_I MULTIPLY p_mset_const 	
			{ 
				$$ = new BinaerOp(multiply,$2,$5);
				$2->Clear(); delete $2;
				$5->Clear(); delete $5;
			}
e_mset_0		: OPEN_I real_expr CLOSE_I MULTIPLY e_mset_const 	
			{ 
				$$ = new BinaerOp(multiply,$2,$5);
				$2->Clear(); delete $2;
				$5->Clear(); delete $5;
			}

/********************************************************************************************************************/
/************************************************ reals /real_expr **************************************************/
/********************************************************************************************************************/
real_expr		: NUMERIC                                            	{ $$ = $1; 			    }
			| int_const					     	{ $$ = $1; 			    }
			| real_const					     	{ $$ = $1; 	    		    }
			| absolut_function					{ $$ = $1; 			    }
			| OPEN_I real_expr CLOSE_I			     	{ $$ = $2; 			    }
			| MINUS real_expr				     	
			{ 
				$$ = new UnaerOp(minus,$2);       
				$2->Clear(); delete $2;
			}
			| real_expr PLUS     real_expr			     	
			{ 
				$$ = new BinaerOp(plus,$1,$3);    
				$1->Clear(); delete $1;
				$3->Clear(); delete $3;
			}
			| real_expr MINUS    real_expr			     	
			{ 
				$$ = new BinaerOp(minus,$1,$3);   
				$1->Clear(); delete $1;
				$3->Clear(); delete $3;
			}
			| real_expr DIVIDE   real_expr			     	
			{ 
				$$ = new BinaerOp(divide,$1,$3);  
				$1->Clear(); delete $1;
				$3->Clear(); delete $3;
			}
			| real_expr MULTIPLY real_expr			     	
			{ 
				$$ = new BinaerOp(multiply,$1,$3);
				$1->Clear(); delete $1;
				$3->Clear(); delete $3;
			}
/*------------------------------------------------------------------------------------------------------------------*/
real_const              : REAL_CONST                                            { $$ = new ConstVal($1); 	    }
int_const               : INT_CONST                                             { $$ = new ConstVal($1); 	    }

/********************************************************************************************************************/
/************************************************ functions *********************************************************/
/********************************************************************************************************************/
absolut_function	: A_FUNCTION                            	{$$=$1; 			    	    }
			| a_function_def			        {$$=new Function(absolut,$1);    	    }
			| a_function_def OPEN_I p_mset_const	CLOSE_I {$$=new Function(absolut,$1,$3);$3->Clear();delete $3;}
			| a_function_def OPEN_I e_mset_const 	CLOSE_I {$$=new Function(absolut,$1,$3);$3->Clear();delete $3;}
			| a_function_def OPEN_I mset_const_expr	CLOSE_I {$$=new Function(absolut,$1,$3);$3->Clear();delete $3;}

a_function_def		: RAUTE STRING		
			{ 
				if (section == def_sec) $$ = $2;
				else			$$ = CheckIfPortGetName($2);
			}
/*------------------------------------------------------------------------------------------------------------------*/
contents_function	: C_FUNCTION                           		{$$=$1; 			    	    }
			| c_function_def				{$$=new Function(contents,$1);   	    }
			| c_function_def OPEN_I p_mset_const    CLOSE_I	{$$=new Function(contents,$1,$3);$3->Clear();delete $3;}
			| c_function_def OPEN_I e_mset_const    CLOSE_I {$$=new Function(contents,$1,$3);$3->Clear();delete $3;}
			| c_function_def OPEN_I mset_const_expr	CLOSE_I {$$=new Function(contents,$1,$3);$3->Clear();delete $3;}

c_function_def		: AT STRING		
			{ 
				if (section == def_sec) $$ = $2;
				else			$$ = CheckIfPortGetName($2);
			}
/********************************************************************************************************************/
/************************************* Token-Multiset-Const/ Token-Multiset-Const-Expr ******************************/
/********************************************************************************************************************/
mset_const_expr		: p_mset_const_expr CLOSE_III		   { $$ = $1; 					    }
			| e_mset_const_expr CLOSE_III		   { $$ = $1; 					    }
/*------------------------------------------------------------------------------------------------------------------*/
p_mset_const_expr       : OPEN_III p_mset_const 		   { $$ = $2; 		    			    }
                        | p_mset_const_expr COMMA p_mset_const     
                        { 
                        	$$ = new BinaerOp(plus,$1,$3); 		    
				$1->Clear(); delete $1;
				$3->Clear(); delete $3;
                        }

e_mset_const_expr	: OPEN_III e_mset_const 		   { $$ = $2; 		    			    }
			| e_mset_const_expr COMMA e_mset_const     
			{ 
				$$ = new BinaerOp(plus,$1,$3); 		    
				$1->Clear(); delete $1;
				$3->Clear(); delete $3;
			}
/*------------------------------------------------------------------------------------------------------------------*/
p_mset_const         	: unit                                     { $$ = new ConstVal(new TokenMSVal($1, 1)); 	    }
                        | INT_CONST TIMES unit                     
                        { 
                        	if ($1 > MAX_QUANT) $1 = MAX_QUANT;
                        	$$ = new ConstVal(new TokenMSVal($3,Quant($1))); 	    
                        }
/*------------------------------------------------------------------------------------------------------------------*/
e_mset_const		: E                 			   { $$ = new ConstVal(new TokenMSVal(NO_COLOR, 1));}
			| INT_CONST TIMES E 	   		   
			{ 
                        	if ($1 > MAX_QUANT) $1 = MAX_QUANT;
				$$ = new ConstVal(new TokenMSVal(NO_COLOR,Quant($1)));
			} 

/********************************************************************************************************************/
/************************************************ Token *************************************************************/
/********************************************************************************************************************/
unit			: STRING part_version STRING			
			{ 
			  	Token    tok(StripOfPath($1),$2,StripOfPath($3));
			  	int tr = TokenRefList.UniqueAppend(tok);
			  	if (tr < 0) tr = -tr;
			  	if (tr > MAX_TREF) yyerror("too much tokens defined (max = 65536)");
			  	$$ = TokenRef(tr);
			}
			| STRING part_version INT_CONST
			{
			  	Token    tok(StripOfPath($1),$2, GetNumberString($3));
			  	int tr = TokenRefList.UniqueAppend(tok);
			  	if (tr < 0) tr = -tr;
			  	if (tr > MAX_TREF) yyerror("too much tokens defined (max = 65536)");
			  	$$ = TokenRef(tr);
			}
			| STRING QUESTIONMARK				
			{ 
			  	Token    tok(StripOfPath($1), WILDCARD, WILDCARD);
			  	int tr = TokenRefList.UniqueAppend(tok);
			  	if (tr < 0) tr = -tr;
			  	if (tr > MAX_TREF) yyerror("too much tokens defined (max = 65536)");
			  	$$ = TokenRef(tr);
			}
/*------------------------------------------------------------------------------------------------------------------*/
part_version		: OPEN_II STRING       CLOSE_II 				 { $$ = StripOfPath($2);    }
			| OPEN_II QUESTIONMARK CLOSE_II					 { $$ = WILDCARD;   	    }
			| OPEN_II INT_CONST    CLOSE_II					 { $$ = GetNumberString($2);}
			| POINT								 { $$ = STDVERSION; 	    }
%%
#include "scan.cc"

/*------------------------------------------------------------------------------------------------------------------*/
/*	 Stelle in der PlaceRefList anhand des Stellennamens ermitteln und Pointer auf diese Stelle zurueckgeben    */
/*------------------------------------------------------------------------------------------------------------------*/
Place*	GetPlaceRef(char* placename)
{
	if (!PlaceRefList.size) YaccError("can't get placeref, PlaceRefList is empty", placename);

	for (PlaceListElement* place=PlaceRefList.first; place; place=place->next)
		if (strcmp(place->lelement.GetName(), placename) == 0) 
			return &(place->lelement);

	YaccError("can't get placeref, placename not found", placename);
}
/*------------------------------------------------------------------------------------------------------------------*/
/*	Ueberpruefen ob placename eine Portstelle ist								    */
/*		ja : dazugehoerige Socketstellenamen zurueckgeben						    */
/*	      nein : PlaceRefList nach placename durchsuchen 							    */
/*------------------------------------------------------------------------------------------------------------------*/
char*	CheckIfPortGetName(char* placename)
{
	if (SocketPortRefList.size)
	{
		for (SocketPortListElement* sp=SocketPortRefList.first; sp; sp=sp->next)
			if (sp->lelement.GetPort() == placename)
				return sp->lelement.GetSocket()->GetName();
	}

	if (!PlaceRefList.size) return placename; 

	for (PlaceListElement* place=PlaceRefList.first; place; place=place->next)
		if (strcmp(place->lelement.GetName(), placename) == 0) 
			return placename;

	return placename;
}
/*------------------------------------------------------------------------------------------------------------------*/
/*	Ueberpruefen ob placename eine Portstelle ist								    */
/*		ja : dazugehoerige Socketstelle zurueckgeben							    */
/*	      nein : placename in PlaceRefList suchen								    */
/*------------------------------------------------------------------------------------------------------------------*/
Place*	CheckIfPortGetRef(char* placename)
{
	if (SocketPortRefList.size)
	{
		for (SocketPortListElement* sp=SocketPortRefList.first; sp; sp=sp->next)
			if (sp->lelement.GetPort() == placename) 
				return sp->lelement.GetSocket();
	}
	return (GetPlaceRef(placename));
}
/*------------------------------------------------------------------------------------------------------------------*/
/*		 portnamen bauen					  				            */
/*------------------------------------------------------------------------------------------------------------------*/
char*	MakePortName(char* str)
{
	char*   local_port_name   = StripOfPath(str);
	char*   absolut_port_name = new char[strlen(actual_subpage_name)+strlen(local_port_name)+3];
	sprintf(absolut_port_name, "%s\\%s", actual_subpage_name, local_port_name);
	return absolut_port_name;
}
/*------------------------------------------------------------------------------------------------------------------*/
/*		 aktuellen Seitennamen setzen				  				            */
/*------------------------------------------------------------------------------------------------------------------*/
void	MakeActualPageName(char* pagename)
{
	if (actual_page_name) 
	{
		delete[] actual_page_name;
		actual_page_name = NULL;
	}
	strcpy(actual_page_name = new char[strlen(pagename)+1],pagename);
}
/*------------------------------------------------------------------------------------------------------------------*/
/*		 aktuellen Subpagenamen kreieren			  				            */
/*------------------------------------------------------------------------------------------------------------------*/
void	MakeActualSubPageName(char* sub_page_name)
{
	if (actual_subpage_name) 
	{ 
		delete actual_subpage_name; 
		actual_subpage_name = NULL; 
	}
	strcpy(actual_subpage_name = new char[strlen(sub_page_name)+1],sub_page_name);
}
/*------------------------------------------------------------------------------------------------------------------*/
/*		 Seite ueberpruefen und ggf. speichern			  				            */
/*------------------------------------------------------------------------------------------------------------------*/
Page*	CheckAndSavePage(char* pagename)
{
	if (strcmp(pagename,"\\__ROOT__") == 0)
	{
		if (root_found)	YaccError("double definition of root-page", pagename);
		else 		 root_found = true;
	}
	else
	{
		if (!root_found)	YaccError("subpage-definition before root-page-definition", pagename);

		bool master_found = false;
		for (PageListElement* p=PageRefList.first; p; p=p->next)
		{
			if (p->lelement.IsSubPage(pagename)) 
			{ 
				master_found = true; 
				break; 
			}
		}
		if (!master_found)	YaccError("subpage is not referenced before its definition", pagename);
	}

	MakeActualPageName(pagename);
	Page* p = NULL;
	if (!(p = PageRefList.UniqueNameAppend(*(new Page(pagename))))) 
		YaccError("Page exists",pagename);
	return p;
}
/*------------------------------------------------------------------------------------------------------------------*/
/*		 Stelle abspeichern			  				            		    */
/*------------------------------------------------------------------------------------------------------------------*/
Place*	SavePlace(Place*& p, PlaceType pt)
{
	Place* place = NULL;

	p->ChangePlaceType(pt);
	if (!(SocketPortRefList.GetElementPtr(p->GetName())))
	{
		if ( !(place=PlaceRefList.UniqueNameAppend(*p)) )
			YaccError("Place exists : ",p->GetName());

		if ( (place->GetCapacity()->IsPlaceDependent()) )
			PlaceCapDepRefList.Append(place);

		PlaceHeaderPtrRefList.Append(place->GetPlaceHeader());
	}
	p->Clear();
	delete p;

	return place;
}
/*------------------------------------------------------------------------------------------------------------------*/
/*		 Definition abspeichern		  				            		    	    */
/*------------------------------------------------------------------------------------------------------------------*/
void	SaveDefinition(Definition*& d)
{
	if (!(DefinitionRefList.UniqueNameAppend(*d)))
		YaccError("Definition allready exists : ", d->GetVar().GetChar());
}
/*------------------------------------------------------------------------------------------------------------------*/
/*		 SubTransition abspeichern		  				            		    */
/*------------------------------------------------------------------------------------------------------------------*/
void	SaveSubTransition(char* sub_t_name)
{
	Textual	txt_sub_t_name(sub_t_name);

	if ((TransitionNameRefList.UniqueAppend(txt_sub_t_name)) < 0)
		YaccError("SubTransitionName exists", sub_t_name);
}
/*------------------------------------------------------------------------------------------------------------------*/
/*		 Transition abspeichern			  				            		    */
/*------------------------------------------------------------------------------------------------------------------*/
void	SaveTransition(Transition*& t)
{ 
	bool 	t_saved = false;
	char* 	t_name  = t->GetName();
	Textual txt_t_name(t_name);

	/*----------------------------------------------------------------------------------*/
	/*      Transition bekommt globale Referenz zugeordnet                              */
	/*----------------------------------------------------------------------------------*/
	no_of_trans++;
	t->SetGlobalTransitionRef(no_of_trans);

	/*----------------------------------------------------------------------------------*/
	/*      Gab's den Namen schon (Vorsicht auch Subtransitionen beachten)              */
	/*----------------------------------------------------------------------------------*/
	if ((TransitionNameRefList.UniqueAppend(txt_t_name)) < 0)
		YaccError("TransitionName allready exists in this net ", t_name);

	/*----------------------------------------------------------------------------------*/
	/*      Transition in bestimmte Liste einordnen und                                 */
	/*      eine neue Liste fuer die Markierungen dieser Transition reservieren         */
	/*----------------------------------------------------------------------------------*/
	if (t->GetBindings()->first->lelement.GetDelay()->GetDelayType() == _IMMEDIATE)
	{
		ImmediateTransitionRefList.Append(*t); 
		t_saved = true;
	}
	else
	{
		bool one_det_found = false;
		bool one_gen_found = false;

		for (BindingListElement* ble=t->GetBindings()->first; ble; ble=ble->next)
		{
			if (ble->lelement.GetDelay()->GetDelayType() == _DETERMINISTIC)
			{
				if (one_det_found)
					YaccError("more than one det-statetransition in transition",t_name);
				DetMarkingListList.UniqueAppend(*(new SMCLists(no_of_trans)));
				one_det_found = true;
				continue;
			}

			if (ble->lelement.GetDelay()->GetDelayType() == _GENERAL)
			{
				if (one_gen_found)
					YaccError("more than one gen-statetransition in transition",t_name);
				GenMarkingListList.UniqueAppend(*(new SMCLists(no_of_trans)));
				GeneralTransitionRefList.Append(*t);
				one_gen_found = true;
				continue;
			}
		}
	}
	if (!t_saved)
	{
		switch (t->GetServerType())
		{
			case Exclusive : ExclusiveTransitionRefList.Append(*t);  	break;
			case Infinite  : InfiniteTransitionRefList.Append(*t);   	break;
			default        : YaccError("can't save transition",t_name); 	break;
		}
	}
}
/*------------------------------------------------------------------------------------------------------------------*/
/*		 Transition ueberpruefen und Typ zaehlen							    */
/*------------------------------------------------------------------------------------------------------------------*/
void	CheckAndCountTransition(char* t_name, ServerType st, BindingList*& bl)
{
	if (bl && bl->first)
	{
		int dt = bl->first->lelement.GetDelay()->GetDelayType(); 

		if (st == Infinite && ( dt == _DETERMINISTIC ) )
			YaccError("IS-transition with det-delay", t_name);

		if (st == Infinite && ( dt == _GENERAL) )
			YaccError("IS-transition with gen-delay", t_name);

		for (BindingListElement* ble=bl->first; ble; ble=ble->next)
		{
			switch (ble->lelement.GetDelay()->GetDelayType())
			{
				case _GENERAL	   : 
					if (st == Infinite) 	  YaccError("IS-transition with gen-delay", t_name);
					if (dt == _IMMEDIATE) 	  YaccError("delay-type mixture (gen/imm)", t_name); 
					if (dt == _DETERMINISTIC) YaccError("delay-type mixture (gen/det)", t_name); 
					if (dt != _GENERAL) 	  dt = _MIXED;
					break;
				case _DETERMINISTIC : 	
					if (st == Infinite)  	  YaccError("IS-transition with det-delay", t_name);
				      	if (dt == _GENERAL) 	  YaccError("delay-type mixture (det/gen)", t_name); 
				      	if (dt == _IMMEDIATE) 	  YaccError("delay-type mixture (det/imm)", t_name); 
					if (dt != _DETERMINISTIC) dt = _MIXED;
					break;
				case _EXPONENTIAL   :
					if (dt == _IMMEDIATE) 	  YaccError("delay-type mixture (exp/imm)", t_name); 
					if (dt != _EXPONENTIAL)   dt = _EXPONENTIAL;
					break;
				case _IMMEDIATE     : 
					if (dt != _IMMEDIATE) 	  YaccError("delay-type mixture (xxx/imm)", t_name); 
					break;
				default		    : 		  YaccError("unknown delay", t_name);
			}
		}
		switch (dt)
		{
			case _GENERAL       : no_of_gen_transitions++; break;
			case _EXPONENTIAL   : no_of_exp_transitions++; break;
			case _DETERMINISTIC : no_of_det_transitions++; break;
			case _IMMEDIATE     : no_of_imm_transitions++; break;
			case _MIXED	    : no_of_mix_transitions++; break;
			default             : YaccError("unknown delay", t_name);
		}
	}
}
/*------------------------------------------------------------------------------------------------------------------*/
/*		 pfad vom gesuchten String abtrennen und string zurueckgeben					    */
/*------------------------------------------------------------------------------------------------------------------*/
char* 	StripOfPath(char* str)
{
	if (!str)		yyerror  ("can't strip of path");
	if (*str != '\\')	YaccError("can't strip of path; path does not begin with \\", str);

	char* tmp_str = NULL;
	char* ret_str = NULL;

	if (!(tmp_str = rindex(str, '\\'))) YaccError("can't strip of path; rindex()", str);
	tmp_str++;
	strcpy (ret_str = new char[(strlen(tmp_str) + 2)], tmp_str);
	return (ret_str);
}
/*------------------------------------------------------------------------------------------------------------------*/
/*			int in char* umwandeln					    				    */
/*------------------------------------------------------------------------------------------------------------------*/
char* 	GetNumberString(int number)
{
	int   number_size = GetNumberSize(number);
	int   rest 	  = 0;
	char* c1   	  = new char[number_size+2](NULL);

	*c1++ = '\0';
	while (number/10 >= 1.0)
	{
		rest   = number % 10;
		number = number / 10 - rest / 10;
		*c1++  = char(rest+48);
	}
	*c1++ = char(number+48);
	*c1   ='\0';

	char* c2 = new char[sizeof(c1)+1](NULL);
	char* c3 = c2;

	for (c1--; *c1 != '\0';) *c2++ = *c1--;
	*c2 = '\0';
	return c3;
}
/*------------------------------------------------------------------------------------------------------------------*/
/*			groesse eines int ermitteln 					    			    */
/*------------------------------------------------------------------------------------------------------------------*/
int 	GetNumberSize(int number) 
{
        int number_size=0;
	for (number_size = 0; number/10 >= 1.0; number = number/10 - (number%10)/10, number_size++);
	return number_size;
}

/*------------------------------------------------------------------------------------------------------------------*/
/*			fehlerausgabe						    				    */
/*------------------------------------------------------------------------------------------------------------------*/
void 	yyerror(char* s)
{
	YaccError(s, NULL);
}
/*------------------------------------------------------------------------------------------------------------------*/
void 	YaccError(char* s1, char* s2)
{
  switch (section)
  {
	case eplace_sec : RRGErr=new RRGError(PARS,s1,lcount,"ElementaryPlace",actual_commend,s2,yytext); break;
	case pplace_sec : RRGErr=new RRGError(PARS,s1,lcount,"ProductPlace",   actual_commend,s2,yytext); break; 
	case def_sec    : RRGErr=new RRGError(PARS,s1,lcount,"Definition",     actual_commend,s2,yytext); break;
	case trans_sec  : RRGErr=new RRGError(PARS,s1,lcount,"Transition",     actual_commend,s2,yytext); break;
	case def_end    : RRGErr=new RRGError(PARS,s1,lcount,"Parameter",      actual_commend,s2,yytext); break;
	case param_sec  : RRGErr=new RRGError(PARS,s1,lcount,"Parameter",      actual_commend,s2,yytext); break;
	case eval_end   : RRGErr=new RRGError(PARS,s1,lcount,"Parameter",      actual_commend,s2,yytext); break;
	case eval_sec   : RRGErr=new RRGError(PARS,s1,lcount,"Evaluate",       actual_commend,s2,yytext); break;
	case param_end  : RRGErr=new RRGError(PARS,s1,lcount,"Page",           actual_commend,s2,yytext); break;
	case page_sec   : RRGErr=new RRGError(PARS,s1,lcount,"Page",           actual_commend,s2,yytext); break;
	default         : RRGErr=new RRGError(PARS,s1,lcount,"unknown",        actual_commend,s2,yytext); break;
  }
  PrepareEnd();
}
/*------------------------------------------------------------------------------------------------------------------*/
