/******************************************************************************

Programm	:	rand

Komponente	:	rand_types.h

Aufgabe		:	Typdefinitionen und externe Prozedurdeklarationen
			fuer die Bestimmung der transienten Loesung von
			Isomorphieklassen zugeordneter Markov-Ketten

Projektleiter	:	Christoph Lindemann TU Berlin

Programmierer	:	Armin Zimmermann

Datum		:	Juni 1993

******************************************************************************/

#define	MIN_REAL	1.0e-50		/* real values less than this are set to 0 */
#define	is_zero(a)	((a > - MIN_REAL) && (a < MIN_REAL))

#define BUFFERENLARGEMENT 1000
#define MEM_BUFFERS 1000
#define MEM_ITEMS  1000

	/* macros for short scanning of linear lists and arrays */
#define scan_list(walk,list)	for (walk = list; walk != NULL; walk = walk->next)
#define scan_array(count,num)	for (count = 0; count < num; count++)

#define SOCKET_READ_ERROR { \
  perror (NULL); \
  fprintf (stderr, "error in %s at line %u: can't read from socket !\n", \
	   __FILE__, __LINE__); \
  my_exit (1); \
}

/* types for sparse matrices and vectors */

	/* entry of a sparse vector */
typedef struct sp_entry
{	int		num;		/* the column number */
	double		val;		/* the entry value */
	struct sp_entry	*next;

}  sp_entry;

	/* sparse vector or a column of a sparse matrix */
typedef struct sp_vector
{	int		num;		/* the column number */
	int		no_entries;	/* number of entries */
	sp_entry	*entries;	/* linear list of entries */
	struct sp_vector *next;
}  sp_vector;

	/* sparse matrix */
typedef struct sp_matrix
{	sp_vector	*rows;		/* linear list of rowss */
}  sp_matrix;


/* types for components of subordinated markov chains */
	/* type for a state */
typedef struct state
{	int		initial;	/* the state is initial */
	double		tau;		/* the real deterministic firing
					   time in this state */
}  state;

	/* type for an isomorphic CSMC */
typedef struct iso_csmc
{	int		csmc_no;	/* the identification number of the CSMC */
	state		*states;	/* csmc states */
	int		*orig_no;	/* original number of preempted states */
}  iso_csmc;

	/* type for a class of isomorphic CSMCs with equal tau */
typedef struct tau_set
{	double		tau;		/* deterministic firing time (relativ) */
	int		R, L;		/* left and right truncation points */
	double		*beta;		/* poisson probabilities */
	double		rem_beta;	/* 1 - (sum of all betas until now) */
	char		*calc;		/* bit; == 1 when the state to calculate
					   is initial in a CSMC of this Tau_Set */
	char		*lev_calc;	/* == 1 when a state of this level is
					   initial in this csmc; branch only */
	sp_vector	*P_row;		/* row of probability matrix */
	sp_vector	*C_row;		/* row of conversion matrix */

	double		*P_row_copy;	/* row of probability matrix for branch */
	double		*C_row_copy;	/* row of conversion matrix for branch */

	int		no_iso_csmcs;	/* number of CSMCs in this Tau_Set */
	iso_csmc	*iso_csmcs;	/* array of CSMCs in this Tau_Set */	
}  tau_set;

/* external definitions */
extern int	class_id;
extern int	no_csmcs;
extern int	no_tau_sets;
extern tau_set	*tau_sets;
extern int	no_total_states;
extern int	no_csmc_states;
extern int	no_preempt_edges;
extern int	first_state_no;

extern int	*last_of_SCC;
extern char	*leaf_state;
extern char	*calc;
extern double	*q_ii;
extern sp_matrix *Q_matrix;
extern sp_matrix *A_matrix;
extern sp_vector **A_matrix_rows;
extern sp_matrix *Delta_matrix;

extern int	branch_n, branch_k;
extern int	**state_succs;
extern char	*visited;
extern int	*branch_level;
extern char	*level_obtained;
extern int	level_start_state_no;


extern void	my_exit();		/* rand_main.c */

extern int	global_state_num();	/* rand_procs.c */
extern void	map_branch();
extern int	no_states_in_level();

extern sp_entry *new_entry();
extern void	del_entry();
extern sp_entry	*add_to_vector();
extern sp_vector *new_vector();
extern void	clear_vector();
extern void	del_vector();
extern sp_vector *copy_vector();
extern sp_matrix *new_matrix();
extern void	clear_matrix();
extern void	del_matrix();
extern void	del_rows_from_matrix();
extern sp_matrix *copy_matrix();
extern void	vector_add_mult();
extern void	mult_vector_matrix();
extern void	add_entry_to_matrix();
extern void	show_entry();
extern void	show_vector();
extern void	show_matrix();
