#ifndef __RG_CC__
#define __RG_CC__

#include "inc_env.h"
#include "inc_classes.h"
#include "inc_examine_rrg.h"
#include "inc_rrg_globals.h"

//##################################################################################################################
//-----------------------------------------------------------------------------------------------------------
//	externe funktionen	
//-----------------------------------------------------------------------------------------------------------
extern "C" { DIR*       opendir(const char*);                   }
extern "C" { int        closedir(DIR*);                         }
extern "C" { char*      getenv(const char*);                    }
extern "C" { double   	strtod(const char*, char**); 		}
extern "C" { long int 	strtol(const char*, char**, int); 	}
extern "C" { int	getopt(int, char* const*, const char*);	}

#if defined(__linux__) || defined(LINUX) || defined (Linux)
extern "C" { char* 	rindex(const char*, int);		}
#endif

void  ShowUsage();
void  PrepareEnd();
void  CleanTmpDir();
void  InitGlobalVars();
void  GetTableString(char*);

void  GetProcSize();
void  CheckEnviron();
void  CheckOptions(int, char**);
void  CheckParameters(char*, double&, char*, int&, char*, ExecMode&);

//-----------------------------------------------------------------------------------------------------------


main(int argc, char** argv)
{
	cout <<endl<<"STARTING NUMERIC ANALYSIS"<<endl;

	//---------------------------------------------------------------------------------------------------
	//	command-line-flags auswerten
	//---------------------------------------------------------------------------------------------------

	CheckOptions(argc, argv);

	//---------------------------------------------------------------------------------------------------
	//	environmet-var's checken	
	//---------------------------------------------------------------------------------------------------
	if (DEBUG) cout <<endl<<"checking environment ... "<<endl;

	CheckEnviron();

	if (DEBUG) cout <<"checking environment finished !"<<endl;
	//---------------------------------------------------------------------------------------------------
	//	Uebergangstabelle holen	
	//---------------------------------------------------------------------------------------------------
	if (DEBUG) cout <<endl<<"getting tablestring ... "<<endl;

	GetTableString(netname);
	if (*netname == '/')
        {
                char* tmp_name = rindex(netname, '/');
                netname = NULL;
                strcpy(netname = new char[(strlen(tmp_name) + 1)], tmp_name);
        }

	if (DEBUG) cout <<"getting tablestring finished !"<<endl;
	//---------------------------------------------------------------------------------------------------
	//	alte rrg-files fuer dieses Netz loeschen	
	//---------------------------------------------------------------------------------------------------
	if (DEBUG) cout <<endl<<"deleting old tmp-files ..."<<endl;

	CleanTmpDir();

	if (DEBUG) cout <<"deleting old tmp-files finished !"<<endl;
	//---------------------------------------------------------------------------------------------------
	//	Gesamt-Zeit nehmen
	//---------------------------------------------------------------------------------------------------

	StartTime(all_start_time); 

	//---------------------------------------------------------------------------------------------------
	//	String scannen, parsen und Klassen fuellen
	//---------------------------------------------------------------------------------------------------
	cout <<endl<<"parsing transition table ..."<<endl;

	yyparse();

	cout <<"parsing transition table finished !"<<endl;
	//---------------------------------------------------------------------------------------------------
	//	.pmf-File schreiben, falls gen-Transitionen existieren
	//---------------------------------------------------------------------------------------------------
	if (no_of_gen_transitions)
	{
		if (DEBUG) cout <<endl<<"writing .pmf-file ..."<<endl;

		WritePMFFile(netname);

		if (DEBUG) cout <<"writing .pmf-file finished !"<<endl;

		GeneralTransitionRefList.Clear();
	}

	//---------------------------------------------------------------------------------------------------
	// 	Textuelle Cell-Stellen-Elemente durch Elementen der PlaceHeaderRefList ersetzten 
	//---------------------------------------------------------------------------------------------------
	if (DEBUG) cout <<endl<<"checking references ..."<<endl;

	TxtToRef();

	if (DEBUG) cout <<"checking references finished !"<<endl;
	//---------------------------------------------------------------------------------------------------
	// 	Textuelle Cell-Stellen-Elemente durch Elementen der PlaceHeaderRefList ersetzten 
	//---------------------------------------------------------------------------------------------------
	if (DEBUG) cout <<endl<<"structural analysis ..."<<endl;

	StructuralAnalysis();

	if (DEBUG) cout <<"structural analysis finished !"<<endl;
	//---------------------------------------------------------------------------------------------------
	//	1) ein bisschen Platz schaffen
	//
	//	2) Initialisierung von :
	//	    - no_of_places	    = globale Var. fuer die PlaceMultisetalloceierung in jeder Mark.
	//  	    - no_of_exc_transitions = globale Var. fuer die Arrayalloceierung in jeder Markierung
	//	    - m_listst, m_count     = Uebergabeparameter der Funktion ExamineRRG
	//
	//	3) globale Hashs kreieren und initialisieren
	//---------------------------------------------------------------------------------------------------
	PageRefList.Clear();
	SocketPortRefList.Clear();
	DefinitionRefList.Clear();

	InitGlobalVars();

	CreateAndInitHash();

	//---------------------------------------------------------------------------------------------------
	//	Erreichbarkeitsgraph erstellen	
	//---------------------------------------------------------------------------------------------------
	cout <<endl<<"creating rrg ... "<<endl;

	if (DEBUG)
	{
		cout <<"\tproc-size (KByte)    = "<<flush;
		GetProcSize();
	}

	StartTime(rrg_start_time);

	RRGAlgo  (InitialMarking);

	StopTime (rrg_stop_time);

	if (DEBUG)
	{
		cout <<"\trrg-size  (markings) = "<< marking_counter<<endl;
		cout <<"\trrg-size  (KByte)    = ";
		cout <<((marking_counter * sizeof(MarkingListElement))+sizeof(MarkingList*[999]))/1024<<endl;
		cout <<"\tproc-size (KByte)    = "<<flush;
		GetProcSize();
	}
	cout <<"creating rrg finished !"<<endl;
	//---------------------------------------------------------------------------------------------------
	//	ein bisschen Platz schaffen	
	//---------------------------------------------------------------------------------------------------

	ExclusiveTransitionRefList.Clear();
	ImmediateTransitionRefList.Clear();
	InfiniteTransitionRefList.Clear();
	PlaceCapDepRefList.Clear();
	PlaceRefList.Clear();
	DeleteImmHash();

	//---------------------------------------------------------------------------------------------------
	//	Erreichbarkeitsgraph auswerten	
	//---------------------------------------------------------------------------------------------------
	if (DEBUG) cout <<endl<<"examining rrg ... "<<endl;

	ExamineRRG(InitialMarking, m_count, m_lists);

	if (DEBUG) cout <<"examining rrg finished!"<<endl;
	//---------------------------------------------------------------------------------------------------
	//	Schnittstelle zu TimeNet (Analyse der Markow-Ketten, Errechnung des pi-Vektors)
	//	(Schreiben der files .CONVxx, .EMCxx und .PROBMARK)
	//---------------------------------------------------------------------------------------------------
	cout <<endl<<"entering TimeNET ... "<<endl;

	AgnesToTimeNet(netname, auto_iter, m_count->no_tang_markings, maxiter, epsilon, execmode,
		       m_lists->exp_marking_list, 
		       m_lists->det_marking_list,
		       m_lists->gen_marking_list);

	cout <<"entering TimeNET finished !"<<endl;
	//---------------------------------------------------------------------------------------------------
	//	Ergebnisvektor ausgeben
	//---------------------------------------------------------------------------------------------------
	if (SHOW_PI) ShowPIVector(m_count->no_tang_markings);

	//---------------------------------------------------------------------------------------------------
	//	Berechnung der Bewertungsgroessen des GSPN/DSPN
	//---------------------------------------------------------------------------------------------------
	cout <<endl<<"calculating net parameters ... "<<endl;

	NetEvaluation(netname);

	cout <<"calculating net parameters finished ! "<<endl;
	//---------------------------------------------------------------------------------------------------
	//	ein bisschen Platz schaffen	
	//---------------------------------------------------------------------------------------------------

	DeleteHash();
	EvalRefList.Clear();
	TokenRefList.Clear();
	TransitionNameRefList.Clear();
	PlaceHeaderPtrRefList.Clear();

	//---------------------------------------------------------------------------------------------------
	//	Zeit-Informationen	
	//---------------------------------------------------------------------------------------------------
	if (SHOW_TIME)
	{
		StopTime(all_stop_time); 

		DiffTime(all_start_time,  all_stop_time,  all_diff_time );
		DiffTime(rrg_start_time,  rrg_stop_time,  rrg_diff_time );

		ShowTime(all_diff_time,  "whole programm"   );
		ShowTime(rrg_diff_time,	 "rrg-creation"     );
	}


	//---------------------------------------------------------------------------------------------------
	//	Ergebnisse ausgeben	
	//---------------------------------------------------------------------------------------------------
	if (SHOW_RES) ShowResults(netname);

	cout <<endl<<endl<<"FINISHED !!"<<endl<<endl;

	exit (0);

}	/* End of main */
//---------------------------------------------------------------------------------------------------------------
//	Aufruf ausgeben
//---------------------------------------------------------------------------------------------------------------
void	ShowUsage()
{
	cout <<"\n";
	cout <<"usage : rg [-?hvrsticfLHPRpd] [-T{tablestring}] -E{epsilon} -N{netname} -M{maxiter}\n\n";
	cout <<"\n";
	cout <<"        {epsilon}     = double  (value for evaluation-precision)\n";
	cout <<"        {maxiter}     = int     (maximum of iterations for solving the LGS)\n";
	cout <<"        {netname}     = string  (name of the net)               \n";
	cout <<"        {tablestring} = string  (table string of the net)       \n";
	cout <<"\n";
	cout <<"MODE-SWITCHES\n";
	cout <<"\n";
	cout <<"	-p = switches execution mode to parallel (default: sequential)\n";
	cout <<"	-i = switches solving mode to iterativ (default: automated)\n";
	cout <<"	-d = switches working mode to default\n";
	cout <<"\n";
	cout <<"DEBUG-OPTIONS\n";
	cout <<"\n";
	cout <<"	-? = shows this usage-information\n";
	cout <<"	-h = shows this usage-information\n";
	cout <<"	-v = shows debug-information (verbose-mode)\n";
	cout <<"	-r = shows reduced reachability graph\n";
	cout <<"	-s = shows subordinated markow chains\n";
	cout <<"	-t = shows time-information\n";
	cout <<"	-T = shows information about host- and rand-process-parameters\n";
	cout <<"	-c = shows communication between main- and rand-process\n";
	cout <<"	-e = shows communication between main- and erand-process\n";
	cout <<"	-f = shows information about host- and rand-process-parameters (.TimeNET)\n";
	cout <<"	-L = shows LGS-evaluation-information\n";
	cout <<"	-H = shows marking-hash-distribution\n";
	cout <<"	-P = shows pi-vector\n";
	cout <<"	-R = shows results\n";
	cout <<"\n";
	exit(-1);
}
//---------------------------------------------------------------------------------------------------------------
//	Ende vorbereiten (im Fehlerfall)
//---------------------------------------------------------------------------------------------------------------
void PrepareEnd()
{
	RRGErr->PrintError();
	exit (-1);
}
//---------------------------------------------------------------------------------------------------------------
//	Programm-grosse ermitteln
//---------------------------------------------------------------------------------------------------------------
void	GetProcSize()
{
	char* command_str = NULL;
	sprintf(command_str = new char[(strlen(ScrDir) + 18)], "%s/get_proc_size.sh", ScrDir);
	system(command_str);
	delete[] command_str;
}
//---------------------------------------------------------------------------------------------------------------
//	environment checken	
//---------------------------------------------------------------------------------------------------------------
void	CheckEnviron()
{
	FILE*	fptr 	= NULL;
	DIR*	dptr 	= NULL;

	TimeNETDir 	= NULL;
	TimeNETRC  	= NULL;
	ModelDir   	= NULL;
	HomeDir    	= NULL;
	TmpDir     	= NULL;
	BinDir     	= NULL;
	ScrDir		= NULL;

	if (!(HomeDir = getenv("HOME")))
	{
		cout <<endl<<endl<<"can't get environment variable HOME"<<endl<<endl;
		exit (-1);
	}
	if (!(TimeNETDir = getenv("TNETHOME")))
	{
		cout <<endl<<endl<<"can't get environment variable TNETHOME"<<endl<<endl;
		exit (-1);
	}
	if (!(ModelDir = getenv("MODELDIR")))
	{
		cout <<endl<<endl<<"can't get environment variable MODELDIR"<<endl<<endl;
		exit (-1);
	}
	sprintf(TimeNETRC = new char[(strlen(HomeDir) + 10)], "%s/.TimeNET",           HomeDir);
	sprintf(ScrDir = new char[(strlen(TimeNETDir) + 21)], "%s/MSAnalysis/scripts", TimeNETDir);
	sprintf(TmpDir = new char[(strlen(TimeNETDir) + 17)], "%s/MSAnalysis/tmp",     TimeNETDir);
	sprintf(BinDir = new char[(strlen(TimeNETDir) + 17)], "%s/MSAnalysis/bin",     TimeNETDir);

	if (!(fptr = fopen(TimeNETRC,"r")))
	{
		cout <<endl<<endl<<"can't open file "<<TimeNETRC<<endl<<endl;
		exit (-1);
	}
	fclose(fptr);
	if (!(dptr = opendir(TimeNETDir)))
	{
		cout <<endl<<endl<<"can't open dir "<<TimeNETDir<<endl<<endl;
		exit (-1);
	}
	closedir(dptr);
	dptr = NULL;
	if (!(dptr = opendir(TmpDir)))
	{
		cout <<endl<<endl<<"can't open dir "<<TmpDir<<endl<<endl;
		exit (-1);
	}
	closedir(dptr);
	dptr = NULL;
	if (!(dptr = opendir(ScrDir)))
	{
		cout <<endl<<endl<<"can't open dir "<<ScrDir<<endl<<endl;
		exit (-1);
	}
	closedir(dptr);
	dptr = NULL;
	if (!(dptr = opendir(BinDir)))
	{
		cout <<endl<<endl<<"can't open dir "<<BinDir<<endl<<endl;
		exit (-1);
	}
	closedir(dptr);

	if (DEBUG)
	{
		cout <<"\t HOME          = "<<HomeDir	<<endl;
		cout <<"\t TNETHOME      = "<<TimeNETDir<<endl;
		cout <<"\t MODELDIR      = "<<ModelDir	<<endl;
		cout <<"\t bin-directory = "<<BinDir	<<endl;
		cout <<"\t tmp-directory = "<<TmpDir	<<endl;
		cout <<"\t timenet-file  = "<<TimeNETRC	<<endl;
	}
}
//---------------------------------------------------------------------------------------------------------------
//	Optionen checken	
//---------------------------------------------------------------------------------------------------------------
void	CheckOptions(int argc, char** argv)
{
	while (1)
	{
		int c = getopt(argc, argv, "E:M:N:T:fHPRLvdrsticphe");

		if (c == -1) break;

		switch (c)
		{
			case 'E' : strcpy(p_epsilon    = new char[strlen(optarg)+1],optarg); break;
			case 'M' : strcpy(p_maxiter    = new char[strlen(optarg)+1],optarg); break;
			case 'N' : strcpy(netname      = new char[strlen(optarg)+1],optarg); break;
			case 'T' : strcpy(table_string = new char[strlen(optarg)+1],optarg); break;
			case 'R' : SHOW_RES 	= true;  	break;
			case 'H' : SHOW_HASH 	= true;  	break;
			case 'P' : SHOW_PI 	= true;  	break;
			case 'L' : SHOW_LGS    	= 1;  		break;
			case 'v' : DEBUG    	= true;  	break;
			case 'r' : SHOW_RRG 	= true;  	break;
			case 's' : SHOW_SMC 	= true;  	break;
			case 't' : SHOW_TIME 	= true;  	break;
			case 'f' : SHOW_INFOS 	= true;  	break;
			case 'c' : SHOW_COMM  	= true;  	break;
			case 'e' : SHOW_ERAND  	= true;  	break;
			case 'p' : execmode     = parallel; 	break;
			case 'i' : auto_iter    = "-i"; 	break;
			case 'd' : default_mode = true;         break;
			case 'h' : ShowUsage();		 	break;
			case '?' : ShowUsage();		 	break;
			default  : ShowUsage();		 	break;
		}
	}
	if (default_mode && DEBUG)
	{
		cout <<endl<<"checking options ... "<<endl;
		cout <<"\t net name   : "<<	netname	<<endl;
		cout <<"\t epsilon    : "<<	epsilon	<<endl;
		cout <<"\t maxiter    : "<<	maxiter	<<endl;
		cout <<"\t execmode   : sequential"	<<endl;
		cout <<"\t solvemode  : iterativ"	<<endl;
		cout <<"checking options finished ! "<<endl;
	}
	else 	CheckParameters(p_epsilon, epsilon, p_maxiter, maxiter, netname, execmode);
}
//---------------------------------------------------------------------------------------------------------------
//	.pmf- .EMCxx- .CONVxx- .RESULT- .AUX- .PROBMARK- Files loeschen	
//---------------------------------------------------------------------------------------------------------------
void	CleanTmpDir()
{
	int abs_length_tmp = (strlen(TmpDir)   + strlen(netname) + 20);
	int abs_length_mod = (strlen(ModelDir) + strlen(netname) + 20);

	char* rm_pmf_str = NULL;
	char* rm_emc_str = NULL;
	char* rm_con_str = NULL;
	char* rm_res_str = NULL;
	char* rm_aux_str = NULL;
	char* rm_pro_str = NULL;

	sprintf(rm_pmf_str = new char[(abs_length_tmp + 3)], "rm %s/%s.pmf 2>/dev/null",      TmpDir,   netname);
	sprintf(rm_aux_str = new char[(abs_length_tmp + 3)], "rm %s/%s.AUX 2>/dev/null",      TmpDir,   netname);
	sprintf(rm_emc_str = new char[(abs_length_tmp + 4)], "rm %s/%s.EMC* 2>/dev/null",     TmpDir,   netname);
	sprintf(rm_con_str = new char[(abs_length_tmp + 5)], "rm %s/%s.CONV* 2>/dev/null",    TmpDir,   netname);
	sprintf(rm_pro_str = new char[(abs_length_tmp + 8)], "rm %s/%s.PROBMARK 2>/dev/null", TmpDir,   netname);
	sprintf(rm_res_str = new char[(abs_length_mod + 7)], "rm %s/%s.RESULTS 2>/dev/null",  ModelDir, netname);

	system(rm_pmf_str);
	system(rm_emc_str);
	system(rm_aux_str);
	system(rm_con_str);
	system(rm_res_str);
	system(rm_pro_str);

	delete[] rm_pmf_str;
	delete[] rm_emc_str;
	delete[] rm_aux_str;
	delete[] rm_con_str;
	delete[] rm_pro_str;
	delete[] rm_res_str;
}
//---------------------------------------------------------------------------------------------------------------
//	Ueberprueft die Parameter epsilon, netname und execmode
//---------------------------------------------------------------------------------------------------------------
void	CheckParameters(char* 	  p_epsilon, 
			double&   epsilon, 
			char* 	  p_maxiter, 
			int& 	  maxiter, 
			char* 	  netname, 
			ExecMode& execmode)
{
	if (!netname)  
	{ 
		cout <<endl<<endl;
		cout <<"\t no netname given => abort !";  
		cout <<endl<<endl;
		ShowUsage(); 
	}

	if (!p_maxiter)  
	{ 
		cout <<endl<<endl;
		cout <<endl<<endl<<"\t no maxiter given => abort !";  
		cout <<endl<<endl;
		ShowUsage(); 
	}
	else	
	{
		maxiter = strtol(p_maxiter,NULL, 10);
		if (errno == ERANGE)
		{
			cout <<endl<<endl;
			cout <<"\t error while converting string to long int (strtod()) : "<<p_maxiter; 
			cout <<endl<<endl;
			ShowUsage();
		}
	}

	if (!p_epsilon)  
	{ 
		cout <<endl<<endl;
		cout <<"\t no epsilon given => abort !";  
		cout <<endl<<endl;
		ShowUsage(); 
	}
	else	
	{
		if ((epsilon = strtod(p_epsilon,NULL)) == 0)
		{
			cout <<endl<<endl;
			cout <<"\t error while converting string to double (strtod()) : "<<p_epsilon; 
			cout <<endl<<endl;
			ShowUsage();
		}
	}
	
	if (DEBUG)
	{
		cout <<endl<<"checking options ... "<<endl;
		cout <<"\t netname  = "<<netname<<endl;
		cout <<"\t epsilon  = "<<epsilon<<endl;
		cout <<"\t maxiter  = "<<maxiter<<endl;
		switch (execmode)
		{
			case parallel   : cout <<"\t execmode = parallel"	<<endl; break;
			case sequential : cout <<"\t execmode = sequential"	<<endl; break;
		}
		cout <<"checking options finished !"<<endl;
	}
}
//---------------------------------------------------------------------------------------------------------------
//	Initialisieren von globalen Variablen nach der RRG-Erstellung
//---------------------------------------------------------------------------------------------------------------
void	InitGlobalVars()
{
	no_of_places		    = PlaceHeaderPtrRefList.size + 1;
	no_of_exc_transitions       = ExclusiveTransitionRefList.size + 1;

	m_count->no_markings        = 0;
	m_count->no_tang_markings   = 0;
	m_count->no_exp_markings    = 0;
	m_count->no_det_markings    = 0;
	m_count->no_pro_markings    = 0;
	m_count->no_imm_markings    = 0;
	m_count->no_absorb_markings = 0;
	m_lists->det_marking_list   = DetMarkingListList;
	m_lists->gen_marking_list   = GenMarkingListList;
}
//---------------------------------------------------------------------------------------------------------------
//		PageStrings holen	
//---------------------------------------------------------------------------------------------------------------
void GetTableString(char* file_prefix)
{
	//--------------------------------------------------------
	//	table_string existiert ???	
	//--------------------------------------------------------
	if (table_string) return;

	if (DEBUG)
	{
		cout <<"\t table string is not defined !"<<endl;
		cout <<"\t try to get table string from "<<ModelDir<<"/"<<netname<<".table !"<<endl;
		cout <<endl;
	}

	char* filename   = NULL;
	FILE* fptr       = NULL;
	int   char_count = 1;

	//--------------------------------------------------------
	//	<name>.table ???	
	//--------------------------------------------------------
        if (*file_prefix == '/')
        {
                filename = new char[(strlen(file_prefix) + 9)];
                sprintf(filename, "%s.table", file_prefix);
        }
        else
        {
                filename = new char[(strlen(ModelDir) + strlen(file_prefix) + 9)];
                sprintf(filename, "%s/%s.table", ModelDir, file_prefix);
	} 

	if (!(fptr = fopen(filename,"r")))
	{
		if (DEBUG) cout <<"\t can't open file : "<<filename<<endl;
		delete[] filename;
		//--------------------------------------------------------
		//	<name>.TABLE ???	
		//--------------------------------------------------------
		filename = new char[(strlen(ModelDir) + strlen(file_prefix) + 9)];
		sprintf(filename, "%s/%s.TABLE", ModelDir, file_prefix);
		if (DEBUG) cout <<"\t try to get table string from "<<filename<<" ..."<<endl;
		fptr = fopen(filename,"r");
	}
	if (!fptr)
	{
		cout <<"\t can't get transition-table"<<endl;
		exit(-1);
	}
	//--------------------------------------------------------
	//	Zeichen zaehlen
	//--------------------------------------------------------
	while ((fgetc(fptr) ) != EOF ) char_count++;
	fclose(fptr);

	//--------------------------------------------------------
	//	Zeichen einlesen
	//--------------------------------------------------------
	char   _in_char_;
	char*  _in_string_  = new char[char_count + 1];
	char* __in_string__ = _in_string_;

	if (!(fptr = fopen(filename,"r")))
	{
		cout <<"\t can't open file :"<<filename<<endl;
		exit(-1);
	}
	while (( _in_char_ = fgetc(fptr) ) != EOF ) 
		*_in_string_++=_in_char_;
	fclose(fptr);
	*_in_string_++=EOF;
        *_in_string_++='\0';

	//--------------------------------------------------------
	//	Zeichen in global string kopieren
	//--------------------------------------------------------
	strcpy(table_string = new char[char_count],__in_string__);
}
//##################################################################################################################
#endif
