%{
#ifdef YY_INPUT
#undef YY_INPUT
#endif
#if defined(__linux__) || defined(LINUX) || defined (Linux) || defined(SUNOS4) || defined(SOL2)
#define YY_INPUT(buf,result,max_size) \
	if ( yy_current_buffer->yy_is_interactive ) \
	{ \
		int c = '*', n; \
		for ( n = 0; (n < max_size) && ((c = *table_string++) != EOF) && (c != '\n'); ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		result = n; \
	} 
#else
#define YY_INPUT(buf,result,max_size) \
	int c = '*', n; \
	for ( n = 0; (n < max_size) && ((c = *table_string++) != EOF) && (c != '\n'); ++n ) \
		buf[n] = (char) c; \
	if ( c == '\n' ) \
		buf[n++] = (char) c; \
	result = n; 
#endif

/*--------------------------------------------------------------*/
/*		   	error-func		      		*/
/*--------------------------------------------------------------*/
void 	LexError		(char*);
void	CatchCommend		(char*, int);
/*--------------------------------------------------------------*/
/*		   	return-func's		      		*/
/*--------------------------------------------------------------*/
int     ReturnString		(char*);
int     ReturnCellContents	(Cell*);
/*--------------------------------------------------------------*/
/*		   	copy-func's		      		*/
/*--------------------------------------------------------------*/
void    CopyCellPtr		(Cell*);
void    CopyString		(char*);
/*--------------------------------------------------------------*/
/*		   	path-adding		      		*/
/*--------------------------------------------------------------*/
char*	AddRoot			(char*);
char*	AddMaster		(char*);
char*	AddActPage		(char*);
char*	AddDefPath	 	(char*);
/*--------------------------------------------------------------*/
/*		   	check's			      		*/
/*--------------------------------------------------------------*/
char*	CheckActDef		(char*);
char*	CheckActPath		(char*);
/*--------------------------------------------------------------*/
/*		   Definition-stuff			      	*/
/*--------------------------------------------------------------*/
int	VarOrDefOrString	(char*);
int	DefOrParamOrString	(char*);
int	DefinitionOrString	(char*);

bool	DefinitionLoop		(char*,        Textual);
Cell*   TestIfParameter		(char*, char*, TextualList*);
Cell*   TestIfDefinition	(char*,        TextualList*);
/*--------------------------------------------------------------*/
/*		   	misc-func		      		*/
/*--------------------------------------------------------------*/
char*	GetPath			(char*);
void	MakeMasterPageName	(char*);
void	CleanActDef		();
/*--------------------------------------------------------------*/

%}
D	[0-9]
E	[Ee][+-]?[0-9]+
%%
[ \t]+ 			;	
"\n"			{ lcount++; }
{D}+			{ 	
			/*-------------------------------------------------------------------------------*/
			/*                              INT_CONST                                        */
			/*-------------------------------------------------------------------------------*/
				yylval.i_val = strtol(yytext, NULL, 10); 
				if (errno == ERANGE)
					LexError("Error while converting string to long int (strtol())");
				return (INT_CONST);
			}
"0.0"			{ 
			/*-------------------------------------------------------------------------------*/
			/*                              REAL_CONST                                       */
			/*-------------------------------------------------------------------------------*/
				yylval.d_val=0.0;
				return (REAL_CONST);
			}
{D}+"."{D}*({E})?	|
{D}+{E}			{ 
			/*-------------------------------------------------------------------------------*/
			/*                              REAL_CONST                                       */
			/*-------------------------------------------------------------------------------*/
				if ( ((yylval.d_val=strtod(yytext, NULL)) == 0) || errno == ERANGE) 
					LexError("Error while converting string to double (strtod())");
				return (REAL_CONST);
			}
"/*"[#\"!a-zA-Z0-9_\\\t\n ]*"*/"		{
			/*-------------------------------------------------------------------------------*/
			/*                              COMMENT                                          */
			/*-------------------------------------------------------------------------------*/
				CatchCommend(yytext,yyleng);
			}
"e"			return (E);
"ES"			return (ES);
"IS"			return (IS);
"IF"			return (IF);
"TRUE"			return (WAHR);
"FALSE"			return (FALSCH);
"assign"		return (ASSIGN);
"subpage"		return (SUBPAGE);
"Uniform"		return (UNIFORM);
"Immediate"		return (IMMEDIATE);
"Exponential"		return (EXPONENTIAL);
"Deterministic"		return (DETERMINISTIC);
"E[" 			return (EXPECT_OPEN);
"P[" 			return (PROB_OPEN);
"TP[" 			return (TP_OPEN);

"Parameter"		{ 
				switch (section)
				{
					case	def_sec	 : section = def_end;   break;
					case	eval_sec : section = eval_end;  break;
					default		 : LexError("wrong section-order (Parameter)"); 
							   break;
				}
				CleanActDef(); 	
				return (PARAMETER);	  
			} 
"Definition"		{ 
				if (section != no_sec)
					LexError("wrong section-order (Definition)");
				section = def_sec; 			
				return (DEFINITION);   	  
			}
"Page"			{ 
				if (section != param_sec)
					LexError("wrong section-order (Page)");
				section = param_end;  
				CleanActDef(); 	
				return (PAGE); 	   	  
			}
"ProductPlace"		{ 
				if (section != page_sec && section != eplace_sec)
					LexError("wrong section-order (ProductPlace)");
				section = pplace_sec; 		
				return (PRODUCTPLACE); 	  
			}
"ElementaryPlace"	{ 
				if (section != page_sec && section != pplace_sec)
					LexError("wrong section-order (ElementaryPlace)");
				section = eplace_sec; 		
				return (ELEMENTARYPLACE); 
			}
"Transition"		{ 
				if (section != eplace_sec && section != pplace_sec)
					LexError("wrong section-order (Transition)");
				section = trans_sec; 			
				return (TRANSITION);	  
			}
"Evaluate"		{ 
				if (section != trans_sec)
					LexError("wrong section-order (Evaluate)");
				section = eval_sec; 			
				return (EVALUATE);	  
			}

"__ROOT__"		{ LexError("found reserved string (__ROOT__)");	  }

"\\"			{ 
			/*-------------------------------------------------------------------------------*/
			/*                              STRING (root-page)                   		 */
			/*-------------------------------------------------------------------------------*/
				if (section != page_sec) LexError("root (\\) found in wrong section");
				return (ReturnString("\\__ROOT__"));
			}

"\\"[a-zA-Z_][a-zA-Z_0-9]*['\\'a-zA-Z_0-9]* {
			/*-------------------------------------------------------------------------------*/
			/*			STRING (absolute "Pfadangabe")				 */
			/*-------------------------------------------------------------------------------*/
				switch (section)
				{
					case def_sec  : return (VarOrDefOrString           (AddRoot (yytext)));
					case page_sec :	return (ReturnString               (AddRoot (yytext)));
					case eval_sec : return (ReturnString 		   (AddRoot (yytext)));
					case param_sec: return (DefinitionOrString         (AddRoot (yytext))); 
					default	      : return (DefOrParamOrString         (AddRoot (yytext)));
				}
			   }
[a-zA-Z_][a-zA-Z0-9_]*     {
			/*-------------------------------------------------------------------------------*/
			/*                      STRING (keine "Pfadangabe")				 */
			/*-------------------------------------------------------------------------------*/
				switch (section)
				{
					case page_sec	: LexError("page-name without path");
					case param_sec	: return (ReturnString       (AddMaster (yytext)));	
					case def_sec	: return (ReturnString       (AddDefPath(yytext)));	
					default		: return (DefOrParamOrString (AddActPage(yytext)));
				}
			}
[a-zA-Z_][a-zA-Z_0-9]*['\\'a-zA-Z_0-9]* {
			/*-------------------------------------------------------------------------------*/
			/*                      STRING (relative "Pfadangabe")				 */
			/*-------------------------------------------------------------------------------*/
				switch (section)
				{
					case page_sec	: LexError("page-name with relativ path");
					case eval_sec	: LexError("eval-string with relativ path");
					case param_sec	: return (ReturnString       (AddMaster (yytext)));
					case def_sec	: return (ReturnString       (AddDefPath(yytext)));
					default		: return (DefOrParamOrString (AddActPage(yytext)));
				}
			}
"General("[ex^RI\[\]()*\-+,\.0-9 ]*")" {
			/*-------------------------------------------------------------------------------*/
			/*                              GENERAL FIRING DELAY                             */
			/*-------------------------------------------------------------------------------*/
				int   open     = 1;
				char* ret_str  = new char[yyleng-8];
				char* tmp1_str = ret_str; 
				char* tmp2_str = index(yytext,'(');
				tmp2_str++;

				while (*tmp2_str != '\0')
				{
					if (*tmp2_str == '(') open++;
					if (*tmp2_str == ')') open--;
					if (open <= 0)        break;

					*tmp1_str = *tmp2_str;
					tmp1_str++;
					tmp2_str++;
				}
				*tmp1_str = '\0';

				strcpy(yylval.s_val = new char[(strlen(ret_str) + 1)],ret_str);
				return (GENERAL);
			}

"in#"			return (INARC);
"out#"			return (OUTARC);
"->"			return (ARROW);

"`"			LexError("undefined character : `");
"^"			LexError("undefined character : ^");
"%"			LexError("undefined character : %");
"~"			LexError("undefined character : ~"); 

"'"			return (TIMES); 
"+"			return (PLUS); 
"-"			return (MINUS); 
"/"			return (DIVIDE); 
"*"			return (MULTIPLY); 

"!"			return (NOT);
"&"			return (AND);
"|"			{ CleanActDef(); return (OR); }
"#"			return (RAUTE);
"@"			return (AT); 

"="			return (EQUAL);
"<"			return (LESS);
">"			return (GREATER);
"!="			return (NOT_EQUAL);
"<="			return (LESS_OR_EQUAL);
">="			return (GREATER_OR_EQUAL);

","			return (COMMA);
"."			return (POINT);
":"			{ 
				switch (section)
				{
					case	def_end		:	section = param_sec; break;
					case	eval_end	:	section = param_sec; break;
					case	param_end	:	section = page_sec;  break;
					default			:			     break;
				}
				return (DOUBLEPOINT); 
			}
";"			return (SEMICOLON);
"?"			return (QUESTIONMARK);
")"			return (CLOSE_I);
"("			return (OPEN_I); 
"]"			return (CLOSE_II); 
"["			return (OPEN_II); 
"}"			return (CLOSE_III); 
"{"			return (OPEN_III); 
%%
//################################################################################################################
//
//	holt aktuellen Kommentar	
//
//################################################################################################################
void    CatchCommend(char* com, int len)
{
	char* tmp = com;
	if (actual_commend)
	{
		delete[] actual_commend;
		actual_commend=NULL;
	}
	tmp++; tmp++;
	strncpy(actual_commend = new char[len-3], tmp, (len-4));
}
//################################################################################################################
//
//	fuegt \__ROOT__ zum String "str" hinzu 
//
//################################################################################################################
char*	AddRoot(char* str)
{
	char*   ret_str = NULL;
	sprintf(ret_str = new char[(strlen(str)+11)], "\\__ROOT__%s", str);
	return (ret_str);
}
//################################################################################################################
//
//	fuegt den Pfad der aktuellen Seite zum String "str" hinzu 
//
//################################################################################################################
char*	AddActPage(char* str)
{
	if (!actual_page_name)	LexError("actual pagename is not defined");

	char*   absolut_name = NULL;
	sprintf(absolut_name = new char[(strlen(str)+strlen(actual_page_name)+3)], "%s\\%s", actual_page_name, str);
	return (absolut_name);
}
//################################################################################################################
//
//	fuegt den Namen der Master-Seite zum String str hinzu 
//
//################################################################################################################
char*	AddMaster(char* str)
{
	if (!master_page_name)	LexError("master pagename is not defined");

	char*   abs_name = NULL;
	sprintf(abs_name = new char[(strlen(str)+strlen(master_page_name)+3)], "%s\\%s", master_page_name, str);
	return (abs_name);
}
//################################################################################################################
//
//	fuegt den Pfad der aktuellen Definition zum String str hinzu 
//
//################################################################################################################
char*	AddDefPath(char* str)
{
	char*   def_path = GetPath(actual_definition);
	char*   abs_name = NULL;
	sprintf(abs_name = new char[(strlen(def_path) + strlen(str) + 3)], "%s\\%s", def_path, str);
	return (abs_name);
}
//################################################################################################################
//
//	ueberprueft ob actual_definition definiert ist
//
//################################################################################################################
char*	CheckActDef(char* str)
{
	if (!actual_definition) 
		strcpy(actual_definition = new char[(strlen(str) + 1)], str);
	return (str);
}
//################################################################################################################
//
//	testet ob der Pfad des str mit dem der aktuellen Seite uebereinstimmt 
//
//################################################################################################################
char*	CheckActPath(char* str)
{
	char* path_end = NULL;

	if (!(path_end = rindex(str, '\\'))) 
		LexError("can't get path of eval-string");

	if (strncmp(actual_page_name, str, (strlen(str) - strlen(path_end)) ) != 0)
		LexError("path of eval-string is not equal actual-page-name");

	return str;
}
//################################################################################################################
//
//	loescht actual_definition	
//
//################################################################################################################
void	CleanActDef()
{
	if (section==def_sec   || section==def_end  || 
	    section==eval_sec  || section==eval_end || 
	    section==param_sec || section==param_end )
	{
		if (actual_definition) { delete[] actual_definition; actual_definition = NULL; }
	}
}
//################################################################################################################
//
//	kreiert master_page_name sofern dieser noch nicht definiert ist	
//
//################################################################################################################
void	MakeMasterPageName(char* param_var)
{
	if (actual_definition) return;
	if (master_page_name)  delete[] master_page_name;

	master_page_name = NULL;
	char* param_path = GetPath(param_var);

	if (strcmp(param_path, "\\__ROOT__") == 0)
	{
		strcpy(master_page_name = new char[10], "\\__ROOT__");
		return;
	}
	master_page_name = GetPath(param_path);
}
//################################################################################################################
//
//	gibt den pfad eines strings zurueck 
//
//################################################################################################################
char* 	GetPath(char* str)
{
	char* 	str_path_end 	= NULL;
	int	str_len		= 0;

        if (!str || *str != '\\') 		  LexError("can't strip of path. Path does not begin with \\");
	if (!(str_path_end = rindex(str,  '\\'))) LexError("can't strip of path from str");
	
	str_len = strlen(str) - strlen(str_path_end);
	char* str_path = new char[(str_len + 2)];

	for (int i=0; i<str_len+1; i++) 
		str_path[i] = '\0';

	strncpy(str_path, str, str_len);
	return (str_path);
}
//################################################################################################################
//
//	- kreiert master_page_name, sofern diese nicht schon existiert
//	- testet, ob "str" eine Definition ist und gibt ggf. den Cell-Inhalt, andernfalls
//	  einen STRING an den Parser zurueck
//
//################################################################################################################
int	DefinitionOrString(char* str) 
{
	Cell* val = NULL;

	if (!actual_definition) MakeMasterPageName(str);
	else
	{
		if ((val=TestIfDefinition(str, new TextualList()))) 
			return (ReturnCellContents(val));
	}
	return (ReturnString(CheckActDef(str)));
}
//################################################################################################################
//
//	testet, ob "str" eine Definition ist und gibt ggf. den Cell-Inhalt, andernfalls
//	einen STRING an den Parser zurueck
//
//################################################################################################################
int	VarOrDefOrString(char* str)
{
	if (!actual_definition) 
	{
		strcpy(actual_definition = new char[(strlen(str) + 1)], str);
		return (ReturnString(str));
	}

	Cell* val = NULL;

	if ((val=TestIfDefinition(str, new TextualList()))) 	
		return (ReturnCellContents(val));
	return (ReturnString(str));
}
//################################################################################################################
//
//	testet, ob "str" eine Definition oder ein Parameter ist und gibt ggf. den Cell-Inhalt, andernfalls
//	einen STRING an den Parser zurueck
//
//################################################################################################################
int	DefOrParamOrString(char* str)
{
	Cell* val = NULL;

	if ((val=TestIfParameter (str, actual_page_name, new TextualList()))) 	return (ReturnCellContents(val));
	if ((val=TestIfDefinition(str,                   new TextualList()))) 	return (ReturnCellContents(val));

	return (ReturnString(str));
}
//################################################################################################################
//
// 	Testet, ob es zum gefundenen String einen passenden Eintrag in der PartameterList der aktuellen Seite
// 	gibt. 
// 	Ja   => Returnwert = gefundenen Eintrag
// 	Nein => Returnwert = NULL
//
//################################################################################################################
Cell*   TestIfParameter(char* test_str, char* pagename, TextualList* tl)
{
	if (!(PageRefList.size)) return NULL;

        //--------------------------------------------------------------------------------------------------------
        //	aktuelle Seite bestimmen
        //--------------------------------------------------------------------------------------------------------
	Page* actual_page = PageRefList.GetElementPtr(pagename);

	if (! actual_page)				return NULL;
	if (!(actual_page->GetParameters())) 		return NULL;
	if (!(actual_page->GetParameters()->size)) 	return NULL;

        Textual txt(test_str);
        //--------------------------------------------------------------------------------------------------------
        //	ueberpruefen ob der gefundene String einen Parameter der aktuellen Seite ist
        //--------------------------------------------------------------------------------------------------------
	for (DefinitionListElement* h=actual_page->GetParameters()->first; h; h=h->next)
	{
		//------------------------------------------------------------------------------------------------
		//	gefundener String ist ein Parameter der aktuellen Seite !!!
		//------------------------------------------------------------------------------------------------
		if (h->lelement.GetVar() == txt) 
		{
			//----------------------------------------------------------------------------------------
			//	Wert des gefundenen Parameters liefert direkt ein Ergebnis
			//----------------------------------------------------------------------------------------
			if (h->lelement.GetVal()->GetResultType() != textual) return (h->lelement.GetVal());

			//----------------------------------------------------------------------------------------
			//	Wert des gefundenen Parameters kann wiederum eine Variable eines Parameters sein
			//----------------------------------------------------------------------------------------
			if ((tl->UniqueAppend(txt)) < 0 ) LexError("parameter-loop");

			Cell* r_val   = NULL;
			char* val_txt = h->lelement.GetVal()->Calculate()->GetText();

			if (txt == val_txt) LexError("parameter-loop");

			//----------------------------------------------------------------------------------------
			//  testen ob der Wert des gefundenen Parameters ein anderen Parameter dieser Seite ist
			//----------------------------------------------------------------------------------------
			if ((r_val = TestIfParameter(val_txt, pagename, tl)))
				return (r_val);

			//----------------------------------------------------------------------------------------
			//  testen ob der Wert des gefundenen Parameters ein Parameter der Master-Seite ist
			//----------------------------------------------------------------------------------------
			for (PageListElement* page=PageRefList.first; page; page=page->next)
			{
				if ((page->lelement.IsSubPage(pagename)))
					if ((r_val = TestIfParameter(val_txt, page->lelement.GetName(), tl)))
			  			return (r_val);
			}
			//----------------------------------------------------------------------------------------
			//  testen ob der Wert des gefundenen Parameters einer Definition entspricht 
			//----------------------------------------------------------------------------------------
			if ((r_val = TestIfDefinition(val_txt, new TextualList())))
				return (r_val);

			//----------------------------------------------------------------------------------------
			//  nichts von allem; Also ist der Parameterwert ein String 
			//----------------------------------------------------------------------------------------
			return (h->lelement.GetVal());
		}
	}
	return NULL;
}
//################################################################################################################
//
// 	Testet, ob es zum gefundenen String einen passenden Eintrag in der "DefinitionRefList"
// 	gibt. 
// 	Ja   => Returnwert = gefundenen Eintrag
// 	Nein => Returnwert = NULL
//
//################################################################################################################
Cell*   TestIfDefinition(char* test_str, TextualList* tl)
{
	if (!DefinitionRefList.size) return NULL;

        Textual txt(test_str);
	for (DefinitionListElement* h=DefinitionRefList.first; h; h=h->next)
	{
		if (h->lelement.GetVar() == txt)
		{
			if ((tl->UniqueAppend(txt)) < 0 ) LexError("definition is definition of itself !");

			if (h->lelement.GetVal()->GetResultType() != textual) return (h->lelement.GetVal());

			if (DefinitionLoop(h->lelement.GetVal()->Calculate()->GetText(),txt)) 
				LexError("definition-loop");

			Cell* r_val = NULL;
			if ((r_val=TestIfDefinition(h->lelement.GetVal()->Calculate()->GetText(),tl)))
			  	return (r_val);
			return (h->lelement.GetVal());
		}
	}
	return NULL;
}
//################################################################################################################
//
// 	Ueberpruefung, ob es Definitions-Schleifen gibt 
//
//################################################################################################################
bool	DefinitionLoop(char* old_val, Textual old_var)
{
	for (DefinitionListElement* h=DefinitionRefList.first; h; h=h->next)
	{
		if (h->lelement.GetVar() == old_val)
		{
			if (h->lelement.GetVal()->GetResultType() != textual) return false;

			if (old_var == h->lelement.GetVal()->Calculate()->GetText()) 
				return true;
			return (DefinitionLoop(h->lelement.GetVal()->Calculate()->GetText(),old_var));
		}
	}
	return false;
}
//################################################################################################################
//
//	Kopiert "str" in yylval.s_val und uebergibt einen STRING an den Parser	
//
//################################################################################################################
int	ReturnString(char* str)
{
	strcpy(yylval.s_val = new char[(strlen(str) + 1)], str);
	return (STRING);
}
//################################################################################################################
//
// 	Copier-Funktionen zur Uebergabe spezifischer Definitionswerte an den Parser
//
//################################################################################################################
void    CopyCellPtr (Cell* def_val)   { yylval._CellPtr     = def_val->Copy(); 				}
void    CopyString  (char* def_str)   { strcpy(yylval.s_val = new char[strlen(def_str)+1],def_str); 	}

//################################################################################################################
//
// 	Untersucht welche Art von Definition vorliegt.
// 	Im Fall von numeric, part_multiset, elementary_multiset absolut_function, contents_function 
//	wird ein CellPtr zurueckgegeben (die "rechte Seite" der Definition).
// 	Im Fall von textual wird die "rechte Seite" der Defionition als char* zurueckgegeben.
//
//################################################################################################################
int     ReturnCellContents(Cell* def_val)
{
        switch (def_val->GetResultType())
	{
		case logic_expr   	: CopyCellPtr(def_val);				return (LOGIC); 
		case p_mset_expr       	: CopyCellPtr(def_val);				return (P_MSET);
		case e_mset_expr 	: CopyCellPtr(def_val);				return (E_MSET);
		case numeric 		: CopyCellPtr(def_val);				return (NUMERIC); 
		case a_function    	: CopyCellPtr(def_val);				return (A_FUNCTION);
		case c_function   	: CopyCellPtr(def_val);				return (C_FUNCTION);
		case textual    	: CopyString(def_val->Calculate()->GetText());	return (STRING);
		default    		: LexError("unknown ResultType in Definition");
	}
	return 0;
}
//################################################################################################################
//
//	Fehlerausgaben
//
//################################################################################################################
void 	LexError(char* s)
{
  switch (section)
  {
	case def_sec    : RRGErr=new RRGError(SCAN,s,lcount,"Definition"     ,actual_commend,NULL,yytext); break;
	case param_sec  : RRGErr=new RRGError(SCAN,s,lcount,"Parameter"      ,actual_commend,NULL,yytext); break;
	case page_sec   : RRGErr=new RRGError(SCAN,s,lcount,"Page"           ,actual_commend,NULL,yytext); break;
	case eplace_sec : RRGErr=new RRGError(SCAN,s,lcount,"ElementaryPlace",actual_commend,NULL,yytext); break;
	case pplace_sec : RRGErr=new RRGError(SCAN,s,lcount,"ProductPlace"   ,actual_commend,NULL,yytext); break;
	case trans_sec  : RRGErr=new RRGError(SCAN,s,lcount,"Transition"     ,actual_commend,NULL,yytext); break;
	case eval_sec   : RRGErr=new RRGError(SCAN,s,lcount,"Evaluation"     ,actual_commend,NULL,yytext); break;
	default         : RRGErr=new RRGError(SCAN,s,lcount,"Unknown"        ,actual_commend,NULL,yytext); break;
  }
  PrepareEnd();
}	
