/****************************************************************************
FUNKTION	:	SolveLGS()					          

KOMPONENTE	:	sor.c

ZWECK		:	Loesen der Matrixgleichung Ax=0
                        mittels SOR

Datum		:	April 1996

Programmierer  	:	Patrick Drescher
	
Projektleiter  	: 	Reinhard German				     

******************************************************************************

Modifiziert	:	Frank Bayer, 1997, Einbindung in agnes

******************************************************************************/
#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include <errno.h>

#include "sor.h"

/*
#define SHOW_MAT
*/

#define SCALE_DONE 0
#define NO_DIAG_ENTRY 1
#define DIAG_ZERO 2

/* vars from SolveLGS.c */

extern int SHOW_LGS;

/************************************************************************/
/*			system dependencies				*/
/************************************************************************/
/* functions  from SolveLGS.c */

#ifndef SOL2
extern display_rp_matrix();
extern double accuracy();
#else
extern double	accuracy	  (int, double*, int*, int*, double*);
extern void 	display_rp_matrix (int, double*, int*, int*);
#endif

/* functions from here */

#ifndef SOL2
int   sor();
int   scal();
void  insert_ones();
#else
int   sor         (int,int*,int*,double*,double*,int,double*,int,char*);
int   scal	  (int,int*,int*,double*,int*);
void  insert_ones (int,int**,int**,double**,int*);
#endif

/**************************************************************************
  Name			: sor

  Zweck			: L"osen der Gleichung Au=0 mittels SOR

  Eingabeparameter	: 
                          n  -> Dimension der Matrix
			  ia -> Zeilenpointer der Matrix
			  ja -> Spaltenindizes
			  a  -> Nichtnullelemente der matrix
			  u  -> Startvektor
			  max_it -> Maximale Anzahl von Iterationen
			  epsilon-> zu erreichende Genauigkeit

  Ausgabeparameter      :
                          u -> erhaltenen L"osung
			  eps -> Genauigkeit der L"osung

  Rueckgabewert		: 
                          ERROR -> Fehlerfall
			  NOT_CONV -> Methode nicht konvergiert
			  sonst: Anzahl der Iterationen
---------------------------------------------------------------------------*/
#ifndef SOL2
int sor(n, ia, ja, a,  Xold, max_it ,eps ,nonnull, netname)
int n, *ia, *ja;
double *a, *Xold; 
int max_it;
double *eps;
int nonnull;
char *netname;
#else
int sor(int n, int* ia, int* ja, double* a, double* Xold, int max_it , double* eps , int nonnull, char* netname)
#endif
{
   /*-----------------------------------------------------------------------------------*/
   /* 				local var's 						*/
   /*-----------------------------------------------------------------------------------*/

	double 	*Xnew, rest_zeile, omega, ff, ffold;	
	double 	*d;     			 /* matrix needed for sor */
	int    	*id, *jd;
	int    	nonnull_d;
	double	epsilon;
	int    	k, ka, kk;
	int    	ii, jj, ibgn, iend;

   /*-----------------------------------------------------------------------------------*/
   /* 				set initial parameters 					*/
   /*-----------------------------------------------------------------------------------*/

	epsilon	= *eps;
	k 	= 0;
	ka 	= 0;
	kk 	= 0;
	omega 	= 1.0;
	ffold 	= 1.0;
  
   /*-----------------------------------------------------------------------------------*/
   /* 				get d-matrix for SOR 					*/
   /*-----------------------------------------------------------------------------------*/

   /*-----------------------------------------------------------------------------------*/
   /* 				copying a-matrix to d-matrix 				*/ 
   /*-----------------------------------------------------------------------------------*/

  	nonnull_d = nonnull;

	if((id = (int*)malloc((unsigned)(n+1)*sizeof(int)))==NULL)
	{
		perror("sor - for Matrix description");
		exit(1);
	}
	if((jd = (int*)malloc((unsigned)nonnull_d*sizeof(int)))==NULL)
	{
		perror("sor - for Matrix description");
		exit(1);
	}
	if((d = (double*)malloc((unsigned)nonnull_d*sizeof(double)))==NULL)
	{
		perror("sor - for Matrix description");
		exit(1);
	}

	id = (int*)memcpy((char*)id, (char*)ia,(int)((n+1)*sizeof(int)));
	jd = (int*)memcpy((char*)jd, (char*)ja,(int)(nonnull_d*sizeof(int)));
	d = (double*)memcpy((char*)d,(char*)a, (int)(nonnull_d*sizeof(double)));


		/*----------------------------------------*/
		/*
		for (ii = 1; ii <= n; ii++)
			printf("_ja[%d]_ = %d\n",ii,ja[ii]);
		printf("\n");
		for (ii = 1; ii <= n; ii++)
			printf("_jd[%d]_ = %d\n",ii,jd[ii]);
		printf("------------------------\n");
		for (ii = 1; ii <= n; ii++)
			printf("_ia[%d]_ = %d\n",ii,ia[ii]);
		printf("\n");
		for (ii = 1; ii <= n; ii++)
			printf("_id[%d]_ = %d\n",ii,id[ii]);
		printf("------------------------\n");
		for (ii = 1; ii <= n; ii++)
			printf("_d[%d]_ = %g\n",ii,d[ii]);
		printf("\n");
		for (ii = 1; ii <= n; ii++)
			printf("_a[%d]_ = %g\n",ii,a[ii]);
		printf("------------------------\n");
		*/
		/*----------------------------------------*/
   /*-----------------------------------------------------------------------------------*/
   /*     		scale d-matrix and remove diagonal elements			*/ 
   /*-----------------------------------------------------------------------------------*/

  	if(scal(n, id, jd, d, &nonnull_d) != SCALE_DONE)
    		return ERROR;

	if (SHOW_LGS)
	{
        	printf("\n\t\t scaled d-matrix with removed diagonal elements:");
		display_rp_matrix(n,d,id,jd);
		fflush(stdout);
	}	

   /*-----------------------------------------------------------------------------------*/
   /* 			exchange last row with ones in d-matrix 			*/
   /*-----------------------------------------------------------------------------------*/
  	insert_ones(n, &id, &jd, &d, &nonnull_d);

	if (SHOW_LGS)
	{
        	printf("\n\t\t last row exchanged with ones in d-matrix except diagonal element:");
		display_rp_matrix(n,d,id,jd);
		fflush(stdout);
	}

	if (NULL == (Xnew = (double*) malloc ((int)n * sizeof(double))))
	{
		perror("p_sor --  malloc: vector Xnew\n");
		exit(1);
	}
 
   /*-----------------------------------------------------------------------------------*/
   /*  		Berechnung: Gesamtschrittverfahren mit adaptiver omega-Bestimmung	*/
   /*			    Startvektor == (1/Dm,...,1/Dm) 				*/
   /*-----------------------------------------------------------------------------------*/

	if (SHOW_LGS)
	{
        	printf("\n\t\t a-matrix for checking reached precision:");
		display_rp_matrix(n,a,ia,ja);
		fflush(stdout);
	}

   /*-----------------------------------------------------------------------------------*/
   /* 				Parameter adjustments 					*/
   /*-----------------------------------------------------------------------------------*/
	--ia, --ja, --a, --Xold, --Xnew , --id, --jd, --d; 

	if ( (max_it >= 100) && SHOW_LGS )
    	{
		/********************
		printf("\n\t\t number of performed iterations (stepsize = 100):\n");
		********************/
		fflush(stdout);
	}

	while (k < max_it)
	{
		++k;
		ka++;
		if (ka == 100)
		{
			/***************************
			if (SHOW_LGS) 
			{ 
				printf("_%u", k); 
				fflush(stdout); 
			}
			***************************/
			ka = 0;
		}

		for (ii = 1; ii <= n; ii++)
		{
			ibgn = id[ii];
			iend = id[ii+1] - 1;
			rest_zeile = 0.0;

			for (jj = ibgn; jj <= iend; jj++)
	    		{
				rest_zeile += d[jj] * Xold[jd[jj]];
			}

			if (ii == n) 	Xnew[ii] = 1.0 - rest_zeile;
          		else 		Xnew[ii] = 0.0 - rest_zeile;
		}

		ff = accuracy(n, a, ia, ja, Xnew);

		if (ff < epsilon)
		{
			if (k >= 100)
          		{
				printf("\n");
				fflush(stdout);
			}
			*eps = ff;
			return(k);
		}
      
		if (ff < ffold) omega = 0.8;
		else
		{
			if (!ff)
			{
				printf("\t\t SolveLGS : division by zero in function sor()"); 
				exit(1);
			}
			omega = fabs((ffold / ff) - (omega * 0.24));
		}

		for (ii = 1; ii <= n; ii++)
		{
			Xnew[ii] = Xold[ii] + (omega * (Xnew[ii] - Xold[ii]));
			Xold[ii] = Xnew[ii];
		}

		ff = accuracy(n, a, ia, ja, Xnew);

		if (ff < epsilon)
		{
			if (k >= 100)
			{
				printf("\n");
				fflush(stdout);
			}
			*eps = ff;
			return(k);
		}
		ffold = ff;

	}	/*  end while	*/

	if (k >= 100)
	{
		printf("\n");
		fflush(stdout);
	}
	*eps = ffold;
	return(NOT_CONV);
}

/**************************************************************************
  Name			: insert_ones

  Zweck			: letzte Zeile der Matrix wird durch Einsen
                          ersetzt (ausser Diagonalelement)

  Eingabeparameter	: 
                          n  -> Dimension der Matrix
			  ix -> Zeilenpointer der Matrix
			  jx -> Spaltenindizes
			  x  -> Nichtnullelemente der Matrix
                          nonnull -> Anzahl der Nichtnullelemente
    
---------------------------------------------------------------------------*/
#ifndef SOL2
void insert_ones(n, ix, jx, x, nonnull)
int n, **ix, **jx, *nonnull;
double **x;
#else
void insert_ones(int n, int** ix, int** jx, double** x, int* nonnull)
#endif
{

/* ... specifications for local variables */
register int no_last_row_entries,
             add_no_elements,
             i_bgn,
             i_end,
             jj, 
             j;

  no_last_row_entries = ix[0][n] - ix[0][n-1];
  add_no_elements = (n - 1) - no_last_row_entries;
  *nonnull += add_no_elements;

  if((*x = (double*)realloc((char*)*x,(unsigned)*nonnull*sizeof(double)))==NULL)
    {
      perror("sor - for Matrix description");
      exit(1);
    }
  if((*jx = (int*)realloc((char*)*jx,(unsigned)*nonnull*sizeof(int)))==NULL)
    {      
      perror("sor - for Matrix description");
      exit(1);
    }

  i_bgn = ix[0][n-1] - 1;
  i_end = i_bgn + add_no_elements - 1;
  for (jj=i_bgn, j=0; jj<=i_end; jj++)
    {
      ++j;
      jx[0][jj] = j;
      x[0][jj] = 1.0;   
    }
  ix[0][n] = ix[0][n-1] + add_no_elements;
}


/**************************************************************************
  Name			: scal

  Zweck			: Extrahieren der Diagonalen aus der Matrix
                          Skalieren des Systems, so da"s die Diagonalelemente
			  zu eins werden. Diagonalelemente entfernen.

  Eingabeparameter	: 
                          n  -> Dimension der Matrix
			  ix -> Zeilenpointer der Matrix
			  jx -> Spaltenindizes
			  x  -> Nichtnullelemente der Matrix
                          nonnull -> Anzahl der Nichtnullelemente
    
  Rueckgabewert		: 
                          NO_DIAG_ENTRY -> Zeile ohne Diagonalelement gefunden
			  DIAG_ZERO     -> Diagonalelement ist Null
			  SCALE_DONE    -> Skalierung vollzogen
---------------------------------------------------------------------------*/
#ifndef SOL2
int scal(n, ix, jx, x, nonnull)
int n, *ix, *jx, *nonnull;
double *x;
#else
int scal(int n, int* ix, int* jx, double* x, int* nonnull)
#endif
{

/* ... specifications for local variables */

register int ibgn, iend, ii, jj, diags_killed;
double di,*d;


  d = (double*)malloc(n*sizeof(double));

  /* Parameter adjustments */
    --d;    --x;    --jx;    --ix;
  /* ... extract  the diagonal out of a */
    for(ii = 1; ii <= n; ii++) {
	ibgn = ix[ii];
	iend = ix[ii + 1] - 1;
	if(ibgn > iend) {
	    printf("\n\t\t NO_DIAG_ENTRY");
	    return NO_DIAG_ENTRY;
	}
	for(jj = ibgn; jj <= iend; jj++) {
	    if(jx[jj] != ii)
		continue;
	    di = x[jj];
	    if(di == 0.0) {
	    	printf("\n\t\t DIAG_ZERO");
		return DIAG_ZERO;
	    }
	    break;
	}
	if(jj > iend) {
	    printf("\n\t\t NO_DIAG_ENTRY");
	    return NO_DIAG_ENTRY;
	}
	d[ii] = di;
    }

  /* ... shift matrix to eliminate diagonal entries and scale shifted matrix */

    diags_killed = 0;
    for(ii = 1; ii <= n; ii++) {
	ibgn = ix[ii];
	iend = ix[ii + 1] - 1;
	for(jj = ibgn; jj <= iend; jj++) {
            if (jx[jj] == ii)
                ++diags_killed;
            else
             {
             	if (!d[ii])
             	{
             		printf("\t\t SolveLGS : division by zero in function scal()");
             		exit(1);
		}
               	jx[jj-diags_killed] = jx[jj];
               	x[jj-diags_killed]  = x[jj]/d[ii];
             }
        }
    }
    for (ii=1; ii<=n; ii++)
        ix[ii+1] -= ii;
    *nonnull -= diags_killed;

 free(++d);

return SCALE_DONE;
}





