#include <sys/time.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/resource.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <netdb.h>

#include <iostream.h>
#include <builtin.h>

#include "portab.h"
#include "globals.h"
#include "Accomplice.h"
#include "Sample.h"
#include "M_Measure.h"

#define HOSTNAME(_a) ((_a)?(_a):"Localhost")

Accomplice::Accomplice () {

  local     = true;
  hostname  = NULL;
  InitProcess();
}


Accomplice::Accomplice (char *host) {

  if (host == NULL)
    local = true;
  else
    local = false;

  hostname  = host;
  InitProcess();
}


void Accomplice::InitProcess() {

  min_firing = false;
  status    = WAIT_FOR_CONNECT;
  modeltime = 0.0;
  sprintf (seed, "%ld", rnd());
  
  switch ((pid = fork())) {
  case -1:
    perror ("Konnte keinen Prozess abspalten fuer den Slave.");
    my_exit (-1);
    break;
  case 0:  // Der Sohn wird ueberlagert mit der rsh(1)
    int temp_fd;
    if ((temp_fd = open(G_netfile, O_RDONLY)) < 0) {
      // Das duerfte nie passieren, da main() den Lesezugriff bereits prueft
      perror ("Fatal error: Ein Slave kann die Netzdatei nicht oeffnen.");
      exit (-1); // hier nicht my_exit, da die anderen Slaves ja nicht gestoppt werden muessen
    }

    dup2 (temp_fd,        0);  // Tablefile auf cin legen
    dup2 (G_slave_out_fd, 1);  // cout und cerr auf die Slave-Ausgabe-Datei umlegen
    dup2 (G_slave_out_fd, 2);
    if (local) {
      if (G_trans_detection)
	execl (G_slavecommand, "slave", 
		"-p", P_portnumber, "-s", seed, "-f", P_min_firings,
		"-m", P_maxsamples, "-c", P_conflevel, NULL);
      else
	execl (G_slavecommand, "slave", 
		"-p", P_portnumber, "-s", seed, "-f", P_min_firings,
		"-m", P_maxsamples, "-c", P_conflevel, "-t", NULL);
      cout << G_slavecommand << '\n';
      perror ("Konnte den Slave nicht starten.");
      exit(-1); // hier nicht my_exit, da die anderen Slaves deshalb ja nicht gestoppt werden muessen
    }
    else {
      if (G_trans_detection)
	execlp ("rsh", "rsh", hostname, G_slavecommand, 
		"-h", P_localhost, "-p", P_portnumber, "-s", seed, "-f", P_min_firings,
		"-m", P_maxsamples, "-c", P_conflevel, NULL);
      else
	execlp ("rsh", "rsh", hostname, G_slavecommand, 
		"-h", P_localhost, "-p", P_portnumber, "-s", seed, "-f", P_min_firings,
		"-m", P_maxsamples, "-c", P_conflevel, "-t", NULL);
      perror ("Konnte die rsh(1) nicht starten.");
    }
    exit (-1); // Hier nicht my_exit, da die anderen Slaves deshalb ja nicht gestoppt werden muessen
    break;
  default: // Ich bin der Vater
    break;
  }
}

char *Accomplice::GetHostName () {
  if (hostname == NULL)
    return NULL;

  char *str = new char[sizeof(hostname)];
  strcpy (str, hostname);
  return str;
}



Accomplice::~Accomplice() {
  Kill();
}


void Accomplice::Expect(char *string) {
  char *from_socket = ReadString();

  if (strcmp(from_socket, string)) {
    cout << "Protokollfehler: '" << string << "' erwartet, '" << from_socket << "' bekommen.\n";
    my_exit(-1);
  }
  delete from_socket;
}
  
  
char *Accomplice::ReadString() {
  // liest bis zu einem \0
  // Ein String darf hoechstens MAXSTRING Zeichen lang sein
  // Sonst ist es ein Protokollfehler!

  char *ergebnis = new char[MAXSTRING];
  char *pointer = ergebnis - 1;
  char *maxpointer = ergebnis+MAXSTRING-1;
  int   count;

  do {
    pointer++;
    while ((count = read(socket, pointer, 1)) != 1) {
      if (count < 0) {
	perror ("Fehler beim String lesen.");
	my_exit(-1);
      }
    }
    if (pointer == maxpointer) {
      pointer = '\0';
      cout << "Ein Slave liefert einen viel zu langen String. \n'";
      cout << ergebnis << "' u.s.w. Protokollfehler!\n";
      my_exit(-1);
    }
  } while (*pointer); // Ende bei *pointer == '\0'

  return ergebnis;
}


int Accomplice::ReadInt() {
  int   ergebnis, count = 0, length = sizeof(int);
  char   *pointer = (char *) &ergebnis;


  while (length && (count = read(socket, pointer, length)) != length) {
    if (count >= 0) {
      pointer += count;
      length  -= count;
    } 
    else {
      perror ("Fehler beim Int lesen.");
      my_exit(-1);
    }
  }

  return (ntohl(ergebnis));
}



double Accomplice::ReadDouble() {
  double ergebnis;
  int count = 0, length = sizeof(double);
  char *pointer = (char *) &ergebnis;

  while (length && (count = read(socket, pointer, length)) != length) {
    if (count >= 0) {
      pointer += count;
      length  -= count;
    }
    else {
      perror ("Fehler beim Double lesen.");
      my_exit(-1);
    }
  }
//  cout << " Wert = " << ergebnis << '\n';
  return ergebnis;
}


M_Measure *Accomplice::ReadReward() {

  char *name;
  int i;
  double size;
  double *definition;

  // zuerst der Name des Measures
  Expect("REW_NAME");
  name = ReadString();

  // Die Groesse des Definitionsarrays
  Expect("REW_DEFINITION");
  size = ReadDouble();
  if (double(int(size)) != size) {
    cout << "Fliesskommazahl als Arraygroesse in Accomplice::ReadReward()";
    cout << " fuer " << name << endl;
    my_exit(-1);
  }

  // Und nun die Definition
  definition = new double[int(size)+1];
  definition[0] = size;
  for (i = 1; i <= int(size); ++i)
    definition[i] = ReadDouble();
  Expect("REW_END");
  
  // Und das Objekt erzeugen (im Konstruktor wird geparst)
  return (new M_Measure(name, definition));

}


void Accomplice::Handshake() {
  int measure_count, primitive_count, i;

  Expect("#MEASURES");
  measure_count = ReadInt();
  Expect ("#PRIMITIVES");
  primitive_count = ReadInt();

  if (PrimList.Size() == 0) {
    // zum ersten mal erhalten wir diese Information
    for (i = 0; i < primitive_count; ++i)
      PrimList.Append(new Statistic());   // Liste der primitiven Ausdruecke
    }
  else {
    // Ein anderer Slave hat bereits die Zahl der Primitiven
    // uebertragen. Wir muessen nur testen, ob dieser Slave
    // genauso viele Primitiven hat. Sonst haben wir ein groesseres Problem...
    if (PrimList.Size() != primitive_count) {
      cout << "Der Slave auf '" << HOSTNAME(hostname) << "' liefert falsche Zahl";
      cout << " der Primitiven. (" << primitive_count;
      cout << " statt " << PrimList.Size() << ")\n";
      my_exit(-1);
    }
  }


  if (StatList.Size() == 0) {
    // Zum ersten Mal erhalten wir diese Information
    // daher StatList aufbauen mit den gelesenen Namen
    for (i = 0; i < measure_count; ++i) 
      StatList.Append(ReadReward());
  }
  else {
    // Ein anderer Slave hat bereits die Daten fuer
    // die StatList uebermittelt. Wir muessen jetzt
    // abgleichen (Alle Slaves sollten die gleichen Measures ermitteln)
    if (measure_count != StatList.Size()) {
      cout << "Der Slave auf '" << HOSTNAME(hostname) << "' liefert falsche Zahl";
      cout << " an Measures. (" << measure_count;
      cout << " statt " << StatList.Size() << ")\n";
      my_exit(-1);
    }
    M_Measure *testmeasure;
    for (i = 0; i < measure_count; ++i) {
      testmeasure = ReadReward();
      if (!(*testmeasure == *(StatList[i]))) {
	cout << "Die Slaves liefern unterschiedliche Definitionen fuer die ";
	cout << "Reward-Measures. " << testmeasure->GetName() << " und ";
	cout << StatList[i]->GetName() << " unterscheiden sich. Abbruch.\n";
	my_exit(-1);
      }
      delete testmeasure;  // ReadReward erzeugt neues Objekt
    }
  }
  
  still_transient = new bool[primitive_count];
  for (i = 0; i < PrimList.Size(); ++i)
    still_transient[i] = true;

  // cout << " Handshake erfolgreich beendet.\n";
  status = RECEIVING;
}



bool Accomplice::CheckIfMySlave(unsigned long iaddress, int new_fd) {
  struct hostent *entry = NULL;

  if (status == WAIT_FOR_CONNECT) {

    if ((hostname == NULL) && (iaddress == htonl(INADDR_LOOPBACK))) {
      // localhost
      socket = new_fd;
      FD_SET (socket, &G_readset);
      Handshake();
      return true;
    }
    

    entry = gethostbyname(hostname);

    if (entry != NULL) {
      if (iaddress == *(unsigned long *)(entry->h_addr)) {
	socket = new_fd;
	FD_SET (socket, &G_readset);
	Handshake();
	return true;
      }
    }
  }
  return false;
}


SlaveStatus Accomplice::GetStatus() {
  return status;
}


int Accomplice::GetPid() {
  return pid;
}


void Accomplice::Kill() {
  int pid2;
  void *handler;
  timeval start_time, akt_time;

  // Den alten Signalhandler mal kurz ausschalten
  handler = signal(SIGCHLD, SIG_DFL);

  sigsend (P_PID, pid, SIGINT);  // siehe portab.h fuer sigsend unter linux

  gettimeofday(&start_time, NULL);
  akt_time= start_time; 

  while (((pid2 = waitpid(pid, NULL, WNOHANG)) == 0) &&
	 ((akt_time.tv_sec-start_time.tv_sec) < 30))
    gettimeofday(&akt_time, NULL); // Dummy-Aufruf

  if (pid2 == 0) {
    perror ("Konnte nach 60s rsh(1) immer noch nicht terminieren.\nProzess bleibt leider uebrig.");
    sigsend (P_PID, pid, SIGKILL);  // Siehe portab.h fuer sigsend unter LinuX
  }

  if (pid2 < 0) {
    perror ("Fehler beim Warten auf einen rsh(1)-Prozess.");
    exit(-1);
  }

  status = DEAD;

  // Und den ueblich Handler wieder aktivieren
  signal (SIGCHLD, (void (*) (int))handler);

  // delete still_transient;
}


double Accomplice::GetCurrentTime() {
  return modeltime;
}


void Accomplice::ReadData(int myindex) {
  // Precondition: Dieser Komplize hat den Status RECEIVING
#ifdef _DEBUG
  if (status != RECEIVING) {
    cout << "Verletzung der Vorbedingung in Accomplice::ReadData\n";
    my_exit(-1);
  }
#endif

  if (!FD_ISSET(socket, &G_readset)) {
    // Dieser Komplize loeste den Select nicht aus.
    FD_SET (socket, &G_readset);      // Erstmal wieder anmelden zum naechsten Lesen
    return;
}

  int prim_index, buf_size;
  Statistic *primitive;
  Sample *samples;


  prim_index = ReadInt();
//cout << "Accomplice::ReadData index=" << prim_index << endl;
  if (prim_index == -1) {
    // Das bedeutet, dass alle Bindings im Slave <min-firings> mal gefeuert haben!
    if (min_firing) {
      // Oh! Ein Fehler. Min_firing war schon auf true und nun kommt dieses
      // Signal vom Slave noch einmal? Es darf nur einmal min_firing gesendet
      // werden. Protokollfehler. Ende.
      cout << "Das \"min_firing\"-Signal wurde vom Slave auf dem Rechner " << hostname;
      cout << "\nzweimal gesendet. Damit verstoesst er gegen die Protokolldefinition.\n";
      my_exit(-1);
    }
    min_firing = true;
    return;
  }

  if (prim_index == -2) {
    // Das ist nur ein Lebenszeichen vom Slave. Damit wird der 60sek Timeout verhindert.
    Expect("WDTIME");
    modeltime = ReadDouble();
    return;
  }

  if ((prim_index < 0) || (prim_index >= PrimList.Size())) {
    cout << "Ein Slave liefert einen ueberhoehten Primitive-Index.\n";
    my_exit(-1);
  }
  primitive = PrimList[prim_index];

  if (still_transient[prim_index]) {
    // Wir muessen erstmal Begin_of_Stationary empfangen
    Expect ("TRANSIENT");
    still_transient[prim_index] = false;
    primitive->AddTransientTime(ReadDouble());
    return;
  }

  Expect("TIME");
  modeltime = ReadDouble();

  Expect("BUFSIZE");
  buf_size = ReadInt();
  samples = new Sample[buf_size];
  Expect("DATA");
  for (int i = 0; i < buf_size; ++i) {
    samples[i].value = ReadDouble();
    samples[i].time = ReadDouble();
  }

  primitive->AddSamples(samples, buf_size, myindex, modeltime);
  delete samples;
  Expect("COMPLETE");
//  cout << '.' << flush;
}







