#ifndef __ACCOMPLICE_H__
#define __ACCOMPLICE_H__

class M_Measure;

#define MAXSTRING 256

typedef enum { WAIT_FOR_CONNECT, // Der Slave initialisiert sich gerade
	       RECEIVING,        // Simulationsdaten werden laufend empfangen
	       DEAD              // Der Slave meldet sich nicht mehr :-(
} SlaveStatus;

//****************************************************
//
//  Der Komplize verwaltet je einen Simulationslauf
//  auf einem anderen Rechner.
//  Fuer die Simulation wird ein Prozess abgespalten,
//  der eine Remoteshell erzeugt und per Socket eine
//  Verbindung zum Slave gehalten.
//  Die empfangenen Daten werden gespeichert und die
//  Laenge der transienten Phase ermittelt.
//  Alle weiteren Daten werden im globalen Store-Objekt
//  gespeichert.
//  Bei Beendigung des Masters werden die Kind-Prozesse
//  noch schnell getoetet.

class Accomplice {
  char         *hostname;        // Name des Slave-Rechners
  SlaveStatus  status;           // aktueller Status
  int          pid;              // Prozess-ID des lokalen 'rsh'-Prozesses
  double       modeltime;        // aktuelle Zeit der Simulation
  int          socket;           // Der Socket zum Slave-Prozess
  char         seed[20];         // Der seed fuer den Slave als Parameter
  bool         local;            // True, wenn der Slave auf dem localhost laufen soll
  bool         *still_transient; // Status der Measures dieses Slaves (true = transient, false = stationaer)
  bool         min_firing;       // Haben alle Bindings des Slaves oft genug geschaltet?
  void         Expect(char *);   // Liest einen vorgegebenen String vom Socket oder exit()
  char         *ReadString();    // Liest einen '\0'-terminierten String vom Socket (len < MAXSTRING)
  int          ReadInt();        // Liest einen Integer-Wert vom Socket (in Network-Byte-Order)
  double       ReadDouble();     // Liest einen Double-Wert vom Socket
  M_Measure    *ReadReward();    // Liest ein Measure-Objekt vom Socket
  void         InitProcess();    // Gemeinsamer Teil beider Konstruktoren
  void         Handshake();      // Liest die Measure-Namen vom Slave und 
                                 // vergleicht sie mit den anderen Slaves

  void         check_consistency();    // Nur zum Debugging
  void         check_consistency(int); // Nur zum Debugging

public:
  Accomplice ();               // Fuer Slaves auf dem localhost 
  Accomplice (char *host);     // Fuer Slaves, die fremd rechnen
  ~Accomplice();

  bool         CheckIfMySlave(unsigned long iaddress, int new_fd);
  SlaveStatus  GetStatus();
  int          GetPid();
  void         Kill();
  void         ReadData(int);
  double       GetCurrentTime();
  char *       GetHostName();
  bool         GetMinFireFlag() {return min_firing;};
  friend void  handle_dead_child(int);
};


#endif
