extern "C" {
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h> 
#include <sys/wait.h>
#include <signal.h>
#include <values.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <ctype.h>
#include <netinet/in.h>
#include <netdb.h>
}

#include <builtin.h>
#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>
#include <GetOpt.h>
#include "globals.h"
#include "portab.h"
#include "Accomplice.h"
#include "M_Measure.h"


#define REAL_TIME_EXCEEDED  -42
#define MODEL_TIME_EXCEEDED -43


#define AGNESFILE       "/.TimeNET"
#define MAXLINE         256      
#define HOSTNAME(_a)    ((_a)?(_a):"Localhost")
#define MAXHOSTNAME     100
#define SLAVEPFAD       "/MSSimulation/bin/slave"
#define SLAVEOUTFILE    "/SimSlaves.ERROR"
#define ERROR_FILE_INIT "Fehlerausgaben\n\nEs folgen die Ausgaben der Simulations-Slaves \nbzw. der Remote-Shell Prozesse.\n\n"

// Die Token beim Parsen des .Agnes-Files
typedef enum { I_EOF,           
	       I_MAX_PROCS,     
	       I_HOSTLISTSTART,
//	       I_USE_ALLWAYS,    noch nicht implementiert. USE ALWAYS wird auf I_DATALINE abgebildet
	       I_HOSTLISTEND,
	       I_LOCAL,
	       I_DISTRIBUTED,
	       I_TOTAL_PROCS,
	       I_TOO_LONG,       // Eingabezeile ist viel zu lang ( > MAXLINE)
	       I_IGNORE,         // Alle Keywords, die vom Simulationsmodul ignoriert werden
	       I_DATALINE        // Alle Zeilen, die nicht mit einem anstaendigen Keyword beginnen
 } T_InitCommand;


// Globale Variablen
//
// manchmal machen sie das Leben viel einfacher... ;-)

char                     P_portnumber[20];          // Die Portnummer des Servers
char                     P_conflevel[20];           // Confidence-Level in % als Parameter fuer den Slave
double                   G_conflevel;               // Confidence-Level zum Rechnen
char                     P_maxsamples[20];          // Hoechstzahl der generierten Samples
char                     P_min_firings[20];         // Mindest-Schaltzahl je Binding
bool                     G_trans_detection;         // Samples der Transiente Phase verwerfen ?
char                    *G_netname;                 // Name des zu simulierenden Netzes
double                   G_maxeps;                  // der maximale relative Fehler
ComfVector<Statistic *>  PrimList;                  // die Liste der Reward_Measure-Primitiven
ComfVector<M_Measure *>  StatList;                  // Liste der Reward-Measures
ComfVector<Accomplice *> SlaveList;                 // Liste der Komplizen
char                    *G_slavecommand;            // Der Kommandoaufruf incl. Pfad vom Slave
RandomInteger            rnd(0,MAXINT, new ACG(0)); // Der Generator fuer die zufaelligen Seeds der Slaves
fd_set                   G_readset;                 // Liste aller lesenden Socket-FD's
int                      G_slave_out_fd;            // Filedescriptor fuer alle cout und cerr der (Slave|rsh)s
char                    *G_netfile;                 // Voller Name des Tablefiles
char                    *G_slaveoutfile;            // Filename fuer alle cout und cerr der (Slave|rsh)s
char                    *P_localhost;               // Name des lokalen Hosts als Parameter fuer den Slave
double                   G_maxmodeltime;            // Maximale Modellzeit (default MAXDOUBLE)
int                      G_maxrealtime;             // Maximale reale Simulationszeit (default MAXDOUBLE)




class Master {

  int                      Server;        // Socket fuer den Server

  void ReadInitFile();                 // Liest das .Agnes-File und instanziiert die Accomplice-Objekte
  void InitSocket();                   // Kreiert einen Socket fuer den Server
  bool StillWaiting();                 // Testet, ob noch Slaves auf einen Connect warten
  void InitSlaves();                   // Kreiert die Accomplicies und startet die Slaves
  void MainLoop();                     // Die Hauptschleife mit Daten lesen und verarbeiten
  T_InitCommand GetInitCommand(ifstream&, int *, char *); // Zum parsen des .Agnes-Files

public:

  Master();
  void PrintResults();                 // Ausgabe der ermittelten Werte am Ende der Simulation
} *master;


Master::Master() {
  
  cout << "Beginn der Simulation...\n" << flush;
  InitSocket();
  ReadInitFile();
  InitSlaves();
  MainLoop();
}


bool m_namecmp (char *s1, char *s2) {

   while (*s1 && *s2 && *s1 != '.' && *s2 != '.') {
    if (*s1 != *s2)
      return false;
    s1++;
    s2++;
  }
  return true;
} 


int m_strcmp (char *s1, char *s2) {

  int i = 0;
//  cout << "Vergleiche '" << s1 << "' und '" << s2 << "'\n";
  while (*s1 && *s2) {
//    cout << "Bin drin\n";
    if (*s1 != *s2)
      return i;
    s1++;
    s2++;
    i++;
  }
  return i;
}


bool m_isempty (char *s1) {

 for (; *s1; ++s1)
   if (!isspace(*s1))
    return false;

  return true;
}


T_InitCommand Master::GetInitCommand(ifstream &f, int *value, char *line) {

  f.getline(line, MAXLINE);
  if (f.fail())
    if (f.eof() && sizeof(line) > 0)
      return I_EOF;
    else
      return I_TOO_LONG;

  if (m_strcmp(line, "MAX PROCESSES PER HOST ") == 23) {
    sscanf(&line[23],"%d", value);
    return I_MAX_PROCS;
  }

  if (m_strcmp(line, "MAX SIMULATION PROCESSES TOTAL ") == 31) {
    sscanf(&line[31],"%d", value);
    return I_TOTAL_PROCS;
  }

  if (m_strcmp(line, "HOSTLISTSTART") == 13)
    return I_HOSTLISTSTART;

  if (m_strcmp(line, "HOSTLISTEND") == 11)
    return I_HOSTLISTEND;

  if (m_strcmp(line, "USE ALWAYS ") == 11) {
    *value = 11;
    return I_DATALINE;
  }

  if (m_strcmp(line, "PRIORITY") == 8)
    return I_IGNORE;

  if (m_strcmp(line, "MEMORY") == 6)
    return I_IGNORE;

  if (m_strcmp(line, "FORCE DISTRIBUTED EXECUTION") == 27)
    return I_DISTRIBUTED;

  if (m_strcmp(line, "FORCE LOCAL EXECUTION") == 21)
    return I_LOCAL;

  if (m_strcmp(line, "EPSILON") == 7)
    return I_IGNORE;

  if (*line == '#')
    return I_IGNORE;

  if (m_isempty(line))
    return I_IGNORE;


  *value = 0;
  return I_DATALINE;
}

  
void Master::ReadInitFile() {

  enum {LOCAL, DISTRIBUTED, NEVER_MIND}   distr = NEVER_MIND;

  char     line[MAXLINE], *dname;
  int      value, maxprocs = 1, totalprocs = 20, lineno = 0, i = 0;
  bool     outer_end = false, inner_end = false;
  ifstream afile;
  ComfVector<char *> namelist;

  dname = getenv ("HOME");
  if (dname == NULL) {
    cout << "$HOME nicht gesetzt.\n";
    exit(-1);
  }

  char *fname = new char[sizeof(dname) + sizeof(AGNESFILE) + 1];
  strcpy (fname, dname);
  strcat (fname, AGNESFILE);
  afile.open(fname);
  if (!afile) {
    cout << "Kann das File '" << fname << "' nicht oeffnen.\n";
    cout << "Nur ein lokaler Slave initialisiert.\n";
    SlaveList.Append(new Accomplice());
    return;
  }

  do {
    lineno++;
    switch(GetInitCommand(afile, &value, line)) {
    case I_IGNORE: break;
    case I_TOO_LONG:
      cout << fname << ':' << lineno << " Eingabezeile viel zu lang. Wird ignoriert.\n";
      break;
    case I_MAX_PROCS:
      if (value < 1) {
        cout << fname << ':' << lineno << " Nur " << value << " Prozesse pro Host? Ich setze auf 1.\n";
        value = 1;
      }
      if (value > 10) {
        cout << fname << ':' << lineno << value << " Prozesse pro Host ist zuviel. Ich setze auf 10\n";
        value = 10;
      }
      maxprocs = value;
      break;

    case I_TOTAL_PROCS:
      if (value < 1) {
        cout << fname << ':' << lineno << " Nur " << value << " Simulations-Prozesse? Ich setze auf 1.\n";
        value = 1;
      }
      if (value > 20) {
        cout << fname << ':' << lineno <<": "<<value << " Prozesse insgesamt sind zuviel. Ich setze auf 20\n";
        value = 20;
      }
      totalprocs = value;
      break;

    case I_DISTRIBUTED:
      if (distr == NEVER_MIND)
        distr = DISTRIBUTED;
      else {
        cout << fname << ':' << lineno << ": FORCE * EXECUTION ist nur einmal erlaubt.";
        cout << " Ich simuliere nur lokal.\n";
        distr = LOCAL;
      }
      break;

    case I_LOCAL:
      if (distr == NEVER_MIND)
        distr = LOCAL;
      else {
        cout << fname << ':' << lineno << ": FORCE * EXECUTION ist nur einmal erlaubt.";
        cout << " Ich simuliere nur lokal.\n";
        distr = LOCAL;
      }
      break;

    case I_HOSTLISTSTART:
      inner_end = false;
      do {
        lineno++;
        switch (GetInitCommand(afile, &value, line)) {
        case I_DATALINE:
          if (m_namecmp (P_localhost, &line[value]))
            SlaveList.Append(new Accomplice());
          else {
            char *str = new char[sizeof(&line[value])];
            strcpy (str, &line[value]);
            namelist.Append(str);
          }
          break;
        case I_EOF:
          cout << fname << ':' << lineno << " HostList ist nicht korrekt terminiert. Bitte mal korrigieren.\n";
          inner_end = true;
          break;
        case I_HOSTLISTEND:
          inner_end = true;
          break;
        case I_IGNORE:
          break;
        default:
          cout << fname << ':' << lineno << " Falsches Schluesselwort in der HostList.\n";
          break;
        }
      } while (!inner_end);
      break;
    case I_EOF:
      outer_end = true;
      break;
    default:
      cout << fname << ':' << lineno << " Unbekanntes Schluesselwort in Zeile '" << line << "'\n";
      break;
    }
  } while (!outer_end);

  // Falls nur lokale Simulation muessen alle fremden Hosts geloescht werden
  if (distr == LOCAL) {
    for (i = 0; i < namelist.Size(); ++i)
      delete namelist[i];
    namelist.Clear();
  }


  // Wenn kein Slavehost definiert wurde, dann wenigstens ein lokaler Slave
  if ((SlaveList.Size() == 0) && (namelist.Size() == 0)) {
    cout << "Ich initialisiere wenigstens einen lokalen Slave...\n";
    SlaveList.Append(new Accomplice());
  }

  // Die verteilten Slaves muessen auch erzeugt werden
  for (i = 0; (i < namelist.Size()) && (SlaveList.Size() < totalprocs); ++i)
    SlaveList.Append(new Accomplice(namelist[i]));


  // Falls mehrere Prozesse pro Host, muessen die Slaves vermehrt werden
  value = SlaveList.Size();
  while ((--maxprocs) && (SlaveList.Size() < totalprocs))
    for (i = 0; i < value; ++i) {
      if (SlaveList.Size() >= totalprocs) break;
      SlaveList.Append(new Accomplice(SlaveList[i]->GetHostName()));
    }
}


void Master::MainLoop() {

  int            i, semaphore, old_semaphore = 0;
  time_t         now, start_time, last_call = 0;
  struct timeval timeout;              // Fuer select (maximal 30 Sekunden)
  double         tsum;                 // Summe der einzelnen Modellzeiten
  bool           give_up = false;      // wird gesetzt, wenn der 30 sek Timeout eintritt. Dann beendet MainLoop
  bool           no_one_cares = false; // Prueft, ob ueberhaupt noch ein Slave Daten liesst oder 
                                       // schon alle tot sind
  bool           minfire = true;       // prueft, ob alle Slaves schon oft genug geschaltet haben

  cout << "Aktuelle Modellzeit : \n" << flush;
  start_time = time(NULL);
  do {
    // Daten empfangen und pruefen, ob ueberhaupt noch ein Slave lebt
    timeout.tv_sec = 60L;
    timeout.tv_usec = 0L;
    if (!select (FD_SETSIZE, &G_readset, NULL, NULL, &timeout))
      // Der Timeout trat ein.
      give_up = true;
    else {
      no_one_cares = true;
      minfire = true;
      for (i = 0; i < SlaveList.Size(); ++i) {
	if (SlaveList[i]->GetStatus() == RECEIVING) {
          minfire = minfire && SlaveList[i]->GetMinFireFlag();
	  no_one_cares = false;
	  SlaveList[i]->ReadData(i);  // Hier wird der Slave auch in den readset wieder eingesetzt
	}
      }
    }

    // Measures berechnen und pruefen, wieviele schon fertig sind
    semaphore = 0;
    for (i = 0; i < PrimList.Size(); ++i) 
      if (PrimList[i]->GetStatus() == COMPLETE)
	semaphore++;

    if (semaphore != old_semaphore) {
      cout << semaphore << " von " << PrimList.Size() << " Primitiven berechnet.\n\n" << flush; 
      old_semaphore = semaphore;
    }

  // Real-Time und Modeltime pruefen
  tsum = 0.0;
  for (i = 0; i < SlaveList.Size(); ++i)
    tsum += SlaveList[i]->GetCurrentTime();
  if (tsum > G_maxmodeltime)
    my_exit(MODEL_TIME_EXCEEDED);

  if (time(&now)-start_time > G_maxrealtime)
    my_exit(REAL_TIME_EXCEEDED);

  if (now - last_call > 3.0) {
    last_call = now;
    cout << "!deletelastline\n";
    cout << "Aktuelle Modellzeit : " << setw(10) << tsum << endl << flush;
    }

  } while ((semaphore < PrimList.Size() || !minfire) && !no_one_cares && !give_up);

  if (minfire && (semaphore >= PrimList.Size()))
    cout << "Alle Masse fertig berechnet.\n";
  else 
    if (no_one_cares)
      cout << "Es stehen keine Slaves mehr zur Verfuegung.\n";
    else
      if (give_up)
        cout << "Es wurden 60 sec lang keine Daten empfangen.\n";
      else
        cout << "Fataler Fehler in Master::MailLoop()\n";

  PrintResults();
}


void Master::PrintResults() {

  int i;

  cout << "\nDas Ergebnis der Simulation\n\nParameter:";
  cout << "\n\tNetzname                    : " << G_netname;
  cout << "\n\tConfidence Level            : " << P_conflevel;
  cout << "\n\tmax Relative Fehler         : " << G_maxeps;
  cout << "\n\tMax #Stichproben je Slave   : " << P_maxsamples;
  if (G_trans_detection)
    cout << "\n\tErkennung der transienten Phase";
  else
    cout << "\n\tKeine Erkennung der transienten Phase";

  cout << "\n\nErgebnisse:\n\n";
  cout << "Measure                  Mean   Confidence-Int.   rel. error\n";

  for (i = 0; i < StatList.Size(); ++i)
    StatList[i]->Result();

  if (M_Measure::IsVolatile()) {
    cout << "* Achtung: Leistungsmasse mit mehreren Primitiven koennen sehr\n";
    cout << "           viel ungenauer sein als die einzelnen Primitiven!\n";
  }

  double tsum = 0.0;
  for (i = 0; i < SlaveList.Size(); ++i)
    tsum += SlaveList[i]->GetCurrentTime();

  cout << "\nSumme der Simulationsdauer aller Slaves: ";
  cout << setprecision(4) << tsum << "\nSimulation beendet.\n";

}


bool Master::StillWaiting() {

  for (int i = 0; i < SlaveList.Size(); ++i)
    if (SlaveList[i]->GetStatus() == WAIT_FOR_CONNECT)
      return true;

  return false;
}


void Master::InitSlaves() {

  int new_socket, i, select_return, slave_counter = 0;
  bool give_up = false;
  int length = sizeof(struct sockaddr_in);
  struct sockaddr_in address;
  struct timeval timeout;
  fd_set  readset;

  while ((!give_up) && StillWaiting()) {
    timeout.tv_sec = 120L; // max. 60 sec Wartezeit. Parsen auf vollen Maschinen dauert lange
    timeout.tv_usec = 0L;
    FD_ZERO (&readset);
    FD_SET (Server, &readset);
    do {
      select_return = select (FD_SETSIZE, &readset, NULL, NULL, &timeout);
    } while (select_return < 0 && errno == EINTR);
    if (select_return < 0) {
      perror ("Fehler beim Select.");
      my_exit(-1);
    }
    if (select_return == 0) {
      // Der Timeout trat ein.
      cout << "TimeOut: ";
      give_up = true;
    }

    else {  // select_return > 0
      new_socket = accept(Server, (struct sockaddr *) &address, &length);
      if (new_socket < 0) {
	perror ("Fehler beim Connect des Clients.");
	my_exit (-1);
      }
      if (new_socket > 0) {
	if (length != sizeof(address)) {
	  perror ("Connect liefert falsche Adressinformation.");
	  my_exit (-1);
	}
	// Ab hier haben wir endlich einen gueltigen CONNECT geschafft
	// "Wem sein Client isses denn?"
        slave_counter++;
	i = 0;
	while((i < SlaveList.Size()) && 
	      !(SlaveList[i]->CheckIfMySlave(address.sin_addr.s_addr, new_socket))) 
	  i++;
	if (i == SlaveList.Size()) {      // Ein Fremder? Da laeuft was schief!
	  perror ("Ein Fremder Prozess versucht einen CONNECT.");
	  my_exit (-1);
	}
      }
    }
  }

  // Alle, die sich bis jetzt nicht zurueckgemeldet haben,
  // koennen bleiben, wo der Pfeffer waechst.
  for (i = 0; i < SlaveList.Size(); ++i)
    if (SlaveList[i]->GetStatus() == WAIT_FOR_CONNECT)
      SlaveList[i]->Kill();

  if (slave_counter == 0) {
    cout << "Es steht leider kein Slave zur Verfuegung. Simulation abgebrochen.\n";
    my_exit (-1);
  }

  if (slave_counter != SlaveList.Size())
    cout << "Von " << SlaveList.Size() << " Slaves beginnen " << slave_counter << " mit der Simulation.\n";

  // Alle Verbindungen erstellt. Den Server-Socket machen wir nun dicht.
  close(Server);
}



void Master::InitSocket () {
  
  struct sockaddr_in address;
  int                length  = sizeof (struct sockaddr_in);


  if ((Server = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
    perror ("Kann Server-Socket nicht erstellen.");
    my_exit (-1);
  }

  address.sin_family      = AF_INET;    // Socket quer durchs Internet
  address.sin_addr.s_addr = INADDR_ANY; // Jeder darf hier ran (erstmal)
  address.sin_port        = 0;          // Portnummer bitte vom System generieren lassen

  if (bind (Server, (struct sockaddr *) &address, length) < 0) {
    perror ("Kann Server-Socket nicht an Adresse binden.");
    my_exit (-1);
  }
  
  getsockname (Server,(struct sockaddr *) &address, &length);
  if (length != sizeof (address)) {
    perror ("Kann die Port-Nummer nicht ermitteln.");
    my_exit (1);
  }
    sprintf (P_portnumber, "%d", ntohs (address.sin_port));

  if (listen (Server, 5)) {
    perror ("Systemcall 'listen()' hat versagt.");
    my_exit (-1);
  }

}


void handle_dead_child (int sig) {

//  int deadpid;
    int i;
    bool all_dead = true;

  // Unter Linux wird der Default-Handler wieder installiert.
  // Das muessen wir verhindern.
  signal (sig, handle_dead_child);

  int status;

  for (i = 0; i < SlaveList.Size(); ++i)
    if (waitpid(SlaveList[i]->pid, &status, WNOHANG) == SlaveList[i]->pid) {
      cout << "Der Slave auf " << HOSTNAME(SlaveList[i]->hostname) << " ist gestorben.\n" << flush;
      SlaveList[i]->status = DEAD;
    }

  // Ist denn ueberhaupt noch ein Slave am Leben?
  for (i = 0; i < SlaveList.Size(); ++i)
    if (SlaveList[i]->status != DEAD)
      all_dead = false;

  if (all_dead)
    my_exit(-1);
}


void current_result (int) {
// Ausgabe eines Zwischenergebnisses bei SIGUSR1

  int i;

  cout << "-----------------------\nZwischenergebnis\n";
  for (i = 0; i < StatList.Size(); ++i)
    StatList[i]->Result();
  cout << "-----------------------\n";
}


void my_exit(int nr) {

if (nr) {  // Ausser bei normalem Ende 
    master->PrintResults();

    cout << "Ursache des Abbruchs: ";
    switch (nr) {
      case SIGINT:  cout << "SIGINT";  break;
      case SIGHUP:  cout << "SIGHUP";  break;
      case SIGQUIT: cout << "SIGQUIT"; break;
      case SIGILL:  cout << "SIGILL";  break;
      case SIGABRT: cout << "SIGABRT"; break;
      case SIGFPE:  cout << "SIGFPE";  break;
      case SIGBUS:  cout << "SIGBUS  Tut mir leid. (mb)";  break;
      case SIGSEGV: cout << "SIGSEGV  Tut mir leid. (mb)"; break;
#ifndef LINUX
      case SIGSYS:  cout << "SIGSYS";  break;
#endif
      case SIGPIPE: cout << "SIGPIPE"; break;
      case REAL_TIME_EXCEEDED: cout << "Maximale Echtzeit ueberschritten"; nr = 0; break;
      case MODEL_TIME_EXCEEDED: cout << "Maximale Modellzeit ueberschritten"; nr = 0; break;
      default:      cout << "Exit-Code " << nr;
    }
  }
  else 
    cout << "Ordentliches Ende.";

  cout << "\nDie Simulatoren werden beendet...\n";
  for (int i = 0; i < SlaveList.Size(); ++i)
    if (SlaveList[i]->GetStatus() != DEAD)
      SlaveList[i]->Kill();

  if (nr != 0)
    cout << "Die Simulation wurde abgebrochen.\n";

  close (G_slave_out_fd);
  char buffer[1000];
  ifstream errorfile(G_slaveoutfile);
  while(!errorfile.eof()) {
    errorfile.getline(buffer, 1000);
    cout << buffer;
    if (!errorfile.fail())
       cout << endl;
  }
  errorfile.close();
  exit(nr);
}



void ShowUsage();
void CheckOptions (int, char **);

int main (int argc, char **argv) {

  // gethostbyname zerschiesst unter SunOS4 den Speicher. Da die
  // struct hostent static ist, genuegt ein Aufruf vorher,
  // um den Speicheruebergriff zu verhindern
  gethostbyname("dengibtsnich");


  char *tnethome;

  //
  // Defaultwerte fuer die Parameter setzen
  //
  P_portnumber[0] = '\0';
  sprintf (P_conflevel, "%d", 99);
  G_conflevel = 0.99;
  sprintf (P_maxsamples, "%d", MAXINT);
  sprintf (P_min_firings, "%d", 30);
  G_trans_detection = true;
  G_netname = "default";
  G_maxeps = 0.1;
  rnd.generator(new ACG(0));
  G_netname = "default";
  G_maxmodeltime = MAXDOUBLE;
  G_maxrealtime = MAXINT;

//  P_localhost = new char[MAXHOSTNAME+1];
//  if (gethostname(P_localhost, MAXHOSTNAME)) {
  if ((P_localhost = getenv("HOST")) == NULL) {
    perror ("Fehler bei gethostname().");
    my_exit (-1);
  }
  
  tnethome = getenv("TNETHOME");
  if (!tnethome) {
    perror ("Environment-Variable TNETHOME ist nicht gesetzt.");
    my_exit (-1);
  }
  G_slavecommand = new char[strlen(tnethome) + strlen(SLAVEPFAD)+1];
  strcpy (G_slavecommand, tnethome);
  strcat (G_slavecommand, SLAVEPFAD);


  //
  // Signalhandler installieren
  //
  if (((long)signal (SIGCHLD, handle_dead_child) < 0) ||
      ((long)signal (SIGUSR1, current_result)    < 0) ||
      ((long)signal (SIGINT,  my_exit)           < 0) ||
      ((long)signal (SIGHUP,  my_exit)           < 0) ||
      ((long)signal (SIGQUIT, my_exit)           < 0) ||
      ((long)signal (SIGILL,  my_exit)           < 0) ||
      ((long)signal (SIGABRT, my_exit)           < 0) ||
      ((long)signal (SIGFPE,  my_exit)           < 0) ||
      ((long)signal (SIGBUS,  my_exit)           < 0) ||
      ((long)signal (SIGSEGV, my_exit)           < 0) ||
#ifndef LINUX
      ((long)signal (SIGSYS,  my_exit)           < 0) ||
#endif
      ((long)signal (SIGPIPE, my_exit)           < 0)) {
    perror ("Konnte Signal-Handler nicht installieren.");
    my_exit(-1);
  }

  FD_ZERO (&G_readset);

  //
  // Die Commandline-Optionslesen
  //
  CheckOptions(argc, argv);


  //
  // Ausgabedatei fuer die Slaves bereitstellen
  //
  char *modeldir = getenv("MODELDIR");
  if (!modeldir) {
    perror ("Environment-Variable MODELDIR ist nicht gesetzt.");
    my_exit (-1);
  }
  G_slaveoutfile = new char [strlen(modeldir) + strlen(SLAVEOUTFILE)+1];
  strcpy (G_slaveoutfile, modeldir);
  strcat (G_slaveoutfile, SLAVEOUTFILE);

  if ((G_slave_out_fd = open(G_slaveoutfile, O_WRONLY | O_CREAT | O_TRUNC, OPENFLAGS)) < 0) {
    perror ("Kann die Ausgabedatei fuer Slaves nicht oeffnen.");
    my_exit (-1);
  }
  write(G_slave_out_fd, ERROR_FILE_INIT, strlen(ERROR_FILE_INIT));


  //
  // Das Tablefile pruefen und oeffnen
  // (benutzt char *modeldir)
  //

  int temp_fd;

  if (G_netname[0] != '/') {
    G_netfile = new char[strlen(modeldir)+strlen(G_netname)+8];
    sprintf(G_netfile, "%s/%s.table", modeldir, G_netname);
  }
  else {
    G_netfile = new char[strlen(G_netname)+7];
    sprintf (G_netfile, "%s.table", G_netname);
  }

  //
  // Lesezugriff einmal pruefen, damit die Slaves nicht alle meckern
  //
  if ((temp_fd = open(G_netfile, O_RDONLY)) < 0) {
    perror ("Kann die Netzdatei nicht oeffnen.");
    my_exit (-1);
  }
  close(temp_fd);


  //
  // Und der Master kann beginnen. Alles initialisiert
  //
  master  = new Master();

  // Das war's. Ende und Aus.
  my_exit(0);

  return (1); // to make the compiler happy ;-)
}



void CheckOptions(int argc, char** argv) {

  GetOpt getopt (argc, argv, "s:n:f:m:c:e:x:r:t");
  int option_char;
          
  // Invokes member function `int operator ()(void);'
  while ((option_char = getopt ()) != EOF)
    switch (option_char)
      {  
      case 's': rnd.generator(new ACG(atoi (getopt.optarg))); break;
      case 'n': G_netname = getopt.optarg; break;
      case 'e': G_maxeps = atoi(getopt.optarg)/100.0; break;
      case 'm': strcpy (P_maxsamples, getopt.optarg); break;
      case 'f': strcpy (P_min_firings, getopt.optarg); break;
      case 't': G_trans_detection = false; break;
      case 'c': strcpy (P_conflevel, getopt.optarg); G_conflevel = atoi(P_conflevel)/100.0; break;
      case 'r': G_maxrealtime = atoi(getopt.optarg); if (!G_maxrealtime) G_maxrealtime = MAXINT; break;
      case 'x': G_maxmodeltime = atof(getopt.optarg); if (!G_maxmodeltime) G_maxmodeltime = MAXDOUBLE; break;
      case '?': ShowUsage(); break;
      }
  if (getopt.optind < argc) {
    cout << "Unbekannte Option: " << argv[getopt.optind] << '\n';
    exit (-1);
  }
}


void ShowUsage()
{
  cout <<"\n";
  cout <<"usage : master [-s <seed>] [-n <Table Name>] [-m <Max Samples>]\n";
  cout <<" [-f <min_firings>] [-c <Confidence-Level>] [-t]\n\n";
  cout <<"-s <seed> sets the seed of the random number generator to <seed> (default = 0)\n";
  cout <<"-n <Table Name> reads the net from file '<Table Name>.TABLE' (default = 'default.TABLE'\n";
  cout <<"-m <Max Samples> sets the maximum number of samples enerated (default = 0 = without limit\n";
  cout <<"-f <min_firings> keeps simulating until every binding has fired at least <min_firing> times (30)\n";
  cout <<"-c <Confidence Level> sets the reliability of the variance estimation (default = 95  (%))\n";
  cout <<"-t switches automatic detection of initial transient phase OFF. (default = ON)\n";
  cout <<"-r <Max Realtime> sets the maximum time (in sec.), the simulation should last.\n";
  cout <<"-x <Max Modeltime> sets the time limit for modeltime\n";
  exit(-1);
}


