#define FREIHEIT 7      // Zahl der Freiheitsgrade fuer die Student-t-Distribution
#define SAFECOEFF 1.5   // Faktor zur Erhoehung des Schaetzzeitpunkts

#include <iostream.h>
#include <iomanip.h>
#include "portab.h"
#include "globals.h"
#include "Statistic.h"

extern "C" {
  double student_distr(int free, double level);
  double SpectralVarAnalysis (double testsamplesarray[], int arraylen);

// Cut-and-Paste aus TimeNET::simu_service.c
// War einfacher, als das ganze simu_service.o einzubinden
// Ausserdem sind kleine Anpassungen (cout, my_exit())

  #include <malloc.h>

  char *er_malloc (int bytes) {
    char *pointer;
    if (!bytes)
      return (NULL);
    pointer = new char[bytes];
    //  pointer = (char *)malloc(bytes);
    if (pointer == NULL)
      {
	cout << "Not enough memory for " << bytes << " bytes\n";
	my_exit(-1);
      }
    return pointer;
  }
}

Statistic::Statistic() {

  Status = DATA_TO_PROCESS;
  Epsilon = 1.0 - G_maxeps;
  SampleCount = 0;
  // Sum wird auf (0.0,0.0) im Konstruktor festgelegt
  BatchSize = 1;
  BatchIndex = -1;
  CheckNumber = 0;
  CheckLocNext = 2*M0;
  // MeanValue = 0.0; wird vom Konstruktor des Samples automatisch gemacht
  TotalTransientTime = 0.0;
  for (int i = 0; i < SlaveList.Size(); ++i)
    Times.Append(0.0);
}

void Statistic::Calculate() {
  // Wird aufgerufen, wenn der naechste Schaetzzeitpunkt erreicht ist
#ifdef _DEBUG
  if (CheckLocNext > SampleCount) {
    cout << "Verstoss gegen die Vorbedingung in Statistic::Calculate()\n";
    my_exit(-1);
  }
#endif

  double variance, t_na;

  MeanValue.value = 0.0;
  MeanValue.time  = 0.0;

  for(int s = 0; s <= BatchIndex; s++)
    MeanValue = MeanValue + SampleSequence[s];

  // Aufruf von Sequence-1 fuer die SpecVarAnalysis, da die Funktion erst
  // ab Index 1 bis Index n die Daten erwartet. Sequence laeuft aber von
  // 0 bis n-1
  variance   = SpectralVarAnalysis(Sequence-1, BatchIndex+1);
  t_na       = student_distr(FREIHEIT, 0.5 + (G_conflevel/2));
  DeltaX     = t_na * sqrt(variance);
  Epsilon    = DeltaX / double(MeanValue);

  if(Epsilon < G_maxeps) {    // G_maxeps ist global.
    Status = COMPLETE;
  }
  CheckNumber++;
  CheckLocNext = int(CheckLocNext * SAFECOEFF);
}


StatisticStatus Statistic::GetStatus() {

  return Status;
}


double Statistic::GetEpsilon() {

  return Epsilon;
}


double Statistic::GetMean() {

  // Die Samples, die noch im aktuellen, angefangenen Batch sind,
  // muessen auch noch mit in den Mean, da sie ja auch in der total_time
  // erzielt wurden.
  MeanValue.value = 0.0;
  MeanValue.time  = 0.0;
  for(int s = 0; s <= BatchIndex; s++)
    MeanValue = MeanValue + SampleSequence[s];
  MeanValue = MeanValue + Sum;

  return (double(MeanValue));

}


void Statistic::AddTransientTime (double time) {

  TotalTransientTime += time;
}


void Statistic::Add2Batch() {

  BatchIndex++;
  SampleSequence[BatchIndex] = Sum;
  Sequence[BatchIndex] = double(Sum);
  Sum.value = 0.0;
  Sum.time  = 0.0;

  if (BatchIndex == 2*M0 - 1) {
    // Die Sequence ist voll. Wir fassen paarweise neu zusammen
    for (int i = 0; i < M0; ++i) {
      SampleSequence[i] = SampleSequence[2*i] + SampleSequence[2*i+1];
      Sequence[i] = double(SampleSequence[i]);
    }
    BatchIndex = M0-1;
    BatchSize *= 2;
  }
}


void Statistic::AddSamples(Sample *sample, int count, int slave, double time) {

  Times[slave] = time;

  for (int i = 0; i < count; ++i) {
    Sum = Sum + sample[i];
    SampleCount++;
    if (SampleCount % BatchSize == 0)
      // Ein Stapel ist voll
      Add2Batch();
  }

// Diese Zeile ist interessant fuer eine transiente Simulation
// Daten filtern ueber grep "#" und dann plotten mit gnuplot
// cout << time << "\t" << GetMean() << " #\n";

#ifdef _DEBUG
  // Mal wieder eine Invariante testen...
  if ((BatchIndex+1) * BatchSize + (SampleCount%BatchSize) != SampleCount) {
    cout << "Verletzung der Invarianten in Statistic::AddSamples\n";
    my_exit(-1);
  }

  StatisticStatus old_status = Status;
#endif
 
  if (SampleCount >= CheckLocNext) {
    // Wir muessen mal wieder die Varianz pruefen
    Calculate();
}
#ifdef _DEBUG
    if ((old_status == COMPLETE) && (Status != COMPLETE)) {
      cout << "Eine Primitive war schon komplett berechnet\n";
      cout << "und ist nun wieder versaut worden!!!\n";
    }
#endif

}

   











