#include <iostream.h>
#include "ComfVector.h"

#define MIN(a,b)  ((a)<(b)?(a):(b))

template <class T>
ComfVector<T>::ComfVector(int initialCapacity, int capacityIncrement) {
  capacity_increment = capacityIncrement;
  element_count = 0;
  capacity = initialCapacity;
  element_data = new T[capacity];
#ifdef _VERBOSE
  print("KONSTRUKTOR2: ");
#endif
}


template <class T>
ComfVector<T>::ComfVector(int initialCapacity) {

  capacity_increment = 10; //Default-Wert
  element_count = 0;
  capacity = initialCapacity;
  element_data = new T[capacity];

#ifdef _VERBOSE
  print("KONSTRUKTOR1: ");
#endif
}

template <class T>
ComfVector<T>::ComfVector() {

  capacity_increment = 10; //Default-Wert
  element_count = 0;
  capacity = 10;           //Default-Wert
  element_data = new T[capacity];

#ifdef _VERBOSE
  print("KONSTRUKTOR0: ");
#endif
}

template <class T>
ComfVector<T>::~ComfVector() {
  
#ifdef _VERBOSE
  print("DELETE: ");
#endif
  delete element_data;
}


template <class T>
void ComfVector<T>::Append(T anArray[], int length) {

  // Reicht der vorhandene Speicher aus? Falls nein, neu alloziieren
  if (capacity - element_count < length)
    Resize(capacity+length);

  // Nun koennen wir einfach die Elemente kopieren.
  for (int i = 0; i < length; ++i)
    element_data[element_count++] = anArray[i];

#ifdef _VERBOSE
  print("APPEND[]");
#endif
}


template <class T>
void ComfVector<T>::Append(const ComfVector<T>& l) {

  //Reicht der vorhandene Speicher aus? Falls nein, neu alloziieren
  if (capacity - element_count < l.element_count)
    Resize (capacity+l.element_count);

  // Nun koennen wir beruhigt kopieren
  for (int i = 0; i < l.element_count; ++i)
    element_data[element_count++] = l.element_data[i];

#ifdef _VERBOSE
  print("APPEND<T>: ");
#endif
}


template <class T>
void ComfVector<T>::Append(T element) {

  // Reicht der Speicher nicht, den Array um capavity_increment verlaengern
  if (capacity-element_count == 0)
    Resize(capacity+capacity_increment);

  element_data[element_count++] = element;

#ifdef _VERBOSE
  print("APPEND T :");
#endif
}


template <class T>
void ComfVector<T>::CutDown(int size) {
#ifdef _VERBOSE
  print ("CutDown :");
#endif

  if (element_count < size) {
    cout << "ComfVector.CutDown: size > element_count! Sehr unlogisch.\n";
    exit (-1);
  }
  element_count = size;
}


template <class T>
void ComfVector<T>::Resize(int size) {

  T *new_array = new T[size];
  T *np = new_array;
  T *op = element_data;

  capacity = size;
  element_count = MIN(size, element_count);
  
  for (int i = 0; i < element_count; ++i)
    *(np++) = *(op++);

  delete element_data;
  element_data = new_array;

#ifdef _VERBOSE
  print("RESIZE :");
#endif
}


template <class T>
void ComfVector<T>::Clear() {
  element_count = 0;

#ifdef _VERBOSE
  print("CLEAR:");
#endif
}


template <class T>
int ComfVector<T>::Capacity() {
  return capacity;
}

template <class T>
int ComfVector<T>::Size() {
  return element_count;
}


template <class T>
int ComfVector<T>::IsEmpty(){
  return (element_count == 0);
}


template <class T>
T& ComfVector<T>::operator[] (int index) {
  if ((index > element_count-1) || (index < 0)) {
    cerr << "Class ComfVector: Index out of Bounds: " << index;
    print ("  Zum Debuggen:\n");
    exit(-1);
  }
  return element_data[index];
}

template <class T>
void ComfVector<T>::print (char *t) {
  cout << t;
  cout <<   "Class ComfVector: " << element_count << " of " << capacity << " Elements used.";
  cout << "\n                  Capacity Increment is " << capacity_increment << "\n";
  cout <<   "Elements are    : ";

  T *p = element_data;
  for (int i = 0; i < element_count; ++i)
    cout << "                  " << *(p++) << "\n";
}


double ComfVector<double>::Fold(VOperator operation) {

  if (element_count < 1) {
    cerr << "Class ComfVector: Faltung ohne Elemente geht nicht.";
    exit(-1);
  }

  double result = *element_data;
  double *p = element_data+1;
  int    index  = 0, i;

  switch (operation) {
  case VECTOR_PLUS:
    for (i = 1; i < element_count; ++i)
      result = result + *(p++);
    break;
  case VECTOR_ARGMIN:
    for (i = 1; i < element_count; ++i)
      if (element_data[index] > element_data[i])
	index = i;
    result = (double)index;
    break;
  default:
    cerr << "Class ComfVector: Operation " << operation << " nicht definiert.\n";
    exit(-1);
  }

#ifdef _VERBOSE
  print("FOLD:");
#endif
  return result;
}


template <class T>
void ComfVector<T>::Replace (int index, T element) {

  if ((index > element_count-1) || (index < 0)) {
    cerr << "Class ComfVector::Replace : Index out of Bounds: " << index;
    print ("  Zum Debuggen:\n");
    exit(-1);
  }
  element_data[index] = element;
}



// Generierung der Templates
class BindingElement;
class Place;
class Transition;
class M_Measure;
class S_Measure;
class Prim;

template class ComfVector<double>;
template class ComfVector<int>;
template class ComfVector<BindingElement *>;
template class ComfVector<Place *>;
template class ComfVector<Transition *>;
template class ComfVector<M_Measure *>;
template class ComfVector<S_Measure *>;
template class ComfVector<Prim *>;

// die folgenden beiden sind fuer den Master
class Statistic;
class Accomplice;

template class ComfVector <Statistic *>;
template class ComfVector <Accomplice *>;

// die folgenden sind fuer Master und Slave wichtig.
template class ComfVector <char *>;




