#include <iostream.h>
#include <string.h>

#include "Measure.h"


Measure::Measure(char *n, double *e) {

  int size, i;

  status = CONSTRUCTING;
  name = n;
  size = int(e[0]);
  if (double(size) != e[0]) {
    cout << "Fliesskommazahl als Arraylaenge in Measure::Measure\n";
    exit(-1);
  }
  for (i = 1; i <= size; ++i)
    Append(e[i]);

  Parse();

}


Measure::Measure(char *n) {

  name = n;
  status = CONSTRUCTING;

}


void Measure::Append(double x) {

  expression.Append(x);
}


double *Measure::Export() {

#ifdef _DEBUG
  if (status != PARSED) {
    cout << "Measure::Export vor kompletter Konstruktion.\n";
    exit(-1);
  }
#endif

  double *p, *a = new double[expression.Size()+1];

  *a = expression.Size();
  p = a + 1;
  for (int i = 0; i < expression.Size(); ++i)
    *(p++) = expression[i];

  return a;
}


void Measure::Parse() {

#ifdef _DEBUG
  // Der hier gepruefte Fall waere zwar nicht schlimm, ist aber unerwartet.
  // Daher erfolgt die Pruefung.
  if (status != CONSTRUCTING) {
    cout << "Merhfacher Aufruf von Parse() in Klasse Measure\n";
    exit(-1);
  }
#endif

  if (expression.Size() != Check(0)) {
    cout << "Die Laenge des Double-Arrays stimmt nicht in Measure::Parse\n";
    exit(-1);
  }
  status = PARSED;
}


int Measure::Check(int index) {

  if (index >= expression.Size()) {
    cout << "Parserfehler in Measure. Index out of Bounds.\n";
    exit(-1);
  }

  if (double(int(expression[index])) != expression[index]){
    cout << "Parserfehler in Measure: Fliesskommazahl als Schluesselwort\n";
    exit(-1);
  }

  switch (int(expression[index])) {
  case M_CONST:       return (index+2); break;
  case M_UNAER_MINUS: return (Check(index+1)); break;
  case M_UNAER_NOT:   return (Check(index+1)); break;
  case M_PLUS:        return (Check(Check(index+1))); break;
  case M_MINUS:       return (Check(Check(index+1))); break;
  case M_TIMES:       return (Check(Check(index+1))); break;
  case M_DIV:         return (Check(Check(index+1))); break;
  case M_EQUAL:       return (Check(Check(index+1))); break;
  case M_GREATER:     return (Check(Check(index+1))); break;
  case M_LOWER:       return (Check(Check(index+1))); break;
  case M_NOTEQUAL:    return (Check(Check(index+1))); break;
  case M_GREATEQ:     return (Check(Check(index+1))); break;
  case M_LOWEQ:       return (Check(Check(index+1))); break;
  case M_AND:         return (Check(Check(index+1))); break;
  case M_OR:          return (Check(Check(index+1))); break;
  case M_PRIMITIVE:   
    if (expression[index+1] != double(int(expression[index+1]))) {
      cout << "Fliesskommazahl als Prim-Index.\n";
      exit(-1);
    }
    prims.Append(int(expression[index+1]));
    return (index+2); 
    break;
  default: 
    cout << "Parserfehler in Measure. Unbekannter Schluessel ";
    cout << expression[index] << endl;
    exit(-1);
    break;
  }

  cout << "Fataler Fehler in Measure::Check()\n";
  exit(-1);

  return (0);
}


bool Measure::operator == (Measure &m) {

  if (strcmp(name, m.name) != 0)
    return false;

  if (expression.Size() != m.expression.Size())
    return false;

  for (int i = 0; i < expression.Size(); ++i)
    if (expression[i] != m.expression[i])
      return false;

  return true;
}


double Measure::Calc() {

  double val = 0.0;

  if(expression.Size() > 0)
    ParseCalc(0, val);

  return val;
}


int Measure::ParseCalc(int index, double &val) {

  double a1, a2;
  int n = 0;

  if (index >= expression.Size()) {
    cout << "Parserfehler in Measure. Index out of Bounds.\n";
    exit(-1);
  }

  if (double(int(expression[index])) != expression[index]){
    cout << "Parserfehler in Measure: Fliesskommazahl als Schluesselwort\n";
    exit(-1);
  }

  switch (int(expression[index])) {
  case M_CONST:       
    val = expression[index+1]; 
    return (index+2); 
    break;
  case M_UNAER_MINUS: 
    n = ParseCalc(index+1, a1);
    val = -a1;
    return (n); 
    break;
  case M_UNAER_NOT:   
    n = ParseCalc(index+1, a1);
    val = !a1;
    return (n); 
    break;
  case M_PLUS:        
    n = ParseCalc(ParseCalc(index+1,a1),a2);
    val = a1 + a2;
    return (n);
    break;
  case M_MINUS:
    n = ParseCalc(ParseCalc(index+1,a1),a2);
    val = a1 - a2;
    return (n);
    break;
  case M_TIMES:
    n = ParseCalc(ParseCalc(index+1,a1),a2);
    val = a1 * a2;
    return (n);
    break;
  case M_DIV:
    n = ParseCalc(ParseCalc(index+1,a1),a2);
    val = a1 / a2;
    return (n);
    break;
  case M_EQUAL:
    n = ParseCalc(ParseCalc(index+1,a1),a2);
    val = (a1 == a2);
    return (n);
    break;
  case M_GREATER:
    n = ParseCalc(ParseCalc(index+1,a1),a2);
    val = (a1 > a2);
    return (n);
    break;
  case M_LOWER: 
    n = ParseCalc(ParseCalc(index+1,a1),a2);
    val = (a1 < a2);
    return (n);
    break;
  case M_NOTEQUAL:
    n = ParseCalc(ParseCalc(index+1,a1),a2);
    val = (a1 != a2);
    return (n);
    break;
  case M_GREATEQ:
    n = ParseCalc(ParseCalc(index+1,a1),a2);
    val = (a1 >= a2);
    return (n);
    break;
  case M_LOWEQ:
    n = ParseCalc(ParseCalc(index+1,a1),a2);
    val = (a1 <= a2);
    return (n);
    break;
  case M_AND:
    n = ParseCalc(ParseCalc(index+1,a1),a2);
    val = (a1 && a2);
    return (n);
    break;
  case M_OR:
    n = ParseCalc(ParseCalc(index+1,a1),a2);
    val = (a1 || a2);
    return (n);
    break;
  case M_PRIMITIVE:
    val = GetPrimVal(int(expression[index+1]));
    return (index+2); 
    break;
  default: 
    cout << "Parserfehler in Measure. Unbekannter Schluessel ";
    cout << expression[index] << endl;
    exit(-1);
    break;
  }

  cout << "Fataler Fehler in Measure::Check()\n";
  exit(-1);

  return (0);
}
