#ifndef __ARCBINDELEMENT_CC__
#define __ARCBINDELEMENT_CC__

#include "Identifier.h"
#include "Textual.h"
#include "TextualList.h"

#include "token_multiset.h"
#include "TokenMultiSet.h"
#include "PlaceHeader.h"

#include "Token.h"

#include "CellVal.h"
#include "Cell.h"

#include "Place.h"

#include "ArcBindElement.h"


//#############################################################################################################
ArcBindElement::ArcBindElement()
{ 
	place   	= NULL; 
	result_tms	= NULL;
	tokenms 	= NULL; 
}
ArcBindElement::ArcBindElement(int)	
{ 
	place   	= NULL; 
	result_tms	= NULL;
	tokenms 	= NULL;
}
ArcBindElement::ArcBindElement(Place* p, Cell* c)  
{ 
	place 		= p; 
	result_tms	= NULL;
	NewCell(c);
}
ArcBindElement::ArcBindElement(const ArcBindElement& a)
{ 
	place 	   	= a.place; 
	result_tms 	= NULL;
	NewCell(a.tokenms);
}
//#############################################################################################################
char*		ArcBindElement::GetName()				{ return DEF_STRING; 		}
Place*		ArcBindElement::GetPlace()				{ return place; 		}
const Place*&	ArcBindElement::GetPlaceRef()				{ return place;			}
bool		ArcBindElement::operator!=(ArcBindElement& a) 		{ return !(*this == a); 	}
bool		ArcBindElement::LessThan(ArcBindElement&) 		{ return false;			}
//------------------------------------------------------------------------------------------------------------
Cell*		ArcBindElement::GetTokenMultiSetCell()			
{ 
	if (tokenms) return tokenms; 		
	else	     return (new ConstVal(new TokenMSVal()));
}
//------------------------------------------------------------------------------------------------------------
void		ArcBindElement::NewCell(Cell* c)
{
	if (!c) tokenms = NULL;
	else
	{
		switch (c->GetResultType())
		{
			case c_function		:	tokenms = c->Copy();	break;
			case e_mset_expr	:	tokenms = c->Copy();	break;
			case p_mset_expr	:	tokenms = c->Copy();	break;
			default			:	tokenms = NULL;		break;
		}
	}
}
//------------------------------------------------------------------------------------------------------------
int	ArcBindElement::GetActColor()
{
	if (result_tms) 
	{
		if (result_tms->tmset && result_tms->tmset->first) 
			return (result_tms->tmset->first->tref);
		return (EMPTY_COLOR);
	}

	CellVal*       cv  = tokenms->Calculate();
	TokenMultiSet* ret = cv->GetTokenMultiSetPtr();

	cv->Clear();
	delete cv;

	if (tokenms->IsPlaceDependent())
	{
		if (ret->tmset && ret->tmset->first) 
			return (ret->tmset->first->tref);
		return (EMPTY_COLOR);
	}
	result_tms = ret;

	if (result_tms->tmset && result_tms->tmset->first) 
		return (result_tms->tmset->first->tref);
	return (EMPTY_COLOR);
}
//------------------------------------------------------------------------------------------------------------
int	ArcBindElement::GetActCapacity()
{
	if (result_tms) return (result_tms->quant);

	CellVal*       cv  = tokenms->Calculate();
	TokenMultiSet* ret = cv->GetTokenMultiSetPtr();

	cv->Clear();
	delete cv;

	if (tokenms->IsPlaceDependent())
		return (ret->quant);

	result_tms = ret;
	return (result_tms->quant);
}
//------------------------------------------------------------------------------------------------------------
TokenMultiSet*	ArcBindElement::GetArcTokenMultiSet()
{
	if (result_tms) return result_tms;

	CellVal*       cv  = tokenms->Calculate();
	TokenMultiSet* ret = cv->GetTokenMultiSetPtr();

	cv->Clear();
	delete cv;

	if (tokenms->IsPlaceDependent())
		return ret;

	result_tms = ret;
	return result_tms;
}
//------------------------------------------------------------------------------------------------------------
void		ArcBindElement::Clear()			
{ 
	if (tokenms)  	{ tokenms->Clear();    delete tokenms;    tokenms    = NULL; }
	if (result_tms) { result_tms->Clear(); delete result_tms; result_tms = NULL; }
}
//------------------------------------------------------------------------------------------------------------
ArcBindElement&	ArcBindElement::operator =(const ArcBindElement& a) 
{
	Clear();
	place = a.place; 
	if (a.result_tms) result_tms = new TokenMultiSet(*(a.result_tms));
	else		  result_tms = NULL;
	if (a.tokenms)    tokenms    = a.tokenms->Copy(); 
	else              tokenms    = NULL; 
	return *this; 
}
//------------------------------------------------------------------------------------------------------------
bool		ArcBindElement::operator==(ArcBindElement& a) 
{ 
	if ( place   && !a.place)				return false;
	if (!place   &&  a.place)				return false;
	if ( tokenms && !a.tokenms)				return false;
	if (!tokenms &&  a.tokenms)				return false;
	if (!tokenms && !a.tokenms && !place && !a.place) 	return true; 
	if (!tokenms && !a.tokenms) 				return ( *(place)   == *(a.place)   );
	if (!place   && !a.place)				return ( *(tokenms) == *(a.tokenms) );
	return ( *(place) == *(a.place) && *(tokenms) == *(a.tokenms) );
}

//################################################################################################################
#endif
