#ifndef __BindingElement_cc__
#define __BindingElement_cc__

#include <iostream.h>

#include "Identifier.h"
#include "Textual.h"
#include "TextualList.h"

#include "token_multiset.h"
#include "TokenMultiSet.h"
#include "PlaceHeader.h"

#include "Token.h"

#include "CellVal.h"
#include "Cell.h"

#include "Place.h"

#include "Delay.h"

#include "ArcBindElement.h"
#include "ArcBindList.h"

#include "BindingElement.h"

// Begin Simulator
#include "ComfVector.h"
// End Simulator

//#######################################################################################################################
BindingElement::BindingElement()    			 
	      : weight		(DEF_ES_WEIGHT), 
	        lc_result	(DEF_COND_RES),
	        del_result	(DEF_DEL_RES),
	      	inarcbind	(), 
	      	outarcbind	(),
                ftl             (0,1),            // Simulator
                bod             (0,1)             // Simulator
	      { 
		delay     = new Immediate();
		localcond = new ConstVal();
		InitVariables   ();               // Simulator
	      }
BindingElement::BindingElement(int) 			 
	      : weight		(DEF_ES_WEIGHT), 
	        lc_result	(DEF_COND_RES),
	        del_result	(DEF_DEL_RES),
	      	inarcbind	(), 
	      	outarcbind	(),
                ftl             (0,1),            // Simulator
                bod             (0,1)             // Simulator
	      { 
		delay     = new Immediate();
		localcond = new ConstVal();
		InitVariables   ();               // Simulator
	      }
BindingElement::BindingElement(const BindingElement& be) 
	      : weight		(be.weight), 
	        lc_result	(be.lc_result),
	        del_result	(be.del_result),
	      	inarcbind	(be.inarcbind), 
	      	outarcbind	(be.outarcbind),
                ftl             (0,1),            // Simulator
                bod             (0,1)             // Simulator
	      { 
		if (!be.delay)     delay     = new Immediate(); else delay     = be.delay->Copy();
		if (!be.localcond) localcond = new ConstVal();  else localcond = be.localcond->Copy();
		InitVariables   ();               // Simulator
	      }

BindingElement::BindingElement(Weight w, const Cell* lc, const Delay* d, const ArcBindList& i, const ArcBindList& o) 
	      : weight		(w) , 
	        lc_result	(DEF_COND_RES),
	        del_result	(DEF_DEL_RES),
	      	inarcbind	(i), 
	      	outarcbind	(o),
                ftl             (0,1),            // Simulator
                bod             (0,1)             // Simulator
	      { 
		if (!d)  delay     = new Immediate(); else delay     = d->Copy();
		if (!lc) localcond = new ConstVal();  else localcond = lc->Copy();
		InitVariables   ();               // Simulator
	      }
BindingElement::BindingElement(Weight w, const Cell* lc, const Delay* d, const ArcBindList* i, const ArcBindList* o) 
	      : weight		(w),
	        lc_result	(DEF_COND_RES),
	        del_result	(DEF_DEL_RES),
                ftl             (0,1),            // Simulator
                bod             (0,1)             // Simulator
	      { 
		if (!d)  delay     = new Immediate(); 	     else  delay      = d->Copy();
		if (!lc) localcond = new ConstVal();  	     else  localcond  = lc->Copy();
		if (i) inarcbind   = *(new ArcBindList(*i)); else  inarcbind  = *(new ArcBindList()); 
		if (o) outarcbind  = *(new ArcBindList(*o)); else  outarcbind = *(new ArcBindList()); 
		InitVariables   ();               // Simulator
	      }

//#######################################################################################################################
char*			BindingElement::GetName()		  		{ return DEF_STRING; 			}
Delay*   		BindingElement::GetDelay()	          		{ return delay; 			}
Weight   		BindingElement::GetWeight()	          		{ return weight; 			}
ArcBindList&		BindingElement::GetInArcBind()	          		{ return inarcbind; 			}
ArcBindList&		BindingElement::GetOutArcBind()		  		{ return outarcbind; 			}
const ArcBindList&	BindingElement::GetInArcBindRef()	  		{ return inarcbind; 			}
const ArcBindList&	BindingElement::GetOutArcBindRef()	  		{ return outarcbind; 			}
void	 		BindingElement::ChangeInArcBind(const ArcBindList& i)   { inarcbind.Clear();  inarcbind  = i; 	}
void	 		BindingElement::ChangeOutArcBind(const ArcBindList& o)  { outarcbind.Clear(); outarcbind = o; 	}
void	 		BindingElement::ChangeWeight(Weight w)			{ weight = w; 				}
bool 	 		BindingElement::LessThan(BindingElement& be)		{ return ( weight < be.weight ); 	}
bool	 		BindingElement::operator!=(BindingElement& be)		{ return !(*this==be); 			} 
//----------------------------------------------------------------------------------------------------------------------
Cell*	 		BindingElement::GetLocalCondition()	  		
{ 
	if (localcond) return localcond; 			
	return new ConstVal(lc_result);
}
//----------------------------------------------------------------------------------------------------------------------
double            	BindingElement::GetDelayVal()         
{ 
	if (del_result != DEF_DEL_RES) 
		return del_result;

	CellVal* cv  = delay->Calculate();
	double	 ret = cv->GetFloat();

	cv->Clear();
	delete cv;

	if (delay->IsPlaceDependent()) 
		return ret;

	del_result = ret;
	return del_result;
}
//----------------------------------------------------------------------------------------------------------------------
bool            	BindingElement::LocalCondTrue()         
{ 
	if (lc_result != DEF_COND_RES) 
		return (lc_result == 0?false:true);

	CellVal* cv  = localcond->Calculate();
	double   val = cv->GetFloat();
	cv->Clear();

	if (localcond->IsPlaceDependent())
	{
		if (!val) return false;
		return true;
	}

	if (!val) lc_result = 0;
	else      lc_result = 1;

	if (localcond)
	{
		localcond->Clear();
		delete localcond;
		localcond = NULL;
	}
	return (lc_result == 0?false:true);
}
//----------------------------------------------------------------------------------------------------------------------
void	BindingElement::Clear()
{
	if (localcond)  { localcond->Clear(); delete localcond;  localcond  = NULL; }
	if (delay)      { delay->Clear();     delete delay;      delay      = NULL; }
	lc_result  = DEF_COND_RES;
	del_result = DEF_DEL_RES;
	inarcbind.Clear(); 
	outarcbind.Clear();
}
//----------------------------------------------------------------------------------------------------------------------
void	 BindingElement::ChangeLocalCondition(const Cell* c)      
{ 
	if (c) 
	{ 
		if (localcond)
		{
			localcond->Clear();
			delete localcond; 
			localcond = NULL;
		}
		localcond = c->Copy(); 
		lc_result = DEF_COND_RES;
	}
}
//----------------------------------------------------------------------------------------------------------------------
void	 BindingElement::ChangeDelay(const Delay* d)              
{ 
	if (d) 
	{ 
		if (delay)
		{
			delay->Clear();
			delete delay; 
			delay = NULL; 
		}
		delay      = d->Copy(); 
		del_result = DEF_DEL_RES;
	} 
}
//----------------------------------------------------------------------------------------------------------------------
bool	 BindingElement::operator==(BindingElement& be) 
{ 
	return (weight    == be.weight    && *delay     == *(be.delay) && *localcond==*(be.localcond) && 
		inarcbind == be.inarcbind && outarcbind == be.outarcbind);
}
//----------------------------------------------------------------------------------------------------------------------
BindingElement& BindingElement::operator =(const BindingElement& be)
{
	Clear();
	weight     = be.weight;
	inarcbind  = be.inarcbind;
	outarcbind = be.outarcbind;
	lc_result  = be.lc_result;
	del_result = be.del_result;
	if (!be.delay)     delay     = new Immediate(); else  delay      = be.delay->Copy();
	if (!be.localcond) localcond = new ConstVal();  else  localcond  = be.localcond->Copy();
	return *this;
}
//#######################################################################################################################

// Begin Simulator

//------------------------------------------------------------------------------------------------
//
//  Ab hier folgen die Ergaenzungen fuer den Simulator
//
//  (kleine Aenderungen sind auch oben im Konstruktor und in der Klassendefinition (*.h)
//
//------------------------------------------------------------------------------------------------


void BindingElement::Go() {
  // Schalten des Bindings OHNE Pruefung der Schaltfaehigkeit
  // Sowohl Global als auch Local Guard muessen stimmen,
  // Eingabe-Marken muessen existieren und die Kapazitaet der
  // Ausgabestellen ist gross genug.

  CellVal *cv;

  firings++;
  if (inarcbind.size) {
    for (ArcBindListElement* iarc=inarcbind.first; iarc; iarc=iarc->next) {
      cv = iarc->lelement.GetTokenMultiSetCell()->Calculate();
      iarc->lelement.GetPlace()->SubTMS(cv->GetTokenMultiSet());
      delete cv;
    }
  }
  if (outarcbind.size) {
    for (ArcBindListElement* oarc=outarcbind.first; oarc; oarc=oarc->next) {
      cv = oarc->lelement.GetTokenMultiSetCell()->Calculate();
      oarc->lelement.GetPlace()->AddTMS(cv->GetTokenMultiSet());
      delete cv;
    }
  }
}


void BindingElement::ISGo() {

#ifdef _DEBUG
  if (servertype != INFINITE) {
    cout << "Verletzung der Vorbedingung in BindingElement::ISGo().\n";
    exit(-1);
  }
  if (grad < 1.0) {
    cout << "IS-Binding mit grad < 1 darf nicht schalten in BindingElement::ISGo().\n";
    exit(-1);
  }
  if (ftl.Size() != grad || bod.Size() != grad) {
    cout << "Liste der Schaltzeiten im IS-Binding hat falsche Laenge in BindingElement::ISGo().\n";
    exit(-1);
  }
#endif

  Go();

  // Bei IS-Bindings muss bei mehrfacher Aktivierung die kuerzeste Schaltzeit aus der Liste
  // heraus. Diese Schaltzeit hat ja gerade geschaltet. Der Grad verringert sich um eins.
  grad--;
  if (grad) {
    ftl.Replace(PV, ftl[grad]);
    bod.Replace(PV, bod[grad]);
  }
  ftl.CutDown(grad);
  bod.CutDown(grad);
}


void BindingElement::Undo() {
  // Schaltet das Binding OHNE Pruefung der Schaltfaehigkeit
  // RUECKWAERTS!

  CellVal *cv;

  firings--;
  if (inarcbind.size) {
    for (ArcBindListElement* iarc=inarcbind.first; iarc; iarc=iarc->next) {
      cv = iarc->lelement.GetTokenMultiSetCell()->Calculate();
      iarc->lelement.GetPlace()->AddTMS(cv->GetTokenMultiSet());
      delete cv;
    }
  }
  if (outarcbind.size) {
    for (ArcBindListElement* oarc=outarcbind.first; oarc; oarc=oarc->next) {
      cv = oarc->lelement.GetTokenMultiSetCell()->Calculate();
      oarc->lelement.GetPlace()->SubTMS(cv->GetTokenMultiSet());
      delete cv;
    }
  }
}



void BindingElement::InitVariables() {
  firing_time = TOO_LONG;
  begin_of_ft = 0.0;
  was_enabled = false;
  influenced = MODIFIED;
  delay_changed = true;
  servertype = NOTYETDETERMINED;
  firings = 0;
  grad = 0;
}



// IsEnabled war mal ArcsOk
// Ist das Binding nicht schaltfaehig, wird die Variable was_enabled = false gesetzt.
// Bei Schaltfaehigkeit wird was_enabled NICHT veraendert. Das Setzen uebernehmen
// [New](E|I)SDelaySample()
bool BindingElement::IsEnabled() {


#ifndef NOOPTIMIZE
  // Wenn kein Einfluss war, dann koennen wir abkuerzen
  if (!influenced)
    return (was_enabled);
#endif

  if (!localcond->CalculateFloat()) {
    was_enabled = false;
    return false;
  }

  if (inarcbind.size)
   {
      for (ArcBindListElement* iarc=inarcbind.first; iarc; iarc=iarc->next) 
	if (!iarc->lelement.GetPlace()->CheckTMS(iarc->lelement.GetTokenMultiSetCell())) {
	  was_enabled = false;
	  return false;
	}
    }
  if (outarcbind.size)
    {
      for (ArcBindListElement* oarc=outarcbind.first; oarc; oarc=oarc->next)
	{
	  bool checked = false;
	  if (inarcbind.size)
	    {
	      for (ArcBindListElement* iarc=inarcbind.first; iarc; iarc=iarc->next)
		{
		  if (iarc->lelement.GetPlace()->GetPlaceNo() == oarc->lelement.GetPlace()->GetPlaceNo())
		    {
		      if (!oarc->lelement.GetPlace()->CheckCapacity(
								    oarc->lelement.GetTokenMultiSetCell(),
								    iarc->lelement.GetTokenMultiSetCell())) {
			was_enabled = false;
			return false;	
		      }
		      checked = true;
		      break;	
		    }
		}
	    }
	  
	  if (checked) continue;
	  
	  if (!oarc->lelement.GetPlace()->TMSConsistence(oarc->lelement.GetTokenMultiSetCell()) || 
	      !oarc->lelement.GetPlace()->CheckCapacity(oarc->lelement.GetTokenMultiSetCell()) ) {
	    was_enabled = false;
	    return false; 
	  }
	}
    }
  if (!outarcbind.size && !inarcbind.size) {
    was_enabled = false;
    return false;
  }

  // Wird beim Schalten die Kapazitaet geaendert, dann muessen alle Kapazitaeten
  // unter der neuen Markierung getestet werden.
  ComfVector<Place *> cccp;

  // Zunaechst Liste aller Places, deren Kapazitaet beim Schalten veraendert wird
  if (inarcbind.size) {
    for (ArcBindListElement* iarc=inarcbind.first; iarc; iarc=iarc->next) 
      iarc->lelement.GetPlace()->GetModifyCapList(cccp);
  }
  if (outarcbind.size) {
    for (ArcBindListElement* oarc=outarcbind.first; oarc; oarc=oarc->next) 
      oarc->lelement.GetPlace()->GetModifyCapList(cccp);
  }

  // Wenn keine Kapazitaeten beeinflusst werden, dann sind wir hier schaltfaehig und fertig
  if (cccp.IsEmpty())
    return true;

  // Nun die Arcs alle schalten und ... 
  this->Go();

  // dann alle Kapazitaeten pruefen und ...
  int maxplace = cccp.Size();
  bool check = true;
  for (int i = 0; i < maxplace; ++i)
    if (!cccp[i]->CheckCapacity()) {
      was_enabled = false;
      check = false;
    }

  // das Schalten wieder rueckgaengig machen. (War ja nur zu Testzwecken)
  this->Undo();

  return check;
}


void BindingElement::CheckDelays (PointInTime time) {
#ifdef _DEBUG
  if (servertype == EXCLUSIVE) {
    cout << "Fehler: Aufruf von CheckDelays() bei ES-Binding!\n";
    exit (-1);
  }
  servertype = INFINITE;
#endif

#ifndef NOOPTIMIZE
  if (delay_changed) {
#endif
    for (int i = 0; i < ftl.Size(); ++i) 
      ftl[i] = UpdateFiringTime (ftl[i], bod[i], time);
    PV = int(ftl.Fold(VECTOR_ARGMIN));
    firing_time = ftl[PV];
#ifndef NOOPTIMIZE
    delay_changed = false;
  }
#endif

}


PointInTime BindingElement::ISDelaySample (PointInTime time) {
  // Precondition: Dieses Binding ist schaltfaehig
#ifdef _DEBUG
  if (servertype == EXCLUSIVE) {
    cout << "Fehler: Aufruf von ISDelaySample bei ES-Binding!\n";
    exit (-1);
  }
  servertype = INFINITE;
  if (!IsEnabled()) {
    cout << "Verletzung der Vorbedingung in ISDelaySample.\n";
    exit (-1);
  }
#endif

#ifndef NOOPTIMIZE
  // Wenn kein Einfluss kam, muss nur die mark.-abh. Schaltzeit neu berechnet werden
  // und dann sind wir hier fertig.
  if (!influenced) {
#ifdef _DEBUG
    if (!was_enabled) {
      cout << "Binding war nicht enabled, ist es jetzt und bekam keinen Einfluss??? Das geht nicht.\n";
      exit(-1);
    }
#endif
    CheckDelays(time);
    return firing_time;
  }
  // Ab hier muessen wir alles neu berechnen
#endif


  if (was_enabled) {  // Binding war schaltfaehig
    // Laenge der Liste anpassen (falls der Grad sich geaendert hat
    // und die alten Delays bei Markierungsabh. neu berechnen
    int k = NeuerGradDerSchaltfaehigkeit();
    if (k < grad) {
      ftl.CutDown (k);
      bod.CutDown (k);
      CheckDelays(time);
    }
    else {    // Der neue Grad ist = oder groesser als der alte
      CheckDelays(time);
      for (int i = grad+1; i <= k; ++i) {
	ftl.Append(time + delay->GetSample());
	bod.Append(time);
      }
    }
    grad = k;
    PV = int(ftl.Fold(VECTOR_ARGMIN));
    firing_time = ftl[PV];
  }
  else {      // Binding war nicht schaltfaehig (aber jetzt dank Precondition!)
      // Neue Liste aufbauen und fuellen mit neuen Schaltzeiten
      grad = NeuerGradDerSchaltfaehigkeit();
      ftl.Clear();
      bod.Clear();
      for (int i = 0; i < grad; ++i) {
	ftl.Append(time + delay->GetSample());
	ftl.Append(time);
      }
      PV = int(ftl.Fold(VECTOR_ARGMIN));
      firing_time = ftl[PV];
    }


  // Einfluss resetten und Ende.
  influenced = NOT_INFLUENCED;
  delay_changed = false;
  was_enabled = true;
  return (firing_time);
}



PointInTime BindingElement::NewISDelaySample (PointInTime time) {
  // Precondition: Dieses Binding ist schaltfaehig
#ifdef _DEBUG
  if (servertype == EXCLUSIVE) {
    cout << "Fehler: Aufruf von NewISDelaySample bei ES-Binding!\n";
    exit (-1);
  }
  servertype = INFINITE;
  if (!IsEnabled()) {
    cout << "Verletzung der Vorbedingung in NewISDelaySample\n";
    exit (-1);
  }
#endif

  // Listen neu anlegen und fuellen
  ftl.Clear();
  bod.Clear();
  grad = NeuerGradDerSchaltfaehigkeit();
  if (grad <= 0) {
    firing_time = TOO_LONG;
    cout << "Fatal Error: Grad der Schaltfaehigkeit in NewESDelaySample ist Null!\n";
    exit (-1);
  }
  else {
    for (int i = 0; i < grad; ++i) {
      ftl.Append(time + delay->GetSample());
      bod.Append(time);
    }
    PV = int(ftl.Fold(VECTOR_ARGMIN));
    firing_time = ftl[PV];
  }

  influenced = NOT_INFLUENCED;
  delay_changed = false;
  was_enabled = true;
  return firing_time;
}


PointInTime BindingElement::NewESDelaySample (PointInTime time) {
  // Precondition: Dieses Binding ist schaltfaehig
#ifdef _DEBUG
  if (servertype == INFINITE) {
    cout << "Fehler: Aufruf von NewESDelaySample bei IS-Binding!\n";
    exit (-1);
  }
  servertype = EXCLUSIVE;
  if (!IsEnabled()) {
    cout << "Verletzung der Vorbedingung in NewESDelaySample\n";
    exit (-1);
  }
#endif
  firing_time = time + delay->GetSample();
  begin_of_ft = time;
  influenced = NOT_INFLUENCED;
  delay_changed = false;
  was_enabled = true;
  return firing_time;
}


PointInTime BindingElement::ESDelaySample (PointInTime time) {
  // Precodition: Dieses Binding ist schaltfaehig und war auch schaltfaehig
#ifdef _DEBUG
  if (servertype == INFINITE) {
    cout << "Fehler: Aufruf von NewESDelaySample bei IS-Binding!\n";
    exit (-1);
  }
  servertype = EXCLUSIVE;
  if (!IsEnabled()) {
    cout << "Verletzung der Vorbedingung in ESDelaySample\n";
    exit (-1);
  }
  if (!was_enabled) {
    cout << "Vorbedingung 'Binding.was_enabled = true' wurde verletzt in ESDelaySample()\n";
    exit (-1);
  }
#endif


#ifndef NOOPTIMIZE  
  if (delay_changed) {
#endif

    firing_time = UpdateFiringTime(firing_time, begin_of_ft, time);

#ifndef NOOPTIMIZE
    delay_changed = false;
     }
#endif
  return firing_time;
}


int BindingElement::NeuerGradDerSchaltfaehigkeit() {

  // Schalten bis es nicht mehr geht...
  int result = 0;
  while (IsEnabled()) {
    result++;
    Go();
  }

  // und wieder zurueckschalten
  for (int i = 0; i < result; ++i)
    Undo();

  return result;
}


PointInTime BindingElement::UpdateFiringTime (PointInTime future, PointInTime past, PointInTime now) {

  //  return (future);   Das war doch nur zum Debugging!

  switch (delay->GetDelayType()) {
  case _EXPONENTIAL:   
    return (now + delay->GetSample()); 
    break;
  case _DETERMINISTIC: 
    Period pnew = delay->GetSample();
    double percentage = (future - now) / (future - past);
    return (now + percentage*pnew);
    break;
  }
  // Alle anderen Delays koennen nicht umbewertet werden. Daher Abbruch
  cout << "Fatal Error. Nur exponentielle und deterministische ";
  cout << "Schaltzeiten duerfen markierungsabhaengig sein.\n";
  exit (-1);
  
  return (TOO_LONG); // just to keep the compiler happy ;-)

}

// End Simulator

#endif



