#ifndef __BindingElement_h__
#define __BindingElement_h__

#include "Types.h"
#include "ArcBindList.h"

// Begin Simulator
#define MODIFIED 3
#define ENABLED  1
#define DISABLED 2
#define NOT_INFLUENCED 0

// Nur fuer die Kontrolle der Zusicherungen beim Debugging
typedef enum { EXCLUSIVE, INFINITE, NOTYETDETERMINED } M_ServerType;

// End Simulator


//#################################################################################################
class CellVal;
class Cell;
class Delay;
//#################################################################################################
class BindingElement
{
	Weight		weight;
	Cell*		localcond;
	short		lc_result;
	Delay*		delay;
	double		del_result;
	ArcBindList	inarcbind;
	ArcBindList	outarcbind;

// Begin Simulator
	M_ServerType    servertype;  // Wird nur fuer die Pruefung der Zusicherungen verwendet
	PointInTime     firing_time; // naechster Schaltzeitpunkt (ES und IS)
	PointInTime     begin_of_ft; // Beginn der Schaltzeit (nur bei ES). Notwendig zum Update der Schaltzeit
        bool            was_enabled; // Zustand des Bindings in letzter Markierung (nur bei ES)
	int             grad;        // Grad der Schaltfaehigkeit (nur bei IS gebraucht)
	ComfVector<PointInTime> ftl; // Firing-Time-List bei Grad > 1 (betrifft nur IS)
	ComfVector<PointInTime> bod; // Begin of Delays
	int             PV;          // Index des kleinsten Eintrags in ftl (nur bei IS)
	char            influenced;
	bool            delay_changed;
	int             firings;     // Zahl der Schaltungen (zur Kontrolle min_firings)
protected:
	PointInTime UpdateFiringTime(PointInTime, PointInTime, PointInTime);
	void        CheckDelays(PointInTime);
	int         NeuerGradDerSchaltfaehigkeit();
	void        InitVariables();
// End Simulator

public:
	BindingElement();
	BindingElement(int);
	BindingElement(Weight, const Cell*, const Delay*, const ArcBindList&, const ArcBindList&);
	BindingElement(Weight, const Cell*, const Delay*, const ArcBindList*, const ArcBindList*);
	BindingElement(const BindingElement&);
	~BindingElement() { Clear(); }

	//-----------------------------------------------------------------------------------------
	//	Delete	
	//-----------------------------------------------------------------------------------------
	void 	Clear();

	//-----------------------------------------------------------------------------------------
	//	"Getten"
	//-----------------------------------------------------------------------------------------
	Weight			GetWeight();
	Cell*			GetLocalCondition();
	Delay*			GetDelay();
	double			GetDelayVal();
	ArcBindList&		GetInArcBind();
	ArcBindList&		GetOutArcBind();
	const ArcBindList&	GetInArcBindRef();
	const ArcBindList&	GetOutArcBindRef();

	//-----------------------------------------------------------------------------------------
	//	Modifizieren	
	//-----------------------------------------------------------------------------------------
	void			ChangeWeight(Weight);
	void			ChangeLocalCondition(const Cell*);
	void			ChangeDelay(const Delay*);
	void			ChangeInArcBind(const ArcBindList&);
	void			ChangeOutArcBind(const ArcBindList&);

	//-----------------------------------------------------------------------------------------
	//	Operatoren	
	//-----------------------------------------------------------------------------------------
	BindingElement&		operator =(const BindingElement&);
	bool			operator==(BindingElement&);
	bool			operator!=(BindingElement&);

	//-----------------------------------------------------------------------------------------
	//	Listen-Operatoren	
	//-----------------------------------------------------------------------------------------
	bool			LessThan(BindingElement&) ;
	static char*		GetName();

	//-----------------------------------------------------------------------------------------
	//	Calculate
	//-----------------------------------------------------------------------------------------
	bool			LocalCondTrue();

// Begin Simulator
	void            Go();                                  //Schalten des Bindings
	void            Undo();
	void            ISGo();                                //Schalten eines IS-Bindings
        PointInTime     ISDelaySample (PointInTime);
	PointInTime     ESDelaySample (PointInTime);
	PointInTime     NewISDelaySample (PointInTime);
	PointInTime     NewESDelaySample (PointInTime);
	void            MaybeEnabled()      {influenced |= MODIFIED;};
	void            MaybeDisabled()     {influenced |= DISABLED;};
	void            MaybeInfluenced()   {influenced |= MODIFIED;};
	void            MaybeDelayChanged() {delay_changed = true;};  
	bool            IsEnabled();                                   // Schaltfaehigkeit pruefen
	int             FiringCount()       {return firings;};
// End Simulator


};
//#################################################################################################
#endif
