#ifndef __Cell_h__
#define __Cell_h__

#include <iostream.h>
#include "Types.h"

//##################################################################################
class Cell
{
public:
	Cell() {};
	virtual ~Cell() {};

	//--------------------------------------------------------------------------
	//	Pflicht-Funtionen	
	//--------------------------------------------------------------------------
	virtual void	  	Clear() 			= 0;
	virtual Cell*     	Copy() 			const 	= 0;
	virtual CellVal*  	Calculate()			= 0;
	virtual CellType	GetCellType()			= 0;
	virtual ResultType	GetResultType()			= 0;
	virtual CellType	GetConstCellType()	const	= 0;
	virtual bool		IsPlaceDependent()		= 0;

	virtual Cell&	  	operator =(const Cell&) 	= 0; 
	virtual bool	  	operator==(const Cell&) 	= 0;
	virtual bool	  	operator!=(const Cell&) 	= 0;

	//--------------------------------------------------------------------------
	//	spezielle RatingCell-Funktionen	
	//--------------------------------------------------------------------------
	virtual double  	CalculateFloat()		{ return 0; }
	virtual double  	CalculateCell1()		{ return 0; }
	virtual double  	CalculateCell2()		{ return 0; }
	virtual	void		SetCell2(int)			{};

	//--------------------------------------------------------------------------
	//	Get/Calculate
	//--------------------------------------------------------------------------
	virtual Operator  	GetOp()			const 	{ return empty;}
	virtual Cell*	  	GetOpd()		const 	{ return NULL; }
	virtual Cell*	  	GetCell1()		const 	{ return NULL; }
	virtual Cell*	  	GetCell2()		const 	{ return NULL; }
	virtual Cell*	  	GetLeftOpd()		const 	{ return NULL; }
	virtual Cell*	  	GetRightOpd()		const 	{ return NULL; }
	virtual CellVal*  	GetCellVal()		const 	{ return NULL; }

	virtual token_multiset* Get_token_multiset()	const 	{ return NULL; }
	virtual PlaceHeader* 	GetPlaceHeader()	const 	{ return NULL; }
	virtual RatingType 	GetRatingType()		const 	{ return no_rt;}

	//--------------------------------------------------------------------------
	//	Modifizieren	
	//--------------------------------------------------------------------------
	virtual void	  ChangePlaceHeader(PlaceHeader*)  	{};

	//--------------------------------------------------------------------------
	//	Error	
	//--------------------------------------------------------------------------
	virtual	void	CellError(char* s) { cerr <<s<<"\n"; exit(CELL_ERROR); }
};
//##################################################################################
class ConstVal : public Cell
{
	CellVal*	constval;

public:
	ConstVal();
	ConstVal(int);
	ConstVal(bool);
	ConstVal(char*);
	ConstVal(double);
	ConstVal(TokenMultiSet&);
	ConstVal(const Cell&);
	ConstVal(const CellVal*);
	~ConstVal() { Clear(); }


	//------------------------------------------------------------------
	//	Clear/Create
	//------------------------------------------------------------------
	void       Clear();			
	Cell*      Copy() const;			

	//------------------------------------------------------------------
	//	Get/Calculate	
	//------------------------------------------------------------------
	CellVal*   Calculate();		
	CellType   GetCellType();
	ResultType GetResultType();
	double     CalculateFloat();		
	CellType   GetConstCellType() 	const;
	CellVal*   GetCellVal() 	const;	
	bool       IsPlaceDependent();

	//------------------------------------------------------------------
	//	Operatoren	
	//------------------------------------------------------------------
	Cell&      operator =(const Cell&);
	bool	   operator==(const Cell&);
	bool	   operator!=(const Cell&);
};
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class UnaerOp : public Cell
{
	Operator 	op;
	Cell*		opd;
public:
	UnaerOp();			
	UnaerOp(Operator, int);
	UnaerOp(Operator, bool);
	UnaerOp(Operator, double);
	UnaerOp(const Cell&);
	UnaerOp(Operator, const Cell*);
	~UnaerOp() { Clear(); } 

	//------------------------------------------------------------------
	//	Clear/Create
	//------------------------------------------------------------------
	void       Clear();			
	Cell*      Copy() const;			

	//------------------------------------------------------------------
	//	Get/Calculate	
	//------------------------------------------------------------------
	Operator   GetOp()  		const;
	Cell*	   GetOpd() 		const;
	CellType   GetConstCellType() 	const;
	bool       IsPlaceDependent();
	double     CalculateFloat();		
	ResultType GetResultType();
	CellType   GetCellType();
	CellVal*   Calculate();		

	//------------------------------------------------------------------
	//	Operatoren	
	//------------------------------------------------------------------
	Cell&      operator =(const Cell&);
	bool	   operator==(const Cell&);
	bool	   operator!=(const Cell&);
};
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class BinaerOp : public Cell
{
	Operator 	op;
	Cell*		lopd;
	Cell*		ropd;
public:
	BinaerOp();	
	BinaerOp(const Cell&); 
	BinaerOp(Operator, int,    int);
	BinaerOp(Operator, bool,   int);
	BinaerOp(Operator, double, int);
	BinaerOp(Operator, int,    bool);
	BinaerOp(Operator, bool,   bool);
	BinaerOp(Operator, double, bool);
	BinaerOp(Operator, int,    double);
	BinaerOp(Operator, bool,   double);
	BinaerOp(Operator, double, double);
	BinaerOp(Operator, TokenMultiSet&, TokenMultiSet&);
	BinaerOp(Operator, const Cell*,    const Cell*);
	BinaerOp(Operator, const CellVal*, const CellVal*);
	~BinaerOp() { Clear(); }

	//------------------------------------------------------------------
	//	Clear/Create
	//------------------------------------------------------------------
	void       Clear();			
	Cell*      Copy() const;			

	//------------------------------------------------------------------
	//	Get/Calculate	
	//------------------------------------------------------------------
	Operator   GetOp()       	const;
	Cell*	   GetLeftOpd()  	const;
	Cell*	   GetRightOpd() 	const;
	CellType   GetConstCellType() 	const;
	bool       IsPlaceDependent();
	double     CalculateFloat();		
	ResultType GetResultType();
	CellType   GetCellType();
	CellVal*   Calculate();		

	//------------------------------------------------------------------
	//	Operatoren	
	//------------------------------------------------------------------
	Cell&      operator =(const Cell&);
	bool	   operator==(const Cell&);
	bool	   operator!=(const Cell&);
};
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class Function : public Cell
{
	Operator	op;
	PlaceHeader*	place;		// Name und Inhalt einer Stelle
	token_multiset*	tms;

	//------------------------------------------------------------------
	//	Compare/Calculate
	//------------------------------------------------------------------
	CellVal* CalculateAll	  (TokenMultiSet*&);
	void	 CompToken        (TokenMultiSet*&, int,      CellVal*&);
	void	 CompNames        (TokenMultiSet*&, Textual&, CellVal*&);
	void	 CompNamesAndSteps(TokenMultiSet*&, Textual&, Identifier&, CellVal*&);
public:
	Function(const Cell&);
	Function(Operator, char*);
	Function(Operator, char*, const Cell*);
	Function(Operator, char*, token_multiset*);
	~Function() {  Clear(); }

	//------------------------------------------------------------------
	//	Create/Delete
	//------------------------------------------------------------------
	Cell*      	Copy() 			const;			
	void		Clear();

	//------------------------------------------------------------------
	//	Get/Calculate	
	//------------------------------------------------------------------
	Operator   	GetOp()        		const;
	token_multiset*	Get_token_multiset()	const;
	PlaceHeader*  	GetPlaceHeader()    	const;
	CellType   	GetConstCellType() 	const;
	bool       	IsPlaceDependent();
	double     	CalculateFloat();		
	ResultType 	GetResultType();
	CellType   	GetCellType();
	CellVal*   	Calculate();		

	//------------------------------------------------------------------
	//	Modifizieren	
	//------------------------------------------------------------------
	void	   	ChangePlaceHeader(PlaceHeader*);

	//------------------------------------------------------------------
	//	Operatoren	
	//------------------------------------------------------------------
	Cell&      	operator =(const Cell&);
	bool	   	operator==(const Cell&);
	bool	   	operator!=(const Cell&);
};
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class RatingCell : public Cell
{
	RatingType	rt;
	Cell*		cell1;
	Cell*		cell2;

	//------------------------------------------------------------------
	//	Create
	//------------------------------------------------------------------
	void	NewC1	  (Cell*);
	void	NewC2	  (Cell*);
	void	NewRatCell(const Cell&);
public:
	RatingCell();
	RatingCell(const Cell&);
	RatingCell(RatingType, int);
	RatingCell(RatingType, char*);
	RatingCell(RatingType, Cell*);
	RatingCell(RatingType, int,   int);
	RatingCell(RatingType, int,   Cell*);
	RatingCell(RatingType, Cell*, int);
	RatingCell(RatingType, Cell*, Cell*);
	~RatingCell() { Clear(); }

	//------------------------------------------------------------------
	//	Copy/Delete
	//------------------------------------------------------------------
	Cell*      	Copy() 			const;			
	void		Clear();

	//------------------------------------------------------------------
	//	"getten"/"setten"
	//------------------------------------------------------------------
	bool       	IsPlaceDependent();
	ResultType 	GetResultType();
	CellType   	GetCellType();
	RatingType 	GetRatingType()		const;
	CellType   	GetConstCellType() 	const;
	Cell*		GetCell1()		const;
	Cell*		GetCell2()		const;
	void		SetCell2(int);
	//------------------------------------------------------------------
	//	Calculate	
	//------------------------------------------------------------------
	CellVal*   	Calculate();		
	double   	CalculateFloat();		
	double   	CalculateCell1();		
	double   	CalculateCell2();		

	//------------------------------------------------------------------
	//	Operatoren	
	//------------------------------------------------------------------
	Cell&      	operator =(const Cell&);
	bool	   	operator==(const Cell&);
	bool	   	operator!=(const Cell&);
};
//############################################################################
#endif
