#ifndef __CELLVAL_CC__
#define __CELLVAL_CC__

#include "Textual.h"

#include "token_multiset.h"
#include "TokenMultiSet.h"

#include "CellVal.h"

//########################################################################################################
//				TextVal
//########################################################################################################
TextVal::TextVal()						: tval(DEF_TXT_NAME)	 { };
TextVal::TextVal(char* c)					: tval(c)		 { };
TextVal::TextVal(Textual t)					: tval(t)	 	 { };
TextVal::TextVal(const TextVal& tv) 				: tval(tv.tval)		 { };
//########################################################################################################
void		TextVal::Clear()				{ tval.Clear();            }
CellVal*       	TextVal::Copy()     		const		{ return new TextVal(tval);}
char*		TextVal::GetText() 		const		{ return tval.GetChar();   }
ResultType	TextVal::GetResultType()	const		{ return textual;	   }
//-------------------------------------------------------------------------------------------------------
bool     	TextVal::IsTMS()    		const   { CellValError("op","????? op char*");   return false; } 
CellVal*	TextVal::operator +(const CellVal&) 	{ CellValError("+", "char* +  ???????"); return NULL;  }
CellVal*	TextVal::operator -(const CellVal&) 	{ CellValError("-", "char* -  ???????"); return NULL;  }
CellVal*	TextVal::operator *(const CellVal&) 	{ CellValError("*", "char* *  ???????"); return NULL;  }
CellVal*	TextVal::operator /(const CellVal&) 	{ CellValError("/", "char* /  ???????"); return NULL;  }
CellVal*	TextVal::operator&&(const CellVal&)	{ CellValError("&&","char* && ???????"); return NULL;  }
CellVal*	TextVal::operator||(const CellVal&)	{ CellValError("||","char* || ???????"); return NULL;  }
CellVal*	TextVal::operator>=(CellVal&) 		{ CellValError(">=","char* >= ???????"); return NULL;  }
CellVal*	TextVal::operator<=(CellVal&) 		{ CellValError("<=","char* <= ???????"); return NULL;  }
CellVal*	TextVal::operator >(CellVal&) 		{ CellValError(">"," char* >  ???????"); return NULL;  }
CellVal*	TextVal::operator <(CellVal&)	 	{ CellValError("<"," char* <  ???????"); return NULL;  }
CellVal*	TextVal::operator!(void) 		{ CellValError("!(unary)","!(char*)");   return NULL;  }
CellVal*	TextVal::operator-(void) 		{ CellValError("-(unary)","-(char*)");   return NULL;  }
CellVal*	TextVal::Abs(void) 			{ CellValError("#","#(char*)"); 	 return NULL;  }
CellVal*	TextVal::Contents(void) 		{ CellValError("@","@(char*)"); 	 return NULL;  }
//-------------------------------------------------------------------------------------------------------
CellVal&	TextVal::operator =(const CellVal& cv)
{
	if (!cv.GetText()) CellValError("=","char* = ????????");
	tval = cv.GetText();
	return *this;
}
//-------------------------------------------------------------------------------------------------------
CellVal*	TextVal::operator ==(CellVal& cv)
{
	if (!cv.GetText()) return new FloatVal(false);
	return new FloatVal(tval == cv.GetText());
}
//-------------------------------------------------------------------------------------------------------
CellVal*	TextVal::operator !=(CellVal& cv)
{
	if (!cv.GetText()) return new FloatVal(true);
	return new FloatVal(tval != cv.GetText());
}
//########################################################################################################
//				FLOATVAL
//########################################################################################################
FloatVal::FloatVal()						: val(0.0)               { };
FloatVal::FloatVal(int i)					: val(double(i))         { };
FloatVal::FloatVal(bool b)					: val(double(b))         { };
FloatVal::FloatVal(double d)					: val(d)                 { };
FloatVal::FloatVal(const FloatVal& fv) 				: val(fv.val)            { };
//########################################################################################################
void		FloatVal::Clear()				{ ; }
CellVal*       	FloatVal::Copy()     		const		{ return new FloatVal(val);}
bool     	FloatVal::IsTMS()    		const           { return false; 	   }
double		FloatVal::GetFloat() 		const		{ return val;		   }
ResultType	FloatVal::GetResultType()	const		{ return numeric;	   }
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator!(void) 	     { return new FloatVal(!(bool(val)));  }
CellVal*	FloatVal::operator-(void) 	     { return new FloatVal(-val); 	   }
CellVal*	FloatVal::Abs(void)       	     { return new FloatVal(val<0?-val:val);}
CellVal*	FloatVal::Contents(void)  	     { return new FloatVal(val); 	   }
//--------------------------------------------------------------------------------------------------------
CellVal&	FloatVal::operator =(const CellVal& cv)
{
	if (cv.IsTMS())	CellValError("=","double = TokenMultiSet");
	val = cv.GetFloat();
	return *this;
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator +(const CellVal& cv)
{
	if (cv.IsTMS()) CellValError("+","double + TokenMultiSet"); 
	return new FloatVal(val + cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator -(const CellVal& cv)
{
	if (cv.IsTMS()) CellValError("-","double - TokenMultiSet"); 
	return new FloatVal(val - cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator *(const CellVal& cv)
{
	if (cv.IsTMS()) return ( *(cv.Copy()) * *this );   
	return new FloatVal(val * cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator /(const CellVal& cv)
{
	if (cv.IsTMS()) CellValError("/","double / TokenMultiSet"); 

	double d = cv.GetFloat();
	if (d == 0) return new FloatVal(val);
	else        return new FloatVal(val / d);
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator ==(CellVal& cv)
{
	if (cv.IsTMS()) return new FloatVal(false);
	return new FloatVal(val == cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator !=(CellVal& cv)
{
	if (cv.IsTMS()) return new FloatVal(false); 
	return new FloatVal(val != cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator >=(CellVal& cv)
{
	if (cv.IsTMS())  return new FloatVal(false);
	return new FloatVal(val >= cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator <=(CellVal& cv)
{
	if (cv.IsTMS())  return new FloatVal(false);
	return new FloatVal(val <= cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator >(CellVal& cv)
{
	if (cv.IsTMS())  return new FloatVal(false);
	return new FloatVal(val > cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator <(CellVal& cv)
{
	if (cv.IsTMS())  return new FloatVal(false);
	return new FloatVal(val < cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator &&(const CellVal& cv)
{
	if (cv.IsTMS())  
	{
		CellVal* tmp_cell = cv.Copy();					 // changed
		double   ret      = (val && tmp_cell->GetTokenMultiSet().quant); // changed
		delete   tmp_cell;					 	 // changed
		return new FloatVal(ret);					 // changed
	}
	return new FloatVal(val && cv.GetFloat());
}
//--------------------------------------------------------------------------------------------------------
CellVal*	FloatVal::operator ||(const CellVal& cv)
{
	if (cv.IsTMS())  
	{
		CellVal* tmp_cell = cv.Copy();					 // changed
		double   ret      = (val || tmp_cell->GetTokenMultiSet().quant); // changed
		delete   tmp_cell;					 	 // changed
		return new FloatVal(ret);					 // changed
	}
	return new FloatVal(val || cv.GetFloat());
}

//########################################################################################################
//				TOKENMSVAL
//########################################################################################################
TokenMSVal::TokenMSVal()                                    		: tms()          { };
TokenMSVal::TokenMSVal(int)                                 		: tms()          { };
TokenMSVal::TokenMSVal(int tr, int q)                                	: tms(tr, q)     { };
TokenMSVal::TokenMSVal(const TokenMSVal& t)                 		: tms(t.tms)     { };
TokenMSVal::TokenMSVal(const token_multiset* tset)           		: tms(tset)      { };
TokenMSVal::TokenMSVal(const TokenMultiSet*  tset)           		: tms(tset)      { };
TokenMSVal::TokenMSVal(const TokenMultiSet&  tset)           		: tms(tset)      { };
//########################################################################################################
  void		TokenMSVal::Clear()					{ tms.Clear();          	 }
CellVal*       	TokenMSVal::Copy()             	const			{ return new TokenMSVal(tms); 	 }
bool           	TokenMSVal::IsTMS()  	       	const			{ return true; 			 }
TokenMultiSet& 	TokenMSVal::GetTokenMultiSet() 				{ return tms;            	 }
TokenMultiSet* 	TokenMSVal::GetTokenMultiSetPtr() 			{ return new TokenMultiSet(tms); }
//--------------------------------------------------------------------------------------------------------
CellVal* 	TokenMSVal::Abs(void)		    { return new FloatVal(int(tms.quant)); 	 	 }
CellVal* 	TokenMSVal::Contents(void)	    { return new TokenMSVal(tms);		 	 }
CellVal*	TokenMSVal::operator -(void)	    { CellValError("-(unary)","-(TokenMultiSet)"); return NULL; }
//--------------------------------------------------------------------------------------------------------
ResultType	TokenMSVal::GetResultType()	const	
{ 
	if (!tms.tmset)				  { return e_mset_expr; }
	if (!tms.tmset->first)			  { return e_mset_expr; }
	if ( tms.tmset->first->tref == NO_COLOR ) { return e_mset_expr;	}
	return p_mset_expr;
}
//--------------------------------------------------------------------------------------------------------
CellVal& 	TokenMSVal::operator =(const CellVal& cv)
{
	if (!cv.IsTMS()) CellValError("=","TokenMSVal = ?????");

	CellVal* tmp = cv.Copy();
	tms = tmp->GetTokenMultiSet();			// changed
	delete tmp;					// changed
	return *this;
}
//--------------------------------------------------------------------------------------------------------
CellVal* 	TokenMSVal::operator +(const CellVal& cv) 
{ 
	if (!cv.IsTMS()) CellValError("+","TokenMultiSet + ?????");

	CellVal* 	tmp     = cv.Copy();
	TokenMultiSet&	tmp_tms = tmp->GetTokenMultiSet();
	TokenMultiSet*	ret_tms = tms.Copy();

	if (tmp_tms.tmset && tmp_tms.tmset->first)
		for (token_referenz* tr=tmp_tms.tmset->first; tr; tr=tr->next)
			ret_tms->AddToken(tr);

	CellVal* ret = new TokenMSVal(*ret_tms);	// changed
	delete ret_tms;					// changed
	delete tmp;
        return ret; 					// changed
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator -(const CellVal& cv) 
{ 
	if (!cv.IsTMS()) CellValError("-","TokenMultiSet - ?????");

	CellVal* 	tmp     = cv.Copy();
	TokenMultiSet&	tmp_tms = tmp->GetTokenMultiSet();
	TokenMultiSet*	ret_tms = tms.Copy();

	if (tmp_tms.tmset && tmp_tms.tmset->first)
		for (token_referenz* tr=tmp_tms.tmset->first; tr; tr=tr->next)
			ret_tms->SubToken(tr);

	CellVal* ret = new TokenMSVal(*ret_tms);	// changed
	delete ret_tms;					// changed
	delete tmp;
        return ret; 					// changed
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator *(const CellVal& cv)
{
	if (cv.IsTMS()) CellValError("*","TokenMultiSet * TokenMultiSet");

	int tmp_quant	        = 0;
	int multiplicator       = int(cv.GetFloat());
	TokenMultiSet*	ret_tms = tms.Copy();

	if (multiplicator <  0) 	CellValError("*","TokenMultiSet * -(double)"); 
	if (multiplicator == 0) 	{ tms.Clear(); return new TokenMSVal(); }

	if (ret_tms->tmset && ret_tms->tmset->first)
	{
		for (token_referenz* tmp=ret_tms->tmset->first; tmp; tmp=tmp->next)
		{
			ret_tms->quant      -= tmp->quant;
			ret_tms->tmset->key -= int(tmp->quant);

			if ((tmp_quant = (int(tmp->quant) * multiplicator)) > MAX_QUANT) 
				tmp->quant = MAX_QUANT;
			else	tmp->quant = Quant(tmp_quant);

			ret_tms->quant      += tmp->quant;
			ret_tms->tmset->key += int(tmp->quant);
		}
	}
	CellVal* ret = new TokenMSVal(*ret_tms);	//changed
	delete ret_tms;					//changed
        return ret; 					//changed
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator /(const CellVal& cv)
{
	if (cv.IsTMS()) CellValError("/","TokenMultiSet / TokenMultiSet");

	int	tmp_quant       = 0;
	double	divisor         = cv.GetFloat();
	TokenMultiSet*	ret_tms = tms.Copy();

	if (divisor <  0) CellValError("/","TokenMultiSet / -(double)"); 
	if (divisor == 0) CellValError("/","TokenMultiSet / NULL"); 

	if (ret_tms->tmset && ret_tms->tmset->first)
	{
		for (token_referenz* tmp=ret_tms->tmset->first; tmp; tmp=tmp->next)
		{
			ret_tms->quant      -= tmp->quant;
			ret_tms->tmset->key -= int(tmp->quant);
			tmp_quant            = int(int(tmp->quant) / divisor); 
			tmp->quant 	     = Quant(tmp_quant);
			ret_tms->quant      += tmp->quant;
			ret_tms->tmset->key += int(tmp->quant);
		}
	}
	CellVal* ret = new TokenMSVal(*ret_tms);	// changed
	delete ret_tms;					// changed
        return ret; 					// changed
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator==(CellVal& cv)
{
	if (!cv.IsTMS()) return new FloatVal(false); 
	return new FloatVal((tms == cv.GetTokenMultiSet()));
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator!=(CellVal& cv)
{
	if (!cv.IsTMS()) return new FloatVal(false); 
	return new FloatVal((tms != cv.GetTokenMultiSet()));
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator <(CellVal& cv)
{
	if (!cv.IsTMS()) return new FloatVal(false); 
	return new FloatVal((tms.quant < cv.GetTokenMultiSet().quant));
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator >(CellVal& cv)
{
	if (!cv.IsTMS()) return new FloatVal(false); 
	return new FloatVal((tms.quant > cv.GetTokenMultiSet().quant));
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator<=(CellVal& cv)
{
	if (!cv.IsTMS()) return new FloatVal(false); 
	return new FloatVal((tms.quant <= cv.GetTokenMultiSet().quant));
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator>=(CellVal& cv)
{
	if (!cv.IsTMS()) return new FloatVal(false); 
	return new FloatVal((tms.quant >= cv.GetTokenMultiSet().quant));
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator&&(const CellVal& cv) 
{ 	
	if (!cv.IsTMS()) return new FloatVal(false);

	CellVal* tmp = cv.Copy();					// changed
	double   ret = (tms.quant && tmp->GetTokenMultiSet().quant);	// changed
	delete   tmp;							// changed
	return new FloatVal(ret);					// changed
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator||(const CellVal& cv) 
{ 	
	if (!cv.IsTMS()) return new FloatVal(false);

	CellVal* tmp = cv.Copy();					// changed
	double   ret = (tms.quant || tmp->GetTokenMultiSet().quant);	// changed
	delete   tmp;							// changed
	return new FloatVal(ret);					// changed
}
//--------------------------------------------------------------------------------------------------------
CellVal* TokenMSVal::operator !(void)        
{ 
	if (tms.quant) return new FloatVal(true);
        return new FloatVal(false);
}
//########################################################################################################
#endif
