#ifndef __CELLVAL_H__
#define __CELLVAL_H__

#include <iostream.h>
#include "Types.h"

//####################################################################################################
class CellVal
{
public:
	CellVal() {};
	virtual ~CellVal() {};

	virtual void	 Clear()				= 0;
	virtual CellVal* Copy()	 const				= 0;
	virtual bool     IsTMS() const				= 0;

	virtual CellVal& operator =(const CellVal&)		= 0;
	virtual CellVal* operator==(CellVal&)			= 0;
	virtual CellVal* operator!=(CellVal&)			= 0;
	virtual CellVal* operator>=(CellVal&)			= 0;
	virtual CellVal* operator<=(CellVal&)			= 0;
	virtual CellVal* operator >(CellVal&) 			= 0;
	virtual CellVal* operator <(CellVal&)			= 0;

	virtual CellVal* operator +(const CellVal&)		= 0;
	virtual CellVal* operator *(const CellVal&)		= 0;
	virtual CellVal* operator -(const CellVal&)		= 0;
	virtual CellVal* operator /(const CellVal&)		= 0;
	virtual CellVal* operator -(void)     			= 0;

	virtual CellVal* operator &&(const CellVal&)		= 0;
	virtual CellVal* operator ||(const CellVal&) 		= 0;
	virtual CellVal* operator  !(void) 			= 0;

	virtual CellVal* Abs(void)				= 0;
	virtual CellVal* Contents(void)				= 0;

	virtual ResultType     GetResultType()	   const 	= 0;

	virtual char*	       GetText()      	   const	{ return NULL; 	}
	virtual double	       GetFloat()      	   const	{ return 0.0;  	}
	virtual TokenMultiSet& GetTokenMultiSet()  		{ return (*(new TokenMultiSet())); }
	virtual TokenMultiSet* GetTokenMultiSetPtr()  		{ return   (new TokenMultiSet());  }

	void	CellValError(char* op, char* s)    const
	{ 
		cout <<"Error in "<<op<<"-Operation : "<<s<<"\n"; 
		exit(CELLVAL_ERROR); 
	}
};
//#####################################################################################################
class TextVal : public CellVal
{
	Textual		tval;
public:
	TextVal();
	TextVal(char*);
	TextVal(Textual);
	TextVal(const TextVal&);
	~TextVal() { Clear(); } 

	void		Clear();
	CellVal* 	Copy()	 	const;
	bool	 	IsTMS() 	const;
	char* 	 	GetText() 	const;
	ResultType	GetResultType()	const;

	//--------------------------------------------------------------------------------------
	//	Operatoren
	//--------------------------------------------------------------------------------------
	CellVal& operator  =(const CellVal& cv);
	CellVal* operator ==(CellVal& cv); 
	CellVal* operator !=(CellVal& cv);
	CellVal* operator >=(CellVal& cv);
	CellVal* operator <=(CellVal& cv);
	CellVal* operator  <(CellVal& cv);
	CellVal* operator  >(CellVal& cv);

	CellVal* operator +(const CellVal& cv);
	CellVal* operator *(const CellVal& cv);
	CellVal* operator /(const CellVal& cv);
	CellVal* operator -(const CellVal& cv);
	CellVal* operator -(void);

	CellVal* operator &&(const CellVal& cv);
	CellVal* operator ||(const CellVal& cv);
	CellVal* operator  !(void);

	CellVal* Abs(void);
	CellVal* Contents(void);
};
//#####################################################################################################
class FloatVal : public CellVal
{
	double		val;
public:
	FloatVal();
	FloatVal(int);
	FloatVal(bool);
	FloatVal(double);
	FloatVal(const FloatVal&);
	~FloatVal() { Clear(); } 

	void		Clear();
	CellVal* 	Copy()	 	const;
	bool	 	IsTMS() 	const;
	double	 	GetFloat() 	const;
	ResultType	GetResultType()	const;

	//--------------------------------------------------------------------------------------
	//	Operatoren
	//--------------------------------------------------------------------------------------
	CellVal& operator  =(const CellVal& cv);
	CellVal* operator ==(CellVal& cv); 
	CellVal* operator !=(CellVal& cv);
	CellVal* operator >=(CellVal& cv);
	CellVal* operator <=(CellVal& cv);
	CellVal* operator  <(CellVal& cv);
	CellVal* operator  >(CellVal& cv);

	CellVal* operator +(const CellVal& cv);
	CellVal* operator *(const CellVal& cv);
	CellVal* operator /(const CellVal& cv);
	CellVal* operator -(const CellVal& cv);
	CellVal* operator -(void);

	CellVal* operator &&(const CellVal& cv);
	CellVal* operator ||(const CellVal& cv);
	CellVal* operator  !(void);

	CellVal* Abs(void);
	CellVal* Contents(void);
};
//####################################################################################################
class TokenMSVal : public CellVal
{
	TokenMultiSet	tms;
public:
	TokenMSVal();
	TokenMSVal(int);
	TokenMSVal(int, int);
	TokenMSVal(const TokenMSVal&);
	TokenMSVal(const TokenMultiSet*);
	TokenMSVal(const TokenMultiSet&);
	TokenMSVal(const token_multiset*);
	~TokenMSVal() { Clear(); }


	void		Clear();
	CellVal*       	Copy()			const;
	bool	       	IsTMS()			const;
	ResultType	GetResultType()		const;
	TokenMultiSet& 	GetTokenMultiSet();
	TokenMultiSet*  GetTokenMultiSetPtr(); 

	//--------------------------------------------------------------------------------------
	//	Operatoren
	//--------------------------------------------------------------------------------------
	CellVal& operator  =(const CellVal&);
	CellVal* operator ==(CellVal& cv);
	CellVal* operator !=(CellVal& cv);
	CellVal* operator >=(CellVal& cv); 
	CellVal* operator <=(CellVal& cv);
	CellVal* operator  <(CellVal& cv);
	CellVal* operator  >(CellVal& cv);

	CellVal* operator +(const CellVal& cv);
	CellVal* operator *(const CellVal& cv);
	CellVal* operator /(const CellVal& cv);
	CellVal* operator -(const CellVal& cv);
	CellVal* operator -(void);

	CellVal* operator &&(const CellVal& cv);
	CellVal* operator ||(const CellVal& cv);
	CellVal* operator  !(void);
	//-------------------------------------------------------------------------------------
	// # (= Abs) und @ (=Contents) Operatoren 
	//-------------------------------------------------------------------------------------
	CellVal* Abs(void);
	CellVal* Contents(void);
};
//############################################################################################################
#endif
