#ifndef __LIST_CC__
#define __LIST_CC__

extern "C" {
#include <strings.h>
}
#include <iostream.h>

#include "List.h"


//###################################################################################
//			LISTELEMENT
//###################################################################################
template<class T> inline ListElement<T>::ListElement() 
{
	T* tmp   = new T(LIST_FAULT_VAL);
	lelement = *tmp; 
	next     = NULL;
}

template<class T> inline ListElement<T>::ListElement(const T& le, ListElement<T>* n) 
{ 
	next     = new ListElement<T>(); 
	next     = n;
	lelement = le; 
}

template<class T> inline ListElement<T>::ListElement(const ListElement<T>& le)
{ 
	next     = new ListElement<T>();
	next     = le.next; 
	lelement = le.lelement; 
}
//#####################################################################################
template<class T> inline ListElement<T>& ListElement<T>::operator =(const ListElement<T>& le)
{
	lelement = le.lelement;
	next     = NULL;
	return *this;
}

//#####################################################################################
//				LIST
//#####################################################################################
template<class T> inline List<T>::List()   
{
	first = new ListElement<T>();
	last  = first;
	size  = 0;
}

template<class T> inline List<T>::List(const T& le)  
{ 
	first           = last = new ListElement<T>(); 
	first->lelement = le;
	last            = first;
	size  = 1;
}

template<class T> inline List<T>::List(const T* le) 
{ 
	first 		= last = new ListElement<T>(); 
	first->lelement = *le;
	last  		= first;
	size  = 1;
}

template<class T> inline List<T>::List(const T& le1, const T& le2) 
{ 
	first 		      = last = new ListElement<T>(); 
	first->next           = new ListElement<T>();
	first->lelement       = le1;
	first->next->lelement = le2;
	last  		      = first->next;
	size  = 2;
}

template<class T> inline List<T>::List(const List<T>& l) 
{
	first 		= last = new ListElement<T>();
	first->lelement = l.first->lelement;
	last            = first;
	for (ListElement<T>* tmp = l.first->next; tmp; tmp=tmp->next)
	{
	    	last->next           = new ListElement<T>();
	    	last->next->lelement = tmp->lelement;
	    	last                 = last->next;
	}
	size = l.size;
}

template<class T> inline List<T>::List(const List<T>& l1, const List<T>& l2) 
{
	first 		= last = new ListElement<T>();
	first->lelement = l1.first->lelement;
	last            = first;
	for (ListElement<T>* tmp1 = l1.first->next; tmp1; tmp1=tmp1->next)
	{
	    	last->next           = new ListElement<T>();
	    	last->next->lelement = tmp1->lelement;
	    	last                 = last->next;
	}
	for (ListElement<T>* tmp2 = l2.first->next; tmp2; tmp2=tmp2->next)
	{
	    	last->next           = new ListElement<T>();
	    	last->next->lelement = tmp2->lelement;
	    	last                 = last->next;
	}
	size = l1.size + l2.size;
}
//#####################################################################################
template<class T> inline void List<T>::Clear()
{
	if (size)
	{
		ListElement<T>* h;
		for (;size;size--)
		{
			if (!first) break;
			if (!first->next)
			{
				first->lelement.Clear();
				delete first;
				break;
			}
			h = first->next; 
			first->lelement.Clear();
			delete first;
     			first = h;
     		}
     	}
	first = NULL;
	last  = NULL;
	size  = 0;
}
//----------------------------------------------------------------------------
 template<class T> inline int List<T>::Append(const T& e)
{
	if (first)
	{
		if (size == 0) 
		{
			delete first;
			first 		= last = new ListElement<T>();
	                first->lelement = e;
                        last            = first;
                        size = 1;
		}
		else
		{
			last->next 	     = new ListElement<T>();
			last->next->lelement = e;
			last		     = last->next;
			size++;
		}
	}
	else
	{
		first 		= last = new ListElement<T>();
		first->lelement = e;
		last		= first;
		size 		= 1;
	}
	return size;
}
//----------------------------------------------------------------------------
 template<class T> inline int List<T>::UniqueAppend(const T& e)
{
	T	tmp(e);
	if (!first || size == 0) return Append(e);
	
	int pos = 1;
	ListElement<T>* help = first;
	for (; help; help=help->next, pos++)
		if (help->lelement == tmp) return -pos;
	return Append(e);
}
//----------------------------------------------------------------------------
 template<class T> inline T* List<T>::UniqueNameAppend(const T& e)
{
	T  tmp(e);

	if (!first || !size) 
	{
		Append(e);
		return &(first->lelement);
	}	
	
	int   pos  = 1;
	char* name = tmp.GetName();

	for (ListElement<T>* help=first; help; help=help->next, pos++)
		if ( strcmp(help->lelement.GetName(),name) == 0) 
			return NULL;
	Append(e);
	return &(last->lelement);
}
//----------------------------------------------------------------------------
template<class T> inline T* List<T>::Insert(const T& e, bool& exists)
{
	T	tmp(e);
	//--------------------------------------------------------------------
	//	Ab, hinten anstellen !!
	//--------------------------------------------------------------------
	if ( !size || last->lelement.LessThan(tmp) ) 
	{
		Append(e);
		exists = false;
		return &(last->lelement);
	}
	//--------------------------------------------------------------------
	//	Ok, darfst vor	!!	
	//--------------------------------------------------------------------
	if ( tmp.LessThan(first->lelement) )
	{
		ListElement<T>* new_e 	= new ListElement<T>();
		new_e->lelement       	= tmp;
		new_e->next  		= first;
		first        		= new_e;
		exists 			= false;
		size++;
		return &(first->lelement);
	}
	//--------------------------------------------------------------------
	//	Dich gibt's schon !!	
	//--------------------------------------------------------------------
	if ( first->lelement == tmp ) { exists=true; return &(first->lelement); }
	if ( last->lelement  == tmp ) { exists=true; return &(last->lelement);  }

	//--------------------------------------------------------------------
	//	Such dein Platz	
	//--------------------------------------------------------------------
	ListElement<T>* h0 = first;
	ListElement<T>* h1 = first->next;
	for (int pos=1; h1; pos++, h0=h1, h1=h1->next)
	{
		if (tmp.LessThan(h1->lelement))
		{
			ListElement<T>* new_el	= new ListElement<T>();
			new_el->lelement       	= tmp;
			h0->next 	      	= new_el;
			new_el->next 		= h1; 
			exists			= false;
			size++;
			return &(new_el->lelement);
		}
		if (h1->lelement == tmp)  { exists=true; return &(h1->lelement); }
	}
	Append(e);
	exists=false;
	return &(last->lelement);
}
//----------------------------------------------------------------------------
 template<class T> inline int List<T>::DeleteElement(int pos)
{
	if (pos < 1 || pos > size || !first ) return LIST_FAULT_VAL;

	if (pos == 1)
	{
		if (first->next) 
		{
			ListElement<T>* h = first;
			first             = first->next;
			delete h;
			size--;
			return 1;
		}
		else             
		{
			delete first;
			first = last = new ListElement<T>();
			last  = first;
			size  = 0;
			return  1;
		}
	}

	ListElement<T>* h0=first;
	ListElement<T>* h1=first;
	for (int i=1; i<pos; i++) 
	{
		h0 = h1;
		h1 = h1->next;
	}
	if (pos == size) last = h0;
	h0->next = h1->next;
	delete h1;
	size--;
	return pos;
}
//----------------------------------------------------------------------------
 template<class T> inline int List<T>::DeleteElement(T e)
{
	if (!first) return LIST_FAULT_VAL;

	if (first->lelement == e)
	{
		if (first->next) 
		{
		        ListElement<T>* h=first;
			first = first->next;
			delete h;
			size--;
			return 1;
		}
		else             
		{
			delete first;
			first = last = new ListElement<T>();
			last  = first;
			size  = 0;
			return  1;
		}
	}

	int		pos = 1;
	ListElement<T>* h0  = first;
	ListElement<T>* h1  = first;
	for (; h1; h0=h1, h1=h1->next, pos++)
	{
		if (h1->lelement==e)
		{	
			if (h1 == last) last = h0;
			h0->next = h1->next;
			delete h1;
			size--;
			return pos;
		}
	}
	return LIST_FAULT_VAL;
}
//----------------------------------------------------------------------------
 template<class T> inline int List<T>::ChangeElement(int pos, const T& new_e)
{
	if (pos > size || pos < 1 || !first ) return LIST_FAULT_VAL;

   	int		i=1;
   	ListElement<T>* h=first; 

   	for (; i<pos; h=h->next, i++) ;
	h->lelement=new_e;
	return i; 
}
//----------------------------------------------------------------------------
 template<class T> inline int List<T>::ChangeElement(T old_e, const T& new_e)
{
	if (!first) return LIST_FAULT_VAL;

	int pos=1;
   	for (ListElement<T>* h=first; h ; h=h->next, pos++) 
	{
		if (h->lelement==old_e)
		{
			h->lelement=new_e;
			return pos;
		}
	}
	return LIST_FAULT_VAL;
}
//----------------------------------------------------------------------------
 template<class T> inline bool List<T>::IsIn(T e) 
{
	if (!first) return false;
	if (first->lelement == e || last->lelement == e) return true;

	for (ListElement<T>* h=first; h != NULL; h=h->next)
		if (h->lelement == e) return true;
	return false;
}
//----------------------------------------------------------------------------
 template<class T> inline int List<T>::GetPos(char* e) 
{
	if (!first) return LIST_FAULT_VAL;

	int	pos=1;
	for (ListElement<T>* h=first; h != NULL; h=h->next, pos++)
		if (strcmp(h->lelement.GetName(),e) == 0) return pos;
	return LIST_FAULT_VAL;
}
//----------------------------------------------------------------------------
 template<class T> inline int List<T>::GetPos(T e) 
{
	if (!first) return LIST_FAULT_VAL;

	int	pos=1;
	for (ListElement<T>* h=first; h != NULL; h=h->next, pos++)
		if (h->lelement == e) return pos;
	return LIST_FAULT_VAL;
}
//----------------------------------------------------------------------------
 template<class T> inline T& List<T>::GetElement(T le) 
{
	for (ListElement<T>* h=first; h; h=h->next)
       	    if (h->lelement == le)  return h->lelement;
	T* help = new T(LIST_FAULT_VAL);
	return *help;
}
//----------------------------------------------------------------------------
 template<class T> inline T& List<T>::GetElement(int pos) 
{
	if (pos < 1 || pos > size || !first )  
	{
		T* help = new T(LIST_FAULT_VAL);
	        return *help;
	}

	ListElement<T>* h=first;
	for (int i=1; i<pos; i++) h=h->next;
       	return h->lelement;
}
//----------------------------------------------------------------------------
 template<class T> inline T& List<T>::GetElement(char* e) 
{
	if (!first) 
	{
		T* help = new T(LIST_FAULT_VAL);
	        return *help;
	}

	for (ListElement<T>* h=first; h; h=h->next)
		if (strcmp(h->lelement.GetName(),e) == 0) return h->lelement;
	T* help = new T(LIST_FAULT_VAL);
	return *help;
}
//----------------------------------------------------------------------------
 template<class T> inline T* List<T>::GetElementPtr(T e) 
{
	if (!first) return NULL;

	for (ListElement<T>* h=first; h ; h=h->next)
		if (h->lelement == e) return (&(h->lelement));
	return NULL;
}
//----------------------------------------------------------------------------
 template<class T> inline T* List<T>::GetElementPtr(int pos) 
{
	if (pos < 1 || pos > size || !first )  return NULL;

	ListElement<T>* h=first;
	for (int i=1; i<pos; i++, h=h->next) ;
	return (&(h->lelement));
}
//----------------------------------------------------------------------------
 template<class T> inline T* List<T>::GetElementPtr(char* e) 
{
	if (!first) return NULL;

	for (ListElement<T>* h=first; h ; h=h->next)
		if (strcmp(h->lelement.GetName(),e) == 0) return (&(h->lelement));
	return NULL;
}
//----------------------------------------------------------------------------
 template<class T> inline List<T>& List<T>::operator =(const List<T>& list)
{ 	
	if (list.first) 
	{
		Clear(); 
		first 		= last = new ListElement<T>();
	        first->lelement = list.first->lelement;
		last            = first;
		for (ListElement<T>* tmp = list.first->next; tmp; tmp=tmp->next)
		{
	    		last->next           = new ListElement<T>();
	    		last->next->lelement = tmp->lelement;
	    		last                 = last->next;
		}
		size = list.size;
	}
	return *this;
}
//----------------------------------------------------------------------------
 template<class T> inline bool List<T>::operator==(List<T>& list) 
{
	List tmp(list);
	int hit=0;

	if (list.size != size)   return false;
	if (!list.size && !size) return true;

	for (ListElement<T>* h=first; h ; h=h->next) 
	{
		for (ListElement<T>* h1=tmp.first; h1; h1=h1->next)
		{
	            	if (h->lelement == h1->lelement) 
	            	{ 
	            		tmp.DeleteElement(h->lelement); 
	            		hit++; 
	            		break; 
			}
		}
        }
	if (hit == size) return true;
	return false;
}
//----------------------------------------------------------------------------
 template<class T> inline bool List<T>::operator!=(List<T>& list)
{
	return !(*this == list);
}
//############################################################################
#endif
