//
// Diese Klasse ueberprueft die Einhaltung des Parameters fuer
// min-firing. Erst wenn alle Bindings mindestens <min_firing>-mal
// gefeuert haben, darf die Simulation beendet werden.
// Der Master erhaelt dann ein Keyword.
//

#include "MinFiring.h"

extern TransitionList		InfiniteTransitionRefList;   // alle Infinite-Server Transitionen des Netzes
extern TransitionList		ImmediateTransitionRefList;  // alle Immediate-Transitionen des Netzes
extern TransitionList		ExclusiveTransitionRefList;  // alle Exclusive-Server Transitionen des Netzes

MinFiring::MinFiring(int min_firings, Output *out) {

  TransitionListElement *t;
  BindingListElement *b;

  status = SOME_ARE_MISSING;
  min_fire = min_firings;
  master = out;


  if (ImmediateTransitionRefList.size > 0)
    for (t = ImmediateTransitionRefList.first; t; t = t->next)
      for (b = t->lelement.GetBindings()->first; b; b = b->next) {
	bindings.Append(&(b->lelement));
	transitions.Append(t->lelement.GetName());
      }

  if (InfiniteTransitionRefList.size > 0)
    for (t = InfiniteTransitionRefList.first; t; t = t->next)
      for (b = t->lelement.GetBindings()->first; b; b = b->next) {
	bindings.Append(&(b->lelement));
	transitions.Append(t->lelement.GetName());
      }

  if (ExclusiveTransitionRefList.size > 0)
    for (t = ExclusiveTransitionRefList.first; t; t = t->next) 
      for (b = t->lelement.GetBindings()->first; b; b = b->next) {
	bindings.Append(&(b->lelement));
	transitions.Append(t->lelement.GetName());
      }

  wait = bindings.Size() * min_fire;
}


MinFiring::~MinFiring() {

  int i;
//  int min_schalt = 0;

  if (bindings.Size() > 0) {
    cout << "Es haben nicht alle Transitionen oft genug geschaltet.\n";
    cout << "Transition      fehlende Schaltungen\n";
    for (i = 0; i < bindings.Size(); ++i)
      cout << transitions[i]+9 << " :        " << min_fire - bindings[i]->FiringCount() << endl;

//    for (i = 0; i < bindings.Size(); ++i)
//      if ((min_fire - bindings[i]->FiringCount()) > min_schalt)
//	min_schalt = min_fire - bindings[i]->FiringCount();
//    cout << "Der langsamtesten Transition fehlen noch " << min_schalt << " Schaltungen.\n";
  }

}


void MinFiring::TestBindings() {

  int max = bindings.Size()-1;
  int cur = 0;
  int missing;

  for(cur = 0; cur <= max; ++ cur) {
    missing = min_fire - bindings[cur]->FiringCount();
    if (missing <= 0) {
      // Dieses Binding kann aus der Liste raus. Es muss nicht mehr geprueft werden
      if (cur < max) {
	// Es ist NICHT der letzte, der rausfliegt
	bindings.Replace(cur, bindings[max]);
        transitions.Replace(cur,transitions[max]);
      }
      bindings.CutDown(max);
      transitions.CutDown(max);
      cur--;
      max--;
    }
    else
      // Das Binding hat noch nicht oft genug gefeuert.
      wait += missing;

  }

  if (max == -1) {
    status = MISSION_FULFILLED;
    master->SendMinFiring();
  }
}



void MinFiring::Check() {

  if (status == MISSION_FULFILLED)
    return;

  wait--;
  if (wait <= 0)
    TestBindings();

}
