extern "C" {
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/signal.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
}

#include <iomanip.h>
#include "Output.h"

#include "portab.h"

#define WATCHDOG -2
#define MINFIRING -1

extern char *hostname;
bool Output::EnableGnuplot = false;

Output::Output (DefinitionList &elist) {

  S_Measure *rew;
  int filedes[2];

  if (elist.size != 0)
    for (DefinitionListElement *e = elist.first; e; e = e->next) {
      rew = new S_Measure(e->lelement.GetName());
      ParseEval(rew, e->lelement.GetVal());
      rew->Parse();
      rewards.Append(rew);
    }

  if (!EnableGnuplot) return;

cout << "GNUPLOT Jucheee!\n",flush;

  if (hostname != NULL) {
    cout << "Zur Ausgabe der Daten mit Gnuplot muss der Slave lokal laufen.\n" << flush;
    EnableGnuplot = false;
    return;
  }

  if (rewards.Size() > 1)
    cout << "Achtung: Es wird nur die erste Primitive des ersten Masses geplottet!\n" << flush;

  if ((logfile = fopen ("slave.log","w")) == NULL) {
    perror("Kann die Logdatei fuer gnuplot-Ausgabe nicht oeffnen.");
    EnableGnuplot = false;
    return;
  }

  if (pipe(filedes)) {
    perror ("Kann keine Pipe fuer die gnuplot-Ausgabe erzeugen.");
    EnableGnuplot = false;
    return;
  }
cout << "Pipes sind ok\n" << flush;
  
  switch((gppid = fork())) {
  case 0:
    cout << "Ich bin der Sohn und werde zu gnuplot\n" << flush;
    close(filedes[1]);
    dup2(filedes[0],0);  // stdin umlegen
    execlp("gnuplot", NULL);
    perror("Konnte gnuplot nicht starten.");
    exit(-1); // Der Sohn stirbt
    break;
  case -1:
    perror("Kann keinen Prozess fuer gnuplot abspalten");
    EnableGnuplot = false;
    return;
    break;
  default:
    cout << "ich bin der Vater\n" << flush;
    close(filedes[0]);
    gpfd = filedes[1];
    write(gpfd,"set style lines\nset xlabel \"Zeit\"\n",34);
  }
}


void Output::UpdateGnuplot(PointInTime now, double val) {

  fprintf(logfile,"%f\t%f\n",now,val);
  write (gpfd, "plot \"slave.log\" title \"Ist mir egal!\" with lines\n",50);
}


void Output::ParseEval(S_Measure *rew, Cell *e) {

  CellVal *cv;

  switch (e->GetCellType()) {
  case const_val:
    rew->Append(M_CONST);
    cv = e->Calculate();
    if (cv->GetResultType() != numeric) {
      cout << "Leistungsmass '" << rew->GetName() << "' enthaelt eine ";
      cout << "nicht-float Konstante.\n";
      exit(-1);
    }
    rew->Append(cv->GetFloat());
    delete cv;
    break;
  case unaer_op:
    switch (e->GetOp()) {
    case '-': rew->Append(M_UNAER_MINUS); break;
    case '!': rew->Append(M_UNAER_NOT); break;
    default :  cout << "Unbekannte Unaeroperation in " << rew->GetName() << endl;
              exit(-1);
              break;
    }
    ParseEval(rew, e->GetOpd());
    break;
  case binaer_op:
    switch (e->GetOp()) {
    case '+': rew->Append(M_PLUS); break;
    case '-': rew->Append(M_MINUS); break;
    case '*': rew->Append(M_TIMES); break;
    case '/': rew->Append(M_DIV); break;
    case '=': rew->Append(M_EQUAL); break;
    case '>': rew->Append(M_GREATER); break;
    case '<': rew->Append(M_LOWER); break;
    case 'n': rew->Append(M_NOTEQUAL); break;
    case 'g': rew->Append(M_GREATEQ); break;
    case 'l': rew->Append(M_LOWEQ); break;
    case '&': rew->Append(M_AND); break;
    case '|': rew->Append(M_OR); break;
    default : cout << "Unbekannte Binaeroperation in " << rew->GetName() << endl;
              exit(-1);
	      break;
    }
    ParseEval(rew, e->GetLeftOpd());
    ParseEval(rew, e->GetRightOpd());
    break;
  case function:
    cout << "Unzulaessige Funktion ausserhalb eines E[] bzw. P[]-Ausdrucks in ";
    cout << rew->GetName() << endl;
    exit(-1);
    break;
  case rating_cell:
    evals.Append(new Prim((RatingCell *)e));
    rew->Append(M_PRIMITIVE);
    rew->Append(double(evals.Size()-1)); // Index des Primitiv-Ausdrucks
    break;
  default:
    cout << "Unbekannter Cell-Typ in " << rew->GetName() << endl;
    exit(-1);
  }
}


Socket::Socket (DefinitionList &elist,
		int  portnumber,
		char *hostname) : Output(elist) {

  int i;

  if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    perror ("Kann keinen Socket erstellen.");
    exit (-1);
  }

  struct sockaddr_in address;

  address.sin_family = AF_INET;            // Socket quer durchs Internet
  address.sin_port = htons(portnumber);
  if (hostname == NULL)
    address.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
  else {
    struct hostent *entry = gethostbyname(hostname);
    if (entry == NULL) {
      perror ("Finde keine Adresse fuer den Host des Masters");
      exit(-1);
    }
    memcpy (&address.sin_addr, entry->h_addr, entry->h_length);
  }
  if (connect(fd, (struct sockaddr *) &address, sizeof(address))) {
    perror ("Kann nicht an den Master connecten.");
    exit (-1);
  } 
  
  // Ab hier ist fd ein gueltiger FileDescriptor
    

  WriteString ("#MEASURES");
  WriteInt(rewards.Size());
  WriteString("#PRIMITIVES");
  WriteInt(evals.Size());

  for (i = 0; i < rewards.Size(); ++i)
    WriteReward(rewards[i]);

  watchdog = 20;
}



Socket::~Socket() {
  close(fd);
  if (EnableGnuplot)
    sigsend(P_PID, gppid, SIGKILL);
}


void Socket::WriteString(char *string) {

  register int len = strlen(string)+1; // +1 damit der \0 gesendet wird

  if (write(fd, string, len) != len) {
    perror ("Konnte String nicht vollstaendig ausgeben!");
    exit(-1);
  }
}


void Socket::WriteDouble (double value) {

  if (write(fd, &value, sizeof(double)) != sizeof(double)) {
    perror ("Konnte Double nicht vollstaendig ausgeben.");
    exit(-1);
  }
}


void Socket::WriteSample(Sample s) {
  WriteDouble(s.value);
  WriteDouble(s.time);
}


void Socket::WriteInt (int value) {

  int netvalue = htonl(value);
  if (write(fd, &netvalue, sizeof(int)) != sizeof(int)) {
    perror ("Konnte Integer nicht vollstaendig ausgeben.");
    exit(-1);
  }
}


void Socket::WriteReward (Measure *r) {

  int i;
  double *rewdef;

  WriteString("REW_NAME");
  WriteString(r->GetName());
  WriteString("REW_DEFINITION");
  rewdef = r->Export();
  WriteDouble(rewdef[0]);                // Laenge der Definition
  for (i = 1; i <= rewdef[0]; ++i)
    WriteDouble(rewdef[i]);
  WriteString ("REW_END");
  delete rewdef;

}


void Socket::EvalAndOut(PointInTime now, Period delta, int trans_ref) {

  Sample *buffer;
  bool send_something = false;

  for (int i = 0; i < evals.Size(); ++i) {
    if (evals[i]->Eval(now, delta, trans_ref, &buffer)) {
      // Beginn der Stationaeren Phase
      WriteInt (i);
      WriteString ("TRANSIENT");
      WriteDouble(evals[i]->GetTransientTime());
      send_something = true;
    }

    if (buffer != NULL) {
      // Raus mit den Daten
      WriteInt (i);
      WriteString ("TIME");
      WriteDouble(now);

      WriteString ("BUFSIZE");
      WriteInt (PRIM_BUFSIZE);
      WriteString ("DATA");
      for (int t = 0; t < PRIM_BUFSIZE; ++t) {
	WriteSample(*(buffer++));
      }
      WriteString("COMPLETE");
      send_something = true;
      if (EnableGnuplot)
	UpdateGnuplot(now, evals[0]->GetEstimator());
    }
  }
  if (!send_something) {
    if(!(--watchdog)) {
      WriteInt(WATCHDOG);
      WriteString("WDTIME");
      WriteDouble(now);
      watchdog = 20;
    }
  }
  else
    watchdog = 20;
}



void Socket::SendMinFiring() {
  WriteInt(MINFIRING);
}


Stdout::Stdout (DefinitionList &elist, bool tokengame) : Output(elist) {

  if (!tokengame)
    {
      cout << "Die Zwischenergebnisse erfolgen nach allen " << PRIM_BUFSIZE;
      cout << " Samples.\nDer Simulationslauf beginnt:\n";
    }
  count = evals.Size();
}



Stdout::~Stdout() {

  cout << "Die letzen Ergebnisse sind:\n";
  for (int i = 0; i < rewards.Size(); ++i)
    rewards[i]->Result();
    //    cout << '\t' << setw(20) << rewards[i]->GetName() << "\t\t" << rewards[i]->Result() << '\n';

  cout << "\nDieser Sklave ist nun beendet.\n";
  if (EnableGnuplot)
    sigsend(P_PID, gppid, SIGKILL);
}


void Stdout::EvalAndOut(PointInTime now, Period delta, int trans_ref) {

  Sample *buffer, tsum;
  int t;

  for (int i = 0; i < count; ++i) {
    if (evals[i]->Eval(now, delta, trans_ref, &buffer)) {
      // Stationaere Phase beginnt
      cout << i << ": Beginn der stationaeren Phase bei "; 
      cout << evals[i]->GetTransientTime() << '\n';
    }

    if (buffer != NULL) {
      //      cout << "Zwischenergebnis " << setw(20) << evals[i]->GetName() << ": ";
      for ( t = 0; t < 10; ++t)
	tsum = tsum + buffer[t];
      cout << now << '\t' << double(tsum) << '\t' << evals[i]->GetEstimator() << " #" << endl << flush;
    }
  }
}


void Stdout::SendMinFiring() {
  cout << "Alle Bindings haben oft genug geschaltet.\n";
}





