#ifndef _OUTPUT_H
#define _OUTPUT_H

#include "Types.h"
#include "inc_classes.h"
#include "DefinitionList.h"
#include "ComfVector.h"
#include "Prim.h"
#include "S_Measure.h"
#include "Sample.h"


class Output {


protected:
  ComfVector <Prim *> evals;          // Liste der Reward-Measure-Primitiven
  ComfVector <S_Measure *> rewards;   // Liste der Reward-Measures

  void  ParseEval (S_Measure *,Cell *); // trennt den Measure-Ausdruck in Operationen
                                        // und Primitiven
  void UpdateGnuplot(PointInTime now, double val);

  int    gppid;    // Process-ID of GnuPlot
  int    gpfd;     // Filedescriptor fuer Gnuplot's stddin
  FILE   *logfile; // Filehandle fuer Ausgabe der Plotdaten

public:

  static bool EnableGnuplot;

  Output (DefinitionList &elist);     // Erstes Element der Eval-Liste

  virtual void EvalAndOut(PointInTime now, Period delta, int trans_ref) = 0;
  virtual void SendMinFiring() = 0;
};



class Socket : public Output {

  int                    fd;          // Filedescriptor zum Socket oder STDOUT
  int                    watchdog;

  void   WriteInt (int);              // Ausgabe eines Integer auf den Socket in Network-Byte-Order
  void   WriteDouble (double);        // Ausgabe eines Doubles auf den Socket
  void   WriteString (char *);        // Ausgabe eines Strings auf den Socket (incl '\0')
  void   WriteSample (Sample);        // Ausgabe eines Samples (Value, Time)
  void   WriteReward (Measure *);     // Ausgabe eines Measures
public:

  Socket (DefinitionList &elist,      // Erstes Element der Eval-Liste
	  int  portnumber,            // Portnummer fuer Socket-Verbindung
	  char *hostname);            // Name des Master-Hosts

  ~Socket();

  void EvalAndOut(PointInTime now, Period delta, int trans_ref);
  void SendMinFiring();
};



class Stdout : public Output {

  int                 count;

public:

  Stdout (DefinitionList &elist, bool tokengame); // Erstes Element der Eval-Liste


  ~Stdout();

  void EvalAndOut(PointInTime now, Period delta, int trans_ref);
  void SendMinFiring();
};

#endif
  


