#ifndef __PAGE_CC__
#define __PAGE_CC__

#include "Identifier.h"
#include "Textual.h"
#include "TextualList.h"

#include "token_multiset.h"
#include "TokenMultiSet.h"
#include "PlaceHeader.h"

#include "Token.h"

#include "CellVal.h"
#include "Cell.h"

#include "Definition.h"
#include "DefinitionList.h"

#include "Page.h"

//###############################################################################################################
Page::Page()		   : pagename(DEF_TXT_NAME) 			{ parameters = NULL; sub_pages = NULL; }
Page::Page(int)		   : pagename(DEF_TXT_NAME) 			{ parameters = NULL; sub_pages = NULL; }
Page::Page(char* n) 	   : pagename(n)  	    			{ parameters = NULL; sub_pages = NULL; } 
Page::Page(const Page& p)  : pagename(p.pagename)   	
{ 
	if (p.sub_pages) 	sub_pages = new TextualList(*(p.sub_pages)); 
	else		 	sub_pages = NULL;

	if (p.parameters) 	parameters = new DefinitionList(*(p.parameters)); 
	else		 	parameters = NULL;
}
//###############################################################################################################
char*			Page::GetName()						{ return pagename.GetChar();	}
Textual			Page::GetPageName()					{ return pagename;		}
TextualList*&		Page::GetSubPages()					{ return sub_pages;		}
DefinitionList*&	Page::GetParameters()					{ return parameters;		}
bool			Page::LessThan(Page&)					{ return false; 		}
bool			Page::operator==(Page& p) 				{ return (pagename==p.pagename);}
bool			Page::operator!=(Page& p)				{ return !(*this == p); 	}
//---------------------------------------------------------------------------------------------------------------
void			Page::Clear() 		       
{ 
	pagename.Clear();
	if (sub_pages) 	{ sub_pages->Clear();  delete sub_pages;  sub_pages  = NULL; } 
	if (parameters) { parameters->Clear(); delete parameters; parameters = NULL; } 
}
//---------------------------------------------------------------------------------------------------------------
bool			Page::IsSubPage(char* sp)
{
	if (!sub_pages) return false;
	for (TextualListElement* subp=sub_pages->first; subp; subp=subp->next)
		if (subp->lelement == sp) return true;
	return false;
}
//---------------------------------------------------------------------------------------------------------------
Definition*		Page::IsParameter(char* d)
{
	if (!parameters) return NULL;
	for (DefinitionListElement* param=parameters->first; param; param=param->next)
		if (param->lelement.GetVar() == d) return &(param->lelement);
	return NULL;
}
//---------------------------------------------------------------------------------------------------------------
void			Page::AddSubPage(char* sp)
{
	if (!sub_pages)	
	{
		sub_pages = new TextualList(*(new Textual(sp)));
		return;
	}
	sub_pages->UniqueAppend(*(new Textual(sp)));
}
//---------------------------------------------------------------------------------------------------------------
void			Page::AddParameter(const Definition& d)
{
	if (!parameters)	
	{
		parameters = new DefinitionList(d);
		return;
	}
	parameters->UniqueAppend(d);
}
//---------------------------------------------------------------------------------------------------------------
void			Page::AddParameters(const DefinitionList& dl)
{
	if (parameters)	
	{
		delete parameters;
		parameters = NULL;
	}
	parameters = new DefinitionList(dl);
}
//---------------------------------------------------------------------------------------------------------------
Page&			Page::operator =(const Page& p)
{
	Clear();
	pagename  = p.pagename;
	if (p.sub_pages) 	sub_pages = new TextualList(*(p.sub_pages)); 
	else		 	sub_pages = NULL;
	if (p.parameters) 	parameters = new DefinitionList(*(p.parameters)); 
	else		 	parameters = NULL;
	return *this;
}
//###############################################################################################################
#endif
