#ifndef __PLACE_CC__
#define __PLACE_CC__

#include <iostream.h>

#include "Identifier.h"
#include "Textual.h"

#include "token_multiset.h"
#include "TokenMultiSet.h"
#include "PlaceHeader.h"

#include "Token.h"

#include "CellVal.h"
#include "Cell.h"

#include "Place.h"

//begin Simulator
#include "Transition.h"
#include "BindingElement.h"
// die folgenden 2 Zeilen ist nur fuer Place::Print() und Place::PrintPlaceList() wichtig 
// (aus inc_slave_globals.h)
#include "TokenList.h"
extern TokenList		TokenRefList;		     // alle Token des Netzes
//End Simulator

//######################################################################################################################
Place::Place(const Place& p)  		        
	: ptype		(p.ptype),	
	  koc		(p.koc), 
	  cap_result	(p.cap_result)
	  { 
	  	if (p.p_head) 	p_head = p.p_head; 	    
	  	else 		p_head = NULL;
	  	if (p.cap)    	cap    = p.cap->Copy(); 	    
	  	else 		cap    = new ConstVal();
	  } 
//----------------------------------------------------------------------------------------------------------------------
Place::Place()   			  	
	: ptype		(noplacetype),	
	  koc		(nokind),
	  cap_result	(DEF_PLACE_CAP)
	  { 
	 	p_head = NULL;
	  	cap    = new ConstVal();
	  } 
//----------------------------------------------------------------------------------------------------------------------
Place::Place(int)               		
	: ptype		(noplacetype),	
	  koc		(nokind), 
	  cap_result	(DEF_PLACE_CAP)
	  { 
	 	p_head = NULL;
	  	cap    = new ConstVal(); 
	  } 
//----------------------------------------------------------------------------------------------------------------------
Place::Place(char* n, int no, PlaceType pt, KindOfCap kc, const Cell* c)
	: ptype 	(pt),
	  koc		(kc),
	  cap_result	(DEF_PLACE_CAP)
	{
	  	p_head = new PlaceHeader(n, no);
	  	if (c) cap = c->Copy(); 
	  	else   cap = new ConstVal();
	}
//----------------------------------------------------------------------------------------------------------------------
Place::Place(char* n, int no, KindOfCap kc, const Cell* c)
	: ptype 	(noplacetype),
	  koc		(kc),
	  cap_result	(DEF_PLACE_CAP)
	{
	  	p_head = new PlaceHeader(n, no);
	  	if (c) cap = c->Copy(); 
	  	else   cap = new ConstVal();
	}
//----------------------------------------------------------------------------------------------------------------------
Place::Place(char* n, int no, PlaceType pt, KindOfCap kc, const Cell* c, TokenMultiSet& t)
	: ptype 	(pt),
	  koc		(kc),
	  cap_result	(DEF_PLACE_CAP)
	{
	  	p_head = new PlaceHeader(n, no, t);
	  	if (c) cap = c->Copy(); 
	  	else   cap = new ConstVal();
	}
//----------------------------------------------------------------------------------------------------------------------
Place::Place(char* n, int no, KindOfCap kc, const Cell* c, TokenMultiSet& t)
	: ptype 	(noplacetype),
	  koc		(kc),
	  cap_result	(DEF_PLACE_CAP)
	{
	  	p_head = new PlaceHeader(n, no, t);
	  	if (c) cap = c->Copy(); 
	  	else   cap = new ConstVal();
	}
//#######################################################################################################################
PlaceHeader*	Place::GetPlaceHeader()				{ return p_head;					}
PlaceType	Place::GetPlaceType()				{ return ptype;						}
KindOfCap	Place::GetKindOfCap()				{ return koc;  						}
Cell*&	 	Place::GetCapacity()	 			{ return cap;  						}
bool		Place::LessThan(Place&) 			{ return false;    					}
bool		Place::operator!=(Place& p) 			{ return !( *this == p );				}
void		Place::ChangePlaceType(PlaceType pt)		{ ptype = pt; 						}
void		Place::ChangeKindOfCap(KindOfCap k)		{ koc   = k; 						}
void    	Place::ChangeTokenMultiSet(TokenMultiSet* t)    { if (p_head)    p_head->ChangeTokenMultiSet(t->tmset); 	}
//-----------------------------------------------------------------------------------------------------------------------
void    	Place::ChangeCapacity(const Cell* c) 	  	
{ 
	if (c && cap) 
	{ 
		cap->Clear();
		delete cap;  
		cap = c->Copy(); 
	}
}
//-----------------------------------------------------------------------------------------------------------------------
void		Place::Clear() 					
{ 
	if (cap)  	
	{ 
		cap->Clear();
		delete cap;  
		cap        = NULL; 
		cap_result = DEF_PLACE_CAP; 	
	}	
	p_head = NULL; 
}
//----------------------------------------------------------------------------------------------------------------------
char* 		Place::GetName() 	 			
{ 
	if (p_head) return (p_head->GetName()); 
	return NULL; 
}
//----------------------------------------------------------------------------------------------------------------------
TokenMultiSet* 	Place::GetTokenMultiSet()  	
{ 
	if (p_head) return (p_head->GetTokenMultiSet()); 
	else 	    return new TokenMultiSet(); 
}
//----------------------------------------------------------------------------------------------------------------------
int    		Place::GetPlaceNo() 	 	
{ 
	if (p_head) return (p_head->GetPlaceNo());  
	else 	    return 0; 
}		
//----------------------------------------------------------------------------------------------------------------------
Textual		Place::GetPlaceName() 	 	
{ 
	if (p_head) return p_head->GetPlaceName();  
	else 	    return (*(new Textual(DEF_TXT_NAME))); 
}		
//----------------------------------------------------------------------------------------------------------------------
bool		Place::operator==(Place& p)      	
{ 
	if ( p_head && !p.p_head)	return false;
	if (!p_head &&  p.p_head)	return false;

	return ( ptype==p.ptype && *p_head==*(p.p_head) );
}
//----------------------------------------------------------------------------------------------------------------------
Place&	Place::operator =(const Place& p)
{
	Clear();

	ptype      = p.ptype;
	koc        = p.koc;
	cap_result = p.cap_result;

	if (p.cap)    cap    = p.cap->Copy();  else cap    = new ConstVal();
	if (p.p_head) p_head = p.p_head;       else p_head = NULL;

	return *this;
}
//----------------------------------------------------------------------------------------------------------------------
bool    Place::ColorConsistent(int color)
{
	if (color == EMPTY_COLOR) return true;

	switch (ptype)
	{
		case elementary : if (color != NO_COLOR) return false; break;
		case object     : if (color == NO_COLOR) return false; break; 
		default         : PlaceError("ColorConsistent() : placetype is undefined");
	}
	return true;  
}
//----------------------------------------------------------------------------------------------------------------------
bool    Place::ColorConsistent(TokenMultiSet* tms)
{
	if (!tms || !(tms->tmset) || !(tms->tmset->first))        
		return true;

	switch (ptype)
	{
		case elementary : if (tms->tmset->first->tref != NO_COLOR) return false; break;
		case object     : if (tms->tmset->first->tref == NO_COLOR) return false; break; 
		default         : PlaceError("ColorConsistent() : placetype is undefined");
	}
	return true;  
}
//----------------------------------------------------------------------------------------------------------------------
bool    Place::TMSIsInPlace(TokenMultiSet* tms)
{
	TokenMultiSet*  place_tms;

	if (!tms || !tms->tmset || !tms->tmset->first) 	return true;
	if (!p_head)					PlaceError("TMSIsInPlace() : empty place-tms");
	if (!(place_tms = p_head->GetTokenMultiSet()))	PlaceError("TMSIsInPlace() : empty place-tms");	

	if (  place_tms->quant          <   tms->quant)         return false;
	if (!(place_tms->tmset) ||  !(place_tms->tmset->first)) return false;

	for (token_referenz* tr=tms->tmset->first; tr; tr=tr->next)
		if (place_tms->GetQuantity(tr->tref) < tr->quant) return false;
	return true;
}
//----------------------------------------------------------------------------------------------------------------------
bool    Place::EnoughCapacity()
{
	double val      = cap->CalculateFloat();
	int    tmp_res  = DEF_PLACE_CAP;

	if (cap_result != DEF_PLACE_CAP) 
		tmp_res = cap_result;
	else
	{
		tmp_res = int(val);
		if (!(cap->IsPlaceDependent())) 
			cap_result = tmp_res;
	}

	switch (koc)
	{
		case logic       : return (tmp_res == 0?false:true); 
		case quantitativ : if (tmp_res == UNFINITE)      return true; 
				   if (tmp_res <= DEF_PLACE_CAP) return false;
				   if (p_head) return (tmp_res >= int(p_head->GetTokenMultiSet()->quant));
				   return true; 
		default          : return false; 
	}
	return false;
}
//----------------------------------------------------------------------------------------------------------------------
bool    Place::EnoughCapacity(int tms_size)
{
	double val      = cap->CalculateFloat();
	int    tmp_res  = DEF_PLACE_CAP;

	if (cap_result != DEF_PLACE_CAP) 
		tmp_res = cap_result;
	else
	{
		tmp_res = int(val);
		if (!(cap->IsPlaceDependent())) 
			cap_result = tmp_res;
	}

	switch (koc)
	{
		case logic       : return (tmp_res == 0?false:true); 
		case quantitativ : if (tmp_res == UNFINITE) return true; 
				   if (p_head) 
				   	return (tmp_res >= (int(p_head->GetTokenMultiSet()->quant) + tms_size));
				   return (tmp_res >= tms_size);
		default          : return false; 
	}
	return false;
}
//----------------------------------------------------------------------------------------------------------------------
void	Place::PlaceError(char* s)
{
	cout <<"\n";
	cout <<"PlaceError \n";
	cout <<s<<"\n";
	cout <<"\n";
	exit(-1);
}
//#######################################################################################################################

// Begin Simulator
bool Place::CheckCapacity() {
  // Inhalt <= Kapazitaet?

  int place_cap = UNFINITE;

  switch (koc)   // Kind of Capacity
    {
    case logic : 
      return (cap->CalculateFloat() != 0.0); 
    case quantitativ : 
      if ((place_cap = int(cap->CalculateFloat())) == UNFINITE)
	return true; 
      return (place_cap >= (GetTokenMultiSet()->quant));
    default : 
      cerr << __FILE__ <<": unknown kind of capacity " << GetName() << ", Line " << __LINE__;
      exit (-1);
    }
  return false;
}


bool Place::CheckCapacity(Cell *tmset) {
  // Inhalt + tms <= Kapazitaet?

  int place_cap = UNFINITE;
  int ret;
  CellVal *cv1, *cv2;

  switch (koc)   // Kind of Capacity
    {
    case logic : 
      return (cap->CalculateFloat() != 0.0); 
    case quantitativ : 
      if ((place_cap = int(cap->CalculateFloat())) == UNFINITE)
	return true; 
      cv1 = tmset->Calculate();
      cv2 = cv1->Abs();
      ret = int(cv2->GetFloat());
      delete cv1, cv2;
      return (place_cap >= (GetTokenMultiSet()->quant) + ret);
      break;
    default : 
      cerr << __FILE__ <<": unknown kind of capacity " << GetName() << ", Line " << __LINE__;
      exit (-1);
    }
  return false;
}



bool Place::CheckCapacity(Cell *out_tms, Cell *in_tms) {
  // Inhalt + tms1 - tms2 <= Kapaziaet?
  TokenMultiSet*  place_tms         = NULL; 
  int             place_cap         = UNFINITE;
//  BinaerOp*	  result_tms        = NULL;
  CellVal         *cv1, *cv2, *cv3, *cv4, *cv5, *cv6;
  int             ret;

  switch (koc)  // Kind of Capacity
    {
    case logic : 
      return (cap->CalculateFloat() != 0.0);
 
    case quantitativ : 
      if ((place_cap = int(cap->CalculateFloat())) == UNFINITE)
	return true; 
      place_tms  = GetTokenMultiSet();
      cv1 = out_tms->Calculate();
      cv2 = in_tms->Calculate();
      cv3 = new TokenMSVal(place_tms);
      cv4 = (*cv3) + (*cv1);
      cv5 = (*cv4) - (*cv2);
      cv6 = cv5->Abs();
      ret = int(cv6->GetFloat());
      delete cv1,cv2,cv3,cv4,cv5,cv6;

      return (place_cap >= ret);

    default : 
      cerr << __FILE__ <<": unknown kind of capacity " << GetName() << ", Line " << __LINE__;
      exit (-1);
    }
  return false;
}


bool Place::CheckTMS (Cell* t_ms)
{
  // Inhalt >= tms?

  TokenMultiSet*  place_tms = GetTokenMultiSet();
  CellVal *cv = t_ms->Calculate();
  TokenMultiSet   tms       = cv->GetTokenMultiSet();
  delete cv;

  if (!place_tms ) {
    cerr << __FILE__ <<": place-tmset ist NULL " << GetName() << ", Line " << __LINE__;
      exit (-1);
  }
	if (place_tms->quant < tms.quant) 
	  return false;

	if (!(place_tms->tmset) || !(place_tms->tmset->first))	
	  return false;

	if (place_tms->tmset && !tms.tmset) 	 	
	  return true; 

	if (place_tms->tmset->first && !tms.tmset->first )
	  return true;

	for (token_referenz* tr=tms.tmset->first; tr; tr=tr->next)
	  if (place_tms->GetQuantity(tr->tref) < tr->quant) return false;
	return true;
}



bool Place::TMSConsistence(Cell *tmset) {
  // tms und Stelle gleicher Typ?
  ResultType rt = tmset->GetResultType();
  switch (ptype)  // Place-Type
    {
    case elementary : 
      if ((rt != e_mset_expr) && (rt != c_function)) 
	return false; 
      break;
    case object : 
      if ((rt != p_mset_expr) && (rt != c_function)) 
	return false; 
      break; 
    default : 
      cerr << __FILE__ <<": no consistence between place and token-multiset" << GetName() << ", Line " << __LINE__;
      exit (-1);

    }
  return true;  
}



void Place::AddTMS(const TokenMultiSet& tms) {

  TokenMSVal t1(tms);
  TokenMSVal t2(GetTokenMultiSet());
  CellVal    *t3 = t1 + t2;
  ChangeTokenMultiSet(&(t3->GetTokenMultiSet()));
  delete t3;

#ifndef NOOPTIMIZE
  influence();
#endif
}


void Place::SubTMS(const TokenMultiSet& tms) {

  TokenMSVal t1(GetTokenMultiSet());
  TokenMSVal t2(tms);
  CellVal    *t3 = t1 - t2;
  ChangeTokenMultiSet(&(t3->GetTokenMultiSet()));
  delete t3;

#ifndef NOOPTIMIZE
  influence();
#endif
}


void Place::influence() {

  int i;

  for (i = 0; i < can_disable_t.Size(); ++i)
    can_disable_t[i]->MaybeInfluenced();

  for (i = 0; i < can_disable_b.Size(); ++i)
    can_disable_b[i]->MaybeDisabled();

  for (i = 0; i < can_enable_t.Size(); ++i)
    can_enable_t[i]->MaybeInfluenced();

  for (i = 0; i < can_enable_b.Size(); ++i)
    can_enable_b[i]->MaybeEnabled();

  for (i = 0; i < can_mod_t.Size(); ++i)
    can_mod_t[i]->MaybeInfluenced();

  for (i = 0; i < can_mod_b.Size(); ++i)
    can_mod_b[i]->MaybeInfluenced();

  for (i = 0; i < changes_delay_t.Size(); ++i)
    changes_delay_t[i]->MaybeInfluenced();

  for (i = 0; i < changes_delay_b.Size(); ++i)
    changes_delay_b[i]->MaybeDelayChanged();
}


void Place::GetModifyCapList (ComfVector<Place *>& liste) {
  liste.Append (can_cap_mod);
}


ComfVector<Transition*>& Place::GetCanDisableTransition() {
  return can_disable_t;
}

ComfVector<BindingElement*>& Place::GetCanDisableBinding() {
  return can_disable_b;
}


void Place::Add2CanMod(Transition *t, BindingElement *b) {
  can_mod_t.Append(t);
  if (b)
    can_mod_b.Append(b);
}
 

void Place::Add2CanMod(ComfVector<Transition*>& tl, ComfVector<BindingElement*>& bl) {
  can_mod_t.Append(tl);
  can_mod_b.Append(bl);
}

void Place::Add2CanEnable(Transition *t, BindingElement *b) {
  can_enable_t.Append(t);
  if (b)
    can_enable_b.Append(b);
}

void Place::Add2CanDisable(Transition *t, BindingElement *b) {
  can_disable_t.Append(t);
  if (b)
    can_disable_b.Append(b);
}

void Place::Add2ChangesDelay(Transition *t, BindingElement *b) {
  changes_delay_t.Append(t);
  if (b) 
    changes_delay_b.Append(b);
}

void Place::Add2ChangesCapacity(Place *p) {
  can_cap_mod.Append(p);
}



void Place::Print (ostream& out) {

//  if (GetTokenMultiSet()->quant) {
// Das obere if wuerde Stellen ohne Marken unterdruecken

    out << "Place: " << GetName() << " #" << GetTokenMultiSet()->quant << "  ";
    if (GetTokenMultiSet()->size > 0) {
      token_referenz *l = GetTokenMultiSet()->tmset->first;
      while (l) {
	if (l->quant > 1)
	  out << l->quant << "* ";
	out << TokenRefList.GetElement(l->tref).GetName();
	out << TokenRefList.GetElement(l->tref).GetVersion().GetChar();
	out << TokenRefList.GetElement(l->tref).GetStep().GetChar() << "\n";
	l = l->next;
      }
    }
    else cout << endl;
}


//  Das ist Marios alte Version.
//  out << "Place: " << GetName() << "   Fuelle: " << GetTokenMultiSet()->quant << "\n";
//  if (GetTokenMultiSet()->size == 0)
//    cout << "    voellig leer\n";
//  else {
//    token_referenz *l = GetTokenMultiSet()->tmset->first;
//    while (l) {
//      out << "  Key: " << l->tref << " Anzahl : " << l->quant << "\n";
//      l = l->next;
//    }
//  }

// End Simulator


// fuer das Tokengame: K.S.
void Place::PrintMultiSet (ostream& out) 
  {
    int not_firsttime= 0;

    char* place_name= (char*)(GetName()+10);
    out << place_name << " ";
    // place_name holen und ausgeben (ohne "\__ROOT__\")

    if (GetTokenMultiSet()->quant) 
      {// mindestens ein Element in der Menge?

	token_referenz *l = GetTokenMultiSet()->tmset->first;
	while (l) 
	  {// fuer jedes Element der Menge tue

	    if (not_firsttime++)   
	      out << "+";
	    // Formattierung

	    int anzahl= l->quant;
	    // Anzahl der Marken eines Parts

	    char* part_name= TokenRefList.GetElement(l->tref).GetName();
	    if (part_name) 
	      {// Existiert ein part_name
		
		if (anzahl>1)
		  out << l->quant << "'"; 
		// Formattierung

		out << part_name;
		// part_name ausgeben

		char* part_version= TokenRefList.GetElement(l->tref).GetVersion().GetChar();
		if (!strcmp(part_version, STDVERSION))
		  out << ".";
		else
		  out << "[" << part_version << "]";
		// part_version ausgeben

		char* part_step= TokenRefList.GetElement(l->tref).GetStep().GetChar();
		out << part_step;
		// part_step ausgeben
	      }
	    else if (anzahl>0)
	      out << anzahl;
	    // Anzahl der Marken ausgeben

	    l = l->next;
	  }
	out << "\n";
      }
    else
      out << "0\n";
  }

#endif 

