#ifndef __PLACE_H__
#define __PLACE_H__

#include "Types.h"

// Begin Simulator
#include "ComfVector.h"
class Transition;
class BindingElement;
// End Simulator

//######################################################################################
class Place
{
	PlaceHeader*	p_head;			// name and tms

	PlaceType	ptype;
	KindOfCap	koc;
	Cell*		cap;			// capacity
	int		cap_result;

// Begin Simulator
        ComfVector<Place *>           can_cap_mod;
	ComfVector<Transition *>      can_disable_t, can_enable_t, can_mod_t, changes_delay_t;
	ComfVector<BindingElement *>  can_disable_b, can_enable_b, can_mod_b, changes_delay_b;
	void            influence();
// Begin Simulator

	//-----------------------------------------------------------------------------
	// 	Error	
	//-----------------------------------------------------------------------------
	void		PlaceError(char*);
public:
	Place();
	Place(int);
	Place(const Place&);
	Place(char*, int, PlaceType, KindOfCap, const Cell*);
	Place(char*, int,            KindOfCap, const Cell*);
	Place(char*, int, PlaceType, KindOfCap, const Cell*, TokenMultiSet&);
	Place(char*, int,            KindOfCap, const Cell*, TokenMultiSet&);
	~Place() { Clear(); } 

	//-----------------------------------------------------------------------------
	// 	Delete	
	//-----------------------------------------------------------------------------
	void     	Clear();

	//-----------------------------------------------------------------------------
	// 	"Getten" 
	//-----------------------------------------------------------------------------
	int		GetPlaceNo();
	Cell*&   	GetCapacity();
	KindOfCap	GetKindOfCap();
	PlaceType	GetPlaceType();
	Textual  	GetPlaceName();
	PlaceHeader*	GetPlaceHeader();
	TokenMultiSet* 	GetTokenMultiSet();

	//-----------------------------------------------------------------------------
	// 	Modifizieren 
	//-----------------------------------------------------------------------------
	void		ChangeTokenMultiSet(TokenMultiSet*);
	void     	ChangePlaceType(PlaceType);
	void	 	ChangeKindOfCap(KindOfCap);
	void     	ChangeCapacity(const Cell*);

	//-----------------------------------------------------------------------------
	// 	Operatoren 
	//-----------------------------------------------------------------------------
	Place&	 	operator =(const Place&);
	bool	 	operator==(Place&);
	bool	 	operator!=(Place&);

	//-----------------------------------------------------------------------------
	// 	Listen-Operatoren
	//-----------------------------------------------------------------------------
	static	bool	LessThan(Place&);
	char*		GetName();

	//-----------------------------------------------------------------------------
	// 	Consistence/Cap/TMS-Checks
	//-----------------------------------------------------------------------------
	bool		TMSIsInPlace(TokenMultiSet*);

	bool		ColorConsistent(TokenMultiSet*);
	bool		ColorConsistent(int);

	bool		EnoughCapacity();
	bool		EnoughCapacity(int);

// Begin Simulator
	bool               CheckCapacity();               // Inhalt <= Kapazitaet?
	bool               CheckCapacity(Cell *);         // Inhalt + tms <= Kapazitaet?
	bool               CheckCapacity(Cell *, Cell *); // Inhalt + tms1 - tms2 <= Kapaziaet?
	bool               CheckTMS (Cell *);             // Inhalt >= tms?
	bool               TMSConsistence(Cell *);        // tms und Stelle gleicher Typ?
        void               GetModifyCapList (ComfVector<Place *>&);
	void               Print (ostream &);
	void               PrintMultiSet (ostream &);
	ComfVector<Transition *>&      GetCanDisableTransition();
	ComfVector<BindingElement *>&  GetCanDisableBinding();

	void            AddTMS(const TokenMultiSet&);     // Einen Tokenmultiset aufnehmen (Keine Kap.Pruefung!)
	void            SubTMS(const TokenMultiSet&);     // TokenMS abziehen (Keine Bestandspruefung !!!)

        // Alle Add2...  erzeugen die Referenzlisten fuer die Verfolgung der 
        // markierungsabhaengigen Ausdruecke
	void            Add2CanMod(Transition *, BindingElement *);
	void            Add2CanMod(ComfVector<Transition *>&, ComfVector<BindingElement *>&);
	void            Add2CanEnable(Transition *, BindingElement *);
	void            Add2CanDisable(Transition *, BindingElement *);
	void            Add2ChangesDelay(Transition *, BindingElement *);
	void            Add2ChangesCapacity(Place *);


// End Simulator
};
//######################################################################################
#endif
