#include <iomanip.h>

#include "Prim.h"


// Cut-and-Paste aus TimeNET::sim_slave.c
// War einfacher, als das ganze sim_spec_slave.o einzubinden
extern "C" {
#include <malloc.h>

  char *er_malloc (int bytes) {
    char *pointer;
    if (!bytes)
      return (NULL);
    pointer = (char *)malloc(bytes);
    if (pointer == NULL)
      {
	printf ("Not enough memory for %d bytes\n", bytes);
	exit (FAIL);
      }
    return pointer;
  }
}


Prim::Prim (RatingCell *expr) {

  eval = expr;
  measure_info = Init_One_DetInitTrans();
  buf_index = 0;
  end_of_transient = 0.0;
  combine_count = 0;

  if (eval->GetCellType() != rating_cell) {
    cout << "Leistungsmass ist nicht RatingCell!" << endl;
    exit(-1);
  }
  switch (((RatingCell *)eval)->GetRatingType()) {
  case no_rt:
    cout << "Fataler Fehler: Leistungsmass ist nicht RatingCell.\n";
    exit(-1);
    break;
  case prob:
  case expect:
    Type = REGULAR;
    break;
  case bayes_p: break;
  case bayes_e:break;
    Type = BAYES;
    break;
  case through_p :
    Type = THROUGH_PUT;
    break;
  default:
    cout << "Unbekannter Typ als Leistungsmass\n";
    exit(-1);
  }
}


bool Prim::Eval (PointInTime now, Period delta, int trans_ref, Sample **data) {

  bool   begin_of_data = false;
  Sample sample;
  double trans_test_val;

  *data = NULL;

#ifdef _DEBUG
  if (eval->GetCellType() != rating_cell) {
    cout << "Leistungsmass ist nicht RatingCell." << endl;
    exit(-1);
  }
#endif

  switch (Type) {
  case REGULAR:
    sample.value = eval->CalculateFloat() * delta;
    sample.time = delta;
    break;
  case BAYES:
    if (eval->GetCell2()->CalculateFloat()) {
      sample.value = eval->GetCell1()->CalculateFloat() * delta;
      sample.time = delta;
    }
    else {return false; } // Bedingung fuer Bayes ist falsch, daher keine weitere Auswertung
    break;
  case THROUGH_PUT:
    if (trans_ref == int(eval->CalculateFloat()))
      sample.value = 1.0;
    else
      sample.value = 0.0;
    sample.time = delta;
    break;
  default:
    cout << "Unbekannter Typ als Leistungsmass in Prim::Eval\n";
    exit(-1);
  }

  // Hat entweder das alte oder das neue Sample eine Nullzeit (haben etwa doch 
  // zwei zeitbehaftete Transitionen gleichzeitig geschaltet?), muessen wir 
  // zusammenfassen, damit ein Schaetzer berechnet werden kann.
  if ((old_sample.time == 0.0) || (sample.time == 0.0)) {
    //    cout << "N" << flush;
    old_sample = old_sample + sample;
    return false;
  }

  if (Type == THROUGH_PUT) {
    if (old_sample.value == 0.0) {
      // Impulsmasse werden speziell zusammengefasst
      // Es werden alle Samples zwischen zwei Schaltungen der Transition
      // zusammengefasst.
      old_sample = old_sample + sample;
      sample.value = sample.time = 0.0;
      if (old_sample.value == 0.0)
        return false;
    }
#ifdef _DEBUG
    if (old_sample.value < 1.0 || double(int(old_sample.value)) != old_sample.value) {
      cout << "TP-Value ist eine nicht-natuerliche Zahl > 0 in Prim::Eval()\n";
      exit(-1);
    }
#endif

    trans_test_val = old_sample.time / old_sample.value;
  }

  else {
    // Nur fuer Zustandsmasse
    // Hat das neue Sample den gleichen Wert wie das letzte Sample, 
    // wird zusammengefasst, damit nicht irrtuemlich eine konstante Reihe
    // erkannt wird, nur weil andere Transitionen hektisch Markierungswechsel
    // erzeugen, die fuer die Primitive uninteressant sind.
    // Es werden maximal 50 Samples zusammengefasst (Zaehler ist combine_count)
    // damit eine konstante Folge nicht bis in alle Ewigkeit zusammengefasst wird.
    if ((double(old_sample) == double(sample)) && (combine_count < 50)) {
      // cout << "Z" << flush;
      combine_count++;
      old_sample = old_sample + sample;
      return false;
    }
    trans_test_val = old_sample.value;
    combine_count = 0;
  }

// Vielleicht hier die Ausgabe fuer eine Transiente Simulation ???
// cout << "sample = " << trans_test_val << endl << flush;
// cout << now << '\t' << sample.value << " #" << endl;

  if (measure_info->StatStatus != STEADY_STATE) {
    current_time = now;                                       // globale Variable fuer TimNET::det_init_trans.c
    if (!DetectInitialTransient (measure_info, trans_test_val)){ // Aufruf der TimeNET-Funktion
      old_sample = sample; 
      // cout << 'T' << flush;
      return false;
    }
    else {
      // Zum ersten mal ist die transiente Phase vorbei
#ifdef _DEBUG
      if (end_of_transient != 0) {
	cout << "Fehler der Zusicherung in Prim::Eval() (mehrfaches EndOfTransient)\n";
	exit(-1);
      }
#endif
      end_of_transient = now - delta;
      begin_of_data = true;
    }
  }

  // Jetzt muss die transiente Phase durchschritten sein.
#ifdef _DEBUG
  if (measure_info->StatStatus != STEADY_STATE) {
    cout << "Verletzung der Zusicherung in Prim::Eval\n";
    exit (-1);
  }
#endif
  estimator = estimator + old_sample;
  buffer[buf_index++] = old_sample;
  if (buf_index >= PRIM_BUFSIZE) {
    // Puffer voll? Dann ausgeben
    buf_index = 0;
    *data = buffer;
  }
  old_sample = sample;
  return begin_of_data;
}


double Prim::GetTransientTime () {

  return (end_of_transient);
}



double Prim::GetEstimator() {
  return (double(estimator));
}



PrimType Prim::GetPrimType() {
  return Type;
}



