#ifndef _PRIM_H
#define _PRIM_H


// ******************************************************
// 
// Prim verwaltet ein Reward-Measure-Primitive, d.h. entweder
// ein TP[]-Mass, ein E[]- oder ein P[]-Mass. Der Ausdruck 'eval'
// wird ausgewertet und zwischengespeichert.
// Mit Hilfe der externen C-Funktion wird die Stationaritaet
// der Sample-Reihe gestestet.
//
// *******************************************************

#include "Types.h"
#include "inc_classes.h"
#include "DefinitionList.h"
#include "Sample.h"
extern "C" {
#include "stat_utils.h"
#include "sim_slave_types.h"
#include "det_init_trans.h"

typedef enum {BAYES,              // Leistungsmasse mit bedingten Ausdruecken
	      REGULAR,            // Erwartungswerte oder Wahrscheinlichkeiten
	      THROUGH_PUT         // Mass ueber die Schaltrate einer Transition
	      } PrimType;

  StatInfo   *Init_One_DetInitTrans();
  T_bool     DetectInitialTransient(StatInfo *, double);
  void       DetInitTrans_data_out(StatInfo *);
  void       DetInitTrans_Init(int);
}

// ------------------------------------------------------------
// Globale Variablen zur Anbindung an TimeNETs det_init_trans.c
// ------------------------------------------------------------

extern double   current_time;
extern int	conf_level;
extern double	eps_max;
extern int      StatSema;
extern long	maxsamples;
extern T_bool	trans_detection;


#define PRIM_BUFSIZE 10

class Prim {

  RatingCell       *eval;
  StatInfo         *measure_info;
  Sample            buffer[PRIM_BUFSIZE];
  Sample            estimator;
  Sample            old_sample;
  int               buf_index;
  PointInTime       end_of_transient;
  PrimType          Type;
  int               combine_count;
public:

  Prim (RatingCell *expr);

  bool         Eval (PointInTime now, Period delta, int trans_ref, Sample **data);
  Period      GetTransientTime();
  PrimType    GetPrimType();
  double      GetEstimator();
};


#endif
