#ifndef __SLAVE_CC__
#define __SLAVE_CC__

#include <GetOpt.h>
extern "C" {
#include <signal.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/resource.h>
}

#include "inc_classes.h"
#include "inc_slave_globals.h"
//#include "Output.h"
#include "MinFiring.h"
//#include "portab.h"
#include "stdlib.h"

#include <limits.h>

#include <fstream.h>
// Importe fuer Ein-/Ausgabe-Stroeme (Tokengame)

#define charP char*
#define MAX_LENGTH 256
// maximale Laenge eines Transitionsnamens (Tokengame)
// mit Pfadangabe (z.Bsp.:"\\SubTrans1\\SubTrans2\\...\\Trans")
//###############################################################################################################
//-----------------------------------------------------------------------------------------------------------
//	externe Ausgabefunktionen	
//-----------------------------------------------------------------------------------------------------------
//extern "C" { double   	strtod(const char*, char**); 		}
//extern "C" { long int 	strtol(const char*, char**, int); 	}
//extern "C" { int              getopt(int, char* const*, const char*); }

void  ShowUsage();
void  PrepareEnd();
void  GetTableString(char*);
void  CheckOptions(int, char**);

//-----------------------------------------------------------------------------------------------------------

ofstream   outlist;
bool       interactive= false;    // siehe Parameter -i
bool       tokengame= false;      // siehe Parameter -g
bool       firsttime= true;       // siehe OutputTransition()
double     max,G_modeltime = 0.0;     // Simulationszeit ist global, damit my_exit() sie auslesen kann
Output     *master;               // damit my_exit() zugreifen kann
MinFiring  *fire_control;         // Modul zur Pruefung der Mindestschaltzeiten
                                  // ist globar, damit exit() zugreifen kann

void AddOccurringPlaces(ComfVector<Place *> &list, Cell *expr) {

  if (!expr)
    return;

#ifdef _DEBUG
    int zaehler = 0;
#endif
  PlaceHeader *ph;

  switch (expr->GetCellType()) {
  case const_val:
    break;
  case unaer_op: 
    AddOccurringPlaces (list, ((UnaerOp *)(expr))->GetOpd());
    break;
  case binaer_op:
    AddOccurringPlaces (list, ((BinaerOp *)(expr))->GetLeftOpd());
    AddOccurringPlaces (list, ((BinaerOp*)(expr))->GetRightOpd());
    break;
  case function:
    ph = ((Function*)(expr))->GetPlaceHeader();
    for (PlaceListElement *p = PlaceRefList.first; p; p = p->next) 
      if (p->lelement.GetPlaceHeader() == ph) {
#ifdef _DEBUG
	if (zaehler > 0) {
	  cout << "Slave: 2 Places zu einem Placeheader in 'AddOccurringPlaces' gefunden!\n";
	  exit(-1);
	}
	zaehler = 1;
#endif
	list.Append(&p->lelement);
      }
#ifdef _DEBUG
    if (zaehler != 1) {
      cout << "Slave: Nicht genau einen Place zum Placeheader gefunden in 'AddOccurringPlaces'\n";
      exit (-1);
    }
#endif
    break;
  case rating_cell:
    AddOccurringPlaces (list, ((RatingCell *)(expr))->GetCell1());
    AddOccurringPlaces (list, ((RatingCell *)(expr))->GetCell2());
    break;
  }
}



void make_dependencies(TransitionList &tlist) {
  
  if (tlist.size == 0)
    return;

  int i;

  ComfVector<Place *> PCanMod, PCanEnable, PCanDisable, PChangesDelay, PCanModT, PCanCapMod;

  for (TransitionListElement *t = tlist.first; t; t = t->next) {
    PCanModT.Clear();
    AddOccurringPlaces (PCanModT, t->lelement.GetGlobalCondition());
    for (i = 0; i < PCanModT.Size(); ++i)
      PCanModT[i]->Add2CanMod(&t->lelement, NULL);

    if (t->lelement.GetBindings()->size > 0) {
      for (BindingListElement *b = t->lelement.GetBindings()->first; b; b = b->next) {
	PCanMod.Clear();
	PCanEnable.Clear();
	PCanDisable.Clear();
	PChangesDelay.Clear();
	AddOccurringPlaces (PCanMod, b->lelement.GetLocalCondition());
	AddOccurringPlaces (PChangesDelay, b->lelement.GetDelay()->GetCell());
	if (b->lelement.GetInArcBindRef().size > 0) 
	  for (ArcBindListElement *in = b->lelement.GetInArcBindRef().first; in; in = in->next) {
	    AddOccurringPlaces (PCanMod, in->lelement.GetTokenMultiSetCell());
	    PCanEnable.Append (in->lelement.GetPlace());
	  }

	if (b->lelement.GetOutArcBindRef().size > 0) 
	  for (ArcBindListElement *out = b->lelement.GetOutArcBindRef().first; out; out = out->next) {
	    AddOccurringPlaces (PCanMod, out->lelement.GetTokenMultiSetCell());
	    PCanDisable.Append (out->lelement.GetPlace());
	  }
      
	for (i = 0; i < PCanMod.Size(); ++i)
	  PCanMod[i]->Add2CanMod(&t->lelement, &b->lelement);
	
	for (i = 0; i < PCanEnable.Size(); ++i)
	  PCanEnable[i]->Add2CanEnable(&t->lelement, &b->lelement);
	
	for (i = 0; i < PCanDisable.Size(); ++i)
	  PCanDisable[i]->Add2CanDisable(&t->lelement, &b->lelement);
	
	for (i = 0; i < PChangesDelay.Size(); ++i)
	  PChangesDelay[i]->Add2ChangesDelay(&t->lelement, &b->lelement);
      }
    }
  }
  
  if (PlaceRefList.size <= 0)
    return;

  for (PlaceListElement *p = PlaceRefList.first; p; p = p->next) {
    PCanCapMod.Clear();
    AddOccurringPlaces (PCanCapMod, p->lelement.GetCapacity());
    for (i = 0; i < PCanCapMod.Size(); ++i) {
      PCanCapMod[i]->Add2ChangesCapacity(&p->lelement);
      PCanCapMod[i]->Add2CanMod(p->lelement.GetCanDisableTransition(), p->lelement.GetCanDisableBinding());
    }
  }
}


void print_marking (ostream& output) {
  for (PlaceListElement *p = PlaceRefList.first; p; p = p->next)
    p->lelement.PrintMultiSet(output);
}


void SendAllTransitions()
{
  outlist << flush;
  // Standartausgabe mit schaltfaehigen Transitionen aktualisieren
  // (Nachricht an Agnes senden)

  outlist.close();
  // Schliessen
}


void WaitForTransitionToFire(charP& Tname, int& Bindex, bool& undo)
{
  int FileEndOff;

  char* infile= new char[strlen(netname) + 6];
  sprintf(infile,"%s.fire",netname);
  do
    {
      ifstream FireTrans(infile);
      // Eingabestrom mit der feuernden Transition oeffnen
	  
      FireTrans.seekg(0, ios::end);
      FileEndOff= FireTrans.tellg();
      // FilePointer am Ende der Datei ermitteln

      FireTrans.close();
      // Schliessen

    } while (!FileEndOff);
  // Warten, bis sich der Inhalt der Datei geaendert hat
  // (Warten auf Nachricht von Agnes)

  firsttime= true;
  // setzen fuer OutputTransition()


  ifstream FireTrans(infile);
  // Eingabestrom mit der feuernden Transition oeffnen
	  
  FireTrans.seekg(0, ios::beg);
  // Ab dem Anfang neu hinzugekommener Daten lesen

  char* name= new char[MAX_LENGTH];

  FireTrans >> name;
  // Namen einlesen
      
  bool h_undo= false;
  if (!strcmp(name, "Undo:"))
    {
      FireTrans >> name;
      // Namen einlesen
      h_undo= true;
    }
  int index;
  FireTrans >> index;
  // Binding-Index lesen
  
  FireTrans.close();
  // Datei schliessen

  Tname= name;
  Bindex= index;
  undo= h_undo;

  FireTrans.open(infile, ios::trunc); FireTrans.close();
  // infile loeschen
}


void Fire(charP& Tname, int& Bindex, bool undo)
{
  TransitionListElement *T= NULL;
  
  if (ImmediateTransitionRefList.size > 0)
    for (T = ImmediateTransitionRefList.first; T; T = T->next) 
      {
	if (!strcmp(T->lelement.GetName(),Tname))
	  {
	    if (!undo && !T->lelement.GetGlobalCondition())
	      {// fire && !globale_erfuellt?

		cout << "Slave: the Transition "<< Tname;
		cout << " could not be fired! (global condition is not enabled)";
		cout << endl << flush;
		exit(-1);
	      }
	    // ab hier: undo oder fire&&global_erfuellt

	    int i; BindingListElement *ble;
	    BindingList *bindings= T->lelement.GetBindings();
	    for ((i=0, ble=bindings->first); ((i < bindings->size) && (i != Bindex)); 
		 (ble = ble->next, ++i));
	    // alle bindings traversieren bis binding gefunden
	    if (i < bindings->size)
	      {
		if (undo)
		  ble->lelement.Undo();
		else
		  if (!ble->lelement.IsEnabled()) 
		    {
		      cout << "Slave: the binding-index " << Bindex;
		      cout << " of Transition "<< Tname;
		      cout << " could not be fired! (binding is not enabled)";
		      cout << endl << flush;
		      exit(-1);
		    }
		  else
		    ble->lelement.Go();
	      }
	    return;
	  }
      }
  // Search for ImmTrans to fire
 
  bool found= false;
  if (!found)
    if (InfiniteTransitionRefList.size > 0)
    for (T = InfiniteTransitionRefList.first; T; T = T->next)
      if (!strcmp(T->lelement.GetName(),Tname))
	{
	  if (!T->lelement.GetGlobalCondition())
	    {
	      cout << "Slave: the Transition "<< Tname;
	      cout << " could not be fired! (global condition is not enabled)";
	      cout << endl << flush;
	      exit(-1);
	    }
	  found= true;
	  break;
	}
  if (!found)
    if (ExclusiveTransitionRefList.size > 0)
      for (T = ExclusiveTransitionRefList.first; T; T = T->next)
	if (!strcmp(T->lelement.GetName(),Tname))
	  {
	  if (!T->lelement.GetGlobalCondition())
	    {
	      cout << "Slave: the Transition "<< Tname;
	      cout << " could not be fired! (global condition is not enabled)";
	      cout << endl << flush;
	      exit(-1);
	    }
	    found= true;
	    break;
	  }
  // search for other Trans to fire

  if (found)
    {
      int i; BindingListElement *ble;
      BindingList *bindings= T->lelement.GetBindings();
      for ((i=0, ble=bindings->first); 
	   ((i < bindings->size) && (i != Bindex)); 
	   (ble = ble->next, ++i));
      // alle bindings traversieren bis binding gefunden
      
      if (i < bindings->size)
	{
	  if (undo)
	    ble->lelement.Undo();
	  else
	    if (!ble->lelement.IsEnabled()) 
	      {
		cout << "Slave: the binding-index " << Bindex;
		cout << " of Transition "<< Tname;
		cout << " could not be fired! (binding is not enabled)";
		cout << endl << flush;
		exit(-1);
	      }
	    else
	      T->lelement.GoBinding(&ble->lelement);
	  // was ist mit den Zeitbedingungen beu Undo???
	  // dies ist keine RueckwaertsSimulation!!!
	}
    }
  else
    {
      cout << "Slave: the binding index " << Bindex;
      cout << " of Transition "<< Tname;
      cout << " could not be fired!" << endl << flush;
      exit(-1);
    }
}


void OutputTransition(char* name, ComfVector<int> &iv)
{// Ausgeben einer Transition mit einer
 // index-Liste seiner schaltfaehigen bindings

  if (firsttime)
    outlist << "Transition-List:" << endl;
  // Ueberschrift vor der ersten Transition ausgeben

  firsttime= false;

  outlist << &name[10] << " ";
  // Name ausgeben ohne "\\__ROOT__\\"

  for(int i=0; i < iv.Size(); i++) outlist << iv[i] << " ";
  // jeden binding-index ausgeben 

  outlist << "-1" << endl;
  // Kennzeichnung fuer das Ende der Liste
}


void handle_immediate() 
{
  ComfVector<BindingElement *> bv;   // bv = binding-vector
  ComfVector<int> iv;                // Indizes der schaltfaehigen Bindings
  ComfVector<int> cv;                // Anzahl der Bindings jeder Transition
  ComfVector<double> wv;             // wv = weight-vector
  double zufall;   

  int i= 0;
  while (!i++ || !bv.IsEmpty()) 
    {// das erste Mal reingehen, ansonsten nur, wenn bv voll ist

      bv.Clear();
      wv.Clear();
      cv.Clear();
      // bv, wv, cv leeren zum sammeln schaltfaehiger bindings

      if (ImmediateTransitionRefList.size > 0)
	for (TransitionListElement *t = ImmediateTransitionRefList.first; t; 
             t = t->next)
	  {
	    t->lelement.EnabledList(bv, wv, iv);
	    // schaltfaehige bindings der Transition t ermitteln

	    if (tokengame && !iv.IsEmpty())
	      {
		OutputTransition(t->lelement.GetName(), iv);
		// Transition und binding-indices ausgeben fuer das Tokengame
		cv.Append(iv.Size());
	      }
	    iv.Clear();
	    // Den index-vector leeren fuer die naechste Transition
	  }
  
      if (!bv.IsEmpty())
	{
	  int index = -1;
	  // Summe der Schaltgewichte und gleichverteilte Zufallszahl bestimmen
	  // und dann den Index in wv bestimmen.
	  zufall = rnd()* wv.Fold(VECTOR_PLUS);
	  do 
	    zufall -= wv[++index];
	  while (zufall > 0);

	  if (tokengame)
	    {
	      int tmp= 0;
	      int FastestT;
	      for (FastestT= 0;FastestT<cv.Size();FastestT++)
		{
		  tmp+= cv[FastestT];
		  if (tmp > index) break;
		}
	      tmp-= cv[FastestT];
	      int FastestB= index-tmp;
		  
	      outlist << "Fastest: " << FastestT << " " << FastestB << " ";
	      outlist << "0.0" << endl;

	      SendAllTransitions();
	      // Sendet alle schaltfaehigen immediate Transitionen ab

	      charP T;
	      int B;
	      bool undo;
	      WaitForTransitionToFire(T, B, undo);
	      // Transition empfangen zum feuern
	      
	      Fire(T, B, undo);
	      // Transition feuern

	      char* infile= new char[strlen(netname) + 6];
	      sprintf(infile,"%s.list",netname);
	      outlist.open(infile);
	      // Ausgabestrom mit der Listendatei oeffnen

	      outlist << "Place-List:" << endl;
	      print_marking(outlist);
	      // Stellenbeschreibung des Netzes ausgeben
	    }
	  else
	    bv[index]->Go();
	}
    }
  bv.Clear();
  wv.Clear();
  cv.Clear();
  // bv, wv, cv endgueltig leeren
}


void select_fastest(PointInTime time, Transition *&next, PointInTime &timestep) {

  ComfVector<BindingElement *> bv;   // bv = binding-vector
  ComfVector<int> iv;                // Indizes der schaltfaehigne Bindings
  ComfVector<double> wv;             // wv = weight-vector

  int FastestT= 0;               
  int FastestT_help= 0;               
  int FastestB= 0;               

  Weight value;
  TransitionListElement *t;

  timestep = TOO_LONG;

  if (InfiniteTransitionRefList.size > 0)
    for (t = InfiniteTransitionRefList.first; t; t = t->next)
      {
	if ((value = t->lelement.DelaySample(time)) < timestep) 
	  {
	    timestep = value;
	    next = &(t->lelement);

	    FastestT= FastestT_help;
	    FastestB= t->lelement.PVindex;
	  }
	if (tokengame)
	  {
	    t->lelement.EnabledList(bv,wv,iv);
	    // Liefert alle schaltfaehigen bindings der Transition
		
	    if (!iv.IsEmpty())
	      {
		OutputTransition(t->lelement.GetName(), iv);
		// Transition und binding-indices ausgeben fuer das Tokengame

		FastestT_help++;
	      }

	    bv.Clear();
	    wv.Clear();
	    iv.Clear();
	    // bv, wv, iv leeren zum sammeln schaltfaehiger bindings
	  }
      }

  if (ExclusiveTransitionRefList.size > 0)
    for (t = ExclusiveTransitionRefList.first; t; t = t->next)
      {
	if ((value = t->lelement.DelaySample(time)) < timestep) 
	  {
	    timestep = value;
	    next = &(t->lelement);

	    FastestT= FastestT_help;
	    FastestB= t->lelement.PVindex;
	  }
	if (tokengame)
	  {
	    t->lelement.EnabledList(bv,wv,iv);
	    // Liefert alle schaltfaehigen bindings der Transition
		
	    if (!iv.IsEmpty())
	      {
		OutputTransition(t->lelement.GetName(), iv);
		// Transition und binding-indices ausgeben fuer das Tokengame

		FastestT_help++;
	      }

	    bv.Clear();
	    wv.Clear();
	    iv.Clear();
	    // bv, wv, iv leeren zum sammeln schaltfaehiger bindings
	  }
      }

  if (timestep == TOO_LONG) 
    {
      // Master vom Tod des Netzes berichten und exit
      cout << "Slave: Das Netz ist tot. Alle Immediate sind abgearbeitet und ";
      cout << "es gibt keine schaltfaehige zeitbehaftete Tranisition." << endl;
      cout << "Letzte Markierung:" << endl;
      print_marking(cout);
      exit (-1);
    }

  if (tokengame)	
    {
      outlist << "Fastest: " << FastestT << " " << FastestB << " ";
      outlist << (timestep-G_modeltime)/max << endl;  

      SendAllTransitions();
      // Sendet alle schaltfaehigen Transitionen ab
    }
}


void reset_signal_handlers() {

  //
  // Signalhandler deinstallieren
  //
  if (((long)signal (SIGINT,  SIG_DFL) < 0) ||
      ((long)signal (SIGHUP,  SIG_DFL) < 0) ||
      ((long)signal (SIGQUIT, SIG_DFL) < 0) ||
      ((long)signal (SIGILL,  SIG_DFL) < 0) ||
      ((long)signal (SIGABRT, SIG_DFL) < 0) ||
      ((long)signal (SIGFPE,  SIG_DFL) < 0) ||
      ((long)signal (SIGBUS,  SIG_DFL) < 0) ||
      ((long)signal (SIGSEGV, SIG_DFL) < 0) ||
#ifndef LINUX
      ((long)signal (SIGSYS,  SIG_DFL) < 0) ||
#endif
      ((long)signal (SIGUSR1, SIG_DFL) < 0) ||
      ((long)signal (SIGUSR2, SIG_DFL) < 0) ||
      ((long)signal (SIGPIPE, SIG_DFL) < 0)) {
    perror ("Konnte Signal-Handler nicht deinstallieren.");
    exit(-1);
  }
}


void my_exit (int sig) {

//  reset_signal_handlers();
  switch (sig) {
  case 0:
    cout << "Slave: Maximale Zahl der Samples erreicht. Simulationszeit : " << G_modeltime << endl; break; 
  case SIGINT:
    if (tokengame)
      cout << "End_Tokengame\n";
    else
      cout << "Slave: Simulator abgebrochen. Gesamte Simulationszeit : " << G_modeltime << endl; break;
  case SIGSEGV:
    cout << "Slave: Segmentation Fault. Tut mir leid. Simulationszeit : " << G_modeltime << endl; break;
  case SIGBUS:
    cout << "Slave: Bus Error. Tut mir leid (mb). Simulationszeit : " << G_modeltime << endl; break;
  default:
    cout << "Slave: Signal " << sig << " erhalten. Gesamte Simulationszeit : " << G_modeltime << endl; break;
  }

  long speicherkosten = 0;
  double zeitkosten = 0.0;

  struct rusage ru;
  getrusage(RUSAGE_SELF,&ru);
  speicherkosten = ru.ru_maxrss * getpagesize() / 8 / 1024;
  zeitkosten = ru.ru_utime.tv_sec  + ru.ru_stime.tv_sec + 
              ((ru.ru_utime.tv_usec + ru.ru_stime.tv_usec) / 1000000.0);
  cout << "Speicherbedarf in Kb: " << speicherkosten << endl << flush;
  cout << "Prozessorzeit in sec: " << zeitkosten << endl << flush;

  delete master;
  delete fire_control; // Ausgabe der Mindestschaltzeiten
  
  reset_signal_handlers();
  exit(0);
}

  
void my_foo(int sig)
{
  if (sig != SIGUSR1)
    cout << "Slave: Unusual signal. Continuing simulation.\n";
  cout << "Es geht weiter...\n";
}


void switch2interactive(int sig)
{
  if (sig != SIGUSR2)
    cout << "Slave: Unusual signal in switch2interactive. Ignoring.\n";
  else
    if (interactive)
      interactive = false;
    else
      interactive = true;
}


void init_signal_handlers() {

  //
  // Signalhandler installieren
  //
  if (((long)signal (SIGINT,  my_exit)           < 0) ||
      ((long)signal (SIGHUP,  my_exit)           < 0) ||
      ((long)signal (SIGQUIT, my_exit)           < 0) ||
      ((long)signal (SIGILL,  my_exit)           < 0) ||
      ((long)signal (SIGABRT, my_exit)           < 0) ||
      ((long)signal (SIGFPE,  my_exit)           < 0) ||
      ((long)signal (SIGBUS,  my_exit)           < 0) ||
      ((long)signal (SIGSEGV, my_exit)           < 0) ||
#ifndef LINUX
      ((long)signal (SIGSYS,  my_exit)           < 0) ||
#endif
      ((long)signal (SIGUSR1, my_foo)            < 0) ||
      ((long)signal (SIGUSR2, switch2interactive)< 0) ||
      ((long)signal (SIGPIPE, my_exit)           < 0)) {
    perror ("Konnte Signal-Handler nicht installieren.");
    exit(-1);
  }
  
}


//-----------------------------------------------------------------------------------------------------------
main(int argc, char** argv)
{
  init_signal_handlers();

  double next_time = 0.0;
  Transition* next_transition;

  CheckOptions(argc, argv);     // command-line-flags auswerten
  yyparse();                    // Netz parsen und Objekte aufbauen
  TxtToRef();                   // Referenzen aufloesen und Hierarchie plattmachen
  DefinitionRefList.Clear();    // ein bisschen Platz schaffen	
  TransitionNameRefList.Clear();
  
  DetInitTrans_Init(maxsamples);

#ifndef NOOPTIMIZE
  make_dependencies(ImmediateTransitionRefList); 
  make_dependencies(ExclusiveTransitionRefList);
  make_dependencies(InfiniteTransitionRefList);
#endif

  if (portnumber == 0)
    master = new Stdout(EvalRefList, tokengame);
  // Ausgabe am Terminal
  else
    master = new Socket(EvalRefList, portnumber, hostname);  
  // Ausgabe auf Socket

  fire_control = new MinFiring(min_firings, master); 
  G_modeltime = 0.0;

  if (tokengame)
    {
      max= 0.0;
      if (InfiniteTransitionRefList.size > 0)
	for (TransitionListElement *T = InfiniteTransitionRefList.first; T; T = T->next)
	  {
	    BindingList *bindings= T->lelement.GetBindings();
	    
	    for (BindingListElement *ble=bindings->first; ble; ble = ble->next)
	      {
		BindingElement bl= ble->lelement;
		if (bl.GetDelay()->CalculateFloat() > max) max= bl.GetDelay()->CalculateFloat();
	      }
	  }
      if (ExclusiveTransitionRefList.size > 0)
	for (TransitionListElement *T = ExclusiveTransitionRefList.first; T; T = T->next)
	  {
	    BindingList *bindings= T->lelement.GetBindings();
	    
	    for (BindingListElement *ble=bindings->first; ble; ble = ble->next)
	      {
		BindingElement bl= ble->lelement;
		if (bl.GetDelay()->CalculateFloat() > max) max= bl.GetDelay()->CalculateFloat();
	      }
	  }  
      char* infile= new char[strlen(netname) + 6];
      sprintf(infile,"%s.list",netname);
      outlist.open(infile);
      // Ausgabestrom mit der Listendatei oeffnen
    }
/*
  Transition *t = &(ExclusiveTransitionRefList.first->lelement);
  BindingElement *b = &(t->GetBindings()->first->lelement);
  Place *p = (b->GetInArcBind().first->lelement.GetPlace());
  Cell *c = b->GetInArcBind().first->lelement.GetTokenMultiSetCell();

  long speicherkosten = 0;
  struct rusage ru;
  getrusage(RUSAGE_SELF,&ru);
  speicherkosten = ru.ru_maxrss * getpagesize() / 8 / 1024;
  cout << "Speicherbedarf in Kb: " << speicherkosten << endl << flush;

  cout << t->GetName()<<endl;
  p->Print(cout);

  for (int i = 0; i < 20000; ++i) {
    p->CheckTMS(c);
  }

  speicherkosten = 0;
  getrusage(RUSAGE_SELF,&ru);
  speicherkosten = ru.ru_maxrss * getpagesize() / 8 / 1024;
  cout << "Speicherbedarf in Kb: " << speicherkosten << endl << flush;

  exit(-1);

*/
// Test fuer Speicherverlust

  // Und los mit der Simulationsschleife
  do 
    {
      handle_immediate();         
      // Schalten aller zeitlosen Transitionen

      select_fastest(G_modeltime, next_transition, next_time); 
      //die schnellste zeitbehaftete T suchen

      master->EvalAndOut(next_time, next_time - G_modeltime, 
			 next_transition->GetGlobalTransitionRef());
      G_modeltime = next_time;    
      // Die Zeit vergeht...

      if (tokengame)
	{	
	  charP T;
	  int B;	
	  bool undo;

	  WaitForTransitionToFire(T, B, undo);
	  // Transition empfangen zum feuern
	      
	  Fire(T, B, undo);
	  // Transition feuern
	}	    
      else
	next_transition->Go();      
      // Transition feuern

      fire_control->Check();      
      // Die Mindestschaltzeiten pruefen

      if (tokengame)
	{
	  char* infile= new char[strlen(netname) + 6];
	  sprintf(infile,"%s.list",netname);
	  outlist.open(infile);
	  // Ausgabestrom mit der Listendatei oeffnen

	  outlist << "Place-List:" << endl;
	  print_marking(outlist);

	}
      if (interactive) 
	{
	  print_marking(cout);
	  pause();
	}
    } while (--maxsamples);       
  // War maxsamples = 0, dann laeuft die Schleife jetzt 
  // 'unendlich' lange, da maxsample = -1, -2, ...
  
  delete master;                
  // Damit bei Stdout die Endergebnisse ausgegeben werden			
  my_exit(0);

}	/* End of main */

//---------------------------------------------------------------------------------------------------------------
//	Aufruf ausgeben
void	ShowUsage()
{
	cout <<"\n";
	cout <<"usage : slave [-s <seed>] [-m <Max Samples>] [-f <min_firing>]\n";
	cout <<" [-c <Confidence-Level>] [-t] [-h <Hostname>] [-p <Portnumber>]\n\n";
	cout <<"-s <seed> sets the seed of the random number generator to <seed> (default = 0)\n";
	cout <<"-n <Net Name> reads the name of the net to be simulated (required for tokengame)\n";
	cout <<"-m <Max Samples> sets the maximum number of samples enerated (default = 0 = without limit\n";
	cout <<"-f <min_firing> keeps simulating until every binding has fired at least <min_firing> times (30)\n";
	cout <<"-c <Confidence Level> sets the reliability of the variance estimation (default = 95  (%))\n";
	cout <<"-t switches automatic detection of initial transient phase OFF. (default = ON)\n";
	cout <<"-h <Hostname> sets the host, to connect to for output of result vie socket (default = localhost)\n";
	cout <<"-p <Portnumber> sets the Port for the socket connection (default = no socket but stdout)\n";
	cout <<"-i switches to interactive mode.Each stable marking is printed. Signal 30 (SIGUSR1) for cont.\n";
	cout <<"-g activates the tokengame mode.All enabled transitions are printed. Signal 30 (SIGUSR1) for the next step.\n";
	cout <<"-d displays data during simulation in gnuplot-window. Only the first measure is displayed\n";
	cout <<"\n The Table String is read from stdin\n";
	exit(-1);
}

void PrepareEnd()
{
        RRGErr->PrintError();
        exit (-1);
}


void CheckOptions(int argc, char** argv) {

  GetOpt getopt (argc, argv, "s:n:m:f:h:p:c:tid:g");
  int option_char;
          
  netname = "default";

  // Invokes member function `int operator ()(void);'
  while ((option_char = getopt ()) != EOF)
    switch (option_char)
      {  
      case 's': rnd.generator(new ACG(atoi (getopt.optarg))); break;
      case 'n': netname = getopt.optarg; break;
      case 'm': maxsamples = atoi(getopt.optarg);
                if (!maxsamples) 
                  maxsamples = MAXINT; 
                break;
      case 'f': min_firings = atoi(getopt.optarg);break;
      case 't': trans_detection = false; break;
      case 'h': hostname = getopt.optarg; break;
      case 'p': portnumber = atoi(getopt.optarg); break;
      case 'c': conf_level = atoi(getopt.optarg); break;
      case 'i': interactive = true; break;
      case 'g': tokengame = true; break;
      case 'd': Output::EnableGnuplot = true;break;
      case '?': ShowUsage(); break;
      }
//  GetTableString(netname);
  if (getopt.optind < argc) {
    cout << "Slave: Unbekannte Option: " << argv[getopt.optind] << '\n';
    exit (-1);
  }
}


//---------------------------------------------------------------------------------------------------------------
//		PageStrings holen	
//---------------------------------------------------------------------------------------------------------------
#if 0
//
// YACC/LEX muessen von stdin lesen, nicht vom String.
// Bei Programmstart ueber rsh(1) unter SunOS4 gibt es sonst
// sinnlose Parser-Fehler
//
void GetTableString(char* file_prefix) {
  FILE* fptr = NULL;
  int   char_count = 1;
  char  *filename, *modeldir, *envtemp;
  char   _in_char_, *in_string_ = NULL, *in_string__ = NULL;

  if (file_prefix[0] != '/') {
    // Pfad aus dem Environment holen
    envtemp = getenv(ENV_MODELDIR_NAME);
    if (envtemp == NULL) {
      cout << "Slave: Environment-Variable MODELDIR ist nicht gesetzt.\n";
      exit(-1);
    }
    modeldir = new char[strlen(envtemp)+2];
    sprintf(modeldir, "%s/", envtemp);
  }
  else
    modeldir = "";

  filename = new char[(strlen(modeldir) + strlen(file_prefix) + 7)];
  sprintf(filename, "%s%s.TABLE", modeldir, file_prefix); 

  //--------------------------------------------------------
  //	Zeichen zaehlen
  //--------------------------------------------------------
  if (!(fptr = fopen(filename,"r")))
    {
      delete[] filename;
      filename = new char[strlen(modeldir) + (strlen(file_prefix) + 7)];
      sprintf(filename, "%s%s.table", modeldir, file_prefix);
      fptr = fopen(filename,"r");
    }
  if (!fptr)
    {
      cout <<"Slave: can't open file : "<<filename<<"\n";
      exit(-1);
    }
  while ((fgetc(fptr) ) != EOF ) char_count++;
  fclose(fptr);
  
  //--------------------------------------------------------
  //	Zeichen einlesen
  //--------------------------------------------------------
  in_string_ = NULL;
  in_string__ = NULL;

  in_string_  = new char[char_count+1];
  in_string__ = in_string_;
  
  if (!(fptr = fopen(filename,"r")))
    {
      cout <<"Slave: can't open file :"<<filename<<"\n";
      exit(-1);
    }
  while (( _in_char_ = fgetc(fptr) ) != EOF ) 
    *in_string_++=_in_char_;
  fclose(fptr);
  *in_string_++='\0';
  //--------------------------------------------------------
  //	Zeichen in global string kopieren
  //--------------------------------------------------------
  strcpy(table_string = new char[char_count+1],in_string__);
}
#endif
#endif

