#ifndef __TOKENMULTISET_CC__
#define __TOKENMULTISET_CC__

#include <iostream.h>

#include "token_multiset.h"
#include "TokenMultiSet.h"

//##########################################################################################
inline void	TokenMultiSet::New_token_multiset(const token_multiset* t)
{
	if (t->first)
	{
		size		= 0;
		quant		= 0;
		tmset 	     	= new token_multiset;
		tmset->first 	= new token_referenz;
		tmset->key   	= t->key;

		for (token_referenz* tok=t->first, *tmp=tmset->first; tok; 
				     tok=tok->next, tmp=tmp->next)
		{
			tmp->tref  = tok->tref;
			tmp->quant = tok->quant;
			quant	  += tok->quant;
			size++;
			if (tok->next) 	tmp->next = new token_referenz;
			else		tmp->next = NULL;
		}
	}
	else 	
	{ 
		size  		= 0; 
		quant 		= 0; 
		tmset 		= NULL;
	}
}
//##########################################################################################
TokenMultiSet::TokenMultiSet()				
{
	size  		= 0;
	quant 		= 0;
	tmset 		= NULL;
}

TokenMultiSet::TokenMultiSet(TokenRef tr, Quant q)
{
	tmset 	     	    = new token_multiset;
	tmset->first 	    = new token_referenz;
	tmset->first->tref  = tr;
	tmset->first->quant = q;
	tmset->first->next  = NULL;
	tmset->key   	    = int(tr) + int(q);
	quant	     	    = q;
	size	     	    = 1;
}

TokenMultiSet::TokenMultiSet(const TokenMultiSet& T)
{
	if (T.tmset) New_token_multiset(T.tmset);
	else	     
	{ 
		size  		= 0; 
		quant 		= 0; 
		tmset 		= NULL;
	}
}

TokenMultiSet::TokenMultiSet(const TokenMultiSet* T)
{
	if (T && T->tmset) New_token_multiset(T->tmset);
	else	     
	{ 
		size  		= 0; 
		quant 		= 0; 
		tmset 		= NULL;
	}
}

TokenMultiSet::TokenMultiSet(const token_multiset* t) 
{ 
	if  (t) New_token_multiset(t); 
	else	     
	{ 
		size  		= 0; 
		quant 		= 0; 
		tmset 		= NULL;
	}
}

TokenMultiSet::TokenMultiSet(token_multiset& t) 
{ 
	New_token_multiset(&t); 
}
//##########################################################################################
TokenMultiSet*	TokenMultiSet::Copy()
{
	TokenMultiSet* ret_tms	= new TokenMultiSet();
	if (!tmset) return ret_tms; 

	ret_tms->size  		= size;
	ret_tms->quant 		= quant;
	ret_tms->tmset  	= new token_multiset;
	ret_tms->tmset->key	= tmset->key;
	ret_tms->tmset->first	= NULL;

	if (tmset->first)
	{
		ret_tms->tmset->first = new token_referenz;

		for (token_referenz* tok=tmset->first, *tmp=ret_tms->tmset->first; tok; 
				     tok=tok->next,     tmp=tmp->next)
		{
			tmp->tref  = tok->tref;
			tmp->quant = tok->quant;
			if (tok->next) 	tmp->next = new token_referenz;
			else		tmp->next = NULL;
		}
	}
	return ret_tms;
}
//-------------------------------------------------------------------------------------------
void	TokenMultiSet::Clear()
{
	if (tmset)
	{
		if (tmset->first)
		{
			if (tmset->first->next)
			{
				for (token_referenz* tmp=tmset->first->next; tmp; tmp=tmp->next)
				{
					delete tmset->first;
					tmset->first = tmp;
				}

			}
			delete tmset->first;
		}
		delete tmset;
	}
	tmset = NULL;
	size  = 0;
	quant = 0;
}
//-------------------------------------------------------------------------------------------
void	TokenMultiSet::AddToken(const token_referenz* tr)
{
	if (!tr) return;
	if (!tmset) 
	{
		tmset        = new token_multiset;
		tmset->key   = 0; 
		tmset->first = NULL; 
		quant	     = 0;
		size	     = 0;
	}

	if (!tmset->first)
	{
		tmset->first 		= new token_referenz;
		tmset->first->tref      = tr->tref;
		tmset->first->quant     = tr->quant;
		tmset->first->next	= NULL;
		tmset->key 	        = int(tr->quant) + int(tr->tref);
		quant                 	= tr->quant;
		size			= 1;
		return;
	}
	for (token_referenz* tmp=tmset->first; tmp; tmp=tmp->next)
	{
		if (tmp->tref == tr->tref) 
		{
			tmp->quant += tr->quant; 
			quant	   += tr->quant;
			tmset->key += int(tr->quant);
			return;
		}
	}
	token_referenz* new_tr = new token_referenz;
	new_tr->tref 	       = tr->tref;
	new_tr->quant	       = tr->quant;
	new_tr->next	       = new token_referenz;
	new_tr->next	       = tmset->first;
	tmset->first	       = new_tr;
	quant      	      += tr->quant; 
	tmset->key 	      += int(tr->quant) + int(tr->tref);
	size++;
}
//-------------------------------------------------------------------------------------------
void	TokenMultiSet::SubToken(const token_referenz* tr)
{
	if (!tr) 	   return;
	if (!tmset) 	   return;
	if (!tmset->first) return;

	if (!tmset->first->next)
	{
		if (tmset->first->tref == tr->tref)
		{
			if (tr->quant >= tmset->first->quant)
			{
				delete tmset->first; tmset->first = NULL;
				delete tmset	   ; tmset        = NULL;
				size         = 0;
				quant	     = 0;
			}
			else
			{
				tmset->first->quant -= tr->quant;
				tmset->key          -= int(tr->quant);
				quant		    -= tr->quant;
			}
		}
		return;
	}
	else
	{
		if (tmset->first->tref == tr->tref)
		{
			if (tr->quant >= tmset->first->quant)
			{
				token_referenz* tmp = tmset->first; 
				quant	     -= tmset->first->quant;
				tmset->key   -= int(tmset->first->quant);
				tmset->key   -= int(tmset->first->tref); 
				tmset->first  = tmset->first->next;
				delete tmp;
				size--;
			}
			else
			{
				tmset->first->quant -= tr->quant;
				tmset->key          -= int(tr->quant);
				quant		    -= tr->quant;
			}
			return;
		}
	}

	for (token_referenz* tmp1=tmset->first, *tmp2=tmset->first->next; tmp2;
			     tmp1=tmp1->next,    tmp2=tmp2->next)
	{
		if (tr->tref == tmp2->tref)
		{
			if (tr->quant >= tmp2->quant)
			{
				tmp1->next  = tmp2->next;
				quant	   -= tmp2->quant;
				tmset->key -= int(tmp2->quant);
				tmset->key -= int(tmp2->tref); 
				size--;
				delete tmp2;
			}
			else
			{
				tmp2->quant -= tr->quant;
				quant       -= tr->quant;
				tmset->key  -= int(tr->quant);
			}
			return;
		}
	}
}
//-------------------------------------------------------------------------------------------
Quant	TokenMultiSet::GetQuantity(TokenRef tr)
{
	if (!tmset) 	   return 0;
	if (!tmset->first) return 0;
	for (token_referenz* tmp=tmset->first; tmp; tmp=tmp->next)
		if (tmp->tref == tr) return tmp->quant;
	return 0;
}
//-------------------------------------------------------------------------------------------
bool	TokenMultiSet::IsIn(TokenRef tr)
{
	if (!tmset) 	   return false;
	if (!tmset->first) return false;
	for (token_referenz* tmp=tmset->first; tmp; tmp=tmp->next)
		if (tmp->tref == tr) return true;
	return false;
}
//-------------------------------------------------------------------------------------------
bool	TokenMultiSet::IsIn(TokenRef tr, Quant q)
{
	if (!tmset) 	   return false;
	if (!tmset->first) return false;
	for (token_referenz* tmp=tmset->first; tmp; tmp=tmp->next)
		if ( (tmp->tref == tr) && (tmp->quant == q) ) 
			return true;
	return false;
}
//-------------------------------------------------------------------------------------------
TokenMultiSet& TokenMultiSet::operator =(const TokenMultiSet& T)
{
	Clear();
	if (T.tmset) New_token_multiset(T.tmset);
	else	     
	{ 
		size  	     = 0; 
		quant 	     = 0; 
		tmset 	     = NULL; 
	}
	return *this;
}
//-------------------------------------------------------------------------------------------
bool	       TokenMultiSet::operator==(TokenMultiSet& T)
{
	if ( T.size  !=  size )	return false;
	if (!T.tmset && !tmset) return true;

	if (T.tmset && tmset)
	{
		if (!T.tmset->first && !tmset->first) 
			return true;

		if (T.tmset->first && tmset->first) 
		{
			bool found;
			for (register token_referenz* tmp1=T.tmset->first; tmp1; tmp1=tmp1->next)
			{
				found = false;
				for (register token_referenz* tmp2=tmset->first; tmp2; tmp2=tmp2->next)
				{
					if (tmp1->tref == tmp2->tref && tmp1->quant == tmp2->quant)
					{
						found = true;
						break;
					}
				}
				if (!found) return false;
			}
			return true;
		}
		return false;
	}
	return false;
}
//-------------------------------------------------------------------------------------------
bool	       TokenMultiSet::operator!=(TokenMultiSet& T) { return !(*this == T); }
//###########################################################################################
#endif
