#ifndef __Transition_cc__
#define __Transition_cc__

extern "C" {
#include <strings.h>
}
#include <iostream.h>

#include "Identifier.h"
#include "Textual.h"
#include "TextualList.h"

#include "token_multiset.h"
#include "TokenMultiSet.h"
#include "PlaceHeader.h"

#include "Token.h"

#include "CellVal.h"
#include "Cell.h"

#include "Place.h"

#include "Delay.h"
#include "BindingElement.h"
#include "BindingList.h"

#include "Transition.h"

//###################################################################################################################
Transition::Transition()		    
	  : 	name		(DEF_TXT_NAME),
	    	gc_result	(DEF_COND_RES),
	    	global_ref	(DEF_TRANS_REF)
	  { 
		servertype = Exclusive; 
		bindings   = new BindingList();
		globalcond = new ConstVal();
		initVars();                      // Simulator
	  }
Transition::Transition(int)		    
	  : 	name		(DEF_TXT_NAME),
	    	gc_result	(DEF_COND_RES),
	    	global_ref	(DEF_TRANS_REF)
	  { 
		servertype = Exclusive; 
		bindings   = new BindingList();
		globalcond = new ConstVal();
		initVars();                      // Simulator
	  }

Transition::Transition(const Transition& t) 
	  : 	name		(t.name),
	    	gc_result	(t.gc_result),
	    	global_ref	(t.global_ref)
	  {
		if (!t.bindings) 
		{
			servertype = Exclusive; 
			bindings   = new BindingList();
			globalcond = new ConstVal();
			return;
		}

		servertype = t.servertype;        
		if (t.bindings->size)
		{
// Simulator		if (servertype == Infinite && 
// Simulator		    t.bindings->first->lelement.GetDelay()->GetDelayType() == _DETERMINISTIC)
// Simulator			TransitionError(name);
		}
		bindings = new BindingList(*(t.bindings));
		if (!t.globalcond)  globalcond = new ConstVal();    
		else                globalcond = t.globalcond->Copy();
		initVars();                      // Simulator
	}

Transition::Transition(char* t, ServerType st, const BindingList* bl, const Cell* c) 
	  : 	name		(t),
	    	gc_result	(DEF_COND_RES),
	    	global_ref	(DEF_TRANS_REF)
	  {
		if (!bl) 
		{
			servertype = Exclusive; 
			bindings   = new BindingList();
			globalcond = new ConstVal();
			return;
		}

		servertype = st;        
		if (bl->size)
		{
// Simulator		if (servertype == Infinite && 
// Simulator		    bl->first->lelement.GetDelay()->GetDelayType() == _DETERMINISTIC)
// Simulator			TransitionError(name);
		}
		bindings = new BindingList(*bl);
		if (!c)  globalcond = new ConstVal();    
		else     globalcond = c->Copy();
		initVars();                      // Simulator
	}
//###################################################################################################################
Textual		Transition::GetTransitionName()					{ return name;	 	 	    }
ServerType	Transition::GetServerType()					{ return servertype; 	 	    }
BindingList*	Transition::GetBindings()					{ return bindings; 	 	    }
int		Transition::GetBindingSize()					{ return (bindings->size);  	    }
int		Transition::GetGlobalTransitionRef()				{ return global_ref; 	 	    }
void		Transition::SetGlobalTransitionRef(int gr)			{ global_ref = gr; 	 	    }
char*		Transition::GetName()	 					{ return name.GetChar(); 	    }
bool		Transition::operator!=(Transition& t) 				{ return !(*this==t); 	 	    }
bool            Transition::LessThan(Transition& t) 				{ return (name<t.name);  	    }
//-------------------------------------------------------------------------------------------------------------------
Cell*		Transition::GetGlobalCondition()				
{ 
	if (globalcond) return globalcond; 	 	    
	return new ConstVal(gc_result);
}
//-------------------------------------------------------------------------------------------------------------------
bool		Transition::GlobalCondTrue()	     			
{ 
	if (gc_result != DEF_COND_RES)  
		return (gc_result == 0?false:true);

	double val = globalcond->CalculateFloat();

	if (globalcond->IsPlaceDependent())
	{
		if (!val) 
			return false;
		return true;
	}

	if (!val) gc_result = 0;
	else	  gc_result = 1;

	if (globalcond) 
	{
		globalcond->Clear();
		delete globalcond;
		globalcond = NULL;
	}

	return (gc_result == 0?false:true);
}
//------------------------------------------------------------------------------------------------------------------
void	Transition::Clear()		     
{ 
	name.Clear();
	if (globalcond) { globalcond->Clear(); delete globalcond; globalcond = NULL; }
	if (bindings)   { bindings->Clear();   delete bindings;   bindings   = NULL; }
	global_ref = DEF_TRANS_REF;
	gc_result  = DEF_COND_RES;
}
//------------------------------------------------------------------------------------------------------------------
Transition&	Transition::operator =(const Transition& t) 
{ 
	Clear(); 
	name        = t.name;
	servertype  = t.servertype;
	global_ref  = t.global_ref;
	gc_result   = t.gc_result;

	if (!t.bindings)    bindings   = new BindingList(); else bindings   = new BindingList(*(t.bindings));
	if (!t.globalcond)  globalcond = new ConstVal();    else globalcond = t.globalcond->Copy();
	return *this;
}
//------------------------------------------------------------------------------------------------------------------
bool		Transition::operator==(Transition& t) 
{ 
return ( name==t.name && servertype==t.servertype && *bindings==*(t.bindings) && *globalcond==*(t.globalcond) ); 
}
//------------------------------------------------------------------------------------------------------------------
void		Transition::TransitionError(Textual& tname)
{
	cerr <<"Error in new Transition "<<tname.GetChar();
	cerr <<" : servertype = infinite, delaytype = deterministic\n";
	exit (SERVERTYPE_DELAY_ERROR);
}

//##################################################################################################################

// Begin Simulator

// Erweiterung des Konstruktors
void Transition::initVars() {
	influenced = true;
	PV = NONE;
	firing_time = TOO_LONG;
      }


void Transition::EnabledList(ComfVector<BindingElement *> &bv, ComfVector<double> &wv,
			     ComfVector<int > &iv) 
{// Liefert eine Liste aller Bindings, die schaltfaehig sind (bv) und deren Delays (wv)
 // und eine Liste der Indizes der schaltfaehigen bindings
 
  int index=0;

  if (globalcond->CalculateFloat()) 
    if (bindings->size > 0)
      for (BindingListElement *ble=bindings->first; ble;(ble = ble->next, ++index))
	if (ble->lelement.IsEnabled()) 
	  {
	    bv.Append(&(ble->lelement));
	    wv.Append(ble->lelement.GetDelay()->CalculateFloat());
	    iv.Append(index);
	  }
}


PointInTime Transition::DelaySample(PointInTime time) {

#ifndef NOOPTIMIZE
  // wenn keine Aenderung, dann ersparen wir uns die Berechnung
  if (!influenced)
    return (firing_time);

  // also gut. Schaltzeit neu berechnen
  influenced = NOT_INFLUENCED;
#endif

  switch (servertype) {
  case Exclusive:
    return ComputeESSample(time);
    break;
  case Infinite:
    return ComputeISSample(time);
    break;
  default:
    cout << "DelaySample can only operate on ES or IS-Transitions. \nBut " << GetName() << " has type ";
    cout << servertype << "\n";
    exit (-1);
    break;
  }
}


PointInTime Transition::ComputeESSample (PointInTime time) {

  if (!globalcond->CalculateFloat()) {
    PV = NONE;
    firing_time = TOO_LONG;
  }
  else {
    if (!(PV && PV->IsEnabled())) {  // this war nicht enabled oder PV wurde ungueltig
      ebv.Clear();
      ewv.Clear();
      if (bindings->size > 0)
	for (BindingListElement *ble=bindings->first; ble; ble = ble->next) {
	  if (ble->lelement.IsEnabled()) {
	    ebv.Append(&(ble->lelement));
	    ewv.Append(ble->lelement.GetWeight());
	  }
	}
      if (ebv.IsEmpty()) {
	PV = NONE;
	firing_time = TOO_LONG;
      } 
      else {
	double zufall = rnd() * ewv.Fold(VECTOR_PLUS);
	int index = -1;
	do {
	  index++;
	  zufall -= ewv[index];
	} while (zufall > 0);
	
	PV = ebv[index];
	PVindex= index;
	firing_time = PV->NewESDelaySample(time);
      }
    }
    else   // PV war enabled und ist es auch jetzt
      // Aufruf nur, um die Schaltzeit zu updaten
      firing_time = PV->ESDelaySample(time);
  }
  return (firing_time);
}


PointInTime Transition::ComputeISSample(PointInTime time) {

  if (globalcond->CalculateFloat()) {
    if (PV) {  // Transition war in letzter Markierung bereits enabled
      PointInTime next_firing = TOO_LONG;
      firing_time = TOO_LONG;
      PV = NONE;
      int i= 0;
      if (bindings->size > 0)
	for (BindingListElement *ble = bindings->first; ble; ble = ble->next)
	  if (ble->lelement.IsEnabled()) {
	    if ((next_firing = ble->lelement.ISDelaySample(time)) < firing_time) {
	      firing_time = next_firing;
	      PV = &(ble->lelement);
	      PVindex= i;
	    }
	    i++;
	  }
    }
    else {  // jetzt schaltfaehig geworden
      PointInTime next_firing = TOO_LONG;
      firing_time = TOO_LONG;
      PV = NONE;
      int i= 0;
      if (bindings->size > 0)
	for (BindingListElement *ble = bindings->first; ble; ble = ble->next)
	  if (ble->lelement.IsEnabled()) {
	    if ((next_firing = ble->lelement.NewISDelaySample(time)) < firing_time) {
	      firing_time = next_firing;
	      PV = &(ble->lelement);
	      PVindex= i;
	    }
	    i++;
	  }
    }
  }
  else {   // Globale Condition ist falsch
    PV = NONE;
    firing_time = TOO_LONG;
  }

  return (firing_time);
}


void Transition::Go() {

  firing_time = TOO_LONG;
  influenced = true;

  switch (servertype) {
  case Infinite:
    PV->ISGo();
    break;
  case Exclusive:
    PV->Go();
    PV = NONE; // Damit neue Schaltzeiten ermittelt werden. Nur fuer ES!
    break;
  default:
    cout << "Unbekannter ServerType in Transition::Go().\n";
    exit(-1);
  }

}


void Transition::GoBinding(BindingElement* b) {
  
  firing_time = TOO_LONG;
  influenced = true;
  
  switch (servertype) 
    {
    case Infinite:
      b->ISGo();
      return;
    case Exclusive:
      b->Go();
      PV = NONE; // Damit neue Schaltzeiten ermittelt werden. Nur fuer ES!
      return;
    default:
      cout << "Unbekannter ServerType in Transition::GoBinding().\n";
      exit(-1);
    }
}
// End Simulator

#endif

