#ifndef __Transition_h__
#define __Transition_h__

//##########################################################################################
#include "Types.h"
#include "BindingList.h"
//##########################################################################################


// Begin Simulator
#include <Uniform.h>
extern Uniform rnd;
#include "ComfVector.h"
#define NONE     NULL
// End Simulator


class Transition 
{
	Textual      	name;
	ServerType	servertype;
	BindingList*	bindings;
	Cell*		globalcond;
	short		gc_result;
	int		global_ref;

	//---------------------------------------------------------------------------------
	//	Error	
	//---------------------------------------------------------------------------------
	void            TransitionError(Textual&);

// Begin Simulator
	bool            influenced;       // Flag, ob ein Markierungswechsel diese Trans. beeinflusst hat
	BindingElement* PV;               // Preselection Variable bei ES Transitionen
                                          // schnellstes Binding bei IS Transitionen
	PointInTime     firing_time;      // der vermutliche naechste Schaltzeitpunkt des Bindings PV
        // die folgenden beiden Eintraege sind nur lokale Variablen fuer 
        // ComputeESSample(). Als Membervariablen werden sie jedoch nicht bei jedem
        // Aufruf neu kreiert und es muesste deshalb schneller laufen. 
	ComfVector<BindingElement *> ebv; // Enabled Bindings fuer ES-Transitionen
	ComfVector<Weight>           ewv; // Weights der Enabled Bindings fuer ES-Transitionen

	void            initVars();                     // Erweiterungen im Konstruktor
	PointInTime     ComputeESSample(PointInTime);   // Schaltzeit fuer ES-TRansitionen bestimmen
        PointInTime     ComputeISSample(PointInTime);   // Schaltzeit fuer IS-Transitionen bestimmen
// End Simulator


public:
	Transition();
	Transition(int);
	Transition(const Transition&);
	Transition(char*, ServerType, const BindingList*, const Cell*);
	~Transition() { Clear(); }

	//---------------------------------------------------------------------------------
	//	Delete
	//---------------------------------------------------------------------------------
	void		Clear();

	//---------------------------------------------------------------------------------
	//	Getten/Calculate	
	//---------------------------------------------------------------------------------
	char*		GetName();
	Textual		GetTransitionName();
	ServerType	GetServerType();
	BindingList*	GetBindings();
	int		GetBindingSize();
	bool		GlobalCondTrue();
	Cell*		GetGlobalCondition();
	int		GetGlobalTransitionRef();

	//---------------------------------------------------------------------------------
	//	Modifizieren
	//---------------------------------------------------------------------------------
	void		SetGlobalTransitionRef(int);

	//---------------------------------------------------------------------------------
	//	Operatoren	
	//---------------------------------------------------------------------------------
	Transition&	operator =(const Transition&);
	bool		operator==(Transition&);
	bool		operator!=(Transition&);
	bool		LessThan(Transition&);

// Begin Simulator

	void            EnabledList(ComfVector<BindingElement *>&, ComfVector<double>&, 
				    ComfVector<int>&);
	BindingElement* GetPV() {return PV;}
	int PVindex;
	void            GoBinding(BindingElement* b);
	// fuer das Tokengame: K.S.

	double          DelaySample(PointInTime time);
	void            Go();
	void            MaybeInfluenced()   {influenced = true;};
// End Simulator
};
//#########################################################################################
#endif

