#ifndef __TXTTOREF_CC__
#define __TXTTOREF_CC__

#include "inc_classes.h"
#include "inc_txt_to_ref.h"

//########################################################################################################################
//
//		Fehlerbehandlung	
//
//########################################################################################################################
void    SubErr(char* s1, char* s2, int line)
{
	if (s2) RRGErr = new RRGError(__FILE__, s1, line, NULL, s2);
	else	RRGErr = new RRGError(__FILE__, s1, line);
	PrepareEnd();
}
//########################################################################################################################
//
//		Ersetzungen in RatingCell (typ = through_put)
//
//########################################################################################################################
void	SetTransitionNo(Cell*& cell)
{
	if (!cell) return;

	char* t_name = cell->GetCell1()->Calculate()->GetText();

	if (!t_name) SubErr("can't substitude transitionname in rating-cell", NULL, __LINE__);

	Transition* t = NULL;

	if ( !(t = ExclusiveTransitionRefList.GetElementPtr(t_name)) )
		if ( !(t = ImmediateTransitionRefList.GetElementPtr(t_name)) )
			if ( !(t = InfiniteTransitionRefList.GetElementPtr(t_name)) )
				SubErr("can't substitude transitionname in rating-cell", t_name, __LINE__);

	cell->SetCell2(t->GetGlobalTransitionRef());
}
//########################################################################################################################
//
//		Ersetzungen in Cells
//
//########################################################################################################################
void	SubstitudeInCells(Cell* cell)
{
	if (!cell) return;
	
	PlaceHeader*    ph = NULL;
	char*		pn = NULL;

	switch (cell->GetCellType())
	{
		case function  		: 	pn = cell->GetPlaceHeader()->GetName();

				  		if (SocketPortRefList.size)
				  		{
							for (SocketPortListElement* sp=SocketPortRefList.first; sp; sp=sp->next)
							{
								if (sp->lelement.GetPort() == pn)
								{
									pn = sp->lelement.GetSocket()->GetName();
									break;
								}
							}
				  		}

				  		ph = PlaceHeaderPtrRefList.GetElement(pn); 
				  		if (!ph) SubErr("can't get PlaceHeader", pn, __LINE__);
				  		cell->ChangePlaceHeader(ph);
				  		break;

		case unaer_op  		: 	SubstitudeInCells(cell->GetOpd());   		break;
		case binaer_op 		: 	SubstitudeInCells(cell->GetLeftOpd());
				  		SubstitudeInCells(cell->GetRightOpd()); 	break;

		case rating_cell	: 	if (cell->GetRatingType() == through_p)
				  		{
				  			SetTransitionNo(cell);
				  			break;
				  		}
				  		SubstitudeInCells(cell->GetCell1());
				  		SubstitudeInCells(cell->GetCell2());		break;

		case const_val 		: 	break;
		default			: 	SubErr("unexpected cell in operation-tree", NULL, __LINE__); 	
	}
}
//########################################################################################################################
//
//		Ersetzungen in ArcBindings
//
//########################################################################################################################
void	SubstitudeInArcBindings(const ArcBindList& arcs)
{
	if (arcs.size)
	{
		for (ArcBindListElement* h=arcs.first; h && arcs.size; h=h->next)
			SubstitudeInCells(h->lelement.GetTokenMultiSetCell());
	}
}
//########################################################################################################################
//
//		Ersetzungen in Transitions
//
//########################################################################################################################
void	SubstitudeInTransitions()
{
	TransitionListElement*  t = NULL;
	BindingOfList*		b = NULL;
	ArcBindList		in;
	ArcBindList		out;
	
	if (ImmediateTransitionRefList.size)
	{
		for (t=ImmediateTransitionRefList.first; t ; t=t->next)
		{
			SubstitudeInCells(t->lelement.GetGlobalCondition()); 

			for (b=t->lelement.GetBindings()->first; b; b=b->next)
			{
				in  = b->lelement.GetInArcBindRef();
				out = b->lelement.GetOutArcBindRef();

				SubstitudeInArcBindings (in);
				SubstitudeInArcBindings (out);
				SubstitudeInCells	(b->lelement.GetLocalCondition());
				SubstitudeInCells	(b->lelement.GetDelay()->GetCell());
			}
		}
	}
	if (ExclusiveTransitionRefList.size)
	{
		for (t=ExclusiveTransitionRefList.first; t ; t=t->next)
		{
			SubstitudeInCells(t->lelement.GetGlobalCondition()); 

			for (b=t->lelement.GetBindings()->first; b; b=b->next)
			{
				in  = b->lelement.GetInArcBindRef();
				out = b->lelement.GetOutArcBindRef();

				SubstitudeInArcBindings (in);
				SubstitudeInArcBindings (out);
				SubstitudeInCells	(b->lelement.GetLocalCondition());
				SubstitudeInCells	(b->lelement.GetDelay()->GetCell());
			}
		}
	}
	if (InfiniteTransitionRefList.size)
	{
		for (t=InfiniteTransitionRefList.first; t ; t=t->next)
		{
			SubstitudeInCells(t->lelement.GetGlobalCondition()); 

			for (b=t->lelement.GetBindings()->first; b; b=b->next)
			{
				in  = b->lelement.GetInArcBindRef();
				out = b->lelement.GetOutArcBindRef();

				SubstitudeInArcBindings (in);
				SubstitudeInArcBindings (out);
				SubstitudeInCells	(b->lelement.GetLocalCondition());
				SubstitudeInCells	(b->lelement.GetDelay()->GetCell());
			}
		}
	}
}

//***********************************************************************************************************************/
//
//		Ersetzungen aller noch mit der Klasse Textual bezeichneten Stellen 
//
//***********************************************************************************************************************/
void  TxtToRef()
{
	//---------------------------------------------------------------------------------------------------------------
	//
	//	- In der Transitions-Klasse und in allen mit ihr verbundenen Klassen werden textuelle Stellen-Elemente
	//	  durch Place-Pointer/Positionen aus der PlaceRefList ersetzt
	//
	//---------------------------------------------------------------------------------------------------------------
	SubstitudeInTransitions();

	//---------------------------------------------------------------------------------------------------------------
	//
	//	- Substitutionen in der EvalRefList
	//
	//---------------------------------------------------------------------------------------------------------------
	if (EvalRefList.size)
		for (DefinitionListElement* dle=EvalRefList.first; dle; dle=dle->next)
			SubstitudeInCells(dle->lelement.GetVal());
}
//***********************************************************************************************************************/
#endif
