#ifndef __RGTYPES__
#define __RGTYPES__
//########################################################################################################################

extern "C" {
#include <limits.h>
}

//------------------------------------------------------------------------------------------------------------------------
//						DEFINE's
//------------------------------------------------------------------------------------------------------------------------
#undef  NULL
#define NULL 0

#define	_UNKNOWN	0x00

/*************************************************/
/**********   Markierungs-/Delaytypen	**********/
/*************************************************/

#define _IMMEDIATE	0x01
#define _EXPONENTIAL	0x02
#define _PREEMP_EXP	0x03
#define _DETERMINISTIC	0x04
#define _GENERAL	0x05	
#define _PROBABILITY	0x06	
#define _MIXED     	0x07
#define _ABSORBING	0x08


/*************************************************/
/**********	Markierungszustaende	**********/
/*************************************************/

#define	_COUNTED	0x01 
#define	_INITIAL	0x02 
#define	_VISITED	0x03 
#define	_R_VISITED	0x04 
#define	_LAST_OF_SCC	0x05 
#define	_R_LAST_OF_SCC	0x06 
#define	_Q_SENDED	0x07 
#define	_D_SENDED	0x08 

/*************************************************/
/**********	     Fehler		**********/
/*************************************************/

#define CELL_ERROR                      -3                      // Exit-Wert bei falscher Cell-Operation 
#define CELLVAL_ERROR                   -4                      // Exit-Wert bei falscher CellVal-Operation 
#define TEXTUAL_ERROR                   -5                      // Exit-Wert bei falscher Textual-Konstruktion
#define IDENTIFIER_ERROR                -6                      // Exit-Wert bei falscher Identifier-Konstruktion
#define DELAY_ERROR                     -7                      // Exit-Wert bei Fehlern waehrend der Delay-Berechnung
#define SERVERTYPE_DELAY_ERROR          -8                      // Exit-Wert der Klasse Transition bei 
                                                                // Inifinite-Servertype und Deterministic-Delay

/*************************************************/
/**********	   Defaultwerte		**********/
/*************************************************/

#define DEF_DEL_RES                     -1                      // Default-Ergebniswert fuer Delays
#define DEF_COND_RES                    -1                      // Default-Ergebniswert fuer Bedingungen
#define DEF_PLACE_CAP                   -1                      // Default-Ergebniswert fuer Stellen-Kapazitaeten
#define DEF_TRANS_REF                   -1                      // Default-Transition-Referenz
#define DEF_ES_WEIGHT                    1                      // Default-Werte der Transitions-/Bindinggewichte
#define DEF_IS_WEIGHT                   -1                      // Default-Werte der Transitions-/Bindinggewichte
#define DEF_DELAY_VAL                    1                      // Default-Werte der DelayVal-Klasse
#define DEF_KEY                          0                      // Default-Werte des Key's einer Markierung 
#define DEF_IMM_WEIGHT             	 1                      // Default-Wert des Gewichtes einer Immediate-Tr. 
#define LIST_FAULT_VAL                  -1                      // Default-Ausgabewert fuer nicht definierte Listeneleme
#define DEF_VAR                         "no_definition"         // Default-Definition-Wert der Definition-Klassen
#define DEF_TXT_NAME                    "no_name"               // Default-Name der Textual-Klassen
#define DEF_STRING                      "no_string"             // Default-String der List/Map-Klassen (GetChar())
#define DEF_ERROR_STRING                "undefined"             // Default-String der Error-Klassen 

/*************************************************/
/**********	    Sonstiges		**********/
/*************************************************/

#define PARS				"parser"		// fuer Fehler beim parsen
#define SCAN				"scanner"		// fuer Fehler beim scannen
#define MAX_TREF			USHRT_MAX		// maximale TokenReferenz
#define MAX_QUANT			USHRT_MAX		// maximale Anzahl eines Tokens
#define WILDCARD                        "wildcard"              // Spezifische Werte fuer TokenRef-Angaben
#define STDVERSION                      "0"                     // Spezifische Werte fuer TokenRef-Angaben
#define NO_COLOR                         0                      // Elementarmarken-Referenzwert
#define EMPTY_COLOR                      -1                     // Referenzwert fuer leere MultiSets
#define UNFINITE                        INT_MAX                 // Bezeichnet unbegrenzte Kapazitaet (class Place) 
#define ACTIVE                          '1'                     // Flag ob ein Binding aktiv ist (class Marking)
#define PASSIV                          '0'                     // Flag ob ein Binding passiv ist (class Marking)
#define REDUCED				-1	

//------------------------------------------------------------------------------------------------------------------------
//						TYPEDEF
//------------------------------------------------------------------------------------------------------------------------
typedef	unsigned short	TokenRef;				// Token-Referenz
typedef	unsigned short	Quant;					// Token-Anzahl 
typedef int    		Key; 					// Paramter der Klasse Marking (Key eines Place-Multiset
typedef int    		Number;					// Paramter der Klasse Marking (Nummer der Markierung)
typedef double 		Weight;					// Paramter der Klasse BindingElement

typedef enum { Exclusive, Infinite            } ServerType;	// Transitionstyp-/Bindingtypunterscheidung
typedef enum { logic, quantitativ, nokind     } KindOfCap;	// Kapazitaets-Arten
typedef enum { parallel, sequential           } ExecMode;	// Berechnungsmodi
typedef enum { noplacetype, elementary, object} PlaceType;	// StellenTypen
typedef enum { imm, exc, inf                  } TransitionType;	// Transitions-Typen

/*****************************************************************/
/**********	    Typen der Bewertungsgroessen	**********/
/*****************************************************************/
typedef enum 
{ 		no_rt,
		prob, 
		expect, 
		bayes_p, 
		bayes_e,
		through_p 
} RatingType;	

/*****************************************************************/
/**********	    Operatoren der Klasse Cell		**********/
/*****************************************************************/
typedef enum 
{ 
		plus         = '+', minus           = '-', multiply = '*', 
		divide       = '/', absolut         = '#', contents = '@', 
		equal        = '=', greater         = '>', less     = '<', 
		less_or_equal= 'l', greater_or_equal= 'g', logic_not= '!',
		not_equal    = 'n', empty           = '0', logic_and= '&',
		logic_or     = '|'   
} Operator;

/*****************************************************************/
/**********	    Celltypen der Klasse Cell		**********/
/*****************************************************************/
typedef enum 
{ 
		const_val, 
		unaer_op, 
		binaer_op,  
		function,
		rating_cell
} CellType;

/*****************************************************************/
/**********	    Resulttypen der Klasse Cell		**********/
/*****************************************************************/
typedef enum 
{  
		c_function, 
		a_function, 
		e_mset_expr, 
		p_mset_expr, 
		logic_expr,
		numeric,    
		textual	
} ResultType;

/*****************************************************************/
/**********	    	Rand-Prozess-Status		**********/
/*****************************************************************/
typedef enum 
{ 
		no_process, 
		waiting_for_connection, 
		connected, 
		running, 
		terminated 
} ProcStatus;

//########################################################################################################################

// Begin Simulator

typedef double PointInTime;
typedef double Period;
extern "C" {
#include <values.h>
}
#define TOO_LONG MAXDOUBLE

// End Simulator

#endif
