
#include "error.h"

typedef char* charP;

//-----------------------------------------------------------------------------

void Error::ShowMessage(char* msg, bool center)
{
  occured=true;
  if (!editor) cerr<<msg<<endl;
  else {
    Error* newerror = new Error(*editor);
    newerror->dialog.OpenMsg("ErrorMessage", "Error", editor->Shell(),
                             XmDIALOG_ERROR, Text("Error: ")+msg, false,
                             DialogCB);
    if (!center)
      newerror->dialog.Center(editor->MousePosition());
  }
}

//-----------------------------------------------------------------------------

void Error::DialogCB(Dialog<Error, bool>&, bool, Ident&)
{
  delete this;
}

//-----------------------------------------------------------------------------

void Error::operator()(ErrorType type, void* first, void* second)
{
  Node* node = 0;
  Connector* conn = 0;
  Page* page = 0;
  char* text;
  Ident* ident;
  bool center = true;

  if (occured) return;

 	char buffer[1024];
  ostrstream msg(buffer,1024);

  switch (type) {
  case TooManyStructures :                        // Fehler fr den Translator
    node=NodeP(first);
    msg<<"Not more than one Structure allowed!"; break;

  case NoStructure:
    page=PageP(first);
    msg<<"Can't find Structure"; break;

  case NoStructRef:
    node=NodeP(first);
    msg<< node->Class()->Label() << " " << node->Name() << endl
       << "has no associated Node in Structure"; break;

  case IllegalStructRef:
    node=NodeP(first);
    msg<< node->Class()->Label() << " " << node->Name() << endl
       << " is a " << NodeP(second)->Class()->Label()
       << " in Structure"; break;

  case DefExists:
    node=NodeP(first);
    msg<< "Can't translate Definition " << node->Name() << "," << endl
       << "Name exists!"; break;

  case NoMatchingArc :
    conn=ConnectorP(first);
    msg<< conn->Class()->Label() << " " << conn->Name() << endl
       << " has no associated Arc in Structure"; break;

  case ArcWithoutWeight :
    conn=ConnectorP(first);
    msg<< conn->Class()->Label() << " " << conn->Name() << endl
       << " has no weight or color"; break;

  case CantLoadTemplate:                          // Fehler fr den Includer
    text=charP(first);
    msg<<"Can't load template net "<<text; break;

  case NotATemplate:
    text=charP(first);
    msg<<text<< " is not a template net"; break;

  case IllegalParamType:
    text=charP(first);
    msg<<"Syntax error in type of Parameter "<<text; break;

  case DifferentParamTypes:
    text=charP(first);
    node=NodeP(context);
    if (context)
      msg<<"Can't instanciate template for symbol "<<node->Name()<<":"<<endl;
    msg<<"Different types for Parameter "<<text; break;

  case NoSubtransName:
    msg<<"Name for Substitution Transition required"; break;

  case NameExists:
    ident=IdentP(first);
    node=NodeP(context);
    msg<<"Can't create node "<<*ident<<", name exists!"; break;

  case ParamSyntaxError:
    text=charP(first);
    msg<<"Syntax error in argument for Parameter"<<text; break;

  case AmbigousInstConditions:
    node=NodeP(context);
    if (context)
      msg<<"Can't instanciate template for symbol "<<node->Name()<<":"<<endl;
    msg<<"Ambigous instanciation conditions"; break;

  case Nothing2Insert:
    node=NodeP(context);
    if (context)
      msg<<"Can't instanciate template for symbol "<<node->Name()<<":"<<endl;
    msg<<"All instanciation conditions are false"; break;

  case AmbigousExistConditions:
    ident=IdentP(first);
    node=NodeP(context);
    if (context)
      msg<<"Can't instanciate template for symbol "<<node->Name()<<":"<<endl;
    msg<<"Ambigous existence conditions for node "<<*ident; break;

  case ErrorInShape:
    text=charP(first);
    msg<<"Parse error in shape of "<<text; break;

  case AmbigousConnConditons:
    node=NodeP(first);
    msg<<"Template "<<node->Class()->Name()<<" of node "<<node->Name()<<endl
       <<"has ambigous "<<(second ? "in":"out") <<"put connection conditions";
    break;

  case ConstraintHurt:
    text=charP(first);
    node=NodeP(context);
    if (context)
      msg<<"Can't instanciate template for symbol "<<node->Name()<<":"<<endl;
    msg<<text; break;

  case EmptyParam:
    ident=IdentP(first);
    node=NodeP(context);
    if (context)
      msg<<"Can't instanciate template for symbol "<<node->Name()<<":"<<endl;
    msg<<"You must enter a value for parameter "<<*ident; break;
    
  case ArcNotInTemplate:
    conn=ConnectorP(first);
    node=second ? conn->GetOutput() : conn->GetInput();
    msg<<"Template "<<node->Class()->Name()<<" of node "<<node->Name()<<endl
       <<"doesn't allow "<<(second ? "incoming":"outgoing")<<" arcs"; break;
    
  case MeltConflict:
    conn=ConnectorP(first);
    msg<<"Nodes "<<conn->GetOutput()->Name()<<" and "
       <<conn->GetOutput()->Name()<<endl
       <<" couldn't be connected due to a melting conflict"; break;

  case IllegalArc:
    conn=ConnectorP(first);
    msg<<conn->Class()->Label()<<" "<<conn->Name()<<"is illegal"; break;

  case NoStructRef2:
    node=NodeP(first);
    msg<< node->Class()->Name() << " " << node->Name() << endl
       << "has no associated Node in Structure"; break;

  case IllegalStructRef2:
    node=NodeP(first);
    msg<< node->Class()->Name() << " " << node->Name() << endl
       << " is a " << NodeP(second)->Class()->Name()
       << " in Structure"; break;

  case VarSyntax:
    text=charP(first);
    node=NodeP(context);
    if (context)
      msg<<"Can't instanciate template for symbol "<<node->Name()<<":"<<endl;
    msg<<"Syntax error in Parameter " << text; break;

  case ExprSyntax:
    ident=IdentP(first);
    node=NodeP(context);
    if (context)
      msg<<"Can't instanciate template for symbol "<<node->Name()<<":"<<endl;
    if (ident) msg<<"Syntax error in expression of node "<<*ident;
    else msg<<"Syntax error in parameter values or definitions";
    break;
    
  default: {}
  }
  msg<<ends;
  
  if (editor) {
    if (node) page = node->GetPage();             // Seite, auf der der Fehler
    if (conn) page = conn->GetPage();             // aufgetreten ist, im
    if (page) page->GetNet()->ChangeEditor(*editor, page); // Editor anzeigen

    if (node) {                                   // Fehler an einem Knoten?
      page->Deselect();                           // Besagten Knoten markieren,
      node->Select(true);                         // im Editor anzeigen
      editor->ScrollVisible(node->Mid(), true);   // und in die Mitte scrollen
    center=false;
    }
    if (conn) {                                   // Fehler an einer Kante?
      page->Deselect();                           // Besagte Kante markieren,
      conn->Select(true);                         // im Editor anzeigen
      editor->ScrollVisible(conn->Mid(), true);   // und in die Mitte scrollen
      center=false;
    }
  }
  ShowMessage(buffer, center);                    // Fenster mit Fehlermeldung
}                                                 // erzeugen

//-----------------------------------------------------------------------------





