
#include "includer.h"
#include "netcls.h"

MODULE_NOTIFY(TemplateIncluder)

//-----------------------------------------------------------------------------

bool TemplateIncluder::Init()
{
  Ident netcl = net.Class()->Name();              // In diesen Netzklassen kann
  return popp_library &&                          // das Modul verwendet werden
       ( netcl=="AHCPN_Prime"                     
      || netcl=="AHCPN_Structure"
      || netcl=="AHCPN_Workplan" );
//    || netcl=="HCPN";
}

bool TemplateIncluder::Open(Editor&,Page& page)
{
  Ident netcl = page.Class()->Name();             // In diese Netze kann ein
  return netcl=="AHCPN_Structure"                 // Template eingefgt werden
      || netcl=="AHCPN_Workplan";
//    || netcl=="HCPN";
}

//-----------------------------------------------------------------------------
// Menu wird bei jedem Anklicken eines Meneintrags aufgerufen
//
void TemplateIncluder::Call(Editor& ed, Page&)
{
  editor = &ed;                                   // Editor merken
  lib_copy = popp_library->Copy();                // Kopie von Library in einem
  sel_editor = lib_copy->OpenEditor();            // neuen Editor anzeigen
  sel_editor->Message("Please select template to include");
                                                  // NodeSelector Starten
  NodeSelector::Activate(lib_copy, this, NodeSelTestFunc, NodeSelCallBack);
}

//-----------------------------------------------------------------------------
// Aufrumroutine
//
void TemplateIncluder::Quit()
{
  if (sel_editor) delete sel_editor; sel_editor=0;// Editor lschen
  if (tplnet) delete tplnet; tplnet=0;            // Template-Netz lschen
  if (CurrentDialog()) {                          // und Dialog schlieen
    CloseDialog();
    DestroyDialog();
  }
}

//-----------------------------------------------------------------------------
// IncludeTemplate wird vom NodeSelector nach Auswahl des Templates aufgerufen.
// Es wird nun anhand der Parameter des Templates eine Dialogbox erstellt.
//
void TemplateIncluder::IncludeTemplate(Node* node)
{
  if (!node) { sel_editor=0; Quit(); return; }    // Editor geschlossen?

  tplnet = node->SubPage()->Copy(0);              // Template-Netz kopieren

  NodeSelector::Deactivate(lib_copy);             // NodeSelector schlieen
  delete sel_editor; sel_editor = 0;

  Error error(*editor);
  Page* target = net.GetPage(editor);
  Ident netcl = target->Class()->Name();

  LinkedList<Node*> struc_params, func_params;    // Parameter des Templates
  GetParameters(tplnet, true, struc_params, error); // heraussuchen
  GetParameters(tplnet, false, func_params, error);
  if (error) { Quit(); return; }
/* 
  if (netcl=="AHCPN_Workplan")                    // Beim Arbeitsplan nur die
    FilterNeededForExistence(tplnet, struc_params); // fr Existenzbedinungen
                                                  // ntigen strukturellen
*/                                                // Parameter abfragen
  char structdef[10000];
  strstream sd(structdef, 10000);

  sd<<"[";                                        // Struct-Definition erzeugen
  if (struc_params) {
    sd<<"text(0) \"Name of Instance\" name = \"\"; ";
    sd<<"struct [";
    Params2StructDef(struc_params, sd, false, error);
    sd<<"] \"Structural parameters:\" structural; ";
  }
  if ((netcl=="AHCPN_Workplan" || netcl=="HCPN") && func_params) {
    if (netcl=="HCPN") sd<<"list [";
    sd<<"struct [";
    Params2StructDef(func_params, sd, false, error);
    if (netcl=="HCPN") sd<<"]";
    sd<<"] \"Functional parameters:\" functional";
  }
  sd<<"]"<<ends;
  if (error) { Quit(); return; }                 

  CreateDialog(structdef);
  OpenDialog(*editor, "Template Parameters", "Ok Cancel");
}

//-----------------------------------------------------------------------------
// DialogCB wird vom Dialog nach Eingabe der Parameter aufgerufen
//
void TemplateIncluder::OnDialogButton(Ident& button)
{
  if (button=="Cancel") { Quit(); return; }
  
  Error error(*editor);
  Page* targetpage = net.GetPage(editor);
  NetClass* netcl = targetpage->Class();
  Ident subtransname = charP(Param("name"));
  Position pos(250, 200); // editor->MidPosition(); 

  if (subtransname=="") {                         // Kein Name?
    error(Error::NoSubtransName); /* Quit(); */ return; }

  if (netcl->Name()=="HCPN") {}                   // spter
  else {
    Net tmpnet(netcl);                            // Temporres Netz erstellen
                                                  // Templ. darin instanziieren
    InstanciateTemplate(tmpnet.Prime(), tplnet, subtransname, pos,
      (Struct*) GetParams()->GetMember("structural"),
      (Struct*) GetParams()->GetMember("functional"), 0,0,0,0, error);
    if (error) { Quit(); return; }

    tmpnet.Prime()->Select();
    targetpage->Deselect();
    targetpage->PageCopy(*tmpnet.Prime());        // Und in gewnschte Seite
    targetpage->UpdateHierarchy();                // kopieren
  }
  Quit();                                         // fertig!
}

//-----------------------------------------------------------------------------
// InstanciateTemplate instanziiert ein Template
//
Node* TemplateIncluder::InstanciateTemplate(Page* target, Page* tplnet,
        Ident& name, Position pos, Struct* sparams, Struct* fparams,
        Page* scontext, Page* fcontext, Text* in, Text* out, Error& error)
{
  bool wpl = fparams!=0;
  
  Ident2Text context2;                            // Definitionen und Parameter
  GetExpressionContext(context2, tplnet, sparams, // fr Expression-Auswertung
                       fparams, 0, 0, scontext,   // heraussuchen
                       fcontext, in, out, error);
  if (error) return 0;
  
  
  Page* source;                                   // Zu instanziierendes
  Page* struc = 0;                                // Template und seine
                                                  // Struktur heraussuchen
  source = SelectTemplate(tplnet, context2, wpl, error);
  if (error) return 0;

  if (wpl) struc = SelectTemplate(tplnet, context2, false, error);
  if (error) return 0;

  Ident2Text context;                             // Context nochmal mit Defs
  GetExpressionContext(context, tplnet, sparams, fparams,
         wpl?struc:source, wpl?source:0, scontext, fcontext, in, out, error);
  if (error) return 0;
  
  Node* subtrans = new Node(target->Class()->GetNodeClass("SubTrans"),
                            target, name, pos);   // neue Vertr.trans. erzeugen
  if (subtrans->Name()!=name) {
    error(Error::NameExists, &name); return 0; }

  Page* subpage = subtrans->SubPage();

  Node2Node CopyMap;
  CopyTemplatePage(source, subpage, CopyMap);     // Template kopieren

  CheckConstraints(CopyMap.Defs(), context, error);
  if (error) return 0;                            // Constraints auswerten

  InsertParameters(CopyMap, context, error);      // Parameter einsetzen
  if (error) return 0;

  DeleteTransitions(CopyMap, struc, context, wpl, error);
  if (error) return 0;                            // Transitionen lschen
  
  if (!wpl)
    CheckForDuplicates(subpage, error);           // Existenzbed. eindeutig?
  if (error) return 0;

  CreateSocketNodes(CopyMap, subtrans, struc,     // Sockelknoten erzeugen
                    context, wpl, in, out, error);
  if (error) return 0;

  LayoutSocketNodes(subtrans);                    // Sockelknoten layouten

  if (wpl)
    DeleteLonelyPlaces(subpage);                  // berflssige Stellen

  target->Renumber();                             // aufrumen
  target->UpdateHierarchy();                      

  return subtrans;
}

//-----------------------------------------------------------------------------
// GetParameters sucht aus einem Template-Netz alle strukturellen bzw.
// funktionellen Parameter heraus. Dabei werden auch mehrere Templates
// bercksichtigt. Fr jedes Template werden die Parameter darin nach ihrer
// Position auf der Seite sortiert. Gleichnamige Parameter in verschiedenen
// Templates mssen den gleichen Typ haben.
//
void TemplateIncluder::GetParameters(Page* page, bool structural,
       LinkedList<Node*>& parameters, Error& error)
{
  Iterator<Node*> pnodes = page->Nodes();         // In allen richtigen Unter-
  for (Node* pnode=pnodes; pnode; pnode=++pnodes) // seiten der Primepage
    if (pnode->Class()->Name()!="Comment" &&
       (pnode->Class()->Name()=="Structure")==structural) {

      LinkedList<Node*> params;
      Iterator<Node*> nodes = pnode->SubPage()->Nodes();
      Node* node, *node2;
      for (node=nodes; node; node=++nodes)        // Alle Parameter heraussuchn
        if (node->Class()->Name()=="Param")
          params.Append(node);

      LinkedList<Node*> sortedparams;             // Dann nach der Position
      Iterator<Node*> sorted = sortedparams;      // auf der Seite sortieren
      while (params) {                            // (damit's schn aussieht)
        node=*params.First(); params.Remove(node);
        for (node2=sorted; node2; node2=++sorted)
          if (node->Mid().x + node->Mid().y <
              node2->Mid().x + node2->Mid().y) break;
        if (node2) sortedparams.Insert(node, sorted.Index());
        else sortedparams.Append(node);
      }

      Iterator<Node*> paramsi=parameters;         // Dann alle Parameter in
      for (node=sorted; node; node=++sorted) {    // Rckgabeliste einhngen
        for (node2=paramsi; node2; node2=++paramsi) // Schon drin?
          if (node2->Name()==node->Name()) {      // Dann mssen die Typen
            if (Text(charP(*node2->Attr("type"))) // gleich sein, sonst
                == *node->Attr("type")) break;    // Fehler
            else {
              Text name = node->FullName();
              error(Error::DifferentParamTypes, charP(name));
              return;
            }
          }
        if (!node2) parameters.Append(node);
      }
    }
}

//-----------------------------------------------------------------------------
// Params2StructDef wandelt eine Liste von Parameter-Knoten in eine fr agnes
// verstndliche Struktur-Definition um und gibt sie in den bergebenen
// Ausgabe-Stream aus. Um Fehler besser lokalisieren zu knnen, wird fr jeden
// Parameter zunchst ein Test-Parsen durchgefhrt.
//
void TemplateIncluder::Params2StructDef(Iterator<Node*> params, ostream& out,
                                        bool hide_inout, Error& error)
{
  for (Node* param=params; param; param=++params){// Fr jeden Parameter
    Text type = charP(*param->Attr("type"));      // Attribute auslesen
    Text type_param = charP(*param->Attr("type_param"));
    Text descr = charP(*param->Attr("descr"));
    Text defaultval = charP(*param->Attr("default"));

    if (type=="input" || type=="output" || type=="inout")
      type = hide_inout ? "hidden text" : "text";

    char structmember[1024];
    strstream sm(structmember,1024);

    sm<<charP(type)<<" ";                         // Aus den Attributen des
    if (type_param!="")                           // Parameter-Knotens
      sm<<"("<<charP(type_param)<<") ";           // eine Member-Definition
    if (descr!="")                                // zusammenbasteln,
      sm<<"\""<<charP(descr)<<"\" ";              // diese in einen temporren
    sm<<charP(param->Name());                     // Stream schreiben
    if (type=="text" || type=="hidden text")
      sm<<"=\""<<charP(defaultval)<<"\""; 
    else sm<<"="<<charP(defaultval);
    sm<<ends;

    Attrib tmpattr;
    tmpattr.Input(sm);                            // Member einzeln parsen
    if (!sm) {                                    // Fehler?
      Text name = param->FullName();
      error(Error::IllegalParamType, charP(name));
      break;
    }

    out<<structmember<<"; ";                     // Member nach out ausgeben
  }
}

//-----------------------------------------------------------------------------
// GetExpressionContext sammelt alle Parameter und Definitionen, die in 
// auszuwertenden Ausdrcken verwendet werden knnen. 
// 
void TemplateIncluder::GetExpressionContext(Ident2Text& context, Page* tplnet,
       Struct* sparams, Struct* fparams, Page* structure, Page* wpl,
       Page* scontext, Page* fcontext, Text*& in, Text*& out, Error& error)
{
  int struc;
  
  for (struc=(fparams?0:1); struc<=1; struc++) {  // Fr Struktur und APLs
    LinkedList<Node*> parameters;
    GetParameters(tplnet, struc!=0, parameters, error); 

    Iterator<Node*> params = parameters;          // Alle Parameter hinzufgen
    for (Node* param=params; param; param=++params) {
      Text type = charP(*param->Attr("type"));
      Value* valuep = (struc?sparams:fparams)->GetMember(param->Name());
      Text value = charP(*valuep);                // Wert aus Dialog-Struktur
     
      if (type=="input" && in) value = *in;       // Wert fr Ein/Ausgangsmarke
      if (type=="input" && !in) in = (Text*) valuep; // aus Kanten
                                                  // oder umgekehrt
      if (type=="output" && out) value = *out;
      if (type=="output" && !out) out = (Text*) valuep;
      
      if (type=="enum") value = value + ".0";     // An Evaluater anpassen
      if (type=="bool" && value == "true") value = "TRUE";
      if (type=="bool" && value == "false") value = "FALSE";
      
      if (value=="") {
        error(Error::EmptyParam, &param->Name()); return; }

      context.Add(&param->Name(), new Text(value)); // und merken
    }
  }    

  for (struc=structure?(wpl?0:1):2; struc<=1; struc++) { 
    Iterator<Node*> nodes = (struc?structure:wpl)->Nodes();
    for (Node* node=nodes; node; node=++nodes)    // Alle Definitionen auf den
      if (node->Class()->Name()=="Def")           // Template-Seiten hinzufgen
        if (!context[&node->Name()])
          context.Add(&node->Name(), new Text(charP(*node->Attr("def"))));
  }

  for (struc=scontext?(fcontext?0:1):2; struc<=1; struc++) { 
    Iterator<Node*> nodes = (struc?scontext:fcontext)->Nodes();
    for (Node* node=nodes; node; node=++nodes)    // Alle Definitionen auf den
      if (node->Class()->Name()=="Def")           // Kontext-Seiten hinzufgen
        if (!context[&node->Name()])
          context.Add(&node->Name(), new Text(charP(*node->Attr("def"))));
  }
    
  AnalyserInterface::EvaluateCondition("1", context, 0, error); // Testen
}

//-----------------------------------------------------------------------------
// SelectTemplate sucht anhand der Instanziierungsbedingungen aus einem
// Template-Netz die Seite heraus, die zur Instanziierung herangezogen wird.
//
Page* TemplateIncluder::SelectTemplate(Page* tplnet, Ident2Text& context,
        bool wpl, Error& error)
{
  Page* page = 0;
  Iterator<Node*> nodes=tplnet->Nodes();
                                                  // Fr alle Vertreterknoten
  for (Node* node=nodes; node; node=++nodes)      // auf der Primepage
    if (node->Class()->Name()!="Comment" &&       // mit dem richtigen Typ
       (node->Class()->Name()=="Workplan") == wpl) {
      if (AnalyserInterface::EvaluateCondition(   // Inst.bedingung auswerten
            charP(*node->Attr("incon")), context, node, error)) {
        if (page) error(Error::AmbigousInstConditions);
        else page=node->SubPage(); }              // Keine Mehrdeutigkeiten!
      if (error) return 0;
    }
  if (!page) error(Error::Nothing2Insert);

  return page;
}

//-----------------------------------------------------------------------------
// CopyTemplatePage kopiert die gewnschte Template-Seite in die Seite des
// Ziel-Netzes. Alle kopierten Knoten haben bereits die korrekte Knotenklasse.
// Die Zuordnung Originalknoten->Kopie wird in der CopyMap festgehalten.
//
void TemplateIncluder::CopyTemplatePage(Page* source, Page* dest,
       Node2Node& CopyMap)
{
  LinkedList<Node*> subnodes;

  Iterator<Node*> nodes = source->Nodes();
  for (Node* node=nodes; node; node=++nodes) {    // Fr alle Knoten
    Ident nodecl = node->Class()->Name();
    Ident newclass = nodecl;
         if (nodecl=="Param") newclass="Def";     // Neue Klasse feststellen,
    else if (nodecl=="EPPlace") newclass="EPlace"; // sonst bleibts die alte
    else if (nodecl=="PPPlace") newclass="PPlace";
    else if (nodecl=="SubTrans") subnodes.Append(node); // Unternetze merken
    else if (nodecl=="Constraint") {              // Constraints nicht kopieren
      CopyMap.Add(node, 0); continue;
    }
    Node* copy = node->Copy(dest);                // Knoten kopieren,
    copy->SetClass(dest->Class()->GetNodeClass(newclass)); // Klasse ndern
    CopyMap.Add(node, copy);                      // und merken
  }

  Iterator<Connector*> citer = source->Connectors(); // Dann alle Kanten
  for(Connector* arc=citer; arc; arc=++citer) {   // kopieren
    Connector* copy = arc->Copy(dest);
    copy->SetClass(dest->Class()->GetConnectorClass(arc->Class()->Name()));
  }

  Iterator<Node*> subs=subnodes;                  // Dann die Unterseiten
  for (Node* sub=subs; sub; sub=++subs)
    CopyTemplatePage(sub->SubPage(),              // CopyTemplateSubPage???
      CopyMap[sub]->SubPage(), CopyMap);
}

//-----------------------------------------------------------------------------
// CheckConstraints berprft die Bedingungen aller im Template-Netz 
// vorhandenen Constraint-Knoten und erzeugt bei Verletzung eines Constraints
// die entsprechende Fehlermeldung
//
void TemplateIncluder::CheckConstraints(Iterator<Node*> nodes,
       Ident2Text& context, Error& error)
{
  for (Node* node=nodes; node; node=++nodes)      // Fr alle Constraint-Knoten
    if (node->Class()->Name()=="Constraint") {
      if (!AnalyserInterface::EvaluateCondition(  // Bedingung nicht erfllt?
        charP(*node->Attr("con")), context, node, error)) {
        Text msg = charP(*node->Attr("message")); // Fehlermeldung erzeugen
        if (msg=="")
          msg=Text("Constraint ")+charP(node->Name())+" hurt";
        error(Error::ConstraintHurt, charP(msg));
      }
      if (error) return;
    }
}

//-----------------------------------------------------------------------------
// InsertParameters setzt fr jeden Parameter-Knoten des TemplateNetzes in
// die in CopyTemplatePage erzeugte Definition das angegebene Argument ein.
//
void TemplateIncluder::InsertParameters(Node2NodeIter CopyMap,
       Ident2Text& context, Error&)
{
  Node* node;
  Node* copy;
  for (CopyMap.Start(node, copy); node; CopyMap.Step(node, copy))
    if (node->Class()->Name()=="Param") {
      *copy->Attr("def") = charP(*context[&node->Name()]);
//    *copy->Attr("param") = true;
    }
}

//-----------------------------------------------------------------------------
// FindStructuralNode sucht fr einen Knoten aus einem Arbeitsplan den 
// entsprechenden assoziierten Knoten aus der Struktur
//
Node* TemplateIncluder::FindStructuralNode(Node* node, Page* structure)
{
  Iterator<Node*> snodes = structure->Nodes();
  for (Node* snode=snodes; snode; snode=++snodes) 
    if (snode->Name()==node->Name()
        && snode->Class()->Name()==node->Class()->Name()) 
      return snode;
  return 0;
}

//-----------------------------------------------------------------------------
// DeleteTransitions lscht alle Transitionen, deren Existenzbedingung nicht
// erfllt ist. Eine Transition in einem Arbeitsplan wird auch gelscht, wenn
// die Existenzbedingung der zugehrigen Transition aus der Struktur nicht
// erfllt ist.
//
void TemplateIncluder::DeleteTransitions(Node2NodeIter CopyMap, Page* struc,
    Ident2Text& context, bool wpl, Error& error)
{
  Node* node;
  Node* copy;

  for (CopyMap.Start(node, copy); node; CopyMap.Step(node, copy)) {
    Ident nodecl=node->Class()->Name();           // Fr alle Transitionen
    if (nodecl=="TimedTrans" || nodecl=="ImmTrans" || nodecl=="SubTrans") {
      bool exists = AnalyserInterface::EvaluateCondition( // Existenzbedingung
        charP(*node->Attr("excon")), context, node, error); // ausw.
      if (error) return;

      if (exists && wpl)      {                  // Ist es eine Transition im
        exists = false;                           // APL, so zugehrige strukt.
        Node* snode = FindStructuralNode(node, struc); // Trans. suchen
        if (!snode) exists = true;
        else exists = AnalyserInterface::EvaluateCondition( // Existenzbed.
               charP(*snode->Attr("excon")), context, snode, error);// erfllt?
        if (error) return;
      }
      if (!exists) delete copy;                   // Nichtexistente Trans.
    }                                             // lschen
  }
}

//-----------------------------------------------------------------------------
// CheckForDuplicates berprft, ob nach dem Auswerten der Existenzbedingungen
// noch Knoten mit gleichen Namen im Strukturnetz existieren.
//
void TemplateIncluder::CheckForDuplicates(Page* page, Error& error)
{
  Iterator<Node*> nodes = page->Nodes();
  for (Node* node=nodes; node; node=++nodes)
    if (page->NameExists(node)) {                 // Name doppelt vorhanden?
      Ident nodecl=node->Class()->Name();         // Fehler abh. von Knotenkl.
      if (nodecl=="TimedTrans" || nodecl=="ImmTrans" || nodecl=="SubTrans")
        error(Error::AmbigousExistConditions, &node->Name());
      else error(Error::NameExists, &node->Name());
      return;
    }
}

//-----------------------------------------------------------------------------
// EvaluateConnectionConditions berprft die Anschlubedingungen von 
// Anschlustellen. Dabei wird entschieden, ob eine Stelle eine Eingangs-
// und/oder Ausgangsstelle des instanziierten Templates ist. Defaultmig wird
// dies anhand der Kanten dieser Stelle entschieden. Sind jedoch Verbindungs-
// bedingungen in der Struktur oder in den Arbeitsplnen angegeben, so werden
// diese ausgewertet.
//
void TemplateIncluder::EvaluateConnectionConditions(Node* node, Node* copy,
       Page* structure, Ident2Text& context, bool wpl, bool& to, bool& from,
       Error& error)
{
  to = copy->Outputs() || !copy->Inputs();        // Defaultwerte
  from = copy->Inputs() || !copy->Outputs();    
                                                
  Node* snode = structure ? FindStructuralNode(node, structure) : node;
                                                  // Stelle in der Struktur
  if (snode && !AnalyserInterface::EvaluateCondition( // Anschlubedingung 
      charP(*snode->Attr("cocon")), context, snode, error)) { // nicht erfllt?
    to = false; from = false; return;             // -> Kein Anschlu
  }

  if (wpl) {                                      // Stelle im APL
    Text cond = charP(*node->Attr("incon"));              
    if (cond!="")                                 // existiert Eingangsbed.?
      to = AnalyserInterface::EvaluateCondition(  // dann auswerten
             cond, context, node, error);
    if (error) return;

    cond = charP(*node->Attr("outcon"));
    if (cond!="")                                 // existiert Ausgangsbed.?
      from = AnalyserInterface::EvaluateCondition( // auswerten
               cond, context, node, error);
    if (error) return;
  }
}

//-----------------------------------------------------------------------------
// CreateSocketNodes erzeugt fr Anschluknoten im Template die entsprechenden
// Sockelknoten auf der Oberseite.
//
void TemplateIncluder::CreateSocketNodes(Node2NodeIter CopyMap, Node* subtrans,
       Page* struc, Ident2Text& context, bool wpl, Text* in, Text* out,
       Error& error)
{
  Node* node;
  Node* copy;
  Page* target = subtrans->GetPage();
  
  for (CopyMap.Start(node, copy); node; CopyMap.Step(node, copy)) {
    Ident nodecl=node->Class()->Name();           // Fr alle Anschluknoten
    if (nodecl=="EPPlace" || nodecl=="PPPlace") {
      bool to, from;
      
      EvaluateConnectionConditions(node, copy, struc, // Anschlubedingung 
        context, wpl, to, from, error); // auswerten
      if (error) return;

      Node* socket;
                                                  // Wenn eine Anschlubed.
      if (to || from) {                           // erfllt, dann
        socket = copy->Copy(target);              // Sockelknoten erzeugen
        Ident newname = subtrans->Name() + "_" + node->Name(); 
        socket->SetName(newname);                 // und umbenennen
        if (target->NameExists(socket)) {
          error(Error::NameExists, &newname); return; }

        copy->SetReal(socket);                    // Verbindung eintragen
        AddScopeQualifier((Text*)socket->Attr("marking"), subtrans);
        AddScopeQualifier((Text*)socket->Attr("capacity"), subtrans);
      }

      ConnectorClass* conncl = target->Class()->  
        GetConnectorClass(nodecl=="EPPlace" ? "EArc" : "PArc"); 

      if (to) {                                   // virtuellen Connector
        Connector* conn = new Connector(conncl, target); // erzeugen 
        conn->SetConnections(socket, subtrans);
        if (in && *in!="0'e" && conn->Attr("weight"))
          *conn->Attr("weight") = charP(*in);     // und beschriften
      }
      if (from) {                                 // genauso
        Connector* conn = new Connector(conncl, target);
        conn->SetConnections(subtrans, socket);
        if (out && *out!="0'e" && conn->Attr("weight"))
          *conn->Attr("weight") = charP(*out);
      }
    }
  }
}

//-----------------------------------------------------------------------------
// AddScopeQualifier fgt in einem Ausdruck (den Attributen eines Port-Nodes)
// den Scope-Qualifier der angegebenen Vertretertransition zu jeder Referenz
// auf Objekte der Unterseite hinzu.
//
void TemplateIncluder::AddScopeQualifier(Text* expr, Node* subtrans)
{
  if (expr) {
    Text newexpr = "";
    Ident ident = "";

    for (char* c=*expr; ; c++) {                  // Den String durchgehen
      if (isalpha(*c) || *c=='_' || (ident!="" && isdigit(*c)))
        ident+= *c;                               // Idents rausfiltern
      else {
        if (ident!="") {                          // Ident gefunden?
          if (subtrans->SubPage()->GetNode(charP(ident))) { // echter Ident?
            newexpr+="\\";
            newexpr+=charP(subtrans->Name());     // Scope hinzufgen
            newexpr+="\\";
            newexpr+=charP(ident);
          }
        }
        if (*c) newexpr+= *c; else break;
      }
    }
    *expr = newexpr;                              // Neuen Ausdruck bergeben
  }
}

//-----------------------------------------------------------------------------
// LayoutSocketNodes orndnet die erzeugten Sockelknoten schn um eine 
// Vertretertransition herum an. Dabei werden alle Eingangsknoten links, die 
// Ausgangsknoten rechts, und alle anderen Knoten unten angeorndet.
//
void TemplateIncluder::LayoutSocketNodes(Node* subtrans)
{
  Position mid = subtrans->Mid();
  LinkedList<Node*> inp_nodes, outp_nodes, inout_nodes;
  
  Iterator<Connector*> conns = subtrans->Inputs();
  Connector* c;
  for (c=conns; c; c=++conns) {                   
    Node* n = c->GetInput();                      // Alle Sockelknoten finden
    if (n->NumInputs()) inout_nodes.Append(n);    // und in Input-, Output-
    else inp_nodes.Append(n);                     // und InOut-Knoten einteilen
  }

  conns = subtrans->Outputs();
  for (c=conns; c; c=++conns) {
    Node* n = c->GetOutput();
    if (!n->NumOutputs()) outp_nodes.Append(n);
  }                                                
                                                  
  double dist_x = 80;                             // Alle Sockelknoten sollen
  double dist_y = 80;                             // sich auf einem Rechteck
  double step_x = 80;                             // 2dist_x * 2dist_y befinden
  double step_y = 60;                             // Abstand zweier Knoten ist
                                                  // step_x bzw. step_y
  double mid_yi = (inp_nodes.Length()-1)*step_y / 2.0; 
  double mid_yo = (outp_nodes.Length()-1)*step_y / 2.0; // Lngen bestimmen
  double mid_x  = (inout_nodes.Length()-1)*step_x / 2.0;

  if (dist_y < mid_yi) dist_y=mid_yi;             // Rechteck so vergrern,
  if (dist_y < mid_yo) dist_y=mid_yo;             // da sich keine Knoten 
  if (dist_x < mid_x+step_x) dist_x=mid_x+step_x; // berlagern
  if (inout_nodes) mid.y=mid.y-dist_y;

  int i;
  for (i=0; i<inp_nodes.Length(); i++)            // Alle Positionen setzen
    (*inp_nodes[i])->SetPos(Position(mid.x-dist_x, mid.y-mid_yi+i*step_y));

  for (i=0; i<outp_nodes.Length(); i++)
    (*outp_nodes[i])->SetPos(Position(mid.x+dist_x, mid.y-mid_yo+i*step_y));

  for (i=0; i<inout_nodes.Length(); i++)
    (*inout_nodes[i])->SetPos(Position(mid.x-mid_x+i*step_x, mid.y+dist_y));

  subtrans->SetPos(mid);
  subtrans->RearrangeConnections();
}

//-----------------------------------------------------------------------------
// DeleteLonelyPlaces lscht in einem instanziierten Arbeitsplan alle Stellen,
// an die keine Kanten mehr angeschlossen sind.
//
void TemplateIncluder::DeleteLonelyPlaces(Page* subpage)
{
  LinkedList<Node*> useless;
  Iterator<Node*> nodes = subpage->Nodes();
  Node* node;
  
  for (node=nodes; node; node=++nodes) {
    Ident nodecl = node->Class()->Name();
    if ((nodecl=="PPlace" || nodecl=="EPlace") && !node->Virtual() &&
        !node->NumInputs() && !node->NumOutputs())
      useless.Append(node);
  }
  
  nodes = useless;
  for (node=nodes; node; node=++nodes) 
    delete node;
}


/*
  - Unterseiten in Templates:
    - Vertretertransitionen haben Existenzbedingungen
    - Unterseiten werden immer komplett kopiert
    - Param-Definitionen knnen drin sein
*/
