#ifndef __INCLUDER_H__
#define __INCLUDER_H__

#include "commmod.h"
#include "main.h"
#include "editor.h"
#include "net.h"
#include "page.h"
#include "netcl.h"
#include "node.h"
#include "int.h"
#include "list.h"
#include "enum.h"

#include "relation.h"
#include "error.h"
#include "interface.h"
#include "nodesel.h"

typedef char* charP;
typedef Map<Node, Node> Node2Node;
typedef MapIterator<Node,Node> Node2NodeIter;


class TemplateIncluder: public CommModule
{
  friend class NetClassCreator;

  Editor* editor;
  Editor* sel_editor;
  Net* lib_copy;
  Page* tplnet;

  void IncludeTemplate(Node* node);
  void Quit();

  static bool NodeSelTestFunc(void*, Node* node)
    { return node->Class()->Name()=="Template"; }
  
  static void NodeSelCallBack(void* caller, Node* node)
    { ((TemplateIncluder*) caller)->IncludeTemplate(node); }  
  
public:
  static Node* InstanciateTemplate(Page* target, Page* tplnet, Ident& name,
    Position pos, Struct* sparams, Struct* fparams, Page* scontext,
    Page* fcontext, Text* in, Text* out, Error& error);

  static void GetParameters(Page* page, bool structural,
    LinkedList<Node*>& parameters, Error& error);

  static void Params2StructDef(Iterator<Node*> params, ostream& out,
    bool hide_inout, Error& error);

  static void GetExpressionContext(Ident2Text& context, Page* tplnet,
    Struct* sparams, Struct* fparams, Page* structure, Page* wpl,
    Page* scontext, Page* fcontext, Text*& in, Text*& out, Error& error);
  
  static Page* SelectTemplate(Page* tplnet, Ident2Text& context, bool wpl,
    Error& error);

  static void CopyTemplatePage(Page* source, Page* dest, Node2Node& CopyMap);

  static void CheckConstraints(Iterator<Node*> nodes, Ident2Text& context,
    Error& error);

  static void InsertParameters(Node2NodeIter CopyMap, Ident2Text& context,
    Error& error);

  static Node* FindStructuralNode(Node* node, Page* structure);

  static void DeleteTransitions(Node2NodeIter CopyMap, Page* struc,
    Ident2Text& context, bool wpl, Error& error);

  static void CheckForDuplicates(Page* page, Error& error);

  static void EvaluateConnectionConditions(Node* node, Node* copy, Page* struc,
    Ident2Text& context, bool wpl, bool& to, bool& from, Error& error);

  static void CreateSocketNodes(Node2NodeIter CopyMap, Node* subtrans,
    Page* struc, Ident2Text& context, bool wpl, Text* in, Text* out,
    Error& error);

  static void AddScopeQualifier(Text* expr, Node* subtrans);

  static void LayoutSocketNodes(Node* subtrans);

  static void DeleteLonelyPlaces(Page* subpage);

  
public:
  TemplateIncluder(Net& n): CommModule(n, "Insert Template"),
                            sel_editor(0), tplnet(0) {}
  bool Init();
  bool Open(Editor&,Page&);
  bool Change(Editor& e ,Page&,Page& p) { return Open(e,p); }

  void Call(Editor&,Page&);
  void OnDialogButton(Ident& button);

  bool ModuleLock() { return sel_editor || tplnet; }  
  void Close(Editor&,Page&) { Quit(); }
  ~TemplateIncluder() { Quit(); }
};

#endif
