#include <sys/types.h>
#include <sys/wait.h>
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <text.h>
#include "interface.h"
#include "list.h"
#include "int.h"
#include "netcl.h"
#include "evalmod.h"
#include "main.h"

extern int ErrNo;
extern double evaluater(char*);

typedef char* charP;

//-----------------------------------------------------------------------------
// Gepatchte Version
//
bool AnalyserInterface::EvaluateCondition(char* expr, Ident2Text& context,
       Node* where, Error& error)
{
  if (Text(expr)=="") return true;

  Text tmpexpr = expr;
  Text newexpr = "";
  Text eval;
  Ident ident = "";
  int count = 0;

  for (char* c=tmpexpr; ; c++) {                  // Den String durchgehen
    if (isalpha(*c) || *c=='_' || *c=='.' || *c=='[' || *c==']'
        || (isdigit(*c) &&ident!=""))
      ident+= *c;                                 // Idents rausfiltern
    else {
      if (ident!="") {
        if (context[&ident]) {                    // Durch Inhalt ersetzen
          Text tmp = Text(" ") + charP(*context[&ident]) + c; // Klammern?
          c = tmpexpr = tmp;
          if (count++>1000) break;
          ident = ""; continue;
        }
        else {
          newexpr += charP(ident);
          ident = "";
        }
      }
      newexpr += *c;
      if (!*c) break;
    }
  }
  
  eval = "Definition: \nPage:\\\nEvaluate: ";
  eval += charP(newexpr);
  eval += "\n";

  ErrNo = 0;
  double result = evaluater(charP(eval));
/*  
  cout<<expr<<" => "<<charP(newexpr)<<" => ";
  if (ErrNo) cout<<"Error "<<ErrNo<<endl;
  else cout<<(result ? "TRUE":"FALSE")<<endl;
*/  
  if (ErrNo) {
    error(Error::ExprSyntax, where ? &where->Name() : 0);
    return 0;
  }

  return result;
}


/* Alte Version mit Variablenbergabe
bool AnalyserInterface::EvaluateCondition(char* expr, Ident2Text& context,
       Node* where, Error& error)
{
  if (Text(expr)=="") return true;

  Text eval = "Definition: ";
  int count = 0;

  Ident* var;
  Text* def;

  Ident2TextIter defs = context;
  for (defs.Start(var, def); var; defs.Step(var, def)) {
    if (!strstr(*def,"@") && !strstr(*def,"#")) {
      if (count++) eval += " | ";
      eval += charP(Text("\\")+charP(*var)+"="+charP(*def));
    }
  }

  eval += "\nPage:\\\nEvaluate: ";
  eval += expr;
  eval += "\n";

  ErrNo = 0;
  double result = evaluater(charP(eval));
  
  cout<<charP(eval)<<result<<" "<<ErrNo<<endl;
  
  if (ErrNo) {
    error(Error::ExprSyntax, where ? &where->Name() : 0);
    return 0;
  }

  return result;
}
*/

//-----------------------------------------------------------------------------

void AnalyserInterface::DescribePage(Page* page, ostream& s, Error& error)
{
  Iterator<Node*> nodes = page->Nodes();
  Node* node;
  int count = 0;

  AddComment(s, "", 0);
  s<<endl<<"Parameter:"; count = 0;
  for (node=nodes; node; node=++nodes)
    if (node->Class()->Name()=="Def"              // Fr alle Parameter
        && bool(*node->Attr("param"))) {
      if (count++) s<<" | ";
      AddComment(s, "Value", node);
      s<<charP(PoppName(node))<<"=";              // <Name>=<Wert> ausgeben
      s<<charP(*node->Attr("def"));            
      if (!s) return;
    }

  s<<endl<<"Page:"<<charP(PoppName(page));
 
  s<<endl<<"ElementaryPlace:";
  for (node=nodes; node; node=++nodes)            // Fr alle Elementarstellen
    if (node->Class()->Name()=="EPlace") {
      if (count++) s<<" |"<<endl;                 // durch "|" getrennt
      s<<charP(node->Name())<<";";                // <Name> ; <Kap> ; <Mark> 
      AddComment(s, "Capacity", node);
      s<<charP(*node->Attr("capacity"))<<";";     // ausgeben


      AddComment(s, "Marking", node);
      Text marking = charP(*node->Attr("marking"));
      if (marking!="")                   
        s<<"("<<charP(marking)<<")*1'e";
      if (!s) return;
    }
  
  s<<endl<<"ProductPlace:"; count = 0; 
  for (node=nodes; node; node=++nodes)            // Fr alle Produktstellen
    if (node->Class()->Name()=="PPlace") {
      if (count++) s<<" |"<<endl;
      s<<charP(node->Name())<<";";                // <Name> ; <Kap> ; <Mark> 
      AddComment(s, "Capacity", node);
      s<<charP(*node->Attr("capacity"))<<";";     // ausgeben
      
      AddComment(s, "Marking", node);
      s<<charP(*node->Attr("marking"));
      if (!s) return;
    }

  s<<endl<<"Transition:"; count = 0;
  for (node=nodes; node; node=++nodes)
    if ((node->Class()->Name()=="TimedTrans"
        || node->Class()->Name()=="ImmTrans")
        && ! bool(*node->Attr("unused"))) {       // Fr alle Transitionen
      
      if (count++) s<<" "<<endl; 
      s<<charP(node->Name());                     
      if (node->Class()->Name()=="TimedTrans" && 
          Text("infinite")==charP(*node->Attr("server_type"))) 
            s<<" IS";                             // Infinite Server

      AddComment(s, "Global Guard", node);
      Text guard = charP(*node->Attr("global_guard"));
      if (guard=="") s<<" (-)"; else s<<" ("<<charP(guard)<<") ";

      s<<"[";
      int count2 = 0;
      Iterator<Value*> bindings=((List*)(node->Attr("bindings")))->Entries();
      for (Value* b=bindings; b; b=++bindings) {  // bergangstabelle ausgeben
        Text number; number = count2 +1;
        AddComment(s, Text("Binding #")+number, node);
        if (count2++) s<<" |"<<endl<<"   ";
        s<<charP(*b);
      }
      s<<"]";
      if (!s) return;
    }
 
  for (node=nodes; node; node=++nodes)            // Vertretertransition
    if (node->Class()->Name()=="SubTrans") {
      if (count++) s<<" "<<endl;

      s<<charP(node->Name())<<" [subpage ";
      s<<charP(node->Name())<<";assign ";

      int subcount = 0;
      Iterator<Node*> subnodes = node->SubPage()->Nodes();
      for (Node* subnode=subnodes; subnode; subnode=++subnodes) 
        if (subnode->Virtual()) {
          if (subcount++) s<<", ";
          s<<charP(subnode->Real()->Name())<<"->";
          s<<charP(subnode->Name());
        }
      s<<"]";
      if (!s) return;
    }

  s<<endl<<"Evaluate:"; count = 0; 
  for (node=nodes; node; node=++nodes)            // Fr alle Resultparameter
    if (node->Class()->Name()=="Result") {
      if (count++) s<<" |"<<endl;
      AddComment(s, "Value", node);
      s<<charP(node->Name())<<"=";                // <Name>=<Wert> ausgeben
      s<<charP(*node->Attr("def"));               // <Formel> ausgeben
      if (!s) return;
    }
  
  for (node=nodes; node; node=++nodes)            // Jetzt noch die Unternetze 
    if (node->Class()->Name()=="SubTrans") {
      DescribePage(node->SubPage(), s, error);
      if (!s) return;
    }
  s<<endl;
}

//-----------------------------------------------------------------------------

void AnalyserInterface::DescribeSymbols(Page* page, ostream& s)
{
  LinkedList<Node*> allnodes;
  Iterator<Node*> nodes = allnodes;
  GetAllNodes(page, allnodes);
  Node* node;

  int count = 0;
  s<<endl<<"Definition:";
  
  for (node=nodes; node; node=++nodes)
    if (node->Class()->Name()=="Def"              // Fr alle Definitionen
        && !bool(*node->Attr("param"))) {
      if (count++) s<<" | ";                      // auf allen Seiten
      AddComment(s, "Value", node);
      s<<charP(PoppName(node))<<"=";              // <Name>=<Wert> ausgeben
      s<<charP(*node->Attr("def"))<<endl;            
      if (!s) return;
    }
}

//-----------------------------------------------------------------------------

void AnalyserInterface::AddComment(ostream& /*s*/, Text /*where*/,
                                   Object* /*ob*/)
{
//  Text commands;
//  if (ob) 
//    commands = Text("!showobject ") + PoppName(ob) + "\n"
//      + "!errormessage \"Syntax error in\\n" + where + " of "
//      + ob->Class()->Label() + " \\\"" + ob->Name() + "\\\"\"";
//  else 
//    commands = "!errormessage \"Syntax error\"";
//  
//  s <<"\n/" << "*\n" << charP(commands) << "\n*/\n";
}

//-----------------------------------------------------------------------------

Text AnalyserInterface::PoppName(Object* ob)
{
  Node* parent = ob->GetPage()->Parent();
  if (parent) return PoppName(parent) + "\\" + ob->Name();
  else return Text("\\") + ob->Name();
}

Text AnalyserInterface::PoppName(Page* page)
{
  Node* parent = page->Parent();
  if (parent) return PoppName(parent);
  else return "\\";
}

//-----------------------------------------------------------------------------

void AnalyserInterface::GetAllNodes(Page* page, LinkedList<Node*>& allnodes)
{
  Iterator<Node*> nodes = page->Nodes();
  LinkedList<Node*> subnodes;
  Iterator<Node*> subs = subnodes;
  Node* node;
  
  for (node=nodes; node; node=++nodes) {
    allnodes.Append(node);
    if (node->SubPage()) subnodes.Append(node);
  }

  for (node=subs; node; node=++subs)
    GetAllNodes(node->SubPage(), allnodes);
}

//-----------------------------------------------------------------------------

class MSAnaModule: public EvalModule
{
public:

	bool Init() { return net.Prime()->Class()->Name() == "HCPN"; }

  void Call(Editor& e, Page& p) { EvalModule::Call(e,p); } // ???

  Text EvalSettings();
  bool AllowExperiments() { return true; }
  bool SetValue(Text& name, double val);
  bool EvalCall(bool experiment, Text& command, Text& stdout_file,
                Text& std_infile, Text& net_file, Text& result_file,
                Text& logwin_title, Text& logwin_buttons);

  MSAnaModule(Net& n): EvalModule(n, "Numerical Analysis") {}
  ~MSAnaModule() {}
};

//-----------------------------------------------------------------------------

Text MSAnaModule::EvalSettings()
{
  return 
    "[ enum(\"Iterative (SOR)\"iter, \"Direct (Gauss/Seidel)\" direct)"
    "       \"Solution Method\" method = direct; "
    "  int \"Maximum of Iterations\" max_iter = 10000;"
    "  text(0) \"Epsilon\" epsilon = \"1e-7\";"
    "  bool \"Verbose Mode\" debug = false; ]";
}

//-----------------------------------------------------------------------------

bool MSAnaModule::EvalCall(bool, Text& command, Text& stdout_file,
                           Text&, Text& net_file, Text& result_file,
                           Text& logwin_title, Text& )
{
  Error error(*LastEditor());
  
  Text netname = Text(net.FileName()) - ".net";  
  net_file = netname + ".table";
  stdout_file = netname + ".out";

  if (charP(net_file)[0]!='/') net_file = model_dir + "/" + net_file;
  if (charP(stdout_file)[0]!='/') stdout_file = model_dir + "/" + stdout_file;

  ofstream file((char*) net_file);
  AnalyserInterface::DescribeSymbols(net.Prime(), file); 
  AnalyserInterface::DescribePage(net.Prime(), file, error);
  file.close();
  if (error || !file) return false;
  
  command = tnet_dir + "/MSAnalysis/bin/rg -E" + Param("epsilon")
          + " -N " + netname + " -M"+Param("max_iter");

  if (Param("debug") == "true")
    command = command + " -v";

  if (Param("method") == "iter")
    command = command + " -i";
  
  result_file = stdout_file;
  logwin_title = "Analysis Window";
  return true;
}

//-----------------------------------------------------------------------------

bool MSAnaModule::SetValue(Text& name, double val)
{
  istrstream ob = charP(name);
  Object* obj = CommandGetObject(ob);

  if (!obj || obj->Class()->Name() != "Def") return false;
  *obj->Attr("def") = val;
  return true;
}


MODULE_NOTIFY(MSAnaModule)
