
#include "netcls.h"
#include "net.h"
#include "error.h"
#include "includer.h"
#include "interface.h"
#include <stdio.h>

typedef char* charP;

NetClassCreator creator;

Net* popp_library;

//-----------------------------------------------------------------------------
// Der Konstruktor wird automatisch aufgerufen, da ein es ein statisches Objekt
// dieser Klasse gibt. Dabei werden die Netzklassen erzeugt.
//
NetClassCreator::NetClassCreator()
{
  CreateHCPNClasses();
}

//-----------------------------------------------------------------------------
// Hier werden die Netzklassen erzeugt.
//
void NetClassCreator::CreateHCPNClasses()
{
  enum ncenum {hcpn=0, astr, awpl, apr,
               tstr, twpl, tpr, libs, lib, sstr, swpl, spr, ende};        

  for (int nc=hcpn; nc<ende; nc++) {              // Fr alle Netzklassen

     char classdef[20000];                        // Puffer fr Netzklassen-
     ostrstream s(classdef,20000);                // Definition anlegen

     bool structure = (nc==hcpn || nc==astr || nc==tstr);
     bool workplan  = (nc==awpl || nc==twpl);
     bool petrinet  = (structure || workplan);
     bool primepage = (nc==apr  || nc==tpr || nc == spr);
     bool templat   = (nc==tstr || nc==twpl || nc==tpr);
     bool symbols   = (nc==sstr || nc==swpl || nc==spr);

     if (nc==hcpn || nc==apr || nc == spr)
       s<<"netclass ["; else  s<<"subclass [";

     s<<"name = ";                                // Name der Netzklasse
     if (nc==hcpn) s<<"HCPN;";
     if (nc==astr) s<<"\"Associated HCPN (Structure)\" AHCPN_Structure;";
     if (nc==awpl) s<<"\"Associated HCPN (Workplan)\" AHCPN_Workplan;";
     if (nc==apr)  s<<"\"Associated HCPN\" AHCPN_Prime;";
     if (nc==tstr) s<<"\"Template HCPN (Structure)\" THCPN_Structure;";
     if (nc==twpl) s<<"\"Template HCPN (Workplan)\" THCPN_Workplan;";
     if (nc==tpr)  s<<"\"Template HCPN\" THCPN_Prime;";
     if (nc==libs) s<<"\"Template Library\" Template_LibSub;";
     if (nc==lib)  s<<"\"Template Library\" Template_Lib;";
     if (nc==sstr) s<<"\"Symbolic Production System (Structure)\""
                           "APSYS_Structure;";
     if (nc==swpl) s<<"\"Symbolic Production System (Workplan)\""
                           "APSYS_Workplan;";
     if (nc==spr)  s<<"\"Symbolic Production System\" APSYS_Prime;";

     s<<"checknames = ";                          // kein Namencheck in
     s<<(workplan || nc==tstr || nc==swpl         // Arbeitsplnen
         ? "false;" : "true;"); 

     s<<"attributes = ["                          // Netzattribute
       "bool \"Use Splines\" use_splines  = true;"
       "bool \"Show Names\"  show_names   = true;";
       if (!symbols) s<<
         "bool \"Show Guards\" show_guards  = true;"
         "bool \"Show Delays\" show_delays  = false;"
         "bool \"Show Capacities\" show_cap = true;";
       if (templat || nc==lib || nc==libs) s<<
         "bool \"Show Instanciation Conditions\" show_excons = false;";
       if (nc==hcpn) s<<
         "bool \"Show AHCPN Usage\" show_use = true;";
       s<<
       "text \"Normal Font\" normal_font = "
         "\"-adobe-helvetica-medium-r-*-*-12-*-*-*-*-*-*-*\";"
       "text \"Medium Font\" medium_font = "
         "\"-adobe-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*\";"
       "text \"Small Font\" small_font = "
         "\"-adobe-helvetica-medium-r-*-*-8-*-*-*-*-*-*-*\";"
       "text \"Marking Font\" marking_font= "
         "\"-adobe-helvetica-*-r-*-*-12-*-*-*-*-*-*-*\";"
     "];"

     "nodes = [";

     for (int port=0; port<=(templat ? 1:0) ; port++) {

       if (structure) {                           // Elementarstelle
         s<<"[";
           if (!port) s<<"name = \"Elementary Place\" EPlace;";
           else s<<"name = \"Elementary Port Place\" EPPlace;";
           s<<"group = Place;"
           "nameprefix = P;"
           "relatives = ([0,-130],[0,140]);"
           "defscale = (0.2,0.2);"

           "attributes = ["
             "text \"Initial Marking\" marking = \"\";"
	           "text \"Capacity\" capacity = \"\";";
           if (port) s<<
             "text \"Connection Condition\" cocon = \"\";";
           s<<"];"

           "shape = ["
             "compo[[color=#selected?blue:black, font=@normal_font], [0,0],"
               "ellarc[[filled=true, color=white], [0,0], 90];"
               "ellarc[[thick=1], [0,0], 90];"
               "ellarc[[";
                  if (!port) s<<"cond=#virtual,";
                  s<<"color=white, dashed=true,thick=1], [0,0], 90];"

               "label[[cond=!(@marking=0|@marking=1|@marking=2|@marking=3),"
                      "font=@marking_font], [0,0],"
	                   "(!@marking)<6 ? @marking : \"...\" , Center];"

               "ellarc[[cond=@marking=1, filled=true], [0,0], 16];"
               "compo[[cond=@marking=2, filled=true], [0,0],"
                 "ellarc[[],[-35,0],16]; ellarc[[],[35,0],16];"
               "];"

               "compo[[cond=@marking=3, filled=true], [0,0],"
                 "ellarc[[],[0,-35],16]; ellarc[[],[-35,25],16];"
                 "ellarc[[],[35,25],16];"
               "];"

               "label[[cond=@show_names, relative=0], [0,0], #name,Center];"
               "label[[cond=@show_cap & @capacity>\"\" , relative=1], [0,0],"
		           "(!@capacity)<6 ? \"[\"+@capacity+\"]\" : \"[...]\",Center];"
             "];"
           "];"
         "],";
       }

       if (petrinet) {                            // Produktstelle
         s<<"[";
           if (!port) s<<"name = \"Product Place\" PPlace;";
           else s<<"name = \"Product Port Place\" PPPlace;";
           s<<"group = Place;"
           "nameprefix = P;"
           "relatives = ([0,-130],[0,140]);"
           "defscale = (0.2,0.2);"

           "attributes = [";
           if (structure) { s<<
             "text \"Initial Marking\" marking = \"\";"
	           "text \"Capacity\" capacity = \"\";";
             if (nc==hcpn)
               s<<"hidden bool unused = false;";
           }
           if (port && structure) s<<
             "text \"Connection Condition\" cocon = \"\";";
           if (port && workplan) s<<
             "text \"Input Connection Condition\" incon = \"\";"
             "text \"Output Connection Condition\" outcon = \"\";";
           s<<"];"

           "shape = ["
             "compo[[color=#selected ? blue : ";
             s<<(nc==hcpn ?"(@show_use & @unused ? grey70 :black),":"black,");
             s<<"font=@normal_font], [0,0],"
               "ellarc[[filled=true, color=white], [0,0], 90];"
               "ellarc[[thick=2], [0,0], 90];"
               "ellarc[[";
                  if (!port) s<<"cond=#virtual,";
                  s<<"color=white, dashed=true, thick=2], [0,0], 90];";

               if (structure) { s<<
                 "label[[font=@marking_font], [0,0],"
	               "(!@marking)<6 ? @marking : \"...\" , Center];"
                 "label[[cond=@show_cap & @capacity>\"\" , relative=1], [0,0],"
	               "(!@capacity)<6 ? \"[\"+@capacity+\"]\" : \"[...]\",Center];";
               }
               s<<"label[[cond=@show_names, relative=0], [0,0], #name,Center];"
             "];"
           "];"
         "],";
       }
     }
       if (petrinet) { s<<                        // Zeitbehaftete Transition
         "["
           "name = \"Timed Transition\" TimedTrans;"
           "group = Transition;"
           "nameprefix = T;"
           "relatives = ([0,-130], [0,140], [0, 210]);"
           "defscale = (0.2,0.2);"

           "attributes = [";
             if (structure) s<<"text \"Global Guard\" global_guard = \"\";";
             if (workplan)  s<<"text \"Local Guard\" guard = \"\";";
             if (workplan)  s<<"text \"Local Weight\" weight = \"1\";";
             if (nc==hcpn)  s<<"list(text) \"Binding Table\" bindings;";
             if (structure) s<<"enum(\"Single Server\" single,"
	                                  "\"Infinite Server\" infinite)"
                                    "\"Server Policy\" server_type=single;";
             s<<"enum(Exponential,Deterministic, General) ";
             if (nc==hcpn)  s<<"\"Display as\"";
             else if (structure) s<<"\"Default Delay Distribution\"";
             else if (workplan) s<<"\"Firing Delay Distribution\"";
             s<<" distr = Exponential;";

             if (nc==hcpn) s<<"hidden bool unused = false;"
                              "hidden text selcol= \"blue\";";
             else if (structure)
               s<<"text \"Default Delay\" delay = \"1\";";
             else if (workplan) 
               s<<"text \"Firing Delay\" delay = \"\";";
             if (templat)
               s<<"text \"Instanciation Condition\" excon = \"\";";
           s<<"];"

           "shape = ["
             "compo[[";
               if (nc==hcpn) s<<"color=#selected ? @selcol : "
                                "(@show_use & @unused ? grey70 :black)";
               else s<<"color=#selected ? blue : black"; 
               s<<", font=@normal_font], [0,0],"
               "pline[[color=(@distr=Exponential ? white :"
		                        "(@distr=General ? grey70 :"
 		                        "(#selected ? blue : ";
               s<<(nc==hcpn?"(@show_use & @unused ? grey70 :black)":"black");
               s<<"))), filled=true],"
               "[-50,90],[50,90],[50,-90],[-50,-90],[-50,90]];"
	             "pline[[thick=2],[-50,90],[50,90],[50,-90],"
                               "[-50,-90],[-50,90]];"
               "label[[cond=@show_names, relative=0], [0,0], #name, Center];"
               "label[[relative=1], [0,0],";
               if (structure) s<<
                 "(@server_type=infinite ? \"IS \" : \"\")"
                 "+(@show_guards & @global_guard>\"\" ?"
                   "((!@global_guard)<12 ? \"[\"+@global_guard+\"]\\n\""
                   ": \"[...]\\n\") : \"\")";
               if (workplan) s<<
                 "(@show_guards & @guard>\"\" ?"
                   "((!@guard)<12 ? \"[\"+@guard+\"]\\n\":\"[...]\\n\"):\"\")";
               if (templat) s<<
                 "+(@show_excons & @excon>\"\" ?"
                   "((!@excon)<12 ? \"<\"+@excon+\">\\n\":\"<...>\\n\"):\"\")";
               if (nc!=hcpn && structure || workplan) s<<
                  "+(@show_delays & @delay>\"\" & !(@delay=\"1\") ?"
                   "((!@delay)<12 ? \"\"+@delay : \"...\"):\"\")";
               s<<", Center];"
             "];"
           "];"
         "],";
       }

       if (petrinet) { s<<                        // Zeitlose Transition
         "["
           "name = \"Immediate Transition\" ImmTrans;"
           "group = Transition;"
           "nameprefix = T;"
           "relatives = ([0,-130], [0,140]);"
           "defscale = (0.2,0.2);"

           "attributes = [";
             if (structure) s<<"text \"Global Guard\" global_guard = \"\";"
                               "text \"Default Priority\" priority = \"1\";";
             if (workplan)  s<<"text \"Local Guard\" guard = \"\";"
                               "text \"Local Weight\" weight = \"1\";"
                               "text \"Local Priority\" priority = \"\";";
             if (nc==hcpn)  s<<"list(text) \"Binding Table\" bindings;"
                               "hidden bool unused = false;"
                               "hidden text selcol= \"blue\";";
             if (templat)
               s<<"text \"Instanciation Condition\" excon = \"\";";
           s<<"];"

           "shape = ["
             "compo[[";
               if (nc==hcpn) s<<"color=#selected ? @selcol : "
                                "(@show_use & @unused ? grey70 :black)";
               else s<<"color=#selected ? blue : black"; 
             s<<", font=@normal_font], [0,0],"
               "pline[[filled=true], [-10,90],[10,90],[10,-90],"
                                    "[-10,-90],[-10,90]];"
               "label[[cond=@show_names, relative=0], [0,0], #name, Center];"
	             "label[[relative=1], [0,0],"
                 "(@show_guards & @guard>\"\" ?"
                 "((!@guard)<12 ?\"[\"+@guard+\"]\\n\": \"[...]\\n\") : \"\")";
               if (templat) s<<
                 "+(@show_excons & @excon>\"\" ?"
                   "((!@excon)<12 ? \"<\"+@excon+\">\":\"<...>\"):\"\")";
             s<<", Center];"
             "];"
           "];"
         "],";
       }

       if (petrinet) { s<<                        // Vertretertransition
         "["
           "name = \"Substitution Transition\" SubTrans;"
           "group = Transition;"
           "nameprefix = T;"
           "relatives = ([0,-130], [0,140]);"
           "defscale = (0.2,0.2);"
           "subclass = ";
           if (nc==hcpn) s<<"HCPN;";
           if (nc==astr || nc==tstr) s<<"AHCPN_Structure;";
           if (nc==awpl || nc==twpl) s<<"AHCPN_Workplan;";

           s<<"attributes = [";
             if (nc==hcpn) s<<"hidden bool unused = false;"
                              "hidden text selcol= \"blue\";";
             if (templat)
               s<<"text \"Instanciation Condition\" excon = \"\";";
           s<<"];"

           "shape = ["
             "compo[[";
               if (nc==hcpn) s<<"color=#selected ? @selcol : "
                                "(@show_use & @unused ? grey70 :black)";
               else s<<"color=#selected ? blue : black"; 
               s<<", font=@normal_font], [0,0],"
               "pline[[color=white, filled=true],"
                     "[-50,90],[50,90],[50,-90],[-50,-90],[-50,90]];"
               "pline[[thick=2],[-50,90],[50,90],[50,-90],[-50,-90],[-50,90]];"
               "pline[[filled=true],[-50,90],[50,90],[50,60],"
                                   "[-50,60],[-50,90]];"
               "pline[[filled=true],[-50,-60],[50,-60],[50,-90],"
                                   "[-50,-90],[-50,-60]];"
               "label[[cond=@show_names, relative=0], [0,0], #name, Center];";
               if (templat) s<<
                 "label[[relative=0], [0,0], (@show_excons & @excon>\"\" ?"
                  "((!@excon)<12 ? \"<\"+@excon+\">\":\"<...>\"):\"\")"
                   ", Center];";
             s<<"];"
           "];"
         "],";
       }

       if (symbols && !primepage) {
         Iterator<Node*> nodes = popp_library->Prime()->Nodes();
         Node* node;

         for (node=nodes; node; node=++nodes) 
           CreateAPSYSNode(node, node, "", "", nc==sstr, s);
       }
    
       if (nc==swpl) {                            // Verzweigungsknoten
         s<<"["
           "name = \"Branch\" Branch;"
           "nameprefix = Branch;"
           "defscale = [0.4, 0.4];"
           "attributes = [];"
           "shape = ["
             "compo[[color=#selected?blue:black, font=@normal_font], [0,0],"
               "pline[[color=white, filled=true],"
                     "[-50,0],[0,50],[50,0],[0,-50],[-50,0]];"
               "pline[[thick=2],[-50,0],[0,50],[50,0],[0,-50],[-50,0]];"
               "label[[], [0,0], \"?\", Center];"
             "];"
           "];"
         "],";
       }
     
       if (!primepage && nc!=lib && nc!=libs) {   // Definition
         s<<"["
           "name = \"Definition\" Def;"
           "group = Defs;"
           "nameprefix = X;"

           "attributes = ["
             "text \"Definition\" def = \"\";"
             "hidden bool param = false;"
           "];"

           "shape = ["
             "label[[cond=#name=\"\", font=@normal_font], [0,0],"
                    "\"K = 50\", Center];"
             "label[[cond=#name>\"\", color=#selected?blue:black,"
                   "font=@normal_font],[0,0], (@param ? \"Parameter \" : \"\")"
                   "+#name+ \" = \" + ((!@def)<20 ? @def : \"...\"), Left];"
           "];"
         "],";
       }

       if (petrinet && !templat || nc==sstr) { s<< // Result
         "["
           "name = \"Result\" Result;"
           "group = Defs;"
           "nameprefix = R;"

           "attributes = ["
             "text \"Definition\" def = \"\";"
             "text \"Computed Value\" val = \"\";"
           "];"

           "shape = ["
             "label[[cond=#name=\"\", font=@normal_font], [0,0],"
                    "\"P[n>0]\", Center];"
             "label[[cond=#name>\"\", color=#selected?blue:black,"
                   "font=@normal_font],[0,0], \"Result \"" 
                   "+#name+ \" = \" + (@val>\"\" ? @val : "
                   "((!@def)<20 ? @def : \"...\")) , Left];"
           "];"
         "],";
       }

       if (petrinet && templat) { s<<             // Parameter
         "["
           "name = \"Parameter\" Param;"
           "group = Defs;"
           "nameprefix = Param;"

           "attributes = ["
             "text \"Textual Description\" descr = \"\";"
             "enum(\"Boolean Constant\" bool,"
                  "\"Integer Constant\" int,"
                  "\"Enumeration\" enum,"
                  "\"Expression\" text,";
                  if (workplan) s<<
                    "\"Input Token\" input,"
                    "\"Output Token\" output";
                  s<<") \"Parameter Type\" type = text;"
             "text \"Type Parameter\" type_param = \"\";"
             "text \"Default Value\" default = \"\";"
           "];"

           "shape = ["
             "label[[cond=#name=\"\", font=@normal_font], [0,0],"
                    "\"Param K\", Center];"
             "label[[cond=#name>\"\", color=#selected?blue:black,"
                   "font=@normal_font],[0,0], \"Parameter \"+ #name, Left];"
           "];"
         "],";
       }

       if (petrinet && templat) { s<<             // Constraint
         "["
           "name = \"Constraint\" Constraint;"
           "group = Defs;"
           "nameprefix = Con;"

           "attributes = ["
             "text \"Constraint\" con = \"\";"
             "text(2) \"Error Message when hurt\" message = \"\";"
           "];"

           "shape = ["
             "label[[cond=#name=\"\", font=@normal_font], [0,0],"
                    "\"Must be\", Center];"
             "label[[cond=#name>\"\", color=#selected?blue:black,"
                   "font=@normal_font],[0,0], \"Constraint \"+"
                   "#name+ \" = \" + ((!@con)<20 ? @con : \"...\"), Left];"
           "];"
         "],";
       }

       if (primepage) {                           // Struktur-Knoten
         s<<"["
           "name = Structure;"
           "nameprefix = Structure;"
           "relatives = ([0,-110], [0,110]);"
           "defscale = [0.3,0.3];"
           "attributes = [";
           if (nc==tpr)
             s<<"text \"Instanciation Condition\" incon = \"\";";
           else
             s<<"bool (\"Active\", \"Inactive\") \"Current Status\""
                        "active = true;"; 
           s<<"];";
           if (nc==apr) s<<"subclass = AHCPN_Structure;";
           if (nc==tpr) s<<"subclass = THCPN_Structure;";
           if (nc==spr) s<<"subclass = APSYS_Structure;";
           s<<"shape = ["
             "compo[[color=#selected?blue:(@active?black:gray70),"
                    "font=@normal_font], [0,0],"
               "pline[[thick=6,color=gray60],[95,-61],[95,71],[-85,71]];"
               "pline[[filled=true,color=white],[-90,0],[-90,-66],[90,-66],"
                      "[90,66],[-90,66],[-90,0]];"
               "pline[[thick=4],[-90,0],[-90,-66],[90,-66],[90,66],[-90,66],"
                      "[-90,0]];"
               "label[[font=@medium_font],[0,0],\"Structure\",Center];"
               "label[[relative=0], [0,0], #name, Center];";
               if (templat) s<<
                 "label[[relative=1, cond=(@show_excons & @incon>\"\")],"
                      " [0,0], (!@incon)<12 ? \"<\"+@incon+\">\" : \"<...>\""
                 ", Center];";
             s<<"]"
           "]"
         "],"

         "["                                      // Arbeitsplan-Knoten
           "name = Workplan;"
           "nameprefix = WP;"
           "relatives = ([0,-110], [0,110]);"
           "defscale = [0.3,0.3];"
           "attributes = [";
           if (nc==tpr)
             s<<"text \"Instanciation Condition\" incon = \"\";";
           else
             s<<"bool (\"Active\", \"Inactive\") \"Current Status\""
                        "active = true;"; 
           s<<"];";
           if (nc==apr) s<<"subclass = AHCPN_Workplan;";
           if (nc==tpr) s<<"subclass = THCPN_Workplan;";
           if (nc==spr) s<<"subclass = APSYS_Workplan;";
           s<<"shape = ["
             "compo[[color=#selected?blue:(@active?black:gray70),"
                    "font=@normal_font], [0,0],"
               "pline[[thick=6,color=gray60],[95,-61],[95,71],[-85,71]];"
               "pline[[filled=true,color=white],[-90,0],[-90,-66],[90,-66],"
                      "[90,66],[-90,66],[-90,0]];"
               "pline[[thick=4],[-90,0],[-90,-66],"
                      "[90,-66],[90,66],[-90,66],[-90,0]];"
               "label[[font=@medium_font], [0,0],\"Workplan\",Center];"
               "label[[relative=0], [0,0], #name, Center];";
               if (templat) s<<
                 "label[[relative=1, cond=(@show_excons & @incon>\"\")],"
                      " [0,0], (!@incon)<12 ? \"<\"+@incon+\">\" : \"<...>\""
                 ", Center];";
             s<<"]"
           "]"
         "],";
       }

       if (nc==libs) {                            
         s<<"["                                   // Template Icon
           "name = Template;"
           "nameprefix = Template;"
           "relatives = ([0,-110]);"
           "defscale = [0.3,0.3];"
           "attributes = ["
             "text \"Name of Template\" real_name=\"\";"
             "text \"Author\" author=\"\";"
             "text \"Date of creation\" date=\"\";"
             "text(5) \"Description\" descr=\"\";"
             "bool \"Use Template as symbol\" symbol = true;"
             "text(5) \"Shape of symbol\" shape=\"\";" 
           "];"
           "subclass = THCPN_Prime;"
           "shape = ["
             "compo[[color=#selected?blue:black, font=@normal_font], [0,0],"
               "pline[[thick=6,color=gray60],[95,-61],[95,71],[-85,71]];"
               "pline[[filled=true,color=white],[-90,0],[-90,-66],[90,-66],"
                      "[90,66],[-90,66],[-90,0]];"
               "pline[[thick=4],[-90,0],[-90,-66],"
                      "[90,-66],[90,66],[-90,66],[-90,0]];"
               "label[[font=@medium_font],[0,0],\"Template\",Center];"
               "label[[relative=0], [0,0],"
                      "@real_name=\"\" ? #name : @real_name, Center];"  
             "]"
           "]"
         "],";
       }

       if (nc==lib || nc==libs) {                            
         s<<"["                                   // Template Group
           "name = \"Template Group\" TGroup;"
           "nameprefix = Group;"
           "relatives = ([0,-110]);"
           "defscale = [0.3,0.3];"
           "attributes = ["
             "text \"Name of Template Group\" real_name=\"\";"
             "bool \"Use templates as symbols\" symbol = true;"
             "text(5) \"Default shape of symbols\" shape=\"\";" 
           "];"
           "subclass = Template_LibSub;"
           "shape = ["
             "compo[[color=#selected?blue:black, font=@normal_font], [0,0],"
               "pline[[],[-90,0],[-90,-66],[90,-66],[90,66],[-90,66],[-90,0]];"
               "pline[[thick=6,color=gray60],[95,-61],[95,71],[-85,71]];"
               "pline[[filled=true,color=white],[-90,0],[-90,-66],[90,-66],"
                      "[90,66],[-90,66],[-90,0]];"
               "pline[[thick=4],[-90,0],[-90,-66],"
                      "[90,-66],[90,66],[-90,66],[-90,0]];"
               "label[[font=@medium_font],[0,0],\"Group\",Center];"
               "label[[relative=0], [0,0],"
                      "@real_name=\"\" ? #name : @real_name, Center];"  
             "]"
           "]"
         "],";
       }
       if (1) {                                   // Kommentar
          s<<"["
           "name = \"Comment\" Comment;"
           "group = Defs;"
           "nameprefix = C;"

           "attributes = ["
             "text[5] \"Comment\" com = \"\";"
           "];"

           "shape = ["
             "label[[cond=#name=\"\", font=@normal_font], [0,0],"
                    "\"/*rem*/\", Center];"
             "label[[cond=#name>\"\", color=#selected?blue:black,"
                    "font=@normal_font],[0,0], \"/* \" + @com + \" */\",Left];"
           "];"
         "],";
       }

     s<<"],"

     "connectors = [";
       if (structure) { s<<
         "["                                      // Elementarkante
           "name = \"Elementary Arc\" EArc;"
           "group = Arc;"
           "defscale = (0.25,0.25);"
           "relatives = ([0,0], [-50,0]);"
           "attributes = [";
             if (nc!=hcpn) s<<"text \"Weight\" weight = \"\";";
           s<<"];"

           "shape = ["
             "compo[[color=#selected?blue:black, font=@normal_font], [0,0],"
               "pline[[thick=1, spline=@use_splines ? 15 : 0]];"
               "pline[[thick=1, filled=true, cpos=100],"
                     "[-60,20],[0,0],[-60,-20]];"
	             "label[[cond=@weight>\"\" & !(@weight=1), cpos=50,filled=true],"
                     "[0,0], (!@weight)<12 ? @weight : \"...\", Center];"
             "];"
           "];"
           "rules=[[Transition, EPlace], [EPlace, Transition]";
           if (templat) s<<",[Transition, EPPlace], [EPPlace, Transition]";
           s<<"]"
         "],";
       }

       if (petrinet || nc==sstr || nc==swpl) { s<<
         "["                                      // Produktkante
           "name = \"Product Arc\" PArc;"
           "group = Arc;"
           "defscale = (0.25,0.25);"
           "relatives = ([0,0], [-50,0]);"

           "attributes = [";
             if (workplan || nc==swpl)
               s<<"text \"Products\" weight = \"\";"
                  "text \"Branch Condition\" brcond = \"\";"
                  "text \"Branch Weight\" brweight = \"\";";
             if (nc==hcpn) s<<"hidden bool unused = false;";
           s<<"];"

           "shape = ["
             "compo[[color=#selected?blue:";
              s<<(nc==hcpn?"(@show_use & @unused ? grey70 :black),":"black,");
              s<<"font=@normal_font], [0,0],"
                "pline[[thick=2, spline=@use_splines ? 15 : 0]];"
                "pline[[thick=2, filled=true, cpos=100],"
                       "[-60,20],[0,0],[-60,-20]];";
                if (workplan || nc==swpl) s<<
                  "label[[cond=@weight>\"\" & !(@weight=1), cpos=50,"
                         "filled=true], [0,0], "
                         "((!@weight)<12 ? @weight : \"...\")+"
                         "((!@brcond+!@brweight)>0 ? (\"\\n[\"+"
                         "((!@brcond+!@brweight)>13 ? \"...\" : "
                         "((!@brcond)=0 ? @brweight :" 
                         "((!@brweight)=0 ? @brcond:@brcond+\",\"+@brweight)))"
                         "+ \"]\"):\"\"), Center];";
 	            s<<"];"
           "];";
           s<<"rules = [";
           if (petrinet) s<<"[Transition, PPlace], [PPlace, Transition]";
           if (templat) s<<",[Transition, PPPlace], [PPPlace, Transition]";
           if (symbols) {} 
           s<<"]"
         "]";
       }
     
       if (nc==lib)
         for (int toreal=1; toreal>=0; toreal--) {
         s<<
         "[";                                     // Gruppenverbindungskante
           if (toreal) 
             s<<"name = \"Allowed Connection\" ToReal;";
           else
             s<<"name = \"Allowed Connection\" FromReal;";
           s<<"group = Arc;"
           "defscale = (0.25,0.25);"
           "relatives = ([60,0], [-80,0]);"
           "attributes = [];"
           "shape = ["
             "pline[[color=#selected?blue:black, thick=2,"
                     "spline=@use_splines ? 15 : 0]];"
             "compo[[color=#selected?blue:black, font=@normal_font], [0,0],"
               "ellarc[[filled=true, color=white, cpos=0], [30,0], 30];"
               "ellarc[[cpos=0], [30,0], 30];";
               if (toreal)
                 s<<"ellarc[[color=white, dashed=true, cpos=0], [30,0], 30];";
               s<<"ellarc[[filled=true, color=white, cpos=100], [-30,0], 30];"
               "ellarc[[cpos=100], [-30,0], 30];";
               if (!toreal)
                 s<<"ellarc[[color=white,dashed=true,cpos=100], [-30,0], 30];";
               s<<"pline[[thick=0, filled=true, cpos=100],"
                     "[-100,20],[-60,0],[-100,-20]];"
             "];"
           "];"
           "rules=[[TGroup, TGroup]]"
         "];";
       }
     s<<"]]"<<ends;

     new NetClass(classdef);
     
     if (nc==lib) {
       LoadLibrary();
       if (!popp_library) return;
     }
  }
}

//-----------------------------------------------------------------------------
// LoadLibrary ldt das Library-Netz zur Erstellung der Netklassen fr die
// Symbolischen Netze. Dabe wird in mehreren Directories nach der Datei
// Library.net gesucht.
//
void NetClassCreator::LoadLibrary()
{
  FILE* file;
  Text netname = "Library.net";
  Text filename = netname;
  
  file = fopen(filename, "r");
  if (!file) {
    filename = Text(getenv("MODELDIR"))+"/"+netname;
    file = fopen(filename, "r");
  }
  if (!file) {
    filename = Text(getenv("TNETHOME"))+"/MSTranslate/"+netname;
    file = fopen(filename, "r");
  }
  if (!file) { popp_library = 0; return; }

  fclose(file);

  popp_library = Net::Load(filename);  
}

//-----------------------------------------------------------------------------
// CreateAPSYSNode erzeugt einen fr ein Template-Objekt in der Library ein
// ensprechendes Funktionssymbol in der APSYS-Netzklasse.
//
void NetClassCreator::CreateAPSYSNode(Node* node, Node* group, Text path, 
                                      Text shape, bool struc, ostrstream& out)
{
  Error error;

  if (node->Class()->Name()=="Comment") return;
  if (!bool(*node->Attr("symbol"))) return;
  
  Text newshape = charP(*node->Attr("shape"));
  if (newshape!="") shape = newshape;
 
  if (node->Class()->Name()=="TGroup") {
    Iterator<Node*> nodes = node->SubPage()->Nodes();
    for (Node* node2=nodes; node2; node2=++nodes) 
      CreateAPSYSNode(node2, group, charP(path+node->Name()+"_"), 
                      shape, struc, out);
  }
  else if (node->Class()->Name()=="Template" && shape!="") {
    LinkedList<Node*> params;
    TemplateIncluder::GetParameters(node->SubPage(), struc, params, error);
    
    char nodestr[10000];
    strstream s(nodestr, 10000);

    Text real_name = charP(*node->Attr("real_name"));
    s<<"[ name = ";
    if (real_name!="") s<<real_name<<" ";
    s<<charP(path + node->Name())<<";";
    s<<"group = "<<charP(group->Name())<<";";
    s<<"nameprefix = "<<charP(group->Name())<<";";
    s<<"defscale = [0.3, 0.3];";
    s<<"attributes = ["
       "text \"Comment\" comment = \""<<node->Name()<<"\";"; 
      TemplateIncluder::Params2StructDef(params, s, true, error);
      if (error) return;
    s<<"];";
    s<<"shape = [";
    s<<"compo[[color=#selected?blue:black,";
    s<<"font=#name>\"\"?@medium_font:@small_font], [0,0],";
    s<<charP(shape);
    s<<"]]];"<<ends;

    Text testnetcls = Text("netclass [ name=test; nodes = [") + nodestr + "]]";
    NetClass* nc = new NetClass(charP(testnetcls));
    if (!nc->NodeClasses()) {
      Text name = node->FullName();
      error(Error::ErrorInShape, charP(name));
    }
    else out<<nodestr;
    delete nc;
  }
}


//-----------------------------------------------------------------------------
// FindTemplateNet findet fr ein Symbol aus den ABSYS-Netzen das ensprechende
// Template aus der Library.
//
Page* NetClassCreator::FindTemplateNet(Ident name, Page* page, Text path)
{
  if (!page) page = popp_library->Prime();
  Iterator<Node*> nodes = page->Nodes();
  Node* node;

  for (node=nodes; node; node=++nodes) 
    if (node->Class()->Name()=="TGroup") {
      Page* found=FindTemplateNet(name, node->SubPage(),path+node->Name()+"_");
      if (found) return found;
    }
    else if (node->Class()->Name()=="Template") 
      if (path+node->Name() == name) return node->SubPage();

  return 0;
}
