
#include "nodesel.h"

MODULE_NOTIFY(NodeSelector)

//-----------------------------------------------------------------------------
// FindSelf sucht das NodeSelector-Modul im angegebenen Netz
//
NodeSelector* NodeSelector::FindSelf(Net* net)
{
  Iterator<Module*> miter = net->Modules();

  for (Module* m=miter; m; m=++miter)
    if (m->Name()=="NodeSelector")
      return (NodeSelector*) m;
  return 0;
}

//-----------------------------------------------------------------------------
// Activate aktiviert den NodeSelector und merkt sich die angegeben Callbacks.
//
bool NodeSelector::Activate(Net* net, void* caller, NodeSelTestFn test_fn,
                            NodeSelCB callback)
{
  NodeSelector* ns = FindSelf(net);
  if (!ns || ns->active) return false;

  ns->active = true;
  ns->caller = caller;
  ns->test = test_fn;
  ns->callback = callback;

  return true;
}

//-----------------------------------------------------------------------------
// Deactivate deactiviert den NodeSelector
//
bool NodeSelector::Deactivate(Net* net)
{
  NodeSelector* ns = FindSelf(net);
  if (!ns || !ns->active) return false;

  ns->active = false;
  return true;
}

//-----------------------------------------------------------------------------
// IsActive gibt den Status des NodeSelectors zurck
//
bool NodeSelector::IsActive(Net* net)
{
  NodeSelector* ns = FindSelf(net);
  if (!ns || !ns->active) return false;

  return ns->active;
}

//-----------------------------------------------------------------------------
// Bei einem Doppelclick auf eins der gewnschten Objekte wird der Callback
// aufgerufen
//
bool NodeSelector::DoubleClick(Editor&, Page&, Object* obj)
{
  if (!active || !obj->IsNode()) return false;
  Node* node = (Node*) obj;
  
  if ((*test)(caller, node)) {
    (*callback)(caller, node);
    return true;
  }
  else return false;
}

//-----------------------------------------------------------------------------
// Beim Schlieen oder Zerstren des Editors wird der Callback aufgerufen.
// 0 = NodeSelector abgeschossen.
//
void NodeSelector::Close(Editor&, Page&)
{
  if (active) (*callback)(caller, 0);
  active = false;
}

//-----------------------------------------------------------------------------

NodeSelector::~NodeSelector()
{
  if (active) (*callback)(caller, 0);
}

