
#ifndef __RELATION_H__
#define __RELATION_H__

#include "linked.h"

//-----------------------------------------------------------------------------
// Ein Assoziatives Array, es werden nur Pointer auf T1 und T2 verwaltet!

template <class T1, class T2> class Map
{
  friend class MapIterator<T1, T2>;
  LinkedList<T1*> defs;
  LinkedList<T2*> vals;

public:
  virtual bool Compare(T1* a, T1* b) { return a==b; } 
  bool Add(T1*, T2*);
  T2* operator [] (T1*);
  Iterator<T1*> Defs() { return Iterator<T1*>(defs); }
  Iterator<T2*> Vals() { return Iterator<T2*>(vals); }
  virtual ~Map() {}
};

//-----------------------------------------------------------------------------

template <class T1, class T2> class ComplexMap : public Map <T1, T2>
{
public:  
  T2* AddSection(T1* a) { T2* b = new T2; Add(a, b); return b; }
  ~ComplexMap();
};

//-----------------------------------------------------------------------------

template <class T1, class T2> class ValueMap : public Map<T1, T2>
{
public:
  bool Compare(T1* a, T1* b) { return *a==*b; }
};

//-----------------------------------------------------------------------------

template <class T1, class T2> class ComplexValueMap : public ComplexMap<T1, T2>
{
public:
  bool Compare(T1* a, T1* b) { return *a==*b; }
};
  
//-----------------------------------------------------------------------------

template <class T1, class T2> class MapIterator
{
  Iterator<T1*> defsI;
  Iterator<T2*> valsI;

public:
  MapIterator(Map<T1, T2>& a): defsI(a.defs), valsI(a.vals) {}
  void Start(T1*& a, T2*& b) { a=defsI; b=valsI; }
  void Step(T1*& a, T2*& b) { a=++defsI; b=++valsI; }
};

//-----------------------------------------------------------------------------

#endif
