#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include "interface.h"
#include "evalmod.h"
#include "netcl.h"
#include "main.h"
#include "net.h"
#include "error.h"

typedef char* charP;

//-----------------------------------------------------------------------------

class MSSimModule: public EvalModule
{

public:
	bool Init() { return net.Prime()->Class()->Name() == "HCPN"; }

  void Call(Editor& e, Page& p) { EvalModule::Call(e,p); } // ???

  Text EvalSettings();
  bool AllowExperiments() { return true; }
  bool SetValue(Text& name, double val);
  bool EvalCall(bool experiment, Text& command, Text& stdout_file,
                Text& std_infile, Text& net_file, Text& result_file,
                Text& logwin_title, Text& logwin_buttons);
  
  void OnLogwinButton(Ident& button);

  MSSimModule(Net& n): EvalModule(n, "Start Simulation") {}
  ~MSSimModule() {}
};

//-----------------------------------------------------------------------------

Text MSSimModule::EvalSettings()
{
  return 
    "[ int \"Max Simulation Time\" max_time = 0;"
    "  int \"Max Model Time\" max_model_time = 0;"
    "  int \"Max Samples ( x 1000)\" max_samples = 0;"
    "  int \"Min Firings per Binding\" min_firings = 30;"
    "  int \"Seed\" seed = 0;"
    "  bool (\"Auto detect\";\"Ignore\")\"Initial Transient\" detect = true;"
    "  int[90,99] \"Confidence Level in %\" conf_level = 95;"
    "  int[1,50] \"Relative Error in %\" rel_error = 10; ]";
}

//-----------------------------------------------------------------------------

bool MSSimModule::EvalCall(bool experiment, Text& command, Text& stdout_file,
                           Text&, Text& net_file, Text& result_file,
                           Text& logwin_title, Text& logwin_buttons)
{
  Error error(*LastEditor());
  
  Text netname = Text(net.FileName()) - ".net";  
  net_file = netname + ".table";
  stdout_file = netname + ".out";

  if (charP(net_file)[0]!='/') net_file = model_dir + "/" + net_file;
  if (charP(stdout_file)[0]!='/') stdout_file = model_dir + "/" + stdout_file;

  ofstream file((char*) net_file);
  AnalyserInterface::DescribeSymbols(net.Prime(), file); 
  AnalyserInterface::DescribePage(net.Prime(), file, error);
  file.close();
  if (error || !file) return false;

  Text max_sam1000; max_sam1000 = 1000 * (int) Param("max_samples");
  
  command = agnes_dir + "/../MSSimulation/bin/master -n " + netname
          + " -s " + Param("seed")
          + " -m " + max_sam1000
          + " -f " + Param("min_firings")
          + " -c " + Param("conf_level")
          + " -e " + Param("rel_error")
          + " -r " + Param("max_time")
          + " -x " + Param("max_model_time");
  
  if (Param("detect") == "false")
    command = command + " -t";

  result_file = stdout_file;
  logwin_title = "Simulation Window";
  if (!experiment)
    logwin_buttons = "Close Stop \"Temp Results\"";
  return true;
}

//-----------------------------------------------------------------------------

bool MSSimModule::SetValue(Text& name, double val)
{
  istrstream ob = charP(name);
  Object* obj = CommandGetObject(ob);

  if (!obj || obj->Class()->Name() != "Def") return false;
  *obj->Attr("def") = val;
  return true;
}

//-----------------------------------------------------------------------------

void MSSimModule::OnLogwinButton(Ident& button)
{
  if (button == "\"Temp Result\"")
    kill(LastPid(), SIGUSR1); 
  else
    if (button == "Stop")
      kill (LastPid(), SIGINT);
    else
      EvalModule::OnLogwinButton(button);
}

//-----------------------------------------------------------------------------

MODULE_NOTIFY(MSSimModule)


