/*$Header: /home/pdv/timenet/CVS/TimeNET/Print/src/PaintTrans.c,v 1.5 1996/05/10 11:02:04 rg Exp $*/
/*******************************************************************************
Modul           :       PaintTrans

Purpose         :       Erzeugen der PostScript-Befehle zum Drucken
                        einer Transition sowie all ihrer Kanten

Project Director:       Christoph Lindemann TU Berlin

Programmer      :       Christian L"uhe

History         :       release 1: November 1992
			release 2: July     1994
			Joachim Schneider: Anpassung an .TN-Format
*******************************************************************************/

#define PAINTTRANS

#include <string.h>
#include <math.h>
#include "netio.h"
#include "Paint.h"
#include "Allocate.h"

#define INHIBITOR_IMAGE 0
#define ARROWHEAD 1
#define NO_IMAGE 2

#define ARROW_LENGTH 12.0
#define INH_IMAGE_LENGTH 6.0
#define Min(A,B) ((A)>(B)?(B):(A))

#define atan2pi(X,Y) ((double)(atan2((X),(Y))/M_PI))

#ifdef ultrix
/* Spezialbehandlung fuer die DEC-Kisten: */
static char *strdup (string)
char *string;
{
  register char *new_string = (char *)malloc (strlen (string) + 1);
  if (new_string)
    strcpy (new_string, string);
  return new_string;
}
#endif

#ifdef __STDC__
  static void adjustPoint(float x, float y,
                          int section, float obj_size, int image,
                          Objects object,
                          Orientation orient,
                          float *new_x, float *new_y);
  static void arcTo(double r,
                    double ax0, double ay0, 
                    double ax1, double ay1, 
                    double ax2, double ay2,
                    double* lastX, double* lastY);
  static void drawArcIn (double r, 
                         double x0, double y0, 
                         double x3, double y3, 
                         double xc, double yc);
  static void DrawArrowHead(float x1,float y1,float x2,float y2);
  static void DrawInhibA(Arc a,Transition t);
  static void DrawInpA(Arc a,Transition t);
  static void drawLines(Coord *points, int npoints);
  static void drawMultiplicity(Arc *a, float x, float y);
  static void DrawOutpA(Arc a,Transition t);
  static int getOrientationShift(Orientation orient);
  static int getSection(float p1x, float p1y, float p2x, float p2y);
  static Objects objtype(TransitionType t);
  void PaintTrans(NetDescr n, Transition t);
  static void splineLines(Coord *points, int npoints);
#else
  static void adjustPoint();
  static void arcTo();
  static void drawArcIn();
  static void DrawArrowHead();
  static void DrawInhibA();
  static void DrawInpA();
  static void drawLines();
  static void drawMultiplicity();
  static void DrawOutpA();
  static int getOrientationShift();
  static int getSection();
  static Objects objtype();
  void PaintTrans();
  static void splineLines();
#endif



#ifdef __STDC__
  void PaintTrans(NetDescr n, Transition t)
#else
  void PaintTrans(n, t)
    NetDescr n;
    Transition t;
#endif
{
  int alpha, i;
  char *type;

  switch (t.orient) {
    case ORIENT_0   : alpha =   0; break;
    case ORIENT_45  : alpha =  45; break;
    case ORIENT_90  : alpha =  90; break;
    case ORIENT_M45 : alpha = -45; break;
  } /* switch */

  switch (t.type) {
    case EXP : type=strdup("exptrans"); break;
    case DET : type=strdup("dettrans"); break;
    case IMM : type=strdup("imtrans");  break;
    case GEN : type=strdup("gentrans"); break;
  } /* switch */

  printf("%d %f %f %s\n",alpha, t.pos.x, t.pos.y, type); 
  printf ("\t(%s) %f %f tag\n",t.name, t.name_pos.x, t.name_pos.y);

  for (i=0; i<t.num_inarcs; i++) {
    DrawInpA( t.in_arc[i], t);
  } /* for */

  for (i=0; i<t.num_outarcs; i++) {
    DrawOutpA( t.out_arc[i], t);
  } /* for */

  for (i=0; i<t.num_inharcs; i++) {
    DrawInhibA( t.inh_arc[i], t);
  } /* for */

} /* PaintTrans */

/************************************************************/

#ifdef __STDC__
  static void DrawArrowHead(float x1,float y1,float x2,float y2)
#else
  static void DrawArrowHead(x1,y1,x2,y2) /*--------------------------------*/
  /* arrow starts at x1,y1 and shows in direction to x2,y2 */
  float x1,y1,x2,y2;
#endif
{
    printf ("%f %f %f %f arrowhead\n",
            PixToInch(ARROW_LENGTH),
            180.0 * atan2pi((double)(y2-y1),(double)(x2-x1)),
            x1,y1);
} /* DrawArrowHead */

/************************************************************/

#ifdef __STDC__
  static void DrawInpA(Arc a,Transition t)
#else
  static void DrawInpA(a,t) /*-------------------------------------------*/
    Arc a;
    Transition t;
#endif
{
    int section;
    float new_x, new_y, last_x, last_y;
    int i;
    Coord* points = (Coord*)Allocate(a.num_pts+2, sizeof(Coord));

    if (a.num_pts > 0)
        section = getSection(t.pos.x,t.pos.y,a.point[0].x,a.point[0].y);
    else 
        section = getSection(t.pos.x,t.pos.y,
			     a.place->pos.x,a.place->pos.y);
    adjustPoint(t.pos.x,t.pos.y,
		section,PixToInch(DEFAULT_SIZE),
                ARROWHEAD,objtype(t.type),t.orient,
                &new_x,&new_y);
    last_x=new_x; last_y=new_y;

    points[0].x = new_x; points[0].y = new_y; 

    DrawArrowHead(new_x, new_y, t.pos.x, t.pos.y);

    for ( i=0; i<a.num_pts; i++) {
        points[i+1].x = a.point[i].x; points[i+1].y = a.point[i].y;
    }
        
    if (a.num_pts > 0)
        section = getSection(a.place->pos.x,a.place->pos.y,
                             a.point[a.num_pts-1].x,
                             a.point[a.num_pts-1].y);
    else 
        section = getSection(a.place->pos.x, a.place->pos.y,
			     t.pos.x,t.pos.y);
    adjustPoint(a.place->pos.x,a.place->pos.y,
		section,PixToInch(DEFAULT_SIZE),
                NO_IMAGE,PLACE,ORIENT_0,
                &new_x,&new_y);

    points[a.num_pts+1].x = new_x; points[a.num_pts+1].y = new_y;

    drawLines(points, a.num_pts+2);

    if (a.num_pts) {
        last_x = a.point[a.num_pts-1].x;
        last_y = a.point[a.num_pts-1].y;
    }

    drawMultiplicity( &a, (last_x+new_x)/2.0, (last_y+new_y)/2.0);

} /* DrawInpA */

/************************************************************/

#ifdef __STDC__
  static void DrawOutpA(Arc a, Transition t)
#else
  static void DrawOutpA(a,t) /*-------------------------------------------*/
    Arc a;
    Transition t;
#endif
{
    int section;
    float new_x, new_y, last_x, last_y;
    int i;
    Coord* points = (Coord*)Allocate(a.num_pts+2, sizeof(Coord));
    
    if (a.num_pts)
        section = getSection(t.pos.x,t.pos.y,
			     a.point[0].x,a.point[0].y);
    else 
        section = getSection(t.pos.x,t.pos.y,
			     a.place->pos.x,a.place->pos.y);
    adjustPoint(t.pos.x,t.pos.y,
		section,PixToInch(DEFAULT_SIZE),
                NO_IMAGE,objtype(t.type),t.orient,
                &new_x,&new_y);
    last_x=new_x; last_y=new_y;

    points[0].x = new_x; points[0].y = new_y;

    for (i=0; i<a.num_pts; i++) {
        points[i+1].x = a.point[i].x; points[i+1].y = a.point[i].y;
    }
        
    if (a.num_pts > 0)
        section = getSection(a.place->pos.x,a.place->pos.y,
                             a.point[a.num_pts-1].x,
                             a.point[a.num_pts-1].y);
    else 
        section = getSection(a.place->pos.x, a.place->pos.y,
			     t.pos.x,t.pos.y);
    adjustPoint(a.place->pos.x,a.place->pos.y,
		section,PixToInch(DEFAULT_SIZE),
                ARROWHEAD,PLACE,ORIENT_0,
                &new_x,&new_y);

    points[a.num_pts+1].x = new_x; points[a.num_pts+1].y = new_y;

    DrawArrowHead(new_x, new_y, a.place->pos.x, a.place->pos.y); 

    drawLines(points, a.num_pts+2);

    if (a.num_pts) {
        new_x = a.point[0].x;
        new_y = a.point[0].y;
    }

    drawMultiplicity( &a, (last_x+new_x)/2.0, (last_y+new_y)/2.0);

} /* DrawOutpA */

/************************************************************/

#ifdef __STDC__
  static void DrawInhibA(Arc a, Transition t)
#else
  static void DrawInhibA(a,t)
    Arc a;
    Transition t;
#endif
{
    int section;
    float new_x, new_y, last_x, last_y;
    int i;
    Coord* points = (Coord*)Allocate(a.num_pts+2, sizeof(Coord));

    if (a.num_pts > 0)
        section = getSection(t.pos.x,t.pos.y,
			     a.point[0].x,a.point[0].y);
    else 
        section = getSection(t.pos.x,t.pos.y,
			     a.place->pos.x,a.place->pos.y);
    adjustPoint(t.pos.x,t.pos.y,
		section,PixToInch(DEFAULT_SIZE),
                INHIBITOR_IMAGE,objtype(t.type),t.orient,
                &new_x,&new_y);
    last_x=new_x; last_y=new_y;

    points[0].x = new_x; points[0].y = new_y;     

    printf("%f %f %f %f inhibitorimage\n",
            PixToInch(INH_IMAGE_LENGTH),
            180.0 * atan2pi((double)(t.pos.y-new_y),
	    (double)(t.pos.x-new_x)),
            new_x,new_y);

    for (i=0; i<a.num_pts; i++){
        points[i+1].x = a.point[i].x; points[i+1].y = a.point[i].y;
    }

    if (a.num_pts > 0)
        section = getSection(a.place->pos.x,a.place->pos.y,
                             a.point[a.num_pts-1].x,
                             a.point[a.num_pts-1].y);
    else 
        section = getSection(a.place->pos.x,a.place->pos.
			     y,t.pos.x,t.pos.y);
    adjustPoint(a.place->pos.x,a.place->pos.y,
		section,PixToInch(DEFAULT_SIZE),
                NO_IMAGE,PLACE,ORIENT_0,
                &new_x,&new_y);

    points[a.num_pts+1].x = new_x; points[a.num_pts+1].y = new_y;

    drawLines(points, a.num_pts+2);

    if (a.num_pts) {
        new_x = a.point[0].x;
        new_y = a.point[0].y;
    }
    drawMultiplicity(&a, (last_x+new_x)/2.0, (last_y+new_y)/2.0);


} /* DrawInhibA */

/************************************************************/

#define pi M_PI
#define NumSect 24
#define SECTOR_ANGLE (pi/(double)(NumSect/2))

/* this stuff is modified version from DSPN-editor by Martin Mueller */

#ifdef __STDC__
  static Objects objtype(TransitionType t)
#else
  static Objects objtype(t) /*----------------*/
    TransitionType t;
#endif
{
  switch (t) {
    case IMM : return(IMTRANS);
    case EXP : return(EXPTRANS);
    case DET : return(DETTRANS);
    case GEN : return(GENTRANS);
  }
} /* objtype */

/************************************************************/

#ifdef __STDC__
  static int getSection(float p1x, float p1y, float p2x, float p2y)
#else
  static int getSection(p1x,p1y,p2x,p2y) /*-------------------------------------*/
    float p1x,p1y,p2x,p2y;
#endif
{
    double alpha,beta;
    int i;

    if(p1x - p2x != 0){
        alpha = atan((double)(p1y - p2y)/(double)(p1x - p2x));
        for (beta=-pi/2.0+SECTOR_ANGLE/2.0,i=0;
             i<NumSect/2; i++,beta+=SECTOR_ANGLE)
            if(alpha<beta) 
            {
                if(p2x < p1x)
                    return i;
                else 
                    return i+NumSect/2;
            }
    }
    if(p2y < p1y)
        return NumSect/2;
    else 
        return 0;

} /* getSection */

/************************************************************/

#ifdef __STDC__
  static void adjustPoint(float x, float y, int section,
			  float obj_size, int image,
                          Objects object, Orientation orient,
                          float *new_x, float *new_y)
#else
  static void adjustPoint(x,y,section,obj_size,image,object,
			  orient,new_x,new_y)
    float x,y,*new_x,*new_y;
    float obj_size;
    int image,section;
    Objects object;
    Orientation orient;
#endif
{
    double delta_x,delta_y,alpha,length,t_width,nx,ny;
    double sector_angle=SECTOR_ANGLE;
    double double_section;
    
    if (object == PLACE)
        length = (double)obj_size/2.0;
    else
    {
        if (object == EXPTRANS || object == DETTRANS || object == GENTRANS)
            t_width = (double)obj_size/2.0;
        else
            t_width = (double)obj_size/4.0;
        alpha = (double)(abs(/*6 + */getOrientationShift(orient) 
                             - section)%12) *
                                 sector_angle;
        if (alpha == 0.0)
            length = (double)obj_size/2.0;
        else
            length = Min(0.5*sqrt(obj_size*obj_size+t_width*t_width),
                         t_width/(2.0*sin(alpha)));
        /*
          length += determOffSet(section,orient);
          */
    }
    
    if (image == ARROWHEAD) 
        length += PixToInch(ARROW_LENGTH);
    if (image == INHIBITOR_IMAGE)
        length += PixToInch(INH_IMAGE_LENGTH);
    double_section=(double)section;
  delta_x = (length) * cos(double_section * sector_angle);
  delta_y = (length) * sin(double_section * sector_angle);
/*
printf("sector:%d       delta_x: %f delta_y: %f\n",sector,delta_x,delta_y);
*/
  nx = (double)x-(delta_y);
  ny = (double)y+(delta_x);
  *new_x = (float)(nx);
  *new_y = (float)(ny);
}

/************************************************************/

#ifdef __STDC__
  static int getOrientationShift(Orientation orient)
#else
  static int getOrientationShift(orient)
    Orientation orient;
#endif
{
  switch(orient)
  {
    case ORIENT_0:return 0;
    case ORIENT_45:return 3;
    case ORIENT_90:return 6;
    case ORIENT_M45:return 9;
  }
} /* getOrientation */

/************************************************************/

/* this stuff is modified version from DSPN-editor by myself */
#ifdef __STDC__
  /* drawLines with newpath and stroke */
  static void drawLines(Coord *points, int npoints)
#else 
  static void drawLines(points, npoints)
    Coord *points;
    int npoints;
#endif 
{
    int i;

    printf("\tnewpath\n");

    /* setting current point */
    printf("\t%f %f moveto\n", points[0].x, points[0].y);  

    if (spline && npoints > 2)
      splineLines(points, npoints);
    else 
      for (i=1; i<npoints; i++)
            printf("\t\t%f %f lineto\n", points[i].x, points[i].y);
    
    printf("\tstroke\n");

} /* drawLines */

/************************************************************/

#ifdef __STDC__
  static void splineLines(Coord *points, int npoints)
#else 
  static void splineLines(points, npoints)
    Coord *points;
    int npoints;
#endif 
{
    int pIndex = 0;
    double l1, l2, r;
    double lastX, lastY;

    if (npoints > 0) lastX = (double)points[0].x, lastY = (double)points[0].y;

    while ( pIndex+2 < npoints ) {
        l1 = sqrt (pow(lastX-(double)points[pIndex+1].x,2.0) +
                   pow(lastY-(double)points[pIndex+1].y,2.0));
        l2 = sqrt (pow((double)(points[pIndex+1].x-
                                (double)points[pIndex+2].x),2.0) +
                   pow((double)(points[pIndex+1].y-
                                (double)points[pIndex+2].y),2.0));
        if (pIndex+3 != npoints) l2 /= 2.0;

        r = (l1 > l2) ? l2 : l1;
        r = (r > DEFAULT_SPLINE_RADIUS) ? /* resize !!! */
                        (double)DEFAULT_SPLINE_RADIUS : r;

        arcTo(r,
              lastX, lastY,
              (double)points[pIndex+1].x,(double)points[pIndex+1].y,
              (double)points[pIndex+2].x,(double)points[pIndex+2].y,
              &lastX, &lastY);
        pIndex++;
    }
/*    drawLine (nw, 
              (int)lastX, (int)lastY, 
              points[npoints-1].x, points[npoints-1].y); */

    printf("\t\t%f %f lineto\n", points[npoints-1].x, points[npoints-1].y);
}

/************************************************************/

#ifdef __STDC__
  static void arcTo(double r,
		    double ax0, double ay0, 
                    double ax1, double ay1,
		    double ax2, double ay2,
                    double* lastX, double* lastY)
#else
  static void arcTo(r, ax0, ay0, ax1, ay1, ax2, ay2, lastX, lastY)
    double r, ax0, ay0, ax1, ay1, ax2, ay2;
    double *lastX, *lastY;
#endif
{        
    double xt0, yt0, xt2, yt2;
    
    if ( r < 0 ) 
        r = 0.0;


    {        /* Now we have to compute the tangent points. */
        /* Basically, the idea is to compute the tangent */
        /* of the bisector by using tan(x+y) and tan(z/2) */
        /* formulas, without ever using any trig. */
        double dx0 = ax0 - ax1, dy0 = ay0 - ay1;
        double dx2 = ax2 - ax1, dy2 = ay2 - ay1;
        
        /* Compute the squared lengths from p1 to p0 and p2. */
        double sql0 = dx0 * dx0 + dy0 * dy0;
        double sql2 = dx2 * dx2 + dy2 * dy2;
        
        /* Compute the distance from p1 to the tangent points. */
        /* This is the only hairy part. */
        double num = dy0 * dx2 - dy2 * dx0;
        double denom = sqrt(sql0 * sql2) - (dx0 * dx2 + dy0 * dy2);
        
        /* Check for collinear points. */
        if ( fabs(num) < 1.0e-6 || fabs(denom) < 1.0e-6 ) {        
            printf("\t\t%f %f lineto\n", ax1, ay1);
            *lastX = ax1;
            *lastY = ay1;
        }
        else {           /* not collinear */
            double dist = fabs(r * num / denom);
            double l0 = dist / sqrt(sql0), l2 = dist / sqrt(sql2);
            double sqlt, sqlt0, sqlt2, sqlc, sqlm, xm, ym, xc, yc;

            xt0 = ax1 + dx0 * l0;
            yt0 = ay1 + dy0 * l0;
            xt2 = ax1 + dx2 * l2;
            yt2 = ay1 + dy2 * l2;

            /* compute the center of the arc .... */
            /* square length from final points of the arc to the point 
               at which the extended tangents meet */
            sqlt = ( (xt0-ax1)*(xt0-ax1) + (yt0-ay1)*(yt0-ay1) );  
            /* length from tangents cross point to arc center */
            sqlc = ( sqlt + r*r );       /* Pythagoras */
            /* middle point between final points of the arc */
            xm = (xt0+xt2) / 2.0, ym = (yt0+yt2) / 2.0;
            /* length from tangents cross point to middle point */
            sqlm = ( (xm-ax1)*(xm-ax1) + (ym-ay1)*(ym-ay1) );
            /* x,y of the arc center */
            xc = ( ax1 - (ax1-xm) * sqrt(sqlc) / sqrt(sqlm) ),
            yc = ( ay1 - (ay1-ym) * sqrt(sqlc) / sqrt(sqlm) );

#define sqlength(x1,y1,x2,y2) ((x2-x1)*(x2-x1)+(y2-y1)*(y2-y1))
          
            sqlt0=sqlength(ax1,ay1,xt0,yt0);
            sqlt2=sqlength(ax1,ay1,xt2,yt2);
            if ( (sql0 < sqlt0) ||
                 (sql2 < sqlt2)) {
                /* the radius of the arc has to be smaller ... */                
                /* got root from squared length, dont be confused from names */
                sql0 = sqrt(sql0); sql2 = sqrt(sql2);
                sqlt0 = sqrt(sqlt0); sqlt2 = sqrt(sqlt2);

                l0 = sqlt0 - sql0; l2 = sqlt2 - sql2;
                dist = (l0 > l2) ? l0 : l2; /* max */
                
                if (dist==l0) {
                    xt0 = ax0; yt0 = ay0;     /* lastXY */
                    dist = sql0/sqlt2;
                    xt2 = ax1 + (xt2-ax1) * dist;
                    yt2 = ay1 + (yt2-ay1) * dist;
                    xc  = ax1 + (xc -ax1) * dist;
                    yc  = ay1 + (yc -ay1) * dist;
                    r   = r * sql0/sqlt0;
                }
                else {
                    xt2 = ax2; yt2 = ay2;
                    dist = sql2/sqlt0;
                    xt0 = ax1 + (xt0-ax1) * dist; 
                    yt0 = ay1 + (yt0-ay1) * dist; 
                    xc  = ax1 + (xc -ax1) * dist;
                    yc  = ay1 + (yc -ay1) * dist;
                    r   = r * sql2/sqlt2;
                }
            }

/*            drawLine (nw, (int)*lastX, (int)*lastY, (int)xt0, (int)yt0); */
            printf("\t\t%f %f lineto\n", xt0, yt0);

            drawArcIn (r, xt0, yt0, xt2, yt2, xc, yc);

            *lastX = xt2;
            *lastY = yt2;
            printf("\t\t%f %f moveto\n", xt2, yt2);
        }
    }
} /* ArcTo */

/**************************************************************/

#ifdef __STDC__
  static void drawArcIn (double r, 
    double x0, double y0, 
    double x3, double y3, 
    double xc, double yc)
#else
  static void drawArcIn (r, x0, y0, x3, y3, xc, yc)
    double r, x0,  y0,  x3, y3, xc, yc;
#endif
{
    /* angles from center to final points of the arc */
    double angle0 = atan2( (-1.0)*(yc-y0), x0-xc ) / M_PI * 180.0;
    double angle1 = atan2( (-1.0)*(yc-y3), x3-xc ) / M_PI * 180.0;
    double help;

    angle0 = (angle0 < 0.0) ? 360.0 + angle0 : angle0;
    angle1 = (angle1 < 0.0) ? 360.0 + angle1 : angle1;

    help = angle1 - angle0;
    if (help > 180.0 || (help < 0.0 && help > -180.0)) {
      printf("\t\t%f %f moveto  %%CHANGED %f \n", x3, y3,help );
      help = angle1, angle1 = angle0, angle0 = help;
  }

/*    drawArc(nw, (int)(xc-r), (int)(yc-r), 
            (unsigned)r*2, (unsigned)r*2, 
            angle0, help); */

    printf("\t\t%f %f %f %f %f arc\n",  xc, yc, r, angle0, angle1);

} /* drawArcIn */

/**************************************************************/

#ifdef __STDC__
  static void drawMultiplicity(Arc *a, float x, float y)
#else
  static void drawMultiplicity(a, x, y)
    Arc *a;
    float x, y;
#endif
{
    if (a->is_value) {
       if (a->multipl.value > 1)
           printf("\t(%d) %f %f CentreShow\n",
               a->multipl.value,
               x, y);
     }
     else
        printf("\t(mark-dep.) %f %f CentreShow\n",
               x, y);
} /* drawMultiplicity */

/************************************************************/

