/*$Header: /home/pdv/timenet/CVS/TimeNET/Print/src/PrintNet.c,v 1.2 1994/07/22 12:30:32 joa Exp $*/
/*******************************************************************************
  Modul           :       PrintNet
  
  Purpose         :       Erzeugen der LaTeX-Beschreibung des 
  DSPNexpress-Petri-Netzes
  - Markierungen in den Stellen
  - MarkierungsParameter
  - Definitionen der Transitionen
  - Ratenparameter
  - Resultat-Definitionen & -Ergebnisse
  
  Project Director:       Christoph Lindemann TU Berlin
  
  Programmer      :       Christian L"uhe
  
  History         :       release 1: November 1992
                          release 2: September 1993
                          release 3: Juni 1994
                          Joachim Schneider: Anpassung an .TN-Format
*******************************************************************************/

#define PRINTNET
#include <stdio.h>
#include <string.h>
#include <dirent.h>
#include "netio.h"
#include "Result.h"
#include "Structure.h"
#include "Paint.h"
#include "Allocate.h"

#ifdef ultrix
  /* Spezialbehandlung fuer die DEC-Kisten: */
  static char *strdup (string)
  char *string;
{
  register char *new_string = (char *)malloc (strlen (string) + 1);
  if (new_string)
    strcpy (new_string, string);
  return new_string;
}
#endif

#ifdef __STDC__
  static void printSyntax(void);
  static void begin_tabular(char *format, char *head);
  static void end_tabular(void);
  static char *toLatex(char *in);
  static char *toLatexString(char *in);
#else
  static void printSyntax();
  static void begin_tabular();
  static void end_tabular();
  static char *toLatex();
  static char *toLatexString();
#endif

#ifdef __STDC__
  main (int argc, char **argv)
#else
  main (argc, argv)
    int argc;
    char **argv;
#endif
{
    NetDescr *net = NULL;
    ResultT *res;
    StructureT *structure;
    int error=0;
    char *netname = NULL;
    int i=1;
    int layer=0;
    
    if (argc>1)
        while (i<argc) {
            int l;
            
            for (l=1; (l<strlen(argv[i])) && (!error) && (argv[i][0]=='-'); l++)
                switch (argv[i][l]) {
                case 'l' : 
                    if ((!layer) && (sscanf(argv[++i], "%d", &layer))) ;
                    else error=1;
                    break;
                default: error=1;
                } /* switch */
            i++;
        }
    else error=1;
    
    if (error) {
      printSyntax();
      return (1); 
    } /* if */
    
    netname = argv[argc-1];

    /* .TN einlesen */
    net = load_net( netname, NULL );

    /* .RESULT einlesen */
    if( net->num_rewards > 0 ) {
      res                  = Allocate( 1, sizeof( ResultT ) );
      res->availableResult = Allocate( net->num_rewards, sizeof( char ) );
      res->value           = Allocate( net->num_rewards, sizeof( double ) );

      error = ReadResult( net, netname, res );
      if( error ) {
        free( res->value );
        free( res->availableResult );
        free( res );
        res = NULL;
      } /* if */
    } /* if */

    structure           = Allocate( 1, sizeof( StructureT ) );
    structure->invCount = structure->ecsCount = 0;
    structure->inv      = Allocate( 20, sizeof( char* ) );
    structure->ecs      = Allocate( 20, sizeof( char* ) );

    error = ReadStructure( net, netname, structure );
    if( error ) {
      free( structure->invCount );
      free( structure->inv );
      free( structure->ecs );
      free( structure );
      structure = NULL;
    } /* if */
    
    /*----------------------- PLACES ----------------------------*/ 
    if (net->num_places > 0) {
      begin_tabular("|l|r|", "{\\bf Place} & {\\bf Token}");
      for (i=0; i<net->num_places; i++) {
        if (net->place[i].is_value && net->place[i].marking.value==0)
          ; /* dont print all the unmarked places */
        else
          if (net->place[i].is_value) 
            printf("%s & %u \\\\\n",
              toLatex(net->place[i].name),
              net->place[i].marking.value);
          else
            printf("%s & %s \\\\\n", 
              toLatex( net->place[i].name), 
              toLatex( net->place[i].marking.markpar->name));
      } /* for */
      end_tabular();
    } /* if */
    
    if (net->num_markpars) {
      begin_tabular("|l|r|", "{\\bf Token Param.} & {\\bf Value}");
      for ( i=0; i < net->num_markpars; i++) {
        printf("%s & %u \\\\\n",
          net->markpar[i].name, net->markpar[i].value);
      } /* for */
      end_tabular();
    } /* if */
    
    /*----------------------- TRANSES --------------------------*/
{
    unsigned imm=0,
    exp=0,
    det=0,
    gen=0,
    mdep=0,
    amdep=0,
    j;
    
    /* get existence/size of tables */
    for ( i=0; i < net->num_trans; i++) {
        switch (net->trans[i].type) {
        case EXP : 
            exp++;
            break;
        case DET :
            det++;
            break;
        case IMM :
            imm++;
            break;
        case GEN :
            gen++;
            break;
        }
        if( net->trans[i].delay_type == DELAYPAR )
          mdep++;

        for( j=0; j < net->trans[i].num_inarcs; j++) 
          if(! net->trans[i].in_arc[j].is_value)
            amdep++;
        for( j=0; j < net->trans[i].num_outarcs; j++) 
          if(! net->trans[i].out_arc[j].is_value)
            amdep++;
        for( j=0; j < net->trans[i].num_inharcs; j++) 
          if(! net->trans[i].inh_arc[j].is_value)
            amdep++;
        
    } /* for */
    
    /*---------------- Immediate Transs ---*/
    if( imm ) {
      begin_tabular("|l|l|r|",
        "{\\bf Im. Trans.} & {\\bf Priority} & {\\bf Weight}");
      for( i=0; i < net->num_trans; i++)
        if (net->trans[i].type==IMM)
          if (net->trans[i].delay_type==DELAY) 
            printf("%s & %d & %f \\\\\n", 
              toLatex(net->trans[i].name),
              net->trans[i].priority,
              net->trans[i].delay.value);
          else
            printf("%s & %d & %s \\\\\n",
              toLatex(net->trans[i].name),
              net->trans[i].priority,
              toLatex(net->trans[i].delay.delay_par->name));
      end_tabular();
    } /* if */
    
    /*---------------- Exponential Transs ---*/
    if( exp ) {
      begin_tabular("|l|r|", "{\\bf Exp. Trans.} & {\\bf Delay}  ");
      for (i=0; i < net->num_trans; i++)
        if (net->trans[i].type == EXP)
          if (net->trans[i].delay_type==DELAY) 
            printf("%s & %f \\\\\n",
              toLatex(net->trans[i].name),
              net->trans[i].delay.value);
          else
            if (net->trans[i].delay_type==DELAYPAR)
              printf("%s & %s \\\\\n",
                toLatex(net->trans[i].name),
                toLatex(net->trans[i].delay.delay_par->name));
            else  /* type == MD_DEF */
              printf("%s & mark.-dep. \\\\\n",
                toLatex(net->trans[i].name));
      end_tabular();        
    } /* if */

    /*---------------- Deterministic Transs. ---*/
    if( det ) {
      begin_tabular("|l|r|", "{\\bf Det. Trans.} & {\\bf Delay}");
      for (i=0; i < net->num_trans; i++)
        if (net->trans[i].type==DET)
          if (net->trans[i].delay_type==DELAY)
            printf("%s & %f \\\\\n",
              toLatex(net->trans[i].name),
              net->trans[i].delay.value);
          else
            if (net->trans[i].delay_type==DELAYPAR)
              printf("%s & %s \\\\\n",
                toLatex(net->trans[i].name),
                toLatex(net->trans[i].delay.delay_par->name));
            else  /* type == MD_DEF */
              printf("%s & mark.-dep. \\\\\n",
                toLatex(net->trans[i].name));
      end_tabular();        
    } /* if */

    /*---------------- Generalized Transs. ---*/
    if( det ) {
      begin_tabular("|l|r|", "{\\bf Gen. Trans.} & {\\bf Delay}");
      for (i=0; i < net->num_trans; i++)
        if (net->trans[i].type==GEN)
          if (net->trans[i].delay_type==DELAY)
            printf("%s & %f \\\\\n",
              toLatex(net->trans[i].name),
              net->trans[i].delay.value);
          else
            if (net->trans[i].delay_type==DELAYPAR)
              printf("%s & %s \\\\\n",
                toLatex(net->trans[i].name),
                toLatex(net->trans[i].delay.delay_par->name));
            else  /* type == MD_DEF */
              printf("%s & mark.-dep. \\\\\n",
                toLatex(net->trans[i].name));
      end_tabular();        
    } /* if */



    /*------- Rate Parameters ---------*/
    if (net->num_delaypars) {
      begin_tabular("|l|r|", "{\\bf Delay / Weight} & {\\bf Value}");
      for (i=0; i < net->num_delaypars; i++)
        printf("%s & %lf \\\\\n",
          toLatex(net->delaypar[i].name),
          net->delaypar[i].delay.value);
      end_tabular();
    } /* if */

    /*-------------------- Marking dependend Definitions */
    if (mdep) {
      int  z = 0;
      begin_tabular("|l|p{8cm}|",
        "{\\bf Trans.} & {\\bf Mark.-dep. Definition}");
      for (i=0; i < net->num_trans; i++)
        if ( (net->trans[i].delay_type==MARKDEP) ||
             (net->trans[i].delay_type==PMFDEF) ) {
          if (z++)
            printf("\\hline\n");
          printf("%s & %s \\\\\n",
            toLatex(net->trans[i].name),
            toLatex(net->trans[i].delay.md_delay));
        } /* if */
      end_tabular();
    } /* if */

    /*------------------ Marking dependend Definitions */
    if (amdep) {
      int  z = 0;
      begin_tabular("|c|p{8cm}|", 
        "{\\bf Arc} & {\\bf Mark.-dep. Definition}");
      for (i=0; i < net->num_trans; i++) {

        /* |<---O */
        for (j=0; j < net->trans[i].num_inarcs; j++)  
          if (! net->trans[i].in_arc[j].is_value) {
            if (z++)
              printf("\\hline\n");
            printf("%s $\\longleftarrow$ %s & %s \\\\\n",
              toLatex(net->trans[i].name),
              toLatex(net->trans[i].in_arc[j].place->name),
              toLatex(net->trans[i].in_arc[j].multipl.md_mult));
          } /* if */

        /* |---->O */
        for (j=0; j < net->trans[i].num_outarcs; j++)   
          if (! net->trans[i].out_arc[j].is_value) {
            if (z++)
              printf("\\hline\n");
            printf("%s $\\longrightarrow$ %s & %s \\\\\n",
              toLatex(net->trans[i].name),
              toLatex(net->trans[i].out_arc[j].place->name),
              toLatex(net->trans[i].out_arc[j].multipl.md_mult));
          } /* if */

        /* |o---O */
        for (j=0; j < net->trans[i].num_inharcs; j++)  
          if (! net->trans[i].inh_arc[j].is_value) {
            if (z++)
              printf("\\hline\n");
            printf("%s \\raisebox{0.5pt}{$\\circ$}\\hspace{-1pt}{\\bf ---} %s & %s \\\\\n",
              toLatex(net->trans[i].name),
              toLatex(net->trans[i].inh_arc[j].place->name),
              toLatex(net->trans[i].inh_arc[j].multipl.md_mult));
          } /* if */
      } /*for*/
      end_tabular();
    } /* if */
} /* TRANSES */



/*----------------------- RESULTS ---------------------------------*/
if (net->reward && net->num_rewards) {
    char format[25]; /* "%s & %s & %...f \\\\\n" */
    static int z=0;
    
    begin_tabular("|l|p{8cm}|r|", 
      "{\\bf Result Par.} & {\\bf Definition} & {\\bf Value}");
    for (i=0; i < net->num_rewards; i++) {
      if (res->availableResult[i]) {
        if(z++)
          printf("\\hline\n");
        sprintf(format, "%%s & %%s & %%.%uf \\\\\n", res->epsilon);
        printf(format,
          toLatex(net->reward[i].name),
          toLatex(net->reward[i].def),
          res->value[i]);
      } else {
        if(z++)
          printf("\\hline\n");
        sprintf(format, "%%s & %%s & --- \\\\\n");
        printf(format,
          toLatex(net->reward[i].name),
          toLatex(net->reward[i].def) );
      } /* if */
    } /* for */
    end_tabular();
}

/*----------------------- STRUCTURE ---------------------------------*/ 
if (structure && structure->invCount){
    static int z=0;
    char placeInvariants[1024];  /* linear combination of places */
    char value[1024];            /* value of that ^ */

    begin_tabular("|p{12cm}|r|", 
                  "{\\bf Place invariants} & {\\bf Value}");
    for (i=0; i < structure->invCount; i++) {
        if(z++) printf("\\hline\n");
        sscanf(structure->inv[i], "%[^=]", placeInvariants);
        sscanf(structure->inv[i], "%*[^=]=%[^\n]", value);
        printf("%s & %s\\\\\n", toLatex(placeInvariants), toLatex(value));
    }
    end_tabular();
    printf("\\\\\\\\%s", structure->comment);
}

if (structure && structure->ecsCount){
    static int z=0;
    int ecs, priority;
    char transitions[1024];

    begin_tabular("|c|c|p{10.2cm}|",
                  "{\\bf ECS} & {\\bf Priority} & {\\bf Transitions}");
    for (i=0; i < structure->ecsCount; i++) {
        if (z++) printf("\\hline\n");
        sscanf(structure->ecs[i],"ecs %d (priority %d): %[^\n]", 
               &ecs, &priority, transitions);
        printf("%d & %d & %s\\\\\n", 
               ecs, priority, toLatex(transitions));
    }
    end_tabular();
}

/* MAKE CURVES FROM EXPERIMENTS ... */
{
    CurveT curve;
    
    if (!readCurves(netname, &curve))
        makeCurves(netname, &curve);
}   
return (0);
}


#ifdef __STDC__
static void printSyntax (void)
#else
    static void printSyntax ()
#endif
{
    fprintf(stderr,"paint: Syntax-Error!\n");
    fprintf(stderr,
            "syntax: 'print [-l levelnumber] netname'\n");
}

/*-------------------------------------------------LaTex Tabulars-------*/
#ifdef __STDC__
static void begin_tabular(char *format, char *head)
#else
    static void begin_tabular(format, head)
    char *format, *head;
#endif
{
    static int first=1;
    
    if (first)
        first=0;
    else 
        printf("\\newline\n\\newline\n\\newline\n\\newline\n");
    printf("\\begin{tabular}{%s}\n",format);
    printf("\\hline\n\\hline\n");
    printf("%s\\\\\n", head);
    printf("\\hline\n");
}

#ifdef __STDC__
static void end_tabular(void)
#else 
    static void end_tabular()
#endif
{
    printf("\\hline\n\\hline\n");
    printf("\\end{tabular}\n");
}

#ifdef __STDC__
static char *toLatex(char *in)
#else
    static char *toLatex(in)
    char * in;
#endif 
{
    char out[1000];
    char foo[20];
    int i,j,
    k=0,
    len=strlen(in);
    
    for (i=0; i<len && in[i]!=';'; i++)  /* i<len is only for safe */
        switch (in[i]) {
        case '#' : ;
        case '$' : ;
        case '%' : ;
        case '&' : ;
        case '~' : ;
        case '^' : ;
        case '\\' : ;
        case '{' : ;
        case '}' : ;
        case '_' : ;
            out[k++]='\\';
            out[k++]=in[i];
            break;
        case '+' : ;
        case '-' : ;
        case '/' : ;
        case '|' : ;
        case '=' : ;
        case '<' : ;
        case '>' :
            if (strcpy(foo," $ ")==NULL) {
                fprintf(stderr, "PrintNet: Not enough memory (strcpy)!\n");
                exit(1);
            }
            for (j=0;j<strlen(foo);j++)
                out[k++]=foo[j];
            out[k++]=in[i];
            for (j=0;j<strlen(foo);j++)
                out[k++]=foo[j];
            break;
        case '*' :
            if (strcpy(foo," $ \\cdot $ ")==NULL) {
                fprintf(stderr, "PrintNet: Not enough memory (strcpy)!\n");
                exit(1);
            }
            for (j=0;j<strlen(foo);j++)
                out[k++]=foo[j];
            break;
        case '\n' : 
            if (strcpy(foo," \\newline ")==NULL) {
                fprintf(stderr, "PrintNet: Not enough memory (strcpy)!\n");
                exit(1);
            }
            for (j=0;j<strlen(foo);j++)
                out[k++]=foo[j];
            break;
        default: out[k++]=in[i];
        }
    out[k]='\000';
    
    return (strdup(out));
}

#ifdef __STDC__
static char *toLatexString(char *in)
#else
    static char *toLatexString(in)
    char * in;
#endif 
{
    char out[1000];
    int i,
    k=0,
    len=strlen(in);
    
    for (i=0; i<len; i++)
        switch (in[i]) {
        case '_' : 
            out[k++]='\\';
            out[k++]='_';
            break;
        default: out[k++]=in[i];
        }
    out[k]='\000';
    
    return (strdup(out));
}
