#ifndef COMPILERCC
#define COMPILERCC


#include "spnlmodule.cc"
#include "spnlprocess.cc"
#include "spnltypes.cc"
#include "spnlbasics.cc"
#include "vektor.cc"
#include "spnlcheck.cc"
#include "exprcalc.cc"
#include "compiler.h"


CompilerT::CompilerT () {
  errorfree= true;
  newprocess= NULL;
  placect= 0;
  transct= 0;
  arcct= 0;
  paramct= 0;
  rewardct= 0;
  measurect= 0;
  portct= 0;
  procct= 0;
}


CompilerT::~CompilerT () {
//cout << "vor CompilerT\n";
//  delete (newprocess);  // wird schon vom Mainmodule geloescht, das ja den
                          //  Pointer durch compile uebergeben kriegt.
//cout << "nach CompilerT\n";
}


void CompilerT::error_mult (char *id, char *src, char *dst,
			    char *multexpr, char *multvalue) {
  Transition *trans= NULL;
  Place *place= NULL;
  Port *port= NULL;

  cout << "\narc";
  if (id)
    cout << ' ' << id;
  cout << " connecting ";
  trans= newprocess->searchTransition (src);
  if (trans)
    cout << "transition " << trans->longid;
  else {
    place= newprocess->searchPlace (src);
    if (place)
      cout << "place " << place->longid;
    else {
      port= newprocess->searchPort (src);
      if (port)
	cout << "port " << port->longid;
      else
	cout << "unknown";
    }
  }
  cout << " with ";
  trans= newprocess->searchTransition (dst);
  if (trans)
    cout << "transition " << trans->longid;
  else {
    place= newprocess->searchPlace (dst);
    if (place)
      cout << "place " << place->longid;
    else {
      port= newprocess->searchPort (dst);
      if (port)
	cout << "port " << port->longid;
      else
	cout << "unknown";
    }
  }
  cout << " has non-cardinal multiplicity: " << multexpr << "= " << multvalue;
  errorfree= false;
}


void CompilerT::PortEintragen (char *pfad, Port *port, int sectionnr) {
  char *globalid= NULL;
  char *newexpr= NULL;
  Port *p;
  globalid= new char[GLOBALNUMRES];
  globalid[0]= 'o';
  if (++portct > MAXGLOBALANZ) {
    cerr << "\nCompiler error: Too many ports.";
    exit (1);
  }
  sprintf (globalid+1,"%d",portct);
  newexpr= globalize_names (port->getMultiplicity(), pfad);
  p= newprocess->addPort (globalid, PortType2String (port->getPortType()),
			  newexpr);
  if (p) {
    p->oldid= newstring (port->getid());
    p->longid= concat (pfad, port->getid());
    p->section= sectionnr;
  }
  delete (newexpr);
  newexpr= NULL;
  delete (globalid);
  globalid= NULL;
}


void CompilerT::copyPort (ProcessTypeDecl *proc, char *pfad, int sectionnr) {
// Aus proc alle Ports auslesen und in den neuen compilierten Prozess ein-
//  tragen (neu erstellen). Dabei die urspruengliche Id unter oldid ab-
//  speichern, die globale Id neu vergeben (id) und unter longid die kom-
//  plette Id mit vorgesetztem Pfad abspeichern. Ebenfalls sectionnr
//  uebernehmen.
  int j=0;
  Port *port= proc->getPort (j);
  while (port) {
    PortEintragen (pfad, port, sectionnr);
    port= proc->getPort (++j);
  }
}


void CompilerT::copyNcheckPort (ProcessTypeDecl *proc, char *pfad,
				int sectionnr) {
// Traegt port ein, bzw. ergaenzt ihn um type & mult.
  char *newexpr= NULL;
  int j=0;
  bool keinportgefunden= true;
  Port *port= proc->getPort (j);
  while (port) {
    // wenn PortType = usedport => Port eintragen, sonst PortType und
    //  Multiplizitaet zu dem vorherigen Port hinfuegen.
    keinportgefunden= true;
    if (port->getPortType() != USEDPORT) {
      int k= 0;
      char *newportname= concat (pfad, port->getid());
      // newportname ist die neue longid, die hinzugefuegt werden koennte,
      //  dann aber doppelt existieren wuerde, da sie schon als useport
      //  eingetragen seien muesste.
      //  ACHTUNG: Wird die Prozessinstanz aber global deklariert, oder
      //  sind nicht alle Ports der Prozessinstanz graphisch angegeben,
      //  so muss dieser Port doch hinzugefuegt werden.
      Port *oldport= newprocess->getPort (k);
      // oldport laeuft die schon in newprocess uebernommenden Ports durch
      //  und setzt den Porttype (usedport) auf den aktuellen Porttypen
      //  des eigentlich einzutragenden Ports und uebernimmt Multiplizitaet.
      while (oldport) {
	if (!strcmp(oldport->longid, newportname)) {
	  oldport->setPortType (port->getPortType ());
	  keinportgefunden= false;
	  oldport->section= sectionnr;
	  if (port->getMultiplicity()) {
	    newexpr= globalize_names (port->getMultiplicity(), pfad);
	    errorfree&= oldport->setMultiplicity (newexpr);
	    delete (newexpr);
	    newexpr= NULL;
	  }
	}
	oldport= newprocess->getPort (++k);
      }
      delete (newportname);
      newportname= NULL;
    }
    if ((port->getPortType() == USEDPORT) || keinportgefunden)
      PortEintragen (pfad, port, sectionnr);
    port= proc->getPort (++j);
  }
}


void CompilerT::copyReward (ProcessTypeDecl *proc, char *pfad,
			    int sectionnr) {
  int j=0;
  Reward *reward= proc->getReward (j);
  Reward *r;
  char *globalid= NULL;
  char *newexpr= NULL;
  while (reward) {
    globalid= new char[GLOBALNUMRES];
    globalid[0]= reward->getType ();
    globalid[1]= 'r';
    if (++rewardct > MAXGLOBALANZ) {
      cerr << "\nCompiler error: Too many rewards.";
      exit (1);
    }
    sprintf (globalid+2,"%d",rewardct);
    r= newprocess->addReward (globalid, globalid[0], reward->getVisibility());
    if (r) {
      newexpr= globalize_names (reward->getRewardExpression (), pfad);
      newprocess->addRewardExpr (globalid, newexpr);
      delete (newexpr);
      newexpr= NULL;
      r->oldid= newstring (reward->getid());
      r->longid= concat (pfad, reward->getid());
      r->section= sectionnr;
    }
    delete (globalid);
    globalid= NULL;
    reward= proc->getReward (++j);
  }
}


void CompilerT::copyMeasure (ProcessTypeDecl *proc, char *pfad,
			     int sectionnr) {
  int j=0;
  Measure *measure= proc->getMeasure (j);
  Measure *m;
  char *globalid= NULL;
  char *newexpr= NULL;
  while (measure) {
    globalid= new char[GLOBALNUMRES];
    globalid[0]= measure->getType ();
    globalid[1]= 'm';
    if (++measurect > MAXGLOBALANZ) {
      cerr << "\nCompiler error: Too many measures.";
      exit (1);
    }
    sprintf (globalid+2,"%d",measurect);
    m= newprocess->addMeasure (globalid,globalid[0],measure->getVisibility());
    if (m) {
      newexpr= globalize_names (measure->getMeasureExpression (), pfad);
      newprocess->addMeasureExpr (globalid, newexpr);
      delete (newexpr);
      newexpr= NULL;
      m->oldid= newstring (measure->getid());
      m->longid= concat (pfad, measure->getid());
      m->section= sectionnr;
    }
    delete (globalid);
    globalid= NULL;
    measure= proc->getMeasure (++j);
  }
}


void CompilerT::copyParameter (ProcessTypeDecl *proc, char *pfad,
			       int sectionnr) {
  int j=0;
  Parameter *param= proc->getParameter (j);
  Parameter *p;
  char *globalid= NULL;
  char *newexpr= NULL;
  while (param) {
    globalid= new char[GLOBALNUMRES];
    globalid[0]= param->getType ();
    globalid[1]= 'p';
    if (++paramct > MAXGLOBALANZ) {
      cerr << "\nCompiler error: Too many parameters.";
      exit (1);
    }
    sprintf (globalid+2,"%d",paramct);
    newexpr= globalize_names (param->getParameterExpression(), pfad);
    p= newprocess->addParameter (globalid, globalid[0], newexpr);
    delete (newexpr);
    newexpr= NULL;
    if (p) {
      if (param->getGuessExpression ()) {
	newexpr= globalize_names (param->getGuessExpression (), pfad);
	newprocess->addParamGuessExpr (globalid, newexpr);
	delete (newexpr);
	newexpr= NULL;
      }
      p->oldid= newstring (param->getid());
      p->longid= concat (pfad, param->getid());
      p->section= sectionnr;
    }
    delete (globalid);
    globalid= NULL;
    param= proc->getParameter (++j);
  }
}


char *CompilerT::global_globalize (char *a, char *pfad) {
  if (!a)
    return (NULL);
  if (!pfad)
    return (newstring (a));

  char *neu= newstring (a);
  char *zwerg= NULL;
  char *name= NULL;
  int i= 0, j, einfuegepos= 0, restlaenge= 0;
  int pfadlaenge= strlen(pfad);
  ProcessInst *proci= NULL;
  char t;
  // Durchsuche neu nach $Name:  $ und ersetze es durch $pfadName$.
  //  Mittels zwerg wird der alte pointer, der geloescht werden muss,
  //  zwischengespeichert. Evtl. muss neuer pfad gesucht werden.

  while (i<strlen(neu)) {
    if (neu[i] == '$') {  // Beginn Name
      j= i;
      while (!((neu[++j] == '$') || (neu[j] == '.') || (neu[j] == '[')));
        // Ende des Prozessinstanznamens suchen
      zwerg= neu;
      if (neu[j] != '$') {  // Prozessinstanz gef., anderen Pfad vorsetzen
	name= new char[j-i];
	memcpy (name, neu+i+1, j-i-1);
	name[j-i-1]= '\0';
	proci= syntaxchecker.searchProcessInst (NULL, NULL, name);
	if (proci) {
	  pfadlaenge= strlen(proci->getmodid ())+1;  // moduleid + 1x'.'
	  neu= new char[strlen(zwerg)+pfadlaenge+1];  // 1x'\0' + pfadlaenge
	  memcpy (neu, zwerg, i+1);  // alles vor Name reinkopieren
	  memcpy (neu+i+1, proci->getmodid (), pfadlaenge-1);
	  neu[i+pfadlaenge]= '.';
	  memcpy (neu+i+1+pfadlaenge, zwerg+i+1, strlen(zwerg)-i);
                                          // Rest reinkopieren incl. '\0'
	  pfadlaenge= strlen(pfad);
	}
	else {
	  cerr << "\nGlobalize error: global processinstance " << name
	       << " not found";
	  exit (1);
	}
      }
      else {
	neu= new char[strlen(zwerg)+pfadlaenge+1];  // 1x'\0' + pfadlaenge
	memcpy (neu, zwerg, i+1);  // alles vor Name reinkopieren
	memcpy (neu+i+1, pfad, pfadlaenge);  // Pfad reinkopieren
	memcpy (neu+i+1+pfadlaenge, zwerg+i+1, strlen(zwerg)-i);
                                        // Rest reinkopieren incl. '\0'
      }
      delete (zwerg);
      zwerg= NULL;
      while (neu[++i] != '$');
    }
    i++;
  }
  return (neu);
}


void CompilerT::copyProcessInstEntry (ProcessInst *procinst, char *pfad,
				      int sectionnr, bool global) {
// Wenn global==true => Prozessinstanz ist auf globaler Ebene deklariert und
//  darf somit auch Werte anderer Prozessinstanzen enthalten, die lieder evtl.
//  in anderen Modulen deklariert sind. Daher muss, wenn Namen globalisiert
//  werden, der Pfad evtl ein anderer sein.
  ProcessInst *p;
//  char *newarray= NULL;
  char *globalid= new char[GLOBALNUMRES];
  globalid[0]= 'p';
  globalid[1]= 'i';
  if (++procct > MAXGLOBALANZ) {
    cerr << "\nCompiler error: Too many processtypes or -usages.";
    exit (1);
  }
  sprintf (globalid+2,"%d",procct);
/*
  if (procinst->getArrayanz())
    if (atoi (procinst->getArrayanz()) == 0)
      newarray= concat (pfad, procinst->getArrayanz());
            // Arraywerte werden um den Pfad erweitert und uebernommen
    else
      newarray= newstring (procinst->getArrayanz());
  p= newprocess->addProcessInst (globalid, newarray,
				 procinst->getProcessType());
  delete (newarray);
  newarray= NULL;
*/
  p= newprocess->addProcessInst (globalid, NULL, procinst->getProcessType());
  if (p) {
    p->setArrayval (procinst->getArrayval ());
    p->section= sectionnr;
    int k= 0;
    char *parexpr, *parguessexpr;
    if (global) {
      parexpr= global_globalize (procinst->getParameterExpression (k), pfad);
      parguessexpr= global_globalize
	(procinst->getParameterGuessExpression (k), pfad);
    }
    else {
      parexpr= globalize_names (procinst->getParameterExpression (k), pfad);
      parguessexpr= globalize_names
	(procinst->getParameterGuessExpression (k), pfad);
    }
    while (parexpr) {
      newprocess->addProcessInstFormPar (globalid, parexpr, parguessexpr);
      delete (parexpr);
      parexpr= NULL;
      delete (parguessexpr);
      parguessexpr= NULL;
      if (global) {
	parguessexpr= global_globalize
	  (procinst->getParameterGuessExpression (++k), pfad);
	parexpr= global_globalize (procinst->getParameterExpression(k), pfad);
      }
      else {
	parguessexpr= globalize_names
	  (procinst->getParameterGuessExpression (++k), pfad);
	parexpr= globalize_names (procinst->getParameterExpression (k), pfad);
      }
    }
    p->oldid= newstring (procinst->getid());
    p->longid= concat (pfad, procinst->getid());
  }
  delete (globalid);
  globalid= NULL;
}


void CompilerT::copyProcessInst (ProcessTypeDecl *proc, char *pfad,
				 int sectionnr) {
  int j=0;
  ProcessInst *procinst= proc->getProcessInst (j);
  while (procinst) {
    copyProcessInstEntry (procinst, pfad, sectionnr, false);
    procinst= proc->getProcessInst (++j);
  }
}


void CompilerT::copyProcessUsage (ProcessTypeDecl *proc, char *pfad) {
  int j=0;
  ProcessUsage *procuse= proc->getProcessUsage (j);
  ProcessUsage *p= NULL;
  char *globalid= NULL;
  while (procuse) {
    globalid= new char[GLOBALNUMRES];
    globalid[0]= 'p';
    globalid[1]= 'u';
    if (++procct > MAXGLOBALANZ) {
      cerr << "\nCompiler error: Too many processtypes or -usages.";
      exit (1);
    }
    sprintf (globalid+2,"%d",procct);
    p= newprocess->addProcessUsage (globalid, procuse->getProcName());
    if (p) {
      p->oldid= newstring (procuse->getid());
      p->longid= concat (pfad, procuse->getid());
//      p->section= sectionnr;  // hier nicht noetig (beim checken gemacht)
    }
    delete (globalid);
    globalid= NULL;
    procuse= proc->getProcessUsage (++j);
  }
}


void CompilerT::copyPlace (ProcessTypeDecl *proc, char *pfad,
			   int sectionnr) {
  int j=0;
  Place *place= proc->getPlace (j);
  Place *p;
  char *globalid= NULL;
  char *newexpr= NULL;
  while (place) {
    globalid= new char[GLOBALNUMRES];
    globalid[0]= 'p';
    if (++placect > MAXGLOBALANZ) {
      cerr << "\nCompiler error: Too many global places.";
      exit (1);
    }
    sprintf (globalid+1,"%d",placect);
    newexpr= globalize_names (place->getMarking(), pfad);
    p= newprocess->addPlace (globalid, newexpr);
    if (p) {
      p->oldid= newstring (place->getid());
      p->longid= concat (pfad, place->getid());
      p->section= sectionnr;
    }
    delete (newexpr);
    newexpr= NULL;
    delete (globalid);
    globalid= NULL;
    place= proc->getPlace (++j);
  }
}


void CompilerT::copyTransition (ProcessTypeDecl *proc, char *pfad,
				int sectionnr) {
  int j=0;
  Transition *trans= proc->getTransition (j);
  Transition *t;
  char *globalid= NULL;
  while (trans) {
    TransitionType type;
    char *prio= NULL;
    DistributionType dist;
    char *distexpr= NULL;
    char *weight= NULL;
    MarkingDependencyType mdep;
    PolicyType policy;
    ConcurrencyDegree conc;
    char *guard= NULL;
    trans->getAttributes(&type, &prio, &dist, &distexpr, &weight,
			 &mdep, &policy, &conc, &guard);
    globalid= new char[GLOBALNUMRES];
    globalid[0]= 't';
    if (++transct > MAXGLOBALANZ) {
      cerr << "\nCompiler error: Too many global Transitions.";
      exit (1);
    }
    sprintf (globalid+1,"%d",transct);
    t= newprocess->addTransition (globalid, TransitionType2String (type));
    if (t) {
      prio= globalize_names (prio, pfad);
      distexpr= globalize_names (distexpr, pfad);
      weight= globalize_names (weight, pfad);
      guard= globalize_names (guard, pfad);
      newprocess->addTransitionAttr (globalid, prio,
	DistributionType2String (dist), distexpr, weight,
	MarkingDependencyType2String (mdep), PolicyType2String (policy),
	ConcurrencyDegree2String (conc), guard);
      t->oldid= newstring (trans->getid());
      t->longid= concat (pfad, trans->getid());
      t->section= sectionnr;
      delete (prio);
      prio= NULL;
      delete (distexpr);
      distexpr= NULL;
      delete (weight);
      weight= NULL;
      delete (guard);
      guard= NULL;
    }
    delete (globalid);
    globalid= NULL;
    trans= proc->getTransition (++j);
  }
}


void CompilerT::copyArc (ProcessTypeDecl *proc, char *pfad, char *portpfad,
			 int sectionnr) {
  int j=0;
  Arc *arc= proc->getArc (j);
  Arc *a;
  char *globalid= NULL;
  while (arc) {
    ArcType type;
    char *source;
    char *destination;
    char *alt= NULL;
    char *multiplicity;
    arc->getAttributes (&type, &source, &destination, &multiplicity);
    globalid= new char[GLOBALNUMRES];
    globalid[0]= 'a';
    if (++arcct > MAXGLOBALANZ) {
      cerr << "\nCompiler error: Too many global arcs.";
      exit (1);
    }
    sprintf (globalid+1,"%d",arcct);

    multiplicity= globalize_names (multiplicity, pfad);

    // bei Arcs, die auf einen Port zeigen den Pfad nicht mit anhaengen!
    alt= source;
    source= concat (portpfad, source);  // so tun, als ob Arc auf Port zeigt
    if (!newprocess->search4longidInPorts (source)) {
      delete (source);                  // falsche Annahme => Arc zeigt nicht
      source= concat (pfad, alt);       //  auf einen Port
    }

    alt= destination;
    destination= concat (portpfad, destination);  // so tun... (s.o.)
    if (!newprocess->search4longidInPorts (destination)) {
      delete (destination);             // falsche Annahme => Arc zeigt nicht
      destination= concat (pfad, alt);  //  auf einen Port
    }

    a= newprocess->addArc (globalid, source, destination,
			   ArcType2String (type), multiplicity);
    if (a) {
      a->oldid= newstring (arc->getid());
      a->longid= concat (pfad, arc->getid()); 
      a->section= sectionnr;
    }
    delete (multiplicity);
    multiplicity= NULL;
    delete (source);
    source= NULL;
    delete (destination);
    destination= NULL;
    delete (globalid);
    globalid= NULL;
    arc= proc->getArc (++j);
  }
}


void CompilerT::copyFormalParameters (ProcessTypeDecl *proc,
				      ProcessInst *procinst, char *pfad,
				      int sectionnr) {
// in newprocess je nach FormalParameterType aus proc rreward, measparam
//  oder parameter mit oldid= FormalParameter.id aus proc und Wert=
//  GuessParameter.id aus procinst (evtl. ist id auch expression) eintragen.
  int j=0;
  char *globalid= NULL;
  FormalParameter *param= proc->getFormalParameter (j);
  while (param) {
    globalid= new char[GLOBALNUMRES];
    FormalParameterType type= param->getFormalParameterType ();
    if (type == RREWARD) {
      globalid[0]= 'r';
      globalid[1]= 'r';
      if (++rewardct > MAXGLOBALANZ) {
	cerr << "\nCompiler error: Too many rewards.";
	exit (1);
      }
      sprintf (globalid+2,"%d",rewardct);
      Reward *r= newprocess->addReward (globalid, 'r', PRIVATE);
      if (r) {
	// Rewards haben keine GuessExpression.
	newprocess->addRewardExpr (globalid,
				   procinst->getParameterExpression (j));
	r->oldid= newstring (param->getid ());
	r->longid= concat (pfad, param->getid());
	r->section= sectionnr;
      }
    }
    else {
      if (type == MEASPARAM)
	globalid[0]= 'm';
      else
	globalid[0]= 'c';
      globalid[1]= 'p';
      if (++paramct > MAXGLOBALANZ) {
	cerr << "\nCompiler error: Too many parameters.";
	exit (1);
      }
      sprintf (globalid+2,"%d",paramct);
      Parameter *p= newprocess->addParameter (globalid, globalid[0],
		       procinst->getParameterExpression (j));
      if (p) {
	p->oldid= newstring (param->getid());
	p->longid= concat (pfad, param->getid());
	p->section= sectionnr;  // ********* hier evtl. noch eine section mehr
	if ((type == MEASPARAM) && (procinst->getParameterGuessExpression(j)))
	  newprocess->addParamGuessExpr (globalid,
			 procinst->getParameterGuessExpression (j));
      }
    }
    delete (globalid);
    globalid= NULL;
    param= proc->getFormalParameter (++j);
  }
}


void CompilerT::copyCompleteProcess (ProcessTypeDecl *proc, char *pfad) {
  // Prozess proc auslesen und alle Elemente mit longid versehen und in
  //  Prozess newprocess uebertragen
  int sectionnr= proc->section;
  copyPort (proc, pfad, sectionnr);
  copyReward (proc, pfad, sectionnr);
  copyMeasure (proc, pfad, sectionnr);
  copyParameter (proc, pfad, sectionnr);
  copyProcessInst (proc, pfad, sectionnr);
  copyProcessUsage (proc, pfad);
  copyPlace (proc, pfad, sectionnr);
  copyTransition (proc, pfad, sectionnr);
  copyArc (proc, pfad, pfad, sectionnr);
}


void CompilerT::copyProcessCompleteInst (ProcessTypeDecl *proc,
					 ProcessInst *procinst, char *pfad) {
  // Prozess proc auslesen und alle Elemente mit longid versehen und in
  //  Prozess newprocess uebertragen. Dann die Uebergabeparameter der
  //  ProzessInstanz aus procinst mit denen von proc gleichsetzen.

/*
  char *anz= procinst->getArrayanz ();
  if (anz) {        
    int arraygroesse= newprocess->getParameterValue (anz);
*/
  int arraygroesse= procinst->getArrayval ();
  int sectionnr= procinst->section;
  if (arraygroesse) {
    char *pfadneu= NULL;
    for (int i=1; i<= arraygroesse; i++) {
      pfadneu= new char[strlen (pfad)+GLOBALNUMRES];
      strcpy (pfadneu, pfad);           // pfad pro Durchlauf veraendern
      pfadneu[strlen(pfadneu)-1]= '[';
      sprintf (pfadneu+strlen(pfadneu), "%d", i);
                   // Zaehler i nach String wandeln und einfuegen
      strcat (pfadneu, "].");           // "]." eintragen
      copyNcheckPort (proc, pfadneu, sectionnr);
      // Ports sollen nur einmal existieren, das heisst, hier wird bei Pro-
      //  zessen Muell gebaut, der spater wieder beruecksichtigt werden muss.
      copyReward (proc, pfadneu, sectionnr);
      copyMeasure (proc, pfadneu, sectionnr);
      copyParameter (proc, pfadneu, sectionnr);
      copyProcessInst (proc, pfadneu, sectionnr);
      copyProcessUsage (proc, pfadneu);
      copyPlace (proc, pfadneu, sectionnr);
      copyTransition (proc, pfadneu, sectionnr);
      copyArc (proc, pfadneu, pfad, sectionnr);
      copyFormalParameters (proc, procinst, pfadneu, sectionnr);
      delete (pfadneu);
      if (dots)
	cout << '.' << flush;
    }
  }
  else {   // keine array-instanz
    copyNcheckPort (proc, pfad, sectionnr);
    copyReward (proc, pfad, sectionnr);
    copyMeasure (proc, pfad, sectionnr);
    copyParameter (proc, pfad, sectionnr);
    copyProcessInst (proc, pfad, sectionnr);
    copyProcessUsage (proc, pfad);
    copyPlace (proc, pfad, sectionnr);
    copyTransition (proc, pfad, sectionnr);
    copyArc (proc, pfad, pfad, sectionnr);
    copyFormalParameters (proc, procinst, pfad, sectionnr);
    if (dots)
      cout << '.' << flush;
  }
}


void CompilerT::includeProcessUsage (ProcessUsage *proc, char *pfad) {
  // In proc->longid steht der alte Pfad, der durch pfad in den Arc-
  //  attributen ersetzt werden muss. Bei den alten Ports muessen diese
  //  geloescht werden (sie existieren ja weiter als Processport)
  pfad[strlen(pfad)-1]= '\0';
  int i= 0;
  ArcType atype;
  char *source;
  char *destination;
  char *multiplicity;
//cout << "\n replacing " << proc->longid << " with " << pfad << ".";

  Arc *arc= newprocess->getArc (i);
  while (arc) {
    arc->getAttributes (&atype, &source, &destination, &multiplicity);
    if (search_substring (source, proc->longid)) {
      source= replace_substring (source, proc->longid, pfad);
      arc->deleteSource ();
      errorfree&= arc->setSource (source);
      delete (source);
      source= NULL;
    }
    if (search_substring (destination, proc->longid)) {
      destination= replace_substring (destination, proc->longid, pfad);
      arc->deleteDestination ();
      errorfree&= arc->setDestination (destination);
      delete (destination);
      destination= NULL;
    }
    arc= newprocess->getArc (++i);
  }

  // in allen Transitionen (prio, dist, weight, guard jeweils reward-expr.),
  //  Rewards und Measures ebenfalls ersetzten
  char *expr= NULL;
  i= 0;
  Reward *r= newprocess->getReward (i);
  while (r) {
    expr= r->getRewardExpression();
    if (search_substring (expr, proc->longid)) {
      expr= replace_substring (expr, proc->longid, pfad);
      r->deleteRewardExpression ();
      errorfree&= r->setRewardExpression (expr);
      delete (expr);
    }
    r= newprocess->getReward (++i);
  }
    
  i= 0;
  Measure *m= newprocess->getMeasure (i);
  while (m) {
    expr= m->getMeasureExpression();
    if (search_substring (expr, proc->longid)) {
      expr= replace_substring (expr, proc->longid, pfad);
      m->deleteMeasureExpression ();
      errorfree&= m->setMeasureExpression (expr);
      delete (expr);
    }
    m= newprocess->getMeasure (++i);
  }
  
  TransitionType ttype;
  char *prio= NULL;
  DistributionType dist;
  char *distexpr= NULL;
  char *weight= NULL;
  MarkingDependencyType mdep;
  PolicyType policy;
  ConcurrencyDegree conc;
  char *guard= NULL;
  i= 0;
  Transition *t= newprocess->getTransition (i);
  while (t) {
    t->getAttributes(&ttype, &prio, &dist, &distexpr, &weight,
		     &mdep, &policy, &conc, &guard);
    if (search_substring (prio, proc->longid)) {
      prio= replace_substring (prio, proc->longid, pfad);
      t->deletePriority ();
      errorfree&= t->setPriority (prio);
      delete (prio);
      prio= NULL;
    }
    if (search_substring (distexpr, proc->longid)) {
      distexpr= replace_substring (distexpr, proc->longid, pfad);
      t->deleteDistributionExpression ();
      errorfree&= t->setDistributionExpression (distexpr);
      delete (distexpr);
      distexpr= NULL;
    }
    if (search_substring (weight, proc->longid)) {
      weight= replace_substring (weight, proc->longid, pfad);
      t->deleteWeight ();
      errorfree&= t->setWeight (weight);
      delete (weight);
      weight= NULL;
    }
    if (search_substring (guard, proc->longid)) {
      guard= replace_substring (guard, proc->longid, pfad);
      t->deleteGuard ();
      errorfree&= t->setGuard (guard);
      delete (guard);
      guard= NULL;
    }
    t= newprocess->getTransition (++i);
  }
}


char *CompilerT::calculateMultiplicity (char *iarcmult, char *oarcmult,
					char *portmult) {
  char *newmult= NULL;
  const NOMULT= 0,
        MULT= 1,
        PORTMULT= 2;
  int iarcval= NOMULT,
      oarcval= NOMULT,
      portval= NOMULT;

  if (portmult && strcmp (portmult, "1"))
    portval= MULT;
  if (iarcmult && !strcmp (iarcmult, "portdep") && portval)
    iarcval= PORTMULT;
  if (iarcmult && strcmp (iarcmult, "1") && strcmp (iarcmult, "portdep"))
    iarcval= MULT;
  if (oarcmult && !strcmp (oarcmult, "portdep") && portval)
    oarcval= PORTMULT;
  if (oarcmult && strcmp (oarcmult, "1") && strcmp (oarcmult, "portdep"))
    oarcval= MULT;

  if (!((!portval && (!(iarcval || oarcval) || (iarcval == PORTMULT
      && oarcval == PORTMULT))) || (iarcval == PORTMULT && !oarcval)
      || (!iarcval && oarcval == PORTMULT))) {
    // Multiplizitaet des neues Arcs bestrimmen:

    if (oarcval == MULT
	     && ((iarcval == PORTMULT) || (iarcval == NOMULT && !portval)))
      newmult= newstring (oarcmult);

    else if (iarcval == MULT
	     && ((oarcval == PORTMULT) || (oarcval == NOMULT && !portval)))
      newmult= newstring (iarcmult);

    else if (iarcval == PORTMULT && oarcval == PORTMULT && portval == MULT)
      newmult= newstring (portmult);

    else if (iarcval == MULT && oarcval == MULT && portval == MULT) {
      newmult= new char[strlen(iarcmult) + strlen(oarcmult)
			+ strlen(portmult) + 11];  // init "(()*())/()\0"
      strcpy (newmult, "((");
      strcat (newmult, iarcmult);
      strcat (newmult, ")*(");
      strcat (newmult, oarcmult);
      strcat (newmult, "))/(");
      strcat (newmult, portmult);
      strcat (newmult, ")");
    }

    else if (iarcval == NOMULT && oarcval == MULT && portval == MULT) {
      newmult= new char[strlen(oarcmult) + strlen(portmult) + 6];
      strcpy (newmult, "(");
      strcat (newmult, oarcmult);
      strcat (newmult, ")/(");
      strcat (newmult, portmult);
      strcat (newmult, ")");
    }

    else if (iarcval == MULT && oarcval == NOMULT && portval == MULT) {
      newmult= new char[strlen(iarcmult) + strlen(portmult) + 6];
      strcpy (newmult, "(");
      strcat (newmult, iarcmult);
      strcat (newmult, ")/(");
      strcat (newmult, portmult);
      strcat (newmult, ")");
    }

    else if (iarcval == NOMULT && oarcval == NOMULT && portval == MULT) {
      newmult= new char[strlen(portmult) + 5];
      strcpy (newmult, "1/(");
      strcat (newmult, portmult);
      strcat (newmult, ")");
//      cerr << "\nGlobalize error: two arcs without multiplicity are "
//	   << "connected by a port with portmult= " << portmult << ".";
//      exit (1);
    }

    else if (iarcval == MULT && oarcval == MULT && !portval) {
      newmult= new char[strlen(iarcmult) + strlen(oarcmult)
			+ strlen(portmult) + 6];  // init "()*()\0"
      strcpy (newmult, "(");
      strcat (newmult, iarcmult);
      strcat (newmult, ")*(");
      strcat (newmult, oarcmult);
      strcat (newmult, ")");
    }
  }	
  return (newmult);
}


void CompilerT::removePorts () {
  Port *port= newprocess->getPort (0);
  char *portid= NULL;
  Vektor <Arc> InList, OutList;
  char *iarcsource= NULL, *iarcdest= NULL, *iarcmult= NULL,
       *oarcsource= NULL, *oarcdest= NULL, *oarcmult= NULL;
  ArcType iarctype, oarctype;
  char *globalid= NULL, *newmult= NULL;
  int sectionnr;
  int i,j;
  while (port) {
    // In einer Liste die eingehenden Kanten sammeln, in einer anderen
    //  die Ausgehenden. Am Ende der Prozedur werden die Listen automatisch
    //  geloescht und rufen auch die Destruktoren der einzelnen Arcs auf,
    //  die vorher dupliziert wurden.
    portid= port->getid();
    i=0;
    Arc *arc= newprocess->getArc (i);
    while (arc) {
      arc->getAttributes (&iarctype, &iarcsource, &iarcdest, &iarcmult);
      if (!strcmp (iarcsource, portid) || !strcmp (iarcdest, portid)) {
	// Arcsource oder -dest stimmen mit einem Port ueberein
	Arc *a= new Arc (MODID, PROCID, arc->getid());  // Arc dublizieren
	errorfree&= a->setSource (iarcsource);
	errorfree&= a->setDestination (iarcdest);
	if (iarctype)
	  a->setArcType (iarctype);
	if (iarcmult)
	  errorfree&= a->setMultiplicity (iarcmult);
	if (!strcmp (iarcsource, portid))  // Arc in die entsprechende Liste
	  OutList.addElement (a);          //  eintragen
	else
	  InList.addElement (a);
      }
      arc= newprocess->getArc (++i);
    }

    // Wenn beide Listen jeweils mindestens ein Element enthalten, Arcs neu
    //  verknuepfen:
    if ((InList.size()>0) && (OutList.size()>0)) {
      sectionnr= port->section;
      for (i= 0; i< InList.size(); i++)  // Arcs in beiden Listen durchgehen
	for (j= 0; j< OutList.size(); j++) {
	  globalid= new char[GLOBALNUMRES];
	  globalid[0]= 'p';
	  globalid[1]= 'a';
	  if (++arcct > MAXGLOBALANZ) {
	    cerr << "\nCompiler error: Too many arcs during compiling.";
	    exit (1);
	  }
	  sprintf (globalid+2,"%d",arcct);
	  InList.readElementAt(i)->getAttributes (&iarctype, &iarcsource,
						  &iarcdest, &iarcmult);
	  OutList.readElementAt(j)->getAttributes (&oarctype, &oarcsource,
						   &oarcdest, &oarcmult);
	  if ((iarctype != oarctype) || strcmp (oarcsource, iarcdest)
	      || strcmp (oarcsource, port->getid())) {
	    cerr << "\narc-port-connecting-error.";
	    exit (1);
	  }
	  bool intval;
	  newmult= calculateMultiplicity (iarcmult, oarcmult,
					  port->getMultiplicity ());
	  Arc *a= newprocess->addArc (globalid, iarcsource, oarcdest,
				      ArcType2String (iarctype), newmult);
	  if (a) {
	    a->longid= newstring (port->longid);
	    a->section= sectionnr;
	  }
          delete (globalid);
	  globalid= NULL;
          delete (newmult);
	  newmult= NULL;
	}
      if (dots)
	cout << '.' << flush;
    }

    // alle Arcs beider Listen loeschen (und zwar auch im Prozess):
    Arc *a= NULL;
    while (InList.size() > 0) {
      a= InList.removeElementAt (0);
      newprocess->removeArc (a->getid());
//      delete (a);
    }
    while (OutList.size() > 0) {
      a= OutList.removeElementAt (0);
      newprocess->removeArc (a->getid());
//      delete (a);
    }
    a= NULL;
    newprocess->removePort (0);
    port= newprocess->getPort (0);
  }
}


void CompilerT::calculateArcs () {
  int i=0;
  char *newmult= NULL;
  ArcType type;
  char *source, *destination, *multiplicity;
  ExprCalc calculator (newprocess);
  bool intvalpt;
  Arc *arc;

  arc= newprocess->getArc (i);
  while (arc) {
//cout << "\nwhile arc[" << i << "]";
    arc->getAttributes (&type, &source, &destination, &multiplicity);
    if (multiplicity)
      calculator.pushExpression (multiplicity);
    if (dots && !(i%10))
      cout << '.' << flush;
    arc= newprocess->getArc (++i);
  }

  calculator.calculateExpressions ();

  i= 0;
  arc= newprocess->getArc (i);
  while (arc) {
    arc->getAttributes (&type, &source, &destination, &multiplicity);
    if (multiplicity) {
      newmult= calculator.popValue (&intvalpt);
      if (newmult && intvalpt) {
        arc->deleteMultiplicity ();
	arc->setMultiplicity (newmult);
      }
      if (!intvalpt)
	error_mult (arc->longid, source, destination, multiplicity, newmult);
      delete (newmult);
    }
    if (dots && !(i%70))
      cout << '.' << flush;
    arc= newprocess->getArc (++i);
  }
}


int CompilerT::getSectionCount () {
  return (syntaxchecker.getSectionCount ());
}


ProcessTypeDecl *CompilerT::compile (Vektor <Module> *mod) {
// Achtung: Wichtiger Hinweis 1:
//  Wenn die Funktion folgendermassen aufgerufen wird:
//  compile (Vektor <Module> mod), werden falscherweise die
//  Destruktoren der module aufgerufen. => Absturz beim
//  eigentlichen Loeschen;
  int i;
  char *pfad= NULL;
  char *zwerg= NULL;
  int modulindex= 0;
  Module *modulept= NULL;

  placect= 0;   // Alle counter fuer globale Identifier vor dem
  transct= 0;   //  Compilieren auf 0 setzen
  arcct= 0;
  paramct= 0;
  rewardct= 0;
  measurect= 0;
  portct= 0;
  procct= 0;
  newprocess= new ProcessTypeDecl (MODID, PROCID);
  newprocess->setType (DECL);

  modules= mod;  // zu compilierende Module vermerken
  // Achtung: Wichtiger Hinweis 2:
  //  Niemals Vektor <Modules> modules= *mod; deklarieren, da so
  //  eine doppelte Verzeigerung auf ein- und dasselbe Objekt
  //  existiert und so die Module zwei mal geloescht werden.

// Vorgehensweise des Compilers:
//  1.) Module auf Korrektheit checken (intern & extern) durch spnlcheck.cc
//  2.) Alle globalen Parameter aus allen Modulen in den Hautprozess kopieren.
//  3.) Alle Prozesse aus allen Modulen in den Hautprozess kopieren.
//       Aufrufe aller globalen Prozessinstanzen uebernehmen.
//       Dabei gleichzeitig globale Identifiers (id) vergeben, Identifier
//       mit Pfad-Angaben (longid) eintragen und die alte id als oldid
//       vermerken.
//  4.) Solange Prozessinstanzen existieren, diese miteinfuegen (bringen 
//       evtl. weitere Prozessinstanzen/usages). Hierbei (und bei 5. auch)
//       die section-Nummer weitergeben an alle Objekte.
//  5.) Solange Prozessusages existieren, diese miteinfuegen (bringen keine 
//       weiteren Prozessinstanzen), dabei identische Ports aufeinander
//       abbilden und section-Nummer weiterreichen bzw. eintragen.
//  6.) Namen in allen Ausdruecken und Elementen auf globale Identifier
//       umlegen.
//  7.) Ports entfernen (wenn Arcs nur hin oder weg, diese loeschen)

  errorfree= syntaxchecker.checkSyntax (mod); // 1.) auf Korrektheit checken

  if (!errorfree) {            // im Fehlerfall gleich beenden
    delete (newprocess);
    return (NULL);
  }

  // Flaches Netz aus allen Modulen neu zusammenstellen:
  for (modulindex= 0; modulindex < modules->size(); modulindex++) {

    modulept= modules->readElementAt (modulindex);

    i=0;
    // 2.) Parameter aus allen Modulen in newprocess kopieren
    Parameter *param= modulept->getParameter (i);
    Parameter *param2;
    char *globalid= NULL;
    while (param) {
      globalid= new char[GLOBALNUMRES];
      globalid[0]= 'c';
      globalid[1]= 'p';
      if (++paramct > MAXGLOBALANZ) {
	cerr << "\nCompiler error: Too many parameters.";
	exit (1);
      }
      sprintf (globalid+2,"%d",paramct);
        // PRE: Parameter duerfen nur einmal existieren.
      // Pfad fuer globale parameter immer weglassen, da in globalen
      // Parametern nur Zahlen oder andere globale Parameter be-
      // nutzt werden koennen.
      param2= newprocess->addParameter (globalid, 'c',
					param->getParameterExpression());
      if (param2) {
	param2->oldid= newstring (param->getid());
	param2->longid= newstring (param->getid());
      }
      delete (globalid);
      globalid= NULL;  // Wichtig: Immer auf NULL setzen!
      param= modulept->getParameter (++i);
      if (dots)
	cout << '.' << flush;
    }

    i=0;
    // 3a.) Prozessinstanzen aus allen Modulen in newprocess kopieren
    //      (nicht aufloesen)
    ProcessInst *procinst= modulept->getProcessInst (i);
    pfad= concat (modulept->getid(), ".");
    while (procinst) {
      copyProcessInstEntry (procinst, pfad, procinst->section, true);
      procinst= modulept->getProcessInst (++i);
      if (dots)
	cout << '.' << flush;
    }
    delete (pfad);
    pfad= NULL;

    i=0;
    // 3b.) Elemente aller Prozessdeklarationen aller Module in newprocess
    //      kopieren. Dabei werden die im spnl-check vergebenen sections
    //      auch mit einkopiert.
    ProcessTypeDecl *proc= modulept->getProcess (i);
    while (proc) {
      if (proc->getType () == DECL) {
	pfad= pointed_concat (proc->getmodid(), proc->getid());
	copyCompleteProcess (proc, pfad);
	delete (pfad);
	pfad= NULL;
      }
      proc= modulept->getProcess (++i);
      if (dots)
	cout << '.' << flush;
    }
  }

  if (dots)
    cout << "net copied\n";

  // 4.) solange ProcessInstanzen existieren, immer eine herausgreifen und 
  //     in den Prozess eintragen.
  i= 0;
  ProcessInst *procinst= newprocess->getProcessInst (i);
  while (procinst) {
    // Nach procinst-typ in allen Modulen suchen und in proc vermerken
    ProcessTypeDecl *proc= syntaxchecker.searchProcessTypeDecl
                                         (NULL, procinst->getProcessType ());
    if (!proc || (proc->getType () == DECL)) {
      cout << "\nprocess instance " << procinst->longid
	   << " declared, but type " << procinst->getProcessType ()
	   << " not found.";
      errorfree= false;
    }
    else {  // proc: processtype; procinst: deren Aufruf
      pfad= concat (procinst->longid, ".");
      // proc mit zusaetzlichen Parametern aus dem procinst-Aufruf eintragen.
      copyProcessCompleteInst (proc, procinst, pfad);
      delete (pfad);
      pfad= NULL;
    }
    procinst= newprocess->getProcessInst (++i);
  }

  if (dots)
    cout << "all process instances copied\n";

  // 5.) solange ProcessUsage existieren, immer eine herausgreifen und in dem
  //     Prozess verknuepfen.
  ProcessUsage *procuse= newprocess->getProcessUsage (0);
  while (procuse) {
    // Nach diesem Typ in allen Modulen suchen und in proc vermerken
    ProcessTypeDecl *proc= syntaxchecker.searchProcessTypeDecl
                                         (NULL, procuse->getProcName ());
    if (!proc || (proc->getType () == TYPE)) {
      cout << "\nprocess usage " << procuse->longid << " declared, but "
	   << "process " << procuse->getProcName () << " not found.";
      errorfree= false;
    }
    else {
      // Modul gefunden => neuen Namen basteln und alle alten Namen in den
      //  Arcs, Transitions, Rewards und Measures ersetzen.
      pfad= pointed_concat (proc->getmodid (), procuse->getProcName ());
      includeProcessUsage (procuse, pfad);
      delete (pfad);
      pfad= NULL;
    }
    newprocess->removeProcessUsage (0);
    procuse= newprocess->getProcessUsage (0);
    if (dots)
      cout << '.' << flush;
  }

  if (dots)
    cout << "all process usages copied\n";

  errorfree&= newprocess->errorfree;

  // 6.) In allen Ausdruecken, Parametern usw. die Referenzen auf andere
  //     Variablen durch deren neue (globale) id ersetzen.
  if (errorfree)
    errorfree&= newprocess->globalize ();
  if (errorfree && dots)
    cout << "global references created\n";

  // 7.) Alle Ports entfernen und die betroffenden Arcs durch neue ersetzen
  if (errorfree)
    removePorts ();
  if (errorfree && dots)
    cout << "all ports removed\n";

  if (calcarcmult && errorfree) {
    calculateArcs ();
    if (dots)
      cout << "all arc multiplicities calculated\n";
  }

  if (!errorfree && !errornetdisplay) {
    delete (newprocess);
    return (NULL);
  }
  else
    return (newprocess);
}



#endif



