#ifndef COMPILERH
#define COMPILERH



char *MODID=  "compiled_spnl_net",
     *PROCID= "resulting_net";

class CompilerT {
  private:
    bool errorfree;
    int placect,       // Counter fuer eindeutigen globalid (id).
        transct,       //  Wird bei jedem neuen Eintrag einer Transition,
        arcct,         //  Platz, Kante, Parameter, ... un eins hochgezaehlt
        paramct,       //  und darf den Wert von MAXGLOBALANZ nicht ueber
        rewardct,      //  schreiten.
        measurect,
        portct,
        procct;
    Vektor <Module> *modules;   // Liste aller zu compilierenden Module

    ProcessTypeDecl *newprocess;   // Fertig compilierter Prozess, wird durch
                                   //  compile gefuellt.

    SyntaxCheckerT syntaxchecker;  // Syntaxchecker wird vor der Compilation
                                   //  aufgerufen und stellt sicher, dass
                                   //  ein korrektes Netz uebersetzt wird.

    void error_mult (char *id, char *src, char *dst, char *multexpr,
		     char *multvalue);
    // Fehlermeldung, dass arc id von src nach dst eine ungueltige Multipli-
    //  zitaet hat.

    void PortEintragen (char *pfad, Port *port, int sectionnr);
    // Port port in den neuen compilierten Prozess eintragen (neu erstellen).
    //  Dabei die urspruengliche Id unter oldid abspeichern, die globale Id
    //  neu vergeben (id) und unter longid die komplette Id mit vorgesetztem
    //  Pfad abspeichern. Ebenfalls sectionnr uebernehmen.
    // Bei den Ports muss der portpfad davorgehaengt werden (da diese bei
    //	Arrays nicht mehrfach existieren duerfen).

    void copyPort (ProcessTypeDecl *proc, char *pfad, int sectionnr);
    // Aus proc alle Ports auslesen und mittels PortEintragen in den neuen
    //  compilierten Prozess eintragen.

    void copyNcheckPort (ProcessTypeDecl *proc, char *pfad, int sectionnr);
    // Aus proc alle Ports auslesen und, wenn:
    //  1.) PortType == usedport => Port in den neuen compilierten Prozess
    //                              eintragen (mittels PortEintragen)
    //  2.) PortType != usedport => PortType und Multiplizitaet zu dem vor-
    //                              herigen Port hinfuegen.
    //  zu 2.) Wird eine Prozessinstanz aber global deklariert, oder sind
    //         nicht alle Ports der Prozessinstanz graphisch angegeben,
    //         so muss dieser Port doch hinzugefuegt werden (auch mittels
    //         PortEintragen).

    void copyReward (ProcessTypeDecl *proc, char *pfad, int sectionnr);
    // Aus proc alle Rewards auslesen und in den neuen compilierten Prozess
    //  eintragen (neu erstellen). Dabei Ids und section wie bei copyPort
    //  beschrieben eintragen und in der Expression und in allen Attributen
    //  den Pfad pfad vor alle Variablen haengen.

    void copyMeasure (ProcessTypeDecl *proc, char *pfad, int sectionnr);
    // Aus proc alle Measures auslesen und in den neuen compilierten Prozess
    //  eintragen (neu erstellen). Dabei Ids und section wie bei copyPort
    //  beschrieben eintragen und in der Expression und in allen Attributen
    //  den Pfad pfad vor alle Variablen haengen.

    void copyParameter (ProcessTypeDecl *proc, char *pfad, int sectionnr);
    // Aus proc alle Parameter auslesen und in den neuen compilierten Prozess
    //  eintragen (neu erstellen). Dabei Ids und section wie bei copyPort
    //  beschrieben eintragen und in der Expression und in allen Attributen
    //  den Pfad pfad vor alle Variablen haengen.

    char *global_globalize (char *expr, char *regulaerer_pfad);
    // Unterfunktion zu copyProcessInstEntry, aehnlich globalize_names, mit
    //  dem Unterschied, evtl. nicht den regulaeren Pfad vor den Namen zu
    //  haengen, sondern einen anderen Pfad, wenn der Name aus einer in einem
    //  anderen Modul deklarierten ProzessInstanz stammt.

    void copyProcessInstEntry (ProcessInst *procinst, char *pfad, int sectnr,
			       bool global);
    // Unterfunktion zu copyProcessInst. Uebergebene ProzessInstanz procinst
    //  in den neuen Prozess eintragen. Dabei Ids und section wie bei copyPort
    //  beschrieben eintragen und in der Expression und in allen Attributen
    //  den Pfad pfad vor alle Variablen haengen (dazu zaehlen auch alle
    //  Uebergabeparameter). Falls die ProzessInstanz global deklariert ist
    //  (global==true), so muss der Pfad evtl. anders als pfad sein, naemlich
    //  der Pfad der anderen ProzessInstanz.

    void copyProcessInst (ProcessTypeDecl *proc, char *pfad, int sectionnr);
    // Aus proc alle ProzessInstanzen auslesen und in den neuen compilierten
    //  Prozess mittels copyProcessInstEntry eintragen (neu erstellen).

    void copyProcessUsage (ProcessTypeDecl *proc, char *pfad);
    // Aus proc alle ProzessUsages auslesen und in den neuen compilierten
    //  Prozess eintragen (neu erstellen). Dabei Ids wie bei copyPort
    //  beschrieben eintragen und in der Expression und in allen Attributen
    //  den Pfad pfad vor alle Variablen haengen (section nicht noetig, da
    //  diese im Prozess direkt angegeben ist).

    void copyPlace (ProcessTypeDecl *proc, char *pfad, int sectionnr);
    // Aus proc alle Plaetze auslesen und in den neuen compilierten Prozess
    //  eintragen (neu erstellen). Dabei Ids und section wie bei copyPort
    //  beschrieben eintragen und in der Expression und in allen Attributen
    //  den Pfad pfad vor alle Variablen haengen.

    void copyTransition (ProcessTypeDecl *proc, char *pfad, int sectionnr);
    // Aus proc alle Transitionen auslesen und in den neuen compilierten Pro-
    //  zess eintragen (neu erstellen). Dabei Ids und section wie bei copyPort
    //  beschrieben eintragen und in der Expression und in allen Attributen
    //  den Pfad pfad vor alle Variablen haengen.

    void copyArc (ProcessTypeDecl *proc, char *pfad, char *portpfad, int sec);
    // Aus proc alle Kanten auslesen und in den neuen compilierten Prozess
    //  eintragen (neu erstellen). Dabei Ids und section wie bei copyPort
    //  beschrieben eintragen und in der Expression und in allen Attributen
    //  den Pfad pfad vor alle Variablen haengen.

    void copyFormalParameters (ProcessTypeDecl *proc, ProcessInst *procinst,
			       char *pfad, int sectionnr);
    // Aus proc alle formalen Parameter auslesen und in den neuen compilierten
    //  Prozess je nach FormalParameterType als privaten RReward, Parameter
    //  oder MeasureParameter eintragen (neu erstellen). Dabei Ids und section
    //  wie bei copyPortbeschrieben eintragen und in der Expression und in
    //  allen Attributen den Pfad pfad vor alle Variablen haengen.

    void copyCompleteProcess (ProcessTypeDecl *proc, char *pfad);
    // Alle oben genannten Funktionen (copyArc, copyMeasure, usw.) mit der
    //  SectionNummer des Prozesses proc aufrufen, d.h.: alle Objekte des
    //  kompletten Prozesses werden in den internen newprocess eingetragen.

    void copyProcessCompleteInst (ProcessTypeDecl *proc,
				  ProcessInst *procinst, char *pfad);
    // Die Deklaration der ProzessInstanz procinst wird aufgeloest und alle
    //  Elemente des Prozesstypen proc (entspricht dem passenden Typen der
    //  ProzessInstanzwerden) werden je nach arraygroesse in den neuen Prozess
    //  newprocess mittels oben beschriebener Funktionen eingetragen.

    void includeProcessUsage (ProcessUsage *proc, char *pfad);
    // In dem Prozess proc deklarierte ProzessUsages wurden mit dem zu diesem
    //  Prozess passenden Pfad in newprocess einkopiert. Da der Prozess auch
    //  an anderer Stelle existieren kann, existieren evtl. mehrere Verweise
    //  auf diesen Prozess, der Prozess selbst jedoch nur einmal. Deshalb
    //  muessen alle lokalen Verweise auf den globalen Prozess (entspricht
    //  proc->longid) umgeaendert werden in den wirklichen Pfad des Prozesses
    //  (pfad). Diese falschen Verweise koennen stehen:
    //  1.) in den Quell- und Zielverweisen der Kanten,
    //  2.) in den Ausdruecken der Rewards und Measures und
    //  3.) in den Ausgruecken (guards, verteilungen, ...) der Transitionen

    char *calculateMultiplicity (char *iarcmult, char *oarcmult,
				 char *portmult);
    // Erstellt die neue Kanten-Multiplizitaet bei der Entfernung von Ports
    //  nach der Formel: (iarcmult)*(oarcmult)/(portmult) und liefert sie
    //  zurueck.

    void calculateArcs ();
    // Berechnet die neue Kantenmultiplizitaet mittels exprcalc.cc.
    //  Multiplizitaeten, die auf Plaetze zugreifen, koennen hier nicht be-
    //  rechnet werden, da die Anzahl der Marken zur Laufzeit sich aendern
    //  koennen.

    void removePorts ();
    // Entfernt alle in newprocess vorkommenden Ports und verknuepft alle
    //  auf oder von ihnen zeigende Kanten neu. Die neue Kanten-Multiplizitaet
    //  wird mittels calculateMultiplicity neu berechnet. Ports, auf die keine
    //  Kante zeigt oder von der keine Kante wegzeigt werden mitsamt ihrer
    //  Knaten geloescht.

  public:
    CompilerT ();
    ~CompilerT ();

    ProcessTypeDecl *compile (Vektor <Module> *mod);
    // Erzeugt einen neuen, compilierten Prozess mit dem Namen PROCID aus
    //  der Liste der uebergebenen, zu compilierenden Module (wenn Fehler
    //  auftreten, wird NULL zurueckgeliefert - mittels Aufruf-Option zu
    //  aendern.
    // Ablauf:
    // 1.) Module auf Korrektheit checken (intern & extern) durch Syntax-
    //      checker aus spnlcheck.cc.
    // 2.) Alle globalen Parameter aus allen Modulen in den Hautprozess
    //      kopieren (dabei keinen Pfad eintragen).
    // 3.) Aufrufe aller globalen Prozessinstanzen uebernehmen (Funktion:
    //      copyProcessInstEntry).
    //     Alle Prozesse aus allen Modulen in den Hautprozess kopieren
    //      (Funktion: copyCompleteProcess).
    //     Dabei gleichzeitig globale Identifiers (id) vergeben, Identifier
    //      mit Pfad-Angaben (longid) eintragen und die alte id als oldid
    //      vermerken.
    // 4.) Solange Prozessinstanzen existieren, diese miteinfuegen (bringen 
    //      evtl. weitere Prozessinstanzen/usages) (Funktion: copyProcess-
    //      CompleteInst). Hierbei (und bei 5. auch) die section-Nummer
    //      weitergeben an alle Objekte.
    // 5.) Solange Prozessusages existieren, diese miteinfuegen (bringen
    //      keine weiteren Prozessinstanzen), dabei identische Ports auf-
    //      einander abbilden und section-Nummer weiterreichen bzw. eintragen
    //      (Funktion: includeProcessUsage).
    // 6.) Namen in allen Ausdruecken auf globale Identifier umlegen
    //      (Methode: globalize aus spnlprocess.cc).
    // 7.) Alle Ports entfernen und die betroffenden Arcs durch neue ersetzen.
    //      (Funktion: removePorts).

    int getSectionCount ();
    // Liefert die Anzahl der verschiedenen Sections.

//    void displayConnections ();
    // Ausgabe aller in den Modulen vorkommenden Connections.
};


#endif
