#ifndef PARSERCC
#define PARSERCC

#include <string.h>	// strcpy, strlen
#include <iostream.h>   // cout
#include "scanner.cc"
#include "compiler.cc"
#include "stdfun.cc"
#include "spnlmodule.cc"
#include "parser.h"
#include "foutput.h"
#include "vektor.cc"



/********* Konstruktoren & Destruktoren *********/

ParserT::ParserT () {
  buffer= NULL;
  pfad= NULL;
  lookahead= 0;
  modpt= NULL;
  errorfree= true;
}


ParserT::~ParserT () {
  delete (pfad);
  // alles, was erzeugt wird, wird gleich wieder geloescht (bis auf die
  //   Module, die aber keine Pointer sind und somit sowieso geloescht werden.
}



/********* allgemein gueltige Funktionen *********/

void ParserT::display () {
  for (int i= 0; i < modules.size(); i++) {
    fnet << "(* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *)";
    (modules.readElementAt (i))->display();
  }
  fnet << "(* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *)";
}


void ParserT::displayItem (int t) {
  if (t<256)
    cout << '\'' << (char)t << '\'';
  else
    switch (t) {
      case DONE: cout << "file end";
		 break;
      case ID:   cout << "identifier";
		 break;
      case NUM:  cout << "number";
		 break;
      case REAL: cout << "real value";
		 break;
      case INT:  cout << "integer value";
		 break;
      default:   cout << "error";
    }
}


void ParserT::displayBuffer () {
  if (lookahead < 256)
    cout << "'" << (char)lookahead << "'";
  else
    cout << "'" << buffer << "'";
}


bool ParserT::match (int t) {
  if ((lookahead == t)
      || ((t == NUM) && ((lookahead == REAL) || (lookahead == INT)))){
    lookahead= scanner->lexan ();
    return (true);
  }
  else {
    scanner->error ("Parser error: ");
    displayItem (t);
    cout << " missing, ";
    displayBuffer ();
    cout << " found.";
    errorfree= false;
    return (false);
  }
}


bool ParserT::matchString (char *s) {
  if ((lookahead == ID) && (!strcmp (buffer,s))) {
    lookahead= scanner->lexan ();
    return (true);
  }
  else {
    scanner->error ("Parser error: '");
    cout << s << "' missing, ";
    displayBuffer ();
    cout << " found.";
    errorfree= false;
    return (false);
  }
}


bool ParserT::matchNumber (char *s) {
  if (((lookahead == REAL) || (lookahead == INT)) && (!strcmp (buffer,s))) {
    lookahead= scanner->lexan ();
    return (true);
  }
  else {
    scanner->error ("Parser error: '");
    cout << s << "' missing, ";
    displayBuffer ();
    cout << " found.";
    errorfree= false;
    return (false);
  }
}


bool ParserT::valid (int t) {
  return ((lookahead == t)
	  || ((t == NUM) && ((lookahead == REAL) || (lookahead == INT))));
}


bool ParserT::validString (char *s) {
  return (!strcmp (buffer,s));
}


void ParserT::scannBehind (char a) {
  while (!(valid (a) || valid (DONE)))
    lookahead= scanner->lexan ();
  if (valid(DONE)) {
    scanner->error ("Parser error: file end reached during recovering.\n");
    exit (1);
  }
  else
    lookahead= scanner->lexan ();
}


void ParserT::fillIdList (Vektor <String> *idlist) {
  String *s;
  if (valid (ID)) {
    if (isValidName (buffer)) {
      s= new String (buffer);
      idlist->addElement (s);
    }
    else {
      scanner->error ("Parser error: ");
      cout << buffer << " is a keyword, not a valid identifier.";
      errorfree= false;
    }
  }
  match (ID);
  while (valid ((int)',')) {
    match (',');
    if (valid (ID)) {
      if (isValidName (buffer)) {
	s= new String (buffer);
	idlist->addElement (s);
      }
      else {
	scanner->error ("Parser error: ");
	cout << buffer << " is a keyword, not a valid identifier.";
	errorfree= false;
      }
    }
    match (ID);
  }
}


void ParserT::emptyIdList (Vektor <String> *idlist) {
  while (idlist->size())  // Liste wieder leeren
    delete (idlist->removeElementAt(idlist->size()-1));
}


/********* speziellere Funktionen fuer Expressions *********/

char *ParserT::scannParameterExpr () {
  int i, j;
  char *zwbuffer= NULL, *expr= NULL, *helpexpr= NULL, *expr2= NULL;
  int klammeranz= 0, LastStringWasAnId= false;
  do {  // erstes Zeichen bereits eingelesen!
    helpexpr= expr;
    if (lookahead == ID) {

      zwbuffer= newstring (buffer);
      match (ID);
      while (valid ('[')) {  // es folgt mind. '[int].id'
	match ('[');
	if (valid (INT)) {
	  char *zwbuffertmp= zwbuffer;
	  zwbuffer= new char[strlen(zwbuffertmp)+strlen(buffer)+4];
	  strcpy (zwbuffer, zwbuffertmp);
	  strcat (zwbuffer, "[");
	  strcat (zwbuffer, buffer);
	  strcat (zwbuffer, "].");
	  delete (zwbuffertmp);
	}
	match (INT);
	match (']');
	match ('.');
	if (valid (ID)) {
	  char *zwbuffertmp= zwbuffer;
	  zwbuffer= new char[strlen(zwbuffertmp)+strlen(buffer)+1];
	  strcpy (zwbuffer, zwbuffertmp);
	  strcat (zwbuffer, buffer);
	  delete (zwbuffertmp);
	}
	match (ID);
      }

      if (helpexpr) {     // helpexpr != NULL => helpexpr vorher anfuegen
	expr= new char[strlen(helpexpr)+strlen(zwbuffer)+3];
	strcpy (expr, helpexpr);
	strcat (expr, "$");
      }
      else {
	expr= new char[strlen(zwbuffer)+3];
	strcpy (expr, "$");
      }
      strcat (expr, zwbuffer);
      strcat (expr, "$");

      LastStringWasAnId= true;
      delete (zwbuffer);
    }
    else if (lookahead >= 256) {  // Number (or DONE?)
      expr= concat (helpexpr, buffer);
//cout << "\n***scanner sagt: "<<helpexpr<<" + "<<buffer<<" = "<<expr;
      LastStringWasAnId= true;   // damit sowas geht: proc.id+1973 guess= 1978
      lookahead= scanner->lexan ();
    }
    else {  // ein Character zurueckgeliefert
      if (valid ('('))
	klammeranz++;
      else if (valid (')'))
	klammeranz--;
      LastStringWasAnId= false;
      expr2= new char[2];
      expr2[0]= (char) lookahead;
      expr2[1]= '\0';
      expr= concat (helpexpr, expr2);
      delete (expr2);
      lookahead= scanner->lexan ();
    }
    delete (helpexpr);
  }  // zweites ID abfangen, ebenso eine Klammer zuviel, ...
  while ((valid ('.') || valid ('^') || valid ('[') || valid (']')
	 || valid ('+') || valid ('-') || valid ('*') || valid ('/')
	 || valid ('(') || (valid (')') && klammeranz) || valid (NUM)
	 || (valid (ID) && !LastStringWasAnId)) && !valid (DONE));
  // '[', ']' werden benoetigt fuer actual parameters (z.B. rreward)
  if (valid(DONE)) {
    scanner->error ("Parser error: File end reached during parameterexpr.\n");
    exit (1);
  }
  return (expr);
}


char *ParserT::scannRewMeasDistExpr (int resc, char **resv, bool dist) {
  int i, j;
  char *expr= NULL, *helpexpr= NULL, *expr2= NULL; char *zwbuffer= NULL;
  int klammeranz= 0,  // Wenn eine Klammer mehr geschlossen wurde, als
                      //  geoeffnet, ist die RewMeasExpression zuende.
      LastStringWasAnId= false,  // Zwischen Identifiern muessen Spaces in der
                                 //  Expression eingefuegt werden.
      notfound;  // Existiert der Identifier in der uebergebenen resv-Liste?
  do {  // erstes Zeichen bereits eingelesen!
    helpexpr= expr;  // hexpr ist alte expression (oder NULL am Anfang)
    if (lookahead == ID) {  // Identifier gefunden

      notfound= true;  // schauen, ob es sich um ein Schluesselwort handelt:
      for (j=0; j<resc; j++)
	if (!strcmp (resv[j], buffer))
	  notfound= false;  // buffer wurde als Schluesselwort gefunden
      
      zwbuffer= newstring (buffer);
      match (ID);
      if (notfound)  // schauen, ob es sich um einen Array-Wert handelt
	while (valid ('[')) {  // es folgt mind. '[int].id'
	  match ('[');
	  if (valid (INT)) {
	    char *zwbuffertmp= zwbuffer;
	    zwbuffer= new char[strlen(zwbuffertmp)+strlen(buffer)+4];
	    strcpy (zwbuffer, zwbuffertmp);
	    strcat (zwbuffer, "[");
	    strcat (zwbuffer, buffer);
	    strcat (zwbuffer, "].");
	    delete (zwbuffertmp);
	  }
	  match (INT);
	  match (']');
	  match ('.');
	  if (valid (ID)) {
	    char *zwbuffertmp= zwbuffer;
	    zwbuffer= new char[strlen(zwbuffertmp)+strlen(buffer)+1];
	    strcpy (zwbuffer, zwbuffertmp);
	    strcat (zwbuffer, buffer);
	    delete (zwbuffertmp);
	  }
	  match (ID);
	}

      if (!LastStringWasAnId) // kein Leerzeichen vorhe Name einfuegen
	if (notfound) {       // es handelt sich um einen Variablennamen
	  if (helpexpr) {     // helpexpr != NULL => helpexpr vorher anfuegen
	    expr= new char[strlen(helpexpr)+strlen(zwbuffer)+3];
	    strcpy (expr, helpexpr);
	    strcat (expr, "$");
	  }
	  else {
	    expr= new char[strlen(zwbuffer)+3];
	    strcpy (expr, "$");
	  }
	  strcat (expr, zwbuffer);
	  strcat (expr, "$");
	}
	else  // es handelt sich um ein Schluesselwort
	  expr= concat (helpexpr, zwbuffer);
 
      else {  // Leerzeichen vor Name einfuegen
	if (notfound) {  // es handelt sich um einen Variablennamen
	  expr= new char[strlen(helpexpr)+strlen(zwbuffer)+4];
	  strcpy (expr, helpexpr);
	  strcat (expr, " $");
	  strcat (expr, zwbuffer);
	  strcat (expr, "$");
	}
	else {
	  expr= new char[strlen(helpexpr)+strlen(zwbuffer)+2];
	  strcpy (expr, helpexpr);
	  strcat (expr, " ");
	  strcat (expr, zwbuffer);
	}
      }
      LastStringWasAnId= true;
      delete (zwbuffer);
    }

    else if (lookahead >= 256) {  // Number (or DONE?)
      if (!LastStringWasAnId)  // kein Leerzeichen vorhe Name einfuegen
	expr= concat (helpexpr, buffer);
      else {                   // Leerzeichen vor Name einfuegen
	expr= new char[strlen(helpexpr)+strlen(buffer)+2];
	strcpy (expr, helpexpr);
	strcat (expr, " ");
	strcat (expr, buffer);
      }
      LastStringWasAnId= true;
      lookahead= scanner->lexan ();
    }

    else {  // ein Character zurueckgeliefert
      if (valid ('('))  // Klammern zaehlen
	klammeranz++;
      else if (valid (')'))
	klammeranz--;
      LastStringWasAnId= false;
      expr2= new char[2];
      expr2[0]= (char) lookahead;
      expr2[1]= '\0';
      expr= concat (helpexpr, expr2);
      delete (expr2);
      lookahead= scanner->lexan ();
    }
    delete (helpexpr);
  }  // Endeklammer, Semikolon und Co. abfangen.
  while ((valid ('.') || valid ('^') || valid ('{') || valid ('}')
	 || valid ('+') || valid ('-') || valid ('*') || valid ('/')
	 || valid ('#') || valid ('>') || valid ('<') || valid ('=')
	 || (valid (',') && dist)  // Distributionexpression darf , ethalten.
	 || (valid ('|') && !dist) // Rew/Meas darf | enthalten.
	 || valid ('(') || (valid (')') && klammeranz)
	 || valid (NUM) || valid (ID)) && !valid (DONE));
  if (valid(DONE)) {
    if (dist)
      scanner->error
	("Parser error: File end reached during distribution expr.\n");
    else
      scanner->error
	("Parser error: File end reached during reward or measure expr.\n");
    exit (1);
  }
  return (expr);
}


char *ParserT::scannPortType () {
// Alle moeglichen Porttypen parsen.
  char *expr= new char[5];
  expr[0]= buffer[0];
  if (buffer[1] == 'o') {
    expr[1]= buffer[1];
    matchString ("to");
  }
  else {
    match (ID);
    expr[1]= (char) lookahead;
    lookahead= scanner->lexan ();
  }
  if (buffer[0] == 'o') {
    expr[2]= buffer[0];
    expr[3]= buffer[1];
    matchString ("ot");
  }
  else {
    expr[2]= (char) lookahead;
    lookahead= scanner->lexan ();
    expr[3]= buffer[0];
    lookahead= scanner->lexan ();
  }
  expr[4]= '\0';
  return (expr);
}



/********* Hauptfunktion parse *********/

int ParserT::parse (char *name) {
  int i=0, j=0;
  char *importmod= NULL, *fname= NULL, *zwerg= NULL;
  errorfree= true;

  pfad= getPath (name);
  fname= getFile (name);
  scanner= new ScannerT (name);
  buffer= scanner->lexbuf;
  lookahead= scanner->lexan ();
  importmod= substring2point (fname);
  Moduleintrag (importmod);  // erstes Modul eintragen
  delete (importmod);
  delete (scanner);
  delete (fname);
  lookahead= 0;
  if (dots)
    cout << '.';

  for (i= 0; i < modules.size(); i++) {
    // Fuer jedes Modul die Importliste durchgehen und importierte Module
    //  einlesen, wenn noch nicht in der Liste vorhanden.
    j= 0;
    importmod= (modules.readElementAt (i))->getImport (j);
    while (importmod) {  // Modul gefunden
      if (!modules.searchId (importmod)) {  // noch nicht eingelesen
	zwerg= concat (importmod, SPNL_ENDING);
	fname= concat (pfad, zwerg);
	scanner= new ScannerT (fname);
	buffer= scanner->lexbuf;
	lookahead= scanner->lexan ();
	Moduleintrag (importmod);  // weitere Module eintragen
	delete (zwerg);
	delete (scanner);
	delete (fname);
	lookahead= 0;
	if (dots)
	  cout << '.' << flush;
      }
      importmod= (modules.readElementAt (i))->getImport (++j);
    }
  }
  for (i= 0; i < modules.size(); i++)
    errorfree&= (modules.readElementAt (i))->isErrorfree ();
  return (errorfree);
}



/********* Funktionen zum Parsen (eng an SPNL-Syntax) *********/

void ParserT::Moduleintrag (char *fileid) {
  matchString ("module");
  if (valid (ID)) {
    if (!validString (fileid)) {
      errorfree= false;
      scanner->error ("module id '");
      cout << buffer << "' does not match filename '" << fileid << "'.";
    }
  }
  modpt= new Module (fileid);
  modules.addElement (modpt);
  match (ID);  // Modulename
  if (!match (';'))
    scannBehind (';');
  if (validString("use")) {  // Importe eintragen
    matchString ("use");
    Imports ();      // mind. ein Import
    while (valid ((int)',')) {
      match (',');
      Imports ();
    }
    if (!match (';'))
      scannBehind (';');
  }

  Declaration ();  // mind. eine Deklaration
  matchString ("end");
  matchString (fileid);
  match ('.');
  match (DONE);
}


void ParserT::Imports () {
  char *filename, *zwerg;
  FILE *file;
  if (valid (ID)) {
    zwerg= concat (buffer, SPNL_ENDING);
    filename= concat (pfad, zwerg);
    delete (zwerg);
    file= fopen (filename,"r");
    if (file == NULL) {
      scanner->error ("use ");
      cout << buffer << " included; file '" << filename
	   << "' does not exist.";
      errorfree= false;
    }
    else
      modpt->addImport (buffer);
    fclose (file);
    delete (filename);
  }
  match (ID);  // Importname
}


void ParserT::Declaration () {
  while (!validString ("end") && !valid (DONE)) {
    if (validString ("parameter"))
      GlobalParameterDecl ();
    else if (validString ("process"))
      ProcessDecl ();
    else if (validString ("processtype"))
      ProcesstypeDecl ();
    else if (validString ("processinstance"))
      GlobalProcessInstantiation ();
    else {
      scanner->error ("Parser error: parameter, process, processtype or ");
      cout << "processinstance missing, ";
      displayBuffer ();
      cout << " found.";
      errorfree= false;
      lookahead= scanner->lexan ();
    }
  }
}


void ParserT::GlobalParameterDecl () {
  int i;
  char *expr= NULL;
  matchString ("parameter");
  while (valid (ID) && !(validString ("processinstance")
	  || validString ("process") || validString ("processtype")
	  || validString ("parameter")
	  || validString ("end")) && !valid (DONE)) {
    fillIdList (&idlist);
    if (!match ('='))
      scannBehind ('=');
    expr= scannParameterExpr ();
    for (i=0; i<idlist.size(); i++)
      modpt->addParameter ((idlist.readElementAt (i))->getid(), expr);
    delete (expr);
    expr= NULL;
    if (!match (';'))
      scannBehind (';');
    emptyIdList (&idlist);
  }
}


void ParserT::ProcessDecl () {
  int i;
  matchString ("process");
  fillIdList (&idlist);
  match (';');
  for (i=0; i<idlist.size(); i++)
    modpt->addProcess ((idlist.readElementAt (i))->getid(), DECL);
  ProcessBody ();
  emptyIdList (&idlist);
}


void ParserT::ProcesstypeDeclParamentry () {
  int i,j;
  char *typ;
  if (!(validString ("param") || validString ("measparam")
	|| validString ("rreward"))) {
    scanner->error
    ("Formalparametertype expected (param, rreward or measparam)");
    displayBuffer ();
    cout << " found. Trying to use 'param'.";
    errorfree= false;
    typ= newstring ("param");
  }
  else
    typ= newstring (buffer);
  match (ID);
  // match (':');
  fillIdList (&idlist2);
  for (i=0; i<idlist.size(); i++)     // Formalparametertype einlesen und fuer
    for (j=0; j<idlist2.size(); j++)  //  alle idlist2 & idlist speichern.
      modpt->addProcessFormPar ((idlist.readElementAt (i))->getid(),
				(idlist2.readElementAt (j))->getid(), typ);
  delete (typ);
  emptyIdList (&idlist2);
}


void ParserT::ProcesstypeDecl () {
  int i;
  matchString ("processtype");
  fillIdList (&idlist);
  for (i=0; i<idlist.size(); i++)
    modpt->addProcess ((idlist.readElementAt (i))->getid(), TYPE);
  if (!valid (';')) {        // Formalparameters
    match ('(');
    ProcesstypeDeclParamentry ();
    while (valid ((int)';')) {
      match (';');
      ProcesstypeDeclParamentry ();
    }
    if (!match (')'))
      scannBehind (')');
  }
  if (!match (';'))
    scannBehind (';');
  ProcessBody ();
  emptyIdList (&idlist);
}


void ParserT::GlobalProcessInstantiationParamentry () {
  int i;
  char *expr= scannParameterExpr ();
  char *guessexpr= NULL;
  if (valid (ID)) {
    matchString ("guess");
    match ('=');
    guessexpr= scannParameterExpr ();
  }
  for (i=0; i<idlist.size(); i++)
    modpt->addProcessInstFormPar ((idlist.readElementAt (i))->getid(),
				  expr, guessexpr);
  delete (expr);
  delete (guessexpr);
}


void ParserT::GlobalProcessInstantiation () {
  int i;
  char *arrayanz= NULL;
  matchString ("processinstance");
  while (valid (ID) && !(validString ("processinstance")
			 || validString ("process")
			 || validString ("processtype")
			 || validString ("parameter")
			 || validString ("end")) && !valid (DONE)) {
    fillIdList (&idlist);
    match (':');
    if (validString ("array")) {
      matchString ("array");
      match ('[');
      matchNumber ("1");
      match ('.');
      if (valid (REAL)) {   // [1..5] wird zelegt in [, 1, ., .5 (real)
	arrayanz= newstring ((char *)(buffer+1));
	match (REAL);
      }
      else {
	match ('.');
	if (valid (ID))
	  arrayanz= newstring (buffer);
	match (ID);
      }
      match (']');
      matchString ("of");
    }
    if (valid (ID))
      for (i=0; i<idlist.size(); i++)
	modpt->addProcessInst ((idlist.readElementAt (i))->getid(),
			       arrayanz, buffer);
    match (ID);
    delete (arrayanz);
    arrayanz= NULL;
    if (!valid (';')) {        // Parameters
      match ('(');
      GlobalProcessInstantiationParamentry ();
      while (valid ((int)',')) {
	match (',');
	GlobalProcessInstantiationParamentry ();
      }
      if (!match (')'))
	scannBehind (')');
    }
    if (!match (';'))
      scannBehind (';');
    emptyIdList (&idlist);
  }
}


void ParserT::ProcessBody () {
// muss alles fuer alle processe in der Liste idlist eintragen!
  int i;
  while (!validString ("private") && !valid (DONE)) {
    if (validString ("port"))
      PortDecl ();
    else if (validString ("rreward") || validString ("ireward") 
	     || validString ("creward"))
      RewardDecl (PUBLIC);
    else if (validString ("smeasure") || validString ("mmeasure"))
      MeasureDecl (PUBLIC);
    else {
      errorfree= false;
      scanner->error ("Parser error: port, {r|i|c}reward, {s,m}measure or ");
      cout << "private part missing, ";
      displayBuffer ();
      cout << " found.";
      lookahead= scanner->lexan ();
    }
  }
  matchString ("private");
  while (!validString ("graphbegin") && !valid (DONE)) {
   if (validString ("parameter"))
      ParameterDecl ();
    else if (validString ("rreward") || validString ("ireward") 
	     || validString ("creward"))
      RewardDecl (PRIVATE);
    else if (validString ("smeasure") || validString ("mmeasure"))
      MeasureDecl (PRIVATE);
    else if (validString ("processinstance"))
      ProcessInstantiation ();
    else if (validString ("useprocess"))
      ProcessUsage ();
    else {
      errorfree= false;
      scanner->error ("Parser error: parameter, {r|i|c}reward, {s,m}measure");
      cout << ", processinstance, useprocess or graphical part missing, ";
      displayBuffer ();
      cout << " found.";
      lookahead= scanner->lexan ();
    }
  }
  matchString ("graphbegin");
  while (!validString ("graphend") && !valid (DONE)) {
   if (validString ("place"))
      PlaceDecl ();
    else if (validString ("trans"))
      TransitionDecl ();
    else if (validString ("arc"))
      ArcDecl ();
    else if (validString ("graphport"))
      GraphPortDecl ();
    else
      lookahead= scanner->lexan ();
  }
  matchString ("graphend");
  match (';');
  while (!validString ("end") && !valid (DONE)) {
    if (validString ("transatt"))
      TransitionAttributes ();
    else if (validString ("rreward") || validString ("ireward") 
             || validString ("creward"))
      RewardDef ();
    else if (validString ("smeasure") || validString ("mmeasure"))
      MeasureDef ();
    else {
      errorfree= false;
      scanner->error ("Parser error: {r|i|c}reward, {s,m}measure, transatt");
      cout << " or end missing, ";
      displayBuffer ();
      cout << " found.";
      lookahead= scanner->lexan ();
    }
  }
  matchString ("end");
  for (i=0; i<idlist.size()-1; i++) {
    matchString ((idlist.readElementAt (i))->getid());
    match (',');
  }
  matchString ((idlist.readElementAt (idlist.size()-1))->getid());
  if (!match (';'))
    scannBehind (';');
}


void ParserT::PortDecl () {
  int i,j;
  char *porttype= NULL, *mult= NULL;
  matchString ("port");
  while (valid (ID) && !(validString ("port") || validString ("private")
	  || validString ("smeasure") || validString ("mmeasure")
       	  || validString ("rreward") || validString ("ireward")
	  || validString ("creward")) && !valid (DONE)) {
    fillIdList (&idlist2);
    if (match (':')) {
      porttype= scannPortType ();
      if (valid (',')) {
	match (',');
	matchString ("mult");
	match ('=');
	mult= scannParameterExpr ();   // *** eigentlich keine Expression
	// ************ vielleicht aber doch sowieso moeglich...
/*
      if (valid (ID) || valid (INT))
	mult= newstring (buffer);
      if (valid (INT))
	match (INT);
      else
	match (ID);
*/
      }
      for (i=0; i<idlist.size(); i++)
	for (j=0; j<idlist2.size(); j++)
	  modpt->addProcessPort ((idlist.readElementAt (i))->getid(),
		    (idlist2.readElementAt (j))->getid(), porttype, mult);
      delete (porttype);
      porttype= NULL;
      delete (mult);
      mult= NULL;
    }
    if (!match (';'))
      scannBehind (';');
    emptyIdList (&idlist2);
  }
}


void ParserT::RewardDecl (VisibleType visible) {
  int i,j;
  char typ;
  if (validString ("rreward")) {
    typ= 'r';
    matchString ("rreward");
  }
  else if (validString ("ireward")) {
    typ= 'i';
    matchString ("ireward");
  }
  else {
    typ= 'c';
    matchString ("creward");
  }
  fillIdList (&idlist2);
  for (i=0; i<idlist.size(); i++)
    for (j=0; j<idlist2.size(); j++)
      modpt->addProcessReward ((idlist.readElementAt (i))->getid(),
		(idlist2.readElementAt (j))->getid(), typ, visible);
  if (!match (';'))
    scannBehind (';');
  emptyIdList (&idlist2);
}


void ParserT::MeasureDecl (VisibleType visible) {
  int i,j;
  char typ;
  if (validString ("smeasure")) {
    typ= 's';
    matchString ("smeasure");
  }
  else {
    typ= 'm';
    matchString ("mmeasure");
  }
  fillIdList (&idlist2);
  for (i=0; i<idlist.size(); i++)
    for (j=0; j<idlist2.size(); j++)
      modpt->addProcessMeasure ((idlist.readElementAt (i))->getid(),
		(idlist2.readElementAt (j))->getid(), typ, visible);
  if (!match (';'))
    scannBehind (';');
  emptyIdList (&idlist2);
}


void ParserT::ParameterDecl () {
  int i,j;
  char *expr= NULL;
  matchString ("parameter");
  while (valid (ID) && !(validString ("graphbegin")
	  || validString ("parameter") || validString ("processinstance")
          || validString ("smeasure") || validString ("mmeasure")
          || validString ("rreward") || validString ("ireward")
          || validString ("creward") || validString ("useprocess"))
	 && !valid (DONE)) {
    fillIdList (&idlist2);
    if (!match ('='))
      scannBehind ('=');
    expr= scannParameterExpr ();
    for (i=0; i<idlist.size(); i++)
      for (j=0; j<idlist2.size(); j++)
	modpt->addProcessParameter ((idlist.readElementAt (i))->getid(),
		  (idlist2.readElementAt (j))->getid(), expr);
    delete (expr);
    expr= NULL;
    if (!match (';'))
      scannBehind (';');
    emptyIdList (&idlist2);
  }
}


void ParserT::ProcessInstantiationParamentry () {
  int i, j;
  char *expr= scannParameterExpr ();
  char *guessexpr= NULL;
  if (valid (ID)) {
    matchString ("guess");
    match ('=');
    guessexpr= scannParameterExpr ();
  }
  for (i=0; i<idlist.size(); i++)
    for (j=0; j<idlist2.size(); j++)
      modpt->addProcessProcInstFormPar ((idlist.readElementAt (i))->getid(),
		(idlist2.readElementAt (j))->getid(), expr, guessexpr);
  delete (expr);
  delete (guessexpr);
}


void ParserT::ProcessInstantiation () {
  int i, j;
  char *arrayanz= NULL;
  matchString ("processinstance");
  while (valid (ID) && !(validString ("graphbegin")
	  || validString ("parameter") || validString ("processinstance")
          || validString ("smeasure") || validString ("mmeasure")
          || validString ("rreward") || validString ("ireward")
          || validString ("creward")) || validString ("useprocess")
	 && !valid (DONE)) {
    fillIdList (&idlist2);
    match (':');
    if (validString ("array")) {
      matchString ("array");
      match ('[');
      matchNumber ("1");
      match ('.');
      if (valid (REAL)) {   // [1..5] wird zelegt in [, 1, ., .5 (real)
	arrayanz= newstring ((char *)(buffer+1));
	match (REAL);
      }
      else {
	match ('.');
	if (valid (ID))
	  arrayanz= newstring (buffer);
	match (ID);
      }
      match (']');
      matchString ("of");
    }
    if (valid (ID))
      for (i=0; i<idlist.size(); i++)
	for (j=0; j<idlist2.size(); j++)
	  modpt->addProcessProcInst ((idlist.readElementAt (i))->getid(),
		    (idlist2.readElementAt (j))->getid(), arrayanz, buffer);
    match (ID);
    delete (arrayanz);
    arrayanz= NULL;
    if (!valid (';')) {        // Parameters
      match ('(');
      ProcessInstantiationParamentry ();
      while (valid ((int)',')) {
	match (',');
	ProcessInstantiationParamentry ();
      }
      if (!match (')'))
	scannBehind (')');
    }
    if (!match (';'))
      scannBehind (';');
    emptyIdList (&idlist2);
  }
}


void ParserT::ProcessUsage () {
  int i,j;
  matchString ("useprocess");
  fillIdList (&idlist2);
  if (!match (':'))
    scannBehind (':');
  if (valid (ID))
    for (i=0; i<idlist.size(); i++)
      for (j=0; j<idlist2.size(); j++)
	modpt->addProcessProcUsage ((idlist.readElementAt (i))->getid(),
		  (idlist2.readElementAt (j))->getid(), buffer);
  match (ID);
  if (!match (';'))
    scannBehind (';');
  emptyIdList (&idlist2);
}


void ParserT::PlaceDecl () {
  int i,j;
  char *expr= NULL;
  matchString ("place");
  while (valid (ID) && !(validString ("place") || validString ("trans")
	  || validString ("arc") || validString ("graphplace")
	  || validString ("graphtrans") || validString ("grapharc")
	  || validString ("graphport") || validString ("graphprocess")
	  || validString ("grapharray") || validString ("graphend"))
	  && !valid (DONE)) {
    fillIdList (&idlist2);
    if (valid (':')) {
      match (':');
      expr= scannParameterExpr ();
    }
    for (i=0; i<idlist.size(); i++)
      for (j=0; j<idlist2.size(); j++)
	modpt->addProcessPlace ((idlist.readElementAt (i))->getid(),
				(idlist2.readElementAt (j))->getid(), expr);
    delete (expr);
    expr= NULL;
    if (!match (';'))
      scannBehind (';');
    emptyIdList (&idlist2);
  }
}


void ParserT::TransitionDecl () {
  int i,j;
  char *expr= NULL;
  matchString ("trans");
  while (valid (ID) && !(validString ("place") || validString ("trans")
	  || validString ("arc") || validString ("graphplace")
	  || validString ("graphtrans") || validString ("grapharc")
	  || validString ("graphport") || validString ("graphprocess")
	  || validString ("grapharray") || validString ("graphend"))
	  && !valid (DONE)) {
    fillIdList (&idlist2);
    if (valid (':')) {
      match (':');
      if (validString ("imm") || validString ("timed"))
	expr= newstring (buffer);
      match (ID);
    }
    for (i=0; i<idlist.size(); i++)
      for (j=0; j<idlist2.size(); j++)
	modpt->addProcessTrans ((idlist.readElementAt (i))->getid(),
				(idlist2.readElementAt (j))->getid(), expr);
    delete (expr);
    expr= NULL;
    if (!match (';'))
      scannBehind (';');
    emptyIdList (&idlist2);
  }
}


void ParserT::ArcDecl () {
  int i,j;
  char *from= NULL, *to= NULL, *type= NULL, *mult= NULL;
  matchString ("arc");
  while (valid (ID) && !(validString ("place") || validString ("trans")
	  || validString ("arc") || validString ("graphplace")
	  || validString ("graphtrans") || validString ("grapharc")
	  || validString ("graphport") || validString ("graphprocess")
	  || validString ("grapharray") || validString ("graphend"))
	  && !valid (DONE)) {
    fillIdList (&idlist2);
    match (':');
    if (!match ('('))
      scannBehind ('(');
    from= newstring (buffer);
    match (ID);
    match (',');
    to= newstring (buffer);
    match (ID);
    if (!match (')'))
      scannBehind (')');
    while (!valid (';')) {
      if (!match (','))
        scannBehind (',');
      if (validString ("type")) {
	match (ID);
	if (!type)
  	  type= newstring (buffer);
	else
	  scanner->error ("arctype defined twice.");
        match (ID);
      }
      else if (validString ("mult")) {
	match (ID);
	if (!mult)
	  if (strcmp (buffer, "portdep"))
	    mult= scannRewMeasDistExpr (REW_NUMRESWORDS, rew_reserved, false);
	  else {
	    mult= newstring (buffer);
	    match (ID);
	  }
	else {
	  scanner->error ("arcmultiplicity defined twice.");
	  delete (scannRewMeasDistExpr (REW_NUMRESWORDS, rew_reserved,false));
	}
      }
    }
    for (i=0; i<idlist.size(); i++)
      for (j=0; j<idlist2.size(); j++)
	modpt->addProcessArc ((idlist.readElementAt (i))->getid(),
		  (idlist2.readElementAt (j))->getid(), from, to, type, mult);
    delete (from);
    delete (to);
    delete (type);
    delete (mult);
    from= NULL;
    to= NULL;
    type= NULL;
    mult= NULL;
    if (!match (';'))
      scannBehind (';');
    emptyIdList (&idlist2);
  }
}


void ParserT::GraphPortDecl () {
  int i;
  matchString ("graphport");
  while (valid (ID) && !(validString ("place") || validString ("trans")
	  || validString ("arc") || validString ("graphplace")
	  || validString ("graphtrans") || validString ("grapharc")
	  || validString ("graphport") || validString ("graphprocess")
	  || validString ("grapharray") || validString ("graphend"))
	  && !valid (DONE)) {
    if (valid (ID))
      for (i=0; i<idlist.size(); i++)
	modpt->addProcessPort ((idlist.readElementAt (i))->getid(), buffer,
			       "usedport", NULL);
    match (ID);
    scannBehind (';');
  }
}


void ParserT::RewardDef () {
  int i,j;
  char *expr;
  if (validString ("rreward"))
    matchString ("rreward");
  else if (validString ("ireward"))
    matchString ("ireward");
  else
    matchString ("creward");
  while (valid (ID) && !(validString ("smeasure") || validString ("mmeasure")
          || validString ("rreward") || validString ("ireward")
          || validString ("creward") || validString ("transatt")
	  || validString ("end")) && !valid (DONE)) {
    fillIdList (&idlist2);
    if (!match ('='))
      scannBehind ('=');
    expr= scannRewMeasDistExpr (REW_NUMRESWORDS, rew_reserved, false);
    for (i=0; i<idlist.size(); i++)
      for (j=0; j<idlist2.size(); j++)
	modpt->addProcessRewardExpr ((idlist.readElementAt (i))->getid(),
		  (idlist2.readElementAt (j))->getid(), expr);
    delete (expr);
    if (!match (';'))
      scannBehind (';');
    emptyIdList (&idlist2);
  }
}


void ParserT::MeasureDef () {
  int i,j;
  char *expr;
  if (validString ("mmeasure"))
    matchString ("mmeasure");
  else
    matchString ("smeasure");
  while (valid (ID) && !(validString ("smeasure") || validString ("mmeasure")
          || validString ("rreward") || validString ("ireward")
          || validString ("creward") || validString ("transatt")
	  || validString ("end")) && !valid (DONE)) {
    fillIdList (&idlist2);
    if (!match ('='))
      scannBehind ('=');
    expr= scannRewMeasDistExpr (MEAS_NUMRESWORDS, meas_reserved, false);
    for (i=0; i<idlist.size(); i++)
      for (j=0; j<idlist2.size(); j++)
	modpt->addProcessMeasureExpr ((idlist.readElementAt (i))->getid(),
		  (idlist2.readElementAt (j))->getid(), expr);
    delete (expr);
    if (!match (';'))
      scannBehind (';');
    emptyIdList (&idlist2);
  }
}
  

void ParserT::TransitionAttributesParamentry () {
  int i,j;
  char *prio= NULL;
  char *dist= NULL;
  char *distexpr= NULL;
  char *weight= NULL;
  char *markdep= NULL;
  char *policy= NULL;
  char *conc= NULL;
  char *guard= NULL;
  if (validString ("prio")) {
    matchString ("prio");
    match ('=');
    prio= scannRewMeasDistExpr (REW_NUMRESWORDS, rew_reserved, false);
  }
  else if (validString ("dist")) {
    bool expoly;
    matchString ("dist");
    match ('=');
    if (validString ("exp") || validString ("uniform")
	|| validString ("truncnormal") || validString ("erlang") 
	|| validString ("weibull") || validString ("phasetype")
	|| validString ("det") || validString ("bernoulli")
	|| validString ("discreteuniform") || validString ("geometric")
	|| validString ("modgeometric") || validString ("binomial")
	|| validString ("poisson")) {
      dist= newstring (buffer);
      match (ID);
      match ('(');
      expoly= false;
    }
    else {
      expoly= true;
      dist= newstring ("expoly");
    }
    distexpr= scannRewMeasDistExpr (DIST_NUMRESWORDS, dist_reserved, true);
    if (!expoly)
      match (')');
  }
  else if (validString ("weight")) {
    matchString ("weight");
    match ('=');
    weight= scannRewMeasDistExpr (REW_NUMRESWORDS, rew_reserved, false);
  }
  else if (validString ("markdep")) {
    matchString ("markdep");
    match ('=');
    if (valid (ID))
      markdep= newstring (buffer);
    match (ID);
  }
  else if (validString ("policy")) {
    matchString ("policy");
    match ('=');
    if (valid (ID))
      policy= newstring (buffer);
    match (ID);
  }
  else if (validString ("concurrency")) {
    matchString ("concurrency");
    match ('=');
    if (valid (ID))
      conc= newstring (buffer);
    match (ID);
  }
  else if (validString ("guard")) {
    matchString ("guard");
    match ('=');
    guard= scannRewMeasDistExpr (REW_NUMRESWORDS, rew_reserved, false);
  }
  else {
    scanner->error ("Parser error: prio, dist, weight, markdep, policy, ");
    cout << "concurrency or guard missing, ";
    displayBuffer ();
    cout << " found.";
    lookahead= scanner->lexan ();
  }
  for (i=0; i<idlist.size(); i++)
    for (j=0; j<idlist2.size(); j++)
      modpt->addProcessTransAttr ((idlist.readElementAt (i))->getid(),
		(idlist2.readElementAt (j))->getid(), prio, dist, distexpr,
		weight, markdep, policy, conc, guard);
  delete (prio);
  delete (dist);
  delete (distexpr);
  delete (weight);
  delete (markdep);
  delete (policy);
  delete (conc);
  delete (guard);
  prio= NULL;
  dist= NULL;
  distexpr= NULL;
  weight= NULL;
  markdep= NULL;
  policy= NULL;
  conc= NULL;
  guard= NULL;
}


void ParserT::TransitionAttributes () {
  matchString ("transatt");
  while (valid (ID) && !(validString ("smeasure") || validString ("mmeasure")
          || validString ("rreward") || validString ("ireward")
          || validString ("creward") || validString ("transatt")
	  || validString ("end")) && !valid (DONE)) {
    fillIdList (&idlist2);
    if (!match (':'))
      scannBehind (':');
    TransitionAttributesParamentry ();
    while (valid ((int)',')) {
      match (',');
      TransitionAttributesParamentry ();
    }
    if (!match (';'))
      scannBehind (';');
    emptyIdList (&idlist2);
  }
}




#endif
