#ifndef PARSERH
#define PARSERH


class ParserT {
  private:

    char *pfad;                // Pfad, in dem die einzulesenden Module ab-
                               //  gelegt sind (z.B.: "../spnl/aloha/").

    ScannerT *scanner;         // Scanner, der File einliesst
    char *buffer;              // Referenz zu dem Buffer aus dem Scanner
    int lookahead;             // aktueller Rueckgabewert des Scanners
    int errorfree;             // wird global auf false gesetzt, sobald ein
                               //  Fehler auftritt.

    Module *modpt;             // intern: aktuelles Modul, zu dem Daten
                               //  angefuegt werden
    Vektor <String> idlist;    // Speichert intern eine Reihung von Prozessen
    Vektor <String> idlist2;   // Speichert intern eine Reihung von Parametern
                               //  & Co. zu den Prozessen aus idlist gehoerig

    void displayItem (int tokenval);
    // Gibt das uebergebene Zeichen als Character, Identifier, File End o.ae.
    //  aus.

    void displayBuffer ();
    // Gibt den aktuellen Buffer als Character oder String in Hochkommata aus.

    bool match (int character);
    bool matchString (char *string);
    bool matchNumber (char *strgnum);
    // Vergleicht den aktuellen lookahead mit der uebergebenen Zeichenkette
    // und schaltet bei Gleichheit lookahead auf die naechste Zeichenkette
    //  bzw. gibt bei Ungleichheit eine Fehlermeldung aus. Je nach Gleichheit
    //  wird true oder false zurueckgeliefert.

    bool valid (int character);
    bool validString (char *string);
    // Vergleicht den aktuellen lookahead mit der uebergebenen Zeichenkette
    //  und liefert je nach Gleichheit true oder false zurueck. Es wird aber
    //  noch nicht auf die naechste Zeichenkette weitergeschaltet.

    void fillIdList (Vektor <String> *idlist);
    // Liest aus dem Scanner die IdList ein und speichert sie in dem Vektor
    //  idlist.

    void emptyIdList (Vektor <String> *idlist);
    // Alle Elemente aus der uebergebenen idlist werden geloescht

    char *scannRewMeasDistExpr (int resc, char **resv, bool dist);
    // Liesst eine DistributionExpression (bei dist= true) oder eine
    //  Reward- oder Measureexpression (dist = false) mittels lexan aus dem
    //  File aus und liefert die Adresse des Strings zurueck.
    //  Alle Namen werden, nachdem sie erkannt wurden, in $-Zeichen gesetzt.
    //  In resv stehen resc reservierte Namen, die nicht umgewandelt werden.

    char *scannParameterExpr ();
    // Liesst eine Parameterexpression mittels lexan aus dem File aus und
    //  liefert die Adresse des Strings zurueck.
    //  Alle Namen werden, nachdem sie erkannt wurden, in $-Zeichen gesetzt.

    char *scannPortType ();
    // Liesst einen PortType mittels lexan aus dem File aus und liefert die
    //  Adresse des Strings zurueck.

    void scannBehind (char a);
    // Scannt das Eingabefile bis zum Erreichen des Uebergabecharacters.
    //  lookahead steht auf dem naechsten Eingabezeichen.

    void Moduleintrag (char *fileid);
    // Liesst die Datei mit dem Namen fileid ein und parst sie nach den
    //  SPNL-Regeln mittels folgender Unterfunktionen:

    void Declaration ();
    void Imports ();
    void GlobalParameterDecl ();
    void ProcessDecl ();
    void ProcesstypeDecl ();
    void ProcesstypeDeclParamentry ();
    void GlobalProcessInstantiation ();
    void GlobalProcessInstantiationParamentry ();

    void ProcessBody ();
    void PortDecl ();
    void RewardDecl (VisibleType visible);
    void MeasureDecl (VisibleType visible);
    void ParameterDecl ();
    void ProcessInstantiation ();
    void ProcessInstantiationParamentry ();
    void ProcessUsage ();
    void PlaceDecl ();
    void TransitionDecl ();
    void ArcDecl ();
    void GraphPortDecl ();
    void RewardDef ();
    void MeasureDef ();
    void TransitionAttributes ();
    void TransitionAttributesParamentry ();

  public:
    Vektor <Module> modules;   // Liste aller zu compilierenden Module

    ParserT ();
    ~ParserT ();

    void display ();
    // Gibt die komplette eingelesende Datenstruktur aus.

    int parse (char *filename);
    // Parst alle Files (einschliesslich der Importe) angefangen mit dem
    //  Initialfile ein. Liefert zurueck, ob es geklappt hat und stellt in
    //  der Liste modules die eingelesende Datenstruktur zur Verfuegung.
};


#endif
