#ifndef SCANNERCC
#define SCANNERCC

#include <string.h>	// strcpy, strlen
#include <stdlib.h>	// exit
#include <iostream.h>	// cerr
#include <ctype.h>      // isdigit
#include "scanner.h"



ScannerT::ScannerT (char *name) {
  filename= new char[strlen(name)+1];
  strcpy (filename,name);
  file.open (filename, ios::in);
  if (!file) {
    cerr << "\nScanner error: unable to open file " << filename << '\n';
    exit (1);
  }
  gesbuffer= new char[(BSIZE<<1)+1];
  if (gesbuffer == NULL) {
    cerr << "\nScanner error: unable to allocate gesbuffer\n"
         << "Increase BSIZE in module scanner.h";
    exit (1);
  }
  lexbuf= new char[LEXSIZE];
  if (lexbuf == NULL) {
    cerr << "\nScanner error: unable to allocate lexbuffer\n"
         << "Increase LEXSIZE in module scanner.h";
    exit (1);
  }
  for (int i=0; i<(BSIZE<<1); i++)
    gesbuffer[i]= ' ';
  buffer1= gesbuffer;
  buffer2= gesbuffer+BSIZE;
  gesbuffer[BSIZE-1]= (char) EOF;
  gesbuffer[(BSIZE<<1)-1]= (char) EOF;
  forward= gesbuffer+(BSIZE<<1)-2;	// vor dem Zeichen EOF
  lineno= 1;
  charno= 1;
  ueberbufferendezurueckgegangen= false;
}


ScannerT::~ScannerT () {
//cout << "vor ScannerT\n";
  file.close ();
  delete (filename);
  delete (gesbuffer);
//cout << "nach ScannerT\n";
}


char ScannerT::getnextchar () {
// forward auf das naechste Zeichen (bei Bufferende auf uebernaechstes)
//  setzten und dieses zurueckliefern.
//  Neuladen des Buffers nur bei ueberbufferendezurueckgegangen == true!
  charno++;
  if (*(++forward) == (char) EOF) {
    if (forward == buffer2 - 1) {
      if (!ueberbufferendezurueckgegangen) {
	file.read (buffer2, BSIZE-1);
	if (file.gcount () < BSIZE-1)
	  buffer2[file.gcount()]= (char) EOF;
      }
      ueberbufferendezurueckgegangen= false;
      forward= buffer2;
    }
    else if (forward == buffer2 + BSIZE - 1) {
      if (!ueberbufferendezurueckgegangen) {
	file.read (buffer1, BSIZE-1);
	if (file.gcount () < BSIZE-1)
	  buffer1[file.gcount()]= (char) EOF;
      }
      ueberbufferendezurueckgegangen= false;
      forward= buffer1;
    }  // else EOF direkt im Buffer => File-Ende, EOF zurueckliefern
//    else // EOF direkt im Buffer
//      return (DONE);            // lexikalische Analyse beendet
//    if (*forward == (char) EOF) // falls Zeiger jetzt auf EOF zeigt
//      return (DONE);            // lexikalische Analyse beendet
  }
  return (*forward);
}


void ScannerT::ungetc () {
  charno--;
  if (--forward < buffer1) {
    forward= buffer2 + BSIZE - 2;
    if (ueberbufferendezurueckgegangen) {
      cerr << '\n' << filename << ':' << lineno << ',' << charno;
      cerr << ": Scanner error: ungetc can't refresh second buffer, "
	   << "BSIZE to low.";
      exit (1);
    }
    else
      ueberbufferendezurueckgegangen= true;
  }
  else if (forward == buffer2 - 1) {
    forward--;
    if (ueberbufferendezurueckgegangen) {
      cerr << '\n' << filename << ':' << lineno << ',' << charno;
      cerr << ": Scanner error: ungetc can't refresh second buffer, "
	   << "BSIZE to low.";
      exit (1);
    }
    else
      ueberbufferendezurueckgegangen= true;
  }
}


/*
void ScannerT::getlineno (int *l, int *c) {
  *l= lineno;
  *c= charno;
}
*/


int ScannerT::lexan () {
  char t;
  int state= 0, b= 0;
  char zwerg, zwerg2;
  while (true) {
    switch (state) {
    case 0:
      b=0;
      t= getnextchar ();  // t ist Lookahead-Zeichen
      if (t == ' ')
	;
      else if (t == '\t')
	charno= (int)(((int)(--charno)/TABSIZE)+1)*TABSIZE;
      else if (t == '\n') {
	charno= 0;
	lineno++;
      }
      else if (t == '+' || t == '-')  // Beginn einer Zahl oder nur '+/-'-Char
	state= 10;
      else if (isdigit (t))  // Beginn einer Zahl
	state= 11;
      else if (t == '.') {  // Beginn einer Zahl oder nur '.'-Character
	zwerg= getnextchar ();
	ungetc ();
	if (isdigit (zwerg))  // Beginn einer Zahl ohne Vorkommastellen
	  state= 52;
	else {
	  lexbuf[0]= '\0';
	  return (t);
	}
      }
      else if (isalpha (t) || t == '_')  // Beginn eines Ids
	state= 20;
      else if (t == EOF) {
	lexbuf[0]= '\0';
	return (DONE);
      }
      else if (t == '(') {  // Beginn eines Kommentars oder '('-Character
	lexbuf[0]= '\0';
	state= 30;
      }
      else {  // anderes Zeichen
	lexbuf[0]= '\0';
	return (t);
      }
      break;

    case 10:  // nach einer Zahl scannen (+/-Zahl)
      zwerg= t;
      t= getnextchar ();
      if (isdigit (t)) {  // Es handelt sich um eine Zahl
//cout << "\n10aadding " << zwerg << " at pos " << b;
	lexbuf[b]= zwerg;
	if (++b >= LEXSIZE) {
	  error ("Scanner error: number to long.");
	  exit (1);
	}
	state= 11;
      }
      else if (t == '.') {  // '+/-.Zahl'
	zwerg2= getnextchar ();
	if (isdigit (zwerg2)) {  // Es handelt sich um eine Zahl
	  ungetc ();
//cout << "\n10badding " << zwerg << " at pos " << b;
	  lexbuf[b]= zwerg;
	  if (++b >= LEXSIZE) {
	    error ("Scanner error: number to long.");
	    exit (1);
	  }
	  state= 52;
	}
	else {
	  ungetc ();
	  ungetc ();
	  lexbuf[0]= '\0';
	  state= 0;
	  return (zwerg);
	}
      }
      else {
	ungetc ();
	lexbuf[0]= '\0';
	state= 0;
	return (zwerg);
      }
      break;
    case 11:  // Zahl
//cout << "\n11adding " << t << " at pos " << b;
      lexbuf[b]= t;
      t= getnextchar();
      if (++b >= LEXSIZE) {
	error ("Scanner error: number to long.");
	exit (1);
      }
      if (isdigit (t))
	;
      else if (t == '.')  // Zahl.Nachkommazahl
	state= 12;
      else if (t == 'e' || t == 'E')  // Zahl e/E Exponent
	state= 14;
      else
	state= 18;  // Zahl zuende
      break;
    case 12:  // Nachkommazahl
      zwerg= t;  // zwerg enthaelt '.'
      t= getnextchar();
      if (isdigit (t)) {  // Es handelt sich weiterhin um eine Zahl
//cout << "\n12adding " << zwerg << " at pos " << b;
	lexbuf[b]= zwerg;  // '.' nur eintragen, wenn Ziffer folgt
	if (++b >= LEXSIZE) {
	  error ("Scanner error: number to long.");
	  exit (1);
	}
	state= 53;
      }
      else  // Ziffer war schon zuende
	state= 19;
      break;
    case 14:
      zwerg= t;  // zwerg enthaelt 'e/E'
      t= getnextchar();
      if (isdigit (t)) {  // Es handelt sich weiterhin um eine Zahl
	ungetc ();  // t zurueckgegeben
	t= zwerg;
	state= 54;
      }
      else if (t == '+' || t == '-') {
	zwerg2= getnextchar ();
	ungetc ();  // zwerg2 zurueckgegeben
	if (isdigit (zwerg2)) {  // Es handelt sich um eine Zahl
	  ungetc ();  // t zurueckgegeben
	  t= zwerg;
	  state= 54;
	}
	else
	  state= 19;
      }
      else  // Ziffer war schon zuende
	state= 19;
      break;
    case 18:  // integerzahl zuende
      lexbuf[b]= '\0';
//cout << "\n18: " << lexbuf;
      if (t != EOF)
	ungetc ();
      state= 0;
      return (INT);
      break;
    case 19:  // integerzahl zuende, zu viele Zeichen eingelesen
      lexbuf[b]= '\0';
//cout << "\n19: " << lexbuf;
      ungetc ();  // './e/E/+/-'/t zurueckgegeben
      ungetc ();
      state= 0;
      return (INT);
      break;

    case 52:  // eventuelle Nachkommazahl einlesen
      zwerg= t;  // zwerg enthaelt '.'
      t= getnextchar();
      if (isdigit (t)) {  // Es handelt sich weiterhin um eine Zahl
//cout << "\n52adding " << zwerg << " at pos " << b;
	lexbuf[b]= zwerg;  // '.' nur eintragen, wenn Ziffer folgt
	if (++b >= LEXSIZE) {
	  error ("Scanner error: number to long.");
	  exit (1);
	}
	state= 53;
      }
      else  // Ziffer war schon zuende
	state= 59;
      break;
    case 53:  // Nachkommazahl einlesen
//cout << "\n53adding " << t << " at pos " << b;
      lexbuf[b]= t;
      t= getnextchar();
      if (++b >= LEXSIZE) {
	error ("Scanner error: number to long.");
	exit (1);
      }
      if (isdigit (t))
	;
      else if (t == 'e' || t == 'E')  // nachkommazahl e/E Exponent
	state= 54;
      else
	state= 58;  // Zahl zuende
      break;
    case 54:  // eventuellen Exponenten einlesen
      zwerg= t;  // zwerg enthaelt 'e/E'
      t= getnextchar();
      if (isdigit (t)) {  // Es handelt sich weiterhin um eine Zahl
//cout << "\n54aadding " << zwerg << " at pos " << b;
	lexbuf[b]= zwerg;  // 'e/E' nur eintragen, wenn Ziffer folgt
	if (++b >= LEXSIZE) {
	  error ("Scanner error: number to long.");
	  exit (1);
	}
	state= 56;
      }
      else if (t == '+' || t == '-') {
	zwerg2= getnextchar ();
	if (isdigit (zwerg2)) {  // Es handelt sich um eine Zahl
	  ungetc ();  // zwerg2 zurueckgegeben
//cout << "\n54badding " << zwerg << " at pos " << b;
	  lexbuf[b]= zwerg;  // 'e/E' eintragen
	  if (++b >= LEXSIZE) {
	    error ("Scanner error: number to long.");
	    exit (1);
	  }
	  state= 55;
	}
	else {
	  ungetc ();  // zwerg2 zurueckgegeben
	  state= 59;
	}	
      }
      else  // Ziffer war schon zuende
	state= 59;
      break;
    case 55:  // +/- Exponent
//cout << "\n55adding " << t << " at pos " << b;
      lexbuf[b]= t;
      t= getnextchar();
      if (++b >= LEXSIZE) {
	error ("Scanner error: number to long.");
	exit (1);
      }
      state= 56;
      break;
    case 56:  // Exponent
//cout << "\n56adding " << t << " at pos " << b;
      lexbuf[b]= t;
      t= getnextchar();
      if (++b >= LEXSIZE) {
	error ("Scanner error: id to long.");
	exit (1);
      }
      if (isdigit (t))
	;
      else
	state= 58;
      break;
    case 58:  // realzahl zuende
      lexbuf[b]= '\0';
//cout << "\n58: " << lexbuf;
      if (t != EOF)
	ungetc ();
      state= 0;
      return (REAL);
      break;
    case 59:  // realzahl zuende, zu viele Zeichen eingelesen
      lexbuf[b]= '\0';
//cout << "\n59: " << lexbuf;
      ungetc ();  // './e/E/+/-'/t zurueckgegeben
      ungetc ();
      state= 0;
      return (REAL);
      break;

    case 20:  // Ids einlesen
      lexbuf[b]= t;
      t= getnextchar();
      if (++b >= LEXSIZE) {
	error ("Scanner error: id to long.");
	exit (1);
      }
      if (isalnum (t) || t == '_') // || t == '[' || t == ']')
	;
      else if (t == '.')
	state= 21;
      else
	state= 29;
      break; 
    case 21:  // Id nach Punkt
      lexbuf[b]= t;
      t= getnextchar();
      if (++b >= LEXSIZE) {
	error ("Scanner error: qualified Id to long.");
	exit (1);
      }
      if (isalpha (t) || t == '_') // || t == '[' || t == ']')
	state= 20;
      else
	state= 28;
      break;
    case 28:
      ungetc ();  // Zeichen nach dem Punkt zurueckgeben
      ungetc ();  // Zeichen '.' selbst zurueckgeben
      lexbuf[--b]= '\0';  // Punkt aus dem String loeschen
      state= 0;
      return (ID);
      break;
    case 29:
      lexbuf[b]= '\0';
      if (t != EOF)
	ungetc ();
      state= 0;
      return (ID);
      break;

    case 30:  // Kommentar entfernen
      t= getnextchar ();
      if (t != '*') {
	ungetc ();
	return ('(');
      }
      else {  // Beginn Kommentar, Ende suchen
	do {  // ')' suchen
	  do {  // '*' suchen
	    t= getnextchar();
	    if (t == '\t')
	      charno= (int)(((int)(--charno)/TABSIZE)+1)*TABSIZE;
	    else if (t == '\n') {
	      charno= 0;
	      lineno++;
	    }
	    else if (t == EOF) {
	      error ("Scanner error: file ends in comment.");
	      exit (1);
	    }
	  }
	  while (t != '*');
	  t= getnextchar();
	  if (t != ')')  // falls Kommentar mir '**)' o.ae. endet
	    ungetc ();
	}
	while (t != ')');
      }
      state= 0;
      break;
    default:
      error ("Scanner error: Wrong state.");
      exit (1);
      break;
    }  // switch
  }  // while
}  // end


void ScannerT::error (char *mess) {
  cout << '\n' << filename << ':' << lineno << ',' << charno << ": " << mess;
}


#endif


/*
void main () {
  int c;
//  ScannerT test ("c:\\daten\\diplom\\listings\\types.h");
  ScannerT test ("protocol_layers.testreal");
  c= test.lexan ();
    while (c != DONE) {
      if (c >= 256)
	cout << test.lexbuf << " | ";
      else
	cout << (char)c << " | ";
      c= test.lexan ();
    }
}
*/
