#ifndef SCANNERH
#define SCANNERH


#include <fstream.h>    // ifstream (input file)


// Konstanten:

const int TABSIZE= 8;   // Anzahl Zeichen des Tabulators

const int BSIZE= 512;	// Buffergroesse von buffer1 bzw. buffer2 (incl. EOF)
const int LEXSIZE= 256;	// Buffergroesse von lexbuf

#define DONE 256        // wird von lexan zurueckgeliefert, falls Fileende
                        //  erreicht
#define ID   257        // Identifier
#define INT  258        // Integer
#define REAL 259        // Real
#define NUM  260        // wird nicht zurueckgeliefert, kann aber gematcht
                        //  werden, entspricht INT or REAL


class ScannerT {
  private:

    ifstream file;
    char *filename;

    char *gesbuffer;
    char *buffer1;      // buffer1 und buffer2 unterteilen den gesbuffer in
    char *buffer2;      //  zwei gleichgrosse Teile

    char *forward;      // Zeiger innerhalb des Buffers sucht bis zum Ende
                        //  des Lexems

    int lineno;         // zaehlt Zeilen
    int charno;         //  und Zeichen mit
    int ueberbufferendezurueckgegangen;  // Falls mit ungetc ein Zeichen
        //  zurueckgesprungen wird, darf der Buffer nicht erneut geladen
	//  werden.

    char getnextchar ();
    // Bewegt den Forward-Pointer um einen Schritt weiter und liefert das
    //  dortige Zeichen als Integer zurueck.

    void ungetc ();
    // Bewegt den Forward-Pointer um einen Schritt zurueck.
    // Nebeneffekt: kann nur ueber einen Buffer hinausgehen (entspricht
    //  BSIZE/2), terminiert sonst mit Fehlermeldung.

  public:
    char *lexbuf;

    ScannerT (char *filename);
    ~ScannerT ();

//    void getlineno (int *lineno, int *charno);

    int lexan ();
    // Liesst die naechste Zeichenkette aus dem File aus und speichert sie
    //  im lexbuf. Zurueckgegeben wird, ob es sich um einen String handelt,
    //  eine Zahl oder das Zeichen selbst. In lexbuf kann dieser Wert
    //  abgefragt werden.

    void error (char *message);
    // Gibt eine Fehlermeldung mit Zeilenangabe aus.
};


#endif
