#ifndef SPNLBASICSH
#define SPNLBASICSH



/**************************************************************************
 *                                                                        *
 *                            Globale Variablen                           *
 *                                                                        *
 **************************************************************************/

bool dots= false; // Waehrend der Berechnung Punkte als Ausgabe?
bool errornetdisplay= false; // Fehlerbehaftete Netze auch ausgegeben?
bool calcarcmult= true; // Arc-Multiplizitaeten auf gradzahlige, positive
                        //  Werte ueberpruefen?


/**************************************************************************
 *                                                                        *
 *                  Konstanten fuer das flache Petri-Netz                 *
 *                                                                        *
 **************************************************************************/

const int GLOBALNUMRES= 8;
// Zu reservierende Felder im globalid-String fuer 1 bis 2 Characters
//  fuer den Typ (*), bis zu 5 Stellen fuer globale Nummer und 1 Character 
//  fuers String-Ende '\0'. oder bei Prozessinstanzenarrays 2 Characters
//  fuer [], 5 fuer die Ziffer und 1 fuer \0. (compiler.h)
//
// *Typ| steht fuer
// ====|=================================================================
//  cp | constant parameter (normale Parameter)
//  mp | measparam (Uebergabeparameter, der eine single measure enthaelt)
//  mm | multiple measure (ueber rate rewards)
//  sm | single measure (ueber alle rewards)
//  rr | rate reward (ueber Plaetze)
//  ir | impulse reward (ueber Transitionen)
//  cr | combined reward (ueber Plaetze und Transitionen)
//  t  | transition
//  p  | places
//  a  | arcs
//  pa | arcs, die ueber einen Port liefen und nun zusammengefasst sind
//  o  | ports
//  pi | processinstance
//  pu | process usage

const int MAXGLOBALANZ= 99999;
// Maximale Anzahl an Transitionen, Plaetzen, Rewards, Prozessen u.a.
//  99999= 10^(GLOBALNUMRES-3)-1.
//  Ist auch maximale Anzahl der Arraygroesse fuer Prozessinstanzen.
//  MAXARRAYANZ <= MAXGLOBALANZ muss gelten, da sonst nicht alle Instanzen
//  erzeugt werden koennen (es wurden mehr Prozesse erlaubt sein, als z.B.
//  Arcs: unlogisch!)


const int MAXARRAYRES= 4;
// Maximal zu reservierende Chars fuer den Arrayindex (spnlprocess.cc)

const int MAXARRAYANZ= 9999;
// Maximal erlaubte Zahl fuer Prozessarrays (MAXARRAYANZ= (10^MAXARRAYRES)-1)
//  MAXARRAYANZ <= MAXGLOBALANZ muss gelten, da sonst nicht alle Instanzen
//  erzeugt werden koennen (es wurden mehr Prozesse erlaubt sein, als z.B.
//  Arcs: unlogisch!)



/**************************************************************************
 *                                                                        *
 *                 Reservierte Woerter fuer SPNL-Konstrukte               *
 *                                                                        *
 **************************************************************************/


const int REW_NUMRESWORDS= 11;

char* rew_reserved[REW_NUMRESWORDS]= {
  "if", "then", "else", "elseif",
  "and", "or", "not", "true", "false",
  "e", "E"
};


const int DIST_NUMRESWORDS= 14;

char* dist_reserved[DIST_NUMRESWORDS]= {  // includes rew_reserved
  "if", "then", "else", "elseif",
  "and", "or", "not", "true", "false",
  "I", "R", "x", "e", "E"     // "portdep"
};


const int MEAS_NUMRESWORDS= 12;

char* meas_reserved[MEAS_NUMRESWORDS]= {  // includes rew_reserved
  "if", "then", "else", "elseif",
  "and", "or", "not", "true", "false",
  "dist", "e", "E"                        // , "ArithmeticMean"
};


const int ALL_NUMRESWORDS= 18;

char* all_reserved[ALL_NUMRESWORDS]= {  // includes meas, dist und rew (s.o.)
  "if", "then", "else", "elseif",
  "and", "or", "not", "true", "false",
  "dist", "ArithmeticMean", "guess",
  "portdep", "I", "R", "x", "e", "E"
};
// ******* evtl. noch aufstocken um "processinstance" "parameter" "end" "arc"



/**************************************************************************
 *                                                                        *
 *                      Basistypen fuer SPNL-Elemente                     *
 *                                                                        *
 **************************************************************************/


enum DistributionType {
	NODISTR=0, EXP, UNIFORM, TRUNCNORMAL, ERLANG, WEIBULL, PHASETYPE, DET,
	BERNOULLI, DISCRETEUNIFORM, GEOMETRIC, MODGEOMETRIC, BINOMIAL,
	POISSON, EXPOLY
	};


enum MarkingDependencyType {
	NOMARKDEP=0, ENABDEP, SCALING, CONSTANT
	};


enum PolicyType {
	NOPOLICY=0, PRD, PRS, PRI
	};


enum ConcurrencyDegree {
	NOCONC=0, IS, SS
	};


enum TransitionType {
	NOTRTYPE=0, TIMED, IMM
	};


enum PortType {
	NOPORTTYPE=0,
	USEDPORT,
	PinT, TinP,		// p<-t, t<-p,
	PoutT, ToutP,		// p->t, t->p,
	PinhiboutT, TinhibinP	// p-ot, to-p
	};


enum ArcType {
//	NOARCTYPE=0, IOARC, HARC
	IOARC=0, HARC
	};


enum VisibleType {
	PRIVATE=0, PUBLIC
	};


enum ProcessTypeDeclType {
	TYPE=0, DECL
	};


enum FormalParameterType {
        PARAM=0, RREWARD, IREWARD, CREWARD, SMEASURE, MMEASURE, MEASPARAM
        };



/**************************************************************************
 *                                                                        *
 *                       Hilfsfunktionen zur Ausgabe                      *
 *                                                                        *
 **************************************************************************/


char *DistributionType2String (DistributionType enumval);
DistributionType String2DistributionType (char *s);

char *MarkingDependencyType2String (MarkingDependencyType enumval);
MarkingDependencyType String2MarkingDependencyType (char *s);

char *PolicyType2String (PolicyType enumval);
PolicyType String2PolicyType (char *s);

char *ConcurrencyDegree2String (ConcurrencyDegree enumval);
ConcurrencyDegree String2ConcurrencyDegree (char *s);

char *TransitionType2String (TransitionType enumval);
TransitionType String2TransitionType (char *s);

char *PortType2String (PortType enumval);
PortType String2PortType (char *s);

char *ArcType2String (ArcType enumval);
ArcType String2ArcType (char *s);

char *VisibleType2String (VisibleType enumval);
VisibleType String2VisibleType (char *s);

char *ProcessTypeDeclType2String (ProcessTypeDeclType enumval);
ProcessTypeDeclType String2ProcessTypeDeclType (char *s);

char *FormalParameterType2String (FormalParameterType enumval);
FormalParameterType String2FormalParameterType (char *s);



/**************************************************************************
 *                                                                        *
 *                        Funktionen zum SPNL-check                       *
 *                                                                        *
 **************************************************************************/


bool isValidName (char *id);
// Liefert zurueck, ob es sich um ein gueltigen Identifier handelt oder um
//  ein vordefiniertes Schluesselwort. (benutzt all_reserved[] s.o.)


#endif
