#ifndef SPNLCHECKCC
#define SPNLCHECKCC


#include "spnlmodule.cc"
#include "spnlprocess.cc"
#include "spnltypes.cc"
#include "spnlbasics.cc"
#include "vektor.cc"
#include "stdfun.cc"
#include "spnlcheck.h"


/************************ GlobalTable ************************/

GlobalTable::GlobalTable (char *id, char *idtype, char *moduleid) {
  this->id= newstring (id);
  this->idtype= newstring (idtype);
  this->moduleid= newstring (moduleid);
}

GlobalTable::~GlobalTable () {
  delete (id);
  id= NULL;
  delete (idtype);
  idtype= NULL;
  delete (moduleid);
  moduleid= NULL;
}

char *GlobalTable::getid () {
  return (id);
}

char *GlobalTable::getidtype () {
  return (idtype);
}

char *GlobalTable::getmoduleid () {
  return (moduleid);
}



/********************** SyntaxCheckerT **********************/

SyntaxCheckerT::SyntaxCheckerT () {
  errorfree= true;
  sectionct= 0;
}


SyntaxCheckerT::~SyntaxCheckerT () {
//cout << "vor SyntaxCheckerT\n";
//cout << "nach SyntaxCheckerT\n";
}


/********************* private functions *********************/

/******************** einheitliche Namen *********************/

void SyntaxCheckerT::add2IdList (char *id, char *idtype, char *moduleid) {
  GlobalTable *s= IdList.searchId (id);
  if (s) {       // kann nicht hinzufuegen, da schon vorhanden
    cout << "\nmodule " << moduleid << ": cannot declare " << idtype << ' '
         << id << ". Identifier " << id << " already defined as "
	 << s->getidtype () << " in module " << s->getmoduleid ();
    errorfree= false;
  }
  else {
    s= new GlobalTable (id, idtype, moduleid);
    IdList.addElement (s);  
  }
}


void SyntaxCheckerT::checkUniqueNames () {
// Innerhalb aller Module schauen, ob alle Parameter, Prozesse,
//  ProzessInstanzen und ProzessTypen eindeutige (unique) Namen haben.

  int i= 0, modulindex= 0;
  Module *modulept= NULL;
  Parameter *param;
  ProcessInst *procinst;
  ProcessTypeDecl *proc;

  for (modulindex= 0; modulindex < modules->size(); modulindex++) {
    modulept= modules->readElementAt (modulindex);

    i= 0;
    param= modulept->getParameter (i);
    while (param) {
      add2IdList (param->getid (), "parameter", modulept->getid ());
      param= modulept->getParameter (++i);
    }

    i= 0;
    procinst= modulept->getProcessInst (i);
    while (procinst) {
      add2IdList (procinst->getid (), "process instance", modulept->getid ());
      procinst= modulept->getProcessInst (++i);
    }

    i= 0;
    proc= modulept->getProcess (i);
    while (proc) {
      if (proc->getType () == DECL)
	add2IdList (proc->getid (), "process", modulept->getid ());
      else
	add2IdList (proc->getid (), "process type", modulept->getid ());
      proc= modulept->getProcess (++i);
    }

  }
}


/********************* rekursive importe *********************/

void SyntaxCheckerT::entryImports (Vektor <String> *importList,
				   Module *modulept) {
// Alle Importe des Moduls modulept in die importList eintragen
  String *s= NULL;
  int i= 0;
  char *import= modulept->getImport (i);
  while (import) {
    if (!importList->searchId (import)) {
      s= new String (import);
      importList->addElement (s);
    }
    import= modulept->getImport (++i);
  }
}


void SyntaxCheckerT::checkRecursiveImports () {
// Ueberprueft, dass keine Module sich rekursiv importieren.
  int modulindex= 0, importindex= 0;
  Module *modulept= NULL, *zwerg= NULL;
  Vektor <String> importList;
  for (modulindex= 0; modulindex < modules->size(); modulindex++) {
    modulept= modules->readElementAt (modulindex);
    entryImports (&importList, modulept);  // Importe des eintragen
    for (importindex=0; importindex < importList.size (); importindex++) {
      zwerg= modules->searchId (importList.readElementAt (importindex)
				->getid ());
      // Aus der Importliste Eintrag fuer Eintrag abarbeiten und diesen
      //  Namen in allen Modulen suchen, entsprechendes Modul in zwerg merken.
      if (zwerg)
	entryImports (&importList, zwerg);  // Alle Importe mit eintragen
      else {
	cerr << "\nSyntaxCheckerT::checkRecursiveImports: module "
	     << importList.readElementAt (importindex)->getid ()
	     << " not found.\n";
	exit (1);
      }
    }  // Aus allen Modulen alle Importe eingetragen
    while (importList.size () > 0) {    // importList checken & loeschen
      // Das Modul modulept (von dem aus die Importe eingetragen wurden), darf
      //  nicht in der Importliste enthalten sein, sonst haette es sich selbst
      //  importiert.
      if (!strcmp (importList.readElementAt (0)->getid (),
		   modulept->getid ()))
	error_message (modulept->getid (), NULL, "module", NULL, 8,
		       modulept->getid ());
      delete (importList.removeElementAt (0));
    }
  }
}


/********************* rekursive aufrufe *********************/

void SyntaxCheckerT::entryProcs (Vektor <String> *procList,
				 ProcessTypeDecl *process) {
// Alle ProzessInstanzen und -usages aus process in die Liste eintragen.
  String *s= NULL;
  int i= 0;
  ProcessUsage *procu= process->getProcessUsage (i);
  while (procu) {
    if (!procList->searchId (procu->getProcName())) {
      s= new String (procu->getProcName());
      procList->addElement (s);
    }
    procu= process->getProcessUsage (++i);
  }
  i= 0;
  ProcessInst *proci= process->getProcessInst (i);
  while (proci) {
    if (!procList->searchId (proci->getProcessType())) {
      s= new String (proci->getProcessType());
      procList->addElement (s);
    }
    proci= process->getProcessInst (++i);
  }
}


void SyntaxCheckerT::entryRewards (Vektor <String> *rewardList,
				   Reward *reward) {
// Alle Identifier aus der Rewardexpression in die Liste eintragen.
  String *s= NULL;
  char *hexpr= NULL;
  char *expression= newstring (reward->getRewardExpression ());
  char *id= get_first_name (expression);
  while (id) {
    if (!rewardList->searchId (id)) {
      s= new String (id);
      rewardList->addElement (s);
    }
    delete (id);
    hexpr= expression;
    expression= replace_first_name (hexpr, NULL);
    delete (hexpr);
    id= get_first_name (expression);
  }
  delete (expression);
}


void SyntaxCheckerT::entryMeasures (Vektor <String> *measureList,
				   Measure *measure) {
// Alle Identifier aus der Measureexpression in die Liste eintragen.
  String *s= NULL;
  char *hexpr= NULL;
  char *expression= newstring (measure->getMeasureExpression ());
  char *id= get_first_name (expression);
  while (id) {
    if (!measureList->searchId (id)) {
      s= new String (id);
      measureList->addElement (s);
    }
    delete (id);
    hexpr= expression;
    expression= replace_first_name (hexpr, NULL);
    delete (hexpr);
    id= get_first_name (expression);
  }
  delete (expression);
}


void SyntaxCheckerT::entryParameters (Vektor <String> *paramList,
				      Parameter *param) {
// Alle Identifier aus der Parameterexpression in die Liste eintragen.
  String *s= NULL;
  char *hexpr= NULL;
  char *expression= newstring (param->getParameterExpression ());
  char *id= get_first_name (expression);
  while (id) {
    if (!paramList->searchId (id)) {
      s= new String (id);
      paramList->addElement (s);
    }
    delete (id);
    hexpr= expression;
    expression= replace_first_name (hexpr, NULL);
    delete (hexpr);
    id= get_first_name (expression);
  }
  delete (expression);
}


void SyntaxCheckerT::checkRecursiveCalls () {
// Ueberprueft, dass weder Prozess[typen], Parameter, Rewards noch Measures
//  sich rekursiv benutzen.
  int modulindex= 0, procindex= 0, rewardindex= 0, measureindex= 0,
      paramindex= 0, listindex= 0;
  Module *checkmodule= NULL;
  ProcessTypeDecl *checkprocess= NULL, *newprocess= NULL;
  Reward *checkreward= NULL, *newreward= NULL;
  Measure *checkmeasure= NULL, *newmeasure= NULL;
  Parameter *checkparam= NULL, *newparam= NULL;
  Vektor <String> procList, rewardList, measureList, paramList;

  for (modulindex= 0; modulindex < modules->size(); modulindex++) {
    checkmodule= modules->readElementAt (modulindex);// alle Module durchgehen

    procindex= 0;
    checkprocess= checkmodule->getProcess (procindex);
    while (checkprocess) {  // in allen Modulen alle Prozess[typen] durchgehen

      // TYPE & DECL durchchecken, da es sonst Probleme mit den formalen
      //  Parametern gibt. Sonst: if (checkprocess->getType() == TYPE) ...
      // Jetzt sammeln aller in diesem Prozesstypen benutzen anderen
      //  Prozesstypen und Prozesse.
      entryProcs (&procList, checkprocess);
      // Jetzt zu jedem Prozesstypen (oder Prozess) seine weiteren benutzten
      //  Prozesstypen und Prozesse heraussuchen und -wenn noch nicht in der
      //  Liste vorhanden- neu dazu eintragen.
      for (listindex= 0; listindex < procList.size(); listindex++) {
	newprocess= searchProcessTypeDecl (NULL,
			  procList.readElementAt (listindex)->getid ());
	if (newprocess)  // dort benutzte Prozesse auch in die Liste einfuegen
	  entryProcs (&procList, newprocess);
	else {
	  cerr << "\nSyntaxCheckerT::checkRecursiveProcs: process[type] "
	       << procList.readElementAt (listindex)->getid ()
	       << " not found.\n";
	  exit (1);
	} // if (newprocess)
      } // listindex-schleife
      while (procList.size () > 0) {    // ProcList checken & loeschen
	// Der Prozess checkprocess (von dem aus alles angefangen hat) darf
	//  nicht in der ProzessListe enthalten sein, sonst haette er sich
	//  selbst aufgerufen.
	if (!strcmp (procList.readElementAt (0)->getid (),
		     checkprocess->getid ()))
	  error_message (checkmodule->getid (), checkprocess->getid (),
			 "process[type]", NULL, 9, checkprocess->getid ());
	delete (procList.removeElementAt (0));
      }

      rewardindex= 0;
      checkreward= checkprocess->getReward (rewardindex);
      while (checkreward) {
	entryRewards (&rewardList, checkreward);
	// Jetzt zu jedem Reward seine weiteren benutzten Rewards heraussuchen
	//  und -wenn noch nicht in der Liste vorhanden- mit eintragen.
	for (listindex= 0; listindex < rewardList.size(); listindex++) {
	  newreward= checkprocess->searchReward
			    (rewardList.readElementAt (listindex)->getid ());
	  if (newreward)
	    entryRewards (&rewardList, newreward); // else: war place oder ae.
	} // listindex-schleife
	while (rewardList.size () > 0) {    // List checken & loeschen
	  if (!strcmp (rewardList.readElementAt (0)->getid (),
		       checkreward->getid ()))
	    error_message (checkmodule->getid (), checkprocess->getid (),
			   "reward", checkreward->getid (), 9,
			   checkreward->getid ());
	  delete (rewardList.removeElementAt (0));
	}
	checkreward= checkprocess->getReward (++rewardindex);
      } // while (checkreward)

      measureindex= 0;
      checkmeasure= checkprocess->getMeasure (measureindex);
      while (checkmeasure) {
	entryMeasures (&measureList, checkmeasure);
	// Jetzt zu jedem Measure seine weiteren benutzten Measures heraus-
	//  suchen und -wenn noch nicht in der Liste vorhanden- mit eintragen.
	for (listindex= 0; listindex < measureList.size(); listindex++) {
	  newmeasure= checkprocess->searchMeasure
			    (measureList.readElementAt (listindex)->getid ());
	  if (newmeasure)
	    entryMeasures (&measureList, newmeasure); // else: warwasandres.
	} // listindex-schleife
	while (measureList.size () > 0) {    // List checken & loeschen
	  if (!strcmp (measureList.readElementAt (0)->getid (),
		       checkmeasure->getid ()))
	    error_message (checkmodule->getid (), checkprocess->getid (),
			   "measure", checkmeasure->getid (), 9,
			   checkmeasure->getid ());
	  delete (measureList.removeElementAt (0));
	}
	checkmeasure= checkprocess->getMeasure (++measureindex);
      } // while (checkmeasure)

      paramindex= 0;
      checkparam= checkprocess->getParameter (paramindex);
      while (checkparam) {
	entryParameters (&paramList, checkparam);
	// Jetzt zu jedem Param seine weiteren benutzten Params heraus-
	//  suchen und -wenn noch nicht in der Liste vorhanden- mit eintragen.
	for (listindex= 0; listindex < paramList.size(); listindex++) {
	  newparam= searchParameter (NULL, checkprocess,
			    paramList.readElementAt (listindex)->getid ());
	  if (newparam && (newparam->getprocid() != NULL))
	    entryParameters (&paramList, newparam); // else: warwasandres.
	} // listindex-schleife
	while (paramList.size () > 0) {    // List checken & loeschen
	  if (!strcmp (paramList.readElementAt (0)->getid (),
		       checkparam->getid ()))
	    error_message (checkmodule->getid (), checkprocess->getid (),
			   "parameter", checkparam->getid (), 9,
			   checkparam->getid ());
	  delete (paramList.removeElementAt (0));
	}
	checkparam= checkprocess->getParameter (++paramindex);
      } // while (checkparam)

      checkprocess= checkmodule->getProcess (++procindex);
    } // while (checkprocess)

    paramindex= 0;  // jetzt globale Parameter
    checkparam= checkmodule->getParameter (paramindex);
    while (checkparam) {
      entryParameters (&paramList, checkparam);
      // Jetzt zu jedem Param seine weiteren benutzten Params heraus-
      //  suchen und -wenn noch nicht in der Liste vorhanden- mit eintragen.
      for (listindex= 0; listindex < paramList.size(); listindex++) {
	newparam= searchParameter (NULL, NULL,
			    paramList.readElementAt (listindex)->getid ());
	if (newparam)
	  entryParameters (&paramList, newparam); // else: warwasandres.
	else {
	  cerr << "\nSyntaxCheckerT::checkRecursiveCalls: parameter "
	       << paramList.readElementAt (listindex)->getid ()
	       << " not found.\n";
	  exit (1);
	} // if (newparam)
      } // listindex-schleife
      while (paramList.size () > 0) {    // List checken & loeschen
	if (!strcmp (paramList.readElementAt (0)->getid (),
		     checkparam->getid ()))
	  error_message (checkmodule->getid (), NULL,
			 "global parameter", checkparam->getid (), 9,
			 checkparam->getid ());
	delete (paramList.removeElementAt (0));
      }
      checkparam= checkmodule->getParameter (++paramindex);
    } // while (checkparam)

    if (dots)
      cout << '.' << flush;

  } // modulindex-schleife
}


/********************* alles definiert? *********************/


void SyntaxCheckerT::error_unknown (char *mod, char *proc, char *typ,
				    char *id, char *unknown, char *procname) {
//if (!procname) cout << "\n**************procname fehlt***********\n"<< flush;else cout << "\n**************procname da: " << flush << procname[0]<< flush << procname[1]<< flush << procname[2] << flush<< procname[3]<< flush << procname[4]<< flush << procname[5]<< flush << procname[6]<< flush << procname[7] << flush<< procname[8] << flush<< procname[9]<< flush << "***********" << flush;
  cout << "\nmodule " << mod;
  if (proc)
    cout << ", process[type] " << proc;
  cout << ", " << typ << ' ' << id;
  if (unknown)
    cout << ": identifier " << unknown;
  if (procname)
    cout << " of proc[type] " << flush << procname;
  cout << " not defined or of wrong type";
  errorfree= false;
}


void SyntaxCheckerT::error_message (char *mod, char *proc, char *typ,
				    char *id, int errormess, char *errid) {
// Fuer die Ausgabe verschiedenster Fehlermeldungen.
  cout << "\nmodule " << mod;
  if (proc)
    cout << ", process[type] " << proc;
  if (id)
    cout << ", " << typ << ' ' << id;
  cout << ": ";
  switch (errormess) {
  case 0:
    cout << "guess expression ";
    display_dollarexpr (errid);
    cout << " can only be used in measure parameters";
    break;
  case 1:
    cout << "missing guess expression for measure parameter " << errid;
    break;
  case 2:
    cout << "process type " << errid << " contains measure parameters and"
         << " so can only be used on the global level";
    break;
  case 3:
    cout << "process type " << errid << " contains measure parameters and"
         << " must not be used in an array of process instances";
    break;
  case 4:
    cout << "too many actual parameters";
    if (errid)
      cout << ", cannot declare " << errid;
    break;
  case 5:
    cout << "too few actual parameters";
    if (errid)
      cout << ", cannot declare " << errid;
    break;
  case 6:
    cout << "array size " << errid << " less than 1";
    break;
  case 7:
    cout << "array size " << errid << " greater than " << MAXARRAYANZ;
    break;
  case 8:
    cout << typ << ' ' << errid << " imports itself recursively";
    break;
  case 9:
    cout << typ << ' ' << errid << " contains itself recursively";
    break;
  case 10:
    cout << "identifier " << errid << " contains wrong array size";
    break;
  case 11:
    cout << "expression " << errid << " contains more than one single name";
    break;
  default:
    cout << "wrong message: " << errormess << " with expr= " << errid;
  }
  errorfree= false;
}


void SyntaxCheckerT::verifyExpression (Module *mod, ProcessTypeDecl *process,
		     char *checkexpression, FormalParameterType exprtype,
		     char *errortype, char *errorid) {
//
//       Typ      | Kann folgende Elemente enthalten:
// ===============|===================================================
//       cp       | cp
//       mp       | sm-id
//       mm       | place, rr, mm, cp, mp
//       sm       | place, trans, sm, cr, ir, rr, cp, mp
//       rr       | place, rr, cp, mp
//       ir       | trans, ir, cp, mp + innerhalb if () auch rr, place
//       cr       | place, trans, rr, ir, cr, cp, mp
//    port-mult.  | siehe cp
//   place-mark.  | siehe cp
//   trans-prio.  | siehe rr
//   trans-dist.  | siehe rr
//  trans-weight  | siehe rr
//   trans-guard  | siehe rr
//    arc-mult.   | siehe rr

  if (!checkexpression)
    return;
  char *hexpr= NULL, *expression= NULL, *arraysize= NULL,
       *id= NULL, *newid= NULL, *newproc= NULL;
  ProcessTypeDecl *usedproc= NULL;
  ProcessInst *proci= NULL;
  Reward *reward= NULL;
  Measure *measure= NULL;
  Place *place= NULL;
  Transition *trans= NULL;
  Parameter *param= NULL;
  FormalParameter *forpar= NULL;
  bool innerhalbarithmeticmean= false;

  char *processid= NULL;
  if (process)
    processid= process->getid ();

/*
  if (((exprtype != PARAM) && !process) || !mod) {
    cerr << "\nSyntaxCheckerT::verifyExpression: falscher Aufruf\n";
    exit (1);
  }
*/

  if (exprtype == MEASPARAM)
    exprtype= SMEASURE; // Measparams werden wie single measures behandelt

  expression= newstring (checkexpression);
  id= get_first_name (expression);
  while (id) {
    newid= lastsubstring (id);  // Schauen, ob ein Punkt in dem Namen
                                //  existiert, wenn ja => newid != NULL
    if (newid && (exprtype != PARAM) && !name_after_dcross (expression)) {
      // Expression benutzt Werte aus einem anderen Prozess, ist aber nur
      //  erlaubt, wenn es kein Parameter, Port, Place oder Transition ist
      //  (nur Rewards und Measures koennen public sein)
      newproc= substring2point (id);  // Liefert Namen bis zum Punkt
      // newproc untersuchen, ob Array:
      if (search_substring (newproc, "[")) {
	strtok (newproc, "[");  // newproc ist jetzt der reine Prozessname
	arraysize= strtok (NULL, "]");  // arraysize nie loeschen!
      }
      else
	arraysize= NULL;
      proci= searchProcessInst (mod, process, newproc);
      // schauen, ob ProcInst newproc im Prozess oder global deklariert ist
      usedproc= searchProcess (mod, process, newproc);
      // sucht entweder den zu proci passenden ProzessTypen oder den benutzten
      //  Prozess (processusage).
      if (arraysize && usedproc) {
	int groesseval= 0;
	sscanf (arraysize, "%d", &groesseval);  // array-index einlesen
	if ((groesseval < 1) || (groesseval > proci->getArrayval ()))
	  error_message (mod->getid(), processid, errortype, errorid, 10, id);
      }
      if (usedproc &&
	  ((innerhalbarithmeticmean &&
	    !(proci && (proci->getArrayval () > 0) && !arraysize)) ||
	   (!innerhalbarithmeticmean &&
	    ((proci && (((proci->getArrayval () > 0) && !arraysize) ||
			((proci->getArrayval () == 0) && arraysize))) ||
	     (!proci && arraysize)))))
	// Wenn ein benutzter Prozess gefunden wurde und sich der aktuelle
	//  Identifier innerhalb eines ArithmeticMean-Ausdrucks befindet, darf
	//  der Identifier kein Array-Index haben aber die passende Prozess-
	//  instanz muss aber ein Array sein.
	// Befindet sich der Identifier nicht innerhalb eines ArithmeticMean-
	//  Ausdrucks, so muss, wenn der Identifier einen Array-Index hat,
	//  auch die entsprechende Prozessinstanz ein Array sein; hat der
	//  Identifier hingegen keinen Array-Index, so darf die entsprechende
	//  Prozessinstanz keinesfalls als Array deklariert sein.
	error_unknown (mod->getid (), processid, errortype,
		       errorid, id, usedproc->getid ());
      else {
	char *usedprocid= NULL;
	if (usedproc)
	  usedprocid= usedproc->getid ();
	if ((exprtype == RREWARD) || (exprtype == IREWARD) ||
	    (exprtype == CREWARD)) {
	  if (usedproc)
	    reward= usedproc->searchReward (newid);
	  if (!(reward && (reward->getVisibility () == PUBLIC) &&
		(((reward->getType() == 'r') &&
		  ((exprtype != IREWARD) || name_during_if (expression))) ||
		 ((reward->getType() == 'i') && (exprtype != RREWARD)) ||
		 (exprtype == CREWARD))))
	    // Wurde ein passender Reward gefunden, der auch public deklariert
	    //  ist, so ist dieser in einem Ausdruck IREWARD erlaubt, wenn
	    //  er selber ein IREWARD ist oder ein RREWARD ist und in einem
	    //  Praedikat-Ausdruck (if) benutzt wird. In einem Ausdruck von
	    //  einem RREWARD hingegen muss er selbst auch ein RREWARD sein,
	    //  in CREWARD-Ausdruecken ist es egal, was als fuer ein Reward
	    //  er deklariert ist.
	    if (usedprocid)
	      error_unknown (mod->getid (), processid, errortype,
			     errorid, newid, usedprocid);
	    else
	      error_unknown (mod->getid (), processid, errortype,
			     errorid, newproc, usedprocid);
	}
	else if ((exprtype == SMEASURE) || (exprtype == MMEASURE)) {
	  if (usedproc) {
	    measure= usedproc->searchMeasure (newid);
	    if (!measure)
	      reward= usedproc->searchReward (newid);
	  }
	  if (!((reward && (reward->getVisibility () == PUBLIC) &&
		 ((exprtype == SMEASURE) || (reward->getType () == 'r'))) ||
		(measure && (measure->getVisibility () == PUBLIC) &&
		 (((measure->getType() == 's') && (exprtype == SMEASURE)) ||
		  ((measure->getType() == 'm') && (exprtype == MMEASURE))))))
	    // Wurde ein passender Reward gefunden, der auch public deklariert
	    //  ist, so muss er, wenn er sich innerhalb eines MMEASURE-Aus-
	    //  drucks befindet, ein RREWARD sein, befindet er sich hingegen
	    //  in einer SMEASURE, so ist es egal, um was fuer einen Typ von
	    //  Reward es sich handelt.
	    // Wurde eine passende, public deklarierte, Measure gefunden, so
	    //  darf sie als SMEASURE nur in SMEASURE-Ausdruecken und als
	    //  MMEASURE nur in MMEASURE-Ausdruecken benutzt werden.
	    if (usedprocid)
	      error_unknown (mod->getid (), processid, errortype,
			     errorid, newid, usedprocid);
	    else
	      error_unknown (mod->getid (), processid, errortype,
			     errorid, newproc, usedprocid);
	}
      }
      delete (newproc);
      delete (newid);
    }
    else {  // aktueller Wert nicht aus einem anderen Prozess
      if (exprtype == PARAM) {
	if (process)
	  forpar= process->searchFormalParameter (id);
	if (!forpar || (forpar->getFormalParameterType () == RREWARD)) {
	  // Formale Parameter sind als RREWARD, MEASPARAM und PARAM moeglich,
	  //  nur RREWARDS sind in Parametern nicht erlaubt.
	  param= searchParameter (mod, process, id);
	  if (!param || name_after_dcross (expression))
	    error_unknown (mod->getid (), processid, errortype,
			   errorid, id, NULL);
	}
      }
      else if ((exprtype == RREWARD) || (exprtype == IREWARD) ||
	  (exprtype == CREWARD)) {
	if (!name_after_dcross (expression) && process)
	  reward= process->searchReward (id);
	if (!(reward &&
	      (((reward->getType() == 'r') &&
		((exprtype != IREWARD) || name_during_if (expression))) ||
	       ((reward->getType() == 'i') &&
		(exprtype != RREWARD)) || (exprtype == CREWARD)))) {
	  // Reward-Bedingungen siehe oben
	  if (name_after_dcross (expression)) {
	    // Nach einem Doppelkreuz (#) duerfen in RREWARDS Plaetze, in
	    //  IREWARDS Transitionen und in CREWARDS beide Typen vorkommen.
	    //  Ausnahme bildet der IREWARDS, der innerhalb eines Praedikates
	    //  ebenfalls Plaetze (wie auch schon RREWARDs) enthalten darf.
	    if ((exprtype != RREWARD) && process)  // ir || cr
	      trans= process->searchTransition (id);
	    if (process && !trans && ((exprtype != IREWARD)
			   || name_during_if (expression)))  // rr || cr
	      place= process->searchPlace (id);
	  }
	  if (!(trans || place)) {
	    if (!name_after_dcross (expression) && process)
	      forpar= process->searchFormalParameter (id);
	    if (!forpar || ((forpar->getFormalParameterType () == RREWARD)
	      && (exprtype == IREWARD) && !name_during_if (expression))) {
	      param= searchParameter (mod, process, id);
	      if (!param)
		error_unknown (mod->getid (), processid, errortype,
			       errorid, id, NULL);
	    }
	  }
	}
      }
      else if ((exprtype == SMEASURE) || (exprtype == MMEASURE)) {
	if (!name_after_dcross (expression) && process)
	  measure= process->searchMeasure (id);
	if (!(measure &&
	      (((measure->getType() == 's') && (exprtype == SMEASURE)) ||
	       ((measure->getType() == 'm') && (exprtype == MMEASURE))))) {
	  // SMEASUREs sind nur in SMEASUREs erlaubt, MMEASUREs nur in
	  //  MMEASUREs (und zwar natuerlich nicht hinter einem #).
	  if (!name_after_dcross (expression) && process)
	    reward= process->searchReward (id);
	  if (!(reward && ((exprtype == SMEASURE)
	      || ((reward->getType() == 'r') && (exprtype == MMEASURE))))) {
	    // MMEASUREs duerfen nur RREWARDs oder Transitionen enthalten.
	    if (name_after_dcross (expression) && process)
	      place= process->searchPlace (id);
	    if (!place) {
	      if ((exprtype == SMEASURE) && name_after_dcross (expression)
		  && process)
		trans= process->searchTransition (id);
	        // Transitionen nur in SMEASUREs erlaubt
	      if (!trans) {
		if (!name_after_dcross (expression) && process)
		  forpar= process->searchFormalParameter (id);
		if (!forpar) {  // alle FormalParameterTypen entlaubt
		  if (!name_after_dcross (expression))
		    param= searchParameter (mod, process, id);
		  if (!(param || ((!strcmp (id, "ArithmeticMean")
				   && (exprtype == SMEASURE)))))
		    error_unknown (mod->getid (), processid,
				   errortype, errorid, id, NULL);
		}
	      }
	    }
	  }
        }
      }
    }  // Puhh, sind das viele Klammern...
    hexpr= expression;
    innerhalbarithmeticmean= false;
    if (!strcmp (id, "ArithmeticMean") && (exprtype == SMEASURE))
      innerhalbarithmeticmean= true;
    expression= replace_first_name (hexpr, NULL);
    delete (hexpr);
    delete (id);
    reward= NULL;
    measure= NULL;
    place= NULL;
    trans= NULL;
    param= NULL;
    forpar= NULL;
    proci= NULL;
    usedproc= NULL;
    id= get_first_name (expression);
  }
  delete (expression);
}


void SyntaxCheckerT::setProcInstArrayval (Module *mod, ProcessTypeDecl *proc,
					  ProcessInst *procinst) {
  char *expr= NULL;
  int arraygroesse= 0;
  char *processid= NULL;
  if (proc)
    processid= proc->getid ();

  expr= procinst->getArrayanz ();  // Arraygroesse als char* auslesen
  if (expr) {
    arraygroesse= getExpressionValue (mod, proc,
		  "array size of process instance", procinst->getid (), expr);
    if (arraygroesse < 1) {  // zu klein
      error_message (mod->getid (), processid, "process instance",
		     procinst->getid (), 6, expr);
      arraygroesse= 1;  // damit weitergerechnet werden kann
    }
    if (arraygroesse > MAXARRAYANZ) {  // zu gross
      error_message (mod->getid (), processid, "process instance",
		     procinst->getid (), 7, expr);
      arraygroesse= 1;
    }
    procinst->setArrayval (arraygroesse);  // dies hier ist die einzige
      // Stelle, in der der direkte Wert der Arraygroesse eingetragen wird.
  }
}


void SyntaxCheckerT::checkProcInst (Module *module, ProcessTypeDecl *process,
				    ProcessInst *procinst) {
// Wenn process == NULL, dann ist die ProzessInstanz auf hoechster Ebene
//  definiert und darf somit auch measparams uebergeben.
  ProcessTypeDecl *proctd;
  char *processid= NULL;
  if (process)
    processid= process->getid ();

  proctd= searchProcessTypeDecl (module, procinst->getProcessType ());
                                 // passenden ProcessType heraussuchen
  if (!proctd || proctd->getType () == DECL) {
    error_unknown (module->getid (), processid, "process instance",
		   procinst->getid (), procinst->getProcessType (), NULL);
    return;  // wenn dieser nicht als Decl. existiert, Fehlermeldung
  }

  int i= 0;
  char *expr= procinst->getParameterExpression (i);
  FormalParameter *forpar= proctd->getFormalParameter (i);
  // expr ist der i-te uebergebene Ausdruck (actual parameter) und forpar der
  //  entsprechende Formale Parameter (Id und Typ).
  while (expr || forpar) {
    if (!expr)  // Formaler Parameter nicht durch aktuellen Parameter belegt
      error_message (module->getid (), processid, "process instance",
		     procinst->getid (), 5, NULL);
    else if (!forpar)  // zu viele aktuelle Parameter uebergeben
      error_message (module->getid (), processid, "process instance",
		     procinst->getid (), 4, expr);
    else {
      if (process && (forpar->getFormalParameterType () == MEASPARAM))
	// measparam innerhalb eines Prozesses benutzt
	error_message (module->getid (), processid, "process instance",
		       procinst->getid (), 2, proctd->getid ());
      else if ((procinst->getArrayval () > 0)
	  && (forpar->getFormalParameterType () == MEASPARAM))
	// measparam innerhalb eines Arrays benutzt
	error_message (module->getid (), processid, "process instance",
		       procinst->getid (), 3, proctd->getid ());
      else {
	verifyExpression (module, process, expr,
			  forpar->getFormalParameterType (),
			  "actual parameter expression of process instance",
			  procinst->getid ());
	if ((forpar->getFormalParameterType () == MEASPARAM)
	    && ((strchr (expr+1, '$') != expr + strlen (expr)-1)
		|| (expr[0] != '$')))
	  // measparam darf nur einen identifier uebergeben bekommen
	  error_message (module->getid (), processid, "process instance",
			 procinst->getid (), 11, expr);

	expr= procinst->getParameterGuessExpression (i);
	if (expr && (forpar->getFormalParameterType () != MEASPARAM))
	  // guess-expression nur fuer measparams erlaubt
	  error_message (module->getid (), processid, "process instance",
			 procinst->getid (), 0, expr);
	else if (!expr && (forpar->getFormalParameterType () == MEASPARAM))
	  // guess-expression fuer measparams Pflicht
	  error_message (module->getid (), processid, "process instance",
			 procinst->getid (), 1,
			 procinst->getParameterExpression (i));
	else  // guess-expression ist reine Parameter-expression
	  verifyExpression (module, process, expr, PARAM,
			    "parameter guess expression of process instance",
			    procinst->getid ());
      }
    }
    expr= procinst->getParameterExpression (++i);
    forpar= proctd->getFormalParameter (i);
  }
}


char SyntaxCheckerT::getArcSourceDest (Module *module,
			   ProcessTypeDecl *process, Arc *arc, ArcType atype,
			   char *source, char *dest, char *errorid) {
  // Liefert je nach uebergebenem source/dest den Typ der Source bzw. der
  //  Destination des Arcs zurueck.
  ProcessTypeDecl *proctd= NULL;
  char *ident= NULL;
  char *procname= NULL;
  Port *portused= NULL;
  char returnval= 0;

  char *token= dest;
  if (source)
    token= source;

  if (process->searchTransition (token))
    return ('t');  // einfacher Fall: Arc zeigt auf/von Transition hin/weg
  else if (process->searchPlace (token))
    return ('p');  // Arc zeigt auf/von Platz hin/weg
  else if (!process->searchPort (token)) {  // sonst muss es ein Port sein
    error_unknown (module->getid (), process->getid (),
		   errorid, arc->getid (), token, NULL);
    return (0);
  }

  // token ist der Name eines Ports (eines eigenen oder eines fremden Ports)
  ident= lastsubstring (token);
  if (ident) {  // Port in einem fremden Prozess deklariert (Proc.Id)
    procname= substring2point (token);
    proctd= searchProcess (module, process, procname);
    if (!proctd)  // Passender Prozess muss existieren
      error_unknown (module->getid (), process->getid (), 
		     errorid, arc->getid (), procname, NULL);
    else {
      portused= proctd->searchPort (ident);  // Dort Port heraussuchen
      if (!portused || (portused->getPortType () == USEDPORT))
	// der Port muss dort public deklariert sein
	error_unknown (module->getid (), process->getid (),
		       errorid, arc->getid (), ident, proctd->getid ());
      else
	if (source)  // Source-Typen je nach Portdeklaration festlegen
	  if (((portused->getPortType () == PoutT)      && (atype == IOARC))
	   || ((portused->getPortType () == PinhiboutT) && (atype == HARC)))
	    returnval= 'p';
	  else if ((portused->getPortType () == ToutP)  && (atype == IOARC))
	    returnval= 't';
	  else
	    error_unknown (module->getid (), process->getid (),
			   errorid, arc->getid (), ident, proctd->getid ());
	else  // Destination-Typen je nach Portdeklaration festlegen
	  if (((portused->getPortType () == TinP)      && (atype == IOARC))
	   || ((portused->getPortType () == TinhibinP) && (atype == HARC)))
	    returnval= 't';
	  else if ((portused->getPortType () == PinT)  && (atype == IOARC))
	    returnval= 'p';
	  else
	    error_unknown (module->getid (), process->getid (),
			   errorid, arc->getid (), ident, proctd->getid ());
    }
    delete (ident);
    delete (procname);
  }
  else {  // Port im eigenen Prozess deklariert
//cout << "\nTeste "<<module->getid ()<<":"<<process->getid ()<<" "<<errorid<<" "<< arc->getid ()<<": id "<<flush<<token<<" of "<<flush<<process->getid ()<<" evtl defekt";
    portused= process->searchPort (token);
    if (!portused || (portused->getPortType () == USEDPORT))
      // der Port muss public deklariert sein
      error_unknown (module->getid (), process->getid (),
		     errorid, arc->getid (), token, process->getid ());
    else
      if (source)  // Source-Typen je nach Portdeklaration festlegen
	if (((portused->getPortType () == TinP)      && (atype == IOARC))
	    || ((portused->getPortType () == TinhibinP) && (atype == HARC)))
	  returnval= 'p';
	else if ((portused->getPortType () == PinT)  && (atype == IOARC))
	  returnval= 't';
	else
	  error_unknown (module->getid (), process->getid (),
			 errorid, arc->getid (), token, process->getid ());
      else  // Destination-Typen je nach Portdeklaration festlegen
	if (((portused->getPortType () == PoutT)      && (atype == IOARC))
	    || ((portused->getPortType () == PinhiboutT) && (atype == HARC)))
	  returnval= 't';
	else if ((portused->getPortType () == ToutP)  && (atype == IOARC))
	  returnval= 'p';
	else
	  error_unknown (module->getid (), process->getid (),
			 errorid, arc->getid (), token, process->getid ());
  }
  return (returnval);
}


void SyntaxCheckerT::checkGlobalNet () {
// Ueberprueft, ob alle in allen definierten Ausdruecken benutzten Variablen
//  auch immer zugaenglich sind.
//  Ebenfalls abchecken, ob auch Importbedingungen eingehalten wurden.

  int modulindex= 0, procindex= 0, index= 0;
  Module          *checkmodule=   NULL;
  ProcessTypeDecl *checkprocess=  NULL;
  Port            *checkport=     NULL;
  Reward          *checkreward=   NULL;
  Measure         *checkmeasure=  NULL;
  Parameter       *checkparam=    NULL;
  ProcessInst     *checkprocinst= NULL;
  ProcessUsage    *checkprocuse=  NULL;
  Place           *checkplace=    NULL;
  Transition      *checktrans=    NULL;
  Arc             *checkarc=      NULL;

  TransitionType ttype;
  char *prio;
  DistributionType dist;
  char *distexpr;
  char *weight;
  MarkingDependencyType markdep;
  PolicyType policy;
  ConcurrencyDegree concurrency;
  char *guard;
  ArcType atype;
  char *from;
  char *to;
  char *mult;
  ProcessTypeDecl *proctd;
  char *ident;
  char *procname;
  Port *portused;
  char sourceT, destT;

  // Zuerst alle ProzessInstanzen durchgehen und deren Arraygroessen eintragen
  //  damit nachher auch moduluebergreifende Aufrufe der ProzessInstanzen
  //  untereinander auch hinsichtlich ihrer Arraygroessen-Korrektheit geprueft
  //  werdeb koennen.
  for (modulindex= 0; modulindex < modules->size(); modulindex++) {
    checkmodule= modules->readElementAt (modulindex);
    // zuerst die Arraygroesse aller globalen Prozessinstanzen eintragen.
    index= 0;  // globale Prozessinstanzen
    checkprocinst= checkmodule->getProcessInst (index);
    while (checkprocinst) {
      setProcInstArrayval (checkmodule, NULL, checkprocinst);
      checkprocinst= checkmodule->getProcessInst (++index);
    }
  }

  for (modulindex= 0; modulindex < modules->size(); modulindex++) {
    checkmodule= modules->readElementAt (modulindex);// alle Module durchgehen

    index= 0;  // globale Parameter
    checkparam= checkmodule->getParameter (index);
    while (checkparam) {
      verifyExpression (checkmodule, NULL,
			checkparam->getParameterExpression (), PARAM,
			"parameter", checkparam->getid ());
      checkparam= checkmodule->getParameter (++index);
    } // while (checkparam)

    index= 0;  // globale Prozessinstanzen
    checkprocinst= checkmodule->getProcessInst (index);
    while (checkprocinst) {
      checkProcInst (checkmodule, NULL, checkprocinst);
      // Jetzt nur noch die ProzessInstanzen checken, Arraygroesse wurde hier
      //  bereits eingetragen.
      checkprocinst= checkmodule->getProcessInst (++index);
    }

    procindex= 0;
    checkprocess= checkmodule->getProcess (procindex);
    while (checkprocess) {  // in allen Modulen alle Prozess[typen] durchgehen

      index= 0;
      checkprocinst= checkprocess->getProcessInst (index);
      while (checkprocinst) {
	setProcInstArrayval (checkmodule, checkprocess, checkprocinst);
	checkProcInst (checkmodule, checkprocess, checkprocinst);
	checkprocinst= checkprocess->getProcessInst (++index);
      }

      index= 0;
      checkprocuse= checkprocess->getProcessUsage (index);
      while (checkprocuse) {
	proctd= searchProcessTypeDecl (checkmodule, checkprocuse->
		      getProcName ());  // passenden ProcessType heraussuchen
	// nur ueberpruefen, ob es diesen Prozess auch gibt
	if (!proctd || proctd->getType () == TYPE)
	  error_unknown (checkmodule->getid (), checkprocess->getid (), 
			 "process usage", checkprocuse->getid (),
			 checkprocuse->getProcName (), NULL);
	checkprocuse= checkprocess->getProcessUsage (++index);
      }

      index= 0;
      checkreward= checkprocess->getReward (index);
      while (checkreward) {
	if (checkreward->getType () == 'r')
	  verifyExpression (checkmodule, checkprocess,
			    checkreward->getRewardExpression (), RREWARD,
			    "rreward", checkreward->getid ());
	else if (checkreward->getType () == 'i')
	  verifyExpression (checkmodule, checkprocess,
			    checkreward->getRewardExpression (), IREWARD,
			    "ireward", checkreward->getid ());
	else
	  verifyExpression (checkmodule, checkprocess,
			    checkreward->getRewardExpression (), CREWARD,
			    "ireward", checkreward->getid ());
        checkreward= checkprocess->getReward (++index);
      }

      index= 0;
      checkmeasure= checkprocess->getMeasure (index);
      while (checkmeasure) {
	if (checkmeasure->getType () == 's')
	  verifyExpression (checkmodule, checkprocess,
			    checkmeasure->getMeasureExpression (), SMEASURE,
			    "smeasure", checkmeasure->getid ());
	else
	  verifyExpression (checkmodule, checkprocess,
			    checkmeasure->getMeasureExpression (), MMEASURE,
			    "mmeasure", checkmeasure->getid ());
	checkmeasure= checkprocess->getMeasure (++index);
      }

      index= 0;
      checkparam= checkprocess->getParameter (index);
      while (checkparam) {
	verifyExpression (checkmodule, checkprocess,
			  checkparam->getParameterExpression (), PARAM,
			  "parameter", checkparam->getid ());
	checkparam= checkprocess->getParameter (++index);
      }

      index= 0;
      checkport= checkprocess->getPort (index);
      while (checkport) {
	if (checkport->getPortType () == USEDPORT) {
	  ident= lastsubstring (checkport->getid ());
	  if (ident) {
	    procname= substring2point (checkport->getid ());
	    proctd= searchProcess (checkmodule, checkprocess, procname);
	    if (!proctd)  // Der Prozess, in dem der Port angeblich deklariert
	                  //  wurde, existiert nicht
	      error_unknown (checkmodule->getid (), checkprocess->getid (), 
			     "port", checkport->getid (), procname, NULL);
	    else {
	      portused= proctd->searchPort (ident);
	      if (!portused || portused->getPortType () == USEDPORT)
		// der Port ist in dem Prozess falsch oder garnicht deklariert
		error_unknown (checkmodule->getid (), checkprocess->getid (),
		       "port", checkport->getid (), ident, proctd->getid ());
	    }
	    delete (ident);
	    ident= NULL;
	    delete (procname);
	  }
	  else  // Wenn der Port ein USEDPORT ist, muss er aus einem anderen
	        //  Prozess stammen. Dieser muss dem Identifier durch einen
	        //  Punkt getrennt vorangestellt werden.
	    error_unknown (checkmodule->getid(), checkprocess->getid(),"port",
			   checkport->getid (), checkport->getid (), NULL);
	}
	else  // eigener Port (im eigenen Prozess)
	  verifyExpression (checkmodule, checkprocess,
			    checkport->getMultiplicity (), PARAM,
			    "multiplicity of port", checkport->getid ());
	checkport= checkprocess->getPort (++index);
      }

      index= 0;
      checkplace= checkprocess->getPlace (index);
      while (checkplace) {
	verifyExpression (checkmodule, checkprocess,
			  checkplace->getMarking (), PARAM,
			  "marking of place", checkplace->getid ());
	checkplace= checkprocess->getPlace (++index);
      }

      index= 0;
      checktrans= checkprocess->getTransition (index);
      while (checktrans) {
	checktrans->getAttributes (&ttype, &prio, &dist, &distexpr, &weight,
				   &markdep, &policy, &concurrency, &guard);
	verifyExpression (checkmodule, checkprocess, prio, RREWARD,
			  "priority of transition", checktrans->getid ());
	verifyExpression (checkmodule, checkprocess, distexpr, RREWARD,
			  "distribution of transition", checktrans->getid ());
	verifyExpression (checkmodule, checkprocess, weight, RREWARD,
			  "weight of transition", checktrans->getid ());
	verifyExpression (checkmodule, checkprocess, guard, RREWARD,
			  "guard of transition", checktrans->getid ());
	checktrans= checkprocess->getTransition (++index);
      }

      index= 0;
      checkarc= checkprocess->getArc (index);
      while (checkarc) {
	checkarc->getAttributes (&atype, &from, &to, &mult);
	sourceT= getArcSourceDest (checkmodule, checkprocess, checkarc,
				   atype, from, NULL, "source of arc");
	destT= getArcSourceDest (checkmodule, checkprocess, checkarc,
				 atype, NULL, to, "destination of arc");
	if (sourceT && destT)
	  if (!(((sourceT != destT) && (atype == IOARC))
		|| ((sourceT == 'p') && (destT == 't'))))
	    // Normale Kanten duerfen von Platzen zu Transitionen (oder
	    //  umgekehrt) zeigen, Hemmende Kanten hingegen nur von Platzen
	    //  zu Transitionen.
	    error_unknown (checkmodule->getid (), checkprocess->getid (),
	      "source or destination of arc", checkarc->getid (), NULL, NULL);
	verifyExpression (checkmodule, checkprocess, mult, RREWARD,
			  "multiplicity of arc", checkarc->getid ());
	checkarc= checkprocess->getArc (++index);
      }

      if (dots)
	cout << '.' << flush;

      checkprocess= checkmodule->getProcess (++procindex);
    } // while (checkprocess)

  } // modulindex-schleife
}


void SyntaxCheckerT::displayConflictSectionProcInst (int errorsectionnr) {
  Module *module;
  int index;
  ProcessInst *procinst;
  for (int modindex2= 0; modindex2 < modules->size (); modindex2++) {
    module= modules->readElementAt (modindex2);
    index= 0;
    procinst= module->getProcessInst (index);
    while (procinst) {
      if (errorsectionnr == procinst->section)
	cout << procinst->getmodid () << ':' << procinst->getid ();
      procinst= module->getProcessInst (++index);
    }
  }
}


void SyntaxCheckerT::entryProcInstUsages (ProcessTypeDecl *process,
					  Vektor <String> *ProcNameList) {
  ProcessInst     *procinst= NULL;
  ProcessUsage    *procuse=  NULL;
  String *stringpt;
  int index= 0;  // procinsts von process durchgehen und eintragen
  procinst= process->getProcessInst (index);
  while (procinst) {
    if (!ProcNameList->searchId (procinst->getProcessType ())) {
      stringpt= new String (procinst->getProcessType ());
      ProcNameList->addElement (stringpt);
    }
    procinst= process->getProcessInst (++index);
  }
  index= 0;  // procusages von checkprocess durchgehen und eintragen
  procuse= process->getProcessUsage (index);
  while (procuse) {
    if (!ProcNameList->searchId (procuse->getProcName ())) {
      stringpt= new String (procuse->getProcName ());
      ProcNameList->addElement (stringpt);
    }
    procuse= process->getProcessUsage (++index);
  }
}


/***************** iterative Bedingungen checken *****************/

void SyntaxCheckerT::checkIterativConnections () {
// Fuer jede Prozessinstanz, die global existiert und auf Prozesstypen mit
//  mit measparams zugreift, muessen alle Prozesse dieser Prozesstypen und
//  Prozesse und weiterer benutzter Prozesstypen und Prozesse, die 'geused'
//  werden, mit derselben Section-Nummer (sectionct) ausgestattet werden.
// Diese Section-Nummern muessen vorher 0 sein, sonst sind diese Arten eines
//  rekursiven Aufrufs nicht erlaubt.
// Desweiteren Section-Nummer fuer jede dieser Prozessinstanzen und auch
//  Prozesse eintragen, diese werden dann beim Compilieren an alle Parameter,
//  Rewards, Transitionen, usw. mitkopiert.

  int modulindex= 0, procindex= 0;
  Module          *checkmodule=   NULL;
  ProcessInst     *checkprocinst= NULL;
  ProcessTypeDecl *checkprocess=  NULL;
//  Module          *module=   NULL;
//  ProcessInst     *procinst= NULL;
//  ProcessUsage    *procuse=  NULL;
  ProcessTypeDecl *process=  NULL;
  Vektor <String> ProcNameList;
  String *name;
  String *stringpt;
  sectionct= 0;  // 0 ist der initiale Wert fuer nicht-measparam-objekte
  bool ersterprozess= true;

  for (modulindex= 0; modulindex < modules->size(); modulindex++) {
    checkmodule= modules->readElementAt (modulindex);// alle Module durchgehen
    procindex= 0;
    checkprocinst= checkmodule->getProcessInst (procindex);
    while (checkprocinst) {
      // measparams duerfen nur in global deklarierten ProzessInstanzen stehen
      stringpt= new String (checkprocinst->getProcessType ());
      ProcNameList.addElement (stringpt);
      // den entsprechenden ProzessTypen in die Liste einfuegen
      ersterprozess= true;  // ProzessTyp ist 'Wurzel' der Section

      while (ProcNameList.size () > 0) {  // Liste abarbeiten
	name= ProcNameList.removeElementAt (0);
	process= searchProcessTypeDecl (NULL, name->getid ());
	// es wird davon ausgegangen, dass dieser Prozess existiert (wurde
	//  vorher gecheckt).
	if (!ersterprozess || process->containingMeasparams ()) {
	  if (ersterprozess) {
	    ersterprozess= false;
	    checkprocinst->section= ++sectionct;
	    // neue Sectionnummer fuer diesen Wurzel-ProzessTypen vergeben
	  }
	  if (process->section && (process->section != sectionct)) {
	    // Dieser Prozess gehoert bereits einer Section an => Fehler
	    cout << "\nmodule " << process->getmodid ();
	    cout << ", process " << process->getid () << ": conflicting ";
	    cout << "iterative usage by " << checkprocinst->getmodid ();
	    cout << ':' <<  checkprocinst->getid () << " and ";
	    displayConflictSectionProcInst (process->section);
	    errorfree= false;
/*
	    for (modindex2= 0; modindex2 < modules->size (); modindex2++) {
	      module= modules->readElementAt (modindex2);
	      index= 0;
	      procinst= module->getProcessInst (index);
	      while (procinst) {
		if ((process->section == procinst->section)
		    && (checkprocinst != procinst))
		  cout << procinst->getmodid () << ':' << procinst->getid ();
		procinst= module->getProcessInst (++index);
	      }
	    }
*/
	  }
	  else {
	    if (process->getType () == DECL)  // Prozess mit section versehen
	      process->section= sectionct;
	    entryProcInstUsages (process, &ProcNameList);
/*
	    index= 0;  // procinsts von process durchgehen
	    procinst= process->getProcessInst (index);
	    while (procinst) {
              stringpt= new String (procinst->getProcessType ());
	      ProcNameList.addElement (stringpt);	      
	      procinst= process->getProcessInst (++index);
	    }
	    index= 0;  // procusages von checkprocess durchgehen
	    procuse= process->getProcessUsage (index);
	    while (procuse) {
              stringpt= new String (procuse->getProcName ());
	      ProcNameList.addElement (stringpt);	      
	      procuse= process->getProcessUsage (++index);
	    }
*/
	  } // if-else
	} // if (durchschachteln)
	delete (name);
      } // while (Listsize > 0)
      if (dots)
	cout << '.' << flush;
      checkprocinst= checkmodule->getProcessInst (++procindex);
    } // while (checkprocinst)
  } // for (modulindex)

  while (ProcNameList.size () > 0)
    delete (ProcNameList.removeElementAt (0));

  // Jetzt nochmals alle Module durchgehen und fuer alle Prozesse und globale
  //  Prozessinstanzen schauen, ob sie keiner section angehoeren. Wenn dies
  //  der Fall ist (section == 0), so duerfen alle benutzten Prozesse eben-
  //  falls noch keiner Section angehoeren, sonst gibt es eine Fehlermeldung.
  for (modulindex= 0; modulindex < modules->size(); modulindex++) {
    checkmodule= modules->readElementAt (modulindex);// alle Module durchgehen
    procindex= 0;
    checkprocinst= checkmodule->getProcessInst (procindex);
    while (checkprocinst) {
      if (!checkprocinst->section) {  // section == 0
	stringpt= new String (checkprocinst->getProcessType ());
	ProcNameList.addElement (stringpt);
      }
      checkprocinst= checkmodule->getProcessInst (++procindex);
    } // while (checkprocinst)
    procindex= 0;
    checkprocess= checkmodule->getProcess (procindex);
    while (checkprocess) {
      if ((checkprocess->getType () == DECL) && (!checkprocess->section)) {
	stringpt= new String (checkprocess->getid ());
	ProcNameList.addElement (stringpt);
      }
      checkprocess= checkmodule->getProcess (++procindex);
    } // while (checkprocess)
    if (dots)
      cout << '.' << flush;
  } // for (modulindex)

  // Jetzt die Liste abarbeiten und weitere Prozesse (-typen) nur eintragen,
  //  wenn noch nicht vorhanden und section==0.
  for (procindex= 0; procindex < ProcNameList.size(); procindex++) {
    name= ProcNameList.readElementAt (procindex);
    process= searchProcessTypeDecl (NULL, name->getid ());
    if (process->section) {  // darf keine section enthalten
      cout << "\nmodule " << process->getmodid () << ", process ";
      cout << process->getid () << ": conflicting iterative usage by ";
      displayConflictSectionProcInst (process->section);
      cout << " and non-iterative section";
      errorfree= false;
/*
      for (modindex2= 0; modindex2 < modules->size (); modindex2++) {
	module= modules->readElementAt (modindex2);
	index= 0;
	procinst= module->getProcessInst (index);
	while (procinst) {
	  if (process->section == procinst->section)
	    cout << procinst->getmodid () << ':' << procinst->getid ();
	  procinst= module->getProcessInst (++index);
	}
      }
*/
    }
    else  // nachfolgende Prozesse einbinden
      entryProcInstUsages (process, &ProcNameList);
/*
      index= 0;  // procinsts von process durchgehen:
      procinst= process->getProcessInst (index);
      while (procinst) {
	if (!ProcNameList.searchId (procinst->getProcessType ())) {
	  stringpt= new String (procinst->getProcessType ());
	  ProcNameList.addElement (stringpt);
	}
	procinst= process->getProcessInst (++index);
      }
      index= 0;  // procusages von checkprocess durchgehen
      procuse= process->getProcessUsage (index);
      while (procuse) {
	if (!ProcNameList.searchId (procuse->getProcName ())) {
	  stringpt= new String (procuse->getProcName ());
	  ProcNameList.addElement (stringpt);
	}
	procuse= process->getProcessUsage (++index);
      }
*/
    if (dots)
      cout << '.' << flush;
  }
  while (ProcNameList.size () > 0)
    delete (ProcNameList.removeElementAt (0));
}


/*********************** Hilfsfunktionen ***********************/


bool SyntaxCheckerT::checkImportOk (Module *modpt, char *modimport) {
  if (!modpt || !modimport)
    return (false);
  if (!strcmp (modpt->getid (), modimport))
    return (true);  // im eigenen Module deklariert
  int i= 0;
  char *modorigimport= modpt->getImport (i);  // Alle Importe durchgehen
  while (modorigimport)
    if (!strcmp (modorigimport, modimport))  // und Uebereinstimmung suchen
      return (true);
    else
      modorigimport= modpt->getImport (++i);
  return (false);
}


Parameter *SyntaxCheckerT::searchParameter (Module *mod,
				 ProcessTypeDecl *proc, char *id) {
  // sucht Parameter id erst lokal (wenn moeglich), dann global
  Parameter *parameter= NULL;
  if (proc) {
    parameter= proc->searchParameter (id);
    if (parameter)
      return (parameter);
  }  // wenn lokal nicht gefunden, global in allen Modulen schauen
  for (int modulindex= 0; modulindex < modules->size(); modulindex++) {
    parameter= (modules->readElementAt (modulindex))->searchParameter (id);
    if (parameter && (!mod || (checkImportOk (mod, parameter->getmodid ()))))
      return (parameter);
  }
  return (NULL);
}


ProcessTypeDecl *SyntaxCheckerT::searchProcess (Module *mod,
				       ProcessTypeDecl *proc, char *id) {
/*
cout << "\n suche mod: "<<mod->getid();
if (proc) cout << ", proc: " << proc->getid();
else cout << ", global";
cout << ", id: " << id;
*/
  // sucht den zu der mit dem neuen Identifier id versehenden ProzessInstanz
  //  oder Prozessusage gehoerigen Prozess heraus.
  char *name= NULL;
  ProcessInst *proci= NULL;
  ProcessUsage *procu= NULL;
  if (proc)
    procu= proc->searchProcUsage (id);
  if (!procu)
    proci= searchProcessInst (mod, proc, id);
  // In procu bzw. proci steht nun die gefundene Deklaration der Prozess-
  //  Instanz bzw. der ProzessUsage
/*
cout << "\n   found ";
if (proci) cout << "proci: "<<proci->getid();
else if (procu) cout << "procu: "<<procu->getid();
else cout << "nothing";
*/
  if (proci)
    name= proci->getProcessType ();  // echten Prozessnamen auslesen
  else if (procu)
    name= procu->getProcName ();
  else
    return (NULL);
  return (searchProcessTypeDecl (mod, name));  // nach Prozess suchen
}


/*********************** public functions ***********************/

ProcessInst *SyntaxCheckerT::searchProcessInst (Module *mod,
				   ProcessTypeDecl *proc, char *id) {
/*
cout << "\n  suche mod: "<<mod->getid();
if (proc) cout << ", proc: " << proc->getid();
else cout << ", global";
cout << ", nach procinst: " << id;
*/
  ProcessInst *proci= NULL;
  if (proc) {  // ProzessInstanz nur innerhalb des Prozesses suchen
    proci= proc->searchProcInst (id);
    if (proci)
      return (proci);
  }  // wenn in proc nichts gefunden wurde, wird nicht global gesucht
  else  // wenn proc==NUL => globale ProcessInst kann gemeint sein
    for (int modulindex= 0; modulindex < modules->size(); modulindex++) {
      proci= (modules->readElementAt (modulindex))->searchProcInst (id);
      if (proci && (!mod || checkImportOk (mod, proci->getmodid ())))
	return (proci);  // globale ProzessInstanz gefunden
    }
  return (NULL);
}


ProcessTypeDecl *SyntaxCheckerT::searchProcessTypeDecl (Module *mod,
							char *id) {
  ProcessTypeDecl *proc= NULL;
  Module *modulept;
  for (int modulindex= 0; modulindex < modules->size(); modulindex++) {
    modulept= modules->readElementAt (modulindex);
    int j= 0;
    proc= modulept->getProcess (j);  // Prozesse in allen Modulen durchgehen
    while (proc)
      if (!strcmp(proc->getid(), id)
	  && (!mod || checkImportOk (mod, proc->getmodid())))
	return (proc);
      else
	proc= modulept->getProcess (++j);
  }
  return (NULL);
}


int SyntaxCheckerT::getExpressionValue (Module *module, ProcessTypeDecl *proc,
				       char *typ, char *id, char *paramexpr) {
  if (!paramexpr)
    return (0);
  int groesseval= 0;
  char *groesse= NULL;
  char *procid= NULL;
  if (proc)
    procid= proc->getid ();
//****** noch sehr vereinfacht
  // es wird angenommen, in der Parameterexpression stehe ein Parameter:
  Parameter *param= searchParameter (module, proc, paramexpr);
  if (param)  // wenn dies der Fall ist, kann er ausgelsen werden
    groesse= param->getParameterExpression ();  // Annahme: groesse ist INT
  else // paramexpr war selber schon Zahl oder nicht def. Parameter
    groesse= paramexpr;  // Annahme: groesse ist INT
  sscanf (groesse, "%d", &groesseval);
  if (groesseval <= 0)
    if (!param)
      error_unknown (module->getid (), procid, typ, id, groesse, NULL);
    else
      error_unknown (module->getid (), procid, 
	 "(expression must only contain an integer) parameter",
	 param->getid (), groesse, NULL);
  return (groesseval);
}


int SyntaxCheckerT::getSectionCount () {
  return (sectionct);
}



/****************** main function: checkSyntax ******************/

bool SyntaxCheckerT::checkSyntax (Vektor <Module> *mod) {

  int i;

  errorfree= true;
  modules= mod;  // zu compilierende Module vermerken

  while (IdList.size () > 0)     // GlobalTable loeschen
    delete (IdList.removeElementAt (0));

  checkRecursiveImports ();
  if (dots)
    cout << ".recursive imports checked\n";
  checkUniqueNames ();
  if (dots)
    cout << ".unique names checked\n";
  if (!errorfree)
    return (false);

  // Module intern checken (Transition def., imm. Tr. keine dist, ...)
  for (i= 0; i < modules->size(); i++) {
    errorfree &= (modules->readElementAt (i))->checkIntern();
    if (dots)
      cout << '.' << flush;
  }
  if (dots)
    cout << "modules checked internally\n";

  // extern auch checken (arcs, processinst/usage, ...)
  checkGlobalNet ();
  if (dots)
    cout << "modules checked externally\n";

  if (errorfree)
    checkRecursiveCalls ();
  if (dots)
    cout << "recursive calls checked\n";

  if (errorfree)
    checkIterativConnections ();
  if (dots)
    cout << "iterative connections checked\n";

  return (errorfree);
}



#endif
