#ifndef SPNLCHECKH
#define SPNLCHECKH



class GlobalTable {
// Dient zur Speicherung aller in allen Modulen vorkommenden an Prozesse
//  und Prozesstypen, globaler ProzessInstanzen und Parametern vergebenen
//  Identifiern.
  private:
    char *id;
    char *idtype;
    char *moduleid;

  public:
    GlobalTable (char *id, char *idtype, char *moduleid);
    ~GlobalTable ();
    char *getid ();
    char *getidtype ();
    char *getmoduleid ();
};


class SyntaxCheckerT {
  private:
    bool errorfree;              // Korrektheit des Netzes
    int sectionct;               // Anzahl der verschiedenen Sections
    Vektor <Module> *modules;    // Liste aller zu compilierenden Module
    Vektor <GlobalTable> IdList; // Liste aller Ids (process[inst|type],param)

    void add2IdList (char *id, char *idtype, char *moduleid);
    // Hinzufuegen in die IdList (wenn nicht vorhanden) sonst error

    void error_unknown (char *mod, char *proc, char *typ,
			char *id, char *unknown, char *procname);
    // gibt eine Fehlermeldung der Art "Identifier unknown evtl. aus procname
    //  benutzt in typ id nicht bekannt oder vom falschen Typ."

    void error_message (char *mod, char *proc, char *typ,
			char *id, int errormess, char *errid);
    // gibt eine Fehlermeldung gemaess errormess aus (siehe spnlcheck.cc)

    void checkUniqueNames ();
    // Innerhalb aller Module schauen, ob alle Parameter, Prozesse,
    //  ProzessInstanzen und ProzessTypen eindeutige (unique) Namen haben.

    void entryImports (Vektor <String> *importList, Module *modulept);
    void checkRecursiveImports ();
    // Ueberprueft, ob es keine rekursiven Aufrufe innerhalb der Importe
    //  gibt.

    void entryProcs (Vektor <String> *procList, ProcessTypeDecl *process);
    void entryRewards (Vektor <String> *rewardList, Reward *reward);
    void entryMeasures (Vektor <String> *measureList, Measure *measure);
    void entryParameters (Vektor <String> *paramList, Parameter *param);
    void checkRecursiveCalls ();
    // Ueberprueft, ob es keine rekursiven Aufrufe innerhalb Prozess[typen]
    //  (und deren benutzten Prozesse und Prozessinstanzen), Parameter,
    //  Rewards oder Measures gibt.

    void verifyExpression (Module *module, ProcessTypeDecl *process,
			   char *checkexpr, FormalParameterType exprtype,
			   char *errortype, char *errorid);
    // Ueberprueft, ob der Ausdruck checkexpr gemaess exprtype korrekt
    //  definiert ist.
    //
    //    exprtype    | Kann folgende Elemente enthalten:
    // ===============|===================================================
    //       cp       | cp
    //       mp       | sm-id
    //       mm       | place, rr, mm, cp, mp
    //       sm       | place, trans, sm, cr, ir, rr, cp, mp
    //       rr       | place, rr, cp, mp
    //       ir       | trans, ir, cp, mp + innerhalb if () auch rr, place
    //       cr       | place, trans, rr, ir, cr, cp, mp
    //    port-mult.  | siehe cp
    //   place-mark.  | siehe cp
    //   trans-prio.  | siehe rr
    //   trans-dist.  | siehe rr
    //  trans-weight  | siehe rr
    //   trans-guard  | siehe rr
    //    arc-mult.   | siehe rr

    char getArcSourceDest (Module *module, ProcessTypeDecl *process, Arc *arc,
			   ArcType atype, char *source, char *dest,
			   char *errortype);
    // Sucht den Typ der Arc-source (bzw. destination) -d.h. immer nur source
    //  oder dest uebergeben- heraus und liefert eine 0 fuer Fehlerfall, 't'
    //  fuer Transition und 'p' fuer Place zurueck.

    void setProcInstArrayval (Module *module, ProcessTypeDecl *process,
			      ProcessInst *procinst);
    // Vergibt fuer die in den ProzessInstanzen evtl. verwendeten Arrays
    //  die Arraygroesse als Integer, die mit getExpressionValue ausgerechent
    //  wird und sodann mit getArrayval zur Verfuegung steht.

    void checkProcInst (Module *module, ProcessTypeDecl *process,
			ProcessInst *procinst);
    // Hilfsfunktion fuer checkGlobalNet, checkt Prozessinstanzen und deren
    //  aktuelle Parameter.

    void checkGlobalNet ();
    // Ueberprueft im gesamten Netz, ob alle in allen definierten Ausdruecken
    //  benutzten Variablen auch immer zugaenglich (importiert) und definiert
    //  sind.

    void displayConflictSectionProcInst (int errorsection);
    // Gibt alle ProzessInstanzen aus, die als section die Nummer errorsection
    //  eingetragen haben (Unterfunktion zu checkIterativConnections).

    void entryProcInstUsages (ProcessTypeDecl *process,
			      Vektor <String> *ProcNameList);
    // Traegt alle in dem Prozess process deklarierten ProzessInstanzen und
    //  -usages in die uebergebende ProcNameList -sofern dort nicht schon
    //  vorhanden- ein (Unterfunktion zu checkIterativConnections).

    void checkIterativConnections ();
    // Ueberprueft, ob alle die iterativen Aufrufe auch voneinander getrennt
    //  sind und nicht durch gemeinsame Prozesse miteinander verbunden sind.
    //  Vergibt ausserdem gleich Nummern fuer die einzelnen Netzteile
    //  (sections) an die globalen Prozessinstanzen und deren benutzte Pro-
    //  zesse. Speichert die Anzahl der verschiedenen Sections im sectionct;

    bool checkImportOk (Module *modorig, char *modimport);
    // Schaut nach, ob der Import des Moduls mit dem Namen modimport von dem
    //  Modul modorig aus erlaubt ist (ist es in der Importliste?).

    Parameter *searchParameter (Module *mod, ProcessTypeDecl *process,
				char *id);
    // Sucht innerhalb des Prozesses process den mit id uebereinstimmenden
    //  Parameter heraus. Wird dort keiner gefunden, wird in den globalen
    //  Parametern gesucht. Wenn der Parameter dort gefunden wird, werden,
    //  wenn ein Modul mod!=NULL uebergeben wurde, die Importbedingungen von
    //  diesem Modul aus ueberprueft und, wenn das entsprechende Modul in dem
    //  der Parameter gefunden wurde korrekt importtiert wurde, er zurueck-
    //  geliefert.

    ProcessTypeDecl *searchProcess (Module *mod, ProcessTypeDecl *process,
				    char *id);
    // Sucht innerhalb des Prozesses process zu der Prozessbenutzung bzw.
    //  -instanziierung von id den passenden ProcessTypeDecl heraus.
    //  Wird noch ein Module != NULL uebergeben, so werden auch die Import-
    //  bedingungen geprueft.

  public:
    SyntaxCheckerT ();
    ~SyntaxCheckerT ();

    bool checkSyntax (Vektor <Module> *mod);
    // Uebergabe: die zu checkenden SPNL-Module.
    //  Prueft alle Module auf Korrektheit in sich und untereinander.

    ProcessInst *searchProcessInst (Module *mod, ProcessTypeDecl *process,
				    char *id);
    // Sucht innerhalb des Prozesses process die mit id uebereinstimmende
    //  ProzessInstanz heraus. Ist kein Prozess angegeben, wird in den globa-
    //  len ProzessInstanzen gesucht. Wenn die ProzessInstanz dort gefunden
    //  wird, werden, wenn ein Modul mod uebergeben wurde, die Importbedin-
    //  gungen von diesem Modul aus ueberprueft und, wenn das entsprechende
    //  Modul in dem die ProzessInstanz gefunden wurde korrekt importiert
    //  wurde, diese zurueckgeliefert.

    ProcessTypeDecl *searchProcessTypeDecl (Module *mod, char *id);
    // Sucht den mit id uebereinstimmenden Prozess aus allen Modulen heraus
    //  und liefert ihn zurueck, wenn er von mod aus korrekt importiert wurde.
    //  Ist mod == NULL, so werden die Importbedingungen nicht ueberprueft.

    int getExpressionValue (Module *module, ProcessTypeDecl *process,
			    char *typ, char *id, char *paramexpr);
    // Liefert den Wert des Parameter-Ausdrucks paramexpr zurueck. Fuer eine
    //  sinnvolle Fehlermeldung werden noch entsprechendes Modul, Prozess
    //  sowie der Typ und Identifier des Objektes, dem dieser Ausdruck
    //  zugeordnet ist, uebergeben.
       //********* Expr. auswerten bisher nur rudimentaer

    int getSectionCount ();
    // Liefert die Anzahl der verschiedenen Sections.
};


#endif
