#ifndef SPNLMODULECC
#define SPNLMODULECC


#include <string.h>
#include <iostream.h>
#include <stdio.h>

#include "vektor.cc"
#include "spnltypes.cc"
#include "spnlprocess.cc"
#include "stdfun.cc"
#include "spnlmodule.h"
#include "foutput.h"


/**** class Module ****/

Module::Module (char *s) {
  id= newstring (s);
  errorfree= true;
}

Module::~Module () {
//cout << "vor Module\n";
  delete (id);
}

void Module::display () {
  int i;
  fnet << "\nmodule " << id << ';';
  for (i=0; i<ImportList.size(); i++) {
    fnet << "\nuse ";
    (ImportList.readElementAt (i))->display();
    fnet << ';';
  }
  for (i=0; i<ParameterList.size(); i++) {
    fnet << "\nglobal_parameter ";
    (ParameterList.readElementAt (i))->display();
  }
  for (i=0; i<ProcessInstList.size(); i++) {
    fnet << "\n";
    (ProcessInstList.readElementAt (i))->display();
  }
  for (i=0; i<ProcessList.size(); i++) {
    fnet << "\n";
    (ProcessList.readElementAt (i))->display();
  }
  fnet << "\n\nend " << id << ".\n";
}

char *Module::getid () {
  return (id);
}

bool Module::isErrorfree () {
  for (int i= 0; i<ProcessList.size(); i++)
    errorfree&= (ProcessList.readElementAt (i))->errorfree;
  return (errorfree);
}

void Module::error_cannotdeclare (char *idtype, char *idname) {
  cout << "\nmodule " << this->id << ": ";
  cout << "Cannot declare " << idtype << ' ' << idname << ". Identifier ";
  cout << idname << " already defined as ";
  if (ImportList.searchId (idname))
    cout << "import";
  else if (ParameterList.searchId (idname))
    cout << "parameter";
  else if (ProcessInstList.searchId (idname))
    cout << "process instance";
  else
    cout << "process[type]";
  errorfree= false;
}

void Module::error_notdefined (char *nichtdatyp, char *nichtdaid,
	     char hinzufuegkuerzel, char *hinzufuegtyp, char *hinzufuegid) {
  cout << "\nmodule " << this->id << ": ";
  cout << nichtdatyp << ' ' << nichtdaid << " not defined in module ";
  cout << this->id << ", cannot add ";
  if (hinzufuegkuerzel)
    cout << hinzufuegkuerzel;
  cout << hinzufuegtyp << ' ' << hinzufuegid;
  errorfree= false;
}

String *Module::addImport (char *importmodule) {
  String *s= ImportList.searchId (importmodule);
  if (!s) {
    s= new String (importmodule);
    ImportList.addElement (s);
    return (s);
  }
  else {
    error_cannotdeclare ("import module", importmodule);
    return (NULL);
  }
}

char *Module::getImport (int i) {
  if (ImportList.size () > i)
    return ((ImportList.readElementAt(i))->getid());
  else
    return (NULL);
}

Parameter *Module::addParameter (char *id, char *expr) {
  Parameter *p= ParameterList.searchId (id);
  if (!p) {
    p= new Parameter (this->id, NULL, id, 'c');
    errorfree&= p->setParameterExpression (expr);
    ParameterList.addElement (p);
    return (p);
  }
  else {
    error_cannotdeclare ("parameter", id);
    return (NULL);
  }
}

Parameter *Module::getParameter (int i) {
  if (ParameterList.size () > i)
    return (ParameterList.readElementAt(i));
  else
    return (NULL);
}

Parameter *Module::searchParameter (char *id) {
  return (ParameterList.searchId (id));
}

ProcessInst *Module::addProcessInst (char *id, char *array, char *proctype) {
  ProcessInst *p= ProcessInstList.searchId (id);
  if (!p) {
    p= new ProcessInst (this->id, NULL, id);
    errorfree&= p->setProcessType (proctype);
    if (array)
      errorfree&= p->setArrayanz (array);
    ProcessInstList.addElement (p);
    return (p);
  }
  else {
    error_cannotdeclare ("process instance", id);
    return (NULL);
  }
}

ProcessInst *Module::addProcessInstFormPar (char *processinst,
			char *paramexpr, char *guessexpr) {
  ProcessInst *p= ProcessInstList.searchId (processinst);
  if (p) {
    p->addParameterExpression (paramexpr, guessexpr);
    return (p);
  }
  else {
    error_notdefined ("process instance", processinst,
		      0, "parameter", paramexpr);
    return (NULL);
  }
}

ProcessInst *Module::getProcessInst (int i) {
  if (ProcessInstList.size () > i)
    return (ProcessInstList.readElementAt(i));
  else
    return (NULL);
}

ProcessInst *Module::searchProcInst (char *id) {
  return (ProcessInstList.searchId (id));
}

ProcessTypeDecl *Module::addProcess (char *id, ProcessTypeDeclType type) {
  ProcessTypeDecl *p= ProcessList.searchId (id);
  if (p) {
    if (type == DECL)
      error_cannotdeclare ("process", id);
    else
      error_cannotdeclare ("process type", id);
    return (NULL);
  }
  else {
    p= new ProcessTypeDecl (this->id, id);
    p->setType (type);
    ProcessList.addElement (p);
    return (p);
  }
}

ProcessTypeDecl *Module::getProcess (int i) {
  if (ProcessList.size () > i)
    return (ProcessList.readElementAt(i));
  else
    return (NULL);
}

ProcessTypeDecl *Module::addProcessFormPar (char *process, char *param,
					    char *paramtype) {
  ProcessTypeDecl *p= ProcessList.searchId (process);
  if ((p) && (p->getType() == TYPE)){
    p->addFormalParameter (param, paramtype);
    return (p);
  }
  else {
    error_notdefined ("process type", process, 0, "parameter", param);
    return (NULL);
  }
}

ProcessTypeDecl *Module::addProcessPort (char *process, char *port,
					 char *type, char *mult) {
  ProcessTypeDecl *p= ProcessList.searchId (process);
  if (p) {
    p->addPort (port, type, mult);
    return (p);
  } 
  else {
    error_notdefined ("process[type]", process, 0, "port", port);
    return (NULL);
  }
}

ProcessTypeDecl *Module::addProcessReward (char *process, char *reward,
					   char typ, VisibleType visible) {
  ProcessTypeDecl *p= ProcessList.searchId (process);
  if (p) {
    p->addReward (reward, typ, visible);
    return (p);
  } 
  else {
    error_notdefined ("process[type]", process, typ, "reward", reward);
    return (NULL);
  }
}

ProcessTypeDecl *Module::addProcessMeasure (char *process, char *measure,
					    char typ, VisibleType visible) {
  ProcessTypeDecl *p= ProcessList.searchId (process);
  if (p) {
    p->addMeasure (measure, typ, visible);
    return (p);
  } 
  else {
    error_notdefined ("process[type]", process, typ, "measure", measure);
    return (NULL);
  }
}

ProcessTypeDecl *Module::addProcessParameter (char *process, char *param,
					      char *expr) {
  ProcessTypeDecl *p= ProcessList.searchId (process);
  if (p) {
    p->addParameter (param, 'c', expr);   // es koennen nur const params sein
    return (p);
  } 
  else {
    error_notdefined ("process[type]", process, 0, "parameter", param);
    return (NULL);
  }
}

ProcessTypeDecl *Module::addProcessProcInst (char *process,
		     char *processinst, char *array, char *processtype) {
  ProcessTypeDecl *p= ProcessList.searchId (process);
  if (p) {
    p->addProcessInst (processinst, array, processtype);
    return (p);
  } 
  else {
    error_notdefined ("process[type]", process,
		      0, "process instance", processinst);
    return (NULL);
  }
}

ProcessTypeDecl *Module::addProcessProcInstFormPar (char *process,
		     char *processinst, char *paramexpr, char *guessexpr) {
  ProcessTypeDecl *p= ProcessList.searchId (process);
  if (p) {
    p->addProcessInstFormPar (processinst, paramexpr, guessexpr);
    return (p);
  }
  else {
    error_notdefined ("process[type]", process,
		      0, "parameters to process instance", processinst);
    return (NULL);
  }
}

ProcessTypeDecl *Module::addProcessProcUsage (char *process,
					      char *newid, char *useproc) {
  ProcessTypeDecl *p= ProcessList.searchId (process);
  if (p) {
    p->addProcessUsage (newid, useproc);
    return (p);
  } 
  else {
    error_notdefined ("process[type]", process, 0, "use process", newid);
    return (NULL);
  }
}

ProcessTypeDecl *Module::addProcessPlace (char *process,
				  char *place, char *marking) {
  ProcessTypeDecl *p= ProcessList.searchId (process);
  if (p) {
    p->addPlace (place, marking);
    return (p);
  } 
  else {
    error_notdefined ("process[type]", process, 0, "place", place);
    return (NULL);
  }
}

ProcessTypeDecl *Module::addProcessTrans (char *process,
				  char *trans, char *type) {
  ProcessTypeDecl *p= ProcessList.searchId (process);
  if (p) {
    p->addTransition (trans, type);
    return (p);
  } 
  else {
    error_notdefined ("process[type]", process, 0, "transition", trans);
    return (NULL);
  }
}

ProcessTypeDecl *Module::addProcessArc (char *process,
				char *arc, char *from, char *to,
				char *type, char *mult) {
  ProcessTypeDecl *p= ProcessList.searchId (process);
  if (p) {
    p->addArc (arc, from, to, type, mult);
    return (p);
  } 
  else {
    error_notdefined ("process[type]", process, 0, "arc", arc);
    return (NULL);
  }
}

ProcessTypeDecl *Module::addProcessTransAttr (char *process, char *trans, 
	                  char *prioexpr, char *distribution, char *distexpr,
			  char *weight, char *markdep, char *policy,
                          char *concurrency, char *guard) {
  ProcessTypeDecl *p= ProcessList.searchId (process);
  if (p) {
    p->addTransitionAttr (trans, prioexpr, distribution, distexpr, weight,
			  markdep, policy, concurrency, guard);
    return (p);
  }
  else {
    error_notdefined ("process[type]", process,
		      0, "attributes to transition", trans);
    return (NULL);
  }
}

ProcessTypeDecl *Module::addProcessRewardExpr (char *process,
					       char *reward, char *expr) {
  ProcessTypeDecl *p= ProcessList.searchId (process);
  if (p) {
    p->addRewardExpr (reward, expr);
    return (p);
  }
  else {
    error_notdefined ("process[type]", process,
		      0, "reward expression to reward", reward);
    return (NULL);
  }
}

ProcessTypeDecl *Module::addProcessMeasureExpr (char *process,
						char *measure, char *expr) {
  ProcessTypeDecl *p= ProcessList.searchId (process);
  if (p) {
    p->addMeasureExpr (measure, expr);
    return (p);
  }
  else {
    error_notdefined ("process[type]", process,
		      0, "measure expression to measure", measure);
    return (NULL);
  }
}

bool Module::checkIntern () {
// checkt zuerst die internen Werte, die erst nach dem Enitragen geckeckt
//  werden koennen.
// Geht dann alle Prozesstypen bzw. Deklarationen fuer sich durch und
//  anschliessend Prozesstypen bzw. Deklarationen in Bezug auf globale Werte
//  geht so aber nicht (Arc trifft auf Port, der woanders deklariert ist)
  int i;

  for (i= 0; i < ParameterList.size(); i++)
    if (!(ParameterList.readElementAt (i))->getParameterExpression ()) {
      cout << "\nmodule " << id << ": parameter "
	   << (ParameterList.readElementAt (i))->getid ()
	   << " declared but not defined";
      errorfree= false;
    }

  for (i= 0; i < ProcessInstList.size(); i++)
    if (!(ProcessInstList.readElementAt (i))->getProcessType ()) {
      cout << "\nmodule " << id << ": process instance "
	   << (ProcessInstList.readElementAt (i))->getid () << " has no type";
      errorfree= false;
    }

  for (i= 0; i < ProcessList.size(); i++)
    errorfree &= (ProcessList.readElementAt (i))->checkIntern ();

  return (errorfree);
}

#endif

