#ifndef SPNLPROCESSCC
#define SPNLPROCESSCC


#include <string.h>
#include <iostream.h>

#include "vektor.cc"
#include "stdfun.cc"
#include "spnltypes.cc"
#include "spnlbasics.cc"
#include "spnlprocess.h"
#include "foutput.h"


/**** class ProcessTypeDecl ****/

ProcessTypeDecl::ProcessTypeDecl (char *modident, char *s) {
  id= newstring (s);
  modid= modident;
  type= DECL;
  longid= NULL;
  oldid= NULL;
  errorfree= true;
  section= 0;
}

ProcessTypeDecl::~ProcessTypeDecl () {
// cout << "vor ProcessTypeDecl\n";
  delete (id);
  delete (longid);
  delete (oldid);
}

void ProcessTypeDecl::display () {
  int i;
  if (type == DECL)
    fnet << "\nprocess " << id;
  else {
    fnet << "\nprocesstype " << id;
    if (FormalParameterList.size() > 0) {
      fnet << " (";
      for (i=0; i<FormalParameterList.size()-1; i++) {
        (FormalParameterList.readElementAt (i))->display();
        fnet << ", ";
      }
      FormalParameterList.readElementAt
	(FormalParameterList.size()-1)->display();
      fnet << ')';
    }
  }
  fnet << ';';
  for (i=0; i<PortList.size(); i++) {
    fnet << "\nport ";
    (PortList.readElementAt (i))->display ();
  }
  for (i=0; i<ProcessInstList.size(); i++)
    (ProcessInstList.readElementAt (i))->display ();
  for (i=0; i<ProcessUsageList.size(); i++)
    (ProcessUsageList.readElementAt (i))->display ();
  
  for (i=0; i<ParameterList.size(); i++) {
    fnet << "\nparameter ";
    (ParameterList.readElementAt (i))->display ();
  }
  for (i=0; i<PlaceList.size(); i++) {
    fnet << "\nplace ";
    (PlaceList.readElementAt (i))->display ();
  }
  for (i=0; i<TransitionList.size(); i++) {
    fnet << "\ntrans ";
    (TransitionList.readElementAt (i))->display ();
  }
  fnet << "\ntransatt";
  for (i=0; i<TransitionList.size(); i++)
    (TransitionList.readElementAt (i))->displayattr (-1);
  for (i=0; i<ArcList.size(); i++) {
    fnet << "\narc ";
    (ArcList.readElementAt (i))->display ();
  }
  for (i=0; i<RewardList.size(); i++) {
    fnet << "\nreward ";
    (RewardList.readElementAt (i))->display ();
  }
  for (i=0; i<MeasureList.size(); i++) {
    fnet << "\nmeasure ";
    (MeasureList.readElementAt (i))->display ();
  }
  fnet << "\nend " << id << ';';
}

void ProcessTypeDecl::display_comp (int sectionanz) {
  int i= 0, sectionct= 0;
  fnet << "(* compiled SPNL *)";
//    for (i=0; i<ProcessUsageList.size(); i++)
//      (ProcessUsageList.readElementAt (i))->display_comp (sectionct);
  if (sectionanz)
    fnet << "\n(* iterative model consisting of " << sectionanz << " iterative sections *)";
  for (sectionct= 0; sectionct <= sectionanz; sectionct++) {
    fnet << "\n\n\nsection SEC" << sectionct << ':';
/*
    for (i=0; i<ProcessInstList.size(); i++)
      (ProcessInstList.readElementAt (i))->display_comp (sectionct);
    for (i=0; i<PortList.size(); i++)
      (PortList.readElementAt (i))->display_comp (sectionct);
*/
    if (ParameterTypeExistsInSection ('c', sectionct)) {
      fnet << "\n\n(* parameters *)\nparameter";
      for (i=0; i<ParameterList.size(); i++)
	(ParameterList.readElementAt (i))->display_param_comp (sectionct);
    }
    if (PlaceExistsInSection (sectionct)) {
      fnet << "\n\n(* places *)\nplace";
      for (i=0; i<PlaceList.size(); i++)
	(PlaceList.readElementAt (i))->display_comp (sectionct);
    }
    if (TransitionExistsInSection (sectionct)) {
      fnet << "\n\n(* transitions *)\ntrans";
      for (i=0; i<TransitionList.size(); i++)
	(TransitionList.readElementAt (i))->display_comp (sectionct);
    }
    if (TransattExistsInSection (sectionct)) {
      fnet << "\n\n(* transition attributes *)\ntransatt";
      for (i=0; i<TransitionList.size(); i++)
	(TransitionList.readElementAt (i))->displayattr (sectionct);
    }
    if (ArcExistsInSection (sectionct)) {
      fnet << "\n\n(* arcs *)\narc";
      for (i=0; i<ArcList.size(); i++)
	(ArcList.readElementAt (i))->display_comp (sectionct);
    }
    if (RewardExistsInSection (sectionct)) {
      fnet << "\n\n(* rewards *)\nreward";
      for (i=0; i<RewardList.size(); i++)
	(RewardList.readElementAt (i))->display_comp (sectionct);
    }
    if (MeasureExistsInSection (sectionct)) {
      fnet << "\n\n(* measures *)\nmeasure";
      for (i=0; i<MeasureList.size(); i++)
	(MeasureList.readElementAt (i))->display_comp (sectionct);
    }
    if (ParameterTypeExistsInSection ('m', sectionct)) {
      fnet << "\n\n(* measure parameters *)\nmeasparam";
      for (i=0; i<ParameterList.size(); i++)
	(ParameterList.readElementAt (i))->display_measparam_comp (sectionct);
    }
  }
  if (ParameterTypeExistsInSection ('m', -1)) {
    fnet << "\n\n\n(* iterative measparam connections *)\nconnection";
    Parameter *param= NULL, *param2= NULL;
    Reward *reward= NULL;
    Measure *measure= NULL;
    int j;
    for (i=0; i<ParameterList.size(); i++) {
      param= ParameterList.readElementAt (i);
      if (param->getType () == 'm') {
        measure= MeasureList.searchId (param->getParameterExpression());
        if (!measure)
	  reward= RewardList.searchId (param->getParameterExpression());
        if (!(reward || measure))
	  param2= ParameterList.searchId (param->getParameterExpression());
	if (!(reward || measure || param2))
	  cout << "\niterative connection error: "
	       << param->getParameterExpression() << " not found";
	else {
	  fnet << "\n " << param->getid () << ": SEC";
	  if (param2)
	    fnet << param2->section;
	  else if (measure)
	    fnet << measure->section;
	  else if (reward)
	    fnet << reward->section;
	  fnet << " -> SEC" << param->section << ';';
	}
	param2= NULL;
	measure= NULL;
	reward= NULL;
      }
    }
  }
}

char *ProcessTypeDecl::getid () {
  return (id);
}

char *ProcessTypeDecl::getmodid () {
  return (modid);
}

bool ProcessTypeDecl::add2IdList (char *id) {
  String *s= IdList.searchId (id);
  if (s)
    return (false);  // kann nicht hinzufuegen, da schon vorhanden
  else {
    s= new String (id);
    IdList.addElement (s);  
    return (true);
  }
}

bool ProcessTypeDecl::TransattExistsInSection (int section) {
  TransitionType type;
  char *prio;
  DistributionType dist;
  char *distexpr;
  char *weight;
  MarkingDependencyType markdep;
  PolicyType policy;
  ConcurrencyDegree conc;
  char *guard;
  int i;
  for (i=0; i<TransitionList.size(); i++) {
    (TransitionList.readElementAt (i))->getAttributes (&type, &prio, &dist,
		 &distexpr, &weight, &markdep, &policy, &conc, &guard);
    if (((TransitionList.readElementAt (i))->section == section)
	&& (prio || dist || weight || markdep || policy || conc || guard))
      return (true);
  }
  return (false);
}

bool ProcessTypeDecl::ParameterTypeExistsInSection (char type, int section) {
  for (int i=0; i<ParameterList.size(); i++)
    if (((ParameterList.readElementAt (i))->getType () == type)
	&& (((ParameterList.readElementAt (i))->section == section)
	|| (section < 0)))
      return (true);
  return (false);
}

bool ProcessTypeDecl::PlaceExistsInSection (int section) {
  for (int i=0; i<PlaceList.size(); i++)
    if ((PlaceList.readElementAt (i))->section == section)
      return (true);
  return (false);
}

bool ProcessTypeDecl::TransitionExistsInSection (int section) {
  for (int i=0; i<TransitionList.size(); i++)
    if ((TransitionList.readElementAt (i))->section == section)
      return (true);
  return (false);
}

bool ProcessTypeDecl::ArcExistsInSection (int section) {
  for (int i=0; i<ArcList.size(); i++)
    if ((ArcList.readElementAt (i))->section == section)
      return (true);
  return (false);
}

bool ProcessTypeDecl::RewardExistsInSection (int section) {
  for (int i=0; i<RewardList.size(); i++)
    if ((RewardList.readElementAt (i))->section == section)
      return (true);
  return (false);
}

bool ProcessTypeDecl::MeasureExistsInSection (int section) {
  for (int i=0; i<MeasureList.size(); i++)
    if ((MeasureList.readElementAt (i))->section == section)
      return (true);
  return (false);
}

void ProcessTypeDecl::setType (ProcessTypeDeclType processtype) {
  type= processtype;
}

ProcessTypeDeclType ProcessTypeDecl::getType () {
  return (type);
}


void ProcessTypeDecl::error_cannotdeclare (char typkuerzel, char *idtype,
					   char *idname) {
  cout << "\nmodule " << modid << ", process";
  if (type == TYPE)
    cout << "type";
  cout << ' ' << this->id << ": cannot declare ";
  if (typkuerzel)
    cout << typkuerzel;
  cout << idtype << ' ' << idname << ". Identifier ";
  cout << idname << " already defined as ";
  if (PortList.searchId (idname))
    cout << "port";
  else if (RewardList.searchId (idname))
    cout << "reward";
  else if (MeasureList.searchId (idname))
    cout << "measure";
  else if (ParameterList.searchId (idname))
    cout << "parameter";
  else if (ProcessInstList.searchId (idname))
    cout << "process instance";
  else if (ProcessUsageList.searchId (idname))
    cout << "process usage";
  else if (PlaceList.searchId (idname))
    cout << "place";
  else if (TransitionList.searchId (idname))
    cout << "transition";
  else if (FormalParameterList.searchId (idname))
    cout << "formal parameter";
  else
    cout << "arc";
  errorfree= false;
}


void ProcessTypeDecl::error_notdefined (char *nichtdatyp, char *nichtdaid,
			    char *hinzufuegtyp, char *hinzufuegid) {
  cout << "\nmodule " << modid << ", process";
  if (type == TYPE)
    cout << "type";
  cout << ' ' << this->id << ": ";
  cout << nichtdatyp << ' ' << nichtdaid << " not defined in process[type] ";
  cout << this->id << ", cannot add " << hinzufuegtyp << ' ';
  display_dollarexpr (hinzufuegid);  // Vorsichtshalber $-Zeichen entfernen
  errorfree= false;
}


void ProcessTypeDecl::error_message (char *typ, char *id,
				     char *mess1, char *mess2) {
  cout << "\nmodule " << modid << ", process";
  if (type == TYPE)
    cout << "type";
  cout << ' ' << this->id << ": " ;
  if (typ && id)
    cout << typ << ' ' << id << ' ';
  cout << mess1;
  if (mess2)
    cout << mess2;
  errorfree= false;
}


FormalParameter *ProcessTypeDecl::addFormalParameter (char *param,
						      char *paramtype) {
  if (add2IdList (param)) {
    FormalParameter *s= new FormalParameter (modid, this->id, param);
    s->setFormalParameterType (String2FormalParameterType (paramtype));
    FormalParameterList.addElement (s);
    return (s);
  }
  else {
    error_cannotdeclare (0, "formal parameter", param);
    return (NULL);
  }
}


FormalParameter *ProcessTypeDecl::getFormalParameter (int i) {
  if (FormalParameterList.size () > i)
    return (FormalParameterList.readElementAt(i));
  else
    return (NULL);
}


FormalParameter *ProcessTypeDecl::searchFormalParameter (char *id) {
  return (FormalParameterList.searchId (id));
}


bool ProcessTypeDecl::containingMeasparams () {
  bool measparam= false;
  for (int index= 0; index < FormalParameterList.size(); index++)
    measparam|= ((FormalParameterList.readElementAt (index))->
		 getFormalParameterType () == MEASPARAM);
  return (measparam);
}


Port *ProcessTypeDecl::addPort (char *port, char *type, char *mult) {
  if (add2IdList (port)) {
    Port *p= new Port (modid, this->id, port);
    if (type)
      p->setPortType (String2PortType (type));
    if (mult)
      errorfree&= p->setMultiplicity (mult);
    PortList.addElement (p);
    return (p);
  }
  else {
    Port *p= PortList.searchId (port);
    if (((p->getPortType() != USEDPORT)
	 && (String2PortType (type) != USEDPORT))
	|| ((p->getPortType() == USEDPORT)
	    && (String2PortType (type) == USEDPORT))) {
      // Nur Fehlermeldung, wenn Port nicht im Graphicalpart defniert wurde
      error_cannotdeclare (0, "port", port);
      return (NULL);
    }
  }
}


Port *ProcessTypeDecl::getPort (int i) {
  if (PortList.size () > i)
    return (PortList.readElementAt(i));
  else
    return (NULL);
}


Port *ProcessTypeDecl::searchPort (char *id) {
  return (PortList.searchId (id));
}


void ProcessTypeDecl::removePort (int i) {
  if (PortList.size () > i)
    delete (PortList.removeElementAt(i));
}


Reward *ProcessTypeDecl::addReward (char *reward, char typ,
				    VisibleType visible) {
  if (add2IdList (reward)) {
    Reward *r= new Reward (modid, this->id, reward, typ);
    r->setVisibility (visible);
    RewardList.addElement (r);
    return (r);
  }
  else {
    error_cannotdeclare (typ, "reward", reward);
    return (NULL);
  }
}


Reward *ProcessTypeDecl::getReward (int i) {
  if (RewardList.size () > i)
    return (RewardList.readElementAt(i));
  else
    return (NULL);
}


Reward *ProcessTypeDecl::searchReward (char *id) {
  return (RewardList.searchId (id));
}


Measure *ProcessTypeDecl::addMeasure (char *measure, char typ,
				      VisibleType visible) {
  if (add2IdList (measure)) {
    Measure *m= new Measure (modid, this->id, measure, typ);
    m->setVisibility (visible);
    MeasureList.addElement (m);
    return (m);
  }
  else {
    error_cannotdeclare (typ, "measure", measure);
    return (NULL);
  }
}


Measure *ProcessTypeDecl::getMeasure (int i) {
  if (MeasureList.size () > i)
    return (MeasureList.readElementAt(i));
  else
    return (NULL);
}


Measure *ProcessTypeDecl::searchMeasure (char *id) {
  return (MeasureList.searchId (id));
}


Parameter *ProcessTypeDecl::addParameter (char *param, char typ, char *expr) {
  if (add2IdList (param)) {
    Parameter *p= new Parameter (modid, this->id, param, typ);
    errorfree&= p->setParameterExpression (expr);
//cout << "\n***addParameter: param " << param << "= " << expr;
    ParameterList.addElement (p);
    return (p);
  }
  else {
    error_cannotdeclare (0, "parameter", param);
    return (NULL);
  }
}


Parameter *ProcessTypeDecl::getParameter (int i) {
  if (ParameterList.size () > i)
    return (ParameterList.readElementAt(i));
  else
    return (NULL);
}


Parameter *ProcessTypeDecl::searchParameter (char *id) {
  return (ParameterList.searchId (id));
}


ProcessInst *ProcessTypeDecl::addProcessInst (char *processinst, char *array,
					      char *processtype) {
  if (add2IdList (processinst)) {
    ProcessInst *p= new ProcessInst (modid, this->id, processinst);
    errorfree&= p->setProcessType (processtype);
    if (array)
      errorfree&= p->setArrayanz (array);
    ProcessInstList.addElement (p);
    return (p);
  }
  else {
    error_cannotdeclare (0, "process instance", processinst);
    return (NULL);
  }
}


ProcessInst *ProcessTypeDecl::getProcessInst (int i) {
  if (ProcessInstList.size () > i)
    return (ProcessInstList.readElementAt(i));
  else
    return (NULL);
}


ProcessInst *ProcessTypeDecl::searchProcInst (char *id) {
  return (ProcessInstList.searchId (id));
}


void ProcessTypeDecl::removeProcessInst (int i) {
  if (ProcessInstList.size () > i)
    delete (ProcessInstList.removeElementAt(i));
}


void ProcessTypeDecl::addProcessInstFormPar (char *processinst,
					     char *paramexpr,
					     char *guessexpr) {
  ProcessInst *p= ProcessInstList.searchId (processinst);
  if (p)
    p->addParameterExpression (paramexpr, guessexpr);
  else
    error_notdefined("process instance", processinst, "parameter", paramexpr);
}


ProcessUsage *ProcessTypeDecl::addProcessUsage (char *newid, char *procid) {
  if (add2IdList (newid)) {
    ProcessUsage *p= new ProcessUsage (modid, this->id, newid, procid);
    ProcessUsageList.addElement (p);
    return (p);
  }
  else {
    error_cannotdeclare (0, "process usage", newid);
    return (NULL);
  }
}


ProcessUsage *ProcessTypeDecl::getProcessUsage (int i) {
  if (ProcessUsageList.size () > i)
    return (ProcessUsageList.readElementAt(i));
  else
    return (NULL);
}


ProcessUsage *ProcessTypeDecl::searchProcUsage (char *id) {
  return (ProcessUsageList.searchId (id));
}


void ProcessTypeDecl::removeProcessUsage (int i) {
  if (ProcessUsageList.size () > i)
    delete (ProcessUsageList.removeElementAt(i));
}


Place *ProcessTypeDecl::addPlace (char *place, char *marking) {
  if (add2IdList (place)) {
    Place *p= new Place (modid, this->id, place);
    if (marking)
      errorfree&= p->setMarking(marking);
    PlaceList.addElement (p);
    return (p);
  }
  else {
    error_cannotdeclare (0, "place", place);
    return (NULL);
  }
}


Place *ProcessTypeDecl::getPlace (int i) {
  if (PlaceList.size () > i)
    return (PlaceList.readElementAt(i));
  else
    return (NULL);
}


Place *ProcessTypeDecl::searchPlace (char *id) {
  return (PlaceList.searchId (id));
}


Transition *ProcessTypeDecl::addTransition (char *trans, char *type) {
  if (add2IdList (trans)) {
    Transition *t= new Transition (modid, this->id, trans);
    if (type)
      t->setTransitionType (String2TransitionType (type));
    TransitionList.addElement (t);
    return (t);
  }
  else {
    error_cannotdeclare (0, "transition", trans);
    return (NULL);
  }
}


Transition *ProcessTypeDecl::getTransition (int i) {
  if (TransitionList.size () > i)
    return (TransitionList.readElementAt(i));
  else
    return (NULL);
}


Transition *ProcessTypeDecl::searchTransition (char *id) {
  return (TransitionList.searchId (id));
}


Arc *ProcessTypeDecl::addArc (char *arc, char *from, char *to,
			      char *type, char *mult) {
  if (add2IdList (arc)) {
    Arc *a= new Arc (modid, this->id, arc);
    errorfree&= a->setSource (from);
    errorfree&= a->setDestination (to);
    if (type)
      a->setArcType (String2ArcType (type));
    if (mult)
      errorfree&= a->setMultiplicity (mult);
    ArcList.addElement (a);
    return (a);
  }
  else {
    error_cannotdeclare (0, "arc", arc);
    return (NULL);
  }
}


Arc *ProcessTypeDecl::getArc (int i) {
  if (ArcList.size () > i)
    return (ArcList.readElementAt(i));
  else
    return (NULL);
}


Arc *ProcessTypeDecl::searchArc (char *id) {
  return (ArcList.searchId (id));
}


void ProcessTypeDecl::removeArc (char *arcid) {
  ArcList.deleteElement (arcid);
}


void ProcessTypeDecl::addTransitionAttr (char *trans, 
	                 char *prioexpr, char *distribution, char *distexpr,
		         char *weight, char *markdep, char *policy,
                         char *concurrency, char *guard) {
  Transition *t= TransitionList.searchId (trans);
  if (t) {
    if (prioexpr)
      errorfree&= t->setPriority (prioexpr);
    if (distribution)
      t->setDistributionType (String2DistributionType(distribution));
    if (distexpr)
      errorfree&= t->setDistributionExpression (distexpr);
    if (weight)
      errorfree&= t->setWeight (weight);
    if (markdep)
      t->setMarkingDependencyType (String2MarkingDependencyType(markdep));
    if (policy)
      t->setPolicyType (String2PolicyType(policy));
    if (concurrency)
      t->setConcurrencyDegree (String2ConcurrencyDegree(concurrency));
    if (guard)
      errorfree&= t->setGuard (guard);
  }
  else
    error_notdefined ("transition", trans,
		      "attributes to transition", trans);
}


void ProcessTypeDecl::addRewardExpr (char *reward, char *expr) {
  Reward *r= RewardList.searchId (reward);
  if (r)
    errorfree&= r->setRewardExpression (expr);
  else
    error_notdefined ("reward", reward, "expression to reward", reward);
}


void ProcessTypeDecl::addMeasureExpr (char *measure, char *expr) {
  Measure *m= MeasureList.searchId (measure);
  if (m)
    errorfree&= m->setMeasureExpression (expr);
  else
    error_notdefined ("measure", measure, "expression to ", measure);
}


void ProcessTypeDecl::addParamGuessExpr (char *param, char *gexpr) {
  Parameter *m= ParameterList.searchId (param);
  if (m)
    errorfree&= m->setGuessExpression (gexpr);
  else
    error_notdefined ("measparam", param,
		      "guess expression to measure parameter", param);
}


//***********************************************************************

/*
Parameter *ProcessTypeDecl::search4globalidInParameters (char *name) {
  Parameter *p;
  for (int i=0; i<ParameterList.size(); i++) {
    p= ParameterList.readElementAt (i);
    if (!strcmp(name, p->globalid))
      return (p);
  }
  return (NULL);
}
*/

char *ProcessTypeDecl::search4longidInPlaces (char *name) {
  Place *p;
  for (int i=0; i<PlaceList.size(); i++) {
    p= PlaceList.readElementAt (i);
    if (!strcmp(name, p->longid)) {
      return (p->getid());
    }
  }
  return (NULL);
}


char *ProcessTypeDecl::search4longidInTransitions (char *name) {
  Transition *p;
  for (int i=0; i<TransitionList.size(); i++) {
    p= TransitionList.readElementAt (i);
    if (!strcmp(name, p->longid)) {
      return (p->getid());
    }
  }
  return (NULL);
}


char *ProcessTypeDecl::search4longidInPorts (char *name) {
  Port *p;
  for (int i=0; i<PortList.size(); i++) {
    p= PortList.readElementAt (i);
//    if (!strcmp(name, p->longid) && (p->getPortType() != USEDPORT)) {
    if (!strcmp(name, p->longid)) {
      return (p->getid());
    }
  }
  return (NULL);
}


char *ProcessTypeDecl::search4longidInRewards (char *name) {
  Reward *p;
  for (int i=0; i<RewardList.size(); i++) {
    p= RewardList.readElementAt (i);
    if (!strcmp(name, p->longid)) {
      return (p->getid());
    }
  }
  return (NULL);
}


char *ProcessTypeDecl::search4longidInMeasures (char *name) {
  Measure *p;
  for (int i=0; i<MeasureList.size(); i++) {
    p= MeasureList.readElementAt (i);
    if (!strcmp(name, p->longid)) {
      return (p->getid());
    }
  }
  return (NULL);
}


char *ProcessTypeDecl::search4longidInParameters (char *name) {
  Parameter *p;
  int i;
  for (i=0; i<ParameterList.size(); i++) {
    p= ParameterList.readElementAt (i);
    if (!strcmp(name, p->longid))
      return (p->getid());
  }
  // wenn nicht gefunden unter den lokalen Parametern, vielleicht
  //  in den globalen Parametern definiert:
  char *zwerg= lastsubstring (name);  // name beschneiden
  if (!zwerg) // Parameter muss global sein.
    return (NULL);
  for (i=0; i<ParameterList.size(); i++) {
    p= ParameterList.readElementAt (i);
    if (!strcmp(zwerg, p->longid)) {
      delete (zwerg);
      return (p->getid());
    }
  }
  delete (zwerg);
  return (NULL);
}


/*
char *ProcessTypeDecl::getExpressionOfLongIdParameter (char *name) {
  if (!name)
    return (NULL);
  Parameter *p;
  for (int i=0; i<ParameterList.size(); i++) {
    p= ParameterList.readElementAt (i);
    if (!strcmp(name, p->longid)) {
      return (p->getParameterExpression());
    }
  }
  // wenn nicht gefunden unter den lokalen Parametern, vielleicht
  //  in den globalen Parametern definiert:
  char *zwerg= lastsubstring (name);  // name beschneiden
  if (!zwerg) // Parameter muss global sein.
    return (NULL);
  for (int i=0; i<ParameterList.size(); i++) {
    p= ParameterList.readElementAt (i);
    if (!strcmp(zwerg, p->longid)) {
      delete (zwerg);
      return (p->getParameterExpression());
    }
  }
  delete (zwerg);
  return (NULL);
}
*/


ProcessInst *ProcessTypeDecl::search4longidInProcessInstances (char *name) {
  ProcessInst *p;
  for (int i=0; i<ProcessInstList.size(); i++) {
    p= ProcessInstList.readElementAt (i);
    if (!strcmp(name, p->longid)) {
      return (p);
    }
  }
  return (NULL);
}

/*
int ProcessTypeDecl::getParameterValue (char *anz) {
  if (!anz)
    return (0);

  char *groesse= NULL;
  int arraygroesse= 0;
  
  groesse= getExpressionOfLongIdParameter (anz);
         // suchen in Parametern, da Arraysize auch ein Id sein kann.
  if (!groesse)   // nicht gefunden, d.h. anz war eine Zahl
    groesse= anz;
  sscanf (groesse, "%d", &arraygroesse);
                    // Arrayanz steht in groesse; nach Integer wandeln
  if (arraygroesse < 1) {
    cerr << "\nCompiler error: processinstance arraysize lesser than 1.";
    exit (1);
  }
  if (arraygroesse > MAXARRAYANZ) {
    cerr << "\nCompiler error: processinstance arraysize too big.";
    exit (1);
  }
  return (arraygroesse);
}
*/

char *ProcessTypeDecl::globalizeExpression (char *expression,
	     FormalParameterType exprtype, char *errortype, char *errorid) {

//       Typ      | Kann folgende Elemente enthalten:
// ===============|===================================================
//       cp       | cp
//       mp       | sm-id
//       mm       | place, rr, mm, cp, mp
//       sm       | place, trans, sm, cr, ir, rr, cp, mp
//       rr       | place, rr, cp, mp
//       ir       | trans, ir, cp, mp + innerhalb if () auch rr, place
//       cr       | place, trans, rr, ir, cr, cp, mp

  char *name= NULL, *expr= NULL, *zwerg= NULL, *global= NULL;
  expr= newstring (expression);
  name= get_first_name (expr);
  bool globalloeschen= false;
  while (name) {

    if ((exprtype == RREWARD) || (exprtype == IREWARD)
	|| (exprtype == CREWARD)) {
      global= search4longidInPlaces (name);  // globalid fuer longid suchen
      if ((exprtype != RREWARD) && !global)
	global= search4longidInTransitions (name);
      if (!global)
	global= search4longidInRewards (name);  // rr: nur rr, ir: nur ir
    }                                           // predicate: rr oder place

    else if (exprtype == MMEASURE) {
      global= search4longidInPlaces (name);  // globalid fuer longid suchen
      if (!global)
	global= search4longidInRewards (name);   // nur rreward
      if (!global)
	global= search4longidInMeasures (name);  // nur mm
    }

//    else if (exprtype == SMEASURE) {
    else if ((exprtype == SMEASURE) || (exprtype == MEASPARAM)) {
      zwerg= lastsubstring (name);
      if (strcmp (zwerg, "ArithmeticMean") || (exprtype == MEASPARAM)) {
	delete (zwerg);   // wenn name!=Pfad.ArithmeticMean
	global= search4longidInPlaces (name);  // globalid fuer longid suchen
	if (!global)
	  global= search4longidInTransitions (name);
	if (!global)
	  global= search4longidInRewards (name);  // r+i+c - reward
	if (!global)
	  global= search4longidInMeasures (name);  // nur sm
      }
      else {  // ArithmeticMean entfernen
	// vorher:  name=   Pfad.ArithmeticMean
	//          expr=   ...$Pfad.ArithmeticMean$($procname.smeasue$)...
	// nachher: global= procname[1].smeasue+...+procname[?].smeasue
	//          expr=   ...($procname.smeasue$)/$Pfad.arrayanz$...
	delete (zwerg);
	zwerg= expr;
	expr= replace_first_name (zwerg, NULL);   // ArithmeticMean loeschen
	delete (zwerg);
	zwerg= NULL;

	delete (name);
	name= get_first_name (expr);
	if (name) {
	  char *pfad= substring2lastpoint (name);
	  char *measure= lastsubstring (name);
	  ProcessInst *proci= search4longidInProcessInstances (pfad);
//	  int arrayanz= getParameterValue (proci->getArrayanz ());
	  int arrayanz= proci->getArrayval ();
	  global= new char[(strlen(pfad)+strlen(measure)+MAXARRAYRES+6)
			   *arrayanz + 4 + MAXARRAYRES];
	  globalloeschen= true;
	  strcpy (global, "(");
	  for (int i=1; i<=arrayanz; i++) {
	    strcat (global, "$");
	    strcat (global, pfad);
	    strcat (global, "[");
	    sprintf (global+strlen(global), "%d", i);
	    strcat (global, "].");
	    strcat (global, measure);
	    strcat (global, "$+");
	  }
	  global[strlen(global)-1]= '\0';    // + loeschen
	  strcat (global, ")/");
	  sprintf (global+strlen(global), "%d", arrayanz);
	  delete (pfad);
	  delete (measure);
	}
	else {
	  cout << "\nGlobalize error 1: " << errortype << " " << errorid
	       << "has ArithmeticMean with no arguments.";
	  errorfree= false;
	  global= "error";
	}
      }
    }

//    else if (exprtype == MEASPARAM)
//      global= search4longidInMeasures (name);  // nur sm

    // Achtung: immer ZULETZT in den Parametern suchen, da sonst evtl.
    //  globale Parameter faelschlicherweise gefunden werden!
    if (!global)
      global= search4longidInParameters (name);
    if (!global) {
      cout << "\nGlobalize error 2: " << name << " used in " << errortype
	   << " " << errorid << ", but not declared.";
      errorfree= false;
    }

    zwerg= expr;
    expr= replace_first_name (zwerg, global);   // $globalid$ ersetzen
    delete (zwerg);
    zwerg= NULL;
    if (globalloeschen)
      delete (global);
    global= NULL;
    globalloeschen= false;

    delete (name);
    name= get_first_name (expr);
  }
  return (expr);
}


void ProcessTypeDecl::globalizePorts () {
  Port *p;
  char *expr= NULL;
  for (int i=0; i<PortList.size(); i++) {
    p= PortList.readElementAt(i);
    if (exist_name (p->getMultiplicity ())) {
      expr= globalizeExpression (p->getMultiplicity (), PARAM, 
				 "multiplicity of port", p->longid);
      p->deleteMultiplicity ();
      p->setMultiplicity (expr);
      delete (expr);
      expr= NULL;
    }
    if (dots && !(i%40))
      cout << '.' << flush;
  }
}


void ProcessTypeDecl::globalizeRewards () {
  Reward *p;
  char *expr= NULL;
  for (int i=0; i<RewardList.size(); i++) {
    p= RewardList.readElementAt(i);
    if (exist_name (p->getRewardExpression ())) {
      if (p->getType () == 'i')
	expr= globalizeExpression (p->getRewardExpression (), IREWARD,
				   "ireward", p->longid);
      else if (p->getType () == 'r')
	expr= globalizeExpression (p->getRewardExpression (), RREWARD,
				   "rreward", p->longid);
      else
	expr= globalizeExpression (p->getRewardExpression (), CREWARD,
				   "creward", p->longid);
      p->deleteRewardExpression ();
      errorfree&= p->setRewardExpression (expr);
      delete (expr);
      expr= NULL;
    }
    if (dots && !(i%10))
      cout << '.' << flush;
  }
}


void ProcessTypeDecl::globalizeMeasures () {
  Measure *p;
  char *expr= NULL;
  for (int i=0; i<MeasureList.size(); i++) {
    p= MeasureList.readElementAt(i);
    if (exist_name (p->getMeasureExpression ())) {
      if (p->getType () == 's')
	expr= globalizeExpression (p->getMeasureExpression (), SMEASURE,
				   "smeasure", p->longid);
      else
	expr= globalizeExpression (p->getMeasureExpression (), MMEASURE,
				   "mmeasure", p->longid);
      p->deleteMeasureExpression ();
      errorfree&= p->setMeasureExpression (expr);
      delete (expr);
      expr= NULL;
    }
    if (dots && !(i%10))
      cout << '.' << flush;
  }
}


void ProcessTypeDecl::globalizeParameters () {
  Parameter *p;
  char *expr= NULL;
  for (int i=0; i<ParameterList.size(); i++) {
    p= ParameterList.readElementAt(i);
    if (exist_name (p->getParameterExpression ())) {
      if (p->getType () == 'c') 
	expr= globalizeExpression (p->getParameterExpression (), PARAM,
				   "parameter", p->longid);
      else 
	expr= globalizeExpression (p->getParameterExpression (), MEASPARAM,
				   "measparam", p->longid);
      p->deleteParameterExpression ();
      errorfree&= p->setParameterExpression (expr);
      delete (expr);
      expr= NULL;
    }
    if ((p->getType () == 'm') && exist_name (p->getGuessExpression ())) {
      expr= globalizeExpression (p->getGuessExpression (), PARAM,
				 "measparam-guess", p->longid);
      p->deleteGuessExpression ();
      errorfree&= p->setGuessExpression (expr);
      delete (expr);
      expr= NULL;
    }
    if (dots && !(i%30))
      cout << '.' << flush;
  }
}


void ProcessTypeDecl::globalizePlaces () {
  Place *p;
  char *expr= NULL;
  for (int i=0; i<PlaceList.size(); i++) {
    p= PlaceList.readElementAt(i);
    if (exist_name (p->getMarking ())) {
      expr= globalizeExpression (p->getMarking (), PARAM, "marking of place",
				 p->longid);
      p->deleteMarking ();
      errorfree&= p->setMarking (expr);
      delete (expr);
      expr= NULL;
    }
    if (dots && !(i%50))
      cout << '.' << flush;
  }
}


void ProcessTypeDecl::globalizeTransitions () {
  Transition *p;
  char *expr= NULL;

  TransitionType ttype;
  char *prio= NULL;
  DistributionType dist;
  char *distexpr= NULL;
  char *weight= NULL;
  MarkingDependencyType mdep;
  PolicyType policy;
  ConcurrencyDegree conc;
  char *guard= NULL;

  for (int i=0; i<TransitionList.size(); i++) {
    p= TransitionList.readElementAt(i);
    p->getAttributes(&ttype, &prio, &dist, &distexpr, &weight,
		     &mdep, &policy, &conc, &guard);

    if (exist_name (prio)) {
      expr= globalizeExpression (prio, RREWARD,
				 "priority of transition", p->longid);
      p->deletePriority ();
      errorfree&= p->setPriority (expr);
      delete (expr);
      expr= NULL;
    }

    if (exist_name (distexpr)) {
      expr= globalizeExpression (distexpr, RREWARD,
				 "distribution of transition", p->longid);
      p->deleteDistributionExpression ();
      errorfree&= p->setDistributionExpression (expr);
      delete (expr);
      expr= NULL;
    }

    if (exist_name (weight)) {
      expr= globalizeExpression (weight, RREWARD,
				 "weight of transition", p->longid);
      p->deleteWeight ();
      errorfree&= p->setWeight (expr);
      delete (expr);
      expr= NULL;
    }

    if (exist_name (guard)) {
      expr= globalizeExpression (guard, RREWARD,
				 "guard of transition", p->longid);
      p->deleteGuard ();
      errorfree&= p->setGuard (expr);
      delete (expr);
      expr= NULL;
    }
    if (dots && !(i%30))
      cout << '.' << flush;
  }
}


void ProcessTypeDecl::globalizeArcs () {
  Arc *p;
  char *expr= NULL, *global= NULL;

  ArcType atype;
  char *source;
  char *destination;
  char *multiplicity;

  for (int i=0; i<ArcList.size(); i++) {
    p= ArcList.readElementAt(i);
    p->getAttributes (&atype, &source, &destination, &multiplicity);

    if (exist_name (multiplicity)) {
      expr= globalizeExpression (multiplicity, RREWARD,
				 "multiplicity of arc", p->longid);
      p->deleteMultiplicity ();
      errorfree&= p->setMultiplicity (expr);
      delete (expr);
      expr= NULL;
    }

    global= search4longidInPlaces (source);
    if (!global)
      global= search4longidInTransitions (source);
    if (!global)
      global= search4longidInPorts (source);
    if (!global) {
      cout << "\nGlobalize error 3: source " << source
	   << " of arc " << p->longid << " does not exist.";
      errorfree= false;
    }
    else {
      p->deleteSource ();
      errorfree&= p->setSource (global);
    }

    global= search4longidInPlaces (destination);
    if (!global)
      global= search4longidInTransitions (destination);
    if (!global)
      global= search4longidInPorts (destination);
    if (!global) {
      cout << "\nGlobalize error 4: destination " << destination
	   << " of arc " << p->longid << " does not exist.";
      errorfree= false;
    }
    else {
      p->deleteDestination ();
      errorfree&= p->setDestination (global);
    }
    if (dots && !(i%20))
      cout << '.' << flush;
  }
}


bool ProcessTypeDecl::globalize () {
//       Typ      | enthaelt
// ===============|==========
//    port-mult.  |    cp
//   place-mark.  |    cp
//   trans-prio.  |    rr
//   trans-dist.  |    rr
//  trans-weight  |    rr
//   trans-guard  |    rr
//    arc-mult.   |    rr
  errorfree= true;
  if (dots)
    cout << '.' << flush;
  globalizePorts ();
  if (dots)
    cout << '.' << flush;
  globalizeArcs ();
  if (dots)
    cout << '.' << flush;
  globalizeRewards ();
  if (dots)
    cout << '.' << flush;
  globalizeMeasures ();
  if (dots)
    cout << '.' << flush;
  globalizeParameters ();
  if (dots)
    cout << '.' << flush;
  globalizePlaces ();
  if (dots)
    cout << '.' << flush;
  globalizeTransitions ();
  if (dots)
    cout << '.' << flush;
  return (errorfree);
}


bool ProcessTypeDecl::checkIntern () {
  int i;
  FormalParameterType forpartype;
  Port            *checkport=     NULL;
  Reward          *checkreward=   NULL;
  Measure         *checkmeasure=  NULL;
  Parameter       *checkparam=    NULL;
  ProcessInst     *checkprocinst= NULL;
  ProcessUsage    *checkprocuse=  NULL;
  Transition      *checktrans=    NULL;
  Arc             *checkarc=      NULL;
  TransitionType ttype;
  char *prio;
  DistributionType dist;
  char *distexpr;
  char *weight;
  MarkingDependencyType markdep;
  PolicyType policy;
  ConcurrencyDegree concurrency;
  char *guard;
  ArcType atype;
  char *from;
  char *to;
  char *mult;

  char *ident= NULL;

  // wenn DECL => keine formalen Parameter:
  if ((type == DECL) && (FormalParameterList.size() > 0)) {
    error_message (NULL, NULL, "processes must not contain any formal ",
		   "parameters");
  }
  else {  // TYPE => entweder sm- oder rr-Parameter (nicht gemischt)
    bool rr= false, mp= false;
    for (i= 0; i < FormalParameterList.size(); i++) {
      forpartype= (FormalParameterList.readElementAt (i))
	           ->getFormalParameterType ();
      mp |= (forpartype == MEASPARAM);
      rr |= (forpartype == RREWARD);
    }
    if (rr && mp)
      error_message (NULL, NULL, "process types must not contain rrewards",
		     " measure parameters at the same time");
  }

  for (i= 0; i < PortList.size(); i++) {
    checkport= PortList.readElementAt (i);
    if (checkport->getPortType () == NOPORTTYPE)
      error_message ("port", checkport->getid (), "has no porttype", NULL);
    ident= lastsubstring (checkport->getid ());
    if (!ident && (checkport->getPortType () == USEDPORT))
      error_message ("type of port", checkport->getid (),
		     "is not defined in this process[type]", NULL);
    delete (ident);
  }

  for (i= 0; i < RewardList.size(); i++) {
    checkreward= RewardList.readElementAt (i);
    if (!checkreward->getRewardExpression ())
      error_message ("reward", checkreward->getid (),
		     "declared but not defined", NULL);
  }
  
  for (i= 0; i < MeasureList.size(); i++) {
    checkmeasure= MeasureList.readElementAt (i);
    if (!checkmeasure->getMeasureExpression ())
      error_message ("measure", checkmeasure->getid (),
		     "declared but not defined", NULL);
  }

  for (i= 0; i < ParameterList.size(); i++) {
    checkparam= ParameterList.readElementAt (i);
    if (!checkparam->getParameterExpression ())
      error_message ("parameter", checkparam->getid (),
		     "declared but not defined", NULL);
  }

  for (i= 0; i < ProcessInstList.size(); i++) {
    checkprocinst= ProcessInstList.readElementAt (i);
    if (!checkprocinst->getProcessType ())
      error_message ("process instance", checkprocinst->getid (),
		     ": process type name not defined", NULL);
  }

  for (i= 0; i < ProcessUsageList.size(); i++) {
    checkprocuse= ProcessUsageList.readElementAt (i);
    if (!checkprocuse->getProcName ())
      error_message ("process usage", checkprocuse->getid (),
		     ": process name not defined", NULL);
  }

  for (i= 0; i < TransitionList.size(); i++) {
    checktrans= TransitionList.readElementAt (i);
    checktrans->getAttributes (&ttype, &prio, &dist, &distexpr, &weight,
			       &markdep, &policy, &concurrency, &guard);
    if (ttype == NOTRTYPE)
      error_message ("transition", checktrans->getid (),
		     "has no transitiontype (immediate or timed)", NULL);
    else if (ttype == IMM) {
      if ((dist != NODISTR) || (distexpr))
	error_message ("immediate transition", checktrans->getid (),
		       "must not have any ", "distributions");
      if (markdep != NOMARKDEP)
	error_message ("immediate transition", checktrans->getid (),
		       "must not have any marking dependecies like ",
		       MarkingDependencyType2String (markdep));
      if (policy != NOPOLICY)
	error_message ("immediate transition", checktrans->getid (),
		       "must not have any policies like ",
		       PolicyType2String (policy));
      if (concurrency == IS)
	error_message ("immediate transition", checktrans->getid (),
		       "must not have any ", "concurrency other than ss");
    }
    else if (ttype == TIMED) {
      if (prio)
	error_message ("timed transition", checktrans->getid (),
		       "must not have any priorities like ", prio);
      if (dist && !distexpr)
	error_message ("timed transition", checktrans->getid (),
		       "has no distribution expression", NULL);
    }
  }

  for (i= 0; i < ArcList.size(); i++) {
    checkarc= ArcList.readElementAt (i);
    checkarc->getAttributes (&atype, &from, &to, &mult);
    if (!from)
      error_message ("arc", checkarc->getid (), ": undefined source", NULL);
    if (!to)
      error_message ("arc", checkarc->getid (), ": undefined destination",
		     NULL);
  }

  return (errorfree);
}




#endif
