#ifndef SPNLPROCESSH
#define SPNLPROCESSH




class ProcessTypeDecl {

  private:
    char *id;
    char *modid;
    ProcessTypeDeclType type;

    Vektor <FormalParameter> FormalParameterList;
          // List of Name (Uebergabeparameter) - nur fuer ProcessDeclaration

    Vektor <Port> PortList;             	// List of Port
    Vektor <Reward> RewardList;          	// List of Reward
    Vektor <Measure> MeasureList;        	// List of Measure

    Vektor <Parameter> ParameterList;   	// List of Parameter
    Vektor <ProcessInst> ProcessInstList;	// List of ProcessInst
    Vektor <ProcessUsage> ProcessUsageList;     // List of ProcessUsage

    Vektor <Place> PlaceList;			// List of Places
    Vektor <Transition> TransitionList;		// List of Transitions
    Vektor <Arc> ArcList;              		// List of Arcs

    Vektor <String> IdList;      // List of all Ids (measure, arc, usw.)
//  bool isInIdList (char *id);  // Sammelt global alle Identifier, die nur
    bool add2IdList (char *id);  //  einmal vergeben werden duerfen.

    bool TransattExistsInSection (int section);
    bool ParameterTypeExistsInSection (char type, int section);
    bool PlaceExistsInSection (int section);
    bool TransitionExistsInSection (int section);
    bool ArcExistsInSection (int section);
    bool RewardExistsInSection (int section);
    bool MeasureExistsInSection (int section);
    // Liefert zurueck, ob mind. ein Objekt des entsprechenden Typs in der
    //  uebergebenen Section existiert.

    void error_cannotdeclare (char typkuerzel, char *idtype, char *idname);
    // Gibt Fehlermeldung aus: idname schon vergeben, kann nicht neuen idtype
    //  mit Namen idname deklarieren.

    void error_notdefined (char *nichtdatyp, char *nichtdaid,
			   char *hinzufuegtyp, char *hinzufuegid);
    // Gibt Fehlermeldung aus: nichtdatyp mit Namen nichtdaid ist nicht
    //  vorhanden, kann hinzufuegtyp mit Namen hinzufuegid nicht hinzufuegen.

    void error_message (char *typ, char *id, char *message1, char *message2);
    // Gibt Fehlermeldung aus: typ mit Namen id: message1message2.

//    char *getExpressionOfLongIdParameter (char *longid);

//    Parameter *search4globalidInParameters (char *name);

    char *search4longidInPlaces (char *name);
    char *search4longidInTransitions (char *name);
    char *search4longidInParameters (char *name);
    char *search4longidInRewards (char *name);
    char *search4longidInMeasures (char *name);
    // Liefern jeweils die normalen Identifier der Objekte, deren longid mit
    //  name uebereinstimmt, zurueck.

    ProcessInst *search4longidInProcessInstances (char *name);
    // Liefert die ProzessInstanz, bei der longid  mit name uebereinstimmt,
    //  zurueck.

    char *globalizeExpression (char *expression, FormalParameterType exprtype,
			       char *errortype, char *errorid);

    void globalizePorts ();
    void globalizeRewards ();
    void globalizeMeasures ();
    void globalizeParameters ();
    void globalizePlaces ();
    void globalizeTransitions ();
    void globalizeArcs ();

  public:
    char *search4longidInPorts (char *name);  // fuer den Compiler

    bool errorfree;

    char *longid;
    char *oldid;
    int section;

    ProcessTypeDecl (char *modid, char *name);
    ~ProcessTypeDecl ();
    void display ();
    void display_comp (int sectionct);
    char *getid ();
    char *getmodid ();

    void setType (ProcessTypeDeclType processtype);
    ProcessTypeDeclType getType ();

    FormalParameter *addFormalParameter (char *paramid, char *paramtype);
    FormalParameter *getFormalParameter (int i);
    FormalParameter *searchFormalParameter (char *id);
    bool containingMeasparams ();

    Port *addPort (char *port, char *type, char *mult);
    Port *getPort (int i);
    Port *searchPort (char *id);
    void removePort (int i);
 
    Reward *addReward (char *reward, char typ, VisibleType visible);
    Reward *getReward (int i);
    Reward *searchReward (char *id);
    void addRewardExpr (char *reward, char *expr);

    Measure *addMeasure (char *measure, char typ, VisibleType visible);
    Measure *getMeasure (int i);
    Measure *searchMeasure (char *id);
    void addMeasureExpr (char *measure, char *expr);

    Parameter *addParameter (char *param, char typ, char *expr);
    Parameter *getParameter (int i);
    Parameter *searchParameter (char *id);
    void addParamGuessExpr (char *param, char *guessexpr);

    ProcessInst *addProcessInst (char *processinst, char *array,
				 char *processtype);
    ProcessInst *getProcessInst (int i);
    ProcessInst *searchProcInst (char *id);
    void removeProcessInst (int i);
    void addProcessInstFormPar (char *processinst, char *paramexpr,
				char *guessexpr);

    ProcessUsage *addProcessUsage (char *newid, char *procid);
    ProcessUsage *getProcessUsage (int i);
    ProcessUsage *searchProcUsage (char *id);
    void removeProcessUsage (int i);

    Place *addPlace (char *place, char *marking);
    Place *getPlace (int i);
    Place *searchPlace (char *id);

    Transition *addTransition (char *trans, char *type);
    Transition *getTransition (int i);
    Transition *searchTransition (char *id);
    void addTransitionAttr (char *trans, char *prioexpr, char *distribution,
			    char *distexpr, char *weight, char *markdep,
			    char *policy, char *concurrency, char *guard);

    Arc *addArc (char *arc, char *from, char *to, char *type, char *mult);
    Arc *getArc (int i);
    Arc *searchArc (char *id);
    void removeArc (char *arcid);

    bool globalize ();
    // Verknuepft alle internen Referenzen auf die neuen (globalen)
    //  Identifier. Dabei werden alle $-Zeichen entfernt.
    // Achtung: Sollte nur auf compilierte Netze angewendet werden!

    bool checkIntern ();
    // Ueberprueft intern alle Objekte (Arcs, Ports, Rewards, ...) (sofern sie
    //  lokal in diesem Modul ueberprueft werden koennen). Ruft dann fuer
    //  alle Prozesse und Prozesstypen deren Methode checkIntern auf.
};


#endif
