#ifndef SPNLTYPESCC
#define SPNLTYPESCC


#include <string.h>
#include <iostream.h>
#include "spnlbasics.cc"
#include "spnltypes.h"
#include "stdfun.cc"
#include "foutput.h"



String::String (char *s) {
  length= strlen(s);
  id= newstring (s);
}

String::~String () {
// cout << "String\n";
  delete (id);
  length= 0;
  id= NULL;
}

void String::display () {
  fnet << id;
}

char *String::getid () {
  return (id);
}




/**************************************************************************
 *                                                                        *
 *                     Basisklassen fuer SPNL-Elemente                    *
 *                                                                        *
 **************************************************************************/


/**** Basisfehlermeldung ****/

void error_attach (char *modid, char *procid, char *errorexprtyp,
		   char *errorexpr, char *errortyp, char *errorid,
		   char *actexpr) {
  cout << "\nmodule " << modid;
  if (procid)
    cout << ", process[type] " << procid;
  cout << ": Cannot attach " << errorexprtyp << " '";
  display_dollarexpr (errorexpr);
  cout << "' to " << errortyp << " " << errorid << " with actual ";
  cout << errorexprtyp << "= '";
  display_dollarexpr (actexpr);
  cout << "'.";
}


/**** class FormalParameter ****/

FormalParameter::FormalParameter (char *modident, char *procident, char *s) {
  id= newstring (s);
  type= PARAM;
  modid= modident;
  procid= procident;
}

FormalParameter::~FormalParameter () {
// cout << "vorFormalParameter\n";
  delete (id);
}

void FormalParameter::display () {
  fnet << FormalParameterType2String (type) << " " << id;
}

char *FormalParameter::getid () {
  return (id);
}

void FormalParameter::setFormalParameterType (FormalParameterType t) {
  type= t;
}

FormalParameterType FormalParameter::getFormalParameterType () {
  return (type);
}


/**** class GuessParameter ****/

GuessParameter::GuessParameter (char *modident, char *procident,
				char *newid, char *name) {
  id= newstring (newid);
  modid= modident;
  procid= procident;
  guessexpr= newstring (name);
}

GuessParameter::~GuessParameter () {
// cout << "vor GuessParameter\n";
  delete (id);
  delete (guessexpr);
}

void GuessParameter::display () {
  fnet << id;
  if (guessexpr)
    fnet << " guess= " << guessexpr;
}

char *GuessParameter::getid () {
  return (id);
}

char *GuessParameter::getGuessExpr () {
  return (guessexpr);
}



/**** class ProcessUsage ****/

ProcessUsage::ProcessUsage (char *modident, char *procident, char *newid,
			    char *name) {
  id= newstring (newid);
  procname= newstring (name);
  modid= modident;
  procid= procident;
  oldid= NULL;
  longid= NULL;
}

ProcessUsage::~ProcessUsage () {
// cout << "vor ProcessUsage\n";
  delete (oldid);
  delete (longid);
  delete (id);
  delete (procname);
}

void ProcessUsage::display () {
  fnet << "\nuseprocess " << id << ": " << procname << ';';
}

void ProcessUsage::display_comp () {
  fnet << "\nuseprocess " << id << '\t' << longid
       << "\t: " << procname << ';';
}

char *ProcessUsage::getid () {
  return (id);
}

//char *ProcessUsage::getmodid () {
//  return (modid);
//}

char *ProcessUsage::getProcName () {
  return (procname);
}



/**** class Place ****/

Place::Place (char *modident, char *procident, char *s) {
  id= newstring (s);
  modid= modident;
  procid= procident;
  marking= NULL;
  oldid= NULL;
  longid= NULL;
  section= 0;
}

Place::~Place () {
// cout << "vor Place\n";
  delete (oldid);
  delete (longid);
  delete (id);
  delete (marking);
}

void Place::display () {
  fnet << id;
  if (marking)
    fnet << ": " << marking;
  fnet << ';';
}

void Place::display_comp (int validsection) {
  if (validsection == section) {
    fnet << "\n " << id << '\t' << longid;
    if (marking)
      fnet << " : " << marking;
    fnet << ';';
  }
}

char *Place::getid () {
  return (id);
}

char *Place::getmodid () {
  return (modid);
}

char *Place::getprocid () {
  return (procid);
}

bool Place::setMarking (char *p) {
  if (!marking && p) {
    marking= newstring (p);
    return (true);
  }
  else {
    error_attach (modid, procid, "marking", p, "place", this->id, marking);
    return (false);
  }
}

char *Place::getMarking () {
  return (marking);
}

void Place::deleteMarking () {
  if (marking)
    delete (marking);
  marking= NULL;
}



/**** class Transition ****/

Transition::Transition (char *modident, char *procident, char *s) {
  id= newstring (s);
  modid= modident;
  procid= procident;
  oldid= NULL;
  longid= NULL;
  type= NOTRTYPE;
  prio= NULL;
  dist= NODISTR;
  distexpr= NULL;
  weight= NULL;
  markdep= NOMARKDEP;
  policy= NOPOLICY;
  concurrency= NOCONC;
  guard= NULL;
  section= 0;
}

Transition::~Transition () {
// cout << "vor Transition\n";
  delete (id);
  delete (oldid);
  delete (longid);
  delete (prio);
  delete (distexpr);
  delete (weight);
  delete (guard);
}

void Transition::display () {
  fnet << id << ": " << TransitionType2String (type) << ';';
}

void Transition::display_comp (int validsection) {
  if (validsection == section)
    fnet << "\n " << id << '\t' << longid << " : "
         << TransitionType2String (type) << ';';
}

void Transition::displayattr (int validsection) {
  if ((prio || dist || weight || markdep || policy || concurrency || guard)
      && ((validsection < 0) || (validsection == section))) {
    int firstattr= true;
    fnet << "\n " << id << " :\t";
    if (prio) {
      fnet << "prio= " << prio;
      firstattr= false;
    }
    if (dist) {
      if (!firstattr)
        fnet << ", ";
      if (dist == EXPOLY)
	fnet << "dist= " << distexpr;
      else
	fnet << "dist= " << DistributionType2String (dist)
	     << " (" << distexpr << ')';
      firstattr= false;
    }
    if (weight) {
      if (!firstattr)
        fnet << ", ";
      fnet << "weight= " << weight;
      firstattr= false;
    }
    if (markdep) {
      if (!firstattr)
        fnet << ", ";
      fnet << "markdep= " << MarkingDependencyType2String (markdep);
      firstattr= false;
    }
    if (policy) {
      if (!firstattr)
        fnet << ", ";
      fnet << "policy= " << PolicyType2String (policy);
      firstattr= false;
    }
    if (concurrency) {
      if (!firstattr)
        fnet << ", ";
      fnet << "concurrency= " << ConcurrencyDegree2String (concurrency);
      firstattr= false;
    }
    if (guard) {
      if (!firstattr)
        fnet << ", ";
      fnet << "guard= " << guard;
    }
    fnet << ';';
  }
}

char *Transition::getid () {
  return (id);
}

char *Transition::getmodid () {
  return (modid);
}

char *Transition::getprocid () {
  return (procid);
}

void Transition::setTransitionType (TransitionType t) {
  type= t;
}

bool Transition::setPriority (char *r) {
  if (!prio && r) {
    prio= newstring (r);
    return (true);
  }
  else {
    error_attach (modid, procid, "priority", r, "transition", this->id, prio);
    return (false);
  }
}

void Transition::deletePriority () {
  if (prio)
    delete (prio);
  prio= NULL;
}

void Transition::setDistributionType (DistributionType d) {
  dist= d;
}

bool Transition::setDistributionExpression (char *r) {
  if (!distexpr && r) {
    distexpr= newstring (r);
    return (true);
  }
  else {
    error_attach (modid, procid, "distribution expression", r, "transition",
		  this->id, distexpr);
    return (false);
  }
}

void Transition::deleteDistributionExpression () {
  if (distexpr)
    delete (distexpr);
  distexpr= NULL;
}

bool Transition::setWeight (char *r) {
  if (!weight && r) {
    weight= newstring (r);
    return (true);
  }
  else {
    error_attach (modid, procid, "weight", r, "transition", this->id, weight);
    return (false);
  }
}

void Transition::deleteWeight () {
  if (weight)
    delete (weight);
  weight= NULL;
}

void Transition::setMarkingDependencyType (MarkingDependencyType m) {
  markdep= m;
}

void Transition::setPolicyType (PolicyType p) {
  policy= p;
}

void Transition::setConcurrencyDegree (ConcurrencyDegree c) {
  concurrency= c;
}

bool Transition::setGuard (char *r) {
  if (!guard && r) {
    guard= newstring (r);
    return (true);
  }
  else {
    error_attach (modid, procid, "guard", r, "transition", this->id, guard);
    return (false);
  }
}

void Transition::deleteGuard () {
  if (guard)
    delete (guard);
  guard= NULL;
}

void Transition::getAttributes (TransitionType *type, char **prio,
		    DistributionType *dist, char **distexpr,
		    char **weight, MarkingDependencyType *markdep,
		    PolicyType *policy, ConcurrencyDegree *concurrency,
		    char **guard) {
  *type= this->type;
  *prio= this->prio;
  *dist= this->dist;
  *distexpr= this->distexpr;
  *weight= this->weight;
  *markdep= this->markdep;
  *policy= this->policy;
  *concurrency= this->concurrency;
  *guard= this->guard;
}


/**** class Arc ****/

Arc::Arc (char *modident, char *procident, char *s) {
  id= newstring (s);
  modid= modident;
  procid= procident;
  oldid= NULL;
  longid= NULL;
  from= NULL;
  to= NULL;
  mult=NULL;
//  type= NOARCTYPE;
  type= IOARC;
  section= 0;
}

Arc::~Arc () {
//cout << "delete arc >"<< id << "< >" << longid << "< >" << oldid << "<\n";
  delete (id);
  delete (oldid);
  delete (longid);
  delete (from);
  delete (to);
  delete (mult);
}

void Arc::display () {
  fnet << id << "\t: (" << from << ", " << to << ')';
//  if (type != NOARCTYPE)
  if (type != IOARC)
    fnet << ", type " << ArcType2String (type);
  if (mult)
    fnet << ", mult " << mult;
  fnet << ';';
}

void Arc::display_comp (int validsection) {
  if (validsection == section) {
    fnet << "\n " << id << '\t' << longid
         << " : (" << from << ", " << to << ')';
//  if (type != NOARCTYPE)
    if (type != IOARC)
      fnet << ", type " << ArcType2String (type);
    if (mult)
      fnet << ", mult " << mult;
    fnet << ';';
  }
}

char *Arc::getid () {
  return (id);
}

void Arc::setArcType (ArcType p) {
  type= p;
}

bool Arc::setSource (char *s) {
  if (!from && s) {
    from= newstring (s);
    return (true);
  }
  else {
    error_attach (modid, procid, "source", s, "arc", this->id, from);
    return (false);
  }
}

void Arc::deleteSource () {
  if (from)
    delete (from);
  from= NULL;
}

bool Arc::setDestination (char *d) {
  if (!to && d) {
    to= newstring (d);
    return (true);
  }
  else {
    error_attach (modid, procid, "destination", d, "arc", this->id, to);
    return (false);
  }
}

void Arc::deleteDestination () {
  if (to)
    delete (to);
  to= NULL;
}

bool Arc::setMultiplicity (char *multiplicity) {
  if (!mult && multiplicity) {
    mult= newstring (multiplicity);
    return (true);
  }
  else {
    error_attach (modid, procid, "multiplicity", multiplicity, "arc",
		  this->id, mult);
    return (false);
  }
}

void Arc::deleteMultiplicity () {
  if (mult)
    delete (mult);
  mult= NULL;
}

void Arc::getAttributes (ArcType *p, char **source, char **destination,
			 char **multiplicity) {
  *p= this->type;
  *source= this->from;
  *destination= this->to;
  *multiplicity= this->mult;
}


/**** class Port ****/

Port::Port (char *modident, char *procident, char *s) {
  id= newstring (s);
  modid= modident;
  procid= procident;
  type= NOPORTTYPE;
  mult= NULL;
  longid= NULL;
  oldid= NULL;
  section= 0;
}

Port::~Port () {
// cout << "vor Port\n";
  delete (mult);
  delete (id);
  delete (longid);
  delete (oldid);
}

void Port::display () {
  fnet << id << ": " << PortType2String (type);
  if (mult)
    fnet << ", mult= " << mult;
  fnet << ';';
}

void Port::display_comp (int validsection) {
  if (validsection == section) {
    fnet << "\nport " << id << '\t' << longid
         << " : " << PortType2String (type);
    if (mult)
      fnet << ", mult= " << mult;
    fnet << ';';
  }
}

char *Port::getid () {
  return (id);
}

char *Port::getmodid () {
  return (modid);
}

char *Port::getprocid () {
  return (procid);
}

void Port::setPortType (PortType p) {
  type= p;
}

PortType Port::getPortType () {
  return (type);
}

bool Port::setMultiplicity (char *multiplicity) {
  if (!mult && multiplicity) {
    mult= newstring (multiplicity);
    return (true);
  }
  else {
    error_attach (modid, procid, "multiplicity", multiplicity, "port",
		  this->id, mult);
    return (false);
  }
}

char *Port::getMultiplicity () {
  return (mult);
}

void Port::deleteMultiplicity () {
  if (mult)
    delete (mult);
  mult= NULL;
}


/**** class Parameter ****/

Parameter::Parameter (char *modident, char *procident, char *s, char typ) {
  id= newstring (s);
  modid= modident;
  procid= procident;
  oldid= NULL;
  longid= NULL;
  expr= NULL;
  guessexpr= NULL;
  type= typ;
  section= 0;
}

Parameter::~Parameter () {
// cout << "vor Parameter\n";
  delete (id);
  delete (oldid);
  delete (longid);
  delete (expr);
  delete (guessexpr);
}

void Parameter::display () {
  fnet << id << "= " << expr;
  if (guessexpr)
    fnet << " guess = " << guessexpr;
  fnet << ';' << ", ";
  if (type == 'm')
    fnet << "measparam";
  else
    fnet << "constparam";
}

void Parameter::display_param_comp (int validsection) {
  if ((type == 'c') && (validsection == section))
    fnet << "\n " << id << '\t' << longid << " = " << expr << ';';
}

void Parameter::display_measparam_comp (int validsection) {
  if ((type == 'm') && (validsection == section)) {
    fnet << "\n " << id << '\t' << longid << " = " << expr;
    if (guessexpr)
      fnet << " guess = " << guessexpr;
    fnet << ';';
  }
}

char *Parameter::getid () {
  return (id);
}

char *Parameter::getmodid () {
  return (modid);
}

char *Parameter::getprocid () {
  return (procid);
}

bool Parameter::setParameterExpression (char *p) {
  if (!expr && p) {
    expr= newstring (p);
    return (true);
  }
  else {
    error_attach (modid, procid, "expression", p, "parameter", this->id,expr);
    return (false);
  }
}

char *Parameter::getParameterExpression () {
  return (expr);
}

void Parameter::deleteParameterExpression () {
  if (expr)
    delete (expr);
  expr= NULL;
}

bool Parameter::setGuessExpression (char *m) {
  if ((!guessexpr) && m && (type == 'm')) {
    guessexpr= newstring (m);
    return (true);
  }
  else {
    error_attach (modid, procid, "guess expression", m, "parameter", this->id,
		  guessexpr);
    return (false);
  }
}

char *Parameter::getGuessExpression () {
  return (guessexpr);
}

void Parameter::deleteGuessExpression () {
  if (guessexpr)
    delete (guessexpr);
  guessexpr= NULL;
}

char Parameter::getType () {
  return (type);
}


/**** class Reward ****/

Reward::Reward (char *modident, char *procident, char *s, char typ) {
  id= newstring (s);
  type= typ;
  modid= modident;
  procid= procident;
  oldid= NULL;
  longid= NULL;
  expr= NULL;
  visible= PUBLIC;
  section= 0;
}

Reward::~Reward () {
// cout << "vor Reward\n";
  delete (id);
  delete (oldid);
  delete (longid);
  delete (expr);
}

void Reward::display () {
  fnet << id << "= " << expr << ", " << type << "reward, "
       << VisibleType2String (visible) << ';';
}

void Reward::display_comp (int validsection) {
  if (validsection == section)
    fnet << "\n " << id << '\t' << longid << " = " << expr << ';';
}

char *Reward::getid () {
  return (id);
}

bool Reward::setRewardExpression (char *r) {
  if (!expr && r) {
    expr= newstring (r);
    return (true);
  }
  else {
    error_attach (modid, procid, "expression", r, "reward", this->id, expr);
    return (false);
  }
}

char *Reward::getRewardExpression () {
  return (expr);
}

void Reward::deleteRewardExpression () {
  if (expr)
    delete (expr);
  expr= NULL;
}

void Reward::setVisibility (VisibleType v) {
  visible= v;
}

VisibleType Reward::getVisibility () {
  return (visible);
}

char Reward::getType () {
  return (type);
}



/**** class Measure ****/

Measure::Measure (char *modident, char *procident, char *s, char typ) {
  id= newstring (s);
  modid= modident;
  procid= procident;
  type= typ;
  oldid= NULL;
  longid= NULL;
  expr= NULL;
  visible= PUBLIC;
  section= 0;
}

Measure::~Measure () {
// cout << "vor Measure\n";
  delete (id);
  delete (oldid);
  delete (longid);
  delete (expr);
}

void Measure::display () {
  fnet << id << "= " << expr << ", " << type << "measure, "
       << VisibleType2String (visible) << ';';
}

void Measure::display_comp (int validsection) {
  if (validsection == section)
    fnet << "\n " << id << '\t' << longid << " = " << expr << ';';
}

char *Measure::getid () {
  return (id);
}

bool Measure::setMeasureExpression (char *m) {
  if (!expr && m) {
    expr= newstring (m);
    return (true);
  }
  else {
    error_attach (modid, procid, "expression", m, "measure", this->id, expr);
    return (false);
  }
}

char *Measure::getMeasureExpression () {
  return (expr);
}

void Measure::deleteMeasureExpression () {
  if (expr)
    delete (expr);
  expr= NULL;
}

void Measure::setVisibility (VisibleType v) {
  visible= v;
}

VisibleType Measure::getVisibility () {
  return (visible);
}

char Measure::getType () {
  return (type);
}


/**** class ProcessInst ****/

ProcessInst::ProcessInst (char *modident, char *procident, char *s) {
  id= newstring (s);
  modid= modident;
  procid= procident;
  arrayanz= NULL;
  arrayval= 0;
  processtype= NULL;
  longid= NULL;
  oldid= NULL;
  section= 0;
}

ProcessInst::~ProcessInst () {
// cout << "vor ProcessInst\n";
  delete (id);
  delete (longid);
  delete (oldid);
  delete (arrayanz);
  delete (processtype);
}

void ProcessInst::display () {
  int i;
  fnet << "\nprocessinstance " << id << ": ";
  if (arrayanz)
    fnet << "array [1.." << arrayanz << "] of ";
  fnet << processtype;
  if (ExprList.size() > 0) {
    fnet << " (";
    for (i=0; i<ExprList.size()-1; i++) {
      (ExprList.readElementAt (i))->display();
      fnet << ", ";
    }
    ExprList.readElementAt (ExprList.size()-1)->display();
    fnet << ')';
  }
  fnet << ';';
}

void ProcessInst::display_comp (int validsection) {
  int i;
  if (validsection == section) {
    fnet << "\nprocessinstance " << id << '\t' << longid << "\t: ";
    if (arrayanz)
      fnet << "array [1.." << arrayanz << "] of ";
    fnet << processtype;
    if (ExprList.size() > 0) {
      fnet << " (";
      for (i=0; i<ExprList.size()-1; i++) {
	(ExprList.readElementAt (i))->display();
	fnet << ", ";
      }
      ExprList.readElementAt (ExprList.size()-1)->display();
      fnet << ')';
    }
    fnet << ';';
  }
}

char *ProcessInst::getid () {
  return (id);
}

char *ProcessInst::getmodid () {
  return (modid);
}

bool ProcessInst::setArrayanz (char *anz) {
  if (!arrayanz && anz) {
    arrayanz= newstring (anz);
    return (true);
  }
  else {
    error_attach (modid, procid, "array size", anz, "process instance",
		  this->id, arrayanz);
    return (false);
  }
}


void ProcessInst::setArrayval (int anz) {
  arrayval= anz;
}

char *ProcessInst::getArrayanz () {
  return (arrayanz);
}

int ProcessInst::getArrayval () {
  return (arrayval);
}

bool ProcessInst::setProcessType (char *type) {
  if (!processtype && type) {
    processtype= newstring (type);
    return (true);
  }
  else {
    error_attach (modid, procid, "process type", type, "process instance",
		  this->id, processtype);
    return (false);
  }
}

char *ProcessInst::getProcessType () {
  return (processtype);
}

void ProcessInst::addParameterExpression (char *param, char *guess) {
  GuessParameter *s= new GuessParameter (modid, this->id, param, guess);
  ExprList.addElement (s);
}

char *ProcessInst::getParameterExpression (int i) {
  if (ExprList.size () > i)
    return (ExprList.readElementAt (i))->getid ();
  else
    return (NULL);
}

char *ProcessInst::getParameterGuessExpression (int i) {
  if (ExprList.size () > i)
    return (ExprList.readElementAt (i))->getGuessExpr ();
  else
    return (NULL);
}


#endif

