
#ifndef SPNLTYPESH
#define SPNLTYPESH

#include "vektor.cc"


/**************************************************************************
 *                                                                        *
 *                           Basisklasse String                           *
 *                                                                        *
 **************************************************************************/


class String {

  private:
    char *id;
    int length;

  public:
    String (char *s);
    ~String ();
    void display ();
    char *getid ();
};


/**************************************************************************
 *                                                                        *
 *                     Basisklassen fuer SPNL-Elemente                    *
 *                                                                        *
 **************************************************************************/

void error_attach (char *modid, char *procid, char *errorexprtyp,
		   char *errorexpr, char *errortyp, char *errorid,
		   char *actexpr);
// Basisfehlermeldung


class FormalParameter {

  private:
    char *id;
    char *modid, *procid;  // Verweise auf die Namen der zug. Module
                           //  und Prozesse
    FormalParameterType type;

  public:

    FormalParameter (char *modid, char *procid, char *name);
    ~FormalParameter ();
    void display ();
    char *getid ();

    void setFormalParameterType (FormalParameterType type);
    FormalParameterType getFormalParameterType ();
};


class GuessParameter {

  private:
    char *id;
    char *modid, *procid;
    char *guessexpr;

  public:
    GuessParameter (char *modid, char *procid, char *id, char *guessexpr);
    ~GuessParameter ();
    void display ();
    char *getid ();

    char *getGuessExpr ();
};


class ProcessUsage {

  private:
    char *id;
    char *modid, *procid;
    char *procname;

  public:
    char *longid;
    char *oldid;

    ProcessUsage (char *modid, char *procid, char *newid, char *name);
    ~ProcessUsage ();
    void display ();
    void display_comp ();
    char *getid ();
//    char *getmodid ();

    char *getProcName ();
};


class Place {

  private:
    char *id;
    char *modid, *procid;
    char *marking;

  public:
    char *longid;
    char *oldid;
    int section;

    Place (char *modid, char *procid, char *name);
    ~Place ();
    void display ();
    void display_comp (int section);
    char *getid ();
    char *getmodid ();
    char *getprocid ();

    bool setMarking (char *p);
    char *getMarking ();
    void deleteMarking ();
};


class Transition {

  private:
    char *id;
    char *modid, *procid;
    TransitionType type;
    char *prio;
    DistributionType dist;
    char *distexpr;               // Distribution als RewardExpr.
    char *weight;                 // dito -^-
    MarkingDependencyType markdep;
    PolicyType policy;
    ConcurrencyDegree concurrency;
    char *guard;

  public:
    char *longid;
    char *oldid;
    int section;

    Transition (char *modid, char *procid, char *name);
    ~Transition ();
    void display ();
    void display_comp (int section);
    void displayattr (int section);
    char *getid ();
    char *getmodid ();
    char *getprocid ();

    void setTransitionType (TransitionType t);

    bool setPriority (char *r);
    void deletePriority ();

    void setDistributionType (DistributionType d);
    bool setDistributionExpression (char *r);
    void deleteDistributionExpression ();

    bool setWeight (char *r);
    void deleteWeight ();

    void setMarkingDependencyType (MarkingDependencyType m);

    void setPolicyType (PolicyType p);

    void setConcurrencyDegree (ConcurrencyDegree c);

    bool setGuard (char *r);
    void deleteGuard ();

    void getAttributes (TransitionType *type, char **prio,
			DistributionType *dist, char **distexpr,
			char **weight, MarkingDependencyType *markdep,
			PolicyType *policy, ConcurrencyDegree *concurrency,
			char **guard);
};


class Arc {

  private:
    char *id;
    char *modid, *procid;
    ArcType type;
    char *from;
    char *to;
    char *mult;

  public:
    char *longid;
    char *oldid;
    int section;

    Arc (char *modid, char *procid, char *name);
    ~Arc ();
    void display ();
    void display_comp (int section);
    char *getid ();

    void setArcType (ArcType p);

    bool setSource (char *s);
    void deleteSource ();

    bool setDestination (char *d);
    void deleteDestination ();

    bool setMultiplicity (char *multiplicity);
    void deleteMultiplicity ();

    void getAttributes (ArcType *p, char **source, char **destination,
			char **multiplicity);
};


class Port {

  private:
    char *id;
    char *modid, *procid;
    PortType type;
    char *mult;

  public:
    char *longid;
    char *oldid;
    int section;

    Port (char *modid, char *procid, char *name);
    ~Port ();
    void display ();
    void display_comp (int section);
    char *getid ();
    char *getmodid ();
    char *getprocid ();

    void setPortType (PortType p);
    PortType getPortType ();

    bool setMultiplicity (char *multiplicity);
    char *getMultiplicity ();
    void deleteMultiplicity ();
};


class Parameter {

  private:
    char *id;
    char *modid, *procid;
    char *expr;
    char *guessexpr;
    char type;

  public:
    char *longid;
    char *oldid;
    int section;

    Parameter (char *modid, char *procid, char *name, char typ);
    ~Parameter ();
    void display ();
    void display_param_comp (int section);
    void display_measparam_comp (int section);
    char *getid ();
    char *getmodid ();
    char *getprocid ();

    bool setParameterExpression (char *p);
    char *getParameterExpression ();
    void deleteParameterExpression ();

    bool setGuessExpression (char *m);
    char *getGuessExpression ();
    void deleteGuessExpression ();

    char getType ();
};


class Reward {

  private:
    char *id;
    char *modid, *procid;
    char *expr;
    char type;
    VisibleType visible;

  public:
    char *longid;
    char *oldid;
    int section;

    Reward (char *modid, char *procid, char *name, char typ);
    ~Reward ();
    void display ();
    void display_comp (int section);
    char *getid ();

    bool setRewardExpression (char *r);
    char *getRewardExpression ();
    void deleteRewardExpression ();

    void setVisibility (VisibleType v);
    VisibleType getVisibility ();

    char getType ();
};


class Measure {

  private:
    char *id;
    char *modid, *procid;
    char *expr;
    char type;
    VisibleType visible;

  public:
    char *longid;
    char *oldid;
    int section;

    Measure (char *modid, char *procid, char *name, char typ);
    ~Measure ();
    void display ();
    void display_comp (int section);
    char *getid ();

    bool setMeasureExpression (char *m);
    char *getMeasureExpression ();
    void deleteMeasureExpression ();

    void setVisibility (VisibleType v);
    VisibleType getVisibility ();

    char getType ();
};


class ProcessInst {

  private:
    char *id;
    char *modid, *procid;
    char *arrayanz;	       // Integer oder Id
    int arrayval;	       // Wert von arrayanz, wird beim checken gesetzt
    char *processtype;
    Vektor <GuessParameter> ExprList;

  public:
    char *longid;
    char *oldid;
    int section;

    ProcessInst (char *modid, char *procid, char *name);
    ~ProcessInst ();
    void display ();
    void display_comp (int section);
    char *getid ();
    char *getmodid ();

    bool setArrayanz (char *anz);
    void setArrayval (int val);
    bool setProcessType (char *type);
    void addParameterExpression (char *paramexpr, char *guessexpr);

    char *getProcessType ();
    char *getArrayanz ();
    int getArrayval ();
    char *getParameterExpression (int i);
    char *getParameterGuessExpression (int i);
};


#endif
