#ifndef VEKTORCC
#define VEKTORCC


#include <stdlib.h>	// NULL

#ifndef NULL
#define NULL 0
#endif

#define DEFAULTLISTSIZE 5
#define DEFAULTLISTINC 5



/**************************************************************************
 *                                                                        *
 *                            Liste als Array                             *
 *                             Deklaratation                              *
 *                                                                        *
 **************************************************************************/


template <class classtype>

class Vektor {
  private:

    struct ListElemT {
	classtype *data;
    };

    ListElemT *List;

    int nextfreeindex,
	actindex;

    int listsize,
	listinc;

    char *AllocNewList ();
    void ListIncreaseSize ();

  public:

    Vektor ();				// Konstruktor
    Vektor (int size);			// Konstruktor
    Vektor (int size, int inc);		// Konstruktor

    ~Vektor ();				// Destruktor

    void addElement (classtype *NewClass);
    void insertElementAt (classtype *NewClass, int position);

    classtype *readElementAt (int position);
    // Liest das Element aus der Liste aus, das an der angegebenden Position
    //  steht und liefert es zurueck.

    classtype *removeElementAt (int position);
    // Liest das Element aus der Liste aus, das an der angegebenden Position
    //  steht und liefert es zurueck. Das Element wird also nur aus der Liste
    //  ausgetragen, der Speicherplatz ist aber immer noch vorhanden!

    void deleteElement (char *elementId);
    // Sucht in der Liste nach dem Element mit der id elementId und loescht
    //  es aus der Liste und loescht das Element selber auch noch!

    int size ();

    classtype *searchId (char *id);
};


/**************************************************************************
 *                                                                        *
 *                            Liste als Array                             *
 *                             Implementation                             *
 *                                                                        *
 **************************************************************************/


template <class classtype>
//ListElemT *Vektor<classtype>::AllocNewList () {
char *Vektor<classtype>::AllocNewList () {
//printf("\n=== Allocating List %d-size",listsize);
  if (listsize <= 0) {
    cerr << "\nVektor::AllocNewList: Wrong listsize in Module vektor.cc\n";
    exit (0);
  }
  if (listinc <= 0) {
    cerr << "\nVektor::AllocNewList: Wrong listinc in Module vektor.cc\n";
    exit (0);
  }
  ListElemT *NewList= new ListElemT[listsize];
  if (NewList == NULL) {
    cerr << "\nVektor::AllocNewList: Error allocating memory for NewList ";
    cerr << "in Module vektor.cc\n";
    exit (0);
  }
  for (int i=0; i<listsize; i++)
    NewList[i].data= NULL;
  return ((char *) NewList);
}


template <class classtype>
Vektor<classtype>::Vektor () {			// Konstruktor
  listsize= DEFAULTLISTSIZE;
  listinc= DEFAULTLISTINC;
  nextfreeindex= 0;
  actindex= 0;
  List= (ListElemT *) AllocNewList();
}


template <class classtype>
Vektor<classtype>::Vektor (int size) {		// Konstruktor
  listsize= size;
  listinc= (size>>1);
  if (listinc <= 0)
    listinc= 1;
  nextfreeindex= 0;
  actindex= 0;
  List= (ListElemT *) AllocNewList();
}


template <class classtype>
Vektor<classtype>::Vektor (int size, int inc) {	// Konstruktor
  listsize= size;
  listinc= inc;
  nextfreeindex= 0;
  actindex= 0;
  List= (ListElemT *) AllocNewList();
}


template <class classtype>
Vektor<classtype>::~Vektor () {
  for (actindex=0; actindex < nextfreeindex; actindex++)
    delete (List[actindex].data);
  listsize= 0;
  nextfreeindex= 0;
  delete (List);
  List= NULL;
}


template <class classtype>
void Vektor<classtype>::ListIncreaseSize () {
//printf("\n=== Increase List from %d to %d-size",listsize,listsize+listinc);
  listsize+= listinc;			// Liste vergroessern
  ListElemT *NewList= (ListElemT *) AllocNewList();	// Neue Liste erzeugen
  for (int i=0; i<nextfreeindex; i++)
    NewList[i].data= List[i].data;	// Daten uebertragen
  delete (List);			// Alte Liste loeschen
  List= NewList;
//printf("\n=== Elemente umtragen und alte Liste loeschen");
}


template <class classtype>
void Vektor<classtype>::addElement (classtype *data) {
  if (nextfreeindex == listsize)
    ListIncreaseSize ();
  if (nextfreeindex < listsize) {
    List[nextfreeindex].data= data;
    nextfreeindex++;
  }
  else {
    cerr << "\nVektor::addElement: Wrong listsize in Module vektor.cc\n";
    exit (0);
  }
}


template <class classtype>
void Vektor<classtype>::insertElementAt (classtype *data, int position) {
  if ((position < 0) || (position > nextfreeindex)) {
    cerr << "\nVektor::insertElementAt: Wrong parameter position: ";
    cerr << position << ", max: " << nextfreeindex-1 << ".\n";
    exit (0);
  }
  if (nextfreeindex == listsize)
    ListIncreaseSize ();
  if (nextfreeindex < listsize) {
    for (int i=nextfreeindex; i>position; i--)
      List[i].data= List[i-1].data;
    List[position].data= data;
    nextfreeindex++;
  }
  else {
    cerr << "\nVektor::insertElementAt: Wrong listsize in Module vektor.cc\n";
    exit (0);
  }
}


template <class classtype>
classtype *Vektor<classtype>::readElementAt (int position) {
  if ((position < 0) || (position >= nextfreeindex)) {
    cerr << "\nVektor::readElementAt: Wrong parameter position: ";
    cerr << position << ", max: " << nextfreeindex-1 << ".\n";
    exit (0);
  }
  return (List[position].data);
}


template <class classtype>
classtype *Vektor<classtype>::removeElementAt (int position) {
  if ((position < 0) || (position >= nextfreeindex)) {
    cerr << "\nVektor::removeElementAt: Wrong parameter position: ";
    cerr << position << ", max: " << nextfreeindex-1 << ".\n";
    exit (0);
  }
  classtype *data= List[position].data;
  for (int i=position; i<nextfreeindex-1; i++)
    List[i].data= List[i+1].data;
  List[--nextfreeindex].data= NULL;
  return (data);
}


template <class classtype>
int Vektor<classtype>::size () {
  return (nextfreeindex);
}


template <class classtype>
classtype *Vektor<classtype>::searchId (char *id) {
  int i;
  for (i=0; i<size(); i++) {
    if (!strcmp((readElementAt (i))->getid (), id))
      return (readElementAt (i));
  }
  return (NULL);
}


template <class classtype>
void Vektor<classtype>::deleteElement (char *id) {
  int i;
  for (i=0; i<size(); i++) {
    if (!strcmp((readElementAt (i))->getid (), id))
      delete (removeElementAt (i));
  }
}



#endif
