#include "xbefore.h"
#include <X11/Intrinsic.h>
#include <Xm/CascadeB.h>
#include <Xm/PushB.h>
#include "xafter.h"

#include <iostream.h>

#include "module.h"
#include "main.h"
#include "editor.h"
#include "net.h"
#include "object.h"
#include "netcl.h"
#include "node.h"
#include "dialog.h"
#include "metrics.h"
#include "spnlstructure.h"
#include "wrapper.h"

#ifndef NoExplicitInstanciation
#define NoExplicitInstanciation
#include "dialog.cc"
#include "wrapper.cc"
DIAINSTANCIATE(SPNLControl,bool)
#endif

class SPNLControl;
class SPNLControl: public Module
{
  SPNLStructure plan;
  Dialog<SPNLControl,bool> dialog;
  StructP setup;

public:

  SPNLControl(Net& n): Module(n,"SPNL Control"),plan(),dialog(this) {}
  bool Init();
  bool Event(Editor& ed,Page& pg,EventInfo& ev,bool before);
  bool Menu(Editor& ed,Page& pg,Ident& name,bool before);
  bool Open(Editor& ed,Page& pg);
  bool Change(Editor&,Page&,Page&) {return false;};
  void Refresh(Editor* ed,Page& pg);
};


// ******** Funktionsdefinitionen **********

bool SPNLControl::Init()
{ // Ist nur fuer SPNL zustaendig!
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Init()\n";
#endif
  if (net.Class()->Name()==Ident("SPNL")) 
     return true;
  else
    return false;
}

bool SPNLControl::Open(Editor& ed,Page& pg)
{
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Open()\n";
#endif
  WrapperID dummy;
  Widget ent=ed.CreateEntry(name,xmCascadeButtonWidgetClass,ed.Mods(),NULL, dummy);
  ed.CreateEntries(ed.CreatePulldown("SPNL-Popup",ent),xmPushButtonWidgetClass,
		   NULL,"SPNLSyntaxCheck SPNLCompile SPNLMake _ SPNLShowErrors SPNLShowCompiledNet SPNLShowList _ SPNLFormat SPNLMerge");

  //cout<<"initialize\n";
  ed.Message("Initializing SPNL-module");
  if (!plan.Get(ed,pg)) { // Initialisierung der Strukturliste
    cerr<<"SPNLControl.cc: structure plan couldn't be initialized correctly!\n";
    exit(1);
  }
  ed.Message("");
  
  return false;
}

void SPNLControl::Refresh(Editor* ed,Page& pg)
{
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Refresh()\n";
#endif
  plan.Add(ed,pg);
}

bool SPNLControl::Event(Editor& ed,Page& pg,EventInfo& ev,bool before)
{
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event()\n";
#endif
  EditorSettings set = ed.Settings();
  Display* dpy = ed.Dpy();

  //ed.ButtonState("Cut Copy Paste Add",false);
  
  if (before) // before == true
  {
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event:before==true\n";
#endif
    //cout << pg.CurrAction() << '\n';
    switch (ev.event) {
      case EventInit:
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event:EventInit\n";
#endif
	if(!bool(pg.CurrAction()))
	  return false;
	pg.SetCurrOb(dpy,ev.pos); // Aktuelles Objekt unter Maus bestimmen
	switch (ev.button) {
	  case ButtonUser:
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event:ButtonUser\n";
#endif
	    if (set.rclick=="Undoes") {
	      plan.Clear();
	      //cout << "plan geloescht\n";
	      //plan.Print();
	      return false; // Undo()
	    }
	    // else fall through
	  case ButtonSelect:
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event:ButtonSelect\n";
#endif
	    if (ev.shift) {
	      cout<<"SHIFT!!!!\n";
	    }
 
	    if (pg.GetCurrOb()) { // Aktuelles Objekt vorhanden?
	      
	      if (ev.shift) {
		return false;
	      }
		
	      if (!pg.GetCurrOb()->Selected()) {
		pg.Deselect();
		pg.GetCurrOb()->Select(true);
	      }

	      if (ev.button!=ButtonSelect || !pg.SubMoving(ed,ev)) {
		if (ev.button==ButtonSelect) {
		  plan.PrepMove(pg.GetCurrOb());
		  pg.ForbidDeselection();
		  return false; // Moving()
		}
	        else {
		  if (set.rclick=="Rotates") {
		    plan.PrepRotate();
		    pg.ForbidDeselection();
		    return false; // Rotating()
		  }
		  else if (set.rclick=="Scales") {
		    plan.PrepScale();
		    pg.ForbidDeselection();
		    return false; // Scaling()
		  }
		  else if (set.rclick=="Removes") {
		    plan.Remove();
		    //plan.Print();
		    pg.ForbidDeselection();
		    return false; // Remove()
		  }
		}
	      }
	      else return false; // Page::Event geht nochmal nach SubMoving() (war schon drin)
	    }
	    else {
	      return false;
	    }
	    return false;
	  case ButtonCreate:
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event:ButtonCreate\n";
#endif
	    return false;
	}
	return false;
      case EventDouble: // Aufruf zum Formatieren
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event:EventDouble\n";
#endif

        if(!bool(pg.CurrAction()))
	  return false;
	pg.SetCurrOb(dpy,ev.pos);

/*
// neu:
        if ((ev.button == ButtonSelect) &&
	    (pg.GetCurrOb()->Class()->Name() == "TexPart")) {
	  Text contense;
	  contense+= (char*)Text("testtesttesttest\nnext line");
	  setup= new Struct ((char*) contense);
	  dialog.Create ("SPNLTextFenster", "SCHNULLER", ed.Shell (), false, DialogCB, "Ok Apply Cancel");
	  return (true);
	}
// alt:
*/

	if ((!pg.GetCurrOb())&&(ev.button==ButtonUser)) {
	  ed.Message("Formating");
	  pg.GetNet()->UndoCopy();
	  plan.FormatEntries(dpy);
	  ed.Update();
	  ed.Message("");
	  //cout << "Format done\n";
	  return true;
	}
	return false;
      case EventMotion:
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event:EventMotion\n";
#endif
	if (pg.CurrAction()=="Drag") {
	  plan.Drag(pg);
	  pg.ForbidDeselection();
	}
	return false;
      case EventExit:
      case EventQuickExit:
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event:EventExit,EventQuickExit\n";
#endif
	if (pg.CurrAction()=="Move") {
          pg.ForbidMelting();
	  return false;
	}
    }
  }
  else { // before == false
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event:before==false\n";
#endif
    switch (ev.event) {
      case EventInit:
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event:EventInit\n";
#endif
	switch (ev.button) {
	  case ButtonSelect:
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event:ButtonSelect\n";
#endif
	    break;
	  case ButtonCreate:
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event:ButtonCreate\n";
#endif
            break;
	  case ButtonUser:
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event:ButtonUser\n";
#endif
	    if (set.rclick=="Undoes") {
	      Page* new_pg = ed.GetNet()->Prime(); // weil pg noch nicht akt. in net->Event
	      plan.Get(ed,new_pg);
	      //cout << "plan wiederhergestellt\n";
	      //plan.Print();
	    }
	    break;
	}
	return false;
      case EventDouble:
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event:EventDouble\n";
#endif
	return false;
      case EventMotion:
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event:EventMotion\n";
#endif
	return false;
      case EventExit:
      case EventQuickExit:
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event:EventExit,QuickExit\n";
#endif
	switch (ev.button)
	{
	  case ButtonSelect:
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event:ButtonSelect\n";
#endif
            break;
	  case ButtonCreate:
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event:ButtonCreate\n";
#endif
	    break;
	  case ButtonUser:
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Event:ButtonUser\n";
#endif
	    if (set.rclick=="Scales") {
	      plan.FinishScale();
	      plan.Evaluate(ed,pg);
	      return false;
	    }
	    if (set.rclick=="Removes") {
	      plan.Get(ed,pg);
	      return false;
	    }
	    if (set.rclick=="Pastes") {
	      plan.Paste(ed,pg);
	      return false;
	    }
	    break;
	}
	plan.Evaluate(ed,pg);
	return false;
    }
  }
  return false; // Wegen Warning
}


bool SPNLControl::Menu(Editor& ed,Page& pg,Ident& name,bool before)
{
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Menu()\n";
#endif
  if (before) { // before == true
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Menu:before==true\n";
#endif
    //cout << name << '\n';
    if (name=="SPNLShowList") {
      ed.Message("Showing SPNL structure list");
      plan.ShowList(ed);
      ed.Message("");
      return true;
    }
    if (name=="SPNLSyntaxCheck") {
      ed.Message("Checking SPNL-Syntax");
      plan.CheckSyntax(ed);
      ed.Message("");
      return true;
    }
    if (name=="SPNLCompile") {
      ed.Message ("Checking SPNL-Syntax on altered Files and compile complete SPNL-net");
      plan.BuildNet (ed, false);
      ed.Message ("");
      return true;
    }
    if (name=="SPNLMake") {
      ed.Message ("Checking SPNL-Syntax on all Files and compile complete SPNL-net");
      plan.BuildNet (ed, true);
      ed.Message ("");
      return true;
    }
    if (name=="SPNLShowCompiledNet") {
      ed.Message ("Showing .spnl_comp-file");
      plan.ShowSPNLcomp (ed);
      ed.Message ("");
      return true;
    }
    if (name=="SPNLShowErrors") {
      ed.Message ("Showing .out or .spnl_errors-file");
      plan.ShowErrorFile (ed);
      ed.Message ("");
      return true;
    }
    if (name=="SPNLFormat") {
      ed.Message("Formating");
      pg.GetNet()->UndoCopy();
      plan.FormatEntries(ed.Dpy());
      ed.Update();
      ed.Message("");
      return true;
    }
    if (name=="SPNLMerge") {
      if (plan.CheckForMerge()) {
      ed.Message("Merging");
      pg.GetNet()->UndoCopy();
      plan.Merge(pg);
      plan.Evaluate(ed,pg);
      ed.Message("");
      }
      else {
	if (!dialog)
	  dialog.OpenMsg("SPNLMessage","SPNL Control",ed.Shell(),XmDIALOG_ERROR,
			 "Sorry, for merging only textual parts\nshould be selected!\nAnd you have to select at least\ntwo of them!",false);
      }
      return true;
    }
    if (name=="Remove") {
      plan.Remove();
      //plan.Print();
      return false;
    }
    if (name=="Cut" || name=="Copy") {
      if (plan.CheckForCutNCopy()) { 
	if (name=="Cut") {
	  plan.Remove();
	  //plan.Print();
	}
	return false;
      }
      else {
	if (!dialog)
	  dialog.OpenMsg("SPNLMessage","SPNL Control",ed.Shell(),XmDIALOG_ERROR,
			 "Sorry, the current set\nof selected objects\nis not valid for\nCut or Copy!\n",false);
	return true;
      }
    }
    if (name=="Undo") {
      plan.Clear();
      //cout << "plan geloescht\n";
      //plan.Print();
      return false;
    }
  }
  else { // before == false
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Menu:before==false\n";
#endif
    if (name=="Undo") {
      Page* new_pg = ed.GetNet()->Prime(); // weil pg noch nicht akt. in net->Event
      plan.Get(ed,new_pg);
      //cout << "plan wiederhergestellt\n";
      //plan.Print();
      return false;
    }
    if (name=="Paste") {
      plan.Paste(ed,pg,true);
      return false;
    }
    if (name=="Remove" || name=="Cut") {
      plan.Get(ed,pg);
      return false;
    }
    if (name=="Save") {  // || (name=="SaveAs") brings nichts, siehe net.cc
      plan.ConvertAndSaveStructure(ed);
#ifdef DEBUG_SPNL
cout << "\n*****.spnl und .numbered generiert" << flush;
#endif
      return (false);
    }
  }
  return false;  
}



MODULE_NOTIFY(SPNLControl)

