#include <sys/types.h>
#include <sys/wait.h>
#include <sys/file.h>
#include <math.h>
#include <fcntl.h>      // open
#include "spnlstructure.h"
#include "main.h"

//#define DEBUG_SPNL

extern "C" {
  int fork();
}

bool SPNLStructure::Insert(Display* dpy,Node* no)
{
  // Fuegt ein Element bzw. Subelement ein, dazu werden zuerst seine Masse ermittelt.
  // Elemente werden einsortiert.
  // Subelemente (Knoten) werden angehaengt, sofern sie innerhalb eines Frames liegen.
  
  //cout << "::Insert1\n";
  Ident name = no->Class()->Name();
  Metrics met = no->GetMetrics(dpy);
  
  Iterator<SPNLElement*> iter(structure_list);
  
  if (name=="TexPart" || name=="Frame")
  {
    for (SPNLElement* elem=iter;elem;elem=++iter) {
      if (elem->metrics.IsLower(met)) {
	// Vor elem einf"ugen
	SPNLElement* insert = new SPNLElement(name,no,met);
	structure_list.Insert(insert,structure_list.Index(elem));
	return true;
      }
    }
    // ganz Unten => Anh"angen
    SPNLElement* insert = new SPNLElement(name,no,met);
    structure_list.Append(insert);
    return true;
  }
  else
  {
    for (SPNLElement* elem=iter;elem;elem=++iter)
      if(elem->type=="Frame") {
	if(elem->metrics.Within(met)) { // Subelement liegt voll innerhalb dieses Frames
	  FrameElement* insert = new FrameElement(no,(no->Mid())-(elem->element->Mid()));
	  elem->subelements.Append(insert);
	  return true;
	}
	else
	  if (elem->metrics.Within(no->Mid())) { // Nur Mittelpunkt liegt innerhalb dieses Frames
	    no->SetPos(elem->metrics.ForceIn(no->Mid(),met)); // Subelement ganz reinschieben
	    if(!elem->metrics.Within(no->GetMetrics(dpy)))
	      cout << no->Name() << " is still not completly inside its frame\n";
	    FrameElement* insert = new FrameElement(no,(no->Mid())-(elem->element->Mid()));
	    elem->subelements.Append(insert);
	    return true;
	  }
      }
    return false; // Weder das ganze Subelement noch dessen Mittelpunkt lagen in einem Frame
  }
}

bool SPNLStructure::Insert(Display*,Connector* cn)
{
  // Fuegt Subelemente (Kanten) ein. Es wird ueberprueft, ob Start- u. Endknoten
  // im gleichen Frame liegen. Ausserdem werden ausserhalb liegende Zwischenpositionen
  // in den Frame geschoben.

  //cout << "::Insert2\n";
  Iterator<SPNLElement*> iter(structure_list);
  Node* in = cn->GetInput();
  Node* out = cn->GetOutput();
  for (SPNLElement* elem=iter;elem;elem=++iter) {
    int found = 0;
    Iterator<FrameElement*> fiter(elem->subelements);
    for (FrameElement* fe=fiter;fe;fe=++fiter)
      if (fe->element==in || fe->element==out)
	++found;
    if (found==2) { // Start- u. Endknoten liegen in einem Frame
      Iterator<Position*> piter(cn->Positions());
      for (Position* p=piter;p;p=++piter) // Zwischenpositionen ueberpruefen
	if (p!=piter.First() && p!=piter.Last() && !elem->metrics.Within(*p)) {
	  *p = elem->metrics.ForceIn(*p); // Zwischenposition in Frame schieben
	  if(!elem->metrics.Within(*p))
	    cout << "Point of "<< cn->Name() << " is still not completly inside its frame\n";
 	}
      FrameElement* insert = new FrameElement(cn,(cn->Mid())-(elem->element->Mid()));
      elem->subelements.Append(insert);
      return true;
    }
  }
  // Connector ist unzul"assig, da "uber Framegrenzen hinweg
  return false;
}

bool SPNLStructure::IsIn(Object* ob)
{
  // Uberprueft, ob das Objekt "ob" in der Strukturliste vorhanden ist.
  
  //cout << "::IsIn\n";
  Ident name = ob->Class()->Name();
  Iterator<SPNLElement*> iter(structure_list);

  if ((name==Ident("TexPart"))||(name==Ident("Frame"))) 
    for (SPNLElement* elem=iter;elem;elem=++iter) {
      if (elem->element==ob)
	return true;
    }
  else
    for (SPNLElement* elem=iter;elem;elem=++iter) {
      Iterator<FrameElement*> fiter(elem->subelements);
      for (FrameElement* fe=fiter;fe;fe=++fiter)
	if (fe->element==ob)
	  return true;
    }
  return false;
}

bool SPNLStructure::UpdateEntries(Display* dpy)
{
  // Ermittelt die Masse der selektierten(!) Elemente
  // Stellt, falls noetig, fuer die selektierten Elemente den Mindestabstand
  // zum Rand her.
  // Ueberprueft die Subelemente der Frames
  
  //cout << "::UpdateEntries\n";
  Position pos;
  Iterator<SPNLElement*> iter(structure_list);
  for (SPNLElement* elem=iter;elem;elem=++iter) {
    //cout << elem->element->Name() << '\n';
    if (elem->element->Selected()) {  // Selektiert
      //cout << "selected\n";
      elem->metrics = elem->element->GetMetrics(dpy);
      if (elem->type==Ident("Frame")) { // Frame
	if (!UpdateSubelements(dpy,elem))
	  return false;
	pos = Position(FRAMEMARGIN-elem->metrics.x_beg,0.0);
	if (pos.x > 0.0)
	  MoveEntry(elem,pos);
      }
      else { // TexPart
	pos = elem->element->Mid();
	pos.x=TEXTMARGIN;
	elem->element->SetPos(pos);
      }
    }
    else  // Nicht selektiert
      if (elem->type==Ident("Frame"))
	if (!UpdateSubelements(dpy,elem))
	  return false;
  }
  return true;
}

bool SPNLStructure::UpdateSubelements(Display* dpy,SPNLElement* elem)
{
  // Ermittelt die Masse der selektierten(!) Subelemente eines Frames
  // Gibt false zurueck, wenn ein selektiertes Subelement nicht
  // vollstaendig im Frame liegt.
  
  //cout << "::UpdateSubElements\n";
  Iterator<FrameElement*> fiter(elem->subelements);
  for (FrameElement* fe=fiter;fe;fe=++fiter)
    if (fe->element->Selected) {
      Metrics met(fe->element->GetMetrics(dpy));
      if (!elem->metrics.Within(met)) {
	//cout << fe->element->Name() << " is out of area\n";
	//elem->metrics.Print();
	//met.Print();
	return false;
      }
      fe->relative = fe->element->Mid() - elem->element->Mid() ;
    }
  return true;
}

void SPNLStructure::ShiftEntries(Display*)
{
  // Shiftet die Eintraege vertikal, wenn der Mindestabstand nicht vorhanden ist.
  // Die Masse werden vorher nicht neu ermittelt.
  
  //cout << "::ShiftEntries\n";
  Iterator<SPNLElement*> iter(structure_list);
  SPNLElement* elem;
  SPNLElement* next;
  double shiftsize;
  for (elem=next=iter;(elem&&(next=++iter));elem=next) {
    shiftsize=elem->metrics.Overlap(next->metrics);
    if (shiftsize>0) 
      MoveEntry(next,Position(0.0,shiftsize));
  }
}

void SPNLStructure::SortEntries() // mit bubble-sort
{
  // Sortiert die Elemente nach aufsteigender Reihenfolge von y_begin.
  // Die Masse werden vorher nicht(!) neu ermittelt.
  
  //cout << "::SortEntries\n";
  bool changed;
  do {
    changed = false;
    Iterator<SPNLElement*> iter(structure_list);
    SPNLElement* elem;
    SPNLElement* next;
    for (elem=next=iter;(elem&&(next=++iter));elem=next) 
      if (elem->metrics.y_beg > next->metrics.y_beg) {
	  SPNLElement change(next);
	  *next = *elem;
	  *elem = change;
	  changed = true;
	}
  } while (changed);
}

void SPNLStructure::MoveEntry(SPNLElement* elem,Position rel)
{
  //cout << "::MoveEntry\n";
  elem->element->Move(rel);
  elem->metrics.Move(rel);
  MoveSubelements(elem,rel);
}

void SPNLStructure::MoveSubelements(SPNLElement* elem,Position rel)
{
  //cout << "::MoveSubelements\n";
  SelectSubs(elem,true); // Damit die Kanten vernuenftig verschoben werden bei element->Move()
  Iterator<FrameElement*> fiter(elem->subelements);
  for (FrameElement* fe=fiter;fe;fe=++fiter)
    fe->element->Move(rel);
  SelectSubs(elem,elem->element->Selected());
}

void SPNLStructure::AlignEntry(Display*,SPNLElement* elem)
{
  //cout << "::AlignEntry\n";
  if (elem->type==Ident("Frame")) {
    MoveEntry(elem,Position(FRAMEMARGIN-elem->metrics.x_beg,0.0));
  }
  else 
    elem->element->SetPos(Position(double(TEXTMARGIN),elem->element->Mid().y));
}

void SPNLStructure::FormatEntries(Display* dpy)
{
  //cout << "::FormatEntries\n";
  double y_beg_new = 10;
  Iterator<SPNLElement*> iter(structure_list);
  for (SPNLElement* elem=iter;elem;elem=++iter) {
    elem->metrics = elem->element->GetMetrics(dpy);
    AlignEntry(dpy,elem);    
    MoveEntry(elem,Position(0.0,y_beg_new - elem->metrics.y_beg));
    y_beg_new = elem->metrics.y_end + MINSPACE;
  }  
}

void SPNLStructure::Print()
{
  //cout << "::Print\n";
  if (bool(structure_list)) {
    cout<<"\nSPNLStructure\n_____________\n";
    Iterator<SPNLElement*> iter(structure_list);
    for (SPNLElement* elem=iter;elem;elem=++iter) {
      cout<<"Type:"<<elem->type<<" Name:"<<elem->element->Name()<<'\n';
      elem->metrics.Print();
      if(bool(elem->subelements)) {
	cout<<"subelements:\n";
	Iterator<FrameElement*> fiter(elem->subelements);
	for (FrameElement* fe=fiter;fe;fe=++fiter)
	  cout<<fe->element->Name()<<':'<<fe->relative.x<<':'<<fe->relative.y<<'\n';
      }
      cout<<'\n';
    }
  }
}

void SPNLStructure::SelectSubs(SPNLElement* elem,bool select)
{
  //cout << "::SelectSubs\n";
  Iterator<FrameElement*> fiter(elem->subelements);
  for (FrameElement* fe=fiter;fe;fe=++fiter) 
    fe->element->Select(select);
}

void SPNLStructure::Remove()
{
  //cout << "::Remove\n";
  Iterator<SPNLElement*> iter(structure_list);
  for (SPNLElement* elem=iter;elem;elem=++iter)
    if (elem->element->Selected()) { // Selektiert
      if (elem->type=="Frame") { // Selektierter Frame
	bool sub_sel = false;
	bool sub_desel = false;
	Iterator<FrameElement*> fiter(elem->subelements);
	for (FrameElement* fe=fiter;fe;fe=++fiter) 
	  if (fe->element->Selected())
	    sub_sel = true;
	  else
	    sub_desel = true;
	if (sub_sel && sub_desel) // Selektierte Subelemente vorhanden
	  elem->element->Select(false);
	else if (!sub_sel) // kein Subelement selektiert
	  SelectSubs(elem,true);
      }
    }
  Clear();
}

void SPNLStructure::PrepScale()
{
  //cout << "::PrepScale\n";
  Iterator<SPNLElement*> iter(structure_list);
  for (SPNLElement* elem=iter;elem;elem=++iter)
    if (elem->type=="Frame" && elem->element->Selected())
      SelectSubs(elem,false);
}

void SPNLStructure::FinishScale()
{
  //cout << "::FinishScale\n";
  Iterator<SPNLElement*> iter(structure_list);
  for (SPNLElement* elem=iter;elem;elem=++iter)
    if (elem->type=="Frame" && elem->element->Selected())
      SelectSubs(elem,true);
}

void SPNLStructure::PrepRotate()
{
  //cout << "::PrepRotate\n";
  Iterator<SPNLElement*> iter(structure_list);
  for (SPNLElement* elem=iter;elem;elem=++iter)
    if (elem->type=="Frame")
      elem->element->Select(false);
}

void SPNLStructure::PrepMove(Object* ob)
{
   //cout << "::PrepMove\n";
 Iterator<SPNLElement*> iter(structure_list);
  for (SPNLElement* elem=iter;elem;elem=++iter)
    if (elem->element==ob)
      SelectSubs(elem,true);
}

void SPNLStructure::PrepCutNCopy(SPNLElement* frame)
{
  // Damit diese Aktionen vernuenftig vollzogen werden koennen,
  // muessen alle Kanten, bei denen nicht Anfangs- und Endknoten selektiert
  // sind, deselektiert werden.

  Iterator<FrameElement*> fiter(frame->subelements);
  for (FrameElement* fe=fiter;fe;fe=++fiter) {
    Object* ob = fe->element;
    if (ob->Selected() && !ob->IsNode()) {// Selektierte Kante
      Connector* co = (Connector*)(ob); // Ist OK, da sicher, dass "ob" Kante ist.
      if (!co->GetInput()->Selected() || !co->GetOutput()->Selected())
	ob->Select(false);
    }
  }
}

bool SPNLStructure::DragboxFrameInside(Page& pg,SPNLElement* elem)
{
  //cout << "::DragboxFrameInside\n";
  if (pg.DragboxInside(Position(elem->metrics.x_beg,elem->metrics.y_beg)))
    return true;
  if (pg.DragboxInside(Position(elem->metrics.x_end,elem->metrics.y_beg)))
    return true;
  if (pg.DragboxInside(Position(elem->metrics.x_beg,elem->metrics.y_end)))
    return true;
  if (pg.DragboxInside(Position(elem->metrics.x_end,elem->metrics.y_end)))
    return true;
  return false;
}

void SPNLStructure::Drag(Page& pg)
{
  //cout << "::Drag\n";
  Iterator<SPNLElement*> iter(structure_list);
  for (SPNLElement* elem=iter;elem;elem=++iter) 
    if (elem->type=="TexPart")
      elem->element->Select(pg.DragboxInside(elem->element->Mid()));
    else {
      if (DragboxFrameInside(pg,elem)) {
	bool sub_out = false;
	bool sub_in = false;
	Iterator<FrameElement*> fiter(elem->subelements);
	for (FrameElement* fe=fiter;fe;fe=++fiter) 
	  if (pg.DragboxInside(fe->element->Mid())) {
	    sub_in = true;
	    fe->element->Select(true);
	  }
	  else {
	    sub_out = true;
	    fe->element->Select(false);
	  }
	if (!sub_in)
	  SelectSubs(elem,true);
	elem->element->Select((sub_in&&sub_out)?false:true);
      }
      else {
	elem->element->Select(false);
	Iterator<FrameElement*> fiter(elem->subelements);
	for (FrameElement* fe=fiter;fe;fe=++fiter)
	  fe->element->Select(pg.DragboxInside(fe->element->Mid()));
      }
    }
}

bool SPNLStructure::Evaluate(Editor& ed,Page& pg)
{
  //cout << "::Evaluate\n";
  Display* dpy = ed.Dpy();
  if (!Get(ed,pg)) {
    Undo(ed,pg);
    //cout << "Get failed!\n";
    return false;
  }
  if (!UpdateEntries(dpy)) {
    //cout << "UpdateEntries failed!\n";
    Undo(ed,pg);
    return false;
  }
  SortEntries();
  ShiftEntries(dpy);
  ed.Update();
  //Print();
  return true;
}

void SPNLStructure::Clear()
{
#ifdef DEBUG_SPNL
  cout<<"SPNLControl::Clear()\n";
#endif
  Iterator<SPNLElement*> iter(structure_list);
  for (SPNLElement* elem=iter;elem;elem=++iter) {
    Iterator<FrameElement*> fiter(elem->subelements);
    for (FrameElement* fe=fiter;fe;fe=++fiter)
	delete fe;
    elem->subelements.Clear();
      delete elem;
  }
  structure_list.Clear();
}

void SPNLStructure::Undo(Editor& ed,Page& pg)
{
  //cout << "::Undo\n";
  Clear();
  pg.GetNet()->Undo();
  Page* new_pg = ed.GetNet()->Prime(); // weil pg noch nicht akt. in net->Event
  Get(ed,new_pg);
  ed.Update();
  pg.GetNet()->UndoCopy();
  //Print();
}

void SPNLStructure::Paste(Editor& ed,Page& pg,bool moveback)
{
#ifdef DEBUG_SPNL
  cout<<"SPNLStructure::Paste()\n";
#endif
  if (moveback) {
#ifdef DEBUG_SPNL
  cout<<"SPNLStructure::Paste:moveback==true\n";
#endif
    bool move = false;
    Iterator<Object*> oiter = pg.Objects(); 
    for (Object* ob=oiter; ob; ob=++oiter)
      if (ob->Selected() && (ob->Class()->Name()=="Frame" || ob->Class()->Name()=="TexPart")) {
	move = true;
	break; // Sobald irgendein Frame oder TexPart selected ist, -> alle selektierten Objekte
	       // zurueckschieben. Nicht, wenn nur Subelemente gepastet, die bleiben so.
      }
    if (move)
      for (Object* ob=oiter; ob; ob=++oiter)
	if (ob->Selected())
	  ob->Move(Position(-10,-10)); // Selektierte (=gepastete) Objekte zurueckschieben.
  }
  Display* dpy = ed.Dpy();
  SPNLStructure paste; // Zwischenliste, wo die gepasteten Objekte eingetragen werden
  Iterator<Node*> niter = pg.Nodes(); 
  for (Node* no=niter; no; no=++niter) { // Erstmal neue Frames und TexParts eintragen
    Ident name = no->Class()->Name();
    if ((name=="Frame" || name=="TexPart") && !IsIn(no))
      paste.Insert(dpy,no);
  }
  if (bool(paste.GetList())) { // neue Frames und TexParts sind vorhanden
#ifdef DEBUG_SPNL
  cout<<"SPNLStructure::Paste:Frames und TexParts vorhanden\n";
#endif
    niter = pg.Nodes();
    for (Node* no=niter; no; no=++niter) // restliche neue Knoten (Subelemente) eintragen
      if (!IsIn(no) && !paste.IsIn(no))
	paste.Insert(dpy,no);
    Iterator<Connector*> citer = pg.Connectors(); 
    for (Connector* cn=citer; cn; cn=++citer) // neue Kanten eintragen 
      if (!IsIn(cn))
	paste.Insert(dpy,cn);
    //paste.Print();
    Insert(paste); // Zwischenliste an richtiger Stelle einfuegen und loeschen!
    //Print();
    ShiftEntries(dpy); // Eintraege ordnungsgemaess verschieben
    ed.Update();
    //Print();
  }
  else { // Nur Subelemente
#ifdef DEBUG_SPNL
  cout<<"SPNLStructure::Paste:Nur Subelemente vorhanden\n";
#endif
    Evaluate(ed,pg);
  }
}

void SPNLStructure::Add(Editor* ed,Page& pg)
{
  // Wird aufgerufen nach Attributaenderung oder bei Addition eines Netzes
#ifdef DEBUG_SPNL
  cout<<"SPNLStructure::Add()\n";
#endif
  Display* dpy = ed->Dpy();
  SPNLStructure paste;
  Iterator<Node*> niter = pg.Nodes(); 
  for (Node* no=niter; no; no=++niter) { // Erstmal neue Frames und TexParts eintragen
    Ident name = no->Class()->Name();
    if ((name=="Frame" || name=="TexPart") && !IsIn(no))
      paste.Insert(dpy,no);
  }
  if (bool(paste.GetList())) { // Frames und TexParts
    niter = pg.Nodes();
    for (Node* no=niter; no; no=++niter) // restliche Knoten (Subelemente) eintragen
      if (!IsIn(no) && !paste.IsIn(no))
	paste.Insert(dpy,no);
    Iterator<Connector*> citer = pg.Connectors(); 
    for (Connector* cn=citer; cn; cn=++citer) // Kanten eintragen 
      if (!IsIn(cn))
	paste.Insert(dpy,cn);
    //paste.Print();
    Append(paste); // Anhaengen an structure_list
    //Print();
    
  }
  ShiftEntries(dpy);
  ed->Update();
  //Print();
}

void SPNLStructure::Insert(SPNLStructure& paste)
{
#ifdef DEBUG_SPNL
  cout<<"SPNLStruct::Insert(paste)\n";
#endif
  int pos = 0;
  Iterator<SPNLElement*> piter(paste.GetList());
  SPNLElement* pelem = piter;
  Iterator<SPNLElement*> iter(structure_list);
  SPNLElement* elem = iter;

  for (;elem;elem=++iter)
    if (elem->metrics.y_beg > pelem->metrics.y_beg) // Gucken an welche Stelle in "plan" die
                                                    // Komplette Liste "paste" eingefuegt werden muss.
      break;
    else
      ++pos;
  
  for (;pelem;pelem=++piter) {
    structure_list.Insert(pelem,pos); // Element (Frame bzw. TexPart) einfuegen
    ++pos;
    }
  
  paste.GetList().Clear(); // Ganz wichtig, da alloziierte Elemente an structure_list gehaengt werden!!
}

void SPNLStructure::Append(SPNLStructure& paste)
{
#ifdef DEBUG_SPNL
  cout<<"SPNLStruct::Append()\n";
#endif
  Iterator<SPNLElement*> piter(paste.GetList());
  for (SPNLElement* pelem=piter;pelem;pelem=++piter)
    structure_list.Append(pelem); // Element (Frame bzw. TexPart) einfuegen
  
  paste.GetList().Clear(); // Ganz wichtig, da alloziierte Elemente an structure_list gehaengt werden!!
}

bool SPNLStructure::Get(Editor& ed,Page& pg)
{
  //cout << "::Get1\n";
  Display* dpy = ed.Dpy();
  Iterator<Node*> niter = pg.Nodes();
  for (Node* no=niter; no; no=++niter) {
    //cout << no->Name() << '\n';
    if (!IsIn(no)) 
      if (!Insert(dpy,no)) {
	//cout << "insertion failed\n";
	return false;
      }
    //else
    //cout << "succesfully inserted\n";
    //else
    //cout << "was already in\n";
  }
  Iterator<Connector*> citer = pg.Connectors();
  for (Connector* cn=citer; cn; cn=++citer) {
    //cout << cn->Name() << '\n';
    if (!IsIn(cn))
      if (!Insert(dpy,cn)) {
	//cout << "insertion failed\n";
	return false;
      }
    //else
    //cout << "succesfully inserted\n";
    //else
    //cout << "was already in\n";
  }
  return true;
}

bool SPNLStructure::Get(Editor& ed,Page* pg)
{
  //cout << "::Get2\n";
  Display* dpy = ed.Dpy();
  Iterator<Node*> niter = pg->Nodes();
  for (Node* no=niter; no; no=++niter) {
    //cout << no->Name() << '\n';
    if (!IsIn(no)) 
      if (!Insert(dpy,no)) {
	//cout << "insertion failed\n";
	return false;
      }
    //else
    //cout << "succesfully inserted\n";
    //else
    //cout << "was already in\n";
  }
  Iterator<Connector*> citer = pg->Connectors();
  for (Connector* cn=citer; cn; cn=++citer) {
    //cout << cn->Name() << '\n';
    if (!IsIn(cn)) 
      if (!Insert(dpy,cn)) {
	//cout << "insertion failed\n";
	return false;
      }
    //else
    //cout << "succesfully inserted\n";
    //else
    //cout << "was already in\n";
  }
  return true;
}

bool SPNLStructure::CheckForCutNCopy()
{
  // Es duerfen entweder nur Elemente selektiert sein,
  // oder Subelemente "eines" Frames!
  //cout << "::CheckForCutNCopy\n";
  SPNLElement* frame_to_prep;
  bool elem_sel = false;
  bool subelem_sel = false;
  //Print();
  Iterator<SPNLElement*> iter(structure_list);
  for (SPNLElement* elem=iter;elem;elem=++iter) 
    if (elem->element->Selected()) 
      elem_sel = true;
    else {
      if (elem->type=="Frame" && elem->subelements.Length()) { // Frame, der Subelemente enthaelt
      Iterator<FrameElement*> fiter(elem->subelements);
      for (FrameElement* fe=fiter;fe;fe=++fiter) 
	if (fe->element->Selected())
	  if (subelem_sel) // Woanders schon Subelemente selektiert
	    return false;
	  else {
	    subelem_sel = true;
	    frame_to_prep = elem;
	    break;
	  }
      }
    }
  if (elem_sel && subelem_sel)
    return false;
  else {
    if (subelem_sel)
      PrepCutNCopy(frame_to_prep);
    return true;
  }
}

bool SPNLStructure::CheckForMerge()
{
  //cout << "::CheckForMerge\n";
  int sel = 0;
  Iterator<SPNLElement*> iter(structure_list);
  for (SPNLElement* elem=iter;elem;elem=++iter)
    if (elem->type=="Frame")
      if (elem->element->Selected())
	return false;
      else {
	Iterator<FrameElement*> fiter(elem->subelements);
	for (FrameElement* fe=fiter;fe;fe=++fiter) 
	  if (fe->element->Selected())
	    return false;
      }
    else if(elem->element->Selected())
      ++sel;
  if (sel > 1)
    return true;
  else
    return false;
}

void SPNLStructure::Merge(Page& pg)
{
  //cout << "::Merge\n";
  SPNLElement* first;
  
  bool first_found = false;
  ValueP first_val;
  ValueP val;
  TextP first_text;
  TextP text;
    
  Iterator<SPNLElement*> iter(structure_list);
  for (SPNLElement* elem=iter;elem;elem=++iter)
    if (elem->type=="TexPart" && elem->element->Selected())
      if (!first_found) {
	first = elem;
	first_found = true;
	first->element->Select(false);
	first_val = first->element->Attr("td");
	first_text = TextP(first_val);
      }
      else {
	val = elem->element->Attr("td");
	text = TextP(val);
	if (((char*)(*first_text))[strlen((char*)(*first_text))-1]!='\n'&&!(*first_text==""))
	  *first_text += '\n';
	*first_text += *text;
	structure_list.Remove(elem);
      }
  //Print();
  pg.Remove();
  first->element->Select(true); 
}


void SPNLStructure::ShowError (Editor& ed,char* msg) {
  if (!dialog)
    dialog.OpenMsg("SPNLError","Error",ed.Shell(),XmDIALOG_ERROR,msg,false);
}


void SPNLStructure::ShowMessage (Editor& ed,char* msg) {
  if (!dialog)
    dialog.OpenMsg("SPNLMessage","Message",ed.Shell(),XmDIALOG_INFORMATION,msg,false);
}


void SPNLStructure::ShowList(Editor& ed)
{
  if (!dialog) {
    ostrstream os(buf,2000); // Achtung nur 2000 Zeichen!
    int no_subs;
    if (bool(structure_list)) {
      Iterator<SPNLElement*> iter(structure_list);
      for (SPNLElement* elem=iter;elem;elem=++iter) {
	os<<elem->element->Name()<<" of type "<<elem->type<<'\n';
	if(bool(elem->subelements)) {
	  os<<"subelements: "<<elem->subelements.Length();
	  no_subs=0;
	  Iterator<FrameElement*> fiter(elem->subelements);
	  for (FrameElement* fe=fiter;fe;fe=++fiter) {
	    if(no_subs)
	      if(no_subs%10) 
		os<<", "<<fe->element->Name();
	      else
		os<<",\n  "<<fe->element->Name();
	    else
	      os<<"\n  "<<fe->element->Name();
	    ++no_subs;
	  }
	  os<<'\n';
	}
	os<<'\n';
      }
      os<<'\0';
    }
    else
      os<<"The list is empty"<<'\0';
    ShowMessage (ed, buf);
  }  
}

bool SPNLStructure::ConvertAndSaveStructure(Editor& ed)
{
  Text basename = Text(ed.GetNet()->FileName()) - NET_ENDING;  
  Text numbered =  (Text) basename + ".numbered";
  Text filename =  (Text) basename + SPNL_ENDING;

  //cout << "::ConvertAndSaveStructure\n";

  text_module = ""; // text_module loeschen

  ValueP value;
  TextP text;
  Position pos;

#ifdef DEBUG_SPNL
cout << "\nBeginne Konvertierung der Struktur" << flush;
#endif
  // Text-Grafik-Darstellungsform in rein textuelle Darstellungsform uebersetzen
  Iterator<SPNLElement*> iter(structure_list);
  for (SPNLElement* elem=iter;elem;elem=++iter) { // Liste durchgehen
    //cout<<elem->element->Name()<<'\n';
    if (elem->type=="TexPart") { // Textelement bearbeiten
      value = elem->element->Attr("td");
      text = TextP(value);
      text_module += *text;
      if (((char*)(text_module))[strlen((char*)(text_module))-1]!='\n'&&!(text_module==""))
	text_module += '\n';
    }
    else { // Frame mit Grafikelementen bearbeiten
      Ident element_type;
      GraphAtts atts; // Soll nur in diesem Bereich gelten (wegen Speicherplatz)
      Object* ob;
      double angle;
      bool default_val;
      
      Iterator<FrameElement*> fiter(elem->subelements);
      for (FrameElement* fe=fiter;fe;fe=++fiter) { // Iterieren ueber alle Grafikelemente
	ob = fe->element;
	element_type = ob->Class()->Name(); // Typ des aktuellen Grafikelements (z.B. place)
	//cout<<ob->Name()<<" is of type "<<element_type<<'\n';
	if (element_type=="Place") {
	  // Daten fuer Stelle ermitteln:
	  Text* place = new Text((char*)(ob->Name())); // Name der Stelle
	  value = ob->Attr("marking"); // Initiale Markierung der Stelle
	  text = TextP(value);
	  default_val = (*text=="0");
	  if (!default_val) { // Nur schreiben, wenn Initiale Markierung nicht Default (0)
	    *place += ':';
	    *place += *text;
	  }
	  *place += ";\n";
	  atts.place.Append(place); 
	  Text* graphplace = new Text((char*)(ob->Name())); // Name der Stelle
	  *graphplace += ":center=";
	  pos = ob->Mid(); // Position der Stelle
	  *graphplace += (char*)(pos);
	  *graphplace += ",name=";
	  pos += (ob->GetRelative(0) * ob->Scale()).Round(); // Position des Labels
	  *graphplace += (char*)(pos);
	  *graphplace += ";\n";
	  atts.graphplace.Append(graphplace);
	  //cout<<*place<<*graphplace<<'\n';
	}
	else if (element_type=="ImmTrans" || element_type=="ExpTrans") {
	  // Daten fuer Transition ermitteln:
	  Text* trans = new Text((char*)(ob->Name())); // Name der Transition
	  *trans += (element_type=="ImmTrans"?":imm;\n":":timed;\n"); // Typ der Transition
	  atts.trans.Append(trans);
	  Text* graphtrans = new Text((char*)(ob->Name())); // Name der Transition 
	  *graphtrans += ":center=";
	  pos = ob->Mid(); // Position der Transition
	  *graphtrans += (char*)(pos);
	  *graphtrans += ",name=";
	  pos += (ob->GetRelative(0) * ob->Scale()).Round(); // Position des Labels
	  *graphtrans += (char*)(pos);
	  angle = (int(ob->Angle() * 180.0 / M_PI) + 360) % 360; // Bogenmass -> Gradmass
	  if (angle) {
	    *graphtrans += ",orient=";
	    *graphtrans += angle; // Orientierung der Transition
	  }
	  *graphtrans += ";\n";
	  atts.graphtrans.Append(graphtrans);
	  //cout<<*trans<<*graphtrans<<'\n';
	}
	else if (element_type=="Arc" || element_type=="Inhibitor") {
	  // Daten fuer Kante ermitteln:
	  Text* arc = new Text((char*)(ob->Name())); // Name der Kante
	  *arc += ":(";
	  *arc += (char*)(((Connector*)(ob))->GetInput()->Name()); // Eingangsknoten der Kante
	  *arc += ',';
	  *arc += (char*)(((Connector*)(ob))->GetOutput()->Name()); // Ausgangsknoten der Kante
	  *arc += ')';
	  default_val = (element_type=="Arc");  // Typ der Kante
	  if (default_val) // Kantentyp ist Default (ioarc)
	    //*arc +=",type ioarc";
	    *arc += "";
	  else // Nicht Default
	    *arc += ",type harc";
	  value = ob->Attr("multi"); // Vielfachheit der Kante
	  text = TextP(value);
	  default_val = (*text=="1");
	  if (!default_val) { // Nur schreiben, wenn Vielfachheit nicht Default (1)
	    *arc += ",mult ";
	    *arc += *text;
	  }
	  *arc += ";\n";
	  atts.arc.Append(arc);
	  int no_vias = ((Connector*)(ob))->NumPositions() - 2; // Anzahl der Zwischenpunkte
	  if (!default_val || no_vias>0) {
	    Text* grapharc = new Text((char*)(ob->Name()));  // Name der Kante
	    *grapharc += ':';
	    if (no_vias) { // Nur schreiben, wenn Zwischenpunkte vorhanden sind.
	      *grapharc += "viapos=(";
	      Iterator<Position*> piter = ((Connector*)(ob))->Positions(); // Liste der Kantenecken
	      for (Position* posi=++piter; no_vias&&posi; --no_vias,posi=++piter) {
		pos = posi->Round();
		*grapharc += (char*)(pos);
		*grapharc += (no_vias>1?',':')');
	      }
	      if (!default_val)
		*grapharc += ',';
	    }
	    if (!default_val) { // Nur schreiben, wenn Vielfachheit nicht Default (1)
	      *grapharc += "mult=";
	      pos = ob->Mid().Round(); // Position der Kantenvielfachheit
	      *grapharc += (char*)(pos);
	    }
	    *grapharc += ";\n";
	    atts.grapharc.Append(grapharc);
	  //cout<<*arc<<*grapharc<<'\n';
	  }
	}
	else if (element_type=="PortNode") {
	  // Daten fuer Port ermitteln:
	  Text* graphport = new Text((char*)(ob->Name())); // Name des Ports
	  *graphport += ":center=";
	  pos = ob->Mid(); // Position des Ports
	  *graphport += (char*)(pos);
	  *graphport += ",name=";
	  pos += (ob->GetRelative(0) * ob->Scale()).Round(); // Position des Labels
	  *graphport += (char*)(pos);
	  *graphport += ";\n";
	  atts.graphport.Append(graphport);
	  //cout<<*graphport<<'\n';
	}
	else if (element_type=="Process") {
	  // Daten fuer Prozess ermitteln:
	  Text* graphprocess = new Text((char*)(ob->Name())); // Name des Prozesses
	  *graphprocess += ":center=";
	  pos = ob->Mid(); // Position des Prozesses
	  *graphprocess += (char*)(pos);
	  *graphprocess += ",name=";
	  pos += (ob->GetRelative(0) * ob->Scale()).Round(); // Position des Labels
	  *graphprocess += (char*)(pos);
	  angle = (int(ob->Angle() * 180.0 / M_PI) + 360) % 360; // Bogenmass -> Gradmass
	  if (angle) {
	    *graphprocess += ",orient=";
	    *graphprocess += angle; // Orientierung des Prozesses
	  }
	  pos = ((ob->Scale() * Position(200,100)).Abs()).Round();
	  *graphprocess += ",width=";
	  *graphprocess += pos.X();
	  *graphprocess += ",length=";
	  *graphprocess += pos.Y();
	  *graphprocess += ";\n";
	  atts.graphprocess.Append(graphprocess);
	  //cout<<*graphprocess<<'\n';
	}
	else if (element_type=="ProcessArray") {
	  // Daten fuer Prozess-Array ermitteln:
	  Text* grapharray = new Text((char*)(ob->Name())); // Name des Arrays
	  *grapharray += ":center=";
	  pos = ob->Mid(); // Position des Arrays
	  *grapharray += (char*)(pos);
	  *grapharray += ",name=";
	  pos += (ob->GetRelative(0) * ob->Scale()); // Position des Labels
	  *grapharray += (char*)(pos);
	  angle = (int(ob->Angle() * 180.0 / M_PI) + 360) % 360; // Bogenmass -> Gradmass
	  if (angle) {
	    *grapharray += ",orient=";
	    *grapharray += angle; // Orientierung des Arrays
	  }
	  pos = ((ob->Scale() * Position(200,100)).Abs()).Round();
	  *grapharray += ",width=";
	  *grapharray += pos.X();
	  *grapharray += ",length=";
	  *grapharray += pos.Y();
	  *grapharray += ";\n";
	  atts.grapharray.Append(grapharray);
	  //cout<<*grapharray<<'\n';
	}
	else {
	  ShowError(ed,"Unknown type of\ngraphical element!\n");
	  return false;
	}
      }
      // atts Schreiben
      Text space("               ");
      
      text_module += "graphbegin\n";

      if (atts.place.Length()) {
	text_module += "  place        ";
	Iterator<Text*> titer = atts.place;
	for (text=titer;text;) {
	  text_module += *text;
	  text=++titer;
	  if (text)
	    text_module += space;
	}
      }
      
      if (atts.trans.Length()) {
	text_module += "  trans        ";
	Iterator<Text*> titer = atts.trans;
	for (text=titer;text;) {
	  text_module += *text;
	  text=++titer;
	  if (text)
	    text_module += space;
        }
      }

      if (atts.arc.Length()) {
	text_module += "  arc          ";
	Iterator<Text*> titer = atts.arc;
	for (text=titer;text;) {
	  text_module += *text;
	  text=++titer;
	  if (text)
	    text_module += space;
        }
      }

      if (atts.graphplace.Length()) {
	text_module += "  graphplace   ";
	Iterator<Text*> titer = atts.graphplace;
	for (text=titer;text;) {
	  text_module += *text;
	  text=++titer;
	  if (text)
	    text_module += space;
        }
      }

      if (atts.graphtrans.Length()) {
	text_module += "  graphtrans   ";
	Iterator<Text*> titer = atts.graphtrans;
	for (text=titer;text;) {
	  text_module += *text;
	  text=++titer;
	  if (text)
	    text_module += space;
        }
      }

      if (atts.grapharc.Length()) {
	text_module += "  grapharc     ";
	Iterator<Text*> titer = atts.grapharc;
	for (text=titer;text;) {
	  text_module += *text;
	  text=++titer;
	  if (text)
	    text_module += space;
        }
      }

      if (atts.graphport.Length()) {
	text_module += "  graphport    ";
	Iterator<Text*> titer = atts.graphport;
	for (text=titer;text;) {
	  text_module += *text;
	  text=++titer;
	  if (text)
	    text_module += space;
        }
      }

      if (atts.graphprocess.Length()) {
	text_module += "  graphprocess ";
	Iterator<Text*> titer = atts.graphprocess;
	for (text=titer;text;) {
	  text_module += *text;
	  text=++titer;
	  if (text)
	    text_module += space;
        }
      }

      if (atts.grapharray.Length()) {
	text_module += "  grapharray   ";
	Iterator<Text*> titer = atts.grapharray;
	for (text=titer;text;) {
	  text_module += *text;
	  text=++titer;
	  if (text)
	    text_module += space;
        }
      }

      text_module += "graphend;\n";
    }
  }
  //cout<<"text_module:\n"<<(char*)(text_module)<<"\n...\n";
#ifdef DEBUG_SPNL
cout << "\nStruktur fertig konvertiert" << flush;
#endif
  
#ifdef DEBUG_SPNL
cout << "\nWriteNumberedFile " << (char *)numbered << flush;
#endif
  if (fork() == 0) {
    WriteNumberedFile(ed,(char *)(numbered));
    exit(1);
  }

#ifdef DEBUG_SPNL
cout << "\nWriteModuleFile " << (char *)filename << flush;
#endif
  WriteModuleFile(ed,(char *)(filename));

#ifdef DEBUG_SPNL
cout << "\nfertig convertandsaveas" << flush;
#endif
  return true;
}


void SPNLStructure::WriteModuleFile(Editor& ed,char* filename)
{
  ofstream file(filename);
  file << (char*)(text_module);
  file.close();
  if (!file) {
    Text err= Text ("file '") + filename + Text ("' couldn't be written");
    ShowError (ed, err);
  }
}


void SPNLStructure::WriteNumberedFile(Editor& ed, char* filename)
{
  char *p = (char*)(text_module);
  int i=1;
  ofstream file(filename);
  file << "  1:";
  while (*p) {
    file << *p;
    if (*p++ == '\n') {
      if (++i < 100)
	if (i < 10)
	  file << "  ";
	else
	  file << ' ';
      file << i << ':';
    }    
  }
  file.close();
  if (!file) {
    Text err= Text ("file '") + filename + Text ("' couldn't be written");
    ShowError (ed, err);
  }
/*
  char *p = (char*)(text_module);
  Text numbered_module;

  if (*p) {  // **** vereinfachen ****
    numbered_module+="  1:";
    for (int i=1;*p;++p) {
      numbered_module += *p;
      if (*p=='\n' && *(p+1)) {
	++i;
	if (i<10) numbered_module += ' ';
	if (i<100) numbered_module += ' '; 
	numbered_module += (double)(i);
	numbered_module += ':';
      }
    }
  }
  ofstream file(filename);
  file << (char*)(numbered_module);
  file.close();
  if (!file) {
    Text err= Text ("file '") + filename + Text ("' couldn't be written");
    ShowError (ed, err);
  }
*/
}


bool file_exist (char *fname) {
  int fd= open (fname, O_RDONLY);
  return (fd != -1);
}


void file_up2date (char *fname) {
  Text befehl= Text ("touch ") + fname;
  if (fork() == 0) {
    system ((char *)befehl);
    exit(1);
  }
/*
  fstream file (fname, ios::app);
  file.seekg (0, ios::beg);
  char firstchar;
  file >> firstchar;
  file << firstchar;
*/
/*
  fstream file (fname, ios::app);
  file << " " << flush;
  file.close ();
*/
  
}


bool SPNLStructure::CheckSyntaxOfFile (Editor& ed, char *basename, bool showmessages) {
  Text outfile =  (Text) basename + OUT_ENDING;
  Text filename =  (Text) basename + SPNL_ENDING;
  Text numbered =  (Text) basename + ".numbered";
  Text checkfile = agnes_dir + "/../SPNL/syntaxcheck/bin/spnl_check";
  Text call = checkfile + Text(" ") + basename + SPNL_ENDING + Text(" >") + outfile + Text(" 2>&1");
  bool errorfree= false;

#ifdef DEBUG_SPNL
cout << "\nloesche altes out-file " << (char *)outfile << flush;
#endif
  ofstream out((char *)outfile);   // altes outfile loeschen
  out.close();

  if (!file_exist (filename) && ((Text) basename == Text(ed.GetNet()->FileName()) - NET_ENDING))
    ConvertAndSaveStructure (ed);

#ifdef DEBUG_SPNL
cout << "\nMache Aufruf: " << (char *)call << flush;
#endif
  system((char *)call);

#ifdef DEBUG_SPNL
cout << "\nREADY " << flush;
#endif
  ifstream OutFile((char*)outfile);
  if (OutFile) {  // Datei erfolgreich geoeffnet?
    OutFile.seekg (0, ios::end);
    int FileEnd= OutFile.tellg ();
    if (FileEnd == OUTFILE_LENGTH) {
      remove (outfile);   // outfile mit "syntax conform ..." loeschen
      errorfree= true;
    }
    if (showmessages)
      ShowSPNLsyntaxerrors (ed);
  }
  if (errorfree)
    file_up2date (filename);
  return (errorfree);
}


void SPNLStructure::CheckSyntax(Editor& ed) {
  Text basename = Text(ed.GetNet()->FileName()) - NET_ENDING;  
  CheckSyntaxOfFile (ed, (char *)basename);
}


void SPNLStructure::CompileNetOfFile (Editor& ed, char *basename, bool showmessages) {
  Text filename = (Text) basename + SPNL_ENDING;
  Text errorfile = (Text) basename + SPNLERRORS_ENDING;
  Text outfile = (Text) basename + OUT_ENDING;
  Text checkfile = agnes_dir + "/../SPNL/compiler/bin/spnl_compile";
  Text call = checkfile + Text(" ") + filename + Text(" > ") + errorfile;
  Text compileerrormessage1= Text("file '") + filename + Text ("' not found\n") +
    Text ("please check syntax of file '") + basename + NET_ENDING + Text ("'");
  Text compileerrormessage2= Text("file '") + filename + Text ("' contains errors\n") +
    Text ("please check syntax of file '") + basename + NET_ENDING + Text ("'");

  if (!file_exist (filename))
    ShowError(ed,compileerrormessage1);
  else if (file_exist (outfile))
    ShowError(ed,compileerrormessage2);
  else {
#ifdef DEBUG_SPNL
    cout << "\nMache Aufruf: " << (char *) call << flush;
#endif
    system((char*)(call));

    ifstream OutFile((char*) errorfile);
    if (OutFile) {  // Datei erfolgreich geoeffnet?
      OutFile.seekg (0, ios::end);
      int FileEnd= OutFile.tellg ();
      if (FileEnd == ERRORFILE_LENGTH)
	remove (errorfile);   // outfile mit "compiling finished ..." loeschen
      if (showmessages)
	ShowSPNLerrors (ed);
    }
  }
}


void SPNLStructure::ShowSPNLsyntaxerrors (Editor& ed) {
  Text basename = Text(ed.GetNet()->FileName()) - NET_ENDING;  
  Text filename = basename + SPNL_ENDING;
  Text outfile = basename + OUT_ENDING;
  Text numbered =  (Text) basename + ".numbered";
  Text compilemessage= Text ("the syntax of file '") + filename + Text ("' is conform to SPNL");
  Text compileerrormessage= Text("file '") + filename + Text ("' not found\n") +
    Text ("please check syntax of file '") + basename + NET_ENDING + Text ("'");
  if (!file_exist (outfile))
    if (file_exist (filename))
      ShowError(ed, (char *)compilemessage);
    else
      ShowError(ed, (char *)compileerrormessage);
  else {
    LogWindow* lw1 = new LogWindow((char*)numbered);
    lw1->Open("SPNLModuleWindow", (char*)numbered, ed.Shell());
    int dummy1;
    waitpid(-1, &dummy1, WNOHANG);

    LogWindow* lw2 = new LogWindow((char*)outfile);
    lw2->Open("SyntaxCheckWindow", "Syntax Check Window", ed.Shell());
    int dummy2;
    waitpid(-1, &dummy2, WNOHANG);
  } 
}


void SPNLStructure::ShowSPNLerrors (Editor& ed) {
  Text basename = Text(ed.GetNet()->FileName()) - NET_ENDING;  
  Text filename = basename + SPNLERRORS_ENDING;
  Text compfilename = basename + SPNLCOMP_ENDING;
  Text compilemessage= Text("file '") + basename + SPNL_ENDING + Text("' has been compiled")
    + Text(" successfully\noutput can be found in '") + compfilename + Text("'");
  Text compileerrormessage= Text("file '") + basename + NET_ENDING + Text ("' has not been compiled");
  if (!file_exist (filename))
    if (!file_exist (compfilename))
      ShowError(ed, (char *)compileerrormessage);
    else
      ShowMessage (ed, (char *)compilemessage);
  else {
    LogWindow* lw1 = new LogWindow((char*)filename);
    lw1->Open("SPNLCompileErrorWindow", (char*)filename, ed.Shell());
    int dummy1;
    waitpid(-1, &dummy1, WNOHANG);
  } 
}


void SPNLStructure::ShowErrorFile (Editor& ed) {
  Text basename = Text(ed.GetNet()->FileName()) - NET_ENDING;  
//  Text errorfile = basename + SPNLERRORS_ENDING;
  Text outfile = basename + OUT_ENDING;
  if (file_exist (outfile))
    ShowSPNLsyntaxerrors (ed);
  else
    ShowSPNLerrors (ed);
}


void SPNLStructure::BuildNet (Editor& ed, bool buildall) {
  Text basename = Text(ed.GetNet()->FileName()) - NET_ENDING;  
  Text prjfile = basename + SPNLPRJ_ENDING;
  Text spnlfile = basename + SPNL_ENDING;
  Text makefile = agnes_dir + "/../SPNL/compiler/bin/spnl_make";
  bool errorfree= true;

  // Zuerst maken:
  Text makecall;
  Text buildmessage= "";

  if (buildall)
    makecall= makefile + Text(" -b ") + spnlfile;
  else
    makecall= makefile + Text(" ") + spnlfile;

  if (!file_exist (spnlfile))
    ConvertAndSaveStructure (ed);

#ifdef DEBUG_SPNL
cout << "\nMache Aufruf: " << (char *)makecall << flush;
#endif
  system((char*)makecall);

  if (file_exist (prjfile)) {
#ifdef DEBUG_SPNL
cout << "\nfile " << (char *) prjfile << " existiert." << flush;
#endif
    // jetzt syntax checken
    char buffer [256];
    int spnl_exist;
    ifstream infile ((char *)prjfile);
    while (!infile.eof ()) {
      infile >> buffer;
      infile >> spnl_exist;
      if (buffer[0] != '\0') {
#ifdef DEBUG_SPNL
	cout << "\nbeginne syntaxcheck von " << buffer << flush;
#endif
	if (spnl_exist) {
	  if (spnl_exist == 1) {
	    buildmessage+= (char *) Text ("expecting file '");
	    buildmessage+= buffer;
	    buildmessage+= SPNL_ENDING;
	    buildmessage+= (char *) Text ("'\n");
	  }
	  else {  // (spnl_exist == 2)
	    buildmessage+= (char *) Text ("file '");
	    buildmessage+= buffer;
	    buildmessage+= NET_ENDING;
	    buildmessage+= (char *) Text ("' does not exist\n");
	  }
	  errorfree= false;
	}
        else if (!CheckSyntaxOfFile (ed, buffer, false)) {
	  buildmessage+= (char *) Text ("the syntax of file '");
	  buildmessage+= buffer;
	  buildmessage+= NET_ENDING;
	  buildmessage+= (char *) Text ("' contains errors\n");
	  errorfree= false;
	}
#ifdef DEBUG_SPNL
	cout << "\nsyntaxcheck von " << (char *) buffer << " abgeschlossen" << flush;
#endif
      }
    }
  }
  remove (prjfile);

#ifdef DEBUG_SPNL
if (errorfree) cout << "\ncompilieren von " << (char *) basename << flush;
#endif
  // zuletzt compilieren
  if (errorfree)
    CompileNetOfFile (ed, (char *)basename, true);
  else {
    buildmessage+= "compilation aborted\nplease remove all errors\n";
    ShowMessage (ed, buildmessage);
  }
}


void SPNLStructure::ShowSPNLcomp (Editor& ed) {
  Text basename = Text(ed.GetNet()->FileName()) - NET_ENDING;  
  Text filename = basename + SPNLCOMP_ENDING;
  Text errorfile = basename + SPNLERRORS_ENDING;
  if (!file_exist (filename) || file_exist (errorfile))
    ShowError(ed,"you first have to compile the net\n");
  else {
    LogWindow* lw1 = new LogWindow((char*)filename);
    lw1->Open("SPNLCompiledNetWindow", (char*)filename, ed.Shell());
    int dummy1;
    waitpid(-1, &dummy1, WNOHANG);
  } 
}


#define NoExplicitInstanciation
#include "linked.cc"
CINSTANCIATE(FrameElement)
CINSTANCIATE(SPNLElement)

