#ifndef __SPNLSTRUCTURE_H__
#define __SPNLSTRUCTURE_H__


#include "ident.h"
#include "node.h"
#include "linked.h"
#include "metrics.h"
#include "editor.h"
#include "dialog.h"
#include "page.h"
#include "net.h"


#define NET_ENDING ".net"
#define SPNL_ENDING ".spnl"
#define OUT_ENDING ".out"
#define SPNLERRORS_ENDING ".spnl_errors"
#define SPNLCOMP_ENDING ".spnl_comp"
#define SPNLPRJ_ENDING ".spnl_project"
// siehe SPNL/compiler/src/foutput.h

#define ERRORFILE_LENGTH 34  // Laenge fuer korrektes error-file
#define OUTFILE_LENGTH 31    // Laenge fuer korrektes out-file


#define TEXTMARGIN 10
#define FRAMEMARGIN 20



// Definitionen der Listenelementtypen fuer die Uebersetzung der 
// grafischen Darstellung in die textuelle Darstellung
struct GraphAtts;
struct GraphAtts {
  LinkedList<Text*> place;
  LinkedList<Text*> trans;
  LinkedList<Text*> arc;
  LinkedList<Text*> graphplace;
  LinkedList<Text*> graphtrans;
  LinkedList<Text*> grapharc;
  LinkedList<Text*> graphport;
  LinkedList<Text*> graphprocess;
  LinkedList<Text*> grapharray;

  GraphAtts(): place(),trans(),arc(),graphplace(),graphtrans(),grapharc(),
               graphport(),graphprocess(),grapharray() {}
  ~GraphAtts() // Alloziierten Speicher freigeben
    {
      Iterator<Text*> titer = place;
      Text* text;
      for (text=titer;text;text=++titer) 
	delete text;
      place.Clear();
      titer = trans;
      for (text=titer;text;text=++titer) 
	delete text;
      trans.Clear();
      titer = arc;
      for (text=titer;text;text=++titer) 
	delete text;
      arc.Clear();
      titer = graphplace;
      for (text=titer;text;text=++titer) 
	delete text;
      graphplace.Clear();
      titer = graphtrans;
      for (text=titer;text;text=++titer) 
	delete text;
      graphtrans.Clear();
      titer = grapharc;
      for (text=titer;text;text=++titer) 
	delete text;
      grapharc.Clear();
      titer = graphport;
      for (text=titer;text;text=++titer) 
	delete text;
      graphport.Clear();
      titer = graphprocess;
      for (text=titer;text;text=++titer) 
	delete text;
      graphprocess.Clear();
      titer = grapharray;
      for (text=titer;text;text=++titer) 
	delete text;
      grapharray.Clear();
    }
};

// Definitionen fuer die SPNL-Strukturliste
struct FrameElement; 
struct FrameElement {
  Object* element;
  Position relative;

  FrameElement(Object* elem, Position rel):element(elem),relative(rel) {}
  FrameElement(FrameElement* fe):
    element(fe->element),relative(fe->relative) {}
};

struct SPNLElement;
struct SPNLElement {
  Ident type;
  Node* element;
  Metrics metrics;
  LinkedList<FrameElement*> subelements;

  SPNLElement(Ident tp, Node* no, Metrics met):
    type(tp),element(no),metrics(met),subelements() {}
  SPNLElement(Ident tp, Node* no, Metrics met, LinkedList<FrameElement*> subs):
    type(tp),element(no),metrics(met),subelements(subs) {}
  SPNLElement(SPNLElement* elem):
    type(elem->type),element(elem->element),
    metrics(elem->metrics),subelements(elem->subelements) {}
};

class SPNLStructure;
class SPNLStructure {

  LinkedList<SPNLElement*> structure_list;
  Text text_module;
  Dialog<SPNLStructure,bool> dialog;
  char buf[2000]; // fuer Ausgaben mit dialog

  void ShiftEntries(Display*);
  void MoveEntry(SPNLElement* elem,Position rel);
  void AlignEntry(Display*,SPNLElement* elem);
  void MoveSubelements(SPNLElement* elem,Position rel);
  void SelectSubs(SPNLElement* elem,bool select);
  void Insert(SPNLStructure& ins);
  void Append(SPNLStructure& ins);
  void PrepCutNCopy(SPNLElement* frame);
  
  bool UpdateSubelements(Display* dpy,SPNLElement* elem);
  bool DragboxFrameInside(Page& pg,SPNLElement* elem);

  void ShowError(Editor& ed,char* msg);
  void ShowMessage (Editor& ed,char* msg);
  void WriteModuleFile(Editor& ed,char* name);
  void WriteNumberedFile(Editor& ed,char* name);
  
public:  
 
  SPNLStructure():structure_list(),text_module(),dialog(this) {}
  ~SPNLStructure() { Clear(); }

  LinkedList<SPNLElement*>& GetList() { return structure_list; }

  void Print();
  void FormatEntries(Display* dpy);
  void Remove();
  void Paste(Editor& ed,Page& pg,bool moveback=false);
  void PrepScale();
  void FinishScale();
  void PrepRotate();
  void PrepMove(Object* ob);
  void Drag(Page& pg);
  void SortEntries();
  void Clear();
  void Undo(Editor& ed,Page& pg);
  void Add(Editor*, Page& pg);
  void Merge(Page& pg);
  void ShowList(Editor& ed);
  void CompileNetOfFile (Editor& ed, char *basename, bool showmessages= true);
  bool CheckSyntaxOfFile (Editor& ed, char *basename, bool showmessages= true);
  void CheckSyntax(Editor& ed);
  void ShowErrorFile (Editor& ed);
  void ShowSPNLerrors (Editor& ed);
  void ShowSPNLsyntaxerrors (Editor& ed);
  void BuildNet(Editor& ed, bool buildall);
  void ShowSPNLcomp(Editor& ed);

  bool UpdateEntries(Display* dpy);
  bool Evaluate(Editor& ed,Page& pg);
  bool Get(Editor& ed,Page& pg);
  bool Get(Editor& ed,Page* pg);
  bool Insert(Display* dpy,Node* no);
  bool Insert(Display*,Connector* cn);
  bool IsIn(Object* ob);
  bool CheckForCutNCopy();
  bool CheckForMerge();
  bool ConvertAndSaveStructure(Editor& ed);

  char* GetText() { return (char*)(text_module); }
};


#endif



  




