#include <stdio.h>
#include <stdlib.h>
#include "types.h"			/* definitions and types */
#include "portab.h"

extern char *emalloc();				/* from utils.c */
extern char *ecalloc();          /* from utils.c */
extern void efree();
extern int  *current_mark;
extern double md_rate();

/*
 * local to global mark obtains the (partial) global marking from a local one
 */
void local_to_global_mark(sub, l_mark, g_mark )
	subnet *sub;
	int    *l_mark;
	int    *g_mark;
{
	int pl;
	for(pl=0; pl<no_places; pl++)
		g_mark[pl] = -1;   /* mark as unknown */
	for(pl=0; pl<sub->no_places; pl++)
		g_mark[sub->places[pl].number] = l_mark[pl];
}

/*
 * returns the (possibly marking dependent) firing weight of the immediate transition
 */
int *weight_mark = NULL;

double imm_transition_weight(sub,l_mark,trans)
	subnet *sub;
	int    *l_mark;
	int     trans;
{
	double ret;
	if (!sub->transitions[trans].wgt_id)
		return sub->transitions[trans].prob;	/* independent transition */
	if (weight_mark == NULL)			/* get memory space */
		weight_mark = (int *)emalloc(no_places * sizeof(int));
	local_to_global_mark(sub, l_mark, weight_mark);
	current_mark = weight_mark;			/* parameter for act_marking() */
	ret = md_rate(sub->transitions[trans].wgt_id);	/* marking dependent rate */
	if (ret > 0.0)
		return ret;
	printf("Error: marking dependent firing weight of transition");
	printf(" %s", sub->transitions[trans].name);
	printf(" evaluates to 0\n");
	exit(1);
}

/*
 *	is_in_packed tests a transition for being a member of a packed set
 */
int is_in_packed(tra, set)
	int	tra;
	byte	*set;
{
	int	i;

	for(i = 1; i <= set[0]; i++)
		if (set[i] == tra)
			return 1;
	return 0;
}

/*
 * bitmasks for marking coding / decoding
 */
word	bit_mask[17] = { 0x000,
	  0x0001, 0x0003, 0x0007, 0x000F, 0x001F, 0x003F, 0x007F, 0x00FF,
	  0x01FF, 0x03FF, 0x07FF, 0x0FFF, 0x1FFF, 0x3FFF, 0x7FFF, 0xFFFF };

/*
 * code_submarking codes a submarking into a packed format using only as
 *	many bits as needed and returns a pointer to the resulting memory area
 */
word *code_submarking(sub_mem, sub_no_pla, sub_pla, mark)
	int	sub_mem;	/* memory needed to store one marking */
	int	sub_no_pla;	/* number of places in the subnet */	
	place	*sub_pla;	/* places of the subnet */
	int	*mark;		/* an array with the numbers of tokens in places */
{
	word	*pack;
	int	i, bits, words, count = 0;

	pack = (word *)ecalloc(sub_mem, sizeof(word));
	for(i = 0; i < sub_no_pla; i++)
	{	bits = sub_pla[i].bits;
		if (mark[i] > sub_pla[i].bound)
		{	printf("ERROR: number of tokens in %s exceeded upper bound %d !\n", 
				sub_pla[i].name, sub_pla[i].bound);
			exit(1);
		}
		words = count / 16;
		pack[words] |= mark[i] << (count % 16);
		if ((count+bits) / 16 != words)
			pack[words+1] |= mark[i] >> (16 - count % 16);
		count += bits;
	}
	return pack;
}

/*
 * decode_submarking decodes a submarking from packed format and returns
 * a pointer to the resulting array with number of tokens in places
 */
void decode_submarking(sub_no_pla, sub_pla, code, to)
	int	sub_no_pla;	/* number of places in the subnet */	
	place	*sub_pla;	/* places of the subnet */
	word	*code;		/* memory space with coded marking */
	int	*to;        /* result place */
{
	int	i, bits, words, count = 0;

	for(i = 0; i < sub_no_pla; i++)
	{	bits = sub_pla[i].bits;
		words = count / 16;
		to[i] = (code[words] >> (count % 16)) & bit_mask[bits];
		if ((count+bits) / 16 != words)
			to[i] |= (bit_mask[bits-16+count%16] & code[words+1])
					<< (16 - count % 16);
		count += bits;
	}
}

/*
 * enter_disconnected enters a disconnected places marking into the tree
 * and returns a pointer to the memory area, where the marking is stored
 */
word *enter_disconnected(sub, g_token)
	t_sub	*sub;		/* subnet containing disconnected places */
	int	*g_token;	/* number of tokens in places (global marking) */
{
	sub_node   *node = sub->tree, **insert;
	sp_pointer *lookup;
	int	pla_nr = 0, i, left, right, mid, tokens, old_bound;
	int	*token;

	if (!sub->no_places)		/* no disconnected places */
		return NULL;
	token = (int *)emalloc(sub->no_places * sizeof(int));
	for (i = 0; i < sub->no_places; i++)		/* local numbers */
		token[i] = g_token[sub->places[i].number];
	if (node == NULL)		/* tree is empty */
	{	node = sub->tree = (sub_node *)emalloc(sizeof(sub_node));
		if (sub->places[0].bound > ARRAY_LIMIT)
		{	node->bound = SPARSE_START;	/* sparse array */
			node->number = 1;
			node->next = (sub_node **)emalloc(SPARSE_START * sizeof(sp_pointer));
			lookup = (sp_pointer *)node->next;
			lookup[0].index = token[0];
			insert = (sub_node **)&(lookup[0].next);
			node = NULL;		/* skip search process */
		} else
		{	node->bound = 0;		/* array */
			node->next = (sub_node **)ecalloc((sub->places[0].bound + 1),
								sizeof(sub_node *));
		}
	}
	while((node != NULL) && (pla_nr < sub->no_places))
	{	tokens = token[pla_nr];
		if (tokens > sub->places[pla_nr].bound)
		{	printf("ERROR: number of tokens in %s(%d) exceeded upper bound %d !\n", 
				sub->places[pla_nr].name, tokens, sub->places[pla_nr].bound);
			exit(1);
		}
		if (node->bound)
		{	lookup = (sp_pointer *)node->next;	/* sparse array */
			left = 0; right = node->number - 1;
			mid = (left + right) / 2;
			if (tokens <= lookup[left].index)
				mid = left;
			if (tokens >= lookup[right].index)
				mid = left = right;
			while((tokens != lookup[mid].index) && (mid != left))
			{	if (tokens > lookup[mid].index)
					left = mid;
				else	right = mid;
				mid = (left + right) / 2;
			}
			if (tokens == lookup[mid].index)	/* found */
				node = (sub_node *)lookup[mid].next;
			else
			{	mid += (tokens > lookup[mid].index);
				if (++(node->number) > node->bound)
				{	old_bound = node->bound;
					node->bound = SPARSE_NEXT(node->bound);
					if (node->bound <= old_bound) node->bound = SPARSE_MAX;
					if ((node->next = (sub_node **)
							realloc(node->next, node->bound * sizeof(sp_pointer)))==NULL)
					{
						eperror("graph");
					}
					lookup = (sp_pointer *)node->next;
				}
				memmove(&(lookup[mid+1]),&(lookup[mid]), 
					sizeof(sp_pointer) * (node->number - mid - 1));
				lookup[mid].index = tokens;
				insert = (sub_node **)&(lookup[mid].next);
				break;
			}
		} else
		{	if (node->next[tokens] == NULL)	/* array */
			{	insert = (sub_node **)&(node->next[tokens]);
				break;
			}
			node = node->next[tokens];
		}
		pla_nr++;
	}
	if (pla_nr >= sub->no_places)			/* marking already exists */
	{	efree((char *)(token));
		return (word *)node;
	}
	/*
	 * insert new nodes into tree
	 */
	for(pla_nr++ ; pla_nr < sub->no_places; pla_nr++)
	{	tokens = token[pla_nr];
		if (tokens > sub->places[pla_nr].bound)
		{	printf("ERROR: number of tokens in %s(%d) exceeded upper bound %d !\n", 
				sub->places[pla_nr].name, tokens, sub->places[pla_nr].bound);
			exit(1);
		}
		*insert = node = (sub_node *)emalloc(sizeof(sub_node));
		if (sub->places[pla_nr].bound > ARRAY_LIMIT)
		{	node->bound = SPARSE_START;	/* sparse array */
			node->number = 1;
			node->next = (sub_node **)emalloc(SPARSE_START * sizeof(sp_pointer));
			lookup = (sp_pointer *)node->next;
			lookup->index = tokens;
			insert = (sub_node **)&(lookup->next);
		} else
		{	node->bound = 0;		/* array */
			node->next = (sub_node **)ecalloc((sub->places[pla_nr].bound + 1),
							sizeof(sub_node *));
			insert = (sub_node **)&(node->next[tokens]);
		}
	}
	*insert = (sub_node *)code_submarking(sub->memory, sub->no_places, sub->places, token);
	efree((char *)(token));
	sub->no_markings++;
	return (word *)*insert;
}

/*
 * enter_sub_marking enters a subnet-marking into the reachability tree,
 * tests for immediate loops and returns a pointer to the marking
 */
sub_marking *enter_submarking(sub, token)
	subnet	*sub;		/* corresponding subnet */
	int	*token;		/* number of tokens in places */
{
	sub_marking *marking;
	sub_node   *node = sub->tree, **insert;
	sp_pointer *lookup;
	int	pla_nr = 0, i, left, right, mid, tokens, old_bound;

	if (node == NULL)		/* tree is empty */
	{	node = sub->tree = (sub_node *)emalloc(sizeof(sub_node));
		if (sub->places[0].bound > ARRAY_LIMIT)
		{	node->bound = SPARSE_START;	/* sparse array */
			node->number = 1;
			node->next = (sub_node **)emalloc(SPARSE_START * sizeof(sp_pointer));
			lookup = (sp_pointer *)node->next;
			lookup[0].index = token[0];
			insert = (sub_node **)&(lookup[0].next);
			node = NULL;		/* skip search process */
		} else
		{	node->bound = 0;		/* array */
			node->number = sub->places[0].bound;
			node->next = (sub_node **)ecalloc((sub->places[0].bound + 1),
								sizeof(sub_node *));
		}
	}
	while((node != NULL) && (pla_nr < sub->no_places))
	{	tokens = token[pla_nr];
		if (tokens > sub->places[pla_nr].bound)
		{	printf("ERROR: number of tokens in %s(%d) exceeded upper bound %d !\n", 
				sub->places[pla_nr].name, tokens, sub->places[pla_nr].bound);
			exit(1);
		}
		if (node->bound)
		{	lookup = (sp_pointer *)node->next;	/* sparse array */
			left = 0; right = node->number - 1;
			mid = (left + right) / 2;
			if (tokens <= lookup[left].index)
				mid = left;
			if (tokens >= lookup[right].index)
				mid = left = right;
			while((tokens != lookup[mid].index) && (mid != left))
			{	if (tokens > lookup[mid].index)
					left = mid;
				else	right = mid;
				mid = (left + right) / 2;
			}
			if (tokens == lookup[mid].index)	/* found */
				node = (sub_node *)lookup[mid].next;
			else
			{	mid += (tokens > lookup[mid].index);
				if (++(node->number) > node->bound)
				{	old_bound = node->bound;
					node->bound = SPARSE_NEXT(node->bound);
					if (node->bound <= old_bound) node->bound = SPARSE_MAX;
					if ((node->next = (sub_node **)
							realloc(node->next, node->bound * sizeof(sp_pointer)))==NULL)
					{
						perror("graph");
					}
					lookup = (sp_pointer *)node->next;
				}
				memmove(&(lookup[mid+1]), &(lookup[mid]),
					sizeof(sp_pointer) * (node->number - mid - 1));
				lookup[mid].index = tokens;
				insert = (sub_node **)&(lookup[mid].next);
				break;
			}
		} else
		{	if (node->next[tokens] == NULL)	/* array */
			{	insert = (sub_node **)&(node->next[tokens]);
				break;
			}
			node = node->next[tokens];
		}
		pla_nr++;
	}
	if (pla_nr >= sub->no_places)			/* marking already exists */
	{	marking = (sub_marking *)node;
		if (marking->paths != NULL)
			return marking;
		printf("ERROR: immediate loop detected, marking:\n");
		for(i=0; i<sub->no_places; i++)
			printf("%s:%d  ", sub->places[i].name, token[i]);
		printf("\n");
		exit(1);
	}
	/*
	 * insert new nodes into tree
	 */
	for(pla_nr++ ; pla_nr < sub->no_places; pla_nr++)
	{	tokens = token[pla_nr];
		if (tokens > sub->places[pla_nr].bound)
		{	printf("ERROR: number of tokens in %s(%d) exceeded upper bound %d !\n", 
				sub->places[pla_nr].name, tokens, sub->places[pla_nr].bound);
			exit(1);
		}
		*insert = node = (sub_node *)emalloc(sizeof(sub_node));
		if (sub->places[pla_nr].bound > ARRAY_LIMIT)
		{	node->bound = SPARSE_START;	/* sparse array */
			node->number = 1;
			node->next = (sub_node **)emalloc(SPARSE_START * sizeof(sp_pointer));
			lookup = (sp_pointer *)node->next;
			lookup->index = tokens;
			insert = (sub_node **)&(lookup->next);
		} else
		{	node->bound = 0;		/* array */
			node->number = sub->places[pla_nr].bound;
			node->next = (sub_node **)ecalloc((sub->places[pla_nr].bound + 1),
							sizeof(sub_node *));
			insert = (sub_node **)&(node->next[tokens]);
		}
	}
	marking = (sub_marking *)emalloc(sizeof(sub_marking));
	*insert = (sub_node *)marking;
	marking->no_paths = 0;
	marking->paths = NULL;
	sub->no_markings++;
	return marking;
}

/*
 * test_global_concession tests, if a transition is enabled (without priority)
 * the variable current_mark must be set to the current global marking
 */
int test_global_conc(mark, i_mat, h_mat, isinfinite)
	int	*mark;		/* local marking      */
	sp_place *i_mat;	/* input matrix row   */
	sp_place *h_mat;	/* inhibit matrix row */
	int    isinfinite;
{
	int	pla, value;
	int   nonzero_inputs;

	nonzero_inputs = 0;
	for(pla = 1; pla <= i_mat[0].num; pla++) {
		value = i_mat[pla].plu_id ? md_arc_mult(i_mat[pla].plu_id)
                                : i_mat[pla].value;
		if (value) {
			nonzero_inputs++;
			if (mark[i_mat[pla].num] < value)
				return 0;		/* not enough input token */
		}
	}
	for(pla = 1; pla <= h_mat[0].num; pla++) {
		value = h_mat[pla].plu_id ? md_arc_mult(h_mat[pla].plu_id) : h_mat[pla].value;
		if (value && (mark[h_mat[pla].num] >= value))
			return 0;		/* too many inhibitor token */
	}
	return (isinfinite ? nonzero_inputs:TRUE);
}

/*
 * test_concession tests if a subnet - transition is enabled (without priority)
 */
int *gh_mark = NULL;
int test_concession(sub, mark, trans )
	subnet *sub;
	int    *mark;
	int     trans;
{
	imm_t *tra = &(sub->transitions[trans]);
	sp_place *i_mat = tra->i_matrix;
	sp_place *h_mat = tra->h_matrix;

	if (gh_mark==NULL)
		gh_mark = (int*) emalloc(no_places * sizeof(int));
	local_to_global_mark(sub, mark, gh_mark );
	current_mark = gh_mark;
	if (tra->ena_id)
		if (!md_ena_function(tra->ena_id))
			return 0;
	return test_global_conc(mark, i_mat, h_mat, FALSE);
}

/*
 * fires a global transition, current_mark has to be set previously
 */
void fire_global_trans(mark, mark_next, num, c_mat)
	int	*mark;		/* marking */
	int	*mark_next;	/* new marking */
	int	num;		/* number of places in this (sub-)net */
	sp_place *c_mat;	/* C - matrix of the transition */
{
	int	pla, value;

	memmove(mark_next,mark, num * sizeof(int));	/* copy old marking */
	for(pla = 1; pla <= c_mat[0].num; pla++) {
		value = c_mat[pla].value;
		if (c_mat[pla].plu_id)
			value += md_arc_mult(c_mat[pla].plu_id);
		if (c_mat[pla].min_id)
			value -= md_arc_mult(c_mat[pla].min_id);
		mark_next[c_mat[pla].num] += value;
	}
}

/*
 * fire_transition fires a local transition
 */
void fire_transition(sub, mark, mark_next, num, c_mat)
	subnet   *sub;
	int      *mark;
	int      *mark_next;
	int       num;
	sp_place *c_mat;
{
	if (!gh_mark)
		gh_mark = (int*)emalloc(no_places * sizeof(int));
	local_to_global_mark(sub,mark,gh_mark);
	current_mark = gh_mark;
	fire_global_trans(mark,mark_next,num,c_mat);
}

/*
 * mix_paths mixes the firing paths of two transitions
 */
void mix_paths(mix_mark, new_mark, factor)
	sub_marking	*mix_mark;	/* set of old paths */
	sub_marking	*new_mark;	/* new paths */
	double		factor;		/* probability factor of new paths */
{
	int		old, new, enter;
	sub_marking	*old_mark;

	old_mark = (sub_marking *)emalloc(sizeof(sub_marking));
	memmove(old_mark, mix_mark, sizeof(sub_marking));	/* copy data */
	mix_mark->paths = (fire_path *)emalloc((old_mark->no_paths + new_mark->no_paths)
							* sizeof(fire_path));
	old = new = enter = 0;
	while((old < old_mark->no_paths) || (new < new_mark->no_paths))
	{	if ((old < old_mark->no_paths) && (new < new_mark->no_paths) &&
		    (old_mark->paths[old].marking == new_mark->paths[new].marking))
		{	mix_mark->paths[enter].marking = old_mark->paths[old].marking;
			mix_mark->paths[enter++].prob = old_mark->paths[old++].prob +
						new_mark->paths[new++].prob * factor;
			continue;
		}
		if ((old >= old_mark->no_paths) || (new < new_mark->no_paths) &&
		    (old_mark->paths[old].marking > new_mark->paths[new].marking))
		{	mix_mark->paths[enter].marking = new_mark->paths[new].marking;
			mix_mark->paths[enter++].prob = new_mark->paths[new++].prob * factor;
		} else
		{	mix_mark->paths[enter].marking = old_mark->paths[old].marking;
			mix_mark->paths[enter++].prob = old_mark->paths[old++].prob;
		}
	}
	mix_mark->no_paths = enter;
	efree((char *)(old_mark->paths));
	efree((char *)(old_mark));
}

/*
 * sub_end_markings calculates the set of submarkings reachable from one initial
 * subnet - marking and returns a pointer to the sub_marking entry in the subnet
 * tree (containing the set of firing paths with their total probabilities)
 */
sub_marking *sub_end_markings(sub, mark, enabled)
	subnet	*sub;		/* the corresponding subnet */
	int	*mark;		/* initial submarking */
	byte	*enabled;	/* set of transitions having concession in this marking */
{
	byte		*enas = enabled, *enas_next, *fire = NULL;
	sub_marking	*marking, *fire_marking, mix_marking;
	int		tra, flag = 0, ecs, i, no_fire = 0, each;
	int		*marking_next;
	double		prob_sum = 0.0, now_prob;

	marking = enter_submarking(sub, mark);		/* look for marking */
	if (marking->paths != NULL)			/* old marking */
		return marking;
	if (enas == NULL)				/* first recursion level */
	{	enas = (byte *)emalloc(sub->no_trans * sizeof(byte));
		for(tra = 0; tra < sub->no_trans; tra++)
			enas[tra] = test_concession(sub, mark, tra);
	}
	for(ecs = 0; (ecs < sub->no_ecs) && !flag; ecs++)
	{	for(i = 1; i <= sub->ecs[ecs][0]; i++)
		{	tra = sub->ecs[ecs][i];		/* transition number */
			if (enas[tra])			/* transition is enabled */
			{	if (!flag)
					fire = (byte *)emalloc(sub->ecs[ecs][0] * sizeof(byte));
				fire[no_fire++] = tra;
				flag = 1;
			}	
		}
	}
	if (!flag)					/* final submarking */
	{	marking->no_paths = 1;
		marking->paths = (fire_path *)emalloc(sizeof(fire_path));
		marking->paths[0].prob = 1.0;		/* probability 1 */
		marking->paths[0].marking = code_submarking(sub->memory,
							sub->no_places, sub->places, mark);
		return marking;
	}
	marking_next = (int  *)emalloc(sub->no_places * sizeof(int));
	enas_next    = (byte *)emalloc(sub->no_trans  * sizeof(byte));
	mix_marking.no_paths = 0;
	mix_marking.paths = NULL;
	for(i = 0; i < no_fire; i++)
	{	tra = fire[i];
		now_prob  = imm_transition_weight(sub,mark,tra);
		prob_sum += now_prob;		/* sum up probabilities */
		fire_transition(sub,mark, marking_next, sub->no_places, 
						sub->transitions[tra].c_matrix);
		/*
		 * calculate new enas
		 */
		memmove(enas_next,enas, sub->no_trans * sizeof(byte));	/* copy data */
		for(each = 0; each < sub->no_trans; each++)		/* get new enas */
		{	flag = 0;
			if (each == tra)
				flag = 1;
			else	if (enas[each])
				{	if (is_in_packed(each, sub->transitions[tra].diss))
						flag = 1;
				} else
					if (is_in_packed(each, sub->transitions[tra].enas))
						flag = 1;
			if (flag)
			enas_next[each] = test_concession(sub,marking_next,each);
		}
		fire_marking = sub_end_markings(sub, marking_next, enas_next);
		mix_paths(&mix_marking, fire_marking, now_prob);
	}
	efree((char *)(marking_next));
	efree((char *)(enas_next));
	efree((char *)(fire));
	if (enabled == NULL)
		efree((char *)(enas));
	marking->no_paths = mix_marking.no_paths;
	marking->paths = (fire_path *)emalloc(marking->no_paths * sizeof(fire_path));
	memmove(marking->paths, mix_marking.paths,marking->no_paths*sizeof(fire_path));
	for(i = 0; i < marking->no_paths; i++)
		marking->paths[i].prob /= prob_sum;		/* normalize probabilities */
	efree((char *)(mix_marking.paths));
	return marking;
}

/*
 * net_end_markings calculates the set of subnet - final markings reachable
 * from one marking and returns a pointer to an array which contains a pointer
 * to another array and a number of entries for each subnet. Each of this
 * arrays describes the set of reachable markings of the corresponding subnet
 * and the probability for this firing path.
 */
sub_marking *net_end_markings(subs, tsub, marking, disc_result)
	subnet	*subs;		/* the net's subnets */
	t_sub	*tsub;		/* the "timed transitions" subnet */
	int	*marking;	/* the marking (NULL -> take M0) */
	word	**disc_result;
	/* the disconnected places resulting marking (return par.) */
{
	subnet	*sub;
	int	net, *sub_mark, i;
	sub_marking *result, *inter;

	result = (sub_marking *)emalloc(no_subnets * sizeof(sub_marking));
	for (net = 0; net < no_subnets; net++)
	{	sub = &(subs[net]);
		sub_mark = (int *)emalloc(sub->no_places * sizeof(int));
		if (marking == NULL)
		{	for(i = 0; i < sub->no_places; i++)
				sub_mark[i] = sub->places[i].m0;
		} else
		{	for(i = 0; i < sub->no_places; i++)
				sub_mark[i] = marking[sub->places[i].number];
		}
		inter = sub_end_markings(sub, sub_mark, (byte*)NULL);
		memmove(&(result[net]), inter,  sizeof(sub_marking));
		efree((char *)(sub_mark));
	}
	if ((sub_mark = marking) == NULL)
	{	sub_mark = (int *)emalloc(no_places * sizeof(int));
		for(i = 0; i < tsub->no_places; i++)
			sub_mark[tsub->places[i].number] = tsub->places[i].m0;
	}
	*disc_result = enter_disconnected(tsub, sub_mark);
	if (marking == NULL)
		efree((char *)(sub_mark));
	return result;
}
