/***************************************************************************

 File:		expoly.c

 Purpose:	routines for manipulating expolynomial distributions
		and for computing the factors needed by the
		randomization procedure

 Programmer:	Reinhard German

 History:	Release 1: June 2, 1993	

***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "expoly.h"

extern char *emalloc();

/* set deterministic distribution */
time_l *set_det_expoly(tau)
double tau;
{
  time_l *expoly, *return_expoly;
  power_m *power;
  expo_n *expo;

  return_expoly=(time_l *) emalloc(sizeof(time_l));
  expoly = return_expoly;

  if(tau>0)
  {
      expoly->type = DETERMINISTIC;
      expoly->x1 = 0.0;
      expoly->x2 = tau;
      expoly->p = 0.0;
      expoly->power = (power_m *) emalloc(sizeof(power_m));
        power = expoly->power;
        power->m = 0;
        power->expo = (expo_n *) emalloc(sizeof(expo_n));
          expo = power->expo;
          expo->lambda = 0.0;
          expo->b = 0.0; /* adapt: vorher 1.0 */
          expo->c = 0.0;
          expo->next = NULL;
        power->next = NULL;
      expoly->next = (time_l *) emalloc(sizeof(time_l));
      expoly = expoly->next;
  }
  expoly->type = DETERMINISTIC;
  expoly->x1 = tau;
  expoly->x2 = UNENDLICH;
  expoly->p = 1.0;
  expoly->power = (power_m *) emalloc(sizeof(power_m));
    power = expoly->power;
    power->m = 0;
    power->expo = (expo_n *) emalloc(sizeof(expo_n));
      expo = power->expo;
      expo->lambda = 0.0;
      expo->b = 1.0; /* adapt: alt 0.0 */
      expo->c = 0.0;
      expo->next = NULL;
    power->next = NULL;
  expoly->next = NULL;

  return return_expoly;
} /* set_det_expoly */


/* set uniform distribution */
time_l *set_uniform_expoly(t1, t2)
double t1, t2;
{
  time_l *expoly, *return_expoly;
  power_m *power;
  expo_n *expo;

  return_expoly=(time_l *) emalloc(sizeof(time_l));
  expoly = return_expoly;

  if(t1>0)
  {
    expoly->type = UNIFORM;
    expoly->x1 = 0.0; 
    expoly->x2 = t1; 
    expoly->p = 0.0;
    expoly->power = (power_m *) emalloc(sizeof(power_m));
    power = expoly->power;
    power->m = 0;
    power->expo = (expo_n *) emalloc(sizeof(expo_n));
    expo = power->expo; 
      expo->lambda = 0.0;
      expo->b = 0.0; /* adapt: vorher 1.0 */
      expo->c = 0.0;
      expo->next = NULL;
    power->next = NULL;
    expoly->next = (time_l *) emalloc(sizeof(time_l));
    expoly = expoly->next;
  }
  expoly->type = UNIFORM;
  expoly->x1 = t1;
  expoly->x2 = t2;
  expoly->p = 0.0;
  expoly->power = (power_m *) emalloc(sizeof(power_m));
    power = expoly->power;
    power->m = 0;
    power->expo = (expo_n *) emalloc(sizeof(expo_n));
      expo = power->expo;
      expo->lambda = 0.0;
      expo->b = -t1/(t2-t1);  /* adapt: alt: 1+t1/(): */
      expo->c = 1/(t2-t1);   
      expo->next = NULL;
    power->next = (power_m *) emalloc(sizeof(power_m));
    power = power->next;
    power->m = 1;
    power->expo = (expo_n *) emalloc(sizeof(expo_n));
      expo = power->expo;
      expo->lambda = 0.0;
      expo->b = 1/(t2-t1);    /* adapt: unklar wie b errechnet wird ||| */
      expo->c = 0.0;          /* alt: -1/...*/
      expo->next = NULL;
    power->next = NULL;
  expoly->next = (time_l *) emalloc(sizeof(time_l));
  expoly = expoly->next;

  expoly->type = UNIFORM;
  expoly->x1 = t2;
  expoly->x2 = UNENDLICH;
  expoly->p = 0.0;
  expoly->power = (power_m *) emalloc(sizeof(power_m));
    power = expoly->power;
    power->m = 0;
    power->expo = (expo_n *) emalloc(sizeof(expo_n));
      expo = power->expo;
      expo->lambda = 0.0;
      expo->b = 1.0;/* adapt: alt 0.0 */
      expo->c = 0.0;
      expo->next = NULL;
    power->next = NULL;
  expoly->next = NULL;

  return return_expoly;
} /* set_test_expoly */


/* print an expolynomial distribution: */
void print_expoly(expoly)
  time_l *expoly;
{
  power_m *power;
  expo_n *expo;
  unsigned l;

  for (l = 0; expoly; expoly = expoly->next, l++) 
  {
    printf("---------------------------------------------------------------\n"); 
    printf("interval %d: ",l);
    printf("\tx1=%.16g, ", expoly->x1);
    if (expoly->next)
      printf("x2=%.16g, ", expoly->x2);
    else 
      printf("x2=infinity, ");
    printf("p=%.16g\n", expoly->p);
    for (power = expoly->power; power; power = power->next)
    {
      printf("power m=%d:", power->m); 
      for (expo = power->expo; expo; expo = expo->next)
      {
	  if(expo == power->expo)
	      printf("\tlambda=%.16g, b=%.16g, c=%.16g\n",
		     expo->lambda, expo->b, expo->c);
	  else
	      printf("          \tlambda=%.16g, b=%.16g, c=%.16g\n",
		     expo->lambda, expo->b, expo->c);
    
      }
    }
  }
  printf("---------------------------------------------------------------\n"); 
} /* print_expoly */

time_l *define_file_access(f_name, access_type)
char *f_name;
int access_type;
{
  time_l *return_expoly;

  return_expoly=(time_l *) emalloc(sizeof(time_l));
 
  return_expoly->type = FILE_REF;
  return_expoly->f_name = f_name;
  return_expoly->sampling_type = access_type;
  return_expoly->next = NULL;

  return return_expoly;
} /* define_file_access */

