%p 4500
%a 4000
%n 1000
%o 5000
%{
/******************************************************************************
*                                                                             *
*  Aufgabe        :     Das Programm dient der lexikalischen Analyse der      *
*                       Definitionen der markierungsabhaengigen Schaltraten   *
*			Es arbeitet als Scanner fuer den im Programm          *
*			pars_TN.y realisierten Parser.           	      *
*                                                                             *
*  Programmierer  :     Enrik Baumann                                         *
*                                                                             *
*  Datum          :     Juni 1991                                             *
*                                                                             *
*  Modifizierung  :     Christian Luehe				     	      *
*									      *
*  Datum 	  :     August 1992					      *
*									      *
*  Modifizierung  :	Armin Zimmermann				      *
*									      *
*  Datum	  :	Februar 1993					      *
*									      *
*  Modifizierung  :	Ch. Kelling					      *
*									      *
*  Datum	  :	Mai 1994					      *
*									      *
*  Modifizierung  :	J . Schneider					      *
*									      *
*  Datum	  :	Juli 94						      *
*									      *
******************************************************************************/
%}

%{
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "y.tab.h"
#include "expoly.h"
#include "netio.h"
#include "pars_TN.h"
%}

%{
#define COMP_OP(x)  { strcpy(yylval.t_str, x); return(COMP_OPER); }
#define ARC_TYPE(x) { yylval.t_int = x; return(KW_ARCTYPE); }
%}

%{
/* external variable definition from pars_TN.y */
extern	FILE		*mdf, *erc, *rdy;
extern	char		defname[], ercname[], rdyname[], *res_proc;
extern	int		no_places, no_trans, no_mpar, no_del_par, no_helps;
extern	int		*now_deter, deter_on, reward_on, pmf_on;
extern	place		*places;
extern	name_nr		*m_par;
extern	name_delay	*d_par;
extern	transition	*transitions;
extern 	int     	SyntaxOnly;
extern 	int     	Simulation;
extern  char		error_text[];

extern  void yyerror();

extern char *vlstrcpy();
extern char *vlstrcat();
extern char *strdup();
extern int strncmp();
extern char *getenv();

/* local declarations */
#ifdef __NetBSD__
int 	yylineno=1;
#endif

int	walk;
char 	no_markings = FALSE;
char    *temp_f_name;
%}
D    [0-9]
E    [Ee][-+]?{D}+
%% 
"-- DEFINITION OF PARAMETERS:"					{
		deter_on	= TRUE;
		no_markings	= TRUE;
		return (S_PARAM);
	} /*--------------------------------------------------*/
"-- MARKING DEPENDENT FIRING DELAYS FOR EXP. TRANSITIONS:"	{	
	/* begin of exp. trans */
		deter_on	= FALSE;
		no_markings	= FALSE;
	if( Simulation ) {
		/* nix */
	} else {
		fprintf(mdf, "\n/* this function returns the marking dependent firing delay\n");
		fprintf(mdf, "   or weight of a transition */\n");
		fprintf(mdf, "double md_rate(id)\n");
		fprintf(mdf, "\tint id;\t\t/* id of transition */\n");
		fprintf(mdf,"{\n");
		fprintf(mdf,"\tdouble rate;\n\n");
		fprintf(mdf,"\tswitch(id)\n\t{\n");
	} /* if */
		return (S_EXP);
	} /*--------------------------------------------------*/
"-- MARKING DEPENDENT FIRING DELAYS FOR DET. TRANSITIONS:"	{
		deter_on	= FALSE;
		no_markings	= FALSE;
		return (S_DET);
	} /*--------------------------------------------------*/
"-- PROBABILITY MASS FUNCTION DEFINITIONS FOR GEN. TRANSITIONS:"	{
		deter_on	= FALSE;
		no_markings	= FALSE;
		pmf_on		= TRUE;
		return (S_GEN);
	} /*--------------------------------------------------*/
"-- MARKING DEPENDENT WEIGHTS FOR IMMEDIATE TRANSITIONS:"	{ 
	/* begin of weight definitions */
		deter_on	= TRUE;
		no_markings	= FALSE;
		pmf_on		= FALSE;
		return (S_WEIGHTS);
	} /*--------------------------------------------------*/
"-- ENABLING FUNCTIONS FOR IMMEDIATE TRANSITIONS:"		{	
	/* begin of guard definition */
	int i, j, no_interval, max_intervals = 1;
	int first_b, this_p_printed, one_p_printed;
	time_l *pmf_def_l;
	expo_n *expo_n_l;
	power_m *power_m_l;
	double func();

	no_markings	= FALSE;

	if( Simulation ) {
#ifdef debug
		fprintf(stderr, 	
			"Printing the 'cases' into md_rate, checking %d transitions\n", no_trans);
#endif
		fprintf(mdf,
			"\n/* these functions are used to compute y=pmf_func(x) for all pmf-functions in the different intervals*/\n");

/* loop for the GEN-transitions to do first */
		for (i = 0; i < no_trans; i++) {
		    if (transitions[i].id) 
			if (transitions[i].ok) {
if (transitions[i].type == GEN){
  pmf_def_l = transitions[i].pmf_def;
  if (pmf_def_l->type == EXPOLY){
    integrate(pmf_def_l, INTEGRATE_NORMAL);
    no_interval = 1;
    while (pmf_def_l != NULL){
      fprintf(mdf,
        "\ndouble func_tra_%d_int_%d(x)\n\tdouble x;\n{\n\treturn ",
	transitions[i].id, no_interval); 
      power_m_l = pmf_def_l->power;
      one_p_printed = FALSE;
      while (power_m_l != NULL){
        this_p_printed = FALSE;
        first_b = TRUE;
        expo_n_l = power_m_l->expo;
        while (expo_n_l != NULL){
	  if (expo_n_l->b != 0.0){
	    if (first_b){
	      if (power_m_l->m == 0)
	        fprintf(mdf,"(");
	      if (power_m_l->m == 1)
	        fprintf(mdf,"x*(");
	      if (power_m_l->m > 1)
	        fprintf(mdf,"pow(x,(double)%d)*(", power_m_l->m);
	      first_b = FALSE;
	      one_p_printed = TRUE;
	      this_p_printed = TRUE;}
	    else 
	      fprintf(mdf,"+");
	    fprintf(mdf,"%f", expo_n_l->b);
	    if (expo_n_l->lambda != 0.0){
	      fprintf(mdf,"*exp(-%f*x)", expo_n_l->lambda);}
	  }/* if b!= 0.0*/
	  expo_n_l = expo_n_l->next;
	  if (!expo_n_l && this_p_printed)
	    fprintf(mdf,")");
        }/* while expo */
        power_m_l = power_m_l->next;
        if (power_m_l && this_p_printed){ 
	  fprintf(mdf,"+");
	  this_p_printed = FALSE;}
        if (this_p_printed)
	  no_interval ++;
	if (no_interval > max_intervals)
	  max_intervals = no_interval;
        if (!power_m_l){
	  if (!one_p_printed){
	    no_interval ++;
	    fprintf(mdf,"(0.0)");}
	  fprintf(mdf,";\n}\n");}
      }/* while power */
      pmf_def_l = pmf_def_l->next;
    } /* while pmf_def_l ...*/
  } /* if == EXPOLY */
  else if (pmf_def_l->type == DETERMINISTIC){
    fprintf(mdf,
        "\ndouble func_tra_%d_int_1(x)\n\tdouble x;\n{\n\treturn ",
	transitions[i].id); 
    if (pmf_def_l->x2 > 0.0) /* first upper bound is not infinity */
      fprintf(mdf,"\t(%f);\n}\n", pmf_def_l->x2);
    else 
      fprintf(mdf,"\t(0.0);\n}\n"); /* special case det=0.0*/
  } /* if DETERMINISTIC */
  else if (pmf_def_l->type == UNIFORM){
    fprintf(mdf,
        "\ndouble func_tra_%d_int_1(x)\n\tdouble x;\n{\n\treturn ",
	transitions[i].id);
    if (pmf_def_l->next->x2 > 0.0) /* second upper bound is not infinity */
      fprintf(mdf,"\trnd_uniform(%f,%f);\n}\n", 
	pmf_def_l->next->x1, pmf_def_l->next->x2);
    else /* special case: uniform start at 0.0 */
      fprintf(mdf,"\trnd_uniform(0.0,%f);\n}\n", pmf_def_l->x2);
  } /* if UNIFORM */
} /* if transition == GEN */	
} /* if transition.ok */ 
			else {
			    fprintf(stderr, 
				    "Transition '%s' is not OK!\n",
				    transitions[i].name);
			    exit(yylineno);
			}
		} /* for all transitions */	
		if (max_intervals == 1)
		  fprintf(mdf, "\ndouble (*pmf_fctn[1])();\n");
		/* falls keine pmf-Definitionen im Netz, Compiler beruhigen*/
		else
		  fprintf(mdf, "\ndouble (*pmf_fctn[%d])();\n", 
		    no_trans*(max_intervals-1));
		fprintf(mdf, "\n/* pmf limits are organized for each general transition as */\n");
		fprintf(mdf, "/* limit_<tra_nr>[interval][x_lower_limit, y_lower_limit, y_upper_limit]*/\n");
		for (i=0; i<no_trans; i++){
		  pmf_def_l = transitions[i].pmf_def;
		  if ((transitions[i].type == GEN)
		   && (pmf_def_l->type == EXPOLY)){
		    fprintf(mdf,"static double pmf_limits_tra_%d[%d][3] ={\n",
		      i,(max_intervals -1));
		    while (pmf_def_l) {
		      fprintf(mdf,"\t{%f,\t%f,\t%f}", 
			pmf_def_l->x1, func(pmf_def_l, pmf_def_l->x1),
			func(pmf_def_l, pmf_def_l->x2));
		      pmf_def_l = pmf_def_l->next;
		      if (pmf_def_l)
			fprintf(mdf,",\n");
		      }  /* while pmf_def_l */
		    fprintf(mdf,"};\n");
		    } /* type == GEN and EXPOLY */
		  } /* fo all trans */
		fprintf(mdf, 
	"\n/* this function prepares the calls of pmf-functions via pointers*/\n");
		fprintf(mdf, "void prepare_pmfs()\n{\n");
		for (i=0; i<no_trans; i++)
		  if (transitions[i].type == GEN){
		    pmf_def_l = transitions[i].pmf_def;
		    j = 0;
		    while ((pmf_def_l) && (pmf_def_l->type == EXPOLY)) {
		      fprintf(mdf,"\tpmf_fctn[%d] = func_tra_%d_int_%d;\n",
			(transitions[i].id-1)*(max_intervals-1)+j,
			transitions[i].id,j+1);
		      pmf_def_l = pmf_def_l->next;
		      j ++;}}
		fprintf(mdf, "\n}\n");

		fprintf(mdf, 
	"\n/* this function returns the marking dependent\n");
		fprintf(mdf, 
	"   firing delay or the weight of a transition */\n");
		fprintf(mdf, "double md_rate(id)\n");
		fprintf(mdf, "\tint id;\t\t/* id of transition */\n");
		fprintf(mdf,"{\n");
		fprintf(mdf,"\tint i;\n");
		fprintf(mdf,"\tdouble rate, x1, x2, rv;\n");
		fprintf(mdf, "\tdouble root_bisection();\n\n");
		fprintf(mdf,"\tswitch(id)\n\t{\n");

/* loop for all transitions to do second */
		for (i = 0; i < no_trans; i++) {
		    if (transitions[i].id) 
			if (transitions[i].ok) {
fprintf(mdf, "\tcase %2d:\n", transitions[i].id);
if (transitions[i].type == GEN) {
  pmf_def_l = transitions[i].pmf_def;
  if (pmf_def_l->type == EXPOLY) {
    fprintf(mdf, "\t\trv=rnd_uniform(0.0,1.0);\n");
    fprintf(mdf, "\t\tfor(i=0;i<%d;i++)  /* search for interval */\n",max_intervals-1);
    fprintf(mdf, "\t\t  if ((rv>=pmf_limits_tra_%d[i][1]) && (rv<pmf_limits_tra_%d[i+1][1]))\n\t\t    break;\n", i, i);
    fprintf(mdf, "\t\tif ((rv>=pmf_limits_tra_%d[i][2]) && (rv<pmf_limits_tra_%d[i+1][1]))\n", i, i);
    fprintf(mdf, "\t\t\t/* special case: Sprungstelle */\n");
    fprintf(mdf, "\t\t\trate = pmf_limits_tra_%d[i+1][0];\n\t\telse{\n", i);
    fprintf(mdf, "\t\t\tx1 = pmf_limits_tra_%d[i][0];\n", i);
    fprintf(mdf, "\t\t\tx2 = pmf_limits_tra_%d[i+1][0];\n", i);
    fprintf(mdf, "\t\t\tbrac_root(id,(i+1),&x1,&x2,rv,%d);\n",max_intervals-1);
    fprintf(mdf, "\t\t\trate = root_bisection(id,(i+1),x1,x2,rv,%d);}\n",
	  max_intervals-1);
    fprintf(mdf, "\t\tbreak;\n");
    }/* if EXPOLY*/ 
  if (pmf_def_l->type == UNIFORM) {
    fprintf(mdf, "\t\trate = func_tra_%d_int_1(1.0); /*dummy arg*/\n",transitions[i].id);
    fprintf(mdf, "\t\tbreak;\n");
    } /* if UNIFORM */
  if (pmf_def_l->type == DETERMINISTIC) {
    fprintf(mdf, "\t\trate = func_tra_%d_int_1(1.0);/*dummy arg*/\n",transitions[i].id);
    fprintf(mdf, "\t\tbreak;\n");
    } /* if DETERMINISTIC */
  if (pmf_def_l->type == FILE_REF){
    fprintf(mdf,
        "\t\trate = get_delay_from_file(\"%s\",%d,%d);\n", pmf_def_l->f_name, 
	transitions[i].id, pmf_def_l->sampling_type);
    fprintf(mdf, "\t\tbreak;\n");
  } /* if FILE */

  }/* if GEN*/ 
else
  fprintf(mdf, "%s\n", transitions[i].def);
} /* if transition.ok */ 
			else {
			    fprintf(stderr, 
				    "Transition '%s' is not OK!\n",
				    transitions[i].name);
			    exit(yylineno);
			}
		}		
		fprintf(mdf, "\tdefault:\n\t\tprintf(\"bad id.\\n\");\n");
		fprintf(mdf, "\t\t\texit(1);");
		fprintf(mdf, "\n\t}\n\treturn (rate);\n}\n\n");
		fprintf(mdf, "\n/* this function returns the marking dependent \n");
		fprintf(mdf, "   enabling function of a transition */\n");
		fprintf(mdf, "int md_ena_function(id)\n");
		fprintf(mdf, "\tint id;\t\t/* id of transition */\n");
		fprintf(mdf,"{\n");
		fprintf(mdf,"\tint fire;\n\n");
		fprintf(mdf,"\tswitch(id)\n\t{\n");

	} else { /* Simulation */

#ifdef debug
		fprintf(stderr, 
			"Printing the 'cases' into md_rate, checking %d transitions\n", no_trans);
#endif
		for (i = 0; i < no_trans; i++) {
/*
	if (description->trans[i_trans].delay_type == DELAYPAR)
*/
		    if (transitions[i].id)
			if (transitions[i].ok) {
			    fprintf(mdf, "\tcase %2d:\n", transitions[i].id);
			    if( transitions[i].def ) {
			        fprintf(mdf, "%s\n", transitions[i].def);
			    } /* if */
			} else {
			    fprintf(stderr, "Transition '%s' is not OK!\n", transitions[i].name);
			    exit(yylineno);
			}
		}		

		fprintf(mdf, "\tdefault:\n\t\tprintf(\"bad id.\\n\");\n");
		fprintf(mdf, "\t\t\texit(1);");
		fprintf(mdf, "\n\t}\n\treturn (rate);\n}\n\n");
		fprintf(mdf, "\n/* this function returns the marking dependent \n");
		fprintf(mdf, "   enabling function of a transition */\n");
		fprintf(mdf, "int md_ena_function(id)\n");
		fprintf(mdf, "\tint id;\t\t/* id of transition */\n");
		fprintf(mdf,"{\n");
		fprintf(mdf,"\tint fire;\n\n");
		fprintf(mdf,"\tswitch(id)\n\t{\n");
	} /* if */
		return (S_ENAFS);
	} /*--------------------------------------------------*/
"-- MARKING DEPENDENT ARC CARDINALITIES:" 			{
	/* begin of m. d. arc definition */
		no_markings	 = FALSE;
		fprintf(mdf, "\t\tdefault:printf(\"bad id.\\n\");\n");
		fprintf(mdf, "\t\t\texit(1);");
		fprintf(mdf, "\n\t}\n\treturn (fire);\n}\n\n");
		fprintf(mdf, "\n/* this function returns the marking dependent \n");
		fprintf(mdf, "   arc cardinality of an input, output or inhibitor arc */\n");
		fprintf(mdf, "int md_arc_mult(id)\n");
		fprintf(mdf, "\tint id;\t\t/* id of arc */\n");
		fprintf(mdf,"{\n");
		fprintf(mdf,"\tdouble rate;\n\n");
		fprintf(mdf,"\tswitch(id)\n\t{\n");
		return (S_ARC);
	} /*--------------------------------------------------*/
"-- REWARD MEASURES:"						{	
	/* begin of reward definition */
		deter_on	= FALSE;
		no_markings	= FALSE;
		reward_on	= TRUE;
		fprintf(mdf, "\t\tdefault:printf(\"bad id.\\n\");\n");
		fprintf(mdf, "\t\t\texit(1);");
		fprintf(mdf, "\n\t}\n\treturn ((int)(rate));\n}\n\n");

		fprintf(erc, "/*\tFile:    %s\n", ercname);
		fprintf(erc, "\tPurpose: procedures for reward measures */\n\n");
		fprintf(erc, "#include <stdio.h>\n");
		fprintf(erc, "#include <math.h>\n\n");
	if( Simulation ) {
		fprintf(erc, "#include \"../Slave/sim_slave_types.h\"\n");
                fprintf(erc, "#include \"num_utils.h\"\n");
		fprintf(erc, "/* avoid 0^0 problem */\n");
/*
		fprintf(erc, "double my_pow(a,b)\n\tdouble a,b;\n");
		fprintf(erc, "{\n\tif ((a==0.0) && (b==0.0))\n");
		fprintf(erc, "\t\treturn (1.0);\n\treturn pow(a,b);\n}\n");
*/
                fprintf(erc, "extern place *places;\n");
                fprintf(erc, "extern int no_measures;\n");
		fprintf(erc, "extern double my_pow();\n");
		fprintf(erc, "extern int update_measure();\n\n"); 
                fprintf(erc, "/* select measures marking depended*/\n");
                fprintf(erc, "void select_measure(meas_id, old_tok)\n");
                fprintf(erc, "\tint meas_id, old_tok;\n{\n");
                fprintf(erc, "\tswitch(meas_id)\n\t{\n");		
	} else {
		fprintf(erc, "/* avoid 0^0 problem */\n");
		fprintf(erc, "double my_pow(a,b)\n\tdouble a,b;\n");
		fprintf(erc, "{\n\tif ((a==0.0) && (b==0.0))\n");
		fprintf(erc, "\t\treturn (1.0);\n\treturn pow(a,b);\n}\n\n");
		fprintf(erc, "extern unsigned digits;\n");
		fprintf(erc, "extern double help[];\n\n");
		fprintf(erc, "extern int act_marking();\n");
		fprintf(erc, "void sum_prob(prob)\n\tdouble prob;\n{\n");
	} /* if */
		return (S_REWARD);
	} /*--------------------------------------------------*/
"-- END OF SPECIFICATION FILE"				{	
	/* end of file */
	if( Simulation ) {
		fprintf(erc, "\t}\n}\n\n");	
                fprintf(rdy, "\n/*\tFile:    %s\n", rdyname);
                fprintf(rdy, 
			"\tPurpose: procedures for finishing distr. simulation */\n\n");
		fprintf(rdy, "#include <stdio.h>\n");
                fprintf(rdy, "#include \"sim_master_types.h\"\n");
                fprintf(rdy, "#include \"num_utils.h\"\n");
                fprintf(rdy, "extern place *places;\n");
                fprintf(rdy, "extern int no_measures, conf_level;\n");
             	fprintf(rdy, 
			"extern double eps_max, out_value();\n");
                fprintf(rdy, 
			"/* calculate reward measures and write <net>.RESULTS */ \n");
                fprintf(rdy, "void calc_measures(file)\n");
                fprintf(rdy,
        		"\tFILE *file;\n{\n\tdouble value, x0, x1, x2, prec=0.0;\n\tint selector, i;\n");
                fprintf(rdy, "\tvoid Globals_out();\n");
                fprintf(rdy, "\tdouble out_value();\n");
		if (res_proc != NULL)
           		fprintf(rdy, "%s", res_proc);
                fprintf(rdy, 
  			"\n\tfprintf(file,\"---------------------------------------------\\n\");");
                fprintf(rdy, 
 			"\n\tfprintf(file,\"Statistical characteristics for all measures:\\n\\n\");");
                fprintf(rdy, 
			"\n\tfprintf(file,\"Confidence level: %%d\\n\", conf_level);");
                fprintf(rdy, 
			"\n\tfprintf(file,\"Maximal relative error: %%f\\n\", eps_max);");
                fprintf(rdy, "\n\tGlobals_out();\n");
		fprintf(rdy, "\tfor(i=0; i<no_measures; i++)\n");
                fprintf(rdy, "\t\tMeasure_Results_out(i);\n");
		fprintf(rdy, "\tfclose (file);\n");
                fprintf(rdy, "}\n");
	} else {
		fprintf(erc, "}\n\ndouble help[%d];\n\n", no_helps+1);
		fprintf(erc, "/* calculate result parameters and write <net>.RESULTS */ \n");
		fprintf(erc, "void calc_results(file)\n");
		fprintf(erc, "\tFILE *file;\n{\n\tdouble value;\n\n");
		if (res_proc != NULL)
			fprintf(erc, "%s", res_proc);
		fprintf(erc, "}\n");
	} /* if */
		return (E_FILE);
	} /*--------------------------------------------------*/
"DELAYPAR_DEF"	return (KW_PARAM);
"EXP_DELAY"	return (KW_EXP);
"DET_DELAY"	return (KW_DET);
"GEN_DELAY"	return (KW_GEN);
"WEIGHT"	return (KW_WEIGHT);
"FUNCTION"	return (KW_FUNC);
"CARDINALITY"	return (KW_CARD);
"MEASURE"	return (KW_MEAS);
"INPARC"	ARC_TYPE(INPUT);
"OUTPARC"	ARC_TYPE(OUTPUT);
"INHARC"	ARC_TYPE(INHIBIT);  /*---------------------------------------*/
^"--".*		;			/* skip comment lines */
[ \t]+	;			/* skip blanks, tabs, newlines */
"\n"		{

#if defined(__NetBSD__) || defined(linux)
		yylineno++
#endif
		;} /*--------------------------------------------------*/
"DETERMINISTIC"	return( KW_PMF_DET );
"UNIFORM"	return (KW_PMF_UNI);
"WEIBULL"	return (KW_PMF_WEI);
"NORMAL"	return (KW_PMF_NOR);
"FILE_REF"	return (KW_PMF_FILE);
"SEQUENCE"	return (KW_FILE_SEQ);
"SEQU_REV"	return (KW_FILE_REV);
"RAND_UNI"	return (KW_FILE_UNI);
"x^"		return (XPOT);
"e^"		return (EPOT);
"R"		return (R);
"I"		return (I);
"x"		return (X);
":"             return (COL);
";"		return (SEM);
","		return (COM);
"P{"		return (PROBABILITY);
"E{"		return (EXPECTED);
"}"		return (CBR);
"IF"		return (IF);
"ELSE"		return (ELSE);
"NOT"		return (LNO);
"OR"		return (LOR);
"AND"		return (LAN);
"("		return (OPA);
")"		return (CPA);
"["		return (EOPA);
"]"		return (ECPA);
"="		COMP_OP("==");
"/="		COMP_OP("!=");
">"		COMP_OP(">" );
"<"		COMP_OP("<" );
">="		COMP_OP(">=");
"<="		COMP_OP("<=");
"+"		return(PLUS);
"-"		return(MINUS);
"*"		if( ! pmf_on ) return(MULTIPLY);
"/"		if( ! pmf_on ) return(DIVIDE);
"^"		return(POWER);  /*---------------------------------------*/

"#"[a-zA-Z][_a-zA-Z0-9]*		{	/* place */
	/*
		if (no_markings) 
		{
			yyerror ("Markings are not allowed in definitions for delay parameters.");
		}
	*/
		for (walk = 0; walk < no_places; walk++)
			if (!strcmp(yytext+sizeof(char), places[walk].name))
				break;
		if (walk >= no_places)
		{	
			sprintf(error_text, "Place '%s' does not exist", 
				yytext);
			yyerror("");
		}
		yylval.t_int = walk;
		if (deter_on)
			now_deter[walk] = 1;
		return (MARKING);
	} /*--------------------------------------------------*/
"#"[^a-zA-Z]			{	/* bad place name */
		sprintf(error_text, "'%s' is an illegal place name", yytext);
		yyerror("");
	} /*--------------------------------------------------*/
[a-zA-Z][_a-zA-Z0-9]*		{	/* transition or place name, marking or delay parameter or even file name (ck)*/
		for (walk = 0; walk < no_trans; walk++)	/* transition ? */
			if (!strcmp(yytext, transitions[walk].name))
				break;
		if (walk < no_trans)
		{	yylval.t_int = walk;
			return (TRA_NAME);
		}
		for (walk = 0; walk < no_places; walk++)	/* place ? */
			if (!strcmp(yytext, places[walk].name))
				break;
		if (walk < no_places)
		{	yylval.t_int = walk;
			return (PLA_NAME);
		}
		for (walk = 0; walk < no_mpar; walk++)		/* marking parameter ? */
			if (!strcmp(yytext, m_par[walk].name))
				break;
		if (walk < no_mpar)
		{	sprintf(yylval.t_str, "%d", m_par[walk].val);
			return (INT_VALUE);
		}
		for (walk = 0; walk < no_del_par; walk++)		/* delay parameter ? */
			if (!strcmp(yytext, 
				    d_par[walk].name+(strlen("DSPN_"))))
				break;
		if (walk < no_del_par)
		{	sprintf(yylval.t_str, "%s", d_par[walk].name);
#ifdef debug
			fprintf(stderr, "\tscanner returning PAR_NAME %s\n",
				d_par[walk].name);
#endif 
			return (PAR_NAME);
		}
		if (reward_on)
		{	strcpy(yylval.t_str, yytext);
			return (MEASURE_NAME);
		}

		sprintf(error_text, 
		"Unknown identifier '%s'\nKeyword or identifier expected.\n",
			yytext);
		yyerror("");
	} /*--------------------------------------------------*/
"//"[_/.a-zA-Z0-9]*		{	/* absolute filename or env. variable+filename*/
		if (fopen (yytext, "r") != NULL)
		{	strcpy(yylval.t_str, yytext);
			return (SAMP_FILE);
		}
		else {
		sprintf(error_text, 
		"File '%s' doesn't exist or is read protected.\n",
			yytext);
		yyerror("");
		}
	}/*--------------------------------------------------*/
"$"[_/.a-zA-Z0-9]*		{	/* env. variable+filename*/
	    vlstrcat (&temp_f_name, getenv("MODELDIR"));
	    vlstrcat(&temp_f_name, strstr(yytext,"/"));
	    if (fopen (temp_f_name, "r") != NULL)
               {
		strcpy(yylval.t_str, yytext);
		temp_f_name = NULL;
		return (SAMP_FILE);
		}
	    else {
		sprintf(error_text, 
		"File '%s' doesn't exist or is read protected.\n",
			temp_f_name);
		yyerror("");
		}
	} /*--------------------------------------------------*/
[0-9]+	{	/* integer constant */
		if (pmf_on) {
			sscanf(yytext,"%i",&yylval.uval);
	   		return(POS_INT);
		} else {
			strcpy(yylval.t_str, yytext);
			return (INT_VALUE);
		}
	} /*--------------------------------------------------*/
{D}+"."{D}*({E})?  |
{D}*"."{D}+({E})?  |
{D}+{E}	{	/* real constant */
		if (pmf_on) {
			sscanf(yytext,"%le",&yylval.dval);
        	        return(POS_REAL);
		} else {
			strcpy(yylval.t_str, yytext);
			return (REAL_VALUE);
		} /* if */

	} /*--------------------------------------------------*/
"-"{D}+"."{D}*({E})?  |
"-"{D}*"."{D}+({E})?  |
"-"{D}+{E}            {sscanf(yytext,"%le",&yylval.dval);
                        return(NEG_REAL);
                      }
%%
/*
void
print_error(str1, str2)
char	*str1, *str2;
{
	char	inter[100];

	sprintf(inter, str1, str2);
	if (SyntaxOnly) {
	    if (islower(inter[0]))
		inter[0] = toupper(inter[0]);
	    printf("%s. \n", inter);
	}
	else
	    printf("Error in line %d of %s: %s.\n", 
		   yylineno, defname, inter);
}
*/

