/*********************************************************************************

	Programm	:	vlstring Bibliotheksfunktionen

	Funktionen	:	char *vlstrcpy(char **dst, char *src)
                                - reserviert genuegend Speicher in 'dst' und
				  kopiert 'src' anschliessend nach 'dst'.

                                char *vlstrcat(char **dst, char *src)
				- vergroessert den Speicher in 'dst' entsprechend
				  der Groesse von 'src' und kopiert 'src' an das
				  Ende von 'dst'.

	Aufgabe		:	Die Funktionen erweitern die String Operationen
                                strcpy() und strcat() um die Alloziierung von
				zusaetzlichem Speicher, sollte der schon in 'dst'
				vorhandene nicht ausreichen.

				Den Funktionen muss die jeweils die Addresse des 
				String-Pointers uebergeben werden (also '&buffer'
				statt nur 'buffer').  'vlstrcat' in einen
				'(char *)NULL' erzeugt unvorhersagbare Ergebnisse.
				siehe auch: pars_MDF.y

				Das Programm muss mit einem ANSI-C Compiler uebersetzt
				werden (z.B. gcc).

	Programmierer	:	Ulrich Hertlein

	Datum		:	Oktober 1992

*********************************************************************************/

#include <stdio.h>
#include <malloc.h>
#include <string.h>

#include "portab.h"

/* display error message and exit */
static void leave(msg)
char *msg;
{
   if( msg )
      fprintf(stderr,"%s\n", msg);

   exit( 1 );
}


/* copy 'src' to 'dst' with proper memory allocation */
char *vlstrcpy(dst, src)
char **dst,  *src;
{

   /* check passed variables */
   if( dst == (char **)NULL )
      leave("vlstrcpy: invalid NULL pointer passed");

   /* free allocated memory (if any) */
   if( *dst )
      free(*dst);

   /* allocate memory & copy string */
   if( (*dst = (char *)strdup(src)) == NULL )
      leave("vlstrcpy: out of virtual memory");

   return( *dst );
}


/* append src to dst with proper memory allocation */
char *vlstrcat(dst, src)
char **dst,  *src;
{
   int len = strlen(src) + 1;

   /* check passed variables */
   if( dst == (char **)NULL )
      leave("vlstrcat: invalid NULL pointer passed");

   /* allocate new memory */
   if( *dst )
   {
      /* resize string area */
      len += strlen(*dst);

      /* re-allocate string area */
      *dst = (char *)realloc(*dst, len);
   }
   else
   {   /* allocate string area */
      *dst = (char *)malloc(len);
      **dst = 0;
   }

   /* check allocation */
   if( *dst == (char *)NULL )
      leave("vlstrcat: out of virtual memory");

   /* append string */
   strcat(*dst,src);

   return( *dst );
}
