/*****************************************************************************

Programm	:	struct

Komponente	:	struct_subnet.c
			Ermittlung der Subnetzstruktur des DSPN
			Ausgabe der Ergebnisse in <netname>.STRUCT

Programmierer	:	Armin Zimmermann
Datum		:	Juni 1992
Aenderung	:	Thu Mar  7 15:12:50 MET 1994

*****************************************************************************/

#include <stdio.h>
#include <math.h>

#include "struct_types.h"			/* definitions and types */
#include "struct_extern.h"			/* global definitions */
#include "portab.h"

#include <netio.h>


/*	this function is used as the parameter-function for sparse_function
	to calculate the c-matrix = o_matrix - i_matrix			*/
void
minus(a, b, r)
	sp_place *a, *b, *r;
{
	if (a == NULL)
	{	r->min_id = b->plu_id;
		r->plu_id = 0;
		r->value =  - b->value;
		return;
	}
	if (b == NULL)
	{	r->min_id = 0;
		r->plu_id = a->plu_id;
		r->value =  a->value;
		return;
	}	
	if (a->min_id || b->min_id)
	{	printf("Error in minus.\n");
		exit(1);
	}
	r->min_id = b->plu_id;
	r->plu_id = a->plu_id;
	r->value = a->value - b->value;
}

/*	sparse_function_row obtains a new sparse row as a result of the 
	parameter-function of two sparse rows				*/
sp_place *
sparse_function_row(row1, row2, function)
	sp_place *row1, *row2;
	void	(* function)();
{
	sp_place *now1, *now2, *last3, *new3;
	sp_place *row3;				/* row3 = function(row1, row2) */

	now1 = row1;
	now2 = row2;
	row3 = new3 = last3 = NULL;
	while(now1 != NULL || now2 != NULL)
	{	new3 = (sp_place *)nMalloc(sizeof(sp_place));
		if (now1 == NULL)			/* no more 1-entries */
		{	(*function)(NULL, now2, new3);
			new3->num = now2->num;
			now2 = now2->next;
		} else if (now2 == NULL)		/* no more 2-entries */
		{	(*function)(now1, NULL, new3);
			new3->num = now1->num;
			now1 = now1->next;
		} else if (now1->num == now2->num)	/* same entries */
		{	(*function)(now1, now2, new3);
			new3->num = now1->num;
			now1 = now1->next;  now2 = now2->next;
		} else if (now1->num < now2->num)	/* 1-entry */
		{	(*function)(now1, NULL, new3);
			new3->num = now1->num;
			now1 = now1->next;
		} else     				/* 2-entry */
		{	(*function)(NULL, now2, new3);
			new3->num = now2->num;
			now2 = now2->next;
		}
		if (new3->value || IS_MD(new3))
		{	if (last3 == NULL)
				row3 = new3;		/* first entry */
			else
				last3->next = new3;	/* next entry */
			last3 = new3;
		} else
		{	free(new3);			/* no new entry */
			new3 = last3;
		}
	}
	if (new3 != NULL)
		new3->next = NULL;			/* last entry */
	return row3;
}

/*	sparse_function obtains a new sparse matrix as a result of the 
	parameter-function of two sparse matrices			*/
sp_place **
sparse_function(matrix1, matrix2, no_elem, function)
	sp_place **matrix1, **matrix2;
	int	no_elem;
	int	(* function)();			/* arguments: (int, int) */
{
	int elem_nr;
	sp_place **matrix3;			/* matrix3 = function(matrix1, matrix2) */

	matrix3 = (sp_place **)nMalloc(no_elem * sizeof(sp_place *));
	for(elem_nr = 0; elem_nr < no_elem; elem_nr++)
		matrix3[elem_nr] = sparse_function_row(matrix1[elem_nr],
						matrix2[elem_nr], function);
	return matrix3;
}

/*	found_transition enters a new transition into a subnet,
	deletes the arcs connected with it and marks it as deleted.	*/
void
found_transition(act_subnet, tra_num, entry_num)
	subnet 	*act_subnet;
	int 	tra_num, entry_num;
{
	act_subnet->transitions[entry_num] = tra_num;		/* real number */
	act_subnet->i_matrix[entry_num] = i_matrix[tra_num];	/* move matrix rows */
	act_subnet->o_matrix[entry_num] = o_matrix[tra_num];
	act_subnet->h_matrix[entry_num] = h_matrix[tra_num];
	transitions[tra_num].subnet = act_subnet->number;
						/* mark as belonging to this subnet */
}

/*	is_connected returns true, if a place is connected to a transition
	in a special (or any) kind; otherwise false.		*/
int
is_connected(tra, pla, kind)
	int tra, pla, kind;
{
	sp_place  *pointer;
	int	i, in, out, inh, wgt, ena, arc;

	in  = (kind == ANY) || (kind == INPUT);
	out = (kind == ANY) || (kind == OUTPUT);
	inh = (kind == ANY) || (kind == INHIBIT);
	wgt = (kind == ANY) || (kind == WEIGHT);
	ena = (kind == ANY) || (kind == ENABLE);
	arc = (kind == ANY) || (kind == ARC_IND);
	/* look for obvious connections */
	pointer = i_matrix[tra];
	while(pointer)
		if (in && (pointer->num == pla))
			return 1;
		else 	pointer = pointer->next;
	pointer = o_matrix[tra];
	while(pointer)
		if (out && (pointer->num == pla))
			return 1;
		else 	pointer = pointer->next;
	pointer = h_matrix[tra];
	while(pointer)
		if (inh && (pointer->num == pla))
			return 1;
		else 	pointer = pointer->next;
	/* look for hidden connections (marking dependency) */
	if (wgt && (transitions[tra].rate_id))
		if (transitions[tra].w_deter[pla])
			return 1;
	if (ena && (transitions[tra].ena_id))
		if (transitions[tra].e_deter[pla])
			return 1;
	/* look for place dependencies of md arcs */
	for (i = 0; i < no_md_arcs; i++)
		if (arc && (md_arcs[i].trans == tra))
			if (md_arcs[i].deter[pla] && md_arcs[i].type != OUTPUT)
				return 1;
	return 0;
}

/*	sort_places sorts the places of a subnet by their upper bounds */
void
sort_places(sub)
	subnet *sub;
{
	int pla_nr, stop = 0, before, now;
	
	while(!stop)
	{	stop = 1;				/* bubble sort */
		for(pla_nr = 1; pla_nr < sub->no_places; pla_nr++)
		{	before = places[sub->places[pla_nr-1]].bound;
			now    = places[sub->places[pla_nr]].bound;
			if (!now || before && before <= now)
				continue;
			stop = 0;
			before = sub->places[pla_nr-1];
			sub->places[pla_nr-1] = sub->places[pla_nr];
			sub->places[pla_nr] = before;
		}
	}
}

/*	inner_matrix generates one row of an "inner" matrix of a subnet
	( by changing the global numbers of places to the local ones ) */
void
inner_matrix(sub, row_p, sparse_p)
	subnet *sub;
	sp_place **row_p, *sparse_p;
{	sp_place *last_p, *new_p, *look_p = sparse_p;
	int pla_nr;

	*row_p = NULL;
	while(look_p)
	{	new_p = (sp_place *)nMalloc(sizeof(sp_place));	/* new sparse entry */
		*new_p=*look_p;
		for(pla_nr = 0; pla_nr < sub->no_places; pla_nr++)
			if (look_p->num == sub->places[pla_nr])
			{	new_p->num = pla_nr;
				break;				/* real number found */
			}
		last_p = *row_p;
		if (last_p == NULL || last_p->num > new_p->num)
		{	new_p->next = *row_p;
			*row_p = new_p;				/* first entry */
		} else
		{	while(last_p->next != NULL && last_p->next->num < new_p->num)
				last_p = last_p->next;
			new_p->next = last_p->next;
			last_p->next = new_p;			/* normal entry */
		}
		look_p = look_p->next;
	}
}

/*	get_subnet_matrices generates the "inner" i, o and h matrices for a subnet */
void
get_subnet_matrices(sub)
	subnet *sub;
{	int tra_nr;

	for(tra_nr = 0; tra_nr < sub->no_trans; tra_nr++)
	{	inner_matrix(sub, &(sub->i_matrix[tra_nr]),
				i_matrix[sub->transitions[tra_nr]]);
		inner_matrix(sub, &(sub->o_matrix[tra_nr]),
				o_matrix[sub->transitions[tra_nr]]);
		inner_matrix(sub, &(sub->h_matrix[tra_nr]),
				h_matrix[sub->transitions[tra_nr]]);
	}
}

/*	calc_subnet is the main procedure obtaining the immediate transition subnets */
void
calc_subnets()
{	int tra_nr, act_nr, no_im_trans, no_rem_places;
	int tra_now, tra_max, pla_now, pla_max;
	subnet *new_sub, *last_sub;

	if (subnet_bit)
		printf("\nCalculating immediate transition subnets...\n");
	for(tra_nr = 0; tra_nr < no_trans; tra_nr++)
		transitions[tra_nr].subnet = 0;		/* clear marker */
#ifdef DEBUG
	printf("The net contains %d timed transitions:\n", no_timed_trans);
#endif
	subnets = (subnet *)nMalloc(sizeof(subnet)); /* first subnet contains timed trans */
	subnets->no_trans = no_timed_trans; 
	subnets->no_ecs = 0; subnets->ecs = NULL;	/* no ecs here */
	subnets->number = 1;
	subnets->transitions = (int *)nMalloc(no_timed_trans * sizeof(int));
	subnets->i_matrix    = (sp_place **)nMalloc(no_timed_trans * sizeof(sp_place *));
	subnets->o_matrix    = (sp_place **)nMalloc(no_timed_trans * sizeof(sp_place *));
	subnets->c_matrix    = (sp_place **)nMalloc(no_timed_trans * sizeof(sp_place *));
	subnets->h_matrix    = (sp_place **)nMalloc(no_timed_trans * sizeof(sp_place *));

	for(tra_nr = 0, act_nr = 0; tra_nr < no_trans; tra_nr++) /* search for timed trans. */
		if (transitions[tra_nr].type != s_IMM)
		{	found_transition(subnets, tra_nr, act_nr);
			subnets->c_matrix[act_nr++] = c_matrix[tra_nr];
			i_matrix[tra_nr] = NULL;
			o_matrix[tra_nr] = NULL;
			c_matrix[tra_nr] = NULL;
			h_matrix[tra_nr] = NULL;
#ifdef DEBUG
			printf("timed transition found: %s\n", transitions[tra_nr].name);
#endif
		}
	for(act_nr = 0; act_nr < no_places; act_nr++)
		places[act_nr].subnet = 1;
	for(tra_nr = 0; tra_nr < no_trans; tra_nr++)	/* search for disconnected places */
		for (act_nr = 0; act_nr < no_places; act_nr++)
			if ((transitions[tra_nr].type == s_IMM) && 
					is_connected(tra_nr, act_nr, ANY))
				places[act_nr].subnet = 0;	/* place is connected */
	subnets->no_places = 0;
	for(act_nr = 0; act_nr < no_places; act_nr++)
		if (places[act_nr].subnet)
			subnets->no_places++;		/* number of disconnected places */
	no_rem_places = no_places - subnets->no_places;	/* number of remaining places */
	subnets->places = (int *)nMalloc(subnets->no_places * sizeof(int));
	tra_nr = -1;
	for(act_nr = 0; act_nr < no_places; act_nr++)
		if (places[act_nr].subnet)
		{	subnets->places[++tra_nr] = act_nr;	/* real number */
#ifdef DEBUG
			printf("Disconnected place: %s\n", places[act_nr].name);
#endif
		}
	sort_places(subnets);				/* sort places by upper bound */

	last_sub = subnets;
	no_subnets = 0;
	no_im_trans = no_trans - no_timed_trans;	/* max number of imm. trans */
	while(no_im_trans)					/* look for subnets */
	{	no_subnets++;
		new_sub = (subnet *)nMalloc(sizeof(subnet)); 
		new_sub->no_ecs = 0;    new_sub->ecs = NULL;
		new_sub->number = no_subnets + 1;		/* number 2.. */
#ifdef DEBUG
		printf("==> Subnet %d\n", new_sub->number);
#endif
		/* for number of places in subnet is no_rem_places an upper bound */
		new_sub->places      = (int *)nMalloc(no_rem_places   * sizeof(int));
		/* for number of transitions in subnet is no_im_trans an upper bound */
		new_sub->transitions = (int *)nMalloc(no_im_trans * sizeof(int));
		new_sub->i_matrix = (sp_place **)nMalloc(no_im_trans * sizeof(sp_place *));
		new_sub->o_matrix = (sp_place **)nMalloc(no_im_trans * sizeof(sp_place *));
		new_sub->h_matrix = (sp_place **)nMalloc(no_im_trans * sizeof(sp_place *));

		for(tra_nr = 0; transitions[tra_nr].subnet; tra_nr++);
							/* first remaining transition */
		found_transition(new_sub, tra_nr, 0);		/* first found trans */

		tra_now = 0; tra_max = 1;			/* one trans */
		pla_now = 0; pla_max = 0;			/* no places */
		while(tra_now < tra_max)
		{	/* look for places connected to transitions known
							as part of this subnet */
			for(; tra_now < tra_max; tra_now++)
			{	tra_nr = new_sub->transitions[tra_now];	/* real number */
#ifdef DEBUG
				printf("Testing transition %s\n", transitions[tra_nr].name);
#endif
				for(act_nr = 0; act_nr < no_places; act_nr++)
					if (!places[act_nr].subnet && 
							is_connected(tra_nr, act_nr, ANY))
					{	new_sub->no_places++;	/* new place found */
						new_sub->places[pla_max++] = act_nr;
						places[act_nr].subnet = new_sub->number;	
#ifdef DEBUG
						printf("- found place %s\n", 
									places[act_nr].name);
#endif
					}
			}
			/* look for transitions connected to places known
							as part of this subnet */
			for(; pla_now < pla_max; pla_now++)
			{	act_nr = new_sub->places[pla_now];
#ifdef DEBUG
				printf("Testing place %s\n", places[act_nr].name);
#endif
				for(tra_nr = 0; tra_nr < no_trans; tra_nr++)
					if (!transitions[tra_nr].subnet && 
								is_connected(tra_nr, act_nr, ANY))
					{	found_transition(new_sub, tra_nr, tra_max++);
#ifdef DEBUG
					printf("- found transition %s\n", 
								transitions[tra_nr].name);
#endif
					}
			}
		}
		new_sub->no_places = pla_max;
		new_sub->no_trans  = tra_max;
#ifdef DEBUG
		printf(".. Subnet %d contains %d transitions and %d places.\n", 
			new_sub->number, new_sub->no_trans, new_sub->no_places);
#endif
		sort_places(new_sub);			/* sort places by upper bound */
		get_subnet_matrices(new_sub);		/* create i o h matrices for subnet */
		new_sub->c_matrix = sparse_function(new_sub->o_matrix, new_sub->i_matrix,
					new_sub->no_trans, minus); /* subnet's c-matrix */
		no_im_trans   -= new_sub->no_trans;
		no_rem_places -= new_sub->no_places;
		last_sub->next = new_sub;
		last_sub = new_sub;
	}
	last_sub->next = NULL;
}

/*	write_sparse writes a row of a sparse matrix to a file */
void
write_sparse(f, pointer)
	FILE     *f;
	sp_place *pointer;
{
	sp_place *walk;
	int	count = 0;

	walk = pointer;
	while(walk != NULL)
	{	count++;
		walk = walk->next;
	}
	fprintf(f, "%d ", count);	/* number of entries */
	walk = pointer;
	while(walk != NULL)
	{	fprintf(f, "%d %d ", walk->num, walk->value);
		fprintf(f, "%d %d ", walk->plu_id, walk->min_id);
		walk = walk->next;
	}
	fprintf(f, "\n");
}

/*	write_set writes a packed set to a file */
void
write_set(f, set)
	FILE *f;
	byte *set;
{
	int count;

	fprintf(f, "%d ", *set);	/* write number of entries */
	for(count = 1; count <= *set; count++)
		fprintf(f, "%d ", set[count]);
	fprintf(f, "\n");
}

/*	print informations about the subnets and write 
	structural informations to net.STRUCT		*/
void
subnet_output(s_file)
	FILE *s_file;
{	subnet	*sub = subnets;
	int	num, tra, bits, bytes = 0;
	transition	trans;

	fprintf(s_file, "structural analysis file\n");
	fprintf(s_file, "%d %d\n", no_trans, no_places);
	if (subnet_bit)
		printf("\nThere are %d timed transition(s):\n", subnets->no_trans);
	fprintf(s_file, "%d\n", subnets->no_trans);
	for(num = 0; num < subnets->no_trans; num++)
	{	trans = transitions[subnets->transitions[num]];
		if (subnet_bit)
			printf("%s  ", trans.name);
		fprintf(s_file, "%s %d %d ", trans.name, trans.number,
#ifndef OLDSTRUCT
#define TYPE2STRUCTTYPE(type) (type==s_EXP)?0:((type==s_DET)?1:((type==s_GEN)?2:3))
				TYPE2STRUCTTYPE(trans.type) );
#else
				(trans.type == DET) );
#endif
		fprintf(s_file, "%d %d ", trans.infinite, trans.mark_dep_nr);
		fprintf(s_file, "%22.16le\n", trans.value);
		write_sparse(s_file, subnets->i_matrix[num]);
		write_sparse(s_file, subnets->h_matrix[num]);
		write_sparse(s_file, subnets->c_matrix[num]);
	}
	fprintf(s_file, "%d\n", subnets->no_places);
	if (subnet_bit)
		printf("\n\nThere are %d independent place(s):\n", subnets->no_places);
	for(num = 0; num < subnets->no_places; num++)
	{	if (subnet_bit)
			printf("%s  ", p_name(num));
		fprintf(s_file, "%s %d ", p_name(num), p_number(num));	/* name, number */
		tra = places[p_number(num)].bound;
		fprintf(s_file, "%d %d ", places[p_number(num)].m0, tra); /* M0 and bound */
		bits = tra ? (int)ceil(log2((double)(tra+1))) : 1;
		fprintf(s_file, "%d\n", bits);				/* # of bits */
		bytes += bits;
	}
	bytes = ceil((double)(bytes)/16.0);
	fprintf(s_file, "%d\n", bytes);

	if (subnet_bit)
		printf("\n\nThe net contains %d Subnet(s).\n", no_subnets);
	fprintf(s_file, "%d\n", no_subnets);
	for(sub = subnets->next; sub; sub = sub->next)
	{	if (subnet_bit)
		{	printf("\n===> subnet %d:\n", sub->number-1);
			printf("This subnet contains %d transition(s):\n", sub->no_trans);
		}
		fprintf(s_file, "%d\n", sub->no_trans);		/* # of trans in subnet */
		for(num = 0; num < sub->no_trans; num++)
		{	trans = transitions[sub->transitions[num]];
			if (subnet_bit)
				printf("%s  ", trans.name);
			fprintf(s_file, "%s %lf", trans.name, trans.value);
			fprintf(s_file, " %d %d\n", trans.rate_id, trans.ena_id);
			write_sparse(s_file, sub->i_matrix[num]);
			write_sparse(s_file, sub->h_matrix[num]);
			write_sparse(s_file, sub->c_matrix[num]);
			write_set(s_file, sub->enas[num]);
			write_set(s_file, sub->diss[num]);
		}
		if (subnet_bit)
			printf("\nThis subnet contains %d place(s):\n", sub->no_places);
		fprintf(s_file, "%d\n", sub->no_places);
		bytes = 0;
		for(num = 0; num < sub->no_places; num++)
		{	if (subnet_bit)
				printf("%s  ", p_name(num));
			fprintf(s_file, "%s %d ", p_name(num), p_number(num));
			tra = places[p_number(num)].bound;
			fprintf(s_file, "%d %d ", places[p_number(num)].m0, tra);
			bits = tra ? (int)ceil(log2((double)(tra+1))) : 1;
			fprintf(s_file, "%d\n", bits);
			bytes += bits;
		}
		bytes = ceil((double)(bytes)/16.0);
		fprintf(s_file, "%d\n", bytes);
		if (subnet_bit)
			printf("\n");
#ifdef DEBUG
		{	sp_place *p_now;
			printf("\nConnections in this subnet:\n");

			for(num = 0; num < sub->no_trans; num++)
			{	printf("Transition %d %s:", num, 
					transitions[sub->transitions[num]].name);
				printf("\nInput:  ");
				p_now = sub->i_matrix[num];
				while(p_now)
				{	printf("  %s", places[sub->places[p_now->num]].name);
					printf("x %d", p_now->value);
					p_now = p_now->next;
				}
				printf("\nOutput:  ");
				p_now = sub->o_matrix[num];
				while(p_now)
				{	printf("  %s", places[sub->places[p_now->num]].name);
					printf("x %d", p_now->value);
					p_now = p_now->next;
				}
				printf("\nC-Matrix-Row:  ");
				p_now = sub->c_matrix[num];
				while(p_now)
				{	printf("  %s", places[sub->places[p_now->num]].name);
					printf("x %d", p_now->value);
					p_now = p_now->next;
				}
				printf("\nInhibitors:  ");
				p_now = sub->h_matrix[num];
				while(p_now)
				{	printf("  %s", places[sub->places[p_now->num]].name);
					printf("x %d", p_now->value);
					p_now = p_now->next;
				}
				printf("\n");
			}
		}
#endif
		/* ecs-output */
		if (ecs_bit)
			printf("This subnet contains %d ecs:\n", sub->no_ecs);
		fprintf(s_file, "%d\n", sub->no_ecs);
		for(num = 0; num < sub->no_ecs; num++)
		{	if (ecs_bit)
				printf("ecs %d (priority %d): ", num, 
							r_pri(sub->ecs[num][1]));
			write_set(s_file, sub->ecs[num]);
			for(tra = 1; tra <= sub->ecs[num][0]; tra++)
				if (ecs_bit)
					printf("%s ", r_name(sub->ecs[num][tra]));
			if (ecs_bit)
				printf("\n");
		}
	}
}

