/*************************************************************************

  filename:	batch_cv.h

  purpose:	Constants and types will be defined for batch_cv.c

  author:	Christian Kelling
		Technical University of Berlin
		Computer Science Department
		Institute for Technical Computer Science

  date:		May 1993

*************************************************************************/


/* Modes for function DetectInitialTransient */
#define STEADY_STATE 	0

/* Constant values for initializing parameters */
#define ALPHAT_DEF      (1.0-(double)conf_level/100.0)
#define FREEDOM_DEF	7

/* Constants used in Determine Batch Size  and CVEstimation */
#define CV_MIN_SAMPLES  40 /* min. number of batch means to use,
			       this is 60 +- 20 see the recommodations
			       by Lavenberg and Welch;
			       80 batches will be rebatched to 40*/

#define CV_START_SAMPLES 50 /* a block of 5 samples from each of the at least
			       10 replications has been received, start CV-
			       analysis now */
#define TWO_TIMES 	 2  /* two batches will be rebatched into one */

/* node of linked list to access a sequence of CVs */
typedef struct t_ControlSequenceList
  {
    SampSeq  *Seq_first;
    SampSeq  *Seq_last;
    SampSeq  *CurrentPosition;
    struct t_ControlSequenceList *Next;
  } ContrSeqList;

/* Environment used by the batch means analysis for each opened STATISTICS */
typedef struct t_CvR_StatisticsEnvironment
{
  char                  name[30];
  int                   kindOfMeasure;          /* 1:probability   0:other */
  bool      		StopSimulation;
  bool                  StopSendedToBackend;
  bool      		NoPrecision;
  int			StatStatus;
  int           	ControlVariates;
  int           	InitLength;
  double		TransTime;
  double		EndTime;
  sample_type		***SampleBlock;
  double		***CVBlock;
  int			*BlockAdminSum;
  int			*BlockAdminFirst;
  SampSeq		*SampSeq_first;
  SampSeq		*SampSeq_last;
  ContrSeqList  	*CVList_first;
  ContrSeqList  	*CVList_last;
  long			SampleCount;
  int                   SendFactor;
  int			NoAnalysedSamples;
  int			SequenceCount;
  int			BatchingCount;
  double		Epsilon;
  double		AveragedMean;
  double		Least_Sq_Mean;
  double		DeltaX;
  double		Variance;
} CvR_Stat;


