/******************************************************************************

  filename :	control.c

  purpose  :	performs the main function of the simulation system as well as
  		the connections between the simulation engine 
     		and the statitical analysis
  
  author   :    Christian Kelling
  		Technical University Berlin
		Computer Science Department

  date     :	June 1993


******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "sim_master_types.h"
#include "stat_utils.h"
#include "batch_cv.h"
#include "spec_var.h"
#include "analysis_str.h"

double current_time, transient_time;
double maxtime = 0.0; /* model time limit, 0.0 means no limit */
double max_execution_time = 0.0; /* real time limit, 0.0 means no limit */
double eps_max;
double  errPercent;    /* hezi 2/97 */
bool min_flag = false;
bool trans_detection = true;
bool PercentRuleOn = true;
int  Visualisation = 1;
int  min_firings;          /* bisher konstant MIN_FIRINGS */ 
int no_sampling_points;
int only_basic_components = true;
meas_names_list *measure_names;


int no_measures;
int StatSema	= 0;  /* TRUE: all statistics finished (upper bounds reached
			 or accuracy is ok) */
int NoPrecCount	= 0;  /* TRUE: all statistics have meet the accuracy limits */
int conf_level;
int initial_seed;
/*int maxsamples = 0x7FFFFFFF;*/  /* max possible value */
int maxsamples = MAX_SAMPLES_DEF;  /* a useful value */
int time_exceeded = false;

bool force_local_execution;
bool force_distributed_execution;

char *er_malloc(); /* ||| muss das als globale fn. definiert werden */
char  model_name[FILE_NAME];

void record_cv();
FILE	*net_file, *inf_file, *res_file, *erc_file, *stat_file;
#ifndef RST
  FILE  *sim_curves_file, *temp_sim_curves_file;
#endif

#ifdef CVR
CvR_Stat  **all_cvr_measures;
#endif
#ifdef STR
Str_Stat  **all_str_measures;
#endif
#ifdef SPEC
SpecStat **all_spec_measures;
#endif

double out_value (selection, i_measure, prec_flag)
     int selection, i_measure;
     bool prec_flag;
{
  static bool new_measure = true, called = false;
  static int first, last;
  int i;
  bool no_precision = false;

#ifdef CVR
  CvR_Stat *CVR_Measure;
  double CVR_Mean_out(), CVR_Lower_out(), CVR_Upper_out();

  if (!prec_flag)
    {
      if (new_measure)
	{
	  called = true;
	  first = i_measure;
	  new_measure = false;
	}
      last = i_measure;
      CVR_Measure = all_cvr_measures[i_measure];
      
      switch (selection)
	{
	case SEL_MEAN:
	  return CVR_Mean_out (CVR_Measure);
	  break;
	case SEL_LO:
	  return CVR_Lower_out (CVR_Measure);
	  break;
	case SEL_UP:
	  return CVR_Upper_out (CVR_Measure);
	  break;
	}
    }
  if (prec_flag)
    if (!called) /* das ist der Fall, wenn das Measure nicht aus Statistiken,
		    sondern nur aus Konstanten besteht */
      return (0.0);
  
    for (i=first; i<=last; i++)
      {
	CVR_Measure = all_cvr_measures[i];
	no_precision = no_precision || CVR_Measure->NoPrecision;
      }
  new_measure = true;
  called = false;
  if (no_precision)
    return (1.0);
  else
    return (0.0);
  
#endif
#ifdef SPEC
  SpecStat *SPEC_Measure;
  double SPEC_Mean_out(), SPEC_Lower_out(), SPEC_Upper_out();

  if (!prec_flag)
    {
      if (new_measure)
	{
	  called = true;
	  first = i_measure;
	  new_measure = false;
	}
      last = i_measure;
      SPEC_Measure = all_spec_measures[i_measure];
      
      switch (selection)
	{
	case SEL_MEAN:
	  return SPEC_Mean_out (SPEC_Measure);
	  break;
	case SEL_LO:
	  return SPEC_Lower_out (SPEC_Measure);
	  break;
	case SEL_UP:
	  return SPEC_Upper_out (SPEC_Measure);
	  break;
	}
    }
  if (prec_flag)
    if (!called) /* das ist der Fall, wenn das Measure nicht aus Statistiken,
		    sondern nur aus Konstanten besteht */
      return (0.0);
  
    for (i=first; i<=last; i++)
      {
	SPEC_Measure = all_spec_measures[i];
	no_precision = no_precision || SPEC_Measure->NoPrecision;
      }
  new_measure = true;
  called = false;
  if (no_precision)
    return (1.0);
  else
    return (0.0);
  
#endif
}

void Measure_Results_out (i_measure)
     int i_measure;
{

#ifdef CVR
  CvR_Stat *CVR_Measure;

  CVR_Measure = all_cvr_measures[i_measure];
  CVR_RewardMeasure_data_out(CVR_Measure);
#endif
#ifdef SPEC
  SpecStat *SPEC_Measure;

  SPEC_Measure = all_spec_measures[i_measure];
  SPEC_RewardMeasure_data_out(SPEC_Measure);
#endif

}

void Globals_out ()
{

#ifdef CVR
  CVR_Globals_out();
#endif
#ifdef SPEC
  SPEC_Globals_out();
#endif

}


  
main (argc, argv)
     int argc;
     char  **argv;
{
  char  net_name[FILE_NAME], model_and_path[FILE_NAME];
  char  erc_name[FILE_NAME], res_name[FILE_NAME], inf_name[FILE_NAME];
  char  stat_name[FILE_NAME];
#ifndef RST
  char sim_curves_name[FILE_NAME], temp_sim_curves_name[FILE_NAME];
#endif

  void print_net ();
  void rand_init();
  double rnd(), atof();
  int controls;
  char *modeldir, *getenv();

  setvbuf(stdout,NULL,_IOLBF,BUFSIZ);
  setvbuf(stderr,NULL,_IOLBF,BUFSIZ);

  /* fetch modeldir */
  if( !(modeldir = getenv("MODELDIR")) )
    {
      fprintf(stderr,"'MODELDIR' not set\n");
      return( 0 );
    }
  
  strcpy(model_name, argv[1]);
  sprintf(model_and_path,"%s/%s", modeldir, argv[1]);
  sprintf(net_name,"%s/%s.TN", modeldir, argv[1]);
  sprintf(erc_name,"%s/%s_ERC.c", modeldir, argv[1]);
  sprintf(res_name,"%s/%s.RESULTS", modeldir, argv[1]);
  sprintf(inf_name,"%s/%s.DEFINFO", modeldir, argv[1]);
  sprintf(stat_name,"%s/%s.STAT_OUT", modeldir, argv[1]);
#ifndef RST
  sprintf(sim_curves_name,"%s/%s.sim_curves", modeldir, argv[1]);
#ifdef STR
  sprintf(temp_sim_curves_name,"%s/%s.temp_sim_curves", modeldir, argv[1]);  
#endif
#endif

  conf_level = atoi(argv[2]);
  printf("\nconf level = %d\n", conf_level);
  eps_max = (double)atof(argv[3])/100.0;
  printf("epsilon max = %f\n", eps_max);
  initial_seed = atoi(argv[4]);
  if (strcmp(argv[5], "0"))
      maxsamples = atoi(argv[5]);
  /* else: otherwise take the default value (see definition above) */
  maxtime = atof(argv[6]);
  max_execution_time = atof(argv[7]);
  if (!strcmp(argv[8], "D"))
    {
      force_distributed_execution = true;
      force_local_execution = false;
    }
  else
    {
      force_local_execution = true;
      force_distributed_execution = false;
    }
  if (!strcmp(argv[9], "Toff"))
    {
      trans_detection = false;
      min_flag = true;
    }
#ifndef STR
  min_firings=atof(argv[10]);
#ifndef RST
  errPercent=atof(argv[11]);             /* Abweichung fuer Konf.level fuer W. */
  if (!strcmp(argv[12], "-Voff"))        /* -Voff : Visualisierung aus */
    {
      Visualisation = 0;
    }
#endif
#endif

#ifdef STR
  transient_time = atof(argv[10]);
  no_sampling_points = atoi(argv[11]);
  if (no_sampling_points < 3)
     no_sampling_points = 3;
  if(!strcmp(argv[12], "Poff"))
    PercentRuleOn = false;
#endif
  printf("random seed = %d\n", initial_seed);
  printf("max.samples");
  if (maxsamples == 0)
    printf(": no limit\n");
  else
    printf(" = %d\n", maxsamples);
  maxsamples /= LOCAL_BATCH_SIZE_DEF; /* ab jetzt nur noch diese Groesse,
                       da Einzelwerte schon im Slave zusammengefasst werden */
  printf("min. firings for each transition = %ld\n",min_firings);
  printf("max.model time");
  if (maxtime == 0.0)
    printf(": no limit\n");
  else
    printf(" = %f\n", maxtime);
  printf("max.real time");
  if (max_execution_time == 0.0)
    printf(": no limit\n");
  else
    printf(" = %f\n", max_execution_time);
#ifndef STR
#ifndef RST
  printf("permitted difference for probabilty measures: %lg\n",errPercent);
  printf("Visualisation for standard steady Simulation: %s \n",
         Visualisation==1 ? "ON" : "OFF");
#endif
#endif
  if ((net_file = fopen(net_name, "r")) == NULL)
    {
      printf ("ERROR: could not open %s for reading\n", net_name);
      exit (FAIL);
    }
  if ((erc_file = fopen(erc_name, "r")) == NULL)
    {
      printf ("ERROR: could not open %s for reading\n", erc_name);
      exit (FAIL);
    }
  if ((inf_file = fopen(inf_name, "r")) == NULL)
    {
      printf ("ERROR: could not open %s for reading\n", inf_name);
      exit (FAIL);
    }
  if ((res_file = fopen(res_name, "w")) == NULL)
    {
      printf ("ERROR: could not open %s for writing\n", res_name);
      exit (FAIL);
    }
  if ((stat_file = fopen(stat_name, "w")) == NULL)
    {
      printf ("ERROR: could not open %s for writing\n", stat_name);
      exit (FAIL);
    }
#ifndef RST
  if ((sim_curves_file = fopen(sim_curves_name, "w")) == NULL)
    {
      printf ("ERROR: could not open %s for writing\n", sim_curves_name);
      exit (FAIL);
    }
#ifdef STR
  if ((temp_sim_curves_file = fopen(temp_sim_curves_name, "w")) == NULL)
    {
      printf ("ERROR: could not open %s for writing\n", temp_sim_curves_name);
      exit (FAIL);
    }
#endif
#endif

  prepare_sim();
/* print_net(); */
  sim_master(model_and_path);
}
