/******************************************************************************

  filename :	prepare_sim.c

  purpose  :	prepare distributed simulation of DSPN-models in 
		DSPNexpress description format
  
  author   :    Christian Kelling
  		Technical University Berlin
		Computer Science Department

  date     :	January 1994


******************************************************************************/

#include <stdio.h>
#include <string.h>

#include "stat_utils.h"
#include "sim_master_types.h"
#include "batch_cv.h"
#include "spec_var.h"
#include "analysis_str.h" 

extern FILE	*net_file, *inf_file;
extern SpecStat **all_spec_measures;
extern CvR_Stat **all_cvr_measures;
extern Str_Stat **all_str_measures;
extern double 	current_time;
extern int	maxsamples;
extern int      no_sampling_points;
extern meas_names_list *measure_names;
extern int	StatSema;

/* global variables used by the modul simulation */

int	no_places, no_trans, no_mark_par, no_del_par;
int	no_md_exp, no_md_det, no_weights, no_ena_fcts;
int     no_md_arcs, no_measures;

int     *probMeasureId;
int     numberProbMeasures;

place		*places;
transition  	*transitions;

#ifdef CVR
  meas_trans_list *meas_trans = NULL;
#endif

/* end of global variables */

void Init_Statistics (controls)
int controls;
{
  int	 i;
#ifdef CVR
  CvR_Stat *CVR_RewardMeasure_Init();
#endif
#ifdef STR
  Str_Stat *STR_RewardMeasure_Init();
#endif
#ifdef SPEC
  SpecStat *SPEC_RewardMeasure_Init();
#endif
  char *er_malloc();

#ifdef SPEC
  all_spec_measures = (SpecStat **)er_malloc(no_measures * sizeof(SpecStat*));

  SPEC_Analysis_Init (maxsamples);
  for (i=0; i<no_measures; i++)
    all_spec_measures[i] = SPEC_RewardMeasure_Init (i);
#endif
#ifdef STR
  all_str_measures = (Str_Stat **)er_malloc(no_measures * sizeof(Str_Stat*));

  for (i=0; i<no_measures; i++)
    all_str_measures[i] = STR_RewardMeasure_Init (i);      
#endif
#ifdef CVR
  all_cvr_measures = (CvR_Stat **)er_malloc(no_measures * sizeof(CvR_Stat*));
  CVR_Analysis_Init (maxsamples);
  read_cvs(); 
#endif
}

#ifdef CVR
void Init_CVR_Measure (i_measure, controls)
     int i_measure, controls;
{
  extern CvR_Stat *CVR_RewardMeasure_Init();

  all_cvr_measures[i_measure] = (CvR_Stat*) CVR_RewardMeasure_Init (i_measure,controls);
}
#endif


/* prepare the data structures for representation of a DSPN */

prepare_sim ()

{
  void read_net();
  void define_measures();
  void compute_connections ();
  void print_net ();
  void Init_Statistics ();
  int i;
#ifndef RST
  meas_names_list *get_measure_names();
#endif

  read_net(net_file, inf_file); /* reads the net description file netname.NET 
                                   and the information netname.DEFINFO */
  compute_connections();
/*   print_net();  */
#ifndef RST
  measure_names = get_measure_names(net_file);
#endif
 define_measures();
 Init_Statistics();
#ifdef debug
 printf("numberProbMasures=%d\n",numberProbMeasures);
#endif
 for (i=0;i<numberProbMeasures;i++)       /* Findet die Wahrscheinlichkeits-measures */
 {
#ifdef SPEC
        all_spec_measures[probMeasureId[i]]->kindOfMeasure=PROB_MEAS; 
#elif CVR
        all_cvr_measures[probMeasureId[i]]->kindOfMeasure=PROB_MEAS;  
#endif
	;
 }
}





