/*NEUE_VERSION (BM)*/
/******************************************************************************
Programm        :       sim_master.c

Purpose         :       Execution of simulation processes either local or
			distributed on different hosts

Programmer      :       Tobias Bading, Christian Kelling

History         :       release 1: January 1994

  Datei im Ramen des PDV-Projekts im WS 95/96 gendert. (BM)

  nderung:	Die Einzelwerte der Leistungsmae werden in Gruppen zusammen-
		gefat und an den Master-Proze gesendet, wobei die Gruppen-
		gre an die Aufzeichnungshufigkeit der Mae angepat wird.

		receive_samples (int,int,int,int)
		  Durch die Umdefinierung von no_samples nach no_all_samples
		  wird der bisherige Einlesealgorithmus von einer Schleife
		  eingefat, die alle no_all_samples Samples in Stcken von
		  no_samples einliet.
		  Bei der Simulation mit CVs mssen alle Samples vom Socket
		  gelesen werden, bevor die CVs sichtbar werden. dazu wird der
		  SampleBuffer verwendet.

		void  receive_messages (receive_mode)
		  no_samples (Anzahl der samples in einem bertragenen Paket)
		  ist variabel, mu aber ein ganzzahliges Vielfaches von
		  STAT_BLOCK_DEF sein.
		  STAT_BLOCK_DEF ist die kleinste sample-Paket-Einheit fr die
		  Zwischenspeicherung der samples.
		  no_samples darf nicht grer als RECEIVE_BLOCK_DEF sein.

		static void  distributed_execution (void),
		static void  local_execution (void)
		  Auf den Speicherblock jedes sample-Pakets wird ber Zeiger
		  zugegriffen. Statt bei der Initialisierung mehrfach einzelne
		  Speicherblcke fr jedes empfangene sample-Paket getrennt zu
		  allozieren, wird ein zusammenhngender Speicherbereich
		  erzeugt, auf den die einzelnen sample-Paket-Zeiger wie
		  bisher zeigen.

  Autoren:	Benjamin Hohnhuser und Mathias Neumann

  Datum:   	April 1996

******************************************************************************/

#include <stdio.h>
#include <malloc.h>
#include <string.h>

#include <math.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <signal.h>
#include <sys/wait.h>
#include <sys/resource.h>
#include <fcntl.h>
#include <setjmp.h>
#include <utmp.h>

#include "portab.h"

#if !defined(ultrix) && !defined(__hpux) && \
    !defined(SOL2) && !defined(__osf__)  && !defined(linux)
#  include <vfork.h>
#endif

#ifdef SOL2
#include <sys/filio.h>
#endif

#include <rpc/rpc.h>
#include "rusers.h" /* <rpcsvc/rusers.h> not identical on all systems */
#include "rstat.h"  /* <rpcsvc/rstat.h> as well */

#include "sim_master_types.h"
#include "stat_utils.h"
#include "num_utils.h"
#include "spec_var.h"
#include "batch_cv.h"
#include "analysis_str.h"

#define TNET_RC ".TimeNET"

/* Spezialbehandlung fuer die DEC-Kisten */
#ifdef ultrix
static char *strdup (string)
char *string;
{
  register char *new_string = (char *)malloc (strlen (string) + 1);
  if (new_string)
    strcpy (new_string, string);
  return new_string;
}
#endif

#define OUT_OF_MEMORY { \
  perror (NULL); \
  fprintf (stderr, "error in %s at line %u: out of memory !\n", \
	   __FILE__, __LINE__); \
  exit (1); \
		      }

#define FILE_WRITE_ERROR(filename) { \
  perror (NULL); \
  fprintf (stderr, "error in %s at line %u: can't write to file '%s'!\n", \
	   __FILE__, __LINE__, filename); \
  exit (1); \
				   }

/************************/
/*** Typdefinitionen: ***/
/************************/
/* Typ des Uebergabeparameters der Funktion receive_messages: */
typedef enum receive_mode {
  wait_for_one, wait_for_all
} receive_mode;


/*******************************/
/*** Funktionsdeklarationen: ***/
/*******************************/
/* extern: */
extern char *getenv ();
extern char *er_malloc();
extern void free_meas_trans_list();
/* modul-lokal: */
void sim_master ();              /* Hauptroutine, wird von control.c aus
				   aufgerufen */
static void my_fwrite ();       /* Senden von Daten an sim_slave-Prozesse */
static void my_read ();         /* Lesen von Daten der sim_slave-Prozesse */
void receive_messages ();       /* Lesen von Nachrichten der sim-Prozesse */
void start_sim_process ();      /* sim-Prozess starten */
static void sigchld_handler ();        /* Handler fuer SIGCHLD-Signale */
static void local_execution ();        /* sim-Prozesse lokal starten */
static void distributed_execution ();  /* sim-Prozesse verteilt starten */
#ifndef STR
#ifndef RST
#ifdef __STDC__
static void visual(int measure_id,
#ifdef SPEC
                   SpecStat 
#elif CVR
                   CvR_Stat
#endif 
                             *Measure);
double uniPdfAreaErr(char   *measureName,
                     double lowerBorder,
                     double upperBorder,
                     int n,
                     double meanValue,
                     double variance,
                     double alpha);             /* Irrtumswahrscheinlichkeit */
#else
static void visual();
double uniPdfAreaErr();
#endif
#endif
#endif

/******************/
/*** Variablen: ***/
/******************/
/* extern */
extern place            *places;
extern transition       *transitions;
extern meas_trans_list  *meas_trans;
extern int              StatSema;
#ifdef CVR
extern CvR_Stat         **all_cvr_measures;
#endif
#ifdef STR
extern Str_Stat         **all_str_measures;
#endif
#ifdef SPEC
extern SpecStat         **all_spec_measures;
#endif
extern int              no_md_arcs;
extern int              time_exceeded;
extern double           maxtime;
extern double           max_execution_time;
extern int              min_firings;
extern double           errPercent;     /* erlaubte Abweichung vom Konfidenzniveau */
					/* fuer Wahrscheinlichkeitsmasze */
extern int              Visualisation;  /* =1:Visualisierung der stat. Simu. */
extern bool             min_flag;
extern bool             trans_detection;
extern FILE             *res_file;

int no_slaves;                          /* Anzahl der parallel laufenden 
					   sim_slave Prozesse */
/* modul-lokal: */
static slaveinfo *slaveinfos;           /* Infos ueber die zu verteilenden
					   sim_slave-Prozesse */
static int no_samples;                  /* Anzahl der in einem Block 
					   gesendeten Werte */
static char *netname;                   /* Name des DSPN */
static jmp_buf jmp_env;                 /* fuer setjmp () und longjmp () */
static time_t start_time;               /* Startzeit der Funktion
					   sim_master-processes */
static int server_fd;                   /* Filedescriptor fuer die
					   Server-Socket */
static unsigned short portnumber;       /* Portnummer dieser Socket */
static char *local_hostname;            /* Name des lokalen Rechners */
static hostinfo *local_host;            /* Informationen ueber den lokalen
					   Rechner */
static double sum_of_rusage_times;      /* Summe der CPU-Zeiten der sim-
					   Prozesse */
static unsigned running_processes;      /* Anzahl der gerade laufenden
					   Prozesse */
static enum {
  local, distributed
} execution_mode;               /* Ausfuehrungsart: lokal oder verteilt */
static unsigned free_hosts;             /* Anzahl freier Rechner */

/* Informationen aus dem Environtment-File: */
static hostinfo **sorted_hosts;         /* Array von freien Rechnern, sortiert
					   nach Verwendungsreihenfolge */
static hostinfo *hostlist;              /* Liste der Rechner, auf denen sim-
					   Prozesse gestartet werden koennen */

static unsigned max_processes_per_host; /* Maximale Anzahl der sim-Prozesse
					   pro Rechner */
static unsigned max_slaves;             /* Maximale Anzahl der sim-Prozesse
					   insgesamt */
static bool fatal_error = false;        /* Fehlernachricht von slave, keine 
					   neuen slaves starten */
static bool calc_stats_done = false;    /* Mesaures bereits ins RESULT-file
					   geschrieben ? */
static FILE *protfile;                  /* Protokoll ueber die Zeiten ...hezi 11/96 */
static double *last_time;               /* letzte Zeit fuer Visualisierung */

static char id_file[256];
static FILE *idfile;                    /* File mit den PIDs des Masters und */
                                        /* aller Slaves zum schnellen Killen */
                                        /* hezi 11/96 */

/* importierte Variablen: */

#ifdef STR
extern double   transient_time;
extern int      no_sampling_points;
#endif
extern int      initial_seed, conf_level;
extern int      no_trans, no_places, no_measures;
extern bool     force_local_execution;  /* lokale Ausfuehrung erzwingen ? */
extern bool     force_distributed_execution; 
				/* verteilte Ausfuehrung erzwingen ? */
#ifdef CVR
extern CvR_Stat         **all_cvr_measures;
#endif
#ifndef RST
#ifndef STR
extern  FILE            *sim_curves_file;
#endif
#endif

/*****************************/
/* variables used by RESTART */
/*****************************/

#ifdef RST

#define END_OF_SEQUENCE 0.0

int  direction,
     event,
     border,
     no_thresholds,
     no_opt_thresholds,
     t_value[MAX_THRESHOLDS+1],
     retrials[MAX_THRESHOLDS+1];

double ret_product=1.0;
double prob_opt[MAX_THRESHOLDS+1];

char measure_place[100];

#endif

/******************************/
/*** Funktionsdefinitionen: ***/
/******************************/
/******************************************************************************

 Funktion:      my_fwrite

 Parameter:     data:   Zeiger auf die zu schreibenden Daten
		length: Anzahl der zu schreibenden Bytes
		filepointer: Filepointer (einer Socket zu einem sim-Prozess)

 Rueckgabewert: ---

 Zweck:         Senden von Datenpaketen (ueber eine Socket)

******************************************************************************/
static void my_fwrite (data, length, filepointer)
char *data;
unsigned length;
FILE *filepointer;
{
  int rt;

 nochmal:
  errno = 0;
  if ((rt = write ((int)fileno (filepointer), data, length)) != length)
    {
    if (rt >= 0)
      {
      data += rt;
      length -= rt;
      goto nochmal;
    }
    perror (__FILE__);
    fprintf (stderr, "error in %s: can't write to socket!\n", __FILE__, rt, length);
    exit (1);
  }
}


/******************************************************************************

 Funktion:      my_read

 Parameter:     data:   Zeiger auf den Bereich, wo die gelesenen Daten
			abgelegt werden
		length: Anzahl der zu lesenden Bytes
		fd:     Filedesciptor (einer Socket zu einem sim-Prozess)

 Rueckgabewert: ---

 Zweck:         Empfangen von Datenpaketen (ueber eine Socket)

******************************************************************************/
static void my_read (data, length, fd)
char *data;
unsigned length, fd;
{
  int rt;

 nochmal:
  errno = 0;
  if ((rt = read (fd, data, length)) != length)
    {
    if (rt >= 0)
      {
	data += rt;
	length -= rt;
	goto nochmal;
      }
    perror (__FILE__);
    fprintf (stderr, "error in %s: can't read from socket!\n", __FILE__, rt, length);
    exit (1);
  }
}

/******************************************************************************

 Funktion:      sigchld_handler

 Parameter:     ---

 Rueckgabewert: ---

 Zweck:         Erloesen von Zombi-Prozessen

******************************************************************************/
static void sigchld_handler ()
{
  int status;

  waitpid (-1, &status, WNOHANG);
}

/******************************************************************************

 Funktion:      user_break

 Parameter:     ---

 Rueckgabewert: ---

 Zweck:         manuelles Beenden durch den Anwender,Speichern aller PIDs

 Autor:         Henning Ziegler 1/97 (s. Diplomarbeit 3/97)

******************************************************************************/
static void user_break ()
{
 static char Befehl[256];
 void calc_measures ();
 int i,j;
#ifdef SPEC 
 SpecStat *HELP_Measure;
#elif CVR
 CvR_Stat *HELP_Measure;
#endif

 printf("\nNOTE: Simulation at master has been stopped by user\n");
 if (!calc_stats_done) /* Measures schreiben, falls nicht schon in receive
 			   samples getan */
 {
#ifdef STR
      Measure_data_out();
#else
      calc_measures (res_file);
#endif
   
      calc_stats_done = true;
      
#ifndef RST
#ifndef STR
      for (j=0; j<no_measures; j++)
	{
#ifdef SPEC
	  HELP_Measure = all_spec_measures[j];
#elif CVR
	  HELP_Measure = all_cvr_measures[j];
#endif
	   if (!HELP_Measure->StopSimulation)
	    {
              HELP_Measure->EndTime=0.0;
	      for (i=0; i<no_slaves; i++)
	        HELP_Measure->EndTime += slaveinfos[i].slave_local_time;
	      HELP_Measure->StopSimulation = true;
	      HELP_Measure->NoPrecision = true;
	      StatSema--;
	      	     
	      if (Visualisation == 1) 
	         update_curves_file(sim_curves_file, j, HELP_Measure->EndTime);
	    }
            close_curves_file(sim_curves_file); 

	}
#endif
#endif /* ifndef RST*/
      fatal_error = true;
  }

  /* Zeiten ausgeben: */
  printf ("\n\nTotal CPU time: %.2f sec.\n", sum_of_rusage_times);
  {
    long seconds, minutes = 0, hours = 0;
    seconds = time (NULL) - start_time;
    if (seconds > 59)
      {
	minutes = seconds / 60;
	seconds %= 60;
      }
    if (minutes > 59)
      {
	hours = minutes / 60;
	minutes %= 60;
      }
    printf ("Total overall time: %02lu:%02lu:%02lu\n",
	    hours, minutes, seconds);
  }

 idfile=fopen(id_file,"a");
 fprintf(idfile,"########## END of %s.pid ############\n",netname);
 fclose(idfile);
 sprintf(Befehl,"$TNETHOME/Simulation/scripts/KillSlaves %s",id_file);
 system(Befehl);
#ifdef debug
#ifndef RST
#ifndef STR
 fclose(protfile);
#endif
#endif
#endif

 exit(0);
}


/******************************************************************************

 Funktion:      establish_server_socket

 Parameter:     ---

 Rueckgabewert: ---

 Zweck:         Erstellen einer Socket, an der sich spaeter die 
		sim_slave-Prozesse melden. Die Portnummer wird dabei vom 
		Kernel bestimmt.

******************************************************************************/
static void establish_server_socket ()
{
  struct sockaddr_in server;
  int length = sizeof (server);

  /* Server-Socket oeffnen: */
  if ((server_fd = socket (AF_INET, SOCK_STREAM, 0)) < 0)
  {
    perror ("socket");
    exit (1);
  }
  memset (&server, 0, sizeof (server));
  server.sin_family = AF_INET;
  server.sin_port = 0;
  if (bind (server_fd,(struct sockaddr *) &server, sizeof (server)))
  {
    perror ("bind");
    exit (1);
  }

  /* Portnummer ermitteln: */
  getsockname (server_fd,(struct sockaddr *) &server, &length);
  if (length == sizeof (server))
    portnumber = ntohs (server.sin_port);
  else
  {
    perror ("getsockname");
    exit (1);
  }

  if (listen (server_fd, 5))
  {
    perror ("listen");
    exit (1);
  }
}


/******************************************************************************

 Funktion:      send_infos

 Parameter:     Filepointer (einer Socketverbindung zu einem sim-Prozess)
		slave_id: teilt dem slave die Nummer mit, unter der er 
		beim master bekannt ist

 Rueckgabewert: ---

 Zweck:         Daten ueber die Dimension des Netzes an sim_slave senden

******************************************************************************/
static void send_infos (fp, id)
FILE *fp;
int id;
{
#ifdef RST
  int name_length,
      i;
#endif

#ifdef STR
  int seed_step = no_slaves;
#endif

  my_fwrite (&id, sizeof (int), fp);
  my_fwrite (&no_trans, sizeof (int), fp);
  my_fwrite (&no_places, sizeof (int), fp);
  my_fwrite (&no_measures, sizeof (int), fp);
  my_fwrite (&no_md_arcs, sizeof (int), fp);
  my_fwrite (&initial_seed, sizeof (int), fp);
  my_fwrite (&conf_level, sizeof (int), fp);
  my_fwrite (&trans_detection, sizeof (int), fp);
  my_fwrite (&min_firings, sizeof (int), fp); 

#ifdef RST
  name_length = strlen(measure_place);
  my_fwrite (&name_length, sizeof (int), fp);
  my_fwrite (measure_place, name_length, fp);
  my_fwrite (&direction, sizeof (int), fp);
  my_fwrite (&event, sizeof (int), fp);
  my_fwrite (&border, sizeof (int), fp);
  my_fwrite (&no_thresholds, sizeof (int), fp);
  my_fwrite (&no_opt_thresholds, sizeof (int), fp);
  for (i=1;i<=no_thresholds;++i)
     {
     my_fwrite (&t_value[i], sizeof (int), fp);
     my_fwrite (&retrials[i], sizeof (int), fp);
     }
  for (i=1;i<=no_opt_thresholds;++i)
     my_fwrite(&prob_opt[i], sizeof (double), fp);
#endif

#ifdef STR
  my_fwrite (&transient_time, sizeof (double), fp);
  my_fwrite (&seed_step, sizeof (int), fp);
  my_fwrite (&no_sampling_points, sizeof (int), fp);
  my_fwrite (&StatSema, sizeof (int), fp);
#endif

  initial_seed ++;
}


/******************************************************************************

 Funktion:      send_transitions

 Parameter:     fp:        Filepointer (einer Socketverbindung zu einem
			   sim-Prozess)

 Rueckgabewert: ---

 Zweck:         senden der Transitions-Liste transitions[]

******************************************************************************/
static void send_transitions (fp)
FILE *fp;
{
  int i, j, name_length;
  aff_list *aff_node;
  cv_list_elem *cv_node; 

  for (i=0; i<no_trans; i++)
    {
      my_fwrite (&i, sizeof(int), fp);
      my_fwrite (&transitions[i].type, sizeof(int), fp);
      my_fwrite (&transitions[i].serv_type, sizeof(int), fp);
      my_fwrite (&transitions[i].fire_policy, sizeof(int), fp);
      my_fwrite (&transitions[i].prio, sizeof(int), fp);
      my_fwrite (&transitions[i].ena_id, sizeof(int), fp);
      my_fwrite (&transitions[i].mark_dep_id, sizeof(int), fp);
      my_fwrite (&transitions[i].no_outputs, sizeof(int), fp);
      my_fwrite (&transitions[i].no_inputs, sizeof(int), fp);
      my_fwrite (&transitions[i].no_inh, sizeof(int), fp);
      my_fwrite (&transitions[i].no_aff, sizeof(int), fp);
#ifdef CVR
      my_fwrite (&transitions[i].no_cv_meas, sizeof(int), fp);
#endif
      my_fwrite (&transitions[i].delay, sizeof(double), fp);
      name_length = strlen(transitions[i].name);
      my_fwrite (&name_length, sizeof(int), fp);
      my_fwrite (transitions[i].name, name_length, fp);

      for (j=0; j<transitions[i].no_outputs; j++)
	  my_fwrite (&transitions[i].out_to_place[j], sizeof(conn_list), fp);
      for (j=0; j<transitions[i].no_inputs; j++)
	  my_fwrite (&transitions[i].in_from_place[j], sizeof(conn_list), fp);
      for (j=0; j<transitions[i].no_inh; j++)
	  my_fwrite (&transitions[i].inh_from_place[j], sizeof(conn_list), fp);
      aff_node = transitions[i].affected_trans;
      for (j=0; j<transitions[i].no_aff; j++)
	{
	  if (aff_node == NULL)
	    {
	      printf("ERROR in send_transitions: aff list too short\n");
	      exit(FAIL);
	    }
	  my_fwrite (&aff_node->number, sizeof(int), fp);
	  aff_node = aff_node->next;
	}
#ifdef CVR
      cv_node = transitions[i].cv_list;
      for (j=0; j<transitions[i].no_cv_meas; j++)
	{
	  if (cv_node == NULL)
	    {
	      printf("ERROR in send_transitions: cv list too short\n");
	      exit(FAIL);
	    }
	  my_fwrite (&cv_node->meas_number, sizeof(int), fp);
	  my_fwrite (&cv_node->cv_number, sizeof(int), fp);
	  cv_node = cv_node->next;
	}
#endif
    }
}

/******************************************************************************

 Funktion:      send_places

 Parameter:     fp:        Filepointer (einer Socketverbindung zu einem
			   sim-Prozess)

 Rueckgabewert: ---

 Zweck:         senden der Stellen-Liste places[]

******************************************************************************/
static void send_places (fp)
FILE *fp;
{
  int i, j, name_length;
  meas_list *meas_node;

  for (i=0; i<no_places; i++)
    {
      my_fwrite (&places[i].token, sizeof(int), fp);
      my_fwrite (&places[i].no_measures, sizeof(int), fp);
      name_length = strlen(places[i].name);
      my_fwrite (&name_length, sizeof(int), fp);
      my_fwrite (places[i].name, name_length, fp);

      meas_node = places[i].measure_list;
      for (j=0; j<places[i].no_measures; j++)
	{
	  if (meas_node == NULL)
	    {
	      printf("ERROR in send_places: measure list too short\n");
	      exit(FAIL);
	    }
	  my_fwrite (&meas_node->id, sizeof(int), fp);
	  meas_node = meas_node->next;
	}
    }
}

/******************************************************************************

 Funktion:      send_cv_config

 Parameter:     fp:        Filepointer (einer Socketverbindung zu einem
			   sim-Prozess)

 Rueckgabewert: ---

 Zweck:         senden der Anzahl der control variates je Measure

******************************************************************************/
#ifdef CVR
static void send_cv_config (fp)
FILE *fp;
{
  int i;
  CvR_Stat *Measure;

  for (i=0; i<no_measures; i++)
    {
      Measure = all_cvr_measures[i];
      my_fwrite (&Measure->ControlVariates, sizeof(int), fp);
    }
}
#endif


/******************************************************************************

 Funktion:      test_min_flags

 Parameter:     none

 Rueckgabewert: none, modifies the global flag min_flag

 Zweck:         check all min flags in order to start the estimation 
		procedures not too early

******************************************************************************/
void test_min_flags ()
{
  int i;
  
  for (i = 0; i < no_slaves; ++i) 
    if (!slaveinfos[i].slave_local_min_flag)
      {
/*      printf ("min flag slave %d not set\n", i);*/
	return;
      }
  min_flag = true;
  return;
}


/******************************************************************************

 Funktion:      send_command

 Parameter:     cmd == MASTER_STOP : Gesamtsimulation fertig
		cmd == MEASURE_STOP: eine Statistik fertig

		id:     Nummer der Statistik, fuer die das Kommando 
			MEASURE_STOP gueltig ist

 Rueckgabewert: ---

 Zweck:         senden von Kommandos an alle laufenden sim_slave-Prozesse

******************************************************************************/
static void send_command (slave_id, cmd, id)
int slave_id, cmd, id;
{
  int i;
  FILE *fp;
  if (slave_id == ALL)
    for (i=0; i<no_slaves; i++)
      {
	register slaveinfo *info = &slaveinfos[i];
	fp = info->connection_fp;
	if (info->status == running)
	  switch (cmd)
	    {
	    case MASTER_STOP:
	    case MASTER_RDY:
	      id = NO_ID;
	      my_fwrite (&cmd, sizeof(int), fp);
	      my_fwrite (&id, sizeof(int), fp);
	      break;
	    case MEASURE_STOP:
	      my_fwrite (&cmd, sizeof(int), fp);
	      my_fwrite (&id, sizeof(int), fp);
	      break;
	    default:
	      perror ("Unknown command in send_commands");
	    }
      }
  else
    {
      register slaveinfo *info = &slaveinfos[slave_id];
      fp = info->connection_fp;
      if (info->status == running)
	switch (cmd)
	  {
	  case MASTER_STOP:
	  case MASTER_RDY:
	    id = NO_ID;
	    my_fwrite (&cmd, sizeof(int), fp);
	    my_fwrite (&id, sizeof(int), fp);
	    break;
	  case MEASURE_STOP:
	    my_fwrite (&cmd, sizeof(int), fp);
	    my_fwrite (&id, sizeof(int), fp);
	    break;
	  default:
	    perror ("Unknown command in send_commands");
	  }
    }
}

/******************************************************************************

 Funktion:      start_sim_process

 Parameter:     Zeiger auf eine slaveinfo-Struktur

 Rueckgabewert: Die Eintraege slaveinfo->status und
		slaveinfo->host->running_processes werden modifiziert

 Zweck:         Starten eines sim_slave-Prozesses auf dem Rechner, der durch
		slaveinfo->host->hostname spezifiziert ist

******************************************************************************/
void start_sim_process (info)
struct slaveinfo *info;
{
  static char *dspnhome, port[10], t_limit[50];
  char ups_argument[200], commandpath[200];
  int stderr_backup_fd;
  register hostinfo *host = info->host;

  if (!port[0])
    sprintf (port, "%u", portnumber);
  sprintf(t_limit, "%20.20f", maxtime);
  /* Pfad fuer das Executable sim_slave bestimmen: */
  sprintf (commandpath, "%s_slave", netname);
  
  /* vfork () statt fork (), da dieser Prozess extrem viel Speicher schluckt.
     Laestiger Nebeneffekt: Bei lokaler Ausfuehrung der sim_slave-Prozesse 
     stimmen wegen vfork () die Speicherbelegungsangaben von getrusage () 
     nicht mehr */
#if !defined(ultrix) && !defined(__hpux) && \
    !defined(SOL2) && !defined(__osf__)
  switch (vfork ())
#else
  switch (fork ())
#endif
    {
    case -1: perror ("vfork");
      exit (1);
    case 0:
      {
#ifndef ultrix
	/* Standarderror-Filedescriptor merken: */
	stderr_backup_fd = dup (2);
#endif

      /* stderr nach /dev/null umleiten, um die rsh-Meldungen zu verhindern: */
	{
	  int fd = open ("/dev/null", O_RDWR, 0777);
	  dup2 (fd, 2);
	  close (fd);
	}
#ifndef DEBUG_SIMS
	if (host != local_host) 
	  {
	/* sim_slave-Prozess auf einem anderen Rechner mittels rsh starten: */
	    execl ("/usr/ucb/rsh", "rsh", host->hostname, "-n", "nice", "+4",
		   commandpath, local_hostname, port, t_limit, NULL);
	    perror ("can't exec rsh process");
	    exit (1);
	  }
	else
	  {
	    /* sim_slave-Prozess lokal starten: */
	    execl 
	      (commandpath, "slave", local_hostname, port, t_limit, NULL);
            /*printf("slave: %s %s %s\n",local_hostname, port, t_limit);*/
	    perror ("can't exec sim_slave process");
	    exit (1);
	  }
#else
	/* sim_slave-Prozess lokal mit Debugger starten: */
	sprintf (ups_argument, "%s %s %s", local_hostname, port, t_limit);
	execl ("/home/X11R5/sun4/bin/ups", "ups", commandpath,
	       "/home/z/zoothorn/TimeNET/Simulation/src/", "-a",
	       ups_argument, NULL);
	perror ("can't exec ups process");
	exit (1);
#endif
      }
    }

  ++host->running_processes;
  info->status = waiting_for_connection;
  
  printf ("Starting DSPN Simulation on station %s.\n\n",
	  host->hostname);
}


/******************************************************************************

 Funktion:      waiting_for_connection

 Parameter:     ---

 Rueckgabewert: Zeiger auf die slaveinfo-Struktur des sim_slave-Prozesses, zu
		welchem eine Socketverbindung aufgebaut wurde. Ausserdem
		werden die Felder status, connection_fd und connection_fp
		des Rueckgabewertes modifiziert.

 Zweck:         Auf die Verbindung zu einem sim_slave-Prozess warten. Ist eine
		Verbindungsaufnahme erfolgt, wird die zur Verbindung gehoerende
		slaveinfo-Struktur modifiziert.

******************************************************************************/
/* Auf die Verbindung zu einem sim_slave-Prozess warten: */
/* Rueckgabewert: Index in die Arrays connection_fds, connection_fps
   und connection_hosts */
static slaveinfo *wait_for_connection ()
{
  struct sockaddr_in connection_addr;
  struct hostent *hostentry;
  int fd, connection_addrlen;
  struct timeval timeout;
  fd_set fd_readset;
  int select_return;
  unsigned i;

  while (1)
  {
    FD_ZERO (&fd_readset);
    FD_SET (server_fd, &fd_readset);
    timeout.tv_sec = 60L;
    timeout.tv_usec = 0L;

/* Maximal 60 Sekunden auf die Verbindung zu einem 
   sim_slave-Prozess warten: 
   unter solaris dauert das ewig !!!, daher von 20 auf 60 sec. vergroessert*/
    do
      {
#ifdef DEBUG_SIMS
	select_return = select (FD_SETSIZE, &fd_readset, NULL, NULL, NULL);
#else
	select_return = select (FD_SETSIZE, &fd_readset, NULL, NULL, &timeout);
#endif
/*      printf("select returns %d\n", select_return); */
      }
    while (select_return < 0 && errno == EINTR);
    if (select_return < 0)
    {
      perror ("select");
      exit (1);
    }
    if (!select_return)
      {
	fprintf(stderr, "timeout in select\n");
	return NULL;
      }

    /* Verbindung herstellen: */
    memset (&connection_addr,0, sizeof (connection_addr));
    do
    {
      connection_addrlen = sizeof (connection_addr);
      fd = accept (server_fd, (struct sockaddr *)&connection_addr, &connection_addrlen);
    }
    while (fd < 0 && errno == EINTR);
    if (fd < 0)
    {
      perror ("accept");
      exit (1);
    }

/* Adresse des Rechners, auf welchem der sim_slave-Prozesse laeuft, 
   ermitteln: */
    if (!(hostentry = gethostbyaddr ((void *)&connection_addr.sin_addr.s_addr, 4,
				     AF_INET)))
    {
      perror ("gethostbyaddr");
      exit (1);
    }
    {
      register char *dot = strchr (hostentry->h_name, '.');
      if (dot)
	*dot = '\0';
    }

    for (i = 0; i < no_slaves; ++i)
    {
      register slaveinfo *info = &slaveinfos[i];
    int host_ok = !strcmp (hostentry->h_name, info->host->hostname) ||
                  !strcmp ("localhost", hostentry->h_name);
                  /* falls Rechner nicht am Netz, Renate, 2.7.97 */
      if (info->status == waiting_for_connection && host_ok) 
      {
	info->status = connected;
	info->connection_fd = fd;
	/* Buffermechanismen der C-Library an die Socketverbindung haengen: */
	if (!(info->connection_fp = fdopen (fd, "w")))
	{
	  perror ("fdopen");
	  exit (1);
	}
	if (execution_mode == distributed)
	  printf ("Connected with %s.\n", hostentry->h_name);
	return info;
      }
    }

    /* Fehler: Verbindung zu einem unbekannten Prozess wurde hergestellt */
    fprintf (stderr, "unknown process on station %s has tried to connect.\n",
	     hostentry->h_name);
    close (fd);
  }
}


#ifdef STR
/******************************************************************************

 Funktion:      str_receive_samples

 Parameter:     slave_fd ist der Filedesciptor der socket
		measure_id kennzeichnet die Statistik, fuer die die 
		Daten bestimmt sind
		no_samples kennzechnet die Anzahl der zu lesenden Einzelwerte
s
 Rueckgabewert: --

 Zweck:         Lesen einer transienten Phase von einem sim_slave-Prozess 

******************************************************************************/
str_receive_samples(slave_fd, slave_id, measure_id, no_samples)
     int measure_id, no_samples, slave_id, slave_fd;
{
  str_sample_type *samples;  
  int i, j, min, max, column;
  Str_Stat *STR_Measure;
  void Analysis ();
  void expand_sample_space();
  
  STR_Measure = all_str_measures[measure_id];

  if (STR_Measure->StopSimulation == true) 
    {
      /* printf("NOTE: samples of measure %d received after MEASURE_STOP\n",
	     measure_id); */ 
    if (STR_Measure->StopSendedToSlave  == false)
      {
	 send_command(ALL,MEASURE_STOP,measure_id);
	 printf("\n Measure %d has reached the predefined accuracy \n",measure_id); 
       STR_Measure->StopSendedToSlave = true;
       } 
    samples = STR_Measure->dummy_samples;   /* Dient dazu, ueberfluessige 
						samples wegzulesen */ 

    my_read(samples,  
	     no_samples * sizeof(str_sample_type), slave_fd);
    }
  else
    {
      /* naechste freie Spalte ermitteln */
      column = STR_Measure->SampleCount; 
      samples = STR_Measure->SampleBlock[column];
      my_read (samples, no_samples * sizeof(str_sample_type), slave_fd);

      STR_Measure->SampleCount++;
      printf ("."); fflush(stdout);  
    } 
  if ( ((STR_Measure->SampleCount % ANALYSIS_SIZE) == 0  &&
	 !STR_Measure->StopSimulation)  ) 
    {
      Analysis (STR_Measure);
      if( (STR_Measure->StopSimulation == false )
          && ( (STR_Measure->SampleCount % (SAMPLE_MALLOC_SIZE)) == 0) )
      expand_sample_space(STR_Measure);
    }


  if ((max_execution_time > 0.0) && 
      ((time(NULL) - start_time) > max_execution_time))
    {
      if(!calc_stats_done)
      {           
        printf("\nNOTE: Simulation at master has exceeded real time limit.\n");
        reached_max_real_time();
        Measure_data_out();
        send_command (ALL, MASTER_STOP);      
        calc_stats_done = true;
      } 
      fatal_error = true;
      receive_messages (wait_for_all);
      
    }
}


#endif


#ifndef STR
/******************************************************************************

 Funktion:      receive_samples

 Parameter:     slave_fd ist der Filedesciptor der socket
		measure_id kennzeichnet die Statistik, fuer die die 
		Daten bestimmt sind
		no_samples kennzechnet die Anzahl der zu lesenden Einzelwerte

 Rueckgabewert: --

 Zweck:         Lesen von Einzelwerten von einem sim_slave-Prozess 

  Funktion im Ramen des PDV-Projekts im WS 95/96 gendert. (BM)

  nderung:     Die vom socket gelesenen Samples werden nicht direkt in die 
                Ringspeicherstruktur samples[]
		geschrieben, sondern erst zwischengespeichert (SampleBuffer[]).
		Aus dem SampleBuffer[] werden sie dann einzeln ausgelesen und 
		in den  Rimgspeicher kopiert.
		Dieser Zwischenschritt ist nicht zwingend ntig, macht aber den
		Programm- und Datenflu bersichtlicher, da es vorkommen kann,
		da ein empfangenes Sample-Paket niht vollstndig in den
		Ringspeicher pat, und so zwischen durch die Analyse aufgerufen
		werden mu. Es wrde also whrend des Lesens vom socket bereits
		eine Auswertung der daten erfolgen. wir haben uns entschlossen,
		das Lesen eines kompletten Telegramms vom socket erst	
		abzuschlieen, bevor wir eine neue Auswertung starten.

  Autoren:      Mathias Neumann und Benjamin Hohnhuser

  Datum:   	April 1996

******************************************************************************/
#ifdef __STDC__
receive_samples(int slave_fd,
		int slave_id,
		int measure_id,
		int no_all_samples)	/*alle empfangenen samples (BM)
					 *Verarbeitung in no_samples-Stcken*/
#else
receive_samples(slave_fd, slave_id, measure_id, no_all_samples)
                int slave_fd;
		int slave_id;
		int measure_id;
		int no_all_samples;
#endif
{
  /*Variablen fr adaptive Blockgren (BM)*/
  int no_samples;	/*Anzahl der in einem Schritt zu lesenden Samples
			 *hchstens STAT_BLOCK_DEF*/
  int no_rest_samples;	/*zur Rckwrtszhlung in der Leseschleife*/
  int no_all_cvs;	/*Anzahl aller CVS-Daten
			 *(wird in no_cvs-Schritten abgearbeitet)*/
  sample_type  SampleBuffer[RECEIVE_BLOCK_DEF];	/*bevor man an die CVS-Daten
						 *herankommt, mssen die 
						 *Samples ausgelesen sein*/
  /*my_read liet also erst alle Samples in einen Zwischenspeicher 
   *(SampleBuffer[]) ein, danach werden sie einzeln nach samples[] kopiert*/

  sample_type *samples;
  double *cvs;
  int i, j, min, max, column,no_cvs;
  static int printlen;
  char printout[80];
  bool rdy = true;
  void calc_measures ();
#ifdef RST
  static int skipped[20]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  static int write_count[20]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
#endif

#ifdef SPEC
  SpecStat *SPEC_Measure, *HELP_Measure;
#ifdef RST
  bool VARIANCE_Analysis ();
#else
  bool SPEC_Analysis ();
#endif
  SPEC_Measure = all_spec_measures[measure_id];

#ifdef RST
  SPEC_Measure->BlockAdminSum[slave_id]=0; /* overwrite old sample */
  SPEC_Measure->BlockAdminFirst[slave_id]=0;
  ++write_count[slave_id];
#ifdef DEBUG_RST
  fprintf(stdout,"write_count[%d]=%d\n",slave_id,write_count[slave_id]);
#endif
  for (i=0; i<no_slaves; ++i)
      if (((write_count[slave_id]-write_count[i])>100)&&!skipped[i]) /* slave i is too slow */
	{
	skipped[i]=1; /* skip slave i */
	fprintf(stdout,"\n");        
	fprintf(stdout,"skipped slave %d\n",i);
	}
#endif

  /*alle Samples in SampleBuffer einlesen (BM)*/
  my_read (SampleBuffer, no_all_samples*sizeof(sample_type), slave_fd);

  /*Schleife ber no_all_samples
   *es werden jeweils no_samples (<=STAT_BLOCK_DEF) Samples eingelesen
   *in den Ringbuffer abgespeichert
   *und dann die Ringbufferverwaltung aktualisiert (BM)
   */
  no_rest_samples = no_all_samples;
  while (no_rest_samples > 0) {
    if (no_rest_samples >= STAT_BLOCK_DEF) {
      no_samples = STAT_BLOCK_DEF;
    } else {
      no_samples = no_rest_samples;
    }
    no_rest_samples -= STAT_BLOCK_DEF;	/* no_rest_samples := brige samples*/

    /* naechste freie Spalte ermitteln */
    column = (SPEC_Measure->BlockAdminFirst[slave_id] + 
	      SPEC_Measure->BlockAdminSum[slave_id]) % MAX_BLOCK_DIFF;
    samples = SPEC_Measure->SampleBlock[slave_id][column];
    if (SPEC_Measure->StopSimulation == true) {
      /*printf("\nNOTE: samples of measure %d received after MEASURE_STOP\n",
       *measure_id);*/
      /*fertig (BM)*/
#ifndef RST
      if (Visualisation==1) visual(measure_id,SPEC_Measure);
#endif
      return;
    }

#if 0
    for (i=0; i<printlen; i++)
      printf("\b");
    sprintf(printout, "reading slave %d, measure %d, block %d, column %d", 
	    slave_id, measure_id,SPEC_Measure->BlockAdminSum[slave_id],column);
    printlen = strlen(printout);
    printf (printout);
#endif
    printf ("."); fflush(stdout);

    /*Sampleteilblock fr die Verarbeitung vorbereiten (BM)*/
    for (i=0; i<no_samples; i++) {
      samples[i] = SampleBuffer[no_all_samples-1-no_rest_samples+i];
    }

    /* ein sample block von sim_slave[slave_id] empfangen */
    SPEC_Measure->BlockAdminSum[slave_id] ++; 
    min = 1; max = SPEC_Measure->BlockAdminSum[0]; 
    for (i=0; i<no_slaves; i++) {
      if (SPEC_Measure->BlockAdminSum[i] > max) /* maximale Anzahl der */
	max = SPEC_Measure->BlockAdminSum[i];   /* aufgezeichneten Bloecke */
						/* suchen */
#ifdef DEBUG_RST     
      printf("Slave%d\tBlockAdminSum[%d]=%d\tskipped[%d]=%d\tmax=%d\n"
             ,i,i,SPEC_Measure->BlockAdminSum[i],i,skipped[i],max);
#endif
#ifdef RST
      if ((SPEC_Measure->BlockAdminSum[i] == 0)&&(!skipped[i]))     
#else
      if (SPEC_Measure->BlockAdminSum[i] == 0)     
#endif
	min = 0;
    }
    if ((min == 0) && (max < MAX_BLOCK_DIFF))
      /*fertig (BM)*/
      continue;
    else { /*nicht laenger warten oder von allen slaves ein block da,
	    *-> einen block auswerten */
#ifdef RST
      printf ("\n"); fflush(stdout);  
#endif
      if (!SPEC_Measure->StopSimulation)
      {
       SPEC_Measure->EndTime=0.0;
       for(i=0;i<no_slaves;i++) 
        SPEC_Measure->EndTime+=slaveinfos[i].slave_local_time;
      }
      for  (j=0; j<no_samples; j++)
	for (i=0; i<no_slaves; i++) {
	  samples = 
	    SPEC_Measure->SampleBlock[i][SPEC_Measure->BlockAdminFirst[i]];
#ifdef RST
	  if (!skipped[i])
#else
	  if (SPEC_Measure->BlockAdminSum[i])
#endif
	    {
	      if (!SPEC_Measure->StopSimulation)
		slaveinfos[i].slave_local_time = samples[j].s_time;
#ifdef DEBUG_RST
	 	printf("measure id: %d, sample: %f\n", 
		       measure_id, samples[j].value); 
#endif
#ifdef RST
	      VARIANCE_Analysis (SPEC_Measure, samples[j].value);
#else
	      SPEC_Analysis (SPEC_Measure, samples[j].value);
#endif
	    }
	}
      for (i=0; i<no_slaves; i++) /* blockzaehler dekrementieren */	        	
      {
#ifdef RST
	  write_count[i]=0;
	  if (!skipped[i])
#else
	  if (SPEC_Measure->BlockAdminSum[i])
#endif
	    {
	      SPEC_Measure->BlockAdminFirst[i] ++;
	      SPEC_Measure->BlockAdminSum[i] --;
	      if (SPEC_Measure->BlockAdminFirst[i] == MAX_BLOCK_DIFF)
		SPEC_Measure->BlockAdminFirst[i] = 0;
	    }
	  else
	    if (slaveinfos[i].status == running)
	      {
		slaveinfos[i].slave_local_min_flag = true;
/*              printf("\nNOTE: No longer waiting for slave at host %s.\n",
		       slaveinfos[i].host->hostname);
		printf("I skip a block of %d samples!\n", no_samples);*/
	      }
	}
    }

#ifdef RST
    VARIANCE_Analysis (SPEC_Measure, END_OF_SEQUENCE);
#endif
    
    /* debug 
     *printf("Statistic: %d \tSample: %f\n", measure_id, sample);*/

    if (SPEC_Measure->StopSimulation) /* Statistik hat vorgegebene Genauigkeit 
				       *erreicht, wird nicht weiter gefuehrt */
      /* |||Jerweitern auf oder-abfrage oder je nach definition */
      {
	printlen = 0;
        SPEC_Measure->EndTime=0.0;
	for (i=0; i<no_slaves; i++)
	  SPEC_Measure->EndTime += slaveinfos[i].slave_local_time;
	send_command (ALL, MEASURE_STOP, measure_id);
	if (SPEC_Measure->NoPrecision == false)
	  printf("\nMeasure %d has reached the predefined accuracy.\n", measure_id);
	else
	  printf("\nMeasure %d has reached the maximum number of samples \nor the maximum model time.\n", measure_id);
	for (i=0; i<no_measures; i++)
	  {
	    HELP_Measure = all_spec_measures[i];
	    rdy = rdy && HELP_Measure->StopSimulation; 
	  }
	if (rdy) /* all measures ok */
	  {
	    calc_measures(res_file);
	    calc_stats_done = true;
	  }
      }
  } /*while (no_rest_measures > 0)*/

  /* Abfrage auf max real time */
  if ((max_execution_time > 0.0) && 
      ((time(NULL) - start_time) > max_execution_time))
    {
      printf("\nNOTE: Simulation at master has exceeded real time limit.\n");
      for (j=0; j<no_measures; j++)
	{
	  HELP_Measure = all_spec_measures[j];
	  if (!HELP_Measure->StopSimulation)
	    {
              HELP_Measure->EndTime=0.0;
	      for (i=0; i<no_slaves; i++)
		HELP_Measure->EndTime += slaveinfos[i].slave_local_time;
	      HELP_Measure->StopSimulation = true;
	      HELP_Measure->NoPrecision = true;
	      StatSema--;
	    }
	}
      send_command (ALL, MASTER_STOP);
      calc_measures(res_file);
      calc_stats_done = true;
      fatal_error = true;
      receive_messages (wait_for_all);
    }
#endif

  /*Behandlung der ControlVariates (BM)*/
#ifdef CVR
  CvR_Stat *CVR_Measure, *HELP_Measure;
  bool CVR_Analysis ();

  CVR_Measure = all_cvr_measures[measure_id];

  /*alle Samples in SampleBuffer einlesen (BM)*/
  my_read (SampleBuffer, no_all_samples*sizeof(sample_type), slave_fd);

  /*Gesamtanzahl der CVS-Daten einlesen (BM)*/
  my_read (&no_all_cvs, sizeof(int), slave_fd);
  if (no_all_cvs != CVR_Measure->ControlVariates*no_all_samples)
    {
      printf ("ERROR: Slave sends %d control variate samples\n", no_cvs);
      printf ("       expected at master: %d control variate samples\n", 
	      CVR_Measure->ControlVariates*no_samples);
      exit (FAIL);
    }
  /*Schleife ber no_all_samples
   *es werden jeweils no_samples (<=STAT_BLOCK_DEF) Samples eingelesen
   *in den Ringbuffer abgespeichert
   *und dann die Ringbufferverwaltung aktualisiert (BM)
   */
  no_rest_samples = no_all_samples;
  while (no_rest_samples > 0) {
    if (no_rest_samples >= STAT_BLOCK_DEF) {
      no_samples = STAT_BLOCK_DEF;
    } else {
      no_samples = no_rest_samples;
    }
    no_rest_samples -= STAT_BLOCK_DEF;	/* no_rest_samples := brige samples*/
    no_cvs = CVR_Measure->ControlVariates*no_samples;

    /* naechste freie Spalte ermitteln */
    column = (CVR_Measure->BlockAdminFirst[slave_id] + 
	      CVR_Measure->BlockAdminSum[slave_id]) % MAX_BLOCK_DIFF;
    samples = CVR_Measure->SampleBlock[slave_id][column];
    cvs = CVR_Measure->CVBlock[slave_id][column];

    /*Sampleteilblock fr die Verarbeitung vorbereiten (BM)*/
    for (i=0; i<no_samples; i++) {
      samples[i] = SampleBuffer[no_all_samples-1-no_rest_samples+i];
    }

    printf ("."); fflush(stdout);
    /* test printouts */
    /*  printf("samples %d  cvs %d\n", no_samples, no_cvs);*/
    my_read (cvs, no_cvs*sizeof(double), slave_fd);
    if (CVR_Measure->StopSimulation == true) 
      {
	/*printf("NOTE: samples of measure %d received after MEASURE_STOP\n",
	 *measure_id);*/
	/*socket leerlesen (restliche CVs) (BM)*/
	continue;
      }
    /* einen sample block von sim_slave[slave_id] empfangen */
    CVR_Measure->BlockAdminSum[slave_id] ++; 
    min = 1; max = CVR_Measure->BlockAdminSum[0]; 
    for (i=0; i<no_slaves; i++)
      {
	if (CVR_Measure->BlockAdminSum[i] > max) /* maximale Anzahl der */
	  max = CVR_Measure->BlockAdminSum[i];   /* aufgezeichneten Bloecke */
	if (CVR_Measure->BlockAdminSum[i] == 0)     /* suchen */
	  min = 0;
      }
    if ((min == 0) && (max < MAX_BLOCK_DIFF)) 
      /* ||| hier auf min. no replic. achten*/
      /*socket leerlesen (restliche CVs) (BM)*/
      continue;
    else /* nicht laenger warten oder von allen slaves ein block da,
	    -> einen block auswerten */
      {
	for  (j=0; j<no_samples; j++)
	  for (i=0; i<no_slaves; i++)
	    {
	      samples = 
		CVR_Measure->SampleBlock[i][CVR_Measure->BlockAdminFirst[i]];
	      cvs = 
		CVR_Measure->CVBlock[i][CVR_Measure->BlockAdminFirst[i]];
	      if (CVR_Measure->BlockAdminSum[i])
		{
		  if(!CVR_Measure->StopSimulation)
		    slaveinfos[i].slave_local_time = samples[j].s_time;
		  CVR_Analysis (CVR_Measure, samples[j].value, cvs, j);
		}
	    }
	for (i=0; i<no_slaves; i++) /* blockzaehler dekrementieren */
	  {
	    if (CVR_Measure->BlockAdminSum[i])
	      {
		CVR_Measure->BlockAdminSum[i] --;
		CVR_Measure->BlockAdminFirst[i] ++;
		if (CVR_Measure->BlockAdminFirst[i] == MAX_BLOCK_DIFF)
		  CVR_Measure->BlockAdminFirst[i] = 0;
	      }
	    else
	      if (slaveinfos[i].status == running)
		{
		  slaveinfos[i].slave_local_min_flag = true;
		  /*printf("\nNOTE: No longer waiting for slave at host %s.\n",
		    slaveinfos[i].host->hostname);
		    printf("I skip a block of %d samples!\n", no_samples);*/
		}
	  }
      }

    if (CVR_Measure->StopSimulation)  /*Statistik hat vorgegebene Genauigkeit 
				       *erreicht, wird nicht weiter gefuehrt */
      {
        CVR_Measure->EndTime=0.0;
	for (i=0; i<no_slaves; i++)
	  CVR_Measure->EndTime += slaveinfos[i].slave_local_time;
	send_command (ALL,MEASURE_STOP, measure_id);
	if (CVR_Measure->NoPrecision == false)
	  printf("\nMeasure %d has reached the predefined accuracy.\n", measure_id);
	else
	  printf("\nMeasure %d has reached the maximum number of samples.\n", measure_id);
	for (i=0; i<no_measures; i++)
	  {
	    HELP_Measure = all_cvr_measures[i];
	    rdy = rdy && HELP_Measure->StopSimulation; 
	  }
	if (rdy) /* all measures ok */
	  {
	    calc_measures(res_file);
	    calc_stats_done = true;
	  }
      }
  } /*while (no_rest_measures > 0) (BM)*/

  /* Abfrage auf max real time */
  if ((max_execution_time > 0.0) && 
      ((time(NULL) - start_time) > max_execution_time))
    {
      printf("\nNOTE: Simulation at master has exceeded real time limit.\n");
      for (j=0; j<no_measures; j++)
	{
	  HELP_Measure = all_cvr_measures[j];
	  if (!HELP_Measure->StopSimulation)
	    {
              HELP_Measure->EndTime=0.0;
	      for (i=0; i<no_slaves; i++)
		HELP_Measure->EndTime += slaveinfos[i].slave_local_time;
	      HELP_Measure->StopSimulation = true;
	      HELP_Measure->NoPrecision = true;
	      StatSema--;
	    }
	}
      send_command (ALL, MASTER_STOP);
      calc_measures(res_file);
      calc_stats_done = true;
      fatal_error = true;
      receive_messages (wait_for_all);
    }
#endif  /* ifdef CVR */
#ifndef RST
      if (Visualisation == 1) visual(measure_id,
#ifdef SPEC
                                                SPEC_Measure);  
#elif CVR                                       
                                                CVR_Measure);
#endif 

#endif /* ifndef RST */
}

#ifndef RST
/**************************************************************************

static void visual(measure_id,Measure)

**************************************************************************/
static void visual(measure_id,Measure)
 int measure_id;
#ifdef CVR
 CvR_Stat *Measure;
#elif SPEC
 SpecStat *Measure;
#endif
{
 int i;
 double inter_time=0.0;

  if(Measure->StopSimulation==false)
  {
   Measure->EndTime=0.0;
   for(i=0;i<no_slaves;i++) 
    Measure->EndTime+=slaveinfos[i].slave_local_time;
  }
 
#ifdef debug                       /* 10/96 protokollierung und visulisierung */
  fprintf(protfile,"%d:\tMeasure%d:\t",Measure->SampleCount,measure_id);
  for(i=0;i<no_slaves;i++) 
   fprintf(protfile,"slave_t[%d]=%g\t",i,slaveinfos[i].slave_local_time);
  fprintf(protfile,"average_t=%g\t",inter_time/no_slaves);
  fprintf(protfile,"MeanValue=%g\tDeltaX=%g\n",
#ifdef CVR
                                       Measure->AveragedMean,Measure->DeltaX);
#endif
#ifdef SPEC
                                       Measure->MeanValue,Measure->DeltaX);
#endif
#endif /*debug*/


  if (last_time[measure_id]<=Measure->EndTime/no_slaves)        /* keine aeltern Werte, als */
  {                                                            /* den letzten darstellen */
   last_time[measure_id]=Measure->EndTime/no_slaves;
#ifdef debug
   printf("Measure->Meanvalue=%g  M->EndTime=%g\n",
#ifdef SPEC
           Measure->MeanValue,Measure->EndTime);   
#elif CVR
           Measure->AveragedMean,Measure->EndTime);   
#endif
#endif
   handle_visualisation(measure_id,Measure->EndTime);
  }
}       /* end of visual*/ 
#endif  /* not RST */

#endif  /* <- ifndef str */


/******************************************************************************

 Funktion:      receive_messages

 Parameter:
		mode == wait_for_one -> Warten, bis ein sim_slave-Prozess 
					terminiert ist, um ggf. einen 
					neuen starten zu koennen
		mode == wait_for_all -> Warten, bis alle sim_slave-Prozesse
					fertig sind, falls keine neuen mehr
					gestartet werden muessen

 Rueckgabewert: Einige Felder von slaveinfo-Strukturen terminierter Prozesse
		werden modifiziert.

 Zweck:         Lesen einer Nachricht (Bildschirmausgabe) eines sim_slave-
		Prozesses.
		Die Nachrichten eines sim_slave-Prozesses werden gesammelt aus-
		gegeben, sobald der Prozess terminiert ist. 
		Beim Eintreffen der sample Ankuendigung SAMPLES wird 
		receive_samples aufgerufen
		Je nach mode
		wartet die Funktion evtl. bis ein oder alle sim_slave-Prozesse
		terminiert sind.
		
  nderung:	no_samples (Anzahl der samples in einem bertragenen Paket) ist
                variabel, mu aber ein ganzzahliges Vielfaches von
		STAT_BLOCK_DEF sein.
		STAT_BLOCK_DEF ist die kleinste sample-Paket-Einheit fr die
		Zwischenspeicherung der samples.
		no_samples darf nicht grer als RECEIVE_BLOCK_DEF sein.

  Autoren:	Benjamin Hohnhuser und Mathias Neumann

  Datum:   	April 1996

******************************************************************************/
void receive_messages (mode)
receive_mode mode;
{
  static char **outputs;                /* Array von Strings, in welchen die
					   Ausgaben jedes sim_slave-Prozesses 
					   gespeichert werden */
  fd_set fd_readset;                    /* Menge von Read-Filedescriptoren
					   fuer select */
  char message_buffer[200], *message, ack;
  bool ready, once_again, sim_end, mess_known;
  int select_return, read_return, slave_id, measure_id, trans_length;
  unsigned i;
  double trans_time;
#ifdef SPEC
  SpecStat *Measure;
#endif
#ifdef CVR
  CvR_Stat *Measure;
#endif
#ifdef STR
  Str_Stat *Measure;
#endif

  if ((long)signal (SIGTERM, user_break) < 0)
  {                             	/* signal-handler for */
                                        /* kill => Master hezi 11/96 */
    perror("signal");
    exit(1);
  }

  if (!outputs)
    if (!(outputs = (char **)calloc (no_slaves + 1, sizeof (char *))))
      OUT_OF_MEMORY;
  
  do
  {
    /* Filedescriptor readset erstellen: */
    FD_ZERO (&fd_readset);
    for (ready = true, i = 0; i < no_slaves; ++i)
      if (slaveinfos[i].status == running)
	{
	  FD_SET (slaveinfos[i].connection_fd, &fd_readset);
	  ready = false;
	}
    if (ready)
      /* Alle sim_slave-Prozesse fertig */
      return;

    once_again = false;
    do
    {
	/* Auf Nachrichten der sim_slave-Prozesse warten: */
	select_return = select (FD_SETSIZE, &fd_readset, NULL, NULL, NULL);
    }
    while (select_return < 0 && errno == EINTR);
    if (select_return < 0)
      {
	perror ("select");
	exit (1);
      }
    if (select_return > 0)
      for (i = 0; i < no_slaves; ++i)
	{
	  register slaveinfo *info = &slaveinfos[i];
	  if (info->status == running &&
	      FD_ISSET (info->connection_fd, &fd_readset))
	    {
	      /* Von sim_slave-Prozess i ist eine Nachricht da */
	      message = message_buffer - 1;
	      sim_end = false;
	      do
		{
		  if ((read_return = 
		       read (info->connection_fd, ++message, 1)) < 0)
		    {
		      if (execution_mode == distributed)
			{
			  printf ("Simulation slave %u at host %s failed!\n",
				  i,  info->host->hostname);
#ifndef RST
			  if (StatSema && !fatal_error) 
			    puts ("trying again...\n");
#endif
			}
		      else
			printf ("Local simulation slave %u failed!\n ", i);
		      if (outputs[i])
			printf("%s", outputs[i]);
		      /* Socket-Verbindung schliessen: */
		      fclose (info->connection_fp);
		      info->connection_fp = (FILE *)0;
		      info->connection_fd = 0;
		      --info->host->running_processes;
		      info->host = (hostinfo *)0;
		      if (sim_end && execution_mode == distributed)
			info->status = no_process;
		      else
			info->status = terminated;
		      --running_processes;
		      free (outputs[i]);
		      outputs[i] = (char *)0;
		      if (mode == wait_for_one)
			return;
		    }
		  else
		    if (!read_return) /* das heisst EOF wurde gesendet, also
					 hat slave ein close gemacht */
		      {
			sim_end = true;
			break;
		      }
		}
	      while (*message && *message != '\n' &&
		     message - message_buffer < 199); 
	      *message = '\n';
	      *++message = '\0';
	      mess_known = false;

	      if (!strncmp (message_buffer, "PID", 3))
		{       /* naechste message ist die PID des Slaves hezi 11/96 */
		  mess_known = true;
		  my_read (&slave_id, sizeof(int), info->connection_fd);
		  if (slave_id != i)
		    {
		      printf("ERROR: PID: slave %d sends id %d\n", i, slave_id);
		      exit(FAIL);
		    }
		  my_read (&slaveinfos[i].PID, sizeof(int), info->connection_fd);
                  idfile=fopen(id_file,"a");
                  fprintf(idfile,"%s\t",slaveinfos[i].host->hostname);
                  fprintf(idfile,"%d\t",slaveinfos[i].PID);
                  fprintf(idfile,"Slave %d\n",i);
                  fclose(idfile);
                }
	      if (!strncmp (message_buffer, "LENGTH", 6))
		{       /* naechste message ist die Laenge der
			   transienten Phase */
		  mess_known = true;
		  my_read (&slave_id, sizeof(int), info->connection_fd);
		  if (slave_id != i)
		    {
		      printf("ERROR: slave %d sends id %d\n", i, slave_id);
		      exit(FAIL);
		    }
		  my_read (&measure_id, sizeof(int), info->connection_fd);
		  my_read (&trans_length, sizeof(int), info->connection_fd);
		  my_read (&trans_time, sizeof(double), info->connection_fd);
#ifdef SPEC
		  Measure = all_spec_measures[measure_id];
#endif
#ifdef CVR
		  Measure = all_cvr_measures[measure_id];
#endif
#ifndef STR
		  if (trans_length > Measure->InitLength)
		      Measure->InitLength = trans_length;
		  if (trans_time > Measure->TransTime)
		      Measure->TransTime = trans_time;
#endif
		}
	      if (!mess_known && (!strncmp (message_buffer, "SAMPLES", 7)))
		{       /* naechste message ist ein block samples */
		  mess_known = true;
		  my_read (&slave_id, sizeof(int), info->connection_fd);
		  if (slave_id != i)
		    {
		      printf("ERROR: slave %d sends id %d\n", i, slave_id);
		      exit(FAIL);
		    }
		  my_read (&(info->slave_local_min_flag), 
			   sizeof(bool), info->connection_fd);
		  my_read (&measure_id, sizeof(int), info->connection_fd);
		  my_read (&no_samples, sizeof(int), info->connection_fd);

#ifdef STR
		  if (no_samples != no_sampling_points)            
#endif
#ifdef RST
		  if (no_samples != 1)
#endif
		  /*no_samples mu ein ganzzahliges Vielfaches von
		   *STAT_BLOCK_DEF und nicht grer als RECEIVE_BLOCK_DEF
		   *sein (BM)
		   */
#ifdef SPEC
		  if ((no_samples > RECEIVE_BLOCK_DEF) ||
		      (no_samples%STAT_BLOCK_DEF != 0))
#endif
#ifdef CVR
		  if ((no_samples > RECEIVE_BLOCK_DEF) ||
		      (no_samples%STAT_BLOCK_DEF != 0))
#endif
		    {
		      printf("ERROR: %d samples announced!\n", no_samples);
		      printf("Can only receive blocks of %d samples.\n",
#ifdef STR
			     no_sampling_points);
#else
			     STAT_BLOCK_DEF);		/*(BM)*/
#endif
		      exit (FAIL);
		    }
#ifdef STR
		  str_receive_samples
		    (info->connection_fd, slave_id, measure_id, no_samples);
#else
		  receive_samples
		    (info->connection_fd, slave_id, measure_id, no_samples);
#endif

		  if (!min_flag)
		    test_min_flags(); /* update min flag globally if modified*/
		}
	      if (!mess_known && (!strncmp (message_buffer, "T_LIMIT", 7)))
	       /* Modellzeit hat Vorgabe ueberschritten */
		{
		 mess_known = true;
		 time_exceeded = true;
	       }
	      if (!mess_known && (!strncmp (message_buffer, "READY", 5)))
		/* ordnungsgemaesses Ende */
		{
		  mess_known = true;
		  printf("\nSlave %d has terminated correctly.\n", i);
		  if (outputs[i])
		    printf("collected messages:\n\n%s", outputs[i]);
		  send_command (i, MASTER_RDY);
		  read (info->connection_fd, &ack, 1);
		  info->status = no_process;
		}
	      if (!strncmp (message_buffer, "ERROR", 5))
		/* Fehlernachricht, sofort auf den Bildschirm */
		  {
		  /* fprintf(stdout,"%s",message_buffer);   nur noch einmal ausgeben: 10.6.96 */
		  fprintf(stdout, "Slave %d sends error message:\n%s\n",
			    i, message_buffer);
		  fatal_error = true; 
		  }
	      if (sim_end)
		{
		  if (info->status == running)
		    {
		      if (execution_mode == distributed)
			{
			  if (fatal_error == true)
			    printf ("Simulation slave %u at host %s failed or terminated by master!\n",
				  i,  info->host->hostname);
			  else
			    if (max_execution_time!=0.0)
			      printf ("Simulation slave %u at host %s terminated!\n",
				    i,  info->host->hostname);
			    else
			      printf ("Simulation slave %u at host %s failed!\n",
				    i,  info->host->hostname);
			  if (StatSema && !fatal_error) 
			    puts ("trying again...\n");
			}
		      else
			if (max_execution_time!=0.0)
			  printf ("Local simulation slave %u terminated!\n ", i);
			else
			  printf ("Local simulation slave %u failed!\n ", i);
		      if (outputs[i])
			printf("%s", outputs[i]);
		    }               
		  /* Socket-Verbindung schliessen: */
		  fclose (info->connection_fp);
		  info->connection_fp = (FILE *)0;
		  info->connection_fd = 0;
		  --info->host->running_processes;
		  info->host = (hostinfo *)0;
		  if (sim_end && execution_mode == distributed)
		    info->status = no_process;
		  else
		    info->status = terminated;
		  --running_processes;
		  free (outputs[i]);
		  outputs[i] = (char *)0;
		  if (mode == wait_for_one)
		    return;
		}
	      /* Die unbekannten Nachrichten und die Zeitmessung eines
		 sim_slave-Prozesses bis zu seiner Beendigung speichern: */
	      if(!mess_known) 
		{
		  /* zeitmessung*/
		  /* Summe der CPU-Zeiten korrigieren: */
		  if (!strncmp (message_buffer, "Elapsed", 7))
		    {
		      if (!strncmp (message_buffer+8, "CPU time", 8))
			{
			  double rusage_time;
			  
			  sscanf (message_buffer + 17, "%lf", &rusage_time);
			  sum_of_rusage_times += rusage_time;
			}
		      if (!strncmp (message_buffer+8, "overall", 7))
			  sim_end = true;
		    }
		  else
		    if (!outputs[i])
		      {
			if (!(outputs[i] = strdup (message_buffer)))
			  OUT_OF_MEMORY;
		      }
		    else
		      {
			if (!(outputs[i] = 
			      realloc (outputs[i], strlen (outputs[i]) +
				       strlen (message_buffer) + 1)))
			  OUT_OF_MEMORY;
			strcat (outputs[i], message_buffer);
		      }
		}
	      once_again = true;
	    }
	}
  }
  while (once_again);
}


/******************************************************************************

 Funktion:      read_environment

 Parameter:     Name des Environment-Files

 Rueckgabewert: ---

 Zweck:         Einlesen der fuer dieses Modul relevanten Eintraege des
		Environment-Files, z.B.: Epsilon, Hostliste, ...

******************************************************************************/
static void read_environment (filename)
char *filename;
{
  FILE *fp;
  char line[300];
  hostinfo *info = NULL, *prev_hostinfo = NULL;
  enum { NONE = 0, SUN, DEC } reading_hostlist = NONE;
  unsigned priority = 1;

  if (!(fp = fopen (filename, "r")))
  {
    fprintf (stderr, "error in %s: can't open file '%s' for reading!\n", __FILE__, filename);
    exit (1);
  }

  /* Default-Werte: */
  max_processes_per_host = 2;
  max_slaves = 10; 
  

  do
  {
    /* Zeile aus dem Environment-File lesen: */
    fgets (line, 298, fp);

    /* Kommentarzeilen ignorieren */
    if (line[0] == '#' || line[0] == '!' || line[0] == '\n' || line[0] == '\0')
      continue;

    /* eventuelles Newline am Zeilenende entfernen: */
    if (line[strlen (line) - 1] == '\n')
      line[strlen (line) - 1] = '\0';

    if (!strncasecmp (line, "HOSTLISTSTART", 13))
    {
#ifdef ultrix
      reading_hostlist = DEC;
#else
      reading_hostlist = SUN;
#endif
      continue;
    }
    if (!strncasecmp (line, "HOSTLISTEND", 11))
    {
      reading_hostlist = NONE;
      continue;
    }

#ifdef ultrix
    if (reading_hostlist == DEC)
#else
    if (reading_hostlist == SUN)
#endif
    {
      if (!strncasecmp (line, "PRIORITY", 8))
      {
	sscanf (line + 9, "%d", &priority);
	continue;
      }

      /* Den gelesenen Rechnernamen in der Hostliste vermerken: */
      if (!(info = (hostinfo *)malloc (sizeof (hostinfo))))
	OUT_OF_MEMORY;
      {
	register char *dot = strchr (line, '.');
	if (dot)
	  *dot = '\0';
      }

      if (!strncasecmp (line, "USE ALWAYS", 10))
      {
	/* Test mittels rusers, ob jemand am Rechner sitzt, unterdruecken */
	register char *character;

	for (character = line + 10; *character && !isalpha (*character); ++character);
	if (!*character)
	{
	  fprintf (stderr, "missing hostname after USE ALWAYS in configuration file!\n");
	  free (info);
	  continue;
	}
	info->no_rusers = true;
	if (!(info->hostname = strdup (character)))
	  OUT_OF_MEMORY;
      }
      else
      {
	info->no_rusers = false;
	if (!(info->hostname = strdup (line)))
	  OUT_OF_MEMORY;
      }
      info->running_processes = 0;
#ifdef ultrix
      info->free = true;
      info->load = 0;
      ++free_hosts;
#else
      info->free = false;
#endif
      info->priority = priority;
      if (prev_hostinfo)
	prev_hostinfo->next = info;
      else
	hostlist = info;
      prev_hostinfo = info;
    }
    else if (!strncasecmp (line, "MAX PROCESSES PER HOST", 22))
    {
      /* Maximale Anzahl der gleichzeitig auf einem Rechner laufenden 
	 sim_slave-Prozesse lesen: */
      sscanf (line + 23, "%u", &max_processes_per_host);
      if (max_processes_per_host < 1)
	max_processes_per_host = 1;
#ifdef CVR 
      if (max_processes_per_host < MIN_PROCESSES)
	max_processes_per_host = MIN_PROCESSES;
#endif
#ifdef RST 
      if (max_processes_per_host < MIN_PROCESSES)
	max_processes_per_host = MIN_PROCESSES;
#endif
    }
    else if (!strncasecmp (line, "MAX SIMULATION PROCESSES TOTAL", 30))
    {
      /* Maximale Anzahl der insgesamt laufenden 
	 sim_slave-Prozesse lesen: */
      sscanf (line + 31, "%u", &max_slaves);
      if (max_slaves < 1)
	max_slaves = 1;
#ifdef CVR
      if (max_slaves < MIN_PROCESSES)
	max_slaves = MIN_PROCESSES;
#endif
#ifdef RST 
      if (max_slaves < MIN_PROCESSES)
	max_slaves = MIN_PROCESSES;
#endif
#ifdef STR
      max_processes_per_host = 1;
#endif
    }
    else if (!strncasecmp (line, "FORCE LOCAL EXECUTION", 21))
      {
	force_local_execution = true;
	force_distributed_execution = false;
	printf
	  ("Choose local simulation according to the configuration file.\n");  
	printf
	  ("Entry overwrites argument of program call.\n");
      }
    else if (!strncasecmp (line, "FORCE DISTRIBUTED EXECUTION", 27))
      {
	if (force_local_execution = false)
	    force_distributed_execution = true;
      }
  }
  while (!feof (fp));

  if (info)
    info->next = (hostinfo *)0;
  else
  {
    fprintf (stderr, "error in %s: No hostlist found in file '%s'!\n",
	     __FILE__, filename);
    exit (1);
  }

  /* Testen, ob der Rechner, auf dem dieser Prozess gerade laeuft, in
     der Hostliste vermerkt ist: */
  {
    char hostname[200];
    hostinfo *info;

    if (gethostname (hostname, 199))
    {
      perror ("gethostname");
      exit (1);
    }
    {
      register char *dot = strchr (hostname, '.');
      if (dot)
	*dot = '\0';
    }

    for (info = hostlist; info; info = info->next)
      if (!strcmp (hostname, info->hostname))
	break;
    if (info)
      local_host = info;
    else
    {
      if (!(info = (hostinfo *)malloc (sizeof (hostinfo))))
	OUT_OF_MEMORY;
      if (!(info->hostname = strdup (hostname)))
	OUT_OF_MEMORY;
      info->running_processes = 0;
      info->free = false;
      info->priority = 0;
      info->next = hostlist;
      hostlist = local_host = info;
    }
    local_hostname = local_host->hostname;
  }

  fclose (fp);
}


/******************************************************************************

 Funktion:      next_free_host

 Parameter:     ---

 Rueckgabewert: Zeiger auf eine hostinfo-Struktur oder NULL, falls kein
		freier Rechner mehr gefunden wurde

 Zweck:         Bestimmung des naechsten freien Rechners aus der Hostliste.
		Frei bedeutet, dass niemand an der Console eingeloggt ist.

******************************************************************************/
/* Den naechsten freien Rechner aus der Hostliste bestimmen: */
static hostinfo *next_free_host ()
{
  static hostinfo *next_host = (hostinfo *)-1;
  unsigned j;
  register hostinfo *host;

  if ((void *)next_host == (void *)-1)
    next_host = hostlist;

  if (!next_host)
    return NULL;

  for (host = next_host; host; host = host->next)
  {
    utmpidlearr rusersinfo;
    register struct utmpidle *array;
    short found = false;

    if (!host->no_rusers)
    {
	  found = rusers2 (host->hostname, &rusersinfo)==0 ? true : false;
      if (found)
	  {
	  /* Ein Rechner gilt als frei, wenn niemand
	     an der Konsole eingeloggt ist */
	  array = rusersinfo.utmpidlearr_val;
	  for (j = 0; j < rusersinfo.utmpidlearr_len; j++)
	    if (!strncmp (array[j].ui_utmp.ut_line, "console", 8))
	    {
	      found = false;
	      break;
	    }
	  }
    }
	else
	  found = true;

	if (found)
    {
      host->free = true;
      ++free_hosts;
      next_host = host->next;
      return host;
    }
  }
  next_host = NULL;
  return NULL;
}


/******************************************************************************

 Funktion:      determine_load

 Parameter:     Zeiger auf eine hostinfo-Struktur

 Rueckgabewert: Das Feld load der uebergebenen Struktur wird modifiziert

 Zweck:         Bestimmung der load (Auslastung) des durch 'host'
		spezifizierten Rechners

******************************************************************************/
static void determine_load (host)
hostinfo *host;
{
  struct statsswtch rstatinfo;

    if (rstat2 (host->hostname, &rstatinfo))
      host->load = host->no_rusers ? 0 : 2557;  /* 9.99 * 256 */
    else
      host->load = rstatinfo.avenrun[0];
}

/******************************************************************************

 Funktion:      compare_hostinfos

 Parameter:     2 Adressen von Zeigern auf hostinfo-Strukturen

 Rueckgabewert: Integer, welcher angibt, ob item1 logisch kleiner, gleich
		oder groesser als item2 ist

 Zweck:         Vergleichsfunktion zum Sortieren der freien Rechner
		(Array sorted_hosts) nach ihrer Auslastung und
		Prioritaet mittels qsort()

******************************************************************************/
static int compare_hostinfos (item1, item2)
register hostinfo **item1, **item2;
{
  register int value1 = ((*item1)->load + 256) / ((*item1)->priority + 1),
	       value2 = ((*item2)->load + 256) / ((*item2)->priority + 1);

  return (value1 - value2);
}


/******************************************************************************

 Funktion:      sort_hosts_by_load

 Parameter:     ---

 Rueckgabewert: ---

 Zweck:         Sortieren des Arrays sorted_hosts nach Auslastung und
		Prioritaet der Rechner

******************************************************************************/
static void sort_hosts_by_load ()
{
  register hostinfo *host, **host_in_array;

  if (!(sorted_hosts = (hostinfo **)malloc (free_hosts * sizeof (hostinfo *))))
    OUT_OF_MEMORY;

  host_in_array = sorted_hosts;
  for (host = hostlist; host; host = host->next)
    if (host->free)
      *host_in_array++ = host;

  if (host_in_array - sorted_hosts != free_hosts)
  {
    fprintf (stderr, "error in %s: free_hosts incorrect!\n", __FILE__);
    exit (1);
  }

#ifndef ultrix
  /* Rechner nach ihrer Prioritaet und ihrer Auslastung sortieren: */
  qsort (sorted_hosts, free_hosts, sizeof (hostinfo *), compare_hostinfos);
#endif
}

/******************************************************************************

 Funktion:      distributed_execution

 Parameter:     ---

 Rueckgabewert: ---

 Zweck:         Diese Funktion uebernimmt das Management der Prozessverteilung
		im Falle einer verteilten Ausfuehrung der sim_slave-Prozesse, 
		also:
		- benutzbare Rechner bestimmen
		- sim_slave-Prozesse verteilen 
		  (Neustart durch Fehler abgebrochener Prozesse)
		- Daten an sim_slave-Prozesse senden
		- sendet MASTER_STOP und MEASURE_STOP an die sim_slave 
		  Prozesse, wenn gesamte Simulation oder eine bestimmte 
		  Statistik fertig ist
		- MASTER_STOP wird durch die slaves quittiert, siehe 
		  receive_messages

  nderung:	Auf den Speicherblock jedes sample-Pakets wird ber Zeiger
                zugegriffen. Statt bei der Initialisierung mehrfach einzelne
		Speicherblcke fr jedes empfangene sample-Paket getrennt zu
		allozieren, wird ein zusammenhngender Speicherbereich
		erzeugt, auf den die einzelnen sample-Paket-Zeiger wie
		bisher zeigen.

  Autoren:	Benjamin Hohnhuser und Mathias Neumann

  Datum:   	April 1996

******************************************************************************/
static void distributed_execution ()
{
  unsigned i, j, k, tries;
  hostinfo *host, **next_host;
  bool process_started, one_process_per_host;
#ifdef SPEC
  SpecStat *Measure;
#endif
#ifdef CVR
  CvR_Stat *Measure;
#endif
#ifdef STR
  Str_Stat *Measure;
#endif

#ifndef ultrix
  printf ("Free hosts found:\n       ");
  do
  {
    host = next_free_host ();
    if (host)
    {
      printf ("%s ", host->hostname);
      fflush (stdout);
    }
  }
  while (host);
  putchar ('\n');

  if (free_hosts < 2)
  {
    puts ("\nLess than 2 free hosts found, switching to local execution...\n");
    execution_mode = local;
    local_execution ();
    return;
  }

  printf ("loads: ");
  for (host = hostlist; host; host = host->next)
    if (host->free)
    {
      register unsigned len = strlen (host->hostname);
      char format[20];

      determine_load (host);
      sprintf (format, "%%-%d.2f ", len);
      printf (format, (float)host->load / (float)256.0);
      fflush (stdout);
      if (host->load > 1 * 256) /* war 2*256 !!! bis 11.3.96*/
      {
	/* Rechner mit einer load > 2.0 werden nicht verwendet */
	host->free = false;
	--free_hosts;
      }
    }
  puts ("\n");

  if (free_hosts < 2)
  {
    puts ("Less than 2 usable hosts found, switching to local execution...\n");
    local_execution ();
    return;
  }
#endif

  no_slaves = MIN((free_hosts * max_processes_per_host), max_slaves);
  slaveinfos = (slaveinfo*) er_malloc(no_slaves*sizeof(slaveinfo));
  for(i=0; i<no_measures; i++)
    {
#ifdef SPEC
      Measure = all_spec_measures[i];
#endif
#ifdef CVR
      Measure = all_cvr_measures[i];
#endif
#ifdef STR
      Measure = all_str_measures[i];
#endif
      Measure->EndTime = 0.0;
#ifdef STR      
      for (j=0; j<no_slaves; j++)
      {
	slaveinfos[j].status = no_process;
	slaveinfos[j].slave_local_time = 0.0;
	slaveinfos[j].slave_local_min_flag = false;
      }
      Measure->dummy_samples = 
	(str_sample_type *)er_malloc( no_sampling_points * 
                                        sizeof(str_sample_type ));
      Measure->SampleBlock = 
	(str_sample_type **)er_malloc( SAMPLE_MALLOC_SIZE 
                                * sizeof(str_sample_type *));
      for ( k = 0; k < (SAMPLE_MALLOC_SIZE); k++ )
	Measure->SampleBlock[k] = 
	   (str_sample_type *)er_malloc( no_sampling_points  * 
                                           sizeof(str_sample_type)); 
#else
      Measure->InitLength = 0; 
      Measure->TransTime = 0.0; 
      Measure->BlockAdminFirst = ivector(0, no_slaves-1);
      Measure->BlockAdminSum = ivector(0, no_slaves-1);
      Measure->SampleBlock = 
	(sample_type ***)er_malloc(no_slaves*sizeof(sample_type **));
#ifdef CVR
      Measure->CVBlock = 
	(double ***)er_malloc(no_slaves*sizeof(double **));
#endif
      for (j=0; j<no_slaves; j++)
	{
	  Measure->BlockAdminFirst[j] = 0;
	  Measure->BlockAdminSum[j] = 0;
	  slaveinfos[j].status = no_process;
	  slaveinfos[j].slave_local_time = 0.0;
	  slaveinfos[j].slave_local_min_flag = false;

	  /*Zeigerfeld fr Zeiger auf sample- und CV-Pakete erzeugen (BM)*/
	  Measure->SampleBlock[j] = 
	    (sample_type **)er_malloc(MAX_BLOCK_DIFF*sizeof(sample_type *));
#ifdef CVR
	  Measure->CVBlock[j] = 
	    (double **)er_malloc(MAX_BLOCK_DIFF*sizeof(double *));
#endif
	  /*zusammenhngenden Speicher fr sample- und CV-Pakete erzeugen*/
	  Measure->SampleBlock[j][0] = 
	      (sample_type *) er_malloc((MAX_BLOCK_DIFF)*(STAT_BLOCK_DEF)*sizeof(sample_type));
#ifdef CVR
	  Measure->CVBlock[j][0] = 
	    (double *) er_malloc(((MAX_BLOCK_DIFF)*(Measure->ControlVariates)*STAT_BLOCK_DEF)*sizeof(double));
#endif

	  /*Initialisierung des Zeigerfeldes (BM)*/
	  for (k=1; k<MAX_BLOCK_DIFF; k++) {
	    Measure->SampleBlock[j][k] =
	      Measure->SampleBlock[j][0] + k*(STAT_BLOCK_DEF);
#ifdef CVR
	    Measure->CVBlock[j][k] =
	      Measure->CVBlock[j][0] + k*((Measure->ControlVariates)*STAT_BLOCK_DEF);
#endif
	  }

	}
#endif
    }
  sort_hosts_by_load ();

  one_process_per_host = false;
  next_host = sorted_hosts;
  tries = 0;

  while (StatSema && !fatal_error)
  {
    /* Soviel sim_slave-Prozesse wie moeglich verteilt parallel laufen lassen*/
    process_started = false;
    for (; next_host < &sorted_hosts[free_hosts] ; ++next_host)
      {
	host = *next_host;
	if (host->free && host->running_processes < max_processes_per_host &&
	    (!one_process_per_host || !host->running_processes))
	  {
	    /* Auf dem Rechner host->hostname kann ein sim_slave-Prozess
	       gestartet werden: */
	    register unsigned j;
 
	    for (j = 0; j < no_slaves; ++j) 
	      if (slaveinfos[j].status == no_process &&
		  running_processes < max_slaves)
		{
		  slaveinfos[j].host = host;
		  start_sim_process (&slaveinfos[j]);
		  process_started = true;
		  ++running_processes;
		  break;
		}
	    
	    if (j > no_slaves)
	      {
		/* Es konnte kein sim_slave-Prozess gestartet werden */
		tries ++;
		next_host = sorted_hosts;
		one_process_per_host = true;
		break;
	      }
	  }
      }
    if (next_host >= &sorted_hosts[free_hosts])
      {
	tries ++;
	next_host = sorted_hosts;
	one_process_per_host = false;
      }
    if (max_processes_per_host == 1) 
      one_process_per_host = true;
   if ((running_processes == no_slaves) || 
       (running_processes == max_slaves))
     process_started = false;
   if (process_started && !one_process_per_host && (tries < MAX_TRIES))
      continue;
    
    /* Auf eine Verbindung mit den gerade gestarteten 
       sim_slave-Prozessen warten:*/
    for (j = 0; j < no_slaves; ++j)
      if (slaveinfos[j].status == waiting_for_connection)
	{
	  j = 0;
	  if (!wait_for_connection ())
	    {
	      /* Fehler aufgetreten: Ein sim_slave-Prozess meldet sich nicht */
	      register unsigned k;
	      
	      running_processes --;
	      for (k = 0; k < no_slaves; ++k)
		if (slaveinfos[k].status == waiting_for_connection)
		  {
		    fprintf 
		      (stderr, 
		       "can't connect to sim_slave process on station '%s'!\n",
		       slaveinfos[k].host->hostname);
		    slaveinfos[k].status = no_process;
		    slaveinfos[k].host->free = false;
		    slaveinfos[k].slave_local_min_flag = true;
		    --slaveinfos[k].host->running_processes;
		    slaveinfos[k].host = (hostinfo *)0;
		  }
	      putchar ('\n');
	      break;
	    }
	}
#ifdef RST
    if (running_processes < MIN_PROCESSES/2)
      {
	fprintf (stderr, "Could not start sufficient number of slaves for RESTART\n");
	exit (1);
      }
#endif
    /* Daten an die gestarteten Prozesse senden: */
    for (j = 0; j < no_slaves; ++j)
      {
	register slaveinfo *info = &slaveinfos[j];
	if (info->status == connected)
	  {
	    send_infos(info->connection_fp, j);
	    send_transitions(info->connection_fp);
	    send_places(info->connection_fp);
#ifdef CVR
	    send_cv_config(info->connection_fp);
#endif
	    if (info->connection_fp)
	      fflush (info->connection_fp);
	    info->status = running;
	  }
      }
#ifdef RST
    receive_messages(wait_for_all);
    /*    tries = MAX_TRIES+1;
	  running_processes = no_slaves;*/
#else

    if ((running_processes == no_slaves) || 
	(running_processes == max_slaves) || 
	(tries > MAX_TRIES)) /* es wird nicht mehr versucht, weitere slaves zu
				starten */
      receive_messages(wait_for_one);
#endif
  } /* end while (StatSema)*/
  if (fatal_error == true)
    send_command (ALL, MASTER_STOP);
  receive_messages(wait_for_all);
}

/******************************************************************************

 Funktion:      local_execution

 Parameter:     ---

 Rueckgabewert: ---

 Zweck:         Diese Funktion steuert den Ablauf im Falle einer lokalen
		Ausfuehrung der sim_slave-Prozesse. Entsprechend dem Eintrag
		MAX_PROCESSES_PER_HOST aus dem Environment-File koennen
		evtl. mehrere Prozesse parallel laufen.

  nderung:	Auf den Speicherblock jedes sample-Pakets wird ber Zeiger
                zugegriffen. Statt bei der Initialisierung mehrfach einzelne
		Speicherblcke fr jedes empfangene sample-Paket getrennt zu
		allozieren, wird ein zusammenhngender Speicherbereich
		erzeugt, auf den die einzelnen sample-Paket-Zeiger wie
		bisher zeigen.

  Autoren:	Benjamin Hohnhuser und Mathias Neumann

  Datum:   	April 1996

******************************************************************************/
static void local_execution ()
{
  unsigned slave_count, i, j, k;
  FILE *slave_fp;
#ifdef SPEC
  SpecStat *Measure;
#endif
#ifdef CVR
  CvR_Stat *Measure;
#endif
#ifdef STR
  Str_Stat *Measure;
#endif

  no_slaves = max_processes_per_host;
  slaveinfos = (slaveinfo*) er_malloc(no_slaves*sizeof(slaveinfo));
  for(i=0; i<no_measures; i++)
    {
#ifdef SPEC
      Measure = all_spec_measures[i];
#endif
#ifdef CVR
      Measure = all_cvr_measures[i];
#endif
#ifdef STR
      Measure = all_str_measures[i];
#endif
      Measure->EndTime = 0.0; 
#ifdef STR
      for (j=0; j<no_slaves; j++)
      {
	slaveinfos[j].status = no_process;
	slaveinfos[j].slave_local_time = 0.0;
	slaveinfos[j].slave_local_min_flag = false;
      }
      Measure->dummy_samples = 
	(str_sample_type *)er_malloc( no_sampling_points * 
                                        sizeof(str_sample_type ));
      Measure->SampleBlock = 
	(str_sample_type **)er_malloc( SAMPLE_MALLOC_SIZE 
                                * sizeof(str_sample_type *));
      for ( k = 0; k < SAMPLE_MALLOC_SIZE; k++ )
	Measure->SampleBlock[k] = 
	   (str_sample_type *)er_malloc( no_sampling_points * 
                                           sizeof(str_sample_type)); 

#else
      Measure->InitLength = 0; 
      Measure->TransTime = 0.0; 
      Measure->BlockAdminFirst = ivector(0, no_slaves-1);
      Measure->BlockAdminSum = ivector(0, no_slaves-1);
      Measure->SampleBlock = 
	(sample_type ***)er_malloc(no_slaves*sizeof(sample_type **));
#ifdef CVR
      Measure->CVBlock = 
	(double ***)er_malloc(no_slaves*sizeof(double **));
#endif
      for (j=0; j<no_slaves; j++)
	{
	  slaveinfos[j].status = no_process;
	  slaveinfos[j].slave_local_time = 0.0;
	  slaveinfos[j].slave_local_min_flag = false;
	  Measure->BlockAdminFirst[j] = 0;
	  Measure->BlockAdminSum[j] = 0;

	  /*Zeigerfeld fr Zeiger auf sample- und CV-Pakete erzeugen (BM)*/
	  Measure->SampleBlock[j] = 
	    (sample_type **)er_malloc(MAX_BLOCK_DIFF*sizeof(sample_type *));
#ifdef CVR
	  Measure->CVBlock[j] = 
	    (double **)er_malloc(MAX_BLOCK_DIFF*sizeof(double *));
#endif
	  /*zusammenhngenden Speicher fr sample- und CV-Pakete erzeugen*/
	  Measure->SampleBlock[j][0] = 
	      (sample_type *) er_malloc((MAX_BLOCK_DIFF)*(STAT_BLOCK_DEF)*sizeof(sample_type));
#ifdef CVR
	  Measure->CVBlock[j][0] = 
	    (double *) er_malloc(((MAX_BLOCK_DIFF)*(Measure->ControlVariates)*STAT_BLOCK_DEF)*sizeof(double));
#endif

	  /*Initialisierung des Zeigerfeldes (BM)*/
	  for (k=1; k<MAX_BLOCK_DIFF; k++) {
	    Measure->SampleBlock[j][k] =
	      Measure->SampleBlock[j][0] + k*(STAT_BLOCK_DEF);
#ifdef CVR
	    Measure->CVBlock[j][k] =
	      Measure->CVBlock[j][0] + k*((Measure->ControlVariates)*STAT_BLOCK_DEF);
#endif
	  }

	}
#endif
    }
  
  running_processes = 0;
  for (slave_count=0; slave_count < no_slaves; ++slave_count)
  {
    register struct slaveinfo *info = &slaveinfos[slave_count];

    if (info->status == terminated)
      continue;

    running_processes ++;
    while (1)
    {
      /* Einen sim_slave-Prozess lokal starten: */
      info->host = local_host;
      start_sim_process (info);

      /* Auf die Verbindung zum sim_slave-Prozess warten: */
      if (wait_for_connection () != info)
	{
	  fprintf (stderr, 
		   "can't connect to sim_slave process on station '%s' !\n\n",
		   local_hostname);
	  running_processes --;
	}
      else
	break;
    }
    slave_fp = info->connection_fp;

    /* Netzbeschreibung senden */
    send_infos(slave_fp, slave_count);
    send_transitions(slave_fp);
    send_places(slave_fp);
#ifdef CVR
    send_cv_config(slave_fp);
#endif
    if (info->connection_fp)
      fflush (slave_fp);
    info->status = running;
  }
#ifdef RST
    if (running_processes < MIN_PROCESSES/2)
      {
	fprintf (stderr, "Could not start sufficient number of slaves for RESTART\n");
	exit (1);
      }
#endif

  receive_messages (wait_for_all); /* auf Quittung der MASTER_STOPs warten */
}

#ifdef CVR
/***************************************************************************

 function:        free_meas_trans

 parameters:      meas_trans: global structure which contains measures and
			   their belonging cv-transitions

 purpose:         free allocated memory of meas_trans
		 
 author:          Ralf Putz

 date:            July 1994

****************************************************************************/

static void free_meas_trans(meas_trans)
	    meas_trans_list *meas_trans;
{
  trans_list *temp_trans;  
  meas_trans_list * temp_meas_trans;

  if(meas_trans != NULL)
  {  
    while(meas_trans != NULL)
    {
      temp_meas_trans = meas_trans;
      if(temp_meas_trans->trans != NULL)
      {
	while(temp_meas_trans->trans->next != NULL)
	{
	  temp_trans = temp_meas_trans->trans;
	  temp_meas_trans->trans = temp_meas_trans->trans->next;
	  free(temp_trans);
	}
	free(temp_meas_trans->trans);
      }
      meas_trans = meas_trans->next;
      free(temp_meas_trans);
    }
    free(meas_trans);
  }    /* end if meas_trans != NULL */
}

/******************************************************************************

 Funktion:      print_control_variates

 Parameter:     

 Rueckgabewert: ---

 Zweck:         Diese Funktion gibt fuer jedes maesure die verwendeten
		Kontrollvariablen aus.

******************************************************************************/
void print_control_variates()
{
  int i_control,i_trans;
  cv_list_elem *new_cv_list;
  trans_list *help_trans;
  meas_trans_list *help_meas_trans;

      help_meas_trans = meas_trans;
      while (meas_trans != NULL)
      {
	i_control = 0;
	help_trans = meas_trans->trans;
	if( help_trans != NULL)
	  {
	    printf("\nMEASURE %d uses transition[s] ",meas_trans->meas_id);
	    while (help_trans != NULL)
	      {
		for(i_trans = 0; i_trans < no_trans; i_trans++)
		  if( !strcmp(transitions[i_trans].name,help_trans->name))
		    break;
		 printf("%s ",transitions[i_trans].name);
		 help_trans = help_trans->next;
	       } /* end while */
	    printf("as control variate[s].\n");
	    meas_trans = meas_trans->next;
	  }
	else
	  {
	    printf("\nMEASURE %d uses no control variates.\n",
					      meas_trans->meas_id);
	    meas_trans = meas_trans->next;
	  }
      }
    free_meas_trans(help_meas_trans);
}
#endif

/*************************************************************************

  function:     get_restart_values

  parameter:    --

  purpose:      get the computed values necessary for restart

  author:       Patrick Drescher

  date:         February 1995   

*************************************************************************/
#ifdef RST

get_restart_values(net_name)
char *net_name;
{
int        i;
char       string[100],
	   *letter,
	   param_file[256],
	   *fgets(),
	   *strrchr();
FILE       *filepointer,
	   *fopen();


sprintf(param_file,"%s.PARAM",net_name);

if ((filepointer=fopen(param_file,"r"))==NULL)
   {
   fprintf(stderr,"error: cannot open %s\n",param_file);
   return(0);        
   }
if (fgets(string,100,filepointer)==NULL)
      {
      fprintf(stderr,"error: cannot read measure_place \n");
      return(0);
      }
   else
      if (!strncmp(string,"measure_place",13))
	 {
	 letter=string+16;
	 i=0;
	 while ((*letter!='\n')&&(!isspace(*letter)))
	   {
	   measure_place[i]=*letter;
	   ++letter, ++i;
	   }
	 measure_place[i]='\0';
	 }
      else
	 {
	 fprintf(stderr,"error: cannot read measure_place \n");
	 return(0);
	 }
if (fgets(string,100,filepointer)==NULL)
   {
   fprintf(stderr,"error: cannot read direction \n");
   return(0);
   }
else 
   if (!strncmp(string,"direction",9))
      direction=atoi(strrchr(string,'=')+1);
   else
      {
      fprintf(stderr,"error: cannot read direction \n");
      return(0);
      }
if (fgets(string,100,filepointer)==NULL)
   {
   fprintf(stderr,"error: cannot read event \n");
   return(0);
   }
else 
   if (!strncmp(string,"event",5))
      event=atoi(strrchr(string,'=')+1);
   else
      {
      fprintf(stderr,"error: cannot read event \n");
      return(0);
      }
if (fgets(string,100,filepointer)==NULL)
   {
   fprintf(stderr,"error: cannot read border \n");
   return(0);
   }
else 
   if (!strncmp(string,"border",6))
      border=atoi(strrchr(string,'=')+1);
   else
      {
      fprintf(stderr,"error: cannot read border \n");
      return(0);
      }
if (fgets(string,100,filepointer)==NULL)
   {
   fprintf(stderr,"error: cannot read no_thresholds \n");
   return(0);
   }
else
   if (!strncmp(string,"no_thresholds",13))
      no_thresholds=atoi(strrchr(string,'=')+1);
   else
      {
      fprintf(stderr,"error: cannot read no_thresholds \n");
      return(0);
      }
if (no_thresholds>MAX_THRESHOLDS)
   {
   fprintf(stderr,"error: too much thresholds \n");
   return(0);
   }
if (fgets(string,100,filepointer)==NULL)
   {
   fprintf(stderr,"error: cannot read no_opt_thresholds \n");
   return(0);
   }
else
   if (!strncmp(string,"no_opt_thresholds",17))
      no_opt_thresholds=atoi(strrchr(string,'=')+1);
   else
      {
      fprintf(stderr,"error: cannot read no_opt_thresholds \n");
      return(0);
      }
if (no_opt_thresholds>no_thresholds)
   {
   fprintf(stderr,"error: no_opt_thresholds > no_thresholds\n");
   return(0);
   }
for (i=1; i<=2*no_thresholds; ++i)
   if (fgets(string,100,filepointer)==NULL)
      {
      fprintf(stderr,"error: cannot read %s%-d] \n",
	      ((i/2)*2==i) ? "retrials[" : "t_value[",i);
      return(0);
      }
   else
      if (!strncmp(string,((i/2)*2==i) ? "retrials[" : "t_value[",
	  ((i/2)*2==i) ? 9 : 8))
	 if ((i/2)*2==i)
	    {
	    retrials[i/2]=atoi(strrchr(string,'=')+1);
	    ret_product*=(double)retrials[i/2];
	    }
	 else
	    t_value[(i+1)/2]=atoi(strrchr(string,'=')+1);
      else
	 {
	 fprintf(stderr,"error: cannot read %s%-d] \n",
		 ((i/2)*2==i) ? "retrials[" : "t_value[",i);
	 return(0);
	 }
for (i=1;i<=no_opt_thresholds;++i)
   if (fgets(string,100,filepointer)==NULL)
      {
      fprintf(stderr,"error: cannot read prob_opt[%d] \n",i);
      return(0);
      }
   else
        if (!strncmp(string,"prob_opt[",9))
           prob_opt[i]=atof(strrchr(string,'=')+1);
        else
          {
            fprintf(stderr,"error: cannot read prob_opt[%d] \n",i);
            return(0);
          }

fclose(filepointer);
/* printf("ret_product = %le\n",ret_product); */
return(1);
}

#endif

/******************************************************************************

 Funktion:      uniPdfAreaErr

 Parameter:     measureName, lowerBorder, upperBorder, n, meanValue, 
                variance, alpha

 Rueckgabewert: Anteil der Flaeche ausserhalb von [0;1] 
                (s. Diplomarbeit H.Ziegler 3/97)

 Zweck:         dient der Berechnung des tatsachlichen Konfidenzniveaus 
                f. Wahrsch.k
                ... wird  in spec_var.c / batch_cv.c aufgrufen.

 Autor:         Henning Ziegler 2/97

******************************************************************************/

double uniPdfAreaErr(measureName, lowerBorder, upperBorder, n, meanValue, variance, alpha)
 char   *measureName;
 double lowerBorder, upperBorder;
 int    n;                                /* #Teilintervalle */
 double meanValue, variance;
 double alpha;                            /* Irrtumswahrscheinlichkeit */
{
 int i;
 double h;
 double area=0.0;

#ifdef __STDC__      /* bei cc nicht erlaubt, Renate */
  double f(value)
  double value;
 {
   return exp(-pow((value-meanValue),2.0)/(2*pow(variance,2.0)))/(variance*sqrt(2*Pi));
 } 
#endif

 /* if (meanValue>1.0 || meanValue<0.0) 
 {
  printf("\nProbability-Measure %s=%g : out of range continue \n",measureName,meanValue);
  return 1.1; 
 }*/
#ifdef debug
 printf("\nlowerBorder=%g\tupperBorder=%g\nmeanValue=%g\tvariance=%g\n",
         lowerBorder,upperBorder,meanValue,variance);
#endif
 if (upperBorder > 1.0) 
 {
  upperBorder = 1.0;
 }
 else if (lowerBorder < 0.0) 
 {
  lowerBorder = 0.0;
 }
 else return 0.0;                         /* Grenzen innerhalb von [0;1] O.K.*/

 if(lowerBorder > upperBorder)    /* gesammtes Konfidenzintervall ausserhalb von [0;1] */
  return (1-alpha)*100;
 h=(upperBorder - lowerBorder)/n;         /* Schrittweite */
#ifdef __STDC__
 area = 0.5*f(lowerBorder);
#else 
 area = 0.5*exp(-pow((lowerBorder-meanValue),2.0)/(2*pow(variance,2.0)))/(variance*sqrt(2*Pi));
#endif
 for (i=1;i<n-1;i++)
 { 
#ifdef __STDC__
  area+=f(lowerBorder+h);
#else
 area+ = exp(-pow((lowerBorder+h-meanValue),2.0)/(2*pow(variance,2.0)))/(variance*sqrt(2*Pi));
#endif
  lowerBorder+=h;
 }
 area+=0.5*(upperBorder);
 area=area*h;
 
#ifdef debug
 printf("\narea=%g\t%g difference at the Confidence level for %s\n"
        ,area,((1-alpha)-area)*100,measureName);
#endif
 return ((1-alpha)-area)*100;            /* Anteil (%) der Fl"ache ausserhalb von [0;1] */
			          	 /* innerhalb der Konfidenzgrenzen */
}


/******************************************************************************

 Funktion:      sim_master

 Parameter:     Name des DSPN

 Rueckgabewert: ---

 Zweck:         Diese Funktion wird nach dem Einlesen und Vorbereiten des 
		Netzen durch prepare_sim aufgrufen.

******************************************************************************/
void sim_master (net_name)
char *net_name;
{
  char filename[400], *home;
  void calc_measures ();
  char prot[256];
  int  i;

  netname = net_name;

  /* SIGCHLD-Signale abfangen: */
  if ((long)signal (SIGCHLD, sigchld_handler) < 0)
  {
    perror ("signal");
    exit (1);
  }
  /* moegliche SIGFPEs fuer "inexact" abfangen.. */
  if ((long)signal (SIGFPE, SIG_IGN) < 0)
  {
    perror ("signal");
    exit (1);
  }


  /* Server-Socket erzeugen: */
  establish_server_socket ();
  start_time = time (NULL);

  /* Environment-File einlesen: */
  if (!(home = getenv ("HOME")))
  {
    fprintf (stderr, "error in %s: environment variable 'HOME' not set!\n", __FILE__);
    exit (1);
  }
  sprintf (filename, "%s/%s", home, TNET_RC);
  read_environment (filename);
#ifdef CVR
  print_control_variates();
#endif

#ifdef RST
  if (!get_restart_values(net_name))
     {
     fprintf (stderr, "failure in getting restart_values \n");
     exit(1);
     }
#endif

  puts ("\n\nSIMULATING THE DSPN ...\n");
#ifdef debug
#ifndef RST
#ifndef STR
 sprintf(prot,"%s.prot",netname);   /* Protokollfile */
 protfile=fopen(prot,"w");
#endif
#endif
#endif
 sprintf(id_file,"%s.pid",netname);
 idfile=fopen(id_file,"w");         /* enthaelt alle PIDs .. */
 fprintf(idfile,"# ################### list of all processnumbers #################\n");
 fprintf(idfile,"# model: %s \n",netname);
 fprintf(idfile,"##################################################################\n");
 fprintf(idfile,"\n%s\t%d\tMaster\n",local_hostname,getpid());
 fclose(idfile);
 last_time = (double*)malloc(no_measures*sizeof(double)); /* Speicher fuer die letzten W. */
 for(i=0;i<no_measures;i++) last_time[i]=0.0; 

  /*** Entscheiden, ob die sim_slave-Prozesse verteilt werden sollen: ***/
  if (force_distributed_execution || !(no_slaves < 2 || force_local_execution))
    {
      /*** sim_slave-Prozesse parallel verteilt ausfuehren: ***/
      execution_mode = distributed;

      printf ("Starting all sim_slave processes parallel on different hosts");
      if (max_processes_per_host > 1)
	printf (" with max. %u concurrent processes.\n\n",
		max_processes_per_host);
      else
	puts (".\n");
      distributed_execution();
    }
  else
    {
      /*** sim_slave-Prozesse lokal ausfuehren: ***/
      execution_mode = local;
      if (max_processes_per_host > 1)
	{ 
	  printf ("Starting all sim_slave processes locally ");
	  printf ("with max. %u concurrent processes.\n\n",
		  max_processes_per_host);
	}
      else
	puts ("Starting one sim_slave process locally.\n");
      local_execution ();      
    }

  if (!calc_stats_done) /* Measures schreiben, falls nicht schon in receive
			   samples getan */
#ifdef STR
      Measure_data_out();
#else
  {
      calc_measures (res_file);
#ifndef RST
#ifndef STR
        close_curves_file(sim_curves_file); 
#endif
#endif
  }
#endif

#ifdef debug
#ifndef RST
#ifndef STR
 fclose(protfile);
#endif
#endif
#endif
  /* Zeiten ausgeben: */
  printf ("\n\nTotal CPU time: %.2f sec.\n", sum_of_rusage_times);

  {
    long seconds, minutes = 0, hours = 0;
    seconds = time (NULL) - start_time;
    if (seconds > 59)
      {
	minutes = seconds / 60;
	seconds %= 60;
      }
    if (minutes > 59)
      {
	hours = minutes / 60;
	minutes %= 60;
      }
    printf ("Total overall time: %02lu:%02lu:%02lu\n",
	    hours, minutes, seconds);
  }

}

