/******************************************************************************

File              :     visual.c

Purpose           :     write Data to <model>.sim_curves_file for Visualisation

Programmer        :     Henning Ziegler

History           :     November 1996

exported function :     

local functions   :     
			void write_header_of_curves_file()
			void update_curves_file()
			void close_curves_file()
			bool send_conditions_reached()
			bool visu_conditions_reached()
			void handle_visualisation()

******************************************************************************/

#include <math.h> 
#include <stdio.h>
#include <time.h> 
#include <sys/time.h>
#include <sys/resource.h>

#include "stat_utils.h"
#include "sim_master_types.h"
#include "visual.h"
#include "spec_var.h"
#include "batch_cv.h"

/* external variables */
extern  double          current_time, maxtime;
extern  int             conf_level, no_slaves, no_runs;
extern  int             no_measures, no_sampling_points;
extern  double          eps_max; 
extern  meas_names_list *measure_names;
extern  int             StatSema;
extern  FILE            *sim_curves_file;
extern  char            model_name[FILE_NAME];
#ifdef CVR
extern  CvR_Stat        **all_cvr_measures;
#endif
#ifdef SPEC
extern  SpecStat        **all_spec_measures;
#endif



/* global variables */
static char             *tnethome;
static Backend_running = false;
static Start_Visualisation = false;
double step=0.0;
time_t *lastTime;
char *er_malloc ();
char *getenv();

/* void bedarf()
{
  long speicherkosten = 0;
  struct rusage ru;
 
  getrusage(RUSAGE_SELF,&ru);
  speicherkosten = ru.ru_maxrss * getpagesize();
  printf("\n Speicherbedarf: %ld",speicherkosten);
} */


/*************************************************************************

  function:     write_header_of_curves_file   

  parameters:   pointer to curves-file

  returns:      --

  purpose:      write the header of the curves-file

  author:       Henning Ziegler

  date:         November 1996

*************************************************************************/

void write_header_of_curves_file(curves_file, average_time) 
FILE *curves_file; 
double average_time;
{  
  int i,k;
#ifdef CVR
  CvR_Stat *Measure;
#endif
#ifdef SPEC
  SpecStat *Measure;
#endif

  /* header schreiben */
  fprintf(curves_file,"# file %s contains the simulation results of the stat. phase\n#\n",curves_file);

  fprintf(curves_file,"# number of x values: a\n");
  fprintf(curves_file,"%d\n", 100);

  fprintf(curves_file,"# number of defined reward measures: b\n");
  fprintf(curves_file,"%d\n", no_measures);

  fprintf(curves_file,"# list of reward measure names:\n");
  for( i = 0;i < no_measures; i++ )
#ifdef CVR
    fprintf(curves_file,"%s ", all_cvr_measures[i]->name);
#endif 
#ifdef SPEC
    fprintf(curves_file,"%s ", all_spec_measures[i]->name);
#endif 

  fprintf(curves_file,"\n");

  fprintf(curves_file,"# last x value:\n");
  fprintf(curves_file,"%f \n", average_time==0 ? 1 : average_time);

  fprintf(curves_file,"# maximum y value:\n");
  for( i = 0;i < no_measures; i++ )
    fprintf(curves_file,"%f ", 1.0);

  fprintf(curves_file,"\n");

  fprintf(curves_file,"# minimum y value:\n");
  for( i = 0; i < no_measures ; i++ )
    fprintf(curves_file,"%f ", 0.0 );

  fprintf(curves_file,"\n");

  fprintf(curves_file,"# in case of an experiment, number of runs: c\n");
  fprintf(curves_file,"%d\n", 1 );    /* wird vorerst nicht unterstuetzt! */

  fprintf(curves_file,"# results: represented as a x 2b+1 matrices\n");
  fflush(curves_file);
  lastTime=(time_t*)malloc(no_measures*sizeof(time_t));
  for(i=0;i<no_measures;i++) lastTime[i]=time(NULL);
}

/*************************************************************************

  function:     update_curves_file    

  parameters:   pointer to curves-file

  returns:      --

  purpose:      write data of all measures into curves-file for the next
		visualisation step 

  author:       Henning Ziegler

  date:         November 1996

*************************************************************************/
void update_curves_file(curves_file, measure_id, end_time)
FILE *curves_file;
int measure_id;
double end_time;
{
  int i,k;
#ifdef CVR
  CvR_Stat *Measure;
#endif
#ifdef SPEC
  SpecStat *Measure;
#endif
      
#ifdef CVR
	  Measure = all_cvr_measures[measure_id];
#endif
#ifdef SPEC
	  Measure = all_spec_measures[measure_id];
#endif
	  if( Measure->StopSendedToBackend == false )
	    {
              fprintf(curves_file,"%d ",measure_id);

              fprintf(curves_file,"%f ",end_time/no_slaves);
#ifdef CVR
	      fprintf(curves_file,"%.6f ",Measure->SampleCount * LOCAL_BATCH_SIZE_DEF*
                                          Measure->Least_Sq_Mean/end_time);

	      fprintf(curves_file,"%.6f ",Measure->SampleCount * LOCAL_BATCH_SIZE_DEF*
                                          Measure->DeltaX/end_time);
#endif
#ifdef SPEC
	      fprintf(curves_file,"%.6f ",Measure->SampleCount*LOCAL_BATCH_SIZE_DEF*
                                          Measure->MeanValue/end_time);

	      fprintf(curves_file,"%.6f ",Measure->SampleCount*LOCAL_BATCH_SIZE_DEF*
                                          Measure->DeltaX/end_time);
#endif
	      fprintf(curves_file,"\n");
	    


              for( i = 0; i < no_measures; i++ )
              {
#ifdef CVR
                Measure = all_cvr_measures[i];
#endif
#ifdef SPEC
                Measure = all_spec_measures[i];
#endif
                if( (Measure->StopSimulation == true)  &&
         	   (Measure->StopSendedToBackend == false) )
	        {
                  if(i!=measure_id)             /* wirklich letzten Wert darstellen ! */
                  {
                    fprintf(curves_file,"%d ",i);
                    fprintf(curves_file,"%f ",Measure->EndTime);
#ifdef CVR
	            fprintf(curves_file,"%.6f ",Measure->SampleCount * LOCAL_BATCH_SIZE_DEF*
                                                Measure->Least_Sq_Mean/Measure->EndTime);
	            fprintf(curves_file,"%.6f ",Measure->SampleCount * LOCAL_BATCH_SIZE_DEF*
                                                Measure->DeltaX/Measure->EndTime);
#endif
#ifdef SPEC
	            fprintf(curves_file,"%.6f ",Measure->SampleCount*LOCAL_BATCH_SIZE_DEF*
                                                Measure->MeanValue/Measure->EndTime);
	            fprintf(curves_file,"%.6f ",Measure->SampleCount*LOCAL_BATCH_SIZE_DEF*
                                                Measure->DeltaX/Measure->EndTime);
#endif
	            fprintf(curves_file,"\n");
                  }
	          fprintf(curves_file,"&%d Measure %s stopped\n",i,Measure->name);
	          Measure->StopSendedToBackend = true;
	        }
              }
	    }

  fflush(curves_file); 
}

/*************************************************************************

  function:     close_curves_file    

  parameters:   curve_file -> pointer to curves-file
		modus -> RESULT, if data should be written into curves-file  

  returns:      --

  purpose:      write simulation stop message into curves-file respectivly
		data

  author:       Henning Ziegler

  date:         November 1996

*************************************************************************/

void close_curves_file(curves_file)
FILE *curves_file;
{  
  fprintf(curves_file,"# Results of Simulation\n");
  fflush(curves_file);
  fclose(curves_file);
}


/*************************************************************************

  function:     send_conditions_reached    

  parameters:   --

  returns:      true: all measures have collected enough data for writing   
		      them into curves-file

  purpose:      check whether enough data have been collected for the next
		visualisation step

  author:       Henning Ziegler

  date:         November 1996

*************************************************************************/

bool send_conditions_reached(measure_id)
int measure_id;
{
double diffTime;
#ifdef CVR
  CvR_Stat *Measure;
#endif
#ifdef SPEC
  SpecStat *Measure;
#endif

#ifdef CVR
  Measure = all_cvr_measures[measure_id];
#endif
#ifdef SPEC
  Measure = all_spec_measures[measure_id];
#endif

  /*if( (Measure->SampleCount >= Measure->SendFactor) || (Measure->StopSimulation == true) )
  {
    /*  if (Measure->SendFactor=5000) Measure->SendFactor*=2;
  else Measure->SendFactor+=2000;*/

#if defined(sun) && !defined(__svr4__)
    diffTime = (double)(time(NULL)-lastTime[measure_id]);
#else
    diffTime = difftime(time(NULL),lastTime[measure_id]); 
#endif
#ifdef debug
    printf("Differenzzeit[%d]=%g\n",measure_id,diffTime);
#endif
  if(Measure->SendFactor==0)
#ifdef SPEC
   if (Measure->MeanValue!=0) Measure->SendFactor=1;
#elif CVR
   if (Measure->Least_Sq_Mean!=0) Measure->SendFactor=1;
#endif
   
  if(Measure->SendFactor)
   /*Die ersten X_FIRST Measures alle darstellen, danach jedes Measure hoechstens*/
   if ((Measure->SampleCount>X_FIRST_M &&        /* alle no_measures*DELAY/2 sec */
        (lastTime[measure_id] && diffTime>=DELAY*no_measures/2)) ||
       (Measure->SampleCount<=X_FIRST_M)
      )
    {
     lastTime[measure_id]=time(NULL);
     return true;
    }
  return false;
/*}
  else
    return false;*/
}

/*************************************************************************

  function:     visu_conditions_reached  

  parameters:   --  

  returns:      true: all measures have collected enough data to start backend

  purpose:      check whether enough datas have been collected to start backend
       
  author:       Henning Ziegler

  date:         September 1996

*************************************************************************/

bool visu_conditions_reached()
{
  int counter,i;
#ifdef CVR
  CvR_Stat *Measure;
#endif 
#ifdef SPEC
  SpecStat *Measure;
#endif

  for(i=0;i<no_measures;i++)
    {
#ifdef CVR
      Measure = all_cvr_measures[i];
#endif
#ifdef SPEC
      Measure = all_spec_measures[i];
#endif
      Measure->SendFactor=2;
      if((Measure->SampleCount >= Measure->SendFactor)  || (Measure->StopSimulation == true))
	counter++;
    }
    if( counter >= no_measures )
      return true;
    else
      return false;
}
/*************************************************************************

  function:     init

  parameters:   -- 

  returns:      --     

  purpose:      ..

  author:       Henning Ziegler

  date:         November 1996

*************************************************************************/

void init()
{
   int i;
#ifdef CVR
  CvR_Stat *Measure;
#endif 
#ifdef SPEC
  SpecStat *Measure;
#endif

  for(i=0;i<no_measures;i++)
    {
#ifdef CVR
      Measure = all_cvr_measures[i];
#endif
#ifdef SPEC
      Measure = all_spec_measures[i];
#endif
      Measure->SendFactor=0;
    }
}


/*************************************************************************

  function:     handle_visualisation 

  parameters:   -- 

  returns:      --     

  purpose:      if enough simulation results has been achieved the backend
		is started. If the backend is running data for the next
		visualisation step is written into the curves-file

  author:       Henning Ziegler

  date:         November 1996

*************************************************************************/

void handle_visualisation(measure_id, end_time)
 int measure_id;
 double end_time;
{
  char command[1000];
 
  if( Start_Visualisation == false )
  {
   Start_Visualisation=true;
   init();
  }    
   /* Start_Visualisation = visu_conditions_reached();*/

  if( (Backend_running == false) && (Start_Visualisation==true) )
  {
     write_header_of_curves_file(sim_curves_file, end_time/no_slaves);
     switch( fork() )
       {
       case -1: perror("Can't start Backend"); 
	        exit(1);
       case 0:
	      {
	        if(  !(tnethome = getenv("TNETHOME")) )
	          {
	            fprintf(stderr,"TNETHOME not set -> can't start Backend\n");
		    exit(-1);
	          }
		 sprintf(command,"%s/Simulation/bin/BackendSim %s.sim_curves -geometry 600x300+200+100",tnethome,model_name);
		 system(command);    
		 exit(0);
	       }
       }
       Backend_running = true;
   }
   if(Backend_running)
     {
	if( StatSema > 0 )
	 {
	   if(send_conditions_reached(measure_id))
	     update_curves_file(sim_curves_file, measure_id, end_time);
	  }
	else       /*Simulation ist beendet ! */
	  { 
	    update_curves_file(sim_curves_file, measure_id, end_time);
            close_curves_file(sim_curves_file); 
	  }
      }
}


