#include <stdio.h>
#include <stdlib.h>

#include "sim_slave_types.h"

#define INIT_SIZE 2

/*********** Externe Variablen ************************/
extern int no_trans;

/*********** Globale Variablen ************************/
static ttf *ttf_list;     /* Zeiger auf das erste Ereignis */
static ttf **tra_first;   /* Tabelle mit Zeigern auf das erste Ereignis
                             einer Transition */
ttf *pch;          /* Zeiger auf zu aendernde Ereignisse einer
		      Transition */
ttf *fir_tra;      /* Zeiger auf zuletzt schaltende Transition */

static ttf *ttf_array;    /* Feld mit ttf-Elementen fuer Speicerverwaltung */
static int ttf_array_size;
static ttf **ttf_freelist;
static int i_ttf_freelist;   /* Index des naechsten freien Elements */

ttf *saved_ttf_list[NO_SAVESTATES];  /* Zeiger auf erstes Ereignis der
				        gesicherten ttf-Liste */
ttf **saved_tra_first[NO_SAVESTATES];
ttf *saved_ttf_array[NO_SAVESTATES];
int saved_ttf_array_size[NO_SAVESTATES];
ttf **saved_ttf_freelist[NO_SAVESTATES];
int saved_i_ttf_freelist[NO_SAVESTATES];

/*************************************************************************
          memory management
**************************************************************************

  function:     init_ttf_array

  parameter:    -
  
  purpose:      initializes the array for the built-in memory management
 
  return:       -

  author:       Matthias Hoffmann

  date:         April 1995

*************************************************************************/
static void init_ttf_array()
{
  int i;
  char *er_malloc();

  ttf_array = (ttf *)er_malloc( INIT_SIZE * sizeof(ttf));
  ttf_array_size = INIT_SIZE;
  ttf_freelist = (ttf **)er_malloc( INIT_SIZE * sizeof(ttf *));
  for( i=0 ; i<INIT_SIZE ; i++ )
    ttf_freelist[i] = ttf_array + i;
  i_ttf_freelist = 0;
  for( i=0 ; i< NO_SAVESTATES ; i++)
  { saved_ttf_list[i] = NULL;
    saved_tra_first[i] = NULL;
    saved_ttf_array[i] = NULL;
    saved_ttf_array_size[i] = 0;
    saved_ttf_freelist[i] = NULL;
    saved_i_ttf_freelist[i] = 0;
  }
}  


/*************************************************************************

  function:     expand_ttf_array

  parameter:    -
  
  purpose:      expands the array for the built-in memory management
 
  return:       -

  author:       Matthias Hoffmann

  date:         April 1995

*************************************************************************/
static void expand_ttf_array()
{
  int i, newsize;
  ttf *new_array;
  ttf **new_freelist;
  char *er_malloc();

  newsize = ttf_array_size<<1;
  new_array = (ttf *)er_malloc( newsize * sizeof(ttf));
  memcpy( new_array, ttf_array, ttf_array_size * sizeof(ttf) );
  for( i=0 ; i<ttf_array_size ; i++ )
  { if( new_array[i].prev )
      new_array[i].prev = new_array + (new_array[i].prev - ttf_array);
    if( new_array[i].next )
      new_array[i].next = new_array + (new_array[i].next - ttf_array);
    if( new_array[i].t_prev )
      new_array[i].t_prev = new_array + (new_array[i].t_prev - ttf_array);
    if( new_array[i].t_next )
      new_array[i].t_next = new_array + (new_array[i].t_next - ttf_array);
  }
  for( i=0 ; i<no_trans; i++ )
    if( tra_first[i] )
      tra_first[i] = new_array + (tra_first[i] - ttf_array);
  ttf_list = new_array + (ttf_list - ttf_array);
  new_freelist = (ttf **)er_malloc( newsize * sizeof(ttf *));
  for( i=ttf_array_size ; i<newsize ; i++ )
    new_freelist[i] = new_array + i;
  ttf_array_size = newsize;
  free(ttf_array);
  free(ttf_freelist);
  ttf_array = new_array;
  ttf_freelist = new_freelist;
}

  
/*************************************************************************

  function:     free_ttf

  parameter:    pointer to ttf-element
  
  purpose:      adds a free element to the array for the built-in memory
                management
 
  return:       -

  author:       Matthias Hoffmann

  date:         April 1995

*************************************************************************/
static void free_ttf( p )
  ttf *p;
{
  ttf_freelist[--i_ttf_freelist] = p;
  p->tra_nr = -1;
}


/*************************************************************************

  function:     new_ttf

  parameter:    values for the new ttf-element
  
  purpose:      gets a free element from the array for the built-in memory
                management and fills it with the given values
 
  return:       pointer to ttf-element

  author:       Matthias Hoffmann

  date:         April 1995

*************************************************************************/
static ttf *new_ttf( tn, type, pr, sr, s, rd, tl )
  int tn; int type; int pr; double sr; double s; double rd; double tl;
{
  ttf *pt;
  
  if( i_ttf_freelist >= ttf_array_size )
    expand_ttf_array();
  pt = ttf_freelist[i_ttf_freelist++];
  pt->prev     = NULL;
  pt->next     = NULL;
  pt->t_prev   = NULL;
  pt->t_next   = NULL;
  pt->tra_nr       = tn;
  pt->ttf_type     = type;
  pt->prio         = pr;
  pt->samp_rnd      = sr;
  pt->sample       = s;
  pt->rem_delay    = rd;
  pt->time_in_list = tl;
  return pt;
}


/*************************************************************************
          other local functions
**************************************************************************

  function:     check_change

  parameter:    -
  
  purpose:      checks the completion of change-operations
 
  return:       -

  author:       Matthias Hoffmann

  date:         December 1994

*************************************************************************/
static void check_change()
{
  if( pch != NULL )
  { printf( "ERROR: transition %d not completely changed", pch->tra_nr );
    exit( FAIL );
  }
}


/*************************************************************************

  function:     cmp_ttf

  parameter:    pointers to ttf_elements
  
  purpose:      compares two ttf_elements for the min rem_delay
 
  return:        < 0 : p1 fires first
                == 0 : same firing delay
                 > 0 ; p2 fires first

  author:       Matthias Hoffmann

  date:         December 1994

*************************************************************************/
static int cmp_ttf( p1, p2 ) 
  ttf *p1; ttf *p2;
{
  if( p1 ) 
  { if( p1->prio > 0 )       /* p1 IMM */
      return( p2->prio - p1->prio );
    else if( p2->prio > 0 )  /* p2 IMM */
      return 1;               
    else if( p1->rem_delay < p2->rem_delay )
      return -1;             /* p1->rem_delay <  p2->rem_delay */
    else                     /* p1->rem_delay >= p2->rem_delay */
      return( p1->rem_delay > p2->rem_delay ); 
  }
  return 1;  /* p1 == NULL */
}


/*************************************************************************

  function:     t_insert

  parameter:    pointers to ttf_elements
  
  purpose:      insertion-sort of the new ttf_element *n behind *p1
                into the t_prev/t_next-list
 
  return:       n->t_prev

  author:       Matthias Hoffmann

  date:         January 1995

*************************************************************************/
static ttf *t_insert( n, p1 )
  ttf *n; ttf *p1;
{
  ttf *p2;

  if( p1 == NULL )                /* zur Sicherheit ( sollte immer so */
    p1 = tra_first[n->tra_nr];    /* uebergeben werden, dass p1 auf zeigt) */
  if( p1 == tra_first[n->tra_nr] )
  { if( p1 == NULL )              /* einzige ttf der Transition */
    { tra_first[n->tra_nr] = n;
      return NULL;
    }
    if( cmp_ttf( p1, n ) > 0 )    /* erste ttf der Transition */
    { n->t_next = p1;
      tra_first[n->tra_nr] = n;
      p1->t_prev = n;
      n->t_prev = NULL;
      return NULL;
    }
  }
  for( p2 = p1->t_next ; cmp_ttf( p2, n ) < 0 ; p1 = p2, p2 = p2->t_next );
  if( p2 == NULL)                 /* letzte ttf der Transition */
  { p1->t_next = n;
    n->t_prev  = p1;
    return p1;
  }
  else                       /* irgendwo in der t_prev/t_next-Verkettung */
  { p1->t_next = n;          /* zwischen p1 und p2 einfuegen */
    p2->t_prev = n;
    n->t_prev  = p1;
    n->t_next  = p2;
    return p1;
  }
}


/*************************************************************************

  function:     list_insert

  parameter:    pointers to ttf_elements
  
  purpose:      insertion-sort of the new ttf_element *n behind *p1
                into the prev/next-list
 
  return:       n->next

  author:       Matthias Hoffmann

  date:         December 1994

*************************************************************************/
static ttf *list_insert( n, p1 )
  ttf *n; ttf *p1;
{
  ttf *p2;

  if( p1 == NULL )                 /* wenn erste ttf der Transition, */
    p1 = ttf_list;                 /* dann ab Listenanfang einfuegen */
  if( p1 == ttf_list ) 
  { if( p1 == NULL )                /* einzige ttf in der ttf_list */
    { ttf_list = n;  
      return NULL;
    }
    if( cmp_ttf( p1, n ) > 0 )      /* erste ttf in der ttf_list */
    { ttf_list = n;
      n->next = p1;
      p1->prev = n;
      return p1;
    }
  }
  for( p2 = p1->next ; cmp_ttf( p2, n ) <= 0 ; p1 = p2, p2 = p2->next );
  if( p2 == NULL)                 /* letzte ttf in der ttf_list */
  { p1->next = n;
    n->prev  = p1;
    return NULL;
  }
  else                            /* irgendwo in der ttf_list */
  { p1->next = n;
    p2->prev = n;
    n->prev  = p1;
    n->next  = p2;
    return p2;
  }
}


/*************************************************************************

  function:     t_delete

  parameter:    pointer to ttf_element
  
  purpose:      deletes the ttf_element n in the t_prev/t_next-list
 
  return:       -

  author:       Matthias Hoffmann

  date:         January 1995

*************************************************************************/
static void t_delete( n )
  ttf *n;
{
  if( n == tra_first[n->tra_nr] )            /* erste ttf der Transition */
  { tra_first[n->tra_nr] = n->t_next;
    if( n->t_next )                   /* noch weitere ttf der Transition */
      n->t_next->t_prev = NULL;
  }
  else
  { n->t_prev->t_next = n->t_next;
    if( n->t_next )                   /* noch weitere ttf der Transition */
      n->t_next->t_prev = n->t_prev;
  }
  n->t_prev = n->t_next = NULL;            /* zur Sicherheit  */
}


/*************************************************************************

  function:     list_delete

  parameter:    pointer to ttf_element
  
  purpose:      deletes the ttf_element *n in the prev/next-list
 
  return:       -

  author:       Matthias Hoffmann

  date:         December 1994

*************************************************************************/
/* Entfernen der ttf n aus der prev/next-Verkettung  */
static void list_delete( n )
  ttf *n;
{
  if( n == ttf_list )            /* erste ttf in der ttf_list */             
  { ttf_list = n->next;
    if( ttf_list != NULL )       /* ttf_list nicht leer */
      ttf_list->prev = NULL;
  }
  else
  { n->prev->next = n->next;
    if( n->next != NULL )        /* nicht die letzte ttf */
      n->next->prev = n->prev;
  }
  n->prev = n->next = NULL;               /* zur Sicherheit */
}


/*************************************************************************

  function:     pass

  parameter:    passed time
  
  purpose:      updates the entries rem_delay and time_in_list for all
                ttf_elements
 
  return:       -

  author:       Matthias Hoffmann

  date:         December 1994

*************************************************************************/
static void pass( t )
  double t;
{
  int i;
  ttf *p;

  for( p=ttf_list ; p ; p=p->next )
  { if( p->ttf_type ==  E_IMM )
    { printf( "ERROR: time passes while IMM-transition enabled!\n" );
      exit( FAIL );
    }
    p->rem_delay -= t;
  }
  for( i=0 ; i<no_trans ; i++ )
    for( p=tra_first[i] ; p ; p=p->t_next )
      p->time_in_list += t;
}

/*************************************************************************
              exported funktions
**************************************************************************

  function:     init_ttf

  parameter:    -
  
  purpose:      initializes the tra-first-array and the
                memory management
 
  return:       -

  author:       Matthias Hoffmann

  date:         april 1995

*************************************************************************/
void init_ttf()
{
  int i;
  char *er_malloc();

  ttf_list = NULL;
  tra_first = (ttf **)er_malloc( no_trans*sizeof(ttf *));
  for( i=0; i<no_trans; i++ )
      tra_first[i]=NULL;
  pch = NULL;
  init_ttf_array();
}  


/*************************************************************************

  function:     add_imm_ttf

  parameter:    values for the new ttf_element
  
  purpose:      adds a ttf_element for an immediate transition
 
  return:       -

  author:       Matthias Hoffmann

  date:         december 1994

*************************************************************************/
void add_imm_ttf( tra_nr, prio, samp_rnd, sample )
  int tra_nr; int prio; double samp_rnd; double sample;
{
  ttf *n, *p;

  check_change();
  n = new_ttf( tra_nr, E_IMM, prio, -1.0, sample, 0.0, 0.0 );
  p = t_insert( n, tra_first[tra_nr] );
  list_insert( n, p );
}


/*************************************************************************

  function:     add_tim_ttf

  parameter:    values for the new ttf_element
  
  purpose:      adds a ttf_element for a timed transition
 
  return:       -

  author:       Matthias Hoffmann

  date:         december 1994

*************************************************************************/
void add_tim_ttf( tra_nr, prio, samp_rnd, sample )
  int tra_nr; int prio; double samp_rnd; double sample;
{
  ttf *n, *p;
  
  check_change();
  n = new_ttf( tra_nr, E_TIM, 0, samp_rnd, sample, sample, 0.0 );
  p = t_insert( n, tra_first[tra_nr] );
  list_insert( n, p );
}


/*************************************************************************

  function:     add_rag_ttf

  parameter:    values for the new ttf_element
  
  purpose:      adds a ttf_element for a timed transition with
                race policy "race age"
 
  return:       -

  author:       Matthias Hoffmann

  date:         december 1994

*************************************************************************/
void add_rag_ttf( tra_nr, prio, samp_rnd, sample )
  int tra_nr; int prio; double samp_rnd; double sample;
{
  ttf *p, *n;
  
  check_change();
  if( tra_first[tra_nr] != NULL )    /* wieder einschalten und einhaengen */
    list_insert( tra_first[tra_nr], ttf_list );
  else
  { n = new_ttf( tra_nr, E_RAG, 0, samp_rnd, sample, sample, 0.0 );
    p = t_insert( n, NULL );
    list_insert( n, p );
  }
}


/*************************************************************************

  function:     del_ttf

  parameter:    transition number, number of ttf in list
  
  purpose:      deletes a ttf_element
 
  return:       sample of the deleted ttf_element

  author:       Matthias Hoffmann

  date:         december 1994

*************************************************************************/
void del_ttf( tra_nr, n_list, sample )
  int tra_nr; int n_list; double *sample; 
{
  ttf *p;
  double d, rnd_uniform();

  check_change();
  d = rnd_uniform( 0.0, (double)n_list );
  for( p = tra_first[tra_nr], d-=1.0 ; d > 0 ; d-=1.0, p=p->t_next );
  t_delete( p );
  list_delete( p );
  *sample = p->sample;
  free_ttf( p );
}


/*************************************************************************

  function:     del_fir_tra

  parameter:    -
  
  purpose:      deletes the ttf_element that has fired last
 
  return:       -

  author:       Matthias Hoffmann

  date:         march 1995

*************************************************************************/
void del_fir_tra()
{
  check_change();
  t_delete( fir_tra );
  list_delete( fir_tra );
  free_ttf( fir_tra );
}


/*************************************************************************

  function:     off_rag_ttf

  parameter:    transition number, (dummy) number of ttf in list
  
  purpose:      deletes a ttf_element with race policy "race age" in
                the prev_next-list. the element itself isn't deleted
 
  return:       sample

  author:       Matthias Hoffmann

  date:         march 1995

*************************************************************************/
void off_rag_ttf( tra_nr, n_list, sample )
  int tra_nr; int n_list; double *sample; 
{
  check_change();
  list_delete( tra_first[tra_nr] );
  *sample = tra_first[tra_nr]->sample;
}


/*************************************************************************

  function:     start_changing

  parameter:    transition number
  
  purpose:      prepare for changing of all ttf_elements of transition
                tra_nr
  
  return:       -

  author:       Matthias Hoffmann

  date:         march 1995

*************************************************************************/
void start_changing( tra_nr )
  int tra_nr;
{
  check_change();
  pch = tra_first[tra_nr];
  tra_first[tra_nr] = NULL;
}


/*************************************************************************

  function:     get_samp_rnd

  parameter:    -
  
  purpose:      get the samp_rnd of the next ttf, that must be changed
  
  return:       -

  author:       Matthias Hoffmann

  date:         march 1995

*************************************************************************/
double get_samp_rnd()
{
  return( pch->samp_rnd );
}


/*************************************************************************

  function:     change_tim_ttf

  parameter:    transition_number, samp_rnd, new sample
  
  purpose:      changes one ttf_element of a timed transition
  
  return:       -

  author:       Matthias Hoffmann

  date:         march 1995

*************************************************************************/
void change_tim_ttf( tra_nr, samp_rnd, sample )
  int tra_nr; double samp_rnd; double sample;
{
  double rem_d;
  ttf    *n, *p;

  rem_d = sample * pch->rem_delay / pch->sample;
  pch->samp_rnd = samp_rnd;
  pch->sample = sample;
  pch->rem_delay = rem_d;
  n = pch;
  pch = pch->t_next;
  if( pch )
    pch->t_prev = NULL;
  n->t_next = NULL;
  list_delete( n );
  p = t_insert( n, NULL );
  list_insert( n, p );
}


/*************************************************************************

  function:     replace_imm_ttf

  parameter:    transition_number, (dummy) samp_rnd, new sample
  
  purpose:      replace the sample of one ttf_element of an 
                immediate transition
  
  return:       -

  author:       Matthias Hoffmann

  date:         march 1995

*************************************************************************/
void replace_imm_ttf( tra_nr, samp_rnd, sample )
  int tra_nr; double samp_rnd; double sample;
{
  ttf  *p, *n;

  pch->samp_rnd = samp_rnd;
  pch->sample = sample;
  n = pch;
  pch = pch->t_next;
  if( pch )
    pch->t_prev = NULL;
  n->t_next = NULL;
  list_delete( n );
  p = t_insert( n, NULL );
  list_insert( n, p );
}


/*************************************************************************

  function:     replace_RS_ttf

  parameter:    transition_number, samp_rnd, new sample
  
  purpose:      replace the sample of one ttf_element of a timed
                transition with race policy "race resampling"
  
  return:       -

  author:       Matthias Hoffmann

  date:         march 1995

*************************************************************************/
void replace_RS_ttf( tra_nr, samp_rnd, sample )
  int tra_nr; double samp_rnd; double sample;
{
  ttf  *p, *n;

  pch->samp_rnd = samp_rnd;
  pch->sample = sample;
  pch->rem_delay = sample;
  pch->time_in_list = 0.0;
  n = pch;
  pch = pch->t_next;
  if( pch )
    pch->t_prev = NULL;
  n->t_next = NULL;
  list_delete( n );
  p = t_insert( n, NULL );
  list_insert( n, p );
}


/*************************************************************************

  function:     trans_to_fire

  parameter:    -
  
  purpose:      selects the ttf_element that will fire next
  
  return:       tra_nr, sample, time_in_list, passed_time

  author:       Matthias Hoffmann

  date:         march 1995

*************************************************************************/
int trans_to_fire( sample, til, passed_time )
  double *sample; double *til; double *passed_time;
{
  ttf *p;
  double sum, d, pa, rnd_uniform();
  int tra_nr;

  if( !ttf_list )
  { printf( "ERROR: DEAD MARKING reached, no transition enabled\n" );
    exit( FAIL );
  }
  p = ttf_list;
  if( p->ttf_type == E_IMM )
  { for( sum=p->sample ; p->next, cmp_ttf( p->next, p )==0 ;
         sum+=p->next->sample, p=p->next );
    d = rnd_uniform( 0.0, sum );
    for( p = ttf_list, d-=p->sample ; d > 0 ; p=p->next, d-=p->sample );
    *sample = p->sample;
    *passed_time = *til = 0;
    pa=0;
  }
  else  /* zeitbehaftet */
  { for( sum=1.0 ; p->next, cmp_ttf( p->next, p )==0 ; sum+=1.0, p=p->next );
    d = rnd_uniform( 0.0, sum );
    for( p = ttf_list, d-=1.0 ; d > 0 ; p=p->next, d-=1.0 );
    *sample = p->sample;
    *passed_time = p->rem_delay;
    *til   = p->time_in_list + *passed_time;
    pass( *passed_time );
  }
  tra_nr = p->tra_nr;
  fir_tra = p;
  return tra_nr;
}


/*************************************************************************

  function:     get_CVsample

  parameter:    transition number
  
  purpose:      get the sample of a transition used as control variate
  
  return:       sample

  author:       Matthias Hoffmann

  date:         february 1995

*************************************************************************/
double get_CVsample( tra_nr )
  int tra_nr;
{
  if( tra_first[tra_nr]->t_next )
  { printf( "ERROR: CV %d is infinite server", tra_nr );
    exit( FAIL );
  }
  return( tra_first[tra_nr]->sample );
}


/*************************************************************************

  function:     save_list

  parameter:    index of saved state
  
  purpose:      save the state of the simulator lists
  
  return:       -

  author:       Matthias Hoffmann

  date:         may 1995

*************************************************************************/
void save_list( nr )
  int nr;
{
  int i;
  char *er_malloc();

  if(saved_ttf_array[nr])
    free(saved_ttf_array[nr]);
  if(saved_tra_first[nr])
    free(saved_tra_first[nr]);
  if(saved_ttf_freelist[nr])
    free(saved_ttf_freelist[nr]);
  saved_ttf_array[nr] = (ttf *)er_malloc( ttf_array_size * sizeof(ttf));
  memcpy( saved_ttf_array[nr], ttf_array, ttf_array_size * sizeof(ttf) );
  for( i=0 ; i<ttf_array_size ; i++ )
  { if( saved_ttf_array[nr][i].prev )
      saved_ttf_array[nr][i].prev = saved_ttf_array[nr] 
                                + (saved_ttf_array[nr][i].prev - ttf_array);
    if( saved_ttf_array[nr][i].next )
      saved_ttf_array[nr][i].next = saved_ttf_array[nr] 
                                + (saved_ttf_array[nr][i].next - ttf_array);
    if( saved_ttf_array[nr][i].t_prev )
      saved_ttf_array[nr][i].t_prev = saved_ttf_array[nr]
                                + (saved_ttf_array[nr][i].t_prev - ttf_array);
    if( saved_ttf_array[nr][i].t_next )
      saved_ttf_array[nr][i].t_next = saved_ttf_array[nr] 
                                + (saved_ttf_array[nr][i].t_next - ttf_array);
  }
  saved_ttf_array_size[nr] = ttf_array_size;
  saved_tra_first[nr] = (ttf **)er_malloc( no_trans * sizeof(ttf *));
  saved_ttf_list[nr] = saved_ttf_array[nr] + (ttf_list - ttf_array);
  for( i=0 ; i<no_trans; i++ )
    if( tra_first[i] )  
      saved_tra_first[nr][i] = saved_ttf_array[nr] 
                             + (tra_first[i] - ttf_array);
    else
      saved_tra_first[nr][i] = NULL;
  saved_ttf_freelist[nr] = (ttf **)er_malloc( ttf_array_size * sizeof(ttf *));
  for( i=i_ttf_freelist ; i<ttf_array_size ; i++ )
    saved_ttf_freelist[nr][i] = saved_ttf_array[nr]
                             + (ttf_freelist[i] - ttf_array);
  saved_i_ttf_freelist[nr] = i_ttf_freelist;
}


/*************************************************************************

  function:     load_list

  parameter:    index of saved state
  
  purpose:      load a saved state of the simulator lists, the saved list
                will not be destroyed
  
  return:       -

  author:       Matthias Hoffmann

  date:         may 1995

*************************************************************************/
void load_list( nr )
  int nr;
{
  free(ttf_array);
  free(ttf_freelist);
  free(tra_first);
  ttf_list       = saved_ttf_list[nr];
  tra_first      = saved_tra_first[nr];
  ttf_array      = saved_ttf_array[nr];
  ttf_array_size = saved_ttf_array_size[nr];
  ttf_freelist   = saved_ttf_freelist[nr];
  i_ttf_freelist = saved_i_ttf_freelist[nr];
  saved_ttf_array[nr]    = NULL;
  saved_ttf_freelist[nr] = NULL;
  saved_tra_first[nr]    = NULL;
  save_list( nr );
}

/*************************************************************************

  function:     free_sim_list_pointer

  parameter:    -
  
  purpose:      free allocated memory for global pointers

  author:       Ralf Putz

  date:         Dezember 1995

*************************************************************************/
void free_sim_list_pointer()
{
  int i;

  free(tra_first); 
  free(ttf_array);    
  free(ttf_freelist);
  for(i =  0; i  < NO_SAVESTATES; i++)
    {
      if(saved_ttf_list[i] != NULL)
       free(saved_ttf_list[i]);
      if(saved_tra_first[i] != NULL)
       free(saved_tra_first[i]);
      if(saved_ttf_array[i] != NULL)
       free(saved_ttf_array[i]);
      if(saved_ttf_freelist[i] != NULL)
       free(saved_ttf_freelist[i]);
    }
  if(pch != NULL) 
    free(pch); 
}








