/*NEUE_VERSION (BM)*/
/******************************************************************************

  filename :	sim_slave.c

  Parameter:    argv[1]: Host, on which the exec_rands-Process
			 is running
		argv[2]: socket portnumber of exec_rands-Process
		argv[3]: Max Modellzeit fuer Simulation (0.0=kein Limit)

  purpose  :	performs one of multiple distrubuted simulations of DSPN-
		models in DSPNexpress description format

  author   :    Christian Kelling
		Technical University Berlin
		Computer Science Department

  date     :	January 1994

  exported functions
	void	update_measure
	void	record_cv

  local functions:
	void	init
	void	read_infos
	void	read_transitions
	void	read_places
	void	read_commands
	void	send_samples
	void	send_trans_length
	void	Init_TransDetection

  Datei im Ramen des PDV-Projekts im WS 95/96 gendert. (BM)

  nderung:	Die Einzelwerte der Leistungsmae werden in Gruppen zusammen-
		gefat und an den Master-Proze gesendet, wobei die Gruppen-
		gre an die Aufzeichnungshufigkeit der Mae angepat wird.
		Eine Funktion, die die adaptive Gruppengre ermittelt,
		ergnzt die Sende-Prozedur "update_measure".

  Autoren:	Benjamin Hohnhuser und Mathias Neumann

  Datum:   	April 1996

  lokale Funktion:
                int	GetBlockSize (StatInfo**,int,int,int)

******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <errno.h>
#include <stdlib.h>
#include <math.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/resource.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <limits.h>			/*fr INT_MAX in GetBlockSize (BM)*/

#define BSD_COMP
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <netdb.h>

#include "../Master/stat_utils.h"
#include "sim_slave_types.h"
#include "det_init_trans.h"
#include "portab.h"

/* global variables used by the simulation slave*/
StatInfo	**all_measures;
double 	current_time, transient_time;
double 	maxtime;
int 	*zero_counters;
int	StatSema;
int	conf_level;
bool    trans_detection;
int     min_firings;
char	*er_malloc();
int     Slave_PID = -1;

/*****************************/
/* variables used by RESTART */
/*****************************/

#ifdef RST

/*#define RESTART_BATCH_SIZE_DEF 1000*/   /* now in sim_slave_types.h */
int direction=0,
      event=-1,
      border=-1,
      no_thresholds=-1,
      no_opt_thresholds=-1,
      *t_value,
      *retrials,
      Satz=1,
      Satz_changed=0;                   
double *prob_opt;
double Prob_rare_event=0.0;
char measure_place[100];
long rare_event_count=0;
double *first_k_trial_time;
double *current_k_time;
double *sum_k;
int ns=0;

double *prob_rare_event;

#endif

/*****************************/

int     no_places, no_trans, no_measures, no_md_arcs,my_slave_id, initial_seed;
int     no_sampling_points;

int maxsamples = MAX_SAMPLES_DEF;  

place	*places;
place   *init_places; 
transition  *transitions;

/* end of global variables */

#define OUT_OF_MEMORY { \
  perror (NULL); \
  fprintf (stderr, "ERROR in %s at line %u: out of memory !\n", \
           __FILE__, __LINE__); \
  my_exit (1); \
		      }

#define SOCKET_READ_ERROR { \
  perror (NULL); \
  fprintf (stderr, "ERROR in %s at line %u: can't read from socket !\n", \
           __FILE__, __LINE__); \
  my_exit (1); \
			  }

#define FILE_WRITE_ERROR(filename) { \
  perror (NULL); \
  fprintf (stderr, "ERROR in %s at line %u: can't write to file '%s' !\n", \
           __FILE__, __LINE__, filename); \
  my_exit (1); \
				   }


/*******************************/
/*** Funktionsdeklarationen: ***/
/*******************************/

unsigned read_info ();
void read_commands ();

/******************/
/*** Variablen: ***/
/******************/

static int socket_fd;/* Filedescriptor fuer die Socket-Verbindung zum Server */
static FILE *socket_fp;  /* Filepointer fuer die Socket */

static time_t starttime;
static int seed_step;

/*** Eigene exit-Routine: ***/
void my_exit (exit_code)
int exit_code;
{ 
  write (socket_fd, "slave exit.\n",12);
  exit (exit_code);
}


/********************************************/
/*** Routinen zur Behandlung von Fehlern: ***/
/********************************************/
/* Da sim_slave auf anderen Rechnern gestartet werden kann,
   wird durch diese signal-handler ein evtl. auftretender
   Fehler ueber die Socket an den sim_master-Prozess gesendet. */
static void sigill_handler ()
{
  fprintf (stderr, "ERROR in %s: Illegal Instruction\n", __FILE__);
  my_exit (1);
}
static void sigfpe_handler ()
{
  fprintf (stderr, "ERROR in %s: Floatingpoint exception\n", __FILE__);
  my_exit (1);
}
static void sigbus_handler ()
{
  fprintf (stderr, "ERROR in %s: Bus error\n", __FILE__);
  my_exit (1);
}
static void sigsegv_handler ()
{
  fprintf (stderr, "ERROR in %s: Segmentation fault\n", __FILE__);
  my_exit (1);
}
static void sigpipe_handler ()
{
  /* Socketverbindung zum Server futsch, also kein my_exit() */
  exit (1);
}


/*************************************************************************

  function:	er_malloc

  parameters:	number of bytes

  purpose:	allocation of memory, exit the program if memory is 
		unsufficient

  return:	pointer to the allocated memory block

  author:	Christian Kelling
		Technical University of Berlin

  date:		May 1993

*************************************************************************/

char *er_malloc (bytes)
     int bytes;
{
  char *pointer;
  if (!bytes)
    return (NULL);
  pointer = (char *)malloc(bytes);
  if (pointer == NULL)
    {
      printf ("Not enough memory for %d bytes\n", bytes);
      exit (FAIL);
    }
  return pointer;
}

/*************************************************************************

  function:	free_Seq

  parameters:	sequence -> pointer to a sample sequnce

  purpose:	free allocated memory of the linked list SampleSequence of a
                reward measure statistic

  author:	Christian Kelling
		Technical University of Berlin

  date:		May 1993

*************************************************************************/

static void free_Seq(sequence)
SampSeq *sequence;
{
  SampSeq *temp;
  
  if( sequence != NULL)
  {
    while (sequence->Next != NULL)
      {
        temp = sequence;
        sequence = sequence->Next;
        free(temp);
      }
      free(sequence);
  } 
} /* free_Seq */


/******************************/
/*** Funktionsdefinitionen: ***/
/******************************/
/******************************************************************************

 Funktion:      init

 Parameter:     argv aus main()

 Rueckgabewert: ---

 Zweck:         - Aufbauen der Verbindung zum Server-Prozess 'sim_master'
                - Umlenken von stdout und stderr in die Socket
		- Initialisieren der max. Modellzeit

******************************************************************************/
static void init (argv)
char *argv[];
{
  struct sockaddr_in server;
  double atof();
  register struct hostent *hp;
  unsigned tries;
  char *outbuf, *errbuf;

  /* Verbindung zum Server (sim_master.c) aufnehmen: */
  if ((socket_fd = socket (AF_INET, SOCK_STREAM, 0)) < 0)
    exit (1);
  if (!(hp = gethostbyname (argv[1])))
    exit (1);

  memcpy (&server.sin_addr,hp->h_addr, hp->h_length);
  server.sin_family = AF_INET;
  server.sin_port = htons (atoi (argv[2]));
  for (tries = 1; tries <= 60; ++tries)
    if (connect (socket_fd, (struct sockaddr *)&server, sizeof (server)) != -1)
      break;
    else
      sleep (3);
  if (tries > 60)
  exit (1);

  /* Einen neuen Filepointer fuer die Socket erstellen: */
  if (!(socket_fp = fdopen (socket_fd, "r+")))
    exit (1);

  /* Standardausgabe und Standardfehler in die Socket umlenken: */
  if (dup2 (socket_fd, 1) < 0)
    exit (1);
  if (dup2 (socket_fd, 2) < 0)
    exit (1);
  
  /* Ausgabe-Zeilen buffern: */
  outbuf = (char *) er_malloc(BUFSIZ);
  errbuf = (char *) er_malloc(BUFSIZ);
  setvbuf(stdout,outbuf,_IOLBF,BUFSIZ);
  setvbuf(stderr,errbuf,_IOLBF,BUFSIZ);

  /* Eigene Fehlerbehandlungsroutinen anmelden: */
  signal (SIGILL, sigill_handler);
  signal (SIGFPE, SIG_IGN);
  signal (SIGBUS, sigbus_handler);
  signal (SIGSEGV, sigsegv_handler);
  signal (SIGPIPE, sigpipe_handler);

  maxtime = atof(argv[3]);
  
}


/******************************************************************************

 Funktion:      my_read

 Parameter:     data:   Zeiger auf den Bereich, wo die gelesenen Daten
 			abgelegt werden
                length: Anzahl der zu lesenden Bytes
                fd: 	Filedesciptor (der Socket zum master)

 Rueckgabewert: ---

 Zweck:         Empfangen von Datenpaketen (ueber eine Socket)

******************************************************************************/
static void my_read (data, length, fd)
char *data;
unsigned length, fd;
{
  int rt;

 nochmal:
  errno = 0;
  if ((rt = read (fd, data, length)) != length)
    {
    if (rt >= 0)
      {
	data += rt;
	length -= rt;
	goto nochmal;
      }
    perror (__FILE__);
    fprintf (stderr, "ERROR in %s: can't read from socket!\n", __FILE__, rt, length);
    exit (1);
  }
}

/*************************************************************************

  function:	send_samples

  parameter:	mesaure identifier, sample sequence and its dimension
  
  purpose:	sends a sequence of samples for a particular measure to the 
		master

  author:	Christian Kelling
		Technical University of Berlin

  date:		January 1994

*************************************************************************/

void send_samples(meas_id, samples, size, min_flag
#ifdef CVR
		  ,cvs, cv_size
#endif
		  )
     int meas_id, size;
     bool min_flag;
#ifdef STR
     str_sample_type *samples;
#else
     sample_type *samples;
#endif
#ifdef CVR
     int cv_size;
     double *cvs;
#endif
{
  write (socket_fd, "SAMPLES\n", 8);
  write (socket_fd, &my_slave_id, sizeof(int));
  write (socket_fd, &min_flag, sizeof(bool));
  write (socket_fd, &meas_id, sizeof(int));
  write (socket_fd, &size, sizeof(int));
#ifdef STR
  write (socket_fd, samples, size*sizeof(str_sample_type));
#else
  write (socket_fd, samples, size*sizeof(sample_type));
#endif
#ifdef CVR
  write (socket_fd, &cv_size, sizeof(int));
  write (socket_fd, cvs, cv_size*sizeof(double));
#endif
}


/*************************************************************************

  function:	send_trans_length

  parameter:	mesaure identifier, length of initial transient phase
  
  purpose:	sends the length of initial transient phase to the master
  		for further statistical procedures

  author:	Christian Kelling
		Technical University of Berlin

  date:		January 1994

*************************************************************************/

void send_trans_length(meas_id, length, trans_time)
     int meas_id, length;
     double trans_time;
{
  write (socket_fd, "LENGTH\n", 7);
  write (socket_fd, &my_slave_id, sizeof(int));
  write (socket_fd, &meas_id, sizeof(int));
  write (socket_fd, &length, sizeof(int));
  write (socket_fd, &trans_time, sizeof(double));
}


/*************************************************************************

  function:	read_infos

  parameter:	--
  
  purpose:	reads dimension of the net and number of measures from the 
		socket connection to the master

  author:	Christian Kelling
		Technical University of Berlin

  date:		January 1994

*************************************************************************/
static unsigned read_infos (){

#ifdef RST

int name_length,i; 

#endif



  if (fread (&my_slave_id, sizeof (int), 1, socket_fp) != 1)
    SOCKET_READ_ERROR;

  if (fread (&no_trans, sizeof (int), 1, socket_fp) != 1)
    SOCKET_READ_ERROR;

  if (fread (&no_places, sizeof (int), 1, socket_fp) != 1)
    SOCKET_READ_ERROR;

  if (fread (&no_measures, sizeof (int), 1, socket_fp) != 1)
    SOCKET_READ_ERROR;

  if (fread (&no_md_arcs, sizeof (int), 1, socket_fp) != 1)
    SOCKET_READ_ERROR;

  if (fread (&initial_seed, sizeof (int), 1, socket_fp) != 1)
    SOCKET_READ_ERROR;

  if (fread (&conf_level, sizeof (int), 1, socket_fp) != 1)
    SOCKET_READ_ERROR;

  if (fread (&trans_detection, sizeof (int), 1, socket_fp) != 1)
    SOCKET_READ_ERROR;
  
  if  (fread (&min_firings, sizeof (int), 1, socket_fp) != 1)
    SOCKET_READ_ERROR;
  

#ifdef RST
  if (fread (&name_length, sizeof (int), 1, socket_fp) != 1)
    SOCKET_READ_ERROR;

  if (fread (measure_place, name_length, 1, socket_fp) != 1)
    SOCKET_READ_ERROR;

  if (fread (&direction, sizeof (int), 1, socket_fp) != 1)
    SOCKET_READ_ERROR;

  if (fread (&event, sizeof (int), 1, socket_fp) != 1)
    SOCKET_READ_ERROR;

  if (fread (&border, sizeof (int), 1, socket_fp) != 1)
    SOCKET_READ_ERROR;

  if (fread (&no_thresholds, sizeof (int), 1, socket_fp) != 1)
    SOCKET_READ_ERROR;
  if (fread (&no_opt_thresholds, sizeof (int), 1, socket_fp) != 1)
    SOCKET_READ_ERROR;
  for (i=1;i<=no_thresholds;++i)
      {
      if (fread (&t_value[i], sizeof (int), 1, socket_fp) != 1)
        SOCKET_READ_ERROR;
      if (fread (&retrials[i], sizeof (int), 1, socket_fp) != 1)
        SOCKET_READ_ERROR;
      }
  prob_opt=(double*)er_malloc((no_opt_thresholds+1)*sizeof(double));
  for (i=1;i<=no_opt_thresholds;++i) 
     if (fread (&prob_opt[i], sizeof (double ), 1, socket_fp) != 1)
        SOCKET_READ_ERROR;    
#endif

#ifdef STR
  if (fread (&transient_time, sizeof (double), 1, socket_fp) != 1)
    SOCKET_READ_ERROR;

 if (fread (&seed_step, sizeof (int), 1, socket_fp) != 1)
    SOCKET_READ_ERROR;

 if (fread (&no_sampling_points, sizeof (int), 1, socket_fp) != 1)
    SOCKET_READ_ERROR;

  if (fread (&StatSema, sizeof (int), 1, socket_fp) != 1)
    SOCKET_READ_ERROR; 

#endif
}


/*************************************************************************

  function:	read_transitions

  parameter:	--
  
  purpose:	read the array of transitions from the master

  author:	Christian Kelling
		Technical University of Berlin

  date:		January 1994

*************************************************************************/

void read_transitions ()
{
  int i, j, name_length;
  cv_list_elem *cv_node;

  transitions = (transition *)er_malloc( no_trans * sizeof(transition));
  for (i=0; i<no_trans; i++)
    {
      if (fread (&j, sizeof(int), 1, socket_fp) != 1)
	SOCKET_READ_ERROR;
      if (i != j)
	{
	  printf("wrong transition number\n");
	  printf("expected %d received %d\n", i, j);
	  my_exit(FAIL);
	}
      if (fread (&transitions[i].type, sizeof(int), 1, socket_fp) != 1)
	SOCKET_READ_ERROR;
      if (fread (&transitions[i].serv_type, sizeof(int), 1, socket_fp) != 1)
	SOCKET_READ_ERROR;
      if (fread (&transitions[i].fire_policy, sizeof(int), 1, socket_fp) != 1)
        SOCKET_READ_ERROR;
      if (fread (&transitions[i].prio, sizeof(int), 1, socket_fp) != 1)
	SOCKET_READ_ERROR;
      if (fread (&transitions[i].ena_id, sizeof(int), 1, socket_fp) != 1)
	SOCKET_READ_ERROR;
      if (fread (&transitions[i].mark_dep_id, sizeof(int), 1, socket_fp) != 1)
	SOCKET_READ_ERROR;
      if (fread (&transitions[i].no_outputs, sizeof(int), 1, socket_fp) != 1)
	SOCKET_READ_ERROR;
      if (fread (&transitions[i].no_inputs, sizeof(int), 1, socket_fp) != 1)
	SOCKET_READ_ERROR;
      if (fread (&transitions[i].no_inh, sizeof(int), 1, socket_fp) != 1)
	SOCKET_READ_ERROR;
      if (fread (&transitions[i].no_aff, sizeof(int), 1, socket_fp) != 1)
        SOCKET_READ_ERROR;
#ifdef CVR
      if (fread (&transitions[i].no_cv_meas, sizeof(int), 1, socket_fp) != 1)
	SOCKET_READ_ERROR;
#endif
      if (fread (&transitions[i].delay, sizeof(double), 1, socket_fp) != 1)
	SOCKET_READ_ERROR;
      if (fread (&name_length, sizeof(int), 1, socket_fp) != 1)
	SOCKET_READ_ERROR;
      if (fread (transitions[i].name, name_length, 1, socket_fp) != 1)
	SOCKET_READ_ERROR;

      if (transitions[i].no_outputs)
        transitions[i].out_to_place =
          (conn_list*)er_malloc(transitions[i].no_outputs*sizeof(conn_list));
      else
        transitions[i].out_to_place = NULL;
      if (transitions[i].no_inputs)
        transitions[i].in_from_place =
          (conn_list*)er_malloc(transitions[i].no_inputs*sizeof(conn_list));
      else
        transitions[i].in_from_place = NULL;
      if (transitions[i].no_inh)
        transitions[i].inh_from_place =
          (conn_list*)er_malloc(transitions[i].no_inh*sizeof(conn_list));
      else
        transitions[i].inh_from_place = NULL;
      if (transitions[i].no_aff)
        transitions[i].affected_trans =
          (int*)er_malloc(transitions[i].no_aff*sizeof(int));
      else
        transitions[i].affected_trans = NULL;
#ifdef CVR
      if (transitions[i].no_cv_meas)
	{
	  transitions[i].cv_list =
	    (cv_list_elem**)er_malloc(transitions[i].no_cv_meas*
				      sizeof(cv_list_elem*));
	  for (j=0; j<transitions[i].no_cv_meas; j++)
	    transitions[i].cv_list[j]= 
	      (cv_list_elem*)er_malloc(sizeof(cv_list_elem));
	}
      else
        transitions[i].cv_list = NULL;
#endif     
      for (j=0; j<transitions[i].no_outputs; j++)
	if (fread (&transitions[i].out_to_place[j], 
		   sizeof(conn_list), 1, socket_fp) != 1)
	  SOCKET_READ_ERROR;
      for (j=0; j<transitions[i].no_inputs; j++)
	if (fread (&transitions[i].in_from_place[j],
                   sizeof(conn_list), 1, socket_fp) != 1)
	  SOCKET_READ_ERROR;
      for (j=0; j<transitions[i].no_inh; j++)
	if (fread (&transitions[i].inh_from_place[j],
                   sizeof(conn_list), 1, socket_fp) != 1)
	  SOCKET_READ_ERROR;
      for (j=0; j<transitions[i].no_aff; j++)
	{
	  if (fread (&transitions[i].affected_trans[j],
		     sizeof(int), 1, socket_fp) != 1)
	    SOCKET_READ_ERROR;
	}
#ifdef CVR
      for (j=0; j<transitions[i].no_cv_meas; j++)
	{
	  cv_node = transitions[i].cv_list[j];
	  if (fread (&(cv_node->meas_number), sizeof (int), 1, socket_fp) != 1)
	    SOCKET_READ_ERROR;
	  if (fread (&(cv_node->cv_number), sizeof (int), 1, socket_fp) != 1)
	    SOCKET_READ_ERROR;
	  cv_node->recorded = false;
	}
#endif
    }
}


/*************************************************************************

  function:	read_places

  parameter:	--
  
  purpose:	read the array of places from the master

  author:	Christian Kelling
		Technical University of Berlin

  date:		January 1994

*************************************************************************/

void read_places ()
{
  int i, j, name_length;

  places = (place *)er_malloc( no_places * sizeof(place));
  for (i=0; i<no_places; i++)
    {
      if (fread (&places[i].token, sizeof(int), 1, socket_fp) != 1)
	SOCKET_READ_ERROR;
      if (fread (&places[i].no_measures, sizeof(int), 1, socket_fp) != 1)
	SOCKET_READ_ERROR;
      if (fread (&name_length, sizeof(int), 1, socket_fp) != 1)
	SOCKET_READ_ERROR;
      if (fread (places[i].name, name_length, 1, socket_fp) != 1)
	SOCKET_READ_ERROR;
      if (places[i].no_measures)
	places[i].meas_list =
          (int*)er_malloc(places[i].no_measures*sizeof(int));
      else
        places[i].meas_list = NULL;
      
      for (j=0; j<places[i].no_measures; j++)
	{
	  if (fread (&places[i].meas_list[j],
		     sizeof(int), 1, socket_fp) != 1)
	    SOCKET_READ_ERROR;
	}
    }
}


/*************************************************************************

  function:	read_commands

  parameter:	--
  
  purpose:	read the various commands from the master, 

  author:	Christian Kelling
		Technical University of Berlin

  date:		January 1994

*************************************************************************/
void read_commands ()
{
  int cmd, measure_id, both_values[2];
  StatInfo *Measure;
  static struct timeval poll_timeout = { 0L, 0L };	/* Timeout fuer
							   select() */
  fd_set fd_readset;			/* Read-Filedescriptoren fuer select */

  FD_ZERO (&fd_readset);  /* Filedescriptor readset erstellen: */
  FD_SET (socket_fd, &fd_readset);
  if (!select (FD_SETSIZE, &fd_readset, NULL, NULL, &poll_timeout))
      return;
  my_read(both_values, 2*sizeof(int), socket_fd);
  cmd = both_values[0];
  measure_id = both_values[1];
  if ((measure_id < -1) || (measure_id >= no_measures))
    {  /* -1 means no measure in MASTER_STOP command */
      printf("ERROR: Wrong measure id (%d) received by slave %d.\n", 
	     measure_id, my_slave_id);
      my_exit(FAIL);
    }
  switch (cmd)
    {
    case MASTER_STOP:
      printf("MASTER_STOP received by slave %d.\n", my_slave_id);
      my_exit(FAIL);
      break;
    case MASTER_RDY:
      break;
    case MEASURE_STOP:
      Measure = all_measures[measure_id];
      Measure->StopSimulation = true;
      StatSema --;
      break;
    default:
      printf("ERROR: Unknowm command (%d) received by slave.\n", cmd);
    }
}

void wait_for_rdy ()
{
  int cmd,  both_values[2];
  static struct timeval poll_timeout = { 0L, 0L };	/* Timeout fuer
							   select() */
  fd_set fd_reaet;			/* Read-Filedescriptoren fuer select */

/*  FD_ZERO (&fd_readset);  Filedescriptor readset erstellen:
  FD_SET (socket_fd, &fd_readset);
  cmd = -1;
  while (cmd != MASTER_RDY)
    {
	if (select (FD_SETSIZE, &fd_readset, NULL, NULL, &poll_timeout))
	  my_read(both_values, 2*sizeof(int), socket_fd);
	cmd = both_values[0];
    }*/
  cmd = -1;
  while (cmd != MASTER_RDY)
    {
      my_read(both_values, 2*sizeof(int), socket_fd);
      cmd = both_values[0];
    }
  write (socket_fd, "E", 1);
}

/*************************************************************************

     int exit_code;
  function:	Init_TransDetection

  parameter:	--
  
  purpose:	initializes the detection procedure for the initial 
		transient phase

  author:	Christian Kelling
		Technical University of Berlin

  date:		January 1994

*************************************************************************/

void Init_TransDetection ()
{
  int	 i,j;
  StatInfo *Init_One_DetInitTrans(), *Measure;
  ContrSeqList *TempSeqPointer;

  all_measures = (StatInfo **)er_malloc(no_measures * sizeof(StatInfo*));
  zero_counters = (int *)er_malloc(no_measures * sizeof(int));

  DetInitTrans_Init (maxsamples);

  for (i=0; i<no_measures; i++)
    {
    all_measures[i] = Init_One_DetInitTrans();
#ifdef CVR
      Measure = all_measures[i];
      if (fread (&Measure->ControlVariates, sizeof(int), 1, socket_fp) != 1)
	SOCKET_READ_ERROR;
      j = Measure->ControlVariates;
      while (j--)
	{
	  TempSeqPointer = (ContrSeqList *) er_malloc(sizeof(ContrSeqList));
	  TempSeqPointer->Next = Measure->CVList_first;
	  TempSeqPointer->Seq_first = NULL;
	  TempSeqPointer->Seq_last = NULL;
	  TempSeqPointer->SeqPointer = NULL;
	  TempSeqPointer->CurrentPosition = NULL;
	  Measure->CVList_first = TempSeqPointer;
	}
#endif
  }
}

#ifdef STR
/*************************************************************************

  function:     Measure_Update

  parameter:    --- 
  
  purpose:      record the next sampling-point for all measures             

  author:       Ralf Putz

  date:         December 1995

*************************************************************************/
void Measure_Update()
{
  StatInfo *Measure;
  int i, j, k, meas_id, index;

for ( meas_id = 0; meas_id < no_measures; meas_id++)
  {
    Measure = all_measures[meas_id];
    if(!Measure->StopSimulation)
      {
      for (j=0; j < no_places; j++)
        if (places[j].no_measures != NO_MEASURE)
	  for (k = 0; k < places[j].no_measures;k++)
	    if ( (places[j].meas_list[k] == meas_id) && 
	         (Measure->UpdatedYet == false) )
	      {
	        select_measure(meas_id, places[j].token);
	        Measure->UpdatedYet = true;
	      }
    Measure->UpdatedYet = false;
    }
  }
    for (i = 0; i < no_measures; i++ )
      {
	Measure = all_measures[i];
	if ( (Measure->SampleCount == (no_sampling_points)) && 
	      (Measure->StopSimulation == false))
	  {
	    read_commands();
	    send_samples(i, Measure->SampPhasis, no_sampling_points, 1);
	  }
      }
}

/*************************************************************************

  function:     Init_one_Measure 

  parameter:    MeasureInfo -> pointer to a reward measure statistic
  
  purpose:      set the initial values for a reward measure           

  author:       Ralf Putz

  date:         December 1995

*************************************************************************/
void Init_One_Measure(MeasureInfo)
StatInfo *MeasureInfo;
{
  int i;

  MeasureInfo->UpdatedYet = false;
  MeasureInfo->SampleCount = 0;
  MeasureInfo->SendCount = 0;
  for (i = 0; i < no_sampling_points; i++)
    MeasureInfo->SampPhasis[i] = 0.0; 
  MeasureInfo->LastUpdateTime = 0.0;

}

/*************************************************************************

  function:     Measure_Init

  parameter:    --- 
  
  purpose:      allocate memory for all measures            

  author:       Ralf Putz

  date:         December 1995

*************************************************************************/

void Measure_Init()
{
  int i;

  all_measures = (StatInfo **)er_malloc(no_measures * sizeof(StatInfo *)); 
  for (i = 0;i < no_measures; i++)
    {
      all_measures[i] = (StatInfo *)er_malloc(no_measures * sizeof(StatInfo));
      all_measures[i]->SampPhasis = (str_sample_type *) er_malloc
			     (no_sampling_points * sizeof(str_sample_type)); 
      all_measures[i]->StopSimulation = false;
    }
}

/*************************************************************************

  function:     save_initial_marking

  parameter:    --- 
  
  purpose:      save the initial marking of a net            

  author:       Ralf Putz

  date:         December 1995

*************************************************************************/
void save_initial_marking()
{
  int i;

  init_places = (place *)er_malloc( no_places * sizeof(place));
  for (i = 0; i < no_places; i++)
    init_places[i].token = places[i].token;
} 

/*************************************************************************

  function:     load_initial_marking

  parameter:    --- 
  
  purpose:      load the initial marking of a net to simulate the next
                transient phase            

  author:       Ralf Putz

  date:         December 1995

*************************************************************************/
void load_initial_marking()
{
  int i;

  for (i = 0; i < no_places; i++)
    places[i].token = init_places[i].token;
} 
#endif


#ifdef STR
/*************************************************************************

  function:     update_measure

  parameter:    identifier for the related measure and the number of token 
		in a particular place before the marking changes
  
  purpose:      computes a sampling-point for the statistic            

  author:       Ralf Putz

  date:         December 1995

*************************************************************************/
void update_measure(i_measure, old_token)
int i_measure, old_token;
{
  int index;
  StatInfo *Measure;  
  
  Measure = all_measures[i_measure];
  index = Measure->SampleCount;
  Measure->SampPhasis[index] = old_token;
  Measure->SampleCount++;

}
#else

/*************************************************************************
  (BM)

  Funktion:	GetBlockSize

  Parameter:	StatInfo** * int * int * int -> int
                IN: AllMeasures:      Datenstruktur zur Verwaltung aller Ma-
		                      parameter und aufgezeichneter, noch nicht
				      gesendeter, Mawerte
		    NumberOfMeasures: Anzahl der Leistungsmae
		    MeasureID:        Nummer des Maes, dessen Sendeblockgre
		                      berechnet werden soll.
		    MaxSendBlockSize: maximale Sendeblockgre,
		                      i.a. "SEND_BLOCK_DEF"
		OUT: adaptive Blockgre fr das angegebene Ma

  Benutzung:	Liefert die adaptive Blockgre fr das angegebene Ma, die mit
                dem aktuellen Fllstand des Sendepuffers verglichen werden
		kann. Ist die errechnete Blockgre erreicht, kann der Puffer
		an den Master gesendet werden.

  Zweck:	Alle gesendeten und aufgezeichneten Mae werden funktionsintern
                in statischen Variablen gezhlt. Die Geschwindigkeit ist dem
		Zhlerstand proportional. Aus dem Verhltnis der Zhlerstnde
		wird die Blockgre berechnet, die zwischen 1 und
		MaxSendBlockSize liegt.
		Mae, deren Aufzeinung abgebrochen wurde, werden nicht mehr
		bercksichtigt.

  Autoren:	Benjamin Hohnhuser und Mathias Neumann

  Datum:	April 1996

*************************************************************************/

#ifdef __STDC__ 

int						/*OUT adaptive Blockgre fr
						 *    das angegebene Ma*/
GetBlockSize ( StatInfo** AllMeasures,		/*IN  Verwaltung aller Mae*/
	       int        NumberOfMeasures,	/*IN  Anzahl der Mae*/
	       int        MeasureID,		/*IN  Manummer*/
	       int        MaxSendBlockSize )	/*IN  max. Blockgre*/
#else
int
GetBlockSize ( AllMeasures, NumberOfMeasures, MeasureID, MaxSendBlockSize)
               StatInfo** AllMeasures;
	       int        NumberOfMeasures;
	       int        MeasureID;
	       int        MaxSendBlockSize;
#endif

{
/*STATIC*/
  static int* MeasureCounter = NULL;		/*Mazhler[MaNummer]*/
  static int* BlockSize      = NULL;		/*Blockgre[MaNummer]*/
  static int  StopAdaptiveBlocksizing = 0;	/*TRUE wenn schnellstes Ma
						 *stopt; initial FALSE*/
  static int  MCmax_index;			/*Nr. des schnellsten Maes*/
/*VAR*/
  int MCmin, MCmax;				/*min., max. MaCounter-Wert
						 *ber alle Mae*/
  int MCi;					/*MaCounter-Wert fr's
						 *aktuelle Ma*/
  int i;					/*Zhler ber Manummern*/

/*BEGIN*/
  /*MeasureCouter und Blockgrenspeicher beim 1. Aufruf allozieren
   *sonst sind diese Datenstrukturen statisch und werden nicht freigegeben, da
   *sie ber die gesamte Laufzeit des Slaves bestehen bleiben*/
  if (MeasureCounter==NULL) {
    MeasureCounter = (int*)calloc(NumberOfMeasures,sizeof(int));
    BlockSize      = (int*)calloc(NumberOfMeasures,sizeof(int));
    if ((MeasureCounter==NULL)||(BlockSize==NULL)) {
      printf ("Not enough memory for MeasureCounter or BlockSize-Memory!\n");
      exit (1);
    }
  }

  /*das "schnellste" Ma bestimmt das verhltnis zur berechnung der Blockgre
   *wenn es nicht mehr generiert wird, werden auch die berechneten Blockgren
   *"eingefroren"*/
  if ( !StopAdaptiveBlocksizing ) {

    MeasureCounter[MeasureID]++;	/*neues Ma registrieren (zhlen)*/

    MCi = MeasureCounter[MeasureID];	/*Measure-Zhler vom aktuellen Ma
					 *MCi ist bersichliches Krzel*/

    /*Zhlerstand des langsamsten Maes finden*/
    MCmin = INT_MAX;
    for ( i=0; i<NumberOfMeasures; i++ ) {
      if (( MeasureCounter[i] < MCmin )&&
	  ( MeasureCounter[i] > 0)&&
	  ( ! AllMeasures[i]->StopSimulation )) {
	MCmin = MeasureCounter[i];
      }
    }

    /*Zhlerstand des schnellsten Maes finden*/
    MCmax = 0;
    for ( i=0; i<NumberOfMeasures; i++ ) {
      if ( MeasureCounter[i] > MCmax ) {
	MCmax = MeasureCounter[i];
	MCmax_index = i;
      }
    }

    /*wenn schnellstes Ma stopt, dann keine Berechnung der adaptiven
     *mehr Blockgre*/
    if (AllMeasures[MCmax_index]->StopSimulation) {
      StopAdaptiveBlocksizing = 1;	/* = TRUE */
    }

    /*erlaubt das Verhltnis von max. und min. Zhlerstand eine proportionale
     *Abbildung auf Buffergren?*/
    if ((int)((double)MCmax/(double)MCmin + 0.5) <= MaxSendBlockSize) {
      /*ja -> Skalierung*/
      BlockSize[MeasureID] =
	(double)MCi/(double)MCmax * MaxSendBlockSize + 0.5;
    } else {
      /*nein -> Skalierung und Bereichsbeschrnkung*/
      BlockSize[MeasureID] =
	((int)((double)MCi/(double)MCmin + 0.5) > MaxSendBlockSize) ?
	MaxSendBlockSize : ((double)MCi/(double)MCmin + 0.5);
    }
  }

  return (BlockSize[MeasureID] >= 1) ?		/*BlockSize zurckgeben*/
    BlockSize[MeasureID] : 1;			/*mind. Blockgre = 1*/
}

/*************************************************************************

  function:	update_measure

  parameter:	identifier for the related measure and the number of token 
		in a particular place before the marking changes 
  
  purpose:	computes the sample for the statistic, collects it in a packet,
		sends the packet to the master and  
		filters and counts zero samples

  author:	Christian Kelling
		Technical University of Berlin

  date:		January 1994

  Funktion im Ramen des PDV-Projekts im WS 95/96 gendert. (BM)

  nderung:	adaptive Blockgre

  Autoren:	Mathias Neumann und Benjamin Hohnhuser 

  Datum:   	April 1996

  
*************************************************************************/

void update_measure (i_measure, old_token)
     int i_measure;
     int old_token;



{
  static note_flag = false;
  StatInfo *Measure; 
  bool 	DetectInitialTransient(), count_firings(); 
  double sample,sum;
  int sample_count, controls;
  ContrSeqList *RawList;
  SampSeq *RawSeq;
  
  int i;

  /*Variablen zur Verwaltung adaptiver Blockgren, Indizes (BM)*/
  int samplei, cvi;			/*Zhler ber Samples und CV-Daten*/
  static double* CVBlockTmp = NULL;	/*umsortierte CV-Daten*/
  static int MaxControlVariates = -1;	/*max. Anzahl der CVs*/

  Measure = all_measures[i_measure];

  /*CVSequenz mu beim Senden mit variabler Blockgre umsortiert werden (BM)*/
#ifdef CVR
  /*Speicherbereich fr die umsortierte CVSequenz alloziieren*/
  if (CVBlockTmp == NULL) {
    MaxControlVariates = Measure->ControlVariates;
    CVBlockTmp = (double*) er_malloc(SEND_BLOCK_DEF *
				     MaxControlVariates *
				     sizeof(double));
  }
  /*Speicherbereich neu alloziieren wenn zu klein*/
  if (MaxControlVariates < Measure->ControlVariates) {
    if (CVBlockTmp != NULL)
      free(CVBlockTmp);
    MaxControlVariates = Measure->ControlVariates;
    CVBlockTmp = (double*) er_malloc(SEND_BLOCK_DEF *
				     MaxControlVariates *
				     sizeof(double));
  }
#endif

  if (Measure->StopSimulation) 
     {
     return;
     }
#ifdef RST
  /* new_start: all variable initialize */
  if (ns)                                                /*HEZI 5/96*/
  {
    Measure->sum=0.0;
    Measure->LastUpdateTime=0.0;
    Measure->SampleCount=0;
    sum_k[1]=0.0;
    current_k_time[1]=0.0;
    first_k_trial_time[1]=0.0;
    ns=0;
  }
  /* reset_thresholds: a new 'Satz' is current, some variable have to initialize */ 
  if (Satz_changed)                              /*HEZI 4/96*/
  {
   Measure->sum=0.0;
   Measure->LastUpdateTime=0.0;
   Measure->SampleCount=0;
   Satz_changed=0;
  }
#endif

/* debug 
   printf("%f\t%f\n", current_time, Measure->LastUpdateTime); */
  
  sample = (current_time - Measure->LastUpdateTime) *old_token;
  Measure->LastUpdateTime = current_time;

 /* debug 
  printf("Statistic: %d \tSample: %f\n", i_measure, sample);*/

  if (sample) /* dont record zero samples, advance clock only */
    {
     if (Measure->StatStatus != STEADY_STATE)
	DetectInitialTransient(Measure, sample);
     else   
	{           
	  if (Slave_PID == -1)      /* nur einmal senden ! */
          {
            /* PID .. an Master senden  hezi 11/96 */
            Slave_PID=getpid();
#ifdef debug
            printf("ERROR: PID=%d\n",Slave_PID);
#endif
            write (socket_fd, "PID\n",4);
            write (socket_fd, &my_slave_id, sizeof(int));
            write (socket_fd, &Slave_PID, sizeof(int));
            send_trans_length 
	      (i_measure, Measure->InitLength, Measure->TransTime);
          }
	  Measure->sum += sample;
#ifdef RST
	  /* determine the Probability of the rare event and sent it to the master */
	  if ((Measure->SampleCount > 0 )
	      && ((Measure->SampleCount % RESTART_BATCH_SIZE_DEF) ==0)
                      && (first_k_trial_time[Satz]!=0.0) && (current_time!=0.0))
	  {
	    sum_k[Satz]+=Measure->sum;                           /*HEZI 2/96*/
  	    Measure->sum=0.0;
	    prob_rare_event[Satz]=sum_k[Satz]*current_k_time[Satz]/first_k_trial_time[Satz];
	    for(i=1;i<=no_thresholds;i++)
	    {
	     prob_rare_event[Satz]=prob_rare_event[Satz]/(double)retrials[i];  
	    }
	    Prob_rare_event=0.0;
	    for (i=1;i<=Satz;i++) Prob_rare_event+=prob_rare_event[i];
	    Prob_rare_event/=current_time; 
	    Measure->SampSequence[Measure->SendCount].value =Prob_rare_event;
#else
	  if ((Measure->SampleCount > 0 )
              && (Measure->SampleCount % LOCAL_BATCH_SIZE_DEF) == 0)
              {
	      Measure->SampSequence[Measure->SendCount].value = 
		Measure->sum/LOCAL_BATCH_SIZE_DEF;
#endif
	      Measure->SampSequence[Measure->SendCount].s_time =
		current_time-Measure->TransTime;
#ifdef CVR
	      controls = Measure->ControlVariates; 
	      RawList = Measure->CVList_first;
	      while (controls --)
		{
		  RawSeq = RawList->SeqPointer;
		  sum = 0.0;
		  sample_count = 0;
		  while (RawSeq != NULL)
		    {
		      sum += RawSeq->Value;
		      RawSeq = RawSeq->Next;
		      sample_count ++;
		    }
		  if (sample_count != 0)
		    Measure->CVSequence[SEND_BLOCK_DEF*controls+Measure->SendCount] = sum/sample_count;
		  else
               Measure->CVSequence[SEND_BLOCK_DEF*controls+Measure->SendCount] 
	              = 0.0;
		  if (RawList->SeqPointer != NULL)
		    free_Seq(RawList->SeqPointer);
		  RawList->SeqPointer = NULL;
		  RawList = RawList->Next;
		}
#endif

	      Measure->SendCount ++;
#ifdef RST
	      if (Measure->SendCount == 1)
		{
		  read_commands ();
                  send_samples
		    (i_measure, Measure->SampSequence,  
		     1, count_firings ());
#else
	      Measure->sum = 0.0;
	      /*optimale Blockgre fr dieses Ma bestimmen (BM)*/
	      if (Measure->SendCount >= GetBlockSize(all_measures,
						     no_measures,
						     i_measure,
						     SEND_BLOCK_DEF))
		{
		  read_commands ();
		  /*ggf. CVSequenz umsortieren (BM)*/
#ifdef CVR
		  for (samplei=0; samplei<Measure->SendCount; samplei++) {
		    for (cvi=0; cvi<Measure->ControlVariates; cvi++) {
		      CVBlockTmp[STAT_BLOCK_DEF *
				(Measure->ControlVariates*
				 (samplei/STAT_BLOCK_DEF) + cvi) +
				samplei%STAT_BLOCK_DEF] =
			Measure->CVSequence[cvi*SEND_BLOCK_DEF + samplei];
		    }
		  }
#endif
		  /*Buffer senden (BM)*/
                  send_samples(i_measure,
			       Measure->SampSequence,  
			       Measure->SendCount,
			       count_firings ()
#ifdef CVR
			       ,CVBlockTmp,
			       Measure->SendCount*Measure->ControlVariates
#endif
			       );
#endif
		  Measure->SendCount = 0;
		}
	    }
	  Measure->SampleCount ++;
#ifdef RST
          rare_event_count=Measure->SampleCount;   /* for simu_kernel.c  12/95*/
#endif
	}
    }
  else
    {
      if ((++zero_counters[i_measure] >= maxsamples/2) && (!note_flag))
	{
/*	  printf
	    ("NOTE: Statistic %d at slave %d has recorded %d zero samples.", 
	     i_measure, my_slave_id,  maxsamples/2);
	  printf(
	"This statistic will probably never reach the predefined accuracy.\n");*/
	  zero_counters[i_measure] = 0;
	  note_flag = true;
	}
    }
  if ((maxtime > 0.0) && (current_time > maxtime))
    /* limit fuer steady-state simulationen, 0.0 bedeutet kein limit */
    {
      write (socket_fd, "T_LIMIT\n", 7);
      printf
	("NOTE: Simulation at slave %d has exceeded model time limit.\n",
	 my_slave_id);
    }
}
#endif

/*************************************************************************

  function:	record_cv

  parameter:	identifiers for the related measure, the particular control
  		variate and the sample itself
  
  purpose:	records the control variate samples during the simulation run 

  author:	Christian Kelling
		Technical University of Berlin

  date:		January 1994

*************************************************************************/


#ifdef CVR
void record_cv(i_measure, i_control, sample)
     int i_measure, i_control;
     double sample;
{
  StatInfo *Measure;
  SampSeq *RawSeq, *TempSeq;
  int i;
  ContrSeqList *RawList;

  Measure = all_measures[i_measure];
  if (Measure->StatStatus == STEADY_STATE)
    {
      RawList = Measure->CVList_first;
      TempSeq = (SampSeq*) er_malloc (sizeof(SampSeq));
      for (i=0; i<i_control; i++)
	RawList = RawList->Next;
      if (RawList == NULL)
	{
	  printf("ERROR: Insufficent number of control variates defined.");
	  printf("Statistic number: %d\n", i_measure);
	  exit (FAIL);
	}
      RawSeq = RawList->SeqPointer;
      TempSeq->Next = RawSeq;
      TempSeq->Value = sample;
      RawList->SeqPointer = TempSeq;
    }
}
#endif


/*************************************************************************
			
  				main procedure 

*************************************************************************/

main (argc, argv)
     int argc;
     char *argv[];
{
  void sim_net ();
  void rand_init();  /* Zufallszahlengenerator initialisieren */
  void prepare_pmfs();
  void wait_for_ready();

  time_t endtime, time();
  char hostname[30];
  int i;

  clock(); /* Prozessuhr starten.. ( fuer show_rusage() )*/
  
  fclose (stdin);
  close (1);
  close (2);

#ifndef ultrix
  if (!gethostname (hostname, 29))
    if (strcmp (argv[1], hostname))
      /* rand auf anderem Rechner als exec_rands, also rsh austricksen: */
      switch (fork ())
	{
        case -1: exit (1);
        case 0:  break;
        default: exit (0);
	}
#endif

/* Kommandozeilenparameter ueberpruefen: */
  if (argc < 4)
    exit (1);

  starttime = time (NULL);
  init (argv);

#ifdef RST
  t_value=(int *)er_malloc((MAX_THRESHOLDS+1)*sizeof(int));
  retrials=(int *)er_malloc((MAX_THRESHOLDS+1)*sizeof(int));
  first_k_trial_time=(double *)er_malloc((MAX_RESETTING_OF_THRESHOLDS+1)*sizeof(double));
  current_k_time=(double *)er_malloc((MAX_RESETTING_OF_THRESHOLDS+1)*sizeof(double));
  sum_k=(double *)er_malloc((MAX_RESETTING_OF_THRESHOLDS+1)*sizeof(double));
  prob_rare_event=(double *)er_malloc((MAX_RESETTING_OF_THRESHOLDS+1)*sizeof(double));
#endif

  read_infos(); 	  /* lesen vom master dimensionen des netzes,
			     anzahl der measures, seed fuer random */
  read_transitions(); 	  /* lesen netz vom master */
  read_places();

  rand_init(); 		  /* Zufallszahlengenerator initialisieren */
  prepare_pmfs();	/* pmf-Funktionen initialisieren */ 
#ifndef STR
  Init_TransDetection (); /* detect initial transient initialisieren */
#endif
#ifdef STR
  Measure_Init();
  save_initial_marking(); 
  do
    {
      for ( i = 0; i < no_measures; i++ )
        if(!all_measures[i]->StopSimulation)
         Init_One_Measure(all_measures[i]);   
      load_initial_marking(); 
      sim_net(no_sampling_points - 1 );  /* Anzahl der Stuetzstellen - 1 = */
                                         /* Anzahl der Intervalle */
      initial_seed += seed_step;

    }
   while ( StatSema > 0); 
#else
sim_net();       /* eigentlicher simulationslauf bis StatSema == 0 */
#endif
  show_rusage();
  endtime = time (NULL);
  {
    long seconds, minutes = 0, hours = 0;
    seconds = endtime - starttime;
    if (seconds > 59)
      {
	minutes = seconds / 60;
	seconds %= 60;
      }
    if (minutes > 59)
      {
	hours = minutes / 60;
	minutes %= 60;
      }
    printf 
      ("Elapsed overall time: %02lu:%02lu:%02lu\n", hours, minutes, seconds);
  }
  printf("READY\n");	  /* an diesem String erkennt der master, das 
			     korrekte Ende */
  wait_for_rdy();
 /*read_commands ();  Warten, bis master die socket leer gelesen hat */
  my_exit (0);
}



