/******************************************************************************

Programm	:	graph

Komponente	:	csmc_graph.c

Aufgabe		:	Prozeduren zur Verarbeitung von abstrakten Graphen

Bemerkungen	:	Kanten werden nur beachtet, wenn extra angegeben

Projektleiter	:	Christoph Lindemann TU Berlin

Programmierer	:	Armin Zimmermann

Datum		:	Juni 1993

Aenderung       :       Juni 1995 (Patrick Drescher)

******************************************************************************/

#include <stdio.h>
#include "graph_types.h"
#include "csmc_types.h"

/* graph operations */

/*  aus normaler tang_marking - Liste abstrakten Graphen herstellen
  Zustaende werden irgendwie numeriert
  Graph wird in new_SMC eingetragen. Vor.: no_states, states
  Achtung: Zielzustaende von verdraengenden Uebergaengen werden nicht
           aufgenommen, aber die Uebergaenge dahin
  Input:  tang_marking - Liste */
void
graph_of_smc(new_SMC)
	SMC	*new_SMC;
{
	graph		*gra;
	tang_marking	*t_walk, *to_mark;
	node		**node_table, *a_node, *b_node;
	int		no_states = new_SMC->no_states;
	int		i, num = 0;
	char		bits;

	node_table = (node **)emalloc(no_states * sizeof(node *));
				/* zum Merken der node - Adressen */
	scan_array(i, no_states)
		node_table[i] = NULL;
	gra = new_graph();				/* neuen Graph anlegen */
	scan_list(t_walk, new_SMC->states)
	{	a_node = new_node();			/* states eintragen */
		if (bit(t_walk->det, 1))
			a_node->bits |= INITIAL;
		a_node->orig_mark = t_walk;
		a_node->number = t_walk->number = num;	/* Hilfsnumerierung */
		paste_node(gra, a_node);		/* node eintragen */
		node_table[num] = a_node;
		num++;
	}
	scan_list(t_walk, new_SMC->states)		/* vertices eintragen */
	{	a_node = node_table[t_walk->number];
		scan_array(i, t_walk->no_exp_paths)
		{	bits = 0;
			to_mark = (tang_marking *)
				t_walk->exp_paths[i].marking;
			if ((!(to_mark->det) || (to_mark->det_num != t_walk->det_num)))
			{	b_node = (node *)to_mark;
				bits |= PREEMPTING;
			}
			else	b_node = node_table[to_mark->number];
			add_edge(gra, a_node, b_node, 
				t_walk->exp_paths[i].prob, bits);
		}		/* edge eintragen */
	}
	efree(node_table);
	new_SMC->graph = gra;
}

/*  neuen Graphen anlegen (leer)
  Output: Graph*/
graph *
new_graph()
{
	graph	*n_graph;

	n_graph = (graph *)emalloc(sizeof(graph));
	n_graph->first_node = NULL;
	n_graph->no_states = 0;
	n_graph->no_cov_states = 0;
	n_graph->cov_states = NULL;
	return n_graph;
}

/*  Graphen loeschen
  Input:  Graph */
void
delete_graph(d_graph)
	graph	**d_graph;
{
	graph	*gra = *d_graph;
	node	*n_walk, *hn_walk;
	edge	*a_walk, *ha_walk;

	if (gra == NULL)
		return;
	n_walk = gra->first_node;
	while(n_walk != NULL)
	{	hn_walk = n_walk->next;
		a_walk = n_walk->successors;
		while(a_walk != NULL)
		{	ha_walk = a_walk->next;
			efree(a_walk);
			a_walk = ha_walk;
		}
		a_walk = n_walk->predecessors;
		while(a_walk != NULL)
		{	ha_walk = a_walk->next;
			efree(a_walk);
			a_walk = ha_walk;
		}
		efree(n_walk);
		n_walk = hn_walk;
	}
	efree(gra->cov_states);
	efree(gra);
	*d_graph = NULL;
}

/* variables for top_sort_acyclic_graph */
graph		*sort_graph, *old_graph;

dfs_sort_3(n_from)
	node		*n_from;
{
	edge		*w_edge;
	node		*n_to;

	n_from->bits |= VISITED;
	cut_node(old_graph, n_from);
	scan_list(w_edge, n_from->successors)
	{	if (w_edge->bits & PREEMPTING)
			continue;
		n_to = (node *)w_edge->to_node;
		if (n_to->bits & VISITED)
			continue;
		dfs_sort_3(n_to);
	}
	paste_node(sort_graph, n_from);
}

/* Graphen topologisch sortieren; Graph ist muss azyklisch sein.
   Es wird die Reihenfolge der linearen Liste sortiert */
void
top_sort_acyclic_graph(gra)
	graph		*gra;
{
	node		*n_walk;

	sort_graph = new_graph();
	old_graph  = gra;

	/* mark states unvisited */
	scan_list(n_walk, gra->first_node)
	{	n_walk->bits &= ~VISITED;
		n_walk->bits |= LAST_OF_SCC;	/* because of acyclicity */
	}

	/* dfs for topological order */
	while ((n_walk = gra->first_node) != NULL)
		dfs_sort_3(n_walk);

	/* get the ordered graph */
	gra->first_node = sort_graph->first_node;
	gra->no_states  = sort_graph->no_states;
	efree(sort_graph);
}

/* variables for top_sort_cyclic_graph */
graph		*o_graph, *x_graph, *y_graph, *scc_graph;
int		*scc_nrs, SCC_Nr;

/* procedure for a depth-first search for top_sort_cyclic_graph */
dfs_sort_1(n_from)
	node		*n_from;
{
	edge		*w_edge;
	node		*n_to;

	n_from->bits |= VISITED;
	cut_node(o_graph, n_from);
	scan_list(w_edge, n_from->successors)
	{	if (w_edge->bits & PREEMPTING)
			continue;
		n_to = (node *)w_edge->to_node;
		if (n_to->bits & VISITED)
			continue;
		dfs_sort_1(n_to);
	}
	paste_node(x_graph, n_from);
}

/* procedure for a depth-first search for top_sort_cyclic_graph */
dfs_sort_2(n_from)
	node		*n_from;
{
	edge		*w_edge;
	node		*n_to;

	n_from->bits |= VISITED;
	cut_node(x_graph, n_from);
	scan_list(w_edge, n_from->predecessors)
	{	n_to = (node *)w_edge->to_node;
		if (n_to->bits & VISITED)
			continue;
		dfs_sort_2(n_to);
	}
	paste_node(y_graph, n_from);
	scc_nrs[n_from->number] = SCC_Nr;
}

/* Graphen topologisch sortieren; Graph ist moeglicherweise zyklisch.
   Es wird die Reihenfolge der linearen Liste sortiert */
top_sort_cyclic_graph(gra, cyclic)
	graph		*gra;
	int		cyclic;
{
	node		*n_walk, *n_to, *help_node, **scc_nodes;
	edge		*e_walk;
	int		cis_nr, from, to, bit;

	if (!cyclic)
	{	top_sort_acyclic_graph(gra);
		return;
	}
	/* cyclic case */
	o_graph = gra;
	scc_nrs = (int *)emalloc(gra->no_states * sizeof(int));

	/* find strongly connected components */
	scan_list(n_walk, gra->first_node)
		n_walk->bits &= ~VISITED;
	x_graph = new_graph();
	y_graph = new_graph();
	scan_array(cis_nr, gra->no_cov_states)
	{	n_walk = gra->cov_states[cis_nr];
		dfs_sort_1(n_walk);
	}
	scan_list(n_walk, x_graph->first_node)
		n_walk->bits &= ~VISITED;
	scc_graph = new_graph();
	while ((n_walk = x_graph->first_node) != NULL)
	{	dfs_sort_2(n_walk);
		SCC_Nr++;
	}

	/* build new acyclic graph of strongly connected components */
	scc_nodes = (node **)emalloc(SCC_Nr * sizeof(node *));
	scan_array(cis_nr, SCC_Nr)
	{	n_walk = new_node();
		n_walk->number = cis_nr;
		scc_nodes[cis_nr] = n_walk;
		paste_node(scc_graph, n_walk);
	}
	scan_list(n_walk, y_graph->first_node)
	{	from = scc_nrs[n_walk->number];
		scan_list(e_walk, n_walk->successors)
		{	if (e_walk->bits & PREEMPTING)
				continue;
			n_to = (node *)e_walk->to_node;
			to = scc_nrs[n_to->number];
			if (from == to)
				continue;	/* inter-SCC edge */
			add_edge(scc_graph, scc_nodes[from], scc_nodes[to], 0.0, 0);
		}
	}

	/* sort acyclic graph of SCCs topological */
	top_sort_acyclic_graph(scc_graph);

	/* rebuild sorted original graph */
	while(1)
	{	if ((n_walk = scc_graph->first_node) == NULL)
			break;
		while (n_walk->next != NULL)
			n_walk = n_walk->next;
		cut_node(scc_graph, n_walk);
		bit = 0;
		n_to = y_graph->first_node;
		while (n_to != NULL)
		{	help_node = n_to->next;
			if (scc_nrs[n_to->number] != n_walk->number)
			{	n_to = help_node;
				continue;
			}
			if (!bit)
			{	bit = 1;
				n_to->bits |= LAST_OF_SCC;	/* mark as last member of one SCC */
			}
			cut_node(y_graph, n_to);
			paste_node(o_graph, n_to);
			n_to = help_node;
		}
	}

	efree(scc_nrs);
	delete_graph(&scc_graph);
	delete_graph(&x_graph);
	delete_graph(&y_graph);
}

/* node operations */
/*  neuen Node herstellen (markiert: neu)
  Output: Node */
node *
new_node()
{
	node	*n_node;

	n_node = (node *)emalloc(sizeof(node));
	n_node->number = 0;
	n_node->bits = 0;
	n_node->successors = NULL;
	n_node->predecessors = NULL;
	n_node->orig_mark = NULL;
	n_node->next = NULL;
	n_node->no_succ = 0;
	n_node->no_pred = 0;
	return n_node;
}

/*  node loeschen (schon aus Graph herausgenommen!)
  Input:  Node */
void
delete_node(d_node)
	node	*d_node;
{
	efree(d_node);
}
 
/*  node in Graphen dazutun (vorne in Liste einhaengen)
  Input:  Graph, Node */
void
paste_node(a_graph, a_node)
	graph	*a_graph;
	node	*a_node;
{
	a_node->next = a_graph->first_node;
	a_graph->first_node = a_node;
	a_graph->no_states++;
}


/*  node aus Graphen herausnehmen (nur aus der Zustandsliste)
  Input:  Graph, Node (NULL: ersten nehmen)
  Output: Node (NULL: nicht gefunden / keiner mehr da) */
node *
cut_node(a_graph, a_node)
	graph	*a_graph;
	node	*a_node;
{
	node	*n_walk;

	n_walk = a_graph->first_node;
	if (n_walk == NULL)
		return NULL;				/* nichts mehr da */
	if ((a_node == NULL) || (n_walk->next == NULL)
	      || (a_node == n_walk))			/* nimm ersten */
	{	a_graph->first_node = n_walk->next;
		a_graph->no_states--;
		return n_walk;
	}
	while (n_walk->next != NULL)
	{	if (n_walk->next == a_node)		/* gefunden */
		{	n_walk->next = a_node->next;
			a_node->next = NULL;
			a_graph->no_states--;
			return a_node;
		}
		n_walk = n_walk->next;
	}
	return NULL;					/* nicht gefunden */
}

/* edge operations */
/*  neuen edge in Graph eintragen (vor- und rueckwaerts!)
    in der Kantenliste wird ein insert-sort durchgefuehrt
  Input:  Graph, from, to als Pointer (to: zeigt auf node oder tang_marking */
void
add_edge(a_graph, from, to, rate, bits)
	graph	*a_graph;
	node	*from, *to; /* preempting case: tang_marking *to */
	double	rate;
	char	bits;
{
	edge	*v_forward, *v_back;

	if ((from == NULL) || (to == NULL))
	{	printf("ERROR in add_edge: NULL-pointer.\n");
		exit(1);
	}
	if (from == to)
	{	printf("ERROR in add_edge: self loop detected.\n");
		exit(1);
	}
	scan_list(v_forward, from->successors)
		if (v_forward->to_node == to)		/* no double arcs */
			if (rate == 0.0)
				return;			/* graph structure algorithm */
			else
			{	printf("ERROR in add_edge: double arc.\n");
				exit(1);
			}
	v_forward = (edge *)emalloc(sizeof(edge));
	v_forward->rate = rate;
	v_forward->bits = bits;
	v_forward->to_node = (void *)to;
	from->no_succ++;
	/* insert sort */
	v_back = from->successors;
	if ((v_back == NULL) || (v_back->rate > v_forward->rate))
	{	v_forward->next = from->successors;
		from->successors = v_forward;
	} else
	{	while ((v_back->next != NULL) && (v_back->next->rate < v_forward->rate))
			v_back = v_back->next;
		v_forward->next = v_back->next;
		v_back->next = v_forward;
	}
	if (!(PREEMPTING & bits))
	{	v_back = (edge *)emalloc(sizeof(edge));
		v_back->bits = bits;
		v_back->to_node = (void *)from;
		v_back->next = to->predecessors;
		to->predecessors = v_back;
		to->no_pred++;
	}
}

/* test output of graph */

void
show_graph(gra, options)
	graph	*gra;
	int	options;
{
	node	*n_walk;

	printf("GRAPH:");
	printf(" states: %d\n", gra->no_states);
	n_walk = gra->first_node;
	while (n_walk != NULL)
	{	show_node(n_walk, options);
		n_walk = n_walk->next;
	}
	fflush(stdout);
}

void
show_node(s, options)
	node	*s;
	int	options;
{
	edge		*vert;
	tang_marking	*om;
	char		*name;
	int		h_int[255];

	printf("\n => node #%d ", s->number);
	if (options & NODE_BITS)
	{	if (s->bits & COV_INITIAL)
			printf("covering_");
		if (s->bits & INITIAL)
			printf("initial ");
		if (s->bits & VISITED)
			printf("visited ");
		if (s->bits & LAST_OF_SCC)
			printf("last_of_scc ");
		if (!s->no_pred)
			printf("root_state ");
		if (!s->no_succ)
			printf("leaf_state ");
	}
	printf("\n");
	if (options & ORIG_MARKS)
	{	om = s->orig_mark;
		printf("orig. marking: #%d: ", om->number);
                global_marking(s->orig_mark->mark, s->orig_mark->disc, h_int);
                print_global_marking(h_int);
		if (options & DET_TRANS)
		{	if (om->det)
			{	name = timed.transitions[om->det_num].name;
				printf("det. trans. %s enabled\n", name);
			} else	printf("no det. trans. enabled\n");
		}
	}
	if ((options & SUCCESSORS) && s->no_succ)
	{	printf(">%d successor(s):\n", s->no_succ);
		vert = s->successors;
		while (vert != NULL)
		{	show_edge(vert, options);
			vert = vert->next;
		}
	}
	if ((options & PREDECESSORS) && s->no_pred)
	{	printf(">%d predecessor(s):\n", s->no_pred);
		vert = s->predecessors;
		while (vert != NULL)
		{	show_edge(vert, options & 0xf7);
			vert = vert->next;
		}
	}
	fflush(stdout);
}

void
show_edge(v, options)
	edge	*v;
	int	options;
{
	node		*to;
	tang_marking	*om;
	int		h_int[255];
	char		*name;

	if ((v->bits & PREEMPTING) && (options & ORIG_MARKS))
	{	om = (tang_marking *)v->to_node;
		printf(" => preempted to orig. marking: #%d: ", om->number);
                global_marking(om->mark, om->disc, h_int);
                print_global_marking(h_int);
		if (options & DET_TRANS)
		{	if (om->det)
			{	name = timed.transitions[om->det_num].name;
				printf("timed trans. %s enabled\n", name);
			} else	printf("no det. trans. enabled\n");
		}
	} else
	{	to = (node *)v->to_node;
		printf(" => to %d ", to->number);
	}
	if (options & RATES)
		printf("    with %lf", v->rate);
	if (options & VERTEX_BITS)
	{	if (v->bits & PREEMPTING)
			printf(" preempting ");
	}
	printf("\n");
	fflush(stdout);
}
