#define ISO_OUTPUT
/******************************************************************************

Programm	:	graph

Komponente	:	csmc_main.c

Aufgabe		:	Hauptprogramm fuer die Untersuchung der
			zugeordneten Markov - Ketten,
			exportiert die Prozedur csmc_main();

Projektleiter	:	Christoph Lindemann TU Berlin

Programmierer	:	Armin Zimmermann

Datum		:	Juni 1993

Aenderung       :       Juni 1995 (Patrick Drescher)

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include "graph_types.h"
#include "csmc_types.h"
#include "exec_types.h"
#include "matrices_P_C.h"

/* global variables */
int		no_smcs = 0;		/* number of subord. Markov chains of det. trans. */
int             no_esmcs = 0;           /* number of subord. Markov chains of gen. trans. */
int		no_csmcs = 0;		/* number of components of SMCs */
int		no_iso_classes = 0;	/* number of isomorphism classes */
int		no_normal_classes;	/* number of isomorphism classes
					   without special topology */

Iso_Class	*Iso_Classes;		/* linear list of isomorphism classes */
Iso_Class	*normal_classes;	/* pointer to the first isomorphism
					   class without special topology in the
					   Iso_Classes - list */
SMC		**SMCs;			/* array of subordinated Markov chains (det) */
SMC             **ESMCs;                /* array of subordinated Markov chains (gen) */

void
csmc_main(model_name)
	char		*model_name;
{
	SMC             *a_smc;
	Iso_Class	*iso_walk;
	int		tra_nr, i;


	signal (SIGFPE, SIG_IGN);
	printf("\nDERIVING COMPONENTS OF SUBORDINATED MARKOV CHAINS ...\n");
	fflush(stdout);

	/* obtain and mark initial states of the RRG */
	mark_initial_states(exp_markings, det_markings);

	Iso_Classes = NULL;
	SMCs = (SMC **)emalloc((no_det_trans - no_gen_trans) * sizeof(SMC *));
	ESMCs = (SMC **)emalloc((no_gen_trans) * sizeof(SMC *));
	/* for each deterministic transition: */
	for (tra_nr = 0; tra_nr < no_det_trans; tra_nr++)
	{	tang_marking *h_mark = det_markings[tra_nr];
		if (h_mark == NULL)
			continue;		/* SMC is empty */

		/* obtain SMC characteristics */
		a_smc = (SMC *)emalloc(sizeof(SMC));
		a_smc->first_state_no = 0;
		a_smc->no_states = no_nodes[tra_nr];
		a_smc->states = h_mark;
		i = a_smc->states->det_num;
		a_smc->trans = &(timed.transitions[i]);
               
                /* es werden zwei SMC-Listen gebildet */

                if (timed.transitions[i].type == 2) /* GENERAL */
		  {
		  ESMCs[no_esmcs] = a_smc;		/* new ESMC */
		  no_esmcs++;
                  }		   
                else
                  {
 		  SMCs[no_smcs] = a_smc;		/* new SMC */
		  no_smcs++;
     		  /* standardize Qi to Tau = 1 */
		  standardize_Qi(a_smc);	
		  }
                   
		/* get abstract graph of SMC (with preempting edges) */
		graph_of_smc(a_smc);		/* get graph */

#ifdef DEBUG
		printf("==================\nMDET of %s\n", a_smc->trans->name);
		show_graph(a_smc->graph, 0xbb);
#endif

		/* obtain partitioning in CSMCs */
                if (timed.transitions[i].type == 1) /* DETERMINISTIC */
  		   part_smc_into_csmcs(a_smc);
	}

	/* number all tangible states and write informations to RATES file */
	number_tang_states();

#ifdef DEBUG
	{	Iso_Class	*class_walk;
		Tau_CSMC	*tau_walk;
		Iso_CSMC	*iso_walk;
		node 		*n_walk;

		printf("===============\nend-zustand:\n");
		scan_list(class_walk, Iso_Classes)
		{	printf("Iso-Class %d:\n", class_walk->class_id);
			printf("no_tau_csmcs %d:\n", class_walk->no_tau_csmcs);
			printf("no_total_csmcs %d:\n", class_walk->no_total_csmcs);
			scan_list(tau_walk, class_walk->Tau_CSMCs)
			{	printf("new Tau-CSMC:\n");
				scan_list(n_walk, tau_walk->graph->first_node)
					show_node(n_walk, 0xfb);
				scan_list(iso_walk, tau_walk->Iso_CSMCs)
				{	printf("new Iso-CSMC:\n");
					scan_list(n_walk, iso_walk->graph->first_node)
						show_node(n_walk, 0xfb);
				}
			}
		}	
	}
#endif

	/* calculate and write EMC00 and CONV00 for exp. markings */
	write_files_for_mexp(model_name);

	/* solve special topologies direct */
	no_normal_classes = no_iso_classes;
	scan_list(iso_walk, Iso_Classes)
	{	if ( !(iso_walk->bits & (DIRECT_TOP | ERLANG_TOP | HYPO_TOP | BRANCH_TOP)) )
			break;
		if (iso_walk->bits & NO_TOP_CALC)
			break;
		no_normal_classes--;
		solve_csmc_topology(model_name, iso_walk);
	}
	normal_classes = iso_walk;

	/* write informations to net.AUX */
	write_aux(model_name);

	/* print statistical informations */
	print_csmc_info();

}

/* write informations to net.AUX */
write_aux(model_name)
	char		*model_name;
{
	register FILE	*aux_file;
	char filename[300];

	/* Anzahl der andauernden Zustaende im File <netname>.AUX ablegen:
	(fuer solve_LGS.c) */

	sprintf (filename, "%s.AUX", model_name);
	if (!(aux_file = fopen (filename, "a")))
	{	perror (NULL);
		fprintf (stderr, "ERROR: can't open file '%s' for appending!", filename);
		exit (1);
	}
	fprintf (aux_file, "TANGIBLE MARKINGS %u\n", no_tang_markings);
	fclose (aux_file);
}

/* print statistical informations */
print_csmc_info()
{
	Iso_Class	*class_walk;
	Tau_CSMC	*tau_walk;
	graph		*gra;

	if (!no_iso_classes)
	{/*	printf("This net is a GSPN.\n");*/
		return;
	}
	printf("\nNumber of subordinated Markov chains of det. trans. (SMC): %d\n", no_smcs);
	printf("Number of subordinated Markov chains of gen. trans. (ESMC): %d\n", no_esmcs);
	printf("Number of connected components of subordinated Markov chains (CC-SMC): %d\n", no_csmcs);
	printf("Number of isomorphism classes of CC-SMCs: %d\n\n", no_iso_classes);
	printf("\nDERIVING ISOMORPHISM CLASSES AND SPECIAL STRUCTURES ...\n\n");

	/* scan the isomorphism classes */
	scan_list(class_walk, Iso_Classes)
	{	gra = class_walk->graph;
#ifndef ISO_OUTPUT
		if (!class_walk->bits)
			continue;
#endif
		printf("isomorphism class %d:", class_walk->class_id);
#ifdef ISO_OUTPUT
		printf("\n  - number of states: %d\n", gra->no_states);
		printf("  - number of covering initial states: %d\n", gra->no_cov_states);
		printf("  - number of sets of CC-SMCs with diff. tau:");
		printf(" %d\n", class_walk->no_tau_csmcs);
#endif
		if (class_walk->bits)
		{	printf("  - properties: ");
			if (class_walk->bits & CYCLIC)
				printf("cyclic structure");
			if (class_walk->bits & DIRECT_TOP)
				printf("topology with closed-form solution");
			if (class_walk->bits & ERLANG_TOP)
				printf("sequence topology ");
			if (class_walk->bits & HYPO_TOP )
				printf("sequence topology ");
			if (class_walk->bits & BRANCH_TOP)
				printf("branching topology");
			printf("\n");
		}
#ifdef ISO_OUTPUT
		scan_list(tau_walk, class_walk->Tau_CSMCs)
			printf("  - set of CC-SMCs with tau = %f contains %d isomorphic CC-SMCs\n", 
					(float)tau_walk->tau, tau_walk->no_isom_csmcs+1);
#endif		
	}
	fflush(stdout);
}

/* look for and mark initial states of the RRG */
mark_initial_states(e_marks, d_marks)
	tang_marking	*e_marks;
	tang_marking	**d_marks;
{
	tang_marking	*from, *to;
	int		i, m_nr, to_nr;


	from = e_marks;
	while (from != NULL)			/* scan exp. markings */
	{	scan_array(i, from->no_exp_paths)
		{	to = (tang_marking *)from->exp_paths[i].marking;
			if (!to->det)
				continue;
			set(&(to->det), 1);	/* mark as initial */
		}
		from = from->next;
	}
	scan_array(m_nr, no_det_trans)
	{	from = d_marks[m_nr];
		while (from != NULL)		/* scan deterministic markings */
		{	scan_array(i, from->no_exp_paths) /* exp. fire paths */
			{	to = (tang_marking *)from->exp_paths[i].marking;
				if (to->det)	/* to is deterministic */
				{	to_nr = timed.transitions[to->det_num].det_no;
					if (to_nr != m_nr)
						set(&(to->det), 1); /* initial m. */
				}
			}
			scan_array(i, from->no_det_paths) /* det. fire paths */
			{	to = (tang_marking *)from->det_paths[i].marking;
				if (to->det)	/* to is deterministic */
					set(&(to->det), 1); /* initial m. */
			}
			from = from->next;
		}	
	}
}


/* standardize Qi to Tau = 1 */
standardize_Qi(smc)
	SMC		*smc;
{
	tang_marking	*t_walk;
	int		i;
	double		tau_i;

	scan_list(t_walk, smc->states)
	{	tau_i = det_transition_delay(t_walk);
		t_walk->sum_of_rates *= tau_i;
		scan_array(i, t_walk->no_exp_paths)
			t_walk->exp_paths[i].prob *= tau_i;
	}
}

/* compare two doubles with predefined epsilon */
int
d_equal(d1, d2)
	double	d1, d2;
{
	double	diff = (d1 - d2);

	if ((diff < COMP_DOUBLE_EPS) && (diff > -COMP_DOUBLE_EPS))
		return 1;
	else	return 0;
}

/* write the RATES file entry for a tangible marking containing the 
   actual firing rates of timed transitions and the RG file entry
   describing the tangible marking with the number of tokens in each place */
static int	*i_mark = NULL;

void
write_rg_and_rates_file(marking)
	tang_marking	*marking;
{
	rates_type	s_rate;
	int		h_fire, tra;

	/* obtain global marking */
	if (i_mark == NULL)
		i_mark = (int *)emalloc(no_places * sizeof(int));
	global_marking(marking->mark, marking->disc, i_mark);

	/* write marking to .RG */
	fwrite (i_mark, sizeof (int), no_places, rg_file);

	/* write rates to .RATES */
	fwrite(&(marking->fire_tra_num), sizeof(byte), 1, rates_file);
	scan_array(h_fire, marking->fire_tra_num)
	{	tra = marking->fire_nums[h_fire];
		s_rate.t_nr = timed.transitions[tra].number;
		if (marking->det && (tra == marking->det_num))
			s_rate.rate = det_transition_delay(marking);
		else	s_rate.rate = exp_transition_rate(&timed, tra, i_mark);
		fwrite(&s_rate, sizeof(s_rate), 1, rates_file);	/* write rates info */
	}
}

/* number all tangible states */
number_tang_states()
{
	Iso_Class	*act_class;
	Tau_CSMC	*act_tau_csmc;
	Iso_CSMC	*act_iso_csmc;
	node		**node_walks;
	tang_marking	*act_tang;
	int		i, j, iso_class_nr, state_counter, csmc_counter;

	/* number exponential states */
	state_counter = 1;
	scan_list(act_tang, exp_markings)
	{	act_tang->number = state_counter++;
		write_rg_and_rates_file(act_tang);
	}	

	/* number CSMCs and CSMC-states */
	iso_class_nr = 0;
	scan_list(act_class, Iso_Classes)
	{	act_class->class_id = ++iso_class_nr;	/* EMC and CONV number */
		act_class->first_state_no = state_counter;

		/* number nodes of CSMC with special topology */
		if ((act_class->bits & (DIRECT_TOP | ERLANG_TOP | HYPO_TOP | BRANCH_TOP))
				&& !(act_class->bits & NO_TOP_CALC))
		{	number_csmc_topology(act_class, &state_counter);
			continue;
		}

		/* get the list roots of all isomorphic CSMCs */
		node_walks = (node **)emalloc(act_class->no_total_csmcs * sizeof(node *));
		csmc_counter = 0;
		scan_list(act_tau_csmc, act_class->Tau_CSMCs)
		{	node_walks[csmc_counter++] = act_tau_csmc->graph->first_node;
			scan_list(act_iso_csmc, act_tau_csmc->Iso_CSMCs)
				node_walks[csmc_counter++] = act_iso_csmc->graph->first_node;
		}

		/* number nodes of normal CSMC */
		scan_array(i, act_class->graph->no_states)
			scan_array(csmc_counter, act_class->no_total_csmcs)
			{	node_walks[csmc_counter]->orig_mark->number = state_counter++;
				write_rg_and_rates_file(node_walks[csmc_counter]->orig_mark);
				node_walks[csmc_counter] = node_walks[csmc_counter]->next;
			}
		efree(node_walks);
	}

        /* number ESMCs */

        last_class_id = iso_class_nr;  /* fuer die indices der ESMCs */

        scan_array(i, no_esmcs)
	{   act_tang=ESMCs[i]->states;
            scan_array(j, ESMCs[i]->no_states)
            {       act_tang->number = state_counter++;
                    write_rg_and_rates_file(act_tang);
                    act_tang=act_tang->next;
            }     
	}

	no_tang_markings = state_counter - 1;	/* set the global number of tang. markings */
}

/* calculate and write EMC00 and CONV00 for exp. markings */
write_files_for_mexp(m_name)
	char		*m_name;
{
	int		f_count, m_len = strlen(m_name) + 20;
	char		*conv_filename, *emc_filename;
	FILE		*emc_fp, *conv_fp;
	tang_marking	*t_walk;
	tang_path	*p_walk;
	double		div_rate_sum;

	/* prepare file handles */
	conv_filename = (char *)emalloc(m_len * sizeof(char));
	emc_filename = (char *)emalloc(m_len * sizeof(char));
	sprintf (conv_filename, "%s.CONV00", m_name);
	sprintf (emc_filename, "%s.EMC00", m_name);
	if ((emc_fp = fopen(emc_filename, "w")) == NULL)
	{	printf("ERROR: could not open %s for writing.\n", emc_filename);
		exit(1);
	}
	if ((conv_fp = fopen(conv_filename, "w")) == NULL)
	{	printf("ERROR: could not open %s for writing.\n", conv_filename);
		exit(1);
	}

	/* scan all exponential markings */
	scan_list(t_walk, exp_markings)
	{	div_rate_sum = 1.0 / t_walk->sum_of_rates;
		/* write header to EMC file */
		P_row_header.row_index = t_walk->number;
		P_row_header.entries_in_row = t_walk->no_exp_paths;
		if (fwrite (&P_row_header, SIZEOF_P_row_header, 1, emc_fp) != 1)
		{	printf("ERROR while writing to %s.\n", emc_filename);
			exit(1);
		}

		/* write CONV file entry */
		C0_entry.sojourn_time = div_rate_sum;
		C0_entry.state_number = t_walk->number;
		if (fwrite (&C0_entry, SIZEOF_C0_entry, 1, conv_fp) != 1)
		{	printf("ERROR while writing to %s.\n", conv_filename);
			exit(1);
		}

		/* write state transitions to EMC */
		scan_array(f_count, t_walk->no_exp_paths)
		{	p_walk = &(t_walk->exp_paths[f_count]);
			P_row_entry.column_index = ((tang_marking *)(p_walk->marking))->number;
			P_row_entry.probability = p_walk->prob * div_rate_sum;
			if (fwrite (&P_row_entry, SIZEOF_P_row_entry, 1, emc_fp) != 1)
			{	printf("ERROR while writing to %s.\n", conv_filename);
				exit(1);
			}
		}
	}

	/* clean up */
	fclose(conv_fp);
	fclose(emc_fp);
	efree(conv_filename);
	efree(emc_filename);
}

/* obtain information for randinfos for exec_rands */
randinfo *randinfos;		/* Informationen ueber die zugeordneten Markoff-Ketten */
unsigned procs_to_start;	/* Anzahl der zu startenden rand-Prozesse */
void
obtain_randinfos()
{
	Iso_Class	*iso_walk;
	Tau_CSMC	*tau_walk;
	tang_marking	*mark_walk;
	node		*n_walk;
	int		act_rand = 1;
	double		max_tau = 0.0;

	/* Informationen ueber die zu startenden rand-Prozesse erzeugen: */
	randinfos = (randinfo *)emalloc ((no_normal_classes + 1) * sizeof (randinfo));
	
	scan_list(iso_walk, normal_classes)
	{	register double max_qii = 0.0;
		register unsigned eta = 0;
		register randinfo *info = &randinfos[act_rand++];

		info->class = iso_walk;
		info->tau = 1.0;
		/* look for biggest tau */
		scan_list(tau_walk, iso_walk->Tau_CSMCs)
			if (tau_walk->tau > max_tau)
				max_tau = tau_walk->tau;

		/* look for biggest Q entry */
		scan_list(n_walk, iso_walk->graph->first_node)
		{	mark_walk = n_walk->orig_mark;
	  		eta += mark_walk->no_exp_paths;
			if (mark_walk->sum_of_rates > max_qii)
				max_qii = mark_walk->sum_of_rates;
		}
		
		info->eta = eta;
		info->max_qii = max_qii;
		info->status = no_process;
	}
}

/* compare two edges for qsort */
int		max_edges;
edge		*sort_edges  = NULL;
int
edgecomp(a, b)
	edge	*a, *b;
{
	return ((char *)a->to_node - (char *)b->to_node);
}

/* send data for a isomorphic csmc to a rand process */
send_iso_data_to_rand(preempted, csmc_gr, bits, fp)
	tang_marking	**preempted;
	graph		*csmc_gr;
	char		bits;
	FILE		*fp;
{
	node		*n_walk, *tn_walk;
	edge		*e_walk;
	tang_marking	*t_walk, *tt_walk;
	tang_path	*path;
	double		tau_i;
	char		md_tau, d_char, bit;
	int		i, j, no_preempted = 0;

	/* tau marking dependent ? */
	md_tau = bits & MARK_DEP_TAU;
	my_fwrite(&md_tau, sizeof(char), fp);

	/* send absolute tau, if not marking dependent */
	if (!md_tau)
	{	n_walk = csmc_gr->first_node;
		t_walk = n_walk->orig_mark;
		/* insert here for multiple tau transient solution */
		/* all tau values relative to tau_i (tau_x/tau_i */
		tau_i = det_transition_delay(t_walk);
		my_fwrite(&tau_i, sizeof(double), fp);
	}

	/* scan csmc states */
	scan_list(n_walk, csmc_gr->first_node)
	{	t_walk = n_walk->orig_mark;

		/* send actual tau, if marking dependent */
		if (md_tau)
		{	tau_i = det_transition_delay(t_walk);
			my_fwrite(&tau_i, sizeof(double), fp);
		}

		/* initial state ? */
		d_char = n_walk->bits & INITIAL;
		my_fwrite(&d_char, sizeof(char), fp);

		if (max_edges <= t_walk->no_det_paths)
		{	max_edges = t_walk->no_det_paths + 1;
			sort_edges = (edge *)realloc(sort_edges, max_edges * sizeof(edge));
		}

		/* assemble delta matrix entries */
		scan_array(i, t_walk->no_det_paths)
		{	path = &(t_walk->det_paths[i]);
			tt_walk = (tang_marking *)path->marking;
			/* state number */
			sort_edges[i].to_node = (void *)((long)tt_walk->number);
			/* probability */
			sort_edges[i].rate = path->prob;
		}
		qsort(sort_edges, t_walk->no_det_paths, sizeof(edge), edgecomp);

		/* delta matrix row */
		my_fwrite(&(t_walk->no_det_paths), sizeof(int), fp);
		scan_array(i, t_walk->no_det_paths)
		{	/* state number */
			my_fwrite(&(sort_edges[i].to_node), sizeof(int), fp);
			/* probability */
			my_fwrite(&(sort_edges[i].rate), sizeof(double), fp);
		}

		/* look for preempted states */
		scan_list(e_walk, n_walk->successors)
		{	if (!(e_walk->bits & PREEMPTING))
				continue;
			t_walk = (tang_marking *)e_walk->to_node;
			preempted[no_preempted++] = t_walk;
		}
	}

	/* send informations about preempted states */
	scan_array(i, no_preempted)
	{	t_walk = preempted[i];
		/* send marking number */
		my_fwrite(&(t_walk->number), sizeof(int), fp);
	}
}

/* send data to rand process (called from exec_rands) */
void
send_data_to_rand(fp, class_p)
	FILE		*fp;
	Iso_Class	*class_p;
{
	Tau_CSMC	*tau_walk;
	Iso_CSMC	*iso_walk;
	node		*n_walk, *tn_walk;
	edge		*e_walk;
	tang_marking	*t_walk, **preempted;
	char		d_char, acyclic, branch;
	int		i, j, e_num, p_num;

	/* id number of the isom. class */
	my_fwrite(&(class_p->class_id), sizeof(int), fp);

	/* number of states */
	my_fwrite(&(class_p->graph->no_states), sizeof(int), fp);

	/* global number of first state */
	my_fwrite(&(class_p->first_state_no), sizeof(int), fp);

	/* number of preempting edges */
	my_fwrite(&(class_p->no_preempt_arcs), sizeof(int), fp);

	/* number of isomorphic csmcs */
	my_fwrite(&(class_p->no_total_csmcs), sizeof(int), fp);

	/* is the class acyclic ? */
	acyclic = !(class_p->bits & CYCLIC);
	my_fwrite(&acyclic, sizeof(char), fp);

	/* has the class branching topology ? */
	branch = (class_p->bits & BRANCH_TOP);
	my_fwrite(&branch, sizeof(char), fp);

	/* informations about branching topology */
	if (branch)
	{	/* number of edges */
		my_fwrite(&(class_p->branch_n), sizeof(int), fp);
		/* number of levels */
		my_fwrite(&(class_p->branch_k), sizeof(int), fp);
	}

	/* write the Q matrix */
	scan_list(n_walk, class_p->graph->first_node)
	{	t_walk = n_walk->orig_mark;

		if (!acyclic)
		{	/* last state of a strongly connected component ? */
			d_char = n_walk->bits & LAST_OF_SCC;
			my_fwrite(&d_char, sizeof(char), fp);
		}

		/* assemble edges */
		if (sort_edges == NULL)
		{	max_edges = 100;
			sort_edges = (edge *)emalloc(max_edges * sizeof(edge));
		}
		if (max_edges <= 2 * t_walk->no_exp_paths)
		{	max_edges = 2 * (t_walk->no_exp_paths + 1);
			sort_edges = (edge *)realloc(sort_edges, max_edges * sizeof(edge));
		}

		/* diagonal entry */
		sort_edges[0].to_node = (void *)((long)t_walk->number);
		sort_edges[0].bits = 0;
		sort_edges[0].rate = -t_walk->sum_of_rates;

		e_num = 1;
		p_num = n_walk->no_succ;
		scan_list(e_walk, n_walk->successors)
		{	tn_walk = (node *)e_walk->to_node;
			t_walk = tn_walk->orig_mark;
			if (e_walk->bits & PREEMPTING)
			{
				j = p_num--;
			}
			else 
			{	j = e_num++;
			/* global number of state */
			sort_edges[j].to_node = (void *)((long)t_walk->number);
			}
			/* rate */
			sort_edges[j].rate = e_walk->rate;

			if (branch)
				my_fwrite(&(t_walk->number), sizeof(int), fp);
		}
		qsort(sort_edges, e_num, sizeof(edge), edgecomp);
		/* number of normal edges */
		my_fwrite(&e_num, sizeof(int), fp);
		scan_array(i, e_num)
		{	/* state number */
			my_fwrite(&(sort_edges[i].to_node), sizeof(int), fp);
			/* rate */
			my_fwrite(&(sort_edges[i].rate), sizeof(double), fp);
		}
		/* number of preempting edges */
		p_num = n_walk->no_succ-p_num;
		my_fwrite(&p_num, sizeof(int), fp);
		for (i = n_walk->no_succ; i < p_num + n_walk->no_succ; i++)
		{	/* rate */
			my_fwrite(&(sort_edges[i].rate), sizeof(double), fp);
		}
	}

	/* number of tau - isomorphic csmcs */
	/* insert here for multiple tau transient solution */
	my_fwrite(&(class_p->no_tau_csmcs), sizeof(int), fp);

	preempted = (tang_marking **)emalloc(class_p->no_preempt_arcs * sizeof(tang_marking *));

	scan_list(tau_walk, class_p->Tau_CSMCs)
	{	/* number of csmcs with the same tau */
		i = tau_walk->no_isom_csmcs + 1;
		my_fwrite(&i, sizeof(int), fp);

		/* relative tau value */
		my_fwrite(&(tau_walk->tau), sizeof(double), fp);

		/* send data about isomorphic csmcs with the same tau */
		send_iso_data_to_rand(preempted, tau_walk->graph, tau_walk->bits, fp);
		scan_list(iso_walk, tau_walk->Iso_CSMCs)
		    send_iso_data_to_rand(preempted, iso_walk->graph, iso_walk->bits, fp);
	}

	efree(preempted);
}
