/****************************************************************************
*                                                                           *
*  Programm       :     derive_RES.c                                        *
*                                                                           *
*  Parameter      :     Name des DSPN                                       *
*                                                                           *
*  Rueckgabewert  :     0 --> Programm erfolgreich beendet                  *
*                       1 --> Fehler waehrend der Programmausfuehrung       *
*                                                                           *
*  Zweck          :     Errechnung der Werte der Resultatparameter          *
*                                                                           *
*                                                                           *
*  Programmierer  :     Enrik Baumann (C-Version (second version))          *
*                                                                           *
*  Datum          :     Juni 1991                                           *
*									    *
*  Modifiziert	  :	Armin Zimmermann				    *
*									    *
*  Datum	  :	Maerz 1993					    *
*                                                                           *
*  Modifiziert	  :	Joachim Schneider				    *
*									    *
*  Datum	  :	July 1994 					    *
*                                                                           *
*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <malloc.h>

#define  maxtoken 65535
#define  maxplace 255

extern int through_main();	/* from derive_TPUT.c */

char    mpdname[100], netname[100], tpdname[100],
	rgname[100],  staname[100];

FILE    *net;     	/*  The DSPN description  */
FILE    *sta;     	/*  The file containing output statistics  */
FILE    *rg_fp;   	/*  The Tangible Reachability Set  */
FILE    *mpdf_fp; 	/*  Tangible Markings' Steady-state Probability  */
FILE    *tpdf_fp; 	/*  Tokens' PDF in each place  */

static struct
{	unsigned val;
}  rg;

double	mpdf, tpdf;
int	no_places;
int	*notok;
int	toptan;
double	*mprob;
double	tprob[maxtoken + 1];
int	transient;
int	M[maxplace + 1];
int	now_marking;

int
act_marking(pla)
	int	pla;
{
	return notok[(now_marking-1)*no_places + pla];
}

/*  The following procedure initializes the program data structures  */
void
init(argc, argv)
	int	argc;
	char	*argv[];
{ 
	char	buf[50];
	int	i, akttan;
   
	if (argc < 2)
	{	fprintf(stderr, "ERROR: missing parameters !\n");
		exit(1);
	}
	strcpy(netname, argv[1]);	strcat(netname, ".TN");
	strcpy(mpdname, argv[1]);	strcat(mpdname, ".PROBMARK");
	strcpy(tpdname, argv[1]);	strcat(tpdname, ".PROBTOK");
	strcpy(rgname,  argv[1]);	strcat(rgname,  ".RG");
	strcpy(staname, argv[1]);	strcat(staname, ".RESULTS");
	if ((net = fopen(netname, "r")) == NULL)
	{	fprintf(stderr, "Can't open file %s for reading\n", netname);
		exit(1);
	}
	no_places = 0;
	while(!no_places)
	{	fscanf(net, "%s", buf);
		if (strcmp(buf, "PLACES:"))
			continue;
		fscanf(net, "%s", buf);
		no_places = atoi(buf);
		if (feof(net))
			exit(1);
	}

	/* read net.rg to get toptan */
	if ((rg_fp = fopen(rgname, "r")) == NULL)
	{	fprintf(stderr, "Can't open file %s for reading\n", rgname);
		exit(1);
	}
	toptan = 0;
	while (!feof(rg_fp))
	{	for (i = 0; i <= (no_places-1); i++)
		{	if (fread(&rg, sizeof(rg), 1, rg_fp) != 1)
				if (ferror(rg_fp))
				{	fprintf(stderr, "\nERROR: on reading to file %s !\n",
									 rgname);
					exit(1);
				}
			if (feof(rg_fp))
				break;
		}
		if (feof(rg_fp))
			break;
		toptan++;
	}
	fclose(rg_fp);
	notok = (int *)calloc(toptan*no_places+1, sizeof(int));
	if(notok == NULL)
	{	fprintf(stderr, "\nERROR in gst: not enough memory for notok[]\n");
		exit(1);
	}
	/* read net.rg again to fill notok[] */
	if ((rg_fp = fopen(rgname, "r")) == NULL)
	{	fprintf(stderr, "Can't open file %s for reading\n", rgname);
		exit(1);
	}
	akttan = 0;
	while (!feof(rg_fp))
	{	for (i = 0; i <= (no_places-1); i++)
		{	if (fread(&rg, sizeof(rg), 1, rg_fp) != 1)
				if (ferror(rg_fp))
				{	fprintf(stderr, "\nERROR: on reading to file %s !\n",
									 rgname);
					exit(1);
				}
			if (feof(rg_fp))
				break;
			notok[akttan*no_places+i] = rg.val;
		} /*  for  */
		if (feof(rg_fp))
			break;
		akttan++;
	}

	/* read net.PROBMARK */
	if ((mpdf_fp = fopen(mpdname, "r")) == NULL)
	{	fprintf(stderr, "Can't open file %s for reading\n", mpdname);
		exit(1);
	}
	i = 0;
	fread((char *)&mpdf, sizeof(double), 1, mpdf_fp);
	fread((char *)&mpdf, sizeof(double), 1, mpdf_fp);
	transient = (mpdf > 0.0);
	mprob = (double *)calloc(toptan+1, sizeof(double));
	if (mprob == NULL)
	{	fprintf(stderr, "\nERROR in gst: not enough memory for mprob[]\n");
		exit(1);
	}
	for (i = 1; i <= toptan; i++)
	{	if (fread((char *)&mpdf, sizeof(double), 1, mpdf_fp) != 1)
		{	if (ferror(mpdf_fp))
			{	fprintf(stderr, "\nERROR: on reading to file %s !\n", mpdname);
				exit(1);
			}
		}
		mprob[i] = mpdf;
	} 
} /* init  */

/*	compute the token probabilities */
void
computeprob()
{
	int	i,j,k;
	int	maxpt[maxplace+1], minpt[maxplace+1];

	for (i = 1; i <= no_places; i++)
	{	for (j = 0; j <= maxtoken; j++)
			tprob[j] = 0.0;
		maxpt[i] = 0;
		minpt[i] = maxtoken;
		for (j = 1; j <= toptan; j++)
		{	k = notok[(j-1)*no_places+i-1];
			if (k > maxpt[i])
				maxpt[i] = k;
			if (k < minpt[i])
				minpt[i] = k;
			tprob[k] = tprob[k] + mprob[j];
		}
		tpdf = (double)minpt[i];
		fwrite((char *)&tpdf, sizeof(double), 1, tpdf_fp);
		tpdf = (double)maxpt[i];
		fwrite((char *)&tpdf, sizeof(double), 1, tpdf_fp);
		for (j = minpt[i]; j <= maxpt[i]; j++)
		{	tpdf = tprob[j];
			fwrite((char *)&tpdf, sizeof(double), 1, tpdf_fp);
		}
	}
}

double epsilon = 1.0e-16;
unsigned digits; /* number of digits to display (needed in $model_ERC.c) */

/*	main procedure of derive_RES */
main(argc, argv)
	int	argc;
	char	*argv[];
{  
	int	i, j;
	double	r, p;

	signal(SIGFPE,SIG_IGN);
	
	if(argc>2)
	  epsilon = (double)atof(argv[2]);
	for( digits = 0, p = epsilon; p < 1.0; digits++, p *= 10.0 );

	init(argc, argv);

	if ((sta = fopen(staname, "w")) == NULL)
	{	printf("Can't open file %s for writing\n", staname);
		exit(1);
	}
	if (toptan > 0)
	{	if ((tpdf_fp = fopen(tpdname, "w")) == NULL)
		{	printf("Can't open file %s for writing\n", tpdname);
			exit(1);
		}
		computeprob();
		for (now_marking = 1; now_marking <= toptan; now_marking++)
			sum_prob(mprob[now_marking]);
		calc_results(sta);
		fclose(tpdf_fp);
	} 
	free(notok);
	free(mprob);
	fclose(net);
	fclose(mpdf_fp);
	fclose(rg_fp);

	
	through_main(argc, argv, sta);		/* calculate throughputs */
	fclose(sta);

	exit(0);
}
