/*******************************************************************************
Modul   	:	derive_TPUT.c

Purpose		: 	Berechnen der Throughput-Werte fuer Transitionen

Project Director:	Christoph Lindemann TU Berlin

Programmer	:	Christian Luehe

History		:	release 1: Maerz 1993
*                                                                           *
*  Modifiziert	  :	Joachim Schneider				    *
*									    *
*  Datum	  :	July 1994 					    *
*******************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <errno.h>

#ifdef __STDC__
static void Perror (char *m);
static void Error (char *m);
static int getNumberTranss(FILE *fp);
#else
static void Perror();
static void Error();
static int getNumberTranss();
#endif

#ifdef __STDC__
through_main (int argc, char *argv[], FILE *res_file)
#else
through_main (argc, argv, res_file)
    int argc;
    char *argv[];
    FILE *res_file;
#endif
{
    FILE *net_fp, *probmark_fp, *rates_fp;
    char net_path[MAXPATHLEN],
         probmark_path[MAXPATHLEN],
         rates_path[MAXPATHLEN];
    struct stat sbuf, sbuf_net;
    int toptan, i, k, notrans;
    double probmark, *throughput;
    unsigned char notrmrk;
    struct RatesType {
	unsigned char notr;
	double        rate;
    } rates;


    fprintf(res_file, "\nTHROUGHPUT RESULTS OF TIMED TRANSITIONS\n");
    fprintf(res_file, "(Throughput of immediate transitions have not been computed and are denoted by 0.0)\n");

    if (argc<2) 
	Error("Deriving throughput not possible - missing NET-filename");

    sprintf(net_path,"%s.TN",
	    argv[1]);
    if (stat(net_path,&sbuf_net) < 0)
	Perror(net_path);

    sprintf(probmark_path,"%s.PROBMARK",
	    argv[1]);
    if (stat(probmark_path,&sbuf) < 0)
	Perror(probmark_path);
    
    /* IS PROBMARK NEWER THAN NET? */
    if (sbuf.st_mtime < sbuf_net.st_mtime)
	Error("Deriving throughput not possible - \
NET-file is newer than results\n\
\tSOLVE the Net first!");

    toptan=(sbuf.st_size/sizeof(double))-2;

    sprintf(rates_path,"%s.RATES",
	    argv[1]);
    if (stat(rates_path,&sbuf) < 0) 
	Perror(rates_path);

    if ((net_fp = fopen(net_path,"r")) == NULL)
	Perror(net_path);

    /* NUMBER OF TRANSITIONS */
    notrans = getNumberTranss(net_fp);

    fclose(net_fp);

    if ((throughput=(double *)calloc(notrans,sizeof(double)))==NULL)
	Error("not enough memory.");

    if ((probmark_fp = fopen(probmark_path,"r")) == NULL)
	Perror(probmark_path);

    if ((rates_fp = fopen(rates_path, "r")) == NULL)
	Perror(rates_path);

    if (fread((char *)&probmark, sizeof(double), 1, probmark_fp) != 1) 
	Perror(probmark_path);
    if (fread((char *)&probmark, sizeof(double), 1, probmark_fp) != 1) 
	Perror(probmark_path);

    for (i=1;i<=toptan;i++) {

	if (fread((char *)&probmark, sizeof(double), 1, probmark_fp) != 1) 
	    Perror(probmark_path);
	if (fread((char *)&notrmrk, sizeof(unsigned char),1,rates_fp) != 1)
	    Perror(rates_path);

	for (k=1;k<=notrmrk;k++) {
	    if (fread((char *)&rates, sizeof(struct RatesType),1,rates_fp) != 1)
		Perror(rates_path);
	    throughput[rates.notr] += probmark / rates.rate;

	} /*for k<=notrmrk*/
	
    } /*for i<=toptan*/

    fclose(probmark_fp);
    fclose(rates_fp);    
    
    for (i = 0; i < notrans; i++)
	fprintf(res_file, "%.15f\n", throughput[i]);
    
    return 0;
    
} /*main*/

#ifdef __STDC__
static void Error(char *m)
#else
static void Error(m)
    char *m;
#endif
{
    printf("ERROR: %s\n", m);
    exit (1);
}

#ifdef __STDC__
static void Perror(char *m)
#else
static void Perror(m)
    char *m;
#endif
{
    perror(m);
    exit (1);
}

#ifdef __STDC__
static int getNumberTranss(FILE *fp)
#else
static int getNumberTranss(fp)
    FILE *fp;
#endif
{
    int notrans,x;
    char s[1000];
    
    while (1)
    {	fscanf(fp, "%s", s);
	if(!strcmp(s, "TRANSITIONS:"))
	{	fscanf(fp, "%d", &notrans);
		return notrans;
	}
    }
}
