/**************************************************************************
 File:		expoly.h

 Purpose:	include-file for expoly.c and rspnrand.c
		contains data structure of an expolynomial distribution
		and headers of functions to manipulate expolynomial
		distributions

 Programmer:	Reinhard German

 History:	Release 1: June 2, 1993
**************************************************************************/

#define MAXINDEX    5000      /* maximum size of array of factors */   
#define MAXRUNS     5         /* maximum number of runs */

#define MINVALUE    (double )1.0e-300
#define MAXVALUE    (double )1.0e300


/**************************************************************************
  internal representation of an expolynomial distribution:

  recursive structure consisting of three hierarchy levels
  intervals on the real time axis: 0 =< l =< L
  powers of x with exponent m    : 0 =< m =< M
  exponentials with rate lambda_n: n =< n =< N
**************************************************************************/

typedef struct time_l {        /* time interval l */
  double x1;                   /* beginning of the interval */
  double x2;                   /* end of the interval (if < infinity) */
  double p;                    /* point mass */
  struct power_m *power;       /* list of powers */
  struct time_l *next;         /* next interval */
} time_l;

typedef struct power_m {       /* power with exponent m */
  unsigned m;                  /* exponent */  
  struct expo_n *expo;         /* list of exponentials */
  struct power_m *next;        /* next power */
} power_m;

typedef struct expo_n {        /* exponential with rate lambda_n */
  double lambda;               /* rate */ 
  double b;                    /* constant corresponding to 1-PDF */
  double c;                    /* constant corresponding to pdf */
  struct expo_n *next;         /* next exponential */
} expo_n;

/**************************************************************************
 function headers:
**************************************************************************/

void print_expoly();
int scan_expoly();
time_l *set_det_expoly();
time_l *set_uniform_expoly();
void poisson_prob();
void calculate_factors();
void print_factors();
