/*****************************************************************************

	Programm	:	graph

	Komponente	:	graph_main.c
				Hauptprogramm und globale Variablen

	Parameter	:	Optionen [-of]
				Netzname

	Rueckgabewert	:	0 -> Programm erfolgreich beendet
				1 -> es trat ein Fehler auf

	Aufgabe		:	graph ermittelt den reduzierten
				Erreichbarkeitsgraphen und
				schreibt ihn in Files oder Sockets

	Programmierer	:	Armin Zimmermann

	Datum		:	Juni 1992

        Aenderung       :       Juni 1995 (Patrick Drescher)

*****************************************************************************/

/* #define GRAPHONLY */		/* obtain only the reachability graph */
/* #define MEMDEBUG */		/* obtain information about mem. alloc. */

#undef _POSIX_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <time.h>
#if ! defined (__hpux) && ! defined (__NetBSD__)
#include <alloca.h>
#endif
#include <string.h>
#include <sys/types.h>
#include <errno.h>

#include "graph_types.h"			/* definitions and types */
#include "portab.h"

#define true  1
#define false 0

#define USAGE "Usage: graph [-ofm] structname epsilon exec_mode\n"
#define COUNT_DYNAMIC				/* count dynamic memory allocation */

extern void generate_rg();			/* from graph_timed.c */
extern void set();
extern void execute_rand_processes ();		/* from execute_rand_processes.c */
extern int execute_erand_processes ();          /* from execute_erand.c */
extern time_t start_time;
void write_tmpfiles();
extern void csmc_main();			/* from csmc_main */
extern void obtain_randinfos();
extern int  no_normal_classes;
extern void show_rusage();
		/******* global variables *******/

int no_places;			/* number of places in the net */
int no_trans;			/* number of transitions in the net */
int no_subnets;			/* number of immediate transition subnets */
int no_det_trans;		/* number of deterministic transitions */
int no_gen_trans;               /* number of general transitions */
int last_class_id;              /* important for .EMCii & .CONVii for gen. trans. */
int no_tang_markings;		/* number of tangible markings */
int no_exp_markings;            /* number of exponential tangible markings */
int no_absorb_states;		/* number of absorbing markings */
int *no_van_submarks;		/* number of vanishing submarkings */

t_sub		timed;		/* timed transitions and disconnected places */
subnet		*subnets;	/* immediate transition subnets */
tang_node 	*r_graph;	/* reduced reachability graph */
tang_marking 	*new_markings;	/* list of all new markings */
tang_marking 	*exp_markings;	/* list of exponential markings */
tang_marking 	**det_markings;	/* lists of markings with enabled det. trans */
int             *no_nodes;      /* number of markings of the corresponding EMC */


FILE 	*struct_file;		/* input file with structural informations */
FILE	*rates_file;		/* RATES - file (firing rates of timed trans. in markings) */
FILE	*rg_file;		/* RG - file (description of tangible markings) */
FILE	*mexp_file;		/* MEXP - file (firing of exponential transitions) */
FILE	*mdet_file;		/* MDET - files (markoff chains of deterministic trans.) */
FILE	*delta_file;		/* DELTA - files (immediate trans. firing probabilities) */
int	out_bit;		/* information output option */
int	mem_bit;		/* memory save option from environment */

/* Globale Variablen, welche von exec_rands.c importiert werden: */
char		write_files = false;	/* Flag, Files .MDET_<k> und .DELTA_<k> schreiben ?
					   (fuer execute_rand_processes.c) */

#ifdef MEMDEBUG

#define	NO_ACCOUNTS	100

typedef struct
{	int num, val;
}  sort_type;

int	max[NO_ACCOUNTS], now[NO_ACCOUNTS], max_sum;

/*	comp is the comparison function for sortspace */
int
comp(a, b)
	sort_type *a, *b;
{
	if (a->val > b->val)
		return -1;
	return (a->val < b->val);
}
#endif

/*	emalloc tries to allocate required memory space and
	tests if it was successfully allocated; if not => exit */
char *
emalloc(bytes, account)
	int bytes, account;
{
	int *point;

	if (!bytes)
		return NULL;
#ifdef MEMDEBUG
	point = (int *)malloc((unsigned)(bytes + 2*sizeof(int)));
#else
	point = (int *)malloc((unsigned)(bytes));
#endif
	if (point == NULL)
	{	printf("The reduced reachability graph of this DSPN\n");
		printf("cannot be generated because of lacking main memory.\n");
		if (mem_bit)
			printf("Retry on a workstation with more main memory.\n");
		exit(2);
	}

#ifdef MEMDEBUG
	*(point++) = account;
	*(point++) = bytes;
	now[account] += bytes;
	sum = 0;
	for(i = 0; i < NO_ACCOUNTS; i++)
		sum += now[i];
	if (sum > max_sum)
	{	max_sum = sum;
		for (i = 0; i < NO_ACCOUNTS; i++)
			max[i] = now[i];
	}
#endif
	return (char *)point;	
}

/*	efree frees a memory pointer */
void
efree(point)
	char	*point;
{
	int	*hpoint = (int *)point;

	if (point == NULL)
		return;
#ifdef MEMDEBUG
	bytes = *(--hpoint);
	acc = *(--hpoint);
	now[acc] -= bytes;
	if (now[acc] < 0)
	{	printf("ERROR in free (MEMDEBUG)\n");
		exit(1);
	}
	
#endif
	free(hpoint);
}

/*	skip_line skips to the beginning of the next line */
char
skip_line(f)
	FILE *f;
{       char buffer[100];
	fgets(buffer, 100, f);
	return buffer[0];
}

/*	read_sparse reads a sparse row */
sp_place *
read_sparse(f)
	FILE *f; 
{
	int count, i;
	sp_place *point;

	fscanf(f, "%d ", &count);	/* number of entries */
	point = (sp_place *)emalloc((count+1) * sizeof(sp_place), 1);
	point[0].num = count;		/* first entry.num = number of entries */
	for(i = 1; i<=count; i++)
	{	fscanf(f, "%d %d", &(point[i].num), &(point[i].value));
		fscanf(f, "%d %d", &(point[i].plu_id), &(point[i].min_id));
	}
	return point;
}

/*	read_set reads a set of transitions */
byte *
read_set(f)
	FILE *f;
{
	int count, i, b;
	byte *point;

	fscanf(f, "%d ", &count);	/* read number of entries */
	point = (byte *)emalloc((count+1) * sizeof(byte), 2);
	point[0] = count;		/* first entry: number of entries */
	for(i = 1; i <= count; i++)
	{	fscanf(f, "%d", &b);
		point[i] = b;
	}
	return point;
}

/*	read_struct reads the structural description file net.STRUCT */
void
read_struct(s_file)
	FILE *s_file;
{
	int	i, num, max;
	timed_t	*t_tra;
	imm_t	*i_tra;
	place	*pla;
	subnet	*sub;

	no_det_trans = no_gen_trans = 0;
	skip_line(s_file);		/* skip first line (comment) */
	fscanf(s_file, "%d %d", &no_trans, &no_places);
	fscanf(s_file, "%d", &max);
	timed.no_trans = max;
	timed.transitions = (timed_t *)emalloc(max * sizeof(timed_t), 3);
	timed.no_markings = 0;
	for(num = 0; num < max; num++)
	{	t_tra = &(timed.transitions[num]);
		fscanf(s_file, "%s %d %d ", t_tra->name, &t_tra->number, &t_tra->type);
		if (t_tra->type) /* DETERMINISTIC (or general) */
			t_tra->det_no = no_det_trans++;
                if (t_tra->type==2) /* GENERAL DISTRIBUTED */
                        no_gen_trans++;
                if (t_tra->type==1 && t_tra->infinite) /* DETERMINISTIC and INFINITE SERVER */
		  {
                  printf("Cannot compute infinite server of deterministic type ! \n");
                  exit(1);
		  }
		fscanf(s_file, "%d %d ", &t_tra->infinite, &t_tra->mark_ind);
		fscanf(s_file, "%lf\n", &t_tra->value);
		t_tra->i_matrix = read_sparse(s_file);
		t_tra->h_matrix = read_sparse(s_file);
		t_tra->c_matrix = read_sparse(s_file);
	}
	fscanf(s_file, "%d\n", &max);		/* disconnected places */
	timed.no_places = max;
	timed.places = (place *)emalloc(max * sizeof(place), 4);
	for(num = 0; num < max; num++)
	{	pla = &(timed.places[num]);
		fscanf(s_file, "%s %d ", pla->name, &pla->number);
		fscanf(s_file, "%d %d ", &pla->m0, &pla->bound);
		fscanf(s_file, "%d", &pla->bits);
	}
	fscanf(s_file, "%d", &timed.memory);
	/* read immediate transition subnets */
	fscanf(s_file, "%d", &no_subnets);
	subnets = (subnet *)emalloc(no_subnets * sizeof(subnet), 5);
	for(num = 0; num < no_subnets; num++)
	{	sub = &(subnets[num]);
		sub->end_mind = NULL;
		sub->no_markings = 0;
		fscanf(s_file, "%d", &max);
		sub->no_trans = max;
		sub->transitions = (imm_t *)emalloc(max * sizeof(imm_t), 6);
		for(i = 0; i<sub->no_trans; i++)
		{	i_tra = &(sub->transitions[i]);
			fscanf(s_file, "%s %lf", i_tra->name, &i_tra->prob);
			fscanf(s_file, "%d %d", &i_tra->wgt_id, &i_tra->ena_id);
			i_tra->i_matrix = read_sparse(s_file);
			i_tra->h_matrix = read_sparse(s_file);
			i_tra->c_matrix = read_sparse(s_file);
			i_tra->enas = read_set(s_file);
			i_tra->diss = read_set(s_file);
		}
		fscanf(s_file, "%d", &max);
		sub->no_places = max;
		sub->places = (place *)emalloc(max * sizeof(place), 7);
		for(i = 0; i < max; i++)
		{	pla = &(sub->places[i]);
			fscanf(s_file, "%s %d ", pla->name, &pla->number);
			fscanf(s_file, "%d %d ", &pla->m0, &pla->bound);
			fscanf(s_file, "%d", &pla->bits);
		}
		fscanf(s_file, "%d", &sub->memory);
		fscanf(s_file, "%d", &sub->no_ecs);
		sub->ecs = (byte **)emalloc(sub->no_ecs * sizeof(byte *), 8);
		for(i = 0; i < sub->no_ecs; i++)
			sub->ecs[i] = read_set(s_file);
	}
	
}

/*	print_info shows some information about the reachability graph */
void
print_info(net_name)
	char	*net_name;
{
	int	sn;
	char	aux_name[FILE_NAME];
	FILE	*aux_f;

	printf("\nThis %sSPN ", no_det_trans ? "D" : "G");
	printf("contains %d subnet%s.\n", no_subnets, no_subnets!=1 ? "s" : "");
	if (!mem_bit)
	{	int h = 0;
		printf("Number of significant submarkings:\n");
		for (sn = 0; sn < no_subnets; sn++)
		{	printf("\tsubnet %d : %d\n", sn+1, no_van_submarks[sn]);
			h += no_van_submarks[sn];
		}
		printf("Overall number of significant submarkings: %d\n", h);
	}

	printf("Number of tangible markings: %d\n", no_tang_markings);
	if (no_det_trans)
	{	printf("In %d markings are ", no_exp_markings);
		printf("only exponential transitions enabled.\n");
	}

	if (no_absorb_states)
	{	printf("%d absorbing state%s detected.\n", no_absorb_states,
			no_absorb_states > 1 ? "s" : "");
		strcpy(aux_name, net_name);
		strcat(aux_name, ".AUX");
		if ((aux_f = fopen(aux_name, "a")) == NULL)
		{	printf("ERROR: could not open %s for appending.\n", aux_name);
			exit(1);
		}
		fprintf(aux_f, "ABSORBING STATES %d\n", no_absorb_states);
		fclose(aux_f);
	}
	if (!out_bit)
		return;
	if (timed.no_places)
		printf("disconnected places: %d submarkings.\n", timed.no_markings);
	if (mem_bit)
		return;
	for(sn = 0; sn < no_subnets; sn++)
		printf("subnet %d (with place %s): %d submarkings.\n",
			sn, subnets[sn].places[0].name, subnets[sn].no_markings);
}

/*	the main procedure handles options and file access and
	calls subordinated procedures */
int main(argc, argv)
	int argc;
	char ** argv;
{
	char struct_name[FILE_NAME], net_name[FILE_NAME];
	char rg_name[FILE_NAME], rates_name[FILE_NAME];
	char options[10];
	int i, arg_num = 1;
	double epsilon;
        char exec_mode[10];

  clock(); /* Prozessuhr starten.. ( fuer show_rusage() )*/
 
#ifdef MEMDEBUG
	for (i=0; i<NO_ACCOUNTS; i++)
		now[i] = max[i] = 0;
	max_sum = 0;
#endif

	if (argc == 1)
	{	printf("ERROR: no netname found\n");
		printf(USAGE);
		exit(1);
	}
	out_bit = write_files = mem_bit = 0;
	if (argv[1][0] == '-')
	{	arg_num++;
		strcpy(options, argv[1]+1);
		for(i = 0; i < strlen(options); i++)
			switch(options[i])
			{	case 'o':	out_bit = 1;
						break;
				case 'm':	mem_bit = 1;
						break;
				case 'f':	write_files = 1;
						break;
				default:	printf(USAGE);
						puts ("  o = print informations");
						puts ("  f = write .MDET_<k> and .DELTA_<k>-Files");
						puts ("  m = save memory");
						exit(1);
			}
	}

	if (argc <= arg_num)
	{	printf("ERROR: no netname found\n");
		printf(USAGE);
		exit(1);
	}

	strcpy(net_name, argv[arg_num]);
	strcpy(struct_name, argv[arg_num]);
	strcpy(rates_name, argv[arg_num]);
	strcpy(rg_name, argv[arg_num]);
	
	arg_num++;
	
	if(argc<=arg_num)
	{	printf("ERROR: no epsilon found\n");
		printf(USAGE);
		exit(1);
	}
	else sscanf(argv[arg_num],"%lg",&epsilon);

        arg_num++;
	
	if(argc<=arg_num)
	{	printf("ERROR: no exec_mode found\n");
		printf(USAGE);
		exit(1);
	}
	else sscanf(argv[arg_num],"%s",exec_mode);
        if(strcmp(exec_mode,"sequential") && strcmp(exec_mode,"parallel"))
	{       printf("ERROR: unknown exec_mode\n");
                printf("exec_mode: [sequential, parallel]\n");
                exit(1);
        }	

	strcat(struct_name, ".STRUCT");
	if ((struct_file = fopen(struct_name, "r")) == NULL)
	{	printf("ERROR: could not open %s for reading\n", struct_name);
		exit(1);
	}
	strcat(rates_name, ".RATES");
	if ((rates_file = fopen(rates_name, "w")) == NULL)
	{	printf("ERROR: could not open %s for writing\n", rates_name);
		exit(1);
	}
	strcat(rg_name, ".RG");
	if ((rg_file = fopen(rg_name, "w")) == NULL)
	{	printf("ERROR: could not open %s for writing\n", rg_name);
		exit(1);
	}

	printf("\nGENERATING THE REDUCED REACHABILITY GRAPH ");
	if (mem_bit)
		printf("USING MEMORY EFFICIENT ALGORITHM ...\n");
	else	printf("USING TIME EFFICIENT ALGORITHM ...\n");
	fflush(stdout);

	read_struct(struct_file);	/* read the structural description file */

	new_markings = NULL;
	exp_markings = NULL;
	no_tang_markings = 0;
	no_van_submarks = (int *)emalloc(no_subnets * sizeof(int));
	for (i=0; i<no_subnets; i++)
		no_van_submarks[i] = 0;
	r_graph = NULL;
	det_markings = (tang_marking **)emalloc(no_det_trans * sizeof(tang_marking *), 9);
	no_nodes = (int *)emalloc(no_det_trans * sizeof(int));
	for(i = 0; i < no_det_trans; i++)
	{	det_markings[i] = NULL;
		no_nodes[i] = 0;
	}

	start_time = time(NULL);
	generate_rg(subnets, &timed);		/* calculate the reduced reachability graph */
	print_info(net_name);				/* show some information */
#ifdef DEBUG
	show_graph_tree(subnets, &timed);
#endif
#ifndef GRAPHONLY
	csmc_main(net_name);			/* calculate CSMCs */
#endif
	printf("\n");
	show_rusage();
#ifdef GRAPHONLY
	printf("Obtaining RG only; stop.\n");
	exit(0);
#endif
	/* write intermediate files, if -f option is set (debugging etc) */
	if (write_files)
		write_tmpfiles(net_name);

	/* obtain informations about rand efforts */
	obtain_randinfos();

	if (no_normal_classes)
		execute_rand_processes (net_name, epsilon, exec_mode);

        if (no_gen_trans)      
                execute_erand_processes (net_name, epsilon);

	fclose(rates_file);
	fclose(struct_file);
	fclose(rg_file);

#ifdef MEMDEBUG
	{	sort_type sortspace[NO_ACCOUNTS];
		int p;
		printf("\nGraph MEMDEBUG information:\n");
		printf("Maximum memory space allocated: %d\n", max_sum);
		for (i = 0; i < NO_ACCOUNTS; i++)
		{	sortspace[i].num = i;
			sortspace[i].val = max[i];
		}
		qsort((char *)(sortspace), NO_ACCOUNTS, sizeof(sort_type), comp);
		printf(" Account     Space   Percent\n");
		for (i = 0; (i < NO_ACCOUNTS) && sortspace[i].val; i++)
		{	p = (int)((100.0 * sortspace[i].val) / max_sum);
			if (!p)
				break;
			printf("%8d  %8d  %6d%%\n", sortspace[i].num, sortspace[i].val, p);
		}
	}
#endif
	exit(0);
}
