/*	test region for graph */

#include <stdio.h>

#include "graph_types.h"			/* definitions and types */
#include "graph_extern.h"

extern int *decode_submarking();
extern void *emalloc();

void
show_marking(sub, mark)
	subnet *sub;
	sub_marking *mark;
{
	int	*tokens, i, j;

	if (mark->paths == NULL)
	{	printf("Marking is new.\n");
		return;
	}
	printf("Marking: (%d paths)\n", mark->no_paths);
	for(i=0; i<mark->no_paths; i++)
	{	tokens = decode_submarking(sub->no_places, sub->places, mark->paths[i].marking);
		printf("	with prob %f to:\n	", mark->paths[i].prob);
		for(j=0; j<sub->no_places; j++)
			printf("%s:%d ", sub->places[j].name, tokens[j]);
		printf("\n");
	}
}
	
void
show_sub_tree(sub, node, level)
	subnet	*sub;		/* corresponding subnet */
	sub_node *node;
	int level;
{
	int pla;
	sp_pointer *lookup;

	if (level >= sub->no_places)
	{	show_marking(sub, (sub_marking *)node);
		return;
	}
	if (node == NULL)
		return;
	if (node->bound)
		for(pla = 0; pla < node->number; pla++)
		{	lookup = (sp_pointer *)node->next;
			printf("level %d %s token %d\n", level, 
				sub->places[level].name, lookup[pla].index);	
			show_sub_tree(sub, lookup[pla].next, level+1);
		}
	else
		for(pla = 0; pla <= sub->places[level].bound; pla++)
			if (node->next[pla] != NULL)
			{	printf("level %d %s token %d\n", level, 
						sub->places[level].name, pla);
				show_sub_tree(sub, node->next[pla], level+1);
			}
}

void
show_tree(sub)
	subnet	*sub;		/* corresponding subnet */
{
	printf("\nTree:\n");
	if (sub->tree == NULL)
	{	printf("tree is empty\n");
		return;
	}
	show_sub_tree(sub, sub->tree, 0);
}

void
show_disc_marking(sub, mark)
	subnet *sub;
	word *mark;
{
	int	*tokens, i, j;

	printf("Marking:\n");
	tokens = decode_submarking(sub->no_places, sub->places, mark);
	for(j=0; j<sub->no_places; j++)
		printf("%s:%d ", sub->places[j].name, tokens[j]);
	printf("\n");
}
	
void
show_disc_tree(sub, node, level)
	t_sub	*sub;
	sub_node 	*node;
	int level;
{
	int pla;
	sp_pointer *lookup;

	if (level >= sub->no_places)
	{	show_disc_marking(sub, (word *)node);
		return;
	}
	if (node == NULL)
		return;
	if (node->bound)
		for(pla = 0; pla < node->number; pla++)
		{	lookup = (sp_pointer *)node->next;
			printf("level %d %s token %d\n", level, 
				sub->places[level].name, lookup[pla].index);	
			show_disc_tree(sub, lookup[pla].next, level+1);
		}
	else
		for(pla = 0; pla <= sub->places[level].bound; pla++)
			if (node->next[pla] != NULL)
			{	printf("level %d %s token %d\n", level, 
						sub->places[level].name, pla);
				show_disc_tree(sub, node->next[pla], level+1);
			}
}

void
show_disc(sub)
	t_sub	*sub;		/* corresponding subnet */
{
	printf("\nDisconnected Tree:\n");
	if (sub->tree == NULL)
	{	printf("tree is empty\n");
		return;
	}
	show_disc_tree(sub, sub->tree, 0);
}

void
show_tang_marking(sub, tsub, mark)
	subnet *sub;
	t_sub  *tsub;
	tang_marking *mark;
{
	int	*tokens, i, j;
	tang_marking *to;

	printf("\ntangible Marking: (%d)\n", mark->number);
	tokens = (int *)emalloc(no_places * sizeof(int), 49);
	global_marking(mark->mark, mark->disc, tokens);
	print_global_marking(tokens);
	if (mark->det)
	{	printf("det. Trans. %s is enabled\n", tsub->transitions[mark->det_num].name);
		printf("%d deterministic firing path(s):\n", mark->no_det_paths);
		for(i=0; i<mark->no_det_paths; i++)
		{	to = (tang_marking *)(mark->det_paths[i].marking);
			printf("  with prob %f to %d\n", mark->det_paths[i].prob, to->number);
		}
	}
	else	printf("no det. Trans. enabled\n");
	printf("%d exponential firing path(s):\n", mark->no_exp_paths);
	for(i=0; i<mark->no_exp_paths; i++)
	{	to = (tang_marking *)(mark->exp_paths[i].marking);
		printf("  with rate %f to %d\n", mark->exp_paths[i].prob, to->number);
	}
	printf("sum of rates: %f\n", mark->sum_of_rates);
}
	
void
show_tang_tree(sub, tsub, node, level)
	subnet	*sub;
	t_sub	*tsub;
	tang_node *node;
	int level;
{
	int pla;
	sp_pointer *lookup;

	if (level > no_subnets)
	{	show_tang_marking(sub, tsub, (tang_marking *)node);
		return;
	}
	for(pla = 0; pla < node->number; pla++)
	{	lookup = (sp_pointer *)node->next;
		show_tang_tree(sub, tsub, lookup[pla].next, level+1);
	}
}

void
show_graph_tree(sub, tsub)
	subnet	*sub;
	t_sub	*tsub;
{
	extern tang_node *r_graph;
	printf("\nreachability graph:\n");
	if (r_graph == NULL)
	{	printf("tree is empty\n");
		return;
	}
	show_tang_tree(sub, tsub, r_graph, 0);
}
