/*
*                                                                           *
*  Modifiziert	  :	Joachim Schneider				    *
*									    *
*  Datum	  :	July 1994 					    *
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/param.h>

#include "fgetline.h"

/*#define debug*/

#ifdef __STDC__
extern int md_arc_mult(int id);
int getRidOf (int fd, char* s);
#else
extern int md_arc_mult();
int getRidOf ();
#endif

char **nameToIdTable,
     type[128],
     trans[128],
     place[128],
     name[3*128];

int no_of_places, *marking;
int no_of_md_arcs;

#ifdef __STDC__
void getNameToIdTable(char *netname)
#else
void getNameToIdTable(netname)
     char *netname;
#endif
{
    FILE *f;
    size_t md_arcs=0;
    int id=0;
    char *line;
    size_t len;
    char path[MAXPATHLEN];
#ifdef debug
        fprintf(stderr, "\tMDC getNameToIdTable() started.\n");
#endif

    if (!netname) {
	fprintf(stderr,"getNameToIdTable: no netname given.\n");
	exit (1);
    }
    
    sprintf(path, "%s.TN", netname);
    if (!(f = fopen(path, "r"))) {
	fprintf(stderr, "getNameToIdTable: could not open %s.\n", path);
	perror("");
	exit(1);
    }
    
    /* GET NUMBER OF MARK-DEP. ARCS */
    while (line = fgetline(f, &len))
      if (strstr(line, "CARDINALITY"))
	md_arcs++;
    
    no_of_md_arcs = md_arcs;

    if (md_arcs && !(nameToIdTable = (char**)calloc(md_arcs, sizeof(char*)))) {
	fprintf(stderr, "getNameToIdTable: could not allocate table.\n");
	exit(1);
    }
    
    rewind (f);
    while ((line = fgetline(f, &len)) && (id < md_arcs)) {
#ifdef debug
        fprintf(stderr, "\t%s\n", line);
#endif
        if (sscanf (line, "CARDINALITY %s %s %s", type, trans, place) == 3) {
	    sprintf(name, "%s%s%s", type, trans, place);
	    nameToIdTable[id] = (char*) calloc(strlen(name)+1, sizeof(char));
	    strcpy(nameToIdTable[id], name);
	    id++;
	}
    }
    fclose (f);
}

#ifdef __STDC__
int nameToId (char *name)
#else
int nameToId (name)
     char *name;
#endif
{
    int id;
#ifdef debug
        fprintf(stderr, "\tMDC nameToId() started for '%s'.\n", name);
#endif    

    for (id = 0; id < no_of_md_arcs; id++)
      if (!strcmp(name, nameToIdTable[id]))
	return (id+1);

    fprintf(stderr, "\tMDC: nameToId(): Name \"%s\" not found.\n", name);
    exit (1);
}

#ifdef __STDC__
void allocMarking(char *netname)
#else
void allocMarking(netname)
     char *netname;
#endif
{
    FILE *f;
    int places = -1;
    char *line;
    size_t len;    
    char path[MAXPATHLEN];

    if (!netname) {
	fprintf(stderr,"\tMDC: allocMarking(): no netname given.\n");
	exit (1);
    }    

    sprintf(path, "%s.TN", netname);
    if (!(f = fopen(path, "r"))) {
	fprintf(stderr, "\tMDC: allocMarking(): could not open %s.\n", path);
	perror("");
	exit(1);
    }
    
    while (line = fgetline (f, &len))
      if (sscanf(line, "PLACES: %d", &places) == 1)
	break;
    
    if (places == -1) {
	fprintf(stderr,
		"\tMDC: allocMarking(): no entry \"PLACES: xxx\" found.");
	exit(1);
    }
    
    no_of_places = places;
    marking = (int*) calloc((size_t)places, sizeof(int));
}

#ifdef __STDC__
void getActMarking(void)
#else
void getActMarking()
#endif
{
    int i, value;

#ifdef debug
    fprintf(stderr, "\tMDC: getActMarking() started\n");
#endif

    for (i = 0; i < no_of_places; i++)
      if (fscanf(stdin, "%d", &value) == 1)
	marking[i] = value;
      else {
	  fprintf(stderr, 
		  "\tMDC: getActMarking(): convertion to integer failed.\n");
	  exit(1);
      }
}

#ifdef __STDC__
int act_marking(int pl_id)
#else
int act_marking(pl_id)
     int pl_id;
#endif
{
#ifdef debug
    fprintf(stderr, "\tMDC: act_marking() started\n");
#endif
    if (pl_id >= no_of_places) {
	fprintf(stderr, 
	       "\tMDC: act_marking(): place id >%d< is not in range.\n", pl_id);
	exit(1);
    }

    return marking[pl_id];
}

#ifdef __STDC__
main (int argc, char **argv)
#else
main (argc, argv)
     int argc;
     char **argv;
#endif
{
    int id, c;
    char s[3*128];
    char netName[MAXPATHLEN], *endName;

#ifdef debug
    fprintf(stderr, "MDC---------------MDC startet--------------started\n");
#endif

    /* name of the executable is <net>_MDC ...
       so we need no parameter for the netname */

    if (argv[0][0] == '/') /* absolute pathname */
	strcpy(netName, argv[0]);
    else   /* set current working dir bevor argv[0] */
	sprintf(netName, "%s/%s", getcwd(NULL, MAXPATHLEN), argv[0]);
    /* strip "_MDC" from netName */
    
    endName = strrchr(netName, '_');
    *endName = '\0';

#ifdef debug
    fprintf(stderr, "\tMDC netname is : %s.\n", netName);
#endif

    getNameToIdTable(netName);

    allocMarking(netName);

    while (1) {
#ifdef debug
	fprintf(stderr, "\tMDC scaninng name...\n", netName);
#endif
	if (fscanf(stdin, "%s %s %s", type, trans, place) != 3) {
	    fprintf(stderr, "MDC  error : couldnt read type, trans, place.\n");
	    exit(1);
	}

	sprintf(s, "%s%s%s", type, trans, place);
	id = nameToId(s);
	
#ifdef debug
	fprintf(stderr,"\tid is: %d.\n", id);
#endif
	
	getActMarking();
	
	c = md_arc_mult(id);
	
#ifdef debug
	fprintf(stderr,"\tcardinality is: %d.\n", c);
#endif
	sprintf (s, "%d\n", c);
	getRidOf(fileno(stdout), s);
    }
}

#ifdef __STDC__
int getRidOf (int fd, char* s)
#else
int getRidOf (fd, s)
    int fd;
    char* s;
#endif
{
    int i, w = 1, len = strlen(s);
    
    for (i = 0; i < len && w != -1; i++) {
        while (!(w=write (fd, &(s[i]), 1)));
    }

    if (w == -1) {
        perror ("ERROR in write() ");
        return w;
    }
    else
        return 0;
}
