%{
/* **********************************************************************
*                                                                              
*  Programm         :     parsdfunc.y (YACC-Quellprogramm)
*                                                                              
*  Aufgabe          :     schreibt struct_time_l
*                                                                              
*                                                                              
*  Programmierer    :     Joerg W. Mitzlaff                               
*                                                                              
*  Datum            :     Juni 1993   
*
*************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "parsdfunc.h"
#include "mem.h"

extern FILE *yyin;

time_l *zaehler;
time_l  *zwerg;   /* eingefuehrt, da bei yacc-internen variablen $x nicht direkt 
		     auf structure-komponenten zugegriffen werden kann */
power_m *zwerg2;  /* dito */


/**********************/
/**********************/

  time_l *erg = NULL;      /* nach parsierung zeigt er auf die gew. stru. */
  int    DET_OR_UNI;       /* UNIFORM oder DETERMINISTIC function in auswertung; wird gesondert integriert */

/**********************/
/**********************/


#ifdef linux
void yyerror(char *string)
{
	fprintf(stderr,string);
}
#endif

time_l *free_whole_stru(p)
     time_l *p;
{
   time_l *i;
   time_l *iold;      
   power_m *j;
   power_m *jold;
   expo_n *k;
   expo_n *kold;

   i = p;
   while(i!=NULL)
   {
      j = i->power;
      while(j!=NULL)
      {
	 k = j->expo;
	 while(k!=NULL)
	 {
	    kold = k;
	    k = k->next;
	    free(kold);
	 }
	 jold = j;
	 j = j->next;
	 free(jold);
      }
      iold = i;
      i = i->next;
      free(iold);
   }
   return(NULL);
}

power_m *zero_tail()
{

    return(make_m_stru(0,
		       make_n_stru(0.0, (double)UNDEF, 0.0, NULL),
		       NULL));
}

time_l *copy_all_intervall(p1)
     time_l *p1;
{
     time_l *basis;
     time_l *neubas;
     time_l *retu;
    
    neubas = make_l_stru(0.0, 0.0, 0.0, NULL, NULL);    
    basis = p1;
    retu = neubas;

    neubas->x1    = basis->x1;
    neubas->x2    = basis->x2;
    neubas->p     = basis->p;
    neubas->power = copy_all_power(basis->power);
    
    while(basis->next!=NULL)
    {
	basis  = basis->next;         /* erhoehen */
	neubas->next = make_l_stru(0.0, 0.0, 0.0, NULL,NULL);
        neubas = neubas->next;
	neubas->x1    = basis->x1;
	neubas->x2    = basis->x2;
	neubas->p     = basis->p;
	neubas->power = copy_all_power(basis->power);
    }

    return(retu);
}


power_m *copy_all_power(p1)
     power_m *p1;
{
    power_m *basis;
    power_m *neubas;
    power_m *retu;
    
    neubas = make_m_stru(0,NULL,NULL);    
    basis = p1;
    retu = neubas;

    neubas->m    = basis->m;
    neubas->expo = copy_all_expo(basis->expo);
    
    while(basis->next!=NULL)
    {
	basis  = basis->next;         /* erhoehen */
	neubas->next = make_m_stru(0,NULL,NULL);
        neubas = neubas->next;
	neubas->m    = basis->m;
	neubas->expo = copy_all_expo(basis->expo);
    }
    return(retu);
}

expo_n *copy_all_expo(p1)
     expo_n *p1;
{
    expo_n *basis;
    expo_n *neubas;
    expo_n *retu;
    
    neubas = make_n_stru(0.0, 0.0, 0.0, NULL);
    
    basis = p1;
    retu = neubas;

    neubas->lambda = basis->lambda;      /* expo kopieren von basis nach retu */
    neubas->b      = basis->b;
    neubas->c      = basis->c;

    while(basis->next!=NULL)
    {
	basis  = basis->next;         /* erhoehen */
	neubas = neubas->next;
	
	neubas = make_n_stru(0.0, 0.0, 0.0, NULL);
	neubas->lambda = basis->lambda;      /*  kopieren von basis nach neubas */
	neubas->b      = basis->b;
	neubas->c      = basis->c;
    }

    return(retu);
}

time_l *melt_intervall(p1, p2)
     time_l *p1;
     time_l *p2;   /* p2 wird in p1 reingesteckt */
{
    time_l *i;      /* geht ueber alle time-stru's von p2 */
    time_l *j;      /* geht ueber alle time-stru's von p1 dessen Intervall <= als jeweiliges p2 ist*/
    time_l *lastj;
    time_l *pneu;
    int    int_eingeordnet;
     
    i = p2;
    do{
	int_eingeordnet = False;
	while(!int_eingeordnet)
	{
	    j = p1;
	    lastj = NULL;
	    /* p2-Intervalle fangen immer am Anfang eine p1-Intervalls an!!! */
	    /* Einstiegspunkt in p1-Intervall finden */
	    while((i->x1 != j->x1) && (j->next!=NULL))
	    {
		lastj = j;
		j = j->next;
	    }

	    if(j->next==NULL)
	    {                               /* Sonderfall j->x2=UNENDLICH (p2-Int.-anf. gesucht, wo p1 keine Int.mehr hat */
		if(lastj==NULL)
		    p1 = i;
		else
		    lastj->next = i;
	        i->p = j->p;
		return(p1);                 /* >>>>>>>>>>>>>>>>>>>>>>>> sofort zurueck (Ziel erreicht) */
	    }
	    if(i->next==NULL)
	    {  				    /* Sonderfall i->x2=UNENDLICH */
		break; /* patch:das erste war auch schon das letzte */
	    }
	    else
		if(i->x2 == j->x2)
		{                           /* Fall 1 (identisch)*/
		    j->p = j->p + i->p;
		    melt_power(j->power,copy_all_power(i->power));
		    int_eingeordnet = True;
		} 
		else
		    if(i->x2 < j->x2)
		    {                       /* Fall 2 (j spaltet sich)*/
			pneu = make_l_stru(i->x2, j->x2, j->p, copy_all_power(j->power), NULL); /* neuer 2. Teil */
			j->x2 = i->x2;								/* neuer 1. Teil */
			melt_power(j->power,copy_all_power(i->power));
			pneu->next = j->next;
			j->next = pneu;  /* verkettung */
			int_eingeordnet = True;
		    }
		    else /*  i->x2 > j->x2 */
		    {   	                    /* Fall 3 (mit go back)*/
			
			melt_power(j->power,copy_all_power(i->power));
			i->x1 = j->x2;    /* verkleinerung von i-Intervall */
			int_eingeordnet = False;
		    }
	}
	if(i->next==NULL)
	    break;  /* patch:das erste war auch schon das letzte */
	else
	    i = i->next;
    }while(i->next!=NULL); /* das letzte "dummy"-intervall von i (p2) wird weggelassen... */
    if(i->p!=0)		   /* ...ausser es gibt eine Punktmasse am Ende */
    {
	j = p1;
	/* p2-Intervalle fangen immer am Anfang eine p1-Intervalls an!!! */
	/* Einstiegspunkt in p1-Intervall finden */
	while(i->x1 != j->x1) 
	    j = j->next;
	j->p =  j->p + i->p;
    }
    return(p1);  
}

void append_one_power(p1, p2)
     power_m *p1;
     power_m *p2; /* ein p2 an p1 angehaengt */
{						    
        power_m *append;

        append = p1;
        while(append->next != NULL)
	    append = append->next;
        append->next = p2;
        p2->next = NULL;
}

void append_one_expo(p1, p2)
     expo_n *p1;
     expo_n *p2; /* ein p2 an p1 angehaengt */
{						    
        expo_n *append;

        append = p1;
        while(append->next != NULL)
	    append = append->next;
        append->next = p2;
        p2->next = NULL;
}

void add_or_append_all_expos(p1, p2)
     expo_n *p1;
     expo_n *p2; /* alle expos von p2 an p1-expos angehaengt */
{						    /* p2 freigeben */
    expo_n *i;
    expo_n *j;
    expo_n *weiter;
    
    i = p2;
    do{                            /* fuer alle expos in p2 */
	j = p1;
	while((j->lambda!=i->lambda) && (j->next!=NULL))
	{
	    j = j->next;           /* stimmt ein lambda aus p1 ueberein? */
	}
	if(j->lambda == i->lambda) /* ja */
	{
	    j->c += i->c;
	    j->b += i->b;
	    i = i->next;           /* hochzaehlen */
	}
	else                       /* nein: neues lambda anhaengen */
	{
	    weiter = i->next;
	    i->next = NULL;        /* ist letztes in Schlange */
	    j->next = i;	   /* anhaengen an p1-expos */

	    i = weiter;             /* hochzaehlen */
	}
    }while(i != NULL);

    /* free all p2 expos */
}

power_m *melt_power(p1, p2)
     power_m *p1;
     power_m *p2;          /* p2 in p1 rein */
{
    power_m *i;   /* geht ueber alle powerstru's von p1 */
    power_m *j;   /* geht ueber alle powerstru's von p2 */
    power_m *nextj;
    
    j       = p2;              /* j ist ausserer zaehler */
    do{
	nextj = j->next;         /* muss man sich merken, da j verloren geht. */

        i       = p1;            /* i ist innerer zahler */
	while( !((i->next == NULL) || (i->m == j->m)))
	      i = i->next;
	
	if(i->m == j->m)   /* gleicher exponent p2->m in p1-stru's */

	    add_or_append_all_expos(i->expo,copy_all_expo(j->expo));
	else
	    append_one_power(i,j);    /* ganz neuer exponent in p2, der in p1 noch nicht vorkam */

	j = nextj;		      /* weiterzaehlen */
    }while( j != NULL);
    return(p1);
}
    
power_m *negate_all_c(p1)
     power_m *p1;    /* Arg. p1 wird durchgereicht */
  {
    power_m *i;   /* geht ueber alle powerstru's */
    expo_n  *j;   /* geht ueber alle expostru's */

    i = p1;
    do{	
        j = i->expo;
	do{
	    j->c = j->c * (-1);  /* faktor negieren */
	    j = j->next;
	}while(j != NULL);
	i = i->next;
    }while(i != NULL);
    
    return(p1);
}

time_l  *make_l_stru(x1, x2, p, p1, p2)
     double x1;
     double x2;
     double p;
     power_m *p1;
     time_l *p2;
{
    time_l *ret;

    ret=(time_l *) Malloc(sizeof(time_l));

    ret->x1 = x1;
    ret->x2 = x2;
    ret->p  = p;
    ret->power = p1;
    ret->next  = p2;

    return ret;
}
 
power_m *make_m_stru(m, p1, p2)
     unsigned int m;
     expo_n *p1;
     power_m *p2;
{	
    power_m *ret;

    ret=(power_m *) Malloc(sizeof(time_l));

    ret->m = m;
    ret->expo = p1;
    ret->next = p2;

    return ret;
}

expo_n  *make_n_stru(l, b, c, p1)
     double l;
     double b;
     double c;
     expo_n *p1;
{
    expo_n *ret;

    ret=(expo_n *) Malloc(sizeof(time_l));

    ret->lambda = l;
    ret->b  = b;
    ret->c  = c;
    ret->next = p1;

    return ret;
}

%}

/*----------------------------------------------------------------*/

%union
{
  double        	dval;
  unsigned      	uval;
  struct time_l  	*tp;
  struct power_m 	*mp;
}

/* konstante Token aus Worten o. Zeichenketten*/
%token TRANSITION DETERMINISTIC UNIFORM XPOT EPOT

/*... aus Zeichen R I x ( ) [ ] , ; + -    */
%token R I X ROPA RCPA EOPA ECPA COM SEM PLUS MINUS

/* variable Token */
%token POS_INT POS_REAL NEG_REAL REAL TRANSITION_NAME

%left PLUS MINUS

/* Typen der var. Token und non-terminals */
%type <uval> POS_INT
%type <dval> POS_REAL NEG_REAL real
%type <tp> pmf_definition pmf_expression impulse rectangle
%type <mp> expolynomial 
%type <mp> expoly_expr

/*----------------------------------------------------------------*/

%%

pmf_definition:   DETERMINISTIC ROPA POS_REAL RCPA SEM        
                      { erg=set_det_expoly($3);
		        DET_OR_UNI = True;
		      }
                | UNIFORM ROPA POS_REAL COM POS_REAL RCPA SEM  
                      {   if($5<=$3)
			  {
			     fprintf(stderr,"ERROR(logic): in intervall-limits.\n");	
			     exit(1);
			  }
 			  else
			     erg=set_uniform_expoly($3,$5);
			  DET_OR_UNI = True;
		      }
                | pmf_expression SEM
                      { erg = $1;
		        DET_OR_UNI = False;
		      }
;
pmf_expression:   pmf_expression PLUS  pmf_expression    
                      { $$ = melt_intervall($1,$3); }
                | pmf_expression MINUS pmf_expression
                      {     /* if any time-stru: MINUS a*I[x] dann error */
			  zaehler = $3;
			  do{	
			      if(zaehler->p>0)
			      {
				  fprintf(stderr,"ERROR(math.): impulse with negative pointmass detected.\n");			     	      exit(1);
			      }
			      zaehler = zaehler->next;
			  }while(zaehler!=NULL);
			  zwerg = $3;
			  negate_all_c( zwerg->next->power );
			  melt_intervall($1, zwerg);
		      }
                | impulse
                      { $$=$1; }
                | rectangle     
                      { $$=$1; }
;
impulse:          I EOPA POS_REAL ECPA  
                      { if($3==0)
			{                      /* Intervall-anfang bei Null ! */
			    $$=make_l_stru(0.0, (double) UNENDLICH, 1.0, zero_tail(), NULL); 
			}
		        else
			{
			    $$=make_l_stru(0.0, $3, 0.0, zero_tail(), 
				       make_l_stru($3, (double) UNENDLICH, 1.0, zero_tail(), NULL));
			}}
                | POS_REAL I EOPA POS_REAL ECPA
                      { if($4==0)
			{                      /* Intervall-anfang bei Null ! */
			    $$=make_l_stru(0.0, (double)UNENDLICH, $1, zero_tail(), NULL); 
			}
		        else
			{
			    $$=make_l_stru(0.0, $4, 0.0, zero_tail(), 
					   make_l_stru($4, (double)UNENDLICH, $1, zero_tail(), NULL)); 
			}}
;
rectangle:        R EOPA POS_REAL COM POS_REAL ECPA
                      {  if($5<=$3)
			 {
			     fprintf(stderr,"ERROR(logic): in intervall-limits.\n");
			     exit(1);
			 }
                         else
			     if($3==0)
			     {                      /* Intervall-anfang bei Null ! */
				 $$=make_l_stru(0.0, $5, 0.0, 
						make_m_stru(0,                            /* bei x^0 */
							    make_n_stru(0.0, (double)UNDEF, 1.0, NULL),  /* ist c=1 */
							    NULL),
						make_l_stru($5, (double)UNENDLICH, 0.0, zero_tail(), NULL)); 
			     }
			     else
			     {
				 $$=make_l_stru(0.0, $3, 0.0, zero_tail(), 
						make_l_stru($3, $5, 0.0, 
							    make_m_stru(0,                            /* bei x^0 */
									make_n_stru(0.0, (double)UNDEF, 1.0, NULL),  /* ist c=1 */
									NULL),
							    make_l_stru($5, (double)UNENDLICH, 0.0, zero_tail(), NULL))); 
			     }}
                | expolynomial R EOPA POS_REAL COM POS_REAL ECPA
                      {  if($4==0)
			 {                      /* Intervall-anfang bei Null ! */
			     $$=make_l_stru(0.0, $6, 0.0, $1, 
					 make_l_stru($6, (double)UNENDLICH, 0.0, zero_tail(), NULL)); 
			 }
			 else
			 {
			     $$=make_l_stru(0.0, $4, 0.0, zero_tail(), 
					    make_l_stru($4, $6, 0.0, $1, 
							make_l_stru($6, (double)UNENDLICH, 0.0, zero_tail(), NULL))); 
			 }}

;
expolynomial:     expolynomial PLUS  expolynomial
                     { $$ = melt_power( $1,$3); }
                | expolynomial MINUS expolynomial
                     { $$ = melt_power( $1, negate_all_c($3) ); }
                | ROPA expolynomial RCPA
                     { $$=$2; }
                | real                                   
                     { $$=make_m_stru(0,
                                     make_n_stru(0.0, (double)UNDEF,$1,NULL),
                                     NULL); }
                | expoly_expr                            
                     { $$=$1; }
                | real expoly_expr
                     { zwerg2 = $2;
		       (zwerg2->expo)->c = $1;
                       $$ = $2; }
;
expoly_expr:      XPOT POS_INT                  
                     {     $$=make_m_stru($2,
                                          make_n_stru(0.0,(double)UNDEF,1.0,NULL),
                                          NULL); }
                | X
                     { $$=make_m_stru(1,
                                     make_n_stru(0.0, (double)UNDEF,1.0,NULL),
                                     NULL); }
    
                | EPOT ROPA NEG_REAL X RCPA        
                     { $$=make_m_stru(0,
                                     make_n_stru($3*(-1.0), (double)UNDEF, 1.0, NULL),
                                     NULL); }
                | XPOT POS_INT EPOT ROPA NEG_REAL X RCPA 
                     {     $$ = make_m_stru($2,
				  	    make_n_stru($5*(-1.0), (double)UNDEF,1.0, NULL),
					    NULL); }

                | X EPOT ROPA NEG_REAL X RCPA 
                     { $$ = make_m_stru(1,
					make_n_stru($4*(-1.0), (double)UNDEF, 1.0, NULL),
					NULL); }
;
real:             POS_REAL { $$ = $1; }
                | NEG_REAL { $$ = $1; }
;
%%

/*----------------------------------------------------------------*/

int file_read(filename,tname,formbuf)     /* formbuf hat groesse buflen */
     char *filename;
     char *tname;
     char *formbuf;          
{
   FILE *fp;

   char buf[BUFLEN], *p;
   int len, rt = False, found = False;

   /* open file */
   if( (fp = fopen(filename,"r")) == NULL )
   {
#ifdef DEBUG
      printf("Unable to open '%s'\n", filename);
#endif
      return( False );
   }

   /* read lines */
   while( !feof(fp) )
   {
      fgets(buf,BUFLEN,fp);

      /* skip comments & empty lines */
      if( buf[0] == '\n' || buf[0] == '#' || buf[0] == '\0' )
      {
         rt = False;
         continue;
      }

      /* remove newline char */
      buf[strlen(buf) - 1] = '\0';

      /* look for keyword */
      if( !strncmp(buf,"TRANSITION",10) )
      {
         /* cut transition name */
         for( p = buf + 10; *p == ' '; p++ );

         /* match transition name */
         if( !strcmp(p,tname) )
	 {
            rt = True;
            found = True;
            formbuf[0] = '\0';
	 }
         else
            rt = False;
      }
      else
         if( rt )
	 {
            /* append line to function */
            strcat(formbuf,buf); strcat(formbuf,"\n");
	 }

      /* flush line */
      buf[0] = '\0';
   }

   /* close file */
   fclose(fp);
   
#ifdef DEBUG
   printf("Found = %d\n", found);
   printf("Formbuf = %s\n", formbuf);
#endif

   return( found );
}


time_l *parse_pmf(filename,tname)     /* oberflaechenunabhaengiger aufruf */
     char *filename;
     char *tname;        
{
   char formbuf[BUFLEN];

   if( file_read(filename, tname, formbuf) == True )
   {
      if( !do_parse(formbuf) )
      {
         if(!DET_OR_UNI)
            integrate(erg,INTEGRATE_NORMAL);
      }
      else
         return(NULL);
   }
   else
      return(NULL);

   return( erg );
}

int do_parse(function)  /* aus integrierter version aufgerufen */
     char *function;
{
   char buf[128];
   FILE *fp;
   int i;

   /* build filename, open file */
   sprintf(buf,"/tmp/pmf_func.%u", getpid());
   if( (fp = fopen(buf,"w")) == (FILE *)NULL )
   {
      fprintf(stderr,"do_parse: unable to write function (%s)\n", buf);
      exit( -1 );
   }
   fprintf(fp,"%s",function);
   fclose(fp);

   /* redirect input to yyin */
   freopen(buf,"r",yyin); 

   /* create free structure */
   erg = free_whole_stru(erg);

   /* parse input */
   i = yyparse();

   /* remove temporary file */
   switch( fork() )
   {
   case -1:
      fprintf(stderr,"do_parse: unable to fork\n");
      exit( -1 );
   case 0:
      execl("/bin/rm","/bin/rm",buf,(char *)NULL);
      fprintf(stderr,"do_parse: unable to execute /bin/rm %s\n", buf);
      exit( -1 );
   }

   return( i );
}
