/********************************************************************
Modul :             discontinuities.c

Purpose :           organisiert die Verwaltung der Diskontinuitaeten
                    in binaerem Baum :
                    -  sucht kleinste Diskontinuitaet in Baum
                    -  legt die von dieser ausgeloesten Diskontinuitaeten
                       in Baum ab ( groesste zuerst !)
                    -  loescht die vorher gesuchte Diskontinuitaet

Project Director :  Reinhard German

Programmer :        Armin Heindl

History :           30.7.95 ( Diplomarbeit )
*********************************************************************/

#include <math.h>
#include <stdio.h>                /* fuer NULL */
#include "frontend.h"  
#include "Options.h"

extern TransientOptions transientOptions;
extern int ac_no_detT;
extern double EXT_PRECISION;

static double last_disc = 0.0; /* Zeitpunkt, ueber den hinaus keine Diskontinuitaeten mehr
				  beruecksichtigt werden muessen                         */

/************************************************************************/
void set_border( daten_basis)
/* setzt last_disc als Obergrenze fuer zu betrachtende Diskontinuitaetszeitpunkte */

solve_basis *daten_basis;
{
  last_disc = transientOptions.transient_time + transientOptions.precision + daten_basis->detMatrices[0].det_firing_time;
}
/****************************************************************************/
void build_up_tree( daten_basis, root, max_element)
/* baut einen initialen binaeren Baum auf, in dem die zukuenftigen, zur Laufzeit
   ermittelten Diskontinuitaetszeitpunkte gespeichert werden */

solve_basis *daten_basis;
tnode **root;
tnode **max_element;
{
  int i;
  tnode *wanderer;

  *root = (tnode *)emalloc(sizeof(tnode));
  (*root)->discontime = daten_basis->detMatrices[0].det_firing_time;
  (*root)->left = NULL;
  wanderer = *root;
  for( i=1; i<ac_no_detT; i++) {
    if (fabs(daten_basis->detMatrices[i].det_firing_time - daten_basis->detMatrices[i-1].det_firing_time) > EXT_PRECISION) { 
      wanderer->right = (tnode *)emalloc(sizeof(tnode));
      wanderer = wanderer->right;
      wanderer->discontime = daten_basis->detMatrices[i].det_firing_time;
      wanderer->left = NULL;
    };
  };
  wanderer->right = NULL;
  *max_element = wanderer;
}
/****************************************************************************/
double get_next_discontinuity( daten_basis, time, root, max_element)
/* Rueckgabewert : Zeitspanne zwischen gerade erreichter Diskontinuitaet und der 
                   naechsten, die es zu suchen gilt */
/* *time enthaelt eingangs die gerade erreichte Diskontinuitaet,
                  ausgangs die zu suchende Diskontinuitaet */

solve_basis *daten_basis;
double *time;
tnode **root;  /* *root niemals NULL */
tnode **max_element; /* *max_element niemals NULL */
{
  int i;
  tnode *min_element, *wanderer;
  double period; /* Rueckgabewert */
  double new_value;

  /* suche naechste Diskontinuitaet : */
  min_element = *root;
  while (min_element->left != NULL) {
    min_element = min_element->left;
  };
  /* min_element zeigt auf Blatt mit kleinstem Wert */
  
  period = min_element->discontime - (*time);
  *time = min_element->discontime;

  new_value = (*time) + daten_basis->detMatrices[ac_no_detT-1].det_firing_time;
  if (new_value <= last_disc) { 
    (*max_element)->right = (tnode *)emalloc(sizeof(tnode));
    *max_element = (*max_element)->right;
    (*max_element)->discontime = new_value;
    (*max_element)->left = NULL;
    (*max_element)->right = NULL;
  };

  for( i=(ac_no_detT-2); i>=0; i--) {
    if (fabs(daten_basis->detMatrices[i+1].det_firing_time - daten_basis->detMatrices[i].det_firing_time) > EXT_PRECISION) {
      new_value = (*time) + daten_basis->detMatrices[i].det_firing_time;
      if (new_value <= last_disc) {
	wanderer = *root;
	
	for( ;; ) {
	  if (fabs(new_value - wanderer->discontime) < EXT_PRECISION) 
	    break;
	  if (new_value > wanderer->discontime) {
	    if (wanderer->right == NULL) {
	      wanderer->right = (tnode *)emalloc(sizeof(tnode));
	      wanderer = wanderer->right;
	      wanderer->discontime = new_value;
	      wanderer->left = NULL;
	      wanderer->right = NULL;
	      break;
	    }
	    else 
	      wanderer = wanderer->right;
	  }
	  else { /* new_value < wanderer->discontime */
	    if (wanderer->left == NULL) {
	      wanderer->left = (tnode *)emalloc(sizeof(tnode));
	      wanderer = wanderer->left;
	      wanderer->discontime = new_value;
	      wanderer->left = NULL;
	      wanderer->right = NULL;
	      break;
	    }
	    else
	      wanderer = wanderer->left;
	  };
	};
      }; /* Ende des last_disc-ifs */
    }; /* Ende des fabs-ifs */
  };

  if (min_element == (*root)) {
    *root = (*root)->right;
    efree( min_element);
  }
  else {
    { 
      tnode *before_min;
      
      before_min = *root;
      while ( before_min->left != min_element)
	before_min = before_min->left;
      before_min->left = min_element->right;
      efree( min_element);
    };
  };
  
  return( period);
}
/*********************************************************************************/

