/*******************************************************************************
Modul:              easy.c

Purpose:            easy case (GSPN) Loesungsverfahren

Project Director:   Reinhard German

Programmer:         Joerg W. Mitzlaff

History:            10.94 (Dipl.Arbeit)
*******************************************************************************/

/* #define DEBUG  */

#include <stdio.h>
#include <math.h>
#include "frontend.h"
#include "Options.h"
#include "shared.h"
#include "portab.h"

/* Import aus main.c */
extern TransientOptions transientOptions;

/* Import aus shared.c */
extern double ggTeiler();
extern sp_matrix *randomi();
extern double calculate_A_Matrix();
extern FILE *open_solve();
extern void update_solve();
extern void close_solve();

/******************************************************************************
 Funktion:  easy_case()

 Parameter: opt:    transient solution options

 Rueckgabewert: ---

 Zweck:
 *****************************************************************************/
void easy_case(tnethome,modelpath,netname,component)
char *tnethome;
char *modelpath;
char *netname;
solve_basis *component;
{
  FILE *ftmp;
  sp_matrix *exp_Q_EE;   /* e^(Q*step) */
  int i;
  double h;
  int numDatas;
  double maxX;

  if(transientOptions.output_data==Point)
    h = ggTeiler(transientOptions.transient_time,transientOptions.stepsize_result);
  else
    h = transientOptions.stepsize_result;

  printf("stepsize h %s to %g.\n",
	 (transientOptions.stepsize_result!=h ? "has changed" : "was set"),
	 h);

  numDatas = ceil(transientOptions.transient_time / h);
  maxX = numDatas * h;
  
  numDatas ++; /* t=0 ist auch ein messwert */

  ftmp = open_solve(tnethome,modelpath,netname,numDatas,maxX);


#ifdef KOSTEN
  log_kosten(2);
#endif

  exp_Q_EE = randomi(component->Q_EE,transientOptions.stepsize_result);
  del_matrix( &(component->Q_EE) );

#ifdef KOSTEN
  log_kosten(1);
#endif

  update_solve(tnethome,modelpath,netname,ftmp,component->no_markings,component->p_E,0.0,maxX);
  for(i=1; i<numDatas;i++)
  {

#ifdef DEBUG
	printf("p(%lf):\n",i*h);
	show_vector( component->p_E );
#endif

    mult_vector_matrix(component->p_E, exp_Q_EE);
    update_solve(tnethome,modelpath,netname,ftmp,component->no_markings,component->p_E,i*h,maxX);
  }

  close_solve(modelpath,netname,ftmp);
  
#ifdef KOSTEN
  log_kosten(2);
#endif

  del_matrix( &(exp_Q_EE) );
}

/******************************************************************************
 Funktion:  case_i()
              verbesserte Variante des easy_case
 ******************************************************************************/
void case_i(tnethome,modelpath,netname,component)
char *tnethome;
char *modelpath;
char *netname;
solve_basis *component;
{
  FILE *ftmp;

  Randomisation_Over_Time_Line rotili;
  int i,j,k,m,n;
  double h;
  int numDatas;
  double maxX;


  if(transientOptions.output_data==Point)
    h = ggTeiler(transientOptions.transient_time,transientOptions.stepsize_result);
  else
    h = transientOptions.stepsize_result;

  printf("step size h %s to %g.\n",
	 (transientOptions.stepsize_result!=h ? "has changed" : "was set"),
	 h);
  printf("cluster size  was set to %d.\n", transientOptions.cluster_size);
  

  numDatas = ceil(transientOptions.transient_time / h);
  maxX = numDatas * h;
  
  numDatas ++; /* t=0 ist auch ein messwert */

  n = (int) transientOptions.cluster_size;
  m = (int) ( ((double)numDatas) / ((double)n) + 0.5);

  /* precision an cluster anzahl anpassen */
  transientOptions.precision /= m;
  if(m>1)
    printf("randomisation precision has changed to %g.\n",transientOptions.precision);


  ftmp = open_solve(tnethome,modelpath,netname,numDatas,maxX);


  /* ersetze Q Matrix durch A Matrix */
  rotili.qmax_ii = calculate_A_Matrix(component -> Q_EE);
  

#ifdef DEBUG
    printf("least timestep = %lf cluster size = %d number of clusters = %d\n",h,n,m);
#endif

  /* alloziere Speicher */
   rotili.Betas = (double **)emalloc(sizeof(double *) * n);

   rotili.R = (int *)emalloc(sizeof(int) * n);
   
   rotili.L = (int *)emalloc(sizeof(int) * n);
     

  printf("Prepeare Randomisation: ...");
  /* Berechne Betas */
  for(j=0;j<n;j++) /* Schritte eines Clusters */
    call_weighter(rotili.qmax_ii * (j+1) * h, &(rotili.L[j]), &(rotili.R[j]), &(rotili.Betas[j]));
  printf(" Truncation:\tmost right = %u\n", rotili.R[n-1]);

  /* alloziere pi'k Elemente Speicher */
  /* kleinste linke Schranke ist meisst 0, es wird also rotili.L[n-1] == 0 angenommen */
  /* + 1, da von o bis k summiert werden muss */
  rotili.pi_summand = (sp_vectors *)emalloc( sizeof(sp_vectors) * (rotili.R[n-1] + 1) );
  for(i=0;i<=rotili.R[n-1];i++)
    rotili.pi_summand[i] = new_vector();


  /* jetzt gehts richtig los */
  for(i=0;i<m;i++) /* ueber alle cluster */
  {

#ifdef KOSTEN
  log_kosten(2);
#endif

      /* neu aufsetzen */
      del_vector( &(rotili.pi_summand[0]) );
      rotili.pi_summand[0] = copy_vector(component->p_E);   /* pi'0 = p(0) * I = p(0) */
      for(k=1;k <= rotili.R[n-1];k++)  /* pi'k = pi'(k-1) * A */
      {
        del_vector( &(rotili.pi_summand[k]) );
        rotili.pi_summand[k] = copy_vector( rotili.pi_summand[k-1] );
        mult_vector_matrix( rotili.pi_summand[k], component -> Q_EE);

#ifdef DEBUG
	printf("p'%d:\n",k);
	show_vector( rotili.pi_summand[k] );
#endif

      }

#ifdef KOSTEN
  log_kosten(1);
#endif

    for(j=0;j<n;j++) /* Schritte eines Clusters */
    {
      update_solve(tnethome, modelpath, netname, ftmp, component->no_markings, component->p_E, (i*n+j)*h, maxX);

#ifdef DEBUG
	printf("p(%lf):\n",(i*m+j)*h);
	show_vector( component->p_E );
#endif

      clear_vector(component->p_E);
      for(k=rotili.L[j];k <= rotili.R[j];k++)
      {
	/* aufsummieren */ 
	vector_add_mult( component->p_E, rotili.pi_summand[k], rotili.Betas[j][ k-rotili.L[j] ] );
      }
    }
  }
  update_solve(tnethome, modelpath, netname, ftmp, component->no_markings, component->p_E, i*n*h, maxX);
  close_solve(modelpath,netname,ftmp);
}
