/*******************************************************************************
Modul:              frontend.c

Purpose:            liest environment vars., ruft netio auf fuer p(0)
                    und ist sozusagen die "Schaltzentrale"
                    frontend == vor eigentlichem solve-alg.

Project Director:   Reinhard German

Programmer:         Joerg W. Mitzlaff

History:            10.94 (Dipl.Arbeit)
*******************************************************************************/

/* #define DEBUG */
/* #define KOSTEN*/

#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#include "frontend.h"
#include "Options.h"
#include "environment.h"
#include "netio.h"
#include "get_rrg.h"

/* globale Variable */
NetDescr* net;

/* Import aus get_rrg.c */
extern solve_basis *build_matrices();
extern mark_t *marking;
extern int preempted_trans;

/* Import aus get_tmark.c */
extern solve_basis *build_p_Null_vector();
extern void free_tmark_mem();

/* Importe aus solve files */
extern void easy_case();
extern void special_case();
extern int particular_sequence();
extern void runge_kutta_case();
extern void case_i();
extern void case_ii();
extern void case_iii();
extern void adaptive_stepcontrol();

/* Import aus main.c */
extern TransientOptions transientOptions;


/*******************************************************************************/
void PrepeareSolve(netname)
/*******************************************************************************/
String netname;
{
  struct stat buf1,buf2;
  String tnethome;
  String modelpath;
  solve_basis *component;     /* Komponenten der entscheidenden Datenstruktur */
  double t;
  double pt;
  char command[2*MAXPATHLEN];
  int i;
  int statres;

  /* get environment variables */
  if ((tnethome = (String)getenv(DSPNPACKAGE_ENV)) == NULL)
  {
    printf("Environment variable %s not set; can't find multi-solve program",DSPNPACKAGE_ENV);
    exit(-1);
  }
  if ((modelpath = (String)getenv(MODELDIR_ENV)) == NULL)
  {
    printf("Environment variable %s not set; can't find specified model",MODELDIR_ENV);
    exit(-1);
  }

  /* Starte GENERATE_RRG if not exist oder datum .rrg aelter als .TN File */
  /* (proc_TN wird leider umsonst mitaufgerufen ) */

  sprintf(command,"%s/%s.TN",modelpath,netname);
  if(stat(command, &buf1))
  {
    printf("stat error reading .TN file\n");
    exit(1);
  }
  sprintf(command,"%s/%s.rrg",modelpath,netname);  
  statres = stat(command, &buf2);

#ifdef KOSTEN
  log_kosten(-1);
  statres = 1;      /* immer generate_rrg aufrufen ! */
#endif

  if(statres || buf1.st_mtime >= buf2.st_mtime)  /* .TN file neuer als .rrg file ? */
  {
    sprintf(command,"%s/Shared/scripts/GENERATE_RRG -t %s", tnethome, netname);
    system(command); 
  }
  else
  {
    printf("REDUCED REACHABILITY GRAPH is up to date.\n");
  }

#ifdef KOSTEN
  log_kosten(0);
#endif

  /* init solve_basis struct fuer einfachsten Fall: GSPN (ohne detT) */
  component = (solve_basis *)emalloc(sizeof(solve_basis));
  component -> Q_EE = NULL;
  component -> Q_Ed = NULL;
  component -> no_detT = 0;
  component -> detMatrices = NULL;
  component -> no_markings = 0;
  component -> p_E = NULL;
  component -> p_d = NULL;

  /* .rrg file lesen und Matrizen aufstellen; eventuell Fehlermeldung und abbruch */
  component = build_matrices(modelpath,netname,component);

  /* netio lesen */
  net = load_net(netname,NULL);
  if(net -> num_rewards == 0)
  {
    printf("No reward measures found. Define them first!");
    exit(-1);
  }

  printf("Now solving model %s %s time %g with precision %g.\n",
	 netname,
	 (transientOptions.output_data==Duration ? "till" : "at"),
	 transientOptions.transient_time,
	 transientOptions.precision);

  /* .tmark file lesen und p(0) in comp eintragen; eventuell Fehlermeldung und abbruch */
  component = build_p_Null_vector(modelpath,netname,component);


#ifdef DEBUG
  printf("init vector p'E:\n");
  show_vector(component->p_E);
  if(component -> p_d)
    for( i=0 ; i<component->no_detT;i++)
    {
      printf("init vector p'd d=%d\n", i);
      show_vector(component->p_d[i]);
    } 
  else
    printf("p_d vectors have 0 entries\n");
#endif

  /* GSPN CASE ? (nur S'E Zustaende) */
  if(component->detMatrices == NULL)
  {
    if(transientOptions.old_case_iii)
    {
      printf("start case 1 a\n");
      easy_case(tnethome,modelpath,netname,component);
    }
    else
    {
      printf("start case 1 b\n");
      case_i(tnethome,modelpath,netname,component);
    }
  }
  else if( (component->Q_EE == NULL) && (!preempted_trans) && (particular_sequence(component)) ) 
  {
    if(transientOptions.old_case_iii)
    {
      printf("start case 2 a\n");
      special_case(tnethome,modelpath,netname,component);
    }
    else
    {
      printf("start case 2 b\n");
      case_ii(tnethome,modelpath,netname,component);
    }
  }
  else
  {

    if(transientOptions.old_solve_alg)
    {
      if (component->Q_EE == NULL) 
	printf("this net can't be solved with the fixed stepsize algorithm !\n");
      else {
	printf("start old case 3 %s\n",
	       (transientOptions.old_case_iii ? "a" : "b")
	       );
	case_iii(tnethome,modelpath,netname,component);
      };
    }
    else
    {
      printf("start case 3 with adaptive stepsize                 \n");
      printf("please note that in this case the eligible options :\n");
      printf("   * stepsize for output ( adaptive )               \n");
      printf("   * algorithm variant   ( always b )               \n");
      printf("   * internal stepsize   ( adaptive )               \n");
      printf("   * cluster size                                   \n");
      printf("have no effect on the calculations !                \n");
      adaptive_stepcontrol(tnethome,modelpath,netname,component);
    }
  }

  if(transientOptions.keep_files)
  {
    printf("keeping temporary files.\n");
  }
  else
  {
    printf("deleting temporary files.\n");

    sprintf(command,"%s/%s.TransSolve",modelpath,netname);
    unlink(command);
    sprintf(command,"%s/%s.tempcurves",modelpath,netname);
    unlink(command);
    if(transientOptions.output_data==Duration)
      sprintf(command,"%s/%s.RESULTS",modelpath,netname);
    else
      sprintf(command,"%s/%s.curves",modelpath,netname);      
    unlink(command);
    sprintf(command,"%s/%s.tmark",modelpath,netname);
    unlink(command);
    sprintf(command,"%s/%s.rrg",modelpath,netname);
    unlink(command);
    sprintf(command,"%s/%s.genrrg",modelpath,netname);
    unlink(command);
    sprintf(command,"%s/%s.DEFINFO",modelpath,netname);
    unlink(command);
    sprintf(command,"%s/%s_ERC.c",modelpath,netname);
    unlink(command);
    sprintf(command,"%s/%s_MDF.c",modelpath,netname);
    unlink(command);
    sprintf(command,"%s/%s.INV",modelpath,netname);
    unlink(command);
    sprintf(command,"%s/%s.ECS",modelpath,netname);
    unlink(command);
    sprintf(command,"%s/%s.STRUCT",modelpath,netname);
    unlink(command);
    sprintf(command,"%s/%s.pmf",modelpath,netname);
    unlink(command);
    sprintf(command,"%s/%s.LOG",modelpath,netname);
    unlink(command);
    sprintf(command,"%s/%s.MEASURE",modelpath,netname);
    unlink(command);
  }

  free(marking);     /* wird in get_rrg.c erzeugt und dann in verschiedenen Modulen benutzt. */
  free_tmark_mem();  /* von get_tmark.c erzeugte structs */
  free_net(net);
  free(component); 

#ifdef KOSTEN
  log_kosten(-2);
#endif

  printf("The End.\n");
}

