/******************************************************************************
 Program:           get_rrg.c

 Purpose:           .rrg daten einlesen und alle Matrizen aufbauen
                    bei Fehler abbrechen

 Project Director:  Reinhard German

 Programmer:        Joerg W. Mitzlaff

 History:           10.94 (Dipl.Arbeit)
                    angelehnt an EDSPNAnalysis/parse_rrg.c 
*******************************************************************************/
/* #define KOSTEN */

#include <stdio.h>
#include "frontend.h"   /* fuer emalloc */
#include "get_rrg.h"

/* file name buffer size */
#define FNAME_SIZE  256

/* import aus main.c */
extern FILE *kosten_file;

/* exported variables to get_tmark.c und shared.c */
int nmarking = 0;             /* no. of markings */
mark_t *marking = NULL;       /* list of markings and its types */

/* exported variables to new_rk.c and frontend.c*/
int preempted_trans = 0;

/* local variables */
static int nline = 0;                /* no. of lines, also Anzahl der Kanten */
static int nnames = 0;               /* no. of transition names */
static int nexpT = 0;                /* notwendiges special case Kriterium */
static int ndetT = 0;                /* ist neue eindeutige detT nummer */
                                     /* 0 nur S'E Markierungen (Zustaende) */
				     /* sonst Anzahl der detT und damit auch der S'd */
static name_t *names = NULL;         /* list of transition names */
static edge_t *edge = NULL;          /* list of edges and its types */


/* forward definition of private functions */
static void spool();
static void read_fileinfo();
static char *next_word();
static void read_names();
static void read_edges();
static void append_edges();
static void free_mem();


/*******************************************************************************/
solve_basis *build_matrices(modelpath,netname,component)
/*******************************************************************************/
char *modelpath;
char *netname;
solve_basis *component;
{
   FILE *rrg_fp;
   char rrg_file[FNAME_SIZE];
   int i,j;
   int q1;
   int q2;

   /* build filename, open file */
   sprintf(rrg_file,"%s/%s.rrg",modelpath,netname);
   if( (rrg_fp = fopen(rrg_file,"r")) == NULL )
   {
      fprintf(stderr,"Unable to open file '%s'\n", rrg_file);
      exit(-1);
   }

   /* read file info */
   read_fileinfo(rrg_fp);

   component -> no_markings = nmarking;

   /* read transition names from .rrg file nach struct name_t */
   read_names(rrg_fp);

     
   /* liest Kanten und names_t infos nach edge_t */
   read_edges(rrg_fp);


#ifdef KOSTEN
   q1 = 0;
   q2 = 0;
   for(i=0;i<nmarking;i++)
   {
     if(marking[i].typeNr == 0)
       q1++;
     if(marking[i].typeNr == 1)
       q2++;
   }
   fprintf(kosten_file, "num_Q1 %d num_Q2 %d ",q1,q2);
#endif


   /* infos aus mark_t nach edge_t */
   for(i=0;i<nline;i++)
   {
      edge[i].fromtypeNr = marking[edge[i].fromMark].typeNr;
      edge[i].totypeNr = marking[edge[i].toMark].typeNr;
      if(edge[i].fromtypeNr == -1)
	nexpT++;
   }

   /* lege Matrizen fuer S'E  und S'd Zustaende an */
   if(nexpT>0)
   {
     component -> Q_EE = new_matrix();
     component -> p_E = new_vector();
   }
   if(ndetT>0)
   {
     component -> no_detT = ndetT;

     /* Q_Ed Matrizen generieren */
     component -> Q_Ed = (sp_matrices *)emalloc(sizeof(sp_matrices) * ndetT);
     for( i=0; i<ndetT; i++)
     {
       component -> Q_Ed[i] = new_matrix();
     }

     /* alle Q_D* und Delta_d* Matrizen generieren */
     component -> detMatrices = (sp_detMatrices *)emalloc(sizeof(sp_detMatrices) * ndetT);
     for( i=0; i<ndetT; i++)
     {
       /* finde zu einem transitions typ i eine passende kante j  */
       for(j=0;j<nline;j++)
       {
	 if(i == edge[j].typeNr)
	   component -> detMatrices[i].det_firing_time = edge[j].delay;
       }
       component -> detMatrices[i].Q_d = new_matrix();
       component -> detMatrices[i].Delta_dE = new_matrix();

       component -> detMatrices[i].Q_dE = new_matrix();

       /* Q_dc Matrizen generieren (insgesammt ndetT^2 Matrizen!) */
       component -> detMatrices[i].Q_dc = (sp_matrices *)emalloc(sizeof(sp_matrices) * ndetT);
       for( j=0; j<ndetT; j++)
	 {
	   component -> detMatrices[i].Q_dc[j] = new_matrix();
	 }
	 
       /* Delta_dc Matrizen generieren (insgesammt ndetT^2 Matrizen!) */
       component -> detMatrices[i].Delta_dc = (sp_matrices *)emalloc(sizeof(sp_matrices) * ndetT);
       for( j=0; j<ndetT; j++)
       {
	 component -> detMatrices[i].Delta_dc[j] = new_matrix();
       }
     }

     /* p_d Vektoren generieren */
     component -> p_d = (sp_vectors *)emalloc(sizeof(sp_vectors) * ndetT);
     for(i=0;i<ndetT;i++)
     {
       component -> p_d[i] = new_vector();
     }
   }

   /* jetzt gehts los */
   append_edges( component );     /* Kanten in Matrizen einbauen */

   /* file parse done */
   fclose(rrg_fp);
   /* marking und names daten wieder loeschen */
   free_mem();   

   return(component);
}


/*******************************************************************************/
static void spool(fp, buf)
/*******************************************************************************/
FILE *fp;
char *buf;
{
   do
   {
      fgets(buf,BUF_SIZE,fp);
   } while( buf[0] == COMMENT );

   if( strlen(buf) > 1 )
      buf[strlen(buf) - 1] = '\0';
}


/*******************************************************************************/
static void read_fileinfo(info_fp) /* schreibt in file globale vars */
/*******************************************************************************/
FILE *info_fp;
{
   char buffer[BUF_SIZE];

   /* read no. of markings */
   spool(info_fp,buffer);
   sscanf(buffer,"%d", &nmarking);

   /* read no. of lines in file */
   spool(info_fp,buffer);
   sscanf(buffer,"%d", &nline);

   /* read no. of timed transitions */
   spool(info_fp,buffer);
   sscanf(buffer,"%d", &nnames);
}


/*******************************************************************************/
static char *next_word(start)
/*******************************************************************************/
char *start;
{
   char *end;

   /* make sure we don't segmentation fault */
   if( start == NULL )
      return( NULL );

   /* search for end of word */
   for( end = start; (*end != ' ') && (*end != '\t') && (*end != '\0'); end++ )
      ;

   /* check if we are at the end of the string */
   if( *end == '\0' )
      return( NULL );
   else
   {
      *end++ = '\0';
      return( end );
   }
}


/*******************************************************************************/
static void read_names(names_fp)
/*******************************************************************************/
FILE *names_fp;
{
   int i;
   char buffer[BUF_SIZE];

   char *bs, *be;

   /* allocate names memory */
   names = (name_t *)emalloc(nnames * sizeof(name_t));

   /* read names and data */
   for( i = 0; i < nnames; i++ )
   {
      /* fetch transition name & params */
      spool(names_fp,buffer);
      bs = buffer;

      /* extract name */
      be = next_word(bs);
      names[i].name = (char *)strdup(bs);
      bs = be;

      /* extract number */
      be = next_word(bs);
      names[i].number = atoi(bs);
      bs = be;

      /* extract type */
      be = next_word(bs);
      names[i].type = *bs;
      bs = be;         

      /* extract server type (optional) */
      be = next_word(bs);
      if( bs != be )
         names[i].sType = (char *)strdup(bs);
      else
         names[i].sType = (char *)strdup("SS");
      bs = be;

      /* extract semantics (optional) */
      be = next_word(bs);
      if( bs != be )
         names[i].semant = (char *)strdup(bs);
      else
         names[i].semant = (char *)strdup("re");
      bs = be;

      /* konsistenz ckeck: names[i].semant muss immer re sein */
      if(strcmp(names[i].semant,"re"))   /* kein race enable semantics ? */
      {
         fprintf(stderr,"Invalid semantic '%s' for transition '%s'\n", names[i].semant,names[i].name);
         exit(-1);
      }

      /* konsistenz check: names[i].sType bei detT muss SS sein  */
      if( names[i].type != EXP_TYPE &&
          (strcmp(names[i].sType,"SS")) )
      {
         fprintf(stderr,"Invalid combination for transition '%s'\n", names[i].name);
         exit(-1);
      }

      /* typeNr detemine */
      if( names[i].type == EXP_TYPE)
      {
          names[i].typeNr = -1;
      }
      else
      {
          names[i].typeNr = ndetT;
	  ndetT++;
      }
   }
}



/*******************************************************************************/
static void read_edges(rrg_fp)   
/*******************************************************************************/
FILE *rrg_fp;
{
   char buffer[BUF_SIZE];
   int fromtypeNr;      /* markingtype before edge is considered */
   int newftNr;         /* and after then */
   int index;
   int line;
   int i;
   char *errname;

   /* allocate markings memory */
   marking = (mark_t *)emalloc(nmarking * sizeof(mark_t));

   for(i=0;i<nmarking;i++)
      marking[i].typeNr = -1;

   /* allocate edges memory */
   edge = (edge_t *)emalloc(nline * sizeof(edge_t));

   /* read transitions write in mark_t und edge_t*/
   line = 0;
   while( !feof(rrg_fp) && line < nline )
   {
      /* neue Kante einlesen */
      spool(rrg_fp,buffer);
      edge[line].sMult = 0;
      sscanf(buffer,"%u %u %u %lf %lf %u",&(edge[line].fromMark),
                                          &(edge[line].toMark),
                                          &(index),
                                          &(edge[line].delay),
                                          &(edge[line].prob),
                                          &(edge[line].sMult));

      /* hole infos aus name_t nach edge_t */
      i = 0;
      while( (i < nnames) && (names[i].number != index) )
	i++;  /* erst mal den richtigen Index finden */
      if( i == nnames )
      {
         fprintf(stderr,"Undefined transition number %d.\n",index);
         exit(-1);
      }
      edge[line].type = names[i].type;
      edge[line].typeNr = names[i].typeNr;
      edge[line].name = names[i].name;

      /* mehr als eine detT aktiviert ? */
      if(edge[line].type==DET_TYPE)
      {
	if( (marking[edge[line].fromMark].typeNr >= 0) &&
	    (marking[edge[line].fromMark].typeNr != edge[line].typeNr))
	{
	  for(i=0;i<nnames;i++)             /* 2. Uebeltaeter namen finden */
	    if(names[i].typeNr == marking[edge[line].fromMark].typeNr)
	      errname = names[i].name;

	  fprintf(stderr,"Deterministic transition '%s' and '%s' are enabled together.\nBut only one is permitted", edge[line].name, errname);
	  exit(-1);
	}
      
	/* Struktur mark_t aktualisieren, macht nur bei detT Sinn, da expT ja sowieso nebenbei schalten koennen */
	marking[edge[line].fromMark].typeNr = edge[line].typeNr;
      }
      
      line++;      
   }
}


/*******************************************************************************/
static void append_edges(component)      /* Kante in Matrizen einbauen */
/*******************************************************************************/
solve_basis *component;
{
   int i;
   int d,c;

   for(i=0;i<nline;i++)
   {
      if(edge[i].type == EXP_TYPE)       /* expT ? */
      {
         if(edge[i].fromtypeNr == -1)           /*  expT schaltet von S'E */
         {
            if(edge[i].totypeNr == -1)               /* expT schaltet von S'E nach S'E */
            {
               /* Eintrag in Q_EE */
	       add_entry_to_matrix(component -> Q_EE,
				   edge[i].fromMark,
				   edge[i].toMark,
				   1.0 / edge[i].delay * edge[i].prob);
 	       add_entry_to_matrix(component -> Q_EE,
				   edge[i].fromMark,
				   edge[i].fromMark,
				   -1.0 / edge[i].delay * edge[i].prob);
            }
            else                                   /* expT schaltet von S'E nach S'd */
            {
               d = edge[i].totypeNr;
               /* Eintrag in Q_Ed */
	       add_entry_to_matrix(component -> Q_Ed[d],
				   edge[i].fromMark,
				   edge[i].toMark,
				   1.0 / edge[i].delay * edge[i].prob);
 	       add_entry_to_matrix(component -> Q_EE,
				   edge[i].fromMark,
				   edge[i].fromMark,
				   -1.0 / edge[i].delay * edge[i].prob);	       
            }
         }
         else                                 /* expT schaltet von S'd */
         {
            if(edge[i].totypeNr == -1)               /* expT schaltet von S'd nach S'E */
            {
	      if(!preempted_trans)
		printf("Preempted transitions found.\n");
	      preempted_trans = 1;
	      d = edge[i].fromtypeNr;
	      /* Eintrag in Q_dE */
	      add_entry_to_matrix(component -> detMatrices[d].Q_dE,
				  edge[i].fromMark,
				  edge[i].toMark,
				  1.0 / edge[i].delay * edge[i].prob);
	      add_entry_to_matrix(component -> detMatrices[d].Q_d,
				  edge[i].fromMark,
				  edge[i].fromMark,
				  -1.0 / edge[i].delay * edge[i].prob);
	    }       
            else                                   /* expT schaltet von S'd nach S'c */
            {
               if(edge[i].fromtypeNr == edge[i].totypeNr)  /* expT schaltet von S'd nach S'd , d==d */
               {
		  d = edge[i].fromtypeNr;   /* hier wuerde auch totypeNr gehen, typeNr sowieso. */
                  /* Eintrag in Q_d */
		  add_entry_to_matrix(component -> detMatrices[d].Q_d,
				      edge[i].fromMark,
				      edge[i].toMark,
				      1.0 / edge[i].delay * edge[i].prob);
		  add_entry_to_matrix(component -> detMatrices[d].Q_d,
				      edge[i].fromMark,
				      edge[i].fromMark,
				      -1.0 / edge[i].delay * edge[i].prob);	       
	       }
               else                                    /* expT schaltet von S'd nach S'c , d!=c */
               {
		 if(!preempted_trans)
		   printf("Preempted transitions found.\n");
		 preempted_trans = 1;
		 d = edge[i].fromtypeNr;   /* hier wuerde auch typeNr gehen */
		 c = edge[i].totypeNr;
		 /* Eintrag in Q_dc */
		 add_entry_to_matrix(component -> detMatrices[d].Q_dc[c],
				     edge[i].fromMark,
				     edge[i].toMark,
				     1.0 / edge[i].delay * edge[i].prob);
		 add_entry_to_matrix(component -> detMatrices[d].Q_d,
				     edge[i].fromMark,
				     edge[i].fromMark,
				     -1.0 / edge[i].delay * edge[i].prob);	       
	       }
	    }
         }
      }
      else if(edge[i].type == DET_TYPE)  /* detT ? */ 
      {
         if(edge[i].fromtypeNr == -1)           /* detT schaltet von S'E */
         {
            /* geht ja wohl nicht ! */
	    fprintf(stderr,"Unmoeglicher Fehler.\n");
	    exit(-1);
         }
         else                                 /* schaltet von S'd */
         {
            if(edge[i].totypeNr == -1)               /* detT schaltet von S'd nach S'E */
            {
               d = edge[i].fromtypeNr;
               /* Eintrag in Delta_dE */
	       add_entry_to_matrix(component -> detMatrices[d].Delta_dE,
				   edge[i].fromMark,
				   edge[i].toMark,
				   edge[i].prob);
	    }
            else                                   /* detT schaltet von S'd nach S'c */
            {
               d = edge[i].fromtypeNr;
               c = edge[i].totypeNr;
               /* Eintrag in Delta_dc */
	       add_entry_to_matrix(component -> detMatrices[d].Delta_dc[c],
				   edge[i].fromMark,
				   edge[i].toMark,
				   edge[i].prob);
            }
         }
      }
      else   /* weder expT noch detT (wohl generalT) */
      {
	fprintf(stderr,"Error: transition type '%c' of transition '%s' cannot be managed.\n",edge[i].type,edge[i].name);
	exit(-1);
      }
   }
}

/*******************************************************************************/
static void free_mem()       /* Struktur aktuallisieren */
/*******************************************************************************/
{
   int i;

   /* free names list */
   for( i = 0; i < nnames; i++ )
   {
      free(names[i].name);
      free(names[i].sType);
      free(names[i].semant);
   }
   free(names);


   /* free edge list */
   free(edge);
}




