/********************************************************************
Modul :             interpolationsfelder.c

Purpose :           Funktionen, die die Interpolationsfelder erneuern
                    ( sowohl der entlang tau-Linien als auch der fuer
		      die p_d(t)'s )

Project Director :  Reinhard German

Programmer :        Armin Heindl

History :           18.5.95 ( Diplomarbeit )
************************************************************************/
/* #define DEBUG_AC */
/* #define KOSTEN   */

#include <stdio.h>       /* fuer NULL */
#include <math.h>
#include "frontend.h"
#include "shared.h"      /* wegen ConcernsRandomisation */

extern int ac_no_detT;

/* aus integrate_interpolate.c : */
extern void newton_per_4();
extern void newton_per_3();
extern void newton_replace_1();
extern void newton_replace_2();
extern void l3_integrale();
extern void l4_integrale();

/* aus shared.c : */
extern sp_vector *ranforfield();    /* spezielle Randomisierungsfunktion fuer Interpolationfelder 
				       ( aus Effektivitaetsgruenden, da weniger Vektoroperationen
				         als mit randomize_quickly() ) */
extern double adapt_precision();

/* aus weighter.c : */
extern void call_new_weighter();

extern double EXT_PRECISION;
/************************************************************************************************        
Vorbemerkung zur Nomenklatur bei Neuaufbau der Interpolationsfelder entlang tau-Linien :

Die Grossbuchstaben D, Z und N beziehen sich im wesentlichen auf die Indikatoren der step_inbetween-
Elemente, die in das Interpolationsfeld einbezogen werden sollen, wobei das ..D bzw. ..ZD-Ende nur
eine extreme Auspraegung bezeichnet. So bedeutet NZD_case(), dass drei neu Stuetzpunkte in das 
Interpolationsfeld integriert werden, wobei das letzte Element ein DISC_ANZEIGE-Element sein koennte,
es aber nicht sein muss. (Zwangslaufig waere dann das vorletzte ein ZWISCHEN_ANZEIGE-Element.)
Wichtig ist noch zu beachten, dass die hier aufgelisteten Funktionen immer nur beim Neuorganisation
eines Feldes aufgerufen werden (im Gegensatz zu dem Fall, dass ein oder zwei Stuetzpunkte gegen neue
ausgetauscht werden, was in update_int_structures() erledigt wird). Dabei koennen bei letzter
Veraenderung der Stuetzpunkte, was nicht im letzten Schritt geschehen sein muss, ein oder zwei
Punkte ersetzt worden sein. Diese wurden mit tn1 bzw. tn2 bezeichnet, wobei tn1 immer der spaetere
bzw. juengere Punkt ist. t0, t1, t2 und t3 stehen fuer die Punkte, die nach dem vorhergehenden Neuaufbau
im Interpolationsfeld auftauchen. Da t3 der juengste Stuetzpunkt ist, belegt er im Feld auf Komponente 0;
analog z.B. t1 auf Komponente 2, da die aeltesten Punkte im Interpolationsfeld auf den hoechsten
Komponenten sitzen muessen, um ausgetauscht werden zu koennen.

Ausserdem ist wichtig, dass in jeder Konstellation die Stuetzpunkte, die jenseits des neuen Zeitpunkts 
t+h liegen, in der step_inbetween-Kette direkt aufeinander folgten. 

Mit erstem und zweitem Cut/Schnitt wird die Lage von den Zeitpunkten t + 0.2*stepsize bzw.
t + stepsize bezeichnet, wobei der letzte aktuelle Zeitpunkt ist und der zweite Cut praktisch de neue.
Der erste Schnitt haengt von Parametern des dopri5()-Verfahrens ab, da dort von altem aktuellen
Zeitpunkt ausgehend zum ersten Mal im Abstand von 0.2*stepsize interpoliert wird.

Beachte, dass bei allen Interpolationsfeldern die Lage (int_points) der Stuetzpunkte sowie der aller 
anderen Objekte wie den Cuts relativ zum Stuetzpunkt in der Komponente 0 angegeben werden. 
In field[0].int_point kann deshalb die Lage des aktuellen Zeitpunkts t relativ zum Bezugspunkt
gespeichert werden.
*************************************************************************************************/
void D_case( k, field, add, timespan, old_stepsize, randdata)
/* Voraussetzungen : - add zeigt auf einzubeziehenden Stuetzpunkt in step_inbetween-Kette
                     - timespan ist zeitlicher Abstand dieses Punktes von letztem aktuellen
		       Zeitpunkt                                                          */
int k;
interpolation_field field;
step_inbetween *add;
double timespan;
double old_stepsize;
ConcernsRandomisation **randdata;
{
  double precision;

  if (0.0 < field[2].int_point) {
    /* tn1tn2t2t3 -> t3tn2tn1D */

    /* loesche t2 */
    del_vector(&(field[1].y_values));

    /* haenge tn1 und t3 um , tn2 bleibt */
    field[1].y_values = field[3].y_values;
    field[3].y_values = field[0].y_values;

    /* berechne Positionen der Stuetzpunkte relativ zu add-Element */
    field[0].int_point = (-1.0)*(field[0].int_point + old_stepsize + timespan);/* nur als Zwischenspeicher */
    field[2].int_point = field[0].int_point + field[2].int_point;
    field[1].int_point = field[0].int_point + field[3].int_point;
    field[3].int_point = field[0].int_point;

  }
  else {
    /* tn1t1t2t3 -> t2t3tn1D */

    /* loesche t1 */
    del_vector(&(field[2].y_values));

    /* haenge t3 um */
    field[2].y_values = field[0].y_values;
    /* tausche t2 und tn1 aus  */
    field[0].y_values = field[3].y_values;
    field[3].y_values = field[1].y_values;
    field[1].y_values = field[0].y_values;
    

    /* berechne Positionen der Stuetzpunkte relativ zu add-Element */
    field[2].int_point = (-1.0)*(field[0].int_point + old_stepsize + timespan);
    /* Austausch : */
    field[0].int_point = field[2].int_point + field[3].int_point;
    field[3].int_point = field[2].int_point + field[1].int_point;
    field[1].int_point = field[0].int_point;
  };

  /* Randomisierung des step_inbetween-Elements nach Interpolationsfeld */
  precision = adapt_precision( add->values[0][k]);
  call_new_weighter( timespan*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
  field[0].y_values = ranforfield(add->values[0][k], randdata[k]);

  /* berechne Position des alten aktuellen Zeitpunkts relativ zu add-Element */
  field[0].int_point = (-1.0)*timespan;

  newton_per_4(field);
}

/***********************************************************************/
void ZD_case( k, field, add, timespan, old_stepsize, randdata)
/* Voraussetzungen : - add zeigt auf ersten einzubeziehenden Stuetzpunkt in step_inbetween-Kette
                       add->next_down-Element ist ebenfalls einzubeziehen
                     - timespan ist zeitlicher Abstand des ersten Punktes von letztem aktuellen
		       Zeitpunkt                                                          */

int k;
interpolation_field field;
step_inbetween *add;
double timespan;
double old_stepsize;
ConcernsRandomisation **randdata; 
{
  double precision;
  double timespan_to_next = timespan + (add->step); /* zeitlicher Abstand des zweiten Elements
						       von letztem aktuellen Zeitpunkt */
  
  if (0.0 < field[2].int_point) {
    /* tn1tn2t2t3 -> tn2tn1ZD */

    /* loesche t2 und t3 */
    del_vector(&(field[0].y_values));
    del_vector(&(field[1].y_values));    
    /* tausche tn1 und tn2 aus */
    field[1].y_values = field[3].y_values;
    field[3].y_values = field[2].y_values;
    field[2].y_values = field[1].y_values;

    
    /* berechne Positionen der Stuetzpunkte relativ zu add->next_down-Element */
    field[1].int_point = (-1.0)*(field[0].int_point + old_stepsize + timespan_to_next); /* Zwischenspeicher */
    /* Austausch : */
    field[0].int_point = field[1].int_point + field[2].int_point;
    field[2].int_point = field[1].int_point + field[3].int_point;
    field[3].int_point = field[0].int_point;
  }
  else {
    /* tn1t1t2t3 -> t3tn1ZD */

    /* loesche t1 und t2 */
    del_vector(&(field[1].y_values));
    del_vector(&(field[2].y_values));
    
    /* haenge tn1 und t3 um */
    field[2].y_values = field[3].y_values;
    field[3].y_values = field[0].y_values;
    
    /* berechne Positionen der Stuetzpunkte relativ zu add->next_down-Element */
    field[1].int_point = (-1.0)*(field[0].int_point + old_stepsize + timespan_to_next);
    field[2].int_point = field[1].int_point + field[3].int_point;
    field[3].int_point = field[1].int_point;
  };

  /* Randomisierung des ersten step_inbetween-Elements nach Interpolationsfeld */
  precision = adapt_precision( add->values[0][k]);
  call_new_weighter( timespan*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
  field[1].y_values = ranforfield(add->values[0][k], randdata[k]);

  field[1].int_point = (-1.0)*(add->step);

  /* Randomisierung des zweiten step_inbetween-Elements nach Interpolationsfeld */
  precision = adapt_precision( add->next_down->values[0][k]);
  call_new_weighter( timespan_to_next*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
  field[0].y_values = ranforfield(add->next_down->values[0][k], randdata[k]);

  field[0].int_point = (-1.0)*timespan_to_next;

  newton_per_4(field);
}
  
/**********************************************************************/
void NZD_case( k, field, add, timespan, old_stepsize, randdata)
/* Voraussetzungen : - add zeigt auf ersten einzubeziehenden Stuetzpunkt in step_inbetween-Kette
                       add->next_down-Element und add->next_down->next_down-Element sind 
		       ebenfalls einzubeziehen
                     - timespan ist zeitlicher Abstand des ersten Punktes von letztem aktuellen
		       Zeitpunkt                                                          */

int k;
interpolation_field field;
step_inbetween *add;
double timespan;
double old_stepsize;
ConcernsRandomisation **randdata; 
{
  double precision;
  double timespan_to_afternext = timespan + (add->step) + (add->next_down->step);
  /* zeitlicher Abstand des add->next_down->next_down-Elements von letztem aktuellen Zeitpunkt */

  /* tn1tn2t2t3 bzw. tn1t1t2t3 -> tn1NZD */

  /* loesche alle bis auf juengstes Element (tn1) */
  del_vector(&(field[0].y_values));
  del_vector(&(field[1].y_values));    
  del_vector(&(field[2].y_values));    
      
  /* berechne Position von tn1 relativ zu add->next_down->next_down-Element */
  field[3].int_point = (-1.0)*(field[0].int_point + old_stepsize + timespan_to_afternext) + field[3].int_point;

  /* Randomisierung des ersten step_inbetween-Elements nach Interpolationsfeld */
  precision = adapt_precision( add->values[0][k]);
  call_new_weighter( timespan*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
  field[2].y_values = ranforfield(add->values[0][k], randdata[k]);

  field[2].int_point = (-1.0)*(add->step + add->next_down->step);
  
  /* Randomisierung des zweiten step_inbetween-Elements nach Interpolationsfeld */
  precision = adapt_precision( add->next_down->values[0][k]);
  call_new_weighter( (timespan + add->step)*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
  field[1].y_values = ranforfield(add->next_down->values[0][k], randdata[k]);

  field[1].int_point = (-1.0)*(add->next_down->step);

  /* Randomisierung des dritten step_inbetween-Elements nach Interpolationsfeld */
  precision = adapt_precision( add->next_down->next_down->values[0][k]);
  call_new_weighter( timespan_to_afternext*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
  field[0].y_values = ranforfield(add->next_down->next_down->values[0][k], randdata[k]);

  field[0].int_point = (-1.0)*(timespan_to_afternext);

  newton_per_4(field);
}
 
/***********************************************************************/
void NNZD_case( k, field, add1, timespan1, add2, timespan2, old_stepsize, randdata)
/* Interpolationsfeld wird komplett erneuert (kein Rueckgriff auf alte Stuetzpunkte)*/
/* Voraussetzungen : - add1 zeigt auf ersten einzubeziehenden Stuetzpunkt in step_inbetween-Kette
                       Nachfolger add1->next_down-Element ist ebenfalls einzubeziehen
		     - add2 zeigt auf dritten einzubeziehenden Stuetzpunkt 
		       Nachfolger add2->next_down-Element ist ebenfalls einzubeziehen
		     - add2-Element liegt hinter add1->next_down-Element, muss aber nicht der
		       direkte Nachfolger sein
                     - timespan1 ist zeitlicher Abstand des add1-Elements von letztem aktuellen
		       Zeitpunkt, timespan2 der des add2-Elements                             */

int k;
interpolation_field field;
step_inbetween *add1;
double timespan1;
step_inbetween *add2;
double timespan2;
double old_stepsize;
ConcernsRandomisation **randdata; 
{
  double precision;
  double to_current_point = (timespan2 + add2->step); /* zeitlicher Abstand des add2->next_down-Elements von 
							 letztem aktuellem Zeitpunkt */

  /* tn1tn2t2t3 bzw. tn1t1t2t3 -> NNZD */

  /* loesche alle Stuetzpunkte */
  del_vector(&(field[0].y_values));     
  del_vector(&(field[1].y_values));    
  del_vector(&(field[2].y_values));    
  del_vector(&(field[3].y_values));

  /* Randomisierung des ersten step_inbetween-Elements nach Interpolationsfeld */
  precision = adapt_precision( add1->values[0][k]);
  call_new_weighter( timespan1*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
  field[3].y_values = ranforfield(add1->values[0][k], randdata[k]);

  field[3].int_point = (timespan1-timespan2-(add2->step));
  
  /* Randomisierung des zweiten step_inbetween-Elements nach Interpolationsfeld */
  precision = adapt_precision( add1->next_down->values[0][k]);
  call_new_weighter( (timespan1 + add1->step)*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
  field[2].y_values = ranforfield(add1->next_down->values[0][k], randdata[k]);

  field[2].int_point = field[3].int_point + add1->step;

  /* Randomisierung des dritten step_inbetween-Elements nach Interpolationsfeld */
  precision = adapt_precision( add2->values[0][k]);
  call_new_weighter( timespan2*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
  field[1].y_values = ranforfield(add2->values[0][k], randdata[k]);

  field[1].int_point = (-1.0)*(add2->step);

  /* Randomisierung des vierten step_inbetween-Elements nach Interpolationsfeld */
  precision = adapt_precision( add2->next_down->values[0][k]);
  call_new_weighter( to_current_point*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
  field[0].y_values = ranforfield(add2->next_down->values[0][k], randdata[k]);

  field[0].int_point = (-1.0)*to_current_point;

  newton_per_4(field);
}
/**********************************************************************/
void kleiner_sonderfall( k, field, add1, timespan1_ext, add2, timespan2_ext, old_stepsize, randdata)
/* Voraussetzungen : - add1->next_down zeigt auf ersten einzubeziehenden Stuetzpunkt in step_inbetween-Kette
                       ( add1-Element ist schon im Interpolationsfeld -> Besonderheit )
		     - add2 zeigt auf zweiten einzubeziehenden Stuetzpunkt 
		       Nachfolger add2->next_down-Element ist ebenfalls einzubeziehen
		     - add2-Element liegt hinter add1->next_down-Element, muss aber nicht der
		       direkte Nachfolger sein
                     - timespan1_ext ist zeitlicher Abstand des add1->next_down Elements von letztem 
		       aktuellen Zeitpunkt, timespan2_ext der des add2->next_down-Elements          
	   wichtig ! - tn1 liegt genau auf letztem aktuellem Zeitpunkt                             */

int k;
interpolation_field field;
step_inbetween *add1;
double timespan1_ext;
step_inbetween *add2;
double timespan2_ext;
double old_stepsize;
ConcernsRandomisation **randdata; 
{
  double precision;
  /* tn1tn2t2t3 bzw. tn1t1t2t3 -> tn1Z1Z3D  */

  /* loesche alle bis auf juengsten Stuetzpunkt */
  del_vector(&(field[0].y_values));
  del_vector(&(field[1].y_values));    
  del_vector(&(field[2].y_values));    
  
  field[3].int_point = (-1.0)*(timespan2_ext); /* tn1 liegt genau auf letztem aktuellem Zeitpunkt */
  
  /* Randomisierung des ersten step_inbetween-Elements nach Interpolationsfeld */
  precision = adapt_precision( add1->next_down->values[0][k]);
  call_new_weighter( timespan1_ext*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
  field[2].y_values = ranforfield(add1->next_down->values[0][k], randdata[k]);

  field[2].int_point = timespan1_ext - timespan2_ext;
  
  /* Randomisierung des zweiten step_inbetween-Elements nach Interpolationsfeld */
  precision = adapt_precision( add2->values[0][k]);
  call_new_weighter( (timespan2_ext - add2->step)*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
  field[1].y_values = ranforfield(add2->values[0][k], randdata[k]);

  field[1].int_point = (-1.0)*(add2->step);
  
  /* Randomisierung des dritten step_inbetween-Elements nach Interpolationsfeld */
  precision = adapt_precision( add2->next_down->values[0][k]);
  call_new_weighter( timespan2_ext*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
  field[0].y_values = ranforfield(add2->next_down->values[0][k], randdata[k]);

  field[0].int_point = (-1.0)*(timespan2_ext);

  /* newton_per_4() wird in sonderbehandlung aufgerufen ! */
}   
/**********************************************************************/
void sonderbehandlung( k, field, add1, timespan1_ext, add2, timespan2_ext, old_stepsize, randdata)
/* Unterschied zu NNZD_case() : Interpolationsfeld wird nicht komplett erneuert, da auf alte 
   Stuetzpunkte zurueckgegriffen werden kann */
/* Voraussetzungen : - add1 zeigt auf ersten evtl. einzubeziehenden Stuetzpunkt in step_inbetween-Kette
                       Nachfolger add1->next_down-Element ist ebenfalls nur evtl. einzubeziehen
		     - add2 zeigt auf dritten evtl. einzubeziehenden Stuetzpunkt 
		       Nachfolger add2->next_down-Element wird auf jeden Fall, aber gebenenfalls
		       als einziger Stuetzpunkt einbezogen
		     - add2-Element liegt hinter add1->next_down-Element, muss aber nicht der
		       direkte Nachfolger sein
                     - timespan1_ext ist zeitlicher Abstand des add1->next_down Elements von letztem 
		       aktuellen Zeitpunkt, timespan2_ext der des add2->next_down-Elements        */

int k;
interpolation_field field;
step_inbetween *add1;
double timespan1_ext;
step_inbetween *add2;
double timespan2_ext;
double old_stepsize;
ConcernsRandomisation **randdata; 
{
   double precision;   
   double from_t3_to_cut = field[0].int_point + old_stepsize + EXT_PRECISION; 
   /* zeitlicher Abstand von t3 von letztem aktuellem Zeitpunkt, dem alten sogenannten zweiten Schnitt/Cut */

   /* es gilt : timespan1_ext == add1->step */
#ifdef DEBUG_AC
   { double debug_diff;

     debug_diff = fabs( timespan1_ext - add1->step);
     if (debug_diff > 100*EXT_PRECISION)
       printf(" FEHLER in sonderbehandlung : timespan1_ext != add1->step !\n");
   }
#endif
   if (0.0 < field[2].int_point) {
     /* tn1tn2t2t3 -> Fallunterscheidung */

     if ((from_t3_to_cut - field[3].int_point) >=  0.0) {
       /* alter zweiter Cut liegt rechts von tn1, d.h. tn1 muss mit altem zweitem Cut zusammenfallen */

#ifdef DEBUG_AC
       if (add1->values[0][k]->num == 0)
	 printf(" Sonderbehandlung : el1->values[0][k] gehoert gar nicht zu Interpolationsfeld\n");
         /* falls der Fall : lieber ganz neu aufbauen mit add1-Element ! */
#endif

       /* tn1tn2t2t3 -> tn1Z1Z3D */
       kleiner_sonderfall(k, field, add1, timespan1_ext, add2, timespan2_ext, old_stepsize, randdata);
       /* kleiner_sonderfall wird weiter unten nochmal aufgerufen */
     }
     else if ((from_t3_to_cut - field[2].int_point) >= 0.0) {
       /* alter zweiter Cut liegt zwischen tn2 und tn1 */
       /* tn1tn2t2t3 -> tn2tn1Z3D */

#ifdef DEBUG_AC
       if (add1->next_down->values[0][k]->num == 0)
	 printf(" Sonderbehandlung : el1->next_down->values[0][k] gehoert gar nicht zu Interpolationsfeld\n");
       /* falls der Fall : lieber ganz neu aufbauen mit add1-Element und add1->next_down-Element! */
#endif

       /* loesche t2 und t3 */
       del_vector(&(field[0].y_values));
       del_vector(&(field[1].y_values));    
       /* tausche tn1 und tn2 aus */
       field[1].y_values = field[3].y_values;
       field[3].y_values = field[2].y_values;
       field[2].y_values = field[1].y_values;
       
       /* berechne Positionen der Stuetzpunkte relativ zu add2->next_down-Element */
       field[3].int_point = (-1.0)*(timespan2_ext + field[3].int_point - field[2].int_point - add1->step);
       field[2].int_point = (-1.0)*(timespan2_ext - add1->step);

       /* Randomisierung des ersten step_inbetween-Elements nach Interpolationsfeld */
       precision = adapt_precision( add2->values[0][k]);
       call_new_weighter( (timespan2_ext - add2->step)*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
       field[1].y_values = ranforfield(add2->values[0][k], randdata[k]);

       field[1].int_point = (-1.0)*(add2->step);
       
       /* Randomisierung des zweiten step_inbetween-Elements nach Interpolationsfeld */
       precision = adapt_precision( add2->next_down->values[0][k]);
       call_new_weighter( timespan2_ext*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
       field[0].y_values = ranforfield(add2->next_down->values[0][k], randdata[k]);

       field[0].int_point = (-1.0)*timespan2_ext;
     }
     else {
       /* d.h. tn1 und tn2 liegen rechts von altem zweitem Cut */
       /* pruefe, ob auch add2-Element schon interpoliert worden ist */
       if (add2->values[0][k]->num) {
	 /* tn1tn2t2t3 -> t3tn2tn1D */

#ifdef DEBUG_AC
	 if (add1->next_down->values[0][k]->num == 0)
	   printf(" Sonderbehandlung : el1->next_down->values[0][k] gehoert gar nicht zu Interpolationsfeld\n");
           /* falls der Fall : lieber ganz neu aufbauen ! */
	 if (add1->next_down->next_down->values[0][k]->num == 0)
	   printf(" Sonderbehandlung : el3->values[0][k] gehoert gar nicht zu Interpolationsfeld\n");
           /* falls der Fall : lieber ganz neu aufbauen ! */
	 if (add1->next_down->next_down != add2) {
	   printf(" Sonderbehandlung : el3->values[0][%d] in Interpolationsfeld aufgenommen \n",k);
	   printf("                    aber Einzelgaenger, da el1->next_down->next_down != el3 \n");
	 };
#endif
	 /* loesche t2 */
	 del_vector(&(field[1].y_values));

	 /* haenge tn1 und t3 um */
	 field[1].y_values = field[3].y_values;
	 field[3].y_values = field[0].y_values;
	 
	 /* berechne Positionen der Stuetzpunkte relativ zu add2->next_down-Element */
	 field[3].int_point = (-1.0)*(timespan2_ext + field[2].int_point - add1->step);
	 field[2].int_point = (-1.0)*(timespan2_ext - add1->step);
	 field[1].int_point = (-1.0)*(add2->step);

	 /* Randomisierung des ersten step_inbetween-Elements nach Interpolationsfeld */
	 precision = adapt_precision( add2->next_down->values[0][k]);
	 call_new_weighter( timespan2_ext*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
	 field[0].y_values = ranforfield(add2->next_down->values[0][k], randdata[k]);

	 field[0].int_point = (-1.0)*timespan2_ext;
       }
       else { /* add2->values[0][k]->num == 0, also add2-Element noch nicht in Interpolationsfeld */
	 /* tn1tn2t2t3 -> t3tn2Z3D */ 
	 /* tn1 wird zugunsten von t3 uebergangen, damit rechts und links von neuem ersten Cut 
	    jeweils ein Stuetzpunkt liegt !!! */

#ifdef DEBUG_AC
	 /* es muss gelten : add1->next_down->next_down != add2 */
	 if (add1->next_down->values[0][k]->num == 0)
	   printf(" Sonderbehandlung : el1->next_down->values[0][%d] gehoert gar nicht zu Interpolationsfeld\n",k);
           /* falls der Fall : lieber ganz neu aufbauen ! */
	 if (add1->next_down->next_down->values[0][k]->num == 0)
	   printf(" Sonderbehandlung : el1->next_down->next_down->values[0][%d] gehoert gar nicht zu Interpolationsfeld\n",k);
           /* falls der Fall : lieber ganz neu aufbauen ! */
#endif	 
	 /* loesche tn1 und t2 */
	 del_vector(&(field[3].y_values));
	 del_vector(&(field[1].y_values));

	 /* haenge t3 um */
	 field[3].y_values = field[0].y_values;

	 /* berechne Positionen der Stuetzpunkte relativ zu add2->next_down-Element */
	 field[3].int_point = (-1.0)*(timespan2_ext + field[2].int_point - add1->step);
	 field[2].int_point = (-1.0)*(timespan2_ext - add1->step);

	 /* Randomisierung des ersten step_inbetween-Elements nach Interpolationsfeld */
	 precision = adapt_precision( add2->values[0][k]);
	 call_new_weighter( (timespan2_ext - add2->step)*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
	 field[1].y_values = ranforfield(add2->values[0][k], randdata[k]);

	 field[1].int_point = (-1.0)*(add2->step);

	 /* Randomisierung des zweiten step_inbetween-Elements nach Interpolationsfeld */
	 precision = adapt_precision( add2->next_down->values[0][k]);
	 call_new_weighter( timespan2_ext*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
	 field[0].y_values = ranforfield(add2->next_down->values[0][k], randdata[k]);

	 field[0].int_point = (-1.0)*timespan2_ext;
       };
     };
   }
   else { /* tn1t1t2t3 -> Fallunterscheidung */
     if ((from_t3_to_cut - field[3].int_point) >=  0.0) {
       /* alter zweiter Cut liegt rechts von tn1, d.h. tn1 muss mit altem zweitem Cut zusammenfallen */

#ifdef DEBUG_AC
       if (add1->values[0][k]->num == 0)
	 printf(" Sonderbehandlung : el1->values[0][k] gehoert gar nicht zu Interpolationsfeld\n");
         /* falls der Fall : lieber ganz neu aufbauen mit add1-Element ! */
#endif

       /* tn1t1t2t3 -> tn1Z1Z3D  */
       kleiner_sonderfall(k, field, add1, timespan1_ext, add2, timespan2_ext, old_stepsize, randdata);
       /* kleiner_sonderfall wird weiter oben nochmal aufgerufen */

     }
     else { /* tn1 liegt rechts von altem zweiten Cut */
       /* tn1t1t2t3 -> t3tn1Z3D */
       /* t3 liegt auf jeden Fall links von altem zweiten Cut, sonst waere ja nicht erneuert worden */

#ifdef DEBUG_AC
       if (add1->next_down->values[0][k]->num == 0)
	 printf(" Sonderbehandlung : el1->next_down->values[0][k] gehoert gar nicht zu Interpolationsfeld\n");
         /* falls der Fall : lieber ganz neu aufbauen ! */
#endif
       /* loesche t2 und t1 */
       del_vector(&(field[1].y_values));
       del_vector(&(field[2].y_values));
    
       /* haenge tn1 und t3 um */
       field[2].y_values = field[3].y_values;
       field[3].y_values = field[0].y_values;

       /* berechne Positionen der Stuetzpunkte relativ zu add2->next_down-Element */
       field[3].int_point = (-1.0)*(timespan2_ext + field[3].int_point - add1->step);
       field[2].int_point = (-1.0)*(timespan2_ext - add1->step);

       /* Randomisierung des ersten step_inbetween-Elements nach Interpolationsfeld */
       precision = adapt_precision( add2->values[0][k]);
       call_new_weighter( (timespan2_ext - add2->step)*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
       field[1].y_values = ranforfield(add2->values[0][k], randdata[k]);

       field[1].int_point = (-1.0)*(add2->step);

       /* Randomisierung des zweiten step_inbetween-Elements nach Interpolationsfeld */
       precision = adapt_precision( add2->next_down->values[0][k]);
       call_new_weighter( timespan2_ext*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision);
       field[0].y_values = ranforfield(add2->next_down->values[0][k], randdata[k]);

       field[0].int_point = (-1.0)*timespan2_ext;
     };
   };
   newton_per_4(field);
 }

/*****************************************************************************************************
Vorbemerkung zur Behandlung der upper_steps :

Je nachdem, wie weit sich die step_inbetween-Kette mit einem zu machenden Schritt nach oben verschiebt,
wird die tau-Linie einer Gruppe (Zustaende, in denen eine bestimmte deterministische Transition 
aktiviert ist) zwischen zwei bestimmte step_inbetween-Elemente oder sogar genau auf eines fallen. Dieser
Unterschied wird in update_int_structures() festgestellt und behandelt, indem die oberste Box der 
upper_steps mit Komponente step angelegt wird ( Komponente integr_weight bleibt unbelegt ). Allerdings
muessen im allgemeinen die Integrationsstuetzpunkte zu anderen Einheiten als in der step_inbetween-Kette
zusammengefasst werden, um mit genuegend hoher Ordnung zu integrieren ( Lagrange-Integration mit Polynomen
von zweiter Ordnung als Ausnahmen, sonst mit Polynomen von dritter Ordnung ). Also muessen abweichende
Integrationsgewichte fuer diese Glieder der step_inbetween-Kette speziell fuer diese Gruppe errechnet
werden, fuer welche dann eigene Kopien fuer jede Gruppe in den sogenannten upper_steps angelegt werden.

Dies geschieht in den folgenden Funktionen, fuer die folgende Voraussetzungen gelten :
- cop_ptr zeigt auf das bereits in update_int_structures() erzeugte, oberste Element der neuen 
  upper_steps der Gruppr k, in dem nur die Komponente step festgelegt sein muss.
- org_ptr zeigt auf das erste step_inbetween-Element der Integrationskette, das echt unterhalb der
  tau-Linie der Gruppe k liegt (z.B. falls kein step_inbetween-Element genau auf diese tau-Linie
  trifft, ist das das el3->next_down-Element, also das erste Element unterhalb des zweiten Schnitts)
- Die Buchstaben O und X machen Aussagen ueber die Indikatoren der upper_steps-Elemente, die nach
  der Ausfuehrung der Funktion auf das cop_ptr-Element folgen. Dabei steht O fuer ein 
  ZWISCHEN_ANZEIGE-Element, X fuer ein Element, das zu zwei Integrationsintervallen gehoert oder 
  ein DISC_ANZEIGE-Element ist.
  Beispiel : OXOX_case() bedeutet, dass zum Beispiel mit dem org_ptr-Element eine Folge von 
             fogenden Elementen in der step_inbetween-Kette beginnt : XOODDOOX.. (D : DISC_ANZEIGE-
	     Element). Bezeichnen wir das cop_ptr-Element mit S, so muss also das Integrations-
	     intervall XOOD zerlegt werden in zwei Integrationsintervalle S-OXOD ( wobei das X zu
	     zum Intervall SOX und dem Intervall XOD gehoert ), da Beibehaltung der Gruppe XOOD
	     bedeuten wuerde, dass zwischen SX nur mit der Trapezregel integriert werden koennte.
- Beachte, dass in allen upper_steps-Elementen nur einmal der Vektor element->values[0][k] fuer die
  eine gerade behandelte Gruppe belegt wird
- Wegen des Dummies ist es unbedingt notwendig, dass immer zuerst die upper_steps der Gruppe mit 
  der kleinsten determ. Schaltzeit ( Index k = 0) behandelt wird.
*****************************************************************************************************/
void OX_case( k, cop_ptr, org_ptr, plow_box)  /*erstes Zeichen ist Buchstabe*/
/* moegliche org_ptr-Folge : OX bzw. OD */
int k;
step_inbetween *cop_ptr;
step_inbetween *org_ptr;
lower_stepbox *plow_box;
{
  int i;
  double integr_gewichte[3];

  /* berechne Integrationsgewichte fuer upper_steps */
  l3_integrale( integr_gewichte, cop_ptr->step, org_ptr->step);

  /* erstes upper_steps-Element */
  cop_ptr->integr_weight = integr_gewichte[0];
  cop_ptr->indicator = ZWISCHEN_ANZEIGE;       /* wichtig insbesondere bei Einarbeitung von allgemein verteilten
						  Schaltzeiten mit generandom.c */
  /* Zuweisung der values-Komponenten wird in update_int_structures() zu Beginn des Randomisierungsteils
     erledigt */

  /* zweites upper_steps-Element */
  cop_ptr->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
  cop_ptr = cop_ptr->next_down;
  
  cop_ptr->step = org_ptr->step;
  cop_ptr->integr_weight = integr_gewichte[1];
  cop_ptr->indicator = ZWISCHEN_ANZEIGE;
  cop_ptr->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
    cop_ptr->values[0][k] = copy_vector( org_ptr->values[0][k]);

  /* drittes upper_steps-Element */
  cop_ptr->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
  cop_ptr = cop_ptr->next_down;
  org_ptr = org_ptr->next_down;  

  cop_ptr->step = org_ptr->step;  
  cop_ptr->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
    cop_ptr->values[0][k] = copy_vector( org_ptr->values[0][k]);

  /* unterscheide, letztes Element der Intervallgruppe gleichzeitig das erste der naechsten
     Intervallgruppe ist oder nicht */
  if (org_ptr->indicator == DISC_ANZEIGE) {
    cop_ptr->integr_weight = integr_gewichte[2];
    cop_ptr->indicator = DISC_ANZEIGE;
  } 
  else {
    cop_ptr->integr_weight = integr_gewichte[2] + org_ptr->indicator;
    cop_ptr->indicator = org_ptr->indicator;
  };
#ifdef KOSTEN
  log_kosten(3);
#endif

  /* evtl. muss ein dummy-Element an das Ende der step_inbetween-Kette angehaengt werden */
  if (org_ptr->next_down == NULL) {
    org_ptr->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
    org_ptr = org_ptr->next_down;
    org_ptr->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
    org_ptr->values[1] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
    for( i=0; i<ac_no_detT; i++) {
	org_ptr->values[0][i] = new_vector();
	org_ptr->values[1][i] = new_vector();
      };
    org_ptr->integr_weight = 0.0;
    org_ptr->next_down = NULL;

    /* das sind die Kennzeichen eines dummy-Elements */
    org_ptr->step = 0.0;                    
    org_ptr->indicator = ZWISCHEN_ANZEIGE;

    cop_ptr->next_down = org_ptr;
    plow_box->next_up = org_ptr;
  }
  else {
    /* klinke Ende der upper_steps an step_inbetween-Liste */
    cop_ptr->next_down = org_ptr->next_down;
  };
}

/*********************************************************************************************/

void OOX_case( k, cop_ptr, org_ptr, plow_box)  /*erste Zeichen sind Buchstaben*/
/* moegliche org_ptr-Folge : XOX bzw. XOD und OOX bzw. OOD */

int k;
step_inbetween *cop_ptr;
step_inbetween *org_ptr;
lower_stepbox *plow_box;
{
  int i, j;
  double integr_gewichte[4];

  /* berechne Integrationsgewichte fuer upper_steps */
  l4_integrale( integr_gewichte, cop_ptr->step, org_ptr->step, org_ptr->next_down->step);  

  /* erstes upper_steps-Elememnt */
  cop_ptr->integr_weight = integr_gewichte[0];
  cop_ptr->indicator = ZWISCHEN_ANZEIGE;       /* wichtig insbesondere bei Einarbeitung von allgemein verteilten
						  Schaltzeiten mit generandom.c */
  /* Zuweisung der values-Komponenten wird in update_int_structures() zu Beginn des Randomisierungsteils
     erledigt */

  /* zweites upper_steps-Elememnt */  
  cop_ptr->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
  cop_ptr = cop_ptr->next_down;

  for( j=1; j<3; j++) {  
    cop_ptr->step = org_ptr->step;
    cop_ptr->integr_weight = integr_gewichte[j];
    cop_ptr->indicator = ZWISCHEN_ANZEIGE;
    cop_ptr->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
      cop_ptr->values[0][k] = copy_vector( org_ptr->values[0][k]);
    
    /* drittes bzw. viertes upper_steps-Element */
    cop_ptr->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
    cop_ptr = cop_ptr->next_down;
    org_ptr = org_ptr->next_down;  
  };
  cop_ptr->step = org_ptr->step;
  cop_ptr->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
    cop_ptr->values[0][k] = copy_vector( org_ptr->values[0][k]);

  /* unterscheide, letztes Element der Intervallgruppe gleichzeitig das erste der naechsten
     Intervallgruppe ist oder nicht */
  if (org_ptr->indicator == DISC_ANZEIGE) {
    cop_ptr->integr_weight = integr_gewichte[3];
    cop_ptr->indicator = DISC_ANZEIGE;
  } 
  else {
    cop_ptr->integr_weight = integr_gewichte[3] + org_ptr->indicator;
    cop_ptr->indicator = org_ptr->indicator;
  };
#ifdef KOSTEN
  log_kosten(3);
#endif
  /* evtl. muss ein dummy-Element an das Ende der step_inbetween-Kette angehaengt werden */
  if (org_ptr->next_down == NULL) {
    org_ptr->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
    org_ptr = org_ptr->next_down;
    org_ptr->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
    org_ptr->values[1] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
    for( i=0; i<ac_no_detT; i++) { 
	org_ptr->values[0][i] = new_vector();
	org_ptr->values[1][i] = new_vector();
      };
    org_ptr->integr_weight = 0.0;
    org_ptr->next_down = NULL;

    /* das sind die Kennzeichen eines dummy-Elements */
    org_ptr->step = 0.0;                    
    org_ptr->indicator = ZWISCHEN_ANZEIGE;

    cop_ptr->next_down = org_ptr;
    plow_box->next_up = org_ptr;
  }
  else {
    /* klinke Ende der upper_steps an step_inbetween-Liste */
    cop_ptr->next_down = org_ptr->next_down;
  };
}

/*********************************************************************************************/


void OXOX_case( k, cop_ptr, org_ptr, plow_box)  /*erstes Zeichen ist Buchstabe*/
/* moegliche org_ptr-Folge : XOOX bzw. XOOD */
int k;
step_inbetween *cop_ptr;
step_inbetween *org_ptr;
lower_stepbox *plow_box;
{
  int i;
  double integr_gewichte[3];

  /* berechne Integrationsgewichte fuer erstes Intervall in upper_steps-Gruppe */
  l3_integrale( integr_gewichte, cop_ptr->step, org_ptr->step);

  /* erstes upper_steps-Elememnt */
  cop_ptr->integr_weight = integr_gewichte[0];
  cop_ptr->indicator = ZWISCHEN_ANZEIGE;       /* wichtig insbesondere bei Einarbeitung von allgemein verteilten
						  Schaltzeiten mit generandom.c */
  /* Zuweisung der values-Komponenten wird in update_int_structures() zu Beginn des Randomisierungsteils
     erledigt */
 
  /* zweites upper_steps-Elememnt */
  cop_ptr->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
  cop_ptr = cop_ptr->next_down;
  
  cop_ptr->step = org_ptr->step;
  cop_ptr->integr_weight = integr_gewichte[1];
  cop_ptr->indicator = ZWISCHEN_ANZEIGE;
  cop_ptr->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
    cop_ptr->values[0][k] = copy_vector( org_ptr->values[0][k]);

  /* drittes upper_steps-Elememnt */
  cop_ptr->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
  cop_ptr = cop_ptr->next_down;
  org_ptr = org_ptr->next_down;  

  cop_ptr->step = org_ptr->step;
  cop_ptr->integr_weight = integr_gewichte[2]; /* vorlaeufig */
  /* cop_ptr->indicator wird erst durch folgende Integrationsgewichte bestimmt */
  cop_ptr->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
    cop_ptr->values[0][k] = copy_vector( org_ptr->values[0][k]);

  org_ptr = org_ptr->next_down;

  /* berechne Integrationsgewichte fuer zweites Intervall in upper_steps-Gruppe */
  l3_integrale( integr_gewichte, cop_ptr->step, org_ptr->step);
  cop_ptr->integr_weight += integr_gewichte[0];
  cop_ptr->indicator = integr_gewichte[0];

  /* viertes upper_steps-Elememnt */
  cop_ptr->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
  cop_ptr = cop_ptr->next_down;

  cop_ptr->step = org_ptr->step;
  cop_ptr->integr_weight = integr_gewichte[1];
  cop_ptr->indicator = ZWISCHEN_ANZEIGE;
  cop_ptr->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
    cop_ptr->values[0][k] = copy_vector( org_ptr->values[0][k]);

  /* fuenftes upper_steps-Elememnt */
  cop_ptr->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
  cop_ptr = cop_ptr->next_down;
  org_ptr = org_ptr->next_down;  

  cop_ptr->step = org_ptr->step;
  cop_ptr->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
    cop_ptr->values[0][k] = copy_vector( org_ptr->values[0][k]);

  /* unterscheide, letztes Element der Intervallgruppe gleichzeitig das erste der naechsten
     Intervallgruppe ist oder nicht */
  if (org_ptr->indicator == DISC_ANZEIGE) {
    cop_ptr->integr_weight = integr_gewichte[2];
    cop_ptr->indicator = DISC_ANZEIGE;
  } 
  else {
    cop_ptr->integr_weight = integr_gewichte[2] + org_ptr->indicator;
    cop_ptr->indicator = org_ptr->indicator;
  };
#ifdef KOSTEN
  log_kosten(3);
#endif
  /* evtl. muss ein dummy-Element an das Ende der step_inbetween-Kette angehaengt werden */
  if (org_ptr->next_down == NULL) {
    org_ptr->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
    org_ptr = org_ptr->next_down;
    org_ptr->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
    org_ptr->values[1] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
    for( i=0; i<ac_no_detT; i++) {
	org_ptr->values[0][i] = new_vector();
	org_ptr->values[1][i] = new_vector();
      };
    org_ptr->integr_weight = 0.0;
    org_ptr->next_down = NULL;

    /* das sind die Kennzeichen eines dummy-Elements */
    org_ptr->step = 0.0;                    
    org_ptr->indicator = ZWISCHEN_ANZEIGE;

    cop_ptr->next_down = org_ptr;
    plow_box->next_up = org_ptr;
  }
  else {
    /* klinke Ende der upper_steps an step_inbetween-Liste */
    cop_ptr->next_down = org_ptr->next_down;
  };
}

/***********************************************************************************************
Bemerkung zur Verwaltung der Interpolationsfelder fuer die p_d(t)'s :
  
Die Datenstruktur dieser Interpolationsfelder ist die gleiche wie die der Interpolationsfelder
fuer die p_d(t,tau_d), also stehen dieselben Funktionen aus integrate_interpolate.c zur 
Verfuegung. Alles, was dort allgemein geschrieben steht, bleibt gueltig ! Allerdings ist fuer 
diese Felder die Anpassung an die laufende Zeit wesentlich einfacher, da mit jedem Schritt
nur ein neuer Stuetzpunkt in das Interpolationsfeld eingebaut werden, naemlich das Ergebnis
fuer p_d(t) zum neuen Zeitpunkt transient_time + stepsize. Nach jedem angenommenen Schritt
wird das Interpolationsfeld abwechselnd komplett neu aufgebaut oder nur der aelteste Stuetzpunkt
durch den neuesten ersetzt. Daneben sind die Faelle, dass sich der aktuellste Stuetzpunkt aufgrund 
einer Schrittablehnung verschiebt ( Aenderung in Lage und Funktionswerten) sowie, dass sich 
nur seine Funktionswerte aufgrund einer Iteration aendern, durch die Funktionen rejected_update()
und repeated_update() abgedeckt.

Und so wird das Interpolationsfeld in der Anlaufrechnung zu Beginn und nach jeder Diskontinuitaet
aufgebaut, wobei a, b, c, d, e und f fuer die nacheinander errechneten Stuetzpunkte von p_d(t)
stehen ( x : Nullinitialisierung ):

                 ( BITTE AUF GANZEM BILDSCHIRM ANSCHAUEN )

Komp. 0: x  empty_a()  a  a_ab()  a  abcc_..()  a  cdbe_cdef()  a  abcd_cdbe()  c  cdbe_cdef()   c
Komp. 1: x             x          b             b               b               d                d
Komp. 2: x  -------->  x  ----->  x  ------->   c  ---------->  c  ---------->  b  ----------->  e ->
Komp. 3: x             x          x             c               d               e                f 

( Die Zustaende beziehen auf den Zeitpunkt direkt vor Aufruf einer der Runge-Kutta-Methoden )

Damit sind wohl im wesentlichen die Namen der einzelnen Funktionen erklaert ! 
Nach Anlaufrechnung werden cdbe_cdef() und abcd_cdbe() immer abwechselnd aufgerufen.

Aus der Anpassungsstrategie folgt, dass der aktuellste Stuetzpunkt immer auf der 3.Komponente
des Interpolationsfeldes sitzt.

Die Funktionswertevektoren der neuen Stuetzpunkte werden aus der upgedateten daten_basis gelesen. 

************************************************************************************************/

void empty_a( stepsize, daten_basis, pdt_fields )
/* setzt ersten Stuetzpunkt nach Diskontinuitaet ins Interpolationsfeld */

double stepsize;
solve_basis *daten_basis;
interpolation_field *pdt_fields;
{
  int i;

  for( i=0; i<ac_no_detT; i++) {
      pdt_fields[i][0].int_point = stepsize;
      del_vector( &(pdt_fields[i][0].y_values));
      pdt_fields[i][0].y_values = copy_vector( (daten_basis->p_d)[i]);
  };
}
/**************************************************************************************************/
void a_ab( daten_basis, pdt_fields )
/* setzt zweiten Stuetzpunkt nach Diskontinuitaet ins Interpolationsfeld */

solve_basis *daten_basis;
interpolation_field *pdt_fields;
{
  int i;

  for( i=0; i<ac_no_detT; i++) {
      del_vector( &(pdt_fields[i][1].y_values));
      pdt_fields[i][1].y_values = copy_vector( (daten_basis->p_d)[i]);
  };  
}
/*************************************************************************************************/
void abcc_int_points( stepsize, pdt_fields)
/* reorganisiert die relative Lage der Stuetzpunkte fuer Hinzunahme des dritten Stuetzpunkts nach
   Diskontinuitaet. Damit danach cdbe_cdef() angewandt werden kann, muss der neue Stuetzpunkt
   auch auf der Komponente 3 eingesetzt werden. */
   
double stepsize;
interpolation_field *pdt_fields;
{
  int i;
  
  pdt_fields[0][1].int_point = pdt_fields[0][0].int_point;
  /* pdt_fields[0][0].int_point bleibt unveraendert */
  pdt_fields[0][2].int_point = pdt_fields[0][0].int_point + stepsize;
  pdt_fields[0][3].int_point = pdt_fields[0][2].int_point;

  /* beachte : Lage der Stuetzpunkte eines Interpolationsfeldes fuer
               die p_d(t)'s fuer alle diese Felder gleich !      */
  for( i=1; i<ac_no_detT; i++) {
    /* pdt_fields[i][0].int_point bleibt unveraendert */
    pdt_fields[i][1].int_point = pdt_fields[0][1].int_point;
    pdt_fields[i][2].int_point = pdt_fields[0][2].int_point;
    pdt_fields[i][3].int_point = pdt_fields[0][3].int_point;
  };
}
/**********************************************************************************************/
void abcc_y_values( daten_basis, pdt_fields)
/* setzt Funktionswertevektor des dritten Stuetzpunkts nach Diskontinuitaet ins Interpolationsfeld.
   Damit danach cdbe_cdef() angewandt werden kann, muss der neue Stuetzpunkt auch auf der 
   Komponente 3 eingesetzt werden. */

solve_basis *daten_basis;
interpolation_field *pdt_fields;
{
  int i;

  for( i=0; i<ac_no_detT; i++) {
    del_vector( &(pdt_fields[i][2].y_values));
    pdt_fields[i][2].y_values = copy_vector( daten_basis->p_d[i]);
    del_vector( &(pdt_fields[i][3].y_values));
    pdt_fields[i][3].y_values = copy_vector( daten_basis->p_d[i]);
    newton_per_3( pdt_fields[i]);
  };
}
/*********************************************************************************************/
void repeated_update( daten_basis, pdt_fields)
/* erneuert nur den Funktionswertevektor des des letzten Stuetzpunkts */

solve_basis *daten_basis;
interpolation_field *pdt_fields;
{
  int i;

  for( i=0; i<ac_no_detT; i++) {
    del_vector( &(pdt_fields[i][3].y_values));
    pdt_fields[i][3].y_values = copy_vector( daten_basis->p_d[i]);
    newton_replace_1( pdt_fields[i]);
  };
}
/***************************************************************************************************/
void abcd_cdbe( stepsize, daten_basis, pdt_fields)
/* ersetzt unter Neuorganisation den aeltesten durch einen neuen Stuetzpunkt und schafft so, die
   Voraussetzung fuer einen spaeteren Aufruf von cdbe_cdef() */
/* Voraussetzung : Je groesser die Komponente des Interpolationsfeldes, desto neuer der 
                   dortige Stuetzpunkt.  oder abcc-Muster !                              */

double stepsize;
solve_basis *daten_basis;
interpolation_field *pdt_fields;
{
  int i;

  pdt_fields[0][2].int_point = pdt_fields[0][1].int_point - pdt_fields[0][2].int_point;
  pdt_fields[0][1].int_point = pdt_fields[0][3].int_point - pdt_fields[0][0].int_point;
  pdt_fields[0][3].int_point = pdt_fields[0][1].int_point + stepsize;
  pdt_fields[0][0].int_point = pdt_fields[0][1].int_point;

  del_vector( &(pdt_fields[0][0].y_values));
  pdt_fields[0][0].y_values = pdt_fields[0][2].y_values;

  pdt_fields[0][2].y_values = pdt_fields[0][1].y_values;

  pdt_fields[0][1].y_values = pdt_fields[0][3].y_values;

  pdt_fields[0][3].y_values = copy_vector( daten_basis->p_d[0]);
  newton_per_4( pdt_fields[0]);

  for( i=1; i<ac_no_detT; i++) {
    /* beachte : Lage der Stuetzpunkte eines Interpolationsfeldes fuer
       die p_d(t)'s fuer alle diese Felder gleich !      */
    pdt_fields[i][0].int_point = pdt_fields[0][0].int_point;
    pdt_fields[i][1].int_point = pdt_fields[0][1].int_point;
    pdt_fields[i][2].int_point = pdt_fields[0][2].int_point;
    pdt_fields[i][3].int_point = pdt_fields[0][3].int_point;

    del_vector( &(pdt_fields[i][0].y_values));
    pdt_fields[i][0].y_values = pdt_fields[i][2].y_values;

    pdt_fields[i][2].y_values = pdt_fields[i][1].y_values;

    pdt_fields[i][1].y_values = pdt_fields[i][3].y_values;

    pdt_fields[i][3].y_values = copy_vector( daten_basis->p_d[i]);
    newton_per_4( pdt_fields[i]);

  };
}
/**********************************************************************************/
void cdbe_cdef( stepsize, daten_basis, pdt_fields)
/* ersetzt unter geringerem Aufwand als Neuorganisation den aeltesten durch einen
   neuen Stuetzpunkt und schafft dabei die Voraussetzungen fuer einen spaeteren
   Aufruf von abcd_cdbe() */
/* Voraussetzung : aeltester Stuetzpunkt auf 3.Komponente, cdbe-Muster !  */

double stepsize;
solve_basis *daten_basis;
interpolation_field *pdt_fields;
{
  int i;


  /* pdt_fields[0][1].int_point bleibt unveraendert */
  pdt_fields[0][2].int_point = pdt_fields[0][3].int_point;
  pdt_fields[0][3].int_point = pdt_fields[0][2].int_point + stepsize;
  pdt_fields[0][0].int_point = pdt_fields[0][2].int_point;

  del_vector( &(pdt_fields[0][2].y_values));
  pdt_fields[0][2].y_values = pdt_fields[0][3].y_values;

  pdt_fields[0][3].y_values = copy_vector( daten_basis->p_d[0]);
  newton_replace_2( pdt_fields[0]);

  for( i=1; i<ac_no_detT; i++) {

    /* beachte : Lage der Stuetzpunkte eines Interpolationsfeldes fuer
       die p_d(t)'s fuer alle diese Felder gleich !      */
    /* pdt_fields[i][1].int_point = pdt_fields[0][1].int_point; bleibt unveraendert */
    pdt_fields[i][0].int_point = pdt_fields[0][0].int_point;
    pdt_fields[i][2].int_point = pdt_fields[0][2].int_point;
    pdt_fields[i][3].int_point = pdt_fields[0][3].int_point;

    del_vector( &(pdt_fields[i][2].y_values));
    pdt_fields[i][2].y_values = pdt_fields[i][3].y_values;

    pdt_fields[i][3].y_values = copy_vector( daten_basis->p_d[i]);
    newton_replace_2( pdt_fields[i]);

  };
}
/****************************************************************************************/
void rejected_update( stepsize, daten_basis, pdt_fields)
/* ersetzt nur den letzten Stuetzpunkt ( Aenderung in Werten und Lage ) */

double stepsize;
solve_basis *daten_basis;
interpolation_field *pdt_fields;
{
  int i;

  if (pdt_fields[0][2].int_point > 0.0)     
    pdt_fields[0][3].int_point = pdt_fields[0][2].int_point + stepsize;
  else 
    pdt_fields[0][3].int_point = pdt_fields[0][1].int_point + stepsize;

  /* pdt_fields[0][0].int_point bleibt unveraendert ! */
    
  del_vector( &(pdt_fields[0][3].y_values));
  pdt_fields[0][3].y_values = copy_vector( daten_basis->p_d[0]);
  newton_replace_1( pdt_fields[0]);

  for( i=1; i<ac_no_detT; i++) {
    /* pdt_fields[i][0].int_point = pdt_fields[0][0].int_point;bleibt unveraendert */
    pdt_fields[i][3].int_point = pdt_fields[0][3].int_point;    
  
    del_vector( &(pdt_fields[i][3].y_values));
    pdt_fields[i][3].y_values = copy_vector( daten_basis->p_d[i]);
    newton_replace_1( pdt_fields[i]);
  };
}
/*********************************************************************************/

