/*******************************************************************************
Modul:              main.c

Parameter explizit: TransSolve  [options] <net-name>
                    options siehe unten.

Project Director:   Reinhard German

Programmer:         Joerg W. Mitzlaff

History:            10.94 (Dipl.Arbeit)
*******************************************************************************/

/* #define DEBUG */
/* #define KOSTEN*/

#define FNAME_SIZE  256
#define ANZ_KOSTEN 7

#include <stdio.h>
#include <sys/time.h>
#include <sys/resource.h>
#include "Options.h"


/* Import aus frontend.c */
extern void PrepeareSolve();

/* Import aus frontend.c */
extern int nmarking;

/* Exported Variable */
TransientOptions  transientOptions={ 10.0,0.1,Duration,0.1,10,1e-7,False,False,False,False };
FILE *kosten_file;


/* file lokale variable */
static String netname;

void help_and_exit()
{
  printf("Usage: TransSolve  [options] <net-name> \n\n");
  printf("       -t DOUBLE transient time                      (default = 10) \n");
  printf("       -p DOUBLE general precision                   (default = 1e-7) \n");
  printf("       -o STRING output-mode one of: curve, point    (default = curve) \n");
  printf("       -r DOUBLE stepsize of result output           (default = 1) \n");
  printf("       -s DOUBLE stepsize of iterative calculation   (default = 0.1) \n");
  printf("       -c INT    cluster size                        (default = 10) \n");
  printf("       -x        show result curves graphically      (default = off) \n");
  printf("       -a        variant a of any case               (default = off) \n");
  printf("       -O        fixed stepsize algorithm for case 3 (default = off) \n");
  printf("       -k        keep intermediate files             (default = off) \n");
  printf("       -h        this help message \n");
  printf("Remark: -c are only relevant in case 1 and 2; -s are only relevant in case 3 \n");
  printf("        for the adaptive algorithm only -t, -p and -o are relevant           \n");
  exit(-1);
}

void main(argc,argv) /* parsiert lediglich die argumente. */
int argc;
char **argv;
{
  int i;

  if(argc==1)
  {
    printf("\nError:no netname specified.\n\n");
    help_and_exit();
  }
  
  if(argc==2)
    netname=argv[1];
  else
  {
    for (i=1;i<argc-1;i++)  /* gehe ueber alle argumente: argc=0 ist prg.name, argc-1 ist letzte arg. */
    {
      if(!strcmp(argv[i],"-t"))
	sscanf(argv[++i],"%lf",&transientOptions.transient_time);
      else
	if(!strcmp(argv[i],"-s"))
	  sscanf(argv[++i],"%lf",&transientOptions.stepsize_method);
	else
	  if(!strcmp(argv[i],"-o"))
	    transientOptions.output_data= (strcmp(argv[++i],"point") ? Duration : Point);
	  else
	    if(!strcmp(argv[i],"-r"))
	      sscanf(argv[++i],"%lf",&transientOptions.stepsize_result);
	    else
	      if(!strcmp(argv[i],"-c"))
		sscanf(argv[++i],"%d",&transientOptions.cluster_size);
	      else
		if(!strcmp(argv[i],"-p"))
		  sscanf(argv[++i],"%lf",&transientOptions.precision);
		else
		  if(!strcmp(argv[i],"-k"))
		    transientOptions.keep_files=True;
		  else
		    if(!strcmp(argv[i],"-x"))
		      transientOptions.graphics_desired=True;
		    else
		      if(!strcmp(argv[i],"-O"))
			transientOptions.old_solve_alg=True;
		      else
			if(!strcmp(argv[i],"-a"))
			  transientOptions.old_case_iii=True;
			else
			  help_and_exit();
    }
    if(i==argc)
    {
      printf("\nError:no netname specified.\n\n");
      help_and_exit();
    }
    else
      netname=argv[i];
  }

  if(!strcmp(netname,"-h"))
    help_and_exit();

  /* evaluate arguments and write to transientOptions */
  PrepeareSolve(netname);
}



#ifdef KOSTEN         /* for TEST und MESSEN */

void log_kosten(wo)
int wo;
{
  static double last_zeit;
  static double zeitkosten[ANZ_KOSTEN];
  static long speicherkosten;
  static long rrg_speicherkosten;

  char k_file[FNAME_SIZE];
  struct rusage ru;
  double neue_zeit;
  double zeitkosten_sum;
  int i;
  

  if(wo == -1)   /* Init */
  {
    for(i=0;i<ANZ_KOSTEN;i++)
    {
      zeitkosten[i] = 0;
    }
    last_zeit = 0;
    speicherkosten = 0;

    /* open file */
    sprintf(k_file,"%s.kosten_file",netname);    
    if( (kosten_file = fopen(k_file,"a")) == NULL )
    {
      fprintf(stderr,"Unable to open file '%s'\n", k_file);
      exit(1);
    }
  }
  else if(wo == -2)   /* ende --> ausgabe */
  {
    zeitkosten_sum = 0;
    for(i=0;i<ANZ_KOSTEN;i++)
    {
      zeitkosten_sum += zeitkosten[i];
    }

    fprintf(kosten_file,"nmark: %d zsum: %g mem: %ld ", nmarking, zeitkosten_sum, speicherkosten);

    for(i=0;i<ANZ_KOSTEN;i++)
    {
      fprintf(kosten_file,"sum[%d]: %g ",i, zeitkosten[i]);
    }
    fprintf(kosten_file,"rrg-mem: %ld\n", rrg_speicherkosten);	   
    
    fclose(kosten_file);

    /* Bildschirmausgabe */
    printf("zeitkosten : %g\n", zeitkosten_sum);
    printf("einzelanteile der zeitkosten : \n");
    for(i=0;i<ANZ_KOSTEN;i++) 
    { 
      printf("sum[%d]: %g ",i, zeitkosten[i]); 
    } 
    printf("\nrrg-Gen-speicher: %ld\n", rrg_speicherkosten);
    printf("speicherkosten : %ld\n", speicherkosten);
    
  }
  else
  {
    if(wo == 0)       /* who = children (fuer generate_rrg) ? */
    {
      if(getrusage(RUSAGE_CHILDREN,&ru))
      {
	perror("error at getrusage");
	exit(1);
      }
    }
    else
    {
      if(getrusage(RUSAGE_SELF,&ru))
      {
	perror("error at getrusage");
	exit(1);
      }
    }

    /* system und user-time zusammen addiedern */
    ru.ru_utime.tv_sec += ru.ru_stime.tv_sec;
    ru.ru_utime.tv_usec += ru.ru_stime.tv_usec;

    /* differenz zu vorigen aufruf bilden */
    neue_zeit = (double)ru.ru_utime.tv_sec;
    neue_zeit += ( ((double)ru.ru_utime.tv_usec) / 1000000 );
    zeitkosten[wo] += neue_zeit - last_zeit;

    if(wo == 0)   /* children zeit und speicher zaehlen extra */
    {  
      /* speicherbedarf erfassen */
      rrg_speicherkosten = (ru.ru_maxrss * getpagesize ()) >> 10;
    }
    else
    {
      last_zeit = neue_zeit;
      speicherkosten = (ru.ru_maxrss * getpagesize ()) >> 10;
    }

/*     printf("speicher: %ld worauf %d ", speicherkosten, wo); */
/*     for(i=0;i<ANZ_KOSTEN;i++) */
/*     { */
/*       printf("sum[%d]: %g ",i, zeitkosten[i]); */
/*     } */
/*     printf("rrg-Gen-speicher: %ld\n", rrg_speicherkosten);	  */
  }
}

#endif








