/*******************************************************************************
Modul:              mat_proc.c 

Purpose:            stellt grundlegende Skalar- Vektor- und Martix-Operationen zur Verf.

Project Director:   Reinhard German

Programmer:         Joerg W. Mitzlaff

History:            10.94 (Dipl.Arbeit)
                    angelehnt an DSPNAnalysis/rand_procs.c
*******************************************************************************/
/* define DEBUG_AC */
#include <stdio.h>		/* for NULL */
#include <stdlib.h>
#include <math.h>
#include "frontend.h"
#include "portab.h"           /* not system dependend: void memmove(dest,src,length) */

/* export nach frontend.c */
double maxdiff;     

/* delete entry */
#define del_entry(item) { \
  item->next = first_free_item; \
  first_free_item = item; \
}

/* foreward definition of private functions */
static sp_entry  *new_entry();
static void      del_rows_from_matrix();


/*	emalloc tries to allocate required memory space and
	tests if it was successfully allocated; if not => exit */
char *
emalloc(bytes)
	int bytes;
{
	int *point;

	if (!bytes)
		return NULL;
	point = (int *)malloc((unsigned)(bytes));
	if (point == NULL)
	{	printf("ERROR:%s (line %u): not enough memory to allocate %d Bytes.\n", __FILE__,__LINE__,bytes);
		exit(3);
	}
	return (char *)point;	
}

/*	efree frees a memory pointer */
void
efree(point)
	char	*point;
{
	if (point == NULL)
		return;
	   free(point);
}

/*****************************************************************************************/
double max( a, b)
/* gibt das Maximum beider Zahlen zurueck */

  double a;
  double b;
  {
    if (a>b)
      return (a);
    else
      return (b);
}
/****************************************************************************************/  
double min( a, b)
/* gibt das Minimum beider Zahlen zurueck */

  double a;
  double b;
  {
    if (a<b)
      return (a);
    else
      return (b);
}
/**************************************************************************************/
double get_infnorm( vec)
/* berechnet die Unendlichnorm eines Vektors */

sp_vector *vec;
{
  int i;
  sp_entry *el_vec;
  double return_val = 0.0;

  el_vec = vec->entries;
  for( i=0;i<vec->no_entries;i++) {
    return_val += fabs(el_vec->val);
    el_vec = el_vec->next;
  };
  return(return_val);
}
/***************************************************************************************/
double get_sum_components( vec)
/* im Gegensatz zur Unendlichnorm werden hier nur die Komponenten (ohne Betragsbildung) aufsummiert */

sp_vector *vec;
{
  int i;
  sp_entry *el_vec;
  double return_val = 0.0;

  el_vec = vec->entries;
  for( i=0;i<vec->no_entries;i++) {
    return_val += el_vec->val;
    el_vec = el_vec->next;
  };
  return(return_val);
}
/*******************************************************************************************/
void set_negs_to_zero( vec)
/* setzt negative Komponenten eines Vektors auf 0.0 */

sp_vector *vec;
{
  sp_entry *enap1, *previous;
  
  for(enap1 = vec->entries, previous = NULL; enap1 != NULL; previous = enap1, enap1 = enap1->next) {
    if (enap1->val < 0.0) {
#ifdef DEBUG_AC
      printf(" Negatives Element %.9f bei Extrapolation/Polynomauswertung Null gesetzt !\n",enap1->val); 
#endif
      add_to_vector(vec, previous, enap1->num, (-1.0)*(enap1->val));
    };
  };    
}
/******************************************************************************************/

/* compare two doubles with predefined epsilon */
int d_comp(d1, d2,eps)
double d1;
double d2;
double eps;
{
	double	diff;

	diff = (d1 - d2);

	if(fabs(diff) < eps)
	{
	  return 1;
	}
	else	
	{
	  if(fabs(diff) > fabs(maxdiff))
	    maxdiff = diff;
	  return 0;
	}
}

/* vector / matrix entry operations */

static sp_entry *first_free_item = NULL;

/* get a new entry  (tobias bading) */
static sp_entry *
new_entry()
{
	static sp_entry		**buffers;
	static unsigned		number_of_buffers, last_used_buffer;

	if (!first_free_item)
	{	number_of_buffers = MEM_BUFFERS;
		buffers = (sp_entry **)emalloc (MEM_BUFFERS * sizeof (sp_entry *));
		buffers[0] = (sp_entry *)calloc (MEM_ITEMS, sizeof (sp_entry));
		last_used_buffer = 0;
		first_free_item = buffers[0] + 1;
		buffers[0]->next = NULL;
		return buffers[0];
	}
	{	register sp_entry *returnvalue = first_free_item;

		if (first_free_item->next)
			first_free_item = first_free_item->next;
		else
			if (++first_free_item - buffers[last_used_buffer] == MEM_ITEMS)
			{	if (++last_used_buffer == number_of_buffers)
				{	number_of_buffers += MEM_BUFFERS;
					buffers = (sp_entry **)realloc (buffers, 
							number_of_buffers * sizeof (sp_entry *));
				}
				buffers[last_used_buffer] = 
					(sp_entry *)calloc(MEM_ITEMS, sizeof (sp_entry));
				first_free_item = buffers[last_used_buffer];
			}
		returnvalue->next = NULL;
		return returnvalue;
	}
}


/* vector operations */

/* get a new vector */
sp_vector *
new_vector()
{
	sp_vector		*new_vec;

	new_vec = (sp_vector *)emalloc(sizeof(sp_vector));
	new_vec->num = 0;
	new_vec->no_entries = 0;
	new_vec->entries = NULL;
	new_vec->next = NULL;
	return new_vec;
}

/* clear vector */
void
clear_vector(vec)
	sp_vector	*vec;
{
	sp_entry	*e_walk, *h_walk;

	if ((vec == NULL) || (!vec->entries))
		return;
	for (e_walk = vec->entries; e_walk->next != NULL; e_walk = e_walk->next);
	e_walk->next = first_free_item;
	first_free_item = vec->entries;
	vec->entries = NULL;
	vec->no_entries = 0;
}

/* delete vector */
void
del_vector(vec)
	sp_vector	**vec;
{
	clear_vector(*vec);
	efree(*vec);
	*vec = NULL;
}

/* quick adding of values to a vector */
/* written by Tobias Bading */
sp_entry *
add_to_vector(vector, prev_item, index, value)
	sp_vector	*vector;
	register sp_entry *prev_item;
	register int	index;
	double		value;
{
	register sp_entry *prev_entry = NULL;
	register sp_entry *entry = vector->entries;

	if (is_zero(value))
		return prev_item;

	if (prev_item)
		entry = prev_item->next;

	while (entry)
		if (entry->num < index)
		{	prev_entry = entry;
			entry = entry->next;
		}
		else
			if (entry->num == index)
			{	entry->val += value;
				return entry;
			}
			else
				break;

	{	register sp_entry *new_e = new_entry();

		vector->no_entries++;
		new_e->num = index;
		new_e->val = value;
		new_e->next = entry;
		if (prev_entry)
			prev_entry->next = new_e;
		else
			if (prev_item)
				prev_item->next = new_e;
			else
				vector->entries = new_e;
		return new_e;
	}
}

/* copy vector */
sp_vector *
copy_vector(vec)
	sp_vector	*vec;
{
	sp_vector	*new_vec = new_vector();
	sp_entry	*entry_walk, *n_entry;
	sp_entry	*copy_walk = NULL;

	if(!vec)
	  return new_vec;
	    
	new_vec->no_entries = vec->no_entries;
	new_vec->num = vec->num;
	scan_list(entry_walk, vec->entries)
	{	n_entry = new_entry();
		n_entry->num = entry_walk->num;
		n_entry->val = entry_walk->val;
		if (copy_walk == NULL)
			new_vec->entries = n_entry;
		else	copy_walk->next = n_entry;
		copy_walk = n_entry;
	}
	return new_vec;
}

/* compare two vectors */
int comp_vector(vec1, vec2, eps)
sp_vector *vec1;
sp_vector *vec2;
double eps;
{
	sp_entry	*e_walk_1 = vec1->entries;
	sp_entry	*e_walk_2 = vec2->entries;

	while((e_walk_1 != NULL) || (e_walk_2 != NULL))
	{
	  if (e_walk_1 != NULL) {
		if ((e_walk_2 == NULL) || (e_walk_2->num > e_walk_1->num))
		{	if (!d_comp(e_walk_1->val, 0.0, eps))
				return 0;
			e_walk_1 = e_walk_1->next;
		} else
		if (e_walk_2->num < e_walk_1->num)
		{	if (!d_comp(e_walk_2->val, 0.0, eps))
				return 0;
			e_walk_2 = e_walk_2->next;
		} else
		{	/* e_walk_2->num == e_walk_1->num */
			if (!d_comp(e_walk_2->val, e_walk_1->val, eps))
				return 0;
			e_walk_1 = e_walk_1->next;
			e_walk_2 = e_walk_2->next;
		};
	  }
	  else 
	  {     /* nur e_walk_2 != NULL */
		if (!d_comp(e_walk_2->val, 0.0, eps))
		        return 0;
		e_walk_2 = e_walk_2->next;
	  };
	}
	return 1;
}

/* compare two vectors by relative errors in components */
/* both vectors have only non-negative entries */
int rel_comp_vector(vec1, vec2, eps)
sp_vector *vec1;
sp_vector *vec2;
double eps;
{
        double          rel_err,max_el;
	sp_entry	*e_walk_1 = vec1->entries;
	sp_entry	*e_walk_2 = vec2->entries;
	
	while((e_walk_1 != NULL) || (e_walk_2 != NULL))
	{
	  if (e_walk_1 != NULL) {
		if ((e_walk_2 == NULL) || (e_walk_2->num > e_walk_1->num))
		{	if (!d_comp(e_walk_1->val, 0.0, MAX_ABS_PREC))
				return 0;
			e_walk_1 = e_walk_1->next;
		} else
		if (e_walk_2->num < e_walk_1->num)
		{	if (!d_comp(e_walk_2->val, 0.0, MAX_ABS_PREC))
				return 0;
			e_walk_2 = e_walk_2->next;
		} else
		{	/* e_walk_2->num == e_walk_1->num */
                        max_el = max(e_walk_2->val, e_walk_1->val);
			if (max_el > MAX_ABS_PREC) 
			  rel_err = fabs(e_walk_2->val - e_walk_1->val)/max_el;
			else 
			  rel_err = fabs(e_walk_2->val - e_walk_1->val)/MAX_ABS_PREC;
			if (rel_err > eps)
				return 0;
			e_walk_1 = e_walk_1->next;
			e_walk_2 = e_walk_2->next;
		};
	  }
	  else 
	  {     /* nur e_walk_2 != NULL */
		if (!d_comp(e_walk_2->val, 0.0, MAX_ABS_PREC))
		        return 0;
		e_walk_2 = e_walk_2->next;
	  };
	}
	return 1;
}

/* matrix operations */

/* get a new matrix */
sp_matrix *
new_matrix()
{
	sp_matrix	*new_mat;

	new_mat = (sp_matrix *)emalloc(sizeof(sp_matrix));
	new_mat->rows = NULL;
	return new_mat;
}

/* clear matrix */
void
clear_matrix(mat)
	sp_matrix	*mat;
{
	sp_vector	*v_walk, *h_walk;

	if (mat == NULL)
		return;
	for (v_walk = mat->rows; v_walk != NULL;)
	{	h_walk = v_walk->next;
		del_vector(&v_walk);
		v_walk = h_walk;
	}
	mat->rows = NULL;
}

/* delete matrix */
void
del_matrix(mat)
	sp_matrix	**mat;
{
	clear_matrix(*mat);
	efree(*mat);
	*mat = NULL;
}

/* add an entry to a matrix; if the entry already exist, add the values */
void
add_entry_to_matrix(mat, row_nr, col_nr, value)
	sp_matrix	*mat;
	int		row_nr, col_nr;
	double		value;
{
	sp_vector	*vec_walk, *h_vec_walk = NULL, *new_vec;
	static sp_vector *old_vec;
	static sp_matrix *old_mat;

	if (is_zero(value) || (mat == NULL))
		return;

/*	if ((old_mat != mat) || (!old_vec) || (old_vec->num > row_nr))*/
		old_vec = mat->rows;
	old_mat = mat;

	for (vec_walk = old_vec; ; vec_walk = vec_walk->next)
	{	if ((vec_walk == NULL) || (vec_walk->num > row_nr))
		{	new_vec = new_vector();
			new_vec->num = row_nr;
			new_vec->no_entries = 1;
			new_vec->next = vec_walk;
			if (h_vec_walk == NULL)
				mat->rows = new_vec;
			else	h_vec_walk->next = new_vec;
			new_vec->entries = new_entry();
			new_vec->entries->num = col_nr;
			new_vec->entries->val = value;
			old_vec = new_vec;
			return;
		}
		if (vec_walk->num == row_nr)
		{	add_to_vector(vec_walk, NULL, col_nr, value);
			old_vec = vec_walk;
			return;
		}
		h_vec_walk = vec_walk;
	}
}

/* delete rows from a matrix */
static void
del_rows_from_matrix(mat, row_num_to)
	sp_matrix	*mat;
	int		row_num_to;
{
	sp_vector	*v_walk, *hv_walk;
	sp_entry	*e_walk;
	int		bit;

	if (mat == NULL)
		return;
	hv_walk = NULL;
	scan_list(v_walk, mat->rows)
	{	if (v_walk->num <= row_num_to)
		{	efree(v_walk);
			mat->rows = v_walk->next;
			continue;
		}
		bit = 1;
		while (e_walk = v_walk->entries)
		{	if (e_walk->num > row_num_to)
				break;
			v_walk->entries = e_walk->next;
			del_entry(e_walk);
			if (!--v_walk->no_entries)
			{	if (hv_walk == NULL)
					mat->rows = v_walk->next;
				else	hv_walk->next = v_walk->next;
				efree(v_walk);
				bit = 0;
				break;
			}
		}
		if (bit)
			hv_walk = v_walk;
	}
}

/* copy matrix */
sp_matrix *
copy_matrix(mat)
	sp_matrix	*mat;
{
	sp_matrix	*new_mat = new_matrix();
	sp_vector	*new_vec, *v_walk;
	sp_vector	*copy_walk = NULL;

	scan_list(v_walk, mat->rows)
	{	new_vec = copy_vector(v_walk);
		if (copy_walk == NULL)
			new_mat->rows = new_vec;
		else	copy_walk->next = new_vec;
		copy_walk = new_vec;
	}
	return new_mat;
}

/* calculations */

/* vector1 += vector2 * factor */
void
vector_add_mult(vec1, vec2, factor)
	sp_vector	*vec1, *vec2;
	double		factor;
{
	sp_entry	*entry1 = vec1->entries;
	sp_entry	*hentry1 = NULL, *hentry;
	sp_entry	*entry2 = vec2->entries;
	int		num1, num2;

	while((entry1 != NULL) || (entry2 != NULL))
	{	num1 = entry1 == NULL ? 10000000 : entry1->num;
		num2 = entry2 == NULL ? 10000000 : entry2->num;
		if (num1 < num2)
		{	hentry1 = entry1;
			entry1 = entry1->next;
		}
		else if (num1 == num2)
		{	entry1->val += entry2->val * factor;
			hentry1 = entry1;
			entry1 = entry1->next;
			entry2 = entry2->next;
		}
		else /* num1 > num2 */
		{	hentry = new_entry();
			hentry->num = num2;
			hentry->val = entry2->val * factor;
			hentry->next = entry1;
			if (hentry1 == NULL)
				vec1->entries = hentry;
			else	hentry1->next = hentry;
			vec1->no_entries++;
			hentry1 = hentry;
			entry2 = entry2->next;
		}
	}
}

/* multiply a vector with a matrix */
/* vec = vec * mat */
void
mult_vector_matrix(vec, mat)
	sp_vector	*vec;
	sp_matrix	*mat;
{
	sp_vector	*pre_vec = new_vector();
	sp_vector	*vec_walk;
	register sp_entry *vec_e_walk;

	memmove(pre_vec,vec, sizeof(sp_vector));
	vec->no_entries = 0;
	vec->entries = NULL;
	vec_walk = mat->rows;

	scan_list(vec_e_walk, pre_vec->entries)
	{	register double	phi_factor = vec_e_walk->val;
		register sp_entry *prev_entry = NULL, *mat_e_walk;

		while (vec_e_walk && vec_walk && (vec_walk->num < vec_e_walk->num))
		  vec_walk = vec_walk->next;
		if (!vec_walk || !vec_e_walk)
		  break;
		if (vec_walk->num > vec_e_walk->num)
		  continue;
		
		scan_list(mat_e_walk, vec_walk->entries)
			prev_entry = add_to_vector(vec, prev_entry, mat_e_walk->num, 
					mat_e_walk->val * phi_factor);
	}
	del_vector(&pre_vec);
}


/*******************************************************************************/
void mult_matrix_matrix(mat1, mat2)                      /* mat1 = mat1 * mat2 */
/*******************************************************************************/
     sp_matrix *mat1;
     sp_matrix *mat2;
{
  sp_matrix *res_mat;  /* Ergebnis Matrix */
  sp_vector *nvap;     /* next vaector pointer */

  nvap = mat1 -> rows;
  while(nvap)  /* gehe durch alle Zeilen */
  {
    mult_vector_matrix(nvap, mat2);

    nvap = nvap -> next;
  }  
}



/*******************************************************************************/
void matrix_add_mult(mat1, mat2, factor)        /* mat1 = mat1 + mat2 * factor */
/*******************************************************************************/
     sp_matrix *mat1;
     sp_matrix *mat2;
     double factor;
{
  sp_vector *nvap1;     /* next vaector pointer */
  sp_vector *nvap2;     
  sp_entry *nep;   /* next entry pointer */

  nvap1 = mat1 -> rows;
  nvap2 = mat2 -> rows;
  while(nvap2)  /* gehe durch mat2 Zeilen */
  {
    if(!nvap1) /* keine mat1 zeilen mehr? */
    {
      /* copy zeileneintraege nach mat1 und mult with factor */
      nep = nvap2 -> entries;
      while(nep)     /* gehe durch mat2 Spalten */
      {
	add_entry_to_matrix(mat1, nvap2 -> num, nep -> num, nep->val * factor);
	
	nep = nep -> next;
      }
      nvap2 = nvap2 -> next;
    }
    else
    {
      if(nvap1->num < nvap2->num)
      {
	nvap1 = nvap1 -> next;
      }
      else if(nvap1->num == nvap2->num)
      {
	vector_add_mult(nvap1,nvap2,factor);
	nvap1 = nvap1 -> next;
	nvap2 = nvap2 -> next;	
      }
      else /* nvap1->num > nvap2->num */
      {
	/* copy zeileneintraege nach mat1 und mult with factor */
	nep = nvap2 -> entries;
	while(nep)     /* gehe durch mat2 Spalten */
	{
	  add_entry_to_matrix(mat1, nvap2 -> num, nep -> num, nep->val * factor);
	  
	  nep = nep -> next;
	}
	nvap2 = nvap2 -> next;
      }
    }
  }  
}


/*******************************************************************************/
void mult_vector_skalar(vec,faktor)                     /* vec  = vec * factor */
/*******************************************************************************/
sp_vector *vec;
double faktor;
{
    sp_entry *nep;

    nep = vec -> entries;
    while(nep)
    {
      nep -> val *= faktor;

      nep = nep -> next;
    }
}


/*******************************************************************************/
void show_entry(e)
/*******************************************************************************/
        sp_entry *e;
{
/*	printf("[%d] = %lf\n", e->num, e->val); */
/* besser fuer kleine Matrizen: */
        printf("[%d] %.10lf , ", e->num, e->val);
}

/*******************************************************************************/
void show_vector(v)
/*******************************************************************************/
	sp_vector *v;
{
	sp_entry *e;

	if(v)
	{

/*	  printf("vector [%d] %d entries\n", v->num, v->no_entries);  */

/* besser fuer kleine Matrizen: */
	  printf("%d:    ", v->num);

	  scan_list(e, v->entries)
	    show_entry(e);

/* besser fuer kleine Matrizen: */
	  printf("\n");

	}
	else
	  printf("vector has 0 entries\n");
}

/*******************************************************************************/
void show_matrix(m)
/*******************************************************************************/
	sp_matrix *m;
{
	sp_vector *v;

	printf("*** Matrix ***\n");
	scan_list(v, m->rows)
		show_vector(v);
	printf("\n");
}

/*******************************************************************************/
int count_elements(m)
/*******************************************************************************/
	sp_matrix *m;
{
	sp_vector *v;
	int summe;
	
	summe = 0;
	scan_list(v, m->rows)
		summe += v -> no_entries;

	return(summe);
}

