/*******************************************************************************
Modul:              new_rk.c 

Purpose:            verbessertes Runge-Kutta Loesungsverfahren
                    (mit verdraengenden expT und weniger Speicher)

Project Director:   Reinhard German

Programmer:         Joerg W. Mitzlaff

History:            10.94 (Dipl.Arbeit)
*******************************************************************************/

/* #define DEBUG_M */


#include <stdio.h>
#include <math.h>
#include "frontend.h"
#include "Options.h"
#include "shared.h"

/* Import aus mat_proc.c */
extern double maxdiff;

/* Import aus get_rrg.c */
extern int preempted_trans;

/* Import aus main.c */
extern TransientOptions transientOptions;

/* Import als showTransCurves.c */
extern void drawNextStep();
extern void showSolveStatus();

/* Import aus shared.c */
extern double ggTeiler();
extern sp_matrix *randomi();
extern PrepeareRandomisation *prepeare_rand();
extern FILE *open_solve();
extern void update_solve();
extern void close_solve();

/* Prototyping */
static int compare_pdt0();
static int middle_part();
static sp_vector *runge_kutta_differenziation();

/******************************************************************************/
void case_iii(tnethome,modelpath,netname,c)
/******************************************************************************/
char *tnethome;
char *modelpath;
char *netname;
solve_basis *c;
{
   FILE *ftmp;
   int time;
   int numDatas;
   int numCompDatas;
   int akt_output;
   double maxX;
   int i,j,k;
   int ndetT;
   sp_vector *temp_vec;
   double omega;             /* ggT von allen detT */
   double h;                 /* ggT von omega und method_stepsize */
   int *steps_in_tau;        /* Anzahl der Messpunkte jeder detT */
   int *pdt0_element;        /* zeigt auf p'd(t,0) Eintrag im p_d_mexp feld */
   int *pdttau_element;      /* zeigt auf p'd(t,tau) Eintrag im p_d_mexp feld */

   sp_vectors **p_d_mexp;    /* akt p'd(t,j) h<=j<=tau'd aus Schritt 1 */
   sp_vectors *p_d_t;        /* akt. p'd(t)  aus Schritt 2 (und Schritt 5) */
   sp_vectors *tmp_p_d_t;        /* akt. p'd(t)  aus Schritt 2 ohne schritt 5 */
   sp_vector *p_E_t;         /* akt. p'E(t)  aus Schritt 3 */
   sp_vectors *p_d_t_0;      /* akt. p'd(t,0) aus Schritt 4 */
   sp_vector *p_t;           /* p(t) ist Endloesung aus Schritt 7 und wird an update_solve() uebergeben */

   sp_vectors *old_pdttau;   /* altes p_d_t_tau, wird in Schritt 3 benoetigt */
   sp_vectors *old_pdt;      /* altes p_d_t, wird in Schritt 3 benoetigt */
   sp_vector *old_pEt;      /* altes p_E_t, wird in Schritt 3 benoetigt */

   sp_matrices *m_exp_Q_d_h; /* von Schritt 1 benoetigt */
   PrepeareRandomisation **prep; /* von Schritt 1 benoetigt (alternativ) */

   int iter;                /* fuer schleife des preemtion fall */
   double old_maxdiff;

   /**** Initialisation  ************************************************************/

   ndetT = c -> no_detT;

   /* KGV von allen tau und h und event. trans. time finden */
   if(transientOptions.output_data==Point)
     omega = transientOptions.transient_time;
   else
     omega = c->detMatrices[0].det_firing_time ; /* erster Durchlauf nun umsonst */

   for(i=0;i<ndetT;i++)                 
      omega = ggTeiler(omega, c->detMatrices[i].det_firing_time);
   h = ggTeiler(transientOptions.stepsize_method,omega);

   printf("internal step size h %s to %g.\n",
	  (transientOptions.stepsize_method!=h ? "has changed" : "was set"),
	  h);

   if(transientOptions.stepsize_result < transientOptions.stepsize_method)
   {
     transientOptions.stepsize_result = transientOptions.stepsize_method;
     printf("step size for output has changed to %g.\n", transientOptions.stepsize_result);
   }
   else
   {
     printf("step size for output was set to %g.\n", transientOptions.stepsize_result);
   }

   steps_in_tau = (int *)emalloc(sizeof(int)*ndetT);
   for(i=0;i<ndetT;i++)
   {
     steps_in_tau[i] = c->detMatrices[i].det_firing_time / h;   /* muss ganze Zahl rauskommen, da h ggT ist */
     steps_in_tau[i] ++;                                        /* da Intervall-werte von 0 bis tau gehen */
   }

   p_d_mexp = (sp_vectors **)emalloc(sizeof(sp_vectors *) * ndetT);
   for(i=0;i<ndetT;i++)
   {
      p_d_mexp[i] = (sp_vectors *)emalloc(sizeof(sp_vectors) * steps_in_tau[i]);
      for(j=0;j<steps_in_tau[i];j++)
         (p_d_mexp[i])[j] = new_vector();
   }

   pdt0_element = (int *)emalloc(sizeof(int)*ndetT);
   for(i=0;i<ndetT;i++)
      pdt0_element[i] = 0;

   pdttau_element = (int *)emalloc(sizeof(int)*ndetT);
   for(i=0;i<ndetT;i++)
      pdttau_element[i] = steps_in_tau[i] - 1;


   old_pdttau = (sp_vectors *)emalloc(sizeof(sp_vectors) * ndetT);
   for(i=0;i<ndetT;i++)
      old_pdttau[i] = new_vector();

   p_d_t = c -> p_d;

   tmp_p_d_t = (sp_vectors *)emalloc(sizeof(sp_vectors) * ndetT);
   for(i=0;i<ndetT;i++)
      tmp_p_d_t[i] = new_vector();

   old_pdt = (sp_vectors *)emalloc(sizeof(sp_vectors) * ndetT);
   for(i=0;i<ndetT;i++)
      old_pdt[i] = new_vector();

   p_E_t = c -> p_E;

   old_pEt = copy_vector(p_E_t);

   p_d_t_0 = (sp_vectors *)emalloc(sizeof(sp_vectors) * ndetT);
   for(i=0;i<ndetT;i++)
      p_d_t_0[i] = new_vector();

   temp_vec = new_vector();

   p_t = copy_vector(p_E_t); /* p_t = p(0) */

   for(i=0;i<ndetT;i++)
   {
     vector_add_mult(p_t, p_d_t[i], 1.0);
   }

#ifdef KOSTEN
  log_kosten(1);
#endif

   if(transientOptions.old_case_iii)
   {
     m_exp_Q_d_h = (sp_matrices *)emalloc(sizeof(sp_matrices) * ndetT);
     for(i=0;i<ndetT;i++)
       m_exp_Q_d_h[i] = randomi(c->detMatrices[i].Q_d, h);

#ifdef DEBUG_M
     for( i=0 ; i<c->no_detT;i++)
     {
       printf("\n e hoch (Q_d * h) Matrizen d=%d\n", i);
       show_matrix( m_exp_Q_d_h[i]);
     }
#endif

   }
   else
   {
     prep = (PrepeareRandomisation **)emalloc(sizeof(PrepeareRandomisation *) * ndetT);
     for(i=0;i<ndetT;i++)
       prep[i] = prepeare_rand(c->detMatrices[i].Q_d, h);

#ifdef DEBUG_M
     for( i=0 ; i<c->no_detT;i++)
     {
       printf("\n A Matrix von Q_d mit d=%d\n", i);
       show_matrix( prep[i] -> A );
     }
#endif

   }

#ifdef KOSTEN
  log_kosten(6);
#endif

#ifdef DEBUG_M
  printf("************************************************************\n");
  printf("Q_EE Matrix\n");
  show_matrix(c->Q_EE);

  for( i=0 ; i<c->no_detT;i++)
  {
    printf("\n Q_Ed Matrizen d=%d\n", i);
    show_matrix(c->Q_Ed[i]);
  }

  for( i=0 ; i<c->no_detT;i++)
  {
      printf("\n Q_d  Matrizen d=%d\n", i);
      show_matrix(c->detMatrices[i].Q_d);

  }

  printf("\n Q_dE Matrizen muessen leer sein, sonst ware ich nicht hier!\n");
  for( i=0 ; i<c->no_detT;i++)
  {
      printf("\n Delta_dE Matrizen d=%d\n", i);
      show_matrix(c->detMatrices[i].Delta_dE);
  }

  for( i=0 ; i<c->no_detT;i++)
  {
    for( j=0 ; j<c->no_detT;j++)
    {
      printf("\n Q_dc Matrizen d=%d c=%d\n", i,j);
      show_matrix(c->detMatrices[i].Q_dc[j]);
    } 
  }

  for( i=0 ; i<c->no_detT;i++)
  {
    for( j=0 ; j<c->no_detT;j++)
    {
      printf("\n Delta_dc Matrizen d=%d c=%d\n", i,j);
      show_matrix(c->detMatrices[i].Delta_dc[j]);
    } 
  }
  printf("************************************************************\n");
#endif

   /* jetzt gehts los ***********************************************************************************/

   numDatas = ceil(transientOptions.transient_time/transientOptions.stepsize_result);
   maxX = numDatas * transientOptions.stepsize_result;
   numDatas ++;    /* t = 0.0 ist auch ein Messwert */

   numCompDatas = ceil(transientOptions.transient_time/transientOptions.stepsize_method);

   ftmp = open_solve(tnethome,modelpath,netname,numDatas,maxX);

   update_solve(tnethome,modelpath,netname,ftmp,c->no_markings,p_t,0.0,maxX);

   akt_output = 1; /* fuer output */
   for(time=1;time<=numCompDatas;time++)
   {

#ifdef KOSTEN
  log_kosten(1);
#endif

      /* (1.) compute p'd(t,j) mit h<=j<=tau_d **************************************************************/
      for(i=0;i<ndetT;i++)
      {
	for(j=0;j<steps_in_tau[i];j++)
	{
	  if(j != pdttau_element[i])
	  {
	    /*  p'd(t,j) *= e^(Q'd * h) */
	    if(transientOptions.old_case_iii)
	    {
	      mult_vector_matrix( (p_d_mexp[i])[j], m_exp_Q_d_h[i]);
	    }
	    else
	    {
	      del_vector( &temp_vec );
	      temp_vec = (p_d_mexp[i])[j];
	      (p_d_mexp[i])[j] = new_vector();

	      for(k=0;k <= prep[i]->R;k++)
	      {
		if(k >= prep[i]->L )
		  vector_add_mult( (p_d_mexp[i])[j], temp_vec, prep[i] -> Betas[ k - prep[i]->L ] );

		mult_vector_matrix( temp_vec,  prep[i] -> A); 
	      }
	    }
	  }
	}

	pdt0_element[i] = pdttau_element[i];
	pdttau_element[i]--;
	if(pdttau_element[i]<0)
	  pdttau_element[i] = steps_in_tau[i] - 1;

	/* setze p'd(t,0) auf 0 */
	clear_vector( (p_d_mexp[i])[pdt0_element[i]] );
      }

#ifdef KOSTEN
  log_kosten(2);
#endif

#ifdef DEBUG_V
  for( i=0 ; i<c->no_detT;i++)
  {
    printf(" p'd (%lf, tau ) d=%d  ", time*h, i);
    show_vector( (p_d_mexp[i])[pdttau_element[i]] );
  } 
#endif
      
      /* (2.) compute p'd(t) with trapezoid rule **********************************************************/
      for(i=0;i<ndetT;i++)
      {
	clear_vector(p_d_t[i]);
	for(j=0;j<steps_in_tau[i];j++)
	{
	  if(middle_part(j, pdt0_element[i], steps_in_tau[i]))
	    vector_add_mult(p_d_t[i], (p_d_mexp[i])[j], h);
	  else
	    vector_add_mult(p_d_t[i], (p_d_mexp[i])[j], h/2.0);

	  del_vector( &(tmp_p_d_t[i]) );
	  tmp_p_d_t[i] = copy_vector(p_d_t[i]);
	}
      }

      

#ifdef KOSTEN
  log_kosten(3);
#endif

#ifdef DEBUG_V
  for( i=0 ; i<c->no_detT;i++)
  {
    printf(" p'd (%lf) d=%d  ", time*h, i);
    show_vector(p_d_t[i]);
  } 
#endif
  
      maxdiff = 0.0;
      iter = 0;
      do
      {
	/* (3.) compute p'E(t) *****************************************************************************/
	
	del_vector( &p_E_t );
	p_E_t = runge_kutta_differenziation(old_pEt, old_pdttau, p_d_mexp, pdttau_element, old_pdt, p_d_t, c, h);


#ifdef KOSTEN
  log_kosten(4);
#endif

#ifdef DEBUG_V
  printf(" p'E (%lf) ",time*h);
  show_vector(p_E_t);
#endif

	/* (4.) compute p'd(t,0) mit p'E(t), p'd(t) und p'd(t,tau) *****************************************/
	for(i=0;i<ndetT;i++)
	{
	  del_vector( &(p_d_t_0[i]) );
	  p_d_t_0[i] = copy_vector(p_E_t);
	  mult_vector_matrix(p_d_t_0[i], c->Q_Ed[i]);
	  for(j=0;j<ndetT;j++)
	  {
	    del_vector(&temp_vec);
            temp_vec = copy_vector((p_d_mexp[j])[pdttau_element[j]]);
            mult_vector_matrix(temp_vec, c->detMatrices[j].Delta_dc[i]);
            vector_add_mult(p_d_t_0[i], temp_vec,1.0);

	    del_vector(&temp_vec);
            temp_vec = copy_vector(p_d_t[j]);
            mult_vector_matrix(temp_vec, c->detMatrices[j].Q_dc[i]);
            vector_add_mult(p_d_t_0[i], temp_vec,1.0);
	  }
	}

#ifdef KOSTEN
  log_kosten(5);
#endif

#ifdef DEBUG_V
  for( i=0 ; i<c->no_detT;i++)
  {
    printf(" p'd (%lf, 0 ) d=%d ", time*h, i);
    show_vector(p_d_t_0[i]);
  } 
#endif

	/* (5.) compute better p'd(t) Schritt 2. Wiederholen (letzte Trapez differenz aufaddieren) */
	for(i=0;i<ndetT;i++)
	{
	  del_vector( &(p_d_t[i]) );
	  p_d_t[i] = copy_vector(tmp_p_d_t[i]);
	  vector_add_mult(p_d_t[i], p_d_t_0[i], h/2.0);	    
	}

#ifdef KOSTEN
  log_kosten(3);
#endif

#ifdef DEBUG
  for( i=0 ; i<c->no_detT;i++)
  {
    printf(" p'd (%lf) d=%d  ", time*h, i);
    show_vector(p_d_t[i]);
  } 
#endif
	
	/* (6.) compare p_d_t_0[i] und (p_d_mexp[i])[pdt0_element] until < error < epsilon **************/
	if(preempted_trans)
	{
	  if( compare_pdt0(p_d_mexp, pdt0_element, p_d_t_0,ndetT, transientOptions.precision) )  /* beide gleich? */
	  {
	    /* copy verbessertes p'd(t,0) aus Schritt 5. */
	    for(i=0;i<ndetT;i++)
	    {
	      del_vector( &( (p_d_mexp[i])[pdt0_element[i]] ) );
	      (p_d_mexp[i])[pdt0_element[i]] = copy_vector( p_d_t_0[i] );
	    }
	    break;
	  }
	  else
	  {
	    if(old_maxdiff == maxdiff || iter>100)
	    {
	      printf("warning: can't reach recommended precision at time:%g after %d repeatings with precision: %g\n",
		     time*h,
		     iter,
		     maxdiff);
	      break;
	    }

#ifdef DEBUG_V
	    printf("time:%g repeatings:%d maxdiff:%g\n",time*h,iter,maxdiff);
#endif

	  }
	}
	
	/* copy verbessertes p'd(t,0) aus Schritt 5. wenn noch nicht schluss ist */
	for(i=0;i<ndetT;i++)
	{
	  del_vector( &( (p_d_mexp[i])[pdt0_element[i]] ) );
	  (p_d_mexp[i])[pdt0_element[i]] = copy_vector( p_d_t_0[i] );
	}

	iter++;
	old_maxdiff = maxdiff;
	maxdiff=0.0;
      } while(preempted_trans); /* falls Modell preemted trans. hat: Endlosschleife (nur durch break zu verlassen) */
      

      /* (7.) output required ? ***********************************************************************/
      if( (time * h) == (akt_output * transientOptions.stepsize_result) )
      {
	 akt_output++;

         /* compute p(t) */
	 del_vector(&p_t);
         p_t = copy_vector(p_E_t);
         for(i=0;i<ndetT;i++)
         {
            vector_add_mult(p_t, p_d_t[i], 1.0);
         }

#ifdef DEBUG
	 printf(" p(%lf) ", time*h);
	 show_vector(p_t);
/*	 printf("------------------------------\n");  */
#endif

	 update_solve(tnethome,modelpath,netname,ftmp,c->no_markings,p_t,time*h,maxX);
      }

      /* (8.) copy akt. p'd(t,tau), p'(t) und p'E(t) nach old_* fuer naechste Runge Kutta Differenziation */
      del_vector( &(old_pEt));
      old_pEt = copy_vector(p_E_t);
      for(i=0;i<ndetT;i++)
      {
	del_vector( &(old_pdttau[i]) );
	old_pdttau[i] = copy_vector( (p_d_mexp[i])[pdttau_element[i]] );

	del_vector( &(old_pdt[i]) );
	old_pdt[i] = copy_vector(p_d_t[i]);
      }
   }
   close_solve(modelpath,netname,ftmp);

   /* free allocated memory */
}

/******************************************************************************/
static int compare_pdt0(p_d_mexp,pdt0_element,p_d_t_0,ndetT,schranke)
/******************************************************************************/
sp_vectors **p_d_mexp;
int *pdt0_element;
sp_vectors *p_d_t_0;
int ndetT;
double schranke;
{
  int i;

  for(i=0;i<ndetT;i++)
    if( !comp_vector( (p_d_mexp[i])[pdt0_element[i]], p_d_t_0[i], schranke))
      return 0;
  return 1;
}


/******************************************************************************/
/*  middle_part()  Hilfsfkt. schnelleren Berechnung der Trapez-Regel          */
/******************************************************************************/
static int middle_part(j,pdt0_element,steps_in_tau)
int j;                 /* akt. step in tau */
int pdt0_element;
int steps_in_tau;
{
  if(pdt0_element > 0)
  {
    if( j != (pdt0_element - 1) )
      return(True);
    else
      return(False);
  }
  else
  {
    if( j !=  (steps_in_tau - 1) )
      return(True);
    else
      return(False);
  }
}

/**********************************************************************************/
static sp_vector *runge_kutta_differenziation(p_E_t, old_pdttau, new_pdtx, pdttau_element, old_pdt, new_pdt, c, h)
/**********************************************************************************/
sp_vector *p_E_t;
sp_vectors *old_pdttau;
sp_vectors **new_pdtx;
int *pdttau_element;
sp_vectors *old_pdt;
sp_vectors *new_pdt;
solve_basis *c;
double h;
{
   int i;
   sp_vector *result_vec;
   sp_vector *temp_vec;
   sp_vector *k1;
   sp_vector *k2;
   sp_vector *k3;
   sp_vector *k4;
   sp_vector *k1_summand;
   sp_vector *k23_summand;
   sp_vector *k4_summand;


   result_vec = copy_vector(p_E_t);
   k1 = copy_vector(p_E_t);
   k2 = copy_vector(p_E_t);
   k3 = copy_vector(p_E_t);
   k4 = copy_vector(p_E_t);

   temp_vec = new_vector();

   k1_summand = new_vector();
   for(i=0;i<c->no_detT;i++)
   {
      del_vector(&temp_vec);
      temp_vec = copy_vector(old_pdttau[i]);
      mult_vector_matrix(temp_vec, c->detMatrices[i].Delta_dE);
      vector_add_mult(k1_summand,temp_vec,1.0);    

      del_vector(&temp_vec);
      temp_vec = copy_vector(old_pdt[i]);
      mult_vector_matrix(temp_vec, c->detMatrices[i].Q_dE);
      vector_add_mult(k1_summand,temp_vec,1.0);    
   }

   k4_summand = new_vector();
   for(i=0;i<c->no_detT;i++)
   {
      del_vector(&temp_vec);
      temp_vec = copy_vector( (new_pdtx[i])[pdttau_element[i]] );
      mult_vector_matrix(temp_vec, c->detMatrices[i].Delta_dE);
      vector_add_mult(k4_summand,temp_vec,1.0);    

      del_vector(&temp_vec);
      temp_vec = copy_vector(new_pdt[i]);
      mult_vector_matrix(temp_vec, c->detMatrices[i].Q_dE);
      vector_add_mult(k4_summand,temp_vec,1.0);    
   }

   k23_summand = new_vector();
   for(i=0;i<c->no_detT;i++)
   {
      del_vector(&temp_vec);
      temp_vec = copy_vector( old_pdttau[i] );
      vector_add_mult(temp_vec, (new_pdtx[i])[pdttau_element[i]], 1.0);
      mult_vector_skalar(temp_vec,0.5);
      mult_vector_matrix(temp_vec, c->detMatrices[i].Delta_dE);
      vector_add_mult(k23_summand,temp_vec,1.0);    

      del_vector(&temp_vec);
      temp_vec = copy_vector( old_pdt[i] );
      vector_add_mult(temp_vec,new_pdt[i],1.0);
      mult_vector_skalar(temp_vec,0.5);
      mult_vector_matrix(temp_vec, c->detMatrices[i].Q_dE);
      vector_add_mult(k23_summand,temp_vec,1.0);    
   }

   mult_vector_matrix(k1,c->Q_EE);
   vector_add_mult(k1,k1_summand,1.0);
   mult_vector_skalar(k1,h);

   vector_add_mult(k2,k1,1.0/2.0);
   mult_vector_matrix(k2,c->Q_EE);
   vector_add_mult(k2,k23_summand,1.0);
   mult_vector_skalar(k2,h);

   vector_add_mult(k3,k2,1.0/2.0);
   mult_vector_matrix(k3,c->Q_EE);
   vector_add_mult(k3,k23_summand,1.0);
   mult_vector_skalar(k3,h);

   vector_add_mult(k4,k3,1.0);
   mult_vector_matrix(k4,c->Q_EE);
   vector_add_mult(k4,k4_summand,1.0);
   mult_vector_skalar(k4,h);

   vector_add_mult(result_vec,k1,1.0/6.0);
   vector_add_mult(result_vec,k2,1.0/3.0);
   vector_add_mult(result_vec,k3,1.0/3.0);
   vector_add_mult(result_vec,k4,1.0/6.0);

   del_vector( &temp_vec );
   del_vector( &k1 );
   del_vector( &k2 );
   del_vector( &k3 );
   del_vector( &k4 );
   del_vector( &k1_summand );
   del_vector( &k23_summand );
   del_vector( &k4_summand );

   return(result_vec);
}

 
