/********************************************************************
Modul :             on_tx_plain.c

Purpose :           stellt im wesentlichen die Funktionen disk_uebertragung()
                    und update_int_structures() zur Verfuegung. 
		    update_int_structures() leistet folgendes :
                    - Einarbeitung neuer Punkte in low_box
                    - Updaten des Interpolationsfeldes
		    - Nachziehen der up_boxes[]-Zeiger
		        mit Veraenderung der Integrationsgewichte
		    - Randomisierung der Werte entlang der x-Achse zum
		        neuen Zeitpunkt und gleichzeitige, vorlaeufige
			Integration ( Punkte aus boundary_conditions() 
			fehlen ja noch);

Project Director :  Reinhard German

Programmer :        Armin Heindl

History :           18.5.95 ( Diplomarbeit )
************************************************************************/
/* #define DEBUG_AC */
/* #define KOSTEN   */   

#include <stdio.h>                /* fuer NULL */
#include <math.h>
#include "frontend.h"
#include "shared.h"


static int add_here = 1;           /* gibt Komponente an, wo in low_box die Randbedingung angehaengt werden soll */
static int conf_add_here;          /* Sicherheitskopie fuer den Fall der Schrittablehnung */

static double merke = DISC_ANZEIGE;/* merkt sich, ob oberste Komponente in low_box ein Integrationsstuetzpunkt
				      direkt bei Diskontinuitaet ist oder ein Randpunkt, der zwei Integrations-
				      intervallen angehoert. In letztem Fall wird in `merke' das Integrationsgewicht 
				      gespeichert, das aus Zugehoerigkeit zo oberem Intervall resultiert */
static double conf_merke;          /* Sicherheitskopie */

static double old_stepsize = 0.0;  /* alte Schrittweite muss hier gespeichert werden, um die relative Position
				      des aktuellen Zeitpunkts in den Interpolationsfeldern nachzustellen */
				      

extern int ac_no_detT;

extern double EXT_PRECISION;       /* fuer erhoehte Genauigkeitsanforderungen */

/* aus shared.c : */
extern void randomize_quickly();   /* schreibt das Ergebnis einer Randomisierung in den Vektor selbst ->
				      guenstiger fuer Randomisierung der Vektoren in step_inbetween-Elementen */
extern sp_vector *ranforfield();   /* liefert Ergebnis der Randomisierung eines Vektors als neuen Vektor ->
				      guenstiger fuer Randomisierung im Zshg mit Interpolationsfeldern */
extern double adapt_precision();
extern int ad_prec_ifnec();
/* aus weighter.c : */
extern void call_new_weighter();

/* aus integrate_interpolate.c : */
extern void l3_integrale();
extern void l4_integrale();
extern void l3_randintegrale();
extern void newton_per_4();
extern void newton_replace_1();
extern void newton_replace_2();
extern sp_vector *polynom_auswertung();

/* aus interpolation_fields.c */
extern void D_case();
extern void ZD_case();
extern void NZD_case();
extern void NNZD_case();
extern void sonderbehandlung();
extern void OX_case();
extern void OOX_case();
extern void OXOX_case();

/***********************************************************************/
void set_old_stepsize( val)

double val;
{
  old_stepsize = val;
}

/**********************************************************************/
void save_static_values()
/* legt Sicherheitskopien von add_here und merke an; wird immer nach angenommenen
   Schritt aufgerufen */
{
  conf_add_here = add_here;
  conf_merke = merke;
}
/**********************************************************************/
void reaffirm_static_values()
/* setzt die Sicherheitskopien wieder in die benutzten Variablen; wird 
   immer nach Schrittablehnung aufgerufen */
{

  add_here = conf_add_here;
  merke = conf_merke;
}
/**********************************************************************/
void disk_uebertragung( daten_basis, init, no_upper_steps, up_boxes, old_no_upper_steps, old_up_boxes, interpol_update, int_fields, randdata, areas_ptr) 
/* trifft alle Vorbereitungen nach dem Erreichen einer Diskontinuitaet, um erneut mit der Anlaufrechnung
   beginnen zu koennen : Neuaufbau der p_d(t,x)-Interpolationsfelder, Uebertragen der upper_steps-Box fuer 
   ausloesende Gruppen und Reflexion der Impulse */ 
/* Voraussetzung : mind. vier Stuetzpunkte (incl. Randpunkte) zwischen zwei Diskontinuitaeten 
                   -> fuer int_fields-Initialisierung */

solve_basis *daten_basis;
int init;
int *no_upper_steps;
step_inbetween **up_boxes;
int *old_no_upper_steps;
step_inbetween **old_up_boxes;
int *interpol_update;
interpolation_field *int_fields;
ConcernsRandomisation **randdata;
areas *areas_ptr;
{
  int i,k;
  step_inbetween *ptr;
  sp_vector *vnap;
  double h_sum;
  double precision;

  if (init) {
        for( i=0; i<ac_no_detT; i++) {
  
        /* fuer alle Gruppen muss bei Erreichen einer Diskontinuitaet ein neuer Vektor an areas_ptr[i].last
	   angehaengt werden -> Ist eine Gruppe eine ausloesende, so gehoert automatisch der Vektor
	   areas_ptr[i]->first zum mit der erreichten Diskontinuitaet verbundenen Impuls     */
    	areas_ptr[i].last->next = new_vector();
    	areas_ptr[i].last = areas_ptr[i].last->next;
         };
#ifdef KOSTEN
	log_kosten(1);
#endif
  };

  for( i=0; i<ac_no_detT; i++) {
#ifdef DEBUG_AC
    printf(" Gruppe %d : #upper_steps = %d, up_boxes[]->step = %f \n", i, no_upper_steps[i], up_boxes[i]->step);
#endif
    if ((no_upper_steps[i] == 1) && (up_boxes[i]->step == DISC_ANZEIGE)) {
      /* Gruppe i ist eine ausloesende Gruppe, es kann mehrere geben  */
#ifdef DEBUG_AC
      printf(" Gruppe %d loest Diskontinuitaet aus! \n", i);
#endif
      /* Eine ausloesende Gruppe kann auch Spruenge in den Wahrscheinlichkeiten verursachen ! */
      if (init) {
	/* Sprung in p_E's : immer >= 0, d.h. p_E kann sich nur vergroessern */
   	vnap = copy_vector( areas_ptr[i].first);
	mult_vector_matrix( vnap, daten_basis->detMatrices[i].Delta_dE);
	vector_add_mult( daten_basis->p_E, vnap, 1.0);
	del_vector(&vnap);

	/* Reflexion an tau-Linie(n), die auch Spruenge in anderen p_d(t)'s verursacht */
	for( k=0; k<ac_no_detT; k++) {
		vnap = copy_vector( areas_ptr[i].first);
		mult_vector_matrix( vnap, daten_basis->detMatrices[i].Delta_dc[k]);
		vector_add_mult( areas_ptr[k].last, vnap, 1.0);
		del_vector(&vnap);
	};
	
	/* Sprung in p_d(t) selbst, Teil I. nur bei ausloesender Gruppe : Abziehen der oberen area */
	vector_add_mult( daten_basis->p_d[i], areas_ptr[i].first, -1.0);
	vnap = areas_ptr[i].first;
	areas_ptr[i].first = areas_ptr[i].first->next;
	del_vector( &vnap);
#ifdef KOSTEN
	log_kosten(3);
#endif
      };     

      /* trage die einzige upper_steps-Box einer ausloesenden Gruppe ueber die Diskontinuitaet */
      ptr = up_boxes[i]->next_down;
      up_boxes[i]->step = ptr->step;
      up_boxes[i]->integr_weight = ptr->integr_weight;
      up_boxes[i]->indicator = ZWISCHEN_ANZEIGE;
      /* indicator muss nicht gesetzt werden */
      del_vector(&(up_boxes[i]->values[0][i]));
      up_boxes[i]->values[0][i] = copy_vector( ptr->values[0][i]);

      up_boxes[i]->next_down = ptr->next_down;

      /* ptr zeigt bei obersten Gruppe auf zu loeschendes step_inbetween-Element */
      if (i == (ac_no_detT-1)) {
	for( k=0; k<ac_no_detT; k++) {
	  del_vector(&(ptr->values[0][k]));
	  del_vector(&(ptr->values[1][k]));
	};
	efree( ptr->values[0]);
	efree( ptr->values[1]);
	efree( ptr);
	ptr = up_boxes[i]->next_down;
      }
      else 
	ptr = ptr->next_down;
#ifdef KOSTEN
      log_kosten(1);
#endif

      /* Reinitialisierung des Interpolationsfeldes mit den ersten vier step_inbetween-Elementen 
         (inklusive des upper_steps-Elements) */
      interpol_update[i] = 1;

      int_fields[i][1].int_point = (-1.0)*(ptr->next_down->step);
      int_fields[i][2].int_point = int_fields[i][1].int_point - (ptr->step);
      int_fields[i][3].int_point = int_fields[i][2].int_point - (up_boxes[i]->step);
      int_fields[i][0].int_point = int_fields[i][3].int_point;

      del_vector(&(int_fields[i][3].y_values));
      int_fields[i][3].y_values = copy_vector( up_boxes[i]->values[0][i]);

      h_sum = up_boxes[i]->step;
      del_vector(&(int_fields[i][2].y_values));  
      precision = adapt_precision( ptr->values[0][i]);
      call_new_weighter( h_sum*(randdata[i]->qmax_ii), &(randdata[i]->L), &(randdata[i]->R), &(randdata[i]->Betas), precision);
      int_fields[i][2].y_values = ranforfield( ptr->values[0][i], randdata[i]); 

      h_sum += ptr->step;
      ptr = ptr->next_down;
      del_vector(&(int_fields[i][1].y_values));  
      precision = adapt_precision( ptr->values[0][i]);
      call_new_weighter( h_sum*(randdata[i]->qmax_ii), &(randdata[i]->L), &(randdata[i]->R), &(randdata[i]->Betas), precision);
      int_fields[i][1].y_values = ranforfield(ptr->values[0][i], randdata[i]); 
      
      h_sum += ptr->step;
      del_vector(&(int_fields[i][0].y_values));  
      precision = adapt_precision( ptr->next_down->values[0][i]);
      call_new_weighter( h_sum*(randdata[i]->qmax_ii), &(randdata[i]->L), &(randdata[i]->R), &(randdata[i]->Betas), precision);
      int_fields[i][0].y_values = ranforfield(ptr->next_down->values[0][i], randdata[i]); 

      /*ptr->values[0][i]->num = 1; da dieser Wert bei Stuetzpunktersetzung */
      /*ptr->values[1][i]->num = 1; eh nicht geprueft wird                  */
      ptr->next_down->values[0][i]->num = 1;
      ptr->next_down->values[1][i]->num = 1;

      newton_per_4( int_fields[i]);

      del_vector(&(int_fields[i][0].aux_coefficients));
      int_fields[i][0].aux_coefficients = copy_vector( int_fields[i][3].y_values);
#ifdef KOSTEN
      log_kosten(6);
#endif
    }
    else {
      /* Gruppe i hat die Diskontinuitaet nicht ausgeloest */
      /* alles kann bleiben, wie's ist; nur Bezugspunkt des alten Interpolationsfelds
	 muss um old_stepsize weitergeschoben werden */
      int_fields[i][0].int_point += old_stepsize;
    };

    /* immer zu machen */
    old_no_upper_steps[i] = no_upper_steps[i];
    old_up_boxes[i] = up_boxes[i];

  }; /* for-Schleife */

  if (init) {
       /* Sprung in p_d, Teil II. fuer alle Gruppen : Hinzuaddieren der neu hinzugekommenen area */
       for( i=0; i<ac_no_detT; i++) 
       	vector_add_mult( daten_basis->p_d[i], areas_ptr[i].last, 1.0);
#ifdef KOSTEN
       log_kosten(3);
#endif
  };

}

/**********************************************************************/

int update_int_structures( discontinuity, stepsize, preemption, init, no_upper_steps, old_no_upper_steps, up_boxes, old_up_boxes, plow_box, interpol_update, int_fields, pdts, randdata, areas_ptr)
/* liefert Index der Komponente von low_box zurueck, an der spaeter das Ergebnis der Randbedingungsgleichung
   angehaengt werden muss ( nur falls keine Diskontinuitaet erreicht wird */

/* update_int_structures() besteht aus fuenf wesentlichen Teilen :
   1. Der spaeter in boundary_conditions() neu hinzukommende Integrationsstuetzpunkt wird formal in die
      Integrationskette einbezogen, indem das unterste Integrationsintervall (in low_box) entsprechend erweitert
      bzw. veraendert wird. Zunaechst werden nur die neuen Integrationsgewichte der unteren Stuetzpunkte
      berechnet, erst im vierten Teil (Randomisierung) wird die numerische Integration ausgefuehrt.
      Folgende Strategie wird gefahren, wobei die Zahlen fuer die Anzahle der Stuetzpunkte des untersten 
      Integrationsintervalls stehen 
       a) Bei erstem Aufruf nach Diskontinuitaet wird auf den Anfangs- und Folgestuetzpunkt die Trapezregel
          angewandt.
       b) Mit dem dritten Stuetzpunkt wird dann ueber ein quadratisches Interpolationspolynom integriert.
       c) neuer Punkt ist vierter Punkt in low_box -> Integration ueber ein kubisches Interpolationspolynom
       d) neuer Punkt ist fuenfter Punkt in low_box -> Integration ueber zwei quadratische Interpolationspolynome
       e) neuer Punkt waere sechster Punkt in low_box -> das obere Integrationsintervall aus vier Stuetzpunkten
            bleibt ab jetzt unveraendert, die ersten drei Stuetzpunkte koennen an die Integrationskette der
	    step_inbetween-Elemente angehaengt werden. Mit den drei unteren Stuetzpunkten wird ein 
	    quadratisches Interpolationspolynom fuer die num. Integration gebildet.
       beachte : - c),d) und e) wiederholen sich zyklisch, bis die naechste Diskontinuitaet erreicht wird
                   und alle low_box-Stuetzpunkte an die step_inbetween-Liste angehaengt werden.
		 - Dieselben Integrationsgewichte aus e) ( bzw. b) ) koennen in d) wiederverwendet werden, 
		   ebenso die aus c) fuer die in die Integrationskette in e) integrierten Elemente.
		 - Eine aehnliche Strategie wird fuer die anderen Enden der Integrationskette an den 
		   tau-Linien in der upper_steps-Behandlung gefahren.
   2. Anpassung der Interpolationsfelder fuer die p_d(t,tau_d)'s an den neu zu interpolierenden Bereich
      zwischen transient_time + 0.2*stepsize bis transient_time + stepsize ( Naeheres siehe weiter unten
      und in interpolation_fields.c )
   3. Behandlung der upper_steps, so dass auch fuer die einzelnen Gruppen ab ihrer tau-Linie eine 
      numerische Integration genuegender Genauigkeit gegeben ist ( Unterschied zu Strategie in Teil 1 :
      Ein DISC_ANZEIGE-Element, das eine Diskontinuitaet in der Integrationskette anzeigt, macht u.U.
      die Bildung eines Randintegrals notwendig, dessen aeusserster Punkt oberhalb der tau-Linie liegt,
      um Genauigkeitsanforderungen zu genuegen )
   4. Die Vektoren der Funktionswerte in den Stuetzpunkten der Integrationskette werden randomisiert
      und - multipliziert mit ihren Integrationsgewichten - zu einem vorlaeufigen Ergebnis fuer p_d(t)'s
      aufsummiert ( Werte aus boundary_conditions() fehlen ja noch ).
      zugrundeliegende Formeln : Randomisierung -> p_d(t+h,x+h) = p_d(t,x)*(e^(Q_d*h))
                                 Integration    -> p_d(t) = x_integral_von_0.0_bis_tau_d{ p_d(t,x) }
      Beachte, dass neben Teil 1 und 3 auch der Teil 2 eine wichtige Voraussetzung fuer Teil 4 ist,
      da der Funktionswertevektor des obersten upper_steps-Elements ( also das Elements auf der tau-Linie )
      aus dem entsprechenden Interpolationsfeld stammt ( meistens ein interpolierter Vektor )          
   5. Extrapolation der p_d(t,0.0)-Vektoren, von denen spaeter mit boundary_conditions() iterativ 
      bessere Loesungen berechnet werden ( nur falls preemption = 1 )                                 */

int discontinuity;         /* 1 zeigt das Erreichen einer Diskontinuitaet an */
double stepsize; 
int preemption;
int init;
int *no_upper_steps;       /* die alten Anzahlen der upper boxes sowie die up_boxes-Zeiger werden in main */
int *old_no_upper_steps;
step_inbetween **up_boxes; /* aufbewahrt und bei Schrittablehnung wieder initialisiert */
step_inbetween **old_up_boxes;
lower_stepbox *plow_box;
int *interpol_update;
interpolation_field *int_fields;
sp_vector **pdts;
ConcernsRandomisation **randdata;
areas *areas_ptr;
{
  int i,j,k;
  int *hits_exactly;            /* zeigt fuer jede Gruppe an, ob tau-Linie genau auf ein step_inbetween-
			           Element faellt, erspart eine Interpolationsauswertung */
  step_inbetween *first_step_added;/* merkt sich bei Uebertragung von Komponenten aus low_box in die 
			           step_inbetween-Liste das erste uebertragene Element, um spaeter
			           dort die Integrationswerte einsetzen zu koennen */
  step_inbetween *ptr, *following;
  step_inbetween *el1, *el3;    /* zeigen auf die step_inbetween-Elemente direkt vor dem ersten bzw. zweiten
			           Schnitt. el1 und el3 soll andeuten die jeweiligen step_inbetween-Elemente
			           im Normalfall beim Neuaufbau des Interpolationsfeldes den ersten und
				   dritten Stuetzpunkt (zeitliche Reihenfolge), also t0 und t2, liefern */
  step_inbetween *extrapol_start;/* merkt sich im Fall discontinuity = 1 das erste Element in der 
			           step_inbetween-Kette, ab dem die Stuetzpunkte fuer eine Extrapolation
			           des Randpunktes herangezogen werden sollen */
  step_inbetween **parser;      /* um im Teil 4 keine Vektoren unnoetig zu randomisieren  */
  double h_sum = 0.0;           /* Abstand von tau-Linie bis zum ersten step_inbetween-Element nach zweitem Cut */
  double h_sum1;                /* Abstand von tau-Linie bis zum ersten step_inbetween-Element nach erstem Cut */
  double hdurch5 = stepsize*0.2;/* Abstand von tau-Linie zu erstem Schnitt */
  double first_cut;             /* gibt Lage des ersten Schnitts relativ zum Bezugspunkt eines Interpolationsfeldes an*/
  sp_vector *w_vec;
  double *precision;
  int *no_change;
  
  hits_exactly = (int *)emalloc(ac_no_detT*sizeof(int));
  precision = (double *)emalloc(ac_no_detT*sizeof(double));
  parser = (step_inbetween **)emalloc(ac_no_detT*sizeof(step_inbetween *));
  
  /* TEIL 1 : LOW_BOX-BEHANDLUNG */

  if (discontinuity) {
    /* Mit kommenden Schritt wird eine Diskontinuitaet erreicht */
    /* Feststellen, ob Dummy-Element existiert. Kennzeichen : step == 0.0 und indicator == ZWISCHEN_ANZEIGE */
    if ((plow_box->next_up->step == 0.0) && (plow_box->next_up->indicator == ZWISCHEN_ANZEIGE)) {
      /* Dummy existiert : Struktur kann fuer erstes zu uebertragendes Element benutzt werden */
      first_step_added = plow_box->next_up;
      for( i=0; i<ac_no_detT; i++) { 
	del_vector(&(first_step_added->values[0][i]));
	del_vector(&(first_step_added->values[1][i]));
      };
      efree(first_step_added->values[0]);  
      efree(first_step_added->values[1]);  
    }
    else {
      first_step_added = (step_inbetween *)emalloc(sizeof(step_inbetween));
      plow_box->next_up->next_down = first_step_added;
    };
    ptr = first_step_added;
    /* Uebertragungen (ausser Integrationsgewichten) von der ersten bis vor die letzte belegte Komponente */
    for(i=0;i<(add_here-1);i++) {
      ptr->step = plow_box->lower_steps[i].step;
      ptr->values[0] = plow_box->lower_steps[i].values[0];  /* Uebertragung aller Funktionswertevektoren */
      ptr->values[1] = plow_box->lower_steps[i].values[1];  
      ptr->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
      ptr = ptr->next_down;
    };
    /* die letzte Zwischenuebertragung */
    ptr->step = stepsize;
    ptr->values[0] = plow_box->lower_steps[add_here-1].values[0];    						 
    ptr->values[1] = plow_box->lower_steps[add_here-1].values[1];

    /* Erzeugung des step_inbetween-Elements fuer die Randbedingungen */
    ptr->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));

    plow_box->next_up = ptr->next_down;
    plow_box->next_up->next_down = NULL;
    plow_box->next_up->step = 0.0; /* Diskontinuitaet */
    /* um bei Randomisierung und Integration nicht staendig abfragen zu muessen, ob NULL oder nicht : */
    plow_box->next_up->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
    plow_box->next_up->values[1] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
    for( i=0; i<ac_no_detT; i++) {
      plow_box->next_up->values[0][i] = new_vector();
      plow_box->next_up->values[1][i] = new_vector();
    };
#ifdef KOSTEN
    log_kosten(1);
#endif

    /* Integrationsgewichte und Indikatoren bestimmen */
    ptr = first_step_added;
    switch (add_here) {
    /* ACHTUNG : `case 2` kann nicht sein, da wir immer vier Stuetzpunkte zwischen zwei Diskontinuitaeten 
                 fordern <- wegen leichterer Handhabung der Interpolationsfelder */
    case 3 : { /* Fall c) mit Diskontinuitaet */
               double integr_gewichte[4];

	       extrapol_start = first_step_added; /* spaeter : Extrapolation mit drei Stuetzpunkten */
	       /* Bestimmung der Integrationsgewichte */
	       l4_integrale(integr_gewichte, plow_box->lower_steps[0].step, plow_box->lower_steps[1].step, stepsize);
	       if (merke == DISC_ANZEIGE) {
		 /* oberstes Element folgte direkt auf Diskontinuitaet */
		 ptr->integr_weight = integr_gewichte[0];
		 ptr->indicator = merke;
	       }
	       else {
		 /* oberstes Element ist Randpunkt zweier Integrationsintervalle */
		 ptr->integr_weight = integr_gewichte[0] + merke;
		 ptr->indicator = integr_gewichte[0];  /* Integrationsgweicht fuer unteres Integrationsintervall 
							  wird in indicator gespeichert */
	       };
	       for(j=1;j<add_here;j++) {
		 ptr = ptr->next_down;
		 ptr->integr_weight = integr_gewichte[j];
		 ptr->indicator = ZWISCHEN_ANZEIGE;
	       };
	       ptr = ptr->next_down;
	       ptr->integr_weight = integr_gewichte[add_here];
	       ptr->indicator = DISC_ANZEIGE;
	     };
	     break;
    case 4 : { /* Fall d) mit Diskontinuitaet */
               double integr_gewichte[3];
	      
	       /* Bestimmung der Integrationsgewichte fuer erstes Integrationsintervall */
	       l3_integrale(integr_gewichte, plow_box->lower_steps[2].step, stepsize);
	       if (merke == DISC_ANZEIGE) {
		 /* oberstes Element folgte direkt auf Diskontinuitaet */
		 ptr->integr_weight = plow_box->lower_steps[0].lagrange3; /* stammt aus Fall e) oder b) */
		 ptr->indicator = merke;
	       }
	       else {
		 /* oberstes Element ist Randpunkt zweier Integrationsintervalle */
		 ptr->integr_weight = plow_box->lower_steps[0].lagrange3 + merke;
		 ptr->indicator = plow_box->lower_steps[0].lagrange3;/* Integrationsgweicht fuer unteres Integrations-
									intervall wird in indicator gespeichert */
	       };
	       ptr = ptr->next_down;

	       extrapol_start = ptr; /* spaeter : Extrapolation mit drei Stuetzpunkten */

	       ptr->integr_weight = plow_box->lower_steps[1].lagrange3;/* stammt aus Fall e) oder b) */
	       ptr->indicator = ZWISCHEN_ANZEIGE;
	      
	       ptr = ptr->next_down;
	       ptr->integr_weight = integr_gewichte[0] + plow_box->lower_steps[2].lagrange3;
	       ptr->indicator = integr_gewichte[0];  /* ist doppelter Randpunkt */

	       ptr = ptr->next_down;
	       ptr->integr_weight = integr_gewichte[1];
	       ptr->indicator = ZWISCHEN_ANZEIGE;
	      
	       ptr = ptr->next_down;
	       ptr->integr_weight = integr_gewichte[2];
	       ptr->indicator = DISC_ANZEIGE;
	     };
	     break;
    case 5 : { /* Fall e) mit Diskontinuitaet */
               double integr_gewichte[3];
	      
	       /* Bestimmung der Integrationsgewichte fuer zweites Integrationsintervall */
	       l3_integrale(integr_gewichte, plow_box->lower_steps[3].step, stepsize);
	       if (merke == DISC_ANZEIGE) {
		 /* oberstes Element folgte direkt auf Diskontinuitaet */
		 ptr->integr_weight = plow_box->lower_steps[0].lagrange4; /* stammt aus Fall c) */
		 ptr->indicator = merke;
	       }
	       else {
		 /* oberstes Element ist Randpunkt zweier Integrationsintervalle */
		 ptr->integr_weight = plow_box->lower_steps[0].lagrange4 + merke;
		 ptr->indicator = plow_box->lower_steps[0].lagrange4;
	       };
	       ptr = ptr->next_down;
	       ptr->integr_weight = plow_box->lower_steps[1].lagrange4;
	       ptr->indicator = ZWISCHEN_ANZEIGE;
	      
	       ptr = ptr->next_down;
	       extrapol_start = ptr;
	       ptr->integr_weight = plow_box->lower_steps[2].lagrange4;
	       ptr->indicator = ZWISCHEN_ANZEIGE; 

	       ptr = ptr->next_down;
	       ptr->integr_weight = integr_gewichte[0] + plow_box->lower_steps[3].lagrange4;
	       ptr->indicator = integr_gewichte[0]; /* zweifacher Randpunkt */
	      
	       ptr = ptr->next_down;
	       ptr->integr_weight = integr_gewichte[1];
	       ptr->indicator = ZWISCHEN_ANZEIGE;

	       ptr = ptr->next_down;
	       ptr->integr_weight = integr_gewichte[2];
	       ptr->indicator = DISC_ANZEIGE;

	     };
	     break;
    default :
#ifdef DEBUG_AC
             printf(" Default-Fehler in update_int_structures : add_here = %d ",add_here);
#endif            
             break;
    };
#ifdef KOSTEN
    log_kosten(3);
#endif
    
    /* low_box nullen.. */
    for(j=0; j<add_here; j++) {
      /* values[] auf keinen Fall loeschen, da Elemente an verkettete Liste gehaengt wurden ! 
	 -> statt dessen : */
      for(i=0; i<2; i++) {
	plow_box->lower_steps[j].values[i] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
	for( k=0; k<ac_no_detT; k++) {
          plow_box->lower_steps[j].values[i][k] = new_vector(); 
	  /* einfach NULL setzen, waere effektiver, aber unuebersichtlicher, da dann immer beachtet
	     werden muesste, dass low_box nur teilweise initialisiert ist */
	};
      };
    };
    /* vom Sinn her haetten diese beiden Zuweisungen auch in disk_uebertragung() stehen koennen */
    merke = DISC_ANZEIGE;
    add_here = 1;  

#ifdef KOSTEN
    log_kosten(1);
#endif
  
  }
  else {  /* discontinuity == 0 : Eine Diskontinuitaet wird nicht erreicht */
    switch (add_here) {
    case 1 : /* Fall a) : Aufruf nur direkt nach Diskontinuitaet */
             plow_box->lower_steps[0].step = stepsize;
             plow_box->lower_steps[0].integr_weight = 0.5*stepsize; /* Trapezregel ! */
             plow_box->lower_steps[1].integr_weight = 0.5*stepsize;
#ifdef KOSTEN
             log_kosten(3);
#endif
	     /* merke ist auf jeden Fall DISC_ANZEIGE */
             add_here += 1;
	     /* n"achste beiden Zeilen nur wegen case 2 kurz vor Extrapolation, denn hier existieren noch
		keine Sicherheitskopien */
	     for(j=0; j<ac_no_detT; j++) {
	       del_vector(&(plow_box->next_up->values[1][j]));
	       plow_box->next_up->values[1][j] = copy_vector(plow_box->next_up->values[0][j]);
	     };
             break;
    case 2 : /* Fall b) : Aufruf nur nach vollendetem rk2_step-Schritt*/
             plow_box->lower_steps[1].step = stepsize;
             { double integr_gewichte[3];

	       /* Bestimmung der Integrationsgewichte fuer Integrationsintervall */
	       l3_integrale( integr_gewichte, plow_box->lower_steps[0].step, stepsize);
	       for(j=0; j<= add_here; j++) {
		 plow_box->lower_steps[j].integr_weight = integr_gewichte[j];
		 plow_box->lower_steps[j].lagrange3 = integr_gewichte[j]; /* wird spaeter in Fall d) wieder benutzt */
	       };
	     };
#ifdef KOSTEN
             log_kosten(3);
#endif
	     /* merke ist auf jeden Fall DISC_ANZEIGE */
             add_here += 1;
             break;
    case 3 : /* Fall c) */
             plow_box->lower_steps[2].step = stepsize;
             { double integr_gewichte[4];

	       /* Bestimmung der Integrationsgewichte fuer Integrationsintervall */
	       l4_integrale( integr_gewichte, plow_box->lower_steps[0].step, plow_box->lower_steps[1].step, stepsize);
	       for(j=0; j<= add_here; j++) {
		 plow_box->lower_steps[j].integr_weight = integr_gewichte[j];
		 plow_box->lower_steps[j].lagrange4 = integr_gewichte[j]; /* wird spaeter in Fall e) wieder benutzt */
	       };
	     };
             if (merke != DISC_ANZEIGE) 
	       plow_box->lower_steps[0].integr_weight += merke;
#ifdef KOSTEN
             log_kosten(3);
#endif
             add_here += 1;
             break;
    case 4 : /* Fall d) */
             plow_box->lower_steps[3].step = stepsize;
             { double integr_gewichte[3];

	       l3_integrale( integr_gewichte, plow_box->lower_steps[2].step, stepsize);
	       if (merke == DISC_ANZEIGE)
		 plow_box->lower_steps[0].integr_weight = plow_box->lower_steps[0].lagrange3;/*stammt aus Fall e)bzw.b)*/ 
	       else
		 plow_box->lower_steps[0].integr_weight = plow_box->lower_steps[0].lagrange3 + merke;

	       plow_box->lower_steps[1].integr_weight = plow_box->lower_steps[1].lagrange3;
	       plow_box->lower_steps[2].integr_weight = plow_box->lower_steps[2].lagrange3 + integr_gewichte[0];	       
	       plow_box->lower_steps[3].integr_weight = integr_gewichte[1];
	       plow_box->lower_steps[4].integr_weight = integr_gewichte[2];
	       /* hier brauchen die neu die berechneten Integrationsgewichte nicht gespeichert zu werden, da
		  Komponenten 2, 3 und 4 kein Integrationsintervall mehr bilden werden ! */
	     };

#ifdef KOSTEN
             log_kosten(3);
#endif
             add_here += 1;
             break;
    case 5 : /* Fall e) inklusive Uebertragung von low_box-Komponenten */
             /* Feststellen, ob Dummy-Element existiert. Kennzeichen : step == 0.0 und indicator == ZWISCHEN_ANZEIGE */
             if ((plow_box->next_up->step == 0.0) && (plow_box->next_up->indicator == ZWISCHEN_ANZEIGE)) {
	       /* Dummy existiert : Struktur kann fuer erstes zu uebertragendes Element benutzt werden */
	       first_step_added = plow_box->next_up;
	       for( i=0; i<ac_no_detT; i++) {
		 del_vector(&(first_step_added->values[0][i]));
		 del_vector(&(first_step_added->values[1][i]));
	       };
	       efree(first_step_added->values[0]);  
	       efree(first_step_added->values[1]);  
	     }
	     else {
	       first_step_added = (step_inbetween *)emalloc(sizeof(step_inbetween));
	       plow_box->next_up->next_down = first_step_added;
	     };
	     /* Uebertragung der ersten Komponente */
	     first_step_added->step = plow_box->lower_steps[0].step;
	     first_step_added->values[0] = plow_box->lower_steps[0].values[0];
	     first_step_added->values[1] = plow_box->lower_steps[0].values[1];
             if (merke == DISC_ANZEIGE) { 
	       first_step_added->integr_weight = plow_box->lower_steps[0].lagrange4; /* stammt aus Fall c) */
	       first_step_added->indicator = merke;
	     }
             else {
	       first_step_added->integr_weight = plow_box->lower_steps[0].lagrange4 + merke;
               first_step_added->indicator = plow_box->lower_steps[0].lagrange4;             
	     };

             merke = plow_box->lower_steps[3].lagrange4;/* letzter Stuetzpunkt des oberen Intervalls bleibt in 
							   low_box, weil gleichzeitig erster des unteren Intervalls */

	     /* Uebertragung der zweiten Komponente */
             ptr = (step_inbetween *)emalloc(sizeof(step_inbetween));
	     first_step_added->next_down = ptr;

             ptr->step = plow_box->lower_steps[1].step;
             ptr->values[0] = plow_box->lower_steps[1].values[0];
             ptr->values[1] = plow_box->lower_steps[1].values[1];
             ptr->integr_weight = plow_box->lower_steps[1].lagrange4;
             ptr->indicator = ZWISCHEN_ANZEIGE;
    						 
	     /* Uebertragung der dritten Komponente (letzte Uebertragung) */
             ptr->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
             ptr = ptr->next_down;

             ptr->step = plow_box->lower_steps[2].step;
             ptr->values[0] = plow_box->lower_steps[2].values[0];
             ptr->values[1] = plow_box->lower_steps[2].values[1];
             ptr->integr_weight = plow_box->lower_steps[2].lagrange4;
             ptr->indicator = ZWISCHEN_ANZEIGE;
             ptr->next_down = NULL;

             plow_box->next_up = ptr;
             plow_box->lower_steps[0].step = plow_box->lower_steps[3].step;
             plow_box->lower_steps[1].step = stepsize;             
             plow_box->lower_steps[0].values[0] = plow_box->lower_steps[3].values[0];
             plow_box->lower_steps[0].values[1] = plow_box->lower_steps[3].values[1];
             plow_box->lower_steps[1].values[0] = plow_box->lower_steps[4].values[0];
             plow_box->lower_steps[1].values[1] = plow_box->lower_steps[4].values[1];
#ifdef KOSTEN
             log_kosten(1);
#endif
             {  double integr_gewichte[3];

	       /* Bestimmung der Integrationsgewichte fuer Integrationsintervall in low_box */
		l3_integrale( integr_gewichte, plow_box->lower_steps[3].step, stepsize);
		plow_box->lower_steps[0].lagrange3 = integr_gewichte[0]; /* wird wieder	in Fall d) benutzt */	
		plow_box->lower_steps[0].integr_weight = integr_gewichte[0] + merke;
		plow_box->lower_steps[1].lagrange3 = integr_gewichte[1];		
		plow_box->lower_steps[1].integr_weight = integr_gewichte[1]; 
		plow_box->lower_steps[2].lagrange3 = integr_gewichte[2];		
		plow_box->lower_steps[2].integr_weight = integr_gewichte[2];
	      };
#ifdef KOSTEN
             log_kosten(3);
#endif
              for(j=2; j<5; j++) {
		/* plow_box->lower_steps[j].step = 0.0;*/
		/* values[] auf keinen Fall loeschen, da Elemente an verkettete Liste gehaengt wurden bzw.
		   in obere Komponenten von low_box umgehaengt wurden  -> statt dessen : */
		for(i=0; i<2; i++) {
		  plow_box->lower_steps[j].values[i] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
		  for( k=0; k<ac_no_detT; k++) {
		    plow_box->lower_steps[j].values[i][k] = new_vector(); 
		    /* einfach NULL setzen, waere effektiver, aber unuebersichtlicher, da dann immer beachtet
		       werden muesste, dass low_box nur teilweise initialisiert ist */
		  };
		};
	      };
              add_here = 3;
              break;
     default : 
#ifdef DEBUG_AC
               printf(" 2nd Default-Fehler in update_int_structures : add_here = %d ",add_here);
#endif
               break;
    };
  };  

  
  /* TEIL 2 : UPDATEN bzw. NEUAUFBAU DER INTERPOLATIONSFELDER ENTLANG TAU-LINIEN */
  
  /* wichtig : Schleife muss bei 0 beginnen wegen upper_steps-Behandlung, in der evtl. ein Dummy
               an step_inbetween-Liste angehaengt werden muss                                 */
  for(k=0; k<ac_no_detT; k++) {

    /* bestimme el1 bzw. el3 als Zeiger auf step_inbetween-Element vor dem ersten Schnitt bei 0.2*stepsize
       bzw vor dem zweiten Schnitt bei stepsize */
    el1 = up_boxes[k];
    h_sum = el1->step;
    while ( h_sum < (hdurch5)) {
      el1 = el1->next_down;
      h_sum += el1->step;
    };
    el3 = el1;
    h_sum1 = h_sum;
    while ( h_sum < (stepsize- EXT_PRECISION)) {
      el3 = el3->next_down;
      h_sum += el3->step;
    };
    /* el1 zeigt auf Element vor dem Schnitt bei 0.2*stepsize */
    /* el3 zeigt auf Element vor dem Schnitt bei stepsize */

#ifdef DEBUG_AC    
    if (el3->indicator == DISC_ANZEIGE) {
      printf(" el3-Zeiger zeigt auf DISC_ANZEIGE-Element !!!! fataler Fehler !\n");
      printf(" Indikator von el3-Nachfolger : %f\n", el3->next_down->indicator);
      printf(" up_boxes[%d]->step = %f, el1->step = %f, el3->step = %f, stepsize = %f \n",
	     k, up_boxes[k]->step, el1->step, el3->step, stepsize);
    };
#endif    
    /* el3->next_down zeigt unter Umstaenden auf das Element der Kette,
       auf das genau der zweite Cut faellt -> erleichtert Integration !*/

#ifdef KOSTEN
    log_kosten(1);
#endif
    
    /* zu Bezeichnungen siehe auch Erlauterungen in interpolation_fields.c */
    /* Eine aufwendigere Behandlung der p_d(t,x)-Interpolationsfelder ist ueberhaupt nur notwendig, falls
       transient_time + stepsize einen Zeitpunkt beschreibt, der jenseits des Bereichs liegt, der von
       den aeusseren Interpolationsstuetzpunkten aufgespannt wird :
       - t0t1t2t3 : transient_time + stepsize > Zeitpunkt von t3 -> einzelne Punkte ersetzen !
       - tn1t1t2t3 oder tn1tn2tt2t3 : transient_time + stepsize > Zeitpunkt von tn1 -> Neuaufbau des Felds */

    if ((((int_fields[k][0].int_point + old_stepsize + stepsize) > EXT_PRECISION) && (interpol_update[k])) || ((!interpol_update[k]) && (int_fields[k][0].int_point + old_stepsize + stepsize > (EXT_PRECISION + int_fields[k][3].int_point)))) {
      
      
      if (!(interpol_update[k])) {
	/* Neuaufbau des Interpolationsfelds */
	interpol_update[k] = 1;

	/* Kriterien fuer die Einbeziehung der Vektoren ins Interpolationsfeld :
	   - Interpolationsfeld sollte moeglichst viel zukuenftige Zeit ueberspannen, um moeglichst
	     lange unveraendert zu bleiben. Beachtet werden muss dabei natuerlich, dass der erste und
	     zweite Cut innerhalb des Interpolationsintervalls liegen. Um hier effektiv zu sein, liegt
	     der erste Schnitt bei transient+0.2*stepsize, an der Stelle, an der in dopri5() der erste
	     Interpolationswert berechnet wird. Es liegt also nach Neufbau hoechstens ein Stuetzpunkt
	     vor dem ersten Cut (es sei denn eine Diskontinuitaet laesst dies nicht zu !), waehrend
	     die anderen Stuetzpunkte den direkt folgenden step_inbetween-Elementen entstammen. Waere dann
	     der zweite Schnitt nicht im Interpolationsintervall enthalten, muessen Stuetzpunkt 3 und 4
	     (entspricht t2 und t3) als Paerchen soweit in die Zukunft verschoben werden, bis t3 rechts
	     von dem zweiten Cut liegt !
	   - Da auf einen Neuaufbau irgendwann immer eine Ersetzung von ein oder zwei Stuetzpunkten folgt
	     und bei Ersetzung nur el3->values[0][k]->num abgefragt wird, muss nur der letzte ins 
	     Interpolationsfeld einbezogene Vektor markiert werden (num gleich 1 setzen). Beachte, dass el3
	     im Falle einer Ersetzung hoechstens auf das letzte step_inbetween-Element, das zur Interpolation
	     herangezogen wurde, zeigen kann, denn sonst laege der neue zweite Cut immer noch innerhalb
	     des Interpolationsintervalls und die Stuetzpunkte muessten nicht veraendert werden.      */

	if (el1 == el3) {  /* erster Cut und zweiter Cut liegen zwischen denselben step_inbetween-Elementen */

	  if (el1 == up_boxes[k]) { /* beide Cuts direkt nach dem oberstem upper_steps-Element */
	    ptr = el3->next_down;
	    if (ptr->indicator == DISC_ANZEIGE) {
	      D_case( k, int_fields[k], ptr, h_sum, old_stepsize, randdata);
	      /* fuer Interpolationsfeld herangezogener Vektor kann unmarkiert bleiben, da das Feld nach
		 Erreichen der Diskontinuitaet erst nach disk_uebertragung() wieder bearbeitet wird */
	    }
	    else if (ptr->next_down->indicator == DISC_ANZEIGE)	 {  
	      ZD_case( k, int_fields[k], ptr, h_sum, old_stepsize, randdata);
	      /* fuer Interpolationsfeld herangezogener Vektor kann unmarkiert bleiben, da das Feld nach
		 Erreichen der Diskontinuitaet erst nach disk_uebertragung() wieder bearbeitet wird */
	    }
	    else {
	      NZD_case( k, int_fields[k], ptr, h_sum, old_stepsize, randdata);
	      ptr->next_down->next_down->values[0][k]->num = 1;
	      /* zeigt an, dass dieser Vektor schon als Stuetzpunkt fuer das Interpolationsfeld einging */
	    };
	  }
	  else {  /* el1 != up_boxes[k] */
	    ptr = el3->next_down;
	    if (ptr->indicator == DISC_ANZEIGE) {
	      if (el1->values[0][k]->num)
		D_case( k, int_fields[k], ptr, h_sum, old_stepsize, randdata);
	      else /* el1->values[0][k]->num == 0, d.h. kein Interpolationspunkt */
		ZD_case( k, int_fields[k], el1, (h_sum-(el1->step)), old_stepsize, randdata);
	    }
	    else if (ptr->next_down->indicator == DISC_ANZEIGE) {
	      if (el1->values[0][k]->num)
		ZD_case( k, int_fields[k], ptr, h_sum, old_stepsize, randdata);
	      else /* el1->values[0][k]->num == 0, d.h kein Interpolationspunkt */
		NZD_case( k, int_fields[k], el1, (h_sum-(el1->step)), old_stepsize, randdata);
	    }
	    else {
	      if (el1->values[0][k]->num) {
		NZD_case( k, int_fields[k], ptr, h_sum, old_stepsize, randdata);
		ptr->next_down->next_down->values[0][k]->num =1;
	      }
	      else { /* el1->values[0][k]->num == 0, d.h kein Interpolationspunkt */
		ptr = ptr->next_down;
		NNZD_case( k, int_fields[k], el1, (h_sum-(el1->step)), ptr, (h_sum + (el1->next_down->step)), old_stepsize, randdata);
		ptr->next_down->values[0][k]->num = 1;
	      }
	    };
	  };
	}
	else if (el1->next_down == el3) { /* *el1 und *el3 sind benachbart */
	  if (el1 == up_boxes[k]) {
	    ptr = el3->next_down;
	    if (ptr->indicator == DISC_ANZEIGE) {
	      if (el3->values[0][k]->num) 
		D_case( k, int_fields[k], ptr, h_sum, old_stepsize, randdata);
	      else {
		/* el3->values[0][k]->num == 0, d.h. kein Interpolationspunkt */
		ZD_case( k, int_fields[k], el3, (h_sum-(el3->step)), old_stepsize, randdata);
	      };
	    }
	    else {	   
	      if (el3->values[0][k]->num) {
		ZD_case( k, int_fields[k], ptr, h_sum, old_stepsize, randdata);
		ptr->next_down->values[0][k]->num = 1;
	      }
	      else { /* el3->values[0][k]->num == 0, d.h. kein Interpolationspunkt */
		NZD_case( k, int_fields[k], el3, (h_sum-(el3->step)), old_stepsize, randdata);
		ptr->next_down->values[0][k]->num = 1;
	      }
	    };
	  }
	  else {  /* el1 != up_boxes[k], aber *el1 und *el3 benachbart */
	    ptr = el3->next_down;
	    if (ptr->indicator == DISC_ANZEIGE) {
	      
	      if (el1->values[0][k]->num) {
		if (el3->values[0][k]->num)
		  D_case( k, int_fields[k], ptr, h_sum, old_stepsize, randdata);
		else /* el3->values[0][k]->num == 0, d.h. kein Unterpolationspunkt */
		  ZD_case( k, int_fields[k], el3, h_sum1, old_stepsize, randdata);
	      }
	      else /* el1->value[0][k]s->num == 0, d.h. kein Interpolationspunkt */
		NZD_case( k, int_fields[k], el1, (h_sum1 - (el1->step)), old_stepsize, randdata);
	    }
	    else { 
	      
	      if (el1->values[0][k]->num) {
		if (el3->values[0][k]->num) {
		  ZD_case( k, int_fields[k], ptr, h_sum, old_stepsize, randdata);
		  ptr->next_down->values[0][k]->num = 1;
		}
		else { /* el3->values[0][k]->num == 0, d.h. kein Interpolationspunkt */
		  NZD_case( k, int_fields[k], el3, h_sum1, old_stepsize, randdata);
		  ptr->next_down->values[0][k]->num = 1;
		}  
	      }
	      else { /* el1->values[0][k]->num == 0, d.h kein Interpolationspunkt */
		NNZD_case( k, int_fields[k], el1, (h_sum1-(el1->step)), ptr, h_sum, old_stepsize, randdata);
		ptr->next_down->values[0][k]->num = 1;
	      }
	    };
	    
	  };
	}
	else { /* el1 und el3 nicht identisch und nicht benachbart : die Normalfaelle  */
	  if (el1 == up_boxes[k]) {
	    sonderbehandlung( k, int_fields[k], el1, h_sum1, el3, h_sum, old_stepsize, randdata);
	    /* ACHTUNG : h_sum's relativ anders */
	    el3->next_down->values[0][k]->num = 1;
	  }
	  else { 
	    NNZD_case( k, int_fields[k], el1, (h_sum1 - (el1->step)), el3, (h_sum - (el3->step)), old_stepsize, randdata);
	    el3->next_down->values[0][k]->num = 1;
	  };
	};
	
      }
      else { /* interpol_update[k] == 1 */
	/* zu Bezeichnungen siehe auch Erlauterungen in interpolation_fields.c */
	/* Es gilt hier immer folgende Lage der Stuetzpunkte im Interpolationsfeld, da ein Neuaufbau
	   des Feldes vorausging :
	   Komponente 0 : juengster Stuetzpunkt t3
	   Komponente 1 : zweitjuengster Stuetzpunkt t2
	   Komponente 2 : zweitaeltester Stuetzpunkt t1
	   Komponente 3 : aeltester Stuetzpunkt t0
	   im folgenden t0t1t2t3 abgekuerzt */  

	/* Wichtig : Alle neu ins Interpolationsfeld integrierten Vektoren rechts des zweiten Schnitts muessen
	             markiert werden fuer nachfolgende Neuaufbau. Das sind maximal zwei ! */
	{ /* block-Klammer */
	  
	  first_cut = int_fields[k][0].int_point + old_stepsize + hdurch5;
	  /* so berechnet sich die Lage des ersten Schnitts */
	  
	  if ((first_cut - int_fields[k][1].int_point) >= 0.0) {
	    /* erster Schnitt liegt rechts von t2 */
	    
	    if (el3->values[0][k]->num) { 
	      /* t3 liegt direkt vor zweitem Cut */
	      /* Situation : t0t1t2t3-cut2-(el3->next_down-Element); cut2 liegt zwischen t2 und cut2 */
	      ptr = el3->next_down;
	      if (ptr->indicator == DISC_ANZEIGE) {
		/* nur t0 wird jenseits des zweiten Cuts versetzt : t0t1t2t3 -> tn1t1t2t3 */
		
		/* relative Position von tn1 zu t3 */
		int_fields[k][3].int_point = int_fields[k][0].int_point + old_stepsize + h_sum;
		
		/* loesche t0 */
		del_vector(&(int_fields[k][3].y_values));
		
		/* Randomisierung des step_inbetween-Elements nach Interpolationsfeld */
		precision[k] = adapt_precision( ptr->values[0][k]);
		call_new_weighter( h_sum*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision[k]);
		int_fields[k][3].y_values = ranforfield(ptr->values[0][k], randdata[k]);
		
		ptr->values[0][k]->num = 1;   /* braeuchte nicht gesetzt werden, da keine Aenderung mehr an diesem
						 Interpolationsfeld bis nach Diskontinuitaet */
		newton_replace_1(int_fields[k]);
	      }
	      else { /* ptr->indicator != DISC_ANZEIGE */
		/* t0 und t1 koennen rechts vom zweiten Cut gelegt werden : t0t1t2t3 -> tn1tn2t2t3 */
		
		/* relative Position von tn1 und tn2 zu t3 */
		int_fields[k][2].int_point = int_fields[k][0].int_point + old_stepsize + h_sum;
		int_fields[k][3].int_point = int_fields[k][2].int_point + ptr->step;
		
		/* loesche t0 und t1 */
		del_vector(&(int_fields[k][2].y_values));
		del_vector(&(int_fields[k][3].y_values));
		
		/* Randomisierung des ersten step_inbetween-Elements nach Interpolationsfeld */
		precision[k] = adapt_precision( ptr->values[0][k]);
		call_new_weighter( h_sum*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision[k]);
		int_fields[k][2].y_values = ranforfield(ptr->values[0][k], randdata[k]);
		
		/* Randomisierung des zweiten step_inbetween-Elements nach Interpolationsfeld */
		precision[k] = adapt_precision( ptr->next_down->values[0][k]);
		call_new_weighter( (h_sum+(ptr->step))*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision[k]);
		int_fields[k][3].y_values = ranforfield(ptr->next_down->values[0][k], randdata[k]);
		
		ptr->values[0][k]->num = 1;
		ptr->next_down->values[0][k]->num = 1;
		newton_replace_2(int_fields[k]);
	      };
	    }
	    else { /* el3->values[0][k]->num == 0 */
	      /* Das step_inbetween-Element direkt vor dem zweiten Cut (el3-Element) wurde noch nicht ins 
		 Interpolationsfeld aufgenommen; somit koennen tn2 bzw.tn1 links und rechts des zweiten Cuts 
		 eingefuegt werden */
	      /* Situation : t0t1t2t3-..-(el3-Element)-cut2 ; cut1 liegt zwischen t3 und cut2 */
	      
	      /* t0t1t2t3 -> tn1tn2t2t3 */
	      
	      /* relative Position von tn1 und tn2 zu t3 */
	      int_fields[k][3].int_point = int_fields[k][0].int_point + old_stepsize + h_sum;
	      int_fields[k][2].int_point = int_fields[k][3].int_point - (el3->step);
	      
	      /* loesche t0 und t1 */
	      del_vector(&(int_fields[k][2].y_values));
	      del_vector(&(int_fields[k][3].y_values));
	      
	      /* Randomisierung des ersten step_inbetween-Elements nach Interpolationsfeld */
	      precision[k] = adapt_precision( el3->values[0][k]);
	      call_new_weighter( (h_sum-(el3->step))*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision[k]);
	      int_fields[k][2].y_values = ranforfield(el3->values[0][k], randdata[k]);
	      
	      /* Randomisierung des zweiten step_inbetween-Elements nach Interpolationsfeld */
	      precision[k] = adapt_precision( el3->next_down->values[0][k]);
	      call_new_weighter( h_sum*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision[k]);
	      int_fields[k][3].y_values = ranforfield(el3->next_down->values[0][k], randdata[k]);
	      
	      /* el3->values[0][k]->num nicht gesetzt, weil *el3 sowieso geloescht wird */
	      
	      el3->next_down->values[0][k]->num = 1;
	      newton_replace_2(int_fields[k]);
	    };
	  }
	  else if ((first_cut - int_fields[k][2].int_point) >= 0.0) {
	    /* erster Schnitt liegt zwischen t1 und t2 */
	    /* nur t0 wird rechts des zweiten Cuts versetzt */
	    /* Situation : t0t1-cut1-t2t3-cut2 ; egal, ob el3->Element gleich t3 ist, da der tn1
	       sowieso rechts von cut2 gelegt wird */
	    
	    /* t0t1t2t3 -> tn1t1t2t3 */
	    
	    /* relative Position von tn1 zu t3 */
	    int_fields[k][3].int_point = int_fields[k][0].int_point + old_stepsize + h_sum;
	    
	    /* loesche t0 */
	    del_vector(&(int_fields[k][3].y_values));
	    
	    /* Randomisierung des step_inbetween-Elements nach Interpolationsfeld */
	    precision[k] = adapt_precision( el3->next_down->values[0][k]);
	    call_new_weighter( h_sum*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision[k]);
	    int_fields[k][3].y_values = ranforfield(el3->next_down->values[0][k], randdata[k]);
	    
	    el3->next_down->values[0][k]->num = 1;
            newton_replace_1(int_fields[k]);
	  }
	  else { 
	    /* erster Schnitt liegt zwischen t0 und t1 */
	    /* t2 wird an die Stelle von t1 gesetzt;  jenseits von zweitem Schnitt */
	    /* Situation : t0-cut1-t1t2t3-cut2 ; egal, ob el3->Element gleich t3 ist, da der tn1
	       sowieso rechts von cut2 gelegt wird */
	    
	    /* t0t1t2t3 -> tn1t0t2t3 */
	    
	    /* loesche t1 */
	    del_vector(&(int_fields[k][2].y_values));
	    
	    /* haenge t0 um */
	    int_fields[k][2].y_values = int_fields[k][3].y_values;
	    /* relative Position von t0 und tn1 zu t3 */
	    int_fields[k][2].int_point = int_fields[k][3].int_point;
	    int_fields[k][3].int_point = int_fields[k][0].int_point + old_stepsize + h_sum;
	    
	    /* Randomisierung des step_inbetween-Elements nach Interpolationsfeld */
	    precision[k] = adapt_precision( el3->next_down->values[0][k]);
	    call_new_weighter( h_sum*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision[k]);
	    int_fields[k][3].y_values =	ranforfield(el3->next_down->values[0][k], randdata[k]);
	    
	    el3->next_down->values[0][k]->num = 1;
	    newton_replace_2(int_fields[k]);
	  };  
	}; /* block-Klammer */

	interpol_update[k] = 0;   /* beim naechsten Mal muss das Feld vollstaendig neu aufgebaut werden,
	                             weil ein neuer Bezugspunkt festgelegt werden muss */
	int_fields[k][0].int_point += old_stepsize;
        /* setze den neuen aktuellen Zeitpunkt; bei aktueller Zeitpunkt+stepsize liegt der zukuenftiger aktuelle Zpkt*/
	
      };  /* if interpol_update... */
    }
    else { /* Interpolationspolynom belassen */
      
      int_fields[k][0].int_point += old_stepsize;  /* neuen aktuellen Zeitpunkt setzen */
    };
#ifdef KOSTEN
    log_kosten(6);
#endif

    /* TEIL 3 : UPPER_STEPS-BEHANDLUNG */

    /* in folgender Programmierung werden alle tau's gleich behandelt, indem step_inbetween's ueber 
     Obergrenze ( oberste tau-Linie ) hinaus beibehalten und erst nachtraeglich geloescht werden */
    /* beachte : values[1] in upper_steps-Elementen bleibt undefiniert, sowie die die Zeiger auf sp_vector
	         mit Index ungleich k in values[0] */
       
    ptr = el3;
    /* ptr zeigt also jetzt auf das step_inbetween-Element echt vor dem zweiten Schnitt */
    following = ptr->next_down;

#ifdef DEBUG_AC 
    if (ptr->indicator == DISC_ANZEIGE)
      printf(" analoger el3-Fehler : ptr zeigt auf DISC_ANZEIGE-Element !\n");      
#endif
#ifdef DEBUG_AC 
    if ((ptr->next_down->indicator == DISC_ANZEIGE) && (discontinuity)) {
      printf(" moeglicher Fehler : Diskontinuitaet wird verfehlt ! \n");
      printf(" Gruppe = %d, h_sum = %.12f, stepsize = %.12f\n", k, h_sum, stepsize);
    };
#endif

     /* Test, ob naechstes Glied der Integrationskette genau getroffen wird, was bedeuten wuerde, dass
	ein step_inbetween-Element genau auf die tau-Linie der Gruppe k faellt.                     */
    
     if (fabs(h_sum-stepsize) <= EXT_PRECISION) {
       /* Ein step_inbetween-Element faellt genau mit der tau-Linie zusammen */
       hits_exactly[k] = 1;    

       if (following->indicator == DISC_ANZEIGE) {
	 /* d.h. Gruppe k ist Ausloeser der Diskontinuitaet und trifft somit genau auf DISC_ANZEIGE-Element */
#ifdef DEBUG_AC
	 if (!discontinuity) 
	   printf(" Fehler : Gruppe %d soll Diskontinuitaet ausloesen, aber discontinuity == 0 !!??", k);
#endif
	 /* es wird ein einziges upper_steps-Element erzeugt, das zwar keinen Beitrag fuer die Integration 
	    bringt und auch in disk_uebertragung() wieder geloescht werden wird, dennoch aus anschaulichen
	    und algorithmischen Gruenden Sinn macht */
	 up_boxes[k] = (step_inbetween *)emalloc(sizeof(step_inbetween));
	 up_boxes[k]->step = DISC_ANZEIGE;    /* != 0.0 als Unterscheidungsmerkmal gegenueber Randintegral */
	 up_boxes[k]->integr_weight = 0.0;
	 /* up_boxes[k]->indicator = DISC_ANZEIGE; ueberfluessig, aber spaeter wichtig bei allgemein verteilten
	    Schaltzeiten fuer generandom() */
	 /* Zuweisung der values-Komponenten wird zu Beginn des Randomisierungsteils erledigt */
	 
	 up_boxes[k]->next_down = following->next_down;
	 /* beachte : Nachfolgeelemente hinter *following existieren auf jeden Fall, da discontinuity gleich 1, 
	              also gerade step_inbetween-Elemente aus low_box erzeugt wurden    */ 
	 
	 no_upper_steps[k] = 1;
	 
       } else if (following->indicator == ZWISCHEN_ANZEIGE) { 
	 if (following->next_down->indicator == DISC_ANZEIGE) {
	   { /* block-Klammer : Randintegralberechnung */
	     double integr_gewichte[3];
	     step_inbetween *walker;
	     
	     l3_randintegrale( integr_gewichte, ptr->step, following->step);
	     
	     /* erzeuge drei upper_steps : */
	     /* erstes upper_steps-Element */
	     up_boxes[k] = (step_inbetween *)emalloc(sizeof(step_inbetween));
	     up_boxes[k]->step = 0.0;        /* Erkennungszeichen fuer Randintegralintervall */
	     up_boxes[k]->integr_weight = integr_gewichte[0];
	     /* up_boxes[k]->indicator = ptr->indicator; ueberfluessig, weil dieses Element sowieso nach seiner
		Aufsummierung fuer Integration sofort wieder geloescht wird */
	     up_boxes[k]->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
	     up_boxes[k]->values[0][k] = copy_vector( ptr->values[0][k]);
	     /* values[1] und values[0][j!=k] bleiben undefiniert ! */
	     
	     /* zweites upper_steps-Element */
	     up_boxes[k]->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
	     walker = up_boxes[k]->next_down;
	     walker->step = following->step;
	     walker->integr_weight = integr_gewichte[1];
	     walker->indicator = ZWISCHEN_ANZEIGE;
	     /* Indikator wurde doch gesetzt, weil default-Wert Probleme machte */
	     /* Zuweisung der values-Komponenten wird zu Beginn des Randomisierungsteils erledigt */
	     
	     /* drittes upper_steps-Element */
	     walker->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
	     walker = walker->next_down;
	     walker->step = 0.0; /* = following->next_down->step */
	     walker->integr_weight = integr_gewichte[2];
	     walker->indicator = DISC_ANZEIGE;
	     walker->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
	     walker->values[0][k] = copy_vector( following->next_down->values[0][k]);
#ifdef KOSTEN
             log_kosten(3);
#endif
	     
	     /* pruefen, ob nach DISC_ANZEIGE-Element noch eins kommt; falls nein, muss dummy-Element
		erzeugt werden */
	     ptr = following->next_down;
	     if (ptr->next_down == NULL) {
	       /* ACHTUNG : dummy-Element bei endgueltiger Ablehnung nicht bedingungslos loeschen,
		  erst mit old_up_boxes[0]->->.. darauf pruefen, ob dummy schon vorher existierte */ 
	       ptr->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
	       ptr = ptr->next_down;
	       /* ACHTUNG : ohne Initialisierung Widerspruch mit Integration (pointer-error) */
	       ptr->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
	       ptr->values[1] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
	       for( i=0; i<ac_no_detT; i++) {
		 ptr->values[0][i] = new_vector();
		 ptr->values[1][i] = new_vector();
	       };
	       ptr->integr_weight = 0.0;
	       ptr->next_down = NULL;

	       ptr->step = 0.0;                    /* das sind die Kennzeichen eines dummy-Elements */
	       ptr->indicator = ZWISCHEN_ANZEIGE;
	       
	       plow_box->next_up = ptr;
	       walker->next_down = ptr;
	     }
	     else {
	       walker->next_down = ptr->next_down;
	     };
	     
	   }; /* Ende der block-Klammer 'Randintegralberechnung' */
	   no_upper_steps[k] = 2; /* weil oberstes Element noch geloescht wird */
	 }
	 else if (following->next_down->indicator == ZWISCHEN_ANZEIGE) {
	   /* Fall 0-0X/D => neues 3er-Integral ! */
	   up_boxes[k] = (step_inbetween *)emalloc(sizeof(step_inbetween));
	   up_boxes[k]->step = following->step;
	   
	   OX_case( k, up_boxes[k], following->next_down, plow_box);
	   no_upper_steps[k] = 3;
	 }
	 else {  /* following->next_down-Element ist Doppelrandpunkt */
	   ptr = following->next_down->next_down;
	   up_boxes[k] = (step_inbetween *)emalloc(sizeof(step_inbetween));
	   up_boxes[k]->step = following->step;
	   if (ptr->next_down->indicator == ZWISCHEN_ANZEIGE) {
	     /* Fall : 0-X00X/D => zwei 3er-Integrale */
	     OXOX_case( k, up_boxes[k], following->next_down, plow_box);	      
	     no_upper_steps[k] = 5;
	   }
	   else {
	     /* Fall : 0-X0X/D => neues 4er-Integral */
	     OOX_case( k, up_boxes[k], following->next_down, plow_box);	      
	     no_upper_steps[k] = 4;
	   };
	 }  /* following->next_down->indicator-Abfrage */
       } /* following-Element ist Doppelrandpunkt */
       else {
	 /* Faelle X-0X/D und X-00X/D */
	 up_boxes[k] = (step_inbetween *)emalloc(sizeof(step_inbetween));
	 up_boxes[k]->step = following->step;
	 up_boxes[k]->integr_weight = following->indicator; /* weil following-Element doppelter Randpunkt */
	 up_boxes[k]->indicator = ZWISCHEN_ANZEIGE; /* erst wichtig fuer generandom() */
	 /* Zuweisung der values-Komponenten wird zu Beginn des Randomisierungsteils erledigt */
 	    
	 up_boxes[k]->next_down = following->next_down;
	 
	 no_upper_steps[k] = 1;
       };
       
     } 
     else {
       /* tau-Linie faellt zwischen zwei step_inbetween-Elemente */
       hits_exactly[k] = 0;
       
       if ( following->indicator == DISC_ANZEIGE ) {
	 /* Randintegralberechnung */

	 /* erzeuge drei upper_steps : */
	 /* erstes upper_steps-Element */
	 up_boxes[k] = (step_inbetween *)emalloc(sizeof(step_inbetween));
	 up_boxes[k]->step = 0.0;
	 /* up_boxes[k]->indicator ungesetzt, weil dieses Element sowieso nach seiner
	    Aufsummierung fuer Integration sofort wieder geloescht wird */

	 { /* block-Klammer */
	   double integr_gewichte[3];
	   double help_step;
	   double abs_val;
	   step_inbetween *walker;
	   
	   help_step = h_sum-stepsize; /* von tau-Linie bis following-Element */
	   abs_val = (ptr->step)-help_step;
#ifdef DEBUG_AC
	   if (abs_val<0.0) 
	     printf(" Schrittweitenfehler bei Randintegralberechnung : erste Schrittweite < 0 \n");
#endif
	   l3_randintegrale( integr_gewichte, abs_val, help_step);
	   
	   up_boxes[k]->integr_weight = integr_gewichte[0];
	   up_boxes[k]->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
	   up_boxes[k]->values[0][k] = copy_vector( ptr->values[0][k]);
	   
	   /* zweites upper_steps-Element */
	   up_boxes[k]->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
	   walker = up_boxes[k]->next_down;
	   
	   walker->step = help_step;
	   walker->integr_weight = integr_gewichte[1];
	   walker->indicator = ZWISCHEN_ANZEIGE;
	   /* Indikator wurde doch gesetzt, weil default-Wert Probleme machte !*/
	   /* Zuweisung der values-Komponenten wird zu Beginn des Randomisierungsteils erledigt */

	   /* drittes upper_steps-Element */
	   walker->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
	   walker = walker->next_down;
	   
	   walker->step = 0.0; /* = following->step */
	   walker->integr_weight = integr_gewichte[2];
	   walker->indicator = DISC_ANZEIGE;
	   walker->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
	   walker->values[0][k] = copy_vector( following->values[0][k]);
	   
#ifdef KOSTEN
	   log_kosten(3);
#endif
	   
	   /* pruefen, ob nach DISC_ANZEIGE-Element noch eins kommt; falls nein, muss dummy-Element
	      erzeugt werden */
	   ptr = following;
	   if (ptr->next_down == NULL) {
	     /* ACHTUNG : dummy-Element bei endgueltiger Ablehnung nicht bedingungslos loeschen,
		erst mit old_up_boxes[0]->->.. darauf pruefen, ob dummy schon vorher existierte */ 
	     ptr->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
	     ptr = ptr->next_down;
	     /* ACHTUNG : ohne Initialisierung Widerspruch mit Integration      */
	     ptr->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
	     ptr->values[1] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
	     for( i=0; i<ac_no_detT; i++) {
	       ptr->values[0][i] = new_vector();
	       ptr->values[1][i] = new_vector();
	     };
	     ptr->integr_weight = 0.0;
	     ptr->next_down = NULL;

	     ptr->step = 0.0;                    /* das sind die Kennzeichen eines dummy-Elements */
	     ptr->indicator = ZWISCHEN_ANZEIGE;
	    
	     plow_box->next_up = ptr;
	     walker->next_down = ptr;
	   }
	   else {
	     walker->next_down = ptr->next_down;
	   };
	   
	 }; /* Ende der block-Klammer : Randintegralberechnung */
	 
	 no_upper_steps[k] = 2;  /* da erstes upper_steps-Element nach Integrationssummation wieder geloescht wird */
       }
       else if (following->indicator == ZWISCHEN_ANZEIGE) {
	 /* Faelle : G-00X/D bzw. G-0X/D */
	 up_boxes[k] = (step_inbetween *)emalloc(sizeof(step_inbetween));
	 up_boxes[k]->step = h_sum-stepsize;
	 ptr = following->next_down;
	 if (ptr->indicator == ZWISCHEN_ANZEIGE) {
	   /* G-OOX/D -> ein 4er Integral */
	   OOX_case( k, up_boxes[k], following, plow_box);
	   no_upper_steps[k] = 4;
	 }
	 else {
	   /* G-OX/D -> ein 3er Integral */
	   OX_case( k, up_boxes[k], following, plow_box);	  
	   no_upper_steps[k] = 3;
	   /* ACHTUNG : bedarf einer Sonderbehandlung, da neue upper_steps-Elemente niemals auf alte verweisen
	      duerfen, denn alte werden ja geloescht ! Also muessen solche alten upper_steps-Elemente nochmals
	      kopiert werden */
	   if ((old_no_upper_steps[k]==5)&&(stepsize<(old_up_boxes[k]->step))) {
	     /* dann muss ptr->indicator != DISC_ANZEIGE gewesen sein */
	     no_upper_steps[k] = 5;
	     following = up_boxes[k]->next_down->next_down;
	     for(i=0;i<2;i++) {
	       following->next_down = (step_inbetween *)emalloc(sizeof(step_inbetween));
	       following = following->next_down;
	       ptr = ptr->next_down;
	       following->step = ptr->step;
	       following->integr_weight = ptr->integr_weight;
	       following->indicator = ptr->indicator;
	       following->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
	       following->values[0][k] = copy_vector( ptr->values[0][k]);
	     };
	     following->next_down = ptr->next_down;
	   };
	 };  /* Ende ptr->indicator-Abfrage */ 
       }
       else { /* following-Element ist Doppelrandpunkt */
	 /* Faelle : G-X00X/D bzw. G-X0X/D */
	 up_boxes[k] = (step_inbetween *)emalloc(sizeof(step_inbetween));
	 up_boxes[k]->step = h_sum-stepsize;
	 ptr = following->next_down;
	 if (ptr->next_down->indicator == ZWISCHEN_ANZEIGE) {
	   /* G-XOOX/D -> zwei 3er Integrale */
	   OXOX_case( k, up_boxes[k], following, plow_box);
	   no_upper_steps[k] = 5;
	 }
	 else {
	   /* G-XOX/D -> ein 4er Integral */
	   OOX_case( k, up_boxes[k], following, plow_box);
	   no_upper_steps[k] = 4;
	 };
       };
     }; /* ENDE : hits_exactly[k] = 0 */
    
  }; /* Ende der k-for-Schleife */
  
#ifdef KOSTEN
  log_kosten(1);
#endif
  
#ifdef DEBUG_AC
  /* Plausibilitaetstest : */
  j=0;
  for(i=0;i<ac_no_detT;i++) 
    j += hits_exactly[i];
  if ((j==0)&&(discontinuity)) 
    printf(" Fehler : Diskontinuitaet angezeigt, aber keine Gruppe loest sie aus !!??? \n");
#endif
  

  /* TEIL 4 : RANDOMISIERUNG UND NUMERISCHE INTEGRATION */
    
  /* Integralvorauswertung */
  for( k=0; k<ac_no_detT; k++) {
    /* wichtig, dass diese Schleife aufsteigend durchlaufen wird */

    clear_vector(pdts[k]);
    
    if (init) {
      /* Randomisierung fuer die areas-Werte und ihre Hinzuaddierung zu pdts[] */
      /* beachte, dass Impulse immer zwischen den DISC_ANZEIGE-Elementen auftreten koennen, die ueberall
	 zwischen tau-Linie und t-Achse liegen koennen. Aus unserer areas-Datenstruktur alleine geht nicht
	 hervor, wo genau der Impuls liegt. Das waere aber notwendig bei allgemeinen verteilten Schaltzeiten,
	 wo z.B. Auswertung der Dichte an der Stelle des Impulses in Berechnung eingeht */
      if (add_here == 2) {
	w_vec = areas_ptr[k].first;   /* mindestens ein Vektor exisiert ! */
      	while (w_vec->next != NULL) {
	  if (w_vec->no_entries) {
	    /* Leervektor ( = new_vector() ), was oft vorkommt, einfach ueberspringen */ 
	    precision[k] = adapt_precision( w_vec);
	    call_new_weighter( stepsize*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision[k]);
	    randomize_quickly(w_vec, randdata[k]); 
#ifdef KOSTEN
	    log_kosten(2);
#endif
	    vector_add_mult( pdts[k], w_vec, 1.0);
#ifdef KOSTEN
	    log_kosten(3);
#endif
	  };
	  w_vec = w_vec->next;
	};
	/* w_vec zeigt auf letztes Element */
	if (w_vec->no_entries) {
	  /* Leervektor ( = new_vector() ), was oft vorkommt, einfach ueberspringen */ 
	  call_new_weighter( stepsize*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), MAX_ABS_PREC);
	  randomize_quickly(w_vec, randdata[k]); 
#ifdef KOSTEN
	  log_kosten(2);
#endif
	  vector_add_mult( pdts[k], w_vec, 1.0);
#ifdef KOSTEN
	  log_kosten(3);
#endif
	};
	
      }
      else { /* add_here ungleich 2 */
	w_vec = areas_ptr[k].first;   /* mindestens ein Vektor exisiert ! */
	while (w_vec != NULL) {
	  if (w_vec->no_entries) {
	    /* Leervektor ( = new_vector() ), was oft vorkommt, einfach ueberspringen */ 
	    precision[k] = adapt_precision( w_vec);
	    call_new_weighter( stepsize*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision[k]);
	    randomize_quickly(w_vec, randdata[k]); 
#ifdef KOSTEN
	    log_kosten(2);
#endif
	    vector_add_mult( pdts[k], w_vec, 1.0);
#ifdef KOSTEN
	    log_kosten(3);
#endif
	  };
	  w_vec = w_vec->next;
	};
      }; /* Ende else-Teil */
    };
    
    ptr = up_boxes[k];
    if (ptr->step == 0.0) { /* gesetzter Erkennungswert fuer Randintegral */
      
      /* traf man auf Diskontinuitaet, so wurde up_boxes[k]->step auf DISC_ANZEIGE (!= 0) gesetzt !! */
      /* Randintegral : erster Punkt liegt oberhalb der tau-Linie und wird nach
	 Integralberechnung nicht mehr benoetigt -> also loeschen */
      
      precision[k] = adapt_precision( ptr->values[0][k]);
      call_new_weighter( stepsize*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision[k]);
      randomize_quickly(ptr->values[0][k], randdata[k]); 
#ifdef KOSTEN
      log_kosten(2);
#endif
      vector_add_mult( pdts[k], ptr->values[0][k], ptr->integr_weight);
#ifdef KOSTEN
      log_kosten(3);
#endif
      up_boxes[k] = ptr->next_down;
      del_vector(&(ptr->values[0][k]));
      efree(ptr->values[0]);
      efree(ptr);
      ptr = up_boxes[k];
    };
    if (hits_exactly[k]) {
      /* tau-Linie traf genau auf ein step_inbetween-Element, das dann auf jeden Fall im Interpolationsfeld
         als Stuetzpunkt vertreten sein muss, denn Punkte links und rechts des oder genau auf dem zweiten
	 Cut werden dort immer einbezogen genauso wie Stuetzpunkte jenseits des zweiten Cuts immer direkt
	 in der step_inbetween-Liste aufeinanderfolgen */
      /* moegliche Fehlerquelle, falls Toleranz zu klein, trat bisher nie auf */
      first_cut = int_fields[k][0].int_point + stepsize; 
      /* ausnahmsweise wird in first_cut die Lage des zweiten Cuts relativ zum Bezugspunkt des bereits 
	 angepassten Interpolationsfeldes gespeichert */
 
      if (fabs(first_cut) < EXT_PRECISION) {
	/* getroffenes Element ist juengster Stuetzpunkt des Interpolationspolynom */
	del_vector(&(int_fields[k][0].aux_coefficients));
	int_fields[k][0].aux_coefficients = copy_vector(int_fields[k][0].y_values);/* nicht nur gleiche Zeiger !*/
	/* setzen dieses Vektors ist wichtig fuer boundary_conditions(), rk2(), rk3() und dopri5() */

	ptr->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
	ptr->values[0][k] = copy_vector(int_fields[k][0].y_values); 
	/* neben Integration auch wichtig, falls im naechsten Schritt Randintegralberechnung ansteht */

	ptr->values[0][k]->num = 1; /* damit dieser Punkt spaeter nicht irrtuemlich noch ins Interpolationsfeld
				       einbezogen wird */
#ifdef KOSTEN
	log_kosten(1);
#endif
	vector_add_mult( pdts[k], ptr->values[0][k], ptr->integr_weight);
#ifdef KOSTEN
	log_kosten(3);
#endif
      }
      else {
	
	for(i=1;i<4;i++) {
	  
	  if (fabs(first_cut - int_fields[k][i].int_point) < EXT_PRECISION) {
	    /* getroffenes Element ist Stuetzpunkt mit Index i des Interpolationsfeldes */
	    del_vector(&(int_fields[k][0].aux_coefficients));
	    int_fields[k][0].aux_coefficients = copy_vector(int_fields[k][i].y_values);/* nicht nur gleiche Zeiger !*/
	    /* setzen dieses Vektors ist wichtig fuer boundary_conditions(), rk2(), rk3() und dopri5() */
	    
	    ptr->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
	    ptr->values[0][k] = copy_vector(int_fields[k][i].y_values); 
	    /* neben Integration auch wichtig, falls im naechsten Schritt Randintegralberechnung ansteht */
	    ptr->values[0][k]->num = 1;/* damit dieser Punkt spaeter nicht irrtuemlich noch ins Interpolationsfeld
					  einbezogen wird */
#ifdef KOSTEN
	    log_kosten(1);
#endif
	    vector_add_mult( pdts[k], ptr->values[0][k], ptr->integr_weight);
#ifdef KOSTEN
	    log_kosten(3);
#endif
	    break;
	  };
	};
#ifdef DEBUG_AC
	if (i == 4) 
	  printf(" Fataler Toleranzfehler : hits_exactly[%d] = 1, aber Chaos mit int_fields[%d] !", k, k);
#endif
      };
    }
    else { /* hits_exactly[k] = 0 */
      /* tau-Linie trifft nicht auf step_inbetween-Element */
      del_vector(&(int_fields[k][0].aux_coefficients));
      /* beachte, dass Interpolationsfeld bereits angepasst wurde */
      int_fields[k][0].aux_coefficients = polynom_auswertung( stepsize, int_fields[k]);
      /* setzen dieses Vektors ist wichtig fuer boundary_conditions(), rk2(), rk3() und dopri5() */
      
      ptr->values[0] = (sp_vectors *)emalloc(ac_no_detT*sizeof(sp_vectors));
      ptr->values[0][k] = copy_vector( int_fields[k][0].aux_coefficients);
      /* neben Integration auch wichtig, falls im naechsten Schritt Randintegralberechnung ansteht */

      ptr->values[0][k]->num = 1; /* damit dieser Vektor nicht versehentlich fuer's Interpolationsfeld
				     herangezogen wird */
#ifdef KOSTEN
      log_kosten(2);
#endif
      vector_add_mult( pdts[k], ptr->values[0][k], ptr->integr_weight);
#ifdef KOSTEN
      log_kosten(3);
#endif
      
    };

    /* ab hier hat hits_exactly eine andere Bedeutung : speichert R"uckgabewert aus ad_prec_ifnec() */
    precision[k] = adapt_precision( ptr->next_down->values[0][k]);
    call_new_weighter( stepsize*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision[k]);

    /* Randomisierung und num. Integration der upper_steps-Elemente nach dem obersten : */
    for( j=1; j<no_upper_steps[k]; j++) {
      ptr = ptr->next_down;
      hits_exactly[k] = ad_prec_ifnec( &(precision[k]), ptr->values[0][k]);
      if (hits_exactly[k])
	call_new_weighter( stepsize*(randdata[k]->qmax_ii), &(randdata[k]->L), &(randdata[k]->R), &(randdata[k]->Betas), precision[k]);

      randomize_quickly(ptr->values[0][k], randdata[k]); 
#ifdef KOSTEN
      log_kosten(2);
#endif
      vector_add_mult( pdts[k], ptr->values[0][k], ptr->integr_weight);
#ifdef KOSTEN
      log_kosten(3);
#endif
    };
    parser[k] = ptr->next_down; /* zeigt auf step_inbetween-Element hinter upper_steps */

  }; /* Ende der for-Schleife ( wichtig, dass fuer k aufsteigend durchlaufen, damit ptr auf
	step_inbetween-Element hinter den upper_steps der Gruppe ac_no_detT-1 zeigt */

  /* folgende Schleife besorgt effektives Randomisieren durch Aufsammeln der zu randomisierenden
     Vektoren ausgehend vom Ende der upper_steps der Gruppe ac_no_detT-1 */
  for( k=(ac_no_detT-2); k>=0; k--) {
    while (ptr->next_down != parser[k]) {
      ptr = ptr->next_down;
      for( j=k+1; j<ac_no_detT; j++) {
	/* nur die Vektoren der hoeheren Gruppen muessen fuer diese step_inbetween-Elemente
	   randomisiert werden, da andere tau-Linien darunter liegen  */
	hits_exactly[j] = ad_prec_ifnec( &(precision[j]), ptr->values[0][j]);
	if (hits_exactly[j])
	  call_new_weighter( stepsize*(randdata[j]->qmax_ii), &(randdata[j]->L), &(randdata[j]->R), &(randdata[j]->Betas), precision[j]);

	randomize_quickly(ptr->values[0][j], randdata[j]); 
#ifdef KOSTEN
	log_kosten(2);
#endif
	vector_add_mult( pdts[j], ptr->values[0][j], ptr->integr_weight);
#ifdef KOSTEN
	log_kosten(3);
#endif
      };
    };
  };
  /* ptr zeigt jetzt auf das step_inbetween-Element, ab dessen Nachfolger die Randomisierung fuer alle
     Vektoren unter values[0] durchgefuehrt werden muss ! 
     Anmerkung : Nachfolger existiert immer, ist aber u.U. das dummy-Element */
  /* falls dummy : integr_weight ist Null, Vektoren alle new_vector()'s */
  ptr = ptr->next_down; /* = parser[0]; */
  
  while (ptr != NULL) {
    for( j=0; j<ac_no_detT; j++) {

      hits_exactly[j] = ad_prec_ifnec( &(precision[j]), ptr->values[0][j]);
      if (hits_exactly[j])
	call_new_weighter( stepsize*(randdata[j]->qmax_ii), &(randdata[j]->L), &(randdata[j]->R), &(randdata[j]->Betas), precision[j]);

      randomize_quickly(ptr->values[0][j], randdata[j]); 
#ifdef KOSTEN
      log_kosten(2);
#endif
      vector_add_mult( pdts[j], ptr->values[0][j], ptr->integr_weight);
#ifdef KOSTEN
      log_kosten(3);
#endif
    };    
    ptr = ptr->next_down;
  };
  /* ptr zeigt ins Nirwana ! */
  
  /* unten angekommen, jetzt fehlen noch die Integrationsstuetzpunkte in low_box */
  for(i=0;i<(add_here-2);i++) { /* add_here-1, weil letzter Punkt erst nach Auswertung der boundary_conditions() 
				   hinzugerechnet werden kann; beachte hierbei die Extrapolation */
    for( j=0; j<ac_no_detT; j++) {
      hits_exactly[j] = ad_prec_ifnec( &(precision[j]), plow_box->lower_steps[i].values[0][j]);
      if (hits_exactly[j])
	call_new_weighter( stepsize*(randdata[j]->qmax_ii), &(randdata[j]->L), &(randdata[j]->R), &(randdata[j]->Betas), precision[j]);
      randomize_quickly(plow_box->lower_steps[i].values[0][j], randdata[j]); 
#ifdef KOSTEN
      log_kosten(2);
#endif
      vector_add_mult( pdts[j], plow_box->lower_steps[i].values[0][j], plow_box->lower_steps[i].integr_weight);
#ifdef KOSTEN
      log_kosten(3);
#endif
    };    
  };
  /* letzter Punkt muss besonders genau randomisiert werden, damit relative Genauigkeit eingehalten wird */
  switch (add_here) {
  case 3:
  case 4:
  case 5: for( j=0; j<ac_no_detT; j++) {
           call_new_weighter( stepsize*(randdata[j]->qmax_ii), &(randdata[j]->L), &(randdata[j]->R), &(randdata[j]->Betas), MAX_ABS_PREC);
	   randomize_quickly(plow_box->lower_steps[(add_here-2)].values[0][j], randdata[j]); 
#ifdef KOSTEN
	   log_kosten(2);
#endif
	   vector_add_mult( pdts[j], plow_box->lower_steps[(add_here-2)].values[0][j], plow_box->lower_steps[(add_here-2)].integr_weight);
#ifdef KOSTEN
	   log_kosten(3);
#endif
	  };    
          break;
  case 2: /* siehe auch Operation auf plow_box->next_up->values[1][j] nach low_box-Behandlung */
	  for( j=0; j<ac_no_detT; j++) {
	    /* Korrekturen bleiben umsonst, falls dummy-Element existiert */
	    vector_add_mult( pdts[j], plow_box->next_up->values[0][j], (-1.0)*(plow_box->next_up->integr_weight));
	    del_vector(&(plow_box->next_up->values[0][j]));
	    
	    call_new_weighter( stepsize*(randdata[j]->qmax_ii), &(randdata[j]->L), &(randdata[j]->R), &(randdata[j]->Betas), MAX_ABS_PREC);
	    plow_box->next_up->values[0][j] = ranforfield(plow_box->next_up->values[1][j], randdata[j]);
	    /* Sicherheitskopie muss unver"andert bleiben */  
	    
	    randomize_quickly(plow_box->lower_steps[0].values[0][j], randdata[j]); 
#ifdef KOSTEN
	    log_kosten(2);
#endif
	    
	    vector_add_mult( pdts[j], plow_box->next_up->values[0][j], plow_box->next_up->integr_weight);
	    vector_add_mult( pdts[j], plow_box->lower_steps[0].values[0][j], plow_box->lower_steps[0].integr_weight);
#ifdef KOSTEN
	    log_kosten(3);
#endif
	  };    
          break;
  case 1: {  /* Korrektur ist immer richtig, da "uber extrapol_start auf das gesuchte Element 
		zugegriffen werden kann */

	     step_inbetween *buf_el;

	     buf_el = extrapol_start->next_down->next_down;

	     for( j=0; j<ac_no_detT; j++) {

	       vector_add_mult( pdts[j], buf_el->values[0][j], (-1.0)*(buf_el->integr_weight));
	       del_vector(&(buf_el->values[0][j]));
	       
	       call_new_weighter( stepsize*(randdata[j]->qmax_ii), &(randdata[j]->L), &(randdata[j]->R), &(randdata[j]->Betas), MAX_ABS_PREC);
	       buf_el->values[0][j] = ranforfield(buf_el->values[1][j], randdata[j]);
	       /* Sicherheitskopie muss unver"andert bleiben */  
#ifdef KOSTEN
	       log_kosten(2);
#endif
	       vector_add_mult( pdts[j], buf_el->values[0][j], buf_el->integr_weight);
#ifdef KOSTEN
	       log_kosten(3);
#endif
	     };    

	  };
          break;
  default:printf(" Fehler in on_tx_plain.c: add_here kurz vor Extrapolation gleich %d", add_here);  
          break;
  };      
  /* Ende der Integralvorauswertung */
  

  /* TEIL 5 : EXTRAPOLATION (optional) */
  
  if (preemption) {
    /* nur dann wird extrapoliert */
    if (discontinuity) {
      /* bei existierendem dummy falsch f"ur Gruppe k=0, genauso falls
	 upper_steps[0]-Elemente unterhalb des extrapol_start-Elements 
	 liegen */
      /* bei existierendem dummy wird die Werte aus Randbedingung f"ur 
	 Gruppe k=0 nie richtig gesetzt, sonst werden schlechte Sch"atzungen 
	 durch Iteration korrigiert ! */
      /* -> gesonderte Extrapolation f"ur Gruppe 0 mit upper_steps[0]-Elementen
	    durchf"uhren (ganz analog der extrapol_routine() ) -> noch zu tun */
      last_extrapolation( extrapol_start);
    }
    else {
      regular_extrapolation( add_here, plow_box, (merke != DISC_ANZEIGE));
    };
  };
#ifdef KOSTEN
  log_kosten(5);
#endif
  
  return(add_here-1);			   
  
}  /* Ende main */

/*****************************************************************************************/
   
      






