/*******************************************************************************
Modul:              runge_k.c 

Purpose:            Kunge-Kutta Loesungsverfahren

Project Director:   Reinhard German

Programmer:         Joerg W. Mitzlaff

History:            10.94 (Dipl.Arbeit)
*******************************************************************************/

/* #define DEBUG */

#include <stdio.h>
#include <math.h>
#include "frontend.h"
#include "Options.h"

/* Import aus main.c */
extern TransientOptions transientOptions;

/* Import ais showTransCurves.c */
extern void drawNextStep();
extern void showSolveStatus();

/* Import aus shared.c */
extern double ggTeiler();
extern sp_matrix *randomi();
extern FILE *open_solve();
extern void update_solve();
extern void close_solve();

/* Prototyping */
static sp_vector *runge_kutta_differenziation();
static int middle_part();

/******************************************************************************/
void runge_kutta_case(tnethome,modelpath,netname,c)
/******************************************************************************/
char *tnethome;
char *modelpath;
char *netname;
solve_basis *c;
{
   FILE *ftmp;
   int time;
   int numCompDatas;
   int numDatas;
   int akt_output;
   double maxX;
   int i,j;
   int ndetT;
   sp_vector *temp_vec;
   sp_matrix *temp_matrix;
   sp_vectors *old_pdttau;
   sp_matrix *m_exp_Q_EE_h;  /* macht erstes omega Intervall genauer */
   sp_matrices *m_exp_Q_d_h; /* macht Integrationsschritt schneller */
 
   /* temporaere Daten zur Speicherung der Zwischenergebnisse */
   double omega;            /* ggT von allen detT */
   double h;                /* ggT von omega und method_stepsize */
   int *steps_in_tau;       /* Anzahl der Messpunkte jeder detT */

   sp_vectors **p_d_t_0s;   /* (buffer fuer voriges Intervall) zur Berechnung von p'd(t,tau) */
   int *new_pdt0s_element;  /* naechster freie Eintrag im vector feld p_d_t_0s[i], der als ring buffer angelegt ist */

   sp_vectors *p_d_t_tau;   /* akt. p'd(t,tau) zur Berechnung von p'E(t) und p'd(t,0) */
   sp_vector *p_E_t;        /* == component -> p_E;  akt. p'E(t) zur Berechnung von p'd(t,0) */
   sp_vectors *p_d_t_0;     /* akt. p'd(t,0) zur Berechnug der p_d_mexp und zum ablegen in p_d_t_0s buffer */

   sp_vectors **p_d_mexp;   /* p'd(t-x,0) * e^(Q'd*x) fuer x von 0 bis tau'd zur Berechnung der p'd(t) */
   int *new_pdmexp_element; /* naechster freie Eintrag im vector feld  p_d_mexp[i], der als ring buffer angelegt ist */

   sp_vectors *p_d_t;       /* == component -> p_d; akt. p'd(t)  zur Berechnung von p(t) */
   sp_vector *p_t;          /* p(t) ist Loesung und wird an update_solve() uebergeben */


   /* Initialisation */
   p_E_t = c -> p_E;
   p_d_t = c -> p_d;
   ndetT = c -> no_detT;

   p_t = new_vector();             /* p_t = p(0) */
   p_t = copy_vector(p_E_t);
   for(i=0;i<ndetT;i++)
   {
     vector_add_mult(p_t, p_d_t[i], 1.0);
   }

   temp_vec = new_vector();
   temp_matrix = new_matrix();

   /* KGV von allen tau und h und event. trans. time finden */
   if(transientOptions.output_data==Point)
     omega = transientOptions.transient_time;
   else
     omega = c->detMatrices[0].det_firing_time ; /* erster Durchlauf nun umsonst */

   for(i=0;i<ndetT;i++)                 
      omega = ggTeiler(omega, c->detMatrices[i].det_firing_time);
   h = ggTeiler(transientOptions.stepsize_method,omega);

   printf("internal step size h %s to %g.\n",
	  (transientOptions.stepsize_method!=h ? "has changed" : "was set"),
	  h);

   if(transientOptions.stepsize_result < transientOptions.stepsize_method)
   {
     transientOptions.stepsize_result = transientOptions.stepsize_method;
     printf("step size for output has changed to %g.\n", transientOptions.stepsize_result);
   }
   else
   {
     printf("step size for output was set to %g.\n", transientOptions.stepsize_result);
   }

   steps_in_tau = (int *)emalloc(sizeof(int)*ndetT);
   for(i=0;i<ndetT;i++)
   {
     steps_in_tau[i] = c->detMatrices[i].det_firing_time / h;   /* muss ganze Zahl rauskommen, da h ggT ist */
     steps_in_tau[i] ++;
   }

   p_d_t_0s = (sp_vectors **)emalloc(sizeof(sp_vectors *) * ndetT);
   for(i=0;i<ndetT;i++)
   {
      p_d_t_0s[i] = (sp_vectors *)emalloc(sizeof(sp_vectors) * steps_in_tau[i]);
      for(j=0;j<steps_in_tau[i];j++)
         (p_d_t_0s[i])[j] = new_vector();
   }
   new_pdt0s_element = (int *)emalloc(sizeof(int)*ndetT);
   for(i=0;i<ndetT;i++)
      new_pdt0s_element[i] = 0;

   old_pdttau = (sp_vectors *)emalloc(sizeof(sp_vectors) * ndetT);
   for(i=0;i<ndetT;i++)
      old_pdttau[i] = new_vector();

   m_exp_Q_d_h = (sp_matrices *)emalloc(sizeof(sp_matrices) * ndetT);

   p_d_t_tau = (sp_vectors *)emalloc(sizeof(sp_vectors) * ndetT);
   for(i=0;i<ndetT;i++)
      p_d_t_tau[i] = new_vector();

   p_d_t_0 = (sp_vectors *)emalloc(sizeof(sp_vectors) * ndetT);
   for(i=0;i<ndetT;i++)
      p_d_t_0[i] = new_vector();

   p_d_mexp = (sp_vectors **)emalloc(sizeof(sp_vectors *) * ndetT);
   for(i=0;i<ndetT;i++)
   {
      p_d_mexp[i] = (sp_vectors *)emalloc(sizeof(sp_vectors) * steps_in_tau[i]);
      for(j=0;j<steps_in_tau[i];j++)
         (p_d_mexp[i])[j] = new_vector();
   }
   new_pdmexp_element = (int *)emalloc(sizeof(int)*ndetT);
   for(i=0;i<ndetT;i++)
      new_pdmexp_element[i] = 0;

#ifdef DEBUG_M
  printf("************************************************************\n");
  printf("\n Q_EE Matrix\n");
  show_matrix(c->Q_EE);
  for( i=0 ; i<c->no_detT;i++)
  {
      printf("\n Q_d  Matrizen d=%d\n", i);
      show_matrix(c->detMatrices[i].Q_d);
  }
  printf("************************************************************\n");
#endif

#ifdef KOSTEN
  log_kosten(1);
#endif

   /* ersetze Q Matrizen durch exp_Q Matrizen *************************************************************/
   m_exp_Q_EE_h = randomi(c->Q_EE, h);

   for(i=0;i<ndetT;i++)
     m_exp_Q_d_h[i] = randomi(c->detMatrices[i].Q_d, h);
   for(i=0;i<ndetT;i++)
   {
     temp_matrix = randomi(c->detMatrices[i].Q_d, c->detMatrices[i].det_firing_time);
     del_matrix( &(c->detMatrices[i].Q_d) );
     c->detMatrices[i].Q_d = temp_matrix;
   }


#ifdef KOSTEN
  log_kosten(2);
#endif

#ifdef DEBUG_M
  printf("************************************************************\n");
  printf("\n e hoch (Q_EE * h) Matrix\n");
  show_matrix(m_exp_Q_EE_h);
  for( i=0 ; i<c->no_detT;i++)
  {
    printf("\n Q_Ed Matrizen d=%d\n", i);
    show_matrix(c->Q_Ed[i]);
  }
  for( i=0 ; i<c->no_detT;i++)
  {
      printf("\n e hoch (Q_d * d) Matrizen d=%d\n", i);
      show_matrix(c->detMatrices[i].Q_d);
  }
  for( i=0 ; i<c->no_detT;i++)
  {
      printf("\n e hoch (Q_d * h) Matrizen d=%d\n", i);
      show_matrix( m_exp_Q_d_h[i]);
  }
  printf("\n Q_dE Matrizen muessen leer sein, sonst ware ich nicht hier!\n");
  for( i=0 ; i<c->no_detT;i++)
  {
      printf("\n Delta_dE Matrizen d=%d\n", i);
      show_matrix(c->detMatrices[i].Delta_dE);
  }
  for( i=0 ; i<c->no_detT;i++)
  {
    for( j=0 ; j<c->no_detT;j++)
    {
      printf("\n Delta_dc Matrizen d=%d c=%d\n", i,j);
      show_matrix(c->detMatrices[i].Delta_dc[j]);
    } 
  }
  printf("************************************************************\n");
#endif


   /* jetzt gehts los ***********************************************************************************/
   /* jetzt gehts los ***********************************************************************************/
   /* jetzt gehts los ***********************************************************************************/
   numDatas = ceil(transientOptions.transient_time/transientOptions.stepsize_result);
   maxX = numDatas * transientOptions.stepsize_result;
   numDatas ++;    /* t = 0.0 ist auch ein Messwert */

   numCompDatas = ceil(transientOptions.transient_time/transientOptions.stepsize_method);

   ftmp = open_solve(tnethome,modelpath,netname,numDatas,maxX);

   update_solve(tnethome,modelpath,netname,ftmp,c->no_markings,p_t,0.0,maxX);
   akt_output = 1; /* fuer output */
   for(time=1;time<=numCompDatas;time++)
   {
      /* copy old p'd(t,tau) fuer Runge Kutta Differenziation ******************************************/
      for(i=0;i<ndetT;i++)
      {
	 clear_vector(old_pdttau[i]);
         old_pdttau[i] = copy_vector(p_d_t_tau[i]);
      }


#ifdef KOSTEN
  log_kosten(1);
#endif

      /* compute p'd(t,tau) mit p'd(t-tau,0)   *********************************************************/
      for(i=0;i<ndetT;i++)
      {
	clear_vector(p_d_t_tau[i]);
	p_d_t_tau[i] =  copy_vector( (p_d_t_0s[i])[new_pdt0s_element[i]] );
	mult_vector_matrix( p_d_t_tau[i], c->detMatrices[i].Q_d);
      }

#ifdef KOSTEN
  log_kosten(3);
#endif

#ifdef DEBUG1
  for( i=0 ; i<c->no_detT;i++)
  {
    printf(" p'd (%lf,tau) d=%d  ", t, i);
    show_vector(p_d_t_tau[i]);
  } 
#endif

      /* compute p'E(t) mit p'd(t,tau) *****************************************************************/
      if( (time * h) < omega)
      {
	/* Matrixexponential p'E(t+h) = p'E(t) * e^(Q'EE * h) */
	mult_vector_matrix(p_E_t, m_exp_Q_EE_h);
      }
      else
      {
	p_E_t = runge_kutta_differenziation(p_E_t, old_pdttau, p_d_t_tau, c, h);
      }

#ifdef KOSTEN
  log_kosten(4);
#endif

#ifdef DEBUG2
  printf(" p'E (%lf) ",t);
  show_vector(p_E_t);
#endif

      /* compute p'd(t,0) mit p'E(t) und p'd(t,tau) fuer alle d in detT ********************************/
      for(i=0;i<ndetT;i++)
      {
	 clear_vector(p_d_t_0[i]);
         p_d_t_0[i] = copy_vector(p_E_t);
         mult_vector_matrix(p_d_t_0[i], c->Q_Ed[i]);
         for(j=0;j<ndetT;j++)
         {
	    clear_vector(temp_vec);
            temp_vec = copy_vector(p_d_t_tau[j]);
            mult_vector_matrix(temp_vec, c->detMatrices[j].Delta_dc[i]);
            vector_add_mult(p_d_t_0[i], temp_vec,1.0);
         }
	 /* kopieren in p'd(t,0) Liste */
	 clear_vector( (p_d_t_0s[i])[new_pdt0s_element[i]] );
	 (p_d_t_0s[i])[new_pdt0s_element[i]] =  copy_vector(p_d_t_0[i]);
	 new_pdt0s_element[i]++;
	 if(new_pdt0s_element[i]==steps_in_tau[i])
	   new_pdt0s_element[i] = 0;
      }

#ifdef KOSTEN
  log_kosten(5);
#endif

#ifdef DEBUG3
  for( i=0 ; i<c->no_detT;i++)
  {
    printf(" p'd (%lf,0) d=%d ", t, i);
    show_vector(p_d_t_0[i]);
  } 
#endif


      /* compute p'd(t) mit p'd(t-x,0) ****************************************************************/

      /* using matrix exponential property of step by step e^(Q*h) */
      for(i=0;i<ndetT;i++)
	for(j=0;j<steps_in_tau[i];j++)
	  mult_vector_matrix( (p_d_mexp[i])[j], m_exp_Q_d_h[i]);

      /* copy p'd(t,0) to p_d_mexp vector feld */
      for(i=0;i<ndetT;i++)
      {
	clear_vector( (p_d_mexp[i])[new_pdmexp_element[i]] );
	(p_d_mexp[i])[new_pdmexp_element[i]] = copy_vector(p_d_t_0[i]);
	new_pdmexp_element[i]++;
	if(new_pdmexp_element[i]==steps_in_tau[i])
	  new_pdmexp_element[i] = 0;
      }

      /* using trapezoid rule for integrating over p_d_mexp vector feld */
      for(i=0;i<ndetT;i++)
      {
	clear_vector(p_d_t[i]);
	for(j=0;j<steps_in_tau[i];j++)
	{
	  if(middle_part(j, new_pdmexp_element[i], steps_in_tau[i]))
	    vector_add_mult(p_d_t[i], (p_d_mexp[i])[j], h);
	  else
	    vector_add_mult(p_d_t[i], (p_d_mexp[i])[j], h/2.0);	    
	}
      }

#ifdef KOSTEN
  log_kosten(6);
#endif

#ifdef DEBUG4
  for( i=0 ; i<c->no_detT;i++)
  {
    printf(" p'd (%lf) d=%d  ", t, i);
    show_vector(p_d_t[i]);
    printf("------------------------------\n");
  } 
#endif


      /* output required ? ***************************************************************************/
      if( (time * h) == (akt_output * transientOptions.stepsize_result) )
      {
	 akt_output++;

         /* compute p(t) */
	 clear_vector(p_t);
         p_t = copy_vector(p_E_t);
         for(i=0;i<ndetT;i++)
         {
            vector_add_mult(p_t, p_d_t[i], 1.0);
         }
	 update_solve(tnethome,modelpath,netname,ftmp,c->no_markings,p_t,time*h,maxX);
      }
   }
   close_solve(modelpath,netname,ftmp);

   /* free allocated memory */
}

/******************************************************************************/
static int middle_part(j,new_pdmexp_element,steps_in_tau)
/******************************************************************************/
int j;                 /* akt. step in tau */
int new_pdmexp_element;
int steps_in_tau;
{
  if(new_pdmexp_element > 0)
  {
    if( j != (new_pdmexp_element - 1) )
      return(True);
    else
      return(False);
  }
  else
  {
    if( j !=  (steps_in_tau - 1) )
      return(True);
    else
      return(False);
  }
}

/**********************************************************************************/
static sp_vector *runge_kutta_differenziation(p_E_t, old_pdttau, new_pdttau, c, h)
/**********************************************************************************/
sp_vector *p_E_t;
sp_vectors *old_pdttau;
sp_vectors *new_pdttau;
solve_basis *c;
double h;
{
   int i;
   sp_vector *temp_vec;
   sp_vector *k1;
   sp_vector *k2;
   sp_vector *k3;
   sp_vector *k4;
   sp_vector *k1_summand;
   sp_vector *k23_summand;
   sp_vector *k4_summand;


   k1 = copy_vector(p_E_t);
   k2 = copy_vector(p_E_t);
   k3 = copy_vector(p_E_t);
   k4 = copy_vector(p_E_t);

   temp_vec = new_vector();

   k1_summand = new_vector();
   for(i=0;i<c->no_detT;i++)
   {
      clear_vector(temp_vec);
      temp_vec = copy_vector(old_pdttau[i]);
      mult_vector_matrix(temp_vec, c->detMatrices[i].Delta_dE);
      vector_add_mult(k1_summand,temp_vec,1.0);    
   }

   k4_summand = new_vector();
   for(i=0;i<c->no_detT;i++)
   {
      clear_vector(temp_vec);
      temp_vec = copy_vector(new_pdttau[i]);
      mult_vector_matrix(temp_vec, c->detMatrices[i].Delta_dE);
      vector_add_mult(k4_summand,temp_vec,1.0);    
   }

   k23_summand = new_vector();
   for(i=0;i<c->no_detT;i++)
   {
      vector_add_mult(old_pdttau[i],new_pdttau[i],1.0);    /* old_pdttau kann ueberschrieben werden */
      mult_vector_skalar(old_pdttau[i],0.5);
      mult_vector_matrix(old_pdttau[i], c->detMatrices[i].Delta_dE);
      vector_add_mult(k23_summand,old_pdttau[i],1.0);    
   }

   mult_vector_matrix(k1,c->Q_EE);
   vector_add_mult(k1,k1_summand,1.0);
   mult_vector_skalar(k1,h);

   vector_add_mult(k2,k1,1.0/2.0);
   mult_vector_matrix(k2,c->Q_EE);
   vector_add_mult(k2,k23_summand,1.0);
   mult_vector_skalar(k2,h);

   vector_add_mult(k3,k2,1.0/2.0);
   mult_vector_matrix(k3,c->Q_EE);
   vector_add_mult(k3,k23_summand,1.0);
   mult_vector_skalar(k3,h);

   vector_add_mult(k4,k3,1.0);
   mult_vector_matrix(k4,c->Q_EE);
   vector_add_mult(k4,k4_summand,1.0);
   mult_vector_skalar(k4,h);

   vector_add_mult(p_E_t,k1,1.0/6.0);
   vector_add_mult(p_E_t,k2,1.0/3.0);
   vector_add_mult(p_E_t,k3,1.0/3.0);
   vector_add_mult(p_E_t,k4,1.0/6.0);

   del_vector( &temp_vec );
   del_vector( &k1 );
   del_vector( &k2 );
   del_vector( &k3 );
   del_vector( &k4 );
   del_vector( &k1_summand );
   del_vector( &k23_summand );
   del_vector( &k4_summand );

   return(p_E_t);
}

 
