/********************************************************************
Modul :             runge_kutta_methods.c

Purpose :           bietet DOPRI5, RK2, RK3 an !

Project Director :  Reinhard German

Programmer :        Armin Heindl

History :           18.5.95 ( Diplomarbeit )
************************************************************************/

#include <math.h>
#include <stdio.h>      /* fuer NULL */
#include "frontend.h"

extern int ac_no_detT;

/* Konstantendefinitionen fuer DOPRI5 */
static double c2  = 0.2;
static double a21 = 0.2;

static double c3  = 0.3;
static double a31 = 3.0/40.0;
static double a32 = 9.0/40.0;

static double c4  = 0.8;
static double a41 = 44.0/45.0;
static double a42 = -56.0/15.0;
static double a43 = 32.0/9.0;

static double c5  = 8.0/9.0;
static double a51 = 19372.0/6561.0;
static double a52 = -25360.0/2187.0;
static double a53 = 64448.0/6561.0;
static double a54 = -212.0/729.0;

static double a61 = 9017.0/3168.0;
static double a62 = -355.0/33.0;
static double a63 = 46732.0/5247.0;
static double a64 = 49.0/176.0;
static double a65 = -5103.0/18656.0;

static double a71 = 35.0/384.0;
/*            a72 = 0.0              */
static double a73 = 500.0/1113.0;
static double a74 = 125.0/192.0;
static double a75 = -2187.0/6784.0;
static double a76 = 11.0/84.0;

static double delta1 = 71.0/57600.0;
/*            delta2 = 0.0            */
static double delta3 = -71.0/16695.0;
static double delta4 = 71.0/1920.0;
static double delta5 = -17253.0/339200.0;
static double delta6 = 22.0/525.0;
static double delta7 = -1.0/40.0;

extern sp_vector *polynom_auswertung();
extern sp_vector *polynom3_auswertung();

/***************************************************************************************************/
/* Belegung von stored_k[] : 0 -> unvollstaendige Summe fuer p_E zum Zeitpunkt t+h
                             1 -> k1 ( dahin muss nach Schrittannahme k7 kopiert werden )
	     fuer i=2,3,4,5: i -> ki
	                     6 -> Zwischensumme fuer pdh_sum in dopri5() 
			     7 -> k7, das spaeter k1 fuer naechsten Schritt wird                    */
/****************************************************************************************************/
void rk2( repeated, stepsize, daten_basis, int_fields, pdt_fields, stored_k)

/*  Schema :  0 |
              1 | 1
              -----------
	        | 0.5 0.5      */    

/* Voraussetzung : - in int_fields[i][0].aux_coefficients stehen die juengst errechneten 
                     p_d(t+rk2_step,tau_d)-Werte                                        
		   - in stored_k[1] ist das k1 fuer rk2() gespeichert, aus normf_evaluation() 
                   - in pdt_fields ist juengster Wert von p_d(t) in Komponente 1 gespeichert */

int repeated;                     /* zeigt wiederholten Durchlauf an, wobei bestimmte Groessen nicht
				     komplett neu berechnet werden muessen */
double stepsize;
solve_basis *daten_basis;
interpolation_field *int_fields;
interpolation_field *pdt_fields;  /* beachte:in pdt_fields sind entsprechende int_points immer gleich ! */
sp_vector *stored_k[8];
{
  int i;
  sp_vector *hp_E;

  /* Formel : d(p_E(t))/dt = p_E(t)*Q_EE + d_sum{p_d(t,tau_d)*Delta_dE + p_d(t)*Q_dE} */

  if (!repeated) {
    /* neues k2 berechnen */
    del_vector(&(stored_k[7]));
    stored_k[7] = copy_vector( int_fields[0][0].aux_coefficients );
    mult_vector_matrix( stored_k[7], daten_basis->detMatrices[0].Delta_dE );
    for( i=1; i<ac_no_detT; i++) {
      hp_E = copy_vector( int_fields[i][0].aux_coefficients );
      mult_vector_matrix( hp_E, daten_basis->detMatrices[i].Delta_dE );
      vector_add_mult( stored_k[7], hp_E, 1.0);
      del_vector(&hp_E);
    };

    /* vorlaeufiges neues p_E berechnen */
    hp_E = copy_vector( daten_basis->p_E);
    vector_add_mult( hp_E, stored_k[1], stepsize);
    mult_vector_matrix( hp_E, daten_basis->Q_EE);
    vector_add_mult( hp_E, stored_k[1], 1.0);

    del_vector(&(stored_k[0]));
    stored_k[0] = copy_vector( daten_basis->p_E);
    vector_add_mult( stored_k[0], hp_E, (0.5*stepsize));
    del_vector(&hp_E);
  };

  /* k2 von rk2() vervollstaendigen und in stored_k[1] als unvollstaendiges k1 fuer rk3() abspeichern */
  del_vector(&(stored_k[1]));
  stored_k[1] = copy_vector(stored_k[7]);
  for( i=0; i<ac_no_detT; i++) {
    hp_E = copy_vector( pdt_fields[i][1].y_values);
    mult_vector_matrix( hp_E, daten_basis->detMatrices[i].Q_dE );
    vector_add_mult( stored_k[1], hp_E, 1.0);
    del_vector(&hp_E);
  };
  /* stored_k[1] kann in rk3 rechensparend eingesetzt werden ! */
  
  /* p_E vervollstaendigen */
  del_vector(&(daten_basis->p_E));
  daten_basis->p_E = copy_vector( stored_k[0]);
  vector_add_mult( daten_basis->p_E, stored_k[1], (0.5*stepsize));
}
/******************************************************************************/
		  
void rk3( repeated, stepsize, daten_basis, int_fields, pdt_fields, stored_k)

/*  Schema :   0   |
               1/2 | 1/2
	       1   |   0   1
	       1   |   0   0   1
	       ---------------------- 
	           | 1/6 2/3   0 1/6    */

/* Voraussetzung : - in int_fields[i][0].aux_coefficients stehen die juengst errechneten 
                     p_d(t+rk3_step,tau_d)-Werte                                        
		   - in stored_k[1] ist das k1 (bis auf p_EE*Q_EE) fuer rk3() gespeichert, aus rk2() 
                   - in pdt_fields ist juengster Wert von p_d(t) in Komponente 2 gespeichert, der
		     vorhergehende in Komponente 1 und der erste (auf Diskontinuitaet) in Komponente 0*/

int repeated;                     /* zeigt wiederholten Durchlauf an, wobei bestimmte Groessen nicht
				     komplett neu berechnet werden muessen */
double stepsize;
solve_basis *daten_basis;
interpolation_field *int_fields;
interpolation_field *pdt_fields;
sp_vector *stored_k[8];
{
  int i;
  static double half_stepsize; /* bleibt fuer Wiederholungen gleich */
  sp_vector *k2,*k3;
  sp_vector *hp_E, *pdh_sum;

  /* Formel : d(p_E(t))/dt = p_E(t)*Q_EE + d_sum{p_d(t,tau_d)*Delta_dE + p_d(t)*Q_dE} */

  if (!repeated) {

    half_stepsize  = 0.5*stepsize;

    /* um spaeter Bearbeitung von stored_k[1] in adaptive_stepcontrol() vornehmen zu koennen */
    del_vector(&(stored_k[7]));
    stored_k[7] = copy_vector( daten_basis->p_E);

    /* neues k1 berechnen (mit Hilfe von stored_k[1]) */
    del_vector(&(stored_k[0]));
    hp_E = copy_vector( daten_basis->p_E);
    mult_vector_matrix( hp_E, daten_basis->Q_EE);
    stored_k[0] = copy_vector( hp_E);

    vector_add_mult( stored_k[0], stored_k[1], 1.0);
    /* in stored_k[0] ist vorerst k1 gespeichert */

    /* stored_k[2] anrechnen <- hp_E */
    /* stored_k[3] anrechnen <- hp_E */
    del_vector(&(stored_k[2]));
    stored_k[2] = copy_vector( hp_E);
    del_vector(&(stored_k[3]));
    stored_k[3] = copy_vector( hp_E);
    del_vector(&hp_E);
    hp_E = copy_vector( stored_k[0]);
    mult_vector_matrix( hp_E, daten_basis->Q_EE);
    vector_add_mult( stored_k[2], hp_E, half_stepsize);
    del_vector(&hp_E);
    for( i=0; i<ac_no_detT; i++) {
      hp_E = polynom_auswertung( half_stepsize, int_fields[i]);
      mult_vector_matrix( hp_E, daten_basis->detMatrices[i].Delta_dE);
      vector_add_mult( stored_k[2], hp_E, 1.0);
      del_vector(&hp_E);
      hp_E = copy_vector( int_fields[i][0].aux_coefficients);
      mult_vector_matrix( hp_E, daten_basis->detMatrices[i].Delta_dE);
      vector_add_mult( stored_k[3], hp_E, 1.0);
      del_vector(&hp_E);
    };
    mult_vector_skalar( stored_k[0], stepsize/6.0); /* Schlussformel 1.Teil */
    vector_add_mult( stored_k[0], daten_basis->p_E, 1.0);
  };

  /* k2 vervollstaendigen <- stored_k[2] */
  k2 = copy_vector( stored_k[2]);
  pdh_sum = copy_vector( stored_k[3]); /* damit enthaelt pdh_sum auch p_E*Q_EE, was die besondere Bearbeitung 
					  von stored_k[1] nach rk3() in adaptive_stepcontrol() veranlasst  */
  for( i=0; i<ac_no_detT; i++) {
    hp_E = polynom3_auswertung( half_stepsize, pdt_fields[i]);
    mult_vector_matrix( hp_E, daten_basis->detMatrices[i].Q_dE);
    vector_add_mult( k2, hp_E, 1.0);
    del_vector(&hp_E);
    hp_E = copy_vector( pdt_fields[i][2].y_values);
    mult_vector_matrix( hp_E, daten_basis->detMatrices[i].Q_dE);
    vector_add_mult( pdh_sum, hp_E, 1.0);
    del_vector(&hp_E);
  };
  /* pdh_sum ist k3 bis auf (k2*h*Q_EE)-Term */
  del_vector(&(stored_k[1]));
  stored_k[1] = copy_vector( pdh_sum);
  /* ACHTUNG : spaeter muss p_E*Q_EE wieder abgezogen werden !*/

  del_vector(&(daten_basis->p_E));
  daten_basis->p_E = copy_vector( stored_k[0]);
  vector_add_mult( daten_basis->p_E, k2, (2.0*stepsize)/3.0);  /* Schlussformel 2.Teil */

  /* k3 vervollstaendigen <- pdh_sum, k2 */
  mult_vector_matrix( k2, daten_basis->Q_EE);
  k3 = copy_vector( pdh_sum);
  vector_add_mult( k3, k2, stepsize);
  /* in k2 k4 berechnen <- pdh_sum, k3 */
  mult_vector_matrix( k3, daten_basis->Q_EE);
  del_vector(&k2);
  k2 = copy_vector( pdh_sum);
  vector_add_mult( k2, k3, stepsize);
  /* 1/6-Schlussformel 3.Teil */
  vector_add_mult( daten_basis->p_E, k2, stepsize/6.0);

  del_vector(&k2);
  del_vector(&k3);
  del_vector(&pdh_sum);
}
/*****************************************************************************************/
void rel_error_est( perr, vec, deltavec)
/* berechnet die mittlere Summe der relativen Fehler in den einzelnen Komponenten des Vektors vec */

double *perr;
sp_vector *vec;
sp_vector *deltavec; /* praktisch y - y_Dach aus dopri5() */
{
	sp_entry	*entry1 = vec->entries;
	sp_entry	*entry2 = deltavec->entries;
	int		num1, num2;	

	while((entry1 != NULL) || (entry2 != NULL))
	{	num1 = entry1 == NULL ? 10000000 : entry1->num;
		num2 = entry2 == NULL ? 10000000 : entry2->num;
		if (num1 < num2)
		{	/* deltavec hat an dieser Stelle keinen Eintrag */
		        /* relativer Fehler wird nicht gr"osser */
			entry1 = entry1->next;
		}
		else if (num1 == num2)
		{	/* relativen Fehler dieser Komponente aufaddieren */
		        if (entry1->val > MAX_ABS_PREC) 
			  *perr += fabs(entry2->val)/(entry1->val);
		        else
			  *perr += fabs(entry2->val)/MAX_ABS_PREC;

			entry1 = entry1->next;
			entry2 = entry2->next;
		}
		else /* num1 > num2 */
		{	/* selten: Komponente wurde zu Null berechnet, aber mit
			           relativem Fehler                          */
		        *perr += fabs(entry2->val)/MAX_ABS_PREC;
			entry2 = entry2->next;
		}
	};
	if (vec->no_entries > 0)
	  *perr = *perr/(float)(vec->no_entries);
}
/*****************************************************************************************/
double dopri5( repeated, stepsize, daten_basis, int_fields, pdt_fields, stored_k)
/* Rueckgabewerte : - `error' fuer nachfolgend zu entscheidende Schrittannahme bzw. -ablehnung */
/*                  -  in daten_basis->p_E: Kandidat fuer den p_E(t)-Wert, falls Schritt angenommen wird,
                       KONSEQUENZ: alter Wert von p_E muss in adaptive_stepcontrol()  zunaechst gespeichert werden,
	               bis klar ist, ob Schritt angenommen werden kann */
/* Voraussetzung : - in int_fields[i][0].aux_coefficients stehen die juengst errechneten 
                     p_d(t+stepsize,tau_d)-Werte, oder wahrscheinlicher das Ergebnis der Auswertung
		     des Interpolationspolynoms an dieser Stelle 
		   - in stored_k[1] ist das k1 fuer dopri5() gespeichert 
                   - in pdt_fields ist juengster Wert von p_d(t) in Komponente 3 gespeichert     */

int repeated;                    /* zeigt wiederholten Durchlauf an, wobei bestimmte Groessen nicht
				    komplett neu berechnet werden muessen */
double stepsize;
solve_basis *daten_basis;
interpolation_field *int_fields; 
interpolation_field *pdt_fields; 				 
sp_vector *stored_k[8];
{
  int i;
  double denom;
  sp_vector *k2,*k3,*k4,*k5,*hp_E,*pdh_sum;
  
  /* bleiben fuer Wiederholungen gleich */
  static double c2_stepsize;
  static double c3_stepsize;
  static double c4_stepsize;
  static double c5_stepsize;

  double error = 0.0;

  /* Formel : d(p_E(t))/dt = p_E(t)*Q_EE + d_sum{p_d(t,tau_d)*Delta_dE + p_d(t)*Q_dE} */

  /* Unterschied zu rk2() und rk3() : stored_k[1] darf nicht veraendert werden, da Schritt abgelehnt
     werden kann, und es dann nochmal benoetigt wird */
  if (!repeated) {
    c2_stepsize = c2*stepsize;
    c3_stepsize = c3*stepsize;
    c4_stepsize = c4*stepsize;
    c5_stepsize = c5*stepsize;
    
    /* invariante Anteile von k2, k3, k4 und k5 in stored_k[i] anrechnen */
    hp_E = copy_vector( daten_basis->p_E);
    mult_vector_matrix( hp_E, daten_basis->Q_EE);
    del_vector(&(stored_k[2]));
    stored_k[2] = copy_vector( hp_E);
    del_vector(&(stored_k[3]));
    stored_k[3] = copy_vector( hp_E);
    del_vector(&(stored_k[4]));
    stored_k[4] = copy_vector( hp_E);
    del_vector(&(stored_k[5]));
    stored_k[5] = copy_vector( hp_E);
    del_vector(&hp_E);

    /* p_E(t+stepsize) in stored_k[0] anrechnen */
    del_vector(&(stored_k[0]));
    stored_k[0] = copy_vector( daten_basis->p_E);
    
    /* invariante Anteile von k2, k3, k4 und k5 in stored_k[i] vervollstaendigen */
    hp_E = copy_vector( stored_k[1]);
    mult_vector_matrix( hp_E, daten_basis->Q_EE);
    vector_add_mult( stored_k[2], hp_E, (a21*stepsize));
    vector_add_mult( stored_k[3], hp_E, (a31*stepsize));
    vector_add_mult( stored_k[4], hp_E, (a41*stepsize));
    vector_add_mult( stored_k[5], hp_E, (a51*stepsize));
    del_vector(&hp_E);

    clear_vector( stored_k[6]); /* in stored_k[6] wird invarianter Anteil von pdh_sum 
				   zwischengespeichert */

    for( i=0; i<ac_no_detT; i++) {
      hp_E = polynom_auswertung( c2_stepsize, int_fields[i]);
      mult_vector_matrix( hp_E, daten_basis->detMatrices[i].Delta_dE);
      vector_add_mult( stored_k[2], hp_E, 1.0);
      del_vector(&hp_E);

      hp_E = polynom_auswertung( c3_stepsize, int_fields[i]);
      mult_vector_matrix( hp_E, daten_basis->detMatrices[i].Delta_dE);
      vector_add_mult( stored_k[3], hp_E, 1.0);
      del_vector(&hp_E);

      hp_E = polynom_auswertung( c4_stepsize, int_fields[i]);
      mult_vector_matrix( hp_E, daten_basis->detMatrices[i].Delta_dE);
      vector_add_mult( stored_k[4], hp_E, 1.0);
      del_vector(&hp_E);

      hp_E = polynom_auswertung( c5_stepsize, int_fields[i]);
      mult_vector_matrix( hp_E, daten_basis->detMatrices[i].Delta_dE);
      vector_add_mult( stored_k[5], hp_E, 1.0);
      del_vector(&hp_E);

      hp_E = copy_vector( int_fields[i][0].aux_coefficients);
      mult_vector_matrix( hp_E, daten_basis->detMatrices[i].Delta_dE);
      vector_add_mult( stored_k[6], hp_E, 1.0);
      del_vector(&hp_E);
    };
  };
  
  /* k2, k3, k4, k5 und pdh_sum vervollstaendigen */
  k2 = copy_vector( stored_k[2]);
  k3 = copy_vector( stored_k[3]);
  k4 = copy_vector( stored_k[4]);
  k5 = copy_vector( stored_k[5]);
  pdh_sum = copy_vector( stored_k[6]);

  for( i=0; i<ac_no_detT; i++) {
    hp_E = polynom_auswertung( c2_stepsize, pdt_fields[i]);
    mult_vector_matrix( hp_E, daten_basis->detMatrices[i].Q_dE);
    vector_add_mult( k2, hp_E, 1.0);
    del_vector(&hp_E);

    hp_E = polynom_auswertung( c3_stepsize, pdt_fields[i]);
    mult_vector_matrix( hp_E, daten_basis->detMatrices[i].Q_dE);
    vector_add_mult( k3, hp_E, 1.0);
    del_vector(&hp_E);
    
    hp_E = polynom_auswertung( c4_stepsize, pdt_fields[i]);
    mult_vector_matrix( hp_E, daten_basis->detMatrices[i].Q_dE);
    vector_add_mult( k4, hp_E, 1.0);
    del_vector(&hp_E);

    hp_E = polynom_auswertung( c5_stepsize, pdt_fields[i]);
    mult_vector_matrix( hp_E, daten_basis->detMatrices[i].Q_dE);
    vector_add_mult( k5, hp_E, 1.0);
    del_vector(&hp_E);

    hp_E = copy_vector( pdt_fields[i][3].y_values);
    mult_vector_matrix( hp_E, daten_basis->detMatrices[i].Q_dE);
    vector_add_mult( pdh_sum, hp_E, 1.0);
    del_vector(&hp_E);
  };
  
  /* k2 ist vollstaendig; nacheinander werden jetzt k3, k4, k5 berechnet ! */
  /* dabei gehen die gerade vervollstaendigten Vorgaenger ein... */
  hp_E = copy_vector( k2);
  mult_vector_matrix( hp_E, daten_basis->Q_EE);
  vector_add_mult( k3, hp_E, (a32*stepsize));
  /* k3 ist vollstaendig */
  vector_add_mult( k4, hp_E, (a42*stepsize));
  vector_add_mult( k5, hp_E, (a52*stepsize));
  del_vector(&hp_E);
  hp_E = copy_vector( k3);
  mult_vector_matrix( hp_E, daten_basis->Q_EE);
  vector_add_mult( k4, hp_E, (a43*stepsize));
  /* k4 ist vollstaendig */
  vector_add_mult( k5, hp_E, (a53*stepsize));
  del_vector(&hp_E);
  hp_E = copy_vector( k4);
  mult_vector_matrix( hp_E, daten_basis->Q_EE);
  vector_add_mult( k5, hp_E, (a54*stepsize));
  /* k5 ist vollstaendig */
  del_vector(&hp_E);

  /* Berechnung von k6 ( in k2): */
  /* Anmerkung : eigentliches k2 geht nur in k3, k4, k5 und k6 ein und wird danach nicht mehr benoetigt */
  mult_vector_skalar( k2, (a62*stepsize));
  vector_add_mult( k2, stored_k[1], (a61*stepsize));
  vector_add_mult( k2, k3, (a63*stepsize));
  vector_add_mult( k2, k4, (a64*stepsize));
  vector_add_mult( k2, k5, (a65*stepsize)); 
  vector_add_mult( k2, stored_k[0], 1.0);
  mult_vector_matrix( k2, daten_basis->Q_EE);
  vector_add_mult( k2, pdh_sum, 1.0);
  /* k6 ( in k2 ) ist vollstaendig */

  /* Berechnung neuen p_E(t)-Wert, der gleichzeitig ein Anrechnen fuer k7 ist  */
  del_vector(&daten_basis->p_E);
  daten_basis->p_E = copy_vector( stored_k[0]);
  vector_add_mult( daten_basis->p_E, stored_k[1], (a71*stepsize));
  vector_add_mult( daten_basis->p_E, k3, (a73*stepsize));
  vector_add_mult( daten_basis->p_E, k4, (a74*stepsize));
  vector_add_mult( daten_basis->p_E, k5, (a75*stepsize)); 
  vector_add_mult( daten_basis->p_E, k2, (a76*stepsize));  
  /* in daten_basis->p_E stehen jetzt die Werte nach dem Schritt mit Schrittweite 'stepsize' */

  /* Zwischensumme fuer y - y-Dach ( in k2 ): */
  mult_vector_skalar( k2, delta6);
  vector_add_mult( k2, stored_k[1], delta1);
  vector_add_mult( k2, k3, delta3);
  vector_add_mult( k2, k4, delta4);
  vector_add_mult( k2, k5, delta5);

  /* Berechnung von k7 (in stored_k[7]) <- daten_basis->p_E */
  del_vector(&(stored_k[7]));
  stored_k[7] = copy_vector( daten_basis->p_E);
  mult_vector_matrix( stored_k[7], daten_basis->Q_EE);
  vector_add_mult( stored_k[7], pdh_sum, 1.0);
  /* k7 ist vollstaendig */

  /* Berechnung von y - y-Dach : */
  vector_add_mult( k2, stored_k[7], delta7);    /* experiment : k3 statt stored_k[7] */
  mult_vector_skalar( k2, stepsize);

  
  /*  Fehlerabschaetzung */
  rel_error_est( &error, daten_basis->p_E, k2);

  del_vector(&k2);
  del_vector(&k3);
  del_vector(&k4);
  del_vector(&k5);
  del_vector(&pdh_sum);

  return(error);
  
}
/*****************************************************************************************************/

