/*******************************************************************************
 Modul:              shared.c

 Purpose:            von allen Loesungskomponenten benutzte Prozeduren wie
                     Matrizenexponentiation mittels randomisation

 Project Director:   Reinhard German

 Programmer:         Joerg W. Mitzlaff

 History:            10.94 (Dipl.Arbeit)
 ******************************************************************************/

/* #define DEBUG_R */
#define BUF_SIZE    1000

#include <sys/param.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "Options.h"
#include "frontend.h"
#include "netio.h"
#include "shared.h"
#include "get_rrg.h"
#include "get_tmark.h"
#include "portab.h"

/* Import von main.c */
extern TransientOptions transientOptions;

/* Import von weighter.c */
extern void weighter();

/* Import von frontend.c */
extern NetDescr* net;

/* Import von get_rrg.c */
extern mark_t *marking;
extern int nmarking;       /* ist gleich component->no_markings; aber randomi() hat comp. struct nicht */

/* Import von get_tmark.c */
extern unsigned nplaces;
extern place_t *place;
extern unsigned ntoken;
extern int *tok_list;

/* Import von _ERC.c file */
extern double help[];

/* export nach _ERC.c file */
int digits;


/* prototyping */
double calculate_A_Matrix();

/* lokale variable */
static Result_Basis resbas;
static double *prob_list;
static int this_mark;
static int help_max;

/******************************************************************************
 Funktion:  act_marking()

 Rueckgabewert: return number of token per place per marking; this function is
                used by the result measures solver in the _ERC.c file (therefore non-static)

 *****************************************************************************/
int act_marking(p)
int p;
{
   register int i,addr;
 
   /* check bounds */
   if( p < 0 || p >= nplaces )
      return( 0 );
 
   /* find place number 'place' in place list */
   for( i = 0; i < nplaces && (p != place[i].number); i++ );
   if( i == nplaces )
   {
      fprintf(stderr,"Unable to find place %d\n", p);
      exit( 1 );
   }
 
   /* get number of token */
   addr = this_mark * nplaces + i;
 
   return( tok_list[addr] );
}
 


/******************************************************************************
 Funktion:  ggTeiler()

 Rueckgabewert: double groesster gemeinsammer Teiler von a und b 

 Wie mach ichs:	n0 = Max |a,b| n1 = Min |a,b| 
                nk = nk-2 - abgerundet(nk-2 / nk-1) * nk-1 
		ist nk == 0 dann war nk-1 der ggT !
 *****************************************************************************/
double ggTeiler(a,b)
double a;
double b;
{
  double n0; /* oder nk-2 */
  double n1; /* oder nk-1 */
  double nk;
 
  n0 =  (a > b ? a : b);
  n1 =  (a < b ? a : b);
  
  while( (nk = n0 - floor(n0/n1)*n1) > 1e-15 )  /* also Erg. nicht Null */
  {
    n0 = n1;
    n1 = nk;
  }
  return(n1);
}
/******************************************************************************/
double calculate_A_Matrix(Q)   /* veraendert Q zu A und gibt max_q'ii zurueck */
/******************************************************************************/
sp_matrix *Q;
{
  double maxq_ii;
  sp_entry *nep;   /* next entry pointer */
  sp_vector *nvap; /* next vector pointer */
  int i;

  /* max |q(i,i)| finden */
  maxq_ii = 0.0;
  nvap = Q -> rows;
  while(nvap)        /* gehe ueber alle zeilen und spalten */
  {
    nep = nvap -> entries;
    while(nep)
    {
      if( nvap->num == nep->num )
	if( fabs(nep->val) > maxq_ii )
	  maxq_ii = fabs(nep->val);

      nep = nep -> next;
    }
    nvap = nvap -> next;
  }

  maxq_ii *= 1.02;

  /* A = Q / qmax_ii */
  nvap = Q -> rows;
  while(nvap)        /* gehe ueber alle zeilen und spalten */
  {
    nep = nvap -> entries;
    while(nep)
    {
      nep->val = nep->val / maxq_ii;

      nep = nep -> next;
    }
    nvap = nvap -> next;
  }
  
  /* A Matrix + Identitaetsmatrix */
  for(i=0;i<nmarking;i++)
    add_entry_to_matrix(Q, i, i, 1.0);

  return maxq_ii;
}
/*************************************************************************************/
PrepeareRandomisation *prepeare_rand(Q,time)
/*************************************************************************************/
sp_matrix *Q;
double time;
{
  PrepeareRandomisation *prep;
  double maxq_ii;

  prep = (PrepeareRandomisation *)emalloc(sizeof(PrepeareRandomisation));

  /* A Matrix aufstellen */

  prep -> A = copy_matrix(Q);

  maxq_ii = calculate_A_Matrix(prep -> A);

  if(maxq_ii==0.0) /* Q war Null Matrix (0) */ 
    return(prep);   /* da e^0*x = 0 */

  call_weighter( time * maxq_ii, &(prep->L), &(prep->R), &(prep->Betas));
  printf ("Truncation:\tleft = %u\tright = %u\n", prep->L, prep->R);
  
  return(prep);
}
/*******************************************************************************/
double adapt_precision( start_vec)
/* passt die absolute Genauigkeit des Randomisierungsverfahrens derart an, dass alle 
   Ergebnisse mind. relativ genau mit transientOptions.precision berechnet werden */

sp_vector *start_vec;
{
  int i;
  sp_entry *el_vec;
  double new_precision = 0.0;
  double el_min;
  double comp_sum = 0.0;

  el_vec = start_vec->entries;

  if (start_vec->no_entries > 0)
    el_min = start_vec->entries->val;
  else
    el_min = 0.0;

  for( i=0;i<start_vec->no_entries;i++) {
    if ((el_vec->val < el_min)&&(el_vec->val > 0.0)) 
      el_min = el_vec->val;
    comp_sum += el_vec->val; /* immer positive Werte ! */
    el_vec = el_vec->next;
  };
#ifdef DEBUG_R
  printf("Komponentensumme: %.17f \n",comp_sum);
  printf("Minimum: %.25f \n",el_min);
  printf("Eintrage: %d \n", start_vec->no_entries);
#endif

  if (comp_sum > MAX_ABS_PREC) {
    if (el_min > MAX_ABS_PREC)
      new_precision = (transientOptions.precision * el_min)/comp_sum;
    else
      new_precision = (transientOptions.precision * MAX_ABS_PREC)/comp_sum;
  }
  else {

    if (start_vec->no_entries == 0)
      new_precision = 1.0;
    else
      new_precision = MAX_ABS_PREC;
  };
#ifdef DEBUG_R
  printf("neue Genauigkeit: %.25f \n",new_precision);
#endif
  return(new_precision);
}
/**********************************************************************************/
int ad_prec_ifnec( pprec, start_vec)
/* passt die absolute Genauigkeit des Randomisierungsverfahrens derart an, dass alle 
   Ergebnisse mind. relativ genau mit transientOptions.precision berechnet werden */
/* eingehende Genauigkeit wird nur ver"andert, falls sie ausserhalb folgender 
   Toleranz liegt : [*pprec - 10%, *pprec + 50%] -> dann R"uckgabewert 1 */
/* speziell f"ur Randomisierung aufeinanderfolgender step_inbetween-Elemente */

double *pprec;
sp_vector *start_vec;
{
  int i;
  sp_entry *el_vec;
  double new_precision = 0.0;
  double el_min;
  double comp_sum = 0.0;

  el_vec = start_vec->entries;

  if (start_vec->no_entries > 0)
    el_min = start_vec->entries->val;
  else
    el_min = 0.0;

  for( i=0;i<start_vec->no_entries;i++) {
    if ((el_vec->val < el_min)&&(el_vec->val > 0.0)) 
      el_min = el_vec->val;
    comp_sum += el_vec->val; /* immer positive Werte ! */
    el_vec = el_vec->next;
  };
  if (comp_sum > MAX_ABS_PREC) {
    if (el_min > MAX_ABS_PREC)
      new_precision = (transientOptions.precision * el_min)/comp_sum;
    else
      new_precision = (transientOptions.precision * MAX_ABS_PREC)/comp_sum;
  }
  else {
    if (start_vec->no_entries == 0)
      new_precision = 1.0;
    else
      new_precision = MAX_ABS_PREC;
  };
  
  /* Toleranzabfrage */
  if ((new_precision < (*pprec - (0.1*(*pprec))))||(new_precision > (*pprec + (0.5*(*pprec))))) {
    *pprec = new_precision;
    return(1);
  }
  else {
    /* *pprec bleibt unver"andert */ 
    return(0);
  };
}
/**********************************************************************************/
sp_vector *ranforfield( start, with_data)

sp_vector *start;
ConcernsRandomisation *with_data;
{
  int k;
  sp_vector *temp_vec;
  sp_vector *res_vec;

  res_vec = new_vector();
  temp_vec = copy_vector( start);

  for(k=0;k <= with_data->R;k++)
    {
      if(k >= with_data->L )
	vector_add_mult( res_vec, temp_vec, with_data->Betas[ k - with_data->L ] );

      mult_vector_matrix( temp_vec,  with_data->A); 
    };
  del_vector(&(temp_vec));
  return( res_vec);
}
/**********************************************************************************/
void randomize_quickly( p_t, with_data)
/* geht auch fuer area-Vektoren, die ueber vec->next miteinander verbunden sind ! */

sp_vector *p_t;
ConcernsRandomisation *with_data;
{
  int k;
  sp_vector *temp_vec;

  temp_vec = new_vector();
  temp_vec->no_entries = p_t->no_entries;
  temp_vec->entries = p_t->entries;

  /* quasi clear_vector() : */
  p_t->entries = NULL;
  p_t->no_entries = 0;

  for(k=0;k <= with_data->R;k++)
    {
      if(k >= with_data->L )
	vector_add_mult( p_t, temp_vec, with_data->Betas[ k - with_data->L ] );

      mult_vector_matrix( temp_vec,  with_data->A); 
    };
  del_vector(&(temp_vec));
}

/******************************************************************************
 Funktion:  randomi()

 Parameter: sp_matrix *Q ist nicht NULL, kann aber 0 Eintraege haben.
            double transient_time

 Rueckgabewert: Neue Matrix sp_matrix

 Zweck:	    Durchfuehren des Randomization-Verfahrens: phi = e^(Q*t)

 History:   angelehnt an EDSPNAnalysis/src/erand.c 
            und DSPNAnalysis/src/rand_main.c
 *****************************************************************************/
sp_matrix *randomi(Q,transient_time)
sp_matrix *Q;
double transient_time;
{
  double maxq_ii;  /* maximales,absolutes Diagonalelement in Q */
  sp_matrix *A;    /* normalisierte Q Matrix */
  sp_matrix *A_hoch_k;  /* rekursive Berechnung mit A */
  sp_matrix *Phi;  /* Ergebnis Aufsummierte A_hoch_k also e^Q*t */
  int L,R;
  double *beta;
  int k;
  double esum;
  sp_vector *nvap; /* next vaector pointer */

#ifdef DEBUG_R
  printf("randomi: *** Q ");
  show_matrix(Q);
#endif

  Phi = new_matrix();


  A = copy_matrix(Q);

  maxq_ii = calculate_A_Matrix(A);

  if(maxq_ii==0.0) /* Q war Null Matrix (0) */
  { 
    for(k=0;k<nmarking;k++)
       add_entry_to_matrix(Phi, k, k, 1.0);
        
    return(Phi);   /* da e^0*x = Einheitsmatrix */
  };

#ifdef DEBUG_R
  printf("randomi: maxq_ii = %lf *** A ",maxq_ii);
  show_matrix(A);
#endif

  printf("Randomisation: ...");
  call_weighter( maxq_ii * transient_time, &L, &R, &beta);
  printf(" Truncation:\tleft = %u\tright = %u\t ...", L, R);
  fflush(stdout);

  /* spannendste Teil */
  esum = 0.0;
  for (k = 0; k <= R; k++)
  {
    /* neues A_hoch_k berechnen */
    if(k == 0)   /* also beim ersten mal */
    {
      /* berechne A^0 * also Identitaetsmatrix von A */
      A_hoch_k = new_matrix();
      nvap = A -> rows;
      while(nvap)  /* gehe durch alle Zeilen von A und fuer jede Zeile schreibe 1 in A_hoch_k'ii */
      {
	add_entry_to_matrix(A_hoch_k, nvap -> num, nvap -> num, 1.0);

	nvap = nvap -> next;
      }  
    }
    else
    {
      /* A^k = A^k-1 * A */
      mult_matrix_matrix(A_hoch_k, A);
    }
   
    if(k>=L)   /* genug der Vorbereitung: es kann losgehen */
    {

#ifdef DEBUG
    printf("*** ( A hoch %d ) * beta[%d]=%lf", k, k, beta[k-L]);
    show_matrix(A_hoch_k);
#endif

      /* betas multiplizieren und Phi aufsummieren: Phi(k) = Phi(k-1) + A_hoch_k * beta[k] */
      matrix_add_mult(Phi,A_hoch_k,beta[k-L]);

      /* frueher abbrechen moeglich? */
      esum += beta[k-L];
      if(esum > (1.0 -  transientOptions.precision))
	break;
    }

#ifdef DEBUG
    printf("*** Phi %d ",k);
    show_matrix(Phi);
#endif

  }
  printf("used k = %d\n",k);

  /* free betas */

  clear_matrix(A);
  clear_matrix(A_hoch_k);
  return(Phi);
}
/******************************************************************************
* open_solve: oeffnet .tempcurfes file und eventuell x fenster
 ******************************************************************************/
FILE *open_solve(tnethome,modelpath,netname,num,maxX)
char *tnethome;
char *modelpath;
char *netname;
int num;
double maxX;
{
  FILE *fcurves;
  char curves_file[MAXPATHLEN];
  int i,j;
  char command[2*MAXPATHLEN];
  double p;
  FILE *ferc;
  char erc_file[MAXPATHLEN];
  char buf[BUF_SIZE];
  
  /* determine number of digits */
  for( digits = 0, p = transientOptions.precision; p < 1.0; digits++, p *= 10.0 )
    ;


  /* File zum schreiben oeffnen */
  sprintf(curves_file,"%s/%s.tempcurves",modelpath,netname);
  if (!(fcurves=fopen(curves_file,"w"))) 
  {
    fprintf(stderr,"Unable to open file '%s' for writing.\n", curves_file);
    exit(-1);
  }

  resbas.maxY = (double *)emalloc(sizeof(double)*(net -> num_rewards));
  resbas.minY = (double *)emalloc(sizeof(double)*(net -> num_rewards));

  /* header schreiben */
  fprintf(fcurves,"# file %s containing solution of TransSolve.\n#\n",curves_file);

  fprintf(fcurves,"# number of x values: a\n");
  fprintf(fcurves,"%d\n", num);

  fprintf(fcurves,"# number of defined reward measures: b\n");
  fprintf(fcurves,"%d\n", net -> num_rewards);

  fprintf(fcurves,"# list of reward measure names:\n");
  for(i=0;i<net -> num_rewards;i++)
    fprintf(fcurves,"%s ", net->reward[i].name);
  fprintf(fcurves,"\n");

  fprintf(fcurves,"# last x value:\n");
  fprintf(fcurves,"%.*g\n", digits, maxX);

  fprintf(fcurves,"# maximum y value:\n");
  for(i=0;i<net -> num_rewards;i++)
  {
    resbas.maxY[i] = 0.0;
  }
  fprintf(fcurves,"\n");

  fprintf(fcurves,"# minimum y value:\n");
  for(i=0;i<net -> num_rewards;i++)
  {
    resbas.minY[i] = 1.0e300;
  }
  fprintf(fcurves,"\n");

  fprintf(fcurves,"# in case of an experiment, number of runs: c\n");
  fprintf(fcurves,"%d\n", 1 );    /* wird vorerst nicht unterstuetzt! */

  fprintf(fcurves,"# results: c times a x b matrices\n");
  fflush(fcurves);

  /* start Backend */
  if((transientOptions.graphics_desired) && (transientOptions.output_data!=Point) )
  {
    /* Renate, 18.09.97 */
    fflush(stdout); /* vor fork Puffer leeren, sonst spaeter doppelte Ausgaben.*/

    if(fork()==0)
    {  
      sprintf(command,"%s/TransientSolution/bin/Backend %s.tempcurves -geometry 600x300+200+100", tnethome, netname);
      system(command);    
      exit(0);
    }
  }

  /* allocate probabilities list */
  if( (prob_list = (double *)malloc(nmarking * sizeof(double))) == NULL )
  {
    fprintf(stderr,"Unable to allocate probabilities list.\n");
    exit( 1 );
  }

  /* um generiertes _ERC.c benutzen zu koennen, muss net.place.number angepasst werden */
  for(i=0;i<nplaces;i++)
  {
    for( j = 0; j < nplaces && strcmp(net->place[i].name, place[j].name) != 0; j++ )
      ;
    if( j == nplaces )
    {
      fprintf(stderr,"Place %s not in .tmark file\n", net->place[i].name);
      exit( 1 );
    }

    /* we have found place 'net->place[i].name' at position 'j'; since the order of */
    /* appearance in .TN and .tmark is not neccessarily the same, we now */
    /* record the index from the .TN file in the place description. */
    place[j].number = i;
  }
  
  /* _ERC.c benutzt help array, und ich muss dessen groesse wissen, um es in jedem Durchgang loeschen zu koennen */
  sprintf(erc_file,"%s/%s_ERC.c",modelpath,netname);
  if (!(ferc=fopen(erc_file,"r"))) 
  {
    fprintf(stderr,"Unable to open file '%s'.\n", erc_file);
    exit(-1);
  }
  do
  {
    fgets(buf,BUF_SIZE,ferc);
  } while( sscanf(buf,"double help[%d];",&help_max)!=1 );

  return(fcurves);
}
/****************************************************************************************/
void reset_digits()
/* nur fuer adaptiven Algorithmus mit relativer Genauigkeit */
{
  digits = 15;
}

/******************************************************************************
 * update_solve: erzeugt prob_list; ruft fkt. der _ERC.c Komponente auf, laesst
 *               also .RESULTS file erzeugen und kopiert diese ins .tempcurves file
 ******************************************************************************/
void update_solve(tnethome,modelpath,netname,ftmp,no_markings,res_vec,t,maxX)
char *tnethome;
char *modelpath;
char *netname;
FILE *ftmp;
unsigned no_markings;
sp_vector *res_vec;
double t;
double maxX;
{
   char res_file[MAXPATHLEN];
   FILE *res_fp;
   int i;
   sp_vector *vec;
   sp_entry *nep;
   char dummy[256]; /* reward measure name */
   double d;
   char command[2*MAXPATHLEN];

   if(transientOptions.output_data==Point || !transientOptions.graphics_desired)
   {
     /* print status meldung alle 10% */
     if(((int)(100.0*t/maxX)) % 10 == 0)
     {
       printf("status: %d %% \n",(int)(100.0*t/maxX));
     }
   }

   if(transientOptions.output_data==Point && t<maxX)
     return;

   /* prob_list erneuern */
   for(i=0, nep=res_vec->entries; i< no_markings; i++)
   {
     if(nep)     /* noch Eintraege */
     {
       if(nep->num>i)   /* i. Eintrag gibt es nicht */
       {
	 prob_list[i] = 0.0;
       }
       else             /* kann nur noch nep->num == i sein */
       {
	 prob_list[i] = nep->val;
	 nep = nep->next;
       }
     }
     else
     {
       prob_list[i] = 0.0;
     }
   }

   /* open .RESULT File for writing */
   sprintf(res_file,"%s/%s.RESULTS",modelpath,netname);
   if( (res_fp = fopen(res_file,"w")) == NULL )
   {
     fprintf(stderr,"Unable to open file '%s' for writing.\n", res_file);
     exit( 1 );
   }

   /* help array auf Null stetzen */
   for(i=0;i<help_max;i++)
     help[i] = 0.0;

   /* _ERC fkt.  aufrufen (war frueher compute_results() aus convert_results.c) */
   for( this_mark = 0; this_mark < nmarking; this_mark++ )
      sum_prob(prob_list[this_mark]);

   /* write results to .RESULTS */
   calc_results(res_fp,digits);

   /* close .RESULTS */
   fclose(res_fp);

   /* copy results to .tempcurves */
   /* open .RESULT File for reading */
   sprintf(res_file,"%s/%s.RESULTS",modelpath,netname);
   if( (res_fp = fopen(res_file,"r")) == NULL )
   {
     fprintf(stderr,"Unable to open file '%s' for reading.\n", res_file);
     exit( 1 );
   }

   /* print status meldung */
   fprintf(ftmp,"#status: %d %% \n",(int)(100.0*t/maxX));

   fprintf(ftmp,"%.*g ", digits, t);  
   for(i=0; i<net -> num_rewards; i++)
   {
     fscanf(res_fp,"%s = %lf",dummy,&d);
     if(resbas.maxY[i]<d)
       resbas.maxY[i] = d;
     if(resbas.minY[i]>d)
       resbas.minY[i] = d;
     fprintf(ftmp,"%.*g ", digits, d);
   }
   fprintf(ftmp,"\n");
   fflush(ftmp);
   
   fclose(res_fp);

   fflush(ftmp);
}

 
/******************************************************************************
 * close_solve: schreibt .curves file mit neuem minY und maxY 
 ******************************************************************************/
void close_solve(modelpath,netname,ftempc)
char *modelpath;
char *netname;
FILE *ftempc;
{
  FILE *fcurves;
  char file_name[MAXPATHLEN];
  char buf[BUF_SIZE];
  int i;

  /* free allocated memory */
  free(prob_list);

  fclose(ftempc);

  sprintf(file_name,"%s/%s.tempcurves",modelpath,netname);
  if (!(ftempc=fopen(file_name,"r"))) 
  {
    fprintf(stderr,"Unable to open file '%s' for reading.\n", file_name);
    exit(-1);
  }

  sprintf(file_name,"%s/%s.curves",modelpath,netname);
  if (!(fcurves=fopen(file_name,"w"))) 
  {
    fprintf(stderr,"Unable to open file '%s' for writing.\n", file_name);
    exit(-1);
  }

  fgets(buf,BUF_SIZE,ftempc);
  fprintf(fcurves,"# file %s containing solution of TransSolve.\n",file_name);

  /* dirty (besser: zeile suchen mit strcmp) */
  for(i=0;i<10;i++)
  {
    fgets(buf,BUF_SIZE,ftempc);
    fputs(buf,fcurves);
  }

  fgets(buf,BUF_SIZE,ftempc);
  for(i=0;i<net -> num_rewards;i++)
    fprintf(fcurves,"%.*g ", digits, resbas.maxY[i]);
  fprintf(fcurves,"\n");

  fgets(buf,BUF_SIZE,ftempc);
  fputs(buf,fcurves);

  fgets(buf,BUF_SIZE,ftempc);
  for(i=0;i<net -> num_rewards;i++)
    fprintf(fcurves,"%.*g ", digits, resbas.minY[i]);
  fprintf(fcurves,"\n");

  fgets(buf,BUF_SIZE,ftempc);
  fputs(buf,fcurves);
  fgets(buf,BUF_SIZE,ftempc);
  fputs(buf,fcurves);
  fgets(buf,BUF_SIZE,ftempc);
  fputs(buf,fcurves);

  while(!feof(ftempc))
  {
    fgets(buf,BUF_SIZE,ftempc);

    if( (!feof(ftempc)) && (buf[0]!='#') )
      fputs(buf,fcurves);
  }
  
  fclose(ftempc);
  fclose(fcurves);
}
/******************************************************************************
 * close_adaptsolve: schreibt .curves file mit neuem minY und maxY und #x-values 
 ******************************************************************************/
void close_adaptsolve(modelpath,netname,ftempc, num)
char *modelpath;
char *netname;
FILE *ftempc;
int num;
{
  FILE *fcurves;
  char file_name[MAXPATHLEN];
  char buf[BUF_SIZE];
  int i;

  /* free allocated memory */
  free(prob_list);

  fclose(ftempc);

  sprintf(file_name,"%s/%s.tempcurves",modelpath,netname);
  if (!(ftempc=fopen(file_name,"r"))) 
  {
    fprintf(stderr,"Unable to open file '%s' for reading.\n", file_name);
    exit(-1);
  }

  sprintf(file_name,"%s/%s.curves",modelpath,netname);
  if (!(fcurves=fopen(file_name,"w"))) 
  {
    fprintf(stderr,"Unable to open file '%s' for writing.\n", file_name);
    exit(-1);
  }

  fgets(buf,BUF_SIZE,ftempc);
  fprintf(fcurves,"# file %s containing solution of TransSolve.\n",file_name);

  /* dirty (besser: zeile suchen mit strcmp) */
  for(i=0;i<2;i++)
  {
    fgets(buf,BUF_SIZE,ftempc);
    fputs(buf,fcurves);
  }
  fgets(buf, BUF_SIZE, ftempc);
  fprintf(fcurves,"%d\n", num);
  for(i=0;i<7;i++)
  {
    fgets(buf,BUF_SIZE,ftempc);
    fputs(buf,fcurves);
  }
  
  fgets(buf,BUF_SIZE,ftempc);
  for(i=0;i<net -> num_rewards;i++)
    fprintf(fcurves,"%.*g ", digits, resbas.maxY[i]);
  fprintf(fcurves,"\n");

  fgets(buf,BUF_SIZE,ftempc);
  fputs(buf,fcurves);

  fgets(buf,BUF_SIZE,ftempc);
  for(i=0;i<net -> num_rewards;i++)
    fprintf(fcurves,"%.*g ", digits, resbas.minY[i]);
  fprintf(fcurves,"\n");

  fgets(buf,BUF_SIZE,ftempc);
  fputs(buf,fcurves);
  fgets(buf,BUF_SIZE,ftempc);
  fputs(buf,fcurves);
  fgets(buf,BUF_SIZE,ftempc);
  fputs(buf,fcurves);

  while(!feof(ftempc))
  {
    fgets(buf,BUF_SIZE,ftempc);

    if( (!feof(ftempc)) && (buf[0]!='#') )
      fputs(buf,fcurves);
  }
  
  fclose(ftempc);
  fclose(fcurves);
}
