/*******************************************************************************
Modul:              special.c

Purpose:            special case Loesungsverfahren

Project Director:   Reinhard German

Programmer:         Joerg W. Mitzlaff

History:            10.94 (Dipl.Arbeit)
*******************************************************************************/

/* #define DEBUG */

#include <stdio.h>
#include <math.h>
#include "get_rrg.h"          /* mark_t definieren */
#include "frontend.h"
#include "Options.h"
#include "shared.h"

/* Import aus main.c */
extern TransientOptions transientOptions;
extern FILE *kosten_file;

/* Import aus get_rrg.c */
extern mark_t *marking;

/* Import aus shared.c */
extern double ggTeiler();
extern sp_matrix *randomi();
extern double calculate_A_Matrix();
extern FILE *open_solve();
extern void update_solve();
extern void close_solve();

/* local functions */
static void reduce_deltas();


/******************************************************************************/
void special_case(tnethome,modelpath,netname,component)
/******************************************************************************/
char *tnethome;
char *modelpath;
char *netname;
solve_basis *component;
{
  sp_matrix *temp_matrix;
  
  FILE *ftmp;
  double t;
  int i,j;
  int ndetT;
  int numDatas;
  int numIntervall;        
  double maxX;
  double h;                /* ggT von allen detT sowie method_stepsize */
  int *steps_in_tau;       /* Anzahl der Messpunkte jeder detT */

#ifdef DEBUG1
  for( i=0 ; i<component->no_detT;i++)
  {
    for( j=0 ; j<component->no_detT;j++)
    {
      printf("\n Delta_dc Matrizen d=%d c=%d\n", i,j);
      show_matrix(component->detMatrices[i].Delta_dc[j]);
    } 
  }
#endif


  reduce_deltas(component);


#ifdef DEBUG2
  for( i=0 ; i<component->no_detT;i++)
  {
    printf("\n Delta_dc Matrizen d=%d c=ist hier egal\n", i);
    show_matrix(component->detMatrices[i].Delta_dc[0]);
  }
#endif

#ifdef DEBUG
  for( i=0 ; i<component->no_detT;i++)
  {
    printf("\n Q_d Matrizen d=%d \n", i);
    show_matrix(component->detMatrices[i].Q_d);
  }
#endif


  ndetT = component -> no_detT;

  /* KGV von allen tau und h und event. trans. time finden */
  if(transientOptions.output_data==Point)
    h = ggTeiler(transientOptions.transient_time,transientOptions.stepsize_result);
  else
    h = transientOptions.stepsize_result;

  for(i=0;i<ndetT;i++)      
    h = ggTeiler(h, component->detMatrices[i].det_firing_time);

  printf("stepsize h %s to %g.\n",
	 (transientOptions.stepsize_result!=h ? "has changed" : "was set"),
	 h);


  steps_in_tau = (int *)emalloc(sizeof(int) * ndetT);
  for(i=0;i<ndetT;i++)
    steps_in_tau[i] = (int)(component->detMatrices[i].det_firing_time / h);   /* muss ganze Zahl rauskommen, da h ggT ist */


#ifdef KOSTEN
  log_kosten(2);
#endif

  /* ersetze Q Matrizen durch exp_Q Matrizen */
  for(i=0;i<ndetT;i++)
  {

#ifdef KOSTEN
    fprintf(kosten_file, "num_Q_d'%d %d ", i, count_elements(component->detMatrices[i].Q_d) );
#endif

    temp_matrix = randomi(component->detMatrices[i].Q_d, h);
    del_matrix( &(component->detMatrices[i].Q_d) );
    component->detMatrices[i].Q_d = temp_matrix;

#ifdef KOSTEN
    fprintf(kosten_file, "num_e^Q_d'%d: %d ", i, count_elements(component->detMatrices[i].Q_d) );
#endif

  }

#ifdef KOSTEN
  log_kosten(1);
#endif

#ifdef DEBUG
  for( i=0 ; i<component->no_detT;i++)
  {
    printf("\n e^(Q_d * h) Matrizen d=%d \n", i);
    show_matrix(component->detMatrices[i].Q_d);
  }
#endif

  maxX = 0.0;
  numDatas = 1;     /* t=0 ist auch ein messwert */
  while(maxX < transientOptions.transient_time)
  {
    i=0;
    while( (maxX < transientOptions.transient_time) && (i<ndetT) )
    {
      j=0;
      while( (maxX < transientOptions.transient_time) && (j<steps_in_tau[i]) )
      {
	numDatas++;
	maxX += h;
	j++;
      }
      if(j==steps_in_tau[i])
	numDatas++;
      i++;
    }
  }
  
  ftmp = open_solve(tnethome,modelpath,netname,numDatas,maxX);

  /* endloses vektor x matrix */
  t = 0.0;
  update_solve(tnethome,modelpath,netname,ftmp,component->no_markings,component->p_d[0],t,maxX);
  while(t<maxX)
  {
    i=0;
    while( (t<maxX) && (i<ndetT) )
    {
      j=0;
      while( (t<maxX) && (j<steps_in_tau[i]) )
      {
	t += h;
	mult_vector_matrix(component->p_d[0],component->detMatrices[i].Q_d);
	update_solve(tnethome,modelpath,netname,ftmp,component->no_markings,component->p_d[0],t,maxX);

#ifdef DEBUG
	printf("p(%lf):\n",t);
	show_vector( component->p_d[0] );
#endif

	j++;
      }
      if(j == steps_in_tau[i])
      {
	mult_vector_matrix(component->p_d[0],component->detMatrices[i].Delta_dc[0]);
	update_solve(tnethome,modelpath,netname,ftmp,component->no_markings,component->p_d[0],t,maxX);

#ifdef DEBUG
	printf("p(%lf):\n",t);
	show_vector( component->p_d[0] );
#endif

      }
      i++;
    }
  }
  close_solve(modelpath,netname,ftmp);
}



/******************************************************************************/
void case_ii(tnethome,modelpath,netname,component)
/******************************************************************************/
char *tnethome;
char *modelpath;
char *netname;
solve_basis *component;
{
  sp_matrix *temp_matrix;
  
  FILE *ftmp;
  double t;
  int f,g,i,j,k;
  int ndetT;
  int numDatas;
  int numIntervall;        
  double maxX;
  double h;                /* ggT von allen detT sowie method_stepsize */
  int *steps_in_tau;       /* Anzahl der Messpunkte jeder detT */
  int *cluster_in_tau;     /* wieviele cluster brauche ich fuer jede detT ? */
  int max_cluster_in_tau;
  Randomisation_Over_Time_Line *rotili;
  int minL = 100000;

  reduce_deltas(component);

  ndetT = component -> no_detT;

  /* KGV von allen tau und h und event. trans. time finden */
  if(transientOptions.output_data==Point)
    h = ggTeiler(transientOptions.transient_time,transientOptions.stepsize_result);
  else
    h = transientOptions.stepsize_result;

  for(i=0;i<ndetT;i++)      
    h = ggTeiler(h, component->detMatrices[i].det_firing_time);

  printf("stepsize h %s to %g. %g\n",
	 ((transientOptions.stepsize_result-h)>1e-12 ? "has changed" : "was set"),
	 h,transientOptions.stepsize_result);
  printf("cluster size  was set to %d.\n", transientOptions.cluster_size);


  steps_in_tau = (int *)emalloc(sizeof(int) * ndetT);
  for(i=0;i<ndetT;i++)
    steps_in_tau[i] = (int)(component->detMatrices[i].det_firing_time / h);   /* muss ganze Zahl rauskommen, da h ggT ist */

  cluster_in_tau = (int *)emalloc(sizeof(int) * ndetT);
  max_cluster_in_tau = 0;
  for(i=0;i<ndetT;i++)
  {
    cluster_in_tau[i] = (steps_in_tau[i] / transientOptions.cluster_size) + 1 ;
    if(cluster_in_tau[i] > max_cluster_in_tau)
      max_cluster_in_tau = cluster_in_tau[i];
  }
    
  /* precision an cluster anzahl anpassen */
  transientOptions.precision /= max_cluster_in_tau;
  if(max_cluster_in_tau>1)
    printf("randomisation precision has changed to %g.\n",transientOptions.precision);

  maxX = 0.0;
  numDatas = 1;     /* t=0 ist auch ein messwert */
  
  while(maxX < (transientOptions.transient_time))
  {
    i=0;
    while( (maxX < (transientOptions.transient_time)) && (i<ndetT) )
    {
      j=0;
      while( (maxX < (transientOptions.transient_time)) && (j<steps_in_tau[i]) )
      {
	numDatas++;
	maxX += h;
	j++;
      }
      if(j==steps_in_tau[i])
	numDatas++;
      i++;
    }
  }
  maxX = transientOptions.transient_time;

  rotili =  (Randomisation_Over_Time_Line *)emalloc( sizeof(Randomisation_Over_Time_Line) * ndetT );

  for(i=0;i<ndetT;i++)
  {
    /* ersetze Q_d Matrix durch A Matrix */
    rotili[i].qmax_ii = calculate_A_Matrix(component->detMatrices[i].Q_d);

#ifdef DEBUG
    printf("\n A Matrizen d=%d \n", i);
    show_matrix(component->detMatrices[i].Q_d);
#endif

    /* alloziere Speicher */
    rotili[i].Betas = (double **)emalloc(sizeof(double *) * transientOptions.cluster_size);
    rotili[i].R = (int *)emalloc(sizeof(int) * transientOptions.cluster_size);
    rotili[i].L = (int *)emalloc(sizeof(int) * transientOptions.cluster_size);
     
    /* Berechne Betas */
    for(j=0;j<transientOptions.cluster_size;j++) /* Schritte eines Clusters */
    {
      call_weighter(rotili[i].qmax_ii * (j+1) * h, &(rotili[i].L[j]), &(rotili[i].R[j]), &(rotili[i].Betas[j]));
      if(rotili[i].L[j] < minL)
	minL = rotili[i].L[j];
    }


    /* alloziere pi'k Elemente Speicher */
    /* kleinste linke Schranke ist meisst 0, es wird also rotili.L[n-1] == 0 angenommen */
    /* + 1, da von o bis k summiert werden muss */
    rotili[i].pi_summand = (sp_vectors *) emalloc ( sizeof(sp_vectors) * 
						    (rotili[i].R[transientOptions.cluster_size-1] + 1) 
						  );
    for(j=0; j <= rotili[i].R[ transientOptions.cluster_size - 1 ]; j++)
      rotili[i].pi_summand[j] = new_vector();
  }

#ifdef KOSTEN
  for(i=0;i<ndetT;i++)
  {
    fprintf(kosten_file, "L'%d: %d R'%d: %d ", i, minL, i, rotili[i].R[transientOptions.cluster_size-1] );
  }
#endif

  /* endloses vektor x matrix */
  ftmp = open_solve(tnethome,modelpath,netname,numDatas,maxX);
  t = 0.0;
  update_solve(tnethome, modelpath, netname, ftmp, component->no_markings, component->p_d[0], t, maxX);
  while(t<maxX)
  {
    f=0;
    while((t<maxX) && (f<ndetT))
    {
      g = 0;
      j = transientOptions.cluster_size;
      while((t<maxX) && (g<steps_in_tau[f]))
      {
	/* mult_vector_matrix(component->p_d[0],component->detMatrices[i].Q_d); */

	if(j == transientOptions.cluster_size)  /* neu aufsetzen ??? --> neue pi'k berechnen */
	{

#ifdef KOSTEN
  log_kosten(2);
#endif

	  del_vector( &(rotili[f].pi_summand[0]) );
	  rotili[f].pi_summand[0] = copy_vector(component->p_d[0]);   /* pi'0 = p(0) * I = p(0) */
	  for(k=1;k <= rotili[f].R[transientOptions.cluster_size-1];k++)  /* pi'k = pi'(k-1) * A */
	  {
	    del_vector( &(rotili[f].pi_summand[k]) );
	    rotili[f].pi_summand[k] = copy_vector( rotili[f].pi_summand[k-1] );
	    mult_vector_matrix( rotili[f].pi_summand[k], component->detMatrices[f].Q_d);
	  }
	  j=0;

#ifdef KOSTEN
  log_kosten(1);
#endif

	}
	  
	clear_vector(component->p_d[0]);
	for(k=rotili[f].L[j];k <= rotili[f].R[j];k++)
	{
	  /* aufsummieren */ 
	  vector_add_mult( component->p_d[0], rotili[f].pi_summand[k], rotili[f].Betas[j][ k-rotili[f].L[j] ] );
	}

	t += h;
	update_solve(tnethome, modelpath, netname, ftmp, component->no_markings, component->p_d[0], t, maxX);

#ifdef DEBUG
	printf("p(%lf):\n",t);
	show_vector( component->p_d[0] );
#endif

	j++;
	g++;
      }
	
      mult_vector_matrix(component->p_d[0],component->detMatrices[f].Delta_dc[0]);
      update_solve(tnethome,modelpath,netname,ftmp,component->no_markings,component->p_d[0],t,maxX);

#ifdef DEBUG
	printf("p(%lf):\n",t);
	show_vector( component->p_d[0] );
#endif

      f++;
    }
  }
  update_solve(tnethome,modelpath,netname,ftmp,component->no_markings,component->p_d[0],t,maxX);
  close_solve(modelpath,netname,ftmp);
}


/******************************************************************************/
int particular_sequence(component)
/* beachte den kleinen logischen Fehler, dass hier in einer Sequenz jede      */
/* deterministische Transition hoechstens einmal auftreten darf !!!           */
/* Andere periodische Faelle sind jedoch denkbar, dann Q^d-Matrizen checken!  */
/* Diese Faelle koennen jedoch mit dem adaptiven Algorithmus behandelt werden */
/******************************************************************************/
solve_basis *component;
{
  int anything_set;
  int num_S_d;
  int init_S_d;
  int prev_S_d;
  int akt_S_d;
  sp_vector *nvp;
  sp_entry *nep;
  int i,j;

  /* Initial Markierungen m'i in eindeutigem S'd finden */
  anything_set = 0;
  init_S_d = 0;
  for(num_S_d=0;num_S_d<component->no_detT; num_S_d++)   /* gehe ueber alle p'd */
  {
    nep = component->p_d[num_S_d] -> entries;
    while(nep)                       /* gehe ueber alle Eintraege in p'd */
    {
      if( (anything_set) && (init_S_d != marking[nep->num].typeNr) )  /* gab es schon anderen Init S'd ? */
      {
	fprintf(stderr,"Init markings enabling 2 detT is not allowed.\n");
	exit(-1);
      }
      else
      {
	init_S_d = marking[nep->num].typeNr;
	anything_set = 1;
      }
      nep = nep -> next;
    }
  }
  
  /* finde fuer alle m'i die gleiche Sequenz von S'ds, die entweder wieder in m'is landen oder versacken */
  prev_S_d = init_S_d;
  for(i=0; init_S_d!=akt_S_d && i<num_S_d; i++)
  {
    anything_set = 0;
    akt_S_d = 0;
    for(j=0;j<component->no_detT;j++)
    {
      nvp = component -> detMatrices[prev_S_d].Delta_dc[j] -> rows;
      while(nvp)            /* gehe ueber alle zeilen der Delta_dc */         
      {
	nep = nvp -> entries;
	while(nep)           /* gehe ueber alle spalten einer zeile der Delta_dc */
	{
	  if(anything_set && akt_S_d != marking[nep->num].typeNr)  /* gab es schon anderen akt. S'd ? */
	    return False;
	  else
	  {
	    akt_S_d = marking[nep->num].typeNr;
	    anything_set = 1;
	  }
	  nep = nep -> next;
	}
	nvp = nvp -> next;
      }
    }
    prev_S_d = akt_S_d;
  }

  return True;
}

/******************************************************************************/
static void reduce_deltas(component)   /* fast das gleiche wie particular_sequence() */
/* beachte den kleinen logischen Fehler, dass hier in einer Sequenz jede      */
/* deterministische Transition hoechstens einmal auftreten darf !!!           */
/* Andere periodische Faelle sind jedoch denkbar, dann Q^d-Matrizen checken!  */
/* Diese Faelle koennen jedoch mit dem adaptiven Algorithmus behandelt werden */
/******************************************************************************/
solve_basis *component;
{
  int anything_set;
  int num_S_d;
  int init_S_d;
  int prev_S_d;
  int akt_S_d;
  int i,j;
  sp_entry *nep;
  sp_detMatrices *newdetMatrices;
  sp_vectors *newVector;

  /* Initial Markierungen m'i in eindeutigem S'd finden */
  anything_set = 0;
  init_S_d = 0;
  for(num_S_d=0;num_S_d<component->no_detT; num_S_d++)   /* gehe ueber alle p'd */
  {
    nep = component->p_d[num_S_d] -> entries;
    while(nep)                       /* gehe ueber alle Eintraege in p'd */
    {
      if(anything_set && init_S_d != marking[nep->num].typeNr)  /* gab es schon anderen Init S'd ? */
      {
	fprintf(stderr,"Init markings enabling 2 detT is not allowed.\n");
	exit(-1);
      }
      else
      {
	init_S_d = marking[nep->num].typeNr;
	anything_set = 1;
      }
      nep = nep -> next;
    }
  }
  
  /* p_d vektoren werden auf den initial vector reduziert.   SPEICHER FREIGEBEN ERGAENZEN */
  component -> p_d[0] = component -> p_d[init_S_d];
  
 /* detMatrices[] und Delta_dc[] entsprechend der gefundenen Sequenz umverteilen */
  newdetMatrices = (sp_detMatrices *)emalloc(sizeof(sp_detMatrices) * component->no_detT);
  prev_S_d = init_S_d;
  for(i=0; i<component->no_detT; i++)     /* gehe durch die Sequenz */
  {
    newdetMatrices[i].det_firing_time = component -> detMatrices[prev_S_d].det_firing_time;
    newdetMatrices[i].Q_d = component -> detMatrices[prev_S_d].Q_d;
    newdetMatrices[i].Q_dE = component -> detMatrices[prev_S_d].Q_dE;
    newdetMatrices[i].Delta_dE = component -> detMatrices[prev_S_d].Delta_dE;
    newdetMatrices[i].Q_dc = component -> detMatrices[prev_S_d].Q_dc;

    newdetMatrices[i].Delta_dc = (sp_matrices *)emalloc(sizeof(sp_matrices) * 1);

    /* jeder Eintrag in Delta_dc liefert den gleichen S'd; sonst waere ich nicht hier; es kann aber sein er ist 0 */
    j = 0;
    while(! component -> detMatrices[prev_S_d].Delta_dc[j] -> rows)
      j++;
    akt_S_d = marking[ component -> detMatrices[prev_S_d].Delta_dc[j] -> rows -> entries -> num ].typeNr;

    newdetMatrices[i].Delta_dc[0] = component -> detMatrices[prev_S_d].Delta_dc[akt_S_d];

    prev_S_d = akt_S_d;
  }

  if(i<component->no_detT)
  {
    printf("warning: not all detT are in closed sequence.");
    component->no_detT = i;
  }

  component -> detMatrices = newdetMatrices;
}
/*******************************************************************************************************/
