/******************************************************************************
 Program:           socket_service.h

 Purpose:           This file defines constants required by the socket
                    operations. These include the representation of the Q and
                    delta matrices on the socket layer. Note that we have to
                    define the size of these data structures explicitly since
                    the compiler makes a mistake calculating these.

 Project Director:  Reinhard German

 Programmer:        Ulrich Hertlein

 History:           Release 1: May 18, 1993
******************************************************************************/


/* protocol version */
#define VERSION     2

/* server listen backlog */
#define BACKLOG     5

/* message receive modes */
#define WAIT_ONE    2         /* wait for one rand process */
#define WAIT_ALL    3         /* wait for all rand processes */

/* executable search path */
#define RSHPATH     "/usr/ucb/rsh"

/* size of socket data structures (sizeof() doesn't work) */
#define Q_TAU_ROW_SIZE        24
#define Q_ROW_SIZE            16
#define Q_COLUMN_SIZE         12
#define DELTA_HEAD_SIZE       8
#define DELTA_ROW_SIZE        8
#define DELTA_COLUMN_SIZE     12


/* Q matrix row w. tau */
typedef struct
{
   double tau;                /* of det. transition        {8} */
   double neg_sum;            /* negative row_sum          {8} */
   unsigned row_index;        /* row index                 {4} */
   unsigned ncolumn;          /* number of columns per row {4} */
} Q_tau_row_t;                /* SIZEOF:                  {24} */

/* Q matrix row */
typedef struct
{
   double neg_sum;            /* negative row_sum          {8} */
   unsigned row_index;        /* row index                 {4} */
   unsigned ncolumn;          /* number of columns per row {4} */
} Q_row_t;                    /* SIZEOF:                  {16} */

/* Q matrix column */
typedef struct
{
   double rate;               /* transition rate {8} */
   unsigned column_index;     /* column index    {4} */
} Q_column_t;                 /* SIZEOF:        {12} */


/* delta matrix header */
typedef struct
{
   double tau;                /* switch rate of det.transition {8} */
} delta_head_t;               /* SIZEOF:                       {8} */

/* delta matrix row */
typedef struct
{
   unsigned row_index;        /* row index                 {4} */
   unsigned ncolumn;          /* number of columns per row {4} */
} delta_row_t;                /* SIZEOF:                   {8} */

/* delta matrix column */
typedef struct
{
   double prob;               /* transition probability {8} */
   unsigned column_index;     /* column index           {4} */
} delta_column_t;             /* SIZEOF:               {12} */

/* connection */
typedef struct
{
   int index;
   int fd;
   FILE *fp;
} connect_t;


/* exported variables */
extern int     version;

extern char   *local_hostname;
extern double  ru_sum;

/* exported functions */
extern void  chld_handler();

extern int   init_connect();
extern int   create_server();
extern int   start_rand();
extern FILE *wait_connect();
extern int   send_MDET();
extern int   send_Q();
extern int   send_DELTA();
extern int   msg_receive();
extern int   free_connect();
