
/* implementation of tree construction functions */

#include "node.h" /* includes treecon.h, nodecode.h */
#include "attrpredef.h"

#include "visitmap.h"

#include "treeact.h"

#include <stdlib.h>

#define _USE_OBSTACK 1

/* use of obstack: */

#if _USE_OBSTACK

#include "obstack.h"
static struct obstack TreeSpace;
static void *_TreeBase;

#ifdef __cplusplus
void* NODEPTR_struct::operator new(size_t size)
{
	return obstack_alloc(&TreeSpace, size);
}
#else
#if defined(__STDC__) || defined(__cplusplus)
char* TreeNodeAlloc(int size)
#else
char* TreeNodeAlloc(size) int size;
#endif
{
	return (char *)(obstack_alloc(&TreeSpace, size));
}
#endif

void InitTree()
{
	obstack_init(&TreeSpace);
	_TreeBase=obstack_alloc(&TreeSpace,0);
}

void FreeTree()
{
	obstack_free(&TreeSpace, _TreeBase);
	_TreeBase=obstack_alloc(&TreeSpace,0);
}

#else

#include <stdio.h>

#ifdef __cplusplus
void* NODEPTR_struct::operator new(size_t size)
{
	void *retval = malloc(size);
	if (retval) return retval;
	fprintf(stderr, "*** DEADLY: No more memory.\n");
	exit(1);
}
#else
#if defined(__STDC__) || defined(__cplusplus)
char* TreeNodeAlloc(int size)
#else
char* TreeNodeAlloc(size) int size;
#endif
{
	char *retval = (char *) malloc(size);
	if (retval) return retval;
	fprintf(stderr, "*** DEADLY: No more memory.\n");
	exit(1);
}
#endif

void InitTree() { }

void FreeTree() { }

#endif

#ifdef MONITOR
#define _SETCOORD(node) \
        node->_coord = _coordref ? *_coordref : NoCoord;
#define _COPYCOORD(node) \
        node->_coord = _currn->_desc1->_coord;
#else
#define _SETCOORD(node)
#define _COPYCOORD(node)
#endif
#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkCompiledSPNL (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkCompiledSPNL (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBCompiledSPNL)) return (_currn);
if (IsSymb (_currn, SYMBG1)) return (Mkrule_000(_coordref, _currn));
return(NULLNODEPTR);
}/* MkCompiledSPNL */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG1 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG1 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG1)) return (_currn);
return(NULLNODEPTR);
}/* MkG1 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkConnections (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkConnections (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBConnections)) return (_currn);
return(NULLNODEPTR);
}/* MkConnections */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkSectionBody (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkSectionBody (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBSectionBody)) return (_currn);
return(NULLNODEPTR);
}/* MkSectionBody */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkSectionElements (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkSectionElements (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBSectionElements)) return (_currn);
if (IsSymb (_currn, SYMBParameters)) return (Mkrule_133(_coordref, _currn));
if (IsSymb (_currn, SYMBPlaces)) return (Mkrule_069(_coordref, _currn));
if (IsSymb (_currn, SYMBTransitions)) return (Mkrule_037(_coordref, _currn));
if (IsSymb (_currn, SYMBTransitionAttributes)) return (Mkrule_021(_coordref, _currn));
if (IsSymb (_currn, SYMBArcs)) return (Mkrule_013(_coordref, _currn));
if (IsSymb (_currn, SYMBRewards)) return (Mkrule_009(_coordref, _currn));
if (IsSymb (_currn, SYMBMeasures)) return (Mkrule_007(_coordref, _currn));
if (IsSymb (_currn, SYMBMeasureParameters)) return (Mkrule_006(_coordref, _currn));
return(NULLNODEPTR);
}/* MkSectionElements */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkMeasureParameters (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkMeasureParameters (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBMeasureParameters)) return (_currn);
return(NULLNODEPTR);
}/* MkMeasureParameters */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkMeasures (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkMeasures (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBMeasures)) return (_currn);
return(NULLNODEPTR);
}/* MkMeasures */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkRewards (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkRewards (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBRewards)) return (_currn);
return(NULLNODEPTR);
}/* MkRewards */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkArcs (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkArcs (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBArcs)) return (_currn);
return(NULLNODEPTR);
}/* MkArcs */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkTransitionAttributes (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkTransitionAttributes (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBTransitionAttributes)) return (_currn);
return(NULLNODEPTR);
}/* MkTransitionAttributes */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkTransitions (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkTransitions (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBTransitions)) return (_currn);
return(NULLNODEPTR);
}/* MkTransitions */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkPlaces (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkPlaces (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBPlaces)) return (_currn);
return(NULLNODEPTR);
}/* MkPlaces */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkParameters (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkParameters (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBParameters)) return (_currn);
return(NULLNODEPTR);
}/* MkParameters */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG2 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG2 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG2)) return (_currn);
if (IsSymb (_currn, SYMBConnection)) return (Mkrule_262(_coordref, _currn));
return(NULLNODEPTR);
}/* MkG2 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkConnection (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkConnection (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBConnection)) return (_currn);
return(NULLNODEPTR);
}/* MkConnection */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG3 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG3 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG3)) return (_currn);
if (IsSymb (_currn, SYMBParameter)) return (Mkrule_266(_coordref, _currn));
return(NULLNODEPTR);
}/* MkG3 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkParameter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkParameter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBParameter)) return (_currn);
return(NULLNODEPTR);
}/* MkParameter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkGlobalId (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkGlobalId (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBGlobalId)) return (_currn);
return(NULLNODEPTR);
}/* MkGlobalId */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkParameterExpression (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkParameterExpression (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBParameterExpression)) return (_currn);
if (IsSymb (_currn, SYMBParameterSum)) return (Mkrule_271(_coordref, _currn));
if (IsSymb (_currn, SYMBParameterTerm)) return (Mkrule_271(_coordref, _currn));
if (IsSymb (_currn, SYMBParameterFactor)) return (Mkrule_271(_coordref, _currn));
if (IsSymb (_currn, SYMBParameterPrimary)) return (Mkrule_271(_coordref, _currn));
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_271(_coordref, _currn));
return(NULLNODEPTR);
}/* MkParameterExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkSign (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkSign (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBSign)) return (_currn);
return(NULLNODEPTR);
}/* MkSign */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkParameterSum (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkParameterSum (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBParameterSum)) return (_currn);
if (IsSymb (_currn, SYMBParameterTerm)) return (Mkrule_273(_coordref, _currn));
if (IsSymb (_currn, SYMBParameterFactor)) return (Mkrule_273(_coordref, _currn));
if (IsSymb (_currn, SYMBParameterPrimary)) return (Mkrule_273(_coordref, _currn));
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_273(_coordref, _currn));
return(NULLNODEPTR);
}/* MkParameterSum */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkAddOp (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkAddOp (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBAddOp)) return (_currn);
return(NULLNODEPTR);
}/* MkAddOp */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkParameterTerm (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkParameterTerm (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBParameterTerm)) return (_currn);
if (IsSymb (_currn, SYMBParameterFactor)) return (Mkrule_275(_coordref, _currn));
if (IsSymb (_currn, SYMBParameterPrimary)) return (Mkrule_275(_coordref, _currn));
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_275(_coordref, _currn));
return(NULLNODEPTR);
}/* MkParameterTerm */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkMulOp (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkMulOp (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBMulOp)) return (_currn);
return(NULLNODEPTR);
}/* MkMulOp */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkParameterFactor (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkParameterFactor (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBParameterFactor)) return (_currn);
if (IsSymb (_currn, SYMBParameterPrimary)) return (Mkrule_277(_coordref, _currn));
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_277(_coordref, _currn));
return(NULLNODEPTR);
}/* MkParameterFactor */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkExpOp (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkExpOp (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBExpOp)) return (_currn);
return(NULLNODEPTR);
}/* MkExpOp */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkParameterPrimary (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkParameterPrimary (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBParameterPrimary)) return (_currn);
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_278(_coordref, _currn));
return(NULLNODEPTR);
}/* MkParameterPrimary */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkNumber (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkNumber (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBNumber)) return (_currn);
return(NULLNODEPTR);
}/* MkNumber */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG4 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG4 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG4)) return (_currn);
if (IsSymb (_currn, SYMBPlace)) return (Mkrule_282(_coordref, _currn));
return(NULLNODEPTR);
}/* MkG4 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkPlace (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkPlace (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBPlace)) return (_currn);
return(NULLNODEPTR);
}/* MkPlace */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG5 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG5 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG5)) return (_currn);
if (IsSymb (_currn, SYMBTransition)) return (Mkrule_287(_coordref, _currn));
return(NULLNODEPTR);
}/* MkG5 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkTransition (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkTransition (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBTransition)) return (_currn);
return(NULLNODEPTR);
}/* MkTransition */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkTransitionType (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkTransitionType (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBTransitionType)) return (_currn);
return(NULLNODEPTR);
}/* MkTransitionType */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG6 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG6 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG6)) return (_currn);
if (IsSymb (_currn, SYMBTransitionAttributeDefItem)) return (Mkrule_294(_coordref, _currn));
return(NULLNODEPTR);
}/* MkG6 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkTransitionAttributeDefItem (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkTransitionAttributeDefItem (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBTransitionAttributeDefItem)) return (_currn);
return(NULLNODEPTR);
}/* MkTransitionAttributeDefItem */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG9 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG9 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG9)) return (_currn);
if (IsSymb (_currn, SYMBG7)) return (Mkrule_300(_coordref, _currn));
if (IsSymb (_currn, SYMBG8)) return (Mkrule_300(_coordref, _currn));
if (IsSymb (_currn, SYMBTransitionAttribute)) return (Mkrule_300(_coordref, _currn));
if (IsSymb (_currn, SYMBGuardAttribute)) return (Mkrule_300(_coordref, _currn));
if (IsSymb (_currn, SYMBConcurrencyAttribute)) return (Mkrule_300(_coordref, _currn));
if (IsSymb (_currn, SYMBPolicyAttribute)) return (Mkrule_300(_coordref, _currn));
if (IsSymb (_currn, SYMBMarkingDependencyAttribute)) return (Mkrule_300(_coordref, _currn));
if (IsSymb (_currn, SYMBWeightAttribute)) return (Mkrule_300(_coordref, _currn));
if (IsSymb (_currn, SYMBDistributionAttribute)) return (Mkrule_300(_coordref, _currn));
if (IsSymb (_currn, SYMBPriorityAttribute)) return (Mkrule_300(_coordref, _currn));
return(NULLNODEPTR);
}/* MkG9 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG7 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG7 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG7)) return (_currn);
if (IsSymb (_currn, SYMBG8)) return (Mkrule_297(_coordref, _currn));
if (IsSymb (_currn, SYMBTransitionAttribute)) return (Mkrule_297(_coordref, _currn));
if (IsSymb (_currn, SYMBGuardAttribute)) return (Mkrule_297(_coordref, _currn));
if (IsSymb (_currn, SYMBConcurrencyAttribute)) return (Mkrule_297(_coordref, _currn));
if (IsSymb (_currn, SYMBPolicyAttribute)) return (Mkrule_297(_coordref, _currn));
if (IsSymb (_currn, SYMBMarkingDependencyAttribute)) return (Mkrule_297(_coordref, _currn));
if (IsSymb (_currn, SYMBWeightAttribute)) return (Mkrule_297(_coordref, _currn));
if (IsSymb (_currn, SYMBDistributionAttribute)) return (Mkrule_297(_coordref, _currn));
if (IsSymb (_currn, SYMBPriorityAttribute)) return (Mkrule_297(_coordref, _currn));
return(NULLNODEPTR);
}/* MkG7 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG8 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG8 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG8)) return (_currn);
if (IsSymb (_currn, SYMBTransitionAttribute)) return (Mkrule_299(_coordref, _currn));
if (IsSymb (_currn, SYMBGuardAttribute)) return (Mkrule_299(_coordref, _currn));
if (IsSymb (_currn, SYMBConcurrencyAttribute)) return (Mkrule_299(_coordref, _currn));
if (IsSymb (_currn, SYMBPolicyAttribute)) return (Mkrule_299(_coordref, _currn));
if (IsSymb (_currn, SYMBMarkingDependencyAttribute)) return (Mkrule_299(_coordref, _currn));
if (IsSymb (_currn, SYMBWeightAttribute)) return (Mkrule_299(_coordref, _currn));
if (IsSymb (_currn, SYMBDistributionAttribute)) return (Mkrule_299(_coordref, _currn));
if (IsSymb (_currn, SYMBPriorityAttribute)) return (Mkrule_299(_coordref, _currn));
return(NULLNODEPTR);
}/* MkG8 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkTransitionAttribute (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkTransitionAttribute (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBTransitionAttribute)) return (_currn);
if (IsSymb (_currn, SYMBGuardAttribute)) return (Mkrule_307(_coordref, _currn));
if (IsSymb (_currn, SYMBConcurrencyAttribute)) return (Mkrule_306(_coordref, _currn));
if (IsSymb (_currn, SYMBPolicyAttribute)) return (Mkrule_305(_coordref, _currn));
if (IsSymb (_currn, SYMBMarkingDependencyAttribute)) return (Mkrule_304(_coordref, _currn));
if (IsSymb (_currn, SYMBWeightAttribute)) return (Mkrule_303(_coordref, _currn));
if (IsSymb (_currn, SYMBDistributionAttribute)) return (Mkrule_302(_coordref, _currn));
if (IsSymb (_currn, SYMBPriorityAttribute)) return (Mkrule_301(_coordref, _currn));
return(NULLNODEPTR);
}/* MkTransitionAttribute */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkPriorityAttribute (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkPriorityAttribute (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBPriorityAttribute)) return (_currn);
return(NULLNODEPTR);
}/* MkPriorityAttribute */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkDistributionAttribute (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkDistributionAttribute (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBDistributionAttribute)) return (_currn);
return(NULLNODEPTR);
}/* MkDistributionAttribute */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkWeightAttribute (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkWeightAttribute (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBWeightAttribute)) return (_currn);
return(NULLNODEPTR);
}/* MkWeightAttribute */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkMarkingDependencyAttribute (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkMarkingDependencyAttribute (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBMarkingDependencyAttribute)) return (_currn);
return(NULLNODEPTR);
}/* MkMarkingDependencyAttribute */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkPolicyAttribute (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkPolicyAttribute (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBPolicyAttribute)) return (_currn);
return(NULLNODEPTR);
}/* MkPolicyAttribute */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkConcurrencyAttribute (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkConcurrencyAttribute (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBConcurrencyAttribute)) return (_currn);
return(NULLNODEPTR);
}/* MkConcurrencyAttribute */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkGuardAttribute (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkGuardAttribute (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBGuardAttribute)) return (_currn);
return(NULLNODEPTR);
}/* MkGuardAttribute */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkRewardExpression (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkRewardExpression (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBRewardExpression)) return (_currn);
if (IsSymb (_currn, SYMBRewardSign)) return (Mkrule_342(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardSum)) return (Mkrule_342(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardTerm)) return (Mkrule_342(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardFactor)) return (Mkrule_342(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardPrimary)) return (Mkrule_342(_coordref, _currn));
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_342(_coordref, _currn));
return(NULLNODEPTR);
}/* MkRewardExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkDistributionExpression (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkDistributionExpression (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBDistributionExpression)) return (_currn);
if (IsSymb (_currn, SYMBExpolyDistribution)) return (Mkrule_409(_coordref, _currn));
if (IsSymb (_currn, SYMBExpolyDistributionSum)) return (Mkrule_409(_coordref, _currn));
if (IsSymb (_currn, SYMBExpolyDistributionPrimary)) return (Mkrule_409(_coordref, _currn));
if (IsSymb (_currn, SYMBContinuousExpression)) return (Mkrule_409(_coordref, _currn));
if (IsSymb (_currn, SYMBImpulseExpression)) return (Mkrule_409(_coordref, _currn));
if (IsSymb (_currn, SYMBPoissonDistribution)) return (Mkrule_408(_coordref, _currn));
if (IsSymb (_currn, SYMBBinomialDistribution)) return (Mkrule_407(_coordref, _currn));
if (IsSymb (_currn, SYMBModGeometricDistribution)) return (Mkrule_406(_coordref, _currn));
if (IsSymb (_currn, SYMBGeometricDistribution)) return (Mkrule_405(_coordref, _currn));
if (IsSymb (_currn, SYMBDiscreteUniDistribution)) return (Mkrule_404(_coordref, _currn));
if (IsSymb (_currn, SYMBBernoulliDistribution)) return (Mkrule_403(_coordref, _currn));
if (IsSymb (_currn, SYMBDeterministicDistribution)) return (Mkrule_402(_coordref, _currn));
if (IsSymb (_currn, SYMBPhaseTypeDistribution)) return (Mkrule_401(_coordref, _currn));
if (IsSymb (_currn, SYMBWeibullDistribution)) return (Mkrule_400(_coordref, _currn));
if (IsSymb (_currn, SYMBErlangDistribution)) return (Mkrule_399(_coordref, _currn));
if (IsSymb (_currn, SYMBTruncNormalDistribution)) return (Mkrule_398(_coordref, _currn));
if (IsSymb (_currn, SYMBUniformDistribution)) return (Mkrule_397(_coordref, _currn));
if (IsSymb (_currn, SYMBExponentialDistribution)) return (Mkrule_396(_coordref, _currn));
return(NULLNODEPTR);
}/* MkDistributionExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkMarkingDependencyType (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkMarkingDependencyType (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBMarkingDependencyType)) return (_currn);
return(NULLNODEPTR);
}/* MkMarkingDependencyType */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkPolicyType (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkPolicyType (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBPolicyType)) return (_currn);
return(NULLNODEPTR);
}/* MkPolicyType */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkConcurrencyDegree (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkConcurrencyDegree (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBConcurrencyDegree)) return (_currn);
return(NULLNODEPTR);
}/* MkConcurrencyDegree */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkPredicate (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkPredicate (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBPredicate)) return (_currn);
if (IsSymb (_currn, SYMBPredicateSum)) return (Mkrule_358(_coordref, _currn));
if (IsSymb (_currn, SYMBPredicateTerm)) return (Mkrule_358(_coordref, _currn));
if (IsSymb (_currn, SYMBPredicateFactor)) return (Mkrule_358(_coordref, _currn));
if (IsSymb (_currn, SYMBPredicatePrimary)) return (Mkrule_358(_coordref, _currn));
if (IsSymb (_currn, SYMBPredicateComp)) return (Mkrule_358(_coordref, _currn));
return(NULLNODEPTR);
}/* MkPredicate */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG10 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG10 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG10)) return (_currn);
if (IsSymb (_currn, SYMBArc)) return (Mkrule_324(_coordref, _currn));
return(NULLNODEPTR);
}/* MkG10 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkArc (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkArc (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBArc)) return (_currn);
return(NULLNODEPTR);
}/* MkArc */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG12 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG12 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG12)) return (_currn);
return(NULLNODEPTR);
}/* MkG12 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG11 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG11 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG11)) return (_currn);
return(NULLNODEPTR);
}/* MkG11 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkArcAttribute (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkArcAttribute (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBArcAttribute)) return (_currn);
if (IsSymb (_currn, SYMBMultiplicityAttribute)) return (Mkrule_331(_coordref, _currn));
if (IsSymb (_currn, SYMBArcTypeAttribute)) return (Mkrule_330(_coordref, _currn));
return(NULLNODEPTR);
}/* MkArcAttribute */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkArcTypeAttribute (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkArcTypeAttribute (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBArcTypeAttribute)) return (_currn);
return(NULLNODEPTR);
}/* MkArcTypeAttribute */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkMultiplicityAttribute (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkMultiplicityAttribute (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBMultiplicityAttribute)) return (_currn);
return(NULLNODEPTR);
}/* MkMultiplicityAttribute */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkArcType (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkArcType (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBArcType)) return (_currn);
return(NULLNODEPTR);
}/* MkArcType */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG13 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG13 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG13)) return (_currn);
if (IsSymb (_currn, SYMBReward)) return (Mkrule_337(_coordref, _currn));
return(NULLNODEPTR);
}/* MkG13 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkReward (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkReward (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBReward)) return (_currn);
return(NULLNODEPTR);
}/* MkReward */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkRewardSign (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkRewardSign (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBRewardSign)) return (_currn);
if (IsSymb (_currn, SYMBRewardSum)) return (Mkrule_347(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardTerm)) return (Mkrule_347(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardFactor)) return (Mkrule_347(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardPrimary)) return (Mkrule_347(_coordref, _currn));
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_347(_coordref, _currn));
return(NULLNODEPTR);
}/* MkRewardSign */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG15 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG15 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG15)) return (_currn);
return(NULLNODEPTR);
}/* MkG15 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG14 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG14 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG14)) return (_currn);
return(NULLNODEPTR);
}/* MkG14 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkRewardSum (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkRewardSum (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBRewardSum)) return (_currn);
if (IsSymb (_currn, SYMBRewardTerm)) return (Mkrule_349(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardFactor)) return (Mkrule_349(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardPrimary)) return (Mkrule_349(_coordref, _currn));
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_349(_coordref, _currn));
return(NULLNODEPTR);
}/* MkRewardSum */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkRewardTerm (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkRewardTerm (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBRewardTerm)) return (_currn);
if (IsSymb (_currn, SYMBRewardFactor)) return (Mkrule_351(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardPrimary)) return (Mkrule_351(_coordref, _currn));
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_351(_coordref, _currn));
return(NULLNODEPTR);
}/* MkRewardTerm */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkRewardFactor (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkRewardFactor (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBRewardFactor)) return (_currn);
if (IsSymb (_currn, SYMBRewardPrimary)) return (Mkrule_353(_coordref, _currn));
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_353(_coordref, _currn));
return(NULLNODEPTR);
}/* MkRewardFactor */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkRewardPrimary (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkRewardPrimary (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBRewardPrimary)) return (_currn);
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_356(_coordref, _currn));
return(NULLNODEPTR);
}/* MkRewardPrimary */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkPredicateSum (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkPredicateSum (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBPredicateSum)) return (_currn);
if (IsSymb (_currn, SYMBPredicateTerm)) return (Mkrule_360(_coordref, _currn));
if (IsSymb (_currn, SYMBPredicateFactor)) return (Mkrule_360(_coordref, _currn));
if (IsSymb (_currn, SYMBPredicatePrimary)) return (Mkrule_360(_coordref, _currn));
if (IsSymb (_currn, SYMBPredicateComp)) return (Mkrule_360(_coordref, _currn));
return(NULLNODEPTR);
}/* MkPredicateSum */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkOrOp (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkOrOp (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBOrOp)) return (_currn);
return(NULLNODEPTR);
}/* MkOrOp */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkPredicateTerm (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkPredicateTerm (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBPredicateTerm)) return (_currn);
if (IsSymb (_currn, SYMBPredicateFactor)) return (Mkrule_362(_coordref, _currn));
if (IsSymb (_currn, SYMBPredicatePrimary)) return (Mkrule_362(_coordref, _currn));
if (IsSymb (_currn, SYMBPredicateComp)) return (Mkrule_362(_coordref, _currn));
return(NULLNODEPTR);
}/* MkPredicateTerm */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkAndOp (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkAndOp (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBAndOp)) return (_currn);
return(NULLNODEPTR);
}/* MkAndOp */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkPredicateFactor (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkPredicateFactor (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBPredicateFactor)) return (_currn);
if (IsSymb (_currn, SYMBPredicatePrimary)) return (Mkrule_364(_coordref, _currn));
if (IsSymb (_currn, SYMBPredicateComp)) return (Mkrule_364(_coordref, _currn));
return(NULLNODEPTR);
}/* MkPredicateFactor */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkNotOp (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkNotOp (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBNotOp)) return (_currn);
return(NULLNODEPTR);
}/* MkNotOp */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkPredicatePrimary (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkPredicatePrimary (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBPredicatePrimary)) return (_currn);
if (IsSymb (_currn, SYMBPredicateComp)) return (Mkrule_367(_coordref, _currn));
return(NULLNODEPTR);
}/* MkPredicatePrimary */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkPredicateComp (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkPredicateComp (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBPredicateComp)) return (_currn);
return(NULLNODEPTR);
}/* MkPredicateComp */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkCompOp (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkCompOp (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBCompOp)) return (_currn);
return(NULLNODEPTR);
}/* MkCompOp */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG16 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG16 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG16)) return (_currn);
if (IsSymb (_currn, SYMBMeasure)) return (Mkrule_371(_coordref, _currn));
return(NULLNODEPTR);
}/* MkG16 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkMeasure (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkMeasure (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBMeasure)) return (_currn);
return(NULLNODEPTR);
}/* MkMeasure */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkMeasureExpression (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkMeasureExpression (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBMeasureExpression)) return (_currn);
if (IsSymb (_currn, SYMBMultMeasureExpr)) return (Mkrule_375(_coordref, _currn));
if (IsSymb (_currn, SYMBSingleMeasureExpr)) return (Mkrule_374(_coordref, _currn));
if (IsSymb (_currn, SYMBMeasureSum)) return (Mkrule_374(_coordref, _currn));
if (IsSymb (_currn, SYMBMeasureTerm)) return (Mkrule_374(_coordref, _currn));
if (IsSymb (_currn, SYMBMeasureFactor)) return (Mkrule_374(_coordref, _currn));
if (IsSymb (_currn, SYMBMeasurePrimary)) return (Mkrule_374(_coordref, _currn));
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_374(_coordref, _currn));
return(NULLNODEPTR);
}/* MkMeasureExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkSingleMeasureExpr (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkSingleMeasureExpr (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBSingleMeasureExpr)) return (_currn);
if (IsSymb (_currn, SYMBMeasureSum)) return (Mkrule_377(_coordref, _currn));
if (IsSymb (_currn, SYMBMeasureTerm)) return (Mkrule_377(_coordref, _currn));
if (IsSymb (_currn, SYMBMeasureFactor)) return (Mkrule_377(_coordref, _currn));
if (IsSymb (_currn, SYMBMeasurePrimary)) return (Mkrule_377(_coordref, _currn));
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_377(_coordref, _currn));
return(NULLNODEPTR);
}/* MkSingleMeasureExpr */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkMultMeasureExpr (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkMultMeasureExpr (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBMultMeasureExpr)) return (_currn);
return(NULLNODEPTR);
}/* MkMultMeasureExpr */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkMeasureSum (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkMeasureSum (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBMeasureSum)) return (_currn);
if (IsSymb (_currn, SYMBMeasureTerm)) return (Mkrule_379(_coordref, _currn));
if (IsSymb (_currn, SYMBMeasureFactor)) return (Mkrule_379(_coordref, _currn));
if (IsSymb (_currn, SYMBMeasurePrimary)) return (Mkrule_379(_coordref, _currn));
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_379(_coordref, _currn));
return(NULLNODEPTR);
}/* MkMeasureSum */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkMeasureTerm (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkMeasureTerm (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBMeasureTerm)) return (_currn);
if (IsSymb (_currn, SYMBMeasureFactor)) return (Mkrule_381(_coordref, _currn));
if (IsSymb (_currn, SYMBMeasurePrimary)) return (Mkrule_381(_coordref, _currn));
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_381(_coordref, _currn));
return(NULLNODEPTR);
}/* MkMeasureTerm */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkMeasureFactor (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkMeasureFactor (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBMeasureFactor)) return (_currn);
if (IsSymb (_currn, SYMBMeasurePrimary)) return (Mkrule_383(_coordref, _currn));
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_383(_coordref, _currn));
return(NULLNODEPTR);
}/* MkMeasureFactor */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkMeasurePrimary (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkMeasurePrimary (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBMeasurePrimary)) return (_currn);
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_384(_coordref, _currn));
return(NULLNODEPTR);
}/* MkMeasurePrimary */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG17 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG17 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG17)) return (_currn);
if (IsSymb (_currn, SYMBMeasureParameter)) return (Mkrule_392(_coordref, _currn));
return(NULLNODEPTR);
}/* MkG17 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkMeasureParameter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkMeasureParameter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBMeasureParameter)) return (_currn);
return(NULLNODEPTR);
}/* MkMeasureParameter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkExponentialDistribution (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkExponentialDistribution (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBExponentialDistribution)) return (_currn);
return(NULLNODEPTR);
}/* MkExponentialDistribution */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkUniformDistribution (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkUniformDistribution (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBUniformDistribution)) return (_currn);
return(NULLNODEPTR);
}/* MkUniformDistribution */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkTruncNormalDistribution (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkTruncNormalDistribution (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBTruncNormalDistribution)) return (_currn);
return(NULLNODEPTR);
}/* MkTruncNormalDistribution */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkErlangDistribution (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkErlangDistribution (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBErlangDistribution)) return (_currn);
return(NULLNODEPTR);
}/* MkErlangDistribution */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkWeibullDistribution (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkWeibullDistribution (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBWeibullDistribution)) return (_currn);
return(NULLNODEPTR);
}/* MkWeibullDistribution */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkPhaseTypeDistribution (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkPhaseTypeDistribution (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBPhaseTypeDistribution)) return (_currn);
return(NULLNODEPTR);
}/* MkPhaseTypeDistribution */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkDeterministicDistribution (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkDeterministicDistribution (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBDeterministicDistribution)) return (_currn);
return(NULLNODEPTR);
}/* MkDeterministicDistribution */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkBernoulliDistribution (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkBernoulliDistribution (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBBernoulliDistribution)) return (_currn);
return(NULLNODEPTR);
}/* MkBernoulliDistribution */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkDiscreteUniDistribution (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkDiscreteUniDistribution (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBDiscreteUniDistribution)) return (_currn);
return(NULLNODEPTR);
}/* MkDiscreteUniDistribution */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkGeometricDistribution (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkGeometricDistribution (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBGeometricDistribution)) return (_currn);
return(NULLNODEPTR);
}/* MkGeometricDistribution */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkModGeometricDistribution (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkModGeometricDistribution (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBModGeometricDistribution)) return (_currn);
return(NULLNODEPTR);
}/* MkModGeometricDistribution */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkBinomialDistribution (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkBinomialDistribution (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBBinomialDistribution)) return (_currn);
return(NULLNODEPTR);
}/* MkBinomialDistribution */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkPoissonDistribution (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkPoissonDistribution (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBPoissonDistribution)) return (_currn);
return(NULLNODEPTR);
}/* MkPoissonDistribution */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkExpolyDistribution (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkExpolyDistribution (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBExpolyDistribution)) return (_currn);
if (IsSymb (_currn, SYMBExpolyDistributionSum)) return (Mkrule_433(_coordref, _currn));
if (IsSymb (_currn, SYMBExpolyDistributionPrimary)) return (Mkrule_433(_coordref, _currn));
if (IsSymb (_currn, SYMBContinuousExpression)) return (Mkrule_433(_coordref, _currn));
if (IsSymb (_currn, SYMBImpulseExpression)) return (Mkrule_433(_coordref, _currn));
return(NULLNODEPTR);
}/* MkExpolyDistribution */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkMatrix (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkMatrix (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBMatrix)) return (_currn);
return(NULLNODEPTR);
}/* MkMatrix */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkVektor (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkVektor (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBVektor)) return (_currn);
return(NULLNODEPTR);
}/* MkVektor */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG20 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG20 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG20)) return (_currn);
if (IsSymb (_currn, SYMBG18)) return (Mkrule_420(_coordref, _currn));
if (IsSymb (_currn, SYMBG19)) return (Mkrule_420(_coordref, _currn));
if (IsSymb (_currn, SYMBVektor)) return (Mkrule_420(_coordref, _currn));
return(NULLNODEPTR);
}/* MkG20 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG18 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG18 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG18)) return (_currn);
if (IsSymb (_currn, SYMBG19)) return (Mkrule_417(_coordref, _currn));
if (IsSymb (_currn, SYMBVektor)) return (Mkrule_417(_coordref, _currn));
return(NULLNODEPTR);
}/* MkG18 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG19 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG19 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG19)) return (_currn);
if (IsSymb (_currn, SYMBVektor)) return (Mkrule_419(_coordref, _currn));
return(NULLNODEPTR);
}/* MkG19 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG23 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG23 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG23)) return (_currn);
if (IsSymb (_currn, SYMBG21)) return (Mkrule_425(_coordref, _currn));
if (IsSymb (_currn, SYMBG22)) return (Mkrule_425(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardExpression)) return (Mkrule_425(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardSign)) return (Mkrule_425(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardSum)) return (Mkrule_425(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardTerm)) return (Mkrule_425(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardFactor)) return (Mkrule_425(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardPrimary)) return (Mkrule_425(_coordref, _currn));
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_425(_coordref, _currn));
return(NULLNODEPTR);
}/* MkG23 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG21 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG21 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG21)) return (_currn);
if (IsSymb (_currn, SYMBG22)) return (Mkrule_422(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardExpression)) return (Mkrule_422(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardSign)) return (Mkrule_422(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardSum)) return (Mkrule_422(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardTerm)) return (Mkrule_422(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardFactor)) return (Mkrule_422(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardPrimary)) return (Mkrule_422(_coordref, _currn));
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_422(_coordref, _currn));
return(NULLNODEPTR);
}/* MkG21 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG22 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG22 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG22)) return (_currn);
if (IsSymb (_currn, SYMBRewardExpression)) return (Mkrule_424(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardSign)) return (Mkrule_424(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardSum)) return (Mkrule_424(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardTerm)) return (Mkrule_424(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardFactor)) return (Mkrule_424(_coordref, _currn));
if (IsSymb (_currn, SYMBRewardPrimary)) return (Mkrule_424(_coordref, _currn));
if (IsSymb (_currn, SYMBNumber)) return (Mkrule_424(_coordref, _currn));
return(NULLNODEPTR);
}/* MkG22 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkExpolyDistributionSum (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkExpolyDistributionSum (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBExpolyDistributionSum)) return (_currn);
if (IsSymb (_currn, SYMBExpolyDistributionPrimary)) return (Mkrule_435(_coordref, _currn));
if (IsSymb (_currn, SYMBContinuousExpression)) return (Mkrule_435(_coordref, _currn));
if (IsSymb (_currn, SYMBImpulseExpression)) return (Mkrule_435(_coordref, _currn));
return(NULLNODEPTR);
}/* MkExpolyDistributionSum */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkExpolyDistributionPrimary (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkExpolyDistributionPrimary (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBExpolyDistributionPrimary)) return (_currn);
if (IsSymb (_currn, SYMBContinuousExpression)) return (Mkrule_437(_coordref, _currn));
if (IsSymb (_currn, SYMBImpulseExpression)) return (Mkrule_436(_coordref, _currn));
return(NULLNODEPTR);
}/* MkExpolyDistributionPrimary */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkImpulseExpression (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkImpulseExpression (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBImpulseExpression)) return (_currn);
return(NULLNODEPTR);
}/* MkImpulseExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkContinuousExpression (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkContinuousExpression (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBContinuousExpression)) return (_currn);
return(NULLNODEPTR);
}/* MkContinuousExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkExpolynomial (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkExpolynomial (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBExpolynomial)) return (_currn);
if (IsSymb (_currn, SYMBExpolinomialSum)) return (Mkrule_442(_coordref, _currn));
if (IsSymb (_currn, SYMBExpolinomialPrimary)) return (Mkrule_442(_coordref, _currn));
return(NULLNODEPTR);
}/* MkExpolynomial */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkExpolinomialSum (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkExpolinomialSum (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBExpolinomialSum)) return (_currn);
if (IsSymb (_currn, SYMBExpolinomialPrimary)) return (Mkrule_444(_coordref, _currn));
return(NULLNODEPTR);
}/* MkExpolinomialSum */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkExpolinomialPrimary (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkExpolinomialPrimary (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBExpolinomialPrimary)) return (_currn);
return(NULLNODEPTR);
}/* MkExpolinomialPrimary */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG25 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG25 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG25)) return (_currn);
return(NULLNODEPTR);
}/* MkG25 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR MkG24 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR MkG24 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (IsSymb (_currn, SYMBG24)) return (_currn);
return(NULLNODEPTR);
}/* MkG24 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_000 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_000 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_000 _currn;
#ifdef __cplusplus
_currn = new _TPrule_000;
#else
_currn = (_TPPrule_000) TreeNodeAlloc (sizeof (struct _TPrule_000));
#endif
_currn->_prod = RULErule_000;
_currn->_desc1 = (_TSPG1) MkG1 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_000: root of subtree no. 1 can not be made a G1 node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_000;
return ( (NODEPTR) _currn);
}/* Mkrule_000 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_001 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_001 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_001 _currn;
#ifdef __cplusplus
_currn = new _TPrule_001;
#else
_currn = (_TPPrule_001) TreeNodeAlloc (sizeof (struct _TPrule_001));
#endif
_currn->_prod = RULErule_001;
_currn->_desc1 = (_TSPG1) MkG1 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_001: root of subtree no. 1 can not be made a G1 node ", 0, _coordref);
_currn->_desc2 = (_TSPConnections) MkConnections (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_001: root of subtree no. 2 can not be made a Connections node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_001;
return ( (NODEPTR) _currn);
}/* Mkrule_001 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_002 (POSITION *_coordref)
#else
NODEPTR Mkrule_002 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_002 _currn;
#ifdef __cplusplus
_currn = new _TPrule_002;
#else
_currn = (_TPPrule_002) TreeNodeAlloc (sizeof (struct _TPrule_002));
#endif
_currn->_prod = RULErule_002;
_SETCOORD(_currn)
_TERMACT_rule_002;
return ( (NODEPTR) _currn);
}/* Mkrule_002 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_003 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_003 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_003 _currn;
#ifdef __cplusplus
_currn = new _TPrule_003;
#else
_currn = (_TPPrule_003) TreeNodeAlloc (sizeof (struct _TPrule_003));
#endif
_currn->_prod = RULErule_003;
_currn->_desc1 = (_TSPG1) MkG1 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_003: root of subtree no. 1 can not be made a G1 node ", 0, _coordref);
_currn->_desc2 = (_TSPSectionBody) MkSectionBody (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_003: root of subtree no. 2 can not be made a SectionBody node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_003;
return ( (NODEPTR) _currn);
}/* Mkrule_003 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_004 (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR Mkrule_004 (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	_TPPrule_004 _currn;
#ifdef __cplusplus
_currn = new _TPrule_004;
#else
_currn = (_TPPrule_004) TreeNodeAlloc (sizeof (struct _TPrule_004));
#endif
_currn->_prod = RULErule_004;
_currn->_desc1 = (_TSPSectionElements) MkSectionElements (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_004: root of subtree no. 1 can not be made a SectionElements node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_004;
return ( (NODEPTR) _currn);
}/* Mkrule_004 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_005 (POSITION *_coordref)
#else
NODEPTR Mkrule_005 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_005 _currn;
#ifdef __cplusplus
_currn = new _TPrule_005;
#else
_currn = (_TPPrule_005) TreeNodeAlloc (sizeof (struct _TPrule_005));
#endif
_currn->_prod = RULErule_005;
_SETCOORD(_currn)
_TERMACT_rule_005;
return ( (NODEPTR) _currn);
}/* Mkrule_005 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_006 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_006 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_006 _currn;
#ifdef __cplusplus
_currn = new _TPrule_006;
#else
_currn = (_TPPrule_006) TreeNodeAlloc (sizeof (struct _TPrule_006));
#endif
_currn->_prod = RULErule_006;
_currn->_desc1 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_006: root of subtree no. 1 can not be made a MeasureParameters node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_006;
return ( (NODEPTR) _currn);
}/* Mkrule_006 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_007 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_007 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_007 _currn;
#ifdef __cplusplus
_currn = new _TPrule_007;
#else
_currn = (_TPPrule_007) TreeNodeAlloc (sizeof (struct _TPrule_007));
#endif
_currn->_prod = RULErule_007;
_currn->_desc1 = (_TSPMeasures) MkMeasures (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_007: root of subtree no. 1 can not be made a Measures node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_007;
return ( (NODEPTR) _currn);
}/* Mkrule_007 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_008 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_008 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_008 _currn;
#ifdef __cplusplus
_currn = new _TPrule_008;
#else
_currn = (_TPPrule_008) TreeNodeAlloc (sizeof (struct _TPrule_008));
#endif
_currn->_prod = RULErule_008;
_currn->_desc1 = (_TSPMeasures) MkMeasures (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_008: root of subtree no. 1 can not be made a Measures node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_008: root of subtree no. 2 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_008;
return ( (NODEPTR) _currn);
}/* Mkrule_008 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_009 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_009 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_009 _currn;
#ifdef __cplusplus
_currn = new _TPrule_009;
#else
_currn = (_TPPrule_009) TreeNodeAlloc (sizeof (struct _TPrule_009));
#endif
_currn->_prod = RULErule_009;
_currn->_desc1 = (_TSPRewards) MkRewards (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_009: root of subtree no. 1 can not be made a Rewards node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_009;
return ( (NODEPTR) _currn);
}/* Mkrule_009 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_010 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_010 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_010 _currn;
#ifdef __cplusplus
_currn = new _TPrule_010;
#else
_currn = (_TPPrule_010) TreeNodeAlloc (sizeof (struct _TPrule_010));
#endif
_currn->_prod = RULErule_010;
_currn->_desc1 = (_TSPRewards) MkRewards (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_010: root of subtree no. 1 can not be made a Rewards node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_010: root of subtree no. 2 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_010;
return ( (NODEPTR) _currn);
}/* Mkrule_010 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_011 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_011 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_011 _currn;
#ifdef __cplusplus
_currn = new _TPrule_011;
#else
_currn = (_TPPrule_011) TreeNodeAlloc (sizeof (struct _TPrule_011));
#endif
_currn->_prod = RULErule_011;
_currn->_desc1 = (_TSPRewards) MkRewards (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_011: root of subtree no. 1 can not be made a Rewards node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasures) MkMeasures (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_011: root of subtree no. 2 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_011;
return ( (NODEPTR) _currn);
}/* Mkrule_011 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_012 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_012 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_012 _currn;
#ifdef __cplusplus
_currn = new _TPrule_012;
#else
_currn = (_TPPrule_012) TreeNodeAlloc (sizeof (struct _TPrule_012));
#endif
_currn->_prod = RULErule_012;
_currn->_desc1 = (_TSPRewards) MkRewards (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_012: root of subtree no. 1 can not be made a Rewards node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasures) MkMeasures (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_012: root of subtree no. 2 can not be made a Measures node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_012: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_012;
return ( (NODEPTR) _currn);
}/* Mkrule_012 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_013 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_013 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_013 _currn;
#ifdef __cplusplus
_currn = new _TPrule_013;
#else
_currn = (_TPPrule_013) TreeNodeAlloc (sizeof (struct _TPrule_013));
#endif
_currn->_prod = RULErule_013;
_currn->_desc1 = (_TSPArcs) MkArcs (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_013: root of subtree no. 1 can not be made a Arcs node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_013;
return ( (NODEPTR) _currn);
}/* Mkrule_013 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_014 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_014 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_014 _currn;
#ifdef __cplusplus
_currn = new _TPrule_014;
#else
_currn = (_TPPrule_014) TreeNodeAlloc (sizeof (struct _TPrule_014));
#endif
_currn->_prod = RULErule_014;
_currn->_desc1 = (_TSPArcs) MkArcs (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_014: root of subtree no. 1 can not be made a Arcs node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_014: root of subtree no. 2 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_014;
return ( (NODEPTR) _currn);
}/* Mkrule_014 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_015 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_015 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_015 _currn;
#ifdef __cplusplus
_currn = new _TPrule_015;
#else
_currn = (_TPPrule_015) TreeNodeAlloc (sizeof (struct _TPrule_015));
#endif
_currn->_prod = RULErule_015;
_currn->_desc1 = (_TSPArcs) MkArcs (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_015: root of subtree no. 1 can not be made a Arcs node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasures) MkMeasures (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_015: root of subtree no. 2 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_015;
return ( (NODEPTR) _currn);
}/* Mkrule_015 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_016 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_016 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_016 _currn;
#ifdef __cplusplus
_currn = new _TPrule_016;
#else
_currn = (_TPPrule_016) TreeNodeAlloc (sizeof (struct _TPrule_016));
#endif
_currn->_prod = RULErule_016;
_currn->_desc1 = (_TSPArcs) MkArcs (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_016: root of subtree no. 1 can not be made a Arcs node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasures) MkMeasures (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_016: root of subtree no. 2 can not be made a Measures node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_016: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_016;
return ( (NODEPTR) _currn);
}/* Mkrule_016 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_017 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_017 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_017 _currn;
#ifdef __cplusplus
_currn = new _TPrule_017;
#else
_currn = (_TPPrule_017) TreeNodeAlloc (sizeof (struct _TPrule_017));
#endif
_currn->_prod = RULErule_017;
_currn->_desc1 = (_TSPArcs) MkArcs (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_017: root of subtree no. 1 can not be made a Arcs node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_017: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_017;
return ( (NODEPTR) _currn);
}/* Mkrule_017 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_018 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_018 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_018 _currn;
#ifdef __cplusplus
_currn = new _TPrule_018;
#else
_currn = (_TPPrule_018) TreeNodeAlloc (sizeof (struct _TPrule_018));
#endif
_currn->_prod = RULErule_018;
_currn->_desc1 = (_TSPArcs) MkArcs (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_018: root of subtree no. 1 can not be made a Arcs node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_018: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_018: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_018;
return ( (NODEPTR) _currn);
}/* Mkrule_018 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_019 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_019 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_019 _currn;
#ifdef __cplusplus
_currn = new _TPrule_019;
#else
_currn = (_TPPrule_019) TreeNodeAlloc (sizeof (struct _TPrule_019));
#endif
_currn->_prod = RULErule_019;
_currn->_desc1 = (_TSPArcs) MkArcs (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_019: root of subtree no. 1 can not be made a Arcs node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_019: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_019: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_019;
return ( (NODEPTR) _currn);
}/* Mkrule_019 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_020 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_020 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_020 _currn;
#ifdef __cplusplus
_currn = new _TPrule_020;
#else
_currn = (_TPPrule_020) TreeNodeAlloc (sizeof (struct _TPrule_020));
#endif
_currn->_prod = RULErule_020;
_currn->_desc1 = (_TSPArcs) MkArcs (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_020: root of subtree no. 1 can not be made a Arcs node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_020: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_020: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_020: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_020;
return ( (NODEPTR) _currn);
}/* Mkrule_020 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_021 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_021 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_021 _currn;
#ifdef __cplusplus
_currn = new _TPrule_021;
#else
_currn = (_TPPrule_021) TreeNodeAlloc (sizeof (struct _TPrule_021));
#endif
_currn->_prod = RULErule_021;
_currn->_desc1 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_021: root of subtree no. 1 can not be made a TransitionAttributes node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_021;
return ( (NODEPTR) _currn);
}/* Mkrule_021 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_022 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_022 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_022 _currn;
#ifdef __cplusplus
_currn = new _TPrule_022;
#else
_currn = (_TPPrule_022) TreeNodeAlloc (sizeof (struct _TPrule_022));
#endif
_currn->_prod = RULErule_022;
_currn->_desc1 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_022: root of subtree no. 1 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_022: root of subtree no. 2 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_022;
return ( (NODEPTR) _currn);
}/* Mkrule_022 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_023 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_023 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_023 _currn;
#ifdef __cplusplus
_currn = new _TPrule_023;
#else
_currn = (_TPPrule_023) TreeNodeAlloc (sizeof (struct _TPrule_023));
#endif
_currn->_prod = RULErule_023;
_currn->_desc1 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_023: root of subtree no. 1 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasures) MkMeasures (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_023: root of subtree no. 2 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_023;
return ( (NODEPTR) _currn);
}/* Mkrule_023 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_024 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_024 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_024 _currn;
#ifdef __cplusplus
_currn = new _TPrule_024;
#else
_currn = (_TPPrule_024) TreeNodeAlloc (sizeof (struct _TPrule_024));
#endif
_currn->_prod = RULErule_024;
_currn->_desc1 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_024: root of subtree no. 1 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasures) MkMeasures (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_024: root of subtree no. 2 can not be made a Measures node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_024: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_024;
return ( (NODEPTR) _currn);
}/* Mkrule_024 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_025 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_025 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_025 _currn;
#ifdef __cplusplus
_currn = new _TPrule_025;
#else
_currn = (_TPPrule_025) TreeNodeAlloc (sizeof (struct _TPrule_025));
#endif
_currn->_prod = RULErule_025;
_currn->_desc1 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_025: root of subtree no. 1 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_025: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_025;
return ( (NODEPTR) _currn);
}/* Mkrule_025 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_026 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_026 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_026 _currn;
#ifdef __cplusplus
_currn = new _TPrule_026;
#else
_currn = (_TPPrule_026) TreeNodeAlloc (sizeof (struct _TPrule_026));
#endif
_currn->_prod = RULErule_026;
_currn->_desc1 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_026: root of subtree no. 1 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_026: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_026: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_026;
return ( (NODEPTR) _currn);
}/* Mkrule_026 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_027 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_027 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_027 _currn;
#ifdef __cplusplus
_currn = new _TPrule_027;
#else
_currn = (_TPPrule_027) TreeNodeAlloc (sizeof (struct _TPrule_027));
#endif
_currn->_prod = RULErule_027;
_currn->_desc1 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_027: root of subtree no. 1 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_027: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_027: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_027;
return ( (NODEPTR) _currn);
}/* Mkrule_027 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_028 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_028 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_028 _currn;
#ifdef __cplusplus
_currn = new _TPrule_028;
#else
_currn = (_TPPrule_028) TreeNodeAlloc (sizeof (struct _TPrule_028));
#endif
_currn->_prod = RULErule_028;
_currn->_desc1 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_028: root of subtree no. 1 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_028: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_028: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_028: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_028;
return ( (NODEPTR) _currn);
}/* Mkrule_028 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_029 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_029 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_029 _currn;
#ifdef __cplusplus
_currn = new _TPrule_029;
#else
_currn = (_TPPrule_029) TreeNodeAlloc (sizeof (struct _TPrule_029));
#endif
_currn->_prod = RULErule_029;
_currn->_desc1 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_029: root of subtree no. 1 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_029: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_029;
return ( (NODEPTR) _currn);
}/* Mkrule_029 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_030 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_030 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_030 _currn;
#ifdef __cplusplus
_currn = new _TPrule_030;
#else
_currn = (_TPPrule_030) TreeNodeAlloc (sizeof (struct _TPrule_030));
#endif
_currn->_prod = RULErule_030;
_currn->_desc1 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_030: root of subtree no. 1 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_030: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_030: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_030;
return ( (NODEPTR) _currn);
}/* Mkrule_030 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_031 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_031 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_031 _currn;
#ifdef __cplusplus
_currn = new _TPrule_031;
#else
_currn = (_TPPrule_031) TreeNodeAlloc (sizeof (struct _TPrule_031));
#endif
_currn->_prod = RULErule_031;
_currn->_desc1 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_031: root of subtree no. 1 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_031: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_031: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_031;
return ( (NODEPTR) _currn);
}/* Mkrule_031 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_032 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_032 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_032 _currn;
#ifdef __cplusplus
_currn = new _TPrule_032;
#else
_currn = (_TPPrule_032) TreeNodeAlloc (sizeof (struct _TPrule_032));
#endif
_currn->_prod = RULErule_032;
_currn->_desc1 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_032: root of subtree no. 1 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_032: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_032: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_032: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_032;
return ( (NODEPTR) _currn);
}/* Mkrule_032 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_033 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_033 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_033 _currn;
#ifdef __cplusplus
_currn = new _TPrule_033;
#else
_currn = (_TPPrule_033) TreeNodeAlloc (sizeof (struct _TPrule_033));
#endif
_currn->_prod = RULErule_033;
_currn->_desc1 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_033: root of subtree no. 1 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_033: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_033: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_033;
return ( (NODEPTR) _currn);
}/* Mkrule_033 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_034 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_034 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_034 _currn;
#ifdef __cplusplus
_currn = new _TPrule_034;
#else
_currn = (_TPPrule_034) TreeNodeAlloc (sizeof (struct _TPrule_034));
#endif
_currn->_prod = RULErule_034;
_currn->_desc1 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_034: root of subtree no. 1 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_034: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_034: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_034: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_034;
return ( (NODEPTR) _currn);
}/* Mkrule_034 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_035 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_035 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_035 _currn;
#ifdef __cplusplus
_currn = new _TPrule_035;
#else
_currn = (_TPPrule_035) TreeNodeAlloc (sizeof (struct _TPrule_035));
#endif
_currn->_prod = RULErule_035;
_currn->_desc1 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_035: root of subtree no. 1 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_035: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_035: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_035: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_035;
return ( (NODEPTR) _currn);
}/* Mkrule_035 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_036 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_036 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_036 _currn;
#ifdef __cplusplus
_currn = new _TPrule_036;
#else
_currn = (_TPPrule_036) TreeNodeAlloc (sizeof (struct _TPrule_036));
#endif
_currn->_prod = RULErule_036;
_currn->_desc1 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_036: root of subtree no. 1 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_036: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_036: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_036: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_036: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_036;
return ( (NODEPTR) _currn);
}/* Mkrule_036 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_037 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_037 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_037 _currn;
#ifdef __cplusplus
_currn = new _TPrule_037;
#else
_currn = (_TPPrule_037) TreeNodeAlloc (sizeof (struct _TPrule_037));
#endif
_currn->_prod = RULErule_037;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_037: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_037;
return ( (NODEPTR) _currn);
}/* Mkrule_037 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_038 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_038 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_038 _currn;
#ifdef __cplusplus
_currn = new _TPrule_038;
#else
_currn = (_TPPrule_038) TreeNodeAlloc (sizeof (struct _TPrule_038));
#endif
_currn->_prod = RULErule_038;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_038: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_038: root of subtree no. 2 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_038;
return ( (NODEPTR) _currn);
}/* Mkrule_038 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_039 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_039 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_039 _currn;
#ifdef __cplusplus
_currn = new _TPrule_039;
#else
_currn = (_TPPrule_039) TreeNodeAlloc (sizeof (struct _TPrule_039));
#endif
_currn->_prod = RULErule_039;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_039: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasures) MkMeasures (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_039: root of subtree no. 2 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_039;
return ( (NODEPTR) _currn);
}/* Mkrule_039 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_040 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_040 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_040 _currn;
#ifdef __cplusplus
_currn = new _TPrule_040;
#else
_currn = (_TPPrule_040) TreeNodeAlloc (sizeof (struct _TPrule_040));
#endif
_currn->_prod = RULErule_040;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_040: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasures) MkMeasures (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_040: root of subtree no. 2 can not be made a Measures node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_040: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_040;
return ( (NODEPTR) _currn);
}/* Mkrule_040 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_041 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_041 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_041 _currn;
#ifdef __cplusplus
_currn = new _TPrule_041;
#else
_currn = (_TPPrule_041) TreeNodeAlloc (sizeof (struct _TPrule_041));
#endif
_currn->_prod = RULErule_041;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_041: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_041: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_041;
return ( (NODEPTR) _currn);
}/* Mkrule_041 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_042 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_042 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_042 _currn;
#ifdef __cplusplus
_currn = new _TPrule_042;
#else
_currn = (_TPPrule_042) TreeNodeAlloc (sizeof (struct _TPrule_042));
#endif
_currn->_prod = RULErule_042;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_042: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_042: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_042: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_042;
return ( (NODEPTR) _currn);
}/* Mkrule_042 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_043 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_043 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_043 _currn;
#ifdef __cplusplus
_currn = new _TPrule_043;
#else
_currn = (_TPPrule_043) TreeNodeAlloc (sizeof (struct _TPrule_043));
#endif
_currn->_prod = RULErule_043;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_043: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_043: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_043: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_043;
return ( (NODEPTR) _currn);
}/* Mkrule_043 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_044 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_044 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_044 _currn;
#ifdef __cplusplus
_currn = new _TPrule_044;
#else
_currn = (_TPPrule_044) TreeNodeAlloc (sizeof (struct _TPrule_044));
#endif
_currn->_prod = RULErule_044;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_044: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_044: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_044: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_044: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_044;
return ( (NODEPTR) _currn);
}/* Mkrule_044 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_045 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_045 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_045 _currn;
#ifdef __cplusplus
_currn = new _TPrule_045;
#else
_currn = (_TPPrule_045) TreeNodeAlloc (sizeof (struct _TPrule_045));
#endif
_currn->_prod = RULErule_045;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_045: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_045: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_045;
return ( (NODEPTR) _currn);
}/* Mkrule_045 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_046 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_046 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_046 _currn;
#ifdef __cplusplus
_currn = new _TPrule_046;
#else
_currn = (_TPPrule_046) TreeNodeAlloc (sizeof (struct _TPrule_046));
#endif
_currn->_prod = RULErule_046;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_046: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_046: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_046: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_046;
return ( (NODEPTR) _currn);
}/* Mkrule_046 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_047 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_047 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_047 _currn;
#ifdef __cplusplus
_currn = new _TPrule_047;
#else
_currn = (_TPPrule_047) TreeNodeAlloc (sizeof (struct _TPrule_047));
#endif
_currn->_prod = RULErule_047;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_047: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_047: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_047: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_047;
return ( (NODEPTR) _currn);
}/* Mkrule_047 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_048 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_048 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_048 _currn;
#ifdef __cplusplus
_currn = new _TPrule_048;
#else
_currn = (_TPPrule_048) TreeNodeAlloc (sizeof (struct _TPrule_048));
#endif
_currn->_prod = RULErule_048;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_048: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_048: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_048: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_048: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_048;
return ( (NODEPTR) _currn);
}/* Mkrule_048 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_049 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_049 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_049 _currn;
#ifdef __cplusplus
_currn = new _TPrule_049;
#else
_currn = (_TPPrule_049) TreeNodeAlloc (sizeof (struct _TPrule_049));
#endif
_currn->_prod = RULErule_049;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_049: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_049: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_049: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_049;
return ( (NODEPTR) _currn);
}/* Mkrule_049 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_050 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_050 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_050 _currn;
#ifdef __cplusplus
_currn = new _TPrule_050;
#else
_currn = (_TPPrule_050) TreeNodeAlloc (sizeof (struct _TPrule_050));
#endif
_currn->_prod = RULErule_050;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_050: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_050: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_050: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_050: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_050;
return ( (NODEPTR) _currn);
}/* Mkrule_050 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_051 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_051 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_051 _currn;
#ifdef __cplusplus
_currn = new _TPrule_051;
#else
_currn = (_TPPrule_051) TreeNodeAlloc (sizeof (struct _TPrule_051));
#endif
_currn->_prod = RULErule_051;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_051: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_051: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_051: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_051: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_051;
return ( (NODEPTR) _currn);
}/* Mkrule_051 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_052 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_052 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_052 _currn;
#ifdef __cplusplus
_currn = new _TPrule_052;
#else
_currn = (_TPPrule_052) TreeNodeAlloc (sizeof (struct _TPrule_052));
#endif
_currn->_prod = RULErule_052;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_052: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_052: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_052: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_052: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_052: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_052;
return ( (NODEPTR) _currn);
}/* Mkrule_052 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_053 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_053 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_053 _currn;
#ifdef __cplusplus
_currn = new _TPrule_053;
#else
_currn = (_TPPrule_053) TreeNodeAlloc (sizeof (struct _TPrule_053));
#endif
_currn->_prod = RULErule_053;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_053: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_053: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_053;
return ( (NODEPTR) _currn);
}/* Mkrule_053 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_054 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_054 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_054 _currn;
#ifdef __cplusplus
_currn = new _TPrule_054;
#else
_currn = (_TPPrule_054) TreeNodeAlloc (sizeof (struct _TPrule_054));
#endif
_currn->_prod = RULErule_054;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_054: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_054: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_054: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_054;
return ( (NODEPTR) _currn);
}/* Mkrule_054 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_055 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_055 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_055 _currn;
#ifdef __cplusplus
_currn = new _TPrule_055;
#else
_currn = (_TPPrule_055) TreeNodeAlloc (sizeof (struct _TPrule_055));
#endif
_currn->_prod = RULErule_055;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_055: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_055: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_055: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_055;
return ( (NODEPTR) _currn);
}/* Mkrule_055 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_056 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_056 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_056 _currn;
#ifdef __cplusplus
_currn = new _TPrule_056;
#else
_currn = (_TPPrule_056) TreeNodeAlloc (sizeof (struct _TPrule_056));
#endif
_currn->_prod = RULErule_056;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_056: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_056: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_056: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_056: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_056;
return ( (NODEPTR) _currn);
}/* Mkrule_056 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_057 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_057 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_057 _currn;
#ifdef __cplusplus
_currn = new _TPrule_057;
#else
_currn = (_TPPrule_057) TreeNodeAlloc (sizeof (struct _TPrule_057));
#endif
_currn->_prod = RULErule_057;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_057: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_057: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_057: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_057;
return ( (NODEPTR) _currn);
}/* Mkrule_057 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_058 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_058 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_058 _currn;
#ifdef __cplusplus
_currn = new _TPrule_058;
#else
_currn = (_TPPrule_058) TreeNodeAlloc (sizeof (struct _TPrule_058));
#endif
_currn->_prod = RULErule_058;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_058: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_058: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_058: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_058: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_058;
return ( (NODEPTR) _currn);
}/* Mkrule_058 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_059 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_059 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_059 _currn;
#ifdef __cplusplus
_currn = new _TPrule_059;
#else
_currn = (_TPPrule_059) TreeNodeAlloc (sizeof (struct _TPrule_059));
#endif
_currn->_prod = RULErule_059;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_059: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_059: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_059: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_059: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_059;
return ( (NODEPTR) _currn);
}/* Mkrule_059 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_060 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_060 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_060 _currn;
#ifdef __cplusplus
_currn = new _TPrule_060;
#else
_currn = (_TPPrule_060) TreeNodeAlloc (sizeof (struct _TPrule_060));
#endif
_currn->_prod = RULErule_060;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_060: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_060: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_060: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_060: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_060: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_060;
return ( (NODEPTR) _currn);
}/* Mkrule_060 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_061 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_061 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_061 _currn;
#ifdef __cplusplus
_currn = new _TPrule_061;
#else
_currn = (_TPPrule_061) TreeNodeAlloc (sizeof (struct _TPrule_061));
#endif
_currn->_prod = RULErule_061;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_061: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_061: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_061: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_061;
return ( (NODEPTR) _currn);
}/* Mkrule_061 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_062 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_062 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_062 _currn;
#ifdef __cplusplus
_currn = new _TPrule_062;
#else
_currn = (_TPPrule_062) TreeNodeAlloc (sizeof (struct _TPrule_062));
#endif
_currn->_prod = RULErule_062;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_062: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_062: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_062: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_062: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_062;
return ( (NODEPTR) _currn);
}/* Mkrule_062 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_063 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_063 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_063 _currn;
#ifdef __cplusplus
_currn = new _TPrule_063;
#else
_currn = (_TPPrule_063) TreeNodeAlloc (sizeof (struct _TPrule_063));
#endif
_currn->_prod = RULErule_063;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_063: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_063: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_063: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_063: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_063;
return ( (NODEPTR) _currn);
}/* Mkrule_063 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_064 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_064 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_064 _currn;
#ifdef __cplusplus
_currn = new _TPrule_064;
#else
_currn = (_TPPrule_064) TreeNodeAlloc (sizeof (struct _TPrule_064));
#endif
_currn->_prod = RULErule_064;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_064: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_064: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_064: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_064: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_064: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_064;
return ( (NODEPTR) _currn);
}/* Mkrule_064 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_065 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_065 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_065 _currn;
#ifdef __cplusplus
_currn = new _TPrule_065;
#else
_currn = (_TPPrule_065) TreeNodeAlloc (sizeof (struct _TPrule_065));
#endif
_currn->_prod = RULErule_065;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_065: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_065: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_065: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_065: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_065;
return ( (NODEPTR) _currn);
}/* Mkrule_065 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_066 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_066 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_066 _currn;
#ifdef __cplusplus
_currn = new _TPrule_066;
#else
_currn = (_TPPrule_066) TreeNodeAlloc (sizeof (struct _TPrule_066));
#endif
_currn->_prod = RULErule_066;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_066: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_066: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_066: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_066: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_066: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_066;
return ( (NODEPTR) _currn);
}/* Mkrule_066 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_067 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_067 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_067 _currn;
#ifdef __cplusplus
_currn = new _TPrule_067;
#else
_currn = (_TPPrule_067) TreeNodeAlloc (sizeof (struct _TPrule_067));
#endif
_currn->_prod = RULErule_067;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_067: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_067: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_067: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_067: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_067: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_067;
return ( (NODEPTR) _currn);
}/* Mkrule_067 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_068 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_068 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_068 _currn;
#ifdef __cplusplus
_currn = new _TPrule_068;
#else
_currn = (_TPPrule_068) TreeNodeAlloc (sizeof (struct _TPrule_068));
#endif
_currn->_prod = RULErule_068;
_currn->_desc1 = (_TSPTransitions) MkTransitions (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_068: root of subtree no. 1 can not be made a Transitions node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_068: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_068: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_068: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_068: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_068: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_068;
return ( (NODEPTR) _currn);
}/* Mkrule_068 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_069 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_069 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_069 _currn;
#ifdef __cplusplus
_currn = new _TPrule_069;
#else
_currn = (_TPPrule_069) TreeNodeAlloc (sizeof (struct _TPrule_069));
#endif
_currn->_prod = RULErule_069;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_069: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_069;
return ( (NODEPTR) _currn);
}/* Mkrule_069 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_070 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_070 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_070 _currn;
#ifdef __cplusplus
_currn = new _TPrule_070;
#else
_currn = (_TPPrule_070) TreeNodeAlloc (sizeof (struct _TPrule_070));
#endif
_currn->_prod = RULErule_070;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_070: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_070: root of subtree no. 2 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_070;
return ( (NODEPTR) _currn);
}/* Mkrule_070 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_071 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_071 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_071 _currn;
#ifdef __cplusplus
_currn = new _TPrule_071;
#else
_currn = (_TPPrule_071) TreeNodeAlloc (sizeof (struct _TPrule_071));
#endif
_currn->_prod = RULErule_071;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_071: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasures) MkMeasures (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_071: root of subtree no. 2 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_071;
return ( (NODEPTR) _currn);
}/* Mkrule_071 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_072 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_072 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_072 _currn;
#ifdef __cplusplus
_currn = new _TPrule_072;
#else
_currn = (_TPPrule_072) TreeNodeAlloc (sizeof (struct _TPrule_072));
#endif
_currn->_prod = RULErule_072;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_072: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasures) MkMeasures (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_072: root of subtree no. 2 can not be made a Measures node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_072: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_072;
return ( (NODEPTR) _currn);
}/* Mkrule_072 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_073 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_073 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_073 _currn;
#ifdef __cplusplus
_currn = new _TPrule_073;
#else
_currn = (_TPPrule_073) TreeNodeAlloc (sizeof (struct _TPrule_073));
#endif
_currn->_prod = RULErule_073;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_073: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_073: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_073;
return ( (NODEPTR) _currn);
}/* Mkrule_073 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_074 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_074 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_074 _currn;
#ifdef __cplusplus
_currn = new _TPrule_074;
#else
_currn = (_TPPrule_074) TreeNodeAlloc (sizeof (struct _TPrule_074));
#endif
_currn->_prod = RULErule_074;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_074: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_074: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_074: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_074;
return ( (NODEPTR) _currn);
}/* Mkrule_074 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_075 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_075 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_075 _currn;
#ifdef __cplusplus
_currn = new _TPrule_075;
#else
_currn = (_TPPrule_075) TreeNodeAlloc (sizeof (struct _TPrule_075));
#endif
_currn->_prod = RULErule_075;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_075: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_075: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_075: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_075;
return ( (NODEPTR) _currn);
}/* Mkrule_075 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_076 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_076 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_076 _currn;
#ifdef __cplusplus
_currn = new _TPrule_076;
#else
_currn = (_TPPrule_076) TreeNodeAlloc (sizeof (struct _TPrule_076));
#endif
_currn->_prod = RULErule_076;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_076: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_076: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_076: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_076: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_076;
return ( (NODEPTR) _currn);
}/* Mkrule_076 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_077 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_077 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_077 _currn;
#ifdef __cplusplus
_currn = new _TPrule_077;
#else
_currn = (_TPPrule_077) TreeNodeAlloc (sizeof (struct _TPrule_077));
#endif
_currn->_prod = RULErule_077;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_077: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_077: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_077;
return ( (NODEPTR) _currn);
}/* Mkrule_077 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_078 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_078 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_078 _currn;
#ifdef __cplusplus
_currn = new _TPrule_078;
#else
_currn = (_TPPrule_078) TreeNodeAlloc (sizeof (struct _TPrule_078));
#endif
_currn->_prod = RULErule_078;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_078: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_078: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_078: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_078;
return ( (NODEPTR) _currn);
}/* Mkrule_078 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_079 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_079 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_079 _currn;
#ifdef __cplusplus
_currn = new _TPrule_079;
#else
_currn = (_TPPrule_079) TreeNodeAlloc (sizeof (struct _TPrule_079));
#endif
_currn->_prod = RULErule_079;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_079: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_079: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_079: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_079;
return ( (NODEPTR) _currn);
}/* Mkrule_079 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_080 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_080 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_080 _currn;
#ifdef __cplusplus
_currn = new _TPrule_080;
#else
_currn = (_TPPrule_080) TreeNodeAlloc (sizeof (struct _TPrule_080));
#endif
_currn->_prod = RULErule_080;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_080: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_080: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_080: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_080: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_080;
return ( (NODEPTR) _currn);
}/* Mkrule_080 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_081 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_081 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_081 _currn;
#ifdef __cplusplus
_currn = new _TPrule_081;
#else
_currn = (_TPPrule_081) TreeNodeAlloc (sizeof (struct _TPrule_081));
#endif
_currn->_prod = RULErule_081;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_081: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_081: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_081: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_081;
return ( (NODEPTR) _currn);
}/* Mkrule_081 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_082 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_082 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_082 _currn;
#ifdef __cplusplus
_currn = new _TPrule_082;
#else
_currn = (_TPPrule_082) TreeNodeAlloc (sizeof (struct _TPrule_082));
#endif
_currn->_prod = RULErule_082;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_082: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_082: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_082: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_082: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_082;
return ( (NODEPTR) _currn);
}/* Mkrule_082 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_083 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_083 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_083 _currn;
#ifdef __cplusplus
_currn = new _TPrule_083;
#else
_currn = (_TPPrule_083) TreeNodeAlloc (sizeof (struct _TPrule_083));
#endif
_currn->_prod = RULErule_083;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_083: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_083: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_083: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_083: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_083;
return ( (NODEPTR) _currn);
}/* Mkrule_083 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_084 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_084 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_084 _currn;
#ifdef __cplusplus
_currn = new _TPrule_084;
#else
_currn = (_TPPrule_084) TreeNodeAlloc (sizeof (struct _TPrule_084));
#endif
_currn->_prod = RULErule_084;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_084: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_084: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_084: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_084: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_084: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_084;
return ( (NODEPTR) _currn);
}/* Mkrule_084 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_085 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_085 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_085 _currn;
#ifdef __cplusplus
_currn = new _TPrule_085;
#else
_currn = (_TPPrule_085) TreeNodeAlloc (sizeof (struct _TPrule_085));
#endif
_currn->_prod = RULErule_085;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_085: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_085: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_085;
return ( (NODEPTR) _currn);
}/* Mkrule_085 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_086 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_086 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_086 _currn;
#ifdef __cplusplus
_currn = new _TPrule_086;
#else
_currn = (_TPPrule_086) TreeNodeAlloc (sizeof (struct _TPrule_086));
#endif
_currn->_prod = RULErule_086;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_086: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_086: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_086: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_086;
return ( (NODEPTR) _currn);
}/* Mkrule_086 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_087 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_087 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_087 _currn;
#ifdef __cplusplus
_currn = new _TPrule_087;
#else
_currn = (_TPPrule_087) TreeNodeAlloc (sizeof (struct _TPrule_087));
#endif
_currn->_prod = RULErule_087;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_087: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_087: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_087: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_087;
return ( (NODEPTR) _currn);
}/* Mkrule_087 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_088 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_088 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_088 _currn;
#ifdef __cplusplus
_currn = new _TPrule_088;
#else
_currn = (_TPPrule_088) TreeNodeAlloc (sizeof (struct _TPrule_088));
#endif
_currn->_prod = RULErule_088;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_088: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_088: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_088: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_088: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_088;
return ( (NODEPTR) _currn);
}/* Mkrule_088 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_089 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_089 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_089 _currn;
#ifdef __cplusplus
_currn = new _TPrule_089;
#else
_currn = (_TPPrule_089) TreeNodeAlloc (sizeof (struct _TPrule_089));
#endif
_currn->_prod = RULErule_089;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_089: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_089: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_089: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_089;
return ( (NODEPTR) _currn);
}/* Mkrule_089 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_090 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_090 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_090 _currn;
#ifdef __cplusplus
_currn = new _TPrule_090;
#else
_currn = (_TPPrule_090) TreeNodeAlloc (sizeof (struct _TPrule_090));
#endif
_currn->_prod = RULErule_090;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_090: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_090: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_090: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_090: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_090;
return ( (NODEPTR) _currn);
}/* Mkrule_090 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_091 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_091 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_091 _currn;
#ifdef __cplusplus
_currn = new _TPrule_091;
#else
_currn = (_TPPrule_091) TreeNodeAlloc (sizeof (struct _TPrule_091));
#endif
_currn->_prod = RULErule_091;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_091: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_091: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_091: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_091: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_091;
return ( (NODEPTR) _currn);
}/* Mkrule_091 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_092 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_092 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_092 _currn;
#ifdef __cplusplus
_currn = new _TPrule_092;
#else
_currn = (_TPPrule_092) TreeNodeAlloc (sizeof (struct _TPrule_092));
#endif
_currn->_prod = RULErule_092;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_092: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_092: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_092: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_092: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_092: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_092;
return ( (NODEPTR) _currn);
}/* Mkrule_092 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_093 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_093 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_093 _currn;
#ifdef __cplusplus
_currn = new _TPrule_093;
#else
_currn = (_TPPrule_093) TreeNodeAlloc (sizeof (struct _TPrule_093));
#endif
_currn->_prod = RULErule_093;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_093: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_093: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_093: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_093;
return ( (NODEPTR) _currn);
}/* Mkrule_093 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_094 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_094 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_094 _currn;
#ifdef __cplusplus
_currn = new _TPrule_094;
#else
_currn = (_TPPrule_094) TreeNodeAlloc (sizeof (struct _TPrule_094));
#endif
_currn->_prod = RULErule_094;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_094: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_094: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_094: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_094: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_094;
return ( (NODEPTR) _currn);
}/* Mkrule_094 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_095 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_095 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_095 _currn;
#ifdef __cplusplus
_currn = new _TPrule_095;
#else
_currn = (_TPPrule_095) TreeNodeAlloc (sizeof (struct _TPrule_095));
#endif
_currn->_prod = RULErule_095;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_095: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_095: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_095: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_095: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_095;
return ( (NODEPTR) _currn);
}/* Mkrule_095 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_096 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_096 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_096 _currn;
#ifdef __cplusplus
_currn = new _TPrule_096;
#else
_currn = (_TPPrule_096) TreeNodeAlloc (sizeof (struct _TPrule_096));
#endif
_currn->_prod = RULErule_096;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_096: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_096: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_096: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_096: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_096: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_096;
return ( (NODEPTR) _currn);
}/* Mkrule_096 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_097 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_097 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_097 _currn;
#ifdef __cplusplus
_currn = new _TPrule_097;
#else
_currn = (_TPPrule_097) TreeNodeAlloc (sizeof (struct _TPrule_097));
#endif
_currn->_prod = RULErule_097;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_097: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_097: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_097: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_097: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_097;
return ( (NODEPTR) _currn);
}/* Mkrule_097 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_098 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_098 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_098 _currn;
#ifdef __cplusplus
_currn = new _TPrule_098;
#else
_currn = (_TPPrule_098) TreeNodeAlloc (sizeof (struct _TPrule_098));
#endif
_currn->_prod = RULErule_098;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_098: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_098: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_098: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_098: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_098: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_098;
return ( (NODEPTR) _currn);
}/* Mkrule_098 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_099 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_099 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_099 _currn;
#ifdef __cplusplus
_currn = new _TPrule_099;
#else
_currn = (_TPPrule_099) TreeNodeAlloc (sizeof (struct _TPrule_099));
#endif
_currn->_prod = RULErule_099;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_099: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_099: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_099: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_099: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_099: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_099;
return ( (NODEPTR) _currn);
}/* Mkrule_099 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_100 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_100 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_100 _currn;
#ifdef __cplusplus
_currn = new _TPrule_100;
#else
_currn = (_TPPrule_100) TreeNodeAlloc (sizeof (struct _TPrule_100));
#endif
_currn->_prod = RULErule_100;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_100: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_100: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_100: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_100: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_100: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_100: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_100;
return ( (NODEPTR) _currn);
}/* Mkrule_100 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_101 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_101 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_101 _currn;
#ifdef __cplusplus
_currn = new _TPrule_101;
#else
_currn = (_TPPrule_101) TreeNodeAlloc (sizeof (struct _TPrule_101));
#endif
_currn->_prod = RULErule_101;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_101: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_101: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_101;
return ( (NODEPTR) _currn);
}/* Mkrule_101 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_102 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_102 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_102 _currn;
#ifdef __cplusplus
_currn = new _TPrule_102;
#else
_currn = (_TPPrule_102) TreeNodeAlloc (sizeof (struct _TPrule_102));
#endif
_currn->_prod = RULErule_102;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_102: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_102: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_102: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_102;
return ( (NODEPTR) _currn);
}/* Mkrule_102 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_103 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_103 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_103 _currn;
#ifdef __cplusplus
_currn = new _TPrule_103;
#else
_currn = (_TPPrule_103) TreeNodeAlloc (sizeof (struct _TPrule_103));
#endif
_currn->_prod = RULErule_103;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_103: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_103: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_103: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_103;
return ( (NODEPTR) _currn);
}/* Mkrule_103 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_104 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_104 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_104 _currn;
#ifdef __cplusplus
_currn = new _TPrule_104;
#else
_currn = (_TPPrule_104) TreeNodeAlloc (sizeof (struct _TPrule_104));
#endif
_currn->_prod = RULErule_104;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_104: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_104: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_104: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_104: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_104;
return ( (NODEPTR) _currn);
}/* Mkrule_104 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_105 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_105 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_105 _currn;
#ifdef __cplusplus
_currn = new _TPrule_105;
#else
_currn = (_TPPrule_105) TreeNodeAlloc (sizeof (struct _TPrule_105));
#endif
_currn->_prod = RULErule_105;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_105: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_105: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_105: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_105;
return ( (NODEPTR) _currn);
}/* Mkrule_105 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_106 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_106 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_106 _currn;
#ifdef __cplusplus
_currn = new _TPrule_106;
#else
_currn = (_TPPrule_106) TreeNodeAlloc (sizeof (struct _TPrule_106));
#endif
_currn->_prod = RULErule_106;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_106: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_106: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_106: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_106: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_106;
return ( (NODEPTR) _currn);
}/* Mkrule_106 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_107 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_107 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_107 _currn;
#ifdef __cplusplus
_currn = new _TPrule_107;
#else
_currn = (_TPPrule_107) TreeNodeAlloc (sizeof (struct _TPrule_107));
#endif
_currn->_prod = RULErule_107;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_107: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_107: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_107: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_107: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_107;
return ( (NODEPTR) _currn);
}/* Mkrule_107 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_108 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_108 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_108 _currn;
#ifdef __cplusplus
_currn = new _TPrule_108;
#else
_currn = (_TPPrule_108) TreeNodeAlloc (sizeof (struct _TPrule_108));
#endif
_currn->_prod = RULErule_108;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_108: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_108: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_108: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_108: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_108: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_108;
return ( (NODEPTR) _currn);
}/* Mkrule_108 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_109 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_109 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_109 _currn;
#ifdef __cplusplus
_currn = new _TPrule_109;
#else
_currn = (_TPPrule_109) TreeNodeAlloc (sizeof (struct _TPrule_109));
#endif
_currn->_prod = RULErule_109;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_109: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_109: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_109: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_109;
return ( (NODEPTR) _currn);
}/* Mkrule_109 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_110 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_110 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_110 _currn;
#ifdef __cplusplus
_currn = new _TPrule_110;
#else
_currn = (_TPPrule_110) TreeNodeAlloc (sizeof (struct _TPrule_110));
#endif
_currn->_prod = RULErule_110;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_110: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_110: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_110: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_110: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_110;
return ( (NODEPTR) _currn);
}/* Mkrule_110 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_111 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_111 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_111 _currn;
#ifdef __cplusplus
_currn = new _TPrule_111;
#else
_currn = (_TPPrule_111) TreeNodeAlloc (sizeof (struct _TPrule_111));
#endif
_currn->_prod = RULErule_111;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_111: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_111: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_111: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_111: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_111;
return ( (NODEPTR) _currn);
}/* Mkrule_111 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_112 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_112 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_112 _currn;
#ifdef __cplusplus
_currn = new _TPrule_112;
#else
_currn = (_TPPrule_112) TreeNodeAlloc (sizeof (struct _TPrule_112));
#endif
_currn->_prod = RULErule_112;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_112: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_112: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_112: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_112: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_112: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_112;
return ( (NODEPTR) _currn);
}/* Mkrule_112 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_113 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_113 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_113 _currn;
#ifdef __cplusplus
_currn = new _TPrule_113;
#else
_currn = (_TPPrule_113) TreeNodeAlloc (sizeof (struct _TPrule_113));
#endif
_currn->_prod = RULErule_113;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_113: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_113: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_113: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_113: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_113;
return ( (NODEPTR) _currn);
}/* Mkrule_113 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_114 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_114 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_114 _currn;
#ifdef __cplusplus
_currn = new _TPrule_114;
#else
_currn = (_TPPrule_114) TreeNodeAlloc (sizeof (struct _TPrule_114));
#endif
_currn->_prod = RULErule_114;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_114: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_114: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_114: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_114: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_114: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_114;
return ( (NODEPTR) _currn);
}/* Mkrule_114 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_115 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_115 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_115 _currn;
#ifdef __cplusplus
_currn = new _TPrule_115;
#else
_currn = (_TPPrule_115) TreeNodeAlloc (sizeof (struct _TPrule_115));
#endif
_currn->_prod = RULErule_115;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_115: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_115: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_115: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_115: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_115: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_115;
return ( (NODEPTR) _currn);
}/* Mkrule_115 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_116 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_116 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_116 _currn;
#ifdef __cplusplus
_currn = new _TPrule_116;
#else
_currn = (_TPPrule_116) TreeNodeAlloc (sizeof (struct _TPrule_116));
#endif
_currn->_prod = RULErule_116;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_116: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_116: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_116: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_116: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_116: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_116: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_116;
return ( (NODEPTR) _currn);
}/* Mkrule_116 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_117 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_117 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_117 _currn;
#ifdef __cplusplus
_currn = new _TPrule_117;
#else
_currn = (_TPPrule_117) TreeNodeAlloc (sizeof (struct _TPrule_117));
#endif
_currn->_prod = RULErule_117;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_117: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_117: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_117: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_117;
return ( (NODEPTR) _currn);
}/* Mkrule_117 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_118 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_118 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_118 _currn;
#ifdef __cplusplus
_currn = new _TPrule_118;
#else
_currn = (_TPPrule_118) TreeNodeAlloc (sizeof (struct _TPrule_118));
#endif
_currn->_prod = RULErule_118;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_118: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_118: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_118: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_118: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_118;
return ( (NODEPTR) _currn);
}/* Mkrule_118 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_119 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_119 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_119 _currn;
#ifdef __cplusplus
_currn = new _TPrule_119;
#else
_currn = (_TPPrule_119) TreeNodeAlloc (sizeof (struct _TPrule_119));
#endif
_currn->_prod = RULErule_119;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_119: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_119: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_119: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_119: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_119;
return ( (NODEPTR) _currn);
}/* Mkrule_119 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_120 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_120 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_120 _currn;
#ifdef __cplusplus
_currn = new _TPrule_120;
#else
_currn = (_TPPrule_120) TreeNodeAlloc (sizeof (struct _TPrule_120));
#endif
_currn->_prod = RULErule_120;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_120: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_120: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_120: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_120: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_120: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_120;
return ( (NODEPTR) _currn);
}/* Mkrule_120 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_121 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_121 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_121 _currn;
#ifdef __cplusplus
_currn = new _TPrule_121;
#else
_currn = (_TPPrule_121) TreeNodeAlloc (sizeof (struct _TPrule_121));
#endif
_currn->_prod = RULErule_121;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_121: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_121: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_121: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_121: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_121;
return ( (NODEPTR) _currn);
}/* Mkrule_121 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_122 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_122 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_122 _currn;
#ifdef __cplusplus
_currn = new _TPrule_122;
#else
_currn = (_TPPrule_122) TreeNodeAlloc (sizeof (struct _TPrule_122));
#endif
_currn->_prod = RULErule_122;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_122: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_122: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_122: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_122: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_122: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_122;
return ( (NODEPTR) _currn);
}/* Mkrule_122 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_123 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_123 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_123 _currn;
#ifdef __cplusplus
_currn = new _TPrule_123;
#else
_currn = (_TPPrule_123) TreeNodeAlloc (sizeof (struct _TPrule_123));
#endif
_currn->_prod = RULErule_123;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_123: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_123: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_123: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_123: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_123: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_123;
return ( (NODEPTR) _currn);
}/* Mkrule_123 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_124 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_124 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_124 _currn;
#ifdef __cplusplus
_currn = new _TPrule_124;
#else
_currn = (_TPPrule_124) TreeNodeAlloc (sizeof (struct _TPrule_124));
#endif
_currn->_prod = RULErule_124;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_124: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_124: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_124: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_124: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_124: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_124: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_124;
return ( (NODEPTR) _currn);
}/* Mkrule_124 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_125 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_125 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_125 _currn;
#ifdef __cplusplus
_currn = new _TPrule_125;
#else
_currn = (_TPPrule_125) TreeNodeAlloc (sizeof (struct _TPrule_125));
#endif
_currn->_prod = RULErule_125;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_125: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_125: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_125: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_125: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_125;
return ( (NODEPTR) _currn);
}/* Mkrule_125 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_126 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_126 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_126 _currn;
#ifdef __cplusplus
_currn = new _TPrule_126;
#else
_currn = (_TPPrule_126) TreeNodeAlloc (sizeof (struct _TPrule_126));
#endif
_currn->_prod = RULErule_126;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_126: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_126: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_126: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_126: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_126: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_126;
return ( (NODEPTR) _currn);
}/* Mkrule_126 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_127 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_127 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_127 _currn;
#ifdef __cplusplus
_currn = new _TPrule_127;
#else
_currn = (_TPPrule_127) TreeNodeAlloc (sizeof (struct _TPrule_127));
#endif
_currn->_prod = RULErule_127;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_127: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_127: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_127: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_127: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_127: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_127;
return ( (NODEPTR) _currn);
}/* Mkrule_127 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_128 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_128 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_128 _currn;
#ifdef __cplusplus
_currn = new _TPrule_128;
#else
_currn = (_TPPrule_128) TreeNodeAlloc (sizeof (struct _TPrule_128));
#endif
_currn->_prod = RULErule_128;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_128: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_128: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_128: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_128: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_128: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_128: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_128;
return ( (NODEPTR) _currn);
}/* Mkrule_128 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_129 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_129 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_129 _currn;
#ifdef __cplusplus
_currn = new _TPrule_129;
#else
_currn = (_TPPrule_129) TreeNodeAlloc (sizeof (struct _TPrule_129));
#endif
_currn->_prod = RULErule_129;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_129: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_129: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_129: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_129: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_129: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_129;
return ( (NODEPTR) _currn);
}/* Mkrule_129 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_130 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_130 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_130 _currn;
#ifdef __cplusplus
_currn = new _TPrule_130;
#else
_currn = (_TPPrule_130) TreeNodeAlloc (sizeof (struct _TPrule_130));
#endif
_currn->_prod = RULErule_130;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_130: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_130: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_130: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_130: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_130: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_130: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_130;
return ( (NODEPTR) _currn);
}/* Mkrule_130 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_131 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_131 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_131 _currn;
#ifdef __cplusplus
_currn = new _TPrule_131;
#else
_currn = (_TPPrule_131) TreeNodeAlloc (sizeof (struct _TPrule_131));
#endif
_currn->_prod = RULErule_131;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_131: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_131: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_131: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_131: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_131: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasures) MkMeasures (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_131: root of subtree no. 6 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_131;
return ( (NODEPTR) _currn);
}/* Mkrule_131 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_132 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7)
#else
NODEPTR Mkrule_132 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
#endif
{	_TPPrule_132 _currn;
#ifdef __cplusplus
_currn = new _TPrule_132;
#else
_currn = (_TPPrule_132) TreeNodeAlloc (sizeof (struct _TPrule_132));
#endif
_currn->_prod = RULErule_132;
_currn->_desc1 = (_TSPPlaces) MkPlaces (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_132: root of subtree no. 1 can not be made a Places node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_132: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_132: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_132: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_132: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasures) MkMeasures (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_132: root of subtree no. 6 can not be made a Measures node ", 0, _coordref);
_currn->_desc7 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_132: root of subtree no. 7 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_132;
return ( (NODEPTR) _currn);
}/* Mkrule_132 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_133 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_133 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_133 _currn;
#ifdef __cplusplus
_currn = new _TPrule_133;
#else
_currn = (_TPPrule_133) TreeNodeAlloc (sizeof (struct _TPrule_133));
#endif
_currn->_prod = RULErule_133;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_133: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_133;
return ( (NODEPTR) _currn);
}/* Mkrule_133 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_134 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_134 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_134 _currn;
#ifdef __cplusplus
_currn = new _TPrule_134;
#else
_currn = (_TPPrule_134) TreeNodeAlloc (sizeof (struct _TPrule_134));
#endif
_currn->_prod = RULErule_134;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_134: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_134: root of subtree no. 2 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_134;
return ( (NODEPTR) _currn);
}/* Mkrule_134 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_135 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_135 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_135 _currn;
#ifdef __cplusplus
_currn = new _TPrule_135;
#else
_currn = (_TPPrule_135) TreeNodeAlloc (sizeof (struct _TPrule_135));
#endif
_currn->_prod = RULErule_135;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_135: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasures) MkMeasures (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_135: root of subtree no. 2 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_135;
return ( (NODEPTR) _currn);
}/* Mkrule_135 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_136 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_136 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_136 _currn;
#ifdef __cplusplus
_currn = new _TPrule_136;
#else
_currn = (_TPPrule_136) TreeNodeAlloc (sizeof (struct _TPrule_136));
#endif
_currn->_prod = RULErule_136;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_136: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasures) MkMeasures (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_136: root of subtree no. 2 can not be made a Measures node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_136: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_136;
return ( (NODEPTR) _currn);
}/* Mkrule_136 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_137 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_137 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_137 _currn;
#ifdef __cplusplus
_currn = new _TPrule_137;
#else
_currn = (_TPPrule_137) TreeNodeAlloc (sizeof (struct _TPrule_137));
#endif
_currn->_prod = RULErule_137;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_137: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_137: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_137;
return ( (NODEPTR) _currn);
}/* Mkrule_137 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_138 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_138 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_138 _currn;
#ifdef __cplusplus
_currn = new _TPrule_138;
#else
_currn = (_TPPrule_138) TreeNodeAlloc (sizeof (struct _TPrule_138));
#endif
_currn->_prod = RULErule_138;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_138: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_138: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_138: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_138;
return ( (NODEPTR) _currn);
}/* Mkrule_138 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_139 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_139 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_139 _currn;
#ifdef __cplusplus
_currn = new _TPrule_139;
#else
_currn = (_TPPrule_139) TreeNodeAlloc (sizeof (struct _TPrule_139));
#endif
_currn->_prod = RULErule_139;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_139: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_139: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_139: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_139;
return ( (NODEPTR) _currn);
}/* Mkrule_139 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_140 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_140 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_140 _currn;
#ifdef __cplusplus
_currn = new _TPrule_140;
#else
_currn = (_TPPrule_140) TreeNodeAlloc (sizeof (struct _TPrule_140));
#endif
_currn->_prod = RULErule_140;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_140: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPRewards) MkRewards (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_140: root of subtree no. 2 can not be made a Rewards node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_140: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_140: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_140;
return ( (NODEPTR) _currn);
}/* Mkrule_140 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_141 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_141 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_141 _currn;
#ifdef __cplusplus
_currn = new _TPrule_141;
#else
_currn = (_TPPrule_141) TreeNodeAlloc (sizeof (struct _TPrule_141));
#endif
_currn->_prod = RULErule_141;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_141: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_141: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_141;
return ( (NODEPTR) _currn);
}/* Mkrule_141 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_142 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_142 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_142 _currn;
#ifdef __cplusplus
_currn = new _TPrule_142;
#else
_currn = (_TPPrule_142) TreeNodeAlloc (sizeof (struct _TPrule_142));
#endif
_currn->_prod = RULErule_142;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_142: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_142: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_142: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_142;
return ( (NODEPTR) _currn);
}/* Mkrule_142 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_143 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_143 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_143 _currn;
#ifdef __cplusplus
_currn = new _TPrule_143;
#else
_currn = (_TPPrule_143) TreeNodeAlloc (sizeof (struct _TPrule_143));
#endif
_currn->_prod = RULErule_143;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_143: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_143: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_143: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_143;
return ( (NODEPTR) _currn);
}/* Mkrule_143 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_144 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_144 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_144 _currn;
#ifdef __cplusplus
_currn = new _TPrule_144;
#else
_currn = (_TPPrule_144) TreeNodeAlloc (sizeof (struct _TPrule_144));
#endif
_currn->_prod = RULErule_144;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_144: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_144: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_144: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_144: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_144;
return ( (NODEPTR) _currn);
}/* Mkrule_144 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_145 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_145 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_145 _currn;
#ifdef __cplusplus
_currn = new _TPrule_145;
#else
_currn = (_TPPrule_145) TreeNodeAlloc (sizeof (struct _TPrule_145));
#endif
_currn->_prod = RULErule_145;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_145: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_145: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_145: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_145;
return ( (NODEPTR) _currn);
}/* Mkrule_145 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_146 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_146 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_146 _currn;
#ifdef __cplusplus
_currn = new _TPrule_146;
#else
_currn = (_TPPrule_146) TreeNodeAlloc (sizeof (struct _TPrule_146));
#endif
_currn->_prod = RULErule_146;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_146: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_146: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_146: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_146: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_146;
return ( (NODEPTR) _currn);
}/* Mkrule_146 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_147 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_147 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_147 _currn;
#ifdef __cplusplus
_currn = new _TPrule_147;
#else
_currn = (_TPPrule_147) TreeNodeAlloc (sizeof (struct _TPrule_147));
#endif
_currn->_prod = RULErule_147;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_147: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_147: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_147: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_147: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_147;
return ( (NODEPTR) _currn);
}/* Mkrule_147 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_148 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_148 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_148 _currn;
#ifdef __cplusplus
_currn = new _TPrule_148;
#else
_currn = (_TPPrule_148) TreeNodeAlloc (sizeof (struct _TPrule_148));
#endif
_currn->_prod = RULErule_148;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_148: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPArcs) MkArcs (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_148: root of subtree no. 2 can not be made a Arcs node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_148: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_148: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_148: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_148;
return ( (NODEPTR) _currn);
}/* Mkrule_148 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_149 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_149 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_149 _currn;
#ifdef __cplusplus
_currn = new _TPrule_149;
#else
_currn = (_TPPrule_149) TreeNodeAlloc (sizeof (struct _TPrule_149));
#endif
_currn->_prod = RULErule_149;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_149: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_149: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_149;
return ( (NODEPTR) _currn);
}/* Mkrule_149 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_150 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_150 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_150 _currn;
#ifdef __cplusplus
_currn = new _TPrule_150;
#else
_currn = (_TPPrule_150) TreeNodeAlloc (sizeof (struct _TPrule_150));
#endif
_currn->_prod = RULErule_150;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_150: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_150: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_150: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_150;
return ( (NODEPTR) _currn);
}/* Mkrule_150 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_151 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_151 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_151 _currn;
#ifdef __cplusplus
_currn = new _TPrule_151;
#else
_currn = (_TPPrule_151) TreeNodeAlloc (sizeof (struct _TPrule_151));
#endif
_currn->_prod = RULErule_151;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_151: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_151: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_151: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_151;
return ( (NODEPTR) _currn);
}/* Mkrule_151 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_152 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_152 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_152 _currn;
#ifdef __cplusplus
_currn = new _TPrule_152;
#else
_currn = (_TPPrule_152) TreeNodeAlloc (sizeof (struct _TPrule_152));
#endif
_currn->_prod = RULErule_152;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_152: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_152: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_152: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_152: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_152;
return ( (NODEPTR) _currn);
}/* Mkrule_152 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_153 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_153 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_153 _currn;
#ifdef __cplusplus
_currn = new _TPrule_153;
#else
_currn = (_TPPrule_153) TreeNodeAlloc (sizeof (struct _TPrule_153));
#endif
_currn->_prod = RULErule_153;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_153: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_153: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_153: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_153;
return ( (NODEPTR) _currn);
}/* Mkrule_153 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_154 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_154 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_154 _currn;
#ifdef __cplusplus
_currn = new _TPrule_154;
#else
_currn = (_TPPrule_154) TreeNodeAlloc (sizeof (struct _TPrule_154));
#endif
_currn->_prod = RULErule_154;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_154: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_154: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_154: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_154: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_154;
return ( (NODEPTR) _currn);
}/* Mkrule_154 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_155 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_155 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_155 _currn;
#ifdef __cplusplus
_currn = new _TPrule_155;
#else
_currn = (_TPPrule_155) TreeNodeAlloc (sizeof (struct _TPrule_155));
#endif
_currn->_prod = RULErule_155;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_155: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_155: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_155: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_155: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_155;
return ( (NODEPTR) _currn);
}/* Mkrule_155 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_156 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_156 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_156 _currn;
#ifdef __cplusplus
_currn = new _TPrule_156;
#else
_currn = (_TPPrule_156) TreeNodeAlloc (sizeof (struct _TPrule_156));
#endif
_currn->_prod = RULErule_156;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_156: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_156: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_156: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_156: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_156: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_156;
return ( (NODEPTR) _currn);
}/* Mkrule_156 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_157 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_157 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_157 _currn;
#ifdef __cplusplus
_currn = new _TPrule_157;
#else
_currn = (_TPPrule_157) TreeNodeAlloc (sizeof (struct _TPrule_157));
#endif
_currn->_prod = RULErule_157;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_157: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_157: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_157: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_157;
return ( (NODEPTR) _currn);
}/* Mkrule_157 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_158 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_158 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_158 _currn;
#ifdef __cplusplus
_currn = new _TPrule_158;
#else
_currn = (_TPPrule_158) TreeNodeAlloc (sizeof (struct _TPrule_158));
#endif
_currn->_prod = RULErule_158;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_158: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_158: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_158: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_158: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_158;
return ( (NODEPTR) _currn);
}/* Mkrule_158 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_159 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_159 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_159 _currn;
#ifdef __cplusplus
_currn = new _TPrule_159;
#else
_currn = (_TPPrule_159) TreeNodeAlloc (sizeof (struct _TPrule_159));
#endif
_currn->_prod = RULErule_159;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_159: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_159: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_159: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_159: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_159;
return ( (NODEPTR) _currn);
}/* Mkrule_159 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_160 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_160 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_160 _currn;
#ifdef __cplusplus
_currn = new _TPrule_160;
#else
_currn = (_TPPrule_160) TreeNodeAlloc (sizeof (struct _TPrule_160));
#endif
_currn->_prod = RULErule_160;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_160: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_160: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_160: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_160: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_160: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_160;
return ( (NODEPTR) _currn);
}/* Mkrule_160 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_161 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_161 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_161 _currn;
#ifdef __cplusplus
_currn = new _TPrule_161;
#else
_currn = (_TPPrule_161) TreeNodeAlloc (sizeof (struct _TPrule_161));
#endif
_currn->_prod = RULErule_161;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_161: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_161: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_161: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_161: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_161;
return ( (NODEPTR) _currn);
}/* Mkrule_161 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_162 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_162 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_162 _currn;
#ifdef __cplusplus
_currn = new _TPrule_162;
#else
_currn = (_TPPrule_162) TreeNodeAlloc (sizeof (struct _TPrule_162));
#endif
_currn->_prod = RULErule_162;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_162: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_162: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_162: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_162: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_162: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_162;
return ( (NODEPTR) _currn);
}/* Mkrule_162 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_163 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_163 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_163 _currn;
#ifdef __cplusplus
_currn = new _TPrule_163;
#else
_currn = (_TPPrule_163) TreeNodeAlloc (sizeof (struct _TPrule_163));
#endif
_currn->_prod = RULErule_163;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_163: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_163: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_163: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_163: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_163: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_163;
return ( (NODEPTR) _currn);
}/* Mkrule_163 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_164 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_164 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_164 _currn;
#ifdef __cplusplus
_currn = new _TPrule_164;
#else
_currn = (_TPPrule_164) TreeNodeAlloc (sizeof (struct _TPrule_164));
#endif
_currn->_prod = RULErule_164;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_164: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_164: root of subtree no. 2 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_164: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_164: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_164: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_164: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_164;
return ( (NODEPTR) _currn);
}/* Mkrule_164 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_165 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_165 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_165 _currn;
#ifdef __cplusplus
_currn = new _TPrule_165;
#else
_currn = (_TPPrule_165) TreeNodeAlloc (sizeof (struct _TPrule_165));
#endif
_currn->_prod = RULErule_165;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_165: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_165: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_165;
return ( (NODEPTR) _currn);
}/* Mkrule_165 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_166 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_166 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_166 _currn;
#ifdef __cplusplus
_currn = new _TPrule_166;
#else
_currn = (_TPPrule_166) TreeNodeAlloc (sizeof (struct _TPrule_166));
#endif
_currn->_prod = RULErule_166;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_166: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_166: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_166: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_166;
return ( (NODEPTR) _currn);
}/* Mkrule_166 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_167 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_167 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_167 _currn;
#ifdef __cplusplus
_currn = new _TPrule_167;
#else
_currn = (_TPPrule_167) TreeNodeAlloc (sizeof (struct _TPrule_167));
#endif
_currn->_prod = RULErule_167;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_167: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_167: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_167: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_167;
return ( (NODEPTR) _currn);
}/* Mkrule_167 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_168 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_168 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_168 _currn;
#ifdef __cplusplus
_currn = new _TPrule_168;
#else
_currn = (_TPPrule_168) TreeNodeAlloc (sizeof (struct _TPrule_168));
#endif
_currn->_prod = RULErule_168;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_168: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_168: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_168: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_168: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_168;
return ( (NODEPTR) _currn);
}/* Mkrule_168 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_169 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_169 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_169 _currn;
#ifdef __cplusplus
_currn = new _TPrule_169;
#else
_currn = (_TPPrule_169) TreeNodeAlloc (sizeof (struct _TPrule_169));
#endif
_currn->_prod = RULErule_169;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_169: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_169: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_169: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_169;
return ( (NODEPTR) _currn);
}/* Mkrule_169 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_170 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_170 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_170 _currn;
#ifdef __cplusplus
_currn = new _TPrule_170;
#else
_currn = (_TPPrule_170) TreeNodeAlloc (sizeof (struct _TPrule_170));
#endif
_currn->_prod = RULErule_170;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_170: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_170: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_170: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_170: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_170;
return ( (NODEPTR) _currn);
}/* Mkrule_170 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_171 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_171 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_171 _currn;
#ifdef __cplusplus
_currn = new _TPrule_171;
#else
_currn = (_TPPrule_171) TreeNodeAlloc (sizeof (struct _TPrule_171));
#endif
_currn->_prod = RULErule_171;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_171: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_171: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_171: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_171: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_171;
return ( (NODEPTR) _currn);
}/* Mkrule_171 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_172 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_172 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_172 _currn;
#ifdef __cplusplus
_currn = new _TPrule_172;
#else
_currn = (_TPPrule_172) TreeNodeAlloc (sizeof (struct _TPrule_172));
#endif
_currn->_prod = RULErule_172;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_172: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_172: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_172: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_172: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_172: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_172;
return ( (NODEPTR) _currn);
}/* Mkrule_172 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_173 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_173 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_173 _currn;
#ifdef __cplusplus
_currn = new _TPrule_173;
#else
_currn = (_TPPrule_173) TreeNodeAlloc (sizeof (struct _TPrule_173));
#endif
_currn->_prod = RULErule_173;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_173: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_173: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_173: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_173;
return ( (NODEPTR) _currn);
}/* Mkrule_173 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_174 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_174 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_174 _currn;
#ifdef __cplusplus
_currn = new _TPrule_174;
#else
_currn = (_TPPrule_174) TreeNodeAlloc (sizeof (struct _TPrule_174));
#endif
_currn->_prod = RULErule_174;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_174: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_174: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_174: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_174: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_174;
return ( (NODEPTR) _currn);
}/* Mkrule_174 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_175 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_175 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_175 _currn;
#ifdef __cplusplus
_currn = new _TPrule_175;
#else
_currn = (_TPPrule_175) TreeNodeAlloc (sizeof (struct _TPrule_175));
#endif
_currn->_prod = RULErule_175;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_175: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_175: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_175: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_175: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_175;
return ( (NODEPTR) _currn);
}/* Mkrule_175 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_176 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_176 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_176 _currn;
#ifdef __cplusplus
_currn = new _TPrule_176;
#else
_currn = (_TPPrule_176) TreeNodeAlloc (sizeof (struct _TPrule_176));
#endif
_currn->_prod = RULErule_176;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_176: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_176: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_176: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_176: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_176: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_176;
return ( (NODEPTR) _currn);
}/* Mkrule_176 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_177 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_177 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_177 _currn;
#ifdef __cplusplus
_currn = new _TPrule_177;
#else
_currn = (_TPPrule_177) TreeNodeAlloc (sizeof (struct _TPrule_177));
#endif
_currn->_prod = RULErule_177;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_177: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_177: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_177: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_177: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_177;
return ( (NODEPTR) _currn);
}/* Mkrule_177 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_178 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_178 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_178 _currn;
#ifdef __cplusplus
_currn = new _TPrule_178;
#else
_currn = (_TPPrule_178) TreeNodeAlloc (sizeof (struct _TPrule_178));
#endif
_currn->_prod = RULErule_178;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_178: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_178: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_178: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_178: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_178: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_178;
return ( (NODEPTR) _currn);
}/* Mkrule_178 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_179 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_179 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_179 _currn;
#ifdef __cplusplus
_currn = new _TPrule_179;
#else
_currn = (_TPPrule_179) TreeNodeAlloc (sizeof (struct _TPrule_179));
#endif
_currn->_prod = RULErule_179;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_179: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_179: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_179: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_179: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_179: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_179;
return ( (NODEPTR) _currn);
}/* Mkrule_179 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_180 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_180 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_180 _currn;
#ifdef __cplusplus
_currn = new _TPrule_180;
#else
_currn = (_TPPrule_180) TreeNodeAlloc (sizeof (struct _TPrule_180));
#endif
_currn->_prod = RULErule_180;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_180: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_180: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_180: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_180: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_180: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_180: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_180;
return ( (NODEPTR) _currn);
}/* Mkrule_180 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_181 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_181 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_181 _currn;
#ifdef __cplusplus
_currn = new _TPrule_181;
#else
_currn = (_TPPrule_181) TreeNodeAlloc (sizeof (struct _TPrule_181));
#endif
_currn->_prod = RULErule_181;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_181: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_181: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_181: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_181;
return ( (NODEPTR) _currn);
}/* Mkrule_181 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_182 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_182 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_182 _currn;
#ifdef __cplusplus
_currn = new _TPrule_182;
#else
_currn = (_TPPrule_182) TreeNodeAlloc (sizeof (struct _TPrule_182));
#endif
_currn->_prod = RULErule_182;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_182: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_182: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_182: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_182: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_182;
return ( (NODEPTR) _currn);
}/* Mkrule_182 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_183 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_183 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_183 _currn;
#ifdef __cplusplus
_currn = new _TPrule_183;
#else
_currn = (_TPPrule_183) TreeNodeAlloc (sizeof (struct _TPrule_183));
#endif
_currn->_prod = RULErule_183;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_183: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_183: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_183: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_183: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_183;
return ( (NODEPTR) _currn);
}/* Mkrule_183 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_184 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_184 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_184 _currn;
#ifdef __cplusplus
_currn = new _TPrule_184;
#else
_currn = (_TPPrule_184) TreeNodeAlloc (sizeof (struct _TPrule_184));
#endif
_currn->_prod = RULErule_184;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_184: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_184: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_184: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_184: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_184: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_184;
return ( (NODEPTR) _currn);
}/* Mkrule_184 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_185 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_185 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_185 _currn;
#ifdef __cplusplus
_currn = new _TPrule_185;
#else
_currn = (_TPPrule_185) TreeNodeAlloc (sizeof (struct _TPrule_185));
#endif
_currn->_prod = RULErule_185;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_185: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_185: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_185: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_185: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_185;
return ( (NODEPTR) _currn);
}/* Mkrule_185 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_186 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_186 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_186 _currn;
#ifdef __cplusplus
_currn = new _TPrule_186;
#else
_currn = (_TPPrule_186) TreeNodeAlloc (sizeof (struct _TPrule_186));
#endif
_currn->_prod = RULErule_186;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_186: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_186: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_186: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_186: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_186: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_186;
return ( (NODEPTR) _currn);
}/* Mkrule_186 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_187 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_187 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_187 _currn;
#ifdef __cplusplus
_currn = new _TPrule_187;
#else
_currn = (_TPPrule_187) TreeNodeAlloc (sizeof (struct _TPrule_187));
#endif
_currn->_prod = RULErule_187;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_187: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_187: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_187: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_187: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_187: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_187;
return ( (NODEPTR) _currn);
}/* Mkrule_187 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_188 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_188 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_188 _currn;
#ifdef __cplusplus
_currn = new _TPrule_188;
#else
_currn = (_TPPrule_188) TreeNodeAlloc (sizeof (struct _TPrule_188));
#endif
_currn->_prod = RULErule_188;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_188: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_188: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_188: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_188: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_188: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_188: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_188;
return ( (NODEPTR) _currn);
}/* Mkrule_188 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_189 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_189 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_189 _currn;
#ifdef __cplusplus
_currn = new _TPrule_189;
#else
_currn = (_TPPrule_189) TreeNodeAlloc (sizeof (struct _TPrule_189));
#endif
_currn->_prod = RULErule_189;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_189: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_189: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_189: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_189: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_189;
return ( (NODEPTR) _currn);
}/* Mkrule_189 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_190 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_190 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_190 _currn;
#ifdef __cplusplus
_currn = new _TPrule_190;
#else
_currn = (_TPPrule_190) TreeNodeAlloc (sizeof (struct _TPrule_190));
#endif
_currn->_prod = RULErule_190;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_190: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_190: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_190: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_190: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_190: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_190;
return ( (NODEPTR) _currn);
}/* Mkrule_190 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_191 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_191 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_191 _currn;
#ifdef __cplusplus
_currn = new _TPrule_191;
#else
_currn = (_TPPrule_191) TreeNodeAlloc (sizeof (struct _TPrule_191));
#endif
_currn->_prod = RULErule_191;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_191: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_191: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_191: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_191: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_191: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_191;
return ( (NODEPTR) _currn);
}/* Mkrule_191 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_192 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_192 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_192 _currn;
#ifdef __cplusplus
_currn = new _TPrule_192;
#else
_currn = (_TPPrule_192) TreeNodeAlloc (sizeof (struct _TPrule_192));
#endif
_currn->_prod = RULErule_192;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_192: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_192: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_192: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_192: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_192: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_192: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_192;
return ( (NODEPTR) _currn);
}/* Mkrule_192 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_193 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_193 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_193 _currn;
#ifdef __cplusplus
_currn = new _TPrule_193;
#else
_currn = (_TPPrule_193) TreeNodeAlloc (sizeof (struct _TPrule_193));
#endif
_currn->_prod = RULErule_193;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_193: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_193: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_193: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_193: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_193: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_193;
return ( (NODEPTR) _currn);
}/* Mkrule_193 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_194 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_194 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_194 _currn;
#ifdef __cplusplus
_currn = new _TPrule_194;
#else
_currn = (_TPPrule_194) TreeNodeAlloc (sizeof (struct _TPrule_194));
#endif
_currn->_prod = RULErule_194;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_194: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_194: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_194: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_194: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_194: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_194: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_194;
return ( (NODEPTR) _currn);
}/* Mkrule_194 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_195 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_195 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_195 _currn;
#ifdef __cplusplus
_currn = new _TPrule_195;
#else
_currn = (_TPPrule_195) TreeNodeAlloc (sizeof (struct _TPrule_195));
#endif
_currn->_prod = RULErule_195;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_195: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_195: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_195: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_195: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_195: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasures) MkMeasures (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_195: root of subtree no. 6 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_195;
return ( (NODEPTR) _currn);
}/* Mkrule_195 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_196 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7)
#else
NODEPTR Mkrule_196 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
#endif
{	_TPPrule_196 _currn;
#ifdef __cplusplus
_currn = new _TPrule_196;
#else
_currn = (_TPPrule_196) TreeNodeAlloc (sizeof (struct _TPrule_196));
#endif
_currn->_prod = RULErule_196;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_196: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitions) MkTransitions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_196: root of subtree no. 2 can not be made a Transitions node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_196: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_196: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_196: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasures) MkMeasures (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_196: root of subtree no. 6 can not be made a Measures node ", 0, _coordref);
_currn->_desc7 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_196: root of subtree no. 7 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_196;
return ( (NODEPTR) _currn);
}/* Mkrule_196 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_197 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_197 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_197 _currn;
#ifdef __cplusplus
_currn = new _TPrule_197;
#else
_currn = (_TPPrule_197) TreeNodeAlloc (sizeof (struct _TPrule_197));
#endif
_currn->_prod = RULErule_197;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_197: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_197: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_197;
return ( (NODEPTR) _currn);
}/* Mkrule_197 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_198 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_198 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_198 _currn;
#ifdef __cplusplus
_currn = new _TPrule_198;
#else
_currn = (_TPPrule_198) TreeNodeAlloc (sizeof (struct _TPrule_198));
#endif
_currn->_prod = RULErule_198;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_198: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_198: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_198: root of subtree no. 3 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_198;
return ( (NODEPTR) _currn);
}/* Mkrule_198 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_199 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_199 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_199 _currn;
#ifdef __cplusplus
_currn = new _TPrule_199;
#else
_currn = (_TPPrule_199) TreeNodeAlloc (sizeof (struct _TPrule_199));
#endif
_currn->_prod = RULErule_199;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_199: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_199: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_199: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_199;
return ( (NODEPTR) _currn);
}/* Mkrule_199 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_200 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_200 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_200 _currn;
#ifdef __cplusplus
_currn = new _TPrule_200;
#else
_currn = (_TPPrule_200) TreeNodeAlloc (sizeof (struct _TPrule_200));
#endif
_currn->_prod = RULErule_200;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_200: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_200: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasures) MkMeasures (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_200: root of subtree no. 3 can not be made a Measures node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_200: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_200;
return ( (NODEPTR) _currn);
}/* Mkrule_200 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_201 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_201 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_201 _currn;
#ifdef __cplusplus
_currn = new _TPrule_201;
#else
_currn = (_TPPrule_201) TreeNodeAlloc (sizeof (struct _TPrule_201));
#endif
_currn->_prod = RULErule_201;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_201: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_201: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_201: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_201;
return ( (NODEPTR) _currn);
}/* Mkrule_201 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_202 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_202 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_202 _currn;
#ifdef __cplusplus
_currn = new _TPrule_202;
#else
_currn = (_TPPrule_202) TreeNodeAlloc (sizeof (struct _TPrule_202));
#endif
_currn->_prod = RULErule_202;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_202: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_202: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_202: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_202: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_202;
return ( (NODEPTR) _currn);
}/* Mkrule_202 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_203 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_203 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_203 _currn;
#ifdef __cplusplus
_currn = new _TPrule_203;
#else
_currn = (_TPPrule_203) TreeNodeAlloc (sizeof (struct _TPrule_203));
#endif
_currn->_prod = RULErule_203;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_203: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_203: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_203: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_203: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_203;
return ( (NODEPTR) _currn);
}/* Mkrule_203 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_204 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_204 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_204 _currn;
#ifdef __cplusplus
_currn = new _TPrule_204;
#else
_currn = (_TPPrule_204) TreeNodeAlloc (sizeof (struct _TPrule_204));
#endif
_currn->_prod = RULErule_204;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_204: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_204: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPRewards) MkRewards (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_204: root of subtree no. 3 can not be made a Rewards node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_204: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_204: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_204;
return ( (NODEPTR) _currn);
}/* Mkrule_204 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_205 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_205 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_205 _currn;
#ifdef __cplusplus
_currn = new _TPrule_205;
#else
_currn = (_TPPrule_205) TreeNodeAlloc (sizeof (struct _TPrule_205));
#endif
_currn->_prod = RULErule_205;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_205: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_205: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_205: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_205;
return ( (NODEPTR) _currn);
}/* Mkrule_205 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_206 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_206 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_206 _currn;
#ifdef __cplusplus
_currn = new _TPrule_206;
#else
_currn = (_TPPrule_206) TreeNodeAlloc (sizeof (struct _TPrule_206));
#endif
_currn->_prod = RULErule_206;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_206: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_206: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_206: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_206: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_206;
return ( (NODEPTR) _currn);
}/* Mkrule_206 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_207 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_207 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_207 _currn;
#ifdef __cplusplus
_currn = new _TPrule_207;
#else
_currn = (_TPPrule_207) TreeNodeAlloc (sizeof (struct _TPrule_207));
#endif
_currn->_prod = RULErule_207;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_207: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_207: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_207: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_207: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_207;
return ( (NODEPTR) _currn);
}/* Mkrule_207 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_208 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_208 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_208 _currn;
#ifdef __cplusplus
_currn = new _TPrule_208;
#else
_currn = (_TPPrule_208) TreeNodeAlloc (sizeof (struct _TPrule_208));
#endif
_currn->_prod = RULErule_208;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_208: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_208: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_208: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_208: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_208: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_208;
return ( (NODEPTR) _currn);
}/* Mkrule_208 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_209 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_209 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_209 _currn;
#ifdef __cplusplus
_currn = new _TPrule_209;
#else
_currn = (_TPPrule_209) TreeNodeAlloc (sizeof (struct _TPrule_209));
#endif
_currn->_prod = RULErule_209;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_209: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_209: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_209: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_209: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_209;
return ( (NODEPTR) _currn);
}/* Mkrule_209 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_210 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_210 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_210 _currn;
#ifdef __cplusplus
_currn = new _TPrule_210;
#else
_currn = (_TPPrule_210) TreeNodeAlloc (sizeof (struct _TPrule_210));
#endif
_currn->_prod = RULErule_210;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_210: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_210: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_210: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_210: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_210: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_210;
return ( (NODEPTR) _currn);
}/* Mkrule_210 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_211 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_211 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_211 _currn;
#ifdef __cplusplus
_currn = new _TPrule_211;
#else
_currn = (_TPPrule_211) TreeNodeAlloc (sizeof (struct _TPrule_211));
#endif
_currn->_prod = RULErule_211;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_211: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_211: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_211: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_211: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_211: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_211;
return ( (NODEPTR) _currn);
}/* Mkrule_211 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_212 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_212 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_212 _currn;
#ifdef __cplusplus
_currn = new _TPrule_212;
#else
_currn = (_TPPrule_212) TreeNodeAlloc (sizeof (struct _TPrule_212));
#endif
_currn->_prod = RULErule_212;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_212: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_212: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPArcs) MkArcs (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_212: root of subtree no. 3 can not be made a Arcs node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_212: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_212: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_212: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_212;
return ( (NODEPTR) _currn);
}/* Mkrule_212 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_213 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_213 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_213 _currn;
#ifdef __cplusplus
_currn = new _TPrule_213;
#else
_currn = (_TPPrule_213) TreeNodeAlloc (sizeof (struct _TPrule_213));
#endif
_currn->_prod = RULErule_213;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_213: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_213: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_213: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_213;
return ( (NODEPTR) _currn);
}/* Mkrule_213 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_214 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_214 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_214 _currn;
#ifdef __cplusplus
_currn = new _TPrule_214;
#else
_currn = (_TPPrule_214) TreeNodeAlloc (sizeof (struct _TPrule_214));
#endif
_currn->_prod = RULErule_214;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_214: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_214: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_214: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_214: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_214;
return ( (NODEPTR) _currn);
}/* Mkrule_214 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_215 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_215 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_215 _currn;
#ifdef __cplusplus
_currn = new _TPrule_215;
#else
_currn = (_TPPrule_215) TreeNodeAlloc (sizeof (struct _TPrule_215));
#endif
_currn->_prod = RULErule_215;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_215: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_215: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_215: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_215: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_215;
return ( (NODEPTR) _currn);
}/* Mkrule_215 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_216 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_216 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_216 _currn;
#ifdef __cplusplus
_currn = new _TPrule_216;
#else
_currn = (_TPPrule_216) TreeNodeAlloc (sizeof (struct _TPrule_216));
#endif
_currn->_prod = RULErule_216;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_216: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_216: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_216: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_216: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_216: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_216;
return ( (NODEPTR) _currn);
}/* Mkrule_216 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_217 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_217 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_217 _currn;
#ifdef __cplusplus
_currn = new _TPrule_217;
#else
_currn = (_TPPrule_217) TreeNodeAlloc (sizeof (struct _TPrule_217));
#endif
_currn->_prod = RULErule_217;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_217: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_217: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_217: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_217: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_217;
return ( (NODEPTR) _currn);
}/* Mkrule_217 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_218 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_218 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_218 _currn;
#ifdef __cplusplus
_currn = new _TPrule_218;
#else
_currn = (_TPPrule_218) TreeNodeAlloc (sizeof (struct _TPrule_218));
#endif
_currn->_prod = RULErule_218;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_218: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_218: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_218: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_218: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_218: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_218;
return ( (NODEPTR) _currn);
}/* Mkrule_218 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_219 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_219 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_219 _currn;
#ifdef __cplusplus
_currn = new _TPrule_219;
#else
_currn = (_TPPrule_219) TreeNodeAlloc (sizeof (struct _TPrule_219));
#endif
_currn->_prod = RULErule_219;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_219: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_219: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_219: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_219: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_219: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_219;
return ( (NODEPTR) _currn);
}/* Mkrule_219 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_220 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_220 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_220 _currn;
#ifdef __cplusplus
_currn = new _TPrule_220;
#else
_currn = (_TPPrule_220) TreeNodeAlloc (sizeof (struct _TPrule_220));
#endif
_currn->_prod = RULErule_220;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_220: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_220: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_220: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_220: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_220: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_220: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_220;
return ( (NODEPTR) _currn);
}/* Mkrule_220 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_221 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_221 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_221 _currn;
#ifdef __cplusplus
_currn = new _TPrule_221;
#else
_currn = (_TPPrule_221) TreeNodeAlloc (sizeof (struct _TPrule_221));
#endif
_currn->_prod = RULErule_221;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_221: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_221: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_221: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_221: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_221;
return ( (NODEPTR) _currn);
}/* Mkrule_221 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_222 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_222 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_222 _currn;
#ifdef __cplusplus
_currn = new _TPrule_222;
#else
_currn = (_TPPrule_222) TreeNodeAlloc (sizeof (struct _TPrule_222));
#endif
_currn->_prod = RULErule_222;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_222: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_222: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_222: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_222: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_222: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_222;
return ( (NODEPTR) _currn);
}/* Mkrule_222 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_223 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_223 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_223 _currn;
#ifdef __cplusplus
_currn = new _TPrule_223;
#else
_currn = (_TPPrule_223) TreeNodeAlloc (sizeof (struct _TPrule_223));
#endif
_currn->_prod = RULErule_223;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_223: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_223: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_223: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_223: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_223: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_223;
return ( (NODEPTR) _currn);
}/* Mkrule_223 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_224 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_224 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_224 _currn;
#ifdef __cplusplus
_currn = new _TPrule_224;
#else
_currn = (_TPPrule_224) TreeNodeAlloc (sizeof (struct _TPrule_224));
#endif
_currn->_prod = RULErule_224;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_224: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_224: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_224: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_224: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_224: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_224: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_224;
return ( (NODEPTR) _currn);
}/* Mkrule_224 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_225 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_225 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_225 _currn;
#ifdef __cplusplus
_currn = new _TPrule_225;
#else
_currn = (_TPPrule_225) TreeNodeAlloc (sizeof (struct _TPrule_225));
#endif
_currn->_prod = RULErule_225;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_225: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_225: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_225: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_225: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_225: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_225;
return ( (NODEPTR) _currn);
}/* Mkrule_225 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_226 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_226 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_226 _currn;
#ifdef __cplusplus
_currn = new _TPrule_226;
#else
_currn = (_TPPrule_226) TreeNodeAlloc (sizeof (struct _TPrule_226));
#endif
_currn->_prod = RULErule_226;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_226: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_226: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_226: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_226: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_226: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_226: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_226;
return ( (NODEPTR) _currn);
}/* Mkrule_226 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_227 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_227 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_227 _currn;
#ifdef __cplusplus
_currn = new _TPrule_227;
#else
_currn = (_TPPrule_227) TreeNodeAlloc (sizeof (struct _TPrule_227));
#endif
_currn->_prod = RULErule_227;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_227: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_227: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_227: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_227: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_227: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasures) MkMeasures (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_227: root of subtree no. 6 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_227;
return ( (NODEPTR) _currn);
}/* Mkrule_227 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_228 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7)
#else
NODEPTR Mkrule_228 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
#endif
{	_TPPrule_228 _currn;
#ifdef __cplusplus
_currn = new _TPrule_228;
#else
_currn = (_TPPrule_228) TreeNodeAlloc (sizeof (struct _TPrule_228));
#endif
_currn->_prod = RULErule_228;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_228: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_228: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_228: root of subtree no. 3 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_228: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_228: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasures) MkMeasures (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_228: root of subtree no. 6 can not be made a Measures node ", 0, _coordref);
_currn->_desc7 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_228: root of subtree no. 7 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_228;
return ( (NODEPTR) _currn);
}/* Mkrule_228 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_229 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_229 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_229 _currn;
#ifdef __cplusplus
_currn = new _TPrule_229;
#else
_currn = (_TPPrule_229) TreeNodeAlloc (sizeof (struct _TPrule_229));
#endif
_currn->_prod = RULErule_229;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_229: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_229: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_229: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_229;
return ( (NODEPTR) _currn);
}/* Mkrule_229 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_230 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_230 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_230 _currn;
#ifdef __cplusplus
_currn = new _TPrule_230;
#else
_currn = (_TPPrule_230) TreeNodeAlloc (sizeof (struct _TPrule_230));
#endif
_currn->_prod = RULErule_230;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_230: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_230: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_230: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_230: root of subtree no. 4 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_230;
return ( (NODEPTR) _currn);
}/* Mkrule_230 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_231 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_231 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_231 _currn;
#ifdef __cplusplus
_currn = new _TPrule_231;
#else
_currn = (_TPPrule_231) TreeNodeAlloc (sizeof (struct _TPrule_231));
#endif
_currn->_prod = RULErule_231;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_231: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_231: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_231: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_231: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_231;
return ( (NODEPTR) _currn);
}/* Mkrule_231 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_232 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_232 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_232 _currn;
#ifdef __cplusplus
_currn = new _TPrule_232;
#else
_currn = (_TPPrule_232) TreeNodeAlloc (sizeof (struct _TPrule_232));
#endif
_currn->_prod = RULErule_232;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_232: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_232: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_232: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPMeasures) MkMeasures (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_232: root of subtree no. 4 can not be made a Measures node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_232: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_232;
return ( (NODEPTR) _currn);
}/* Mkrule_232 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_233 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_233 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_233 _currn;
#ifdef __cplusplus
_currn = new _TPrule_233;
#else
_currn = (_TPPrule_233) TreeNodeAlloc (sizeof (struct _TPrule_233));
#endif
_currn->_prod = RULErule_233;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_233: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_233: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_233: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_233: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_233;
return ( (NODEPTR) _currn);
}/* Mkrule_233 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_234 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_234 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_234 _currn;
#ifdef __cplusplus
_currn = new _TPrule_234;
#else
_currn = (_TPPrule_234) TreeNodeAlloc (sizeof (struct _TPrule_234));
#endif
_currn->_prod = RULErule_234;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_234: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_234: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_234: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_234: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_234: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_234;
return ( (NODEPTR) _currn);
}/* Mkrule_234 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_235 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_235 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_235 _currn;
#ifdef __cplusplus
_currn = new _TPrule_235;
#else
_currn = (_TPPrule_235) TreeNodeAlloc (sizeof (struct _TPrule_235));
#endif
_currn->_prod = RULErule_235;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_235: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_235: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_235: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_235: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_235: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_235;
return ( (NODEPTR) _currn);
}/* Mkrule_235 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_236 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_236 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_236 _currn;
#ifdef __cplusplus
_currn = new _TPrule_236;
#else
_currn = (_TPPrule_236) TreeNodeAlloc (sizeof (struct _TPrule_236));
#endif
_currn->_prod = RULErule_236;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_236: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_236: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_236: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPRewards) MkRewards (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_236: root of subtree no. 4 can not be made a Rewards node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_236: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_236: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_236;
return ( (NODEPTR) _currn);
}/* Mkrule_236 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_237 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_237 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_237 _currn;
#ifdef __cplusplus
_currn = new _TPrule_237;
#else
_currn = (_TPPrule_237) TreeNodeAlloc (sizeof (struct _TPrule_237));
#endif
_currn->_prod = RULErule_237;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_237: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_237: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_237: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_237: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_237;
return ( (NODEPTR) _currn);
}/* Mkrule_237 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_238 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_238 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_238 _currn;
#ifdef __cplusplus
_currn = new _TPrule_238;
#else
_currn = (_TPPrule_238) TreeNodeAlloc (sizeof (struct _TPrule_238));
#endif
_currn->_prod = RULErule_238;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_238: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_238: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_238: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_238: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_238: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_238;
return ( (NODEPTR) _currn);
}/* Mkrule_238 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_239 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_239 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_239 _currn;
#ifdef __cplusplus
_currn = new _TPrule_239;
#else
_currn = (_TPPrule_239) TreeNodeAlloc (sizeof (struct _TPrule_239));
#endif
_currn->_prod = RULErule_239;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_239: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_239: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_239: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_239: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_239: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_239;
return ( (NODEPTR) _currn);
}/* Mkrule_239 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_240 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_240 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_240 _currn;
#ifdef __cplusplus
_currn = new _TPrule_240;
#else
_currn = (_TPPrule_240) TreeNodeAlloc (sizeof (struct _TPrule_240));
#endif
_currn->_prod = RULErule_240;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_240: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_240: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_240: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_240: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_240: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_240: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_240;
return ( (NODEPTR) _currn);
}/* Mkrule_240 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_241 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_241 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_241 _currn;
#ifdef __cplusplus
_currn = new _TPrule_241;
#else
_currn = (_TPPrule_241) TreeNodeAlloc (sizeof (struct _TPrule_241));
#endif
_currn->_prod = RULErule_241;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_241: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_241: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_241: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_241: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_241: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_241;
return ( (NODEPTR) _currn);
}/* Mkrule_241 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_242 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_242 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_242 _currn;
#ifdef __cplusplus
_currn = new _TPrule_242;
#else
_currn = (_TPPrule_242) TreeNodeAlloc (sizeof (struct _TPrule_242));
#endif
_currn->_prod = RULErule_242;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_242: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_242: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_242: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_242: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_242: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_242: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_242;
return ( (NODEPTR) _currn);
}/* Mkrule_242 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_243 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_243 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_243 _currn;
#ifdef __cplusplus
_currn = new _TPrule_243;
#else
_currn = (_TPPrule_243) TreeNodeAlloc (sizeof (struct _TPrule_243));
#endif
_currn->_prod = RULErule_243;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_243: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_243: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_243: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_243: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_243: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasures) MkMeasures (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_243: root of subtree no. 6 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_243;
return ( (NODEPTR) _currn);
}/* Mkrule_243 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_244 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7)
#else
NODEPTR Mkrule_244 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
#endif
{	_TPPrule_244 _currn;
#ifdef __cplusplus
_currn = new _TPrule_244;
#else
_currn = (_TPPrule_244) TreeNodeAlloc (sizeof (struct _TPrule_244));
#endif
_currn->_prod = RULErule_244;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_244: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_244: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_244: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPArcs) MkArcs (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_244: root of subtree no. 4 can not be made a Arcs node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_244: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasures) MkMeasures (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_244: root of subtree no. 6 can not be made a Measures node ", 0, _coordref);
_currn->_desc7 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_244: root of subtree no. 7 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_244;
return ( (NODEPTR) _currn);
}/* Mkrule_244 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_245 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_245 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_245 _currn;
#ifdef __cplusplus
_currn = new _TPrule_245;
#else
_currn = (_TPPrule_245) TreeNodeAlloc (sizeof (struct _TPrule_245));
#endif
_currn->_prod = RULErule_245;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_245: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_245: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_245: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_245: root of subtree no. 4 can not be made a TransitionAttributes node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_245;
return ( (NODEPTR) _currn);
}/* Mkrule_245 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_246 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_246 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_246 _currn;
#ifdef __cplusplus
_currn = new _TPrule_246;
#else
_currn = (_TPPrule_246) TreeNodeAlloc (sizeof (struct _TPrule_246));
#endif
_currn->_prod = RULErule_246;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_246: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_246: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_246: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_246: root of subtree no. 4 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_246: root of subtree no. 5 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_246;
return ( (NODEPTR) _currn);
}/* Mkrule_246 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_247 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_247 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_247 _currn;
#ifdef __cplusplus
_currn = new _TPrule_247;
#else
_currn = (_TPPrule_247) TreeNodeAlloc (sizeof (struct _TPrule_247));
#endif
_currn->_prod = RULErule_247;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_247: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_247: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_247: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_247: root of subtree no. 4 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_247: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_247;
return ( (NODEPTR) _currn);
}/* Mkrule_247 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_248 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_248 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_248 _currn;
#ifdef __cplusplus
_currn = new _TPrule_248;
#else
_currn = (_TPPrule_248) TreeNodeAlloc (sizeof (struct _TPrule_248));
#endif
_currn->_prod = RULErule_248;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_248: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_248: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_248: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_248: root of subtree no. 4 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc5 = (_TSPMeasures) MkMeasures (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_248: root of subtree no. 5 can not be made a Measures node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_248: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_248;
return ( (NODEPTR) _currn);
}/* Mkrule_248 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_249 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_249 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_249 _currn;
#ifdef __cplusplus
_currn = new _TPrule_249;
#else
_currn = (_TPPrule_249) TreeNodeAlloc (sizeof (struct _TPrule_249));
#endif
_currn->_prod = RULErule_249;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_249: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_249: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_249: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_249: root of subtree no. 4 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_249: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_249;
return ( (NODEPTR) _currn);
}/* Mkrule_249 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_250 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_250 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_250 _currn;
#ifdef __cplusplus
_currn = new _TPrule_250;
#else
_currn = (_TPPrule_250) TreeNodeAlloc (sizeof (struct _TPrule_250));
#endif
_currn->_prod = RULErule_250;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_250: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_250: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_250: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_250: root of subtree no. 4 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_250: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_250: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_250;
return ( (NODEPTR) _currn);
}/* Mkrule_250 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_251 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_251 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_251 _currn;
#ifdef __cplusplus
_currn = new _TPrule_251;
#else
_currn = (_TPPrule_251) TreeNodeAlloc (sizeof (struct _TPrule_251));
#endif
_currn->_prod = RULErule_251;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_251: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_251: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_251: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_251: root of subtree no. 4 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_251: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasures) MkMeasures (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_251: root of subtree no. 6 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_251;
return ( (NODEPTR) _currn);
}/* Mkrule_251 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_252 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7)
#else
NODEPTR Mkrule_252 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
#endif
{	_TPPrule_252 _currn;
#ifdef __cplusplus
_currn = new _TPrule_252;
#else
_currn = (_TPPrule_252) TreeNodeAlloc (sizeof (struct _TPrule_252));
#endif
_currn->_prod = RULErule_252;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_252: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_252: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_252: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_252: root of subtree no. 4 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc5 = (_TSPRewards) MkRewards (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_252: root of subtree no. 5 can not be made a Rewards node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasures) MkMeasures (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_252: root of subtree no. 6 can not be made a Measures node ", 0, _coordref);
_currn->_desc7 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_252: root of subtree no. 7 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_252;
return ( (NODEPTR) _currn);
}/* Mkrule_252 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_253 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR Mkrule_253 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	_TPPrule_253 _currn;
#ifdef __cplusplus
_currn = new _TPrule_253;
#else
_currn = (_TPPrule_253) TreeNodeAlloc (sizeof (struct _TPrule_253));
#endif
_currn->_prod = RULErule_253;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_253: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_253: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_253: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_253: root of subtree no. 4 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc5 = (_TSPArcs) MkArcs (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_253: root of subtree no. 5 can not be made a Arcs node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_253;
return ( (NODEPTR) _currn);
}/* Mkrule_253 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_254 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_254 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_254 _currn;
#ifdef __cplusplus
_currn = new _TPrule_254;
#else
_currn = (_TPPrule_254) TreeNodeAlloc (sizeof (struct _TPrule_254));
#endif
_currn->_prod = RULErule_254;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_254: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_254: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_254: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_254: root of subtree no. 4 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc5 = (_TSPArcs) MkArcs (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_254: root of subtree no. 5 can not be made a Arcs node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_254: root of subtree no. 6 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_254;
return ( (NODEPTR) _currn);
}/* Mkrule_254 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_255 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_255 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_255 _currn;
#ifdef __cplusplus
_currn = new _TPrule_255;
#else
_currn = (_TPPrule_255) TreeNodeAlloc (sizeof (struct _TPrule_255));
#endif
_currn->_prod = RULErule_255;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_255: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_255: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_255: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_255: root of subtree no. 4 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc5 = (_TSPArcs) MkArcs (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_255: root of subtree no. 5 can not be made a Arcs node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasures) MkMeasures (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_255: root of subtree no. 6 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_255;
return ( (NODEPTR) _currn);
}/* Mkrule_255 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_256 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7)
#else
NODEPTR Mkrule_256 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
#endif
{	_TPPrule_256 _currn;
#ifdef __cplusplus
_currn = new _TPrule_256;
#else
_currn = (_TPPrule_256) TreeNodeAlloc (sizeof (struct _TPrule_256));
#endif
_currn->_prod = RULErule_256;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_256: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_256: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_256: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_256: root of subtree no. 4 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc5 = (_TSPArcs) MkArcs (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_256: root of subtree no. 5 can not be made a Arcs node ", 0, _coordref);
_currn->_desc6 = (_TSPMeasures) MkMeasures (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_256: root of subtree no. 6 can not be made a Measures node ", 0, _coordref);
_currn->_desc7 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_256: root of subtree no. 7 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_256;
return ( (NODEPTR) _currn);
}/* Mkrule_256 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_257 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR Mkrule_257 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	_TPPrule_257 _currn;
#ifdef __cplusplus
_currn = new _TPrule_257;
#else
_currn = (_TPPrule_257) TreeNodeAlloc (sizeof (struct _TPrule_257));
#endif
_currn->_prod = RULErule_257;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_257: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_257: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_257: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_257: root of subtree no. 4 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc5 = (_TSPArcs) MkArcs (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_257: root of subtree no. 5 can not be made a Arcs node ", 0, _coordref);
_currn->_desc6 = (_TSPRewards) MkRewards (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_257: root of subtree no. 6 can not be made a Rewards node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_257;
return ( (NODEPTR) _currn);
}/* Mkrule_257 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_258 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7)
#else
NODEPTR Mkrule_258 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
#endif
{	_TPPrule_258 _currn;
#ifdef __cplusplus
_currn = new _TPrule_258;
#else
_currn = (_TPPrule_258) TreeNodeAlloc (sizeof (struct _TPrule_258));
#endif
_currn->_prod = RULErule_258;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_258: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_258: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_258: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_258: root of subtree no. 4 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc5 = (_TSPArcs) MkArcs (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_258: root of subtree no. 5 can not be made a Arcs node ", 0, _coordref);
_currn->_desc6 = (_TSPRewards) MkRewards (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_258: root of subtree no. 6 can not be made a Rewards node ", 0, _coordref);
_currn->_desc7 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_258: root of subtree no. 7 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_258;
return ( (NODEPTR) _currn);
}/* Mkrule_258 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_259 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7)
#else
NODEPTR Mkrule_259 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
#endif
{	_TPPrule_259 _currn;
#ifdef __cplusplus
_currn = new _TPrule_259;
#else
_currn = (_TPPrule_259) TreeNodeAlloc (sizeof (struct _TPrule_259));
#endif
_currn->_prod = RULErule_259;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_259: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_259: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_259: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_259: root of subtree no. 4 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc5 = (_TSPArcs) MkArcs (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_259: root of subtree no. 5 can not be made a Arcs node ", 0, _coordref);
_currn->_desc6 = (_TSPRewards) MkRewards (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_259: root of subtree no. 6 can not be made a Rewards node ", 0, _coordref);
_currn->_desc7 = (_TSPMeasures) MkMeasures (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_259: root of subtree no. 7 can not be made a Measures node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_259;
return ( (NODEPTR) _currn);
}/* Mkrule_259 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_260 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7, NODEPTR _desc8)
#else
NODEPTR Mkrule_260 (_coordref,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7,_desc8)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
	NODEPTR _desc8;
#endif
{	_TPPrule_260 _currn;
#ifdef __cplusplus
_currn = new _TPrule_260;
#else
_currn = (_TPPrule_260) TreeNodeAlloc (sizeof (struct _TPrule_260));
#endif
_currn->_prod = RULErule_260;
_currn->_desc1 = (_TSPParameters) MkParameters (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_260: root of subtree no. 1 can not be made a Parameters node ", 0, _coordref);
_currn->_desc2 = (_TSPPlaces) MkPlaces (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_260: root of subtree no. 2 can not be made a Places node ", 0, _coordref);
_currn->_desc3 = (_TSPTransitions) MkTransitions (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_260: root of subtree no. 3 can not be made a Transitions node ", 0, _coordref);
_currn->_desc4 = (_TSPTransitionAttributes) MkTransitionAttributes (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_260: root of subtree no. 4 can not be made a TransitionAttributes node ", 0, _coordref);
_currn->_desc5 = (_TSPArcs) MkArcs (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_260: root of subtree no. 5 can not be made a Arcs node ", 0, _coordref);
_currn->_desc6 = (_TSPRewards) MkRewards (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_260: root of subtree no. 6 can not be made a Rewards node ", 0, _coordref);
_currn->_desc7 = (_TSPMeasures) MkMeasures (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_260: root of subtree no. 7 can not be made a Measures node ", 0, _coordref);
_currn->_desc8 = (_TSPMeasureParameters) MkMeasureParameters (_coordref, _desc8);	
if (((NODEPTR)_currn->_desc8) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_260: root of subtree no. 8 can not be made a MeasureParameters node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_260;
return ( (NODEPTR) _currn);
}/* Mkrule_260 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_261 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_261 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_261 _currn;
#ifdef __cplusplus
_currn = new _TPrule_261;
#else
_currn = (_TPPrule_261) TreeNodeAlloc (sizeof (struct _TPrule_261));
#endif
_currn->_prod = RULErule_261;
_currn->_desc1 = (_TSPG2) MkG2 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_261: root of subtree no. 1 can not be made a G2 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_261;
return ( (NODEPTR) _currn);
}/* Mkrule_261 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_262 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_262 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_262 _currn;
#ifdef __cplusplus
_currn = new _TPrule_262;
#else
_currn = (_TPPrule_262) TreeNodeAlloc (sizeof (struct _TPrule_262));
#endif
_currn->_prod = RULErule_262;
_currn->_desc1 = (_TSPConnection) MkConnection (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_262: root of subtree no. 1 can not be made a Connection node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_262;
return ( (NODEPTR) _currn);
}/* Mkrule_262 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_263 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_263 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_263 _currn;
#ifdef __cplusplus
_currn = new _TPrule_263;
#else
_currn = (_TPPrule_263) TreeNodeAlloc (sizeof (struct _TPrule_263));
#endif
_currn->_prod = RULErule_263;
_currn->_desc1 = (_TSPG2) MkG2 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_263: root of subtree no. 1 can not be made a G2 node ", 0, _coordref);
_currn->_desc2 = (_TSPConnection) MkConnection (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_263: root of subtree no. 2 can not be made a Connection node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_263;
return ( (NODEPTR) _currn);
}/* Mkrule_263 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_264 (POSITION *_coordref, int _TERM1, int _TERM2, int _TERM3)
#else
NODEPTR Mkrule_264 (_coordref, _TERM1, _TERM2, _TERM3)
	POSITION *_coordref;
	int _TERM1;
	int _TERM2;
	int _TERM3;
#endif
{	_TPPrule_264 _currn;
#ifdef __cplusplus
_currn = new _TPrule_264;
#else
_currn = (_TPPrule_264) TreeNodeAlloc (sizeof (struct _TPrule_264));
#endif
_currn->_prod = RULErule_264;
_SETCOORD(_currn)
_TERMACT_rule_264;
return ( (NODEPTR) _currn);
}/* Mkrule_264 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_265 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_265 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_265 _currn;
#ifdef __cplusplus
_currn = new _TPrule_265;
#else
_currn = (_TPPrule_265) TreeNodeAlloc (sizeof (struct _TPrule_265));
#endif
_currn->_prod = RULErule_265;
_currn->_desc1 = (_TSPG3) MkG3 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_265: root of subtree no. 1 can not be made a G3 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_265;
return ( (NODEPTR) _currn);
}/* Mkrule_265 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_266 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_266 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_266 _currn;
#ifdef __cplusplus
_currn = new _TPrule_266;
#else
_currn = (_TPPrule_266) TreeNodeAlloc (sizeof (struct _TPrule_266));
#endif
_currn->_prod = RULErule_266;
_currn->_desc1 = (_TSPParameter) MkParameter (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_266: root of subtree no. 1 can not be made a Parameter node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_266;
return ( (NODEPTR) _currn);
}/* Mkrule_266 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_267 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_267 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_267 _currn;
#ifdef __cplusplus
_currn = new _TPrule_267;
#else
_currn = (_TPPrule_267) TreeNodeAlloc (sizeof (struct _TPrule_267));
#endif
_currn->_prod = RULErule_267;
_currn->_desc1 = (_TSPG3) MkG3 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_267: root of subtree no. 1 can not be made a G3 node ", 0, _coordref);
_currn->_desc2 = (_TSPParameter) MkParameter (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_267: root of subtree no. 2 can not be made a Parameter node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_267;
return ( (NODEPTR) _currn);
}/* Mkrule_267 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_268 (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_268 (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_268 _currn;
#ifdef __cplusplus
_currn = new _TPrule_268;
#else
_currn = (_TPPrule_268) TreeNodeAlloc (sizeof (struct _TPrule_268));
#endif
_currn->_prod = RULErule_268;
_currn->_desc1 = (_TSPGlobalId) MkGlobalId (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_268: root of subtree no. 1 can not be made a GlobalId node ", 0, _coordref);
_currn->_desc2 = (_TSPParameterExpression) MkParameterExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_268: root of subtree no. 2 can not be made a ParameterExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_268;
return ( (NODEPTR) _currn);
}/* Mkrule_268 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_269 (POSITION *_coordref, int _TERM1, int _TERM2, NODEPTR _desc1)
#else
NODEPTR Mkrule_269 (_coordref, _TERM1, _TERM2,_desc1)
	POSITION *_coordref;
	int _TERM1;
	int _TERM2;
	NODEPTR _desc1;
#endif
{	_TPPrule_269 _currn;
#ifdef __cplusplus
_currn = new _TPrule_269;
#else
_currn = (_TPPrule_269) TreeNodeAlloc (sizeof (struct _TPrule_269));
#endif
_currn->_prod = RULErule_269;
_currn->_desc1 = (_TSPParameterExpression) MkParameterExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_269: root of subtree no. 1 can not be made a ParameterExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_269;
return ( (NODEPTR) _currn);
}/* Mkrule_269 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_270 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_270 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_270 _currn;
#ifdef __cplusplus
_currn = new _TPrule_270;
#else
_currn = (_TPPrule_270) TreeNodeAlloc (sizeof (struct _TPrule_270));
#endif
_currn->_prod = RULErule_270;
_currn->_desc1 = (_TSPSign) MkSign (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_270: root of subtree no. 1 can not be made a Sign node ", 0, _coordref);
_currn->_desc2 = (_TSPParameterSum) MkParameterSum (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_270: root of subtree no. 2 can not be made a ParameterSum node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_270;
return ( (NODEPTR) _currn);
}/* Mkrule_270 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_271 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_271 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_271 _currn;
#ifdef __cplusplus
_currn = new _TPrule_271;
#else
_currn = (_TPPrule_271) TreeNodeAlloc (sizeof (struct _TPrule_271));
#endif
_currn->_prod = RULErule_271;
_currn->_desc1 = (_TSPParameterSum) MkParameterSum (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_271: root of subtree no. 1 can not be made a ParameterSum node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_271;
return ( (NODEPTR) _currn);
}/* Mkrule_271 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_272 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_272 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_272 _currn;
#ifdef __cplusplus
_currn = new _TPrule_272;
#else
_currn = (_TPPrule_272) TreeNodeAlloc (sizeof (struct _TPrule_272));
#endif
_currn->_prod = RULErule_272;
_currn->_desc1 = (_TSPParameterSum) MkParameterSum (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_272: root of subtree no. 1 can not be made a ParameterSum node ", 0, _coordref);
_currn->_desc2 = (_TSPAddOp) MkAddOp (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_272: root of subtree no. 2 can not be made a AddOp node ", 0, _coordref);
_currn->_desc3 = (_TSPParameterTerm) MkParameterTerm (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_272: root of subtree no. 3 can not be made a ParameterTerm node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_272;
return ( (NODEPTR) _currn);
}/* Mkrule_272 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_273 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_273 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_273 _currn;
#ifdef __cplusplus
_currn = new _TPrule_273;
#else
_currn = (_TPPrule_273) TreeNodeAlloc (sizeof (struct _TPrule_273));
#endif
_currn->_prod = RULErule_273;
_currn->_desc1 = (_TSPParameterTerm) MkParameterTerm (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_273: root of subtree no. 1 can not be made a ParameterTerm node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_273;
return ( (NODEPTR) _currn);
}/* Mkrule_273 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_274 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_274 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_274 _currn;
#ifdef __cplusplus
_currn = new _TPrule_274;
#else
_currn = (_TPPrule_274) TreeNodeAlloc (sizeof (struct _TPrule_274));
#endif
_currn->_prod = RULErule_274;
_currn->_desc1 = (_TSPParameterTerm) MkParameterTerm (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_274: root of subtree no. 1 can not be made a ParameterTerm node ", 0, _coordref);
_currn->_desc2 = (_TSPMulOp) MkMulOp (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_274: root of subtree no. 2 can not be made a MulOp node ", 0, _coordref);
_currn->_desc3 = (_TSPParameterFactor) MkParameterFactor (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_274: root of subtree no. 3 can not be made a ParameterFactor node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_274;
return ( (NODEPTR) _currn);
}/* Mkrule_274 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_275 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_275 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_275 _currn;
#ifdef __cplusplus
_currn = new _TPrule_275;
#else
_currn = (_TPPrule_275) TreeNodeAlloc (sizeof (struct _TPrule_275));
#endif
_currn->_prod = RULErule_275;
_currn->_desc1 = (_TSPParameterFactor) MkParameterFactor (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_275: root of subtree no. 1 can not be made a ParameterFactor node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_275;
return ( (NODEPTR) _currn);
}/* Mkrule_275 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_276 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_276 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_276 _currn;
#ifdef __cplusplus
_currn = new _TPrule_276;
#else
_currn = (_TPPrule_276) TreeNodeAlloc (sizeof (struct _TPrule_276));
#endif
_currn->_prod = RULErule_276;
_currn->_desc1 = (_TSPParameterFactor) MkParameterFactor (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_276: root of subtree no. 1 can not be made a ParameterFactor node ", 0, _coordref);
_currn->_desc2 = (_TSPExpOp) MkExpOp (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_276: root of subtree no. 2 can not be made a ExpOp node ", 0, _coordref);
_currn->_desc3 = (_TSPParameterPrimary) MkParameterPrimary (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_276: root of subtree no. 3 can not be made a ParameterPrimary node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_276;
return ( (NODEPTR) _currn);
}/* Mkrule_276 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_277 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_277 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_277 _currn;
#ifdef __cplusplus
_currn = new _TPrule_277;
#else
_currn = (_TPPrule_277) TreeNodeAlloc (sizeof (struct _TPrule_277));
#endif
_currn->_prod = RULErule_277;
_currn->_desc1 = (_TSPParameterPrimary) MkParameterPrimary (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_277: root of subtree no. 1 can not be made a ParameterPrimary node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_277;
return ( (NODEPTR) _currn);
}/* Mkrule_277 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_278 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_278 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_278 _currn;
#ifdef __cplusplus
_currn = new _TPrule_278;
#else
_currn = (_TPPrule_278) TreeNodeAlloc (sizeof (struct _TPrule_278));
#endif
_currn->_prod = RULErule_278;
_currn->_desc1 = (_TSPNumber) MkNumber (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_278: root of subtree no. 1 can not be made a Number node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_278;
return ( (NODEPTR) _currn);
}/* Mkrule_278 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_279 (POSITION *_coordref, int _TERM1)
#else
NODEPTR Mkrule_279 (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	_TPPrule_279 _currn;
#ifdef __cplusplus
_currn = new _TPrule_279;
#else
_currn = (_TPPrule_279) TreeNodeAlloc (sizeof (struct _TPrule_279));
#endif
_currn->_prod = RULErule_279;
_SETCOORD(_currn)
_TERMACT_rule_279;
return ( (NODEPTR) _currn);
}/* Mkrule_279 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_280 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_280 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_280 _currn;
#ifdef __cplusplus
_currn = new _TPrule_280;
#else
_currn = (_TPPrule_280) TreeNodeAlloc (sizeof (struct _TPrule_280));
#endif
_currn->_prod = RULErule_280;
_currn->_desc1 = (_TSPParameterExpression) MkParameterExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_280: root of subtree no. 1 can not be made a ParameterExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_280;
return ( (NODEPTR) _currn);
}/* Mkrule_280 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_281 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_281 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_281 _currn;
#ifdef __cplusplus
_currn = new _TPrule_281;
#else
_currn = (_TPPrule_281) TreeNodeAlloc (sizeof (struct _TPrule_281));
#endif
_currn->_prod = RULErule_281;
_currn->_desc1 = (_TSPG4) MkG4 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_281: root of subtree no. 1 can not be made a G4 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_281;
return ( (NODEPTR) _currn);
}/* Mkrule_281 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_282 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_282 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_282 _currn;
#ifdef __cplusplus
_currn = new _TPrule_282;
#else
_currn = (_TPPrule_282) TreeNodeAlloc (sizeof (struct _TPrule_282));
#endif
_currn->_prod = RULErule_282;
_currn->_desc1 = (_TSPPlace) MkPlace (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_282: root of subtree no. 1 can not be made a Place node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_282;
return ( (NODEPTR) _currn);
}/* Mkrule_282 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_283 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_283 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_283 _currn;
#ifdef __cplusplus
_currn = new _TPrule_283;
#else
_currn = (_TPPrule_283) TreeNodeAlloc (sizeof (struct _TPrule_283));
#endif
_currn->_prod = RULErule_283;
_currn->_desc1 = (_TSPG4) MkG4 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_283: root of subtree no. 1 can not be made a G4 node ", 0, _coordref);
_currn->_desc2 = (_TSPPlace) MkPlace (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_283: root of subtree no. 2 can not be made a Place node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_283;
return ( (NODEPTR) _currn);
}/* Mkrule_283 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_284 (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR Mkrule_284 (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	_TPPrule_284 _currn;
#ifdef __cplusplus
_currn = new _TPrule_284;
#else
_currn = (_TPPrule_284) TreeNodeAlloc (sizeof (struct _TPrule_284));
#endif
_currn->_prod = RULErule_284;
_currn->_desc1 = (_TSPGlobalId) MkGlobalId (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_284: root of subtree no. 1 can not be made a GlobalId node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_284;
return ( (NODEPTR) _currn);
}/* Mkrule_284 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_285 (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_285 (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_285 _currn;
#ifdef __cplusplus
_currn = new _TPrule_285;
#else
_currn = (_TPPrule_285) TreeNodeAlloc (sizeof (struct _TPrule_285));
#endif
_currn->_prod = RULErule_285;
_currn->_desc1 = (_TSPGlobalId) MkGlobalId (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_285: root of subtree no. 1 can not be made a GlobalId node ", 0, _coordref);
_currn->_desc2 = (_TSPParameterExpression) MkParameterExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_285: root of subtree no. 2 can not be made a ParameterExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_285;
return ( (NODEPTR) _currn);
}/* Mkrule_285 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_286 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_286 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_286 _currn;
#ifdef __cplusplus
_currn = new _TPrule_286;
#else
_currn = (_TPPrule_286) TreeNodeAlloc (sizeof (struct _TPrule_286));
#endif
_currn->_prod = RULErule_286;
_currn->_desc1 = (_TSPG5) MkG5 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_286: root of subtree no. 1 can not be made a G5 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_286;
return ( (NODEPTR) _currn);
}/* Mkrule_286 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_287 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_287 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_287 _currn;
#ifdef __cplusplus
_currn = new _TPrule_287;
#else
_currn = (_TPPrule_287) TreeNodeAlloc (sizeof (struct _TPrule_287));
#endif
_currn->_prod = RULErule_287;
_currn->_desc1 = (_TSPTransition) MkTransition (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_287: root of subtree no. 1 can not be made a Transition node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_287;
return ( (NODEPTR) _currn);
}/* Mkrule_287 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_288 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_288 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_288 _currn;
#ifdef __cplusplus
_currn = new _TPrule_288;
#else
_currn = (_TPPrule_288) TreeNodeAlloc (sizeof (struct _TPrule_288));
#endif
_currn->_prod = RULErule_288;
_currn->_desc1 = (_TSPG5) MkG5 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_288: root of subtree no. 1 can not be made a G5 node ", 0, _coordref);
_currn->_desc2 = (_TSPTransition) MkTransition (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_288: root of subtree no. 2 can not be made a Transition node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_288;
return ( (NODEPTR) _currn);
}/* Mkrule_288 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_289 (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR Mkrule_289 (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	_TPPrule_289 _currn;
#ifdef __cplusplus
_currn = new _TPrule_289;
#else
_currn = (_TPPrule_289) TreeNodeAlloc (sizeof (struct _TPrule_289));
#endif
_currn->_prod = RULErule_289;
_currn->_desc1 = (_TSPGlobalId) MkGlobalId (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_289: root of subtree no. 1 can not be made a GlobalId node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_289;
return ( (NODEPTR) _currn);
}/* Mkrule_289 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_290 (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_290 (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_290 _currn;
#ifdef __cplusplus
_currn = new _TPrule_290;
#else
_currn = (_TPPrule_290) TreeNodeAlloc (sizeof (struct _TPrule_290));
#endif
_currn->_prod = RULErule_290;
_currn->_desc1 = (_TSPGlobalId) MkGlobalId (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_290: root of subtree no. 1 can not be made a GlobalId node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionType) MkTransitionType (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_290: root of subtree no. 2 can not be made a TransitionType node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_290;
return ( (NODEPTR) _currn);
}/* Mkrule_290 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_291 (POSITION *_coordref)
#else
NODEPTR Mkrule_291 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_291 _currn;
#ifdef __cplusplus
_currn = new _TPrule_291;
#else
_currn = (_TPPrule_291) TreeNodeAlloc (sizeof (struct _TPrule_291));
#endif
_currn->_prod = RULErule_291;
_SETCOORD(_currn)
_TERMACT_rule_291;
return ( (NODEPTR) _currn);
}/* Mkrule_291 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_292 (POSITION *_coordref)
#else
NODEPTR Mkrule_292 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_292 _currn;
#ifdef __cplusplus
_currn = new _TPrule_292;
#else
_currn = (_TPPrule_292) TreeNodeAlloc (sizeof (struct _TPrule_292));
#endif
_currn->_prod = RULErule_292;
_SETCOORD(_currn)
_TERMACT_rule_292;
return ( (NODEPTR) _currn);
}/* Mkrule_292 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_293 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_293 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_293 _currn;
#ifdef __cplusplus
_currn = new _TPrule_293;
#else
_currn = (_TPPrule_293) TreeNodeAlloc (sizeof (struct _TPrule_293));
#endif
_currn->_prod = RULErule_293;
_currn->_desc1 = (_TSPG6) MkG6 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_293: root of subtree no. 1 can not be made a G6 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_293;
return ( (NODEPTR) _currn);
}/* Mkrule_293 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_294 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_294 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_294 _currn;
#ifdef __cplusplus
_currn = new _TPrule_294;
#else
_currn = (_TPPrule_294) TreeNodeAlloc (sizeof (struct _TPrule_294));
#endif
_currn->_prod = RULErule_294;
_currn->_desc1 = (_TSPTransitionAttributeDefItem) MkTransitionAttributeDefItem (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_294: root of subtree no. 1 can not be made a TransitionAttributeDefItem node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_294;
return ( (NODEPTR) _currn);
}/* Mkrule_294 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_295 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_295 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_295 _currn;
#ifdef __cplusplus
_currn = new _TPrule_295;
#else
_currn = (_TPPrule_295) TreeNodeAlloc (sizeof (struct _TPrule_295));
#endif
_currn->_prod = RULErule_295;
_currn->_desc1 = (_TSPG6) MkG6 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_295: root of subtree no. 1 can not be made a G6 node ", 0, _coordref);
_currn->_desc2 = (_TSPTransitionAttributeDefItem) MkTransitionAttributeDefItem (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_295: root of subtree no. 2 can not be made a TransitionAttributeDefItem node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_295;
return ( (NODEPTR) _currn);
}/* Mkrule_295 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_296 (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR Mkrule_296 (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	_TPPrule_296 _currn;
#ifdef __cplusplus
_currn = new _TPrule_296;
#else
_currn = (_TPPrule_296) TreeNodeAlloc (sizeof (struct _TPrule_296));
#endif
_currn->_prod = RULErule_296;
_currn->_desc1 = (_TSPG9) MkG9 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_296: root of subtree no. 1 can not be made a G9 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_296;
return ( (NODEPTR) _currn);
}/* Mkrule_296 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_297 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_297 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_297 _currn;
#ifdef __cplusplus
_currn = new _TPrule_297;
#else
_currn = (_TPPrule_297) TreeNodeAlloc (sizeof (struct _TPrule_297));
#endif
_currn->_prod = RULErule_297;
_currn->_desc1 = (_TSPG8) MkG8 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_297: root of subtree no. 1 can not be made a G8 node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_297;
return ( (NODEPTR) _currn);
}/* Mkrule_297 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_298 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_298 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_298 _currn;
#ifdef __cplusplus
_currn = new _TPrule_298;
#else
_currn = (_TPPrule_298) TreeNodeAlloc (sizeof (struct _TPrule_298));
#endif
_currn->_prod = RULErule_298;
_currn->_desc1 = (_TSPG7) MkG7 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_298: root of subtree no. 1 can not be made a G7 node ", 0, _coordref);
_currn->_desc2 = (_TSPG8) MkG8 (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_298: root of subtree no. 2 can not be made a G8 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_298;
return ( (NODEPTR) _currn);
}/* Mkrule_298 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_299 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_299 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_299 _currn;
#ifdef __cplusplus
_currn = new _TPrule_299;
#else
_currn = (_TPPrule_299) TreeNodeAlloc (sizeof (struct _TPrule_299));
#endif
_currn->_prod = RULErule_299;
_currn->_desc1 = (_TSPTransitionAttribute) MkTransitionAttribute (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_299: root of subtree no. 1 can not be made a TransitionAttribute node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_299;
return ( (NODEPTR) _currn);
}/* Mkrule_299 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_300 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_300 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_300 _currn;
#ifdef __cplusplus
_currn = new _TPrule_300;
#else
_currn = (_TPPrule_300) TreeNodeAlloc (sizeof (struct _TPrule_300));
#endif
_currn->_prod = RULErule_300;
_currn->_desc1 = (_TSPG7) MkG7 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_300: root of subtree no. 1 can not be made a G7 node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_300;
return ( (NODEPTR) _currn);
}/* Mkrule_300 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_301 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_301 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_301 _currn;
#ifdef __cplusplus
_currn = new _TPrule_301;
#else
_currn = (_TPPrule_301) TreeNodeAlloc (sizeof (struct _TPrule_301));
#endif
_currn->_prod = RULErule_301;
_currn->_desc1 = (_TSPPriorityAttribute) MkPriorityAttribute (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_301: root of subtree no. 1 can not be made a PriorityAttribute node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_301;
return ( (NODEPTR) _currn);
}/* Mkrule_301 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_302 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_302 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_302 _currn;
#ifdef __cplusplus
_currn = new _TPrule_302;
#else
_currn = (_TPPrule_302) TreeNodeAlloc (sizeof (struct _TPrule_302));
#endif
_currn->_prod = RULErule_302;
_currn->_desc1 = (_TSPDistributionAttribute) MkDistributionAttribute (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_302: root of subtree no. 1 can not be made a DistributionAttribute node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_302;
return ( (NODEPTR) _currn);
}/* Mkrule_302 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_303 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_303 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_303 _currn;
#ifdef __cplusplus
_currn = new _TPrule_303;
#else
_currn = (_TPPrule_303) TreeNodeAlloc (sizeof (struct _TPrule_303));
#endif
_currn->_prod = RULErule_303;
_currn->_desc1 = (_TSPWeightAttribute) MkWeightAttribute (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_303: root of subtree no. 1 can not be made a WeightAttribute node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_303;
return ( (NODEPTR) _currn);
}/* Mkrule_303 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_304 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_304 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_304 _currn;
#ifdef __cplusplus
_currn = new _TPrule_304;
#else
_currn = (_TPPrule_304) TreeNodeAlloc (sizeof (struct _TPrule_304));
#endif
_currn->_prod = RULErule_304;
_currn->_desc1 = (_TSPMarkingDependencyAttribute) MkMarkingDependencyAttribute (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_304: root of subtree no. 1 can not be made a MarkingDependencyAttribute node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_304;
return ( (NODEPTR) _currn);
}/* Mkrule_304 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_305 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_305 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_305 _currn;
#ifdef __cplusplus
_currn = new _TPrule_305;
#else
_currn = (_TPPrule_305) TreeNodeAlloc (sizeof (struct _TPrule_305));
#endif
_currn->_prod = RULErule_305;
_currn->_desc1 = (_TSPPolicyAttribute) MkPolicyAttribute (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_305: root of subtree no. 1 can not be made a PolicyAttribute node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_305;
return ( (NODEPTR) _currn);
}/* Mkrule_305 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_306 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_306 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_306 _currn;
#ifdef __cplusplus
_currn = new _TPrule_306;
#else
_currn = (_TPPrule_306) TreeNodeAlloc (sizeof (struct _TPrule_306));
#endif
_currn->_prod = RULErule_306;
_currn->_desc1 = (_TSPConcurrencyAttribute) MkConcurrencyAttribute (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_306: root of subtree no. 1 can not be made a ConcurrencyAttribute node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_306;
return ( (NODEPTR) _currn);
}/* Mkrule_306 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_307 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_307 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_307 _currn;
#ifdef __cplusplus
_currn = new _TPrule_307;
#else
_currn = (_TPPrule_307) TreeNodeAlloc (sizeof (struct _TPrule_307));
#endif
_currn->_prod = RULErule_307;
_currn->_desc1 = (_TSPGuardAttribute) MkGuardAttribute (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_307: root of subtree no. 1 can not be made a GuardAttribute node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_307;
return ( (NODEPTR) _currn);
}/* Mkrule_307 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_308 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_308 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_308 _currn;
#ifdef __cplusplus
_currn = new _TPrule_308;
#else
_currn = (_TPPrule_308) TreeNodeAlloc (sizeof (struct _TPrule_308));
#endif
_currn->_prod = RULErule_308;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_308: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_308;
return ( (NODEPTR) _currn);
}/* Mkrule_308 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_309 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_309 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_309 _currn;
#ifdef __cplusplus
_currn = new _TPrule_309;
#else
_currn = (_TPPrule_309) TreeNodeAlloc (sizeof (struct _TPrule_309));
#endif
_currn->_prod = RULErule_309;
_currn->_desc1 = (_TSPDistributionExpression) MkDistributionExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_309: root of subtree no. 1 can not be made a DistributionExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_309;
return ( (NODEPTR) _currn);
}/* Mkrule_309 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_310 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_310 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_310 _currn;
#ifdef __cplusplus
_currn = new _TPrule_310;
#else
_currn = (_TPPrule_310) TreeNodeAlloc (sizeof (struct _TPrule_310));
#endif
_currn->_prod = RULErule_310;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_310: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_310;
return ( (NODEPTR) _currn);
}/* Mkrule_310 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_311 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_311 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_311 _currn;
#ifdef __cplusplus
_currn = new _TPrule_311;
#else
_currn = (_TPPrule_311) TreeNodeAlloc (sizeof (struct _TPrule_311));
#endif
_currn->_prod = RULErule_311;
_currn->_desc1 = (_TSPMarkingDependencyType) MkMarkingDependencyType (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_311: root of subtree no. 1 can not be made a MarkingDependencyType node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_311;
return ( (NODEPTR) _currn);
}/* Mkrule_311 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_312 (POSITION *_coordref)
#else
NODEPTR Mkrule_312 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_312 _currn;
#ifdef __cplusplus
_currn = new _TPrule_312;
#else
_currn = (_TPPrule_312) TreeNodeAlloc (sizeof (struct _TPrule_312));
#endif
_currn->_prod = RULErule_312;
_SETCOORD(_currn)
_TERMACT_rule_312;
return ( (NODEPTR) _currn);
}/* Mkrule_312 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_313 (POSITION *_coordref)
#else
NODEPTR Mkrule_313 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_313 _currn;
#ifdef __cplusplus
_currn = new _TPrule_313;
#else
_currn = (_TPPrule_313) TreeNodeAlloc (sizeof (struct _TPrule_313));
#endif
_currn->_prod = RULErule_313;
_SETCOORD(_currn)
_TERMACT_rule_313;
return ( (NODEPTR) _currn);
}/* Mkrule_313 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_314 (POSITION *_coordref)
#else
NODEPTR Mkrule_314 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_314 _currn;
#ifdef __cplusplus
_currn = new _TPrule_314;
#else
_currn = (_TPPrule_314) TreeNodeAlloc (sizeof (struct _TPrule_314));
#endif
_currn->_prod = RULErule_314;
_SETCOORD(_currn)
_TERMACT_rule_314;
return ( (NODEPTR) _currn);
}/* Mkrule_314 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_315 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_315 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_315 _currn;
#ifdef __cplusplus
_currn = new _TPrule_315;
#else
_currn = (_TPPrule_315) TreeNodeAlloc (sizeof (struct _TPrule_315));
#endif
_currn->_prod = RULErule_315;
_currn->_desc1 = (_TSPPolicyType) MkPolicyType (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_315: root of subtree no. 1 can not be made a PolicyType node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_315;
return ( (NODEPTR) _currn);
}/* Mkrule_315 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_316 (POSITION *_coordref)
#else
NODEPTR Mkrule_316 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_316 _currn;
#ifdef __cplusplus
_currn = new _TPrule_316;
#else
_currn = (_TPPrule_316) TreeNodeAlloc (sizeof (struct _TPrule_316));
#endif
_currn->_prod = RULErule_316;
_SETCOORD(_currn)
_TERMACT_rule_316;
return ( (NODEPTR) _currn);
}/* Mkrule_316 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_317 (POSITION *_coordref)
#else
NODEPTR Mkrule_317 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_317 _currn;
#ifdef __cplusplus
_currn = new _TPrule_317;
#else
_currn = (_TPPrule_317) TreeNodeAlloc (sizeof (struct _TPrule_317));
#endif
_currn->_prod = RULErule_317;
_SETCOORD(_currn)
_TERMACT_rule_317;
return ( (NODEPTR) _currn);
}/* Mkrule_317 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_318 (POSITION *_coordref)
#else
NODEPTR Mkrule_318 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_318 _currn;
#ifdef __cplusplus
_currn = new _TPrule_318;
#else
_currn = (_TPPrule_318) TreeNodeAlloc (sizeof (struct _TPrule_318));
#endif
_currn->_prod = RULErule_318;
_SETCOORD(_currn)
_TERMACT_rule_318;
return ( (NODEPTR) _currn);
}/* Mkrule_318 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_319 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_319 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_319 _currn;
#ifdef __cplusplus
_currn = new _TPrule_319;
#else
_currn = (_TPPrule_319) TreeNodeAlloc (sizeof (struct _TPrule_319));
#endif
_currn->_prod = RULErule_319;
_currn->_desc1 = (_TSPConcurrencyDegree) MkConcurrencyDegree (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_319: root of subtree no. 1 can not be made a ConcurrencyDegree node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_319;
return ( (NODEPTR) _currn);
}/* Mkrule_319 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_320 (POSITION *_coordref)
#else
NODEPTR Mkrule_320 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_320 _currn;
#ifdef __cplusplus
_currn = new _TPrule_320;
#else
_currn = (_TPPrule_320) TreeNodeAlloc (sizeof (struct _TPrule_320));
#endif
_currn->_prod = RULErule_320;
_SETCOORD(_currn)
_TERMACT_rule_320;
return ( (NODEPTR) _currn);
}/* Mkrule_320 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_321 (POSITION *_coordref)
#else
NODEPTR Mkrule_321 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_321 _currn;
#ifdef __cplusplus
_currn = new _TPrule_321;
#else
_currn = (_TPPrule_321) TreeNodeAlloc (sizeof (struct _TPrule_321));
#endif
_currn->_prod = RULErule_321;
_SETCOORD(_currn)
_TERMACT_rule_321;
return ( (NODEPTR) _currn);
}/* Mkrule_321 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_322 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_322 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_322 _currn;
#ifdef __cplusplus
_currn = new _TPrule_322;
#else
_currn = (_TPPrule_322) TreeNodeAlloc (sizeof (struct _TPrule_322));
#endif
_currn->_prod = RULErule_322;
_currn->_desc1 = (_TSPPredicate) MkPredicate (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_322: root of subtree no. 1 can not be made a Predicate node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_322;
return ( (NODEPTR) _currn);
}/* Mkrule_322 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_323 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_323 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_323 _currn;
#ifdef __cplusplus
_currn = new _TPrule_323;
#else
_currn = (_TPPrule_323) TreeNodeAlloc (sizeof (struct _TPrule_323));
#endif
_currn->_prod = RULErule_323;
_currn->_desc1 = (_TSPG10) MkG10 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_323: root of subtree no. 1 can not be made a G10 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_323;
return ( (NODEPTR) _currn);
}/* Mkrule_323 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_324 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_324 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_324 _currn;
#ifdef __cplusplus
_currn = new _TPrule_324;
#else
_currn = (_TPPrule_324) TreeNodeAlloc (sizeof (struct _TPrule_324));
#endif
_currn->_prod = RULErule_324;
_currn->_desc1 = (_TSPArc) MkArc (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_324: root of subtree no. 1 can not be made a Arc node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_324;
return ( (NODEPTR) _currn);
}/* Mkrule_324 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_325 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_325 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_325 _currn;
#ifdef __cplusplus
_currn = new _TPrule_325;
#else
_currn = (_TPPrule_325) TreeNodeAlloc (sizeof (struct _TPrule_325));
#endif
_currn->_prod = RULErule_325;
_currn->_desc1 = (_TSPG10) MkG10 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_325: root of subtree no. 1 can not be made a G10 node ", 0, _coordref);
_currn->_desc2 = (_TSPArc) MkArc (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_325: root of subtree no. 2 can not be made a Arc node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_325;
return ( (NODEPTR) _currn);
}/* Mkrule_325 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_326 (POSITION *_coordref, int _TERM1, NODEPTR _desc1, int _TERM2, int _TERM3, NODEPTR _desc2)
#else
NODEPTR Mkrule_326 (_coordref, _TERM1,_desc1, _TERM2, _TERM3,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	int _TERM2;
	int _TERM3;
	NODEPTR _desc2;
#endif
{	_TPPrule_326 _currn;
#ifdef __cplusplus
_currn = new _TPrule_326;
#else
_currn = (_TPPrule_326) TreeNodeAlloc (sizeof (struct _TPrule_326));
#endif
_currn->_prod = RULErule_326;
_currn->_desc1 = (_TSPGlobalId) MkGlobalId (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_326: root of subtree no. 1 can not be made a GlobalId node ", 0, _coordref);
_currn->_desc2 = (_TSPG12) MkG12 (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_326: root of subtree no. 2 can not be made a G12 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_326;
return ( (NODEPTR) _currn);
}/* Mkrule_326 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_327 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_327 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_327 _currn;
#ifdef __cplusplus
_currn = new _TPrule_327;
#else
_currn = (_TPPrule_327) TreeNodeAlloc (sizeof (struct _TPrule_327));
#endif
_currn->_prod = RULErule_327;
_currn->_desc1 = (_TSPArcAttribute) MkArcAttribute (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_327: root of subtree no. 1 can not be made a ArcAttribute node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_327;
return ( (NODEPTR) _currn);
}/* Mkrule_327 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_328 (POSITION *_coordref)
#else
NODEPTR Mkrule_328 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_328 _currn;
#ifdef __cplusplus
_currn = new _TPrule_328;
#else
_currn = (_TPPrule_328) TreeNodeAlloc (sizeof (struct _TPrule_328));
#endif
_currn->_prod = RULErule_328;
_SETCOORD(_currn)
_TERMACT_rule_328;
return ( (NODEPTR) _currn);
}/* Mkrule_328 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_329 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_329 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_329 _currn;
#ifdef __cplusplus
_currn = new _TPrule_329;
#else
_currn = (_TPPrule_329) TreeNodeAlloc (sizeof (struct _TPrule_329));
#endif
_currn->_prod = RULErule_329;
_currn->_desc1 = (_TSPG12) MkG12 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_329: root of subtree no. 1 can not be made a G12 node ", 0, _coordref);
_currn->_desc2 = (_TSPG11) MkG11 (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_329: root of subtree no. 2 can not be made a G11 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_329;
return ( (NODEPTR) _currn);
}/* Mkrule_329 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_330 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_330 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_330 _currn;
#ifdef __cplusplus
_currn = new _TPrule_330;
#else
_currn = (_TPPrule_330) TreeNodeAlloc (sizeof (struct _TPrule_330));
#endif
_currn->_prod = RULErule_330;
_currn->_desc1 = (_TSPArcTypeAttribute) MkArcTypeAttribute (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_330: root of subtree no. 1 can not be made a ArcTypeAttribute node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_330;
return ( (NODEPTR) _currn);
}/* Mkrule_330 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_331 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_331 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_331 _currn;
#ifdef __cplusplus
_currn = new _TPrule_331;
#else
_currn = (_TPPrule_331) TreeNodeAlloc (sizeof (struct _TPrule_331));
#endif
_currn->_prod = RULErule_331;
_currn->_desc1 = (_TSPMultiplicityAttribute) MkMultiplicityAttribute (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_331: root of subtree no. 1 can not be made a MultiplicityAttribute node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_331;
return ( (NODEPTR) _currn);
}/* Mkrule_331 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_332 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_332 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_332 _currn;
#ifdef __cplusplus
_currn = new _TPrule_332;
#else
_currn = (_TPPrule_332) TreeNodeAlloc (sizeof (struct _TPrule_332));
#endif
_currn->_prod = RULErule_332;
_currn->_desc1 = (_TSPArcType) MkArcType (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_332: root of subtree no. 1 can not be made a ArcType node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_332;
return ( (NODEPTR) _currn);
}/* Mkrule_332 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_333 (POSITION *_coordref)
#else
NODEPTR Mkrule_333 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_333 _currn;
#ifdef __cplusplus
_currn = new _TPrule_333;
#else
_currn = (_TPPrule_333) TreeNodeAlloc (sizeof (struct _TPrule_333));
#endif
_currn->_prod = RULErule_333;
_SETCOORD(_currn)
_TERMACT_rule_333;
return ( (NODEPTR) _currn);
}/* Mkrule_333 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_334 (POSITION *_coordref)
#else
NODEPTR Mkrule_334 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_334 _currn;
#ifdef __cplusplus
_currn = new _TPrule_334;
#else
_currn = (_TPPrule_334) TreeNodeAlloc (sizeof (struct _TPrule_334));
#endif
_currn->_prod = RULErule_334;
_SETCOORD(_currn)
_TERMACT_rule_334;
return ( (NODEPTR) _currn);
}/* Mkrule_334 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_335 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_335 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_335 _currn;
#ifdef __cplusplus
_currn = new _TPrule_335;
#else
_currn = (_TPPrule_335) TreeNodeAlloc (sizeof (struct _TPrule_335));
#endif
_currn->_prod = RULErule_335;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_335: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_335;
return ( (NODEPTR) _currn);
}/* Mkrule_335 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_336 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_336 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_336 _currn;
#ifdef __cplusplus
_currn = new _TPrule_336;
#else
_currn = (_TPPrule_336) TreeNodeAlloc (sizeof (struct _TPrule_336));
#endif
_currn->_prod = RULErule_336;
_currn->_desc1 = (_TSPG13) MkG13 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_336: root of subtree no. 1 can not be made a G13 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_336;
return ( (NODEPTR) _currn);
}/* Mkrule_336 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_337 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_337 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_337 _currn;
#ifdef __cplusplus
_currn = new _TPrule_337;
#else
_currn = (_TPPrule_337) TreeNodeAlloc (sizeof (struct _TPrule_337));
#endif
_currn->_prod = RULErule_337;
_currn->_desc1 = (_TSPReward) MkReward (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_337: root of subtree no. 1 can not be made a Reward node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_337;
return ( (NODEPTR) _currn);
}/* Mkrule_337 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_338 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_338 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_338 _currn;
#ifdef __cplusplus
_currn = new _TPrule_338;
#else
_currn = (_TPPrule_338) TreeNodeAlloc (sizeof (struct _TPrule_338));
#endif
_currn->_prod = RULErule_338;
_currn->_desc1 = (_TSPG13) MkG13 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_338: root of subtree no. 1 can not be made a G13 node ", 0, _coordref);
_currn->_desc2 = (_TSPReward) MkReward (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_338: root of subtree no. 2 can not be made a Reward node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_338;
return ( (NODEPTR) _currn);
}/* Mkrule_338 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_339 (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_339 (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_339 _currn;
#ifdef __cplusplus
_currn = new _TPrule_339;
#else
_currn = (_TPPrule_339) TreeNodeAlloc (sizeof (struct _TPrule_339));
#endif
_currn->_prod = RULErule_339;
_currn->_desc1 = (_TSPGlobalId) MkGlobalId (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_339: root of subtree no. 1 can not be made a GlobalId node ", 0, _coordref);
_currn->_desc2 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_339: root of subtree no. 2 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_339;
return ( (NODEPTR) _currn);
}/* Mkrule_339 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_340 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_340 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_340 _currn;
#ifdef __cplusplus
_currn = new _TPrule_340;
#else
_currn = (_TPPrule_340) TreeNodeAlloc (sizeof (struct _TPrule_340));
#endif
_currn->_prod = RULErule_340;
_currn->_desc1 = (_TSPPredicate) MkPredicate (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_340: root of subtree no. 1 can not be made a Predicate node ", 0, _coordref);
_currn->_desc2 = (_TSPRewardSign) MkRewardSign (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_340: root of subtree no. 2 can not be made a RewardSign node ", 0, _coordref);
_currn->_desc3 = (_TSPG15) MkG15 (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_340: root of subtree no. 3 can not be made a G15 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_340;
return ( (NODEPTR) _currn);
}/* Mkrule_340 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_341 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR Mkrule_341 (_coordref,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	_TPPrule_341 _currn;
#ifdef __cplusplus
_currn = new _TPrule_341;
#else
_currn = (_TPPrule_341) TreeNodeAlloc (sizeof (struct _TPrule_341));
#endif
_currn->_prod = RULErule_341;
_currn->_desc1 = (_TSPPredicate) MkPredicate (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_341: root of subtree no. 1 can not be made a Predicate node ", 0, _coordref);
_currn->_desc2 = (_TSPRewardSign) MkRewardSign (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_341: root of subtree no. 2 can not be made a RewardSign node ", 0, _coordref);
_currn->_desc3 = (_TSPG15) MkG15 (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_341: root of subtree no. 3 can not be made a G15 node ", 0, _coordref);
_currn->_desc4 = (_TSPRewardSign) MkRewardSign (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_341: root of subtree no. 4 can not be made a RewardSign node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_341;
return ( (NODEPTR) _currn);
}/* Mkrule_341 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_342 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_342 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_342 _currn;
#ifdef __cplusplus
_currn = new _TPrule_342;
#else
_currn = (_TPPrule_342) TreeNodeAlloc (sizeof (struct _TPrule_342));
#endif
_currn->_prod = RULErule_342;
_currn->_desc1 = (_TSPRewardSign) MkRewardSign (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_342: root of subtree no. 1 can not be made a RewardSign node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_342;
return ( (NODEPTR) _currn);
}/* Mkrule_342 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_343 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_343 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_343 _currn;
#ifdef __cplusplus
_currn = new _TPrule_343;
#else
_currn = (_TPPrule_343) TreeNodeAlloc (sizeof (struct _TPrule_343));
#endif
_currn->_prod = RULErule_343;
_currn->_desc1 = (_TSPPredicate) MkPredicate (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_343: root of subtree no. 1 can not be made a Predicate node ", 0, _coordref);
_currn->_desc2 = (_TSPRewardSign) MkRewardSign (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_343: root of subtree no. 2 can not be made a RewardSign node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_343;
return ( (NODEPTR) _currn);
}/* Mkrule_343 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_344 (POSITION *_coordref)
#else
NODEPTR Mkrule_344 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_344 _currn;
#ifdef __cplusplus
_currn = new _TPrule_344;
#else
_currn = (_TPPrule_344) TreeNodeAlloc (sizeof (struct _TPrule_344));
#endif
_currn->_prod = RULErule_344;
_SETCOORD(_currn)
_TERMACT_rule_344;
return ( (NODEPTR) _currn);
}/* Mkrule_344 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_345 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_345 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_345 _currn;
#ifdef __cplusplus
_currn = new _TPrule_345;
#else
_currn = (_TPPrule_345) TreeNodeAlloc (sizeof (struct _TPrule_345));
#endif
_currn->_prod = RULErule_345;
_currn->_desc1 = (_TSPG15) MkG15 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_345: root of subtree no. 1 can not be made a G15 node ", 0, _coordref);
_currn->_desc2 = (_TSPG14) MkG14 (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_345: root of subtree no. 2 can not be made a G14 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_345;
return ( (NODEPTR) _currn);
}/* Mkrule_345 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_346 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_346 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_346 _currn;
#ifdef __cplusplus
_currn = new _TPrule_346;
#else
_currn = (_TPPrule_346) TreeNodeAlloc (sizeof (struct _TPrule_346));
#endif
_currn->_prod = RULErule_346;
_currn->_desc1 = (_TSPSign) MkSign (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_346: root of subtree no. 1 can not be made a Sign node ", 0, _coordref);
_currn->_desc2 = (_TSPRewardSum) MkRewardSum (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_346: root of subtree no. 2 can not be made a RewardSum node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_346;
return ( (NODEPTR) _currn);
}/* Mkrule_346 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_347 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_347 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_347 _currn;
#ifdef __cplusplus
_currn = new _TPrule_347;
#else
_currn = (_TPPrule_347) TreeNodeAlloc (sizeof (struct _TPrule_347));
#endif
_currn->_prod = RULErule_347;
_currn->_desc1 = (_TSPRewardSum) MkRewardSum (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_347: root of subtree no. 1 can not be made a RewardSum node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_347;
return ( (NODEPTR) _currn);
}/* Mkrule_347 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_348 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_348 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_348 _currn;
#ifdef __cplusplus
_currn = new _TPrule_348;
#else
_currn = (_TPPrule_348) TreeNodeAlloc (sizeof (struct _TPrule_348));
#endif
_currn->_prod = RULErule_348;
_currn->_desc1 = (_TSPRewardSum) MkRewardSum (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_348: root of subtree no. 1 can not be made a RewardSum node ", 0, _coordref);
_currn->_desc2 = (_TSPAddOp) MkAddOp (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_348: root of subtree no. 2 can not be made a AddOp node ", 0, _coordref);
_currn->_desc3 = (_TSPRewardTerm) MkRewardTerm (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_348: root of subtree no. 3 can not be made a RewardTerm node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_348;
return ( (NODEPTR) _currn);
}/* Mkrule_348 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_349 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_349 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_349 _currn;
#ifdef __cplusplus
_currn = new _TPrule_349;
#else
_currn = (_TPPrule_349) TreeNodeAlloc (sizeof (struct _TPrule_349));
#endif
_currn->_prod = RULErule_349;
_currn->_desc1 = (_TSPRewardTerm) MkRewardTerm (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_349: root of subtree no. 1 can not be made a RewardTerm node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_349;
return ( (NODEPTR) _currn);
}/* Mkrule_349 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_350 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_350 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_350 _currn;
#ifdef __cplusplus
_currn = new _TPrule_350;
#else
_currn = (_TPPrule_350) TreeNodeAlloc (sizeof (struct _TPrule_350));
#endif
_currn->_prod = RULErule_350;
_currn->_desc1 = (_TSPRewardTerm) MkRewardTerm (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_350: root of subtree no. 1 can not be made a RewardTerm node ", 0, _coordref);
_currn->_desc2 = (_TSPMulOp) MkMulOp (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_350: root of subtree no. 2 can not be made a MulOp node ", 0, _coordref);
_currn->_desc3 = (_TSPRewardFactor) MkRewardFactor (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_350: root of subtree no. 3 can not be made a RewardFactor node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_350;
return ( (NODEPTR) _currn);
}/* Mkrule_350 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_351 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_351 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_351 _currn;
#ifdef __cplusplus
_currn = new _TPrule_351;
#else
_currn = (_TPPrule_351) TreeNodeAlloc (sizeof (struct _TPrule_351));
#endif
_currn->_prod = RULErule_351;
_currn->_desc1 = (_TSPRewardFactor) MkRewardFactor (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_351: root of subtree no. 1 can not be made a RewardFactor node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_351;
return ( (NODEPTR) _currn);
}/* Mkrule_351 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_352 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_352 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_352 _currn;
#ifdef __cplusplus
_currn = new _TPrule_352;
#else
_currn = (_TPPrule_352) TreeNodeAlloc (sizeof (struct _TPrule_352));
#endif
_currn->_prod = RULErule_352;
_currn->_desc1 = (_TSPRewardFactor) MkRewardFactor (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_352: root of subtree no. 1 can not be made a RewardFactor node ", 0, _coordref);
_currn->_desc2 = (_TSPExpOp) MkExpOp (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_352: root of subtree no. 2 can not be made a ExpOp node ", 0, _coordref);
_currn->_desc3 = (_TSPRewardPrimary) MkRewardPrimary (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_352: root of subtree no. 3 can not be made a RewardPrimary node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_352;
return ( (NODEPTR) _currn);
}/* Mkrule_352 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_353 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_353 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_353 _currn;
#ifdef __cplusplus
_currn = new _TPrule_353;
#else
_currn = (_TPPrule_353) TreeNodeAlloc (sizeof (struct _TPrule_353));
#endif
_currn->_prod = RULErule_353;
_currn->_desc1 = (_TSPRewardPrimary) MkRewardPrimary (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_353: root of subtree no. 1 can not be made a RewardPrimary node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_353;
return ( (NODEPTR) _currn);
}/* Mkrule_353 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_354 (POSITION *_coordref, int _TERM1)
#else
NODEPTR Mkrule_354 (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	_TPPrule_354 _currn;
#ifdef __cplusplus
_currn = new _TPrule_354;
#else
_currn = (_TPPrule_354) TreeNodeAlloc (sizeof (struct _TPrule_354));
#endif
_currn->_prod = RULErule_354;
_SETCOORD(_currn)
_TERMACT_rule_354;
return ( (NODEPTR) _currn);
}/* Mkrule_354 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_355 (POSITION *_coordref, int _TERM1)
#else
NODEPTR Mkrule_355 (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	_TPPrule_355 _currn;
#ifdef __cplusplus
_currn = new _TPrule_355;
#else
_currn = (_TPPrule_355) TreeNodeAlloc (sizeof (struct _TPrule_355));
#endif
_currn->_prod = RULErule_355;
_SETCOORD(_currn)
_TERMACT_rule_355;
return ( (NODEPTR) _currn);
}/* Mkrule_355 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_356 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_356 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_356 _currn;
#ifdef __cplusplus
_currn = new _TPrule_356;
#else
_currn = (_TPPrule_356) TreeNodeAlloc (sizeof (struct _TPrule_356));
#endif
_currn->_prod = RULErule_356;
_currn->_desc1 = (_TSPNumber) MkNumber (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_356: root of subtree no. 1 can not be made a Number node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_356;
return ( (NODEPTR) _currn);
}/* Mkrule_356 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_357 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_357 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_357 _currn;
#ifdef __cplusplus
_currn = new _TPrule_357;
#else
_currn = (_TPPrule_357) TreeNodeAlloc (sizeof (struct _TPrule_357));
#endif
_currn->_prod = RULErule_357;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_357: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_357;
return ( (NODEPTR) _currn);
}/* Mkrule_357 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_358 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_358 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_358 _currn;
#ifdef __cplusplus
_currn = new _TPrule_358;
#else
_currn = (_TPPrule_358) TreeNodeAlloc (sizeof (struct _TPrule_358));
#endif
_currn->_prod = RULErule_358;
_currn->_desc1 = (_TSPPredicateSum) MkPredicateSum (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_358: root of subtree no. 1 can not be made a PredicateSum node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_358;
return ( (NODEPTR) _currn);
}/* Mkrule_358 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_359 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_359 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_359 _currn;
#ifdef __cplusplus
_currn = new _TPrule_359;
#else
_currn = (_TPPrule_359) TreeNodeAlloc (sizeof (struct _TPrule_359));
#endif
_currn->_prod = RULErule_359;
_currn->_desc1 = (_TSPPredicateSum) MkPredicateSum (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_359: root of subtree no. 1 can not be made a PredicateSum node ", 0, _coordref);
_currn->_desc2 = (_TSPOrOp) MkOrOp (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_359: root of subtree no. 2 can not be made a OrOp node ", 0, _coordref);
_currn->_desc3 = (_TSPPredicateTerm) MkPredicateTerm (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_359: root of subtree no. 3 can not be made a PredicateTerm node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_359;
return ( (NODEPTR) _currn);
}/* Mkrule_359 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_360 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_360 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_360 _currn;
#ifdef __cplusplus
_currn = new _TPrule_360;
#else
_currn = (_TPPrule_360) TreeNodeAlloc (sizeof (struct _TPrule_360));
#endif
_currn->_prod = RULErule_360;
_currn->_desc1 = (_TSPPredicateTerm) MkPredicateTerm (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_360: root of subtree no. 1 can not be made a PredicateTerm node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_360;
return ( (NODEPTR) _currn);
}/* Mkrule_360 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_361 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_361 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_361 _currn;
#ifdef __cplusplus
_currn = new _TPrule_361;
#else
_currn = (_TPPrule_361) TreeNodeAlloc (sizeof (struct _TPrule_361));
#endif
_currn->_prod = RULErule_361;
_currn->_desc1 = (_TSPPredicateTerm) MkPredicateTerm (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_361: root of subtree no. 1 can not be made a PredicateTerm node ", 0, _coordref);
_currn->_desc2 = (_TSPAndOp) MkAndOp (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_361: root of subtree no. 2 can not be made a AndOp node ", 0, _coordref);
_currn->_desc3 = (_TSPPredicateFactor) MkPredicateFactor (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_361: root of subtree no. 3 can not be made a PredicateFactor node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_361;
return ( (NODEPTR) _currn);
}/* Mkrule_361 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_362 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_362 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_362 _currn;
#ifdef __cplusplus
_currn = new _TPrule_362;
#else
_currn = (_TPPrule_362) TreeNodeAlloc (sizeof (struct _TPrule_362));
#endif
_currn->_prod = RULErule_362;
_currn->_desc1 = (_TSPPredicateFactor) MkPredicateFactor (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_362: root of subtree no. 1 can not be made a PredicateFactor node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_362;
return ( (NODEPTR) _currn);
}/* Mkrule_362 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_363 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_363 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_363 _currn;
#ifdef __cplusplus
_currn = new _TPrule_363;
#else
_currn = (_TPPrule_363) TreeNodeAlloc (sizeof (struct _TPrule_363));
#endif
_currn->_prod = RULErule_363;
_currn->_desc1 = (_TSPNotOp) MkNotOp (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_363: root of subtree no. 1 can not be made a NotOp node ", 0, _coordref);
_currn->_desc2 = (_TSPPredicateFactor) MkPredicateFactor (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_363: root of subtree no. 2 can not be made a PredicateFactor node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_363;
return ( (NODEPTR) _currn);
}/* Mkrule_363 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_364 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_364 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_364 _currn;
#ifdef __cplusplus
_currn = new _TPrule_364;
#else
_currn = (_TPPrule_364) TreeNodeAlloc (sizeof (struct _TPrule_364));
#endif
_currn->_prod = RULErule_364;
_currn->_desc1 = (_TSPPredicatePrimary) MkPredicatePrimary (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_364: root of subtree no. 1 can not be made a PredicatePrimary node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_364;
return ( (NODEPTR) _currn);
}/* Mkrule_364 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_365 (POSITION *_coordref)
#else
NODEPTR Mkrule_365 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_365 _currn;
#ifdef __cplusplus
_currn = new _TPrule_365;
#else
_currn = (_TPPrule_365) TreeNodeAlloc (sizeof (struct _TPrule_365));
#endif
_currn->_prod = RULErule_365;
_SETCOORD(_currn)
_TERMACT_rule_365;
return ( (NODEPTR) _currn);
}/* Mkrule_365 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_366 (POSITION *_coordref)
#else
NODEPTR Mkrule_366 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_366 _currn;
#ifdef __cplusplus
_currn = new _TPrule_366;
#else
_currn = (_TPPrule_366) TreeNodeAlloc (sizeof (struct _TPrule_366));
#endif
_currn->_prod = RULErule_366;
_SETCOORD(_currn)
_TERMACT_rule_366;
return ( (NODEPTR) _currn);
}/* Mkrule_366 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_367 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_367 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_367 _currn;
#ifdef __cplusplus
_currn = new _TPrule_367;
#else
_currn = (_TPPrule_367) TreeNodeAlloc (sizeof (struct _TPrule_367));
#endif
_currn->_prod = RULErule_367;
_currn->_desc1 = (_TSPPredicateComp) MkPredicateComp (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_367: root of subtree no. 1 can not be made a PredicateComp node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_367;
return ( (NODEPTR) _currn);
}/* Mkrule_367 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_368 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_368 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_368 _currn;
#ifdef __cplusplus
_currn = new _TPrule_368;
#else
_currn = (_TPPrule_368) TreeNodeAlloc (sizeof (struct _TPrule_368));
#endif
_currn->_prod = RULErule_368;
_currn->_desc1 = (_TSPPredicate) MkPredicate (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_368: root of subtree no. 1 can not be made a Predicate node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_368;
return ( (NODEPTR) _currn);
}/* Mkrule_368 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_369 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_369 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_369 _currn;
#ifdef __cplusplus
_currn = new _TPrule_369;
#else
_currn = (_TPPrule_369) TreeNodeAlloc (sizeof (struct _TPrule_369));
#endif
_currn->_prod = RULErule_369;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_369: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc2 = (_TSPCompOp) MkCompOp (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_369: root of subtree no. 2 can not be made a CompOp node ", 0, _coordref);
_currn->_desc3 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_369: root of subtree no. 3 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_369;
return ( (NODEPTR) _currn);
}/* Mkrule_369 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_370 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_370 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_370 _currn;
#ifdef __cplusplus
_currn = new _TPrule_370;
#else
_currn = (_TPPrule_370) TreeNodeAlloc (sizeof (struct _TPrule_370));
#endif
_currn->_prod = RULErule_370;
_currn->_desc1 = (_TSPG16) MkG16 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_370: root of subtree no. 1 can not be made a G16 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_370;
return ( (NODEPTR) _currn);
}/* Mkrule_370 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_371 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_371 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_371 _currn;
#ifdef __cplusplus
_currn = new _TPrule_371;
#else
_currn = (_TPPrule_371) TreeNodeAlloc (sizeof (struct _TPrule_371));
#endif
_currn->_prod = RULErule_371;
_currn->_desc1 = (_TSPMeasure) MkMeasure (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_371: root of subtree no. 1 can not be made a Measure node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_371;
return ( (NODEPTR) _currn);
}/* Mkrule_371 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_372 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_372 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_372 _currn;
#ifdef __cplusplus
_currn = new _TPrule_372;
#else
_currn = (_TPPrule_372) TreeNodeAlloc (sizeof (struct _TPrule_372));
#endif
_currn->_prod = RULErule_372;
_currn->_desc1 = (_TSPG16) MkG16 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_372: root of subtree no. 1 can not be made a G16 node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasure) MkMeasure (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_372: root of subtree no. 2 can not be made a Measure node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_372;
return ( (NODEPTR) _currn);
}/* Mkrule_372 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_373 (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_373 (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_373 _currn;
#ifdef __cplusplus
_currn = new _TPrule_373;
#else
_currn = (_TPPrule_373) TreeNodeAlloc (sizeof (struct _TPrule_373));
#endif
_currn->_prod = RULErule_373;
_currn->_desc1 = (_TSPGlobalId) MkGlobalId (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_373: root of subtree no. 1 can not be made a GlobalId node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasureExpression) MkMeasureExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_373: root of subtree no. 2 can not be made a MeasureExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_373;
return ( (NODEPTR) _currn);
}/* Mkrule_373 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_374 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_374 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_374 _currn;
#ifdef __cplusplus
_currn = new _TPrule_374;
#else
_currn = (_TPPrule_374) TreeNodeAlloc (sizeof (struct _TPrule_374));
#endif
_currn->_prod = RULErule_374;
_currn->_desc1 = (_TSPSingleMeasureExpr) MkSingleMeasureExpr (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_374: root of subtree no. 1 can not be made a SingleMeasureExpr node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_374;
return ( (NODEPTR) _currn);
}/* Mkrule_374 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_375 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_375 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_375 _currn;
#ifdef __cplusplus
_currn = new _TPrule_375;
#else
_currn = (_TPPrule_375) TreeNodeAlloc (sizeof (struct _TPrule_375));
#endif
_currn->_prod = RULErule_375;
_currn->_desc1 = (_TSPMultMeasureExpr) MkMultMeasureExpr (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_375: root of subtree no. 1 can not be made a MultMeasureExpr node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_375;
return ( (NODEPTR) _currn);
}/* Mkrule_375 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_376 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_376 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_376 _currn;
#ifdef __cplusplus
_currn = new _TPrule_376;
#else
_currn = (_TPPrule_376) TreeNodeAlloc (sizeof (struct _TPrule_376));
#endif
_currn->_prod = RULErule_376;
_currn->_desc1 = (_TSPSign) MkSign (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_376: root of subtree no. 1 can not be made a Sign node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasureSum) MkMeasureSum (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_376: root of subtree no. 2 can not be made a MeasureSum node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_376;
return ( (NODEPTR) _currn);
}/* Mkrule_376 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_377 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_377 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_377 _currn;
#ifdef __cplusplus
_currn = new _TPrule_377;
#else
_currn = (_TPPrule_377) TreeNodeAlloc (sizeof (struct _TPrule_377));
#endif
_currn->_prod = RULErule_377;
_currn->_desc1 = (_TSPMeasureSum) MkMeasureSum (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_377: root of subtree no. 1 can not be made a MeasureSum node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_377;
return ( (NODEPTR) _currn);
}/* Mkrule_377 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_378 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_378 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_378 _currn;
#ifdef __cplusplus
_currn = new _TPrule_378;
#else
_currn = (_TPPrule_378) TreeNodeAlloc (sizeof (struct _TPrule_378));
#endif
_currn->_prod = RULErule_378;
_currn->_desc1 = (_TSPMeasureSum) MkMeasureSum (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_378: root of subtree no. 1 can not be made a MeasureSum node ", 0, _coordref);
_currn->_desc2 = (_TSPAddOp) MkAddOp (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_378: root of subtree no. 2 can not be made a AddOp node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureTerm) MkMeasureTerm (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_378: root of subtree no. 3 can not be made a MeasureTerm node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_378;
return ( (NODEPTR) _currn);
}/* Mkrule_378 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_379 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_379 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_379 _currn;
#ifdef __cplusplus
_currn = new _TPrule_379;
#else
_currn = (_TPPrule_379) TreeNodeAlloc (sizeof (struct _TPrule_379));
#endif
_currn->_prod = RULErule_379;
_currn->_desc1 = (_TSPMeasureTerm) MkMeasureTerm (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_379: root of subtree no. 1 can not be made a MeasureTerm node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_379;
return ( (NODEPTR) _currn);
}/* Mkrule_379 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_380 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_380 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_380 _currn;
#ifdef __cplusplus
_currn = new _TPrule_380;
#else
_currn = (_TPPrule_380) TreeNodeAlloc (sizeof (struct _TPrule_380));
#endif
_currn->_prod = RULErule_380;
_currn->_desc1 = (_TSPMeasureTerm) MkMeasureTerm (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_380: root of subtree no. 1 can not be made a MeasureTerm node ", 0, _coordref);
_currn->_desc2 = (_TSPMulOp) MkMulOp (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_380: root of subtree no. 2 can not be made a MulOp node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasureFactor) MkMeasureFactor (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_380: root of subtree no. 3 can not be made a MeasureFactor node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_380;
return ( (NODEPTR) _currn);
}/* Mkrule_380 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_381 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_381 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_381 _currn;
#ifdef __cplusplus
_currn = new _TPrule_381;
#else
_currn = (_TPPrule_381) TreeNodeAlloc (sizeof (struct _TPrule_381));
#endif
_currn->_prod = RULErule_381;
_currn->_desc1 = (_TSPMeasureFactor) MkMeasureFactor (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_381: root of subtree no. 1 can not be made a MeasureFactor node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_381;
return ( (NODEPTR) _currn);
}/* Mkrule_381 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_382 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_382 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_382 _currn;
#ifdef __cplusplus
_currn = new _TPrule_382;
#else
_currn = (_TPPrule_382) TreeNodeAlloc (sizeof (struct _TPrule_382));
#endif
_currn->_prod = RULErule_382;
_currn->_desc1 = (_TSPMeasureFactor) MkMeasureFactor (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_382: root of subtree no. 1 can not be made a MeasureFactor node ", 0, _coordref);
_currn->_desc2 = (_TSPExpOp) MkExpOp (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_382: root of subtree no. 2 can not be made a ExpOp node ", 0, _coordref);
_currn->_desc3 = (_TSPMeasurePrimary) MkMeasurePrimary (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_382: root of subtree no. 3 can not be made a MeasurePrimary node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_382;
return ( (NODEPTR) _currn);
}/* Mkrule_382 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_383 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_383 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_383 _currn;
#ifdef __cplusplus
_currn = new _TPrule_383;
#else
_currn = (_TPPrule_383) TreeNodeAlloc (sizeof (struct _TPrule_383));
#endif
_currn->_prod = RULErule_383;
_currn->_desc1 = (_TSPMeasurePrimary) MkMeasurePrimary (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_383: root of subtree no. 1 can not be made a MeasurePrimary node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_383;
return ( (NODEPTR) _currn);
}/* Mkrule_383 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_384 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_384 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_384 _currn;
#ifdef __cplusplus
_currn = new _TPrule_384;
#else
_currn = (_TPPrule_384) TreeNodeAlloc (sizeof (struct _TPrule_384));
#endif
_currn->_prod = RULErule_384;
_currn->_desc1 = (_TSPNumber) MkNumber (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_384: root of subtree no. 1 can not be made a Number node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_384;
return ( (NODEPTR) _currn);
}/* Mkrule_384 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_385 (POSITION *_coordref, int _TERM1)
#else
NODEPTR Mkrule_385 (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	_TPPrule_385 _currn;
#ifdef __cplusplus
_currn = new _TPrule_385;
#else
_currn = (_TPPrule_385) TreeNodeAlloc (sizeof (struct _TPrule_385));
#endif
_currn->_prod = RULErule_385;
_SETCOORD(_currn)
_TERMACT_rule_385;
return ( (NODEPTR) _currn);
}/* Mkrule_385 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_386 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_386 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_386 _currn;
#ifdef __cplusplus
_currn = new _TPrule_386;
#else
_currn = (_TPPrule_386) TreeNodeAlloc (sizeof (struct _TPrule_386));
#endif
_currn->_prod = RULErule_386;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_386: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_386;
return ( (NODEPTR) _currn);
}/* Mkrule_386 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_387 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_387 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_387 _currn;
#ifdef __cplusplus
_currn = new _TPrule_387;
#else
_currn = (_TPPrule_387) TreeNodeAlloc (sizeof (struct _TPrule_387));
#endif
_currn->_prod = RULErule_387;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_387: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc2 = (_TSPPredicate) MkPredicate (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_387: root of subtree no. 2 can not be made a Predicate node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_387;
return ( (NODEPTR) _currn);
}/* Mkrule_387 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_388 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_388 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_388 _currn;
#ifdef __cplusplus
_currn = new _TPrule_388;
#else
_currn = (_TPPrule_388) TreeNodeAlloc (sizeof (struct _TPrule_388));
#endif
_currn->_prod = RULErule_388;
_currn->_desc1 = (_TSPMeasureExpression) MkMeasureExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_388: root of subtree no. 1 can not be made a MeasureExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_388;
return ( (NODEPTR) _currn);
}/* Mkrule_388 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_389 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_389 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_389 _currn;
#ifdef __cplusplus
_currn = new _TPrule_389;
#else
_currn = (_TPPrule_389) TreeNodeAlloc (sizeof (struct _TPrule_389));
#endif
_currn->_prod = RULErule_389;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_389: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_389;
return ( (NODEPTR) _currn);
}/* Mkrule_389 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_390 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_390 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_390 _currn;
#ifdef __cplusplus
_currn = new _TPrule_390;
#else
_currn = (_TPPrule_390) TreeNodeAlloc (sizeof (struct _TPrule_390));
#endif
_currn->_prod = RULErule_390;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_390: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc2 = (_TSPPredicate) MkPredicate (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_390: root of subtree no. 2 can not be made a Predicate node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_390;
return ( (NODEPTR) _currn);
}/* Mkrule_390 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_391 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_391 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_391 _currn;
#ifdef __cplusplus
_currn = new _TPrule_391;
#else
_currn = (_TPPrule_391) TreeNodeAlloc (sizeof (struct _TPrule_391));
#endif
_currn->_prod = RULErule_391;
_currn->_desc1 = (_TSPG17) MkG17 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_391: root of subtree no. 1 can not be made a G17 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_391;
return ( (NODEPTR) _currn);
}/* Mkrule_391 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_392 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_392 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_392 _currn;
#ifdef __cplusplus
_currn = new _TPrule_392;
#else
_currn = (_TPPrule_392) TreeNodeAlloc (sizeof (struct _TPrule_392));
#endif
_currn->_prod = RULErule_392;
_currn->_desc1 = (_TSPMeasureParameter) MkMeasureParameter (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_392: root of subtree no. 1 can not be made a MeasureParameter node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_392;
return ( (NODEPTR) _currn);
}/* Mkrule_392 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_393 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_393 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_393 _currn;
#ifdef __cplusplus
_currn = new _TPrule_393;
#else
_currn = (_TPPrule_393) TreeNodeAlloc (sizeof (struct _TPrule_393));
#endif
_currn->_prod = RULErule_393;
_currn->_desc1 = (_TSPG17) MkG17 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_393: root of subtree no. 1 can not be made a G17 node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasureParameter) MkMeasureParameter (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_393: root of subtree no. 2 can not be made a MeasureParameter node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_393;
return ( (NODEPTR) _currn);
}/* Mkrule_393 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_394 (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_394 (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_394 _currn;
#ifdef __cplusplus
_currn = new _TPrule_394;
#else
_currn = (_TPPrule_394) TreeNodeAlloc (sizeof (struct _TPrule_394));
#endif
_currn->_prod = RULErule_394;
_currn->_desc1 = (_TSPGlobalId) MkGlobalId (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_394: root of subtree no. 1 can not be made a GlobalId node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasureExpression) MkMeasureExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_394: root of subtree no. 2 can not be made a MeasureExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_394;
return ( (NODEPTR) _currn);
}/* Mkrule_394 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_395 (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_395 (_coordref, _TERM1,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_395 _currn;
#ifdef __cplusplus
_currn = new _TPrule_395;
#else
_currn = (_TPPrule_395) TreeNodeAlloc (sizeof (struct _TPrule_395));
#endif
_currn->_prod = RULErule_395;
_currn->_desc1 = (_TSPGlobalId) MkGlobalId (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_395: root of subtree no. 1 can not be made a GlobalId node ", 0, _coordref);
_currn->_desc2 = (_TSPMeasureExpression) MkMeasureExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_395: root of subtree no. 2 can not be made a MeasureExpression node ", 0, _coordref);
_currn->_desc3 = (_TSPParameterExpression) MkParameterExpression (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_395: root of subtree no. 3 can not be made a ParameterExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_395;
return ( (NODEPTR) _currn);
}/* Mkrule_395 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_396 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_396 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_396 _currn;
#ifdef __cplusplus
_currn = new _TPrule_396;
#else
_currn = (_TPPrule_396) TreeNodeAlloc (sizeof (struct _TPrule_396));
#endif
_currn->_prod = RULErule_396;
_currn->_desc1 = (_TSPExponentialDistribution) MkExponentialDistribution (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_396: root of subtree no. 1 can not be made a ExponentialDistribution node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_396;
return ( (NODEPTR) _currn);
}/* Mkrule_396 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_397 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_397 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_397 _currn;
#ifdef __cplusplus
_currn = new _TPrule_397;
#else
_currn = (_TPPrule_397) TreeNodeAlloc (sizeof (struct _TPrule_397));
#endif
_currn->_prod = RULErule_397;
_currn->_desc1 = (_TSPUniformDistribution) MkUniformDistribution (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_397: root of subtree no. 1 can not be made a UniformDistribution node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_397;
return ( (NODEPTR) _currn);
}/* Mkrule_397 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_398 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_398 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_398 _currn;
#ifdef __cplusplus
_currn = new _TPrule_398;
#else
_currn = (_TPPrule_398) TreeNodeAlloc (sizeof (struct _TPrule_398));
#endif
_currn->_prod = RULErule_398;
_currn->_desc1 = (_TSPTruncNormalDistribution) MkTruncNormalDistribution (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_398: root of subtree no. 1 can not be made a TruncNormalDistribution node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_398;
return ( (NODEPTR) _currn);
}/* Mkrule_398 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_399 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_399 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_399 _currn;
#ifdef __cplusplus
_currn = new _TPrule_399;
#else
_currn = (_TPPrule_399) TreeNodeAlloc (sizeof (struct _TPrule_399));
#endif
_currn->_prod = RULErule_399;
_currn->_desc1 = (_TSPErlangDistribution) MkErlangDistribution (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_399: root of subtree no. 1 can not be made a ErlangDistribution node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_399;
return ( (NODEPTR) _currn);
}/* Mkrule_399 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_400 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_400 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_400 _currn;
#ifdef __cplusplus
_currn = new _TPrule_400;
#else
_currn = (_TPPrule_400) TreeNodeAlloc (sizeof (struct _TPrule_400));
#endif
_currn->_prod = RULErule_400;
_currn->_desc1 = (_TSPWeibullDistribution) MkWeibullDistribution (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_400: root of subtree no. 1 can not be made a WeibullDistribution node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_400;
return ( (NODEPTR) _currn);
}/* Mkrule_400 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_401 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_401 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_401 _currn;
#ifdef __cplusplus
_currn = new _TPrule_401;
#else
_currn = (_TPPrule_401) TreeNodeAlloc (sizeof (struct _TPrule_401));
#endif
_currn->_prod = RULErule_401;
_currn->_desc1 = (_TSPPhaseTypeDistribution) MkPhaseTypeDistribution (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_401: root of subtree no. 1 can not be made a PhaseTypeDistribution node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_401;
return ( (NODEPTR) _currn);
}/* Mkrule_401 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_402 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_402 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_402 _currn;
#ifdef __cplusplus
_currn = new _TPrule_402;
#else
_currn = (_TPPrule_402) TreeNodeAlloc (sizeof (struct _TPrule_402));
#endif
_currn->_prod = RULErule_402;
_currn->_desc1 = (_TSPDeterministicDistribution) MkDeterministicDistribution (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_402: root of subtree no. 1 can not be made a DeterministicDistribution node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_402;
return ( (NODEPTR) _currn);
}/* Mkrule_402 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_403 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_403 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_403 _currn;
#ifdef __cplusplus
_currn = new _TPrule_403;
#else
_currn = (_TPPrule_403) TreeNodeAlloc (sizeof (struct _TPrule_403));
#endif
_currn->_prod = RULErule_403;
_currn->_desc1 = (_TSPBernoulliDistribution) MkBernoulliDistribution (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_403: root of subtree no. 1 can not be made a BernoulliDistribution node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_403;
return ( (NODEPTR) _currn);
}/* Mkrule_403 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_404 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_404 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_404 _currn;
#ifdef __cplusplus
_currn = new _TPrule_404;
#else
_currn = (_TPPrule_404) TreeNodeAlloc (sizeof (struct _TPrule_404));
#endif
_currn->_prod = RULErule_404;
_currn->_desc1 = (_TSPDiscreteUniDistribution) MkDiscreteUniDistribution (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_404: root of subtree no. 1 can not be made a DiscreteUniDistribution node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_404;
return ( (NODEPTR) _currn);
}/* Mkrule_404 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_405 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_405 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_405 _currn;
#ifdef __cplusplus
_currn = new _TPrule_405;
#else
_currn = (_TPPrule_405) TreeNodeAlloc (sizeof (struct _TPrule_405));
#endif
_currn->_prod = RULErule_405;
_currn->_desc1 = (_TSPGeometricDistribution) MkGeometricDistribution (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_405: root of subtree no. 1 can not be made a GeometricDistribution node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_405;
return ( (NODEPTR) _currn);
}/* Mkrule_405 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_406 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_406 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_406 _currn;
#ifdef __cplusplus
_currn = new _TPrule_406;
#else
_currn = (_TPPrule_406) TreeNodeAlloc (sizeof (struct _TPrule_406));
#endif
_currn->_prod = RULErule_406;
_currn->_desc1 = (_TSPModGeometricDistribution) MkModGeometricDistribution (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_406: root of subtree no. 1 can not be made a ModGeometricDistribution node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_406;
return ( (NODEPTR) _currn);
}/* Mkrule_406 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_407 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_407 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_407 _currn;
#ifdef __cplusplus
_currn = new _TPrule_407;
#else
_currn = (_TPPrule_407) TreeNodeAlloc (sizeof (struct _TPrule_407));
#endif
_currn->_prod = RULErule_407;
_currn->_desc1 = (_TSPBinomialDistribution) MkBinomialDistribution (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_407: root of subtree no. 1 can not be made a BinomialDistribution node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_407;
return ( (NODEPTR) _currn);
}/* Mkrule_407 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_408 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_408 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_408 _currn;
#ifdef __cplusplus
_currn = new _TPrule_408;
#else
_currn = (_TPPrule_408) TreeNodeAlloc (sizeof (struct _TPrule_408));
#endif
_currn->_prod = RULErule_408;
_currn->_desc1 = (_TSPPoissonDistribution) MkPoissonDistribution (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_408: root of subtree no. 1 can not be made a PoissonDistribution node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_408;
return ( (NODEPTR) _currn);
}/* Mkrule_408 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_409 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_409 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_409 _currn;
#ifdef __cplusplus
_currn = new _TPrule_409;
#else
_currn = (_TPPrule_409) TreeNodeAlloc (sizeof (struct _TPrule_409));
#endif
_currn->_prod = RULErule_409;
_currn->_desc1 = (_TSPExpolyDistribution) MkExpolyDistribution (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_409: root of subtree no. 1 can not be made a ExpolyDistribution node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_409;
return ( (NODEPTR) _currn);
}/* Mkrule_409 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_410 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_410 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_410 _currn;
#ifdef __cplusplus
_currn = new _TPrule_410;
#else
_currn = (_TPPrule_410) TreeNodeAlloc (sizeof (struct _TPrule_410));
#endif
_currn->_prod = RULErule_410;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_410: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_410;
return ( (NODEPTR) _currn);
}/* Mkrule_410 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_411 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_411 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_411 _currn;
#ifdef __cplusplus
_currn = new _TPrule_411;
#else
_currn = (_TPPrule_411) TreeNodeAlloc (sizeof (struct _TPrule_411));
#endif
_currn->_prod = RULErule_411;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_411: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc2 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_411: root of subtree no. 2 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_411;
return ( (NODEPTR) _currn);
}/* Mkrule_411 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_412 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_412 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_412 _currn;
#ifdef __cplusplus
_currn = new _TPrule_412;
#else
_currn = (_TPPrule_412) TreeNodeAlloc (sizeof (struct _TPrule_412));
#endif
_currn->_prod = RULErule_412;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_412: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc2 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_412: root of subtree no. 2 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_412;
return ( (NODEPTR) _currn);
}/* Mkrule_412 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_413 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_413 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_413 _currn;
#ifdef __cplusplus
_currn = new _TPrule_413;
#else
_currn = (_TPPrule_413) TreeNodeAlloc (sizeof (struct _TPrule_413));
#endif
_currn->_prod = RULErule_413;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_413: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc2 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_413: root of subtree no. 2 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_413;
return ( (NODEPTR) _currn);
}/* Mkrule_413 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_414 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_414 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_414 _currn;
#ifdef __cplusplus
_currn = new _TPrule_414;
#else
_currn = (_TPPrule_414) TreeNodeAlloc (sizeof (struct _TPrule_414));
#endif
_currn->_prod = RULErule_414;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_414: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc2 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_414: root of subtree no. 2 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_414;
return ( (NODEPTR) _currn);
}/* Mkrule_414 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_415 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_415 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_415 _currn;
#ifdef __cplusplus
_currn = new _TPrule_415;
#else
_currn = (_TPPrule_415) TreeNodeAlloc (sizeof (struct _TPrule_415));
#endif
_currn->_prod = RULErule_415;
_currn->_desc1 = (_TSPMatrix) MkMatrix (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_415: root of subtree no. 1 can not be made a Matrix node ", 0, _coordref);
_currn->_desc2 = (_TSPVektor) MkVektor (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_415: root of subtree no. 2 can not be made a Vektor node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_415;
return ( (NODEPTR) _currn);
}/* Mkrule_415 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_416 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_416 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_416 _currn;
#ifdef __cplusplus
_currn = new _TPrule_416;
#else
_currn = (_TPPrule_416) TreeNodeAlloc (sizeof (struct _TPrule_416));
#endif
_currn->_prod = RULErule_416;
_currn->_desc1 = (_TSPG20) MkG20 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_416: root of subtree no. 1 can not be made a G20 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_416;
return ( (NODEPTR) _currn);
}/* Mkrule_416 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_417 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_417 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_417 _currn;
#ifdef __cplusplus
_currn = new _TPrule_417;
#else
_currn = (_TPPrule_417) TreeNodeAlloc (sizeof (struct _TPrule_417));
#endif
_currn->_prod = RULErule_417;
_currn->_desc1 = (_TSPG19) MkG19 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_417: root of subtree no. 1 can not be made a G19 node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_417;
return ( (NODEPTR) _currn);
}/* Mkrule_417 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_418 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_418 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_418 _currn;
#ifdef __cplusplus
_currn = new _TPrule_418;
#else
_currn = (_TPPrule_418) TreeNodeAlloc (sizeof (struct _TPrule_418));
#endif
_currn->_prod = RULErule_418;
_currn->_desc1 = (_TSPG18) MkG18 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_418: root of subtree no. 1 can not be made a G18 node ", 0, _coordref);
_currn->_desc2 = (_TSPG19) MkG19 (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_418: root of subtree no. 2 can not be made a G19 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_418;
return ( (NODEPTR) _currn);
}/* Mkrule_418 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_419 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_419 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_419 _currn;
#ifdef __cplusplus
_currn = new _TPrule_419;
#else
_currn = (_TPPrule_419) TreeNodeAlloc (sizeof (struct _TPrule_419));
#endif
_currn->_prod = RULErule_419;
_currn->_desc1 = (_TSPVektor) MkVektor (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_419: root of subtree no. 1 can not be made a Vektor node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_419;
return ( (NODEPTR) _currn);
}/* Mkrule_419 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_420 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_420 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_420 _currn;
#ifdef __cplusplus
_currn = new _TPrule_420;
#else
_currn = (_TPPrule_420) TreeNodeAlloc (sizeof (struct _TPrule_420));
#endif
_currn->_prod = RULErule_420;
_currn->_desc1 = (_TSPG18) MkG18 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_420: root of subtree no. 1 can not be made a G18 node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_420;
return ( (NODEPTR) _currn);
}/* Mkrule_420 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_421 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_421 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_421 _currn;
#ifdef __cplusplus
_currn = new _TPrule_421;
#else
_currn = (_TPPrule_421) TreeNodeAlloc (sizeof (struct _TPrule_421));
#endif
_currn->_prod = RULErule_421;
_currn->_desc1 = (_TSPG23) MkG23 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_421: root of subtree no. 1 can not be made a G23 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_421;
return ( (NODEPTR) _currn);
}/* Mkrule_421 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_422 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_422 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_422 _currn;
#ifdef __cplusplus
_currn = new _TPrule_422;
#else
_currn = (_TPPrule_422) TreeNodeAlloc (sizeof (struct _TPrule_422));
#endif
_currn->_prod = RULErule_422;
_currn->_desc1 = (_TSPG22) MkG22 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_422: root of subtree no. 1 can not be made a G22 node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_422;
return ( (NODEPTR) _currn);
}/* Mkrule_422 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_423 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_423 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_423 _currn;
#ifdef __cplusplus
_currn = new _TPrule_423;
#else
_currn = (_TPPrule_423) TreeNodeAlloc (sizeof (struct _TPrule_423));
#endif
_currn->_prod = RULErule_423;
_currn->_desc1 = (_TSPG21) MkG21 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_423: root of subtree no. 1 can not be made a G21 node ", 0, _coordref);
_currn->_desc2 = (_TSPG22) MkG22 (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_423: root of subtree no. 2 can not be made a G22 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_423;
return ( (NODEPTR) _currn);
}/* Mkrule_423 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_424 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_424 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_424 _currn;
#ifdef __cplusplus
_currn = new _TPrule_424;
#else
_currn = (_TPPrule_424) TreeNodeAlloc (sizeof (struct _TPrule_424));
#endif
_currn->_prod = RULErule_424;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_424: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_424;
return ( (NODEPTR) _currn);
}/* Mkrule_424 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_425 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_425 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_425 _currn;
#ifdef __cplusplus
_currn = new _TPrule_425;
#else
_currn = (_TPPrule_425) TreeNodeAlloc (sizeof (struct _TPrule_425));
#endif
_currn->_prod = RULErule_425;
_currn->_desc1 = (_TSPG21) MkG21 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_425: root of subtree no. 1 can not be made a G21 node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_425;
return ( (NODEPTR) _currn);
}/* Mkrule_425 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_426 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_426 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_426 _currn;
#ifdef __cplusplus
_currn = new _TPrule_426;
#else
_currn = (_TPPrule_426) TreeNodeAlloc (sizeof (struct _TPrule_426));
#endif
_currn->_prod = RULErule_426;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_426: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_426;
return ( (NODEPTR) _currn);
}/* Mkrule_426 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_427 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_427 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_427 _currn;
#ifdef __cplusplus
_currn = new _TPrule_427;
#else
_currn = (_TPPrule_427) TreeNodeAlloc (sizeof (struct _TPrule_427));
#endif
_currn->_prod = RULErule_427;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_427: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc2 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_427: root of subtree no. 2 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_427;
return ( (NODEPTR) _currn);
}/* Mkrule_427 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_428 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_428 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_428 _currn;
#ifdef __cplusplus
_currn = new _TPrule_428;
#else
_currn = (_TPPrule_428) TreeNodeAlloc (sizeof (struct _TPrule_428));
#endif
_currn->_prod = RULErule_428;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_428: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc2 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_428: root of subtree no. 2 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc3 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_428: root of subtree no. 3 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_428;
return ( (NODEPTR) _currn);
}/* Mkrule_428 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_429 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_429 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_429 _currn;
#ifdef __cplusplus
_currn = new _TPrule_429;
#else
_currn = (_TPPrule_429) TreeNodeAlloc (sizeof (struct _TPrule_429));
#endif
_currn->_prod = RULErule_429;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_429: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc2 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_429: root of subtree no. 2 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_429;
return ( (NODEPTR) _currn);
}/* Mkrule_429 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_430 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_430 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_430 _currn;
#ifdef __cplusplus
_currn = new _TPrule_430;
#else
_currn = (_TPPrule_430) TreeNodeAlloc (sizeof (struct _TPrule_430));
#endif
_currn->_prod = RULErule_430;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_430: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc2 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_430: root of subtree no. 2 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_430;
return ( (NODEPTR) _currn);
}/* Mkrule_430 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_431 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_431 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_431 _currn;
#ifdef __cplusplus
_currn = new _TPrule_431;
#else
_currn = (_TPPrule_431) TreeNodeAlloc (sizeof (struct _TPrule_431));
#endif
_currn->_prod = RULErule_431;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_431: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc2 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_431: root of subtree no. 2 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc3 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_431: root of subtree no. 3 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_431;
return ( (NODEPTR) _currn);
}/* Mkrule_431 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_432 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_432 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_432 _currn;
#ifdef __cplusplus
_currn = new _TPrule_432;
#else
_currn = (_TPPrule_432) TreeNodeAlloc (sizeof (struct _TPrule_432));
#endif
_currn->_prod = RULErule_432;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_432: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc2 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_432: root of subtree no. 2 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc3 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_432: root of subtree no. 3 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_432;
return ( (NODEPTR) _currn);
}/* Mkrule_432 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_433 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_433 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_433 _currn;
#ifdef __cplusplus
_currn = new _TPrule_433;
#else
_currn = (_TPPrule_433) TreeNodeAlloc (sizeof (struct _TPrule_433));
#endif
_currn->_prod = RULErule_433;
_currn->_desc1 = (_TSPExpolyDistributionSum) MkExpolyDistributionSum (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_433: root of subtree no. 1 can not be made a ExpolyDistributionSum node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_433;
return ( (NODEPTR) _currn);
}/* Mkrule_433 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_434 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_434 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_434 _currn;
#ifdef __cplusplus
_currn = new _TPrule_434;
#else
_currn = (_TPPrule_434) TreeNodeAlloc (sizeof (struct _TPrule_434));
#endif
_currn->_prod = RULErule_434;
_currn->_desc1 = (_TSPExpolyDistributionSum) MkExpolyDistributionSum (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_434: root of subtree no. 1 can not be made a ExpolyDistributionSum node ", 0, _coordref);
_currn->_desc2 = (_TSPAddOp) MkAddOp (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_434: root of subtree no. 2 can not be made a AddOp node ", 0, _coordref);
_currn->_desc3 = (_TSPExpolyDistributionPrimary) MkExpolyDistributionPrimary (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_434: root of subtree no. 3 can not be made a ExpolyDistributionPrimary node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_434;
return ( (NODEPTR) _currn);
}/* Mkrule_434 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_435 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_435 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_435 _currn;
#ifdef __cplusplus
_currn = new _TPrule_435;
#else
_currn = (_TPPrule_435) TreeNodeAlloc (sizeof (struct _TPrule_435));
#endif
_currn->_prod = RULErule_435;
_currn->_desc1 = (_TSPExpolyDistributionPrimary) MkExpolyDistributionPrimary (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_435: root of subtree no. 1 can not be made a ExpolyDistributionPrimary node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_435;
return ( (NODEPTR) _currn);
}/* Mkrule_435 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_436 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_436 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_436 _currn;
#ifdef __cplusplus
_currn = new _TPrule_436;
#else
_currn = (_TPPrule_436) TreeNodeAlloc (sizeof (struct _TPrule_436));
#endif
_currn->_prod = RULErule_436;
_currn->_desc1 = (_TSPImpulseExpression) MkImpulseExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_436: root of subtree no. 1 can not be made a ImpulseExpression node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_436;
return ( (NODEPTR) _currn);
}/* Mkrule_436 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_437 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_437 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_437 _currn;
#ifdef __cplusplus
_currn = new _TPrule_437;
#else
_currn = (_TPPrule_437) TreeNodeAlloc (sizeof (struct _TPrule_437));
#endif
_currn->_prod = RULErule_437;
_currn->_desc1 = (_TSPContinuousExpression) MkContinuousExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_437: root of subtree no. 1 can not be made a ContinuousExpression node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_437;
return ( (NODEPTR) _currn);
}/* Mkrule_437 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_438 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_438 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_438 _currn;
#ifdef __cplusplus
_currn = new _TPrule_438;
#else
_currn = (_TPPrule_438) TreeNodeAlloc (sizeof (struct _TPrule_438));
#endif
_currn->_prod = RULErule_438;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_438: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_438;
return ( (NODEPTR) _currn);
}/* Mkrule_438 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_439 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_439 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_439 _currn;
#ifdef __cplusplus
_currn = new _TPrule_439;
#else
_currn = (_TPPrule_439) TreeNodeAlloc (sizeof (struct _TPrule_439));
#endif
_currn->_prod = RULErule_439;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_439: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc2 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_439: root of subtree no. 2 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_439;
return ( (NODEPTR) _currn);
}/* Mkrule_439 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_440 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_440 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_440 _currn;
#ifdef __cplusplus
_currn = new _TPrule_440;
#else
_currn = (_TPPrule_440) TreeNodeAlloc (sizeof (struct _TPrule_440));
#endif
_currn->_prod = RULErule_440;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_440: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc2 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_440: root of subtree no. 2 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_440;
return ( (NODEPTR) _currn);
}/* Mkrule_440 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_441 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_441 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_441 _currn;
#ifdef __cplusplus
_currn = new _TPrule_441;
#else
_currn = (_TPPrule_441) TreeNodeAlloc (sizeof (struct _TPrule_441));
#endif
_currn->_prod = RULErule_441;
_currn->_desc1 = (_TSPExpolynomial) MkExpolynomial (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_441: root of subtree no. 1 can not be made a Expolynomial node ", 0, _coordref);
_currn->_desc2 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_441: root of subtree no. 2 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc3 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_441: root of subtree no. 3 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_441;
return ( (NODEPTR) _currn);
}/* Mkrule_441 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_442 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_442 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_442 _currn;
#ifdef __cplusplus
_currn = new _TPrule_442;
#else
_currn = (_TPPrule_442) TreeNodeAlloc (sizeof (struct _TPrule_442));
#endif
_currn->_prod = RULErule_442;
_currn->_desc1 = (_TSPExpolinomialSum) MkExpolinomialSum (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_442: root of subtree no. 1 can not be made a ExpolinomialSum node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_442;
return ( (NODEPTR) _currn);
}/* Mkrule_442 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_443 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_443 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_443 _currn;
#ifdef __cplusplus
_currn = new _TPrule_443;
#else
_currn = (_TPPrule_443) TreeNodeAlloc (sizeof (struct _TPrule_443));
#endif
_currn->_prod = RULErule_443;
_currn->_desc1 = (_TSPExpolinomialSum) MkExpolinomialSum (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_443: root of subtree no. 1 can not be made a ExpolinomialSum node ", 0, _coordref);
_currn->_desc2 = (_TSPAddOp) MkAddOp (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_443: root of subtree no. 2 can not be made a AddOp node ", 0, _coordref);
_currn->_desc3 = (_TSPExpolinomialPrimary) MkExpolinomialPrimary (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_443: root of subtree no. 3 can not be made a ExpolinomialPrimary node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_443;
return ( (NODEPTR) _currn);
}/* Mkrule_443 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_444 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_444 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_444 _currn;
#ifdef __cplusplus
_currn = new _TPrule_444;
#else
_currn = (_TPPrule_444) TreeNodeAlloc (sizeof (struct _TPrule_444));
#endif
_currn->_prod = RULErule_444;
_currn->_desc1 = (_TSPExpolinomialPrimary) MkExpolinomialPrimary (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_444: root of subtree no. 1 can not be made a ExpolinomialPrimary node ", 0, _coordref);
_COPYCOORD(_currn)
_TERMACT_rule_444;
return ( (NODEPTR) _currn);
}/* Mkrule_444 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_445 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR Mkrule_445 (_coordref,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	_TPPrule_445 _currn;
#ifdef __cplusplus
_currn = new _TPrule_445;
#else
_currn = (_TPPrule_445) TreeNodeAlloc (sizeof (struct _TPrule_445));
#endif
_currn->_prod = RULErule_445;
_currn->_desc1 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_445: root of subtree no. 1 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc2 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_445: root of subtree no. 2 can not be made a RewardExpression node ", 0, _coordref);
_currn->_desc3 = (_TSPRewardExpression) MkRewardExpression (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_445: root of subtree no. 3 can not be made a RewardExpression node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_445;
return ( (NODEPTR) _currn);
}/* Mkrule_445 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_446 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR Mkrule_446 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	_TPPrule_446 _currn;
#ifdef __cplusplus
_currn = new _TPrule_446;
#else
_currn = (_TPPrule_446) TreeNodeAlloc (sizeof (struct _TPrule_446));
#endif
_currn->_prod = RULErule_446;
_currn->_desc1 = (_TSPExpolynomial) MkExpolynomial (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_446: root of subtree no. 1 can not be made a Expolynomial node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_446;
return ( (NODEPTR) _currn);
}/* Mkrule_446 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_447 (POSITION *_coordref)
#else
NODEPTR Mkrule_447 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_447 _currn;
#ifdef __cplusplus
_currn = new _TPrule_447;
#else
_currn = (_TPPrule_447) TreeNodeAlloc (sizeof (struct _TPrule_447));
#endif
_currn->_prod = RULErule_447;
_SETCOORD(_currn)
_TERMACT_rule_447;
return ( (NODEPTR) _currn);
}/* Mkrule_447 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_448 (POSITION *_coordref)
#else
NODEPTR Mkrule_448 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_448 _currn;
#ifdef __cplusplus
_currn = new _TPrule_448;
#else
_currn = (_TPPrule_448) TreeNodeAlloc (sizeof (struct _TPrule_448));
#endif
_currn->_prod = RULErule_448;
_SETCOORD(_currn)
_TERMACT_rule_448;
return ( (NODEPTR) _currn);
}/* Mkrule_448 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_449 (POSITION *_coordref)
#else
NODEPTR Mkrule_449 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_449 _currn;
#ifdef __cplusplus
_currn = new _TPrule_449;
#else
_currn = (_TPPrule_449) TreeNodeAlloc (sizeof (struct _TPrule_449));
#endif
_currn->_prod = RULErule_449;
_SETCOORD(_currn)
_TERMACT_rule_449;
return ( (NODEPTR) _currn);
}/* Mkrule_449 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_450 (POSITION *_coordref)
#else
NODEPTR Mkrule_450 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_450 _currn;
#ifdef __cplusplus
_currn = new _TPrule_450;
#else
_currn = (_TPPrule_450) TreeNodeAlloc (sizeof (struct _TPrule_450));
#endif
_currn->_prod = RULErule_450;
_SETCOORD(_currn)
_TERMACT_rule_450;
return ( (NODEPTR) _currn);
}/* Mkrule_450 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_451 (POSITION *_coordref)
#else
NODEPTR Mkrule_451 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_451 _currn;
#ifdef __cplusplus
_currn = new _TPrule_451;
#else
_currn = (_TPPrule_451) TreeNodeAlloc (sizeof (struct _TPrule_451));
#endif
_currn->_prod = RULErule_451;
_SETCOORD(_currn)
_TERMACT_rule_451;
return ( (NODEPTR) _currn);
}/* Mkrule_451 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_452 (POSITION *_coordref)
#else
NODEPTR Mkrule_452 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_452 _currn;
#ifdef __cplusplus
_currn = new _TPrule_452;
#else
_currn = (_TPPrule_452) TreeNodeAlloc (sizeof (struct _TPrule_452));
#endif
_currn->_prod = RULErule_452;
_SETCOORD(_currn)
_TERMACT_rule_452;
return ( (NODEPTR) _currn);
}/* Mkrule_452 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_453 (POSITION *_coordref)
#else
NODEPTR Mkrule_453 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_453 _currn;
#ifdef __cplusplus
_currn = new _TPrule_453;
#else
_currn = (_TPPrule_453) TreeNodeAlloc (sizeof (struct _TPrule_453));
#endif
_currn->_prod = RULErule_453;
_SETCOORD(_currn)
_TERMACT_rule_453;
return ( (NODEPTR) _currn);
}/* Mkrule_453 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_454 (POSITION *_coordref)
#else
NODEPTR Mkrule_454 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_454 _currn;
#ifdef __cplusplus
_currn = new _TPrule_454;
#else
_currn = (_TPPrule_454) TreeNodeAlloc (sizeof (struct _TPrule_454));
#endif
_currn->_prod = RULErule_454;
_SETCOORD(_currn)
_TERMACT_rule_454;
return ( (NODEPTR) _currn);
}/* Mkrule_454 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_455 (POSITION *_coordref)
#else
NODEPTR Mkrule_455 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_455 _currn;
#ifdef __cplusplus
_currn = new _TPrule_455;
#else
_currn = (_TPPrule_455) TreeNodeAlloc (sizeof (struct _TPrule_455));
#endif
_currn->_prod = RULErule_455;
_SETCOORD(_currn)
_TERMACT_rule_455;
return ( (NODEPTR) _currn);
}/* Mkrule_455 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_456 (POSITION *_coordref)
#else
NODEPTR Mkrule_456 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_456 _currn;
#ifdef __cplusplus
_currn = new _TPrule_456;
#else
_currn = (_TPPrule_456) TreeNodeAlloc (sizeof (struct _TPrule_456));
#endif
_currn->_prod = RULErule_456;
_SETCOORD(_currn)
_TERMACT_rule_456;
return ( (NODEPTR) _currn);
}/* Mkrule_456 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_457 (POSITION *_coordref)
#else
NODEPTR Mkrule_457 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_457 _currn;
#ifdef __cplusplus
_currn = new _TPrule_457;
#else
_currn = (_TPPrule_457) TreeNodeAlloc (sizeof (struct _TPrule_457));
#endif
_currn->_prod = RULErule_457;
_SETCOORD(_currn)
_TERMACT_rule_457;
return ( (NODEPTR) _currn);
}/* Mkrule_457 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_458 (POSITION *_coordref)
#else
NODEPTR Mkrule_458 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_458 _currn;
#ifdef __cplusplus
_currn = new _TPrule_458;
#else
_currn = (_TPPrule_458) TreeNodeAlloc (sizeof (struct _TPrule_458));
#endif
_currn->_prod = RULErule_458;
_SETCOORD(_currn)
_TERMACT_rule_458;
return ( (NODEPTR) _currn);
}/* Mkrule_458 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_459 (POSITION *_coordref)
#else
NODEPTR Mkrule_459 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_459 _currn;
#ifdef __cplusplus
_currn = new _TPrule_459;
#else
_currn = (_TPPrule_459) TreeNodeAlloc (sizeof (struct _TPrule_459));
#endif
_currn->_prod = RULErule_459;
_SETCOORD(_currn)
_TERMACT_rule_459;
return ( (NODEPTR) _currn);
}/* Mkrule_459 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_460 (POSITION *_coordref)
#else
NODEPTR Mkrule_460 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_460 _currn;
#ifdef __cplusplus
_currn = new _TPrule_460;
#else
_currn = (_TPPrule_460) TreeNodeAlloc (sizeof (struct _TPrule_460));
#endif
_currn->_prod = RULErule_460;
_SETCOORD(_currn)
_TERMACT_rule_460;
return ( (NODEPTR) _currn);
}/* Mkrule_460 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_461 (POSITION *_coordref)
#else
NODEPTR Mkrule_461 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_461 _currn;
#ifdef __cplusplus
_currn = new _TPrule_461;
#else
_currn = (_TPPrule_461) TreeNodeAlloc (sizeof (struct _TPrule_461));
#endif
_currn->_prod = RULErule_461;
_SETCOORD(_currn)
_TERMACT_rule_461;
return ( (NODEPTR) _currn);
}/* Mkrule_461 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_462 (POSITION *_coordref)
#else
NODEPTR Mkrule_462 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_462 _currn;
#ifdef __cplusplus
_currn = new _TPrule_462;
#else
_currn = (_TPPrule_462) TreeNodeAlloc (sizeof (struct _TPrule_462));
#endif
_currn->_prod = RULErule_462;
_SETCOORD(_currn)
_TERMACT_rule_462;
return ( (NODEPTR) _currn);
}/* Mkrule_462 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_463 (POSITION *_coordref, int _TERM1)
#else
NODEPTR Mkrule_463 (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	_TPPrule_463 _currn;
#ifdef __cplusplus
_currn = new _TPrule_463;
#else
_currn = (_TPPrule_463) TreeNodeAlloc (sizeof (struct _TPrule_463));
#endif
_currn->_prod = RULErule_463;
_SETCOORD(_currn)
_TERMACT_rule_463;
return ( (NODEPTR) _currn);
}/* Mkrule_463 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_464 (POSITION *_coordref, int _TERM1)
#else
NODEPTR Mkrule_464 (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	_TPPrule_464 _currn;
#ifdef __cplusplus
_currn = new _TPrule_464;
#else
_currn = (_TPPrule_464) TreeNodeAlloc (sizeof (struct _TPrule_464));
#endif
_currn->_prod = RULErule_464;
_SETCOORD(_currn)
_TERMACT_rule_464;
return ( (NODEPTR) _currn);
}/* Mkrule_464 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_465 (POSITION *_coordref, int _TERM1, NODEPTR _desc1, int _TERM2)
#else
NODEPTR Mkrule_465 (_coordref, _TERM1,_desc1, _TERM2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	int _TERM2;
#endif
{	_TPPrule_465 _currn;
#ifdef __cplusplus
_currn = new _TPrule_465;
#else
_currn = (_TPPrule_465) TreeNodeAlloc (sizeof (struct _TPrule_465));
#endif
_currn->_prod = RULErule_465;
_currn->_desc1 = (_TSPG25) MkG25 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_465: root of subtree no. 1 can not be made a G25 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_465;
return ( (NODEPTR) _currn);
}/* Mkrule_465 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_466 (POSITION *_coordref, int _TERM1, int _TERM2, NODEPTR _desc1, int _TERM3)
#else
NODEPTR Mkrule_466 (_coordref, _TERM1, _TERM2,_desc1, _TERM3)
	POSITION *_coordref;
	int _TERM1;
	int _TERM2;
	NODEPTR _desc1;
	int _TERM3;
#endif
{	_TPPrule_466 _currn;
#ifdef __cplusplus
_currn = new _TPrule_466;
#else
_currn = (_TPPrule_466) TreeNodeAlloc (sizeof (struct _TPrule_466));
#endif
_currn->_prod = RULErule_466;
_currn->_desc1 = (_TSPG25) MkG25 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_466: root of subtree no. 1 can not be made a G25 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_466;
return ( (NODEPTR) _currn);
}/* Mkrule_466 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_467 (POSITION *_coordref, int _TERM1)
#else
NODEPTR Mkrule_467 (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	_TPPrule_467 _currn;
#ifdef __cplusplus
_currn = new _TPrule_467;
#else
_currn = (_TPPrule_467) TreeNodeAlloc (sizeof (struct _TPrule_467));
#endif
_currn->_prod = RULErule_467;
_SETCOORD(_currn)
_TERMACT_rule_467;
return ( (NODEPTR) _currn);
}/* Mkrule_467 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_468 (POSITION *_coordref, int _TERM1, int _TERM2)
#else
NODEPTR Mkrule_468 (_coordref, _TERM1, _TERM2)
	POSITION *_coordref;
	int _TERM1;
	int _TERM2;
#endif
{	_TPPrule_468 _currn;
#ifdef __cplusplus
_currn = new _TPrule_468;
#else
_currn = (_TPPrule_468) TreeNodeAlloc (sizeof (struct _TPrule_468));
#endif
_currn->_prod = RULErule_468;
_SETCOORD(_currn)
_TERMACT_rule_468;
return ( (NODEPTR) _currn);
}/* Mkrule_468 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_469 (POSITION *_coordref)
#else
NODEPTR Mkrule_469 (_coordref)
	POSITION *_coordref;
#endif
{	_TPPrule_469 _currn;
#ifdef __cplusplus
_currn = new _TPrule_469;
#else
_currn = (_TPPrule_469) TreeNodeAlloc (sizeof (struct _TPrule_469));
#endif
_currn->_prod = RULErule_469;
_SETCOORD(_currn)
_TERMACT_rule_469;
return ( (NODEPTR) _currn);
}/* Mkrule_469 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR Mkrule_470 (POSITION *_coordref, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR Mkrule_470 (_coordref,_desc1,_desc2)
	POSITION *_coordref;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	_TPPrule_470 _currn;
#ifdef __cplusplus
_currn = new _TPrule_470;
#else
_currn = (_TPPrule_470) TreeNodeAlloc (sizeof (struct _TPrule_470));
#endif
_currn->_prod = RULErule_470;
_currn->_desc1 = (_TSPG25) MkG25 (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_470: root of subtree no. 1 can not be made a G25 node ", 0, _coordref);
_currn->_desc2 = (_TSPG24) MkG24 (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_470: root of subtree no. 2 can not be made a G24 node ", 0, _coordref);
_SETCOORD(_currn)
_TERMACT_rule_470;
return ( (NODEPTR) _currn);
}/* Mkrule_470 */
